
########################################################
#
# Copyright (c) 2003-2012 by University of Queensland
# Earth Systems Science Computational Center (ESSCC)
# http://www.uq.edu.au/esscc
#
# Primary Business: Queensland, Australia
# Licensed under the Open Software License version 3.0
# http://www.opensource.org/licenses/osl-3.0.php
#
########################################################

__copyright__="""Copyright (c) 2003-2012 by University of Queensland
Earth Systems Science Computational Center (ESSCC)
http://www.uq.edu.au/esscc
Primary Business: Queensland, Australia"""
__license__="""Licensed under the Open Software License version 3.0
http://www.opensource.org/licenses/osl-3.0.php"""
__url__="https://launchpad.net/escript-finley"

"""
test for unary operations without tagged data

:remark: use see `test_util`
:var __author__: name of author
:var __copyright__: copyrights
:var __license__: licence agreement
:var __url__: url entry point on documentation
:var __version__: version
:var __date__: date of the version
"""

__author__="Lutz Gross, l.gross@uq.edu.au"

import unittest
import numpy
from esys.escript import *
from test_util_base import Test_util_base

class Test_util_binary_no_tagged_data(Test_util_base):
   """
   test for the binary operation add, mult, ...,inner, without tagged data
   """
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_update_float0_float0(self):
      arg0=Data(1)
      arg0+=0.5
      ref=Data(1.5)
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(+)")
      arg0=Data(1)
      arg0-=0.5;
      ref=Data(0.5)
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(-)")
      arg0=Data(1)
      arg0/=2;
      ref=Data(0.5)
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(/)")
      arg0=Data(1)
      arg0*=0.5;
      ref=Data(0.5)
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(*)")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_update_rank1_float0(self):
      arg0=Data([1,1])
      arg0+=0.5
      ref=Data([1.5,1.5])
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(+)")
      arg0=Data([1,1])
      arg0-=0.5;
      ref=Data([0.5,0.5])
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(-)")
      arg0=Data([1,1])
      arg0/=2;
      ref=Data([0.5,0.5])
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(/)")
      arg0=Data([1,1])
      arg0*=0.5;
      ref=Data([0.5,0.5])
      self.assertTrue(Lsup(arg0-ref)<=self.RES_TOL*Lsup(ref),"wrong result(*)")
      
      
      
   def test_add_float_rank0_float_rank0(self):
      arg0=-1.85774987434
      arg1=4.45161012244
      res=add(arg0,arg1)
      ref=2.59386024809
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_array_rank0(self):
      arg0=4.19995915455
      arg1=numpy.array(4.05252282996)
      res=add(arg0,arg1)
      ref=numpy.array(8.25248198451)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_array_rank1(self):
      arg0=-0.789081574676
      arg1=numpy.array([3.0989284588762782, -3.1117486239932548])
      res=add(arg0,arg1)
      ref=numpy.array([2.3098468842001276, -3.9008301986694054])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_array_rank2(self):
      arg0=4.22980968095
      arg1=numpy.array([[3.8263049168469063, -3.8107552261242961, -0.95796615082511938, 0.89217770425415566, -1.367568567046141], [-4.589204301639521, -1.8350562314947796, -2.6796711234567594, -1.517018639147719, -4.2515860077769299], [-4.9092386861845885, 1.6349807040932944, -2.8837636918332867, -2.1359802030830366, 1.1877691579576872], [0.64362175988040171, 4.6388736621003481, -4.6187851625380176, -0.19670267515157747, -4.3703403640373848]])
      res=add(arg0,arg1)
      ref=numpy.array([[8.0561145978015265, 0.41905445483032411, 3.2718435301295008, 5.1219873852087758, 2.8622411139084791], [-0.3593946206849008, 2.3947534494598406, 1.5501385574978608, 2.7127910418069012, -0.021776326822309677], [-0.6794290052299683, 5.8647903850479146, 1.3460459891213334, 2.0938294778715836, 5.4175788389123074], [4.8734314408350219, 8.8686833430549683, -0.38897548158339745, 4.0331070058030427, -0.14053068308276462]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_array_rank3(self):
      arg0=-3.53456651889
      arg1=numpy.array([[[0.87245810664775725, -1.4729436956315931], [3.6218982325488049, -0.1513040114711881]], [[-4.803075549420659, -3.3367420122420577], [0.38976894263068385, -0.74090579708685667]], [[1.7577043473476319, -4.8686481097693255], [2.4278786968981949, 2.7439916415727748]], [[2.5828147603109199, 4.0079889484954307], [2.524821182993823, -2.0178731238106442]], [[2.151779590210027, -1.6354150996023509], [4.9382783295598127, -4.3504386799573149]], [[0.092273497705008367, -4.1474467200211969], [-0.60528749249019143, 1.9950618216367015]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[-2.6621084122403653, -5.0075102145197157], [0.087331713660682375, -3.6858705303593107]], [[-8.3376420683087815, -6.8713085311301807], [-3.1447975762574387, -4.2754723159749792]], [[-1.7768621715404906, -8.4032146286574481], [-1.1066878219899277, -0.79057487731534781]], [[-0.95175175857720262, 0.47342242960730818], [-1.0097453358942996, -5.5524396426987668]], [[-1.3827869286780956, -5.169981618490473], [1.4037118106716902, -7.8850051988454375]], [[-3.4422930211831142, -7.6820132389093194], [-4.139854011378314, -1.539504697251421]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_array_rank4(self):
      arg0=3.80068783153
      arg1=numpy.array([[[[1.3978431583839148, 1.9361714942297823, 3.7535362284174631, -2.4337387184773607], [0.5635202106026318, 0.5241409957813481, -3.4140270807067696, -4.0577985422064451], [-2.0363977249490803, -2.3513631626652174, -2.5087195515781522, -3.0669612737249476]], [[4.6317401569348267, -4.1439582154345613, -3.6372822610617805, 4.6787916736384147], [-2.5381953826017636, -2.8933892555674765, -0.0015529869204300439, 3.368719966547113], [-1.54864554787543, -0.26211688245501907, 0.67676437972178505, 0.12480115088498778]]], [[[4.894835715060978, -3.3315078814192765, 1.8436785068574633, 3.4665920947384645], [-3.723024429466204, 4.2365082256016997, 0.72839877670346365, 0.90510572900029551], [-3.2047842373454714, -0.33107722023435482, -3.9336604429996869, 0.65895488106780942]], [[4.6190345897679812, 3.6494961841140832, 2.2829072964851331, 4.7807363050102971], [-1.7472315219470702, -0.42863932143535166, 2.9525380415836278, 2.1875556999155874], [2.3575775491104451, 0.32119760477087844, -0.37696828550394113, -2.6964854519636559]]], [[[4.3982042140790654, 2.3176626766340434, 3.4260859254330853, -2.0671657497831522], [-3.4340980107632046, -0.48771134437134123, 2.8880405802296929, -1.0519636764358919], [-3.2701944375206917, -4.4066273243377907, -3.1497730012828362, -2.1539355246109393]], [[-1.0427009575469182, -4.4421507628283061, 4.6981075789821212, 0.33354438781718088], [-1.7993614761104681, 3.8567553357167359, -2.9189515487035655, 2.2452482328529575], [3.0261417383010603, 2.1481041392277875, -4.4526930233913689, -0.011526918493140137]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[5.1985309899152865, 5.736859325761154, 7.5542240599488348, 1.366949113054011], [4.3642080421340035, 4.3248288273127198, 0.38666075082460205, -0.25711071067507341], [1.7642901065822914, 1.4493246688661543, 1.2919682799532195, 0.73372655780642404]], [[8.4324279884661983, -0.34327038390318965, 0.16340557046959114, 8.4794795051697864], [1.2624924489296081, 0.90729857596389518, 3.7991348446109416, 7.1694077980784847], [2.2520422836559417, 3.5385709490763526, 4.4774522112531567, 3.9254889824163595]]], [[[8.6955235465923497, 0.46917995011209523, 5.6443663383888349, 7.2672799262698362], [0.077663402065167642, 8.0371960571330714, 4.5290866082348353, 4.7057935605316672], [0.59590359418590033, 3.4696106112970169, -0.13297261146831518, 4.4596427125991811]], [[8.4197224212993529, 7.4501840156454548, 6.0835951280165048, 8.5814241365416688], [2.0534563095843015, 3.37204851009602, 6.7532258731149994, 5.9882435314469591], [6.1582653806418168, 4.1218854363022501, 3.4237195460274306, 1.1042023795677158]]], [[[8.1988920456104371, 6.118350508165415, 7.226773756964457, 1.7335220817482195], [0.36658982076816704, 3.3129764871600305, 6.6887284117610646, 2.7487241550954797], [0.53049339401067996, -0.60593949280641901, 0.65091483024853547, 1.6467523069204324]], [[2.7579868739844535, -0.64146293129693444, 8.4987954105134929, 4.1342322193485526], [2.0013263554209035, 7.6574431672481076, 0.88173628282780614, 6.0459360643843292], [6.8268295698324319, 5.9487919707591592, -0.65200519185999717, 3.7891609130382315]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_constData_rank0(self):
      arg0=2.35482602484
      arg1=Data(-1.75764487301,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(0.597181151825,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_constData_rank1(self):
      arg0=-4.08948100306
      arg1=Data(numpy.array([-0.016471342920057808, -0.44847422512511859]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-4.1059523459788441, -4.5379552281839048]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_constData_rank2(self):
      arg0=-2.56745749608
      arg1=Data(numpy.array([[-2.3029390682099082, 1.4295953141991617, -0.56084530396881416, -2.8574339203423946, -2.9165894610513474], [2.3619178172931079, 0.86359445821872605, 4.6737388110442577, -1.4370611290572244, 1.5357863607532387], [0.45252277488710213, 2.2443491296499865, 1.0077804345422585, 1.8940556365182557, -0.084761000616873083], [-2.1888602716382231, 2.6468775075702276, -1.3190824924732691, -0.91818098816045168, 0.9617601275468699]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-4.870396564294289, -1.137862181885219, -3.1283028000531949, -5.4248914164267754, -5.4840469571357282], [-0.20553967879127288, -1.7038630378656547, 2.1062813149598769, -4.0045186251416052, -1.0316711353311421], [-2.1149347211972787, -0.32310836643439433, -1.5596770615421223, -0.67340185956612508, -2.6522184967012539], [-4.7563177677226038, 0.079420011485846764, -3.8865399885576499, -3.4856384842448325, -1.6056973685375109]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_constData_rank3(self):
      arg0=-0.709741937158
      arg1=Data(numpy.array([[[-0.061249786137445028, -4.8627991904153678], [-1.5999354808136426, -4.8067460897192831]], [[1.0893477333147414, -2.6279269220096579], [2.5218892229740906, -0.67056201455284992]], [[1.7185502503293097, 3.3400238069534858], [0.37738121324869223, -1.2257147190919793]], [[-0.66523522998579576, -2.3320065120603282], [-0.9845840332612088, 4.4499847257165506]], [[0.44211704567061005, -3.6129530518906172], [-4.3886297706270021, -1.373377459230082]], [[-2.8380278520902689, 1.2683977856655773], [3.0303778309363416, -1.5172989776410617]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-0.77099172329565757, -5.5725411275735803], [-2.3096774179718551, -5.5164880268774956]], [[0.3796057961565289, -3.3376688591678705], [1.812147285815878, -1.3803039517110625]], [[1.0088083131710972, 2.6302818697952732], [-0.33236072390952032, -1.9354566562501918]], [[-1.3749771671440083, -3.0417484492185407], [-1.6943259704194213, 3.740242788558338]], [[-0.26762489148760249, -4.3226949890488298], [-5.0983717077852146, -2.0831193963882946]], [[-3.5477697892484814, 0.55865584850736472], [2.3206358937781291, -2.2270409147992742]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_constData_rank4(self):
      arg0=-4.34128905836
      arg1=Data(numpy.array([[[[1.0357041081133929, -3.7517458309119367, -2.8947725548960088, 3.0395740583222128], [-2.8360178331578014, 4.2274371889777029, 3.5361632007109822, 2.6627044210714867], [2.2467890175752814, 1.461757496329489, -3.0618202279979787, -0.712590675791132]], [[3.4222196636825881, -4.1390931292808073, 3.9638940058277949, -4.081690756344365], [-4.6458877810031129, 0.75941968973279739, -4.2662221128849884, -0.34362371441538819], [0.53243754491381345, -1.7402345788400164, 0.39104344049589379, -4.6755369588983333]]], [[[-4.644752591725128, -3.3045799322296743, -1.656135180913072, -2.5590928922155176], [-1.1036992125334844, -0.70334792464694207, -1.8871914915830623, -0.75869375904431546], [-3.8201619786584553, 1.6179471249543473, -4.8789114487473864, 4.1084644106649542]], [[-2.0810800622981827, -3.1038195601766483, 4.0632065869562837, 2.9658044349255004], [4.4532595665785486, -3.4204073109291255, 0.77838217419011002, 2.7874885199997177], [0.3000812730739133, -0.10042490155611183, 3.1350652182413103, -3.7146876835951135]]], [[[3.1960547749300758, 2.401371034203386, -0.91651947626425123, 2.8809638561372122], [-0.073128925729001537, -4.4807884812897933, 4.4845487505744472, 1.6523767523700306], [-1.3204206159060119, -0.012579385009651389, -3.7232425291925919, 1.7162055184672234]], [[-0.18456994589283493, -2.60724993891706, -0.017599020742514959, -0.13610558129630768], [-2.9833660051000122, -0.56269798150890438, -2.1174354711901886, 4.7304193703197353], [-1.7697262835667695, -4.9820825514859468, 0.36812906382106902, -3.5720741638533338]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-3.3055849502427641, -8.0930348892680932, -7.2360616132521658, -1.3017150000339441], [-7.1773068915139584, -0.11385186937845404, -0.80512585764517475, -1.6785846372846702], [-2.0945000407808756, -2.8795315620266679, -7.4031092863541357, -5.053879734147289]], [[-0.91906939467356885, -8.4803821876369643, -0.37739505252836203, -8.4229798147005219], [-8.9871768393592699, -3.5818693686233596, -8.6075111712411463, -4.6849127727715452], [-3.8088515134423435, -6.0815236371961738, -3.9502456178602632, -9.0168260172544912]]], [[[-8.986041650081285, -7.6458689905858312, -5.997424239269229, -6.9003819505716741], [-5.4449882708896418, -5.044636983003099, -6.2284805499392188, -5.0999828174004724], [-8.1614510370146114, -2.7233419334018096, -9.2202005071035433, -0.23282464769120281]], [[-6.4223691206543396, -7.4451086185328048, -0.2780824713998733, -1.3754846234306566], [0.11197050822239163, -7.7616963692852821, -3.562906884166047, -1.5538005383564393], [-4.0412077852822437, -4.4417139599122688, -1.2062238401148466, -8.0559767419512696]]], [[[-1.1452342834260811, -1.9399180241527709, -5.2578085346204082, -1.4603252022189448], [-4.4144179840851585, -8.8220775396459494, 0.14325969221829027, -2.6889123059861264], [-5.6617096742621689, -4.3538684433658084, -8.0645315875487498, -2.6250835398889336]], [[-4.5258590042489919, -6.948538997273217, -4.3588880790986719, -4.4773946396524646], [-7.3246550634561691, -4.9039870398650613, -6.4587245295463456, 0.38913031196357828], [-6.1110153419229265, -9.3233716098421038, -3.9731599945350879, -7.9133632222094903]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_expandedData_rank0(self):
      arg0=1.76311838444
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.77226058376)+(1.-msk_arg1)*(0.412008256265)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.00914219932)+(1.-msk_ref)*(2.17512664071)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_expandedData_rank1(self):
      arg0=-0.504412364386
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.9921533258491984, -4.6259171358901092])+(1.-msk_arg1)*numpy.array([4.0129104531915214, -0.13988376090421895])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-4.4965656902356521, -5.130329500276563])+(1.-msk_ref)*numpy.array([3.5084980888050676, -0.64429612529067271])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_expandedData_rank2(self):
      arg0=2.17716970217
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.8131055362655282, -0.7514838229580505, -0.60033151487952807, -4.4289288135380147, 3.9785741860005306], [-2.6354999257010157, -4.6724232545247792, 1.9155575254103425, 3.7214301524682547, 4.2817101543718508], [-0.43867934697933642, 2.3417991468874053, -0.73276064853387091, 1.1962341396768794, -0.87364010850748208], [2.4575513647025682, 4.15927498681142, 4.9190841426120375, -1.7668155137055663, 4.6883638281449951]])+(1.-msk_arg1)*numpy.array([[3.319813755337309, 3.5237762525812482, -4.2566115213926459, -2.0581671219191322, 2.0848536870423917], [-3.2700592440192566, -0.41291068236184092, 0.59948607238558349, 1.2287223644060772, 0.90426641926430662], [1.6550694393095986, -4.7117674670511516, 4.6655741229383612, -0.70189591885743585, 2.8958959825961541], [4.9675001087382054, 1.5222503737275144, 2.040023207468761, -4.5201869835428488, 3.7182906048687983]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.9902752384398221, 1.4256858792162435, 1.5768381872947659, -2.2517591113637208, 6.1557438881748245], [-0.4583302235267217, -2.4952535523504853, 4.0927272275846365, 5.8985998546425487, 6.4588798565461447], [1.7384903551949575, 4.5189688490616993, 1.444409053640423, 3.3734038418511734, 1.3035295936668119], [4.6347210668768621, 6.336444688985714, 7.0962538447863315, 0.41035418846872762, 6.865533530319289]])+(1.-msk_ref)*numpy.array([[5.496983457511603, 5.7009459547555421, -2.0794418192183519, 0.11900258025516175, 4.2620233892166857], [-1.0928895418449627, 1.764259019812453, 2.7766557745598774, 3.4058920665803711, 3.0814361214386006], [3.8322391414838926, -2.5345977648768576, 6.8427438251126551, 1.4752737833168581, 5.0730656847704481], [7.1446698109124993, 3.6994200759018083, 4.2171929096430549, -2.3430172813685548, 5.8954603070430922]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_expandedData_rank3(self):
      arg0=-3.4277435804
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.3667655378960948, -0.040564208830195625], [-4.7398266228075077, -3.2412496716836614]], [[-2.125593021037254, -2.2407941583076854], [2.2372727603914431, -4.2790560801448274]], [[0.68720835205358011, 2.4059893868804307], [-2.4964395423263408, 3.1745348353480622]], [[-4.9075702105934997, -4.9431164153204303], [-0.36043730260638007, 0.42691490242330854]], [[1.4179477912549476, 3.326452959805998], [1.3560731213960029, -0.46102835566925382]], [[3.3782733559655824, 2.090288099439177], [-2.6857999013168654, 1.3585843360631298]]])+(1.-msk_arg1)*numpy.array([[[4.4938161858323067, 1.5931510430370563], [4.696507901895183, 3.8539864360665828]], [[3.4958480064806814, -4.9774435447926155], [-1.1442003125479094, -2.7341070988450089]], [[-2.5968388662322694, 4.6425307414676773], [0.022578761912285472, -1.6818160819470487]], [[-3.6684032758674068, -1.8490089454186887], [-1.3695984802097696, -3.7571783693651764]], [[-4.7345542971273904, 4.0275167708886741], [1.3537075831257699, -2.6886129591653916]], [[2.0970632990006521, -0.84516650168325214], [4.684668775643118, 1.8671292564746604]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-5.7945091183010167, -3.4683077892351175], [-8.1675702032124295, -6.6689932520885833]], [[-5.5533366014421759, -5.6685377387126072], [-1.1904708200134788, -7.7067996605497493]], [[-2.7405352283513418, -1.0217541935244911], [-5.9241831227312627, -0.25320874505685964]], [[-8.3353137909984216, -8.3708599957253522], [-3.788180883011302, -3.0008286779816133]], [[-2.0097957891499743, -0.10129062059892391], [-2.071670459008919, -3.8887719360741757]], [[-0.049470224439339461, -1.3374554809657448], [-6.1135434817217877, -2.0691592443417921]]])+(1.-msk_ref)*numpy.array([[[1.0660726054273848, -1.8345925373678655], [1.2687643214902611, 0.42624285566166087]], [[0.068104426075759505, -8.4051871251975374], [-4.5719438929528309, -6.1618506792499304]], [[-6.0245824466371918, 1.2147871610627554], [-3.4051648184926364, -5.1095596623519706]], [[-7.0961468562723287, -5.2767525258236105], [-4.7973420606146915, -7.1849219497700982]], [[-8.1622978775323123, 0.59977319048375222], [-2.074035997279152, -6.1163565395703134]], [[-1.3306802814042697, -4.272910082088174], [1.2569251952381961, -1.5606143239302614]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_expandedData_rank4(self):
      arg0=-4.82349391804
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.8109549360227235, -1.3578743659797397, -1.5307488465365049, 1.0995740321806471], [-1.8283475037924779, 0.25260880491340476, -1.4429213896834092, 2.3268743622905017], [4.9731871059293962, -2.063416595442801, 1.315376982673615, -3.809855869589418]], [[-4.8902650317303529, -4.7144665236613754, 1.5202146754225367, -1.931518729228785], [-3.8847500473125898, 4.3867702181759043, 1.8940301389418037, 2.4329833823640215], [-1.2082757809402835, -0.83047266047446833, 2.2612219009520551, 4.639906260760263]]], [[[-4.5922406054129619, -3.3090319850155634, -0.81711775519249485, -0.72102149605557209], [2.8051873600354966, -4.9304797325002925, 0.08450373603638095, 4.3824557957977515], [0.432042655953083, 2.1903859731742568, 4.5126338452224974, -1.8218060301476569]], [[2.2493555987865435, -4.1902867199417084, -2.3893780255933041, -4.147052641364497], [-2.1048598086596968, -4.6356381079134588, -4.276799709157685, -3.5315164247719744], [-2.3519110246426687, -1.6614443381712354, 2.9385286009249922, 4.0996351564150917]]], [[[1.7104570918047308, 0.2235597679830974, -3.4917239383453511, 0.87133958814354706], [-0.28810732352935453, 4.6278305539238556, 3.6035025224879966, -2.121177037151647], [-0.56589289362297279, 4.2940943961545059, -2.2108275043603243, -0.37651332795693104]], [[0.6578187034936267, -2.8655996123955427, -2.4907326443067368, -4.7898037053595619], [3.2321208553708622, 2.4831791881685152, 0.95312974395303929, 2.2609801150226314], [-1.7851896770877937, 0.42156063765933371, -1.8379248609183243, 4.2123297453359037]]]])+(1.-msk_arg1)*numpy.array([[[[-4.1321016159626023, -4.8113703944978559, 2.4028239884241085, -3.4302225408424492], [0.90351586424905506, 0.52660889258420962, 4.0435956180305546, -2.6607474227569639], [-0.7832580356081511, -0.33442954239381173, -1.2561360976342097, -2.2350468099654641]], [[1.1250963489572454, -0.87979079585863129, 2.1478695136255412, 4.1105578894489359], [-0.44340244267575279, -2.7961650387130863, -3.2323191921253391, 2.4342529430946094], [3.8445768917161285, -4.9716543799691459, -0.79269017613685744, 4.405102562986853]]], [[[4.5645304913391964, 4.9165683584068702, -3.2829387981742042, -0.53889403485878518], [-0.73921008647266273, 4.0350083353222193, -0.73020558781198819, -0.24064926836505052], [0.16438247300546571, -1.6446268542027909, 2.4503440972397268, -1.1907122824654914]], [[-4.1074300732924591, -0.067858202710286974, -0.028415585086349004, 4.4634408454547181], [4.1841128649341783, 4.1761493395633291, 2.4035029342319492, 3.4393059799507064], [-2.2004044533027267, -3.632821962195024, -2.3980108435935521, 4.5236186873410205]]], [[[2.8644780568283394, 1.659967686466743, 2.7197053129349635, -2.761375672874423], [-1.1773175597763896, 4.6301944373010588, -3.4489909340126945, 4.0249839112076931], [4.4993349080399536, -1.9642384639900801, 3.1916685340315727, -1.7621738983806843]], [[4.7831338763505507, 1.0031396748124122, -4.7052094854052484, -0.14144022568461789], [-4.5702356565627813, -3.0718436491226973, -3.4989330324035461, 2.526069954914564], [-4.6424659831601254, 4.5228542484255421, 3.2984796317107481, -1.1667722423294711]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.6344488540610982, -6.1813682840181139, -6.3542427645748791, -3.7239198858577272], [-6.6518414218308521, -4.5708851131249695, -6.2664153077217835, -2.4966195557478725], [0.14969318789102193, -6.8869105134811752, -3.5081169353647592, -8.6333497876277931]], [[-9.713758949768728, -9.5379604416997488, -3.3032792426158375, -6.7550126472671597], [-8.7082439653509631, -0.43672369986246995, -2.9294637790965705, -2.3905105356743528], [-6.0317696989786578, -5.6539665785128426, -2.5622720170863191, -0.18358765727811122]]], [[[-9.4157345234513361, -8.1325259030539385, -5.6406116732308691, -5.5445154140939463], [-2.0183065580028776, -9.7539736505386667, -4.7389901820019933, -0.44103812224062278], [-4.3914512620852912, -2.6331079448641175, -0.3108600728158768, -6.6452999481860306]], [[-2.5741383192518308, -9.0137806379800836, -7.2128719436316784, -8.9705465594028713], [-6.928353726698071, -9.459132025951833, -9.1002936271960593, -8.355010342810349], [-7.1754049426810429, -6.4849382562096096, -1.884965317113382, -0.72385876162328255]]], [[[-3.1130368262336434, -4.5999341500552768, -8.3152178563837253, -3.9521543298948272], [-5.1116012415677288, -0.19566336411451868, -1.2199913955503776, -6.9446709551900216], [-5.389386811661347, -0.5293995218838683, -7.0343214223986985, -5.2000072459953053]], [[-4.1656752145447475, -7.689093530433917, -7.3142265623451106, -9.613297623397937], [-1.591373062667512, -2.340314729869859, -3.870364174085335, -2.5625138030157428], [-6.608683595126168, -4.4019332803790405, -6.6614187789566985, -0.61116417270247059]]]])+(1.-msk_ref)*numpy.array([[[[-8.9555955340009774, -9.6348643125362301, -2.4206699296142657, -8.2537164588808238], [-3.9199780537893192, -4.2968850254541646, -0.77989830000781968, -7.4842413407953376], [-5.6067519536465253, -5.157923460432186, -6.079630015672584, -7.0585407280038384]], [[-3.6983975690811288, -5.7032847138970055, -2.675624404412833, -0.71293602858943839], [-5.266896360714127, -7.6196589567514605, -8.0558131101637134, -2.3892409749437649], [-0.97891702632224575, -9.7951482980075202, -5.6161840941752317, -0.41839135505152125]]], [[[-0.25896342669917782, 0.093074440368495992, -8.1064327162125789, -5.3623879528971594], [-5.562704004511037, -0.78848558271615499, -5.5536995058503624, -5.0641431864034248], [-4.6591114450329085, -6.4681207722411651, -2.3731498207986474, -6.0142062005038657]], [[-8.9309239913308325, -4.8913521207486612, -4.8519095031247232, -0.36005307258365615], [-0.63938105310419591, -0.64734457847504512, -2.4199909838064251, -1.3841879380876678], [-7.023898371341101, -8.4563158802333973, -7.2215047616319268, -0.29987523069735378]]], [[[-1.9590158612100348, -3.1635262315716313, -2.1037886051034107, -7.5848695909127972], [-6.0008114778147643, -0.1932994807373154, -8.2724848520510683, -0.79851000683068118], [-0.32415900999842062, -6.7877323820284543, -1.6318253840068015, -6.5856678164190585]], [[-0.040360041687823589, -3.8203542432259621, -9.5287034034436218, -4.9649341437229921], [-9.3937295746011564, -7.8953375671610715, -8.3224269504419208, -2.2974239631238103], [-9.4659599011984987, -0.30063966961283217, -1.5250142863276261, -5.9902661603678453]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_Symbol_rank0(self):
      arg0=4.58635587393
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-4.31091150464)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.275444369289)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_Symbol_rank1(self):
      arg0=4.4744829082
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-4.9680064667685464, -4.5511267989231508])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.49352355857036834, -0.076643890724972685])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_Symbol_rank2(self):
      arg0=-3.07454986574
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[4.1412478820857928, -2.7018642680402891, 3.3256994166022675, 0.52236922109603334, 1.8043946242411835], [-0.78685256991081953, -2.2055711693225066, 3.0058461269182555, -1.953299488338204, -4.2276674176271767], [0.96166936702801831, 1.9994193020553883, 0.48607828085469684, 2.7104735431531477, 3.867059074384473], [1.9476045779265743, 4.7176858648120952, 2.3019157431851802, -2.4668650012062687, -4.5197956265245676]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[1.0666980163485156, -5.7764141337775659, 0.25114955086499036, -2.5521806446412438, -1.2701552414960937], [-3.8614024356480967, -5.2801210350597838, -0.068703738819021698, -5.0278493540754816, -7.3022172833644543], [-2.1128804987092589, -1.0751305636818889, -2.5884715848825803, -0.36407632258412947, 0.79250920864719587], [-1.1269452878107029, 1.643135999074818, -0.77263412255209696, -5.5414148669435459, -7.5943454922618443]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_Symbol_rank3(self):
      arg0=3.46531781526
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-4.3363277001154756, -0.54529380075444678], [1.3226792458865324, 3.1370593530705033]], [[4.128129025541206, 4.6237219562851912], [-3.7545684418668115, 3.002100785453635]], [[4.6204731352838095, 2.4010071532077877], [-1.6875066466758035, -2.9580139131405723]], [[-0.40697800877773993, 1.3156291535055056], [-2.797725893076497, 0.40745980171637353]], [[1.0104155249317532, 0.86837294668118048], [-2.0201519317121508, 1.3816095350849187]], [[-2.7113588826521418, 4.505507148712292], [1.2691632140632816, 2.3427111590423149]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.87100988485109809, 2.9200240145099308], [4.7879970611509099, 6.6023771683348809]], [[7.5934468408055835, 8.0890397715495688], [-0.28925062660243395, 6.4674186007180126]], [[8.0857909505481871, 5.8663249684721652], [1.777811168588574, 0.50730390212380527]], [[3.0583398064866376, 4.7809469687698831], [0.66759192218788055, 3.8727776169807511]], [[4.4757333401961308, 4.333690761945558], [1.4451658835522267, 4.8469273503492962]], [[0.75395893261223579, 7.9708249639766695], [4.7344810293276591, 5.8080289743066924]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_float_rank0_Symbol_rank4(self):
      arg0=4.87448645573
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[4.8231015092065466, -0.8733006783309003, -0.84553967390328033, -2.6798567242803362], [2.9148704683834321, 2.276922490608662, 3.6068468113976415, -3.5955481389072985], [-3.6037339904415622, 1.6660812555497797, 0.15183434474671742, -3.9092222642251375]], [[-1.8325506914193133, 4.9815867978393413, 4.992160061922009, -4.9441017637029034], [-2.8238270749979599, 3.7516923570822147, 1.6240475821251277, -2.5168995609562472], [0.83527019818977788, 1.9084815442266398, -2.4039555678120705, -1.1228799890616772]]], [[[-1.9319677684520387, 3.0039415764432142, 2.1643237641285351, 3.0530576280047725], [-4.3684708166219046, 3.6195065112371516, 2.5199263303335968, 3.0495188889255811], [-0.56723461757061688, -4.0107725795996156, -0.11668341198683851, 1.9526617188813331]], [[2.3363733100504707, 0.052986334566856641, -3.8858316327649431, -1.9542104283766513], [-3.1405596805449756, 0.66354494486993243, -4.8654170889568062, -2.7260594614159848], [3.7841195940298018, 2.2546340560787348, -1.0779187218090849, -4.3073136199076476]]], [[[-2.5326470238674412, 1.9999160620696692, 3.7703940059030092, -4.9639748035653337], [0.9600399338422001, 3.5918472047290386, -3.7318537817438124, 0.51126590078121925], [-0.83768813180396684, -1.4420607470738087, -3.2099905387719563, 3.5942719280820601]], [[-2.0924192973912401, 1.5187396632730827, -1.8822693150129988, -1.9407219860386093], [2.8212923460942365, -2.3307209206797475, -0.18059144514730985, 3.8128688493899006], [-1.4454592948801412, 4.6565764980033002, 4.8523257855612751, 1.8231176092365731]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[9.6975879649394674, 4.0011857774020205, 4.0289467818296405, 2.1946297314525847], [7.7893569241163529, 7.1514089463415829, 8.4813332671305623, 1.2789383168256223], [1.2707524652913587, 6.5405677112827005, 5.0263208004796383, 0.96526419150778331]], [[3.0419357643136076, 9.8560732535722622, 9.8666465176549298, -0.069615307969982609], [2.0506593807349609, 8.6261788128151355, 6.4985340378580485, 2.3575868947766736], [5.7097566539226987, 6.7829679999595607, 2.4705308879208503, 3.7516064666712436]]], [[[2.9425186872808822, 7.8784280321761351, 7.0388102198614559, 7.9275440837376934], [0.50601563911101621, 8.4939929669700724, 7.3944127860665176, 7.9240053446585019], [4.3072518381623039, 0.86371387613330519, 4.7578030437460823, 6.8271481746142539]], [[7.2108597657833915, 4.9274727902997775, 0.98865482296797769, 2.9202760273562696], [1.7339267751879452, 5.5380314006028533, 0.0090693667761145846, 2.1484269943169361], [8.6586060497627226, 7.1291205118116556, 3.796567733923836, 0.56717283582527322]]], [[[2.3418394318654796, 6.87440251780259, 8.64488046163593, -0.089488347832412884], [5.8345263895751209, 8.4663336604619595, 1.1426326739891084, 5.3857523565141401], [4.036798323928954, 3.4324257086591121, 1.6644959169609645, 8.4687583838149809]], [[2.7820671583416807, 6.3932261190060036, 2.9922171407199221, 2.9337644696943115], [7.6957788018271573, 2.5437655350531734, 4.693895010585611, 8.6873553051228214], [3.4290271608527796, 9.531062953736221, 9.726812241294196, 6.6976040649694939]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_float_rank0(self):
      arg0=numpy.array(2.38317374412)
      arg1=2.36940652044
      res=add(arg0,arg1)
      ref=numpy.array(4.75258026456)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_float_rank0(self):
      arg0=numpy.array([1.6042930994751856, 1.007121679463852])
      arg1=1.32834203209
      res=add(arg0,arg1)
      ref=numpy.array([2.9326351315657506, 2.335463711554417])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_float_rank0(self):
      arg0=numpy.array([[-3.7532313430987605, 4.5302670575758714, -4.2580362689864639, -1.7120459202168217, -3.5319860499180402], [-1.3288856768048838, -2.3758863669706196, 3.6864682201450591, -3.8052095854722987, -4.9623101490208708], [-4.6001117997329519, -0.73642425828320501, -4.2922113350669644, -4.2102155238932673, 2.9769593503098877], [4.5551705175098558, -2.6992149606809193, 0.98747845345735463, 3.4562881818126261, -2.031282760391222]])
      arg1=1.85032943598
      res=add(arg0,arg1)
      ref=numpy.array([[-1.9029019071191628, 6.3805964935554691, -2.4077068330068663, 0.13828351576277598, -1.6816566139384426], [0.52144375917471386, -0.52555693099102196, 5.5367976561246568, -1.9548801494927011, -3.1119807130412731], [-2.7497823637533543, 1.1139051776963926, -2.4418818990873667, -2.3598860879136696, 4.8272887862894853], [6.4054999534894534, -0.84888552470132161, 2.8378078894369523, 5.3066176177922237, -0.18095332441162437]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_float_rank0(self):
      arg0=numpy.array([[[1.4910450657310372, 4.0173156155044847], [-1.2708076832213067, 4.3673140853814996]], [[1.690052116553372, -3.1533563655658545], [-0.38018556786379332, -2.6613086083113782]], [[3.0231634649823604, -3.3251144042183487], [3.7026206440395963, 1.1772299869005609]], [[-1.9309449738859086, 2.0076785635424113], [0.1594969257372556, -1.9487263974764444]], [[-1.0363406405435849, -0.66296928909350683], [3.4828811037771423, 1.613443903778057]], [[2.0094990195603737, -4.5677942385151766], [2.6684442004618081, -3.2902212208081916]]])
      arg1=-0.19805765964
      res=add(arg0,arg1)
      ref=numpy.array([[[1.2929874060906634, 3.8192579558641109], [-1.4688653428616805, 4.1692564257411258]], [[1.4919944569129981, -3.3514140252062283], [-0.57824322750416712, -2.859366267951752]], [[2.8251058053419866, -3.5231720638587225], [3.5045629843992225, 0.9791723272601871]], [[-2.1290026335262824, 1.8096209039020374], [-0.038560733903118205, -2.1467840571168182]], [[-1.2343983001839587, -0.86102694873388064], [3.2848234441367685, 1.4153862441376832]], [[1.8114413599199999, -4.7658518981555504], [2.4703865408214343, -3.4882788804485654]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[-4.6318713314440272, 0.32992122843829375, -4.3305065735956436, 0.66214105444459737], [-3.6774863753451239, 1.0048038926583125, 1.5940124503517712, -3.3335913986729295], [-2.5919188359554743, -3.0861565701458673, -0.27381459528204211, -2.3811572641514855]], [[-1.3087211054194348, 1.7502375221859845, 4.7394896808235654, -4.0263876500808928], [-2.5547537047463287, 0.16225810997317147, -3.0299656927454546, -2.020068231740729], [1.4562833182814527, -4.9929460066209197, 3.699792812778032, 4.2431200611001483]]], [[[0.72154828294466355, -1.9665432354872481, 2.9707147097543798, -0.05954393325510754], [-1.2504190901241454, 4.9750395461494641, -1.3861596996758632, 2.4153641408327911], [-4.3958531366669717, 1.2042227750044274, 1.2139633799816689, 0.38908250553922574]], [[1.6398099034499678, 4.1925440278441588, -1.62868396858486, -1.3482997564846624], [1.9275141767965494, -3.2404603042417293, -3.1154214119258681, 4.6245999631097838], [-2.1394990909765244, 3.106417922170003, -2.7957935977788075, 4.1246271568222763]]], [[[0.71355550405330881, 0.47250528402715286, 4.808883901738719, 2.7335995100102251], [-0.29039904918905624, 4.3168244070916355, -4.2995847680604404, -4.440315116333883], [4.0036535017096728, -4.5256899514529225, -3.8202870542297229, 1.2888173873195701]], [[3.8743301316245784, -2.6000803337068135, 4.4704236185911252, -3.3938647638192911], [1.8957700598353959, 3.3173347547763541, -2.0092060146361188, 2.7906778119742608], [-3.1189533989453744, -1.456658173030041, 2.7797291161980819, 4.4307400356790154]]]])
      arg1=-3.35661382179
      res=add(arg0,arg1)
      ref=numpy.array([[[[-7.9884851532337029, -3.026692593351382, -7.6871203953853193, -2.6944727673450783], [-7.0341001971347996, -2.3518099291313632, -1.7626013714379045, -6.6902052204626052], [-5.9485326577451501, -6.442770391935543, -3.6304284170717178, -5.7377710859411613]], [[-4.6653349272091109, -1.6063762996036912, 1.3828758590338897, -7.3830014718705685], [-5.9113675265360044, -3.1943557118165042, -6.3865795145351303, -5.3766820535304047], [-1.900330503508223, -8.3495598284105945, 0.34317899098835625, 0.88650623931047257]]], [[[-2.6350655388450122, -5.3231570572769238, -0.38589911203529592, -3.4161577550447833], [-4.6070329119138211, 1.6184257243597884, -4.7427735214655389, -0.94124968095688466], [-7.7524669584566475, -2.1523910467852483, -2.1426504418080068, -2.96753131625045]], [[-1.7168039183397079, 0.83593020605448309, -4.9852977903745357, -4.7049135782743381], [-1.4290996449931264, -6.597074126031405, -6.4720352337155438, 1.2679861413201081], [-5.4961129127662005, -0.2501958996196727, -6.1524074195684832, 0.76801333503260061]]], [[[-2.6430583177363669, -2.8841085377625229, 1.4522700799490433, -0.62301431177945066], [-3.6470128709787319, 0.9602105853019598, -7.6561985898501161, -7.7969289381235587], [0.64703967991999711, -7.8823037732425982, -7.1769008760193991, -2.0677964344701056]], [[0.51771630983490269, -5.9566941554964892, 1.1138097968014495, -6.7504785856089669], [-1.4608437619542798, -0.039279067013321622, -5.3658198364257945, -0.56593600981541492], [-6.4755672207350496, -4.8132719948197167, -0.57688470559159377, 1.0741262138893397]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_array_rank0(self):
      arg0=numpy.array(-0.00476703296164)
      arg1=numpy.array(0.298008631333)
      res=add(arg0,arg1)
      ref=numpy.array(0.293241598371)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_array_rank0(self):
      arg0=numpy.array([-4.2490656901064616, -3.6035258027505677])
      arg1=numpy.array(-4.95703994009)
      res=add(arg0,arg1)
      ref=numpy.array([-9.2061056301927309, -8.560565742836836])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_AAAadd_array_rank0_array_rank1(self):
      arg0=numpy.array(-4.19219670267)
      arg1=numpy.array([-1.4956124890848423, -1.9457489768254597])
      res=add(arg0,arg1)
      ref=numpy.array([-5.6878091917526552, -6.1379456794932725])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_array_rank0(self):
      arg0=numpy.array([[-1.2073469933316803, -2.5719257887179294, 2.3731574302185461, 2.559107598722397, 4.8120744496147729], [-0.80958434935306745, 1.6509570806468634, -3.1682777066983689, 4.088169114706707, 1.5789451859482329], [1.5798804850179131, -0.71661726542526694, -0.9242175880816097, -2.202474695573593, -1.8614529593729801], [-3.100515079656204, 0.63965392065915516, -2.5805545339083791, 0.37718816203634553, -3.9409679252680996]])
      arg1=numpy.array(3.52295258272)
      res=add(arg0,arg1)
      ref=numpy.array([[2.3156055893896594, 0.95102679400341028, 5.8961100129398858, 6.0820601814437367, 8.3350270323361126], [2.7133682333682723, 5.1739096633682031, 0.35467487602297076, 7.6111216974280467, 5.1018977686695726], [5.1028330677392528, 2.8063353172960728, 2.59873499463973, 1.3204778871477467, 1.6614996233483597], [0.42243750306513572, 4.1626065033804949, 0.94239804881296063, 3.9001407447576852, -0.41801534254675987]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_array_rank2(self):
      arg0=numpy.array(0.921717560057)
      arg1=numpy.array([[-2.6376606442722106, -2.5450350461679538, 3.0776063903864124, -1.2182844062274101, 2.1577351811933143], [-1.0500197144789878, 3.1018771810298702, -1.0374330157429759, 2.3802967978005807, -1.6964344556258992], [-2.8198673537061012, -2.202395728503046, -0.5814170546775097, -2.5917367046660491, 3.2960519461552682], [3.8262800364388525, 2.1942741486262136, 0.1267807558110583, 0.57807380916485762, -1.8391694035340644]])
      res=add(arg0,arg1)
      ref=numpy.array([[-1.7159430842156609, -1.6233174861114041, 3.9993239504429621, -0.29656684617086038, 3.079452741249864], [-0.12830215442243809, 4.0235947410864199, -0.11571545568642616, 3.3020143578571304, -0.77471689556934953], [-1.8981497936495515, -1.2806781684464963, 0.34030050537904, -1.6700191446094994, 4.2177695062118179], [4.7479975964954022, 3.1159917086827633, 1.048498315867608, 1.4997913692214073, -0.91745184347751474]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_array_rank0(self):
      arg0=numpy.array([[[-3.2843317895265756, -4.9180086412877646], [-0.65130274904348529, -4.2436854182413359]], [[-4.0664961097086625, 0.60045630744651834], [-1.8617735170547522, -2.2746468059171301]], [[3.4476543376040816, 1.8652721379327044], [-1.1093052324935302, 2.0735612251187181]], [[-1.6825172700066582, -3.186021603713165], [2.1768630501203523, -4.3283170194421441]], [[-0.13336388903442575, -3.0582002583830201], [2.2960378806573338, 1.6371890561810511]], [[-3.0904985408535635, -4.574808248245315], [3.0868496102863361, 0.77465847942717225]]])
      arg1=numpy.array(4.8896702396)
      res=add(arg0,arg1)
      ref=numpy.array([[[1.6053384500747754, -0.028338401686413661], [4.2383674905578657, 0.645984821360015]], [[0.82317412989268846, 5.4901265470478693], [3.0278967225465987, 2.6150234336842209]], [[8.3373245772054325, 6.7549423775340554], [3.7803650071078208, 6.963231464720069]], [[3.2071529695946928, 1.7036486358881859], [7.0665332897217032, 0.56135322015920686]], [[4.7563063505669252, 1.8314699812183308], [7.1857081202586848, 6.5268592957824021]], [[1.7991716987477875, 0.31486199135603599], [7.976519849887687, 5.6643287190285232]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_array_rank3(self):
      arg0=numpy.array(-1.28150406444)
      arg1=numpy.array([[[-1.5901370480611146, 4.6165155084884439], [-3.5121102970514082, 4.9430738821348505]], [[2.6504092274269357, -0.65352740450155178], [3.9280418461125315, -0.19076070360867448]], [[1.0588296225372176, 3.9169165624029141], [0.14844011577448235, 0.33505942459659988]], [[0.29007157367791603, -1.8012584802175202], [-3.9765803770039208, -4.6737934555461624]], [[-4.9237884110830787, 3.8840362158609842], [3.3349488661630371, -1.1874616233064641]], [[0.67507275712204606, 3.7545570346174131], [3.4486663014614809, -2.8194124659854323]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[-2.8716411125034185, 3.33501144404614], [-4.7936143614937121, 3.6615698176925466]], [[1.3689051629846318, -1.9350314689438557], [2.6465377816702276, -1.4722647680509784]], [[-0.22267444190508634, 2.6354124979606102], [-1.1330639486678216, -0.94644463984570404]], [[-0.99143249076438789, -3.0827625446598241], [-5.2580844414462247, -5.9552975199884663]], [[-6.2052924755253827, 2.6025321514186803], [2.0534448017207332, -2.468965687748768]], [[-0.60643130732025785, 2.4730529701751092], [2.167162237019177, -4.1009165304277362]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[0.26915919801196875, 4.7218230369143637, 0.025728303253822915, -2.1167001069129299], [-4.5666760046398682, -3.4506421509430374, 1.5590824350213488, -1.7748669413688423], [-1.1634257233823089, 2.3312523994832111, -3.1793983882974617, -0.2608033678460453]], [[-4.2278328955901214, 0.98252352566210099, -3.0727120470665414, 1.6656632962218243], [2.588482426482031, 2.2419545404571872, -4.2414668679059577, 3.9114581908945709], [3.0520617499571685, 0.51610212829791102, 1.7862143219029143, 1.0751396599179248]]], [[[-1.481546222429678, 0.26025799904837665, -2.1825064365298363, 3.1514948421007265], [0.52295579070889087, 4.672713832467398, 1.2357668443116356, 2.7255766039553979], [1.5290869563492215, 3.3341118230249265, 3.9527153268003055, 3.9842988957701628]], [[-4.9259862870156432, 2.4536024285155644, 3.1938115312732123, 4.9835854453082256], [-4.4163596622855703, 0.61545248779144579, -2.0834847650524067, 2.9342171453860368], [1.5710455072679537, 1.4599860070359449, -1.0142749445907184, -1.8777469105637854]]], [[[-1.9250040836312921, 2.4186035985949328, -1.4675920341397952, -2.9369263042079155], [-2.2148606460636824, 2.6910527606967438, 1.7740788985038778, 4.8452550561800738], [3.9690190658724731, -4.7054516494142042, -0.026426570524904491, -0.29849272600142829]], [[-3.2686873374417722, -2.9660615085482025, 1.1279523224212049, -0.16914692689189081], [-0.37483236233949846, 1.9182204864522499, 3.5414572953396721, 4.542192058473244], [-3.0991076731556841, -0.25878813319753924, 3.3703751606677983, 3.5165011039831402]]]])
      arg1=numpy.array(0.694051607941)
      res=add(arg0,arg1)
      ref=numpy.array([[[[0.9632108059526745, 5.4158746448550694, 0.71977991119452867, -1.4226484989722241], [-3.8726243966991625, -2.7565905430023316, 2.2531340429620546, -1.0808153334281365], [-0.46937411544160312, 3.0253040074239168, -2.4853467803567559, 0.43324824009466045]], [[-3.5337812876494157, 1.6765751336028067, -2.3786604391258357, 2.35971490416253], [3.2825340344227367, 2.936006148397893, -3.547415259965252, 4.6055097988352767], [3.7461133578978743, 1.2101537362386168, 2.48026592984362, 1.7691912678586306]]], [[[-0.78749461448897229, 0.9543096069890824, -1.4884548285891306, 3.8455464500414323], [1.2170073986495966, 5.3667654404081038, 1.9298184522523414, 3.4196282118961037], [2.2231385642899273, 4.0281634309656322, 4.6467669347410112, 4.6783505037108686]], [[-4.2319346790749375, 3.1476540364562702, 3.887863139213918, 5.6776370532489313], [-3.7223080543448646, 1.3095040957321515, -1.3894331571117009, 3.6282687533267426], [2.2650971152086594, 2.1540376149766507, -0.32022333665001268, -1.1836953026230796]]], [[[-1.2309524756905863, 3.1126552065356385, -0.77354042619908947, -2.2428746962672097], [-1.5208090381229766, 3.3851043686374496, 2.4681305064445835, 5.5393066641207795], [4.6630706738131789, -4.0114000414734985, 0.66762503741580126, 0.39555888193927746]], [[-2.5746357295010665, -2.2720099006074967, 1.8220039303619107, 0.52490468104881494], [0.31921924560120729, 2.6122720943929556, 4.2355089032803779, 5.2362436664139498], [-2.4050560652149784, 0.43526347474316651, 4.064426768608504, 4.2105527119238459]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_array_rank4(self):
      arg0=numpy.array(-3.50055541717)
      arg1=numpy.array([[[[-2.7921233792676881, 1.4518660542252091, -2.5957720416270824, 3.8469184699717971], [1.2561651413954502, 0.62237044380650364, -3.9771251495103446, 2.9006328668683619], [1.8254333604913509, 2.0555687076044276, 1.6865948185269932, -0.90614429975510546]], [[3.3654853061248993, -3.6552470865880746, -0.39173995437403697, 0.83073115192597058], [4.415907728954739, 1.9768512102477818, 0.65921396849240477, -3.3600753837269526], [4.6381460547428883, 3.7627830157940938, -1.7017417399383028, -4.4343460944632387]]], [[[3.3133004217013671, 0.87683971075770728, 1.4309825330623767, 3.5645482919792073], [1.4073075303718428, -1.478758692959393, -1.6604371093540058, -1.9843450208159386], [-3.4528042660171367, 4.1687085596909377, 3.7920459371255433, -4.5893267693698858]], [[-2.1877001826293538, 2.5425541388339941, -3.6139938712114206, -4.9161150905906217], [4.6935060733436202, -1.1385971046149699, -2.3689581494147074, 3.6847378761947791], [-3.0784966371524813, -3.3905314719188584, -1.2157108962925136, -3.5800237322103676]]], [[[1.3880053051070806, 1.642755393083597, -0.28251889773738448, -3.5510542010740096], [4.0211816099264386, -1.2953845856815271, -3.8654722551819445, -4.2229001313485357], [-4.6863789834596892, -2.1806763503075666, -3.6022170445351565, 3.0353160980312772]], [[-0.5068069165760134, 3.7628338599866904, -2.7379708610109632, -1.8285924429714373], [-4.9219909915260862, 2.5903520823976649, -1.327116424654621, 3.3799592191036183], [4.7425359478859583, 4.301895056839431, 3.5841178214520077, -2.212351247576505]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[-6.2926787964359683, -2.0486893629430707, -6.0963274587953622, 0.34636305280351731], [-2.2443902757728296, -2.8781849733617761, -7.4776805666786244, -0.59992255029991792], [-1.6751220566769289, -1.4449867095638522, -1.8139605986412866, -4.4066997169233852]], [[-0.13507011104338051, -7.1558025037563544, -3.8922953715423168, -2.6698242652423092], [0.91535231178645926, -1.523704206920498, -2.841341448675875, -6.8606308008952324], [1.1375906375746085, 0.26222759862581402, -5.2022971571065826, -7.9349015116315185]]], [[[-0.18725499546691271, -2.6237157064105725, -2.0695728841059031, 0.063992874810927525], [-2.093247886796437, -4.9793141101276728, -5.1609925265222856, -5.4849004379842183], [-6.9533596831854165, 0.66815314252265789, 0.29149051995726349, -8.0898821865381656]], [[-5.6882555997976336, -0.95800127833428572, -7.1145492883797008, -8.4166705077589015], [1.1929506561753405, -4.6391525217832497, -5.8695135665829872, 0.18418245902649932], [-6.5790520543207611, -6.8910868890871377, -4.7162663134607934, -7.0805791493786474]]], [[[-2.1125501120611991, -1.8578000240846828, -3.7830743149056643, -7.0516096182422894], [0.5206261927581588, -4.7959400028498074, -7.3660276723502243, -7.7234555485168155], [-8.186934400627969, -5.6812317674758468, -7.1027724617034362, -0.46523931913700256]], [[-4.0073623337442932, 0.26227844281841062, -6.2385262781792434, -5.3291478601397166], [-8.422546408694366, -0.91020333477061488, -4.8276718418229008, -0.12059619806466149], [1.2419805307176786, 0.80133963967115118, 0.083562404283727965, -5.7129066647447848]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_array_rank1(self):
      arg0=numpy.array([0.89204258467738828, -4.6241418717807292])
      arg1=numpy.array([-3.8432158609349876, -2.9712360581677766])
      res=add(arg0,arg1)
      ref=numpy.array([-2.9511732762575993, -7.5953779299485058])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_array_rank1(self):
      arg0=numpy.array([[-3.4310104540531681, -3.4078681636663499], [3.4632430705163877, 3.3617013756585266]])
      arg1=numpy.array([2.9026457109358228, -4.0769726819144818])
      res=add(arg0,arg1)
      ref=numpy.array([[-0.52836474311734527, -0.50522245273052713], [-0.61372961139809412, -0.71527130625595525]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_array_rank2(self):
      arg0=numpy.array([-0.81718210838440974, -1.4685466577903714])
      arg1=numpy.array([[2.1943352629319204, 4.6430102156284399], [-1.9195597281529677, -2.8719384327168909]])
      res=add(arg0,arg1)
      ref=numpy.array([[1.3771531545475106, 3.8258281072440301], [-3.3881063859433391, -4.3404850905072623]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_array_rank1(self):
      arg0=numpy.array([[[4.8480791661411846, -3.3139745022267677, 0.23075402943809387, 1.1460077243120201, 0.5430057803744992], [3.1966458427396578, -2.3099355752149684, -0.80993398761333601, 2.7377783492166818, -0.52642531146664595], [-2.3405623436273215, -4.6986809091986279, -4.5327719148713417, 1.0842206169324182, -1.3827159954219015], [-4.3314886100620251, 1.5439450020197185, -4.4576280346485646, -4.9473020620086841, -4.2100166474002618]], [[-0.64634838603928735, 1.6158881982668651, 3.2016038443099966, -4.6130610189609325, 3.3248292761667422], [-3.785156541201331, -2.5117751782092599, 4.8863625851326802, -4.1428745488986971, -1.0411200162174583], [2.1152254516342435, -1.455760072996922, 4.3665108820706671, 0.96848281645678469, 2.976394418880024], [-2.5254720531206565, 1.1119630787341013, 3.9988823099750075, 2.5472116254180266, 3.9352959539153396]]])
      arg1=numpy.array([2.9504404931533941, -1.0523978626173935])
      res=add(arg0,arg1)
      ref=numpy.array([[[7.7985196592945787, -0.36353400907337363, 3.1811945225914879, 4.0964482174654142, 3.4934462735278933], [6.1470863358930519, 0.64050491793842568, 2.1405065055400581, 5.6882188423700759, 2.4240151816867481], [0.60987814952607255, -1.7482404160452338, -1.5823314217179476, 4.0346611100858123, 1.5677244977314926], [-1.3810481169086311, 4.4943854951731126, -1.5071875414951705, -1.99686156885529, -1.2595761542468678]], [[-1.6987462486566809, 0.5634903356494716, 2.149205981692603, -5.665458881578326, 2.2724314135493486], [-4.837554403818725, -3.5641730408266534, 3.8339647225152866, -5.1952724115160906, -2.0935178788348519], [1.0628275890168499, -2.5081579356143155, 3.3141130194532735, -0.083915046160608853, 1.9239965562626304], [-3.57786991573805, 0.059565216116707731, 2.9464844473576139, 1.4948137628006331, 2.8828980912979461]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_array_rank3(self):
      arg0=numpy.array([-2.2898564034696278, 4.4193876348458776])
      arg1=numpy.array([[[1.3836448504950072, -1.2771567592674593, 0.99788358778879171, 4.0786129679533776, -2.418218626164589], [-4.1121013524402699, 0.46073007670794297, -1.6092950263915906, -1.0395441999201251, 4.2514484711087128], [2.192601754127236, -2.502892404580852, -2.6246331317839964, -4.7307392472393541, -2.75755102649231], [2.8350887708088042, 4.1435832734055627, -3.7452714656379174, -2.7760560469485429, -1.8766780930815852]], [[0.79300550194821717, 3.5500186866035826, 1.0620719781151697, -0.037605509149448935, -3.5097961357443683], [2.8069464999288982, 2.8516484824169144, -3.3431131243591992, 3.969787882211925, -0.70258592577141243], [-0.34354835337907019, -3.2843292735063825, 3.9198809886387984, 3.6019806857498882, -4.9849759940480904], [-4.9051958979035124, 3.3661470525649886, 2.3115462478411875, 2.6808046361223825, -2.340331435874111]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[-0.90621155297462064, -3.5670131627370871, -1.2919728156808361, 1.7887565644837498, -4.7080750296342169], [-6.4019577559098977, -1.8291263267616849, -3.8991514298612184, -3.329400603389753, 1.961592067639085], [-0.097254649342391808, -4.7927488080504794, -4.9144895352536242, -7.0205956507089819, -5.0474074299619378], [0.54523236733917635, 1.8537268699359348, -6.0351278691075452, -5.0659124504181712, -4.166534496551213]], [[5.2123931367940948, 7.9694063214494602, 5.4814596129610473, 4.3817821256964287, 0.90959149910150927], [7.2263341347747758, 7.271036117262792, 1.0762745104866784, 8.3891755170578026, 3.7168017090744652], [4.0758392814668074, 1.1350583613394951, 8.339268623484676, 8.0213683205957658, -0.56558835920221284], [-0.4858082630576348, 7.7855346874108662, 6.7309338826870651, 7.1001922709682601, 2.0790561989717666]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_array_rank1(self):
      arg0=numpy.array([[[[2.5103936566467668, -2.0390924910031338], [-2.1993452176878217, -0.34621614679682722]], [[-0.84514184021590744, 2.2740635493242785], [2.3783255651985371, 4.2061153353864604]], [[3.0226777401952134, 4.9022654560108556], [-0.87127978081883217, 2.2207059675799812]], [[2.6594602581843461, 4.6334712039864332], [2.1325620938727363, -4.0374969618148402]], [[-2.8668751868999562, 4.0512245963673479], [-0.13429656719510952, -3.0467988996922224]], [[-1.9167653590702285, -3.6416881564872297], [1.8147889611585653, -4.0915719127677601]]], [[[3.9280800303689887, 1.3496085240856353], [0.31038197427590308, 4.1641255475919561]], [[-2.9583427988061017, -4.3981765750537694], [-1.2693582533257963, 0.99005843926215675]], [[3.9624587605520372, -4.2817248602011855], [-1.2628416077771853, 4.3535767653713044]], [[-1.4949476331978695, 0.24311931631928907], [-3.6896773793773683, -4.2492218443129719]], [[0.59237224656472076, -4.003962799627045], [1.4188257014470738, 2.336164948449019]], [[-1.7007659405846534, 4.6697746346927698], [0.25130711141437256, -0.88866717142117135]]]])
      arg1=numpy.array([0.10797743018794215, 0.33402842836277635])
      res=add(arg0,arg1)
      ref=numpy.array([[[[2.618371086834709, -1.9311150608151917], [-2.0913677874998795, -0.23823871660888507]], [[-0.73716441002796529, 2.3820409795122206], [2.4863029953864793, 4.3140927655744026]], [[3.1306551703831555, 5.0102428861987978], [-0.76330235063089003, 2.3286833977679233]], [[2.7674376883722882, 4.7414486341743753], [2.2405395240606785, -3.929519531626898]], [[-2.7588977567120141, 4.15920202655529], [-0.02631913700716737, -2.9388214695042802]], [[-1.8087879288822863, -3.5337107262992875], [1.9227663913465074, -3.983594482579818]]], [[[4.262108458731765, 1.6836369524484116], [0.64441040263867944, 4.4981539759547324]], [[-2.6243143704433254, -4.064148146690993], [-0.93532982496301997, 1.3240868676249331]], [[4.2964871889148135, -3.9476964318384091], [-0.928813179414409, 4.6876051937340808]], [[-1.1609192048350931, 0.57714774468206542], [-3.3556489510145919, -3.9151934159501955]], [[0.92640067492749711, -3.6699343712642687], [1.7528541298098501, 2.6701933768117954]], [[-1.366737512221877, 5.0038030630555461], [0.58533553977714892, -0.554638743058395]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_array_rank4(self):
      arg0=numpy.array([3.7518071356804903, -1.9041790140670067])
      arg1=numpy.array([[[[-3.2728960625368533, -4.182975470946273], [0.50166107416151107, 1.9033756836243274]], [[-3.2818478146658103, -3.9612056433096132], [-1.6404990573785949, -1.4847107044277044]], [[-2.0558251600164024, -1.9052003868539327], [0.94295735376089418, -3.3755553862364995]], [[0.60535597726572821, -0.97580839450195533], [-0.28708633414262508, -1.542866243623795]], [[2.0312466148739716, -4.7206948220643348], [3.1162114064636448, 4.5032953474655955]], [[-1.1431940395194138, 0.060787170786806399], [0.30951226009041211, 0.89525908663356901]]], [[[4.2507806949460392, -3.4579965072722496], [2.0742526817133591, 4.8811003135629321]], [[0.56997803531888991, 0.78411016871976713], [0.48784369788951665, -1.8617012977648741]], [[3.0991254241361386, 0.11164154885224153], [3.6547124010490037, 1.6186370725000288]], [[4.7711681541482278, 1.4231043259148111], [-2.7007887058400026, 0.78876530217467344]], [[-0.91127101210405037, -0.58692331099470429], [-0.89256321606332989, -2.5196431582091239]], [[1.6181038941534549, 1.83026267412958], [2.8825122810413726, 1.8161724244498334]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[0.47891107314363701, -0.43116833526578269], [4.2534682098420014, 5.6551828193048177]], [[0.46995932101467996, -0.20939850762912293], [2.1113080783018954, 2.2670964312527859]], [[1.6959819756640879, 1.8466067488265576], [4.6947644894413845, 0.37625174944399076]], [[4.3571631129462185, 2.775998741178535], [3.4647208015378652, 2.2089408920566953]], [[5.7830537505544619, -0.96888768638384448], [6.8680185421441351, 8.2551024831460857]], [[2.6086130961610765, 3.8125943064672967], [4.0613193957709024, 4.6470662223140593]]], [[[2.3466016808790324, -5.3621755213392568], [0.17007366764635234, 2.9769212994959253]], [[-1.3342009787481168, -1.1200688453472396], [-1.4163353161774901, -3.7658803118318809]], [[1.1949464100691318, -1.7925374652147652], [1.750533386981997, -0.28554194156697799]], [[2.8669891400812211, -0.48107468815219567], [-4.6049677199070089, -1.1154137118923333]], [[-2.8154500261710571, -2.491102325061711], [-2.7967422301303366, -4.4238221722761306]], [[-0.28607511991355183, -0.073916339937426745], [0.97833326697436584, -0.088006589617173336]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_array_rank2(self):
      arg0=numpy.array([[-2.3932536057761733, 4.6044945009204756, 1.2999081070423539, -1.9592981433763956, 3.1646707378213481], [0.44074283211048382, 2.8140818237553411, 1.9533041067253789, 0.50594992425220475, -1.6329078312134113], [2.032167266226832, -3.1383228895803117, -3.4785914234267032, -2.6654509176332919, 1.1901692816981324], [-3.6728166346815261, -0.65516651098922818, 3.1900000481748094, 2.542120532552536, 2.8679967416023375]])
      arg1=numpy.array([[-3.8909305291257779, -1.8158618973461849, 4.7848218549372241, -4.4671585130773606, -3.7549945863186651], [-3.8595154057683678, 2.9370390337334031, -4.057756827572609, 3.6365495098807195, 3.6830760746398994], [-0.0079678139186034613, -1.0881170586162936, 0.43969247086615937, -1.1742947139199322, -0.21032239984684509], [3.4659017294010486, -3.0657575595410802, 4.8147426787990515, -3.7537774017847605, 1.1761183793296928]])
      res=add(arg0,arg1)
      ref=numpy.array([[-6.2841841349019507, 2.7886326035742908, 6.0847299619795781, -6.4264566564537562, -0.59032384849731701], [-3.418772573657884, 5.7511208574887442, -2.1044527208472301, 4.1424994341329242, 2.0501682434264881], [2.0241994523082285, -4.2264399481966048, -3.0388989525605439, -3.8397456315532241, 0.97984688185128732], [-0.20691490528047751, -3.7209240705303084, 8.0047427269738609, -1.2116568692322245, 4.0441151209320303]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_array_rank2(self):
      arg0=numpy.array([[[4.1175075159634247, 4.542167990108247], [0.19284380691997693, 3.0418399376068663], [1.1050408430711078, 0.41000056238432947], [3.8729434050398446, -4.3384428977277594], [1.4222030700165389, -3.4399298490028229]], [[-2.2891903651668066, -2.1598983658210491], [1.5520532076385578, 2.5255643449641161], [0.74223566849527156, -1.4055358511777403], [-0.25810909593639764, -1.1190540165460936], [-3.3580605136110755, 0.67182373083657243]], [[3.8761498320230903, 3.371979323190974], [-4.014381901748191, 2.9579498604360062], [0.63646979068009202, 4.3798700981207688], [0.15540201599420289, -2.4676594378473196], [1.7561044589929029, -0.56767801291649]], [[-1.1517304856208965, 3.3880304374066625], [4.92750601712698, 3.5166773877344255], [-3.0281395328985603, 0.93511927080632606], [-2.95617861025534, 0.88412573887346557], [-0.17936364232855073, 0.39782122018110755]]])
      arg1=numpy.array([[-0.11789777831407644, -2.2368626498282707, 1.0721412206455447, 2.7605271523683008, 1.521664133467552], [-2.0018876329299875, 1.7387958322649713, 4.3738825199665481, 3.3714013349220888, 2.5862112533881305], [-1.9667173744001798, 0.154122093956234, -1.5963229066842901, 4.2650913886356996, -2.1389528806427069], [-3.0883128948751901, 0.57342823614888161, 4.8828887394666989, -3.3314671084784484, 4.4112181735812364]])
      res=add(arg0,arg1)
      ref=numpy.array([[[3.9996097376493482, 4.4242702117941706], [-2.0440188429082937, 0.80497728777859567], [2.1771820637166526, 1.4821417830298742], [6.6334705574081454, -1.5779157453594586], [2.9438672034840909, -1.9182657155352709]], [[-4.2910779980967941, -4.1617859987510366], [3.2908490399035291, 4.2643601772290873], [5.1161181884618196, 2.9683466687888078], [3.1132922389856912, 2.2523473183759952], [-0.77184926022294498, 3.2580349842247029]], [[1.9094324576229105, 1.4052619487907942], [-3.860259807791957, 3.1120719543922402], [-0.95985311600419809, 2.7835471914364787], [4.4204934046299025, 1.79743195078838], [-0.38284842164980404, -2.7066308935591969]], [[-4.2400433804960862, 0.29971754253147243], [5.5009342532758616, 4.0901056238833071], [1.8547492065681386, 5.8180080102730249], [-6.2876457187337884, -2.4473413696049828], [4.2318545312526856, 4.8090393937623439]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_array_rank3(self):
      arg0=numpy.array([[1.5754164222679758, -2.5122850182046674, 3.7779621086136661, -4.8053171835989321, -1.8659105173687864], [0.74145530968957019, -2.380638358943715, 2.9186380590656515, 3.1143007502074109, -3.7619303773107458], [0.47725716770278659, 3.9883347825124833, 4.9592569646862774, 4.3261705099006349, 4.213223331194806], [-0.99918609600999719, -4.1864682183757012, 0.19682361200920617, -3.0549648809563621, -2.0833464414596192]])
      arg1=numpy.array([[[-3.1274742010681278, 4.4896862365691472], [-4.6080048930411905, -3.9170007056111631], [0.97431443901890802, -1.7912163693057472], [4.68680758237719, -0.92222761160976319], [2.9836463018379789, -2.1477362451632187]], [[-2.6730769359730977, 0.84442355341361619], [-4.8773294913408014, 2.4352659218511814], [-0.41172410237021495, -3.7554415716316791], [-2.1298934504786726, 1.8495051558421611], [2.7410046648788891, -0.87862956021926486]], [[0.36895914751083314, -1.7908748859480328], [-2.4713730134184067, 0.63921733010383797], [-1.4316215625115767, -1.4768578828908496], [0.86714361007569707, -1.111604239323094], [3.9851026118490527, 0.61519171679510443]], [[-1.3655194293849879, 2.1562280008039769], [1.6237025360956157, 2.2428800384139569], [-0.46462599420193929, -1.7382655960688389], [-4.7644473307255053, -2.6923244448857107], [-0.2798800027031545, 1.8777029637677565]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[-1.5520577788001519, 6.0651026588371231], [-7.120289911245858, -6.4292857238158305], [4.7522765476325741, 1.9867457393079189], [-0.11850960122174214, -5.7275447952086953], [1.1177357844691924, -4.0136467625320051]], [[-1.9316216262835275, 1.5858788631031864], [-7.2579678502845164, 0.054627562907466398], [2.5069139566954366, -0.83680351256602759], [0.98440729972873831, 4.9638059060495721], [-1.0209257124318567, -4.6405599375300106]], [[0.84621631521361973, -1.3136177182452462], [1.5169617690940767, 4.6275521126163213], [3.5276354021747007, 3.4823990817954278], [5.193314119976332, 3.2145662705775409], [8.1983259430438586, 4.8284150479899104]], [[-2.3647055253949851, 1.1570419047939797], [-2.5627656822800855, -1.9435881799617443], [-0.26780238219273311, -1.5414419840596327], [-7.8194122116818674, -5.7472893258420727], [-2.3632264441627737, -0.20564347769186275]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[-2.5046461689953903, -4.2244632383820102, -0.44723640804478126, 1.3998577615833803, 2.155746113856738], [-2.508276712143811, 4.2277819225053221, 0.60909994492729158, -2.6804491084216222, -4.9095874380789928], [0.74975630368198587, 3.6335900081332966, 1.3712615132026613, -2.8980820615440828, 2.2425760596786768], [-4.1079301196631226, -3.2093856313996802, -0.68731677342682573, 4.9764221748634778, 3.3563398761528838]], [[1.3129674663897282, 1.659104839627517, -2.6850117116704886, -0.28459698262047084, -3.9795511646822659], [4.9525903835483476, 1.7537063342619792, 1.5927053731413219, 0.39549172142929656, -4.7164304776388093], [2.523735720249082, 2.9467727799175911, 2.8723921003088551, -1.9352713309149561, -4.3150251208191168], [-1.2039703622958999, -0.84341441276691143, 2.5003719104538122, 0.55132588338546373, -0.92752708006642948]], [[-3.1018884373860836, 2.5489263704136018, 0.45416957021377957, 3.5099059943951687, 0.23872479658090828], [1.822709430369307, 1.7526109408276618, -0.46892156505811222, -4.1875932384122034, 1.9929089326436831], [-2.2652626627443553, -0.067063747500943904, -3.0410537440184218, -0.21360064073601048, 3.056142904815971], [1.4863397231698086, -2.5844494922934604, -0.18422928115497506, 4.933016947023253, 2.7544214182995832]], [[-2.4847830871570933, 3.5026959846750838, -3.5320768072732855, -1.7468192409379348, 4.8681480184851527], [3.4923242468946754, -3.2877365267507641, 4.1785084439837092, -4.4829715414771751, 0.35709129013571062], [-0.79773065573400714, -2.250304422919005, -2.0053393896242069, 0.23748449225979229, -1.1125258801075102], [-2.3715897629926417, -2.1388188126213903, 3.5858579671993382, -0.84038613848794519, -1.4354025678503159]], [[-3.2822113106664363, -0.13946122765855584, -2.7715850830573774, -2.0356895021517962, 2.0937754417481962], [3.7563823289770362, -2.0824275284824378, 4.2067400277325557, -3.9016016096824311, 0.17638738576000712], [4.8017902112210571, 1.92203919975735, -2.8893815775499156, -4.479413951418918, -0.55313927407352548], [-3.8761366567748787, -3.5396748187666982, -1.0850578182213386, 2.7857774453125996, 4.1788740765489187]]], [[[-1.9651382428352737, 4.8616489057900871, -0.43172513252083, -0.054859737031574873, -3.5460081800863499], [-4.810310812183678, 1.8207261716815388, -4.3223015938437017, 1.800839870029959, -2.2203603834124754], [3.565423638137295, 0.027824082459369848, -1.7836518092622877, -3.3258321999379681, 3.163457929000284], [-4.0052109881853406, -1.8148065222834298, 0.82215134704610282, 2.0193418039296462, 4.4471214980901177]], [[-2.8657059084201206, -2.3329499171320234, -2.2162961082830934, -4.8079117409277652, -2.197700947170361], [3.9715032123302638, 0.65999945078663913, 4.1588184818858913, -0.18755039575474175, 1.1354055022396237], [1.255487142958696, 3.1430236673767293, -2.1154988428417774, 3.0966701219915311, 4.3141393239523538], [-4.8817176883361775, 1.4600917371891615, 3.7487872365665975, -1.474314785809109, -4.2810653238151817]], [[-1.8231488954160158, -3.4814582839201034, 4.3885470815806986, -3.1970505743690323, -3.0658393815483653], [3.9247734660693734, -3.1793025189906166, -2.9237785337276612, -1.7240558515060345, -1.6817771153357008], [-3.888049614859499, 1.256416323201214, -1.7072992866185066, -0.58728091758395529, -4.6273047847960243], [2.367471620618268, 0.015018822799611087, -0.29060233151567516, 2.7957927875993116, -1.2703809614085335]], [[2.0555646534644678, -4.0824306185884076, -4.8327824602961291, -4.193455466679251, 0.80041064274131468], [-3.6646017876054318, 4.2540720928232041, -4.9703392037974892, -0.23928754654725548, 4.5828900717319083], [2.0304054011722954, -1.9727377879258667, 3.8533547747417565, -0.31921572896113215, -3.6275514939627773], [-4.5876717083999328, -0.017435221331201234, 2.0983134776536847, -1.2284916351829422, 4.1201597597518997]], [[3.9196394768490759, 4.9483270712052523, 4.423804818263612, 3.1975456400713167, 0.33426198637637494], [-2.4126015697079737, -1.3088448034883235, 4.9672641665700539, -2.7158551936396078, -0.76963649748323704], [2.4722783151214642, -0.3876458860667249, -1.6511204499442123, 4.1303250551320545, 0.9326495794030274], [3.3365736689270911, -2.517166846685762, 0.44906256710072778, -3.0373755159107141, -2.9303083956282228]]], [[[-4.3431662378354075, 2.539726773369023, 0.99280761534602568, -1.9255269883547435, -0.942727820741041], [-4.1906786823303275, -1.5795032340549744, 0.48073473127577682, 3.5941149454360843, -2.4617986342690923], [-1.7960257166064952, -1.4683147789759197, -0.27592876729811611, -3.0479691744265534, 4.1434875654704371], [-2.4694107325654899, -0.46694580675032782, -0.6312593190743776, -1.7644052849665193, -4.805487215552894]], [[2.4952363068057162, -4.2777524942321774, -3.514135851850698, 2.0251657196993431, 2.4287383907419589], [4.889568072257843, 3.9646414408752779, 2.7386990517708814, 4.6330716454066572, -2.2665108603911222], [-1.3416627285528615, -2.5377970468857503, 4.9447471742763103, 3.2433766620515438, -0.6189768011583876], [-0.28975918906883003, -0.16228377405352212, -2.2948398623911381, 0.35144736248781605, 1.7226694984576145]], [[-3.4623650092900062, -0.30920689303172733, -1.0786564545020241, -2.6213912883593826, -1.1921363280439348], [3.0482365564059783, -3.5867167982048054, 3.0671305985542485, -3.0463998211846697, -3.9376960536638848], [-1.1035079645712531, 4.1753410446853039, 4.3594794744424235, -4.5334021430069837, 1.5831648775871354], [-1.4665161865408418, -4.6241804083435181, -2.315925746483888, 0.42030641056100926, 3.4847920080173935]], [[0.95235423546563869, -3.9699986182355653, -3.3274941572181027, 2.5903943358283819, 1.1765259704473454], [0.75888780465270234, -4.1932042190534133, 0.70486414759477345, -0.62140463664983514, -1.2231194370797671], [3.7477215639862589, -4.2055889444542158, -3.1020409322238942, -0.68787830569812503, -0.46918980188225579], [4.4869097091540624, -1.2525021567272021, -2.1240312706522548, -4.2821221163563807, -0.70107303250834541]], [[2.4162623822794806, -0.73444331564069909, -1.5348011309226806, 2.0567342577268501, 4.8545025901058843], [-0.19331776405546197, 4.7283805410468496, -3.2375041502972204, 4.1591274218828467, 2.0825776580889723], [0.63191672076259486, -3.7385876363932748, -1.221832347010511, 4.0203887599549049, 0.82615965684098658], [-0.63897976149787894, -4.4624601894426261, 3.5928829259431954, 4.2083882710744689, -4.3247202178196975]]], [[[1.4228522934434906, 0.83110992155134067, -4.4503562920839359, -2.2601364529178838, -3.6643418532582963], [4.6761152839532212, -4.6536747983764517, 4.7158568110217232, -0.64622454680247188, 4.7054421295158679], [-4.3563300968027372, -0.37572079474052078, 2.1095019827661137, -2.545000196840749, 2.4277640414926207], [1.0527463392923178, 0.0047664586261797481, -1.6463238063161101, 1.0680524780654537, 2.2789934989529739]], [[3.1873879837728722, -4.727986513740559, 0.61925766979179198, 4.5387943081272084, -0.78462043193908926], [4.4821895186421763, -0.28680370469972161, -1.5535411363580165, -4.5521096647127326, -0.40626583141160921], [2.6194953404182009, -2.2881015723111755, -3.6612468100878215, 3.1623230810972043, -1.6830059633963956], [-4.6775773276507167, 3.7764648932906315, -3.3753223497614782, 4.708910124308332, -2.1537135004667487]], [[4.8158388485023025, 3.769593211478611, 0.065618812245730851, 4.4976556902119658, -2.2633031326963247], [-4.417295428567928, -2.3865943978852044, -1.633093481140441, 4.434975177416959, 1.4712910457977149], [4.073342194177064, 0.75395449351767763, 4.2431780655648907, 3.7079870633727374, 3.6092901491239022], [4.2832915796380906, -3.7511243406555095, 1.7863322393372609, -4.6883062726082301, 1.3643638533069771]], [[0.36619692998106945, -3.036742747198975, -2.9247432830413853, 1.0342886367741144, 2.8648250740561396], [3.128894426392602, -4.5947230267564452, -1.3577251801838655, 1.0244775401185757, 4.8860090210932299], [4.9489864309786462, -1.3877587203205142, 4.0659219734030607, 2.2187513948686677, -4.2979254917176233], [-1.7725791596616691, 1.244244693400006, -2.5758532954575974, -0.10299520609133239, 3.7741245426998908]], [[-0.75837447420683723, -0.19251739642004928, -1.4186453602426328, 4.6907726733294375, 3.5188772030276532], [3.2381148575477301, 2.5706485604029297, 4.9698764055856994, -0.10260633371904948, 0.64546397346189011], [-1.2377836524122712, 0.16839646874719882, 3.1642248496334417, 0.47992683064039721, -4.0951556783875773], [-0.51375573418917675, 0.31966893917731909, 4.292842409284793, -2.9934553339927037, 0.34048534129331376]]]])
      arg1=numpy.array([[-1.0906768054337936, -2.82761557168139, -2.1760186426021253, 3.6478506437752483, 0.24771354302712822], [-0.1889216392349411, 3.2270364688774951, -2.5623126148961237, -4.7246130377995543, 3.5364402640962052], [-1.4805507113197214, 0.93657904616790688, -1.3799205358396138, 4.1524254757674974, -4.1553290886582026], [0.39685970402536697, 2.3835254424075023, 1.2425962718187131, -2.793849174408586, 3.6313349551159284]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[-3.5953229744291839, -5.3151400438158038, -1.5379132134785749, 0.30918095614958663, 1.0650693084229443], [-3.5989535175776046, 3.1371051170715285, -0.48157686050650206, -3.7711259138554158, -6.0002642435127864], [-0.34092050175180777, 2.5429132026995029, 0.28058470776886768, -3.9887588669778764, 1.1518992542448832], [-5.1986069250969162, -4.3000624368334739, -1.7779935788606194, 3.8857453694296842, 2.2656630707190901]], [[-1.5146481052916618, -1.168510732053873, -5.512627283351879, -3.1122125543018608, -6.8071667363636559], [2.1249748118669576, -1.0739092374194108, -1.2349101985400681, -2.4321238502520934, -7.5440460493201993], [-0.303879851432308, 0.11915720823620113, 0.044776528627465062, -4.7628869025963461, -7.1426406925005068], [-4.0315859339772899, -3.6710299844483014, -0.32724366122757775, -2.2762896882959263, -3.7551426517478195]], [[-5.2779070799882088, 0.37290772781147652, -1.7218490723883457, 1.3338873517930434, -1.937293846021217], [-0.3533092122328183, -0.42340770177446352, -2.6449402076602375, -6.3636118810143287, -0.18310970995844222], [-4.4412813053464806, -2.2430823901030692, -5.2170723866205471, -2.3896192833381358, 0.88012426221384565], [-0.68967891943231674, -4.7604681348955857, -2.3602479237571004, 2.7569983044211277, 0.5784027756974579]], [[1.163067556618155, 7.1505466284503321, 0.11577383650196271, 1.9010314028373134, 8.5159986622604009], [7.1401748906699236, 0.36011411702448415, 7.8263590877589575, -0.83512089770192688, 4.0049419339109589], [2.8501199880412411, 1.3975462208562432, 1.6425112541510414, 3.8853351360350405, 2.5353247636677381], [1.2762608807826066, 1.509031831153858, 7.2337086109745865, 2.8074645052873031, 2.2124480759249323]], [[-3.0344977676393081, 0.10825231536857238, -2.5238715400302492, -1.787975959124668, 2.3414889847753244], [4.0040958720041644, -1.8347139854553096, 4.454453570759684, -3.6538880666553029, 0.42410092878713535], [5.0495037542481853, 2.1697527427844783, -2.6416680345227874, -4.2317004083917897, -0.30542573104639725], [-3.6284231137477505, -3.29196127573957, -0.8373442751942104, 3.0334909883397279, 4.426587619576047]]], [[[-2.1540598820702148, 4.672727266555146, -0.6206467717557711, -0.24378137626651597, -3.734929819321291], [-4.999232451418619, 1.6318045324465977, -4.5112232330786428, 1.6119182307950179, -2.4092820226474165], [3.3765019989023539, -0.16109755677557125, -1.9725734484972288, -3.5147538391729092, 2.9745362897653429], [-4.1941326274202817, -2.0037281615183709, 0.63322970781116172, 1.8304201646947051, 4.2581998588551766]], [[0.36133056045737444, 0.89408655174547169, 1.0107403605944016, -1.5808752720502701, 1.029335521707134], [7.1985396812077589, 3.8870359196641342, 7.3858549507633864, 3.0394860731227533, 4.3624419711171187], [4.4825236118361911, 6.3700601362542244, 1.1115376260357177, 6.3237065908690262, 7.5411757928298488], [-1.6546812194586824, 4.6871282060666566, 6.9758237054440926, 1.752721683068386, -1.0540288549376866]], [[-4.3854615103121395, -6.0437708988162271, 1.8262344666845749, -5.759363189265156, -5.6281519964444886], [1.3624608511732497, -5.7416151338867403, -5.4860911486237853, -4.2863684664021582, -4.2440897302318241], [-6.4503622297556227, -1.3058962916949097, -4.2696119015146303, -3.149593532480079, -7.189617399692148], [-0.19484099427785573, -2.5472937920965126, -2.8529149464117989, 0.23348017270318788, -3.8326935763046572]], [[-2.6690483843350865, -8.8070436563879611, -9.5573954980956835, -8.9180685044788053, -3.9242023950582396], [-8.3892148254049861, -0.47054094497635024, -9.6949522415970435, -4.9639005843468098, -0.14172296606764601], [-2.6942076366272589, -6.6973508257254206, -0.87125826305779785, -5.0438287667606865, -8.3521645317623321], [-9.312284746199488, -4.7420482591307556, -2.6262995601458696, -5.9531046729824961, -0.60445327804765459]], [[7.456079740945281, 8.4847673353014574, 7.9602450823598172, 6.7339859041675219, 3.8707022504725801], [1.1238386943882315, 2.2275954606078816, 8.503704430666259, 0.82058507045659734, 2.7668037666129681], [6.0087185792176694, 3.1487943780294803, 1.8853198141519929, 7.6667653192282597, 4.4690898434992326], [6.8730139330232962, 1.0192734174104432, 3.9855028311969329, 0.49906474818549107, 0.60613186846798239]]], [[[-5.8237169491551288, 1.0591760620493016, -0.4877430959736957, -3.4060776996744648, -2.4232785320607624], [-5.6712293936500489, -3.0600539453746958, -0.99981598004394456, 2.113564234116363, -3.9423493455888137], [-3.2765764279262166, -2.948865490295641, -1.7564794786178375, -4.5285198857462747, 2.6629368541507157], [-3.9499614438852113, -1.9474965180700492, -2.111810030394099, -3.2449559962862407, -6.2860379268726154]], [[3.4318153529736231, -3.3411734480642705, -2.5775568056827911, 2.96174476586725, 3.3653174369098657], [5.8261471184257498, 4.9012204870431848, 3.6752780979387882, 5.569650691574564, -1.3299318142232153], [-0.40508368238495462, -1.6012180007178434, 5.8813262204442172, 4.1799557082194507, 0.31760224500951928], [0.64681985709907686, 0.77429527211438476, -1.3582608162232312, 1.2880264086557229, 2.6592485446255214]], [[-4.8422855451296201, -1.6891274288713412, -2.4585769903416379, -4.0013118241989964, -2.5720568638835486], [1.6683160205663645, -4.9666373340444192, 1.6872100627146347, -4.4263203570242835, -5.3176165895034986], [-2.4834285004108669, 2.79542050884569, 2.9795589386028096, -5.9133226788465976, 0.2032443417475216], [-2.8464367223804556, -6.0041009441831319, -3.6958462823235019, -0.95961412527860457, 2.1048714721777797]], [[5.1047797112331361, 0.18242685753193211, 0.82493131854939472, 6.7428198115958793, 5.3289514462148428], [4.9113132804201998, -0.040778743285915908, 4.8572896233622709, 3.5310208391176623, 2.9293060386877303], [7.9001470397537563, -0.053163468686718396, 1.0503845435436032, 3.4645471700693724, 3.6832356738852416], [8.6393351849215598, 2.8999233190402953, 2.0283942051152426, -0.12969664058888331, 3.451352443259152]], [[-1.739066706378722, -4.8897724042989017, -5.6901302195808832, -2.0985948309313525, 0.69917350144768164], [-4.3486468527136646, 0.57305145238864696, -7.392833238955423, 0.0037983332246440682, -2.0727514305692303], [-3.5234123678956077, -7.8939167250514775, -5.3771614356687136, -0.13494032870329775, -3.329169431817216], [-4.7943088501560815, -8.6177892781008296, -0.56244616271500725, 0.053059182416266282, -8.4800493064779001]]], [[[1.8197119974688576, 1.2279696255767076, -4.0534965880585689, -1.8632767488925168, -3.2674821492329293], [5.0729749879785881, -4.2568150943510847, 5.1127165150470901, -0.24936484277710491, 5.1023018335412349], [-3.9594703927773702, 0.021138909284846186, 2.5063616867914806, -2.148140492815382, 2.8246237455179877], [1.4496060433176847, 0.40162616265154671, -1.2494641022907431, 1.4649121820908206, 2.6758532029783408]], [[5.5709134261803746, -2.3444610713330567, 3.0027831121992943, 6.9223197505347107, 1.598905010468413], [6.8657149610496786, 2.0967217377077807, 0.82998430604948581, -2.1685842223052303, 1.9772596109958931], [5.0030207828257032, 0.095423870096326802, -1.2777213676803192, 5.5458485235047066, 0.70051947901110667], [-2.2940518852432144, 6.1599903356981338, -0.99179690735397585, 7.0924355667158343, 0.22981194194075361]], [[6.0584351203210156, 5.0121894832973242, 1.308215084064444, 5.7402519620306789, -1.0207068608776115], [-3.1746991567492149, -1.1439981260664913, -0.39049720932172782, 5.6775714492356721, 2.713887317616428], [5.3159384659957771, 1.9965507653363908, 5.4857743373836039, 4.9505833351914506, 4.8518864209426154], [5.5258878514568037, -2.5085280688367964, 3.0289285111559741, -3.4457100007895169, 2.6069601251256902]], [[-2.4276522444275166, -5.830591921607561, -5.7185924574499714, -1.7595605376344716, 0.070975899647553575], [0.33504525198401591, -7.3885722011650312, -4.1515743545924515, -1.7693716342900103, 2.0921598466846438], [2.1551372565700602, -4.1816078947291002, 1.2720727989944747, -0.57509777953991836, -7.0917746661262093], [-4.5664283340702552, -1.5496044810085801, -5.3697024698661835, -2.8968443804999184, 0.98027536829130479]], [[2.8729604809090912, 3.4388175586958791, 2.2126895948732956, 8.3221076284453659, 7.1502121581435816], [6.8694498126636585, 6.2019835155188581, 8.6012113607016278, 3.5287286213968789, 4.2767989285778185], [2.3935513027036572, 3.7997314238631272, 6.7955598047493702, 4.1112617857563256, -0.46382072327164892], [3.1175792209267517, 3.9510038942932475, 7.9241773644007214, 0.63787962112322472, 3.9718202964092422]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_array_rank4(self):
      arg0=numpy.array([[2.7785956856444551, -1.0393254760391333, 3.7940467324892797, -0.12514562601400669, 4.8857420051683214], [2.2324138667562918, -4.9541437567872588, 0.75646215941666295, -2.6994591144076905, -3.2278522642315135], [-3.7428426325668838, -0.37559306525741043, -0.87685296648245892, -4.336115919934354, 2.3476613452902697], [1.4843752930855727, 3.3863386142003851, -0.97620494691586046, 1.5116871740876743, -4.1640308199522345]])
      arg1=numpy.array([[[[-2.8847140903626856, 4.5301005762052693, -4.4034045124846202, 2.4021072991142827, 3.6802514386058185], [-3.2499148960161497, -1.5309022853249941, -0.28698632160374604, -4.802100019776673, -1.3580072547924149], [1.1897370136052379, -1.8541180586965353, 1.1905335743937249, 0.98608970192652823, 4.9080431314365196], [-3.6960430064000871, -4.4745140718412282, 1.5651250483370589, -3.6425598343391097, -2.2911821203391716]], [[-1.0897711019446632, -3.926186299652926, 0.09570038807585135, -4.4975213454271357, -0.73580857039537584], [-0.67306273425968843, 2.9675463603572885, -3.4780918458823704, 2.4005737959966735, 3.0900728486777069], [1.5292896345525602, 2.8335187391790226, -3.9143485292665536, -0.83290037325656296, 1.4555105062554707], [-2.8876279844146238, 2.4418605759064294, 4.1485259351670969, -4.2862662704654033, -4.7427369837838658]], [[3.6259395982621339, -4.5198255010881674, 3.131571367171853, -3.0985809558640618, 3.5465311226191467], [4.376169548811955, -3.4854404730252995, -1.536196268405905, -2.1400458689959709, 1.8757652986797275], [-2.4774016727229609, -0.96140973105898553, 3.1349454124501595, 0.16471206082226075, -2.8026604307834257], [1.4299950015943574, -0.26882874834537773, 3.2470689419838159, -4.3563815357327682, 3.3610635670085891]], [[2.4064173010289638, -1.1621704339914753, 4.3605930803797719, -1.3640529768808172, 4.1282587795821808], [-1.8583221730572275, 3.9496332501238189, 1.4709522163298994, 4.444279900522119, 3.092735113270308], [3.3259941320841548, 3.3252840560471721, -0.016158424349731249, 1.5409925016276933, 2.4395599061035336], [4.2439903692753909, -3.467636597811532, 0.78167947098890522, 4.2522198035696377, -3.5398830108135004]], [[0.66403701661435655, 0.52931205243668167, 1.7258772857054261, -4.4293684470709795, -0.17702270736168302], [-1.5151784408865376, -4.6919572695385572, 2.5977855379644126, 2.1522758996844047, -2.1744716801212647], [3.5041261323448332, 4.0865873291299817, -4.3207649310661367, 1.2833932956384828, -0.56441804397003192], [0.54274290867358932, -3.1995644074126717, -2.2013877785580362, -3.2196546811052151, 0.57255639726768504]]], [[[-2.7016732105111263, 4.4590195279834646, 3.7569970286655483, 4.5612972155969107, 0.0040470053263330641], [3.9561160353484386, 4.0578643792316846, 3.2238472475057893, 4.7724180072345135, -0.43602100619855388], [1.6193169847368791, 3.2557273126151465, -2.193997896320854, 2.2369361709801172, -4.0763587987537706], [-4.3239913730293322, 0.86013618492116528, 1.4709880518330776, -2.5777870725550667, 1.3089454403599854]], [[1.51921641306462, -3.3786016782210804, -2.9733472211441248, 2.9158862748378356, 3.7721296342959025], [1.3095939422083305, -4.5430643818949745, -1.4445437326138002, 3.3016416917852602, 2.7014977402164639], [4.4456018151117469, 3.0023140434470754, 2.0198931145250167, 2.9518058006842463, -1.862843053670872], [-4.437246823047106, 0.11507857119042697, -4.4600559187197097, -2.8581203795110763, 0.60717877365178019]], [[-0.6425928026430423, 4.50281242803376, 2.678989380468316, 1.9662371075655019, -3.276065293070598], [4.4234399746434825, 4.8135855302617259, -4.153700385296113, 2.1791101884587141, -3.1630037632694483], [-1.0744589847388974, -1.7032701337189948, -1.3273591262929587, 4.8229304768164329, -1.1516163636243961], [-4.1489350261017144, -4.377635324133812, -0.81303736071653621, -1.9337221487515244, 1.5009441445418501]], [[-2.3726581318642315, -3.1395974449848385, 3.2573925878028156, -1.2658404652379582, 0.69325414564968302], [-2.9281378667995748, 3.8845226967324642, -1.6446557591456945, -2.5008315930549072, 4.1182479561692489], [-1.9368488217839896, -3.4587756297242258, 1.8434807358519842, -3.7200861159528742, 4.2296601538462664], [-0.76643214811182681, -2.8106568768848685, -4.7211019057619446, -1.7665660887889336, -4.9401747812272303]], [[1.7702592182688033, -3.3779427523225491, -3.5532155650281871, 2.4161310111659606, 1.3849226082467405], [-2.6123980723637095, 2.9271667522470066, -3.8932293174612465, 0.65657321978474048, 1.9338766313448108], [-3.7373302000663133, 2.6673863371485096, 3.6087327685482897, 4.239814403707749, 4.689904089789545], [-0.19879733194320881, 3.1359415174600578, -4.1317024809820655, 1.3462064935530815, 2.8511944377283607]]], [[[-3.0454553224688361, -4.2624408774261093, -4.4118554678479409, 0.66982913023489665, 4.6211567643126585], [1.4088360447092629, 4.3936800683534614, 1.8451977126727108, -2.1891816198767158, -3.121334682702682], [2.5064923315433258, 0.65094206102781449, 1.2464566210918262, 4.3689464406243737, -4.3331109320735131], [-3.0289163690739529, 3.8826101653079892, 4.9636148802069897, -4.6771403157328537, 2.0215955063801463]], [[1.0940751188053754, -4.5315780362387148, 3.2502382225270772, 1.0185935980409697, -3.4518468274315275], [1.143936439583527, -3.4946888078283687, 2.5540781733872207, 4.0348801776151042, -1.8055477998468037], [3.9974746326618096, -3.4348522299648412, 1.0048350859357509, -0.34758080285042503, -1.645439618265454], [2.2844880282555717, 1.3896187521218017, -4.3688955805569964, 0.64393661751579767, -4.8252783444973071]], [[2.5230556892516915, 1.3496790494839388, -0.3045618509010275, -0.940671726167297, -3.9910008581945586], [-2.8222849160619834, 2.0297124751402027, 4.7351236689737295, -4.711100762315815, 1.6785625304158653], [-4.8437727791014309, 3.6720220859611405, -0.828047438364238, 3.9675346976625843, -1.2743032008633302], [4.0762048533890205, 1.5537047647647455, 1.4038285639928052, -3.7453487779312744, -2.5078754972757089]], [[-1.551089216489304, -2.2071102539252885, -0.78567153627260655, 2.6234871709890015, -1.0845985325691574], [-0.18373918320027638, 0.52831829540759045, -4.858416702056755, 4.0001409033278019, -2.1605985079408532], [3.8651454846757307, -3.8361478713967823, 3.4086370963296435, -2.2383766311383155, -1.2995949731349055], [0.95216316432277637, 2.3807474676291847, -4.499399236647009, 1.8293615168857453, 2.9268640537504318]], [[-4.1874086878639289, 0.94762878545914564, -4.6290151596723863, -2.3462476741666394, -1.4991299839160668], [1.1557559051894231, 2.9592217749640506, -3.4531515925716558, -4.3326058013668813, 0.35062396135482388], [0.10352236359494693, -2.0426708702901974, -1.3840906791920271, -1.6073003903364089, 2.6272340858201373], [1.2955086615687508, 2.1840269186199235, 3.90212999009497, 2.8200225783397315, -0.96679304813221556]]], [[[0.15545636318707423, 0.37792566461955523, 0.083593262087995335, 1.9533265471393584, -0.45360148784276255], [-4.6849891832749986, 0.68937076545058318, 4.4623581089677629, -0.71908778542911556, -3.9689950202529358], [3.0624541145644031, -0.4949617851494903, -2.3826558233667319, -3.8709747741242451, -0.62316909126761377], [0.49817349136570499, 4.1933492142203672, -0.99615549414611504, 1.7468266971962345, -2.3054840679070798]], [[4.9226463499819566, 3.2243417174853057, -3.6596392314224802, -4.0112038676730899, 3.9974552383118098], [0.46587390156158826, -0.26380834905648598, -1.3470978764117971, -3.7118005097282025, -4.3217257683668562], [4.8206762475484659, -4.8885067763001615, 3.7138504405481214, 3.1791768251986401, -4.4577029401778496], [4.0467484201025101, -3.214657079853882, 4.6787753759780824, 2.5408753408482232, -4.9423230438682335]], [[1.5537820335850929, -1.2763186448429984, 4.5331511095952539, 1.7629570228112481, 3.2835515714634678], [3.254693669774527, 2.5643144799706459, -0.36508941354492386, 2.7885017029698087, -3.2616751728609072], [-1.2643555895033485, -3.1943819839230558, 1.459205354087425, 1.1637310685652213, -3.619639200426831], [3.8361332865156594, 0.99260865879742255, 2.7977695419162014, -4.4585762014343864, -1.328146560512443]], [[4.5054662661344942, 2.350341292722085, 1.0886420953859481, 4.4743765301108098, -0.44574632791776381], [-0.81085411680797392, -0.64734497403316205, 0.057850900395445493, 2.4155852363008679, 2.8781648489694858], [4.3487155489847478, 2.0416359899942762, -4.0760289011860795, 0.54249361158190545, 1.5217232998482562], [-0.98397449707201989, 3.53294823657858, 1.429483869647167, -4.5811681836295817, -2.9242304662391847]], [[2.0229210353033018, -2.9356226982368061, -3.8900531544155834, -3.7231423921993558, 3.535500551460709], [-2.1408847677427199, 1.6049118743800728, 3.8652807076461286, -0.048303121370598667, 2.1453571673386804], [-2.4540666038841188, -4.6319275387934367, -3.3056720735845713, -3.9882984079943617, 1.8726959635112985], [-2.1564254511009073, -1.820671513792782, 3.6224199176151863, -0.41636721544821143, -2.5621075239623217]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[-0.10611840471823042, 7.3086962618497244, -1.624808826840165, 5.1807029847587378, 6.4588471242502736], [-0.47131921037169455, 1.2476934003194611, 2.4916093640407091, -2.0235043341322179, 1.4205884308520402], [3.968332699249693, 0.92447762694791979, 3.96912926003818, 3.7646853875709834, 7.6866388170809747], [-0.91744732075563196, -1.695918386196773, 4.343720733981514, -0.86396414869465454, 0.48741356530528357]], [[-2.1290965779837965, -4.9655117756920593, -0.94362508796328193, -5.5368468214662689, -1.7751340464345091], [-1.7123882102988217, 1.9282208843181552, -4.5174173219215037, 1.3612483199575403, 2.0507473726385737], [0.4899641585134269, 1.7941932631398894, -4.9536740053056869, -1.8722258492956962, 0.41618503021633746], [-3.926953460453757, 1.4025350998672961, 3.1092004591279636, -5.3255917465045366, -5.782062459822999]], [[7.4199863307514136, -0.72577876859888768, 6.9256180996611327, 0.69546577662521791, 7.3405778551084264], [8.1702162813012347, 0.30860625946398024, 2.2578504640833748, 1.6540008634933088, 5.6698120311690072], [1.3166450597663188, 2.8326370014302942, 6.9289921449394392, 3.9587587933115405, 0.99138630170585396], [5.2240417340836371, 3.525217984143902, 7.0411156744730956, -0.5623348032434885, 7.1551102994978688]], [[2.2812716750149571, -1.287316060005482, 4.2354474543657652, -1.4891986028948239, 4.0031131535681741], [-1.9834677990712342, 3.8244876241098122, 1.3458065903158927, 4.3191342745081123, 2.9675894872563013], [3.2008485060701481, 3.2001384300331654, -0.14130405036373794, 1.4158468756136866, 2.3144142800895269], [4.1188447432613842, -3.5927822238255387, 0.65653384497489853, 4.127074177555631, -3.6650286368275071]], [[5.5497790217826779, 5.415054057605003, 6.6116192908737474, 0.45637355809734181, 4.7087192978066383], [3.3705635642817837, 0.19378473562976417, 7.4835275431327339, 7.038017904852726, 2.7112703250470567], [8.3898681375131545, 8.972329334298303, 0.56497707410218467, 6.1691353008068042, 4.3213239611982894], [5.4284849138419107, 1.6861775977556497, 2.6843542266102851, 1.6660873240631062, 5.4582984024360064]]], [[[-0.46925934375483447, 6.6914333947397564, 5.9894108954218401, 6.7937110823532025, 2.2364608720826249], [6.1885299021047304, 6.2902782459879765, 5.4562611142620812, 7.0048318739908053, 1.7963928605577379], [3.851730851493171, 5.4881411793714383, 0.038415970435437874, 4.469350037736409, -1.8439449319974788], [-2.0915775062730404, 3.0925500516774571, 3.7034019185893694, -0.34537320579877484, 3.5413593071162772]], [[-3.4349273437226389, -8.3327454350083396, -7.9274909779313836, -2.0382574819494232, -1.1820141224913563], [-3.6445498145789283, -9.4972081386822325, -6.398687489401059, -1.6525020650019986, -2.2526460165707949], [-0.50854194167551192, -1.9518297133401834, -2.9342506422622421, -2.0023379561030126, -6.8169868104581308], [-9.3913905798343649, -4.8390651855968319, -9.4141996755069677, -7.8122641362983352, -4.3469649831354786]], [[0.11386935677362064, 5.259274587450423, 3.435451539884979, 2.7226992669821648, -2.5196031336539351], [5.1799021340601454, 5.5700476896783888, -3.39723822587945, 2.935572347875377, -2.4065416038527854], [-0.3179968253222345, -0.94680797430233188, -0.57089696687629576, 5.5793926362330959, -0.39515420420773317], [-3.3924728666850514, -3.621173164717149, -0.056575201299873257, -1.1772599893348614, 2.257406303958513]], [[-5.0721172462719224, -5.839056559392529, 0.55793347339512511, -3.9652995796456487, -2.0062049687580075], [-5.6275969812072653, 1.1850635823247737, -4.3441148735533854, -5.2002907074625977, 1.4187888417615584], [-4.6363079361916801, -6.1582347441319163, -0.85597837855570624, -6.4195452303605647, 1.5302010394385759], [-3.4658912625195173, -5.510115991292559, -7.420561020169635, -4.4660252031966241, -7.6396338956349208]], [[-1.4575930459627102, -6.6057950165540626, -6.7810678292597011, -0.81172125306555287, -1.842929655984773], [-5.840250336595223, -0.30068551198450688, -7.12108158169276, -2.571279044446773, -1.2939756328867027], [-6.9651824642978273, -0.56046592708300391, 0.38088050431677622, 1.0119621394762355, 1.4620518255580315], [-3.4266495961747223, -0.09191074677145572, -7.359554745213579, -1.881645770678432, -0.37665782650315283]]], [[[-6.7882979550357199, -8.0052835099929922, -8.1546981004148247, -3.0730135023319871, 0.87831413174577477], [-2.3340065878576208, 0.6508374357865776, -1.8976449198941729, -5.9320242524435995, -6.8641773152695658], [-1.2363503010235579, -3.0919005715390693, -2.4963860114750576, 0.62610380805748989, -8.0759535646403968], [-6.7717590016408362, 0.13976753274110543, 1.2207722476401059, -8.4199829482997366, -1.7212471261867375]], [[0.71848205354796502, -4.9071711014961252, 2.8746451572696667, 0.64300053278355929, -3.8274398926889379], [0.76834337432611655, -3.8702818730857791, 2.1784851081298102, 3.6592871123576938, -2.1811408651042141], [3.6218815674043991, -3.8104452952222516, 0.62924202067834045, -0.72317386810783546, -2.0210326835228645], [1.9088949629981613, 1.0140256868643913, -4.7444886458144069, 0.26834355225838724, -5.2008714097547175]], [[1.6462027227692326, 0.47282608300147988, -1.1814148173834864, -1.8175246926497559, -4.8678538246770175], [-3.6991378825444423, 1.1528595086577438, 3.8582707024912706, -5.5879537287982739, 0.80170956393340642], [-5.7206257455838898, 2.7951691194786816, -1.7049004048466969, 3.0906817311801253, -2.1511561673457891], [3.1993518869065616, 0.67685179828228659, 0.52697559751034628, -4.6222017444137329, -3.3847284637581678]], [[-5.8872051364236579, -6.543226173859642, -5.1217874562069605, -1.7126287489453524, -5.4207144525035114], [-4.5198551031346303, -3.8077976245267635, -9.1945326219911081, -0.33597501660655205, -6.4967144278752071], [-0.47097043525862325, -8.1722637913311367, -0.92747882360471046, -6.5744925510726695, -5.635710893069259], [-3.3839527556115776, -1.9553684523051693, -8.8355151565813621, -2.5067544030486086, -1.4092518661839222]], [[-1.8397473425736592, 3.2952901307494153, -2.2813538143821166, 0.0014136711236303334, 0.84853136137420293], [3.5034172504796928, 5.3068831202543203, -1.1054902472813861, -1.9849444560766116, 2.6982853066450936], [2.4511837088852166, 0.30499047500007226, 0.9635706660982426, 0.74036095495386078, 4.974895431110407], [3.6431700068590205, 4.5316882639101932, 6.2497913353852397, 5.1676839236300012, 1.3808682971580541]]], [[[1.6398316562726469, 1.8623009577051279, 1.567968555173568, 3.4377018402249311, 1.0307738052428101], [-3.2006138901894259, 2.1737460585361559, 5.9467334020533356, 0.76528750765645714, -2.4846197271673631], [4.5468294076499758, 0.9894135079360824, -0.89828053028115917, -2.3865994810386724, 0.86120620181795893], [1.9825487844512777, 5.6777245073059399, 0.48821979893945766, 3.2312019902818072, -0.82110877482150713]], [[8.3089849641823417, 6.6106803316856908, -0.27330061722209509, -0.62486525347270483, 7.3837938525121949], [3.8522125157619733, 3.1225302651438991, 2.0392407377885879, -0.32546189552781746, -0.93538715416647111], [8.207014861748851, -1.5021681620997764, 7.1001890547485065, 6.5655154393990252, -1.0713643259774646], [7.4330870343028952, 0.17168153434650302, 8.0651139901784674, 5.9272139550486083, -1.5559844296678484]], [[0.57757708666923246, -2.2525235917588589, 3.5569461626793935, 0.78675207589538765, 2.3073466245476073], [2.2784887228586665, 1.5881095330547854, -1.3412943604607843, 1.8122967560539482, -4.2378801197767677], [-2.2405605364192089, -4.1705869308389163, 0.48300040717156456, 0.18752612164936089, -4.5958441473426914], [2.859928339599799, 0.016403711881562089, 1.821564595000341, -5.4347811483502468, -2.3043515074283034]], [[6.0171534402221685, 3.8620284668097593, 2.6003292694736224, 5.9860637041984841, 1.0659408461699105], [0.70083305727970036, 0.86434220005451223, 1.5695380744831198, 3.9272724103885421, 4.38985202305716], [5.860402723072422, 3.5533231640819505, -2.5643417270984052, 2.0541807856695797, 3.0334104739359304], [0.52771267701565439, 5.0446354106662543, 2.9411710437348413, -3.0694810095419074, -1.4125432921515104]], [[-2.1411097846489326, -7.0996535181890401, -8.054083974367817, -7.8871732121515903, -0.62853026849152549], [-6.3049155876949543, -2.5591189455721617, -0.29875011230610582, -4.2123339413228331, -2.018673652613554], [-6.6180974238363532, -8.795958358745672, -7.4697028935368053, -8.1523292279465966, -2.2913348564409359], [-6.3204562710531418, -5.984702333745016, -0.54161090233704812, -4.5803980354004459, -6.7261383439145561]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_array_rank3(self):
      arg0=numpy.array([[[-4.8162648037926603, 1.6603316889571289], [-4.264768521630482, -2.6575953987648058]], [[3.907455930273855, -4.1075890801164858], [2.5379884577624559, 1.1198040078348708]], [[1.3239499094085794, -1.4283957879090634], [-1.244028896377769, 4.567019310390366]], [[2.2831789885970206, 3.408163062651159], [0.1736715776497908, -2.4818883823433047]], [[-4.4680336274062835, 3.375950563827125], [0.21327360899315906, -2.3434323125449206]], [[-2.1434619393174223, -4.0795642964997452], [-4.4205716818355656, -1.2065001510986004]]])
      arg1=numpy.array([[[3.233897134372409, 2.5553644352512244], [1.4630308795654585, -4.3663648911630286]], [[4.1040252813031657, -0.6193714354551938], [3.1367378951521268, 1.3964441417176925]], [[-4.8790080756826164, 1.9059365708205185], [-2.8334112277542491, 3.5624332865880177]], [[3.7374980712691848, -3.5071195381512297], [1.0723936164845522, -3.2106391895822228]], [[1.7611000663974998, 0.092358589748772424], [-2.4561993990644684, 2.3762893867204795]], [[-3.8682692222162718, 4.5709428503156264], [-0.33675148546058598, -0.57318919954880521]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[-1.5823676694202513, 4.2156961242083533], [-2.8017376420650235, -7.0239602899278344]], [[8.0114812115770206, -4.7269605155716796], [5.6747263529145826, 2.5162481495525633]], [[-3.555058166274037, 0.47754078291145508], [-4.0774401241320177, 8.1294525969783837]], [[6.0206770598662054, -0.098956475500070695], [1.246065194134343, -5.6925275719255275]], [[-2.7069335610087837, 3.4683091535758974], [-2.2429257900713093, 0.032857074175558942]], [[-6.0117311615336941, 0.49137855381588125], [-4.7573231672961516, -1.7796893506474056]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[4.4393146128816579, 1.4490240710644322], [-2.9419314540361983, 2.8478200185360141]], [[1.6153209637498565, -0.093651273199339968], [1.7281244062891243, 3.0615610567180447]]], [[[-3.2168405811976006, -2.0136383653893217], [-2.8289297409311533, -3.73011482707465]], [[-3.0633325356330756, -0.43761018679317498], [0.41252562386887437, -1.4658535730578137]]], [[[1.0907894559026623, 4.6138252971488001], [2.6130122184292439, -3.9406604201830433]], [[1.1106951384954407, -1.6034886449483166], [-3.5260207611482386, -2.2377275039777733]]], [[[0.9763952838443597, -1.9564804756188412], [-2.6800446471867225, -3.3729943506301852]], [[4.0513826011596574, 4.3941768671545969], [4.4447292791553998, -3.1423952604000602]]], [[[-4.5339892081721711, -2.3567642325976523], [-3.7303530131330276, -2.398449515910678]], [[4.8000803528527403, -0.72571192696479692], [4.0428474579852356, -2.1341947649102266]]], [[[1.9690351276429521, 2.8324989969193179], [-1.7356396901989504, 1.9747523335471895]], [[-3.6277020865600171, -1.8382189746029907], [0.81584805155681206, 4.680815531110337]]]])
      arg1=numpy.array([[[-4.7673791937287477, 3.7154632393990727], [-4.6877095274671454, -3.8982868392370618]], [[-3.9741286813198231, 0.13209936717769466], [3.045841866071104, -4.2606123931471886]], [[-4.1396601056109077, -4.6830183558067118], [1.712169742293888, 3.9538116043777674]], [[-4.9590921306442626, 2.9986434130450466], [1.3330381425427635, -2.116340132766358]], [[-3.20634382873103, 0.64758548422823736], [-1.6212311634161116, -3.9893744788499417]], [[4.3153253897604706, -1.9740281141053826], [-3.0751216990509955, -1.3407608487685163]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[-0.32806458084708989, -3.3183551226643155], [0.77353178536287448, 6.5632832579350868]], [[-3.072388563717289, -4.7813608006664854], [-2.1701624329479374, -0.83672578251901708]]], [[[-7.1909692625174237, -5.9877670467091448], [-2.6968303737534587, -3.5980154598969554]], [[-0.01749066956197165, 2.608231679277929], [-3.8480867692783143, -5.7264659662050024]]], [[[-3.0488706497082454, 0.47416519153789238], [-2.0700061373774679, -8.623678775989756]], [[2.8228648807893286, 0.10868109734557141], [0.42779084322952876, 1.716084100399994]]], [[[-3.9826968467999029, -6.9155726062631038], [0.31859876585832403, -0.3743509375851386]], [[5.384420743702421, 5.7272150096973604], [2.3283891463890418, -5.2587353931664182]]], [[[-7.7403330369032011, -5.5631080613286823], [-3.0827675289047902, -1.7508640316824406]], [[3.1788491894366286, -2.3469430903809085], [0.053472979135293919, -6.1235692437601683]]], [[[6.2843605174034227, 7.1478243866797886], [-3.709667804304333, 0.00072421944180689479]], [[-6.7028237856110131, -4.9133406736539857], [-0.52491279721170425, 3.3400546823418207]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_array_rank4(self):
      arg0=numpy.array([[[-3.6168293521064578, -3.017783259045669], [-3.2181771404881765, -2.2859842516189444]], [[-4.6607180657697134, 4.753847627023017], [-1.5262762233340288, -0.19975459683040686]], [[1.4628503134209172, -0.33520203556978423], [-1.2914390091941783, -4.9521945997646064]], [[2.823350498159158, 4.6925770857511786], [-0.84721445530974471, 1.0110870361960806]], [[1.0564576548406022, -3.0537000214256196], [-2.9434117548841932, 4.3385116307666785]], [[-4.7831336968150762, 0.36413231328448603], [4.9112363385924436, 1.984628884819327]]])
      arg1=numpy.array([[[[4.941404152579068, 2.2705511170843913], [-3.915212526111973, 2.6902222158982099]], [[3.0065880078640745, -3.8668984930451558], [-0.7035933136620045, 1.7990704691713075]]], [[[-1.3097045373307759, -4.4623223180322578], [-2.5484159423873143, 4.4589555168020958]], [[0.25482043717761993, -3.8568928968582905], [-3.3492837217277707, 0.035185030279927254]]], [[[-3.6490131961023966, 2.5691324403830542], [2.1112580829261098, -0.72097650109338218]], [[2.9981238560026648, -2.454852675933068], [-1.9058317772134838, 2.3330701181771518]]], [[[-3.0543444769565653, 3.1758802428435686], [0.43419074321997364, -2.0940014610030202]], [[-4.3316990420058108, -1.9410939183092148], [-2.5734712705239793, -1.6401917642342734]]], [[[-4.1756361990012287, -2.5516816990554139], [-2.9850002665551223, -3.6427922428918547]], [[-4.9930997539057786, 2.410042480707391], [-1.8529396860163616, -3.0651837594578071]]], [[[4.3339251388264604, -3.5872934423541434], [2.4784939885193999, -3.9824701166037202]], [[-4.9943262890045128, 3.4229469807352011], [-4.298316287465016, -4.4076951165236693]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[1.3245748004726101, -1.3462782350220666], [-6.932995785157642, -0.32756104314745915]], [[-0.21158913262410195, -7.0850756335333323], [-2.9895775652809489, -0.48691378244763683]]], [[[-5.9704226031004897, -9.1230403838019711], [2.2054316846357027, 9.2128031438251128]], [[-1.2714557861564089, -5.3831691201923189], [-3.5490383185581775, -0.16456956655047961]]], [[[-2.1861628826814794, 4.0319827538039714], [1.7760560473563256, -1.0561785366631664]], [[1.7066848468084865, -3.7462916851272463], [-6.8580263769780903, -2.6191244815874546]]], [[[-0.23099397879740735, 5.9992307410027266], [5.1267678289711522, 2.5985756247481584]], [[-5.1789134973155555, -2.7883083736189596], [-1.5623842343278986, -0.62910472803819273]]], [[[-3.1191785441606266, -1.4952240442148117], [-6.0387002879807419, -6.6964922643174738]], [[-7.9365115087899714, -0.53336927417680213], [2.4855719447503168, 1.2733278713088714]]], [[[-0.44920855798861581, -8.3704271391692195], [2.842626301803886, -3.6183378033192342]], [[-0.083089950412069236, 8.3341833193276447], [-2.3136874026456891, -2.4230662317043423]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[0.20707199587456415, -2.8389586455840554, -1.4188431515093147, -1.5007258976297844], [4.1780896919113246, 0.41370523497105793, 3.3172576680099244, 4.8988860263969798], [0.95864181795463832, -4.3098910169217195, 4.333851981884397, -1.4135593957145964]], [[3.5832575678913727, 1.5324925432023351, -3.3963943517212689, -2.1466822344802217], [2.1738415883853897, 4.0882065623806607, 2.5476255491489486, -3.5574616591181032], [4.82913663235718, 1.6962851647438431, 4.565957622257006, 4.9166165254259724]]], [[[3.3370816889492954, 0.51922608449754293, -4.9009419782762738, 1.9356440975342863], [1.2941682468457163, 3.4476611127651147, -3.4621338206790861, -3.8519039019066015], [1.8794061425144477, -3.8755542830091851, -2.5594438097811842, 4.5100770560125181]], [[-4.3774278646768403, -2.2413395008091475, -1.5356834242226967, -2.9882533136924039], [4.3710954372305988, -0.178814287090451, 1.5886832579298371, -1.5850731242110596], [2.7762847490735121, -4.0179143756718911, 0.74975534739382965, -1.3435636487398117]]], [[[-1.2846985979955292, -1.8050585602992109, 3.6884182630231539, 1.9860303880987287], [3.2667423001524334, 0.68769889003960216, 3.5150011116435884, 4.6536274273340421], [-1.9740333387752509, -3.6457687846294018, -0.21977046054667149, 3.1112277618152078]], [[4.2999661726965339, 4.345492310509794, -2.5281918668153356, 0.98026859980009728], [-3.0135980268779816, -3.8131809797016292, -3.5587365673156368, -0.62640235530673571], [2.0191668179310049, -2.3600713401751641, -3.4404244351342648, -3.8805217468959476]]]])
      arg1=numpy.array([[[[1.1138526891519849, -4.8465740282892265, 2.8504860185910914, -4.7066030422504168], [4.4454574423686797, 1.6399924913779369, -3.9425765443906204, 1.8345134654423658], [0.16921289634268, -4.1149726046189024, -3.8985285194547692, 4.5517556913682302]], [[2.5472743590695881, -3.2809328643991731, -4.4114702616399324, 4.4088680114609033], [-3.1312413067695188, 4.5957843462307668, 4.4176418099324923, -3.1806616944375445], [4.0697428199251835, -3.6752083343903497, -4.0576349149613655, -0.98144487715212847]]], [[[-4.8326104317198402, -3.2916302267986799, -1.6183183180436291, 2.2939346707747017], [-1.0848031940206218, 0.28494407928834509, 2.22106027477352, -4.9543128911282119], [-0.92782697971742234, 4.2621644610647795, -1.140804795291591, -1.4243204037339128]], [[4.9709348752035503, -2.2831685852029873, 3.9361279001785672, 2.7810077916828169], [3.4430315533724638, -4.70707264622654, -1.4483171688294973, 4.0894432299277099], [4.4826982347042001, 2.1550760387491321, -3.2704296865112168, 4.3915305523324228]]], [[[0.51796219816416578, 4.6431871713130377, -4.2581280183958823, -0.78755218271505445], [-1.5109214489606693, -1.515141241280423, 1.9242611987554712, 0.63393242913819314], [2.2128380231246991, -4.442445128369763, -4.7325496868154566, -2.216136734462566]], [[4.313660138580639, 0.16184750040414198, -1.7140079361203453, 4.4470495445894116], [4.0220813138875595, -3.2798509400048745, -4.3208226063445423, -2.8021294165793922], [3.9746010252769679, -3.6968121510622112, -0.092370941208526958, 0.91539399072196215]]]])
      res=add(arg0,arg1)
      ref=numpy.array([[[[1.320924685026549, -7.6855326738732819, 1.4316428670817767, -6.2073289398802007], [8.6235471342800043, 2.0536977263489948, -0.62531887638069605, 6.7333994918393456], [1.1278547142973183, -8.4248636215406218, 0.43532346242962783, 3.1381962956536338]], [[6.1305319269609608, -1.748440321196838, -7.8078646133612013, 2.2621857769806817], [-0.95739971838412918, 8.6839909086114275, 6.9652673590814409, -6.7381233535556477], [8.8988794522823635, -1.9789231696465066, 0.50832270729564044, 3.9351716482738439]]], [[[-1.4955287427705448, -2.772404142301137, -6.5192602963199029, 4.229578768308988], [0.20936505282509454, 3.7326051920534598, -1.2410735459055662, -8.8062167930348139], [0.95157916279702537, 0.38661017805559439, -3.7002486050727752, 3.0857566522786053]], [[0.59350701052670995, -4.5245080860121352, 2.4004444759558705, -0.20724552200958701], [7.8141269906030626, -4.885886933316991, 0.14036608910033976, 2.5043701057166503], [7.2589829837777122, -1.862838336922759, -2.5206743391173871, 3.0479669035926111]]], [[[-0.76673639983136344, 2.8381286110138269, -0.56970975537272839, 1.1984782053836742], [1.7558208511917641, -0.82744235124082088, 5.4392623103990596, 5.2875598564722353], [0.23880468434944824, -8.0882139129991657, -4.9523201473621281, 0.89509102735264179]], [[8.6136263112771729, 4.507339810913936, -4.2421998029356809, 5.4273181443895089], [1.0084832870095779, -7.0930319197065037, -7.8795591736601791, -3.4285317718861279], [5.9937678432079728, -6.0568834912373752, -3.5327953763427917, -2.9651277561739855]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_constData_rank0(self):
      arg0=numpy.array(1.08658937568)
      arg1=Data(3.65141620993,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(4.73800558561,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_constData_rank0(self):
      arg0=numpy.array([3.5280575234979707, 4.1465688862900478])
      arg1=Data(-1.36307882552,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([2.1649786979777743, 2.7834900607698514]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_constData_rank1(self):
      arg0=numpy.array(-3.385118395)
      arg1=Data(numpy.array([1.085706277824805, 3.8406256213182406]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-2.2994121171759776, 0.45550722631745799]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_constData_rank0(self):
      arg0=numpy.array([[-3.0477207454583342, -3.0282959083223329, 0.7749831657258941, 2.2073872448035026, -3.7893888343855853], [0.5056893479560145, -3.9931013948895542, 0.64289331930294313, 1.5293872363775343, 1.7719605880888309], [4.761986051881248, 0.95146295589421293, 4.3254401779764127, 3.098982283003231, 2.3783040188654949], [3.6009352632269849, -1.0081042135408502, 3.9993844068665698, -2.4416357558142856, -3.6111770255188591]])
      arg1=Data(3.96111381725,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[0.91339307178848284, 0.93281790892448413, 4.7360969829727111, 6.1685010620503196, 0.17172498286123172], [4.4668031652028315, -0.031987577642737186, 4.6040071365497601, 5.4905010536243513, 5.7330744053356479], [8.723099869128065, 4.9125767731410299, 8.2865539952232297, 7.060096100250048, 6.3394178361123119], [7.5620490804738019, 2.9530096037059668, 7.9604982241133868, 1.5194780614325314, 0.34993679172795789]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_constData_rank2(self):
      arg0=numpy.array(0.245502573595)
      arg1=Data(numpy.array([[-3.347386236437945, -4.6038209544801942, -3.5666187117124726, 2.8146042254161943, -1.5550935369486787], [0.37813565573573982, 1.5402207449279128, 4.4046765968804902, 0.15966966316012243, -0.44805648217900007], [0.99860196185272176, -1.330939726195389, 2.9997006743083645, 2.7071238598906717, 0.76807732551295693], [-3.2741221969535395, 2.5142299816695957, -3.9962842508079568, -3.6221757152495293, 4.2729910604551211]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-3.1018836628431972, -4.3583183808854464, -3.3211161381177248, 3.0601067990109421, -1.3095909633539309], [0.62363822933048763, 1.7857233185226606, 4.650179170475238, 0.40517223675487024, -0.20255390858425226], [1.2441045354474696, -1.0854371526006412, 3.2452032479031123, 2.9526264334854195, 1.0135798991077047], [-3.0286196233587916, 2.7597325552643435, -3.750781677213209, -3.3766731416547815, 4.5184936340498689]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[-4.8760725432093457, -2.4320485747519962], [0.22343669466661087, 1.3063243978050378]], [[4.8645820226350249, -1.5196963249695172], [3.0779274966733929, 3.0605575871487787]], [[-3.6783835986969549, 0.64699995109955832], [-4.2006918725202453, 0.24282648215983915]], [[-0.91845753535747932, -3.7796246458366145], [2.8429441447387855, 3.2474441929073468]], [[-3.8776268215997445, 4.3104602214874799], [3.6792269317970234, -1.9498562138166395]], [[2.8742858031281617, -4.0269956630186137], [-3.1673012889040342, 2.4813424216184652]]])
      arg1=Data(2.76874531517,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-2.1073272280365272, 0.33669674042082232], [2.9921820098394294, 4.0750697129778564]], [[7.6333273378078434, 1.2490489902033013], [5.8466728118462115, 5.8293029023215972]], [[-0.90963828352413634, 3.4157452662723768], [-1.4319465573474268, 3.0115717973326577]], [[1.8502877798153392, -1.010879330663796], [5.6116894599116041, 6.0161895080801653]], [[-1.108881506426926, 7.0792055366602984], [6.4479722469698419, 0.81888910135617898]], [[5.6430311183009803, -1.2582503478457951], [-0.39855597373121565, 5.2500877367912837]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_constData_rank3(self):
      arg0=numpy.array(1.56702109346)
      arg1=Data(numpy.array([[[-1.3274772152884582, -0.97180571143130035], [2.6608136717593318, -3.6169826289923792]], [[-2.1623932906776409, 4.5805786048914303], [1.8005801739055887, -2.4928651318745434]], [[-0.56815281881184809, 2.0083775489363056], [-1.3007826854702698, -1.6229294664235496]], [[-2.6376285630495699, -3.8819487079682204], [3.8227340472803757, 3.6326641398582336]], [[-0.52821848477423039, -2.4663666254753069], [1.2871299483985768, -2.9439183662419675]], [[-1.1032952125692841, 4.2922431807084109], [3.8169836472318348, -4.4152986571792274]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[0.23954387816840672, 0.59521538202556457], [4.2278347652161967, -2.0499615355355143]], [[-0.59537219722077594, 6.1475996983482952], [3.3676012673624536, -0.92584403841767848]], [[0.99886827464501682, 3.5753986423931705], [0.26623840798659515, -0.055908372966684716]], [[-1.070607469592705, -2.3149276145113555], [5.3897551407372406, 5.1996852333150985]], [[1.0388026086826345, -0.89934553201844203], [2.8541510418554417, -1.3768972727851025]], [[0.46372588088758082, 5.8592642741652758], [5.3840047406886997, -2.8482775637223625]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[-0.38000849757054755, -4.67613031168744, 0.10163285231516639, 3.1003173714705756], [-1.0971053399203026, 3.9911749716252416, 3.3572620298476767, 3.2146832855684124], [2.8960016662144206, -1.0430469281355759, 3.9998202420318325, -1.6559279424621645]], [[-3.5156930576395293, -0.48597850185245317, 3.7257929438079795, -4.1940095797975072], [-2.3096213898909235, -3.6340920079081185, -4.7700735518790953, -2.2339075528794918], [0.20614635973201789, 4.9806994714110715, -2.7530593314426186, 2.9830417882330096]]], [[[-2.7432466375239195, 1.1429716817385858, -0.93269032081848913, 4.311637016133048], [-0.63104519316364183, 4.4154426898402761, -4.2532695261352425, 4.4482583634171959], [1.2769472884211366, -2.0419442658110185, -1.4118869594094718, -2.3518706970118086]], [[-3.0611152612339865, 2.2066550902203979, -2.4752383284222823, -0.41623234583664726], [-2.4327557926133228, -0.39424423280582399, -4.8673728447706734, -3.6689858049217361], [0.75963398714993691, -3.9265463066210291, -0.26832728265191808, 1.3766908891334237]]], [[[3.0084995918718906, -3.2214522080267036, 2.8036711098677252, -0.61895370707546604], [2.4696848170803145, 0.57390353065400568, 1.1348836122681583, -4.8577409188472807], [-2.9920739915278158, 0.78088062815757198, 0.8111487027398212, 4.9996664785566018]], [[-3.3796179893674405, -2.578919846883303, 0.37436921793775912, -0.59870001465307077], [-3.4805701616581541, -4.4117016206229458, -2.6433859931937653, -0.5827338927498289], [-4.4766069154127406, 4.1008142643804852, -2.7856573674927656, -2.5597325874899934]]]])
      arg1=Data(0.275517936362,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-0.10449056120904388, -4.4006123753259363, 0.37715078867667007, 3.3758353078320793], [-0.82158740355879889, 4.2666929079867453, 3.6327799662091804, 3.4902012219299161], [3.1715196025759242, -0.76752899177407219, 4.2753381783933362, -1.3804100061006608]], [[-3.2401751212780256, -0.21046056549094949, 4.0013108801694832, -3.9184916434360035], [-2.0341034535294198, -3.3585740715466148, -4.4945556155175916, -1.9583896165179882], [0.48166429609352157, 5.2562174077725752, -2.4775413950811149, 3.2585597245945133]]], [[[-2.4677287011624158, 1.4184896181000894, -0.65717238445698545, 4.5871549524945516], [-0.35552725680213815, 4.6909606262017798, -3.9777515897737388, 4.7237762997786996], [1.5524652247826403, -1.7664263294495148, -1.1363690230479682, -2.0763527606503049]], [[-2.7855973248724828, 2.4821730265819015, -2.1997203920607786, -0.14071440947514358], [-2.1572378562518191, -0.11872629644432031, -4.5918549084091698, -3.3934678685602324], [1.0351519235114406, -3.6510283702595254, 0.0071906537095856038, 1.6522088254949274]]], [[[3.2840175282333943, -2.9459342716651999, 3.0791890462292288, -0.34343577071396236], [2.7452027534418182, 0.84942146701550936, 1.4104015486296619, -4.582222982485777], [-2.7165560551663122, 1.0563985645190757, 1.0866666391013249, 5.2751844149181055]], [[-3.1041000530059368, -2.3034019105217993, 0.6498871542992628, -0.32318207829156709], [-3.2050522252966505, -4.1361836842614421, -2.3678680568322616, -0.30721595638832522], [-4.2010889790512369, 4.3763322007419889, -2.5101394311312619, -2.2842146511284898]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_constData_rank4(self):
      arg0=numpy.array(2.81041052209)
      arg1=Data(numpy.array([[[[1.2307826611393757, -3.8662040472860051, 0.9806767100046283, -4.6916207764987856], [-4.7602334193577516, -1.21962512123332, 4.4686693436364102, -0.15623600416114858], [3.9390707769362958, 2.7985922915462789, -1.5615882500031919, -0.19683139525321458]], [[-2.1856697279459048, 0.56959511659904116, 1.8308962982620809, -0.021681863063705009], [-3.5482366472171103, 4.4887600660916167, -0.00013818205776416903, 2.4547876173066285], [4.6094427486838612, 0.88607977342562805, -2.4526974004843116, 3.2501910549078676]]], [[[2.9450290468070541, 1.5448794797300671, 2.4057832184870511, 2.7747709951026653], [-4.155118196734283, 3.0314451917192766, -3.4040048679846269, 3.3989035902773264], [2.3610997072140947, -0.22395856367735689, -1.5721089891494113, 2.7236223985428882]], [[1.8235554180878584, 3.3063710178376002, -4.3887024958780447, -2.9391701723554187], [0.91508758449939887, -2.4160465984421586, 1.0246857734137604, 0.91913021214545143], [1.4530979842840965, -2.3001241658008609, -2.9998128795019285, 3.7767938728595034]]], [[[-1.8958919868867694, -4.6403026516731973, 2.5702402524152124, -2.9951428137137603], [-1.9057099085962403, 2.4474186076438675, 1.4069805925703456, -4.361641017486642], [-2.0727218861953234, 4.5236969215095399, -3.0160538253391076, 4.0129130605886836]], [[2.0706158939616479, 2.2322411383838014, -0.93303291879747707, 3.3826200928243182], [-4.460195998821435, 1.6087300203935495, -2.5051110617218511, -3.8733154376218595], [2.3044250765985979, -0.16837706537052544, 2.1503764556314202, 1.6898236289827633]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[4.0411931832330117, -1.0557935251923691, 3.7910872320982643, -1.8812102544051497], [-1.9498228972641156, 1.5907854008603159, 7.2790798657300462, 2.6541745179324874], [6.7494812990299318, 5.6090028136399148, 1.2488222720904441, 2.6135791268404214]], [[0.6247407941477312, 3.3800056386926771, 4.6413068203557168, 2.7887286590299309], [-0.73782612512347434, 7.2991705881852527, 2.8102723400358718, 5.2651981394002645], [7.4198532707774971, 3.696490295519264, 0.35771312160932434, 6.0606015770015036]]], [[[5.7554395689006901, 4.3552900018237031, 5.2161937405806871, 5.5851815171963013], [-1.344707674640647, 5.8418557138129126, -0.59359434589099092, 6.2093141123709623], [5.1715102293077306, 2.5864519584162791, 1.2383015329442246, 5.5340329206365242]], [[4.6339659401814943, 6.1167815399312362, -1.5782919737844088, -0.12875965026178271], [3.7254981065930348, 0.39436392365147732, 3.8350962955073964, 3.7295407342390874], [4.2635085063777325, 0.51028635629277508, -0.18940235740829259, 6.5872043949531394]]], [[[0.91451853520686655, -1.8298921295795614, 5.3806507745088483, -0.18473229162012439], [0.90470061349739561, 5.2578291297375035, 4.2173911146639815, -1.5512304953930061], [0.73768863589831257, 7.3341074436031759, -0.20564330324547164, 6.8233235826823195]], [[4.8810264160552839, 5.0426516604774374, 1.8773776032961589, 6.1930306149179541], [-1.6497854767277991, 4.4191405424871855, 0.3052994603717849, -1.0629049155282235], [5.1148355986922338, 2.6420334567231105, 4.9607869777250562, 4.5002341510763992]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_constData_rank1(self):
      arg0=numpy.array([-3.3596764032837232, -0.62914598804125532])
      arg1=Data(numpy.array([-2.9122735967358659, 3.8550610160036456]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-6.2719500000195891, 3.2259150279623903]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_constData_rank1(self):
      arg0=numpy.array([[2.3825933653622657, 0.42851169841824976], [0.46568198109130687, 0.8255960585588511]])
      arg1=Data(numpy.array([-1.68703679490942, -0.8863704182659804]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[0.69555657045284569, -1.2585250964911703], [-0.42068843717467352, -0.0607743597071293]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_constData_rank2(self):
      arg0=numpy.array([4.217663001280739, -4.679685407904266])
      arg1=Data(numpy.array([[0.87105539176877045, -3.0340137359672537], [-4.2193610162528481, 3.8654124854618672]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[5.0887183930495095, 1.1836492653134854], [-8.8990464241571132, -0.81427292244239879]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_constData_rank1(self):
      arg0=numpy.array([[[1.0985173607158929, 1.1848100515586024, 3.8574035277399155, 2.6015631763778293, 4.6575712416566493], [-1.3065676750152022, 3.5931313113830647, 4.2059792424232647, -3.8604251861616201, -3.5444627467386072], [2.1517207299765175, 0.32652232615408217, 1.9282115933558526, 1.8414080725312747, 0.42458272299923916], [-2.2538157825225591, 2.5620148519804662, -4.5528026485208164, -3.9928711949989748, -1.2089047084365911]], [[0.26540605768766135, -2.020462440074823, 0.81119981604763147, -1.0622685906686566, 4.2422114124165784], [2.5419855750281615, 1.844785874027882, 2.2291436489774963, 0.43143265935409048, 2.3336767782491563], [-3.1301146342023376, -1.1505153770616596, 4.5039205827614719, 3.3384086033856288, 3.3210580605360249], [4.6365960087522655, 2.4948309974026941, 3.3216713220322074, -2.8811715322891551, -1.8763999528224371]]])
      arg1=Data(numpy.array([2.3722840465843404, -1.9236456924961356]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[3.4708014073002333, 3.5570940981429429, 6.2296875743242559, 4.9738472229621697, 7.0298552882409897], [1.0657163715691382, 5.9654153579674052, 6.5782632890076052, -1.4881411395772797, -1.1721787001542667], [4.524004776560858, 2.6988063727384226, 4.300495639940193, 4.2136921191156151, 2.7968667695835796], [0.11846826406178135, 4.9342988985648066, -2.180518601936476, -1.6205871484146344, 1.1633793381477493]], [[-1.6582396348084742, -3.9441081325709586, -1.1124458764485041, -2.9859142831647922, 2.3185657199204428], [0.61833988253202588, -0.078859818468253629, 0.30549795648136069, -1.4922130331420451, 0.41003108575302072], [-5.0537603266984732, -3.0741610695577952, 2.5802748902653363, 1.4147629108894932, 1.3974123680398893], [2.7129503162561299, 0.5711853049065585, 1.3980256295360718, -4.8048172247852907, -3.8000456453185727]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_constData_rank3(self):
      arg0=numpy.array([2.6394859282029328, 0.47554606415998979])
      arg1=Data(numpy.array([[[4.2075107851721114, -2.9834814949260293, -4.6402371996699934, -1.255511599921757, 4.5063303046395653], [1.6710346052243574, -4.7975384914953798, -3.7064148452087906, 1.6651817248498464, 3.4376853459371137], [-1.0762202417788025, -1.2742243656432919, -1.4336250987118371, 0.02217931741118484, -0.8760271313673087], [-2.2645471788305018, -0.75728009215272341, -3.0124704339979678, -0.53486829146474868, -4.6876494965294224]], [[-3.2214567951490336, 4.9181819842283918, -4.5843353128735229, 0.18319164487528994, -1.705542820867735], [1.8765786339897499, -1.7784546188644668, 4.3347929937156806, 0.85911348662032427, -1.3296707604618963], [-4.6327053983891266, 2.5173647325712798, -4.2000007177475336, 0.53617953816541686, 4.8215475549537548], [-2.5104280346557371, -0.94564165092314845, -2.4654051303541302, 2.4531020760021693, -3.4156315132656578]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[6.8469967133750442, -0.3439955667230965, -2.0007512714670606, 1.3839743282811758, 7.1458162328424981], [4.3105205334272902, -2.158052563292447, -1.0669289170058578, 4.3046676530527792, 6.0771712741400465], [1.5632656864241303, 1.3652615625596409, 1.2058608294910957, 2.6616652456141177, 1.7634587968356241], [0.37493874937243099, 1.8822058360502094, -0.37298450579503495, 2.1046176367381841, -2.0481635683264896]], [[-2.7459107309890438, 5.3937280483883816, -4.1087892487135331, 0.65873770903527973, -1.2299967567077452], [2.3521246981497397, -1.302908554704477, 4.8103390578756704, 1.3346595507803141, -0.85412469630190646], [-4.1571593342291369, 2.9929107967312696, -3.7244546535875438, 1.0117256023254066, 5.2970936191137445], [-2.0348819704957473, -0.47009558676315866, -1.9898590661941404, 2.9286481401621591, -2.940085449105668]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_constData_rank1(self):
      arg0=numpy.array([[[[3.8746981625231207, 0.75646777202774995], [4.0662061712786084, 0.26207534606165073]], [[-0.1878601083202156, -3.1483218336469978], [-1.476711869524665, 0.92320549301514987]], [[3.1056139177710573, 2.4260447869505777], [-3.5085855583468648, -3.8199382365446857]], [[-3.5240447895921068, -0.139448472634081], [-2.9176468309512407, 2.29093649278113]], [[4.8348672633738641, -3.5122023424433468], [-4.3839098560644878, 1.8938174534125265]], [[3.7041818445548493, 0.91749124663789772], [-4.3918248153087989, 4.4426238334090975]]], [[[3.2168546900385682, -1.9943049304741258], [3.8312428098927658, -3.7762084497727786]], [[-4.2988774179298463, -1.9629077411274212], [-4.0357694842688172, 3.2924904313105916]], [[0.99801677396921384, -0.25246442453144802], [-1.0536167076699368, 3.4918449419957653]], [[-4.269253644382724, -2.4498725301172684], [2.8070428590966028, -1.5059046663714817]], [[3.4088961240565325, 0.5743312098707154], [-1.6645109257888935, -4.8724978187610759]], [[-2.6275022638870582, -0.35493417688280626], [-2.7570783828030967, -0.8151256052999063]]]])
      arg1=Data(numpy.array([-4.77309241085187, 2.9835412036921927]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-0.89839424832874926, -4.01662463882412], [-0.70688623957326158, -4.5110170647902192]], [[-4.9609525191720856, -7.9214142444988678], [-6.2498042803765355, -3.8498869178367201]], [[-1.6674784930808126, -2.3470476239012923], [-8.2816779691987357, -8.5930306473965565]], [[-8.2971372004439772, -4.912540883485951], [-7.6907392418031106, -2.48215591807074]], [[0.061774852521994106, -8.2852947532952168], [-9.1570022669163578, -2.8792749574393435]], [[-1.0689105662970206, -3.8556011642139723], [-9.164917226160668, -0.33046857744277247]]], [[[6.2003958937307608, 0.98923627321806684], [6.8147840135849584, -0.79266724608058592]], [[-1.3153362142376537, 1.0206334625647715], [-1.0522282805766245, 6.2760316350027843]], [[3.9815579776614065, 2.7310767791607446], [1.9299244960222559, 6.475386145687958]], [[-1.2857124406905314, 0.53366867357492431], [5.7905840627887954, 1.477636537320711]], [[6.3924373277487252, 3.5578724135629081], [1.3190302779032992, -1.8889566150688832]], [[0.35603893980513446, 2.6286070268093864], [0.22646282088909597, 2.1684155983922864]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_constData_rank4(self):
      arg0=numpy.array([-0.78639111262371486, 0.45827346624463328])
      arg1=Data(numpy.array([[[[-3.3841271468804024, -3.2157607334156815], [-4.5789125838464226, -0.90149790517015305]], [[4.8006487302317726, 1.9969594729031996], [-1.6495317200198265, 4.4126319717460412]], [[-2.5167325893286985, -1.287444451753764], [-2.2193279315464087, -2.2992866691354683]], [[3.0786396776395915, 3.226887924127853], [4.5702159514962108, -1.6065232933310467]], [[0.071464522171826417, -4.3026494112430091], [-4.4574139614119481, 4.03934456762933]], [[0.65429807936292406, -1.2389616519894719], [2.6117560941413878, -4.7180648532632574]]], [[[-1.4471599025080595, 2.3539535421851552], [-2.7940362807699892, -0.6842593700589763]], [[3.7525752704614384, -2.0284801521715421], [3.8478708188398958, 2.737042875480606]], [[-1.1297120458560936, 3.1588449759757378], [3.9918271698257808, 1.4220666981692078]], [[0.89184938889958243, 1.5250464920196301], [-4.5404286891441785, 4.4163109430450884]], [[-1.5793560999017995, -3.763424126675484], [-3.7282708510542006, 0.5621313902070959]], [[0.79880298809049499, -1.4708348760539991], [0.11822405589665319, -1.8192435849481536]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-4.1705182595041173, -4.0021518460393963], [-5.3653036964701375, -1.6878890177938679]], [[4.0142576176080578, 1.2105683602794848], [-2.4359228326435414, 3.6262408591223263]], [[-3.3031237019524133, -2.0738355643774788], [-3.0057190441701236, -3.0856777817591832]], [[2.2922485650158766, 2.4404968115041381], [3.783824838872496, -2.3929144059547616]], [[-0.71492659045188844, -5.089040523866724], [-5.243805074035663, 3.2529534550056152]], [[-0.1320930332607908, -2.0253527646131868], [1.8253649815176729, -5.5044559658869723]]], [[[-0.98888643626342621, 2.8122270084297885], [-2.3357628145253559, -0.22598590381434303]], [[4.2108487367060716, -1.5702066859269088], [4.3061442850845291, 3.1953163417252393]], [[-0.67143857961146036, 3.617118442220371], [4.4501006360704141, 1.8803401644138411]], [[1.3501228551442157, 1.9833199582642633], [-4.0821552228995452, 4.8745844092897217]], [[-1.1210826336571662, -3.3051506604308507], [-3.2699973848095674, 1.0204048564517292]], [[1.2570764543351283, -1.0125614098093658], [0.57649752214128647, -1.3609701187035204]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_constData_rank2(self):
      arg0=numpy.array([[1.196768330253847, 2.1196638481093029, -0.16679759963264296, 2.0349466328234218, -4.2201172892854384], [1.2806194894163241, 2.423141384483781, -4.8081569493812371, 0.61998574300412024, 1.676372696710744], [4.8056270310225191, 4.8528505011002832, 2.5892916709377509, -0.91039933527188133, -0.25794384746876364], [-2.3598796755905616, -1.5678155220869883, 0.95593370945686118, 1.8634874312160621, 3.6142211509259674]])
      arg1=Data(numpy.array([[-4.8771767123778789, -0.5215272527225876, 0.44924765678924583, 0.24469718396752516, 3.0813140121110845], [4.0276378033764875, -4.2805351699076581, -3.6672202887485539, -0.98249536637007573, 3.5030698735972869], [0.4641301859561251, -3.126208312054132, 1.9682677812349505, 4.8983739207502861, 2.2232212626072361], [-1.234808583110941, 4.9665149558365105, -3.86001634525591, -2.9676094773625703, -0.76338324287111803]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-3.6804083821240319, 1.5981365953867153, 0.28245005715660287, 2.279643816790947, -1.1388032771743539], [5.3082572927928116, -1.8573937854238771, -8.4753772381297914, -0.36250962336595549, 5.1794425703080309], [5.2697572169786442, 1.7266421890461512, 4.5575594521727014, 3.9879745854784048, 1.9652774151384724], [-3.5946882587015025, 3.3986994337495222, -2.9040826357990488, -1.1041220461465082, 2.8508379080548494]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_constData_rank2(self):
      arg0=numpy.array([[[1.1113951087200231, 0.58741013047277413], [2.0985280679075755, 0.6473524153756447], [-1.5805153247492387, -1.370033426579699], [-0.76371388107602023, -3.7551551181401353], [-4.577677730174857, -1.7779134342780658]], [[-1.2255241134155193, -1.6669431653042324], [1.6211822248814833, 2.3511150852554721], [-3.0334180026023407, 2.7555278060210586], [-3.2480837234403848, 0.53183256370784715], [-2.2890283945813064, 3.5314088749601886]], [[-4.5591637879631355, -1.7125952056949822], [-3.4250784932595724, -2.7635841494226252], [-1.8169094521580096, 4.1654885273555333], [-1.0039900382270659, -0.36156590057739457], [-4.872840667930781, 0.3085233501066833]], [[1.1030912910177957, -4.1466310803806019], [2.7668280249143464, 3.7981212812684557], [3.206268761606534, -0.9382427084624867], [0.28198438175467988, -1.1109621374962186], [-3.1640525535623198, -3.3995686932288374]]])
      arg1=Data(numpy.array([[0.60517304393055227, -1.4576251345052604, 1.0965171266828406, 3.2334613068182385, 3.5536152718194494], [-2.2722281638044048, -3.8084379815412825, -1.3736176799009128, -1.3499550995889829, 2.9778230788276128], [2.6463469745346639, -1.4306691462885546, 1.2030662444953428, 4.9675682163827091, 0.65157348157302764], [3.2495021990782096, 0.19462300361412144, -3.0645759622142577, -0.0087581760099908479, -3.3862179786862621]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[1.7165681526505754, 1.1925831744033264], [0.64090293340231508, -0.81027271912961574], [-0.4839981980663981, -0.27351629989685833], [2.4697474257422183, -0.52169381132189674], [-1.0240624583554077, 1.7757018375413836]], [[-3.4977522772199241, -3.9391713291086372], [-2.1872557566597992, -1.4573228962858105], [-4.4070356825032535, 1.3819101261201459], [-4.5980388230293681, -0.8181225358811357], [0.68879468424630641, 6.5092319537878014]], [[-1.9128168134284715, 0.9337517688396817], [-4.8557476395481274, -4.1942532957111798], [-0.61384320766266676, 5.3685547718508762], [3.9635781781556432, 4.6060023158053145], [-4.2212671863577533, 0.96009683167971094]], [[4.3525934900960053, -0.89712888130239232], [2.9614510285284679, 3.9927442848825772], [0.1416927993922763, -4.0028186706767439], [0.27322620574468903, -1.1197203135062095], [-6.5502705322485824, -6.7857866719150994]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_constData_rank3(self):
      arg0=numpy.array([[0.26111204865097726, -1.6132810265211619, -2.2151416734869853, 4.6239524598554791, 2.8691293967522649], [4.1290941292366146, 4.1379096784474783, -3.2790089466506869, -3.9401148696450203, 4.6998313312897899], [1.7659603524079799, -0.49550026474787767, 3.9490012745983805, 4.9024056305450863, 1.0232538148652992], [-4.1949758241675408, 4.6128363900308074, 1.2837258801799321, -2.4634716501958733, 0.91292619359647187]])
      arg1=Data(numpy.array([[[3.0478706052300382, 4.7899215729507461], [1.2320230683961206, -2.1804953428570184], [3.1540398131141085, 3.1762205082576287], [-3.5247256598317978, -0.51918066107483529], [-4.7245085131135811, 4.9792625394975314]], [[3.3166815122276709, 1.4325962376001833], [-4.473911508470402, 4.9815239350075178], [0.39518188533608534, -1.1465522023319896], [1.2153020768603673, 3.426987635672468], [-1.7700049158227529, 2.127420133396912]], [[-0.69307261521535146, 2.7484488026982117], [-2.2074104716686547, 3.0261887605036542], [-1.6671604568076557, -4.0571063341730511], [-2.3245102806291773, 4.4682556715241653], [1.0283923571460765, 3.0617440900873323]], [[1.6044347794697185, -4.8025179501481556], [-1.0749297020373625, -4.0975618907550393], [-0.21888561766658299, 3.7777621117745994], [-1.3860618514526712, 2.306220592808824], [3.4284876348385147, -2.5107188456199703]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[3.3089826538810154, 5.0510336216017233], [-0.38125795812504126, -3.7937763693781803], [0.93889813962712321, 0.96107883477064338], [1.0992268000236813, 4.1047717987806438], [-1.8553791163613162, 7.8483919362497963]], [[7.4457756414642855, 5.5616903668367978], [-0.33600183002292372, 9.119433613454996], [-2.8838270613146015, -4.4255611489826769], [-2.724812792784653, -0.51312723397255233], [2.929826415467037, 6.827251464686702]], [[1.0728877371926284, 4.5144091551061916], [-2.7029107364165323, 2.5306884957557765], [2.2818408177907248, -0.10810505957467065], [2.577895349915909, 9.3706613020692515], [2.0516461720113757, 4.0849979049526315]], [[-2.5905410446978223, -8.9974937743156964], [3.5379066879934449, 0.51527449927576807], [1.0648402625133491, 5.0614879919545315], [-3.8495335016485446, -0.15725105738704936], [4.3414138284349866, -1.5977926520234984]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[-1.8074233450009594, 1.2928682357815919, -0.12012672835495142, 2.1091874200256608, -0.24965322058969086], [2.478787159366461, 0.54530302442139078, -4.516656324099352, -0.11999636601281516, -0.090602153688142728], [3.4075847162888202, 1.3401548583292389, -4.2586816602308186, -2.0375810418765639, -1.1161334004208889], [-3.8270194781117928, -3.846509880270379, 2.5001521356915726, 3.1750307587180622, -4.0352882927249523]], [[1.8582887215949988, -1.8419473718514148, -0.64457468508066995, 1.4100062122710213, -0.63479402059105894], [1.6229367537436339, -3.5141982128478966, 3.7549645902407178, 3.1079800083249012, 1.4651022149814938], [-4.2185229130647608, -3.5153666368746626, -3.5994622642818275, 3.1954405133656056, 1.4607527381141052], [-0.5981889162720897, 0.78948645015309626, 2.2145189197320772, -2.7311549130873503, -4.6610276103788015]], [[-3.0984463311444665, 1.3739393647161577, -1.7419054923868229, 4.6733105853150043, 1.1021035817495983], [-4.0974549432309848, -4.9112653555946491, 4.6971255664265978, 3.6462480215042028, 3.0627884112110522], [1.2305624350958775, -0.075772538950818813, -1.1449218414373852, -3.0317996001038572, -4.2094299741588417], [-0.63828284500458654, 4.511283453926394, -4.7813396664574199, 3.9945406076566154, 0.46532261610150272]], [[2.8437662627505889, -0.55449836281884224, -3.7384432485750994, 0.32875431077062878, 1.5059889438939154], [3.8615025576174862, 1.1140162933508897, -0.60207353305099165, 4.5190019458569743, -4.1991434930051277], [-3.5431193551704965, 3.6515932315211987, 1.6145414834544161, -3.207303738831655, 0.3589818485136842], [-4.7623428746410044, 3.9106462393011814, -1.5989902246869248, 2.9781281976188287, -1.4545606992909796]], [[3.8134857685812307, -4.5395447267926006, 3.3135893875345488, -4.7868605398041373, 2.0649067628312121], [-4.7108110709380675, -1.6606827814310599, 1.1162854952402412, -3.4703543332328559, -4.1447458127151204], [2.0154341767000101, -0.98178846486905691, 0.58473868966129317, 0.68090903790693424, 2.8297306717224346], [-0.29240743846228057, -2.4396912456419981, -1.5357684459221255, -1.7501737609067201, -2.2578798225054797]]], [[[-0.99849716017310097, -4.847176549022012, -4.5311352536302021, 0.81310869476648406, -4.1556871735111001], [-1.0061813330663947, 4.9440322989112513, -3.5851686303049144, -3.5128016214453197, 0.64609055469028931], [-0.43024574423282758, 3.4620391239559112, 4.5128365048152137, 3.8960424173038675, -3.4254668141242304], [4.2472718436736852, 4.7075281473522637, -0.037446269952607025, 0.22350572424830339, 0.56710559732997723]], [[-0.73884277917770103, 4.6500182271744386, -2.8229190846412422, -3.8546564603771003, 0.69081779372094143], [-4.0748111275556429, 4.915028629587546, -2.9440197921320821, -4.2184248184901332, -2.4025298795855008], [1.5964403041936723, 2.3643540860331091, 2.1711616255126138, -3.4134438543338899, 0.31729446187809884], [2.8256280439213013, 1.9939170064769414, 3.5768777764338804, 4.9007761256555842, 0.15756489949956531]], [[-3.9692617823574206, 1.6398542012844031, 0.52470055412530048, 3.9757064554111672, -1.8472108485738858], [2.9328810608616802, -2.1126927703827159, 4.279975180933862, 3.1754560776255261, 1.9964783684408447], [-3.3694311886528725, 4.6748300627758343, -1.5920988186530707, 2.5750045894363058, -2.285161475142826], [-2.8656057323729565, 3.0780370669006647, -2.5329427419136676, -3.2498261281916307, -1.6571781396134533]], [[-2.2542639718175419, -2.8444184878823977, 2.7361017769149409, -2.6518622844218265, 4.0974607805978316], [-4.7763189872186915, 0.90150553701863601, 3.1418846837086978, -2.2313551058235448, 0.68652981597819362], [1.3213067268766707, 0.11255200641852259, -0.035044153376754039, 3.0074583998438751, 0.66726445551452152], [3.7014083362436505, -1.9166847197267201, -2.9498158321332446, -3.8890697275912558, -3.8273568651279399]], [[0.44222290206361681, -0.30561185416458603, 3.64916765749307, 4.7507001342581621, -3.6462358013365215], [3.4464759102367779, 1.2807285293927793, -1.028623928023038, -3.4397954961129704, -3.0196017521613072], [4.8118721127472703, 2.9953066059880173, -1.1860544200577374, 1.9339053310629861, -1.1049234898259641], [3.8739544867214306, -0.48035119562717732, -0.49969464733114144, -3.7734577641925684, -3.9348161589136565]]], [[[3.9493562760625682, -1.6594206534261402, -0.49004103311767189, 0.56059506333196918, -1.946403948698614], [4.3772559802820705, 3.0455683865210936, -2.4643051083074106, -2.2550688045495604, 1.3254857681724026], [-4.3071141486579467, 3.3597173404707021, 2.3295475060861479, -1.0501353640846469, -3.0805857218678123], [-3.0709540817997913, -1.3297746231425354, 4.3468454414004167, 4.669073957416952, 4.9714886874155777]], [[2.7978024006066482, 3.616884038008644, -4.6025788399591514, 0.18113912435978019, 2.2448508636974163], [2.9577536287444453, 3.499501493361608, -2.2065748060582102, -1.296577709173008, 2.8064150350414501], [-2.0187665505145613, 2.1245478726641318, -1.5203050918805272, -0.63074623791553019, 2.6233558719312935], [-0.0051328977570586076, -3.173517139876588, -0.22620206653303132, -0.71647421228178487, 1.3550536195876841]], [[-2.0413834860268976, 4.468832188112815, 1.6741366743614297, -0.81975753811319851, 0.3197393465940781], [-0.066200390238234874, 4.380455654416517, 1.7712630288578746, -2.8844833808338599, 4.2931213894842717], [-2.7069141568590274, -3.2496066290090662, 2.1196499562497273, 3.4124240181530183, 2.0069910480136999], [-3.8143615233164363, 0.83298073951466822, -3.5938222555580457, -3.1887404391152829, -0.99937404350202996]], [[-1.2822805130015515, 0.65202422642198687, -4.0567089508550289, -1.4698227234275096, -4.5104965787276399], [2.9019097035424171, 0.40938501806635941, 4.9857794612588133, -1.3534028014957888, -3.8575192474758602], [-0.89999613828160019, 1.9744944033312599, 0.59880900068479992, -2.3392655902460344, -2.8996416960804803], [2.6386162350974249, 3.8770337248564815, -4.3844491345521668, -2.9424847401763667, -0.32400081176207607]], [[-4.763467798137456, 0.91842704481917092, 0.09581840976303635, -1.3287470905026888, 3.3437659685937344], [-2.1454890125220452, -2.8451213331556069, 4.1722816610514784, -3.6918984994030346, -0.99338247246933165], [-3.3870671147227052, 3.357012604546366, 1.2903705353939108, -0.17871078101138949, -0.29639920575587908], [-2.8790174924037504, 1.5659533817255298, 1.4946598306941858, -3.535212499642677, -0.99453464692178706]]], [[[-3.5269589579694696, 1.6502776357750832, 3.8492891819150863, -4.2127091649145267, -0.17017138869706194], [4.6897205712591745, -0.29448440560867084, 2.1644649856039777, 0.86531876943053732, -1.7507398760284465], [4.4945553783342653, -2.0580214112214734, 0.23554193103632404, 1.7629412749959759, -1.1980315467945313], [0.33897748261898109, -4.0352572663539128, 1.8750238134151056, 0.35650239998483624, -3.6675218262730072]], [[-3.8540864719531873, -3.0698251135474539, -0.95218607814308243, -3.639730404734677, -1.097390667710354], [-3.78220361080405, 2.0058979863871791, 4.5863183006450221, -3.1219457577301224, 4.2842191973948136], [-3.2639299144850131, -4.7454074905715711, -3.4917600933830006, 0.74005501640920812, -3.4981602719621954], [-1.2033985066331643, -0.6256850758073762, 0.33479300951329805, 1.836434578014055, 1.404689628872859]], [[0.63618500144595114, 2.2453069662621257, -2.494835891043147, 3.509627385351985, 3.9958576704091371], [0.11216730007916453, 0.634749208497702, -0.45352556274114697, -2.8173922614701419, -2.049704094130834], [-1.5516238061884478, -0.81288784734968811, -0.74921990550949502, -2.1340524653201975, -0.44770199826281409], [1.0921824433485794, 4.3989685870620079, 2.0259067747309487, -3.729426865602421, 2.8606913398994607]], [[2.5983396174258164, 1.7388289661832097, -1.5120004956458564, -3.2399494077016486, -4.6797613736398427], [-3.502346438743186, 0.850660864096076, 3.1707036081455442, 0.34769483568041526, -0.11627073795179488], [-4.3958655289529958, -0.2959180773779071, -2.7958898045224823, 2.7657686756607456, 3.2830818962171637], [3.855021522890226, 4.4397054476313933, -3.8023175950439567, -4.0981812503705655, -3.9915209036174359]], [[1.9657194035645276, -0.66181099935762955, -2.102778266359322, 4.2795224887402998, -4.3707756492895689], [-1.3612728740308846, -1.1011001670479414, 2.145947976998082, 4.6829559617278722, -1.1573452829843989], [-0.57562017703247204, -4.1504147038509505, -4.4473637116847753, -3.5503781572973683, 0.53624537540475181], [-0.010036413268483813, 2.9861445420259534, -3.2667983645232059, 2.2845221994019846, 0.49212179308968729]]]])
      arg1=Data(numpy.array([[2.4011705292041352, 2.3495098899905749, -2.2169747135392148, -0.34748964916941372, 4.4084641839832202], [1.8042311607235888, 1.2211690101397954, 1.82215521394542, -0.20526170094908558, -3.5240113302016085], [1.3262238418264065, 4.4991597680398403, 2.6668813860279927, 0.028265638518827352, 1.9986975536127627], [4.9341112506659286, 3.3411096628897337, -1.9711207925249008, 4.4203100836961653, -2.4774589956953719]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[0.59374718420317585, 3.6940387649857271, 2.2810438008491838, 4.510357949229796, 2.1515173086144443], [4.8799576885705962, 2.946473553625526, -2.1154857948952168, 2.28117416319132, 2.3105683755159925], [5.8087552454929554, 3.7413253875333741, -1.8575111310266834, 0.36358948732757135, 1.2850371287832463], [-1.4258489489076576, -1.4453393510662438, 4.9013226648957078, 5.5762012879221974, -1.6341177635208171]], [[4.2077986115855737, 0.50756251813916009, 1.7049352049099049, 3.7595161022615962, 1.7147158693995159], [3.9724466437342087, -1.1646883228573217, 6.1044744802312927, 5.457489898315476, 3.8146121049720687], [-1.8690130230741859, -1.1658567468840877, -1.2499523742912526, 5.5449504033561805, 3.8102626281046801], [1.7513209737184852, 3.1389963401436711, 4.564028809722652, -0.38164502309677539, -2.3115177203882267]], [[-5.3154210446836814, -0.84303534882305708, -3.9588802059260377, 2.4563358717757895, -1.1148711317896165], [-6.3144296567701996, -7.1282400691338639, 2.480150852887383, 1.429273307964988, 0.84581369767183734], [-0.98641227844333734, -2.2927472524900336, -3.3618965549766, -5.2487743136430716, -6.4264046876980565], [-2.8552575585438014, 2.2943087403871791, -6.9983143799966347, 1.7775658941174006, -1.7516520974377121]], [[2.4962766135811751, -0.90198801198825596, -4.0859328977445131, -0.01873533839878494, 1.1584992947245016], [3.5140129084480725, 0.76652664418147598, -0.94956318222040537, 4.1715122966875606, -4.5466331421745414], [-3.8906090043399102, 3.304103582351785, 1.2670518342850023, -3.5547933880010687, 0.011492199344270482], [-5.1098325238104181, 3.5631565901317677, -1.9464798738563385, 2.630638548449415, -1.8020503484603934]], [[8.2219499525644508, -0.13108054280938042, 7.7220535715177689, -0.37839635582091713, 6.4733709468144323], [-0.30234688695484735, 2.7477814025521603, 5.5247496792234614, 0.93810985075036424, 0.26371837126809972], [6.4238983606832303, 3.4266757191141632, 4.9932028736445133, 5.0893732218901544, 7.2381948557056548], [4.1160567455209396, 1.968772938341222, 2.8726957380610947, 2.6582904230765001, 2.1505843614777405]]], [[[0.80573400055048783, -3.0429453882984232, -2.7269040929066133, 2.6173398554900729, -2.3514560127875113], [0.79804982765719412, 6.7482634596348401, -1.7809374695813256, -1.7085704607217309, 2.4503217154138781], [1.3739854164907612, 5.2662702846795, 6.3170676655388025, 5.7002735780274563, -1.6212356534006416], [6.051503004397274, 6.5117593080758525, 1.7667848907709818, 2.0277368849718922, 2.371336758053566]], [[0.48232623096209437, 5.871187237314234, -1.6017500745014468, -2.6334874502373049, 1.9119868038607368], [-2.8536421174158475, 6.1361976397273414, -1.7228507819922867, -2.9972558083503378, -1.1813608694457054], [2.8176093143334677, 3.5855230961729045, 3.3923306356524092, -2.1922748441940945, 1.5384634720178942], [4.0467970540610967, 3.2150860166167368, 4.7980467865736758, 6.1219451357953796, 1.3787339096393607]], [[-2.1471065684120005, 3.4620094152298231, 2.3468557680707205, 5.7978616693565872, -0.025055634628465828], [4.7550362748071002, -0.29053755643729584, 6.102130394879282, 4.9976112915709461, 3.8186335823862647], [-1.5472759747074525, 6.4969852767212544, 0.23005639529234934, 4.3971598033817259, -0.46300626119740595], [-1.0434505184275364, 4.9001922808460847, -0.71078752796824762, -1.4276709142462107, 0.16497707433196673]], [[-2.4595256727666275, -3.0496801888314833, 2.5308400759658554, -2.8571239853709121, 3.892199079648746], [-4.9815806881677771, 0.69624383606955043, 2.9366229827596122, -2.4366168067726304, 0.48126811502910805], [1.1160450259275851, -0.092709694530562992, -0.24030585432583962, 2.8021966988947895, 0.46200275456543594], [3.4961466352945649, -2.1219464206758056, -3.1550775330823302, -4.0943314285403414, -4.0326185660770255]], [[-3.0817884281379917, -3.8296231843661945, 0.12515632729146153, 1.2266888040565536, -7.1702471315381295], [-0.077535419964830599, -2.2432828008088292, -4.5526352582246465, -6.9638068263145794, -6.5436130823629153], [1.2878607825456618, -0.52870472421359116, -4.7100657502593464, -1.5901059991386224, -4.6289348200275722], [0.34994315651982211, -4.0043625258287854, -4.0237059775327495, -7.2974690943941773, -7.4588274891152651]]], [[[5.2755801178889747, -0.33319681159973369, 0.83618280870873463, 1.8868189051583757, -0.62018010687220748], [5.703479822108477, 4.3717922283475001, -1.1380812664810041, -0.92884496272315387, 2.6517096099988091], [-2.9808903068315402, 4.6859411822971087, 3.6557713479125544, 0.2760884777417596, -1.7543618800414058], [-1.7447302399733848, -0.0035507813161288482, 5.6730692832268232, 5.9952977992433585, 6.2977125292419842]], [[7.2969621686464885, 8.1160438060484843, -0.1034190719193111, 4.6802988923996205, 6.7440106317372566], [7.4569133967842856, 7.9986612614014483, 2.2925849619816301, 3.2025820588668323, 7.3055748030812904], [2.480393217525279, 6.6237076407039721, 2.9788546761593131, 3.8684135301243101, 7.1225156399711338], [4.4940268702827817, 1.3256426281632523, 4.272957701506809, 3.7826855557580554, 5.8542133876275244]], [[0.62549790000109518, 7.1357135741408078, 4.3410180603894224, 1.8471238479147942, 2.9866207326220708], [2.6006809957897579, 7.0473370404445097, 4.4381444148858673, -0.21760199480586717, 6.9600027755122644], [-0.040032770831034625, -0.58272524298107342, 4.78653134227772, 6.0793054041810111, 4.6738724340416926], [-1.1474801372884436, 3.499862125542661, -0.92694086953005295, -0.52185905308729019, 1.6675073425259628]], [[-1.2540148744827242, 0.68028986494081423, -4.0284433123362016, -1.4415570849086823, -4.4822309402088125], [2.9301753420612444, 0.43765065658518676, 5.0140450997776407, -1.3251371629769615, -3.8292536089570328], [-0.87173049976277284, 2.0027600418500873, 0.62707463920362727, -2.3109999517272071, -2.8713760575616529], [2.6668818736162523, 3.9052993633753088, -4.3561834960333394, -2.9142191016575394, -0.29573517324324872]], [[-2.7647702445246933, 2.9171245984319336, 2.094515963375799, 0.66995046311007389, 5.3424635222064971], [-0.14679145890928247, -0.8464237795428442, 6.170979214664241, -1.6932009457902719, 1.005315081143431], [-1.3883695611099425, 5.3557101581591287, 3.2890680890066735, 1.8199867726013732, 1.7022983478568836], [-0.88031993879098769, 3.5646509353382925, 3.4933573843069485, -1.5365149460299143, 1.0041629066909756]]], [[[1.407152292696459, 6.5843888864410118, 8.7834004325810149, 0.72140208575140186, 4.7639398619688667], [9.6238318219251031, 4.6396268450572578, 7.0985762362699063, 5.7994300200964659, 3.1833713746374821], [9.4286666290001939, 2.8760898394444552, 5.1696531817022526, 6.6970525256619045, 3.7360797038713973], [5.2730887332849097, 0.8988539843120158, 6.8091350640810342, 5.2906136506507648, 1.2665894243929214]], [[-0.51297680906345366, 0.27128454934227975, 2.3889235847466512, -0.29862074184494336, 2.2437189951793797], [-0.44109394791431633, 5.3470076492769127, 7.9274279635347558, 0.21916390515961126, 7.6253288602845473], [0.077179748404720527, -1.4042978276818374, -0.15065043049326698, 4.0811646792989418, -0.15705060907246171], [2.1377111562565694, 2.7154245870823575, 3.6759026724030317, 5.1775442409037886, 4.7457992917625926]], [[-1.3349357910789497, 0.27418617373722487, -4.4659566835680478, 1.5385065928270842, 2.0247368778842363], [-1.8589534924457363, -1.3363715840271988, -2.4246463552660478, -4.7885130539950431, -4.0208248866557348], [-3.5227445987133486, -2.7840086398745889, -2.7203406980343958, -4.1051732578450988, -2.4188227907877149], [-0.87893834917632141, 2.4278477945371071, 0.054785982206047912, -5.7005476581273218, 0.88957054737455987]], [[7.0186497011219817, 6.159139049879375, 2.9083095880503089, 1.1803606759945167, -0.25945128994367739], [0.91796364495297933, 5.2709709477922413, 7.5910136918417095, 4.7680049193765806, 4.3040393457443704], [0.024444554743169533, 4.1243920063182582, 1.624420279173683, 7.1860787593569109, 7.703391979913329], [8.2753316065863913, 8.8600155313275586, 0.61799248865220857, 0.32212883332559983, 0.42878918007872935]], [[-0.5117395921308443, -3.1392699950530014, -4.5802372620546938, 1.802063493044928, -6.8482346449849407], [-3.8387318697262565, -3.5785591627433133, -0.33151101869728983, 2.2054969660325003, -3.6348042786797707], [-3.0530791727278439, -6.6278736995463223, -6.9248227073801472, -6.0278371529927401, -1.9412136202906201], [-2.4874954089638557, 0.5086855463305815, -5.7442573602185778, -0.19293679629338722, -1.9853372026056846]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_constData_rank4(self):
      arg0=numpy.array([[2.8580250601066446, 1.188749513838582, 0.77547152415519793, 0.27409642008861645, 1.2878782664518553], [-3.1340667692596846, -3.0715072427403536, -1.8040223750853035, 1.987654568826148, -3.7091100330548188], [-0.90548765778826557, 0.35810565581452281, 3.3460412928015124, 0.73937905959498806, -4.4329531894213936], [-3.8008589485309621, -3.6770964647562785, -4.1998819874636881, 0.81624591189610918, 1.5359583398710068]])
      arg1=Data(numpy.array([[[[1.8504333649568947, -4.762147212898169, 4.7615753940390295, 0.18043493824447587, -0.052405721481669332], [-1.8621351221482607, -4.5251783683904083, -1.611315498890562, -0.24053697580681632, 3.2965640584339546], [-1.4438593124499621, 3.6507281174143387, 4.9958477640343943, -4.2956809128917515, -4.1813620276060393], [3.3188208846431611, -2.8822810991615633, -3.9386662860907706, -0.32651651491182676, -4.115506003732734]], [[0.27554024837070124, 0.40819093523262406, 0.2121273775518171, -2.6845159861856005, 3.787531239593509], [-1.1125690820164111, -3.9329354330954902, 2.4051570985580506, 3.7328103303325726, 1.6708809590338056], [1.7786153210095792, 4.858922760432046, -4.6794690345228638, -2.6538496580960249, 4.1671929821813531], [-0.83661116800103663, -1.4609922711816461, 3.4918121506809836, -0.65246745210086665, -3.8040179204352076]], [[-4.7040711019232333, 1.6136911504133868, -2.3090405521745518, 1.1036531126578053, 2.3623932394838176], [3.1888175878375229, -0.19235140322289457, 1.9951867361239568, -4.3162393016848712, 1.4349129808484173], [0.12027984186361351, -2.8299965905468474, -4.9374408204818359, -1.3779111357415639, 3.9220193116124538], [-2.4485274696751826, 1.6132871970129052, 0.061593150839761357, -2.4424326625574189, -4.5244894258823756]], [[4.2690871405862847, 3.8406825725312483, -1.5629891290272715, -1.7968633646707355, -0.87262098528729126], [-4.1363742971384223, -4.3179046701874038, 3.7147395499045501, -4.7540078706893185, -3.7475635452161384], [2.8883550479900109, 2.4044875692232361, 2.5112077698324375, 1.1036758395708652, 3.8023049185277955], [-0.66526375234993385, 3.4193299111339144, -2.8709273237972677, -2.9075228980957779, -3.6448033072350619]], [[-4.224048674593881, -0.3118915048287576, -1.5662302781259916, 1.8920077170230094, 2.788877198042103], [3.6651015802328075, 1.0556608950864668, 4.6132128416563152, -4.0423411509029252, 4.4371304420626974], [-4.2643652412012569, 0.99775404620370267, 2.9903310945953407, -4.2418144172804473, 2.2008727614354138], [4.8124717059339783, -0.31257283186843399, -0.97968701087139998, -2.1728354368176839, 0.92523850700189492]]], [[[-1.1863486887578221, 2.2982009463468431, -2.8360344694427777, -3.1851045317410431, 0.49147289907774638], [3.9984232465772784, -0.68318690357174194, -1.3740954770096492, -1.354713883555243, -2.4183479368760361], [1.7308320350168813, 4.2070376537064877, -4.7958071152260642, 0.38743893460219603, -4.8078708164989479], [-0.54305733290268776, -3.3168080330313576, 4.4350377851024518, -2.9224652623144722, -3.9003918751539537]], [[-2.8403035165174639, 1.6092248749007236, -4.0913773667189837, 4.6642498164107487, -3.1314514924982917], [-2.814317020028513, -1.9008827851700936, 2.8833802722902968, -0.63865908838697472, -2.8122149609638103], [3.2934656831463709, -4.2809744322476764, 3.3118652152592034, -4.3062074797391956, 3.255824063681219], [4.5429023109118418, 0.73251894777712856, -0.41576797600934512, -0.97487497851384575, 4.3181124524849945]], [[-4.1316595211780438, -2.4146385652126767, -3.8244418481783757, -4.3702993333998927, 1.4601818972269029], [4.5228563417864809, -0.38566395818319599, 3.3907193499226889, 3.4814781220851554, 1.4065207113878113], [-3.6955806647015099, -0.58872780160874605, -1.6499154627320323, -4.2429666013598251, 2.917356218329143], [0.023469643057369538, 2.2739972693492838, 2.3855773243363485, -3.9083681941703077, 2.7824827573455471]], [[-4.4599880641910818, -2.8257413455804659, 1.6783164389425052, -3.7584235511769393, -4.9817427576709044], [1.3318524716826738, -4.1453567629070687, 0.82851611332255182, 2.7474466349699034, -2.1474871965631817], [-4.951061569000295, 0.018182291642081516, 1.4239075518959377, 4.6532526050441412, 1.0586461104012388], [1.2161633109679055, 1.4477566373645363, 4.2334047239506525, -4.9310751714874677, 0.99619541341950146]], [[-2.7689891296158899, -2.905032521023172, -1.5264039983105624, 4.0658669903054836, -1.225849659658369], [2.8928966835199974, -0.45952310186604528, -4.3981749303693993, 3.6117224437546795, 3.211830443022567], [-3.5850133299845499, 2.0967777764389979, -3.6437555102060024, 3.8025861912217742, 3.9648526855547868], [-0.84204778959356474, -0.43764073847771101, -1.8384973132083537, -4.964101843922915, -2.2152058603869773]]], [[[2.0882949439050371, -3.4649826810804853, -3.4350561953191083, -2.5183988363728647, -3.9890415898720279], [-2.6111677869878491, -1.7270944652631268, -2.1311460286948303, 3.7152284475863855, -2.4556972251596454], [-2.6737911893548425, 4.4905416219523371, -0.63154204260999514, 0.50798568405367739, 4.9751106818864805], [-0.2773241269984128, -2.7300135301355608, 1.4935592632482635, -0.92526333643181058, 1.6007430829515883]], [[2.8959657046442953, -4.3531818942748277, 2.192005815782883, 1.4161156689577652, 4.1106457071404758], [1.5483962877617312, -0.70443779590603661, -0.11893761457318686, 2.2278021087104429, 2.8857822395636603], [-3.3844202411990141, -2.3797101164646586, 0.73596633817142276, -3.9985771403672632, 4.9176755153757039], [2.1139544024374546, -2.0310102192877846, 1.6035442566777069, -4.3380464708942696, 3.8727357987540945]], [[3.7436234658684047, 1.3793273468616549, -0.79952272763287979, -4.6764995229624269, 2.7160127222779824], [1.7196185871622269, 0.36130158532060186, 0.46881341590830328, 2.0398952296353432, -2.2586213088416249], [-4.1698045240754125, 1.4563395598280247, 0.11250837495297894, -1.0911747417912543, 0.099203225734520117], [0.83218939220810473, -0.72002130591228752, 1.4494137838099981, 2.9854703777946332, -1.883799384479683]], [[-1.0792482039821327, 3.3328340164353794, 4.479194534588931, -2.7583467751381461, 1.3958547820397795], [1.259651141456775, 2.7358538788976841, 3.6506116331711187, 4.1846732363766179, 2.1681799729527889], [-0.33349163515652869, -4.8242695874321173, 4.6553588521801128, 2.7426311151752021, -4.7351543133157232], [4.726407197578256, 3.3537043298744837, 4.4507429821853481, -4.1756575109953786, 0.66670143399686133]], [[-4.7879123328311817, 2.2247343480872068, 3.7629345323177095, 3.8235490592751518, 2.9552537961979457], [-1.6902942874319193, 3.2257610337130966, 1.7211083155370952, -3.3857580026513787, 3.7848194649302975], [-1.5519916797377165, -2.7366590133650401, -4.0031330599442612, 3.2523048160616543, 0.68595984865923665], [-1.7696496841256137, -3.3189016871687849, -3.9250902561715506, 2.1108954142999314, 2.1416336618572487]]], [[[-2.023341380061626, -4.2697784369742688, 2.3276408003416682, 2.1220154066360628, 0.37173041770138049], [-1.4124750316851484, -3.831639696822168, 3.5847004123916886, 1.2308909338678706, -1.2575793821080241], [-4.0357059290608817, -0.81606490155036759, -4.7388671701938456, -4.4197752515978808, -2.4266661593232151], [0.24983451924030931, -1.4515473412449009, -4.5359410915376817, -1.8791493084500219, 2.09685682944402]], [[4.5859883427148223, -1.867259504914526, -2.725521414244978, -3.0379475857437477, -4.6078623403041483], [-1.6230791448228565, 2.0874219223183399, 0.30485037259515924, 0.41265395524574178, -1.4756618034173838], [3.7017293114524961, -0.45705103805503278, 0.76901403903799448, 2.430474281104086, 3.0037211904166004], [4.9216633781162713, 2.9789386406355209, -2.1973538006019275, -3.3473330737909457, -3.5954462512139096]], [[4.3290106627533493, 1.2826364155647969, -0.68726424966154021, -1.1700603599341797, -2.9943994372347271], [4.0809461314364572, -4.9751163746165652, 0.4022940077873649, 0.0066056645768881594, -3.9035107309898898], [1.519438574373785, -1.4273412527019067, 2.6503328273976026, -0.81393871097954928, -1.0376214129333592], [-0.3300537349042818, 1.7871271388164969, -1.041854210402958, -3.9850989962759265, -0.026286930114521212]], [[-2.587572972827398, -2.3736919501895679, -3.6510222769979181, 1.2108565608750261, -4.594810191250609], [-4.7111848407208736, -3.6262718861340595, 4.6788226113022731, 4.1055163283282585, -4.5109598902438686], [1.5797185232197686, 0.91184359948499694, 4.870196012796292, -3.8409321683144624, -2.1500627811902753], [-2.6235830698654592, 3.0492548271742486, 0.55746205136961002, -1.9463844410813813, -0.30041155497086613]], [[3.092138144699172, -1.1638862761247069, -4.1476300253532923, 1.2082240637362807, 1.3448936254238397], [-3.6056863621596849, 3.4493245953495464, 2.3810931077977813, -0.75163071705157769, -4.4402078570135464], [-4.8701559784355988, -2.7287765933388486, 4.0005658241799917, -4.2138564251397277, 2.3991878972791501], [-1.270650018873746, -1.5047292771096132, 0.051769888548166776, -3.5439170883528703, 2.7470012395183847]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[4.7084584250635393, -1.9041221527915244, 7.6196004541456741, 3.0384599983511205, 2.8056193386249753], [0.99588993795838388, -1.6671533082837637, 1.2467095612160826, 2.6174880842998283, 6.1545891185405992], [1.4141657476566825, 6.5087531775209833, 7.8538728241410389, -1.4376558527851069, -1.3233369674993947], [6.1768459447498056, -0.024256039054918688, -1.080641225984126, 2.5315085451948178, -1.2574809436260894]], [[1.4642897622092832, 1.596940449071206, 1.4008768913903991, -1.4957664723470185, 4.976280753432091], [0.076180431822170824, -2.7441859192569082, 3.5939066123966326, 4.9215598441711546, 2.8596304728723876], [2.9673648348481612, 6.0476722742706279, -3.4907195206842818, -1.4651001442574429, 5.3559424960199351], [0.35213834583754533, -0.27224275734306413, 4.6805616645195656, 0.53628206173771531, -2.6152684065966256]], [[-3.9285995777680354, 2.3891626745685848, -1.5335690280193539, 1.8791246368130032, 3.1378647636390156], [3.9642891119927208, 0.58312012093230337, 2.7706582602791547, -3.5407677775296733, 2.2103845050036153], [0.89575136601881145, -2.0545250663916494, -4.1619692963266379, -0.60243961158636594, 4.6974908357676517], [-1.6730559455199847, 2.3887587211681032, 0.83706467499495929, -1.6669611384022209, -3.7490179017271776]], [[4.5431835606749011, 4.1147789926198648, -1.288892708938655, -1.522766944582119, -0.59852456519867481], [-3.8622778770498059, -4.0438082500987873, 3.9888359699931666, -4.4799114506007021, -3.473467125127522], [3.1624514680786273, 2.6785839893118526, 2.785304189921054, 1.3777722596594817, 4.076401338616412], [-0.39116733226131739, 3.6934263312225308, -2.5968309037086512, -2.6334264780071615, -3.3707068871464454]], [[-2.9361704081420257, 0.97598676162309772, -0.27835201167413626, 3.1798859834748647, 4.0767554644939583], [4.9529798466846628, 2.3435391615383221, 5.9010911081081705, -2.7544628844510699, 5.7250087085145527], [-2.9764869747494016, 2.285632312655558, 4.2782093610471961, -2.953936150828592, 3.4887510278872691], [6.1003499723858337, 0.97530543458342134, 0.30819125558045535, -0.88495717036582855, 2.2131167734537502]]], [[[-4.3204154580175071, -0.83586582291284151, -5.9701012387024619, -6.3191713010007273, -2.6425938701819383], [0.8643564773175938, -3.8172536728314266, -4.5081622462693343, -4.4887806528149277, -5.5524147061357212], [-1.4032347342428033, 1.0729708844468031, -7.9298738844857493, -2.7466278346574886, -7.9419375857586321], [-3.6771241021623724, -6.4508748022910423, 1.3009710158427672, -6.0565320315741573, -7.0344586444136379]], [[-5.9118107592578175, -1.4622823678396299, -7.1628846094593372, 1.5927425736703951, -6.2029587352386457], [-5.8858242627688666, -4.9723900279104472, -0.18812697045005677, -3.7101663311273283, -5.8837222037041634], [0.22195844040601731, -7.35248167498803, 0.24035797251884983, -7.3777147224795492, 0.18431682094086543], [1.4713950681714882, -2.338988294963225, -3.4872752187496987, -4.0463822212541993, 1.246605209744641]], [[-5.9356818962633469, -4.2186609402979798, -5.6284642232636788, -6.1743217084851967, -0.34384047785840055], [2.7188339667011774, -2.1896863332684995, 1.5866969748373854, 1.6774557469998519, -0.39750166369749218], [-5.4996030397868134, -2.3927501766940495, -3.4539378378173358, -6.0469889764451281, 1.1133338432438395], [-1.780552732027934, 0.46997489426398031, 0.58155494925104501, -5.7123905692556107, 0.97846038226024357]], [[-2.4723334953649339, -0.83808677675431786, 3.6659710077686531, -1.7707689823507913, -2.9940881888447564], [3.3195070405088218, -2.1577021940809207, 2.8161706821486998, 4.7351012037960514, -0.15983262773703366], [-2.963407000174147, 2.0058368604682295, 3.4115621207220856, 6.6409071738702892, 3.0463006792273868], [3.2038178797940535, 3.4354112061906843, 6.2210592927768005, -2.9434206026613197, 2.9838499822456495]], [[-6.4780991626707092, -6.6141425540779908, -5.2355140313653816, 0.35675695725066481, -4.9349596927131874], [-0.81621334953482139, -4.1686331349208636, -8.1072849634242186, -0.097387589300139243, -0.49727959003225175], [-7.2941233630393683, -1.6123322566158209, -7.3528655432608208, 0.093476158166955425, 0.25574265249996797], [-4.5511578226483831, -4.1467507715325294, -5.5476073462631721, -8.6732118769777333, -5.9243158934417961]]], [[[1.1828072861167716, -4.3704703388687509, -4.3405438531073735, -3.4238864941611302, -4.8945292476602935], [-3.5166554447761147, -2.6325821230513924, -3.0366336864830958, 2.8097407897981199, -3.361184882947911], [-3.5792788471431081, 3.5850539641640715, -1.5370297003982607, -0.39750197373458818, 4.069623024098215], [-1.1828117847866784, -3.6355011879238264, 0.58807160545999793, -1.8307509942200761, 0.69525542516332273]], [[3.2540713604588181, -3.9950762384603049, 2.5501114715974058, 1.774221324772288, 4.4687513629549986], [1.906501943576254, -0.34633214009151381, 0.23916804124133595, 2.5859077645249657, 3.2438878953781831], [-3.0263145853844913, -2.0216044606501358, 1.0940719939859456, -3.6404714845527404, 5.2757811711902267], [2.4720600582519774, -1.6729045634732618, 1.9616499124922298, -3.9799408150797468, 4.2308414545686173]], [[7.0896647586699171, 4.7253686396631673, 2.5465185651686326, -1.3304582301609145, 6.0620540150794948], [5.0656598799637393, 3.7073428781221143, 3.8148547087098157, 5.3859365224368556, 1.0874199839598875], [-0.82376323127390005, 4.8023808526295371, 3.4585496677544914, 2.2548665510102581, 3.4452445185360325], [4.1782306850096171, 2.6260199868892249, 4.7954550766115105, 6.3315116705961456, 1.4622419083218294]], [[-0.33986914438714466, 4.0722130760303674, 5.2185735941839191, -2.018967715543158, 2.1352338416347676], [1.9990302010517631, 3.4752329384926721, 4.3899906927661068, 4.924052295971606, 2.9075590325477769], [0.40588742443845938, -4.0848905278371292, 5.3947379117751009, 3.4820101747701901, -3.9957752537207352], [5.4657862571732441, 4.0930833894694718, 5.1901220417803362, -3.4362784514003906, 1.4060804935918494]], [[-9.2208655222525753, -2.2082188413341868, -0.67001865710368413, -0.60940413014624184, -1.4776993932234479], [-6.1232474768533134, -1.207192155708297, -2.7118448738842984, -7.8187111920727723, -0.64813372449109607], [-5.9849448691591096, -7.1696122027864337, -8.4360862493656548, -1.1806483733597393, -3.746993340762157], [-6.2026028735470078, -7.751854876590178, -8.3580434455929442, -2.3220577751214622, -2.2913195275641449]]], [[[-5.8242003285925881, -8.0706373855052309, -1.4732181481892939, -1.6788435418948993, -3.4291285308295816], [-5.2133339802161105, -7.6324986453531301, -0.21615853613927349, -2.5699680146630914, -5.0584383306389862], [-7.8365648775918437, -4.6169238500813297, -8.5397261187248077, -8.2206342001288419, -6.2275251078541771], [-3.5510244292906528, -5.2524062897758625, -8.3368000400686437, -5.680008256980984, -1.7040021190869421]], [[0.90889187795854376, -5.5443559696708045, -6.4026178790012569, -6.7150440505000262, -8.2849588050604268], [-5.300175609579135, -1.5896745424379386, -3.3722460921611193, -3.2644425095105367, -5.1527582681736623], [0.02463284669621757, -4.1341475028113113, -2.908082425718284, -1.2466221836521925, -0.67337527433967814], [1.2445669133599928, -0.69815782412075755, -5.874450265358206, -7.0244295385472242, -7.2725427159701876]], [[0.12912867528966121, -2.9172455718988912, -4.8871462371252283, -5.3699423473978678, -7.1942814246984153], [-0.11893585602723089, -9.1749983620802524, -3.7975879796763232, -4.1932763228868, -8.103392718453577], [-2.6804434130899031, -5.6272232401655948, -1.5495491600660856, -5.0138206984432374, -5.2375034003970473], [-4.5299357223679699, -2.4127548486471913, -5.2417361978666461, -8.1849809837396137, -4.2261689175782093]], [[-1.7713270609312888, -1.5574460382934587, -2.834776365101809, 2.0271024727711353, -3.7785642793544998], [-3.8949389288247644, -2.8100259742379503, 5.4950685231983822, 4.9217622402243677, -3.6947139783477594], [2.3959644351158778, 1.7280895113811061, 5.6864419246924012, -3.0246862564183532, -1.3338168692941661], [-1.80733715796935, 3.8655007390703577, 1.3737079632657192, -1.1301385291852721, 0.51583435692524304]], [[4.6280964845701789, 0.37207206374629997, -2.6116716854822855, 2.7441824036072875, 2.8808519652948466], [-2.069728022288678, 4.9852829352205532, 3.9170514476687881, 0.78432762281942914, -2.9042495171425395], [-3.3341976385645919, -1.1928182534678418, 5.5365241640509986, -2.6778980852687209, 3.9351462371501569], [0.26530832099726087, 0.031229062761393589, 1.5877282284191736, -2.0079587484818635, 4.2829595793893915]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[1.4855757851565006, 4.592422017921864], [2.0280059245681166, 4.2601244951399018]], [[1.7191615244012759, 1.4326389626940728], [0.90578593066574431, -0.22794924855318932]], [[4.0092236364391738, 0.52795009860981601], [-0.10435676267763405, 2.7339288680534235]], [[1.5778656730443466, 4.4510976136290807], [0.42242953886981649, 0.36441554106929619]], [[-2.4841378596114252, 3.3239651217851964], [0.53309243052212629, 3.0555705419578061]], [[3.480841945191349, -3.3670686033382271], [-1.6766223721465225, 3.6804429353275978]]])
      arg1=Data(numpy.array([[[2.7929270130010924, 2.2270977999144002], [0.66405123130490473, -0.80494237058631146]], [[-2.8774802414319933, 4.578452062902528], [-4.3698941872869126, -2.0673385268993929]], [[-1.8047092288469155, -4.382456549462967], [0.97270035640662833, -3.1474452878190862]], [[-4.8856773378698133, -1.9989278777281818], [-0.92623553823177218, 0.5407018501828631]], [[-4.3808927355120089, 3.2060595939649215], [1.5663285405992831, -0.81591321250481386]], [[0.072778306027705852, -2.2213953844918697], [4.3640020296432862, -2.863020868975986]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[4.2785027981575929, 6.8195198178362642], [2.6920571558730213, 3.4551821245535903]], [[-1.1583187170307174, 6.0110910255966008], [-3.4641082566211683, -2.2952877754525822]], [[2.2045144075922583, -3.854506450853151], [0.86834359372899428, -0.41351641976566267]], [[-3.3078116648254667, 2.4521697359008989], [-0.50380599936195569, 0.90511739125215929]], [[-6.8650305951234341, 6.5300247157501179], [2.0994209711214094, 2.2396573294529922]], [[3.5536202512190549, -5.5884639878300968], [2.6873796574967637, 0.81742206635161185]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[2.474134670629744, 4.4842465874243764], [1.4722153036469532, 0.77363883776305364]], [[-4.9259610174629387, -3.0322926343163981], [-4.2158043710614326, -2.5044309884367055]]], [[[-4.2785349478662935, -3.8567856586138962], [-2.3011637901395678, 0.55516391816189348]], [[-3.2365871687441796, 0.4969721129578053], [-2.6383460435648862, 0.05209479962936836]]], [[[4.0645581938416573, -1.8836901710596745], [-0.63711423386463295, -2.3812889662264372]], [[1.3144366351155243, -2.7264559754703042], [2.1339292851015434, 4.1035108707408359]]], [[[1.0976056948168988, 3.9787908513142103], [3.1981683893255859, -1.4570760047554066]], [[4.7224263465411518, -1.0216600494154982], [1.3805464603558004, -2.3990587475048586]]], [[[4.1396048367039899, 1.517724946116024], [-1.0019791303587287, 4.9221162695009344]], [[2.2704241486979253, -3.6951359672208071], [3.9012294080771746, 4.3746231373392259]]], [[[2.3673198845373236, -4.0888781807106351], [4.0493911768444111, -4.2198150018531591]], [[2.3050672142283792, 4.366730830872239], [-4.6585580501363584, 3.5409308209507433]]]])
      arg1=Data(numpy.array([[[1.9326907074685487, 3.6355958407163076], [1.0307295377428281, -0.79264054034315734]], [[-0.89452504096874108, -0.96963817458160229], [-2.6934180041237155, -3.7818498530817104]], [[1.2429864853175001, -1.1990661605405082], [-3.8970096129185903, 4.5354987684768364]], [[1.5002420194790833, -1.4564755013231769], [3.9538901660856229, 3.8197495107984327]], [[3.8256506683005718, 2.0039200009579448], [-0.84380276093090156, 2.7572206688276024]], [[4.4841363776124918, 1.825484584127266], [0.2577935543700649, 0.093196543461898429]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[4.4068253780982927, 6.4169372948929251], [5.1078111443632608, 4.4092346784793612]], [[-3.8952314797201106, -2.00156309657357], [-5.0084449114045899, -3.2970715287798629]]], [[[-5.1730599888350346, -4.7513106995826373], [-3.2708019647211701, -0.41447425641970881]], [[-5.9300051728678955, -2.1964458911659102], [-6.4201958966465966, -3.729755053452342]]], [[[5.3075446791591574, -0.6407036857421744], [-1.8361803944051411, -3.5803551267669453]], [[-2.5825729778030659, -6.6234655883888944], [6.6694280535783799, 8.6390096392176723]]], [[[2.5978477142959822, 5.4790328707932936], [1.741692888002409, -2.9135515060785835]], [[8.6763165126267747, 2.9322301166701248], [5.2002959711542331, 1.4206907632935741]]], [[[7.9652555050045617, 5.3433756144165958], [1.0019408705992161, 6.9260362704588792]], [[1.4266213877670237, -4.5389387281517086], [6.658450076904777, 7.1318438061668283]]], [[[6.8514562621498154, 0.39525819690185671], [5.8748757609716771, -2.3943304177258931]], [[2.5628607685984441, 4.6245243852423039], [-4.5653615066744599, 3.6341273644126417]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_constData_rank4(self):
      arg0=numpy.array([[[0.87929878755024671, 1.4194273370701262], [-3.9866216206545069, -2.9841020248999706]], [[0.92393523223474361, 0.65442793172150182], [-1.5778811676323947, 1.7598978940956744]], [[-0.3058091926182005, -1.5671620444826084], [2.7834687233724065, 3.2038285430767743]], [[-1.3150801102930387, -0.50348263254637615], [2.1849621054979451, -2.2050780058038932]], [[-3.4572600566446852, -0.23492688296647746], [0.42935548051574557, 1.1922821407966779]], [[-2.6014057673919835, 3.0916749286841085], [-3.7514412975306763, 4.9845519872927753]]])
      arg1=Data(numpy.array([[[[1.1458256626962351, 2.7225998999937371], [1.7841840749516447, 0.81173748266682022]], [[-1.6236673673943014, -3.6518994812687824], [2.0734809014836797, 4.4320950788052045]]], [[[-2.2360747337584854, -1.6409566802199804], [1.1428338440775576, -0.70563265838477562]], [[4.0000279809190911, 1.7974025366123625], [2.2856323776563379, 2.4031860681147634]]], [[[-1.6291581653941298, 3.6730016201745759], [4.4044631379672516, 3.1262060036606201]], [[3.5634493045733002, -2.5474250321606862], [0.38518282463545006, 4.1061703824281874]]], [[[2.3497190922100692, 4.0757273926960043], [1.9221857862465193, -1.8805723075610525]], [[4.8299339144390778, 0.28726384397307747], [-4.9808188665491269, 0.54446459525247981]]], [[[-4.9665553553819546, 1.1127557155934751], [-3.2405055043914208, 1.6654311983508894]], [[-1.639139408901229, -4.0117290183646599], [2.3809485833870951, 2.7117186918783078]]], [[[2.4591501532029936, -3.8093651908590029], [2.5963885479737563, -1.1210588064973734]], [[4.5858340066382475, 0.53497872508606736], [-0.24181800527410235, -2.2765716173900188]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[2.0251244502464818, 3.6018986875439838], [3.2036114120217709, 2.2311648197369465]], [[-5.6102889880488078, -7.6385211019232893], [-0.91062112341629087, 1.4479930539052339]]], [[[-1.3121395015237418, -0.71702144798523682], [1.7972617757990594, -0.051204726663273803]], [[2.4221468132866963, 0.21952136897996777], [4.0455302717520123, 4.1630839622104379]]], [[[-1.9349673580123303, 3.3671924275563754], [2.8373010934846432, 1.5590439591780116]], [[6.3469180279457067, 0.23604369121172031], [3.5890113677122244, 7.3099989255049618]]], [[[1.0346389819170305, 2.7606472824029655], [1.4187031537001431, -2.3840549401074287]], [[7.0148960199370229, 2.4722259494710226], [-7.1858968723530197, -1.6606134105514134]]], [[[-8.4238154120266397, -2.3445043410512101], [-3.4754323873578983, 1.4305043153844119]], [[-1.2097839283854834, -3.5823735378489143], [3.5732307241837731, 3.9040008326749858]]], [[[-0.14225561418898991, -6.4107709582509864], [5.6880634766578648, 1.9706161221867351]], [[0.83439270910757113, -3.216462572444609], [4.7427339820186729, 2.7079803699027565]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[3.9966200745410809, 1.2028622982370241, -1.8135461319610426, -1.5195273264205733], [0.29690221401331574, -3.6382142458415059, -3.0501949480769177, -1.8280428627572576], [3.0726464012762449, -3.8362520423687698, 2.8749553418225107, 3.8421808906823909]], [[2.3061082935350612, -2.8517676710746565, -4.1856406719805612, 2.8650282445593787], [-2.4048028594006654, 3.7666862895520197, -1.331246874082912, 0.63850222294528525], [4.0089470874746063, -3.1697492377395187, 4.4924327062801144, -0.80925245192099204]]], [[[3.3018366400039199, 1.0751046148751566, -1.9477061303084255, -1.0076106163913359], [4.3962984233050442, -4.9954647727544099, -1.2581321291689704, 4.796555377332858], [-1.3569950917708606, 1.2583065674917249, -2.6504013261101891, 0.2682480975978665]], [[-2.161210576845801, -4.0060957532079575, -0.48468652831013959, -2.4607169105069469], [-3.6801085864116989, -2.2131837432858328, -2.7974581268102883, -2.1030044118139521], [3.6725319192738688, -3.5625383315388461, -0.69627599070529644, 3.6123705220140661]]], [[[0.66793016177823894, 1.0808168489402483, -2.7513581031607726, 0.81579401000778429], [2.3689123255981235, 0.29161716302733964, -1.7211296517853683, -0.10217864392211773], [-4.570632426848416, 3.7312545479631218, 1.58208984774891, 0.69117333613936438]], [[3.2248700092819487, 1.9105465008516749, -2.0850063667188214, 0.180386156727951], [-2.9371071869299215, 4.1460039955308883, 4.9856740243290272, -2.3235719256028329], [-0.92730315915474648, -4.6112626510913692, -0.16497684436998306, 4.6396315697339769]]]])
      arg1=Data(numpy.array([[[[-0.68285498064339301, -0.14086726111035031, 4.2021519517894195, -3.782516178231079], [4.2334952985345549, 2.3395551323807862, -0.18905719419452627, 1.6713989138981749], [3.078131425978885, -4.8277582990623564, -1.875666726259233, 1.8800890537204253]], [[2.3267109119324116, 2.8851078887187818, 3.6961099009586995, -3.7841788451876637], [0.85708627105687185, -4.274163126242235, 4.131290250967961, 4.8706286708049671], [4.0543037244090119, 3.2611506059182531, -4.372882017495332, -0.56080161126494144]]], [[[1.9545834332451317, 3.2082205263902335, -0.63290842822035653, -0.99325550153374209], [-3.1034123130296565, 3.7189316816848041, -3.6820276431822965, 1.4178097163921111], [-0.8613235785818496, 0.075951319247209881, 2.7359512534002413, -0.48246576548086306]], [[3.1415112764769511, -1.4670632874026013, -0.15961675748523874, 0.45637837331374609], [-1.3935025459283832, 1.0648200618737969, 4.0253653598331347, 0.69897838068494522], [2.0928075769048569, 1.5134575161100683, -1.6872438763780648, -1.4558646550396981]]], [[[-4.1178106256130356, -0.43483937514580617, 1.7174834839390405, -2.3089116401597276], [-0.99364655611639918, -4.3893347416383026, -2.3042950616430491, -0.37071452713660769], [4.794533375165976, 0.98587287932622658, 4.3993983233394811, -0.92846510663843773]], [[-4.298118062103824, -0.39448215293227928, -3.6328803224419657, 2.0508766237883203], [2.3686777973396689, 3.1232384523211074, 1.5935223767143487, 1.9645601445031815], [-0.78192421462220363, -1.8459631111008932, -4.2864724591233641, 4.7210908063645789]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[3.3137650938976879, 1.0619950371266738, 2.3886058198283768, -5.3020435046516523], [4.5303975125478706, -1.2986591134607197, -3.239252142271444, -0.15664394885908273], [6.1507778272551299, -8.6640103414311262, 0.99928861556327764, 5.7222699444028162]], [[4.6328192054674728, 0.033340217644125314, -0.48953077102186171, -0.91915060062828502], [-1.5477165883437936, -0.50747683669021537, 2.800043376885049, 5.5091308937502523], [8.0632508118836181, 0.091401368178734455, 0.11955068878478237, -1.3700540631859335]]], [[[5.2564200732490516, 4.2833251412653901, -2.5806145585287821, -2.000866117925078], [1.2928861102753877, -1.2765330910696058, -4.9401597723512669, 6.214365093724969], [-2.2183186703527102, 1.3342578867389348, 0.085549927290052175, -0.21421766788299657]], [[0.98030069963115007, -5.4731590406105592, -0.64430328579537832, -2.0043385371932008], [-5.0736111323400817, -1.1483636814120359, 1.2279072330228464, -1.4040260311290069], [5.7653394961787257, -2.0490808154287778, -2.3835198670833613, 2.156505866974368]]], [[[-3.4498804638347966, 0.64597747379444215, -1.0338746192217321, -1.4931176301519433], [1.3752657694817243, -4.097717578610963, -4.0254247134284178, -0.47289317105872541], [0.22390094831756002, 4.7171274272893484, 5.9814881710883911, -0.23729177049907335]], [[-1.0732480528218753, 1.5160643479193956, -5.7178866891607871, 2.2312627805162712], [-0.56842938959025258, 7.2692424478519957, 6.5791964010433759, -0.35901178109965137], [-1.7092273737769501, -6.4572257621922624, -4.4514493034933471, 9.3607223760985558]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(4.60573897745)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.25269155651)+(1.-msk_arg1)*(-1.23770285741)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.35304742094)+(1.-msk_ref)*(3.36803612004)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([2.2900319386209951, -0.047592113791001367])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-4.98403942172)+(1.-msk_arg1)*(-2.81701471572)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-2.6940074830967111, -5.0316315355087076])+(1.-msk_ref)*numpy.array([-0.52698277709476837, -2.8646068295067648])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(-4.00112431289)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.9629919063161996, 1.2765499415043582])+(1.-msk_arg1)*numpy.array([-0.9073047739676996, -1.7646674858753264])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.038132406575114963, -2.7245743713869564])+(1.-msk_ref)*numpy.array([-4.9084290868590141, -5.7657917987666405])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[-2.9629578377835672, -2.6264422578203819, -1.1088919201404579, 1.5964046917929089, -0.83225525989595006], [1.1892753666066316, -1.5735739125194295, 1.5688687500818466, 2.9147772831287702, -3.348076613620603], [-2.2894758449431096, 2.662648292021613, 4.2178629478859335, -1.5580962346132732, -0.62664433189724278], [2.1660815173065151, -0.6155154221384489, 0.65430184225608112, -3.8147837355630498, -2.933535895121083]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.6066905502)+(1.-msk_arg1)*(4.68093942402)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-5.5696483879829346, -5.2331328080197492, -3.7155824703398252, -1.0102858584064585, -3.4389458100953174], [-1.4174151835927358, -4.1802644627187968, -1.0378218001175208, 0.30808673292940281, -5.9547671638199704], [-4.896166395142477, 0.055957741822245666, 1.6111723976865662, -4.164786784812641, -3.2333348820966101], [-0.44060903289285225, -3.2222059723378162, -1.9523887079432862, -6.4214742857624172, -5.5402264453204504]])+(1.-msk_ref)*numpy.array([[1.7179815862346866, 2.0544971661978719, 3.5720475038777959, 6.2773441158111627, 3.8486841641223037], [5.8702147906248854, 3.1073655114988243, 6.2498081741001004, 7.595716707147024, 1.3328628103976508], [2.3914635790751442, 7.3435877160398668, 8.8988023719041873, 3.1228431894049806, 4.054295092121011], [6.8470209413247689, 4.0654240018798049, 5.3352412662743349, 0.86615568845520396, 1.7474035288971708]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(3.38090071426)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-2.9470358042208655, -3.8304675083019015, -1.2982540158949964, 1.1801250555067879, -4.6982852593800741], [-0.41923346775052295, 1.9987918776830496, -4.9410769542298718, 3.0299688949586265, 4.8555443920406169], [1.9403761425197512, -3.9599253672326995, -2.1961931458000095, -3.8636223760508601, 2.4740127777147549], [4.28535209881505, 1.922639288328865, 4.8101975818734548, -1.781400869082308, 3.1635918597018211]])+(1.-msk_arg1)*numpy.array([[-3.7221657731108659, -2.8121902336705928, 3.6607358174884723, -0.91780337830377157, -4.1288937127729302], [-4.7621480451289768, -1.1599815241662883, 1.9710788247226674, -3.9024560439274989, -3.7303929121375656], [3.0929788420766435, -3.2404004842391676, -0.81956042839048759, 4.7767758413406707, -0.63005574639846262], [-2.9853286381703201, -1.2971386598842241, 1.4111808906334842, 0.83506922922911375, -2.5608605181097088]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.43386491003672845, -0.44956679404430755, 2.0826466983625975, 4.5610257697643819, -1.3173845451224802], [2.961667246507071, 5.3796925919406435, -1.5601762399722778, 6.4108696092162205, 8.2364451062982109], [5.3212768567773452, -0.57902465297510552, 1.1847075684575845, -0.48272166179326614, 5.8549134919723489], [7.6662528130726439, 5.303540002586459, 8.1910982961310488, 1.599499845175286, 6.5444925739594151]])+(1.-msk_ref)*numpy.array([[-0.34126505885327196, 0.56871048058700113, 7.0416365317460663, 2.4630973359538224, -0.74799299851533618], [-1.3812473308713829, 2.2209191900913057, 5.3519795389802614, -0.52155532966990492, -0.34949219787997166], [6.4738795563342375, 0.14050023001842638, 2.5613402858671064, 8.1576765555982647, 2.7508449678591314], [0.39557207608727385, 2.0837620543733699, 4.7920816048910782, 4.2159699434867077, 0.82004019614788515]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[-2.9965957016436393, 2.5513496079212752], [2.0523313415190785, -0.44495902037177704]], [[-4.4152858390991634, 2.4062495601144507], [0.88265312575714372, 1.4004060636252857]], [[4.3026462767988374, 0.50561631402907636], [3.7709669995862836, -0.39715492204412417]], [[2.3182516436021796, 1.1197213788595537], [1.916962719158458, 4.6452191838096404]], [[-1.7982123418191884, -1.8548467478545017], [3.1431719021094331, 1.589623643211401]], [[3.7084120036306132, 1.9805885957585705], [-0.24004768865483417, -1.3526411664571283]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.612726686437)+(1.-msk_arg1)*(0.354716228902)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.3838690152062423, 3.1640762943586722], [2.6650580279564755, 0.16776766606561999]], [[-3.8025591526617664, 3.0189762465518477], [1.4953798121945407, 2.0131327500626828]], [[4.9153729632362344, 1.1183430004664734], [4.3836936860236806, 0.21557176439327286]], [[2.9309783300395766, 1.7324480652969507], [2.529689405595855, 5.2579458702470374]], [[-1.1854856553817914, -1.2421200614171046], [3.7558985885468301, 2.2023503296487981]], [[4.3211386900680102, 2.5933152821959675], [0.37267899778256286, -0.73991448001973126]]])+(1.-msk_ref)*numpy.array([[[-2.6418794727411985, 2.906065836823716], [2.4070475704215193, -0.090242791469336225]], [[-4.0605696101967226, 2.7609657890168915], [1.2373693546595845, 1.7551222925277266]], [[4.6573625057012782, 0.86033254293151717], [4.1256832284887244, -0.042438693141683359]], [[2.6729678725046204, 1.4744376077619945], [2.2716789480608988, 4.9999354127120812]], [[-1.4434961129167476, -1.5001305189520608], [3.4978881310118739, 1.9443398721138418]], [[4.063128232533054, 2.3353048246610113], [0.11466854024760664, -0.99792493755468747]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(2.14379852417)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.6600665448460337, 0.39158054031152467], [4.2343525379142601, 0.64976700277080379]], [[4.9842847347892398, 3.644098148943911], [3.8874752660815908, 3.3663470814645873]], [[-4.6490822834154644, -3.0798116648583318], [3.1260739526726837, -3.8868887711749514]], [[-2.8642097063176877, -4.2669947917986235], [3.0032745142568391, 3.4794815844630875]], [[1.5288121462414948, 2.0991145250981713], [1.8475229456152373, 2.6327392607684637]], [[-3.712191166628843, 3.1177490397165357], [1.606700597299735, -2.4625923368587808]]])+(1.-msk_arg1)*numpy.array([[[-0.62883645206957439, 0.89239105373864813], [0.4363105856796734, 4.3785028805098047]], [[-4.4509546138452265, 2.0474958169941981], [0.97334676124469155, -0.5617039202427101]], [[-4.2739902202480549, -2.9840391229359131], [-3.7323990207426561, -2.5802135304134142]], [[0.72420849963517675, 1.447077441136563], [0.9219091575814673, 1.2268740485361809]], [[3.4736358477117761, 1.9312595345930355], [-3.4601687079725618, -3.6049558199872567]], [[-1.2569729676943564, -0.94051849356611239], [3.3412035223645304, -0.99059603799875529]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.8038650690136677, 2.5353790644791587], [6.3781510620818942, 2.7935655269384378]], [[7.1280832589568739, 5.7878966731115451], [6.0312737902492248, 5.5101456056322213]], [[-2.5052837592478303, -0.93601314069069774], [5.2698724768403178, -1.7430902470073173]], [[-0.72041118215005362, -2.1231962676309895], [5.1470730384244732, 5.6232801086307216]], [[3.6726106704091288, 4.2429130492658054], [3.9913214697828714, 4.7765377849360977]], [[-1.5683926424612089, 5.2615475638841698], [3.7504991214673691, -0.31879381269114671]]])+(1.-msk_ref)*numpy.array([[[1.5149620720980597, 3.0361895779062822], [2.5801091098473075, 6.5223014046774388]], [[-2.3071560896775924, 4.1912943411618322], [3.1171452854123256, 1.582094603924924]], [[-2.1301916960804208, -0.84024059876827906], [-1.588600496575022, -0.43641500624578011]], [[2.8680070238028108, 3.5908759653041971], [3.0657076817491014, 3.3706725727038149]], [[5.6174343718794102, 4.0750580587606695], [-1.3163701838049278, -1.4611572958196226]], [[0.88682555647327765, 1.2032800306015217], [5.4850020465321645, 1.1532024861688788]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[2.1823008198263256, -2.3152674786714633, -3.1127002262575933, 2.191405372350447], [-1.7729056650225883, 1.2715198918258057, 4.5680218088123414, -1.5092705871342127], [3.5174063500473878, -4.4606564854649982, 3.5334770594031042, 3.079668163881319]], [[1.899482549088809, -1.4440115299378631, -4.8551344423215319, 1.3495264630058017], [-2.5706260800313907, 1.6556225815866368, -0.65512755257743649, 2.8824461661642067], [1.6968882502780964, 3.4375994085335506, 2.1931274321762348, -4.7147891701569566]]], [[[-1.9553652005469466, 4.5719331923404241, 2.4322058373914608, 4.0253842756959859], [-2.658633066939494, -1.1398171696473334, 4.0272863887186325, 3.7670594774519799], [-1.3746505909758335, 4.4992205617450143, 4.2931588972856325, 2.4403022129179757]], [[-0.7381837945755958, 2.106479165114445, 4.4610289578805382, 0.30530165140419196], [2.3638772249926294, -3.9692931185668878, 4.7710467956452405, -1.0837893193606396], [-1.0832601946364697, -0.34460157870270969, 2.5605870461575595, 2.8752053435110589]]], [[[3.1601547151200453, -2.3733043342334414, -2.9907574203388956, 0.1777714228353533], [3.6378965896220219, 4.0503704167903152, -3.6012745150737988, -2.2860806597418035], [-0.86663162915989655, -4.1874531386712182, 4.9587921504621715, 0.27508923389959428]], [[4.4912620455354713, 4.0418923967212557, 2.7647916782994484, 4.5503056223637675], [0.39009270679185715, 2.9852981155544969, -3.707782238686268, 4.5473200613816598], [1.3162870342761988, 4.0248285414876346, 2.1703729759773669, 2.9353850590538277]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.61548093281)+(1.-msk_arg1)*(3.93973020395)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.7977817526371567, 1.3002134541393677, 0.5027807065532377, 5.806886305161278], [1.8425752677882428, 4.8870008246366368, 8.1835027416231725, 2.1062103456766184], [7.1328872828582188, -0.84517555265416711, 7.1489579922139352, 6.6951490966921501]], [[5.51496348189964, 2.171469402872968, -1.2396535095107009, 4.9650073958166328], [1.0448548527794403, 5.2711035143974678, 2.9603533802333946, 6.4979270989750377], [5.3123691830889275, 7.0530803413443817, 5.8086083649870659, -1.0993082373461256]]], [[[1.6601157322638844, 8.1874141251512551, 6.0476867702022918, 7.6408652085068169], [0.95684786587133708, 2.4756637631634977, 7.6427673215294636, 7.3825404102628109], [2.2408303418349975, 8.1147014945558453, 7.9086398300964635, 6.0557831457288067]], [[2.8772971382352353, 5.721960097925276, 8.0765098906913693, 3.920782584215023], [5.9793581578034605, -0.35381218575605677, 8.3865277284560715, 2.5316916134501914], [2.5322207381743613, 3.2708793541081214, 6.1760679789683905, 6.49068627632189]]], [[[6.7756356479308764, 1.2421765985773896, 0.62472351247193547, 3.7932523556461843], [7.2533775224328529, 7.6658513496011462, 0.01420641773703224, 1.3294002730690275], [2.7488493036509345, -0.57197220586038711, 8.5742730832730025, 3.8905701667104253]], [[8.1067429783463023, 7.6573733295320867, 6.3802726111102794, 8.1657865551745985], [4.0055736396026882, 6.6007790483653279, -0.092301305875436945, 8.1628009941924908], [4.9317679670870298, 7.6403094742984656, 5.7858539087881979, 6.5508659918646588]]]])+(1.-msk_ref)*numpy.array([[[[6.1220310237783018, 1.6244627252805128, 0.8270299776943828, 6.1311355763024231], [2.1668245389293879, 5.2112500957777819, 8.5077520127643176, 2.4304596168177635], [7.4571365539993639, -0.520926281513022, 7.4732072633550803, 7.0193983678332952]], [[5.8392127530407851, 2.4957186740141131, -0.91540423836955576, 5.2892566669577779], [1.3691041239205854, 5.5953527855386129, 3.2846026513745397, 6.8221763701161828], [5.6366184542300726, 7.3773296124855268, 6.132857636128211, -0.77505896620498049]]], [[[1.9843650034050295, 8.5116633962924002, 6.3719360413434369, 7.965114479647962], [1.2810971370124822, 2.7999130343046428, 7.9670165926706087, 7.706789681403956], [2.5650796129761426, 8.4389507656969904, 8.2328891012376086, 6.3800324168699518]], [[3.2015464093763804, 6.0462093690664211, 8.4007591618325144, 4.2450318553561681], [6.3036074289446056, -0.029562914614911673, 8.7107769995972166, 2.8559408845913365], [2.8564700093155064, 3.5951286252492665, 6.5003172501095356, 6.8149355474630351]]], [[[7.0998849190720215, 1.5664258697185347, 0.94897278361308057, 4.1175016267873294], [7.577626793573998, 7.9901006207422913, 0.33845568887817734, 1.6536495442101726], [3.0730985747920796, -0.24772293471924201, 8.8985223544141476, 4.2148194378515704]], [[8.4309922494874474, 7.9816226006732318, 6.7045218822514245, 8.4900358263157436], [4.3298229107438333, 6.925028319506473, 0.23194796526570816, 8.4870502653336359], [5.2560172382281749, 7.9645587454396107, 6.110103179929343, 6.8751152630058039]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(-2.07689461008)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.97201154258677214, -1.6963580647408802, -0.84214550696182133, -1.9833943544838153], [4.2272686986455774, -4.045462544750551, 3.9428631910892733, -0.43484906501067222], [3.03270504256359, 0.70692372456239383, 1.545312580200827, -1.9481817826687364]], [[-0.20862835805177671, -2.3857838611048221, 3.203949261344043, -4.8660536695259236], [4.5334078495440462, -3.8327019616128655, -4.0692703335183653, 4.6392151236330683], [0.7610852270498043, 4.0536338277552577, 1.853217859186528, -1.7219491013460155]]], [[[-4.467898207071026, 3.3945415674616335, -2.6888488821618353, 1.5178062175673777], [3.5234453927307126, 1.51647537148766, -2.2768668113827237, 1.974437888294859], [4.8073937643995244, -1.4874821438533692, -0.45634183042491028, -2.3767137848106055]], [[-4.8018798445144029, 4.6357310949035391, -1.6029658399808264, -2.7494400796220795], [0.70384491619381251, -4.0032344870588696, 3.3079211678339355, -4.7059749917343225], [1.7829450693011095, 1.1276730331815221, 1.2624729629451181, 4.7455492884037529]]], [[[4.2056882218672129, -0.82316872553680298, -2.0001302464709125, -0.95925596208400421], [3.2645937015763398, 4.9562353299622544, 0.9038318953084632, -3.7869908429772248], [4.6557203544484889, -2.4924370596234136, 0.55363665483289815, 0.89493400599397965]], [[0.30232184836053033, -3.2853160238730128, 2.4651709593260742, -4.457485437944201], [4.787459253526233, -3.3880732884771514, 3.5687404270355589, 3.4651447531447737], [4.3433946888123618, 3.4450205494612245, 3.0417659634823373, 2.1528913679998949]]]])+(1.-msk_arg1)*numpy.array([[[[2.0759692187557208, -1.2684665643760029, -0.69298268334834567, -2.9881132000197796], [-0.40317665727650365, -1.8466459254467771, 0.96327725704196698, 2.9252788620556336], [1.9466794625458839, -4.9422288471297362, -2.5181436908674937, -2.7968289875502537]], [[3.8576155892979695, -4.5515390249682479, 3.4427168834073534, 1.7790636700740272], [-1.0170352614261411, -1.7751323130380268, 2.4679875345026243, -2.6788908909978937], [-4.0496935502002342, 0.74940706753586905, 0.73653201661187939, 0.25615289956514786]]], [[[2.4156630301587967, -1.1557801953106948, 0.860736640006313, 0.35246630980212146], [1.9170720879637315, 1.6253927060302225, 0.72493001481675101, -3.8349824987028445], [-0.019107092626987843, -2.5491350127557642, 0.20474515293419415, 4.2198418656931338]], [[0.05659512997617, 0.38817094877683278, -0.080198188909704982, -1.05122267538035], [1.3078843587918056, 0.69628507984258903, 1.6503581283978122, 4.0741233129300536], [-0.43870200424096595, -1.4112033326199258, 4.4469884998423641, -0.77069240450599619]]], [[[-3.7890665896042206, -0.52760077096252189, 4.5458786983200614, 1.5101772287532675], [1.5615222723460054, 4.5473118314156142, -4.079798376972585, -2.1749911271966682], [1.3838446927546588, 0.37200492312992672, -2.325703054321596, 4.9622825491330662]], [[-0.88537795962315791, -0.13118918645030142, -0.67281415815571766, 0.87276370744177179], [3.9319844257339582, 1.856517508369131, -4.9862581265657333, -2.3149148107991446], [2.5196109539893738, 1.7216699132252948, -3.9604079391893245, 2.9723058741752482]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.0489061526655954, -3.7732526748197035, -2.9190401170406446, -4.0602889645626385], [2.1503740885667542, -6.1223571548293743, 1.86596858101045, -2.5117436750894955], [0.95581043248476671, -1.3699708855164294, -0.5315820298779963, -4.0250763927475592]], [[-2.2855229681306, -4.4626784711836454, 1.1270546512652198, -6.9429482796047468], [2.4565132394652229, -5.9095965716916883, -6.1461649435971886, 2.5623205135542451], [-1.315809383029019, 1.9767392176764345, -0.22367675089229522, -3.7988437114248388]]], [[[-6.5447928171498493, 1.3176469573828102, -4.7657434922406585, -0.5590883925114456], [1.4465507826518893, -0.56041923859116327, -4.353761421461547, -0.10245672178396426], [2.7304991543207011, -3.5643767539321924, -2.5332364405037335, -4.4536083948894287]], [[-6.8787744545932261, 2.5588364848247158, -3.6798604500596497, -4.8263346897009027], [-1.3730496938850107, -6.0801290971376929, 1.2310265577551123, -6.7828696018131458], [-0.29394954077771374, -0.94922157689730113, -0.81442164713370513, 2.6686546783249296]]], [[[2.1287936117883897, -2.9000633356156262, -4.0770248565497358, -3.0361505721628275], [1.1876990914975165, 2.8793407198834311, -1.1730627147703601, -5.8638854530560476], [2.5788257443696656, -4.5693316697022368, -1.5232579552459251, -1.1819606040848436]], [[-1.7745727617182929, -5.3622106339518361, 0.38827634924725096, -6.5343800480230243], [2.7105646434474098, -5.4649678985559742, 1.4918458169567357, 1.3882501430659504], [2.2665000787335385, 1.3681259393824012, 0.96487135340351404, 0.075996757921071634]]]])+(1.-msk_ref)*numpy.array([[[[-0.00092539132310243133, -3.3453611744548262, -2.7698772934271689, -5.0650078100986029], [-2.4800712673553269, -3.9235405355256003, -1.1136173530368563, 0.84838425197681033], [-0.13021514753293939, -7.0191234572085595, -4.5950383009463174, -4.8737235976290769]], [[1.7807209792191463, -6.6284336350470712, 1.3658222733285301, -0.29783094000479604], [-3.0939298715049643, -3.85202692311685, 0.39109292442380106, -4.7557855010767174], [-6.1265881602790575, -1.3274875425429542, -1.3403625934669439, -1.8207417105136754]]], [[[0.33876842007997343, -3.232674805389518, -1.2161579700725103, -1.7244283002767018], [-0.15982252211509174, -0.4515019040486008, -1.3519645952620722, -5.9118771087816677], [-2.0960017027058111, -4.6260296228345874, -1.8721494571446291, 2.1429472556143105]], [[-2.0202994801026533, -1.6887236613019905, -2.1570927989885282, -3.1281172854591732], [-0.76901025128701761, -1.3806095302362342, -0.4265364816810111, 1.9972287028512303], [-2.5155966143197892, -3.4880979426987491, 2.3700938897635409, -2.8475870145848194]]], [[[-5.8659611996830439, -2.6044953810413451, 2.4689840882412382, -0.56671738132555571], [-0.51537233773281788, 2.470417221336791, -6.1566929870514082, -4.2518857372754919], [-0.69304991732416443, -1.7048896869488965, -4.4025976644004192, 2.885387939054243]], [[-2.9622725697019812, -2.2080837965291247, -2.7497087682345409, -1.2041309026370515], [1.855089815655135, -0.22037710170969227, -7.0631527366445566, -4.3918094208779674], [0.44271634391055059, -0.35522469685352842, -6.0373025492681478, 0.89541126409642491]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([-0.11615604807470259, 0.77617041391929931])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.297052202763747, 2.0899018942884426])+(1.-msk_arg1)*numpy.array([2.8155753444009015, 0.11356971064863863])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.41320825083844959, 2.8660723082077419])+(1.-msk_ref)*numpy.array([2.6994192963261989, 0.88974012456793794])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[-1.6624274366095904, -2.6213454739501585], [2.6487456410906081, -4.3959044947467349]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.0971878493427667, 1.2742200567746256])+(1.-msk_arg1)*numpy.array([0.045482077426734158, -1.2851117608893938])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.4347604127331763, 1.4758423753926082], [3.9229656978652336, -3.1216844379721094]])+(1.-msk_ref)*numpy.array([[-1.6169453591828562, -2.5758633965234243], [1.3636338802012142, -5.6810162556361288]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_expandedData_rank2(self):
      arg0=numpy.array([-2.1870540723043987, 1.5756813736345467])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.0266837433531757, -1.0021302983906812], [0.59527983386483818, 1.7826943177764933]])+(1.-msk_arg1)*numpy.array([[-2.2667171096214367, 0.49629966897668876], [-4.4710663061770282, -2.9901553965551075]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-6.2137378156575744, -3.18918437069508], [2.1709612074993849, 3.3583756914110401]])+(1.-msk_ref)*numpy.array([[-4.4537711819258359, -1.69075440332771], [-2.8953849325424814, -1.4144740229205608]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_expandedData_rank1(self):
      arg0=numpy.array([[[-0.12956725054919183, 4.8220614900062895, 1.5533139234803608, -2.2747254417197404, -0.33960001210992097], [2.1764799302513493, 2.8148085117902983, 2.9036667299574432, 2.3224838586437455, 2.8244896568416644], [0.29085825754997163, 3.5723054387154143, 1.4152036560376757, -3.2044270195127824, 2.3874341352901363], [0.050691153925111365, 2.6046706642557904, -2.6785145812543796, -0.63266637255317271, 2.6896917629070494]], [[-2.4611684785477941, 0.8651513535335944, 4.1865218874460943, 0.51940508548450115, -0.6154699257261198], [-4.8815631840642411, -1.7531770362342582, -0.54178122383865457, -4.1555928631733714, 4.813981433474062], [2.0944110609799003, -0.92745010906252823, 3.9706715045374192, -2.9813458716070054, 2.0144972177717477], [0.073413703621736381, 0.95844173369420105, -4.3489095413584655, -1.904559758398312, 1.5842933125051486]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.3947362639006009, 0.069870593557721961])+(1.-msk_arg1)*numpy.array([-3.2471652471356451, -4.5622038387101842])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.2651690133514091, 6.2167977539068904, 2.9480501873809617, -0.87998917781913955, 1.0551362517906799], [3.5712161941519502, 4.2095447756908992, 4.2984029938580441, 3.7172201225443464, 4.2192259207422653], [1.6855945214505725, 4.9670417026160152, 2.8099399199382766, -1.8096907556121815, 3.7821703991907372], [1.4454274178257123, 3.9994069281563913, -1.2837783173537787, 0.7620698913474282, 4.0844280268076503]], [[-2.3912978849900721, 0.93502194709131636, 4.2563924810038163, 0.58927567904222311, -0.54559933216839784], [-4.8116925905065191, -1.6833064426765363, -0.47191063028093261, -4.0857222696156494, 4.883852027031784], [2.1642816545376222, -0.85757951550480627, 4.0405420980951412, -2.9114752780492834, 2.0843678113294697], [0.14328429717945834, 1.028312327251923, -4.2790389478007436, -1.8346891648405901, 1.6541639060628706]]])+(1.-msk_ref)*numpy.array([[[-3.376732497684837, 1.5748962428706443, -1.6938513236552843, -5.521890688855386, -3.5867652592455661], [-1.0706853168842958, -0.4323567353453468, -0.3434985171782019, -0.92468138849189963, -0.42267559029398072], [-2.9563069895856735, 0.32514019157976914, -1.8319615910979694, -6.4515922666484276, -0.85973111184550888], [-3.1964740932105338, -0.64249458287985473, -5.9256798283900247, -3.8798316196888178, -0.55747348422859577]], [[-7.0233723172579783, -3.6970524851765898, -0.37568195126408988, -4.0427987532256831, -5.177673764436304], [-9.4437670227744253, -6.3153808749444424, -5.1039850625488388, -8.7177967018835556, 0.25177759476387784], [-2.4677927777302839, -5.4896539477727124, -0.59153233417276496, -7.5435497103171896, -2.5477066209384365], [-4.4887901350884478, -3.6037621050159832, -8.9111133800686488, -6.4667635971084962, -2.9779105262050356]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_expandedData_rank3(self):
      arg0=numpy.array([-4.4226850544175731, -3.5064184525599251])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.8998499538055, -4.1921130859415667, -4.4827107750770807, -1.6584600895237744, -4.1080931255923856], [-1.7778394572608027, 4.2746451568520705, -0.10904402948457825, 3.8121467771216491, 3.338064696034488], [1.2743126042635406, 3.9389068960014377, 2.0370606978774948, 2.3521083775801177, -2.5779920959100302], [2.2197178982604813, 1.112284698269451, 4.8387727338978657, 3.2163718280546263, 3.5178903083715163]], [[-1.3135188855508471, -2.662577405698098, 4.5177322170092964, -1.0262758558416083, -1.4438511780139773], [4.2054738674485765, 4.2861799189611194, -1.0126036058697974, 4.994471467131028, -4.0843142859000778], [-3.6160704612948225, -0.60514325922962797, -4.0103288842097058, -1.6864701526049011, 0.040786657534202853], [3.8540228645443371, 2.0904606259666032, -3.2797359797943937, -2.7388766428150682, -1.7062721544161832]]])+(1.-msk_arg1)*numpy.array([[[-4.335642555957774, 3.6769843704539635, -2.791655486804431, -2.3273208891535955, 4.1003591047415071], [-4.1618623343891841, 3.8417443041034396, -2.5907923504604593, -4.0626138220249199, 3.158883919231922], [2.8512028089729533, -2.8083399723664746, -1.0798260057758213, -1.850767950605805, 2.6146224045442619], [4.9815440083141045, 0.42448565305791242, -3.8067707873979009, 0.088576327646434194, -2.5673844302299975]], [[0.86457973483070027, 0.27618304646388925, -2.9434133124117801, 3.2035562390376349, -0.64123637191235971], [-4.2046180198692937, -1.7522843462901614, 2.2313301121841311, 1.6362672016694599, -1.5168237416281083], [-3.0836576915827427, -0.93015917411946525, 1.355650572000525, -3.6670513281671999, 4.3744468600844577], [-4.4727330110286267, -0.5511193638547196, 0.62386977462344362, -0.61966166038022052, -4.5160930557666381]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.5228351006120731, -8.6147981403591398, -8.9053958294946547, -6.0811451439413471, -8.5307781800099587], [-6.2005245116783758, -0.14803989756550262, -4.5317290839021513, -0.61053827729592403, -1.0846203583830851], [-3.1483724501540324, -0.48377815841613536, -2.3856243565400783, -2.0705766768374554, -7.0006771503276033], [-2.2029671561570918, -3.3104003561481221, 0.41608767948029257, -1.2063132263629468, -0.90479474604605681]], [[-4.8199373381107726, -6.1689958582580235, 1.0113137644493713, -4.532694308401533, -4.9502696305739029], [0.69905541488865142, 0.77976146640119426, -4.5190220584297229, 1.4880530145711028, -7.5907327384600034], [-7.1224889138547471, -4.1115617117895535, -7.5167473367696314, -5.1928886051648266, -3.4656317950257223], [0.34760441198441194, -1.4159578265933219, -6.7861544323543193, -6.2452950953749937, -5.2126906069761088]]])+(1.-msk_ref)*numpy.array([[[-8.7583276103753462, -0.74570068396360956, -7.2143405412220041, -6.7500059435711686, -0.32232594967606598], [-8.5845473888067581, -0.5809407503141335, -7.0134774048780324, -8.4852988764424921, -1.2638011351856511], [-1.5714822454446198, -7.2310250267840477, -5.5025110601933944, -6.2734530050233781, -1.8080626498733112], [0.5588589538965314, -3.9981994013596607, -8.229455841815474, -4.3341087267711389, -6.9900694846475702]], [[-2.6418387177292249, -3.2302354060960359, -6.4498317649717052, -0.30286221352229026, -4.1476548244722853], [-7.7110364724292193, -5.2587027988500861, -1.275088340375794, -1.8701512508904652, -5.0232421941880334], [-6.5900761441426674, -4.4365776266793908, -2.1507678805594002, -7.1734697807271246, 0.86802840752453259], [-7.9791514635885523, -4.0575378164146443, -2.8825486779364815, -4.1260801129401461, -8.0225115083265628]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_expandedData_rank1(self):
      arg0=numpy.array([[[[-0.61395482706517068, 1.6217058146312802], [-0.98768925287394893, 1.7202972401084482]], [[-1.3944225164516055, 2.1759871981421401], [-0.76269022815220033, -0.043658342468365419]], [[3.8216679022719866, -2.0755791954259264], [2.3062339310870614, 4.8749408274989126]], [[1.9233079106627446, 4.4276539292630428], [-2.0308657476121308, -1.8281902970332031]], [[2.978473926422244, -2.5882428295697335], [-4.9842932281211736, 1.7069480935841561]], [[1.2065152995931783, -3.0988325378170423], [-4.8490681749601503, 0.77240920709691441]]], [[[1.2804288160478698, 4.0850108403634184], [4.4343028036699668, 3.9202698792457049]], [[-0.018402887963874903, -1.4268843655333221], [2.154080323583047, -0.13370761798127884]], [[-2.4902339031082121, -1.0594862997182339], [-3.3258365817077316, -3.277490117385514]], [[-4.7894245132965807, -0.78381684885984271], [-1.4970587924394452, 4.0191412790604399]], [[1.2841564410383031, -3.3514333399563965], [4.3047411908019164, -0.72283662405227034]], [[-0.92427461814751943, 3.9565831887770422], [-0.73487054485643277, -2.1791180935417942]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.3282107067139699, 0.24085532457492143])+(1.-msk_arg1)*numpy.array([1.3589088719435694, 0.65772027741370032])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.9421655337791406, -1.7065048920826897], [-4.3158999595879184, -1.6079134666055217]], [[-4.722633223165575, -1.1522235085718298], [-4.0909009348661698, -3.3718690491823353]], [[0.4934571955580167, -5.4037899021398967], [-1.0219767756269085, 1.5467301207849427]], [[-1.4049027960512253, 1.0994432225490729], [-5.3590764543261002, -5.156401003747173]], [[-0.34973678029172595, -5.9164535362837034], [-8.3125039348351439, -1.6212626131298138]], [[-2.1216954071207916, -6.4270432445310117], [-8.1772788816741198, -2.5558014996170555]]], [[[1.5212841406227913, 4.3258661649383399], [4.6751581282448882, 4.1611252038206263]], [[0.22245243661104652, -1.1860290409584007], [2.3949356481579684, 0.10714770659364259]], [[-2.2493785785332907, -0.81863097514331251], [-3.0849812571328101, -3.0366347928105926]], [[-4.5485691887216593, -0.54296152428492128], [-1.2562034678645237, 4.2599966036353614]], [[1.5250117656132245, -3.110578015381475], [4.5455965153768378, -0.48198129947734891]], [[-0.68341929357259801, 4.1974385133519636], [-0.49401522028151135, -1.9382627689668728]]]])+(1.-msk_ref)*numpy.array([[[[0.74495404487839867, 2.9806146865748495], [0.37121961906962042, 3.0792061120520176]], [[-0.03551364450803618, 3.5348960700857095], [0.59621864379136902, 1.3152505294752039]], [[5.180576774215556, -0.71667032348235704], [3.6651428030306308, 6.2338496994424819]], [[3.282216782606314, 5.7865628012066121], [-0.67195687566856144, -0.46928142508963377]], [[4.3373827983658133, -1.2293339576261642], [-3.6253843561776042, 3.0658569655277255]], [[2.5654241715367476, -1.7399236658734729], [-3.4901593030165809, 2.1313180790404838]]], [[[1.9381490934615702, 4.7427311177771188], [5.0920230810836671, 4.5779901566594052]], [[0.63931738944982541, -0.76916408811962178], [2.8118006009967473, 0.52401265943242148]], [[-1.8325136256945118, -0.40176602230453362], [-2.6681163042940312, -2.6197698399718137]], [[-4.1317042358828804, -0.12609657144614239], [-0.83933851502574486, 4.6768615564741403]], [[1.9418767184520034, -2.6937130625426962], [4.9624614682156167, -0.065116346638570022]], [[-0.26655434073381912, 4.6143034661907425], [-0.077150267442732456, -1.5213978161280939]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_expandedData_rank4(self):
      arg0=numpy.array([-4.7158916126049135, 1.1252678639050506])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.7274963705339532, 4.7195633043632661], [4.73636524920982, -4.2391422562907746]], [[2.2689799648543953, -1.8498889275597827], [-3.7174995253416574, 1.8182220241563645]], [[2.3729579806579384, 2.6548600855283224], [-0.31070672475390548, 2.4531898171013804]], [[-0.019277816095603306, -3.335326535581209], [-4.778079199572673, 2.6064587798734955]], [[4.3931398304307692, 4.5975423150528911], [1.3810426240468887, -3.6870195694842902]], [[-3.5339098365574326, 3.840639969839545], [2.4922510906576054, 3.9718123671363283]]], [[[2.9844723758716167, -2.5974589891293931], [-0.016968300761742938, 2.6548491821217146]], [[-3.8822173097798984, -4.1178336945158032], [-1.7826449442204959, 4.4253050774195462]], [[-2.0742742751294649, -0.17140888514521979], [3.7289107649790711, 4.8396520332969395]], [[-1.1082838234780357, -0.53804795767446922], [2.9887013495321044, -4.3028306070505948]], [[1.3946457009979785, -4.012482646920537], [1.4076070453983922, 4.4670906262086501]], [[-0.98910961040326928, 4.7527311510752579], [1.0688645243888297, 1.6956676270128508]]]])+(1.-msk_arg1)*numpy.array([[[[0.12261135318004435, 4.3912906706421353], [-0.31763235241825249, -1.8369053129255741]], [[1.4123502579588809, 0.72994979519038683], [1.5980741012660085, -2.8904114306756332]], [[-3.8626623190511831, -4.8439762106943771], [3.4727221036725222, 4.8813000729962699]], [[2.89652525155081, -4.5612879267069664], [-4.5388938844085667, 0.80054950869428154]], [[1.7751480918335076, -2.9979276630818044], [-2.6914681248973595, 0.99617058549010729]], [[-3.5051871944328541, -0.30447921886317619], [4.4753598558532843, -3.6606018515864989]]], [[[4.1345152361382187, 1.7992268444762969], [-2.2240143347715433, 2.9395629941148922]], [[0.32717323111329222, 4.039437246879622], [4.0993102564645874, -1.5283447277259068]], [[4.1939950274354096, 3.3952133310820241], [-4.5348215131386596, 2.0285245550389464]], [[1.9981862149738649, -3.5988093525739173], [-3.2161933870485546, 4.1406452650205221]], [[3.4331428921203173, 4.2566360379383905], [-4.6863798126895482, 0.67581930068134533]], [[-4.282674087030113, -0.22557811227906921], [4.3721648940770805, 1.3468794042673151]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-9.4433879831388658, 0.0036716917583525799], [0.020473636604906531, -8.9550338688956881]], [[-2.4469116477505182, -6.5657805401646963], [-8.4333911379465718, -2.897669588448549]], [[-2.3429336319469751, -2.0610315270765911], [-5.026598337358819, -2.2627017955035331]], [[-4.7351694287005168, -8.0512181481861234], [-9.4939708121775865, -2.109432832731418]], [[-0.32275178217414435, -0.11834929755202239], [-3.3348489885580248, -8.4029111820892037]], [[-8.2498014491623461, -0.87525164276536849], [-2.2236405219473081, -0.74407924546858517]]], [[[4.1097402397766674, -1.4721911252243425], [1.1082995631433077, 3.7801170460267652]], [[-2.7569494458748478, -2.9925658306107525], [-0.65737708031544528, 5.5505729413245968]], [[-0.94900641122441431, 0.95385897875983083], [4.8541786288841218, 5.9649198972019901]], [[0.016984040427014957, 0.5872199062305814], [4.113969213437155, -3.1775627431455442]], [[2.5199135649030291, -2.8872147830154864], [2.5328749093034428, 5.5923584901137007]], [[0.13615825350178135, 5.8779990149803085], [2.1941323882938804, 2.8209354909179014]]]])+(1.-msk_ref)*numpy.array([[[[-4.5932802594248692, -0.32460094196277822], [-5.033523965023166, -6.5527969255304876]], [[-3.3035413546460326, -3.9859418174145267], [-3.1178175113389051, -7.6063030432805467]], [[-8.5785539316560957, -9.5598678232992906], [-1.2431695089323913, 0.16540846039135637]], [[-1.8193663610541035, -9.2771795393118808], [-9.2547854970134793, -3.915342103910632]], [[-2.9407435207714059, -7.7138192756867179], [-7.4073597375022731, -3.7197210271148062]], [[-8.2210788070377667, -5.0203708314680897], [-0.24053175675162919, -8.3764934641914124]]], [[[5.2597831000432693, 2.9244947083813475], [-1.0987464708664927, 4.0648308580199428]], [[1.4524410950183428, 5.1647051107846726], [5.224578120369638, -0.40307686382085617]], [[5.3192628913404603, 4.5204811949870747], [-3.4095536492336089, 3.153792418943997]], [[3.1234540788789156, -2.4735414886688667], [-2.090925523143504, 5.2659131289255727]], [[4.5584107560253679, 5.3819039018434411], [-3.5611119487844975, 1.801087164586396]], [[-3.1574062231250624, 0.89968975162598142], [5.4974327579821312, 2.4721472681723657]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[0.87504978232714237, 0.88669539546189302, 0.91223224259084024, -3.4228713114199163, -2.3542694678956488], [-0.00054770116750813003, -4.0190695516370631, -0.014541295070815252, 1.2975243441836328, 2.0506677925839076], [-2.950451983682898, 3.6099637198446626, 0.30226385140547052, -2.4257717735844233, 4.7787139193649804], [3.8686212758143093, 3.8429232721006308, 3.3029149378634362, 1.5870277585988655, -1.6390934242986068]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.7174736991800863, 3.6894371409646354, -0.65036676288996542, -2.3538612822292704, 4.4847779796255267], [-4.1048938787948073, -1.5394488350545132, 1.9692182996981327, 4.9972961088113621, 1.7193201365444892], [1.4729803246785345, -1.3126423193605619, -1.2256012868732014, 0.94610593586021885, -2.2601621858768004], [-1.1243050480798154, -4.9200929026679683, -3.0521328163359396, 3.416567041656684, -1.0142435303032062]])+(1.-msk_arg1)*numpy.array([[-2.6750184432634172, 1.3520726793432125, 2.7142662758307354, 1.8064621447800144, -4.715757439146552], [-4.0313988795919853, 3.2268302228983288, 2.3761731251683651, -4.4872337797959059, -2.5279045391425372], [0.5026012425561035, 3.7323416419336883, 3.5888730223768093, -1.3497248049030288, -3.9398083729729114], [2.8924192989916033, 2.8180793726689179, 3.3843766945184921, 0.5253792231270733, 2.5595722812396771]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.84242391685294393, 4.5761325364265284, 0.26186547970087481, -5.7767325936491867, 2.1305085117298779], [-4.1054415799623154, -5.5585183866915759, 1.9546770046273174, 6.2948204529949949, 3.7699879291283969], [-1.4774716590043635, 2.2973214004841007, -0.92333743546773084, -1.4796658377242045, 2.51855173348818], [2.7443162277344939, -1.0771696305673375, 0.25078212152749657, 5.0035948002555495, -2.6533369546018131]])+(1.-msk_ref)*numpy.array([[-1.7999686609362748, 2.2387680748051055, 3.6264985184215757, -1.6164091666399019, -7.0700269070422008], [-4.0319465807594934, -0.79223932873873437, 2.3616318300975498, -3.189709435612273, -0.47723674655862958], [-2.4478507411267945, 7.3423053617783509, 3.8911368737822798, -3.7754965784874521, 0.83890554639206893], [6.7610405748059126, 6.6610026447695487, 6.6872916323819283, 2.1124069817259388, 0.92047885694107023]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_expandedData_rank2(self):
      arg0=numpy.array([[[3.6329384634138968, -3.3116577613389464], [1.824043881346519, 0.1670046052362455], [-3.3547990995336576, -0.61316502878188128], [4.6021870248727907, -3.2971181142518571], [-4.404290616021723, -2.9649772090253768]], [[-0.43147100020192752, -4.2166648550661048], [4.6425352097469226, -4.2189649903353166], [-1.82822582018325, 3.0113979986274266], [-2.2442725786109818, -4.099924296473608], [3.4277946398794832, 1.8893320149769046]], [[-3.7571484676957736, 0.97293544330055148], [-0.057696555165098751, 2.9018594695128686], [4.5935570123365075, -3.4247522677846867], [-1.6239005116827068, -4.5417489508361664], [1.743630612848202, 0.10287304653918117]], [[-4.2077768949986538, -0.51176886563246615], [-2.8715796108353131, 3.0687736284760323], [-4.1455028490254442, 2.2257246114318043], [4.1347259842182815, -4.1887655646234894], [-0.8874247966387987, -2.6033330007295619]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.2248410936901024, 1.2167329409248939, -2.7591148751818464, -3.4719323741204757, -2.9079523754963308], [0.17920267478073448, -4.4324287473273589, 0.80490661124281537, -3.3222680807171701, -3.4717897361981107], [-3.293695872328295, -3.5084698395773852, 0.56460092474128754, 2.715540114473459, 0.99151100165985007], [2.4818756478462127, 2.1224371442921255, -1.1291801644025368, 4.3857173560650828, -2.3793180914333978]])+(1.-msk_arg1)*numpy.array([[-2.9897673226800925, 1.4483319149277385, 0.67301174575337708, -0.95764297872868465, 3.2023518036195728], [-2.7329967131337121, 0.37678896014769769, 0.7330676918376513, 0.8570716918039869, 1.8556114630283105], [-0.31730000787958179, 4.0758813691418982, -0.25202166614935706, -3.0217656184455457, 2.6570413965820947], [1.8621245024606035, 1.2943222909121976, 4.1014142866589562, -2.3529918121555271, 0.92191191369442205]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.8577795571039992, -0.086816667648843993], [3.0407768222714129, 1.3837375461611394], [-6.113913974715504, -3.3722799039637277], [1.130254650752315, -6.7690504883723328], [-7.3122429915180538, -5.8729295845217075]], [[-0.25226832542119304, -4.0374621802853703], [0.21010646241956366, -8.6513937376626764], [-1.0233192089404346, 3.816304609870242], [-5.5665406593281519, -7.4221923771907781], [-0.043995096318627525, -1.5824577212212061]], [[-7.050844340024069, -2.3207604290277435], [-3.566166394742484, -0.60661037006451668], [5.158157937077795, -2.8601513430433991], [1.0916396027907522, -1.8262088363627074], [2.735141614508052, 1.0943840481990312]], [[-1.7259012471524411, 1.9701067822137466], [-0.74914246654318761, 5.1912107727681578], [-5.274683013427981, 1.0965444470292676], [8.5204433402833644, 0.19695179144159347], [-3.2667428880721965, -4.9826510921629596]]])+(1.-msk_ref)*numpy.array([[[0.64317114073380433, -6.3014250840190389], [3.2723757962742575, 1.615336520163984], [-2.6817873537802805, 0.059846716971495795], [3.6445440461441061, -4.2547610929805417], [-1.2019388124021502, 0.23737459459419608]], [[-3.1644677133356396, -6.9496615681998168], [5.0193241698946203, -3.8421760301876189], [-1.0951581283455987, 3.7444656904650779], [-1.3872008868069949, -3.2428526046696211], [5.2834061029077937, 3.7449434780052151]], [[-4.0744484755753554, 0.65563543542096969], [4.0181848139767995, 6.9777408386547668], [4.3415353461871504, -3.6767739339340437], [-4.6456661301282525, -7.5635145692817121], [4.4006720094302967, 2.7599144431212759]], [[-2.3456523925380504, 1.3503556368281373], [-1.5772573199231155, 4.3630959193882299], [-0.044088562366487949, 6.3271388980907606], [1.7817341720627544, -6.5417573767790165], [0.034487117055623351, -1.6814210870351398]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_expandedData_rank3(self):
      arg0=numpy.array([[4.9488656891814333, -0.21560335799239549, 4.1970483915199885, 1.1689651306408608, -0.33325944612937874], [-3.6718566393597598, 1.0541577214006583, -1.3244939734065477, -4.5023379369180763, 4.0895764158042009], [-4.809877277214289, 3.2879091997740915, -2.847000330160252, -2.4317930688382559, 4.9415310678726687], [3.6920266231808814, 1.3746212743288471, -2.7341120477959659, -1.7660101299364914, 0.90000283409352821]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.6370352285612082, 0.70296882323477128], [-0.80063401119203981, -1.7781152927759889], [1.8836423816438455, -3.7929123185482903], [-1.5370026860893726, -1.6505612385770974], [2.1318062100587962, 0.89248921935175307]], [[-4.4700074617663805, -4.4779081291849661], [-0.34613031275107708, 4.0182796233941502], [1.3192355129185209, -0.80107655326860439], [-1.860915291387788, 4.6434271454169913], [-3.1635378620107502, 1.4299684162620867]], [[4.0094702929470429, 0.21039031006335751], [1.0596294119644671, 3.4535768065737358], [0.93351951476863171, -2.6959260008999362], [1.0312358879795367, 3.8787507961903813], [4.6712663739907825, 2.7994173764901173]], [[-2.3902521812264532, 2.4315332972102519], [4.1351973731330247, 2.4769389782898985], [4.0883308296236098, -2.6598447007153405], [1.9694505217054701, -2.7396998798218872], [3.5740205331134263, -1.4551101506288955]]])+(1.-msk_arg1)*numpy.array([[[0.20957961584079854, 0.33518007563904462], [-3.3011635197287048, -1.073376894106354], [2.7625090990938519, -2.4438483595412199], [-4.9442257568995629, -0.65283148979895778], [-2.3645965997580132, -3.141767966943827]], [[4.2551966231953298, -4.3264679524665883], [3.6358560437015655, -2.583344335770088], [1.3790555721506026, -0.19730146001942295], [-2.201200846111675, 3.4267095584034983], [-3.9066565396581243, 2.1643571558346046]], [[4.5958723939625621, 3.6788670064817648], [2.0083914802161864, 1.4995275721724601], [-2.3759576388348034, 0.14885043994241176], [3.9768433528488512, -0.9316154969064776], [1.5235300684250515, 3.4642870658304972]], [[-3.713098752044842, -3.3769668470880019], [2.0453298276769729, -1.0577813104880409], [3.2567093687510855, -0.23649127928966696], [2.2722983033791238, 4.2004786455644467], [1.5271391186250458, -0.64909200627976382]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.3118304606202251, 5.6518345124162046], [-1.0162373691844353, -1.9937186507683844], [6.080690773163834, 0.40413607297169829], [-0.3680375554485118, -0.48159610793623653], [1.7985467639294175, 0.55922977322237433]], [[-8.1418641011261403, -8.1497647685447259], [0.70802740864958125, 5.0724373447948086], [-0.005258460488026806, -2.1255705266751521], [-6.3632532283058643, 0.14108920849891504], [0.92603855379345079, 5.5195448320662877]], [[-0.8004069842672461, -4.5994869671509315], [4.3475386117385586, 6.7414860063478272], [-1.9134808153916203, -5.5429263310601886], [-1.4005571808587192, 1.4469577273521255], [9.6127974418634512, 7.7409484443627861]], [[1.3017744419544282, 6.1235599203911333], [5.5098186474618718, 3.8515602526187456], [1.3542187818276439, -5.3939567485113065], [0.20344039176897866, -4.5057100097583787], [4.4740233672069545, -0.55510731653536727]]])+(1.-msk_ref)*numpy.array([[[5.1584453050222319, 5.2840457648204779], [-3.5167668777211003, -1.2889802520987494], [6.9595574906138404, 1.7532000319787686], [-3.7752606262587021, 0.51613364084190305], [-2.6978560458873919, -3.4750274130732057]], [[0.58333998383556995, -7.9983245918263481], [4.6900137651022238, -1.5291866143694297], [0.054561598744054862, -1.5217954334259707], [-6.7035387830297513, -1.0756283785145779], [0.18291987614607663, 6.2539335716388056]], [[-0.21400488325172695, -1.1310102707325242], [5.2963006799902779, 4.7874367719465516], [-5.2229579689950558, -2.6981498902178402], [1.5450502840105953, -3.3634085657447335], [6.4650611362977202, 8.4058181337031659]], [[-0.021072128863960593, 0.3150597760928795], [3.41995110200582, 0.31683996384080615], [0.52259732095511957, -2.9706033270856329], [0.50628817344263233, 2.4344685156279553], [2.427141952718574, 0.25091082781376439]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[2.3162108977577436, 1.730480826888309, 3.2280203996501378, -3.8749989019493922, 0.3106050803635485], [0.42650735086678626, 0.732726485030323, 0.81456570000273754, -4.9692173516510527, 0.15309195748151083], [-4.2124661446708576, -0.57665622287890805, 1.122655982491545, 1.7061129209978105, 3.559645317119891], [2.9296700901289334, 2.4518255312983523, -4.084235914426654, -4.9902423198639489, 3.6172335863143417]], [[-2.5296100373837946, -2.368617688913881, -3.0362436399217563, 0.26253356677376605, 3.6538193361575484], [-2.4474063052551207, 0.83258317998367204, -1.2095608343823483, -3.1291130344495697, -3.0944818859214687], [-0.94481198126626076, -2.3466510872042301, -3.3190456154622905, 0.37223688242552644, -1.9436358270074305], [-2.1541595633099431, 4.4017236742029482, 0.075364602960419802, 2.1513576178462079, -0.64017532054967319]], [[-4.9903126925032826, -3.9481012328124496, 1.9712100160162693, -4.6556085772504829, -4.6614041071541656], [-0.15655788634387591, 3.2457261675680336, 4.1415674934115358, 0.47826882987288677, -1.1395064262430732], [4.298100839076362, -2.0597556647773718, -4.7291513347114602, -3.1888598327993343, -3.1672604055534235], [0.43424008293700034, -0.99622742275366427, -4.3103421386671652, -3.6840620800256527, 3.0194024178730015]], [[-2.6924872516748231, 1.8592211966303562, -3.7622345869169771, 4.9229386889697828, 3.4819449030352878], [3.3027036987339962, 4.0066230320866687, -3.0096541719143568, 0.37641435396412515, -0.77915924988534258], [-3.1137848508611521, 0.55014053021884735, 2.2021782337345783, -1.7449375821897051, 1.2600542772914283], [-0.62681898392224866, -1.7591906859666393, -3.3359542710630006, -1.2100329463911299, -0.21248431029430215]], [[2.700357349535798, 0.77769848977013911, 3.5741099781918528, -2.0308652384664772, -2.9580726841927918], [-0.4092080867036465, -0.52858394950601539, -4.8366235892910812, 3.3045035907636624, -0.63124651454396918], [1.8750083804869613, 3.9959595414958482, -0.40545368604103338, -4.9978663744863248, 0.49606317148573531], [1.9302476583363193, -1.1721958301133215, -1.5690598461767093, -4.3223642784665062, -2.6183835891758278]]], [[[0.44515349375502744, 4.7575640578592822, 4.0001998097793905, 3.7209682689306707, 2.4915038241333987], [-4.8952715156287532, 4.9314747189035266, 2.7354826565505919, 3.5684788341425744, 1.5845211096466123], [-3.629495993951557, -3.9281343493568932, -0.94609142775629529, 3.1440872304386787, -1.1075077572639436], [4.90993898849338, -0.84249422382930916, 0.81209684015318295, -2.46993482733909, 4.3967907873281717]], [[1.8837967441881158, 1.0455267853060111, -1.2309036255965209, 2.3764937940339745, 2.1880870910274304], [3.3539955686873633, 4.1472259010252728, -3.3069443789302322, 0.26596486846630274, -3.3929758312643656], [0.96886490307606543, 2.7314024459479302, -4.3545471603129862, 0.81117212304525665, -4.6612473930416787], [-1.2988959021970081, -0.0092050940258854297, -2.135989769158706, 0.23266738092716377, -0.050282894601397565]], [[-4.4533800799350285, 2.2098204539941975, 1.4397502579102097, -0.43603970279459325, 4.0729892504318439], [-0.91360808920167091, -2.0112552319780508, 1.6963288495673678, 1.5642961310410799, 4.906926059821707], [-2.529277634855239, 1.8623327999787058, -2.8798970260554513, 1.176238968978021, -1.8325001037539099], [4.4856765863715893, 0.066002614197905096, -2.4802301152306128, 3.1189830892694239, 3.9886018092888627]], [[1.6919998112882171, -3.5080177454715722, -0.79214134594954544, 0.77167238248863956, -4.5429527449108464], [-3.9660721289357381, -2.41596341874014, -0.5431068857943, 2.6265854214337461, 1.5525011873305283], [-4.5167543102300343, 4.5761539088205669, 2.5980684304851245, 1.5076742120661946, -1.6761354854966006], [0.21180649169755306, -4.1410072939319189, 0.41453860453295732, -0.34270872825566023, -0.71122889214294016]], [[1.5530749508200259, -4.5138044129945865, -2.0648755589817669, 1.9078859474727068, 1.6555633028422942], [2.9583478209514613, 4.7641545928484224, 4.2246266274324977, 1.7540040669370685, 3.2014647438800594], [-0.55767686173776987, -1.250182798269166, -2.2947283772902671, -4.0578107124459297, -3.9826200049749705], [3.6113558064563307, -0.27966811805294789, 3.5132777325323055, -0.25757417456401832, -0.48021429268338345]]], [[[0.012753917394745073, 4.6005259906318621, 0.31514980316814523, -4.0707983498642353, 0.44854039165962423], [-1.9409404300241819, -1.6289446247952579, 1.6449130372347787, -0.56118709526788102, -1.4746129183849299], [2.5889198520382486, -2.4194517228474766, 2.3345970212587428, -3.494480943295788, 3.4884689287727824], [-4.2293232680029389, -4.7961662350206993, 0.13886534569144438, 2.2804546123503391, -1.3114191753514213]], [[3.5161254529405053, 3.3701194516938422, -0.46371507016739866, 4.5475787185070367, -4.213613770942942], [0.44789268591656128, 1.8642073393112852, -1.4210511436192008, -3.703176222056288, 2.6160332365684642], [-1.9479170817800471, -2.9392531457977356, -1.2595199262738985, -3.0471755117499253, -2.2605264489267318], [-2.9898529345645319, -3.5461585677274932, -1.4456541671699408, -3.3317292520590014, -0.38208749913247431]], [[0.39498573177214702, 1.2705780959885775, -4.2225908818030744, -1.1354922556378213, 4.6958128170162343], [1.8207496323800552, 3.8550013136363201, 0.014859055263415577, 1.355074362854233, -0.12294117102859126], [-2.9373130056827335, -2.3591266419916179, -3.8824679954522221, 4.8395130480161832, -2.538526081705065], [4.9275171444897055, -4.0148379939622538, 2.200773400954203, 1.5935365518975155, -4.5819129260651827]], [[0.65164193770634071, 4.6270131105231052, -2.3678399668771477, 3.5353204234791935, 4.1603986163898856], [-1.5222497099147345, -4.6947566272503831, 2.1840230313803897, 0.48204440729491793, 3.5322739043689335], [-0.066103087636641966, 2.5626159823230861, 0.32828479782656927, -4.0532840061790107, -3.2378278867630863], [1.9764456686453444, 3.8741179315965866, 1.5242322993744297, -0.65591972833239343, -1.045463899652729]], [[-3.858756655975184, -2.4586826040015506, -4.4793816680646774, 1.3976005752575311, -3.4006171608981797], [0.76499499866913112, -1.9767466298146861, 2.3476952867354006, 2.4952778334239367, -1.959287673435556], [-4.0683968921893845, 4.7783549036380304, 3.1650891337316125, -3.3860620446097567, -2.656657966491843], [-4.280907361215859, -2.3292159839853865, -1.9921347212102472, 4.8893970756031155, 0.58545340192941442]]], [[[0.22520391889970259, -2.0411181990003358, 0.54962377861671996, -2.9766377076127917, 0.60204205953490231], [1.8608709119560585, -3.0307739502561093, 3.4865202503695425, 3.9540357208864254, 3.9735425453368869], [-3.3060571020972942, 4.0337909005843962, -1.7786506480680044, 3.1348928854361571, 4.5462257387280438], [-3.6773870674117335, 1.6496627392882406, 4.0391239197229503, 1.4304030946111688, -2.4075170349722939]], [[-3.2224865855128204, -0.99064981585553369, -4.118591198842708, -0.45683564697208912, 2.4364656779519853], [1.4581606244857408, 4.1059806596927277, -3.6131553850515186, 0.72555570514437395, 0.26138551940355548], [3.1712441510460447, -4.3494786947593935, -0.662037567344953, -2.7256466841260272, -1.5427387566054263], [0.040099118413106538, -1.1399736486736023, 4.5076079917455267, 3.7907960468164017, 0.25760496172686409]], [[1.6858129169420355, 0.077851605923760836, -0.88647682767117608, -4.4908114143373359, -1.7311885451142937], [-3.3903137203879377, -0.5236135165073863, -1.5728785369561482, -4.7768689023855178, 4.9242362844353273], [0.86472577080595059, -4.8955409337810423, -3.4034039379158596, -4.290207282375075, 4.798970581642287], [0.11688665693227307, -0.76240792154516424, 3.2136369378679088, 1.7290873011212762, -3.4877966585667819]], [[2.4327237028412796, 2.3591037846859937, -1.1705502899488374, -3.407497015153691, -1.263522286029878], [-1.6536884533743712, -1.6237128719326277, -3.6872282357476194, 1.8111316523349448, -3.7126511795279473], [2.8047791763601193, -3.4045357568569914, -4.7908919976889326, 2.9571377526023817, 1.0917334429592671], [-0.051318129645185451, -0.75027469932896729, -1.9867418295009429, 1.8844559886167271, 2.4301752181520007]], [[-0.52252451788804866, -2.6719603594042285, -3.6891439058764322, 2.7476056701124527, 4.2083274705389719], [-0.18796722835246094, 1.0387811837838186, -3.1040129148292825, -4.3994830931797306, 3.9666133028324175], [-4.4570989798610583, -4.6564698829524618, -4.5942094245700718, -3.3630180426275889, 2.1982228054564761], [3.1576113917934805, -1.7015343800562146, 2.6308282974866959, -4.3021393767749316, 4.7872291291289049]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.17513519875471495, 4.6695054745031133, 4.187671951402006, 4.9725106840528053, 1.9160706683262951], [0.3263629889569426, 0.36325791099229754, -0.087311489662593722, -3.5433879210942774, -3.7937260765986283], [2.5899132527224742, -1.9696798064103325, 1.9909261605440021, -0.017337565930107601, -0.71663462951397072], [0.36605049648665844, 3.0963931741949065, 4.7130542162812503, 3.2212531873184727, 4.7995891992839574]])+(1.-msk_arg1)*numpy.array([[0.35334168169576685, 4.1020757069632552, 3.5408655461201874, 1.5320555858755647, -4.7269253100511799], [3.8525067012626497, -0.034350841940103294, -2.2428500897249783, -1.8394782146189401, 4.4107566578819899], [1.2823776982231641, 0.6952411093443196, 1.9630336377590538, 4.0486715966068321, -4.5915788681209335], [-0.46212390399173131, -2.9342369541963178, 0.52676371944356948, -4.1061610521743654, -3.1929627727350729]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.4913460965124585, 1.9056160256430239, 3.4031555984048527, -3.6998637031946773, 0.48574027911826345], [0.60164254962150121, 0.90786168378503795, 0.98970089875745249, -4.7940821528963378, 0.32822715623622578], [-4.0373309459161426, -0.4015210241241931, 1.2977911812462599, 1.8812481197525255, 3.7347805158746059], [3.1048052888836484, 2.6269607300530673, -3.9091007156719391, -4.815107121109234, 3.7923687850690566]], [[2.1398954371193186, 2.3008877855892322, 1.633261834581357, 4.9320390412768793, 8.3233248106606617], [2.2220991692479926, 5.5020886544867853, 3.459944640120765, 1.5403924400535436, 1.5750235885816446], [3.7246934932368525, 2.3228543872988832, 1.3504598590408228, 5.0417423569286397, 2.7258696474956827], [2.5153459111931702, 9.0712291487060615, 4.7448700774635331, 6.8208630923493212, 4.0293301539534401]], [[-0.80264074110127659, 0.23957071858955636, 6.1588819674182753, -0.46793662584847695, -0.47373215575215966], [4.0311140650581301, 7.4333981189700395, 8.3292394448135418, 4.6659407812748928, 3.0481655251589328], [8.485772790478368, 2.1279162866246342, -0.54147938330945422, 0.99881211860267172, 1.0204115458485825], [4.6219120343390063, 3.1914445286483417, -0.12267018726515921, 0.50360987137635327, 7.2070743692750074]], [[2.2800234323779822, 6.8317318806831615, 1.2102760971358282, 9.8954493730225881, 8.4544555870880931], [8.2752143827868014, 8.979133716139474, 1.9628565121384485, 5.3489250380169304, 4.1933514341674627], [1.8587258331916532, 5.5226512142716526, 7.1746889177873836, 3.2275731018631002, 6.2325649613442335], [4.3456917001305566, 3.213319998086166, 1.6365564129898047, 3.7624777376616754, 4.7600263737585031]], [[4.6164280178620931, 2.6937691580964342, 5.490180646518148, -0.11479457014018202, -1.0420020158664967], [1.5068625816226486, 1.3874867188202797, -2.920552920964786, 5.2205742590899575, 1.284824153782326], [3.7910790488132564, 5.9120302098221433, 1.5106169822852618, -3.0817957061600296, 2.4121338398120304], [3.8463183266626144, 0.74387483821297362, 0.34701082214958578, -2.406293610140211, -0.70231292084953267]]], [[[0.77151648271197004, 5.0839270468162248, 4.3265627987363331, 4.0473312578876133, 2.8178668130903413], [-4.5689085266718106, 5.2578377078604692, 3.0618456455075345, 3.894841823099517, 1.9108840986035549], [-3.3031330049946144, -3.6017713603999506, -0.61972843879935269, 3.4704502193956213, -0.78114476830700097], [5.2363019774503226, -0.51613123487236656, 1.1384598291101256, -2.1435718383821474, 4.7231537762851143]], [[2.2470546551804134, 1.4087846962983086, -0.86764571460422335, 2.7397517050262721, 2.5513450020197279], [3.7172534796796608, 4.5104838120175703, -2.9436864679379346, 0.62922277945860028, -3.0297179202720681], [1.332122814068363, 3.0946603569402278, -3.9912892493206886, 1.1744300340375542, -4.2979894820493811], [-0.93563799120471058, 0.35405281696641211, -1.7727318581664084, 0.59592529191946131, 0.31297501639089997]], [[-4.5406915695976222, 2.1225089643316037, 1.352438768247616, -0.52335119245718698, 3.9856777607692502], [-1.0009195788642646, -2.0985667216406445, 1.6090173599047741, 1.4769846413784862, 4.8196145701591133], [-2.6165891245178328, 1.7750213103161121, -2.967208515718045, 1.0889274793154273, -1.9198115934165036], [4.3983650967089956, -0.021308875464688626, -2.5675416048932065, 3.0316715996068302, 3.9012903196262689]], [[-1.8513881098060603, -7.0514056665658496, -4.3355292670438228, -2.7717155386056378, -8.0863406660051247], [-7.5094600500300155, -5.9593513398344173, -4.0864948068885774, -0.91680249966053129, -1.990886733763749], [-8.0601422313243116, 1.0327659877262896, -0.94531949060915288, -2.0357137090280828, -5.219523406590878], [-3.3315814293967243, -7.6843952150261963, -3.12884931656132, -3.8860966493499376, -4.2546168132372175]], [[-2.2406511257786024, -8.3075304895932156, -5.8586016355803956, -1.8858401291259215, -2.138162773756334], [-0.83537825564716695, 0.97042851624979409, 0.43090055083386947, -2.0397220096615598, -0.59226133271856884], [-4.3514029383363981, -5.0439088748677943, -6.0884544538888949, -7.851536789044558, -7.7763460815735987], [-0.18237027014229756, -4.0733941946515762, -0.2804483440663228, -4.0513002511626466, -4.2739403692820117]]], [[[2.6026671701172193, 7.1904392433543363, 2.9050630558906194, -1.4808850971417611, 3.0384536443820984], [0.64897282269829226, 0.96096862792721627, 4.2348262899572529, 2.0287261574545932, 1.1153003343375443], [5.1788331047607228, 0.17046152987499763, 4.924510273981217, -0.90456769057331377, 6.0783821814952566], [-1.6394100152804647, -2.2062529822982251, 2.7287785984139186, 4.8703678650728133, 1.2784940773710529]], [[1.5464456465301728, 1.4004396452835097, -2.4333948765777311, 2.5778989120967042, -6.1832935773532745], [-1.5217871204937712, -0.10547246709904723, -3.3907309500295333, -5.6728560284666205, 0.64635343015813174], [-3.9175968881903795, -4.908932952208068, -3.229199732684231, -5.0168553181602578, -4.2302062553370643], [-4.9595327409748649, -5.5158383741378252, -3.4153339735802732, -5.3014090584693339, -2.3517673055428068]], [[2.3859118923161491, 3.2615042565325796, -2.2316647212590723, 0.85543390490618076, 6.6867389775602364], [3.8116757929240572, 5.8459274741803222, 2.0057852158074176, 3.3460005233982351, 1.8679849895154108], [-0.94638684513873139, -0.36820048144761586, -1.89154183490822, 6.8304392085601853, -0.54759992116106293], [6.9184433050337075, -2.0239118334182518, 4.1916995614982051, 3.5844627124415176, -2.5909867655211807]], [[0.63430437177623311, 4.6096755445929976, -2.3851775328072553, 3.5179828575490859, 4.143061050459778], [-1.5395872758448421, -4.7120941931804907, 2.1666854654502821, 0.46470684136481033, 3.5149363384388259], [-0.083440653566749567, 2.5452784163929785, 0.31094723189646167, -4.0706215721091183, -3.2551654526931939], [1.9591081027152368, 3.856780365666479, 1.5068947334443221, -0.67325729426250103, -1.0628014655828366]], [[-4.5753912854891547, -3.1753172335155213, -5.1960162975786481, 0.6809659457435604, -4.1172517904121504], [0.048360369155160399, -2.6933812593286568, 1.6310606572214299, 1.778643203909966, -2.6759223029495267], [-4.7850315217033552, 4.0617202741240597, 2.4484545042176418, -4.1026966741237274, -3.3732925960058138], [-4.9975419907298297, -3.0458506134993573, -2.7087693507242179, 4.1727624460891448, -0.1311812275845563]]], [[[0.59125441538636103, -1.6750677025136773, 0.9156742751033784, -2.6105872111261332, 0.96809255602156075], [2.2269214084427169, -2.6647234537694509, 3.8525707468562009, 4.3200862173730838, 4.3395930418235453], [-2.9400066056106358, 4.3998413970710546, -1.412600151581346, 3.5009433819228155, 4.9122762352147022], [-3.3113365709250751, 2.015713235774899, 4.4051744162096087, 1.7964535910978272, -2.0414665384856354]], [[-0.12609341131791396, 2.1057433583393728, -1.0221980246478015, 2.6395575272228173, 5.5328588521468918], [4.5545537986806472, 7.2023738338876342, -0.51676221085661211, 3.8219488793392804, 3.3577786935984619], [6.2676373252409512, -1.2530855205644871, 2.4343556068499534, 0.37074649006887928, 1.5536544175894802], [3.136492292608013, 1.9564195255213042, 7.6040011659404332, 6.8871892210113081, 3.3539981359217705]], [[6.3988671332232858, 4.7909058222050112, 3.8265773886100742, 0.22224280194391444, 2.9818656711669567], [1.3227404958933127, 4.189440699773864, 3.1401756793251021, -0.063814686104267437, 9.6372905007165777], [5.5777799870872009, -0.18248671749979195, 1.3096502783653907, 0.42284693390617534, 9.5120247979235373], [4.8299408732135234, 3.9506462947360861, 7.9266911541491591, 6.4421415174025265, 1.2252575577144684]], [[5.6539768901597522, 5.5803569720044663, 2.0507028973696353, -0.18624382783521831, 1.9577309012885946], [1.5675647339441015, 1.597540315385845, -0.46597504842914672, 5.0323848396534174, -0.49139799220947467], [6.026032363678592, -0.18328256953851874, -1.5696388103704599, 6.1783909399208543, 4.3129866302777398], [3.1699350576732872, 2.4709784879895054, 1.2345113578175297, 5.1057091759351998, 5.6514284054704733]], [[4.2770646813959088, 2.127628839879729, 1.1104452934075253, 7.5471948693964102, 9.0079166698229294], [4.6116219709314965, 5.838370383067776, 1.6955762844546749, 0.40010610610422681, 8.766202502116375], [0.34249021942289914, 0.14311931633149566, 0.20537977471388569, 1.4365711566563686, 6.9978120047404335], [7.9572005910774379, 3.0980548192277428, 7.4304174967706533, 0.49744982250902581, 9.5868183284128623]]]])+(1.-msk_ref)*numpy.array([[[[2.6695525794535104, 2.0838225085840758, 3.5813620813459046, -3.5216572202536254, 0.66394676205931535], [0.77984903256255311, 1.0860681667260899, 1.1679073816985044, -4.6158756699552859, 0.50643363917727768], [-3.8591244629750907, -0.2233145411831412, 1.4759976641873118, 2.0594546026935774, 3.9129869988156578], [3.2830117718247003, 2.8051672129941192, -3.7308942327308872, -4.6369006381681821, 3.9705752680101085]], [[1.5724656695794605, 1.7334580180493742, 1.0658320670414989, 4.3646092737370212, 7.7558950431208036], [1.6546694017081345, 4.9346588869469272, 2.8925148725809069, 0.97296267251368551, 1.0075938210417865], [3.1572637256969944, 1.7554246197590251, 0.7830300915009647, 4.4743125893887816, 2.1584398799558246], [1.9479161436533121, 8.5037993811662034, 4.177440309923675, 6.2534333248094631, 3.461900386413582]], [[-1.4494471463830951, -0.40723568669226218, 5.5120755621364568, -1.1147430311302955, -1.1205385610339782], [3.3843076597763115, 6.786591713688221, 7.6824330395317233, 4.0191343759930742, 2.4013591198771143], [7.8389663851965494, 1.4811098813428156, -1.1882857885912728, 0.35200571332085318, 0.37360514056676397], [3.9751056290571878, 2.5446381233665232, -0.76947659254697776, -0.14319653390546527, 6.5602679639931889]], [[-1.1604316657992584, 3.3912767825059209, -2.2301790010414124, 6.4549942748453475, 5.0140004889108525], [4.8347592846095608, 5.5386786179622334, -1.4775985860387921, 1.9084699398396898, 0.7528963359902221], [-1.5817292649855874, 2.082196116094412, 3.734233819610143, -0.2128819963141404, 2.7921098631669929], [0.90523660195331601, -0.22713510009107463, -1.8038986851874359, 0.32202263948443477, 1.3195712755812625]], [[-2.0265679605153819, -3.9492268202810408, -1.1528153318593271, -6.757790548517657, -7.6849979942439717], [-5.1361333967548264, -5.2555092595571953, -9.5635488993422619, -1.4224217192875175, -5.3581718245951491], [-2.8519169295642186, -0.73096576855533169, -5.1323789960922133, -9.7247916845375038, -4.2308621385654446], [-2.7966776517148606, -5.8991211401645014, -6.2959851562278892, -9.0492895885176861, -7.3453088992270077]]], [[[4.2976601950176772, 8.6100707591219319, 7.8527065110420402, 7.5734749701933204, 6.3440105253960484], [-1.0427648143661035, 8.7839814201661763, 6.5879893578132416, 7.4209855354052241, 5.437027810909262], [0.22301070731109274, -0.075627648094243494, 2.9064152735063544, 6.9965939317013284, 2.7449989439987061], [8.7624456897560297, 3.0100124774333405, 4.6646035414158327, 1.3825718739235597, 8.2492974885908215]], [[1.8494459022480125, 1.0111759433659078, -1.2652544675366242, 2.3421429520938712, 2.1537362490873271], [3.31964472674726, 4.1128750590851695, -3.3412952208703355, 0.23161402652619945, -3.4273266732044689], [0.93451406113596214, 2.697051604007827, -4.3888980022530895, 0.77682128110515336, -4.695598234981782], [-1.3332467441371114, -0.043555935965988724, -2.1703406110988093, 0.19831653898706048, -0.084633736541500859]], [[-6.6962301696600068, -0.033029635730780882, -0.80309983181476863, -2.6788897925195716, 1.8301391607068656], [-3.1564581789266493, -4.2541053217030296, -0.5465212401576105, -0.67855395868389845, 2.6640759700967287], [-4.7721277245802174, -0.38051728974627252, -5.1227471157804292, -1.0666111207469573, -4.0753501934788883], [2.2428264966466109, -2.1768474755270733, -4.7230802049555916, 0.87613299954444557, 1.7457517195638843]], [[-0.14747840333072304, -5.3474959600905123, -2.6316195605684856, -1.0678058321303006, -6.3824309595297866], [-5.8055503435546782, -4.2554416333590801, -2.3825851004132401, 0.78710720681480595, -0.28697702728841179], [-6.3562325248489744, 2.7366756942016268, 0.75859021586618436, -0.33180400255274556, -3.5156137001155408], [-1.6276717229213871, -5.980485508550859, -1.4249396100859828, -2.1821869428746004, -2.5507071067618803]], [[5.9638316087020158, -0.10304775511259656, 2.345881098900223, 6.3186426053546967, 6.0663199607242841], [7.3691044788334512, 9.1749112507304122, 8.6353832853144876, 6.1647607248190583, 7.6122214017620493], [3.85307979614422, 3.1605738596128239, 2.1160282805917228, 0.35294594543606017, 0.42813665290701941], [8.0221124643383206, 4.131088539829042, 7.9240343904142954, 4.1531824833179716, 3.9305423651986064]]], [[[1.2951316156179091, 5.8829036888550261, 1.5975275013913093, -2.7884206516410712, 1.7309180898827883], [-0.65856273180101788, -0.34656692657209387, 2.9272907354579427, 0.72119060295528303, -0.19223522016176586], [3.8712975502614126, -1.1370740246243125, 3.6169747194819069, -2.2121032450726239, 4.7708466269959464], [-2.9469455697797748, -3.5137885367975352, 1.4212430439146084, 3.5628323105735031, -0.02904147712825722]], [[4.2113665622848249, 4.0653605610381618, 0.23152603917692094, 5.2428198278513563, -3.5183726615986224], [1.1431337952608809, 2.5594484486556048, -0.72581003427488122, -3.0079351127119685, 3.3112743459127838], [-1.2526759724357275, -2.244012036453416, -0.56427881692957893, -2.3519344024056057, -1.5652853395824122], [-2.2946118252202123, -2.8509174583831736, -0.75041305782562118, -2.6364881427146818, 0.31315361021184529]], [[2.3580193695312008, 3.2336117337476313, -2.2595572440440206, 0.82754138212123252, 6.6588464547752881], [3.783783270139109, 5.818034951395374, 1.9778926930224694, 3.3181080006132868, 1.8400924667304626], [-0.97427936792367964, -0.39609300423256411, -1.9194343576931683, 6.802546685775237, -0.57549244394601118], [6.8905507822487593, -2.0518043562032, 4.1638070387132569, 3.5565701896565693, -2.6188792883061289]], [[4.7003135343131728, 8.6756847071299372, 1.6808316297296844, 7.5839920200860256, 8.2090702129967177], [2.5264218866920976, -0.64608503064355105, 6.2326946279872217, 4.53071600390175, 7.5809455009757656], [3.9825685089701901, 6.6112875789299181, 4.3769563944334013, -0.0046124095721786773, 0.81084370984374576], [6.0251172652521765, 7.9227895282034186, 5.5729038959812618, 3.3927518682744386, 3.0032076969541031]], [[-8.4503355240961184, -7.0502614721224841, -9.0709605361856109, -3.1939782928634024, -7.9921960290191132], [-3.8265838694518024, -6.5683254979356196, -2.2438835813855329, -2.0963010346969968, -6.5508665415564895], [-8.6599757603103171, 0.18677603551709687, -1.426489734389321, -7.9776409127306902, -7.2482368346127766], [-8.8724862293367934, -6.9207948521063205, -6.5837135893311807, 0.29781820748218202, -4.0061254661915191]]], [[[-0.23691998509202872, -2.5032421029920671, 0.087499874624988649, -3.438761611604523, 0.139918155543171], [1.3987470079643272, -3.4928978542478406, 3.0243963463778112, 3.4919118168946941, 3.5114186413451556], [-3.7681810060890255, 3.5716669965926648, -2.2407745520597357, 2.6727689814444258, 4.0841018347363125], [-4.1395109714034648, 1.1875388352965093, 3.577000015731219, 0.96827919061943746, -2.8696409389640252]], [[-6.1567235397091382, -3.9248867700518515, -7.0528281530390258, -3.3910726011684069, -0.49777127624433248], [-1.476076329710577, 1.1717437054964099, -6.5473923392478364, -2.2086812490519439, -2.6728514347927623], [0.23700719684972693, -7.2837156489557113, -3.5962745215412708, -5.659883638322345, -4.4769757108017441], [-2.8941378357832113, -4.0742106028699201, 1.5733710375492089, 0.8565590926200839, -2.6766319924694537]], [[2.212576636385605, 0.60461532536733031, -0.3597131082276066, -3.9640476948937664, -1.2044248256707242], [-2.8635500009443682, 0.0031502029361831774, -1.0461148175125787, -4.2501051829419483, 5.4510000038788968], [1.3914894902495201, -4.3687772143374728, -2.8766402184722901, -3.7634435629315055, 5.3257343010858564], [0.64365037637584255, -0.23564420210159476, 3.7404006573114783, 2.2558510205648457, -2.9610329391232124]], [[-1.6734373493330859, -1.7470572674883718, -5.2767113421232033, -7.5136580673280564, -5.3696833382042435], [-5.7598495055487362, -5.7298739241069931, -7.7933892879219844, -2.2950293998394207, -7.8188122317023128], [-1.3013818758142461, -7.5106968090313568, -8.8970530498632989, -1.1490232995719838, -3.0144276092150983], [-4.1574791818195509, -4.8564357515033327, -6.0929028816753084, -2.2217050635576383, -1.6759858340223648]], [[-3.7154872906231216, -5.8649231321393014, -6.882106678611505, -0.44535710262262018, 1.015364697803899], [-3.3809300010875338, -2.1541815889512543, -6.2969756875643554, -7.5924458659148035, 0.77365053009734464], [-7.6500617525961312, -7.8494326556875347, -7.7871721973051446, -6.5559808153626618, -0.99473996727859681], [-0.035351380941592403, -4.8944971527912875, -0.56213447524837701, -7.4951021495100045, 1.594266356393832]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_expandedData_rank4(self):
      arg0=numpy.array([[3.413552830644738, 4.9907455623773469, 0.55283009644976566, 3.5952154267421577, 4.5253568338022561], [-3.6317818597598004, -3.2755423086309463, 1.0198088397612972, 0.68821456202118458, -3.7916846480646162], [-0.091189422872552051, 1.8289994043787585, -2.6299197453154077, -4.9573940040278366, -3.8254140129750338], [-2.2633699842560304, -2.0406870364743277, 1.733588140446944, 0.18088500599928636, 4.6994198608356843]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.25636652722941111, -2.8039283448744556, -3.2928751796348421, -2.9800561135103742, 3.2234117554913606], [-0.98833665889860356, -0.44619111569988945, -1.6270120836443533, -4.9724496093892379, 0.35595383656441015], [0.30786475321511197, -1.321410190784138, -0.52591494841654196, -4.2447063128692619, 0.7265389770731181], [2.7112072463148866, 3.3226542263471561, -3.6108658759502763, 2.6232054496648107, -1.5676703673837142]], [[2.5424214273803445, -2.179369744365923, 3.9022151657915529, -0.77785177064323197, -1.3928236805423055], [1.6392194084720222, -3.6193157331520656, 1.593748897708867, -0.56175380233957917, 0.84102649741711133], [-3.2398768247103114, 1.1655883201980997, -3.5393791261785825, -1.0345162796399188, 0.77943696926730066], [-2.1552935745662571, -3.978877987444458, 0.42199460398242206, 3.5002240140815282, 3.1247415486996886]], [[-0.008995692751074813, 4.3179640997066961, 1.9369767717321569, -3.532437836211181, 4.5845426168653294], [-3.4791988776336611, -1.4026924341389591, 2.8946696405389503, -2.2374311393972235, -2.8368903257205624], [-0.69492354629588693, 0.66065194169611008, -4.3644424754138562, 4.4980885146284866, 0.28184983042698786], [-4.270533746053065, 2.4580524394670045, -4.4175175986617461, -1.5786894575400687, -1.3859474269606444]], [[-2.369154234295384, 2.1700237794137589, -4.4142861865508856, 4.2475548646857337, 4.1107842461884729], [-2.410335188344721, 2.9236978488845331, 0.56835471749784183, -2.7831624852199424, -3.784477368849799], [3.0154897906502232, -0.43605852890219765, 4.2068698723675197, 3.6994265899272563, 1.5924808240251807], [-0.24427375580971944, -4.0330941093370019, 0.30153886862906187, 0.55108833823844794, -2.6481176448726638]], [[-3.0043008643067823, 1.5586345009362956, 2.5954944836633125, 2.3735262175564777, 3.3107257126037481], [-0.96144050575101936, -2.639542006196308, 2.9243262322523567, 0.19602129882254538, -0.071424227651176508], [-1.8401387023847491, 0.72701055983842, -2.6007779184273341, -4.4201189639394336, -2.2342016973451506], [3.5725707530949009, -3.3269670763420356, -3.2514513817074517, 2.4435640336583431, 4.3332655304625032]]], [[[2.5274772491297872, 3.3316278895847056, 2.8016375824058706, 4.9497983765123692, 1.2295460270818221], [3.7899790613566218, -0.27187177010716113, -1.1379646524623732, -1.1902375702457881, -4.8558451010950385], [1.5103328729541543, -0.60619182858829301, 0.26421056393031073, -1.9675108306949673, -4.9600021441109163], [-0.10883027054653027, -4.3900751604985242, -1.9322638926623048, -2.8236669857857786, 2.8291701865053085]], [[0.14027180544229267, 3.346284284224069, 0.21376121762271882, -2.7557261684424228, -1.4473109097051418], [-3.041524342700769, -3.1996116069878977, 1.8076369746160728, -3.4823852208222061, 1.5398630982711978], [-0.17787104037158574, -2.7860021683539515, 4.4220815638768762, -0.22286370857537641, -3.2498266274286838], [3.7651085627850804, 2.446154208464816, 1.696758091418948, 2.508517137913401, 3.590241619457629]], [[2.2166816075028972, -4.139100727658775, 1.9287194399784671, -3.709433262930161, -4.9713050011514435], [-2.4488064204961879, -0.058546974714507982, 0.76077091874265079, 1.5011640836624585, -2.3598276877924853], [1.088156695791719, -1.18702799945802, 4.7357493167425506, -3.6249787398114774, 2.3541378414348575], [-4.0035571929782465, -3.3092379055114152, 0.31163247750999545, 4.0485956706188837, -2.7254679350835209]], [[2.1550600136293143, -0.64622174442837821, -2.4598593979142533, 3.9671481812406348, -4.1642035896646625], [-4.8261663524430247, -0.75472251236041377, -0.19938641223651654, 0.9144622822620736, -2.5212427177867927], [-3.1073484211034508, -1.6870520432849756, 0.050407646432304887, -3.2839909342792737, -3.5148689231559249], [-4.836454383824452, 1.6390610446897487, 4.7977162561724302, 4.5919195193022126, -2.0856364198832034]], [[-0.48922115610031192, -2.1678417392922702, -0.075536504239270386, -1.156419925428116, 3.4882565026352612], [2.0659172721893952, 3.5583309524672551, 4.6130467153621311, 0.20393195262176178, -0.87210873386523602], [2.6523052994955378, 2.6372353110838054, 2.2689082070755964, -2.863199419370086, 2.3119399613706095], [-3.65293557918727, 4.7746802851430346, 4.4723480628883436, -3.9557820445415501, 1.7485777646838843]]], [[[-1.1468800436632822, 1.5790277960889574, -2.1031083494780614, -0.059894268577309617, 4.9833307855173192], [1.6479230227436323, -4.7860547395759614, -2.6520214343796225, 1.6062503760902249, -2.8019455768623489], [2.7010048982561683, -1.2360370956269664, 2.7874584196694485, 0.10305128044211997, -0.13246673758836813], [-1.5132308830442787, -4.3872020037133534, -1.4453214800919065, 0.63159473710805258, 2.0520366207807728]], [[4.0623440919435829, 1.9082478376007543, -0.79193987886616846, 1.3353707353448465, -3.7492075673569425], [-1.5154814337279432, -1.5536399579459745, 4.5634129350727211, 3.3804196342779012, 3.2112682106830839], [-0.25984485801736312, 4.5426736805911521, 3.7551542763704706, 1.7900462277904525, 3.6628063584370611], [-2.2745053890451605, 3.2240382772039595, -1.9897772495283874, 4.0585972104053099, -1.5590766084683927]], [[-0.37056212224438578, -2.8031293412101412, -4.0122155202865759, 0.70465751655720421, -3.7253205129966673], [0.43375342369254088, 1.1394599004159724, 3.4594922343439656, -2.1990465600858675, -4.9035546219180803], [0.75790621225961985, 3.7002623400387762, -0.60727796223015407, 0.42958017522841097, -2.8644475196648402], [-4.8873791287301902, 4.7634103649381281, 1.0847763473454028, 2.7860646217330434, 4.3921415056155322]], [[-4.4600481827019509, -0.48842465042294148, 0.13994667587490373, 1.2409918198206373, -0.68500732706287959], [3.6299051283299111, 4.4397546195600981, 2.0177748266215563, -0.1494151970314519, -3.5144138432533314], [-2.9834945709090945, -2.33108982058438, -4.5782918661638528, 0.41607393366120071, 3.5397537249736679], [-1.7461086205743417, -2.8042704482577738, -1.4207058536654502, 3.3439309341820174, 4.5392143770051305]], [[-3.7914911287507023, 4.7271666519748425, 2.631289200394006, 1.4271471102878008, -2.4560694230545899], [-3.6744376955856906, -1.4275853519879833, -4.4394853400662999, 4.335870006224571, -0.24870780849679441], [-3.3825960653051967, -2.6421286670356849, -4.6575525432283005, 0.9419403353672342, 2.0589689090601695], [-1.5928201763965033, -1.0203052235279864, 0.50648511911151939, 3.6335247698423565, -0.16392003414818923]]], [[[-2.104900783159632, -2.192590871697746, -3.4538684108885231, 1.7048658726629187, 1.4905519897639854], [2.3730682474708331, 1.542166404953818, 4.1543595091920622, -1.1578541843129364, 1.7740680942702731], [3.0935922145133539, -2.5220919519081599, -2.6253748583367864, 0.15144510018346935, 0.16601142464628182], [-1.8578710462851578, 0.4301263345136741, 1.2438935596161693, 4.7605958945804279, 3.9486714077693179]], [[2.292877511245619, 0.8648210870538664, -0.13048079481249175, -0.22372138575125788, -4.0586242126785246], [4.968346838696295, 0.69586895528144321, -0.15076687626093932, 1.944739982578545, 1.9654058912616126], [3.584105273716677, -3.7716773228936082, -4.145852026247737, 0.74658457064715744, -4.5967282428704213], [1.1751376730627818, -4.0857176320847612, 4.3503651860354164, 0.37752752027169478, -3.5330343076082249]], [[-3.5370840115134472, 3.44949571181143, -2.6220465334924126, -0.38578566928079816, -2.4242083422118741], [-2.7236647484117613, -4.9552295136845572, 2.7359991672622979, -4.0671293708892886, 4.3820002549328603], [1.7286422041802609, -4.2147446231433694, -3.193506492388829, -4.3790321349194521, -3.727399594873928], [-2.6119947235741883, -0.11384476222688011, 3.9604682727994049, -0.50442340159708809, 1.1173637806962695]], [[2.5634597902192038, 0.40441801350387507, 4.6380895480956568, 3.8437527729404923, -2.3467729835414444], [-2.7524315184877279, -1.5920598458973254, -3.3316966551887917, 3.8140363507064983, 0.20525337161538992], [3.9592216783396204, 4.2958286552380756, -4.4589321305260139, -1.9656468944474303, -1.492570216601977], [2.2971001930917563, 0.69530379917067986, 2.760750402546277, 0.99787707866195063, 2.3268943342026782]], [[-4.1604981998659385, 4.189126799002068, -4.8742543547413844, 1.9906817239783381, 3.6054265187525676], [0.85732588811316646, 0.59753260429887867, 2.5848823848803946, 4.9348914882661781, -4.2579963719534781], [4.9927828113911463, -1.2257725815945628, 2.1626458562948283, -0.31370941607021141, -1.1508557520787877], [1.8495551225262927, 0.59733148956172233, -1.0456098539254146, 2.3110888555911133, 0.076516685324187428]]]])+(1.-msk_arg1)*numpy.array([[[[-0.4886756011343909, -0.44534407645434815, 1.9269679109047377, -0.42475062479379933, 0.76113960244103662], [0.65051370186012569, 4.0261411936340394, -2.6273329802163001, 2.5249830725735558, 1.6664606846824723], [-3.5385330150351435, 4.0189899185076268, 0.7563603611012768, -0.75557522826994816, 3.6578056345727479], [-3.2079106520376346, 2.5641932571610058, 2.3544080075388294, 3.2802353606040349, 2.3617976532756195]], [[-3.2435967953765399, -3.7703455468358236, 1.6264418470153261, 2.3855072726703233, -1.8403798027639362], [1.7215497095549512, -0.46996644681035882, 4.7040194417241903, -1.1100630147164892, 2.7864047981708531], [-2.3101209359391941, 1.3937077391152277, -4.6887621626068317, -0.64602807658070471, -2.6956284974690847], [4.7596671859562338, -3.5902515479353916, -4.676001409628129, 1.4758828614071531, 1.5342991865824063]], [[2.237819902279349, 0.74083385435220883, -1.2852648538420874, -0.52100724676176569, 4.6012834693211886], [3.0742282162710879, 3.516261467746137, -4.6591368386422367, -4.6983690626757388, -4.2133338193495664], [-4.9527525341585088, -0.91195793820504445, -3.6696926572596178, -2.992207175387207, 4.7388636992697428], [-0.9246526457310198, 0.22565399154752086, -3.4379195923818342, 0.40821038936037013, -1.8802832129646552]], [[2.6235135974420709, -3.6356492571197063, 3.0109077079633835, -1.2613647448118925, 2.7922321737440861], [-4.7294440896175463, -4.6706898392417662, 2.2355243769523963, 4.1857240834549501, -3.1498421228640918], [-4.5232095712319644, -3.5650686671557663, -2.9117548355080758, 2.6377166781292321, -3.8219987883023476], [1.0175622936202933, 1.6317496761103936, 4.84898215404999, -2.6025319141315908, 3.6412913838923551]], [[1.3865641978607925, -3.4970824735118136, -2.3954264917054626, -1.4404226382506713, -1.8901183363874772], [3.9823600078147798, 2.7854617704743321, 4.362585075288532, 1.1412718253491594, -4.159027401823411], [-2.0611686726547465, -0.62524600721672563, 4.4909538083864327, 2.144159963592144, -2.2746986815379868], [4.1823158140106127, -3.5859414272965475, -3.8847092109238424, -3.5506432259605694, -1.4948837396266645]]], [[[4.7298572629710218, -3.6149566373967525, -4.2958211091382568, -2.1669024743430985, 1.8789138617383037], [0.68890315967369276, 2.9119340525340407, -3.1742837791791221, -2.494460467052193, -0.18210199292817819], [-0.86178302787129457, -3.919963062427513, -3.5220794215090843, -2.986302324155087, -4.2954249270263887], [2.1875860637249307, 0.92248131782718623, 3.671508283492507, -3.7844410421931851, -3.3352492455391602]], [[1.4035683430853751, 4.0193193677842594, 4.3142952741798766, -0.43490453848334987, -0.18447633822118981], [2.3918960849629221, 1.9097719332199512, -0.16191969893690938, 4.9044980872695518, 3.3250621303668559], [0.37549063374906844, 4.4810850120900945, 3.3657523393202915, -4.9664940278597509, 0.712678433215026], [-3.038309447159623, -0.35270395338862581, 2.7456764570799503, 0.12260085419000699, 2.7452532276798483]], [[1.8385981466756327, -4.1286602941196415, 2.9578218293254102, 2.0021824747299206, 1.6993272735573512], [4.5196076507752618, 2.1934471849623316, -3.4223449525194716, -2.1124231689965711, -0.46984368417784061], [-4.2659119269921604, -3.489185855188266, -1.0515461336010268, 2.4074666593677474, 2.718963836313975], [-0.40860150528553518, -4.6909919917590015, -2.4506101436501018, -4.6350382639903813, -3.9351179705673465]], [[-0.4368975557909156, 4.6687784964457943, 4.568415602485759, -4.9061207995153779, -4.8262118574969728], [1.2952635354258817, 4.5295707707972372, 1.1072785204111932, -0.23195289071690883, 1.4306189259560949], [4.2465581389005003, 2.0092579794582406, 4.7662815706627413, -3.5735471868596393, -3.9514229848503568], [-0.2091067921018519, 1.1644610335184726, -2.5610103008707141, -1.0646348048921772, -4.8192332944941221]], [[3.2086006014465145, 4.9047492389811413, 3.725480953160222, -3.5147049355423277, -2.0497932775057395], [0.68639424609507493, -2.1994958120577888, -0.10711839599910533, -2.543085824313136, 0.7812679307021817], [-3.6911641548284244, -3.8822747383086242, 0.019662525660325336, -0.71787864281625868, 3.8341592473320478], [-0.22431646478831802, 0.31421528887606609, 1.1246225063320416, 2.8725683736736149, 2.5958438570702409]]], [[[-0.93804431525083576, 3.9434926186002457, 4.9560001080623177, 4.96875601642307, 1.2364961117512818], [-4.4087452928564463, -1.280620017696148, -4.2369382350408813, -1.9524737169702422, -1.2232892764940462], [-4.1654082274573989, -1.7974978717586607, 3.1156799546839604, 1.7661907775887773, -4.7127042276049549], [1.9949968860329337, 2.7711088501463275, -1.599972605953468, 1.7383976488851038, -2.5270989354614359]], [[2.4263247828627632, -2.9481037212088932, -3.4639734198926631, -0.89436278979313144, 1.4602311369962937], [1.0723185751808595, -1.7853201972769686, 2.1532740553380059, 4.0747678778920875, -4.730441426274675], [-4.9945674176323287, -2.182176122144186, -0.4093986781618959, 4.0263870682826663, 4.3599958201121254], [4.3234908549885649, 4.6031558410857674, -0.21068152457013412, 3.4706909484729174, -2.4870527387037278]], [[-0.76687944617954606, -3.5257737248124013, -1.4053099240220215, 3.632082035819975, -1.6130254310259695], [2.3575022456159456, -3.9843420182853939, 0.37480375068443816, -1.5512374282371044, -0.055291317768555004], [-2.1121637451875674, -2.6168544959376092, -0.25431324524035492, -1.4068123672760757, -1.5626099732314236], [2.3969466744753163, 4.0347877138010695, -2.8392519960166727, 2.5344623602309735, 3.3924869847610122]], [[4.0049990861231528, 0.57972492583667989, -3.2438619567106364, -4.7246732698973526, -2.1923380275845239], [-0.11524545565395972, -4.336898032281451, 3.8194038941373023, -2.3868587435180268, 4.2576329662420473], [-4.709598673151695, 0.46288070750810117, 0.60722415688929487, 0.93468767631348015, -3.5733681874696721], [1.4930696110941124, -0.62853667164306959, -0.12051171850361975, 4.5493567614844537, -4.3847486816898087]], [[-1.4176755691236576, 2.4453598996656307, -3.6958776962914253, -0.56135869739005972, 0.040273989419755907], [0.32971052898971642, -3.1917916757223699, -4.8747575324200643, -1.7133970832745771, 4.4832527911353868], [0.36926948926633596, 1.921231135893775, 1.9600542910537344, 1.7193753231834457, 3.9549516094774102], [4.3391848326804645, 3.5497255924722122, 1.817449042922008, 1.2712098174031379, 0.35241209166391751]]], [[[0.34153151078710753, -3.7419788835827115, -4.7787371023713003, 4.4742605514214979, 3.4141095970509365], [3.0767951713085662, 4.8245008482305636, -3.5365170268269432, 4.5432662200386709, -1.1883533801463808], [3.7823590002706791, -2.2945980599319915, -4.2424335655609209, -2.9771938074287196, -4.6078117526945928], [-2.0151227108426104, -1.3939447080208875, 4.6309888238261152, -1.6448629575079874, -1.2097769916052381]], [[-2.8328973365696717, 1.1090385826411975, -4.782061365379203, 4.6564212827694273, -3.1055616741261494], [-2.8429835774716414, 3.7835300581108537, -1.0429958550466045, 0.09521054848214483, 4.7949698416304578], [-4.0398899822983134, -2.5455494543185377, 4.8522427558373007, -2.0712174839328004, -4.3983346307690487], [-4.2763142192238925, 4.5569774985184672, 1.3878783000788184, -0.34426698160256652, 0.8765448578157562]], [[-1.7901437065891801, -2.0306187014716159, 3.8259195091815741, 2.6253568326083476, 3.8890081974674473], [4.2416034796378916, -4.0223688338132018, -0.96550437376796605, -4.3842717278011767, -2.6980411575420984], [-4.0198257661807926, 4.572727128810806, -2.832483848043883, -0.6736995813090072, 3.0091910485722071], [-0.84978773305103417, 1.3483030503627136, 4.7336168335269981, -2.3386316163730534, -4.3508566799891462]], [[-2.6522137705826889, 2.7080025265579177, -0.31422524771162941, -3.4409197004878922, -2.3084113700232347], [-0.98383839671141704, 0.95805916872489849, 0.9499707548847649, -1.9939716280790956, 0.2908997005833136], [-2.2051889333358785, 3.6118971919665128, -1.6268343710857271, 3.3540324247340916, -4.0061369146105381], [1.3478004483155983, -2.7407353032116619, -2.1929708675539858, -2.8843265140124763, 4.9393545268705985]], [[4.2693072620529051, -1.2753636478908028, -3.4227458450765602, -4.8066174525633514, -0.038865506636271085], [-1.925187985401553, 0.23950580854013204, -1.8380477639754389, 2.8605774906380086, -1.5385785659156204], [-1.1575468681005843, 1.2168994116357821, 3.9145445190572126, -0.15804671650569801, 1.8533872600795647], [-3.2855853315356276, 0.55919528326437451, -0.87781816860159179, -3.0170324519102953, 1.4111596768004189]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.1571863034153269, 0.60962448577028239, 0.12067765100989591, 0.43349671713436377, 6.6369645861360986], [2.4252161717461345, 2.9673617149448486, 1.7865407470003847, -1.5588967787444998, 3.7695066672091482], [3.72141758385985, 2.0921426398606, 2.8876378822281961, -0.8311534822245239, 4.1400918077178561], [6.1247600769596247, 6.7362070569918941, -0.19731304530553828, 6.0367582803095488, 1.8458824632610238]], [[7.5331669897576914, 2.8113758180114239, 8.8929607281688998, 4.2128937917341149, 3.5979218818350414], [6.6299649708493691, 1.3714298292252813, 6.5844944600862139, 4.4289917600377677, 5.8317720597944582], [1.7508687376670355, 6.1563338825754466, 1.4513664361987644, 3.9562292827374281, 5.7701825316446476], [2.8354519878110898, 1.0118675749328889, 5.412740166359769, 8.4909695764588751, 8.1154871110770355]], [[0.54383440369869085, 4.8707941961564618, 2.4898068681819225, -2.9796077397614154, 5.137372713315095], [-2.9263687811838954, -0.84986233768919339, 3.4474997369887159, -1.6846010429474578, -2.2840602292707968], [-0.14209344984612127, 1.2134820381458757, -3.8116123789640906, 5.0509186110782522, 0.83467992687675352], [-3.7177036496032994, 3.0108825359167701, -3.8646875022119804, -1.0258593610903031, -0.83311733051087877]], [[1.2260611924467737, 5.7652392061559166, -0.8190707598087279, 7.8427702914278914, 7.7059996729306306], [1.1848802383974366, 6.5189132756266908, 4.1635701442399995, 0.8120529415222153, -0.18926194210764136], [6.6107052173923808, 3.15915689783996, 7.8020852991096774, 7.2946420166694139, 5.1876962507673383], [3.3509416709324382, -0.43787868259484419, 3.8967542953712195, 4.1463037649806056, 0.94709778186949389]], [[1.5210559694954737, 6.0839913347385517, 7.1208513174655685, 6.8988830513587338, 7.8360825464060042], [3.5639163280512367, 1.8858148276059481, 7.4496830660546127, 4.7213781326248014, 4.4539326061510796], [2.685218131417507, 5.2523673936406761, 1.924578915374922, 0.10523786986282246, 2.2911551364571054], [8.097927586897157, 1.1983897574602205, 1.2739054520948043, 6.9689208674605991, 8.8586223642647592]]], [[[-1.1043046106300132, -0.30015397017509482, -0.83014427735392982, 1.3180165167525688, -2.4022358326779782], [0.15819720159682138, -3.9036536298669615, -4.7697465122221736, -4.8220194300055885, -8.4876269608548398], [-2.1214489868056461, -4.2379736883480934, -3.3675712958294897, -5.5992926904547673, -8.5917840038707176], [-3.7406121303063307, -8.0218570202583237, -5.5640457524221052, -6.455448845545579, -0.8026116732544919]], [[-3.1352705031886536, 0.070741975593122675, -3.0617810910082275, -6.0312684770733691, -4.7228532183360876], [-6.3170666513317153, -6.4751539156188436, -1.4679053340148736, -6.757927529453152, -1.7356792103597485], [-3.4534133490025321, -6.0615444769848974, 1.1465392552459299, -3.4984060172063227, -6.5253689360596301], [0.48956625415413413, -0.82938810016613029, -1.5787842172119984, -0.76702517071754528, 0.31469931082668268]], [[3.2364904472641944, -3.1192918878974778, 2.9485282797397643, -2.6896244231688637, -3.9514961613901463], [-1.4289975807348907, 0.96126186504678923, 1.780579758503948, 2.5209729234237557, -1.3400188480311881], [2.1079655355530162, -0.16721915969672274, 5.7555581565038478, -2.6051699000501802, 3.3739466811961547], [-2.9837483532169493, -2.289429065750118, 1.3314413172712927, 5.0684045103801809, -1.7056590953222237]], [[2.8432745756504989, 0.041992817592806375, -1.7716448358930688, 4.6553627432618194, -3.4759890276434779], [-4.1379517904218401, -0.066507950339229183, 0.48882814978466804, 1.6026768442832582, -1.8330281557656081], [-2.4191338590822662, -0.99883748126379102, 0.73862220845348947, -2.5957763722580891, -2.8266543611347403], [-4.1482398218032674, 2.3272756067109333, 5.4859308181936148, 5.2801340813233972, -1.3974218578620188]], [[-4.2809058041649282, -5.9595263873568864, -3.8672211523038866, -4.9481045734927322, -0.30342814542935503], [-1.725767375875221, -0.23335369559736119, 0.82136206729751482, -3.5877526954428545, -4.6637933819298523], [-1.1393793485690784, -1.1544493369808109, -1.5227764409890199, -6.6548840674347023, -1.4797446866940067], [-7.4446202272518862, 0.98299563707841831, 0.68066341482372739, -7.7474666926061664, -2.043106883380732]]], [[[-1.2380694665358343, 1.4878383732164053, -2.1942977723506134, -0.15108369144986167, 4.8921413626447672], [1.5567335998710803, -4.8772441624485134, -2.7432108572521745, 1.5150609532176729, -2.893134999734901], [2.6098154753836162, -1.3272265184995184, 2.6962689967968965, 0.011861857569567924, -0.22365616046092018], [-1.6044203059168307, -4.4783914265859055, -1.5365109029644586, 0.54040531423550053, 1.9608471979082207]], [[5.8913434963223414, 3.7372472419795129, 1.0370595255125901, 3.164370139723605, -1.920208162978184], [0.31351797065081533, 0.27535944643278398, 6.3924123394514796, 5.2094190386566597, 5.0402676150618424], [1.5691545463613954, 6.3716730849699106, 5.5841536807492291, 3.619045632169211, 5.4918057628158197], [-0.44550598466640201, 5.053037681582718, -0.1607778451496289, 5.8875966147840684, 0.26992279591036583]], [[-3.0004818675597935, -5.433049086525549, -6.6421352656019836, -1.9252622287582035, -6.3552402583120751], [-2.1961663216228668, -1.4904598448994353, 0.82957248902855785, -4.8289663054012752, -7.5334743672334881], [-1.8720135330557879, 1.0703425947233685, -3.2371977075455618, -2.2003395700869968, -5.494367264980248], [-7.5172988740455979, 2.1334906196227204, -1.5451433979700049, 0.15614487641763564, 1.7622217603001245]], [[-9.4174421867297866, -5.4458186544507781, -4.8174473281529329, -3.7164021842071993, -5.6424013310907162], [-1.3274888756979255, -0.51763938446773849, -2.9396191774062803, -5.1068092010592885, -8.471807847281168], [-7.9408885749369311, -7.2884838246122161, -9.5356858701916885, -4.5413200703666359, -1.4176402790541687], [-6.7035026246021783, -7.7616644522856104, -6.3780998576932868, -1.6134630698458192, -0.41817962702270606]], [[-7.6169051417257361, 0.90175263899980873, -1.1941248125810278, -2.398266902687233, -6.2814834360296237], [-7.4998517085607244, -5.2529993649630171, -8.2648993530413328, 0.51045599324953717, -4.0741218214718282], [-7.2080100782802301, -6.4675426800107187, -8.4829665562033334, -2.8834736776077996, -1.7664451039148643], [-5.4182341893715371, -4.8457192365030206, -3.3189288938635144, -0.1918892431326773, -3.989334047123223]]], [[[-4.3682707674156624, -4.4559608559537764, -5.7172383951445536, -0.55850411159311175, -0.77281799449204502], [0.10969826321480269, -0.72120357930221246, 1.8909895249360318, -3.4212241685689668, -0.4893018899857573], [0.83022223025732345, -4.7854619361641904, -4.8887448425928168, -2.1119248840725611, -2.0973585596097486], [-4.1212410305411886, -1.8332436497423563, -1.0194764246398611, 2.4972259103243974, 1.6853014235132875]], [[0.25219047477129131, -1.1758659494204613, -2.1711678312868194, -2.2644084222255856, -6.0993112491528523], [2.9276598022219673, -1.3448180811928845, -2.191453912735267, -0.09594705389578273, -0.075281145212715117], [1.5434182372423493, -5.8123643593679359, -6.1865390627220647, -1.2941024658271703, -6.637415279344749], [-0.86554936341154587, -6.1264046685590889, 2.3096781495610887, -1.6631595162026329, -5.5737213440825526]], [[-1.8034958710665032, 5.183083852258374, -0.88845839304546859, 1.3478024711661458, -0.69062020176493011], [-0.99007660796481733, -3.2216413732376132, 4.4695873077092418, -2.3335412304423446, 6.1155883953798043], [3.4622303446272049, -2.4811564826964254, -1.459918351941885, -2.6454439944725081, -1.993811454426984], [-0.87840658312724429, 1.6197433782200639, 5.6940564132463489, 1.2291647388498559, 2.8509519211432135]], [[2.7443447962184901, 0.58530301950316144, 4.8189745540949431, 4.0246377789397787, -2.165887977542158], [-2.5715465124884416, -1.411174839898039, -3.1508116491895053, 3.9949213567057846, 0.38613837761467629], [4.1401066843389067, 4.4767136612373619, -4.2780471245267275, -1.784761888448144, -1.3116852106026906], [2.4779851990910426, 0.87618880516996622, 2.9416354085455634, 1.178762084661237, 2.5077793402019646]], [[0.53892166096974581, 8.8885466598377523, -0.17483449390570005, 6.6901015848140224, 8.3048463795882519], [5.5567457489488508, 5.296952465134563, 7.2843022457160789, 9.6343113491018624, 0.44142348888220617], [9.6922026722268306, 3.4736472792411215, 6.8620657171305126, 4.3857104447654729, 3.5485641087568967], [6.548974983361977, 5.2967513503974066, 3.6538100069102697, 7.0105087164267976, 4.7759365461598717]]]])+(1.-msk_ref)*numpy.array([[[[2.9248772295103471, 2.9682087541903899, 5.3405207415494758, 2.9888022058509387, 4.1746924330857746], [4.0640665325048637, 7.4396940242787775, 0.78621985042843789, 5.9385359032182938, 5.0800135153272103], [-0.12498018439040548, 7.4325427491523648, 4.1699131917460148, 2.6579776023747899, 7.0713584652174859], [0.20564217860710343, 5.9777460878057438, 5.7679608381835674, 6.6937881912487729, 5.7753504839203575]], [[1.747148767000807, 1.2204000155415233, 6.617187409392673, 7.3762528350476702, 3.1503657596134107], [6.7122952719322981, 4.5207791155669881, 9.6947650041015372, 3.8806825476608577, 7.7771503605482], [2.6806246264381528, 6.3844533014925746, 0.30198339977051525, 4.3447174857966422, 2.2951170649082622], [9.7504127483335807, 1.4004940144419553, 0.31474415274921785, 6.4666284237845, 6.5250447489597532]], [[2.7906499987291147, 1.2936639508019745, -0.73243475739232178, 0.031822849687999977, 5.1541135657709543], [3.6270583127208536, 4.0690915641959027, -4.106306742192471, -4.1455389662259732, -3.6605037228998007], [-4.3999224377087431, -0.35912784175527879, -3.1168625608098521, -2.4393770789374414, 5.2916937957195085], [-0.37182254928125413, 0.77848408799728652, -2.8850894959320685, 0.96104048581013579, -1.3274531165148895]], [[6.2187290241842286, -0.040433830377548663, 6.6061231347055411, 2.3338506819302651, 6.3874476004862437], [-1.1342286628753886, -1.0754744124996085, 5.8307398036945539, 7.7809395101971077, 0.44537330387806584], [-0.92799414448980677, 0.030146759586391347, 0.68346059123408187, 6.2329321048713897, -0.22678336156018997], [4.6127777203624509, 5.2269651028525512, 8.4441975807921477, 0.99268351261056686, 7.2365068106345127]], [[5.9119210316630486, 1.0282743602904425, 2.1299303420967934, 3.0849341955515848, 2.6352384974147789], [8.5077168416170359, 7.3108186042765881, 8.8879419090907881, 5.6666286591514154, 0.36632943197884504], [2.4641881611475096, 3.9001108265855304, 9.0163106421886887, 6.6695167973944001, 2.2506581522642692], [8.7076726478128688, 0.93941540650570854, 0.6406476228784137, 0.9747136078416867, 3.0304730941755915]]], [[[1.0980754032112214, -7.2467384971565529, -7.9276029688980572, -5.7986843341028989, -1.7528679980214967], [-2.9428787000861076, -0.71984780722575969, -6.8060656389389225, -6.1262423268119939, -3.8138838526879786], [-4.493564887631095, -7.5517449221873134, -7.1538612812688847, -6.6180841839148874, -7.9272067867861891], [-1.4441957960348697, -2.7093005419326142, 0.039726423732706628, -7.4162229019529855, -6.9670311052989611]], [[-1.8719739655455712, 0.74377705915331305, 1.0387529655489303, -3.7104468471142962, -3.4600186468521361], [-0.88364622366802426, -1.3657703754109951, -3.4374620075678557, 1.6289557786386055, 0.04951982173590963], [-2.9000516748818779, 1.2055427034591482, 0.090210030689345189, -8.2420363364906972, -2.5628638754159203], [-6.3138517557905693, -3.6282462620195721, -0.52986585155099597, -3.1529414544409393, -0.53028908095109806]], [[2.8584069864369299, -3.1088514543583443, 3.9776306690867074, 3.0219913144912178, 2.7191361133186485], [5.539416490536559, 3.2132560247236288, -2.4025361127581744, -1.0926143292352739, 0.5499651555834566], [-3.2461030872308632, -2.4693770154269687, -0.031737293839729563, 3.4272754991290446, 3.7387726760752722], [0.61120733447576203, -3.6711831519977043, -1.4308013038888046, -3.6152294242290841, -2.9153091308060493]], [[0.25131700623026898, 5.3569930584669789, 5.2566301645069435, -4.2179062374941934, -4.1379972954757882], [1.9834780974470663, 5.2177853328184218, 1.7954930824323778, 0.45626167130427575, 2.1188334879772794], [4.9347727009216849, 2.6974725414794252, 5.4544961326839259, -2.8853326248384548, -3.2632084228291722], [0.47910776991933268, 1.8526755955396572, -1.8727957388495295, -0.37642024287099263, -4.1310187324729375]], [[-0.58308404661810176, 1.1130645909165251, -0.066203694904394261, -7.3063895836069435, -5.8414779255703557], [-3.1052904019695413, -5.991180460122405, -3.8988030440637216, -6.3347704723777518, -3.0104167173624345], [-7.4828488028930407, -7.6739593863732409, -3.7720221224042909, -4.5095632908808749, 0.042474599267431579], [-4.0160011128529343, -3.4774693591885502, -2.6670621417325746, -0.9191162743910013, -1.1958407909943753]]], [[[-1.0292337381233878, 3.8523031957276936, 4.8648106851897657, 4.8775665935505179, 1.1453066888787298], [-4.4999347157289984, -1.3718094405687, -4.3281276579134333, -2.0436631398427942, -1.3144786993665982], [-4.256597650329951, -1.8886872946312128, 3.0244905318114084, 1.6750013547162252, -4.8038936504775069], [1.9038074631603816, 2.6799194272737754, -1.6911620288260201, 1.6472082260125518, -2.618288358333988]], [[4.2553241872415217, -1.1191043168301347, -1.6349740155139045, 0.93463661458562708, 3.2892305413750522], [2.901317979559618, 0.043679207101789963, 3.9822734597167644, 5.903767282270846, -2.9014420218959165], [-3.1655680132535702, -0.35317671776542747, 1.4196007262168626, 5.8553864726614249, 6.1889952244908839], [6.1524902593673234, 6.4321552454645259, 1.6183178798086244, 5.2996903528516759, -0.65805333432496926]], [[-3.3967991914949538, -6.155693470127809, -4.0352296693374292, 1.0021622905045673, -4.2429451763413777], [-0.27241749969946216, -6.6142617636008012, -2.2551159946309696, -4.1811571735525117, -2.6852110630839627], [-4.7420834905029752, -5.2467742412530169, -2.8842329905557627, -4.0367321125914835, -4.1925297185468313], [-0.23297307084009145, 1.4048679684856618, -5.4691717413320804, -0.095457385084434243, 0.76256723944560445]], [[-0.95239491790468378, -4.3776690781911567, -8.2012559607384734, -9.6820672739251883, -7.1497320316123609], [-5.0726394596817963, -9.2942920363092867, -1.1379901098905343, -7.3442527475458634, -0.69976103778578924], [-9.6669926771795325, -4.4945132965197354, -4.3501698471385417, -4.0227063277143564, -8.5307621914975087], [-3.4643243929337242, -5.5859306756709062, -5.0779057225314563, -0.40803724254338292, -9.3421426857176453]], [[-5.2430895820986914, -1.3800541133094031, -7.5212917092664586, -4.3867727103650935, -3.7851400235552779], [-3.4957034839853174, -7.0172056886974037, -8.7001715453950972, -5.5388110962496109, 0.65783877816035297], [-3.4561445237086978, -1.9041828770812588, -1.8653597219212994, -2.1060386897915881, 0.1295375965023764], [0.5137708197054307, -0.27568842050282161, -2.0079649700530258, -2.5542041955718959, -3.4730019213111163]]], [[[-1.9218384734689229, -6.0053488678387419, -7.0421070866273308, 2.2108905671654675, 1.1507396127949061], [0.81342518705253575, 2.5611308639745332, -5.7998870110829737, 2.2798962357826404, -3.4517233644024112], [1.5189890160146486, -4.5579680441880219, -6.5058035498169513, -5.2405637916847496, -6.8711817369506232], [-4.2784926950986408, -3.657314692276918, 2.3676188395700848, -3.9082329417640178, -3.4731469758612685]], [[-4.8735843730439994, -0.93164845383313022, -6.8227484018535307, 2.6157342462950997, -5.1462487106004771], [-4.8836706139459691, 1.742843021636526, -3.0836828915209322, -1.9454764879921829, 2.7542828051561301], [-6.0805770187726411, -4.5862364907928654, 2.811555719362973, -4.1119045204071281, -6.4390216672433764], [-6.3170012556982202, 2.5162904620441395, -0.65280873639550929, -2.3849540180768942, -1.1641421786585715]], [[-0.056555566142236113, -0.29703056102467196, 5.5595076496285181, 4.3589449730552916, 5.6225963379143913], [5.9751916200848356, -2.2887806933662578, 0.76808376667897793, -2.6506835873542327, -0.96445301709515441], [-2.2862376257338486, 6.30631526925775, -1.098895707596939, 1.0598885591379368, 4.7427791890191511], [0.88380040739590982, 3.0818911908096576, 6.4672049739739421, -0.60504347592610941, -2.6172685395422022]], [[-2.4713287645834026, 2.8888875325572041, -0.13334024171234304, -3.2600346944886058, -2.1275263640239483], [-0.80295339071213068, 1.1389441747241849, 1.1308557608840513, -1.8130866220798092, 0.47178470658259997], [-2.0243039273365921, 3.7927821979657992, -1.4459493650864408, 3.5349174307333779, -3.8252519086112517], [1.5286854543148847, -2.5598502972123756, -2.0120858615546995, -2.7034415080131899, 5.1202395328698849]], [[8.9687271228885894, 3.4240562129448815, 1.2766740157591241, -0.10719759172766707, 4.6605543541994132], [2.7742318754341313, 4.9389256693758163, 2.8613720968602454, 7.5599973514736929, 3.1608412949200639], [3.5418729927351, 5.9163192724714664, 8.6139643798928969, 4.5413731443299863, 6.552807120915249], [1.4138345293000567, 5.2586151441000588, 3.8216016922340925, 1.682387408925389, 6.1105795376361032]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[3.0375466687956099, -2.3208203419688345], [4.4188049152675895, 4.0104863863246756]], [[-4.8579023433555886, 3.6025242293605881], [-1.7536341369826891, -3.0315480347692714]], [[-2.1962133124896743, -0.2105278171406173], [3.6883769437881746, -4.5211038984829415]], [[-0.85470445756831026, 1.2328397817211929], [-3.8522091885679313, -2.9330412926877814]], [[-1.5601254928097532, -1.8277085242069422], [-3.4879555270772666, 1.6846233639800046]], [[4.5874480238694222, -0.41769025321377029], [-1.9495024775351188, -1.821152667001984]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.9436946201698158, -2.0888465513384133], [3.302199854437939, 1.4404566260549156]], [[-3.0455633218431952, -2.0085785502071185], [2.1859372235227212, -0.65592197756721671]], [[-1.0037586735481185, 2.51521604702448], [-2.9847388393229157, -3.2167190473113862]], [[0.92948514742103416, 3.7497473418205178], [-3.9043690519131182, 1.1280734677974156]], [[-0.58095464936006991, -1.3762968755410254], [-3.9269082222104599, 3.2728219448531828]], [[-0.1462067740350399, 3.0211985800119834], [-3.7715938052894993, -0.11019209125010221]]])+(1.-msk_arg1)*numpy.array([[[0.21231921032862555, 3.7063442555767452], [-1.8984442253048615, -2.2108285356189681]], [[1.445670242443021, -3.1481323043686125], [0.16224567219952313, 4.4176660163578667]], [[-4.2629910301345237, 3.7567928170878755], [-0.78441064591608711, -4.3344298651524866]], [[2.8727649820693566, -4.1323316317036554], [0.62896333699929396, 1.5500249859593005]], [[4.0029723251215739, -0.51726525846121518], [-4.6940157367164925, -3.3526198053443457]], [[-3.2309921404564248, 1.2887509864310598], [-0.4246882434427004, -3.407485191400077]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.093852048625794104, -4.4096668933072483], [7.7210047697055284, 5.4509430123795912]], [[-7.9034656651987838, 1.5939456791534696], [0.43230308654003213, -3.6874700123364881]], [[-3.1999719860377929, 2.3046882298838627], [0.70363810446525887, -7.7378229457943277]], [[0.074780689852723903, 4.9825871235417107], [-7.7565782404810495, -1.8049678248903658]], [[-2.1410801421698231, -3.2040053997479676], [-7.4148637492877265, 4.9574453088331873]], [[4.4412412498343823, 2.6035083267982131], [-5.7210962828246181, -1.9313447582520862]]])+(1.-msk_ref)*numpy.array([[[3.2498658791242354, 1.3855239136079107], [2.520360689962728, 1.7996578507057075]], [[-3.4122321009125676, 0.45439192499197567], [-1.591388464783166, 1.3861179815885953]], [[-6.4592043426241981, 3.5462649999472582], [2.9039662978720875, -8.8555337636354281]], [[2.0180605245010463, -2.8994918499824625], [-3.2232458515686373, -1.3830163067284809]], [[2.4428468323118206, -2.3449737826681574], [-8.1819712637937592, -1.6679964413643411]], [[1.3564558834129974, 0.8710607332172895], [-2.3741907209778192, -5.2286378584020614]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[-4.9407783252525679, -0.14424175273414352], [-4.3545195599278514, -3.8686580478958801]], [[1.7425577670443015, -0.3951190268107494], [-4.4907386430361367, -2.8108131631641573]]], [[[-0.075099569244216902, 4.0887904658084828], [4.0463519265595576, 1.6192761630580961]], [[-3.8737287658098305, -4.7394259307583457], [0.368153406977477, -4.9483556236364681]]], [[[-3.5683005037082918, 1.4242049761970623], [2.620195527695123, -3.1477482580412031]], [[1.0876020960683119, -4.2996264212844961], [-3.8001454401025048, -4.4701199797760038]]], [[[-1.3588645993283164, 4.8085871551782908], [-3.0252709580280204, 1.4664447535918868]], [[-0.71884429318254739, 1.9742312519676783], [-2.0019765595787309, -1.4305495117255895]]], [[[-0.79634063020349899, -2.9541777967788541], [-2.9373302340336336, -1.7517914190720143]], [[2.4077852343230512, -0.073969092729551988], [-3.6667125240402854, 0.051769693949651696]]], [[[3.8126167457681692, -2.498143046851359], [-3.7396731275215025, -3.6902480052146291]], [[-2.0919733809295615, -3.5413220908795662], [1.7033636008269184, -3.8210183386230776]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.5646817902965502, 4.9273036898982898], [4.2290469849042474, 1.12758062327811]], [[-2.5527072270238973, 4.1126506110498919], [3.5924661996207679, 4.671252371793571]], [[-1.271502866785621, -0.73601200052148918], [-1.4008452590472564, 0.93037965731568129]], [[-3.0366057208944284, 2.8810293366365292], [-2.3963386585886459, 3.3054634301136492]], [[4.6856079925384773, -3.1338376645210397], [-3.8675242552947253, -0.88569683161757151]], [[4.0868360753748139, 3.4985448419039606], [4.0703207230620038, 4.7080190182159338]]])+(1.-msk_arg1)*numpy.array([[[-0.43354560971988843, 2.203513193832034], [4.5311103625538713, -4.5210537726922606]], [[0.22190220196093602, 3.5795464554700391], [3.4700992167327964, 2.5745888379826649]], [[4.9657509544094811, 4.9756270326869014], [3.0505771159862487, -0.65990862154152197]], [[1.9462425674708861, -3.3274330761441107], [-2.4987428941635539, 4.2222761100664048]], [[4.1515445645442064, 4.7466161741612929], [1.9172116407644673, -2.4969930283197348]], [[-3.8337672241687439, -2.1509820304718339], [4.4256521897855627, 1.485636682620064]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-9.5054601155491181, -4.7089235430306937], [0.57278412997043837, 1.0586456420024097]], [[5.9716047519485489, 3.833927958093498], [-3.3631580197580266, -1.6832325398860473]]], [[[-2.6278067962681142, 1.5360832387845855], [8.1590025376094495, 5.731926774107988]], [[-0.28126256618906265, -1.1469597311375779], [5.039405778771048, -0.27710325184289708]]], [[[-4.8398033704939127, 0.15270210941144136], [1.8841835271736338, -3.8837602585626922]], [[-0.3132431629789445, -5.7004716803317521], [-2.8697657827868235, -3.5397403224603226]]], [[[-4.3954703202227448, 1.7719814342838625], [-0.14424162139149121, 4.3474740902284159]], [[-3.1151829517711933, -0.42210740662096757], [1.3034868705349183, 1.8749139183880597]]], [[[3.8892673623349783, 1.7314301957596232], [-6.0711678985546733, -4.885629083593054]], [[-1.4597390209716741, -3.9414933480242773], [-4.5524093556578569, -0.83392713766791982]]], [[[7.8994528211429831, 1.5886930285234548], [-0.24112828561754185, -0.1917031633106685]], [[1.9783473421324422, 0.52899863218243759], [6.4113826190428522, 0.88700067959285622]]]])+(1.-msk_ref)*numpy.array([[[[-5.3743239349724563, -0.57778736245403195], [-2.1510063660958174, -1.665144854063846]], [[6.2736681295981729, 4.1359913357431219], [-9.0117924157283973, -7.3318669358564179]]], [[[0.14680263271671912, 4.3106926677694188], [7.6258983820295967, 5.1988226185281352]], [[-0.40362954907703408, -1.2693267140255493], [2.9427422449601419, -2.3737667856538032]]], [[[1.3974504507011893, 6.3899559306065434], [7.5958225603820244, 1.8278787746456984]], [[4.1381792120545606, -1.2490493052982474], [-4.4600540616440263, -5.1300286013175258]]], [[[0.58737796814256971, 6.7548297226491769], [-6.352704034172131, -1.8609883225522239]], [[-3.2175871873461013, -0.52451164219587554], [2.2202995504876739, 2.7917265983408153]]], [[[3.3552039343407074, 1.1973667677653523], [1.8092859401276593, 2.9948247550892786]], [[4.3249968750875185, 1.8432425480349153], [-6.1637055523600202, -2.4452233343700831]]], [[[-0.021150478400574713, -6.331910271020103], [-5.8906551579933364, -5.841230035686463]], [[2.3336788088560012, 0.88433009890599656], [3.1890002834469824, -2.3353816560030136]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_expandedData_rank4(self):
      arg0=numpy.array([[[1.6447569414674419, -2.295741962205089], [-4.4918773354351069, -1.117543037315889]], [[0.22242231036159499, -4.9843709339327491], [2.840388974918354, 4.177744599319503]], [[-3.4444892787723704, 2.9933296151998032], [-0.66201237626009135, -2.0626589985640842]], [[-3.7665124492090305, -3.8979520569483128], [2.0377095057369718, 4.5134080679875197]], [[3.333392922725448, -0.26835657169502625], [2.7936687440567054, 0.97462816778735917]], [[-4.5664305446874129, 4.3665433206034443], [4.9925063869548119, -3.2696297286936868]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.495269447859112, 2.7654595684037631], [-4.9765388350565223, -1.2954371863293455]], [[-4.8947169508126578, -4.9344512995588339], [1.8022463688570802, 4.9714230490991937]]], [[[-3.1726945494926193, 0.99429036690305495], [3.0585880933949685, 3.2228624860664929]], [[2.9526371849861812, -3.4645793920483037], [1.5379847965451106, 1.9091770771415337]]], [[[4.8023312167673957, -1.2828804187588947], [-0.75098071787032428, -2.8796733949564768]], [[4.1882100081938329, 4.5151319571256092], [-3.4801719247528484, -0.58792383355739553]]], [[[3.0768780499284798, 3.3596018898371902], [-2.0710167648810107, -2.5528256024032983]], [[-1.9664819469846826, -4.9714629175255158], [1.6036810330010702, -3.0681899746855423]]], [[[-1.1852006495493139, -1.1577391558152508], [2.4670740963509008, 0.70137149893834927]], [[2.586505815921269, -4.0397107103693415], [4.2340379948111657, 4.925121844410727]]], [[[2.4285686471336092, 2.4948787312822764], [4.8340041893923651, -3.3450925741147652]], [[-1.6849049506279998, 3.732084286015585], [-4.3741054237491941, -2.7114155090528689]]]])+(1.-msk_arg1)*numpy.array([[[[-2.8898509517482474, 0.29683381658014341], [-2.025778302704059, -2.9469703671691461]], [[4.6132869291444507, 4.2021757162541267], [2.1395104897466517, 3.8373392002531261]]], [[[-4.8990320041550186, 0.59800884529124332], [-4.2344413843488056, -3.6241637896220666]], [[-3.9391121283338992, -1.8747478189517652], [3.6139365794079481, 2.5754991237061295]]], [[[0.74476531228321541, 4.8708573977043574], [-4.9320243704635001, 3.5937988557580027]], [[-2.4887064045311047, 1.1840827469462099], [-3.6464312324028136, -3.2552817297364669]]], [[[-3.5376885308626385, 3.1773930152579126], [-3.7138405324823944, -3.8622953232719706]], [[-0.82923184919801329, 0.98497533478601618], [-2.2313144363203063, -0.64271683560391679]]], [[[4.1883295221464927, -3.4093823106104493], [3.005788842150432, -4.5637066225676666]], [[0.12899572918106195, 2.8296456854213279], [3.2210066812486176, 0.82167173524936743]]], [[[3.685777860942185, 0.84168667477981884], [3.8171812489859587, 0.19305678111132796]], [[-0.61120955540878974, -2.5868103080909677], [-0.3258396007000659, -4.4218800769744906]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.1400263893265539, 4.4102165098712049], [-7.2722807972616117, -3.5911791485344344]], [[-9.3865942862477638, -9.4263286349939399], [0.68470333154119123, 3.8538800117833047]]], [[[-2.9502722391310243, 1.2167126772646499], [-1.9257828405377806, -1.7615084478662562]], [[5.7930261599045352, -0.62419041712994972], [5.7157293958646136, 6.0869216764610368]]], [[[1.3578419379950253, -4.7273696975312651], [2.242348897329479, 0.11365622024332644]], [[3.5261976319337416, 3.8531195808655179], [-5.5428309233169326, -2.6505828321214797]]], [[[-0.6896343992805507, -0.40691055937184029], [-5.9689688218293231, -6.4507776593516111]], [[0.071227558752289255, -2.933753411788544], [6.1170891009885899, 1.4452180933019774]]], [[[2.148192273176134, 2.1756537669101972], [2.1987175246558746, 0.43301492724332302]], [[5.3801745599779744, -1.2460419663126361], [5.2086661625985249, 5.8997500121980861]]], [[[-2.1378618975538037, -2.0715518134051365], [9.2005475099958094, 1.0214507464886791]], [[3.307601436326812, 8.7245906729703968], [-7.6437351524428809, -5.9810452377465557]]]])+(1.-msk_ref)*numpy.array([[[[-1.2450940102808055, 1.9415907580475853], [-4.3215202649091484, -5.2427123293742355]], [[0.12140959370934379, -0.28970161918098025], [1.0219674524307627, 2.7197961629372371]]], [[[-4.6766096937934236, 0.82043115565283831], [-9.2188123182815538, -8.6085347235548149]], [[-1.0987231534155453, 0.96564115596658873], [7.7916811787274511, 6.7532437230256326]]], [[[-2.699723966489155, 1.426368118931987], [-1.9386947552636968, 6.587128470957806]], [[-3.150718780791196, 0.5220703706861185], [-5.7090902309668978, -5.3179407283005506]]], [[[-7.304200980071669, -0.58911943395111788], [-7.6117925894307072, -7.7602473802202834]], [[1.2084776565389586, 3.022684840522988], [2.2820936316672134, 3.8706912323836029]]], [[[7.5217224448719406, -0.075989387885001314], [2.7374322704554057, -4.8320631942626928]], [[2.9226644732377673, 5.6233144294780333], [4.1956348490359767, 1.7962999030367266]]], [[[-0.88065268374522798, -3.7247438699075941], [8.183724569589403, 4.5596001017147723]], [[4.3812968315460221, 2.4056960788638442], [-3.5954693293937527, -7.6915098056681774]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[3.8701602651161036, 0.50954964199084429, 0.14927991787008033, 3.8304531128781356], [0.33259539048517972, -3.7833046556467655, 0.035526344471827898, 1.8348337929517093], [2.5868695080397783, -1.9246505865067123, -0.49354223687480392, -0.19832702078029474]], [[-0.72353412669570361, -1.0668302657559559, 3.4654805163762656, 2.3061681210365812], [1.694566709744123, 1.9286491626770141, -2.5233389663167669, 3.0899581215590537], [-3.970137201013185, 1.5568924279289984, 1.4395758813617192, -1.9701575573450336]]], [[[-1.4470293282799318, 2.02451686276593, -3.4201230576769923, -3.7782038777554794], [1.9882285788729952, -2.4087997248203874, -4.7447612087758699, 4.4672321626999825], [1.1220620996866524, 2.7182869627297102, 2.3768075937825541, -0.33229877862627966]], [[-2.4876593831981761, 2.0667288652601945, -4.8122963541660235, 4.0825599581113199], [4.6001709097319061, 1.2493958367453173, -2.6223621221638203, 0.23566054389351088], [-2.692500319789346, 0.75153688742200053, -4.038244909677652, -2.7168803189479238]]], [[[0.20281369635485902, -2.8852975888952237, 3.7903051534502623, 2.0164483112690492], [3.9718489459713719, -2.4596169830847883, 4.5938107151184422, -1.4579726113669498], [-1.8294860814782243, -2.6260347239187523, -0.89928097830869191, -2.9344245904752841]], [[3.1856771530383092, 2.1006802171585512, 2.7777127242949682, -4.7948374736610848], [1.3702005265584472, 4.1140671374807631, 3.5571312712910661, 1.7653426972037147], [-2.7291753084854986, 2.3787176604532263, 0.99912592180664461, 0.8536168145113523]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.19451709343487344, 4.5770920061724425, 3.9982481567361443, -4.8324775464974934], [-4.4424010855469938, 4.660021459194617, 2.7907052917064874, 2.0841616275536978], [1.2643254581057937, 0.85741333096824501, 2.638113009897789, -3.5245227608115179]], [[-4.4836080793567579, -0.11015320542279028, 4.1594257006116493, 1.0410690287410471], [0.87040137116893934, 4.323983038262055, 1.3601383323555902, -4.5317588590343423], [1.7584107552472394, -3.0314584466484487, -3.7291315270834802, 3.1510171950419696]]], [[[-4.6806958034552366, -1.2409105227816797, 2.3513942110696897, 4.8064770111648123], [-2.3524966297819514, 0.12269876313284467, -2.3701835738065826, -2.2337147927953529], [-3.3956189992612473, -0.92273080184294809, -1.5320087959523541, 4.986655608269178]], [[-3.6998789138279942, 2.1386465586353918, 1.2164710159571204, -2.4316606527248865], [-1.4253998012169466, -3.2797061876075806, 4.8083422177576161, 0.99224066653165011], [-3.9706006155765361, 0.36813694034108746, 0.28276190740001628, -1.7560069603048802]]], [[[3.3217394484686835, -3.1355341140328399, 1.661291894389616, 2.8749869677018616], [-0.89843478897567852, -4.3427548797086022, -4.8139331798245122, -1.645309268535339], [-2.4120453436444587, -2.784280578486733, -3.3745874341942206, -2.7294071093365737]], [[4.7293838097464338, 4.2493105808668492, 0.078130064194308346, 2.0429707371804229], [0.20388175224661964, 3.8976324051359157, -3.1755428239050829, 1.856332284122189], [1.4369223357822598, 4.8588285245020604, -0.69057026167179281, 0.36403275208727415]]]])+(1.-msk_arg1)*numpy.array([[[[-0.019223712678879679, 1.2422438235294369, 3.6133261429663186, -3.7010108183348445], [-2.120518555150781, -4.1797812154401228, 1.1817851388964185, -0.28342945311600154], [4.3945006436640472, -4.6519171501963807, 1.6160360255094695, 0.60450625306693784]], [[0.46946850966075004, 2.8386655084782948, 4.6205973652228423, 0.96521459071841775], [-0.64895885949998533, 3.1638030004227087, -3.2330365356719604, -2.9267968341886941], [3.3833589324980444, 0.65708965742753023, -0.46058077646507822, 1.1456370657907371]]], [[[2.8272979670317167, -1.1620748513547561, 4.5553658284971768, -3.0942301160451926], [1.676406775354172, 1.5887542545161271, -1.5146700120769383, -1.9702116218150314], [-4.7735596378561347, 0.96071606740066784, 3.6460892339361735, -1.554957193860953]], [[4.0757752486417118, 0.89683105739400126, -3.7338420636609735, -4.6712570163714133], [-2.6311215379972519, 2.7802461774958243, -0.67393826548219593, 2.7995374296888151], [1.331346314062066, -3.3541504268733591, -4.6391574246711542, -4.2657907093067191]]], [[[-1.3661806436946975, 3.9776215882015205, 1.4769421191959093, -0.9036573773305312], [-0.9342176815950145, -3.2425416043463295, -0.98767709855303565, 1.2989744811283881], [-3.5278274982944868, -4.1100196578702786, -1.0045526792631074, 3.9948432403598275]], [[3.4405356131441085, -3.8784797377266278, -0.45465127512168202, 0.0048613446554579554], [-4.5920188002740296, 1.6570693489149875, -1.5488639816865621, 4.8018268016772065], [-2.8180587424375347, -0.19837413133840709, 4.2287325946838159, 3.7707090228362556]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.6756431716812301, 5.0866416481632868, 4.1475280746062246, -1.0020244336193578], [-4.109805695061814, 0.87671680354785142, 2.8262316361783153, 3.9189954205054072], [3.851194966145572, -1.0672372555384673, 2.144570773022985, -3.7228497815918127]], [[-5.2071422060524615, -1.1769834711787461, 7.6249062169879149, 3.3472371497776283], [2.5649680809130624, 6.2526322009390691, -1.1632006339611767, -1.4418007374752886], [-2.2117264457659456, -1.4745660187194503, -2.289555645721761, 1.180859637696936]]], [[[-6.1277251317351684, 0.78360633998425033, -1.0687288466073026, 1.0282731334093329], [-0.3642680509089562, -2.2861009616875427, -7.1149447825824526, 2.2335173699046296], [-2.2735568995745949, 1.7955561608867621, 0.84479879783020007, 4.6543568296428983]], [[-6.1875382970261708, 4.2053754238955863, -3.5958253382089032, 1.6508993053864334], [3.1747711085149595, -2.0303103508622633, 2.1859800955937958, 1.227901210425161], [-6.6631009353658825, 1.119673827763088, -3.7554830022776358, -4.472887279252804]]], [[[3.5245531448235425, -6.0208317029280636, 5.4515970478398783, 4.8914352789709108], [3.0734141569956934, -6.8023718627933905, -0.22012246470607, -3.1032818799022888], [-4.241531425122683, -5.4103153024054853, -4.2738684125029121, -5.6638316998118583]], [[7.915060962784743, 6.3499907980254005, 2.8558427884892765, -2.7518667364806619], [1.5740822788050668, 8.0116995426166788, 0.38158844738598319, 3.6216749813259037], [-1.2922529727032388, 7.2375461849552867, 0.30855566013485181, 1.2176495665986264]]]])+(1.-msk_ref)*numpy.array([[[[3.8509365524372239, 1.7517934655202811, 3.7626060608363989, 0.12944229454329115], [-1.7879231646656013, -7.9630858710868884, 1.2173114833682463, 1.5514043398357078], [6.9813701517038256, -6.576567736703093, 1.1224937886346655, 0.4061792322866431]], [[-0.25406561703495356, 1.7718352427223389, 8.0860778815991079, 3.271382711754999], [1.0456078502441377, 5.0924521630997228, -5.7563755019887273, 0.16316128737035962], [-0.58677826851514059, 2.2139820853565286, 0.97899510489664099, -0.82452049155429652]]], [[[1.3802686387517848, 0.86244201141117394, 1.1352427708201844, -6.872433993800672], [3.6646353542271672, -0.82004547030426034, -6.2594312208528082, 2.4970205408849511], [-3.6514975381694823, 3.679003030130378, 6.0228968277187276, -1.8872559724872326]], [[1.5881158654435357, 2.9635599226541958, -8.546138417826997, -0.58869705826009344], [1.9690493717346542, 4.0296420142411415, -3.2963003876460162, 3.035197973582326], [-1.36115400572728, -2.6026135394513585, -8.6774023343488054, -6.9826710282546429]]], [[[-1.1633669473398385, 1.0923239993062968, 5.2672472726461717, 1.112790933938518], [3.0376312643763574, -5.7021585874311178, 3.6061336165654065, -0.15899813023856169], [-5.3573135797727112, -6.7360543817890308, -1.9038336575717993, 1.0604186498845434]], [[6.6262127661824177, -1.7777995205680766, 2.3230614491732862, -4.7899761290056269], [-3.2218182737155825, 5.7711364863957506, 2.008267289604504, 6.5671694988809213], [-5.5472340509230333, 2.1803435291148192, 5.2278585164904605, 4.6243258373476079]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(-4.47192871421)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-0.994656650032)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-5.46658536424)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([4.7316220993229479, -0.27418741243809208])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(3.25381102268)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([7.9854331220060413, 2.9796236102450013])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(3.94200915762)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([0.13513554926292048, 4.061857308270362])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([4.0771447068818789, 8.0038664658893204])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[-0.58020763927283259, 0.60485804924583864, 4.9259644719532787, -2.1486954063895256, 2.7934642190283299], [1.6822159521784155, 2.7483124573452136, -2.8004331565202198, 2.9438221066670476, -2.5864136655771022], [0.9101345547641273, 4.8357937536703144, -3.5367998859469263, -2.0694611786669439, 4.2626165752161072], [-4.5220527903149641, -3.1964999090581756, 0.23269630643061756, -0.0072734125635163238, -1.3970124456729383]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(4.90605848579)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[4.3258508465184553, 5.5109165350371265, 9.8320229577445666, 2.7573630794017623, 7.6995227048196178], [6.5882744379697034, 7.6543709431365015, 2.105625329271068, 7.8498805924583355, 2.3196448202141857], [5.8161930405554152, 9.7418522394616023, 1.3692585998443616, 2.836597307124344, 9.168675061007395], [0.38400569547632379, 1.7095585767331123, 5.1387547922219055, 4.8987850732277716, 3.5090460401183496]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(-2.12053457399)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[4.5173353010468951, 2.8786652805833732, 1.5955340829923781, -2.4130959114722836, -0.29040785737722885], [0.66619216479382537, -3.2916227754910485, -4.6826358866854756, 0.089824275062710512, -1.2938807246062387], [-3.9333778300471698, -1.2006085122827246, -1.1255115122552182, 3.9788209630774816, 2.804202030760913], [-3.3592030812168638, -2.1487020184632986, 3.4730435821484846, 3.7132268171008533, 2.5541706104285797]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[2.3968007270581313, 0.75813070659460946, -0.52500049099638568, -4.5336304854610479, -2.4109424313659926], [-1.4543424091949384, -5.4121573494798128, -6.8031704606742398, -2.0307102989260533, -3.4144152985950025], [-6.053912404035934, -3.3211430862714884, -3.2460460862439819, 1.8582863890887178, 0.68366745677214924], [-5.4797376552056276, -4.2692365924520619, 1.3525090081597209, 1.5926922431120896, 0.43363603643981596]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[-4.4879082778629362, 2.0145869612406209], [-4.750338221649379, -2.6471533445484652]], [[3.9552770481468684, 0.43581580074175807], [0.96651350692065741, 4.558178090539327]], [[4.8565657558103599, -1.7643683387935374], [-0.18162060180462625, -0.66785288476699201]], [[1.0600037763615493, 0.56388130965752215], [0.8144263422850333, -3.9829827764512329]], [[0.32623128638073062, 2.7300470107894235], [1.1222062602809633, -4.2672726278791071]], [[-4.7220859231675529, 4.7273220596197199], [2.7956687654768579, 1.5839867175852351]]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-2.97897452242)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-7.4668828002812822, -0.96438756117772506], [-7.7293127440677249, -5.6261278669668116]], [[0.9763025257285225, -2.5431587216765879], [-2.0124610154976885, 1.5792035681209811]], [[1.8775912333920139, -4.7433428612118833], [-3.1605951242229722, -3.6468274071853379]], [[-1.9189707460567966, -2.4150932127608238], [-2.1645481801333126, -6.9619572988695788]], [[-2.6527432360376153, -0.24892751162892246], [-1.8567682621373827, -7.246247150297453]], [[-7.7010604455858989, 1.7483475372013739], [-0.18330575694148799, -1.3949878048331108]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(-4.92133080405)
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[0.95903449289262532, 3.7437049510086489], [-2.3956024251720267, -0.035245394753246728]], [[-4.2437958706161911, -0.58265590616375817], [1.2351849723133723, -4.1373689721945093]], [[-1.4305416766926315, -2.1373379401123351], [3.6312390173511844, -3.5168703780930608]], [[0.36020416982131209, 1.6021724103095645], [-1.8836798787082696, 0.64275987238325527]], [[-2.0056220539118517, 4.6743286344238015], [3.8406955144030235, -2.9955028153914052]], [[2.4707264574997145, -3.0682952726235344], [2.7023797918905599, -1.1769499691065031]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-3.9622963111600216, -1.177625853043998], [-7.3169332292246736, -4.9565761988058936]], [[-9.1651266746688371, -5.5039867102164051], [-3.6861458317392746, -9.0586997762471562]], [[-6.3518724807452784, -7.058668744164982], [-1.2900917867014625, -8.4382011821457077]], [[-4.5611266342313348, -3.3191583937430824], [-6.8050106827609165, -4.2785709316693916]], [[-6.9269528579644986, -0.24700216962884536], [-1.0806352896496234, -7.9168336194440521]], [[-2.4506043465529324, -7.9896260766761813], [-2.218951012162087, -6.09828077315915]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[-2.1684001485832169, -3.5186002014949889, 3.7752339603636997, -1.1270000583921269], [0.11421089471583379, 0.35901104666661965, -0.099075796925709447, -4.5647327211967106], [2.8474512667203129, 2.438315809600371, -2.6880207289695246, -3.1120819526497057]], [[-3.3885278098539007, -1.1376867855499539, -0.070416457621735873, 1.7716814376742338], [-0.53493321780572245, -1.0586964778293972, -0.4763495204744741, -1.959825133838252], [4.7071177058951079, -4.83300370279352, -2.1345435881475918, -2.1988009350515836]]], [[[1.8572655293025875, -2.3219143378976281, -1.9191965844471905, -1.9386724086241669], [-3.7568311676263679, -1.3408781247958568, -2.0274612677761947, 4.5258322341407329], [-3.3710673019386537, 1.0499532961858327, 1.3365257382423321, 1.6025964786299518]], [[0.85865194535353062, 1.2931837175747587, 3.8713433776769186, -4.5057060301320053], [0.50803390563299544, -4.139611667223317, -1.0606135982252249, 1.6381680046585707], [1.5521090287337636, 0.7432625105586288, 2.5728730262335953, 2.395569688988564]]], [[[4.7254064937235576, 2.0448090508646066, -3.2047560663308894, 2.6551638357714165], [-2.9015569883827363, 0.52367359249619838, 4.9722855094471541, 2.9648418410244872], [-4.5387514761914183, 0.50851788554586896, 4.1782047899123178, 2.7584286680815246]], [[2.3215147708723771, -2.4873449202922715, 3.010610962740607, 4.979346540239165], [4.5346771103702608, 2.6772492959587444, -0.7030932772280627, 2.7813716597928986], [3.2804161471466848, -1.2844651891456782, 0.92421359345383358, 2.860413736050508]]]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(0.62299305591)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-1.5454070926729786, -2.8956071455847505, 4.3982270162739381, -0.50400700248188857], [0.73720395062607214, 0.982004102576858, 0.5239172589845289, -3.9417396652864722], [3.4704443226305512, 3.0613088655106093, -2.0650276730592863, -2.4890888967394673]], [[-2.7655347539436623, -0.5146937296397156, 0.55257659828850247, 2.3946744935844722], [0.088059838104515897, -0.4357034219191589, 0.14664353543576425, -1.3368320779280136], [5.3301107618053463, -4.2100106468832816, -1.5115505322373535, -1.5758078791413452]]], [[[2.4802585852128258, -1.6989212819873898, -1.2962035285369522, -1.3156793527139286], [-3.1338381117161296, -0.71788506888561843, -1.4044682118659564, 5.1488252900509712], [-2.7480742460284153, 1.672946352096071, 1.9595187941525705, 2.2255895345401902]], [[1.481645001263769, 1.916176773484997, 4.494336433587157, -3.882712974221767], [1.1310269615432338, -3.5166186113130786, -0.43762054231498659, 2.2611610605688091], [2.1751020846440019, 1.3662555664688671, 3.1958660821438336, 3.0185627448988024]]], [[[5.3483995496337959, 2.667802106774845, -2.581763010420651, 3.2781568916816548], [-2.278563932472498, 1.1466666484064367, 5.5952785653573924, 3.5878348969347256], [-3.9157584202811799, 1.1315109414561073, 4.8011978458225562, 3.381421723991763]], [[2.9445078267826155, -1.8643518643820332, 3.6336040186508454, 5.6023395961494034], [5.1576701662804991, 3.3002423518689827, -0.080100221317824349, 3.4043647157031369], [3.9034092030569232, -0.66147213323543985, 1.5472066493640719, 3.4834067919607463]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(2.99234094925)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[4.7241807029377529, 4.9180200729360006, 2.6145909450537888, -3.1942253790494979], [3.873123713204107, -2.3553971137256093, -2.5554753910992742, 3.2468355714617783], [4.5555976185678535, -2.6958254969593143, 4.6437170879391321, -1.0669870222183953]], [[1.2792554964762086, 0.018734621530476225, 0.45476410237351672, -4.2975062664061667], [0.1947752051679803, -0.91550441874675226, -0.034448691704223044, -0.62324546070114728], [-0.38171044408996124, -2.2411458426085282, 4.8644596571028629, -0.46297198956930163]]], [[[2.8534023877959678, -0.79646773185160136, 3.7750307462560677, 0.056210430436495429], [-1.4633680392656121, 1.7735620399325782, 3.1076940590056772, 1.7422077934761688], [-4.5816290172464846, -3.7820081082963055, -2.3340070811956739, 2.3196325840198053]], [[-2.2129543447787459, -4.6993783438507091, 0.29355064915731166, 3.2331716075706751], [-4.6920367622909867, 1.8238549877119858, -0.80551106479887391, -4.2373146494303651], [0.86632027501163389, -0.33401471830356222, -2.7197267521797661, 0.66953529839336579]]], [[[-3.8325786863333966, -3.3984990729420019, 2.2585518107109781, 2.5022674619634833], [2.4362828414707636, -1.3726204486977531, -2.3842334334219064, -4.7592595486772122], [-3.8931122097727879, 4.2845363357314081, 4.9453770204798086, -2.9382194451126442]], [[3.7856864952926674, -4.5965940569678203, 4.8946037799392847, -4.0725015082434499], [-4.4252578403612262, -1.674387972658943, 3.9488127435751785, -2.90880010605182], [2.5131353596509234, -3.9108057679057695, 0.33156050778387147, 0.98916442129308368]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[7.7165216521833004, 7.9103610221815481, 5.6069318942993362, -0.20188442980395038], [6.8654646624496545, 0.63694383551993816, 0.43686555814627326, 6.2391765207073258], [7.547938567813401, 0.29651545228623322, 7.6360580371846796, 1.9253539270271522]], [[4.2715964457217561, 3.0110755707760237, 3.4471050516190642, -1.3051653171606192], [3.1871161544135278, 2.0768365304987952, 2.9578922575413245, 2.3690954885444002], [2.6106305051555863, 0.75119510663701927, 7.8568006063484104, 2.5293689596762459]]], [[[5.8457433370415153, 2.1958732173939461, 6.7673716955016152, 3.0485513796820429], [1.5289729099799354, 4.7659029891781257, 6.1000350082512247, 4.7345487427217163], [-1.5892880680009371, -0.78966715905075802, 0.65833386804987359, 5.3119735332653528]], [[0.77938660446680164, -1.7070373946051616, 3.2858915984028592, 6.2255125568162226], [-1.6996958130454392, 4.8161959369575333, 2.1868298844466736, -1.2449737001848176], [3.8586612242571814, 2.6583262309419853, 0.27261419706578138, 3.6618762476389133]]], [[[-0.8402377370878491, -0.40615812369645443, 5.2508927599565256, 5.4946084112090308], [5.4286237907163111, 1.6197205005477944, 0.60810751582364109, -1.7669185994316647], [-0.90077126052724044, 7.2768772849769556, 7.9377179697253561, 0.054121504132903286]], [[6.7780274445382149, -1.6042531077222728, 7.8869447291848322, -1.0801605589979024], [-1.4329168911156787, 1.3179529765866045, 6.941153692820726, 0.083540843193727454], [5.5054763088964709, -0.918464818660222, 3.323901457029419, 3.9815053705386312]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([2.0080424168683706, -1.4106062224871319])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([4.089682963577344, -2.1353135579226556])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([6.0977253804457145, -3.5459197804097875])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-4.4343561247068672, 4.5962156611491469], [-1.2310006237532067, 0.031728215970050044]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-1.6633775870468437, 2.9510116445604648])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-6.0977337117537109, 2.9328380741023032], [1.7200110208072581, 2.9827398605305149]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_Symbol_rank2(self):
      arg0=numpy.array([-1.7162225282162846, 0.026009389173900388])
      arg1=Symbol(shape=(2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[-0.49946042170325455, -4.1771545948935005], [1.001086947155633, -1.7197357066668664]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-2.2156829499195392, -5.8933771231097847], [1.0270963363295333, -1.693726317492966]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_Symbol_rank1(self):
      arg0=numpy.array([[[-4.3559963026363571, 3.8081860473505262, -3.8407484791905357, 3.1619946963765386, 1.9525874502600518], [-3.7839573226004952, 3.825522902107064, -4.9544993149776229, -1.6273721541056139, -1.8566113866753584], [-4.0943308765072484, -3.1363936106352353, 2.9725263187462216, -3.2444639785579299, -1.0313428834711336], [-2.9826576633065316, 1.3591537008880312, -1.9035915510608925, -2.0003738856910038, 4.1850553918003257]], [[-1.2537424272943207, 4.2817501115606404, -0.52878779708227963, -1.562385820909904, 0.57463065284796677], [-1.2014917384555335, 0.30181126561790617, 1.7840973469398067, -4.3255227493518564, 3.1331683347411907], [-3.8599282191458983, 2.6565668235767479, -0.51462292601794157, 1.6963812889953269, -0.089028436324198879], [2.7902103277412849, -2.3244467831724815, -0.73715284287300342, -3.7594695822423665, -2.4831142698697519]]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-2.6502074887692748, -0.52879673008931594])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-7.0062037914056319, 1.1579785585812514, -6.4909559679598106, 0.51178720760726382, -0.69762003850922305], [-6.4341648113697705, 1.1753154133377892, -7.6047068037468977, -4.2775796428748887, -4.5068188754446332], [-6.7445383652765232, -5.7866010994045105, 0.32231882997694683, -5.8946714673272051, -3.6815503722404084], [-5.6328651520758068, -1.2910537878812436, -4.5537990398301673, -4.6505813744602786, 1.5348479030310509]], [[-1.7825391573836367, 3.7529533814713245, -1.0575845271715956, -2.0911825509992199, 0.045833922758650836], [-1.7302884685448494, -0.22698546447140977, 1.2553006168504908, -4.8543194794411724, 2.6043716046518748], [-4.3887249492352147, 2.127770093487432, -1.0434196561072575, 1.167584558906011, -0.61782516641351481], [2.2614135976519689, -2.8532435132617975, -1.2659495729623194, -4.2882663123316824, -3.0119109999590679]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_Symbol_rank3(self):
      arg0=numpy.array([3.817542641683362, 1.911113282165199])
      arg1=Symbol(shape=(2, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[-0.63686024364535321, -3.26122224433959, 3.7479556940852259, -3.4215245141585071, -1.3436423241472206], [-4.3162668777656163, -2.8061909664722684, -3.3409660678881323, 4.963072489449079, -4.8176286476361359], [0.7789757441616354, 1.4084962712036262, 3.0023376159093615, 2.4265732350665754, -1.2844664003911133], [0.22336827192255804, -3.7826321622486816, 1.0066887951387207, -3.6264867717868086, -0.18349025180016021]], [[3.9609122483685937, 0.24763515902160727, -3.9945237538493714, 4.7083848328520528, -2.1772419444383018], [-1.148300749257066, 1.9278068860079127, 4.436663969577733, 1.7903291128122367, 2.3978636425539124], [-1.4820392655749548, -1.2226741849594314, -2.134349647188809, 1.9708775918216404, 2.2691609750125714], [-2.3307845661593953, -3.7536087817289654, 3.5273386119244439, 2.9006811242446426, -4.7277224734326868]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[3.1806823980380088, 0.55632039734377203, 7.5654983357685879, 0.39601812752485488, 2.4739003175361414], [-0.49872423608225436, 1.0113516752110936, 0.47657657379522966, 8.780615131132441, -1.0000860059527739], [4.5965183858449974, 5.2260389128869882, 6.8198802575927235, 6.2441158767499374, 2.5330762412922487], [4.04091091360592, 0.034910479434680397, 4.8242314368220827, 0.19105586989655343, 3.6340523898832018]], [[5.8720255305337927, 2.1587484411868063, -2.0834104716841724, 6.6194981150172518, -0.26612866227310272], [0.762812532908133, 3.8389201681731118, 6.3477772517429321, 3.7014423949774358, 4.3089769247191114], [0.42907401659024424, 0.68843909720576768, -0.22323636502361, 3.8819908739868394, 4.1802742571777705], [-0.41967128399419629, -1.8424954995637663, 5.4384518940896429, 4.8117944064098417, -2.8166091912674878]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_Symbol_rank1(self):
      arg0=numpy.array([[[[-1.09745670005718, 3.9834117147310195], [4.1443000180165335, -4.8604174344880304]], [[4.6492768420284527, 2.1750561243036071], [-1.8022015377541889, 2.9354401288132959]], [[-4.30216656736647, 2.8029944908340365], [2.0241372398953121, 1.8602110122459292]], [[3.5296871922141086, -2.0648821962424946], [-3.986952234613117, -1.57294401233119]], [[4.385908084812284, -4.4633401541743822], [-0.5272732505634341, -2.2006203397311808]], [[1.6203392720654142, -3.6292714112378932], [0.050019676467166541, 3.8034183605744314]]], [[[3.9571875122422622, -4.693539557472298], [0.30894917827554025, -4.6939266623172839]], [[4.6507611949368961, 3.320441500221067], [-2.041539846932694, -1.3198761481919985]], [[0.14021374776337403, 1.7977419326196289], [-0.52943153391919484, -3.6515006971998956]], [[-3.4511764469532724, 3.8889961284293371], [1.2964786565805921, 2.1334807614403068]], [[-3.2306512521618935, -4.0287721255399873], [-3.8122669063237247, -1.9447612492595558]], [[-0.31082510487309989, 2.3874680860143425], [2.5194292845479493, 1.9034861587369587]]]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-2.6638757865440388, -1.4429119028360149])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-3.7613324866012188, 1.3195359281869807], [1.4804242314724947, -7.5242932210320692]], [[1.9854010554844139, -0.48881966224043172], [-4.4660773242982277, 0.27156434226925708]], [[-6.9660423539105087, 0.13911870428999773], [-0.63973854664872665, -0.80366477429810956]], [[0.86581140567006987, -4.7287579827865329], [-6.6508280211571558, -4.2368197988752287]], [[1.7220322982682452, -7.127215940718421], [-3.1911490371074729, -4.8644961262752195]], [[-1.0435365144786246, -6.2931471977819324], [-2.6138561100768722, 1.1395425740303926]]], [[[2.5142756094062473, -6.1364514603083133], [-1.1339627245604746, -6.1368385651532993]], [[3.2078492921008812, 1.8775295973850521], [-3.4844517497687089, -2.7627880510280134]], [[-1.3026981550726409, 0.354830029783614], [-1.9723434367552097, -5.0944126000359109]], [[-4.8940883497892873, 2.4460842255933222], [-0.14643324625542276, 0.69056885860429196]], [[-4.6735631549979084, -5.4716840283760018], [-5.2551788091597391, -3.3876731520955707]], [[-1.7537370077091148, 0.94455618317832757], [1.0765173817119345, 0.46057425590094381]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank1_Symbol_rank4(self):
      arg0=numpy.array([-0.044722267412026895, -1.0335435228033574])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-1.7555223380078266, -2.44519334052523], [-4.5901200554331547, 2.4171810783794401]], [[-4.4265330311127862, -2.8103620673041565], [-0.70779922005504936, -2.4322677764964884]], [[1.6763243190333696, 4.7424824830844283], [-2.6901125927434713, -2.6398596945775408]], [[0.9365209626671902, 2.9379536228292968], [1.8053785672090701, -2.5970870272729183]], [[1.1503189172483896, 2.6213922036281403], [3.6638777286840156, -0.7771805378040364]], [[3.4571163736014299, 1.8518223526235822], [-4.2736236223077597, 2.9656624601513926]]], [[[4.3173914356417704, 1.7915441476084881], [2.5873211259775299, 1.9248386777444964]], [[-4.119796722464482, 0.56092200516920698], [-3.1755012130685669, -0.19176681060780609]], [[1.3569242777756898, -4.0227706619260504], [4.7959197113731413, 3.894779991093861]], [[-4.3985019264167748, -3.9992116839349858], [2.1030582380144827, -3.8504780098478308]], [[3.7598510186102789, 2.8985505497038204], [1.5006745710375649, -2.5919490666121927]], [[2.9066605554678002, 4.9383256552799839], [-4.7708963359124636, 2.4586903307174026]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-1.8002446054198535, -2.4899156079372569], [-4.6348423228451816, 2.3724588109674132]], [[-4.4712552985248131, -2.8550843347161834], [-0.75252148746707626, -2.4769900439085153]], [[1.6316020516213428, 4.6977602156724014], [-2.7348348601554981, -2.6845819619895677]], [[0.89179869525516331, 2.8932313554172699], [1.7606562997970432, -2.6418092946849452]], [[1.1055966498363627, 2.5766699362161134], [3.6191554612719887, -0.82190280521606329]], [[3.412394106189403, 1.8071000852115553], [-4.3183458897197866, 2.9209401927393657]]], [[[3.283847912838413, 0.75800062480513075], [1.5537776031741726, 0.89129515494113898]], [[-5.1533402452678398, -0.47262151763415039], [-4.2090447358719238, -1.2253103334111635]], [[0.32338075497233243, -5.0563141847294073], [3.762376188569784, 2.8612364682905036]], [[-5.4320454492201318, -5.0327552067383436], [1.0695147152111253, -4.8840215326511878]], [[2.7263074958069216, 1.865007026900463], [0.46713104823420748, -3.6254925894155501]], [[1.8731170326644428, 3.9047821324766265], [-5.8044398587158206, 1.4251468079140452]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-0.31985862971808299, -2.5172788364499752, 1.926043209600552, -4.847054241563626, -1.1717896466650233], [-0.27796288179505968, -4.45993341614935, 4.4308166935086764, 1.8376899610271833, 1.492936684771772], [1.1725918536342741, 4.2484876818243471, -0.97066113975151502, 1.0269465902713417, 1.0819277838065124], [-0.21908447997299163, -2.2450493876372688, -1.8274726373628889, -0.97505317456929141, -4.7138258626452352]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[-4.9573065837560044, -2.2808240260679069, 1.1246651074825644, -3.9984166305547673, 4.4523879636406125], [-0.82852445648615181, 4.6122987820315871, 3.4204079837831909, -3.5633251518392717, 1.7287298515093106], [2.1336347333652199, -2.6581327886543136, 0.40841868887932531, -2.8270675888004861, 2.9986876695966993], [3.9380587086198364, -0.11715087896316234, 3.5512501773517151, -1.8378887081314965, 0.12035861402014092]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-5.2771652134740874, -4.7981028625178821, 3.0507083170831164, -8.8454708721183941, 3.2805983169755892], [-1.1064873382812115, 0.15236536588223704, 7.8512246772918672, -1.7256351908120884, 3.2216665362810826], [3.306226586999494, 1.5903548931700335, -0.56224245087218971, -1.8001209985291444, 4.0806154534032117], [3.7189742286468448, -2.3622002666004311, 1.7237775399888262, -2.8129418827007879, -4.5934672486250943]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_Symbol_rank2(self):
      arg0=numpy.array([[[-3.6858032899362678, -4.38042748485607], [-4.5496687662719379, -4.4964499699784248], [-3.1661589077711003, -1.8470998840291895], [-1.5244632808193548, -2.1399123363059882], [2.8678750517401284, 1.0778783835750305]], [[2.2274986506065888, 0.86931192429742943], [-0.61075098316238563, -3.9384316358210714], [-4.563252068852802, 2.1088086745097492], [0.025569794566790982, 3.0725003875086614], [-2.6897908129824799, -1.7546170998447708]], [[-2.1847693135279647, 1.2174306635654011], [1.3658821645408814, -4.7679498592562304], [1.2183681114507845, -2.1303478822117325], [-4.2222573901055638, -2.1305107685365043], [-0.15783835104424959, -1.7753455453566467]], [[3.6717938468166444, -3.0404615340724837], [4.7423026590228918, -1.8368456869335015], [-1.4726384878634535, -3.061073304476122], [1.5737551522369362, -3.9465637867592376], [0.34455509621447078, 2.5863579098270337]]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[1.8074707770492129, -2.4330977933232081, -0.25724785751558521, 1.427191036735838, -3.7026135104527622], [0.35933341169108868, 1.702025325209128, -1.538768948244976, -3.9685691780154233, -4.8091908503490153], [2.5054062033363085, -2.6549119059314217, 0.55369728707701071, -4.2069607764493693, 3.8415935003321113], [-1.7335719512675265, 3.9446525663001335, -3.0047765841633822, 3.8750507333445903, -1.5631300052886465]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-1.8783325128870549, -2.5729567078068571], [-6.982766559595146, -6.9295477633016329], [-3.4234067652866855, -2.1043477415447747], [-0.097272244083516757, -0.71272129957015018], [-0.83473845871263386, -2.6247351268777317]], [[2.5868320622976775, 1.2286453359885181], [1.0912743420467423, -2.2364063106119434], [-6.102021017097778, 0.57003972626477317], [-3.9429993834486323, -0.89606879050676191], [-7.4989816633314952, -6.5638079501937856]], [[0.32063688980834382, 3.7228368669017096], [-1.2890297413905403, -7.4228617651876521], [1.7720653985277952, -1.5766505951347218], [-8.4292181665549322, -6.3374715449858741], [3.6837551492878617, 2.0662479549754647]], [[1.9382218955491179, -4.7740334853400102], [8.6869552253230253, 2.107806879366632], [-4.4774150720268358, -6.0658498886395043], [5.4488058855815265, -0.07151305341464731], [-1.2185749090741758, 1.0232279045383872]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_Symbol_rank3(self):
      arg0=numpy.array([[3.9490345746071078, 2.2220426895000198, -2.9437075204058685, -2.2299735900628015, 0.2858517471200841], [-3.2067229874149081, 4.2248943259964538, -4.8661414123179618, -3.8580179291540793, 2.0170787502542709], [-3.7315902821587557, -4.554119947226102, -3.7568872303601064, 4.9800473333782431, -4.307723470282161], [0.66733114047748998, -3.4226640328893101, -3.276301992084, 3.4580449661585018, 0.65377278756153956]])
      arg1=Symbol(shape=(4, 5, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-1.1010353109004924, 1.5169556318511903], [0.27802317267268428, -2.5589103605112817], [-0.37385966400569792, 3.2990867630840626], [1.4445633789566648, 2.3167713097837126], [-3.5029979954844412, 0.71952112813549274]], [[-2.2309693623804883, 2.939575410400729], [3.6623136656147555, 1.732127588800938], [-3.936408577196481, -0.55395904452014832], [0.30779115863933626, -4.219561483338234], [2.4538491277962979, 1.0351078394336932]], [[2.0256373386894833, 3.2391260640202582], [4.06047348846867, 0.9212212968052409], [4.4832875861368819, -1.2725675276666082], [-1.6544366707313749, 4.2885327196727268], [0.023249948461071313, -1.3456769406526936]], [[-4.1003813190089309, -2.5009883576586525], [1.8078896635147359, 3.7663662362097323], [-3.4704170539363179, 4.5604603308038563], [-3.7021204407049115, 4.1821095584048926], [-0.71130360801243242, 4.8577009181442286]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[2.8479992637066154, 5.4659902064582981], [2.5000658621727041, -0.33686767101126192], [-3.3175671844115664, 0.3553792426781941], [-0.78541021110613674, 0.086797719720911104], [-3.2171462483643571, 1.0053728752555768]], [[-5.4376923497953964, -0.26714757701417913], [7.8872079916112092, 5.9570219147973917], [-8.8025499895144428, -5.4201004568381101], [-3.550226770514743, -8.0775794124923124], [4.4709278780505688, 3.0521865896879641]], [[-1.7059529434692724, -0.4924642181384975], [-0.49364645875743207, -3.6328986504208611], [0.72640035577677553, -5.0294547580267146], [3.3256106626468682, 9.2685800530509699], [-4.2844735218210896, -5.6534004109348546]], [[-3.4330501785314409, -1.8336572171811625], [-1.6147743693745742, 0.34370220332042223], [-6.7467190460203179, 1.2841583387198563], [-0.24407547454640977, 7.6401545245633944], [-0.057530820450892861, 5.5114737057057681]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[2.6479867499676022, -1.4816578310017823, -4.0306640807233372, -3.7836974656858855, -0.50928031777277205], [2.9794645234076462, -3.3043844304270511, 0.31673051372570882, -0.90366403372089366, -4.7575207771551939], [1.3230966790214138, -0.65866316749747256, 3.7021300994646857, -4.6907793935268183, 1.2512611326226288], [-1.3296021716937458, -3.9113899981052991, 2.5352751608020743, -1.3112173382213586, 3.5213016170928828]], [[1.1147456734416297, -0.6234753340979502, -3.1939621611618083, 1.2868122453690054, -4.2065604363676758], [-1.6499814924582412, 1.8971500244958017, -4.5027647221416176, 3.9499836024298354, 2.0047239131455319], [1.7506369956977643, -2.286431977342235, -3.7780311083076001, -2.1360486815327184, -0.90525427762968391], [-0.97119707672111399, 4.7833908519446524, -4.5966441185486664, 0.9755536972297314, -3.4298122527912223]], [[-1.0450041142278232, 4.3745919026352524, 1.7406736078308835, 0.097756145047643628, -3.4765082026198977], [-1.4318258356049904, 0.56041066842534448, 0.31308125823274047, 4.6550180497350642, 2.2031283952876199], [-3.8787451529376984, -4.8235765254481073, 1.3132644928771793, 1.279569939012875, 1.2531159730429167], [-2.7279525624309517, -0.82243028047200628, -3.6533822311169395, -1.5800155703722583, 1.8178516114903687]], [[2.0587184077365004, -2.3462907391336119, 4.7834441487427455, -1.4009985129170275, -3.7025343140386413], [0.11558588286406657, -4.091381052544552, -3.0019285909674598, 3.4937306140463456, 1.6063423575687459], [3.4020679236557996, -2.65964181062605, -1.6411873168148619, -4.1790776329309329, -2.1672207595895285], [-1.9114021621709618, -3.1153642660729464, 2.6093750782413414, 2.5102495122645951, -0.48208700158833739]], [[2.0602321494229336, -4.7640765433225285, 4.6674390604474087, 2.0817210348047697, -3.3004637168680384], [3.5575141110995609, 4.9922742586223983, -3.5513685855573387, 3.1600949415269319, -0.67153607783400293], [1.0536573498572954, 4.6595133842903476, -1.5277797926109491, -0.7498458724182564, 2.5105760365511367], [-4.6296485317601554, 1.7376681265960983, -2.3060663805326498, -2.9755390620996125, 4.9545537330838414]]], [[[2.2891596818189788, 1.2872764807839427, -0.33218734239240533, -2.1033066077701665, -4.9849494549381781], [0.18388955563056175, 0.49703852146942307, -3.1406575828641659, 0.22182779045513801, 3.7774714643154148], [-3.2634497325371226, -2.8566313999792623, 4.1529130330265822, -4.5156440878048514, -0.37693520058379981], [2.5533292534207233, 1.8794794822461149, -2.7036646749889526, 0.9738827371044847, -3.932048417606997]], [[2.0078141024047031, -1.1727488922237326, 4.6566681404767767, -3.052088068490455, 0.15995393184093931], [-0.62192332240212611, 4.2863424913219852, 0.94870915048444271, -1.225189897343999, -1.9742310524105031], [2.8890883473607234, -2.6445525248288213, 2.9850805117215806, -3.8959357214514756, 0.7221887673987224], [0.36311406260310708, -0.17592300337980937, -4.2260035960536264, 4.959966656662294, -2.3099059019426074]], [[2.330640582743162, -0.32698704321493821, 2.7164547889581723, -2.7423541501709261, 1.4372280426558035], [-4.8616152289716243, -0.49483081165618792, -3.5745299711237988, 4.1648544254574578, 3.4996258181806024], [1.5665611873383405, -2.2103467153638681, -1.447810138026294, -0.33973396661848021, -1.7542525945472045], [3.0115712191123745, 3.723456583693352, -0.25974267514918736, -3.2892007773958634, 1.2713102276524939]], [[-0.39180760476446252, 4.9018772019190866, 0.97727893977539182, 0.49530727804202002, -4.9118627040011074], [-3.1886018955543074, -3.3409973954720873, 1.116167818276514, 0.91076217333345078, 4.5649228237672581], [1.9588441961421683, 2.691013310042897, -1.4387308032543822, 2.8114893382244786, 3.3173529706592984], [-3.0585707945042104, -0.92458055839694353, -2.5245327335625256, 0.14039408676587151, 1.9006552646025288]], [[3.0029694799529842, 2.7701191412427946, 2.3440663469058105, 3.9466946779551133, -2.2173137464238102], [1.0370030889426758, 1.762896943244419, -1.9274179648279812, -1.8942401239230087, 4.9531828468238199], [4.3144090463550739, -2.83773704673838, 2.2986248713253001, -3.4166537946751907, 1.2148555869606508], [-4.1148404445037041, 2.7507420175519179, 3.7407123197011707, 4.342398738591088, -2.2929636263990858]]], [[[-3.1342374444724319, -4.7469243278839848, -3.7861867545742731, 3.7353928149629247, 3.4493485551215546], [3.6214963703263532, -4.0241280708639993, 1.8366153045902127, 1.2645203066410149, -3.1316746474028276], [-2.1407013175366041, -3.1011423736537838, 4.118445978011172, 4.2047452000650196, -4.459190275356538], [0.043529796220017403, -4.1969063795126278, 2.8275779861823427, 3.7496758739422766, 1.9617192000524089]], [[4.023264412804691, -3.6619974123558716, 2.9431591418140526, -4.6475319544259683, -1.3274141049144026], [-0.12016019266044786, -3.8263871617271561, 2.1640812933923677, 2.6873369729920826, 0.011922355928367878], [-3.2317552855481457, -0.029195107095357997, -4.3346028012267794, -4.0393956650561709, -3.4342927713498748], [4.9883641857059775, 3.2084999591964092, -2.3139871136337851, 3.4393684119210253, -2.1879531916915629]], [[-4.568711958124986, 4.3028872147217339, 2.2538492433519943, -4.860403165000033, 1.9600125557680714], [-3.3735463897121001, -0.25749561372915952, -2.3508089935497809, 1.3133019020553371, -2.5973780506534272], [1.0001426272964054, -2.6706207642973645, 0.53027188426479555, 4.6252802805218316, 0.98507114868321111], [4.1026196443967642, -0.90787855716675736, 2.1569132875715811, 4.1005585754680336, 3.1882724677076837]], [[2.1533243764731234, 4.2174995190070703, -1.7369362717184034, 0.26786815569621858, 3.4955599494461804], [-2.2711128624274579, 4.6066389701849264, 4.7185565941968584, -3.6622577755517116, 2.1297575081842126], [3.9503639683422076, -0.60897888046007953, 0.15218960212305088, -3.4549029780418019, -0.57201404309963788], [-2.965105596066798, -3.6378530491625174, 1.7128471239241314, 2.8544949231945473, 1.7647457380772735]], [[-0.62752191042744343, 4.466465542255877, 2.2428499068069572, -4.7091492206818772, 0.71267110459280758], [-2.3935315332026672, -1.2528926331757062, 0.62880095029287819, 0.1769393362744589, -0.62432059530293493], [1.9488715138274815, 4.6324970664773772, 0.46303119048688313, 3.7680801632148704, -3.5756771229404838], [4.7337436158532444, 1.9094823009676478, 4.5537617706654956, 4.4298470057736896, 3.3815688023690758]]], [[[3.1638910628378483, -3.2687620835442557, -1.9045153865274358, 4.2786218097310247, -4.0416302559318718], [2.0528913692192639, -2.9652336440138791, -0.87737333011619612, 0.15658587707361082, -4.9633744524315411], [4.1304404838679503, 1.0327386813097217, 2.9935140701652259, -2.6488310454584942, 1.9410762658980483], [4.9693782865809251, 1.1613470282366523, 2.5161070573582336, -4.8821468333672557, 4.316039169171793]], [[-1.9535350459599221, 3.831806343453124, 1.9565165465614225, -4.92705847504293, -3.6552578099649646], [0.91558002634639113, -4.0634619550255913, -4.4387406706793184, -1.0993863986270549, 3.1594256502080977], [4.7257709172709976, 1.6565145625390363, -2.3019228445195825, 1.0443018425322226, 1.9277093061156183], [1.855289935401597, -1.3154162008790502, 2.9778333721745662, 2.7372858009945409, -2.1015259937015163]], [[-1.672278657037686, 1.2206755940634721, 1.2135945360636473, -4.5361755287839287, -3.071874573266884], [0.98162911583306123, 1.759091154973679, -2.8351118595910219, -3.2735674271577428, -0.040080794580118706], [2.8837483794569705, -3.4948030407054764, -4.4049377297875676, 1.2189734752629739, 0.97137095166622167], [0.8888827356761535, -3.9347984558205473, -2.1610880500243033, -2.3979054636544839, -3.5276947136975698]], [[3.5275007896886255, -2.3306324935847589, -1.6381821202172975, 4.2836864515868012, 4.6693018720347919], [0.7123435682065864, -2.2568146436841396, 3.0649399908212143, 2.6846508325128919, 2.1586289237967051], [2.1890469010823432, 1.2551287432746143, -3.4562530348575136, -2.4493708188277443, 3.7155638131108741], [0.040932338603555607, 1.294106630475361, 4.0805474667970927, 1.8051986372054083, 0.12749009609805739]], [[-1.8640740095582196, -3.42386337459775, -4.6150515152395233, -3.7171245404189577, -0.78059858400802362], [-4.8647988127220483, 1.1041833495657816, -2.9784612443123915, 0.54942494790190111, -4.7766996599782088], [-2.6499498482566777, 1.8213921508404081, -0.024931834441868084, 4.8376312567769624, 2.5310474629937634], [3.7272131536765603, 4.6848421211150075, -4.0476070473354646, 4.9924378710631281, 4.6357517702627931]]]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[-2.711614822077979, 3.3183112560695278, 3.2974389274449294, -3.9923820390379658, -0.18647543712015313], [-3.3186667647424404, -1.9634311177074224, -0.94176111811217744, -1.134857489643093, 3.156793200449048], [0.72138317581506506, -3.8934642017901524, -1.7747364389524698, 1.0669211753712959, -1.4621186117125387], [-2.0114304706475741, 2.4029523408093754, -3.3162987122964225, -3.4618294893071155, -0.33604490883683624]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.063628072110376799, -4.1932726530797613, -6.7422789028013161, -6.4953122877638645, -3.220895139850751], [0.26784970132966723, -6.0159992525050301, -2.3948843083522702, -3.6152788557988726, -7.4691355992331729], [-1.3885181430565652, -3.3702779895754515, 0.99051527738670675, -7.4023942156047973, -1.4603536894553502], [-4.0412169937717248, -6.6230048201832776, -0.17633966127590472, -4.0228321602993375, 0.80968679501490382]], [[4.4330569295111575, 2.6948359219715776, 0.1243490949077195, 4.6051235014385332, -0.88824918029814803], [1.6683297636112866, 5.2154612805653295, -1.1844534660720898, 7.2682948584993632, 5.3230351692150597], [5.0689482517672921, 1.0318792787272928, -0.45971985223807232, 1.1822625745368094, 2.4130569784398439], [2.3471141793484138, 8.1017021080141802, -1.2783328624791386, 4.2938649532992592, -0.11150099672169445]], [[2.2524348132171061, 7.6720308300801818, 5.0381125352758129, 3.395195072492573, -0.17906927517496829], [1.865613091839939, 3.8578495958702739, 3.6105201856776699, 7.9524569771799936, 5.5005673227325493], [-0.581306225492769, -1.5261375980031779, 4.6107034203221087, 4.5770088664578044, 4.5505549004878461], [0.56948636501397765, 2.4750086469729231, -0.3559433036720101, 1.7174233570726711, 5.1152905389352981]], [[-1.9336636313014655, -6.3386727781715777, 0.79106210970477964, -5.3933805519549933, -7.6949163530766072], [-3.8767961561738993, -8.0837630915825187, -6.9943106300054261, -0.4986514249916203, -2.38603968146922], [-0.59031411538216627, -6.6520238496640154, -5.6335693558528277, -8.1714596719688988, -6.1596027986274944], [-5.9037842012089277, -7.1077463051109122, -1.3830069607966244, -1.4821325267733707, -4.4744690406263032]], [[1.8737567123027805, -4.9505519804426816, 4.4809636233272556, 1.8952455976846165, -3.4869391539881915], [3.3710386739794078, 4.8057988215022451, -3.7378440226774918, 2.9736195044067788, -0.85801151495415606], [0.86718191273714229, 4.4730379471701944, -1.7142552297311022, -0.93632130953840953, 2.3241005994309836], [-4.8161239688803086, 1.5511926894759451, -2.4925418176528029, -3.1620144992197656, 4.7680782959636883]]], [[[-1.0295070829234616, -2.0313902839584976, -3.6508541071348457, -5.4219733725126069, -8.3036162196806185], [-3.1347772091118786, -2.8216282432730173, -6.4593243476066062, -3.0968389742873024, 0.45880469957297443], [-6.5821164972795625, -6.1752981647217027, 0.83424626828414183, -7.8343108525472918, -3.6956019653262402], [-0.76533751132171712, -1.4391872824963254, -6.022331439731393, -2.3447840276379557, -7.2507151823494373]], [[0.044382984697280747, -3.136180009931155, 2.6932370227693543, -5.0155191861978778, -1.8034771858664831], [-2.5853544401095485, 2.3229113736145628, -1.0147219672229797, -3.1886210150514214, -3.9376621701179255], [0.92565722965330099, -4.6079836425362437, 1.0216493940141582, -5.8593668391588984, -1.2412423503087], [-1.6003170551043153, -2.1393541210872318, -6.1894347137610488, 2.9965355389548716, -4.2733370196500298]], [[1.3888794646309846, -1.2687481613271157, 1.7746936708459948, -3.6841152682831035, 0.49546692454362606], [-5.8033763470838018, -1.4365919297683654, -4.5162910892359758, 3.2230933073452803, 2.5578647000684249], [0.62480006922616305, -3.1521078334760455, -2.3895712561384714, -1.2814950847306577, -2.6960137126593819], [2.069810101000197, 2.7816954655811745, -1.2015037932613648, -4.2309618955080408, 0.32954910954031646]], [[-1.5266650944075555, 3.7670197122759936, -0.15757854986770115, -0.63955021160107295, -6.0467201936441999], [-4.3234593851974008, -4.4758548851151803, -0.018689671366578953, -0.22409531630964219, 3.4300653341241651], [0.82398670649907535, 1.556155820399804, -2.5735882928974751, 1.6766318485813856, 2.1824954810162054], [-4.1934282841473038, -2.0594380480400365, -3.6593902232056186, -0.99446340287722146, 0.7657977749594358]], [[6.1597626804020322, 5.9269123416918426, 5.5008595473548585, 7.1034878784041613, 0.9394794540252378], [4.1937962893917238, 4.919690143693467, 1.2293752356210668, 1.2625530765260393, 8.1099760472728679], [7.4712022468041219, 0.31905615371066798, 5.4554180717743481, -0.25986059422614272, 4.3716487874096988], [-0.95804724405465613, 5.9075352180009659, 6.8975055201502187, 7.499191939040136, 0.86382957404996219]]], [[[-2.4128542686573669, -4.0255411520689197, -3.0648035787592081, 4.4567759907779898, 4.1707317309366196], [4.3428795461414182, -3.3027448950489342, 2.5579984804052778, 1.9859034824560799, -2.4102914715877626], [-1.419318141721539, -2.3797591978387187, 4.839829153826237, 4.9261283758800847, -3.7378070995414729], [0.76491297203508246, -3.4755232036975627, 3.5489611619974077, 4.4710590497573417, 2.6831023758674739]], [[0.12980021101453865, -7.555461614146024, -0.95030505997609982, -8.5409961562161207, -5.220878306704555], [-4.0136243944506003, -7.7198513635173089, -1.7293829083977847, -1.2061272287980698, -3.8815418458617845], [-7.1252194873382981, -3.9226593088855104, -8.2280670030169318, -7.9328598668463233, -7.3277569731400272], [1.0948999839158251, -0.68496424259374322, -6.2074513154239375, -0.45409578986912713, -6.0814173934817148]], [[-6.3434483970774558, 2.5281507757692641, 0.47911280439952453, -6.6351396039525028, 0.18527611681560163], [-5.1482828286645699, -2.0322320526816293, -4.1255454325022507, -0.46143453689713265, -4.372114489605897], [-0.77459381165606445, -4.4453572032498343, -1.2444645546876743, 2.8505438415693618, -0.78966529026925869], [2.3278832054442944, -2.6826149961192272, 0.38217684861911128, 2.3258221365155638, 1.4135360287552139]], [[3.2202455518444193, 5.2844206943783663, -0.67001509634710743, 1.3347893310675145, 4.5624811248174764], [-1.204191687056162, 5.6735601455562223, 5.7854777695681543, -2.5953366001804157, 3.1966786835555085], [5.0172851437135035, 0.45794229491121641, 1.2191107774943468, -2.387981802670506, 0.49490713227165806], [-1.8981844206955021, -2.5709318737912215, 2.7797682992954273, 3.9214160985658433, 2.8316669134485695]], [[-2.0896405221399821, 3.0043469305433383, 0.78073129509441852, -6.1712678323944159, -0.74944750711973107], [-3.8556501449152059, -2.7150112448882449, -0.83331766141966046, -1.2851792754380797, -2.0864392070154736], [0.48675290211494282, 3.1703784547648386, -0.99908742122565553, 2.3059615515023317, -5.037795734653022], [3.2716250041407058, 0.44736368925510916, 3.0916431589529569, 2.967728394061151, 1.9194501906565371]]], [[[1.1524605921902742, -5.2801925541918298, -3.9159458571750099, 2.2671913390834506, -6.0530607265794458], [0.041460898571689775, -4.9766641146614532, -2.8888038007637702, -1.8548445935739633, -6.9748049230791151], [2.1190100132203762, -0.97869178933785239, 0.98208359951765178, -4.6602615161060683, -0.070354204749525806], [2.957947815933351, -0.85008344241092182, 0.50467658671065951, -6.8935773040148298, 2.3046086985242189]], [[0.44941729484945325, 6.2347586842624994, 4.3594688873707979, -2.5241061342335547, -1.2523054691555893], [3.3185323671557665, -1.6605096142162159, -2.0357883298699431, 1.3035659421823205, 5.5623779910174731], [7.128723258080373, 4.0594669033484116, 0.10102949628979285, 3.4472541833415979, 4.3306616469249937], [4.2582422762109724, 1.0875361399303252, 5.3807857129839416, 5.1402381418039162, 0.30142634710785909]], [[-4.9885773693341084, -2.0956231182329503, -2.1027041762327752, -7.8524742410803512, -6.3881732855633064], [-2.3346695964633613, -1.5572075573227435, -6.1514105718874443, -6.5898661394541653, -3.3563795068765412], [-0.43255033283945199, -6.8111017530018989, -7.7212364420839901, -2.0973252370334485, -2.3449277606302008], [-2.427415976620269, -7.2510971681169698, -5.4773867623207257, -5.7142041759509059, -6.8439934259939923]], [[0.065671300381509923, -5.792461982891874, -5.1000116095244135, 0.82185696227968563, 1.2074723827276763], [-2.7494859211005291, -5.7186441329912547, -0.39688949848590127, -0.7771786567942236, -1.3032005655104104], [-1.2727825882247723, -2.2067007460325012, -6.9180825241646291, -5.9112003081348599, 0.2537343238037586], [-3.4208971507035599, -2.1677228588317545, 0.61871797748997714, -1.6566308521017072, -3.3343393932090581]], [[-2.2001189183950558, -3.7599082834345863, -4.9510964240763595, -4.0531694492557939, -1.1166434928448599], [-5.2008437215588845, 0.76813844072894533, -3.3145061531492277, 0.21338003906506486, -5.112744568815045], [-2.9859947570935139, 1.4853472420035718, -0.36097674327870433, 4.5015863479401261, 2.1950025541569271], [3.391168244839724, 4.3487972122781713, -4.3836519561723009, 4.6563929622262918, 4.2997068614259568]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank2_Symbol_rank4(self):
      arg0=numpy.array([[0.82079242815354903, 0.85307925268533591, 1.1179191379445008, -0.39214866876211563, 0.28922584171317745], [-0.65605888640043464, 1.7791565764213555, -3.0139111996958556, 2.7090403361618396, -2.9952687341528872], [2.1350477478225329, -0.11155571043461698, 4.076900748177728, 3.5543789495636773, 2.631930072133585], [4.7672321857072539, -3.1672140063987051, 3.6507571879356941, -2.3348334505901489, 0.52695774081606839]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-2.9037088070192585, -2.3088443806905481, -3.5491161861909113, 4.0816701673192632, 4.398817876463454], [-3.164020019072292, -3.1144810241912593, 0.65593574008539601, 1.5965737406032376, -1.8879592716549753], [1.6059404170685996, -3.6688430766796443, 3.4833953105184055, 0.87476964739523666, -0.22163192175715185], [-4.3073833059103475, -1.2487875602229703, 0.054974856668705385, 3.9085139611574036, 0.36981385771817976]], [[-0.053055274678345477, -3.626616354750527, 2.9253248059269588, 0.16056875614303578, -1.1867316192976594], [2.9721938065327667, 4.2397571782529528, -4.9357900663893925, 1.6971132812188037, 3.6955067330190641], [-3.109972956771232, -0.23345340440622664, 4.9800892998536543, 0.15967686456883357, -1.1216281246070525], [-1.3981194473051497, 1.2224574297797215, 4.5141124666219294, 1.6678153498554424, -3.6753921454292682]], [[-0.089565418438930067, 3.4266243853480738, 2.0263525204266095, 4.728031938058729, -1.279274955234726], [3.7606098398734584, -2.1367801213192985, -3.8065349552897088, 1.7822327748530595, 1.037204046518645], [-4.3795293032719931, -1.1481281573255684, 1.1361904586249283, -4.2486906825959627, 2.5399327559678317], [4.9759845583958668, -1.3124378397600167, 2.9683754786976717, -4.0422096713779512, 1.1297491808499203]], [[-1.6577690025589122, -0.07603236046870876, 2.0389872686225754, -3.3207551814548451, -1.9391329965482829], [2.8699740106931371, -3.3236956383024463, -1.9402342811177342, -3.4265984491967192, -1.2819494016561475], [1.8424024256513434, -0.029282688893451869, 4.6624519878150181, -1.5197654403276859, 3.284710477187005], [-3.303455972534417, 2.6662957193333758, 3.3027268068953859, -0.54504455586076528, 0.99975179278159931]], [[3.3249030597167195, 3.8158845641187789, 0.18930794155570752, -2.4246262221227566, -2.212744823672784], [1.6425565607464794, 2.9512461581548122, 0.13992588732632072, 0.42858842483123016, -2.3869656532921688], [-1.9286697301019586, 2.0811220740018026, -4.7350032488037961, 1.6705200869835126, 4.0818699891609267], [3.3005787696849733, -2.951545256139394, 1.3590945181111449, 3.6151917035387022, -2.0872061580997636]]], [[[-3.1186018004720859, 4.5641800057609121, 2.3974524636389347, -0.647596953460714, -0.71358928470511263], [0.54344148553681748, -1.6211025652152977, -3.4586029084603096, -3.6682543774772345, -0.64998374674554427], [-0.56802303330734549, -0.86035546517422823, 1.9676730966358402, -1.1575342485623819, -4.2598114073751603], [-1.6946941793807446, -1.4998726503963109, -3.659401216309297, 4.0561421650724334, 3.1608647414606175]], [[-3.9007257754505775, 4.1047436381026756, -1.5303007774211022, 0.28749786795889687, -4.4117739354334891], [4.5425877642303476, 0.021915699244199338, 1.3125181593394046, 1.0510659602544381, 1.8790269757655365], [1.2564547862660946, 0.091107574192382579, 2.3091813609149288, 1.023462259313396, -3.8835372741264895], [2.8202512499171757, -2.3309217026373918, 3.5389047783251666, 2.8089626390426883, -2.5093689230168414]], [[-4.3118984509233389, -4.0530515346338447, 4.7047939562986159, -1.2616841425713932, -2.6038861194142537], [2.0255589752590613, 2.4245393158310993, 1.2990514743281167, -3.1341788952554452, -3.7605553276413737], [-4.1253790336287128, -1.8571238557307046, 3.2509163585939174, -3.9192675271272694, -4.7206645523660029], [4.500111298972449, -0.78910379924150043, -1.7159185555772316, -1.0263674434088008, 3.8057942111572043]], [[-2.086678685258152, 0.21291767134524164, 4.7556288039809669, -1.5278664123998031, 2.3182605780074637], [3.9346673571563411, 2.4537058227736814, 3.3732666021298812, 2.1484727491052444, -3.7448850592719163], [-1.9154872855781022, -4.1313137094069665, 1.7347131881947835, 0.88329510001694267, 2.1138529827384547], [4.5190655352467601, 1.7557703660284671, 2.1603283553306198, 1.4958079377877151, -3.3715820869756552]], [[2.697878203102829, 3.5841187151814484, -3.5054392242318633, -4.5763711324258001, 0.2482788893589527], [3.3336676954579154, -1.1273195539497549, -3.9835107034260497, -2.3926896948407625, -0.91064813823116886], [-2.9073836682420016, 1.1832380496251202, -4.7282740057037129, 1.3767425623369736, -0.50483236549286303], [3.2216492989988925, 1.0899559197226782, -0.97624927229300873, 2.3805532212750391, 0.42166542486668845]]], [[[-2.9420332131441382, 3.2392305712128735, -1.8131334064811977, -1.1482638160703438, -2.6888397130136967], [3.9091358591469927, 4.485058656438607, 3.0609692794082015, 0.11039861988860267, 3.6769425233065149], [4.9876952900805698, 1.9371220938807916, 4.1121362047017467, -1.2567288854467309, 3.3374954955636156], [1.7102419214013311, 0.27653253724370241, -2.227244274991401, 1.2488291897015102, -4.3021725745470079]], [[-0.74842837498864689, -0.24963892526461162, 1.8286320372720857, -1.2349200609482249, -2.0702486224926906], [-1.5246399841950211, 4.8475070015304986, 1.866110284671616, 3.6553384392554999, -4.5916794748403591], [-1.9392531703766469, 4.6045358389385349, -4.6197470786784738, 4.8035213950849087, -3.5626564853884002], [3.4445060444649869, -3.3255506457723696, -3.6889180530000489, 0.67893448478247276, 4.0137667158535475]], [[-2.4326549077595261, 4.8316496544330008, -2.2780331042911106, 0.78507606061427726, 1.6445203602465588], [3.3039462359029805, 3.6130303452000661, -2.927267931615237, 3.2228473650868814, 4.5152765881744124], [-4.6107029745239192, 3.5060072056606337, 3.0715584492079167, 3.5351737285279814, -4.8116810983141711], [-0.98083206464823913, 1.658439568506954, -0.42280618808195314, -4.0274643960355512, 1.8374309464201701]], [[1.1495298831846243, 4.700161236872173, -0.77530081245193472, 3.6317488607431567, -1.5946078125752727], [-1.2915486046719593, 2.2705286827439064, -4.743368456984534, 2.7278695558618837, -3.2047488759577911], [0.72911324895546858, -3.5356349856528455, 2.9598648555728762, -0.21463338618892269, -1.0137073810942976], [-2.7279594369049756, -3.3945921339966967, -2.0691256622875756, 4.0024617963799347, 1.7842247013178181]], [[-0.67143570881842951, -1.7681207131840471, -2.3430425049608807, -1.5246164591806286, -2.2863852994284564], [4.0386959914559348, -4.728127250555195, -1.2253027824356266, -4.0828123011987572, 1.9075626753875641], [3.8503183114376256, 0.57519993926961366, 0.72230456786710384, 0.37377973232660011, -2.8581475031761574], [-3.8692738793734249, -2.4221314204564215, -1.9977497302932323, 1.5421292415778236, -0.92949288164397093]]], [[[1.6232218141252979, 3.3001210268111052, -0.58836272273680024, 0.98862234230750445, 4.1470338352230574], [3.2702496179658382, -1.3842858715465542, -4.7573133376401291, 4.9192048081497504, 4.9203925256629475], [3.4144260925346508, -2.6814788973170991, -1.6558906953344401, 0.41045496837063489, 1.8147533717836337], [3.9116059907732552, 3.0404412234091414, 1.0650759056595271, 4.3802582326235164, -4.8114193491359813]], [[1.3522681846484765, 0.58727452430983185, 4.234367955176868, -3.661149502040332, -0.068303276333004703], [4.5982702281299428, -3.4785600329201269, 3.9967005903913364, 4.6271853141093384, 1.042423377648114], [-4.2066136956992528, -2.2315917898708149, 1.6409146745007144, 4.2420810807315164, -0.066465980785471501], [-0.52889419355630451, -3.5449609421183261, 4.0067008800204373, 0.29439600621725237, 3.1573619067558099]], [[-0.25438498668454024, -3.3020747957503485, 2.9244549084604703, 2.829565294721295, 1.3386809880429915], [-0.35837452255310254, -1.8766408215579169, -0.81503286511798656, 1.6283431847151286, 1.5071779557533995], [-1.4626118845407121, 4.7220823548986388, 0.41182329809094576, 0.56844914973003213, 1.3462143112827283], [3.8065792572343042, -3.5952635518938658, -1.0921786696178315, -2.5305214666245748, -0.62120298719256795]], [[-4.3876718686043699, 4.750044331185947, 4.4461457100678015, -3.2498530692889185, 3.6619960409457946], [3.0957480255961585, -3.7945007983183263, -4.8503160633547004, -3.8869230412190001, -0.98675926849345785], [3.311436321824182, -0.85997488762666663, 0.41483619780284364, 4.6231939088925991, 2.7087958340039133], [1.2749804589676312, 4.378327694474919, 0.85459096940476353, -3.6447073622578974, -2.1986252906274761]], [[-4.3088984240758368, -2.2922251930970718, -3.7964252376442174, 3.7796228074999032, 4.7812301971352387], [0.0043247035007665247, 0.42345128399948351, -4.0855633591394858, -1.7236363005699316, 2.0915520024230911], [-1.2484463320612993, -4.6174824167772517, 4.4344584047185496, 4.429926712427557, -2.1343781460102385], [-3.0425345002318349, 1.4106628338357643, 2.3623386729991127, 3.6478360451833289, 0.55684226350401911]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-2.0829163788657095, -1.4880519525369991, -2.7283237580373623, 4.9024625954728123, 5.219610304617003], [-2.343227590918743, -2.2936885960377102, 1.476728168238945, 2.4173661687567867, -1.0671668435014263], [2.4267328452221486, -2.8480506485260952, 4.3041877386719545, 1.6955620755487857, 0.59916050639639717], [-3.4865908777567984, -0.4279951320694213, 0.87576728482225441, 4.7293063893109526, 1.1906062858717288]], [[0.80002397800699043, -2.7735371020651911, 3.7784040586122947, 1.0136480088283717, -0.33365236661232345], [3.8252730592181026, 5.0928364309382887, -4.0827108137040566, 2.5501925339041396, 4.5485859857044], [-2.2568937040858961, 0.61962584827910927, 5.8331685525389902, 1.0127561172541695, -0.26854887192171661], [-0.54504019461981379, 2.0755366824650574, 5.3671917193072654, 2.5208946025407784, -2.8223128927439323]], [[1.0283537195055708, 4.5445435232925746, 3.1442716583711103, 5.8459510760032298, -0.16135581729022519], [4.8785289778179592, -1.0188609833747977, -2.6886158173452079, 2.9001519127975603, 2.1551231844631458], [-3.2616101653274923, -0.03020901938106757, 2.2541095965694291, -3.1307715446514619, 3.6578518939123326], [6.0939036963403677, -0.19451870181551589, 4.0862946166421725, -2.9242905334334504, 2.2476683187944211]], [[-2.0499176713210279, -0.46818102923082439, 1.6468385998604598, -3.7129038502169607, -2.3312816653103985], [2.4778253419310214, -3.7158443070645619, -2.3323829498798498, -3.8187471179588348, -1.6740980704182631], [1.4502537568892278, -0.4214313576555675, 4.2703033190529025, -1.9119141090898015, 2.8925618084248894], [-3.6956046412965327, 2.2741470505712602, 2.9105781381332703, -0.9371932246228809, 0.60760312401948369]], [[3.614128901429897, 4.1051104058319563, 0.47853378326888496, -2.1354003804095791, -1.9235189819596066], [1.9317824024596568, 3.2404719998679896, 0.42915172903949816, 0.71781426654440761, -2.0977398115789914], [-1.6394438883887812, 2.37034791571498, -4.4457774070906186, 1.9597459286966901, 4.3710958308741041], [3.5898046113981508, -2.6623194144262166, 1.6483203598243223, 3.9044175452518797, -1.7979803163865862]]], [[[-3.7746606868725205, 3.9081211193604775, 1.7413935772385001, -1.3036558398611486, -1.3696481711055473], [-0.11261740086361716, -2.2771614516157324, -4.1146617948607442, -4.3243132638776691, -1.3060426331459789], [-1.2240819197077801, -1.5164143515746629, 1.3116142102354056, -1.8135931349628165, -4.9158702937755949], [-2.3507530657811793, -2.1559315367967455, -4.3154601027097321, 3.4000832786719988, 2.5048058550601828]], [[-2.121569199029222, 5.8839002145240311, 0.24885579900025334, 2.0666544443802524, -2.6326173590121336], [6.3217443406517031, 1.8010722756655548, 3.0916747357607601, 2.8302225366757936, 3.658183552186892], [3.0356113626874501, 1.8702641506137381, 4.0883379373362843, 2.8026188357347515, -2.104380697705134], [4.5994078263385312, -0.55176512621603635, 5.3180613547465221, 4.5881192154640438, -0.73021234659548595]], [[-7.3258096506191945, -7.0669627343297003, 1.6908827566027602, -4.2755953422672484, -5.6177973191101094], [-0.98835222443679438, -0.58937188386475636, -1.7148597253677389, -6.1480900949513009, -6.7744665273372293], [-7.1392902333245685, -4.8710350554265602, 0.23700515889806173, -6.9331787268231251, -7.7345757520618585], [1.4862000992765934, -3.8030149989373561, -4.7298297552730872, -4.0402786431046565, 0.79188301146134865]], [[0.62236165090368756, 2.9219580075070812, 7.4646691401428065, 1.1811739237620364, 5.0273009141693032], [6.6437076933181807, 5.162746158935521, 6.0823069382917208, 4.857513085267084, -1.0358447231100767], [0.79355305058373737, -1.4222733732451269, 4.4437535243566231, 3.5923354361787823, 4.8228933189002943], [7.2281058714085997, 4.4648107021903067, 4.8693686914924594, 4.2048482739495547, -0.66254175081381561]], [[-0.29739053105005819, 0.58884998102856123, -6.5007079583847505, -7.5716398665786873, -2.7469898447939345], [0.33839896130502822, -4.1225882881026426, -6.9787794375789369, -5.3879584289936497, -3.905916872384056], [-5.9026524023948888, -1.8120306845277669, -7.7235427398566001, -1.6185261718159136, -3.5001010996457502], [0.22638056484600533, -1.9053128144302089, -3.9715180064458959, -0.61471551287784809, -2.5736033092861987]]], [[[-0.80698546532160531, 5.3742783190354064, 0.32191434134133523, 0.98678393175218915, -0.5537919651911638], [6.0441836069695256, 6.6201064042611399, 5.1960170272307344, 2.2454463677111356, 5.8119902711290479], [7.1227430379031027, 4.0721698417033245, 6.2471839525242796, 0.87831886237580203, 5.4725432433861485], [3.8452896692238641, 2.4115802850662353, -0.092196527168868059, 3.3838769375240432, -2.1671248267244749]], [[-0.85998408542326388, -0.36119463569922861, 1.7170763268374687, -1.3464757713828419, -2.1818043329273076], [-1.6361956946296381, 4.7359512910958816, 1.754554574236999, 3.543782728820883, -4.7032351852749761], [-2.0508088808112639, 4.4929801285039179, -4.7313027891130908, 4.6919656846502917, -3.6742121958230172], [3.3329503340303699, -3.4371063562069866, -3.8004737634346659, 0.56737877434785577, 3.9022110054189305]], [[1.6442458404182019, 8.9085504026107287, 1.7988676438866174, 4.8619768087920052, 5.7214211084242867], [7.3808469840807085, 7.689931093377794, 1.1496328165624909, 7.2997481132646094, 8.5921773363521403], [-0.53380222634619123, 7.5829079538383617, 7.1484591973856446, 7.6120744767057094, -0.73478035013644316], [3.0960686835294888, 5.7353403166846819, 3.6540945600957748, 0.049436352142176787, 5.9143316945978981]], [[4.7039088327483016, 8.2545401864358503, 2.7790781371117426, 7.186127810306834, 1.9597711369884045], [2.262830344891718, 5.8249076323075837, -1.1889895074208567, 6.282248505425561, 0.34963007360588616], [4.2834921985191459, 0.018743963910831773, 6.5142438051365534, 3.3397455633747546, 2.5406715684693797], [0.82641951265870173, 0.1597868155669806, 1.4852532872761017, 7.556840745943612, 5.3386036508814954]], [[1.9604943633151555, 0.86380935894953792, 0.28888756717270425, 1.1073136129529564, 0.34554477270512862], [6.6706260635895198, -2.09619717842161, 1.4066272896979584, -1.4508822290651722, 4.5394927475211491], [6.4822483835712106, 3.2071300114031986, 3.3542346400006888, 3.0057098044601851, -0.22621743104257241], [-1.2373438072398399, 0.20979865167716349, 0.63418034184035266, 4.1740593137114086, 1.7024371904896141]]], [[[6.3904539998325518, 8.0673532125183591, 4.1788694629704537, 5.7558545280147584, 8.9142660209303113], [8.0374818036730922, 3.3829463141606997, 0.0099188480671248414, 9.6864369938570043, 9.6876247113702014], [8.1816582782419047, 2.0857532883901548, 3.1113414903728138, 5.1776871540778888, 6.5819855574908877], [8.6788381764805091, 7.8076734091163953, 5.8323080913667811, 9.1474904183307704, -0.044187163428727416]], [[-1.8149458217502286, -2.5799394820888732, 1.0671539487781629, -6.8283635084390371, -3.2355172827317098], [1.4310562217312377, -6.645774039318832, 0.82948658399263131, 1.4599713077106333, -2.1247906287505911], [-7.3738277020979579, -5.39880579626952, -1.5262993318979907, 1.0748670743328113, -3.2336799871841766], [-3.6961081999550096, -6.7121749485170312, 0.83948687362173224, -2.8728180001814527, -0.0098520996428952401]], [[3.3963722012511539, 0.34868239218534569, 6.5752120963961644, 6.4803224826569892, 4.9894381759786857], [3.2923826653825916, 1.7741163663777773, 2.8357243228177076, 5.2791003726508228, 5.1579351436890937], [2.188145303394982, 8.372839542834333, 4.0625804860266399, 4.2192063376657263, 4.9969714992184224], [7.4573364451699984, 0.055493636041828331, 2.5585785183178626, 1.1202357213111194, 3.0295542007431262]], [[-6.7225053191945188, 2.4152108805957981, 2.1113122594776526, -5.5846865198790674, 1.3271625903556457], [0.76091457500600956, -6.1293342489084752, -7.1851495139448494, -6.221756491809149, -3.3215927190836068], [0.97660287123403311, -3.1948083382168155, -1.9199972527873053, 2.2883604583024502, 0.37396238341376442], [-1.0598529916225177, 2.0434942438847701, -1.4802424811853854, -5.9795408128480467, -4.5334587412176255]], [[-3.7819406832597684, -1.7652674522810035, -3.269467496828149, 4.3065805483159716, 5.3081879379513071], [0.53128244431683491, 0.95040902481555189, -3.5586056183234174, -1.1966785597538632, 2.6185097432391595], [-0.7214885912452309, -4.0905246759611833, 4.961416145534618, 4.9568844532436254, -1.6074204051941701], [-2.5155767594157665, 1.9376205746518327, 2.8892964138151811, 4.1747937859993973, 1.0838000043200875]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-1.6945198235687542, -4.1357612833645767], [-3.5138317310929876, -0.16871813077230513]], [[4.0629340527707143, -0.40324138048890212], [-1.9518997046458364, -2.8727324435899106]], [[3.3512152282660566, 1.4215055023227379], [1.0162398176903498, -4.9905286408106262]], [[1.0076265659149852, -4.1412844417985104], [0.77318701890134545, -2.3879753208694798]], [[4.4424096483389128, -4.4439783598146567], [-2.5253433913779033, 2.2423090703568018]], [[-1.0119717422174457, 3.2552681736849074], [2.6456755747289549, 4.7666855616578871]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-1.7386857182739757, -0.33425607030334703], [-4.680751140315901, -0.15898693179051371]], [[2.1228790307925589, -2.6036610003085983], [-3.8658082341952458, -3.4465757263447028]], [[-2.0042969454005286, -1.0613987190524767], [-0.45035950385249901, 4.1552924189465212]], [[1.1795802775403708, 0.18439165954040693], [-2.6093887411538228, 1.528410162394211]], [[-2.7684706760645215, 0.71412066549437547], [-3.9680145521601027, 3.8007482637958461]], [[2.4265783347943213, 3.9031823817703621], [2.7634552597524813, -0.15743347911005312]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-3.4332055418427299, -4.4700173536679237], [-8.1945828714088886, -0.32770506256281884]], [[6.1858130835632732, -3.0069023807975004], [-5.8177079388410817, -6.3193081699346134]], [[1.3469182828655279, 0.36010678327026113], [0.56588031383785076, -0.83523622186410496]], [[2.187206843455356, -3.9568927822581035], [-1.8362017222524774, -0.85956515847526882]], [[1.6739389722743914, -3.7298576943202812], [-6.4933579435380064, 6.0430573341526479]], [[1.4146065925768756, 7.1584505554552695], [5.4091308344814362, 4.609252082547834]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[-1.6922426852767636, 0.27817179325641916], [-0.47548270623897615, 4.8616980306900217]], [[-2.1567583478455443, 4.8843906276516496], [-1.5441189667625399, -1.8107112228193447]]], [[[-3.0131465276480283, -3.8127181384248376], [-4.839590639951675, 3.3931852929735733]], [[-0.42603883408317955, -0.36105640821485707], [1.1461448889125325, -1.376120803662185]]], [[[-3.4521462096803202, -1.0755722870459925], [-0.10837662734416664, -3.8810864149234603]], [[-4.81872907759477, -2.5636344854727531], [1.8827275801823191, 4.6849423097360283]]], [[[-3.3315957873080326, 0.41577763401766088], [3.4365920656897071, 2.2193375267821942]], [[0.83988200439367233, -4.3273905126150058], [4.7891226571724967, 1.6142229073606256]]], [[[-1.8354355377968243, -0.46825540174735369], [-0.49673219286400982, 0.45845756356086298]], [[-2.2809786608794589, -4.2006717171094667], [-4.1676784055537475, 2.7859023826581888]]], [[[1.2918161638387353, -2.7400608543446401], [-2.0740280911852671, -2.3036029283981865]], [[-3.1421427544157341, 2.3809342596077077], [-1.4092967264865832, -1.365445157021469]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-2.1861793683300821, -4.29834166737351], [3.1320017523265609, 0.26178341012459327]], [[-2.2566075608682521, 0.08907205337418489], [4.0335923288753897, -0.6898881476728107]], [[2.2048038514100812, 3.8189453877497019], [-1.6573463645629505, 1.5199119515333024]], [[-0.01854837389638142, -0.71044385483278294], [-2.2989217403872129, 4.6494445729742555]], [[4.6253322482383563, 4.3911995145542519], [-3.0874870647023309, 3.1722308166396669]], [[1.2256674995272938, -4.8874144115779439], [-3.9743604030991166, 0.41572308305696826]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-3.8784220536068457, -1.9080075750736629], [-4.7738243736124861, 0.56335636331651173]], [[0.97524340448101654, 8.0163923799782104], [-1.2823355566379466, -1.5489278126947514]]], [[[-5.2697540885162804, -6.0693256992930902], [-4.7505185865774902, 3.4822573463477582]], [[3.6075534947922101, 3.6725359206605326], [0.45625674123972182, -2.0660089513349957]]], [[[-1.247342358270239, 1.1292315643640887], [3.7105687604055353, -0.062141027173758356]], [[-6.47607544215772, -4.2209808500357031], [3.4026395317156215, 6.2048542612693307]]], [[[-3.3501441612044141, 0.39722926012127946], [2.7261482108569242, 1.5088936719494113]], [[-1.4590397359935405, -6.6263122530022187], [9.4385672301467523, 6.2636674803348811]]], [[[2.789896710441532, 4.1570768464910026], [3.8944673216902421, 4.8496570781151149]], [[-5.3684657255817898, -7.2881587818117977], [-0.99544758891408058, 5.9581331992978557]]], [[[2.5174836633660291, -1.5143933548173463], [-6.9614425027632105, -7.1910173399761304]], [[-7.1165031575148507, -1.593426143491409], [-0.99357364342961496, -0.9497220739645007]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank3_Symbol_rank4(self):
      arg0=numpy.array([[[0.51677071894364079, 2.4226362111601567], [-0.16763901643237666, -4.3282004538193881]], [[2.6522488499010795, 1.3715256221174821], [4.2314809159322948, 4.1101063879770354]], [[-3.3866108602031253, 1.4660556619606071], [3.429740421795028, -2.6140326849307502]], [[-0.38897761965553457, 2.1366360569105955], [0.92949151184878875, -2.6220687656109885]], [[2.0516162803152058, -4.9792837437824389], [-3.6473998818341813, 3.5362941414594182]], [[3.2019618970696655, -2.4491438483789407], [-3.8134201048744898, -4.3909805996485236]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-2.5875293860181103, -3.5241596984462706], [-4.7934160670718367, -4.4390577580962454]], [[-4.9770060779473067, -3.4588076743198615], [-0.30511954951621956, 3.7260052723052066]]], [[[-0.14295413202049634, 3.4602858621434791], [-0.53061547346779125, 2.9440946155084804]], [[2.9313813652175034, 2.46844680090841], [4.2357146609208076, -0.80787183362547133]]], [[[2.9169059246069313, 2.1449955648891761], [-2.9109759733471274, -2.2816820434103602]], [[0.94601592878468477, -2.8954209295514199], [-0.17166524696927166, 0.02807249940303258]]], [[[-0.44318516852556744, -4.4110720354586341], [0.039489074647423017, 1.7125421653357815]], [[0.48877562836193622, -4.646761387559275], [4.5663642263475204, 3.5490727192541751]]], [[[-3.7657676082975478, 2.3011896908526133], [-1.9122450455378415, 3.1272163287667567]], [[3.3487038066933188, -1.895493836473674], [0.36105006064445089, -4.705378179027023]]], [[[-0.64500649309775504, -2.6778930649326158], [-0.008276398585564948, -1.8861377473775933]], [[-3.5749518466617172, 0.39223271531760773], [-1.2898957418988366, -4.0891201150099432]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-2.0707586670744695, -3.0073889795026298], [-2.3707798559116799, -2.0164215469360887]], [[-5.1446450943796833, -3.6264466907522381], [-4.6333200033356077, -0.6021951815141815]]], [[[2.5092947178805831, 6.1125347120445586], [0.84091014864969083, 4.3156202376259625]], [[7.1628622811497982, 6.6999277168407048], [8.345821048897843, 3.3022345543515641]]], [[[-0.46970493559619397, -1.2416152953139492], [-1.4449203113865203, -0.81562638144975308]], [[4.3757563505797128, 0.53431949224360809], [-2.7856979319000219, -2.5859601855277177]]], [[[-0.83216278818110201, -4.8000496551141687], [2.1761251315580186, 3.8491782222463771]], [[1.418267140210725, -3.7172698757104863], [1.9442954607365319, 0.92700395364318666]]], [[[-1.7141513279823419, 4.3528059711678191], [-6.8915287893202803, -1.8520674150156822]], [[-0.29869607514086249, -5.5428937183078553], [3.8973442021038691, -1.1690840375676048]]], [[[2.5569554039719105, 0.52406883213704969], [-2.4574202469645057, -4.335281595756534]], [[-7.3883719515362074, -3.4211873895568821], [-5.6808763415473607, -8.4801007146584659]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[3.7978523213958031, -3.4249619201712997, 1.2020459486643915, 1.7811263494475345], [3.1766964840442036, -1.7459004162025593, 2.1111694956411799, -2.6744227746348215], [-4.2771956337886223, -1.9443021965122509, 1.360485302796266, -4.5361105899383976]], [[-4.8094687455897969, -4.1129601198434962, -4.6394966631729897, -4.7967687844531293], [-0.94579188581287177, -1.8818963818276382, -1.4187918824384571, 2.099494741131581], [1.1610897568379253, 1.7490975966476308, -1.0222670797741653, 1.1520884157326758]]], [[[2.7164268492215706, 1.4016553297339271, 0.40448275338645878, 4.6805411872010332], [0.8859731342640913, -2.4549188767792272, -0.43584702040953882, -4.9052003093463163], [0.76746097302729055, 2.3087322259524745, 3.2869282983650141, -0.2939187696551846]], [[0.81675985908939541, -1.8972205229999703, -0.46127613159541614, -1.1611986958270517], [0.34433512256442533, 1.2005316816602916, 2.8708375475254977, 2.0862907330411442], [0.13295606474434063, 2.2046495410108129, 1.8937268896133794, -1.9106153474180498]]], [[[-3.0783464786859094, 0.66839057771206267, -0.34895773343027159, 2.778719645408855], [0.57643261903676191, -3.331763362836575, -1.7199606455804695, 4.5772131467584174], [-1.4391845802532277, -4.0749858307349696, -4.9525973379898289, -2.4161900730915531]], [[-3.1444136255323429, -4.0157360821329213, 3.3697425234109843, -0.0063677664014409885], [2.5554276868389438, 3.9466821487790522, -1.8960843078263223, 1.9016672093218823], [4.0700171032773618, -3.489456044163437, 2.5557694015917285, -2.8020054731745461]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[1.9703820602037192, 2.6252423049511533, -1.1437085892144006, 4.2729923680185937], [3.3684841162275951, -4.738475017717219, 0.43354709918337253, -4.3620376444220463], [1.0686471723804143, -2.9262561608049396, 2.4272486449483281, -4.5866877458730926]], [[1.2161615870743825, -4.6704811886088802, -4.3074024725758893, 3.4350821080584719], [-3.1029407593678049, 0.24906048316215745, 1.1694890199715324, 3.5958541087041258], [-0.17740363510561608, 1.4960617499856195, -3.9501980445025886, -2.0150234959201452]]], [[[4.3253368427265215, 3.4023309000065787, 4.5304543698818751, 3.6351365395428807], [-3.6574501776249893, 3.3480112903997323, 0.82578502331960202, 2.4374293760029175], [-2.6377895163094323, 1.3062778688324528, -2.9601939438373872, 1.7998259339038398]], [[3.1825124847200801, -2.3133041308403648, 0.92499552290408182, -3.3549649462015498], [4.1535306095417859, -2.4786830622540554, -2.0459846960320407, -1.1025651576105622], [-0.55354450182142489, -2.0270024222770111, -0.29973858049284008, -4.8351995850311882]]], [[[2.4758768213009743, 3.6799750892949579, 1.9988684521937516, 0.23875994372543197], [-2.9646681569347733, -2.489583818774598, -2.5745803998661376, 2.7216606906954057], [3.1443439406283566, 4.6133964395037435, -3.9822562804931172, -4.5906663055755859]], [[-3.8980044247627452, -4.6134973913830626, -0.65043170135567596, 2.2156024998670318], [-1.800343281761708, 4.7891230539973648, 3.0633056660331253, -0.78454692271157445], [-2.187202375695354, -4.6298317207546171, -4.8690144834446594, -1.7815525506474525]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[5.7682343815995223, -0.79971961522014645, 0.058337359449990966, 6.0541187174661282], [6.5451806002717987, -6.4843754339197783, 2.5447165948245525, -7.0364604190568674], [-3.208548461408208, -4.87055835731719, 3.7877339477445942, -9.1227983358114901]], [[-3.5933071585154144, -8.7834413084523764, -8.946899135748879, -1.3616866763946573], [-4.0487326451806762, -1.6328358986654807, -0.24930286246692468, 5.6953488498357068], [0.98368612173230918, 3.2451593466332502, -4.9724651242767539, -0.86293508018746934]]], [[[7.0417636919480922, 4.8039862297405058, 4.9349371232683339, 8.3156777267439139], [-2.771477043360898, 0.89309241362050518, 0.3899380029100632, -2.4677709333433988], [-1.8703285432821417, 3.6150100947849273, 0.32673435452762689, 1.5059071642486552]], [[3.9992723438094755, -4.2105246538403351, 0.46371939130866568, -4.5161636420286015], [4.4978657321062112, -1.2781513805937639, 0.824852851493457, 0.98372557543058203], [-0.42058843707708427, 0.17764711873380179, 1.5939883091205393, -6.745814932449238]]], [[[-0.60246965738493508, 4.3483656670070205, 1.6499107187634801, 3.0174795891342869], [-2.3882355378980114, -5.8213471816111735, -4.2945410454466071, 7.2988738374538231], [1.7051593603751289, 0.53841060876877389, -8.934853618482947, -7.0068563786671394]], [[-7.0424180502950886, -8.6292334735159848, 2.7193108220553084, 2.2092347334655908], [0.75508440507723584, 8.735805202776417, 1.167221358206803, 1.1171202866103078], [1.8828147275820077, -8.1192877649180542, -2.3132450818529309, -4.5835580238219986]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_float_rank0(self):
      arg0=Data(-3.70217402322,self.functionspace)
      arg1=4.85173293914
      res=add(arg0,arg1)
      ref=Data(1.14955891592,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([-4.0388736537610006, 2.7555235021305302]),self.functionspace)
      arg1=-3.21775048183
      res=add(arg0,arg1)
      ref=Data(numpy.array([-7.2566241355904131, -0.46222697969888227]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[-2.4752701793606899, 4.5192359481879993, -2.8733230595669301, 2.7283526152104347, -2.6447479412860018], [-1.7681337040034393, -0.065356322797431865, -3.8577243690454832, -0.00019227650198416768, -1.1841047338163033], [-0.82696763590178168, 0.86452411508529892, 1.8720913107645369, 1.4620527140783048, 3.6465708647242874], [-3.6762523672191638, 0.21157372538753627, -4.7488677235402683, 3.8984506149186622, -3.8610240182152999]]),self.functionspace)
      arg1=0.10295843898
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-2.3723117403806002, 4.6221943871680891, -2.7703646205868404, 2.8313110541905244, -2.5417895023059121], [-1.6651752650233496, 0.037602116182657852, -3.7547659300653935, 0.10276616247810555, -1.0811462948362136], [-0.72400919692169197, 0.96748255406538863, 1.9750497497446267, 1.5650111530583946, 3.7495293037043771], [-3.5732939282390741, 0.31453216436762599, -4.6459092845601786, 4.0014090538987519, -3.7580655792352102]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[2.475094689883802, -4.9633813705569931], [-0.38872267889866308, 0.37227688472055753]], [[-0.34809066022969581, 3.9364223796852453], [-2.0993492587916638, -4.2473922269876923]], [[1.5023016304558041, 3.6466009966679618], [-2.8414636311256611, 1.4602418844453933]], [[3.6809191713957166, 1.6083987140558325], [1.7204808567369732, -2.5553644209021931]], [[2.7304538595543111, 1.2922511134173931], [4.4298443632043529, 4.4765093085097014]], [[-2.4322683522459321, 2.9940636990239708], [4.527722934682366, 0.40442058727411201]]]),self.functionspace)
      arg1=4.61159539765
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[7.0866900875309664, -0.35178597290982871], [4.2228727187485013, 4.983872282367722]], [[4.2635047374174686, 8.5480177773324098], [2.5122461388555006, 0.36420317065947216]], [[6.1138970281029685, 8.2581963943151262], [1.7701317665215033, 6.0718372820925577]], [[8.292514569042881, 6.2199941117029969], [6.3320762543841376, 2.0562309767449714]], [[7.3420492572014755, 5.9038465110645575], [9.0414397608515173, 9.0881047061568658]], [[2.1793270454012323, 7.6056590966711353], [9.1393183323295304, 5.0160159849212764]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[-2.9482832182957628, -1.9868756352516472, -1.1164721319484472, 2.6159966367492657], [4.7434350949204358, 1.0014496568673401, -1.5446783825512354, -0.99638244678373233], [-4.448092778100543, -1.2901493876650005, -1.0470382121857491, -2.9005182538626242]], [[-1.4078990771671229, -1.5501781493842817, 2.0054547089554262, 4.2240917931882009], [0.2151771493445489, 0.11665166704638352, -0.63498415430187904, -2.9246570223107748], [-4.2148002452811442, 0.83681666454752968, -2.892992768248559, -0.25085859470182914]]], [[[4.4971882725757055, 2.6511736720426544, -4.2317161807649786, 0.23512845024567852], [2.6851166161233762, 2.9616373589105871, 2.5438790249791392, 1.7584468254528618], [2.5617779945944648, -4.1798433056583937, -3.8436070652387624, 3.1854310585876497]], [[3.3940169333880057, 3.419586933323508, 0.95994091294055028, -0.17134507671567878], [-4.6121758951074385, 1.7695660158414963, -4.571083745386975, 0.39289382783435212], [0.15195852194403781, 0.19362640385205321, 2.5437770274842322, 0.98400304611663536]]], [[[-0.539322435799515, -1.4905198611715686, 0.02259411616233642, -2.8354865930997617], [-0.39352434869564856, -4.3320236184606733, -4.6946410161446464, -4.8612213411740672], [-3.0182367060845117, -3.2954962133961829, -1.5922295937708473, 1.6919089261167342]], [[-2.4022778906047848, 3.9665768659659548, 3.8488350963728237, 1.060115020609067], [3.253607241799461, -1.7067658066002886, 4.5433966999663831, -3.51209948126849], [-1.6089520236451236, -2.268572058140339, -1.4125690516060532, 4.9985413592990966]]]]),self.functionspace)
      arg1=-2.2171118076
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-5.1653950258989232, -4.2039874428548076, -3.3335839395516076, 0.39888482914610535], [2.5263232873172754, -1.2156621507358203, -3.7617901901543958, -3.2134942543868927], [-6.6652045857037034, -3.5072611952681609, -3.2641500197889095, -5.1176300614657846]], [[-3.6250108847702833, -3.7672899569874421, -0.21165709864773419, 2.0069799855850405], [-2.0019346582586115, -2.1004601405567769, -2.8520959619050394, -5.1417688299139357], [-6.4319120528843046, -1.3802951430556307, -5.1101045758517198, -2.4679704023049895]]], [[[2.2800764649725451, 0.43406186443949402, -6.448827988368139, -1.9819833573574819], [0.46800480852021575, 0.7445255513074267, 0.3267672173759788, -0.45866498215029861], [0.34466618699130436, -6.3969551132615541, -6.0607188728419228, 0.96831925098448934]], [[1.1769051257848453, 1.2024751257203476, -1.2571708946626101, -2.3884568843188392], [-6.8292877027105989, -0.44754579176166409, -6.7881955529901354, -1.8242179797688083], [-2.0651532856591226, -2.0234854037511072, 0.32666521988107178, -1.233108761486525]]], [[[-2.7564342434026754, -3.7076316687747291, -2.194517691440824, -5.0525984007029221], [-2.610636156298809, -6.5491354260638337, -6.9117528237478068, -7.0783331487772276], [-5.2353485136876721, -5.5126080209993429, -3.8093414013740077, -0.52520288148642624]], [[-4.6193896982079448, 1.7494650583627944, 1.6317232887696633, -1.1569967869940934], [1.0364954341963006, -3.923877614203449, 2.3262848923632227, -5.7292112888716504], [-3.826063831248284, -4.4856838657434999, -3.6296808592092136, 2.7814295516959362]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_array_rank0(self):
      arg0=Data(-4.46049050539,self.functionspace)
      arg1=numpy.array(-1.81936212081)
      res=add(arg0,arg1)
      ref=Data(-6.27985262621,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([-4.637153952954753, -3.9618032965143857]),self.functionspace)
      arg1=numpy.array(-2.33021196473)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-6.9673659176813878, -6.292015261241021]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_array_rank1(self):
      arg0=Data(-2.59159619428,self.functionspace)
      arg1=numpy.array([2.8650096770997759, 2.0514954881782081])
      res=add(arg0,arg1)
      ref=Data(numpy.array([0.27341348281691902, -0.54010070610464878]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[1.8374891147197978, 0.43536058523132048, -4.932465963593291, -4.4379320508561211, 1.512069917737751], [-1.9224739906731756, 0.45895440531416298, 4.4710421016878552, -4.6553064980644852, 3.4376677701124176], [0.75575495970562034, 3.3355395868449165, -0.11018602327624638, -4.9743067170451365, -2.7230639052251657], [-3.9319046350544253, 2.4631178911522369, 3.9284287535826881, 3.3411839749442329, -4.1585508409021692]]),self.functionspace)
      arg1=numpy.array(1.86479543942)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[3.7022845541444998, 2.3001560246560224, -3.0676705241685891, -2.5731366114314191, 3.376865357162453], [-0.057678551248473653, 2.3237498447388649, 6.3358375411125571, -2.7905110586397832, 5.3024632095371196], [2.6205503991303223, 5.2003350262696184, 1.7546094161484556, -3.1095112776204346, -0.85826846580046379], [-2.0671091956297234, 4.3279133305769388, 5.79322419300739, 5.2059794143689349, -2.2937554014774673]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_array_rank2(self):
      arg0=Data(1.63679678235,self.functionspace)
      arg1=numpy.array([[1.3610535726892783, 4.587642130448053, 2.2700655633421309, 3.1341855541243344, 2.7801702328043891], [3.3432469389571082, 4.6204376815897561, -4.5632816965545917, -3.0835028681005774, -3.9061322338299007], [-2.9500154599022332, 4.2265447050362237, -2.563099682111678, -2.981922446799917, -4.9224438048803512], [-3.2595239579673994, -3.9241407085390181, 0.49851058055231334, -2.2316374413019666, 0.9944138130464717]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[2.9978503550352666, 6.2244389127940414, 3.9068623456881193, 4.7709823364703228, 4.4169670151503775], [4.9800437213030966, 6.2572344639357445, -2.9264849142086033, -1.446706085754589, -2.2693354514839124], [-1.3132186775562449, 5.8633414873822121, -0.92630289976568969, -1.3451256644539287, -3.2856470225343628], [-1.622727175621411, -2.2873439261930297, 2.1353073628983017, -0.59484065895597826, 2.63121059539246]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[1.7074873779289179, 2.426239847464231], [2.7131075331218142, 1.1192802007943961]], [[1.0508089962542355, 1.1748431729833442], [-2.1131509298509665, -2.1454947037749896]], [[3.3776048600947313, -1.9286638349952612], [-3.8320488708874656, 0.91686933932797032]], [[-0.16706671295512177, 1.6332854381796906], [2.5720801685635033, 1.0048958554999929]], [[-3.2933885575175292, -1.1503121687032438], [-1.0385182605836283, 4.0649714319356622]], [[-1.3305060118701997, -1.996607407732677], [3.4917931092630035, -1.0934314625871022]]]),self.functionspace)
      arg1=numpy.array(0.533087339573)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[2.2405747175016844, 2.9593271870369975], [3.2461948726945806, 1.6523675403671625]], [[1.583896335827002, 1.7079305125561106], [-1.5800635902782001, -1.6124073642022232]], [[3.9106921996674977, -1.3955764954224947], [-3.2989615313146992, 1.4499566789007368]], [[0.36602062661764467, 2.1663727777524571], [3.1051675081362697, 1.5379831950727594]], [[-2.7603012179447628, -0.61722482913047738], [-0.50543092101086184, 4.5980587715084287]], [[-0.79741867229743324, -1.4635200681599105], [4.0248804488357699, -0.56034412301433578]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_array_rank3(self):
      arg0=Data(0.323639419139,self.functionspace)
      arg1=numpy.array([[[4.6352846825536673, -0.92045031029851199], [-4.5852912499333156, 1.8521259500224652]], [[-3.108003407774004, -0.35382209639959861], [-2.5470364580834581, -4.8230957742275695]], [[1.4761867081280569, -3.1918089183331064], [1.5628033706215554, 1.4068410101426201]], [[2.7588042307895675, 0.491293260260937], [-3.6623187522937792, -3.1218022760755657]], [[0.3123049642961977, 0.70976792387954646], [4.4490362378435417, 2.5514026951595792]], [[-1.5232202576541787, -2.012094451697056], [-2.1591542717402366, -4.7440635198664873]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[4.9589241016931638, -0.59681089115901553], [-4.2616518307938192, 2.1757653691619616]], [[-2.7843639886345075, -0.030182677260102153], [-2.2233970389439617, -4.4994563550880731]], [[1.7998261272675533, -2.86816949919361], [1.8864427897610518, 1.7304804292821165]], [[3.0824436499290639, 0.81493267940043346], [-3.3386793331542828, -2.7981628569360693]], [[0.63594438343569415, 1.0334073430190429], [4.7726756569830382, 2.8750421142990756]], [[-1.1995808385146822, -1.6884550325575596], [-1.8355148526007401, -4.4204241007269909]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[0.51866211437750032, -1.1441924367307879, 0.10442422943416219, 3.0358720788728082], [-3.6336034039423626, -4.0585813151763208, 2.5204421297349153, 1.1758217778454307], [2.0643924996160168, -0.31277968094197828, -3.0158481276509419, -1.4944406322280845]], [[-0.5932700557688424, 3.0236496527613586, 0.4498118968522089, -4.8236227744504241], [3.922393095177938, 4.4671726010998078, -3.3779827255839114, -1.3273820269733005], [2.5936740361393831, 2.0581944847593769, -3.2558965806210138, 1.4309347502116951]]], [[[1.2588049093232652, -0.31863768793530589, 2.5779363911687518, 4.5736079545057944], [2.0416211709417977, -2.5115710825401747, 3.7050113040552688, -0.40377591666256674], [-2.9734228958152165, -1.3226778029143995, -3.4784680364669942, -4.9710791076741003]], [[-3.4028339951402464, 4.7028669167000956, -0.80078003404031595, 3.0492224182039198], [-0.17798459491821284, -1.8223593836454066, -1.2259419608797764, 2.6656034265059247], [-3.5286178324415554, 2.3523576869490004, 2.8694492929619049, -1.4142835719911306]]], [[[0.54393784608684648, 0.21752484903897784, 0.5422747368473253, 2.2831136795664193], [-3.7867288285021159, 3.4375568724404619, -0.036183191593337938, 3.9035157296903797], [-2.6939132537065911, 4.7631067929096726, 2.529953609833214, 0.43261964795205898]], [[2.939928571635769, 0.092543113665160881, -4.0034515340176391, 4.2272003703695376], [-1.5875861891810219, -3.5152322813944723, -1.0700311884085014, 3.415603735054848], [-4.7991922243908594, 1.2017062165732213, 2.1862285305106299, 4.2973423370934984]]]]),self.functionspace)
      arg1=numpy.array(1.87013132504)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[2.3887934394190333, 0.72593888831074516, 1.9745555544756952, 4.9060034039143412], [-1.7634720789008296, -2.1884499901347878, 4.3905734547764483, 3.0459531028869637], [3.9345238246575498, 1.5573516440995547, -1.1457168026094089, 0.37569069281344847]], [[1.2768612692726906, 4.8937809778028916, 2.3199432218937419, -2.9534914494088911], [5.792524420219471, 6.3373039261413409, -1.5078514005423784, 0.54274929806823247], [4.4638053611809161, 3.92832580980091, -1.3857652555794808, 3.3010660752532281]]], [[[3.1289362343647982, 1.5514936371062271, 4.4480677162102848, 6.4437392795473274], [3.9117524959833307, -0.64143975749864168, 5.5751426290968018, 1.4663554083789663], [-1.1032915707736834, 0.5474535221271335, -1.6083367114254612, -3.1009477826325673]], [[-1.5327026700987134, 6.5729982417416286, 1.0693512910012171, 4.9193537432454528], [1.6921467301233202, 0.047771941396126394, 0.64418936416175665, 4.5357347515474578], [-1.6584865074000223, 4.2224890119905334, 4.7395806180034379, 0.45584775305040237]]], [[[2.4140691711283795, 2.0876561740805109, 2.4124060618888583, 4.1532450046079523], [-1.9165975034605829, 5.3076881974819949, 1.8339481334481951, 5.7736470547319128], [-0.82378192866505806, 6.6332381179512057, 4.4000849348747471, 2.302750972993592]], [[4.810059896677302, 1.9626744387066939, -2.1333202089761061, 6.0973316954110706], [0.28254513586051111, -1.6451009563529393, 0.80010013663303159, 5.2857350600963811], [-2.9290608993493263, 3.0718375416147543, 4.0563598555521629, 6.1674736621350315]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_array_rank4(self):
      arg0=Data(1.83942343606,self.functionspace)
      arg1=numpy.array([[[[-0.31381356182938447, 2.0275196312649761, 0.32904364646013651, 3.7758294022020582], [-3.3318409123232762, -2.9080626624618624, 1.3396589078376593, 4.7226487195751226], [-4.8826814930462294, 4.6479050954828605, -0.027944398055384845, -2.2219923453083656]], [[3.2396572678215865, -0.71664758760689917, -0.25873443807402641, -2.4842174453587051], [-0.93804907892020672, -4.7280022258242269, -2.6591692152370063, 3.3749331498080508], [-3.9765843056890224, -0.5632741679991522, -1.3516415768460757, 4.039873869213741]]], [[[-3.722479657025942, 0.75949948342866413, 3.9436213175731343, -2.3006944794526598], [1.2348816390708954, -4.1616024834833576, -3.4441761948322847, -0.59417337173913687], [0.21742402943597217, 4.7597461316709122, -0.85658747348055098, -3.9669243501113396]], [[-4.7850431678998042, 0.77082059356661681, 2.4973390344321373, -0.58814403568971496], [-4.3424431556730401, -4.1205753114916801, 0.67251673161203041, 4.7223701278152532], [-3.8151216693520542, -1.4939316079923568, -2.9958271414194226, 4.1461125643039551]]], [[[-3.1102170354266114, 0.79878468738305752, -2.9005492462626092, -2.2493276758068594], [0.27153708944211008, 4.1879414862020532, -4.0469068202066865, 4.1989028420471257], [2.2355266623707246, -1.9476585717156745, 4.9943107625320646, -2.6783514458881816]], [[-1.3111479233058954, 1.5049997267928017, -0.43476775936564493, -4.9663794788011382], [-3.8443564405633568, -2.6255512163535375, 2.6265586656267228, 0.32892885298667984], [1.8913814709335766, -2.7353844873246072, -1.7953893673175472, -0.3031685889274609]]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[1.5256098742272162, 3.8669430673215768, 2.1684670825167371, 5.6152528382586588], [-1.4924174762666755, -1.0686392264052618, 3.1790823438942599, 6.5620721556317232], [-3.0432580569896288, 6.4873285315394611, 1.8114790380012158, -0.38256890925176501]], [[5.0790807038781871, 1.1227758484497015, 1.5806889979825742, -0.64479400930210451], [0.90137435713639391, -2.8885787897676263, -0.8197457791804057, 5.2143565858646514], [-2.1371608696324218, 1.2761492680574484, 0.48778185921052497, 5.8792973052703417]]], [[[-1.8830562209693413, 2.5989229194852648, 5.7830447536297349, -0.46127104339605918], [3.0743050751274961, -2.322179047426757, -1.604752758775684, 1.2452500643174638], [2.0568474654925728, 6.5991695677275128, 0.98283596257604966, -2.127500914054739]], [[-2.9456197318432036, 2.6102440296232174, 4.336762470488738, 1.2512794003668857], [-2.5030197196164394, -2.2811518754350795, 2.511940167668631, 6.5617935638718539], [-1.9756982332954536, 0.34549182806424383, -1.156403705362822, 5.9855360003605558]]], [[[-1.2707935993700108, 2.6382081234396582, -1.0611258102060086, -0.4099042397502588], [2.1109605254987107, 6.0273649222586538, -2.2074833841500858, 6.0383262781037264], [4.0749500984273253, -0.10823513565907383, 6.8337341985886653, -0.838928009831581]], [[0.52827551275070528, 3.3444231628494023, 1.4046556766909557, -3.1269560427445375], [-2.0049330045067562, -0.78612778029693686, 4.4659821016833234, 2.1683522890432805], [3.7308049069901772, -0.89596105126800651, 0.044034068739053467, 1.5362548471291397]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([4.0600397148448337, -0.11537044325742762]),self.functionspace)
      arg1=numpy.array([-2.7268734378624671, 0.80038633870505915])
      res=add(arg0,arg1)
      ref=Data(numpy.array([1.3331662769823667, 0.68501589544763153]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-2.4511079471568542, 3.0446430805506015], [-3.0318322512651008, 0.15204005742998916]]),self.functionspace)
      arg1=numpy.array([-1.6210016108970016, 1.9849774118721273])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-4.0721095580538558, 1.4236414696535999], [-1.0468548393929735, 2.1370174693021164]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_array_rank2(self):
      arg0=Data(numpy.array([3.1351577940799569, -0.068062691509139128]),self.functionspace)
      arg1=numpy.array([[-1.1383953408425054, -0.025743475399853999], [-1.454724508556958, 3.3308761949836114]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[1.9967624532374515, 3.1094143186801029], [-1.5227872000660971, 3.2628135034744723]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_array_rank1(self):
      arg0=Data(numpy.array([[[-4.4707087220289212, 0.57121875013525081, -3.92388339145886, 2.5377879831118069, 2.3962819274420859], [0.22163058622863474, 4.0726373239667861, -1.7935257043273047, 3.802603296938095, -2.7300969178255343], [-1.8868327714340563, 2.295041078430323, 2.6661210282279271, -2.7558492046355623, 0.87710245362669781], [-3.4072886148193615, -0.29897836286689383, 2.0445238693231724, -0.31401566904958322, -1.8276513192827859]], [[-0.28940437223873694, 2.6197485022212028, -4.8292185835862451, -0.52580308676740994, -3.2022907216854568], [2.6492833640990288, -2.0810724487766494, -3.0284341764873268, -3.7910575644196989, 1.7130435569842275], [3.153273481020797, 3.8838237223572634, 3.8617345034333379, -2.1715112103488297, -1.2163690423407947], [-4.6068450687494957, -3.6070536365920916, -2.2536525405032304, -1.4661565449047522, -4.8628179633327164]]]),self.functionspace)
      arg1=numpy.array([-2.0255349304962067, -3.3158920914180223])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-6.4962436525251279, -1.4543161803609559, -5.9494183219550667, 0.51225305261560017, 0.3707469969458792], [-1.803904344267572, 2.0471023934705794, -3.8190606348235114, 1.7770683664418883, -4.755631848321741], [-3.912367701930263, 0.26950614793411631, 0.64058609773172037, -4.781384135131769, -1.1484324768695089], [-5.4328235453155678, -2.3245132933631005, 0.018988938826965729, -2.3395505995457899, -3.8531862497789926]], [[-3.6052964636567593, -0.69614358919681951, -8.1451106750042683, -3.8416951781854323, -6.5181828131034791], [-0.66660872731899357, -5.3969645401946718, -6.3443262679053491, -7.1069496558377212, -1.6028485344337948], [-0.16261861039722536, 0.56793163093924104, 0.54584241201531558, -5.487403301766852, -4.5322611337588175], [-7.9227371601675181, -6.9229457280101139, -5.5695446319212527, -4.7820486363227745, -8.1787100547507379]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_array_rank3(self):
      arg0=Data(numpy.array([0.70221814219295542, 0.4423036630567978]),self.functionspace)
      arg1=numpy.array([[[4.662291394664944, 4.6230493309139025, -2.2214676544868039, -2.1857828664925116, 2.6285437359495925], [-0.37501089228856976, 2.8604296564504619, 4.5430349698530144, 2.077325201772303, 4.3877155242337746], [3.266638221778507, 4.8826074256898817, -4.1896941694344649, -1.2867298019556319, -0.60773772140887239], [-4.4228659359495044, -0.68590320211604716, -4.0990611425374475, 4.2160243000149595, 2.649620619810019]], [[4.2617420024253878, 4.2396520021092705, -4.5579005700477104, 0.82399929255670123, -0.47311540226721149], [2.3823719115189554, 1.3252107055558229, -3.0120598898376838, -1.6997927111615918, 2.3783837959823151], [3.9185591306312713, -0.0076172682085555721, 2.4347152908790681, -4.6543857699999238, 3.1248262072492725], [-1.3019194491298092, 0.050531041614807037, 3.9522307010598237, 3.3820281713519655, -0.14629290494374381]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[5.3645095368578994, 5.3252674731068579, -1.5192495122938485, -1.4835647242995562, 3.3307618781425479], [0.32720724990438566, 3.5626477986434173, 5.2452531120459698, 2.7795433439652584, 5.0899336664267301], [3.9688563639714625, 5.5848255678828371, -3.4874760272415095, -0.58451165976267649, 0.09448042078408303], [-3.720647793756549, 0.016314940076908258, -3.396843000344492, 4.918242442207915, 3.3518387620029744]], [[4.7040456654821856, 4.6819556651660683, -4.1155969069909126, 1.266302955613499, -0.030811739210413691], [2.8246755745757532, 1.7675143686126207, -2.569756226780886, -1.257489048104794, 2.8206874590391129], [4.3608627936880691, 0.43468639484824223, 2.8770189539358659, -4.212082106943126, 3.5671298703060703], [-0.85961578607301137, 0.49283470467160484, 4.3945343641166215, 3.8243318344087633, 0.296010758113054]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_array_rank1(self):
      arg0=Data(numpy.array([[[[4.5203023911755604, 4.2578161655634403], [-4.396607693607022, 3.3155633085999252]], [[3.5878097418058168, 0.91822613470749204], [2.3551210414913708, 2.4679071775972918]], [[-1.1724747742930495, 2.1460144565842079], [0.65349984091856772, -1.5995587145541448]], [[2.9745526987794699, -2.5976773613668627], [-1.2363950489328701, -0.786788800814886]], [[-2.9770056594607928, -3.7693725009511567], [3.8033459845913811, -0.7151665966626739]], [[3.2557652654101688, 0.27500237373339953], [-4.0322254541914635, -2.9154048040398917]]], [[[-4.6227964476736343, -2.4569040614394213], [2.3086990652470867, 3.2569174224260493]], [[-0.85990296615269468, 2.7674118363688027], [3.5639459003050682, 3.7006895004250193]], [[-2.501506561795972, 2.0305591065168427], [2.7154452335415913, 3.9411657527663486]], [[-0.48901970882291579, 2.8436842270246254], [-1.6912375497341294, 0.14479803889334697]], [[-2.7111764086779688, -3.7460657174795586], [-0.4673913128868934, -1.8835485471147893]], [[-1.7891375039530644, 0.35886594104741931], [-1.4811898958548877, 1.8999782557284375]]]]),self.functionspace)
      arg1=numpy.array([2.5939546410340428, -0.34024512478619062])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[7.1142570322096033, 6.8517708065974832], [-1.8026530525729791, 5.909517949633968]], [[6.1817643828398596, 3.5121807757415349], [4.9490756825254136, 5.0618618186313347]], [[1.4214798667409934, 4.7399690976182507], [3.2474544819526105, 0.99439592647989805]], [[5.5685073398135128, -0.0037227203328198399], [1.3575595921011727, 1.8071658402191568]], [[-0.38305101842674993, -1.1754178599171139], [6.3973006256254239, 1.8787880443713689]], [[5.8497199064442116, 2.8689570147674424], [-1.4382708131574207, -0.32145016300584883]]], [[[-4.9630415724598249, -2.7971491862256119], [1.9684539404608961, 2.9166722976398587]], [[-1.2001480909388853, 2.4271667115826121], [3.2237007755188776, 3.3604443756388287]], [[-2.8417516865821626, 1.6903139817306521], [2.3752001087554007, 3.600920627980158]], [[-0.82926483360910641, 2.5034391022384348], [-2.0314826745203201, -0.19544708589284365]], [[-3.0514215334641595, -4.0863108422657497], [-0.80763643767308402, -2.2237936719009799]], [[-2.129382628739255, 0.01862081626122869], [-1.8214350206410783, 1.5597331309422469]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_array_rank4(self):
      arg0=Data(numpy.array([2.305328651348387, 1.6870050592978192]),self.functionspace)
      arg1=numpy.array([[[[-3.5823712921820841, -4.7661652895814166], [-2.9107845426478764, -2.8159297375928762]], [[-4.9390706666034419, 0.6902446349508482], [4.5035356558943533, 1.9612509352748111]], [[2.9589285210225604, 1.0869462297341412], [-4.15351394043706, -1.7263392851719814]], [[-0.83650180487941483, -2.8374350612556909], [-4.1507135660404799, -3.5832752997038653]], [[0.7077819020008711, 1.0288725920807851], [2.0019545304033493, 4.5358891880777072]], [[-0.90365213196637129, 2.2560064049508357], [0.37145452209969498, -4.449481254059795]]], [[[0.99111445136182308, 4.8468090818917879], [0.88624028261150833, 1.0256445855808138]], [[1.7360373817416868, 3.6656943033767782], [2.1424021738960271, -2.7958331903411371]], [[2.9322244156624535, -2.0693472009108014], [-1.9016389010643087, -1.1282924148465723]], [[-1.6206806638374029, -2.9617208247303717], [1.9970697515409617, -4.8979362677151936]], [[3.39239075659167, -0.90329865520388353], [2.5968342573758925, -2.5178456003506966]], [[-1.8289276318276682, -1.8106616452381363], [4.2392012880775756, -1.4812504099724966]]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-1.277042640833697, -2.4608366382330296], [-0.6054558912994894, -0.51060108624448919]], [[-2.6337420152550548, 2.9955732862992352], [6.8088643072427404, 4.2665795866231981]], [[5.2642571723709475, 3.3922748810825283], [-1.848185289088673, 0.57898936617640562]], [[1.4688268464689722, -0.53210640990730385], [-1.8453849146920929, -1.2779466483554782]], [[3.0131105533492581, 3.3342012434291721], [4.3072831817517363, 6.8412178394260943]], [[1.4016765193820158, 4.5613350562992228], [2.676783173448082, -2.144152602711408]]], [[[2.6781195106596423, 6.5338141411896071], [2.5732453419093275, 2.712649644878633]], [[3.4230424410395059, 5.3526993626745973], [3.8294072331938462, -1.1088281310433179]], [[4.6192294749602727, -0.38234214161298219], [-0.21463384176648947, 0.55871264445124691]], [[0.066324395460416241, -1.2747157654325525], [3.6840748108387809, -3.2109312084173744]], [[5.0793958158894892, 0.78370640409393566], [4.2838393166737117, -0.83084054105287741]], [[-0.14192257252984897, -0.12365658594031714], [5.9262063473753948, 0.20575464932532261]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[-1.9320845300508047, -4.2147282407111408, -0.73849420894463513, -4.842365405809824, 1.5030991512265537], [-1.2809658343763966, 0.26410921855392466, 4.2763031923674841, 2.3659897741760769, -0.60378636152829923], [-4.9699051013426576, -0.52571846619295748, -3.8595681964347763, 4.8597754636803199, -2.6624930811061356], [2.884083906167203, -2.3166632910513441, -0.1066881280094627, 2.0589487040037104, -0.13272101140985537]]),self.functionspace)
      arg1=numpy.array([[-0.19240585263039822, 3.3206991589384991, 3.7873084573980194, -2.1801553850176472, -2.378444642997819], [-3.0730182785137039, -3.2731617514896447, 3.643658559836414, 1.2213231271040108, 4.4072634540960625], [-3.6873552097787989, 2.6393124014257836, -4.5899866626553845, 4.7042177388721118, 3.1656253089646764], [-2.0733582373907122, -1.0850315152837164, -0.093912783898064234, -3.1744513124290363, 3.1975404144255961]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-2.1244903826812029, -0.89402908177264173, 3.0488142484533842, -7.0225207908274712, -0.8753454917712653], [-4.3539841128901005, -3.0090525329357201, 7.9199617522038981, 3.5873129012800877, 3.8034770925677632], [-8.6572603111214566, 2.1135939352328261, -8.4495548590901599, 9.5639932025524317, 0.50313222785854084], [0.81072566877649077, -3.4016948063350605, -0.20060091190752694, -1.1155026084253259, 3.0648194030157407]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_array_rank2(self):
      arg0=Data(numpy.array([[[-2.1781603719717477, 1.5795712648004283], [-1.3532716582576878, 4.6105921396229608], [-4.4570946446485564, 3.9928960837210514], [3.5386313328886754, 4.1934896404356667], [1.5279658757051076, -1.0550353175293248]], [[-2.3118063151654766, 0.33347310806449482], [3.8772032676309713, 0.71091866289249772], [-2.1962528937369052, 1.2232002671618041], [-3.9762245138104424, 4.1927266418531985], [-3.110546352886189, 2.7921191671696395]], [[4.655489652059341, 2.3450215707548487], [-0.54670412920811629, -0.84689417292528546], [-3.5886659718034997, 3.3494651603502401], [4.4329481084067908, -0.65966817568961211], [2.5997659221368199, 4.5913686848023652]], [[-4.6453891566025955, 4.3982542056059231], [3.1194112729952224, -4.7118177396631369], [2.018812870987988, 3.4487434839616178], [4.6966236245059321, -0.00259849160589809], [1.2552007742037228, 0.71756340141766906]]]),self.functionspace)
      arg1=numpy.array([[0.72784218729464278, 4.7188717104841977, 2.8454977679989284, -0.34930410343847029, -0.95385510422056008], [0.72569189604733353, 2.9345589771971348, -3.198754428254329, 0.83260282199936597, -1.9033504505137087], [2.4098987795084748, -0.90384037804616657, 0.26346048333739169, 0.67285188374523575, 4.6418598885088276], [-2.4154888987955223, -1.6290532347443367, 4.9574474988775776, 2.2956380743502356, -4.1722105844732029]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-1.4503181846771049, 2.3074134520950711], [3.3656000522265099, 9.3294638501071585], [-1.611596876649628, 6.8383938517199798], [3.1893272294502051, 3.8441855369971965], [0.57411077148454748, -2.0088904217498849]], [[-1.5861144191181431, 1.0591650041118283], [6.8117622448281061, 3.6454776400896325], [-5.3950073219912342, -1.9755541610925249], [-3.1436216918110764, 5.0253294638525645], [-5.0138968033998976, 0.88876871665593082]], [[7.0653884315678157, 4.7549203502633235], [-1.4505445072542829, -1.750734550971452], [-3.325205488466108, 3.6129256436876318], [5.1057999921520265, 0.013183708055623633], [7.2416258106456475, 9.2332285733111927]], [[-7.0608780553981179, 1.9827653068104008], [1.4903580382508856, -6.3408709744074736], [6.9762603698655656, 8.4061909828391954], [6.9922616988561677, 2.2930395827443375], [-2.9170098102694801, -3.4546471830555339]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_array_rank3(self):
      arg0=Data(numpy.array([[2.9942323206677433, -3.9361938290250964, 3.9294687258775838, -2.9646435823954786, 0.090114578773836662], [3.7727446831260654, 2.168357306571508, 4.3636325292464235, -0.92992508298779164, 1.421653026831339], [2.6799491096713703, -2.728823112071753, 3.7489907271912735, -1.997074560904244, 3.2665741130708241], [-4.0204364907759036, 4.6270269592530902, 1.1874854226035865, 3.0210586900968437, -4.5354402220426895]]),self.functionspace)
      arg1=numpy.array([[[2.7361648989608955, 4.0719609374878836], [-4.4062694767228301, -1.9081236495058729], [0.62652576857205666, -4.9925151314536205], [-2.5851056582428331, 3.9473210696163097], [-3.3409316617197682, -4.0946067355157432]], [[0.75754583038537859, 4.1387930360739702], [-1.2718884050135104, -3.2890492695566955], [-4.3411744888532278, -0.18675977479497696], [-2.4484851446103759, 2.1943881740643505], [0.8891932702130152, 2.1333712433365832]], [[1.7077087403314959, -3.5514718480409391], [-2.2410250186346623, -0.18983036158680022], [4.6944431976494005, 3.9864401334998174], [4.1057256565884277, 3.3963075102956068], [0.2834226372389379, 3.2324102396505801]], [[-4.4511829715766034, 4.2541988661768855], [-4.7054436149644712, -4.211727931283578], [-0.87491301478121208, -0.97197301298026151], [-3.9609391127354998, 2.159773567684832], [0.34165179728378181, -3.9622176706265511]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[5.7303972196286388, 7.0661932581556268], [-8.3424633057479269, -5.8443174785309697], [4.5559944944496404, -1.0630464055760367], [-5.5497492406383113, 0.98267748722083104], [-3.2508170829459315, -4.0044921567419065]], [[4.530290513511444, 7.9115377192000356], [0.89646890155799763, -1.1206919629851875], [0.022458040393195766, 4.1768727544514466], [-3.3784102275981676, 1.2644630910765589], [2.3108462970443542, 3.5550242701679222]], [[4.3876578500028662, -0.87152273836956873], [-4.9698481307064153, -2.9186534736585532], [8.443433924840674, 7.7354308606910909], [2.1086510956841837, 1.3992329493913629], [3.549996750309762, 6.4989843527214042]], [[-8.471619462352507, 0.23376237540098188], [-0.078416655711381011, 0.41529902796951212], [0.31257240782237439, 0.21551240962332496], [-0.93988042263865612, 5.1808322577816757], [-4.1937884247589077, -8.4976578926692401]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[-4.9700547658814873, 2.0649778799688914, 2.9037937430448926, -1.2134605192031831, -3.5889224974269194], [-4.4154605462626453, 4.3407879469328385, 4.0385234118122728, -4.191705308028574, -2.1585389723174506], [1.7817299494911589, 4.6838072988233836, 0.47023565600587602, 1.9056725665499616, 4.8375076935135866], [2.8629876230000164, 4.2079005505555678, -4.456909125062305, 1.8232891075985407, -4.454048071998927]], [[-3.4446986023441339, -3.9284119578890375, 1.1386885121356052, 3.1657409324313335, 4.6276820833974082], [1.6948363924602852, 1.1734041003965432, -0.088895512855994951, 3.5240231369525095, -0.097026330424192508], [0.35550902238360127, 1.3199051697839899, -2.9671836923728678, -3.5758477420917201, -0.95695633780500344], [-1.0510196268021521, 2.9487822048406684, 1.0166378255922845, -1.4384827782177947, -0.55312525368082976]], [[-2.571988651492533, 4.7882843391676353, 2.8476061351992676, 4.5361542188742341, 1.1465727389974898], [1.9434882228501849, 3.2178911155908381, -2.0497414813210266, 1.8789073169638257, -2.1735302983587337], [3.8670804549015827, 4.9949483090535853, -2.300325686539435, 3.038470276509635, 4.3874580667430259], [0.95267166853862406, 4.5335197262149176, -4.9299417478935057, -3.5875750417439543, 3.2083640689417479]], [[-1.5905689083328864, -3.8082562682552412, 4.5370843089568407, 1.0670390818283799, 2.4164747351821632], [0.48919324552301013, 2.5258249170139173, -1.4753487371021121, 0.97466892734755284, 1.1654213925887857], [-0.41286040512113242, -1.6365527766047485, -2.474126479109402, 3.4797313598352169, -4.4583367550778794], [1.7908873655156139, 0.19540862144721061, -2.64790968897859, 2.4788497001001346, 3.2993882090749693]], [[4.84176140143658, -4.1431361827631621, 2.6332594661683837, -1.5234744097847077, 4.8617408841591647], [-3.174922324531785, 2.6608455692370256, 4.65357847972572, 2.361412317841415, 1.380793743171628], [3.8168831982438896, 0.65253507236860209, 2.0016878171900974, -1.5234039001017816, -2.840863246889108], [1.6616364089625097, -1.728234369200278, 4.7051214899017921, 2.155021881341824, -0.8919303104584797]]], [[[-3.4771525942253332, -4.189980268674983, -2.7005671276683474, -1.5992591428873082, -4.6517702027649648], [-1.1423967599882823, -2.0357273980435711, 4.6364502620704826, 1.199656918371435, -1.0155704872533544], [-2.6431756087702416, -3.2250265795609847, -3.8502843444289878, 2.9973012848743092, 4.5512544230787544], [4.6220866347659761, 2.1566504122687489, -4.0775668340269515, 4.1098987625734882, 4.1305422136766961]], [[-0.37261691368824579, -0.25327933960644522, 4.5326822867287184, -3.164546394177945, -2.5451433621236683], [-2.0775395225017301, -3.1647792853302636, -0.57852400789835201, 3.081604900667875, -3.4475454570844963], [-0.8259758367668848, 3.9132865311166523, 1.4881753443263754, -4.7241474860316215, -4.2682971113224184], [4.1131373192696827, -0.11795334878177943, -1.0929021264306407, 2.3170101964717418, 1.5169545020936939]], [[2.507919214149064, 4.2202843656340523, -3.5638946020598694, 2.5059391919146892, -2.1165421969758769], [-3.510806449018089, 0.22977752395416395, 0.92315394818643526, 2.9283054391412389, -0.4301942222081081], [4.969678596150306, -0.53070374387100383, -3.7461930772074292, -1.2121507980198909, 1.3521444228531259], [-0.72769219179613653, 2.5095537402174548, 2.463327876618516, -1.3527349935897046, 0.39371397630214933]], [[0.73644486219530592, -1.5806764329786671, -0.56799704503439052, -2.5023911510522865, 3.7080716368426359], [0.075892458480261382, 1.360778866831021, -1.5785832739407688, 1.7289477341333184, 1.9362452117676119], [-0.48481449737081928, -3.6920363393643374, -2.5505978587349687, -3.4310158638014068, 1.1237420114265584], [4.7066306155892228, 3.8731683372092345, 2.9219209257903014, 3.5907376925216088, 0.79797133286244915]], [[0.84737315658806711, -1.4194308349546461, 4.4316799837175012, 4.591549091825101, 0.76067883615131571], [-2.7056511957607743, -0.94403490342980945, 3.2860215424770107, 4.2981061190802627, 1.1388610341314473], [4.1920456079900763, -3.1373509775827504, 2.8089305872184749, -3.3566587967915904, -4.7951747822844002], [3.2410663439568541, -2.0940076733844393, -2.3438039890792051, -1.9898210650219084, -1.0922037793148673]]], [[[0.33004528257826315, 3.5425884890443786, -4.3912893970923115, 0.34723618423067748, -1.6979461936208318], [-0.13594097777508374, -1.3467920820529988, 2.4143429691153981, 3.7542072103210948, 1.6280572827506647], [-1.8555051693462641, -1.5512967286780888, -1.7154912167034073, 0.4371164460354473, -0.77273796463010935], [2.1220389510952318, 2.5814295633062105, 4.7685465718745803, 3.8119227851267006, -1.1696123563265468]], [[-0.30774019438682565, 2.9839485891920141, 0.40953410492603393, 1.0528630303200961, -4.9531582741475289], [-3.3389668869047351, 3.3352701090136652, 0.3660072982926792, -2.4612278453667038, 2.1574302208995828], [-3.3630998534958998, -4.1807675029755496, -4.9556196790876239, 0.81199919938800846, -3.067836963030274], [-0.14607817003201884, 3.2964211313233065, 4.3138513127565687, 2.7856683012606016, -0.063939812877231716]], [[3.5731769583204187, -1.5222004180299376, 3.4531895571741487, 3.5532511225215764, -3.9761433809212923], [2.9496535113162778, -2.6112684335169636, 0.6306912049635276, 1.9268583779771662, -2.4412408906982241], [-2.3506444529372192, 3.4687419019043233, -4.0242539877513899, -4.5510817126131453, -2.5056516088099281], [-0.5884526330254749, 3.5343599057203594, 1.7855590311803784, 4.3493757372868949, 1.739383371281181]], [[0.84159773870790566, 0.68954426577173855, 2.3075318499203199, 2.3438818920892999, 3.0413496751403652], [-2.7351716564199791, -0.003455061334015852, -2.1230729198641098, 0.37022319730896314, -0.50578314180592532], [-2.1647842005545592, -1.0042347704699059, 4.5937684683896389, 2.2558548672563417, 4.4945206389030119], [-3.494265389970054, 3.4726326497517768, 4.6041885097947439, -0.21106350269618446, -3.4527352257173605]], [[-2.0789456089854186, -1.9028773133586596, -2.4332902775468956, -4.8637073402165694, 2.5489091696788204], [-2.946479420316761, -0.37562357769046528, 2.1861268449379736, 0.83545879198728912, -2.8945839958477348], [0.92100123689378677, -0.86142738044056877, -3.4231797091871483, -0.80634221756806923, -3.6477994853292692], [-1.1725506659426035, 3.8704086160253652, -1.2917190685303881, -2.6196302075895117, -2.0094902577410223]]], [[[1.2330889669130629, -1.8742672788824222, 2.2753960318544566, -3.1864800296044726, 1.3384711820935387], [-3.0755907286926121, 0.99898788227850055, 3.5985619019125146, -1.3665837125597768, -0.66094101020231388], [-1.6358790410703317, 2.2279206975540378, 4.6407849724645303, -0.34169255823628308, 3.8446157804234637], [1.1149587606442415, 3.5071776770446839, -3.7307751391989088, 3.4623697957005408, -2.9076955723521838]], [[-2.159140416154596, -2.4949580066921948, 0.076191446421361775, 1.2356653342824551, 3.5260154963591024], [-4.0244124983817748, 0.74117061812666485, -1.4835834204065823, -4.6963833134970816, -4.0100835935128973], [2.0596244410264468, -4.4295446318395095, 3.6729529481064382, 2.6988246738359525, 0.69873962461121231], [0.17762031203197459, 4.1986052753977994, -1.2632165582546193, 0.62317776334872121, -4.4143058296643209]], [[-4.8039034163505221, -0.60143720755880636, 2.4089294533276862, -4.7643509334687852, 4.81424238331177], [0.66428319418270299, -4.300536755643404, -3.7284072361701517, 4.5666434629000836, -3.9874022457922429], [0.50418472316138541, -4.7888980976232443, -0.023909497678681113, 2.1610278949335369, 1.929540783437246], [2.1402766305007273, 1.1935989538200662, 4.1550037646877964, 3.404308694115679, 2.5329835899257489]], [[1.4888191745333028, 0.69636848487035241, -1.2706164573715908, -2.8779276991197023, 1.6464904279836476], [-4.384924849368951, -1.8327911943222155, -3.4440610353243342, -2.1762046104831922, -1.7383154151160927], [1.7782234750411785, -0.91193978272442422, -2.5298582869681105, 4.5559250749876377, -3.8340356641385478], [-3.5962949574957737, 0.09474979987928922, -3.8808830787944713, -3.5764255757578023, -2.7738103125204217]], [[3.1977021831978849, -1.342966071142043, 0.76478258539704047, -1.478528307182787, 1.5722864746862513], [4.7115774175607523, -4.4819293565046214, -1.5938225727928987, -4.6782758254700543, 0.61357617044959767], [1.745687039507259, -2.031451905403292, -3.129256947973623, 3.9824488166276311, 0.87300337082483725], [-1.2448555739466158, 1.1695783435891016, -3.6492369280759061, -0.73464086808373708, -4.1610125312137107]]]]),self.functionspace)
      arg1=numpy.array([[2.6402833506061025, 2.8969333613995953, -1.4180991289538158, 3.5513150826638107, -2.224728916286919], [-1.7461750333946702, 4.3200172502112544, 0.44036380286962551, 0.94918011809502634, 1.2328009032672913], [-0.81331161868630808, -1.8703542459569578, 1.0110179844198699, 3.8295780508473172, 1.6986359124668136], [-1.7135909616363509, 4.3264216269546747, 3.0777761514756499, -4.1596719668267212, -0.92290081916796041]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-2.3297714152753848, 4.7052612305749939, 5.5440770936509951, 1.4268228314029194, -0.94863914682081685], [-1.7751771956565428, 6.9810712975389411, 6.6788067624183753, -1.5514219574224715, 0.48174437828865191], [4.4220133000972615, 7.3240906494294862, 3.1105190066119786, 4.5459559171560642, 7.4777910441196891], [5.5032709736061189, 6.8481839011616703, -1.8166257744562024, 4.4635724582046432, -1.8137647213928245]], [[-0.5477652409445386, -1.0314785964894422, 4.0356218735352005, 6.0626742938309288, 7.5246154447970035], [4.5917697538598805, 4.0703374617961385, 2.8080378485436004, 6.4209564983521048, 2.7999070309754028], [3.2524423837831966, 4.2168385311835852, -0.07025033097327249, -0.67891438069212473, 1.9399770235945919], [1.8459137345974432, 5.8457155662402638, 3.9135711869918799, 1.4584505831818007, 2.3438081077187656]], [[-3.9900877804463488, 3.3701852102138194, 1.4295070062454518, 3.1180550899204182, -0.27152638995632605], [0.52538909389636901, 1.7997919866370222, -3.4678406102748425, 0.46080818801000989, -3.5916294273125495], [2.4489813259477669, 3.5768491800997695, -3.7184248154932509, 1.6203711475558191, 2.96935893778921], [-0.46542746041519178, 3.1154205972611018, -6.3480408768473211, -5.0056741706977697, 1.790264939987932]], [[1.9607461743309242, -0.25694118559143053, 8.0883993916206514, 4.6183541644921906, 5.9677898178459738], [4.0405083281868208, 6.077139999677728, 2.0759663455616986, 4.5259840100113635, 4.7167364752525964], [3.1384546775426783, 1.9147623060590622, 1.0771886035544087, 7.0310464424990275, -0.90702167241406872], [5.3422024481794246, 3.7467237041110213, 0.90340539368522066, 6.0301647827639453, 6.85070329173878]], [[2.6170324851496609, -6.3678650990500811, 0.40853054988146464, -3.7482033260716268, 2.6370119678722457], [-5.3996512408187041, 0.4361166529501066, 2.428849563438801, 0.13668340155449599, -0.84393517311529109], [1.5921542819569705, -1.572193843918317, -0.22304109909682168, -3.7481328163887007, -5.065592163176027], [-0.5630925073244093, -3.952963285487197, 2.4803925736148731, -0.069707034945095003, -3.1166592267453987]]], [[[-5.2233276276200034, -5.9361553020696531, -4.4467421610630176, -3.3454341762819784, -6.397945236159635], [-2.8885717933829524, -3.7819024314382412, 2.8902752286758124, -0.54651811502323522, -2.7617455206480246], [-4.3893506421649118, -4.9712016129556549, -5.596459377823658, 1.2511262514796391, 2.8050793896840842], [2.875911601371306, 0.4104753788740787, -5.8237418674216217, 2.363723729178818, 2.3843671802820259]], [[3.9474003365230086, 4.0667379106048092, 8.8526995369399728, 1.1554708560333093, 1.7748738880875861], [2.2424777277095242, 1.1552379648809907, 3.7414932423129024, 7.4016221508791293, 0.87247179312675804], [3.4940414134443696, 8.2333037813279066, 5.8081925945376298, -0.40413023582036711, 0.051720138888835976], [8.4331545694809371, 4.2020639014294749, 3.2271151237806137, 6.6370274466829962, 5.8369717523049482]], [[2.9482830170186896, 4.6606481685036778, -3.1235307991902439, 2.9463029947843147, -1.6761783941062514], [-3.0704426461484635, 0.67014132682378946, 1.3635177510560608, 3.3686692420108644, 0.010169580661517408], [5.4100423990199316, -0.090339941001378321, -3.3058292743378037, -0.77178699515026539, 1.7925082257227514], [-0.28732838892651102, 2.9499175430870803, 2.9036916794881416, -0.91237119072007911, 0.83407777917177484]], [[1.6856249802903323, -0.63149631488364077, 0.38118307306063581, -1.5532110329572602, 4.6572517549376622], [1.0250725765752877, 2.3099589849260473, -0.62940315584574247, 2.6781278522283447, 2.8854253298626382], [0.46436562072420706, -2.7428562212693111, -1.6014177406399424, -2.4818357457063804, 2.0729221295215847], [5.6558107336842491, 4.8223484553042608, 3.8711010438853277, 4.5399178106166351, 1.7471514509574755]], [[2.0801740598553584, -0.18662993168735476, 5.6644808869847925, 5.8243499950923923, 1.993479739418607], [-1.472850292493483, 0.28876599983748186, 4.518822445744302, 5.530907022347554, 2.3716619373987387], [5.4248465112573676, -1.9045500743154591, 4.0417314904857662, -2.1238578935242991, -3.5623738790171089], [4.4738672472241454, -0.86120677011714797, -1.1110030858119138, -0.75702016175461706, 0.14059712395242396]]], [[[-0.48326633610804492, 2.7292768703580705, -5.2046010157786196, -0.4660754344556306, -2.5112578123071398], [-0.94925259646139182, -2.1601037007393069, 1.6010313504290901, 2.9408955916347868, 0.81474566406435667], [-2.6688167880325722, -2.3646083473643968, -2.5288028353897154, -0.37619517265086078, -1.5860495833164174], [1.3087273324089237, 1.7681179446199025, 3.9552349531882722, 2.9986111664403925, -1.9829239750128549]], [[-2.1780944403437834, 1.1135943432350563, -1.4608201410309238, -0.81749121563686167, -6.8235125201044866], [-5.2093211328616924, 1.4649158630567074, -1.5043469476642786, -4.3315820913236616, 0.28707597494262505], [-5.2334540994528576, -6.0511217489325073, -6.8259739250445817, -1.0583550465689493, -4.9381912089872317], [-2.0164324159889766, 1.4260668853663487, 2.443497066799611, 0.91531405530364385, -1.9342940588341895]], [[4.5841949427402886, -0.51118243361006765, 4.4642075415940186, 4.5642691069414463, -2.9651253965014224], [3.9606714957361477, -1.6002504490970937, 1.6417091893833975, 2.9378763623970361, -1.4302229062783542], [-1.3396264685173493, 4.4797598863241932, -3.01323600333152, -3.5400637281932754, -1.4946336243900582], [0.42256535139439499, 4.5453778901402293, 2.7965770156002483, 5.3603937217067648, 2.7504013557010509]], [[4.6711757895552228, 4.5191223166190557, 6.1371099007676371, 6.173459942936617, 6.8709277259876824], [1.0944063944273381, 3.8261229895133013, 1.7065051309832073, 4.1998012481562803, 3.3237949090413919], [1.6647938502927579, 2.8253432803774112, 8.4233465192369561, 6.0854329181036588, 8.3240986897503291], [0.33531266087726319, 7.3022107005990939, 8.4337665606420611, 3.6185145481511327, 0.3768428251299567]], [[-0.38030969651860502, -0.20424140089184606, -0.73465436508008208, -3.1650714277497558, 4.247545082145634], [-1.2478435078499475, 1.3230123347763483, 3.8847627574047872, 2.5340947044541027, -1.1959480833809213], [2.6196371493606003, 0.83720853202624479, -1.7245437967203348, 0.89229369489874433, -1.9491635728624557], [0.52608524652421007, 5.5690445284921788, 0.40691684393642547, -0.92099429512269815, -0.31085434527420874]]], [[[-0.48050199472328803, -3.5878582405187731, 0.56180507021810566, -4.900070991240824, -0.37511977954281228], [-4.789181690328963, -0.71460307935785039, 1.8849709402761636, -3.0801746741961278, -2.3745319718386648], [-3.3494700027066826, 0.51432973591768683, 2.9271940108281793, -2.055283519872634, 2.1310248187871128], [-0.59863220099210945, 1.7935867154083329, -5.4443661008352597, 1.7487788340641899, -4.6212865339885347]], [[2.1672812108000787, 1.8314636202624799, 4.4026130733760365, 5.5620869612371298, 7.8524371233137771], [0.30200912857289985, 5.0675922450813395, 2.8428382065480924, -0.36996168654240691, 0.31633803344177736], [6.3860460679811215, -0.10312300488483483, 7.9993745750611129, 7.0252463007906272, 5.025161251565887], [4.5040419389866493, 8.5250269023524741, 3.0632050687000554, 4.9495993903033959, -0.087884202709646253]], [[-1.7261272648748722, 2.4763389439168435, 5.486705604803336, -1.6865747819931354, 7.8920185347874199], [3.7420593456583529, -1.2227606041677541, -0.65063108469450182, 7.6444196143757335, -0.90962609431659303], [3.5819608746370353, -1.7111219461475944, 3.0538666537969688, 5.2388040464091867, 5.0073169349128959], [5.2180527819763771, 4.2713751052957161, 7.2327799161634463, 6.4820848455913289, 5.6107597414013988]], [[-2.6708527922934184, -3.4633034819563688, -5.430288424198312, -7.0375996659464235, -2.5131815388430736], [-8.5445968161956714, -5.9924631611489367, -7.6037330021510554, -6.3358765773099135, -5.8979873819428139], [-2.3814484917855427, -5.0716117495511455, -6.6895302537948318, 0.39625310816091641, -7.993707630965269], [-7.7559669243224949, -4.064922166947432, -8.0405550456211934, -7.736097542584524, -6.9334822793471425]], [[2.2748013640299245, -2.2658668903100034, -0.15811823377091994, -2.4014291263507475, 0.64938565551829086], [3.7886765983927919, -5.4048301756725818, -2.5167233919608591, -5.6011766446380147, -0.30932464871836274], [0.8227862203392986, -2.9543527245712524, -4.0521577671415834, 3.0595479974596707, -0.049897448343123152], [-2.1677563931145762, 0.24667752442114121, -4.5721377472438665, -1.6575416872516975, -5.0839133503816711]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_array_rank4(self):
      arg0=Data(numpy.array([[-1.8504406608823598, -2.8363327414069204, -4.8271502388533065, 2.0111747761759391, -4.0554142996533518], [2.7194235016753847, 3.1409085428898145, 2.0876404107112432, 0.34391250234105364, -4.920458281762297], [-3.3365000946098231, 0.11826104196156528, -3.1914542970517932, 3.0849123451699096, 2.6047062761915249], [-3.7385123819818276, 1.373147195583627, -2.1370572651830431, -1.2741422717312401, 2.624735104927427]]),self.functionspace)
      arg1=numpy.array([[[[3.6532793056013197, 3.8382702051462161, -3.5503938291348449, 0.083905514649425861, 3.316790547727944], [2.9237825900302141, 0.19988725779366412, -4.5823644935603616, 0.60077748258611052, 4.9988117884803742], [-4.1230597195519181, 3.4874572793548495, 3.5772119965223599, -3.0094089513739561, -4.5871865964944138], [0.30524411685735231, -2.0754188961813638, -3.5020985683924675, 3.365292335025087, -4.2475223503071193]], [[-1.4992521616159959, -3.0059706833702338, 2.5087008545936147, 1.2763998192186774, 4.4702955954579888], [-4.3389435603686577, -2.6485174515735266, 2.8808715048718492, 4.2680612491810841, 3.1487164140207007], [-0.19384150927883326, -3.4379017048238012, -0.58990566756911189, 4.0729395187399469, 4.259871636121618], [0.062555814536718657, 2.7711315102343601, 1.5341515622724513, -0.91162217409097668, 3.6067563895408465]], [[2.856903853056072, 1.6084638151521808, 0.30641971826199743, -3.4432231636783572, 0.56143054426989192], [-0.75960384296481287, 2.3736482868079358, 2.5206474563138883, 3.0579923679184944, 2.2679133349993643], [-2.72747998343121, -4.9741542795110982, -2.9122379025019187, -3.4861407979045902, -2.3652196545092541], [1.5946994512476653, 0.73716163306312588, 1.0724634269307707, 0.011255128560408778, 4.3695327571017124]], [[1.6819522299678047, 2.7571336524381316, -0.22598958865249941, 2.3696792575171752, 4.2036332736270197], [-3.8104935067461643, -1.7462864648897058, -3.2986795334798211, -0.24071075683630117, 2.6412544076065103], [1.4228964615913569, -3.4018272968441696, -1.9877569157967621, 2.3023603976009959, 1.9233400049575824], [2.4866839963386846, 4.7458882713670132, 2.7531286361989746, 1.2335784261870808, 1.4465688364867226]], [[-2.0198280153224681, 4.9312252312567111, -3.3489469629804178, 2.4259619499712413, -4.4060084222352636], [-2.6973818224560899, 4.3002428764981602, -0.022350001053212054, -1.5247590122521815, -2.9338325689197453], [-1.078607792509171, 4.1736967241560698, -1.3956467380782236, 2.9763983361055484, 4.6419226328684093], [1.8120382150969334, 1.6290491361555777, 3.7391612073189968, 0.26318894148691641, 2.1557947058275868]]], [[[-2.6006896854403418, 2.7089703137484591, 2.1724704939008541, -1.3765002257421068, 3.582941097466195], [-0.20404488578883306, 3.2438863176879096, 0.96203466227754397, 1.4991400309840959, -0.053623381430732309], [-0.7341249607060174, -3.3632135940653987, -2.2100806955459342, -0.15796390094655255, -1.4427083353290717], [1.6158813481389878, -2.8931105660242826, 3.3986681784655488, 0.95029107529536283, 0.63940307873799807]], [[-3.3657127201946615, 1.1246731941417689, 4.1302650492343442, -2.2155312451097431, 0.67339743761309201], [1.0022816774707373, -0.56838976781985018, -0.20105363771409124, 2.9423249707294623, -3.1195474542001076], [4.548585789844843, -2.300523504658083, -1.9674434091704951, -4.6012074221631138, 0.43086135992619745], [-4.2472000776891807, -3.478573792852131, -4.3926793243192801, 2.5988338233703629, 3.3255925707170242]], [[1.1449528977706516, -3.6028448482792195, 0.91968154551955905, -0.41670627833488894, -4.9482955281205552], [-2.7930349678062436, 0.98187241800066349, 3.1381411133229129, 2.9169404795123803, -2.5191197000825785], [3.888415827342385, 2.5299182788116532, -2.5345808595950361, 1.9509543921497823, -1.9550476260596472], [0.87284133286654519, 0.43918978461349045, 3.9198239317452614, 0.38345186616903426, -0.91928061127495386]], [[3.7265249427676537, 0.61510295953765759, -0.61529338239021758, -4.9567948648034621, -0.8208991583212546], [-1.3122118736108135, 3.8844339360550464, 0.52606331943843454, -3.7633809061749348, -1.0095651521588311], [-1.6068920873982053, 1.5164776916552274, 3.8586774819726504, 0.1100814617070629, -4.3849973175014139], [-4.0539617427999364, 4.3004043736240423, -3.8415271946714427, 4.3670323481811586, -4.1713876212871526]], [[-4.5476404677056967, -4.5545341682252811, -2.3130112573377968, -3.8778118209940682, 1.3543289027729983], [0.38354150343953197, 0.19758029326423276, 1.0005924330496576, 3.5206829083086451, 4.6617208553415939], [2.5109613092816305, 0.81648239702776948, -0.69444223437013441, 2.3214356086729824, -4.5851599463358426], [0.92532589909704033, 0.6995085921621591, -0.92525701037154295, 1.4200875939615507, -4.581712026223971]]], [[[-4.744010226658375, -3.1281153299631272, 0.098080927921980532, -3.9990370888629414, 1.0064914562456906], [-4.0661193806249063, 4.1744921146548926, 0.55804142532373291, -1.2169872662313983, 4.5951708724984854], [-3.1641306471921538, -1.547250055824092, 2.8357478724562126, -3.1858311727081743, -0.57876811738590295], [-0.77679340312770151, -0.01940866776820549, 1.5364897352991029, -1.6463877426326947, -2.763646357552374]], [[-0.16299047467600403, -1.9749218138973514, 2.7084700663560524, 4.7804281465562841, -1.5094520190795424], [0.64842178524310601, 4.5226523482003564, 0.27217403703919629, 0.11013097469305588, -0.55785486152657437], [4.734396062098039, -2.4324095256745473, -0.34551662964918606, -1.6986059109685692, -4.1607847826828603], [-1.2965245610745635, -2.4304311895302, -2.5756475408192356, 4.4177501697215842, -2.4567671741888453]], [[0.5724991371397703, 0.84817652200181115, -4.7473093380079137, 1.9006963988863426, -4.6673390545532687], [-4.9672478159667302, -1.934999995073702, 3.2024463743762794, 1.4806382648816099, 1.7979107277106499], [2.515540098589879, -0.78371540682911878, 2.4450049354257821, 0.71535082226472824, 4.0736346927234699], [1.7979884852863757, 3.2848842411243986, -3.4637678392130722, 4.4652998287947661, -4.8421326807835383]], [[-0.492295108256946, 3.2142156120066652, 0.7253366300514017, 0.95028598873630088, -4.1307250106488684], [-1.3765317833992166, 1.1854249102068524, -2.2784336330295982, -1.5300129687574482, -1.2532345825674374], [0.96111203643523346, 0.39711306721492257, -0.37874555645027641, 1.1121650564930938, -4.5130331107714312], [-1.4473481036588822, 2.9839997737475423, 0.96592674961505764, 1.7697493371554351, -2.2229560136976891]], [[4.6038494861542443, 3.5680709870550356, -0.33729108910015793, -0.034874041608912165, 0.79777161100426408], [0.91128818829224389, -2.59372004914739, 4.9176082964769918, 4.3303917263798972, 4.2962102621833171], [-1.1024023662408933, 1.485853488123869, -1.9324756524126876, -0.27782504774569716, -4.3744262051191507], [0.22966590447020963, -1.0940159056705667, -4.7713651711497533, -0.69063287560011144, 4.727180555671918]]], [[[3.2915447259678547, -1.3153037841087292, 2.7018148095282228, 2.4620363164311252, 3.165850773767735], [-3.944651495211895, 2.8112955351478996, -4.9501401658879116, -1.3151766756977725, 1.9698299283384424], [-0.91762080014117586, 4.8902684882315306, -4.4137618355065591, -3.2762852145976087, 2.5938229138134181], [2.5903845338480957, 2.2832948541419293, -1.1213411022171851, -2.3488275528209068, -4.115599669585511]], [[4.4329770121103849, -4.144134187831872, 2.5010611951499193, -0.46174132721717687, 2.9544355509010316], [-1.1552382826327658, -2.165536264710588, -1.4243178654063904, 3.6508452594997376, -2.616603762896188], [3.2991752767987652, -2.3090875900557628, -3.7698140913654941, 4.8638116113284653, 0.30515872686834111], [-4.175866722833713, -1.3780554867310055, -2.3887162698745712, -4.2818132532152937, -3.4300702319577603]], [[3.5074374109130506, -4.6592513610273869, 0.1620875192520117, 0.89504270582155154, 4.3996642453542094], [0.7023221008754934, 3.446117838748826, 3.4521288165791493, 1.3755620476184127, -4.4620533881212072], [-2.4786127898680208, 1.2007579229376066, -2.2585807112717027, 1.3575171112082014, -1.4382997316804169], [-2.8067445338171138, 0.74067079981306527, -1.8080837963182517, 0.54530654769651576, 3.498773003793044]], [[-3.1549000974647514, 4.6626661976750441, -3.6352646867645966, -1.2930491929809627, 3.2949005340813393], [-4.8205063476101842, 1.5024321780493146, -0.55194997523220479, -2.2903458659744738, 4.4611321783202502], [-2.2485338833883084, 1.7912950425386232, 1.5484325602305038, -2.269161005690691, -1.5714035193695377], [-1.7532378707847105, 1.0881684644122629, -0.47714655123338545, 2.3713910206525703, 1.3932594153403244]], [[-4.6364108993728967, -4.9939702778762651, 1.0691956671758964, 1.6306568851169096, 4.6704907586278299], [3.0452538207147999, -2.2980867560748974, 1.3355499155003177, 1.3502486310699826, 3.0670212357859441], [2.7991817030423549, -2.8481594998495319, -1.4843886373465853, 2.5025521491700076, -4.3742869078691946], [-3.2518661606781452, -2.2101167820569714, -1.5589223736378557, 2.5440302015391802, -2.8349688499274284]]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[1.8028386447189599, 1.9878295442638563, -5.4008344900172052, -1.766535146232934, 1.4663498868455842], [1.0733419291478543, -1.6505534030886957, -6.4328051544427218, -1.2496631782962493, 3.1483711275980144], [-5.9735003804342774, 1.6370166184724897, 1.7267713356400001, -4.8598496122563155, -6.4376272573767732], [-1.5451965440250075, -3.9258595570637236, -5.3525392292748268, 1.5148516741427271, -6.0979630111894796]], [[-4.3355849030229159, -5.8423034247771541, -0.32763188681330568, -1.559932922188243, 1.6339628540510684], [-7.1752763017755781, -5.4848501929804474, 0.044538763464928799, 1.4317285077741637, 0.31238367261378031], [-3.0301742506857536, -6.2742344462307216, -3.4262384089760323, 1.2366067773330265, 1.4235388947146976], [-2.7737769268702017, -0.06520123117256027, -1.3021811791344691, -3.7479549154978971, 0.77042364813392616]], [[-1.9702463857972345, -3.2186864237011257, -4.5207305205913091, -8.2703734025316642, -4.2657196945834146], [-5.5867540818181194, -2.4535019520453707, -2.3065027825394182, -1.7691578709348121, -2.5592369038539422], [-7.5546302222845165, -9.8013045183644039, -7.7393881413552252, -8.3132910367578958, -7.1923698933625602], [-3.2324507876056412, -4.0899886057901806, -3.7546868119225358, -4.8158951102928977, -0.45761748175159411]], [[3.6931270061437438, 4.7683084286140707, 1.7851851875234397, 4.3808540336931143, 6.2148080498029588], [-1.7993187305702252, 0.26488831128623325, -1.287504757303882, 1.7704640193396379, 4.6524291837824494], [3.4340712377672959, -1.3906525206682305, 0.023417860379177036, 4.313535173776935, 3.9345147811335215], [4.4978587725146237, 6.7570630475429523, 4.7643034123749137, 3.2447532023630199, 3.4577436126626617]], [[-6.0752423149758199, 0.87581093160335932, -7.4043612626337696, -1.6294523496821105, -8.4614227218886153], [-6.7527961221094417, 0.24482857684480841, -4.0777643007065638, -5.5801733119055328, -6.9892468685730975], [-5.1340220921625228, 0.11828242450271809, -5.4510610377315754, -1.0790159635478034, 0.58650833321505758], [-2.2433760845564183, -2.4263651634977741, -0.31625309233435495, -3.7922253581664354, -1.899619593825765]]], [[[0.11873381623504287, 5.4283938154238438, 4.8918939955762388, 1.3429232759332779, 6.3023645991415798], [2.5153786158865516, 5.9633098193632943, 3.6814581639529287, 4.2185635326594806, 2.6658001202446524], [1.9852985409693673, -0.64379009239001395, 0.50934280612945049, 2.5614596007288322, 1.276715166346313], [4.3353048498143725, -0.17368706434889791, 6.1180916801409335, 3.6697145769707475, 3.3588265804133828]], [[-0.22480417730484703, 4.2655817370315834, 7.2711735921241587, 0.92537729778007138, 3.8143059805029065], [4.1431902203605517, 2.5725187750699643, 2.9398549051757232, 6.0832335136192768, 0.021361088689706875], [7.6894943327346574, 0.8403850382317315, 1.1734651337193194, -1.4602988792732994, 3.5717699028160119], [-1.1062915347993663, -0.3376652499623165, -1.2517707814294656, 5.7397423662601774, 6.4665011136068387]], [[3.2325933084818947, -1.5152044375679763, 3.0073219562308022, 1.6709341323763542, -2.860655117409312], [-0.70539455709500043, 3.0695128287119067, 5.2257815240341561, 5.0045808902236235, -0.43147928937133528], [5.9760562380536282, 4.6175586895228964, -0.44694044888379292, 4.0385948028610255, 0.13259278465159596], [2.9604817435777884, 2.5268301953247336, 6.0074643424565046, 2.4710922768802774, 1.1683597994362893]], [[4.0704374451087073, 0.95901546187871123, -0.27138088004916394, -4.6128823624624085, -0.47698665598020096], [-0.96829937126975985, 4.2283464383961, 0.86997582177948818, -3.4194684038338812, -0.6656526498177775], [-1.2629795850571517, 1.860390193996281, 4.202589984313704, 0.45399396404811654, -4.0410848151603602], [-3.7100492404588827, 4.6443168759650959, -3.497614692330389, 4.7109448505222122, -3.8274751189460989]], [[-9.4680987494679947, -9.4749924499875782, -7.2334695391000938, -8.7982701027563657, -3.5661293789892987], [-4.5369167783227651, -4.7228779884980643, -3.9198658487126394, -1.399775373453652, -0.25873742642070319], [-2.4094969724806665, -4.1039758847345276, -5.6149005161324315, -2.5990226730893147, -9.5056182280981396], [-3.9951323826652567, -4.2209496896001379, -5.84571529213384, -3.5003706878007463, -9.5021703079862689]]], [[[-8.0805103212681981, -6.4646154245729504, -3.2384191666878426, -7.3355371834727645, -2.3300086383641325], [-7.4026194752347294, 0.83799202004506945, -2.7784586692860902, -4.5534873608412214, 1.2586707778886623], [-6.5006307418019773, -4.8837501504339151, -0.50075222215361048, -6.5223312673179974, -3.9152682119957261], [-4.1132934977375246, -3.3559087623780286, -1.8000103593107202, -4.9828878372425178, -6.1001464521621971]], [[-0.044729432714438744, -1.8566607719357862, 2.8267311083176176, 4.8986891885178494, -1.3911909771179771], [0.76668282720467129, 4.6409133901619217, 0.39043507900076158, 0.22839201665462117, -0.43959381956500909], [4.8526571040596043, -2.314148483712982, -0.22725558768762077, -1.5803448690070039, -4.0425237407212951], [-1.1782635191129982, -2.3121701475686347, -2.4573864988576704, 4.5360112116831495, -2.33850613222728]], [[-2.6189551599120229, -2.343277775049982, -7.9387636350597068, -1.2907578981654506, -7.8587933516050619], [-8.1587021130185242, -5.1264542921254952, 0.01099207732448626, -1.7108160321701833, -1.3935435693411433], [-0.67591419846191414, -3.975169703880912, -0.74644936162601105, -2.4761034747870649, 0.88218039567167672], [-1.3934658117654175, 0.093429944072605409, -6.6552221362648654, 1.2738455317429729, -8.0335869778353306]], [[2.5926172369129636, 6.2991279571765748, 3.8102489752213113, 4.0351983339062105, -1.0458126654789588], [1.7083805617706931, 4.270337255376762, 0.80647871214031142, 1.5548993764124615, 1.8316777626024723], [4.0460243816051431, 3.4820254123848322, 2.7061667887196332, 4.1970774016630035, -1.4281207656015216], [1.6375642415110274, 6.068912118917452, 4.0508390947849673, 4.8546616823253448, 0.8619563314722205]], [[7.2085557623457692, 6.1727772632465605, 2.267415187091367, 2.5698322345826128, 3.402477887195789], [3.5159944644837688, 0.010986227044134989, 7.5223145726685168, 6.9350980025714222, 6.9009165383748421], [1.5023039099506317, 4.0905597643153939, 0.67223062377883736, 2.3268812284458278, -1.7697199289276258], [2.8343721806617346, 1.5106903705209582, -2.1666588949582284, 1.9140734005914135, 7.3318868318634429]]], [[[-0.44696765601397281, -5.0538161660905567, -1.0366975724536047, -1.2764760655507024, -0.57266160821409251], [-7.6831638771937225, -0.92721684683392791, -8.6886525478697401, -5.0536890576796001, -1.7686824536433852], [-4.6561331821230034, 1.151756106249703, -8.1522742174883867, -7.0147975965794362, -1.1446894681684094], [-1.1481278481337318, -1.4552175278398982, -4.8598534841990126, -6.0873399348027348, -7.8541120515673386]], [[5.8061242076940118, -2.7709869922482451, 3.8742083907335463, 0.9114058683664501, 4.3275827464846586], [0.21790891295086112, -0.79238906912696105, -0.051170669822763415, 5.0239924550833646, -1.243456567312561], [4.6723224723823922, -0.93594039447213584, -2.3966668957818671, 6.2369588069120923, 1.6783059224519681], [-2.802719527250086, -0.0049082911473785629, -1.0155690742909442, -2.9086660576316667, -2.0569230363741333]], [[1.3703801457300075, -6.79630862621043, -1.9749697459310314, -1.2420145593614915, 2.2626069801711663], [-1.4347351643075497, 1.309060573565783, 1.3150715513961062, -0.76149521756463034, -6.5991106533042503], [-4.6156700550510639, -0.93629934224543643, -4.3956379764547453, -0.77954015397484167, -3.57535699686346], [-4.9438017990001573, -1.3963864653699778, -3.9451410615012947, -1.5917507174865273, 1.3617157386100009]], [[-4.4290423691959919, 3.388523925943804, -4.9094069584958362, -2.5671914647122027, 2.0207582623500993], [-6.0946486193414238, 0.22828990631807455, -1.8260922469634449, -3.5644881377057138, 3.1869899065890102], [-3.5226761551195485, 0.51715277080738309, 0.27429028849926373, -3.5433032774219311, -2.8455457911007778], [-3.0273801425159506, -0.18597380731897717, -1.7512888229646255, 1.0972487489213303, 0.11911714360908432]], [[-2.0116757944454697, -2.3692351729488381, 3.6939307721033234, 4.2553919900443367, 7.2952258635552569], [5.6699889256422269, 0.32664834885252958, 3.9602850204277447, 3.9749837359974096, 5.6917563407133711], [5.4239168079697819, -0.22342439492210486, 1.1403464675808417, 5.1272872540974346, -1.7495518029417676], [-0.62713105575071815, 0.41461832287045564, 1.0658127312895713, 5.1687653064666073, -0.21023374500000136]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[2.9295934407975333, -0.022953977101789036], [3.8426148511765046, -2.0541619110840283]], [[0.49914405954185259, 1.262535784013453], [-0.67379065793777748, -2.8556140197531068]], [[-4.0340293357317716, -2.6924249007230525], [-1.2670314098871041, -3.9797951932072406]], [[3.9586914937153299, 4.1908074865794518], [-1.3451155928593304, 4.2845253843202915]], [[-3.904255067699347, -2.098649915297738], [-2.1142335986987471, -3.1401481209624285]], [[-2.8356637680848786, -4.154142925604873], [-3.4759311065004193, 0.035527190383055185]]]),self.functionspace)
      arg1=numpy.array([[[4.5960706695683697, 4.7364947830420832], [2.5494422462953015, 2.4444081208934056]], [[2.2905526869981312, 2.4522743919313843], [-4.0593090173433124, -4.0847951687678998]], [[-2.5947808103875749, 4.5126837193697398], [-0.20988593962329638, 0.37118169755922548]], [[-3.2380114636465507, 3.0225940737337513], [2.8345369387574806, 4.1690032199862443]], [[-3.6839830104566182, 2.7091607111134586], [0.37838314659913852, 2.0201744591960225]], [[4.0974245059860941, 3.7051657536319453], [-1.3884338676082066, -0.36606888037739971]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[7.525664110365903, 4.7135408059402941], [6.3920570974718061, 0.3902462098093773]], [[2.7896967465399838, 3.7148101759448373], [-4.7330996752810899, -6.9404091885210066]], [[-6.6288101461193465, 1.8202588186466873], [-1.4769173495104004, -3.6086134956480151]], [[0.72068003006877923, 7.2134015603132031], [1.4894213458981502, 8.4535286043065359]], [[-7.5882380781559657, 0.61051079581572054], [-1.7358504520996085, -1.119973661766406]], [[1.2617607379012155, -0.44897717197292764], [-4.8643649741086259, -0.33054168999434452]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[-1.1105618428722597, 1.8898357521019697], [1.7060585672749546, -3.543932953039203]], [[-2.9810973234580338, -0.56909485651525493], [3.6453701062293788, 2.2785764915447952]]], [[[4.8808849061801443, -4.4558048797668537], [-2.4702346510924378, 4.3299748951875738]], [[-3.3433064246320408, -2.4392685742087918], [-4.8036026108101098, 0.08934050248499581]]], [[[-3.5642298467477707, -4.2584223883556396], [2.0113358494981846, -4.9858920374356401]], [[0.47243677050237576, -2.9044945680973813], [4.8054115813574008, 1.3269541144092925]]], [[[-2.7757017735555167, -3.3743344962517008], [4.4937832137968829, 1.0401437725019234]], [[0.28377296192499646, 0.081357790660399942], [-4.049194432772528, -4.2240182444480903]]], [[[-0.072581817229327328, -2.3675588684202831], [-2.2676776339716054, 1.0173933471558758]], [[-3.2366983391624951, 0.33864896215610063], [-2.2086625320955133, -0.83323130239046783]]], [[[-3.2071402775939961, 1.865831552201124], [-4.8473600622222257, -4.1002954699214946]], [[1.8857157027729174, 0.46239747728970126], [-2.9837460204809751, 1.1827796500804624]]]]),self.functionspace)
      arg1=numpy.array([[[-3.4279773964888296, -1.7544458302707113], [1.1195313615761684, 3.1507196820727241]], [[-3.4020743873280734, 3.2487466348095246], [-2.9007384295017036, -4.0934777664462789]], [[1.4950620430640438, 0.47885512750264425], [4.5206513564772823, -0.26704042814054141]], [[2.1112502036665077, -2.7765208737794165], [2.72130261888387, 1.2191359671176727]], [[-0.59171662844424056, 1.5222712759577064], [-1.7057791542085021, -2.5707114680235685]], [[-1.8961702313037145, -1.5862226961739259], [-2.8807040697993602, 4.5415883030730253]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-4.5385392393610893, -1.5381416443868599], [-0.048387262995756686, -5.2983787833099143]], [[-1.8615659618818654, 0.55043650506091346], [6.796089788302103, 5.4292961736175194]]], [[[1.4788105188520708, -7.8578792670949271], [0.77851198371708685, 7.5787215299970985]], [[-6.2440448541337439, -5.340007003710495], [-8.8970803772563887, -4.0041372639612831]]], [[[-2.0691678036837269, -2.7633603452915958], [2.4901909770008288, -4.5070369099329959]], [[4.9930881269796581, 1.6161567883799011], [4.5383711532168594, 1.059913686268751]]], [[[-0.664451569889009, -1.2630842925851931], [1.7172623400174665, -1.7363771012774931]], [[3.0050755808088665, 2.80266040954427], [-2.8300584656548553, -3.0048822773304176]]], [[[-0.66429844567356788, -2.9592754968645236], [-0.74540635801389898, 2.5396646231135822]], [[-4.9424774933709976, -1.3671301920524015], [-4.7793740001190823, -3.4039427704140364]]], [[[-5.1033105088977102, -0.030338679102590582], [-6.4335827583961516, -5.6865181660954205]], [[-0.99498836702644278, -2.418306592509659], [1.5578422825920502, 5.7243679531534877]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_array_rank4(self):
      arg0=Data(numpy.array([[[-1.0022509017152572, -0.97128347719572794], [3.8291120514057404, -2.6073780136790581]], [[-4.6442543334396618, 0.49326194151392322], [-4.0329153901290562, 1.6475753105371904]], [[1.3052716594646885, -2.0322876948774193], [2.285086931979901, 4.1512735010060648]], [[2.9300304777155528, -1.8272573859124961], [4.877225352571017, 4.0371637665467794]], [[-0.92077005533962275, 1.5749312333814656], [-3.5569374686891244, 1.6177494832372421]], [[-1.999043313723301, -2.0717938251991197], [-2.7731880510579709, 0.90676393818851864]]]),self.functionspace)
      arg1=numpy.array([[[[3.3601294900683438, -1.1438245311706288], [4.9582493984582801, -1.3186579004159906]], [[-1.4112280196684699, -3.1151589401352577], [2.2813067292958253, 4.8744031542502668]]], [[[-2.154125122291497, -1.6889414514857828], [2.0081756542723088, -0.37210652919471787]], [[-3.2096157563042294, -1.0354966160527592], [4.0455395197222614, -1.640548194464416]]], [[[-2.9856845940018442, 0.24781208838427471], [-2.6171906421502689, 2.1444639180345728]], [[-0.83325135838088649, 4.633949857390828], [2.6421749179535103, -2.1505747502187731]]], [[[-2.3404990457858563, 4.9363772102492671], [-0.080060660225039548, -4.0666522440475319]], [[-2.4285776997370356, 0.31860620926353533], [2.7943619457320708, -1.7574186159953795]]], [[[3.9161849145367036, 1.4130795586554434], [0.20865653033566112, -0.36471362569950561]], [[0.40669720481564831, -3.6604401243137117], [3.0247930776975824, -2.8004984755145799]]], [[[0.37737898292344418, -1.0730633840604509], [2.0616097325652563, 4.0499349246826579]], [[2.6407091657000077, -3.5076407714004509], [0.81405656236420576, -1.5870442656990891]]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[2.3578785883530866, -2.146075432885886], [3.9869659212625521, -2.2899413776117186]], [[2.4178840317372705, 0.71395311127048267], [-0.32607128438323274, 2.2670251405712087]]], [[[-6.7983794557311583, -6.3331957849254445], [2.501437595786232, 0.12115541231920535]], [[-7.2425311464332855, -5.0684120061818154], [5.6931148302594519, 0.0070271160727743975]]], [[[-1.6804129345371557, 1.5530837478489632], [-4.6494783370276878, 0.11217622315715348]], [[1.4518355735990145, 6.919036789370729], [6.7934484189595752, 2.0006987507872918]]], [[[0.58953143192969648, 7.8664076879648199], [-1.9073180461375356, -5.893909629960028]], [[2.4486476528339813, 5.1958315618345523], [6.8315257122788502, 2.2797451505513999]]], [[[2.9954148591970808, 0.49230950331582068], [1.7835877637171267, 1.21021760768196]], [[-3.1502402638734761, -7.2173775930028361], [4.6425425609348245, -1.1827489922773378]]], [[[-1.6216643307998568, -3.0721066977837519], [-0.01018409263386344, 1.9781410994835382]], [[-0.13247888535796326, -6.2808288224584219], [1.7208205005527244, -0.68028032751057044]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[-3.481847182847396, -2.3876539612529668, -2.3828202286545999, -4.7530618084668514], [4.1744287704076548, -2.0577081812541644, 4.2720140368084962, -0.1074184962800846], [-3.5427177874600635, -0.49081441349427024, 1.3340666910124419, -1.7567626592076904]], [[3.0524677847303217, 2.8198300664586116, -3.6049073663164153, 2.1718193692331766], [2.0471442522902796, -2.3573446349203309, -3.5858279121157111, -3.2552265294638536], [-4.281228619661074, 4.7482811318359168, -1.0265319422051111, -0.96529227243463289]]], [[[2.7143953228094251, 0.70464726394083677, 1.4895836907594218, -4.5699498342776437], [-3.4624268263897684, -3.9808272265120417, -2.4523465205068593, 1.6353192501639118], [4.4569186592514338, -4.0126816683807496, -0.84064331824644434, 4.1047716291729461]], [[3.0861492852406087, -1.7061680368120156, 2.1825786414243176, -3.3179710316397548], [1.8385238994267521, 1.2984914416810618, -2.8409661688329866, 2.7507430812620406], [1.6162731839591711, -3.5486304441573879, -2.9094525130097058, -0.29095404088268406]]], [[[4.7148645371006825, 1.3498595803407518, -0.76977757187106732, -0.3327965905983028], [-3.1848813108916052, 3.4837846351433388, -0.66253916996538909, -0.36722583999284808], [1.2301758515399204, -1.2525086124431728, 2.0369128265708447, 2.7322490337187073]], [[1.5115100858340957, 0.19785731942984075, -0.45924599980360892, -2.7658732651547044], [4.9841568544822952, 4.9533858239330257, 4.8288305386579182, 0.61515989945531757], [0.20791920467341818, 4.8615502153984131, -3.1571293141749313, 0.82633102932093649]]]]),self.functionspace)
      arg1=numpy.array([[[[1.1458749053760329, -2.788847986131632, 4.9715326740858519, -4.1651337118816389], [-3.3414009732994208, -0.62556652143260472, 0.013303843956538053, -0.29194770692027472], [-2.2365692092134304, -3.6546967722010759, 2.338299344909335, 1.7621180182876737]], [[-2.2022824874186675, -3.953833999385826, 0.38120453554165756, 1.5155675692913304], [3.9115956838790407, 2.1244382879606789, 3.2796088949420685, -0.73195256864863811], [-2.5475454573896195, 3.5453561503213944, -2.8917437810687496, 2.6976913317627291]]], [[[0.50057587352964372, -0.64580117566214312, 0.048013546018085052, -3.5320352476761863], [-2.9367576753893609, 1.3881963913797239, -1.7086000338958982, -0.95108012692120791], [1.477128269683087, 3.2814160685338472, 3.8539496074237523, -0.73080478516992109]], [[4.3647246802462885, 2.9131621741231957, 3.9216827392446394, 0.41282755478774913], [3.3843080482743755, 1.5324071661313576, -1.038277511760338, 3.7593508175006978], [3.6720350944394795, 1.4379879811713874, -0.90335001977796647, -4.6471060924493592]]], [[[-3.799780189992914, -0.14688681692678962, -0.94193192663695768, 3.3624709251990996], [0.722649943499313, -1.479028839496328, 4.1720786335214157, -2.7342862871806171], [3.9751023967772667, -4.5749480840170031, 4.7602711306347167, 0.87693716039519476]], [[4.2725044473243496, -0.29808884947164405, 2.6438399247585185, -2.7612425065835531], [-4.7525492609572071, -2.975439327009497, -4.3896040685255091, -3.3682985092808035], [-4.8468966687554795, 4.8910452366278943, -1.1618382484700462, -2.8934265173744791]]]])
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-2.3359722774713632, -5.1765019473845992, 2.588712445431252, -8.9181955203484904], [0.83302779710823405, -2.6832747026867692, 4.2853178807650343, -0.39936620320035932], [-5.7792869966734939, -4.1455111856953462, 3.6723660359217769, 0.0053553590799833017]], [[0.85018529731165415, -1.1340039329272145, -3.2237028307747577, 3.6873869385245071], [5.9587399361693203, -0.23290634695965196, -0.30621901717364253, -3.9871790981124917], [-6.8287740770506939, 8.2936372821573112, -3.9182757232738608, 1.7323990593280962]]], [[[3.2149711963390688, 0.058846088278693642, 1.5375972367775068, -8.1019850819538295], [-6.3991845017791293, -2.5926308351323177, -4.1609465544027575, 0.68423912324270386], [5.9340469289345208, -0.73126559984690243, 3.0133062891773079, 3.373966844003025]], [[7.4508739654868972, 1.2069941373111801, 6.104261380668957, -2.9051434768520057], [5.2228319477011276, 2.8308986078124194, -3.8792436805933246, 6.5100938987627384], [5.2883082783986506, -2.1106424629860006, -3.8128025327876722, -4.9380601333320433]]], [[[0.91508434710776854, 1.2029727634139622, -1.711709498508025, 3.0296743346007968], [-2.4622313673922922, 2.0047557956470108, 3.5095394635560266, -3.1015121271734651], [5.2052782483171871, -5.8274566964601764, 6.7971839572055615, 3.6091861941139021]], [[5.7840145331584454, -0.1002315300418033, 2.1845939249549096, -5.5271157717382575], [0.23160759352508808, 1.9779464969235288, 0.43922647013240912, -2.753138609825486], [-4.6389774640820614, 9.7525954520263074, -4.3189675626449775, -2.0670954880535426]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_constData_rank0(self):
      arg0=Data(2.81669635066,self.functionspace)
      arg1=Data(-4.27193197955,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(-1.45523562889,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([-1.2722838660544209, -0.078341567137614199]),self.functionspace)
      arg1=Data(0.553001768536,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-0.71928209751885941, 0.47466020139794729]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_constData_rank1(self):
      arg0=Data(-3.13805793854,self.functionspace)
      arg1=Data(numpy.array([-1.8598756348282897, -1.8834652997833814]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([-4.9979335733691226, -5.0215232383242148]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[-0.011486191128216028, -0.73565714997192533, 4.9702597169280356, -4.3417019877044876, -2.647529511335609], [2.0797200047707696, 4.6827166184896178, 3.3840050815718801, -4.6553890653573937, 0.95008709285794612], [-2.1804613977495366, 4.4198127772394642, 0.021280988110524035, -3.3318906430130255, -3.3975091766699119], [-4.6533392277318173, 4.3790958978854064, 4.0188314478768259, -4.0695990098103429, -2.2330709456770172]]),self.functionspace)
      arg1=Data(3.55539857675,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[3.5439123856236998, 2.8197414267799905, 8.5256582936799514, -0.78630341095257172, 0.90786906541630685], [5.6351185815226854, 8.2381151952415337, 6.9394036583237959, -1.0999904886054779, 4.505485669609862], [1.3749371790023792, 7.97521135399138, 3.5766795648624399, 0.22350793373889033, 0.15788940008200392], [-1.0979406509799015, 7.9344944746373223, 7.5742300246287417, -0.51420043305842711, 1.3223276310748986]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_constData_rank2(self):
      arg0=Data(4.17163555199,self.functionspace)
      arg1=Data(numpy.array([[4.9202090367916522, -0.40302305677359662, 0.36282212633141064, -2.6500183676328124, 2.5806192909344405], [0.24516597948315066, 3.1942460000463448, 3.3510150152258831, 0.17740275660391713, 4.844785779047653], [3.009594084360776, -4.9041097114533212, -2.7372562637989395, -2.4691558668981193, -0.35305372922915979], [2.062835158903578, 3.0301117309054337, 0.18723338012349444, -0.68202285070412838, 3.8530680792077874]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[9.0918445887769881, 3.7686124952117392, 4.5344576783167465, 1.5216171843525235, 6.7522548429197764], [4.4168015314684865, 7.3658815520316807, 7.5226505672112189, 4.349038308589253, 9.0164213310329888], [7.1812296363461119, -0.73247415946798533, 1.4343792881863964, 1.7024796850872166, 3.8185818227561761], [6.2344707108889139, 7.2017472828907696, 4.3588689321088303, 3.4896127012812075, 8.0247036311931232]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-4.2070769921368614, -2.6138515088084247], [4.0366489478150918, 0.41618369888741125]], [[-3.6114513002930293, -3.2171687348097278], [-4.2145055520141579, -4.1689731884983363]], [[2.1280722139699897, 2.4356821828016253], [-2.8270883787736514, 1.2148858783645275]], [[-0.59334119285139053, -1.8064816132489678], [2.2181017752329621, -2.1763373917055984]], [[-4.2326665284923806, 0.11744935332349371], [3.7441434175829116, 4.0824420862185029]], [[4.8958883066125818, -3.8834618906485985], [-4.286798694212365, 3.9829487469590408]]]),self.functionspace)
      arg1=Data(1.90754577472,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-2.299531217421678, -0.70630573409324127], [5.9441947225302751, 2.3237294736025946]], [[-1.7039055255778459, -1.3096229600945444], [-2.3069597772989745, -2.2614274137831529]], [[4.0356179886851731, 4.3432279575168087], [-0.91954260405846799, 3.1224316530797109]], [[1.3142045818637929, 0.10106416146621555], [4.1256475499481455, -0.26879161699041498]], [[-2.3251207537771972, 2.0249951280386771], [5.651689192298095, 5.9899878609336863]], [[6.8034340813277652, -1.9759161159334151], [-2.3792529194971817, 5.8904945216742242]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_constData_rank3(self):
      arg0=Data(-2.9257829744,self.functionspace)
      arg1=Data(numpy.array([[[2.155825346374078, -2.2355272705888574], [-2.4982912749978468, 1.5147028414039196]], [[0.62264304246157387, -3.9930594754023394], [-3.220345685333259, 4.4815747684583052]], [[0.36650265335908561, 2.4151162213861763], [-0.32249450620113151, 0.44691173009491081]], [[-2.6608761054019769, -3.5835263077609336], [-0.327157737885722, 4.9299655477775044]], [[-1.9254493101904337, -0.13666080338197606], [-2.2881714380247251, -3.394043293186022]], [[1.8360050689679426, 3.8448243083356886], [-0.52373729401489655, -1.4423727681113894]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-0.76995762802761591, -5.1613102449905508], [-5.4240742493995402, -1.4110801329977742]], [[-2.30313993194012, -6.9188424498040337], [-6.1461286597349529, 1.5557917940566113]], [[-2.5592803210426083, -0.51066675301551756], [-3.2482774806028254, -2.4788712443067831]], [[-5.5866590798036704, -6.5093092821626275], [-3.2529407122874159, 2.0041825733758105]], [[-4.8512322845921272, -3.0624437777836699], [-5.2139544124264194, -6.3198262675877164]], [[-1.0897779054337513, 0.91904133393399468], [-3.4495202684165904, -4.3681557425130837]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[0.052461366809564147, -4.110553750932981, -3.2046082146170929, -0.31754260721337424], [4.0718393409836686, -3.8415723202774785, -4.6814600197970515, 3.0212401690620787], [-1.886251777821105, 2.2933065015486562, 1.7904320415762083, 3.4176886607381576]], [[-0.308518660376639, -1.2762490732849594, -3.9525746553103467, -2.7136066969327888], [0.015874401754583545, 2.4872917665120173, -0.09920490899863843, 2.9236384580342909], [-3.9635529278954085, 1.6125649798675834, -0.57829975570831049, -3.8637274097805232]]], [[[4.3008813023538988, -0.19799601601000383, 2.6335304037413394, 3.0390289084656068], [4.2857319183800158, 1.8253667067908044, 2.0658566027637786, 4.5382029592533577], [-0.10270297234483028, -0.55666074466669802, 0.26779088616885893, -3.2063809336359239]], [[3.6022521053950065, -4.7973124074982776, -4.596256829313587, -2.1713860334569546], [-1.2986618824491405, -3.8085756433063866, 3.2139711781225433, -0.17023631110438942], [3.35954025645451, 1.4179758565321068, 4.6733191443893656, 1.8169543465856801]]], [[[-2.9981539477088734, 2.5985034269690788, -4.004282578189633, 3.1067868096416547], [3.8001015457698255, 3.8900971908968192, 0.91367740982576606, -4.9343855721000072], [-1.7126115698597322, -1.0273396330819464, -2.2720615258718437, 4.5107733167636699]], [[3.2568786389269846, 2.0846778758482207, -2.4570017108968214, 4.4523565141609325], [-2.8246967018982505, -0.01712939581611117, -2.3268237863899257, -0.27024352919098771], [-3.9880311513268154, 3.6714790796125882, 3.2057486410962639, -3.330089253357571]]]]),self.functionspace)
      arg1=Data(2.38178018512,self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[2.4342415519300289, -1.7287735658125163, -0.82282802949662814, 2.0642375779070905], [6.4536195261041334, -1.4597921351570138, -2.2996798346765868, 5.4030203541825434], [0.49552840729935976, 4.675086686669121, 4.172212226696673, 5.7994688458586223]], [[2.0732615247438257, 1.1055311118355053, -1.5707944701898819, -0.33182651181232403], [2.3976545868750483, 4.8690719516324821, 2.2825752761218263, 5.3054186431547556], [-1.5817727427749437, 3.9943451649880481, 1.8034804294121543, -1.4819472246600585]]], [[[6.6826614874743635, 2.1837841691104609, 5.0153105888618041, 5.4208090935860715], [6.6675121035004805, 4.2071468919112691, 4.4476367878842433, 6.9199831443738224], [2.2790772127756345, 1.8251194404537667, 2.6495710712893237, -0.82460074851545917]], [[5.9840322905154713, -2.4155322223778128, -2.2144766441931223, 0.21039415166351016], [1.0831183026713243, -1.4267954581859219, 5.595751363243008, 2.2115438740160753], [5.7413204415749748, 3.7997560416525715, 7.0550993295098303, 4.1987345317061449]]], [[[-0.6163737625884087, 4.9802836120895435, -1.6225023930691682, 5.4885669947621194], [6.1818817308902902, 6.2718773760172839, 3.2954575949462308, -2.5526053869795424], [0.66916861526073257, 1.3544405520385183, 0.10971865924862101, 6.8925535018841346]], [[5.6386588240474493, 4.4664580609686855, -0.075221525776356657, 6.8341366992813972], [-0.44291651677778576, 2.3646507893043536, 0.054956398730539036, 2.111536655929477], [-1.6062509662063507, 6.053259264733053, 5.5875288262167286, -0.94830906823710626]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_constData_rank4(self):
      arg0=Data(-3.20842764987,self.functionspace)
      arg1=Data(numpy.array([[[[-2.6234399586702208, 4.4311514687043978, -4.1008252477232352, -1.3147052024613046], [4.9417375611190231, 0.81212262599414409, 2.3358326896790551, -2.6539674158911275], [-3.6670874257361419, -4.0291010349188339, 1.9198897647511579, 2.1717602657506578]], [[3.3572962081249642, -0.81791901556249247, -4.4504291471148392, -4.9051770232190171], [-4.0497112295561202, 0.41916455305765865, 3.2229547635965421, 3.8937672784913087], [4.9940003879544808, 4.793451832168186, -1.8584165519098206, 4.647519522972658]]], [[[-0.53503299352415379, 2.0839176647235256, 0.91595403246686491, -1.3052923755698673], [-4.8161034219702747, 1.6364293051155032, 0.1202588614480371, 1.2653242168794634], [2.1981991015387887, 2.6912042994544496, 4.9336834100442353, 4.5549397131944325]], [[-0.57007354496107787, 0.81122008331773099, 4.5482557709239764, -1.2552063267967362], [-4.8538477617493756, -0.12120537723704317, 1.5183899774558949, 4.4446127174634213], [1.5420996270270937, 2.2436300496068675, 1.8816760632759184, -1.6900292906933756]]], [[[3.7771946156899396, 3.0542987269866746, 4.5772441625908762, 0.24426189023640354], [0.3003955873159434, -4.9355096381515704, 1.7262057045408543, 0.70648633617220469], [3.2668356789151183, 4.2115560403883183, 1.6165846839993065, -1.2570952197266805]], [[-1.9628267755299911, 3.3971351326531725, 4.3018214716481769, 1.4851927271492453], [-4.8399518148951257, -0.63664374038129701, 3.6958883149380952, -0.35046359027050134], [0.93113060163432682, -4.1929337674612626, 1.4022206543037878, -3.8552741088192422]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-5.8318676085420256, 1.2227238188325931, -7.3092528975950399, -4.5231328523331094], [1.7333099112472183, -2.3963050238776606, -0.87259496019274962, -5.8623950657629322], [-6.8755150756079466, -7.2375286847906386, -1.2885378851206468, -1.0366673841211469]], [[0.14886855825315948, -4.0263466654342972, -7.6588567969866439, -8.1136046730908227], [-7.2581388794279249, -2.7892630968141461, 0.014527113724737362, 0.68533962861950393], [1.785572738082676, 1.5850241822963813, -5.0668442017816258, 1.4390918731008533]]], [[[-3.7434606433959585, -1.1245099851482792, -2.2924736174049398, -4.5137200254416721], [-8.0245310718420804, -1.5719983447563015, -3.0881687884237676, -1.9431034329923413], [-1.010228548333016, -0.51722335041735512, 1.7252557601724305, 1.3465120633226277]], [[-3.7785011948328826, -2.3972075665540737, 1.3398281210521716, -4.4636339766685413], [-8.0622754116211794, -3.3296330271088479, -1.6900376724159099, 1.2361850675916166], [-1.666328022844711, -0.96479760026493722, -1.3267515865958863, -4.8984569405651808]]], [[[0.56876696581813491, -0.15412892288513014, 1.3688165127190715, -2.9641657596354012], [-2.9080320625558613, -8.1439372880233751, -1.4822219453309504, -2.5019413136996], [0.058408029043313547, 1.0031283905165136, -1.5918429658724982, -4.4655228695984857]], [[-5.1712544254017958, 0.18870748278136773, 1.0933938217763721, -1.7232349227225594], [-8.0483794647669313, -3.8450713902531017, 0.48746066506629049, -3.5588912401423061], [-2.2772970482374779, -7.4013614173330673, -1.8062069955680169, -7.0637017586910469]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([4.1540633027139418, 4.2691688992684149]),self.functionspace)
      arg1=Data(numpy.array([-0.9921097979965694, -0.24578558349860025]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([3.1619535047173724, 4.0233833157698147]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[4.9487620724261578, -2.3502803754916748], [1.4120618353455585, 2.3414005910700908]]),self.functionspace)
      arg1=Data(numpy.array([-2.0286253623582273, 0.80373338429263619]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[2.9201367100679305, -4.3789057378499017], [2.2157952196381947, 3.145133975362727]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_constData_rank2(self):
      arg0=Data(numpy.array([-3.8189678633302861, -3.8242095995628866]),self.functionspace)
      arg1=Data(numpy.array([[-4.2988704710551753, 0.42327775185501881], [0.74563612720694117, -0.26201836966069436]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-8.1178383343854605, -3.3956901114752673], [-3.0785734723559455, -4.0862279692235806]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_constData_rank1(self):
      arg0=Data(numpy.array([[[2.4306253984613493, 0.27097597915067606, 4.8407087992044424, -4.2002700623033986, -3.2242044806962298], [-4.7570173975127759, 0.77582079398612702, -3.1524257112778153, -4.4903111744985322, -2.5590460647575175], [-2.888447150803628, -0.7234157156219867, -3.7421740328688657, -3.8487280099265373, -1.2994605035139051], [-3.5538166157110709, 3.9189814647204795, 0.55884092119966056, -0.80385864509695448, -0.49097482321832953]], [[-3.4910898871069405, 0.68773031701308796, -2.544148654646964, -2.1169147222883655, -0.69322156951080061], [2.0076858043290198, -4.7573943340009599, -3.6476864198115821, 4.0853265391235833, 1.0903557907924375], [-3.0455570664553844, -2.5768634875431209, 0.082845699573152132, 0.94709746958400487, 2.9440828420913601], [2.511045052815513, 4.1147994884681864, 2.5767140945094624, -4.3348115222476649, -2.1991212934934543]]]),self.functionspace)
      arg1=Data(numpy.array([0.97083789363730055, -2.8382489158913393]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[3.4014632920986498, 1.2418138727879766, 5.811546692841743, -3.2294321686660981, -2.2533665870589292], [-3.7861795038754753, 1.7466586876234276, -2.1815878176405148, -3.5194732808612317, -1.5882081711202169], [-1.9176092571663275, 0.24742217801531385, -2.7713361392315652, -2.8778901162892367, -0.32862260987660452], [-2.5829787220737703, 4.88981935835778, 1.5296788148369611, 0.16697924854034607, 0.47986307041897103]], [[-6.3293388029982793, -2.1505185988782514, -5.3823975705383038, -4.9551636381797053, -3.5314704854021399], [-0.83056311156231954, -7.5956432498922997, -6.485935335702921, 1.247077623232244, -1.7478931250989018], [-5.8838059823467237, -5.4151124034344598, -2.7554032163181872, -1.8911514463073344, 0.10583392620002074], [-0.32720386307582627, 1.2765505725768471, -0.26153482138187689, -7.1730604381390037, -5.0373702093847932]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_constData_rank3(self):
      arg0=Data(numpy.array([-4.2277988748051794, 4.7490099852319041]),self.functionspace)
      arg1=Data(numpy.array([[[-1.8361616647506143, -4.4264769953087066, 3.1437669893888422, -4.2474799542916273, 0.73212514668399109], [2.0808940518005601, -3.0326664375048349, 1.9218309743531767, -2.2955342043323022, -1.814855399043358], [-2.1412546055908752, -1.0903919266485307, 4.5118182239016171, -1.9724890331900902, 2.0313748118631931], [4.7865019147759433, -4.4747325427745448, 3.3604016107879211, 1.5910878211972159, 0.63928194555016127]], [[-4.742869500144236, 4.3109346241811846, -2.6181040699977807, 3.4190373817932667, -3.5482889281507237], [-0.95708619606326195, -3.7474732005303943, -2.5906433726806402, 1.7251608585489686, 2.6446952098070211], [3.6838843256791947, 1.2261090016170106, -1.6103279577600214, -0.022640873636357028, -3.7576014048720774], [4.2809717169894288, 3.1459851201529396, 0.68211387830240078, 0.86272432592548398, -1.6239375485261665]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-6.0639605395557936, -8.6542758701138851, -1.0840318854163371, -8.4752788290968066, -3.4956737281211883], [-2.1469048230046193, -7.2604653123100142, -2.3059679004520026, -6.5233330791374815, -6.0426542738485374], [-6.3690534803960546, -5.31819080145371, 0.28401934909643778, -6.2002879079952695, -2.1964240629419862], [0.55870303997076398, -8.7025314175797241, -0.86739726401725825, -2.6367110536079634, -3.5885169292550181]], [[0.0061404850876680683, 9.0599446094130887, 2.1309059152341234, 8.1680473670251708, 1.2007210570811804], [3.7919237891686421, 1.0015367847015098, 2.1583666125512639, 6.4741708437808727, 7.3937051950389252], [8.4328943109110988, 5.9751189868489147, 3.1386820274718827, 4.7263691115955471, 0.99140858035982671], [9.0299817022213329, 7.8949951053848437, 5.4311238635343049, 5.6117343111573881, 3.1250724367057376]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_constData_rank1(self):
      arg0=Data(numpy.array([[[[1.1294111756348348, -3.4461371106010308], [-2.8769732409095763, -4.8251222640695595]], [[-0.36528972331886678, -1.1286710704303058], [2.5041999427595059, -4.4167449960694665]], [[-4.2882549981523148, 0.3408258965951072], [0.78872492869125033, -1.0862658609239983]], [[-2.5667353676117477, -3.5060283756867672], [1.5432507219046396, -4.7310937093245107]], [[0.17867076986635233, -0.9644716006399392], [3.377323976909457, -2.1788544245021866]], [[2.0749175150340911, 1.251521376145682], [3.1916969005573055, -1.166563016395016]]], [[[4.6855290503185678, 4.5095376828768803], [1.4116728420145455, -2.6958130453295914]], [[4.353448313867931, 4.3650848170085723], [1.4790184266111304, -4.0150664208852671]], [[3.1619609804200586, -2.6150967062514385], [-3.0134454072632213, 1.1573123664739882]], [[-3.9667033170398955, 0.1165789264743724], [0.04273399038036807, -3.1166565565193736]], [[-4.7315042506188734, -1.0209064540753121], [3.6195508316056539, 2.3271191429686517]], [[4.8978593945116273, -1.6140364518650485], [4.702912634844056, 4.2601382046839493]]]]),self.functionspace)
      arg1=Data(numpy.array([2.5761660498240966, -0.74855059446050287]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[3.7055772254589314, -0.8699710607769342], [-0.30080719108547971, -2.2489562142454629]], [[2.2108763265052298, 1.4474949793937908], [5.0803659925836024, -1.8405789462453699]], [[-1.7120889483282182, 2.9169919464192038], [3.3648909785153469, 1.4899001889000982]], [[0.0094306822123488843, -0.92986232586267059], [4.1194167717287362, -2.1549276595004141]], [[2.7548368196904489, 1.6116944491841574], [5.9534900267335535, 0.39731162532191]], [[4.6510835648581876, 3.8276874259697786], [5.7678629503814021, 1.4096030334290806]]], [[[3.9369784558580649, 3.7609870884163774], [0.66312224755404259, -3.4443636397900943]], [[3.6048977194074281, 3.6165342225480694], [0.73046783215062749, -4.76361701534577]], [[2.4134103859595557, -3.3636473007119414], [-3.7619960017237242, 0.40876177201348529]], [[-4.7152539115003984, -0.63197166798613047], [-0.7058166040801348, -3.8652071509798764]], [[-5.4800548450793762, -1.769457048535815], [2.8710002371451511, 1.5785685485081489]], [[4.1493088000511245, -2.3625870463255514], [3.9543620403835531, 3.5115876102234465]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_constData_rank4(self):
      arg0=Data(numpy.array([-4.5013539608915112, 0.095390805681432767]),self.functionspace)
      arg1=Data(numpy.array([[[[0.45812811904893369, -0.1795815215759875], [4.0641974845930342, -2.6333542301377677]], [[-2.8815098170696682, -1.9699856937307905], [-3.400456811613032, 3.0805436496194201]], [[0.89198723369701405, 1.2750132072619529], [-3.5591781427892268, 0.55980682442331364]], [[0.7748897826823411, 3.0903824892197829], [-1.0171368574269213, -3.3834916240350008]], [[1.8738767468841466, 4.2992152605381158], [4.1089741418971535, -2.2464766541110714]], [[4.3706104374494679, -2.598181790979214], [-3.3954009168715595, -3.1880876942790159]]], [[[3.3626426996079211, 3.3972123546159878], [-0.50688914736552704, 1.4637217220424867]], [[2.5889018045139478, -3.2065662185980068], [3.4248098780691372, 1.5703032980879819]], [[-2.0101404032795491, -3.9045885786321155], [-2.3103461720953633, -1.3787461066182258]], [[1.1697792784851107, -3.7066853028641256], [-2.3859428944013938, -0.34388257800046773]], [[-3.4912245491345848, 4.1535676332405949], [-1.506135420636936, -0.30874210987715855]], [[-4.9600735218914203, -3.4657214465763211], [-4.3884052335200856, 4.4062234086598995]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-4.0432258418425775, -4.6809354824674987], [-0.43715647629847698, -7.1347081910292793]], [[-7.3828637779611794, -6.4713396546223017], [-7.9018107725045432, -1.4208103112720911]], [[-3.6093667271944971, -3.2263407536295583], [-8.0605321036807389, -3.9415471364681975]], [[-3.7264641782091701, -1.4109714716717283], [-5.5184908183184325, -7.884845584926512]], [[-2.6274772140073646, -0.20213870035339543], [-0.3923798189943577, -6.7478306150025826]], [[-0.13074352344204332, -7.0995357518707252], [-7.8967548777630707, -7.6894416551705271]]], [[[3.4580335052893538, 3.4926031602974206], [-0.41149834168409427, 1.5591125277239195]], [[2.6842926101953806, -3.1111754129165741], [3.5202006837505699, 1.6656941037694146]], [[-1.9147495975981164, -3.8091977729506827], [-2.2149553664139305, -1.2833553009367931]], [[1.2651700841665434, -3.6112944971826928], [-2.2905520887199611, -0.24849177231903496]], [[-3.395833743453152, 4.2489584389220276], [-1.4107446149555032, -0.21335130419572579]], [[-4.8646827162099875, -3.3703306408948883], [-4.2930144278386528, 4.5016142143413322]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-4.5201375548133083, 3.4686404543077227, -1.4190006125197252, 0.10788635630039067, 1.5264239890140283], [-3.0095165044013084, 4.3421971282997838, -2.2533213461584953, 2.2591257650630991, 0.64038408415410686], [4.2661436440725318, 2.9104789293227391, 1.1063730014072064, -3.1918618619884729, 0.45680118313031137], [0.21278793265339324, -1.7351642599572559, -3.0745923735800851, -0.65810004688862378, -1.1393812276483342]]),self.functionspace)
      arg1=Data(numpy.array([[-1.884607406512544, -1.8964479873639317, -1.5025983471646742, 2.9057722396792531, 4.6475192744848197], [2.1961346411170437, 1.9344713424166384, -4.4118453746641029, -4.6959831302833068, -0.2125036869337551], [-3.7639689371588902, 4.536802771170894, -0.10579375814487069, 2.979750104145249, 1.8632955556898265], [0.69830575155958208, 2.6747646125700975, -3.2111802794315789, -0.72010020705642575, 3.1274120928165825]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[-6.4047449613258518, 1.572192466943791, -2.9215989596843994, 3.0136585959796438, 6.173943263498848], [-0.81338186328426465, 6.2766684707164222, -6.6651667208225982, -2.4368573652202077, 0.42788039722035176], [0.50217470691364152, 7.4472817004936331, 1.0005792432623357, -0.21211175784322389, 2.3200967388201379], [0.91109368421297532, 0.93960035261284158, -6.285772653011664, -1.3782002539450495, 1.9880308651682483]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_constData_rank2(self):
      arg0=Data(numpy.array([[[0.78487162261268928, 2.4787277670874053], [-3.7345326998281059, 3.8954520101389463], [1.0672607803191045, 4.1915506751059048], [-2.777861828652938, 1.25275478944166], [-2.5490684734166136, -3.8673608645739499]], [[3.2896130461842112, 0.87537151162145133], [-0.037619387125969794, 2.215695630633066], [-2.7141857007680126, -1.0577480182535872], [1.0668799425811883, -1.9879046800379383], [-3.9414830821555569, 1.1609955211587604]], [[-2.6756996798136443, 1.1130158230460676], [1.4505675357228256, -3.523579445209525], [4.9746077441323688, 0.65233325162230216], [-0.51780438304200427, 0.59057858467027469], [4.4171676304321394, 0.12032553851702943]], [[-3.8563421453545841, 1.5114530860612359], [-4.0196672925720565, -4.8837644045033937], [1.8391887508219043, 1.3642341980883028], [-4.6496157578360098, 3.3912801615448949], [2.8881209125256149, 1.3552890757626983]]]),self.functionspace)
      arg1=Data(numpy.array([[0.83572997220838019, -0.23712528980153724, 1.2481965628282365, 0.29227280553381441, -0.41367318836786282], [-3.8602409717111987, -2.622225887109404, -1.0294352665640205, 1.6056077835169988, -1.5886272676700464], [-0.17099540636022237, 4.7264567976665255, -1.2854852914065473, -0.42390331791514058, -4.8500466219402476], [-4.8437187680939093, -3.7168829061701238, -4.9305678750121773, 0.79334613932304521, -2.378953092004191]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[1.6206015948210695, 3.3144577392957855], [-3.9716579896296431, 3.658326720337409], [2.3154573431473411, 5.4397472379341414], [-2.4855890231191236, 1.5450275949754744], [-2.9627416617844764, -4.2810340529418127]], [[-0.57062792552698749, -2.9848694600897474], [-2.6598452742353738, -0.406530256476338], [-3.7436209673320331, -2.0871832848176077], [2.6724877260981872, -0.38229689652093946], [-5.5301103498256037, -0.42763174651128599]], [[-2.8466950861738667, 0.94202041668584524], [6.1770243333893511, 1.2028773524570004], [3.6891224527258215, -0.63315203978424517], [-0.94170770095714484, 0.16667526675513411], [-0.43287899150810816, -4.7297210834232182]], [[-8.7000609134484925, -3.3322656820326735], [-7.7365501987421803, -8.6006473106735175], [-3.0913791241902731, -3.5663336769238745], [-3.8562696185129646, 4.1846263008679401], [0.50916782052142384, -1.0236640162414927]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_constData_rank3(self):
      arg0=Data(numpy.array([[-3.7062206036944456, 1.9637519057844477, 1.5654303792067701, -1.1334611694253427, 1.609137529009903], [1.8863203324437441, -3.85305266354362, -0.62717774028177775, 1.2941258092318053, -0.69693635534929399], [0.73993932543087038, 3.3359686635339614, 0.73428854696678236, -1.5214519424524617, 4.9012701661166229], [-3.7245209480150399, 1.6436409114090855, -0.80420125699546219, 4.0135847324117009, 1.3645404180378531]]),self.functionspace)
      arg1=Data(numpy.array([[[4.2514269937958922, 2.3228939348875599], [3.759403620094572, 2.4881009665079983], [0.30957239752430343, 3.7052468190615944], [1.9886516297580501, -2.4199383954086304], [-2.6041764050498415, 3.6295983675198595]], [[-3.3759385388209919, -1.9405075817558739], [-4.5030638823131417, -4.1009880648649677], [-3.7853549761628127, 3.8896399629240346], [-0.97899345116184122, 0.093623116615474622], [4.2580408200329583, -2.8559882899888613]], [[3.9329859331801682, 0.41129562373807271], [-4.1827549088343199, 2.7892952942565508], [-0.8615481422024196, 0.31123638444155688], [-4.0273599360920418, -1.8387691853504484], [-1.2417219867498019, 3.2501354869381469]], [[2.6402026884392242, 1.10339030700138], [-2.6458478316297809, -0.80222844329025467], [-3.9382353273647519, -0.98281702269850779], [2.4539732885152414, -2.5920511582767078], [0.35059546095792182, -3.9766173050900444]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[0.54520639010144656, -1.3833266688068857], [5.7231555258790197, 4.451852872292446], [1.8750027767310735, 5.2706771982683644], [0.8551904603327074, -3.5533995648339731], [-0.99503887603993846, 5.2387358965297626]], [[-1.4896182063772478, -0.054187249312129815], [-8.3561165458567608, -7.9540407284085877], [-4.4125327164445904, 3.2624622226422568], [0.31513235806996409, 1.3877489258472799], [3.5611044646836643, -3.5529246453381553]], [[4.6729252586110386, 1.1512349491689431], [-0.84678624530035851, 6.1252639577905121], [-0.12725959523563724, 1.0455249314083392], [-5.5488118785445035, -3.3602211278029102], [3.659548179366821, 8.1514056530547698]], [[-1.0843182595758156, -2.6211306410136599], [-1.0022069202206954, 0.84141246811883086], [-4.7424365843602141, -1.78701827969397], [6.4675580209269423, 1.421533574134993], [1.7151358789957749, -2.6120768870521913]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[-1.5497044459822105, -2.027188677825269, 0.86912636964369483, 2.1170895369208234, -3.3425988196343424], [-4.3359036001933484, -0.99041995619279888, 0.61138361513824435, -3.1038514851410004, -0.79451577691362818], [-3.1585098709974266, 4.2587307738654996, -2.1940753331605531, -2.7272034000118275, -1.9065236896099091], [-0.21533995538831796, -0.019421947936256956, 3.3716175233097232, -4.5395877227950727, -2.4406943611679557]], [[3.0111079737027957, -1.1635740656811433, -3.783490440795565, -3.167514308156949, 4.0900956645140703], [3.488360500332373, -4.5498373915691337, 3.0016626309953605, -3.375192754469265, -1.6944334877971299], [-0.82983667770281855, 1.7463236035234431, 0.88050434780317577, 1.7883877132265784, 3.8993848496885413], [-1.6482873914800091, -1.5802574469440178, -0.064856443199400537, 3.8489962545187559, -4.5819790804727738]], [[1.6584810581168732, -4.766882627258715, 4.0000337243626873, 1.7581048828931536, 2.0052193888809384], [1.8640232966415891, -3.57833198267548, 0.20453401018374606, -2.357665232514814, 1.0233989109233175], [-3.1763561064337944, -1.2221111818048378, 2.2614170755049479, -1.236683485575143, 3.5487036785034523], [-1.600132280864166, 2.2490041891490629, -2.2888931606334229, -3.6723685633001479, -2.0808436827062993]], [[4.7892927727314252, -2.5865201294198616, -4.7286051542454732, 3.2245766781087255, -0.19244053373016712], [-1.0768205293131219, -2.1378288094481546, 4.8055283314790369, -2.2400607031310829, 3.1697057036867093], [4.6291229424617182, -0.14311884897468374, -3.8207340371118992, 2.6399115005900855, 2.7616622264428612], [1.0516409144223884, -4.9327877589098517, -4.7951542419861735, -0.25343595503386052, 3.328157514347609]], [[-2.9112823791650788, 2.7965208038267271, -4.0072292845930271, -4.8847484166147384, -1.1618807027052434], [4.4363838151419497, -0.49158362750483597, -2.774179336794933, -1.7185275031171532, -0.90755011384058015], [4.7165233875288806, -1.5722244552633526, 2.6592832427840083, -1.6917815270199044, -3.8553429200135989], [-1.076678881831834, -4.9539343792132149, -1.7729486364956681, 3.2330562272831092, 3.0262531007811617]]], [[[2.6076309899575438, 2.6968357200295383, 0.88768018664046089, 0.092909142766299802, 0.9033052116073117], [1.9523739616542706, -0.73148498148165153, -4.6204064469567276, -3.8523485436037666, 1.3344091615315126], [3.2276758676637165, 1.1211967752207261, 2.1494827400855261, -2.4834633596247113, 3.1950142271664497], [3.9788338466924564, 2.4412420576378091, 1.6235190579745709, 3.8693747027748451, -1.5004914595719732]], [[2.9515350686541817, 4.3440853490822633, 3.5008845497277843, -0.9642928279835159, 4.2379821111593472], [0.60984362399053449, 1.6859642604082108, 1.5146714476095848, 2.2876446357211595, -4.785535411012054], [2.6730903079845083, -3.3416777184185298, -2.1211589220781635, -0.53282349968731957, 0.47509045837893105], [3.869192081818694, 1.6120274514378679, 1.1865554629685269, -2.0617598581480201, -1.3455643040445988]], [[4.6177425621617179, 1.2470674281499221, -3.2845605276710952, -3.1743569712588648, -1.1482969875132367], [3.1695524989391046, 1.1708653130571705, -2.2956059450191981, -4.1430936860377008, -3.5896774649696681], [2.2338959260634015, 1.9254278060935084, -2.4591542858135651, -3.4192234297122415, 2.2410312575721845], [4.3759064894400215, -3.0679713571873588, -3.5622297740850062, -0.86922063600588473, 0.09888948575572698]], [[-4.9338277230185117, -3.8013921118302854, -4.6231017116141837, 4.5715991083214966, -0.55360573991459638], [1.5891353032055422, 0.81863872473341015, 0.62934735025278155, 1.6241263203256562, 4.8879450454781832], [-4.9254640848827655, 0.3149444815009268, 4.5082415712194415, -4.1696641458138641, -4.5404065509668632], [1.0865846730321316, -4.8916029438123445, 3.8020434591351595, -1.9817058988683067, 2.6614763246251307]], [[-4.0295890904205152, -2.9456950420661929, 2.4285077395251218, -1.5537930307676131, 2.9582995731982127], [2.6607885247651595, -1.1450572001604034, 4.3646226261772085, 3.0532354801169923, -4.9391763269166482], [3.8561432495696284, -1.3363344274780244, 3.0041246903591521, 0.75766934871637304, -1.2060474281982514], [-0.332936961889434, -1.9557859653206124, 0.16388140931057116, -2.3802797741542356, 4.4269629664357097]]], [[[2.4297444861296604, -3.8213249767960846, 4.780207915281224, 4.9176992301701112, -2.5917713812928787], [1.6364476155954648, 4.9545039873800167, 1.0337923469398733, 2.6009319330922889, 3.7294680620659744], [-0.77570735640436794, 2.7179859242238011, 2.3271507584757583, -1.8461658136731405, -0.27328048478772082], [0.62622372685162819, 3.8530107413177834, 3.4733108096729772, -4.7650615378817314, 0.59182485869611234]], [[-1.2838088908492926, 2.3441806804026548, 2.3413315438124416, -1.0110159802505123, -3.6694002022319285], [4.4904678011019215, 0.48484234513985225, 0.90320833383015131, -4.5573696022155072, -1.408342296017624], [-1.1135058365386277, 2.8621526507726056, -1.4044839425641706, 3.6741493920518629, -4.9070785645932959], [3.4205965179578932, -0.42332771708990968, -3.9125699564238436, 4.5731390302789876, 2.5392252299759086]], [[3.1977773283208162, -4.4998206683880948, 2.8985846500754642, -3.953499757497152, -1.4583678653075727], [-1.9250541879334939, -2.2946272915345514, -3.9015194307768351, 4.6387456150434669, 2.1202362175346874], [3.1625677527517642, 4.862106126286049, 2.6456865761457662, 2.570125801848353, 4.8212379723219492], [2.0757479082821853, -1.0914654590848327, -0.54744398759417301, -0.84867919269904224, 2.9782036963408887]], [[-2.1520872509350673, -4.4994588209447652, 2.4688613096582284, -3.0327847017630916, -0.66186379195716771], [4.0687227664974888, 3.4577907253679854, 0.41577586124560995, 2.4965648133204574, 1.774300531754025], [-2.4848938430399992, -2.2426543731968063, -4.9310453157349663, 0.71806966223737945, 1.3023132168548868], [-0.86521234105613054, -4.7946285214600017, 3.2758961205027006, -3.6159299095261996, -0.039431415226406585]], [[4.3713075540210546, 3.377197772775137, -2.0125539343288077, 1.0071447579009902, -1.8207866585917931], [-1.3718270009797449, 3.807489528356891, -3.3178467182131213, -2.0608609243678777, -3.0500660598531768], [1.8088672225882041, -3.3871325258873854, 4.3650024171674104, 4.3658892478997231, 4.0196108418963679], [-0.95230076413198184, 4.6799062604433832, -4.2988378819075272, 0.48940863646122157, -2.6607418814909467]]], [[[-2.9355064202070871, 4.5042667209686034, -3.9393952721934902, -0.83185216502723769, -3.5247393767403281], [-2.3156866286665845, 4.444474013302445, -2.6829193539729976, -0.45424845364179589, -2.6285162057078559], [-4.5863201373889604, 2.7591604353140875, -3.4397351778749985, 0.72345293393704679, 0.90489116297050831], [-2.1803551123377831, -2.5470400348919209, -0.51734718876427976, -1.8448365594929008, -4.7757166275652292]], [[-4.3488069990600708, -0.85247471577836986, 2.7192465853296675, -0.11813549685769953, -0.6771102303259946], [4.7699145402724206, -3.0399522995911465, 1.1064303209701798, -2.6346817309522264, 2.2552237136889994], [-1.5695662771565786, 4.2940674048794101, 1.9809783154646565, 0.52323314201158411, -1.3018943399540071], [-1.0868659301294836, -3.6419901089926032, 3.2990719551965491, 4.0880320518536237, 1.1614785151693567]], [[1.3787183187727345, -0.53838609659959236, 1.8180947895546042, -1.4815026096502448, -2.4882504086954471], [-2.1655943367570862, -4.2954712016347365, 4.6437221515477756, -2.0262555430101692, -4.1332694551395752], [4.9880975962135885, 1.9378828833603965, 2.4839346602898047, 3.8320572911106847, 3.38593769478544], [-3.7156676530710362, -1.5998948007095826, -0.78293026336379512, -4.5397871667512577, 3.2435768234036608]], [[0.38600037873220039, -0.88812583386855959, -4.4008580140998079, 0.39583259743714994, -2.2448329557545934], [-3.0337372661907414, -2.5805672413083114, -2.6809113110733618, -0.73853230088966448, 1.7692571519454283], [4.4097136943328579, -1.8529036868722839, -1.5533859015004881, -4.0951656775823864, -0.42341509659941856], [-0.36140859192607344, -2.2392551384944359, 0.98384409425118946, -3.0155147861310185, 4.4393011580949988]], [[-4.1524354337969562, 3.554639411442885, -3.6063933247722026, -4.6355058362077939, 3.5062424274551169], [-2.5800018363873836, 3.8848713244361104, -3.3061303099209782, 4.9214471982203314, 3.9426576414042316], [3.8403752269159384, 3.0316020229789959, -4.1295808250222077, -0.48105907939425929, 2.4778457561129708], [-1.1267185823582002, 3.1310338201633705, 1.1483516916839145, -2.215992786922599, 3.6318633589376219]]]]),self.functionspace)
      arg1=Data(numpy.array([[4.3486967934627803, -4.1911769600147517, 4.7483656534106462, 2.2846810218071116, -0.95177200432936981], [-3.0093529178472256, -0.55669545067556925, 3.1373567035340368, -3.2714040728185321, 0.71891694650373061], [-0.57272895604335439, -1.6976353393137278, 4.5965571634355982, -2.8796571851712729, -0.97422361881305974], [-1.3313210697748081, -1.3276191347873221, -4.954731231991528, 4.8472377053335887, 1.4909553877361406]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[2.7989923474805698, 2.3215081156375112, 5.2178231631064751, 6.4657863303836036, 1.0060979738284379], [0.012793193269431846, 3.3582768372699814, 4.9600804086010246, 1.2448453083217799, 3.5541810165491521], [1.1901869224653536, 8.6074275673282798, 2.1546214603022271, 1.6214933934509528, 2.4421731038528711], [4.1333568380744623, 4.3292748455265233, 7.7203143167725035, -0.19089092933229246, 1.9080024322948246]], [[-1.180068986311956, -5.3547510256958954, -7.9746674008103167, -7.3586912681717003, -0.10108129550068146], [-0.70281645968237871, -8.7410143515838854, -1.1895143290193912, -7.5663697144840167, -5.8856104478118816], [-5.0210136377175703, -2.4448533564913086, -3.3106726122115759, -2.4027892467881733, -0.29179211032621044], [-5.8394643514947608, -5.77143440695877, -4.2560334032141522, -0.34218070549599577, -8.7731560404875246]], [[6.4068467115275194, -0.01851697384806883, 8.7483993777733335, 6.5064705363037998, 6.7535850422915846], [6.6123889500522353, 1.1700336707351662, 4.9528996635943923, 2.3907004208958322, 5.7717645643339637], [1.5720095469768518, 3.5262544716058084, 7.0097827289155941, 3.5116821678355032, 8.2970693319140985], [3.1482333725464802, 6.9973698425597091, 2.4594724927772234, 1.0759970901104983, 2.6675219707043469]], [[7.0739737945385368, -0.30183910761275001, -2.4439241324383616, 5.5092576999158371, 2.0922404880769445], [1.2078604924939897, 0.14685221235895707, 7.0902093532861485, 0.044620318676028781, 5.454386725493821], [6.9138039642688298, 2.1415621728324279, -1.5360530153047876, 4.9245925223971971, 5.0463432482499728], [3.3363219362295, -2.64810673710274, -2.5104732201790618, 2.0312450667732511, 5.6128385361547206]], [[-3.8630543834944486, 1.8447487994973573, -4.9590012889223969, -5.8365204209441082, -2.1136527070346132], [3.4846118108125799, -1.4433556318342058, -3.7259513411243028, -2.670299507446523, -1.85932211816995], [3.7647513831995107, -2.5239964595927225, 1.7075112384546385, -2.6435535313492742, -4.8071149243429687], [-2.0284508861612038, -5.9057063835425847, -2.724720640825038, 2.2812842229537393, 2.0744810964517919]]], [[[-0.40172192788968175, -0.31251719781768728, -2.1216727312067647, -2.9164437750809258, -2.1060477062399139], [-1.056978956192955, -3.7408378993288771, -7.6297593648039532, -6.8617014614509921, -1.674943756315713], [0.21832294981649092, -1.8881561426264994, -0.85987017776169949, -5.4928162774719365, 0.18566130931922409], [0.96948092884523085, -0.56811086020941648, -1.3858338598726547, 0.86002178492761949, -4.5098443774191992]], [[2.3948396179786124, 3.787389898406694, 2.944189099052215, -1.5209882786590851, 3.681286660483778], [0.053148173314965241, 1.1292688097326415, 0.95797599693401558, 1.7309491850455903, -5.3422308616876233], [2.1163948573089391, -3.898373169094099, -2.6778543727537327, -1.0895189503628888, -0.081604992296638201], [3.3124966311431248, 1.0553320007622986, 0.62986001229295763, -2.6184553088235893, -1.9022597547201681]], [[7.7550992656957547, 4.3844241316839589, -0.14720382413705835, -0.037000267724828007, 1.9890597160208001], [6.3069092024731415, 4.3082220165912073, 0.84175075851483872, -1.0057369825036639, -0.45232076143563127], [5.3712526295974383, 5.0627845096275452, 0.67820241772047174, -0.28186672617820463, 5.3783879611062213], [7.5132631929740583, 0.069385346346678034, -0.42487307055096935, 2.2681360675281521, 3.2362461892897638]], [[-8.2052317958370438, -7.0727961846488174, -7.8945057844327158, 1.3001950355029646, -3.8250098127331285], [-1.6822687696129899, -2.4527653480851219, -2.6420567225657505, -1.6472777524928759, 1.6165409726596511], [-8.1968681577012976, -2.9564595913176053, 1.2368374984009094, -7.4410682186323962, -7.8118106237853953], [-2.1848193997864005, -8.1630070166308766, 0.53063938631662744, -5.2531099716868388, -0.60992774819340134]], [[-3.3106721439167845, -2.2267780955624623, 3.1474246860288524, -0.83487608426388249, 3.6772165197019433], [3.3797054712688901, -0.42614025365667274, 5.0835395726809391, 3.7721524266207229, -4.2202593804129176], [4.575060196073359, -0.61741748097429383, 3.7230416368628827, 1.4765862952201037, -0.48713048169452078], [0.38597998461429661, -1.2368690188168818, 0.88279835581430177, -1.661362827650505, 5.1458799129394404]]], [[[1.857015530086306, -4.394053932839439, 4.2074789592378696, 4.3449702741267568, -3.164500337336233], [1.0637186595521104, 4.3817750313366624, 0.46106339089651893, 2.0282029770489345, 3.15673910602262], [-1.3484363124477223, 2.1452569681804468, 1.7544218024324039, -2.4188947697164949, -0.84600944083107521], [0.053494770808273806, 3.280281785274429, 2.9005818536296228, -5.3377904939250858, 0.019095902652757957]], [[-2.9814442301630204, 0.64654534108892703, 0.64369620449871379, -2.7086513195642401, -5.3670355415456559], [2.7928324617881937, -1.2127929941738755, -0.79442700548357648, -6.2550049415292346, -3.1059776353313517], [-2.8111411758523555, 1.1645173114588778, -3.1021192818778984, 1.9765140527381351, -6.6047139039070242], [1.7229611786441654, -2.1209630564036375, -5.6102052957375719, 2.8755036909652598, 0.84158989066218082]], [[7.7943344917564144, 0.096736495047503368, 7.4951418135110623, 0.64305740593844618, 3.1381892981280255], [2.6715029755021042, 2.3019298719010468, 0.69503773265876312, 9.2353027784790651, 6.7167933809702856], [7.7591249161873623, 9.4586632897216472, 7.2422437395813644, 7.1666829652839512, 9.4177951357575473], [6.6723050717177834, 3.5050917043507654, 4.0491131758414252, 3.7478779707365559, 7.5747608597764868]], [[-5.0317444361063401, -7.3791160061160381, -0.41079587551304453, -5.9124418869343645, -3.5415209771284406], [1.1890655813262159, 0.57813354019671248, -2.4638813239256629, -0.38309237185081546, -1.1053566534172479], [-5.3645510282112721, -5.1223115583680787, -7.8107025009062392, -2.1615875229338934, -1.5773439683163861], [-3.7448695262274034, -7.6742857066312746, 0.39623893533142773, -6.4955870946974725, -2.9190886003976795]], [[3.3970839352079949, 2.4029741539620773, -2.9867775531418674, 0.032921139087930484, -2.7950102774048529], [-2.3460506197928046, 2.8332659095438313, -4.292070337026181, -3.0350845431809375, -4.0242896786662365], [0.83464360377514435, -4.3613561447004452, 3.3907787983543507, 3.3916656290866634, 3.0453872230833081], [-1.9265243829450416, 3.7056826416303235, -5.273061500720587, -0.48481498235183818, -3.6349655003040064]]], [[[-4.2668274899818952, 3.1729456511937952, -5.2707163419682983, -2.1631732348020458, -4.8560604465151362], [-3.6470076984413926, 3.1131529435276368, -4.0142404237478058, -1.785569523416604, -3.9598372754826641], [-5.9176412071637685, 1.4278393655392794, -4.7710562476498062, -0.60786813583776134, -0.42642990680429982], [-3.5116761821125912, -3.878361104666729, -1.8486682585390879, -3.1761576292677089, -6.1070376973400373]], [[-5.6764261338473929, -2.1800938505656919, 1.3916274505423454, -1.4457546316450216, -2.0047293651133167], [3.4422954054850985, -4.3675714343784691, -0.22118881381714228, -3.9623008657395484, 0.92760457890167736], [-2.8971854119439007, 2.966448270092088, 0.65335918067733445, -0.80438599277573797, -2.6295134747413291], [-2.4144850649168057, -4.9696092437799253, 1.971452820409227, 2.7604129170663017, -0.16614061961796533]], [[-3.5760129132187934, -5.4931173285911203, -3.1366364424369237, -6.4362338416417728, -7.4429816406869751], [-7.1203255687486138, -9.2502024336262636, -0.31100908044375242, -6.9809867750016972, -9.0880006871311032], [0.033366364222060518, -3.0168483486311315, -2.4707965717017233, -1.1226739408808433, -1.5687935372060879], [-8.6703988850625642, -6.5546260327011101, -5.7376614953553231, -9.4945183987427857, -1.7111544085878672]], [[5.2332380840657891, 3.9591118714650291, 0.44637969123378074, 5.2430703027707386, 2.6024047495789953], [1.8135004391428473, 2.2666704640252773, 2.1663263942602269, 4.1087054044439242, 6.616494857279017], [9.2569513996664465, 2.9943340184613048, 3.2938518038331006, 0.7520720277512023, 4.4238226087341701], [4.4858291134075152, 2.6079825668391527, 5.8310817995847781, 1.8317229192025701, 9.2865388634285875]], [[-2.6614800460608157, 5.0455947991790255, -2.1154379370360621, -3.1445504484716533, 4.9971978151912575], [-1.089046448651243, 5.3758267121722509, -1.8151749221848377, 6.4124025859564719, 5.4336130291403721], [5.3313306146520789, 4.5225574107151365, -2.6386254372860671, 1.0098963083418813, 3.9688011438491113], [0.36423680537794034, 4.6219892078995111, 2.639307079420055, -0.72503739918645849, 5.1228187466737625]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_constData_rank4(self):
      arg0=Data(numpy.array([[-0.71489104262041359, -4.9417504800090288, -0.022246783233022427, -2.319374383024877, 3.8694749133365587], [-2.0608525516829568, -4.1762827312561326, 2.593539920776208, 3.9044109802932976, -0.81799011465572136], [2.4692999152044903, -0.8405808980057472, -3.3504652517438869, 3.7214989166689119, -1.1814247604613226], [-0.20224642365354573, -1.2779692173950998, 0.3028140061884379, -3.2932192929949675, -4.4095065457881333]]),self.functionspace)
      arg1=Data(numpy.array([[[[2.4365833674491917, 4.0204922014950863, -4.1958520139379942, 1.5498931201188615, -4.4777209139973024], [-4.4604455185736995, -3.8047923912039652, 0.35786357232293486, -1.7049361028226517, 4.4447809814336345], [0.41158640175037231, 3.3768261261839019, 1.594253692929442, 2.1704364148855584, -4.846970310737337], [-1.7846432253657198, -0.50340631844522399, -3.7625334149263621, 4.1393041651441145, -2.040685288562234]], [[-2.376961027056387, -1.935131364456403, -3.4895122226694975, 4.8854376626440281, -1.244454329069089], [-2.7042057516096207, 0.17113300689884881, -4.1239626628728532, -1.9745921712090766, -4.4134511571109254], [1.7503760334038345, 4.4208316397405998, -0.77311081339752619, 1.5981132325207872, 1.1713245685586715], [1.1509384418982211, -0.26227351227875495, -1.5831765495306782, -4.8181407624380679, -0.24266377136707895]], [[1.1493020280168835, 4.6396458692025746, -3.2264956252649304, -0.28901150771702167, 2.107228944734489], [0.95876568649723293, 1.5766366224150588, -1.7955443694847562, 4.7415263874020042, 3.3678141279191909], [-0.1655288068636267, 4.2646829063017133, 4.1010805176686524, 3.4010917634836666, 4.8571447810169701], [-1.8821132152625584, 1.6123258502647895, 1.8702365675432153, -2.1573878758280109, -4.4144218284068515]], [[0.034046770062752252, 1.7908297809483207, -2.6918649593606272, -4.7390255858271289, 0.84091269579824068], [-3.6915362198275989, 2.3100538496330936, 4.3982055354266887, -4.6828058266010526, 3.0685643808763992], [-0.6983167414139011, 0.93932667759090815, 2.9004456265181124, -2.4859609551694408, -0.71311300056353844], [-0.97198969971904958, -2.6540090558864637, -1.1473230589580363, -2.2399163456976199, 3.906833874612456]], [[-1.2831897582389562, -4.9064178798025129, 0.9439181463405335, -2.3750858055669677, -2.5130846285762187], [2.9489050679556943, -3.63250788918171, 0.030112982611226968, 1.1548874750639824, 3.0072113659859099], [-3.041610631703354, -3.5122633679812134, 0.89319870514042599, -3.8147115265721911, 4.460612395832003], [-0.84328260701206403, 3.2430369297413471, 0.91070748602971641, 2.4723493590160084, -2.9555112195899169]]], [[[4.7639001713168661, -2.5752213421050998, -1.4022720734883976, 4.8745125222122585, -2.6111349539092741], [-3.2921994686146583, -2.1381629360624221, -2.7434536625268069, -3.2172343577964204, -3.8879338114368531], [4.8063879468843389, 2.3290593393810557, -4.4107052444028225, -2.6016975235647077, -4.3672579690014492], [0.33751373138550456, 0.07095158431944526, 1.801943744872327, 2.3542032353668656, 4.5530486000634269]], [[-4.6124318097287116, -3.9204841243332975, -2.9434137858367104, -3.9349543227895678, -2.2978109994404052], [3.6900837581419257, -1.2820643130780418, -3.0240174874088366, 1.5852923358596263, 1.33173018210538], [-0.91598966343181676, -4.8344325972865834, 0.37072120083562066, 1.0639618377986579, -1.3788472649884396], [4.8118627210264631, -2.9638858939873391, 4.0460260844113112, -3.2880708604909668, 1.6084316896244211]], [[2.139014527579878, 3.9041291822165558, -2.6849441534249161, -3.0964279958714549, -3.4272788120942277], [-4.5073025004470724, -2.3159072062543675, 1.5071165767949815, -3.969094334553418, -2.4422587941131644], [2.136439301844586, 0.23390577067845619, 3.5862502569241865, 4.1012715989304027, 3.5105889374603549], [-1.5338425580118518, 1.1627045538723104, -2.1297296018895286, -3.5638427526970973, -2.7041039867356131]], [[-1.2136007390665151, -1.8459989346839056, -2.5842128283008394, 1.8972974652610635, -1.3084990891313231], [2.8165557915480042, -3.243957527982122, -2.4649094362007729, -2.1604971430244868, -0.37468588224656063], [0.84503231684493141, -0.12776239709251414, 3.3356987080488185, -3.4308361516468411, -2.2062381412682996], [3.4169309459717283, 2.1281043221791895, -3.0669173660389761, 2.3252736013622455, -0.09213293197894501]], [[3.2860667761381137, 4.5355087829694458, -1.9490695065949581, -2.1347011460851451, -4.549618387662079], [-4.4750605744858865, 0.78012724050392812, 0.91192907623122377, -1.5504508235772096, 0.24565080440512865], [-3.3415316211643287, -3.5910721811801913, -3.3942185587981291, 3.708890015951269, 0.54595398275345453], [4.1705770937996025, 0.1151344329874231, 0.97758752003569782, 2.3249802918452271, -0.30628450049263733]]], [[[2.8345291685706, 4.296287896869682, -3.3821257201348387, 1.4983024221078081, -2.4580101037232627], [-2.0265429370357513, 4.7820110641380502, 2.9843809532611747, 1.1296985950076808, 1.0942252757245701], [0.48483739463293229, -3.7270230718418063, -3.8532669589515756, 2.5591726077783132, 4.6566291856448441], [3.8165161183140324, 3.931551979723185, 0.40222203033031256, 1.6681235561079699, 0.21102439064319967]], [[4.542319521589155, -1.8701544116122615, -1.2351752252009538, -1.1024907215377833, 4.6958801472101346], [3.6838306842487665, -4.0691407047208594, -4.1251700651420773, -2.7957358357578399, -1.6686103240301851], [0.06530346006042631, 3.1513286773545293, -4.3587973447482309, -4.6024872770063521, -4.3807188098557157], [1.0102334696684974, -4.4561834328970491, -4.2880208096644932, -0.1415774070778566, 1.3325125298464737]], [[-1.3810350709319175, 2.5966935165396787, 3.2105894048291503, -3.0703849780480996, -3.4461635221275353], [0.74873203743639571, -1.861293998965492, -0.12456275072731149, 2.1470179548621466, 1.7569348854378566], [-1.607793407954492, -1.6890092092306439, 2.3880307988593987, -0.70592498867460662, 1.7796784843383335], [4.663345753129315, -2.602351253770141, -1.1494765297368748, 1.9064433096365816, 4.3584749246808396]], [[-1.419201543482882, 2.7762137709001298, -2.6179924350879391, 4.4895440444684134, -3.491256858131444], [2.396160560369256, 3.303302274828523, 0.95173996653481119, 2.9402917081912072, 4.0964055526450611], [0.15054058385607227, -1.4158091987707278, -2.3852694697918251, 0.83323173644369142, -0.37841819475290617], [2.6110001577829314, 3.3668394030514541, -1.5971642631843963, -4.3407019783619845, -2.1300855740747227]], [[-1.8282829736787254, -2.622123800038799, -1.7899523826567245, -3.9639802114790381, 2.2613053962119114], [-1.985905334973169, -4.8565219327506535, -3.634355340225448, 2.6454481957223663, -4.7717507817264888], [2.5982041221386973, 0.36062188369028725, 1.8632625137137984, -2.495317959396802, -1.0911403280921963], [-2.9223387862639791, 0.43109706014966065, 3.2473877859206617, -3.0704910567544497, 3.1271581216398179]]], [[[2.2594231948290862, 3.9342362151873118, -3.2220567321882787, -0.49534299991912523, -4.9338313029002077], [-1.2813261076471969, -3.9840393491247825, 3.7285682506689923, -3.6469665347626035, -1.6238172663699215], [3.307401183738758, 2.5790565291112273, 0.10953322030740065, 2.540785526162793, 0.37119153546039385], [-0.42764344424658951, 3.5269269831459873, -3.4247690124908092, -4.8561742112919104, -3.3168953918152866]], [[-1.9387150048357258, 3.8813835010838407, 0.64401777200117305, -3.5705853450860845, 3.6493684144035168], [-4.867899653644379, -0.91996199898330389, -2.6947807339522614, 1.9641201966934947, -0.40774221776783648], [-2.9349350995300014, 3.7419417237895356, 4.6858472874108177, -0.83244196086740718, 2.4650909608362879], [-3.1122242485640794, -0.037505069920127809, -4.9154672741042926, 2.548218900246245, 0.12651168622218911]], [[0.38059900584441309, 2.5109578267273278, -4.8010850390269875, 0.69820877782674629, -3.7126905752512354], [0.60723637127424368, -0.54707619799983398, -0.29945167926485627, -1.0048161496557038, -3.508603509616063], [-3.5760536265190113, -1.8555527667429326, 4.4811905495016422, 4.3381402415994312, -2.7971537793896193], [-4.4038415832617392, -3.1623564563667173, -1.5827116510546779, -2.8064602295652152, -2.3716449472310375]], [[3.0142599834189969, 0.92965454723651852, 4.9662784231985011, -4.5248765508974262, -4.4107612939186414], [3.8218987186865174, -2.4757181085004132, -2.4505038129456382, 0.45081122737500401, 2.7887933056387109], [-0.76242716452737547, 3.8914778065214968, -3.6701150556343007, -4.9509721357976773, 2.9447678346920423], [-1.1526794544718633, -2.2964257892460074, 4.1725287032470977, -2.9584361808341928, -2.8880253583243554]], [[-1.9349349957858242, 0.96931594008982547, 0.61593356451131598, -2.032249630267863, 1.8002365435200662], [2.1874765293503255, 0.10871728756739429, 0.41668722021961635, -3.3068234224534434, -0.82855625829458468], [-0.38199036073580217, 4.0791909041544159, -1.3928257830937252, 4.6142340581995178, 2.4346047890233971], [4.2923752739348018, -2.2488654097596203, -1.2777441955427973, -4.6398939297591664, -4.9317629321173717]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[1.7216923248287781, 3.3056011588746728, -4.9107430565584078, 0.8350020774984479, -5.192611956617716], [-5.1753365611941131, -4.5196834338243788, -0.35702747029747872, -2.4198271454430653, 3.729889938813221], [-0.30330464087004128, 2.6619350835634883, 0.87936265030902838, 1.4555453722651448, -5.5618613533577506], [-2.4995342679861334, -1.2182973610656376, -4.4774244575467756, 3.4244131225237009, -2.7555763311826476]], [[-7.3187115070654158, -6.8768818444654318, -8.4312627026785272, -0.056312817365000711, -6.1862048090781183], [-7.6459562316186496, -4.77061747311018, -9.0657131428818829, -6.9163426512181054, -9.3552016371199542], [-3.1913744466051943, -0.52091884026842905, -5.714861293406555, -3.3436372474882416, -3.7704259114503573], [-3.7908120381108077, -5.2040239922877838, -6.5249270295397075, -9.7598912424470967, -5.1844142513761078]], [[1.1270552447838611, 4.6173990859695522, -3.2487424084979528, -0.31125829095004409, 2.0849821615014665], [0.9365189032642105, 1.5543898391820363, -1.8177911527177786, 4.7192796041689817, 3.3455673446861685], [-0.18777559009664913, 4.2424361230686909, 4.07883373443563, 3.3788449802506442, 4.8348979977839477], [-1.9043599984955808, 1.590079067031767, 1.8479897843101929, -2.1796346590610334, -4.4366686116398739]], [[-2.2853276129621247, -0.52854460207655629, -5.0112393423855046, -7.0583999688520063, -1.4784616872266363], [-6.0109106028524764, -0.0093205333917834032, 2.0788311524018117, -7.00218020962593, 0.74918999785152218], [-3.0176911244387781, -1.3800477054339688, 0.58107124349323547, -4.8053353381943182, -3.0324873835884154], [-3.2913640827439266, -4.9733834389113412, -3.4666974419829133, -4.5592907287224964, 1.587459491587579]], [[2.5862851550976025, -1.0369429664659542, 4.8133930596770922, 1.494389107769591, 1.35639028476034], [6.818379981292253, 0.23696702415484872, 3.8995878959477857, 5.0243623884005411, 6.8766862793224686], [0.82786428163320469, 0.35721154535534527, 4.7626736184769847, 0.054763386764367539, 8.3300873091685617], [3.0261923063244947, 7.1125118430779057, 4.7801823993662751, 6.3418242723525671, 0.91396369374664177]]], [[[2.7030476196339093, -4.6360738937880566, -3.4631246251713543, 2.8136599705293017, -4.6719875055922309], [-5.3530520202976151, -4.1990154877453794, -4.8043062142097632, -5.2780869094793772, -5.9487863631198099], [2.7455353952013821, 0.26820678769809891, -6.4715577960857793, -4.6625500752476645, -6.428110520684406], [-1.7233388202974522, -1.9899009673635115, -0.25890880681062978, 0.29335068368390882, 2.4921960483804702]], [[-8.7887145409848451, -8.0967668555894292, -7.1196965170928426, -8.1112370540457004, -6.4740937306965378], [-0.4861989731142069, -5.4583470443341744, -7.2003002186649692, -2.5909903953965063, -2.8445525491507526], [-5.0922723946879493, -9.010715328542716, -3.8055615304205119, -3.1123208934574746, -5.5551299962445722], [0.63557998977033048, -7.1401686252434722, -0.13025664684482141, -7.4643535917470993, -2.5678510416317115]], [[4.732554448356086, 6.4976691029927638, -0.091404232648708117, -0.50288807509524691, -0.8337388913180197], [-1.9137625796708644, 0.27763271452184046, 4.1006564975711894, -1.37555441377721, 0.15128112666304361], [4.729979222620794, 2.8274456914546642, 6.1797901777003945, 6.6948115197066107, 6.1041288582365629], [1.0596973627643562, 3.7562444746485184, 0.46381031888667934, -0.97030283192088929, -0.11056406595940516]], [[2.6908102412267825, 2.0584120456093919, 1.3201981519924582, 5.801708445554361, 2.5959118911619745], [6.7209667718413018, 0.66045345231117558, 1.4395015440925247, 1.7439138372688108, 3.529725098046737], [4.749443297138229, 3.7766485832007834, 7.2401096883421161, 0.4735748286464565, 1.6981728390249979], [7.3213419262650259, 6.0325153024724871, 0.83749361425432145, 6.2296845816555431, 3.8122780483143526]], [[2.4680766614823924, 3.7175186683137245, -2.7670596212506795, -2.9526912607408664, -5.3676085023178004], [-5.2930506891416078, -0.037862874151793235, 0.093938961575502411, -2.368440938232931, -0.5723393102505927], [-4.15952173582005, -4.4090622958359127, -4.2122086734538504, 2.8908999012955476, -0.27203613190226683], [3.3525869791438812, -0.70285568166829826, 0.15959740537997646, 1.5069901771895058, -1.1242746151483587]]], [[[5.3038290837750903, 6.7655878120741724, -0.91282580493034837, 3.9676023373122984, 0.011289811481227652], [0.44275697816873905, 7.2513109793425405, 5.453680868465665, 3.5989985102121711, 3.5635251909290604], [2.9541373098374226, -1.2577231566373159, -1.3839670437470852, 5.0284725229828036, 7.1259291008493344], [6.2858160335185227, 6.4008518949276754, 2.8715219455348029, 4.1374234713124602, 2.68032430584769]], [[3.7017386235834078, -2.7107353096180087, -2.075756123206701, -1.9430716195435305, 3.8552992492043874], [2.8432497862430193, -4.9097216027266066, -4.9657509631478245, -3.6363167337635871, -2.5091912220359323], [-0.77527743794532089, 2.3107477793487821, -5.1993782427539781, -5.4430681750120993, -5.2212997078614629], [0.16965257166275016, -5.2967643309027963, -5.1286017076702404, -0.9821583050836038, 0.49193163184072652]], [[-4.7315003226758048, -0.75377173520420815, -0.13987584691473653, -6.4208502297919861, -6.7966287738714222], [-2.6017332143074912, -5.2117592507093793, -3.4750280024711984, -1.2034472968817402, -1.5935303663060303], [-4.9582586596983784, -5.0394744609745308, -0.96243445288448815, -4.0563902404184935, -1.5707867674055533], [1.3128805013854281, -5.9528165055140274, -4.4999417814807616, -1.4440219421073053, 1.0080096729369528]], [[2.3022973731860299, 6.4977126875690416, 1.1035064815809728, 8.2110429611373252, 0.23024205853746782], [6.1176594770381678, 7.0248011914974349, 4.673238883203723, 6.6617906248601191, 7.817904469313973], [3.8720395005249841, 2.3056897178981841, 1.3362294468770868, 4.5547306531126033, 3.3430807219160057], [6.3324990744518432, 7.088338319720366, 2.1243346534845156, -0.6192030616930726, 1.5914133425941892]], [[-3.009707734140048, -3.8035485605001216, -2.9713771431180471, -5.1454049719403603, 1.0798806357505888], [-3.1673300954344916, -6.0379466932119765, -4.8157801006867711, 1.4640234352610437, -5.953175542187811], [1.4167793616773747, -0.82080287677103536, 0.68183775325247575, -3.6767427198581246, -2.2725650885535189], [-4.1037635467253022, -0.75032770031166196, 2.0659630254593391, -4.2519158172157727, 1.9457333611784953]]], [[[2.0571767711755404, 3.7319897915337661, -3.4243031558418244, -0.69758942357267095, -5.1360777265537534], [-1.4835725313007426, -4.1862857727783283, 3.5263218270154466, -3.8492129584161492, -1.8260636900234672], [3.1051547600852123, 2.3768101054576816, -0.092713203346145079, 2.3385391025092472, 0.16894511180684813], [-0.62988986790013524, 3.3246805594924416, -3.627015436144355, -5.0584206349454561, -3.5191418154688323]], [[-3.2166842222308256, 2.6034142836887408, -0.63395144539392678, -4.8485545624811843, 2.3713991970084169], [-6.1458688710394789, -2.1979312163784037, -3.9727499513473612, 0.68615097929839486, -1.6857114351629363], [-4.2129043169251013, 2.4639725063944358, 3.4078780700157179, -2.110411178262507, 1.187121743441188], [-4.3901934659591788, -1.3154742873152276, -6.1934364914993925, 1.2702496828511451, -1.1514575311729107]], [[0.68341301203285099, 2.8137718329157657, -4.4982710328385496, 1.0010227840151842, -3.4098765690627975], [0.91005037746268158, -0.24426219181139608, 0.0033623269235816267, -0.70200214346726586, -3.2057895034276251], [-3.2732396203305734, -1.5527387605544947, 4.7840045556900801, 4.6409542477878691, -2.4943397732011814], [-4.1010275770733013, -2.8595424501782793, -1.27989764486624, -2.5036462233767773, -2.0688309410425996]], [[-0.27895930957597059, -2.363564745758449, 1.6730591302035336, -7.8180958438923938, -7.7039805869136089], [0.52867942569154991, -5.7689374014953803, -5.7437231059406058, -2.8424080656199635, -0.50442598735625666], [-4.055646457522343, 0.59825851352652926, -6.9633343486292683, -8.2441914287926448, -0.34845145830292523], [-4.4458987474668312, -5.5896450822409749, 0.87930941025213016, -6.2516554738291603, -6.1812446513193233]], [[-6.344441541573957, -3.4401906056983078, -3.7935729812768173, -6.4417561760559963, -2.609270002268067], [-2.2220300164378077, -4.300789258220739, -3.9928193255685169, -7.7163299682415767, -5.238062804082718], [-4.7914969065239355, -0.33031564163371741, -5.8023323288818585, 0.20472751241138454, -1.9749017567647362], [-0.1171312718533315, -6.6583719555477536, -5.6872507413309306, -9.0494004755472996, -9.3412694779055059]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[-3.9680332630804527, -1.2485343614520037], [2.8517544931675722, -2.4145010521873109]], [[1.6336300233618068, -0.57383267508517122], [0.92454530530968615, -1.9971184261253869]], [[0.9708965338441331, 0.62434821608296431], [-1.6107401479994032, 4.6865540792218709]], [[-4.5671624458684068, 2.8641254866605204], [3.4426059768942281, -4.6973649898604712]], [[3.7097216056290705, 4.8028529380777361], [3.0423175949895978, 0.74130152679527139]], [[1.9834007722289506, -0.7460511005782724], [0.57691219936794358, -0.29060054777219868]]]),self.functionspace)
      arg1=Data(numpy.array([[[-3.7692764133737544, -3.438832582164105], [-4.3264432497535044, 3.3119277926202155]], [[-3.1189099396535616, -3.1780923878495226], [3.7022558842819411, -3.5447709793948521]], [[4.1409915998582605, 4.4042545622567655], [3.4092701218443917, -1.4695577318215349]], [[-0.93954316739612587, -2.5991638395584147], [3.48043368803779, -1.9968652923457553]], [[-0.91554553914499603, 1.2905708750478073], [-0.30517746990303074, -3.6511793348658905]], [[1.8371004554183523, -1.9718295508763086], [-1.7700190461390717, -1.3130957974969535]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[-7.7373096764542071, -4.6873669436161087], [-1.4746887565859321, 0.8974267404329046]], [[-1.4852799162917547, -3.7519250629346939], [4.6268011895916272, -5.541889405520239]], [[5.1118881337023936, 5.0286027783397298], [1.7985299738449885, 3.216996347400336]], [[-5.5067056132645327, 0.26496164710210568], [6.9230396649320181, -6.6942302822062265]], [[2.7941760664840745, 6.0934238131255434], [2.737140125086567, -2.9098778080706191]], [[3.8205012276473029, -2.717880651454581], [-1.1931068467711281, -1.6036963452691522]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[2.6825074184223823, -3.0157462900311871], [-2.8454903494271511, -2.517386331074225]], [[4.2527421348024941, 2.2228604456200305], [-3.4793534756338595, 2.2032572280626344]]], [[[-3.4097003149047467, 0.27337836441595176], [-3.2404704835409315, -2.4197675003761243]], [[-2.5236803427603438, 4.6379917789100649], [-3.5617600312080642, 3.4200890236071491]]], [[[-3.8362676469468315, -4.0784822260953977], [0.66757475629508178, -2.8517155505674099]], [[0.53239355649368925, 2.483307015539018], [4.7533550224549117, 4.7991881259082607]]], [[[0.14593140014915917, 0.088888563351343208], [4.1161914660748877, 3.1273580173351316]], [[-2.5284393064765842, -3.8441081734517155], [-2.2575520955920911, -1.3919006416117874]]], [[[-4.82929515382415, 0.032998325319702815], [2.6733567450850684, -3.952618791804301]], [[4.9022528143356716, -2.2686318406936645], [-3.3965639866430397, -2.8359856725229635]]], [[[3.3728068618592921, 3.5012617223570484], [1.2629129259444793, 4.7410732018537747]], [[-1.5610213842146403, -3.3342873576937961], [0.67890542341597637, 1.3969652323323558]]]]),self.functionspace)
      arg1=Data(numpy.array([[[2.7688528058351469, -0.65766424887270958], [-1.6318904322806063, -1.4152138661856184]], [[-2.780458011071465, 1.5767167809656337], [2.5476624669742804, 2.4379122704803109]], [[0.75327280607937031, -1.2101480314861379], [-2.0699693033028819, 0.1861683103975551]], [[0.0027477260092965139, 1.9812269618966605], [2.7388881019783202, 0.66434965224064246]], [[1.525342179241056, 1.7413043684897769], [-0.96655441671190978, -1.0235663872769338]], [[3.5949965218149593, -0.58356772508187049], [2.3799966095826131, 2.9319998875994715]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[5.4513602242575292, -0.24689348419604018], [-3.5031545982998606, -3.1750505799469346]], [[2.6208517025218878, 0.59097001333942423], [-4.8945673418194779, 0.78804336187701596]]], [[[-6.1901583259762116, -2.5070796466555132], [-1.6637537025752978, -0.84305071941049059]], [[0.02398212421393664, 7.1856542458843453], [-1.1238477607277533, 5.85800129408746]]], [[[-3.0829948408674612, -3.3252094200160274], [-0.54257327519105614, -4.0618635820535474]], [[-1.5375757468091926, 0.41333771223613613], [4.9395233328524668, 4.9853564363058158]]], [[[0.14867912615845569, 0.091636289360639722], [6.0974184279715482, 5.1085849792317921]], [[0.21044879550173601, -1.1052200714733953], [-1.5932024433514487, -0.72755098937114493]]], [[[-3.303952974583094, 1.5583405045607588], [4.4146611135748453, -2.2113144233145241]], [[3.9356983976237618, -3.2351862574055743], [-4.4201303739199735, -3.8595520597998973]]], [[[6.9678033836742514, 7.0962582441720077], [0.67934520086260886, 4.1575054767719042]], [[0.81897522536797274, -0.95429074811118308], [3.6109053110154479, 4.3289651199318273]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_constData_rank4(self):
      arg0=Data(numpy.array([[[-2.2723599708749864, -4.2613431270805915], [-2.2574810873126836, 3.1556428815518274]], [[0.6077149250896996, 2.5715055053322642], [-4.0857771539279328, 2.9872138903608576]], [[-0.42530217221002253, 0.40611649047273168], [-0.26115587645103666, 2.4072979756174906]], [[4.6492401037274682, -3.0987512952372995], [4.0086783960346022, 4.8410679636603788]], [[0.0089533698531347738, 1.4167295806670213], [1.6315206147386574, 1.3343787004278331]], [[1.689704356191422, -1.846829035751627], [1.4086535050733211, 1.1968904747692903]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-3.3154211527675459, 1.7228456091673632], [0.89538842748558611, -0.2837940548494462]], [[2.2051002184740334, -2.0963415657203823], [2.2352894778238666, 1.3204182076706701]]], [[[-3.8809253311347511, -4.452408079497328], [3.8361663283139205, 4.8076893456149197]], [[1.5062772597337695, -1.6200176163166926], [0.075567794005886313, -4.190760003754205]]], [[[0.95090400854861912, 0.45613720660888557], [4.6826997075424899, 2.7615982192328392]], [[-1.3681780583753467, 2.0372894657460758], [0.30703789218638278, -1.2255359502101193]]], [[[3.7928721139020922, 1.5153311335206805], [1.8634628888805516, 3.0642251123219655]], [[2.8320270878621461, -4.1746907070018233], [-2.3960611349703842, 2.3870733882857404]]], [[[3.1537245908636251, 3.7445298084777292], [4.2798581138256004, -3.6141260301868128]], [[1.0698708835436737, -0.77506526682680743], [3.7370544690260328, 2.5246341699541475]]], [[[-1.1050961293324746, 3.6938876432256382], [2.8377428557101334, -0.031094142238678124]], [[0.0086073429928781309, 3.9232125603177153], [0.33252920072858494, -0.066404310491892993]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-5.5877811236425323, -0.54951436170762324], [-3.3659546995950054, -4.5451371819300377]], [[-0.05238086883865023, -4.3538226530330659], [5.3909323593756939, 4.4760610892224975]]], [[[-3.2732104060450515, -3.8446931544076284], [6.4076718336461846, 7.3791948509471839]], [[-2.5794998941941634, -5.7057947702446254], [3.0627816843667439, -1.2035461133933474]]], [[[0.52560183633859658, 0.030835034398863037], [5.0888161980152216, 3.1677147097055709]], [[-1.6293339348263833, 1.7761335892950392], [2.7143358678038734, 1.1817620254073713]]], [[[8.4421122176295604, 6.1645712372481487], [-1.235288406356748, -0.034526182915334047]], [[6.8407054838967483, -0.16601231096722113], [2.4450068286899946, 7.2281413519461193]]], [[[3.1626779607167599, 3.753483178330864], [5.6965876944926217, -2.1973964495197915]], [[2.701391498282331, 0.85645534791184996], [5.0714331694538659, 3.8590128703819806]]], [[[0.58460822685894742, 5.3835919994170602], [0.99091381995850636, -1.8779231779903052]], [[1.4172608480661992, 5.3318660653910364], [1.5294196754978753, 1.1304861642773973]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-0.18900486011293438, 3.5051205998454833, -2.3665317560023871, -1.9877279666163608], [-1.3314190755777524, -0.78410763139626205, 3.7224857326170184, -1.4604085607170472], [1.7179883058514314, -1.1618244243885387, -1.61982037717042, 0.79638926206268046]], [[-1.8205499673306669, -2.4520364941311081, -3.6831284099031283, 0.93089158267828509], [3.9769205046672091, 3.4476169249358897, 1.4847750829480555, 0.14575112002029211], [-3.5590851304584428, 0.043879438094119649, -2.7755781552266079, -3.313156602765023]]], [[[-1.1583013392858641, -3.2038429092406409, -2.7953468091469444, 2.329088965766303], [2.4911085752436772, -2.8666411395339719, 2.1940184434471801, -4.5685570237453996], [-2.7004484464461429, 0.55004174389856431, 0.64045224639739118, -1.0501669938935967]], [[-2.0034727722237609, -4.1176582341777559, -1.9040645647993881, 2.9488079170042614], [3.7029139395536337, 1.3727989652370711, 4.5625430386861883, 2.065216274058125], [-0.36651079453525615, 4.0385559984800032, 2.3646661729072243, -2.1433904510578285]]], [[[3.4247436971243186, -4.8861996915611714, 3.5550238018793738, -0.90555034261471157], [-3.1137889734429458, -4.1845532595067532, 1.0728359452185918, 3.1006240634498781], [-1.1853704023834402, 1.4494267033365738, -2.3248122923890389, -0.88742162200548957]], [[-2.3117799715215579, -2.0823351463836115, 0.0038152630643049079, -2.8717110478630223], [0.36906071975967425, -1.0749563905308515, -0.5447207830429317, -4.6961022668789143], [1.0214821515140704, -3.4229847461361973, 0.69864243637614187, -2.0392152464047673]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-3.5652452186040007, -4.0830850205911959, 3.7961733887869631, 4.8971760050636917], [-4.3159312428542185, 2.6367803186145728, 0.85017213134359437, 2.0183603441217119], [-0.68203105048388668, -4.5609596389478284, -2.1906837723079331, -2.5194366329356068]], [[-1.5911855418882945, -0.58316677642110815, 1.7979331916075934, -3.5081684238373745], [-1.33012294517179, -4.3545285329540429, -4.8593050597315859, -1.008532530509779], [-3.5689679792968221, -3.4941949812022322, 4.3163173339981853, -4.5588200095161593]]], [[[1.6095479434305524, 1.9548813413225643, -0.99269102320892788, 2.9016743863621217], [3.7032130908872123, 4.2624276411722537, -4.240103138484967, 1.424007137682346], [-0.9402847644456731, 2.7939904772678563, -1.6505593098436822, 1.707432302305901]], [[-4.8797470909384542, 4.2554500994680389, -3.1170340061686028, 2.6264074296601354], [-0.41293529296971698, 0.97680549651253656, 4.8961259018085421, -3.5913934137355707], [0.31009539138460518, 1.2139474396677059, 4.6105788044448701, -0.41488794945627205]]], [[[-3.4148222180747814, -2.7934679104141513, -2.2220965044295591, -0.38191170011071751], [-2.2328478684463891, -2.928337335707385, -1.8841221588904555, 1.4774922011945577], [-2.3158827282417405, -0.95289615594278132, -0.71656886191542135, 4.6215975971208838]], [[-2.4197542293103655, -0.64820175295205651, -4.148349705376396, -3.230931179386519], [4.9728993132407346, -3.44399640159059, -1.8898661555816831, -1.7393681012689735], [2.120102975153797, 0.69474579714023132, -3.352732261210539, 0.66238822032283284]]]]),self.functionspace)
      res=add(arg0,arg1)
      ref=Data(numpy.array([[[[-3.7542500787169351, -0.57796442074571264, 1.429641632784576, 2.9094480384473309], [-5.6473503184319709, 1.8526726872183108, 4.5726578639606128, 0.55795178340466478], [1.0359572553675447, -5.7227840633363671, -3.8105041494783531, -1.7230473708729264]], [[-3.4117355092189614, -3.0352032705522163, -1.8851952182955349, -2.5772768411590894], [2.6467975594954192, -0.90691160801815318, -3.3745299767835304, -0.86278141048948687], [-7.1280531097552648, -3.4503155431081125, 1.5407391787715774, -7.8719766122811823]]], [[[0.45124660414468831, -1.2489615679180766, -3.7880378323558723, 5.2307633521284247], [6.1943216661308895, 1.3957865016382818, -2.0460846950377869, -3.1445498860630536], [-3.640733210891816, 3.3440322211664206, -1.010107063446291, 0.65726530841230435]], [[-6.883219863162215, 0.13779186529028298, -5.0210985709679914, 5.5752153466643968], [3.2899786465839167, 2.3496044617496077, 9.4586689404947304, -1.5261771396774457], [-0.056415403150650967, 5.2525034381477091, 6.9752449773520944, -2.5582784005141006]]], [[[0.0099214790495372185, -7.6796676019753232, 1.3329272974498148, -1.2874620427254291], [-5.3466368418893353, -7.1128905952141377, -0.81128621367186371, 4.5781162646444358], [-3.5012531306251806, 0.49653054739379243, -3.0413811543044602, 3.7341759751153942]], [[-4.7315342008319234, -2.730536899335668, -4.1445344423120911, -6.1026422272495413], [5.3419600330004089, -4.518952792121441, -2.4345869386246148, -6.4354703681478878], [3.1415851266678674, -2.728238948995966, -2.6540898248343971, -1.3768270260819344]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_expandedData_rank0(self):
      arg0=Data(-0.451776787022,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-4.96078854564)+(1.-msk_arg1)*(1.70956953022)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-5.41256533266)+(1.-msk_ref)*(1.2577927432)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([1.9555630648437621, -3.2249859024626515]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.13052028704)+(1.-msk_arg1)*(1.14803753784)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.0860833518787718, -0.094465615427641758])+(1.-msk_ref)*numpy.array([3.1036006026835139, -2.0769483646228997])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_expandedData_rank1(self):
      arg0=Data(-3.96044087015,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.2984367334696163, 3.6190852378059226])+(1.-msk_arg1)*numpy.array([0.89827816647682113, -1.3906353778124467])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.33799586332112597, -0.34135563234256772])+(1.-msk_ref)*numpy.array([-3.0621627036716692, -5.3510762479609371])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[-4.5310047516305554, 3.7515460562109144, -4.2323827824999327, 0.30132212847309425, -2.670490672932412], [-0.40913386129173634, 1.8833967151441708, 1.2746569384179205, -1.4014413002853088, 2.1764246971634682], [4.3830263611207787, 3.3385727384441584, 2.7734524640624105, -2.8559216965993284, -4.4878405988670895], [3.0031577208462608, -1.7085845437004887, -0.03643431346888093, -0.73677701520750816, 4.3232630744323846]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.72400476769)+(1.-msk_arg1)*(-3.29800426409)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.8069999839445279, 5.4755508238969419, -2.5083780148139052, 2.0253268961591218, -0.94648590524638454], [1.3148709063942912, 3.6074014828301983, 2.998661706103948, 0.32256346740071873, 3.9004294648494957], [6.1070311288068062, 5.0625775061301859, 4.497457231748438, -1.1319169289133009, -2.763835831181062], [4.7271624885322883, 0.015420223985538772, 1.6875704542171466, 0.98722775247851935, 6.0472678421184121]])+(1.-msk_ref)*numpy.array([[-7.8290090157189525, 0.45354179212251777, -7.5303870465883289, -2.9966821356153024, -5.9684949370208091], [-3.707138125380133, -1.4146075489442258, -2.0233473256704761, -4.6994455643737059, -1.1215795669249284], [1.0850220970323821, 0.040568474355761719, -0.52455180002598611, -6.153925960687725, -7.7858448629554857], [-0.29484654324213588, -5.0065888077888854, -3.3344385775572776, -4.0347812792959044, 1.0252588103439879]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_expandedData_rank2(self):
      arg0=Data(2.26906278137,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.8232535555521547, 3.7424594098872923, 3.8573198153508379, 1.5311105182557725, 1.9911345273872794], [4.1557910980765964, -0.81347233295562571, -4.4178214020746314, 1.2456211769588377, -0.22486552409758431], [-3.118504714357023, -4.3577823763867674, -4.4156663396951785, 3.3159403785831216, -2.2841625677417401], [1.9470304127781626, -0.057370199782358355, 3.0528927197366187, 0.7650108448021502, -3.4673328675115189]])+(1.-msk_arg1)*numpy.array([[1.6948805056484417, 2.7284358484814675, -1.5204131252236373, 4.4890755940426157, 2.8668580160850663], [3.5938943599175168, 4.9348600026174338, 4.5975336890663669, 0.45006989166663836, 4.6568401336398644], [-1.541036343313078, 4.5884783989788005, -0.39469955051333727, -2.6280355972649194, -3.9294049024644471], [0.43192581753708126, -1.9263123882624411, -3.1408470411303293, -1.8406327472048556, -4.8145064922329066]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.0923163369228446, 6.0115221912579822, 6.1263825967215277, 3.8001732996264623, 4.2601973087579692], [6.4248538794472863, 1.4555904484150641, -2.1487586207039415, 3.5146839583295275, 2.0441972572731055], [-0.84944193298633319, -2.0887195950160775, -2.1466035583244887, 5.5850031599538115, -0.01509978637105025], [4.2160931941488524, 2.2116925815883315, 5.3219555011073085, 3.03407362617284, -1.1982700861408291]])+(1.-msk_ref)*numpy.array([[3.9639432870191316, 4.9974986298521573, 0.74864965614705259, 6.7581383754133055, 5.1359207974557561], [5.8629571412882067, 7.2039227839881237, 6.8665964704370568, 2.7191326730373282, 6.9259029150105542], [0.7280264380576118, 6.8575411803494903, 1.8743632308573526, -0.35897281589422958, -1.6603421210937572], [2.7009885989077711, 0.34275039310824873, -0.87178425975963947, 0.42843003416583425, -2.5454437108622168]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[-2.930290660204653, -0.48655137804373183], [-0.39068047272594963, -0.39207940842227451]], [[0.83692578357283853, -4.2738070576671383], [2.0051973033495525, 3.2336724425395627]], [[1.9793853315447318, -4.0861229491394333], [2.7194962720690876, -2.196650454292369]], [[4.2005299438629358, -1.3006253862553354], [-2.1150812322174293, -2.0035354665174698]], [[1.9531576344551151, -1.1617647986077451], [-4.1504729498977389, 1.9100667217339726]], [[-0.94877919512908004, -2.640532289504105], [2.5154864806493924, -4.1996201363458283]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.25429239274)+(1.-msk_arg1)*(2.2340153796)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.67599826746626146, 1.7677410146946597], [1.8636119200124419, 1.862212984316117]], [[3.0912181763112301, -2.0195146649287468], [4.259489696087944, 5.4879648352779542]], [[4.2336777242831234, -1.8318305564010418], [4.9737886648074792, 0.0576419384460225]], [[6.4548223366013273, 0.95366700648305613], [0.13921116052096227, 0.25075692622092172]], [[4.2074500271935067, 1.0925275941306465], [-1.8961805571593473, 4.1643591144723642]], [[1.3055131976093115, -0.38623989676571346], [4.7697788733877839, -1.9453277436074368]]])+(1.-msk_ref)*numpy.array([[[-0.69627528060186794, 1.7474640015590532], [1.8433349068768354, 1.8419359711805106]], [[3.0709411631756236, -2.0397916780643532], [4.2392126829523376, 5.4676878221423477]], [[4.2134007111475169, -1.8521075695366482], [4.9535116516718727, 0.037364925310416019]], [[6.4345453234657208, 0.93338999334744965], [0.11893414738535579, 0.23047991308531524]], [[4.1871730140579002, 1.07225058099504], [-1.9164575702949538, 4.1440821013367577]], [[1.285236184473705, -0.40651690990131994], [4.7495018602521775, -1.9656047567430432]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_expandedData_rank3(self):
      arg0=Data(4.02536028787,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.5314948986680141, -0.49437473619891126], [-2.1047094631868091, -0.0022098127579370797]], [[-3.3333741303562614, 2.3662067338534589], [-2.8361110268467336, 2.3907451165171718]], [[4.839601039895955, 1.9408613064419677], [2.3384222765821105, -3.1395212220980384]], [[-0.58362627152187585, -1.8185104336381439], [0.88129497078717378, 4.2122033721277052]], [[-0.21362442430003181, -1.0751792421450954], [0.13527118171099328, 4.4120964573868484]], [[-2.4945956630506707, 4.7263072078786763], [2.9166689825985559, 2.013283610116047]]])+(1.-msk_arg1)*numpy.array([[[-3.7331277599586796, -3.4995258519736994], [0.38793447982448193, 0.29398352580243525]], [[4.329217060940481, -2.5385271533870735], [2.2724832866000781, -1.9852848411174415]], [[-0.80971108975812722, 3.5998227589311718], [-4.8756782977053987, -2.0233161283684211]], [[-1.7640291680052678, -0.95955001527843109], [1.8156562289993348, -2.8368515611362612]], [[4.4672168411732773, -4.3285368933238786], [2.6282568394211827, 0.26920387247327326]], [[0.85446555158889748, 3.6211167683752539], [-2.431396285480576, 1.241130404335296]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.4938653891985458, 3.5309855516676487], [1.9206508246797509, 4.0231504751086229]], [[0.69198615751029857, 6.3915670217200189], [1.1892492610198264, 6.4161054043837318]], [[8.864961327762515, 5.9662215943085277], [6.3637825644486705, 0.88583906576852156]], [[3.4417340163446841, 2.2068498542284161], [4.9066552586537338, 8.2375636599942652]], [[3.8117358635665282, 2.9501810457214646], [4.1606314695775533, 8.4374567452534084]], [[1.5307646248158893, 8.7516674957452363], [6.9420292704651159, 6.0386438979826069]]])+(1.-msk_ref)*numpy.array([[[0.29223252790788035, 0.52583443589286061], [4.4132947676910419, 4.3193438136689952]], [[8.3545773488070409, 1.4868331344794865], [6.2978435744666381, 2.0400754467491184]], [[3.2156491981084327, 7.6251830467977317], [-0.85031800983883876, 2.0020441594981389]], [[2.2613311198612922, 3.0658102725881289], [5.8410165168658947, 1.1885087267302987]], [[8.4925771290398373, -0.30317660545731862], [6.6536171272877427, 4.2945641603398332]], [[4.8798258394554574, 7.6464770562418138], [1.593964002385984, 5.2664906922018559]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[-1.8081102459147615, 1.4169695807085834, -2.9645942113266708, -2.3294748414293274], [2.7735480744811909, -3.9058454979582189, 2.3226242643185664, 4.5359163572994561], [-0.17442063632607141, 2.8177903053666657, -4.9217808370455938, -0.43652983019232039]], [[4.8865356889101648, 3.13145192590747, -0.45513914770353381, 2.1709158448528765], [-1.9899313103031355, 1.7980050621915318, -2.9405655207226413, 1.8732240212545568], [2.6902440941304215, -1.7663048638546606, 1.108047737694581, -3.6643558743188342]]], [[[-0.81741182604882745, -3.6083253400912465, -2.0171916215717891, 3.9245230485673979], [-1.2381317287384954, 4.6709035409240851, -0.34295773722119005, -4.0645180031161301], [0.31396060956700023, 4.5676451355602623, 4.8074976829744749, -3.2718981934902294]], [[0.63799590866927236, 4.178635002427681, -0.54332910047796279, 1.2288012270194795], [-3.0234857783165658, -0.48299290514749327, 0.042626872818264161, 4.562854827163866], [2.0287828988529064, -4.9628389663356653, 4.5597708522033891, -2.1496649465505513]]], [[[-0.90814282121353962, -4.8510374471624784, -1.4669688459928878, -1.4754653302266152], [-3.1250645738223537, 4.1343334242546828, 2.0616451419449415, 4.6027950860849849], [-1.7953487634796241, -4.161000887031391, -1.8522501013299983, 1.4554300553616271]], [[3.697510529953945, 3.1467350490869688, -4.8740286387568315, -4.6603805254519592], [1.6992446071101988, -1.701737477176045, -4.1479703736600761, -3.6377802591709552], [-4.5639822402489036, 2.7191039849803325, 0.68720121943543599, -2.2263152325026105]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.864999420079)+(1.-msk_arg1)*(-4.29068662691)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.6731096659936497, 0.55197016062969517, -3.829593631405559, -3.1944742615082156], [1.9085486544023027, -4.7708449180371071, 1.4576248442396782, 3.6709169372205679], [-1.0394200564049596, 1.9527908852877776, -5.786780257124482, -1.3015292502712086]], [[4.0215362688312766, 2.2664525058285818, -1.320138567782422, 1.3059164247739883], [-2.8549307303820237, 0.93300564211264359, -3.8055649408015295, 1.0082246011756686], [1.8252446740515333, -2.6313042839335488, 0.24304831761569279, -4.5293552943977229]]], [[[-1.6824112461277156, -4.4733247601701347, -2.8821910416506773, 3.0595236284885097], [-2.1031311488173836, 3.8059041208451969, -1.2079571573000782, -4.9295174231950183], [-0.55103881051188797, 3.7026457154813741, 3.9424982628955867, -4.1368976135691176]], [[-0.22700351140961583, 3.3136355823487929, -1.408328520556851, 0.36380180694059128], [-3.888485198395454, -1.3479923252263815, -0.82237254726062403, 3.6978554070849778], [1.1637834787740182, -5.8278383864145535, 3.6947714321245009, -3.0146643666294395]]], [[[-1.7731422412924278, -5.7160368672413666, -2.331968266071776, -2.3404647503055034], [-3.9900639939012419, 3.2693340041757946, 1.1966457218660533, 3.7377956660060967], [-2.6603481835585123, -5.0260003071102792, -2.7172495214088865, 0.59043063528273887]], [[2.8325111098750568, 2.2817356290080806, -5.7390280588357196, -5.5253799455308474], [0.83424518703131056, -2.5667368972549331, -5.0129697937389643, -4.5027796792498433], [-5.4289816603277918, 1.8541045649014443, -0.1777982006434522, -3.0913146525814987]]]])+(1.-msk_ref)*numpy.array([[[[-6.0987968728202571, -2.8737170461969122, -7.2552808382321663, -6.6201614683348229], [-1.5171385524243046, -8.1965321248637153, -1.9680623625869291, 0.24522973039396057], [-4.4651072632315669, -1.4728963215388298, -9.2124674639510893, -4.7272164570978159]], [[0.59584906200466925, -1.1592347009980255, -4.7458257746090293, -2.119770782052619], [-6.280617937208631, -2.4926815647139637, -7.2312521476281368, -2.4174626056509387], [-1.600442532775074, -6.0569914907601561, -3.1826388892109145, -7.9550425012243302]]], [[[-5.108098452954323, -7.899011966996742, -6.3078782484772846, -0.36616357833809765], [-5.5288183556439909, 0.38021691401858959, -4.6336443641266856, -8.3552046300216247], [-3.9767260173384953, 0.27695850865476679, 0.51681105606897937, -7.5625848203957249]], [[-3.6526907182362232, -0.11205162447781447, -4.8340157273834583, -3.061885399886016], [-7.3141724052220614, -4.7736795320529888, -4.2480597540872314, 0.27216820025837052], [-2.2619037280525891, -9.2535255932411609, 0.26908422529789355, -6.4403515734560468]]], [[[-5.1988294481190351, -9.141724074067973, -5.7576554728983833, -5.7661519571321111], [-7.4157512007278488, -0.15635320265081276, -2.229041484960554, 0.31210845917948937], [-6.0860353903851196, -8.4516875139368857, -6.1429367282354939, -2.8352565715438685]], [[-0.59317609695155049, -1.1439515778185267, -9.164715265662327, -8.9510671523574548], [-2.5914420197952968, -5.9924241040815405, -8.4386570005655717, -7.9284668860764507], [-8.8546688671543983, -1.5715826419251631, -3.6034854074700595, -6.5170018594081061]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_expandedData_rank4(self):
      arg0=Data(0.415968380946,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.9964026660550802, -1.2022763821935314, 1.545946841641296, 3.8122491184020291], [3.2149717525039971, 2.1196386310086126, 0.98289658780072475, -3.4559431647536076], [1.7157176943432226, -0.66122615198877721, 3.8022856664539422, -3.5174852693756264]], [[2.5293460436501096, -3.4655420173723006, 1.9851632694614594, 0.20781744440155592], [3.4680347478770237, 2.4516896969838511, 1.6601943818676537, -4.7945969199913936], [-1.7333507460988553, -1.1400482422571545, -4.6017950241425067, 1.0300125286472035]]], [[[-3.0659746747983418, 3.4989669224003901, -1.0507461565034326, -3.7038317807458254], [-1.2869029047441338, -0.89469225970853739, 4.1405040965090674, -0.94718236382694876], [1.4772349468377124, 4.578751221081383, 1.4109854462924041, 4.0618672614760989]], [[-2.9359066927930666, -2.0491850439033565, -2.0852899693679872, 3.1594313265598242], [1.9205641925346413, 0.6584080965725061, -2.7621978826946325, 1.2129580095349741], [-1.1820958426193364, 0.049217224264106463, 4.1826407196243327, 3.8330108984719597]]], [[[0.78472962467066765, 0.53542958900314019, -2.2076587650436954, -4.7520601953064725], [-3.0657725623014009, -2.0124762735748005, -4.8714470422287368, 2.7031042190986021], [-3.258938383196984, 3.185967368791502, -4.0481100832481687, -4.2030532605314983]], [[-4.2884201699701707, 2.3351805122475122, 2.6087190850434947, 4.599365625378887], [0.020766454726183881, 1.1359029456929504, -4.7009987865137512, -4.9794648846702128], [4.6822913249284639, -3.3116932851939893, 4.7317567030701984, 2.6254308810497475]]]])+(1.-msk_arg1)*numpy.array([[[[-1.3137937672133395, -3.2324297199922238, -4.4471528972116934, 4.5341764966635143], [-0.18754144217855462, 4.5591600668637646, -4.3014023812432995, -0.98879941636628832], [-3.3993859166083009, 1.2633488831698294, 3.6579652717762094, -0.68480213827680991]], [[-2.4568311118250366, 2.9207816161706379, 1.161356465287648, 3.5338437496319255], [1.0501351261863778, -2.8633448814279694, -3.2215357471569717, 2.8462985569607007], [-4.277275926983549, -0.81725857082627762, 1.8744790794634181, -1.1140725006434216]]], [[[1.8189647232816455, -2.9847698247307064, 4.6173104884199176, 0.7317013739431335], [-0.48711006594614226, -2.7607059017846836, 0.90391469350081444, -1.2570788617925821], [-4.8349601646450413, -3.8221826283572922, -2.9386399840069579, 1.1719042725156505]], [[-2.7744162008348958, 4.0110398745871532, -2.7008906644472495, 0.76092781345011407], [4.6213311965499528, -2.3918389804425253, -4.6282061843973841, 1.6592633889294532], [-0.18043959258599696, -2.2509001761785195, 0.95235379931659914, -2.0142097235351084]]], [[[-1.9056591515355583, 3.0516395110536081, 3.1217461614967199, -2.1175028587404423], [-2.305285182928726, 0.059210007683994981, -2.9351869183181845, 4.046020449147651], [-2.0088796896123293, 3.3591822589807254, -3.6942653889874331, 1.5385671989961045]], [[2.628324515126927, 4.7873650889026305, 0.61612930036002744, -1.4650827757393223], [4.1659955089672067, 1.2828055319888652, -4.0252742250597677, 4.1898126797223689], [3.5162503415337003, 0.24173200359571823, -3.0214765856999737, 3.5623920797706852]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.4123710470012023, -0.78630800124740929, 1.9619152225874181, 4.2282174993481512], [3.6309401334501192, 2.5356070119547347, 1.3988649687468468, -3.0399747838074855], [2.1316860752893447, -0.24525777104265511, 4.2182540474000643, -3.1015168884295043]], [[2.9453144245962317, -3.0495736364261785, 2.4011316504075815, 0.62378582534767801], [3.8840031288231458, 2.8676580779299732, 2.0761627628137758, -4.3786285390452715], [-1.3173823651527332, -0.72407986131103241, -4.1858266431963846, 1.4459809095933256]]], [[[-2.6500062938522198, 3.9149353033465122, -0.63477777555731052, -3.2878633997997033], [-0.87093452379801173, -0.47872387876241529, 4.5564724774551895, -0.53121398288082666], [1.8932033277838345, 4.9947196020275051, 1.8269538272385262, 4.477835642422221]], [[-2.5199383118469445, -1.6332166629572344, -1.6693215884218651, 3.5753997075059463], [2.3365325734807634, 1.0743764775186282, -2.3462295017485104, 1.6289263904810962], [-0.76612746167321433, 0.46518560521022856, 4.5986091005704548, 4.2489792794180818]]], [[[1.2006980056167897, 0.95139796994926229, -1.7916903840975733, -4.3360918143603504], [-2.6498041813552788, -1.5965078926286784, -4.4554786612826147, 3.1190726000447242], [-2.8429700022508619, 3.6019357497376241, -3.6321417023020466, -3.7870848795853762]], [[-3.8724517890240486, 2.7511488931936343, 3.0246874659896168, 5.0153340063250091], [0.43673483567230598, 1.5518713266390725, -4.2850304055676292, -4.5634965037240907], [5.098259705874586, -2.8957249042478672, 5.1477250840163205, 3.0413992619958696]]]])+(1.-msk_ref)*numpy.array([[[[-0.89782538626721742, -2.8164613390461017, -4.0311845162655713, 4.9501448776096364], [0.22842693876756748, 4.9751284478098867, -3.8854340002971774, -0.57283103542016622], [-2.9834175356621788, 1.6793172641159515, 4.0739336527223315, -0.26883375733068782]], [[-2.0408627308789145, 3.33674999711676, 1.5773248462337701, 3.9498121305780476], [1.4661035071324999, -2.4473765004818473, -2.8055673662108496, 3.2622669379068228], [-3.8613075460374269, -0.40129018988015552, 2.2904474604095402, -0.69810411969729946]]], [[[2.2349331042277676, -2.5688014437845843, 5.0332788693660397, 1.1476697548892556], [-0.071141685000020161, -2.3447375208385615, 1.3198830744469365, -0.84111048084646001], [-4.4189917836989192, -3.4062142474111701, -2.5226716030608358, 1.5878726534617726]], [[-2.3584478198887737, 4.4270082555332753, -2.2849222835011274, 1.1768961943962362], [5.0372995774960749, -1.9758705994964032, -4.212237803451262, 2.0752317698755753], [0.23552878836012514, -1.8349317952323974, 1.3683221802627212, -1.5982413425889863]]], [[[-1.4896907705894362, 3.4676078919997302, 3.537714542442842, -1.7015344777943202], [-1.8893168019826039, 0.47517838863011708, -2.5192185373720624, 4.4619888300937731], [-1.5929113086662072, 3.7751506399268475, -3.278297008041311, 1.9545355799422266]], [[3.0442928960730491, 5.2033334698487526, 1.0320976813061495, -1.0491143947932002], [4.5819638899133288, 1.6987739129349873, -3.6093058441136456, 4.605781060668491], [3.9322187224798224, 0.65770038454184032, -2.6055082047538516, 3.9783604607168073]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([0.84291169795825294, -4.43886052191076]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.39506944581459447, -2.5328861481719587])+(1.-msk_arg1)*numpy.array([-1.8223930281899623, -4.1615944648247538])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.2379811437728474, -6.9717466700827186])+(1.-msk_ref)*numpy.array([-0.97948133023170936, -8.6004549867355138])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[-1.6987213243826216, -4.9524820723633383], [1.9893144857688485, 2.5940532524890303]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.4051017881673857, -2.1982345723526207])+(1.-msk_arg1)*numpy.array([4.7932974309830918, 3.8198296520449055])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.7063804637847642, -1.5473802841959525], [-0.20892008658377215, 0.39581868013640964]])+(1.-msk_ref)*numpy.array([[3.0945761066004702, -0.15918464138024646], [5.809144137813754, 6.4138829045339358]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_expandedData_rank2(self):
      arg0=Data(numpy.array([4.962257964796553, 1.8211444989744496]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.1594481475735243, -0.76599838133660114], [-3.7516120261471295, -2.0819476755349631]])+(1.-msk_arg1)*numpy.array([[1.9888409152530615, 2.9111132485764113], [3.9184272791769494, -4.5265147600015823]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.1217061123700773, 4.1962595834599519], [-1.9304675271726799, -0.26080317656051344]])+(1.-msk_ref)*numpy.array([[6.9510988800496145, 7.8733712133729643], [5.7395717781513991, -2.7053702610271326]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_expandedData_rank1(self):
      arg0=Data(numpy.array([[[2.2248625691681854, 2.6877904582958392, 2.5532324552806287, 4.2614710362223125, -0.89130093345420658], [3.2394361162783465, 0.34661650658164866, -0.62647144597096993, 0.73635454552851876, -4.7878393843448697], [-4.4708876296400115, 2.1767301103807259, 1.9636795357681569, 3.0834871855956116, -2.2997094709207655], [3.6716585306609257, 0.65180282842876025, -3.3980909865503417, -3.7179049383459875, 1.4721365984601897]], [[-3.1203865840371114, -1.5406493347618753, 4.4743678297491218, 4.6997237442676916, -0.12171037513669969], [0.52485938641883934, 1.2179068165473241, -2.883451214910985, -0.4840224530052728, -1.616375461637547], [-0.58420498980954427, -0.018529987737986531, -4.7415278174532443, 0.43385392866445738, 2.9038039778527374], [-1.1197693042165913, -1.5699430596887289, -2.9769596310539779, 4.7703123480346257, -4.8217082003296952]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.9378130968959173, -2.0607483634174573])+(1.-msk_arg1)*numpy.array([1.2944351940602994, -3.5158921964958978])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.7129505277277319, -2.2500226386000781, -2.3845806416152886, -0.67634206067360481, -5.8291140303501239], [-1.6983769806175708, -4.5911965903142686, -5.5642845428668872, -4.2014585513673985, -9.7256524812407861], [-9.4087007265359297, -2.7610829865151914, -2.9741335611277604, -1.8543259113003057, -7.2375225678166828], [-1.2661545662349916, -4.286010268467157, -8.335904083446259, -8.6557180352419039, -3.4656764984357276]], [[-5.1811349474545683, -3.6013976981793325, 2.4136194663316646, 2.6389753808502343, -2.182458738554157], [-1.5358889769986179, -0.84284154687013313, -4.9441995783284423, -2.5447708164227301, -3.6771238250550042], [-2.6449533532270015, -2.0792783511554438, -6.8022761808707015, -1.6268944347529999, 0.8430556144352801], [-3.1805176676340485, -3.6306914231061862, -5.0377079944714351, 2.7095639846171684, -6.8824565637471524]]])+(1.-msk_ref)*numpy.array([[[3.5192977632284848, 3.9822256523561386, 3.8476676493409281, 5.5559062302826119, 0.40313426060609281], [4.5338713103386459, 1.6410517006419481, 0.66796374808932946, 2.0307897395888181, -3.4934041902845703], [-3.1764524355797121, 3.4711653044410253, 3.2581147298284563, 4.377922379655911, -1.0052742768604661], [4.9660937247212251, 1.9462380224890596, -2.1036557924900423, -2.4234697442856881, 2.7665717925204891]], [[-6.6362787805330097, -5.0565415312577731, 0.95847563325322405, 1.1838315477717938, -3.6376025716325975], [-2.9910328100770585, -2.2979853799485737, -6.3993434114068828, -3.9999146495011706, -5.1322676581334452], [-4.1000971863054421, -3.5344221842338843, -8.257420013949142, -3.0820382678314404, -0.61208821864316043], [-4.6356615007124891, -5.0858352561846267, -6.4928518275498757, 1.2544201515387279, -8.337600396825593]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_expandedData_rank3(self):
      arg0=Data(numpy.array([-2.1935945602208395, 1.7147700303254094]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.2024475785390218, 2.5610222647149801, 2.3271028126393407, 2.0308415504838422, 3.8441817997061829], [-2.5061610996974046, -1.9020698204253295, 1.9174330777662618, -1.9420580898737407, -0.86052190569161802], [-4.3840328286826269, 1.6471758826374838, 3.1004282436159656, -4.4437946584435748, 2.596389227325087], [3.1942739720033249, -0.65654114237955596, 4.9826562208871863, -4.8809501002191524, 1.1346566038270254]], [[-0.56394090498475258, 4.3313845139702458, -0.52683720719014815, -3.8066937070092131, -1.0707749785726639], [-0.74642402535781827, 1.3569230252694116, 4.7829823362765431, 3.8826385013740623, -0.52752280070457847], [-2.3404340855398251, -1.2857942520984169, 2.1176657703384203, 4.623589458269544, -2.4281030265489854], [0.71386209088054819, -1.0908021391635137, -0.57779696344688602, -2.8303705237693979, 1.3138212770791382]]])+(1.-msk_arg1)*numpy.array([[[0.42525515061633268, 2.1526997460704331, -3.6765876038335419, 0.42161875114507286, -1.6244767212086595], [-1.5998466145488432, -0.090659335928614304, -3.0631481553074669, 4.2800765705908042, 2.9943852430138662], [-2.2629714183040237, -4.5796764605250431, 2.9308687826707933, 3.2691686909169508, -4.8081984182165316], [1.0807873947574889, -1.1689545216093133, 2.381918387949538, -0.80628167725222966, -4.0415194757964787]], [[-4.418129982513844, 3.0877841641543906, -2.2596726003005241, -0.39544320928664867, -3.9271357355754799], [-4.4982180120349096, 3.1570857294978971, -2.8071213764534231, -4.8360438993728199, -4.4587650888834158], [-3.6159151752016361, 1.4289622120286447, -0.18357173815825245, 2.6201257621426608, -1.985583462334668], [0.075751773046511595, -4.9394807964852436, -3.6799881196344177, -3.6014060748953867, -3.7452699067595443]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-4.3960421387598618, 0.36742770449414053, 0.13350825241850117, -0.16275300973699736, 1.6505872394853434], [-4.6997556599182442, -4.095664380646169, -0.27616148245457772, -4.1356526500945803, -3.0541164659124576], [-6.5776273889034664, -0.54641867758335572, 0.90683368339512604, -6.6373892186644143, 0.40279466710424749], [1.0006794117824853, -2.8501357026003955, 2.7890616606663468, -7.074544660439992, -1.0589379563938142]], [[1.1508291253406568, 6.0461545442956552, 1.1879328231352613, -2.0919236766838036, 0.64399505175274552], [0.96834600496759116, 3.071693055594821, 6.4977523666019525, 5.5974085316994717, 1.187247229620831], [-0.6256640552144157, 0.42897577822699251, 3.8324358006638297, 6.3383594885949535, -0.71333299622357593], [2.4286321212059576, 0.62396789116189577, 1.1369730668785234, -1.1156004934439885, 3.0285913074045476]]])+(1.-msk_ref)*numpy.array([[[-1.7683394096045069, -0.040894814150406411, -5.8701821640543814, -1.7719758090757667, -3.818071281429499], [-3.7934411747696828, -2.2842538961494538, -5.256742715528306, 2.0864820103699646, 0.80079068279302668], [-4.4565659785248632, -6.7732710207458826, 0.73727422244995378, 1.0755741306961113, -7.0017929784373711], [-1.1128071654633507, -3.3625490818301529, 0.18832382772869849, -2.9998762374730692, -6.2351140360173183]], [[-2.7033599521884346, 4.8025541944798, -0.54490256997511466, 1.3193268210387608, -2.2123657052500705], [-2.7834479817095001, 4.8718557598233065, -1.0923513461280137, -3.1212738690474104, -2.7439950585580064], [-1.9011451448762267, 3.1437322423540541, 1.531198292167157, 4.3348957924680702, -0.27081343200925856], [1.790521803371921, -3.2247107661598342, -1.9652180893090083, -1.8866360445699772, -2.0304998764341349]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_expandedData_rank1(self):
      arg0=Data(numpy.array([[[[-2.0937165342865893, -0.33111430784524476], [-3.2729057335377556, -2.3944667335344305]], [[4.290394032135195, 3.362534346913737], [0.89317014280877061, -1.7135019951945187]], [[2.3408143470276821, -2.1813328496229047], [0.62635895108586226, -0.70579122497101299]], [[-3.6263089431982989, -3.7319859006483824], [0.95470253863147114, 4.5787412602859838]], [[-1.5944289739315529, 4.326568728402254], [1.4467586973773496, -4.0693043482991067]], [[-3.5856078971487082, 0.15921194784014325], [1.3059560626152331, 0.86038155483471623]]], [[[2.4486811529884296, 2.0418102630794079], [-4.4181043737972399, 4.2717968817782186]], [[1.0013467499913178, 0.50031365674488271], [-1.7497703046173485, 2.9788283554622863]], [[1.3964233524270657, -2.5675319638469918], [3.8396926199312915, 2.9992405158812439]], [[-0.68837010468136128, -0.7617999611827333], [4.1599915367826537, 4.4056171519334235]], [[2.7835909141195589, 4.8900774529919353], [2.790250208568084, -2.5236870318812201]], [[4.2326099877422667, 2.265859690507992], [1.7732266606085361, -1.2190739660426586]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.8486151259440673, 3.3718602448727779])+(1.-msk_arg1)*numpy.array([-2.1429011085511274, -0.88583017745675097])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.75489859165747797, 2.5175008180988225], [-0.42429060759368831, 0.45414839240963678]], [[7.1390091580792623, 6.2111494728578043], [3.7417852687528379, 1.1351131307495486]], [[5.1894294729717494, 0.66728227632116255], [3.4749740770299296, 2.1428239009730543]], [[-0.77769381725423159, -0.88337077470431513], [3.8033176645755384, 7.4273563862300511]], [[1.2541861520125144, 7.1751838543463213], [4.2953738233214169, -1.2206892223550394]], [[-0.73699277120464091, 3.0078270737842105], [4.1545711885593004, 3.7089966807787835]]], [[[5.8205413978612075, 5.4136705079521859], [-1.0462441289244619, 7.6436571266509965]], [[4.3732069948640957, 3.8721739016176606], [1.6220899402554294, 6.3506886003350642]], [[4.7682835972998436, 0.80432828102578613], [7.2115528648040694, 6.3711007607540218]], [[2.6834901401914166, 2.6100602836900446], [7.5318517816554316, 7.7774773968062014]], [[6.1554511589923369, 8.2619376978647132], [6.1621104534408619, 0.84817321299155779]], [[7.6044702326150446, 5.6377199353807699], [5.145086905481314, 2.1527862788301193]]]])+(1.-msk_ref)*numpy.array([[[[-4.2366176428377162, -2.4740154163963721], [-5.415806842088883, -4.5373678420855583]], [[2.1474929235840676, 1.2196332383626096], [-1.2497309657423568, -3.8564031037456461]], [[0.19791323847655473, -4.3242339581740321], [-1.5165421574652651, -2.8486923335221404]], [[-5.7692100517494262, -5.8748870091995098], [-1.1881985699196562, 2.4358401517348565]], [[-3.7373300824826803, 2.1836676198511267], [-0.69614241117377773, -6.2122054568502341]], [[-5.728509005699836, -1.9836891607109841], [-0.83694504593589425, -1.2825195537164111]]], [[[1.5628509755316786, 1.155980085622657], [-5.3039345512539908, 3.3859667043214676]], [[0.11551657253456682, -0.38551652071186826], [-2.6356004820740995, 2.0929981780055353]], [[0.51059317497031476, -3.4533621413037427], [2.9538624424745406, 2.1134103384244929]], [[-1.5742002821381123, -1.6476301386394843], [3.2741613593259027, 3.5197869744766725]], [[1.897760736662808, 4.0042472755351843], [1.9044200311113331, -3.4095172093379711]], [[3.3467798102855157, 1.380029513051241], [0.8873964831517851, -2.1049041434994096]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_expandedData_rank4(self):
      arg0=Data(numpy.array([3.5475410081497589, 1.730414418282848]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-2.0721075157890692, -4.0117248464296367], [-1.8954974523383408, 1.8885008056348855]], [[2.5766517990646758, -1.1410118499720987], [1.3933562462994118, -1.3541074214516335]], [[1.7654421727919516, 1.9099543529001757], [4.0311922822647457, -0.69623418218951905]], [[4.0578970644278982, -0.42363761257665011], [-2.2923270746673419, -4.5031789771736639]], [[3.550564350098055, 0.11910378085073692], [-3.4913406825393931, 2.9523804318336957]], [[-0.30051171273249189, 0.17743769132861686], [-2.414725365254331, -0.36801705537685869]]], [[[-4.4097829805502675, -3.8624299056883604], [-1.8790831365171536, 4.456876055629909]], [[0.40518175201934525, -0.058100792313963368], [4.2560701213859407, 2.6281815528278551]], [[4.4033686863869548, -2.9726324936983541], [3.5364063589998338, -2.4422498464932429]], [[4.0782591884424555, 2.8586186361004904], [4.5909379781073518, 0.94073183807237548]], [[3.6350241060966244, -1.0498402850976882], [4.9417356502914256, 3.5710047250219858]], [[-2.929231014907745, -2.8711479246064808], [3.4776019663301856, -4.3386628036185861]]]])+(1.-msk_arg1)*numpy.array([[[[3.063064525464366, 1.6774678918917028], [2.8468917416264024, 3.8113549341821447]], [[1.3161934867406364, -4.0379294109835397], [-2.5333593480857672, 0.22938355842376268]], [[-2.6959577778784025, 2.1479189090871103], [4.434580979102023, 0.80150088525819108]], [[0.43586748353274096, -0.28502660884471975], [4.1827752149693858, 0.64924348864483417]], [[2.6054531191529096, 3.2224786723469485], [-0.11478320770106887, 4.1108524602674503]], [[4.7109258102884262, -4.1848108224211691], [0.46152675969147161, 2.8742897634042084]]], [[[-1.450247598787584, -4.0515262795398543], [-1.8738754800846911, 1.3054770817450931]], [[-2.058422762576285, -0.10008053954813256], [0.56881461279261458, 1.6063712260871155]], [[3.1126521726395033, 0.63457379493545041], [-0.10915175736730642, -3.3879870057429882]], [[-2.2859852890217658, 1.8564061554028974], [0.79523955776231947, 2.0372328827321589]], [[4.456114288168866, 2.1089459820534469], [-4.792830770973957, -3.9951825149010931]], [[0.8232362576331127, 0.14903473512062337], [-4.3825341853884625, -2.8862044366588044]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.4754334923606898, -0.46418383827987775], [1.6520435558114182, 5.4360418137846445]], [[6.1241928072144347, 2.4065291581776602], [4.9408972544491707, 2.1934335866981254]], [[5.3129831809417105, 5.4574953610499346], [7.5787332904145046, 2.8513068259602399]], [[7.6054380725776571, 3.1239033955731088], [1.2552139334824171, -0.95563796902390497]], [[7.098105358247814, 3.6666447890004958], [0.056200325610365809, 6.4999214399834546]], [[3.247029295417267, 3.7249786994783758], [1.1328156428954279, 3.1795239527729002]]], [[[-2.6793685622674195, -2.1320154874055124], [-0.14866871823430561, 6.187290473912757]], [[2.1355961703021933, 1.6723136259688847], [5.9864845396687887, 4.3585959711107032]], [[6.1337831046698028, -1.2422180754155061], [5.2668207772826818, -0.71183542821039492]], [[5.8086736067253035, 4.5890330543833384], [6.3213523963901999, 2.6711462563552235]], [[5.3654385243794724, 0.6805741331851598], [6.6721500685742736, 5.3014191433048339]], [[-1.1988165966248969, -1.1407335063236328], [5.2080163846130336, -2.6082483853357381]]]])+(1.-msk_ref)*numpy.array([[[[6.610605533614125, 5.2250089000414617], [6.3944327497761613, 7.3588959423319036]], [[4.8637344948903953, -0.49038840283378082], [1.0141816600639917, 3.7769245665735216]], [[0.85158323027135641, 5.6954599172368692], [7.9821219872517819, 4.34904189340795]], [[3.9834084916824999, 3.2625143993050392], [7.7303162231191447, 4.1967844967945931]], [[6.1529941273026685, 6.7700196804967074], [3.4327578004486901, 7.6583934684172092]], [[8.2584668184381851, -0.63726981427141016], [4.0090677678412305, 6.4218307715539673]]], [[[0.28016681949526401, -2.3211118612570063], [-0.14346106180184304, 3.0358915000279412]], [[-0.32800834429343695, 1.6303338787347155], [2.2992290310754626, 3.3367856443699635]], [[4.8430665909223514, 2.3649882132182984], [1.6212626609155416, -1.6575725874601401]], [[-0.55557087073891775, 3.5868205736857455], [2.5256539760451675, 3.7676473010150069]], [[6.186528706451714, 3.8393604003362949], [-3.062416352691109, -2.2647680966182451]], [[2.5536506759159607, 1.8794491534034714], [-2.6521197671056145, -1.1557900183759564]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[4.1329057756774574, 4.0701583553914471, -1.6551918053535433, 3.5452278195057865, -0.81711703873437713], [0.43287624072826958, -2.224161939263638, 3.318494240278083, 2.9277637469060753, 2.0309132572042898], [-3.2194917465092923, 4.885155409643513, 3.8404971044297032, 3.2494234893725977, 0.92177459452624877], [-2.6891802447121291, -0.019611528316614724, 4.0610318831816805, 3.6417336913883798, -1.4599648619768413]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[1.5865765625632289, -0.40420440699923876, -0.74752541155132857, 0.49922579954561375, 1.8245279904561995], [4.6780371104690861, 4.050276637073571, 1.2066331660452327, 0.86561495119116483, -3.2975780361736975], [-2.9716682541278781, -0.66596325450460814, 3.4540699844781528, -0.88159348810738081, -4.5111687055412961], [-2.0728768436175371, -0.21180958595824606, 4.734184852158851, -4.6868091610061118, -3.1456233536711462]])+(1.-msk_arg1)*numpy.array([[-1.1785559519189492, 4.8985426939361592, -0.27508538418688389, 0.76096531801282197, 3.7263510116395455], [3.701737985231242, -0.66105774307792586, -2.6103252726723367, -1.8818964288267761, 4.3603335483809538], [0.11891487600750406, 4.8657322774255825, 3.0041213612182656, 0.011742090264823091, -3.5959686156877071], [2.1889080393294442, -4.3281723811195469, -1.1018364496368314, 1.961425633823036, -4.0620205444827837]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.7194823382406863, 3.6659539483922083, -2.4027172169048718, 4.0444536190514002, 1.0074109517218224], [5.1109133511973557, 1.826114697809933, 4.5251274063233158, 3.7933786980972402, -1.2666647789694077], [-6.1911600006371703, 4.2191921551389049, 7.294567088907856, 2.3678300012652169, -3.5893941110150474], [-4.7620570883296658, -0.23142111427486078, 8.7952167353405315, -1.045075469617732, -4.6055882156479875]])+(1.-msk_ref)*numpy.array([[2.9543498237585082, 8.9687010493276063, -1.9302771895404272, 4.3061931375186084, 2.9092339729051684], [4.1346142259595116, -2.8852196823415639, 0.70816896760574632, 1.0458673180792992, 6.3912468055852436], [-3.1005768705017882, 9.7508876870690955, 6.8446184656479687, 3.2611655796374208, -2.6741940211614583], [-0.50027220538268491, -4.3477839094361617, 2.9591954335448492, 5.6031593252114158, -5.521985406459625]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_expandedData_rank2(self):
      arg0=Data(numpy.array([[[3.5960456065370465, -2.1532015554469783], [-1.3273699161108263, -1.6973774528495467], [-1.5719201662304161, 3.7108233010333418], [-1.7923550371451569, 4.9658889932869936], [-2.501144420104561, -4.7188648552391275]], [[3.5068046587622916, -4.4548993092593623], [0.84507176905898973, 0.01779194397316175], [-2.3758099998905724, -4.7621746519946306], [4.7474371366552877, -0.7335124213437938], [-4.9358404302765848, -4.0880653643148648]], [[-4.931350533368918, 0.11222121306730504], [-0.50947672648910203, -3.5118930064644394], [-0.46138421033869559, 3.6519205808066371], [-2.5656362399086308, -3.9164220131378391], [0.53686765492575006, 4.5962282453226919]], [[-2.1367408398185939, 3.9322438086638698], [0.38732752910637913, 2.2708724050331259], [-4.1488051078539279, 3.3286644235872718], [-3.3740239790477222, -1.5691786638703329], [-3.0735110533763654, -4.3298194759067989]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.30981890868200335, -2.0144861567728398, -1.6443371453205726, -4.2788805133482093, -0.68400054356648887], [4.6602223119530137, 2.8313118003533786, 1.6517600525170586, 2.1624976467833079, 3.3836703182393126], [-1.6445492321184654, -4.3702685364459093, 3.2152473286879975, 1.9123432100965729, 1.6139810757505844], [1.1481683133793554, 3.5118836950194456, 3.5582081306965581, 1.869087639927228, -2.6933526051251455]])+(1.-msk_arg1)*numpy.array([[4.7484768087884337, 0.98805894359706414, -1.5432184282524961, -4.7752519640729698, 1.3775461772947661], [3.2319582412102257, -0.27629192352770104, 2.8088195784409091, -2.9252872069734104, -2.6063533062128652], [-0.042019966566845923, 4.288661299180152, -2.3338508537423976, 3.9624041263385763, -1.786380632466452], [3.3685954819366408, 3.0839255532613858, -0.42622886280117811, 3.5217683858671354, 4.2690676075170035]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.9058645152190499, -1.843382646764975], [-3.3418560728836662, -3.7118636096223865], [-3.2162573115509887, 2.0664861557127692], [-6.0712355504933662, 0.68700847993878433], [-3.1851449636710498, -5.4028653988056163]], [[8.1670269707153054, 0.20532300269365145], [3.6763835694123683, 2.8491037443265403], [-0.7240499473735138, -3.110414599477572], [6.9099347834385956, 1.4289852254395141], [-1.5521701120372722, -0.70439504607555214]], [[-6.5758997654873834, -1.5323280190511603], [-4.8797452629350113, -7.8821615429103487], [2.7538631183493019, 6.8671679094946345], [-0.65329302981205783, -2.0040788030412662], [2.1508487306763344, 6.2102093210732763]], [[-0.98857252643923843, 5.0804121220432252], [3.8992112241258248, 5.7827561000525716], [-0.59059697715736981, 6.8868725542838298], [-1.5049363391204942, 0.29990897605689515], [-5.7668636585015109, -7.0231720810319445]]])+(1.-msk_ref)*numpy.array([[[8.3445224153254802, 2.5952752533414554], [-0.33931097251376219, -0.70931850925248252], [-3.1151385944829122, 2.1676048727808457], [-6.5676070012181267, 0.19063702921402381], [-1.1235982428097948, -3.3413186779443613]], [[6.7387628999725173, -1.2229410680491366], [0.56877984553128869, -0.25849997955453929], [0.43300957855033673, -1.9533550735537215], [1.8221499296818773, -3.6587996283172042], [-7.54219373648945, -6.69441867052773]], [[-4.973370499935764, 0.070201246500459114], [3.77918457269105, 0.77676829271571268], [-2.7952350640810932, 1.3180697270642394], [1.3967678864299455, 0.045982113200737196], [-1.2495129775407019, 2.80984761285624]], [[1.231854642118047, 7.3008392906005106], [3.471253082367765, 5.3547979582945118], [-4.575033970655106, 2.9024355607860937], [0.14774440681941314, 1.9525897219968025], [1.1955565541406381, -0.060751868389795405]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_expandedData_rank3(self):
      arg0=Data(numpy.array([[1.3349662800906543, -0.39326994699494389, 2.8498513460902872, 0.92973603484183709, 1.8943506681593769], [0.83627302799376935, -3.2825121866120455, 2.8993816639634389, 3.4395893755857951, 4.1993289408176686], [0.19018081612045634, -4.612322992130081, -4.2285187446759487, 1.5951104111625565, -2.9010065327157983], [-3.1611595372358678, -2.625882918798478, 1.1804846028459064, -1.0291848735198328, -4.8008044261475789]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.2111341801558746, 3.7858228179626341], [3.5041496459992061, -1.2768473976880945], [1.3628262548053023, -0.5527707488291389], [2.1138205420525233, 3.9794063945017637], [-0.70044666647691223, -3.82876344105473]], [[-3.2288475128131999, 1.767221347801744], [0.81366069351352355, -1.7737750176240619], [-2.5817540956950671, 1.3155179398243355], [1.8139770929230368, 3.6725591623868645], [2.2895602709932694, 3.568305128651776]], [[4.470978880327106, 1.4811893639802989], [-4.4984779617677537, 4.7531108730444043], [-1.5246821634258847, 1.2216121464682317], [3.5154426993810262, -4.6472876883790271], [1.9498776052174964, -3.8168116802778398]], [[-3.1715846727066985, 2.7530386617208222], [-3.0909298077110572, 4.4134226102206711], [-2.2479415972130177, 3.7771625803232318], [-2.1056046891453617, -0.77879429033322722], [1.260816043279311, 2.0560114880346836]]])+(1.-msk_arg1)*numpy.array([[[0.85490727134320643, 3.839051080272295], [0.47804094894286919, 4.5100731315614819], [0.37612272198905927, -1.696284025897282], [-2.0100706771975032, -4.3604371624481351], [-0.017406332153411697, -4.7930674459305074]], [[4.9126684377069338, 0.17708203657585564], [3.8534762107381333, 3.2937277341797575], [-2.634193260027148, -1.0592269495186732], [3.6641126673491655, 0.68699206914377164], [2.8855255412202387, 3.0891742656584249]], [[-0.5782501612025488, 2.5281499757884509], [-1.2847906424709779, -4.978582704676664], [1.7402650530298356, 4.1190589988733866], [4.1150750302457197, -2.8777852179729537], [-4.7742478543646341, -1.6009194349134304]], [[3.9381724249561696, -3.6051648478119835], [-0.3461275465207283, 4.0245136503557273], [-4.4710308940248975, -0.61762733931834646], [-2.6012305624677543, 3.6747769193186137], [1.5455556711790566, -4.9765498952214147]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.8761679000652203, 5.1207890980532884], [3.1108796990042622, -1.6701173446830384], [4.2126776008955895, 2.2970805972611483], [3.0435565768943604, 4.9091424293436008], [1.1939040016824647, -1.9344127728953531]], [[-2.3925744848194306, 2.6034943757955133], [-2.4688514930985219, -5.0562872042361073], [0.3176275682683718, 4.2148996037877744], [5.2535664685088319, 7.1121485379726597], [6.488889211810938, 7.7676340694694446]], [[4.6611596964475623, 1.6713701801007552], [-9.1108009538978347, 0.14078788091432326], [-5.7532009081018334, -3.006906598207717], [5.1105531105435826, -3.0521772772164706], [-0.95112892749830191, -6.7178182129936381]], [[-6.3327442099425664, -0.40812087551504561], [-5.7168127265095352, 1.787539691422193], [-1.0674569943671113, 4.9576471831691382], [-3.1347895626651945, -1.80797916385306], [-3.5399883828682679, -2.7447929381128953]]])+(1.-msk_ref)*numpy.array([[[2.1898735514338608, 5.1740173603629493], [0.084771001947925306, 4.116803184566538], [3.2259740680793465, 1.1535673201930052], [-1.0803346423556661, -3.430701127606298], [1.8769443360059652, -2.8987167777711305]], [[5.7489414657007032, 1.013355064569625], [0.57096402412608782, 0.011215547567712036], [0.26518840393629084, 1.8401547144447656], [7.1037020429349607, 4.1265814447295668], [7.0848544820379074, 7.2885032064760935]], [[-0.38806934508209245, 2.7183307919089073], [-5.8971136346010589, -9.590905696806745], [-2.4882536916461131, -0.10945974580256213], [5.7101854414082762, -1.2826748068103972], [-7.6752543870804324, -4.5019259676292283]], [[0.77701288772030175, -6.7663243850478514], [-2.9720104653192063, 1.3986307315572493], [-3.2905462911789911, 0.56285726352755994], [-3.6304154359875871, 2.6455920457987809], [-3.2552487549685223, -9.7773543213689926]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[0.79347549641551041, 1.5211595989756974, -4.4198616574172087, -4.1824513538790544, -1.2684505285444105], [-0.48336922527371762, 1.2609757753162381, 2.1509492868836482, 0.9829182877979159, -0.50875236407627611], [0.70689426671651123, 1.5291015734365523, 4.6462035377929194, -4.7788150280978501, 1.9838069751063294], [0.82005805698654033, -4.3381846942356548, 0.82751209475797705, -2.0011567430860469, 0.56494159634899965]], [[4.6102210220141497, -1.1914939354749468, -2.0374374322941815, 3.2666969468532212, -0.45293566263705465], [3.7035286381079739, 4.059095985026639, -4.5261424066539524, 4.3903646769871685, -0.57663719680138481], [3.9939857084143284, -3.6983216572033681, -2.8367162827291481, -1.6381351855216888, 3.9324443510173559], [3.7178097106933841, -1.7786627796511514, 2.5925208265122999, -0.73930554563764517, -2.8944218030468027]], [[1.4819943958367681, 2.6886614115544472, -2.3146071063413585, 4.9814821301869827, -4.3697014248653367], [-1.7953428848757289, -2.8632822025533757, 3.8882003857020262, 4.4789889813110992, -2.0075772911941057], [2.1346412979892939, -4.2100481338999529, -4.5382284221448854, 2.1936420572066231, 4.6112684721352579], [-4.2397874859165716, 0.5283558881791981, 3.5275961985936064, -3.8054383309946713, 0.30770345364579832]], [[1.798062041088933, 3.3793133939664415, -0.047875909830970897, -4.6318200095799522, 3.7620024081177341], [2.9450549343914751, 4.2572991742293631, -0.6898379542286257, -1.4197287178303188, -1.3876176744522786], [-4.1482974346556123, -3.8615894362326331, -4.7335392665858684, 0.09935276252952896, -1.5479521137389507], [3.8850353578153012, -1.4104744405505567, 0.20914587736594648, 0.85839330646218315, -0.086097471407086346]], [[4.5923495844645217, 4.3184390604250353, 3.4025013683864831, 2.3043182398864523, -3.9394067864900348], [3.3397343852214316, -1.332000653278663, -2.9538451591986514, -0.3174694660151145, -3.4347297355953499], [-3.2554665687758422, -2.8926389835273691, -4.2219028383995738, 4.8512843270388242, 3.9195805936099433], [0.00067150444002095355, -4.5573173666188165, -4.1164934157887236, 3.5838252166150788, 2.0352919494990118]]], [[[-2.559666047366651, 3.9021908116642798, -1.8661307473130249, 1.8963175882412466, -1.7278712346267309], [2.5223081040725726, -0.65874811258017285, 0.18293855457303998, -3.1822120527662632, -3.8349339851134365], [-2.1933184965442831, 4.7431967293134107, -0.052687520872646942, 0.59193403442002968, 3.1591236530359446], [2.9108871308500088, -2.6941561244941603, 1.3710026836989178, -2.4586824715812048, -1.7144555370479333]], [[0.5324543578881924, -1.244636726161227, 1.9719496829628005, 1.624412446212431, 4.3955747679263659], [-4.9294086894682421, -3.7150467244102092, -0.88204796221499748, -1.1275871509880386, -4.1573336134689685], [-0.25651830461853109, -2.5402871973067462, 4.7495174566946758, -0.87542369350969906, 3.4531274960951137], [-1.0327262850621253, -0.9778300078432629, 0.41760931489642772, 2.0273579794551564, -0.49818633550147506]], [[-2.4977398082593929, 3.279190883245203, -1.1622458892577536, -1.0807006239507078, -0.40659922956876127], [2.4810372799862268, -2.5817589231354332, -0.73980579488813447, 0.43708696126013535, -3.392304273933374], [0.037988913047588113, 3.5340101470340617, 4.2309746780733484, -3.4122318475277433, -0.57298510147382231], [2.5961049466595378, -4.1515970177843649, 3.4727973187294943, 4.770744903120594, 4.0412272123348796]], [[-4.4352780108640077, 1.2077686723508929, 0.10028837659595879, 1.6401667501373876, -2.5020615930719292], [1.259992951734179, -2.3266950077807613, 0.63444207798979235, 3.5968645093659291, 2.5468401481306291], [2.9920949923809905, 1.539756887637572, -2.1038810444866818, 4.6614132038977445, 1.3500967780580257], [3.2416075055157414, 0.74651141275529209, 0.98438094661545783, 0.68859899442760586, -3.4748963243345443]], [[1.8733839631190472, 0.1771698240284465, -1.6190931814396237, 4.4693991422972736, 3.1005743813224118], [1.6964568961122186, 2.2484722477750152, 2.9479174963438739, 2.5316247755090497, 3.5552813355949109], [1.8294141495493923, -1.372278115128144, 0.50599522431729849, 0.50012361483696921, -1.059368802028092], [-4.1156863070471159, -0.0016121316050377033, 4.4657982315033511, 0.040138822230014526, 3.9448638970540255]]], [[[3.1655911956947147, 2.6295535032675952, -0.16169382935809473, 1.6620978081174425, -1.8786985564118694], [2.484118920451496, -3.6165417810678777, -2.9347906327003792, -1.4102049323225341, -0.79525908029617209], [-3.486365929082138, 3.6647256281276341, 1.9210336672363271, 0.70404330580251173, -2.6247081136789663], [-0.88127530370231266, -2.2075717048326093, 3.6477131244952101, -0.62138128801079873, 2.0415402891381076]], [[0.41634846763525424, 2.7867786811799231, 2.5181947796327364, 4.4441300868277871, -2.1235239593414756], [-1.718420514748459, 3.4886663289527018, -3.1427778262660535, 4.8557384711937353, -2.0480611453896502], [2.0794527230180213, 4.8583248493981834, -4.3322626135485365, 0.58903070012960512, -2.3822689487101512], [4.8120122912637164, -1.3601464175403311, 1.0820197374403993, 4.552637348302401, 2.2077341595807463]], [[-2.1831125630523518, -0.61256306488055934, -3.1059698757227161, 2.1942948983891064, -1.1978178244999627], [-4.2711072609603491, -2.3155852045938943, -0.021354001045094329, 3.9278559824819901, -2.8792567455765719], [1.4630215225252563, 2.1288431059241741, 3.6546627609945332, 3.8554294643280755, 0.67723759005693651], [-2.1661562890158992, 4.4189066853704979, 4.9541981250705511, 2.2878275390221248, -2.8751138409709376]], [[-2.4940267860086509, -3.4801431524283233, 0.8478163184139822, -0.68643014321944218, -4.7998047357675757], [0.91135568517433185, -4.3137732368145709, 3.8542350777370764, 0.24842938954420468, -0.87462786319373542], [2.3528257667488965, 2.4387615903747841, 1.9612201881357709, 4.6026580619078263, 0.39757486074580317], [0.12227738108456698, 2.4524168585113149, 1.9997681277525814, 1.0921951637797092, -3.342431166821803]], [[-0.30910246306559053, 0.098805138229934641, -1.788160791743163, -3.0433510132684649, -0.12246127235540616], [-1.2144887882383379, 4.2247186895091158, 0.59572135486643241, 0.19254174873438945, 2.8052881775986291], [-2.4062139007957839, 4.4488343711376803, 0.0071173975337490702, 3.6928074227351022, 0.21168674401426024], [-1.3299282899734144, -3.2009716097901397, -4.0363531285296865, -1.9688078443803505, -3.4293020856547853]]], [[[-4.820971790337742, -2.2547059223014729, -2.6307820886212485, 2.7293807555651384, 0.79361226823488984], [-3.3660678150773227, -0.013967141475101919, -4.0485797959334269, -2.9638928497575403, -2.9501727579983128], [4.4226183421574916, -3.9294880565892134, -3.6479314651158323, 3.3971321580209981, 4.8892167592598152], [-4.6127763427010917, -0.58647685869319321, 2.574220889041273, -2.1523867384628459, -0.21962238283989777]], [[-4.3658614897278305, -2.3412886057383622, -0.89866031726614715, -0.033356015721782484, 3.545609997414056], [-4.9859553114909092, 3.1127874876936588, -3.6669718986758593, 0.15091250216826335, 0.56084194193857151], [3.0932803472617092, -3.5954649472506639, -3.8198581428582514, 1.7990949077357747, -4.9883509081541133], [3.0670045855791894, -2.5242693662918847, 1.7988578459680982, 0.52112746563841483, -0.58859881007912129]], [[-3.3142174710695049, 4.8952706030221211, -0.14104408724726447, 3.4607420238242845, 4.8044107990795446], [-3.4223091344704359, 0.80908973587392019, -3.0283765025471423, -0.072188006261285409, 2.7397174190324627], [4.070635082785838, 1.6112169295721577, 1.3965902793136529, 2.641505420695399, 4.7098115685496218], [3.4635449749298157, -4.9866927378393484, -4.2591000392412433, -4.5714774563105474, 1.7282588185736314]], [[2.4506938786201227, 4.9381615025509227, -2.4238855916826285, 3.4626589320364101, 1.3549858437246067], [1.4556903068583606, 3.0251739768871069, 3.607516118923165, 3.706123157473769, 3.4758845150417148], [-0.54588047340551338, -1.291887403058646, -2.0775877099470152, -2.6978609823438617, -3.2573191321439898], [-2.6205473299984714, -1.7123196869875725, 4.2891080402844199, 0.88106421548358149, 4.2017027837047163]], [[1.3999727723283328, 0.057629832193605068, 2.5100586546083479, 0.24599733322030115, -0.38860884805696916], [-0.31741080562815505, -4.2109084074581311, -4.8260863175306064, -3.5518342308849538, -1.7989034657803913], [-4.7637761077394192, -4.8141998867830251, -4.8055303105411529, -0.87682577364442338, -4.4829109791368342], [2.0387155145506668, -3.5890121700912903, -4.8760649095085036, 4.7711925759856229, -2.1693665502800008]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.6578247835259017, 3.2778805853131843, 1.253091029286467, 1.1558351736939869, 2.5924118286215823], [-3.812309632472819, 1.0737499315984032, -0.78563261583126653, -0.8192398152215139, 2.2816575758546787], [0.50285429802626425, 1.8494594120600727, -1.6451017011871794, 1.9732613010405391, -2.010316534333926], [-4.1288729979902241, -0.62484079769356349, 3.1800885285359914, -4.9210514019485716, -1.4007782827566695]])+(1.-msk_arg1)*numpy.array([[4.7841873573585811, 3.4604311294713792, -0.99155307371202017, 0.34294283591110197, 3.8877614161167813], [-1.6527637007247109, 4.7773527722745417, -1.6931934659987524, -1.3519147902215867, -1.0910498608354544], [4.1043547087746681, 4.1317436254642743, 2.6167075889461922, 0.44789308478869216, -4.1250207560138064], [-1.2276096906985998, -0.79967243543482613, -2.2936217777593129, -4.4813770673792526, -3.8187942640931305]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.4513002799414121, 4.1789843825015991, -1.762036873891307, -1.5246265703531527, 1.3893742549814911], [2.1744555582521841, 3.9188005588421397, 4.8087740704095499, 3.6407430713238176, 2.1490724194496256], [3.3647190502424129, 4.186926356962454, 7.3040283213188211, -2.1209902445719484, 4.6416317586322311], [3.477882840512442, -1.6803599107097531, 3.4853368782838787, 0.65666804043985483, 3.2227663798749013]], [[7.888101607327334, 2.0863866498382375, 1.2404431530190028, 6.5445775321664055, 2.8249449226761296], [6.9814092234211582, 7.3369765703398233, -1.2482618213407681, 7.6682452623003527, 2.7012433885117995], [7.2718662937275127, -0.42044107189018387, 0.44116430258403616, 1.6397453997914955, 7.2103249363305402], [6.9956902960065683, 1.4992178056620329, 5.8704014118254841, 2.5385750396755391, 0.38345878226638153]], [[2.735085425123235, 3.9417524408409141, -1.0615160770548915, 6.2345731594734497, -3.1166103955788698], [-0.54225185558926192, -1.6101911732669087, 5.1412914149884932, 5.7320800105975662, -0.75448626190763868], [3.3877323272757609, -2.9569571046134859, -3.2851373928584184, 3.4467330864930901, 5.8643595014217249], [-2.9866964566301046, 1.7814469174656651, 4.7806872278800734, -2.5523473017082043, 1.5607944829322653]], [[2.9538972147829199, 4.5351485676604284, 1.107959263863016, -3.4759848358859653, 4.917837581811721], [4.1008901080854621, 5.41313434792335, 0.46599721946536121, -0.26389354413633193, -0.23178250075829165], [-2.9924622609616254, -2.7057542625386461, -3.5777040928918815, 1.2551879362235159, -0.39211694004496378], [5.0408705315092881, -0.25463926685656979, 1.3649810510599334, 2.0142284801561701, 1.0697377022869006]], [[7.1847614130861039, 6.9108508890466176, 5.9949131970080654, 4.8967300685080346, -1.3469949578684526], [5.9321462138430139, 1.2604111753429192, -0.36143333057706917, 2.2749423626064678, -0.84231790697376763], [-0.66305474015425991, -0.30022715490578689, -1.6294910097779916, 7.4436961556604064, 6.5119924222315255], [2.5930833330616032, -1.9649055379972342, -1.5240815871671414, 6.1762370452366611, 4.6277037781205941]]], [[[-6.37197567983947, 0.08988117919146088, -5.6784403797858438, -1.9159920442315723, -5.5401808670995498], [-1.2900015284002464, -4.4710577450529918, -3.629371077899779, -6.9945216852390821, -7.6472436175862555], [-6.0056281290171025, 0.93088709684059179, -3.8649971533454659, -3.2203755980527893, -0.6531859794368744], [-0.90142250162281012, -6.5064657569669793, -2.4413069487739012, -6.2709921040540237, -5.5267651695207523]], [[1.6062042894865955, -0.17088679456282385, 3.0456996145612036, 2.6981623778108341, 5.4693246995247691], [-3.8556587578698389, -2.6412967928118061, 0.19170196938340567, -0.053837219389635482, -3.0835836818705653], [0.81723162697987206, -1.4665372657083431, 5.823267388293079, 0.19832623808870409, 4.5268774276935169], [0.04102364653627788, 0.095919923755140246, 1.4913592464948309, 3.1011079110535595, 0.57556359609692809]], [[-3.2833724240906594, 2.4935582674139365, -1.9478785050890202, -1.8663332397819743, -1.1922318454000278], [1.6954046641549603, -3.3673915389666997, -1.525438410719401, -0.34854565457113118, -4.1779368897646405], [-0.74764370278367842, 2.7483775312027952, 3.4453420622420818, -4.1978644633590099, -1.3586177173050888], [1.8104723308282713, -4.9372296336156314, 2.6871647028982277, 3.9851122872893274, 3.255594596503613]], [[-5.2545178260855216, 0.38852885712937901, -0.71895143862555511, 0.82092693491587365, -3.3213014082934431], [0.44075313651266512, -3.1459348230022752, -0.18479773723172155, 2.7776246941444152, 1.7276003329091152], [2.1728551771594766, 0.72051707241605811, -2.9231208597081957, 3.8421733886762306, 0.53085696283651185], [2.4223676902942275, -0.072728402466221809, 0.16514113139394393, -0.13064082079390804, -4.2941361395560582]], [[4.1550415389737259, 2.4588273998831252, 0.66256439441505499, 6.7510567181519523, 5.3822319571770905], [3.9781144719668973, 4.5301298236296939, 5.2295750721985526, 4.8132823513637284, 5.8369389114495895], [4.111071725404071, 0.90937946072653464, 2.7876528001719771, 2.7817811906916479, 1.2222887738265866], [-1.8340287311924373, 2.280045444249641, 6.7474558073580297, 2.3217963980846932, 6.2265214729087042]]], [[[3.6684454937209789, 3.1324078012938594, 0.34116046866816951, 2.1649521061437067, -1.3758442583856052], [2.9869732184777602, -3.1136874830416135, -2.431936334674115, -0.90735063429626983, -0.29240478226990785], [-2.9835116310558738, 4.1675799261538984, 2.4238879652625913, 1.206897603828776, -2.121853815652702], [-0.37842100567604842, -1.7047174068063451, 4.1505674225214744, -0.11852698998453448, 2.5443945871643718]], [[2.2658078796953269, 4.6362380932399958, 4.3676541916928091, 6.2935894988878598, -0.27406454728140295], [0.13103889731161367, 5.3381257410127745, -1.2933184142059808, 6.7051978832538079, -0.19860173332957753], [3.928912135078094, 6.7077842614582561, -2.4828032014884638, 2.4384901121896778, -0.53280953665007846], [6.6614717033237891, 0.48931299451974164, 2.9314791495004719, 6.4020967603624737, 4.057193571640819]], [[-3.8282142642395312, -2.2576647660677387, -4.7510715769098955, 0.54919319720192705, -2.842919525687142], [-5.9162089621475289, -3.9606869057810736, -1.6664557022322737, 2.2827542812948107, -4.5243584467637508], [-0.18208017866192305, 0.48374140473699478, 2.0095610598073539, 2.2103277631408962, -0.96786411113024284], [-3.8112579902030785, 2.7738049841833186, 3.3090964238833718, 0.64272583783494541, -4.5202155421581169]], [[-0.52076548496811181, -1.5068818513877842, 2.8210776194545213, 1.2868311578210969, -2.8265434347270366], [2.8846169862148709, -2.3405119357740318, 5.8274963787776155, 2.2216906905847438, 1.0986334378468037], [4.3260870677894356, 4.4120228914153232, 3.93448148917631, 6.5759193629483654, 2.3708361617863423], [2.0955386821251061, 4.425678159551854, 3.9730294287931205, 3.0654564648202483, -1.3691698657812639]], [[-2.3194189973995165, -1.9115113961039913, -3.798477326077089, -5.0536675476023909, -2.1327778066893321], [-3.2248053225722639, 2.2144021551751898, -1.4145951794674936, -1.8177747855995365, 0.79497164326470315], [-4.4165304351297099, 2.4385178368037543, -2.0031991368001769, 1.6824908884011762, -1.7986297903196657], [-3.3402448243073404, -5.2112881441240653, -6.0466696628636125, -3.9791243787142765, -5.4396186199887113]]], [[[-8.9498447883279653, -6.383578920291697, -6.7596550866114722, -1.3994922424250857, -3.3352607297553343], [-7.4949408130675472, -4.142840139465326, -8.177452793923651, -7.0927658477477644, -7.0790457559885365], [0.2937453441672675, -8.0583610545794375, -7.7768044631060569, -0.73174083996922601, 0.76034376126959113], [-8.7416493406913158, -4.7153498566834173, -1.5546521089489511, -6.2812597364530696, -4.3484953808301219]], [[-4.990702287421394, -2.9661294034319257, -1.5235011149597106, -0.65819681341534597, 2.9207691997204925], [-5.6107961091844727, 2.4879466900000953, -4.2918126963694228, -0.47392829552530014, -0.063998855754991979], [2.4684395495681457, -4.2203057449442269, -4.4446989405518149, 1.1742541100422113, -5.6131917058476768], [2.4421637878856259, -3.1491101639854482, 1.1740170482745347, -0.10371333205514865, -1.2134396077726848]], [[-0.13412894253351348, 8.0753591315581126, 3.039044441288727, 6.640830552360276, 7.984499327615536], [-0.2422206059344445, 3.9891782644099116, 0.15171202598884914, 3.107900522274706, 5.9198059475684541], [7.2507236113218294, 4.7913054581081491, 4.5766788078496443, 5.8215939492313904, 7.8899000970856132], [6.6436335034658072, -1.806604209303357, -1.0790115107052518, -1.3913889277745559, 4.9083473471096228]], [[-2.4703575233284489, 0.017110100602351075, -7.3449369936312001, -1.4583924699121615, -3.5660655582239649], [-3.465361095090211, -1.8958774250614647, -1.3135352830254066, -1.2149282444748026, -1.4451668869068568], [-5.466931875354085, -6.2129388050072176, -6.9986391118955869, -7.6189123842924333, -8.1783705340925614], [-7.541598731947043, -6.6333710889361441, -0.63194336166415166, -4.0399871864649901, -0.71934861824385532]], [[-0.00080551042833665321, -1.3431484505630644, 1.1092803718516784, -1.1547809495363683, -1.7893871308136386], [-1.7181890883848245, -5.6116866902148006, -6.2268646002872758, -4.9526125136416237, -3.1996817485370608], [-6.1645543904960887, -6.2149781695396946, -6.2063085932978224, -2.2776040564010929, -5.8836892618935037], [0.63793723179399731, -4.9897904528479593, -6.2768431922651731, 3.3704142932289534, -3.5701448330366703]]]])+(1.-msk_ref)*numpy.array([[[[5.5776628537740915, 6.3053469563342786, 0.36432569994137243, 0.60173600347952672, 3.5157368288141706], [4.3008181320848635, 6.0451631326748192, 6.9351366442422293, 5.767105645156497, 4.275434993282305], [5.4910816240750924, 6.3132889307951334, 9.4303908951515005, 0.0053723292607310569, 6.7679943324649106], [5.6042454143451215, 0.44600266312292636, 5.6116994521165582, 2.7830306142725343, 5.3491289537075808]], [[8.0706521514855289, 2.2689371939964325, 1.4229936971771977, 6.7271280763246004, 3.0074954668343246], [7.1639597675793532, 7.5195271144980182, -1.0657112771825732, 7.8507958064585477, 2.8837939326699944], [7.4544168378857076, -0.23789052773198893, 0.6237148467422311, 1.8222959439496904, 7.3928754804887351], [7.1782408401647633, 1.6817683498202278, 6.0529519559836791, 2.721125583833734, 0.56600932642457646]], [[0.49044132212474789, 1.697108337842427, -3.3061601800533786, 3.9899290564749625, -5.3612544985773569], [-2.7868959585877491, -3.8548352762653959, 2.896647311990006, 3.4874359075990791, -2.9991303649061258], [1.1430882242772737, -5.2016012076119731, -5.5297814958569056, 1.2020889834946029, 3.6197153984232378], [-5.2313405596285918, -0.46319718553282208, 2.5360431248815862, -4.7969914047066915, -0.68384962006622185]], [[2.1410048770000349, 3.7222562298775435, 0.29506692608013108, -4.2888771736688502, 4.1049452440288361], [3.2879977703025771, 4.6002420101404651, -0.34689511831752373, -1.0767858819192169, -1.0446748385411766], [-3.8053545987445103, -3.5186466003215311, -4.3905964306747665, 0.44229559844063093, -1.2050092778278487], [4.2279781937264032, -1.0675316046394547, 0.55208871327704845, 1.2013361423732851, 0.25684536450401563]], [[8.480111000581303, 8.2062004765418166, 7.2902627845032644, 6.1920796560032336, -0.051645370373253563], [7.2274958013382129, 2.5557607628381183, 0.93391625691812985, 3.5702919501016668, 0.45303168052143139], [0.6322948473409391, 0.99512243258941213, -0.33414142228279253, 8.7390457431556055, 7.8073420097267245], [3.8884329205568022, -0.6695559505020352, -0.22873199967194235, 7.4715866327318601, 5.9230533656157931]]], [[[-4.2124297480913615, 2.2494271109395689, -3.5188944480377358, 0.24355388751653573, -3.3806349353514418], [0.86954440334786165, -2.3115118133048838, -1.4698251461516709, -4.8349757534909745, -5.4876976858381479], [-3.846082197268994, 3.0904330285886998, -1.7054512215973578, -1.0608296663046812, 1.5063599523112337], [1.2581234301252979, -4.3469198252188708, -0.28176101702579315, -4.1114461723059161, -3.3672192377726442]], [[5.3098071301627341, 3.5327160461133147, 6.7493024552373422, 6.4017652184869727, 9.1729275402009076], [-0.15205591719370037, 1.0623060478643325, 3.8953048100595442, 3.6497656212865031, 0.62001915880557323], [4.5208344676560106, 2.2370655749677955, 9.5268702289692175, 3.9019290787648426, 8.2304802683696554], [3.7446264872124164, 3.7995227644312788, 5.1949620871709694, 6.8047107517296981, 4.2791664367730666]], [[-4.1909332742581453, 1.5859974172464506, -2.8554393552565061, -2.7738940899494602, -2.0997926955675137], [0.78784381398747438, -4.2749523891341852, -2.4329992608868869, -1.2561065047386171, -5.0854977399321264], [-1.6552045529511643, 1.8408166810353093, 2.5377812120745959, -5.1054253135264958, -2.2661785674725747], [0.90291148066078541, -5.8447904837831173, 1.7796038527307418, 3.0775514371218415, 2.3480337463361272]], [[-5.7871928010855944, -0.14414611787069376, -1.2516264136256279, 0.28825195991580088, -3.8539763832935159], [-0.091921838487407648, -3.678609798002348, -0.71747271223179432, 2.2449497191443424, 1.1949253579090424], [1.6401802021594039, 0.18784209741598534, -3.4557958347082685, 3.3094984136761578, -0.0018180121635609225], [1.8896927152941547, -0.60540337746629458, -0.36753384360612884, -0.66331579579398081, -4.826811114556131]], [[0.78233410228359279, -0.91388003680700791, -2.7101430422750781, 3.3783492814618192, 2.0095245204869574], [0.6054070352767642, 1.1574223869395608, 1.8568676355084195, 1.4405749146735953, 2.4642314747594565], [0.73836428871393789, -2.4633279759635984, -0.58505463651815592, -0.5909262459984852, -2.1504186628635464], [-5.2067361678825703, -1.0926619924404921, 3.3747483706678967, -1.0509110386054399, 2.8538140362185711]]], [[[7.2699459044693828, 6.7339082120422633, 3.9426608794165734, 5.7664525168921106, 2.2256561523627987], [6.5884736292261641, 0.48781292770679041, 1.1695640760742889, 2.694149776452134, 3.309095628478496], [0.61798877969253008, 7.7690803369023023, 6.0253883760109952, 4.8083980145771799, 1.4796465950957018], [3.2230794050723555, 1.8967830039420588, 7.7520678332698782, 3.4829734207638694, 6.1458949979127757]], [[4.5480920930995286, 6.9185223066441974, 6.6499384050970107, 8.5758737122920614, 2.0082196661227987], [2.4133231107158153, 7.6204099544169761, 0.98896579919822081, 8.9874820966580096, 2.0836824800746241], [6.2111963484822956, 8.9900684748624577, -0.20051898808426216, 4.7207743255938794, 1.7494746767541232], [8.9437559167279908, 2.7715972079239433, 5.2137633629046736, 8.6843809737666753, 6.3394777850450206]], [[0.43359502589384036, 2.0041445240656328, -0.48926228677652395, 4.8110024873352986, 1.4188897644462295], [-1.6543996720141569, 0.30112238435229788, 2.5953535879010978, 6.5445635714281822, -0.2625491566303797], [4.0797291114714485, 4.7455506948703663, 6.2713703499407254, 6.4721370532742677, 3.2939451790031287], [0.45055129993029297, 7.0356142743166901, 7.5709057140167433, 4.9045351279683169, -0.25840625202474543]], [[-2.0461337012199587, -3.0322500676396311, 1.2957094032026744, -0.23853705843075002, -4.3519116509788836], [1.359248769963024, -3.8658801520258788, 4.3021281625257686, 0.69632247433289685, -0.42673477840504326], [2.8007188515375887, 2.8866546751634763, 2.4091132729244631, 5.0505511466965185, 0.84546794553449534], [0.57017046587325915, 2.900309943300007, 2.4476612125412736, 1.5400882485684013, -2.8945380820331108]], [[-4.4341232190793969, -4.0262156177838717, -5.9131815477569694, -7.1683717692822713, -4.2474820283692125], [-5.3395095442521443, 0.099697933495309421, -3.5292994011473739, -3.9324790072794169, -1.3197325784151772], [-6.5312346568095903, 0.32381361512387397, -4.1179033584800573, -0.43221333327870415, -3.9133340119995461], [-5.4549490459872203, -7.3259923658039465, -8.1613738845434938, -6.0938286003941569, -7.5543228416685917]]], [[[-6.0485814810363419, -3.4823156130000728, -3.8583917793198483, 1.5017710648665386, -0.43399742246370998], [-4.593677505775922, -1.2415768321737017, -5.2761894866320267, -4.1915025404561401, -4.1777824486969131], [3.1950086514588918, -5.1570977472878132, -4.8755411558144317, 2.1695224673223983, 3.6616070685612154], [-5.8403860333996915, -1.814086549391793, 1.3466111983426732, -3.3799964291614457, -1.4472320735384976]], [[-5.1655339251626566, -3.1409610411731883, -1.6983327527009733, -0.83302845115660862, 2.7459375619792299], [-5.7856277469257353, 2.3131150522588326, -4.4666443341106854, -0.64875993326656278, -0.23883049349625463], [2.2936079118268831, -4.3951373826854905, -4.6195305782930776, 0.99942247230094861, -5.7880233435889394], [2.2673321501443633, -3.3239418017267108, 0.99918541053327203, -0.2785449697964113, -1.3882712455139474]], [[-5.6078392488288173, 2.6016488252628083, -2.4346658650065773, 1.1671202460649717, 2.5107890213202317], [-5.7159309122297488, -1.4845320418853927, -5.3219982803064552, -2.3658097840205983, 0.44609564127314982], [1.7770133050265251, -0.68240484818715519, -0.89703149844565999, 0.34788364293608609, 2.4161897907903089], [1.1699231971705029, -7.2803145155986613, -6.5527218170005561, -6.8650992340698602, -0.56536295918568147]], [[-2.0306831887591299, 0.45678443517167011, -6.9052626590618811, -1.0187181353428425, -3.1263912236546458], [-3.0256867605208919, -1.4562030904921457, -0.87386094845608753, -0.7752539099054836, -1.0054925523375378], [-5.0272575407847659, -5.7732644704378986, -6.5589647773262678, -7.1792380497231143, -7.7386961995232424], [-7.101924397377724, -6.1936967543668251, -0.19226902709483262, -3.6003128518956711, -0.27967428367453628]], [[-2.4188214917647977, -3.7611644318995254, -1.3087356094847826, -3.5727969308728293, -4.2074031121501001], [-4.1362050697212851, -8.0297026715512612, -8.6448805816237364, -7.3706284949780843, -5.6176977298735213], [-8.5825703718325492, -8.6329941508761561, -8.6243245746342829, -4.6956200377375534, -8.3017052432299643], [-1.7800787495424637, -7.4078064341844208, -8.6948591736016336, 0.95239831189249236, -5.9881608143731313]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_expandedData_rank4(self):
      arg0=Data(numpy.array([[4.6025970830087441, 1.3203932183935088, 0.79145012376977331, -3.1833712315075391, 0.73499399687808165], [-3.9921799203249897, -1.7068368354741481, 2.6110174920884841, -1.6212329991350307, -3.5309732741885069], [-1.1395288117243876, 0.24623775719366847, -3.8731898698307763, 3.3474520163765078, 1.754702487398113], [-0.53361607304726366, 1.7964197892362179, 3.4622353857662489, -3.9714809098536508, 3.3691945195611623]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.7286590036674774, -1.8292471502840293, -0.77215100560519545, 1.3893785767555658, -1.9496677891066172], [3.503106089830176, 0.015834463094730467, 2.4758069315777576, 3.992829871236049, -3.0851589772234234], [-3.6866102575000079, -1.220864093259586, 0.82344578311444749, 1.6019570331708373, 2.741849086658549], [2.643334099316319, 2.171285364320009, -2.8810655018469311, 3.7820641797276124, -3.7750698615228382]], [[-2.4867913834808961, 3.276495059988358, -1.6021073317560774, -0.46980215920735979, -2.0459709276278648], [0.34995386767001513, 1.1518794072056355, 2.0979799381633146, -4.4757375202143725, -1.1719098430117314], [-0.31714181901118899, 0.69844682459347585, -0.18897639263738242, -4.2489099357339546, -3.5382129192162126], [3.1665090985504101, -3.0040766865123256, -2.0585995800571775, 1.5364990263326108, 2.1802950312082618]], [[1.2977450579419294, 4.5948429511555524, 4.3622026275012065, 1.4474519251017455, -3.7684882047916659], [-2.835284260047902, 3.5550432122924285, 4.0926047905332599, -3.8772859779677282, 1.2498787327468879], [0.98660700962635239, -2.9890703792583775, 3.9531028845141254, -2.3571644710344364, -1.4421271340175377], [-3.6611807821463538, 1.6153640723951179, 0.42415483673309051, 4.3967792184329362, -1.5851227136999357]], [[2.1337982004179681, 2.1194484793492672, -4.0262855239115583, 1.3873833478405784, 0.46791455304336171], [0.16299187133590376, 0.754135002266203, 4.0151366560508563, 1.0575243181051039, 1.7169515511301672], [4.5150813510885452, 3.6623958297320662, 0.47945331049707285, -1.9185728795350743, 4.6584612941406007], [-2.1570776490295418, -0.031200976676565517, 1.5543330411397704, -1.9100989509366864, 2.3676753181761629]], [[-4.0372668984297277, -4.9798154587364589, 3.2059905525226853, -0.097487473795863977, -3.4734761345057463], [2.5623087031310101, 3.8809721752381936, -0.25601178201203201, 3.1954214338107949, 3.5408629891160288], [3.045350182985052, 3.5921709145116267, 4.3278573819592374, 2.1778744383589252, -1.0800520483401197], [3.2166810710672458, -0.55592458495610941, -2.2140663684581074, 0.37939919447151382, -0.59243242574929855]]], [[[4.7574297421923255, -4.0929830135595395, -0.4903383691591916, 0.08476588437606658, 0.21540445154031396], [-0.6936222934297529, 4.7564425282021254, 2.9501298088176195, -4.1592191583414619, 0.17509375816786932], [3.679453246463428, -2.1992276329437388, -0.5243955332040553, -4.2765427416716673, -3.1153908973007018], [1.6188135256976963, 0.5568124281730622, 2.6131806667437028, -0.76470582851924096, -4.2556109458714122]], [[4.6436744928370004, -4.6571409242976536, -1.534800039732962, 1.4061301052897734, 2.0134044863706313], [-3.2148075063890413, -1.3762078643097677, -4.4554870899327481, -2.0688490206596724, 3.3679725773242541], [4.0883619316292865, 2.9141814228442753, 2.8698410883860683, -4.51108993986049, -0.49301614307363195], [2.484730874650257, 3.8840851478212564, 1.075672327910727, -2.9231039262753331, 3.8828139834073543]], [[2.7582779578239967, 3.8488023896577861, 0.54780564492073403, 0.076714033883283683, -3.3831843799313255], [1.5548821394735111, 3.8105581042715606, -3.0023353643212292, 2.4982929472026596, 0.0087829972909663212], [1.2412948852208761, -4.3305219348813919, -4.4620073639238083, -4.4260435176467672, 0.30780314221749538], [1.7117546814132645, 1.4407517357558355, 1.1208777586398941, 0.44185793937113615, 0.72343866086734643]], [[-0.89631284760883112, -1.2632496906828594, -3.581285945369256, 1.9660927786647537, 1.5981325191449001], [-0.95380487183015816, -3.9758411044348563, 4.4103337293161218, 1.1531826326342856, -1.9113898222960177], [-0.6718711087582161, 1.0160847102916195, 4.1685341181442421, 0.30853391279753506, 0.72375087667243321], [0.64096456090063558, 1.5615358692990897, 1.7549940104257047, -0.69858551358497323, -0.49591921544026807]], [[-4.2080241362381869, -0.89280695698304413, -0.70040634004215541, -1.809054028720011, -1.8372411270545719], [4.5279334642786431, -1.0712866832882462, 3.1976403846936243, 0.92846601581690713, -3.5284425686581744], [-3.1521719066100831, 2.5379803869583597, -0.59901260747099272, -0.47713917121250926, 1.2586047506161382], [1.3971227849670331, -2.7000144699503359, 2.1189760188624129, -3.5498854511167099, 2.4819712444802535]]], [[[-1.3446464959892026, 2.5778371621903222, -2.5500526985075842, 1.9929831575383883, 2.9527535044203681], [0.75605368085789681, 1.3997855324787594, 1.3417628615640877, -4.0599441810449228, 0.39063654320784735], [-3.8536707898866629, -1.7069521542787847, -0.77434054279329523, 3.4389667613846822, 2.728543813197934], [4.6991307214330522, -0.28946673518890709, -1.8765420056336124, 0.88508546123390541, 1.8603356181842523]], [[-2.3947268858389004, 0.87267638452802032, -3.3006484454912388, 4.6660530695835085, -4.7912908889330508], [-0.96372850240283103, -1.8151711488939029, 3.9461165287305615, -4.4797693342664937, -4.772945995676201], [2.8897677157250072, -3.0950017840516098, -0.74192034765194848, 3.7382768558342629, -0.023270439122466513], [1.7341092449856719, -1.585540401282306, 4.6665996695264269, -0.77288829742570098, 3.7069377891350648]], [[-1.6811957348917939, 1.9515724303579525, -1.2986955385438259, 0.36910858052777407, 0.021183469260121157], [-4.7857414452876004, -4.0883455814235079, 2.7343476483031024, -3.3078423504545782, -3.8029742805857625], [1.5662052392721746, 0.2869261494112525, -3.8372653321059813, -0.53891516953722451, -3.2769672496633264], [-3.477855679240661, 4.7692442217477176, -3.7542576208122869, 0.23518425813214527, -3.7672298411551042]], [[0.2872057926882432, 1.8755919085459976, -0.070772956304820056, 4.6117304510316917, 4.9607057748462893], [2.9651279309291469, 1.987870182142192, -0.81454607625117159, -2.6668300998550709, 4.8810281914475357], [0.35049360035884725, 4.6922234246314058, 2.4259698293540497, 3.4718942456369373, -2.4003322930777662], [0.63778780265297996, 3.3651216880362114, 1.0375098538286984, 4.0728568577264266, -3.3944231721149887]], [[-4.2247598570669318, -4.1839496351273606, -4.4683012030214018, 4.1580131873407709, 0.51902621698887952], [-4.2646362713129502, 3.9936350234794329, -1.3705045060718222, 1.6251976108310719, 3.1228124100920134], [1.4444529691744004, -3.3876163280390403, 2.5355052685306756, -4.4559347697300611, 3.3002328321215444], [-4.2725775265977379, -0.071232814575603598, -1.1978559504117969, 4.8970503608376941, 2.0285564768901772]]], [[[2.3940877773969742, -2.0439847021001354, 1.27175572859589, 4.0479734109688774, 2.3424320881568743], [3.1588774981173113, 3.8181162214716693, -3.8108776856714641, 1.6810270678415407, 1.2107919785746057], [-0.22522223512704809, 0.66804285972888344, 1.5950796469701469, -1.3280846582787307, -3.4513500398176511], [4.0631135093768442, -1.166728930170259, -3.6234635696273845, 4.25778455390647, 4.5212593052058949]], [[4.4890144691694758, 0.18274080627423928, 4.4634974480401617, -0.79608668979374553, 4.0276892653065204], [1.8689926175944755, 3.7756585573126724, -2.5167983458011491, 4.0548374953517765, 2.1470404801560203], [0.2157115480621048, -3.2914167386596107, 2.5414304801295309, -0.30246713487589361, 4.079634845242726], [2.7304468381113312, -0.52149423055391964, -1.6320820321129084, -0.37168137902640197, 4.8502663744980854]], [[2.1971315649408254, 3.7018514728424918, -0.12670858635731275, 3.0808611151560505, -3.9504061124948189], [2.9203516520337471, -2.1029102694442203, 0.39535172916092431, -0.47653131483951672, 0.89806883362573942], [2.7173856565263925, -4.0040303967763311, -3.4889813532588665, -4.6144561538016795, 1.8748946254037255], [2.9899925558022256, -3.2557559030512593, 1.3596219527562354, 3.8339927552638535, -2.4515203485221848]], [[0.34963966945851865, 4.4242276552481457, 0.28655677089161458, -4.9501772972080609, -1.41369541723672], [-2.0653692303694915, -4.1455982356087064, 1.9660951880742275, 3.0567519982198554, -2.9281501167297885], [0.33965479490218442, -3.4674563586437666, -3.4368695229856447, 3.5557130953420444, 1.2502342112526712], [0.20976243193375232, -3.7775146068749654, -0.048994630398725647, -0.55355304726238952, 2.5123547199126062]], [[2.2788843250201598, 0.96337842702498033, -2.5040732266392194, -1.1256623338553862, -1.7855885799084934], [-4.1375823281246813, 2.6256917198951193, 2.8676253985861866, 3.2158706503465808, 0.38205380384818177], [-3.0359515537673767, 1.9244599743543631, 1.3417423310868219, 1.1442419868790621, 0.37162390665875034], [-1.8076536127876128, 4.5324552214381271, -1.197261211131484, 2.2242413859352546, -3.5244985602130718]]]])+(1.-msk_arg1)*numpy.array([[[[-2.7933620830229691, -2.9011833673278176, -1.8713469791810144, 0.450177323741487, 0.53110589688597365], [0.35334780746251848, -3.1689287584567549, 0.99800572350918859, 0.65272146114691765, 1.044046759630227], [-2.6815799267984985, 1.8016364366834567, 1.7635301912816645, -3.5579344370267627, -4.7745497263952892], [-0.471207228811382, -0.97849006998505406, 3.9256448273707409, -1.2917020412854097, -2.9262633836437999]], [[1.9213872908583109, -1.0084613819943335, -3.5845660066069462, -3.7064490510752943, -3.4756903922802063], [-2.3263632319144345, 0.21149208228304062, -1.0143566119986058, -0.3227135189562782, 4.7296680194917418], [-4.8843658786156929, 3.2131141206643647, 2.6135384684212379, -0.75873333795373643, 1.0818647048610996], [-1.6560062025718136, 1.186356659610782, -1.0395158947015357, -1.9926735345880475, 0.78349666958670028]], [[4.3101364589952702, 4.4476553702439343, 1.1312376682504652, -3.920830184053373, -1.2338201259842005], [-1.6397329000197001, -4.1025011683967545, -4.227730343495157, 1.319102516880152, 3.8690447523267135], [4.6808500491251657, -0.73227973309942485, -0.5094239450872271, 2.3952958863163634, -2.1515870843895533], [-3.0010838908473159, 3.2255447343410832, 0.045844834679043878, -3.6894877780804136, 4.4697780646757543]], [[2.3706815890743194, -1.5414947925754721, -0.90843893325129521, 4.9590958151900359, -0.7646496454955285], [-0.86259757844249574, 3.3003360275303084, -0.83820239063151636, -3.1133438552006822, 3.9847409116564751], [2.3041439475515162, -2.7376519011268741, 0.93284184375008206, -1.7755009055949222, -3.8312353239805761], [-3.7016282136626955, 1.833034500186117, -3.962770619636701, -0.28199259839567858, -1.8637800807879188]], [[-3.8551470761519102, 0.25931949914696251, -2.3174358377207147, 1.8062360726050128, 1.410466879739019], [-3.3825563829387773, -1.3194182157758139, -3.1166120123408936, 3.0508264905192419, 2.7448884895787824], [4.2529954653282083, -3.8449040052592522, 1.4041429690151563, -2.1061930208620785, 2.1594778496415135], [-3.2489142379974609, -2.6701648942836567, -0.14364751641609974, 1.8436525493448253, 1.5175456490410379]]], [[[-3.0461632909607061, 4.4930814141068165, 2.0872069709540924, 0.99752396990543613, -1.4520061255372152], [-4.1825714469429069, -0.59329848028208865, 4.0045398987451648, -1.3476250083940808, 3.6811321235202286], [-1.6762850227938522, 4.2281042031774838, -0.42673143006401038, 2.5406518757740812, 3.8092693752487126], [1.0078134487893031, 3.2103176528020292, -1.4178203681573001, 1.1832793903156515, -4.0754191149664321]], [[3.2300342190512321, 3.8306338751656028, -4.7981764855035127, -0.66050771765838423, 0.18184119181797431], [-1.94559973000315, 2.7559650642307503, 4.9402847928019007, -2.0479798873617208, -2.3235631498618412], [4.6205240279094291, 1.2282675442297011, -1.2125594814673866, 3.2196167705871552, 2.4396827342377225], [-1.9476431448074258, 1.2364652385115305, 1.7595564593904633, -2.8355630596058576, 1.581848196229986]], [[-1.102208380335993, -0.67883392317559199, -0.64564379866414257, 1.0458653573979575, -4.2982178198691674], [-2.1966155891718597, -4.2827765554174233, -2.7270804316279387, 1.3017533956102678, -4.8336995797375835], [3.4090499186793419, 2.2884609080599327, -1.3744459058797931, -2.6776108504700336, -0.44920205500347787], [2.081885357013407, 1.9600501223887417, 2.106901458620035, -1.8713133649524227, 2.175866937915627]], [[3.4725186245888935, -1.9626253354897552, 1.2382807302542691, 4.0797645866253323, -3.2255010843813468], [3.7001579010080761, -3.167381810144736, -0.64146681749265788, -1.253828093723933, 0.1302486626315762], [-0.46119523488547998, -4.5896479973300082, 2.2928758598694348, 4.4131560760070485, 4.8492226583377711], [-0.85614165439597834, -1.3291360299904564, -1.1871297208599687, 3.8235973667632361, -3.1925942699912513]], [[1.2486027509163975, 2.4440818749032616, 1.268778006794804, 0.20608758207253697, -1.5399124153938026], [-2.4121111621956559, 1.3946001322880086, 1.5786488587508529, 4.3296822173470879, -4.1502771458059922], [-2.2490910210846526, -3.7258658662519153, 2.0812867775051576, -3.5392944597380005, 0.2557358308339408], [-0.63766568911426624, 4.2326720246884015, -2.5117307667405111, 0.84993908188565293, -3.0746987880887957]]], [[[4.1174498940188116, 3.9965751171384305, -2.2618298755883046, -4.4503702324100836, 2.9657893893340184], [-0.56789373377671293, 3.0264900011839106, 0.7386952772425337, -3.0812764795640812, -3.7160323563858189], [0.64968892031264325, 0.78235929846438257, -3.7614976466377446, -1.233452999950797, -0.47488930271135121], [-2.0065517267040911, 3.8666458467128635, -0.032998923612430175, -0.53925510555041267, -4.0258716646948898]], [[4.4467750000930284, 3.1021529308528546, -1.8972304337174095, 2.2672799719247383, 1.7334577320222948], [-4.7336776914346057, -4.0933821787383993, -3.9609741968909273, -2.9652860317307264, -3.2984924191454281], [2.3967080380663317, -3.4535825099190784, 0.18706265620374385, 4.5577173663520778, -2.5305862623282218], [-3.5469207737351791, 1.3909078404537114, -2.0467331221130722, 3.2965069958054158, -4.988420638502542]], [[-1.3652656336458859, -0.40122267927501909, 0.59351723470480433, -0.48515982596659946, 2.7433631819095403], [-1.7009038267680232, 3.022095158513574, -2.968777995213089, 0.85580174807008191, 2.4278531996915307], [1.9447130055125523, -0.71364637545128229, 3.195032697469312, -0.52763235592785307, -3.1099622164200613], [-1.6583638330872894, 4.5119951311462554, -2.6774257748787891, -3.2184951016792551, 1.1137954681491999]], [[-0.20517330332497785, 4.4858890750095402, -3.0698220194336057, -2.126971790285638, -2.3881442888461999], [-3.6084670095384186, 1.5078601315889308, -1.7710656127319546, 4.9012462541650184, 3.0647861619416599], [-0.74376525252465697, -1.5651104358020804, -0.17584436844610174, 3.2946849745643352, -0.5031400719021768], [0.69801882812870986, -3.1230067168320885, 0.061472949345670713, 4.0198311752410607, -0.1389449485882075]], [[-3.7040242712818272, -2.6955110921583145, 4.0867710802282371, 2.7887558626399258, -3.4972585489005983], [2.7447732541548948, -4.4579563601971444, 3.8672013260705747, -0.21279570981964824, -1.1941253437381083], [2.1890021686599512, -2.3155275174650161, 4.9165359095830183, -4.5867597093742294, 1.6318686858660918], [-2.5511198513066158, -3.6170954443670822, 0.71622562900127562, -1.7590700098504319, -0.34324782335479931]]], [[[-1.433407510323371, 0.12482593668786102, 2.4157377265184898, 3.9970690059236382, -0.7840953144185665], [-2.9360499035009093, 1.0168399950549869, 4.5579973025095022, -2.8652940000218443, 4.6130163784689806], [2.3945239316156712, 3.5663029349214685, -1.1581372781732635, -2.4446812819964792, 0.51985327959768934], [-0.77087273301280046, 3.4175180174412212, -2.0855791700087223, 0.44384784798642851, 2.799073841594419]], [[4.6651251931692723, 1.0142657106881021, 1.2452461231206202, 2.0384470433110753, 2.2733992652300739], [4.2925984028375712, -4.9042924207540315, 4.182304680956701, 2.5030592170668706, -2.2511075741953537], [4.1421054883674273, 1.2194824242944033, -0.25520299255283074, 0.90034991934677766, -3.6324556337866043], [2.3223541078098853, 2.307700090411271, 3.7008767393014956, 2.1460906665584423, -1.5320964345315358]], [[-4.2217772428551692, 0.68811654135845313, 2.9688803439517848, -1.846360409998995, -2.9143032545744774], [-1.521798792732223, -2.7293300682450305, 2.7993133466230251, -1.8687313257706686, -4.3243620691923903], [1.5445725024995651, -2.3032138724276194, -3.686213458966817, 3.8603468713558424, 1.1413179833466645], [2.4054085470890696, -3.0857058378182667, 3.7158926280518507, -2.3657134385823362, -0.85194550478241915]], [[4.1881785460937486, 4.2536761763445305, -0.063487355470090101, -4.0514269372128418, -0.61295089468644548], [-0.137972847141973, 0.59385706565508656, 3.3234950049211047, 2.9756175599103649, 0.84358126094511743], [2.275279395879596, -1.7426621006604437, 4.9649536688450464, 1.8563380916458341, -1.1496932623617706], [3.5077000618428045, 4.6968305990009025, -0.42447146349364218, 2.3003695982998797, -2.8052324771014936]], [[1.3595765650608023, 0.65125018043651828, 3.9217898640745759, -4.3992619601138871, -3.863148061970775], [3.6556526824375357, -4.4923239382243985, 3.2918561546851528, -4.6489009401223882, 1.2808286482002575], [-0.11109743152879226, 4.9230003566302791, 4.8571730668163955, -4.440486803342286, -2.91495115210616], [1.7416266531681401, -3.9925187869476773, -4.4850744836140466, 0.92728961768815843, 2.6737897263509502]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.3312560866762215, 2.7733499327247149, 3.8304460774035487, 5.9919756597643099, 2.652929293902127], [8.1057031728389202, 4.6184315461034746, 7.0784040145865017, 8.5954269542447932, 1.5174381057853208], [0.91598682550873622, 3.3817329897491581, 5.4260428661231916, 6.2045541161795814, 7.3444461696672931], [7.2459311823250632, 6.7738824473287531, 1.721531581161813, 8.3846612627363566, 0.82752722148590596]], [[-1.1663981650873874, 4.5968882783818668, -0.28171411336256869, 0.85059105918614897, -0.72557770923435605], [1.6703470860635239, 2.4722726255991443, 3.4183731565568234, -3.1553443018208638, 0.14848337538177736], [1.0032513993823198, 2.0188400429869846, 1.1314168257561263, -2.9285167173404458, -2.2178197008227039], [4.4869023169439188, -1.6836834681188169, -0.73820636166366871, 2.8568922447261196, 3.5006882496017706]], [[2.0891951817117027, 5.3862930749253257, 5.1536527512709798, 2.2389020488715188, -2.9770380810218926], [-2.0438341362781287, 4.3464933360622018, 4.8840549143030332, -3.0858358541979549, 2.0413288565166612], [1.7780571333961257, -2.1976202554886042, 4.7445530082838987, -1.5657143472646631, -0.65067701024776436], [-2.8697306583765805, 2.4068141961648912, 1.2156049605028638, 5.1882293422027095, -0.79367258993016243]], [[-1.049573031089571, -1.0639227521582719, -7.2096567554190969, -1.7959878836669607, -2.7154566784641774], [-3.0203793601716353, -2.4292362292413361, 0.83176542454331726, -2.1258469134024351, -1.4664196803773719], [1.3317101195810062, 0.47902459822452714, -2.7039179210104662, -5.1019441110426129, 1.4750900626330616], [-5.3404488805370809, -3.2145722081841046, -1.6290381903677686, -5.093470182444225, -0.81569591333137614]], [[-3.3022729015516461, -4.2448214618583773, 3.9409845494007669, 0.63750652308221767, -2.7384821376276647], [3.2973027000090918, 4.6159661721162752, 0.47898221486604964, 3.9304154306888766, 4.2758569859941105], [3.7803441798631336, 4.3271649113897084, 5.0628513788373191, 2.9128684352370069, -0.34505805146203805], [3.9516750679453274, 0.17906941192197223, -1.4790723715800258, 1.1143931913495955, 0.14256157112878309]]], [[[0.76524982186733581, -8.0851629338845292, -4.4825182894841813, -3.9074140359489231, -3.7767754687846757], [-4.6858022137547426, 0.76426260787713574, -1.0420501115073701, -8.1513990786664507, -3.8170861621571204], [-0.31272667386156172, -6.1914075532687285, -4.516575453529045, -8.268722661996657, -7.1075708176256915], [-2.3733663946272934, -3.4353674921519275, -1.3789992535812869, -4.7568857488442307, -8.2477908661964019]], [[2.9368376573628523, -6.3639777597718012, -3.2416368752071101, -0.30070673018437466, 0.30656765089648319], [-4.921644341863189, -3.0830446997839158, -6.1623239254068967, -3.7756858561338205, 1.661135741850106], [2.3815250961551384, 1.2073445873701272, 1.1630042529119202, -6.2179267753346377, -2.1998529785477801], [0.77789403917610889, 2.1772483123471083, -0.63116450756342113, -4.6299407617494808, 2.1759771479332062]], [[5.3692954499124808, 6.4598198817462702, 3.1588231370092181, 2.6877315259717678, -0.77216688784284138], [4.1658996315619952, 6.4215755963600447, -0.39131787223274506, 5.1093104392911437, 2.6198004893794504], [3.8523123773093602, -1.7195044427929078, -1.8509898718353242, -1.8150260255582831, 2.9188206343059795], [4.3227721735017486, 4.0517692278443196, 3.7318952507283782, 3.0528754314596203, 3.3344561529558305]], [[-2.5175458467438618, -2.8844826898178901, -5.2025189445042868, 0.34485977952972302, -0.023100479990130651], [-2.5750378709651889, -5.597074103569887, 2.7891007301810911, -0.46805036650074516, -3.5326228214310484], [-2.2931041078932468, -0.60514828884341121, 2.5473011190092114, -1.3126990863374957, -0.89748212246259751], [-0.98026843823439513, -0.059697129835940999, 0.13376101129067397, -2.3198185127200039, -2.1171522145752988]], [[-7.7389974104266939, -4.4237802311715511, -4.2313796142306623, -5.3400273029085179, -5.3682144012430788], [0.99696019009013614, -4.6022599574767531, -0.3333328894948826, -2.6025072583715998, -7.0594158428466809], [-6.68314518079859, -0.99299288723014723, -4.1299858816594996, -4.0081124454010162, -2.2723685235723687], [-2.1338504892214738, -6.2309877441388428, -1.411997255326094, -7.0808587253052169, -1.0490020297082534]]], [[[-2.4841753077135902, 1.4383083504659346, -3.6895815102319718, 0.8534543458140007, 1.8132246926959805], [-0.38347513086649077, 0.26025672075437178, 0.20223404983970017, -5.1994729927693104, -0.74889226851654023], [-4.9931996016110505, -2.8464809660031722, -1.9138693545176828, 2.2994379496602946, 1.5890150014735465], [3.5596019097086646, -1.4289955469132947, -3.016070817358, -0.25444335049048217, 0.72080680645986472]], [[-2.1484891286452319, 1.1189141417216888, -3.0544106882975703, 4.912290826777177, -4.5450531317393823], [-0.71749074520916256, -1.5689333917002344, 4.19235428592423, -4.2335315770728252, -4.5267082384825326], [3.1360054729186757, -2.8487640268579413, -0.49568259045828, 3.9845146130279314, 0.22296731807120196], [1.9803470021793403, -1.3393026440886375, 4.9128374267200954, -0.5266505402320325, 3.9531755463287332]], [[-5.5543856047225706, -1.9216174394728238, -5.1718854083746022, -3.5040812893030022, -3.8520064005706551], [-8.6589313151183767, -7.9615354512542842, -1.1388422215276739, -7.1810322202853545, -7.6761641504165388], [-2.3069846305586017, -3.5862637204195238, -7.7104552019367576, -4.4121050393680008, -7.1501571194941027], [-7.3510455490714373, 0.89605435191694127, -7.6274474906430632, -3.638005611698631, -7.6404197109858805]], [[3.634657809064751, 5.2230439249225054, 3.2766790600716877, 7.9591824674081995, 8.3081577912227971], [6.3125799473056547, 5.3353221985186998, 2.5329059401253362, 0.68062191652143689, 8.2284802078240435], [3.697945616735355, 8.0396754410079136, 5.7734218457305575, 6.8193462620134451, 0.94711972329874161], [3.9852398190294878, 6.7125737044127192, 4.3849618702052062, 7.4203088741029344, -0.046971155738480874]], [[-2.4700573696688188, -2.4292471477292477, -2.7135987156232888, 5.9127156747388838, 2.2737287043869925], [-2.5099337839148372, 5.7483375108775459, 0.38419798132629079, 3.3799000982291849, 4.8775148974901263], [3.1991554565725133, -1.6329138406409274, 4.2902077559287886, -2.7012322823319481, 5.0549353195196574], [-2.517875039199625, 1.6834696728225094, 0.55684653698631603, 6.651752848235807, 3.7832589642882901]]], [[[1.8604717043497105, -2.577600775147399, 0.73813965554862637, 3.5143573379216138, 1.8088160151096107], [2.6252614250700477, 3.2845001484244056, -4.3444937587187278, 1.147410994794277, 0.67717590552734208], [-0.75883830817431175, 0.13442678668161978, 1.0614635739228833, -1.8617007313259943, -3.9849661128649148], [3.5294974363295806, -1.7003450032175227, -4.1570796426746481, 3.7241684808592064, 3.9876432321586313]], [[6.2854342584056937, 1.9791605955104572, 6.2599172372763796, 1.0003330994424724, 5.8241090545427383], [3.6654124068306935, 5.5720783465488903, -0.72037855656493122, 5.8512572845879944, 3.9434602693922383], [2.0121313372983227, -1.4949969494233928, 4.3378502693657488, 1.4939526543603243, 5.8760546344789439], [4.5268666273475491, 1.2749255586822983, 0.16433775712330956, 1.424738410209816, 6.6466861637343033]], [[5.6593669507070743, 7.1640868586087407, 3.3355267994089361, 6.5430965009222994, -0.48817072672856998], [6.382587037799996, 1.3593251163220286, 3.8575871149271732, 2.9857040709267322, 4.3603042193919883], [6.1796210422926414, -0.54179501101008221, -0.026745967492617595, -1.1522207680354306, 5.3371300111699744], [6.4522279415684745, 0.20647948271498962, 4.8218573385224843, 7.2962281410301024, 1.0107150372440641]], [[-3.6218412403951321, 0.45274674539449489, -3.6849241389620362, -8.9216582070617108, -5.3851763270903703], [-6.0368501402231427, -8.1170791454623572, -2.0053857217794233, -0.91472891163379533, -6.8996310265834389], [-3.6318261149514663, -7.4389372684974173, -7.4083504328392955, -0.41576781451160638, -2.7212466986009796], [-3.7617184779198984, -7.7489955167286162, -4.0204755402523764, -4.5250339571160403, -1.4591261899410446]], [[5.6480788445813221, 4.3325729465861427, 0.86512129292194295, 2.2435321857057762, 1.5836059396526689], [-0.76838780856351896, 5.9948862394562816, 6.2368199181473489, 6.5850651699077432, 3.7512483234093441], [0.33324296579378565, 5.2936544939155254, 4.7109368506479843, 4.5134365064402244, 3.7408184262199127], [1.5615409067735495, 7.9016497409992894, 2.1719333084296784, 5.5934359054964169, -0.1553040406519095]]]])+(1.-msk_ref)*numpy.array([[[[1.8092349999857751, 1.7014137156809266, 2.7312501038277297, 5.0527744067502312, 5.1337029798947178], [4.9559448904712626, 1.4336683245519892, 5.6006028065179327, 5.2553185441556618, 5.6466438426389711], [1.9210171562102456, 6.4042335196922009, 6.3661272742904087, 1.0446626459819814, -0.17195264338654503], [4.1313898541973622, 3.6241070130236901, 8.5282419103794851, 3.3108950417233345, 1.6763336993649443]], [[3.2417805092518197, 0.31193183639917521, -2.2641727882134375, -2.3860558326817856, -2.1552971738866975], [-1.0059700135209257, 1.5318853006765494, 0.30603660639490293, 0.99767969943723056, 6.0500612378852505], [-3.5639726602221842, 4.5335073390578735, 3.9339316868147467, 0.56165988043977233, 2.4022579232546084], [-0.33561298417830487, 2.5067498780042907, 0.28087732369197305, -0.67228031619453876, 2.103889887980209]], [[5.1015865827650435, 5.2391054940137076, 1.9226877920202385, -3.1293800602835997, -0.44237000221442724], [-0.84828277624992676, -3.3110510446269812, -3.4362802197253837, 2.1105526406499253, 4.6604948760964868], [5.472300172894939, 0.059170390670348461, 0.2820261786825462, 3.1867460100861367, -1.36013696061978], [-2.2096337670775426, 4.0169948581108565, 0.83729495844881718, -2.8980376543106403, 5.2612281884455276]], [[-0.81268964243321973, -4.7248660240830116, -4.0918101647588347, 1.7757245836824969, -3.9480208770030676], [-4.0459688099500344, 0.11696479602276932, -4.021573622139055, -6.2967150867082218, 0.80136968014893606], [-0.8792272839560229, -5.9210231326344136, -2.250529387757457, -4.9588721371024613, -7.0146065554881147], [-6.8849994451702345, -1.3503367313214221, -7.1461418511442396, -3.4653638299032177, -5.0471513122954583]], [[-3.1201530792738286, 0.99431349602504415, -1.5824418408426331, 2.5412300694830945, 2.1454608766171006], [-2.6475623860606956, -0.58442421889773222, -2.3816180154628119, 3.7858204873973236, 3.479882486456864], [4.9879894622062899, -3.1099100083811706, 2.139136965893238, -1.3711990239839968, 2.8944718465195951], [-2.5139202411193793, -1.935170897405575, 0.5913464804619819, 2.5786465462229069, 2.2525396459191196]]], [[[-7.0383432112856958, 0.5009014937818268, -1.9049729493708973, -2.9946559504195536, -5.4441860458622049], [-8.1747513672678966, -4.5854784006070783, 0.012359978420175111, -5.3398049287190705, -0.31104779680476113], [-5.6684649431188419, 0.23592428285249412, -4.4189113503890001, -1.4515280445509084, -0.18291054507627713], [-2.9843664715356866, -0.78186226752296051, -5.4100002884822898, -2.8089005300093381, -8.0675990352914226]], [[1.523197383577084, 2.1237970396914547, -6.5050133209776604, -2.3673445531325323, -1.5249956436561738], [-3.6524365654772981, 1.0491282287566022, 3.2334479573277526, -3.7548167228358689, -4.0303999853359898], [2.913687192435281, -0.47856929124444703, -2.9193963169415347, 1.5127799351130071, 0.73284589876357442], [-3.6544799802815739, -0.47037159696261766, 0.052719623916315239, -4.5423998950800062, -0.12498863924416215]], [[1.5088091117524911, 1.9321835689128921, 1.9653736934243415, 3.6568828494864416, -1.6872003277806833], [0.41440190291662438, -1.6717590633289392, -0.11606293953945457, 3.9127708876987519, -2.2226820876490994], [6.020067410767826, 4.8994784001484168, 1.236571586208691, -0.06659335838154945, 2.1618154370850062], [4.6929028491018911, 4.5710676144772258, 4.7179189507085191, 0.73970412713606137, 4.7868844300041111]], [[1.8512856254538628, -3.5838583346247859, -0.38295226888076161, 2.4585315874903015, -4.8467340835163775], [2.0789249018730453, -4.7886148092797667, -2.2626998166276886, -2.8750610928589637, -1.4909843365034545], [-2.0824282340205107, -6.2108809964650389, 0.67164286073440405, 2.7919230768720178, 3.2279896592027404], [-2.4773746535310091, -2.9503690291254872, -2.8083627199949994, 2.2023643676282054, -4.813827269126282]], [[-2.2823705232721094, -1.0868913992852454, -2.2621952673937029, -3.32488569211597, -5.07088568958231], [-5.9430844363841633, -2.1363731419004983, -1.952324415437654, 0.79870894315858099, -7.6812504199944991], [-5.78006429527316, -7.2568391404404222, -1.4496864966833494, -7.0702677339265074, -3.2752374433545661], [-4.1686389633027732, 0.70169875049989461, -6.042704040929018, -2.681034192302854, -6.6056720622773026]]], [[[2.977921082294424, 2.8570463054140429, -3.4013586873126922, -5.5898990441344711, 1.8262605776096308], [-1.7074225455011005, 1.8869611894595231, -0.40083353448185388, -4.2208052912884693, -4.8555611681102064], [-0.48983989141174433, -0.35716951326000501, -4.9010264583621321, -2.3729818116751846, -1.6144181144357388], [-3.1460805384284787, 2.7271170349884759, -1.1725277353368178, -1.6787839172748003, -5.1654004764192774]], [[4.6930127572866969, 3.3483906880465231, -1.650992676523741, 2.5135177291184068, 1.9796954892159633], [-4.4874399342409372, -3.8471444215447308, -3.7147364396972589, -2.7190482745370579, -3.0522546619517597], [2.6429457952600002, -3.20734475272541, 0.43330041339741232, 4.8039551235457463, -2.2843485051345533], [-3.3006830165415106, 1.6371455976473799, -1.8004953649194038, 3.5427447529990843, -4.7421828813088736]], [[-5.2384555034766622, -4.2744125491057954, -3.279672635125972, -4.3583496957973757, -1.129826687921236], [-5.5740936965987995, -0.85109471131720227, -6.8419678650438653, -3.0173881217606944, -1.4453366701392456], [-1.928476864318224, -4.5868362452820586, -0.67815717236146433, -4.4008222257586294, -6.9831520862508381], [-5.5315537029180657, 0.6388052613154791, -6.5506156447095654, -7.0916849715100314, -2.7593944016815763]], [[3.1422787130515299, 7.8333410913860479, 0.27762999694290214, 1.2204802260908698, 0.95930772753030791], [-0.26101499316191079, 4.8553121479654386, 1.5763864036445532, 8.2486982705415262, 6.4122381783181677], [2.6036867638518508, 1.7823415805744274, 3.1716076479304061, 6.642136990940843, 2.844311944474331], [4.0454708445052177, 0.22444529954441927, 3.4089249657221785, 7.3672831916175685, 3.2085070677883003]], [[-1.9493217838837142, -0.94080860476020156, 5.8414735676263501, 4.5434583500380388, -1.7425560615024853], [4.4994757415530078, -2.7032538727990314, 5.6219038134686876, 1.5419067775784647, 0.56057714366000466], [3.9437046560580642, -0.5608250300669031, 6.6712383969811313, -2.8320572219761164, 3.3865711732642048], [-0.79641736390850282, -1.8623929569689692, 2.4709281163993886, -0.0043675224523189726, 1.4114546640433137]]], [[[-1.9670235833706347, -0.40879013635940264, 1.8821216534712262, 3.4634529328763746, -1.3177113874658302], [-3.469665976548173, 0.48322392200772324, 4.0243812294622385, -3.398910073069108, 4.0794003054217169], [1.8609078585684076, 3.0326868618742049, -1.6917533512205272, -2.9782973550437428, -0.013762793449574318], [-1.3044888060600641, 2.8839019443939575, -2.6191952430559859, -0.08976822506083515, 2.2654577685471553]], [[6.4615449824054902, 2.81068549992432, 3.0416659123568381, 3.8348668325472932, 4.0698190544662918], [6.0890181920737891, -3.1078726315178136, 5.978724470192919, 4.2994790063030885, -0.45468778495913575], [5.9385252776036452, 3.0159022135306213, 1.5412167966833872, 2.6967697085829956, -1.8360358445503864], [4.1187738970461032, 4.1041198796474889, 5.4972965285377136, 3.9425104557946602, 0.26432335470468216]], [[-0.7595418570889203, 4.150351927124702, 6.4311157297180337, 1.6158749757672539, 0.54793213119177153], [1.9404365930340259, 0.73290531752121835, 6.261548732389274, 1.5935040599955803, -0.8621266834261414], [5.006807888265814, 1.1590215133386295, -0.22397807320056806, 7.3225822571220913, 4.6035533691129134], [5.8676439328553185, 0.37652954794798221, 7.1781280138180996, 1.0965219471839127, 2.6102898809838297]], [[0.21669763624009786, 0.28219526649087978, -4.0349682653237409, -8.0229078470664916, -4.5844318045400962], [-4.1094537569956238, -3.3776238441985642, -0.64798590493254604, -0.99586334994328585, -3.1278996489085333], [-1.6962015139740547, -5.7141430105140945, 0.99347275899139564, -2.1151428182078167, -5.1211741722154214], [-0.46378084801084629, 0.72534968914725173, -4.3959523733472929, -1.671111311553771, -6.7767133869551444]], [[4.7287710846219646, 4.0204446999976806, 7.2909843836357382, -1.0300674405527248, -0.49395354240961264], [7.0248472019986981, -1.1231294186632361, 6.6610506742463151, -1.2797064205612259, 4.6500231677614199], [3.2580970880323701, 8.2921948761914415, 8.2263675863775578, -1.0712922837811236, 0.45424336745500238], [5.1108211727293025, -0.62332426738651492, -1.1158799640528843, 4.2964841372493208, 6.0429842459121126]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[-0.33580497990791436, 3.3118418875514806], [3.9535752895971203, 4.3396272888596528]], [[-2.9279747471563109, -2.2804955564137019], [1.3014155979506858, 3.8449143191665627]], [[-0.80984099681663935, -2.092225884015785], [-4.571044476680922, 2.3777961596996491]], [[2.2008046678919948, 1.559000593205031], [1.0891279330380392, -2.4240577486015349]], [[-1.7585762100468072, 4.288058550220704], [4.6839624033289198, -1.047285210579826]], [[-4.9822922648146664, -0.58118991047564883], [-4.3260395373367082, -2.8933761886995391]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.3284472679722903, 3.8817783246853406], [2.7663920616637334, -2.69955038298452]], [[0.72937242108648803, -1.624838586325601], [-2.8859155449927885, 3.6541219113553929]], [[0.42151497124054771, 2.095507523084966], [3.8829091390419492, -2.7123088855381541]], [[3.1475490774784944, 4.2626531435574702], [-0.75562502113725039, 1.5974276273387051]], [[0.34136613990677134, 4.45594158416365], [3.4145151226625341, -1.6623865831742632]], [[1.5725825482777971, 3.4553094240419959], [1.7590326450188476, -1.1640613537790037]]])+(1.-msk_arg1)*numpy.array([[[4.9116581425260026, -1.6857355787822681], [2.4747507163571676, 3.177298041503855]], [[-2.6553273682429737, -4.751000883459648], [4.2292833874606544, 0.60362957765049785]], [[-2.8961420141112795, 3.6873678446562099], [-4.1154272069873512, -0.92478556282234692]], [[2.0984497945338489, -4.3878269345012804], [-3.5317478459180487, -0.17493265929025448]], [[-3.8365243636935298, -0.71950120942087725], [1.6287209393629585, 2.3400603023831756]], [[3.1785921582280903, -0.74281019833495066], [-3.6377586053734379, 1.8345364813123606]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.6642522478802046, 7.1936202122368211], [6.7199673512608538, 1.6400769058751328]], [[-2.1986023260698229, -3.9053341427393029], [-1.5844999470421026, 7.4990362305219556]], [[-0.38832602557609164, 0.0032816390691809971], [-0.68813533763897272, -0.334512725838505]], [[5.3483537453704892, 5.8216537367625012], [0.33350291190078885, -0.82663012126282975]], [[-1.4172100701400359, 8.7440001343843541], [8.0984775259914539, -2.7096717937540893]], [[-3.4097097165368693, 2.8741195135663471], [-2.5670068923178606, -4.0574375424785423]]])+(1.-msk_ref)*numpy.array([[[4.5758531626180883, 1.6261063087692125], [6.4283260059542879, 7.5169253303635077]], [[-5.5833021153992846, -7.0314964398733499], [5.5306989854113402, 4.4485438968170605]], [[-3.7059830109279188, 1.5951419606404249], [-8.6864716836682732, 1.4530105968773022]], [[4.2992544624258437, -2.8288263412962493], [-2.4426199128800095, -2.5989904078917894]], [[-5.5951005737403374, 3.5685573407998268], [6.3126833426918783, 1.2927750918033496]], [[-1.8037001065865761, -1.3240001088105995], [-7.9637981427101465, -1.0588397073871785]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[-0.92741387926905716, -1.1963022751234287], [1.05521953972488, 3.3459931896930968]], [[-4.4634540391171118, 1.5473237382729526], [4.2838676735241634, 0.46082307744147766]]], [[[1.3896023524560412, -2.9529489045213109], [-4.1483706502264255, 1.9389745469931032]], [[-0.38225485788896041, -1.220953927674274], [2.3090447974652717, -3.0182685235693016]]], [[[2.6609285156123716, 0.61708055690885555], [3.0516643983079472, 1.2863656863794004]], [[-3.0072642881613088, 3.8563248998804873], [-3.251759852132178, -0.18817495641686133]]], [[[0.30440371461111315, -1.6200415440988634], [0.75385725681026283, 4.2571027777699495]], [[-3.6233558585631997, 2.9777255131821043], [2.0999248480952071, 1.1065341188121742]]], [[[-0.14005348653535243, -4.7895322486250773], [-2.397727078838241, -3.0708584283819151]], [[-1.8447913731691914, 4.9126276898259764], [-3.0837149796206664, 3.9210957025264044]]], [[[-3.6847355615400801, -3.3805007620183525], [-0.25229886731645479, -4.9611897660580349]], [[-2.7940310667497323, 4.5653587168826633], [0.20820899635344503, -2.9250796098190044]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.3926068210043532, 2.3572330640206847], [2.8873139514143435, 3.2693251810041488]], [[-2.3215380633456362, 1.4515485719343113], [-1.7214514255586657, -3.3215533128010053]], [[-3.8698341450774754, 4.6863185544146031], [-0.49421873293667673, 2.019021292095168]], [[1.7494795717427367, 4.7394947197867925], [2.9325545618229771, 4.1906144509026646]], [[-3.7687456352215962, 3.3643116225425516], [-0.43805974943670112, -2.3266760632993155]], [[-2.9270827358887219, 1.1365256376027055], [4.9844859170566753, 2.1457026486463615]]])+(1.-msk_arg1)*numpy.array([[[3.5733752660479485, 4.76233742838523], [4.127722989671625, 0.80436507349339248]], [[4.4122263260924441, 4.2165390409312469], [-4.5687195838427694, -3.0425367017233018]], [[4.2558774171972118, 4.413787678089049], [-1.8186137138154956, 4.6476706416570686]], [[0.16669923701945955, -0.6810245814126219], [-3.9280468606407637, 3.9594542007530382]], [[4.7682934591064647, 2.9747216391069475], [4.6371046493795749, -3.6632063460063691]], [[4.2366877516168238, -3.1994922955229468], [2.8615373866115457, -0.73909470099084995]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.465192941735296, 1.1963045458809245], [3.4124526037455647, 5.7032262537137814]], [[-1.5761400877027683, 4.4346376896872961], [7.5531928545283122, 3.7301482584456265]]], [[[-0.93193571088959493, -5.274486967866947], [-2.6968220782921142, 3.3905231189274145]], [[-2.1037062834476261, -2.9424053532329397], [-1.0125085153357336, -6.3398218363703069]]], [[[-1.2089056294651037, -3.2527535881686198], [7.7379829527225503, 5.9726842407940035]], [[-3.5014830210979855, 3.3621061669438106], [-1.23273856003701, 1.8308463356783067]]], [[[2.0538832863538499, 0.12943802764387335], [5.4933519765970553, 8.996597497556742]], [[-0.69080129674022261, 5.9102800750050815], [6.2905392989978717, 5.2971485697148388]]], [[[-3.9087991217569487, -8.5582778838466744], [0.96658454370431057, 0.29345319416063642]], [[-2.2828511226058925, 4.4745679403892753], [-5.410391042919982, 1.5944196392270888]]], [[[-6.611818297428802, -6.3075834979070748], [0.88422677028625074, -3.8246641284553293]], [[2.190454850306943, 9.5498446339393386], [2.3539116449998065, -0.77937696117264288]]]])+(1.-msk_ref)*numpy.array([[[[2.6459613867788914, 2.3770729909245198], [5.81755696811011, 8.1083306180783268]], [[-0.33573104944548682, 5.6750467279445775], [5.0882327470175559, 1.2651881509348701]]], [[[5.8018286785484854, 1.4592774215711333], [0.068168390704821391, 6.1555135879243501]], [[-4.9509744417317298, -5.7896735115170435], [-0.73349190425803013, -6.0608052252926035]]], [[[6.9168059328095834, 4.8729579741060673], [7.4654520763969963, 5.7001533644684494]], [[-4.8258780019768039, 2.0377111860649917], [1.3959107895248906, 4.4594956852402072]]], [[[0.4711029516305727, -1.4533423070794038], [0.072832675397640934, 3.5760781963573276]], [[-7.5514027192039634, -0.95032134745865937], [6.0593790488482453, 5.0659883195652124]]], [[[4.6282399725711123, -0.021238789518612577], [0.57699456026870655, -0.096136789274967605]], [[2.7923132762103835, 9.5497323392055513], [-6.7469213256270351, 0.25788935652003531]]], [[[0.55195219007674368, 0.85618698959847128], [-3.4517911628394016, -8.1606820615809816]], [[0.067506319861813324, 7.4268961034942089], [-0.53088570463740492, -3.6641743108098543]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_expandedData_rank4(self):
      arg0=Data(numpy.array([[[1.9849114148705631, 3.2211439512578561], [-4.1364507198131744, 4.5113993555715641]], [[1.4136330069746172, 3.3004856367073838], [-1.5713668246232357, 3.231586660100584]], [[4.4601484816354429, -1.2402487437931287], [0.24617763250851787, -1.7389192899256578]], [[0.41804879390227434, 4.0881261614384989], [-2.9235347734300312, 4.1155073674320555]], [[-2.6566447299190141, -2.4337416268703893], [-4.9733171928257827, 3.5492603853695783]], [[0.94828652502450073, 4.5520871107981069], [4.6387821375992093, 0.65063186568594311]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.0749925446210815, 1.825957719562048], [-2.0560365860495544, 4.9543038408546831]], [[0.36633943388951451, -4.5938726208198286], [2.895296113575804, -1.5393724248336991]]], [[[-3.489691156325275, 2.7022910859268983], [1.8652567475177486, 4.4127744479823576]], [[-2.2798384170752053, 1.8633141829395239], [-2.4601283819421571, -1.7755346792660509]]], [[[-3.0204590082297167, 3.3813286220524841], [3.6934424750621879, 1.3775607460617341]], [[-0.46330560723288983, 4.0232536598010213], [1.6764563682141649, 0.15133977470293747]]], [[[3.7304680931471008, -4.8196848469669558], [-1.5465912348672894, 2.3313639949605047]], [[2.4133794732766889, 3.3677315881377048], [-4.4216538604642883, -4.8518065727437607]]], [[[4.4017972765815454, -4.7435195423897625], [3.6882723804103215, -1.4510060429265992]], [[3.4046952655777911, 4.2263295349449397], [-4.1727248353560009, -4.808639707620344]]], [[[3.5455146789060539, 4.6147264393745484], [0.77127235639938796, -0.099614341383713167]], [[2.1377486792093237, 1.0391551551958678], [1.9520389982751771, -2.247564265760996]]]])+(1.-msk_arg1)*numpy.array([[[[3.3006456205282113, -2.2717423528035652], [-0.48768991167429476, -0.69050422680197254]], [[0.45186375408439616, -1.5791506391458499], [-4.4394667823272185, 0.7866459153808627]]], [[[-4.3982496767169774, 4.7943853962394769], [1.9368030012687374, -2.6979740451936229]], [[-1.0186890143445302, -4.9081257707917256], [2.23240248347249, -3.3067708633967841]]], [[[1.7639759765311078, -4.2295831744771997], [3.8745320880113372, 0.66772966762037989]], [[4.6674521003426737, 1.4950853046343315], [0.41004958621908827, -3.6213904373715944]]], [[[2.4561605918296987, -3.0345796381955861], [2.471044208221044, -4.5201819630384055]], [[3.6000842340742096, -0.55145176146563735], [-3.1202410785803316, -1.0446509367154242]]], [[[-1.8721290856553519, 3.8332321881474059], [0.40342581308881442, 0.89154357808448559]], [[-4.6648538114550666, 2.1257406365806739], [3.9031944877884683, -0.091555436602233087]]], [[[-3.0930165720359359, -2.8404092815168447], [-1.3952160548895023, 3.7286651365014247]], [[-1.0217551933525604, 3.5657937665891684], [-1.2543114396205981, -2.9621898637555253]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.0599039594916446, 3.8108691344326111], [1.1651073652083017, 8.1754477921125392]], [[-3.7701112859236598, -8.730323340633003], [7.4066954691473681, 2.9720269307378651]]], [[[-2.0760581493506578, 4.1159240929015155], [5.1657423842251324, 7.7132600846897414]], [[-3.851205241698441, 0.29194735831628815], [0.77145827815842694, 1.4560519808345331]]], [[[1.4396894734057262, 7.8414771036879269], [2.4531937312690593, 0.1373120022686054]], [[-0.21712797472437195, 4.2694312923095392], [-0.062462921711492925, -1.5875795152227203]]], [[[4.1485168870493752, -4.4016360530646814], [2.5415349265712095, 6.4194901563990037]], [[-0.51015530015334232, 0.44419681470767358], [-0.30614649303223285, -0.73629920531170523]]], [[[1.7451525466625313, -7.4001642723087766], [1.2545307535399322, -3.8847476697969885]], [[-1.5686219272479915, -0.74698765788084298], [-0.62346444998642259, -1.2593793222507657]]], [[[4.4938012039305546, 5.5630129643990491], [5.3233594671974949, 4.4524727694143937]], [[6.776530816808533, 5.6779372927950771], [2.6026708639611202, -1.5969324000750529]]]])+(1.-msk_ref)*numpy.array([[[[5.2855570353987744, -0.28683093793300207], [2.7334540395835614, 2.5306397244558836]], [[-3.6845869657287782, -5.7156013589590238], [0.071932573244345654, 5.2980452709524268]]], [[[-2.9846166697423602, 6.2080184032140941], [5.2372886379761212, 0.60251159151376088]], [[-2.5900558389677659, -6.4794925954149614], [5.4639891435730741, -0.075184203296200014]]], [[[6.2241244581665507, 0.23056530715824319], [2.6342833442182085, -0.57251907617274878]], [[4.9136297328511915, 1.7412629371428494], [-1.3288697037065695, -5.3603097272972526]]], [[[2.874209385731973, -2.6165308442933117], [6.5591703696595429, -0.43205580159990653]], [[0.67654946064417842, -3.4749865348956686], [0.99526628885172386, 3.0708564307166313]]], [[[-4.5287738155743664, 1.1765874582283917], [-2.0303158137815749, -1.5421980487859037]], [[-9.6381710042808493, -2.8475765562451087], [7.4524548731580467, 3.4577049487673452]]], [[[-2.1447300470114352, -1.892122756492344], [3.1568710559086046, 8.2807522472995316]], [[3.6170269442466489, 8.2045759041883777], [-0.60367957393465499, -2.3115579980695822]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-4.2653882745269307, -0.19011807242889134, 1.7181025750165198, 1.190085633858871], [2.9451920543803647, 1.7980173142431477, 0.29364166549559023, -0.97262568287803042], [1.4959845864044672, 3.7415840055586767, -0.21359289577701013, -0.65663469970851018]], [[-4.2157698474192955, 3.6743973172740105, -1.1849564734741449, 2.6280628766173884], [-0.31430090209559403, 4.8259413432266971, 2.0483088223179537, 3.5540514043328013], [0.38325049961345492, 2.5443350928199617, 4.7026415860071342, -0.77892386463710306]]], [[[1.5134536466964352, -2.1728145263404262, 3.0929361260530257, 1.7615866362567889], [2.7387971496669286, -2.6806622949642813, 2.0842644065225304, 1.1686015692018472], [-2.7691507908861466, -0.60888186198191274, 3.7499875619920164, 0.72576490137006111]], [[-3.0735664229766413, -0.50392379176395163, -0.95548159680480182, 1.0219723944244654], [-4.0444359548181268, -0.028762125369997449, -2.370573972625265, 1.6849280247602705], [-1.8337039900006413, 3.721578659627836, -1.4092470448957704, -1.8420560676723263]]], [[[-3.701307260432376, -0.0085047182274955446, 1.8151830067985575, -4.6094934493293858], [4.9307973426903757, 0.85706451514586668, 0.1100366180238801, -4.5040782430415929], [-0.7687084543578262, -1.4053962945383236, 3.2535659052537049, 2.4513015713480799]], [[-1.2999899910003343, -0.67669251520088913, 2.6469755978751071, 3.1221672072433595], [-4.487819173561082, 0.52427374215247902, 4.8528008379550407, 3.3600023234710115], [-4.3886238524809169, 3.5242623813659062, -2.3969907383482325, -0.29289074674398652]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.9696633990847863, 0.59286943569009765, -3.2396697618622703, -2.1551854563641095], [-2.7049731707604394, -0.081510110363971933, -1.0481443671920596, 1.419212944484423], [-1.4656471096438528, 4.0752922915106087, 2.5427224084127111, -3.5661286909447263]], [[-3.2714515031318445, 2.9455201268810507, -4.4208628524478009, 4.1309682545994733], [-1.1001232566461008, -3.2783998740236209, 3.3743583002420454, -2.1317765385113807], [-1.0826604627524716, 4.2240761622633087, -4.5188151627227722, -0.52065606278516618]]], [[[2.082334875204408, -3.273523489233753, -1.2288997060541726, -2.6415852542701677], [3.8856879451795194, 0.36608244406822621, -2.4204497477212206, 3.0303079985039574], [0.69908938604403925, -2.4342224868217963, -0.47152994845851115, 4.7298014462018205]], [[1.4363666516036275, -4.1543375042037542, -3.1551663406423316, -1.2064217603986451], [-0.86464907940327329, -4.5342303064682223, 3.3116186555759182, 3.6953273316026163], [-1.3884950107369196, -3.7131504751341451, -4.0364878066784895, -1.9730213148382383]]], [[[-1.9719808325923505, 3.8216457440242309, -1.8470402316730752, 1.9567280347513663], [-4.7284601271979687, -0.61461733065966495, 3.522712699348272, 3.752062693334075], [1.1821706325958568, 4.4600040857045862, -1.6359853484573961, -2.5120356400728938]], [[2.3351676579882081, -0.32965065690538786, 3.9299765247363752, 3.0453269854915774], [0.89967038581911751, -2.7020614794477771, 1.1611911786563969, 2.2639288529628141], [4.2826774566151347, 3.7464854005897212, -2.9121900053455607, -0.30057896525704919]]]])+(1.-msk_arg1)*numpy.array([[[[1.7582791887663518, 0.70034159343434244, 4.2376684983729636, -4.5574371105273359], [-3.2415329473603016, 0.12255597997259304, 4.6600830556204524, 4.5588735022177609], [1.1695078641847632, -2.1645631082103476, -1.3230294116783439, 2.3443530307228624]], [[3.1168754593295738, -4.7420733565988833, -0.015666678234051545, 0.18762282220759374], [3.6138650330933171, 2.5519814280360738, 2.7005539930058067, -3.4543049633429668], [-4.1443237870262379, -4.749022727489141, 1.7832840178041174, 4.8425254173334018]]], [[[2.6256516430612269, -3.4877246611623605, -0.43980528086948656, -4.650367572388542], [1.984748609418105, 3.6684678818115461, -3.262158655380599, 3.9191481247479665], [1.8330120281421411, -1.7986720202990805, -4.0852188627181327, 4.5205498284869563]], [[1.5988538624550586, 4.567130813087557, -3.047740155807086, -0.13724264515684936], [2.2650675790626549, -0.20893350313470549, 4.6519598803749158, 0.10569637298671175], [3.8853764287117798, 4.3805665900396775, -4.4178515171260324, -1.7589444283395972]]], [[[0.3091369485459694, 1.4077048823903588, 2.4900168710608757, 1.7850083823370113], [2.5009675100219955, 3.4854677924338553, 0.37768193945753392, 1.4870837156822958], [4.686476030869775, 1.1734715538208205, 4.7310952908500798, -0.99897315131515452]], [[4.3782034608621636, 0.35015608567962264, -0.31799328361873602, -0.80652505097382221], [-3.9566730663053407, -1.4897888404088686, 2.2698355421838814, 2.3805889944094076], [1.3203932308274187, -3.0575897472922819, 3.6551780215617384, -2.6853978492624875]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.2350516736117179, 0.40275136326120631, -1.5215671868457505, -0.96509982250523851], [0.24021888361992527, 1.7165072038791758, -0.75450270169646938, 0.44658726160639262], [0.030337476760614379, 7.8168762970692853, 2.329129512635701, -4.2227633906532365]], [[-7.48722135055114, 6.6199174441550612, -5.6058193259219458, 6.7590311312168616], [-1.4144241587416948, 1.5475414692030762, 5.4226671225599992, 1.4222748658214206], [-0.69940996313901671, 6.7684112550832705, 0.18382642328436205, -1.2995799274222692]]], [[[3.5957885219008432, -5.4463380155741792, 1.8640364199988531, -0.87999861801337875], [6.624485094846448, -2.3145798508960551, -0.33618534119869015, 4.1989095677058046], [-2.0700614048421073, -3.0431043488037091, 3.2784576135335053, 5.4555663475718816]], [[-1.6371997713730138, -4.6582612959677059, -4.1106479374471334, -0.18444936597417971], [-4.9090850342214001, -4.5629924318382198, 0.94104468295065313, 5.3802553563628868], [-3.2221990007375609, 0.0084281844936908712, -5.4457348515742598, -3.8150773825105646]]], [[[-5.6732880930247269, 3.8131410257967353, -0.031857224874517698, -2.6527654145780195], [0.20233721549240702, 0.24244718448620173, 3.6327493173721521, -0.75201554970751783], [0.41346217823803055, 3.0546077911662626, 1.6175805567963089, -0.060734068724813906]], [[1.0351776669878738, -1.006343172106277, 6.5769521226114822, 6.1674941927349369], [-3.5881487877419644, -2.1777877372952981, 6.0139920166114376, 5.6239311764338256], [-0.10594639586578225, 7.2707477819556274, -5.3091807436937932, -0.59346971200103571]]]])+(1.-msk_ref)*numpy.array([[[[-2.5071090857605789, 0.5102235210054511, 5.9557710733894833, -3.3673514766684649], [-0.29634089297993693, 1.9205732942157407, 4.9537247211160427, 3.5862478193397305], [2.6654924505892303, 1.5770208973483291, -1.536622307455354, 1.6877183310143522]], [[-1.0988943880897217, -1.0676760393248728, -1.2006231517081964, 2.8156856988249821], [3.2995641309977231, 7.3779227712627709, 4.7488628153237604, 0.099746440989834539], [-3.761073287412783, -2.2046876346691793, 6.4859256038112516, 4.0636015526962987]]], [[[4.1391052897576621, -5.6605391875027866, 2.6531308451835391, -2.8887809361317531], [4.7235457590850336, 0.98780558684726483, -1.1778942488580686, 5.0877496939498137], [-0.93613876274400543, -2.4075538822809932, -0.33523130072611629, 5.2463147298570174]], [[-1.4747125605215827, 4.0632070213236053, -4.0032217526118874, 0.88472974926761605], [-1.7793683757554719, -0.23769562850470294, 2.2813859077496508, 1.7906243977469822], [2.0516724387111385, 8.1021452496675135, -5.8270985620218028, -3.6010004960119235]]], [[[-3.3921703118864066, 1.3992001641628633, 4.3051998778594331, -2.8244850669923744], [7.4317648527123712, 4.342532307579722, 0.48771855748141402, -3.0169945273592971], [3.9177675765119488, -0.23192474071750313, 7.9846611961037848, 1.4523284200329254]], [[3.0782134698618293, -0.32653642952126649, 2.328982314256371, 2.3156421562695373], [-8.4444922398664222, -0.96551509825638959, 7.1226363801389221, 5.7405913178804191], [-3.0682306216534982, 0.46667263407362425, 1.2581872832135059, -2.978288596006474]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_Symbol_rank0(self):
      arg0=Data(1.62619089341,self.functionspace)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(4.34797816823)
      sub=res.substitute({arg1:s1})
      ref=Data(5.97416906164,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-3.5848273506017589, 0.85989344766652387]),self.functionspace)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(3.45697731442)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.1278500361805337, 4.3168707620877491]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_Symbol_rank1(self):
      arg0=Data(-2.11378848297,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-2.704831030888176, -2.2554365860385905])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-4.8186195138598338, -4.3692250690102483]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[0.81735249053982884, 2.4896184060170565, -0.10950513571852749, 0.94358638259397498, 3.6350983686696665], [-0.24797872080596495, -1.4106667551946925, 2.0325224103703103, 0.51270581889091993, 0.63190662859406022], [-1.349830253251195, -2.7063165798604318, -3.6975921644604002, 2.6828626290325186, -4.3712323245075204], [1.4755522392546698, -4.9291272749473789, -3.750767913166694, 0.5053512300569194, -0.75722031032456982]]),self.functionspace)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-2.65318265295)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-1.8358301624103754, -0.16356424693314775, -2.7626877886687318, -1.7095962703562293, 0.98191571571946223], [-2.9011613737561692, -4.0638494081448968, -0.62066024257989394, -2.1404768340592844, -2.0212760243561441], [-4.0030129062013993, -5.3594992328106361, -6.3507748174106045, 0.029679976082314319, -7.0244149774577247], [-1.1776304136955345, -7.5823099278975832, -6.4039505661168983, -2.1478314228932849, -3.4104029632747741]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_Symbol_rank2(self):
      arg0=Data(-3.97348778979,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[1.5057496840251279, 1.9643994884719094, -2.6434571269600893, 4.7532444475166091, 4.2167426537651487], [-1.3471403452648878, -0.25168328769878912, -1.6753257876994585, 0.99420607057040655, -0.045495636807503459], [-1.9858368859776676, -1.1095958367462067, -1.5859745792383553, -1.1641495835967963, -1.3565684626521834], [-4.5413421297681387, 4.3498784079675676, -0.69739334847258672, 0.62103386372269398, 2.6311450934584943]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-2.467738105765072, -2.0090883013182905, -6.6169449167502892, 0.77975665772640923, 0.24325486397494878], [-5.3206281350550881, -4.225171077488989, -5.6488135774896584, -2.9792817192197933, -4.0189834265977034], [-5.9593246757678671, -5.083083626536407, -5.5594623690285552, -5.1376373733869958, -5.3300562524423833], [-8.5148299195583377, 0.37639061817736774, -4.6708811382627866, -3.3524539260675059, -1.3423426963317056]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[-3.619537570779868, 0.95317613980372329], [2.7186652392889279, -1.8533409577282347]], [[-1.8815597462125289, -2.5336397016946313], [-2.0649654407284501, 3.2732556558843733]], [[-2.1238739201728585, -1.29366030684956], [-1.2534338391973132, 1.8900475196591726]], [[-0.83227930922824545, -2.2819201021438449], [3.4821218091036936, 1.3931490911848856]], [[-3.5545816552232457, 1.7309960005136009], [-3.8174132329513477, 0.44978087383837906]], [[4.3885344863591342, -4.9073478485850961], [-4.4016358046881994, -2.7116054218088959]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(2.26840824113)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-1.3511293296474189, 3.2215843809361724], [4.987073480421377, 0.41506728340421439]], [[0.38684849491992024, -0.26523146056218216], [0.20344280040399898, 5.5416638970168224]], [[0.14453432095959062, 0.97474793428288908], [1.0149744019351359, 4.1584557607916217]], [[1.4361289319042037, -0.013511861011395787], [5.7505300502361427, 3.6615573323173347]], [[-1.2861734140907966, 3.99940424164605], [-1.5490049918188986, 2.7181891149708282]], [[6.6569427274915833, -2.638939607452647], [-2.1332275635557503, -0.44319718067644676]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_Symbol_rank3(self):
      arg0=Data(3.43954827945,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[0.744924266256759, 3.747599273119242], [2.3406925602745785, -4.4790099863506452]], [[-4.0520813328214764, -0.12612302545489928], [3.5667926288170193, 1.8261568871814067]], [[3.8830346074491935, 2.8449298232984663], [-4.1986869695342941, 3.8062354245590022]], [[-0.91303386700839617, -2.0602438003866688], [-2.956524222865732, -3.5310096117817533]], [[4.2880245630717209, -2.086211935800264], [3.2905946681758422, -2.1019763630992108]], [[4.4994748457257785, -3.8664492368900607], [3.9526054186791377, 3.3699420037550976]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[4.1844725457059821, 7.1871475525684652], [5.7802408397238016, -1.039461706901422]], [[-0.61253305337225328, 3.3134252539943239], [7.0063409082662425, 5.2657051666306298]], [[7.3225828868984166, 6.2844781027476895], [-0.75913869008507096, 7.2457837040082254]], [[2.526514412440827, 1.3793044790625544], [0.48302405658349112, -0.091461332332530176]], [[7.727572842520944, 1.3533363436489592], [6.7301429476250654, 1.3375719163500124]], [[7.9390231251750016, -0.42690095744083756], [7.3921536981283609, 6.8094902832043207]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[1.5480537598602817, 2.6989905732859665, 4.9862866505355132, -2.4648848731236903], [-4.9180662797474195, 3.1749435641310626, -4.1588701391061029, -1.169501977908082], [-2.6688813345787543, -4.3458177384346994, -3.5883534144224516, 0.92007194934034686]], [[-1.1466722577611774, 3.3768167806014535, -3.8571249698522561, -4.4940494515788467], [2.5747333313577982, -4.9259402900971967, -3.3456945798318003, 1.223041849342728], [-1.4508925920527371, 2.0201320119898636, 1.4170771820495878, -3.4263741108612211]]], [[[-1.1200927687276918, 4.6568647576039233, -3.9344650330915645, 0.50896625677001062], [1.0803240832262162, 4.204677978575674, -2.9207923097315804, 3.3367276563764143], [1.4418170405561126, -0.29143865299786409, 1.8779646581615728, 0.24755398559099806]], [[-1.1689679196079261, 4.9510975183997363, -1.9911929627082579, 0.98203672205364434], [-3.613744133365282, 0.39213748953285776, 1.8719656511219371, 3.147865318892709], [-1.6313600840387856, 3.7406217554314143, 0.88857408382621728, -4.6155653639791083]]], [[[4.3150198820227903, -1.9520743303462993, 0.65437407376151757, -3.2253090075772795], [-2.8171763370771519, 3.9453999811658811, -1.6879139811151123, 2.8969442654915998], [-1.0957612362058775, -1.3023130372892333, -2.2866429004504196, 2.9807915404231569]], [[2.4288925030801343, -3.1248919610324299, -2.5718850730062148, 2.4034395838326814], [-0.84100000170375822, 1.1813631335138064, 0.19176213517463214, -2.7296580481279986], [0.76444851917345957, -0.047396779753933416, -0.75755175802965002, -1.4619910798393621]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(0.358101549443)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[1.9061553093031725, 3.0570921227288572, 5.344388199978404, -2.1067833236807996], [-4.5599647303045288, 3.5330451135739533, -3.8007685896632122, -0.81140042846519123], [-2.3107797851358636, -3.9877161889918087, -3.2302518649795608, 1.2781734987832376]], [[-0.78857070831828668, 3.7349183300443443, -3.4990234204093653, -4.135947902135956], [2.932834880800689, -4.567838740654306, -2.9875930303889096, 1.5811433987856187], [-1.0927910426098464, 2.3782335614327543, 1.7751787314924785, -3.0682725614183304]]], [[[-0.76199121928480107, 5.014966307046814, -3.5763634836486737, 0.86706780621290136], [1.4384256326691069, 4.5627795280185648, -2.5626907602886897, 3.6948292058193051], [1.7999185899990033, 0.066662896445026654, 2.2360662076044635, 0.6056555350338888]], [[-0.81086637016503538, 5.3091990678426271, -1.6330914132653671, 1.3401382714965351], [-3.2556425839223913, 0.75023903897574851, 2.2300672005648279, 3.5059668683355998], [-1.2732585345958949, 4.098723304874305, 1.246675633269108, -4.2574638145362176]]], [[[4.673121431465681, -1.5939727809034085, 1.0124756232044083, -2.8672074581343887], [-2.4590747876342611, 4.3035015306087718, -1.3298124316722215, 3.2550458149344905], [-0.73765968676298677, -0.94421148784634257, -1.9285413510075289, 3.3388930898660476]], [[2.786994052523025, -2.7667904115895392, -2.2137835235633241, 2.7615411332755722], [-0.48289845226086747, 1.5394646829566971, 0.54986368461752289, -2.3715564986851079], [1.1225500686163503, 0.31070476968895733, -0.39945020858675928, -1.1038895303964713]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank0_Symbol_rank4(self):
      arg0=Data(4.87397662763,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[0.08071953612048155, -1.0855878180733516, -1.0976892535422165, -4.258343844184644], [2.4530993180964824, -3.7011278538188686, -1.2458179740356998, -3.5733048245392851], [-4.7231696210181129, 2.1261672930405862, 3.771482151522946, -3.7307862027304206]], [[-1.6001041770645963, 2.3585967403133345, 3.9613599059000038, -4.766021890493696], [3.9713034496223916, 0.23951304934830109, 3.0965508833747037, 1.5791963743882578], [1.5159122835560801, -0.58979393589073759, -0.52663804903033107, -0.47334834064147735]]], [[[-2.7718982620481492, 4.8774508864412205, 1.7302787883489259, -4.5386290512185532], [-2.657576466292082, -4.0280559938488816, -4.7803645997117226, -3.1279575266969442], [0.12984150743368517, 2.4053926531451744, -0.64553900758624128, -1.8359997019933196]], [[4.8727825989857827, -4.965663919524788, 0.60321899821623415, -0.043444696145964912], [-0.76953095067224631, 4.4667608506802132, -4.1585814157263226, -4.8195816233915698], [1.8879519185229601, -1.6511340795408955, -0.72388446592755074, 4.3319327799060385]]], [[[4.4110151197367458, -0.77121941212784595, -2.9694715015876669, -4.8761503682104088], [-0.85090516028799801, 4.6173892198426429, -1.4768799355442761, 0.82594922288101635], [-2.4479146973250399, 0.89837393905236951, -4.2136386742650211, -4.653750482238209]], [[4.3678615306243724, 1.7444360887692429, -1.6605403763313831, 3.7200563990904651], [2.0754494842071747, -4.4403351818252288, -1.4750670246299826, 3.9928743947720768], [3.6947147686934638, -4.8390397698859866, 3.4542032831630536, -1.2940072964662774]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[4.9546961637504889, 3.7883888095566558, 3.7762873740877909, 0.61563278344536343], [7.3270759457264898, 1.1728487738111388, 3.6281586535943076, 1.3006718030907223], [0.1508070066118945, 7.0001439206705935, 8.6454587791529534, 1.1431904248995868]], [[3.2738724505654111, 7.2325733679433419, 8.8353365335300111, 0.10795473713631143], [8.845280077252399, 5.1134896769783085, 7.9705275110047111, 6.4531730020182652], [6.3898889111860875, 4.2841826917392698, 4.3473385785996763, 4.40062828698853]]], [[[2.1020783655818582, 9.7514275140712279, 6.6042554159789333, 0.33534757641145418], [2.2164001613379254, 0.84592063378112581, 0.093612027918284824, 1.7460191009330632], [5.0038181350636926, 7.2793692807751817, 4.2284376200437661, 3.0379769256366878]], [[9.7467592266157901, -0.091687291894780643, 5.4771956258462415, 4.8305319314840425], [4.1044456769577611, 9.3407374783102206, 0.71539521190368482, 0.054395004238437572], [6.7619285461529675, 3.2228425480891119, 4.1500921617024567, 9.2059094075360459]]], [[[9.2849917473667531, 4.1027572155021614, 1.9045051260423405, -0.0021737405804014287], [4.0230714673420094, 9.4913658474726503, 3.3970966920857313, 5.6999258505110237], [2.4260619303049675, 5.7723505666823769, 0.66033795336498624, 0.22022614539179841]], [[9.2418381582543798, 6.6184127163992503, 3.2134362512986243, 8.5940330267204725], [6.9494261118371821, 0.43364144580477859, 3.3989096030000248, 8.8668510224020842], [8.5686913963234712, 0.034936857744020777, 8.328179910793061, 3.57996933116373]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([4.6509931813330248, -2.2336788465051916]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([1.3862941764500114, -4.7154351860489623])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([6.0372873577830362, -6.9491140325541538]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[-3.6524345055019247, 1.4505464052958148], [1.7822938898485994, -3.9247060741816475]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([2.2514633020653481, -0.53096594853848078])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-1.4009712034365767, 3.7020097073611629], [1.2513279413101186, -4.4556720227201279]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_Symbol_rank2(self):
      arg0=Data(numpy.array([4.6805387455600194, 0.59265699129202609]),self.functionspace)
      arg1=Symbol(shape=(2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[-1.2662518698296688, 1.1239519355583223], [0.43027058471475055, 2.625095192013398]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[3.4142868757303506, 5.8044906811183417], [1.0229275760067766, 3.2177521833054241]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_Symbol_rank1(self):
      arg0=Data(numpy.array([[[4.2810206466522214, -4.9278848954509211, -4.2957989878314065, 4.2258416850511544, -1.820252865783849], [-0.45883878898724451, 0.47839968241574748, 0.10929960633506575, -1.436443011509013, 0.28506133035292702], [3.6833176707220332, 0.19691034363947946, 3.1318023633859546, -1.4674028728273592, -4.0207801243537773], [4.4123203806468894, 2.9716512896656262, -4.0039176845315714, -2.4792243226910013, 1.0362783500189607]], [[4.1695039189180676, 0.70866690422756928, -2.3003238376056956, -2.7830205657000349, -1.2862344601918552], [0.49740787276060594, -0.79693313500593188, 2.3074558814932029, 1.8681539567152727, 1.8004047036354267], [1.9935216163056966, 1.3872884796929714, 1.210385126765356, 2.4403504328199581, 0.97462651967085279], [-3.6188761177267379, -4.7566935696844617, 4.1346182247342504, 3.9930209523833415, 4.3183741393032946]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([3.2819707018145508, 1.9652777538173538])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[7.5629913484667721, -1.6459141936363704, -1.0138282860168557, 7.5078123868657052, 1.4617178360307017], [2.8231319128273062, 3.7603703842302982, 3.3912703081496165, 1.8455276903055378, 3.5670320321674778], [6.9652883725365839, 3.4788810454540302, 6.4137730652005054, 1.8145678289871916, -0.73880942253922655], [7.6942910824614401, 6.2536219914801769, -0.72194698271702062, 0.8027463791235494, 4.3182490518335115]], [[6.1347816727354214, 2.6739446580449231, -0.33504608378834178, -0.81774281188268105, 0.67904329362549865], [2.4626856265779598, 1.1683446188114219, 4.2727336353105567, 3.8334317105326265, 3.7656824574527805], [3.9587993701230504, 3.3525662335103252, 3.1756628805827098, 4.4056281866373119, 2.9399042734882066], [-1.6535983639093841, -2.7914158158671079, 6.0998959785516043, 5.9582987062006953, 6.2836518931206484]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_Symbol_rank3(self):
      arg0=Data(numpy.array([1.7211967657765959, 3.5362408080433703]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[-1.0623100294506549, 0.8591936944799059, -1.1518778636079672, -3.5685693747904246, -0.22149736258070085], [0.61671208546062761, -3.144144739030954, 3.6757264127360685, -1.1957513251207064, 0.57293562527112307], [-1.8605763439593748, -3.1786347150017447, 0.93203314894504175, -4.9738861401663312, -0.036562596456245267], [-3.8984663260348573, 4.6282116976601984, -4.5423764334247396, -2.6376631705551401, 1.0488971375249099]], [[-2.5644962284320605, 2.5768061731493352, -4.4669916324057226, -1.0464058806294672, 4.0874472958347283], [1.1911237138076594, -2.7760899315755028, -0.64238759089541464, -1.0624788344377869, -3.6679709783650596], [-0.67248349090046844, 2.3143655272348216, -4.6803213604335836, 1.5633559658340301, 2.8129521829502702], [-0.95815623844799358, 2.8691353680524267, -0.27732766010563736, 0.51090286228464166, -4.4798664709164946]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.65888673632594097, 2.5803904602565018, 0.56931890216862868, -1.8473726090138287, 1.499699403195895], [2.3379088512372235, -1.4229479732543582, 5.3969231785126643, 0.5254454406558895, 2.2941323910477189], [-0.13937957818277891, -1.4574379492251488, 2.6532299147216376, -3.2526893743897354, 1.6846341693203506], [-2.1772695602582615, 6.3494084634367942, -2.8211796676481438, -0.91646640477854424, 2.7700939033015057]], [[0.97174457961130978, 6.1130469811927055, -0.93075082436235235, 2.4898349274139031, 7.6236881038780986], [4.7273645218510296, 0.7601508764678675, 2.8938532171479556, 2.4737619736055834, -0.13173017032168932], [2.8637573171429018, 5.8506063352781918, -1.1440805523902133, 5.0995967738774004, 6.3491929909936404], [2.5780845695953767, 6.405376176095797, 3.2589131479377329, 4.0471436703280119, -0.94362566287312433]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_Symbol_rank1(self):
      arg0=Data(numpy.array([[[[3.2213523520847236, -0.0056960112065551627], [-3.1239827258309463, -4.3401409055949074]], [[1.6712963986792593, 3.443735247248874], [0.37974868338618784, 2.9403696445447967]], [[2.6976506537104408, -2.3825758661671923], [1.5164627367063224, -3.2879402860115214]], [[2.6730820986574964, 2.1069036664861418], [-2.4328402728483578, 0.030268947291274273]], [[1.3250591502919731, -2.3981466931921624], [0.25985202489557135, -0.18493761924186369]], [[2.4332774038261693, -1.1445604522596051], [0.34437902937516274, -3.0944966454978564]]], [[[-0.85945093471792866, 3.6356568432397456], [4.2533117220990917, 3.5678894105664689]], [[-2.5708624105178144, -3.8753853366054014], [-1.545715966362462, -1.4891457876043992]], [[-4.1764701333583298, 4.9193595455176933], [-1.9465380194405757, -2.4459724854556306]], [[1.3546555978155626, -4.9899775886840638], [4.8275235932443366, 1.4652510460378902]], [[1.5736815482611712, 0.26246122164938335], [2.5182484443127535, -0.65141083199738503]], [[-2.7314484212512635, -0.0606080935343698], [2.8975222742630571, 4.3590971001051759]]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-0.67931280944646666, -4.7809929342462993])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[2.5420395426382569, -0.68500882065302182], [-3.8032955352774129, -5.0194537150413741]], [[0.99198358923279262, 2.7644224378024074], [-0.29956412606027882, 2.26105683509833]], [[2.0183378442639741, -3.061888675613659], [0.83714992725985571, -3.9672530954579881]], [[1.9937692892110297, 1.4275908570396751], [-3.1121530822948245, -0.64904386215519239]], [[0.64574634084550642, -3.0774595026386291], [-0.4194607845508953, -0.86425042868833035]], [[1.7539645943797026, -1.8238732617060718], [-0.33493378007130392, -3.7738094549443231]]], [[[-5.640443868964228, -1.1453360910065538], [-0.52768121214720765, -1.2131035236798304]], [[-7.3518553447641137, -8.6563782708517003], [-6.3267089006087609, -6.2701387218506985]], [[-8.9574630676046283, 0.13836661127139394], [-6.727530953686875, -7.22696541970193]], [[-3.4263373364307368, -9.7709705229303623], [0.046530658998037211, -3.3157418882084091]], [[-3.2073113859851281, -4.518531712596916], [-2.2627444899335458, -5.4324037662436844]], [[-7.5124413554975629, -4.8416010277806691], [-1.8834706599832423, -0.42189583414112342]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank1_Symbol_rank4(self):
      arg0=Data(numpy.array([2.0373577555211266, -0.51903859192019652]),self.functionspace)
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[1.4628504821471253, 1.072140955741169], [3.8302000831787808, 2.8965064449369642]], [[2.6839878531520212, -1.684755339127916], [-1.2167127005326051, 4.141703453242453]], [[-3.2614239069920101, -3.3928730826535181], [4.7679614741027105, -4.5020567291606985]], [[4.7054432520457095, 0.1095025862789587], [1.766896662275288, 1.5769138170307997]], [[-3.5101148845064936, -3.5221386049097267], [-4.8499258085220021, 3.323597225895444]], [[2.0599146432649018, -4.6020818632685572], [1.5793256539812948, -4.0132757478755288]]], [[[-4.7491947482813774, 1.4634275949158049], [2.8923616805741297, 4.837962369129901]], [[-4.818571285988364, 0.16409996572068763], [-2.9690519393090877, 0.98184958415790469]], [[1.0781926773950019, -4.5873533288824309], [1.8563798152322075, -4.7958441014892443]], [[-4.5207295582161056, 2.4252982172595825], [-4.1056417195999177, -3.7932637965687297]], [[-2.7301831025293954, -3.9641752650174986], [-2.7548324758685903, 1.1391632523102482]], [[3.5072393165274942, -0.29245799815967732], [4.1350639219803416, 1.3429246884634241]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[3.500208237668252, 3.1094987112622956], [5.8675578386999074, 4.9338642004580908]], [[4.7213456086731478, 0.35260241639321066], [0.82064505498852158, 6.1790612087635797]], [[-1.2240661514708835, -1.3555153271323914], [6.8053192296238372, -2.4646989736395719]], [[6.7428010075668361, 2.1468603418000853], [3.8042544177964146, 3.6142715725519263]], [[-1.4727571289853669, -1.4847808493886001], [-2.8125680530008754, 5.3609549814165707]], [[4.0972723987860284, -2.5647241077474305], [3.6166834095024214, -1.9759179923544021]]], [[[-5.268233340201574, 0.94438900299560835], [2.3733230886539332, 4.3189237772097044]], [[-5.3376098779085606, -0.35493862619950889], [-3.4880905312292843, 0.46281099223770816]], [[0.55915408547480538, -5.1063919208026274], [1.337341223312011, -5.3148826934094409]], [[-5.0397681501363021, 1.906259625339386], [-4.6246803115201143, -4.3123023884889262]], [[-3.249221694449592, -4.4832138569376951], [-3.2738710677887868, 0.62012466039005165]], [[2.9882007246072977, -0.81149659007987385], [3.616025330060145, 0.82388609654322753]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-0.014492981508123393, 2.4495755941947719, -4.984431977391532, -3.3371924573143339, 4.353958398896383], [1.1449186045537676, 3.617728829751572, 0.69365768897813496, -4.8062105568668878, 2.6349940429096383], [3.9194004644035658, -3.0647400150964477, 2.4974501511771212, 3.5500020472847034, 4.3864640323370878], [1.6782622548501882, 3.5798429069585946, 2.7157306307637601, 1.9637631306829659, -4.0653382326693581]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[4.4853381873973461, 1.9057323142665918, -4.179111702493187, 3.5242010025280983, 1.3959082116478356], [-1.952678890031978, 2.3268678197543871, -3.0269120701836414, -1.8520941308414973, 3.1734117343725892], [2.6830871626453412, -3.6590203748383776, -0.36193606538996814, 4.6473164657037298, 0.94930729353167731], [-0.43786928725542484, -4.7509128620199013, -4.7051239265000069, 2.6726446783444713, -3.9828755203971991]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[4.4708452058892227, 4.3553079084613637, -9.1635436798847181, 0.18700854521376442, 5.7498666105442187], [-0.80776028547821044, 5.944596649505959, -2.3332543812055064, -6.6583046877083856, 5.8084057772822275], [6.602487627048907, -6.7237603899348253, 2.1355140857871531, 8.1973185129884332, 5.3357713258687651], [1.2403929675947634, -1.1710699550613066, -1.9893932957362468, 4.6364078090274372, -8.0482137530665572]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_Symbol_rank2(self):
      arg0=Data(numpy.array([[[2.4984466531793217, 3.1107617407304602], [-1.4312607626063456, 4.5462004973198074], [-1.5192372726813108, -4.3875826371952762], [4.7428596950673167, 4.0550486969250699], [-3.1172072749161259, 4.2693539655965544]], [[3.274910222132922, -4.0549253121138964], [-1.3584782541574016, 2.6969541604002059], [-0.94376317435489732, 0.33485874956040274], [-3.1807635049840677, -0.27300808958485501], [0.42407485966910574, -1.2117399946955665]], [[0.022966306130513203, 4.5044429613175652], [1.9662922464112285, -2.7416745701375556], [-1.5700627644190401, -2.6513391590145408], [-2.6062099929407259, 1.4976124872838845], [2.9308368606827635, -1.0658765269480028]], [[4.8140946079298637, 2.0736523508182838], [0.19398360817708493, 0.66858660219814681], [-2.1303142491492153, 0.11763322623920569], [-2.8856198990764015, -4.1061918797515933], [2.5120804788129503, -2.5367603575605422]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[-0.31693368463895677, -0.18554380695197281, 2.4405339819040108, 3.7346953101036622, 2.8147975884878926], [-3.804571632870446, -2.465755536897889, -1.6985927261455558, -1.9346472249324478, 2.6562740610113567], [3.7526157616807421, 1.6444784338210887, 1.6247880318399277, -2.0377504012454226, 1.1489109940107607], [-4.2352245158923463, -1.8828580030052033, 3.2457202295509653, 0.32129826808816819, 2.5731425002303148]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[2.1815129685403649, 2.7938280560915034], [-1.6168045695583184, 4.3606566903678345], [0.92129670922270002, -1.9470486552912654], [8.4775550051709789, 7.789744007028732], [-0.30240968642823329, 7.0841515540844471]], [[-0.52966141073752393, -7.8594969449843424], [-3.8242337910552906, 0.2311986235023169], [-2.6423559005004531, -1.3637339765851531], [-5.1154107299165155, -2.2076553145173028], [3.0803489206804624, 1.4445340663157902]], [[3.7755820678112553, 8.2570587229983072], [3.6107706802323172, -1.0971961363164668], [0.054725267420887569, -1.0265511271746131], [-4.6439603941861485, -0.5401379139615381], [4.0797478546935242, 0.083034467062757855]], [[0.57887009203751738, -2.1615721650740625], [-1.6888743948281184, -1.2142714008070565], [1.11540598040175, 3.363353455790171], [-2.5643216309882333, -3.7848936116634251], [5.085222979043265, 0.036382142669772577]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_Symbol_rank3(self):
      arg0=Data(numpy.array([[-0.19904864313926129, 2.0800736693642783, -2.0855871903220811, -1.5803528255891997, 3.5953452391349714], [-3.1925430878096051, 1.7509728808017266, 0.48547484199234692, 3.3476819729712659, 4.6729532929658006], [-2.698422964411693, 1.0831386333213793, 2.8685216104643203, -4.8655983374682048, 2.1480853562794096], [-3.4900815595755197, 0.40903016425257022, 4.057253366421067, 1.2494644754275299, -4.9358209523160079]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-2.6511253307001867, -1.2722089039623494], [0.76560479547608384, -1.9457210539684278], [3.0056465915112476, -4.4289604913589153], [-0.82607154754509882, 2.1697335297112339], [-2.8673441571408818, -3.1417628083498816]], [[4.2500207212073118, 4.3144170111988878], [2.6407399249675549, -3.0086348958654154], [2.3998838399953204, -0.52941387436342602], [2.5772625203158643, -2.5344659432656313], [-1.8394984021498271, 1.924255067686075]], [[-3.4575854073083381, 4.5662481541711983], [4.2162838642620919, -3.9528927006760304], [4.9064787993051304, -1.0903125541261316], [0.6195820034147248, -1.0819837453698353], [4.0426738963392488, -2.8961808000588052]], [[3.9928585878565244, 4.9392220008636762], [0.27917127338352898, 1.4788051050931683], [3.7728826807850613, 1.314638767962709], [3.7058242360038633, -3.9040815018747144], [-2.4955761809495924, -4.1025794718758055]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-2.850173973839448, -1.4712575471016107], [2.8456784648403621, 0.13435261539585053], [0.92005940118916651, -6.5145476816809964], [-2.4064243731342985, 0.58938070412203425], [0.72800108199408964, 0.45358243078508975]], [[1.0574776333977067, 1.1218739233892827], [4.3917128057692816, -1.2576620150636888], [2.8853586819876673, -0.043939032371079101], [5.9249444932871302, 0.81321602970563456], [2.8334548908159736, 6.5972083606518757]], [[-6.1560083717200307, 1.8678251897595053], [5.2994224975834712, -2.8697540673546511], [7.7750004097694507, 1.7782090563381887], [-4.24601633405348, -5.9475820828380401], [6.1907592526186583, -0.74809544377939563]], [[0.50277702828100468, 1.4491404412881566], [0.6882014376360992, 1.8878352693457385], [7.8301360472061283, 5.371892134383776], [4.9552887114313933, -2.6546170264471844], [-7.4313971332655999, -9.0384004241918134]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[1.2510128330743262, 4.5800931159408549, 0.92299049365884045, -4.7075558385147973, -3.4636845994270873], [-0.99179681936184316, 0.48172083672443833, -0.20022779305290861, -2.2246248431164171, -4.1183773772355492], [1.5110622324926224, 1.9181705440953998, -3.6236503567183553, 4.7883559825588797, 0.25287188431696439], [0.011569322025324347, 3.9464341070468016, 4.6372771408089939, -3.6907121112805665, -4.0296839305334329]], [[-0.011472107358781791, 2.1437742794594064, -4.4217780339552544, 1.3829754028734698, 4.6090389437871853], [-2.4627662748832089, -1.6285211476800376, -4.6299796603626087, 4.423773607519859, -1.1286838182831413], [-2.4561455840238287, -1.3448821752938001, -4.6527046182987162, 3.6655179361814731, -2.2107149279844163], [4.4984226518408637, -2.3170529152420718, -4.8917293744566779, -3.225485412267731, 3.2311572390566088]], [[-2.9770627496478328, 4.8245699374757898, 4.1706215553674948, 2.0914458182889959, 0.77276416434560868], [3.0152424829536137, 1.4272421001220517, -0.50811325641827843, 0.54090052189036086, -2.55947985716247], [1.8004328617331486, 3.6347101079103901, 0.17744913071876844, -1.5711681919755582, -4.5927672980398695], [1.375493040714467, -2.5619516358005887, -2.8700823901274575, 2.8307802964207394, 3.9600879243891089]], [[-0.48114600177784261, -2.2899063157205548, -1.8366097970272399, 2.2494946441778021, 4.163874156090813], [-4.6253679881356273, 2.2545927321142454, 3.4936960329381463, -3.7503710553196279, -3.6018989627414957], [-3.4650416072823056, -3.4300403380481139, 1.3878460325341093, -2.0465678543553745, 4.8678387176706153], [1.2014532588746762, 0.19990129041994642, -4.8315417134156426, 4.0069945704847747, 2.5295835106751028]], [[0.2870714614632206, 2.967626715828068, 4.9279254735117615, 3.1503125677824695, -3.5458424613874548], [0.44822230767058802, 4.6243588315169699, 0.016690938224562579, -1.4721643204932966, 4.6243945669604045], [-1.4579391227053531, -4.9880962626165628, 4.0221820718186532, 4.8472198391708758, 3.8542402501240893], [-1.0830767198385649, 0.75028284291245395, 0.86057936584070038, 0.9703551708907554, 1.9464444968549994]]], [[[3.2977410598503276, 1.4506985221488851, 3.218376249882489, -0.11268720844003077, -0.60202297307046315], [-2.4917080802934333, 2.2073896930968484, -0.93910870662211821, -0.88015913007306423, 4.2481298681712492], [-2.2097495856725957, 1.374507227456502, -2.480950475428445, -2.211330761254755, -3.2543680994979338], [-2.7893545447099157, -3.0277600980090802, 4.8861222707036678, -4.1258677102675803, -0.23715303485721684]], [[-1.1062363347096973, 1.2216918280599565, -0.046357587315672077, 4.1141003799589058, -1.6304149134831292], [3.5110020933475852, -0.42118511941633585, 4.9309342520566162, 1.51248893250688, -3.8881343945646165], [3.6891260710678342, 3.6031201596695457, 2.2768277434338771, 3.5129478187984429, -4.8656860745937225], [3.3072539634917213, 3.0965591333682347, -3.9698635888516409, 1.9839445447685033, 4.2809912400963768]], [[-4.8867605935686553, 0.92723522034668182, 4.2454768156313367, -3.8053042098122161, 2.182907959069567], [-0.81786445764374527, 2.6192252215311216, -0.45118217290686502, 4.7515126698470471, -3.9876200540483921], [1.3082050736777973, -2.28471109283264, 2.6765682763065364, -2.5866745793534482, 2.7042059713504996], [-0.82039183193188503, 2.8493593222850429, 3.3162702342485542, -0.75346326368917449, 0.87176231631931955]], [[2.459405878506371, 0.19018396058674192, 2.8767122404804688, -4.9087376144590404, -0.63414349881023124], [-1.7983324654336452, 0.75930728622504517, -3.1920007891687052, -4.9199812822341551, 1.0116679676608484], [4.7218958258857207, -1.2014757204399662, 4.0927879558177036, 0.82807462607038307, 0.27169508658165142], [-2.3598169116858978, 4.4876957875607282, -0.36961715868443079, -2.1131691842078726, 3.4708490887075545]], [[-1.0198476025272374, 4.4401419448280386, 2.5732910284198072, 2.2204803180284793, -1.7040449505991715], [-4.2909427150463308, 4.4332405351823674, -0.64491254118205354, -1.4205006897713632, -1.6164883758695323], [2.6982004011772371, 3.1091725309933054, -3.609388703091494, 3.4065584670094768, -1.1604630248577772], [-3.064017091044712, -2.8533646346298069, 4.6731896955062417, -2.3451793522524476, -4.869282996030952]]], [[[2.5786080866688508, 3.3356897089705111, -3.1756088668573588, 2.0897872310397663, -2.576473136507583], [4.8312438928509156, 3.8104719524832635, -1.3783889301955887, 1.5241093891761768, -3.6628151233081843], [4.9787455834711452, -1.758823529462008, 4.4698372259907408, -1.5567942296888235, 0.49489393628940981], [4.1247630262887256, 3.8644923701970466, 2.4638627401990547, -2.6280269186365315, -4.5288182445479164]], [[-2.5042399214260138, 1.9914055137886058, -3.7368647481632933, 4.4443696821738019, -4.1789618592064652], [4.0156962483252663, 0.63767761386565347, -0.81014919641960503, 4.4543546995968768, 2.5637308836139363], [2.7112832174787682, -0.03462766532695305, 4.2869624068846441, 3.278550977518579, 4.770469465347345], [-3.2362966988485775, -0.77907299626872284, -0.93861207859277851, -4.5757697043948786, 1.7509699885823977]], [[-0.16071531603778233, 4.9836558095665424, 1.7466226696603062, 3.8265601710865678, 1.5000822339651645], [-3.286666470264783, -4.4104504192110845, -1.7763328758101227, 4.5528924838925615, -0.70252229848288383], [-2.6861794573773459, -4.2021994060330146, 4.4159676407769606, -0.19709287328115632, 4.8485850433110596], [1.1018932596918471, 1.1725031816582412, -2.1974611440616032, 2.8881065649839011, 3.426984424034627]], [[3.5195877131787263, 1.5661881166705784, 1.6302718051156635, 0.68724576113825009, 3.8881924724596857], [3.9618432235851682, -3.9139185366037976, -2.4223935527275464, 0.36619178707753797, 2.1361325270338245], [0.89857926204229521, -2.2923563743107014, 1.3456597739943277, 3.3771531999524402, -1.546551299938006], [-4.4677914314744474, 0.22033047250622051, -2.9077433565617117, 1.1836798006859226, -1.8511474205137155]], [[2.9959747118405842, 4.4708123542318496, -4.3342532430247029, 2.1236337591909171, 2.3699082449656004], [1.5860472264282688, -1.8065430280560624, -2.6430779331853738, 2.9911268435790621, 3.7740755095982834], [2.7311722448291844, 1.6535405378742576, 4.6680238349333614, -3.9018227853004772, -1.0851513792283707], [-2.6278924173243401, -3.2051151461521963, -0.041121405242861009, 3.7400460127084436, -1.647631100883753]]], [[[1.3256350020971404, -1.0584512026420545, 0.92899502341853601, 1.9324083975959248, 4.1282048896220687], [-2.3875021651989101, 4.2637175118383013, -1.8150511255713808, 3.5960295545489558, 3.1141745627123392], [-0.36823546751453495, -0.59258263427032354, -3.7308446256253767, -0.74455095591164167, 3.9692515677443048], [-1.8630622688901055, 0.14618816737033669, -0.43294159085396799, -1.0800343479385042, -0.98647802524671224]], [[3.0244923665938011, -2.8768009585598753, -0.48137768323858321, -2.4490095321650562, 4.3899187515964684], [4.8309868775270441, -0.28890634532849546, 2.64618383934046, -3.6654949190788, -2.6589979666821737], [4.4982197056021818, 1.8151953772823823, -2.5076795489407875, 3.9451834234834084, 3.6604096135462942], [4.2131729249460008, 1.4829753174150175, 4.9029212951811214, -3.8202151659484973, 3.8881237779650704]], [[-1.7617318678038032, 0.42993597350218948, 1.1120733644872338, 3.6960863278207619, 2.2144962760904709], [2.8599205287161498, -3.2738845766914739, 2.6951323015938522, 4.6464883104282677, -4.8315360734247506], [-1.3112474673362016, 0.93198925834410584, -0.4733318503901387, 3.9033817679258327, -4.6035849226120256], [-2.3749047432013271, -3.6151496299853614, 1.8250215439096955, 4.5393385375905648, 2.3544508889752711]], [[2.7262646921332987, 3.7455418456833574, -0.73476130517050375, 3.5081198472811828, 1.7058344355596899], [0.29542703173049745, 2.8642956782569238, 4.0524698303204385, 2.5486070204445879, 2.4290601848125304], [4.587208576484878, -2.5011640137043334, -2.8347566378625455, -0.49339710784129043, 0.78797563063028164], [2.1862180423780835, 3.2835246186990155, -2.1482034267679273, 3.3118121805492216, -1.4822133121418188]], [[-4.4809926994572411, 2.8491067279054141, 3.6833928246411443, 2.7665580886775434, -0.50045999736773439], [-3.5632730198181859, -3.4341164793150067, 3.8948854855836483, 3.1977533928560682, 0.10313886640325709], [4.2778780637785623, 2.5360137837503576, 3.7408989602984715, -0.62031567956235989, 0.38795421833294519], [-4.0715570199664652, 2.7788034403328759, -1.5606147954306349, 3.0146312887540887, -1.0428797270724699]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[1.2921095749096168, 0.23020186359163475, -3.1700383816057665, 4.4559525141541805, 0.46826530996776938], [1.1982323667085657, -0.2978508637496553, -2.776327632030049, -0.34696036766572469, -2.2213840049697655], [2.0108530505966717, -3.6729009369830967, 4.8777451768049094, 1.4996554366709667, 1.9280549090588242], [1.2766004504370567, -4.2934140229517332, -2.908367253930737, 4.6066117556428026, 2.3351415924692196]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[2.543122407983943, 5.8722026908504716, 2.2151000685684572, -3.4154462636051806, -2.1715750245174705], [0.30031275554777359, 1.7738304116340551, 1.0918817818567081, -0.93251526820680031, -2.8262678023259324], [2.8031718074022391, 3.2102801190050165, -2.3315407818087386, 6.0804655574684965, 1.5449814592265811], [1.3036788969349411, 5.2385436819564184, 5.9293867157186106, -2.3986025363709498, -2.7375743556238161]], [[0.21872975623285296, 2.3739761430510411, -4.1915761703636196, 1.6131772664651045, 4.83924080737882], [-2.2325644112915741, -1.3983192840884029, -4.3997777967709739, 4.6539754711114938, -0.8984819546915066], [-2.2259437204321939, -1.1146803117021653, -4.4225027547070814, 3.8957197997731079, -1.9805130643927815], [4.7286245154324984, -2.0868510516504371, -4.6615275108650431, -2.9952835486760963, 3.4613591026482435]], [[-6.1471011312535992, 1.6545315558700233, 1.0005831737617283, -1.0785925633167706, -2.3972742172601578], [-0.15479589865215271, -1.7427962814837148, -3.6781516380240449, -2.6291378597154056, -5.729518238768236], [-1.3696055198726178, 0.46467172630462361, -2.992589250886998, -4.7412065735813247, -7.7628056796456359], [-1.7945453408912995, -5.7319900174063552, -6.0401207717332239, -0.33925808518502709, 0.7900495427833425]], [[3.9748065123763379, 2.1660461984336257, 2.6193427171269406, 6.7054471583319826, 8.6198266702449935], [-0.16941547398144685, 6.7105452462684259, 7.9496485470923268, 0.70558145883455259, 0.8540535514126848], [0.99091090687187489, 1.0259121761060666, 5.8437985466882898, 2.409384659798806, 9.3237912318247957], [5.6574057730288567, 4.6558538045741269, -0.37558919926146217, 8.4629470846389552, 6.9855360248292833]], [[0.75533677143098998, 3.4358920257958374, 5.3961907834795309, 3.6185778777502389, -3.0775771514196855], [0.9164876176383574, 5.0926241414847393, 0.48495624819233196, -1.0038990105255272, 5.0926598769281739], [-0.9896738127375837, -4.5198309526487934, 4.4904473817864226, 5.3154851491386452, 4.3225055600918587], [-0.61481140987079552, 1.2185481528802233, 1.3288446758084698, 1.4386204808585248, 2.4147098068227688]]], [[[4.4959734265588933, 2.6489308888574508, 4.4166086165910547, 1.0855451582685349, 0.59620939363810255], [-1.2934757135848676, 3.4056220598054141, 0.25912366008644749, 0.31807323663550147, 5.4463622348798149], [-1.01151721896403, 2.5727395941650677, -1.2827181087198793, -1.0130983945461893, -2.0561357327893681], [-1.59112217800135, -1.8295277313005145, 6.0843546374122335, -2.9276353435590146, 0.96107933185134886]], [[-1.4040871984593526, 0.92384096431030116, -0.34420845106532738, 3.8162495162092505, -1.9282657772327845], [3.2131512295979299, -0.71903598316599115, 4.6330833883069609, 1.2146380687572247, -4.1859852583142718], [3.3912752073181789, 3.3052692959198904, 1.9789768796842218, 3.2150969550487876, -5.1635369383433778], [3.009403099742066, 2.7987082696185794, -4.2677144526012967, 1.686093681018848, 3.9831403763467215]], [[-7.6630882255987043, -1.8490924116833671, 1.4691491836012878, -6.581631841842265, -0.59341967296048193], [-3.5941920896737942, -0.15710241049892737, -3.227509804936914, 1.9751850378169982, -6.7639476860784411], [-1.4681225583522517, -5.0610387248626889, -0.099759355723512577, -5.3630022113834972, -0.072121660679549393], [-3.596719463961934, 0.073031690254993897, 0.53994260221850521, -3.5297908957192234, -1.9045653157107294]], [[2.1124455108406464, -0.15677640707898277, 2.5297518728147441, -5.2556979821247651, -0.98110386647595593], [-2.1452928330993699, 0.41234691855932049, -3.5389611568344299, -5.2669416498998798, 0.66470759999512374], [4.374935458219996, -1.5484360881056909, 3.7458275881519789, 0.48111425840465838, -0.075265281084073266], [-2.7067772793516225, 4.1407354198950035, -0.71657752635015548, -2.4601295518735973, 3.1238887210418298]], [[-3.2412316074970029, 2.2187579398582731, 0.35190702345004166, -0.00090368694128617477, -3.925428955568937], [-6.5123267200160964, 2.2118565302126019, -2.8662965461518191, -3.6418846947411287, -3.8378723808392978], [0.47681639620747163, 0.88778852602353986, -5.830772708061259, 1.1851744620397113, -3.3818470298275427], [-5.2854010960144775, -5.0747486395995729, 2.4518056905364762, -4.5665633572222131, -7.0906670010007176]]], [[[4.5894611372655225, 5.3465427595671828, -1.164755816260687, 4.100640281636438, -0.56562008591091129], [6.8420969434475873, 5.8213250030799353, 0.63246412040108302, 3.5349624397728485, -1.6519620727115125], [6.9895986340678169, 0.2520295211346637, 6.4806902765874126, 0.45405882090784822, 2.5057469868860816], [6.1356160768853973, 5.8753454207937184, 4.4747157907957265, -0.61717386803985974, -2.5179651939512446]], [[-6.1771408584091105, -1.681495423194491, -7.4097656851463896, 0.77146874519070519, -7.851862796189562], [0.34279531134216956, -3.0352233231174433, -4.4830501334027018, 0.78145376261378008, -1.1091700533691604], [-0.96161771950432851, -3.7075286023100498, 0.61406146990154742, -0.39434995946451767, 1.0975685283642482], [-6.9091976358316742, -4.4519739332518196, -4.6115130155758752, -8.2486706413779753, -1.921930948400699]], [[4.717029860767127, 9.8614009863714518, 6.6243678464652156, 8.7043053478914771, 6.3778274107700739], [1.5910787065401264, 0.46729475759382488, 3.1014123009947867, 9.4306376606974709, 4.1752228783220255], [2.1915657194275635, 0.67554577077189482, 9.29371281758187, 4.6806523035237531, 9.7263302201159689], [5.9796384364967565, 6.0502483584631506, 2.6802840327433062, 7.7658517417888104, 8.3047296008395364]], [[5.019243149849693, 3.0658435533415451, 3.1299272417866302, 2.1869011978092168, 5.3878479091306524], [5.4614986602561348, -2.4142630999328309, -0.92273811605657974, 1.8658472237485046, 3.6357879637047912], [2.3982346987132619, -0.7927009376397347, 2.8453152106652944, 4.8768086366234069, -0.046895863267039317], [-2.9681359948034807, 1.7199859091771872, -1.408087919890745, 2.6833352373568893, -0.35149198384274882]], [[4.9240296208994083, 6.3988672632906738, -2.4061983339658788, 4.0516886682497413, 4.2979631540244245], [3.5141021354870929, 0.12151188100276178, -0.71502302412654961, 4.9191817526378863, 5.7021304186571076], [4.6592271538880086, 3.5815954469330817, 6.5960787439921855, -1.9737678762416531, 0.84290352983045347], [-0.69983750826551594, -1.2770602370933721, 1.8869335038159631, 5.6681009217672678, 0.28042380817507118]]], [[[2.6022354525341971, 0.21814924779500222, 2.2055954738555927, 3.2090088480329815, 5.4048053400591254], [-1.1109017147618534, 5.540317962275358, -0.5384506751343241, 4.8726300049860125, 4.3907750131493959], [0.90836498292252177, 0.68401781616673318, -2.4542441751883199, 0.53204949452541506, 5.2458520181813615], [-0.58646181845304879, 1.4227886178073934, 0.84365885958308873, 0.19656610249855255, 0.29012242519034448]], [[-1.2689216563579322, -7.1702149815116085, -4.7747917061903165, -6.7424235551167895, 0.096504728644735138], [0.5375728545753109, -4.5823203682802287, -1.6472301836112733, -7.9589089420305328, -6.9524119896339069], [0.20480568265044852, -2.4782186456693509, -6.8010935718925207, -0.3482305994683248, -0.63300440940543901], [-0.080241098005732425, -2.8104387055367157, 0.60950727222938816, -8.1136291889002301, -0.40529024498666288]], [[-4.6700991217345402, -2.4784312804285475, -1.7962938894435032, 0.78771907389002482, -0.69387097784026608], [-0.048446725214587261, -6.182251830622211, -0.21323495233688483, 1.7381210564975307, -7.7399033273554876], [-4.2196147212669386, -1.9763779955866312, -3.3816991043208757, 0.99501451399509566, -7.5119521765427626], [-5.2832719971320641, -6.5235168839160984, -1.0833457100210415, 1.6309712836598278, -0.55391636495546592]], [[7.3328764477761013, 8.3521536013261599, 3.8718504504722988, 8.1147316029239853, 6.3124461912024925], [4.9020387873733, 7.4709074338997263, 8.6590815859632411, 7.1552187760873904, 7.035671940455333], [9.1938203321276806, 2.1054477419384692, 1.7718551177802571, 4.1132146478015121, 5.3945873862730842], [6.792829798020886, 7.890136374341818, 2.4584083288748753, 7.9184239361920241, 3.1243984435009837]], [[-2.1458511069880215, 5.1842483203746337, 6.0185344171103639, 5.101699681146763, 1.8346815951014852], [-1.2281314273489663, -1.0989748868457871, 6.2300270780528679, 5.5328949853252878, 2.4382804588724767], [6.6130196562477819, 4.8711553762195772, 6.0760405527676911, 1.7148259129068597, 2.7230958108021648], [-1.7364154274972456, 5.1139450328020954, 0.77452679703858474, 5.3497728812233083, 1.2922618653967497]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank2_Symbol_rank4(self):
      arg0=Data(numpy.array([[-3.5841115381248034, -2.1231005301329597, -0.71275652322575311, -0.06568151070427497, -3.9442405865648245], [4.2859406328786136, -3.6539133602559062, 4.9515617260735798, -1.3296865200159766, -0.81822094996259676], [3.1599309025387239, 2.8943930812741598, 1.271908621286383, 0.58447817326036322, 3.8881002560583884], [-0.35379268782364637, 3.9105253136996527, 1.9229871042506641, -3.7464546272070711, 1.1425746374682362]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[[1.2290053493846829, -3.1110570639438717, -0.2613616329927968, -2.0716623890866179, -4.655673745712595], [0.52792961967164054, -3.6963923030486212, 2.8136918028228655, 0.69597936114064396, -2.5399946217091296], [4.964887469807449, 4.8177130775649815, 2.4142216533449803, -1.8626122183249461, 2.213966777780767], [-2.5379591815087812, 1.3827775652258225, 3.3041126434439967, -1.8753071342810324, 4.1296450614938678]], [[4.2059350631497754, 3.3181892019590045, -2.8855668034954975, 2.1406841160197345, -4.2120986555999576], [4.4355902066655855, 2.7473428345133719, 4.6311702388922029, -2.1195549951669634, 2.2742444099803452], [2.8677378081858675, -3.9219934819076743, -0.34584362626324161, 2.2764260506319456, -4.3320264685404117], [-2.5144596910585437, -2.6198621035813954, -3.5315950160648804, -0.82492480100368404, 4.2852864153908161]], [[2.2147443604230768, -3.9055347379544489, 3.844580612754843, -0.9795531892004119, 4.4545849533104516], [-4.628082622028618, -4.3874535710711982, 4.7226985619836022, 1.9638872422221851, 1.4047130503257481], [4.6052939962271218, 4.8866157228718059, 1.1082985440335573, -3.1107580785280486, 0.55107088640113933], [-1.2635088756997304, -4.2927918630466184, -3.2482803997271148, 1.219739862636815, 1.4242933661776167]], [[1.3651058714845643, -0.75515584489791987, 0.56445104812139313, -2.339876252882982, -1.2615295123613146], [4.7032213774672034, -0.49207425633371038, 2.5351081548730487, 2.0534531576170698, -1.6066888076237786], [-2.1907361067077948, -4.1093806148083374, -0.6681417294059111, -1.9448957298659431, -1.619998419859825], [-3.0993496824252444, -2.1071523876487852, 0.82456728747233932, -0.39028467178006032, 1.3376147009101249]], [[4.2746757465353937, -0.3436534477370321, 3.3879922692824085, 3.7568628388638974, 0.45020161418369309], [-2.7870365800970829, 2.9272771498440857, -4.6442265756573917, -1.8704043743037149, -2.5333011356525104], [-4.0845952518365856, 0.095627913491830085, -4.2093038785583659, -2.7119496690765743, -0.63315959502573271], [0.50845353274365035, -2.7495092517999153, -0.087125376777771102, -1.9035328514206471, 4.4254815470266387]]], [[[-2.1215620435380522, 0.062063209529435781, -4.0019247607464061, 0.71856125548776362, 2.6899451237032723], [4.5460307586927726, -4.9176031964408153, -2.8539686228729169, -0.26843472353510478, 1.9738882871977221], [-4.4257787868213718, 0.13680826202822161, 3.5423356054036894, 3.3153904233537332, -1.6933118325044303], [4.0612378596815617, -0.22938551199407264, 0.60507351657954711, -1.7182655049021536, -1.1629737447232005]], [[1.9915668258286363, -3.4857845443982596, 3.2586929547733021, 2.6433501629877663, 3.8186913386958281], [-4.2524801463037267, 1.9553388382482693, -0.67007426412897608, -4.1063646551671349, 1.9577122970593441], [-3.2133525201712887, 0.56968290953696155, -1.2031109849082844, 3.7310976933484401, -3.2399530231912443], [-2.3344373009068065, -1.2405906733328678, 0.56505180197010851, -4.8425578926770205, 3.1709165123644389]], [[-3.1345895909811459, -4.5156600077984255, -4.4187950501616937, 3.3076876049236645, 0.60053515948775971], [1.627396635927215, -4.7082669620945126, -0.53557930871800785, 1.5065006069944742, -3.8370942196345359], [3.7914531934607183, 4.3799080374343333, 4.4015925404686449, -0.9206035317748924, -1.6346342715810991], [-1.8121863329279728, -3.1854092778521501, -0.0028749731447250682, -3.3922540833024897, -1.8059342006636827]], [[1.315540219469149, -2.1060902222181941, -2.651626232854456, 0.7015186280752097, 3.3953452023699544], [-2.6558496351580785, 2.7048273678498393, -1.2446076871630964, -3.6118487975956581, 4.7813410384677528], [-0.96641404836778122, -2.1844557000615037, 1.2472843380859615, 4.3528677732520116, -1.824057548857978], [-1.2676558322205311, 4.5696476069870169, -1.4269979931865473, 0.55819431028693778, -1.0712944059000309]], [[-1.3356439648696528, 4.2198624550120698, -1.6289968742386174, 4.4125813682782304, -4.2910677231144456], [-2.7275166656760863, 2.2742359333152002, -4.8766604517921976, 4.0571124568077259, -1.3918610447369528], [-4.1308093090288693, -1.4370978209771934, 4.4279934477289871, 4.5508689964026132, -2.6336743485473391], [3.0200383500245529, -3.2177598970764407, -4.258130825675722, 4.8896154971479895, -3.0148801472926579]]], [[[-3.7228777223766887, 0.39860419668676617, -4.9056741619355373, 2.6574705108113639, 2.4144499982559564], [-0.11294203073916176, 4.410189915297444, -1.0099997682424009, 2.4415865986129024, 4.9467237739194179], [-0.76660584340035776, 1.4996107854880139, -4.2209685431065171, 2.0100173019286238, -0.57327818870366443], [-4.3191379242846306, -4.9777395900225034, 1.3773479859960416, 3.4541331460282585, 3.5385737079805661]], [[-4.0862739979070941, -1.2973496159310427, 4.6327867038195372, -0.69854229553462321, -2.230473624229854], [-3.8819047487332572, 0.47446437990768509, -1.4467177957600463, 4.171162365471325, -4.6504113731092414], [0.63832217989939011, 1.9065864728614432, -2.4601887132092726, -3.7050155535524052, 1.7844464273434344], [2.2968715176122849, -1.4378426042501102, -3.4850034788365125, 0.97363464565970137, 2.1590679930255252]], [[-0.77750666835124349, -3.7167811622788873, 1.4124175004075559, -0.20928273864016855, 1.0858492538930333], [4.1942561392459883, 1.62286439776521, -2.0089754423209416, 2.2972197458650943, 1.7444150906204934], [-3.9073792780561902, -4.1979734361434149, 0.96384528706114558, -4.059105569554518, 0.15968085021665601], [-0.52923478539652713, -3.1799866444274749, 1.0434159998060739, 3.3617138136539939, -4.4929216182846288]], [[3.5763342309963697, 4.1385704111341166, 2.911557013614555, 2.8817837403735211, -3.7735071691844304], [-3.2580271469405364, 4.5120117538206799, -2.9310421364399697, -3.4590917311430145, -0.10017431507439767], [3.8890967209113221, 3.4510102945461778, 4.2772734170028475, 1.4402436459491943, -1.0306877952284408], [-3.9192480378415651, -3.126689857318067, -3.3430628929373363, 3.5596182508545606, -4.3261492127005372]], [[-0.55351867147362377, -2.9637380586359665, 0.63030356553842815, 1.9034086176673064, -3.097483902383309], [4.5909555037061356, 3.0254731599372917, -0.7385916884387953, 2.7960560885965506, -2.4279240776525035], [-2.9530843413241303, -4.9615688328165071, -1.9210752842376513, 1.1260017820229349, -0.62862100979715319], [-3.8618383216347842, -4.5484913634886901, 4.2982780081364531, -2.4046331520165065, 2.9421523943652836]]], [[[-1.1963964807399652, 3.9658934044053122, -0.88795761265764384, -0.22490919746292892, -3.1391371889585318], [2.5968592670191617, 4.0587227592941453, -1.2275275768055014, -0.47508704771814791, -4.9684447952582822], [-3.2164271100071429, 1.3340635718614058, -2.3460983613293029, 4.1794498969026641, 1.4704888165519332], [0.57994418845057183, 4.3666364198251237, -3.9362033468570026, 0.26742966521664258, 1.12060089992569]], [[1.1753085981908171, -1.8864231962895452, 1.8963302310065746, -4.61054454983155, 0.82451364390168003], [2.0522899461601565, -1.2763866220930953, -2.0191504188633767, 4.9399478652251414, 4.1114063542301444], [-3.2420674702737906, 4.5325450984180051, -1.5291088258929495, -3.6210309497196622, 4.8342414650587564], [-2.0002345359770155, 1.1734267306153985, 2.5608223009822577, 1.1336314509704968, 1.2972949678421699]], [[3.4785511450093995, 0.87257877524682748, -0.58813309914792189, -3.8743905257869815, -3.6700014086272992], [-2.8042438067328201, -4.2451088950647726, 3.1564660360836214, -2.5164606637054465, 1.8692377414780221], [0.93811124686413194, -2.6488089946050652, 3.5046460546886848, 3.8361063807142788, 4.4325461522587162], [-4.8941618324350022, -0.10509204800939997, 4.120475682978487, 0.80284959181851256, -4.8873193181508556]], [[-1.6971560145876277, 1.1139797028333733, 4.7170306356488254, -2.1353227309584852, -2.1717321653361199], [-1.8733039640549154, 4.8510557654155679, -4.5397715076214329, 2.6307582534198719, 4.8908848048374995], [-3.2225402227957147, -1.1810004664019136, -1.2376025481797659, -0.027961467727370071, 0.76757607439598274], [-2.0041653638198911, -4.9773160488538668, -3.0293440500276025, -4.0567056378942912, -0.5198036756000679]], [[4.2221994292764808, -1.3786434505180312, 3.0825330808876217, 1.7762678559046927, 0.56181959096073708], [-2.174786650533842, -4.8555632326145259, -3.0297720456408674, -3.00512599624841, 2.4823230666386573], [4.7823420493039208, 2.883092824953307, -2.2101888535495418, 0.50692300427316361, 2.5560204933981732], [-4.3707569483548694, -2.6941265796877598, -3.5245004605697803, -3.5274657155704379, -2.1441593131369507]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-2.3551061887401206, -6.6951686020686747, -3.8454731711176002, -5.6557739272114214, -8.2397852838373993], [-3.0561819184531629, -7.2805038411734246, -0.77041973530193797, -2.8881321769841595, -6.124106159833933], [1.3807759316826456, 1.233601539440178, -1.1698898847798231, -5.4467237564497495, -1.3701447603440364], [-6.1220707196335846, -2.2013339728989809, -0.27999889468080674, -5.4594186724058353, 0.54553352336906435]], [[2.0828345330168156, 1.1950886718260447, -5.0086673336284573, 0.017583585886774777, -6.3351991857329173], [2.3124896765326257, 0.62424230438041217, 2.5080697087592432, -4.2426555252999236, 0.15114387984738542], [0.7446372780529078, -6.045094012040634, -2.4689441563962014, 0.15332552049898585, -6.4551269986733715], [-4.637560221191503, -4.7429626337143551, -5.6546955461978401, -2.9480253311366438, 2.1621858852578564]], [[1.5019878371973236, -4.6182912611802021, 3.1318240895290899, -1.692309712426165, 3.7418284300846985], [-5.3408391452543711, -5.1002100942969513, 4.009942038757849, 1.251130718996432, 0.691956527099995], [3.8925374730013687, 4.1738591996460528, 0.39554202080780421, -3.8235146017538018, -0.16168563682461379], [-1.9762653989254835, -5.0055483862723715, -3.9610369229528679, 0.50698333941106188, 0.71153684295186359]], [[1.2994243607802893, -0.82083735560219484, 0.49876953741711816, -2.4055577635872569, -1.3272110230655896], [4.6375398667629284, -0.55775576703798535, 2.4694266441687738, 1.9877716469127948, -1.6723703183280536], [-2.2564176174120698, -4.1750621255126124, -0.73382324011018607, -2.010577240570218, -1.6856799305640999], [-3.1650311931295194, -2.1728338983530602, 0.75888577676806435, -0.45596618248433529, 1.2719331902058499]], [[0.33043515997056927, -4.287894034301857, -0.55624831728241597, -0.18737774770092708, -3.4940389723811314], [-6.7312771666619078, -1.0169634367207387, -8.5884671622222157, -5.8146449608685398, -6.4775417222173353], [-8.0288358384014096, -3.8486126730729944, -8.1535444651231899, -6.6561902556413983, -4.5774001815905567], [-3.4357870538211741, -6.6937498383647398, -4.031365963342596, -5.847773437985472, 0.48124096046181419]]], [[[2.1643785893405614, 4.3480038424080494, 0.28401587213220747, 5.0045018883663772, 6.9758857565818859], [8.8319713915713862, -0.63166256356220174, 1.4319720100056967, 4.0175059093435088, 6.2598289200763357], [-0.13983815394275823, 4.4227488949068352, 7.828276238282303, 7.6013310562323468, 2.5926288003741833], [8.3471784925601753, 4.056555120884541, 4.8910141494581607, 2.56767512797646, 3.1229668881554131]], [[-1.6623465344272699, -7.1396979046541658, -0.39522040548260406, -1.0105631972681399, 0.16477797843992192], [-7.9063935065596329, -1.6985745220076369, -4.3239876243848823, -7.7602780154230411, -1.6962010631965621], [-6.8672658804271949, -3.0842304507189446, -4.8570243451641906, 0.077184333092533919, -6.8938663834471505], [-5.9883506611627126, -4.894504033588774, -3.0888615582857977, -8.4964712529329276, -0.48299684789146724]], [[1.8169721350924339, 0.43590171827515434, 0.53276667591188609, 8.2592493309972443, 5.5520968855613395], [6.5789583620007948, 0.24329476397906724, 4.415982417355572, 6.458062333068054, 1.1144675064390439], [8.7430149195342981, 9.3314697635079131, 9.3531542665422247, 4.0309581942986874, 3.3169274544924807], [3.139375393145607, 1.7661524482214297, 4.9486867529288547, 1.5593076427710901, 3.1456275254098971]], [[-0.014146300546827639, -3.4357767422341707, -3.9813127528704326, -0.62816789194076694, 2.0656586823539778], [-3.9855361551740551, 1.3751408478338627, -2.5742942071790731, -4.9415353176116348, 3.4516545184517762], [-2.2961005683837579, -3.5141422200774803, -0.082402181930015139, 3.0231812532360349, -3.1537440688739546], [-2.5973423522365078, 3.2399610869710402, -2.7566845132025239, -0.77149220972903887, -2.4009809259160075]], [[-2.1538649148322495, 3.401641505049473, -2.4472178242012141, 3.5943604183156337, -5.1092886730770424], [-3.5457376156386831, 1.4560149833526035, -5.6948814017547944, 3.2388915068451292, -2.2100819946995496], [-4.9490302589914661, -2.2553187709397902, 3.6097724977663903, 3.7326480464400165, -3.4518952985099358], [2.2018174000619561, -4.0359808470390375, -5.0763517756383187, 4.0713945471853927, -3.8331010972552546]]], [[[-0.5629468198379648, 3.5585350992254901, -1.7457432593968134, 5.8174014133500878, 5.5743809007946803], [3.0469888717995621, 7.5701208178361679, 2.149931134296323, 5.6015175011516263, 8.1066546764581418], [2.3933250591383661, 4.6595416880267377, -1.0610376405677933, 5.1699482044673477, 2.5866527138350595], [-1.1592070217459067, -1.8178086874837796, 4.5372788885347655, 6.6140640485669824, 6.69850461051929]], [[-1.1918809166329343, 1.5970434653431171, 7.527179785093697, 2.1958507857395366, 0.66391945704430588], [-0.98751166745909735, 3.3688574611818449, 1.4476752855141135, 7.0655554467454849, -1.7560182918350815], [3.5327152611735499, 4.8009795541356031, 0.43420436806488727, -0.81062247227824535, 4.6788395086175942], [5.1912645988864448, 1.4565504770240496, -0.59061039756235267, 3.8680277269338612, 5.0534610742996851]], [[0.49440195293513955, -2.4448725409925043, 2.6843261216939389, 1.0626258826462145, 2.3577578751794164], [5.4661647605323713, 2.894773019051593, -0.73706682103455856, 3.5691283671514773, 3.0163237119068764], [-2.6354706567698072, -2.9260648148570318, 2.2357539083475286, -2.7871969482681349, 1.4315894715030391], [0.74267383588985592, -1.9080780231410919, 2.315324621092457, 4.633622434940377, -3.2210129969982457]], [[4.1608124042567329, 4.7230485843944798, 3.4960351868749182, 3.4662619136338844, -3.1890289959240672], [-2.6735489736801732, 5.0964899270810431, -2.3465639631796065, -2.8746135578826513, 0.48430385818596555], [4.4735748941716853, 4.035488467806541, 4.8617515902632107, 2.0247218192095575, -0.44620962196807756], [-3.3347698645812018, -2.5422116840577038, -2.7585847196769731, 4.1440964241149238, -3.741671039440174]], [[3.3345815845847646, 0.92436219742242187, 4.5184038215968165, 5.7915088737256948, 0.79061635367507943], [8.479055759764524, 6.9135734159956801, 3.1495085676195931, 6.684156344654939, 1.4601761784058849], [0.93501591473425805, -1.0734685767581187, 1.9670249718207371, 5.0141020380813233, 3.2594792462612352], [0.026261934423604139, -0.66039110743030172, 8.1863782641948415, 1.4834671040418819, 6.830252650423672]]], [[[-1.5501891685636116, 3.6121007165816659, -1.2417503004812902, -0.57870188528657529, -3.4929298767821781], [2.2430665791955153, 3.7049300714704989, -1.5813202646291478, -0.82887973554179428, -5.3222374830819286], [-3.5702197978307892, 0.98027088403775942, -2.6998910491529493, 3.8256572090790177, 1.1166961287282868], [0.22615150062692546, 4.0128437320014774, -4.2899960346806489, -0.086363022607003792, 0.76680821210204364]], [[5.0858339118904698, 2.0241021174101075, 5.8068555447062273, -0.70001923613189732, 4.7350389576013328], [5.9628152598598092, 2.6341386916065574, 1.8913748948362761, 8.8504731789247941, 8.0219316679297972], [0.66845784342586212, 8.4430704121176579, 2.3814164878067032, 0.28949436397999051, 8.7447667787584091], [1.9102907777226372, 5.0839520443150512, 6.4713476146819104, 5.0441567646701495, 5.2078202815418226]], [[5.4015382492600637, 2.7955658794974916, 1.3348540051027422, -1.9514034215363174, -1.7470143043766351], [-0.88125670248215604, -2.3221217908141085, 5.0794531403342855, -0.59347355945478242, 3.7922248457286862], [2.861098351114796, -0.72582189035440114, 5.4276331589393489, 5.7590934849649429, 6.3555332565093803], [-2.9711747281843381, 1.8178950562412641, 6.0434627872291511, 2.7258366960691767, -2.9643322139001915]], [[-5.4436106417946988, -2.6324749243736978, 0.97057600844175429, -5.8817773581655564, -5.9181867925431906], [-5.6197585912619861, 1.1046011382084968, -8.2862261348285031, -1.1156963737871992, 1.1444301776304284], [-6.9689948500027858, -4.9274550936089847, -4.9840571753868375, -3.7744160949344412, -2.9788785528110884], [-5.7506199910269622, -8.723770676060937, -6.7757986772346737, -7.8031602651013623, -4.266258302807139]], [[5.364774066744717, -0.23606881304979499, 4.2251077183558579, 2.9188424933729289, 1.7043942284289733], [-1.0322120130656058, -3.7129885951462898, -1.8871974081726313, -1.8625513587801739, 3.6248977041068935], [5.924916686772157, 4.0256674624215432, -1.0676142160813056, 1.6494976417413998, 3.6985951308664093], [-3.2281823108866332, -1.5515519422195236, -2.3819258231015441, -2.3848910781022017, -1.0015846756687146]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[-2.7997077515801614, 1.5625163622646054], [3.7735029210939892, -4.2760029827646475]], [[1.9281471709816795, -2.6225104458719537], [4.8079219192227747, -2.843000650349139]], [[-0.39891566224769104, 1.1749451630639118], [1.9496856002504268, 1.5688897164372158]], [[-3.8817165904710702, 2.7848954351711619], [4.6990780621843715, -1.8341020854770838]], [[-4.9398380459724978, 1.8184838084633448], [-4.9369445375625807, 0.059209496834224495]], [[-0.20311804382334131, -0.53468578365951558], [3.0381808365305929, -0.34474283662492144]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[1.2151552730648341, 0.92935679262298265], [2.8455265040439368, 0.31705967267095758]], [[0.69808540318872403, -0.83678812102348932], [-4.9702188424610361, 0.4589031930102685]], [[3.5613644546390919, 3.4206327091396744], [-4.3680888666289164, -4.7632111610891865]], [[-2.1961599015218991, -2.1364520421652444], [-0.38338595074429449, -4.3571116331538988]], [[-2.6636096425610436, -1.9946836674127244], [-4.4788595567149443, -1.6074320806578024]], [[4.268193334349462, 2.6726700134336721], [-3.4184009880004593, 3.4777705935987804]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-1.5845524785153273, 2.4918731548875881], [6.619029425137926, -3.9589433100936899]], [[2.6262325741704036, -3.459298566895443], [-0.16229692323826139, -2.3840974573388705]], [[3.1624487923914009, 4.5955778722035863], [-2.4184032663784896, -3.1943214446519708]], [[-6.0778764919929689, 0.64844339300591747], [4.315692111440077, -6.1912137186309826]], [[-7.6034476885335414, -0.17619985894937962], [-9.4158040942775258, -1.5482225838235779]], [[4.0650752905261207, 2.1379842297741565], [-0.38022015146986643, 3.1330277569738589]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[4.8960245645728886, -0.6288668445775798], [-4.1980097132314329, 2.2259966658109143]], [[-1.2062367927466946, 0.24251270017080095], [-3.5670862261846947, 0.41459849051974551]]], [[[-3.4005766430133368, 4.5977999239733744], [0.29861065340326576, 2.7199403248304002]], [[-4.7128058726591773, -0.65271350734750833], [-3.7035491406893817, 4.9927189668869829]]], [[[-4.1322686258125616, 4.9959963956585902], [-0.31931240504718961, -2.5916521436075977]], [[-1.406824381230888, -3.7108253016400781], [1.4764956039814523, 4.811381716258099]]], [[[0.44594977546632819, 2.3926601979093576], [1.5568092841581898, -3.1906221776962385]], [[-0.18519206114504616, 4.5840365394388147], [3.2367530515175709, 2.4006308067385929]]], [[[3.2074960522586888, 0.61547186727687642], [4.3428717061937974, 4.3425612001387233]], [[-0.32470121211304903, 1.4856794595113589], [-4.8111892447441278, -1.7330149310262244]]], [[[0.36041611852200983, 3.8789049209131043], [-1.4635079490131577, 0.34208580360609808]], [[4.4545168570534859, -2.7173017008658573], [0.12430816554614399, -3.6910211817647975]]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-2.9092682551282523, 4.7434845771726], [-2.6927292029879011, 1.4911748079246898]], [[-0.77346605556771131, 4.7105196161276695], [-2.7057914163239349, 2.475070174920182]], [[-3.375814762261018, -0.055554925595020066], [4.1825269593253509, 2.1798600306096816]], [[0.27312118453149292, -2.0447921624084775], [-4.6938567609492736, 2.2551374842808194]], [[-3.4718557377287929, 2.1811833027515837], [2.0345595879301301, 3.7295312984589337]], [[1.5004717742722926, 4.5813715240238615], [3.843800808314807, 2.2949639355975835]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[1.9867563094446363, -3.5381350997058321], [0.54547486394116707, 6.9694812429835142]], [[-3.8989659957345957, -2.4502165028171001], [-2.0759114182600049, 1.9057732984444353]]], [[[-4.1740426985810481, 3.8243338684056631], [5.0091302695309352, 7.4304599409580696]], [[-7.4185972889831122, -3.3585049236714433], [-1.2284789657691997, 7.4677891418071649]]], [[[-7.5080833880735796, 1.6201816333975723], [-0.37486733064220967, -2.6472070692026177]], [[2.7757025780944629, 0.47170165768527283], [3.6563556345911339, 6.9912417468677805]]], [[[0.71907095999782111, 2.6657813824408505], [-0.48798287825028774, -5.2354143401047164]], [[-4.8790488220943198, -0.10982022151045889], [5.4918905357983903, 4.6557682910194123]]], [[[-0.26435968547010402, -2.8563838704519164], [6.524055008945381, 6.5237445028903069]], [[1.7098583758170811, 3.5202390474414891], [-1.0816579462851941, 1.9965163674327093]]], [[[1.8608878927943024, 5.379376695185397], [3.1178635750107038, 4.9234573276299596]], [[8.2983176653682929, 1.1264991074489497], [2.4192721011437275, -1.396057246167214]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank3_Symbol_rank4(self):
      arg0=Data(numpy.array([[[4.9960283049418734, 2.3981584950242505], [3.0348020695621347, -4.1949691387372159]], [[-2.7584603273679242, -3.6432787235076134], [0.20184720392262712, 0.35034311851531008]], [[2.5504423278933608, 2.2931704437433371], [0.8705697177571583, -0.73927642101555691]], [[3.9676030415520245, 4.1348294113277397], [-2.2667052863089077, 1.1223914010051192]], [[2.1086313426908374, -2.9808130157153578], [-3.6176415824955397, 2.9860399374219462]], [[4.3935721251060684, 1.8886904574191385], [-1.5876243831835799, 4.1263758402436714]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-2.7800200086863436, -0.84694585768020758], [-2.6459905279566378, 2.349446580301727]], [[-4.9359149147586381, -4.7797839327791021], [4.7914454235403259, -0.96259925033265858]]], [[[-3.1073737414151061, -2.654319866769328], [1.6940562446597749, 3.5702055392020942]], [[3.8478871804114601, 4.8070378099590787], [-3.5258245122817602, 4.5133621901608372]]], [[[4.5226146234717781, 4.5981959204600678], [-3.7818126285201537, -2.8073362375520339]], [[3.4808570969513504, 2.7824946615435229], [0.25284637343007699, -3.7749970311818104]]], [[[3.7446544240984849, -4.3243650406150627], [-0.47437356906114303, -3.6793164119758845]], [[4.4057868771826669, -2.6531231999609495], [-3.0312261354103134, -2.4834805783262115]]], [[[-0.47470781275416662, 3.2659192259726186], [-2.5249625561238243, 0.1194880760255721]], [[-3.9975927615623599, -4.7561414565879119], [-3.6700327504695673, 2.2111219988450648]]], [[[-3.6016947797649115, 0.92578237159447418], [-0.12507992836376403, -1.014865811868817]], [[1.3564494088483947, 3.9752621275447808], [3.3528287398567276, 0.055730368268575248]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[2.2160082962555299, 4.1490824472616659], [-0.24783203293238731, 4.7476050753259775]], [[-1.9011128451965034, -1.7449818632169674], [0.59647628480310999, -5.1575683890698745]]], [[[-5.8658340687830304, -5.4127801941372518], [-1.9492224788478385, -0.073073184305519234]], [[4.0497343843340872, 5.0088850138817058], [-3.1754813937664501, 4.8637053086761473]]], [[[7.0730569513651389, 7.1486382483534285], [-1.4886421847768165, -0.51416579380869676]], [[4.3514268147085087, 3.6530643793006812], [-0.48643004758547992, -4.5142734521973669]]], [[[7.7122574656505094, -0.35676199906303818], [3.6604558422665967, 0.45551299935185519]], [[2.1390815908737593, -4.9198284862698571], [-1.9088347344051941, -1.3610891773210922]]], [[[1.6339235299366708, 5.374550568663456], [-5.5057755718391821, -2.8613249396897857]], [[-7.6152343440578996, -8.3737830390834525], [-0.68399281304762116, 5.197161936267011]]], [[[0.79187734534115695, 5.3193544967005426], [1.7636105290553745, 0.87382464555032158]], [[-0.23117497433518519, 2.3876377443612009], [7.479204580100399, 4.1821062085122467]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[2.1699956444285071, -0.68549960784104602, -0.44259019887955553, 1.3743694553699459], [-2.4820972590659274, 2.0313384101006315, -1.3941618524132879, 0.37740717874872054], [0.35522858324490425, -2.1177742623389175, 0.73651801755445412, 3.4100469232084656]], [[-0.57217513441357681, -3.2625789482271941, -2.6183640832005137, -3.9996470216597411], [-3.1721127322588138, -0.1523664657920385, -1.3420552212506154, -2.7727903589062555], [-1.06938962197205, -3.9695369999657828, -1.3907038379671288, 4.5133240116399413]]], [[[1.3465425279744414, -1.1365531446840427, 3.2377213286422712, -1.4688421621038392], [4.2439141551431128, -1.8972268291268835, 1.6851202355989621, -3.610031897870857], [3.4677878813657586, 1.8144656471328879, 0.9666744591588472, 3.3893009029525309]], [[0.35709037759252915, 2.0873379640383005, -1.5712064499657341, 3.3924572339349357], [-2.4865128703185313, 3.8910682132278591, 3.7349936027588484, -2.4809899987902142], [-0.79274901314288115, -2.0103384233212229, -0.66581290859042941, -4.4975579597906989]]], [[[-3.1415925573790027, 0.832572282233496, -2.0644988983961889, 4.7176697586242824], [-2.8272256820269872, -0.60993984118286448, 1.8183655292519703, 4.5642191041542741], [3.0817069567742745, 2.3016608274698322, 0.71070898394888715, 0.32073547844112316]], [[-0.19037750358576577, 2.9440162280567472, 4.4647638844070929, 2.24958906647087], [-0.23228272808053596, 3.1151087240427273, 4.9758778103967938, 0.65511099428148434], [0.60935680319877594, 4.613841589106725, -2.6116140318602508, 4.3790369938268334]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[0.80001980367074843, -1.1101791359246285, 2.1247260126023821, -0.45254492207711294], [2.2033383314929678, -2.2451674179364711, -0.82089537850670524, 4.315328151057388], [-1.2132966944675161, -0.36820590988966195, -0.86998866606286729, 1.1962534694963605]], [[1.8492522478447384, 4.1876321504722096, -1.2379199147356736, 2.5724432414174538], [-2.6962807014541412, -4.5373518687652536, 3.6921466728064161, -2.0599908981570572], [-0.24031217560290052, -2.0476087232874693, 0.90487735087763532, 4.4571318051342477]]], [[[-1.1316851885134991, -1.5576596142224695, -2.0322479227741264, 4.7740726177860431], [0.2826774815843125, -3.958864279624259, -3.4827834085852816, 2.4759847901777556], [-4.7408633538098135, -0.070165484075380036, 1.3498783072378453, 2.9786634897049149]], [[0.043659891436881537, -3.1644260524016996, 3.6540323852697298, 3.3467556177082756], [3.3908569832307371, 2.437683052497488, -2.8333512591810104, 3.7890227456274737], [4.5278159014528754, 0.79562754960307558, 0.72025929271838685, -1.4829166996234369]]], [[[-1.5374359106683233, -0.57793935018444742, -0.66503060391361579, 0.74298497586465029], [-0.75502835516512867, 1.9209321675390356, 1.9425030978913229, 1.7435326537928573], [2.0527424512844163, -3.4888016291242838, -1.1425811816586471, -1.9602185027674182]], [[-0.37836046130714784, -0.25129746459537561, 2.4606450589280353, 4.2509916514137416], [-0.85920854154508142, -3.0109582048266104, 2.6834189151020755, 4.0983304230191191], [2.5126065693108881, -2.8597440355379202, 1.2042351578373731, -3.8166759455884347]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[2.9700154480992556, -1.7956787437656745, 1.6821358137228266, 0.92182453329283298], [-0.27875892757295961, -0.21382900783583958, -2.2150572309199932, 4.6927353298061085], [-0.85806811122261184, -2.4859801722285795, -0.13347064850841317, 4.6063003927048261]], [[1.2770771134311616, 0.92505320224501553, -3.8562839979361874, -1.4272037802422872], [-5.8683934337129546, -4.6897183345572921, 2.3500914515558007, -4.8327812570633126], [-1.3097017975749505, -6.0171457232532521, -0.4858264870894935, 8.970455816774189]]], [[[0.21485733946094232, -2.6942127589065121, 1.2054734058681449, 3.3052304556822039], [4.5265916367274253, -5.8560911087511425, -1.7976631729863195, -1.1340471076931014], [-1.2730754724440549, 1.7443001630575079, 2.3165527663966925, 6.3679643926574458]], [[0.40075026902941069, -1.0770880883633991, 2.0828259353039957, 6.7392128516432113], [0.90434411291220584, 6.3287512657253471, 0.90164234357783801, 1.3080327468372595], [3.7350668883099942, -1.2147108737181473, 0.054446384127957437, -5.9804746594141358]]], [[[-4.679028468047326, 0.25463293204904858, -2.7295295023098047, 5.4606547344889327], [-3.5822540371921159, 1.3109923263561711, 3.7608686271432932, 6.3077517579471314], [5.1344494080586909, -1.1871408016544516, -0.43187219770975993, -1.639483024326295]], [[-0.56873796489291362, 2.6927187634613716, 6.9254089433351282, 6.5005807178846116], [-1.0914912696256174, 0.10415051921611695, 7.6592967254988693, 4.7534414173006034], [3.121963372509664, 1.7540975535688048, -1.4073788740228776, 0.56236104823839872]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.862701097538)+(1.-msk_arg0)*(-1.03003202048)
      arg1=1.72889137739
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(2.59159247493)+(1.-msk_ref)*(0.698859356907)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-4.9858124965967452, -2.5057565607268382])+(1.-msk_arg0)*numpy.array([-3.4010000453603526, -1.8136858458186031])
      arg1=-2.03739076926
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-7.0232032658522421, -4.5431473299823351])+(1.-msk_ref)*numpy.array([-5.4383908146158495, -3.8510766150741])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.8841009841663858, 2.5922754235523513, -3.8867494040491568, -2.0541124282302903, -1.1663836012298026], [3.5778861429902307, 1.820705187919077, 1.6650955559202636, 4.291020086169743, -1.3437244023589958], [1.7839762360576374, -3.0443444140454567, -2.0771163256026339, 2.1556409912044661, -1.9861894117988199], [4.701343912917924, 4.0715373244962976, -4.7716903356896001, 1.8307240552647173, 0.25052479421018781]])+(1.-msk_arg0)*numpy.array([[-0.31259441360677176, -1.0118434156271814, 4.727867314274544, 1.1419688697339536, -4.1774219329382669], [4.6986965416202509, 4.1912493427500319, -4.3657680294517185, 1.3874688189081663, -2.6572114523169321], [-2.1799211910559144, -4.5619556187653032, -4.6841195112682215, -0.39155371960013863, -1.910616010971069], [-0.82447688143379594, -0.61474842763707116, -4.7672490392147289, -3.8994634242669166, -3.5263925180500788]])
      arg1=-4.23729454506
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.3531935608920893, -1.6450191215061238, -8.1240439491076319, -6.2914069732887654, -5.4036781462882777], [-0.65940840206824447, -2.4165893571393982, -2.5721989891382115, 0.053725541111267816, -5.581018947417471], [-2.4533183090008377, -7.2816389591039314, -6.3144108706611091, -2.0816535538540091, -6.2234839568572955], [0.46404936785944884, -0.16575722056217757, -9.0089848807480752, -2.4065704897937579, -3.9867697508482873]])+(1.-msk_ref)*numpy.array([[-4.5498889586652469, -5.2491379606856565, 0.49057276921606885, -3.0953256753245215, -8.4147164779967412], [0.46140199656177572, -0.046045202308443223, -8.6030625745101936, -2.8498257261503088, -6.8945059973754077], [-6.4172157361143896, -8.7992501638237783, -8.9214140563266966, -4.6288482646586138, -6.1479105560295437], [-5.0617714264922711, -4.8520429726955463, -9.004543584273204, -8.1367579693253909, -7.7636870631085539]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.0752794443931846, -0.43637499471748953], [2.2278665920637666, 0.77413324831947161]], [[-2.2621919280308722, 4.1097739213320743], [-2.1907740543645096, -1.635114589248178]], [[-3.8959687760277117, -2.3136464137997024], [-1.8943992277015207, -3.3407440640458996]], [[-0.045916977483444477, 2.1631279004742776], [1.9275438484077814, 2.3769980322443729]], [[1.8622108971052551, 3.2662658801762952], [2.2235414106020777, -0.627271482816675]], [[-2.3472912958055545, -3.4653800943437685], [-3.8261579236995624, 2.9759758417749875]]])+(1.-msk_arg0)*numpy.array([[[-3.8114018498092195, -2.1761677603314333], [-2.4218839310730034, -4.6836177509560049]], [[1.1556416930172588, 1.2957519371391113], [0.003343321390834042, 1.2798960430452668]], [[1.9003386904459161, -4.4934638079609357], [-0.33179238284804047, -0.26866558318268208]], [[-4.0745117273673301, -0.65238673023757876], [0.02805036566084862, -4.8391170861922408]], [[-1.1293909406535239, -4.9706071744480571], [0.4333705729691717, 0.89434117124274959]], [[0.099330104033136024, -3.2127701302240763], [2.4934013804565582, 1.1315519444268975]]])
      arg1=3.25303626824
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.3283157126345957, 2.8166612735239216], [5.4809028603051777, 4.0271695165608827]], [[0.99084434021053891, 7.3628101895734854], [1.0622622138769016, 1.6179216789932331]], [[-0.64293250778630062, 0.93938985444170875], [1.3586370405398904, -0.087707795804488509]], [[3.2071192907579666, 5.4161641687156887], [5.1805801166491925, 5.630034300485784]], [[5.1152471653466662, 6.5193021484177063], [5.4765776788434888, 2.6257647854247361]], [[0.90574497243585661, -0.21234382610235736], [-0.5731216554581513, 6.2290121100163987]]])+(1.-msk_ref)*numpy.array([[[-0.55836558156780836, 1.0768685079099778], [0.83115233716840775, -1.4305814827145937]], [[4.40867796125867, 4.5487882053805224], [3.2563795896322452, 4.532932311286678]], [[5.1533749586873272, -1.2404275397195246], [2.9212438853933707, 2.984370685058729]], [[-0.82147545912591902, 2.6006495380038324], [3.2810866339022597, -1.5860808179508297]], [[2.1236453275878873, -1.717570906206646], [3.6864068412105828, 4.1473774394841607]], [[3.3523663722745471, 0.040266138017334807], [5.7464376486979694, 4.3845882126683087]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.0431683365340199, -0.69771800735630496, -4.3564698306353931, -1.3186732467450968], [3.7178790427362731, -2.3605937587568371, -3.1093782760448887, 1.4974617675226884], [-4.4481172725364635, -1.6140792821597794, -2.8578829906076209, 2.9439294526468887]], [[-0.84952756298886811, -3.4189132634429944, 3.5938381093104326, 4.3754680161146311], [-2.5446606978365036, 0.97732180399877056, 1.3944420807059732, -3.7621946022590502], [2.04391534664357, 1.2489809466234778, 1.3202147467373226, 4.5007485359513506]]], [[[-4.866601265502462, 4.967878083821347, -0.76711633617188912, 0.45495528610266955], [-3.8822289139382016, 4.6885488645294888, 1.5495372419260436, 1.7538453570989629], [0.63900121338747162, -1.8020056459412603, 3.7165360916929888, -4.0546138674688823]], [[1.7663645453803722, -1.5826769001792842, -2.8136025784336907, -1.9464106651971571], [2.7234244034086732, 1.2388188036558745, -2.3428002737436424, 1.5953187814111853], [-0.66834207506626075, 2.2402558983520837, 2.4125414403740253, 3.170374992007849]]], [[[4.083138930907058, -2.8467099120505734, -0.42536738672227603, 3.1033407537903308], [4.8516454619050631, -3.9526114889483219, 3.3411398141142552, 2.5391521645357118], [3.5285645621702368, 0.045302494885445022, -4.9151597702687075, 4.7182719891057765]], [[-2.1184648842617069, -4.4230192024046744, 3.3962024060283795, -0.017380403434352765], [-3.9987175727009951, 4.2667344806757228, 1.2731638820431019, -1.5832362680510115], [-3.9556553447462472, 1.4082638422113725, 0.93857419169412815, 3.1610344065318561]]]])+(1.-msk_arg0)*numpy.array([[[[-3.5548422295121029, -4.5826403427245008, -1.333554831152127, -0.49306709311592734], [2.4489640768776608, 3.3925127105617161, -4.002167151204211, -2.6129126643723399], [-0.2232285155337177, 3.0336056015704571, -2.9470382945600768, 2.5163753137812179]], [[4.42738363417652, -1.2617195244891652, 3.3079857363828413, -4.8419360439442372], [0.95149668238483009, -1.7538690617497132, 4.6111037053606925, 2.3808601201443285], [-0.46180567801002503, 4.8684695110728651, 1.3959690121028601, 1.2481432448492482]]], [[[2.2997039628189313, 0.13623515820638765, -3.9546555110622794, -0.85926820869769038], [-2.0664097797973255, 0.38439582019159602, 1.3510886188364228, 1.6592844605239261], [4.3501350734055571, -2.4053041197272762, 0.74992230953703931, -1.0613611784785935]], [[-0.45959521265420467, 2.1396562332335032, -3.9908672807241152, 1.6071195926592861], [-2.23297405351496, 0.63174367343484139, 1.5071301349964035, 2.7369098916277927], [1.9462059275737271, 0.11505381396864056, -1.3178553488750944, -3.2838288969114471]]], [[[-4.9357167249717326, 2.5770342383201115, -4.2993990723075868, 0.97335626450623014], [-4.0488355013509603, 3.1707517683286319, -4.647840927273009, 4.8253843404118921], [0.13128572704198849, -3.1220157914746851, -4.3542398854132518, 3.6464319633177524]], [[3.2048893577144142, 3.3841826621674649, 0.22042125035934301, 1.5721310504602828], [2.6303223002778999, -4.6394626042680445, 2.5946256974505433, 0.47012672955558799], [2.1668695495313992, -2.8724577867963221, -4.5295967961860297, -2.1062217995650001]]]])
      arg1=0.702383063571
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.3407852729626182, 0.004665056215096719, -3.6540867670639914, -0.61629018317369511], [4.4202621063076748, -1.6582106951854354, -2.406995212473487, 2.1998448310940901], [-3.7457342089650618, -0.91169621858837768, -2.1554999270362192, 3.6463125162182903]], [[-0.14714449941746643, -2.7165301998715927, 4.2962211728818342, 5.0778510796860328], [-1.8422776342651019, 1.6797048675701722, 2.0968251442773749, -3.0598115386876485], [2.7462984102149717, 1.9513640101948795, 2.0225978103087243, 5.2031315995227523]]], [[[-4.1642182019310603, 5.6702611473927487, -0.064733272600487446, 1.1573383496740712], [-3.1798458503667999, 5.3909319281008905, 2.2519203054974453, 2.4562284206703646], [1.3413842769588733, -1.0996225823698587, 4.4189191552643905, -3.3522308038974806]], [[2.4687476089517739, -0.88029383660788252, -2.111219514862289, -1.2440276016257554], [3.4258074669800749, 1.9412018672272762, -1.6404172101722407, 2.297701844982587], [0.034040988505140923, 2.9426389619234854, 3.114924503945427, 3.8727580555792507]]], [[[4.7855219944784597, -2.1443268484791718, 0.27701567684912565, 3.8057238173617325], [5.5540285254764648, -3.2502284253769202, 4.0435228776856569, 3.2415352281071135], [4.2309476257416385, 0.7476855584568467, -4.2127767066973059, 5.4206550526771782]], [[-1.4160818206903052, -3.7206361388332727, 4.0985854695997812, 0.68500266013704891], [-3.2963345091295935, 4.9691175442471245, 1.9755469456145036, -0.8808532044796098], [-3.2532722811748456, 2.1106469057827741, 1.6409572552655298, 3.8634174701032578]]]])+(1.-msk_ref)*numpy.array([[[[-2.8524591659407013, -3.8802572791530991, -0.63117176758072535, 0.20931597045547434], [3.1513471404490625, 4.0948957741331178, -3.2997840876328093, -1.9105296008009383], [0.47915454803768398, 3.7359886651418588, -2.2446552309886751, 3.2187583773526196]], [[5.1297666977479217, -0.55933646091776357, 4.010368799954243, -4.1395529803728355], [1.6538797459562318, -1.0514859981783116, 5.3134867689320942, 3.0832431837157301], [0.24057738556137664, 5.5708525746442668, 2.0983520756742617, 1.9505263084206499]]], [[[3.002087026390333, 0.83861822177778933, -3.2522724474908777, -0.15688514512628871], [-1.3640267162259239, 1.0867788837629977, 2.0534716824078245, 2.3616675240953278], [5.0525181369769587, -1.7029210561558745, 1.452305373108441, -0.35897811490719178]], [[0.24278785091719701, 2.8420392968049049, -3.2884842171527136, 2.3095026562306877], [-1.5305909899435584, 1.3341267370062431, 2.2095131985678051, 3.4392929551991944], [2.6485889911451288, 0.81743687754004224, -0.61547228530369269, -2.5814458333400454]]], [[[-4.233333661400331, 3.2794173018915131, -3.5970160087361851, 1.6757393280776318], [-3.3464524377795586, 3.8731348319000336, -3.9454578637016073, 5.5277674039832938], [0.83366879061339016, -2.4196327279032834, -3.6518568218418501, 4.3488150268891541]], [[3.9072724212858159, 4.0865657257388666, 0.92280431393074469, 2.2745141140316845], [3.3327053638493016, -3.9370795406966428, 3.297008761021945, 1.1725097931269897], [2.8692526131028009, -2.1700747232249205, -3.8272137326146281, -1.4038387359935984]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.27422201347)+(1.-msk_arg0)*(-3.04737470368)
      arg1=numpy.array(-1.87084174262)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-6.14506375609)+(1.-msk_ref)*(-4.91821644631)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.63056586517266489, -4.1969680638112745])+(1.-msk_arg0)*numpy.array([4.9344402665439873, -1.8674247699704329])
      arg1=numpy.array(-0.38556751237)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.24499835280279747, -4.5825355761811419])+(1.-msk_ref)*numpy.array([4.5488727541741198, -2.2529922823403004])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.60968018003)+(1.-msk_arg0)*(-3.82371339071)
      arg1=numpy.array([-2.6762091976030069, -0.28669926205929741])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.9334709824270524, 3.3229809179707619])+(1.-msk_ref)*numpy.array([-6.4999225883139644, -4.1104126527702549])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.0742867800742175, 0.33522402728508016, 0.79716874533671689, 2.1599403435304021, 0.15863012862783776], [-2.5680119581242788, -2.5328416705573273, 3.0465576822371148, -3.4097268180051135, 2.6122200260688144], [2.1289820627681202, 2.7906765458799656, 4.9945477569237724, 2.0970777502539386, 1.3964517047552114], [-0.36294181513659129, -2.1161381735044671, -4.2714171381678199, -4.5050492185816404, -4.9976776319524792]])+(1.-msk_arg0)*numpy.array([[-0.34287343889490973, -0.12484288984756642, -2.2194401373457873, -2.4568613512842239, 2.2907372756973619], [-2.7145103156409545, 2.6096219911741461, 0.19635432633352767, 4.9621397924935238, 3.1916597167104523], [0.15170674773061332, -2.9744988962006049, 1.0597907082786122, 0.22093838154279233, -4.9697880416380338], [1.1842654012729472, -0.73745697381149977, 1.1230206666065286, -2.2044757333853706, 3.8293276710387918]])
      arg1=numpy.array(-2.18021211595)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-6.2544988960227883, -1.8449880886634906, -1.3830433706118539, -0.020271772418168688, -2.021581987320733], [-4.7482240740728496, -4.7130537865058981, 0.86634556628854398, -5.5899389339536842, 0.43200791012024364], [-0.051230053180450597, 0.61046442993139483, 2.8143356409752016, -0.083134365694632173, -0.78376041119335937], [-2.5431539310851621, -4.2963502894530379, -6.4516292541163907, -6.6852613345302112, -7.1778897479010499]])+(1.-msk_ref)*numpy.array([[-2.5230855548434805, -2.3050550057961372, -4.3996522532943576, -4.6370734672327947, 0.11052515974879107], [-4.8947224315895248, 0.42940987522557528, -1.9838577896150431, 2.781927676544953, 1.0114476007618816], [-2.0285053682179575, -5.1547110121491757, -1.1204214076699586, -1.9592737344057785, -7.1500001575866046], [-0.99594671467562357, -2.9176690897600706, -1.0571914493420422, -4.3846878493339414, 1.649115555090221]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.50771602082)+(1.-msk_arg0)*(0.433449735114)
      arg1=numpy.array([[0.64374989441147967, 3.1256765495822627, 2.8810072422959978, 3.9439424333999291, -0.69651771481486868], [-3.0630289014693703, 0.5879337203117263, 2.5299094300624176, -3.2414157544516229, 0.72273043440297169], [4.8120746448714264, -4.2288093153982178, -2.2090930370449957, 1.2250015803343057, 4.2303987191393251], [1.2621522032488297, -4.271181967403801, 0.73621783071942914, 2.9519257273840154, 1.0001620194901104]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.8639661264097667, -0.38203947123898363, -0.62670877852524853, 0.43622641257868278, -4.204233735636115], [-6.5707449222906167, -2.91978230050952, -0.9778065907588287, -6.7491317752728692, -2.7849855864182747], [1.3043586240501801, -7.7365253362194641, -5.716809057866242, -2.2827144404869406, 0.72268269831807874], [-2.2455638175724166, -7.7788979882250473, -2.7714981901018172, -0.55579029343723096, -2.5075540013311359]])+(1.-msk_ref)*numpy.array([[1.0771996295258104, 3.5591262846965934, 3.3144569774103285, 4.3773921685142598, -0.26306797970053797], [-2.6295791663550396, 1.021383455426057, 2.9633591651767484, -2.8079660193372922, 1.1561801695173024], [5.2455243799857572, -3.7953595802838871, -1.775643301930665, 1.6584513154486364, 4.6638484542536558], [1.6956019383631604, -3.8377322322894702, 1.1696675658337599, 3.3853754624983461, 1.4336117546044411]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-2.3348446786987176, -3.8910416664397598], [-0.10370096936416129, -4.9734900975580301]], [[4.7331328936671717, 1.2186249425091935], [3.270544692783119, 1.3623560008926772]], [[-2.3350535323498245, 3.2321843965661294], [-2.0356592551566353, 1.7259075413575289]], [[4.6154787714690606, -4.4986800386009005], [-1.9577775311091905, -4.9381083681658522]], [[-3.5404089812115402, -3.9353976464096716], [0.67573951593700698, -1.3947999702295077]], [[-4.0963849146396356, -0.46525659179587375], [-1.1370654828230364, -2.1600033933839424]]])+(1.-msk_arg0)*numpy.array([[[-1.903370372702442, 4.8961157893865543], [-2.5697801638487885, 4.1905260696446636]], [[-1.3889443845578739, 1.9032605872129613], [-0.080862748535345474, -2.0186840902049785]], [[0.56339847422606937, 1.5973535208817138], [-4.0348648337327129, -4.3297530101586155]], [[-4.4383275393439705, 1.7272120439306491], [-3.4952484390828884, 1.5191321095418022]], [[0.66501334916551613, -3.1428713137384379], [-2.3691960451273286, 2.6285270801417679]], [[2.4414014961765238, -4.8672791635276171], [2.5057443896735805, 2.848919968287067]]])
      arg1=numpy.array(-2.8901098974)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-5.2249545761024212, -6.781151563843463], [-2.9938108667678645, -7.8635999949617332]], [[1.8430229962634685, -1.6714849548945097], [0.38043479537941582, -1.527753896511026]], [[-5.2251634297535272, 0.34207449916242627], [-4.9257691525603384, -1.1642023560461743]], [[1.7253688740653574, -7.3887899360046037], [-4.8478874285128937, -7.8282182655695554]], [[-6.4305188786152438, -6.8255075438133748], [-2.2143703814666962, -4.2849098676332105]], [[-6.9864948120433388, -3.3553664891995769], [-4.0271753802267396, -5.050113290787646]]])+(1.-msk_ref)*numpy.array([[[-4.7934802701061452, 2.0060058919828512], [-5.4598900612524917, 1.3004161722409604]], [[-4.2790542819615771, -0.98684931019074185], [-2.9709726459390486, -4.9087939876086821]], [[-2.3267114231776338, -1.2927563765219894], [-6.924974731136416, -7.2198629075623186]], [[-7.3284374367476737, -1.1628978534730541], [-6.3853583364865916, -1.3709777878619009]], [[-2.225096548238187, -6.0329812111421415], [-5.2593059425310322, -0.26158281726193522]], [[-0.44870840122717937, -7.7573890609313203], [-0.38436550773012268, -0.041189929116636215]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.29209757812)+(1.-msk_arg0)*(-0.981425823317)
      arg1=numpy.array([[[1.2283360489932029, -1.9821921319352387], [3.7065584330160277, 0.22420876449978877]], [[3.6096353750496881, 3.2035181028588688], [-3.6025600256770387, 4.0429184089713672]], [[-2.6654431621141752, 0.30692965453912979], [-2.7687826400626605, 3.3237725443425266]], [[-4.9995068830154912, -1.4426174036883674], [-3.1121811239980959, -3.6432429796568289]], [[3.3084581992593218, -1.4852287015127819], [-2.613203727640665, 0.77110845778391468]], [[-1.5786520105886459, 2.1999367543422057], [4.7141263661090278, -4.8357369954508522]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.0637615291273681, -5.2742897100558093], [0.4144608548954567, -3.0678888136207823]], [[0.3175377969291171, -0.088579475261702179], [-6.8946576037976097, 0.75082083085079621]], [[-5.9575407402347462, -2.9851679235814412], [-6.0608802181832315, 0.031674966221955536]], [[-8.2916044611360622, -4.7347149818089385], [-6.4042787021186669, -6.9353405577773994]], [[0.016360621138750808, -4.7773262796333533], [-5.9053013057612365, -2.5209891203366563]], [[-4.8707495887092165, -1.0921608237783653], [1.4220287879884568, -8.1278345735714232]]])+(1.-msk_ref)*numpy.array([[[0.24691022567596388, -2.9636179552524777], [2.7251326096987887, -0.75721705881745027]], [[2.6282095517324491, 2.2220922795416298], [-4.5839858489942777, 3.0614925856541282]], [[-3.6468689854314142, -0.67449616877810925], [-3.7502084633798995, 2.3423467210252875]], [[-5.9809327063327302, -2.4240432270056065], [-4.0936069473153349, -4.6246688029740675]], [[2.3270323759420828, -2.4666545248300209], [-3.594629550957904, -0.21031736553332436]], [[-2.5600778339058849, 1.2185109310249667], [3.7327005427917888, -5.8171628187680913]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.9686894541265589, 1.3035878460348407, -3.5786729198596365, -2.8139260922608331], [-0.84024366813642803, 3.6476371097953333, 4.0039933720834533, -0.70922660543681371], [4.2837815729052195, -1.3932330158759907, -2.2666127802343947, 0.12735574283345663]], [[-1.002337197208881, 0.91080816554581023, -4.2134827017911896, -4.2308729877014226], [0.91953991641489008, -2.7832859546563684, -1.7362347501993747, -3.4593439639123567], [-2.7429159945706738, 4.1535982117897703, 1.4308678057314905, 4.0090461159857202]]], [[[3.2452459340634849, 0.75389152853921892, 2.4893017239825612, -0.056399098378761892], [1.678989856532608, -0.66733666348371745, 1.2944140675165192, -0.8908597236449376], [1.236023215223204, -2.9986981445804997, -1.7239648964827605, 4.6449935624086685]], [[-4.4367104442433778, 0.21164332248252737, -3.5856712486172384, 0.55304142314070415], [-2.782297748445778, -1.9362867886212412, 2.4401261643367986, -2.4434879218476335], [0.70483237055788983, -4.2688904098071125, -3.9568053294170937, 2.2501110674929272]]], [[[-2.3453955388264158, 1.2371800097762211, 1.0354746691451711, 3.2808002182189782], [1.5969840788822998, -1.9655604242857261, 0.72128560254471541, -4.4582804919385026], [3.5398599406883786, 1.3469813232398939, 4.2158067826608399, -2.6582987497119293]], [[-3.6647757460681007, 2.3094345078452969, 4.6628844230723008, -3.2008977105869918], [3.6630249470898466, -1.7271960673337139, -4.8546122299642835, 3.5017842945819595], [1.6560878857403765, -4.7411271383821294, 0.52811521466282318, -0.79346990000426576]]]])+(1.-msk_arg0)*numpy.array([[[[-3.31979433456429, -2.477190713209525, 4.2253526729050268, 4.8662005080433843], [3.0713598278307099, -4.4416391447185042, -1.7863111569740808, -2.3716795799516643], [-1.9765235559210872, -0.55153568329043701, 0.48345371198147546, -3.5950371210598453]], [[0.04709665706162447, -4.5142552954785629, 1.3927340477265897, 1.593833723814539], [2.2080677121877965, -2.0440883984601066, 0.84280211026229512, -3.7429448623286743], [0.064699064047752941, 2.1990748620052116, 3.2169747149446586, 0.26167642898680032]]], [[[1.0992805704327182, -3.5505048708939277, -1.730860950834856, -2.7006720898753187], [-1.2827336454259983, -1.698561693901377, 3.7856506156131005, -0.94543698158558787], [-3.3377631503816305, -3.2450103056849944, -4.659265709456907, 1.1416375558872023]], [[4.7701914889329391, -4.7332542972075835, -0.14974687822323052, -0.71089074903618155], [1.7209783122601774, 2.3119075064318553, 2.5134622786490617, -1.7665647396667139], [-1.1961666289223327, -1.6057149265217738, 3.9773255120122997, 4.9708409409040932]]], [[[0.26346141913522114, 4.683960312123558, -3.7094024128825676, 0.059931185463049985], [-0.44647109806008611, 0.88796359312811379, 2.5589239112253246, 2.7609749673658648], [3.9535080514209824, 0.14355150199176414, 3.3338914430855873, 4.9253827774947272]], [[3.8921155836792511, -0.17961492745737395, -0.49795579269644819, 4.0797158096168875], [3.5042708059171055, 3.4156327736418763, 3.1936983428665489, -1.9001603752000165], [-1.4290755607250749, -0.0033064383098313499, 2.9369492926801657, -1.6645135699622191]]]])
      arg1=numpy.array(-2.64739680501)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.6787073508870467, -1.343808958978765, -6.2260697248732422, -5.4613228972744388], [-3.4876404731500337, 1.0002403047817277, 1.3565965670698477, -3.3566234104504193], [1.6363847678916139, -4.0406298208895963, -4.9140095852480004, -2.520041062180149]], [[-3.6497340022224867, -1.7365886394677954, -6.8608795068047952, -6.8782697927150283], [-1.7278568885987156, -5.430682759669974, -4.3836315552129808, -6.1067407689259623], [-5.3903127995842794, 1.5062014067761647, -1.2165289992821151, 1.3616493109721146]]], [[[0.5978491290498793, -1.8935052764743867, -0.15809508103104442, -2.7037959033923675], [-0.96840694848099762, -3.3147334684973231, -1.3529827374970864, -3.5382565286585432], [-1.4113735897904016, -5.6460949495941053, -4.3713617014963662, 1.9975967573950628]], [[-7.0841072492569834, -2.4357534825310783, -6.233068053630844, -2.0943553818729015], [-5.4296945534593837, -4.5836835936348468, -0.20727064067680701, -5.0908847268612387], [-1.9425644344557158, -6.9162872148207182, -6.6042021344306994, -0.39728573752067842]]], [[[-4.992792343840021, -1.4102167952373845, -1.6119221358684346, 0.63340341320537252], [-1.0504127261313059, -4.6129572292993313, -1.9261112024688902, -7.1056772969521083], [0.89246313567477298, -1.3004154817737117, 1.5684099776472342, -5.3056955547255349]], [[-6.3121725510817068, -0.33796229716830872, 2.0154876180586951, -5.8482945156005979], [1.015628142076241, -4.3745928723473195, -7.5020090349778892, 0.85438748956835386], [-0.99130891927322917, -7.388523943395735, -2.1192815903507825, -3.4408667050178714]]]])+(1.-msk_ref)*numpy.array([[[[-5.9671911395778956, -5.1245875182231302, 1.5779558678914212, 2.2188037030297787], [0.4239630228171043, -7.0890359497321098, -4.433707961987686, -5.0190763849652704], [-4.6239203609346928, -3.1989324883040426, -2.1639430930321302, -6.2424339260734509]], [[-2.6003001479519812, -7.1616521004921685, -1.2546627572870159, -1.0535630811990666], [-0.43932909282580912, -4.6914852034737127, -1.8045946947513105, -6.3903416673422804], [-2.5826977409658527, -0.44832194300839401, 0.56957790993105295, -2.3857203760268053]]], [[[-1.5481162345808874, -6.1979016759075334, -4.3782577558484617, -5.3480688948889243], [-3.930130450439604, -4.3459584989149826, 1.1382538105994948, -3.5928337865991935], [-5.9851599553952362, -5.8924071106986, -7.3066625144705126, -1.5057592491264034]], [[2.1227946839193335, -7.3806511022211891, -2.7971436832368362, -3.3582875540497872], [-0.92641849275342825, -0.33548929858175036, -0.13393452636454395, -4.4139615446803191], [-3.8435634339359384, -4.2531117315353795, 1.329928706998694, 2.3234441358904876]]], [[[-2.3839353858783845, 2.0365635071099524, -6.3567992178961727, -2.5874656195505557], [-3.0938679030736917, -1.7594332118854918, -0.088472893788281048, 0.11357816235225915], [1.3061112464073767, -2.5038453030218415, 0.68649463807198163, 2.2779859724811216]], [[1.2447187786656455, -2.8270117324709796, -3.1453525977100538, 1.4323190046032819], [0.85687400090349986, 0.7682359686282707, 0.54630153785294322, -4.5475571802136221], [-4.076472365738681, -2.650703243323437, 0.28955248766656005, -4.3119103749758247]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.98958583366)+(1.-msk_arg0)*(2.07183750021)
      arg1=numpy.array([[[[-2.0035554961638926, -4.0629277374904351, 3.6552044443621234, 1.8514247449812498], [-2.8882447984770723, 0.12329753513397623, -3.0168253512591878, -1.1062701687343948], [3.4366127172254117, 4.5729796726310532, 3.4592581446264887, 2.6621921682600478]], [[3.5344357120438534, -2.9486064088435535, 0.0863280079732478, 1.7158357155926698], [2.6911434842834456, -0.67881655333002033, -0.42169505058291623, 1.9763555068235732], [4.4919921505182767, 1.9875760702252485, -2.6650788898627873, 0.08293265962955676]]], [[[-1.0710569811694617, -4.260824003508529, -0.2041483567347866, 2.4369135481746014], [3.6956122747146516, 0.55618143478718984, -3.3838050300299569, -1.3708045441353267], [-0.23469238817923355, 2.3960763296173138, 1.5825323246963832, -2.8121629955311986]], [[2.4468665689982974, 3.3085740148885439, -3.6892028667433676, -1.1661721360027197], [-4.5209496542087289, -0.17564680359305207, -2.8597348433310876, 3.9475485227790603], [1.1873760110995395, -2.8310694742463385, -0.61520111460484728, 0.20923902236297565]]], [[[-0.62877908060276333, 0.21110085164881554, 3.9295234278512545, 2.3022804866263407], [1.3470902256654806, 2.1127166714245993, 2.7830483978495559, -0.090703350334061739], [-2.258937428587251, -1.275847292968372, 1.3695546592200873, -4.5557308496974667]], [[3.8284618249663147, 0.81851043834555171, -0.32086472424367241, -4.8896951297523339], [-0.62322859946999021, -0.4248731942212256, -3.3601984425330889, 1.8890172992071044], [0.39309967521342593, -4.7064505000648316, -3.287080728615559, -3.6247600412799672]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-4.9931413298212659, -7.0525135711478084, 0.66561861070475015, -1.1381610886761235], [-5.8778306321344456, -2.8662882985233971, -6.0064111849165611, -4.0958560023917681], [0.44702688356803844, 1.5833938389736799, 0.46967231096911544, -0.32739366539732551]], [[0.54484987838648014, -5.9381922425009268, -2.9032578256841255, -1.2737501180647035], [-0.29844234937392766, -3.6684023869873936, -3.4112808842402895, -1.0132303268338001], [1.5024063168609034, -1.0020097634321248, -5.6546647235201606, -2.9066531740278165]]], [[[-4.060642814826835, -7.2504098371659023, -3.1937341903921599, -0.55267228548277192], [0.7060264410572783, -2.4334043988701834, -6.3733908636873302, -4.3603903777927], [-3.2242782218366068, -0.59350950404005953, -1.4070535089609901, -5.8017488291885719]], [[-0.54271926465907594, 0.31898818123117056, -6.6787887004007409, -4.155757969660093], [-7.5105354878661021, -3.1652326372504254, -5.8493206769884605, 0.95796268912168703], [-1.8022098225578338, -5.8206553079037118, -3.6047869482622206, -2.7803468112943976]]], [[[-3.6183649142601366, -2.7784849820085578, 0.93993759419388123, -0.68730534703103263], [-1.6424956079918926, -0.876869162232774, -0.20653743580781736, -3.080289183991435], [-5.2485232622446247, -4.2654331266257453, -1.6200311744372859, -7.54531668335484]], [[0.83887599130894142, -2.1710753953118216, -3.3104505579010457, -7.8792809634097072], [-3.6128144331273635, -3.4144590278785989, -6.3497842761904622, -1.1005685344502689], [-2.5964861584439474, -7.6960363337222049, -6.2766665622729327, -6.6143458749373405]]]])+(1.-msk_ref)*numpy.array([[[[0.068282004042214517, -1.991090237284328, 5.7270419445682306, 3.9232622451873569], [-0.81640729827096514, 2.1951350353400834, -0.9449878510530807, 0.96556733147171236], [5.5084502174315189, 6.6448171728371603, 5.5310956448325959, 4.7340296684661549]], [[5.6062732122499606, -0.87676890863744639, 2.1581655081793549, 3.7876732157987769], [4.7629809844895528, 1.3930209468760868, 1.6501424496231909, 4.0481930070296803], [6.5638296507243838, 4.0594135704313556, -0.59324138965668016, 2.1547701598356639]]], [[[1.0007805190366454, -2.1889865033024218, 1.8676891434713205, 4.5087510483807085], [5.7674497749207587, 2.628018934993297, -1.3119675298238498, 0.70103295607078042], [1.8371451120268736, 4.4679138298234209, 3.6543698249024903, -0.74032549532509151]], [[4.5187040692044045, 5.380411515094651, -1.6173653665372605, 0.90566536420338739], [-2.4491121540026217, 1.8961906966130551, -0.7878973431249805, 6.0193860229851675], [3.2592135113056466, -0.75923197404023135, 1.4566363856012599, 2.2810765225690828]]], [[[1.4430584196033438, 2.2829383518549227, 6.0013609280573617, 4.3741179868324478], [3.4189277258715878, 4.1845541716307064, 4.8548858980556631, 1.9811341498720454], [-0.18709992838114387, 0.79599020723773517, 3.4413921594261945, -2.4838933494913595]], [[5.9002993251724218, 2.8903479385516588, 1.7509727759624347, -2.8178576295462268], [1.4486089007361169, 1.6469643059848815, -1.2883609423269817, 3.9608547994132115], [2.4649371754195331, -2.6346129998587244, -1.2152432284094519, -1.5529225410738601]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.47706500329972812, 4.3545479082439389])+(1.-msk_arg0)*numpy.array([-0.13102613993303169, 2.2263913173722667])
      arg1=numpy.array([0.98345232813450512, 3.8325413280529759])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.4605173314342332, 8.1870892362969148])+(1.-msk_ref)*numpy.array([0.85242618820147342, 6.0589326454252426])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.8446216313907478, -2.670128420605542], [0.69915547992621363, -3.8243177985962884]])+(1.-msk_arg0)*numpy.array([[4.4122297848047296, 2.6155406524343192], [-1.7895423156381938, 4.3288405236108805]])
      arg1=numpy.array([-4.5514528226341469, 0.16436748013170899])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.2931688087566009, -7.2215812432396884], [0.86352296005792262, -3.6599503184645794]])+(1.-msk_ref)*numpy.array([[-0.13922303782941725, -1.9359121701998276], [-1.6251748355064848, 4.4932080037425894]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.76826951984328407, -3.0544740871214238])+(1.-msk_arg0)*numpy.array([-4.7903239403220184, 0.66697837024991635])
      arg1=numpy.array([[0.18426375088907143, -4.0988908579433083], [3.4331705842844773, 2.4107805276224212]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.58400576895421263, -4.8671603777865924], [0.37869649716305354, -0.64369355949900253]])+(1.-msk_ref)*numpy.array([[-4.6060601894329469, -8.8892147982653267], [4.1001489545343937, 3.0777588978723376]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.89086462688881074, -3.6096581681695108, 4.5297677434883674, -4.0957644852566775, -4.4334876080715873], [-4.1716835737057432, 1.8722413196872258, -2.1244734130732468, -4.1139242230248954, -3.538772634619721], [-1.2883170912074737, 2.4206951797135412, 2.5111374762174288, 0.94067061991384016, -1.5663120598877978], [2.0699055794199914, -1.9595879187698548, 0.38980600930455012, 0.17890621356437819, -1.2567214899689363]], [[-0.39923104589107528, 2.0212488652471174, 0.0029685689285141592, 4.3918669722957677, 1.5542134545379458], [1.4829790708890345, 1.6582356736788908, 0.091545737681193451, 0.7217042508226168, 3.1686614189907001], [-0.81873528900577774, 2.1941028424733311, -3.9874331760103745, 0.67783199729320653, -1.0722433040887824], [-2.477457559766826, -0.61394079888474984, 2.4943804474728974, 4.1542288278060706, 3.5376538748032633]]])+(1.-msk_arg0)*numpy.array([[[0.3744298688011547, -0.62652550020347864, -4.8210918637674229, -3.957366829142174, 0.74221086295585614], [-0.4907394065795847, -4.6667906659566274, -2.4410961269650122, -2.5404838711774245, -2.0700628299947832], [-0.30375870474310673, -3.7046640917679499, 1.3729630385818119, -4.1642830770890713, -4.5469423543172098], [1.0053634652520929, 2.1126832807844709, -0.44755587239693639, 0.31230839924080289, 2.8405340701075605]], [[1.4145337458293605, -0.6587192598206002, -4.6922293638948336, 4.295548235830303, -3.2318050629006301], [-1.5321953425492096, -0.55364654224383525, -0.86944643087898221, -4.6242015281265783, 1.546244433801915], [3.1812451309172047, -2.9909659868978933, 0.62188116445822317, 3.6405975089754463, 1.4398600580223206], [1.1850776868031643, 0.54298894777953777, 1.5367860309547234, -4.3625260937193726, 4.9424221045697418]]])
      arg1=numpy.array([-2.9020348961474363, -1.7623733754628077])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.0111702692586255, -6.5116930643169475, 1.6277328473409312, -6.9977993814041142, -7.3355225042190231], [-7.0737184698531799, -1.0297935764602104, -5.0265083092206826, -7.0159591191723312, -6.4408075307671577], [-4.1903519873549104, -0.48133971643389506, -0.39089741993000748, -1.9613642762335961, -4.4683469560352336], [-0.83212931672744483, -4.8616228149172915, -2.5122288868428861, -2.7231286825830581, -4.158756386116373]], [[-2.161604421353883, 0.25887548978430974, -1.7594048065342935, 2.62949359683296, -0.20815992092486191], [-0.27939430457377323, -0.1041377017839169, -1.6708276377816142, -1.0406691246401909, 1.4062880435278924], [-2.5811086644685854, 0.43172946701052339, -5.7498065514731822, -1.0845413781696012, -2.8346166795515901], [-4.2398309352296337, -2.3763141743475575, 0.73200707201008974, 2.3918554523432629, 1.7752804993404556]]])+(1.-msk_ref)*numpy.array([[[-2.5276050273462816, -3.5285603963509149, -7.7231267599148588, -6.8594017252896098, -2.1598240331915801], [-3.392774302727021, -7.5688255621040632, -5.3431310231124485, -5.4425187673248612, -4.9720977261422199], [-3.205793600890543, -6.6066989879153866, -1.5290718575656244, -7.066317973236508, -7.4489772504646456], [-1.8966714308953434, -0.78935161536296539, -3.3495907685443727, -2.5897264969066334, -0.061500826039875722]], [[-0.34783962963344717, -2.4210926352834079, -6.4546027393576413, 2.5331748603674953, -4.9941784383634378], [-3.2945687180120173, -2.3160199177066429, -2.6318198063417899, -6.386574903589386, -0.21612894166089269], [1.418871755454397, -4.753339362360701, -1.1404922110045845, 1.8782241335126386, -0.32251331744048706], [-0.57729568865964342, -1.2193844276832699, -0.22558734450808426, -6.1248994691821803, 3.1800487291069341]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.5116217715959337, 3.4189592787088916])+(1.-msk_arg0)*numpy.array([1.307465776807466, -4.8560141264068406])
      arg1=numpy.array([[[-1.7878665417288184, -2.8492746115529668, -1.1205956183416141, -1.4347366911583768, -2.2681150034504851], [4.1798585530258343, -3.210527943809558, 3.3446718857447877, -0.40727168480971088, 3.1792866193858789], [-3.5109440751490872, -0.054042962239915227, -1.2011129989369596, 0.30825965798229138, -2.8197674803360071], [4.8444726352118419, 1.8940312425167445, 2.6913638978742425, 4.9129154538491022, 3.0434862855622882]], [[-1.2572084825406069, 0.090988644959574039, -0.31565914840584064, 0.56790585296926466, 3.6283415055968469], [4.515715126224265, -4.1806880700174442, -1.6579828704812605, -2.2349573329633419, -2.6741999926705029], [-2.7324018948522499, -1.4288162905785518, -3.4518399656786034, -2.2849573329598738, -1.4565922729], [-1.755560545645932, 4.3205273657517456, -4.5348790273813115, -1.3800490501497817, 1.9235989383234937]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.72375522986711527, -0.3376528399570331, 1.3910261532543196, 1.0768850804375569, 0.24350676814544858], [6.691480324621768, -0.69890617221362428, 5.8562936573407214, 2.1043500867862228, 5.6909083909818126], [-0.99932230355315355, 2.4575788093560185, 1.3105087726589741, 2.8198814295782251, -0.30814570874007341], [7.3560944068077756, 4.4056530141126782, 5.2029856694701762, 7.4245372254450359, 5.5551080571582219]], [[2.1617507961682847, 3.5099479236684656, 3.103300130303051, 3.9868651316781563, 7.0473007843057385], [7.9346744049331566, -0.76172879130855264, 1.7609764082276311, 1.1840019457455497, 0.7447592860383887], [0.68655738385664167, 1.9901429881303399, -0.032880686969711803, 1.1340019457490178, 1.9623670058088916], [1.6633987330629596, 7.7394866444606372, -1.1159197486724199, 2.0389102285591099, 5.3425582170323853]]])+(1.-msk_ref)*numpy.array([[[-0.48040076492135242, -1.5418088347455008, 0.18687015846585187, -0.12727091435091076, -0.96064922664301911], [5.4873243298333003, -1.903062167002092, 4.6521376625522537, 0.90019409199775513, 4.4867523961933449], [-2.2034782983416212, 1.2534228145675508, 0.10635277787050645, 1.6157254347897574, -1.5123017035285411], [6.1519384120193079, 3.2014970193242105, 3.9988296746817085, 6.2203812306565682, 4.3509520623697542]], [[-6.1132226089474475, -4.7650254814472666, -5.1716732748126812, -4.2881082734375759, -1.2276726208099937], [-0.34029900018257564, -9.0367021964242848, -6.5139969968881015, -7.0909714593701825, -7.5302141190773435], [-7.588416021259091, -6.2848304169853924, -8.307854092085444, -7.140971459366714, -6.3126063993068406], [-6.6115746720527726, -0.53548676065509504, -9.3908931537881521, -6.2360631765566223, -2.9324151880833469]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.8054343454705695, 3.9077079557151464], [-1.2109967429252655, -4.8952809511841338]], [[-3.6155018686463078, -4.2632978014608209], [-3.3426300235708051, 1.6324854372528659]], [[4.743919428726592, 0.16456277168957101], [0.059706180378297447, 2.0494661285709137]], [[-1.9534691387729519, 4.0563835695438648], [-4.4033851858056376, -4.411923147604309]], [[0.23937497732649771, -0.49206794997548631], [-2.955711422353259, 3.1783786665321632]], [[0.98795073663021338, 1.9834379921007912], [1.6655257632292191, 4.5308306534674418]]], [[[-1.7517328278805522, 4.3995674059326255], [4.5422883117536692, 4.7868135697106435]], [[-2.1301315211324692, -1.380552068738452], [-1.5498964115531022, 2.1533331215265648]], [[4.3672189182650882, -3.450140746879736], [3.1108477428114316, -1.0748645092166131]], [[-4.6815017605539477, -3.2772151095085511], [0.12671360266675347, 2.488753056957977]], [[2.1629476883277823, -1.767200271588397], [-0.14545887038465732, -1.1625821043516482]], [[3.6565328820822067, 1.9853552825152541], [-2.0342498327901302, -0.25039054609790412]]]])+(1.-msk_arg0)*numpy.array([[[[3.0366230563620533, 2.6774341812073228], [2.8232527527543816, -3.9484381228636223]], [[3.6799265038876996, 0.97167081276324296], [-0.13723253473939678, 3.3642965531671258]], [[1.2609266725722694, -1.6563626403468392], [1.4619516310520506, -4.097413493941362]], [[-2.9254138524716113, -2.3688027455116734], [2.7763731799467237, 3.3877671951826791]], [[-0.044568175413688849, -1.8547459326355078], [3.4039199448197284, -2.4185042164315416]], [[-3.6902289860567352, 3.1757670736058596], [2.899236998578429, 4.7979431831476766]]], [[[-3.0767612184480497, -2.5097528981453667], [-3.1268355666899628, -2.0486216456678252]], [[2.917209271131239, -1.6341754485217064], [-0.17977089414990566, -1.002989390165431]], [[4.8818628894813596, 0.44627327333682487], [2.5387658960023751, -4.34304313480642]], [[4.6267860656436586, -0.79670406118799697], [-4.1767429647996579, 0.073203357334929109]], [[-2.9431237125029863, 3.910777747154194], [2.4938852059608605, -3.8625141674299455]], [[-0.89319182109830386, -2.8842197071743545], [1.5372603543990824, -2.7276240351316359]]]])
      arg1=numpy.array([2.9544324737721244, 3.3962319271659336])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.7598668192426938, 6.8621404294872708], [1.7434357308468589, -1.9408484774120094]], [[-0.66106939487418348, -1.3088653276886966], [-0.38819754979868071, 4.5869179110249902]], [[7.6983519024987164, 3.1189952454616954], [3.0141386541504218, 5.0038986023430381]], [[1.0009633349991724, 7.0108160433159892], [-1.4489527120335133, -1.4574906738321847]], [[3.1938074510986221, 2.4623645237966381], [-0.0012789485811346424, 6.1328111403042875]], [[3.9423832104023377, 4.9378704658729156], [4.6199582370013434, 7.4852631272395662]]], [[[1.6444990992853814, 7.7957993330985591], [7.9385202389196028, 8.1830454968765771]], [[1.2661004060334644, 2.0156798584274815], [1.8463355156128314, 5.5495650486924983]], [[7.7634508454310218, -0.053908819713802458], [6.5070796699773652, 2.3213674179493204]], [[-1.2852698333880141, 0.11901681765738248], [3.5229455298326871, 5.8849849841239106]], [[5.5591796154937159, 1.6290316555775366], [3.2507730567812763, 2.2336498228142854]], [[7.0527648092481403, 5.3815872096811876], [1.3619820943758034, 3.1458413810680295]]]])+(1.-msk_ref)*numpy.array([[[[5.9910555301341777, 5.6318666549794472], [5.777685226526506, -0.99400564909149791]], [[6.634358977659824, 3.9261032865353673], [2.8171999390327276, 6.3187290269392502]], [[4.2153591463443938, 1.2980698334252851], [4.416384104824175, -1.1429810201692376]], [[0.029018621300513026, 0.58562972826045101], [5.7308056537188481, 6.3421996689548035]], [[2.9098642983584355, 1.0996865411366166], [6.3583524185918527, 0.5359282573405828]], [[-0.73579651228461085, 6.130199547377984], [5.8536694723505533, 7.752375656919801]]], [[[0.31947070871788386, 0.88647902902056686], [0.26939636047597082, 1.3476102814981084]], [[6.3134411982971725, 1.7620564786442272], [3.2164610330160279, 2.3932425370005026]], [[8.2780948166472932, 3.8425052005027585], [5.9349978231683087, -0.94681120764048643]], [[8.0230179928095922, 2.5995278659779366], [-0.78051103763372431, 3.4694352845008627]], [[0.45310821466294726, 7.3070096743201276], [5.8901171331267941, -0.46628224026401188]], [[2.5030401060676297, 0.51201221999157909], [4.933492281565016, 0.66860789203429771]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.15384151334399387, -0.018846239673883325])+(1.-msk_arg0)*numpy.array([-0.16388377723122538, 2.9775852759726638])
      arg1=numpy.array([[[[3.1016840285400953, -1.5279043644059618], [3.4863005394814781, -1.8645418980720994]], [[-2.2422082884183849, -4.6174615460961039], [2.5902247301962102, -0.43644679006284015]], [[4.2475925471187903, 2.085953594218922], [-3.3795193631259721, -4.8767123039300975]], [[4.5443414001690723, -2.3645650317300779], [1.0551736692395046, -1.4569442234360408]], [[-1.3148086495081523, 0.82213251959253331], [2.1004343525504492, 0.28757831438352532]], [[4.8513103708503031, -4.1518021655838586], [3.309784144704647, -0.13228590503314486]]], [[[3.2978505754897505, 4.8863314763391923], [-2.168146038077218, -0.92981188107844392]], [[-2.3814918108109162, -3.0757867764596147], [1.8833456466829714, 0.029166159188163121]], [[-3.6887481773438937, 0.06994165070569025], [-4.4011573896576994, 3.2063268483175236]], [[-2.815401503740953, 1.024722743592906], [-4.4724607971334143, 3.3287143485850006]], [[-2.5110224569208706, 0.27625511452291107], [-0.95329426403276329, 1.0525179899944312]], [[1.1961013302078216, -0.074742129512265443], [-0.52615026514951424, 3.7521567799832507]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.9478425151961014, -1.6817458777499557], [3.3324590261374842, -2.0183834114160932]], [[-2.3960498017623788, -4.7713030594400978], [2.4363832168522164, -0.59028830340683403]], [[4.0937510337747964, 1.9321120808749281], [-3.533360876469966, -5.0305538172740913]], [[4.3904998868250784, -2.5184065450740718], [0.90133215589551074, -1.6107857367800347]], [[-1.4686501628521462, 0.66829100624853943], [1.9465928392064553, 0.13373680103953145]], [[4.6974688575063093, -4.3056436789278525], [3.1559426313606531, -0.28612741837713873]]], [[[3.2790043358158671, 4.867485236665309], [-2.1869922777511013, -0.94865812075232725]], [[-2.4003380504847995, -3.0946330161334981], [1.8644994070090881, 0.010319919514279796]], [[-3.707594417017777, 0.051095411031806925], [-4.4200036293315828, 3.1874806086436402]], [[-2.8342477434148363, 1.0058765039190227], [-4.4913070368072976, 3.3098681089111173]], [[-2.5298686965947539, 0.25740887484902775], [-0.97214050370664662, 1.0336717503205479]], [[1.1772550905339383, -0.093588369186148768], [-0.54499650482339757, 3.7333105403093674]]]])+(1.-msk_ref)*numpy.array([[[[2.9378002513088699, -1.6917881416371872], [3.3224167622502527, -2.0284256753033247]], [[-2.4060920656496103, -4.7813453233273293], [2.4263409529649849, -0.60033056729406553]], [[4.0837087698875649, 1.9220698169876966], [-3.5434031403571975, -5.0405960811613229]], [[4.3804576229378469, -2.5284488089613033], [0.89128989200827924, -1.6208280006672662]], [[-1.4786924267393777, 0.65824874236130793], [1.9365505753192238, 0.12369453715229994]], [[4.6874265936190778, -4.315685942815084], [3.1459003674734216, -0.29616968226437024]]], [[[6.2754358514624142, 7.8639167523118561], [0.80943923789544581, 2.0477733948942198]], [[0.59609346516174755, -0.098201500486950977], [4.8609309226556352, 3.0067514351608269]], [[-0.71116290137122995, 3.047526926678354], [-1.4235721136850357, 6.1839121242901873]], [[0.16218377223171077, 4.0023080195655698], [-1.4948755211607505, 6.3062996245576644]], [[0.4665628190517932, 3.2538403904955748], [2.0242910119399005, 4.030103265967095]], [[4.1736866061804854, 2.9028431464603983], [2.4514350108231495, 6.7297420559559145]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.2170299635947934, -1.8599530988618351, 3.7076862699018633, 4.4890080984082754, -3.3513206689159878], [-2.0049276014467745, 4.3866924617592424, -1.2048830812671776, -1.6630846950003839, -0.31395050190329599], [-0.38356034873013556, 0.22901797948903813, -4.2666098154847436, -0.80816655822936401, -1.7824308483807529], [1.3306405391291261, -3.2221928410956844, 1.1012897571449844, -2.8937679026573973, 4.8203134779040653]])+(1.-msk_arg0)*numpy.array([[-3.2559523092622911, 2.7232804145587544, -4.9442296800214312, -0.28359785946932359, -3.2073194439416355], [3.5307020980480885, 3.1885377808939825, -3.2419564621287833, 1.7882246360483798, 2.5474242979779573], [1.2256708214417067, -4.6907585208103608, -2.9978647408156309, 0.68305825598849879, 0.17257044141507905], [-4.1860053408122031, -3.5891532280526284, 2.7174894229920206, -1.4705448066249316, 1.2180525217787448]])
      arg1=numpy.array([[-0.63546783460552536, -0.066149431371557377, 0.70684229336854898, 2.6454389402750875, 4.8243766925486788], [-2.3810824289100574, -0.084440467077174119, -4.2936653529613444, 0.71834151899033394, 1.0533400529193129], [-4.1663478965429457, 0.49243062668646065, 2.2959428085498885, -0.32416259353509691, 0.70123192948733948], [1.3636238116240973, -1.997727732830159, 2.8211904114904618, -1.9630461639762284, 1.5974381082932956]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.5815621289892681, -1.9261025302333925, 4.4145285632704123, 7.1344470386833629, 1.4730560236326911], [-4.3860100303568323, 4.3022519946820683, -5.498548434228522, -0.94474317601004998, 0.73938955101601689], [-4.5499082452730812, 0.72144860617549877, -1.9706670069348551, -1.1323291517644609, -1.0811989188934135], [2.6942643507532233, -5.2199205739258439, 3.9224801686354462, -4.8568140666336257, 6.4177515861973609]])+(1.-msk_ref)*numpy.array([[-3.8914201438678164, 2.657130983187197, -4.2373873866528822, 2.3618410808057639, 1.6170572486070434], [1.1496196691380312, 3.1040973138168084, -7.5356218150901277, 2.5065661550387137, 3.6007643508972702], [-2.940677075101239, -4.1983278941239002, -0.7019219322657424, 0.35889566245340188, 0.87380237090241852], [-2.8223815291881058, -5.5868809608827874, 5.5386798344824824, -3.43359097060116, 2.8154906300720404]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.090804256078954815, 3.2366473795118509], [-2.9916348587271777, -1.2025967015190711], [1.3938088996944931, 0.025057211760329601], [-4.581547627720556, 4.4248490367845772], [-1.8548281252376242, -0.74392370510229533]], [[-3.5371583785053917, 2.4266754816588652], [0.63651895212476717, -4.6980157836682936], [-4.1065211120354963, 3.0274481440898633], [-1.4918189830183257, -4.2651945509481113], [-0.38287683345291157, -3.3471584644933952]], [[3.1859169124671354, -2.8197832685298052], [0.80454899372305189, 0.70195029254341712], [4.268554144943435, -0.5303540083008409], [-3.4138711483983943, -4.0274250533222178], [2.5747831079943104, 0.10307456126413328]], [[2.7660144407824188, 4.5552026354409687], [-2.4627550077445362, 1.9595571963472445], [0.24403003936082968, -3.5720394249464746], [-2.4425897583363385, -2.0102260351745862], [-1.7877869356795282, 1.9137607313358078]]])+(1.-msk_arg0)*numpy.array([[[2.2579752985490664, -0.35714175159729056], [0.95898064595494148, 0.67926615205951446], [-1.4427579082435518, -2.7234691238063444], [-1.5806823810337747, 2.8178790642793841], [-2.70840951906436, 2.2179851991479271]], [[-3.6429959592592001, -2.9031041855055606], [-0.7527047157618636, -0.53896786209945979], [1.895330691308251, -3.9931129149183722], [2.2954355648775584, 3.4352506812345069], [4.754714167619893, 1.5115293874643019]], [[-0.46880668343364462, -0.69628630942340841], [0.41110087386237737, -0.98535099255006386], [-1.3704161242562583, 0.038919165696405855], [0.67724131682451727, -1.3835558213575738], [4.4325892945150773, -4.4193184714316969]], [[3.8506007035242433, 4.3324064053390323], [0.33368283768116846, -1.5575319159812837], [3.8391406952981306, 3.4478208998704556], [0.81002808636448353, -3.2032921710472815], [2.6508842287344141, -1.2440746106444167]]])
      arg1=numpy.array([[-3.746332135013116, -4.5329055212832241, -1.3473000680819993, -4.3347406459041942, 2.1151539811645694], [-3.7252561720708979, 4.0311592750848604, 0.74908949103428579, 1.5952026405330564, 0.562834402821486], [-0.36194909654608232, -4.3892388962502862, -3.4866125323700059, 3.0029372528071825, 2.889206807501445], [4.2540229033476891, 2.1152899702093189, -0.37039080258179791, -1.6280588649159613, -1.4536797609532615]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.6555278789341612, -0.50968475550126513], [-7.5245403800104018, -5.7355022228022952], [0.046508831612493751, -1.3222428563216697], [-8.9162882736247511, 0.090108390880383027], [0.26032585592694524, 1.3712302760622741]], [[-7.26241455057629, -1.2985806904120327], [4.6676782272096276, -0.66685650858343326], [-3.3574316210012105, 3.7765376351241491], [0.10338365751473066, -2.6699919104150549], [0.17995756936857443, -2.7843240616719092]], [[2.8239678159210531, -3.1817323650758875], [-3.5846899025272343, -3.6872886037068691], [0.78194161257342909, -4.0169665406708468], [-0.41093389559121185, -1.0244878005150353], [5.4639899154957554, 2.9922813687655783]], [[7.0200373441301078, 8.8092255387886578], [-0.34746503753521729, 4.0748471665565633], [-0.12636076322096823, -3.9424302275282725], [-4.0706486232522998, -3.6382849000905475], [-3.2414666966327896, 0.46008097038254636]]])+(1.-msk_ref)*numpy.array([[[-1.4883568364640496, -4.1034738866104066], [-3.5739248753282826, -3.8536393692237096], [-2.7900579763255511, -4.0707691918883437], [-5.9154230269379688, -1.5168615816248101], [-0.59325553789979057, 4.3331391803124966]], [[-7.3682521313300979, -6.628360357576458], [3.2784545593229968, 3.4921914129854006], [2.6444201823425368, -3.2440234238840864], [3.8906382054106148, 5.0304533217675633], [5.317548570441379, 2.0743637902857879]], [[-0.83075577997972694, -1.0582354059694907], [-3.9781380223879088, -5.37458988880035], [-4.8570286566262642, -3.4476933666736], [3.6801785696316998, 1.6193814314496087], [7.3217961020165223, -1.5301116639302519]], [[8.1046236068719324, 8.5864293086867214], [2.4489728078904873, 0.55775805422803515], [3.4687498927163327, 3.0774300972886577], [-0.81803077855147777, -4.8313510359632428], [1.1972044677811526, -2.6977543715976782]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.1151063062009703, -4.3097592422528272, 1.8830011035201579, 0.40830346783946858, -2.1920583322461109], [-1.7162481277592034, 3.4944604037354221, -0.68211495051072735, 3.6158039356005194, 4.8411156271535507], [-4.8575516288012386, 0.58750990144814885, -0.61871797904166748, -3.768553977897231, -3.1437428057767347], [-0.65530129163238993, -0.77699640931141545, -3.6980174948126923, 3.3777081105078839, -2.0331812761255383]])+(1.-msk_arg0)*numpy.array([[-0.78918660512546346, -0.62402367167888162, -1.9304549929258497, 2.4905653334232589, -3.0925809120239647], [-0.43190907959443692, -4.9179962908396657, -1.6610933010414208, 3.4299046042656514, 0.72926299225383673], [4.3317855149390212, 1.5662466619574591, -3.5623229617421002, 3.3730459614620916, 4.4926370787805023], [-1.4527844723492889, -4.425841688180121, -2.8186857955536535, 4.7423643760502436, 2.7749116976021435]])
      arg1=numpy.array([[[4.5990074441639521, -3.9754193263852233], [-3.4007681563182635, -0.9780778983320868], [-3.2389116714518549, -1.5551337668384346], [0.058578123784942093, 0.47449129761709674], [-3.7224341631245093, 0.98450943255759871]], [[1.4832620369206424, 4.8981303748601821], [0.85849259211621387, 0.25420542894161713], [4.2425244942816995, -4.3230919447967429], [-3.1631408778959855, 3.7351880095659382], [-0.90218763833327298, -3.3062109694409259]], [[1.7756060384788164, -4.2251888886379039], [0.46417029324156811, 0.57292553299700089], [-0.35426511586069509, 3.0870585732406255], [4.7036967442269724, 1.8795136516195052], [0.76167264658811007, -2.8808960097515177]], [[-0.67065153178715864, -2.4017665560681101], [0.44823390409443498, 2.846540979574856], [-1.1063599364875056, 4.1513616876847994], [3.224079885009262, -2.4192508520349518], [-1.0681678280533715, -0.24170424696606574]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.7141137503649224, -2.860313020184253], [-7.7105273985710907, -5.2878371405849141], [-1.355910567931697, 0.3278673366817233], [0.46688159162441067, 0.88279476545656532], [-5.9144924953706202, -1.2075488996885122]], [[-0.23298609083856103, 3.1818822471009787], [4.352952995851636, 3.7486658326770392], [3.5604095437709722, -5.0052068953074702], [0.45266305770453386, 7.3509919451664576], [3.9389279888202777, 1.5349046577126249]], [[-3.0819455903224222, -9.0827405174391416], [1.051680194689717, 1.1604354344451497], [-0.97298309490236257, 2.468340594198958], [0.93514276632974136, -1.8890403262777258], [-2.3820701591886246, -6.0246388155282524]], [[-1.3259528234195486, -3.0570678477005], [-0.32876250521698047, 2.0695445702634405], [-4.8043774313001979, 0.45334419287210714], [6.6017879955171459, 0.95845725847293206], [-3.1013491041789099, -2.2748855230916041]]])+(1.-msk_ref)*numpy.array([[[3.8098208390384887, -4.7646059315106868], [-4.0247918279971451, -1.6021015700109684], [-5.169366664377705, -3.4855887597642843], [2.549143457208201, 2.9650566310403557], [-6.8150150751484739, -2.108071479466366]], [[1.0513529573262055, 4.4662212952657452], [-4.0595036987234518, -4.6637908618980486], [2.5814311932402787, -5.9841852458381641], [0.26676372636966583, 7.1650926138315896], [-0.17292464607943625, -2.5769479771870891]], [[6.1073915534178376, 0.10659662630111733], [2.0304169551990272, 2.13917219495446], [-3.9165880776027953, -0.47526438850147468], [8.0767427056890639, 5.2525596130815968], [5.2543097253686124, 1.6117410690289846]], [[-2.1234360041364475, -3.8545510284173989], [-3.977607784085686, -1.5793007086052651], [-3.9250457320411591, 1.3326758921311459], [7.9664442610595056, 2.3231135240152918], [1.706743869548772, 2.5332074506360778]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.8816622142892645, -1.5774507816013141, -2.7532439533523281, -0.44623976044998592, -1.3236189998945971], [0.14022977931903391, -2.3397515127339128, 0.93753663350527727, -3.2583359524350373, 1.4624193824916274], [3.4576312861084144, -0.0078947281292229832, 3.5033583198482336, -2.5101562882864226, -2.7643173962306946], [-0.94668027339561345, 3.7410046393743208, -4.773099315848909, -1.085767124608382, 2.0914157628067969]], [[1.7467823368111297, 0.34081532163842354, 3.137947486330134, 2.6887156111466668, -2.7414406112502263], [-2.3329514515482019, 4.9048296390676036, 2.4344966435208484, -1.1278291548308417, -2.8514869721139622], [0.45359006564295168, 3.5543005755427917, 1.3223418490330374, 4.9813787376017267, -3.0664523420429237], [1.937409273258007, 2.8506060956338786, 4.7166982756586115, 1.3427409440442712, 3.7786631371194019]], [[2.2525036920395021, -0.56464315036081292, -0.029786852669136721, 1.4586525098596024, 4.0022429166209665], [3.7584494036011495, -1.5895399534180754, -1.1081995109957443, -3.7734802029172734, -0.08224217937112055], [-1.757778943164876, -1.4848788524614411, 0.59258044223619333, 1.7880613155297409, 2.409648067829], [4.683370387322789, 4.147244139489823, -0.92319760453293576, -4.6487261254271051, 4.269943038644378]], [[-1.414594718607296, -3.8396524206569413, -0.37859768802138394, -0.72281891866610071, 1.8834110622077187], [-3.0080923264686632, -4.356884359878709, 4.1687885642358449, 4.6521124922585724, 1.9106352893070602], [-2.6454360265682153, 2.136100957606911, 2.0370027283901386, 1.7149566624932175, 2.1608812442926535], [1.8561070672389661, 0.89331933657686946, -1.0122209837709972, 1.7312281465671013, -4.3082252420285618]], [[4.4347096764159293, -0.90399178997803009, 4.3879339111217135, 0.56226211295708062, -3.5888079137825546], [2.5706854930067182, 0.94177068144848164, 4.1811418707521586, -3.7809266070238778, -4.0488317270129635], [2.4824390019183991, -3.2541510113478864, 0.23905279428881965, 0.79098970064519403, 0.032658280677212659], [1.2327263710520189, -0.41749275149604692, -1.3905431004962931, 0.8462225809834969, 0.58187872689176245]]], [[[-1.8629439245544455, -1.6540379288055074, -2.3363576984816206, -3.4586168846751706, 1.1538957210765108], [-2.1905548643803865, 4.7014118723359566, 0.0030227493640646586, 3.9653050087414936, -4.1548438870219373], [-4.9977398988877226, 1.1487002527338426, 3.0132366678051863, 0.52679526582243152, 4.6654133926282686], [-4.4802179960162727, -0.77014284578926073, -0.8434279021416522, 0.084625034439030244, 0.1251530892185384]], [[4.02173310397413, -3.0210857854660542, 0.6888791223756936, 1.9850458094239638, 3.8483400573084534], [4.2389348890801184, -0.42249184384489791, -4.1629814529346456, -1.9183066352976272, -1.1736513056579243], [0.79837848398310207, -2.4633297806700116, 0.35346228461988005, -3.0410788243011631, 3.586431055136412], [1.5948324205620432, -2.6705638588423364, 4.7745576439917023, 4.0000681971745031, 3.6336730598810831]], [[2.156991241305275, 4.9775819337852987, -0.96771670781674324, 1.3661755584443416, -1.00646638877837], [0.035346193545443327, 4.6128098983733068, 2.853435067310512, -4.9233162117594409, 2.010641490029677], [3.2723377389740484, -3.9077857658708437, -1.4562484013678767, 1.4128375079696571, 4.3087099757916434], [0.64707376354131974, 2.3941051957695834, 1.9521532908918768, -0.36595348950844553, 1.5515831289603677]], [[-1.9915760451893485, 1.0453880584008255, -3.0948138640126954, -0.59779630660977823, -4.4055463167510016], [-4.6218550518647303, -4.3159151090064496, 2.1108285689331305, 4.2448862322641805, -2.7014447166149749], [-0.6151906875971882, 2.0459366910161219, 0.24874591091681975, 3.9664701642632245, 3.809179711671451], [-0.994089485468308, -4.136168178962663, -4.7330382403296252, 2.261439197489608, 2.38154785370204]], [[-1.3845106061162795, 3.3234993012768275, -0.20275912112016758, -1.9388617620828574, -2.4017807680222503], [1.2229738979333229, 3.9428888110404881, -4.2171908765208057, -2.7153945160978408, 0.65437824588503624], [0.99276647806436813, -4.7189608002962258, -2.9037910459986396, 3.4523222800676177, -1.7278294865475741], [4.6848411064302447, 1.2429967974412541, 0.54356264949576438, -2.594481515154615, -4.5999065940025163]]], [[[1.1803822431130708, 4.0882724796081877, 2.266073015931438, -2.2258481748275507, -1.5840919020051469], [4.5514369648548527, -2.174037033757461, 2.9882836831729263, -3.0023652420189939, 2.9925597609712558], [1.5425304787991232, -3.7423543300920512, -4.0877347829790427, -0.96864559063070477, 2.0727540543743732], [-2.8850082473596936, 4.2098643687971347, -3.3890955341510032, -2.015604478540082, -1.8544812768167973]], [[-2.4159708267821056, -1.4474963815023223, -3.0839313574285185, 4.765731168093474, 0.82466530867413201], [-3.8604144494420667, -1.6863370475542796, 4.0468716586009617, -4.9973756570229853, 3.4298969979393483], [-1.0258851244205234, 2.5311350262037777, 2.4212800327855284, 2.4242650800756085, -1.3097484712084473], [-0.77147533057530726, -4.5200738342692262, 3.2034941019927814, -1.5114920567128118, 3.2150549616472919]], [[3.447634490382617, -1.4981868131064613, 3.6318863391694158, 0.6120462373445692, -3.18122613093613], [0.0092462290345904918, -2.5417933811182447, -2.9971796005069749, -2.3563555198752004, 3.1334114103253974], [-3.1903077092138874, -2.6805604012324356, -1.5103593451130228, -2.2053878845761887, -1.4794643892731161], [4.0378591569717841, 2.4182669954411455, -0.15644921198467632, 4.0131660725607858, -0.6643690304706773]], [[3.6757337282156275, -3.4527440335505002, 2.7028779005841761, -0.64617166929267533, -2.6052498718200612], [-2.2690726330766067, -0.16754456289016417, 4.0311924516560147, 4.2042206995167923, 2.1410599136341393], [4.6589187923089597, 1.9571801022525648, 2.1605871109629238, -3.2059845330138192, 3.935771996312905], [-4.9539688739120518, -4.6839655743742057, 3.8500762922315968, -2.8272358613055082, 0.93887768948311567]], [[-4.8081150159947406, 2.6414431890832279, 2.7996834766870897, -3.1813346010862364, -2.9419699749706183], [-0.8115849583525323, 0.60223018186675503, 0.56523606520123337, 2.6184896244892268, 3.9550529491319324], [-2.3564399565310579, -1.3627740407531661, -2.3015761925317468, -3.2717302563723272, -2.1094031686590444], [-1.6027959666838578, -3.3274843860071801, -1.3047903688562856, -2.9095494639912101, -1.5849065189239808]]], [[[1.0322010148108749, -0.42434327951147921, -1.6222834976924929, 4.8699114530591388, -3.2279635541509508], [0.34890256371545991, 1.8803012508314021, -0.3142010427280244, -0.86429403092783819, -0.54925054258982176], [-0.67280225292987961, -1.6800666436602385, -2.9353504440980958, -0.78858181959332452, -1.3796841574401575], [-0.64047382230322825, 4.5677558528440549, 3.6069305917600332, 2.7915987057646614, 3.3180165755346227]], [[-4.5016326374667024, 4.7934589649771127, 4.8332550426509364, -1.3563044323481268, 0.56361980478085627], [1.1385752224391679, 4.3647015229468096, 0.75029177289003712, 0.89613379526985959, -1.0371522275567124], [-2.581681844928001, -4.3349203801915728, 2.5128576916813667, 4.200756240084063, 2.6647432812676772], [2.391107120760811, 0.5309014849358471, -0.94688849898027083, -3.406206344753584, -2.495973156349971]], [[0.95095880716346759, 1.0442778353046753, -3.2684666437871801, 2.8303097709607332, 3.5230682966747828], [-3.4778514764890978, 1.2932345056727357, -1.9798647050176186, 0.61386312325764525, 2.6090902217160119], [4.8339929529308066, -1.6201255480669419, 4.8847806554856774, -1.6959092913941909, -4.4844295071648581], [-1.7520492289786684, 3.0968653156038481, 3.4882417362602531, -0.88888312218545806, -4.9922995060941489]], [[0.94940937206080633, -0.03070354066756309, 4.7424246373970043, -4.3086084147627925, 4.5485682440906459], [-3.5756165069724211, 3.594124824615287, 0.6612045722054507, 3.081911532577875, 2.0055694567236113], [4.4461405887614109, -2.4990088072017089, -1.7576694367182375, -4.0971375517487827, 4.4653869291669412], [-2.530762685452812, -4.6935226893164472, -3.6689381272808874, 2.449835967139653, -2.9965648237248899]], [[-2.2289562300384702, -0.069846700483765289, 3.2392799893016324, -3.5997804143238854, 4.1705863978659696], [2.5454678628622593, -1.6393568443427506, -2.9330510462577042, 0.072358639005349445, 1.5033518273931499], [-2.7159675558784615, -0.041352723464903818, 2.8757211807129757, 2.403728032038055, 1.7968618377626591], [-4.3555112547004269, -3.2647399246324014, -4.7544269042452054, 4.2250576023003497, -0.96675700175670798]]]])+(1.-msk_arg0)*numpy.array([[[[4.237437497806976, -4.1859907676688941, -4.0178376048751439, 4.4729535050155196, -3.1012682971577608], [-0.16869879669460541, 4.25987058852815, -2.3961218266957554, 3.6140544219568547, 0.76584321862247151], [-3.8364269224596459, -4.3167271536099037, -2.0343165500381355, -2.5878781275581018, 4.3952918782199841], [-4.4429436728320324, -3.4505169862895881, 0.057106556241022233, -3.8704770042218497, -4.4381074519058803]], [[2.4142779545207125, 4.8901639713085334, 0.77831078955300015, 0.49931802471285103, -1.0741584564989015], [4.9976302017477163, -3.5529173800956029, 1.879328552976455, 2.2136273232332782, -4.8128940748167759], [-1.4214468559278535, 0.4995542202515324, -4.424798136003556, -0.35618529853210479, -2.3846170013101164], [-2.9656078415297205, -3.9401976764304694, -2.0694523581362212, -1.3831335640772457, 2.0750102337505947]], [[2.591420009243067, -2.7038827780396959, 0.42183017803817169, 4.1234315769541965, -0.41990636272233672], [-2.7547758480210502, -0.082734870463595733, -0.98870317420076592, 4.988726707358488, 2.4586910869488126], [-4.6312890489792462, 1.095823560132767, -3.2386324517077334, 3.7284769740535797, -0.3935298088863215], [-0.85831974827221469, 4.2418137549955066, 3.576478788179454, 1.7425109521105497, -4.7056202771581281]], [[4.259346097709928, 3.4472866610094322, 1.0101943620729799, -1.6573419237660101, -2.0893422061449818], [-1.2577490215086606, -1.1700742627721739, -4.5774013066123977, 1.4943339763302763, -2.2142913604426662], [1.521821099251043, 0.34826898142280882, 2.1335507760441308, 4.377501904518418, -4.9563543182863565], [-1.2052233580206826, -1.4264973194727979, -0.3944533305586706, -0.37937690109225741, -2.7645233087833843]], [[0.00070259939542971495, -1.2373554066917025, -3.0167889949407769, -1.5390722713840352, 3.4731702467362631], [3.401534775512836, 3.5051583087759859, -4.3443887984344407, 2.6042097749309541, 2.6306455681396743], [2.0805870015552195, -4.6963295885170329, 2.3001827157182317, -4.9463754606849362, 1.2145996143361835], [-4.8517775155735432, -4.0984086983030537, 3.7687918101604723, -3.6390557979598723, 2.1044314778967479]]], [[[-1.1888719950290128, 2.2809477569872634, 3.3035878260403972, -2.4969473404544864, 4.9758746519318571], [-1.4624047040682866, 2.9146598666481349, 4.9996368088893455, -1.17705534627241, 3.3749084716166564], [4.0564102119709808, 4.4230528118498587, -0.96610535992824431, 0.82052100420755991, -2.596359181687502], [1.6041134563509303, -4.1365352616078237, 1.8884260264157673, 3.3320615786706735, 0.79409904636525308]], [[-4.8393533158551509, 3.6607513055777332, -0.43895411667373541, 2.5878623951380106, -0.40513965957942233], [-3.5681184974276037, 2.3237084544306699, -1.3733705801255147, -3.4868538917787451, -4.2611279780627145], [-4.6814671253813351, 0.037894646836170942, -1.030304643415322, 0.31297418542762667, -0.99640318441939524], [-0.14448586134181429, 3.9981629956381841, 0.023373691460308876, -1.1501645666298907, 0.91036095898991665]], [[-0.044889936011514031, 4.7873135285291859, 4.5086208716403355, 3.908386563373007, 2.7102327062121336], [4.5510258007138535, -2.6403901106621728, 1.0029842364203425, -4.468068105506747, -3.5632518232639243], [-0.61376185861519161, 0.6539548130347308, 3.9995768678373587, 0.6087314360965399, 4.1321345477291622], [3.9120311344913041, -4.6816798888152142, 2.8298235890569927, -1.1182797325446248, -0.2688866702213355]], [[3.8933502365943244, 2.3209307586478891, 2.515111535963622, -4.8208449208278301, -2.514509051217122], [3.8896231684687095, -2.55391676204614, 1.8928080374454819, 4.9808222855336961, 3.2472950951472885], [4.3210691112004866, 2.3383502346737508, 4.9437293624312133, 0.42398569978057488, 3.222405012255221], [-2.3892456635429014, -0.78336559896478786, -4.7738510243952037, 2.0964800466607851, -4.7465525892197116]], [[-4.0297829923701203, -4.8061856350220928, -0.074247556169245499, 4.3850891596773032, 0.67252416389291625], [-0.60665775662844457, 2.6532914747689933, 1.3394276753840737, -4.1735727411374723, -3.6780169015380104], [0.87272846180981922, 0.97949988891101647, -2.1233355363333031, -2.7394487542930399, 2.4095995027310906], [-2.7566170025444006, -2.1852221182209961, -4.3386465105935654, 1.1856618302864739, 2.5765365181166366]]], [[[2.8102108043281548, -4.1730286795756353, 2.8386445687873483, -4.567764055328122, 4.156290443813953], [4.57654582222799, -0.83545018129687509, -3.6455386390751867, 1.5002412266915579, -4.8506028768667573], [4.1823447995284173, 1.8779660262557822, 1.8075624765577372, -3.062337964955141, 3.2783772590722702], [2.858746084630357, -1.0410009357758119, 2.4525901040222022, -3.1171523703256234, -2.5349602412402672]], [[0.83486395571604621, 2.7668864003166158, -1.2206199172069656, 2.2941165569617592, -2.0966891526613529], [-2.3473495934660358, -2.0532415228515335, -0.7780459806215978, -3.4626443721239131, -0.18848832716187403], [1.3150993195452445, 2.0089689324355522, 3.7963649543470357, 1.7455793868900997, -3.1387520039445493], [0.38324386120778087, 2.4851283819084644, 4.5043291126431484, -4.9023014802097586, -0.96192084979751336]], [[-4.5989483676998688, 1.9510959910388879, 4.6785401056913205, 2.4470388830764991, 4.1963166955362201], [-4.5206438793810744, 2.3359631284350622, -2.4950946092404722, 4.2262299789303732, 3.5069254133669165], [2.6179820873691808, -4.6861922538840162, 1.3521114916542114, -1.672441469732695, 2.388052775099716], [1.3109206720488586, 0.64757246346950836, -3.9196704646235982, 3.6456211672410337, 4.0146236394713792]], [[0.53230625092856343, 2.1553419729773449, -0.26112870221970308, -3.9272668816173795, -1.8347475975789465], [4.43305603320686, 1.9213238500663454, 0.98955598325640892, -3.6458163567983681, -1.4448762502572765], [2.9242713579038551, -3.1101463291805675, -0.10585038621232368, -2.7343197280701959, -0.70681858122583296], [-0.15795009185985265, 1.7822454008238893, -4.7659184413753888, 1.614336911858282, 2.0736562265775262]], [[4.3225605804321301, -1.6838487870738161, 3.1986741133110925, 3.0112023465459146, 1.2272409229617365], [3.2845773806218617, 2.7873904946453942, -4.4785425662417708, -4.3162017399479353, 3.2276674972317458], [-3.2731199947991585, 1.8294793459113281, -1.1903350207540431, 3.2166831653511778, -2.62114357797397], [2.6754266242487548, -3.724907527076764, -1.1082852682825397, 2.3582706509037994, -3.6825556741850942]]], [[[-2.4592873860215461, -4.5076844711084689, 4.2902962796519652, -3.4072828541285558, 2.092968125508281], [-4.6335094075530794, -1.9353422903265205, 0.5102176402154166, 1.2674262257910964, 3.21493399993747], [-4.8013321694819613, 3.5124645949441273, 2.1188990261741738, -0.35226506573492244, -0.062643790802777488], [-2.1828173430286935, 3.367198966223647, -0.52124558138024923, 3.5627318716550853, -4.7742867329841987]], [[3.6095689755604035, -3.0257318615278885, -4.6506937392684131, 3.6530022300961242, -1.1593180662520943], [-0.20357834104232975, -2.3950944433520363, -0.23311209474107919, 3.4119867022493295, 0.19705695024116032], [-2.8070200109421983, -1.2429661936563963, 3.4824974558809423, 0.2593440596083898, -2.0968715461337171], [2.8474456046527248, 1.7516408869109767, -2.2403759917948896, -3.692461346443443, -3.8616366031946403]], [[-1.7503246400028463, -4.5488837890949965, 0.25413226119651533, 3.4554096832525829, 0.62349765527017453], [-3.7616918339702377, 2.0312785275651013, -4.445486877304222, 0.42034705148145513, 2.6356750878669954], [3.7638914181227179, 4.8333109194524404, -1.1730304936274196, 4.4744453553202241, -4.0756255824113143], [-1.9527299484753522, -3.900227699461575, 0.19915029351041724, -4.6540350071445049, -0.103136162404871]], [[-3.2049916882893159, 0.91520556475658577, -1.7685763055346415, 2.7607409769824471, 1.0963757209420208], [2.6402361760833379, 3.2266609292005235, 2.2955598101800483, 4.7137206786671193, 4.5092892595714549], [-3.335424584412686, 4.3920366037372069, 3.6547376818472515, -2.1086023623610304, 1.6315263964089493], [-4.6450652140701969, -0.23840866766586721, 1.7190091847676587, 3.1174358405283638, 3.3830509757506633]], [[3.1289002467828304, -3.1705358805081163, -3.1617174807832682, 4.2687730715114416, 1.4407809426292015], [-1.4377187449668662, -2.1190221951997223, 0.87087367160310869, -3.3097696885289918, -1.9775930141533182], [1.5816535071252469, -4.375200847125523, 4.228046593144315, -1.4289226205102432, -0.17325873272413528], [-4.6656465600676498, -0.78071648685463213, 1.105712180579701, -2.8105168554292046, -2.0783865085841335]]]])
      arg1=numpy.array([[3.5378272104966282, 2.3236427588120527, 2.9864527825654221, 1.8677567980618894, 1.0395984752270406], [4.0688657278761866, 0.19955705701197424, -1.5757564541998925, 2.5726907911386654, -2.9058585989961849], [2.6485895766997807, 1.0572390460048622, -0.64238632267062368, 4.4500808446749378, -1.9449550565511387], [0.26803259839577365, 3.6377771225804985, 3.4774954023246742, 2.7327720275643568, 3.7167350387331801]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.65616499620736368, 1.960376428895314, 0.78458325714430011, 3.0915874500466423, 2.2142082106020311], [3.6780569898156621, 1.1980756977627154, 4.4753638440019055, 0.27949125806159092, 5.0002465929882556], [6.9954584966050426, 3.5299324823674052, 7.0411855303448618, 1.0276709222102056, 0.77350981426593357], [2.5911469371010147, 7.278831849870949, -1.2352721053522808, 2.4520600858882462, 5.629242973303425]], [[4.0704250956231824, 2.6644580804504763, 5.4615902451421867, 5.0123583699587195, -0.41779785243817358], [-0.0093086927361492044, 7.2284723978796563, 4.7581394023329011, 1.1958136039812111, -0.52784421330190945], [2.7772328244550044, 5.8779433343548444, 3.6459846078450902, 7.3050214964137794, -0.74280958323087098], [4.2610520320700598, 5.1742488544459313, 7.0403410344706643, 3.666383702856324, 6.1023058959314547]], [[5.2389564746049242, 2.4218096322046092, 2.9566659298962854, 4.4451052924250245, 6.9886956991863887], [6.7449021861665717, 1.3969128291473467, 1.8782532715696778, -0.78702742035185125, 2.9042106031943016], [1.2286738394005461, 1.501573930103981, 3.5790332248016155, 4.774514098095163, 5.3961008503944221], [7.6698231698882111, 7.1336969220552451, 2.0632551780324864, -1.6622733428616829, 7.2563958212098001]], [[0.45316207945459341, -1.9718956225950519, 1.4891591100405055, 1.1449378793957887, 3.7511678602696081], [-1.1403355284067738, -2.4891275618168196, 6.0365453622977343, 6.5198692903204618, 3.7783920873689496], [-0.77767922850632587, 4.0038577556688004, 3.9047595264520281, 3.582713460555107, 4.0286380423545429], [3.7238638653008556, 2.7610761346387589, 0.8555358142908922, 3.5989849446289908, -2.4404684439666724]], [[5.4743081516429699, 0.13560668524901054, 5.4275323863487541, 1.6018605881841212, -2.549209438555514], [3.6102839682337589, 1.9813691566755223, 5.2207403459791992, -2.7413281317968372, -3.0092332517859228], [3.5220374771454397, -2.2145525361208458, 1.2786512695158603, 1.8305881758722347, 1.0722567559042533], [2.2723248462790595, 0.62210572373099371, -0.35094462526925252, 1.8858210562105375, 1.6214772021188031]]], [[[2.2059218033217411, 2.4148277990706792, 1.7325080293945661, 0.61024884320101602, 5.2227614489526974], [1.8783108634958001, 8.7702776002121432, 4.0718884772402513, 8.0341707366176802, -0.085978159145750688], [-0.92887417101153602, 5.2175659806100292, 7.0821023956813729, 4.5956609936986181, 8.7342791205044552], [-0.41135226814008607, 3.2987228820869259, 3.2254378257345344, 4.1534907623152169, 4.194018817094725]], [[4.2212901609861042, -2.82152872845408, 0.88843617938766783, 2.184602866435938, 4.0478971143204276], [4.4384919460920926, -0.22293478683292367, -3.9634243959226714, -1.718749578285653, -0.97409424864595007], [0.99793554099507631, -2.2637727236580374, 0.55301934163185429, -2.8415217672891888, 3.7859881121483863], [1.7943894775740175, -2.4710068018303621, 4.9741147010036766, 4.1996252541864774, 3.8332301168930574]], [[0.58123478710538246, 3.4018254795854062, -2.5434731620166358, -0.20958089575555094, -2.5822228429782625], [-1.5404102606544492, 3.0370534441734143, 1.2776786131106195, -6.4990726659593339, 0.43488503582978444], [1.6965812847741559, -5.4835422200707367, -3.0320048555677692, -0.1629189462302354, 2.7329535215917509], [-0.92868269065857278, 0.81834874156969084, 0.37639683669198432, -1.941709943708338, -0.024173325239524868]], [[0.58111474594931689, 3.6180788495394909, -0.52212307287403004, 1.9748944845288872, -1.8328555256123362], [-2.0491642607260649, -1.7432243178677842, 4.6835193600717959, 6.8175770234028459, -0.12875392547630948], [1.9575001035414772, 4.6186274821547872, 2.8214367020554851, 6.5391609554018899, 6.3818705028101164], [1.5786013056703574, -1.5634773878239976, -2.1603474491909598, 4.8341299886282734, 4.9542386448407054]], [[-4.290369205112464, 0.41764070228064254, -3.1086177201163525, -4.8447203610790428, -5.3076393670184352], [-1.682884701062862, 1.0370302120443031, -7.1230494755169911, -5.6212531150940261, -2.2514803531111487], [-1.9130921209318168, -7.6248193992924111, -5.8096496449948241, 0.54646368107143273, -4.633688085543759], [1.7789825074340597, -1.6628618015549308, -2.3622959495004205, -5.5003401141508004, -7.5057651929987017]]], [[[3.8289718198128515, 6.7368620563079684, 4.9146625926312186, 0.42274140187222997, 1.0644976746946337], [7.2000265415546334, 0.47455254294231963, 5.6368732598727069, -0.35377566531921323, 5.6411493376710364], [4.1911200554989039, -1.0937647533922705, -1.439145206279262, 1.6799439860690759, 4.7213436310741539], [-0.23641867065991296, 6.8584539454969153, -0.7405059574512225, 0.63298509815969872, 0.79410829988298337]], [[-1.3587317807772434, -0.39025733549746011, -2.0266923114236564, 5.8229702140983362, 1.8819043546789942], [-2.8031754034372045, -0.62909800154941742, 5.1041107046058238, -3.9401366110181231, 4.4871360439442105], [0.031353921584338806, 3.5883740722086399, 3.4785190787903906, 3.4815041260804707, -0.2525094252035851], [0.28576371542955492, -3.462834788264364, 4.2607331479976436, -0.45425301070794966, 4.2722940076521541]], [[2.8052481677119934, -2.140573135777085, 2.9895000164987922, -0.030340085326054478, -3.8236124536067537], [-0.63314009363603319, -3.1841797037888684, -3.6395659231775985, -2.9987418425458241, 2.4910250876547737], [-3.8326940318845111, -3.3229467239030592, -2.1527456677836465, -2.8477742072468124, -2.1218507119437398], [3.3954728343011604, 1.7758806727705219, -0.79883553465529999, 3.3707797498901622, -1.306755353141301]], [[8.1258145728905653, 0.99733681112443762, 7.1529587452591139, 3.8039091753822625, 1.8448309728548766], [2.1810082115983311, 4.2825362817847736, 8.4812732963309525, 8.6543015441917301, 6.5911407583090771], [9.1089996369838975, 6.4072609469275026, 6.6106679556378616, 1.2440963116611186, 8.3858528409878428], [-0.50388802923711395, -0.23388472969926788, 8.3001571369065346, 1.6228449833694296, 5.3889585341580535]], [[-6.7530700725458797, 0.6964881325320893, 0.85472842013595107, -5.1262896576373755, -4.8869250315217574], [-2.756540014903671, -1.3427248746843836, -1.3797189913499053, 0.67353456793808819, 2.0100978925807937], [-4.3013950130821961, -3.3077290973043048, -4.246531249082885, -5.2166853129234658, -4.0543582252101835], [-3.5477510232349965, -5.2724394425583192, -3.2497454254074243, -4.8545045205423492, -3.5298615754751195]]], [[[1.3002336132066485, -0.15631068111570556, -1.3542508992967193, 5.1379440514549124, -2.9599309557551772], [0.61693516211123356, 2.1483338492271757, -0.046168444332250758, -0.59626143253206454, -0.28121794419404811], [-0.40476965453410596, -1.4120340452644649, -2.6673178457023221, -0.52054922119755087, -1.1116515590443838], [-0.3724412239074546, 4.8357884512398286, 3.8749631901558068, 3.0596313041604351, 3.5860491739303963]], [[-0.86385551488620393, 8.4312360875576111, 8.4710321652314349, 2.2814726902323716, 4.2013969273613547], [4.7763523450196663, 8.002478645527308, 4.3880688954705356, 4.533910917850358, 2.600624895023786], [1.0560952776524974, -0.69714325761107432, 6.1506348142618652, 7.8385333626645615, 6.3025204038481757], [6.0288842433413095, 4.1686786075163456, 2.6908886236002276, 0.23157077782691449, 1.1418039662305275]], [[4.4284542094881418, 4.5217732376293496, 0.20902875853749414, 6.3078051732854075, 7.0005636989994571], [-0.00035607416442351436, 4.7707299079974099, 1.4976306973070557, 4.0913585255823195, 6.0865856240406861], [8.3114883552554808, 1.8573698542577324, 8.3622760578103517, 1.7815861109304834, -1.0069341048401839], [1.7254461733460058, 6.5743607179285224, 6.9657371385849274, 2.5886122801392162, -1.5148041037694746]], [[3.6821813996251631, 2.7020684868967937, 7.475196664961361, -1.5758363871984358, 7.2813402716550026], [-0.84284447940806428, 6.3268968521796438, 3.3939765997698075, 5.8146835601422318, 4.7383414842879681], [7.1789126163257677, 0.23376322036264785, 0.97510259084611928, -1.3643655241844259, 7.198158956731298], [0.20200934211154475, -1.9607506617520905, -0.93616609971653064, 5.1826079947040098, -0.26379279616053308]], [[1.4877788086947099, 3.6468883382494148, 6.9560150280348125, 0.11695462440929472, 7.8873214365991497], [6.2622029015954395, 2.0773781943904295, 0.78368399247547593, 3.7890936777385296, 5.22008686612633], [1.0007674828547186, 3.6753823152682763, 6.5924562194461558, 6.1204630707712351, 5.5135968764958392], [-0.63877621596724676, 0.45199511410077875, -1.0376918655120253, 7.9417926410335298, 2.7499780369764721]]]])+(1.-msk_ref)*numpy.array([[[[7.7752647083036042, -0.64816355717226593, -0.48001039437851567, 8.0107807155121478, 0.43655891333886743], [3.3691284138020228, 7.7976977990247782, 1.1417053838008728, 7.1518816324534829, 4.3036704291190997], [-0.29859971196301771, -0.77889994311327548, 1.5035106604584927, 0.94994908293852642, 7.9331190887166123], [-0.90511646233540421, 0.087310224207040044, 3.5949337667376504, -0.33264979372522152, -0.90028024140925211]], [[4.7379207133327652, 7.2138067301205862, 3.1019535483650529, 2.8229607835249038, 1.2494843023131512], [7.3212729605597691, -1.2292746212835501, 4.2029713117885077, 4.5372700820453309, -2.4892513160047232], [0.90219590288419926, 2.8231969790635851, -2.1011553771915032, 1.9674574602799479, -0.060974242498063713], [-0.64196508271766772, -1.6165549176184166, 0.25419040067583154, 0.94050919473480699, 4.3986529925626474]], [[5.5778727918084892, 0.28257000452572623, 3.4082829606035938, 7.1098843595196186, 2.5665464198430854], [0.23167693454437188, 2.9037179121018264, 1.9977496083646562, 7.9751794899239101, 5.4451438695142347], [-1.6448362664138241, 4.0822763426981892, -0.25217966914231127, 6.7149297566190018, 2.5929229736791006], [2.1281330342932074, 7.2282665375609287, 6.5629315707448761, 4.7289637346759719, -1.719167494592706]], [[6.1271028957718174, 5.3150434590713216, 2.8779511601348693, 0.21041487429587935, -0.22158540808309235], [0.6100077765532288, 0.69768253528971558, -2.7096445085505083, 3.3620907743921657, -0.34653456238077673], [3.3895778973129325, 2.2160257794846983, 4.0013075741060202, 6.2452587025803075, -3.0885975202244671], [0.66253344004120684, 0.44125947858909154, 1.4733034675032188, 1.488379896969632, -0.89676651072149483]], [[1.0403010746224703, -0.19775693146466189, -1.9771905197137363, -0.49947379615699461, 4.5127687219633037], [4.4411332507398766, 4.5447567840030265, -3.3047903232074001, 3.6438082501579947, 3.6702440433667149], [3.1201854767822601, -3.6567311132899922, 3.3397811909452724, -3.9067769854578955, 2.2541980895632241], [-3.8121790403465026, -3.058810223076013, 4.8083902853875129, -2.5994573227328317, 3.1440299531237885]]], [[[2.8799937328471739, 6.34981348486345, 7.3724535539165839, 1.5719183874217002, 9.0447403798080437], [2.6064610238079, 6.9835255945243215, 9.0685025367655321, 2.8918103816037766, 7.443774199492843], [8.1252759398471675, 8.4919185397260453, 3.1027603679479423, 4.8893867320837465, 1.4725065461886846], [5.6729791842271169, -0.067669533731637088, 5.9572917542919539, 7.4009273065468602, 4.8629647742414397]], [[-4.6397962588431767, 3.8603083625897074, -0.23939705966176117, 2.7874194521499849, -0.20558260256744809], [-3.3685614404156294, 2.5232655114426441, -1.1738135231135405, -3.2872968347667708, -4.0615709210507402], [-4.4819100683693609, 0.23745170384814518, -0.83074758640334778, 0.51253124243960091, -0.796846127407421], [0.055071195670159945, 4.1977200526501584, 0.22293074847228311, -0.95060750961791651, 1.1099180160018909]], [[-1.6206463902114066, 3.2115570743292934, 2.932864417440443, 2.3326301091731145, 1.1344762520122411], [2.975269346513961, -4.2161465648620648, -0.57277221777955001, -6.0438245597066391, -5.1390082774638168], [-2.1895183128150841, -0.92180164116516172, 2.4238204136374661, -0.96702501810335262, 2.5563780935292697], [2.3362746802914116, -6.2574363430151063, 1.2540671348571002, -2.6940361867445173, -1.844643124421228]], [[6.4660410277329898, 4.8936215497865545, 5.0878023271022874, -2.2481541296891647, 0.058181739921543407], [6.4623139596073749, 0.018774029092525346, 4.4654988285841473, 7.5535130766723615, 5.8199858862859539], [6.893759902339152, 4.9110410258124162, 7.5164201535698787, 2.9966764909192403, 5.7950958033938864], [0.18344512759576403, 1.7893251921738775, -2.2011602332565383, 4.6691708377994505, -2.1738617980810462]], [[-6.9356415913663056, -7.7120442340182773, -2.9801061551654304, 1.4792305606811182, -2.2333344351032687], [-3.5125163556246295, -0.25256712422719163, -1.5664309236121112, -7.0794313401336577, -6.5838755005341953], [-2.0331301371863657, -1.9263587100851685, -5.0291941353294884, -5.6453073532892244, -0.49625909626509435], [-5.662475601540585, -5.0910807172171815, -7.2445051095897508, -1.7201967687097111, -0.32932208087954828]]], [[[5.4588003810279355, -1.5244391028758546, 5.487234145487129, -1.9191744786283413, 6.8048800205137336], [7.2251353989277707, 1.8131393954029056, -0.99694906237540604, 4.1488308033913386, -2.2020133001669766], [6.830934376228198, 4.5265556029555629, 4.4561520532575178, -0.41374838825536031, 5.9269668357720509], [5.5073356613301376, 1.6075886409239688, 5.1011796807219829, -0.46856279362584274, 0.11362933545951348]], [[1.8921030017209084, 3.824125446321478, -0.16338087120210343, 3.3513556029666214, -1.0394501066564907], [-1.2901105474611736, -0.99600247684667131, 0.27919306538326438, -2.4054053261190509, 0.86875071884298816], [2.3723383655501067, 3.0662079784404144, 4.8536040003518979, 2.8028184328949619, -2.0815129579396872], [1.440482907212643, 3.5423674279133266, 5.5615681586480106, -3.8450624342048965, 0.095318196207348826]], [[-5.2413346903704925, 1.3087096683682642, 4.0361537830206968, 1.8046525604058754, 3.5539303728655964], [-5.163030202051698, 1.6935768057644385, -3.1374809319110959, 3.5838436562597495, 2.8645390906962929], [1.9755957646985571, -5.3285785765546398, 0.70972516898358773, -2.3148277924033187, 1.7456664524290924], [0.66853434937823497, 0.0051861407988846864, -4.5620567872942219, 3.00323484457041, 3.3722373168007556]], [[4.9823870956035012, 6.6054228176522827, 4.1889521424552347, 0.52281396305755834, 2.6153332470959914], [8.8831368778817978, 6.3714046947412832, 5.4396368279313467, 0.80426448787656968, 3.0052045944176613], [7.3743522025787929, 1.3399345154943703, 4.3442304584626141, 1.715761116604742, 3.7432622634491048], [4.2921307528150852, 6.2323262454988271, -0.31583759670045097, 6.0644177565332198, 6.523737071252464]], [[2.3776055238809914, -3.6288038436249548, 1.2537190567599539, 1.0662472899947759, -0.71771413358940217], [1.339622324070723, 0.84243543809425558, -6.4234976227929099, -6.2611567964990744, 1.2827124406806072], [-5.2180750513502971, -0.11547571063981055, -3.1352900773051817, 1.2717281088000392, -4.5660986345251082], [0.73047156769761612, -5.6698625836279026, -3.0532403248336784, 0.41331559435266074, -5.6275107307362333]]], [[[-2.1912547876257724, -4.2396518727126953, 4.5583288780477389, -3.1392502557327822, 2.3610007239040547], [-4.3654768091573057, -1.6673096919307469, 0.77825023861119025, 1.5354588241868701, 3.4829665983332436], [-4.5332995710861876, 3.780497193339901, 2.3869316245699475, -0.084232467339148798, 0.20538880759299616], [-1.9147847446329198, 3.6352315646194207, -0.25321298298447559, 3.830764470050859, -4.5062541345884251]], [[7.2473460981409019, 0.61204526105260992, -1.0129166166879147, 7.2907793526766227, 2.4784590563284041], [3.4341987815381687, 1.2426826792284622, 3.4046650278394193, 7.049763824829828, 3.8348340728216588], [0.83075711163830013, 2.3948109289241022, 7.1202745784614407, 3.8971211821888883, 1.5409055764467814], [6.4852227272332232, 5.3894180094914752, 1.3974011307856089, -0.054684223862944581, -0.22385948061414185]], [[1.727170762321828, -1.0713883867703222, 3.7316276635211896, 6.9329050855772572, 4.1009930575948488], [-0.28419643164556341, 5.5087739298897755, -0.96799147497954774, 3.8978424538061294, 6.1131704901916697], [7.2413868204473921, 8.3108063217771146, 2.3044649086972546, 7.9519407576448984, -0.5981301800866401], [1.524765453849322, -0.42273229713690075, 3.6766456958350915, -1.1765396048198307, 3.3743592399198032]], [[-0.47221966072495913, 3.6479775923209425, 0.96419572202971526, 5.4935130045468039, 3.8291477485063776], [5.3730082036476947, 5.9594329567648803, 5.0283318377444051, 7.4464927062314761, 7.2420612871358117], [-0.60265255684832919, 7.1248086313015637, 6.3875097094116082, 0.6241696652033264, 4.364298423973306], [-1.9122931865058401, 2.4943633598984896, 4.4517812123320155, 5.8502078680927205, 6.11582300331502]], [[6.8456352855160105, 0.54619915822506382, 0.55501755794991192, 7.9855081102446217, 5.1575159813623817], [2.2790162937663139, 1.5977128435334578, 4.5876087103362888, 0.40696535020418834, 1.7391420245798619], [5.298388545858427, -0.65846580839234292, 7.9447816318774951, 2.2878124182229369, 3.5434763060090448], [-0.94891152133446965, 2.936018551878548, 4.8224472193128811, 0.90621818330397552, 1.6383485301490466]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.021906758041000174, -4.7842355138858883, -1.0598501137653082, -0.51832835199102423, -2.4566158375672051], [4.5436718862807215, 4.7435647113540931, -2.1775779501624837, -3.2408272140762238, -0.052644602754990721], [-1.3343566892343537, 4.5138177350751256, -1.1840906001001796, -2.6393466558238678, 2.8006238274642037], [-0.78232129686259722, -1.8749256143748827, -1.8891339603468627, 3.769544667914893, -1.8643500739267038]])+(1.-msk_arg0)*numpy.array([[-4.1044402804333604, 3.7994415502858345, -1.3111654376477802, -1.7091243536607479, 4.4074463933304493], [1.4086297858689454, 2.7383663193772536, -4.8015414966103034, 1.2919717363088337, 3.655504443269713], [-0.053603600609632984, -3.5882637682208154, -1.5757509032630779, -1.2297412228702056, 0.9939713394703622], [4.934819077672719, -1.072712243305689, -3.2082576829358613, -3.731406101375363, 3.9071803972647032]])
      arg1=numpy.array([[[[1.3182987895435172, -3.6786200735410457, -4.4954657306198564, -0.36907521221182016, -1.7754461312403267], [4.6055753340863763, 2.7710910645528264, 2.4069995950744785, -3.2949093769713853, -1.1195457788218546], [-0.91932244424972254, -0.86794674023892782, 3.5805427995975538, 4.3008842284035769, 2.2464592022780376], [3.8536425446577027, 2.4615667370112959, -1.6641960056042948, -4.3803802826341673, -0.53980501835651573]], [[1.9481229243560989, -0.61881189684526028, -0.81470761573843298, -3.9385942803840024, -4.1594593747962092], [2.7843265149017293, 2.8185510725019736, 3.7846562797331647, -3.3995337089653486, 2.5044244423167052], [2.4610180721836672, -3.7827345890499142, 0.47298745981525148, -3.9575465447932867, -4.9414087155191], [-0.75005572666453713, 1.1460708893608444, 2.7809588581487592, 4.5479606022457233, -1.0315314828922686]], [[4.3149998135752572, -3.9360352895253694, -0.47697276466140615, 4.6662118488244708, -3.9549404800202259], [3.8141893371428566, -1.1507145176987335, -3.8789959578999254, 2.3341155820453015, 3.5721139099372206], [-4.1939754825880904, 2.5716665979912943, -0.9193348462941735, 2.5967192422314387, 4.3677357863986863], [-3.126468754895313, 2.2910435756975858, -4.0590455730691613, -0.16137420804675617, -4.5694024882925905]], [[0.62129199675573332, -2.3337255813692259, -3.9395136577939569, -4.1600359480132223, 3.0637844517324115], [-4.9002756929708458, -1.3334253493104908, -0.33903271382672795, -2.115271337285185, 0.42996094603948176], [3.8673812016519999, -4.4145637376484084, 4.7199455788944249, 2.9071124023907426, -1.410936148254831], [2.7458311717234976, 4.7179872249638208, 2.1853687408050817, 2.8610929151673563, -0.60831769353805143]], [[4.3174440416113118, -4.1173240620177634, -2.6372034538597546, -3.963982748032906, 3.0684787502681576], [1.4529343972434123, 1.5007956475394835, -3.1057394758412071, -0.45243979137933898, -2.1624685662133802], [3.5744522957188387, 2.5466647307293311, -3.484510319118642, 3.6390753922278201, 4.3169458488928942], [-1.9515997492760082, 4.2306386396176805, -2.4239175427574833, 4.8843212953202872, 1.9086662344845706]]], [[[0.77494071371906781, 0.31098105019976341, 1.38061634394262, -2.3074849003112119, 0.0020822790498487009], [1.6014134371189179, 4.2352141276730197, -2.4175778000586301, -2.5266521115704932, -3.4881202358430641], [-2.3272944349765403, -4.4588843571175278, 3.5639039433798949, 4.9618029142007156, -4.5139634222406499], [-4.4441167859596646, -1.0909023672542117, -2.7434470612915609, 0.97037567207051012, 1.9375453668618769]], [[-3.1644940085360194, -3.6043834768947391, -4.8501831697110038, 3.9401337624433665, -4.5314177415761527], [2.0989772017360604, -2.2104222868033951, 1.0315904783018777, -2.3848855069355546, -2.2630739812571954], [4.174095221225862, 2.5946926532700765, -1.7963643778565421, 3.0990165133374425, 2.612025730159373], [-0.54797254322859246, 2.3358446169770195, -1.9122116959579571, -0.7302319376341071, 4.5148090627463429]], [[-4.7152175892907007, 2.535133420622639, -3.1976068903463006, 0.3723028038274343, 1.4415580366619452], [-3.9441393881682796, 1.0582220132323705, 2.1260543022604326, -0.67014215966946811, -3.817783781843731], [-4.1778473179562683, -2.414674459704611, 1.5020674523153774, -3.4792370050765351, -4.7336787825880116], [3.250756530609312, 3.8848898093837896, 3.691924685770001, 0.078995341996996871, -4.5353532276366266]], [[1.307488092338545, 3.3095749572886781, 1.7606839709311242, 0.32063778413448674, -3.1676493823509979], [4.9921759005445061, 0.73841777291066091, 3.6733140877894357, 2.7768986686501416, 0.39718985882673508], [-0.20208482952340567, -4.7095497624426796, -0.78407043130174525, -2.3783453946296405, 4.5805497679235057], [-1.3803579933612742, 3.7894147613975253, 2.6938130854935363, 0.47680576529696683, -2.9190101656275456]], [[4.489377877901088, -3.1001554584928162, -3.9725878913307588, 2.4947437053723078, -2.8857136438802335], [2.152948597996712, -4.9072494784484668, 4.5294238654136016, -3.4473995548375758, -3.3545567683589148], [4.0548915105517231, 2.314177069165547, -2.3001382586484831, -3.3674828468353759, -2.5669188337707483], [-1.0120596011395575, 2.1254341114905184, -0.86105368655272851, 1.0368973964735311, -4.1585535616214431]]], [[[-4.7287905219345436, 3.7817650737153841, 3.1342579537567339, 4.2309969095948254, 0.6601744809081822], [-2.3419934274745389, -2.5629687361516851, -4.6824769981596734, -4.3718800718021855, 2.3386223199490166], [4.295222332610539, 4.6457882593694215, -3.8417499319961914, 4.5730847030673143, 4.2008656016718717], [0.97004787453628705, 4.2851309220288414, -2.8689151760485512, 2.2138185414735725, -1.3473539862754667]], [[4.847188906401918, 3.9597219126366934, -2.3629830310399766, 1.1438562303495079, 2.5999412721229547], [4.1669027998651025, 1.8639481788650203, -2.524345647893167, 2.6539813613345125, -2.7503735622243219], [-2.4832005866961748, -4.5698341684181845, -0.59880038105355027, -3.365942248415188, -2.3568828708674481], [0.73323183981544204, 1.9876125848030481, -3.7336367343626753, -4.6911963857411889, 2.8056919098252671]], [[1.1918243850147112, 4.1785133836352273, 4.6568284741570096, -0.23726694002443516, -3.2748795712385856], [1.6416968375485261, -1.9836504372544783, -2.8004465008413226, -4.9551753307275259, 4.2337044773580974], [-4.0316108493191329, 2.4683185274747643, -1.5434830757405482, -3.7513641148964338, 1.2292652432780438], [-4.4999159958425334, -2.5568607429733445, 2.6557658765825991, -4.3942280762984502, 4.6115800614277056]], [[-4.0149554049525804, -1.687431494121614, -3.6847271147592009, 2.6943754183020028, -2.7742005094661359], [-4.1436417343020153, 3.329961583319184, 4.3519471135257906, -3.7701973407334677, -0.3354734170008653], [2.1213944481767708, -0.55092153235613139, -3.5137307671262432, -4.2635414739454234, 0.29637195849705567], [-1.3784665616117109, -2.9781224115543967, 2.9497631280535561, -4.7786361337863772, -0.54050699309813943]], [[1.3244559926661523, 4.6051592551120244, -1.2444251570528744, 0.25159877434458799, 4.5802041605591484], [-3.9493962505962319, -1.7930087789684968, 4.8807554958014414, -4.5643940713206019, 3.9119457421378527], [3.9365333804896814, -3.5294607607438189, -0.65139500011999996, 4.8136239789640545, 3.9683145772773525], [3.7966225820859254, 3.1884498030820421, -3.1689056806881224, -2.5122553193360853, -4.4899682850787572]]], [[[-0.78537831622943788, 4.8375254239955865, 3.2288092589078659, 0.036162075496121382, 3.9569105548260861], [2.8529119124123596, -3.0686451662584924, -3.1288523345600372, -3.9126544278053776, -3.5803004512971603], [-0.3484087495239816, 0.73530035113953396, -4.7997017022429214, -3.6300179201677176, 4.2736633159998085], [-4.9658945947987156, -2.3639214777508055, 2.1497450988170304, -2.8640236589216816, 1.6382744148130426]], [[-1.6464574496158848, 0.37857244700952197, -1.7465920445057281, 3.3058098288767219, 3.2183011548533198], [-3.0136774487097631, 0.78074747904131581, 2.5882760764226562, -4.2700256966461261, -2.3082161794972267], [1.0796317380188203, -1.3701821865398403, -1.0632340513677696, 1.0386427077181812, 1.518781232339057], [-3.2842075806789306, 2.5473869143607466, 0.77850082079655358, -4.0652188314187088, -1.2420219299076054]], [[-0.086364490071240141, -3.8568475496395358, 0.89433620761051014, -4.7462473308949127, -1.4353013658101288], [-0.49907263112096523, -3.2631965632924462, -4.2366211970471408, -3.7910428845311173, -4.530724568439811], [1.21010966527618, -4.483158325508743, -3.3257178903579607, 2.0959520822629818, 3.9539196441662678], [-3.3489600026954536, -1.0235570907548652, -4.8196359590227722, 1.191714399628597, 4.2801013013608333]], [[-2.7349799838978086, -4.4563948124125172, -2.6008788122137485, 2.1666467195327774, -4.6521686928586936], [-3.9842573719897856, 3.4312803710192412, -3.4155342889518527, -3.8217771595229113, 3.1237649213333931], [0.11751104699913562, 2.8007923823111538, -3.9527680229834172, -0.41250989455606213, 3.6429219396214592], [-3.6637275914616518, -1.0589345843532403, -1.6439815483778366, -2.1011027188112186, 2.1013295495296962]], [[-4.7139524644297293, 4.6691081522191755, 4.5083628763757844, -3.2209508266037536, -2.4180814025897734], [-3.1008665775466184, -1.3642504305593075, 0.046847159854016773, -0.28235113721795457, 4.5868578468216601], [-3.5464055848169904, 0.30400706650541665, -4.2721196849989358, -4.1272676572258487, 3.1414110188956439], [4.8435222253186456, -3.2836741200855988, -0.5957503701526532, -0.20571342817601224, 0.11368699218917566]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.296392031502517, -3.7005268315820459, -4.5173724886608566, -0.39098197025282033, -1.7973528892813269], [4.5836685760453761, 2.7491843065118262, 2.3850928370334783, -3.3168161350123855, -1.1414525368628547], [-0.94122920229072271, -0.88985349827992799, 3.5586360415565537, 4.2789774703625767, 2.2245524442370375], [3.8317357866167026, 2.4396599789702957, -1.686102763645295, -4.4022870406751675, -0.56171177639751591]], [[-2.8361125895297894, -5.4030474107311486, -5.5989431296243213, -8.7228297942698916, -8.9436948886820975], [-1.999908998984159, -1.9656844413839147, -0.99957923415272365, -8.183769222851236, -2.2798110715691831], [-2.3232174417022211, -8.5669701029358016, -4.3112480540706368, -8.7417820586791741, -9.7256442294049883], [-5.5342912405504254, -3.6381646245250439, -2.0032766557371291, -0.23627491164016501, -5.8157669967781569]], [[3.2551496998099489, -4.9958854032906777, -1.5368228784267144, 3.6063617350591626, -5.0147905937855342], [2.7543392233775483, -2.2105646314640417, -4.9388460716652336, 1.2742654682799932, 2.5122637961719123], [-5.2538255963533986, 1.511816484225986, -1.9791849600594817, 1.5368691284661304, 3.3078856726333781], [-4.1863188686606208, 1.2311934619322775, -5.1188956868344695, -1.2212243218120644, -5.6292526020578988]], [[0.10296364476470909, -2.8520539333602501, -4.4578420097849811, -4.6783643000042465, 2.5454560997413873], [-5.41860404496187, -1.8517537013015151, -0.85736106581775218, -2.6335996892762092, -0.088367405951542466], [3.3490528496609757, -4.9328920896394326, 4.2016172269034007, 2.3887840503997184, -1.9292645002458553], [2.2275028197324733, 4.1996588729727966, 1.6670403888140575, 2.3427645631763321, -1.1266460455290757]], [[1.8608282040441066, -6.5739398995849685, -5.0938192914269598, -6.4205985856001107, 0.6118629127009525], [-1.0036814403237928, -0.95582019002772167, -5.5623553134084123, -2.9090556289465441, -4.6190844037805849], [1.1178364581516336, 0.090048893162125943, -5.9411261566858471, 1.1824595546606149, 1.8603300113256891], [-4.4082155868432134, 1.7740228020504754, -4.8805333803246889, 2.4277054577530821, -0.54794960308263452]]], [[[5.3186125999997893, 4.8546529364804849, 5.9242882302233415, 2.2361869859695096, 4.5457541653305702], [6.1450853233996394, 8.7788860139537412, 2.1260940862220914, 2.0170197747102283, 1.0555516504376574], [2.2163774513041812, 0.084787529163193653, 8.1075758296606164, 9.5054748004814371, 0.029708464040071547], [0.099555100321056855, 3.4527695190265097, 1.8002248249891606, 5.5140475583512316, 6.4812172531425984]], [[1.5790707028180737, 1.1391812344593539, -0.10661845835691075, 8.6836984737974596, 0.21214696977794034], [6.8425419130901535, 2.5331424245506979, 5.7751551896559707, 2.3586792044185385, 2.4804907300968977], [8.917659932579955, 7.3382573646241696, 2.947200333497551, 7.8425812246915356, 7.3555904415134661], [4.1955921681255006, 7.0794093283311126, 2.8313530153961359, 4.013332773719986, 9.2583737741004359]], [[-6.8927955394531839, 0.35755547046015534, -5.3751848405087843, -1.8052751463350494, -0.73601991350053853], [-6.1217173383307628, -1.1193559369301131, -0.051523647902051106, -2.8477201098319518, -5.9953617320062147], [-6.3554252681187524, -4.5922524098670952, -0.67551049784710626, -5.6568149552390192, -6.9112567327504948], [1.0731785804468283, 1.7073118592213059, 1.5143467356075173, -2.0985826081654868, -6.7129311777991099]], [[-1.9333391217376787, 0.06874774321245436, -1.4801432431450996, -2.920189429941737, -6.4084765964272217], [1.7513486864682823, -2.5024094411655629, 0.43248687371321193, -0.4639285454260822, -2.8436373552494887], [-3.4429120435996294, -7.9503769765189034, -4.024897645377969, -5.6191726087058642, 1.3397225538472819], [-4.6211852074374979, 0.54858754732130155, -0.5470141285826875, -2.7640214487792569, -6.1598373797037693]], [[4.4367332751460973, -3.1528000612478069, -4.0252324940857491, 2.4420991026173171, -2.9383582466352243], [2.1003039952417213, -4.9598940812034575, 4.4767792626586109, -3.5000441575925665, -3.4072013711139055], [4.0022469077967324, 2.2615324664105563, -2.3527828614034738, -3.4201274495903666, -2.619563436525739], [-1.0647042038945482, 2.0727895087355277, -0.91369828930771924, 0.98425279371854035, -4.2111981643764338]]], [[[-6.0631472111688973, 2.4474083844810304, 1.7999012645223802, 2.8966402203604718, -0.67418220832617148], [-3.6763501167088926, -3.8973254253860388, -6.0168336873940271, -5.7062367610365392, 1.0042656307146629], [2.9608656433761853, 3.3114315701350678, -5.1761066212305451, 3.2387280138329606, 2.866508912437518], [-0.36430881469806664, 2.9507742327944877, -4.2032718652829049, 0.8794618522392188, -2.6817106755098203]], [[9.3610066414770436, 8.473539647711819, 2.150834704035149, 5.6576739654246335, 7.1137590071980803], [8.6807205349402281, 6.3777659139401459, 1.9894720871819587, 7.1677990964096381, 1.7634441728508037], [2.0306171483789508, -0.056016433343058836, 3.9150173540215754, 1.1478754866599377, 2.1569348642076775], [5.2470495748905677, 6.5014303198781738, 0.78018100071245033, -0.17737865066606329, 7.3195096449003927]], [[0.0077337849145315829, 2.9944227835350476, 3.47273787405683, -1.4213575401246148, -4.4589701713387653], [0.4576062374483465, -3.167741037354658, -3.9845371009415023, -6.1392659308277056, 3.0496138772579178], [-5.2157014494193126, 1.2842279273745847, -2.7275736758407279, -4.935454714996613, 0.045174643177864127], [-5.6840065959427131, -3.7409513430735242, 1.4716752764824195, -5.5783186763986299, 3.427489461327526]], [[-6.6543020607764483, -4.3267781499454818, -6.3240737705830687, 0.055028762478134929, -5.4135471652900033], [-6.7829883901258832, 0.69061492749531617, 1.7126004577019227, -6.4095439965573355, -2.9748200728247332], [-0.51795220764709704, -3.1902681881799992, -6.1530774229501111, -6.9028881297692912, -2.3429746973268122], [-4.0178132174355792, -5.6174690673782646, 0.31041647222968827, -7.417982789610245, -3.1798536489220073]], [[4.125079820130356, 7.4057830825762281, 1.5561986704113293, 3.0522226018087917, 7.380827988023352], [-1.1487724231320282, 1.0076150484957069, 7.6813793232656451, -1.7637702438563982, 6.7125695696020564], [6.7371572079538851, -0.72883693327961518, 2.1492288273442037, 7.6142478064282582, 6.7689384047415562], [6.5972464095501291, 5.9890736305462458, -0.36828185322391871, 0.28836850812811843, -1.6893444576145535]]], [[[-1.5676996130920351, 4.0552041271329893, 2.4464879620452686, -0.74615922136647583, 3.1745892579634889], [2.0705906155497624, -3.8509664631210896, -3.9111736314226344, -4.6949757246679749, -4.3626217481597571], [-1.1307300463865788, -0.047020945723063257, -5.5820229991055186, -4.4123392170303148, 3.4913420191372113], [-5.7482158916613129, -3.1462427746134027, 1.3674238019544331, -3.6463449557842789, 0.85595311795044537]], [[-3.5213830639907675, -1.4963531673653607, -3.6215176588806108, 1.4308842145018392, 1.3433755404784371], [-4.8886030630846458, -1.0941781353335669, 0.71335046204777353, -6.1449513110210088, -4.1831417938721094], [-0.79529387635606241, -3.245107800914723, -2.9381596657426523, -0.83628290665670146, -0.35614438203582566], [-5.1591331950538137, 0.6724612999858639, -1.0964247935783291, -5.9401444457935915, -3.1169475442824881]], [[-1.9754984504181028, -5.7459815099863984, -0.99479775273635251, -6.6353812912417753, -3.3244353261569914], [-2.3882065914678279, -5.1523305236393089, -6.1257551573940034, -5.6801768448779804, -6.4198585287866736], [-0.67902429507068263, -6.3722922858556057, -5.2148518507048234, 0.20681812191611915, 2.0647856838194052], [-5.2380939630423162, -2.9126910511017279, -6.7087699193696348, -0.69741956071826561, 2.3909673410139707]], [[1.0345646840170843, -0.68685014449762427, 1.1686658557011445, 5.9361913874476704, -0.88262402494380066], [-0.21471270407489262, 7.2008250389341342, 0.35401037896304022, -0.05223249160801835, 6.8933095892482861], [3.8870557149140286, 6.5703370502260467, -0.18322335506852427, 3.3570347733588308, 7.4124666075363521], [0.10581707645324112, 2.7106100835616527, 2.1255631195370563, 1.6684419491036744, 5.8708742174445891]], [[-6.5783025383564331, 2.8047580782924717, 2.6440128024490805, -5.0853009005304575, -4.2824314765164768], [-4.9652166514733223, -3.2286005044860113, -1.817502914072687, -2.1467012111446584, 2.7225077728949563], [-5.4107556587436942, -1.5603430074212872, -6.1364697589256396, -5.9916177311525525, 1.2770609449689401], [2.9791721513919418, -5.1480241940123026, -2.460100444079357, -2.0700635021027161, -1.7506630817375282]]]])+(1.-msk_ref)*numpy.array([[[[-2.7861414908898432, -7.7830603539744061, -8.5999060110532177, -4.4735154926451806, -5.8798864116736871], [0.50113505365301592, -1.333349215880534, -1.6974406853588819, -7.3993496574047457, -5.223986059255215], [-5.0237627246830829, -4.9723870206722882, -0.52389748083580656, 0.19644394797021647, -1.8579810781553228], [-0.25079773577565767, -1.6428735434220645, -5.7686362860376548, -8.4848205630675277, -4.6442452987898761]], [[5.7475644746419334, 3.1806296534405742, 2.9847339345474015, -0.13915273009816786, -0.36001782451037467], [6.5837680651875639, 6.6179926227878081, 7.5840978300189992, 0.39990784132048596, 6.3038659926025398], [6.2604596224695017, 0.016706961235920303, 4.272429010101086, -0.15810499450745219, -1.1419671652332655], [3.0493858236212974, 4.945512439646679, 6.5804004084345937, 8.3474021525315578, 2.7679100673935659]], [[3.003834375927477, -5.2472007271731496, -1.7881382023091863, 3.3550464111766907, -5.2661059176680061], [2.5030238994950764, -2.4618799553465136, -5.1901613955477055, 1.0229501443975213, 2.2609484722894404], [-5.5051409202358705, 1.2605011603435141, -2.2305002839419537, 1.2855538045836585, 3.0565703487509062], [-4.4376341925430935, 0.97987813804980561, -5.3702110107169414, -1.4725396456945363, -5.8805679259403707]], [[-1.0878323569050146, -4.0428499350299738, -5.6486380114547048, -5.8691603016739702, 1.3546600980716637], [-6.6094000466315936, -3.0425497029712387, -2.0481570674874758, -3.8243956909459329, -1.2791634076212661], [2.1582568479912521, -6.1236880913091563, 3.010821225233677, 1.1979880487299948, -3.1200605019155789], [1.0367068180627497, 3.008862871303073, 0.47624438714433381, 1.1519685615066084, -2.3174420471987993]], [[8.7248904349417611, 0.29012233131268594, 1.7702429394706947, 0.44346364529754334, 7.4759251435986069], [5.8603807905738616, 5.9082420408699328, 1.3017069174892422, 3.9550066019511103, 2.2449778271170691], [7.981898689049288, 6.9541111240597804, 0.92293607421180734, 8.0465217855582694, 8.7243922422233435], [2.4558466440544411, 8.6380850329481298, 1.983528850572966, 9.2917676886507365, 6.3161126278150199]]], [[[2.1835704995880132, 1.7196108360687088, 2.7892461298115654, -0.89885511444226651, 1.4107120649187941], [3.0100432229878633, 5.6438439135419651, -1.0089480141896847, -1.1180223257015478, -2.0794904499741187], [-0.91866464910759493, -3.0502545712485825, 4.9725337292488403, 6.370432700069661, -3.1053336363717046], [-3.0354870000907193, 0.31772741861473364, -1.3348172754226155, 2.3790054579394555, 3.3461751527308223]], [[-0.42612768915876575, -0.86601715751748554, -2.1118168503337502, 6.6785000818206202, -1.7930514221988991], [4.837343521113314, 0.52794403257385847, 3.7699567976791313, 0.35348081244169904, 0.47529233812005822], [6.9124615406031156, 5.3330589726473301, 0.94200194152071148, 5.8373828327146962, 5.3503920495366266], [2.1903937761486612, 5.0742109363542731, 0.82615462341929646, 2.0081343817431465, 7.2531753821235965]], [[-9.5167590859010041, -2.2664080759876644, -7.9991483869566036, -4.4292386927828691, -3.3599834599483582], [-8.745680884778583, -3.7433194833779329, -2.6754871943498708, -5.4716836562797715, -8.6193252784540348], [-8.9793888145665726, -7.2162159563149144, -3.299474044294926, -8.2807785016868394, -9.5352202791983149], [-1.5507849660009914, -0.9166516872265138, -1.1096168108403024, -4.7225461546133065, -9.33689472424693]], [[2.5994598286473787, 4.6015466935975118, 3.0526557072399578, 1.6126095204433204, -1.8756776460421642], [6.2841476368533398, 2.0303895092194946, 4.9652858240982694, 4.0688704049589752, 1.6891615951355687], [1.089886906785428, -3.417578026133846, 0.50790130500708841, -1.0863736583208068, 5.8725215042323393], [-0.08838625705244052, 5.081386497706359, 3.9857848218023699, 1.7687775016058005, -1.6270384293187119]], [[8.144882321170801, 0.55534898477689687, -0.31708344806104583, 6.1502481486420209, 0.76979079938947947], [5.808453041266425, -1.2517450351787538, 8.1849283086833147, 0.20810488843213726, 0.30094767491079821], [7.7103959538214362, 5.96968151243526, 1.3553661846212299, 0.28802159643433711, 1.0885856094989648], [2.6434448421301555, 5.7809385547602314, 2.7944507567169845, 4.6924018397432441, -0.50304911835173005]]], [[[-4.7823941225441766, 3.7281614731057511, 3.0806543531471009, 4.1773933089851925, 0.60657088029854922], [-2.3955970280841719, -2.6165723367613181, -4.7360805987693064, -4.4254836724118185, 2.2850187193393836], [4.241618732000906, 4.5921846587597885, -3.8953535326058244, 4.5194811024576813, 4.1472620010622387], [0.91644427392665406, 4.2315273214192084, -2.9225187766581842, 2.1602149408639395, -1.4009575868850996]], [[1.2589251381811026, 0.37145814441587799, -5.951246799260792, -2.4444075378713075, -0.98832249609786071], [0.57863903164428709, -1.7243155893557951, -6.1126094161139823, -0.93428240688630293, -6.3386373304451373], [-6.0714643549169907, -8.158097936638999, -4.1870641492743657, -6.9542060166360038, -5.9451466390882635], [-2.8550319284053733, -1.6006511834177672, -7.3219005025834907, -8.2794601539620043, -0.78257185839554833]], [[-0.38392651824836665, 2.6027624803721494, 3.0810775708939317, -1.813017843287513, -4.8506304745016635], [0.06594593428544826, -3.5594013405175562, -4.3761974041044009, -6.5309262339906038, 2.6579535740950195], [-5.6073617525822108, 0.89256762421168645, -3.1192339790036261, -5.3271150181595122, -0.34648565998503411], [-6.0756668991056113, -4.1326116462364224, 1.0800149733195212, -5.9699789795615281, 3.0358291581646277]], [[-5.244696627822786, -2.9171727169918196, -4.9144683376294065, 1.4646341954317972, -4.003941732336342], [-5.3733829571722209, 2.1002203604489784, 3.122205890655585, -4.9999385636036733, -1.5652146398710709], [0.89165322530656521, -1.780662755226337, -4.7434719899964488, -5.493282696815629, -0.93336926437314993], [-2.6082077844819165, -4.2078636344246023, 1.7200219051833505, -6.0083773566565828, -1.770248215968345]], [[2.3184273321365145, 5.5991305945823866, -0.25045381758251217, 1.2455701138149502, 5.5741755000295106], [-2.9554249111258697, -0.79903743949813455, 5.8747268352718036, -3.5704227318502397, 4.9059170816082149], [4.9305047199600436, -2.5354894212734567, 0.34257633935036225, 5.8075953184344167, 4.9622859167477147], [4.7905939215562876, 4.1824211425524043, -2.1749343412177602, -1.5182839798657231, -3.495996945608395]]], [[[4.1494407614432811, 9.7723445016683055, 8.1636283365805848, 4.9709811531688404, 8.8917296324988051], [7.7877309900850786, 1.8661739114142266, 1.8059667431126818, 1.0221646498673413, 1.3545186263755586], [4.5864103281487374, 5.6701194288122529, 0.13511737542979763, 1.3048011575050014, 9.2084823936725275], [-0.031075517125996654, 2.5708975999219135, 7.0845641764897493, 2.0707954187510373, 6.5730934924857616]], [[-2.7191696929215738, -0.69413979629616707, -2.8193042878114172, 2.2330975855710329, 2.1455889115476308], [-4.0863896920154517, -0.29196476426437323, 1.5155638331169672, -5.3427379399518156, -3.3809284228029157], [0.0069194947131312468, -2.4428944298455293, -2.1359462946734586, -0.034069535587507804, 0.446068989033368], [-4.3569198239846196, 1.4746746710550576, -0.29421142250913546, -5.1379310747243974, -2.3147341732132944]], [[-3.2946221730071015, -7.0651052325753971, -2.3139214753253512, -7.954505013830774, -4.6435590487459901], [-3.7073303140568266, -6.4714542462283076, -7.4448788799830021, -6.9993005674669782, -7.7389822513756723], [-1.9981480176596813, -7.6914160084446044, -6.5339755732938221, -1.1123056006728795, 0.74566196123040651], [-6.5572176856313149, -4.2318147736907266, -8.0278936419586344, -2.0165432833072643, 1.071843618424972]], [[-6.4663860852731716, -8.1878009137878802, -6.3322849135891115, -1.5647593818425856, -8.3835747942340575], [-7.715663473365149, -0.30012573035612178, -7.1469403903272157, -7.5531832608982743, -0.60764118004196987], [-3.6138950543762274, -0.9306137190642092, -7.6841741243587798, -4.1439159959314251, -0.088484161753903834], [-7.3951336928370148, -4.7903406857286033, -5.3753876497531996, -5.8325088201865816, -1.6300765518456668]], [[-0.80677206716502603, 8.5762885494838788, 8.4155432736404876, 0.68622957066094958, 1.4890989946749298], [0.8063138197180848, 2.5429299667053957, 3.95402755711872, 3.6248292600467487, 8.4940382440863633], [0.36077481244771281, 4.2111874637701199, -0.36493928773423256, -0.22008725996114542, 7.0485914161603471], [8.7507026225833489, 0.62350627717910445, 3.31143002711205, 3.701466969088691, 4.0208673894538789]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.68410168383386072, 1.0485569912367598], [1.6499969062007205, -4.3052368018092402]], [[-3.9062509139543944, 4.6720162993219638], [4.7670191956728321, -2.8490820759196511]], [[-0.16308225912029428, 4.4993197895974752], [1.7858224879548512, 4.4125934789062171]], [[1.2691718425725806, 4.1763658812129041], [-3.7787461305109895, -1.1451912130680295]], [[0.27547988674999413, -0.017768514507644717], [0.0327567031679461, 1.3809749150610067]], [[-4.0746203396383649, 3.3814057805010069], [-3.2627482266980481, -4.7162728181561793]]])+(1.-msk_arg0)*numpy.array([[[-3.9777503655297339, -4.6702693495723757], [4.4985940131279829, -4.0415191096220209]], [[-4.4005413692054631, -1.0925007098347415], [2.9345494895165283, -1.0758148357314457]], [[-2.0214991975969534, 4.9581999769205112], [1.5756738897177724, 2.5153379943729126]], [[0.60130179583693888, -1.4159147744113385], [-3.42561237801592, 1.9485644929814114]], [[-2.7419162444312652, 2.6137901126709853], [3.1470696765985622, -1.8281628489387169]], [[-2.0176475083374612, -2.9934747697808817], [3.3978398707817909, 0.71787766421197219]]])
      arg1=numpy.array([[[-4.4672937782020758, 4.29985198216289], [2.4969850277306964, 4.7157237978808286]], [[1.3410603781876649, -2.9020069675256446], [0.89408459503154347, -1.0442958048106821]], [[-0.36936904107290314, -4.3569006240201205], [-4.787366331182719, 0.47852704043991068]], [[-2.4771463980754249, 4.3617056435026171], [-2.2962605120595958, 0.89519354372021986]], [[-0.15672536639465218, -4.3574711126965324], [3.0258898968950998, 0.7728804918667711]], [[-0.22528963968610061, 3.8995533033525582], [-1.6521163808952775, -4.340532622145604]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.7831920943682151, 5.3484089733996498], [4.1469819339314169, 0.41048699607158845]], [[-2.5651905357667295, 1.7700093317963193], [5.6611037907043755, -3.8933778807303332]], [[-0.53245130019319742, 0.14241916557735479], [-3.0015438432278678, 4.8911205193461278]], [[-1.2079745555028443, 8.5380715247155212], [-6.0750066425705853, -0.24999766934780965]], [[0.11875452035534195, -4.3752396272041771], [3.0586466000630459, 2.1538554069277778]], [[-4.2999099793244655, 7.2809590838535652], [-4.9148646075933256, -9.0568054403017832]]])+(1.-msk_ref)*numpy.array([[[-8.4450441437318098, -0.37041736740948572], [6.9955790408586793, 0.67420468825880775]], [[-3.0594809910177982, -3.994507677360386], [3.8286340845480717, -2.1201106405421277]], [[-2.3908682386698565, 0.60129935290039072], [-3.2116924414649466, 2.9938650348128233]], [[-1.875844602238486, 2.9457908690912786], [-5.7218728900755158, 2.8437580367016313]], [[-2.8986416108259174, -1.7436810000255472], [6.172959573493662, -1.0552823570719458]], [[-2.2429371480235618, 0.90607853357167656], [1.7457234898865135, -3.6226549579336318]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.6979303652518025, -3.7557063677396449], [-3.1205473874399594, -2.4371213212013698]], [[4.7157359824676206, 0.088200881850044155], [-3.9808622863686827, 4.4834212280354464]]], [[[-4.951896831411581, -3.2630509243778114], [-3.7968266439313858, 1.1303079519532684]], [[3.4617075149242957, 0.066089388108636626], [-3.2579236487189256, 3.427414920866628]]], [[[4.4267189868698527, -1.3966772924034396], [1.1890955729961483, 0.48338107540395026]], [[-2.8514200260545284, -4.2564118412411709], [-4.9267164714700717, 4.4651387494990971]]], [[[-1.2060527352543957, -3.4757835322654707], [4.2757444465873924, 0.79578072680951184]], [[2.5918971956723516, 0.30846081777395185], [-2.5950235073292447, -4.8418020498481589]]], [[[1.4534045982902359, -4.7645371286252312], [-1.0788057780017613, 4.1929159677411896]], [[3.8391252431205025, -0.16054501695774448], [4.67216472613962, 3.6020931213889913]]], [[[-2.7663516045959824, 2.5947877907106047], [2.1470123604428784, -2.6421343978674692]], [[2.1368287280129383, -1.9064770304752843], [-4.1155397101401494, -1.0430516044814175]]]])+(1.-msk_arg0)*numpy.array([[[[0.23033953637303917, -1.8220475017012463], [1.0283300083620439, 2.5103738832715496]], [[4.5994290710505474, -1.0764339353660271], [2.8467353504144643, 4.1613894783214871]]], [[[2.5306222012383746, 0.41943265605926516], [0.16924424167134777, -4.2383788637845701]], [[0.60040795181834916, -0.46342427131860831], [-2.5886635111282508, -0.085160341813813822]]], [[[2.2854008893128945, 4.5335892977631094], [1.9373112746158263, 1.1777374794597781]], [[-1.1951078960587882, -0.60781655272141677], [-4.0362853271589474, 0.9639113625932012]]], [[[-1.4595667118529363, -2.7320788067891022], [2.8717732570960557, 2.6965529403621158]], [[-1.3055293954599323, -3.9128283791534866], [-0.11451633687090901, 0.19904912681176334]]], [[[-4.840775345380913, 4.004517002944155], [-1.1618998727373855, -3.9199051570068555]], [[3.1744259483996533, -0.44766400160209496], [-0.34381699374122832, -1.2355923767333197]]], [[[-1.6646749750740906, -4.1263239377183432], [-1.6222631727016767, 4.0636697716925223]], [[1.0959505511512679, 3.8154705919809562], [3.9754119789395013, -4.2827204144292184]]]])
      arg1=numpy.array([[[4.4066314889848748, -2.5268387425931103], [-2.0547207157203253, -3.4934433043457513]], [[-1.0091973529759688, -2.7255395443326469], [3.6364181795285972, 2.2754136041558413]], [[1.4769059632000792, -2.076171249509442], [3.8238790495739057, -4.4974950380629766]], [[2.8894903530503235, -1.9952599548435348], [-2.4169932138751404, 2.7229614805123852]], [[4.220230575282514, -3.5571686579019426], [2.5079042617815439, -0.95594897331555018]], [[3.1501604703517145, -2.6044264256950611], [0.70664181031419027, 1.156199763740676]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.70870112373307226, 0.65092512124522983], [-5.6473861300330697, -4.9639600637944801]], [[2.6610152667472953, -1.9665198338702812], [-7.474305590714434, 0.98997792368969506]]], [[[-5.9610941843875498, -4.2722482773537802], [-6.5223661882640327, -1.5952315923793785]], [[7.098125694452893, 3.7025075676372339], [-0.98251004456308433, 5.7028285250224693]]], [[[5.9036249500699318, 0.080228670796639534], [-0.88707567651329366, -1.5927901741054917]], [[0.9724590235193773, -0.43253279166726522], [-9.4242115095330483, -0.032356288563879509]]], [[[1.6834376177959278, -0.58629317921514712], [2.2804844917438576, -1.199479228034023]], [[0.17490398179721112, -2.1085323961011886], [0.12793797318314049, -2.1188405693357737]]], [[[5.6736351735727499, -0.54430655334271716], [-4.6359744359037034, 0.63574730983924699]], [[6.3470295049020464, 2.3473592448237994], [3.7162157528240698, 2.6461441480734411]]], [[[0.38380886575573214, 5.7449482610623193], [-0.45741406525218276, -5.2465608235625307]], [[2.8434705383271286, -1.199835220161094], [-2.9593399463994734, 0.11314815925925847]]]])+(1.-msk_ref)*numpy.array([[[[4.6369710253579139, 2.5845839872836285], [-1.4985087342310663, -0.01646485932156061]], [[2.5447083553302221, -3.1311546510863524], [-0.64670795393128699, 0.66794617397573575]]], [[[1.5214248482624058, -0.58976469691670363], [-2.5562953026612991, -6.963918408117217]], [[4.2368261313469464, 3.1729939082099889], [-0.31324990697240951, 2.1902532623420274]]], [[[3.7623068525129737, 6.0104952609631885], [-0.13885997489361568, -0.89843377004966385]], [[2.6287711535151175, 3.2160624968524889], [-8.533780365221924, -3.5335836754697754]]], [[[1.4299236411973872, 0.15741154626122134], [0.87651330225252089, 0.70129298551858099]], [[-3.7225226093350727, -6.329821593028627], [2.6084451436414762, 2.9220106073241485]]], [[[-0.62054477009839903, 8.224747578226669], [-4.7190685306393281, -7.4770738149087981]], [[5.6823302101811972, 2.0602402601794489], [-1.2997659670567785, -2.1915413500488699]]], [[[1.485485495277624, -0.97616346736662862], [-4.2266895983967379, 1.4592433459974612]], [[1.8025923614654582, 4.5221124022951464], [5.1316117426801773, -3.1265206506885423]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.03585542757504, -0.9934953349134803], [-1.4377584721971006, -2.5494078125192257]], [[3.5986805483309929, -4.6097988839994519], [2.1438952260381487, 3.8717298988943387]], [[0.18920061223689189, -1.0154091213552685], [-1.6278326232961629, 3.6521990833343825]], [[-0.68614032310435036, -1.1268633932529526], [-3.5131872887367588, 4.3799578699247537]], [[-0.5051458352581033, -2.3923113691539544], [-4.6086335609453988, 3.8144262179314907]], [[-1.5524194953518844, -3.7975278127875169], [1.6197127653193739, 4.2731091165435906]]])+(1.-msk_arg0)*numpy.array([[[-2.324495989493931, 1.0015471070665143], [-1.7643353555386856, -3.0568209097708046]], [[-1.2904233423286526, -0.92052240382423189], [0.15911288649576427, 3.9640063830500676]], [[-4.148092674675298, 2.3040494737275639], [2.4277604740776129, 0.1923497163729504]], [[1.3110358023321229, 3.6304512624761092], [4.7573815843430101, -3.8464382708411424]], [[3.9259928097063987, -4.8805188587911257], [1.6685885291710525, 3.2197694265055432]], [[-3.3766159531146469, 1.1579288860782802], [1.1913356486002025, 1.7084764872593761]]])
      arg1=numpy.array([[[[-2.8389451334388314, -0.018048515864370174], [1.3715743487400314, -3.4260056567065265]], [[-2.1064766867669449, -4.0755259177129037], [-2.2128017385656609, -3.2811452184028003]]], [[[2.7229610387026497, 3.302620126408101], [-3.7126204400833798, 1.5720735842320899]], [[1.0972542099685416, 3.7833472239412878], [-4.904058834475153, 2.2997382473628409]]], [[[-4.7594674091455227, 2.1692905942188503], [-2.2004656984572235, -4.690655622526684]], [[-2.2522828687287233, -4.9856517912864895], [4.9331894673685959, -0.72809086960461755]]], [[[1.9380500151659854, -1.3109392491311489], [3.7736211756649478, -2.8438708239478583]], [[1.1336144927496647, -4.5869182032021927], [0.72668623829889256, 3.4688098487305083]]], [[[3.1163079938727201, 1.4391804849263812], [2.459692732334851, 2.2057859423929624]], [[3.3163733245233971, 4.3545128608186996], [-1.5785812676712996, 2.0763326350967066]]], [[[-4.0067180966423415, -3.1440252247971987], [-0.88948182948185028, 0.57117516206993368]], [[-2.1318608141028115, 2.6114175627063947], [-3.722529262655665, -2.7857316838015276]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.19691029413620864, 3.0178069117106698], [0.37807901382655107, -4.4195009916200068]], [[-3.5442351589640455, -5.5132843899100044], [-4.7622095510848865, -5.8305530309220259]]], [[[6.3216415870336427, 6.9013006747390939], [-8.3224193240828317, -3.0377252997673621]], [[3.2411494360066904, 5.9272424499794365], [-1.0323289355808143, 6.1714681462571797]]], [[[-4.5702667969086308, 2.3584912064557422], [-3.215874819812492, -5.7060647438819529]], [[-3.8801154920248861, -6.6134844145826523], [8.5853885507029783, 2.9241082137297649]]], [[[1.251909692061635, -1.9970795722354993], [2.6467577824119952, -3.9707342172008109]], [[-2.379572795987094, -8.100105491938951], [5.1066441082236462, 7.848767718655262]]], [[[2.6111621586146168, 0.93403464966827787], [0.067381363180896603, -0.18652542676099193]], [[-1.2922602364220017, -0.25412070012669918], [2.2358449502601911, 5.8907588530281974]]], [[[-5.5591375919942259, -4.6964447201490831], [-4.6870096422693672, -3.2263526507175833]], [[-0.51214804878343756, 4.2311303280257686], [0.55057985388792563, 1.487377432742063]]]])+(1.-msk_ref)*numpy.array([[[[-5.1634411229327624, -2.3425445053583012], [2.3731214558065457, -2.4244585496400122]], [[-3.8708120423056305, -5.8398612732515893], [-5.2696226483364654, -6.3379661281736048]]], [[[1.4325376963739971, 2.0121967840794484], [-4.6331428439076117, 0.65155118040785798]], [[1.2563670964643059, 3.942460110437052], [-0.94005245142508542, 6.2637446304129085]]], [[[-8.9075600838208207, -1.9788020804564477], [0.10358377527034035, -2.3866061487991201]], [[0.17547760534888956, -2.5578913172088766], [5.1255391837415463, -0.53574115323166716]]], [[[3.2490858174981083, 9.655320097401443e-05], [7.404072438141057, 0.78658043852825088]], [[5.8909960770926748, 0.17046338114081738], [-3.1197520325422499, -0.37762842211063408]]], [[[7.0423008035791188, 5.3651732946327799], [-2.4208261264562747, -2.6747329163981632]], [[4.9849618536944496, 6.0231013899897521], [1.6411881588342436, 5.2961020616022498]]], [[[-7.3833340497569884, -6.5206411779118456], [0.26844705659642987, 1.7291040481482138]], [[-0.94052516550260901, 3.8027532113065972], [-2.0140527753962889, -1.0772551965421515]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.7014755042586156, -4.2925695927046856, -3.3719137841363009, -3.1805947809836765], [1.1398646170318569, 3.3199658015065889, -4.5121808274708481, -1.1200845958281267], [1.2309268821927883, 4.4319175249099345, 0.64641062788040671, 2.7844446354566479]], [[-4.1768148526992634, 0.64271445782263203, 1.6485090873490291, -0.8440310387642409], [3.8353647636794612, 4.0842439163403341, 4.5332941681589709, 3.9628937766802448], [-3.0142647702612102, -3.9851544684749243, 4.8531435903519, -1.8333855031262711]]], [[[-0.20328707008254732, -4.1692812761356315, 4.1525320063355817, 2.0372412722517907], [4.5667027494377876, -1.7293447901173566, 4.9392509253104393, 2.4033569512883011], [-3.8628309102588663, 1.7114036823832137, -0.95914172738023584, -2.0465222989691325]], [[-1.1082320883232333, -0.16983814536982589, -4.5735331189830744, 2.2470337390771373], [3.3540418017682896, 0.78885123107363508, -3.6261363732366814, -2.9723042474429748], [1.8560153091491403, -4.8423221491394983, 4.231403692238926, 1.9743487024046793]]], [[[4.4502302757449907, 0.83421827563926776, -3.0809710330993711, -0.19251380314102029], [-4.0418263793917486, -3.2018641839775763, 4.3023329879203551, -2.4652759362176391], [3.5424165103985192, -2.2587085144988572, -4.3474934724752856, 4.670294160124639]], [[-2.4785356320318086, -4.9003108248805916, 4.1636746384456078, -4.6565395484962293], [4.8495693836658571, -2.3824780247014354, 4.1315005115854557, 4.5628222376885823], [0.92383150305682982, -1.2122184869520289, -4.52309338456981, -0.98716808223671482]]]])+(1.-msk_arg0)*numpy.array([[[[-4.208947945528358, 2.6842957341771285, -0.38299832810727352, -0.93128720897877937], [-1.0051908921894226, 3.7947939338681014, 0.96372163917398623, -4.3118046096317277], [-1.9365836549965598, -1.9849631575403404, -4.5810483081758449, -4.7500534570287325]], [[-2.4338709872097364, -4.2601693235114908, 1.7134852054156351, 1.3168891619416367], [-2.5275110841526027, 1.7908749652465952, -3.9106724840849671, 0.98886110598324173], [3.5839605249970994, -4.8278864295279806, 3.1923570561536714, -1.7455422987611966]]], [[[4.9146100161044046, 0.26989045371543519, 3.3730831248180166, 0.1947972339764501], [-2.9873105957621893, 4.2884763587860242, -0.15425763649092339, -3.1153200467981588], [2.0669959371736732, -2.5038518153346234, 4.3262300163882799, -4.3881140521708195]], [[2.1697463084100743, -3.7145696251954572, 3.6498395546545872, -2.1382868034158955], [4.7850837902835419, 3.0202277922403837, 2.2159649670905708, 3.8805976277780729], [3.6642499492885197, -3.0264591226742263, -2.6597031489933176, -4.9387453793454963]]], [[[-4.5672642947185462, 3.4326143551066561, -1.8737713341570519, -1.2562784061629815], [4.1313092184693918, 4.6101101799645807, -0.67801895739447104, 0.12302511616629452], [-0.59752846492514067, 1.2901486451003032, -4.0459624415095146, 4.6938715399448796]], [[-3.1189875853956694, -3.1228949826481269, 1.1175854481094563, -0.28732879547789736], [-4.0468269385072011, 0.7873277636635585, 4.4115895428136493, -3.7221617545128414], [2.6333474607191647, -3.1885805656905664, 3.6962823784100358, -2.0202235141868172]]]])
      arg1=numpy.array([[[[0.44091043953628084, 3.866857355268996, 0.31755372811716942, 1.8528295382287494], [-3.7458847551780239, -4.1838643982171719, -2.7603449154801996, -0.38566510672265597], [2.9192168958546176, -1.4050933304461832, 2.2722349301222504, 2.4639034663576798]], [[-4.2225923450755696, 4.1528078115243154, 0.55811798512735322, -3.7805622890821891], [-0.29837503283025946, 3.11896457296797, -2.1693445899845623, 4.3681224708482027], [4.3615418345436101, 1.8187790287974899, -2.3517706270649952, -1.0691298712661288]]], [[[1.3208757525887149, -3.2514666681533644, 2.4235721258233633, -4.1606387442542356], [3.2318757657445509, -2.4412361165022265, 3.505708232333566, -1.9165062888465769], [4.3917893563233541, -4.4237409687275209, -0.51005403311623088, -1.1845327790250995]], [[-2.2485896710505338, 1.1868007191792351, -2.7203100996973797, -4.1433196056675108], [-2.2960310545551623, -2.8237698263243449, -3.20390076194934, -4.0419899323559525], [-1.4462839965749428, 2.2335487861238112, 4.9644322832367838, -3.6640028323240745]]], [[[-1.7149288303965147, -2.0450767329414687, 2.082722534681305, -3.7796192457907116], [-3.3644754056304937, 1.7794957999607819, -3.3401935998365606, 0.070025162827106691], [-2.3152086462624033, 4.7275001828990391, -0.4405415781583697, 2.9123060319151453]], [[3.4925214792355153, -4.048721810614718, 1.5546159373300945, -4.4010291453246886], [3.6178762789011838, 2.5570910690796183, 0.018537711790108879, 1.4654357264867679], [-4.1617567169740752, 1.4943538144870594, 3.7468247688442453, 1.941835514439191]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.2605650647223348, -0.42571223743568964, -3.0543600560191315, -1.3277652427549271], [-2.606020138146167, -0.863898596710583, -7.2725257429510481, -1.5057497025507827], [4.150143778047406, 3.0268241944637513, 2.9186455580026571, 5.2483481018143276]], [[-8.399407197774833, 4.7955222693469475, 2.2066270724763823, -4.62459332784643], [3.5369897308492018, 7.2032084893083042, 2.3639495781744086, 8.3310162475284475], [1.3472770642823999, -2.1663754396774344, 2.5013729632869048, -2.9025153743923999]]], [[[1.1175886825061676, -7.4207479442889959, 6.5761041321589451, -2.1233974720024449], [7.7985785151823386, -4.1705809066195831, 8.4449591576440053, 0.48685066244172415], [0.52895844606448783, -2.7123372863443072, -1.4691957604964667, -3.231055077994232]], [[-3.356821759373767, 1.0169625738094092, -7.293843218680454, -1.8962858665903735], [1.0580107472131273, -2.0349185952507098, -6.8300371351860214, -7.0142941797989273], [0.40973131257419748, -2.6087733630156871, 9.1958359754757097, -1.6896541299193952]]], [[[2.7353014453484761, -1.210858457302201, -0.99824849841806618, -3.9721330489317319], [-7.4063017850222419, -1.4223683840167944, 0.96213938808379451, -2.3952507733905324], [1.2272078641361159, 2.4687916684001818, -4.7880350506336553, 7.5826001920397843]], [[1.0139858472037067, -8.9490326354953105, 5.7182905757757023, -9.0575686938209188], [8.4674456625670409, 0.17461304437818281, 4.1500382233755646, 6.0282579641753502], [-3.2379252139172454, 0.28213532753503046, -0.77626861572556471, 0.95466743220247618]]]])+(1.-msk_ref)*numpy.array([[[[-3.7680375059920772, 6.5511530894461245, -0.065444599990104102, 0.92154232924997004], [-4.7510756473674469, -0.38907046434907055, -1.7966232763062133, -4.6974697163543837], [0.98263324085805781, -3.3900564879865236, -2.3088133780535944, -2.2861499906710527]], [[-6.6564633322853055, -0.10736151198717536, 2.2716031905429883, -2.4636731271405523], [-2.8258861169828622, 4.9098395382145652, -6.080017074069529, 5.3569835768314444], [7.9455023595407095, -3.0091074007304908, 0.8405864290886762, -2.8146721700273254]]], [[[6.2354857686931195, -2.9815762144379292, 5.79665525064138, -3.9658415102777855], [0.24456516998236166, 1.8472402422837977, 3.3514505958426426, -5.0318263356447357], [6.4587852934970273, -6.9275927840621438, 3.816175983272049, -5.572646831195919]], [[-0.078843362640459436, -2.5277689060162221, 0.92952945495720751, -6.2816064090834063], [2.4890527357283796, 0.19645796591603881, -0.98793579485876926, -0.16139230457787956], [2.2179659527135769, -0.7929103365504151, 2.3047291342434661, -8.6027482116695708]]], [[[-6.2821931251150609, 1.3875376221651874, 0.20895120052425309, -5.0358976519536931], [0.76683381283889807, 6.3896059799253626, -4.0182125572310312, 0.19305027899340121], [-2.912737111187544, 6.0176488279993423, -4.4865040196678843, 7.6061775718600249]], [[0.37353389383984581, -7.171616793262845, 2.6722013854395508, -4.6883579408025859], [-0.4289506596060173, 3.3444188327431768, 4.4301272546037582, -2.2567260280260735], [-1.5284092562549105, -1.694226751203507, 7.4431071472542811, -0.078387999747626225]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.04701604766)+(1.-msk_arg0)*(0.671136793049)
      arg1=Data(0.0364712022598,self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(3.08348724992)+(1.-msk_ref)*(0.707607995308)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.7696640613428958, -0.43078277159595579])+(1.-msk_arg0)*numpy.array([1.7702518517838914, -2.6105032258625807])
      arg1=Data(0.987806693748,self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.7818573675951512, 0.55702392215178875])+(1.-msk_ref)*numpy.array([2.758058545531636, -1.6226965321148361])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.43295112782)+(1.-msk_arg0)*(-3.70555698047)
      arg1=Data(numpy.array([4.9587421110957024, -3.7601681084164786]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([8.3916932389151562, -0.32721698059702486])+(1.-msk_ref)*numpy.array([1.2531851306262025, -7.4657250888859785])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-1.2287783208813319, 0.34485750605751697, -0.082127610682531937, 3.9334286912908762, 3.9418717195396749], [1.5930079703595332, 2.9115229305905075, 0.99565635297015476, -4.5822656338594019, 2.8296703967601564], [4.4744657412637707, 3.1505472333013103, -0.8335442368741699, -0.44447660585632143, 2.410055263774046], [-1.1082742406865984, 1.5913851394541965, 4.4606896436268482, 0.92569025818600892, 3.8181287428559791]])+(1.-msk_arg0)*numpy.array([[0.48545913332037749, 3.5370182453111791, -0.62864795177850841, -1.29766942802953, 3.392825843499871], [-1.5907043064229964, -1.8459231212223202, -2.8849736975201514, -4.0792788878834845, 2.1702054176324985], [0.69256025755490214, 1.5386959083745255, 1.4567174719620581, 1.7784486554704459, -2.8028347618618055], [0.47314101538310727, 4.4020355908586914, 4.7661371159023265, -0.007542780966657503, -0.1625391295829548]])
      arg1=Data(4.26794754239,self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.0391692215132253, 4.6128050484520742, 4.1858199317120253, 8.2013762336854334, 8.2098192619342321], [5.8609555127540904, 7.1794704729850647, 5.263603895364712, -0.31431809146484468, 7.0976179391547136], [8.7424132836583279, 7.4184947756958675, 3.4344033055203873, 3.8234709365382358, 6.6780028061686032], [3.1596733017079588, 5.8593326818487537, 8.7286371860214054, 5.1936378005805661, 8.0860762852505363]])+(1.-msk_ref)*numpy.array([[4.7534066757149347, 7.8049657877057363, 3.6392995906160488, 2.9702781143650272, 7.6607733858944282], [2.6772432359715608, 2.422024421172237, 1.3829738448744058, 0.18866865451107273, 6.4381529600270557], [4.9605077999494593, 5.8066434507690827, 5.7246650143566153, 6.0463961978650032, 1.4651127805327517], [4.7410885577776645, 8.6699831332532487, 9.0340846582968837, 4.2604047614278997, 4.1054084128116024]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.567197875858)+(1.-msk_arg0)*(3.47231973501)
      arg1=Data(numpy.array([[-3.2415229130292298, 4.9168455592447806, 1.2095193308913483, 3.9152992243992326, 2.2335468908014411], [-2.4078755756624748, -2.3425630974187159, -2.3864063199179721, 1.144548017937419, 2.8156550084654892], [-2.0089448901378528, 3.8543831103001267, 0.85396484749931645, -1.9787897573790092, -1.9466106334379929], [2.7367758440917687, -1.1300159627939141, -4.4121399017193585, 3.5451657802186567, -4.2802885360479355]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.8087207888873031, 4.3496476833867073, 0.64232145503327498, 3.3481013485411593, 1.6663490149433677], [-2.9750734515205481, -2.9097609732767893, -2.9536041957760455, 0.57735014207934565, 2.2484571326074159], [-2.5761427659959262, 3.2871852344420533, 0.2867669716412431, -2.5459876332370825, -2.5138085092960663], [2.1695779682336953, -1.6972138386519875, -4.9793377775774319, 2.9779679043605833, -4.8474864119060088]])+(1.-msk_ref)*numpy.array([[0.23079682198172069, 8.3891652942557311, 4.6818390659022988, 7.3876189594101831, 5.7058666258123916], [1.0644441593484757, 1.1297566375922345, 1.0859134150929783, 4.6168677529483695, 6.2879747434764397], [1.4633748448730977, 7.3267028453110772, 4.3262845825102669, 1.4935299776319413, 1.5257091015729576], [6.2090955791027191, 2.3423037722170363, -0.93982016670840807, 7.0174855152296072, -0.80796880103698498]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.4150663398908616, 3.2047291500389452], [0.58311134016637656, 2.0094290192607112]], [[4.2307648514867324, 2.34116103987302], [0.0042936707843042043, -3.4546680048926026]], [[3.5866442261034681, 3.129490218317498], [4.3875897710410516, -4.7106831221812673]], [[-0.82220830177776172, -1.1025870767555377], [-1.5132543273037191, 3.0336456209538927]], [[-3.9480649973648809, 0.81601670378246638], [4.4928852099284171, -3.2017414203734806]], [[1.5000929199179902, -1.356873566930533], [2.6827069471023783, 3.6711659828470395]]])+(1.-msk_arg0)*numpy.array([[[-0.24179991344356999, -4.9952957857460092], [1.0832523044648754, -2.2327954490189539]], [[0.29472313355938606, 4.5604840581694113], [-4.2745585311654919, -1.9329970449970455]], [[2.3098093836279876, -2.7863752888386029], [2.7201464214601456, -3.3587265987376913]], [[-2.2648114059662494, 3.420602191124237], [4.9868098042286082, 3.0595282030861792]], [[-2.227602242593484, -3.3715539979970721], [-0.22041882427500958, -2.9560678236823259]], [[2.7414301528119687, -4.5227406688797069], [-2.1200347889349516, -3.9342004403113018]]])
      arg1=Data(0.0111260881337,self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.4261924280245779, 3.2158552381726615], [0.59423742830009285, 2.0205551073944275]], [[4.2418909396204487, 2.3522871280067363], [0.015419758918020499, -3.4435419167588863]], [[3.5977703142371844, 3.1406163064512143], [4.3987158591747679, -4.699557034047551]], [[-0.81108221364404542, -1.0914609886218214], [-1.5021282391700028, 3.044771709087609]], [[-3.9369389092311646, 0.82714279191618267], [4.5040112980621334, -3.1906153322397643]], [[1.5112190080517065, -1.3457474787968167], [2.6938330352360946, 3.6822920709807558]]])+(1.-msk_ref)*numpy.array([[[-0.2306738253098537, -4.9841696976122929], [1.0943783925985917, -2.2216693608852376]], [[0.30584922169310236, 4.5716101463031276], [-4.2634324430317756, -1.9218709568633292]], [[2.3209354717617039, -2.7752492007048866], [2.7312725095938619, -3.347600510603975]], [[-2.2536853178325331, 3.4317282792579533], [4.9979358923623245, 3.0706542912198955]], [[-2.2164761544597678, -3.3604279098633558], [-0.20929273614129329, -2.9449417355486096]], [[2.752556240945685, -4.5116145807459906], [-2.1089087008012353, -3.9230743521775855]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.5973371372)+(1.-msk_arg0)*(3.52109170609)
      arg1=Data(numpy.array([[[-3.5444618562038133, -4.788708333480221], [-4.1816039389348694, 4.2085836506661227]], [[0.93274391227463749, -4.646972530206491], [4.1248260793603126, -0.45395607284020478]], [[-4.4915892380961715, 0.25728999547708042], [4.8168015071677992, 0.37819503823322886]], [[-0.28161886743928655, -2.5319440840983418], [3.6126716177516407, 3.7427990564812159]], [[1.5067457110244229, 1.2754569438441754], [-1.9983779892101836, 2.4086847281729584]], [[1.1824743055111053, -0.58726240588781486], [2.4367556367834675, 1.7372750172008491]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.94712471899985351, -2.1913711962762612], [-1.5842668017309096, 6.8059207878700825]], [[3.5300810494785972, -2.0496353930025313], [6.7221632165642724, 2.143381064363755]], [[-1.8942521008922117, 2.8546271326810402], [7.414138644371759, 2.9755321754371886]], [[2.3157182697646732, 0.065393053105617938], [6.2100087549556005, 6.3401361936851757]], [[4.1040828482283827, 3.8727940810481352], [0.59895914799377614, 5.0060218653769182]], [[3.7798114427150651, 2.0100747313161449], [5.0340927739874273, 4.3346121544048088]]])+(1.-msk_ref)*numpy.array([[[-0.023370150118631638, -1.2676166273950393], [-0.66051223284968774, 7.7296753567513043]], [[4.4538356183598191, -1.1258808241213094], [7.6459177854454943, 3.0671356332449768]], [[-0.97049753201098987, 3.7783817015622621], [8.3378932132529808, 3.8992867443184105]], [[3.2394728386458951, 0.98914762198683981], [7.1337633238368223, 7.2638907625663975]], [[5.0278374171096045, 4.796548649929357], [1.522713716874998, 5.92977643425814]], [[4.703566011596287, 2.9338293001973668], [5.9578473428686491, 5.2583667232860307]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.2316669437418977, 4.0584618081762294, 4.775947440262291, 0.18057407705190798], [-1.9654913154499676, -0.095696476842011435, -2.4523048245478773, -1.4213455156960118], [-4.6358074233159527, -1.4127334427874603, -2.6711237326545301, 0.10471693594978237]], [[-3.2569222307210612, -3.3894581272809177, -3.3261212902380777, 4.8645460807628389], [3.4744673736572196, 4.8825420851133217, 4.6409583786054096, -4.9204589794317268], [-2.5483583066335203, -1.226062093472029, 2.3244821694113789, -3.5096929046920557]]], [[[-4.6028769206654996, 4.00102290494765, -4.0686731010453023, 4.678834497827264], [-4.3590737128664543, 4.5641436191071314, 3.2543462785683523, 2.766206740594833], [-1.1084392778387575, -1.9444404723858733, -0.74806608292187082, -2.370776311209978]], [[-3.9341055942617431, -1.1108300652199601, -4.5167527220265526, -2.6357755942820038], [-3.6764702538277625, 3.9781586868900813, 4.209930204810858, 2.9353418482829916], [-4.7840080013321913, -0.51636399628978857, 1.4432322104459869, 0.4426661240636145]]], [[[-3.0395740830747098, -4.8279980959639648, -1.4651928676222017, 3.5388547125227081], [4.8441402497417183, 2.9304311110765004, -1.4635470984423105, -3.6523384144567617], [0.56970861052600696, 3.2938440653958878, -3.8418398027425029, -2.6976703840821301]], [[-4.9637393357651449, 4.5479362023709911, 0.72744607106140169, 1.0077929548961837], [-2.6872198692168139, -1.7069714680544514, -2.4274293747354339, 3.5308818542877614], [2.5991890705341847, -4.2463818857555982, -0.038043055716135754, 2.1998569101072647]]]])+(1.-msk_arg0)*numpy.array([[[[-0.044758767501552832, 0.81293492671455958, -4.3004488712053481, -4.4345856256747442], [1.30378502820263, 2.7844619144969451, -1.9473879036009114, 0.6441155002529273], [-2.2148765940368986, -3.916499558081231, -1.32512173297336, -1.5942482750263034]], [[3.5090405378321226, 4.7572425638161082, 2.2106301093985889, 0.37018645001983952], [2.5903541371506345, -0.23186745222462779, -4.4111007876195449, 0.50895223701994752], [4.4711902315732424, -0.88832958489150471, -2.9823198207128199, 3.3084019691102142]]], [[[-4.3662471818532547, 4.3647703184787279, -4.216450087889978, 4.7735722269336556], [2.115780916941155, 1.2131987965646083, 2.40074629089683, 0.55681730299662568], [0.42970161794353601, -4.3673930730452382, -4.6144131929378442, -0.21618639403716955]], [[-3.6039556845778584, 4.4238547850362373, -2.1738148306539804, -1.0893265703308841], [-0.23288275002959757, 2.1929333216142641, -4.3138302081906463, 4.0781389690962833], [-3.4805235410254101, 3.5459725300295375, 1.3340850368260746, 0.9408805053339826]]], [[[-3.9720853241450715, -3.9697560722410632, 0.64947383694557459, -3.3589467591802835], [-1.152187882956206, 1.7139822089872183, -0.24479141385099901, 1.4713401643198463], [1.8037356446411774, 3.3671073599000181, -2.7676413279985965, 1.2693694276320269]], [[-0.5801786534589084, -0.35145078940788466, 3.8666486764843313, -3.8296184308262795], [1.0307539581648744, 1.6646338928178981, 1.1057945965589138, 4.7065467745568252], [-4.6393111898873594, 1.8573576212066758, -4.681530394819684, 4.6969940579412413]]]])
      arg1=Data(3.20968833843,self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.0219786053126132, 7.2681501466055138, 7.9856357786915755, 3.3902624154811924], [1.2441970229793169, 3.113991861587273, 0.75738351388140712, 1.7883428227332727], [-1.4261190848866683, 1.7969548956418242, 0.5385646057747544, 3.3144052743790668]], [[-0.047233892291776769, -0.17976978885163319, -0.11643295180879321, 8.0742344191921234], [6.6841557120865041, 8.0922304235426061, 7.8506467170346941, -1.7107706410024424], [0.66133003179576422, 1.9836262449572555, 5.5341705078406633, -0.30000456626277128]]], [[[-1.3931885822362151, 7.2107112433769345, -0.85898476261601786, 7.8885228362565485], [-1.1493853744371698, 7.7738319575364159, 6.4640346169976368, 5.9758950790241174], [2.101249060590527, 1.2652478660434112, 2.4616222555074136, 0.83891202721930647]], [[-0.72441725583245864, 2.0988582732093244, -1.3070643835972682, 0.57391274414728066], [-0.46678191539847802, 7.1878470253193658, 7.4196185432401425, 6.1450301867122761], [-1.5743196629029068, 2.6933243421394959, 4.6529205488752714, 3.652354462492899]]], [[[0.17011425535457469, -1.6183097575346803, 1.7444954708070828, 6.7485430509519926], [8.0538285881710028, 6.1401194495057849, 1.746141239986974, -0.44265007602747719], [3.7793969489552914, 6.5035324038251723, -0.63215146431321845, 0.51201795434715436]], [[-1.7540509973358605, 7.7576245408002755, 3.9371344094906862, 4.2174812933254682], [0.52246846921247059, 1.502716870374833, 0.78225896369385062, 6.7405701927170458], [5.8088774089634692, -1.0366935473263137, 3.1716452827131487, 5.4095452485365492]]]])+(1.-msk_ref)*numpy.array([[[[3.1649295709277316, 4.0226232651438441, -1.0907605327760637, -1.2248972872454598], [4.5134733666319145, 5.9941502529262296, 1.2623004348283731, 3.8538038386822118], [0.99481174439238584, -0.70681121965194649, 1.8845666054559245, 1.6154400634029811]], [[6.718728876261407, 7.9669309022453927, 5.4203184478278734, 3.579874788449124], [5.8000424755799189, 2.9778208862046567, -1.2014124491902605, 3.718640575449232], [7.6808785700025268, 2.3213587535377798, 0.2273685177164646, 6.5180903075394987]]], [[[-1.1565588434239702, 7.5744586569080123, -1.0067617494606935, 7.98326056536294], [5.3254692553704395, 4.4228871349938927, 5.6104346293261145, 3.7665056414259102], [3.6393899563728205, -1.1577047346159537, -1.4047248545085598, 2.9935019443921149]], [[-0.39426734614857395, 7.6335431234655218, 1.0358735077753041, 2.1203617680984004], [2.9768055883996869, 5.4026216600435486, -1.1041418697613619, 7.2878273075255677], [-0.27083520259612559, 6.755660868458822, 4.5437733752553591, 4.1505688437632671]]], [[[-0.76239698571578707, -0.76006773381177872, 3.8591621753748591, -0.14925842075099904], [2.0575004554730785, 4.9236705474165028, 2.9648969245782855, 4.6810285027491307], [5.0134239830704619, 6.5767956983293026, 0.44204701043068795, 4.4790577660613113]], [[2.6295096849703761, 2.8582375490213998, 7.0763370149136158, -0.61993009239699504], [4.2404422965941588, 4.8743222312471826, 4.3154829349881982, 7.9162351129861097], [-1.4296228514580749, 5.0670459596359603, -1.4718420563903996, 7.9066823963705257]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.100658362609)+(1.-msk_arg0)*(-2.83293842392)
      arg1=Data(numpy.array([[[[2.3502660078065443, -2.7277652121050866, -2.8623602690244621, 4.9682136489107958], [-2.5835029787704866, -4.3945537325394426, -4.4465636265592945, -2.0478822115914577], [-0.20317199075721959, 0.0012053197267487548, 0.43634928104563819, -1.1716848143346184]], [[0.38888715268221397, -3.1355189331728228, 1.9086177743313995, -3.2290032349945061], [0.3360894451144496, 2.5714706895317629, 3.7863807996601277, 4.4869325820512707], [-4.5560559468358726, 4.3293198849848853, 4.6147485271160207, -4.1088156963026181]]], [[[1.0029730684305429, 4.7682725415990372, -4.5378418481423672, -2.1134601940440376], [-4.7727468919812761, -2.6527709114203768, -0.45726459470346725, -4.2818250497290826], [-3.8073288458950625, 1.0368313419288118, -3.0732865173475843, -4.6975800731809505]], [[-0.83663011355928241, -1.7525121866369719, 2.316720361881246, 4.074780135773862], [-4.6508815274216975, 4.4131987731461972, -4.1562289788618845, -3.3178511608201022], [1.3688439772572742, 3.7950384532323724, 4.4310526431346702, 4.4477297291329023]]], [[[-2.1145816888300963, -2.0676881523239929, -4.8456103380632172, 1.5952576014486164], [3.145682361970394, -1.505633223930043, -4.2316706461052158, 4.1328496324414878], [4.9335682219516208, 1.1315232657084628, -4.4309724257945184, -0.15185476370507445]], [[-1.905506288140979, -3.3260493009915582, 2.6504962568083137, 0.4168150022494661], [-2.583303539239965, 3.7767618179344478, 3.7472964497806043, -2.9054129817814145], [-2.8372358882961493, -3.2102860333698144, -2.7658527147827741, -3.7615662880991785]]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.2496076451974636, -2.8284235747141673, -2.9630186316335427, 4.8675552863017151], [-2.6841613413795673, -4.4952120951485233, -4.5472219891683752, -2.1485405742005383], [-0.30383035336630027, -0.099453042882331921, 0.33569091843655752, -1.272343176943699]], [[0.28822879007313329, -3.2361772957819035, 1.8079594117223188, -3.3296615976035868], [0.23543108250536893, 2.4708123269226823, 3.6857224370510471, 4.38627421944219], [-4.6567143094449532, 4.2286615223758046, 4.51409016450694, -4.2094740589116988]]], [[[0.90231470582146223, 4.6676141789899566, -4.6385002107514479, -2.2141185566531183], [-4.8734052545903568, -2.7534292740294575, -0.55792295731254793, -4.3824834123381633], [-3.9079872085041432, 0.93617297931973109, -3.173944879956665, -4.7982384357900312]], [[-0.93728847616836308, -1.8531705492460526, 2.2160619992721653, 3.9741217731647813], [-4.7515398900307781, 4.3125404105371166, -4.2568873414709651, -3.4185095234291829], [1.2681856146481936, 3.6943800906232918, 4.3303942805255895, 4.3470713665238216]]], [[[-2.215240051439177, -2.1683465149330736, -4.9462687006722978, 1.4945992388395357], [3.0450239993613133, -1.6062915865391236, -4.3323290087142965, 4.0321912698324072], [4.8329098593425401, 1.0308649030993822, -4.5316307884035991, -0.25251312631415512]], [[-2.0061646507500597, -3.4267076636006388, 2.549837894199233, 0.31615663964038543], [-2.6839619018490457, 3.6761034553253671, 3.6466380871715236, -3.0060713443904952], [-2.9378942509052299, -3.3109443959788951, -2.8665110773918547, -3.8622246507082592]]]])+(1.-msk_ref)*numpy.array([[[[-0.48267241610969691, -5.5607036360213282, -5.6952986929407032, 2.1352752249945546], [-5.4164414026867274, -7.2274921564556838, -7.2795020504755357, -4.8808206355076988], [-3.0361104146734608, -2.8317331041894924, -2.396589142870603, -4.0046232382508595]], [[-2.4440512712340272, -5.968457357089064, -0.92432064958484172, -6.0619416589107473], [-2.4968489788017916, -0.26146773438447823, 0.95344237574388657, 1.6539941581350295], [-7.3889943707521137, 1.4963814610686441, 1.7818101031997795, -6.9417541202188593]]], [[[-1.8299653554856983, 1.9353341176827961, -7.3707802720586084, -4.9463986179602788], [-7.6056853158975173, -5.485709335336618, -3.2902030186197084, -7.1147634736453238], [-6.6402672698113037, -1.7961070819874294, -5.9062249412638259, -7.5305184970971917]], [[-3.6695685374755236, -4.5854506105532131, -0.51621806203499521, 1.2418417118576208], [-7.4838199513379386, 1.5802603492299561, -6.9891674027781256, -6.1507895847363434], [-1.4640944466589669, 0.96210002931613126, 1.598114219218429, 1.6147913052166611]]], [[[-4.9475201127463375, -4.9006265762402341, -7.6785487619794583, -1.2376808224676248], [0.31274393805415279, -4.3385716478462841, -7.064609070021457, 1.2999112085252467], [2.1006297980353796, -1.7014151582077783, -7.2639108497107596, -2.9847931876213156]], [[-4.7384447120572197, -6.1589877249077993, -0.18244216710792749, -2.4161234216667751], [-5.4162419631562067, 0.94382339401820659, 0.9143580258643631, -5.7383514056976557], [-5.6701743122123904, -6.0432244572860556, -5.5987911386990152, -6.5945047120154197]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.5396932828498526, -3.5615435942057196])+(1.-msk_arg0)*numpy.array([-0.97055649986543102, 4.7796879376207713])
      arg1=Data(numpy.array([1.1870637799828243, -0.61502019877375069]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-2.3526295028670283, -4.1765637929794703])+(1.-msk_ref)*numpy.array([0.21650728011739329, 4.1646677388470206])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.70111962144683826, -2.5619239955683115], [0.20068080981952363, 3.7863979689939136]])+(1.-msk_arg0)*numpy.array([[3.6046914828602308, -4.4069939048945592], [-0.83576543006098802, 4.8715460010378013]])
      arg1=Data(numpy.array([0.17634089436241851, -0.98435667569753882]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.52477872708441975, -2.385583101205893], [-0.78367586587801519, 2.8020412932963747]])+(1.-msk_ref)*numpy.array([[3.7810323772226493, -4.2306530105321407], [-1.8201221057585268, 3.8871893253402625]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.3249830277925856, -0.7316126242936436])+(1.-msk_arg0)*numpy.array([3.1003070630775991, 3.74489062084659])
      arg1=Data(numpy.array([[-1.5296625879273029, -1.590419704629241], [-4.0813476572229712, -3.2869194342881491]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.7953204398652827, 1.7345633231633446], [-4.8129602815166148, -4.0185320585817923]])+(1.-msk_ref)*numpy.array([[1.5706444751502961, 1.5098873584483581], [-0.33645703637638125, 0.45797118655844082]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-1.1195602581640918, -0.17544518350709382, -0.053975963628242418, -3.7622112643281955, 3.1771169465351843], [-3.508963307601265, 2.001921916526852, -0.47493478146619239, -2.1505527335907959, 0.12386546878839688], [2.9890121191735073, -0.91402609432888671, 1.8032704122260643, -0.86990622515846994, 4.422300383980847], [-1.396227199248016, 3.475018642012472, 2.9562190042914267, 2.9132216135849003, 2.1309968376398345]], [[1.3117104460566651, 1.2816237303895957, 4.1440063485614687, 4.5079473881161185, 1.4476434961623053], [-1.7205447026151099, 4.4474433666984989, 1.1579990324497382, 1.3266133771133868, -4.3775558222394997], [-3.2920252273321795, 4.3183087050898585, -0.96262245125183199, -3.6584988652916883, -2.7863690297031773], [-0.53688607612812245, -2.5784183682389559, 2.9018373185226096, -2.1201647212011308, 3.0204692414183452]]])+(1.-msk_arg0)*numpy.array([[[1.4966909312623899, 4.3294256046504831, 2.0918923537400138, -3.0154452204528406, 2.6948074545740113], [-0.50306786632602041, -0.97183391799622498, 3.7445453968557665, -3.9385826695874271, -3.6613279824589338], [3.3313216976050271, 3.7253677755847168, 3.2717089750691581, -1.0025193157295451, 3.9391506989586293], [-4.6352397073417748, 2.251968097055097, 2.0728849223209895, -4.176761733734125, 0.94121607633339455]], [[2.5464748606414176, 4.7133497794446306, 4.8082689988508331, -1.8825288690049313, -2.7187816909333593], [-0.16070921621690992, 4.9256970688398063, 3.0528342941265905, 1.0077486589021492, -3.3381201426861762], [4.3900506483162101, -0.26600148321108907, 1.6524711364161373, 3.7407347937698834, -0.63563885675032061], [-2.7376030375718186, 1.4280994571109007, -1.5890069663360529, -2.1874900972736877, -2.4034017104441476]]])
      arg1=Data(numpy.array([-3.0898261435640095, -2.6833400739787749]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-4.2093864017281017, -3.2652713270711033, -3.1438021071922519, -6.8520374078922046, 0.087290802971174752], [-6.598789451165274, -1.0879042270371575, -3.5647609250302019, -5.2403788771548054, -2.9659606747756126], [-0.10081402439050224, -4.0038522378928967, -1.2865557313379452, -3.9597323687224795, 1.3324742404168375], [-4.4860533428120259, 0.38519249844846248, -0.13360713927258283, -0.17660452997910925, -0.95882930592417503]], [[-1.3716296279221099, -1.4017163435891793, 1.4606662745826937, 1.8246073141373436, -1.2356965778164697], [-4.4038847765938849, 1.764103292719724, -1.5253410415290367, -1.3567266968653882, -7.0608958962182751], [-5.9753653013109549, 1.6349686311110836, -3.6459625252306069, -6.3418389392704633, -5.4697091036819518], [-3.2202261501068974, -5.2617584422177313, 0.21849724454383468, -4.8035047951799061, 0.33712916743957022]]])+(1.-msk_ref)*numpy.array([[[-1.5931352123016196, 1.2395994610864736, -0.99793378982399572, -6.1052713640168506, -0.39501868898999826], [-3.5928940098900299, -4.0616600615602341, 0.65471925329175695, -7.0284088131514366, -6.7511541260229428], [0.24149555404101752, 0.63554163202070724, 0.18188283150514861, -4.0923454592935542, 0.84932455539461982], [-7.7250658509057839, -0.83785804650891249, -1.01694122124302, -7.266587877298134, -2.148610067230615]], [[-0.13686521333735735, 2.0300097054658557, 2.1249289248720582, -4.5658689429837063, -5.4021217649121347], [-2.8440492901956849, 2.2423569948610313, 0.36949422014781552, -1.6755914150766258, -6.0214602166649511], [1.7067105743374351, -2.949341557189864, -1.0308689375626376, 1.0573947197911084, -3.3189789307290956], [-5.4209431115505939, -1.2552406168678742, -4.2723470403148278, -4.8708301712524626, -5.086741784422923]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.25473716560154358, 3.6836535222321736])+(1.-msk_arg0)*numpy.array([3.1152307266291803, 2.4737306356078905])
      arg1=Data(numpy.array([[[3.141872090051212, 2.6928750588327404, -4.6778774363517321, 3.0999785524245205, 4.7028356341496576], [-4.205584148005185, -4.0891816037263338, 1.5062192946193544, -0.7088566203518889, 0.29622455786403901], [-3.0227683529214957, 1.2976874747044036, -3.2070160574848483, 1.5932859439081497, 4.1811266333867483], [0.41918185699682731, 2.1510554884612363, 4.5693004242314554, -1.7899581067407979, -2.5649729705226676]], [[-4.1545250403685063, -2.9651819015801149, 4.9401429308105342, 3.9104928496942275, 0.092048995253675159], [-3.8158311253492738, -0.24826331487511322, 4.3676520027771488, 3.2182479265829791, 2.1638673847278014], [2.1219360717944351, -0.12944839730584246, -2.6875607880224441, 2.286861619911436, 0.13335850339519872], [-3.7807490402722763, 2.8846747471853025, 4.2490539003630658, 2.3415808132455327, 0.04208481423885857]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.3966092556527556, 2.9476122244342839, -4.4231402707501886, 3.3547157180260641, 4.9575727997512011], [-3.9508469824036414, -3.8344444381247902, 1.760956460220898, -0.45411945475034532, 0.55096172346558259], [-2.7680311873199521, 1.5524246403059472, -2.9522788918833047, 1.8480231095096933, 4.4358637989882919], [0.67391902259837089, 2.4057926540627799, 4.824037589832999, -1.5352209411392543, -2.310235804921124]], [[-0.47087151813633277, 0.71847162065205872, 8.6237964530427078, 7.5941463719264011, 3.7757025174858487], [-0.13217760311710025, 3.4353902073570604, 8.0513055250093224, 6.9019014488151527, 5.847520906959975], [5.8055895940266087, 3.5542051249263311, 0.99609273420972944, 5.9705151421436096, 3.8170120256273723], [-0.097095518040102746, 6.5683282694174761, 7.9327074225952394, 6.0252343354777063, 3.7257383364710321]]])+(1.-msk_ref)*numpy.array([[[6.2571028166803924, 5.8081057854619207, -1.5626467097225518, 6.2152092790537008, 7.8180663607788379], [-1.0903534213760047, -0.97395087709715344, 4.6214500212485348, 2.4063741062772914, 3.4114552844932193], [0.09246237370768462, 4.4129182013335839, -0.091785330855667979, 4.70851667053733, 7.2963573600159286], [3.5344125836260076, 5.2662862150904166, 7.6845311508606358, 1.3252726198883824, 0.55025775610651273]], [[-1.6807944047606158, -0.49145126597222433, 7.4138735664184248, 6.384223485302118, 2.5657796308615657], [-1.3421004897413833, 2.2254673207327773, 6.8413826383850393, 5.6919785621908696, 4.6375980203356919], [4.5956667074023256, 2.3442822383020481, -0.21383015241455361, 4.7605922555193265, 2.6070891390030893], [-1.3070184046643858, 5.358405382793193, 6.7227845359709564, 4.8153114488534232, 2.5158154498467491]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-1.2607317394593585, -2.3123074578819116], [-2.4366996560128964, 2.0117320931320615]], [[4.8949734504980764, -0.090032562198338439], [-3.0029199321944136, -3.5514840916174584]], [[0.24712336826007153, -0.78653089451918756], [4.4387335323565349, -3.6187547693412436]], [[0.83479088410293123, -2.4258777110346696], [-1.6948881058153544, -3.7472836608713331]], [[-3.3380143792853092, 3.9193782890958353], [-3.1940035892335006, -2.7792972941827978]], [[4.5550203803185845, 1.6162466669153277], [-4.4461048995914076, -2.6941471911564596]]], [[[4.0119854320667034, 1.0962505745244391], [-0.41165523034244877, -3.9738046097488833]], [[4.3082791359655701, -1.3835982892225287], [2.3279384556683649, -3.0755870363152127]], [[0.93107651565129324, -1.0625219985822723], [-4.4727171937589647, 3.5860924193821457]], [[4.652812177603419, -1.1675265710280236], [1.7509576198530974, -0.55605067811028341]], [[-3.5498855731678614, -3.2930072653210285], [-2.1215980421192127, 4.4404703632833513]], [[-0.78655636850142052, -3.267934082222177], [-1.2694509720846203, -0.84928768352068307]]]])+(1.-msk_arg0)*numpy.array([[[[4.5214018461325001, -0.92899966255871025], [-0.64107580399601982, 1.7784834864802388]], [[-1.0868591833373364, 2.9991368478468692], [4.124150295107988, -2.7996655176468299]], [[-0.15060250070676151, 3.2418867391085922], [-2.7335385958624192, 1.816028209444517]], [[3.5619792344349186, -4.3825887758897704], [4.034395489990958, -1.5991509671709125]], [[0.55888310011331477, 2.5893171446916607], [-2.0594953435181038, -3.418519690124092]], [[-2.9737517258110246, 0.79293581275826508], [0.077810894895353755, 4.8780976773903024]]], [[[3.4129744408662805, 3.1249808204380773], [-3.8757861186108187, 0.67127799025282986]], [[3.1315198032972056, 3.0826593704160885], [1.4565320020833035, -4.9582703201638969]], [[-1.9351882959556352, 2.5961046653747424], [-2.9270958752340972, -1.7732563548655476]], [[3.2034980099600947, -2.7623353620612248], [0.48876147432135042, 2.2485005532147673]], [[-3.5958755378707732, -0.06325911469611345], [0.84938891239595549, -0.095012673681794446]], [[1.2295074993765907, 1.1784434889530546], [0.75385702838951385, 4.9595473743867142]]]])
      arg1=Data(numpy.array([-0.00081818492704144319, -0.84970128701594572]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.2615499243863999, -2.3131256428089531], [-2.4375178409399378, 2.01091390820502]], [[4.894155265571035, -0.090850747125379883], [-3.003738117121455, -3.5523022765444998]], [[0.24630518333303009, -0.78734907944622901], [4.4379153474294935, -3.6195729542682851]], [[0.83397269917588979, -2.4266958959617111], [-1.6957062907423959, -3.7481018457983746]], [[-3.3388325642123506, 3.9185601041687939], [-3.194821774160542, -2.7801154791098392]], [[4.554202195391543, 1.6154284819882863], [-4.4469230845184491, -2.6949653760835011]]], [[[3.1622841450507577, 0.24654928750849336], [-1.2613565173583945, -4.8235058967648285]], [[3.4585778489496244, -2.2332995762384744], [1.4782371686524192, -3.9252883233311584]], [[0.081375228635347518, -1.912223285598218], [-5.3224184807749104, 2.7363911323662]], [[3.8031108905874733, -2.0172278580439693], [0.90125633283715167, -1.4057519651262291]], [[-4.3995868601838071, -4.1427085523369742], [-2.9712993291351584, 3.5907690762674056]], [[-1.6362576555173662, -4.1176353692381227], [-2.119152259100566, -1.6989889705366288]]]])+(1.-msk_ref)*numpy.array([[[[4.5205836612054586, -0.92981784748575169], [-0.64189398892306126, 1.7776653015531974]], [[-1.0876773682643779, 2.9983186629198277], [4.1233321101809466, -2.8004837025738714]], [[-0.15142068563380295, 3.2410685541815507], [-2.7343567807894607, 1.8152100245174756]], [[3.5611610495078772, -4.3834069608168118], [4.0335773050639165, -1.599969152097954]], [[0.55806491518627332, 2.5884989597646193], [-2.0603135284451453, -3.4193378750511334]], [[-2.9745699107380661, 0.79211762783122364], [0.076992709968312312, 4.877279492463261]]], [[[2.5632731538503348, 2.2752795334221316], [-4.7254874056267644, -0.17842329676311586]], [[2.2818185162812599, 2.2329580834001428], [0.60683071506735775, -5.8079716071798426]], [[-2.784889582971581, 1.7464033783587967], [-3.7767971622500429, -2.6229576418814933]], [[2.353796722944149, -3.6120366490771705], [-0.3609398126945953, 1.3987992661988216]], [[-4.4455768248867189, -0.91296040171205917], [-0.00031237461999022997, -0.94471396069774016]], [[0.37980621236064493, 0.3287422019371089], [-0.095844258626431866, 4.1098460873707685]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.1751627289412525, -3.7789475962766108])+(1.-msk_arg0)*numpy.array([1.7603845058818566, 0.98520473015407184])
      arg1=Data(numpy.array([[[[2.5034744005130385, 0.70253893287126967], [0.8624989540662753, 2.3494834348542115]], [[-3.2098501113037581, -0.94195210689116138], [-4.3865460633539417, 2.0687367929058498]], [[4.4713184747264787, -3.7584683228995686], [1.1557648094687467, -4.6207392327724754]], [[-4.5050699218484436, -1.660980063086904], [4.6007568563018779, 1.9638283543909365]], [[-3.9130849171763424, 2.1043019382699484], [3.0193580736076893, 1.1857274977206984]], [[1.6334167366105543, 3.5000870418380323], [-0.30287856604867969, 2.7398140185199331]]], [[[-1.2847101469602737, -2.2916219857682663], [-3.4181020790502425, -3.8679985562683425]], [[-3.1809414776717948, -4.7770553169099319], [-4.872312745884555, -3.0938103600649502]], [[-4.1621282012432284, -0.2686596947860922], [-3.213571913647427, 3.1452031868299599]], [[3.2440107658712485, 4.1008501654696836], [-4.1693018473244123, 3.4160452661923859]], [[3.1753738820815816, -3.3374746443243697], [-0.041238625650597882, -3.598176553678829]], [[4.8020249750884609, 1.1751067868785103], [2.4386192804736968, -1.9991955882117352]]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.678637129454291, 2.8777016618125222], [3.0376616830075278, 4.524646163795464]], [[-1.0346873823625056, 1.2332106220500911], [-2.2113833344126892, 4.2438995218471023]], [[6.6464812036677312, -1.583305593958316], [3.3309275384099992, -2.4455765038312229]], [[-2.3299071929071911, 0.51418266585434846], [6.7759195852431304, 4.138991083332189]], [[-1.7379221882350899, 4.2794646672112009], [5.1945208025489418, 3.3608902266619509]], [[3.8085794655518068, 5.6752497707792848], [1.8722841628925728, 4.9149767474611856]]], [[[-5.0636577432368846, -6.0705695820448771], [-7.1970496753268538, -7.6469461525449534]], [[-6.9598890739484052, -8.5560029131865427], [-8.6512603421611658, -6.8727579563415606]], [[-7.9410757975198392, -4.047607291062703], [-6.9925195099240378, -0.63374440944665089]], [[-0.53493683040536233, 0.32190256919307281], [-7.9482494436010231, -0.3629023300842249]], [[-0.60357371419502925, -7.1164222406009809], [-3.8201862219272087, -7.3771241499554403]], [[1.0230773788118501, -2.6038408093981005], [-1.340328315802914, -5.7781431844883464]]]])+(1.-msk_ref)*numpy.array([[[[4.2638589063948951, 2.4629234387531262], [2.6228834599481319, 4.1098679407360681]], [[-1.4494656054219015, 0.8184323989906952], [-2.6261615574720851, 3.8291212987877064]], [[6.2317029806083353, -1.998083817017712], [2.9161493153506033, -2.8603547268906189]], [[-2.744685415966587, 0.09940444279495253], [6.3611413621837345, 3.7242128602727931]], [[-2.1527004112944859, 3.864686444151805], [4.7797425794895458, 2.946112003602555]], [[3.3938012424924109, 5.2604715477198889], [1.4575059398331769, 4.5001985244017897]]], [[[-0.29950541680620191, -1.3064172556141944], [-2.4328973488961707, -2.8827938261142707]], [[-2.195736747517723, -3.7918505867558601], [-3.8871080157304831, -2.1086056299108784]], [[-3.1769234710891565, 0.71654503536797964], [-2.2283671834933552, 4.1304079169840318]], [[4.2292154960253203, 5.0860548956237555], [-3.1840971171703405, 4.4012499963464577]], [[4.1605786122356534, -2.3522699141702978], [0.94396610450347396, -2.6129718235247572]], [[5.7872297052425328, 2.1603115170325822], [3.4238240106277686, -1.0139908580576633]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.9848003542347721, -4.8583615017846302, -0.69809516419995443, -0.69717317964159253, -0.084019242709323372], [-0.45191786574878634, 3.027618193326969, -0.15328565701638652, 1.1495767328789031, 1.3549543817293443], [1.8938106087029274, -0.13006658944953742, -3.4152895282567695, -2.6265806891052748, 3.8989622792587006], [-0.17569192235393771, 0.49437365908396647, 2.6323918005584606, 3.9695288742799004, 0.79532006720465009]])+(1.-msk_arg0)*numpy.array([[2.8201233809035289, -1.2232375532086381, -4.0594361623293587, 2.4971025997052152, 2.6754710388073324], [-2.4579431778461682, 2.9603723488417799, 3.8726862683949186, 1.613553936564478, -3.2020257234258254], [-0.19875816020779524, -3.0357604648034595, 3.9611324831624888, -2.8282667922921423, -1.1947931544906432], [0.61870418876583777, -2.1510483438241348, 2.0477466236281359, -3.644308250127489, -4.4429444230605828]])
      arg1=Data(numpy.array([[1.8726367945622915, 4.1971069157863354, -4.9645095978023681, 3.3142058788757236, -1.5158404777110701], [1.9438085399443006, 2.8149357153902024, -1.1864007944920116, -1.3431682572182515, -3.0700397267923663], [-0.45553467948117898, -0.63238572742469135, -4.673188432452962, -1.3156796270088633, 0.13052711046125864], [-3.4196831068270028, -4.970526163531491, -0.73956142636959132, 3.4299710953296429, 4.2667338812616098]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.1121635596724806, -0.66125458599829479, -5.6626047620023225, 2.6170326992341311, -1.5998597204203935], [1.4918906741955142, 5.8425539087171714, -1.3396864515083982, -0.19359152433934845, -1.7150853450630219], [1.4382759292217484, -0.76245231687422876, -8.0884779607097315, -3.9422603161141381, 4.0294893897199593], [-3.5953750291809405, -4.4761525044475245, 1.8928303741888692, 7.3994999696095434, 5.0620539484662599]])+(1.-msk_ref)*numpy.array([[4.6927601754658204, 2.9738693625776973, -9.0239457601317277, 5.8113084785809388, 1.1596305610962623], [-0.51413463790186764, 5.7753080642319823, 2.686285473902907, 0.27038567934622648, -6.2720654502181912], [-0.65429283968897423, -3.6681461922281509, -0.71205594929047322, -4.143946419301006, -1.0642660440293845], [-2.8009789180611651, -7.1215745073556258, 1.3081851972585445, -0.2143371547978461, -0.176210541798973]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.3216628944710909, -2.541190531601635], [0.96195215756200714, 0.091737185293188972], [0.56581228111030146, 1.6106074177938066], [-3.2341774918464128, -3.8627149222180055], [3.9300104943877905, 4.3150154569667833]], [[0.70509867082768185, -0.75041898790869155], [1.1438924728035191, 0.92817026823566628], [-4.4906513841005964, -4.7003675246108632], [2.3114669046659655, 0.12236568620827093], [-4.1578465400804507, -1.7942953448990604]], [[-2.6655309588595966, -3.46824876372958], [-4.9741400229816533, 4.3424837671177272], [2.7616450137959125, 2.0048057139163955], [-1.2547856439762062, -1.2584444568900279], [4.8802066608750039, -3.0433219353224263]], [[-1.512818056674349, -4.1469494137394918], [3.7353411272216857, -2.4549402428434153], [0.30060788721043252, -4.0533616104741821], [-0.72159907128568612, -2.7199449592664884], [-4.3778207465612144, -0.6939327606492327]]])+(1.-msk_arg0)*numpy.array([[[2.3156357838276973, -4.5723585497286345], [3.1681573789151827, 0.25426360947602955], [-4.1520896775516567, -3.4187794553264803], [-4.2795521096198685, -1.7472267332781808], [-1.4808712937466062, 1.2946866263392174]], [[0.74457172269788519, 4.8467489032827356], [0.032000564283838351, -4.3472718262680257], [4.0597905204436877, 1.746188986759063], [3.2255508628280882, 0.59992916079570779], [2.7475232533353875, 1.5757063584033881]], [[4.3756191813547005, -3.9095688776384732], [-0.16961310622309966, -0.69782210081540441], [-4.9690079022220743, 2.6281805961911555], [-2.988454754705546, -3.0877890433028412], [-2.1039735677067162, -2.497336872421656]], [[-2.9660515419179867, -4.2203175678782747], [-3.0261204984837855, -3.9556632562346405], [0.95855694394954138, 2.9348568683443208], [1.5891584455987493, 0.99612301778478596], [-3.1995261324690594, -3.2589618087496488]]])
      arg1=Data(numpy.array([[-2.0801532450616769, 1.9839398459021957, 3.675325008516678, 1.2901812645305704, -1.9770035735683802], [-0.70347595318614964, 4.1275207080761831, 2.7342055108725871, 2.8877142301598777, 1.3379482615604186], [3.285258436049487, -4.3083294097581, -0.77094982121772926, -1.5099962316261326, -0.39118986406726197], [-2.6974176112700254, -4.499153230349366, -3.8631245894659139, 1.5415958354261114, -0.48305875262828035]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.241509649409414, -4.6213437766633119], [2.9458920034642029, 2.0756770311953847], [4.2411372896269794, 5.2859324263104845], [-1.9439962273158424, -2.5725336576874351], [1.9530069208194103, 2.3380118833984032]], [[0.001622717641532212, -1.4538949410948412], [5.2714131808797022, 5.0556909763118494], [-1.7564458732280093, -1.966162013738276], [5.1991811348258432, 3.0100799163681486], [-2.8198982785200322, -0.45634708333864182]], [[0.61972747718989041, -0.18299032768009305], [-9.2824694327397523, 0.034154357359627241], [1.9906951925781833, 1.2338558926986662], [-2.7647818756023388, -2.7684406885161605], [4.4890167968077419, -3.4345117993896883]], [[-4.2102356679443744, -6.8443670250095172], [-0.76381210312768033, -6.9540934731927813], [-3.5625167022554813, -7.9164861999400955], [0.81999676414042533, -1.1783491238403769], [-4.8608794991894948, -1.1769915132775131]]])+(1.-msk_ref)*numpy.array([[[0.23548253876602043, -6.6525117947903114], [5.1520972248173784, 2.2382034553782253], [-0.47676466903497872, 0.25654555319019767], [-2.9893708450892982, -0.45704546874761043], [-3.4578748673149864, -0.68231694722916281]], [[0.041095769511735547, 4.1432729500965859], [4.1595212723600214, -0.21975111819184256], [6.7939960313162748, 4.4803944976316501], [6.1132650929879659, 3.4876433909555855], [4.0854715148958061, 2.9136546199638067]], [[7.6608776174041875, -0.62431044158898619], [-4.4779425159811996, -5.0061515105735044], [-5.7399577234398036, 1.8572307749734263], [-4.4984509863316786, -4.5977852749289738], [-2.4951634317739781, -2.888526736488918]], [[-5.6634691531880126, -6.9177351791483002], [-7.5252737288331515, -8.4548164865840061], [-2.9045676455163725, -0.92826772112159306], [3.1307542810248608, 2.5377188532108974], [-3.6825848850973397, -3.7420205613779292]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.65315309064230043, 0.089280912244936061, -4.7803710303744236, 3.7664211551587083, -3.9159345004014967], [-3.3016913124489888, 3.491410044347397, 3.7972623138789245, 4.09869041148918, -1.1180694526678394], [4.4269129777415976, 4.7258092478876179, -0.40483911752208535, 3.9310362036624547, -2.6460081131406801], [4.5257193048346274, -2.5982695697465941, -1.820483817971017, -4.3957236537455664, -0.52785767309763454]])+(1.-msk_arg0)*numpy.array([[3.1750043898291338, -4.3796890317957864, -0.89141345489536938, 1.0004550332998114, -4.6045392188312881], [-3.615806968598041, -1.326756997201751, -2.6639226938551275, 3.2458171167353598, 3.2417536198757713], [-4.8539137258374616, 4.0552459414981676, 4.2535766105737665, -1.768952333907392, -4.8662117927267978], [-4.2315771798615014, -1.6650179393397737, -2.4795393155444359, 4.5045937711688513, -4.9394894681437265]])
      arg1=Data(numpy.array([[[-2.8365282771346934, -0.34875202255527871], [-0.040122348831078014, -0.15960344624650524], [-2.9534098467918368, -2.4315964409411461], [3.742505219149038, -2.7596050672228598], [0.92239367559002616, 1.9151766662592484]], [[-3.3665671681390466, 1.5342288352237512], [1.7927431522989536, 1.6562619800846692], [-0.054020959088672882, 3.6147744128730377], [-0.31356436409024724, -4.2697677819218347], [-0.47604079431254753, 1.2418340579469458]], [[-3.8542747751235718, -1.0534591798471382], [-4.6662992509976782, 4.041834409569848], [-3.8738668530395062, 3.1016210742033703], [-2.8079941324159288, -0.050485623856194373], [-3.7017823934051028, 4.6116253536486891]], [[-2.1639747320593274, -0.41081397509343986], [0.47108740999754595, 2.6898031085680953], [2.7681205947464083, 0.26896378282040079], [0.88812917313684459, -4.4348003631164614], [0.35922687193297875, 3.9798975511760286]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.4896813677769938, -1.0019051131975791], [0.049158563413858047, -0.070322534001569181], [-7.7337808771662608, -7.2119674713155693], [7.5089263743077463, 1.0068160879358485], [-2.9935408248114705, -2.0007578341422483]], [[-6.6682584805880349, -1.7674624772252376], [5.2841531966463506, 5.1476720244320662], [3.7432413547902517, 7.4120367267519622], [3.7851260473989328, -0.1710773704326547], [-1.5941102469803869, 0.12376460527910638]], [[0.57263820261802589, 3.3734537978944594], [0.059509996889939742, 8.7676436574574659], [-4.2787059705615915, 2.696781956681285], [1.1230420712465259, 3.8805505798062603], [-6.3477905065457829, 1.965617240508009]], [[2.3617445727752999, 4.1149053297411875], [-2.1271821597490481, 0.091533538821501281], [0.94763677677539127, -1.5515200351506162], [-3.5075944806087218, -8.8305240168620287], [-0.16863080116465579, 3.4520398780783941]]])+(1.-msk_ref)*numpy.array([[[0.33847611269444045, 2.8262523672738551], [-4.4198113806268644, -4.5392924780422916], [-3.8448233016872062, -3.3230098958365155], [4.7429602524488494, -1.7591500339230484], [-3.682145543241262, -2.6893625525720397]], [[-6.9823741367370875, -2.0815781333742898], [0.46598615509720265, 0.32950498288291818], [-2.7179436529438004, 0.95085171901791021], [2.9322527526451125, -1.0239506651864749], [2.7657128255632237, 4.4835876778227171]], [[-8.7081885009610325, -5.9073729056845998], [-0.61105330949951053, 8.0970803510680156], [0.37970975753426028, 7.3551976847771368], [-4.5769464663233208, -1.8194379577635864], [-8.5679941861319016, -0.25458643907810874]], [[-6.3955519119208288, -4.6423911549549413], [-1.1939305293422278, 1.0247851692283216], [0.28858127920197241, -2.2105755327240351], [5.3927229443056959, 0.069793408052389871], [-4.5802625962107477, -0.95959191696769786]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.6865665599390791, 0.87936396338136458, 2.4389041870774628, -3.8430879554281603, -1.4963697681054788], [4.492259351130329, 2.6355724608339202, -0.80371207014234791, -0.74265709375562672, 4.5346320022761653], [2.4391650583699303, 4.4450433939967056, -3.2681630324604871, -1.6155392322052728, 2.5228724460317888], [1.7477260191011821, 4.1897882578792363, -2.4236845050225098, 2.174963273983356, 0.38183440404432556]], [[2.5806794168747826, -2.888530620164568, -4.3415425762946942, -4.9761020346610598, -1.6470895941413866], [-3.7962409195703883, 1.5205883207797992, -3.8400575072020549, 2.7836957665521576, -0.91258889047501324], [-2.1406704015406608, -4.1541908004524464, -0.50401197565046552, -3.6083445645902601, -4.6290940815896278], [-1.7359929912983354, -2.7761592686671266, 2.4127587714778009, 2.340952368897983, -0.35098925499136513]], [[1.4236973408825069, 4.2476585692686282, -0.025114918914058038, 3.346139362667488, -4.7642499444913273], [0.33761840582591951, 2.8899463499597076, 1.1366882302672963, 0.32700049386977259, 2.6908642407345447], [-4.9451386716929875, -4.8468727760398789, 3.5130387362425282, -3.0396181354950391, 1.7433042576113458], [3.5728944032264138, 1.7048577031306493, 1.7099969335115723, 4.9546185288557005, -4.6851307451426569]], [[-3.6659644825530933, 3.8622065157499144, -2.3791724732962125, -0.058412301941537237, 3.4411006020419563], [-2.6564112213225313, 3.577084185829996, -2.3478923855705136, 0.59299387534449544, -1.8694470576003228], [-0.58084272947865401, 4.0618751845276968, -2.7101069820341692, -0.39090169366038463, 2.1770713401111195], [2.3372897845176821, -0.47656252325837123, 0.82464451494386548, -2.2008108654345806, 4.6091868737621517]], [[-3.6036272078438705, 3.0703301284957192, -3.4760012025206768, 0.25902771414740489, -3.2801341296462496], [-1.6831104239152719, -3.178223331019959, 0.55384316602638606, 4.327684261308189, -4.237835828663691], [3.1665154441437942, -0.38714714170121312, 3.953430016624095, 1.4440588396579965, -4.7641301323915854], [-2.8963027152439724, 1.046870687461908, -4.4350971711632523, 0.32338866740510586, -1.9315134206931548]]], [[[3.2966722083838746, 2.2548871760020486, -3.1642044991978571, 1.0451334316051835, -0.95186763193074153], [3.5958321201090317, 3.8072832865720319, 1.1481778246954484, -3.6846189798439521, -1.1793264604985829], [-4.8497867108037838, -0.5305539555998795, -3.8408879952322894, -4.7166605416264282, -2.3916083574367075], [0.31979478996597877, -2.4922824468708416, 0.87846902097836388, 4.0156076377144316, 3.3674405384488821]], [[-4.9777072204414727, -3.7097267484916188, 3.940012600298255, -2.9755115052657146, 0.15014773097186485], [-1.9249696993079732, 3.381938358373457, 2.3740658162132631, -2.6785908841649766, 3.4922551365183541], [4.313286134862393, -1.4961435199049444, -3.6070332637172253, -1.1408469782361088, -2.3349972710642222], [-3.8922734867726607, 3.5918011792581659, 1.0589764546423863, 1.0433242359992621, 4.7848838087834391]], [[0.4739791557060471, 0.45283892258688674, -4.4267681924265769, 4.3626326259248476, 1.3007786507981676], [0.073095986183274952, -3.1646686484467357, 1.7411327447863965, 1.7683038800138098, -1.0789361741144265], [-4.4627743098769113, 4.9178756395844463, 2.2170918433840336, 2.2888888250651496, -3.8507289200781001], [-1.9081105855572442, -1.5466145993850144, 0.13944707219294283, 0.40866083215239346, 4.7396763098067556]], [[1.5329026880197478, 1.4650130803315466, -1.2677448718207573, -3.9639303501679359, -0.31101427540037996], [3.1031818983631894, -3.5361495852227067, -3.1229529414891397, 0.47224684049715115, -1.8749113902222581], [0.52493183851097047, -0.70248886502475649, 0.80355369562736367, 2.6813238013299765, -4.4170796876967247], [0.54879759537058526, -4.8823389999154863, -3.3408290680162542, -0.73369320139049066, 0.015379172340642455]], [[1.1048186604376697, 3.7371307885562857, -2.9512732933442676, 4.2676261336980144, -3.3214798478481669], [-2.1135755157998162, 1.3337305548729228, -4.3378482143567707, 0.99568749359737563, -4.281257440659604], [1.760246655946859, 3.7476852956036009, -1.1285477857010418, -1.4579087809132396, -0.80581810204565141], [-2.0881975217630999, -2.2390502274552015, -4.0525191131472491, -1.8301272021732604, -0.08062790903903938]]], [[[-2.7786965154248988, -3.4319066400623468, -3.0231537770406423, 1.9287561761767549, 1.7914926889275664], [-2.1700031460253468, -3.8788254491748217, 2.9508438888717343, 3.6600459872173712, -3.7500114943409302], [0.42833321433148264, -0.72980029334534535, 2.7646806893100742, -0.39860445832774438, -2.4658319283981465], [4.311199841454803, 3.0375750926251222, -3.4842349977928402, 2.1867679546856937, -4.9008879704821791]], [[-0.83103857523527758, 3.3716489795158626, -4.4070664921847378, -1.7072266266871661, 0.14454793883483141], [-1.6137015632352258, -1.3661463695975682, -4.3066771922911578, -1.7667276842167756, 0.12600082069107454], [1.1299019478322734, -2.7713225586493606, -0.32002536162123896, -2.5571505695993855, 4.5405048911267176], [2.7149022095398525, 0.011616587017035052, 2.4367493322582803, 0.24196362763426382, -0.53439990522878844]], [[-2.1985894861369646, 3.4912903556471129, 3.7033713762052507, -2.6185296871294597, -4.8612365744629473], [-2.9292270108004317, -3.836677106666917, 3.2660153420548443, -4.723347117895754, 3.8583932693076743], [-1.4812042144921209, -1.2522478048357444, 0.3633329717925502, -0.74056774575142281, 2.2330744822767077], [-2.3076992270514687, 4.3065709794728022, 2.3648292219442277, 3.3079869272415969, -0.030836718957186982]], [[-2.4543314071275435, -3.6927213100255116, -0.76680811963241347, 1.3042877529901231, -4.7109854727095453], [3.8757863968692767, -4.1767542231083423, 1.6525486806629317, -2.9734075239167934, -2.3794806748180908], [1.6313277895833798, 0.49975936164334467, 2.2738540151876263, -1.6315127347119573, -1.6605882207028611], [-2.0617241483185564, 2.5917855435488226, -3.8863327410806905, 4.2497905075534934, -3.7532268630230248]], [[-1.8221918630039968, -0.89042202730399733, 2.3439691740651583, -2.3169443412896893, 3.680151801770279], [3.547030574407998, 0.0031176918213917659, 2.9028002793334124, -2.9092047121680622, -1.047697199724956], [-1.612590542120742, 3.3182592573850975, 4.5789564569831374, 1.7956530029483702, -2.5167308680266687], [-3.0156788988202923, 1.7170531165351752, 2.8800429954380435, -1.1192673383469218, 1.3742629757812965]]], [[[2.8448633896173643, -4.1299359255780068, -4.5969147265463173, 0.75814596238575316, -4.7784799516770633], [4.6641743505139015, 4.2355686638184267, 0.16900858578891764, -0.33617437746535472, -4.7115899514631803], [3.5874785724550851, -0.98026366573168922, 2.6923980620747034, 3.2496897734358896, -1.453407416091923], [0.23118007479432556, -4.5720166093902037, 3.3423012161694903, 4.7795192506986144, -2.3395021796242057]], [[3.0022788556414284, 0.193932539559869, -0.91944911479707425, -4.1084515161770421, -2.3451775914664728], [-1.1087713215659334, -1.8221764712206467, 0.84615221570317001, 3.3739997543273521, -4.4899544360777028], [4.6302647020806518, -4.2345697408668324, 3.2755597392479103, 3.8826392485076688, 1.1674565315891439], [-0.4872018345512652, -1.9678216952065841, -4.2922264094251332, -4.1032322089709394, 2.8046581500211429]], [[3.6091113989140098, -4.7014959893833073, 1.4466010351965437, -3.627809370884262, -4.4959025132564525], [1.0298880830512331, -3.2017490605316756, 3.6969046470631799, -3.8543194316066351, 3.2873897704653103], [3.1985661494405537, 4.8412754404031304, -4.3040933946315842, 0.90301303534340072, 2.9257753090890484], [-1.6044345453560114, -1.2120831593517813, -0.77869561353045214, -4.3046309271992156, 1.0382422250756314]], [[3.4745033452321632, 1.2339732346426491, 4.3916008955701535, 2.7520988144655583, 0.71953887440781106], [-1.9595201165263774, -2.0250952672052644, -3.0929608114604723, -0.82324119090551306, 4.7848107412420493], [1.4157697666233213, -3.7391641848200683, 0.48207357742377255, -4.6210442638387894, 0.29349311524207966], [4.8209411906319524, -3.1387480262733392, 4.2131074148961183, 3.7757415029332027, 3.4089445314276183]], [[0.13254399033168696, 3.2227052255215707, -3.2458040844019198, 3.4764289833419415, -2.2176702121773904], [2.96624518329509, 4.1720454626010834, -4.0495362440632663, -4.0408087343376318, -0.097335728542829791], [2.3569916589026487, -0.40274381110957869, 2.941344766566182, 0.68897621522433639, -1.0798160827469605], [-0.75081669884895597, 3.727599044184867, -1.6272568302893164, 1.3143739297534962, 4.2287168893363933]]]])+(1.-msk_arg0)*numpy.array([[[[-3.4243316254994047, -2.2740450349289976, 2.182087070203651, 4.7844780934571318, 2.2719441910885525], [-0.72868923305190236, 3.8817378440457979, -1.2317927101443127, 0.19916809612310882, -1.1919393869099992], [-4.5081713844413382, 1.5116511317158192, -3.6810852344496601, -1.0065672345437116, 1.5158011143425654], [0.85353102972550854, 1.5367176559123852, 0.6897075916232609, 0.18706355870112734, -4.9710384793722397]], [[-3.6218822226360423, -0.08761380875155389, -2.0048679668931371, 1.8080491613440994, -4.0653789785077796], [3.0201430182398674, 4.9471008892387776, 2.5069962635085483, 0.45667701201881528, -3.1202525222290554], [4.5727863428869302, 3.923214012611794, -3.2800453843372948, -4.5205963638310678, 3.3663686255933118], [2.0170010304362576, 0.92333937964082846, -1.5770404499227855, 1.263344650942761, -4.0855836108995405]], [[-1.3168373887681648, 0.87737197827310709, -2.7078451074723011, -1.0230693455836493, -1.9767036114105982], [-2.6344986692463057, -1.6545867767706355, 1.688772902912933, -2.0121230630803408, -2.4347536333867739], [-1.3883833935907197, -2.2595544685553017, 1.0003487825348278, 4.1475455608156846, 1.1847149768679399], [-4.6311268570516209, -3.4936400428174439, -3.9695675223736817, 0.12078235541209992, 0.75646097428439329]], [[2.6805767392575213, -2.6638306557708971, -0.36572395344535824, 0.64741145332061567, -0.91803974333064886], [-4.3725465560444636, 0.77190589021862177, 0.1263128679614951, -4.8126582099329482, -4.5164956161934473], [4.4792815454287087, 1.4089169768797083, -0.15082769667298734, -4.0629465017372439, -3.468418058888374], [0.82341128347231507, -4.3450343117893828, -3.1146893238178484, -4.6148839212986097, -3.5536608047110207]], [[0.881049537013471, -0.77141485889796613, 1.2264925701568643, 2.9561939010832825, 4.4956241837297668], [-0.90832065029377773, 3.1849723870298057, 4.6959107321005451, -0.32679440172442309, -3.2242948659062751], [-2.8893290568531382, -0.40844156788060815, 1.0142066972330879, 4.0679767205858202, -2.3867443147639635], [-4.851895302535759, 3.778195908508895, -3.1500006534635783, -4.9106345516260328, -3.5344343565558467]]], [[[4.2928177863197394, 1.1576900771025063, 2.863014038337286, 2.9773736252110838, 4.0745739424869125], [3.0940599089918503, -4.9586239785272213, -4.2306051129726026, 1.7602881073077778, 1.8192858626500339], [-2.5768234180657079, -0.0060861716872926408, -4.5737549335238441, -4.998029765595696, -1.5686297338522492], [2.2867697774743974, 3.9019256000929019, 4.457578712718087, -3.0437433542252035, -2.0513258273297783]], [[4.590017568467772, 1.5074559640614762, 3.2999575755896426, 0.46273044560269572, 4.6559585447842657], [2.2921905587936253, 2.4172655847499724, 2.0089307499976048, 1.337595944812044, 3.0189965728440509], [3.9083461251688156, 1.7370721862723473, 4.6663195519757537, 4.7908995556832839, -3.0719726528773972], [-4.2395869595602917, -0.88011731011140704, -3.4170824917318043, 1.2637836324065868, 0.15341561628211942]], [[2.8828159564456755, -2.8591984876150578, 0.98874591324947403, 0.36368651149473497, 2.242059019270453], [4.8758713590026268, -3.6954663193108797, -2.9229404335350164, 4.9011907368120049, -3.2576238128900572], [4.47408089733708, 4.3648955821418856, -4.8580270853035534, -2.5006536396969126, 2.0971801090504272], [-0.50727824000400368, 1.8646753748898206, -1.9416024623871255, -4.7057889960038448, 1.1357243361064109]], [[3.4193963592656704, 4.0133354300797901, 3.759324119765445, 1.9203427457762521, -2.6988392916943882], [-3.2134968503273189, -0.49220880199016825, -0.64367078686710233, -3.5990809514195767, 3.0611527379648837], [0.27612433464088149, 0.75639990232008891, 3.6211717361068683, -1.9372880729244835, -1.5146422678198777], [-3.5176385282099876, 4.5718153789675959, -3.1747650655098547, 1.6904298835119658, -2.0303474272859567]], [[-2.3596515570448728, 3.1233305107946912, 0.85531768230235006, -3.4598762605877118, 2.6940951719433324], [4.5924121542648049, -2.1772700272765801, 1.6722744618554746, -0.75232768144352846, -2.8946668158768576], [1.0194417294106763, -4.1925637492655765, -4.4024589790839519, -2.0659743208722126, -1.8870030949826844], [1.3043716005014172, 1.8776525546556586, 3.5043375351855293, -2.8889493272069644, -4.2878037237077287]]], [[[-2.7855743809221769, -2.1323649882320628, -2.7279001549574655, 3.7928376078792034, 4.142814077042658], [3.3453448765933587, -1.1816607673974744, -0.72967429316686516, -0.91494039457883058, 2.1113992533505899], [2.6511923165835523, 1.456352631300514, 4.1440037726889827, 0.12964596875318168, -0.14964563084280602], [-0.48473812101243574, -2.4624301022655635, 0.89379496440865402, 3.969236082398373, 2.0928888662638929]], [[4.7018056186805754, -1.5926240064610298, -3.3950766745040886, -3.4723997262104667, -3.4165986911516777], [2.9652532824263318, -0.11687163529858768, -4.2524998713471138, 2.6396491327499874, -0.93603107951138931], [-4.0076817548068338, 1.0737397796377763, -0.99259756118315678, -1.1614252289451663, -0.60058395148233146], [1.4076990517629229, -0.69812069404827604, -3.862952706883588, 4.0007217792347767, -3.9793843503518445]], [[-2.8724999271666354, 3.7454354469643434, 2.0248666648970568, 0.64830614748008308, 4.0737355510261324], [1.4524306482866649, 1.3521590437390074, -0.37062293481019637, -3.5662638999155938, -3.8385114697857836], [4.1132101682167299, 2.3179362914534192, 1.045660340657756, 1.917445648990511, -0.0051664685703300961], [-3.4270198086428261, 1.0546437261140715, -3.6733376105023217, 0.74379286120358223, 2.4104726424587675]], [[-0.52115029669409818, -1.4888634567089056, -4.9924037562868753, 1.0025288998364337, -0.27666054869917023], [2.1782259671235193, -3.470924984138982, 1.9130405946321627, 2.1526872296237922, 3.0147942535377563], [1.7173922809810414, 0.102394624209313, 3.570991789424264, 0.73919103749804904, -0.099291758197121105], [0.62617163274173127, 4.7384784043842085, -2.1224314854624895, 4.6457889215137538, 4.686670292027447]], [[-4.5187376150299299, -2.3201851199962595, -4.6149408359637309, 4.2235752028443745, -2.337972251223098], [4.990758392735767, 3.5622431952173557, 1.7738149874719076, -0.61395004515682228, 3.9233003472827015], [-1.2410839977695498, 1.3207874913584146, -2.4466142564788473, 4.4724073629579149, -0.39842832829680397], [1.0076750058668988, -2.7354449245071533, 4.714017064029667, 3.3514572302205146, -4.4926495504691166]]], [[[0.24889683727267098, 1.808805325284796, -4.2376982535592402, -4.1410452181324429, -0.70893995655192299], [4.1623644896693897, 4.8913406205320271, -0.91699558347473342, -4.3512286923782426, 2.5835637158322307], [-0.48454958614188559, 1.7869550361615207, 4.4308693945934134, -4.2516037496538841, -2.3681993220049593], [2.0545924363766002, 0.34233482523060221, 4.8811962924146339, 4.7612963596892506, -2.3774372431935342]], [[-1.4088693844200195, -3.4187087725766552, -1.6397602141090495, -0.31896034409287477, 1.6457360415114515], [-0.73240138622122419, -2.6878807401777403, -4.7361487891695484, -1.4155777227272104, 3.0807273997455482], [-1.8902202937715327, 0.064886104329723082, -0.23779899514889635, -1.9887669061865463, -1.2806083062841846], [-1.2519729319461712, -2.7738830266675651, 2.8772726854217687, 3.5886399104373403, 0.69663316517290319]], [[3.3743710492187571, -2.4203222877229482, 0.27185990956179396, 0.063814005637333082, -2.4541982834877429], [-3.699910459072302, 2.4286909647705777, -1.1028919136552329, 1.1334205138884847, 0.84837229280475945], [-4.2540330595761322, 4.2410140372132403, -4.2436809552685482, -1.7512468189472221, -1.4212548045165629], [-4.9202691392232856, -2.4402279793645096, 3.3071025958481997, -4.654536433056883, 0.34671523166253859]], [[3.1834660291161399, 1.267668131642238, 3.8576249822681703, 0.8264356220349045, -1.0026227017583733], [-4.4432005798548122, 3.2803521110130536, -3.606664124084241, -1.2992608942315966, 1.4711131292030908], [-0.85202752045153218, 3.4410676856385507, -0.49820477097770155, -3.293144652996113, 0.87756860968175054], [3.4215663725207541, -3.6250175848156418, -3.1547003420521347, -3.6033014241555614, 0.38353850413958757]], [[-1.4211969056272267, -2.473111312008538, -2.359238481021686, -0.20759729078699696, 0.13355108114654879], [4.7003861010543702, -2.0172065826694019, -2.9542818313048649, -0.36676368267350767, 2.3485226524375165], [-0.15594465064786078, 3.0050870753467933, 2.4939441495087422, 2.4640552793177068, 4.8575279070854958], [-3.0780290519984033, -4.1294868143645189, -3.8909610771697545, 0.22938572553452108, -4.1932139322667119]]]])
      arg1=Data(numpy.array([[0.20123811872623598, -4.4321350187326498, -3.955050677975275, -1.087950358491959, 0.42466893493708113], [-2.4107494666867559, 1.3339669042216027, 3.8299125986025633, 0.71839122696461288, -1.5794074908999809], [4.9056501262505563, -0.49038232567399032, 4.6474586738960539, 4.7876425643934368, 0.081650138515440673], [1.420335034280642, -3.0074670850826237, 0.41056124595367915, 0.39056527536145946, 2.6053942119220164]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.8878046786653151, 1.0806020821076006, 2.6401423058036988, -3.6418498367019243, -1.2951316493792429], [4.693497469856565, 2.8368105795601561, -0.60247395141611193, -0.54141897502939074, 4.7358701210024012], [2.6404031770961662, 4.6462815127229415, -3.0669249137342511, -1.4143011134790369, 2.7241105647580248], [1.9489641378274181, 4.3910263766054722, -2.2224463862962738, 2.376201392709592, 0.58307252277056154]], [[-1.8514556018578672, -7.3206656388972178, -8.773677595027344, -9.4082370533937087, -6.0792246128740359], [-8.2283759383030386, -2.9115466979528506, -8.2721925259347042, -1.6484392521804923, -5.344723909207663], [-6.5728054202733102, -8.5863258191850953, -4.9361469943831153, -8.0404795833229095, -9.0612291003222776], [-6.1681280100309852, -7.2082942873997764, -2.0193762472548489, -2.0911826498346668, -4.7831242737240149]], [[-2.5313533370927681, 0.29260789129335318, -3.980165596889333, -0.60891131530778697, -8.7193006224666014], [-3.6174322721493555, -1.0651043280155674, -2.8183624477079787, -3.6280501841055024, -1.2641864372407303], [-8.9001893496682634, -8.801923454015153, -0.44201194173274683, -6.9946688134703141, -2.2117464203639292], [-0.38215627474886116, -2.2501929748446257, -2.2450537444637026, 0.99956785088042555, -8.6401814231179319]], [[-4.7539148410450522, 2.7742561572579554, -3.4671228317881715, -1.1463626604334962, 2.3531502435499974], [-3.7443615798144902, 2.489133827338037, -3.4358427440624726, -0.49495648314746354, -2.9573974160922818], [-1.668793087970613, 2.9739248260357378, -3.7980573405261282, -1.4788520521523436, 1.0891209816191605], [1.2493394260257231, -1.5645128817503302, -0.2633058435480935, -3.2887612239265396, 3.5212365152701928]], [[-3.1789582729067893, 3.4949990634328003, -3.0513322675835957, 0.68369664908448602, -2.8554651947091685], [-1.2584414889781907, -2.7535543960828779, 0.97851210096346719, 4.7523531962452701, -3.8131668937266099], [3.5911843790808753, 0.037521793235868017, 4.3780989515611761, 1.8687277745950777, -4.3394611974545043], [-2.4716337803068913, 1.4715396223989892, -4.0104282362261712, 0.74805760234218699, -1.5068444857560737]]], [[[0.88592274169711871, -0.15586229068470736, -5.574953965884613, -1.3656160350815725, -3.3626170986174975], [1.1850826534222758, 1.3965338198852759, -1.2625716419913076, -6.0953684465307081, -3.5900759271853389], [-7.2605361774905397, -2.9413034222866354, -6.2516374619190458, -7.1274100083131842, -4.8023578241234635], [-2.0909546767207772, -4.9030319135575979, -1.5322804457083921, 1.6048581710276757, 0.9566910717621262]], [[-3.64374031621987, -2.3757598442700161, 5.2739795045198576, -1.6415446010441119, 1.4841146351934675], [-0.59100279508637055, 4.7159052625950597, 3.7080327204348658, -1.3446239799433739, 4.8262220407399568], [5.6472530390839957, -0.16217661568334174, -2.2730663594956226, 0.19311992598549388, -1.0010303668426195], [-2.558306582551058, 4.9257680834797686, 2.392943358863989, 2.3772911402208647, 6.1188507130050418]], [[4.3038917543086104, 4.28275152118945, -0.59685559382401365, 8.1925452245274109, 5.1306912494007308], [3.9030085847858382, 0.6652439501558276, 5.5710453433889597, 5.5982164786163731, 2.7509764244881367], [-0.63286171127434798, 8.7477882381870096, 6.0470044419865969, 6.1188014236677128, -0.020816321475536803], [1.9218020130453191, 2.2832979992175488, 3.9693596707955061, 4.2385734307549567, 8.5695889084093189]], [[2.2512939149843607, 2.1834043072961595, -0.54935364485614446, -3.2455391232033231, 0.40737695156423293], [3.8215731253278022, -2.8177583582580938, -2.4045617145245268, 1.190638067461764, -1.1565201632576452], [1.2433230654755834, 0.015902361939856391, 1.5219449225919766, 3.3997150282945894, -3.6986884607321118], [1.2671888223351981, -4.1639477729508734, -2.6224378410516413, -0.015301974425877773, 0.73377039930525534]], [[-0.47458883046231115, 2.1577232976563048, -4.5306807842442485, 2.6882186427980335, -4.9008873387481477], [-3.6929830066997971, -0.24567693602705809, -5.9172557052567516, -0.58371999730260526, -5.8606649315595849], [0.18083916504687814, 2.16827780470362, -2.7079552766010226, -3.0373162718132205, -2.3852255929456323], [-3.6676050126630808, -3.8184577183551824, -5.63192660404723, -3.4095346930732413, -1.6600353999390203]]], [[[2.1269536108256575, 1.4737434861882095, 1.882496349209914, 6.8344063024273112, 6.6971428151781227], [2.7356469802252095, 1.0268246770757345, 7.8564940151222906, 8.5656961134679275, 1.1556386319096261], [5.3339833405820389, 4.1758498329052109, 7.6703308155606305, 4.5070456679228119, 2.4398181978524098], [9.2168499677053592, 7.9432252188756785, 1.4214151284577161, 7.09241808093625, 0.0047621557683772409]], [[-1.3214209009092679, 2.8812666538418723, -4.8974488178587281, -2.1976089523611564, -0.3458343868391589], [-2.1040838889092162, -1.8565286952715585, -4.7970595179651481, -2.2571100098907659, -0.36438150498291577], [0.63951962215828306, -3.2617048843233509, -0.81040768729522927, -3.0475328952733758, 4.0501225654527273], [2.2245198838658622, -0.47876573865695526, 1.94636700658429, -0.24841869803972649, -1.0247822309027788]], [[2.4488691877590893, 8.1387490295431668, 8.3508300501013046, 2.0289289867665943, -0.21377790056689339], [1.7182316630956223, 0.81078156722913697, 7.9134740159508983, -0.075888443999700073, 8.5058519432037283], [3.166254459403933, 3.3952108690603096, 5.0107916456886041, 3.9068909281446311, 6.8805331561727616], [2.3397594468445853, 8.9540296533688561, 7.0122878958402817, 7.9554456011376509, 4.616621954938867]], [[2.3333111572658933, 1.0949212543679252, 4.0208344447610234, 6.0919303173835599, 0.076657091683891565], [8.6634289612627136, 0.6108883412850945, 6.4401912450563685, 1.8142350404766434, 2.408161889575346], [6.4189703539768166, 5.2874019260367815, 7.0614965795810631, 3.1561298296814795, 3.1270543436905758], [2.7259184160748804, 7.3794281079422595, 0.90130982331274634, 9.0374330719469302, 1.0344157013704121]], [[-1.7405417244885562, -0.80877188878855666, 2.4256193125805989, -2.2352942027742486, 3.7618019402857197], [3.6286807129234386, 0.084767830336832439, 2.9844504178488531, -2.8275545736526215, -0.96604706120951533], [-1.5309404036053014, 3.3999093959005382, 4.660606595498578, 1.8773031414638108, -2.435080729511228], [-2.9340287603048516, 1.7987032550506159, 2.9616931339534842, -1.0376171998314812, 1.4559131142967372]]], [[[4.2651984238980063, -2.7096008912973648, -3.1765796922656753, 2.1784809966663952, -3.3581449173964213], [6.0845093847945435, 5.6559036980990687, 1.5893436200695596, 1.0841606568152873, -3.2912549171825383], [5.0078136067357271, 0.44007136854895279, 4.1127330963553455, 4.6700248077165316, -0.033072381811281026], [1.6515151090749676, -3.1516815751095617, 4.7626362504501323, 6.1998542849792564, -0.9191671453435637]], [[-0.0051882294411953467, -2.8135345455227547, -3.926916199879698, -7.1159186012596658, -5.3526446765490965], [-4.1162384066485576, -4.8296435563032709, -2.1613148693794537, 0.36653266924472838, -7.4974215211603266], [1.622797616998028, -7.2420368259494561, 0.2680926541652866, 0.87517216342504511, -1.8400105534934799], [-3.4946689196338889, -4.9752887802892083, -7.299693494507757, -7.1106992940535632, -0.20280893506148079]], [[4.0196726448676889, -4.2909347434296281, 1.8571622811502229, -3.2172481249305829, -4.0853412673027734], [1.4404493290049123, -2.7911878145779965, 4.107465893016859, -3.443758185652956, 3.6979510164189895], [3.6091273953942329, 5.2518366863568096, -3.8935321486779051, 1.3135742812970799, 3.3363365550427275], [-1.1938732994023322, -0.80152191339810219, -0.36813436757677298, -3.8940696812455364, 1.4488034710293105]], [[3.8650686205936227, 1.6245385100041085, 4.782166170931613, 3.1426640898270177, 1.1101041497692705], [-1.5689548411649179, -1.634529991843805, -2.7023955360990128, -0.4326759155440536, 5.1753760166035088], [1.8063350419847808, -3.3485989094586088, 0.87263885278523201, -4.2304789884773299, 0.68405839060353912], [5.2115064659934118, -2.7481827509118797, 4.6036726902575777, 4.1663067782946621, 3.7995098067890778]], [[2.7379382022537033, 5.828099437443587, -0.64040987247990344, 6.0818231952639579, 0.38772399974462601], [5.5716393952171064, 6.7774396745230998, -1.4441420321412499, -1.4354145224156154, 2.5080584833791866], [4.9623858708246651, 2.2026504008124377, 5.5467389784881984, 3.2943704271463528, 1.5255781291750559], [1.8545775130730604, 6.3329932561068834, 0.97813738163269992, 3.9197681416755126, 6.8341111012584097]]]])+(1.-msk_ref)*numpy.array([[[[-3.2230935067731687, -2.0728069162027616, 2.383325188929887, 4.9857162121833678, 2.4731823098147885], [-0.52745111432566638, 4.0829759627720339, -1.0305545914180767, 0.4004062148493448, -0.99070126818376325], [-4.3069332657151023, 1.7128892504420552, -3.4798471157234241, -0.80532911581747557, 1.7170392330688014], [1.0547691484517445, 1.7379557746386212, 0.89094571034949688, 0.38830167742736332, -4.7698003606460038]], [[-8.0540172413686921, -4.5197488274842037, -6.4370029856257869, -2.6240858573885504, -8.4975139972404286], [-1.4119920004927824, 0.51496587050612774, -1.9251387552241015, -3.9754580067138345, -7.5523875409617052], [0.14065132415428039, -0.50892100612085578, -7.7121804030699446, -8.9527313825637176, -1.065766393139338], [-2.4151339882963923, -3.5087956390918214, -6.0091754686554353, -3.1687903677898888, -8.5177186296321903]], [[-5.2718880667434398, -3.0776786997021679, -6.6628957854475761, -4.9781200235589242, -5.9317542893858732], [-6.5895493472215811, -5.6096374547459105, -2.266277775062342, -5.9671737410556158, -6.3898043113620488], [-5.3434340715659943, -6.2146051465305767, -2.9547018954404471, 0.19249488284040961, -2.7703357011073351], [-8.586177535026895, -7.4486907207927189, -7.9246182003489567, -3.8342683225631751, -3.1985897036908817]], [[1.5926263807655623, -3.751781014262856, -1.4536743119373172, -0.44053890517134331, -2.0059901018226078], [-5.4604969145364226, -0.31604446827333721, -0.96163749053046388, -5.9006085684249072, -5.6044459746854063], [3.3913311869367497, 0.32096661838774931, -1.2387780551649463, -5.1508968602292029, -4.5563684173803329], [-0.26453907501964391, -5.4329846702813418, -4.2026396823098073, -5.7028342797905687, -4.6416111632029793]], [[1.3057184719505521, -0.34674592396088499, 1.6511615050939454, 3.3808628360203636, 4.9202931186668479], [-0.4836517153566966, 3.6096413219668868, 5.1205796670376262, 0.097874533212658044, -2.799625930969194], [-2.4646601219160571, 0.016227367056472985, 1.438875632170169, 4.4926456555229013, -1.9620753798268824], [-4.4272263675986778, 4.2028648434459761, -2.7253317185264971, -4.4859656166889517, -3.1097654216187656]]], [[[1.8820683196329835, -1.2530593895842497, 0.45226457165053002, 0.5666241585243279, 1.6638244758001566], [0.68331044230509441, -7.3693734452139772, -6.6413545796593585, -0.65046135937897809, -0.59146360403672205], [-4.9875728847524634, -2.4168356383740486, -6.9845044002106, -7.4087792322824519, -3.9793792005390052], [-0.12397968921235858, 1.491176133406146, 2.0468292460313311, -5.454492820911959, -4.4620752940165342]], [[5.9239844726893747, 2.8414228682830789, 4.6339244798112453, 1.7966973498242984, 5.9899254490058684], [3.626157463015228, 3.7512324889715751, 3.3428976542192075, 2.6715628490336467, 4.3529634770656536], [5.2423130293904183, 3.07103909049395, 6.0002864561973563, 6.1248664599048865, -1.7380057486557945], [-2.9056200553386891, 0.45384959411019565, -2.0831155875102016, 2.5977505366281894, 1.4873825205037221]], [[6.7127285550482387, 0.97071411098750549, 4.8186585118520373, 4.1935991100972982, 6.0719716178730163], [8.7057839576051901, 0.13444627929168362, 0.90697216506754685, 8.7311033354145682, 0.57228878571250608], [8.3039934959396433, 8.1948081807444488, -1.0281144867009901, 1.3292589589056507, 5.9270927076529905], [3.3226343585985596, 5.6945879734923839, 1.8883101362154378, -0.87587639740128154, 4.9656369347089742]], [[4.1377875862302833, 4.731726657044403, 4.4777153467300579, 2.638733972740865, -1.9804480647297753], [-2.495105623362706, 0.22618242497444463, 0.074720440097510554, -2.8806897244549639, 3.7795439649294966], [0.99451556160549437, 1.4747911292847018, 4.3395629630714811, -1.2188968459598706, -0.79625104085526477], [-2.7992473012453747, 5.2902066059322088, -2.4563738385452418, 2.4088211104765787, -1.3119562003213439]], [[-3.9390590479448537, 1.5439230198947103, -0.72408980859763084, -5.0392837514876927, 1.1146876810433515], [3.013004663364824, -3.756677518176561, 0.092866970955493677, -2.3317351723435094, -4.4740743067768385], [-0.55996576148930455, -5.7719712401655574, -5.9818664699839328, -3.6453818117721934, -3.4664105858826653], [-0.27503589039856369, 0.29824506375567772, 1.9249300442855484, -4.4683568181069457, -5.8672112146077096]]], [[[2.1200757453283794, 2.7732851380184935, 2.1777499712930908, 8.6984877341297597, 9.0484642032932143], [8.250995002843915, 3.7239893588530819, 4.1759758330836911, 3.9907097316717257, 7.0170493796011462], [7.5568424428341086, 6.3620027575510703, 9.049653898939539, 5.035296095003738, 4.7560044954077503], [4.4209120052381206, 2.4432200239849928, 5.7994450906592103, 8.8748862086489293, 6.9985389925144492]], [[4.2114232930065851, -2.0830063321350201, -3.8854590001780789, -3.962782051884457, -3.906981016825668], [2.4748709567523415, -0.607253960972578, -4.7428821970211041, 2.149266807075997, -1.4264134051853796], [-4.4980640804808241, 0.58335745396378602, -1.4829798868571471, -1.6518075546191566, -1.0909662771563218], [0.91731672608893255, -1.1885030197222664, -4.3533350325575784, 3.5103394535607864, -4.4697666760258343]], [[1.7749587467294186, 8.3928941208603973, 6.6723253387931107, 5.295764821376137, 8.7211942249221863], [6.0998893221827188, 5.9996177176350614, 4.2768357390858576, 1.0811947739804602, 0.80894720411027032], [8.7606688421127838, 6.9653949653494731, 5.69311901455381, 6.5649043228865649, 4.6422922053257238], [1.2204388652532279, 5.7021024000101255, 0.97412106339373228, 5.3912515350996362, 7.0579313163548214]], [[4.2664922676993386, 3.2987791076845312, -0.20476119189343844, 5.7901714642298705, 4.5109820156942666], [6.9658685315169562, 1.3167175802544548, 6.7006831590255995, 6.940329794017229, 7.8024368179311931], [6.5050348453744782, 4.8900371886027498, 8.3586343538177008, 5.5268336018914859, 4.6883508061963157], [5.4138141971351681, 9.5261209687776454, 2.6652110789309473, 9.4334314859071906, 9.4743128564208838]], [[-4.4370874765144892, -2.2385349814808189, -4.5332906974482903, 4.3052253413598152, -2.2563221127076574], [5.0724085312512077, 3.6438933337327963, 1.8554651259873483, -0.5322999066413816, 4.0049504857981422], [-1.1594338592541091, 1.4024376298738552, -2.3649641179634067, 4.5540575014733555, -0.3167781897813633], [1.0893251443823395, -2.6537947859917126, 4.7956672025451077, 3.4331073687359552, -4.410999411953676]]], [[[1.669231871553313, 3.229140359565438, -2.8173632192785982, -2.7207101838518009, 0.71139507772871902], [5.5826995239500317, 6.3116756548126691, 0.50333945080590858, -2.9308936580976006, 4.0038987501128727], [0.93578544813875641, 3.2072900704421627, 5.8512044288740555, -2.8312687153732421, -0.9478642877243173], [3.4749274706572422, 1.7626698595112442, 6.3015313266952759, 6.1816313939698926, -0.95710220891289222]], [[-4.4163364695026432, -6.4261758576592793, -4.6472272991916732, -3.3264274291754985, -1.3617310435711723], [-3.7398684713038479, -5.695347825260364, -7.7436158742521721, -4.4230448078098341, 0.073260314662924486], [-4.8976873788541564, -2.9425809807529006, -3.2452660802315201, -4.99623399126917, -4.2880753913668084], [-4.2594400170287949, -5.7813501117501893, -0.130194399660855, 0.58117282535471659, -2.3108339199097205]], [[3.7849322951724362, -2.0097610417692691, 0.68242115551547311, 0.47437525159101224, -2.0436370375340638], [-3.2893492131186228, 2.8392522107242568, -0.69233066770155371, 1.5439817598421639, 1.2589335387584386], [-3.843471813622453, 4.6515752831669195, -3.833119709314869, -1.340685572993543, -1.0106935585628838], [-4.5097078932696064, -2.0296667334108305, 3.7176638418018788, -4.2439751871032039, 0.75727647761621775]], [[3.5740313044775993, 1.6582334070036975, 4.2481902576296298, 1.217000897396364, -0.61205742639691385], [-4.0526353044933527, 3.670917386374513, -3.2160988487227815, -0.90869561887013717, 1.8616784045645502], [-0.46146224509007272, 3.8316329610000102, -0.10763949561624209, -2.9025793776346536, 1.26813388504321], [3.8121316478822136, -3.2344523094541824, -2.7641350666906752, -3.212736148794102, 0.77410377950104703]], [[1.1841973062947897, 0.13228289991347841, 0.24615573090033038, 2.3977969211350194, 2.7389452930685652], [7.3057803129763865, 0.58818762925261447, -0.34888761938284851, 2.2386305292485087, 4.9539168643595328], [2.4494495612741556, 5.6104812872688097, 5.0993383614307586, 5.0694494912397232, 7.4629221190075121], [-0.4726348400763869, -1.5240926024425026, -1.2855668652477381, 2.8347799374565374, -1.5878197203446955]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.8862207568896574, 3.5489515358885271, 0.54889578728363997, 1.3627694365075325, 0.29216341725234596], [0.15592458644549279, -1.8489892363307572, -4.1949701228841612, 3.3738061307456064, -3.8323944820649114], [-4.4997522784382404, -2.492151975857527, -2.9915227494226104, -4.5210218092833889, 3.2318018146138101], [-2.5007756215513419, -2.5474413836323482, -0.68828554843315981, 0.72726052838003064, 3.9749776033543647]])+(1.-msk_arg0)*numpy.array([[2.6094974002212536, 3.0511054675313432, 2.7127807901798908, -3.4521371295533965, -3.1604431085686189], [1.4103140382009887, 0.12604357748649342, -1.6397235400593257, 0.91765096459056483, -0.33157667028438098], [-4.5295722789044319, -4.4136020303277359, 0.85250583019635418, -0.59693523399197446, -0.86263420851712258], [2.7947775957974228, -4.9647644407288727, -1.6148438785585038, -0.57541032011560489, -1.3277058567775435]])
      arg1=Data(numpy.array([[[[4.6038885427458425, 3.8488784122848987, -0.38799413012926554, -4.7384937620740715, -1.1599873213183818], [-3.1236497386682762, -2.7108853674256315, -1.7978102094250392, -2.9483599555979678, -1.68983958205318], [-4.825699695828952, 3.9639146160156784, -2.9368206779325821, -4.2797314560653978, 3.088945570864734], [4.1123768895775559, -3.4312678356158077, 3.4752550035694814, 2.8594289326258, -0.06881606242090399]], [[1.8192234138610051, 3.8006352024288184, 4.3205958481716209, 1.0576980460057959, -1.0644253302356077], [3.4319263415598744, -1.916680733913394, -2.2992262394744225, 2.7360065018355693, -0.12186590660889962], [-1.4459441972280307, -3.6407862955420911, -0.20661099405501204, -3.0332279659070438, 4.4027789636343631], [0.19053126368554629, -3.3571522189470904, -3.084688125375262, -4.5060419559059053, 0.61106627232668664]], [[3.5092437493350435, -0.024242918306407901, -4.6949773884392156, -4.1261339358506604, -2.4048198474166904], [-4.3708712945246262, 4.2383874119619698, 2.6908028540710349, 4.3373343888493849, -4.8713933708138875], [-3.6602722875137039, 2.0050248459040612, 1.4410926328460931, -1.6453029169692268, 4.1167346388972685], [-0.80386486301102789, 0.51853867018602973, -1.2759824923830996, 4.8983198581969258, -1.3724650372756511]], [[-1.1694148505393276, 0.85245130991041584, 1.6923352465453787, 1.4946406292614176, 0.28684153960661085], [0.74816900929967112, 1.239241948272614, -2.310119902303478, -2.7449414247594905, -2.3361780808806532], [-4.4277102160309489, -4.8437728262769078, -1.3473082447538842, 1.7767114215523696, -4.7882352865953068], [0.98263201612064943, -0.62388505091688984, 2.0044614237297784, 0.23512894992351896, 0.14389148856160627]], [[-0.86942265570533017, 2.0186166776835455, 2.9348450502716359, -4.8205008459731067, 2.9517418565513998], [-1.7469679450258626, -4.354209411513339, -1.0470687026595393, 3.9738313976496755, 2.390802390241805], [-3.2949870732921638, 3.8806315619297909, 2.0585114755959264, 4.3069783029164128, 3.1986330346755896], [-1.5066291649547461, 1.8619577164825385, -0.66931271327851505, 4.2405694098292219, 1.750343349118082]]], [[[-2.8752637951439155, 0.44710076113075292, -1.0034625993681887, 0.084651802987573355, -4.1078829315215195], [-1.2278091563798066, 4.8521981498340701, -3.6126518585268039, -2.859969145434742, -1.7184414386612303], [2.9731602418846688, 3.3748719862109962, -0.88423735653738511, 0.8591438508031235, -2.3997945457075822], [1.8469035193183174, -2.4563740134864807, -0.85917497538143195, -1.2351810053690269, 0.52103171165969275]], [[-2.9852032343124755, 3.6798499069187933, 4.3822394698216325, -0.35402811871465456, -4.1446769555135123], [-4.342171223713013, 0.76133306534860345, -3.609091104703761, 2.9220386946577648, 4.7406021868867505], [3.7275024127976017, 1.1613464015057575, 2.6186390361418503, 2.9101348451644702, -4.2338116705253164], [-2.1451342787172898, -1.0927403707115002, -2.3043513537161351, 2.6360002465320118, -0.48236622841028609]], [[1.9961010693973149, -0.15848079596539311, 1.4501778723298298, 0.8293272950481354, 0.27960796261322507], [-3.8065828893046461, -1.3496871634306307, -0.24089051016569307, 4.140885446908305, 2.0954035817397854], [-2.3055628107599913, -3.359776444991124, -2.523725610570573, 4.3313341106249652, 4.8413615205047407], [2.7668129215086434, 0.76726604239619256, -0.63627736803118218, -2.1114218293289735, 2.8643466528657777]], [[-0.38452965270903494, 2.8629586291288369, 3.1887007283313213, -3.7257435856750565, -0.91136613652164833], [2.9860641256251306, -2.3545950477723441, -2.5090866090380501, -3.3214383958419589, -4.3050055293187679], [4.9240817069895009, 3.1236420462941474, -2.3545688834154643, 2.874727421629224, 2.147950108828435], [3.187046711269339, 0.832492155936853, 3.6926795158381118, 2.1897055673741885, 3.0455508621321634]], [[1.255052183566673, 0.31947766528966781, -4.2488369655313418, 4.2201669532120629, -0.71480903312630151], [4.4895370360781008, -1.6110608960098762, 1.7766380234304542, -4.4935335431764125, -2.5941371761703458], [-3.5787170123908418, 1.4053872927956768, -0.035714831365667443, 4.8102605587349601, 0.29513236018044253], [1.5710767167979469, -3.1176044472140587, 3.6739250120613836, -1.3192693813312353, 0.19676133172640142]]], [[[-4.5544120116476687, -2.79907044218516, 2.7308213058577238, 3.8859260967509659, -1.7980364134846925], [-0.35427797289429375, -4.3038148790213073, -2.0382788904811044, 4.0175653369768014, 4.9385483236209353], [-2.4726213667475285, -2.7161505573656655, 1.0749443101213938, -2.6782226944159158, 0.28506001073239062], [1.0223165730084425, -0.73223243077668343, 2.32471920976635, -0.10046933243371026, -1.0725659007673425]], [[3.9290425479714877, 2.6158343631337519, -2.2499042109930087, -2.6283425046076268, -4.3972573172269254], [-4.3457847078622116, 1.4007185546797043, 1.3628652508823382, -3.7498141493673289, 2.3452973300416255], [0.014427685273962432, -3.0653993974325333, -3.985603446150412, 0.60651024639581497, -0.656429186647431], [-3.7226735830752533, -4.9514515808215522, -3.4145226049670474, 3.6979519663535516, 1.6079695848008235]], [[4.7638761886129881, -3.5253182349439327, 1.5858054152940984, 4.8831035052735423, -1.787177412803115], [-4.9642896820495954, 2.5263306273790285, 1.6803496902157731, 2.4480477972431105, -4.7250731985129999], [1.871260948052063, -4.2251319631136592, 2.0625532465724463, -4.9883565713804128, -1.8514977344012373], [-4.3758172287599484, -0.52173908067269714, 3.4992555892020309, -2.2597500419674157, 3.2612113234532174]], [[3.2622317114384778, 4.7952556687634136, 2.8222792170795072, -0.33674280437674753, -0.83237316101202552], [-4.491664983179577, -1.4095425018847365, -2.6013916839451365, 2.378101611564488, -0.92468632171510379], [0.31780473120478447, -3.284553148075593, 0.55427732953920472, 1.6010759353525694, -0.72593622675943514], [-1.8079678271207964, 3.3065502687509323, -2.4999283825291596, -3.8007861784401156, 0.63734584161054819]], [[-4.8797980831941921, -1.229342275680787, 3.5512137370779104, -0.73285103200450141, 0.7078196097038667], [-2.6309262325870622, -1.7039868836458005, 3.7344703161558552, 3.6760347258174004, 4.2101697164085046], [4.0724542562505626, 2.6227677557895444, -2.5246333583246727, -0.64699140904766939, -1.596353333037448], [-4.9130127773282819, 0.55520220360387196, -3.252656312072959, 4.5356195807616952, -4.6549880299340032]]], [[[-4.200695630852942, -2.1483551059009431, 2.6891859009870265, -0.25036195956911111, 2.405435059177579], [4.8471118056203704, 1.3244659691609515, -0.88245254852245747, 2.4909611339837054, -1.8735269270512269], [4.7978180413588927, 1.7827681442578154, 4.2984831795099883, 1.852909397967486, 3.4759130731659553], [-0.53324261177114707, -0.6981924936496684, 3.8150071834977535, 2.3563224488338284, -4.1590840949321182]], [[2.6686494862107368, 2.2312587549077296, -1.3963827849539037, -3.8378530241178179, -0.82288473096045678], [-2.6147335526395521, -4.4537880655833453, -3.0176509157571676, -3.1877203851737343, -4.097176881099049], [1.3953710951453582, 0.5158515089793605, 2.9056377945239644, 1.9406401690033377, -3.6894562699876454], [-0.52729762320448703, 2.6955961850124543, -4.5522759887975592, -2.7928505857446062, -3.7162802499154459]], [[2.4744540938688306, 0.18735365015362415, -1.517483098142669, -1.5127058535889617, 4.4186705749353816], [-2.5263506525146062, -2.3787963051040895, -1.7044638873119835, -4.8167669939949063, 0.22182327248874945], [0.18451383703579438, -2.3340221932268768, 1.5803404986643557, 3.3593680662453753, -3.6185354614244192], [4.4606571371083987, 0.16841997770290895, -2.1986938082235499, -1.5380742380167565, 0.93105872866336536]], [[4.8806483891290693, 4.7050478245236054, -3.3654854620518773, -1.2695153183162402, 0.42744352038582534], [3.2397083835856861, 2.7191251905813649, 0.75433743979591128, -3.5287526686615909, 2.2696441163755399], [0.89382615735300597, 3.0227697309645958, -1.9003846076942734, -4.9070696946076309, -3.3920245777052802], [2.8219934383369853, 1.0202971700607488, -2.0521424083812345, -4.5444685582973419, 4.4810001973702285]], [[0.65802121154611903, -0.2042669871116658, 4.5868577431527982, -3.7584648728490668, 4.5903887973523378], [-4.1370679543809246, 2.3888121791082835, 2.9360095344963959, 2.2674372100680484, -4.7232312828190732], [-4.6665858910049787, 1.3326926121522034, 2.2484378553968254, 0.70213819102260189, 1.9521157363957098], [-0.54829772206397109, -0.7848905807938289, -3.6421537822386729, -0.98279569071616368, 0.77306175611711225]]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.4901092996354999, 5.7350991691745561, 1.4982266267603919, -2.8522730051844141, 0.72623343557127562], [-1.2374289817786188, -0.82466461053597406, 0.088410547464618183, -1.0621391987083104, 0.19638117483647743], [-2.9394789389392946, 5.8501353729053358, -1.0505999210429247, -2.3935106991757404, 4.9751663277543914], [5.9985976464672133, -1.5450470787261503, 5.3614757604591388, 4.7456496895154574, 1.8174046944687534]], [[5.3681749497495321, 7.3495867383173454, 7.869547384060148, 4.6066495818943229, 2.4845262056529194], [6.9808778774484015, 1.6322708019751331, 1.2497252964141046, 6.2849580377240963, 3.4270856292796275], [2.1030073386604964, -0.091834759653564024, 3.342340541833515, 0.51572356998148328, 7.9517304995228901], [3.7394827995740734, 0.1917993169414367, 0.46426341051326503, -0.95709042001737821, 4.1600178082152137]], [[4.0581395366186834, 0.52465286897723207, -4.1460816011555757, -3.5772381485670204, -1.8559240601330504], [-3.8219755072409862, 4.7872831992456097, 3.2396986413546749, 4.8862301761330249, -4.3224975835302475], [-3.111376500230064, 2.5539206331877011, 1.9899884201297331, -1.0964071296855868, 4.6656304261809085], [-0.25496907572738792, 1.0674344574696697, -0.72708670509945961, 5.4472156454805658, -0.82356924999201109]], [[0.19335458596820487, 2.2152207464179483, 3.0551046830529112, 2.8574100657689501, 1.6496109761141433], [2.1109384458072036, 2.6020113847801465, -0.94735046579594551, -1.3821719882519581, -0.97340864437312069], [-3.0649407795234165, -3.4810033897693753, 0.01546119175364824, 3.1394808580599021, -3.4254658500877744], [2.3454014526281819, 0.73888438559064262, 3.3672308602373109, 1.5978983864310514, 1.5066609250691387]], [[-0.57725923845298421, 2.3107800949358914, 3.2270084675239818, -4.5283374287207607, 3.2439052738037457], [-1.4548045277735167, -4.062045994260993, -0.75490528540719337, 4.2659948149020215, 2.6829658074941509], [-3.0028236560398178, 4.1727949791821368, 2.3506748928482724, 4.5991417201687588, 3.4907964519279355], [-1.2144657477024001, 2.1541211337348845, -0.37714929602616909, 4.5327328270815679, 2.0425067663704279]]], [[[-2.7193392086984227, 0.60302534757624571, -0.84753801292269593, 0.24057638943306614, -3.9519583450760267], [-1.0718845699343138, 5.0081227362795628, -3.4567272720813111, -2.7040445589892492, -1.5625168522157376], [3.1290848283301615, 3.5307965726564889, -0.72831277009189233, 1.0150684372486163, -2.2438699592620894], [2.0028281057638102, -2.3004494270409879, -0.70325038893593916, -1.0792564189235341, 0.67695629810518554]], [[-4.8341924706432327, 1.8308606705880361, 2.5332502334908753, -2.2030173550454117, -5.9936661918442695], [-6.1911604600437702, -1.0876561709821537, -5.4580803410345187, 1.0730494583270076, 2.8916129505559933], [1.8785131764668446, -0.68764283482499966, 0.76964979981109316, 1.061145608833713, -6.0828009068560736], [-3.994123515048047, -2.9417296070422574, -4.1533405900468923, 0.78701101020125463, -2.3313554647410433]], [[-2.1988690534868462, -4.3534509188495543, -2.7447922505543314, -3.3656428278360258, -3.9153621602709361], [-8.0015530121888077, -5.5446572863147914, -4.4358606330498542, -0.0540846759758562, -2.0995665411443758], [-6.500532933644152, -7.5547465678752852, -6.7186957334547337, 0.13636398774080405, 0.64639139762057951], [-1.4281572013755177, -3.4277040804879686, -4.8312474909153433, -6.3063919522131346, -1.3306234700183834]], [[2.9892764780365715, 6.2367647598744433, 6.5625068590769278, -0.35193745492945006, 2.4624399942239581], [6.359870256370737, 1.0192110829732624, 0.86471952170755628, 0.052367734903647545, -0.93119939857316147], [8.2978878377351073, 6.4974481770397539, 1.0192372473301421, 6.2485335523748304, 5.5217562395740414], [6.5608528420149455, 4.2062982866824594, 7.0664856465837182, 5.563511698119795, 6.4193569928777698]], [[-2.5773422984982384, -3.5129168167752436, -8.0812314475962523, 0.38777247114715152, -4.5472035151912129], [0.65714255401318944, -5.4434553780747876, -2.0557564586344572, -8.3259280252413248, -6.4265316582352572], [-7.4111114944557528, -2.4270071892692346, -3.8681093134305788, 0.97786607667004866, -3.5372621218844689], [-2.2613177652669645, -6.9499989292789701, -0.15846947000352785, -5.1516638633961467, -3.63563315033851]]], [[[-9.0541642900859092, -7.2988227206234004, -1.7689309725805167, -0.61382618168727454, -6.2977886919229329], [-4.8540302513325342, -8.8035671574595469, -6.5380311689193444, -0.48218694146143903, 0.43879604518269488], [-6.972373645185769, -7.2159028358039059, -3.4248079683168466, -7.1779749728541562, -4.2146922677058498], [-3.477435705429798, -5.2319847092149239, -2.1750330686718904, -4.6002216108719507, -5.5723181792055829]], [[1.4368905721139607, 0.12368238727622494, -4.7420561868505358, -5.1204944804651538, -6.8894092930844524], [-6.8379366837197386, -1.0914334211778227, -1.1292867249751888, -6.2419661252248559, -0.14685464581590146], [-2.4777242905835646, -5.5575513732900603, -6.477755422007939, -1.885641729461712, -3.148581162504958], [-6.2148255589327803, -7.4436035566790792, -5.9066745808245749, 1.2057999904960246, -0.88418239105670349]], [[1.7723534391903777, -6.5168409843665431, -1.4057173341285121, 1.8915807558509319, -4.778700162225725], [-7.9558124314722054, -0.46519212204358196, -1.3111730592068374, -0.54347495217949993, -7.7165959479356108], [-1.1202618013705474, -7.2166547125362701, -0.92896950285016411, -7.9798793208030236, -4.8430204838238478], [-7.3673399781825584, -3.5132618300953076, 0.50773283977942052, -5.2512727913900257, 0.26968857403060698]], [[-1.2587900978449111, 0.27423385948002466, -1.6987425922038817, -4.8577646136601365, -5.3533949702954144], [-9.012686792462965, -5.9305643111681254, -7.1224134932285255, -2.142920197718901, -5.4457081309984927], [-4.2032170780786045, -7.8055749573589814, -3.9667444797441842, -2.9199458739308195, -5.2469580360428241], [-6.3289896364041853, -1.2144715405324567, -7.020950191812549, -8.3218079877235045, -3.8836759676728407]], [[-1.647996268580382, 2.0024595389330231, 6.7830155516917205, 2.4989507826093087, 3.9396214243176768], [0.60087558202674796, 1.5278149309680096, 6.9662721307696653, 6.9078365404312105, 7.4419715310223147], [7.3042560708643727, 5.8545695704033545, 0.70716845628913738, 2.5848104055661407, 1.6354484815763621], [-1.6812109627144718, 3.7870040182176821, -0.020854497459148913, 7.7674213953755054, -1.4231862153201931]]], [[[-6.7014712524042839, -4.6491307274522846, 0.18841027943568456, -2.751137581120453, -0.095340562373762872], [2.3463361840690284, -1.1763096523903904, -3.3832281700737994, -0.0098144875676364762, -4.3743025486025688], [2.2970424198075508, -0.71800747729352654, 1.7977075579586463, -0.64786622358385593, 0.9751374516146134], [-3.034018233322489, -3.1989681152010103, 1.3142315619464116, -0.14445317271751357, -6.6598597164834601]], [[0.12120810257838865, -0.31618262872461855, -3.9438241685862518, -6.3852944077501661, -3.370326114592805], [-5.1621749362719003, -7.0012294492156935, -5.5650922993895158, -5.7351617688060825, -6.6446182647313972], [-1.15207028848699, -2.0315898746529877, 0.35819641089161625, -0.60680121462901049, -6.2368976536199936], [-3.0747390068368352, 0.14815480138010617, -7.0997173724299074, -5.3402919693769544, -6.2637216335477941]], [[1.7861685454356708, -0.50093189827953566, -2.2057686465758288, -2.2009914020221215, 3.7303850265022218], [-3.214636200947766, -3.0670818535372493, -2.3927494357451433, -5.5050525424280661, -0.46646227594441037], [-0.50377171139736543, -3.0223077416600366, 0.89205495023119585, 2.6710825178122155, -4.306821009857579], [3.7723715886752389, -0.51986557073025086, -2.8869793566567097, -2.2263597864499163, 0.24277318023020555]], [[5.6079089175090999, 5.432308352903636, -2.6382249336718466, -0.54225478993620957, 1.154704048765856], [3.9669689119657168, 3.4463857189613956, 1.4815979681759419, -2.8014921402815602, 2.9969046447555705], [1.6210866857330366, 3.7500302593446264, -1.1731240793142428, -4.1798091662276002, -2.6647640493252496], [3.549253966717016, 1.7475576984407795, -1.3248818800012039, -3.8172080299173112, 5.2082607257502591]], [[4.6329988149004837, 3.7707106162426989, 8.5618353465071628, 0.21651273050529785, 8.5653664007067025], [-0.16209035102655989, 6.3637897824626481, 6.9109871378507606, 6.2424148134224131, -0.74825367946470855], [-0.691608287650614, 5.307670215506568, 6.2234154587511901, 4.6771157943769666, 5.9270933397500745], [3.4266798812903936, 3.1900870225605358, 0.33282382111569175, 2.992181912638201, 4.7480393594714769]]]])+(1.-msk_ref)*numpy.array([[[[7.2133859429670961, 6.4583758125061523, 2.2215032700919881, -2.1289963618528178, 1.4495100789028719], [-0.51415233844702257, -0.10138796720437782, 0.81168719079621443, -0.33886255537671417, 0.91965781816807368], [-2.2162022956076983, 6.5734120162369321, -0.32732327771132841, -1.6702340558441442, 5.6984429710859876], [6.7218742897988095, -0.82177043539455408, 6.084752403790735, 5.4689263328470537, 2.5406813378003497]], [[4.8703288813923482, 6.8517406699601615, 7.3717013157029641, 4.108803513537139, 1.9866801372957354], [6.4830318090912176, 1.1344247336179492, 0.75187922805692065, 5.7871119693669124, 2.9292395609224435], [1.6051612703033125, -0.58968082801074795, 2.8444944734763311, 0.017877501624299352, 7.4538844311657062], [3.2416367312168894, -0.30604675141574722, -0.033582657843918895, -1.4549364883745621, 3.6621717398580298]], [[6.2220245395149343, 2.6885378718734829, -1.9821965982593248, -1.4133531456707695, 0.30796094276320041], [-1.6580905043447354, 6.9511682021418606, 5.4035836442509257, 7.0501151790292758, -2.1586125806339966], [-0.9474914973338131, 4.717805636083952, 4.1538734230259839, 1.067477873210664, 6.8295154290771594], [1.9089159271688629, 3.2313194603659205, 1.4367982977967912, 7.6111006483768167, 1.3403157529042398]], [[-4.6215519800927236, -2.5996858196429806, -1.7598018830080178, -1.9574965002919789, -3.1652955899467856], [-2.7039681202537253, -2.2128951812807824, -5.7622570318568744, -6.1970785543128866, -5.7883152104340496], [-7.879847345584345, -8.2959099558303038, -4.7994453743072807, -1.6754257080010269, -8.2403724161487037], [-2.469505113432747, -4.0760221804702859, -1.447675705823618, -3.2170081796298775, -3.3082456409917902]], [[-4.029865764273949, -1.1418264308850734, -0.22559805829698298, -7.9809439545417256, -0.20870125201721912], [-4.9074110535944815, -7.5146525200819578, -4.2075118112281586, 0.81338828908105665, -0.76964071832681391], [-6.4554301818607822, 0.72018845336117199, -1.1019316329726925, 1.1465351943477939, 0.038189926106970695], [-4.667072273523365, -1.2984853920860804, -3.8297558218471339, 1.0801263012606031, -1.4100997594505369]]], [[[-1.4649497569429268, 1.8574147993317416, 0.40685143883279995, 1.494965841188562, -2.6975688933205308], [0.18250488182118207, 6.2625121880350587, -2.2023378203258153, -1.4496551072337533, -0.30812740046024167], [4.3834742800856574, 4.7851860244119848, 0.52607668166360355, 2.2694578890041122, -0.98948050750659355], [3.2572175575193061, -1.046059975285492, 0.55113906281955671, 0.1751330328319618, 1.9313457498606814]], [[-2.8591596568259821, 3.8058934844052867, 4.5082830473081259, -0.22798454122816114, -4.0186333780270189], [-4.2161276462265196, 0.88737664283509687, -3.4830475272172676, 3.0480822721442582, 4.8666457643732439], [3.8535459902840952, 1.2873899789922509, 2.7446826136283438, 3.0361784226509636, -4.107768093038823], [-2.0190907012307964, -0.96669679322500679, -2.1783077762296417, 2.7620438240185052, -0.35632265092379267]], [[0.35637752933798916, -1.7982043360247189, -0.18954566772949599, -0.81039624501119034, -1.3601155774461007], [-5.4463064293639718, -2.9894107034899564, -1.8806140502250188, 2.5011619068489792, 0.45568004168045961], [-3.945286350819317, -4.9994999850504502, -4.1634491506298987, 2.6916105705656395, 3.2016379804454149], [1.1270893814493177, -0.87245749766313319, -2.2760009080905079, -3.7511453693882992, 1.224623112806452]], [[0.53312131188152989, 3.7806095937194018, 4.1063516929218862, -2.8080926210844916, 0.0062848280689165037], [3.9037150902156954, -1.4369440831817792, -1.5914356444474853, -2.403787431251394, -3.3873545647282031], [5.8417326715800657, 4.0412930108847123, -1.4369179188248995, 3.7923783862197888, 3.0656010734189998], [4.1046976758599039, 1.7501431205274178, 4.6103304804286767, 3.1073565319647534, 3.9632018267227282]], [[0.92347551328229205, -0.012099004994713169, -4.5804136358157228, 3.8885902829276819, -1.0463857034106825], [4.1579603657937199, -1.9426375662942572, 1.4450613531460732, -4.8251102134607935, -2.9257138464547268], [-3.9102936826752228, 1.0738106225112958, -0.36729150165004842, 4.4786838884505791, -0.036444310103938449], [1.2395000465135659, -3.4491811174984397, 3.3423483417770026, -1.6508460516156163, -0.13481533855797956]]], [[[-9.0839842905521007, -7.3286427210895919, -1.7987509730467082, -0.64364618215346603, -6.3276086923891244], [-4.8838502517987257, -8.8333871579257384, -6.5678511693855359, -0.51200694192763052, 0.4089760447165034], [-7.0021936456519605, -7.2457228362700974, -3.4546279687830381, -7.2077949733203477, -4.2445122681720413], [-3.5072557058959895, -5.2618047096811154, -2.2048530691380819, -4.6300416113381422, -5.6021381796717744]], [[-0.48455948235624824, -1.797767667193984, -6.6635062413207446, -7.0419445349353627, -8.8108593475546613], [-8.7593867381899475, -3.0128834756480316, -3.0507367794453977, -8.1634161796950657, -2.0683047002861104], [-4.3991743450537735, -7.4790014277602692, -8.3992054764781479, -3.8070917839319209, -5.0700312169751669], [-8.1362756134029901, -9.365053611149289, -7.8281246352947829, -0.71565006397418429, -2.8056324455269124]], [[5.6163820188093423, -2.6728124047475785, 2.4383112454904525, 5.7356093354698965, -0.93467158260676086], [-4.1117838518532412, 3.3788364575753826, 2.5328555204121272, 3.3005536274394647, -3.8725673683166457], [2.7237667782484172, -3.3726261329173051, 2.9150590767688005, -4.1358507411840586, -0.99899190420488315], [-3.5233113985635942, 0.33076674952365703, 4.3517614193983851, -1.4072442117710615, 4.1137171536495716]], [[2.6652964774465033, 4.1983204347714391, 2.2253439830875328, -0.93367803836872199, -1.429308395004], [-5.0886002171715514, -2.006477735876711, -3.198326917937111, 1.7811663775725135, -1.5216215557070782], [-0.27913050278718998, -3.8814883820675674, -0.042657904452769735, 1.004140701360595, -1.3228714607514096], [-2.4049030611127709, 2.7096150347589578, -3.0968636165211341, -4.39772141243209, 0.040410607618573735]], [[-5.7424322917113146, -2.0919764841979096, 2.6885795285607879, -1.595485240521624, -0.15481459881325588], [-3.4935604411041847, -2.5666210921629231, 2.8718361076387326, 2.8134005173002778, 3.347535507891382], [3.20982004773344, 1.7601335472724218, -3.3872675668417953, -1.509625617564792, -2.4589875415545706], [-5.7756469858454045, -0.30743200491325062, -4.1152905205900812, 3.6729853722445727, -5.5176222384511258]]], [[[-1.4059180350555192, 0.64642248989647966, 5.4839634967844493, 2.5444156362283117, 5.2002126549750018], [7.6418894014177932, 4.1192435649583743, 1.9123250472749653, 5.2857387297811282, 0.92125066874619588], [7.5925956371563155, 4.5775457400552382, 7.0932607753074111, 4.6476869937649088, 6.2706906689633781], [2.2615349840262757, 2.0965851021477544, 6.6097847792951763, 5.1511000446312512, -1.3643064991346954]], [[-2.2961149545181359, -2.7335056858211431, -6.3611472256827764, -8.8026174648466906, -5.7876491716893295], [-7.5794979933684248, -9.4185525063122171, -7.9824153564860403, -8.1524848259026079, -9.0619413218279217], [-3.5693933455835145, -4.4489129317495122, -2.0591266462049083, -3.024124271725535, -8.654220710716519], [-5.4920620639333597, -2.2691682557164183, -9.5170404295264319, -7.7576150264734789, -8.6810446906443186]], [[0.85961021531032689, -1.4274902284048796, -3.1323269767011728, -3.1275497321474655, 2.8038266963768779], [-4.1411945310731095, -3.9936401836625932, -3.3193077658704873, -6.43161087255341, -1.3930206060697543], [-1.4303300415227094, -3.9488660717853805, -0.03450337989414809, 1.7445241876868716, -5.2333793399829229], [2.845813258549895, -1.4464239008555948, -3.8135376867820536, -3.1529181165752602, -0.68378514989513839]], [[4.3052380690134644, 4.1296375044080005, -3.9408957821674822, -1.8449256384318451, -0.14796679972977955], [2.6642980634700812, 2.14371487046576, 0.17892711968030639, -4.1041629887771958, 1.694233796259935], [0.31841583723740108, 2.4473594108489909, -2.4757949278098783, -5.4824800147232358, -3.9674348978208851], [2.2465831182213805, 0.44488684994514394, -2.6275527284968394, -5.1198788784129468, 3.9055898772546236]], [[-0.66968464523142446, -1.5319728438892093, 3.2591518863752547, -5.0861707296266108, 3.2626829405747944], [-5.4647738111584676, 1.06110632233074, 1.6083036777188524, 0.93973135329050494, -6.0509371395966163], [-5.9942917477825226, 0.0049867553746598681, 0.9207319986192819, -0.6255676657549416, 0.6244098796181663], [-1.8760035788415146, -2.1125964375713724, -4.9698596390162164, -2.3105015474937072, -0.55464410066043124]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.8191456095278511, -1.9065189873535884], [-3.5944498843021035, 1.7432745203980025]], [[-1.7012881098994903, -0.69933375026896805], [-3.5834116218990308, 0.10123466237612266]], [[0.2212541100002019, -0.79674995804849402], [-4.8190054226409682, 3.8915060835462967]], [[2.2262228931937935, -0.12034351658750087], [-0.76438292882453851, -2.2255316983799265]], [[-1.6863226758727858, 3.8507401035882403], [2.7783953056818618, -0.52054589644028226]], [[-1.5864227961594568, 1.6680228381252071], [3.7454213480970111, -0.38084874416991621]]])+(1.-msk_arg0)*numpy.array([[[-4.7531617272714488, 4.2662122689748685], [3.3906854095821579, -2.9430209369872138]], [[-3.4735791909596636, -2.4329084944358961], [4.1190797376474624, 2.0377984800101059]], [[-4.178456488871416, -1.5906446748243495], [3.2230453460920909, 2.6842781492061993]], [[3.462695517693037, -1.2051861649417406], [-4.5117830351634378, -2.9279753655910001]], [[1.8035948314929451, 0.84295270713309911], [-0.32209808915334559, -1.6863372562646584]], [[-4.8698087196331308, -2.0932514500203778], [-2.0205038905495254, 2.6654288581070382]]])
      arg1=Data(numpy.array([[[-1.5804122543687784, -1.9045334383889911], [0.9474942466648244, -4.6635146623994741]], [[4.7448663321474989, -0.65477493943943443], [-2.4423346583889005, 4.3111312575781735]], [[-1.1112888829439491, -4.8370701063858892], [-4.9363848140445752, -1.7976352967791187]], [[-1.707442286512185, 0.51154843800981364], [-3.3428208763998732, 3.3097557779641225]], [[4.1951738050404757, -4.6228486413736984], [-4.5405816215789372, -3.4572130039960456]], [[-3.2939651830388819, 0.91756248835875542], [-0.00090171658176796399, -2.8534060157133903]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-5.3995578638966295, -3.8110524257425795], [-2.6469556376372791, -2.9202401420014716]], [[3.0435782222480086, -1.3541086897084025], [-6.0257462802879314, 4.4123659199542962]], [[-0.89003477294374722, -5.6338200644343832], [-9.7553902366855425, 2.093870786767178]], [[0.51878060668160852, 0.39120492142231278], [-4.1072038052244118, 1.084224079584196]], [[2.5088511291676898, -0.77210853778545818], [-1.7621863158970754, -3.9777589004363278]], [[-4.8803879791983391, 2.5855853264839626], [3.7445196315152431, -3.2342547598833065]]])+(1.-msk_ref)*numpy.array([[[-6.3335739816402272, 2.3616788305858774], [4.3381796562469823, -7.6065355993866879]], [[1.2712871411878353, -3.0876834338753305], [1.6767450792585619, 6.3489297375882794]], [[-5.2897453718153651, -6.4277147812102386], [-1.7133394679524843, 0.88664285242708063]], [[1.755253231180852, -0.69363772693192693], [-7.8546039115633111, 0.38178041237312232]], [[5.9987686365334207, -3.7798959342405993], [-4.8626797107322828, -5.1435502602607039]], [[-8.1637739026720126, -1.1756889616616224], [-2.0214056071312934, -0.18797715760635203]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.22576226730387106, -4.4428021312597279], [3.6274635986765364, -1.0193671764741241]], [[2.8690056376624398, -4.9858652652249171], [-0.71922027309815739, -3.2129819460578855]]], [[[-1.651830678134055, -1.2109163762177113], [-3.4136671800193552, 1.8196013171602452]], [[1.5164017636845353, -2.0274141908517382], [3.040674888851747, -0.29106520126528945]]], [[[-0.11470349059391616, 3.5051201740481357], [2.5746095934412399, -2.0670165542899852]], [[-3.5359382720363808, 1.8412827777833183], [2.5083914473565088, 4.9446609711299807]]], [[[1.4601641184578789, -4.7372889601285557], [-3.8852531734289588, 2.8617642979951308]], [[0.30503589825738953, 1.9420640908682945], [-4.7623310356926885, 2.5738657338769659]]], [[[4.2612116253050374, -2.2553912560176572], [-1.0472379672285936, 3.8033123117222267]], [[-3.7030074998381766, 2.0266472807430622], [-3.6938239844508045, -0.68289121184310808]]], [[[0.30127607377238608, 4.7555621136640553], [-1.2169748478438471, 3.7149738241840708]], [[2.417426930035389, -2.9647223581400164], [-0.23135720172097329, 4.8725916636291231]]]])+(1.-msk_arg0)*numpy.array([[[[2.0877590115163489, -4.4926242134285319], [-3.7542711842486929, -0.70923342334468487]], [[-4.7875449671766566, 0.78715875699895399], [-3.1093034833345499, 1.4264263465147584]]], [[[-4.9267887441796887, -3.5617456887709587], [-1.8613441291409649, 3.6121823093208967]], [[-1.6576067034390221, -4.9512125711336799], [-1.0352202745527519, 2.1152004629144292]]], [[[3.6965280968603249, -1.7073362601390043], [1.0814163307260705, 2.0948793411574158]], [[-2.9666724383190535, -4.1652936815083645], [-0.50908765148881319, -4.268982988466469]]], [[[3.6993812970448818, -3.7549128789390505], [-3.0860518687359728, -2.2570933974504657]], [[-2.379088822084058, -3.6002350842994724], [-4.7447879407217366, -3.1229075684378635]]], [[[-0.69117164823280852, 2.5825992577845476], [1.3378020982531442, 1.2997210051006576]], [[-2.9467910695225283, -4.5175847982467223], [-3.3639164649056621, -4.6875815278667226]]], [[[-2.5803684650446512, -1.31845730436957], [-1.7868754368574957, 2.7996688009844704]], [[1.2803865955899942, 4.5387526318472613], [-2.9763102589785762, 3.1307315062851639]]]])
      arg1=Data(numpy.array([[[2.3514027225668785, -2.0273806071134812], [-4.1301037344324012, -0.64890738586259022]], [[-1.004025409356549, 0.012180529540507301], [-4.7506487528525643, -2.8300595356048119]], [[-0.39672598654636992, 4.1761726950510205], [3.6844622554393354, -0.62060383549587073]], [[-0.39791964792643952, -3.4265208652795707], [-0.33263054867635589, -4.7901088473261755]], [[4.3280584473610002, 2.4029356741561081], [-2.8264756920253307, 0.91464300932625786]], [[-4.9547059175816806, 2.2781793108009039], [-0.87385040273464654, -1.5710679467145328]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.1256404552630075, -2.0913994086928493], [1.6000829915630552, -3.0467477835876053]], [[-1.2610980967699614, -9.1159689996573192], [-1.3681276589607476, -3.8618893319204757]]], [[[-2.6558560874906041, -2.2149417855742604], [-3.4014866504788479, 1.8317818467007525]], [[-3.234246989168029, -6.7780629437043025], [0.21061535324693503, -3.1211247368701014]]], [[[-0.51142947714028608, 3.1083941875017658], [6.7507822884922604, 2.1091561407610353]], [[0.14852398340295458, 5.5257450332226536], [1.887787611860638, 4.32405713563411]]], [[[1.0622444705314393, -5.1352086080549952], [-7.3117740387085295, -0.5647565672844399]], [[-0.02759465041896636, 1.6094335421919386], [-9.552439883018863, -2.2162431134492095]]], [[[8.5892700726660376, 2.072667191343343], [1.3556977069275145, 6.2062479858783348]], [[-6.5294831918635072, -0.79982841128226845], [-2.7791809751245466, 0.23175179748314978]]], [[[-4.6534298438092945, -0.19914380391762521], [1.0612044629570567, 5.9931531349849747]], [[1.5435765273007425, -3.8385727608746629], [-1.8024251484355061, 3.3015237169145903]]]])+(1.-msk_ref)*numpy.array([[[[4.4391617340832275, -2.1412214908616534], [-5.7816517913621741, -2.7366140304581661]], [[-8.9176487016090569, -3.3429449774334472], [-3.7582108691971401, 0.77751896065216819]]], [[[-5.9308141535362378, -4.5657710981275077], [-1.8491635996004576, 3.624362838861404]], [[-6.4082554562915863, -9.7018613239862432], [-3.8652798101575638, -0.71485907269038274]]], [[[3.299802110313955, -2.1040622466853742], [5.257589025777091, 6.2710520362084363]], [[0.71778981712028189, -0.48083142606902918], [-1.1296914869846839, -4.8895868239623397]]], [[[3.3014616491184423, -4.15283252686549], [-6.5125727340155439, -5.6836142627300363]], [[-2.7117193707604139, -3.9328656329758283], [-9.534896788047913, -7.913016415764039]]], [[[3.6368867991281917, 6.9106577051455478], [3.7407377724092523, 3.7026566792567657]], [[-5.773266761547859, -7.344060490272053], [-2.4492734555794042, -3.7729385185404647]]], [[[-7.5350743826263322, -6.2731632219512505], [0.49130387394340813, 5.0778481117853742]], [[0.4065361928553477, 3.6649022291126148], [-4.5473782056931089, 1.5596635595706312]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.9663958042061198, 4.5842073471551839], [-1.7822113235268455, -2.2862590795401774]], [[-1.053597980161026, 4.3106817552345724], [0.2689324631672676, -2.1122894494950382]], [[-4.5301003903094301, 2.4678375633514937], [-3.7724523489572714, 4.3741146702987272]], [[-4.5873030373001242, 2.81858684798725], [2.4733940467047644, -4.2173531149342791]], [[0.60388208825460588, -0.16795673415511914], [-1.5462652719655434, 3.7851192946150594]], [[1.2038173452999992, -2.9804468996404476], [3.4485550397946483, 2.0887495274872752]]])+(1.-msk_arg0)*numpy.array([[[-4.1967873402844971, -3.4468892185757083], [1.1233651037052717, 1.0645269687691528]], [[3.7451442835185347, -3.0407782387923166], [1.0406857410911652, 4.5128786508485472]], [[-3.628986483308041, -3.9852233758854427], [4.7937647231878948, -2.1139534462672547]], [[3.6673274955624393, 3.2047512273335066], [-1.4576016339644493, -3.5342751873906622]], [[-3.6849146648943099, -3.3315592390912108], [-1.3090659367051041, 4.3565804104134234]], [[-0.32490286756680931, 0.45764762809465598], [-1.5355238431062435, 2.7319917609307156]]])
      arg1=Data(numpy.array([[[[-1.9281960934374687, -2.9251621075439727], [-1.1831514771644134, -0.81698845699397804]], [[1.2183831109739032, -2.5291709671495024], [-2.3971182048958752, -0.69675710423812021]]], [[[3.0561494973303365, -0.91714637985976477], [3.455965071496319, -3.3578455455397549]], [[-2.1480360670320553, -4.0139408292505596], [2.2134122727099754, -1.9992475482469176]]], [[[0.53471469061826937, -4.6820549141450476], [4.6845328480564401, 1.7586643984384809]], [[-0.073116040232416246, -0.46677447688981921], [-2.0652446628568724, 3.1681206450790267]]], [[[0.052813496610001565, -4.682821227607306], [-0.41976713281090383, -1.7286670060087084]], [[0.0095840700548048829, 3.609510238016199], [-0.35787242402115815, -2.2208059684169634]]], [[[-0.2127393639112114, -1.4612974324888182], [-3.9746787471324718, 4.5153303873686763]], [[-0.65995191429137812, 3.8448645164600013], [4.3274532597012509, 3.5295484949808991]]], [[[-0.36000636393814567, -4.8958553296594722], [2.9182257775303357, 2.2814079087024428]], [[-3.504198151805987, -4.2241984118155438], [-2.3072266734100433, -3.3998931939784569]]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0381997107686511, 0.041233696662147157], [3.4010558699907705, 3.7672188901612058]], [[-0.56382821255294235, -4.3113822906763479], [-4.6833772844360526, -2.9830161837782976]]], [[[2.0025515171693105, -1.9707443600207908], [7.7666468267308915, 0.95283620969481753]], [[-1.8791036038647877, -3.745008366083292], [0.10112282321493726, -4.1115369977419558]]], [[[-3.9953856996911608, -9.2121553044544768], [7.1523704114079338, 4.2265019617899746]], [[-3.8455683891896877, -4.2392268258470907], [2.3088700074418549, 7.5422353153777539]]], [[[-4.5344895406901227, -9.2701242649074302], [2.3988197151763462, 1.0899198419785416]], [[2.4829781167595693, 6.0829042847209633], [-4.5752255389554373, -6.438159083351243]]], [[[0.39114272434339448, -0.85741534423421228], [-4.1426354812875914, 4.3473736532135572]], [[-2.2062171862569215, 2.2985992444944578], [8.1125725543163103, 7.3146677895959584]]], [[[0.84381098136185351, -3.6920379843594731], [-0.062221122110111882, -0.69903899093800481]], [[-0.055643112011338758, -0.77564337202089551], [-0.21847714592276812, -1.3111436664911817]]]])+(1.-msk_ref)*numpy.array([[[[-6.1249834337219653, -7.1219494478284702], [-4.6300406957401217, -4.2638776755696863]], [[2.3417482146791748, -1.4058058634442308], [-1.3325912361267225, 0.36776986453103255]]], [[[6.8012937808488712, 2.8279979036587699], [0.41518683270400247, -6.3986237843320719]], [[-1.1073503259408901, -2.9732550881593944], [6.7262909235585226, 2.5136311026016296]]], [[[-3.0942717926897716, -8.3110413974530886], [0.69930947217099737, -2.2265589774469619]], [[4.7206486829554786, 4.3269902462980756], [-4.179198109124127, 1.054167198811772]]], [[[3.7201409921724409, -1.0154937320448667], [2.7849840945226028, 1.4760842213247982]], [[-1.4480175639096444, 2.1519086040517497], [-3.8921476114118203, -5.7550811558076251]]], [[[-3.8976540288055213, -5.1462120973831276], [-7.3062379862236826, 1.1837711482774655]], [[-1.9690178509964822, 2.5357985797548972], [8.6840336701146743, 7.8861289053943224]]], [[[-0.68490923150495497, -5.2207581972262815], [3.3758734056249917, 2.7390555367970988]], [[-5.0397219949122309, -5.7597222549217868], [0.42476508752067232, -0.6679014330477413]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.9149208656774315, 0.80533185044103117, 0.92582997759535157, -2.2243660583870906], [4.7224239052999533, 1.4042853258225634, 2.2229417766314183, -0.077515069131832703], [-0.23527181588520207, -4.2761031709378194, -4.613910262533456, 4.4528634735314334]], [[-1.2667156479126742, 4.7292083128362741, 3.686261944251136, 0.87221026605843122], [3.4805019771206034, 3.1933740943932136, 4.4539302771819376, -2.8068206187925551], [3.1657159871699463, 3.1258053491956055, 4.3529232030668261, -4.9506778044228019]]], [[[1.5277153113829929, 4.7871721581630489, 4.2510371921828742, 4.2306993799328474], [3.7728498077275407, -1.4606561041919077, -1.7473803270697355, -3.7894473215683178], [-3.5875586387642668, 3.4662429771787213, 4.3316740873453146, -2.915151364321249]], [[0.78930619253962586, -0.78297060810118246, -2.1027302066386966, 1.6535461325455056], [-1.2759040918518316, -4.4651642516364189, 0.51441239861241694, -1.9863752765127995], [-2.2802584198620246, 0.1738163648537121, -3.4373622249369538, 0.88402248662089278]]], [[[2.7648660031622709, -2.8776499746752049, -3.6261175679763724, -1.2983803240883773], [-2.1873498187845888, -0.84505048100606661, -3.2395358877552596, 0.78324158251610321], [4.80968898333815, 3.5849375135385539, -2.5726985012201453, 0.071809729035114955]], [[-4.5814970143966551, 3.1124768592189422, 2.6397108842444172, 2.3497413988747029], [-3.4110498040349482, 1.9545806363657725, -1.7377875944574752, -3.4733216871704844], [2.3343542699942219, -4.8606133616274985, -3.9328576467355902, 4.5519996669293565]]]])+(1.-msk_arg0)*numpy.array([[[[0.10719693753490844, 1.0261862691256347, -1.457690444543589, -2.071073502563693], [0.87783282910702631, -3.6171531228961062, -3.6121068817817994, 4.6310370605879818], [0.95861989020743721, -2.6304523232442865, -1.0869078309702198, -4.6085350245886634]], [[4.2148411705208986, -0.23368531190989827, 2.1496743168371966, 2.5792606868574151], [0.059503524981071365, 2.3990152877862858, 4.3698415170058418, -0.22223227691117309], [2.0613882429937913, -3.9918643381915686, 1.7222571817086507, -2.7234608310972233]]], [[[4.8272025795403088, 0.5287357010897491, 4.3546531920019529, -1.361494481043767], [-0.86161506819955846, -2.5980817806566203, 4.0219801780400299, -3.9011050884391665], [-0.35172326635516349, 0.85408377976426753, -2.6128537223364101, -1.1753057260886903]], [[-1.5487442611991318, -0.3634854518595434, 2.1177430996350655, 1.9407274711253111], [-2.720390592219899, 3.0236837925785895, 4.3546716606699505, -4.8617403456544022], [2.9408730458976438, -1.2947869038265001, -3.292576715252892, 3.8332780958913641]]], [[[2.4344470719941, 2.8428056056673991, 1.5846087996010585, 0.22809514803714137], [-2.9672072411024706, 4.8984040393748405, 0.0029651668574910417, 4.568910748987653], [-1.0108120739386353, 2.3558545175580932, -1.845926551624022, -0.25745818284659183]], [[-2.9152065469241606, 0.46648523656521856, -2.2623637318830117, 2.0869200553107872], [2.5709520345451553, 3.0005431923047023, -0.99585834916664417, 2.1821550295237921], [-0.25027508832823919, 3.5944424758348905, -2.9666630910429284, 1.4092598389571167]]]])
      arg1=Data(numpy.array([[[[-4.5029805646935355, -3.5882036401367792, 1.2879246878620174, -2.9635918096717697], [2.610757314312167, -0.14039166347576781, 0.48790095926479093, 4.821522122056809], [1.0142898535695322, -0.44713201494944776, 3.4236056918439424, -3.0377734707063686]], [[-0.088708102944545786, -4.2924139567712576, 3.4328183248363739, 0.51317834101631465], [1.8766075008728711, -4.0963423322646788, -4.9629530146739835, 1.5817930231821089], [-0.22423521483058551, -1.3830326254949155, 0.56575614844303868, -4.932956012601438]]], [[[-4.0495705361971073, 1.795578275211362, 2.4285875177109597, 1.7038315309387002], [-0.056743474783255365, 4.6290488458336085, -1.2492965487933807, -0.21251552046906408], [1.24132540076425, -2.9741314742401856, 4.9797975753736807, -2.742157734362868]], [[2.935349725106196, -1.1975203708388293, 1.0500169493027975, -3.5848094395881969], [1.3209259551812211, 3.6787812668212325, 3.2951152919015829, -3.1975310553245184], [-3.7834719734004283, -2.3330754523257213, 4.2644222808794918, 2.9862136594004438]]], [[[-2.5064828535147554, 4.4921742875326931, -4.005770828684815, 0.76725398482917129], [-1.12611003629566, 2.3277058098808929, -1.8140879663417895, 4.2968147404183128], [1.9342885907641092, 0.89275947676943801, -2.7159728187097434, -4.0198587765630602]], [[3.3534462302370383, 4.8382572612102095, -2.8803103771750926, -2.7419561913643067], [-1.0733372887221124, 0.58409853491173802, 0.83584003850280109, -2.1265475957669935], [0.81761930511136072, -0.17715427429396158, -3.5180353928288066, 2.2850775559697967]]]]),self.functionspace)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.588059699016104, -2.782871789695748, 2.213754665457369, -5.1879578680588603], [7.3331812196121202, 1.2638936623467956, 2.7108427358962093, 4.7440070529249763], [0.77901803768433009, -4.7232351858872672, -1.1903045706895137, 1.4150900028250648]], [[-1.35542375085722, 0.43679435606501649, 7.1190802690875099, 1.3853886070747459], [5.3571094779934745, -0.90296823787146518, -0.50902273749204596, -1.2250275956104462], [2.9414807723393608, 1.74277272370069, 4.9186793515098648, -9.8836338170242399]]], [[[-2.5218552248141144, 6.5827504333744109, 6.6796247098938339, 5.9345309108715476], [3.7161063329442854, 3.1683927416417008, -2.9966768758631162, -4.0019628420373818], [-2.3462332380000168, 0.49211150293853567, 9.3114716627189953, -5.6573090986841166]], [[3.7246559176458218, -1.9804909789400118, -1.052713257335899, -1.9312633070426912], [0.045021863329389511, -0.78638298481518643, 3.8095276905139999, -5.183906331837318], [-6.0637303932624533, -2.1592590874720092, 0.82706005594253806, 3.8702361460213366]]], [[[0.25838314964751552, 1.6145243128574882, -7.6318883966611875, -0.53112633925920605], [-3.3134598550802488, 1.4826553288748263, -5.0536238540970491, 5.080056322934416], [6.7439775741022592, 4.477696990307992, -5.2886713199298887, -3.9480490475279453]], [[-1.2280507841596169, 7.9507341204291517, -0.2405994929306754, -0.39221479248960378], [-4.4843870927570606, 2.5386791712775105, -0.90194755595467413, -5.5998692829374779], [3.1519735751055826, -5.03776763592146, -7.4508930395643969, 6.8370772228991532]]]])+(1.-msk_ref)*numpy.array([[[[-4.395783627158627, -2.5620173710111445, -0.16976575668157157, -5.0346653122354628], [3.4885901434191933, -3.7575447863718741, -3.1242059225170085, 9.4525591826447908], [1.9729097437769694, -3.0775843381937342, 2.3366978608737226, -7.646308495295032]], [[4.1261330675763528, -4.5260992686811559, 5.5824926416735705, 3.0924390278737297], [1.9361110258539425, -1.697327044478393, -0.59311149766814175, 1.3595607462709358], [1.8371530281632058, -5.3748969636864841, 2.2880133301516894, -7.6564168436986613]]], [[[0.77763204334320157, 2.3243139763011111, 6.7832407097129126, 0.34233704989493319], [-0.91835854298281383, 2.0309670651769882, 2.7726836292466492, -4.1136206089082306], [0.88960213440908653, -2.1200476944759181, 2.3669438530372706, -3.9174634604515584]], [[1.3866054639070642, -1.5610058226983727, 3.167760048937863, -1.6440819684628858], [-1.3994646370386779, 6.7024650593998221, 7.6497869525715334, -8.0592714009789201], [-0.84259892750278453, -3.6278623561522214, 0.97184556562659985, 6.819491755291808]]], [[[-0.072035781520655373, 7.3349798932000922, -2.4211620290837566, 0.99534913286631266], [-4.0933172773981301, 7.2261098492557334, -1.8111227994842984, 8.8657254894059658], [0.92347651682547394, 3.2486139943275312, -4.5618993703337658, -4.2773169594096521]], [[0.43823968331287766, 5.3047424977754281, -5.1426741090581043, -0.65503613605351951], [1.4976147458230429, 3.5846417272164404, -0.16001831066384309, 0.055607433756798663], [0.56734421678312152, 3.4172882015409289, -6.4846984838717354, 3.6943373949269134]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.82112464173)+(1.-msk_arg0)*(2.33890509959)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.67990575081)+(1.-msk_arg1)*(-3.35294812942)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(6.50103039254)+(1.-msk_ref)*(-1.01404302983)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.9162195453526429, -3.5554256246538509])+(1.-msk_arg0)*numpy.array([4.1409469533423309, 1.6589589620316101])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.58853260387)+(1.-msk_arg1)*(1.43795940274)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.3276869414838375, -5.1439582285226564])+(1.-msk_ref)*numpy.array([5.5789063560865682, 3.0969183647758474])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.90020509413)+(1.-msk_arg0)*(2.43858755031)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.3893751346821848, 0.58096455750972975])+(1.-msk_arg1)*numpy.array([3.903298753705208, 2.1236321405969774])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-2.5108299594458607, -4.3192405366183158])+(1.-msk_ref)*numpy.array([6.3418863040149631, 4.5622196909067325])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.1061051027561799, 0.44751311981713027, 3.32903670955187, 2.0981160528322151, 0.56845901872828097], [3.3677460888929804, -3.2660498077045874, -4.4229270839726631, 3.5123577200679676, 3.840169747083138], [-1.7881502738976316, -1.1050141879509412, -1.7572054901012404, 0.60872957951766438, 0.96813696171722441], [-2.4547588664820061, -2.5659305024233436, 0.37583767002209445, -4.4736336036387652, 2.8071757597082394]])+(1.-msk_arg0)*numpy.array([[0.077196203617511117, 2.414230190044206, -2.8983170645258873, 3.2047690833426632, -4.279159279961517], [1.2970444266449066, 2.7215858550640082, 1.2254067766232888, 0.72055239381421554, 4.5443298826168679], [3.3925069437214574, -3.3226464154161728, -3.7680244513669892, 1.5759488283809908, 1.4522931756651225], [4.1843443191620828, 0.16288140423364617, -0.89002018408314676, 4.5156765108959167, -2.3463511955028138]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.38898185067)+(1.-msk_arg1)*(-0.611878070303)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.4950869534286948, 4.8364949704896452, 7.7180185602243849, 6.48709790350473, 4.9574408694007959], [7.7567279395654953, 1.1229320429679275, -0.033945233300148203, 7.9013395707404825, 8.2291515977556529], [2.6008315767748833, 3.2839676627215737, 2.6317763605712745, 4.9977114301901793, 5.3571188123897393], [1.9342229841905088, 1.8230513482491713, 4.7648195206946093, -0.084651752966250271, 7.1961576103807543]])+(1.-msk_ref)*numpy.array([[-0.5346818666858697, 1.8023521197408252, -3.5101951348292681, 2.5928910130392824, -4.8910373502648978], [0.68516635634152578, 2.1097077847606274, 0.61352870631990797, 0.10867432351083472, 3.9324518123134871], [2.7806288734180766, -3.9345244857195536, -4.3799025216703704, 0.96407075807761, 0.84041510536174169], [3.572466248858702, -0.44899666606973465, -1.5018982543865276, 3.9037984405925359, -2.9582292658061946]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.0459827435097)+(1.-msk_arg0)*(4.17634294323)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.5420538629107057, -3.0631498817625804, 3.1126308246678676, -3.5144889546175664, -4.1457444810271857], [1.0286925935040543, -0.23640549040666947, -4.9580836427029595, -0.51469630189082149, 3.4014952079223395], [-2.9411495319897774, 4.2279059309828728, -0.49152880876399596, -1.7294168876463978, -2.833772635754622], [-0.98278215492399923, 0.837147049489813, 4.3854350041461778, -0.4882549046199971, -0.16025672403166791]])+(1.-msk_arg1)*numpy.array([[4.6110300340580395, -0.42852519290856517, 4.5920546411084544, 3.7683205035805383, 1.740021039078683], [4.1923818007189748, -3.3327097654536497, 3.1180931110377994, 1.674822463822859, -1.4462543528859695], [-2.3308489087312356, 0.0016618610181398097, 3.21891691823385, 0.82777675290381758, -0.11313244425754121], [-3.1575184094696276, 1.5029518355130609, 4.1934215157639763, 1.2021768175808702, -1.3225166260138179]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-4.5880366064204514, -3.1091326252723261, 3.0666480811581218, -3.5604716981273121, -4.1917272245369315], [0.98270984999430855, -0.28238823391641521, -5.0040663862127053, -0.56067904540056723, 3.3555124644125938], [-2.9871322754995231, 4.1819231874731271, -0.5375115522737417, -1.7753996311561435, -2.8797553792643678], [-1.028764898433745, 0.79116430598006726, 4.3394522606364321, -0.53423764812974284, -0.20623946754141365]])+(1.-msk_ref)*numpy.array([[8.7873729772833649, 3.7478177503167602, 8.7683975843337798, 7.9446634468058637, 5.9163639823040084], [8.3687247439443002, 0.84363317777167568, 7.2944360542631248, 5.8511654070481844, 2.7300885903393559], [1.8454940344940898, 4.1780048042434652, 7.3952598614591754, 5.004119696129143, 4.0632104989677842], [1.0188245337556978, 5.6792947787383863, 8.3697644589893017, 5.3785197608061956, 2.8538263172115075]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.788515446410182, -3.8000613519338424], [2.2882218195134127, 3.2178965686161796]], [[-3.8955910627384571, 4.6414626950084177], [2.1981730476233494, -2.6375684732553362]], [[-3.6772679123344556, 1.5613415708712184], [1.8483014669663325, 0.91029996970495208]], [[0.23170390762127546, -0.57774967245166664], [0.98816712631728176, -2.9851281668383733]], [[1.5784715774264182, 3.1941862586518717], [2.6968524288480182, 1.0560317279029254]], [[4.8556138639616524, -3.9835718042382506], [1.8605869468433598, 4.5379589837574663]]])+(1.-msk_arg0)*numpy.array([[[4.6532857074985685, 0.1688689390281457], [-3.6147290768777571, -4.8458519654623426]], [[1.9846816479169229, -2.74554304188392], [4.3594465565781491, -1.8693617487319001]], [[-2.4269834331265603, 2.2162144428514612], [-1.2599210802587848, 2.7792169874526449]], [[-1.5852592160005443, 0.68190821443926186], [-1.4417970327505736, -3.0763774826351153]], [[0.88872697897684638, 4.2525396478395532], [1.7181003338179854, -2.6292087245097329]], [[-3.7890545708567003, -3.322294679314024], [-4.9720042348653664, -0.37468819699336109]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.65333793698)+(1.-msk_arg1)*(4.81155666735)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-6.4418533833874125, -5.453399288911073], [0.63488388253618222, 1.564558631638949]], [[-5.5489289997156881, 2.9881247580311872], [0.54483511064611889, -4.2909064102325667]], [[-5.3306058493116861, -0.09199636610601214], [0.19496352998910194, -0.74303796727227844]], [[-1.4216340293559551, -2.2310876094288972], [-0.66517081065994876, -4.6384661038156043]], [[-0.074866359550812334, 1.5408483216746411], [1.0435144918707877, -0.59730620907430509]], [[3.2022759269844219, -5.6369097412154812], [0.20724900986612926, 2.8846210467802358]]])+(1.-msk_ref)*numpy.array([[[9.4648423748505017, 4.9804256063800789], [1.1968275904741761, -0.034295298110409433]], [[6.7962383152688561, 2.0660136254680133], [9.1710032239300823, 2.9421949186200331]], [[2.3845732342253729, 7.0277711102033944], [3.5516355870931484, 7.5907736548045781]], [[3.2262974513513889, 5.4934648817911951], [3.3697596346013596, 1.7351791847168179]], [[5.7002836463287796, 9.0640963151914864], [6.5296570011699187, 2.1823479428422004]], [[1.0225020964952329, 1.4892619880379092], [-0.16044756751343314, 4.4368684703585721]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.305048851368)+(1.-msk_arg0)*(0.999775247002)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.381883915667474, 0.70158635933446512], [1.3020658741330582, -1.4910904334516486]], [[-2.9822988094633471, 1.9022934492924151], [1.8923356682194123, 0.34240321002299989]], [[-4.8360929018498942, -4.7873152235125538], [-4.0870930206438141, -0.25984089633958352]], [[-4.4405632102428081, -1.0123180636224802], [0.86981028724213427, -3.7094768343574911]], [[0.083097454754119227, -3.8531582098733619], [3.4736412013567435, -4.8557443311937787]], [[0.08366769226862214, 2.0487905689594683], [4.8687162869462597, -3.7299590488338605]]])+(1.-msk_arg1)*numpy.array([[[3.5495994190951645, 0.64936611300089542], [4.2878662385033763, -4.921917999080712]], [[0.84855290536816774, 4.6456119374567937], [0.3695264507346252, 2.2789665020174947]], [[0.086726395526043554, -3.1644724966260163], [4.2286938959406211, 1.9955898804955226]], [[-2.2820385501154172, -4.3632421293838437], [-4.1452812796993666, -3.4871290885804385]], [[0.057257291184736658, -1.291812335121846], [-4.0900979989942767, -3.9832781007315941]], [[1.9293773365375602, 4.8149865059093138], [4.8376968381840069, -4.5786053444260872]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.6869327670354255, 0.39653750796651366], [0.99701702276510673, -1.7961392848196001]], [[-3.2873476608312986, 1.5972445979244636], [1.5872868168514609, 0.037354358655048436]], [[-5.1411417532178456, -5.0923640748805052], [-4.3921418720117655, -0.56488974770753497]], [[-4.7456120616107595, -1.3173669149904317], [0.56476143587418282, -4.0145256857254425]], [[-0.22195139661383223, -4.1582070612413133], [3.168592349988792, -5.1607931825617301]], [[-0.22138115909932932, 1.7437417175915169], [4.5636674355783082, -4.035007900201812]]])+(1.-msk_ref)*numpy.array([[[4.5493746660972469, 1.6491413600029778], [5.2876414855054588, -3.9221427520786296]], [[1.8483281523702502, 5.6453871844588761], [1.3693016977367076, 3.2787417490195772]], [[1.086501642528126, -2.1646972496239338], [5.2284691429427035, 2.995365127497605]], [[-1.2822633031133348, -3.3634668823817613], [-3.1455060326972841, -2.487353841578356]], [[1.0570325381868191, -0.29203708811976359], [-3.0903227519921943, -2.9835028537295116]], [[2.9291525835396426, 5.8147617529113962], [5.8374720851860893, -3.5788300974240048]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.85403100299997625, -2.4711398979116703, 4.9524793776429288, 2.8638806203920462], [-4.4486003477573464, 1.5662211746304466, -3.5288476257613057, -3.8696842459445224], [-2.0264849445273923, -0.1402240537485504, -4.2213552291441792, 1.4981358237718467]], [[-2.9732525789497921, 2.7827496937928808, 3.1496931939628929, -2.0145730275031037], [3.722073616028176, -0.70250283725854068, 0.49542001148649462, 0.65616617962048096], [2.1942566174449052, -3.1102721654990129, 2.6193324464014269, 0.30930327184564899]]], [[[1.1970358486723667, -3.426665712344994, -2.3341353423783371, 0.064058774201981805], [-4.0552888392678375, 4.0547020661331388, -4.7178349261791039, 0.53986707183190408], [0.31745748233791371, 1.525574306695928, -1.0308810619742212, 0.33128451471230846]], [[0.4812686309573655, -1.3985814284220921, 3.57631548393854, -4.5108634822198077], [-0.4169967676481896, -1.1069204667353061, 4.0142651018644955, -0.73232731250100613], [4.2210163345699598, 1.9930891727606124, -0.67349836850941092, -2.6065009814971782]]], [[[4.835873296903582, 4.6517090552374238, -4.2578417620080131, 4.2927292436978348], [-4.8920163656776587, 3.1317551641523522, -3.3520898088742768, -3.4739757611319355], [-0.3996620878522803, -0.20630027471082002, 4.5352633703252252, -4.5462354850918842]], [[0.65698835280308376, 3.6576771442876481, 1.5862348566768816, -4.735174687613755], [0.30865605615665626, -2.6623003995459338, 0.98244509007864878, -1.4406925128271943], [-3.5991677647071096, -4.8959811943239124, 1.2920729220597362, 1.4248969574414874]]]])+(1.-msk_arg0)*numpy.array([[[[0.904190650291536, 4.2558624527682891, -0.15593135928961477, -2.2955645708808827], [-3.1804008993109258, 3.1886351345698785, -0.69244765466237812, 3.504579790414466], [2.5561554775087618, -2.7737628640938636, 0.18328108782763763, 3.9746399117850721]], [[4.5219907958368619, 4.3211934964812073, -1.7283798979280141, -3.4858685736740416], [0.82088538314224113, -2.2297394760842995, -1.3348708407460386, -3.96919942167405], [1.4951658228986382, -4.7602454577771205, 2.4502895044090343, 2.0587095875264811]]], [[[3.4490793027073838, -1.9025632640301202, -2.4646890078956076, 3.8306776014244761], [4.0174978641807755, 1.5635991394398383, 0.21189039952738575, 4.804389726846825], [0.89661142878953015, -0.721126612946974, -2.868584021174525, -3.270457527445001]], [[-4.7036765508562448, -3.7522348105388215, -3.3798959901951142, 4.3756826692121145], [-3.9024565979697545, 1.1863327664291914, -0.56736828344575052, 3.7370172983852683], [-0.97258515086511999, -3.6378726077991255, 2.2446107358144207, 3.1146613376192214]]], [[[-0.6521227719299123, -3.8710529624279619, -4.9377953132753021, -1.9722586212672768], [-2.6158555560244032, 3.7210070607819912, -1.6407232805517005, -1.9085397876776113], [-2.4667880618357785, -1.6058289624447566, -2.9381521202852867, -0.8596305609676218]], [[3.8060911259076988, 3.9171313623961836, 3.7012252411349706, 0.8686634866199352], [-3.6545045741787643, -3.2397340517014395, -2.7886531900534783, -2.3271455904772163], [-1.7907468221064358, 4.172099300994768, 1.9115363756612647, 0.21550983254348921]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.318429891922)+(1.-msk_arg1)*(4.87844796668)
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.53560111107833297, -2.152710005990027, 5.2709092695645721, 3.1823105123136894], [-4.1301704558357031, 1.8846510665520899, -3.2104177338396624, -3.5512543540228791], [-1.708055052605749, 0.17820583817309288, -3.9029253372225359, 1.81656571569349]], [[-2.6548226870281488, 3.1011795857145241, 3.4681230858845362, -1.6961431355814605], [4.0405035079498193, -0.3840729453368974, 0.8138499034081379, 0.97459607154212424], [2.5126865093665485, -2.7918422735773696, 2.9377623383230702, 0.62773316376729227]]], [[[1.51546574059401, -3.1082358204233507, -2.0157054504566938, 0.38248866612362509], [-3.7368589473461942, 4.3731319580547821, -4.3994050342574607, 0.85829696375354736], [0.63588737425955699, 1.8440041986175713, -0.71245117005257796, 0.64971440663395175]], [[0.79969852287900878, -1.0801515365004488, 3.8947453758601833, -4.1924335902981644], [-0.098566875726546321, -0.7884905748136628, 4.3326949937861388, -0.41389742057936285], [4.5394462264916031, 2.3115190646822557, -0.35506847658776763, -2.288071089575535]]], [[[5.1543031888252253, 4.9701389471590671, -3.9394118700863698, 4.6111591356194781], [-4.5735864737560155, 3.4501850560739955, -3.0336599169526335, -3.1555458692102922], [-0.081232195930637019, 0.11212961721082326, 4.8536932622468685, -4.227805593170241]], [[0.97541824472472705, 3.9761070362092914, 1.9046647485985249, -4.4167447956921118], [0.62708594807829954, -2.3438705076242905, 1.3008749820002921, -1.1222626209055511], [-3.2807378727854664, -4.5775513024022692, 1.6105028139813795, 1.7433268493631306]]]])+(1.-msk_ref)*numpy.array([[[[5.7826386169715072, 9.1343104194482603, 4.7225166073903564, 2.5828833957990884], [1.6980470673690453, 8.0670831012498496, 4.186000312017593, 8.3830277570944371], [7.434603444188733, 2.1046851025861075, 5.0617290545076088, 8.8530878784650433]], [[9.4004387625168331, 9.1996414631611785, 3.1500680687519571, 1.3925793930059296], [5.6993333498222123, 2.6487084905956717, 3.5435771259339326, 0.90924854500592112], [6.3736137895786094, 0.11820250890285067, 7.3287374710890054, 6.9371575542064523]]], [[[8.327527269387355, 2.9758847026498509, 2.4137589587843635, 8.7091255681044473], [8.8959458308607466, 6.4420471061198095, 5.0903383662073569, 9.6828376935267961], [5.7750593954695013, 4.1573213537329972, 2.0098639455054461, 1.6079904392349702]], [[0.17477141582372635, 1.1262131561411497, 1.4985519764848569, 9.2541306358920856], [0.97599136871021663, 6.0647807331091625, 4.3110796832342206, 8.6154652650652395], [3.9058628158148512, 1.2405753588808457, 7.1230587024943919, 7.9931093042991925]]], [[[4.2263251947500589, 1.0073950042520092, -0.059347346595330919, 2.9061893454126944], [2.2625924106555679, 8.5994550274619623, 3.2377246861282707, 2.9699081790023598], [2.4116599048441927, 3.2726190042352146, 1.9402958463946844, 4.0188174057123494]], [[8.68453909258767, 8.7955793290761548, 8.5796732078149418, 5.7471114532999064], [1.2239433925012069, 1.6387139149785317, 2.0897947766264928, 2.5513023762027549], [3.0877011445735354, 9.0505472676747392, 6.7899843423412358, 5.0939577992234604]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.78362224309)+(1.-msk_arg0)*(-1.61996667253)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.58767760232554611, 0.99300818930861645, 3.6000459674435064, 3.1800549353985055], [-3.2742430183133542, -3.0377700793622973, -4.6411015865227867, 3.4569856437234119], [-4.7267775369744003, 4.9528033955928112, -1.3883079106793952, 3.6071482713011598]], [[-3.2293853211290524, 2.9219346522182583, 4.9422989446259926, -4.6009812076007384], [-4.2929989923618175, -2.8921014654616881, 3.2297087556760466, -2.2597070757151947], [-4.95900195413825, -3.619654319682474, 3.2235849165703954, 1.7996035069914971]]], [[[0.029452993218324508, -3.4246305827755021, 1.9984963431937244, -2.6729553114660387], [3.1877067099653615, 0.43123002935100452, -0.56524468815184559, 3.5043144882922377], [-3.145267440049142, -4.8347249982056208, -1.8269903033279888, -2.0253014627670876]], [[-2.4096245628134594, -3.1091599847111437, -3.3292039418657815, -3.9799764861313793], [-3.3156038371776422, 1.3165618852381282, 2.8053190434442463, 1.2781338160004543], [-1.3306243308476127, 4.4678984639607648, 3.3550927990789496, -0.55169489065065136]]], [[[-2.5951656376415091, -0.93825854224432348, -0.92525272819497939, -4.6277410200394433], [0.20798578156093672, 1.9750269280486759, -1.3432361427549813, 2.9139934058485792], [-4.211582731531534, -4.6199823069923651, 2.0043607183412702, 4.9810849740590495]], [[-1.9995011575175914, 3.1420865106281557, -0.35280414557739537, 4.9641720656462152], [-4.4661894924762802, 4.1890851052090365, -2.1777326028582946, 1.5821197979132897], [1.7859752498743511, 4.1509270673587633, -2.5170582224464777, 2.964208681376034]]]])+(1.-msk_arg1)*numpy.array([[[[-2.1974665637117576, -4.2656563110158618, 3.2553663653962541, -3.7764487723440876], [1.7785408668265426, -0.46214122915833045, 2.8362339598904871, -0.48368687084092166], [-1.103431334097229, 1.0905171881276488, -1.1804273871929394, -3.6857307068055203]], [[4.9863715505229038, 3.1184235468340873, 3.6140766875046904, 2.5229975874235633], [3.3693567953333172, 4.8011338702380719, -3.6310481793436358, 2.3365035272858501], [4.7178149644625602, -2.2997617325594266, 4.7152099159595799, 4.224471960065328]]], [[[2.6356613124638786, 2.4440188483973131, 0.97760089754254231, -2.0354422567770039], [-0.52773854959050226, 0.60353069265939752, 4.8999062609040251, 3.1896369161485953], [4.1160864293457458, 1.1572712617805934, -4.4400803874509283, -3.2722447119564779]], [[2.0245914637976092, 4.0900040072903856, 0.013471409341246243, 4.3143340510550665], [1.9562717475535907, 0.94876073356643431, -1.7882239214545734, -1.1179833873476976], [4.2793282285208498, -0.54746466773333147, 4.9732202254883013, -2.9248758111642714]]], [[[-1.6446300966438567, 0.94427913862825541, -2.1767344086206508, -1.8636453147381484], [3.5682406245085136, 1.9491290014621647, 3.5024340286200069, 2.1077791174479739], [3.7227759898004731, -0.016522659436462028, -0.14360990567100096, 3.1232088079460834]], [[-4.5272759389208357, 2.5632431163568672, -0.053050630210041305, 2.6680529866719906], [4.3298457070602563, -3.7632290573065763, -2.181023708654374, 1.2922608240471263], [-4.0621557459908395, -1.682513129007519, 4.506505681893703, -4.9897030804285798]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.1959446407644458, 3.7766304323986084, 6.3836682105334983, 5.9636771784884974], [-0.49062077522336223, -0.25414783627230531, -1.8574793434327947, 6.2406078868134038], [-1.9431552938844083, 7.7364256386828032, 1.3953143324105968, 6.3907705143911517]], [[-0.44576307803906046, 5.7055568953082503, 7.7259211877159846, -1.8173589645107464], [-1.5093767492718255, -0.10847922237169616, 6.0133309987660386, 0.52391516737479726], [-2.175379711048258, -0.83603207659248202, 6.0072071596603873, 4.583225750081489]]], [[[2.8130752363083165, -0.64100833968551019, 4.7821185862837163, 0.11066693162395325], [5.9713289530553535, 3.2148522724409965, 2.2183775549381464, 6.2879367313822296], [-0.36164519695915009, -2.0511027551156289, 0.95663193976200311, 0.7583207803229044]], [[0.37399768027653257, -0.32553774162115179, -0.54558169877578955, -1.1963542430413874], [-0.53198159408765022, 4.1001841283281202, 5.5889412865342383, 4.0617560590904462], [1.4529979122423793, 7.2515207070507568, 6.1387150421689416, 2.2319273524393406]]], [[[0.18845660544848286, 1.8453637008456685, 1.8583695148950126, -1.8441187769494514], [2.9916080246509287, 4.7586491711386678, 1.4403861003350107, 5.6976156489385712], [-1.427960488441542, -1.8363600639023732, 4.7879829614312621, 7.7647072171490414]], [[0.78412108557240057, 5.9257087537181476, 2.4308180975125966, 7.7477943087362071], [-1.6825672493862882, 6.9727073482990285, 0.60588964023169734, 4.3657420410032817], [4.569597492964343, 6.9345493104487552, 0.26656402064351425, 5.7478309244660259]]]])+(1.-msk_ref)*numpy.array([[[[-3.8174332362402374, -5.8856229835443417, 1.6353996928677743, -5.3964154448725674], [0.15857419429806274, -2.0821079016868103, 1.2162672873620073, -2.1036535433694015], [-2.7233980066257089, -0.52944948440083106, -2.8003940597214192, -5.3056973793340001]], [[3.366404877994424, 1.4984568743056075, 1.9941100149762105, 0.90303091489508347], [1.7493901228048374, 3.1811671977095921, -5.2510148518721156, 0.71653685475737028], [3.0978482919340804, -3.9197284050879064, 3.0952432434311001, 2.6045052875368482]]], [[[1.0156946399353988, 0.82405217586883328, -0.6423657749859375, -3.6554089293054837], [-2.1477052221189821, -1.0164359798690823, 3.2799395883755453, 1.5696702436201155], [2.4961197568172659, -0.46269541074788645, -6.0600470599794081, -4.8922113844849573]], [[0.40462479126912942, 2.4700373347619058, -1.6064952631872336, 2.6943673785265867], [0.33630507502511087, -0.6712059389620455, -3.4081905939830532, -2.7379500598761775], [2.65936155599237, -2.1674313402618113, 3.3532535529598215, -4.5448424836927508]]], [[[-3.2645967691723365, -0.6756875339002244, -3.7967010811491306, -3.4836119872666282], [1.9482739519800338, 0.32916232893368491, 1.882467356091527, 0.48781244491949405], [2.1028093172719933, -1.6364893319649418, -1.7635765781994808, 1.5032421354176035]], [[-6.1472426114493155, 0.94327644382838738, -1.6730173027385211, 1.0480863141435108], [2.7098790345317765, -5.3831957298350561, -3.8009903811828538, -0.32770584848135353], [-5.6821224185193193, -3.3024798015359988, 2.8865390093652232, -6.6096697529570596]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.30357805951486494, 1.7789908195938704])+(1.-msk_arg0)*numpy.array([-3.4999084381820147, 2.0687439704233013])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.29560909960201354, 2.8131428474818829])+(1.-msk_arg1)*numpy.array([1.8534134757911067, -2.9570599593966373])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.59918715911687848, 4.5921336670757533])+(1.-msk_ref)*numpy.array([-1.646494962390908, -0.88831598897333608])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.4568452753441861, 4.156182630831589], [-2.7124200704769441, 4.6349254341555373]])+(1.-msk_arg0)*numpy.array([[-1.7192363811809508, -2.689356706339936], [-1.9733217485911227, -2.8956118636844597]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.31883792521880228, -0.68707649682042238])+(1.-msk_arg1)*numpy.array([0.10135550412562733, 3.6436258832935451])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.1380073501253838, 3.8373447056127867], [-3.3994965672973665, 3.9478489373351149]])+(1.-msk_ref)*numpy.array([[-1.6178808770553235, -2.5880012022143086], [1.6703041347024223, 0.74801401960908542]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.8630530636775928, -0.96498609307528849])+(1.-msk_arg0)*numpy.array([-1.759806705651854, 2.7419414177278689])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.4995794951572101, -2.2296495934470508], [4.592731811274323, 1.6481453020444032]])+(1.-msk_arg1)*numpy.array([[-2.1917098149577239, -0.76567191383794153], [3.6307188111362905, -1.5206385965571103]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.36347356852038271, -5.0927026571246437], [3.6277457181990345, 0.68315920896911475]])+(1.-msk_ref)*numpy.array([[-3.9515165206095779, -2.5254786194897956], [6.3726602288641594, 1.2213028211707586]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.9990423025367781, -2.9280100434623701, 4.0118558539292586, -3.8154889856521432, -3.6368981327204111], [-0.50143491065161339, -4.6660244534216497, -2.4753686269335171, 4.3085271200015338, 4.6225403173317297], [-2.2744508217266368, -3.8213760666423413, 4.4705384235183026, 4.4014691749757979, 0.63518326930132485], [-4.9754652302123636, 2.0900165933963795, 2.1358926335217525, 4.0281754747291174, 2.6667190638117351]], [[2.8097247807404653, -2.409550636697162, 4.1484789891218377, 1.0013886187580106, 1.9852256459733306], [0.78033641532092268, 4.5585724539660166, -2.2561553606304505, 0.55536666248500133, 3.8943843106818967], [-2.1169456727879798, -2.1639790574958182, 2.162032470911802, 0.31168477760727953, -1.277687289499323], [0.75934513321347019, 1.5182786622384121, -2.0713486193737807, -3.2445789461435206, 0.64959686252244264]]])+(1.-msk_arg0)*numpy.array([[[0.55610932014025671, 4.0743519140754643, -3.7691779757607446, -1.0122739087677255, -4.2807812858191872], [-3.0797963668708661, -3.3872726708484704, 2.6033221432386844, -3.0239939134595364, -1.0831090525693829], [3.3456675242271405, 0.14762172533032558, -1.414537207926192, 0.26657695296319162, 4.129036288593964], [0.49565095006444437, 1.740798337165832, -1.2237889036169602, -3.8756694494462849, -2.663738689121995]], [[-1.4007475882152418, 0.063304030595166871, -4.6493856875327575, -3.6031336931563152, 0.2562315466572489], [-4.5468806767068735, -4.8651736832937154, -0.1580933023110731, -1.2330376963913503, -0.06070183446802524], [-1.9740953987006193, -4.9837660307508402, -2.7063980447971736, -0.29656070288304015, 4.3927284360829724], [0.89565646467082871, 2.6599132143455639, 1.4690723220271256, 3.9434836834617606, 2.1038271093364358]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.96964103842889227, 2.815611630588446])+(1.-msk_arg1)*numpy.array([-4.2799002717764569, 0.83088160849476367])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-4.0294012641078858, -1.9583690050334779, 4.9814968923581509, -2.845847947223251, -2.6672570942915188], [0.46820612777727888, -3.6963834149927575, -1.5057275885046248, 5.2781681584304261, 5.5921813557606219], [-1.3048097832977446, -2.851735028213449, 5.4401794619471948, 5.3711102134046902, 1.6048243077302171], [-4.0058241917834714, 3.0596576318252717, 3.1055336719506448, 4.9978165131580097, 3.6363601022406273]], [[5.6253364113289113, 0.40606099389128403, 6.9640906197102836, 3.8170002493464565, 4.8008372765617766], [3.5959480459093687, 7.3741840845544626, 0.55945626995799547, 3.3709782930734473, 6.7099959412703427], [0.69866595780046614, 0.65163257309262779, 4.9776441015002479, 3.1272964081957255, 1.537924341089123], [3.5749567638019162, 4.3338902928268581, 0.74426301121466532, -0.42896731555507461, 3.4652084931108886]]])+(1.-msk_ref)*numpy.array([[[-3.7237909516362002, -0.20554835770099267, -8.0490782475372011, -5.2921741805441824, -8.5606815575956432], [-7.359696638647323, -7.6671729426249273, -1.6765781285377725, -7.3038941852359933, -5.3630093243458399], [-0.9342327475493164, -4.1322785464461314, -5.6944374797026489, -4.0133233188132653, -0.15086398318249294], [-3.7842493217120126, -2.5391019346106249, -5.5036891753934167, -8.1555697212227418, -6.9436389608984523]], [[-0.56986597972047814, 0.89418563908993054, -3.8185040790379938, -2.7722520846615515, 1.0871131551520126], [-3.7159990682121098, -4.0342920747989517, 0.67278830618369057, -0.40215608789658663, 0.77017977402673843], [-1.1432137902058557, -4.1528844222560766, -1.8755164363024099, 0.53432090561172352, 5.223610044577736], [1.7265380731655924, 3.4907948228403276, 2.2999539305218892, 4.7743652919565243, 2.9347087178311995]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.4408441786188781, 2.5600825875097977])+(1.-msk_arg0)*numpy.array([-1.6283996798525124, 1.4538498085535778])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.8868132667406989, 2.5913352271516219, -2.175159523417407, 2.1542774449668309, -2.2638430369399112], [-0.27860863512781364, 2.5490211291841849, -3.0722049177109545, -0.58568416629063957, 2.2618822428259779], [-3.2262283987138609, 0.23053249640812723, 2.7311578895487711, 3.5786620611245628, 2.0818880916896063], [3.4274106861775113, -3.0345594255391695, 4.1813394894412639, 1.5986891061543274, -4.833956795224541]], [[2.3215065159563739, 0.58884715467147331, -1.4660078659320384, -3.4155526731086603, 1.5541655055213708], [-1.5292511976801437, 2.7467923287859115, -4.38502653743409, 0.16321552437752551, -4.0410566423702132], [-3.1229191880974252, 2.2786482773664058, 4.413143890023802, 1.4737631037843135, -1.0229179057195434], [2.2572286478098835, 0.96679904556283702, 1.923303823983975, -2.3405535335439911, 1.2742962174319512]]])+(1.-msk_arg1)*numpy.array([[[-0.6351687348233046, 0.29368960576337333, 1.1195683267678636, 2.495735635969023, -2.192466328190239], [1.6351543313835712, -1.5257356040067824, 1.5775407602984242, -1.1437345844775604, -3.9600477566276093], [2.0115425921863439, 2.9232777754041965, -1.1944102723604932, -0.26923258104628722, -1.8631256160161347], [3.5753461496412662, -4.0776973558290042, 0.45295812661010615, -2.5351025030820553, 3.6730838119106402]], [[4.3662374958692105, 2.0221612436317535, -0.95128825494581903, -3.5422150947417395, 1.9412007008699224], [3.1933086037569662, 2.7441646708674918, 4.5198954501946158, 1.7861865996125115, 1.0810416100996036], [0.62957143004364013, 0.78442541434370039, -1.3747122123190625, 0.50356569705550314, 4.0082868374434142], [3.0962047856512989, 1.8077789571785194, 2.1406508574085228, 3.3851052675051001, 1.276242136393658]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.4459690881218208, 0.15049104853274375, -4.6160037020362852, -0.28656673365204721, -4.7046872155587893], [-2.7194528137466918, 0.10817695056530674, -5.5130490963298326, -3.0265283449095177, -0.17896193579290021], [-5.6670725773327391, -2.2103116822107509, 0.290313710929893, 1.1378178825056846, -0.35895608692927183], [0.98656650755863318, -5.4754036041580481, 1.7404953108223857, -0.84215507246455079, -7.2748009738434192]], [[4.8815891034661716, 3.148929742181271, 1.0940747215777593, -0.85547008559886262, 4.1142480930311685], [1.0308313898296539, 5.3068749162957092, -1.8249439499242923, 2.7232981118873232, -1.4809740548604156], [-0.56283660058762752, 4.8387308648762035, 6.9732264775335997, 4.0338456912941112, 1.5371646817902542], [4.8173112353196812, 3.5268816330726347, 4.4833864114937727, 0.21952905396580658, 3.8343788049417489]]])+(1.-msk_ref)*numpy.array([[[-2.263568414675817, -1.3347100740891391, -0.50883135308464889, 0.8673359561165106, -3.8208660080427514], [0.0067546515310588084, -3.1541352838592949, -0.050858919554088278, -2.7721342643300728, -5.5884474364801218], [0.38314291233383146, 1.2948780955516841, -2.8228099522130057, -1.8976322608987997, -3.4915252958686471], [1.9469464697887537, -5.7060970356815162, -1.1754415532424063, -4.1635021829345682, 2.0446841320581277]], [[5.8200873044227883, 3.4760110521853314, 0.50256155360775878, -2.0883652861881616, 3.3950505094235002], [4.6471584123105441, 4.1980144794210696, 5.9737452587481936, 3.2400364081660893, 2.5348914186531815], [2.083421238597218, 2.2382752228972782, 0.079137596234515328, 1.957415505609081, 5.462136645996992], [4.5500545942048767, 3.2616287657320973, 3.5945006659621006, 4.8389550760586779, 2.7300919449472358]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.6415037132883743, 2.2366425853501219], [-0.39047088380503769, -2.4287138539610975]], [[-1.6561784027528894, 3.9864505697901542], [-0.17522227935180901, 3.8150500015126561]], [[0.61725840288372247, -3.4647495352318725], [1.9629487495438003, 1.1408872683441409]], [[2.3930649010319573, 1.7103576225817108], [4.4520994790745529, -3.0713874289028995]], [[4.7784245905369289, 4.343992844123937], [2.4785111769319013, -4.5005179754311895]], [[-1.7453103893257707, -4.6337819791351311], [-0.44007583664022665, 4.2349142198995189]]], [[[4.9749585514345647, 3.6814714972215032], [0.73985936115524886, -4.8294816022108842]], [[3.2424305236854103, -1.8050907046336029], [-4.7959960745764398, 4.3701859037129935]], [[-0.95715319168202395, -1.6705880252138714], [4.0453723282652856, 4.297604306697842]], [[-0.42749897110470947, -3.2329441823966754], [0.0855142224860721, 1.9151106143288139]], [[1.8445074970650808, -0.18929749825558861], [-3.3773168083959271, -4.6660357734504991]], [[-4.2325115006523104, 4.0223275357082162], [3.6026729823688886, -2.2500974968886034]]]])+(1.-msk_arg0)*numpy.array([[[[3.524460454071308, -1.8944207639183244], [-0.73127390841727014, 2.0161901416523342]], [[3.3203380023839717, 4.5971025792905706], [-3.1822058639648523, 4.0739040209158794]], [[1.986598067694505, 0.67270334274742805], [-0.058854517347269208, 2.3748120355024618]], [[4.4628812624205256, -4.3752377859505422], [3.3399641587058699, -0.77742131931078173]], [[-0.9488545741783625, 3.2953895507163278], [2.3614728185054474, 4.9926304099517846]], [[-3.0338533211588894, 2.284259295074083], [3.0708606892433856, -3.3920078697211595]]], [[[-2.7962372820452606, -3.4773291877663102], [3.323197999795557, 1.4269175149102553]], [[-0.63363567504099549, 2.48116121465495], [-1.8676418790572322, -1.7458680871733345]], [[-0.30368451888388925, 0.6930183123452105], [3.9288613531578402, 1.9288017743766712]], [[0.80359149259249829, -4.0960828050389653], [-2.9070293793177293, 1.9559093000733156]], [[-2.5345025442274673, -1.407769793310385], [-1.9926035269070477, 1.1725357945370805]], [[4.166150430459755, -1.2524471159305217], [-2.0126239799264232, 2.4123711997398072]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.28723270518914568, 1.1987648176047436])+(1.-msk_arg1)*numpy.array([4.9516080429830787, 0.50582760635032997])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.3542710080992286, 1.9494098801609763], [-0.67770358899418337, -2.7159465591502432]], [[-1.9434111079420351, 3.6992178646010085], [-0.46245498454095468, 3.5278172963235104]], [[0.33002569769457679, -3.7519822404210181], [1.6757160443546546, 0.85365456315499522]], [[2.1058321958428117, 1.4231249173925651], [4.1648667738854073, -3.3586201340920452]], [[4.4911918853477832, 4.0567601389347914], [2.1912784717427556, -4.7877506806203352]], [[-2.0325430945149163, -4.9210146843242768], [-0.72730854182937232, 3.9476815147103732]]], [[[6.1737233690393083, 4.8802363148262469], [1.9386241787599925, -3.6307167846061406]], [[4.4411953412901539, -0.60632588702885926], [-3.5972312569716962, 5.5689507213177372]], [[0.24161162592271967, -0.47182320760912777], [5.2441371458700292, 5.4963691243025856]], [[0.77126584650003416, -2.0341793647919317], [1.2842790400908157, 3.1138754319335575]], [[3.0432723146698244, 1.009467319349155], [-2.1785519907911834, -3.4672709558457555]], [[-3.0337466830475668, 5.2210923533129598], [4.8014377999736322, -1.0513326792838598]]]])+(1.-msk_ref)*numpy.array([[[[8.4760684970543867, 3.0571872790647543], [4.2203341345658085, 6.9677981846354129]], [[8.2719460453670504, 9.5487106222736493], [1.7694021790182264, 9.0255120638989581]], [[6.9382061106775836, 5.6243113857305067], [4.8927535256358095, 7.3264200784855404]], [[9.4144893054036043, 0.57637025703253641], [8.2915722016889486, 4.1741867236722969]], [[4.0027534688047162, 8.2469975936994064], [7.3130808614885261, 9.9442384529348633]], [[1.9177547218241893, 7.2358673380571616], [8.0224687322264643, 1.5596001732619191]]], [[[-2.2904096756949306, -2.9715015814159802], [3.8290256061458869, 1.9327451212605853]], [[-0.12780806869066552, 2.98698882100528], [-1.3618142727069023, -1.2400404808230046]], [[0.20214308746644072, 1.1988459186955405], [4.4346889595081702, 2.4346293807270012]], [[1.3094190989428283, -3.5902551986886353], [-2.4012017729673993, 2.4617369064236456]], [[-2.0286749378771374, -0.90194218696005501], [-1.4867759205567177, 1.6783634008874104]], [[4.6719780368100849, -0.74661950958019174], [-1.5067963735760932, 2.9181988060901372]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.033937741640214, -2.8439779356530961])+(1.-msk_arg0)*numpy.array([0.85229031221232709, -2.0711636562339439])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.2238599154669991, -2.190840472029417], [1.8892668204159646, 1.4302638947775961]], [[-3.7950753882208166, -4.5739416790969862], [4.4003338127332494, 3.7915900270589713]], [[4.6014129118924743, -0.97030375616703068], [-4.1650572154179164, 2.1011640823584754]], [[2.9993010523172314, 1.8547060641261943], [1.3925352967482727, 0.080716687965670886]], [[0.32245912961547063, 2.4980111290024132], [-4.3212913958762691, 4.2165652677915464]], [[-1.3203177333306639, 0.72565872260362063], [4.4485262805375569, 3.9394851336840073]]], [[[-4.4714153405996946, 3.7834372649763672], [4.5511612942217585, -3.1521102330645157]], [[0.67712152311474227, 1.5157051443343752], [2.2783612192089633, 3.6430301454899823]], [[0.12937606711638949, 2.1396382458355401], [4.8211314999908144, -1.5045539189433188]], [[3.737205586019666, -4.8628072444289527], [-0.25777238603555208, -1.0665315181497528]], [[2.7063386933439961, -3.6008496425997985], [-4.782355848095917, -2.3028263581271347]], [[2.7549018075357692, 0.77176270994271245], [4.5824194869896466, -0.55295886952255913]]]])+(1.-msk_arg1)*numpy.array([[[[-4.7222236376725988, 2.0420182989578084], [-3.6197291361476891, -2.5847072908385238]], [[4.6409869083405582, -3.6477225542281722], [-0.92326349385103157, 2.6473963563489882]], [[-0.98865037277483481, -0.6944666786550151], [-4.2541459444164289, -3.7679076025851588]], [[-3.4232868906562741, -1.2201965957370966], [-1.5382443476634977, -3.1861785304732795]], [[4.0991696297863491, -3.6110376421811319], [1.9823595744929463, -4.8635751844232402]], [[3.1831619880829116, 3.9223577640849356], [-4.3080189891487439, 1.2730407890025202]]], [[[-3.5631363811603189, 2.5039847139393103], [1.1328607842406546, 1.6491291921878144]], [[-1.6830740759514051, 0.45161997872386017], [0.3649293727479499, 2.3591952468697883]], [[1.9028989822811457, -3.7807022818047207], [-4.9390447680428249, -4.3476635699501234]], [[-4.3360977335112461, -2.3631275274311991], [3.978751677601073, 4.1708319082657965]], [[3.4093963330657928, -1.9370596988035294], [4.1269122213140079, -2.290217615553952]], [[-1.5730749183990769, -1.8627880481516268], [1.3939972864311336, -2.5667038711523604]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.1899221738267851, -0.15690273038920299], [3.9232045620561786, 3.4642016364178101]], [[-1.7611376465806026, -2.5400039374567722], [6.4342715543734634, 5.8255277686991853]], [[6.6353506535326883, 1.0636339854731833], [-2.1311194737777024, 4.1351018239986894]], [[5.0332387939574454, 3.8886438057664083], [3.4264730383884867, 2.1146544296058849]], [[2.3563968712556846, 4.5319488706426272], [-2.2873536542360551, 6.2505030094317604]], [[0.71362000830955008, 2.7595964642438346], [6.482464022177771, 5.9734228753242213]]], [[[-7.3153932762527907, 0.93945932932327114], [1.7071833585686624, -5.9960881687176117]], [[-2.1668564125383538, -1.3282727913187209], [-0.56561671644413281, 0.79905220983688618]], [[-2.7146018685367066, -0.70433968981755601], [1.9771535643377183, -4.3485318545964144]], [[0.89322765036656993, -7.7067851800820488], [-3.1017503216886482, -3.9105094538028489]], [[-0.13763924230909996, -6.4448275782528945], [-7.6263337837490131, -5.1468042937802307]], [[-0.089076128117326903, -2.0722152257103836], [1.7384415513365505, -3.3969368051756552]]]])+(1.-msk_ref)*numpy.array([[[[-3.8699333254602717, 2.8943086111701355], [-2.767438823935362, -1.7324169786261967]], [[5.4932772205528853, -2.7954322420158451], [-0.070973181638704474, 3.4996866685613153]], [[-0.13636006056250771, 0.15782363355731199], [-3.4018556322041018, -2.9156172903728317]], [[-2.570996578443947, -0.36790628352476951], [-0.68595403545117062, -2.3338882182609524]], [[4.9514599419986762, -2.7587473299688048], [2.8346498867052734, -4.0112848722109131]], [[4.0354523002952387, 4.7746480762972627], [-3.4557286769364168, 2.1253311012148473]]], [[[-5.6343000373942633, 0.43282105770536639], [-0.93830287199328932, -0.42203446404612954]], [[-3.7542377321853491, -1.6195436775100838], [-1.706234283485994, 0.28803159063584438]], [[-0.1682646739527982, -5.8518659380386646], [-7.0102084242767688, -6.4188272261840673]], [[-6.40726138974519, -4.4342911836651435], [1.907588021367129, 2.0996682520318526]], [[1.3382326768318489, -4.0082233550374733], [2.055748565080064, -4.3613812717878959]], [[-3.6442385746330208, -3.9339517043855707], [-0.67716636980281031, -4.6378675273863044]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.9163517396493699, -0.31140149337273826, -4.0945814294340686, -2.2873541940176345, 4.9873150982725782], [-4.7272237424805308, 0.44162922608202138, 3.3610756989222086, 2.1398374164759577, -2.6793270434917114], [2.4275442137949108, 3.7686621264269959, -4.1500150227660972, -3.0351858683707578, -0.75336521489783337], [3.986957202168341, 0.77212755036399905, -3.5957079498009947, -0.43835738409783698, 0.31793629793973643]])+(1.-msk_arg0)*numpy.array([[-0.061979932246455149, 0.83405635059461058, 2.1367056239009186, 1.0790438306422532, 1.2435499840814117], [4.602505893630557, -3.4492605592035854, -2.487055802327045, 3.4227406589612848, -1.224543346565202], [1.8698813498620401, -0.96625560592011528, 0.91611360735577385, 3.6155256191588183, -4.4898269871553413], [0.89794300464314958, 0.26207528652477752, -0.10588845376660139, -1.5921127151183647, 1.5566746466372985]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[1.5071029704293624, 4.5677032889135205, 4.0283837482369513, 0.029757568890504515, 0.3947324322734671], [2.6704060641988638, 4.3884621941526998, 1.5097452749568427, -4.5769890690486514, 4.6255667235221267], [3.8994801226030731, 3.0229545082121181, 0.76710295979582366, -0.63283652189617179, 2.0298843042924428], [-0.59283567892064859, 3.5874382218588359, -1.7061999295011221, 1.5322874311562344, 0.14426293699579951]])+(1.-msk_arg1)*numpy.array([[-2.6642313264776627, -3.7366444100335041, 2.3169723537351681, 1.2061760698966868, 0.68769588310336438], [3.0296549146725447, 1.8809771873205978, -2.3793102635565355, 1.3285605850879811, -4.5513684447860889], [2.1466033702358551, 3.7989757006530862, -0.95038352059224174, -0.55860720246107398, -4.9881207115350961], [-0.17732215900483883, -2.7168666280159357, 3.4023092500180123, -3.6623832824077307, -3.0922722177425408]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.4092487692200075, 4.2563017955407823, -0.06619768119711722, -2.25759662512713, 5.3820475305460453], [-2.056817678281667, 4.8300914202347212, 4.8708209738790513, -2.4371516525726937, 1.9462396800304154], [6.3270243363979839, 6.791616634639114, -3.3829120629702736, -3.6680223902669296, 1.2765190893946095], [3.3941215232476925, 4.3595657722228349, -5.3019078793021173, 1.0939300470583975, 0.46219923493553594]])+(1.-msk_ref)*numpy.array([[-2.7262112587241178, -2.9025880594388935, 4.4536779776360866, 2.28521990053894, 1.9312458671847761], [7.6321608083031016, -1.5682833718829876, -4.8663660658835806, 4.7513012440492659, -5.775911791351291], [4.0164847200978953, 2.8327200947329709, -0.034269913236467886, 3.0569184166977443, -9.4779476986904374], [0.72062084563831075, -2.4547913414911582, 3.2964207962514109, -5.2544959975260959, -1.5355975711052423]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.5327412845794059, 2.7079384167236595], [-2.8680225169356719, 3.7199641994341128], [-1.4077403471310435, -1.1011441790930077], [-2.6595888791591129, 0.46516780814500613], [0.84822585907442605, -0.68438463716655651]], [[2.5951082589293728, 3.6019270810535051], [-1.2456400407027743, 3.8670407645030238], [-4.0788166807561534, 4.565299875453757], [-1.9026993763942732, -1.1132789198174331], [-3.4444804711551811, -2.3735910967382088]], [[-1.6736301862514824, -4.8751823603621647], [1.4930093924799861, 0.8326130919743493], [0.60894690453778377, -0.61288058980646909], [-2.3849522262480161, 2.0708566387762195], [-2.4095562980084462, 4.2132342561460252]], [[-2.7206798110054975, -2.6781431948159162], [-1.8004732938793788, -1.3260588346111279], [-3.6577658370840247, 1.2793073878198671], [-3.5666732262438816, 0.58152898637249706], [1.792523460130699, 2.2551575483953625]]])+(1.-msk_arg0)*numpy.array([[[2.331585225449329, 2.9033818500440614], [1.8106051099511351, 2.5486516175215836], [2.9301581468379183, 4.1674730574978458], [-4.4080977158430592, 0.69358959948233601], [2.3823271228944591, -4.0337521709072943]], [[4.8430597374621023, 2.813262200132181], [0.46922116204662068, -3.2713346991568262], [-4.5397947810081893, 0.54142840948865256], [4.0021630864411755, -0.2430693676693032], [1.9012247068957215, 3.2975149475799412]], [[0.64102510251951106, -1.6293943413403298], [3.8991462621596682, 3.817291097307093], [-3.4630534894793632, -4.3961877816564083], [1.2202850697018235, 3.6899052931078913], [-4.3185551020761821, 2.6768291459361828]], [[1.4121848918007798, 4.7064489290694684], [4.6611076007525511, -1.1522339993336606], [1.2462899189855676, 4.0852605842198049], [0.57085214801260253, -1.5859664517464167], [4.2463912859357347, -0.95138538967981656]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.7056776630992472, -4.9827243381206943, 0.23561566470456796, -2.0643042008634716, -1.4611290682630909], [-3.5851799679116372, -0.73751437195586522, 4.0824414021955988, 0.16088135054205477, -0.91468216701181237], [4.6793708679868313, -2.1768506739622309, 2.1932393928436724, -0.21028294100609912, -2.1384906853882413], [-2.1151834880323985, -3.6009681229380197, -4.8577455239822136, -4.1567132717169972, 1.4117813746503067]])+(1.-msk_arg1)*numpy.array([[2.5309889426174061, 1.8733693735434951, -2.4748304475443494, 1.7293056704957976, 0.49037691360724533], [2.153693405432441, -2.2949464342084838, 4.1491254560213235, -3.8807006115196874, -0.91791566582349216], [1.0974148513447455, 2.9366604269834777, -1.0292838389168479, 4.4532329038756675, -1.0137508642765014], [3.5061824389316225, 0.34741607695093624, -4.6248265393189092, -3.700923454416408, -0.82470143660273809]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.82706362148015877, 5.4136160798229067], [-7.8507468550563662, -1.2627601386865814], [-1.1721246824264755, -0.8655285143884397], [-4.7238930800225845, -1.5991363927184654], [-0.61290320918866481, -2.1455137054296474]], [[-0.99007170898226438, 0.016747113141867942], [-1.9831544126586396, 3.1295263925471586], [0.0036247214394453664, 8.6477412776493559], [-1.7418180258522185, -0.95239756927537833], [-4.3591626381669935, -3.2882732637500212]], [[3.0057406817353489, -0.19581149237533335], [-0.68384128148224477, -1.3442375819878816], [2.8021862973814562, 1.5803588030372033], [-2.5952351672541152, 1.8605736977701204], [-4.5480469833966879, 2.0747435707577839]], [[-4.8358632990378965, -4.7933266828483152], [-5.4014414168173985, -4.9270269575491472], [-8.5155113610662383, -3.5784381361623465], [-7.7233864979608793, -3.5751842853445002], [3.2043048347810057, 3.6669389230456693]]])+(1.-msk_ref)*numpy.array([[[4.8625741680667351, 5.4343707926614675], [3.6839744834946302, 4.4220209910650787], [0.45532769929356887, 1.6926426099534964], [-2.6787920453472616, 2.4228952699781336], [2.8727040365017045, -3.543375257300049]], [[6.9967531428945433, 4.966955605564622], [-1.8257252721618631, -5.5662811333653099], [-0.3906693249868658, 4.6905538655099761], [0.12146247492148809, -4.1237699791889906], [0.98330904107222938, 2.3795992817564491]], [[1.7384399538642565, -0.53197948999558431], [6.8358066891431459, 6.7539515242905708], [-4.4923373283962107, -5.4254716205732567], [5.673517973577491, 8.1431381969835588], [-5.3323059663526831, 1.6630782816596814]], [[4.9183673307324023, 8.2126313680010909], [5.0085236777034874, -0.80481792238272432], [-3.3785366203333416, -0.53956595509910432], [-3.1300713064038055, -5.2868899061628252], [3.4216898493329966, -1.7760868262825547]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.16617711043823036, 2.2135099261505653, 2.2079620251115557, 3.7504059874291009, 1.5340730700811527], [1.0168301976130669, -1.7005979690464779, 0.1906755555528612, -1.5667751444367961, 1.6997799505142712], [0.043251046488775202, 1.8270662616719129, 3.5962551675993417, -3.0873943553561478, 1.488437092147608], [-3.3154466019426501, -2.587953782449095, 2.7971912296439339, 0.85972909093307592, -1.7223808405574603]])+(1.-msk_arg0)*numpy.array([[1.3466522543784309, 2.7205311074365088, -1.4872996006633032, 4.4865286456988365, 0.00558024398931245], [-1.0668434993517195, 1.9691904110007101, 1.8817942171041668, 4.8584553299085353, -3.6587531642253381], [-0.75752768268404225, -3.1743694070095896, 4.8515939357808762, -0.32034714038946088, 0.68335664402809915], [-0.96932508496701608, -4.8920282387049649, 3.4117228801005712, -3.3743451821849724, -2.6149460417911286]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.1475492145741635, 2.1635940484849296], [-3.7164473063544401, -3.9545155165501011], [-2.7492070919647418, -3.4746361136079584], [-1.5675118654987741, -2.5693128586405392], [0.12561200180196685, 0.90334759846044399]], [[4.6801784229575176, -1.2618114455270648], [0.55622023940654941, -4.4125794776411977], [-3.2659342678044965, 1.4774111849271643], [4.2395072611865992, 0.86261728220079625], [0.040560289704336228, 2.5202323534158468]], [[3.9471893288235176, 4.2260172938653184], [0.21817579611455162, -4.8760760709318527], [-0.62556429555229975, -3.8314212949922908], [3.3232657428465568, -0.81141978666320114], [-4.9574214891942052, 4.6602736322104974]], [[1.7657078365570857, 1.4438135914014669], [2.6667403717704161, 2.6467335038300117], [-3.7242501792341134, -3.2338682187275456], [1.1419593339659606, 4.9645116807755567], [-3.9488698718953974, 3.3435228862015727]]])+(1.-msk_arg1)*numpy.array([[[-4.477512271737182, 3.5003289335417875], [-2.8907294736592428, -1.2806360454631105], [-3.7284957097784632, -3.7118764747645008], [-2.5184571631583563, 4.1151489507495551], [4.0922169930233263, -0.88919414322209001]], [[-2.4225689250793634, 3.399157443013701], [1.9508281404198611, 4.4859683383099824], [-2.6748791606265176, 1.486650011492423], [2.9986868021249453, -4.4231322295260211], [-3.0080215133478285, 2.9237594370054918]], [[-4.6191646779478948, -0.53865968201888048], [0.85603038513984053, 1.5513522555785819], [-2.9038233106600742, 0.17037726175570356], [3.7867390877450884, 4.5219126900627664], [4.9676708084382959, -3.0760599435885663]], [[1.4460579649267, 0.21328817659046706], [-2.9805845969288782, -3.3668117172050103], [3.5868453922568317, -2.9304761121556302], [1.201075324152109, -2.5591891207506565], [-2.5921109623180438, -1.1443931967880094]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.9813721041359331, 2.32977115892316], [-1.5029373802038748, -1.7410055903995358], [-0.54124506685318607, -1.2666740884964027], [2.1828941219303268, 1.1810931287885618], [1.6596850718831195, 2.4374206685415967]], [[5.6970086205705845, -0.24498124791399789], [-1.1443777296399285, -6.1131774466876756], [-3.0752587122516353, 1.6680867404800255], [2.672732116749803, -0.70415786223599985], [1.7403402402186074, 4.2200123039301181]], [[3.9904403753122928, 4.2692683403540936], [2.0452420577864645, -3.0490098092599398], [2.970690872047042, -0.23516612739294906], [0.23587138749040903, -3.8988141420193489], [-3.4689843970465972, 6.1487107243581054]], [[-1.5497387653855643, -1.8716330105411831], [0.078786589321321099, 0.058779721380916694], [-0.92705894959017954, -0.43667698908361174], [2.0016884248990365, 5.8242407717086326], [-5.6712507124528582, 1.6211420456441124]]])+(1.-msk_ref)*numpy.array([[[-3.1308600173587511, 4.8469811879202185], [-0.17019836622273399, 1.4398950619733983], [-5.2157953104417665, -5.199176075427804], [1.9680714825404801, 8.6016775964483916], [4.0977972370126388, -0.88361389923277756]], [[-3.4894124244310829, 2.3323139436619815], [3.9200185514205712, 6.4551587493106926], [-0.79308494352235082, 3.3684442285965899], [7.8571421320334807, 0.43532310038251421], [-6.6667746775731667, -0.73499372721984635]], [[-5.376692360631937, -1.2961873647029227], [-2.3183390218697491, -1.6230171514310077], [1.947770625120802, 5.0219711975365797], [3.4663919473556275, 4.2015655496733055], [5.651027452466395, -2.3927032995604671]], [[0.47673287995968394, -0.75603690837654902], [-7.8726128356338432, -8.2588399559099752], [6.998568272357403, 0.48124676794494103], [-2.1732698580328633, -5.9335343029356284], [-5.2070570041091724, -3.759339238579138]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.66255840379699027, -0.4004340188754707, -0.11361900052623319, -1.4083572224024232, 1.1725525242900066], [-0.29088760914477163, -4.1458994800242186, 1.113409866098694, 3.2327989756937434, 0.18678382375531566], [-3.7593023531945047, -4.5467853372746276, 2.0547036372150611, -0.90955381367254695, -0.93818360896637643], [-3.3837416894134553, -3.7962813351709856, 2.0269288743712908, -1.3645486981591306, 3.2347750462031719]], [[4.2251294706095663, -2.1579810906530303, 0.066334659095899973, 1.7480154786761251, -2.4132836585725039], [1.1094848244016404, 3.0575065719308885, -3.9122755819121902, -4.5973282518455472, 0.6888429962747491], [-2.3711285450399555, 3.2715606329559765, -4.4434295340756762, -1.4262896983608133, -1.0003907934320067], [3.665069936786475, 4.7161614203356734, -0.74562787426966892, 4.5637578941954988, 2.1150486137604751]], [[-0.91911008123943727, 4.6244290100299619, -0.3992651935336653, -4.2183428357579933, -1.3661031445857184], [0.33172721074567768, 1.3410868729056702, -2.1510865642641566, 4.4403855486810766, -0.50894894850213568], [-3.2062352827211447, 2.2510914446448709, -3.3018721344312718, 3.0226764952513303, 0.98734580501753655], [4.4055828941581634, 1.6947616907069518, 1.02595931171102, -0.70157605925889222, 3.1343958389768574]], [[4.9075573715101619, -1.2732827687265824, -1.5879049161655647, 1.5211690769350508, -1.0531948874191697], [0.32534887944017044, 1.841767576702674, -4.5333688735212183, -4.1251529440388524, -2.4411155761130199], [-2.5984455020903532, -3.9639653831744215, -2.2522902250804622, -3.2202378090146411, 1.4926702734965556], [-1.6595843937671328, 2.1776654990151432, 4.7006394701849956, 0.87210691565418763, 0.15389605689139962]], [[2.8935856589615216, -3.3017926803531661, -4.2622625799674587, 0.075594174087244781, -0.30158580698828708], [-0.039801198998203446, 1.990882343801359, -4.5937340570997982, -1.6738841710030328, 1.4325845833055908], [3.5180827015184377, -4.2568973304000455, -3.3021554930321919, 0.58697329011225285, -2.2113127875159746], [1.796904438427215, 2.8766865073686212, -4.4274896162750856, 3.1325448965153857, -4.6893825647978069]]], [[[-2.0931693048887934, -1.8948366976962072, -1.008446249840552, -2.7184695242209225, 4.0685819701109409], [-0.7613442760588951, 1.0552608344914187, -2.0768889315481487, 3.4132874442168255, -1.9943833122545893], [0.46042998053209683, -2.2214715383024686, -4.4203411057696709, 4.7796795246824857, 3.1803102620226191], [-0.37750702247601886, 4.8623411649910846, 1.7270570804158991, -2.2983260134602168, 1.8224460122246677]], [[-4.1210071250270541, 1.3633138457911631, -2.1968514723888797, 3.0902181560122273, -0.43596657564455299], [4.5017595971949902, -4.3323404132271524, -0.68883147759579266, 2.7893436997835703, -2.7172941258689285], [3.6534125466114311, 0.59443369642319155, -1.7495191762845397, -0.88908996365174886, -2.5719943320123706], [3.6660824868234965, 3.1122046587065011, 1.1831630217015698, -3.6817466597655049, 2.8667402373430182]], [[1.3904592596254091, 0.46497238427085819, 1.4477824118558233, 4.0967887816697814, 3.1750184287433605], [-0.037507202610642665, 2.8869168708006789, -4.6934533905325226, 1.2971372101269116, -1.0659191286024052], [-4.8883640105422232, 1.1761856615749178, -1.1077962954222311, 2.3969844127186155, -0.69705930770370372], [-2.0731969842273799, 1.0556835810291076, -3.3079957373432523, -0.42634699288257938, 0.70185177969903645]], [[-0.69254452161988667, 4.6604785008649738, -0.2750337688410962, -0.83143516565424047, -4.6992829599018204], [2.446244721996619, -2.5961697320252233, -4.0133865333890277, 3.4357175367560977, -3.2483679257538611], [2.10394896940696, -1.9862683705043551, -2.4247133001376509, -2.2217309988857226, -4.3165145312069573], [-2.1397853420723068, -4.3128491136519775, 3.3355720451828539, -4.0897355058867308, -3.8476164252485203]], [[-2.1870879481450389, -2.5249725285389402, -4.4681777183110221, -0.77901801495763578, 1.1949177774126527], [0.27570252537750051, 3.7491981366985971, -3.5058654843516148, -0.20029185315155651, 2.3226814275681296], [-4.6724397568489104, 4.77095391887622, 2.7271115693097681, -4.7568797449790043, -0.18079230602725449], [2.7708143272810739, -0.84183469811614664, -0.095229371195852508, -0.42492193668218725, -0.049709679347659019]]], [[[-4.8847548334674453, 0.33491392975493461, -0.28560656154918007, 0.34465969765338578, -4.7980148528548048], [4.2090938894984813, 2.1144097610231727, 3.470624148554398, -3.5752400414279197, 2.8968552930226092], [1.3011753306410512, 4.8025141370571092, 2.2189733440142403, -0.27943327879237945, -0.94737279212435244], [1.1166858044785553, -0.54787202130936841, -0.92446967749976938, 4.7791833723864467, 1.6073749298767739]], [[3.33022399989426, 1.3896111918879805, -1.8171424389941904, -4.8581031512046957, -0.50935281298586066], [-2.2238595371517178, -0.86609619487837008, -1.351456270950564, 4.417476758469153, -2.452461718903729], [1.7639086780847251, -2.6596426544269569, -1.4162122977298344, -4.3391038825731085, 4.3051330302556021], [-1.3672360771085512, 3.6631885640106567, -0.59916844109758838, 3.1137699984992437, 1.8515096017033112]], [[0.46509721144545413, -0.32479102965741369, 4.3118459087603718, -4.2763705365266631, 0.22582078204910072], [-4.7369842657148409, 2.1987090845472403, 0.71163302477053314, 2.716196006009878, 2.748244257370426], [-2.1781727840568266, 1.8525090607057422, 3.3378519606925927, 0.43838336471002926, -2.931630603058327], [-4.7755512394916364, -4.6029833674453133, 2.7620368532793629, -4.38446159205653, -1.6897492950021076]], [[-1.5074186528058462, 1.192205108318741, 2.1599355992116038, -0.24672596432673632, -0.26072012608997586], [-3.329941835437352, 0.36022707243956287, -2.9054240217703486, -2.9208723025433416, -3.8042751568437505], [-0.0074523173134943832, -0.76342196209727131, 2.1176039242823563, 1.845326754712878, 0.27737561945098577], [-2.8658350533031864, 3.4409351333443965, 0.057315501786881384, 1.5095230073746846, 1.8284342697685938]], [[-2.0740735602814695, -3.7580750722153757, 3.6201251127033665, -2.7858609482191401, -2.4031596597146923], [-4.140648820333408, 2.1017095514645137, -2.4439298962745171, -4.0745047186714789, 1.0326282545798957], [4.7225172620490827, 4.9253565797778549, 2.2347258044757732, 1.0818721467589478, 1.0109397865847223], [-2.5509565954065003, 2.8153031226088672, 3.1015262828407764, -1.4186544180079497, -4.4608121016141036]]], [[[-4.0885819002083803, -1.7426512759056223, -0.85359753226344726, 1.2074337738085639, 3.178188002883843], [2.4370676759571506, 0.10576539813814012, 3.2352537369794945, 4.2115519306880138, 0.27574825185523544], [-0.97120538125082145, 2.0318730938288274, -4.5928506160636413, -1.84508563431831, -2.336679657400722], [-1.5193864872168614, -0.38588341783821889, -2.4663901352386528, -4.9531855150121906, 0.80799293918082427]], [[-2.4352344729540421, -1.622485119240519, -2.9060090627060955, -3.9618304886687761, 1.8974121671179738], [2.368151305332229, 3.7266779090431328, -2.832972734574545, 4.1657136572605182, -3.5215175136557098], [-3.4527181144486221, 3.304218935908823, -4.7885849652317036, 3.2037385758815944, 1.5936161283694963], [-3.3910680529727166, 3.7814723592498147, -4.007781824993395, 2.0873805035910165, 0.011988661273038659]], [[4.4315896229854328, -3.1138318080229723, -2.5700355542276823, -1.0179068948029304, -1.9721754988896159], [3.7471468008062327, -3.9833201256489295, -1.2488633174697572, 2.3081843729945994, 1.5829609167057619], [-3.052597514632386, 1.9962549909310159, 1.9311456194671397, 0.78668334844476107, 4.9649234459684521], [1.6649223264407524, 4.5822454839160294, 3.0799954054811085, -3.1678789454274181, 1.8566536579647472]], [[-1.4604018824865825, 4.822028775360911, 4.61491907977517, -4.9097168444464057, 0.20717897180818134], [-2.1016826050787172, -2.199557142384192, 2.2674530393280783, 2.1417578522507092, 1.7509027877457264], [2.7191661604240398, -3.2606428287235945, -3.2462101468632132, 3.5385149649495791, -1.2897704107808283], [4.4156312563278135, 0.25014748359468708, -3.9135807353924834, -2.6883194776387977, 2.4463905750696036]], [[-0.43891556966673306, -0.3540624121165683, -2.6390063872109035, 1.0337305730409199, 2.7791539522321598], [-2.5406812451261205, 4.8796428763470541, -0.36205072922985337, -2.7470088463147677, -1.3860978960403125], [-0.93384576468022651, 1.5467179494910788, 2.8125458598129027, 3.6594786802935388, -1.0371481195714582], [-0.8558161309976402, 0.18544064609888444, -4.2002390119318278, 4.9559602172280677, -1.1029485778973704]]]])+(1.-msk_arg0)*numpy.array([[[[4.78851461666906, -0.924485415000591, -3.5670653548279194, -3.5910163104936399, 4.5514864561804416], [1.9123485275279783, -0.22336143366246564, -1.0367183701301244, -2.4088957030392, 2.5313229821267402], [2.8551331830966609, 4.0800024307960587, -3.9676489188328601, -4.583719528518797, -1.9197766568667616], [-4.0479969050836564, -4.7696458816418215, 2.8645385688046963, -4.6839409274469643, 3.7651054491050697]], [[-0.13548167377279619, 3.0190774340836448, 3.3928304851697604, -0.48432473753024041, 3.8306271245374575], [2.1394646318478756, -3.5181735942358516, -4.9472667797079586, 0.30598591137143138, 3.238875220210387], [2.5493215728368144, -1.5869476968512775, 2.161388614696782, 0.097653166723708829, -4.6644554259863247], [-0.35364840003089348, -4.7351710813284154, -4.4581175288293684, 4.1187735498712321, 2.9970518431168101]], [[1.9427621489662572, 0.5461164733925985, -1.2441870320277912, 4.8166758001162311, 1.4261905407092934], [0.77420982276695405, 1.8582148064923585, 1.0416836340601598, -4.8120106437954204, 2.5219774668256463], [-2.5157563987066434, 3.5604051669624024, -0.086886420156316646, -1.7972055864449255, 1.1459618104990525], [-4.4944955742529675, 2.9032239874606978, 0.99121808413759194, -0.39916572525372906, 3.5643849938595018]], [[-0.53062827346149977, -4.482046758948659, -0.36678958256241856, 2.3737550338483091, -2.6379002943172338], [2.4211769769609601, 4.59248038906237, 2.4121885743441984, -2.848665285659143, -3.4234498858364582], [1.9775247491584711, 0.49158176725927927, 2.7461855664017065, 4.1088878329145331, 3.0916099970710071], [-3.3180553626348406, -0.68940553000695282, 3.3111217639910073, -2.4232795194965098, 2.2403786202007119]], [[1.4859375263133225, 1.4980502109452747, 2.3625745547254882, 0.41993272589454733, -4.5470211445366404], [-4.6993845405676513, 4.2458624408199697, -0.9225051885429183, 1.7470034765297058, 0.7716686893025928], [2.0236321885642177, -4.2903333148488274, 3.1228653911096949, 4.339952301489518, -0.47437064423484365], [-4.0760017671511983, -1.6757282634401505, 1.3207270273270977, -3.8725917795084097, 4.4709521100996703]]], [[[4.710029115196571, -1.2247397562717453, -2.8967827500997254, 2.1261387982271662, -1.7997116576039627], [-0.046429032534877734, -0.38402506149994586, 4.9746049019291672, 1.1531860253611441, -1.4001009946671594], [2.7075229985850964, 2.834387755006464, -1.8141951608550055, -4.0283713688606237, -3.0141692613431958], [-2.6899588766851332, 1.9433792172197499, 0.84630891023534804, 0.53712955866577872, 1.277889619412961]], [[-4.4504939840926339, 1.3063071020019326, 4.7778630319348672, -4.9242854990632914, -0.94470720262858698], [-1.7009324749070309, -3.2140017484186045, -0.10609411526312762, -3.0763553126858167, -4.6703289062854587], [-4.1634233424126501, -1.3397364615975316, -3.2331736446842152, -1.6636470599491702, -0.598192818993871], [1.4478449196246235, -0.94368146712140621, 0.93443340582808698, -0.69203769464913378, -1.5030726023060916]], [[-1.1129243605835071, 0.49820381311557949, 0.067064133556455019, 2.1943307479073235, 1.5246105610400029], [1.5095307373261049, 2.8988880107662593, 4.3775495698482754, 1.8947136678525069, -4.2546589046305385], [-1.3113364728720311, -0.10513243090216395, 2.3726573369753279, -0.21681171326704352, -1.5074340014324417], [1.0863734049559435, 1.9166181145138896, 0.38969119583581424, 2.4946217661899528, 0.50928522587585601]], [[2.9224076257753708, 4.6089586127350106, -3.859526476083095, -3.7426092148124601, -4.0302349473650629], [4.7252997590491344, -4.5961052217275604, -3.6414095009691581, 1.5155928317594602, 4.2171812265404256], [-0.16215434591115407, -4.8447467748327711, 4.9700103754022891, -0.36043839949633316, 0.8870008708537549], [2.4950095288496765, 2.4425094487000578, -1.9538810827714759, 4.1628055568207163, -3.2776413107418847]], [[0.42695909771999396, 3.7150432769128408, 1.8397727023979762, 2.3904011919331323, 4.4109206254905899], [-1.1751745435930161, -0.92110756329105481, 2.8630160012279671, 3.0483360265707962, 4.5350025412657953], [4.4804186036556821, -0.0046124338804425236, 1.0415501753043799, 3.3284186915830958, 0.18894405128864733], [2.7557178799368574, 1.4731801176131718, 4.6789921832229133, 4.9752832505436224, 4.4464115683831267]]], [[[0.84128078917745786, 1.4098456270720972, 4.1439553247022332, -1.208720751776533, -0.33007997691377078], [-0.042033012659020486, -2.264564009628196, 0.018782982579059748, -3.755469838415153, -1.9955493349303701], [-4.1669272465430032, -3.0004484202274639, 0.88676100188601836, 1.1075708593700027, 1.0012076265751197], [-2.3820957809412855, -1.2440337698828765, -2.5493188866813368, -3.6506549441634606, 2.5487242735067142]], [[4.5617802706220481, -2.4799774834945811, -2.3069989690796922, -1.6312590236035431, -0.39364350566410611], [2.7181461083912017, 0.84168433732536663, -3.2439224901703256, 0.61649716281238653, 2.7560431786264648], [-0.39080176562703173, 1.0976503844402181, -3.0003508540896764, 0.37674609301041428, 2.2169845289776831], [1.8448298141302457, -4.5601340213947967, -3.2309437839878363, -0.02409259013987608, 1.1196563847444256]], [[3.8837399671963997, 3.9422457447850459, -1.0872615038304123, 2.1127983465129727, -0.62451625102596076], [-2.4557256313061582, 1.9676766653781605, 3.6778477805605565, -4.7765150251761792, 2.1314099394720945], [1.1978462706166653, 0.21343435094691188, 0.43826037840823773, 1.0294037840638204, -3.8414600265077157], [-0.69644036321648262, -2.309061756778179, -2.0399874737225487, 2.3697258590071089, -0.77760461782917201]], [[-0.46382465307573373, -4.4367118707665929, 4.193292887309866, 4.2386662696767807, -3.0486782762373243], [1.1199293755118838, -2.6173882199274301, -3.4386566952882056, 3.8347404679738624, -4.941500984528064], [4.7279726727232116, 3.0520337647356861, -0.10752046288619077, 3.9354048155321273, -0.098743607425695323], [-3.7394870119663723, 1.5673946757706698, 0.82144505935172862, -1.3965444089097967, -2.6467209405314271]], [[-2.7814869276325247, -0.73168692660196655, 2.1207428554598629, -2.6934210548570778, -3.5025437411260074], [1.0740062733222269, -1.679960926726852, -3.3430712161232767, -4.5209963788683698, -2.1019827093198771], [2.7858992919640162, 1.8552695316478864, 4.5703518430995267, 0.48113821946750157, -2.9266579882352675], [4.4075108730807671, -0.3019124567933229, 4.1482754647681901, -0.094571333769462029, 4.3380405426474713]]], [[[3.8907667038121421, -4.4481106476445342, -3.8120290765667231, -3.6422830319142694, -0.32860852107652772], [-1.4772473247570419, 2.3968512174151684, 4.0935611896294137, -4.4932280963418663, -1.371170459517479], [4.0648959157800988, -3.3158195138433411, -2.5607695724937596, -3.755865240682347, -2.4921507009466728], [-3.3424939889678775, -1.2236908059184595, 1.8193324315398627, -3.1475046256214179, -3.2037547369201591]], [[2.0470958786118167, 1.2370424381200387, -1.1320522739726115, -4.5402262228865542, 3.006003554160591], [-1.2120357015781913, 0.83036146860304338, -1.6967185635642315, -1.17906712325621, 0.47569105896167763], [4.4143529534760333, 1.8392570066655702, 2.8176308482810395, 3.3560402140381118, -0.14752105053347542], [-1.9767208744546072, -2.790014738362443, 3.4123796557435799, -3.2574613824803733, -2.1678152301812159]], [[-1.9949843184970706, 3.1337793956659592, -0.94678704823075499, 1.0052188022382342, 0.33694715568761424], [4.777222611370096, -2.0989623151029688, -0.11042358514733408, -4.2862238553097063, -4.9930673513560411], [4.2121375776886509, -0.13579497908831595, -2.1959318404556303, -2.9589172057349247, -4.5501166228464633], [3.2558672146656171, -4.7320886872291972, -4.9333492719336123, -2.4018888660317361, -2.2830183437254812]], [[4.4651944901774172, 3.3033156185390489, 4.781475029539239, -3.8970453110938719, 4.9355637995661876], [-1.766061440286455, -2.2880373359705075, 3.6209591457581887, 1.2649967380886373, 1.7838654585789415], [3.2744791725848224, 4.2790667555489144, -1.3859710744885447, -2.8650403534658353, 2.1176400321968742], [1.22570798662057, 1.2949966085937312, 3.9364659262567887, -0.3141254880002089, 4.6546286842253242]], [[2.6113516763329532, 1.4037148283464829, -4.4667164647815802, -3.1452893989942021, -1.4437584031823314], [-0.73236642529579044, -3.6566769811864965, -4.8040665217935725, -2.6380814957023278, 3.6772020505453433], [4.2398451779379833, -0.47700349879341442, -4.1570476927149569, -1.3166982121188076, 2.1961947987007662], [4.4057528065649638, 2.1793499325297656, -4.4148118843626261, -1.0599801658635952, -3.3171042840178053]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.1846109636696394, 3.3737793727861778, -2.0967947988166156, -2.0296913449439713, 2.727149300967163], [-3.0415495816521432, 1.423604952847473, -1.5891306768246438, -2.0224525608978472, 4.0152468265503174], [0.26631024207212661, -1.3561962144588344, -1.8932258094439405, 0.031847648959111297, -2.100142635114862], [-4.6180825071826099, -3.3375606810407565, -4.0136863251127881, 4.1522023073866006, -4.2126158648319834]])+(1.-msk_arg1)*numpy.array([[-2.6272493768656071, 4.9732280787236007, 2.4708609570620421, 1.403986298801323, -3.3604550033161606], [1.927001277696009, -1.7303289171531389, -3.9907615771245073, -0.29464201845958105, -3.8766731973798185], [0.26052549521761481, -0.51016532420167149, -0.15496687086830896, -1.258867256794729, -3.7664658418699357], [0.69164642270925292, -0.67104243848687073, 0.99601819927748192, 0.57830252905823354, -4.9945273005430675]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.5220525598726491, -3.5850449825451101, -3.2982299641958726, -4.5929681860720626, -2.0120584393796328], [-3.475498572814411, -7.330510443693858, -2.0712010975709454, 0.048188012024104054, -2.9978271399143237], [-6.943913316864144, -7.731396300944267, -1.1299073264545783, -4.0941647773421863, -4.1227945726360158], [-6.5683526530830942, -6.980892298840625, -1.1576820892983486, -4.54915966182877, 0.05016408253353255]], [[7.5989088433957441, 1.2157982821331474, 3.4401140318820778, 5.1217948514623028, 0.96049571421367386], [4.4832641971878182, 6.4312859447170663, -0.53849620912601237, -1.2235488790593694, 4.0626223690609269], [1.0026508277462223, 6.6453400057421543, -1.0696501612894984, 1.9474896744253645, 2.3733885793541711], [7.0388493095726528, 8.0899407931218512, 2.6281514985165089, 7.9375372669816766, 5.4888279865466529]], [[-3.0159048800560528, 2.5276342112133463, -2.4960599923502809, -6.3151376345746089, -3.4628979434023339], [-1.7650675880709379, -0.75570792591094538, -4.2478813630807721, 2.343590749864461, -2.6057437473187512], [-5.3030300815377602, 0.15429664582825531, -5.3986669332478874, 0.92588169643471474, -1.109448993799079], [2.3087880953415478, -0.40203310810966375, -1.0708354871055956, -2.7983708580755078, 1.0376010401602418]], [[2.8778660265661906, -3.3029741136705537, -3.617596261109536, -0.50852226800892053, -3.082886232363141], [-1.7043424655038009, -0.18792376824129731, -6.5630602184651892, -6.1548442889828241, -4.4708069210569912], [-4.6281368470343249, -5.9936567281183928, -4.281981570024433, -5.2499291539586128, -0.53702107144741573], [-3.6892757387111041, 0.14797415407117187, 2.6709481252410243, -1.1575844292897837, -1.8757952880525717]], [[5.6207349599286847, -0.57464337938600307, -1.5351132790002957, 2.8027434750544078, 2.4255634939788759], [2.6873481019689596, 4.718031644768522, -1.8665847561326352, 1.0532651299641302, 4.1597338842727538], [6.2452320024856007, -1.5297480294328825, -0.5750061920650289, 3.3141225910794159, 0.51583651345118842], [4.524053739394378, 5.6038358083357842, -1.7003403153079226, 5.8596941974825487, -1.9622332638306439]]], [[[-5.1347188865409361, -4.9363862793483504, -4.0499958314926952, -5.7600191058730656, 1.0270323884587977], [-3.8028938577110383, -1.9862887471607245, -5.1184385132002923, 0.37173786256468233, -5.0359328939067325], [-2.5811196011200463, -5.2630211199546117, -7.4618906874218141, 1.7381299430303425, 0.13876068037047595], [-3.419056604128162, 1.8207915833389414, -1.3144925012362441, -5.3398755951123604, -1.2191035694274754]], [[-2.6974021721795811, 2.7869187986386361, -0.77324651954140666, 4.5138231088597003, 0.98763837720292003], [5.9253645500424632, -2.9087354603796793, 0.73477347525168035, 4.2129486526310433, -1.2936891730214555], [5.0770174994589041, 2.0180386492706646, -0.32591422343706666, 0.53451498919572416, -1.1483893791648976], [5.0896874396709695, 4.5358096115539741, 2.6067679745490429, -2.2581417069180318, 4.2903451901904912]], [[-0.19867141719923476, -1.1241582925537856, -0.14134826496882047, 2.5076581048451376, 1.5858877519187167], [-1.6266378794352865, 1.2977861939760351, -6.2825840673571669, -0.29199346669773218, -2.655049805427049], [-6.4774946873668675, -0.41294501524972604, -2.6969269722468749, 0.80785373589397169, -2.2861899845283475], [-3.6623276610520237, -0.53344709579553617, -4.8971264141678965, -2.0154776697072232, -0.88727889712560737]], [[-2.7149970825177339, 2.6380259399671266, -2.2974863297389434, -2.8538877265520877, -6.7217355207996672], [0.42379216109877182, -4.6186222929230709, -6.0358390942868745, 1.4132649758582505, -5.2708204866517079], [0.081496408509112772, -4.0087209314022019, -4.447165861035498, -4.2441835597835702, -6.338967092104804], [-4.1622379029701539, -6.3353016745498252, 1.3131194842850067, -6.1121880667845776, -5.870068986146368]], [[1.8281588784052785, 1.4902742980113772, -0.45293089176070467, 3.2362288115926816, 5.2101646039629701], [4.2909493519278179, 7.7644449632489145, 0.50938134219870257, 3.8149549733987609, 6.337928254118447], [-0.65719293029859305, 8.7862007454265374, 6.7423583958600855, -0.74163291842868695, 3.8344545205230629], [6.7860611538313913, 3.1734121284341708, 3.9200174553544649, 3.5903248898681301, 3.9655371472026584]]], [[[-4.6184445913953187, 0.60122417182706123, -0.019296319477053459, 0.61096993972551239, -4.5317046107826782], [4.4754041315706079, 2.3807200030952993, 3.7369343906265247, -3.3089297993557931, 3.1631655350947359], [1.5674855727131778, 5.0688243791292358, 2.4852835860863669, -0.013123036720252834, -0.68106255005222582], [1.3829960465506819, -0.28156177923724179, -0.65815943542764277, 5.0454936144585734, 1.8736851719489005]], [[1.9740277854354256, 0.03341497742914612, -3.1733386534530248, -6.2142993656635301, -1.8655490274446951], [-3.5800557516105522, -2.2222924093372045, -2.7076524854093984, 3.0612805440103186, -3.8086579333625634], [0.40771246362589064, -4.0158388688857913, -2.7724085121886688, -5.695300097031943, 2.9489368157967677], [-2.7234322915673856, 2.3069923495518223, -1.9553646555564228, 1.7575737840404093, 0.49531338724447682]], [[-1.4281285979984863, -2.2180168391013542, 2.4186200993164313, -6.1695963459706036, -1.6674050273948398], [-6.6302100751587814, 0.30548327510329987, -1.1815927846734073, 0.82297019656593751, 0.85501844792648551], [-4.071398593500767, -0.040716748738198305, 1.4446261512486522, -1.4548424447339112, -4.8248564125022675], [-6.6687770489355769, -6.4962091768892538, 0.8688110438354224, -6.2776874015004704, -3.5829751044460481]], [[-1.4755710038467349, 1.2240527572778523, 2.1917832481707151, -0.21487831536762503, -0.22887247713086456], [-3.2980941864782407, 0.39207472139867416, -2.8735763728112373, -2.8890246535842303, -3.7724275078846392], [0.024395331645616913, -0.73157431313816002, 2.1494515732414676, 1.8771744036719893, 0.30922326841009706], [-2.8339874043440751, 3.4727827823035078, 0.089163150745992681, 1.5413706563337959, 1.8602819187277051]], [[-4.1742161953963315, -5.8582177073302377, 1.5199824775885045, -4.8860035833340021, -4.5033022948295542], [-6.24079145544827, 0.0015669163496516703, -4.5440725313893786, -6.1746473537863409, -1.0675143805349663], [2.6223746269342207, 2.8252139446629929, 0.13458316936091119, -1.0182704883559142, -1.0892028485301397], [-4.6510992305213623, 0.71516048749400518, 1.0013836477259144, -3.5187970531228117, -6.5609547367289656]]], [[[-8.7066644073909902, -6.3607337830882322, -5.4716800394460572, -3.4106487333740461, -1.439894504298767], [-2.1810148312254594, -4.5123171090444698, -1.3828287702031155, -0.40653057649459612, -4.3423342553273745], [-5.5892878884334314, -2.5862094133537825, -9.2109331232462512, -6.4631681415009199, -6.9547621645833324], [-6.1374689943994714, -5.0039659250208288, -7.0844726424212627, -9.5712680221948006, -3.8100895680017857]], [[-5.7727951539947986, -4.9600458002812751, -6.2435697437468516, -7.2993911697095326, -1.4401485139227828], [-0.96940937570852759, 0.38911722800237625, -6.1705334156153011, 0.82815297621976169, -6.8590781946964663], [-6.7902787954893782, -0.033341745131933553, -8.1261456462724606, -0.13382210515916215, -1.7439445526712603], [-6.7286287340134727, 0.44391167820905819, -7.345342506034152, -1.2501801774497401, -3.3255720197677179]], [[0.41790329787264469, -7.1275181331357604, -6.5837218793404704, -5.0315932199157185, -5.985861824002404], [-0.26653952430655536, -7.9970064507617176, -5.2625496425825453, -1.7055019521181887, -2.4307254084070262], [-7.0662838397451742, -2.0174313341817722, -2.0825407056456484, -3.227002976668027, 0.95123712085566403], [-2.3487639986720357, 0.56855915880324126, -0.93369091963167961, -7.1815652705402062, -2.1570326671480409]], [[2.691800424900018, 8.9742310827475116, 8.7671213871617706, -0.75751453705980509, 4.3593812791947819], [2.0505197023078834, 1.9526451650024086, 6.4196553467146789, 6.2939601596373098, 5.903105095132327], [6.8713684678106404, 0.89155947866300611, 0.90599216052338738, 7.6907172723361796, 2.8624318966057722], [8.5678335637144141, 4.4023497909812876, 0.23862157199411715, 1.4638828297478028, 6.5985928824562041]], [[-4.6515314344987164, -4.5666782769485517, -6.8516222520428869, -3.1788852917910635, -1.4334619125998236], [-6.7532971099581038, 0.66702701151507071, -4.5746665940618367, -6.9596247111467511, -5.5987137608722959], [-5.1464616295122099, -2.6658979153409046, -1.4000700050190806, -0.55313718453844452, -5.2497639844034421], [-5.0684319958296236, -4.0271752187330989, -8.4128548767638112, 0.74334435239608432, -5.3155644427293538]]]])+(1.-msk_ref)*numpy.array([[[[2.1612652398034529, -3.5517347918661981, -6.1943147316935265, -6.2182656873592475, 1.9242370793148345], [-0.71490084933762876, -2.8506108105280727, -3.6639677469957315, -5.0361450799048075, -0.095926394738866883], [0.22788380623105375, 1.4527530539304516, -6.5948982956984672, -7.2109689053844042, -4.5470260337323687], [-6.6752462819492635, -7.3968952585074286, 0.23728919193908915, -7.3111903043125714, 1.1378560722394626]], [[4.8377464049508045, 7.9923055128072455, 8.3660585638933611, 4.4889033411933603, 8.8038552032610582], [7.1126927105714763, 1.4550544844877491, 0.025961299015642147, 5.2792139900950321, 8.2121032989339877], [7.5225496515604151, 3.3862803818723233, 7.1346166934203827, 5.0708812454473096, 0.30877265273727605], [4.6195796786927072, 0.23805699739518538, 0.51511054989423233, 9.0920016285948329, 7.9702799218404108]], [[4.4136231060282993, 3.0169774304546406, 1.2266739250342509, 7.2875367571782732, 3.8970514977713355], [3.2450707798289962, 4.3290757635544006, 3.512544591122202, -2.3411496867333783, 4.9928384238876884], [-0.044895441644601242, 6.0312661240244445, 2.3839745369057255, 0.67365537061711667, 3.6168227675610947], [-2.0236346171909254, 5.3740849445227399, 3.4620790411996341, 2.0716952318083131, 6.0352459509215439]], [[0.87335802533982321, -3.078060460147336, 1.0371967162389044, 3.7777413326496321, -1.2339139955159109], [3.8251632757622831, 5.996466687863693, 3.8161748731455214, -1.44467898685782, -2.0194635870351352], [3.3815110479597941, 1.8955680660606022, 4.1501718652030295, 5.5128741317158561, 4.4955962958723301], [-1.9140690638335176, 0.71458076879437016, 4.7151080627923303, -1.0192932206951868, 3.6443649190020349]], [[-1.8745174770028381, -1.862404792370886, -0.99788044859067249, -2.9405222774216133, -7.9074761478528011], [-8.0598395438838111, 0.88540743750380901, -4.2829601918590789, -1.6134515267864549, -2.5887863140135678], [-1.336822814751943, -7.6507883181649881, -0.23758961220646579, 0.9794972981733574, -3.8348256475510043], [-7.436456770467359, -5.0361832667563107, -2.039727975989063, -7.2330467828245704, 1.1104971067835097]]], [[[6.63703039289258, 0.70226152142426379, -0.96978147240371637, 4.0531400759231753, 0.12728962009204636], [1.8805722451611313, 1.5429762161960632, 6.9016061796251762, 3.0801873030571532, 0.52690028302884961], [4.6345242762811054, 4.761389032702473, 0.1128061168410035, -2.1013700911646147, -1.0871679836471868], [-0.76295759898912419, 3.870380494915759, 2.7733101879313571, 2.4641308363617878, 3.20489089710897]], [[-6.1808229012457723, -0.42402181515120629, 3.0475341147817283, -6.6546144162164307, -2.6750361197817258], [-3.4312613920601698, -4.9443306655717434, -1.8364230324162665, -4.8066842298389556, -6.400657823438598], [-5.8937522595657885, -3.0700653787506704, -4.9635025618373536, -3.393975977102309, -2.3285217361470099], [-0.28248399752851538, -2.6740103842745451, -0.79589551132505187, -2.4223666118022726, -3.2334015194592305]], [[-5.1036859377080148, -3.4925577640089278, -3.9236974435680523, -1.7964308292171838, -2.4661510160845044], [-2.4812308397984024, -1.0918735663582479, 0.38678799272376807, -2.0960479092720004, -8.2454204817550458], [-5.3020980499965384, -4.0958940080266713, -1.6181042401491794, -4.2075732903915508, -5.498195578556949], [-2.9043881721685638, -2.0741434626106177, -3.6010703812886931, -1.4961398109345545, -3.4814763512486513]], [[2.6277656073157898, 4.3143165942754296, -4.1541684945426756, -4.0372512332720412, -4.324876965824644], [4.4306577405895533, -4.8907472401871415, -3.9360515194287391, 1.2209508132998792, 3.9225392080808446], [-0.45679636437073512, -5.1393887932923521, 4.675368356942708, -0.65508041795591421, 0.59235885239417385], [2.2003675103900955, 2.1478674302404768, -2.2485231012310569, 3.8681635383611352, -3.5722833292014657]], [[-3.4497140996598246, -0.16162992046697777, -2.0369004949818423, -1.4862720054466863, 0.53424742811077142], [-5.0518477409728346, -4.7977807606708733, -1.0136571961518515, -0.82833717080902236, 0.65832934388597675], [0.60374540627586359, -3.881285631260261, -2.8351230220754386, -0.54825450579672275, -3.6877291460911712], [-1.1209553174429612, -2.4034930797666467, 0.8023189858430948, 1.0986100531638039, 0.56973837100330815]]], [[[1.1018062843950727, 1.670371122289712, 4.404480819919848, -0.94819525655891823, -0.069554481696155968], [0.21849248255859433, -2.0040385144105812, 0.27930847779667456, -3.4949443431975382, -1.7350238397127553], [-3.9064017513253884, -2.7399229250098491, 1.1472864971036332, 1.3680963545876175, 1.2617331217927346], [-2.1215702857236707, -0.98350827466526169, -2.288793391463722, -3.3901294489458458, 2.809249768724329]], [[4.0516149464203766, -2.9901428076962526, -2.8171642932813636, -2.1414243478052146, -0.9038088298657776], [2.2079807841895303, 0.33151901312369514, -3.7540878143719971, 0.10633183861071505, 2.2458778544247933], [-0.90096708982870322, 0.58748506023854663, -3.5105161782913479, -0.1334192311912572, 1.7068192047760116], [1.3346644899285742, -5.0702993455964682, -3.7411091081895078, -0.53425791434154757, 0.60949106054275415]], [[3.7287730963280907, 3.7872788739167369, -1.2422283746987213, 1.9578314756446638, -0.77948312189426971], [-2.6106925021744671, 1.8127097945098516, 3.5228809096922475, -4.9314818960444882, 1.9764430686037855], [1.0428793997483563, 0.058467480078602918, 0.28329350753992877, 0.87443691319551142, -3.9964268973760246], [-0.85140723408479158, -2.464028627646488, -2.1949543445908577, 2.2147589881387999, -0.93257148869748097]], [[-1.7226919098704627, -5.6955791275613219, 2.9344256305151371, 2.9797990128820517, -4.3075455330320533], [-0.13893788128284523, -3.8762554767221591, -4.6975239520829346, 2.5758732111791334, -6.200368241322793], [3.4691054159284826, 1.7931665079409571, -1.3663877196809198, 2.6765375587373983, -1.3576108642204243], [-4.9983542687611013, 0.30852741897594083, -0.43742219744300037, -2.6554116657045257, -3.9055881973261561]], [[-6.5479527695024604, -4.4981527684719023, -1.6457229864100729, -6.4598868967270135, -7.2690095829959436], [-2.6924595685477088, -5.4464267685967878, -7.1095370579932124, -8.2874622207383055, -5.8684485511898128], [-0.98056654990591952, -1.9111963102220493, 0.80388600122959097, -3.2853276224024341, -6.6931238301052032], [0.64104503121083134, -4.0683782986632586, 0.38180962289825437, -3.8610371756393977, 0.57157470077753558]]], [[[4.582413126521395, -3.7564642249352813, -3.1203826538574702, -2.9506366092050165, 0.3630379016327252], [-0.78560090204778898, 3.0884976401244213, 4.7852076123386666, -3.8015816736326133, -0.67952403680822604], [4.7565423384893517, -2.6241730911340881, -1.8691231497845067, -3.064218817973094, -1.8005042782374199], [-2.6508475662586246, -0.53204438320920655, 2.5109788542491156, -2.4558582029121649, -2.5121083142109062]], [[1.376053440124946, 0.56599999963316794, -1.8030947124594823, -5.2112686613734249, 2.3349611156737202], [-1.883078140065062, 0.15931903011617266, -2.3677610020511022, -1.8501095617430807, -0.1953513795251931], [3.7433105149891626, 1.1682145681786995, 2.1465884097941688, 2.6849977755512411, -0.81856348902034615], [-2.6477633129414779, -3.4610571768493137, 2.7413372172567092, -3.928503820967244, -2.8388576686680866]], [[-0.99896611921958867, 4.1297975949434411, 0.049231151046726929, 2.0012370015157162, 1.3329653549650962], [5.7732408106475779, -1.1029441158254869, 0.88559461413014784, -3.2902056560322244, -3.9970491520785592], [5.2081557769661329, 0.86022322018916597, -1.1999136411781484, -1.9628990064574428, -3.5540984235689814], [4.251885413943099, -3.7360704879517153, -3.9373310726561304, -1.4058706667542542, -1.2870001444479993]], [[5.0434970192356507, 3.8816181475972824, 5.3597775585974725, -3.3187427820356383, 5.5138663286244212], [-1.1877589112282214, -1.709734806912274, 4.1992616748164222, 1.8432992671468709, 2.3621679876371751], [3.8527817016430559, 4.8573692846071479, -0.80766854543031119, -2.2867378244076018, 2.6959425612551078], [1.8040105156788036, 1.8732991376519648, 4.5147684553150222, 0.26417704105802464, 5.2329312132835577]], [[-2.3831756242101143, -3.5908124721965846, -9.4612437653246477, -8.1398166995372705, -6.4382857037253984], [-5.7268937258388579, -8.651204281729564, -9.79859382233664, -7.6326087962453952, -1.3173252499977242], [-0.75468212260508416, -5.4715307993364819, -9.1515749932580235, -6.311225512661875, -2.7983325018423013], [-0.58877449397810366, -2.8151773680133019, -9.4093391849056935, -6.0545074664066627, -8.3116315845608728]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.3848948999725295, 1.4571320832257584, -4.1950308382993819, 0.84058144155828884, -4.18537853891781], [1.9443322987316769, -2.7619426483147835, -4.1331749327222624, -4.0761393971154325, -0.90977584575042592], [2.9218363020279963, -1.6996003272461655, -0.27147240006714757, 0.99677403870454206, 4.7614584245526146], [-3.027989886143847, 0.2648581514062549, 1.4995046051209835, 0.11271561851469691, -4.9242057776214976]])+(1.-msk_arg0)*numpy.array([[-2.6766733061744983, 1.8666306816310918, 0.90581306191554045, -1.27336118093646, 2.1496615600365772], [-1.2848923342724583, 3.3722831638814341, -2.4991306054931526, -1.90500220610366, 1.449153959697461], [-3.2157599323562258, -1.5148142122438877, -3.6946034964281727, -3.835761362181306, -1.6545558659093551], [3.7742218205427083, -1.0862354085917278, 1.5138617511807109, 0.23329305378641418, 2.5598322489290801]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.8658095083237791, 1.8413279406182852, 0.36151023963533557, 0.046359020572149134, 2.7279301336554713], [1.6241432524840915, 0.6164336731082054, 3.7118783229299659, 2.9821554989629009, -4.2707514380273883], [1.7507934824090041, -1.6644220937330489, -3.702131733028502, -0.31308093285036698, 1.6239058328796752], [-3.5017319704757588, -2.1781381645725273, -1.555598218882297, 0.26977547475276253, -2.9867641740026016]], [[3.1827761878476899, -4.8738150549035772, 2.7428253613242495, -3.0181513719268862, 3.2278401214382377], [-2.724955233909947, -1.1774697732377906, 3.3247985321495275, -3.3679616703546578, 0.053197177052057398], [1.3563308340928701, -1.3421773582463303, -0.78304755439324225, 1.1257576392224937, 0.44427039521280332], [-1.0090721078677181, -0.50769998607061595, 4.986140302216528, 2.2999465462604407, -4.5021834363655682]], [[-3.0312179668764183, -4.0551864026474416, 2.4431459104738682, 3.0335801791529455, -1.2044733103645342], [-0.49505451269981204, 1.7050727849176841, -4.8068922990051641, -1.6480411491275215, -1.7404213907955657], [-3.9269637185862671, -0.38855174683340099, 0.45551506656881635, 2.2813922537518483, 4.9511457376024222], [-0.17616612528379605, 1.773775285508588, -2.8438938059789498, 1.643784662755162, -0.33932134819963444]], [[1.5897256970445142, 2.9016310355613539, 1.0653005170275867, -0.86995451140413316, 2.5006543419960492], [-0.25088972177194879, 1.9196033149824467, -4.4263814791263254, -0.61446476348328805, -3.5871567078383459], [-3.5528066455336402, -4.2838483056797951, -2.2585271450846376, 0.32722705551881415, 4.8657104517672085], [0.63893829937281232, 0.16013291177393718, -2.8817767000888885, 1.4431791579628239, 1.9161581306455941]], [[0.38482769385996285, 1.5935511308632853, 0.72739628250345767, 2.7669156756868514, 2.3852721339853913], [1.4841120196222244, -4.0545084833434641, -1.6225847341955637, 3.383644574201135, 1.2660472980846222], [0.44080707008885867, -1.6790901569839001, -2.3954508671201822, -1.9777090858674997, -0.84511876373575934], [-0.69160614477174054, -3.9957630304868683, -0.042209796503746233, -4.4985071994543411, -4.6992600292698601]]], [[[-1.7313847957867559, 0.034259204062073501, -4.8815156742288268, 0.80550903302396026, -0.40310411172850191], [-4.0738412183912711, -2.9634909497018889, 4.2609714201536768, -3.0864733025163749, -1.0704797862604911], [-1.5131557071251689, 3.9531654746848837, 0.31855967029349941, 3.3510528119840899, -1.1377497837283035], [0.95907383601988982, 4.2302149364527928, -3.6156007633824352, -0.31636802147685472, -2.0505618046116614]], [[1.0381944878894975, -3.5350071135508077, 3.0396479047332683, -1.3572114064076759, -3.1010700127413928], [1.2902030395844752, 4.8962393715547918, 1.1380450251633976, -3.1426093047215797, 3.8242487132851704], [-0.063229387824669914, 2.5943852626257993, 2.2337721310668703, -3.6477082723347589, 0.90219064946646466], [4.7502433818678576, 4.6882667061144385, 1.0430685164579785, -2.0341552655900461, 1.6475220121883289]], [[4.7818094768085864, -2.3928427567135682, -1.5081733037146581, -0.90191258963386556, 2.824653645705947], [3.3842567322054489, -2.5563153404674299, 0.76504920245256347, 3.1194808753271435, 0.53868577062418943], [-0.45244685032478316, 0.56693273941696187, -4.5754692954022671, -3.8806347782604869, -4.649083756699504], [0.13251407468526999, -4.4356225961189777, 0.37650787045470402, -2.0398307448579169, -3.0004144360982767]], [[4.897940816491916, -0.20559773884905397, -4.8383669924930715, -3.7667133675748579, -3.3699242760784367], [-3.0622213004193743, 3.5310580319249514, -4.6179177654500423, 2.1818172947391403, 4.9941368891154259], [-2.2761211630544231, -3.8188204913225721, -1.52153557188793, -4.6616556594024701, 4.0852173987381217], [4.6170441556623238, 2.3927336121776417, 2.8558332579721499, -4.1203967850334067, -1.0121748687526999]], [[-1.7149128046097974, 3.3152150219386414, 3.2593980072988789, -0.82304331145933052, -3.947335683468749], [1.1843416417285182, -0.17468778737989865, -2.139891971642661, 4.3350999145076479, -4.5858622187077689], [4.5666474938085191, -2.198428572030644, 2.8496780838381275, -1.0136968581211123, 1.2182118024488169], [-0.31170063133592052, 0.11301175607327174, -1.3072252406309239, 4.443355434703177, -0.018575076672394886]]], [[[-4.5731587069414061, 2.6077161344196398, -0.2426240449575694, -4.4221653836665187, -3.2036059755866644], [3.8118568146746661, 0.89401158819541049, -4.4999756951434398, -2.4062275247577292, 1.8738662616685549], [2.5311586497005205, -3.4828472083029149, -3.7454447637363608, 4.6345465220276587, 2.4301350502077206], [0.24656283217504349, 1.0885436512593403, -0.78427824314066896, 3.4863852888808555, -4.1034369868463605]], [[3.6546921503123215, 0.88599677301717161, -0.75436169751399795, 2.2690984045056561, 2.7864878568646674], [3.0947788555324145, 4.630932202047088, 2.5147952643459162, 0.39261377748696891, -3.0886637385445925], [2.2090336411352816, -1.5154277532031193, 2.0121683360558009, -2.6133470315575256, 2.112176422273599], [-0.68406060322782825, -1.6718407439557348, -2.4181425153162071, 4.6005018884409008, 0.47117868425392295]], [[4.4790989441241056, 0.016595263030300345, -3.9613069018933289, -2.1721424806449932, 1.4031123569844537], [-3.802665239786621, -1.9721570416557221, 0.5669225365489492, 3.4567888354202694, 2.9338345858976966], [4.3954724311878284, -0.8205927673175939, -3.4497983781147901, 4.5097201242925493, 4.5540907493568294], [3.2011821095404862, 1.7387779802881775, -3.3804036873969689, -2.2207020175634149, -4.3715411233172947]], [[-4.3066034992850994, -3.6126005292479668, -4.7579918355405573, 1.8530836250552971, 4.9696665598872993], [3.984547618477718, -0.59933058858207655, 1.3954706687332159, -1.9233626305035312, -3.7844889174540275], [-4.7252860530104277, 1.2170139053467421, -0.45918433405649139, -1.9116951745761277, -3.3228032513855368], [-1.4492523883446049, 2.9446829299232666, -1.8123706713902044, 1.6408935209914848, 2.5540608121082453]], [[2.6736720766056674, -4.2980561677270632, 1.8869848899645199, 2.3188200008627131, 1.7944225578362758], [-3.7055370185527714, -4.482609777281743, -3.4267898335501332, 2.7270197433767436, 1.7841822214995595], [4.1332606466109674, -2.202648661821919, -0.82199971658441129, 0.51552422335752546, 2.5839984781240979], [2.6456296215258828, -2.2033726329165324, 3.9514220746317257, -0.56195837385262237, 3.9142332290067845]]], [[[-0.55186367169218897, -0.36096242228985886, -4.9730566326909926, -4.9371201728682532, 2.9229851325180887], [4.9954288252512331, 3.4580175599482583, -2.1786596319053411, -2.9872944321039627, -3.0721545360524805], [-0.11469680692842932, -3.2448118925450711, -4.7053894403951553, -4.4166789647786766, 2.1996867213674962], [1.8155307263907083, -1.1986673080573684, -4.2196050208298761, -0.50507677897233627, -3.4365548532943349]], [[-0.77366516838687183, -4.909296402749753, 4.6999584779559775, 0.15238719920066046, 4.0409612338497709], [-2.1880284276476445, -3.3851122610744788, -1.4732743497581891, -2.4725802766499294, 1.9705590630101781], [2.8329420042808495, -1.2302305652533754, -4.5572569564043564, 2.177494191468031, 4.9273832595252287], [4.387345993709836, -1.207614133991691, -2.9450710151144199, 1.5510238761387036, 3.4326949681292884]], [[-3.3868739299958071, 0.55474485093780856, 3.2638388563901675, -3.9149091348246055, -3.393786103521983], [1.3700008726984505, 0.93404819079671153, 2.3986828145906216, 0.6718810828176327, 0.66941178393525291], [2.3311756212709973, -1.1873868724256251, 1.9834550007919809, -2.8613066585458249, -1.7749493164594385], [2.3115902640282684, -4.1878408168581185, -3.7418021554036365, -4.4390371480216659, 3.4888829303408926]], [[1.2141798329387301, 2.8601622733932786, 4.2572995603419006, -1.1705557543794241, 3.9817578878047328], [-2.6396924285066081, -4.7090092054289014, -1.6098477166578884, -2.3408675650187316, 2.4225891596487612], [-4.4644428294466092, -3.8370334093929315, -1.0203189648699507, -1.789884320233639, -0.84361625782527661], [-1.8876633796509124, -0.42607542659274422, -0.69788242047517546, -0.48682235045476752, -4.4451014331981948]], [[-3.4555859365003294, 0.3228648137678336, 2.8559118414794522, -1.7546599851376756, 3.9651618887181517], [3.1794816221764837, 2.0561662983799174, -0.10481768041632034, -3.1151913621471583, 3.207582270474779], [1.5591930047583178, -3.847737922895079, 0.37095893525177814, 2.6066881339993042, -0.73025390099745291], [-0.87888482036442106, 1.5814326457974284, 1.6857433693299759, 3.6889508138370672, 2.0749466441013826]]]])+(1.-msk_arg1)*numpy.array([[[[3.0134647671532182, -2.6459075027037526, -1.6092541030223586, 2.4255793630824751, -4.7643023050507001], [-1.8236857460251521, 4.6008394792388323, -0.85464756419812993, -4.7765328751757794, 4.4840497800129757], [-2.0523028626194884, -2.8056906917749602, 3.5673027807782152, -0.98449986215177088, 1.0445085576850008], [-0.66658878171497626, -2.1858129397294821, -1.3367506273766439, 3.1501686112298106, 1.089615269227985]], [[2.8450468169967902, 1.0358519027097142, -0.68947673971140055, -0.043317877824062379, 3.9991985563810086], [2.2403347735245873, -2.8887538840868787, 4.1002366990190104, 2.0523190087023471, -0.72251149815804805], [-1.2137150913485693, -1.2494596213692013, 0.31069729075281494, -0.45820075435321872, 1.5446737730917377], [-3.3048531626010202, 1.8697839793079671, -1.5313017070486836, 2.2926244630923218, 4.8241339110430417]], [[-4.6854618985026422, -2.543743398357432, -4.3710675360150582, 3.8905367529479609, -0.47477613001877561], [-0.80379999078144326, 3.1655367463944657, -0.45366712032517498, 4.7355688253227903, 4.4754150822880696], [0.86810077444941491, 3.047841820570822, -3.4651610594298079, 0.0031234153599903891, -1.1588211839372131], [-0.030197811546597286, 3.6080746423472458, -0.98271778284505018, -0.070948627492098737, 2.3635071549652231]], [[-3.3639595864501191, -3.9169162657684176, -4.347373778406701, 3.6561829785028728, -2.4233662317005065], [4.7721028546895798, -4.9281191175460819, 1.0869830124754376, -4.5549251908402226, 1.5255200807755749], [2.0001260559310596, 4.2245950796225049, 1.5063795587090656, 3.3023972763049656, 3.9405186970948076], [-0.054375283192850787, 2.8082905126514612, -1.6419115859986122, -3.41920274542891, -2.3144550670166431]], [[3.9794134648893724, -3.700691029946368, -3.3198933800786223, 3.5858075253813997, -2.5502740020629022], [-3.3719714231027931, 0.033977689148101398, 4.8092273715628142, 3.5708735748809755, 4.097127906380674], [0.98362137706485608, 0.34470843126653783, 4.494207584725757, -0.92543111527651156, 4.9927321661405397], [1.986008996106527, -2.742671217701723, -3.3951560333643371, 3.1154193428497816, 4.3621813726935201]]], [[[3.6671141654638699, 3.4681938806392996, 2.6393643989483877, -2.2003672189265142, 3.3198810699203989], [-0.97486560822575452, -3.6597137462491225, 1.2894962625188997, -2.8862779908507843, 1.5188239253208131], [4.9405805410879928, 4.089568860089706, 1.3611417678544679, -1.2448228149273421, -0.040591771303795987], [-0.97953401233117532, 1.4465149887028748, 3.7432731250869242, 4.4616670202811299, 3.3540309545646689]], [[3.9297285032144966, -3.3431077206828519, 3.4957687420632375, -1.0167981308020768, 2.0981672428971221], [-4.1377849477519435, 3.4995317167532356, -1.075122816667716, 3.7885346303548477, -2.2391636576942298], [3.0422175565532186, 3.4582709331371593, 0.83366157155980858, 0.050266531719068475, -0.95247811651992187], [3.7614900391133368, 1.8932888355411137, 0.68277177820277579, 3.0889268089402524, 2.4389046997728761]], [[-0.24397393120861111, 0.8027327579591006, -1.9024685651423781, 3.2530000804884871, 0.96295084950020726], [-4.7333654160141689, 3.6056580468845461, 0.72020790284910241, 2.5049077532715955, -2.8587436587768424], [-0.59569744566841809, -4.3792950691405652, 2.4737491300491179, -1.8595088922748793, 3.1383140908329317], [-1.8360480209471266, 3.0000895816965425, 2.0541125080214426, 3.5389866086452315, 0.70596240975408531]], [[-0.072228833161302575, -2.2882378366281007, 2.1029214781386614, -2.2787737706212607, 1.9603667920513796], [-1.8239367778116478, 0.48558606734564158, -0.54639308631168415, -3.6872848898353494, -0.20652632595480203], [-4.0635953483747356, -3.3242168224933568, -4.9661747423290228, -4.1678047866052133, 2.8986578580750875], [-4.110030907112149, 0.28010674346348807, 3.5842616444769462, -4.7466724736209454, 2.8730020215006693]], [[0.036495917476642603, 3.4292087465713408, 0.71075665535568611, 0.048095488447409629, -0.52567644167724303], [2.9728162172962067, -1.7017549119326612, -0.0057240455551763603, -2.595222512410845, 1.2672352019731656], [-0.96994578184051505, -4.5519145455022469, -1.6436906871889723, -4.0942057805527687, -0.23087708578675947], [-3.5155840039607353, 3.497028366677732, -1.1242040154483233, -1.0378275100307368, -1.7125222068335644]]], [[[2.2952912946202257, 2.4913478354108136, -4.8980040575133419, 1.7619756527045674, 3.2231340876615526], [-2.8555158394067623, 0.24967532735982445, -3.3346692104294196, -0.0076936294395180127, -0.18562070614609549], [0.2141763575212865, -3.7234682069815452, 0.1154472413533858, 3.5221132786158886, 2.1554294565862548], [-0.78062070642551618, 4.0938368983962405, -1.0827446683779005, 1.9762650624281068, 0.91111292640693176]], [[2.7638191029434456, -3.9196393370436935, 3.2943754985365885, -4.0252707223711957, -2.5069633995413296], [0.37704137814224481, -0.27228263581477119, -0.76060137232873526, -2.9048282057682604, 2.4899336033485175], [-1.8676799131464095, -0.33148990672991818, -2.8329741678893718, -1.8563147237754549, 1.7225045297078045], [2.2215724833115713, 0.95283064861173195, -4.4070387398812736, 3.1992431690950216, 2.7353490967033967]], [[-4.9692286980796467, -1.4605473120380221, -1.2334818584541205, 0.9718054203211004, -2.0007231499346432], [1.8244366342394134, -4.509231236215248, -1.082412023941457, -1.7223675184001266, 2.103852334255178], [-1.3696653346920282, -4.7384657973160618, 4.0242321664717249, 1.9819377925066632, 0.82666613591234306], [-0.95829373081027569, -4.2503197145809821, -3.1051714377562725, 0.33215081931900503, 0.40996964760069332]], [[3.2127504957390265, 3.2899638072713131, -1.7647834524181527, -1.9820641646387607, 1.6609057040187061], [-4.3926835384995471, 1.8542918703279963, -1.4556813653257517, -4.2025938434617851, 4.1148670149524733], [0.82938698814517942, -2.8828690864502016, -1.5575767210103777, 2.0949507944803605, 4.7961938585489214], [-0.52494125389340951, -0.48249927893011968, -0.4859274931890134, -0.54124762512328584, 0.83377560097487979]], [[1.9011549563764882, 4.8911832542478777, -0.17450334061414274, -2.0845650857641376, -2.6850192466562737], [-3.1436974949781051, 3.3753734249249838, 3.9136144834832791, -3.0819461168610776, -0.00080016833737950321], [4.1387002158688411, -0.18735639164424001, 2.5493441068443987, 0.0052225759012882733, -2.1295373115382379], [1.5919962180530858, -0.031519055220766035, -4.8426391560748296, -3.058027862498701, -4.3564473125914924]]], [[[0.21464498771587071, -2.9494872789984372, -4.9727137886849064, 3.4466087422293423, -2.826687496367235], [-3.4232472255675575, 0.75562815023473107, 3.1130477400578211, 3.1220412578269183, -2.181334622387455], [0.43700876980539149, 1.3822526857998998, -3.7293325040526968, 3.173113150875885, 0.85589074079640604], [-0.89242621542436673, -0.27600507595844004, 2.5991477724381831, 4.6363174251016304, 2.764647870999517]], [[3.1650084397026106, -1.751316173531495, -3.0727767082556734, -2.0343060624943563, 4.4004468240914232], [2.7327718898366173, 2.4483225192828328, -1.5451814323108612, 0.42003864141621072, -2.9088379380691096], [1.4883702669501897, -0.34099431107498646, 4.7821535272021443, -0.03430381149360251, 3.745818578255383], [-3.9541437306870177, -2.8084977149252053, -1.7448255882592631, 2.9920503698486822, -2.18374304852476]], [[-4.6618922212490066, 0.062622947695278874, -1.757795337121081, -1.1357562283771672, 4.752970291749957], [3.008974217574492, 1.8326147396738923, -1.6250432216429189, 2.6773408536105725, -0.15963485010417067], [-2.6737969082022683, -2.7855120354325749, 0.55466826573788275, 2.9397780770126136, 1.2530248449228578], [2.8645007918085206, 2.5438372036278425, 0.43156074352426188, 2.9824840021475465, -0.59562334381422843]], [[-3.7348743594081348, 2.8232615632622871, 1.9566148322129875, -2.4406466156235815, -1.3112444307773661], [4.7780336241216794, -0.21046189415833716, 0.35297522285632432, 3.7562349141188438, -0.79181337016574016], [-3.5721543049708671, -2.751867894833083, -1.174016331950817, 0.72990411869202099, -0.86927490602337976], [-3.1841543793727434, -1.9367464672717851, -2.7065663077618174, 2.8534195062296783, -1.0236534292408681]], [[-3.2130418490704726, 3.5751605449717268, 4.4401154424307077, -4.8432008923149361, 4.8221878824102369], [3.7051111482900687, -1.5019636866959485, -3.5974851128700438, 0.39607865445258827, -2.6349087337112853], [3.8339606121917598, -3.2561232098868342, -2.1470953167312334, 1.0746926316395804, -4.9853347696695565], [0.21852922161116428, 4.2067712246570714, -0.0099952665058662404, 0.15710826748923967, -3.7491649431441312]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.2507044082963086, 3.2262228405908147, 1.746405139607865, 1.4312539205446786, 4.1128250336280008], [3.009038152456621, 2.0013285730807349, 5.0967732229024953, 4.3670503989354303, -2.8858565380548589], [3.1356883823815336, -0.27952719376051949, -2.3172368330559725, 1.0718139671221625, 3.0088007328522046], [-2.1168370705032293, -0.79324326459999783, -0.17070331890976753, 1.654670374725292, -1.6018692740300722]], [[4.6399082710734483, -3.4166829716778189, 4.1999574445500079, -1.5610192887011278, 4.6849722046639961], [-1.2678231506841886, 0.27966230998796782, 4.7819306153752859, -1.9108295871288994, 1.5103292602778158], [2.8134629173186285, 0.11495472497942805, 0.67408452883251613, 2.5828897224482521, 1.9014024784385617], [0.44805997535804032, 0.94943209715514243, 6.4432723854422864, 3.7570786294861991, -3.0450513531398098]], [[-7.2262488051757998, -8.2502172409468244, -1.7518849278255137, -1.1614506591464364, -5.3995041486639161], [-4.690085350999194, -2.4899580533816978, -9.0019231373045461, -5.843071987426903, -5.9354522290949472], [-8.1219945568856495, -4.5835825851327829, -3.7395157717305656, -1.9136385845475337, 0.75611489930304021], [-4.371196963583178, -2.421255552790794, -7.0389246442783318, -2.55124617554422, -4.5343521864990164]], [[2.430307138602803, 3.7422124771196428, 1.9058819585858755, -0.029373069845844313, 3.341235783554338], [0.58969171978634005, 2.7601847565407356, -3.5858000375680366, 0.2261166780750008, -2.7465752662800571], [-2.7122252039753514, -3.4432668641215063, -1.4179457035263487, 1.167808497077103, 5.7062918933254974], [1.4795197409311012, 1.000714353332226, -2.0411952585305997, 2.2837605995211128, 2.756739572203883]], [[-3.8005508450578471, -2.5918274080545247, -3.4579822564143523, -1.4184628632309586, -1.8001064049324187], [-2.7012665192955856, -8.239887022261275, -5.8079632731133737, -0.80173396471667502, -2.9193312408331877], [-3.7445714688289513, -5.86446869590171, -6.5808294060379922, -6.1630876247853097, -5.0304973026535693], [-4.8769846836895505, -8.1811415694046783, -4.2275883354215562, -8.6838857383721511, -8.8846385681876701]]], [[[0.21294750294492104, 1.9785915027937504, -2.9371833754971499, 2.7498413317556372, 1.541228187003175], [-2.1295089196595942, -1.019158650970212, 6.2053037188853537, -1.142141003784698, 0.87385251247118578], [0.43117659160650801, 5.8974977734165606, 2.2628919690251763, 5.2953851107157668, 0.80658251500337341], [2.9034061347515667, 6.1745472351844697, -1.6712684646507583, 1.6279642772548222, -0.10622950587998448]], [[-1.723748160425286, -6.2969497618655907, 0.27770525641848476, -4.1191540547224594, -5.8630126610561764], [-1.4717396087303083, 2.1342967232400083, -1.623897623151386, -5.9045519530363633, 1.0623060649703868], [-2.8251720361394534, -0.16755738568898426, -0.5281705172479132, -6.4096509206495424, -1.8597519988483189], [1.988300733553074, 1.926324057799655, -1.718874131856805, -4.7960979139048296, -1.1144206361264546]], [[0.64863454408632393, -6.5260176894358306, -5.6413482364369205, -5.035087522356128, -1.3085212870163154], [-0.74891820051681357, -6.6894902731896924, -3.368125730269699, -1.013694057395119, -3.594489162098073], [-4.5856217830470456, -3.5662421933053006, -8.7086442281245304, -8.0138097109827484, -8.7822586894217665], [-4.0006608580369925, -8.568797528841241, -3.7566670622675584, -6.1730056775801794, -7.1335893688205392]], [[0.82180141937648354, -4.2817371359644865, -8.9145063896085048, -7.8428527646902904, -7.4460636731938692], [-7.1383606975348073, -0.54508136519048112, -8.6940571625654748, -1.8943221023762922, 0.91799749199999336], [-6.3522605601698556, -7.8949598884380041, -5.5976749690033625, -8.7377950565179034, 0.0090780016226892357], [0.54090475854689135, -1.6834057849377908, -1.2203061391432826, -8.1965361821488401, -5.0883142658681324]], [[-2.6246886503602234, 2.4054391761882155, 2.3496221615484529, -1.7328191572097564, -4.8571115292191749], [0.27456579597809228, -1.0844636331303246, -3.0496678173930869, 3.425324068757222, -5.4956380644581948], [3.6568716480580932, -3.1082044177810699, 1.9399022380877016, -1.9234727038715382, 0.30843595669839097], [-1.2214764770863464, -0.79676408967715417, -2.2170010863813499, 3.533579588952751, -0.9283509224228208]]], [[[-1.6513224049134099, 5.5295524364476361, 2.6792122570704269, -1.5003290816385224, -0.28176967355866811], [6.7336931167026624, 3.8158478902234068, -1.5781393931154435, 0.5156087772702671, 4.7957025636965511], [5.4529949517285168, -0.56101090627491867, -0.82360846170836455, 7.556382824055655, 5.3519713522357169], [3.1683991342030398, 4.0103799532873365, 2.1375580588873273, 6.4082215909088518, -1.1816006848183642]], [[1.955091823066156, -0.81360355422899389, -2.4539620247601635, 0.56949807725949064, 1.0868875296185019], [1.395178528286249, 2.9313318748009225, 0.81519493709975066, -1.3069865497591966, -4.7882640657907576], [0.50943331388911606, -3.2150280804492848, 0.31256800880963542, -4.3129473588036911, 0.41257609502743353], [-2.3836609304739937, -3.3714410712019003, -4.1177428425623726, 2.9009015611947353, -1.2284216429922425]], [[4.207626544056958, -0.25487713703684722, -4.2327793019604769, -2.4436148807121407, 1.1316399569173061], [-4.0741376398537685, -2.2436294417228697, 0.29545013648180163, 3.1853164353531218, 2.6623621858305491], [4.1240000311206808, -1.0920651673847415, -3.7212707781819376, 4.2382477242254017, 4.2826183492896819], [2.9297097094733386, 1.46730558022103, -3.6518760874641165, -2.4921744176305625, -4.6430135233844423]], [[-3.3098294605805574, -2.6158264905434248, -3.7612177968360152, 2.8498576637598392, 5.9664405985918414], [4.9813216571822601, 0.39744345012246551, 2.392244707437758, -0.92658859179898911, -2.7877148787494854], [-3.7285120143058856, 2.2137879440512842, 0.53758970464805067, -0.91492113587158563, -2.3260292126809947], [-0.45247834964006284, 3.9414569686278087, -0.81559663268566229, 2.6376675596960268, 3.5508348508127874]], [[7.435130501158282, 0.46340225682555136, 6.6484433145171344, 7.0802784254153277, 6.5558809823888904], [1.0559214059998432, 0.27884864727087155, 1.3346685910024814, 7.4884781679293582, 6.5456406460521741], [8.894719071163582, 2.5588097627306956, 3.9394587079682033, 5.27698264791014, 7.3454569026767125], [7.4070880460784974, 2.5580857916360822, 8.7128804991843403, 4.1995000506999922, 8.6756916535593991]]], [[[-3.579853557836036, -3.3889523084337059, -8.0010465188348405, -7.9651100590121002, -0.10500475362575834], [1.9674389391073861, 0.43002767380441131, -5.2066495180491881, -6.0152843182478097, -6.1001444221963279], [-3.1426866930722763, -6.2728017786889181, -7.7333793265390023, -7.4446688509225236, -0.8283031647763508], [-1.2124591597531387, -4.2266571942012154, -7.2475949069737231, -3.5330666651161833, -6.4645447394381819]], [[-0.50880701698061692, -4.6444382513434981, 4.9648166293622324, 0.41724535060691537, 4.3058193852560258], [-1.9231702762413896, -3.1202541096682239, -1.2084161983519341, -2.2077221252436745, 2.235417214416433], [3.0978001556871044, -0.96537241384712047, -4.2923988049981014, 2.4423523428742859, 5.1922414109314836], [4.6522041451160909, -0.94275598258543614, -2.680212863708165, 1.8158820275449585, 3.6975531195355433]], [[-1.8873693248748236, 2.0542494560587921, 4.763343461511151, -2.415404529703622, -1.8942814984009995], [2.869505477819434, 2.433552795917695, 3.8981874197116051, 2.1713856879386162, 2.1689163890562364], [3.8306802263919808, 0.31211773269535836, 3.4829596059129644, -1.3618020534248414, -0.27544471133845505], [3.8110948691492519, -2.688336211737135, -2.242297550282653, -2.9395325429006824, 4.9883875354618761]], [[1.326895451453427, 2.9728778919079755, 4.3700151788565975, -1.0578401358647271, 4.0944735063194297], [-2.5269768099919112, -4.5962935869142045, -1.4971320981431915, -2.2281519465040347, 2.5353047781634581], [-4.3517272109319123, -3.7243177908782346, -0.90760334635525375, -1.6771687017189421, -0.73090063931057969], [-1.7749477611362154, -0.31335980807804731, -0.58516680196047854, -0.3741067319400706, -4.3323858146834979]], [[-8.3797917141218274, -4.601340963853664, -2.0682939361420454, -6.6788657627591732, -0.95904388890334591], [-1.7447241554450139, -2.8680394792415802, -5.0290234580378179, -8.0393971397686563, -1.7166235071467186], [-3.3650127728631798, -8.7719437005165766, -4.5532468423697194, -2.3175176436221934, -5.6544596786189505], [-5.8030905979859186, -3.3427731318240692, -3.2384624082915217, -1.2352549637844303, -2.849259133520115]]]])+(1.-msk_ref)*numpy.array([[[[0.33679146097871993, -5.3225808088782509, -4.2859274091968569, -0.25109394309202315, -7.4409756112251983], [-4.5003590521996504, 1.924166173064334, -3.5313208703726282, -7.4532061813502777, 1.8073764738384774], [-4.7289761687939862, -5.4823639979494585, 0.89062947460371689, -3.6611731683262692, -1.6321647484894974], [-3.3432620878894745, -4.8624862459039804, -4.0134239335511417, 0.47349530505531234, -1.5870580369465133]], [[4.711677498627882, 2.902482584340806, 1.1771539419196912, 1.8233128038070294, 5.8658292380121004], [4.1069654551556791, -1.0221232024557869, 5.9668673806501022, 3.9189496903334389, 1.1441191834730438], [0.65291559028252255, 0.6171710602618905, 2.1773279723839067, 1.4084299272778731, 3.4113044547228295], [-1.4382224809699284, 3.7364146609390589, 0.33532897458240818, 4.1592551447234136, 6.6907645926741335]], [[-3.7796488365871017, -1.6379303364418916, -3.4652544740995177, 4.7963498148635013, 0.43103693189676484], [0.10201307113409719, 4.0713498083100061, 0.45214594159036547, 5.6413818872383308, 5.3812281442036101], [1.7739138363649554, 3.9536548824863624, -2.5593479975142674, 0.90893647727553084, -0.25300812202167267], [0.87561525036894317, 4.5138877042627863, -0.076904720929509729, 0.83486443442344171, 3.2693202168807636]], [[-4.6373207673865791, -5.1902774467048776, -5.620734959343161, 2.3828217975664128, -3.6967274126369665], [3.4987416737531198, -6.2014802984825419, -0.1863781684610224, -5.8282863717766826, 0.25215889983911488], [0.72676487499459963, 2.9512338986860449, 0.23301837777260559, 2.0290360953685056, 2.6671575161583476], [-1.3277364641293108, 1.5349293317150012, -2.9152727669350722, -4.69256392636537, -3.5878162479531031]], [[6.1290750249259496, -1.5510294699097908, -1.1702318200420452, 5.7354690854179768, -0.40061244202632507], [-1.2223098630662159, 2.1836392491846786, 6.9588889315993914, 5.7205351349175526, 6.2467894664172512], [3.1332829371014332, 2.494369991303115, 6.6438691447623341, 1.2242304447600656, 7.1423937261771169], [4.1356705561431042, -0.5930096576651458, -1.24549447332776, 5.2650809028863588, 6.5118429327300973]]], [[[2.3822218311914116, 2.1833015463668413, 1.3544720646759294, -3.4852595531989725, 2.0349887356479406], [-2.2597579424982128, -4.9446060805215808, 0.0046039282464414022, -4.1711703251232422, 0.23393159104835481], [3.6556882068155345, 2.8046765258172477, 0.07624943358200964, -2.5297151491998005, -1.3254841055762543], [-2.2644263466036336, 0.16162265443041646, 2.4583807908144659, 3.1767746860086716, 2.0691386202922106]], [[7.3020116670959307, 0.029175443198582229, 6.8680519059446716, 2.3554850330793573, 5.4704504067785562], [-0.76550178387050938, 6.8718148806346697, 2.2971603472137181, 7.1608177942362818, 1.1331195061872044], [6.4145007204346527, 6.8305540970185934, 4.2059447354412427, 3.4225496956005026, 2.4198050473615123], [7.1337732029947709, 5.2655719994225478, 4.0550549420842099, 6.4612099728216865, 5.8111878636543102]], [[-2.7431045367017637, -1.696397847534052, -4.4015991706355306, 0.75386947499533452, -1.5361797559929453], [-7.2324960215073215, 1.1065274413913935, -1.7789227026440502, 0.0057771477784429592, -5.357874264269995], [-3.0948280511615707, -6.8784256746337178, -0.025381475444034685, -4.3586394977680314, 0.63918348533977909], [-4.3351786264402792, 0.50095897620338992, -0.44501809747170995, 1.0398560031520789, -1.7931681957390673]], [[-1.9772310392649626, -4.1932400427317607, 0.19791927203500137, -4.1837759767249203, 0.055364585947719647], [-3.7289389839153078, -1.4194161387580184, -2.4513952924153442, -5.5922870959390094, -2.111528532058462], [-5.9685975544783956, -5.2292190285970168, -6.8711769484326828, -6.0728069927088733, 0.99365565197142747], [-6.015033113215809, -1.6248954626401719, 1.6792594383732862, -6.6516746797246054, 0.9679998153970093]], [[1.4856498771741036, 4.8783627062688018, 2.1599106150531471, 1.4972494481448706, 0.92347751802021794], [4.4219701769936677, -0.25260095223520018, 1.4434299141422846, -1.146068552713384, 2.7163891616706266], [0.47920817785694592, -3.102760585804786, -0.1945367274915113, -2.6450518208553078, 1.2182768739107015], [-2.0664300442632744, 4.946182326375193, 0.3249499442491377, 0.41132644966672416, -0.26336824713610341]]], [[[-0.92046863773600007, -0.7244120969454122, -8.1137639898695681, -1.4537842796516585, 0.0073741553053268127], [-6.0712757717629877, -2.9660846049964014, -6.5504291427856458, -3.2234535617957438, -3.4013806385023213], [-3.0015835748349393, -6.9392281393377715, -3.10031269100284, 0.30635334625966282, -1.060330475769971], [-3.996380638781742, 0.87807696604001473, -4.2985046007341268, -1.239494869928119, -2.3046470059492941]], [[1.2490048906995579, -5.4344535492875812, 1.7795612862927008, -5.5400849346150833, -4.0217776117852173], [-1.1377728341016429, -1.7870968480586589, -2.2754155845726229, -4.4196424180121481, 0.97511939110462986], [-3.3824941253902971, -1.8463041189738059, -4.3477883801332595, -3.3711289360193426, 0.20769031746391686], [0.70675827106768363, -0.56198356363215574, -5.9218529521251613, 1.6844289568511339, 1.220534884459509]], [[-8.6638321945078189, -5.1551508084661943, -4.9280853548822936, -2.7227980761070723, -5.6953266463628154], [-1.8701668621887593, -8.2038347326434202, -4.7770155203696296, -5.4169710148282988, -1.5907511621729946], [-5.0642688311202004, -8.433069293744234, 0.32962867004355223, -1.7126657039215094, -2.8679373605158296], [-4.6528972272384479, -7.9449232110091543, -6.7997749341844447, -3.3624526771091676, -3.2846338488274793]], [[-0.62301086644227954, -0.54579755490999293, -5.6005448145994592, -5.8178255268200667, -2.1748556581626], [-8.2284449006808522, -1.9814694918533098, -5.2914427275070572, -8.0383552056430911, 0.27910565277116728], [-3.0063743740361266, -6.718630448631508, -5.3933380831916837, -1.7408105677009456, 0.96043249636761541], [-4.3607026160747155, -4.3182606411114257, -4.3216888553703194, -4.3770089873045919, -3.0019857612064262]], [[0.24659909046713313, 3.2366273883385226, -1.8290592065234978, -3.7391209516734927, -4.3395751125656288], [-4.7982533608874602, 1.7208175590156287, 2.259058617573924, -4.7365019827704327, -1.6553560342467346], [2.484144349959486, -1.8419122575535951, 0.89478824093504361, -1.6493332900080668, -3.784093177447593], [-0.062559647856269329, -1.6860749211301211, -6.4971950219841847, -4.7125837284080561, -6.0110031785008475]]], [[[3.988866808258579, 0.82473454154427106, -1.1984919681421982, 7.2208305627720506, 0.94753432417547323], [0.35097459497515082, 4.5298499707774393, 6.8872695606005294, 6.8962630783696266, 1.5928871981552533], [4.2112305903480998, 5.1564745063426081, 0.044889316490011488, 6.9473349714185932, 4.6301125613391143], [2.8817956051183415, 3.4982167445842682, 6.3733695929808913, 8.4105392456443386, 6.5388696915422253]], [[2.0787730311108827, -2.8375515821232229, -4.1590121168474017, -3.1205414710860842, 3.3142114154996953], [1.6465364812448895, 1.362087110691105, -2.6314168409025891, -0.66619676717551712, -3.9950733466608375], [0.40213485835846186, -1.4272297196667143, 3.6959181186104164, -1.1205392200853304, 2.6595831696636552], [-5.0403791392787456, -3.8947331235169331, -2.831060996850991, 1.9058149612569544, -3.2699784571164878]], [[-3.1480304700682957, 1.5764846988759897, -0.24393358594037018, 0.3781055228035437, 6.2668320429306679], [4.5228359687552029, 3.3464764908546032, -0.11118147046220805, 4.1912026047912834, 1.3542269010765402], [-1.1599351570215575, -1.271650284251864, 2.0685300169185936, 4.4536398281933245, 2.7668865961035687], [4.3783625429892314, 4.0576989548085534, 1.9454224947049727, 4.4963457533282574, 0.91823840736648243]], [[-3.5015813056217207, 3.0565546170487012, 2.1899078859994017, -2.2073535618371674, -1.0779513769909519], [5.0113266779080936, 0.022831159628077025, 0.58626827664273851, 3.989527967905258, -0.55852031637932598], [-3.3388612511844529, -2.5185748410466688, -0.94072327816440282, 0.96319717247843517, -0.63598185223696557], [-2.9508613255863292, -1.7034534134853709, -2.4732732539754032, 3.0867125600160925, -0.79036037545445392]], [[-0.65320960014139251, 6.1349927939008069, 6.9999476913597878, -2.283368643385856, 7.382020131339317], [6.2649433972191488, 1.0578685622331316, -1.0376528639409637, 2.9559109033816684, -0.075076484782205188], [6.3937928611208399, -0.69629096095775411, 0.41273693219784668, 3.6345248805686605, -2.4255025207404763], [2.7783614705402444, 6.7666034735861516, 2.5498369824232139, 2.7169405164183198, -1.1893326942150511]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-2.5750061444572294, 1.1269390803408399], [-3.7028235547396902, 2.5853574156215622]], [[-2.8391574593737303, 3.9634235684579338], [3.9961729078083721, -3.9024747227933663]], [[0.83664557793551708, -0.28443069949121913], [2.3165398010799105, 2.9944050936535813]], [[0.33976094146465918, 0.067122452712553127], [-3.2569186703911388, 4.8247074077673648]], [[1.3723477382057281, -0.49610830543928675], [-0.042676732083171132, 3.0848886808027824]], [[3.7541498376067342, 4.4025030518848958], [0.81007318003227624, -2.6346060575157173]]])+(1.-msk_arg0)*numpy.array([[[-4.1710585877026043, -3.0239803281540132], [-4.802374714952629, -2.7051940990234504]], [[3.480569196423172, -1.2596795807140171], [2.1058871034156743, -0.74308058631230267]], [[3.2167833231635203, -0.73830603872340195], [-2.9130024794035969, 0.2019304294075539]], [[-0.49340361093631202, -3.3410236913515856], [2.2244475150390528, -1.7296175767442747]], [[1.3009350224237384, 4.2134157939676129], [-3.3397860203728857, -2.3697469182207764]], [[2.9441333837834405, -3.7338839298150859], [1.5846507352340309, 2.2198524703229525]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.4321981218757234, -0.92865579096130979], [-0.56009910767902404, -0.14249990916773037]], [[1.0053516746737854, 2.111314400927002], [1.3862550096814497, -3.2025445012596778]], [[-1.7211635443338813, 1.3854988369152217], [4.8552856299606102, 4.7287965727598475]], [[-2.9264098563362628, 4.6984798948298216], [-4.6355393701341017, 1.752384410192807]], [[-2.0112793527022719, 0.39501492103323965], [0.57568123915906888, -1.4137840084885047]], [[-4.9348346857419614, 1.8848316549347741], [-2.141917147941006, 1.4441122154926287]]])+(1.-msk_arg1)*numpy.array([[[2.9245987929998085, 0.81507597868785719], [-2.8913791598060588, 2.5522063783282523]], [[2.8857339920737415, 0.48313313676536218], [2.8617147203814932, 3.4972209974548036]], [[-0.61032860945613443, -0.25130934021592211], [-0.17076870898170249, 4.6075257338192639]], [[1.1882511421554822, 1.5876901493326523], [1.7210420008260385, -3.6815408576090638]], [[4.2468312719083539, 1.2747490012947846], [-0.64673578906850793, -3.3383177703738545]], [[3.2550482211973613, 1.9256732628665416], [-0.12802353530624444, 2.7770222213260389]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.85719197741849396, 0.1982832893795301], [-4.2629226624187142, 2.4428575064538318]], [[-1.8338057846999449, 6.0747379693849357], [5.3824279174898217, -7.1050192240530441]], [[-0.88451796639836422, 1.1010681374240026], [7.1718254310405207, 7.7232016664134289]], [[-2.5866489148716036, 4.7656023475423748], [-7.8924580405252405, 6.5770918179601718]], [[-0.63893161449654379, -0.10109338440604709], [0.53300450707589775, 1.6711046723142777]], [[-1.1806848481352272, 6.2873347068196699], [-1.3318439679087297, -1.1904938420230886]]])+(1.-msk_ref)*numpy.array([[[-1.2464597947027958, -2.208904349466156], [-7.6937538747586878, -0.15298772069519817]], [[6.3663031884969135, -0.77654644394865491], [4.9676018237971675, 2.754140411142501]], [[2.6064547137073859, -0.98961537893932405], [-3.0837711883852994, 4.8094561632268178]], [[0.6948475312191702, -1.7533335420189333], [3.9454895158650913, -5.4111584343533385]], [[5.5477662943320922, 5.4881647952623975], [-3.9865218094413937, -5.7080646885946305]], [[6.1991816049808017, -1.8082106669485443], [1.4566271999277864, 4.9968746916489915]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.79796899072209193, -4.3341220263302347], [0.51020538324619924, 1.7983600552286187]], [[-2.30311821752756, -2.4797663434424133], [-2.7364746208965518, -1.6290435570355175]]], [[[-0.52197734098829418, 4.0881937614771875], [-1.6299146570057976, -3.2562015299790756]], [[0.29256100608876867, -0.77568140294368426], [4.8023503427967782, 3.0820739496789749]]], [[[-3.7850506379542512, 4.4668566303296551], [3.4687669640455923, -1.3304422661964557]], [[3.7440883978643686, 3.0710162403958385], [-4.6250986802053617, 3.1129616277494456]]], [[[4.5115515969776165, -1.5180947364201236], [1.4600918941619287, 3.491222189831225]], [[3.9780760406179425, -0.25824834803427343], [-1.7095475143685332, 1.9473159688206652]]], [[[2.7644106722675286, -1.6797100567451717], [-1.3186292275499012, 0.17085602195932115]], [[-2.8614083683714897, -0.71007254168163492], [-4.5486234877957186, -2.8572021331765649]]], [[[1.0893091130691577, -0.054748034402583912], [-1.6765485979704922, 4.4321519350353178]], [[0.19406030022867338, 2.9107827738867806], [3.7193488963523293, -2.9487854680333516]]]])+(1.-msk_arg0)*numpy.array([[[[0.5788434003386751, 3.1662275486290206], [2.9771098671925955, 0.88346093343860765]], [[3.6794822283839288, -4.3740438558524666], [-0.5866202082632368, 4.2114550683090766]]], [[[-2.9734418267548279, -4.3376823724804439], [3.4723106053292359, -4.0173269763006036]], [[2.7674244376184829, 2.9525234420244786], [-3.9380691609648864, 0.92493623508987177]]], [[[-1.5518241879252526, -0.92776445857331957], [-1.4342294676975897, -3.0726958348204416]], [[-3.2011842792760561, -1.5487845398804598], [-2.4595858455594897, 2.3617524213380117]]], [[[3.6316920672968056, 1.2400415987158917], [1.7843326263864192, 4.1666891291562305]], [[-1.1884473202369215, 3.080040448327793], [-2.851865630295074, 4.6979605552902974]]], [[[4.2815687942672032, -4.2537715767822695], [3.426587474260403, -0.98906091384518469]], [[-3.0303974542383125, 4.9663338981109337], [-4.9739835226902827, -3.7325728354321495]]], [[[0.22900315036267926, 3.8363993409556087], [-3.9304506692159755, -2.9062862426385228]], [[-3.4941072111366713, 4.6887711702706483], [-4.7606946581804248, -3.128458677332504]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.12694834984378733, -0.89946238259502387], [4.6379056726715486, 1.4890401759395324]], [[-0.27481152895457406, -2.1424307174544053], [-4.7418925102486504, -2.2937604218839844]], [[-1.7393423313430945, -1.0407678042426438], [-0.92488648732126499, 0.95526805488875155]], [[0.39927811028386184, 2.4617474395471621], [-4.7895421926893036, 0.88747634448784929]], [[-3.0977925265542892, 4.3314636463357523], [-3.1823929629890149, -4.9877830488859161]], [[0.0064123892430112406, 4.2515278409859079], [-4.125162635658624, 4.9430033978186287]]])+(1.-msk_arg1)*numpy.array([[[1.411008103945699, -1.6668313041491212], [1.0915437071951297, -1.7186587621210423]], [[2.8222151537358906, -1.9840348445233049], [-0.88492496534437848, 3.8091527300261419]], [[-2.4905231393077374, -4.6965718746248442], [2.8006834550023196, -1.1423021145621348]], [[-0.86568411289142855, -4.63601571362911], [-0.88173272434663108, 3.4425493481391491]], [[-0.91953939696080234, 0.94541440784606934], [1.3615756338312881, -3.6276506749793471]], [[1.1840790234499767, 1.8427146885439587], [-0.39991720196053571, -0.19698487960289945]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.92491734056587926, -4.461070376174022], [-0.38925699934882463, 0.89889767263359488]], [[2.3347874551439887, 2.1581393292291353], [-1.2474344449570194, -0.14000338109598509]]], [[[-0.79678886994286824, 3.8133822325226134], [-3.7723453744602029, -5.3986322474334809]], [[-4.4493315041598818, -5.5175739131923347], [2.5085899209127938, 0.78831352779499042]]], [[[-5.5243929692973452, 2.7275142989865606], [2.4279991598029484, -2.3712100704390995]], [[2.8192019105431037, 2.1461297530745735], [-3.6698306253166102, 4.0682296826381972]]], [[[4.9108297072614784, -1.1188166261362618], [3.9218393337090909, 5.9529696293783871]], [[-0.81146615207136108, -5.047790540723577], [-0.82207116988068396, 2.8347923133085144]]], [[[-0.33338185428676059, -4.7775025832994604], [3.0128344187858511, 4.5023196682950735]], [[-6.0438013313605046, -3.8924655046706498], [-9.5364065366816355, -7.844985182062481]]], [[[1.0957215023121689, -0.048335645159572671], [2.5749792430154157, 8.6836797760212256]], [[-3.9311023354299506, -1.2143798617718433], [8.6623522941709581, 1.9942179297852771]]]])+(1.-msk_ref)*numpy.array([[[[1.9898515042843741, 4.5772356525747195], [1.3102785630434743, -0.78337037071051352]], [[4.7710259355790585, -3.2825001486573369], [-2.3052789703842791, 2.4927963061880343]]], [[[-0.15122667301893733, -1.5154672187445533], [1.4882757608059309, -6.0013618208239086]], [[1.8824994722741044, 2.0675984766801001], [-0.1289164309387445, 4.7340889651160136]]], [[[-4.0423473272329904, -3.418287597881057], [-6.1308013423224335, -7.7692677094452858]], [[-0.40050082427373646, 1.2518989151218598], [-3.6018879601216245, 1.2194503067758768]]], [[[2.7660079544053771, 0.37435748582446315], [-2.8516830872426908, -0.46932658447287956]], [[-2.0701800445835525, 2.1983077239811619], [0.59068371784407514, 8.1405099034294466]]], [[[3.3620293973064008, -5.1733109737430718], [4.3720018821064723, -0.043646505999115348]], [[-1.6688218204070244, 6.3279095319422218], [-8.6016341976696289, -7.3602235104114966]]], [[[1.413082173812656, 5.0204783644055855], [-2.0877359806720168, -1.0635715540945641]], [[-3.894024413097207, 4.2888539683101126], [-4.9576795377833243, -3.3254435569354035]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.1028895256764528, -4.994035046398305], [-0.35797617284764183, -0.39353484564442631]], [[-1.7666243346015778, -4.1438504617273653], [-3.4436283006707411, -1.9358387378688926]], [[4.3900133782386384, -0.6348625443439273], [3.9523223418448907, 2.5008714880432823]], [[3.3258925316625678, 3.4173941738592379], [-2.4480722237148655, 4.886058587891851]], [[4.0661091970764964, -4.7232246901358828], [-1.3403785311093728, 3.6601898903036076]], [[0.98126927950755949, 4.4569876771187662], [-3.9139324858469218, 0.048006257682523312]]])+(1.-msk_arg0)*numpy.array([[[4.6727276105395923, -1.7614117920613213], [-4.0813374023698978, -2.7767178595805033]], [[-4.956453724243179, 2.6845465302976033], [-3.5682882635668123, 4.6235981937150026]], [[0.46518639125803762, 1.0148914972370502], [-1.760497845138743, -1.1860905045463288]], [[0.78297778369204174, -1.7289156833449724], [-2.2522943685106602, -0.93490756213288595]], [[3.3640043397508581, 2.4007693407925927], [-4.5416086826269808, -1.692035853212289]], [[2.871670732548397, -0.013089128648045012], [-1.5226307158221553, -0.047879055693135442]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.2875338652365738, 2.8661573079204947], [-2.5052651337890151, 1.5470053915403517]], [[4.2548237872890322, 1.1877930317085372], [-2.9050988890620912, 2.665506941911068]]], [[[3.6845125748655221, -1.1178843452571066], [-4.3743931535069418, 3.8482477328340199]], [[-0.46062885266678144, -2.3611221882062061], [2.0543344894586992, 3.4773931101953206]]], [[[-3.9832106358988586, -3.1458870560956829], [4.7519055090318698, 1.0717269166225476]], [[-0.85897610141968173, 3.7772549438183916], [2.7800559706847574, -0.57731933104022914]]], [[[-2.6225209457071319, -2.7869856813635927], [0.83520842711208054, 2.0147809901672167]], [[-0.50966497709733272, 0.16981325731775154], [-4.9393639627476471, 1.2188173804607727]]], [[[4.8437714595766383, -2.9240592387513074], [1.5049765137474242, -0.32511209170105282]], [[-2.6336453495320269, -1.866636949960776], [1.133397030872759, -2.7211710502219177]]], [[[-1.7243896583714333, -1.7110378751199495], [-4.1322651404705724, 3.1160477471429786]], [[4.2797863188358232, 3.8510014404369421], [-3.9454038787868528, 3.1749471402375313]]]])+(1.-msk_arg1)*numpy.array([[[[-1.5194193686913837, -2.8332918028457699], [-1.9192963215885896, 3.9887806568410085]], [[-2.8327591668505105, -1.8104623251833321], [-1.7956586699294963, 0.81900294309365052]]], [[[-2.1709134691159049, 4.4740291307613695], [-2.7183540204274168, -1.2442522727874872]], [[-4.9339315346437038, 0.32610383839791623], [3.0744674938255816, -2.1110272577390465]]], [[[0.88252692461940718, 0.61076362167720077], [4.2059965378824238, 4.8573377831044713]], [[0.87672030514583721, 4.4361706361498161], [-3.5732575804149991, -2.7788359212777047]]], [[[2.7209081978249374, 0.99813881215985845], [3.2378823327300825, -3.5543003393779236]], [[3.0945177988733228, 0.73079691317338824], [4.3262860098209845, 4.2194980370987309]]], [[[2.2662081176597209, -0.56947655251472806], [4.8223540037479413, 0.98626540278059061]], [[-3.3228062462332808, -0.63359581500445294], [-1.4303540891120892, -0.38614493970814046]]], [[[-3.552980951462029, 2.6659604909616519], [-3.3804238385835186, -3.862683043371864]], [[-2.6477555759099869, -0.67835723596630171], [-1.3452194312298902, -1.9144747267429252]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.81535566043987906, 6.9690468335969475], [-7.49930018018732, -3.4470296548579533]], [[3.8968476144413904, 0.8298168588608954], [-3.2986337347065176, 2.2719720962666416]]], [[[1.9178882402639443, -2.8845086798586843], [-8.518243615234308, -0.29560272889334538]], [[-3.9042571533375225, -5.8047504888769472], [0.11849575158980663, 1.541554372326428]]], [[[0.40680274233977975, 1.2441263221429555], [4.1170429646879425, 0.43686437227862029]], [[3.093346240425209, 7.7295772856632823], [5.2809274587280397, 1.9235521570030532]]], [[[0.70337158595543592, 0.53890685029897512], [4.2526026009713185, 5.4321751640264546]], [[-2.9577372008121983, -2.278258966397114], [-0.053305374855796117, 6.1048759683526237]]], [[[8.9098806566531348, 1.1420499583251891], [-3.2182481763884585, -5.0483367818369356]], [[-3.9740238806413997, -3.2070154810701488], [4.7935869211763666, 0.9390188400816899]]], [[[-0.74312037886387383, -0.72976859561239005], [0.32472253664819384, 7.5730354242617448]], [[0.3658538329889014, -0.062931045409979625], [-3.8973976211043295, 3.2229533979200546]]]])+(1.-msk_ref)*numpy.array([[[[3.1533082418482086, 1.8394358076938224], [-3.6807081136499109, 2.2273688647796872]], [[-6.9140965692204084, -5.8917997275532299], [-4.5723765295099996, -1.9577149164868528]]], [[[-7.1273671933590839, -0.48242459348180944], [-0.033807490129813544, 1.4402942575101161]], [[-8.5022197982105165, -3.242184425168896], [7.6980656875405842, 2.512570935975956]]], [[[1.3477133158774448, 1.0759500129352384], [5.220888035119474, 5.8722292803415215]], [[-0.88377753999290576, 2.6756727910110731], [-4.7593480849613279, -3.9649264258240335]]], [[[3.5038859815169792, 1.7811165958519002], [1.5089666493851102, -5.2832160227228959]], [[0.84222343036266256, -1.521497455337272], [3.3913784476880986, 3.284590474965845]]], [[[5.630212457410579, 2.79452778723613], [7.2231233445405341, 3.3870347435731833]], [[-7.8644149288602616, -5.1752044976314338], [-3.1223899423243782, -2.0781807929204295]]], [[[-0.68131021891363197, 5.5376312235100489], [-3.3935129672315636, -3.875772172019909]], [[-4.1703862917321421, -2.200987951788457], [-1.3930984869230256, -1.9623537824360606]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.967623052091497, -1.2855378889359503, 2.9734584611011261, -3.6758966223333269], [2.356350968526125, 3.7983821674198879, 3.0070446417554102, 3.0502087038000205], [4.5144468535426086, -0.48214367140000736, -2.4776355189616828, -2.6229577082186948]], [[4.1886984884017249, -2.1852431737005684, -2.3361129431941419, -2.3697242760682222], [2.9781170127211958, 1.9467874345558531, 3.362414057956558, 0.37566419098736592], [3.0536955710180038, -2.2825100587093496, -3.2533497233005928, -4.2439541392784434]]], [[[4.6119902379509234, -3.4414904669966004, 2.9048687440114049, -3.3463869864689446], [-1.6291108018852132, 0.63001237165119228, 0.7109251906165035, 0.26948066229881551], [2.9593356403956221, -2.0487621406962231, -0.86031368703555788, -3.7868787763447411]], [[0.31335934786305586, 2.1214201267869113, -2.5416143704117822, -1.227432972041258], [-4.921594549530278, -1.9122257843980019, 1.9516585357566338, 3.6236543557018184], [-3.6260646518422046, 1.0881569714724026, 4.2099523510455494, 3.7117906163715499]]], [[[-4.9510887428023089, 1.7124656386936907, -4.1235821390236804, -0.51953741805411102], [3.8324866978449084, -2.2587009533073168, -4.4892843620777088, 2.9160003120132627], [-0.55752801073363578, -0.9393327748852851, -1.923546677255874, -4.3623491345165029]], [[0.84360911564364471, 4.0527434848511508, -3.6308025877075822, 4.3457624236309194], [1.9065785278333607, 0.16918220658186289, 3.0031804171330645, -4.3552132154962422], [4.623778162974391, -0.73619142642058222, 2.7619560711573774, -2.3461910546171874]]]])+(1.-msk_arg0)*numpy.array([[[[4.7898575673613486, -1.8767481599216298, -1.731608826586577, -3.3029259831308666], [-0.23052366480589725, -0.44586669550209201, 1.9400667646042056, -3.1581884977022412], [0.28220612506708509, -0.76533263735460011, 1.5206226937903793, 0.40701725490773555]], [[2.3536304982563845, -3.8414510464631912, 2.1520644454583788, -3.5204904266649906], [3.0777470332022254, -1.0616052087924421, -0.77331714027130438, -1.2988122733489882], [-2.0993968891928052, -1.0150304252625575, 0.24311819706930038, 1.9937602270752919]]], [[[-3.3650680847094829, -0.793627907647938, -1.0591872989613984, -4.965132871081205], [-0.8526448577552852, -2.8485140410445817, 2.8354504107728751, -0.59211474156843114], [2.2859263002804964, -1.043095803000619, -4.1518023694976227, -4.0226768891682543]], [[-1.9809896834704279, 2.4656141324645056, 1.4417073201787591, 4.3659714772292073], [0.92802313449221874, 0.80999804192738001, 1.7169300760451156, 1.7950103575448386], [-3.1917863947304492, 0.97877348359738736, -2.0666493572965594, 3.9349408824722421]]], [[[-1.7675397614205854, -4.9056906644672003, -0.36052601314595556, -3.0308926357544088], [-4.4816907285199949, 3.4050433107839559, -2.1886393058438989, -3.6669723962278802], [4.4108013878253249, -2.032799916405128, 0.83526820076260755, -4.145427275380019]], [[-4.4731471197769039, -2.254972333977554, -0.60890063434063801, -3.0647256451995428], [-1.5972984604181537, 0.8730257363307885, -3.9300333063940815, -2.3860509298534485], [0.21310979763689453, 3.5686347530257265, 3.8201278646375663, 4.778379295702317]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-2.8102597302401868, -3.1931720325573334, 2.7222509777473913, 2.1875198077245139], [-3.9661390865544064, -3.2311104173742313, -1.7815929130137653, -1.5769063003130146], [3.2807637533182614, -3.2922376366245452, 1.0881468627295989, -1.4092985954740378]], [[2.7086906185837565, 4.2983125656607815, 0.37759631348930345, -4.1230224180758697], [3.4865278332843115, -3.8757512184487242, 2.6856348262040619, 4.2282666055623217], [0.45765161785801478, 0.25703664291580086, 4.2472760195006511, -2.2250361389371087]]], [[[-2.071201019470339, -0.98519189205508084, 2.1569661995943079, -3.0448728270051708], [0.51546591521487883, -3.6088736758015028, 4.9617068467811496, -2.818537833033794], [0.82354095841325581, 1.504771510546659, 3.8191431679923671, 0.77709866969612218]], [[1.3602870938813236, 3.5784288550318717, 0.80142785850365517, -3.1592208650496731], [-3.0159716923456314, -0.87189499659551561, -0.5393940143245004, 0.39578868910197595], [-3.4668065921660696, 1.4851469120990126, 2.020896361782424, -4.5340121492730638]]], [[[-4.566291853502582, -3.4492302271488349, 3.2574689699675066, 0.36143892439778025], [-3.0139091100684636, -3.9648425079034597, -1.7975834268069137, 1.0734546866854515], [4.2248579971180895, 2.7098645218816984, -0.76699635039061143, -3.8097783149554623]], [[-0.18306185140472664, 3.584706669746323, 1.1556548719032307, 0.13008625666079254], [0.035057031965324725, -0.57983892708640283, 4.1540407592035571, -3.8678352597894836], [3.3163098660792354, -0.21244242233116584, -0.64434831458187514, 4.9561089957970665]]]])+(1.-msk_arg1)*numpy.array([[[[1.9499765485109064, -1.6855153072162476, 4.5967431302214958, 0.54260514666289783], [3.0914671091394208, -4.5250851595784694, 2.1533774269715353, -1.1097999467852091], [-2.4671663108510309, -4.9474994278328008, 0.026979582527628665, 4.2665404354724359]], [[-2.1288759023162429, -3.096353943367177, 3.472938004694722, 2.3531286766575477], [-4.6069494031868006, -1.3092033564651762, -2.4018924182522685, 2.6626512817330372], [1.9231078429183635, 0.94444976780138035, 2.3687095832480214, -0.91247591432186503]]], [[[2.8964745410556079, -3.6255217583806565, 0.97685867495973611, -0.49342854098212996], [4.1342288911063232, 4.8615535466211863, -4.1839386479584046, -3.0089158323277454], [0.30873306419156332, 2.5236261217067915, -2.7175414779942377, 4.2778834626340547]], [[-2.9771992086220567, 3.2109996243221079, 1.9526635158538248, -3.3670030359567029], [-2.5912319182399668, -1.5667101844324574, 2.9761173253566184, 4.4287110993304317], [-3.8139147137951293, 4.3810295032804039, 4.6117864480487007, -1.1198610836531699]]], [[[0.51273109014415041, -4.3476002864744974, 4.8080447574938496, 2.8754700513669693], [-4.4573760648127276, 4.1444219687317663, 3.3214462081442768, -0.32553909120267299], [0.92744484701415075, -0.9191233138393029, -3.982486426986398, -0.21879925343000473]], [[-2.3592423775657254, -3.5463089375541212, 4.0210988040875577, 1.0677966771363838], [4.7221132299735054, 4.4859267446195243, -1.6263079870773858, 0.81350995471949794], [4.6205138406039445, 3.7097012825002214, -1.4553991732422222, -3.3391909293602517]]]])
      res=add(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.15736332185131019, -4.4787099214932837, 5.6957094388485174, -1.488376814608813], [-1.6097881180282814, 0.56727175004565655, 1.2254517287416449, 1.4733024034870059], [7.79521060686087, -3.7743813080245525, -1.3894886562320838, -4.0322563036927326]], [[6.8973891069854814, 2.113069391960213, -1.9585166297048384, -6.4927466941440919], [6.4646448460055073, -1.928963783892871, 6.0480488841606199, 4.6039307965496876], [3.5113471888760186, -2.0254734157935488, 0.99392629620005835, -6.4689902782155517]]], [[[2.5407892184805845, -4.4266823590516813, 5.0618349436057128, -6.3912598134741154], [-1.1136448866703343, -2.9788613041503105, 5.6726320373976531, -2.5490571707349785], [3.7828765988088779, -0.54399063014956406, 2.9588294809568092, -3.0097801066486189]], [[1.6736464417443795, 5.699848981818783, -1.740186511908127, -4.3866538370909307], [-7.9375662418759099, -2.7841207809935176, 1.4122645214321334, 4.0194430448037943], [-7.0928712440082737, 2.5733038835714153, 6.2308487128279735, -0.8222215329015139]]], [[[-9.517380596304891, -1.7367645884551441, -0.86611316905617386, -0.15809849365633077], [0.81857758777644474, -6.2235434612107765, -6.2868677888846225, 3.9894549986987142], [3.6673299863844537, 1.7705317469964132, -2.6905430276464855, -8.1721274494719651]], [[0.66054726423891807, 7.6374501545974738, -2.4751477158043516, 4.4758486802917119], [1.9416355597986854, -0.41065672050453994, 7.1572211763366216, -8.2230484752857258], [7.9400880290536264, -0.94863384875174805, 2.1176077565755023, 2.6099179411798792]]]])+(1.-msk_ref)*numpy.array([[[[6.739834115872255, -3.5622634671378774, 2.8651343036349188, -2.7603208364679688], [2.8609434443335235, -4.9709518550805614, 4.0934441915757409, -4.2679884444874503], [-2.1849601857839458, -5.7128320651874009, 1.547602276318008, 4.6735576903801714]], [[0.22475459594014158, -6.9378049898303686, 5.6250024501531009, -1.1673617500074429], [-1.5292023699845751, -2.3708085652576183, -3.1752095585235729, 1.363839008384049], [-0.17628904627444175, -0.070580657461177143, 2.6118277803173218, 1.0812843127534268]]], [[[-0.46859354365387507, -4.4191496660285949, -0.082328624001662298, -5.458561412063335], [3.281584033351038, 2.0130395055766046, -1.3484882371855296, -3.6010305738961765], [2.5946593644720597, 1.4805303187061725, -6.8693438474918604, 0.25520657346580045]], [[-4.958188892092485, 5.6766137567866135, 3.3943708360325839, 0.99896844127250439], [-1.6632087837477481, -0.75671214250507735, 4.693047401401734, 6.2237214568752703], [-7.005701108525578, 5.3598029868777912, 2.5451370907521413, 2.8150797988190721]]], [[[-1.254808671276435, -9.2532909509416967, 4.4475187443478941, -0.1554225843874395], [-8.9390667933327226, 7.5494652795157222, 1.1328069023003779, -3.9925114874305532], [5.3382462348394757, -2.9519232302444309, -3.1472182262237904, -4.3642265288100237]], [[-6.8323894973426293, -5.8012812715316748, 3.4121981697469197, -1.996928968063159], [3.1248147695553516, 5.3589524809503128, -5.5563412934714673, -1.5725409751339505], [4.8336236382408391, 7.2783360355259479, 2.364728691395344, 1.4391883663420653]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.549084152437)+(1.-msk_arg0)*(-0.760019348457)
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(2.39547283018)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.84638867774)+(1.-msk_ref)*(1.63545348172)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-4.5523590284053448, -2.6127565701744171])+(1.-msk_arg0)*numpy.array([2.0812391044972189, 0.41025245536422794])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(3.99337512928)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.55898389912059621, 1.3806185591103315])+(1.-msk_ref)*numpy.array([6.0746142337819675, 4.4036275846489765])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.87490013624)+(1.-msk_arg0)*(-1.60013567215)
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([2.160238959619269, -3.0433707064126114])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.71466117661851891, -5.9182708426503989])+(1.-msk_ref)*numpy.array([0.56010328747085936, -4.6435063785610211])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.27756329946700653, -1.3051830033078735, 1.3028459838779662, 2.9926243718325463, -2.3733009781481207], [-3.7991430960131609, 0.29219972895949997, 1.3563350622585402, -4.7897195280670086, -1.8911672112250266], [-2.1794331609897508, -0.2245947450805037, -4.4231721461171869, -2.7793891054821862, -4.7404033963762124], [-2.1561674638479045, -4.1412459924115499, 4.3448170787937634, 3.2468618055284573, 3.2537929502717216]])+(1.-msk_arg0)*numpy.array([[-1.3419131705018259, 3.2250057575011457, -0.013425271284655871, -4.4647345359338688, -1.8238496332693366], [-3.319532998048528, 2.0386926598834165, 0.020124734081355733, 1.9680824889167248, -0.075960191094122997], [-4.0332736404518021, -4.9820878565690432, 2.3745857824171166, -1.1507654220235275, -2.1417299347936014], [1.6012219736819224, -3.4337342599108833, -0.63005930751068462, -4.1787589635219469, 2.5004836468911975]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-3.56560685417)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.8431701536336407, -4.8707898574745077, -2.2627608702886679, -0.57298248233408788, -5.9389078323147544], [-7.3647499501797951, -3.2734071252071342, -2.209271791908094, -8.3553263822336419, -5.4567740653916612], [-5.745040015156385, -3.7902015992471378, -7.988779000283821, -6.3449959596488199, -8.3060102505428475], [-5.7217743180145391, -7.706852846578184, 0.7792102246271293, -0.31874504863817688, -0.31181390389491259]])+(1.-msk_ref)*numpy.array([[-4.90752002466846, -0.34060109666548843, -3.57903212545129, -8.030341390100503, -5.3894564874359707], [-6.8851398522151621, -1.5269141942832176, -3.5454821200852784, -1.5975243652499094, -3.6415670452607571], [-7.5988804946184363, -8.5476947107356764, -1.1910210717495175, -4.7163722761901621, -5.7073367889602356], [-1.9643848804847117, -6.9993411140775175, -4.1956661616773188, -7.744365817688581, -1.0651232072754366]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.42372512522)+(1.-msk_arg0)*(1.3293392819)
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[3.9131385452328331, -0.054316295960157746, 3.8542763644644236, 4.4160117981494231, -4.8564732409389046], [2.7231792846189684, 1.4976551988669602, -4.4261528420499827, 3.6341804877356214, -4.9615781121702138], [2.16752422454105, 3.0752044518735069, -0.96060941206963957, -3.433416121390731, -4.335880824102726], [-2.425192168669108, -3.7772779652996338, -1.5903046993065519, -4.2300109188935817, -1.5532885311228783]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.51058657998620571, -4.4780414211791966, -0.56944876075461526, -0.0077133270696156941, -9.2801983661579435], [-1.7005458406000704, -2.9260699263520786, -8.8498779672690215, -0.78954463748341741, -9.3853032373892518], [-2.2562009006779888, -1.3485206733455319, -5.3843345372886784, -7.8571412466097694, -8.7596059493217648], [-6.8489172938881469, -8.2010030905186717, -6.0140298245255908, -8.6537360441126197, -5.9770136563419172]])+(1.-msk_ref)*numpy.array([[5.2424778271333974, 1.2750229859404065, 5.1836156463649878, 5.7453510800499874, -3.5271339590383404], [4.0525185665195327, 2.8269944807675245, -3.0968135601494184, 4.9635197696361857, -3.6322388302696496], [3.4968635064416143, 4.4045437337740712, 0.36872986983092471, -2.1040768394901668, -3.0065415422021617], [-1.0958528867685438, -2.4479386833990695, -0.26096541740598767, -2.9006716369930174, -0.22394924922231407]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.3372371908177776, -0.055734966371744576], [1.6016791786085358, 3.9159958504448902]], [[-0.41581849875651145, -3.3023082696341586], [4.9069357788104426, -3.6064466483026925]], [[-0.42870187672381022, -1.0983734348052234], [4.9871948439245593, 0.86477772601415026]], [[1.7670612833806052, 4.8726504039191472], [1.4217952304279455, -3.6787558911059905]], [[-1.1688552259791116, -0.62546496760000014], [-1.9492900495600853, 4.1828187320734251]], [[-0.56840259036981511, 3.7677984574463235], [-2.7040466214497116, 3.8794820294256382]]])+(1.-msk_arg0)*numpy.array([[[2.9696561263433594, 2.3451764896558078], [2.6655159325207674, -1.9377319741814789]], [[2.1676780186169156, -3.7576585872508304], [-3.1405989115908319, 2.3576966729719819]], [[4.3757107074313257, -2.4706600847726956], [-3.8748814971708958, -3.0206443245457737]], [[-2.1450920708394641, -0.53615563451642956], [0.24390657260171, -3.5038625490775601]], [[-4.5475560462038969, -0.2903100185013745], [-3.6207317009257078, 4.7427619268471553]], [[4.3423720902170508, -0.61953133163246399], [-3.5645444423154347, -1.3460128505430857]]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(0.952478943066)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.2897161338835454, 0.89674397669402328], [2.5541581216743037, 4.868474793510658]], [[0.53666044430925641, -2.3498293265683907], [5.8594147218762105, -2.6539677052369246]], [[0.52377706634195764, -0.14589449173945557], [5.9396737869903271, 1.8172566690799181]], [[2.7195402264463731, 5.825129346984915], [2.3742741734937134, -2.7262769480402227]], [[-0.21637628291334376, 0.32701397546576771], [-0.99681110649431748, 5.135297675139193]], [[0.38407635269595275, 4.7202774005120913], [-1.7515676783839438, 4.8319609724914061]]])+(1.-msk_ref)*numpy.array([[[3.9221350694091273, 3.2976554327215757], [3.6179948755865352, -0.98525303111571105]], [[3.1201569616826834, -2.8051796441850625], [-2.1881199685250641, 3.3101756160377498]], [[5.3281896504970936, -1.5181811417069278], [-2.9224025541051279, -2.0681653814800058]], [[-1.1926131277736962, 0.4163233085493383], [1.1963855156674779, -2.5513836060117923]], [[-3.595077103138129, 0.66216892456439336], [-2.66825275785994, 5.6952408699129231]], [[5.2948510332828187, 0.33294761143330387], [-2.6120654992496668, -0.39353390747731787]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-1.39908069541)+(1.-msk_arg0)*(-0.535571650285)
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[1.6643373656775493, 0.8159143774120583], [0.61898534474439248, 4.170945735848159]], [[4.5811555739347671, 4.9866250825161522], [2.382236365711961, 4.2818242095345802]], [[-3.0215291401666802, -2.0491201646377712], [-4.2763007712915799, 4.2926602408893881]], [[-3.8452857745299296, -1.4707194619672839], [-4.6490899325623891, -2.1193440548227036]], [[-3.5709391271686473, -4.4720730260829207], [2.8646079407642357, -2.176568648396926]], [[2.1839124339756246, 4.7473308611721023], [2.2891309613430169, -3.6119517909993668]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.26525667027172517, -0.58316631799376584], [-0.78009535066143165, 2.7718650404423348]], [[3.182074878528943, 3.5875443871103281], [0.98315567030613682, 2.8827435141287561]], [[-4.4206098355725043, -3.4482008600435954], [-5.6753814666974041, 2.893579545483564]], [[-5.2443664699357537, -2.869800157373108], [-6.0481706279682133, -3.5184247502285277]], [[-4.9700198225744714, -5.8711537214887448], [1.4655272453584116, -3.5756493438027501]], [[0.78483173856980049, 3.3482501657662782], [0.89005026593719272, -5.0110324864051909]]])+(1.-msk_ref)*numpy.array([[[1.1287657153921797, 0.28034272712668873], [0.083413694459022913, 3.6353740855627894]], [[4.0455839236493976, 4.4510534322307826], [1.8466647154265914, 3.7462525592492106]], [[-3.5571007904520497, -2.5846918149231408], [-4.8118724215769495, 3.7570885906040186]], [[-4.3808574248152992, -2.0062911122526534], [-5.1846615828477587, -2.6549157051080732]], [[-4.1065107774540168, -5.0076446763682902], [2.3290362904788662, -2.7121402986822956]], [[1.6483407836902551, 4.2117592108867328], [1.7535593110576473, -4.1475234412847364]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.32139470804328063, -2.6618370989896678, -1.5717779703542476, 0.71882393049017246], [-2.9158143388322899, -0.6321937782257141, 1.8303557177066407, 3.0655893413077369], [-2.1941852939373985, 0.14666311307585644, -3.6215874311196226, 2.9989173720484521]], [[-1.293551737175612, -0.21343318390921429, 3.8037540565438803, 2.1323196691673472], [3.8768342128995563, -4.8381156731765831, 4.4128891146032849, -2.7824468661964739], [-2.5993789818415145, -3.5170707375685351, 1.4016302391876243, 0.29201252618559526]]], [[[-3.7175083313917812, -2.5084449193644298, 0.65855590885881909, -2.7823956166115704], [-1.3107998669165957, -1.9357135118877156, -1.9094811034349624, -3.4337126832578613], [3.1430811370302081, 3.541891607626825, -0.8712500733842603, -3.8748684917053522]], [[-1.1028453592101748, -2.1799346691521935, 1.610379491989141, -3.1991439997974078], [-2.2702798189694375, 1.3706134517067978, 4.4213868342930898, 3.8348530211166558], [4.6731225911835157, -3.8959470575541766, -0.2453971326221982, 3.2470663461261573]]], [[[-0.77290934250582133, 2.0731034142196858, -1.8022704516415331, -1.5179672677298219], [4.8882976658242114, -1.3279078297251914, -0.72225967165024407, -3.9414614911896195], [2.2659265464520413, 2.9673584061659621, -2.6041073004616786, 3.0849228378320017]], [[3.9067001881794727, -4.3974287170743658, 1.056944189235077, -2.089075935896302], [0.97706226077003056, -4.2164663826592319, 2.0399194691373985, 3.7425378909438969], [4.9095265050662462, 2.8859782709855262, -0.2616048664223829, -0.90946309525355673]]]])+(1.-msk_arg0)*numpy.array([[[[2.0270114925792635, -1.087969292188109, -1.9728407494678981, -1.9983045864478544], [0.49271639743416884, 4.2251957095169921, 0.65789477363959215, 0.58454957003218055], [4.6519745729956803, 4.8076595602817704, -2.2416579579435525, -2.353926891402506]], [[-1.3117887707236942, 4.8381130177535709, 1.1931551357756724, -1.1373546352206807], [-4.6791585965991125, 2.343124947807671, 0.53207646592255919, -2.7118469238301257], [-4.3880403736955529, -3.3662648651315594, -0.52730641173303283, 3.4846733344556018]]], [[[-4.0564047182853455, 1.9016777684543342, -0.53864710201235866, -0.98621567459448034], [1.9208258251943091, 4.428746355085277, -0.41364537213603914, 3.8511498332149454], [1.5174466765275882, -3.8112598089277205, -3.5938857930000969, 4.1940948752805625]], [[-3.1701631588592916, 2.9491036425674642, 4.6770592046965742, 0.83277910256343013], [1.0550013281829971, 0.71695187027809215, -1.6388285619529777, -0.9271447815079048], [-1.6664028734625447, 4.472649026130691, 2.5759747206306658, -4.0716377539167752]]], [[[2.0062846055931294, -3.4245344663123491, 1.3560283579848029, -0.57523068580960413], [2.6060497538599829, -4.7724644783957215, -3.6907656012651282, 3.6747144395103319], [-3.1326058835194592, 3.4573771567790104, -2.9017222369492677, 3.3168032136554721]], [[-0.050581926279668288, 1.2970568994322997, 1.7100974795003783, 2.1293794718366623], [-0.83047995194402802, 1.0220200423858454, 1.3590115187891101, -1.2338792787252228], [-3.8255695687998559, 3.4051067516672031, 3.8718385768111947, -4.1444518508512642]]]])
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s1=numpy.array(-3.77684156083)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-4.098236268872034, -6.4386786598184216, -5.3486195311830009, -3.0580176303385809], [-6.6926558996610428, -4.4090353390544674, -1.9464858431221126, -0.71125221952101647], [-5.9710268547661514, -3.6301784477528969, -7.3984289919483759, -0.77792418878030123]], [[-5.0703932980043653, -3.9902747447379676, 0.026912495715126994, -1.6445218916614062], [0.09999265207080299, -8.6149572340053364, 0.63604755377453159, -6.5592884270252272], [-6.3762205426702678, -7.2939122983972879, -2.375211321641129, -3.4848290346431581]]], [[[-7.4943498922205345, -6.2852864801931831, -3.1182856519699342, -6.5592371774403233], [-5.087641427745349, -5.7125550727164693, -5.6863226642637157, -7.2105542440866142], [-0.63376042379854525, -0.23494995320192835, -4.6480916342130136, -7.6517100525341055]], [[-4.8796869200389281, -5.9567762299809468, -2.1664620688396123, -6.9759855606261612], [-6.0471213797981909, -2.4062281091219555, 0.64454527346433643, 0.058011460287902494], [0.89628103035476236, -7.6727886183829295, -4.0222386934509515, -0.52977521470259603]]], [[[-4.5497509033345747, -1.7037381466090675, -5.5791120124702864, -5.2948088285585753], [1.1114561049954581, -5.1047493905539447, -4.4991012324789974, -7.7183030520183724], [-1.5109150143767121, -0.80948315466279119, -6.3809488612904319, -0.69191872299675161]], [[0.1298586273507194, -8.1742702779031191, -2.7198973715936763, -5.8659174967250554], [-2.7997793000587228, -7.9933079434879852, -1.7369220916913548, -0.034303669884856447], [1.1326849442374929, -0.89086328984322716, -4.0384464272511362, -4.6863046560823101]]]])+(1.-msk_ref)*numpy.array([[[[-1.7498300682494898, -4.8648108530168628, -5.7496823102966514, -5.7751461472766081], [-3.2841251633945845, 0.44835414868823875, -3.1189467871891612, -3.1922919907965728], [0.87513301216692696, 1.0308179994530171, -6.0184995187723054, -6.1307684522312593]], [[-5.0886303315524479, 1.0612714569248176, -2.5836864250530809, -4.914196196049434], [-8.4560001574278658, -1.4337166130210823, -3.2447650949061941, -6.488688484658879], [-8.1648819345243062, -7.1431064259603128, -4.3041479725617862, -0.29216822637315154]]], [[[-7.8332462791140989, -1.8751637923744191, -4.315488662841112, -4.7630572354232337], [-1.8560157356344442, 0.6519047942565237, -4.1904869329647925, 0.074308272386192087], [-2.2593948843011651, -7.5881013697564743, -7.3707273538288502, 0.41725331445180913]], [[-6.9470047196880449, -0.82773791826128917, 0.90021764386782088, -2.9440624582653232], [-2.7218402326457563, -3.0598896905506612, -5.415670122781731, -4.7039863423366581], [-5.443244434291298, 0.69580746530193771, -1.2008668401980875, -7.8484793147455285]]], [[[-1.770556955235624, -7.2013760271411025, -2.4208132028439504, -4.3520722466383575], [-1.1707918069687704, -8.5493060392244757, -7.4676071620938815, -0.10212712131842139], [-6.9094474443482126, -0.31946440404974297, -6.678563797778021, -0.46003834717328118]], [[-3.8274234871084216, -2.4797846613964536, -2.0667440813283751, -1.647462088992091], [-4.6073215127727813, -2.7548215184429079, -2.4178300420396432, -5.0107208395539757], [-7.6024111296286092, -0.37173480916155022, 0.094997015982441368, -7.9212934116800175]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.30408568992)+(1.-msk_arg0)*(-4.34159418357)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[1.5643378441883264, -1.5148473069884218, 2.8952506537889402, -0.7851146387127006], [-1.5483490899734855, 0.15788254975798832, 4.974168679232843, -0.12540867858220395], [-1.5672466289210805, -1.5400488237885979, -1.086836734714629, -2.8777515142216203]], [[-2.6445289293136853, -4.8443071308560528, -2.3566418342613762, 1.5372512825106615], [-3.2539437858231057, -0.11500168810360556, 3.9160496412780432, 4.7019181433125929], [-0.17253236305960051, 3.1452246376151134, 4.0019805723276605, -4.028035343873654]]], [[[-0.88285975895685276, 4.6288543561802058, -4.3639392344451693, -4.7251248139067474], [3.3249818483762965, 1.8920814457605832, 4.3163788486877959, 1.6817897173243122], [-1.5945832211135293, -3.9980140066546808, 2.6336396878029209, 2.9834264646997255]], [[4.9749114212673735, -3.8187674630758597, -1.3244723577842157, -4.8261904594145433], [1.7608967449336497, 2.4861089126260527, 1.2229992745717011, -1.0236933224165279], [2.845279006529605, -3.8583685786976609, 3.9975333637686283, -2.1764893037388289]]], [[[-1.4293843514386784, 2.7003326266400842, 3.8955173377413672, 1.3604215567761244], [1.9623903314871525, -4.7669272289252804, 4.5055529873350419, 4.4270180517141089], [-0.26573394932553995, 2.0073115844678435, 4.9500881377532586, 2.3405441753454292]], [[1.3218958734562509, 4.5242134537987599, -2.7630925186078938, 1.0258301725056551], [4.2570706337076452, -0.96169660563703818, 3.0761682378099149, 2.8175044255284707], [4.9320062786528123, 0.089625592479915639, 0.031399827054379159, -3.6747036700569291]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.8684235341113018, 0.78923838293455351, 5.1993363437119156, 1.5189710512102748], [0.75573659994948983, 2.4619682396809637, 7.2782543691558184, 2.1786770113407714], [0.73683906100189489, 0.76403686613437749, 1.2172489552083463, -0.57366582429864499]], [[-0.34044323939070997, -2.5402214409330774, -0.052556144338400834, 3.8413369724336368], [-0.94985809590013037, 2.1890840018193698, 6.2201353312010186, 7.0060038332355683], [2.1315533268633748, 5.4493103275380887, 6.3060662622506358, -1.7239496539506787]]], [[[1.4212259309661226, 6.9329400461031812, -2.0598535445221939, -2.4210391239837721], [5.6290675382992719, 4.1961671356835586, 6.6204645386107712, 3.9858754072472875], [0.70950246880944601, -1.6939283167317054, 4.9377253777258963, 5.2875121546227009]], [[7.2789971111903489, -1.5146817731528843, 0.97961333213875967, -2.5221047694915679], [4.064982434856625, 4.7901946025490281, 3.5270849644946765, 1.2803923675064475], [5.1493646964525803, -1.5542828887746856, 6.3016190536916037, 0.12759638618414648]]], [[[0.87470133848429699, 5.0044183165630596, 6.1996030276643426, 3.6645072466990998], [4.2664760214101278, -2.4628415390023051, 6.8096386772580173, 6.7311037416370842], [2.0383517405974354, 4.3113972743908189, 7.254173827676234, 4.6446298652684046]], [[3.6259815633792263, 6.8282991437217353, -0.45900682868491849, 3.3299158624286305], [6.5611563236306205, 1.3423890842859372, 5.3802539277328902, 5.121590115451446], [7.2360919685757876, 2.393711282402891, 2.3354855169773545, -1.3706179801339538]]]])+(1.-msk_ref)*numpy.array([[[[-2.7772563393829826, -5.8564414905597308, -1.4463435297823688, -5.1267088222840096], [-5.8899432735447945, -4.1837116338133207, 0.63257449566153401, -4.4670028621535129], [-5.9088408124923895, -5.8816430073599069, -5.4284309182859385, -7.2193456977929298]], [[-6.9861231128849948, -9.1859013144273618, -6.6982360178326852, -2.8043429010606475], [-7.5955379693944147, -4.4565958716749146, -0.42554454229326577, 0.36032395974128395], [-4.5141265466309095, -1.1963695459561956, -0.33961361124364853, -8.3696295274449639]]], [[[-5.2244539425281618, 0.2872601726088968, -8.7055334180164792, -9.0667189974780555], [-1.0166123351950125, -2.4495127378107258, -0.02521533488351313, -2.6598044662469968], [-5.9361774046848383, -8.3396081902259898, -1.7079544957683881, -1.3581677188715835]], [[0.63331723769606452, -8.1603616466471678, -5.6660665413555247, -9.1677846429858523], [-2.5806974386376593, -1.8554852709452563, -3.1185949089996079, -5.3652875059878369], [-1.496315177041704, -8.1999627622689708, -0.34406081980268066, -6.5180834873101379]]], [[[-5.7709785350099878, -1.6412615569312248, -0.44607684582994178, -2.9811726267951846], [-2.3792038520841565, -9.1085214124965894, 0.16395880376373295, 0.085423868142799897], [-4.6073281328968489, -2.3342825991034655, 0.60849395418194963, -2.0010500082258798]], [[-3.0196983101150581, 0.18261927022745095, -7.1046867021792028, -3.3157640110656539], [-0.084523549863663838, -5.3032907892083472, -1.2654259457613941, -1.5240897580428383], [0.59041209508150327, -4.2519685910913934, -4.3101943565169298, -8.0162978536282381]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.0914956026325973, 0.80393988958617868])+(1.-msk_arg0)*numpy.array([3.6407606712029423, -2.6585220365337459])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([0.84375666980258401, -0.75812444849603722])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.24773893283001325, 0.045815441090141462])+(1.-msk_ref)*numpy.array([4.4845173410055263, -3.4166464850297831])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-3.3556723573446146, -3.6424030299479715], [-4.8701962351564223, 0.064222949809118468]])+(1.-msk_arg0)*numpy.array([[-0.93872816110870971, -3.4600330893643472], [-1.3626865847139169, -1.238226357815615]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([1.1361002318136144, -2.9055099093553216])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.2195721255310001, -2.5063027981343571], [-7.7757061445117444, -2.8412869595462031]])+(1.-msk_ref)*numpy.array([[0.1973720707049047, -2.3239328575507328], [-4.2681964940692385, -4.1437362671709366]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.6675377781306953, -3.2606804301236947])+(1.-msk_arg0)*numpy.array([-3.4640178459883488, -1.6171491697333948])
      arg1=Symbol(shape=(2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[-0.27762720076639802, 3.652392772431023], [0.20001412731850099, -3.2150495784251323]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.3899105773642972, 8.3199305505617183], [-3.0606663028051937, -6.475730008548827]])+(1.-msk_ref)*numpy.array([[-3.7416450467547469, 0.18837492644267417], [-1.4171350424148939, -4.8321987481585271]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.9196458103332112, 4.2324640090451062, -4.6858953626929969, 1.9305962718453218, 2.7371341892174872], [-3.5336119204760119, 1.1875170097644911, -2.4911238236988922, -3.4493947913901013, 0.48346496259404859], [2.680776947917396, -3.1322081057043105, -3.7902587412718125, 0.17329564350395987, -3.8373595945726335], [-0.11735729090590041, -0.48496970677834561, 3.5201282412840271, -1.7491157228061494, -2.097088415732574]], [[-1.511622287851079, -3.2561305426135156, 2.2455354911478054, -3.7162991172697355, -0.21929974234897731], [-2.6069983580222242, 1.4464906954947434, 1.3537953484244269, -1.4256663506499145, -1.7555057888195158], [0.96534169582046747, 0.85144465484343002, 2.5237995063324314, -4.4420391578793659, 4.8732159071245498], [2.9034180609147429, -0.39630635613713139, 4.2574822433837909, 1.4427992800840634, -2.5868155309486984]]])+(1.-msk_arg0)*numpy.array([[[4.2863883515729562, -3.2277750230148738, -2.2777254808756342, 2.1460230411116257, 0.20544433742211332], [3.4466284544455057, 0.83100799167668349, 0.25978251534957941, 0.03703828373164697, -2.372037088819352], [-3.3883438537867661, 3.8135731698489543, -4.8370112526408295, -0.63676997869167895, 3.0340317500686567], [-1.4279969295326191, -3.5950409084311854, -4.0626425561532944, 4.6947689510735415, 0.16206306813208649]], [[0.9330020116850406, 0.295015777781372, 3.1438549318087059, -0.094022106313213172, 0.90488189434361921], [4.9708270534178673, 1.0473017608464454, -1.0109702922102848, -3.1014385731495175, 0.567552648251473], [3.59734361652988, -4.570498345149363, 1.5808922874913192, 3.2247338689121037, 2.6812200066028877], [-4.3540868666266972, -2.9333596490143368, -2.8614360888627632, -4.5999051611329946, 4.5971573627518563]]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([-0.17357335333916346, 4.2870604706589006])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.7460724569940478, 4.0588906557059428, -4.8594687160321604, 1.7570229185061583, 2.5635608358783237], [-3.7071852738151754, 1.0139436564253277, -2.6646971770380556, -3.6229681447292648, 0.30989160925488513], [2.5072035945782325, -3.305781459043474, -3.963832094610976, -0.00027770983520358783, -4.0109329479117974], [-0.29093064424506387, -0.65854306011750907, 3.3465548879448637, -1.9226890761453128, -2.2706617690717374]], [[2.7754381828078216, 1.030929928045385, 6.532595961806706, 0.57076135338916512, 4.0677607283099233], [1.6800621126366764, 5.733551166153644, 5.6408558190833276, 2.8613941200089861, 2.5315546818393848], [5.2524021664793681, 5.1385051255023306, 6.8108599769913321, -0.15497868722046526, 9.1602763777834504], [7.1904785315736435, 3.8907541145217692, 8.5445427140426915, 5.7298597507429641, 1.7002449397102022]]])+(1.-msk_ref)*numpy.array([[[4.1128149982337927, -3.4013483763540373, -2.4512988342147977, 1.9724496877724622, 0.03187098408294986], [3.2730551011063422, 0.65743463833752003, 0.086209162010415952, -0.13653506960751649, -2.5456104421585155], [-3.5619172071259295, 3.6399998165097909, -5.010584605979993, -0.81034333203084241, 2.8604583967294932], [-1.6015702828717826, -3.7686142617703489, -4.2362159094924579, 4.5211955977343781, -0.011510285207076976]], [[5.2200624823439412, 4.5820762484402726, 7.4309154024676065, 4.1930383643456874, 5.1919423650025198], [9.257887524076768, 5.3343622315053461, 3.2760901784486158, 1.1856218975093831, 4.8546131189103736], [7.8844040871887806, -0.28343787449046243, 5.8679527581502198, 7.5117943395710043, 6.9682804772617883], [-0.067026395967796581, 1.3537008216445638, 1.4256243817961374, -0.31284469047409402, 8.8842178334107569]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.9443640534298972, 0.46738228633604706])+(1.-msk_arg0)*numpy.array([4.2970613509788667, 2.3947163786449774])
      arg1=Symbol(shape=(2, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[-2.2466630399978915, 1.3281913437031037, 1.6867098595218009, 0.20133610723087525, 2.4322655148583214], [3.4325115387525287, 3.3069443622183154, 1.5387916070446668, 3.918865083329683, 1.9926904012619158], [-1.4589091761985404, -1.2779251894497268, -2.3127907445319296, 1.5869084076477016, -4.7231742044766447], [-1.0570132259780896, 0.54901063826966201, -4.6191971764817179, -0.23743733605630446, 1.0371041297618291]], [[3.7645117457269279, -3.2185201987430112, 0.25424560382133343, 1.4097742255919581, 4.3150719533203166], [-1.1107303302984253, -3.9757407351745053, 1.7951625729779455, -3.9829482175987199, 1.0665388404661496], [-0.85853192982332072, 4.6000135984812278, -4.6982309132865021, -4.8199828258736162, -2.7922044183259596], [-3.290421092368784, -2.8116276131774565, -2.8838972362574244, -1.6002396453717536, -3.9236155008918083]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.69770101343200563, 4.2725553971330008, 4.631073912951698, 3.1457001606607724, 5.3766295682882186], [6.3768755921824258, 6.2513084156482126, 4.4831556604745639, 6.8632291367595801, 4.9370544546918129], [1.4854548772313567, 1.6664388639801704, 0.63157330889796759, 4.5312724610775987, -1.7788101510467476], [1.8873508274518076, 3.4933746916995592, -1.6748331230518207, 2.7069267173735927, 3.9814681831917262]], [[4.231894032062975, -2.7511379124069641, 0.72162789015738049, 1.8771565119280051, 4.7824542396563636], [-0.64334804396237821, -3.5083584488384583, 2.2625448593139925, -3.5155659312626728, 1.5339211268021966], [-0.39114964348727366, 5.0673958848172749, -4.230848626950455, -4.3526005395375691, -2.3248221319899125], [-2.8230388060327369, -2.3442453268414094, -2.4165149499213774, -1.1328573590357065, -3.4562332145557613]]])+(1.-msk_ref)*numpy.array([[[2.0503983109809751, 5.6252526946819703, 5.9837712105006675, 4.4983974582097419, 6.7293268658371881], [7.7295728897313953, 7.6040057131971821, 5.8358529580235334, 8.2159264343085496, 6.2897517522407824], [2.8381521747803262, 3.0191361615291399, 1.9842706064469371, 5.8839697586265682, -0.42611285349777805], [3.2400481250007771, 4.8460719892485287, -0.32213582550285125, 4.0596240149225622, 5.3341654807406957]], [[6.1592281243719054, -0.82380382009803377, 2.6489619824663109, 3.8044906042369355, 6.709788331965294], [1.2839860483465522, -1.5810243565295279, 4.1898789516229229, -1.5882318389537424, 3.461255219111127], [1.5361844488216567, 6.9947299771262053, -2.3035145346415247, -2.4252664472286387, -0.39748803968098212], [-0.89570471372380656, -0.41691123453247902, -0.48918085761244701, 0.79447673327322388, -1.5288991222468309]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.054399610932163611, -2.753587257860961], [-2.7621310151645586, 1.8155875060600586]], [[-2.1048204532906456, -2.1798360829238375], [2.2767899100380049, 3.3556249743458917]], [[2.2949681379263476, 0.23831150314717142], [-2.1004430521237638, 2.2356804861149184]], [[-3.8101007142692822, 2.8648011479405122], [4.8759207570548213, -1.8062697612281688]], [[-1.1894953390513843, 4.2451773614827264], [-1.0930959412703709, 3.7105479029654216]], [[-2.0957256787889311, 0.97098761360767138], [-1.1348359142041451, -3.5689266065518255]]], [[[0.52707981249026759, 3.3239833966558106], [3.745872023494595, -0.40128640662279391]], [[-1.5168507008198717, 0.54528411750310202], [3.0254295597086198, 1.0655402343875622]], [[1.8461631038190376, -4.6571607080166091], [1.1186191428067263, 1.1989310417490637]], [[0.67696143401496833, -3.2530390097381567], [-0.58376258580847917, 4.7331180063974596]], [[-3.8171262393188545, -4.7683050496731214], [-4.9010103171116608, -2.2081444103662751]], [[4.3235006448239481, -4.0511885020164149], [1.0795841313217398, -2.0814564152453419]]]])+(1.-msk_arg0)*numpy.array([[[[3.2828309837484966, -1.3338161100238954], [-1.6550249726507813, -3.0917789703819532]], [[0.01179951411120328, -4.4380506863361111], [3.1835712061838866, 4.0959644488970746]], [[0.17042225457836224, 3.9755732706018811], [-1.1723195303084175, -2.0083734411353618]], [[-2.5003700492235903, -3.6673715107750846], [2.8038463586912545, 3.8568665649773859]], [[-4.9865941637111169, -1.5980996382100621], [2.7074629652575641, 2.6389472486128849]], [[-0.069130425396586759, 0.49732559334817594], [-2.3747986979460212, -0.11965832968128609]]], [[[1.1160010027271756, 1.9232019318807669], [3.9896936238451559, 3.0913319299610187]], [[-1.3884921051412826, -3.3153975477678688], [-3.513920017176928, -3.7064579889568727]], [[4.2604717071097458, 4.907731913780923], [1.947435004500468, 2.5390758802643383]], [[-4.1970087758879169, -3.6023613137100021], [3.7151980307849062, -2.375547876045089]], [[4.4782623337275798, -2.5526499006040568], [0.81216340762152139, -1.2124815033313263]], [[2.4414929628713997, -3.3065092928587037], [-3.6374746695015236, -1.8174239122424773]]]])
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s1=numpy.array([0.85449734300358759, -3.4517723523387067])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.80009773207142398, -1.8990899148573734], [-1.907633672160971, 2.6700848490636462]], [[-1.250323110287058, -1.3253387399202499], [3.1312872530415925, 4.2101223173494793]], [[3.1494654809299352, 1.092808846150759], [-1.2459457091201762, 3.090177829118506]], [[-2.9556033712656946, 3.7192984909440998], [5.7304181000584089, -0.95177241822458125]], [[-0.33499799604779668, 5.099674704486314], [-0.23859859826678331, 4.5650452459690092]], [[-1.2412283357853435, 1.825484956611259], [-0.2803385712005575, -2.7144292635482379]]], [[[-2.9246925398484391, -0.12778895568289617], [0.29409967115588831, -3.8530587589615006]], [[-4.9686230531585789, -2.9064882348356047], [-0.4263427926300869, -2.3862321179511445]], [[-1.6056092485196691, -8.1089330603553158], [-2.3331532095319805, -2.252841310589643]], [[-2.7748109183237384, -6.7048113620768639], [-4.0355349381471859, 1.2813456540587529]], [[-7.2688985916575612, -8.2200774020118281], [-8.3527826694503666, -5.6599167627049818]], [[0.87172829248524142, -7.5029608543551216], [-2.3721882210169669, -5.533228767584049]]]])+(1.-msk_ref)*numpy.array([[[[4.1373283267520842, -0.47931876702030785], [-0.80052762964719371, -2.2372816273783656]], [[0.86629685711479087, -3.5835533433325235], [4.0380685491874742, 4.9504617919006622]], [[1.0249195975819498, 4.8300706136054687], [-0.31782218730482992, -1.1538760981317742]], [[-1.6458727062200027, -2.812874167771497], [3.6583437016948421, 4.7113639079809735]], [[-4.1320968207075293, -0.74360229520647447], [3.5619603082611517, 3.4934445916164725]], [[0.78536691760700084, 1.3518229363517635], [-1.5203013549424336, 0.7348390133223015]]], [[[-2.3357713496115311, -1.5285704204579398], [0.53792127150644919, -0.36044042237768803]], [[-4.8402644574799893, -6.767169900106575], [-6.9656923695156348, -7.1582303412955799]], [[0.80869935477103905, 1.4559595614422163], [-1.5043373478382387, -0.91269647207436844]], [[-7.6487811282266236, -7.0541336660487088], [0.26342567844619946, -5.8273202283837957]], [[1.026489981388873, -6.0044222529427635], [-2.6396089447171853, -4.664253855670033]], [[-1.010279389467307, -6.7582816451974104], [-7.0892470218402304, -5.2691962645811845]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank1_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.1614720945957089, 3.8267255982900181])+(1.-msk_arg0)*numpy.array([-2.0245831261868954, 3.6384822569294109])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[1.3202787701502299, 1.814140103700689], [-2.2910951985135455, 1.5801401680077296]], [[-1.6066394075097659, 2.4183553551799992], [-3.8012007537198222, -4.9990112974569545]], [[-2.1328870770626041, -2.9115833114997391], [-4.3890321948756235, -4.6000452005931036]], [[2.5616038760206568, -3.7520947940854175], [-4.156367086805691, -3.3381700709951581]], [[0.55214205171022268, 4.3310558399908547], [-4.6740171609330394, 0.94249489019660704]], [[-1.5781519602547922, -0.31748358057880921], [0.087035331174607933, 4.8249306367014544]]], [[[-2.528958026670681, -0.88187782179679353], [-3.3938483791466609, 2.1744966294434418]], [[0.34598804677766282, 0.12214409195801501], [3.0846661399516933, 4.5915544682395417]], [[-2.4271663919904949, -1.4101872807373961], [-1.3343245719788799, 1.2418211000639898]], [[1.467751153087641, 2.0430724694946809], [1.4526602665605273, 1.6184660428875439]], [[1.7063786693348115, -4.6931798569177916], [-2.1492305529009847, -1.4088022478904083]], [[1.2570137659457732, -1.3528592891478661], [-0.27211024086521718, -3.655497483668313]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.84119332444547901, -0.34733199089501987], [-4.4525672931092544, -0.58133192658797928]], [[-3.7681115021054747, 0.25688326058429034], [-5.9626728483155311, -7.1604833920526634]], [[-4.2943591716583125, -5.073055406095448], [-6.5505042894713323, -6.7615172951888125]], [[0.40013178142494787, -5.9135668886811263], [-6.3178391814013999, -5.4996421655908669]], [[-1.6093300428854862, 2.1695837453951459], [-6.8354892555287483, -1.2189772043991018]], [[-3.7396240548505011, -2.4789556751745181], [-2.0744367634211009, 2.6634585421057455]]], [[[1.2977675716193371, 2.9448477764932246], [0.43287721914335719, 6.0012222277334599]], [[4.1727136450676809, 3.9488696902480331], [6.9113917382417114, 8.4182800665295598]], [[1.3995592062995232, 2.4165383175526221], [2.4924010263111382, 5.068546698354008]], [[5.2944767513776592, 5.869798067784699], [5.2793858648505454, 5.445191641177562]], [[5.5331042676248297, -0.86645425862777348], [1.6774950453890334, 2.4179233503996098]], [[5.0837393642357913, 2.473866309142152], [3.5546153574248009, 0.17122811462170517]]]])+(1.-msk_ref)*numpy.array([[[[-0.70430435603666552, -0.21044302248620639], [-4.3156783247004409, -0.44444295817916579]], [[-3.6312225336966613, 0.39377222899310382], [-5.8257838799067176, -7.0235944236438499]], [[-4.157470203249499, -4.9361664376866345], [-6.4136153210625189, -6.624628326779999]], [[0.53702074983376136, -5.7766779202723129], [-6.1809502129925864, -5.3627531971820535]], [[-1.4724410744766727, 2.3064727138039594], [-6.6986002871199348, -1.0820882359902884]], [[-3.6027350864416876, -2.3420667067657046], [-1.9375477950122875, 2.800347510514559]]], [[[1.1095242302587298, 2.7566044351326173], [0.24463387778274992, 5.8129788863728526]], [[3.9844703037070737, 3.7606263488874259], [6.7231483968811041, 8.2300367251689526]], [[1.211315864938916, 2.2282949761920148], [2.304157684950531, 4.8803033569934007]], [[5.1062334100170519, 5.6815547264240918], [5.0911425234899381, 5.2569482998169548]], [[5.3448609262642224, -1.0546975999883808], [1.4892517040284261, 2.2296800090390025]], [[4.895496022875184, 2.2856229677815447], [3.3663720160641937, -0.017015226738902101]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.84066257618148121, 3.5882497677461807, 3.9089642436103098, 2.1042578525490025, -4.0578030156444465], [-4.5145600845858693, 2.0937762984251895, 0.1443283097492678, -2.9872570109176366, 3.7502863952172518], [-2.9152048137356559, -0.012488317040983077, -3.1921019674403093, -3.565133176354319, -4.4845724748756277], [-4.6217214142860152, -0.066600209173351743, 4.746351261952654, -3.61020207828071, -3.0493286204460768]])+(1.-msk_arg0)*numpy.array([[-4.4511614372095156, -4.0793090726696244, 0.56722362190377762, 3.893778153504643, 3.0306633058526504], [0.15329522395900952, -4.4403535423820735, -2.5543637725183412, -1.3419220374141636, -3.9551311753780425], [0.45140739265818652, -0.98574960164947356, 3.7547849171494381, 4.7543377403957514, 1.7588290621436009], [-0.72803143736949139, -3.1399757537523012, -0.92340777575882349, -0.46596734126389183, 2.0530422126698697]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[1.1071705856164336, -4.7929245709325263, -2.7494164038284108, 4.7571582696505761, -0.29765014996915795], [-3.6412522674880021, 0.080703028605890736, -1.7150509663482927, 0.40424178796057397, 4.3023173746220937], [-3.8524327273864891, -2.4696650247401388, 3.5003717566410852, -2.8027151066052869, -1.5471050651749696], [1.7734039202562482, 4.8540335486691646, 1.2666801315239704, 1.71754031008191, 2.6166160228918258]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.26650800943495234, -1.2046748031863457, 1.159547839781899, 6.8614161221995786, -4.3554531656136044], [-8.1558123520738715, 2.1744793270310803, -1.5707226565990249, -2.5830152229570627, 8.0526037698393456], [-6.767637541122145, -2.4821533417811219, 0.30826978920077597, -6.3678482829596064, -6.0316775400505973], [-2.8483174940297671, 4.7874333394958128, 6.0130313934766244, -1.8926617681988001, -0.43271259755425096]])+(1.-msk_ref)*numpy.array([[-3.3439908515930821, -8.8722336436021507, -2.1821927819246332, 8.650936423155219, 2.7330131558834925], [-3.4879570435289926, -4.3596505137761827, -4.2694147388666339, -0.93768024945358963, 0.34718619924405125], [-3.4010253347283026, -3.4554146263896124, 7.2551566737905233, 1.9516226337904645, 0.21172399696863131], [1.0453724828867568, 1.7140577949168634, 0.34327235576514692, 1.2515729688180182, 4.6696582355616956]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.5584222055184416, 2.7387951229697594], [-3.1969155250676851, 2.6192158610772207], [0.67217531230575034, -3.3770534714400311], [-4.0584112917678663, 3.9625793446964312], [1.9653917934585943, 1.5771634579923122]], [[3.9224999114987291, 2.9420042229802101], [2.6045355859771799, 3.1641939523435525], [0.47872495987600416, -4.3233521277581488], [-3.4369557160122644, 1.8586525970836476], [-3.153732625290103, -4.0209137251960296]], [[-4.1073949985359315, 2.1582246345539851], [0.90079717274635485, 2.2215323996014034], [0.50333960347189954, 1.0384816025141124], [2.9712038901790381, -0.20904156493273796], [2.3489039542744461, 4.6292959506621525]], [[4.0005802185066983, 2.6147598744783105], [-0.46145322170692449, 1.7118275056269008], [-0.64923204385211442, -1.5244809587054364], [0.41022110145283097, 3.5050025481425813], [-4.6631859800969302, -4.5353143410130823]]])+(1.-msk_arg0)*numpy.array([[[0.26273519317430516, -2.6605792151985606], [2.6625069317619401, 3.9556164606740456], [-4.240948633035825, -3.8095144773777081], [2.2653656316726831, 0.30983201365400781], [1.6094119549458821, 4.5677809393060045]], [[2.0767893904886954, 2.2418354386437178], [-2.967229111844023, 2.6307404394766865], [3.8669175749129341, -3.8196594536145314], [-2.5681306545293046, 1.6454151321666011], [0.6909696124020126, -0.92115367847899865]], [[1.7021721809782715, -2.6507573391507968], [-4.4484371410427901, -2.9324489101858218], [0.74695088007986055, 3.4433288807686253], [2.3292894923367662, -4.5967759513370545], [-3.1452063269882204, -1.341153713813823]], [[-0.39628736155874034, -3.3264260599415971], [3.1908618481584767, 4.9034866406172419], [-4.1045773608869744, -3.4456276640266403], [4.8827784552618798, -4.5568759693284822], [-3.6065212007579452, 0.90457480380278099]]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[-4.0480555971913716, -2.8860886118606635, 0.57890273527955927, 1.0667651458000851, 0.73391707371227266], [2.5032103426487673, -0.88806317514484157, 4.5907920016002564, 4.4447434305239639, -2.3678179397823005], [0.43270636632995618, 1.3492899661727806, 4.9159556399590638, -2.8120861751146875, 0.86200282802934858], [-3.4762134539742986, -2.8562610712944791, -4.9201528648132351, -2.760538335823346, -1.8555825242145172]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.4896333916729301, -1.3092604742216123], [-6.0830041369283485, -0.26687275078344275], [1.2510780475853096, -2.7981507361604718], [-2.9916461459677812, 5.0293444904965163], [2.6993088671708669, 2.3110805317045848]], [[6.4257102541474964, 5.4452145656289774], [1.7164724108323384, 2.2761307771987109], [5.0695169614762605, 0.26743987384210754], [1.0077877145116996, 6.3033960276076115], [-5.5215505650724035, -6.3887316649783301]], [[-3.6746886322059753, 2.5909310008839412], [2.2500871389191355, 3.570822365774184], [5.4192952434309634, 5.9544372424731762], [0.15911771506435057, -3.0211277400474255], [3.2109067823037947, 5.4912987786915011]], [[0.52436676453239972, -0.86145357949598811], [-3.3177142930014036, -1.1444335656675784], [-5.5693849086653495, -6.4446338235186715], [-2.350317234370515, 0.74446421231923532], [-6.5187685043114474, -6.3908968652275995]]])+(1.-msk_ref)*numpy.array([[[-3.7853204040170665, -6.7086348123899322], [-0.22358168009872337, 1.0695278488133821], [-3.6620458977562658, -3.2306117420981488], [3.3321307774727682, 1.3765971594540929], [2.3433290286581547, 5.3016980130182771]], [[4.5799997331374627, 4.7450457812924851], [-3.8552922869888646, 1.7426772643318449], [8.4577095765131904, 0.77113254798572495], [1.8766127759946594, 6.090158562690565], [-1.6768483273802879, -3.2889716182612991]], [[2.1348785473082277, -2.2180509728208406], [-3.0991471748700095, -1.5831589440130411], [5.6629065200389244, 8.3592845207276891], [-0.48279668277792132, -7.4088621264517425], [-2.2832034989588719, -0.4791508857844744]], [[-3.8725008155330389, -6.8026395139158957], [0.33460077686399758, 2.0472255693227628], [-9.0247302257002104, -8.3657805288398759], [2.1222401194385339, -7.3174143051518286], [-5.4621037249724624, -0.9510077204117362]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.3045867459642846, 4.5760994811379891, 4.433559973175754, 3.001538367492687, 3.1329907292706789], [3.3860607763543484, 4.9874354292595484, -1.2604959639113988, -0.95819311386677786, 4.3971873657280938], [-4.6375883549810712, 0.98500177693150093, -3.839910990127354, -3.1193774543285779, -1.3952113881638417], [1.8817935542982669, 1.8810454819920768, -0.45698518423819312, -2.5385375111721831, 3.6421866751448135]])+(1.-msk_arg0)*numpy.array([[-2.7786492214604253, -2.9273945863113902, -0.13849841550779729, 0.49482981041542029, -1.0720723424272913], [-2.921729663331428, -4.9731943724378436, 4.6884704504833756, -3.1779116455931913, 0.62273311265041187], [1.1947274034110711, -1.9498781078157235, -0.30469833547459579, -3.2722216236294432, -1.7113168103588459], [-0.70345364338091354, 3.2225632328845304, -3.5565740216270871, -3.350321895082029, -2.6895642320720148]])
      arg1=Symbol(shape=(4, 5, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-1.2723227725366435, 4.1156471662855072], [4.736303658079672, -2.3288706952792912], [-0.33618275466784109, 4.6179885650091794], [-1.6332713676731689, 2.9554050812054875], [-3.4321326588338041, 2.7947270252593484]], [[0.4606900549085502, 2.4256459537775772], [-1.2990273327873902, -2.4705885172880393], [4.2563686977174182, 3.7274490362815254], [2.0225448072503589, -4.299139125243773], [-2.0249444736518818, -3.6517045402264414]], [[0.55815617198985024, -4.9631536206201554], [1.3215013398759829, 1.644728736309915], [0.9864478776751362, 2.5348632315778099], [1.2073968973577482, 3.190927289226682], [-2.8690256522465454, 2.3056157757145606]], [[-4.8088322718147358, -2.639131065726402], [-2.8358864680675056, 3.0038173488856792], [-3.1856997449764526, 2.0624681154275404], [3.9055749413295366, -0.021294612001295121], [2.9034110074011412, 3.4050882531381514]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.0322639734276411, 7.4202339122497918], [9.3124031392176612, 2.2472287858586979], [4.0973772185079129, 9.0515485381849334], [1.368266999819518, 5.9569434486981745], [-0.29914192956312524, 5.9277177545300272]], [[3.8467508312628986, 5.8117067301319256], [3.6884080964721582, 2.5168469119715091], [2.9958727338060194, 2.4669530723701265], [1.064351693383581, -5.2573322391105508], [2.372242892076212, 0.74548282550165235]], [[-4.079432182991221, -9.6007419756012276], [2.3065031168074839, 2.6297305132414159], [-2.8534631124522178, -1.3050477585495441], [-1.9119805569708297, 0.07154983489810407], [-4.2642370404103875, 0.91040438755071884]], [[-2.9270387175164689, -0.7573375114281351], [-0.95484098607542878, 4.884862830877756], [-3.6426849292146457, 1.6054829311893473], [1.3670374301573536, -2.5598321231734782], [6.5455976825459548, 7.047274928282965]]])+(1.-msk_ref)*numpy.array([[[-4.0509719939970683, 1.3369979448250819], [1.8089090717682819, -5.2562652815906814], [-0.47468117017563838, 4.4794901495013821], [-1.1384415572577486, 3.4502348916209078], [-4.504205001261095, 1.722654682832057]], [[-2.4610396084228778, -0.49608370955385084], [-6.2722217052252338, -7.4437828897258829], [8.9448391482007938, 8.415919486764901], [-1.1553668383428324, -7.4770507708369642], [-1.4022113610014699, -3.0289714275760296]], [[1.7528835754009213, -3.7684262172090843], [-0.62837676793974051, -0.30514937150580845], [0.6817495422005404, 2.2301648961032141], [-2.064824726271695, -0.081294334402761237], [-4.5803424626053912, 0.59429896535571469]], [[-5.5122859151956494, -3.3425847091073155], [0.38667676481702484, 6.2263805817702096], [-6.7422737666035397, -1.4941059061995468], [0.5552530462475076, -3.3716165070833242], [0.21384677532912644, 0.71552402106613666]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.4504577011815272, -4.0153477679460634, -0.12905060006893176, 3.6955404318643819, -1.1887007035420316], [1.8479091540970591, -4.6668801669753197, 0.55734519535370097, 0.88290357242015816, -1.9793438518147011], [-1.683374495910205, -2.8227284048119228, 2.0770579487798324, 2.7101267640809041, -4.9483101595525998], [-2.6706822951115337, 4.6153091032676574, -4.5186461130373381, -3.9633403788146007, -3.7077957392161465]], [[-0.62416871783799088, 2.1102039967693287, 0.48499824847530792, 3.8271120417126667, 0.44276670853792854], [3.4571711084958618, -3.65272245251967, 2.5476488192359668, 3.930459585205643, -0.73697835337932638], [0.40466412768700799, 1.9155004289767783, 1.5045538898288182, 3.8385997647392109, 3.5751981065408192], [4.491952110479156, 1.1022282976500408, 1.1480023909200723, 3.4714435086097737, -3.2082311867733582]], [[0.6451393108895731, -0.021818471933174877, 1.5673084355456615, -3.3334462387904162, 1.4322865194270271], [-2.0420777417559011, 0.30545885708818243, 1.2164499679360885, -2.1089390146890086, 1.971156613644923], [-4.8513167689207881, -1.0380109185567976, -4.2507126983648833, -3.1608118785833539, -3.0456823036103953], [3.3244766103564896, -2.0636181222904346, -2.9653758354515771, -4.7638204516993765, 3.5131915686431565]], [[1.0230540031697402, -2.3630789950381947, 2.8608897101185784, -3.0168994731214696, 4.4159439452366005], [4.7509657252474433, 2.6168496063263822, 3.118464048971072, -1.2968772056450248, -0.84381210096575199], [0.84363943157639198, 2.9589681650519397, 4.2443047863359205, 1.3942937108928461, 0.086443518813441145], [-3.9905635366217309, -0.36816376775943027, 0.47257744959792269, 1.0267555978650584, 3.1210575010874368]], [[1.4999383782644369, -4.7514278186115453, 2.8681669725416645, 0.89092727830013985, -3.7437885940293603], [0.16322152185881755, 0.7580182927896999, -2.3602904110698097, 2.1301882468331108, 0.21446319028591709], [3.5274202547619495, -0.70382059395298047, 3.5588008261649406, -2.2333423634157832, -2.6498127720967779], [1.1613064931276851, 2.8824101976573147, -0.77198578668654783, -3.8884849439614211, 1.5933624781855205]]], [[[1.0046668079361751, -2.2644355480237364, -4.0670586702555847, 3.9215731939803451, 0.14719183570407424], [4.2787382951496529, 4.1235999332566493, -2.2077451603972742, 4.0662617722414893, 3.6666002965955062], [3.7732791179858722, -2.1874562019005426, -2.7656849317449019, -3.0822071055917744, -3.974981899098069], [2.8916167632970264, 3.9689921811639746, 4.9580678900410273, -4.8308178164913018, -4.9899111755657435]], [[-4.4760659654400499, 4.3805324189792181, -0.75488709745645188, -3.2509158498638815, -0.36048956089753315], [3.7502461024878713, -3.4696712835265151, -4.6006851465713172, 2.2064961526202342, -3.6971099810045738], [3.35495079403454, 4.0125522888879743, -3.1566598949769329, 1.5947199135290386, -2.1988188259508412], [1.333533220653834, 3.7184869133926775, 4.4735482252447554, -1.310310106046213, 2.3860190166018267]], [[3.3966675526119658, 2.6229722560127877, 4.285707363101535, -3.0981058993224342, -1.2279146062644775], [-3.3570943894467509, -1.7128614003527987, 1.9799614950797073, -1.3033167866465756, -1.0259449653824948], [-3.0431714194526593, -2.8146393937310252, -0.95662263276863868, 3.4388707708708885, -0.68930874741292225], [0.44459058645562077, 3.1813192405041697, -1.7025697001325248, 1.4488073489642526, 1.6487413458681885]], [[1.414389226997085, 3.4174804291138834, -2.8701905522203974, 1.0714672720784337, 3.8105341807185376], [2.5118508802539203, 4.9782408183484108, -0.59475370951758055, -3.8721802519065829, -0.079397905229438237], [-2.9482765461319516, -1.4706711286393181, -4.5172727213016231, -2.0557720759416878, -3.5925517333075039], [1.7611882401815802, -1.244054349447512, 2.6682981332460116, -4.3484938031706024, 0.26678266589533894]], [[-0.52558742681854476, 4.7176335231387085, 1.1698759227744855, -3.0221745575263514, 1.6744731171851557], [1.0236186731842292, -4.6779450847912596, -2.7455249485160547, -0.085631858796457294, 4.8368555508135369], [-1.2948826103031341, 0.23830115161337773, -4.5712564488999465, -4.0230995600395794, 1.0736601173574059], [1.9580991815528499, -1.9404736277401549, 4.3766638484126865, -1.0729568048709548, -0.54112095324797238]]], [[[-1.7145283395826674, -3.0280293660000157, -0.27369278174419875, 2.560775170385182, 1.6283670823506711], [4.4424965420044753, -3.4767725535101093, 0.28715916247219919, 1.5049347124530454, -0.20979421165664114], [-1.2000743150700375, 0.41411586166702907, -0.478483687508926, -2.6585163433740897, -4.2751366196246918], [4.1434713588377114, 3.2482223071438714, 0.97350443976083945, 1.5647808725709726, 3.4327275471106908]], [[4.2315870591956433, 2.6042235673921246, 0.50291970383358642, 1.4362946520343591, 2.5287901916417974], [1.5341138388371771, 0.3200774082593707, 3.5940859242347969, 4.698114759714402, 3.6769144246604526], [-0.21416386274591659, 0.89130168395310516, -3.421819313796731, 4.389881315199494, -2.6862366650151168], [-1.1897936204413586, -3.698415982499351, 3.4289995176685544, -4.9572529947605437, 3.4871113522383865]], [[-2.9485719753181328, 4.710799408637218, 4.2053142465429989, 1.6686319053428056, 1.1963332035114984], [1.5968311815192706, -3.5418821464337569, 0.38691958540102078, 2.6777842334086372, -4.245880113820343], [3.8625138294042127, -1.6212762879250775, 4.0776764909630163, -1.6719303429726651, 3.0524473148992097], [3.3159643688620815, 2.2013319409420404, 2.3476814379620281, 1.2184018613550425, -4.7801590259909945]], [[-0.89145163037406494, -3.7407530454660765, -2.2431621374561028, 2.7936718890365846, -2.2094024777784793], [-1.0662069562082843, 1.9157166715565044, -0.54081878219643809, 3.7056297079300187, -3.5916713375823814], [2.859684777611621, 2.0587451176383862, 2.5681464301542647, -0.10498316393116625, 4.3826947863183072], [-2.9794778049652626, -4.9632727613516758, -0.43083348523310949, 0.51047153186897098, 1.702269893514849]], [[0.66918845446260811, -0.51681426376098383, 0.056641157511041484, -3.4773339959854521, 0.8020917280608586], [-4.5468383687125469, -2.8806657937997726, 4.6056493949377195, -2.1990483885268519, -4.4875056730210741], [-2.8676686820244592, 0.47891499175316898, -2.7847466083556691, 4.5769451087956234, 3.5210051911817715], [-0.89258700885301767, 4.1244541639548924, -2.5366607754056725, 1.7907419978539991, 4.8242643155714244]]], [[[-2.1726635374894721, -0.36596673450501083, 2.7024109108023175, 2.9312417568870028, -0.09219637692438809], [-2.3978974390654226, 1.0914770750314142, 1.5513657974394626, 3.3869428897860114, 1.9763993959060606], [1.607288072073894, 4.4367706660882629, -1.2819955863995181, 3.3432655231471777, -2.4880193470714564], [-0.71209648047323348, 3.15861208672057, -0.94959438365867932, -2.8530362879638185, -3.5545806820697736]], [[3.3422995605969525, -4.7164440206504761, 4.185884983185419, -0.56668139705458209, -2.3547973013149859], [-3.9578860736945085, 3.5037626185502351, 1.4331424024464958, -0.46274713162144732, -0.37924514257011932], [4.1277221331215603, 0.49164024295080111, 3.7665632764238595, -1.9250214438359956, 2.32139407597111], [-1.9759157552092752, -1.4397788581816338, -2.0974879130831159, 0.74872752863303393, 0.0020264078845508848]], [[1.5658816602854717, -4.6949259004898289, -2.5816236565268835, -2.2963777155949048, 2.6762478300543746], [-0.75121429397936534, 2.4400218967298448, 0.17843793271146691, 2.1686296606972144, -1.9138762884316152], [4.0913096896372654, -0.5738740711242416, 3.4690212359282029, -0.62504937422476914, 1.4906947523799925], [2.2542845011208019, 4.512731439414301, 2.9572769788364752, 1.8356431323122742, 1.8326017322738819]], [[2.6355833625367611, 3.8146878992434026, -0.45014335865187771, 0.22589643498656908, 1.3936387083735973], [-2.9048295071893193, 1.0854759106553358, 0.59695170806410847, -3.9420298708071222, 1.5001320528718773], [0.31317773708300489, -2.1257530713558417, 1.7459541543514634, 1.9662037650251474, -3.0625190374777658], [-1.36795248880478, -4.9664343788550172, 3.7374997043944962, -3.0763344562790373, 3.0880268075240735]], [[-0.77557480789944222, 1.7299368479551429, 1.8443572210962067, 2.5798394383553527, 2.4509551132709415], [-3.627209113643505, -4.9512023182173071, -4.1650655884789352, 4.9041925034497496, 4.1515261220980406], [-1.3762841792599669, -0.16583249273789757, 0.35858876456218791, 4.7763560893056969, -2.2286019671361554], [4.9324324747020363, 1.7385394150964917, 1.761157187753545, -3.0151718073074352, -3.5942529948694601]]]])+(1.-msk_arg0)*numpy.array([[[[-2.9346147028260603, 0.57348276077576266, -1.7099605886065774, 1.887189786435326, -1.8819981714542577], [-4.4488976489058008, -2.4505077258295485, -3.9855711079369351, 1.6904070356797343, -0.45182227517287377], [1.9236950605766694, 3.3906006667730644, 2.4548123804251176, 4.5953238078279348, 1.502220582065295], [-1.2150397466917253, 1.134845920381987, 1.6066925801258662, 3.1951986562604553, 4.5059742998399201]], [[0.064139454960724152, -0.31053164428882951, 3.7884117895977436, -2.3866626792243162, -3.8139252232270904], [0.551783804730964, 1.6668280992223794, -0.0055784442849606819, -0.16284955434265136, 4.614386921951235], [-0.21004727419532276, -1.545665835236302, -3.3225179105080329, 3.3982476475735872, 3.4931948423201291], [3.3795443397826848, 2.2399843046708225, -4.3113532896179301, -4.868299547693498, -2.2081179020374719]], [[2.8881517813108246, 3.344425968834706, 2.5892092467225316, 0.32445246005092621, -2.3047800589389302], [0.36939195733251928, 3.8102953728959648, -3.9702037354524613, -2.4129979828299963, 1.200035172974669], [-4.5748790344846926, 0.79708016429043038, -0.83486603140867111, 2.5136430508652463, -3.6201897137713024], [-0.74569882009679489, -2.1280911297945417, -2.0989363915745018, -0.79974500008244576, -2.6529039964560139]], [[-3.4619357036027329, 1.3907226090827152, 4.1646612838149526, -1.0308254977034812, 3.0811402726398835], [4.1907671693082928, 3.534987552349234, 4.2906414625349889, 1.1810347151232712, -4.0185490764586138], [0.32796656225351661, -2.6525485081567712, 4.1689754568894735, -4.4155227285436274, -2.4857692083700655], [-1.0411620683792124, -2.3486675139365798, -4.8544498430743079, 2.0361225525924347, -1.0277875131540659]], [[2.9204487770920151, 3.6971803240762551, -4.663321603337728, -3.1468215384261478, 2.5130284066607835], [2.0866215781393489, -0.063132014329820407, -4.3025305686383071, 3.1570600385684919, -2.5345358602089241], [-0.82083144700263677, 2.5311733452066987, 4.9005378959665578, 1.9948671988744806, -3.0241802288625266], [0.65896374548673542, -4.7652922074788853, -0.89758005397149176, -3.7883703783740721, 4.8818238172970947]]], [[[-4.1969052033139134, -0.60751159203252492, -2.8255536202119211, 1.2813229864578171, -0.65105038269383719], [-0.47128077347101449, -4.6022482154246642, 2.3770664972855604, -3.1762292615416987, -2.7987240043745363], [4.598340411234398, 4.6578575544431668, -2.6185652956575458, -2.9026206015701961, 0.50013616761938451], [0.62840874471669661, -4.0525535934954515, -3.1575809644441977, 4.4052400388710904, -4.3463814972323123]], [[3.65196207712399, 3.8184847051658508, -1.3307148898465249, 0.32496251380472874, -3.1362753713648148], [-0.15865079191918774, 0.46486206125549501, -3.7279113606656611, -2.5553409578476662, 2.1804384669757235], [-4.8185241176965086, 2.2405495468047443, -1.4695492725263959, 0.7881048174632399, 1.9245059687783357], [-4.606877695121776, 0.36083101548452934, 1.1918012062505898, -0.97092877687390367, -4.8809316310343975]], [[2.1823807959466262, -2.7271753093093674, -2.600139610096984, -3.672061456675598, -4.2710745455095633], [-2.3315919302707533, -0.61865637823522945, 1.3507636000546235, -3.7855680738748987, 4.6404143273669813], [0.056703660906736175, 3.5836315087621866, 0.56967051667156454, 0.45666721535891064, -3.2058153768157815], [-0.16347341953070327, 2.4918805353409326, -3.9594480550038194, 0.51409393380184376, 1.7011866963118223]], [[0.46074187718373327, -3.9810897080239438, 1.4007339635124181, 2.5148432531305778, 1.4602396483764304], [1.7793410513973127, -3.7894388255297784, -1.40971964754391, 1.7196966548782369, -1.3748964505544672], [1.4367842541994245, 4.3267174395882826, 1.0417537154764211, 2.7218048494182341, -1.149056441677978], [4.9378320908526305, -3.6554078771225429, -1.7381997861097664, 1.1343951078557755, 0.32333748040906407]], [[3.3103089990563443, 3.0769200744152041, -4.6570739898937576, 0.50057718301567267, 1.4305913658921634], [4.1817059892011255, 3.4675293814042121, 0.54590641546260521, -4.2497841531673757, 1.6766050350559425], [2.8316142212257036, 2.7134044287315469, -2.2733288752096357, 0.44998958099791686, -1.3727272845022198], [4.5329461045540693, 1.0592237650958038, -2.0608942926886407, 2.1110662209615336, 2.0986524163193785]]], [[[4.8832260503013885, -2.7337395723717206, 2.8487488060093833, -4.5786219380504622, 1.4868954173243605], [4.02991617705724, 4.6692734176858899, 3.7530734753566524, 0.74573678266431465, 3.7242303705450155], [-1.009813408975849, -3.0371347220359004, 1.9508048523918653, 3.6350761768605029, 4.1399577841130295], [3.355174534095946, 2.074984596402432, 1.6103848782756094, -2.4840509764410337, 3.4498058962024931]], [[-4.1727071809506402, -3.2122753193981479, 0.96371555230617822, -3.9169985821879374, 4.5128922356140553], [3.9686077047171704, -1.4186887994238386, -2.5071690520281642, -0.15661239962280327, 2.3696120545414621], [2.9712854518208331, -1.0287913330643828, -2.2303899255173878, 4.29657138594796, -0.51398502795201839], [1.4312080539897218, 0.10887787207008781, -1.2790804034900329, 2.8290394285538305, 4.122150473324206]], [[3.5532874252977429, -0.7252949122935668, 3.0378907575064602, -0.32222863133714519, 3.3105520458301072], [1.881422513055897, -1.1365641707737781, -2.3083406004275195, -4.8552247390886265, 0.54047785397535364], [-4.7883117057150919, -3.8900469134701723, -1.9345832554212574, 1.7384562759644684, -2.1237223119489181], [2.1430637441761515, -3.8359335787349158, -1.0622794423152437, -3.8624009062049094, -4.158532389659678]], [[1.523076258352595, 1.7400876521644779, 0.90619996429535377, 4.8668017195522602, -0.78467131225433562], [2.0422060866476963, 0.68453580982526407, -4.546723751353019, 4.0180361795518387, 1.5584705602163478], [-1.5646536568779901, 1.4526946615908631, 1.5036870384030898, 0.074193662267055593, -2.010797604984579], [4.4414416487721269, -0.88722893708727746, 0.45895016237285269, -1.5666099939206046, -1.6570703018144375]], [[-1.6041494764726782, -4.0976936455641155, 1.8325179268048046, -1.9854040319072954, 1.7384040056739387], [2.4714112006345959, -1.794106246580105, 2.8363306690676273, -0.97459766192217856, 3.4050186120421131], [4.5837929182854218, -4.0175889564078693, 0.031378606798941888, -3.0065331405354834, 1.494191219096118], [1.5473681763350164, 4.2765057515314062, -3.3960990134729583, 0.62525578834059825, -1.245270411545917]]], [[[-3.3224192977133926, -0.31263333611013611, -4.3838286170162535, -3.7948990215995027, 0.26772481514412938], [-3.8667369264494922, 0.35298062595059143, 4.9149887608469207, 1.3181260751274246, -1.2647909818281478], [-3.3846858411536598, -3.0126167201147092, 1.1264926833551145, 2.3494990310157124, -0.8740326401091103], [-0.34720574899498757, 2.1094497101968983, -1.597965564820397, 3.4907041435006541, 4.2185832016121179]], [[-1.3883142589927324, 4.7423976547667781, -1.8052003094459201, 3.9943052090191262, -4.4853292990292326], [-4.7582572828158129, -1.143956039004741, 4.89146330351989, 4.6205716314526555, -0.92561334099051962], [-0.2355304959950093, -1.6031761701887604, -0.25937457060790514, -2.0126291935909082, 3.7343610581519062], [-4.9475533214972884, 2.2873248262434558, -2.5975563183702732, 4.2105079971659993, 3.9838600067635248]], [[-3.4296118840607779, -2.094161747793668, 1.5361469385596997, -2.7426132446043141, -3.3067868350678387], [3.6351192696025194, 2.754336752228328, 0.78017913620624491, -0.21380301232799503, 2.9002229024320521], [4.7790532873688214, -3.1532908617218389, 3.7719896751232103, -2.8070917388646333, 2.5498314950975196], [-2.3896125207645822, 3.4555706256328502, 1.8959573511407921, -0.33370785354457144, 1.8326736438119546]], [[-4.7802860351543783, -2.4901769624321424, -2.9026720934387318, -2.9910251103973917, 2.7974257319350944], [3.4256173242927588, -1.9216744550886089, -0.95979537725362007, 1.5796975294580173, 3.6097153246357667], [2.5698213862405241, 4.0863090732765137, 3.4873712595251671, -3.1474222475211722, -4.1589607729474398], [2.6059704225696976, 2.1651643960824742, 0.19659222735316106, -3.8877878695231405, 0.68461632042254372]], [[3.6505964470632684, 2.8367383598695683, -0.72022992749354309, 0.58862019947225708, -1.0616310598592915], [-0.7085132466938493, -3.8120727795425644, -4.3763162709358969, 3.2382693591675853, 3.1847025360490591], [0.70740648890523339, -1.6436787960958132, 3.4001417086575181, 3.5843899372687567, -2.0646759686361307], [2.7928710763242206, 2.4327240302629516, 4.1171482494332601, -4.1598195747126629, -3.6607501205840398]]]])
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[4.668644009602259, -3.6379291492222965, 0.90100064817997882, -3.9786889588938177, 3.383402290060106], [-3.6787394096943951, -4.0949892014097289, 1.3253076050594315, -4.4564714427279881, -3.9924578729186444], [0.5520831783659581, -2.657381601931208, -4.0352709122256289, -1.7308320192602791, -1.8387782007586284], [2.1460357808167085, 4.8000336780559714, -1.2905254572515821, -2.4407402480948051, 3.6765379620437795]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.2181863084207318, 0.6532962416561956, 4.5395934095333272, 8.3641844414666409, 3.4799433060602274], [6.5165531636993181, 0.0017638426269392937, 5.22598920495596, 5.5515475820224172, 2.6893001577875579], [2.985269513692054, 1.8459156047903362, 6.7457019583820914, 7.3787707736831631, -0.27966614995034078], [1.9979617144907253, 9.2839531128699164, 0.14999789656492091, 0.70530363078765834, 0.96084827038611254]], [[-4.2620978670602874, -1.5277251524529678, -3.1529309007469886, 0.18918289249037024, -3.1951624406843679], [-0.18075804072643464, -7.2906516017419669, -1.0902803299863297, 0.29253043598334649, -4.3749075026016229], [-3.2332650215352885, -1.7224287202455182, -2.1333752593934783, 0.20067061551691445, -0.062731042681477334], [0.8540229612568595, -2.5357008515722557, -2.4899267583022242, -0.1664856406125228, -6.8461603359956547]], [[1.5461399590695519, 0.87918217624680395, 2.4683090837256403, -2.4324455906104374, 2.3332871676070059], [-1.1410770935759222, 1.2064595052681613, 2.1174506161160673, -1.2079383665090297, 2.8721572618249018], [-3.9503161207408093, -0.13701027037681879, -3.3497120501849045, -2.2598112304033751, -2.1446816554304164], [4.2254772585364684, -1.1626174741104558, -2.0643751872715983, -3.8628198035193977, 4.4141922168231353]], [[-2.9556349557240775, -6.3417679539320124, -1.1177992487752393, -6.9955884320152872, 0.43725498634278281], [0.77227676635362563, -1.3618393525674355, -0.8602249099227457, -5.2755661645388425, -4.8225010598595697], [-3.1350495273174257, -1.019720793841878, 0.26561582744210277, -2.5843952480009715, -3.8922454400803765], [-7.9692524955155486, -4.346852726653248, -3.506111509295895, -2.9519333610287593, -0.85763145780638084]], [[4.8833406683245428, -1.3680255285514393, 6.2515692626017705, 4.2743295683602458, -0.36038630396925431], [3.5466238119189235, 4.1414205828498059, 1.0231118789902962, 5.5135905368932168, 3.597865480346023], [6.9108225448220555, 2.6795816961071255, 6.9422031162250466, 1.1500599266443228, 0.73358951796332805], [4.544708783187791, 6.2658124877174206, 2.6114165033735581, -0.50508265390131513, 4.9767647682456264]]], [[[-2.6740726017582199, -5.9431749577181314, -7.7457980799499797, 0.24283378428595004, -3.5315475739903208], [0.59999888545525781, 0.44486052356225425, -5.8864845700916693, 0.38752236254709427, -0.012139113098888821], [0.094539708291477176, -5.8661956115949376, -6.444424341439297, -6.7609465152861699, -7.6537213087924645], [-0.78712264639736862, 0.29025277146957951, 1.2793284803466323, -8.5095572261856969, -8.6686505852601385]], [[-8.5710551668497779, 0.28554321756948919, -4.8498762988661808, -7.3459050512736104, -4.455478762307262], [-0.34474309892185762, -7.564660484936244, -8.695674347981047, -1.8884930487894946, -7.7920991824143027], [-0.74003840737518889, -0.082436912521754557, -7.2516490963866618, -2.5002692878806902, -6.2938080273605701], [-2.7614559807558949, -0.37650228801705143, 0.37855902383502649, -5.4052993074559419, -1.7089701848079022]], [[4.7219751576713973, 3.9482798610722192, 5.6110149681609665, -1.7727982942630027, 0.097392998794954], [-2.0317867843873194, -0.3875537952933672, 3.3052691001391388, 0.021990818412855884, 0.29936263967693666], [-1.7178638143932279, -1.4893317886715938, 0.36868497229079278, 4.76417837593032, 0.63599885764650921], [1.7698981915150522, 4.5066268455636012, -0.37726209507309338, 2.774114954023684, 2.97404895092762]], [[-3.0420822157309031, -1.0389910136141047, -7.3266619949483855, -3.3850041706495544, -0.64593726200945056], [-1.9446205624740678, 0.52176937562042269, -5.0512251522455687, -8.328651694634571, -4.5358693479574264], [-7.4047479888599401, -5.9271425713673063, -8.9737441640296112, -6.5122435186696759, -8.049023176035492], [-2.695283202546408, -5.7005257921755002, -1.7881733094819765, -8.8049652458985896, -4.1896887768326492]], [[-4.5180452997371887, 0.72517565022006414, -2.8225819501441589, -7.0146324304449958, -2.3179847557334887], [-2.9688391997344152, -8.6704029577099035, -6.7379828214346986, -4.0780897317151013, 0.84439767789489251], [-5.2873404832217785, -3.7541567213052667, -8.5637143218185905, -8.0155574329582233, -2.9187977555612385], [-2.0343586913657945, -5.9329315006587997, 0.38420597549404212, -5.0654146777895992, -4.5335788261666163]]], [[[-1.1624451612167093, -2.4759461876340576, 0.27839039662175935, 3.1128583487511401, 2.1804502607166292], [4.9945797203704334, -2.9246893751441512, 0.8392423408381573, 2.0570178908190035, 0.34228896670931697], [-0.64799113670407937, 0.96619904003298718, 0.073599490857032102, -2.1064331650081316, -3.7230534412587337], [4.6955545372036696, 3.8003054855098295, 1.5255876181267976, 2.1168640509369308, 3.9848107254766489]], [[1.5742054572644353, -0.05315803453908341, -2.1544618980976216, -1.2210869498968489, -0.12859141028941057], [-1.1232677630940309, -2.3373041936718373, 0.93670432230358891, 2.040733157783194, 1.0195328227292446], [-2.8715454646771246, -1.7660799179781028, -6.079200915727939, 1.732499713268286, -5.3436182669463248], [-3.8471752223725666, -6.355797584430559, 0.77161791573734639, -7.6146345966917517, 0.82972975030717855]], [[-6.9838428875437621, 0.67552849641158907, 0.17004333431736995, -2.3666390068828234, -2.8389377087141305], [-2.4384397307063583, -7.5771530586593858, -3.6483513268246082, -1.3574866788169917, -8.281151026045972], [-0.17275708282141622, -5.6565472001507064, 0.042405578737387373, -5.707201255198294, -0.98282359732641922], [-0.71930654336354749, -1.8339389712835885, -1.6875894742636008, -2.8168690508705865, -8.8154299382166244]], [[-2.622283649634344, -5.4715850647263551, -3.9739941567163819, 1.0628398697763055, -3.9402344970387584], [-2.7970389754685634, 0.1848846522962253, -2.2716508014567172, 1.9747976886697396, -5.3225033568426605], [1.1288527583513419, 0.32791309837810712, 0.83731441089398562, -1.8358151831914453, 2.6518627670580281], [-4.7103098242255417, -6.6941047806119549, -2.1616655044933886, -1.2203604873913081, -0.028562125745430045]], [[-1.1695897462960203, -2.3555924645196122, -1.7821370432475869, -5.316112196744081, -1.0366864726977698], [-6.3856165694711748, -4.719443994558401, 2.7668711941790911, -4.0378265892854799, -6.3262838737797029], [-4.7064468827830872, -1.3598632090054594, -4.6235248091142971, 2.738166908036995, 1.6822269904231431], [-2.7313652096116461, 2.285675963196264, -4.3754389761643004, -0.048036202904629288, 2.985486114812796]]], [[[-0.026627756672763514, 1.7800690463116977, 4.8484466916190261, 5.0772775377037114, 2.0538394038923204], [-0.25186165824871409, 3.2375128558481228, 3.6974015782561711, 5.53297867060272, 4.1224351767227692], [3.7533238528906026, 6.5828064469049714, 0.86404019441719049, 5.4893013039638863, -0.34198356625474791], [1.4339393003434751, 5.3046478675372786, 1.1964413971580292, -0.70700050714710994, -1.4085449012530651]], [[8.1423332386529239, 0.083589657405495288, 8.9859186612413904, 4.2333522810013893, 2.4452363767409855], [0.84214760436146285, 8.3037962966062064, 6.2331760805024672, 4.337286546434524, 4.420788535485852], [8.9277558111775317, 5.2916739210067725, 8.5665969544798308, 2.8750122342199758, 7.1214277540270814], [2.8241179228466962, 3.3602548198743376, 2.7025457649728555, 5.5487612066890053, 4.8020600859405222]], [[0.27535620303388963, -5.985451357741411, -3.8721491137784656, -3.5869031728464869, 1.3857223728027925], [-2.0417397512309474, 1.1494964394782627, -1.1120875245401152, 0.87810420344563234, -3.2044017456831972], [2.8007842323856833, -1.8643995283758237, 2.1784957786766208, -1.9155748314763512, 0.20016929512841042], [0.96375904386921984, 3.2222059821627189, 1.6667515215848931, 0.54511767506069209, 0.54207627502229982]], [[0.19484311444195601, 1.3739476511485975, -2.8908836067466828, -2.214843813108236, -1.0471015397212078], [-5.3455697552841244, -1.3552643374394693, -1.8437885400306966, -6.3827701189019272, -0.94060819522292771], [-2.1275625110118002, -4.5664933194506467, -0.69478609374334166, -0.47453648306965768, -5.5032592855725708], [-3.8086927368995851, -7.4071746269498222, 1.2967594562996911, -5.517074704373842, 0.64728655942926849]], [[2.9009631541443373, 5.4064748099989224, 5.5208951831399862, 6.2563774003991321, 6.127493075314721], [0.049328848400274516, -1.2746643561735276, -0.48852762643515568, 8.580730465493529, 7.8280640841418201], [2.3002537827838125, 3.5107054693058819, 4.0351267266059674, 8.4528940513494764, 1.447935994907624], [8.6089704367458157, 5.4150773771402712, 5.4376951497973245, 0.6613661547363443, 0.082284967174319323]]]])+(1.-msk_ref)*numpy.array([[[[1.7340293067761987, 5.2421267703780217, 2.9586834209956816, 6.555833796037585, 2.7866458381480013], [0.21974636069645825, 2.2181362837727105, 0.68307290166532386, 6.3590510452819933, 4.2168217344293852], [6.5923390701789284, 8.0592446763753234, 7.1234563900273766, 9.2639678174301938, 6.170864591667554], [3.4536042629105337, 5.803489929984246, 6.2753365897281252, 7.8638426658627143, 9.1746183094421792]], [[-3.5737896942615723, -3.948460793511126, 0.15048264037544712, -6.0245918284466127, -7.4518543724493869], [-3.0861453444913325, -1.9711010499999171, -3.6435075935072572, -3.8007787035649478, 0.97645777272893852], [-3.8479764234176193, -5.1835949844585985, -6.9604470597303294, -0.23968150164870927, -0.14473430690216738], [-0.25838480943961173, -1.3979448445514739, -7.9492824388402266, -8.5062286969157945, -5.8460470512597684]], [[3.7891524294908034, 4.2454266170146848, 3.4902098949025104, 1.225453108230905, -1.4037794107589514], [1.2703926055124981, 4.7112960210759436, -3.0692030872724825, -1.5119973346500175, 2.1010358211546478], [-3.6738783863047138, 1.6980808124704092, 0.066134616771307719, 3.4146436990452251, -2.7191890655913236], [0.15530182808318393, -1.2270904816145629, -1.197935743394523, 0.10125564809753307, -1.7519033482760351]], [[-7.4406246624965506, -2.5879663498111025, 0.18597232492113491, -5.0095144565972989, -0.89754868625393414], [0.21207821041447517, -0.44370140654458368, 0.31195250364117122, -2.7976542437705465, -7.9972380353524315], [-3.6507223966403011, -6.6312374670505889, 0.19028649799565578, -8.3942116874374442, -6.4644581672638832], [-5.0198510272730301, -6.327356472830397, -8.8331388019681256, -1.942566406301383, -5.0064764720478836]], [[6.3038510671521211, 7.080582614136361, -1.2799193132776221, 0.2365807516339582, 5.8964306967208895], [5.4700238681994549, 3.3202702757302855, -0.91912827857820112, 6.5404623286285979, 0.84886642985118188], [2.5625708430574692, 5.9145756352668046, 8.2839401860266637, 5.3782694889345866, 0.35922206119757938], [4.0423660355468414, -1.3818899174187793, 2.4858222360886142, -0.40496808831396613, 8.2652261073572006]]], [[[-7.8756446130083084, -4.28625100172692, -6.5042930299063162, -2.397416423236578, -4.3297897923882323], [-4.1500201831654095, -8.2809876251190602, -1.3016729124088346, -6.8549686712360938, -6.4774634140689313], [0.9196010015400029, 0.97911814474877179, -6.2973047053519409, -6.5813600112645911, -3.1786032420750105], [-3.0503306649776984, -7.7312930031898466, -6.8363203741385927, 0.72650062917669533, -8.0251209069267073]], [[-0.44302712428573887, -0.27650449624387807, -5.4257040912562537, -3.7700266876050001, -7.2312645727745437], [-4.2536399933289166, -3.6301271401542339, -7.8229005620753895, -6.6503301592573951, -1.9145507344340054], [-8.9135133191062366, -1.8544396546049846, -5.5645384739361248, -3.306884383946489, -2.1704832326313932], [-8.7018668965315058, -3.7341581859251995, -2.9031879951591391, -5.0659179782836326, -8.9759208324441264]], [[3.5076884010060576, -1.4018677042499359, -1.2748320050375526, -2.3467538516161666, -2.9457669404501319], [-1.0062843252113218, 0.70665122682420201, 2.6760712051140549, -2.4602604688154672, 5.9657219324264128], [1.3820112659661676, 4.9089391138216181, 1.894978121730996, 1.7819748204183421, -1.8805077717563501], [1.1618341855287282, 3.817188140400364, -2.6341404499443879, 1.8394015388612752, 3.0264943013712537]], [[-3.9957295655442548, -8.4375611507519324, -3.05573747921557, -1.9416281895974103, -2.9962317943515577], [-2.6771303913306754, -8.2459102682577665, -5.8661910902718981, -2.7367747878497513, -5.8313678932824553], [-3.0196871885285637, -0.12975400313970553, -3.414717727251567, -1.734666593309754, -5.6055278844059657], [0.48136064812464241, -8.1118793198505301, -6.1946712288377546, -3.3220763348722127, -4.133133962318924]], [[-0.68214887386230005, -0.9155377985034403, -8.6495318628124025, -3.4918806899029717, -2.561866507026481], [0.18924811628248106, -0.52492849151443233, -3.4465514574560392, -8.2422420260860196, -2.3158528378627019], [-1.1608436516929408, -1.2790534441870975, -6.2657867481282796, -3.5424682919207275, -5.3651851574208642], [0.54048823163542492, -2.9332341078228406, -6.0533521656072846, -1.8813916519571108, -1.8938054565992659]]], [[[5.4353092286673466, -2.1816563940057625, 3.4008319843753414, -4.0265387596845041, 2.0389785956903186], [4.5819993554231981, 5.221356596051848, 4.3051566537226105, 1.2978199610302728, 4.2763135489109736], [-0.45773023060989093, -2.4850515436699423, 2.5028880307578234, 4.187159355226461, 4.6920409624789876], [3.9072577124619041, 2.6270677747683902, 2.1624680566415675, -1.9319677980750756, 4.0018890745684512]], [[-6.8300887828818482, -5.8696569213293559, -1.6936660496250298, -6.5743801841191454, 1.8555106336828473], [1.3112261027859624, -4.0760704013550466, -5.1645506539593722, -2.8139940015540112, -0.28776954738974592], [0.31390384988962516, -3.6861729349955907, -4.8877715274485958, 1.639189784016752, -3.1713666298832264], [-1.2261735479414861, -2.5485037298611202, -3.9364620054212409, 0.17165782662262252, 1.4647688713929981]], [[-0.48198348692788606, -4.7605658245191957, -0.99738015471916874, -4.3574995435627741, -0.72471886639552174], [-2.1538483991697319, -5.1718350829994071, -6.3436115126531485, -8.8904956513142555, -3.4947930582502753], [-8.8235826179407209, -7.9253178256958012, -5.9698541676468864, -2.2968146362611606, -6.1589932241745471], [-1.8922071680494774, -7.8712044909605448, -5.0975503545408731, -7.8976718184305383, -8.1938033018853069]], [[-0.20775576090768411, 0.0092556329041988405, -0.82463205496492531, 3.1359697002919811, -2.5155033315146147], [0.31137406738741724, -1.046296209435015, -6.2775557706132981, 2.2872041602915596, -0.1723614590439313], [-3.2954856761382691, -0.27813735766941594, -0.22714498085718926, -1.6566383569932235, -3.7416296242448581], [2.7106096295118478, -2.6180609563475565, -1.2718818568874264, -3.2974420131808837, -3.3879023210747166]], [[-3.4429276772313067, -5.9364718463227444, -0.0062602739538237806, -3.8241822326659238, -0.1003741950846897], [0.63263299987596744, -3.6328844473387334, 0.99755246830899891, -2.813375862680807, 1.5662404112834847], [2.7450147175267934, -5.8563671571664973, -1.8073995939596865, -4.8453113412941118, -0.34458698166251045], [-0.29141002442361197, 2.4377275507727778, -5.2348772142315863, -1.2135224124180302, -3.0840486123045454]]], [[[-1.1763835168966841, 1.8334024447065724, -2.2377928361995449, -1.6488632407827941, 2.4137605959608379], [-1.7207011456327836, 2.4990164067673, 7.0610245416636293, 3.4641618559441332, 0.88124479898856078], [-1.2386500603369512, -0.86658093929800062, 3.2725284641718231, 4.495534811832421, 1.2720031407075982], [1.798830031821721, 4.2554854910136068, 0.54807021599631156, 5.6367399243173626, 6.3646189824288264]], [[3.4117194190632389, 9.5424313328227495, 2.9948333686100512, 8.7943388870750976, 0.31470437902673876], [0.041776395240158415, 3.6560776390512304, 9.6914969815758614, 9.4206053095086268, 3.8744203370654517], [4.5645031820609621, 3.196857507867211, 4.5406591074480662, 2.7874044844650632, 8.5343947362078776], [-0.14751964344131707, 7.0873585042994272, 2.2024773596856981, 9.0105416752219707, 8.7838936848194962]], [[-4.72013734131236, -3.3846872050452501, 0.24562148130811767, -4.0331387018558962, -4.5973122923194207], [2.3445938123509373, 1.4638112949767459, -0.51034632104533717, -1.5043284695795771, 1.60969744518047], [3.4885278301172393, -4.443816318973421, 2.4814642178716282, -4.0976171961162153, 1.2593060378459375], [-3.6801379780161643, 2.1650451683812681, 0.60543189388921004, -1.6242333107961535, 0.54214818656037256]], [[-7.2210262832491834, -4.9309172105269479, -5.3434123415335364, -5.4317653584921963, 0.35668548384028931], [0.98487707619795373, -4.362414703183414, -3.4005356253484251, -0.86104271863678772, 1.1689750765409617], [0.12908113814571909, 1.6455688251817087, 1.0466310114303621, -5.5881624956159772, -6.5997010210422449], [0.16523017447489252, -0.27557585201233081, -2.244148020741644, -6.328528117617946, -1.7561239276722613]], [[7.3271344091070478, 6.5132763219133478, 2.9563080345502364, 4.2651581615160366, 2.614906902184488], [2.9680247153499302, -0.13553481749878493, -0.69977830889211745, 6.9148073212113648, 6.8612404980928385], [4.3839444509490129, 2.0328591659479662, 7.0766796707012976, 7.2609278993125361, 1.6118619934076488], [6.4694090383680001, 6.1092619923067311, 7.7936862114770395, -0.48328161266888348, 0.01578784145973966]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank2_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.9713691536198041, 0.42637179243409307, -0.34288779664696367, -1.0881499135426544, -0.47019951493835244], [-0.67746591055348215, 2.8383599555158225, 3.9216781152105771, -3.0552416093384007, -1.9052011281116341], [1.4985064902749823, -3.8770905646017884, -2.2137389577591859, -4.6863280930944518, 2.8101060333666652], [0.13200058666568637, -2.526898552227077, 2.0186448334427851, -2.7898568042437666, 0.62189478662462694]])+(1.-msk_arg0)*numpy.array([[-1.1102543321111558, -1.4729878117092565, -3.7387617597199685, -0.19100359127751343, 1.3251292055081496], [-0.77293632271298307, -1.4343055942432557, -3.8051222210704227, -2.7989748580981635, 3.9305041783932175], [0.029516705934542031, -0.12001362656287817, 3.8806374199512721, 0.78125979492632069, 1.136923690035637], [0.85900303036237524, 4.6055378318546563, -3.8078344095759786, 1.0661934454222655, 2.7382076885951534]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-0.55559121503340592, -3.2271852497901268, -0.064409860914807382, 1.4695373339795452, -2.7066735080644291], [0.38544256878510819, 4.3388799095893607, -4.1453583662874021, 0.44874219097348877, -3.0124511948114572], [2.850336219324392, 1.1958419612108271, -1.4373120637854617, 2.2629651568632161, 0.81081602382924611], [1.630815153221147, -4.9754592311832848, 4.6454399258943084, -0.1651716024144898, -4.7863546757098447]], [[0.29431931367209785, 3.0576172082301802, -1.2825605025764819, 2.1300643268635788, 0.29628370418706051], [4.0437997403938386, -2.5694558556378122, 1.1442851639575755, -3.9309277308214474, 0.020286472689194035], [-2.039130827848842, 3.121944561892926, -0.49524986435471785, 3.2675805415396013, 2.0933367007952342], [-3.6139544596627191, -2.2919719415246465, 4.1534892920517201, 3.9314756870543217, 0.86742225020656605]], [[-0.24805668391907787, 3.3978140889141226, 4.9061205122505651, 3.8111085511641036, -1.2403555665206447], [-3.5006288643593777, -0.52058249686299618, -4.4244843001572312, 0.68805913738420266, -1.3149050776856797], [1.505608956379664, 1.1621648995382747, 4.2390924616890882, 0.55473527983156057, -2.6430977688893345], [1.8829543085699818, 0.95847447638604955, 1.8604396842267192, 2.1688734997612382, -4.3030850731286696]], [[2.4822058050736455, 2.7228737838759294, 2.4977653353168066, -3.6277414839504729, -3.1593486397071402], [-2.9527741747715375, -0.95441598772679015, -1.2933217444430287, 0.047994190978229767, 0.36367351616465449], [-3.982370391808352, -2.6492281786605689, -2.4196542600438899, 1.6894445137768663, -3.560367988746127], [4.1718260056280929, -0.04928385100094701, -1.3089131746424929, -2.7185411985099717, 4.1550953882506096]], [[1.5747162975258817, 0.35660294862220354, 0.83699511764022461, 3.780880314392725, 1.0711532613283081], [-1.0126200878085356, 2.1974835068430867, 3.5475593744802953, -1.1254314523197015, -4.5949619795685024], [-3.1396162114174064, 0.60406655565259815, 4.9898225494645061, -0.16248504963891364, -2.8646422366455226], [-4.8255938778946872, 0.52734947111661246, -2.732327504617901, -1.1056747944378631, -1.3204633427262467]]], [[[2.2700218836407924, -3.4513313631335696, -0.28111322580986364, 0.099087526035951612, -4.5206076072315549], [1.567070053101336, 3.8130619739003428, -2.9095485483066286, -1.9304520465343069, -4.9260160570555653], [-4.2144307476313312, 1.1446386253586596, -1.7832623993811847, 1.9669215351639382, -4.2671179050483374], [-0.99778127084120527, 4.2934170751266247, -2.3652036982692271, 2.4932932835759756, 3.5823793551918897]], [[-0.44042717040488455, 4.1211063386101578, -1.6934323696618869, -3.3316947362529348, 3.4313423699892471], [0.34471471282766686, -1.6025671511963138, -3.2772593369457068, -0.64752617753619202, 0.35172869109719596], [1.5667916042725381, 3.067563488721099, -3.3807310260544288, 1.3426197963652751, -0.50116514767501208], [0.37189295107078024, -0.69470191242409296, 0.97256276174350553, 2.1101301092313749, -4.4471552482754255]], [[-1.7548292975138216, -1.2354286788595181, 0.7318155533856574, 3.4353655604987203, -0.32469055760336651], [3.1173130504305888, 0.88943477681137217, 4.0636014779170271, -2.0697381354141289, -4.5247109735157123], [-3.1093128293130579, 3.4742829147409289, 1.3413734732995417, 0.061274443137325108, 0.70307476852136652], [0.85671070070894562, 3.2645077700816358, 1.9326328165161266, -0.88330856320411577, 4.8892765065857873]], [[3.4851166800142774, 0.52638998551035421, 2.0280668717207808, -2.9529863942711501, -3.6035508065849511], [2.7931414286461829, 1.3554593060531914, 1.7510572213030091, -1.9314941804410823, 1.9415066314508582], [1.7787809299213366, 3.4563016490800855, 3.316234018184808, 2.9507764508660594, -2.3465951795553952], [0.54143335971214679, 0.46680458594713059, -4.8710295423832219, 4.7333766096348473, -4.6097337616401681]], [[-2.5702039495602347, -1.2550162957528332, -3.0170877689290498, 1.8333547197464508, 0.57258125247284131], [4.2299272595275728, 0.61820368239728651, 4.1199010145327222, -4.1370870828247499, 4.241424532327903], [-1.6473550440154119, 0.53010669422165169, 1.5650883171811181, -0.26457653010400239, -2.075286051060854], [2.301749471895171, 2.7405935956953833, 1.4937241857071824, 3.0307311120025471, -3.2396440020191895]]], [[[-0.50794380273232242, -0.32750021991355904, 0.15310762751032936, 3.2171086758296212, 1.5490925518802703], [1.514198250642325, -2.8860693855520125, 3.1377405254331183, -4.4384272258210933, 2.5192647040530947], [-4.9351873436733946, 4.4101764494385982, 1.3343729065207661, -3.7735540058794648, 3.9270486044737325], [-1.1876666235163382, 4.4275997260970641, 4.8797743865153347, 4.1150919161286232, -1.4968324861147417]], [[1.5858150100526931, 4.7424752751616523, -2.3177940107975861, -4.9130566245526079, 0.94152277827467401], [4.391267677724354, -4.9985252172272947, -0.39518725508742047, -1.7135708777935035, 4.6160008582767666], [-1.9876884135905071, 0.8891598668333458, 3.7238739989328575, -3.4040854947747921, -3.2212604023055533], [-2.3398395340629241, 4.6359544130139358, -2.2009256244675726, 2.0976140367503255, -2.0365074042210662]], [[-2.2902517704591006, -4.5107250740341485, -0.23371864615404636, -3.704183632673268, 4.6790588723275803], [-4.6993574389289092, -4.5854433098133098, -3.6549452724739142, -0.39330595656933731, -4.5364709656660303], [-4.8328390453324257, 2.0345674454705831, -3.8850110814037686, 4.6461884473293846, -2.0652668853045841], [2.670170367642295, -4.5743360595032891, 0.39322962934197037, -3.6330457081121481, 4.3315839838918571]], [[4.3365579922772, -4.3563631344023221, -0.93339487587043379, 2.0394871206464646, -2.4529078387102912], [-4.7231467072110336, -4.6834820169319524, 1.3884703558204308, 0.0052584934036286057, 1.8205225482441136], [-3.6602250863745613, 4.6959461318807563, 1.3797261919465331, 1.7628276336068787, 3.6639018551410114], [3.5686467011881344, -3.6089829469054067, -4.6110927375319024, 3.3628179347987714, -0.52831306818925938]], [[3.8509860308932247, -2.3537839502885269, -4.5602514451601852, 0.72531361818489959, 2.6718885958691097], [1.3255202795069474, 3.5413039619804305, 0.56006668598653953, 4.2456663668792718, 1.9541929979783825], [4.8157376890340018, 1.4254173035824715, -2.2435324269033909, -1.134496787461762, 4.3350999477722425], [0.14458154105484322, -4.1020785386643324, 1.6488411879512777, -4.0984256235780414, -2.05708286533386]]], [[[3.8628553724898893, 3.012315839690185, 3.7773094257946465, 4.9481930800149136, -4.4671304808142924], [3.5872675449735727, 3.9970313380794771, -0.38522177527130452, -4.2599028278309863, 0.46600121365393843], [-3.4726630509871259, 4.9614778996790143, -0.60345224095433458, 2.0341181916609443, -3.7482275243785557], [-2.796008109889041, -4.9051221892309025, 0.33989592679274416, -0.31203202018168064, -4.8685277714156321]], [[-0.041202193577624335, 0.025926940743669746, 1.0298444856631885, -1.3318105433542806, -0.19484033441739523], [-2.9288957429437525, -3.0558543467713695, 3.4532361551133448, 1.0664259285534152, -4.8162379508339734], [1.5712971137870895, -0.73731609132667586, 2.3140983827485737, 1.4999392490393468, 1.9036080524141816], [3.4662896427698637, -3.4008192352168454, -0.48256774795441881, -4.7925790382079745, 3.5200010513336562]], [[1.3804358791321425, 2.0427993631823673, -3.2505927551749081, 0.91691639249148871, 2.0506612034544851], [-2.455030864595976, 0.54124192386112302, -4.575932194948753, -1.4329335271800128, 3.4504145616160269], [3.7940172926545195, -1.3372462674219587, 0.95923884407288895, -4.3702010811940237, 2.2454689446282625], [2.8236577974701174, 1.5405414745092951, 0.30446473509657324, -4.8326291782252593, -3.6176850544333892]], [[1.1772936986627647, -4.2584037988394527, -4.4940304190290172, -1.7578450722994168, 3.7768243539963855], [3.8634669236514156, 0.98779381790779475, 2.7606718058066262, -4.5567118089961776, 3.7891363731107557], [-2.8259036572344618, 2.9111446151564433, -1.9862156995528424, -0.5296664588342912, 3.3068104034995862], [-2.3679793217162937, -1.5270803159318183, 1.7729648909540874, 0.87009495534426229, -0.77151391100296074]], [[1.9985864680665975, 2.2613167991351188, 1.0729037593970849, -1.1524552787220665, -2.8417531167316135], [-1.7987397800942473, 2.7617110602296524, 4.7922051316461136, -1.8517488568661156, 1.772140634817494], [2.7819802761978281, 3.8985571944111932, -1.3524571818672406, -1.8477426960383427, 1.7735767831267468], [-1.3710458750458479, -0.75786871412532264, 0.86982964207468516, 0.36809211200159986, 3.9853184439730054]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.5269603686532101, -6.1985544034099309, -3.0357790145346115, -1.5018318196402589, -5.6780426616842332], [-2.585926584834696, 1.3675107559695565, -7.1167275199072062, -2.5226269626463154, -5.9838203484312613], [-0.12103293429541218, -1.7755271924089771, -4.4086812174052659, -0.70840399675658805, -2.160553129790558], [-1.3405540003986571, -7.946828384803089, 1.6740707722745043, -3.1365407560342939, -7.7577238293296489]], [[0.72069110610619092, 3.4839890006642733, -0.85618871014238884, 2.5564361192976719, 0.72265549662115358], [4.4701715328279317, -2.1430840632037191, 1.5706569563916686, -3.5045559383873544, 0.4466582651232871], [-1.612759035414749, 3.5483163543270191, -0.068878071920624784, 3.6939523339736944, 2.5197084932293272], [-3.187582667228626, -1.8656001490905534, 4.5798610844858132, 4.3578474794884148, 1.2937940426406591]], [[-0.59094448056604154, 3.0549262922671589, 4.5632327156036014, 3.46822075451714, -1.5832433631676084], [-3.8435166610063414, -0.86347029350995985, -4.7673720968041948, 0.34517134073723899, -1.6577928743326433], [1.1627211597327003, 0.81927710289131106, 3.8962046650421245, 0.2118474831845969, -2.9859855655362981], [1.5400665119230181, 0.61558667973908587, 1.5175518875797556, 1.8259857031142745, -4.6459728697756333]], [[1.3940558915309911, 1.6347238703332749, 1.4096154217741521, -4.7158913974931274, -4.2474985532497946], [-4.0409240883141919, -2.0425659012694446, -2.3814716579856832, -1.0401557225644247, -0.72447639737799996], [-5.0705203053510068, -3.7373780922032234, -3.5078041735865444, 0.60129460023421188, -4.6485179022887815], [3.0836760920854385, -1.1374337645436015, -2.3970630881851474, -3.8066911120526261, 3.0669454747079552]], [[1.1045167825875293, -0.11359656631614889, 0.36679560270187217, 3.3106807994543725, 0.60095374638995569], [-1.4828196027468881, 1.7272839919047343, 3.0773598595419429, -1.595630967258054, -5.0651614945068548], [-3.6098157263557589, 0.13386704071424571, 4.5196230345261537, -0.63268456457726607, -3.3348417515838751], [-5.2957933928330396, 0.05714995617826002, -3.2025270195562534, -1.5758743093762155, -1.7906628576645991]]], [[[1.5925559730873102, -4.1287972736870522, -0.9585791363633458, -0.57837838451753054, -5.198073517785037], [0.8896041425478538, 3.1355960633468607, -3.5870144588601107, -2.6079179570877891, -5.6034819676090475], [-4.8918966581848133, 0.46717271480517741, -2.4607283099346668, 1.289455624610456, -4.9445838156018196], [-1.6752471813946874, 3.6159511645731426, -3.0426696088227092, 1.8158273730224934, 2.9049134446384075]], [[2.397932785110938, 6.9594662941259804, 1.1449275858539356, -0.49333478073711223, 6.2697023255050697], [3.1830746683434894, 1.2357928043195088, -0.43889938142988427, 2.1908337779796305, 3.1900886466130185], [4.4051515597883606, 5.9059234442369215, -0.54237107053860623, 4.1809797518810976, 2.3371948078408105], [3.2102529065866028, 2.1436580430917296, 3.8109227172593281, 4.9484900647471974, -1.608795292759603]], [[2.1668488176967555, 2.686249436351059, 4.6534936685962345, 7.3570436757092974, 3.5969875576072106], [7.0389911656411659, 4.8111128920219492, 7.9852795931276042, 1.8519399797964482, -0.6030328583051352], [0.81236528589751922, 7.3959610299515059, 5.2630515885101188, 3.9829525583479022, 4.6247528837319436], [4.7783888159195227, 7.1861858852922129, 5.8543109317267037, 3.0383695520064613, 8.8109546217963643]], [[0.42987507067587671, -2.5288516238280465, -1.0271747376176199, -6.0082280036095508, -6.6587924159233518], [-0.26210018069221785, -1.6997823032852093, -1.3041843880353916, -4.9867357897794831, -1.1137349778875425], [-1.2764606794170641, 0.4010600397416848, 0.26099240884640729, -0.10446515847234128, -5.4018367888937959], [-2.5138082496262539, -2.5884370233912701, -7.9262711517216227, 1.6781350002964466, -7.6649753709785688]], [[-4.4754050776718692, -3.1602174238644674, -4.9222888970406835, -0.071846408365183301, -1.3326198756387928], [2.3247261314159386, -1.2869974457143476, 2.2146998864210881, -6.042288210936384, 2.3362234042162688], [-3.552556172127046, -1.3750944338899824, -0.34011281093051604, -2.1697776582156365, -3.9804871791724881], [0.39654834378353687, 0.83539246758374919, -0.41147694240445176, 1.125529983890913, -5.1448451301308236]]], [[[0.99056268754265986, 1.1710062703614232, 1.6516141177853116, 4.7156151661046035, 3.0475990421552526], [3.0127047409173073, -1.3875628952770303, 4.6362470157081006, -2.939920735546111, 4.0177711943280769], [-3.4366808533984123, 5.9086829397135805, 2.8328793967957484, -2.2750475156044825, 5.4255550947487148], [0.31083986675864406, 5.9261062163720464, 6.378280876790317, 5.6135984064036055, 0.0016740041602405498]], [[-2.2912755545490953, 0.86538471055986399, -6.1948845753993744, -8.7901471891543963, -2.9355677863271143], [0.51417711312256564, -8.8756157818290831, -4.2722778196892088, -5.5906614423952918, 0.73891029367497829], [-5.8647789781922954, -2.9879306977684426, -0.15321656566893083, -7.2811760593765804, -7.0983509669073417], [-6.2169300986647125, 0.75886384841214749, -6.0780161890693609, -1.7794765278514628, -5.9135979688228542]], [[-4.5039907282182865, -6.7244640317933344, -2.4474576039132323, -5.9179225904324539, 2.4653199145683944], [-6.9130963966880952, -6.7991822675724958, -5.8686842302331002, -2.6070449143285233, -6.7502099234252162], [-7.0465780030916116, -0.17917151228860284, -6.0987500391629546, 2.4324494895701987, -4.2790058430637696], [0.45643140988310904, -6.7880750172624751, -1.8205093284172156, -5.8467846658713345, 2.1178450261326711]], [[-0.34977010081725179, -9.0426912274967748, -5.6197229689648855, -2.6468409724479871, -7.1392359318047429], [-9.4094748003054853, -9.3698101100264033, -3.297857737274021, -4.6810695996908231, -2.8658055448503381], [-8.3465531794690122, 0.0096180387863045169, -3.3066019011479186, -2.9235004594875731, -1.0224262379534403], [-1.1176813919063173, -8.2953110399998593, -9.2974208306263542, -1.3235101582956803, -5.2146411612837111]], [[6.6610920642598899, 0.45632208307813826, -1.75014541179352, 3.5354196515515648, 5.4819946292357749], [4.1356263128736126, 6.3514099953470957, 3.3701727193532047, 7.055772400245937, 4.7642990313450477], [7.625843722400667, 4.2355233369491367, 0.56657360646327426, 1.6756092459049032, 7.1452059811389077], [2.9546875744215084, -1.2919725052976672, 4.4589472213179429, -1.2883195902113762, 0.75302316803280522]]], [[[3.9948559591555757, 3.1443164263558714, 3.9093100124603328, 5.0801936666806, -4.3351298941486061], [3.7192681316392591, 4.1290319247451635, -0.25322118860561815, -4.1279022411652999, 0.5980018003196248], [-3.3406624643214395, 5.0934784863447007, -0.47145165428864821, 2.1661187783266307, -3.6162269377128693], [-2.6640075232233547, -4.7731216025652161, 0.47189651345843053, -0.18003143351599427, -4.7365271847499457]], [[-2.5681007458047014, -2.5009716114834073, -1.4970540665638885, -3.8587090955813577, -2.7217388866444723], [-5.4557942951708291, -5.5827528989984465, 0.92633760288626776, -1.4604726236736618, -7.34313650306105], [-0.95560143843998757, -3.2642146435537529, -0.21280016947850333, -1.0269593031877302, -0.62329049981289542], [0.93939109054278669, -5.9277177874439229, -3.0094663001814959, -7.3194775904350511, 0.99310249910657911]], [[3.3990807125749276, 4.0614441966251524, -1.231947921732123, 2.9355612259342738, 4.0693060368972702], [-0.43638603115319086, 2.5598867573039081, -2.5572873615059679, 0.58571130626277235, 5.469059395058812], [5.8126621260973046, 0.68139856602082638, 2.9778836775156741, -2.3515562477512386, 4.2641137780710476], [4.8423026309129025, 3.5591863079520802, 2.3231095685393583, -2.8139843447824742, -1.5990402209906041]], [[-1.6125631055810019, -7.0482606030832198, -7.2838872232727834, -4.5477018765431829, 0.98696754975261891], [1.0736101194076491, -1.8020629863359718, -0.029184998437140397, -7.3465686132399437, 0.9992795688669891], [-5.6157604614782279, 0.12128781091267671, -4.776072503796609, -3.3195232630780578, 0.51695359925581963], [-5.1578361259600598, -4.3169371201755844, -1.0168919132896792, -1.9197618488995043, -3.5613707152467273]], [[2.6204812546912244, 2.8832115857597458, 1.6947985460217119, -0.53056049209743961, -2.2198583301069865], [-1.1768449934696203, 3.3836058468542793, 5.4140999182707406, -1.2298540702414886, 2.3940354214421209], [3.403875062822455, 4.5204519810358201, -0.73056239524261368, -1.2258479094137158, 2.3954715697513738], [-0.74915108842122091, -0.1359739275006957, 1.4917244286993121, 0.9899868986262268, 4.6072132305976323]]]])+(1.-msk_ref)*numpy.array([[[[-1.6658455471445617, -4.3374395819012825, -1.1746641930259631, 0.35928300186838946, -3.8169278401755848], [-0.72481176332604758, 3.2286255774782049, -5.2556126983985578, -0.66151214113766699, -4.1227055269226129], [1.7400818872132362, 0.085587629099671325, -2.5475663958966175, 1.1527108247520603, -0.29943830828190965], [0.52056082110999125, -6.0857135632944406, 3.5351855937831527, -1.2754259345256456, -5.8966090078210005]], [[-1.1786684980371587, 1.5846293965209237, -2.7555483142857384, 0.6570765151543223, -1.176704107522196], [2.5708119286845821, -4.0424436673470687, -0.32870264775168101, -5.4039155425307044, -1.4527013390200625], [-3.5121186395580986, 1.6489567501836695, -1.9682376760639744, 1.7945927298303448, 0.62034888908597763], [-5.0869422713719761, -3.764959753233903, 2.6805014803424636, 2.4584878753450652, -0.60556556150269047]], [[-3.9868184436390464, -0.34094767080584587, 1.1673587525305966, 0.072346791444135139, -4.9791173262406137], [-7.2393906240793466, -4.2593442565829651, -8.1632460598771992, -3.0507026223357658, -5.0536668374056486], [-2.2331528033403045, -2.5765968601816938, 0.50033070196911966, -3.1840264798884079, -6.3818595286093025], [-1.8558074511499867, -2.7802872833339189, -1.8783220754932493, -1.5698882599587303, -8.0418468328486377]], [[2.2912022137961321, 2.5318701925984159, 2.3067617440392931, -3.8187450752279863, -3.3503522309846536], [-3.1437777660490509, -1.1454195790043036, -1.4843253357205421, -0.14300940029928366, 0.17266992488714106], [-4.1733739830858649, -2.8402317699380824, -2.6106578513214034, 1.4984409224993529, -3.7513715800236405], [3.9808224143505795, -0.24028744227846044, -1.4999167659200063, -2.9095447897874851, 3.9640917969730962]], [[2.8998455030340313, 1.6817321541303532, 2.1621243231483742, 5.1060095199008746, 2.3962824668364577], [0.31250911769961398, 3.5226127123512363, 4.872688579988445, 0.19969775318844807, -3.2698327740603528], [-1.8144870059092568, 1.9291957611607478, 6.3149517549726557, 1.162644155869236, -1.539513031137373], [-3.5004646723865376, 1.8524786766247621, -1.4071982991097514, 0.21945441107028651, 0.0046658627819029519]]], [[[1.4970855609278093, -4.2242676858465522, -1.0540495485228467, -0.67384879667703146, -5.2935439299445379], [0.79413373038835289, 3.0401256511873598, -3.6824848710196116, -2.70338836924729, -5.6989523797685484], [-4.9873670703443143, 0.3717023026456765, -2.5561987220941678, 1.1939852124509551, -5.0400542277613205], [-1.7707175935541883, 3.5204807524136417, -3.1381400209822101, 1.7203569608629925, 2.8094430324789066]], [[-1.8747327646481402, 2.6868007443669022, -3.1277379639051426, -4.7660003304961904, 1.9970367757459915], [-1.0895908814155888, -3.0368727454395694, -4.7115649311889625, -2.0818317717794477, -1.0825769031460597], [0.1324860100292824, 1.6332578944778433, -4.8150366202976844, -0.09168579787798059, -1.9354707419182677], [-1.0624126431724754, -2.1290075066673486, -0.46174283249975012, 0.67582451498811924, -5.8814608425186812]], [[-5.5599515185842439, -5.0405508999299409, -3.0733066676847653, -0.36975666057170242, -4.1298127786737897], [-0.68780917063983393, -2.9156874442590506, 0.25847925684660433, -5.8748603564845521, -8.3298331945861346], [-6.9144350503834806, -0.33083930632949388, -2.463748747770881, -3.7438477779330976, -3.1020474525490562], [-2.9484115203614771, -0.54061445098878691, -1.8724894045542961, -4.688430784274539, 1.0841542855153645]], [[0.68614182191611395, -2.2725848725878093, -0.77090798637738267, -5.7519612523693135, -6.4025256646831146], [-0.0058334294519806207, -1.4435155520449721, -1.0479176367951544, -4.7304690385392458, -0.85746822664730526], [-1.0201939281768269, 0.65732679098192204, 0.51725916008664452, 0.15180159276789595, -5.1455700376535587], [-2.2575414983860167, -2.3321702721510329, -7.6700044004813854, 1.9344017515366838, -7.4087086197383316]], [[1.3603002288329828, 2.6754878826403843, 0.91341640946416769, 5.7638588981396683, 4.5030854308660588], [8.1604314379207903, 4.548707860790504, 8.0504051929259397, -0.20658290443153238, 8.1719287107211205], [2.2831491343778056, 4.4606108726148692, 5.4955924955743356, 3.6659276482892151, 1.8552181273323636], [6.2322536502883885, 6.6710977740886008, 5.4242283641003999, 6.9612352903957646, 0.69086017637402808]]], [[[-0.47842709679778039, -0.29798351397901701, 0.18262433344487139, 3.2466253817641633, 1.5786092578148123], [1.5437149565768671, -2.8565526796174705, 3.1672572313676604, -4.4089105198865512, 2.5487814099876367], [-4.9056706377388526, 4.4396931553731402, 1.3638896124553082, -3.7440372999449227, 3.9565653104082745], [-1.1581499175817962, 4.4571164320316061, 4.9092910924498767, 4.1446086220631653, -1.4673157801801997]], [[1.4658013834898149, 4.6224616485987742, -2.4378076373604642, -5.0330702511154861, 0.82150915171179584], [4.2712540511614758, -5.1185388437901729, -0.51520088165029865, -1.8335845043563817, 4.4959872317138885], [-2.1077020401533852, 0.76914624027046763, 3.6038603723699794, -3.5240991213376702, -3.3412740288684315], [-2.4598531606258023, 4.5159407864510577, -2.3209392510304507, 1.9776004101874474, -2.1565210307839444]], [[1.5903856494921715, -0.63008765408287637, 3.6469187737972257, 0.17645378727800409, 8.5596962922788524], [-0.81872001897763713, -0.70480588986203774, 0.22569214747735789, 3.4873314633819348, -0.65583354571475816], [-0.9522016253811536, 5.9152048654218552, -0.0043736614524965134, 8.5268258672806567, 1.815370534646688], [6.5508077875935671, -0.69369863955201705, 4.2738670492932425, 0.24759171183912398, 8.2122214038431292]], [[5.1178177872035207, -3.5751033394760015, -0.1521350809441131, 2.8207469155727853, -1.6716480437839705], [-3.9418869122847129, -3.9022222220056317, 2.1697301507467515, 0.7865182883299493, 2.6017823431704343], [-2.8789652914482406, 5.477205926807077, 2.1609859868728538, 2.5440874285331994, 4.4451616500673321], [4.3499064961144551, -2.827723151979086, -3.8298329426055817, 4.1440777297250921, 0.25294672673706131]], [[4.9879097209288616, -1.21686026025289, -3.4233277551245482, 1.8622373082205366, 3.8088122859047466], [2.4624439695425844, 4.6782276520160675, 1.6969903760221765, 5.3825900569149088, 3.0911166880140195], [5.9526613790696388, 2.5623409936181085, -1.106608736867754, 0.0024269025738750116, 5.4720236378078795], [1.2815052310904802, -2.9651548486286954, 2.7857648779869146, -2.9615019335424044, -0.920159175298223]]], [[[4.7218584028522645, 3.8713188700525603, 4.6363124561570217, 5.8071961103772889, -3.6081274504519172], [4.4462705753359479, 4.8560343684418523, 0.47378125509107072, -3.400899797468611, 1.3250042440163137], [-2.6136600206247507, 5.8204809300413896, 0.25555078940804066, 2.8931212220233196, -2.8892244940161804], [-1.9370050795266658, -4.0461191588685272, 1.1988989571551194, 0.54697101018069461, -4.0095247410532568]], [[4.5643356382770319, 4.631464772598326, 5.6353823175178448, 3.2737272885003756, 4.410697497437261], [1.6766420889109037, 1.5496834850832868, 8.0587739869680011, 5.6719637604080715, -0.21070011897931717], [6.1768349456417457, 3.8682217405279804, 6.91963621460323, 6.1054770808940031, 6.5091458842688379], [8.07182747462452, 1.2047185966378109, 4.1229700839002374, -0.18704120635331822, 8.1255388831883124]], [[-2.4273985304438361, -1.7650350463936113, -7.0584271647508867, -2.8909180170844899, -1.7571732061214935], [-6.2628652741719542, -3.2665924857148556, -8.3837666045247321, -5.2407679367559918, -0.35741984795995174], [-0.013817116921459149, -5.1450806769979369, -2.8485955655030897, -8.1780354907700019, -1.5623654649477161], [-0.98417661210586127, -2.2672929350666835, -3.5033696744794054, -8.6404635878012375, -7.4255194640093674]], [[2.2434871440850301, -3.1922103534171873, -3.4278369736067518, -0.69165162687715132, 4.843017799418651], [4.9296603690736811, 2.0539872633300602, 3.8268652512288917, -3.4905183635739121, 4.8553298185330211], [-1.7597102118121963, 3.9773380605787088, -0.92002225413057692, 0.53652698658797426, 4.3730038489218517], [-1.3017858762940282, -0.46088687050955279, 2.8391583363763528, 1.9362884007665278, 0.29467953441930472]], [[4.7367941566617509, 4.9995244877302722, 3.8111114479922383, 1.5857524098730869, -0.10354542813646006], [0.93946790850090611, 5.4999187488248058, 7.530412820241267, 0.88645883172903783, 4.5103483234126474], [5.5201879647929815, 6.6367648830063466, 1.3857505067279128, 0.89046499255681066, 4.5117844717219002], [1.3671618135493055, 1.9803389744698308, 3.6080373306698386, 3.1062998005967533, 6.7235261325681588]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.4577137193719123, -3.1189246245467506], [-0.061552454409898871, -3.3259130624676203]], [[3.9245146840272511, 3.0634638476477942], [3.3036049694349501, -3.3068117327529665]], [[1.159368868770275, -4.7121450375346621], [4.8645541454044139, 3.216023263660162]], [[4.3472678467828985, 2.9809790452208063], [3.2381374839026282, 2.4234362672240763]], [[3.7165535002736867, 0.3665138858792707], [-0.71074647667358448, 0.28344233970183197]], [[-1.327877320831031, 4.2271176159761463], [-4.5679269785314647, 1.4241746455094342]]])+(1.-msk_arg0)*numpy.array([[[-3.1234183076863884, -3.4034909491728351], [-2.0453746505304493, 0.67485517685752416]], [[-2.7502310361577043, -1.4767018152450238], [-2.9057035758122538, -3.251809485500293]], [[-1.5131430639850798, 2.6100660989764366], [-2.4408554743512054, -4.9882149529322586]], [[1.0115882693312557, -0.29516828128414474], [-1.6308662611642877, -3.6237296902780103]], [[1.2720081946466433, 1.392829210925683], [0.10225785265415954, 1.3847983851069037]], [[4.1623487016322329, -3.7177311501811938], [-0.33904394495208567, 1.8374518701497866]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-4.5483319407880884, 1.9015355252728074], [3.2216604017378323, -0.46958098668987169]], [[3.3659545867239213, 3.9971646354560271], [4.2576964489857545, -1.8000302207327925]], [[0.88789930438653997, 1.4044657937263825], [-4.4031677183155544, -4.4134865460596213]], [[-4.0836337713610824, -4.2496082060398521], [-2.7359337753640633, -3.6973163708629766]], [[-4.0259910411469431, 2.172012180735936], [3.598454098328844, 3.290787770170482]], [[-2.0239826081555612, -0.93539010856108895], [-3.0290342073180589, 2.7227727798541723]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-8.0060456601600016, -1.2173890992739431], [3.1601079473279334, -3.795494049157492]], [[7.2904692707511725, 7.0606284831038213], [7.5613014184207046, -5.106841953485759]], [[2.047268173156815, -3.3076792438082796], [0.46138642708885946, -1.1974632823994593]], [[0.26363407542181605, -1.2686291608190459], [0.50220370853856489, -1.2738801036389003]], [[-0.30943754087325637, 2.5385260666152067], [2.8877076216552595, 3.5742301098723139]], [[-3.3518599289865922, 3.2917275074150574], [-7.5969611858495236, 4.1469474253636065]]])+(1.-msk_ref)*numpy.array([[[-7.6717502484744768, -1.5019554239000277], [1.176285751207383, 0.20527419016765247]], [[0.61572355056621708, 2.5204628202110033], [1.3519928731735007, -5.0518397062330855]], [[-0.62524375959853984, 4.0145318927028191], [-6.8440231926667598, -9.4017014989918799]], [[-3.0720455020298267, -4.5447764873239969], [-4.3668000365283515, -7.3210460611409864]], [[-2.7539828465002998, 3.5648413916616191], [3.7007119509830035, 4.6755861552773856]], [[2.1383660934766717, -4.6531212587422832], [-3.3680781522701446, 4.5602246500039589]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.9948398736112294, 4.7867143876842988], [-0.3487974003916392, -2.77541628420088]], [[-4.1426195779078032, -1.8892817036795027], [4.4581716629420463, -2.6434428871721325]]], [[[-2.4920577748321913, -3.968319652763721], [-4.6908327269141958, -1.1347154636728138]], [[2.5632714755637309, -4.3283115426250953], [-4.6958435588472156, 0.89796448329609557]]], [[[3.8316050672568309, -2.4876631747411251], [-4.1796626522700944, 3.7813528858664149]], [[3.3985353715057638, -2.373301577532331], [0.99951782280850576, 3.4109450680917153]]], [[[-0.56135215242132652, -3.1897462511789652], [-0.45278943071142308, 4.3256279260518973]], [[3.3466573861884577, 4.0110557693675695], [1.318216127342664, -4.9505225607955063]]], [[[1.6253584006397297, 0.064127753731359149], [0.33958477410819032, 1.4715425108101812]], [[4.795315542248769, 0.98186338512183813], [-3.7927710857467902, -0.34730805401144416]]], [[[1.3236391974974007, -0.85457781459741344], [-2.1926577348515011, 3.1344488332342646]], [[-2.2967874133554278, -3.9976989457834766], [-2.0000953391464504, -1.8284527216203106]]]])+(1.-msk_arg0)*numpy.array([[[[3.3810615239182784, -1.6462072512809431], [2.2583378786113446, 4.6138069914993043]], [[0.77820374458134633, -1.1318039588308837], [4.4280686823647635, -3.5333705305498899]]], [[[-0.53666545941342658, 3.8307146086589601], [-1.6547422687660474, 2.0383431091540896]], [[-0.63154528747624994, 1.7188061201579874], [2.853219580152885, -4.8738253115709167]]], [[[1.993430603783902, 4.3070271192671896], [2.6119383547771227, 2.9906740030307226]], [[0.56337211226552686, 3.4039750389251875], [-2.4731169717966415, -4.6245646896288894]]], [[[3.2618957977811931, 1.0534798609896709], [-3.4918633206487124, -4.9584359350261655]], [[-3.9830155354591001, 0.2599215531546184], [-2.5429140141297344, -0.23233710403052044]]], [[[1.8310343770529638, -4.5772970821716985], [-4.8650539267005897, 1.1583694877142587]], [[-1.7308204725348197, -3.2967539137596202], [-2.0770273353282684, 1.518393947312151]]], [[[2.7475467223382033, 0.94915446255504055], [3.7164703018481493, 3.1026255653268873]], [[-0.58674952784214174, 3.1703964208612696], [-0.7590000408241977, 0.54284331803318597]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[-0.91109264223478093, -4.5641455267388427], [-2.2440687308945839, -2.7831993760029872]], [[3.1650576391728844, -4.4016698323419723], [1.6368360667401163, -4.7553718373147449]], [[-0.18257525413355769, 0.326827099766966], [2.4454659596190691, 3.6167314258957699]], [[3.8418762312808923, 0.36101860085839199], [-2.5421502439639641, 3.2386389837155534]], [[1.1993444802598123, -4.5694241155159778], [3.0883656616674333, 2.5064726459819395]], [[-4.4790571858485597, 2.9714748423362938], [3.414249570907927, 4.7763167333845686]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.9059325158460103, 3.8756217454495179], [-4.9129429271304819, -7.3395618109397223]], [[-6.3866883088023876, -4.133350434574087], [1.6749722869390591, -5.4266422631751201]]], [[[0.67299986434069314, -0.8032620135908366], [-9.0925025592561681, -5.5363852960147861]], [[4.2001075423038472, -2.691475475884979], [-9.4512153961619596, -3.8574073540186493]]], [[[3.6490298131232732, -2.6702384288746828], [-3.8528355525031284, 4.1081799856333809]], [[5.8440013311248329, 0.072164382086738144], [4.6162492487042757, 7.0276764939874852]]], [[[3.2805240788595658, 0.65212998010192713], [-0.091770829853031088, 4.6866465269102893]], [[0.8045071422244936, 1.4689055254036054], [4.5568551110582174, -1.711883577079953]]], [[[2.824702880899542, 1.2634722339911715], [-4.2298393414077875, -3.0978816047057967]], [[7.8836812039162023, 4.0702290467892714], [-1.2862984397648507, 2.1591645919704954]]], [[[-3.1554179883511591, -5.3336350004459732], [0.77881710748479271, 6.1059236755705584]], [[1.1174621575524992, -0.5834493748755496], [2.7762213942381182, 2.9478640117642581]]]])+(1.-msk_ref)*numpy.array([[[[2.4699688816834975, -2.5572998935157241], [-2.3058076481274981, 0.049661464760461627]], [[-1.4658649863132376, -3.3758726897254676], [1.6448693063617763, -6.3165699065528766]]], [[[2.6283921797594578, 6.9957722478318445], [-6.0564121011080196, -2.3633267231878827]], [[1.0052907792638663, 3.3556421868981037], [-1.9021522571618599, -9.6291971488856625]]], [[[1.8108553496503443, 4.1244518651336319], [2.9387654545440887, 3.3175011027976886]], [[3.008838071884596, 5.8494409985442566], [1.1436144540991284, -1.0078332637331195]]], [[[7.1037720290620854, 4.8953560922705632], [-3.1308447197903204, -4.5974173341677735]], [[-6.5251657794230642, -2.2822286908093457], [0.69572496958581898, 3.0063018796850329]]], [[[3.0303788573127761, -3.3779526019118862], [-9.4344780422165684, -3.4110546278017191]], [[1.3575451891326136, -0.20838825209218692], [0.42944531065367109, 4.0248665932940906]]], [[[-1.7315104635103564, -3.5299027232935192], [6.6879451441844431, 6.074100407663181]], [[2.8275000430657853, 6.5846459917691966], [4.0173166925603709, 5.3191600514177546]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank3_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.6551723965154714, 3.9465188533798372], [-3.2736224235003144, -2.8396383378195633]], [[-1.7267008318936714, -3.8386680629200964], [-0.56223432473277946, -0.065917485889097627]], [[-0.14781724210345626, 3.8074196617873604], [2.0515688685893956, -4.5372212529680231]], [[-2.4029016351701182, -1.8779945797399487], [-3.3878201323932866, 4.8022292033928533]], [[-4.600564464818536, -1.2858859159828073], [-3.5517641907043718, 0.54360973159391346]], [[-0.12852486068064106, 4.1368770227934686], [-4.9901444979201992, 1.422967227199889]]])+(1.-msk_arg0)*numpy.array([[[1.3492803143031127, 0.74998789745146421], [2.5391701640218791, -1.3771523373773551]], [[0.72120128484349522, -0.75658418913351255], [-2.8181461626650783, 3.1161326580267836]], [[1.1351917008685177, -0.45317349921544459], [-3.3375260981730603, 0.52991427534354685]], [[-1.3790892222560602, 4.6635649278822928], [1.0579070136463411, -3.1288295583436367]], [[2.4604177111257153, -1.0650983149031457], [-4.9264608504017922, -4.6612630122464687]], [[-1.5584907108120829, -3.8901061767628855], [-3.9862406796497272, 2.7859735197204758]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-4.6995529803193739, 2.8439037862468961], [3.1527251024915142, 0.024101457121013858]], [[-4.2102414919804296, -4.9860421989186712], [-2.2754137125870666, 3.8706610843372076]]], [[[4.9179993718744992, -0.63196708939593282], [0.27281342647858509, 4.242536823775696]], [[3.8534417158883869, -4.5191292478784284], [-0.55604458088671116, -0.70075033337960502]]], [[[-4.6098675318223616, 3.2975630978699702], [-2.4933862580304158, 3.8900866377059717]], [[-2.1410917789689576, -1.4142090593961933], [-1.3249500104456393, -1.9141493670478757]]], [[[1.4455795804240337, -0.063651649770321228], [-3.6173993558415454, 4.6599957355037809]], [[-0.56682894461003208, -2.3080432569786136], [0.6880218016541324, -1.2828840398140673]]], [[[-3.7690653957865452, -4.9696955034389951], [-3.3328973834740783, 0.095692011929282117]], [[4.1254952223697856, -3.7058459928206968], [-1.9738997291519524, -1.7038422014427534]]], [[[-1.7376138733809454, -3.8019331703794466], [3.5482221344950098, -1.8342510708815682]], [[-1.4493937204941476, 3.0803791664158648], [-3.4246706836811569, -3.8137007825412015]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.3547253768348462, -0.81126861026857533], [7.0992439558713514, 3.9706203105008511]], [[-7.483863915480744, -8.2596646224189847], [-5.1150520504066304, 1.0310227465176443]]], [[[3.1912985399808278, -2.3586679212896042], [-3.5658546364415113, 0.40386876085559953]], [[3.2912073911556075, -5.0813635726112079], [-0.62196206677580879, -0.76666781926870264]]], [[[-4.7576847739258179, 3.1497458557665139], [1.3140334037569446, 7.6975062994933321]], [[-0.08952291037956206, 0.63735980919320223], [-5.8621712634136625, -6.4513706200158989]]], [[[-0.95732205474608456, -2.4665532849404395], [-5.495393935581494, 2.7820011557638322]], [[-3.9546490770033187, -5.6958633893719002], [5.4902510050469857, 3.519345163578786]]], [[[-8.3696298606050803, -9.5702599682575311], [-4.6187832994568856, -1.1901939040535252]], [[0.57373103166541384, -7.2576101835250686], [-1.430289997558039, -1.16023246984884]]], [[[-1.8661387340615865, -3.9304580310600876], [7.6850991572884784, 2.3026259519119003]], [[-6.4395382184143468, -1.9097653315043344], [-2.0017034564812679, -2.3907335553413125]]]])+(1.-msk_ref)*numpy.array([[[[-3.3502726660162612, 4.1931841005500088], [3.9027129999429784, 0.77408935457247807]], [[-1.6710713279585505, -2.4468720348967921], [-3.6525660499644217, 2.4935087469598525]]], [[[5.6392006567179944, 0.089234195447562392], [-0.48377076265492747, 3.4859526346421834]], [[1.0352955532233086, -7.3372754105435067], [2.5600880771400725, 2.4153823246471786]]], [[[-3.4746758309538439, 4.4327547987384879], [-2.9465597572458604, 3.4369131384905272]], [[-5.4786178771420175, -4.7517351575692537], [-0.79503573510209247, -1.3842350917043289]]], [[[0.06649035816797344, -1.4427408720263815], [1.0461655720407474, 9.3235606633860737]], [[0.49107806903630902, -1.2501362433322725], [-2.4408077566895043, -4.4117135981577036]]], [[[-1.3086476846608299, -2.5092777923132799], [-4.3979956983772244, -0.96940630297386354]], [[-0.80096562803200655, -8.632306843222489], [-6.6351627413984211, -6.3651052136892226]]], [[[-3.2961045841930283, -5.3604238811915295], [-0.34188404226787572, -5.7243572476444538]], [[-5.4356344001438748, -0.90586151323386233], [-0.6386971639606811, -1.0277272628207257]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.7877685083954047, 3.2711791365289784, -3.21158325322636, 4.8991440532618942], [0.42491540358384938, -0.23498799211577115, 4.8612635363193348, 4.8705590846473878], [3.549097654387289, -4.9055317410865884, -4.054814342259804, 4.2603132231169756]], [[-0.83966324663404279, -4.779595606231231, 3.5508012439336998, 4.0651087022544807], [0.33380399370615343, -2.7848175859826494, -0.67631176595011855, -1.1875334864482587], [1.1516447689386311, -4.8503559883883955, 0.55801041642183247, -2.6240110705449493]]], [[[3.9432152665652396, 1.1826208703329657, 0.4346621327821083, -2.2449915250559362], [2.7696057515898644, -0.74119578432734734, 1.5099845524814546, 1.5709217250128757], [0.80160369870718284, -2.0201528067658203, -3.8265830506041021, 3.4357238926974336]], [[-4.0315275277724414, 1.3404262915473497, 1.1499669336802079, -1.9779011579321368], [-3.1011840237117472, 4.1536113798000933, -0.057989068363849761, 4.6505846203927277], [1.746351139997051, 4.4544696620815429, -4.9159004005324745, 2.1664965126289619]]], [[[-1.0815638228154101, -1.6473111587889431, 1.4128006998851985, 1.1789837114252606], [-0.56492758590169423, 2.1790487575232973, -1.6071640936253182, 2.0039370159666809], [3.9025660507233262, -0.61561196350990688, 1.1472536422245501, -1.4712818116235491]], [[2.3586063008709495, 3.084142910178219, 1.4721446960640741, -3.4363656311011779], [-2.8696376677381141, 2.7413622161987803, 0.90915009600098262, 2.2819919775854842], [4.8820992850126377, 3.3483700503844691, -2.9540159414593892, 4.6757447011024329]]]])+(1.-msk_arg0)*numpy.array([[[[-2.5562897168475631, -3.7517499465563766, -4.8295391244489627, 0.8177234773744626], [-1.8870645793743135, -1.6750432406097602, -0.58322842109218698, -0.056043611711257491], [-2.5407699858052046, -0.8624587982932006, -2.9348125821052107, 4.8383725116262326]], [[-0.64623685838573586, -3.9174095552013899, 4.6987242282153012, -0.65162310964961279], [-0.60516491359709601, 1.5694979044021826, 4.0516373374710426, -4.5511390061511934], [-1.2226311628126618, 1.863218120751351, 2.7156669128177153, 4.927727446956073]]], [[[-4.9839996886696749, 0.69265140211336274, -3.5684504011785378, 3.4229914252590472], [1.5564342527421262, -2.7460669786290426, -2.8842365337812281, -0.21827662106125167], [-2.2421111654111892, 1.9715763724378368, 1.6372075691801866, -1.274211018433812]], [[0.5734804877260018, -3.2821521801654807, -4.1907811736226197, 2.2995100164981341], [3.6159909147426852, -0.14385862994095433, 3.1396679801014749, -1.6708202301997686], [3.839109541563646, 2.9108867210247169, -4.8000849540315693, -4.384652814367457]]], [[[1.952208877103887, 0.14955613700187254, 3.0632767656958073, 2.3707764365113002], [-4.9649540577237037, 2.0261025575656042, -2.9039759490047978, -1.9077448590011228], [1.8524264863652782, -2.9539421249982825, -3.6598876685092661, -0.80627392328351366]], [[0.9870799399328698, 2.1213222852586711, 2.049617904769633, 3.9494508664248222], [-1.1150257880396963, -3.648015857678244, -1.9056320222094381, -1.5567331439446006], [-0.95013657995585987, -4.6927398639976312, -2.4307836882469989, 2.1877586441965011]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s1=numpy.array([[[[-1.0082823551248299, 0.58239209101477485, -1.8050467790383093, -1.0856593918006894], [-4.3599014749702256, -1.3630046447340094, -2.8644499879672489, -3.4425705377394591], [-1.6159758220897604, 4.4866342054687465, -1.0932348711940234, -4.5984746326588599]], [[0.80609808482938217, 2.583805963112896, 0.59867234523334822, -2.188105734533774], [-0.055651990913334615, -3.6525228138095525, -4.8718191603466483, -1.4967557245575316], [-2.9163683681327512, -4.3781309091613227, 1.4618428867058739, 4.1438855038523474]]], [[[-4.6614936884265399, -0.3860104904218975, -3.0489261283939073, 2.787563297592019], [-2.5373055398327282, 1.0844797758348594, 2.3001409075507109, -4.4786608806677828], [-3.7973502912395074, 3.9152346667062119, -4.9341476501735082, 1.6585242647977507]], [[4.5438797960907102, 0.84405198993016217, -3.4190133600896147, -2.3218817187657539], [-3.2461779465747065, 4.1524933364457421, 0.79210501502519115, 2.006495434248067], [0.15512742712539307, -0.89606755759406553, 4.3210815930688344, -1.5253514970933124]]], [[[4.0234117917572529, 1.1329349754752114, 3.2822732193128985, 0.033551330300895188], [-0.39317622525261342, 4.9037236554784531, -0.4029843627931875, -2.1506065051852365], [-2.1180245503735362, 0.85169259205220893, 2.1821011688213332, 1.471336748219211]], [[1.7992169395969659, -1.3166744984535628, 0.61149223296708932, 4.9538826152846003], [-0.8914743363283808, -4.5842599838034772, -1.9654094767145001, -3.4513808780978517], [-1.5905863790499342, -4.3272826753112836, -3.1543810250639206, 4.943220655153107]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-5.7960508635202341, 3.8535712275437533, -5.0166300322646693, 3.8134846614612048], [-3.9349860713863762, -1.5979926368497805, 1.9968135483520859, 1.4279885469079288], [1.9331218322975285, -0.41889753561784193, -5.1480492134538274, -0.33816140954188434]], [[-0.033565161804660626, -2.195789643118335, 4.149473589167048, 1.8770029677207067], [0.27815200279281882, -6.4373403997922019, -5.5481309262967669, -2.6842892110057903], [-1.7647235991941201, -9.2284868975497183, 2.0198533031277064, 1.5198744333073981]]], [[[-0.71827842186130031, 0.7966103799110682, -2.614263995611799, 0.5425717725360828], [0.23230021175713622, 0.3432839915075121, 3.8101254600321655, -2.907739155654907], [-2.9957465925323246, 1.8950818599403916, -8.7607307007776107, 5.0942481574951843]], [[0.51235226831826886, 2.1844782814775119, -2.2690464264094068, -4.2997828766978907], [-6.3473619702864532, 8.3061047162458355, 0.73411594666134139, 6.6570800546407947], [1.9014785671224441, 3.5584021044874774, -0.59481880746364002, 0.64114501553564951]]], [[[2.9418479689418429, -0.51437618331373169, 4.695073919198097, 1.2125350417261558], [-0.95810381115430765, 7.0827724130017504, -2.0101484564185057, -0.1466694892185556], [1.78454150034979, 0.23608062854230205, 3.3293548110458833, 5.4936595661914112e-05]], [[4.1578232404679154, 1.7674684117246562, 2.0836369290311634, 1.5175169841834224], [-3.7611120040664949, -1.8428977676046969, -1.0562593807135174, -1.1693889005123674], [3.2915129059627035, -0.97891262492681452, -6.1083969665233102, 9.6189653562555399]]]])+(1.-msk_ref)*numpy.array([[[[-3.564572071972393, -3.1693578555416018, -6.6345859034872721, -0.26793591442622677], [-6.2469660543445391, -3.0380478853437696, -3.4476784090594359, -3.4986141494507166], [-4.1567458078949651, 3.6241754071755459, -4.0280474532992336, 0.23989787896737269]], [[0.15986122644364631, -1.3336035920884939, 5.2973965734486494, -2.8397288441833868], [-0.66081690451043062, -2.0830249094073698, -0.8201818228756057, -6.047894730708725], [-4.1389995309454131, -2.5149127884099718, 4.1775097995235893, 9.0716129508084205]]], [[[-9.6454933770962157, 0.30664091169146523, -6.6173765295724447, 6.2105547228510662], [-0.98087128709060201, -1.6615872027941831, -0.58409562623051725, -4.6969375017290345], [-6.039461456650697, 5.8868110391440487, -3.2969400809933216, 0.38431324636393871]], [[5.117360283816712, -2.4381001902353185, -7.6097945337122344, -0.022371702267619753], [0.36981296816797871, 4.0086347065047878, 3.931772995126666, 0.33567520404829843], [3.9942369686890391, 2.0148191634306514, -0.47900336096273488, -5.910004311460769]]], [[[5.97562066886114, 1.2824911124770839, 6.3455499850087058, 2.4043277668121954], [-5.3581302829763171, 6.9298262130440573, -3.3069603117979853, -4.0583513641863593], [-0.26559806400825803, -2.1022495329460735, -1.4777864996879329, 0.66506282493569735]], [[2.7862968795298357, 0.80464778680510829, 2.6611101377367223, 8.9033334817094225], [-2.0065001243680771, -8.2322758414817212, -3.8710414989239381, -5.0081140220424523], [-2.540722959005794, -9.0200225393089148, -5.5851647133109195, 7.1309792993496082]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=2.64167853117
      res=add(arg0,arg1)
      s0=numpy.array(2.01258195177)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(4.65426048294)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=2.49339906968
      res=add(arg0,arg1)
      s0=numpy.array([1.900719394033227, -1.6682277046717386])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([4.3941184637142028, 0.82517136500923716])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=-2.41037590919
      res=add(arg0,arg1)
      s0=numpy.array([[-1.8851388437514913, 2.5833041732601183, -2.2059958149382028, 1.371788292643763, -0.26798103321475786], [-0.48420128534974971, 2.6954556895816122, 4.1394151342208225, 4.6442064415557187, -3.0524769479068183], [-1.1386992902551372, -0.75168530417616708, 2.3416400554483774, 1.4579475702104663, 2.5426461163620715], [-1.13183071824424, 3.9545864941035784, 3.9252066813024467, 3.5373078938086699, -4.1688535320613944]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-4.2955147529426885, 0.17292826406892114, -4.6163717241294, -1.0385876165474341, -2.678356942405955], [-2.8945771945409469, 0.28507978039041504, 1.7290392250296254, 2.2338305323645216, -5.4628528570980155], [-3.5490751994463343, -3.1620612133673642, -0.068735853742819764, -0.95242833898073087, 0.13227020717087434], [-3.5422066274354371, 1.5442105849123813, 1.5148307721112495, 1.1269319846174728, -6.5792294412525916]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-0.266945591716
      res=add(arg0,arg1)
      s0=numpy.array([[[0.11097939939003965, -4.2141347476009372], [-0.026689318491113667, -2.977293048781573]], [[1.2947389624404213, -0.019180563399133987], [3.0467207586242075, -2.057964681208666]], [[2.8328181445561871, -3.8846511374431425], [1.9670568217707265, 0.21522673380540347]], [[3.2088034211382777, -4.4512306572617497], [-4.4609711918455712, -3.8496867084428468]], [[-4.1890365160283549, -1.0010867101419718], [-2.0758226262481463, 2.516014526723426]], [[1.254696485456547, -4.6712341364564107], [2.2231344817974907, 4.5551392719255404]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.15596619232594477, -4.4810803393169216], [-0.29363491020709809, -3.2442386404975574]], [[1.0277933707244369, -0.28612615511511841], [2.7797751669082231, -2.3249102729246505]], [[2.5658725528402027, -4.1515967291591274], [1.7001112300547421, -0.051718857910580951]], [[2.9418578294222932, -4.7181762489777341], [-4.7279167835615556, -4.1166323001588312]], [[-4.4559821077443393, -1.2680323018579562], [-2.3427682179641307, 2.2490689350074415]], [[0.98775089374056257, -4.9381797281723951], [1.9561888900815063, 4.2881936802095559]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=4.94806889834
      res=add(arg0,arg1)
      s0=numpy.array([[[[1.6297633962685296, -4.651527858599426, 2.5610063310452436, 2.0069864227164613], [-3.438135497789061, -0.82993782383617365, -0.86937020791335584, 1.5331482088320865], [-0.83835979944527761, -3.4666807450131687, -0.77481124975534144, 1.7928441094259027]], [[4.5745852132955953, -4.4341891624247243, 0.40690083077065342, 3.8052057762047156], [-1.7378259136223919, -1.5987417337358902, 3.2113888214172661, 3.0645313326418879], [1.5984872633324336, 0.10833644961201117, 4.1072419552031203, -0.81414252667640419]]], [[[-1.6852003487612399, -4.0913416822983155, 1.5444587630509998, -3.3213914678897813], [3.854507927366889, 3.7454829160979592, -0.37423408156746518, 3.1290872562764651], [3.7620725672588531, -3.6782100051326703, -4.5995251975925218, -2.570735541010313]], [[-3.078772480480235, 4.3759998698215501, -4.3531702596679533, 3.1760146708564765], [-1.7424651198236054, 0.38895155377032431, 3.788363191502695, -0.38559685160815427], [1.4081472902631305, 0.28586662061733659, 2.6492385536571197, 1.6183558394027688]]], [[[-4.864293073329117, 0.84590806904893512, 3.5895636781564679, -0.12486502728115667], [1.9874093849913361, 4.8083101712211, -3.7045576062658769, -3.8833419646759895], [3.3122452535884115, -1.5335290353936504, -2.7123429964445278, 0.53760658974163]], [[3.0287554019778966, -1.678083011496232, 3.964188162374727, 4.0508811320359897], [-1.8350976699719626, 3.3386668519582763, -2.6348221719986586, -3.3850695237414352], [-0.67439461577150084, 2.6994417187184272, -4.5463288912655022, -1.2977026939705949]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[6.5778322946126, 0.29654103974464441, 7.5090752293893139, 6.9550553210605317], [1.5099334005550094, 4.1181310745078967, 4.0786986904307145, 6.4812171071761568], [4.1097090988987928, 1.4813881533309017, 4.1732576485887289, 6.740913007769973]], [[9.5226541116396657, 0.5138797359193461, 5.3549697291147238, 8.7532746745487859], [3.2102429847216785, 3.3493271646081801, 8.1594577197613365, 8.0126002309859583], [6.546556161676504, 5.0564053479560815, 9.0553108535471907, 4.1339263716676662]]], [[[3.2628685495828305, 0.85672721604575486, 6.4925276613950702, 1.626677430454289], [8.8025768257109593, 8.6935518144420296, 4.5738348167766052, 8.0771561546205355], [8.7101414656029235, 1.2698588932114001, 0.34854370075154861, 2.3773333573337574]], [[1.8692964178638354, 9.3240687681656205, 0.59489863867611703, 8.1240835692005469], [3.205603778520465, 5.3370204521143947, 8.7364320898467653, 4.5624720467359161], [6.3562161886072008, 5.233935518961407, 7.5973074520011901, 6.5664247377468392]]], [[[0.083775825014953398, 5.7939769673930055, 8.5376325765005383, 4.8232038710629137], [6.9354782833354065, 9.7563790695651704, 1.2435112920781934, 1.0647269336680809], [8.2603141519324819, 3.41453986295042, 2.2357259018995426, 5.4856754880857004]], [[7.976824300321967, 3.2699858868478384, 8.9122570607187974, 8.99895003038006], [3.1129712283721078, 8.2867357503023467, 2.3132467263454117, 1.5629993746026352], [4.2736742825725695, 7.6475106170624976, 0.40174000707856816, 3.6503662043734755]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-2.61841272004)
      res=add(arg0,arg1)
      s0=numpy.array(4.97454982141)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(2.35613710137)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(1.36093281023)
      res=add(arg0,arg1)
      s0=numpy.array([2.9089262489827323, -2.3286413256199099])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([4.2698590592156398, -0.96770851538700242])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([4.1512151162787809, -0.66905109239753902])
      res=add(arg0,arg1)
      s0=numpy.array(-4.74450364443)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.59328852815485789, -5.4135547368311778])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(0.159465810372)
      res=add(arg0,arg1)
      s0=numpy.array([[2.2384478601214983, 3.7130610247220162, -3.7412007638275737, -2.4315510933819748, 4.1120898047630057], [2.887452788286935, -1.0654756973290369, 4.6625784239246943, -0.1899442920214689, -4.9107528423885167], [-0.40088069648277358, -0.7787898054435356, -0.83345824221645692, -4.6804015046349177, -3.4108300534640437], [-1.5057676718879787, 0.70136107496543332, 3.9433699981652239, -0.11288573433014903, -2.8460369557902352]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[2.3979136704938782, 3.872526835094396, -3.5817349534551939, -2.272085283009595, 4.2715556151353855], [3.0469185986593148, -0.90600988695665707, 4.8220442342970742, -0.030478481649089062, -4.7512870320161369], [-0.24141488611039374, -0.61932399507115576, -0.67399243184407709, -4.5209356942625378, -3.2513642430916638], [-1.3463018615155988, 0.86082688533781315, 4.1028358085376038, 0.0465800760422308, -2.6865711454178554]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-4.1717769611803934, -3.6328267820311408, -0.54683271729745009, 3.1141609665388099, 1.7659155711475103], [3.0037437249101213, 0.85404594060279848, -0.93416217515491429, 0.51586093972599656, 2.9358647015794475], [1.1795902729448269, -3.8905831918785081, -3.9415651125609488, 3.8964895032047195, -0.19916588258302959], [3.4345611805083696, 3.9615699337520507, -1.1753423496584481, -4.2310091890584971, -4.0460579374820451]])
      res=add(arg0,arg1)
      s0=numpy.array(-4.45015296518)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-8.621929926363368, -8.0829797472141145, -4.9969856824804246, -1.3359919986441646, -2.6842373940354642], [-1.4464092402728532, -3.5961070245801761, -5.3843151403378888, -3.934292025456978, -1.5142882636035271], [-3.2705626922381477, -8.3407361570614817, -8.3917180777439242, -0.553663461978255, -4.6493188477660041], [-1.015591784674605, -0.48858303143092385, -5.6254953148414231, -8.6811621542414716, -8.4962109026650197]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(4.7888467547)
      res=add(arg0,arg1)
      s0=numpy.array([[[-0.68705107232420293, 4.1269457396550546], [2.6143451382984573, 1.7377251601954757]], [[3.9521720737261141, 0.45948984859846664], [-1.0842083199338037, -1.1503385515893538]], [[-0.15640598199548883, -2.1988906214688173], [2.7111804509058635, -0.85378518473057774]], [[3.8397140305899349, -3.9286337123840944], [-3.2489126889155315, 0.99778850215738935]], [[-2.5639250995240204, -0.23102038941453706], [-1.5808497556435097, -2.5710860413157222]], [[-1.6409976322498396, 4.43820964713154], [3.6780169738104345, -4.1085021837112103]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[4.101795682376582, 8.9157924943558395], [7.4031918929992422, 6.5265719148962607]], [[8.741018828426899, 5.2483366032992516], [3.7046384347669812, 3.6385082031114311]], [[4.6324407727052961, 2.5899561332319676], [7.5000272056066484, 3.9350615699702072]], [[8.6285607852907198, 0.86021304231669049], [1.5399340657852534, 5.7866352568581743]], [[2.2249216551767645, 4.5578263652862478], [3.2079969990572752, 2.2177607133850628]], [[3.1478491224509453, 9.2270564018323249], [8.4668637285112194, 0.68034457098957457]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[2.9154476176660102, -2.7688932116469855], [-0.88824681313009801, -4.667575738619635]], [[-3.1666772449666594, 2.8415659027842333], [3.2753783103761744, -2.3006844614080579]], [[0.58271855360972857, -1.5004566219799811], [4.8842567689990801, -1.8367460431811655]], [[1.0095026259244975, -0.70237670440262967], [-0.32187437055666202, -0.93274721604292843]], [[1.8754711705983151, -0.44224616882865941], [2.4622150548623285, 2.4034410280906968]], [[4.9007472298604231, 1.8819082668675522], [-1.2808887364438606, -4.6816631413417262]]])
      res=add(arg0,arg1)
      s0=numpy.array(0.967558967287)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[3.8830065849527946, -1.8013342443602012], [0.079312154156686354, -3.7000167713328507]], [[-2.199118277679875, 3.8091248700710176], [4.2429372776629588, -1.3331254941212736]], [[1.5502775208965129, -0.53289765469319672], [5.8518157362858645, -0.86918707589438116]], [[1.9770615932112818, 0.2651822628841547], [0.64568459673012235, 0.034811751243855937]], [[2.8430301378850995, 0.52531279845812495], [3.4297740221491129, 3.3709999953774812]], [[5.8683061971472075, 2.8494672341543366], [-0.31332976915707622, -3.7141041740549419]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(2.60042004845)
      res=add(arg0,arg1)
      s0=numpy.array([[[[2.1433066353052581, 0.9617574191264282, 2.0920996179669977, -0.32418794575420229], [2.4890073172172125, 2.774923673956553, -4.8313154814218793, 3.8212274343488613], [2.6945085733124552, 2.3709323571831735, -3.2736736957505608, -4.3188260077360932]], [[-2.6694678347371523, -4.785238695051798, 0.10379639552821818, 4.4881551465155827], [-4.9932111622358031, -1.0381165854787988, 0.28999994134346085, -1.9858397574369704], [3.6981721236189085, 3.6276074607606308, 1.2001476115091503, -0.14571991181694344]]], [[[1.9203516226830031, 0.024130521563915153, -0.97622260936091809, -3.6963130094143013], [0.99235043852617633, 1.303043871171699, 0.60773992916008268, -4.7563111786251842], [3.0100319639584683, -2.6521814721357897, -3.4574256711507214, -1.1371466608756107]], [[-1.3680061922790774, 3.0492013166309047, -4.4126110993956758, 4.3671455374005852], [-4.2263802106990358, -1.8365523320410237, -2.5177907881437811, -3.682150578824761], [-3.0805660040778529, -4.4475813083189086, -2.4251912349401819, -1.8321916374122682]]], [[[-0.89503422752042106, -1.0503191972334625, 1.9917503095097651, -0.47084498788975004], [-4.2965888722310002, -2.153092047363633, -4.6426236872918007, -0.67701066326026105], [2.7364522068300525, 3.0948547858711244, 2.6560556104819426, -2.6726501994837615]], [[0.21227874917428657, -2.3332379896406987, 3.4363504392792166, 1.5822032286485266], [-1.5268032816661448, 1.2985237978643465, 3.5041595113044242, 3.2672267437526799], [-3.047406613245359, 1.3997043090688042, -2.5458596565278913, -4.1161071043146356]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[4.7437266837525396, 3.5621774675737097, 4.6925196664142792, 2.2762321026930792], [5.0894273656644939, 5.3753437224038345, -2.2308954329745978, 6.4216474827961427], [5.2949286217597367, 4.971352405630455, -0.6732536473032793, -1.7184059592888117]], [[-0.069047786289870849, -2.1848186466045165, 2.7042164439754997, 7.0885751949628641], [-2.3927911137885216, 1.5623034629684827, 2.8904199897907423, 0.61458029101031109], [6.2985921720661899, 6.2280275092079123, 3.8005676599564318, 2.454700136630338]]], [[[4.5207716711302846, 2.6245505700111966, 1.6241974390863634, -1.0958929609670198], [3.5927704869734578, 3.9034639196189804, 3.2081599776073642, -2.1558911301779027], [5.6104520124057498, -0.051761423688508224, -0.85700562270343994, 1.4632733875716708]], [[1.2324138561682041, 5.6496213650781861, -1.8121910509483943, 6.9675655858478667], [-1.6259601622517543, 0.76386771640625772, 0.082629260303500374, -1.0817305303774796], [-0.48014595563057139, -1.8471612598716272, 0.17522881350709962, 0.76822841103501327]]], [[[1.7053858209268604, 1.550100851213819, 4.5921703579570465, 2.1295750605575314], [-1.6961688237837187, 0.44732800108364845, -2.0422036388445193, 1.9234093851870204], [5.336872255277334, 5.6952748343184059, 5.2564756589292241, -0.072230151036480006]], [[2.812698797621568, 0.26718205880658275, 6.036770487726498, 4.1826232770958081], [1.0736167667811367, 3.8989438463116279, 6.1045795597517056, 5.8676467921999613], [-0.44698656479807752, 4.0001243575160856, 0.054560391919390216, -1.5156870558673541]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[-2.8865651190083397, -4.3835426701872526, -4.3468221423851814, 3.4001147036912389], [-0.73540680987445839, -4.9389839007937724, -3.505844970693027, 4.5275698243077294], [1.1549712953642253, 0.49991569864218732, -2.6660758589516034, 0.20330531186746637]], [[2.7765695165362212, 1.5650734680321987, 1.6637471290949986, 3.7628416427427087], [-3.8863871360242976, -3.0454132002517733, -3.4909892150798205, -3.0626379455843287], [-1.7732124857193607, 2.7817168469501841, -0.86105143078492929, -4.2161334689715773]]], [[[-1.5109035018709216, -0.42072129371644529, 2.1380374581275161, 0.88896148705538458], [1.2099848910700199, -0.57431149889751865, 1.7942689922640085, 3.4796686674316604], [-0.65114792329831239, -3.4735275010485922, -0.061555627554196413, -1.5849157987032934]], [[3.8424411022243774, -3.0866641626024274, -2.4159006560534237, -0.20968351368171945], [-1.8170845579059414, 0.21990709113972429, -0.65545909034749883, -1.566121050179671], [-2.6186538602805109, -0.46115837333334575, 3.0122486191711388, -3.5557020009890072]]], [[[-3.020342293488679, -4.3431103151482775, -3.7026269817703419, 1.3329034594017619], [4.4483244660123127, 1.4693084404971177, 2.0436692125600828, 1.5197890874704809], [3.8393070557369064, 3.3674357428058546, 3.9084450583603694, -1.2673377632649583]], [[-1.6973455642678603, 0.70567522659158932, -3.2560804229395313, 4.0220845083935224], [-4.8133491406348847, 1.1139703085329211, 2.8390870501160768, 0.47058757943226315], [-4.6105302535301442, 0.16604914945335381, -4.6071003580947103, -1.714301303462995]]]])
      res=add(arg0,arg1)
      s0=numpy.array(-1.70468643757)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-4.5912515565772338, -6.0882291077561472, -6.051508579954076, 1.6954282661223448], [-2.4400932474433525, -6.6436703383626661, -5.2105314082619216, 2.8228833867388352], [-0.54971514220466888, -1.2047707389267068, -4.370762296520498, -1.5013811257014278]], [[1.071883078967327, -0.13961296953669544, -0.040939308473895597, 2.0581552051738146], [-5.5910735735931922, -4.7500996378206679, -5.1956756526487151, -4.7673243831532233], [-3.4778989232882549, 1.0770304093812899, -2.5657378683538234, -5.920819906540471]]], [[[-3.2155899394398157, -2.1254077312853394, 0.43335102055862196, -0.81572495051350957], [-0.49470154649887421, -2.2789979364664128, 0.089582554695114336, 1.7749822298627662], [-2.3558343608672065, -5.1782139386174864, -1.7662420651230906, -3.2896022362721875]], [[2.1377546646554832, -4.791350600171322, -4.1205870936223175, -1.9143699512506136], [-3.5217709954748355, -1.4847793464291699, -2.360145527916393, -3.2708074877485651], [-4.3233402978494055, -2.1658448109022399, 1.3075621816022447, -5.2603884385579018]]], [[[-4.7250287310575736, -6.0477967527171721, -5.4073134193392356, -0.37178297816713224], [2.7436380284434185, -0.23537799707177642, 0.33898277499118867, -0.18489735009841324], [2.1346206181680123, 1.6627493052369604, 2.2037586207914752, -2.9720242008338524]], [[-3.4020320018367545, -0.99901121097730483, -4.960766860508425, 2.3173980708246282], [-6.5180355782037793, -0.59071612903597304, 1.1344006125471826, -1.234098858136631], [-6.3152166910990388, -1.5386372881155403, -6.311786795663604, -3.4189877410318892]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([2.6446464431237082, -2.2939025666202282])
      res=add(arg0,arg1)
      s0=numpy.array([-1.4879249865156696, 2.7300745941846971])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([1.1567214566080386, 0.43617202756446893])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=numpy.array([-0.8136895874133625, 4.404122420278231])
      res=add(arg0,arg1)
      s0=numpy.array([[4.5974079075239995, -0.037733108886472522], [-4.3462249185663149, -4.1091207133427909]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[3.783718320110637, -0.85142269629983502], [0.0578975017119161, 0.29500170693544003]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_array_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[-1.6183225262428236, -3.7174991415482452], [-0.49836007938890425, -3.0422731091790087]])
      res=add(arg0,arg1)
      s0=numpy.array([2.9530985399006768, 1.2211347169962714])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[1.3347760136578533, -0.7644006016475684], [0.72277463760736715, -1.8211383921827373]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_array_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=numpy.array([-2.7605020551011838, -4.0466282902695827])
      res=add(arg0,arg1)
      s0=numpy.array([[[4.2379230107755941, 2.9532365132464742, 4.6206592398429258, -3.1509591533973822, -3.8749235225301004], [0.38907489494341263, 3.5995697229241745, 4.0063507007818799, -4.1049368733253679, 0.28844962637680105], [4.9467429435677381, 2.3129856510114895, -4.0661636429077905, -3.4450249326671734, 1.8620856586661159], [-1.7580431510519712, -0.53856979452411657, -4.1390691693806136, -3.9667031176298773, 2.5522060047071786]], [[-1.9909525956509522, -2.7877591491419174, 2.3417614479744051, 2.2621485725755264, -4.7645076380725415], [3.5431463886051482, -4.9200050492702356, 2.309624685478374, -4.375167770118761, 2.1004934944402827], [4.4917356028730104, -3.6998696312241455, -0.57247357563399071, 3.5019734499466431, 2.8911986819736191], [-3.9683608062996365, 3.5292129480167755, -1.1494755394722498, 2.574958752816797, 4.1324632916701063]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1.4774209556744102, 0.19273445814529033, 1.860157184741742, -5.911461208498566, -6.6354255776312847], [-2.3714271601577712, 0.83906766782299069, 1.2458486456806961, -6.8654389284265518, -2.4720524287243828], [2.1862408884665543, -0.4475164040896944, -6.8266656980089744, -6.2055269877683568, -0.89841639643506799], [-4.5185452061531546, -3.2990718496253004, -6.8995712244817975, -6.7272051727310611, -0.20829605039400523]], [[-6.0375808859205353, -6.8343874394115005, -1.7048668422951776, -1.7844797176940563, -8.8111359283421251], [-0.50348190166443452, -8.9666333395398183, -1.7370036047912087, -8.4217960603883437, -1.9461347958293], [0.44510731260342773, -7.7464979214937282, -4.6191018659035734, -0.54465484032293965, -1.1554296082959636], [-8.0149890965692201, -0.51741534225280716, -5.1961038297418325, -1.4716695374527857, 0.085835001400523581]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_array_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[4.3746882806105933, -1.6111305451620428, -0.51734736694524575, -0.88224550789186829, 0.32406063002255259], [-0.22217987106429415, -0.63388044322604031, -3.0945233788120419, 2.9732194359895301, -2.7119747877574141], [-4.3571372984913035, 1.4417855930402004, 2.5114076174819271, 0.43765796022108194, 3.1909380699525478], [4.9128077142738995, 4.1260640760352434, 2.9006198690287333, -0.83329395988009392, 2.373096866045918]], [[-2.0167008061255407, -0.050870194739013108, -2.7285044917741921, 4.9245105753642768, 1.9435599980493832], [-3.5653023853230326, -3.4711431609909891, -0.49110059448973598, 2.926738456071952, -3.3114262408524562], [3.2024072419828116, -1.4132423605250621, 1.1270989466547814, -1.9885151095080444, 1.4864694108659933], [-1.9484057119362932, -0.22994581061857478, 3.7121672915335093, 2.5407700992939368, 4.0740542873318457]]])
      res=add(arg0,arg1)
      s0=numpy.array([-3.4138972626664987, -4.3823641427102338])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.96079101794409461, -5.0250278078285415, -3.9312446296117445, -4.296142770558367, -3.0898366326439461], [-3.6360771337307929, -4.047777705892539, -6.5084206414785406, -0.44067782667696864, -6.1258720504239133], [-7.7710345611578022, -1.9721116696262984, -0.90248964518457164, -2.9762393024454168, -0.22295919271395093], [1.4989104516074008, 0.71216681336874466, -0.51327739363776548, -4.2471912225465926, -1.0408003966205808]], [[-6.3990649488357745, -4.4332343374492469, -7.1108686344844259, 0.54214643265404305, -2.4388041446608506], [-7.9476665280332668, -7.8535073037012229, -4.8734647371999698, -1.4556256866382817, -7.69379038356269], [-1.1799569007274222, -5.7956065032352964, -3.2552651960554524, -6.3708792522182787, -2.8958947318442405], [-6.330769854646527, -4.6123099533288086, -0.67019685117672445, -1.841594043416297, -0.30830985537838806]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_array_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=numpy.array([-2.8617587863448812, -1.8979784889648688])
      res=add(arg0,arg1)
      s0=numpy.array([[[[0.34545845274594722, -2.6325074959492643], [4.1140466772220901, -4.7648752595634356]], [[1.0966805361118093, -0.64398447332444153], [-1.908738632277398, 2.5612857395741084]], [[3.9768763212397147, 3.4896354676513326], [-1.686821947165893, 3.2351592635692441]], [[-2.1070744135480934, -0.73516480940772588], [-2.2588811872375203, -4.8581305920909514]], [[2.3925831255501144, -4.1804727813765936], [-1.4946425407985373, 0.42906646873035825]], [[1.9581103698997566, -1.8932335799404565], [4.0746268346430004, 1.7817884375629305]]], [[[1.4655196019366903, -1.6897534122623723], [-2.2389269580645452, 0.91076756245984569]], [[-0.29156830748476992, -0.87348368377050534], [-1.2618251753785703, -3.0039887229614939]], [[-0.25432287525647634, 3.307405802877458], [3.75787718005882, 0.34153555282351711]], [[1.8983712379382203, -0.95945097523970535], [3.0049113524919573, -0.27055011206210011]], [[1.506964739689689, -3.8492047068916033], [-3.7735604963475966, -2.192357093260374]], [[-4.0020696068489201, -3.6736835308524149], [-1.0005228532589241, -2.3719478703719901]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-2.516300333598934, -5.494266282294145], [1.2522878908772088, -7.6266340459083164]], [[-1.765078250233072, -3.5057432596693228], [-4.7704974186222788, -0.3004730467707728]], [[1.1151175348948335, 0.62787668130645136], [-4.5485807335107742, 0.37340047722436287]], [[-4.9688331998929751, -3.5969235957526071], [-5.1206399735824011, -7.7198893784358322]], [[-0.4691756607947668, -7.0422315677214744], [-4.3564013271434181, -2.432692317614523]], [[-0.90364841644512461, -4.7549923662853377], [1.2128680482981191, -1.0799703487819508]]], [[[-0.43245888702817847, -3.5877319012272411], [-4.136905447029414, -0.98721092650502307]], [[-2.1895467964496387, -2.7714621727353741], [-3.1598036643434391, -4.9019672119263626]], [[-2.1523013642213451, 1.4094273139125892], [1.8598986910939512, -1.5564429361413517]], [[0.00039274897335150882, -2.8574294642045741], [1.1069328635270885, -2.1685286010269689]], [[-0.39101374927517973, -5.7471831958564721], [-5.6715389853124654, -4.0903355822252427]], [[-5.9000480958137889, -5.5716620198172837], [-2.8985013422237929, -4.2699263593368588]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_array_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[[0.99587881133767553, -1.2085227707446879], [-4.1233658107466056, -3.4479370055787983]], [[2.1281948312622676, 4.9070283723717072], [-1.1257004263729407, -4.1034479725853519]], [[-2.6888336276185232, 0.18269452676716469], [4.249953896421637, 2.8533631977460274]], [[2.9692065637267406, -3.4622029210149288], [-1.1746573545691161, 0.59878202096699606]], [[-3.8406099693069242, -0.009788468886354984], [4.7252588775837658, 0.68444363322233492]], [[-0.73500977665093803, -0.11512436820196292], [2.9094480614059108, -0.24871369196474546]]], [[[0.69758916092976708, 2.823295151384178], [-3.3495933803017475, 1.4896669178055957]], [[3.2245767380202697, -4.699979580427625], [-3.9026804742516816, 0.91977607946941475]], [[3.9884626972000081, 2.3107332231828002], [4.9693460140301688, -3.9626655722855055]], [[4.081212385645463, -4.5014215867039216], [-4.8443735585164491, 3.1466435771377377]], [[4.4329462647760458, -1.79041015361158], [-4.5101532278673684, -2.7369155450381486]], [[4.3919346811501949, -2.0487743786121007], [-3.1373401415575652, -4.7842143447720034]]]])
      res=add(arg0,arg1)
      s0=numpy.array([-4.4653049622252698, 3.3559823897663925])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-3.4694261508875943, -5.6738277329699578], [-8.5886707729718754, -7.9132419678040682]], [[-2.3371101309630022, 0.44172341014643735], [-5.5910053885982105, -8.5687529348106217]], [[-7.154138589843793, -4.2826104354581052], [-0.21535106580363284, -1.6119417644792424]], [[-1.4960983984985292, -7.9275078832401986], [-5.639962316794386, -3.8665229412582738]], [[-8.3059149315321932, -4.4750934311116248], [0.25995391535849599, -3.7808613290029349]], [[-5.2003147388762079, -4.5804293304272328], [-1.555856900819359, -4.7140186541900153]]], [[[4.0535715506961596, 6.1792775411505705], [0.0063890094646450457, 4.8456493075719882]], [[6.5805591277866622, -1.3439971906612325], [-0.54669808448528912, 4.2757584692358073]], [[7.3444450869664006, 5.6667156129491927], [8.3253284037965614, -0.60668318251911302]], [[7.4371947754118555, -1.145439196937529], [-1.4883911687500566, 6.5026259669041302]], [[7.7889286545424383, 1.5655722361548126], [-1.1541708381009759, 0.61906684472824391]], [[7.7479170709165874, 1.3072080111542919], [0.21864224820882727, -1.4282319550056108]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[-4.927495204736168, 4.5410589800240579, -3.6863986754715841, 0.54646890414776994, -1.9997801657828473], [2.4473478937010515, -0.04500308034016598, 2.536261507975742, -4.783763641649454, 3.9516623709998751], [-2.8413330704684734, -4.2589529865588514, -0.73600609547736706, 1.3536028975636851, 4.0410295602815758], [3.263633978159044, 2.4093085988966942, 0.006669593301832144, 4.2583074922647093, -4.4695171887320271]])
      res=add(arg0,arg1)
      s0=numpy.array([[0.83390952678492525, -2.3444920346337672, -4.6271132393498879, -0.92189997013104374, -1.0877262416044942], [-2.7622375699848534, -2.5852971995243337, -3.7264581305619826, -1.5369642402421113, -1.8395330088365123], [3.7393954106807588, -0.30553256169936027, 2.6110684427318018, -3.6278714376605836, 2.2117900252574731], [-0.89432279647587798, -4.7203344665382287, -0.56562938864791779, -0.56690963981390485, -2.9069941299480462]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-4.0935856779512427, 2.1965669453902907, -8.313511914821472, -0.3754310659832738, -3.0875064073873415], [-0.31488967628380182, -2.6303002798644997, -1.1901966225862406, -6.3207278818915658, 2.1121293621633628], [0.89806234021228537, -4.5644855482582116, 1.8750623472544348, -2.2742685400968985, 6.2528195855390489], [2.369311181683166, -2.3110258676415345, -0.55895979534608564, 3.6913978524508044, -7.3765113186800733]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=numpy.array([[4.811616148208234, 2.6799227084378741, 1.2705710309951872, -1.7842409243277402, 1.7164590350441769], [-1.8739165928060917, -4.5680596149575416, -1.6298942209156575, -2.8410354442306964, -1.288829780270647], [2.9652467894786838, 2.2279867698802587, -3.7692817739143383, -4.3658543690034568, -2.8355625003739284], [-2.2400227166962461, 2.6651999280531378, 1.1290491718587745, 0.68578570799437877, 0.4260126458500979]])
      res=add(arg0,arg1)
      s0=numpy.array([[[0.47270275577722121, -4.3547493344459154], [-3.6193472465515963, 1.4564028472964683], [0.21895377858092502, 4.7465162576293647], [3.9274694582505241, 0.43524896358873733], [3.3429868008834198, -0.066531295774189481]], [[0.59181143270714021, -4.1620013643661489], [-1.2425673921791605, 1.2446373403067712], [-2.9881344479768899, 3.5082120493336806], [-3.0994577313743776, -2.2558572810299813], [0.99281080010595524, 0.69586452096927598]], [[0.41788182823395026, -1.8894857581449811], [-2.9384756135479986, -1.9143457809289508], [-1.5645585793424619, -1.0175277184878695], [0.57440331926255528, -3.7176402756803375], [-4.516154744905883, 2.9005930174846153]], [[3.8435501473771954, -0.72223310124950846], [1.4903065862228937, 1.7952426152541348], [-3.1436205722690702, 0.91507330964836964], [3.1224483376861887, 0.38193148523548626], [0.46594612555333281, -0.1671653930479069]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[5.2843189039854552, 0.45686681376231864], [-0.93942453811372229, 4.1363255557343424], [1.4895248095761122, 6.0170872886245519], [2.1432285339227839, -1.3489919607390028], [5.0594458359275967, 1.6499277392699874]], [[-1.2821051600989515, -6.0359179571722406], [-5.8106270071367021, -3.3234222746507704], [-4.6180286688925474, 1.8783178284180231], [-5.9404931756050736, -5.0968927252606777], [-0.29601898016469175, -0.59296525930137101]], [[3.3831286177126341, 1.0757610313337027], [-0.7104888436677399, 0.31364098895130788], [-5.3338403532568002, -4.7868094924022078], [-3.7914510497409015, -8.0834946446837943], [-7.3517172452798114, 0.065030517110686858]], [[1.6035274306809493, -2.9622558179457545], [4.1555065142760315, 4.4604425433072725], [-2.0145714004102957, 2.0441224815071442], [3.8082340456805674, 1.067717193229865], [0.89195877140343072, 0.25884725280219101]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_array_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[1.5799386916357676, -0.49661971038224006], [4.3336195741080381, 0.63629765842763231], [4.6069156422603843, 0.22053692974824912], [-2.4932443320307121, -3.0716683251691057], [1.4174670824379518, 2.9003019473678728]], [[0.46473131356060815, -0.7421194036428167], [-0.78763802455592558, -3.8120367891237459], [0.088751753640343267, 2.8411443368366829], [-1.788815311974977, 0.016076097176057402], [4.5936586031843589, -0.6253268806337422]], [[-3.6219941377569298, 0.49811932327827257], [-0.72329839279149866, -2.531739847753085], [-2.3642009064234237, -1.4368185215923281], [2.2824691239240282, 3.9499371555842302], [0.27091695411699313, 0.49040047369594664]], [[4.8773128693395442, 4.3318830877533312], [1.7259860181902953, -4.2576185334777108], [-0.89740721024304104, -3.3204943522232897], [-2.7648941845083619, 3.9936900731801401], [1.634948893024891, 1.5671877663601048]]])
      res=add(arg0,arg1)
      s0=numpy.array([[-4.0707155380632933, -3.7148844798001592, 3.0644148294545666, -0.025356302478282799, -2.8405742565702887], [0.56346315181089146, -2.9357004259305874, -0.39586948491877294, 1.5113137628246207, -1.9386166888847556], [-1.2318521001923011, 1.6617465997813543, -4.3549157769668181, -3.8507896873966829, 3.8697802227238327], [0.72328230549526751, -1.2921701880305059, 4.1567754734434175, -1.6149883973966039, 4.9278491075174653]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-2.4907768464275257, -4.5673352484455334], [0.61873509430787887, -3.0785868213725269], [7.6713304717149509, 3.2849517592028157], [-2.5186006345089949, -3.0970246276473885], [-1.4231071741323369, 0.05972769079758411]], [[1.0281944653714996, -0.17865625183192524], [-3.723338450486513, -6.7477372150543333], [-0.30711773127842967, 2.4452748519179099], [-0.2775015491503563, 1.5273898600006781], [2.6550419142996033, -2.5639435695184978]], [[-4.8538462379492309, -0.73373277691402849], [0.93844820698985565, -0.86999324797173072], [-6.7191166833902418, -5.7917342985591462], [-1.5683205634726547, 0.099147468187547272], [4.1406971768408258, 4.3601806964197793]], [[5.6005951748348117, 5.0551653932485987], [0.43381583015978942, -5.5497887215082162], [3.2593682632003764, 0.83628112122012777], [-4.3798825819049654, 2.3787016757835362], [6.5627980005423563, 6.4950368738775701]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=numpy.array([[-4.3054645956859829, -2.1372219216274213, -3.5530576915559822, -4.2755656238523132, 1.5476818698612318], [1.9073729468366629, -1.7820680738730674, 2.1594719332166648, 3.6711248845375088, 3.8146912023598389], [-0.98678191296191997, 2.3518148535824519, 4.224795676660424, 4.343705903008118, 2.0202227466440821], [-1.1309801277544542, 1.907367942129814, -4.9399206202707742, -0.74716687221818212, -2.2810545686966099]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[2.201680509474496, -1.0649367012142372, -3.9450111407508412, -1.0667316570446248, -4.3617407711583125], [4.7909563307274965, 4.1761322607495934, 3.0252703371143905, -2.1358945636387459, 2.3412665212217343], [-4.2103578901479519, -1.894728992844402, 1.727263863367611, -4.7205904925581565, 1.5581900509702091], [-0.17368881421131466, 3.2591085006722196, -2.3983517868051751, -3.7117210817405777, 0.14880490649184264]], [[-1.9979690264074512, -0.87134631440383448, 2.82729464624739, -2.6373083938828201, 1.7886595589321548], [0.84370743921849645, 0.20956929266023927, -4.2480066998388191, 1.1318116698284753, -2.1733980342764037], [-2.5277848918995813, -1.6386664007316529, -4.581622999888574, 0.86878405572499062, 2.6422919640316511], [2.0518724311381664, -3.0770704982249599, -3.6885993910352388, -4.091333512519145, -0.57091704533988974]], [[0.047353059960927979, 1.1697017317126548, 1.4412282151711997, 2.8896320597130787, 2.7462367475615403], [-0.52950497798909257, 2.5120396472262341, -0.87035397053532471, -4.4606656936694407, -4.8054993854453745], [1.9346130738614606, -3.0266663586813847, 3.4379241911096479, 1.2349320570465281, -2.8989918560519734], [3.4988127503593116, 3.2986902171398889, 1.8576312414183915, -3.2345443266224483, -0.1302174258048403]], [[0.3631136460038249, 3.6325564440238765, 1.7637666053939016, -0.79044491876173772, 1.1652854871508742], [-3.6676673811745042, -4.1485131160484565, -0.25518219775494533, -4.6584784359926514, -2.7377182464278249], [-3.4202273545021997, 2.2217015807705049, -4.1964865850176576, 3.8404776925373003, 2.4644411478122388], [0.17125023726870481, -0.88883894465469382, 4.0190112205963651, -1.2343477680923609, -4.9347058506754298]], [[1.0197512613378992, -2.6540443770610178, 4.1601341453740321, -3.1681803137193123, 4.584440679122606], [-1.3714151720982204, -4.4842224346714064, -2.3733331701032636, 3.6916004991214386, 4.7437985053991731], [-1.935215188682359, 1.939943026583137, 0.25169657193886419, 2.0752904183525551, 0.31109538557020056], [-1.9666941108467317, 1.9536918752285848, 2.6311564699543952, 1.578554585229897, -4.6169315421558856]]], [[[2.3954279886226555, -3.2157023217843972, -2.5693798606330964, -0.30433010657125692, -3.0796204406206451], [2.5174137040275459, 3.1078522433964366, 3.5601601726408934, -2.1088821189144746, 0.84199038767391876], [-3.4116512224916118, -2.2004168303074669, -1.0235335657099567, -1.2061284096899438, 2.2103426578888472], [-3.836850825228344, -2.9020096710451337, -4.7055685384967747, -0.0048729877688380441, 0.60381114752471188]], [[0.014511433289086995, -1.9921058174481265, -4.2159445793562913, -1.7948720020909881, -1.9807435900138328], [2.2039576010423136, 2.7789612102180623, -4.3559289930635172, -1.0166822082730507, 0.98774042806303974], [0.492407705110951, 4.8245567032361834, 1.9489433161882719, -2.86062620095207, 4.9656391527949459], [-2.2372032079945159, 2.2957060252934971, 3.2773679317196347, 0.58690978713737785, 1.5818506002834827]], [[0.99286105289143922, 0.92380936839156824, 3.9324578068864753, 4.5815033795751852, -3.835258373660706], [-2.207995696142504, 3.5897735357645661, -2.2820017264636236, 0.12626166506660397, 3.1749673936428238], [2.2503342132429101, 0.46554318175142217, -1.5631914173323946, 1.498759698526432, 3.3306037913976247], [-4.4646347738637893, 4.2997075335823336, 3.6243434719097891, -1.5075522206299885, -3.4405314544083865]], [[-2.5215769921558797, 1.1342164420379675, -3.3311404188772977, -1.6213444634057019, -2.8378675999260805], [-3.1382063582898856, 1.5449690583735478, -2.6596566468911051, 3.6885905250571422, -4.635097098672154], [-3.5052060298372343, 3.2032317584363881, -4.360272342631565, 3.7419241978074513, 0.56390479749365507], [-1.6377876514504672, 3.9593956159175541, -1.8464825908138383, -2.8820100603510856, 1.7756963891639685]], [[-4.3760002220954188, 3.4316286627049006, 4.0724315017514439, -4.1845904180820277, -1.6476598419657664], [-3.1491553947431337, -1.4316478368686161, -0.21313345585078558, 3.3010942648967081, -3.248182815883367], [-3.385788015065232, 0.10652109054983327, -1.3199039648536957, 1.8256085678614653, 0.56671749879390276], [0.69275017903517799, 4.163172123934709, 3.4293709579793799, 1.7711149149916814, 2.0625871719008799]]], [[[3.8023896600090943, -0.43106938287441388, 4.8864607600881609, -0.1746165353014284, 3.7820814800904863], [1.6774282349663121, -0.69315829842758081, 3.7094683495678975, -2.2106963378327462, 1.9224739235194974], [-2.9576765668007097, -3.8795155902426171, 4.3141314795805314, 0.4324602001775526, -0.57152962653172512], [-4.2666939983988552, -2.6628681876074225, -4.6719019328248432, 1.5176899296759219, 0.018404560433835648]], [[0.46953588070773122, 4.9118822002721902, -2.9126566746575442, -4.3313196207564904, -2.2196850656874223], [2.4907970556897912, 2.4797742799633538, 4.5104633131349594, 3.7800145599889419, -2.8273395218571951], [-4.7207060153730227, -2.6314898372780728, -4.6287288739426113, -4.3496529942203068, 2.8988132244421019], [0.3958582943984803, -4.0706761969351071, 2.202700876579212, 4.146249906455191, 1.9915602536737653]], [[-2.9925362181617308, 0.17335254227463714, 1.8290445098334764, -4.8451006487543697, -1.1910041481016176], [0.3933950594308655, 2.3794517861657223, 0.39157641560609857, 0.7879792249929416, -3.3956742498281156], [-2.3776044265293317, 1.6701755328209416, -2.2580947998917447, -0.94570950306987456, -4.6520976303619479], [-2.0868894503179556, 3.7997424501775061, -1.1969013096697587, -3.4231020193733386, -3.1258102343060603]], [[3.0833645997816479, 3.6550734837879659, 3.2979043381712287, 3.3133845977913108, 0.9377589626025582], [-4.1204939351082697, 0.7084528918228008, -2.7410078931509254, -1.6806665342311158, -4.4113216974423306], [-1.1414162859552412, -3.7637172876232072, -1.3792196439627391, -1.1663740745914097, -0.42320728167045463], [0.2877250680762975, -2.8654309426064595, 4.9708365834795565, -0.98775810546373854, 1.0531609366548427]], [[-0.065120422409023604, 3.1875198470439088, -0.88793551918131985, -0.44137584012277387, -4.5106183128120749], [-3.2092114183745926, -0.17172540488974786, -2.920776337266914, 1.6101111116582363, -0.16147410765535053], [-4.1242289854892844, 1.1175441699028488, 2.9085561805347986, 4.0523470204219478, 4.4054156518865621], [4.8642766852303403, 2.5095728049811541, 3.4891464300707007, 0.91715898077248958, 3.1768816297186149]]], [[[-2.5992136736180838, -3.1588383644363569, 1.865654898329967, 2.2255367201750706, -2.5735420049660043], [1.4999730994141309, -0.33207874318932351, 3.9029243449230435, 2.3970684870469601, -3.502282046006322], [0.91878346029816882, -3.1641846017824591, 4.1946146743629136, 1.5395721141213681, 3.9599289708571632], [-0.99346301814327731, -1.8565148494173087, -0.15784511127563849, -3.0816988876700977, -3.432759768147895]], [[2.4538082000365069, -3.8977372067273297, 3.3235293466348992, -4.3618039702408771, -0.27135711364930692], [-3.5579811291621222, 2.2359482790183716, -3.6941148769167178, -1.4948272725480272, 3.5717143511426404], [-1.767672866339054, -2.0461034550830104, -4.3438324960063817, 2.1510751749440393, -4.9056475841129803], [1.9019629812691798, -3.5042307100224299, -0.7400145357745247, -2.2093221171917854, 0.52760488954424112]], [[-0.42631063718252182, 4.8983056366763371, 1.1269203943619752, 0.51365242940325206, -3.5173551159079208], [4.1288285745640216, 2.8419447323098641, -0.27406467775287346, 0.88012619466088182, -4.3543225822102407], [-2.5910840361022092, 2.3214177362074997, 4.1876347470760003, 4.5618266529066958, 2.2288299295538403], [3.6107393927758462, 3.3884611787222987, -1.6455450384690362, -3.7471127523039738, 0.51800805270103911]], [[-2.1786761873333624, 2.3471923339975254, -0.86655931796121966, 2.3834775204877676, 4.9912617530078744], [-2.2492468542129282, -1.1555391108864077, -2.4673497083439178, -4.9373471651143896, 4.8661744409903704], [-3.893567206428552, 3.3678622348466103, 4.9134353461849418, 1.0616522594242319, 1.2644877677363002], [-1.3020953213674402, -2.5194827317909327, 4.182449429958389, -0.88643581967600404, -3.757136885335759]], [[2.0791526788676453, -2.346277195926846, -2.8510845021641984, -0.19323922131025384, -1.4055321336279771], [-4.9404803971402256, -3.9005073685172285, 1.9511294271667881, 2.9449428974485219, 3.9224699093024515], [1.8072562119043543, 3.5301924605710671, 2.9716374318246785, 1.5297924211370093, -2.8898425432193777], [-2.4976782871662739, 4.0286393932473779, 4.3790938929500207, 4.9896194340095175, 2.9020560902538373]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-2.1037840862114869, -5.3704012969002202, -8.2504757364368242, -5.3721962527306077, -8.6672053668442963], [0.48549173504151355, -0.12933233493638951, -1.2801942585715924, -6.4413591593247288, -1.9641980744642487], [-8.5158224858339349, -6.2001935885303849, -2.578200732318372, -9.0260550882441386, -2.7472745447157738], [-4.4791534098972976, -1.0463560950137634, -6.7038163824911585, -8.0171856774265606, -4.1566596891941403]], [[-4.1351909480348725, -3.0085682360312558, 0.69007272461996871, -4.7745303155102414, -0.34856236269526653], [-1.2935144824089249, -1.927652628967182, -6.3852286214662399, -1.0054102517989461, -4.3106199559038245], [-4.6650068135270022, -3.7758883223590742, -6.7188449215159949, -1.2684378659024307, 0.50507004240422981], [-0.085349490489254887, -5.2142924198523808, -5.8258213126626597, -6.2285554341465659, -2.708138966967311]], [[-3.5057046315950542, -2.3833559598433274, -2.1118294763847825, -0.66342563184290348, -0.80682094399444182], [-4.0825626695450747, -1.0410180443297481, -4.4234116620913069, -8.0137233852254219, -8.3585570770013575], [-1.6184446176945215, -6.5797240502373668, -0.1151335004463343, -2.3181256345094541, -6.452049547607956], [-0.05424494119667056, -0.25436747441609331, -1.6954264501375906, -6.7876020181784309, -3.6832751173608225]], [[-3.9124519778484883, -0.64300917982843675, -2.5117990184584116, -5.0660105426140509, -3.110280136701439], [-7.9432330050268174, -8.4240787399007697, -4.5307478216072585, -8.9340440598449646, -7.0132838702801381], [-7.6957929783545129, -2.0538640430818083, -8.4720522088699717, -0.43508793131501289, -1.8111244760400744], [-4.1043153865836084, -5.164404568507007, -0.25655440325594814, -5.5099133919446741, -9.2102714745277439]], [[2.567433131199131, -1.106362507199786, 5.7078160152352639, -1.6204984438580805, 6.1321225489838378], [0.1762666977630114, -2.9365405648101746, -0.82565130024203182, 5.2392823689826704, 6.2914803752604049], [-0.38753331882112718, 3.4876248964443688, 1.799378441800096, 3.6229722882137869, 1.8587772554314324], [-0.41901224098549994, 3.5013737450898166, 4.178838339815627, 3.1262364550911288, -3.0692496722946538]]], [[[4.3028009354593184, -1.3083293749477343, -0.6620069137964335, 1.603042840265406, -1.1722474937839822], [4.4247866508642089, 5.0152251902330995, 5.4675331194775563, -0.20150917207781172, 2.7493633345105817], [-1.5042782756549489, -0.29304388347080401, 0.88383938112670624, 0.70124453714671908, 4.1177156047255101], [-1.9294778783916811, -0.99463672420847082, -2.7981955916601118, 1.9024999590678249, 2.5111840943613748]], [[-1.7675566405839804, -3.774173891321194, -5.9980126532293587, -3.5769400759640555, -3.7628116638869002], [0.42188952716924621, 0.99689313634499488, -6.1379970669365846, -2.7987502821461181, -0.79432764581002768], [-1.2896603687621164, 3.0424886293631159, 0.16687524231520445, -4.6426942748251374, 3.1835710789218785], [-4.0192712818675833, 0.51363795142042967, 1.4952998578465673, -1.1951582867356896, -0.20021747358958475]], [[3.152332986108104, 3.083281301608233, 6.09192974010314, 6.7409753127918499, -1.6757864404440412], [-0.048523762925839264, 5.7492454689812309, -0.1225297932469589, 2.2857335982832687, 5.3344393268594885], [4.4098061464595748, 2.6250151149680869, 0.59628051588427011, 3.6582316317430967, 5.4900757246142895], [-2.3051628406471245, 6.4591794667989983, 5.7838154051264539, 0.65191971258667625, -1.2810595211917217]], [[1.1495478923816291, 4.8053413265754763, 0.33998446566021112, 2.0497804211318069, 0.83325728461142834], [0.53291852624762326, 5.2160939429110567, 1.0114682376464037, 7.3597154095946511, -0.96397221413464518], [0.16591885470027457, 6.8743566429738969, -0.6891474580940562, 7.4130490823449602, 4.2350296820311639], [2.0333372330870416, 7.6305205004550629, 1.8246422937236706, 0.7891148241864232, 5.4468212737014774]], [[-0.56130901973557989, 7.2463198650647396, 7.8871227041112828, -0.36989921572218876, 2.1670313603940725], [0.66553580761670528, 2.3830433654912229, 3.6015577465090534, 7.115785467256547, 0.56650838647647195], [0.42890318729460697, 3.9212122929096722, 2.4947872375061433, 5.6402997702213042, 4.3814087011537417], [4.5074413813950169, 7.977863326294548, 7.2440621603392188, 5.5858061173515203, 5.8772783742607189]]], [[[2.8156077470471743, -1.4178512958363338, 3.8996788471262409, -1.1613984482633484, 2.7952995671285663], [0.69064632200439213, -1.6799402113895008, 2.7226864366059775, -3.1974782507946662, 0.93569201055757745], [-3.9444584797626296, -4.866297503204537, 3.3273495666186115, -0.55432171278436737, -1.5583115394936451], [-5.2534759113607752, -3.6496501005693425, -5.6586838457867632, 0.5309080167140019, -0.96837735252808432]], [[2.8213507342901831, 7.2636970538546421, -0.56084182107509228, -1.9795047671740384, 0.1321297878950296], [4.8426119092722431, 4.8315891335458057, 6.8622781667174113, 6.1318294135713938, -0.47552466827474316], [-2.3688911617905708, -0.27967498369562094, -2.2769140203601594, -1.9978381406378549, 5.2506280780245538], [2.7476731479809322, -1.7188613433526552, 4.5545157301616639, 6.4980647600376429, 4.3433751072562172]], [[1.2322594584986932, 4.3981482189350611, 6.0538401864939004, -0.62030497209394575, 3.0337915285588064], [4.6181907360912895, 6.6042474628261463, 4.6163720922665226, 5.0127749016533656, 0.82912142683230838], [1.8471912501310923, 5.8949712094813655, 1.9667008767686793, 3.2790861735905494, -0.42730195370152391], [2.1379062263424684, 8.0245381268379301, 3.0278943669906653, 0.80169365728708541, 1.0989854423543637]], [[7.4270705027897659, 7.9987793867960839, 7.6416102411793467, 7.6570905007994288, 5.2814648656106762], [0.22321196789984832, 5.0521587948309188, 1.6026980098571926, 2.6630393687770022, -0.067615794434212617], [3.2022896170528767, 0.57998861538491076, 2.9644862590453789, 3.1773318284167082, 3.9204986213376634], [4.6314309710844155, 1.4782749604016585, 9.3145424864876745, 3.3559477975443794, 5.3968668396629607]], [[1.9551023242350585, 5.2077425936879909, 1.1322872274627622, 1.5788469065213082, -2.4903955661679928], [-1.1889886717305105, 1.8484973417543342, -0.90055359062283191, 3.6303338583023184, 1.8587486389887315], [-2.1040062388452023, 3.1377669165469309, 4.9287789271788807, 6.0725697670660299, 6.4256383985306442], [6.8844994318744224, 4.5297955516252362, 5.5093691767147828, 2.9373817274165717, 5.197104376362697]]], [[[-3.730193801372538, -4.2898184921908111, 0.73467477057551278, 1.0945565924206164, -3.7045221327204585], [0.36899297165967671, -1.4630588709437777, 2.7719442171685893, 1.2660883592925058, -4.6332621737607766], [-0.2121966674562854, -4.2951647295369133, 3.0636345466084594, 0.40859198636691385, 2.828948843102709], [-2.1244431458977315, -2.9874949771717629, -1.2888252390300927, -4.2126790154245519, -4.5637398959023496]], [[4.3611761421663209, -1.9903692645975157, 5.2308972887647132, -2.4544360281110631, 1.6360108284805071], [-1.6506131870323082, 4.1433162211481855, -1.7867469347869038, 0.41254066958178681, 5.4790822932724543], [0.13969507579076001, -0.13873551295319642, -2.4364645538765677, 4.0584431170738533, -2.9982796419831663], [3.8093309233989938, -1.5968627678926159, 1.1673534063552893, -0.30195417506197142, 2.4349728316740551]], [[-5.366231257453296, -0.041614983594437049, -3.8130002259087989, -4.4262681908675221, -8.4572757361786941], [-0.81109204570675253, -2.0979758879609101, -5.2139852980236476, -4.0597944256098923, -9.2942432024810149], [-7.5310046563729838, -2.6185028840632745, -0.75228587319477391, -0.37809396736407841, -2.7110906907169339], [-1.329181227494928, -1.5514594415484755, -6.5854656587398104, -8.6870333725747475, -4.4219125675697351]], [[-2.9258430595515446, 1.6000254617793432, -1.6137261901794018, 1.6363106482695855, 4.2440948807896923], [-2.9964137264311104, -1.9027059831045898, -3.2145165805620999, -5.6845140373325718, 4.1190075687721883], [-4.6407340786467337, 2.6206953626284282, 4.1662684739667597, 0.31448538720604979, 0.51732089551811811], [-2.0492621935856223, -3.2666496040091149, 3.4352825577402069, -1.6336026918941862, -4.5043037575539415]], [[-0.20190188982896462, -4.627331764623456, -5.1321390708608083, -2.4742937900068638, -3.6865867023245871], [-7.2215349658368355, -6.1815619372138384, -0.32992514152982189, 0.66388832875191195, 1.6414153406058416], [-0.47379835679225568, 1.2491378918744571, 0.6905828631280686, -0.75126214755960063, -5.1708971119159877], [-4.7787328558628843, 1.7475848245507679, 2.0980393242534108, 2.7085648653129075, 0.62100152155722732]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_array_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[[-3.1850483477226188, -1.9879996471317671, 1.5724633183635834, -3.6421843676455401, -4.6530206124476106], [-1.4103058564720374, 0.28727904386567182, 0.17151825617519201, -1.0208105464995043, 3.5742903426303396], [-3.3239940316927541, 3.4481979897537229, 3.3766513335298001, -1.0916916899690876, 2.4775264776291621], [-3.7173943986400948, 2.1027517357870904, -0.5221593581818027, 2.1731044618494888, -3.1955297099014079]], [[-3.4155162261308512, -1.4242820020467595, -1.8184053925396961, 4.6443659698249249, -3.1420959044571894], [0.3325163629740775, 4.5024805982009184, 0.048751405363614886, 3.8498448947487542, -2.0485593201624308], [0.6138774652997121, 2.5745852039114148, 4.8233021600909556, 3.1485877775911248, -3.1649087028728484], [-1.248051576796616, -0.84277742761836372, 4.7610877916392944, -0.32219023978131833, 3.6039138298598932]], [[-1.4238890385197092, -1.4719026454041084, 2.4512886888442846, 2.8254195935623105, -0.32979331927069033], [2.9605496294156497, -1.9238780097790031, 2.1871538071209624, 4.4418822514978569, 4.0324779137009479], [3.6822761584803949, -2.3849321266688026, 3.5177890319855756, -2.0923433875331421, -0.73871626301767801], [-2.2066165011344916, 0.97361679883109975, -4.9963596618749397, -2.8580636873256928, -4.2288470255273145]], [[-2.9428239856197527, 1.4723742987526816, -2.8865560458057082, 0.82161916642274946, 4.068546872032373], [-2.2834018828490041, -4.685298917155257, -2.6039882417777638, 1.5577088453636323, 4.3501991386307921], [-0.33780514103745851, -1.0264473863172032, 1.1632580183202235, -0.89999787113345331, 2.5757928844726603], [1.1729637356894651, -3.6601005082869822, -3.362685892712177, 3.8297270598091835, 2.6601932839676525]], [[-3.7584552992416729, -3.5837733600815658, 1.8328482372739163, 3.8710177457736901, 0.82759670175774147], [2.0859032423349628, 4.6810632287263569, 1.4766736535059799, -4.5461865594694348, -0.56699080537869584], [3.8761220181144989, 2.1386755076919961, 2.8736693220163403, -2.7312934259904331, 0.46468156740015942], [-1.0604248020336327, 4.4645115184995383, -4.8497225354109137, -1.4600395533758914, -4.2294079444166552]]], [[[-1.7128690548954015, 3.5212194021807228, 4.4777834848264941, -4.0594541194518428, 3.423753982382161], [-2.646527070607906, -1.6887773814034079, -0.93050762723812763, 3.0698307013629016, -3.4026256012355214], [1.0595731325244708, 4.8219587789463638, -3.9555341520961718, -3.3757561209359457, 4.6313944301736214], [4.2129857607072267, -4.2049380291016032, -2.5933433117959503, 0.070760215476735056, -3.2153098758241394]], [[-0.26788432987649813, -1.3828108749970101, 1.1868750096093468, 2.0949258006915636, -0.26608333904317227], [4.810367350907395, -2.6818232754386662, 1.1659781829814211, 4.991864546306326, -1.7289782019804347], [-0.050073129614406575, -3.3212883687710546, -4.817688859639099, 3.7759182866178804, -2.0732652460344161], [-0.5498172278394815, 1.339522145181232, 0.96101589107153806, 0.95282396670628344, 2.1505692418708673]], [[-0.097018858450759637, -0.41699389332177716, -3.4329017459415514, -4.8722171128934484, -4.5230458674271876], [-0.25076300278144892, -4.8303254763536998, 4.3985462305536966, -2.6772733279968266, 2.7409138340645631], [2.8413651120123564, -4.1886672456579834, 0.78771729927166589, -4.7067433625129951, 4.5335792473777747], [-2.4461721132662175, 4.081685856073058, 1.2246425622870518, 2.9936933203395881, -4.1811485331839897]], [[-1.356083022033121, -3.7255415688466109, -1.7066314352370959, -1.1323942045979649, -2.250146783391239], [1.8199870892154966, 0.73717656555950128, -3.4917961695268431, 1.5491097551302531, 2.4771534699652209], [2.3532834536390315, 4.0776570057773025, 4.7651772860634605, -2.1310393240941785, 0.71528400646956847], [-3.083792442823694, -2.735151483497634, -2.1638448150567227, 0.22774781795016708, -1.293606314489808]], [[-3.7417934710047387, 4.9128704840366932, 1.7988565047007956, -3.6918624017235926, -2.849603385090874], [-3.8579946593304593, -1.0671845582085648, -0.18690656215886925, 3.4906620657583982, 1.8925331720489158], [-1.8945323933941802, 4.8752131397100502, 1.6773287752952299, 3.4053053545170453, 0.13720492249386407], [-1.7580390817259572, -4.0759573223707068, 1.9011106606642416, -3.6486728303377602, -1.2876077903888703]]], [[[2.9142961826865319, -4.9731630621124632, 1.1324971523296412, 0.34690285874861004, 3.2268735465758134], [3.02716441499377, -1.4897972843065785, -1.5484896108333404, -0.72302933855596496, -1.1675924480259745], [4.9856603398798853, 3.5631062515421394, 0.96831973539550997, 3.5753009420004496, -4.4090320596248223], [0.71948472458307045, 0.10091949271813139, 0.69311186910028333, -1.3048791358991982, 4.0142211573164328]], [[-0.51280328912767903, 1.5547895274578405, 0.97573440034155823, 1.7759126189168404, 0.097853039700606104], [-0.7495274083322272, 0.91517185913184029, 3.2537003276815959, 0.091234211779333663, -1.813681785562673], [-2.0539805449927542, 4.9821038143327261, 4.8995131838065138, -4.1677857677767349, -0.46509871138413228], [2.4288258911019387, -4.2938551753217489, -2.783605096202554, 4.1419440375395773, -3.717460591308499]], [[-4.8461552018499265, 2.9830371014846646, 4.7828648517952104, 3.0442714452760455, -1.8302171233500006], [2.2625325510211249, -2.2711612229350986, -1.3921442876592485, 2.6178406844347464, -1.2625916173254614], [-3.6248348670496555, 1.2270098873141224, 2.012481091329156, -0.16970052729849439, 1.0292012163847994], [1.7846902574003423, 2.5066792666631077, 3.0632237908520992, 4.7117762207588996, 1.9221677110078712]], [[3.7090971048508106, 1.8271532939431214, -2.0817747712243562, 4.1911211118930236, -0.95478493781096763], [-4.6236726358760869, -3.8983723104967085, 4.8864595307229983, 4.5824958136060019, 1.0375624566086072], [-0.79185343095605543, -0.6097136771186209, -1.0672299007095178, -4.6206080630715274, 1.2656816895990257], [-3.7773747577594028, -4.1209462983995166, -1.5559088373632934, -4.8696716188306475, 1.4212198567387704]], [[-0.86845651566754167, -0.21951284669069437, 4.0479788226006406, -2.8316932543668125, 2.9878761374502218], [0.014924525352800089, 1.7671311930590132, 1.5567020199281147, 0.64766578527615692, -1.8124379155899062], [2.2497497150842829, -1.3079026854056899, -2.9033015306623899, 2.6434363998829102, 0.44321558946852768], [0.62244697142889116, 2.286498446613737, -2.3469148189167033, 1.2408850759461743, 0.65575343378081019]]], [[[-4.4501995685622084, -3.9810778304571537, -3.7726950948936135, -3.0061059619085952, 1.6818725587974299], [3.6457729553188436, 0.1456243515555764, 4.7630781103357904, 0.10020719640284526, -3.4221324937840403], [-3.5508647543353424, -2.3693028083712466, -0.56556187239324807, -2.5829459256557485, -0.4404983653720933], [-4.5595685610000594, -2.0663310682248093, 2.1643028252327445, 2.0871081798212163, -0.40573013256430901]], [[-1.1064476137266444, -3.6720461031497065, 2.4022223740710791, 0.88427983786808717, -2.2568441749471022], [-0.11020223498879655, -2.607654764845404, -0.13285501656372212, -2.920776688395704, -1.7758469912123722], [1.4223443353762466, 3.5780240889659343, -4.3589951158728715, 3.8796127310006501, -4.5163862405014505], [-1.2561268121959679, 4.4293526992412389, 2.9019175861277251, 2.8540886866153281, 4.205215336380645]], [[-2.2509110292648407, 3.8410595141447921, -2.7478349348168307, 1.2891858396674749, 3.5204552543385539], [-4.2629901050051586, 0.16789515744393491, -4.6693617528337175, -3.0960702901496462, 1.5151719626233726], [3.2207471571707345, 2.0234189053230223, 1.3417335856413626, -0.31186399139834986, 3.7255324112875741], [-1.6966710344831801, 2.0884355353703548, 2.8364322871244259, -1.6532142784720416, -0.17348172772342707]], [[4.1055981065140603, -3.039598255991196, 1.027187387687639, -1.1859030251418967, -1.1505814611674325], [-3.1270877199693481, 4.6350547802301048, 4.6925715151744622, -4.0058788607420599, -1.8064736552811134], [-1.6602811739199375, 0.92424398749998993, -4.3118719805499026, -3.0858523905587809, 4.6974668108740882], [1.674418963222287, -3.1381264467773953, -1.5047730046712351, -0.16839072202579164, -0.26235316458924096]], [[-1.2700522742515794, -2.247776540879145, -4.5829968605536573, -3.3542109692275188, 0.062288789465663186], [-4.8568500733571902, -0.11820993481442699, -3.5974880273866527, 4.987329217325259, -3.2016007159499473], [1.484917602633649, -4.7019559020154595, 2.4680183534847844, -4.0854747686803954, -4.4675563921805228], [4.4905045175797333, -4.5940112350903961, 4.2983502249059509, -3.4722504432467516, 1.3463033079949271]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[1.7598297920281158, 3.05570767007295, 1.7052446341096106, -4.4024015696450958, -0.21674620771928055], [0.4171683645760238, 3.6561470809527474, 0.59791414028763334, 1.2901769660239815, -3.3478711384287418], [-3.2663762602907189, 2.6698987620154169, 0.38717124586076768, -1.0964426655450668, -1.1359377655511214], [-0.90252536417595763, -3.9165170182923967, -3.6608358464734159, 1.8595538198275294, -1.2516714654164405]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-1.425218555694503, -0.22816985510365129, 3.3322931103916993, -1.8823545756174243, -2.8931908204194947], [0.34952393555607841, 2.0471088358937877, 1.9313480482033079, 0.73901924552861153, 5.3341201346584555], [-1.5641642396646382, 5.2080277817818388, 5.1364811255579159, 0.66813810205902824, 4.237356269657278], [-1.957564606611979, 3.8625815278152063, 1.2376704338463131, 3.9329342538776046, -1.4356999178732921]], [[-0.35980855605790119, 1.6314256680261905, 1.2373022775332538, 7.7000736398978749, -0.086388234384239482], [3.3882240330470275, 7.5581882682738684, 3.1044590754365649, 6.9055525648217042, 1.0071483499105192], [3.6695851353726621, 5.6302928739843647, 7.8790098301639055, 6.2042954476640748, -0.10920103279989846], [1.8076560932763339, 2.2129302424545862, 7.8167954617122444, 2.7335174302916316, 6.6596214999328431]], [[0.28135559558990142, 0.23334198870550216, 4.1565333229538952, 4.5306642276719211, 1.3754513148389202], [4.6657942635252603, -0.21863337566939256, 3.892398441230573, 6.1471268856074675, 5.7377225478105585], [5.3875207925900055, -0.67968749255919203, 5.2230336660951862, -0.38709875342353151, 0.96652837109193257], [-0.50137186702488101, 2.6788614329407103, -3.2911150277653292, -1.1528190532160822, -2.5236023914177039]], [[-7.3452255552648484, -2.9300272708924142, -7.288957615450804, -3.5807824032223463, -0.33385469761272279], [-6.6858034524940999, -9.0877004868003528, -7.00638981142286, -2.8446927242814635, -0.052202431014303663], [-4.7402067106825543, -5.428848955962299, -3.2391435513248723, -5.3023994407785491, -1.8266086851724355], [-3.2294378339556307, -8.0625020779320771, -7.7650874623572728, -0.57267450983591228, -1.7422082856774432]], [[-3.9752015069609534, -3.8005195678008463, 1.6161020295546358, 3.6542715380544095, 0.61085049403846092], [1.8691570346156823, 4.4643170210070764, 1.2599274457866994, -4.7629327671887154, -0.78373701309797639], [3.6593758103952183, 1.9219292999727156, 2.6569231142970597, -2.9480396337097137, 0.24793535968087888], [-1.2771710097529132, 4.2477653107802578, -5.0664687431301942, -1.676785761095172, -4.4461541521359358]]], [[[-1.2957006903193777, 3.9383877667567466, 4.8949518494025179, -3.642285754875819, 3.8409223469581848], [-2.2293587060318822, -1.2716090168273841, -0.51333926266210383, 3.4869990659389254, -2.9854572366594976], [1.4767414971004946, 5.2391271435223876, -3.538365787520148, -2.9585877563599219, 5.0485627947496452], [4.6301541252832505, -3.7877696645255794, -2.1761749472199265, 0.48792858005275885, -2.7981415112481156]], [[3.3882627510762493, 2.2733362059557374, 4.8430220905620942, 5.7510728816443111, 3.3900637419095752], [8.4665144318601424, 0.97432380551408126, 4.8221252639341685, 8.6480116272590735, 1.9271688789723127], [3.6060739513383409, 0.33485871218169283, -1.1615417786863516, 7.4320653675706279, 1.5828818349183313], [3.1063298531132659, 4.9956692261339795, 4.6171629720242855, 4.6089710476590309, 5.8067163228236147]], [[0.50089528183687371, 0.18092024696585618, -2.8349876056539181, -4.274302972605815, -3.9251317271395543], [0.34715113750618443, -4.2324113360660665, 4.99646037084133, -2.0793591877091933, 3.3388279743521965], [3.4392792522999898, -3.5907531053703501, 1.3856314395592992, -4.1088292222253617, 5.1314933876654081], [-1.8482579729785842, 4.6795999963606913, 1.8225567025746852, 3.5916074606272215, -3.5832343928963564]], [[-0.065906056009139569, -2.4353646028226295, -0.41645446921311446, 0.15778276142601655, -0.95996981736725751], [3.110164055239478, 2.0273535315834827, -2.2016192035028617, 2.8392867211542345, 3.7673304359892024], [3.6434604196630129, 5.367833971801284, 6.0553542520874419, -0.84086235807019705, 2.0054609724935499], [-1.7936154767997126, -1.4449745174736526, -0.8736678490327412, 1.5179247839741485, -0.0034293484658265427]], [[-7.0896646094334805, 1.5649993456079514, -1.5490146337279462, -7.0397335401523344, -6.1974745235196158], [-7.2058657977592011, -4.4150556966373067, -3.5347777005876111, 0.14279092732965637, -1.455337966379826], [-5.2424035318229221, 1.5273420012813084, -1.6705423631335119, 0.057434216088303458, -3.2106662159348778], [-5.1059102201546995, -7.4238284607994487, -1.4467604777645002, -6.9965439687665025, -4.6354789288176121]]], [[[-0.35208007760418703, -8.2395393224031821, -2.1338791079610777, -2.9194734015421089, -0.039502713714905546], [-0.23921184529694894, -4.7561735445972975, -4.8148658711240593, -3.9894055988466839, -4.4339687083166934], [1.7192840795891664, 0.29672999125142052, -2.298056524895209, 0.30892468170973064, -7.6754083199155412], [-2.5468915357076485, -3.1654567675725875, -2.5732643911904356, -4.5712553961899172, 0.74784489702571388]], [[2.1570954728877378, 4.2246882894732574, 3.6456331623569751, 4.4458113809322573, 2.767751801716023], [1.9203713536831897, 3.5850706211472572, 5.9235990896970128, 2.7611329737947505, 0.85621697645274386], [0.6159182170226627, 7.652002576348143, 7.5694119458219307, -1.4978870057613181, 2.2048000506312846], [5.0987246531173556, -1.623956413306332, -0.11370633418713716, 6.8118427995549942, -1.0475618292930822]], [[-4.4589839559891589, 3.3702083473454323, 5.170036097655978, 3.4314426911368132, -1.443045877489233], [2.6497037968818926, -1.8839899770743309, -1.0049730417984808, 3.0050119302955141, -0.87542037146469376], [-3.2376636211888878, 1.6141811331748901, 2.3996523371899237, 0.21747071856227329, 1.4163724622455671], [2.1718615032611099, 2.8938505125238754, 3.4503950367128668, 5.0989474666196672, 2.3093389568686389]], [[2.6126544393057438, 0.73071062839805467, -3.1782174367694229, 3.0946784463479569, -2.0512276033560344], [-5.7201153014211537, -4.9948149760417753, 3.7900168651779316, 3.4860531480609351, -0.05888020893645951], [-1.8882960965011222, -1.7061563426636877, -2.1636725662545846, -5.7170507286165941, 0.16923902405395896], [-4.8738174233044695, -5.2173889639445834, -2.6523515029083602, -5.9661142843757142, 0.32477719119370363]], [[-2.004394281218663, -1.3554506122418157, 2.9120410570495192, -3.9676310199179339, 1.8519383718991005], [-1.1210132401983213, 0.63119342750789187, 0.42076425437699339, -0.48827198027496443, -2.9483756811410275], [1.1138119495331615, -2.4438404509568112, -4.0392392962135109, 1.5074986343317889, -0.69272217608259368], [-0.5134907941222302, 1.1505606810626157, -3.4828525844678246, 0.1049473103950529, -0.48018433177031117]]], [[[-5.3527249327381661, -4.8836031946331113, -4.6752204590695712, -3.9086313260845529, 0.77934719462147228], [2.743247591142886, -0.75690101262038123, 3.8605527461598328, -0.80231816777311238, -4.3246578579599984], [-4.4533901185113001, -3.2718281725472043, -1.4680872365692057, -3.4854712898317062, -1.3430237295480509], [-5.462093925176017, -2.9688564324007669, 1.2617774610567869, 1.1845828156452587, -1.3082554967402666]], [[-5.0229646320190415, -7.5885631214421032, -1.5142946442213177, -3.0322371804243096, -6.1733611932394989], [-4.0267192532811933, -6.5241717831378008, -4.0493720348561189, -6.8372937066881008, -5.692364009504769], [-2.4941726829161501, -0.33849292932646247, -8.2755121341652682, -0.036904287291746662, -8.4329032587938464], [-5.1726438304883651, 0.51283568094884213, -1.0145994321646716, -1.0624283316770686, 0.28869831808824831]], [[-5.9117468757382561, 0.18022366767137621, -6.4086707812902466, -2.371650006805941, -0.14038059213486198], [-7.9238259514785749, -3.492940689029481, -8.3301975993071338, -6.7569061366230621, -2.1456638838500433], [-0.44008868930268141, -1.6374169411503936, -2.3191022608320533, -3.9726998378717657, 0.064696564814158197], [-5.3575068809565956, -1.5724003111030611, -0.82440355934899001, -5.3140501249454575, -3.8343175741968429]], [[5.9651519263415897, -1.1800444361636666, 2.8867412075151684, 0.67365079468563271, 0.70897235866009689], [-1.2675339001418187, 6.4946086000576342, 6.5521253350019917, -2.1463250409145305, 0.053080164546416064], [0.19927264590759197, 2.7837978073275194, -2.4523181607223732, -1.2262985707312515, 6.5570206307016177], [3.5339727830498164, -1.2785726269498658, 0.35478081515629434, 1.6911630978017378, 1.5972006552382885]], [[-2.5217237396680199, -3.4994480062955855, -5.8346683259700978, -4.6058824346439593, -1.1893826759507773], [-6.1085215387736307, -1.3698814002308675, -4.8491594928030928, 3.7356577519088185, -4.4532721813663878], [0.23324613721720855, -5.9536273674319, 1.2163468880683439, -5.3371462340968359, -5.7192278575969633], [3.2388330521632929, -5.8456827005068366, 3.0466787594895104, -4.7239219086631916, 0.094631842578486669]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[3.9817919363492891, 4.5608590972496792], [1.7014987852552572, 1.0452669031825037]], [[2.2356915712310688, 0.51312111928436011], [-0.26712660897932849, -4.8022002400582062]], [[2.5925463023847044, 3.6555883041124329], [1.6670309401379289, 3.5443735692979512]], [[-3.2151387202999215, 4.361089574498358], [3.2282254627920697, 3.9698499767647206]], [[0.6418223731367938, 4.663851313567255], [-2.3544039502983516, -1.0325061340507622]], [[2.1808742162369699, -3.8391347952661015], [2.3219926430890645, -3.7641699053901334]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[-2.1583143507491984, 0.35754661843754842], [1.6148818170979187, 1.6787033235973681]], [[2.8471484683468695, -2.1104253214597311], [-0.21301971191927471, -0.31279968579898387]], [[-2.6139613700146827, 4.8126833052489317], [0.89535897648539553, -1.1638054053032953]], [[-2.315783263940796, -0.036462503048676709], [-3.8696498945756952, -0.38371794887225175]], [[0.6274702439948987, -0.17192120806796574], [1.9949293934420638, 0.14230380316649338]], [[-0.38577052291371761, 4.1674204960197549], [-2.4792161339449259, 1.9124492744035066]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1.8234775856000907, 4.9184057156872276], [3.3163806023531759, 2.7239702267798718]], [[5.0828400395779383, -1.5973042021753709], [-0.48014632089860321, -5.1149999258571901]], [[-0.021415067629978246, 8.4682716093613646], [2.5623899166233244, 2.3805681639946559]], [[-5.5309219842407176, 4.3246270714496813], [-0.64142443178362551, 3.5861320278924689]], [[1.2692926171316925, 4.4919301054992893], [-0.35947455685628782, -0.89020233088426881]], [[1.7951036933232523, 0.32828570075365349], [-0.1572234908558614, -1.8517206309866268]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=numpy.array([[[-3.8841540814906814, 1.1340868288297958], [-2.6786107184530761, -3.4301573972082045]], [[1.6076222062949022, -4.3895533367214465], [-2.3184396207481308, -1.8671874567964308]], [[-4.7080977213504225, 2.8618850801895732], [-0.055244417407526392, -0.9598027633353734]], [[2.6865955014788163, 3.3733664950471471], [-0.63454321833788629, -4.6012889135928239]], [[-4.0156079365269539, -0.50548912889536357], [3.5270182042180025, 3.2652688570303958]], [[3.2144367357020514, 0.74281977515378728], [-2.1062988099273969, -1.4869660666559916]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[-1.2650050208921018, 0.3806152070368185], [1.641876098247689, 3.4602728656632671]], [[-4.8014053280184559, -1.4546611375185803], [-0.23927893727610794, -2.2331481410851231]]], [[[3.3156192101612376, -4.9112681189920622], [0.56680609997800779, 1.6690726422193531]], [[2.1570906931846654, -2.53653499194077], [-3.2464913364790213, -4.7753031835726141]]], [[[-4.9206684497219033, -1.3831879123727511], [4.7695460964648486, 0.028615532934836985]], [[-2.6936258579420889, 1.1303026716149276], [1.960250217120457, -4.7582307330054388]]], [[[4.207814265184636, 0.73582836267512253], [-4.1645760588274863, -3.7089881672408032]], [[-3.016868096038646, 2.0459566667869566], [-0.59503931498490292, -1.1178747026145985]]], [[[3.4429871525229245, 2.0215618626373155], [2.4737862831899768, 4.5441793164812481]], [[1.6549916562583435, 4.8047990206258895], [2.9470962834371406, -1.0956537194321503]]], [[[2.6126739069796914, 0.8335328520301406], [4.7360972526212528, 3.0671856522800702]], [[-0.39896320140381825, 1.5013291208473936], [-4.7966692497328189, -0.29108564280592653]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-5.1491591023827832, -3.5035388744538629], [2.7759629270774848, 4.5943596944930629]], [[-7.4800160464715315, -4.1332718559716568], [-3.6694363344843124, -5.6633055382933275]]], [[[4.9232414164561398, -3.30364591269716], [-3.8227472367434387, -2.7204806945020934]], [[-0.16134892756346542, -4.8549746126889008], [-5.1136787932754526, -6.6424906403690454]]], [[[-9.6287661710723249, -6.0912856337231736], [7.6314311766544218, 2.8905006131244102]], [[-2.7488702753496153, 1.0750582542074012], [1.0004474537850836, -5.7180334963408121]]], [[[6.8944097666634523, 3.4224238641539388], [-0.79120956378033913, -0.33562167219365602]], [[-3.6514113143765323, 1.4114134484490704], [-5.1963282285777268, -5.7191636162074229]]], [[[-0.57262078400402938, -1.9940460738896384], [1.9682971542946133, 4.0386901875858845]], [[5.182009860476346, 8.331817224843892], [6.2123651404675364, 2.1696151375982455]]], [[[5.8271106426817427, 4.047969587732192], [5.4789170277750401, 3.8100054274338575]], [[-2.5052620113312152, -0.60496968908000337], [-6.2836353163888106, -1.7780517094619182]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_array_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[[1.9175526738042956, -4.2936982022277705], [4.6605297588195587, 1.8604781349509105]], [[-1.5375570842710253, 3.343497725948092], [-3.3907883779892045, 3.5218476876871101]]], [[[-2.1240259372031165, -2.4296599649950847], [0.90875372408088761, -2.6568563584671234]], [[3.7250157191000994, 4.6184705481331019], [-4.0538065360738429, -3.9871240141964437]]], [[[0.63090162059441468, 0.020895264374765965], [-2.3173116760760681, 0.67361094820229539]], [[0.8389667863952246, 2.5323287945895014], [4.2954206436141682, 4.541804624274894]]], [[[3.6677179979505841, 0.74673699586297637], [2.5189833460511615, 0.098999973472329472]], [[-3.0208551607016787, -2.8490175715721078], [1.1265041233789299, 3.6742096395189368]]], [[[-1.1888057767006899, -1.8857091251701616], [2.1595599653862152, -1.7364354521507219]], [[1.6345894592920471, 2.5037173483889621], [0.3132863147194529, -4.0928196458102173]]], [[[4.5966971612433465, -0.13555692448630197], [-0.08080602889467059, -4.3330092829326903]], [[3.4359398867905409, -2.1769136441780903], [4.1767322614139957, 2.2549052871205681]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[-2.925619664626943, -3.0893719407329288], [-3.9890100616811495, 1.0522481434192743]], [[1.0268097127733373, -3.9298719593150166], [1.4718740676800159, -3.4058622439700459]], [[3.1481595317101014, -1.0257392839574631], [-3.9588778221460297, 1.0650169087529644]], [[-0.84027062450763434, 2.9964926425585494], [-4.4400677151061805, -3.503944423310128]], [[-4.5187503728774647, -3.750396106919236], [2.8003929676333241, 0.38891958388197256]], [[-1.8166550779934934, -2.1449621873258762], [-4.3165918375707291, 4.2564395772379235]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-1.0080669908226474, -7.2193178668547136], [1.5711578180866299, -1.2288938057820182]], [[-5.5265671459521748, -0.64551233573305744], [-2.3385402345699302, 4.5740958311063844]]], [[[-1.0972162244297792, -1.4028502522217474], [-3.021118235234129, -6.5867283177821401]], [[5.1968897867801154, 6.0903446158131178], [-7.4596687800438888, -7.3929862581664896]]], [[[3.7790611523045161, 3.1690547960848674], [-3.3430509600335312, -0.35212833575516767]], [[-3.1199110357508051, -1.4265490275565282], [5.3604375523671326, 5.6068215330278584]]], [[[2.8274473734429497, -0.093533628644657973], [5.5154759886097109, 3.0954926160308789]], [[-7.4609228758078592, -7.2890852866782883], [-2.3774402999311981, 0.17026521620880875]]], [[[-5.7075561495781546, -6.4044594980476264], [-1.5908361415330208, -5.4868315590699579]], [[4.4349824269253713, 5.3041103160222862], [0.70220589860142546, -3.7039000619282447]]], [[[2.7800420832498531, -1.9522120024797953], [-2.2257682162205468, -6.4779714702585665]], [[-0.88065195078018821, -6.4935054817488194], [8.4331718386519192, 6.5113448643584917]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[1.2018552091189481, 4.4360812601069384, -0.95487736423574532, -4.0018598906136029], [4.1883586009872857, 4.3930263135981633, 4.4491291109224047, -1.4365392637785721], [4.3570558295141222, -2.3723559423349294, 1.2119375073032383, -1.9794968856714523]], [[2.0465121661169894, -2.8999845454544717, 4.2311488970544779, 3.2168678259722689], [-2.2622876633564495, 1.4589300722842822, 4.4757624816385704, -3.589976108540661], [-2.8877083993547936, -3.7190208980776083, 1.9043843127257079, -0.68894203575457702]]], [[[-3.5737845317306149, -2.7814719094098237, 0.7312399032438659, 2.797763836223953], [-2.3116802722505869, -0.16062597211821839, 3.1710120977900278, -3.3288641351095984], [-4.5189625915568499, -1.0048798826178307, -0.44445770135610907, 2.7824489447380927]], [[3.3214283737514254, 2.7971231522547253, 4.1880509069358869, 2.3372962072536163], [4.9961452866599529, -2.6604597944202188, 0.81867925201435021, 0.82509580743743793], [3.0588184807327075, -2.2517030024653675, 0.59356875415451604, 3.7802885209754642]]], [[[-1.1170931181541977, 2.9460470543711441, -2.9580184012295496, -1.1319845661207264], [3.0707809941254443, -0.83532408679323922, 2.2009056249986703, 2.170723806164764], [0.25889810685250314, -2.0203600713261127, 0.79844512546939495, -1.0885691520138154]], [[4.4083278687796987, -3.1599803833559106, -2.7706691469558278, -2.6936800797997771], [1.3427878604963972, -0.034774313977666615, 0.42857559282645141, 1.8720076262234375], [2.9779871357126195, 3.6464794497401236, 2.5622799214065157, 1.8770238705513398]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[-4.1566891840776146, 2.1628238261774708, -2.6138016021362853, -0.44564474513904173], [4.3132488514882272, -3.5135910015118799, 4.8497641724452265, -2.7940886704509538], [-1.6739838551982746, -3.3166308845420645, -3.6405317190329334, 0.68363784932298355]], [[-1.6404087506110345, 0.43076664549169497, -1.7835297080840604, 2.1239907724696669], [4.4824149419878516, -2.7711619930200104, 4.4817925784215724, -4.1490105149947869], [-2.8285751098819798, -4.126237844659383, 2.3187594964853346, -1.0941692022069116]]], [[[-3.4422411897855874, 4.2881285693230957, 3.8649356613135204, 2.5762710213097497], [-1.5968404420748783, 2.2849988411762929, -3.5756512946337935, -1.8247286946112808], [4.1912377027486336, -3.9991637128859026, 4.8761790098290021, 3.1354254057818718]], [[2.3354174204776719, -0.64010288166902907, -1.6537590308981587, -3.9471985650124175], [3.8822919379975502, -4.8424487469510158, 2.112654027544135, 2.1246133954270041], [0.14101181581080446, -3.8175054347019275, -3.1231880486532138, -2.8654888116294219]]], [[[-0.72143887343577351, 4.0951941062114017, 0.0620311838919827, 0.020510355267210123], [0.46857120019240828, 2.5090566331544837, 0.85873834895695289, -2.2249276767916926], [2.6883047972636342, 1.8370556511642615, -0.95347132563817283, 3.8848664354266198]], [[-4.6262331712460014, 0.9313889730332221, 3.2262322801539742, 1.6994791357369552], [2.6019162543314733, 0.7970800850287505, 0.17659706688673715, -4.4536031732982995], [1.1526635171986053, -0.33609250562293091, 1.0721005166876445, 0.41407737317539706]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-2.9548339749586665, 6.5989050862844092, -3.5686789663720306, -4.4475046357526447], [8.501607452475513, 0.87943531208628345, 9.2988932833676312, -4.2306279342295259], [2.6830719743158475, -5.6889868268769934, -2.428594211729695, -1.2958590363484688]], [[0.40610341550595486, -2.4692178999627767, 2.4476191889704175, 5.3408585984419359], [2.2201272786314021, -1.3122319207357283, 8.9575550600601428, -7.7389866235354479], [-5.7162835092367734, -7.8452587427369913, 4.2231438092110425, -1.7831112379614886]]], [[[-7.0160257215162023, 1.506656659913272, 4.5961755645573863, 5.3740348575337027], [-3.9085207143254652, 2.1243728690580745, -0.4046391968437657, -5.1535928297208793], [-0.32772488880821626, -5.0040435955037328, 4.431721308472893, 5.9178743505199645]], [[5.6568457942290973, 2.1570202705856962, 2.5342918760377282, -1.6099023577588012], [8.8784372246575032, -7.5029085413712346, 2.9313332795584852, 2.949709202864442], [3.1998302965435119, -6.069208437167295, -2.5296192944986977, 0.91479970934604227]]], [[[-1.8385319915899712, 7.0412411605825458, -2.8959872173375669, -1.1114742108535163], [3.5393521943178525, 1.6737325463612445, 3.0596439739556232, -0.054203870626928552], [2.9472029041161374, -0.18330442016185122, -0.15502620016877788, 2.7962972834128044]], [[-0.21790530246630269, -2.2285914103226885, 0.45556313319814645, -0.99420094406282189], [3.9447041148278705, 0.76230577105108388, 0.60517265971318857, -2.5815955470748619], [4.1306506529112248, 3.3103869441171927, 3.6343804380941602, 2.2911012437267368]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(-0.099264022339,self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array(-4.43101105597)
      sub=res.substitute({arg0:s0})
      ref=Data(-4.53027507831,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(1.53318275317,self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([2.1716752520957927, 3.6901138001593647])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([3.7048580052701601, 5.2232965533337321]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-0.43874032344510461, -3.7324749969533166]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array(-0.818979822397)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-1.257720145841934, -4.5514548193501465]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(2.2169280942,self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[4.5181447511674584, 3.7718360741521639, -1.6479699748781416, 3.4008117258094437, -0.89252705759334994], [-3.8158307877798734, -2.2990564452199735, -2.7801883038848088, 0.24828073211970203, 4.3369662513721661], [1.7233004308960584, -0.67221312039573, -0.021244314389181618, -4.5244738525365271, 3.4190042242511804], [4.7168954604490505, 4.2223444255167504, -1.9149745743106692, -4.0615646120659923, -3.614436564423591]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[6.7350728453629243, 5.9887641683476298, 0.56895811931732432, 5.6177398200049096, 1.3244010366021159], [-1.5989026935844075, -0.082128351024507662, -0.56326020968934287, 2.4652088263151679, 6.553894345567632], [3.9402285250915243, 1.5447149737997359, 2.1956837798062843, -2.3075457583410612, 5.6359323184466463], [6.9338235546445164, 6.4392725197122163, 0.30195351988479668, -1.8446365178705264, -1.3975084702281251]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[4.4536246474951859, -0.6580409177670834, 0.15950565966814878, -1.5729684844326419, -2.2522830697560914], [-1.6885511268970932, -0.63241151251520211, 2.0769487678865675, 3.896420292251765, 2.8663409968307025], [1.4832815871966876, 2.2081540612967814, -3.4106664648240006, -2.4720989671871072, 0.44261597484560156], [-3.6648690426370334, -1.1305211249444347, 1.2156538378529227, -0.63709736395817806, -1.5999786920045276]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array(-3.66191013462)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.79171451287939476, -4.3199510523828746, -3.5024044749476424, -5.2348786190484331, -5.9141932043718821], [-5.3504612615128844, -4.2943216471309933, -1.5849613667292237, 0.23451015763597383, -0.79556913778508864], [-2.1786285474191036, -1.4537560733190098, -7.0725765994397918, -6.1340091018028984, -3.2192941597701896], [-7.3267791772528241, -4.7924312595602263, -2.4462562967628685, -4.2990074985739692, -5.2618888266203188]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(-0.684502901944,self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[-4.8370658989924831, -3.0903440031675968], [-3.4581737950519607, 0.95083479068505561]], [[0.51575377478135742, -1.5389348255566224], [2.6046171758919581, -0.44103789368898472]], [[-0.77451945591027638, -3.1312971546626911], [-1.2451705780168352, -1.0601074108586985]], [[-4.0217594718139091, 3.4389935751992944], [4.9716727328040111, -0.47834010302965169]], [[0.0044749181267809135, 1.2848453670330109], [-3.9643900139054278, -0.82867902689594963]], [[3.9011738657131225, 0.20417081451825325], [-3.5215121573334729, -0.034661637551080204]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-5.5215688009367003, -3.7748469051118141], [-4.1426766969961779, 0.26633188874083835]], [[-0.16874912716285984, -2.2234377275008397], [1.9201142739477408, -1.125540795633202]], [[-1.4590223578544936, -3.8158000566069084], [-1.9296734799610524, -1.7446103128029158]], [[-4.7062623737581264, 2.7544906732550771], [4.2871698308597939, -1.1628430049738689]], [[-0.68002798381743634, 0.60034246508879363], [-4.648892915849645, -1.5131819288401669]], [[3.2166709637689053, -0.480332087425964], [-4.2060150592776901, -0.71916453949529746]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[2.8600059965124105, -0.74499849248737515], [0.1287206962433789, -4.0721322894032541]], [[-1.4253631011417491, 1.6517938083116634], [0.32514173131488722, 0.47365922172174724]], [[-1.9302406082664616, -3.5987952780512913], [2.7436808649571418, 4.9971518932613996]], [[0.45671330379898833, -3.0289624152845143], [-1.1512075064527636, -1.4619415885020546]], [[2.4158731041180905, 2.396712316122712], [2.4316009694554177, -1.1635438222981485]], [[-1.8911803976413619, 1.465762936807085], [-3.3537807583183743, -4.1992974555592708]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array(2.25085102229)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[5.1108570187985993, 1.5058525297988137], [2.3795717185295677, -1.8212812671170653]], [[0.8254879211444397, 3.9026448305978523], [2.575992753601076, 2.7245102440079361]], [[0.32061041401972723, -1.3479442557651025], [4.9945318872433306, 7.2480029155475885]], [[2.7075643260851772, -0.77811139299832544], [1.0996435158334252, 0.78890943378413425]], [[4.6667241264042794, 4.6475633384089008], [4.6824519917416065, 1.0873071999880404]], [[0.35967062464482691, 3.7166139590932739], [-1.1029297360321855, -1.948446433273082]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(-3.42710200028,self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[[0.68208617763326451, 3.9386763868083463, 3.8093111053987734, -3.8260776668501695], [-4.2293080269400276, 2.7799885421802273, 2.8799946807029553, 4.952087964194865], [0.9803231628759228, -0.97371020294394928, 2.1443013623544793, -1.6845713511642559]], [[1.4597835843917153, 2.762456063985069, 3.1551783230008024, -2.4267704038023541], [0.25106081285886983, 3.1611430873651987, -0.97316396965358187, 1.7215600078936095], [-0.00011622431326507865, -0.92139054425571132, 3.7734166620386986, -4.0240600675143243]]], [[[-1.2317973407484217, 1.3780265582751845, -3.5224998363440054, 0.31142819401624244], [-3.9434596538497546, 4.7862730414917625, 3.7991288242479584, 4.1016007866951263], [1.9754276865562694, 4.4726053396880907, -4.8814734050311825, -0.66434243230555712]], [[0.93739381859692106, -2.9770739744452168, 4.8567998379850934, -3.2180744148836746], [2.3417587671490656, -1.3525046889108561, 1.1993171543360308, 4.4003083772135536], [4.0316612997010832, -3.2346203928711481, 2.725673958102659, -3.8362302587938055]]], [[[4.6565572958669428, -0.33022881434403573, -2.3990047667377978, 0.80763500150710765], [1.9984134574543031, 3.4285342716735148, -3.8111906569047962, 3.1179185807811081], [0.86321725200158284, -3.2407924280952338, -3.3958192181057836, -0.76575086346441523]], [[-4.8051650337036635, -4.7034438742713576, 1.6980995938914099, -3.6442324543946691], [-0.60813846112263903, 3.0492456126084164, 0.33399079901878714, -0.6568663742838563], [-3.6489246775967468, 3.1171034151543626, 2.2394229514356496, 1.6917792520983941]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-2.7450158226500796, 0.5115743865250022, 0.38220910511542927, -7.253179667133514], [-7.6564100272233713, -0.64711345810311682, -0.54710731958038883, 1.5249859639115209], [-2.4467788374074213, -4.4008122032272929, -1.2828006379288648, -5.1116733514475996]], [[-1.9673184158916288, -0.66464593629827506, -0.27192367728254174, -5.8538724040856982], [-3.1760411874244743, -0.26595891291814544, -4.4002659699369264, -1.7055419923897346], [-3.4272182245966092, -4.3484925445390559, 0.34631466175535452, -7.451162067797668]]], [[[-4.6588993410317663, -2.0490754420081596, -6.94960183662735, -3.1156738062671017], [-7.3705616541330983, 1.3591710412084184, 0.37202682396461428, 0.67449878641178218], [-1.4516743137270747, 1.0455033394047466, -8.3085754053145262, -4.0914444325889008]], [[-2.489708181686423, -6.4041759747285614, 1.4296978377017493, -6.6451764151670183], [-1.0853432331342785, -4.7796066891941997, -2.2277848459473133, 0.9732063769302095], [0.60455929941773912, -6.6617223931544922, -0.70142804218068511, -7.2633322590771492]]], [[[1.2294552955835987, -3.7573308146273798, -5.8261067670211419, -2.6194669987762365], [-1.428688542829041, 0.0014322713901706585, -7.2382926571881399, -0.30918341950223605], [-2.5638847482817613, -6.6678944283785775, -6.8229212183891281, -4.1928528637477598]], [[-8.2322670339870072, -8.1305458745547021, -1.7290024063919343, -7.0713344546780128], [-4.0352404614059836, -0.37785638767492769, -3.093111201264557, -4.0839683745672009], [-7.0760266778800904, -0.30999858512898149, -1.1876790488476945, -1.73532274818495]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[3.8493361517899451, 3.6041008045928518, 3.7499851131965229, 3.5752016562909681], [-0.80261048602038176, -4.7284846083977534, 1.9525079160392007, 1.5067623484889285], [-1.989967605278693, -0.18669040225698375, 4.8067468974156373, -1.8647150418067349]], [[-2.5318759222266065, -2.0656146505941697, -0.11366028086970736, -2.7098166578631644], [4.4583941558845162, 1.828467822371227, 4.8013730833129067, -3.6758978784135357], [-3.2664020196139898, -0.35229919215697514, 0.37851967132928088, -1.7950103042118393]]], [[[-4.5691298972234105, 1.0687205370983204, 4.044725758652671, -3.1628491678381545], [4.6730635472211173, 1.3435264932401614, 4.7793284884452536, -3.9530764153334488], [-3.6730383294087599, 2.0310233570495111, -4.9227926582991168, -4.0795310875051989]], [[1.8116504331160463, 1.0542069446606606, -2.3810658363105244, -4.7067228780648351], [-0.25365766547572122, -0.34045207714719883, 0.24479315254792589, 3.0924388434756249], [-2.5878435201489758, 4.7339204043295684, 3.8163616573611652, -2.4634139200857508]]], [[[-1.532101078125101, 4.1571560850105875, 0.34021294706883332, 0.037735073084308013], [3.5273774842947816, 1.9555468135571799, -0.052254505222908421, 3.7553294468450495], [-4.4510531438382035, -4.6621679034968286, -4.6864477448103159, 3.8446217093087363]], [[0.67762048755059467, 2.4339674777841891, -4.3676211208403295, 4.1014758924342569], [2.7812504493099235, -4.4695798983428539, 0.69282525936537986, -4.3966633739901173], [1.0331497350940388, 1.8207589262498169, 3.4837369138391168, 4.3259902637692278]]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array(0.403599876176)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[4.2529360279659514, 4.007700680768858, 4.1535849893725292, 3.9788015324669743], [-0.39901060984437553, -4.3248847322217472, 2.3561077922152069, 1.9103622246649348], [-1.5863677291026868, 0.21690947391902249, 5.2103467735916436, -1.4611151656307286]], [[-2.1282760460506003, -1.6620147744181635, 0.28993959530629887, -2.3062167816871582], [4.8619940320605224, 2.2320676985472332, 5.2049729594889129, -3.2722980022375294], [-2.8628021434379836, 0.051300684019031095, 0.78211954750528712, -1.391410428035833]]], [[[-4.1655300210474042, 1.4723204132743266, 4.4483256348286773, -2.7592492916621483], [5.0766634233971235, 1.7471263694161676, 5.1829283646212598, -3.5494765391574425], [-3.2694384532327536, 2.4346232332255173, -4.5191927821231106, -3.6759312113291926]], [[2.2152503092920526, 1.4578068208366668, -1.9774659601345181, -4.3031230018888289], [0.14994221070028502, 0.063147799028807405, 0.64839302872393212, 3.4960387196516312], [-2.1842436439729696, 5.1375202805055746, 4.2199615335371714, -2.0598140439097445]]], [[[-1.1285012019490948, 4.5607559611865938, 0.74381282324483955, 0.44133494926031425], [3.9309773604707878, 2.3591466897331861, 0.35134537095309781, 4.1589293230210558], [-4.0474532676621973, -4.2585680273208224, -4.2828478686343097, 4.2482215854847425]], [[1.0812203637266009, 2.8375673539601953, -3.9640212446643233, 4.5050757686102632], [3.1848503254859297, -4.0659800221668476, 1.0964251355413861, -3.9930634978141111], [1.4367496112700451, 2.2243588024258232, 3.887336790015123, 4.729590139945234]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-1.1276519025581369, 4.7831607690921025]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([1.9057423975769989, -0.63864529575161022])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.77809049501886207, 4.1445154733404923]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Data(numpy.array([3.3400830747497388, -2.6344860888811503]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[3.4808289970944415, -3.1477611455166632], [-3.042746526612734, 4.3871856394056348]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[6.8209120718441802, 0.19232192923307556], [-5.6772326154938844, 1.7526995505244845]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_constData_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[0.14126715460057415, 4.2405965223295929], [2.3963466150374853, 0.88093440175646354]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([-3.1093953046878964, 0.26986788821730734])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-2.9681281500873222, 1.1312012176416966], [2.6662145032547926, 1.1508022899737709]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_constData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Data(numpy.array([3.6090062283980124, -2.9323619851087601]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[-1.3121874165614065, -0.74728919675435268, 3.149861993229381, -4.473967695103946, -4.5963861398173247], [1.7545261247933812, 2.605969916000987, 0.35328604942349262, 0.59442326095098785, -0.86593264897625222], [-2.8099195500903171, -2.3198570236722613, 3.5275035517766167, 4.3888680307028629, 3.5801183413119713], [3.5897090899054902, 1.5860672923444028, -0.51104532611926956, 1.4218015201126128, 2.9587720315918853]], [[1.3328895284376205, -4.415003908804584, 4.266864543617876, -4.8187085046176827, 2.4961906256356787], [4.5504895821781375, 2.2920333654118297, -3.833199371238678, -0.10885957861856976, 2.9401467778433057], [2.2053527873087253, -1.6201611408587535, -3.0477230483108864, 0.8211930928387714, -0.86667363051508683], [-0.99164585985788634, 2.0661028403937678, 3.3873750549956654, -2.468217879802669, 0.24975413832154292]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[2.296818811836606, 2.8617170316436598, 6.7588682216273934, -0.86496146670593355, -0.98737991141931225], [5.3635323531913937, 6.2149761443989995, 3.9622922778215051, 4.2034294893490003, 2.7430735794217602], [0.79908667830769531, 1.2891492047257511, 7.1365097801746291, 7.9978742591008753, 7.1891245697099837], [7.1987153183035026, 5.1950735207424152, 3.0979609022787429, 5.0308077485106253, 6.5677782599898977]], [[-1.5994724566711396, -7.3473658939133442, 1.3345025585091159, -7.7510704897264429, -0.43617135947308139], [1.6181275970693774, -0.64032861969693045, -6.7655613563474386, -3.0412215637273299, 0.0077847927345455759], [-0.72700919780003481, -4.5525231259675136, -5.9800850334196465, -2.1111688922699887, -3.799035615623847], [-3.9240078449666465, -0.86625914471499232, 0.45501306988690526, -5.4005798649114292, -2.6826078467872172]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_constData_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[-3.9751145968893784, -1.7815011019332658, -0.34921428117874598, 2.9471297803471375, 0.68862307621881413], [-3.9194642178237435, -4.4113998740513072, -2.0999169035392997, 2.4423286414900511, -3.2356224951168677], [2.9174319288165504, -1.1880673285834362, -2.6669154275153439, -1.5669414168135432, 2.2244052151197389], [-4.3520169992019939, -4.3508874338277508, 4.8985467924998662, -3.6335347221690215, 1.2173206533408223]], [[1.6044862113983802, 2.8474931866319695, -1.5960041809371353, 1.722103272589008, -0.056125106591238705], [4.2071256604004539, 2.327159068787914, 0.70210674493569414, 1.3373999858242671, -2.8691546276578817], [-4.6517676241496311, 0.31161875557239505, 1.1036010602527311, -0.43143375201614464, 3.2416457699680201], [1.9560978389963148, 1.6271532063527125, -1.5587805426556836, -1.6988711156150571, -3.1117257536073026]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([0.86630676972762721, 4.2012738297999839])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-3.1088078271617512, -0.91519433220563862, 0.51709248854888124, 3.8134365500747647, 1.5549298459464413], [-3.0531574480961163, -3.54509310432368, -1.2336101338116725, 3.3086354112176783, -2.3693157253892405], [3.7837386985441777, -0.32176055885580901, -1.8006086577877167, -0.70063464708591594, 3.0907119848473661], [-3.4857102294743667, -3.4845806641001236, 5.7648535622274935, -2.7672279524413943, 2.0836274230684495]], [[5.8057600411983641, 7.0487670164319534, 2.6052696488628486, 5.9233771023889918, 4.1451487232087452], [8.4083994902004378, 6.5284328985878979, 4.903380574735678, 5.538673815624251, 1.3321192021421022], [-0.4504937943496472, 4.5128925853723789, 5.304874890052715, 3.7698400777838392, 7.442919599768004], [6.1573716687962987, 5.8284270361526964, 2.6424932871443003, 2.5024027141849268, 1.0895480761926812]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_constData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Data(numpy.array([0.369103021205194, 1.4744033380644641]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[[2.9848837135031676, -4.8400330434384911], [1.5903658720874851, -2.7938472281695326]], [[-2.8598335610150882, 0.301975403733028], [-0.63030852348700872, -4.507139738093505]], [[2.9798291992601298, 1.0588556599745118], [0.58821862944753089, 1.822435649831478]], [[-0.6065035333176958, -0.17772532645023453], [-3.3533357287325782, 1.9945025017121178]], [[-4.4819548290001006, -0.654320006755432], [4.9224208377639549, 0.066385999413011376]], [[-2.0682586744332996, -4.936458420773512], [-4.6016255192974374, 3.102390255324595]]], [[[-3.8129774160640642, 3.7604814237191864], [1.2634671543567606, 3.5233132771777349]], [[-4.7260281165518281, 0.085638964235337234], [-0.090265825242354047, -4.1392294879675084]], [[-4.0865705137771382, -1.7152493511568778], [-4.1852051204643104, 2.238188830214888]], [[-0.78443423388925648, -3.1287127833665194], [-3.4630610732479958, -4.4333433925322261]], [[-0.36131141617188334, -3.0104937286047475], [1.1822594740613503, -4.2430555844133728]], [[-2.4267752769959294, -0.43386970874213393], [2.3210758252177088, 3.4955031402854981]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[3.3539867347083616, -4.4709300222332971], [1.9594688932926791, -2.4247442069643386]], [[-2.4907305398098942, 0.67107842493822201], [-0.26120550228181472, -4.138036716888311]], [[3.3489322204653238, 1.4279586811797058], [0.9573216506527249, 2.191538671036672]], [[-0.23740051211250179, 0.19137769475495947], [-2.9842327075273842, 2.3636055229173119]], [[-4.1128518077949066, -0.285216985550238], [5.2915238589691489, 0.43548902061820538]], [[-1.6991556532281056, -4.567355399568318], [-4.2325224980922433, 3.471493276529789]]], [[[-2.3385740779996, 5.2348847617836505], [2.7378704924212247, 4.997716615242199]], [[-3.251624778487364, 1.5600423022998013], [1.3841375128221101, -2.6648261499030443]], [[-2.6121671757126741, -0.24084601309241371], [-2.7108017823998463, 3.7125921682793521]], [[0.68996910417520763, -1.6543094453020553], [-1.9886577351835317, -2.958940054467762]], [[1.1130919218925808, -1.5360903905402834], [2.6566628121258145, -2.7686522463489087]], [[-0.95237193893146532, 1.0405336293223302], [3.7954791632821729, 4.9699064783499622]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_constData_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[[-1.7582106206755013, 4.3314049717971859], [-1.1006206259611484, 2.8694083313239762]], [[2.9701378957446138, -2.1357229217604603], [-3.6984849606365433, -2.7155524854922861]], [[3.5891739010035035, 0.74841474663498531], [0.69498633190126569, -2.6751044999697635]], [[2.3226124223089952, 3.058341892164762], [-4.9845244615604773, 3.9805543566838075]], [[4.6426004181965634, 2.5688187987879143], [4.7841256010994169, -4.920173583044285]], [[3.9282975234206727, -3.7609915293151817], [-3.5188670992087445, 4.6574456406968157]]], [[[-3.4042658473013363, -2.9976363418267384], [4.0109006851045699, 1.1192705883017871]], [[-1.0435904975644728, -4.3358719743943874], [-1.5127089953803949, -1.1802024080969509]], [[0.21334046222759184, -4.6980128984504743], [-3.8512939343521757, 2.6047639213565219]], [[4.9604762460480263, -4.2445554955256029], [-0.5834101728151051, -0.36610536815253347]], [[3.0410690876073225, -1.6884211182833955], [-0.7093949211030246, 3.6971963123250458]], [[-2.2271559788893613, -2.8854100595848609], [2.5305467851239882, -4.7142488956455573]]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([2.6978349395586463, 0.32261539047167886])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.93962431888314502, 7.0292399113558321], [1.5972143135974979, 5.5672432708826225]], [[5.6679728353032601, 0.56211201779818598], [-1.000650021077897, -0.017717545933639833]], [[6.2870088405621498, 3.4462496861936316], [3.392821271459912, 0.022730439588882767]], [[5.0204473618676415, 5.7561768317234083], [-2.2866895220018311, 6.6783892962424538]], [[7.3404353577552097, 5.2666537383465606], [7.4819605406580632, -2.2223386434856387]], [[6.626132462979319, -1.0631565897565354], [-0.82103215965009824, 7.3552805802554619]]], [[[-3.0816504568296574, -2.6750209513550596], [4.3335160755762487, 1.4418859787734659]], [[-0.72097510709279389, -4.0132565839227086], [-1.1900936049087161, -0.85758701762527201]], [[0.53595585269927071, -4.3753975079787955], [-3.5286785438804968, 2.9273793118282008]], [[5.2830916365197051, -3.921940105053924], [-0.26079478234342623, -0.043489977680854608]], [[3.3636844780790014, -1.3658057278117166], [-0.38677953063134574, 4.0198117027967246]], [[-1.9045405884176825, -2.562794669113182], [2.8531621755956671, -4.3916335051738784]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[1.6344235556984597, -4.8184445530984146, -3.3060304427110019, -0.70816050751717086, -4.3407239048273567], [-4.5918929994252924, 2.0583697082723695, 1.2416026362312493, -4.9614144079584559, 0.87575864205241061], [0.92508638792214182, -1.1826797693803637, -1.9642416497498596, 0.53265941175075415, 0.78568629935778755], [1.212579169248162, -1.5582023765233464, -0.737097732015215, 0.37809060305346787, -1.7324255973969427]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[-1.9447411398243419, -2.6696485008656867, -2.4252826768643265, -0.36710163258244677, -3.8085059930255283], [0.16304633074955976, -3.0039207532793388, 3.2478834518906297, -1.8781823191381664, 2.998175800522227], [-3.3746787951573145, -2.3601902348791146, 1.3030385736767833, 4.6241647435548252, -0.14071439831772281], [-0.48853972714839067, -0.2772454793355541, -4.1386750512062971, 1.8628851687356782, -0.55496558525688755]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.31031758412588228, -7.4880930539641017, -5.7313131195753284, -1.0752621400996176, -8.1492298978528854], [-4.4288466686757326, -0.94555104500696929, 4.489486088121879, -6.8395967270966223, 3.8739344425746376], [-2.4495924072351727, -3.5428700042594783, -0.66120307607307627, 5.1568241553055794, 0.64497190104006474], [0.72403944209977134, -1.8354478558589005, -4.8757727832215121, 2.2409757717891461, -2.2873911826538302]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Data(numpy.array([[-3.7943859112943876, 3.0889957458416255, -4.9813256230058292, 2.0734069995810369, 3.3838077363124235], [4.0857480489571305, 2.0492515030223357, 0.95503769549910267, 2.1910125365149202, 0.10835349206563194], [-3.7022106609104011, 0.70242419161583047, 1.8961660627236938, 4.7593273793805864, 0.87996547020843963], [2.1468032840155722, -4.7350401722750988, 3.773472249086069, 2.1656776510157698, -3.2395035395639091]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[-3.2437577770617878, 2.0077483011795341], [-1.0553454998346945, -4.0914560031844118], [2.5543559182011748, 0.82240629469007143], [2.942750063632217, 4.978592446838265], [4.6508179191086239, 4.1786679623623257]], [[4.3956199552593986, -1.0059699115266096], [0.43818691225431383, -2.9418092523931829], [4.1674373201293999, 1.4799196468352509], [3.3763348371156372, -3.7135108028959483], [4.770124546286576, -1.114676097440388]], [[2.9235554639150285, 4.974215688112702], [0.67237309268924506, 1.5855434879639105], [-3.656268859690476, 0.98811658623205822], [1.5099173730545328, -3.4295522812879184], [-3.5704207532397492, -1.3050133727494497]], [[-4.1903855237979295, -0.5854531926328308], [0.73157715696383541, -3.7725698150043083], [-0.29916658322297884, -4.0701009807868402], [0.032256791755942871, 2.6667872697099231], [0.92753491288837697, 4.351785323252674]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-7.0381436883561754, -1.7866376101148536], [2.033650246006931, -1.0024602573427863], [-2.4269697048046543, -4.1589193283157577], [5.0161570632132539, 7.0519994464193019], [8.0346256554210473, 7.5624756986747492]], [[8.4813680042165291, 3.0797781374305209], [2.4874384152766496, -0.8925577493708472], [5.1224750156285026, 2.4349573423343536], [5.5673473736305574, -1.522498266381028], [4.8784780383522079, -1.006322605374756]], [[-0.77865519699537256, 1.2720050272023009], [1.3747972843050755, 2.287967679579741], [-1.7601027969667822, 2.884282648955752], [6.2692447524351191, 1.3297750980926679], [-2.6904552830313095, -0.4250479025410101]], [[-2.0435822397823573, 1.5613500913827414], [-4.0034630153112634, -8.5076099872794071], [3.4743056658630902, -0.2966287317007712], [2.1979344427717127, 4.8324649207256929], [-2.3119686266755322, 1.1122817836887648]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_constData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[1.6763222109832876, 2.7822926349632855], [4.8137102485619074, -2.5204405346622916], [-0.5518987295814437, 4.1077757471982252], [2.6871501285860848, -3.7391761106638288], [-0.73860329950925596, 0.456271111680846]], [[-0.80497903390180259, -2.3238622111652854], [-0.84777898791162265, 3.0704155708832896], [3.4779658278547139, -4.8603340380395688], [-1.002803686935454, 2.9494628847449551], [4.4835011704563463, 3.3990170334941876]], [[4.8870031682449113, -3.1431455511871564], [1.9504467619155754, 1.5158222437163058], [-4.0184071869992133, -0.98985998682217513], [-4.4505399490968669, 3.8713946638928967], [-3.9848353711368247, 4.1870461413497253]], [[-2.2374392746692617, -2.1380113677127888], [1.1686543560754519, -2.0324077946983699], [-3.6848790867215584, -2.7282944166015266], [-3.6307039810990984, 0.9097896702160142], [0.59283784622745017, -2.0339663928555121]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[-2.1341681854130878, -2.6923111771537531, -1.5333509289332605, 3.35878590734076, 3.4833823333502316], [-3.554944088779739, -3.960472020512098, 4.9519254859046082, -4.4121564277529366, 1.9174933235076974], [-0.66290553851710765, 3.0638327284885474, -3.8485133597151, -4.8348019503008333, -3.7405002629140749], [-1.8780438185874515, 0.54932577527941362, -2.575470290995816, 1.0095604922851891, 2.6872045701369949]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.45784597442980024, 0.64812444955019766], [2.1213990714081543, -5.2127517118160451], [-2.0852496585147042, 2.5744248182649647], [6.0459360359268448, -0.38039020332306883], [2.7447790338409757, 3.9396534450310776]], [[-4.3599231226815416, -5.8788062999450243], [-4.8082510084237207, -0.89005644962880837], [8.4298913137593221, 0.091591447865039477], [-5.414960114688391, -1.4626935430079815], [6.4009944939640437, 5.316510357001885]], [[4.2240976297278037, -3.8060510897042641], [5.0142794904041228, 4.5796549722048532], [-7.8669205467143133, -4.8383733465372751], [-9.2853418993977002, -0.9634072864079366], [-7.7253356340509001, 0.44654587843565041]], [[-4.1154830932567137, -4.0160551863002407], [1.7179801313548655, -1.4830820194189562], [-6.2603493777173744, -5.3037647075973426], [-2.6211434888139094, 1.9193501625012033], [3.280042416364445, 0.65323817728148281]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Data(numpy.array([[-3.4260486676008028, 4.3518023038089932, 1.6641817779133081, -0.59585590842578195, -1.3983257161121743], [-1.7152572506658958, 2.8662124638183339, -2.4254587271247638, 0.27239040922070945, -1.0281439207207721], [0.26880292459592336, 3.1055719278336618, 0.82599381500387459, 3.9505730854537511, 4.2770580910190379], [-1.3533570527645411, -1.6292021968024804, -0.66785665668275396, -4.863144581346841, 2.4522071393578839]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[[4.51878187611028, -4.7866925072989304, 2.8134411150819982, 3.2646539917062114, 3.0023794047631736], [0.88028722279539817, -3.996044158777857, 4.616764993707335, -3.2463431319130711, 3.2240016906068902], [0.90188423954376074, -2.8188571055069511, 4.7044240121712839, -3.7273575258241074, 3.2920527366030559], [0.62384329561416241, -4.8854814733219474, 4.2933967051989281, 1.2299113130873787, 1.7114729016251538]], [[1.1378806570384672, 3.4696114622616037, 2.8283040228642928, 4.358206584807693, 3.5271084141481168], [3.1800028624277843, -4.9366271079362543, -2.6247758952836211, -1.2100917555647737, -1.4627546844494645], [2.6949122945381854, 1.0528266122044165, 0.68928488332564619, 0.42832546042400477, -3.3572783184931421], [1.481833591456895, -2.8374819988312048, -0.072263260508584004, -0.64492018389771388, 1.1141243046828588]], [[3.8666871251719588, -2.3602700423270861, 2.7187862508301786, -2.2880504833419768, -1.1836641708230502], [-4.7666877200915945, -4.2997882761065895, 3.1202852585273249, -3.7575583369078425, -3.3281998169464644], [-2.1393318106507953, -0.14578862937703718, 4.7714777184136139, 4.8046187905507622, -1.5425584447241594], [-0.91740827168573702, -2.3537807311662649, -0.058643226279711413, -2.0985852392233637, 2.1559289855168418]], [[-3.6566272801567132, 0.078451318575158524, -4.3405594915435035, -4.4984265072419385, -1.7772257975858197], [0.27284442234725326, -2.3058016806348283, 0.81069591759589077, -1.2303606009934267, 2.2803060491820606], [2.561623668012281, 1.1969201408071983, -4.7599595928984675, -4.1556602523725967, 2.1661731380389257], [-3.9898045594933604, -0.1908755128299191, -2.5971488036487198, 4.764659695259116, 4.703203185097145]], [[4.2361190333489525, -3.8102654384186541, 4.8335855706801567, 0.24199082267511507, 4.1560586389675382], [4.6048657875274284, 4.1409367507300914, 2.048980994646997, 3.1255959775786142, -1.4430128439596235], [1.6322707541305821, 4.1994942958463621, 0.2613322919165757, 1.3802206078593002, -2.7116915293177324], [-1.1831797109217765, 2.5299324286873794, -2.0951513006299405, -3.0564166276558002, -1.4749807662863921]]], [[[3.7225737656985576, 4.7950969164813824, -3.5491478633297935, 4.3164013731085067, -4.5581037340892472], [0.74991113571856172, -2.3550875800345663, 2.8329937914018171, 1.9341531485110206, -0.4383762799358113], [1.525931394191069, 2.9367176402741011, -0.31136670742979522, 1.422418107194642, 0.76010160556553874], [3.6055573496128961, -3.4152242653911227, 1.5705052988270118, -0.045108258705206516, 0.40093115574942928]], [[4.8945114498727023, 4.2295101998191882, 3.1229679933406587, 0.28032925872743064, -2.0000118247520202], [-3.9661954405821973, -3.7751958120750331, -1.220928136195595, -0.56445679060978993, 0.89432724135132347], [2.8804958937337535, -4.4692617592910588, 4.4378718198858742, 0.33523512439806336, -4.4190308488442698], [-2.2848929438196866, 2.1875867899394681, -1.3979367912269591, -0.10553740005186008, 3.1803456659447527]], [[1.8969832976307242, -2.7927530607972182, 0.77026814434835078, 4.3345275415633431, 0.65170046079219279], [-3.173775046884014, -1.688588186311101, 0.07741776196126704, -4.5831038539182147, 3.2820669639145645], [2.1632215761910878, 3.834936574304372, 4.5926009906909293, -3.1621375153821041, -0.92192779802599212], [1.917638825204401, -2.5619948565692887, -4.7349929581422838, -1.1200679526305835, 0.76298563239236472]], [[3.569377918162143, 1.6422692669428329, 0.7869978046415369, -1.7036293971264129, -1.152833725511746], [-3.4985638650915076, -4.4084713544205778, -2.0473630616792473, 0.99513978358373745, 2.6511775183482067], [-2.5319550339328813, 4.3985421832072227, 3.1591221188331975, -2.312859906358554, 4.2967836247129849], [1.6589915976628378, 3.3797221478311137, 0.25201147406602331, -4.8979712085014278, -2.680149731866559]], [[-3.3937969621813835, -0.50708905078087874, -1.0324606948401138, -2.0364255955705666, 0.90260476523196331], [2.0495249509427893, -0.42869576805683884, 1.8535484254665153, 0.015300720573437232, -3.2991298450973394], [2.2795311418684037, 0.46843481112156393, 0.66216603568586407, -2.1000517458996604, -2.695711997695188], [-3.7690483583579102, -3.6541268766683155, -1.0526936797323714, -3.1879179473400567, 2.6398467508692383]]], [[[0.067771620733879878, -3.7378625761012119, -1.7113357506243032, 2.4311269824687756, 4.9238930988129042], [1.1491947940109979, 1.8659405937844227, -3.8515120342410567, 0.28474844771650254, 0.69742702684417512], [1.8478371312822315, -1.5301544056497662, 2.25276582844229, -1.1175849424234761, -3.8027966541411917], [-4.8564401821174812, 4.9161562943104649, 4.9832820415783647, 4.0800994824377526, 3.0885935825062489]], [[1.5228998521634765, -0.012949721116557456, 4.4608263771594725, -1.8611832743482992, 3.5598478147261652], [1.3068082036185746, 4.6427969623674681, 2.1287842144958873, 3.8855729006199198, -4.2102787226754241], [-1.7974029420871194, -2.7714581202683233, -1.8282925693040539, -3.7380874700517266, -4.6881221080732924], [3.7151967506697474, 4.0162073140804804, 1.6285968388528076, 2.3102580581903869, 1.479375201172207]], [[1.6404843931608379, 4.7597769839946764, 4.9019514838613691, 2.4509743562260589, 1.5747082526028127], [0.90654885622438197, -3.4028930908812285, -4.4277669175947274, 0.83235644292833122, -1.8842256364024088], [-1.4439646726102984, 3.2057048370823633, 2.0299054450676888, -1.1839618234461735, -0.42961084064065158], [-3.4103701594233069, 0.65906964911068755, 1.6904779028124857, 4.0292037890181494, -4.7349158527465942]], [[0.87240544977270407, -2.9211110334463628, -0.41102721573272571, 4.0846891597467998, -4.3872070146353686], [-0.68324819500419842, 3.1761763645291836, -2.5226996930847867, 0.96152998648355315, 3.8686791702023786], [3.7453617287470937, 3.6351596661766798, 1.3676119559179556, 0.41512183398059754, -4.2049278265727406], [2.615540812008085, -0.62994577636074212, 4.5162501763860536, -4.904272263407834, -0.64770962120658826]], [[-2.8782990336152836, 4.5740788920023334, -2.824598024005156, -3.9136441392708408, -1.1050548714265176], [4.962391126345171, -0.31373518342200946, 1.3428315754083151, -0.14639870640497143, -2.782321964507676], [-3.0260217555223399, 4.8620735589767552, -4.6098264082756648, 3.608803251942998, 2.3512751432002421], [-0.89344284744866975, 0.083439010920500323, -0.85673474673689398, -2.4778439589488102, -1.0010623146316613]]], [[[-2.9951447994582825, -4.620536572965559, 4.4385981725318242, -3.9986376248565882, -0.87405975520307422], [0.49872011175692599, -4.2384855338972827, -1.9813496672949951, 1.4570288796347217, -4.8160021020993362], [4.4853483082009564, 3.2471669414154434, -2.2840661787747973, -4.9108285093039044, -1.4996221847547533], [3.7358286442665314, -1.8090440079294181, 1.2312151466880756, 4.1331312465060197, -2.5667376891767368]], [[-1.8818588682328983, 3.5581368520571708, -3.5595480074970007, 0.98538064113338297, 2.7123966860259729], [-0.86410673346997058, -2.7197511535112384, 3.8703115021158556, 4.9918244586533387, 0.53628922365962506], [-3.6318611096486917, 4.4417901832747084, -4.2284847163744068, 4.5685549824935325, -4.3007179373907523], [-1.0256793905621118, -1.3387869196137947, -4.1746868916444253, -2.3693299651719846, 3.3437777747369815]], [[-3.8074394759016306, 4.6406382846574541, 1.1690293790764441, -1.8990827565315147, -0.83215794899198769], [2.4011252505707521, -4.8042583331810116, -1.9013565765830953, -2.5150650005622377, 1.8271156548358682], [4.9650942339455497, 0.54477582610937603, -3.6970796575242426, 0.90391019724101263, -4.8307710195861304], [0.61333237872374369, 4.3986455056606637, 2.5156950273442966, 4.6576009382355128, 3.7148886763112081]], [[3.4550752383480514, -3.0899445947521742, 2.8419020526971366, -3.8172520544682453, 0.66429812223557594], [2.5342186825147284, 0.57638396875834363, 1.3769821998538632, -4.8309894383504313, 2.1652264693389203], [-3.1612905802883597, -3.2787416690979301, 4.2552905171530551, -0.46691818411433683, 1.9194681452767135], [1.0245990861760559, -0.736750970271288, 1.0570684069898384, 1.4146273939327081, -3.2554334767515591]], [[-2.9371032277050046, -2.604842801755348, 0.73100551776113853, 2.373669946316916, 3.8628361549786376], [-2.9496064030363267, -3.1188025017145371, 3.851936436442486, -0.95160649806678688, -1.7643208616736894], [-3.5839784702177537, 2.2174934230852603, 0.53998978240506013, 1.3330143230344618, 3.4661670065550823], [-2.0181115588952512, 1.207448251021674, -0.72174605503197231, -0.64465877598007815, 2.1905935326894781]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.0927332085094772, -8.2127411748997332, -0.61260755251880461, -0.16139467589459144, -0.42366926283762929], [-2.5457614448054047, -7.4220928263786599, 1.1907163261065321, -6.672391799513874, -0.20204697699391261], [-2.5241644280570421, -6.244905773107754, 1.2783753445704811, -7.1534061934249102, -0.13399593099774698], [-2.8022053719866404, -8.3115301409227502, 0.86734803759812529, -2.1961373545134242, -1.7145757659756491]], [[5.4896829608474604, 7.8214137660705969, 7.1801063266732861, 8.7100088886166862, 7.8789107179571101], [7.5318051662367775, -0.58482480412726101, 1.7270264085253721, 3.1417105482442196, 2.8890476193595287], [7.0467145983471786, 5.4046289160134098, 5.0410871871346394, 4.780127764232998, 0.99452398531585118], [5.8336358952658882, 1.5143203049777885, 4.2795390433004092, 3.7068821199112794, 5.4659266084918521]], [[5.5308689030852669, -0.69608826441377802, 4.3829680287434867, -0.62386870542866868, 0.48051760709025793], [-3.1025059421782863, -2.6356064981932814, 4.784467036440633, -2.0933765589945343, -1.6640180390331563], [-0.47515003273748713, 1.5183931485362709, 6.435659496326922, 6.4688005684640704, 0.12162333318914875], [0.7467735062275711, -0.68959895325295673, 1.6055385516335967, -0.43440346131005558, 3.8201107634301499]], [[-4.2524831885824952, -0.51740458985062343, -4.9364153999692855, -5.0942824156677204, -2.3730817060116016], [-0.3230114860785287, -2.9016575890606102, 0.21484000917010881, -1.8262165094192087, 1.6844501407562786], [1.9657677595864991, 0.60106423238141637, -5.3558155013242494, -4.7515161607983787, 1.5703172296131438], [-4.5856604679191424, -0.78673142125570106, -3.1930047120745018, 4.1688037868333341, 4.107347276671363]], [[2.8377933172367782, -5.2085911545308283, 3.4352598545679824, -1.1563348934370592, 2.757732922855364], [3.2065400714152541, 2.7426110346179171, 0.65065527853482275, 1.7272702614664399, -2.8413385600717977], [0.23394503801840782, 2.8011685797341879, -1.1369934241955986, -0.018105108252874036, -4.1100172454299067], [-2.5815054270339508, 1.1316067125752052, -3.4934770167421147, -4.4547423437679745, -2.8733064823985663]]], [[[2.0073165150326617, 3.0798396658154865, -5.2644051139956893, 2.6011441224426108, -6.2733609847551435], [-0.96534611494733413, -4.0703448307004617, 1.1177365407359212, 0.21889589784512475, -2.1536335306017071], [-0.1893258564748268, 1.2214603896082052, -2.0266239580956911, -0.29283914347125384, -0.95515564510035711], [1.8903000989470002, -5.130481516057019, -0.14475195183888401, -1.7603655093711024, -1.3143260949164666]], [[7.7607239136910362, 7.0957226636375221, 5.9891804571589926, 3.1465417225457646, 0.86620063906631373], [-1.0999829767638634, -0.90898334825669913, 1.6452843276227389, 2.301755673208544, 3.7605397051696574], [5.7467083575520874, -1.6030492954727249, 7.3040842837042081, 3.2014475882163973, -1.5528183850259358], [0.58131951999864739, 5.053799253757802, 1.4682756725913748, 2.7606750637664739, 6.0465581297630866]], [[-0.52847542949403969, -5.218211787921982, -1.6551905827764131, 1.9090688144385792, -1.7737582663325711], [-5.5992337740087779, -4.1140469134358648, -2.3480409651634968, -7.0085625810429786, 0.85660823678980069], [-0.26223715093367606, 1.4094778471796081, 2.1671422635661655, -5.5875962425068675, -3.347386525150756], [-0.50781990192036286, -4.9874535836940526, -7.1604516852670477, -3.5455266797553473, -1.6624730947323991]], [[3.8417683273828525, 1.9146596761635424, 1.0593882138622464, -1.4312389879057035, -0.88044331629103656], [-3.2261734558707982, -4.1360809451998684, -1.7749726524585379, 1.2675301928044469, 2.9235679275689161], [-2.2595646247121719, 4.6709325924279321, 3.4315125280539069, -2.0404694971378445, 4.5691740339336944], [1.9313820068835472, 3.6521125570518231, 0.52440188328673276, -4.6255807992807183, -2.4077593226458496]], [[-4.421940882902156, -1.5352329715016508, -2.0606046155608859, -3.0645695162913387, -0.12553915548880878], [1.0213810302220172, -1.4568396887776109, 0.82540450474574323, -1.0128432001473349, -4.3272737658181111], [1.2513872211476316, -0.55970910959920817, -0.36597788503490802, -3.1281956666204325, -3.7238559184159601], [-4.7971922790786827, -4.6822707973890871, -2.0808376004531435, -4.2160618680608284, 1.6117028301484662]]], [[[0.33657454532980324, -3.4690596515052885, -1.4425328260283798, 2.6999299070646989, 5.1926960234088275], [1.4179977186069213, 2.134743518380346, -3.5827091096451333, 0.5535513723124259, 0.96622995144009849], [2.1166400558781548, -1.2613514810538429, 2.5215687530382134, -0.84878201782755269, -3.5339937295452684], [-4.5876372575215578, 5.1849592189063882, 5.2520849661742881, 4.3489024070336759, 3.3573965071021723]], [[4.6284717799971382, 3.0926222067171043, 7.5663983049931343, 1.2443886534853625, 6.6654197425598269], [4.4123801314522364, 7.7483688902011298, 5.234356142329549, 6.9911448284535815, -1.1047067948417624], [1.3081689857465424, 0.33411380756533848, 1.2772793585296078, -0.63251554221806483, -1.5825501802396307], [6.8207686785034092, 7.1217792419141421, 4.7341687666864694, 5.4158299860240486, 4.5849471290058688]], [[2.4664782081647125, 5.585770798998551, 5.7279452988652437, 3.2769681712299334, 2.4007020676066873], [1.7325426712282566, -2.5768992758773539, -3.6017731025908528, 1.6583502579322058, -1.0582318213985342], [-0.61797085760642378, 4.0316986520862379, 2.8558992600715634, -0.35796800844229892, 0.39638297436322301], [-2.5843763444194323, 1.4850634641145621, 2.5164717178163603, 4.8551976040220239, -3.9089220377427196]], [[4.8229785352264551, 1.0294620520073883, 3.5395458697210254, 8.0352622452005509, -0.43663392918161748], [3.2673248904495527, 7.1267494499829347, 1.4278733923689644, 4.9121030719373042, 7.8192522556561297], [7.6959348142008448, 7.5857327516304309, 5.3181850413717067, 4.3656949194343486, -0.25435474111898948], [6.566113897461836, 3.320627309093009, 8.4668232618398047, -0.95369917795408288, 3.3028634642471628]], [[1.3987590574037543, 8.8511369830213713, 1.4524600670138819, 0.36341395174819713, 3.1720032195925203], [9.2394492173642089, 3.9633229075970284, 5.619889666427353, 4.1306593846140665, 1.4947361265113619], [1.251036335496698, 9.1391316499957931, -0.33276831725662692, 7.8858613429620359, 6.62833323421928], [3.3836152435703681, 4.3604971019395382, 3.4203233442821439, 1.7992141320702277, 3.2759957763873766]]], [[[-4.3485018522228236, -5.9738936257301001, 3.0852411197672831, -5.3519946776211293, -2.2274168079676153], [-0.85463694100761511, -5.5918425866618238, -3.3347067200595362, 0.10367182687018062, -6.1693591548638773], [3.1319912554364153, 1.8938098886509023, -3.6374232315393384, -6.2641855620684455, -2.8529792375192944], [2.3824715915019903, -3.1624010606939592, -0.12214190607646547, 2.7797741937414786, -3.9200947419412779]], [[-3.5110610650353786, 1.9289346552546904, -5.1887502042994811, -0.64382155566909738, 1.0831944892234926], [-2.4933089302724509, -4.3489533503137192, 2.2411093053133753, 3.3626222618508583, -1.0929129731428553], [-5.261063306451172, 2.8125879864722281, -5.8576869131768872, 2.9393527856910522, -5.9299201341932326], [-2.6548815873645921, -2.9679891164162751, -5.8038890884469057, -3.9985321619744649, 1.7145755779345011]], [[-4.4752961325843845, 3.9727816279747001, 0.50117272239369015, -2.5669394132142687, -1.5000146056747417], [1.7332685938879981, -5.4721149898637655, -2.5692132332658493, -3.1829216572449917, 1.1592589981531143], [4.2972375772627958, -0.12308083057337793, -4.3649363142069966, 0.23605354055825867, -5.4986276762688844], [-0.05452427795901027, 3.7307888489779097, 1.8478383706615427, 3.9897442815527588, 3.0470320196284542]], [[-1.4080693429987896, -7.9530891760990148, -2.0212425286497044, -8.6803966358150859, -4.1988464591112651], [-2.3289258988321127, -4.2867606125884974, -3.4861623814929779, -9.6941340196972732, -2.6979181120079208], [-8.0244351616351999, -8.1418862504447702, -0.60785406419378596, -5.3300627654611779, -2.9436764360701275], [-3.8385454951707851, -5.599895551618129, -3.8060761743570026, -3.4485171874141329, -8.1185780580984002]], [[-0.48489608834712072, -0.15263566239746407, 3.1832126571190225, 4.8258770856747999, 6.3150432943365216], [-0.49739926367844278, -0.66659536235665318, 6.3041435758003699, 1.500600641291097, 0.68788627768419452], [-1.1317713308598698, 4.6697005624431442, 2.9921969217629441, 3.7852214623923457, 5.9183741459129662], [0.43409558046263275, 3.6596553903795579, 1.7304610843259116, 1.8075483633778058, 4.642800672047362]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_constData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[[-3.033574764312589, 3.012260580256557, -2.3178453768543825, 0.64379576208176825, 1.1417343838086413], [-4.9152552442680664, 3.5939261461740877, -1.182398160314917, -1.8706244652617543, -3.745457176573769], [-3.9491060519328047, -1.7672185479305735, 2.9008370330606779, -4.7395932459446319, -0.25517991185893241], [2.5231229160893989, -1.7587381757243072, -3.8731025459870527, -3.4221502896767064, 3.6049595529099339]], [[-1.2932146863128979, -3.7579798938917408, 4.1777587637296261, -4.8934897506329103, -4.1760246026883756], [0.8526852522715398, 3.1098883835186939, -4.6957244762449886, -0.085184203682487691, 4.8118675092170466], [-1.3826362676619364, -2.9729216752048373, 1.5365706808275092, -2.5547531380666744, -1.1344970339022611], [2.5767279109637826, -1.1557105392167744, -3.2102631526182335, -1.5714957027530287, 0.04449913947180395]], [[-1.1196958881896268, -1.9671289757828814, -2.2327839440659423, -4.0500056968863305, -0.68757396014469663], [-1.6808960512784021, 3.8017733014750092, -2.7859239770209152, -1.1443266889205548, 2.4749791840595945], [4.9019707367484635, -2.2501450901329245, 1.6605323152295846, -2.7837789941483857, -1.1605555712049598], [-3.4989028638804074, -4.3385506055216645, -4.7777729731482443, 2.1328136956953951, -4.499209153770285]], [[1.825959855764844, 2.5383867454469051, -1.8527832197308181, -0.13153999976669617, 0.25497458461068767], [-4.8081700281206761, 4.0127601562923907, 4.3079299227065615, 1.1484577278447006, 3.8696958684304086], [4.8621535871034567, -4.3727444593100975, 3.4803921428953117, -2.0228995754696544, -1.6214202559999933], [0.95727137474348911, -4.6200950237880427, 1.536388640365737, -0.10611731295442972, 4.3869310462781925]], [[3.7196321152136704, -3.7224963390739161, 2.2947909756817957, -4.6161229955075767, -1.9921455971565107], [-3.0912879813492622, 0.50759176021699126, -1.6259260038026522, -2.7055111865635184, -2.6315754869768804], [3.0956080658516854, -3.0571782872142759, -0.98688362631532911, -4.7621392858586518, -1.3451634527374057], [-2.1589747720303731, -3.2163074019027169, 4.8424028970283572, 0.39421068135906978, -4.0399661681902677]]], [[[3.991209266291575, 3.243684245838784, 3.9260575791131558, 0.12006514509474009, -4.8163069484624863], [4.7516934200986221, -3.601088421202947, 1.1604915575450185, 3.1769690652494731, 0.071666988408784604], [-4.5718656329226812, 3.8905294583792465, -3.0665993386681576, -1.4643349237147629, -4.0525299526960721], [-2.4310403540538381, -2.4186864269857966, 0.55026692005315869, 0.015716126267037112, -3.9802659455381271]], [[-2.8208235230136145, 0.014164903779626314, -1.4751587471057648, -0.91010367840080963, 0.81860453219456009], [4.0605817707036884, -1.7837591813324014, -2.4319240213419411, -0.48571005137515399, 4.756685420795252], [3.9277525042728527, 0.77235447414513558, -2.0590329350284922, 3.8863427820577332, -4.6166498700548697], [-2.7455069627340336, 4.2483600880800605, 0.37264518347923747, 0.46199512349997196, 4.9223281360732436]], [[-0.16896725539717128, 4.4951995829423499, 3.2724231347954778, -4.853454657186802, 3.2006623633938407], [2.5507803557904341, -0.091917834989808433, 1.5558918784774018, -2.5237862634378363, -1.1245895164237139], [2.7248765560177617, -2.8177702842042596, 1.1179541011111747, -1.0930864375539349, 0.77196965937908679], [-4.7388689409225115, -1.0056740635984172, -4.0453503599778449, -2.5021008144977817, -1.8287098871440199]], [[0.76317406542773547, -1.6705139626464813, 0.29802859004482229, 4.2814686302656089, 4.373831098618993], [-2.3769336055365962, -1.5436210869255218, -4.0763372563947264, 3.1104880855007497, -4.6385229520055464], [3.9123138932875285, 0.32963456374912159, 3.6066904338874188, -3.8630424809310293, 0.10513832146843871], [-0.44350080993032304, -2.9368527111435192, 3.5804480033378852, 4.076102489402615, 3.7783571377317191]], [[4.4101858891957111, -4.4200569823844109, 1.4850144322565768, -1.0002890673485885, -3.1024457008139605], [0.82757159458725482, -2.5071097596713141, -2.5967993033277361, 1.9077214019848281, 4.4049856055906531], [-4.5177256691144532, 1.8471100814821728, 3.8412298399943516, -4.1771232443534556, -3.6686966336071016], [3.6122660974096696, 3.831914315904644, 3.6565808527931853, 3.9438442017585107, -0.81090442848728905]]], [[[-3.362918102990605, -1.8247107050191902, -4.5593678312402321, -3.2135211313172607, -1.0063304360638812], [4.8284143182606485, -2.0421410593167835, 0.026192003870067815, -1.1001838524787755, 0.75268104188098484], [1.4860806539543949, -2.3161144065739503, -4.2483162904284733, -0.75724333200616112, -2.4851796333766774], [-3.2440800958138283, -3.60514980841244, -3.1360551911316072, -4.9663713768964177, 2.0126073031624978]], [[0.11160060753098122, 1.1752354301610515, 4.613575691949988, 3.0935102411015656, 0.84612391734395587], [2.8372877382516561, 1.7617847981115133, 4.862352020661449, 1.6390913970642043, 3.045581688875167], [1.2627256524587382, -2.0913259170442702, -0.53449394294888375, 0.4346154214998732, 1.1312902735464645], [-1.776172649242894, -0.46665362372417185, -1.5637837431498456, 3.3785411626290376, -0.67233722645761418]], [[-3.4074855131233095, -3.6764452211848155, 4.9444628397886365, 4.8085630820003207, -0.20131475340422789], [-3.5109119624986631, -1.5573392521209559, -1.2626273369462302, 4.3655919703862054, 2.8743969710401149], [-3.7356236887237593, 1.5149030693802885, -0.35609456675257078, 4.4632041202890136, 2.3238709198464313], [3.4006930701310107, 0.33349433739905798, -4.2893907420601662, -2.1998650608943748, 4.2463444294021944]], [[-3.7873640675674016, -4.6633491019884943, 1.3249924732662972, 3.0004033619899051, -2.0139483435817751], [-2.4336495418037849, 2.2937799502130005, -0.51709894104194554, 4.5365557711253519, 0.044482681049496264], [4.5240196853960484, 0.37791261392108666, 4.5500855553768424, 1.8055877682493024, -1.9490323234663975], [-3.3490435964680776, -4.6727172093172804, -1.8274948154561921, 3.923454251129801, -3.8320851559572011]], [[-3.4520529592666396, -3.197153163258355, -1.5195095015728488, 4.5033612140852775, 0.9155039288639264], [-2.0070443482784404, -0.75560358829459418, -2.6437060405420585, 1.0899979714240047, 0.095956575616091833], [-0.48004648491322399, -1.1496586927049348, 1.5127651095234089, 4.386309119929594, -0.96404188088304643], [-3.020899021615878, 4.8336806582283565, 3.0456252859464978, -2.9491650258678179, -0.42494799876889289]]], [[[-1.613542164629501, 3.670419026816198, -0.29250362652243478, -3.8766365158363483, -4.8659986606247143], [-3.1474769600734573, -3.238065023538387, 3.5006395052104917, -3.5754494612164911, 1.4660734329383107], [-2.0198828224122711, -2.7899603330581235, -4.3431181987119611, 2.2538625017994995, 4.0590949883890719], [-3.8215301584376862, 4.064167972854225, -3.155918873708643, -4.2728671130287763, 4.9889530202583678]], [[-0.74678954059538682, -0.26201609933903747, 0.49140516562766212, -4.3389046912256424, -3.9987829488451099], [-3.2341742066096058, -0.79862911283050675, -1.2752845749004935, -3.2172167217216794, 2.7359389862467616], [4.4719020611812041, 3.8611524833232043, 1.9313766877473277, -4.6579717716635729, -1.0212730535440806], [-2.8442542801060822, -2.3237342529795599, 2.7078099720083948, -1.5861379862187097, -3.9854292913737934]], [[-0.39787572444117281, 4.4055746591076588, 4.1041369764780118, 1.684807228721855, -4.3528138644355785], [-1.7165875527005516, 2.1735556099552191, -3.7020094773328802, -4.4305745654894801, 4.4015115497711115], [-1.6099472881592214, -0.84856275119407432, 2.8183127594102695, 4.532871886852659, 4.7743726599623333], [-0.97860336292763339, -3.2365999430600443, -4.9094704585964886, 1.222180003394052, -3.7080546981536644]], [[4.4429370674803685, 2.5684907011701359, -2.7340904021066228, 3.5045021650420178, -0.79312364003995306], [-1.0815115154849542, -2.863304791545902, -1.0068179999262226, 3.3357366383367726, 3.894868928952425], [1.2722085262143921, 1.3820339295409498, -0.83773835789428208, 2.5990872842800536, 2.9795213331945138], [-3.3581196203801191, 2.202568397243474, -3.7878986030156794, 4.8990715385787951, -0.76523302372348745]], [[0.28262611599202359, 2.0009380367024239, -0.64111822197429191, 2.1788147478373361, 1.3476553218449894], [-2.3774220851613883, 4.5976331686622363, 0.49069824362180992, -2.757636771143972, 3.3430132549599723], [-1.9977406146918639, -0.47276772728299932, 3.2482228922804239, 3.2119262140669704, -3.986437211562015], [-1.3639226885267162, -3.6271601128805688, 4.8511233453879417, 0.60492084861050088, 3.9138680493727858]]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[1.8292858213978178, -3.3392640136383611, 4.7796044780793263, -3.4167290111362414, 2.2977945462598797], [-3.7774098321161453, 3.6993685250856814, -1.5294858173310253, 4.9165996210229785, 2.3352007040538609], [-0.18326219446303327, -3.8040214664370597, -4.8988764262610243, 1.0488459194479791, 1.3610906286911515], [1.2525376951939418, -2.7925495890114957, -3.2130240317169978, -3.0676306276174312, -1.4130181853075738]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-1.2042889429147712, 4.8415464016543748, -0.48855955545656471, 2.4730815834795861, 2.9710202052064592], [-3.0859694228702486, 5.4232119675719055, 0.64688766108290086, -0.041338643863936486, -1.9161713551759512], [-2.1198202305349869, 0.062067273467244277, 4.7301228544584957, -2.9103074245468141, 1.5741059095388854], [4.3524087374872167, 0.070547645673510573, -2.0438167245892349, -1.5928644682788886, 5.4342453743077517]], [[-4.6324786999512586, -7.0972439075301015, 0.83849475009126495, -8.2327537642712709, -7.5152886163267372], [-2.4865787613668213, -0.22937563011966722, -8.0349884898833501, -3.4244482173208488, 1.4726034955786855], [-4.7219002813002975, -6.3121856888431989, -1.8026933328108519, -5.8940171517050359, -4.4737610475406218], [-0.7625361026745785, -4.494974552855135, -6.5495271662565946, -4.9107597163913894, -3.2947648741665572]], [[3.6599085898896995, 2.8124755022964449, 2.546820534013384, 0.72959878119299582, 4.0920305179346297], [3.0987084268009242, 8.5813777795543356, 1.9936805010584111, 3.6352777891587715, 7.2545836621389208], [9.6815752148277898, 2.5294593879464018, 6.4401367933089109, 1.9958254839309406, 3.6190489068743665], [1.2807016141989189, 0.44105387255766182, 0.0018315049310819731, 6.9124181737747215, 0.28039532430904135]], [[-1.5907691553713974, -0.87834226568933627, -5.2695122308670594, -3.5482690109029376, -3.1617544265255537], [-8.2248990392569183, 0.5960311451561493, 0.89120091157032011, -2.2682712832915408, 0.45296685729416719], [1.4454245759672153, -7.7894734704463389, 0.063663131759070346, -5.4396285866058953, -5.0381492671362347], [-2.4594576363927523, -8.0368240349242832, -1.8803403707705044, -3.5228463240906711, 0.97020203514195114]], [[6.0174266614735501, -1.4247017928140364, 4.5925855219416754, -2.318328449247697, 0.30564894910336893], [-0.79349343508938253, 2.8053863064768709, 0.67186854245722749, -0.40771664030363874, -0.33378094071700071], [5.393402612111565, -0.7593837409543962, 1.3109109199445506, -2.4643447395987721, 0.95263109352247399], [0.13881977422950653, -0.91851285564283724, 7.1401974432882369, 2.6920052276189494, -1.742171621930388]]], [[[0.21379943417542968, -0.53372558627736133, 0.14864774699701044, -3.6573446870214052, -8.5937167805786316], [0.97428358798247672, -7.3784982533190924, -2.6169182745711268, -0.60044076686667225, -3.7057428437073607], [-8.3492754650388257, 0.11311962626310113, -6.8440091707843029, -5.2417447558309078, -7.8299397848122174], [-6.2084501861699835, -6.1960962591019424, -3.2271429120629866, -3.7616937058491082, -7.7576757776542724]], [[0.87854500207206687, 3.7135334288653077, 2.2242097779799166, 2.7892648466848717, 4.5179730572802415], [7.7599502957893698, 1.91560934375328, 1.2674445037437403, 3.2136584737105274, 8.4560539458809334], [7.6271210293585341, 4.471722999230817, 1.6403355900571892, 7.5857113071434146, -0.91728134496918834], [0.95386156235164776, 7.9477286131657419, 4.0720137085649188, 4.1613636485856533, 8.621696661158925]], [[-1.6984530727281966, 2.9657137656113246, 1.7429373174644525, -6.3829404745178273, 1.6711765460628154], [1.0212945384594088, -1.6214036523208337, 0.026406061146376558, -4.0532720807688616, -2.6540753337547391], [1.1953907386867364, -4.3472561015352849, -0.41153171621985063, -2.6225722548849602, -0.7575161579519385], [-6.2683547582535368, -2.5351598809294424, -5.5748361773088702, -4.0315866318288069, -3.3581957044750452]], [[5.679773686450714, 3.2460856583764972, 5.2146282110678008, 9.1980682512885874, 9.2904307196419715], [2.5396660154863824, 3.3729785340974567, 0.8402623646282521, 8.0270877065237283, 0.27807666901743211], [8.8289135143105071, 5.2462341847721001, 8.5232900549103974, 1.0535571400919492, 5.0217379424914173], [4.4730988110926555, 1.9797469098794593, 8.4970476243608637, 8.9927021104255935, 8.6949567587546976]], [[6.745386593249572, -2.08485627833055, 3.8202151363104377, 1.3349116367052725, -0.76724499676009961], [3.1627722986411158, -0.17190905561745318, -0.26159859927387519, 4.242922106038689, 6.740186309644514], [-2.1825249650605922, 4.1823107855360337, 6.1764305440482126, -1.8419225402995947, -1.3334959295532407], [5.9474668014635306, 6.167115019958505, 5.9917815568470463, 6.2790449058123716, 1.5242962755665719]]], [[[-3.5461802974536383, -2.0079728994822235, -4.7426300257032654, -3.396783325780294, -1.1895926305269144], [4.6451521237976152, -2.2254032537798167, -0.15707019059296545, -1.2834460469418087, 0.56941884741795157], [1.3028184594913617, -2.4993766010369836, -4.4315784848915065, -0.94050552646919439, -2.6684418278397106], [-3.4273422902768615, -3.7884120028754733, -3.3193173855946405, -5.149633571359451, 1.8293451086994645]], [[-3.6924208589060785, -2.6287860362760083, 0.80955422551292822, -0.71051122533549416, -2.9578975490931039], [-0.9667337281854036, -2.0422366683255464, 1.0583305542243893, -2.1649300693728555, -0.75843977756189274], [-2.5412958139783215, -5.8953473834813295, -4.338515409385943, -3.3694060449371865, -2.6727311928905952], [-5.5801941156799533, -4.270675090161232, -5.3678052095869049, -0.42548030380802215, -4.4763586928946744]], [[-8.3063619393843346, -8.5753216474458398, 0.045586413527612279, -0.090313344260703587, -5.1001911796652522], [-8.4097883887596865, -6.4562156783819802, -6.1615037632072545, -0.53328445587481887, -2.0244794552209093], [-8.6345001149847835, -3.3839733568807357, -5.254970993013595, -0.43567230597201068, -2.5750055064145929], [-1.4981833561300135, -4.5653820888619663, -9.1882671683211896, -7.0987414871553991, -0.6525319968588299]], [[-2.7385181481194225, -3.6145031825405152, 2.3738383927142763, 4.0492492814378842, -0.96510242413379599], [-1.3848036223558058, 3.3426258696609796, 0.53174697840603358, 5.585401690573331, 1.0933286004974754], [5.5728656048440275, 1.4267585333690658, 5.5989314748248216, 2.8544336876972816, -0.90018640401841843], [-2.3001976770200985, -3.6238712898693013, -0.77864889600821297, 4.9723001705777801, -2.783239236509222]], [[-2.0909623305754881, -1.8360625345672035, -0.15841887288169731, 5.8644518427764289, 2.2765945575550779], [-0.64595371958728887, 0.60548704039655732, -1.282615411850907, 2.4510886001151562, 1.4570472043072433], [0.8810441437779275, 0.21143193598621668, 2.8738557382145604, 5.7473997486207455, 0.39704874780810506], [-1.6598083929247265, 6.194771286919508, 4.4067159146376493, -1.5880743971766664, 0.93614262992225861]]], [[[-0.36100446943555919, 4.9229567220101398, 0.96003406867150698, -2.6240988206424065, -3.6134609654307726], [-1.8949392648795156, -1.9855273283444452, 4.7531772004044335, -2.3229117660225493, 2.7186111281322525], [-0.76734512721832937, -1.5374226378641818, -3.0905805035180194, 3.5064001969934413, 5.3116326835830137], [-2.5689924632437444, 5.3167056680481668, -1.9033811785147012, -3.0203294178348346, 6.2414907154523096]], [[-3.5393391296068826, -3.0545656883505332, -2.3011444233838336, -7.1314542802371381, -6.7913325378566061], [-6.0267237956211019, -3.5911787018420025, -4.0678341639119893, -6.0097663107331751, -0.056610602764734175], [1.6793524721697084, 1.0686028943117085, -0.86117290126416801, -7.4505213606750686, -3.8138226425555763], [-5.6368038691175784, -5.1162838419910557, -0.084739617003100953, -4.3786875752302059, -6.7779788803852892]], [[-3.6108997561581706, 1.192550627390661, 0.89111294476101399, -1.5282168029951428, -7.5658378961525763], [-4.9296115844175494, -1.0394684217617787, -6.915033509049878, -7.6435985972064779, 1.1884875180541137], [-4.8229713198762187, -4.0615867829110721, -0.39471127230672831, 1.3198478551356612, 1.5613486282453355], [-4.1916273946446312, -6.4496239747770421, -8.1224944903134855, -1.9908440283229458, -6.9210787298706622]], [[1.3753064398629373, -0.49913992644729532, -5.801721029724054, 0.43687153742458662, -3.8607542676573843], [-4.1491421431023854, -5.9309354191633332, -4.0744486275436538, 0.26810601071934137, 0.82723830133499376], [-1.7954221014030392, -1.6855966980764814, -3.9053689855117133, -0.46854334333737757, -0.088109294422917372], [-6.4257502479975503, -0.86506223037395724, -6.8555292306331106, 1.8314409109613639, -3.8328636513409187]], [[-1.1303920693155503, 0.58791985139485003, -2.0541364072818658, 0.76579656252976225, -0.065362863462584464], [-3.7904402704689621, 3.1846149833546624, -0.92231994168576392, -4.1706549564515463, 1.9299950696523984], [-3.4107587999994378, -1.8857859125905732, 1.8352047069728501, 1.7989080287593966, -5.3994553968695893], [-2.77694087383429, -5.0401782981881427, 3.4381051600803678, -0.80809733669707295, 2.5008498640652119]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[3.546023137463834, -0.28819011724893784], [0.50440030141136916, 1.56592788105017]], [[0.28907736646878934, 1.1812929233566152], [2.6567811653020312, 3.201214118174434]], [[4.7782589989546427, -2.9084482403234557], [1.8397152161826948, 1.3934983698603816]], [[4.5535156364700935, -3.2203074913131791], [-3.4298375920393953, 3.6142467977297219]], [[-2.656033983042152, 3.0471334947960429], [-2.6434952014604263, 1.2687130852671755]], [[2.6356837935464883, -2.1003108178589427], [0.43495766345519637, 4.3373448218733266]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[-4.302497631769584, 2.4054836275654541], [1.4911347956042622, -1.54345301173475]], [[-1.9494690208110343, 3.2314892259175494], [2.3783410987070681, 2.2440563456062428]], [[-0.73502661564916671, -1.6429511811443556], [-0.9016998968404355, 1.9649742795845153]], [[-0.91818217200338648, 3.8010687892957584], [-3.079881921767643, -3.6205165616102652]], [[0.93537013833096694, 3.9308864977773723], [-3.1688406217857534, 0.80582902909685483]], [[-1.6205166782042557, -0.37422191379826764], [-2.1982982049471325, -0.52558913701884435]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.75647449430574998, 2.1172935103165162], [1.9955350970156314, 0.022474869315419976]], [[-1.660391654342245, 4.4127821492741646], [5.0351222640090993, 5.4452704637806768]], [[4.043232383305476, -4.5513994214678117], [0.93801531934225935, 3.3584726494448969]], [[3.635333464466707, 0.58076129798257936], [-6.5097195138070383, -0.0062697638805433442]], [[-1.7206638447111851, 6.9780199925734152], [-5.8123358232461797, 2.0745421143640304]], [[1.0151671153422326, -2.4745327316572103], [-1.7633405414919361, 3.8117556848544822]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Data(numpy.array([[[0.77176425354461564, 0.31114106584057222], [-4.0647073292686029, 4.4460235554416947]], [[4.3348212177975878, 3.580598829340131], [-3.2768215316887526, 2.0889384415596517]], [[-4.2928740013925495, -0.69862357910675144], [-0.35291339758446227, -4.6837480693087361]], [[-2.8443461907431553, 2.8622751035862066], [3.1488758424988497, -1.6962066931382025]], [[4.0207145002644662, 0.91517819701933689], [1.7498010712721523, 0.87797503496970819]], [[3.5489348649360721, 4.2874821233249421], [2.312581219574497, 2.0160865155767578]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[[-0.81926212439499135, 1.7737264863326576], [-2.1080769297827109, -2.5845973387732855]], [[1.4067973991002427, 3.1991066559937416], [-4.8708705170990241, -4.8851450943493653]]], [[[-4.3306215208629641, 1.621106272469131], [-4.8631214725428489, -3.5739740612523385]], [[2.7609903489872778, 1.461115349045067], [-0.16895298669492664, 3.6692287567970325]]], [[[-4.7778715214744292, -0.88390195505149549], [2.5678392355505677, -2.4022591286878026]], [[3.5647245922262147, -2.993687557778435], [-1.9028081416360809, 0.80965847111297329]]], [[[-3.7749426478074466, -1.4406649140977601], [2.3628770848971117, 2.5335743566208082]], [[3.6014224395147281, 2.0280226750065253], [-0.67796735993369328, -3.7057240035353702]]], [[[-2.5469714142633135, -2.3226473893783108], [4.0223047366705682, 1.8546481858485633]], [[-4.1748880194897344, 1.0275260864174438], [4.8232563686313892, -3.4330241359286138]]], [[[-3.9359944758567886, 4.8513101424850582], [3.3239220351790806, 2.7097663194252233]], [[2.2172263004893775, 0.88638771352151124], [-2.4685098633883609, -2.8222385122976537]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.047497870850375712, 2.5454907398772733], [-1.7969358639421387, -2.2734562729327132]], [[-2.6579099301683602, -0.86560067327486134], [-0.42484696165732938, -0.43912153890767058]]], [[[0.0041996969346236313, 5.9559274902667187], [-1.2825226432027179, 0.0066247680877924431]], [[-0.51583118270147477, -1.8157061826436856], [1.919985454864725, 5.7581671983566842]]], [[[-9.0707455228669787, -5.176775956444045], [1.8692156564438163, -3.100882707794554]], [[3.2118111946417525, -3.3466009553628973], [-6.586556210944817, -3.8740895981957628]]], [[[-6.6192888385506023, -4.2850111048409154], [5.2251521884833183, 5.3958494602070148]], [[6.7502982820135777, 5.176898517505375], [-2.3741740530718958, -5.4019306966735723]]], [[[1.4737430860011527, 1.6980671108861554], [4.9374829336899051, 2.7698263828679002]], [[-2.4250869482175821, 2.7773271576895961], [5.7012314036010974, -2.5550491009589056]]], [[[-0.3870596109207165, 8.4002450074211303], [7.6114041585040226, 6.9972484427501653]], [[4.5298075200638745, 3.1989689330960083], [-0.45242334781160309, -0.80615199672089588]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_constData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[[1.0851853807755401, -0.32751024053335343], [1.6935401505089942, 4.4714057540816086]], [[4.9771747093506953, -3.3616179895351284], [-2.0745038041132648, -4.9285968491005541]]], [[[-2.5435840235505438, 3.0253106537238903], [3.5108778263119831, -1.9993621529923908]], [[3.521788192724582, 3.0695015896535356], [3.673782501353509, -0.96025401301818025]]], [[[0.93201998709941947, 2.2244529587411233], [0.6720886265403605, 1.0747983151224734]], [[3.4000146434596594, -1.4134618844747573], [1.8925472259199028, -1.210828245151041]]], [[[-0.16502691171010575, -3.2302356856941872], [-3.8048581695779116, 2.3420556938520862]], [[-2.5512339922447334, 0.071849277610519735], [-1.9308532275636936, 1.2501176368322087]]], [[[0.14499543784901192, -3.3616642075844871], [-4.3826532273040115, -0.59167792594836044]], [[-0.56786125253896369, 4.520355568482973], [-0.4520536595568112, 0.46283202617155705]]], [[[-1.9488525424351932, -0.75396754450314418], [2.3823104353470193, 0.86727718682187405]], [[2.0804093613291048, 1.7697087001772003], [4.8902452758784758, 2.5782061137354448]]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[0.7038708045326123, 3.3683251027633716], [0.80164468253782317, -3.3758066817458876]], [[-3.0829966173240217, -3.8492183113496714], [-0.44158191379106171, -4.7947413959503677]], [[3.0518627148669726, -0.091289713285066298], [-0.89721550683136009, -3.2560742331256218]], [[-4.4159762835841736, 1.8421095723717551], [4.3583047925037466, 0.69945964422785245]], [[1.9407601208896876, -3.9003758406722175], [4.6491455089714844, -4.7360812531040253]], [[-0.93446468735001531, 1.5624452064615335], [4.1477886421408154, -3.3146578214862874]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.7890561853081524, 0.37636056399925888], [5.0618652532723658, 7.8397308568449802]], [[5.7788193918885185, -2.5599733069973052], [-5.450310485859152, -8.3044035308464412]]], [[[-5.6265806408745656, -0.057685963600131451], [-0.33834048503768832, -5.8485804643420618]], [[3.0802062789335203, 2.6279196758624739], [-1.1209588945968587, -5.754995408968548]]], [[[3.983882701966392, 5.2763156736080958], [0.5807989132552942, 0.98350860183740707]], [[2.5027991366282993, -2.3106773913061174], [-1.3635270072057191, -4.4669024782766629]]], [[[-4.5810031952942794, -7.6462119692783608], [-1.9627485972061565, 4.1841652662238413]], [[1.8070708002590132, 4.4301540701142663], [-1.2313935833358411, 1.9495772810600611]]], [[[2.0857555587386996, -1.4209040866947995], [-8.28302906797623, -4.492053766620578]], [[4.0812842564325207, 9.1695010774544574], [-5.1881349126608365, -4.2732492269324682]]], [[[-2.8833172297852085, -1.6884322318531595], [3.9447556418085528, 2.4297223932834076]], [[6.2281980034699203, 5.9174973423180157], [1.5755874543921884, -0.73645170775084257]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[2.8959410464851132, -0.20565820462951301, 1.1037797597401324, -2.2605405134287135], [3.1689012085575232, 3.6936496387017002, -0.56018282867377778, 0.68952458091914259], [3.275343846819732, -4.9455157569166719, -3.9919983739406328, 4.3688736778928554]], [[-0.75856428162690115, 2.7003677028856456, 1.2064070327922272, -3.0838728151517305], [2.6085222086195312, -2.0012681509935781, 2.7623341170964419, 0.18522144397424434], [-4.6369547771944211, -4.2429363313130599, -2.781537830164682, 2.9196528375843798]]], [[[2.3716906602585404, 0.87579676486911584, 4.7311382782596354, 2.08651831077532], [1.4104776047587944, 4.2038174956621326, -2.8357735732168288, 4.9974617154186483], [-4.7663544734324415, 4.8850038170521088, 1.28295904476873, 2.8271135967882079]], [[-0.60649110101563863, 3.6391692210000102, 3.4844731181939856, -3.1302199286133039], [-1.5456120298393849, -1.2450725645216432, 2.5640694922286844, -1.1433416107444794], [0.51758862527888638, -2.3894387075784218, 1.5297078365834818, -4.5376254610027411]]], [[[-1.9537516054461457, -3.9713637818566339, -4.5399070037380014, 2.9707988149685969], [-3.0209983670491845, 1.0764619624153777, -2.5234682996724747, 4.9830089167157894], [3.8739800741627946, 4.2122967939635476, -1.1015252513712595, -3.9446963880923693]], [[3.6866680409707016, 4.9196397917391153, 4.505740559038319, 4.9752914231820942], [-0.52645635851341854, -0.20990035194351897, -4.6818262620342761, 0.018799546990996063], [-1.9407039936621273, 0.5130726179827052, 2.0748323016477324, 3.5694561068179063]]]]),self.functionspace)
      res=add(arg0,arg1)
      s0=numpy.array([[[[3.9161613160962965, 1.3411789816744646, 4.7467374610503832, 3.4856093522697034], [-0.90029763563891407, -0.74375362805016998, 4.7411299070156794, 1.0360484407889414], [2.3737978747431887, -2.9448603778285243, -0.23958733294708523, 4.543281600575547]], [[-4.385780946824017, 0.77761241011268289, -3.1087887204817468, -0.50717507265468864], [2.3938508222782895, 2.8590382990176977, -0.54597038008131449, 1.3427977813298071], [-3.5664380788731718, 3.1693179828912292, 1.3264802639219342, 2.742633574665744]]], [[[-1.2930938961133922, -3.8748057102803193, 1.9173387265180084, -4.1325337133082618], [2.7258632987858702, -4.4956479481458933, 2.4655366163588788, -0.48902866933520439], [-1.8335499490575122, -2.264871733931356, -0.64878974610293305, 0.019200032613530205]], [[0.85495012062533249, 1.5393476210748167, -2.742332825998739, 3.8205262605328887], [2.0949625236952416, 1.2470873068504691, 0.74324609429317601, -2.3844792704855533], [2.9996640673156012, -2.7942813143950449, -3.4616883641716409, -1.5757116138838945]]], [[[0.54454435226576337, 4.3225096216506778, 2.5335673609839526, 4.6622773038651495], [4.2036074567898858, -4.7407167487952639, 1.3381725220679801, 3.1400598938220483], [3.6256104114444199, -3.8465387907822346, -0.57023201725471928, 2.13064173037594]], [[2.0442030056457892, -0.95823249586061277, 0.61234954990050472, 1.4205368213099341], [0.57342026872562535, -3.6583370408114115, 4.3623866495452805, -3.5957931995459491], [2.0343814837180503, -3.8387076454084901, -0.12344936310518051, -0.18193713210999807]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[6.8121023625814097, 1.1355207770449516, 5.8505172207905156, 1.2250688388409898], [2.2686035729186091, 2.9498960106515302, 4.1809470783419016, 1.725573021708084], [5.6491417215629207, -7.8903761347451962, -4.2315857068877181, 8.9121552784684024]], [[-5.1443452284509181, 3.4779801129983285, -1.9023816876895197, -3.5910478878064191], [5.0023730308978207, 0.85777014802411955, 2.2163637370151275, 1.5280192253040514], [-8.2033928560675928, -1.0736183484218307, -1.4550575662427478, 5.6622864122501237]]], [[[1.0785967641451482, -2.9990089454112034, 6.6484770047776438, -2.0460154025329418], [4.1363409035446645, -0.29183045248376072, -0.37023695685794999, 4.5084330460834439], [-6.5999044224899537, 2.6201320831207529, 0.63416929866579697, 2.8463136294017382]], [[0.24845901960969385, 5.1785168420748269, 0.74214029219524669, 0.69030633191958479], [0.54935049385585666, 0.0020147423288259247, 3.3073155865218604, -3.5278208812300327], [3.5172526925944876, -5.1837200219734667, -1.9319805275881592, -6.1133370748866351]]], [[[-1.4092072531803823, 0.3511458397940439, -2.0063396427540487, 7.6330761188337464], [1.1826090897407013, -3.6642547863798862, -1.1852957776044946, 8.1230688105378377], [7.4995904856072144, 0.36575800318131302, -1.6717572686259787, -1.8140546577164294]], [[5.7308710466164907, 3.9614072958785025, 5.1180901089388238, 6.3958282444920282], [0.046963910212206805, -3.8682373927549305, -0.31943961248899555, -3.5769936525549531], [0.093677490055922963, -3.3256350274257849, 1.9513829385425518, 3.3875189747079082]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.92346975691)+(1.-msk_arg1)*(-3.90151291361)
      res=add(arg0,arg1)
      s0=numpy.array(1.17350797629)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.74996178062)+(1.-msk_ref)*(-2.72800493732)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.25030882048)+(1.-msk_arg1)*(0.311529606724)
      res=add(arg0,arg1)
      s0=numpy.array([3.3432712767576334, 4.6341939123556752])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([7.5935800972421887, 8.8845027328402306])+(1.-msk_ref)*numpy.array([3.6548008834821166, 4.9457235190801585])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.6479877904747555, -3.5899273987876343])+(1.-msk_arg1)*numpy.array([0.4579809843760998, 0.65350437337978562])
      res=add(arg0,arg1)
      s0=numpy.array(2.64698485113)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-2.0010029393476989, -0.94294254766057772])+(1.-msk_ref)*numpy.array([3.1049658355031564, 3.3004892245068422])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.39021841516)+(1.-msk_arg1)*(3.38999267093)
      res=add(arg0,arg1)
      s0=numpy.array([[3.3511457419938608, 0.65416310244409814, 2.6426236413464377, -3.238924603363369, -1.7314667481645465], [0.70679556585204306, 3.7233423042596598, -1.0663566345366773, 3.6375408966550786, -4.029627169721409], [-1.9763063411548076, 3.9577465115657056, -4.4895139525325103, -2.973902960235788, 1.1634448880894466], [3.261220824667463, 1.3930723277641306, 4.0867530149322828, -0.20033088573692126, -4.3478104679100262]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.7413641571551715, 5.0443815176054088, 7.0328420565077483, 1.1512938117979417, 2.6587516669967641], [5.0970139810133537, 8.1135607194209705, 3.3238617806246333, 8.0277593118163892, 0.36059124543990162], [2.413912074006503, 8.3479649267270162, -0.099295537371199671, 1.4163154549255226, 5.5536633032507572], [7.6514392398287736, 5.7832907429254412, 8.4769714300935934, 4.1898875294243894, 0.042407947251284384]])+(1.-msk_ref)*numpy.array([[6.7411384129281124, 4.0441557733783497, 6.0326163122806893, 0.15106806757088265, 1.6585259227697051], [4.0967882367862947, 7.1133349751939114, 2.3236360363975743, 7.0275335675893302, -0.63963449878715739], [1.413686329779444, 7.3477391824999572, -1.0995212815982587, 0.41608971069846357, 4.5534375590236982], [6.6512134956017146, 4.7830649986983822, 7.4767456858665344, 3.1896617851973303, -0.95781779697577463]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-2.8452254516244704, 4.7873497014112871, -2.4890959882126475, -4.6871006962697273, 2.7352491703548516], [-1.624392541907306, -4.5444943017899657, -3.7740057418860906, -2.098253428836597, -2.4648353739788975], [-0.83603190868421695, 1.4760987690502461, -2.3810917797638362, 4.6483468240981303, -2.0208457647801459], [2.7196556994587908, -3.470525090633485, -2.8924781159380411, 0.78207159911216451, 2.2927170353686641]])+(1.-msk_arg1)*numpy.array([[3.4245345336194237, 0.31184843028983167, -4.9796903621467026, -3.3866067367295463, 4.5547755935655765], [-4.4554328208074159, 1.9967764304272562, 4.9530972993945639, -3.6848741626114445, -4.5215796937161201], [2.2719611718380603, 1.5647785442887541, -4.5860927701040355, -4.8956032138544714, -2.937721278294021], [-4.0547283048183935, -0.44574347689161975, -4.4523680254142288, 1.9976055718206549, 4.0088537505821087]])
      res=add(arg0,arg1)
      s0=numpy.array(1.77584098731)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.0693844643132309, 6.5631906887225266, -0.71325500090140803, -2.9112597089584877, 4.5110901576660911], [0.15144844540393354, -2.7686533144787262, -1.9981647545748511, -0.32241244152535753, -0.688994386667658], [0.93980907862702256, 3.2519397563614856, -0.60525079245259672, 6.4241878114093698, -0.24500477746890637], [4.4954966867700303, -1.6946841033222455, -1.1166371286268015, 2.557912586423404, 4.0685580226799036]])+(1.-msk_ref)*numpy.array([[5.2003755209306632, 2.0876894176010712, -3.2038493748354631, -1.6107657494183067, 6.330616580876816], [-2.6795918334961764, 3.7726174177384957, 6.7289382867058034, -1.909033175300205, -2.7457387064048806], [4.0478021591492999, 3.3406195315999936, -2.810251782792796, -3.1197622265432319, -1.1618802909827814], [-2.278887317507154, 1.3300975104196198, -2.6765270381029893, 3.7734465591318944, 5.7846947378933482]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.05305367703)+(1.-msk_arg1)*(-1.33637613304)
      res=add(arg0,arg1)
      s0=numpy.array([[[3.8287449035147425, 4.2085288326313854], [4.4773352901024062, -1.8272633415088944]], [[2.8848803960453049, 4.2977066779363824], [-2.4365304238886631, 3.9025516417336235]], [[0.47830637642162621, 2.2638221035678452], [-4.6517967898137691, -4.2045300012699727]], [[3.6069394734522433, 3.6317579928709058], [4.0477757344028333, -3.2888757804715985]], [[3.2986559095542063, -3.8611822859799472], [4.9167295641310176, 3.2533940707331404]], [[4.6382675606492558, 4.0766062972173032], [-4.8986746030976747, -0.502862166898006]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.8817985805442188, 5.2615825096608617], [5.5303889671318824, -0.77420966447941808]], [[3.9379340730747812, 5.3507603549658587], [-1.3834767468591869, 4.9556053187630997]], [[1.5313600534511025, 3.3168757805973215], [-3.5987431127842928, -3.1514763242404964]], [[4.6599931504817196, 4.6848116699003821], [5.1008294114323096, -2.2358221034421222]], [[4.3517095865836826, -2.8081286089504709], [5.9697832411604939, 4.3064477477626166]], [[5.6913212376787321, 5.1296599742467794], [-3.8456209260681984, 0.55019151013147027]]])+(1.-msk_ref)*numpy.array([[[2.4923687704710282, 2.8721526995876712], [3.1409591570586919, -3.1636394745526086]], [[1.5485042630015906, 2.9613305448926681], [-3.7729065569323774, 2.5661755086899092]], [[-0.85806975662208806, 0.92744597052413091], [-5.9881729228574834, -5.5409061343136869]], [[2.270563340408529, 2.2953818598271916], [2.7113996013591191, -4.6252519135153127]], [[1.962279776510492, -5.197558419023661], [3.5803534310873033, 1.9170179376894261]], [[3.3018914276055416, 2.7402301641735889], [-6.2350507361413889, -1.8392382999417203]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.0026007235933294, -4.4580312555128279], [-1.7323910127236508, 3.8918844463894935]], [[1.9174057252940093, 3.6744411045532832], [-3.9481000025363588, 3.4720414051391568]], [[-2.6758784303436034, 1.1581865507268834], [-4.7564076361937468, -2.689438229730936]], [[-1.5730137130645119, 2.3426626473157519], [-2.2930200803096081, -1.2237739758704702]], [[-1.0267719839742107, 0.90894654353180915], [-3.7178814696241735, -0.90800464254416013]], [[-4.8636623713056695, 0.1424495577300311], [-2.332592924392344, 3.4502560720709301]]])+(1.-msk_arg1)*numpy.array([[[1.9029326782260885, 1.3186902970292724], [3.7891303593772747, 1.4462006637692015]], [[0.64650906469293012, 0.55828760799143762], [2.4728702682467478, -0.28687549679942315]], [[-2.8497455276502235, -3.2601152744300377], [-4.1226669048482991, -4.0521518180555738]], [[1.1070511712085676, 4.6787174831414209], [-3.0232024773262478, -0.15253698094547552]], [[-3.1843703737096405, 0.16541317554394563], [-1.9916708271040218, -0.063072192572219521]], [[1.8330241566841412, 0.65802486230576385], [-1.9230278839732651, 2.875418297636755]]])
      res=add(arg0,arg1)
      s0=numpy.array(-0.00775181951993)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.9948489040733959, -4.4657830750327614], [-1.7401428322435843, 3.88413262686956]], [[1.9096539057740758, 3.6666892850333497], [-3.9558518220562924, 3.4642895856192233]], [[-2.6836302498635369, 1.1504347312069498], [-4.7641594557136804, -2.6971900492508696]], [[-1.5807655325844454, 2.3349108277958184], [-2.3007718998295417, -1.2315257953904037]], [[-1.0345238034941442, 0.90119472401187561], [-3.7256332891441071, -0.91575646206409367]], [[-4.871414190825603, 0.13469773821009756], [-2.3403447439122775, 3.4425042525509966]]])+(1.-msk_ref)*numpy.array([[[1.8951808587061549, 1.3109384775093389], [3.7813785398573412, 1.438448844249268]], [[0.63875724517299659, 0.55053578847150408], [2.4651184487268143, -0.29462731631935668]], [[-2.857497347170157, -3.2678670939499712], [-4.1304187243682327, -4.0599036375755073]], [[1.099299351688634, 4.6709656636214874], [-3.0309542968461813, -0.16028880046540905]], [[-3.192122193229574, 0.1576613560240121], [-1.9994226466239553, -0.070824012092153055]], [[1.8252723371642077, 0.65027304278583031], [-1.9307797034931986, 2.8676664781168215]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.884145355807)+(1.-msk_arg1)*(-0.322271599354)
      res=add(arg0,arg1)
      s0=numpy.array([[[[-3.7532445504955625, -0.47834282115475002, -1.2567945665271782, 4.8283776209344467], [-4.2864933454781928, 0.5838357509138179, -4.6874458114494324, -4.0814568888079181], [-4.599795771685625, -0.38195588500160138, 3.7716723115000672, 4.3296373167339848]], [[4.4936476606999705, 1.1517679965710128, -3.0296084226331277, -2.5644151313901276], [4.2232064359445332, -2.065697299425179, -4.6031450955482569, 1.2507503463850256], [-3.8698704079001276, 2.3043703352783442, 0.99421739702083212, 0.54897477279465612]]], [[[-3.1838813705622435, 2.6014649807193795, -2.1104046980776161, -4.7433338597709902], [-3.5943784965961023, 3.5430858776019143, -0.55127395486126218, 4.2940968209681358], [3.3355461926675574, 3.1685307908870168, -0.78838287751587632, 0.03746098861500613]], [[4.8894022719629824, 3.5677927050143587, -4.0469332064278962, -2.7041086176104656], [4.4720959065710524, -1.4965424183675955, 1.2391170038596497, 0.022443789799920566], [1.6560364863347674, 2.4721633842604369, -4.2593792265825394, 0.089881134759090209]]], [[[4.6903936681075891, 4.6651115997122066, -3.9005857394050603, -0.21377905943927278], [-2.5084297157299673, 2.8526812784942397, -2.0119297946511816, -3.2596810343149505], [-0.023893254160846311, 2.0906448299663536, -4.2836428907245097, 2.7041043254174832]], [[0.64457510758393877, 4.580726426780231, -2.982540805132686, 2.5682667082065809], [-0.30392156281977023, 2.2235843865093425, -0.8393489299516208, 1.3063611712960768], [4.751526864986376, 3.8588344432129507, 4.042473059530451, -4.4270385542578801]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-4.6373899063025483, -1.3624881769617359, -2.1409399223341641, 3.9442322651274608], [-5.1706387012851787, -0.30030960489316794, -5.5715911672564182, -4.965602244614904], [-5.4839411274926109, -1.2661012408085872, 2.8875269556930814, 3.4454919609269989]], [[3.6095023048929846, 0.26762264076402698, -3.9137537784401135, -3.4485604871971134], [3.3390610801375473, -2.9498426552321648, -5.4872904513552427, 0.36660499057803975], [-4.7540157637071134, 1.4202249794713584, 0.11007204121384628, -0.33517058301232971]]], [[[-4.0680267263692294, 1.7173196249123936, -2.9945500538846019, -5.627479215577976], [-4.4785238524030877, 2.6589405217949285, -1.435419310668248, 3.40995146516115], [2.4514008368605715, 2.284385435080031, -1.6725282333228622, -0.8466843671919797]], [[4.0052569161559965, 2.6836473492073729, -4.9310785622348821, -3.5882539734174514], [3.5879505507640665, -2.3806877741745813, 0.3549716480526639, -0.86170156600706527], [0.77189113052778158, 1.5880180284534511, -5.1435245823895253, -0.79426422104789562]]], [[[3.8062483123006032, 3.7809662439052207, -4.7847310952120461, -1.0979244152462586], [-3.3925750715369531, 1.9685359226872539, -2.8960751504581674, -4.1438263901219363], [-0.90803860996783214, 1.2064994741593678, -5.1677882465314955, 1.8199589696104974]], [[-0.23957024822304707, 3.6965810709732452, -3.8666861609396719, 1.684121352399595], [-1.1880669186267561, 1.3394390307023567, -1.7234942857586066, 0.42221581548909093], [3.8673815091793902, 2.9746890874059648, 3.1583277037234652, -5.3111839100648659]]]])+(1.-msk_ref)*numpy.array([[[[-4.0755161498490651, -0.8006144205082526, -1.5790661658806808, 4.5061060215809441], [-4.6087649448316954, 0.26156415156031532, -5.009717410802935, -4.4037284881614207], [-4.9220673710391276, -0.70422748435510396, 3.4494007121465646, 4.0073657173804822]], [[4.1713760613464679, 0.82949639721751023, -3.3518800219866303, -2.8866867307436301], [3.9009348365910306, -2.3879688987786816, -4.9254166949017595, 0.928478747031523], [-4.1921420072536302, 1.9820987359248416, 0.67194579766732954, 0.22670317344115354]]], [[[-3.5061529699157461, 2.2791933813658769, -2.4326762974311187, -5.0656054591244928], [-3.9166500959496049, 3.2208142782484117, -0.87354555421476476, 3.9718252216146333], [3.0132745933140548, 2.8462591915335143, -1.1106544768693789, -0.28481061073849645]], [[4.5671306726094798, 3.2455211056608562, -4.3692048057813988, -3.0263802169639682], [4.1498243072175498, -1.8188140177210981, 0.91684540450614715, -0.29982780955358201], [1.3337648869812648, 2.1498917849069343, -4.581650825936042, -0.23239046459441237]]], [[[4.3681220687540865, 4.342840000358704, -4.2228573387585628, -0.53605065879277536], [-2.8307013150834699, 2.5304096791407371, -2.3342013940046842, -3.5819526336684531], [-0.34616485351434889, 1.768373230612851, -4.6059144900780122, 2.3818327260639807]], [[0.32230350823043619, 4.2584548274267284, -3.3048124044861886, 2.2459951088530783], [-0.62619316217327281, 1.90131278715584, -1.1616205293051234, 0.98408957194257418], [4.4292552656328734, 3.5365628438594481, 3.7202014601769484, -4.7493101536113826]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.9547807205374585, -1.8720561139970826, -1.434292186799687, -3.5895475094279572], [2.9609044626356784, -3.8042536096991517, 1.781407615217498, -3.0752244099036217], [1.4806052590244771, 2.8899251284457526, 0.5776693843423093, 0.019957368893413374]], [[-1.8781427225785094, -0.36024911212249933, -4.6559086125149332, 4.5399580619704825], [1.967946226067645, 3.6958105764984168, -3.5292482276686412, 2.9974079179984603], [-1.4870532590009122, -2.3392873219520838, -2.2634924623468065, -0.94099037374191496]]], [[[-3.2390242929475788, 2.8772950456666333, 2.3547493554117915, -4.1682630475448743], [4.2828341815415278, -3.2446509485033301, 0.59863289282752596, -3.7758866294584861], [-0.73745867221253558, 1.2671804353546587, 4.9800972946708697, 1.3188308076956172]], [[-2.4445891193691169, 4.2137947766030042, -2.5122490813087186, -2.4859071667483388], [-2.9840827130149306, 2.5105068119249161, -0.41253297651502674, 2.8965062934725037], [-3.7473216035957568, -4.8840666742948962, 2.9890661627496451, 0.7805024485781189]]], [[[1.0967212932948653, -4.6665461986489714, 2.9820159789261202, 3.5590428087761001], [1.0822741478018845, -0.80882508931959407, -2.8443400528645038, 3.7983513993264157], [-3.6395590674075895, -1.2762818819282087, 3.7119372152665271, -1.2594364664825672]], [[1.4581534264268452, -2.4856289962857381, 3.7054951054990557, 3.6006716370655294], [-0.51701228634143348, 2.3524332855778232, -2.5568018280217633, -0.91291442205921136], [1.9733094646018774, 2.915818302147331, 4.9855762086810156, 4.8865648707222711]]]])+(1.-msk_arg1)*numpy.array([[[[-0.14720735152531272, 4.1139348887915954, 0.53274713552119124, 2.1629883607959099], [-3.5980409108946034, 4.2487920014117204, 4.9033834980032616, 1.9498452874532113], [0.69234261498243477, -2.1024066679793671, -3.6376271273096439, 4.9572067789876364]], [[-2.1366450134726591, 2.0289782168293735, 2.0032571779877788, -0.74039541054980162], [4.8614776505182036, -4.8193995169803463, -2.8270940922660639, -0.044288334626969572], [0.62837568074234795, 3.6003100807745803, 1.0058331956203848, 1.6198579274526281]]], [[[3.5866404281203987, -0.35417630248494891, 3.215124735599403, 4.8430704678129519], [2.8978214651201704, -2.2532510474428538, 0.26098682474342283, 1.2233815096525174], [-1.8730019363457351, -4.5552500542238423, -3.498681799572485, 3.2182113427398242]], [[-4.6624874153115661, -2.7825595961222138, -0.87131650493604162, -0.12925242625530586], [-4.2844557688456462, -1.3409116735640261, -3.1458296691586796, -0.39818657764637244], [-2.7770149112199141, 4.7359989003124632, 2.6110353342920263, 0.65181206104183875]]], [[[-0.55102812341755936, -1.9746281953794775, -0.0131379766791202, -3.1873767352853042], [2.0697576200007362, -0.88126066756562071, 3.2967671444182596, 4.0780271452434018], [1.6076110841894033, 1.7349996474255294, -4.2283143483257994, -0.38967102853492896]], [[2.5098905255484629, -1.0630009642124207, 3.3996620915975111, 1.808615702000357], [0.019661243457003152, -0.15397735665778622, -4.1621799134419932, 3.0456217961499839], [3.798205866971669, 3.4369903055546054, 4.5418101090748735, 0.64415158338180678]]]])
      res=add(arg0,arg1)
      s0=numpy.array(-3.77956888622)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.1752118343193998, -5.6516250002151409, -5.2138610730177462, -7.3691163956460155], [-0.81866442358238034, -7.5838224959172109, -1.9981612710005607, -6.8547932961216809], [-2.2989636271935816, -0.8896437577723062, -3.2018995018757495, -3.7596115173246454]], [[-5.6577116087965678, -4.1398179983405576, -8.4354774987329915, 0.76038917575242371], [-1.8116226601504137, -0.083758309719641932, -7.3088171138867004, -0.78216096821959846], [-5.266622145218971, -6.1188562081701425, -6.0430613485648657, -4.7205592599599733]]], [[[-7.018593179165638, -0.90227384055142545, -1.4248195308062672, -7.9478319337629326], [0.50326529532346909, -7.0242198347213893, -3.1809359933905328, -7.5554555156765453], [-4.5170275584305948, -2.5123884508634, 1.2005284084528109, -2.4607380785224415]], [[-6.2241580055871761, 0.43422589038494541, -6.2918179675267769, -6.2654760529663971], [-6.7636515992329898, -1.2690620742931427, -4.1921018627330859, -0.8830625927455551], [-7.526890489813816, -8.6636355605129545, -0.79050272346841366, -2.9990664376399399]]], [[[-2.6828475929231934, -8.4461150848670297, -0.79755290729193851, -0.22052607744195862], [-2.6972947384161743, -4.5883939755376524, -6.623908939082563, 0.018782513108356991], [-7.4191279536256483, -5.0558507681462679, -0.067631670951531664, -5.0390053527006256]], [[-2.3214154597912136, -6.2651978825037968, -0.074073780719003057, -0.17889724915252936], [-4.2965811725594918, -1.4271356006402356, -6.3363707142398216, -4.6924833082772697], [-1.8062594216161814, -0.8637505840707278, 1.2060073224629568, 1.1069959845042123]]]])+(1.-msk_ref)*numpy.array([[[[-3.9267762377433715, 0.33436600257353666, -3.2468217506968675, -1.6165805254221488], [-7.3776097971126617, 0.46922311519366167, 1.1238146117852028, -1.8297235987648475], [-3.087226271235624, -5.8819755541974263, -7.4171960135277022, 1.1776378927695776]], [[-5.9162138996907174, -1.7505906693886852, -1.77631170823028, -4.5199642967678599], [1.0819087643001448, -8.5989684031984055, -6.6066629784841222, -3.8238572208450283], [-3.1511932054757108, -0.1792588054434785, -2.773735690597674, -2.1597109587654306]]], [[[-0.19292845809766002, -4.1337451887030081, -0.56444415061865572, 1.0635015815948932], [-0.8817474210978884, -6.0328199336609121, -3.5185820614746359, -2.5561873765655414], [-5.6525708225637938, -8.3348189404419006, -7.2782506857905442, -0.56135754347823452]], [[-8.4420563015296253, -6.5621284823402721, -4.6508853911540999, -3.9088213124733646], [-8.0640246550637045, -5.1204805597820844, -6.9253985553767379, -4.1777554638644308], [-6.5565837974379733, 0.95643001409440442, -1.1685335519260325, -3.12775682517622]]], [[[-4.3305970096356177, -5.7541970815975363, -3.792706862897179, -6.9669456215033634], [-1.7098112662173226, -4.6608295537836799, -0.48280174179979918, 0.29845825902534306], [-2.1719578020286554, -2.0445692387925294, -8.0078832345438578, -4.1692399147529873]], [[-1.2696783606695958, -4.8425698504304791, -0.37990679462054766, -1.9709531842177017], [-3.7599076427610556, -3.933546242875845, -7.9417487996600524, -0.7339470900680749], [0.018636980753610199, -0.34257858066345337, 0.76224122285681473, -3.135417302836252]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.7275968656232266, -1.455015702554916])+(1.-msk_arg1)*numpy.array([-2.5574288867951811, -2.5428103405955493])
      res=add(arg0,arg1)
      s0=numpy.array([-1.7135910141411257, 3.1723505550645239])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.0140058514821009, 1.7173348525096079])+(1.-msk_ref)*numpy.array([-4.2710199009363068, 0.62954021446897457])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.6559931410324733, 3.6395058228607589])+(1.-msk_arg1)*numpy.array([1.4818436623888562, -4.5347900863017996])
      res=add(arg0,arg1)
      s0=numpy.array([[-4.6216049101809489, -3.95682905133493], [-0.16564971613915169, 0.6782074823653641]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.9656117691484756, -2.3008359103024567], [3.4738561067216072, 4.317713305226123]])+(1.-msk_ref)*numpy.array([[-3.1397612477920926, -2.4749853889460738], [-4.7004398024409513, -3.8565826039364355]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_expandedData_rank2(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.2440503325406569, 0.78585302112773459], [-1.5925146355421771, -0.14952981048020764]])+(1.-msk_arg1)*numpy.array([[-0.71128024173169013, -1.8823014566395226], [-1.7256560427891685, -1.1922627465302083]])
      res=add(arg0,arg1)
      s0=numpy.array([3.5712795821829886, 2.6423455037035559])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.3272292496423317, 4.3571326033107232], [1.0498308681613788, 2.4928156932233483]])+(1.-msk_ref)*numpy.array([[2.8599993404512984, 1.6889781255434659], [0.91668946091438741, 1.4500827571733477]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.1723948405292592, -2.427400638782423])+(1.-msk_arg1)*numpy.array([2.1206902458018382, -0.70851420938638476])
      res=add(arg0,arg1)
      s0=numpy.array([[[2.1680733822297338, -1.0709754264699511, 4.1600082607289508, 3.94320596511694, -1.0292788434259768], [3.8005287857178907, 1.3925676769134743, -4.4191508370088055, -0.16604255763771558, -2.0533538540209593], [-3.4088311482223457, 3.4086382621030769, -1.9965178958934917, -0.86898381493482013, 3.4352054671387027], [4.9531031090579081, 3.1320654728733004, -2.0445883320340053, 4.7974513653555384, -1.6231068239570834]], [[-1.3852373561480622, 0.51040475127872753, -1.7861781356596227, 1.3804935205973035, 2.5377091634677296], [-3.4894476316081322, 2.6191908587680848, -4.9831186719777021, -0.6746257395265145, -4.1663375794453685], [-2.9676666560959317, 4.988775180657786, 4.5262928037214305, 2.1756124668705601, 2.5703106199513748], [-0.11380013771713582, 0.54036041486243569, 4.1385294271248227, -1.1449700967448075, 4.4362705050851723]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.0043214582995255, -4.2433702669992108, 0.98761342019969156, 0.77081112458768075, -4.201673683955236], [0.62813394518863142, -1.779827163615785, -7.5915456775380648, -3.3384373981669748, -5.2257486945502185], [-6.5812259887516049, 0.23624342157381761, -5.168912736422751, -4.0413786554640794, 0.26281062660944343], [1.7807082685286488, -0.040329367655958848, -5.216983172563264, 1.6250565248262792, -4.7955016644863431]], [[-3.8126379949304852, -1.9169958875036954, -4.2135787744420456, -1.0469071181851195, 0.11030852468530661], [-5.9168482703905552, 0.19179021998566181, -7.410519310760125, -3.1020263783089375, -6.5937382182277915], [-5.3950672948783547, 2.5613745418753631, 2.0988921649390075, -0.25178817191186287, 0.14290998116895182], [-2.5412007764995588, -1.8870402239199873, 1.7111287883423998, -3.5723707355272305, 2.0088698663027493]]])+(1.-msk_ref)*numpy.array([[[4.288763628031572, 1.049714819331887, 6.280698506530789, 6.0638962109187782, 1.0914114023758614], [5.9212190315197288, 3.5132579227153125, -2.2984605912069673, 1.9546476881641226, 0.067336391780878913], [-1.2881409024205075, 5.529328507904915, 0.12417234990834647, 1.251706430867018, 5.5558957129405409], [7.0737933548597463, 5.2527557186751386, 0.076101913767832929, 6.9181416111573766, 0.49758342184475479]], [[-2.093751565534447, -0.19810945810765723, -2.4946923450460075, 0.67197931121091869, 1.8291949540813448], [-4.197961840994517, 1.9106766493817, -5.6916328813640868, -1.3831399489128993, -4.8748517888317533], [-3.6761808654823165, 4.2802609712714013, 3.8177785943350457, 1.4670982574841753, 1.86179641056499], [-0.82231434710352058, -0.16815379452394907, 3.430015217738438, -1.8534843061311923, 3.7277562956987875]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_expandedData_rank3(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.8952822928021211, 3.8659353661975935, -0.4582961852780727, -2.9513570623448104, -2.3902717996617695], [-1.301374202554241, -2.9715129266759766, 2.0544222386139444, 2.0261791695661486, -1.9909561094495642], [2.6353286058141991, -1.4033773916010404, -2.561699701045729, -3.1859877735557172, -0.69893697256590581], [-2.6597799339159311, -2.3747937039979106, -1.9816121193690206, -3.3469587534280079, 1.4764684761745448]], [[3.2203807184439412, 2.4807219654637329, -3.0117412408479085, 2.005812995663856, 3.7652288104519371], [0.059462861475312323, 3.2773032653512448, 4.7167989647589543, 3.9827410614957692, 0.46481881398967317], [2.555233433605836, 3.6910135991231918, -4.5463326299618432, -1.4100260294992872, 0.85552923715286777], [-0.63597656466488672, -2.0539780328026671, -2.3247512663172998, -3.3320326025455262, 4.2876762002577529]]])+(1.-msk_arg1)*numpy.array([[[-1.734758324393828, -4.3547832085977669, -4.5580050972901329, -2.4072011653373062, -0.67873224544245048], [2.8687944383542643, 4.6882235615392283, 2.721426123231848, 0.44673060018205391, -0.85959156639840106], [0.72770301935744364, -2.3334126267139932, -0.80306535715565008, 4.3590903187086667, 1.2214787367892974], [-1.4515788454412712, 3.5934600994360704, -3.5339765151329097, 4.8152354673553361, 2.1525491075227912]], [[-0.002438436322243831, -2.313815203718427, -1.0480708163781207, 2.6677586558146338, -2.8359628900651437], [1.6236781017801416, -3.1825363082557647, -3.202249515229, -2.3852125148767356, 4.5161814671414344], [-0.44461681814783471, 0.11171677277545555, -2.6416437462359479, -3.3020186716837427, -4.3138417970090872], [2.9098230482592049, 4.9106862869931138, -0.3472307058051598, -2.1646097401427209, 3.1283242807817846]]])
      res=add(arg0,arg1)
      s0=numpy.array([0.43132474796186671, -0.50462062408852848])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.4639575448402544, 4.2972601141594602, -0.026971437316205993, -2.5200323143829437, -1.9589470516999028], [-0.87004945459237426, -2.5401881787141098, 2.4857469865758111, 2.4575039175280153, -1.5596313614876975], [3.0666533537760658, -0.97205264363917365, -2.1303749530838623, -2.7546630255938505, -0.26761222460403911], [-2.2284551859540644, -1.9434689560360439, -1.5502873714071539, -2.9156340054661412, 1.9077932241364115]], [[2.7157600943554128, 1.9761013413752044, -3.516361864936437, 1.5011923715753275, 3.2606081863634087], [-0.44515776261321616, 2.7726826412627164, 4.2121783406704258, 3.4781204374072408, -0.039801810098855306], [2.0506128095173075, 3.1863929750346633, -5.0509532540503717, -1.9146466535878157, 0.35090861306433929], [-1.1405971887534152, -2.5585986568911956, -2.8293718904058283, -3.8366532266340547, 3.7830555761692244]]])+(1.-msk_ref)*numpy.array([[[-1.3034335764319613, -3.9234584606359002, -4.1266803493282662, -1.9758764173754395, -0.24740749748058377], [3.300119186316131, 5.119548309501095, 3.1527508711937147, 0.87805534814392061, -0.42826681843653436], [1.1590277673193103, -1.9020878787521265, -0.37174060919378338, 4.7904150666705334, 1.6528034847511641], [-1.0202540974794045, 4.0247848473979371, -3.102651767171043, 5.2465602153172028, 2.5838738554846579]], [[-0.50705906041077231, -2.8184358278069555, -1.5526914404666492, 2.1631380317261053, -3.3405835141536722], [1.1190574776916131, -3.6871569323442932, -3.7068701393175285, -2.8898331389652641, 4.0115608430529059], [-0.94923744223636319, -0.39290385131307293, -3.1462643703244764, -3.8066392957722712, -4.8184624210976157], [2.4052024241706764, 4.4060656629045853, -0.85185132989368828, -2.6692303642312494, 2.6237036566932561]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.7075707066105705, -1.9049315294567002])+(1.-msk_arg1)*numpy.array([4.9653536632825492, 3.9022176951484067])
      res=add(arg0,arg1)
      s0=numpy.array([[[[2.9880955007979519, 1.066491950948282], [-0.96410236114001613, 2.783478003179467]], [[0.067173025485383064, -3.135952831436013], [-3.8329225861719216, -4.3444291902062719]], [[1.1719803237706365, 0.89534647311251092], [1.5527435988545424, 3.7824059968377419]], [[-2.6147570226608332, -4.667295149647841], [2.7873180330046035, -4.3517471816369016]], [[-0.97475793945865874, -4.1979240626508094], [-2.4719179888205778, 2.8721172012946887]], [[-1.3390846482074137, 1.6466480644126404], [4.6881707705382833, -3.4690306377440172]]], [[[3.6873296892680951, -3.9480598289675326], [4.8037823766565904, -3.9175345106402579]], [[3.0746486586784432, -4.2609551141155233], [1.2810499409790088, -0.77857843919259562]], [[-1.9962513585841291, 1.4142118964070605], [-4.3152750274797995, -1.3265240936805167]], [[0.86393633299084627, 1.0013635475104117], [-2.0451754665632835, 1.6489815334384268]], [[0.69867496379641914, 2.5101233728729246], [-2.3529112631530005, -4.366110776319216]], [[1.2573830509102173, 0.11794886320158859], [-3.3983398796743192, -1.5748404771830113]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.6956662074085225, 3.7740626575588525], [1.7434683454705544, 5.4910487097900376]], [[2.7747437320959536, -0.42838212482544247], [-1.1253518795613511, -1.6368584835957014]], [[3.8795510303812071, 3.6029171797230815], [4.2603143054651129, 6.4899767034483125]], [[0.092813683949737324, -1.9597244430372704], [5.494888739615174, -1.644176475026331]], [[1.7328127671519118, -1.4903533560402389], [0.23565271778999275, 5.5796879079052593]], [[1.3684860584031568, 4.354218771023211], [7.3957414771488539, -0.76145993113344668]]], [[[1.7823981598113949, -5.8529913584242328], [2.8988508471998902, -5.8224660400969581]], [[1.169717129221743, -6.1658866435722235], [-0.62388158847769137, -2.6835099686492958]], [[-3.9011828880408292, -0.49071963304963973], [-6.2202065569364997, -3.2314556231372169]], [[-1.0409951964658539, -0.90356798194628851], [-3.9501069960199837, -0.2559499960182734]], [[-1.2062565656602811, 0.60519184341622445], [-4.2578427926097007, -6.2710423057759161]], [[-0.6475484785464829, -1.7869826662551116], [-5.3032714091310194, -3.4797720066397115]]]])+(1.-msk_ref)*numpy.array([[[[7.9534491640805012, 6.0318456142308312], [4.0012513021425331, 7.7488316664620163]], [[5.0325266887679323, 1.8294008318465362], [1.1324310771106276, 0.62092447307627729]], [[6.1373339870531858, 5.8607001363950602], [6.5180972621370916, 8.7477596601202912]], [[2.350596640621716, 0.29805851363470826], [7.7526716962871527, 0.61360648164564768]], [[3.9905957238238905, 0.76742960063173982], [2.4934356744619715, 7.837470864577238]], [[3.6262690150751355, 6.6120017276951897], [9.6535244338208326, 1.496323025538532]]], [[[7.5895473844165018, -0.045842133819125941], [8.706000071804997, -0.015316815491851266]], [[6.9768663538268498, -0.35873741896711664], [5.1832676361274155, 3.123639255955811]], [[1.9059663365642776, 5.3164295915554671], [-0.41305733233139286, 2.5756936014678899]], [[4.7661540281392529, 4.9035812426588183], [1.8570422285851231, 5.5511992285868335]], [[4.6008926589448258, 6.4123410680213313], [1.5493064319954062, -0.4638930811708093]], [[5.159600746058624, 4.0201665583499953], [0.5038778154740875, 2.3273772179653953]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_expandedData_rank4(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.7334277359955337, -2.7651888362177801], [1.2482639639517465, 4.6311549210425582]], [[-4.8392287197231019, 0.2717769057842041], [-0.82540505637120276, -4.1139267395303172]], [[-4.0523127196600122, -3.8275764798082665], [-3.6854593389438284, -4.5394497231332762]], [[-0.456845152920061, -3.4537734076708193], [1.2181783439924718, 3.2040118921128329]], [[2.8687102172566048, 0.11455939668704751], [4.7192793700244362, 2.4392307145747143]], [[-3.5935016686915255, -4.3495132624800847], [-0.33002936975291952, 1.9225928797743936]]], [[[2.8770896532591106, 3.4010387373705502], [-3.7053591941822042, -3.2971725378444692]], [[4.4161739188944384, 4.3853237844105912], [1.0290930946913202, 4.3166834302185748]], [[-2.1828947415061983, 1.4565859984722609], [1.5538411335383948, 1.4114622918494408]], [[3.8493526417502579, 3.8850652365677476], [-1.4332118207435771, -2.9467750784927436]], [[-4.9786700067897307, -2.349060148987645], [0.26958223656325053, 4.8193595497055419]], [[-2.7822990001051151, 1.1364584912381588], [3.0555454570357501, -4.8479781874847285]]]])+(1.-msk_arg1)*numpy.array([[[[1.2662299744889918, 0.074732017679900231], [-2.1518465687943045, 4.1811401245582793]], [[-2.1448374796897527, 3.1145151316719719], [-3.8258180451081545, -1.857240547128749]], [[-2.5137662094469437, -4.5464132972902416], [-3.7198871820147361, 4.8621353488629726]], [[1.3649612513362799, 1.8415250672601786], [-2.4614185101341377, 0.054984595692282312]], [[-3.9447967281874377, -4.6938759745504992], [-3.4432602715101979, 3.2744021316058713]], [[3.3061912450388959, -2.2168846554571289], [-2.5194568363541689, 2.9752545316282824]]], [[[3.233512026217749, -2.0310093907292703], [4.5956266018723646, -4.903050155106035]], [[-2.2280068833234723, -1.0190388611893408], [-4.850090096925225, 3.2191297283695608]], [[4.2853709825339674, 4.1157544349451776], [1.5474945346111948, -0.6509984420098629]], [[-0.81487102880001583, -1.4816686582689478], [-4.4590861328804099, 4.1479231156006247]], [[3.2854495018049832, -1.548475040030306], [-1.8146900714468872, -4.0879267143077875]], [[-3.7069570738063384, 3.4326053376327774], [1.9286071291320264, 0.85334821056892451]]]])
      res=add(arg0,arg1)
      s0=numpy.array([-3.8446797409273925, 1.2022061195075127])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.88874799506814117, -6.6098685771451731], [-2.596415776975646, 0.78647518011516571]], [[-8.6839084606504944, -3.5729028351431884], [-4.6700847972985953, -7.9586064804577097]], [[-7.8969924605874047, -7.672256220735659], [-7.5301390798712209, -8.3841294640606687]], [[-4.3015248938474535, -7.2984531485982114], [-2.6265013969349207, -0.64066784881455963]], [[-0.97596952367078771, -3.730120344240345], [0.8745996290970437, -1.4054490263526782]], [[-7.4381814096189185, -8.1941930034074772], [-4.174709110680312, -1.9220868611529989]]], [[[4.0792957727666233, 4.6032448568780628], [-2.5031530746746915, -2.0949664183369565]], [[5.6183800384019511, 5.5875299039181039], [2.2312992141988328, 5.5188895497260875]], [[-0.98068862199868567, 2.6587921179797736], [2.7560472530459075, 2.6136684113569535]], [[5.0515587612577706, 5.0872713560752603], [-0.23100570123606445, -1.7445689589852309]], [[-3.776463887282218, -1.1468540294801324], [1.4717883560707632, 6.0215656692130546]], [[-1.5800928805976024, 2.3386646107456714], [4.2577515765432628, -3.6457720679772159]]]])+(1.-msk_ref)*numpy.array([[[[-2.5784497664384007, -3.7699477232474923], [-5.9965263097216965, 0.33646038363088682]], [[-5.9895172206171452, -0.73016460925542059], [-7.670497786035547, -5.7019202880561419]], [[-6.3584459503743362, -8.3910930382176332], [-7.5645669229421291, 1.0174556079355801]], [[-2.4797184895911126, -2.0031546736672139], [-6.3060982510615302, -3.7896951452351102]], [[-7.7894764691148302, -8.5385557154778908], [-7.2879400124375904, -0.57027760932152116]], [[-0.53848849588849657, -6.0615643963845214], [-6.3641365772815615, -0.86942520929911016]]], [[[4.4357181457252617, -0.82880327122175768], [5.7978327213798773, -3.7008440355985224]], [[-1.0258007638159596, 0.18316725831817182], [-3.6478839774177123, 4.4213358478770735]], [[5.4875771020414801, 5.3179605544526902], [2.7497006541187075, 0.55120767749764976]], [[0.38733509070749683, -0.27946253876143512], [-3.2568800133728972, 5.3501292351081373]], [[4.4876556213124958, -0.34626892052279334], [-0.61248395193937455, -2.8857205948002749]], [[-2.5047509542988258, 4.6348114571402901], [3.130813248639539, 2.0555543300764372]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.7733158475802995, 0.080411677385336944, 2.0727431531575373, 2.9514566107978091, -4.7672860605534231], [-0.97491850367369537, 2.5912278491216476, 3.314565752568539, 1.5308614040920006, -0.008400843333168595], [2.4001279783692775, -2.2470847767339097, -2.5605961932668473, 0.94435565798263621, -2.97293392372737], [3.6598147317134817, -3.8042483167431538, 1.4495119573910396, 0.68658596594431209, 3.7136404194473283]])+(1.-msk_arg1)*numpy.array([[3.0175577098454323, 3.9992026070096625, 0.1031449154645836, -3.3660635952188511, -3.6972731135134298], [-2.452840120319971, -2.0909001245315864, 4.2482786318986054, -0.50594999269013741, 3.7838970439427442], [-1.4481929519537564, 2.2086502933122985, -1.2736184434233619, 4.0502242246416245, -0.14686183349457149], [2.1295101543177486, -1.1905530338271131, 2.0821140814266954, 1.1362268501281729, -1.513030743797823]])
      res=add(arg0,arg1)
      s0=numpy.array([[4.1137987444019721, 1.5758132363124675, -3.9860783892316065, 1.5656027977156848, -2.3557527642597966], [-1.3598541810802791, 4.6307100159124186, 4.4812387458849248, -1.8374055347956042, 3.9203748301299193], [4.5478093508191364, -2.3669448832829021, -3.5787389846674378, -1.5757352508414026, -0.68703457769360199], [1.022482064815982, 1.884050108711552, 3.2933685681047713, -1.6512649569990212, 3.5548440693721179]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[8.8871145919822716, 1.6562249136978044, -1.9133352360740692, 4.5170594085134939, -7.1230388248132197], [-2.3347726847539745, 7.2219378650340662, 7.7958044984534638, -0.30654413070360365, 3.9119739867967507], [6.9479373291884139, -4.6140296600168114, -6.1393351779342851, -0.63137959285876644, -3.659968501420972], [4.6822967965294637, -1.9201982080316018, 4.7428805254958109, -0.96467899105470911, 7.2684844888194462]])+(1.-msk_ref)*numpy.array([[7.1313564542474044, 5.57501584332213, -3.8829334737670229, -1.8004607975031663, -6.0530258777732264], [-3.8126943014002501, 2.5398098913808322, 8.7295173777835302, -2.3433555274857416, 7.7042718740726635], [3.09961639886538, -0.15829458997060364, -4.8523574280907997, 2.4744889738002218, -0.83389641118817348], [3.1519922191337306, 0.6934970748844389, 5.3754826495314667, -0.51503810687084828, 2.041813325574295]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.8889420700928845, -4.0695196185947582, 2.4708991393605286, 0.41340705544589618, 2.5657373579519396], [3.5419982035476245, 2.9247721035642273, 4.3943732805368789, 4.4390320970275337, 0.64303849737712504], [3.172293289649101, 1.8965060956853979, -0.50815132166905919, 0.058982367950253689, 2.9521212708452262], [3.8672913639602839, -0.9355422540943259, -4.5215327091077668, -4.917644701097065, 1.1730092285229503]])+(1.-msk_arg1)*numpy.array([[3.1100422986702938, 2.1866820247158536, 2.7606333751788679, -1.5304492587880167, -4.5428883633995323], [-4.5678164446953993, -1.5807410300785971, 1.5256166577858989, 1.8267612326523244, 3.5247580674946359], [-3.4262207405957334, -1.7729883714312713, -2.0349968683104134, 2.2800662361312805, 0.6961674013805208], [-1.7089423017565499, 1.0121819646430144, 0.66070660076840682, -4.5701339103366152, 2.2528959361675476]])
      res=add(arg0,arg1)
      s0=numpy.array([[[3.1161290359094647, -1.4464800170640224], [-2.5019240176872359, -1.7879495665513967], [0.78270582317047754, -2.146108069266508], [-0.36704503556059276, 4.8629015629000989], [4.5492514397460919, 0.51284064725730261]], [[3.3175333991363694, 4.2163976430207608], [4.7805692546754361, 1.7221637385776329], [-1.4991448383595674, -1.351720548931985], [2.9270456457821545, -0.70857028633723473], [-3.4267188134349045, -2.2715576908256265]], [[2.0105437642569743, -1.2695528851609037], [-2.8973433210429675, -4.1629879719518312], [2.1013446749992157, 0.91108181743082106], [-4.3828464865879884, -4.1804096394408949], [-3.5727813840093003, 1.3624864403590999]], [[-0.96843404691363943, -4.2887922546696657], [0.60558945182131207, -0.7840718654774399], [3.7276882001598572, -4.7164881090483304], [-1.692818267562084, -2.2937874117697543], [-1.2624660166909738, 0.24018740452161591]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.0050711060023492, 2.4424620530288621], [-6.5714436362819946, -5.8574691851461544], [3.2536049625310062, 0.32479107009402064], [0.046362019885303418, 5.2763086183459951], [7.1149887976980315, 3.0785780052092422]], [[6.8595316026839939, 7.7583958465683853], [7.7053413582396635, 4.6469358421418603], [2.8952284421773116, 3.0426527316048939], [7.3660777428096882, 3.7304618106902989], [-2.7836803160577794, -1.6285191934485015]], [[5.1828370539060753, 1.9027404044881973], [-1.0008372253575697, -2.2664818762664334], [1.5931933533301565, 0.40293049576176188], [-4.3238641186377347, -4.1214272714906413], [-0.6206601131640741, 4.3146077112043262]], [[2.8988573170466445, -0.42150089070938179], [-0.32995280227301382, -1.7196141195717658], [-0.79384450894790959, -9.2380208181560981], [-6.6104629686591494, -7.2114321128668193], [-0.089456788168023493, 1.4131966330445662]]])+(1.-msk_ref)*numpy.array([[[6.2261713345797585, 1.6635622816062714], [-0.31524199297138233, 0.3987324581644569], [3.5433391983493454, 0.61452530591235988], [-1.8974942943486095, 3.3324523041120822], [0.0063630763465596374, -4.0300477161422297]], [[-1.2502830455590299, -0.35141880167463846], [3.199828224596839, 0.14142270849903582], [0.026471819426331589, 0.17389610885391393], [4.7538068784344789, 1.1181909463150896], [0.098039254059731462, 1.2532003766690094]], [[-1.4156769763387591, -4.6957736257566367], [-4.6703316924742388, -5.9359763433831025], [0.066347806688802358, -1.1239150508795923], [-2.1027802504567079, -1.9003434033096145], [-2.8766139826287795, 2.0586538417396207]], [[-2.6773763486701894, -5.9977345564262157], [1.6177714164643264, 0.22811009916557445], [4.3883948009282641, -4.0557815082799236], [-6.2629521778986987, -6.8639213221063695], [0.99042991947657377, 2.4930833406891635]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.8524882124090354, 1.7149087934422464], [3.9097061805299145, -2.5112148828403988], [0.50684576474228837, -2.5032438436021396], [0.47466710394470901, -3.0445608545274547], [1.4617380631436108, -2.0669122642227009]], [[3.380920468245094, -3.502728270163141], [-1.5917468306120819, -2.1611626876063772], [-3.8099026132254563, 0.1889710756063403], [4.8551626692027323, -2.9019032023271798], [-1.4913378459369806, 0.19921584592694419]], [[2.3570379643494093, -2.0444147020767311], [0.3270322096850613, -3.1955736076126384], [3.5876195614748632, 4.7019313829572109], [-2.0173775338884372, -2.0812524495573346], [-0.019696449048667297, 0.52667329854301848]], [[-0.44357469010092654, 0.22346322436617783], [-4.659351209956597, 1.0017227790307972], [1.5940552068217535, 0.4664039374506368], [-1.4487683078513536, 4.9646608128241514], [-4.2203772663324255, -2.1390887085931274]]])+(1.-msk_arg1)*numpy.array([[[-1.8659621325624975, 0.16851754353089721], [-1.9105749977225042, -3.3932667657906745], [-3.9540509689329086, -1.4144749488346253], [3.9193906229587228, -4.7116484517028159], [-4.8636538085726357, -1.0070143150666624]], [[3.271549523714846, -4.0887338902630805], [3.6490944155126623, 2.8491339286125852], [4.5005301001813827, 3.3924150934892445], [-3.2840223046703887, 3.315220176470401], [4.628265183728022, 3.0501509796009394]], [[0.71921703414593896, 1.2228836638815519], [2.1715300497160204, -2.3664915339904615], [2.852785306633443, -0.62906221397027196], [2.2967713905532339, -2.8298761989641354], [4.2367502762020788, 4.1049348899580966]], [[4.5181488923457032, -4.854599661221112], [1.9508250501861895, 1.9999822708721329], [4.0039453798418307, -3.0843547790679726], [-0.85886941280263418, -3.8086969520403038], [0.44827930673616923, 2.5827417310476832]]])
      res=add(arg0,arg1)
      s0=numpy.array([[2.8218721124262363, 4.464024758790643, -1.7649057197130427, 3.5113030721132716, -2.3128509495945506], [-4.9857231585804076, -1.2291249948620986, -2.676076830471652, 0.47863953290487515, 0.58825451933105821], [2.9930576032273217, 3.8785630171251402, -3.0861297132208754, 1.1724819030937015, 2.1382096889765752], [4.7690180034994754, -0.99025277885507279, -4.8316906372412269, -3.0658977422763432, 0.86679210688149677]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.6743603248352716, 4.5367809058684827], [8.3737309393205575, 1.9528098759502441], [-1.2580599549707543, -4.2681495633151822], [3.9859701760579807, 0.46674221758581691], [-0.85111288645093985, -4.3797632138172515]], [[-1.6048026903353136, -8.4884514287435486], [-2.8208718254741805, -3.3902876824684758], [-6.4859794436971079, -2.4871057548653117], [5.3338022021076075, -2.4232636694223046], [-0.90308332660592239, 0.7874703652580024]], [[5.350095567576731, 0.94864290115059058], [4.2055952268102015, 0.68298940951250176], [0.50148984825398779, 1.6158016697363355], [-0.84489563079473573, -0.90877054646363309], [2.1185132399279079, 2.6648829875195936]], [[4.3254433133985488, 4.9924812278656532], [-5.6496039888116698, 0.011470000175724415], [-3.2376354304194734, -4.3652866997905901], [-4.5146660501276967, 1.8987630705478082], [-3.3535851594509287, -1.2722966017116306]]])+(1.-msk_ref)*numpy.array([[[0.95590997986373871, 2.9903896559571335], [2.5534497610681388, 1.0707579929999684], [-5.7189566886459513, -3.179380668547668], [7.4306936950719944, -1.2003453795895442], [-7.1765047581671864, -3.319865264661213]], [[-1.7141736348655616, -9.0744570488434881], [2.4199694206505638, 1.6200089337504866], [1.8244532697097307, 0.71633826301759251], [-2.8053827717655135, 3.7938597093752762], [5.2165197030590802, 3.6384054989319976]], [[3.7122746373732607, 4.2159412671088736], [6.0500930668411605, 1.5120714831346787], [-0.23334440658743238, -3.7151919271911473], [3.4692532936469354, -1.657394295870434], [6.3749599651786539, 6.2431445789346718]], [[9.2871668958451785, -0.085581657721636617], [0.96057227133111667, 1.0097294920170601], [-0.82774525739939619, -7.9160454163091991], [-3.9247671550789773, -6.8745946943166469], [1.315071413617666, 3.44953383792918]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.1973519791460863, 3.0379243603803676, -1.5371708780375282, 3.5727458900118911, -4.5594936016442924], [0.81095565070679498, 2.2840833296131944, -0.084566177508200724, 3.6596227379791291, -2.6970972638635549], [3.5179943195843038, 4.7625846434664147, -3.7095729703141043, 2.7341371885980381, -3.0040128535468469], [-3.8816478164485533, 0.045557846387922041, 3.9280504041716746, 3.4760640180623419, -0.28439566822221618]])+(1.-msk_arg1)*numpy.array([[-2.6284016034409099, 3.3556183652378486, 3.477323769158275, 2.8725271565351953, 4.489338236217904], [-4.0931253698068808, 2.4529065676848276, 4.4459079194644993, 1.4143816470194963, -0.99432036631989718], [1.9590407305936166, 4.7740341236037516, 1.7535560115421092, -3.9249589405981187, -3.5407706515285509], [-0.93089942815293192, 1.1264366013736291, -2.6580597084558013, -4.93406812918521, 1.221126981525817]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[-2.3281034450436309, -4.3561205219620334, 1.9415099670479261, -2.2444842970720558, -2.6444270676299118], [4.905026027338419, -1.7956672999994892, -1.6066652204489706, -4.0046192018081816, -2.941012355627973], [2.1906334884605538, -3.6227673766150992, -0.27766907666379037, -0.85869787595203828, -2.9003286874745324], [-2.4743776934738069, -2.7806340228622242, -0.74548699839394583, 4.8381613184404024, -3.0308653920190176]], [[3.745733493028041, -2.9366895429369233, 2.2855389195446367, -0.5298408287842582, -0.88066683558957859], [0.65531698209238076, -4.5200090747373967, -2.9894007626008099, -0.019991652104978996, -1.7592220075807798], [-4.2353262418479982, 3.2122022854423182, -1.691291537949835, -3.1133674513466039, -4.3745707453854834], [-3.3642758632377721, 1.4071309808429655, 2.4874121485002156, -0.94619250975657643, 3.8376285086475939]], [[1.0790058378866512, -1.0928999200070622, 3.1554083154523251, -1.6740653462910537, 0.4379926466339521], [-4.9078302315864386, 0.4500065876964463, -1.9865231263953254, 4.2982809396512067, -3.3731426298009182], [1.2004769395861592, 1.7418395880778519, -3.4625164205382997, 4.2214561096810748, -1.8429268624121464], [3.3995172528787627, 1.518757395204096, -2.1959203786217243, 2.2782747550169411, -2.0722284525819914]], [[1.5578513438645771, -2.6706589780883006, -0.86581599232602713, -0.23654546547150801, -4.20090679900105], [1.2584155459394717, 3.510710060717555, 4.3245976767102032, -3.0287027098528316, 2.3607334858214246], [2.0386641344217713, 4.9602721524822755, 4.118873622700896, -4.3027364700780293, 2.6319927183783012], [1.4611761229971965, -4.3569228411309986, 1.2828230626563997, 1.6855480020596145, -2.484770164628106]], [[3.7694192303092304, -2.8415080329272593, 1.7957100469700116, -1.8504915647212461, -2.5846262978791623], [-3.8127386526516629, 0.90465713525008518, -0.6481098395116911, -1.1013487852234882, -2.5188335695089439], [1.5502010263653174, 2.8812031211561724, -4.1953655883616694, 3.8209140815858884, 0.15888633663621032], [3.5849292252262384, -1.1783706125687674, -0.17499862723317428, 4.1096599411501327, -4.7451186953475748]]], [[[4.8137794275194885, -0.30475685651844664, 1.4641526710240713, 0.7688992369412615, 2.2019698386222384], [0.38314100664963391, -1.1472545466472148, 4.8458628772369448, 4.8482511601814657, -1.4972055249535643], [1.9578447087041955, -0.64837971159820729, -4.1576682940755454, -3.1284782064343353, -4.0344461323385001], [4.2468952873385373, -4.4101446146401555, 2.2881893138201388, 3.4480464032812961, -3.0245443507465488]], [[-4.6614368785687335, -4.2400094195485476, 3.9293056775562221, 0.56688450871734997, 2.9919409348705015], [-4.1648412912299095, -2.77758702969252, 2.5251139252825618, -4.705025480074541, 2.7474434122310303], [0.8256797015064441, -1.1862781988008742, -3.3458773550587728, -2.8549079806816788, 3.6421331690176135], [4.3630915375798356, -0.018575207071503286, -4.1172877661090173, -0.58778268868297179, -1.4454173493557345]], [[0.084685641619253182, -2.8292099885753483, -2.7857914416310505, -0.42922169269719923, -1.1731256969101933], [-1.4733083691013227, -3.5248181719673575, -0.20113350768961347, 4.2597898463050115, -4.9462857148567423], [3.797996680637711, -0.2072234821414396, 4.5138777603146192, -2.0878852197497935, -3.6201308533941146], [-3.4677616518079732, -0.19032041593327342, 3.8968894543858124, 4.0178557974035645, -0.73674444543227668]], [[-0.17125474410328678, -4.7563764340947268, 4.7719509884571867, -0.28235253576751251, 2.0843301418201508], [3.5108587793899897, 4.1305192747096005, 0.30125413045594218, -1.1513463036686433, 4.669703427748118], [-4.61992110697794, -1.5119246674845401, -4.4667160269311887, -3.8003216730979394, 1.8217632386737321], [1.6602031332510183, -2.5561583434042356, 3.2060846508198821, 3.9697029430882811, 4.2848014134494701]], [[0.30901625570232039, 0.36416343543262109, 0.64630369366510809, -1.7890629664919047, -1.8492796627406838], [-4.073902735914892, 1.398383644293566, -0.46420364047038376, 1.8200656840456961, -1.6685850641360931], [0.061260783718344314, 0.28984626868310492, -0.85191562772449547, -3.0476163515993235, -0.49621688517592855], [-4.2838317527716079, -0.19462742453140969, -4.457247044247441, -3.968545633806857, -2.8982191087167175]]], [[[4.2751463585890335, 2.2281278549185011, -0.57074236709080495, -3.5114905847676625, -3.4573159840758683], [3.0362436331746263, -1.7532739401006916, -3.1698226142629848, -3.8759450976003453, 3.3287097331135964], [-1.8564546035672413, 1.2394438202478328, 4.6437439720900304, -2.9032948876839013, 1.8490156213919109], [4.6314132737226661, 0.029729468035638362, 1.6229953552479977, 4.5139228480138751, 2.2544105504745646]], [[-1.3744196177955326, 1.8294260613243001, -2.6212247061808314, 0.089751014086434822, 0.63075134396357413], [1.4746090379759007, -1.5547073869289632, 4.8756047319089717, -1.0061912922201399, -2.3516137032635545], [-1.3503190099500886, 2.9949350160825459, 4.1980684168420428, -4.367817551254662, -1.3849188089573539], [-3.3098352202666046, -3.0484113094832854, 1.727871847235428, -3.8671487865088872, 4.4474846198430402]], [[-0.98842816575415959, 4.6597917590786011, -4.5614512332405255, -3.5318373898061983, -0.8372928147060863], [-2.0203292270531827, -3.9751111866900271, 0.85753074896766535, 4.9667247820452243, 3.2042606682667678], [-0.53483535444975594, -0.75710317949484285, 0.380406061589321, 4.3721876517622231, -1.06297305494674], [1.6994553884954406, -3.5933207974776171, -3.6298226614089035, -1.5030705928543506, -4.6652652911577004]], [[-0.68916465416255512, 4.2915550090216072, 2.4491190087301842, 2.4242165318865005, -1.3741194216767694], [3.7510651943507298, 3.5634727251002101, -0.36054338281882625, -4.2557404635693876, 2.9874708076676404], [-4.8875585603141705, 1.1522455628001618, -0.40306396582279724, 0.11643717751319649, -4.5777736528396709], [-2.4035420137003074, -4.2591848774741381, -0.24748118291163479, -3.5638259309152556, 3.9703503997984093]], [[-4.3865025164015883, 1.3519421214241332, -2.2693154563635485, -2.6017635474000969, 2.1522178450070815], [-4.5023891711461115, -1.6347878693068418, -4.3513728487184675, 0.2107496909209825, 2.9814669340677664], [2.9355789210234757, 3.502650261124483, -1.1823190462342925, 2.4615005690950049, 2.9762359716244449], [-3.4379752926529727, -3.2798379682606003, -1.7957123479953663, -3.8920100755850449, -4.850997283713796]]], [[[2.1059641634069166, -2.4097829474073116, 0.6000222286935406, 1.3703668500369401, 0.93566466873034493], [-0.97518570839364926, -4.6576100870488704, 3.9618201176728185, 3.7949404705891538, 0.39176997374603673], [-1.5875675597086492, -2.9371314757504088, 3.1170472912188334, -1.1578715751943225, -4.1773669856441469], [1.690923574986841, 2.701318470975008, 1.067459546597826, -1.0191538666267927, -4.9563540454817385]], [[0.970351435813396, -2.6620488122300157, 2.0867524107995905, -2.3285740041382263, 1.3481547795833144], [4.8972785702663089, -1.8933488562837852, -1.3284665755657885, -3.1567042220658048, 1.7334011488592838], [-2.094010478593582, 0.72170240818468123, -4.9238235118049021, -2.0233307864586649, -0.33226022794281462], [4.2502964627172588, -3.4060958929981333, -3.9304131121381225, 2.7451398587300622, 1.7873008661641521]], [[1.1287424974365026, -1.5398773689240244, 4.7443234766917612, -1.3701581042148692, -0.90466488527629352], [-3.7299019805427136, 0.10755904298692531, 1.2490035753242834, 0.58905693812938864, 2.9943784061278329], [0.42700095766680679, -4.6163534194265843, 1.3108661868711433, 4.2083261115161967, -0.47745513330544931], [4.1008155271636895, 1.0694703812105448, 3.7671997740149816, -1.2060254366884782, 3.8769914032712531]], [[-3.6504552056651738, 0.43601608384620505, 2.4735469354766648, -4.1806556569292583, -4.1341614152220441], [2.1661420724269238, 3.2934296520841535, 2.6992532194864296, 2.4048401856344412, 0.18239129598725068], [1.7982999168128178, -3.4093639288608846, 4.5254312106326928, 4.5775734396565912, -4.6977744410106981], [0.31173753751752109, -1.8821874966974663, 3.2844595199452424, -2.4014663715338367, -1.1592372498238945]], [[-4.6530246325386324, 2.3580782840992072, -1.7090839812775238, -1.0534820983573887, 3.8025611859150317], [1.6700491719272517, -0.28554219189081387, 2.2104313960539415, -2.8425079611316786, 3.9426082205558011], [-4.2175503289256504, 4.407238315372684, -3.0110435089676857, -1.2586155933474377, -3.0134580303313738], [-4.6810729549295571, 4.7654147814511933, 4.4628713469066987, 0.13079719140921853, 1.2348625586683157]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.8692485341024554, -0.1587685428159471, 6.1388619461940124, 1.9528676820740305, 1.5529249115161745], [9.1023780064845052, 2.4016846791465971, 2.5906867586971156, 0.19273277733790461, 1.2563396235181132], [6.3879854676066401, 0.57458460253098709, 3.9196829024822959, 3.338654103194048, 1.2970232916715538], [1.7229742856722794, 1.4167179562838621, 3.4518649807521404, 9.0355132975864887, 1.1664865871270687]], [[6.7836578534084087, 0.10123481744344431, 5.3234632799250043, 2.5080835315961094, 2.157257524790789], [3.6932413424727484, -1.482084714357029, 0.048523597779557726, 3.0179327082753886, 1.2787023527995878], [-1.1974018814676306, 6.2501266458226858, 1.3466328224305326, -0.075443090966236248, -1.3366463850051158], [-0.32635150285740444, 4.4450553412233331, 5.5253365088805833, 2.0917318506237912, 6.8755528690279615]], [[-0.458165040150877, -2.6300707980445903, 1.618237437414797, -3.2112362243285819, -1.0991782314035761], [-6.4450011096239663, -1.0871642903410819, -3.5236940044328535, 2.7611100616136786, -4.9103135078384463], [-0.33669393845136897, 0.20466871004032372, -4.9996872985758278, 2.6842852316435466, -3.3800977404496746], [1.8623463748412346, -0.018413482833432138, -3.7330912566592525, 0.74110387697941293, -3.6093993306195196]], [[5.1305972338764683, 0.90208691192359058, 2.706929897685864, 3.3362004245403831, -0.62816090898915888], [4.8311614359513628, 7.0834559507294461, 7.8973435667220944, 0.5440431801590595, 5.9334793758333157], [5.6114100244336624, 8.5330180424941666, 7.6916195127127871, -0.72999058006613815, 6.2047386083901923], [5.0339220130090876, -0.78417695111910746, 4.8555689526682908, 5.2582938920715057, 1.0879757253837852]], [[-0.79007437133506198, -7.4010016345715517, -2.7637835546742808, -6.4099851663655389, -7.1441198995234547], [-8.3722322542959553, -3.6548364663942072, -5.2076034411559835, -5.6608423868677811, -7.0783271711532363], [-3.009292575278975, -1.67829048048812, -8.7548591900059627, -0.73857952005840399, -4.4006072650080821], [-0.97456437641805405, -5.7378642142130598, -4.7344922288774667, -0.44983366049415974, -9.3046122969918663]]], [[[5.6247350782262835, 0.50619879418834834, 2.2751083217308663, 1.5798548876480565, 3.0129254893290334], [1.1940966573564289, -0.33629889594041984, 5.6568185279437397, 5.6592068108882607, -0.68624987424676931], [2.7688003594109905, 0.1625759391085877, -3.3467126433687504, -2.3175225557275403, -3.2234904816317052], [5.0578509380453323, -3.5991889639333605, 3.0991449645269338, 4.2590020539880911, -2.2135887000397538]], [[-2.3773535489555391, -1.9559260899353532, 6.2133890071694164, 2.8509678383305443, 5.2760242644836959], [-1.8807579616167152, -0.49350370007932565, 4.8091972548957562, -2.4209421504613466, 5.0315267418442247], [3.1097630311196385, 1.0978051308123202, -1.0617940254455784, -0.57082465106848446, 5.9262164986308079], [6.64717486719303, 2.2655081225416911, -1.8332044364958229, 1.6963006409302226, 0.83866598025745986]], [[0.00011946411105245858, -2.9137761660835491, -2.8703576191392512, -0.51378787020539995, -1.257691874418394], [-1.5578745466095234, -3.6093843494755582, -0.28569968519781419, 4.1752236687968107, -5.030851892364943], [3.7134305031295103, -0.29178965964964032, 4.4293115828064185, -2.1724513972579942, -3.7046970309023153], [-3.552327829316174, -0.27488659344147415, 3.8123232768776116, 3.9332896198953637, -0.82131062294047741]], [[3.4883679938758423, -1.0967536961155977, 8.4315737264363158, 3.3772702022116166, 5.7439528797992798], [7.1704815173691188, 7.7901420126887295, 3.9608768684350713, 2.5082764343104857, 8.329326165727247], [-0.96029836899881094, 2.1476980704945889, -0.80709328895205967, -0.14069893511881038, 5.4813859766528612], [5.3198258712301474, 1.1034643945748934, 6.8657073887990112, 7.6293256810674102, 7.9444241514285991]], [[-2.3880810081612345, -2.3329338284309338, -2.0507935701984468, -4.4861602303554591, -4.5463769266042391], [-6.7709999997784465, -1.2987136195699889, -3.1613009043339386, -0.87703157981785873, -4.3656823279996484], [-2.6358364801452105, -2.4072509951804499, -3.5490128915880503, -5.744713615462878, -3.1933141490394834], [-6.9809290166351623, -2.8917246883949645, -7.1543443081109963, -6.6656428976704118, -5.5953163725802728]]], [[[7.7931406781733372, 5.7461221745028048, 2.9472519524934988, 0.0065037348166412379, 0.060678335508435488], [6.5542379527589301, 1.7647203794836122, 0.34817170532131891, -0.35795077801604158, 6.8467040526979002], [1.6615397160170624, 4.7574381398321366, 8.1617382916743342, 0.61469943190040244, 5.3670099409762146], [8.1494075933069698, 3.5477237876199421, 5.1409896748323014, 8.0319171675981789, 5.7724048700588684]], [[3.3881650256708822, 6.5920107047907148, 2.1413599372855834, 4.8523356575528496, 5.3933359874299889], [6.2371936814423155, 3.2078772565374516, 9.6381893753753864, 3.7563933512462748, 2.4109709402028603], [3.4122656335163262, 7.7575196595489606, 8.9606530603084575, 0.39476709221175277, 3.3776658345090609], [1.4527494231998102, 1.7141733339831293, 6.4904564907018427, 0.89543585695752759, 9.210069263309455]], [[-4.6980011360682639, 0.95021878876449684, -8.2710242035546298, -7.2414103601203026, -4.5468657850201906], [-5.729902197367287, -7.6846841570041313, -2.8520422213464389, 1.25715181173112, -0.50531230204733646], [-4.2444083247638602, -4.4666761498089471, -3.3291669087247833, 0.66261468144811886, -4.7725460252608443], [-2.0101175818186636, -7.3028937677917209, -7.3393956317230078, -5.2126435631684549, -8.3748382614718047]], [[2.044972534435483, 7.0256921976196454, 5.1832561973282223, 5.1583537204845387, 1.3600177669212687], [6.4852023829487679, 6.2976099136982482, 2.3735938057792119, -1.5216032749713495, 5.7216079962656785], [-2.1534213717161323, 3.8863827513981999, 2.3310732227752409, 2.8505743661112346, -1.8436364642416327], [0.33059517489773071, -1.5250476888761, 2.4866560056864033, -0.82968874231721745, 6.7044875883964474]], [[-7.3905153699484352, -1.6520707321227137, -5.2733283099103954, -5.6057764009469437, -0.85179500853976542], [-7.5064020246929584, -4.6388007228536887, -7.3553857022653144, -2.7932631626258644, -0.022545919479080467], [-0.068433932523371155, 0.49863740757763608, -4.1863318997811394, -0.54251228445184196, -0.027776881922402019], [-6.4419881461998196, -6.2838508218074471, -4.7997252015422127, -6.8960229291318917, -7.8550101372606429]]], [[[-1.7756836530416367, -6.2914307638558649, -3.2816255877550127, -2.5112809664116131, -2.9459831477182084], [-4.8568335248422025, -8.5392579034974236, 0.080172301224265219, -0.086707345859399432, -3.4898778427025166], [-5.4692153761572024, -6.8187792921989621, -0.76460052522971989, -5.0395193916428758, -8.059014802092701], [-2.1907242414617123, -1.1803293454735453, -2.8141882698507272, -4.900801683075346, -8.8380018619302909]], [[1.015909282201318, -2.6164909658420936, 2.1323102571875125, -2.2830161577503043, 1.3937126259712365], [4.9428364166542309, -1.8477910098958632, -1.2829087291778665, -3.1111463756778828, 1.7789589952472058], [-2.04845263220566, 0.76726025457260327, -4.87826566541698, -1.9777729400707429, -0.28670238155489258], [4.2958543091051808, -3.3605380466102113, -3.8848552657502005, 2.7906977051179842, 1.8328587125520741]], [[5.0567929016081772, 2.3881730352476502, 8.6723738808634359, 2.5578922999568054, 3.0233855188953811], [0.19814842362896101, 4.0356094471585999, 5.177053979495958, 4.5171073423010633, 6.9224288102995075], [4.3550513618384814, -0.68830301525490967, 5.2389165910428179, 8.1363765156878713, 3.4505952708662253], [8.0288659313353641, 4.9975207853822194, 7.6952501781866562, 2.7220249674831964, 7.8050418074429277]], [[-0.1743911876028319, 3.9120801019085469, 5.9496109535390067, -0.70459163886691645, -0.65809739715970217], [5.6422060904892657, 6.7694936701464954, 6.1753172375487715, 5.8809042036967831, 3.6584553140495926], [5.2743639348751596, 0.066700089201457313, 8.0014952286950347, 8.0536374577189331, -1.2217104229483562], [3.787801555579863, 1.5938765213648756, 6.7605235380075843, 1.0745976465285052, 2.3168267682384474]], [[-4.9374203007608486, 2.073682615876991, -1.99347964949974, -1.3378777665796049, 3.5181655176928155], [1.3856535037050355, -0.56993786011303005, 1.9260357278317253, -3.1269036293538948, 3.6582125523335849], [-4.5019459971478666, 4.1228426471504678, -3.2954391771899019, -1.5430112615696538, -3.29785369855359], [-4.9654686231517733, 4.4810191132289772, 4.1784756786844826, -0.15359847681299765, 0.95046689044609955]]]])+(1.-msk_ref)*numpy.array([[[[-4.9565050484845408, -6.9845221254029433, -0.68689163639298378, -4.8728859005129657, -5.2728286710708216], [2.2766244238975091, -4.424068903440399, -4.2350668238898805, -6.6330208052490915, -5.5694139590688829], [-0.43776811498035606, -6.2511689800560095, -2.9060706801047003, -3.4870994793929482, -5.5287302909154423], [-5.1027792969147168, -5.4090356263031341, -3.3738886018348557, 2.2097597149994925, -5.6592669954599275]], [[7.1013518582658897, 0.41892882230092532, 5.6411572847824853, 2.8257775364535904, 2.47495152964827], [4.0109353473302294, -1.164390709499548, 0.36621760263703873, 3.3356267131328696, 1.5963963576570688], [-0.8797078766101496, 6.5678206506801668, 1.6643268272880136, 0.24225091389124476, -1.0189523801476348], [-0.0086574979999234358, 4.7627493460808141, 5.8430305137380643, 2.4094258554812722, 7.1932468738854425]], [[4.5563296070449262, 2.3844238491512129, 6.6327320846106002, 1.8032584228672213, 3.9153164157922271], [-1.4305064624281636, 3.9273303568547213, 1.4908006427629497, 7.7756047088094817, 0.10418113935735684], [4.6778007087444342, 5.2191633572361269, 0.014807348619975347, 7.6987798788393498, 1.6343969067461286], [6.8768410220370377, 4.996081164362371, 1.2814033905365507, 5.7555985241752161, 1.4050953165762836]], [[4.4303785003997724, 0.20186817844689475, 2.0067111642091682, 2.6359816910636873, -1.3283796424658547], [4.130942702474667, 6.3832372172527503, 7.1971248332453985, -0.15617555331763633, 5.2332606423566199], [4.9111912909569666, 7.8327993090174708, 6.9914007792360913, -1.430209313542834, 5.5045198749134965], [4.3337032795323918, -1.4843956845958033, 4.155350219191595, 4.5580751585948098, 0.38775699190708934]], [[8.2587574665271344, 1.6478302032906447, 6.2850482831879155, 2.6388466714966579, 1.9047119383387416], [0.67659958356624106, 5.3939953714679891, 3.8412283967062129, 3.3879894509944157, 1.9705046667089601], [6.0395392625832214, 7.3705413573740763, 0.29397264785623456, 8.3102523178037924, 4.6482245728541143], [8.0742674614441423, 3.3109676236491365, 4.3143396089847297, 8.5989981773680366, -0.25578045912967085]]], [[[0.72065405771260771, -4.3978822263253274, -2.6289726987828095, -3.3242261328656193, -1.8911555311846424], [-3.7099843631572469, -5.2403799164540956, 0.75273750743006396, 0.75512579037458494, -5.5903308947604451], [-2.1352806611026853, -4.7415050814050881, -8.2507936638824262, -7.2216035762412165, -8.1275715021453809], [0.15376991753165647, -8.5032699844470372, -1.804936055986742, -0.64507896652558472, -7.1176697205534296]], [[-2.2085303108839058, -1.78710285186372, 6.3822122452410497, 3.0197910764021776, 5.4448475025553291], [-1.7119347235450819, -0.32468046200769241, 4.9780204929673895, -2.2521189123897134, 5.2003499799158579], [3.2785862691912717, 1.2666283688839535, -0.8929707873739452, -0.40200141299685122, 6.0950397367024411], [6.8159981052646632, 2.4343313606133243, -1.6643811984241896, 1.8651238790018558, 1.0074892183290931]], [[4.5305935610837524, 1.6166979308891509, 1.6601164778334487, 4.0166862267673, 3.272782222554306], [2.9725995503631766, 0.92108974749714179, 4.2447744117748858, 8.7056977657695107, -0.50037779539224303], [8.2439046001022103, 4.2386844373230597, 8.9597856797791184, 2.3580226997147058, 0.8257770660703847], [0.97814626765652601, 4.2555875035312258, 8.3427973738503116, 8.4637637168680637, 3.7091634740322226]], [[1.2431269029162095, -3.3419947870752305, 6.186332635476683, 1.1320291112519838, 3.4987117888396471], [4.925240426409486, 5.5449009217290968, 1.7156357774754385, 0.26303534335085299, 6.0840850747676143], [-3.2055394599584437, -0.097543020465043817, -3.0523343799116924, -2.3859400260784431, 3.2361448856932284], [3.0745847802705146, -1.1417766963847393, 4.6204662978393785, 5.3840845901077774, 5.6991830604689664]], [[-0.68530411061757679, -0.63015693088727609, -0.34801667265478908, -2.7833833328118018, -2.843600029060581], [-5.0682231022347892, 0.40406327797366881, -1.4585240067902809, 0.82574531772579896, -2.6629054304559903], [-0.93305958260155286, -0.70447409763679225, -1.8462359940443926, -4.0419367179192207, -1.4905372514958257], [-5.2781521190915051, -1.1889477908513069, -5.4515674105673382, -4.9628660001267537, -3.8925394750366147]]], [[[6.2341870891826501, 4.1871685855121177, 1.3882983635028117, -1.5524498541740459, -1.4982752534822517], [4.9952843637682429, 0.20576679049292501, -1.2107818836693682, -1.9169043670067287, 5.287750463707213], [0.10258612702637526, 3.1984845508414494, 6.602784702683647, -0.94425415709028471, 3.8080563519855275], [6.5904540043162827, 1.988770198629255, 3.5820360858416143, 6.4729635786074917, 4.2134512810681812]], [[3.399614505808219, 6.6034601849280516, 2.1528094174229202, 4.8637851376901864, 5.4047854675673257], [6.2486431615796523, 3.2193267366747884, 9.6496388555127233, 3.7678428313836116, 2.4224204203401971], [3.423715113653663, 7.7689691396862974, 8.9721025404457944, 0.40621657234908959, 3.3891153146463977], [1.464198903337147, 1.7256228141204661, 6.5019059708391795, 0.90688533709486441, 9.2215187434467918]], [[0.76512784578794957, 6.4133477706207103, -2.8078952216984163, -1.7782813782640892, 0.91626319683602286], [-0.26677321551107358, -2.2215551751479179, 2.6110867605097745, 6.7202807935873334, 4.957816679808877], [1.2187206570923532, 0.99645283204726631, 2.1339620731314302, 6.1257436633043323, 0.69058295659536917], [3.4530114000375498, -1.839764785935508, -1.8762666498667944, 0.25048541868775853, -2.9117092796155912]], [[-4.6141235947606738, 0.36659606842348857, -1.4758399318679345, -1.5007424087116181, -5.299078362274888], [-0.17389374624738885, -0.3614862154979086, -4.2855023234169449, -8.1806994041675054, -0.93748813293047828], [-8.8125175009122891, -2.7727133777979569, -4.3280229064209159, -3.8085217630849222, -8.5027325934377895], [-6.3285009542984261, -8.1841438180722577, -4.1724401235097535, -7.4887848715133742, 0.04539145920029064]], [[-7.9272731679301387, -2.1888285301044177, -5.8100861078920989, -6.1425341989286473, -1.3885528065214694], [-8.0431598226746619, -5.1755585208353931, -7.8921435002470179, -3.3300209606075684, -0.55930371746078444], [-0.60519173050507513, -0.038120390404067894, -4.7230896977628429, -1.0792700824335459, -0.564534679904106], [-6.9787459441815241, -6.8206086197891516, -5.3364829995239171, -7.4327807271135953, -8.3917679352423473]]], [[[1.1750647352539847, -3.3406823755602435, -0.33087719945939131, 0.43946742188400822, 0.0047652405774130102], [-1.9060851365465812, -5.5885095152018023, 3.0309206895198866, 2.8640410424362219, -0.53912945440689519], [-2.5184669878615811, -3.8680309039033407, 2.1861478630659015, -2.0887710033472544, -5.1082664137970788], [0.76002414683390906, 1.7704190428220761, 0.13656011844489413, -1.9500532947797247, -5.8872534736346704]], [[2.0967880371870251, -1.5356122108563866, 3.2131890121732196, -1.2021374027645972, 2.4745913809569435], [6.023715171639938, -0.76691225491015613, -0.2020299741921594, -2.0302676206921757, 2.8598377502329129], [-0.96757387721995292, 1.8481390095583103, -3.797386910431273, -0.89689418508503582, 0.79417637343081449], [5.3767330640908879, -2.2796592916245042, -2.8039765107644934, 3.8715764601036913, 2.9137374675377812]], [[-1.5293172110192987, -4.1979370773798257, 2.0862637682359599, -4.02821781267067, -3.5627245937320948], [-6.3879616889985149, -2.550500665468876, -1.4090561331315179, -2.0690027703264127, 0.3363186976720316], [-2.2310587507889945, -7.2744131278823856, -1.347193521584658, 1.5502664030603954, -3.1355148417612506], [1.4427558187078882, -1.5885893272452565, 1.1091400655591803, -3.8640851451442795, 1.2189316948154518]], [[-8.5845233348503847, -4.4980520453390049, -2.4605211937085452, -9.1147237861144674, -9.068229544407254], [-2.7679260567582862, -1.6406384771010565, -2.2348149096987804, -2.5292279435507687, -4.7516768331979593], [-3.1357682123723922, -8.3434320580460941, -0.40863691855251716, -0.35649468952861874, -9.6318425701959072], [-4.6223305916676889, -6.8162556258826763, -1.6496086092399675, -7.3355345007190467, -6.0933053790091041]], [[-3.4318976510128154, 3.5792052656250242, -0.48795699975170681, 0.16764488316842829, 5.0236881674408487], [2.8911761534530687, 0.93558478963500313, 3.4315583775797585, -1.6213809796058616, 5.1637352020816181], [-2.9964233473998334, 5.628365296898501, -1.7899165274418687, -0.037488611821620665, -1.7923310488055568], [-3.4599459734037401, 5.9865417629770104, 5.6839983284325157, 1.3519241729350355, 2.4559895401941327]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.022344908729786894, 3.0413504040628201, -2.5863329634469601, 0.72487156186797463, -1.3100773605715279], [0.98454162858794181, 2.7519682716632321, 4.6589831668984925, 1.5652475213572536, -2.5685934505932861], [-4.180636161348418, -3.2447902643082882, 2.7689332722744222, -0.49900535370802857, 2.7821364253556125], [-2.8727435739127394, 4.3380870086711472, 3.1305983948063663, -3.8778869428958016, -4.4737975013969322]], [[-2.8680464678804709, 2.9136400484536029, -1.6670742654146053, -0.852358110067307, 3.5764401478845542], [2.94725210646161, 0.51629016320075394, -0.88976735689955877, 3.750967012688097, 1.2521594035005226], [-3.3499512324256209, -1.2924235532888275, 0.31729248705529312, 1.9309434094548017, -2.9173539567497553], [-4.1564971364364762, -2.0430874315417724, 1.6003123756110931, 1.3809953699297752, 2.0003092080315739]], [[-1.8187142063989148, -1.0454248991094115, 4.3314806604966023, 3.1828448554635571, 1.4709862672690335], [0.088449985708171752, 4.6637487724340474, -4.7263859308671616, -0.35397284834183829, -3.0841739242052424], [4.1340222952714516, -3.7291291237887627, 0.53027874552281062, -4.5928608562342221, -1.2728547591059538], [-3.9798978235994831, 4.2104591623838505, 0.038393269594737767, -1.3998667734803183, 4.6935602354578538]], [[3.7076541743102229, -3.577931922995524, -4.1337497086124184, -2.6610768073053457, -2.7926080687571977], [3.4140435378197829, 1.9672424438774101, -1.116617264141444, 0.39308830329442213, 1.9947320557288215], [4.3910669409295995, -2.4525133090132458, 4.4795559462381256, -3.3717078910342844, 4.9333252352484767], [3.5561744349941655, 0.16386912885224625, 4.0943415187491219, 4.2337563200728905, -2.5880634996242504]], [[3.9501870347798071, 2.7927801312812575, 2.6062937408773976, -1.0695462565091129, -4.2578695382943179], [3.230331201893982, 2.3754914821840849, -3.9134952002789927, -0.12988844792820498, 4.9008183768156872], [-1.2069757135602934, 2.5613702379390277, -1.7659157471540388, 2.6061664103091742, -2.2519309395818823], [-3.0123233987261653, 1.728660398668131, 0.0026843764328390307, -0.55708753977692727, -2.6095011066195415]]], [[[3.5107522557185202, 3.9253655091430719, 3.6643668720190146, 4.9608433373230625, -3.1393341126786667], [2.7312122497038969, -0.37357423637107168, 1.1787024089028915, -0.16922488922300261, -1.1894234781584934], [0.3295772726480255, -3.9013180555898552, 0.97979447971265721, 4.8304751228193208, 4.0915175558290322], [-4.8811266290897306, 1.9621435677157484, 3.6798124010705155, -0.8305375373702697, 2.2051992370589959]], [[-3.4922339708113204, 1.1657739332873742, 1.8247338200490368, 0.11573512248062112, -0.9728900544202439], [4.2675227999691305, 4.264125830908549, 4.196845742212842, -0.22314961478097395, -0.29266126658562452], [-0.53241292705566234, -0.77331914404176061, -0.39941296423490513, -0.23113576432980487, -1.1733377688830258], [-4.2124535974010255, -3.1046408938002488, -4.8025731596344272, -0.61476551675793445, 1.6982564774059981]], [[-2.5506953748575523, -3.8646248847291984, -1.8167198513542546, -2.1552422028895011, -0.29247802963513347], [-0.92225452248947715, -4.7470982008550253, 0.2817567958730347, 1.8796476727276445, -2.2632877378081337], [0.96254211065907924, -0.070157344921999432, -4.3645099376626808, 0.071225636973339057, -2.5793653953749116], [3.7243372765537988, -0.4740956141443311, 4.02640510750925, -2.1294025657307358, 2.54869866772008]], [[-3.6630090126221324, -0.0087467053048566612, 0.57300269252003133, 0.32062360938958356, 3.522372023604138], [4.0402313717114922, 1.983093416794544, -1.3034100259542503, 3.5473954377199917, -3.0532301755080571], [-0.31542136850002311, 4.9580895034750601, -1.918349253749061, -0.76911932292514162, 2.9888587674555929], [0.75720427602701168, 0.48404376129187199, 4.8674933140121706, -2.3550326603172755, 3.6871235415727703]], [[-1.6344391294916449, -1.3923003863612471, 3.6268159037149736, -0.63066044932412169, 0.54771588033710117], [-3.7424259460909779, -2.5910883144491716, -1.7357167394064357, -1.7263891337918147, 3.4678434224291834], [-0.4682652428416727, -1.7889019079324839, -0.59158509258169989, 1.3561406412389889, -1.0225953980475513], [0.038479187733196696, -0.27666506538987168, -1.111164485605264, 1.3865920210712543, 0.22108760758697343]]], [[[-3.6228082784653237, -3.8969041582991828, 3.7657420107632351, -0.68420320731291806, -0.30624116781073951], [1.0180093251936784, 2.1090564724654737, 4.6874796302318149, -3.0202790809051239, 0.7076243670686031], [0.26481266214382337, 3.0263050541086471, -1.7323693842100463, 0.80734758101047444, 4.5108477676959637], [-4.6288475797375011, 4.587885154809646, 1.7252393186590753, -3.8718420445123902, 0.77846610321236298]], [[-0.51699071401659058, -2.7296075293543431, -4.3762651568692661, -4.7841080891729284, -1.142130941323555], [4.9988469938749667, -3.4388350517028599, 0.47470643442977689, -3.0727795744438202, 1.0667745235837902], [-3.7019196962689973, 1.918141311290408, -4.0842186213475777, 1.1749471087166334, -4.8930936990828737], [-2.180489261869397, -3.17455140693761, -0.5541574312396369, -4.9694691485982325, -2.9762894111033167]], [[-3.5054063305903438, 1.0729697480655407, -2.0553048762355051, -4.005896535898219, 4.3876028075773945], [1.2165549863240841, 4.038286778008116, 1.4569538007997842, -2.1741700986544799, -2.8915342629437091], [4.2180183235549311, -4.0044825949927834, 0.62183175406761126, -0.58751854550275961, -3.0940154031157263], [-4.2073883789759456, 4.4772024242399073, -0.70922192439076781, -4.4714791009359711, 1.8796441310554588]], [[4.6978256461590959, -0.12884729596520472, -1.1780704588616198, -0.38241648681731544, -3.382488576168019], [-2.6375057261917534, 3.996923442082668, 1.2235065984792506, 2.1184102862551857, 0.46197511442994799], [-0.10802088020573564, -0.57145634922330757, -1.7667119821208086, 3.5473515870637264, -1.7390795680717828], [0.44068635282534263, -1.6137164855134856, -3.6766204095638564, -1.1786865221679665, -2.0850497512805144]], [[3.5965255109292684, 4.9135224394302401, -1.4581644469788158, 1.2532528469880706, 1.4381329354996888], [1.7615392971995716, 4.4562856521797389, 3.8502296942012038, -4.7694149962344952, 4.3409056837840811], [4.5533859651173376, -0.070909115146509549, -1.7086777123703643, -1.9189246525112127, 0.60718172442807994], [0.52886313100217297, -0.66446429836933696, 0.015070754210065651, -2.8345360983410197, 0.26156627741132166]]], [[[-2.7528747760857897, 4.2314457225093385, -2.4281514214658664, 2.9368769887278798, -2.2920848141132222], [3.1550239361026211, 2.2998748419419899, -0.66590870555172099, -4.1027798532258855, -4.2320224343538992], [0.19635044603275631, 2.7747214135513971, -3.6676501415220066, 2.7957421986143016, 0.1551449195805521], [3.49104796422872, 2.3233613992611035, 4.3196557506019797, -4.2224999456977521, -1.4829630698527909]], [[3.8770591031242816, -0.76252331378567817, 4.5643141507356368, -2.6334024450485272, -1.1532143801393957], [-0.77689639835234114, 2.4584238893928712, 1.0098519932284242, -0.24351952332636451, -4.5366593882864681], [3.8938781252830985, 2.9528873140918668, 2.9660511124786018, 2.5263066978387796, 4.5942921404887329], [3.0036360275791871, -0.74791876890648368, -4.0661996123001174, 1.5011579601731668, 1.5887171200856027]], [[-0.63920706595939514, -3.6196703599123503, -3.412450054773025, -1.0016679934911954, -3.1838863791876504], [3.0219924124461652, 2.3478148524364926, -0.84838899586464755, 3.5967350950865455, -2.8725086600412411], [-3.8142784350788608, 3.321055535969931, -1.5478542347640198, -0.48556694583548587, -3.9760992809270301], [-3.0107278741371712, 0.68437923344851992, -2.4361579105435904, -2.8044892800779508, -2.9836311033486496]], [[1.323268092028834, -3.497607198293152, -3.7471764148493092, 4.5399036650415638, 1.7000453794527211], [-0.19783086849616893, -3.5112225763369089, -4.9272493641958821, 0.82064305092697865, 2.5227602566016971], [-0.22267360437954764, -4.2541394204345648, -0.57842920858915825, 3.2662102111628997, 1.3365420491176137], [-0.45779722166939152, 0.7014175332021253, -2.4792771488097332, 3.2712724125797088, -0.22185512603959801]], [[-2.3429298199101236, -4.8107760758557347, 3.8408229674810421, -0.48506330176434442, -4.6734605150702002], [3.4360714467234761, -1.8702662733360031, 3.8322580967830504, 4.3048763806652541, 4.7630981538565926], [3.4679919661477356, -4.0442375774078627, 2.0577702921750802, -3.9455436380877185, 0.86417099821807319], [-3.6530297386343147, 0.60102861087852677, -1.7673628658714446, 2.6182389824725716, -4.7738113127227866]]]])+(1.-msk_arg1)*numpy.array([[[[-1.49062069524736, 2.0149416859603475, 0.18976355179996141, 4.2724461581338797, 3.9337654521090197], [2.2269021732115215, 1.4358908083721476, 2.5276665104956786, 2.3662403822646976, -2.289446115907936], [-3.3606922135322237, 2.1308851110821037, 3.2647711181790164, 1.0535160911587855, -0.53556966460772149], [-0.47204082695447802, -0.46813236892537446, 1.0856875116419049, 1.0480588764837515, -2.62332781797718]], [[-1.8962034298937382, -2.204588536303782, 2.5923954774481262, 4.1243588878075084, -1.1928850488279172], [0.61520727358927108, 1.6331405622311088, 4.5718115323446433, 1.2672005495616636, -3.6620712744600095], [-4.7823706630703757, 2.6941051829465748, 2.5907975731265074, -2.3048498390493011, 0.027667612731923974], [3.6681716762264074, -3.3105366849571927, 2.0460764188957361, 1.1118548644269239, 2.3825252141373827]], [[-3.3292774039995376, 4.2231783451884777, -4.856507442637227, 0.63294181469767175, 3.5344396521743029], [0.82904108070720905, -0.39715027777442824, 3.1530051728308433, 2.6721373112452218, 0.47772695051753544], [-1.2850964436170775, 0.55508887780183436, -2.7650310887696428, 0.77004661954410203, 3.7573886099925833], [0.3150557511888552, 3.1697231963109758, -1.8070885388135292, 1.5577123814004565, 2.268503940573722]], [[-2.5444117190777349, 1.6372614979549347, -2.2957191044421297, -2.419184755243029, -3.2015916926362831], [-0.79626361281918534, 0.032199923506443184, -4.0144720662157312, -3.4429201140492403, 4.2881892645130684], [2.8636352200863913, 2.2745047258335642, 4.5254084292312644, 4.674637314430143, 3.1065426373164264], [0.51613970987204905, -2.6047215468892482, -0.56628666463032928, 1.7885183841621721, -3.3535673805062514]], [[2.6496722492706217, 0.33772601217629017, 3.0806885452291493, -1.7922016340426374, -1.7147619863367449], [-4.4493301360666919, -1.9771516679726373, -4.4895464538610756, -1.607328247698284, -3.0039452052176796], [-4.6824747864588261, 4.0907246951921774, -1.110490222189874, 0.45491762141096359, -4.5374245521296324], [2.7342842706111936, 4.9265245137126339, -0.45463238652757898, -4.231869997567923, 2.5367693680968264]]], [[[-2.5914528798204381, 0.98805318977971712, 3.6125967281046751, 1.0369452048730867, -3.9230482186637685], [-4.2946478434985371, -4.8087278479104523, 3.8515668775089615, -0.98964432643235689, 3.7069788396637744], [-2.3004298660942348, -1.3165957609076462, -2.3325806617788523, -3.0339267724483987, -2.3358244890052573], [3.9825413252400708, -1.3354000224138041, -1.1707902781113844, -1.7957968300936344, 0.0099242056698702896]], [[3.4287520011889256, 0.26015509170991269, -4.5774665126335856, -4.7851832299899009, -2.0275223540339682], [-0.42004504585412761, 1.4304021191279839, 0.20473109279463575, 3.6144465863710131, 0.14061144029687433], [-0.76426180685109379, 0.360815396149059, -0.54506294058372795, -2.3038405735633596, 2.9954291371858242], [1.8443084455041445, 0.52782022472524659, -3.5480111618384278, -2.1557472289116109, 1.76855256416217]], [[2.0201021141798918, 3.6190338746788679, -2.280893673190322, -1.9239097405999028, -3.8473790713403542], [3.6026842680748814, -4.8525760323250458, 3.7403307752316444, -1.0697112470898107, -0.43412864544270402], [2.5675386371236186, -2.5852086421632214, -1.9032982030548986, 2.999746634131033, -2.8238259058997217], [-2.4409611039433865, -4.7452944272414417, -3.8308579274353494, 2.4834469712922207, -4.5325037966516843]], [[4.7858105681975864, 4.7283102160050756, 4.1132088215139824, -1.4244768845924192, 2.6533099649217835], [-2.2987149306196408, -1.8792509296458446, -1.7199525096204016, -0.11579822009652663, -0.35640266399734877], [-0.31709731969766963, 4.6735854271171853, -2.8187036987949932, -0.079355200502089218, 2.6035037644830794], [-4.4236621663710771, 0.15088214043191961, 1.3068300967863253, 0.64671846201272221, 1.1504200639557531]], [[3.1746807199575287, -1.2746300985240113, 0.58767150872204788, -4.3939695111082635, 3.0074043359550693], [2.1062664809905343, 3.8549947416911081, 0.54903810038889489, 4.6382122455851089, 4.3059813213720606], [3.7996479855821423, -0.28471047366511204, -0.8066923659554881, -4.9705899943135501, -4.4064950079692808], [-4.8052513351553836, -3.8718305102883344, -0.16132963306775583, -0.78184714448417214, -2.1708825292151146]]], [[[-1.7227000249066284, 0.15991030192783917, 1.1272549127048981, 4.4956820193672939, -3.0780642518612797], [-0.91328359229572698, -3.2144303939086551, 4.312098029813491, -4.9071570001150908, -2.1675951963591666], [-3.0867795956347277, 1.5164844866849645, -1.0813128949775663, 2.188950602895428, 0.53188375569034818], [-4.5015922947517195, -4.1174667312161537, 4.716334874609128, 3.1704055486806109, -3.0578022544050256]], [[-2.0341921486248813, -3.2995018559288813, -2.783772044562741, -1.2607996679388753, 2.0482063967171724], [3.0867994523135316, -4.8267818407730347, -4.4112973614035234, 2.674650792229686, -3.0863769664656813], [-0.72043572927705846, -2.9274851967043212, 3.6548596021918129, 2.4887016532175732, 4.8472155774390124], [-3.616901823690275, 2.5576761056674355, 2.2066324864281297, -2.1861999704593602, -0.62332431037061564]], [[1.5652773254113574, -0.86456477183022429, -4.3047569182059302, -1.5273784953052925, 4.3421637027372242], [4.3393184239000053, -2.6590007394225523, 0.30992215317189853, 3.8260969136276941, 4.2447579987425481], [3.7123579575079155, 3.8484544913843948, -1.8830150633136178, 1.398476999977464, -2.7902136052954551], [1.4345472361400837, -3.4922759046402252, 1.5256913686897269, -4.7680542717489427, -1.8559054263645312]], [[-2.0217766308954332, -3.3992567897605985, -2.4199654725328501, 1.3475560266024056, 2.9606432057848338], [3.7403224164568591, 1.2193853167316879, 3.0256711724576029, 4.8915673906656814, 4.442758505003539], [-2.8761474645756802, 1.2461107123726558, -4.9494256380418449, 4.797430091985591, -1.4756698032937554], [-4.8315932654577418, 3.3447426480303584, -2.6290614368195522, 2.18388358921273, -4.0459175985241611]], [[-1.7486570862420026, 2.2327848342887862, -2.1265443856688493, -2.0279359495798301, 3.3263172415205062], [-4.2412266602689588, 2.6356662375631936, 2.0787605726737413, 1.3161759561390465, -2.0920627035028829], [4.1826052611000435, -0.5814281633301821, -4.6765183128813952, 2.1751935273306247, 4.8377470583433144], [4.9749896352903047, 4.7749361280535076, 1.4950615833612186, -3.848857572130794, 0.082939643769592841]]], [[[-2.8571333787755027, -3.2787409650399679, 2.0055389092603901, -0.2486615106866461, 1.2393697228901255], [-2.6316476939999967, 3.1741051608672173, -0.076606079039356345, 2.7623411023994731, -3.7818680185605702], [0.1786375686916859, -4.1590064353565328, 0.12667785414479571, 1.84476080648282, -3.3109235973074389], [-2.8671909300402474, 2.0781361585961333, 4.107079643079496, -0.74569592421111075, -3.2384933851221209]], [[-3.697391323708763, -0.0042939663126642102, -1.4209581679651984, -2.7741994958101901, 3.8249512505518108], [-3.3647967331262461, 3.6663685769436736, -4.4404730327146691, -2.7425049877495331, -4.8356446650133229], [-4.4517197610761103, 3.1024429903694735, 4.8839696362426217, -3.3940629008500247, 0.79379247637380157], [-0.29456299740177716, 4.8921270237299037, 0.94677916188062738, 1.5688164065148849, -3.0732838394751916]], [[-4.4660109330615425, -1.2898851050209812, 4.450008542148284, 1.3664466815833345, -2.0376060203343127], [1.5647595631161462, 1.4899525637231275, 3.7636402821520711, -3.2992179948104763, 2.4579488556173699], [1.9261869151805202, 0.74155826174962591, 2.4117382163805168, 0.2312286204275722, 1.2088842379512741], [3.0203217585008986, 3.6993214504583491, -4.2427320461557514, -4.8181474444287051, -1.3705449609920772]], [[-3.0737325251822636, -4.4100212734852571, 4.4084755764206882, -0.32568201344652259, 2.0752073822926267], [1.4124813992411367, -0.24787210703287066, 4.1137964510892324, -4.8714622174747539, -1.3121923326749618], [-4.1871152294746281, -0.80810426208867625, -3.6862209133412316, 4.2750583665010193, -3.6417259263704906], [2.1422753060742679, -4.5230029236132232, 0.2753720045038115, 1.1602409182542868, -2.8402739888145065]], [[-0.25935625287471353, -1.6847016774444166, 2.6775751104178074, 0.50920094086852252, -1.5388498727311104], [4.0533222336173154, 2.5200955988275489, -3.6234034600878227, -3.7997353668892564, -0.9862246159200474], [2.6062420909541109, 4.9357123507912455, 0.60276249362790502, 2.0960594466861497, -4.561154589162733], [-2.4569859860887213, 3.3618747420252628, 1.335121321556997, 4.2185614299586884, 1.4255453957681432]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[-2.1989852566012393, -2.5394810070190332, 4.1609465882544381, 0.58940182119406082, 2.3682368024748568], [2.2708091028953028, -4.9009800547178823, -1.3930211292681305, -0.26471081136291463, 3.0402178923636676], [-4.9855561825120258, -3.2415419354984087, 3.826453799395658, -1.2705258882173256, 2.0388623600936153], [-4.615749553401221, 1.6753056368052368, 2.3555610999099823, 0.91195956221992347, 2.3136997297643758]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.2213301653310262, 0.84236514746158075, -4.7853182200481994, -1.4741136947332647, -3.5090626171727672], [-1.2144436280132975, 0.55298301506199277, 2.4599979102972531, -0.63373773524398569, -4.7675787071945255], [-6.3796214179496573, -5.4437755209095275, 0.56994801567318287, -2.6979906103092679, 0.58315116875437312], [-5.0717288305139787, 2.1391017520699078, 0.93161313820512692, -6.0768721994970409, -6.6727827579981716]], [[-5.4075274748995046, 0.37415904143456968, -4.2065552724336381, -3.3918391170863402, 1.036959140865521], [0.40777109944257672, -2.0231908438182793, -3.429248363918592, 1.2114860056690637, -1.2873216035185107], [-5.8894322394446537, -3.8319045603078608, -2.2221885199637401, -0.60853759756423154, -5.4568349637687881], [-6.695978143455509, -4.5825684385608056, -0.93916863140794016, -1.158485637089258, -0.53917179898745937]], [[2.3422323818555233, 3.1155216891450266, 8.4924272487510404, 7.3437914437179952, 5.6319328555234716], [4.2493965739626098, 8.8246953606884855, -0.56543934261272355, 3.8069737399125998, 1.0767726640491957], [8.2949688835258897, 0.43181746446567537, 4.6912253337772487, -0.43191426797978405, 2.8880918291484843], [0.18104876465495501, 8.3714057506382886, 4.1993398578491758, 2.7610798147741198, 8.8545068237122919]], [[4.2970559955042837, -2.9885301018014632, -3.5443478874183576, -2.0716749861112849, -2.2032062475631369], [4.0034453590138437, 2.5566442650714709, -0.52721544294738321, 0.98249012448848294, 2.5841338769228823], [4.9804687621236603, -1.863111487819185, 5.0689577674321864, -2.7823060698402236, 5.5227270564425375], [4.1455762561882263, 0.75327095004630706, 4.6837433399431827, 4.8231581412669513, -1.9986616784301896]], [[6.3184238372546639, 5.1610169337561143, 4.9745305433522544, 1.2986905459657438, -1.8896327358194611], [5.5985680043688388, 4.7437282846589417, -1.5452583978041359, 2.2383483545466518, 7.269055179290544], [1.1612610889145634, 4.9296070404138845, 0.60232105532081803, 4.974403212784031, 0.11630586289297451], [-0.64408659625130849, 4.0968972011429878, 2.3709211789076958, 1.8111492626979295, -0.24126430414468469]]], [[[5.781561358613823, 6.1961746120383747, 5.9351759749143174, 7.2316524402183653, -0.86852500978336389], [5.0020213525991997, 1.8972348665242311, 3.4495115117981943, 2.1015842136723002, 1.0813856247368094], [2.6003863755433283, -1.6305089526945524, 3.25060358260796, 7.1012842257146236, 6.362326658724335], [-2.6103175261944278, 4.2329526706110512, 5.9506215039658183, 1.4402715655250331, 4.4760083399542987]], [[-8.3932140255292023, -3.7352061214305081, -3.0762462346688455, -4.7852449322372612, -5.8738701091381262], [-0.63345725474875181, -0.63685422380933332, -0.70413431250504033, -5.1241296694988563, -5.1936413213035069], [-5.4333929817735447, -5.6742991987596429, -5.3003930189527875, -5.1321158190476872, -6.0743178236009081], [-9.113433652118907, -8.005620948518132, -9.7035532143523096, -5.5157455714758168, -3.2027235773118843]], [[-3.9437165041256828, -5.2576460139973289, -3.2097409806223851, -3.5482633321576316, -1.685499158903264], [-2.3152756517576076, -6.1401193301231558, -1.1112643333950958, 0.48662654345951406, -3.6563088670762642], [-0.43047901860905124, -1.4631784741901299, -5.7575310669308113, -1.3217954922947914, -3.9723865246430421], [2.3313161472856683, -1.8671167434124616, 2.6333839782411195, -3.5224236949988663, 1.1556775384519495]], [[-3.927719823985047, -0.27345751666777129, 0.3082918811571167, 0.055912798026668931, 3.2576612122412234], [3.7755205603485775, 1.7183826054316294, -1.5681208373171649, 3.282684626357077, -3.3179409868709717], [-0.58013217986293775, 4.6933786921121454, -2.1830600651119756, -1.0338301342880563, 2.7241479560926782], [0.49249346466409705, 0.21933294992895735, 4.6027825026492559, -2.6197434716801902, 3.4224127302098557]], [[1.4057787628720226, 1.6479175060024205, 6.6670337960786412, 2.4095574430395459, 3.5879337727007687], [-0.70220805372731032, 0.44912957791449593, 1.3045011529572319, 1.3138287585718529, 6.508061314792851], [2.5719526495219949, 1.2513159844311836, 2.4486327997819677, 4.3963585336026565, 2.0176224943161163], [3.0786970800968643, 2.7635528269737959, 1.9290534067584035, 4.4268099134349219, 3.261305499950641]]], [[[-8.6083644609773486, -8.8824603408112086, -1.2198141717487907, -5.6697593898249439, -5.2917973503227653], [-3.9675468573183474, -2.8764997100465521, -0.2980765522802109, -8.0058352634171506, -4.2779318154434227], [-4.7207435203682024, -1.9592511284033787, -6.7179255667220721, -4.1782086015015514, -0.47470841481606207], [-9.6144037622495269, -0.39767102770237983, -3.2603168638529505, -8.8573982270244151, -4.2070900792996628]], [[-3.7585326495149993, -5.9711494648527523, -7.6178070923676753, -8.0256500246713376, -4.3836728768219633], [1.7573050583765579, -6.680376987201269, -2.7668355010686319, -6.3143215099422285, -2.1747674119146185], [-6.9434616317674056, -1.3234006242080008, -7.3257605568459869, -2.0665948267817753, -8.1346356345812829], [-5.4220311973678061, -6.4160933424360191, -3.7956993667380456, -8.2110110840966417, -6.2178313466017254]], [[0.32104746880531421, 4.8994235474611987, 1.7711489231601529, -0.17944273650256104, 8.2140566069730525], [5.0430087857197421, 7.864740577403774, 5.2834076001954422, 1.6522837007411781, 0.93491953645194892], [8.0444721229505891, -0.17802879559712537, 4.4482855534632693, 3.2389352538928984, 0.73243839627993168], [-0.38093457958028765, 8.3036562236355653, 3.1172318750048902, -0.64502530154031312, 5.7060979304511168]], [[3.4272997579417703, -1.3993731841825303, -2.4485963470789454, -1.652942375034641, -4.6530144643853451], [-3.908031614409079, 2.7263975538653424, -0.04701928973807501, 0.84788439803786009, -0.80855077378737761], [-1.3785467684230612, -1.8419822374406332, -3.0372378703381342, 2.2768256988464008, -3.0096054562891084], [-0.82983953539198296, -2.8842423737308112, -4.947146297781182, -2.4492124103852921, -3.35557563949784]], [[5.6353878710228837, 6.9523847995238555, 0.58069791311479957, 3.2921152070816859, 3.4769952955933041], [3.8004016572931869, 6.4951480122733543, 5.8890920542948191, -2.7305526361408798, 6.3797680438776965], [6.5922483252109529, 1.9679532449471058, 0.33018464772325107, 0.11993770758240263, 2.6460440845216953], [2.5677254910957883, 1.3743980617242784, 2.053933114303681, -0.79567373824740439, 2.300428637504937]]], [[[-7.3686243294870106, -0.38430383089188247, -7.0439009748670873, -1.6788725646733411, -6.9078343675144431], [-1.4607256172985998, -2.3158747114592311, -5.2816582589529419, -8.7185294066271055, -8.8477719877551202], [-4.4193991073684646, -1.8410281398498238, -8.2833996949232276, -1.8200073547869193, -4.4606046338206689], [-1.124701589172501, -2.2923881541401174, -0.29609380279924125, -8.8382494990989731, -6.0987126232540119]], [[5.5523647399295184, 0.91278232301955864, 6.2396197875408737, -0.95809680824329035, 0.52209125666584111], [0.89840923845289566, 4.133729526198108, 2.685157630033661, 1.4317861134788723, -2.8613537514812313], [5.5691837620883353, 4.6281929508971036, 4.6413567492838386, 4.2016123346440164, 6.2695977772939697], [4.6789416643844239, 0.92738686789875313, -2.3908939754948806, 3.1764635969784036, 3.2640227568908395]], [[1.7163540339505872, -1.264109260002368, -1.0568889548630427, 1.3538931064187869, -0.82832527927766808], [5.3775535123561475, 4.7033759523464749, 1.5071721040453347, 5.9522961949965278, -0.51694756013125875], [-1.4587173351688785, 5.6766166358799133, 0.80770686514596246, 1.8699941540744964, -1.6205381810170478], [-0.65516677422718894, 3.0399403333585022, -0.080596810633608129, -0.44892818016796854, -0.62807000343866726]], [[2.2352276542487575, -2.5856476360732286, -2.8352168526293857, 5.4518632272614873, 2.6120049416726445], [0.71412869372375454, -2.5992630141169855, -4.0152898019759586, 1.7326026131469021, 3.4347198188216206], [0.68928595784037583, -3.3421798582146414, 0.33353035363076522, 4.1781697733828231, 2.2485016113375371], [0.45416234055053195, 1.6133770954220488, -1.5673175865898097, 4.1832319747996323, 0.69010443618032546]], [[-0.029230090145747845, -2.497076346091359, 6.1545226972454179, 1.8286364280000313, -2.3597607853058244], [5.7497711764878519, 0.44343345642837262, 6.1459578265474262, 6.6185761104296299, 7.0767978836209684], [5.7816916959121114, -1.7305378476434869, 4.371470021939456, -1.6318439083233427, 3.177870727982449], [-1.3393300088699389, 2.9147283406429025, 0.54633686389293112, 4.9319387122369474, -2.4601115829584108]]]])+(1.-msk_ref)*numpy.array([[[[-3.6896059518485993, -0.18404357064089183, -2.0092217048012779, 2.0734609015326404, 1.7347801955077804], [0.027916916610282172, -0.76309444822909178, 0.32868125389443925, 0.16725512566345824, -4.4884313725091758], [-5.5596774701334635, -0.06810014551913568, 1.0657858615777771, -1.1454691654424538, -2.7345549212089608], [-2.6710260835557174, -2.6671176255266138, -1.1132977449593344, -1.1509263801174878, -4.8223130745784193]], [[-4.435684436912771, -4.7440695433228157, 0.052914470429092919, 1.5848778807884751, -3.7323660558469505], [-1.9242737334297622, -0.90634044478792442, 2.0323305253256101, -1.2722804574573696, -6.2015522814790423], [-7.3218516700894085, 0.15462417592754152, 0.051316566107474149, -4.8443308460683348, -2.5118133942871093], [1.1286906692073742, -5.8500176919762259, -0.49340458812329713, -1.4276261425921093, -0.15695579288165051]], [[0.83166918425490044, 8.3841249334429158, -0.69556085438278892, 4.7938884029521098, 7.695386240428741], [4.9899876689616471, 3.7637963104800098, 7.3139517610852813, 6.8330838994996599, 4.6386735387719735], [2.8758501446373606, 4.7160354660562724, 1.3959154994847953, 4.9309932077985401, 7.9183351982470214], [4.4760023394432933, 7.3306697845654138, 2.3538580494409089, 5.7186589696548946, 6.42945052882816]], [[-1.9550098978836741, 2.2266633191489955, -1.7063172832480689, -1.8297829340489682, -2.6121898714422223], [-0.20686179162512452, 0.621601744700504, -3.4250702450216703, -2.8535182928551794, 4.8775910857071292], [3.4530370412804521, 2.863906547027625, 5.1148102504253252, 5.2640391356242038, 3.6959444585104873], [1.1055415310661099, -2.0153197256951874, 0.023115156563731531, 2.3779202053562329, -2.7641655593121905]], [[5.0179090517454785, 2.705962814651147, 5.4489253477040061, 0.57603516843221936, 0.65347481613811187], [-2.0810933335918351, 0.39108513450221949, -2.1213096513862189, 0.76090855477657282, -0.63570840274282281], [-2.3142379839839693, 6.4589614976670342, 1.2577465802849828, 2.8231544238858204, -2.1691877496547756], [5.1025210730860504, 7.2947613161874907, 1.9136044159472778, -1.8636331950930662, 4.9050061705716832]]], [[[-0.32064377692513535, 3.2588622926750199, 5.8834058309999779, 3.3077543077683895, -1.6522391157684657], [-2.0238387406032343, -2.5379187450151495, 6.1223759804042643, 1.2811647764629459, 5.9777879425590772], [-0.029620763198932032, 0.95421334198765662, -0.06177155888354946, -0.7631176695530959, -0.065015386109954498], [6.2533504281353736, 0.93540908048149873, 1.1000188247839184, 0.47501227280166836, 2.2807333085651731]], [[-1.4722280535289567, -4.6408249630079697, -9.4784465673514688, -9.6861632847077832, -6.9285024087518501], [-5.3210251005720099, -3.4705779355898985, -4.6962489619232466, -1.2865334683468692, -4.760368614421008], [-5.6652418615689761, -4.5401646585688233, -5.4460429953016103, -7.2048206282812419, -1.9055509175320582], [-3.0566716092137378, -4.3731598299926357, -8.4489912165563101, -7.0567272836294936, -3.1324274905557123]], [[0.62708098491176134, 2.2260127454107375, -3.6739148024584525, -3.3169308698680333, -5.2404002006084847], [2.2096631388067509, -6.2455971615931762, 2.3473096459635139, -2.4627323763579412, -1.8271497747108345], [1.1745175078554881, -3.9782297714313519, -3.2963193323230291, 1.6067255048629026, -4.2168470351678522], [-3.833982233211517, -6.1383155565095722, -5.2238790567034794, 1.0904258420240902, -5.9255249259198148]], [[4.5210997568346718, 4.463599404642161, 3.8484980101510677, -1.6891876959553338, 2.3885991535588689], [-2.5634257419825555, -2.1439617410087592, -1.9846633209833162, -0.38050903145944126, -0.6211134753602634], [-0.58180813106058427, 4.4088746157542706, -3.0834145101579078, -0.34406601186500385, 2.3387929531201648], [-4.6883729777339918, -0.11382867093099502, 1.0421192854234107, 0.38200765064980757, 0.8857092525928385]], [[6.2148986123211962, 1.7655877938396562, 3.6278894010857154, -1.3537516187445959, 6.0476222283187369], [5.1464843733542018, 6.8952126340547757, 3.5892559927525625, 7.6784301379487765, 7.3461992137357282], [6.8398658779458099, 2.7555074186985555, 2.2335255264081795, -1.9303721019498825, -1.3662771156056133], [-1.765033442791716, -0.83161261792466679, 2.8788882592959117, 2.2583707478794954, 0.86933536314855298]]], [[[-6.7082562074186542, -4.8256458805841866, -3.8583012698071277, -0.48987416314473187, -8.0636204343733056], [-5.8988397748077528, -8.199986576420681, -0.67345815269853482, -9.8927131826271157, -7.1531513788711925], [-8.0723357781467531, -3.4690716958270613, -6.0668690774895921, -2.7966055796165978, -4.4536724268216776], [-9.4871484772637444, -9.1030229137281786, -0.26922130790289778, -1.815150633831415, -8.0433584369170514]], [[-5.2757340841232896, -6.5410437914272901, -6.0253139800611493, -4.5023416034372836, -1.1933355387812363], [-0.15474248318487716, -8.068323776271443, -7.6528392969019325, -0.5668911432687227, -6.3279189019640896], [-3.9619776647754672, -6.1690271322027304, 0.4133176666934042, -0.75284028228083555, 1.6056736419406037], [-6.8584437591886838, -0.68386582983097322, -1.034909449070279, -5.4277419059577685, -3.8648662458690244]], [[5.3917311248070154, 2.9618890275654337, -0.47830311881027221, 2.2990753040903655, 8.1686175021328822], [8.1657722232956633, 1.1674530599731057, 4.1363759525675565, 7.6525507130233521, 8.0712117981382061], [7.5388117569035735, 7.6749082907800528, 1.9434387360820402, 5.224930799373122, 1.0362401941002028], [5.2610010355357417, 0.3341778947554328, 5.3521451680853849, -0.9416004723532847, 1.9705483730311268]], [[-3.2923025191127588, -4.6697826779779241, -3.6904913607501757, 0.077030138385079994, 1.6901173175675082], [2.4697965282395336, -0.051140571485637665, 1.7551452842402773, 3.6210415024483558, 3.1722326167862134], [-4.1466733527930053, -0.02441517584466979, -6.2199515262591705, 3.5269042037682654, -2.746195691511081], [-6.1021191536750674, 2.0742167598130328, -3.8995873250368778, 0.91335770099540436, -5.3164434867414867]], [[0.29020527385161277, 4.2716471943824015, -0.087682025575233968, 0.010926410513785179, 5.3651796016141216], [-2.2023643001753435, 4.6745285976568089, 4.1176229327673566, 3.3550383162326618, -0.053200343409267603], [6.2214676211936588, 1.4574341967634332, -2.6376559527877799, 4.21405588742424, 6.8766094184369297], [7.01385199538392, 6.813798488147123, 3.533923943454834, -1.8099952120371787, 2.1218020038632082]]], [[[-7.4728829321767236, -7.8944905184411889, -2.6102106441408308, -4.8644110640878671, -3.3763798305110955], [-7.2473972474012172, -1.4416443925340037, -4.6923556324405773, -1.8534084510017479, -8.397617571961792], [-4.4371119847095351, -8.7747559887577538, -4.4890716992564252, -2.7709887469184009, -7.9266731507086599], [-7.4829404834414683, -2.5376133948050876, -0.50866991032172493, -5.3614454776123317, -7.8542429385233419]], [[-2.0220856869035262, 1.6710116704925726, 0.25434746884003845, -1.0988938590049533, 5.5002568873570477], [-1.6894910963210092, 5.3416742137489104, -2.7651673959094323, -1.0671993509442963, -3.1603390282080861], [-2.7764141242708735, 4.7777486271747103, 6.5592752730478585, -1.7187572640447879, 2.4690981131790384], [1.3807426394034596, 6.5674326605351405, 2.6220847986858642, 3.2441220433201217, -1.3979782026699548]], [[-2.1104498331515602, 1.0656759948890011, 6.8055696420582663, 3.7220077814933168, 0.3179550795756696], [3.9203206630261285, 3.8455136636331098, 6.1192013820620534, -0.94365689490049398, 4.8135099555273522], [4.2817480150905025, 3.0971193616596082, 4.7672993162904991, 2.5867897203375545, 3.5644453378612564], [5.3758828584108809, 6.0548825503683315, -1.8871709462457691, -2.4625863445187228, 0.98501613891790507]], [[-2.1617729629623401, -3.4980617112653336, 5.3204351386406117, 0.58627754877340088, 2.9871669445125502], [2.3244409614610602, 0.66408745518705281, 5.0257560133091559, -3.9595026552548305, -0.40023277045503836], [-3.2751556672547046, 0.10385530013124722, -2.7742613511213081, 5.1870179287209428, -2.7297663641505672], [3.0542348682941913, -3.6110433613932997, 1.187331566723735, 2.0722004804742102, -1.9283144265945831]], [[2.0543434768896622, 0.6289980523199592, 4.9912748401821831, 2.8229006706328983, 0.77484985703326537], [6.3670219633816911, 4.8337953285919246, -1.3097037303234469, -1.4860356371248806, 1.3274751138443284], [4.9199418207184866, 7.2494120805556213, 2.9164622233922808, 4.4097591764505255, -2.2474548593983572], [-0.14328625632434555, 5.6755744717896386, 3.6488210513213728, 6.5322611597230642, 3.739245125532519]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.2380400272931364, 1.7483073662147444], [-1.1013563899165737, 0.057138336254721978]], [[2.4339361360325036, 2.8434571672471449], [-4.9935180482073136, -0.93477468741129144]], [[-1.6741589232700802, 1.887092658064101], [3.3636145121458618, -2.3364621010166973]], [[-0.31320437527172817, 0.94219281107285902], [-2.9111378476720731, -3.6213553727473649]], [[-2.4008421778239599, -3.3011697434576073], [-3.8599954010326187, -4.4935753364550859]], [[-2.7373704421989364, -4.3004141656316541], [-2.8665014279766443, 0.75298095973351664]]])+(1.-msk_arg1)*numpy.array([[[3.0501659253806217, 4.5144105341967702], [0.97470972742751805, -2.7759919983202899]], [[-1.0505578616343367, 0.07115867259418529], [-4.7956893443731063, 2.6112597324625133]], [[-4.2726134338334685, 2.9896673562590603], [-1.7422775480114039, -2.0585604175619734]], [[-2.2526102270096962, 3.9309625638691195], [-3.5114470809498979, -0.89532313302807509]], [[0.020457192924446588, 3.2154754191788868], [0.005551711705963136, -4.5887070169546522]], [[4.5411551816054274, -4.1764824541936241], [2.4604814856250616, -1.860847551658499]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[-2.3993284901930467, -1.4320915488860866], [1.8187138026720291, 1.0173287810927611]], [[4.6036483278108093, -2.1403099498993541], [2.4564164329625546, 0.73439523825187791]], [[4.5408716159266795, -0.56045933450919438], [4.9139002035887618, -1.4310655799709826]], [[-0.49302387709172812, -4.9829973028833949], [-4.8977950515701014, 1.9545367790327912]], [[2.0252380929264113, 4.5834779365286735], [0.34491408727708706, -0.54422188259244564]], [[-0.19797811696940393, 3.3441875528664138], [-4.9219297015525365, 1.3000638215261819]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.83871153710008972, 0.31621581732865778], [0.71735741275545539, 1.0744671173474831]], [[7.0375844638433129, 0.70314721734779084], [-2.537101615244759, -0.20037944915941353]], [[2.8667126926565993, 1.3266333235549066], [8.2775147157346236, -3.7675276809876799]], [[-0.8062282523634563, -4.0408044918105359], [-7.8089328992421745, -1.6668185937145736]], [[-0.3756040848975486, 1.2823081930710662], [-3.5150813137555317, -5.0377972190475315]], [[-2.9353485591683404, -0.95622661276524035], [-7.7884311295291813, 2.0530447812596986]]])+(1.-msk_ref)*numpy.array([[[0.65083743518757498, 3.0823189853106836], [2.7934235300995471, -1.7586632172275287]], [[3.5530904661764726, -2.0691512773051688], [-2.3392729114105517, 3.3456549707143912]], [[0.26825818209321106, 2.4292080217498659], [3.1716226555773579, -3.489625997532956]], [[-2.7456341041014243, -1.0520347390142755], [-8.4092421325199993, 1.0592136460047161]], [[2.0456952858508579, 7.7989533557075603], [0.35046579898305019, -5.1329288995470979]], [[4.3431770646360235, -0.83229490132721029], [-2.4614482159274749, -0.56078373013231708]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.26576492992056266, 4.151622689097989], [-2.3328560254499942, 2.3504626759317224]], [[-1.2126267145369596, -4.7791286443854215], [-1.4499676563520012, -4.3233365606853518]], [[-2.8300683933694937, 1.8755977699767881], [-3.4814100268692174, 3.6940863305262521]], [[0.34856308782051837, 4.9845069821687797], [-3.0292208558297782, -4.3645305428034575]], [[-1.6757673599117586, -1.2788840622936415], [-0.58560893230655608, 0.52714285048575338]], [[3.6590993289346301, -1.974133771676172], [2.2904110120608401, -1.4710169259931005]]])+(1.-msk_arg1)*numpy.array([[[-4.7506391296660455, -2.9378157113753387], [4.087553634720889, 4.1823530914873412]], [[3.8882983867089074, -4.1324671331488902], [1.7622976984862753, 2.6159115565710964]], [[-2.672788907062503, -2.2610201271961916], [-1.8193215969101253, -0.20802303820156265]], [[-1.6655325127502598, 0.18166264987472225], [4.3084067917349174, -0.18583201227081592]], [[-1.9890973596657946, -0.37030168912123074], [3.9988447007609533, -3.7412888983727886]], [[3.9141175567652962, -2.4353255115943906], [3.5143057798001784, -3.8193727873263672]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[-0.17967030335796252, 3.2571796461610845], [-3.5281456610223549, 3.0968064113195126]], [[-1.8641045676523826, -2.126060198818692], [1.5101896718593064, -1.73136265547148]]], [[[4.0068457400602515, -2.4056632906829565], [-0.88026055152019467, -1.5044181822381706]], [[2.0827747176636553, 2.6616990459706225], [3.7704733402391604, 4.7273715630946658]]], [[[-1.887839459021643, -2.6794756251169183], [3.8423937776266612, -2.9485468881059731]], [[2.7680027116225991, -2.6157801220767785], [-2.4476620975319707, -4.7555305938435311]]], [[[2.6492197287613415, -0.10988391541042386], [1.9577133701252674, -2.1694970117349053]], [[4.1135036855151785, -3.5016540798540752], [1.3253445005298552, 0.12895213812072193]]], [[[-4.0185234597465582, 4.6441267100215242], [-3.6272823303361958, 3.0402825017387656]], [[-0.74341107346523927, 0.27018619083767703], [1.5468940967378071, -3.026711265107469]]], [[[3.1220615059239822, 1.2108726388041902], [-4.6650826159872754, -3.2171929341879322]], [[2.0374332791797647, -4.8568462195308317], [-4.6866304249612503, 1.6013493974112949]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.086094626562600141, 3.5229445760816471], [0.6234770280756341, 7.2484291004175017]], [[-4.1969605931023768, -4.4589162242686857], [3.8606523477910288, 0.61910002046024237]]], [[[2.7942190255232919, -3.6182900052199161], [-5.6593891959056162, -6.2835468266235921]], [[0.6328070613116541, 1.2117313896186213], [-0.55286322044619141, 0.404035002409314]]], [[[-4.7179078523911366, -5.5095440184864124], [5.7179915476034493, -1.072949118129185]], [[-0.71340731524661827, -6.0971901489459963], [1.2464242329942814, -1.0614442633172789]]], [[[2.9977828165818599, 0.23867917241009451], [6.9422203522940471, 2.8150099704338745]], [[1.0842828296854004, -6.5308749356838529], [-3.0391860422736023, -4.2355784046827356]]], [[[-5.6942908196583168, 2.9683593501097656], [-4.9061663926298369, 1.7613984394451241]], [[-1.3290200057717954, -0.31542274146887905], [2.0740369472235605, -2.4995684146217156]]], [[[6.7811608348586123, 4.8699719677388202], [-6.6392163876634473, -5.1913267058641042]], [[4.3278442912406048, -2.5664352074699917], [-6.1576473509543508, 0.13033247141819437]]]])+(1.-msk_ref)*numpy.array([[[[-4.930309433024008, -1.493459483504961], [-6.4659613723976932, 0.15899069994417392]], [[2.2234490670685063, 1.9614934359021969], [5.6925427633466477, 2.4509904360158612]]], [[[7.8951441267691589, 1.4826350960259509], [-5.0127276846690849, -5.6368853153870608]], [[3.8450724161499306, 4.4239967444568977], [6.3863848968102568, 7.3432831196657622]]], [[[-4.5606283660841456, -5.3522645321794213], [1.5813736504304696, -5.2095670153021647]], [[0.94868111471247385, -4.4351017189869033], [-2.6556851357335334, -4.9635536320450937]]], [[[0.98368721601108167, -1.7754164281606837], [2.1393760199999896, -1.987834361860183]], [[8.4219104772500959, 0.80675271188084219], [1.1395124882590393, -0.05687987415009399]]], [[[-6.0076208194123524, 2.6550293503557296], [-3.9975840194574266, 2.6699808126175348]], [[3.2554336272957141, 4.2690308915986304], [-2.1943948016349815, -6.7680001634802576]]], [[[7.0361790626892784, 5.1249901955694863], [-7.100408127581666, -5.6525184457823228]], [[5.5517390589799431, -1.3425404397306533], [-8.5060032122876166, -2.2180233899150723]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_expandedData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.6015430005992446, -1.868876183443593], [-2.7507729128122529, 3.3444630352186344]], [[-3.5862384817698412, -3.6287510512076593], [0.96798234561468277, -1.1905117049833369]]], [[[-4.2235177647391513, 4.7256519871860601], [2.3667233540678536, 4.4698296977746601]], [[3.4411463753088078, -2.7175277005107668], [3.6332343836498282, 2.0999875328559749]]], [[[-4.8702180121233383, 4.2081581376506207], [-2.4500767754653641, -1.5623525801634894]], [[-1.6270906318947178, 3.1897117075658841], [1.6460631498513809, 2.5270963733956222]]], [[[2.9812741309998936, 2.2514400134366692], [-1.8461233137624191, 4.9963959503048745]], [[2.6917111687078137, -0.28450258351081636], [-4.3516163895773268, 0.85277454860210877]]], [[[-3.7872426306101392, -2.9587425308180682], [-1.0435944462975808, 2.992534720875101]], [[-0.34294155740808385, -3.5819761236198633], [1.7143539323555723, 3.3459286598359288]]], [[[4.8825188103882429, -3.0500040911417168], [-3.6816967026618985, 2.0780969771286903]], [[0.91995758272840877, 0.79660783734425245], [3.5208712177194208, 1.3214186518596263]]]])+(1.-msk_arg1)*numpy.array([[[[-2.6525948381133277, -1.9898792366331461], [-3.6145641700021827, 3.7880752855039059]], [[4.3219542575346299, -2.0024888336913471], [4.8663496619225768, 3.9815471471033064]]], [[[-4.9914940814513917, -4.251275152840142], [3.3432369730920311, 0.14414668236624539]], [[3.8348383104744919, -4.3628009147517286], [-2.5468010076572858, 0.23851346630558012]]], [[[-4.9866660089108406, -3.9073732224364379], [2.0705448213492472, 2.9116203468297179]], [[2.8485137613933134, 2.024713408813545], [-4.7727919384460593, -2.8029399737742589]]], [[[-4.6579834833325853, -2.4824430770431851], [3.7748276133823939, 0.4474069452701448]], [[3.8483516100505604, -0.69843835320624681], [1.9927200770750417, -2.917162413159927]]], [[[2.64649470223201, -0.90595540575589872], [1.731926873865552, -2.2761563678504837]], [[1.2474241003419539, -1.5374882240032814], [-4.4465422095966129, -3.8021782137885185]]], [[[-3.0478710601674641, -2.5170056900416737], [3.4180620961826804, -4.9600274185257254]], [[4.7153297234780549, 4.9943202114749319], [2.9360988298018889, 1.677498587290243]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[-3.8141783596443011, 3.7218412645479777], [1.5388175051840092, -0.054170592014758157]], [[2.4203408645179412, -1.8310054967962044], [-4.6980363940293586, 2.5517523878517254]], [[-4.3140861704055267, 4.0582107913272338], [3.2278934658830813, 0.42328508292535005]], [[2.9240991521371278, 4.2150604181362219], [4.0267454549222386, 1.0891033556866372]], [[-4.7968978791308086, -4.4085766632099883], [1.3480188794362258, 2.0449224630415896]], [[3.2450142032719818, -0.75208295632572053], [-1.7954001604344638, 4.0851984478251673]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.2126353590450565, -5.683054543087894], [0.97106835173572481, 7.0663042997666121]], [[-2.047420976585832, -2.0899335460236501], [0.91381175359992461, -1.2446822969980951]]], [[[-1.8031769002212101, 7.1459928517040012], [0.53571785727164922, 2.6388242009784557]], [[-1.2568900187205507, -7.4155640945401249], [6.1849867715015536, 4.6517399207077004]]], [[[-9.1843041825288658, -0.10592803275490592], [1.6081340158618698, 2.4958582111637444]], [[1.6008028339883635, 6.4176051734489654], [2.069348232776731, 2.9503814563209723]]], [[[5.9053732831370214, 5.175539165573797], [2.3689371043738028, 9.2114563684410964]], [[6.7184566236300522, 3.7422428714114222], [-3.2625130338906896, 1.941877904288746]]], [[[-8.5841405097409478, -7.7556404099488763], [-5.4521711095075691, -1.4160419423348873]], [[1.0050773220281419, -2.2339572441836375], [3.759276395397162, 5.3908511228775184]]], [[[8.1275330136602246, 0.19501011213026498], [-4.4337796589876195, 1.3260140208029698]], [[-0.87544257770605505, -0.99879232309021138], [7.6060696655445881, 5.4066170996847935]]]])+(1.-msk_ref)*numpy.array([[[[-6.4667731977576288, -5.8040575962774472], [0.10727709454579504, 7.5099165500518836]], [[5.8607717627186391, -0.46367132850733794], [4.8121790699078186, 3.9273765550885482]]], [[[-2.5711532169334506, -1.8309342883222008], [1.5122314762958267, -1.686858814429959]], [[-0.86319808355486671, -9.0608373087810872], [0.0049513801944396363, 2.7902658541573055]]], [[[-9.3007521793163672, -8.2214593928419646], [6.1287556126764811, 6.9698311381569518]], [[6.0764072272763947, 5.2526068746966263], [-4.3495068555207093, -2.3796548908489088]]], [[[-1.7338843311954575, 0.44165607509394267], [7.9898880315186158, 4.6624673634063667]], [[7.8750970649727989, 3.3283071017159918], [3.0818234327616789, -1.8280590574732898]]], [[[-2.1504031768987986, -5.7028532848867073], [-2.6766497893444363, -6.684733031060472]], [[2.5954429797781797, -0.18946934456705566], [-2.4016197465550233, -1.7572557507469289]]], [[[0.19714314310451764, 0.72800851323030802], [2.6659791398569599, -5.7121103748514459]], [[2.9199295630435911, 3.1989200510404681], [7.0212972776270561, 5.7626970351154103]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.6584218969922713, -2.2721513964415663, 3.3034427297461271, -1.2653561518446601], [3.3636305077395416, -3.9857791828100666, 4.9980909130618212, -3.6889642585037699], [-3.2766735369072189, -0.44508171138643693, -1.2598826477859149, 2.1679228739516407]], [[-4.8941759136937399, -1.8661299275041521, -3.6528846094651533, -3.563377287252476], [-3.1822286785732201, 2.4283717872568733, -2.7276640811021977, 2.604783754113261], [-1.322154790991946, 4.4762340606215805, -2.7146476226486262, -0.85112755427332765]]], [[[2.3391043865942276, -4.0758703542598615, 0.073414070735655201, -4.6108374949648034], [-1.6348746600983102, 3.0822542128539414, -3.3330063661547014, 3.4904933635351814], [-0.052940817240532567, 4.0119797663552017, 1.8704718050429037, -1.6313668942563062]], [[-3.5440828297257685, 0.14829334944753292, 3.6614325067160145, 2.5750523890300077], [2.6167648973725512, -3.2853093655636814, -4.1610978884298913, 0.12812911206918542], [-0.81193776881790747, -3.8396226122518007, 2.0956701929324932, 0.095129313665197834]]], [[[-0.36399115309040475, 2.6376696152083623, -4.2145980927020954, -3.2922046119707682], [0.11339939193450466, -2.4077983512483438, 4.0707898602868955, 4.7215777966792292], [-3.4548066701432099, 3.700897869291186, 4.0582325837892608, 0.90922642953532939]], [[-4.6082575029506669, -0.6862971542569305, 2.9500048938440857, 3.432246788480116], [4.6376153325023957, 3.4430522838073223, 4.2062855669003714, 1.5066948984011175], [2.5269627733190339, -4.2788250011750026, 2.0240187051796878, -3.0809639593608908]]]])+(1.-msk_arg1)*numpy.array([[[[4.2162394200744782, 3.9502511801699036, -4.4559031390602142, 2.8203003271071614], [3.2692554222957781, -1.3647024030912656, 4.694384680280729, -1.4877552992923428], [-3.7063094799938434, 2.0310685742112273, -1.7120362950363588, 4.8827342926585953]], [[-0.61024764770826501, -4.333777809320047, -3.4123931542017374, -2.8908759732160574], [-4.8968869128882355, -2.0503373183827844, 3.4983691223166193, -1.6564052121569741], [-4.3427578308320927, 4.9869842780727751, -1.1043122540790842, -4.3795488734188295]]], [[[4.0268921201961909, -2.0226814740051213, -3.7638011504330748, 1.7525291830291305], [1.1232018960088386, 3.2214940157542742, 4.4728060561130434, 2.7922987815949156], [4.5988258452063917, -1.3364410053805811, 3.1282526082718185, -3.3837890239593804]], [[-1.7318405831623638, 1.1070672583122976, 1.833313665440258, 4.2573927567523118], [-4.1080271816740819, -2.4009796657128089, 1.9121128169258936, -0.15920435928879151], [1.2743977343176809, -0.87584841338793407, 3.9463247886027943, -4.4314524440055933]]], [[[4.8308175720925277, 0.84796820823104646, 1.9705690459191594, 2.7728057025372168], [4.1762870616726762, 2.724974715114743, 3.6206977578984976, 0.79393684121426222], [3.280648640078665, -2.1282859418543931, -0.88459145989107313, 3.8134790040013868]], [[3.20468795694741, 4.992266756528295, 4.1928289649266706, 4.6590545997272539], [-2.7635585341068056, 3.8286147538807782, 3.5565251753247367, 2.463726124373915], [3.4038640251523002, 1.9861332580168085, 0.89115075246950326, -4.7714775730182089]]]])
      res=add(arg0,arg1)
      s0=numpy.array([[[[4.9535032138848081, 4.5787476702129233, 0.16306015353887915, 4.0116589213126943], [3.90163088727431, -4.295569816300004, 0.90550514868791687, 3.721976847077709], [-3.0168538406681056, 1.1256206588575415, 1.3550141080081612, -0.16770414141937895]], [[-1.1580491145889269, -3.9794188599995453, -1.8821954777950198, 3.8529193715571033], [1.0233430839184159, 4.9887180681263068, 1.8869932422884794, -0.47035254137341909], [0.97933431853787756, 1.6932023291078115, -3.9719133348350919, -1.8578999942848826]]], [[[-1.3467688544199463, 1.1902718302034216, -3.1934309143908846, -1.4413648789968536], [-4.1707584298142315, 4.6093248683423269, 4.1347989808071102, 4.5529474310285227], [4.8232588145416617, -3.6038895761362557, -3.1004564528419074, 0.051163134104358576]], [[-4.2839248546713158, 1.7540263181139215, 4.5136207657575707, -4.1913941665557441], [-4.1329212574991372, 3.0412142937856803, -4.0503917886123277, 1.9587114893887545], [1.0535035564152029, 1.2568632189909135, -0.88423569641473598, 3.0894173561115394]]], [[[0.20376199254843552, 3.4473222862061057, -1.9619307857742232, 4.8261986642537096], [2.8432148610558059, -1.3686756938049696, 3.8066748009404776, -3.55449257181226], [3.8632155591247539, 1.1828399696553031, 2.0466607616370434, 1.3239172031130577]], [[-0.41336063345173457, 3.4685976377171279, -3.483163394037454, 2.1170415072946991], [0.15175384500785594, 2.1653136884608868, -0.25734984039486797, -4.2644586966304532], [2.1099247348170174, 3.5379703461988363, -3.5750930000799794, -3.5472638504646059]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.2950813168925368, 2.306596273771357, 3.4665028832850062, 2.7463027694680342], [7.2652613950138516, -8.2813489991100706, 5.9035960617497381, 0.033012588573939183], [-6.2935273775753249, 0.68053894747110455, 0.095131460222246389, 2.0002187325322618]], [[-6.0522250282826668, -5.8455487875036969, -5.5350800872601731, 0.28954208430462725], [-2.1588855946548042, 7.4170898553831801, -0.84067083881371829, 2.1344312127398419], [-0.34282047245406844, 6.169436389729392, -6.6865609574837181, -2.7090275485582103]]], [[[0.99233553217428128, -2.8855985240564399, -3.1200168436552294, -6.0522023739616575], [-5.8056330899125417, 7.6915790811962683, 0.80179261465240881, 8.0434407945637041], [4.7703179973011292, 0.40809019021894599, -1.2299846477990037, -1.5802037601519476]], [[-7.8280076843970843, 1.9023196675614544, 8.1750532724735852, -1.6163417775257365], [-1.5161563601265859, -0.24409507177800105, -8.2114896770422199, 2.0868406014579399], [0.24156578759729541, -2.5827593932608872, 1.2114344965177573, 3.1845466697767373]]], [[[-0.16022916054196923, 6.084991901414468, -6.1765288784763186, 1.5339940522829414], [2.9566142529903106, -3.7764740450533134, 7.8774646612273731, 1.1670852248669692], [0.40840888898154404, 4.8837378389464892, 6.1048933454263041, 2.2331436326483871]], [[-5.0216181364024015, 2.7823004834601974, -0.5331585001933683, 5.5492882957748151], [4.7893691775102516, 5.6083659722682091, 3.9489357265055034, -2.7577637982293357], [4.6368875081360512, -0.74085465497616632, -1.5510742949002916, -6.6282278098254963]]]])+(1.-msk_ref)*numpy.array([[[[9.1697426339592862, 8.5289988503828269, -4.292842985521335, 6.8319592484198557], [7.1708863095700881, -5.6602722193912696, 5.5998898289686458, 2.2342215477853662], [-6.723163320661949, 3.1566892330687688, -0.35702218702819755, 4.7150301512392163]], [[-1.7682967622971919, -8.3131966693195913, -5.2945886319967572, 0.96204339834104591], [-3.8735438289698196, 2.9383807497435224, 5.3853623646050988, -2.1267577535303932], [-3.3634235122942151, 6.6801866071805867, -5.0762255889141761, -6.2374488677037121]]], [[[2.6801232657762446, -0.83240964380169968, -6.957232064823959, 0.31116430403227691], [-3.0475565338053929, 7.8308188840966011, 8.6076050369201536, 7.3452462126234384], [9.4220846597480534, -4.9403305815168368, 0.027796155429911096, -3.3326258898550218]], [[-6.0157654378336796, 2.8610935764262191, 6.3469344311978286, 0.065998590196567619], [-8.24094843917322, 0.64023462807287146, -2.1382789716864341, 1.799507130099963], [2.3279012907328838, 0.3810148056029794, 3.0620890921880584, -1.3420350878940539]]], [[[5.0345795646409632, 4.2952904944371522, 0.0086382601449361829, 7.5990043667909264], [7.0195019227284821, 1.3562990213097734, 7.4273725588389752, -2.7605557305979977], [7.1438641992034189, -0.94544597219908999, 1.1620693017459702, 5.1373962071144446]], [[2.7913273234956755, 8.4608643942454229, 0.70966557088921656, 6.776096107021953], [-2.6118046890989497, 5.993928442341665, 3.2991753349298687, -1.8007325722565382], [5.5137887599693176, 5.5241036042156448, -2.6839422476104762, -8.3187414234828143]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s0=numpy.array(-1.78016014593)
      s1=numpy.array(-2.51732062981)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-4.29748077573)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s0=numpy.array([-2.8960568664340016, -0.44049127603861926])
      s1=numpy.array(-3.35121260058)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-6.2472694670171887, -3.7917038766218059])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s0=numpy.array(-3.95680308163)
      s1=numpy.array([1.1393520763070173, -3.7420417118706095])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-2.8174510053196435, -7.6988447934972708])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s0=numpy.array([[3.6955382431357613, -1.6450507257250049, -2.9214625454691978, 4.8197841174754021, -2.499399624323122], [1.7096099657313122, 2.8746816973924041, -1.5273588889128478, -0.047132588724237756, -1.1405730120040349], [-1.487419634734064, 3.8121549926312213, -4.298404378865694, 0.69009979786702846, -3.8084544553085764], [1.1742989115124161, -4.658936480441719, 2.9438920028197764, -4.1564739269281237, 2.0571523246146324]])
      s1=numpy.array(2.48781148008)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[6.1833497232118111, 0.84276075435104492, -0.43365106539314802, 7.3075955975514519, -0.011588144247072218], [4.197421445807362, 5.3624931774684539, 0.96045259116320203, 2.440678891351812, 1.3472384680720149], [1.0003918453419858, 6.2999664727072711, -1.8105928987896442, 3.1779112779430783, -1.3206429752325266], [3.6621103915884659, -2.1711250003656692, 5.4317034828958262, -1.6686624468520739, 4.5449638046906822]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s0=numpy.array(3.50351919124)
      s1=numpy.array([[-0.48940694479749336, 3.4285339348741548, -3.7683708939603369, 0.72722102951559808, -4.7837171918739312], [-2.2695764136343657, 0.18384217332021446, -2.3256910360055025, -3.7310767631572608, 4.3249571972882475], [-1.0035467617593818, -1.7801223113483577, 0.2621038145693424, -0.83442150456772524, -1.9297675763084943], [-2.7111985372325673, 1.4761959244917753, 4.3199543525424939, -4.8311539897870324, 4.1789874842787569]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[3.0141122464455679, 6.932053126117216, -0.26485170271727565, 4.2307402207586593, -1.28019800063087], [1.2339427776086955, 3.6873613645632757, 1.1778281552375587, -0.22755757191419956, 7.8284763885313087], [2.4999724294836794, 1.7233968798947035, 3.7656230058124036, 2.669097686675336, 1.5737516149345669], [0.7923206540104939, 4.9797151157348365, 7.8234735437855552, -1.3276347985439712, 7.6825066755218181]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s0=numpy.array([[[1.8991321008839606, -4.0193427937454009], [0.072825235527468557, -3.1205592414215468]], [[-1.5017628476864009, -1.4872658656061031], [-2.8168228240025597, -1.7406713506869629]], [[-2.0010683443927268, -1.3064885293598429], [-3.7371297009694016, 1.9790315893481756]], [[0.63529721428379915, 4.2345531644109151], [-3.1481398039820121, 1.3636023661244963]], [[1.2819671238500785, -0.94847030915506103], [2.9815315209287574, -3.7542980280423457]], [[-2.2055077085448191, -2.1983267970412479], [1.1771024405382864, -4.7322819929260662]]])
      s1=numpy.array(2.68464951749)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[4.5837816183702476, -1.3346932762591139], [2.7574747530137556, -0.43590972393525984]], [[1.1828866697998861, 1.1973836518801839], [-0.13217330651627268, 0.94397816679932411]], [[0.68358117309356015, 1.3781609881264441], [-1.0524801834831146, 4.6636811068344626]], [[3.3199467317700861, 6.9192026818972021], [-0.4634902864957251, 4.0482518836107833]], [[3.9666166413363655, 1.736179208331226], [5.6661810384150444, -1.0696485105560587]], [[0.47914180894146785, 0.4863227204450391], [3.8617519580245734, -2.0476324754397792]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s0=numpy.array(3.2445129263)
      s1=numpy.array([[[0.39854168711954152, -2.949628834109157], [-0.93584643943854129, 4.407536546121479]], [[2.6432287237345529, 4.4000546043358728], [-1.2436967361378337, 3.5339122955838818]], [[-2.4439577069452865, -4.4584764832879555], [1.0254590972099749, 2.6609676997250329]], [[1.6148620988106535, 3.0110472589434796], [3.8347587093961639, 2.8058568785665656]], [[-0.37103871373567721, -1.8368424799453686], [-3.0857631859975632, 3.0100071760217357]], [[-3.8168909114227745, -2.7915160546664963], [-3.9738220694833326, 1.2182719909361142]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[3.643054613423784, 0.29488409219508549], [2.3086664868657012, 7.6520494724257215]], [[5.8877416500387953, 7.6445675306401153], [2.0008161901664088, 6.7784252218881242]], [[0.80055521935895602, -1.2139635569837131], [4.2699720235142173, 5.9054806260292754]], [[4.8593750251148959, 6.2555601852477221], [7.0792716357004064, 6.0503698048708081]], [[2.8734742125685653, 1.4076704463588738], [0.15874974030667932, 6.2545201023259782]], [[-0.572377985118532, 0.45299687163774616], [-0.72930914317909012, 4.4627849172403566]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=add(arg0,arg1)
      s0=numpy.array([[[[3.4580404150973862, -2.6512045749996016, -2.5313135047498014, -0.46500879357922109], [-0.0072562750240914298, 2.6492058079630745, 1.4354843465922347, 4.1944507939037159], [0.96281184709368972, -2.2050774868968492, 3.5848548615419222, 3.9864479680268499]], [[0.36941325581321216, 4.6941651611818234, 1.3283047441947495, 4.126928896091318], [1.659435306013739, -3.350210316675688, 3.9318166659610299, -3.2086388141790358], [-2.1112336120234323, -4.150471557303459, 3.3356228538894843, 3.0700534880577841]]], [[[3.6004420763505571, -3.2432064983826461, -4.3106135327621953, -1.4639006054780035], [-3.0586121216487063, 2.4291741228590089, 4.6450271988523113, -1.8275882559207357], [-2.0279189026565847, 1.6947809243328358, 0.25236053265974867, -1.927962521484341]], [[1.116854302504283, 0.51243148533258953, -3.560537908372774, -3.7077403944105516], [-0.67428150261403008, -4.6690597022242439, -2.2609191104863191, -2.6056759607855162], [-2.976539128005161, -4.3397850461815581, 0.93926895879559513, 2.3576039048509534]]], [[[-3.5905615466353966, 4.7366372990305479, 1.9098513427717414, 3.1442216218570227], [-2.8887028362695477, -3.5647430633343449, -1.8195441715670437, 4.5816144456339121], [2.6810019338005686, 2.700663042999464, 0.10136050315100675, 4.9530970819172833]], [[-2.0748105752502144, -0.46110001107294529, -2.2471364633022564, -4.0713370682029968], [0.032295946625274574, 4.9717765969951504, -4.6298374689367812, 1.0965721375978132], [-2.58273615330019, -4.731496084797147, -3.6170498861685862, 3.226734974368922]]]])
      s1=numpy.array(-2.06924771111)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.3887927039887504, -4.7204522861082374, -4.6005612158584377, -2.5342565046878569], [-2.0765039861327272, 0.57995809685443866, -0.63376336451640114, 2.1252030827950801], [-1.1064358640149461, -4.274325198005485, 1.5156071504332864, 1.9172002569182141]], [[-1.6998344552954237, 2.6249174500731876, -0.74094296691388628, 2.0576811849826822], [-0.4098124050948968, -5.4194580277843238, 1.862568954852394, -5.2778865252876717], [-4.1804813231320681, -6.2197192684120948, 1.2663751427808485, 1.0008057769491483]]], [[[1.5311943652419213, -5.3124542094912819, -6.3798612438708311, -3.5331483165866393], [-5.1278598327573421, 0.3599264117503731, 2.5757794877436755, -3.8968359670293715], [-4.0971666137652205, -0.37446678677580003, -1.8168871784488871, -3.9972102325929768]], [[-0.95239340860435284, -1.5568162257760463, -5.6297856194814102, -5.7769881055191874], [-2.7435292137226659, -6.7383074133328797, -4.3301668215949549, -4.6749236718941525], [-5.0457868391137968, -6.4090327572901939, -1.1299787523130407, 0.28835619374231758]]], [[[-5.6598092577440324, 2.6673895879219121, -0.15939636833689441, 1.0749739107483869], [-4.957950547378184, -5.6339907744429807, -3.8887918826756795, 2.5123667345252763], [0.6117542226919328, 0.63141533189082821, -1.9678872079576291, 2.8838493708086474]], [[-4.1440582863588507, -2.5303477221815811, -4.3163841744108922, -6.1405847793116326], [-2.0369517644833612, 2.9025288858865146, -6.699085180045417, -0.97267557351082257], [-4.6519838644088258, -6.8007437959057828, -5.6862975972772221, 1.1574872632602862]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s0=numpy.array(3.57590453874)
      s1=numpy.array([[[[-1.1777443588414052, -1.6711315159841456, 0.8278865242910971, 1.7403488972674026], [1.8625744829099027, -3.1153096745323703, 3.2739867980221575, -4.0003077749986264], [2.2901803556172116, 3.4580381580769135, 2.6016393764011152, 0.49762360224449598]], [[-2.985735669310067, -2.6631848215685796, 3.9836080373111873, 0.59461187907572111], [-3.9345703792341014, 3.2301689655431645, -2.7153786961020288, -0.82731661852091776], [-3.6849199983998382, 4.1117992175526261, 2.7249454663587098, 3.0454109463320265]]], [[[2.2073866350222904, -0.56406306569750164, -3.3554416674106191, 3.3921006443410047], [1.6221632031362709, 4.7539687419056555, -4.9084191422294499, -4.1339638412432578], [0.87429146732946084, -4.1175191810353553, 1.5118926970687525, -2.4187025635727224]], [[2.8660874124370768, 4.0321884212076817, 1.3729694632721099, 2.1096551020574941], [1.5858883764650935, -1.9239079419993423, -1.7019970825933628, 0.52418790248085756], [-1.2815445377843959, 2.1346283362866973, 1.0067481768952717, -4.5901449930646923]]], [[[-3.9854926370069355, 3.0602010750811033, 2.7551151026213887, 2.86429579673527], [-1.9313023605393376, 1.0112507818345211, 1.4499961323570956, 3.9550975864973488], [-3.2964125488257157, -0.52225289522992213, 2.3328724092660602, 2.2332677057601202]], [[-1.3336529006495654, 4.2158001056215326, 4.4581720463599357, -4.1818536628168248], [-1.4417386820318723, -2.889861918986838, -3.7479144463067793, -0.82235209593697434], [0.73304179802981739, 1.6447459090147687, 3.5185913132518554, -0.96717401672136205]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[2.39816017989925, 1.9047730227565096, 4.4037910630317523, 5.3162534360080578], [5.4384790216505579, 0.46059486420828488, 6.8498913367628127, -0.42440323625797127], [5.8660848943578667, 7.0339426968175687, 6.1775439151417704, 4.0735281409851511]], [[0.59016886943058822, 0.91271971717207556, 7.5595125760518425, 4.1705164178163763], [-0.35866584049344619, 6.8060735042838196, 0.86052584263862641, 2.7485879202197374], [-0.10901545965918302, 7.6877037562932813, 6.300850005099365, 6.6213154850726816]]], [[[5.7832911737629455, 3.0118414730431535, 0.22046287133003606, 6.9680051830816598], [5.1980677418769261, 8.3298732806463107, -1.3325146034887947, -0.55805930250260261], [4.450196006070116, -0.54161464229470013, 5.0877972358094077, 1.1572019751679328]], [[6.4419919511777319, 7.6080929599483369, 4.9488740020127651, 5.6855596407981492], [5.1617929152057487, 1.6519965967413128, 1.8739074561472924, 4.1000924412215127], [2.2943600009562592, 5.7105328750273525, 4.5826527156359269, -1.0142404543240371]]], [[[-0.40958809826628029, 6.6361056138217585, 6.3310196413620439, 6.4402003354759252], [1.6446021782013176, 4.5871553205751763, 5.0259006710977507, 7.5310021252380039], [0.27949198991493951, 3.053651643510733, 5.9087769480067154, 5.8091722445007754]], [[2.2422516380910897, 7.7917046443621878, 8.0340765851005909, -0.60594912407616963], [2.1341658567087829, 0.6860426197538172, -0.17200990756612411, 2.7535524428036808], [4.3089463367704726, 5.2206504477554239, 7.0944958519925105, 2.6087305220192931]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s0=numpy.array([-4.2579138717579141, -0.85601533737844981])
      s1=numpy.array([-3.6742549422356543, 0.099769105366293687])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-7.9321688139935684, -0.75624623201215613])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s0=numpy.array([[-0.47974580613693796, -3.0565756336478991], [2.9006752909725027, 1.5048554199263169]])
      s1=numpy.array([-1.1950584787334062, -2.707174224235275])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-1.6748042848703442, -4.2516341123813053], [0.19350106673722767, -1.2023188043089581]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_Symbol_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 2))
      res=add(arg0,arg1)
      s0=numpy.array([-4.7490030322976011, 4.9927447603592956])
      s1=numpy.array([[4.4833080435678632, -0.44459976343946295], [-0.60137503807637849, 4.2385561540956971]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.26569498872973796, -5.1936027957370641], [4.3913697222829171, 9.2313009144549927]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s0=numpy.array([[[-0.67518211253366101, -4.1468457482948704, 3.8106704737756569, -1.2756317712394538, 1.0696960300344767], [-0.10068006002548024, 2.2701466485455128, -2.1697531949779258, 1.5787151644270816, -3.1876813351164435], [-2.8643601505384222, 1.3004456551787014, -0.34305445812281921, -3.7579114327248253, -4.300546377850079], [-1.4817156778668261, 2.9117782785845048, 2.324915913058387, -2.1739447775543841, -2.2591491740075629]], [[-2.4693522664432601, -1.877846964371618, -3.6986656724676257, 0.89655666777733956, 0.013564210187821502], [-2.1941297601629959, -2.4452224289166145, 0.88238989016355873, 2.5214346642111041, 0.20539419713853402], [-1.4755346706169838, -2.8225337126522221, -2.3785018543859069, -4.7538894067285762, 2.0900008355776478], [1.4949454097697661, 3.7723917711921846, -4.815232507713068, -2.4701739809390242, 4.3302076389356436]]])
      s1=numpy.array([-3.3497004376962538, 4.4401916238950658])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-4.0248825502299148, -7.4965461859911242, 0.46097003607940312, -4.6253322089357081, -2.2800044076617771], [-3.4503804977217341, -1.079553789150741, -5.5194536326741801, -1.7709852732691722, -6.5373817728126973], [-6.214060588234676, -2.0492547825175524, -3.692754895819073, -7.1076118704210796, -7.6502468155463328], [-4.8314161155630799, -0.43792215911174903, -1.0247845246378668, -5.5236452152506379, -5.6088496117038167]], [[1.9708393574518057, 2.5623446595234478, 0.74152595142744016, 5.3367482916724054, 4.4537558340828873], [2.24606186373207, 1.9949691949784514, 5.3225815140586246, 6.9616262881061699, 4.6455858210335998], [2.964656953278082, 1.6176579112428437, 2.0616897695091589, -0.31369778283351035, 6.5301924594727137], [5.9351370336648319, 8.2125833950872504, -0.37504088381800216, 1.9700176429560416, 8.7703992628307095]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_Symbol_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 4, 5))
      res=add(arg0,arg1)
      s0=numpy.array([4.7581987287127863, 2.3853943228564445])
      s1=numpy.array([[[-0.15847344006070596, 3.2519607783560591, -1.9901921432891498, -0.52140623093847704, -2.6637621862343352], [-0.81933678780093189, 1.3683503072661196, -1.0583152668385876, -0.45637758199339995, 0.33880304898876368], [2.3452785738174908, 1.4656508492052112, -0.49638055612378196, 0.12402267755968666, -0.68053735276265126], [2.719641992599179, -3.2379088472159054, 3.6250392239761968, -0.25958828402167367, -4.3717447149090507]], [[-1.2964747100631815, 2.9446224775582595, -4.1935819359866242, -3.2731881833869547, -2.8054072381747188], [-0.0052508476157306916, -3.6340113626336299, -0.64662600215627197, 1.3421943627411181, 2.8912593126183825], [-3.7308318021650355, 1.9563164990301143, -3.3782336712354133, -3.4233463540316622, 0.86519304733479263], [-3.5236213935573355, 3.0387673371406905, 4.3862780170136144, 2.0046326932541101, 1.3117120791132493]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[4.5997252886520803, 8.0101595070688454, 2.7680065854236364, 4.2367924977743092, 2.094436542478451], [3.9388619409118544, 6.1265490359789059, 3.6998834618741987, 4.3018211467193863, 5.0970017777015499], [7.1034773025302771, 6.2238495779179974, 4.2618181725890043, 4.8822214062724729, 4.077661375950135], [7.4778407213119653, 1.5202898814968808, 8.3832379526889831, 4.4986104446911126, 0.38645401380373556]], [[1.088919612793263, 5.3300168004147039, -1.8081876131301797, -0.88779386053051024, -0.42001291531827434], [2.3801434752407138, -1.2486170397771854, 1.7387683207001725, 3.7275886855975626, 5.276653635474827], [-1.345437479308591, 4.3417108218865588, -0.99283934837896881, -1.0379520311752177, 3.2505873701912371], [-1.138227070700891, 5.424161659997135, 6.7716723398700589, 4.3900270161105546, 3.6971064019696938]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=add(arg0,arg1)
      s0=numpy.array([[[[2.1394364866120821, -1.7874815031662772], [4.5907652999092203, 4.8684383901760917]], [[-1.401409830637852, 4.7011347869477564], [-1.169346625640737, 1.1044554750837072]], [[-3.7549012105071289, 3.0572060738339815], [4.1381719554050918, 4.5381419628455362]], [[0.8414619478511387, -2.3216751547948808], [2.3205113985478913, -1.2715766282005236]], [[4.941201681306449, 1.902727371807913], [2.8893738138279339, 1.4620213330872662]], [[2.8601465242082833, 1.9239211589294847], [1.5766932960532873, 4.0468838222246415]]], [[[-3.2748785391623869, 3.6426414534205556], [-0.20509581311505354, 3.7519971671282413]], [[2.0236011757092518, 4.0522086792114163], [3.7470126098709038, -1.4137062778168641]], [[2.0591540676893141, -0.92483357161819768], [-4.3007116151805516, -1.0007267445789414]], [[2.2431045129586753, 0.016964745596919073], [-0.27413411120506748, -0.98949127324153174]], [[1.3110529453042528, 0.90412394706688382], [2.9830390125715001, -2.5704754559403131]], [[-0.090473527839415979, 4.1611291322277939], [-0.15251019456438453, 3.2371544240821031]]]])
      s1=numpy.array([3.1891264705060252, 1.2691709193914793])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[5.3285629571181072, 1.401644967339748], [7.7798917704152455, 8.0575648606821169]], [[1.7877166398681732, 7.8902612574537816], [2.0197798448652882, 4.2935819455897324]], [[-0.56577474000110373, 6.2463325443400066], [7.3272984259111169, 7.7272684333515613]], [[4.0305884183571639, 0.86745131571114431], [5.5096378690539165, 1.9175498423055015]], [[8.1303281518124741, 5.0918538423139381], [6.0785002843339591, 4.6511478035932914]], [[6.0492729947143085, 5.1130476294355098], [4.7658197665593125, 7.2360102927306666]]], [[[-2.0057076197709076, 4.9118123728120349], [1.0640751062764258, 5.0211680865197206]], [[3.2927720951007311, 5.3213795986028956], [5.0161835292623831, -0.14453535842538479]], [[3.3283249870807934, 0.34433734777328162], [-3.0315406957890723, 0.26844417481253791]], [[3.5122754323501546, 1.2861356649883984], [0.99503680818641183, 0.27967964614994756]], [[2.5802238646957321, 2.1732948664583631], [4.2522099319629794, -1.3013045365488338]], [[1.1786973915520633, 5.4303000516192732], [1.1166607248270948, 4.5063253434735824]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank1_Symbol_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=add(arg0,arg1)
      s0=numpy.array([2.3983584621057723, -2.5782769061373978])
      s1=numpy.array([[[[-2.9854887064114655, 1.935935554833172], [-1.2480070384288133, -4.8402811398692611]], [[0.93636705626798911, 4.1731241781202257], [-2.0217410515606229, -3.3040752402593498]], [[-0.070267947331097425, 0.26393431292202951], [-0.57788435166276209, -2.2040298273660386]], [[-4.6834904495397822, 2.4749990460788132], [0.87067204326102843, -0.37813025419559665]], [[-4.4206862054109299, 3.4453349268738673], [3.2227568339026398, -2.8842936353890281]], [[3.0124330551446441, 1.7190140186963436], [1.8163890092212931, -4.8523825874963054]]], [[[-0.71933798437602015, 0.807429430689516], [1.1300236637766483, -2.3911605725124243]], [[-4.5841037580786654, 0.65184000439300149], [-3.1408048820741774, -1.9903085633778019]], [[-0.7342214549069892, 4.2355200568218585], [0.18301088165597701, 0.5834576264208593]], [[-0.60133724377389797, 1.0699013743956218], [-3.515376702365427, -0.018666252331065891]], [[-3.9309700327674157, -3.9838487677410539], [3.4475222035994175, -3.0478191551511022]], [[1.7525984383611695, -4.4401631136932256], [3.2786969930909677, 3.1637396139428251]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.58713024430569316, 4.3342940169389443], [1.150351423676959, -2.4419226777634888]], [[3.3347255183737614, 6.571482640225998], [0.37661741054514941, -0.90571677815357754]], [[2.3280905147746749, 2.6622927750278018], [1.8204741104430102, 0.19432863473973372]], [[-2.2851319874340099, 4.8733575081845855], [3.2690305053668007, 2.0202282079101757]], [[-2.0223277433051576, 5.8436933889796396], [5.6211152960084121, -0.48593517328325575]], [[5.4107915172504164, 4.1173724808021159], [4.2147474713270654, -2.4540241253905331]]], [[[-3.2976148905134179, -1.7708474754478818], [-1.4482532423607495, -4.9694374786498221]], [[-7.1623806642160632, -1.9264369017443963], [-5.7190817882115752, -4.5685854695151997]], [[-3.312498361044387, 1.6572431506844607], [-2.3952660244814208, -1.9948192797165385]], [[-3.1796141499112958, -1.508375531741776], [-6.0936536085028248, -2.5969431584684637]], [[-6.5092469389048135, -6.5621256738784517], [0.86924529746201973, -5.6260960612885]], [[-0.82567846777622833, -7.0184400198306234], [0.70042008695356994, 0.58546270780542731]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s0=numpy.array([[-1.2653498284451747, 4.39756125640635, -4.4204854648378644, 0.87551283694188609, -3.029614274613579], [4.419274741928735, 1.5227622051213361, 0.91948258092945512, 0.057635175103566638, -2.0505814999463978], [3.1700346903127485, -1.6493351978488704, 1.3650212289183825, -2.2502515689390612, -1.8429183345911548], [-2.4032470735246489, -4.7058866550560463, 4.545773652555166, 2.0209287458948024, 2.8880577239689051]])
      s1=numpy.array([[-2.620883594554706, 2.1520446799483883, -4.9363148688390073, 3.0925357386787375, 2.3644036695557702], [1.7462273018230787, -4.2268868740392183, -4.2752280555309383, -3.548535742002894, -4.9898747261468408], [3.7057679933202525, -1.6894226907522403, 3.4319524108466712, 3.2032051131313235, -1.8819395061233193], [-1.3297117271543089, 0.64948417403559766, -4.804480449561936, 2.8594361861873665, -2.5475418165135233]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-3.8862334229998807, 6.5496059363547383, -9.3568003336768726, 3.9680485756206236, -0.66521060505780882], [6.1655020437518138, -2.7041246689178822, -3.3557454746014832, -3.4909005668993274, -7.0404562260932391], [6.8758026836330011, -3.3387578886011107, 4.7969736397650538, 0.9529535441922623, -3.7248578407144741], [-3.7329588006789578, -4.0564024810204486, -0.25870679700677002, 4.8803649320821689, 0.34051590745538185]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s0=numpy.array([[[2.2410551302149635, 1.3296953749445075], [3.7518696861560201, -1.6893510268207654], [-4.6046430033250498, 1.0327727007777741], [1.9109253431641973, 1.1467137885288148], [-0.81646188617653248, 2.0061561147933373]], [[0.33707105577407326, -4.3189234337898199], [-2.9362700393202656, 1.3389625528401536], [3.3481758036133797, 1.823946284885503], [-3.4466288407284873, -1.0074432616738194], [-0.73815567307879348, -2.0106764248462818]], [[2.6548514881707552, -3.7963286639645268], [-0.16715432344848935, 4.677178933238693], [0.91216934755475521, -2.9870658534196659], [0.21221475601815065, -1.1256630861743657], [2.0939666966666994, 0.7694018466965602]], [[-3.9491547580304278, 3.8193137228184], [2.1208824832185789, -3.2812648158011539], [2.9588832759364081, -3.241435340489244], [-2.4590567210930048, 1.9089483856483813], [-3.4043344699617184, -3.8632058889082552]]])
      s1=numpy.array([[0.71560004628301499, -1.3173279607873809, -2.9829230594485656, 3.8202815460375756, 4.3842607678130054], [-2.5044376519199485, 1.1567901468304029, 3.4983939544410596, -0.29652919335354522, 3.999454149257069], [-2.9141348896133423, -0.24653030747606142, 2.6464704509174606, 0.064164646230306488, 0.78598446813358969], [4.077336730701612, -0.29229907852444192, 2.686924517610545, 3.454794569145653, -3.2955626727183542]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[2.9566551764979785, 2.0452954212275225], [2.4345417253686392, -3.0066789876081463], [-7.5875660627736154, -1.9501503586707916], [5.7312068892017729, 4.9669953345663904], [3.567798881636473, 6.3904168826063428]], [[-2.1673665961458752, -6.8233610857097684], [-1.7794798924898627, 2.4957526996705566], [6.8465697580544393, 5.3223402393265626], [-3.7431580340820325, -1.3039724550273646], [3.2612984761782755, 1.9887777244107872]], [[-0.2592834014425871, -6.7104635535778687], [-0.41368463092455077, 4.4306486257626316], [3.5586397984722158, -0.34059540250220532], [0.27637940224845714, -1.0614984399440592], [2.8799511648002891, 1.5553863148301499]], [[0.12818197267118414, 7.8966504535200119], [1.8285834046941369, -3.5735638943255958], [5.6458077935469531, -0.55451082287869902], [0.9957378480526482, 5.3637429547940343], [-6.6998971426800722, -7.1587685616266095]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 2))
      res=add(arg0,arg1)
      s0=numpy.array([[-2.0198849161873467, -4.0153168252092728, -0.65253541200075293, 0.028383411413837045, 1.1950584182937956], [4.9416479358046956, 2.497698673758868, 3.2892453957495942, -1.5634796549668906, 0.19696171141305996], [3.8547743727029591, -1.9443560162252513, -0.9095793481880774, -3.5437150368748616, 2.2121110665411612], [-1.8311693679255492, 2.9769415165464093, 1.3039875389628506, 4.7283553790582804, 1.4942844468586403]])
      s1=numpy.array([[[3.8566186546682335, 3.1273776766775363], [-2.1693922211131413, 3.1722079487135204], [-1.4848384590581687, -2.5827411402629776], [0.52609560518704246, 4.345734776941903], [-1.8972293604489376, -1.2278576341810767]], [[-3.8065396487934997, 1.2710116552483273], [3.4538672202056748, -1.792891848392526], [3.1159088692861623, 3.682805656030002], [-4.8216680764050484, -4.2422198508171984], [-2.2789860826649786, -1.9393254432735532]], [[2.7509412252951968, 1.6859395913929118], [-0.56628874254843975, 1.1444906331107205], [4.8661417640596643, -4.7949216557932886], [-0.41430229036681077, 4.0922560243180008], [-0.15989770689661142, -3.0728487419761388]], [[-1.9339722284362724, 4.3053581629964572], [1.4691738720057232, 4.3519695900568287], [-3.3254288274640675, 0.11646402833322256], [-3.9334701054827006, 2.0705903037489097], [4.3784384280363646, -1.3900678897877263]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[1.8367337384808868, 1.1074927604901896], [-6.1847090463224141, -0.84310887649575239], [-2.1373738710589216, -3.2352765522637306], [0.5544790166008795, 4.3741181883557401], [-0.70217094215514209, -0.032799215887281186]], [[1.1351082870111959, 6.2126595910530229], [5.9515658939645428, 0.70480682536634198], [6.4051542650357565, 6.9720510517795962], [-6.3851477313719389, -5.805699505784089], [-2.0820243712519186, -1.7423637318604932]], [[6.6057155979981559, 5.5407139640958709], [-2.5106447587736911, -0.79986538311453081], [3.9565624158715869, -5.7045010039813659], [-3.9580173272416723, 0.54854098744313928], [2.0522133596445498, -0.86073767543497759]], [[-3.7651415963618216, 2.4741887950709081], [4.4461153885521325, 7.328911106603238], [-2.021441288501217, 1.4204515672960731], [0.79488527357557981, 6.7989456828071901], [5.8727228748950049, 0.10421655707091393]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Symbol(shape=(4, 5))
      res=add(arg0,arg1)
      s0=numpy.array([[[[-4.7974657435014585, -4.9153786427102109, -0.89882320955626049, 1.0539289854293576, -0.87261109422948913], [-1.2053432760637719, 0.076660068720758368, 4.9647608710977646, 2.8008986483864708, 1.0207775410781128], [2.0312141230180361, 3.2458438014660747, -2.0276853648454738, -1.189205404846839, -1.2998329732446634], [1.3092935333122906, 2.0538722813824322, 3.0972382574566044, 0.50792596805883683, -3.4911543020172608]], [[-4.1814529647209469, -3.9831920026568088, 3.043426663545695, 4.1037997063936, 2.7907438627135974], [4.3247596697866157, -0.4221233024416513, -3.9531652985306334, -2.0747504068911415, 2.5578815368797407], [2.5492517825652481, 1.4232264123113714, 2.5249795385058, 4.82944881447143, 3.5337629225836196], [3.6551897603354284, -0.0039425389693361623, -1.4026701818462417, -3.6619548318297546, 3.2813605660565628]], [[4.4135779580310697, -2.396538156456669, 2.2443048315361409, 2.1146022156472171, -3.5901227104390276], [-4.6348092440066662, 2.0820660028252522, 2.9239056908165146, -1.8865164744312315, 3.887510993542703], [-3.6338974524668499, -3.4153751693284939, 4.7044579211131783, -3.2927800028723175, 0.25271895147576373], [4.3960552399938617, -2.0576725983651798, -3.3965490399315081, 2.507537055282846, -1.8740395927785869]], [[-1.4902322164143889, 4.3931190401592506, -3.3647229718327143, -3.1386638563826983, 0.88318911046923176], [2.9479498787340388, -2.5608822573873438, -4.072320667503031, 4.7093895325358446, -0.59533195226017721], [4.0227269639177408, 4.5640425212754518, -1.3500160665497951, 1.3444862408025511, -4.8178067921138377], [-4.4089935624987042, -0.30060482662156662, 1.4143351353867395, -1.8283013930157943, 3.222584384163218]], [[-4.9523718601654201, -1.9972199867281457, 2.3435987720673515, -3.4212700971885566, 0.61793359462404318], [-2.8470958383188405, 0.41169193242923008, 1.6421976303862706, 3.2854441965744101, -1.2815422649421837], [-0.096259090025437644, 3.0667581474879846, 2.5591377609798238, 0.86584502499694604, -2.7428637967738148], [-3.0671390560368685, 0.50516900974065848, 1.4574348048784982, 4.597565502622027, 0.18248213401435809]]], [[[-2.676846553779936, 1.0048536062119364, -3.6601395692801542, 1.2186249906689977, 0.15773797518246369], [4.9095279249700177, 2.0343793913906092, -0.60209593606252465, 2.7703177518426294, 2.1036565213018923], [-1.3974543861804967, -1.4988259114569691, 1.1132330112176847, -4.5819646847099911, -2.4178191566549612], [-0.039803395402938868, 1.4634024922839526, -3.0763502677536536, -4.0614646827569114, 0.2703609176433357]], [[1.8845978697724455, -3.1591930160967854, 1.1987804907506554, -0.3376003428279315, 0.21054979212556368], [-2.9433406928849495, 4.8539092454168582, -3.1916034359693635, -1.9964924291520534, -1.9298693283618729], [-4.8713204310869243, -3.3565788608990346, -1.0173448753599743, -1.5893176115363095, 2.7085728601656633], [1.4229897213396843, -0.33461238002730198, -2.5681441727783225, 1.5097520421959301, 0.71914296715213943]], [[4.0902660075670223, -3.0114564264343424, 2.8552016299266807, -2.9812241377455253, 1.4690387636631916], [4.647056408105863, -0.10540948682455387, 1.2301476164947811, 2.3261054478636192, -2.0520106630719659], [0.75995296408822544, -2.3349824557369123, 0.32378801853928962, 3.0561739621060191, -3.988894627694537], [4.9196523645812071, -0.21249885390986378, -0.65816275777577005, 2.5988534783940924, -0.31742413262302449]], [[1.1141675130682174, -4.9203014081994425, -0.37723827984001179, -4.0725613851323192, -3.2550774742745068], [3.8378339122734442, -0.56519383631039588, -2.9177209895836542, 1.2399592119093938, -2.3854606655123103], [0.91011261897718931, -1.5749978542296708, 2.372208512880861, 1.1255547607068275, 3.5987367174296221], [-1.2627390537708481, -1.6025813300413141, -2.3272070329111259, -1.5922619262402327, -4.0389399193971967]], [[2.3964025813630876, 0.39649910156455892, -4.6606197410476655, 4.1204247769017641, 3.4513236370836449], [-3.2186987993895624, -3.4711178606493664, 3.7396820208396662, 4.706979924928067, -0.13270298913259282], [4.9633384686474038, -0.13922906826650117, 4.5105503788015415, 0.058920405816481392, 4.048162497441254], [-4.4683186563796369, -2.7641825886598772, -1.5161219438358686, -3.3499333063826442, -3.0896204051139007]]], [[[-0.68627120068989722, 0.71187978321022083, -2.6111117256853067, 3.1396858047151017, 3.1476876249610584], [3.2712755412690129, 3.0258924778175942, 1.4572594857849497, 2.5679980698630693, -4.3411240905018458], [2.5149189203825673, 1.8172863714134833, -3.0384799165301457, -2.0576778178578214, 4.2678540032262493], [0.18573484651438488, 3.2280690325510353, -4.4077842342478863, -4.531875786337439, 4.7753761026365211]], [[-0.77700216536400379, 4.7196004985010909, 0.32862163073790107, -1.3325292045298829, -1.8309785816269617], [-1.3596862036216728, -4.676226271060151, 1.8289047531299349, 4.8836075257710494, 4.3077457955198248], [-4.8684124894106304, 2.2947130518996008, -1.140342162678103, -1.894504056356443, 0.64526619991459633], [1.0564002748925114, 0.6039729350716847, 4.4898804379949233, -2.452526428777031, -2.97643208435613]], [[-4.1481482454383078, -4.0897278918221778, -3.2873289444595422, 1.9464552200129672, -3.9636370071307891], [3.7606951140903284, 0.58153662506157211, 0.2260937978958566, -0.41919366861741381, 1.5751051726965928], [-2.38140818237187, -4.1361317576622989, -1.1624215620580447, 2.357665270042288, 2.358479692540465], [-4.5553353580612495, -4.1922169464067549, -0.5763230951141276, -0.24864011808133135, 4.6058929002657436]], [[-3.8680822069093579, 2.0330054798373185, -2.5175265975912389, -3.1279531767068516, -3.6872200754182649], [0.16089501926166871, -0.58564360660508719, 2.2010371449227613, -2.6404007091452639, 1.1712074306038254], [-4.2190665454588352, 0.080410415785503453, -0.7761380050353992, -0.001868188499232204, -0.46082419928361951], [-0.077830845909891977, 2.1622065776911068, -2.7848728219800476, 4.4964535428718371, 3.3710768838738137]], [[-0.62730370615429276, -2.3222463385013303, 4.8332155959312857, -0.74935717346262276, -3.8787150374368218], [4.212509715296779, -2.9731690137806668, 1.9861975037372641, 3.0799326722440163, -3.6526215053838484], [2.7849033587908325, 1.1749952788417497, 1.048735777682932, -3.0311591923292425, -1.60656462509733], [0.92621881859909472, -3.2012802459895084, -0.5003966919008036, 3.1124374111031106, 1.2006402070660211]]], [[[-1.6006699313149864, -4.3573307132890982, 3.5081372591387208, -2.9161065831054902, 0.37698594932549323], [-3.673827126557935, -3.9401458282903334, -0.42203086224186848, -4.5326151806692225, -1.6452676700361857], [-4.1496161094038966, -4.5970799282965853, 1.5554200865049772, 0.17716354086345554, 2.9811640846893548], [-2.2792103810707074, 3.0451716059109319, 3.4364790896230559, -2.6671638306398138, 0.8142647329511199]], [[3.1929361035550592, 4.6347693501458576, -2.2691549941306297, -4.868032641408143, -1.1510321848311902], [-4.1749634752317411, 0.88335004574970011, 4.040573670543159, 2.2531761998480802, -3.9158529384227583], [-0.92222630074840595, 2.1271210489379744, -3.9383712487267051, -0.29019033296101693, 4.7555352493331053], [2.9695742970636587, 3.0517910390687959, -0.67792858327381023, -4.5891097887672565, -3.7644939314081594]], [[2.7940438256804594, 4.6261303701344225, 0.33673578168270968, 2.185752581358777, 3.9576722818747392], [-0.26192397762758723, -0.072576502771113383, 2.1916851600431073, 3.0625454682566087, 3.426935781445609], [-3.6454989336749453, 3.1969096092471645, -1.8821292929584423, -1.152334967399228, 2.2460266669308222], [-4.281750637789921, -3.8959870068422164, -4.5469943499067069, -2.684910988703769, -4.6705872544815588]], [[-2.6439628705731977, 4.7584275535414431, -3.8665362970885999, 2.1722295673288858, 2.1179398701112166], [-0.1839935947105209, -0.9440342673681581, -3.4706908633133926, -2.8291527164603925, -2.8670316016644479], [-1.2914402328242689, -1.0851012975340391, 1.6175108816370809, 4.5719455351423459, 1.4484607981496431], [-0.25309301028092612, -0.26458227298920001, -4.2466364285354814, 0.83425753113895329, 2.8001167774113132]], [[-0.93185530230959834, 1.5925081678190676, 4.5493734684322931, -3.4073473027806411, 1.4469869166113138], [4.6973521933815725, 3.1407493532956892, 2.9526807840793658, -0.51108585436858078, -3.4498526333939861], [-0.29921962352696152, -0.55577362454190737, -4.7844807879054088, -3.932854849948284, 0.92030161946885869], [-3.2011461258961162, 4.9228365632207272, -2.071975484045312, 3.5794540318986421, 3.4119516438000179]]]])
      s1=numpy.array([[4.3243627372559512, -1.7808746233382102, 1.0786435661717961, -4.3042243237867384, -0.13314732101083671], [-2.7517300444186499, 0.8398188954622432, 1.8530021032100095, 4.1337128293224197, 4.6071865537999432], [-4.2754860801403751, 3.4986299859499415, -3.1584804237284239, 4.0222370242558831, 4.5345143488111219], [1.2546367626608257, 4.6462841669100534, 4.9308600481440372, 1.1497967278669874, -0.83819846907229856]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.47310300624550727, -0.59101590545425964, 3.4255395276996907, 5.3782917226853089, 3.4517516430264621], [3.1190194611921793, 4.4010228059767096, 9.2891236083537159, 7.1252613856424221, 5.345140278334064], [6.3555768602739873, 7.5702065387220259, 2.2966773724104774, 3.1351573324091122, 3.0245297640112878], [5.6336562705682418, 6.3782350186383834, 7.4216009947125556, 4.832288705314788, 0.83320843523869037]], [[-5.9623275880591571, -5.7640666259950191, 1.2625520402074848, 2.3229250830553898, 1.0098692393753872], [2.5438850464484055, -2.2029979257798615, -5.7340399218688436, -3.8556250302293518, 0.77700691354153051], [0.76837715922703786, -0.35764821102683886, 0.74410491516758981, 3.0485741911332198, 1.7528882992454093], [1.8743151369972182, -1.7848171623075464, -3.1835448051844519, -5.4428294551679652, 1.5004859427183526]], [[5.4922215242028658, -1.3178945902848729, 3.322948397707937, 3.1932457818190132, -2.5114791442672315], [-3.55616567783487, 3.1607095689970484, 4.0025492569883108, -0.80787290825943536, 4.9661545597144992], [-2.5552538862950538, -2.3367316031566978, 5.7831014872849744, -2.2141364367005214, 1.3313625176475599], [5.4746988061656578, -0.97902903219338366, -2.317905473759712, 3.5861806214546421, -0.79539602660679076]], [[-5.7944565402011268, 0.088894716372512228, -7.6689472956194527, -7.4428881801694367, -3.4210352133175066], [-1.3562744450526996, -6.8651065811740821, -8.3765449912897694, 0.40516520874910622, -4.8995562760469156], [-0.2814973598689976, 0.25981819748871349, -5.6542403903365335, -2.9597380829841873, -9.122031115900576], [-8.7132178862854417, -4.604829150408305, -2.8898891883999989, -6.1325257168025331, -1.0816399396235203]], [[-5.0855191811762568, -2.1303673077389824, 2.2104514510565147, -3.5544174181993933, 0.48478627361320648], [-2.9802431593296772, 0.27854461141839337, 1.5090503093754339, 3.1522968755635734, -1.4146895859530204], [-0.22940641103627435, 2.9336108264771479, 2.4259904399689871, 0.73269770398610934, -2.8760111177846515], [-3.2002863770477052, 0.37202168872982178, 1.3242874838676615, 4.4644181816111903, 0.049334813003521383]]], [[[-5.4285765981985854, -1.7468764382067135, -6.411869613698804, -1.5331050537496522, -2.5939920692361862], [2.1577978805513678, -0.71735065302804069, -3.3538259804811745, 0.018587707423979527, -0.64807352311675759], [-4.1491844305991465, -4.250555955875619, -1.6384970332009652, -7.3336947291286414, -5.169549201073611], [-2.7915334398215887, -1.2883275521346973, -5.8280803121723039, -6.8131947271755617, -2.4813691267753142]], [[2.7244167652346887, -2.3193741206345422, 2.0385993862128986, 0.5022185526343117, 1.0503686875878069], [-2.1035217974227063, 5.6937281408791014, -2.3517845405071203, -1.1566735336898102, -1.0900504328996297], [-4.0315015356246811, -2.5167599654367914, -0.17752597989773111, -0.74949871607406626, 3.5483917556279065], [2.2628086168019275, 0.50520651543494122, -1.7283252773160793, 2.3495709376581733, 1.5589618626143826]], [[5.9432681107770318, -1.1584543232243329, 4.7082037331366902, -1.1282220345355158, 3.3220408668732011], [6.5000585113158724, 1.7475926163854556, 3.0831497197047906, 4.1791075510736286, -0.19900855986195642], [2.6129550672982349, -0.48198035252690286, 2.1767901217492991, 4.9091760653160286, -2.1358925244845275], [6.7726544677912166, 1.6405032493001457, 1.1948393454342394, 4.4518555816041019, 1.535577970586985]], [[5.2478803423906371, -0.78658857887702283, 3.7564745494824079, 0.061151444190100435, 0.8786353550479129], [7.9715467415958638, 3.5685189930120238, 1.2159918397387655, 5.3736720412318135, 1.7482521638101094], [5.043825448299609, 2.5587149750927489, 6.5059213422032807, 5.2592675900292472, 7.7324495467520418], [2.8709737755515716, 2.5311314992811056, 1.8065057964112938, 2.541450903082187, 0.094772909925223026]], [[7.0035891351630308, 5.0036856553645022, -0.05343318724772228, 8.7276113307017074, 8.0585101908835881], [1.3884877544103809, 1.1360686931505768, 8.3468685746396094, 9.3141664787280103, 4.4744835646673504], [9.570525022447347, 4.4679574855334421, 9.1177369326014848, 4.6661069596164246, 8.6553490512411972], [0.13886789742030636, 1.843003965140066, 3.0910646099640746, 1.257253247417299, 1.5175661486860426]]], [[[-4.9617572808302723, -3.5636062969301543, -6.8865978058256818, -1.1358002754252734, -1.1277984551793168], [-1.0042105388713622, -1.2495936023227809, -2.8182265943554254, -1.7074880102773058, -8.6166101706422218], [-1.7605671597578079, -2.4581997087268919, -7.3139659966705208, -6.3331638979981966, -0.0076320769141258182], [-4.0897512336259902, -1.0474170475893398, -8.6832703143882615, -8.8073618664778142, 0.499890022496146]], [[2.7216278205859377, 8.2182304844510323, 3.8272516166878425, 2.1661007814200586, 1.6676514043229798], [2.1389437823282687, -1.1775962851102095, 5.3275347390798764, 8.3822375117209909, 7.8063757814697663], [-1.369782503460689, 5.7933430378495423, 2.3582878232718385, 1.6041259295934984, 4.1438961858645378], [4.5550302608424529, 4.1026029210216262, 7.9885104239448648, 1.0461035571729105, 0.52219790159381141]], [[-7.3066286691667317, -7.2482083155506016, -6.445809368187966, -1.2120252037154566, -7.1221174308592126], [0.60221469036190456, -2.5769437986668517, -2.9323866258325673, -3.5776740923458377, -1.5833752510318311], [-5.5398886061002939, -7.2946121813907228, -4.3209019857864686, -0.80081515368613587, -0.80000073118795889], [-7.7138157817896733, -7.3506973701351788, -3.7348035188425515, -3.4071205418097552, 1.4474124765373197]], [[0.15415481734652525, 6.0552425040932016, 1.5047104266646443, 0.89428384754903156, 0.33501694883761823], [4.1831320435175519, 3.436593417650796, 6.2232741691786444, 1.3818363151106192, 5.1934444548597085], [-0.19682952120295205, 4.1026474400413866, 3.2460990192204839, 4.0203688357566509, 3.5614128249722636], [3.9444061783459912, 6.1844436019469899, 1.2373642022758355, 8.5186905671277202, 7.3933139081296968]], [[3.9072106426568292, 2.2122680103097916, 9.3677299447424076, 3.7851571753484992, 0.65579931137430014], [8.7470240641079009, 1.5613453350304551, 6.520711852548386, 7.6144470210551383, 0.88189284342727348], [7.3194177076019544, 5.7095096276528716, 5.5832501264940539, 1.5033551564818795, 2.9279497237137919], [5.4607331674102166, 1.3332341028216135, 4.0341176569103183, 7.6469517599142325, 5.7351545558771431]]], [[[-0.34603316865416067, -3.1026939506282725, 4.7627740217995465, -1.6614698204446645, 1.6316227119863189], [-2.4191903638971093, -2.6855090656295078, 0.83260590041895721, -3.2779784180083968, -0.39063090737535999], [-2.8949793467430709, -3.3424431656357596, 2.8100568491658029, 1.4318003035242812, 4.2358008473501805], [-1.0245736184098817, 4.2998083685717576, 4.6911158522838816, -1.4125270679789881, 2.0689014956119456]], [[7.8392202704651126, 9.281053517055911, 2.3771291727794237, -0.22174847449808954, 3.4952519820788632], [0.47132069167831236, 5.5296342126597535, 8.6868578374532124, 6.8994603667581336, 0.73043122848729514], [3.7240578661616475, 6.7734052158480278, 0.70791291818334834, 4.3560938339490365, 9.4018194162431588], [7.6158584639737121, 7.6980752059788493, 3.9683555836362432, 0.057174378142796911, 0.88179023550189406]], [[7.7249038738244966, 9.5569904182784597, 5.2675958298267469, 7.1166126295028143, 8.8885323300187764], [4.66893607051645, 4.8582835453729238, 7.1225452081871445, 7.9934055164006459, 8.3577958295896462], [1.285361114469092, 8.1277696573912017, 3.0487307551855949, 3.7785250807448092, 7.1768867150748594], [0.64910941035411618, 1.0348730413018208, 0.3838656982373303, 2.2459490594402682, 0.26027279366247846]], [[-1.4941661427062103, 5.9082242814084305, -2.7167395692216125, 3.3220262951958732, 3.267736597978204], [0.96580313315646649, 0.20576246049882929, -2.3208941354464052, -1.6793559885934051, -1.7172348737974605], [-0.14164350495728151, 0.064695430332948334, 2.7673076095040683, 5.7217422630093333, 2.5982575260166305], [0.89670371758606127, 0.88521445487778738, -3.096839700668494, 1.9840542590059407, 3.9499135052783005]], [[-1.7700537713818969, 0.75430969874676901, 3.7111749993599945, -4.2455457718529397, 0.60878844753901529], [3.8591537243092739, 2.3025508842233906, 2.1144823150070673, -1.3492843234408793, -4.2880511024662846], [-1.1374180925992601, -1.3939720936142059, -5.6226792569777073, -4.7710533190205826, 0.082103150396560132], [-4.0393445949684148, 4.0846380941484286, -2.9101739531176105, 2.7412555628263435, 2.5737531747277194]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank2_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=add(arg0,arg1)
      s0=numpy.array([[-2.0732973613170822, -2.8155617887710993, 2.5427312002155205, 2.3119078360765108, -3.2370275975911902], [-0.47606538800001097, -1.5394358377850823, 0.14972881763006729, -1.0688340972506296, -1.3320539287934272], [2.2281406173771572, -2.0436899850223722, 0.57338154603671931, -1.3575894416954783, -2.0399889131122553], [1.0854338968792421, -4.487693212967466, -0.27686940017099015, -3.0295363518692398, 0.85171279459365667]])
      s1=numpy.array([[[[3.8595265692118286, -3.4592437427831415, -0.074458582997093181, 4.8758777037517316, 3.0539559787194577], [3.0181657938123791, -4.8818031660861863, 1.9773700376477503, -2.6744832441751321, -3.4540075806771342], [0.037980005694310037, -1.3599092996976827, 0.97937219390603403, 2.3332078649895962, -1.16421183724316], [2.5234791537670551, -2.9752184353974167, -3.3978781340592135, 2.4339706501567182, -4.5309748084786374]], [[0.69769439408101519, -0.28835880998355634, -1.05657038388388, 0.29334521683944637, 0.83646508169744216], [2.9591472499839693, -1.2036482672619666, -0.77115580387608773, 2.3780335811022413, -0.18439902941232678], [3.3579978741992349, 3.6088013959787393, 2.7250712969803192, -3.8363610993318087, 4.9784597793247851], [2.6160545035735714, 4.9000142737204087, 4.5279457151254103, -4.9225345960753542, -1.3224400688679316]], [[3.005051457304809, -0.37381763953885461, 0.76958871248867311, 3.0872956845518011, -1.8098221158637062], [0.65360426358286094, 0.039482002138306882, 1.33125934062413, -2.0048986727045062, 3.4219240744579889], [-3.9332782417890999, 4.5866867184570523, -3.594612189018993, -3.0899861590982503, -0.43921008606177558], [4.6728182792738711, 0.39543565409949899, 1.3572298599301398, 1.5702228368598616, -4.7777475781088432]], [[-1.0283863120056513, 3.3877028749519891, -3.3569004286599369, 2.1008852076844775, -2.108836749891565], [4.9097276520701332, 4.3958426461366109, -4.5372015106314549, -4.2119354377375977, -1.5129822088812963], [1.8107866644817525, 4.6616466294692795, -2.5575915646993308, 0.3952755290154597, -3.9043852852898864], [0.95875173691890136, -4.3490382031474635, -1.6387942874318053, 2.4826735426317006, -3.0295907536008726]], [[-1.7500576559927996, -1.0387875157366344, 2.6397287931741271, 0.076770592935869963, 1.1191501496563756], [-3.6316798352892388, 4.6581378407703333, 4.1985024146211938, 4.9446661617275929, -4.9710112712545005], [1.3580344650116336, 4.0868774267959687, -0.038720990337813177, -3.5889072035922975, -1.4701893038294833], [2.9904372586106778, 4.083942163175486, 2.7944825124417418, -0.12200515420540992, 3.4431466621476794]]], [[[4.3019631941748191, 4.2050601442884723, -3.1853639610024409, 4.8821188642289641, 3.76407124647214], [-4.2491256625767164, 0.74716752589642521, -2.0028805794470772, 4.8117082638445652, -3.883998862998792], [-2.8991546803658732, 0.36183160159868955, -2.3372113459787114, -3.5234644952770733, -2.7555623468741208], [4.6077883497470733, 3.9429199487492976, -4.1478564221000305, 2.6088888567079183, -2.1486342603033326]], [[1.8110813020562659, 3.2922613018114859, 0.65039768412926247, 0.42669998275083287, -2.921681817339421], [3.5442781783995514, 4.7238065251308967, 3.1199601441969929, 4.9766847485039971, 0.00465163013852532], [-3.3243642688675648, -1.0029839225889869, 0.6714579736423465, 0.65106234400544327, 0.37032715108495928], [-2.2060475664609513, -3.6856319814675165, 1.7007728957454784, -2.4955763886339088, -2.9238642350986987]], [[1.6332709318698608, -1.8514848517437574, -1.806919068325211, 0.41345537974350499, 3.7192659891111539], [0.58310206490423333, 2.261462511143046, -2.4945632631610373, 3.5423145098480084, -4.092227841830212], [0.54163127604678607, -1.3014044156828986, -0.9698256927990867, -1.9814869768029753, 3.4361224700940944], [2.0936396806680522, 2.7038548268210985, -3.1234136237932342, 2.3406199162956787, 4.5980363110343365]], [[-1.8347031259741495, 2.1132988040684566, -3.9636570703438645, 1.7068113891928309, 2.0845007063873089], [2.2499654738283725, -0.22555335255288256, 2.0849899535313394, 4.5791240141794596, 0.050988867908154489], [3.0495342119372886, 4.5491764314574397, -1.3200642398559017, -0.42363932709525987, 1.4777877116175659], [-2.926354724924245, 3.6050795443875732, -2.6850067497813788, -3.4893076800547442, 2.1352603704785347]], [[0.76994149165379255, 4.3126607350474657, -4.5436927056670555, -3.0473476374014341, 1.3889720643874757], [-3.2634751041938861, -3.0131565526621209, 0.95252475637841805, 1.1292358015034338, 4.8336615766747091], [0.12076100184231642, -4.6152088130070936, -2.0946279895169639, -4.8905500906102608, 2.4727943004844324], [1.9991639561610022, 2.7838018945993124, -2.9000529098662486, -1.6816986229405364, 3.6705257748547933]]], [[[4.8734967001732628, 4.4968961567631567, 3.2881406902019332, -2.7381072928290253, -0.93372857626881522], [4.3702112222535945, 3.180962459046377, 4.0219135416000089, -4.9580691330495767, 1.6340479835441899], [1.7793929816211271, -3.7082715798194412, 0.10177474570198353, 2.4053775081700826, 3.3455434556693717], [-0.99580410021031884, 0.37973873681603454, 4.0513178984344886, 3.4976902553185099, -2.0033160230010982]], [[0.1680422021135719, 4.0958069377062696, -3.7061469983135673, -1.9179500610237987, -1.8186064231103458], [-0.18109149389770174, 1.3561740967461624, -4.4027234322605917, 4.4221763262274845, -1.225697629849507], [-4.9598345322977133, -0.85256055950428156, 3.6011677121303141, 3.1275656669907352, 2.9136319137153421], [-3.7609315543345567, 2.9536746029393939, 4.2486846229088133, 2.1740991766927307, 3.6463981031427224]], [[0.85887433832394322, 0.014160054423197899, -1.2880885705368894, -2.6783023811423101, -2.3821161270410407], [1.3301827812816125, 0.90338966041676816, -0.39599272668843977, -4.6207285304227952, -3.1034629798006899], [-1.0522623626147407, 1.9865191109324911, 3.8433864975332241, -1.4891775500880202, -4.7347572398913425], [0.31900313778985634, -0.64098088329801772, 3.9792958623940393, 0.96709510519846376, 3.1871729728412372]], [[-1.4266765782848156, -2.2494961762316743, -1.9279804589937632, -4.5223848061634362, 0.14367724959825434], [-0.23582849370598513, 2.1784170948176147, -4.1993053913005767, 2.60018306526828, -3.2616115713509188], [-1.752459917449448, -2.5635200265525837, -0.63731638406356428, -2.310934076550871, -3.6592276613535066], [-4.5888101215958805, 0.11538474295833101, -2.8848128999373057, 0.17059978661027309, 4.4019404043444741]], [[3.8464518099207474, -3.2313935675840444, 2.7958566549751493, -1.908956636584703, 2.5761991635390658], [0.96819939210498607, -3.4212354888054208, -2.4979448872237278, -2.7599240580353013, 3.9099431722601867], [2.1122406628579435, 0.8978300827556609, 2.8764800190034032, -0.87637742328895119, -4.9110575766221043], [2.163839189808848, 1.4876589655637948, 4.5032566337717519, -1.1130708202839146, -3.7331168512247705]]], [[[-4.8600427312705961, 3.4675882652029255, 3.022755923318039, -1.1486319713982596, -4.5437142021183661], [4.1418663981747184, -1.3145131294064227, -1.3714387611726444, -1.4730012160097381, 4.6017495159234176], [-3.2345428522281461, 4.3790562910185873, 3.3700268796559261, -3.0145709652492423, 4.8016785902467536], [1.6804825937765342, 3.9645938552242992, 0.72963117506530573, -3.9723884309409963, -2.9184514952485072]], [[-3.1551130688259432, -0.30716776872523432, 3.7768724011547423, 4.2772226785508121, 4.2533412706185789], [3.2519406087660503, 2.9624212784979989, -3.5686270906490316, -2.1497236928009689, -0.33288013570332353], [-4.6499805223112665, 3.888765294198544, -0.98857306047869908, 0.32091936621255002, 3.5824292495992047], [3.7073058445749627, 2.0774555405654338, 4.9585971114097926, 0.49717023130815363, -3.9001409700827452]], [[3.2164525817722236, 4.2612757737805165, -1.7855679266105016, 1.0520075424122055, 1.517517651912156], [-3.043437460082774, -2.7256869425830663, -0.40225103891807201, -1.2539391585783202, 1.7789632681664393], [3.9666725078965275, -0.30595759042958193, -4.807200641003412, -3.6971722711190811, 0.33094858028336205], [4.4586958385812032, 4.5846430509134208, 1.1449389750037051, 1.3800817147183118, 4.3278083687898867]], [[4.8619189323631957, 4.3400210773027315, 3.6034407650723317, -1.7050755375117097, -1.6508063340757584], [-2.8175748388409461, 3.3929385824001699, 1.2830961694005278, -0.68483114480321738, 2.7897208250225223], [3.6556361620956199, 4.8992811820066908, 2.3924596217844174, 3.0592896866832202, 0.74965566012083951], [4.7193496225085276, -4.8942647349353958, 4.816243869073368, -1.6944391998501054, 0.54732581201237718]], [[1.2591442265404007, -1.6922497716132021, -1.028766060936551, -1.586872232590415, 4.9979815490057717], [3.5427277888685147, -1.0613549977919376, 1.0238796683235023, -4.53915870594642, 1.1590153786859183], [-3.4582550097769218, -1.2594235550576593, 3.6061155238390565, 2.9488012096861986, -2.9461291645240548], [-3.9466682852473589, -3.1457327984073524, 0.67254908280249914, 0.38226762942308312, -2.5889065046553874]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.7862292078947464, -5.5325411041002237, -2.1477559443141754, 2.8025803424346494, 0.98065861740237548], [0.94486843249529695, -6.9551005274032685, -0.095927323669331876, -4.7477806054922143, -5.5273049419942168], [-2.0353173556227722, -3.4332066610147649, -1.0939251674110482, 0.259910503672514, -3.2375091985602422], [0.45018179244997292, -5.0485157967144989, -5.4711754953762952, 0.36067328883963601, -6.6042721697957196]], [[-2.1178673946900841, -3.1039205987546556, -3.8721321726549793, -2.5222165719316529, -1.9790967070736571], [0.14358546121286997, -4.0192100560330655, -3.586717592647187, -0.43752820766885803, -2.9999608181834261], [0.54243608542813559, 0.79323960720763997, -0.090490491790780059, -6.6519228881029075, 2.1628979905536858], [-0.19950728519752792, 2.0844524849493093, 1.712383926354311, -7.7380963848464539, -4.1380018576390309]], [[5.5477826575203295, 2.1689135606766659, 3.3123199127041936, 5.6300268847673216, 0.73290908435181423], [3.1963354637983814, 2.5822132023538273, 3.8739905408396504, 0.53783252751101429, 5.9646552746735093], [-1.3905470415735794, 7.1294179186725728, -1.0518809888034726, -0.54725495888272979, 2.1035211141537449], [7.2155494794893915, 2.9381668543150194, 3.8999610601456602, 4.1129540370753821, -2.2350163778933227]], [[1.2835215240708595, 5.6996107110284999, -1.044992592583426, 4.4127930437609884, 0.20307108618494585], [7.221635488146644, 6.7077504822131218, -2.2252936745549441, -1.9000276016610869, 0.79892562719521454], [4.1226945005582634, 6.9735544655457904, -0.24568372862281995, 2.7071833650919706, -1.5924774492133755], [3.2706595729954122, -2.0371303670709526, 0.67311354864470552, 4.7945813787082114, -0.71768291752436175]], [[-4.9870852535839898, -4.275815113327825, -0.59729880441706307, -3.1602570046553202, -2.1178774479348146], [-6.868707432880429, 1.4211102431791431, 0.96147481703000359, 1.7076385641364027, -8.2080388688456907], [-1.8789931325795566, 0.84984982920477847, -3.2757485879290034, -6.8259348011834877, -4.7072169014206739], [-0.24659033898051241, 0.84691456558429579, -0.44254508514944835, -3.3590327517966001, 0.20611906455648921]]], [[[3.8258978061748081, 3.7289947562884613, -3.6614293490024519, 4.4060534762289532, 3.288005858472129], [-4.7251910505767274, 0.27110213789641424, -2.4789459674470882, 4.3356428758445542, -4.3600642509988035], [-3.3752200683658842, -0.11423378640132142, -2.8132767339787224, -3.9995298832770843, -3.2316277348741318], [4.1317229617470623, 3.4668545607492867, -4.6239218101000414, 2.1328234687079073, -2.6246996483033436]], [[0.27164546427118363, 1.7528254640264036, -0.88903815365581984, -1.1127358550342494, -4.4611176551245038], [2.0048423406144691, 3.1843706873458144, 1.5805243064119106, 3.4372489107189148, -1.534784207646557], [-4.8638001066526471, -2.5424197603740692, -0.86797786414273581, -0.88837349377963903, -1.169108686700123], [-3.7454834042460337, -5.2250678192525992, 0.1613370579603961, -4.0350122264189912, -4.4633000728837811]], [[1.7829997494999281, -1.7017560341136901, -1.6571902506951437, 0.56318419737357228, 3.8689948067412212], [0.73283088253430062, 2.4111913287731133, -2.34483444553097, 3.6920433274780757, -3.9424990242001448], [0.69136009367685336, -1.1516755980528313, -0.82009687516901941, -1.831758159172908, 3.5858512877241617], [2.2433684982981195, 2.8535836444511657, -2.9736848061631669, 2.490348733925746, 4.7477651286644038]], [[-2.9035372232247791, 1.044464706817827, -5.0324911675944941, 0.63797729194220132, 1.0156666091366793], [1.1811313765777429, -1.2943874498035122, 1.0161558562807098, 3.51028991692883, -1.0178452293424751], [1.980700114686659, 3.4803423342068101, -2.3888983371065313, -1.4924734243458895, 0.40895361436693634], [-3.9951888221748746, 2.5362454471369436, -3.7538408470320084, -4.5581417773053738, 1.0664262732279051]], [[-0.56211243713963466, 2.9806068062540385, -5.8757466344604827, -4.3794015661948613, 0.056918135594048458], [-4.5955290329873133, -4.3452104814555481, -0.37952917241500916, -0.20281812728999338, 3.5016076478812819], [-1.2112929269511108, -5.9472627418005208, -3.4266819183103912, -6.222604019403688, 1.1407403716910052], [0.66711002736757496, 1.4517479658058852, -4.2321068386596759, -3.0137525517339636, 2.338471846061366]]], [[[7.1016373175504199, 6.7250367741403139, 5.5162813075790904, -0.50996667545186813, 1.294412041108342], [6.5983518396307517, 5.4091030764235342, 6.2500541589771661, -2.7299285156724196, 3.8621886009213471], [4.0075335989982843, -1.480130962442284, 2.3299153630791407, 4.6335181255472397, 5.5736840730465289], [1.2323365171668383, 2.6078793541931917, 6.2794585158116458, 5.725830872695667, 0.22482459437605895]], [[-1.8756477829088003, 2.0521169526838974, -5.7498369833359391, -3.9616400460461709, -3.862296408132718], [-2.2247814789200739, -0.6875158882762098, -6.4464134172829635, 2.3784863412051123, -3.2693876148718792], [-7.003524517320086, -2.8962505445266538, 1.5574777271079419, 1.083875681968363, 0.86994192869296993], [-5.8046215393569289, 0.90998461791702168, 2.2049946378864411, 0.13040919167035847, 1.6027081181203502]], [[1.4322558843606625, 0.58754160045991721, -0.7147070245001701, -2.1049208351055908, -1.8087345810043214], [1.9035643273183318, 1.4767712064534875, 0.17738881934827955, -4.0473469843860759, -2.5300814337639705], [-0.4788808165780214, 2.5599006569692104, 4.4167680435699435, -0.9157960040513009, -4.1613756938546231], [0.89238468382657565, -0.067599337261298409, 4.5526774084307586, 1.5404766512351831, 3.7605545188779566]], [[-2.784266019980294, -3.6070856179271527, -3.2855699006892416, -5.8799742478589145, -1.213912192097224], [-1.5934179354014635, 0.82082765312213635, -5.556894832996055, 1.2425936235728017, -4.6192010130463972], [-3.1100493591449263, -3.9211094682480621, -1.9949058257590426, -3.6685235182463494, -5.016817103048985], [-5.9463995632913589, -1.2422046987371473, -4.2424023416327845, -1.1869896550852053, 3.0443509626489957]], [[1.8064628968084921, -5.2713824806962997, 0.75586774186289407, -3.9489455496969583, 0.53621025042681048], [-1.0717895210072692, -5.4612244019176757, -4.537933800335983, -4.7999129711475561, 1.8699542591479315], [0.072251749745688265, -1.1421588303565944, 0.83649110589114795, -2.9163663364012065, -6.9510464897343596], [0.12385027669659276, -0.55232994754846043, 2.4632677206594966, -3.1530597333961699, -5.7731057643370258]]], [[[-3.774608834391354, 4.5530221620821676, 4.1081898201972811, -0.063198074519017489, -3.458280305239124], [5.2273002950539604, -0.22907923252718065, -0.28600486429340233, -0.38756731913049602, 5.6871834128026597], [-2.149108955348904, 5.4644901878978294, 4.4554607765351681, -1.9291370683700002, 5.8871124871259957], [2.7659164906557763, 5.0500277521035413, 1.8150650719445478, -2.8869545340617542, -1.8330175983692651]], [[-7.6428062817934093, -4.7948609816927004, -0.71082081181272372, -0.21047053441665398, -0.23435194234888712], [-1.2357526042014157, -1.5252719344694672, -8.0563203036164985, -6.6374169057684345, -4.8205733486707896], [-9.1376737352787316, -0.59892791876892204, -5.4762662734461651, -4.166773846754916, -0.90526396336826132], [-0.78038736839250333, -2.4102376724020322, 0.47090389844232661, -3.9905229816593124, -8.3878341830502112]], [[2.9395831816012334, 3.9844063736095263, -2.0624373267814917, 0.77513814224121536, 1.2406482517411659], [-3.3203068602537642, -3.0025563427540565, -0.67912043908906217, -1.5308085587493103, 1.5020938679954492], [3.6898031077255373, -0.58282699060057208, -5.0840700411744022, -3.9740416712900712, 0.054079180112371894], [4.1818264384102131, 4.3077736507424307, 0.86806957483271496, 1.1032123145473216, 4.0509389686188966]], [[1.832382580493956, 1.3104847254334917, 0.57390441320309193, -4.7346118893809495, -4.6803426859449981], [-5.8471111907101854, 0.36340223053093013, -1.7464401824687119, -3.7143674966724571, -0.23981552684671747], [0.62609981022638017, 1.8697448301374511, -0.63707673008482235, 0.029753334813980459, -2.2798806917484002], [1.6898132706392879, -7.9238010868046356, 1.7867075172041282, -4.7239755517193451, -2.4822105398568626]], [[2.1108570211340574, -0.84053697701954544, -0.17705326634289431, -0.73515943799675831, 5.8496943435994284], [4.3944405834621714, -0.20964220319828097, 1.875592462917159, -3.6874459113527633, 2.010728173279575], [-2.6065422151832651, -0.40771076046400268, 4.4578283184327132, 3.8005140042798553, -2.0944163699303981], [-3.0949554906537022, -2.2940200038136958, 1.5242618773961558, 1.2339804240167398, -1.7371937100617307]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s0=numpy.array([[[2.534948790040346, 0.16939584986736644], [-1.6144618866817497, 1.9840623894076295]], [[-4.7242562139687196, 1.2006962641038923], [-1.0930832250385358, 3.7585329808190231]], [[1.4578992901921497, -2.0488304348256525], [-2.2539855437226128, 0.58107006050980026]], [[2.4184489486033032, -2.442596541708022], [4.9878472500583477, 2.9380245213277387]], [[0.15338736172216016, -4.0828067817489195], [2.6913442060563497, -4.819771679587169]], [[-0.76116100008893905, -4.7371915696297959], [-3.5995921783304787, -1.0528184647957506]]])
      s1=numpy.array([[[-1.048582686147761, -3.45194664351295], [4.506169855395882, -2.6587092224844699]], [[2.1261616112789108, -1.0526159910442754], [1.4401697310682637, 1.3210595104677427]], [[-1.5095464453684015, 0.35976805155253011], [-1.5213532512194385, 3.6701321691814357]], [[4.9435390873693343, -0.8305035774882441], [-2.877131185991646, 3.8606441806834795]], [[2.6261095851551204, 3.7305680694341135], [-4.0692683658236586, 2.8452152706684437]], [[-1.4121331732341003, 3.154894275750749], [-4.8976645425848488, 1.9019756952596705]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[1.4863661038925851, -3.2825507936455836], [2.8917079687141323, -0.67464683307684048]], [[-2.5980946026898089, 0.14808027305961691], [0.3470865060297279, 5.0795924912867658]], [[-0.051647155176251847, -1.6890623832731224], [-3.7753387949420514, 4.251202229691236]], [[7.3619880359726375, -3.2731001191962661], [2.1107160640667018, 6.7986687020112182]], [[2.7794969468772805, -0.35223871231480608], [-1.3779241597673089, -1.9745564089187253]], [[-2.1732941733230393, -1.5822972938790469], [-8.4972567209153276, 0.84915723046391989]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=add(arg0,arg1)
      s0=numpy.array([[[[4.1792729668046569, 4.6972209206976672], [-1.103261205629166, 2.4901344697286829]], [[0.24988906118499976, 3.2116132527042858], [4.0989393693700293, -2.7405602425402709]]], [[[-2.101369951406562, 0.63246802906110045], [1.0726767627576805, -4.3652884747168939]], [[3.7335615365577031, -4.7423058314606319], [0.11922819504636983, -2.8487156090246502]]], [[[-2.6011421072787897, -3.9051224993731526], [-0.10522554895485214, 4.3575607907042979]], [[2.5594848102296455, 0.94594902203709808], [3.0545695742361545, -2.4743453485343627]]], [[[-1.6263785698282218, 0.20721146278201896], [4.943033124527517, -0.22702065744177879]], [[0.17043382457309875, 4.1587967324133803], [-2.0049026596030894, -0.27824488908562461]]], [[[2.6279075899909721, -3.8594898398655686], [3.3096143926969788, 0.94166265043756336]], [[-4.9758992920967486, 2.5421164925284421], [3.125173190771374, -4.6834939129757824]]], [[[3.9973218603503806, -3.3151759105144074], [2.7045364949312294, 4.3499389494161491]], [[4.8673211742572846, -3.1040136364106994], [1.1105633389187419, 3.9609833251998801]]]])
      s1=numpy.array([[[4.0465348765661844, 4.2138117928180456], [-0.094135420059497044, -4.6886575781421787]], [[2.7435935192991252, 3.2826755699088146], [-3.8358663137814464, -1.3750293198030148]], [[-0.80859372622824832, -3.1816686051906018], [-3.7966833192164628, 2.8256062241431241]], [[-1.7538662541590644, -1.0956548610470405], [1.5542403945546646, -1.0492585208042762]], [[0.14968007679991846, 0.68107032224718544], [3.3930694533239354, 4.8473305425053415]], [[4.7006189712585957, 2.9530597099288585], [-3.6632658084601588, 4.9325332750256852]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[8.2258078433708413, 8.7437557972638515], [3.1105505871888797, 6.7039462625467285]], [[0.15575364112550272, 3.1174778326447887], [-0.58971820877214931, -7.4292178206824495]]], [[[0.64222356789256319, 3.3760615483602257], [4.3553523326664951, -1.0826129048080793]], [[-0.10230477722374332, -8.5781721452420783], [-1.255801124756645, -4.2237449288276654]]], [[[-3.409735833507038, -4.7137162256014005], [-3.2868941541454539, 1.1758921855136961]], [[-1.2371985089868174, -2.8507342971793648], [5.8801757983792786, 0.3512608756087614]]], [[[-3.3802448239872862, -1.5466547913770454], [3.8473782634804765, -1.3226755184888193]], [[1.7246742191277633, 5.7130371269680449], [-3.0541611804073656, -1.3275034098899008]]], [[[2.7775876667908905, -3.7098097630656501], [3.9906847149441642, 1.6227329726847488]], [[-1.5828298387728132, 5.9351859458523775], [7.9725037332767155, 0.16383662952955902]]], [[[8.6979408316089764, 1.3854430607441883], [5.6575962048600879, 7.3029986593450076]], [[1.2040553657971258, -6.7672794448708586], [6.0430966139444271, 8.8935166002255652]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank3_Symbol_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=add(arg0,arg1)
      s0=numpy.array([[[-1.7397588715491663, -0.9663236187732247], [-2.1096306743281836, 4.6897577721855743]], [[4.8638353548892113, -4.3692641112662969], [1.3127331149931178, 1.295436288684475]], [[-1.9339647285003423, -1.5449381448113209], [2.8848642496128445, -3.3327173022235748]], [[1.0170493446011521, -0.13919579184666908], [2.0158956214286654, -1.2083406922506024]], [[4.9141084205296028, 1.7692996058294517], [1.9374992777347195, 4.2668584496775921]], [[0.79889640504736903, 0.26039397211969106], [-2.2418708274985164, 1.1907850205403427]]])
      s1=numpy.array([[[[-2.9596457916649999, 0.22384171010344822], [-2.6254479265844566, -2.8382986658100604]], [[-1.1948455592568266, 2.0326303854993446], [1.8171575984103052, 3.3305148613138353]]], [[[3.0942190053352494, 4.2475677685964364], [1.2014987370902084, 4.2237888549766609]], [[-4.1355394440516866, 2.5390465955477435], [0.76155395794672476, -3.3756921595307841]]], [[[2.8403724719542387, -0.08427334987347912], [1.5990088333140129, 2.4775955801287566]], [[4.2535097039572349, 3.9831782154574746], [1.4906468376637445, -3.3223682551331057]]], [[[-1.0898658748219048, -3.9074598108046521], [-2.5811529157345827, 2.4312723569463532]], [[3.1448229273355661, 1.5079615510863107], [-1.6291210398551517, -4.3413999587101877]]], [[[-0.15470855653710913, 1.8629045012290835], [-3.6664134696144277, 2.8441087412224286]], [[4.8749305444140347, -2.1294218458311498], [-3.2478913139897414, 0.38569476584773366]]], [[[4.0171065316617742, 0.53347626322738684], [3.7918094381388396, 3.6479211283163906]], [[2.2863115529174252, -3.3658889213657983], [0.25597394008216501, 1.8634117127362702]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-4.6994046632141657, -1.5159171614457181], [-3.5917715453576813, -3.8046222845832851]], [[-3.3044762335850102, -0.077000288828839025], [6.5069153705958795, 8.0202726334994097]]], [[[7.9580543602244607, 9.1114031234856476], [-3.1677653741760885, -0.14547525628963598]], [[-2.8228063290585688, 3.8517797105408613], [2.0569902466311998, -2.080255870846309]]], [[[0.90640774345389641, -2.0182380783738214], [0.054070688502692033, 0.93265743531743572]], [[7.1383739535700794, 6.8680424650703191], [-1.8420704645598303, -6.6550855573566805]]], [[[-0.072816530220752629, -2.8904104662035], [-2.7203487075812518, 2.2920765650996842]], [[5.1607185487642315, 3.5238571725149761], [-2.837461732105754, -5.5497406509607901]]], [[[4.7593998639924937, 6.7770129217586863], [-1.897113863784976, 4.6134083470518803]], [[6.8124298221487543, -0.19192256809643027], [1.0189671356878507, 4.6525532155253257]]], [[[4.8160029367091433, 1.3323726682747559], [4.0522034102585307, 3.9083151004360817]], [[0.044440725418908755, -5.6077597488643143], [1.4467589606225078, 3.0541967332766129]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_add_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=add(arg0,arg1)
      s0=numpy.array([[[[-3.2970713096458204, 4.7329531663988753, 0.042843324366153013, -2.2044350488664199], [1.2125692784018085, -1.2453844464427677, -3.9026157579325051, 2.3979341558790086], [-1.4338168039011823, -3.9303401798721129, 0.69328504354284881, -1.2738120263988808]], [[-4.1344217191993344, 2.9756446408600459, 1.872585035193759, -0.30349984586805512], [0.5432608356191686, -4.1882020575685166, -3.0561021171884581, 3.8178636145846383], [3.5395458387081806, -1.9875196803785955, 1.2476854254545735, 4.0017133956353774]]], [[[2.4530184206589336, -0.98799754327370781, -0.64858207200974327, -0.37991947016472771], [-4.8843757199737761, 1.2611993323458961, 1.9408806224334683, -1.3751468127697875], [4.9346538203269503, -1.486614746537068, 0.73658260661281805, -1.746500483483564]], [[2.3936642807975339, 1.4064371249541487, 4.3879670478011761, 2.4079571906922439], [-0.15924354168753485, -4.66401985688743, -0.22296906977652053, 2.2128931503678926], [3.9224799518959301, 2.418276638997936, -3.0088038962148431, 4.5528298118926944]]], [[[1.0230195953236407, 1.6459044699522787, 4.3102345194871603, 1.8200607662860699], [1.2709965018886411, 0.91134925088055851, 4.5403762847584268, 3.6686256258424592], [-4.1965329146492882, 3.9259949584845124, 4.0876574863155213, 4.2530950694962737]], [[-1.6996511064591346, -0.032174495080940169, 4.9241463942676251, -2.3858777590581823], [-3.1664126031955755, 2.6862587682696955, -3.8910710370075186, -2.0400646023037972], [-4.79734166336622, -2.3978998010466523, 3.3266903012389761, -2.131770687653689]]]])
      s1=numpy.array([[[[4.7138326740037915, 0.19889177769140343, 0.54805746221327833, -2.8106277698915392], [3.1880048718333338, -3.8269258591376798, 3.0387649399704468, -3.6175862797162903], [1.3491454084176784, 1.8660166677013477, -1.3093374766501076, 1.2167118549523614]], [[4.0408148627301692, 2.6747400519679578, -2.7924311373779434, -1.5706329471422285], [-2.1780572999780503, 2.6174396212780646, -0.73733424446546891, -3.0426584438161806], [-2.5717706296843246, -1.5082790881396804, 4.3819065884248491, -3.9378564385547286]]], [[[-3.7599024430835772, 3.8257571127112797, -0.14643516394782452, 4.3616431730352225], [0.25783134999480506, 4.9213068271240381, 1.2381818043387858, -2.811697324921294], [-0.68027906943792082, -3.4990935730267214, -4.482423400174028, 4.5449355461715797]], [[4.22811201970719, 0.72844629943458639, -0.46476214376261726, 3.1393082874330922], [2.147451467808267, -2.1223645633783175, 0.0087761606105640766, -3.9309755922937195], [3.6644528101737599, 4.8579489121177417, -3.9255967328251398, 4.7982424818875398]]], [[[-4.4965974105149122, -4.5693148352026016, 1.4003387591579628, -2.8784683556229753], [1.7858351351308377, 3.6389291087290214, -3.3615180055782492, -1.22303209817471], [-0.0055733661218306807, -2.6863308525909502, 0.6410918207188967, 0.73731894736944437]], [[-0.90097643890131884, -1.240378160948902, -1.97982856334347, 3.2728934276884054], [-2.0802954010427941, -4.4376806320232056, 1.081482129839336, 0.67980151403445177], [-1.6896857141796318, -4.6916502606951695, -0.57512357688749205, 2.7684520999610047]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.4167613643579711, 4.9318449440902787, 0.59090078657943135, -5.0150628187579596], [4.4005741502351423, -5.0723103055804479, -0.86385081796205831, -1.2196521238372817], [-0.084671395483503886, -2.0643235121707653, -0.61605243310725877, -0.057100171446519354]], [[-0.093606856469165223, 5.6503846928280037, -0.91984610218418439, -1.8741327930102836], [-1.6347964643588817, -1.570762436290452, -3.7934363616539271, 0.77520517076845774], [0.96777520902385605, -3.4957987685182759, 5.6295920138794227, 0.063856957080648868]]], [[[-1.3068840224246436, 2.8377595694375719, -0.79501723595756779, 3.9817237028704948], [-4.626544369978971, 6.1825061594699342, 3.179062426772254, -4.1868441376910814], [4.2543747508890295, -4.9857083195637895, -3.7458407935612099, 2.7984350626880157]], [[6.6217763005047239, 2.1348834243887351, 3.9232049040385588, 5.5472654781253361], [1.9882079261207322, -6.7863844202657475, -0.21419290916595646, -1.7180824419258269], [7.58693276206969, 7.2762255511156777, -6.9344006290399829, 9.3510722937802342]]], [[[-3.4735778151912715, -2.9234103652503229, 5.7105732786451231, -1.0584075893369054], [3.0568316370194788, 4.5502783596095799, 1.1788582791801776, 2.4455935276677492], [-4.2021062807711189, 1.2396641058935622, 4.728749307034418, 4.9904140168657181]], [[-2.6006275453604535, -1.2725526560298421, 2.9443178309241551, 0.88701566863022308], [-5.2467080042383696, -1.7514218637535102, -2.8095889071681825, -1.3602630882693454], [-6.4870273775458518, -7.0895500617418215, 2.7515667243514841, 0.63668141230731568]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_float_rank0(self):
      arg0=-4.44854979991
      arg1=2.70917505155
      res=mult(arg0,arg1)
      ref=-12.0519001335
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_array_rank0(self):
      arg0=1.34102607781
      arg1=numpy.array(-2.11122757215)
      res=mult(arg0,arg1)
      ref=numpy.array(-2.83121123046)
      if not isinstance(res,float):
          self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
          self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_array_rank1(self):
      arg0=2.15488654794
      arg1=numpy.array([2.9902558021569892, 4.8609585250369474])
      res=mult(arg0,arg1)
      ref=numpy.array([6.443662002959039, 10.474814135682417])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_array_rank2(self):
      arg0=3.96360993995
      arg1=numpy.array([[3.6954011568549721, -0.76520904514208432, -3.1823851487946273, 0.12702335249263186, 3.7762499738267152], [-2.4780003241153592, 3.3087789256208282, -2.7704058423336599, 0.89571037973797818, 4.9537148985584203], [-0.033440379985231417, -4.576300859487632, -0.38876800171710801, -3.2769140632252283, -0.90142535792707701], [0.80603880445635578, 3.8247229892729742, -4.2778993008553856, 3.7391928348068788, 3.8495231997834019]])
      res=mult(arg0,arg1)
      ref=numpy.array([[14.64712875740528, -3.0329901774631951, -12.613733408504913, 0.50347102254529952, 14.967581931987509], [-9.8218267158577177, 13.114709038680799, -10.980808134363388, 3.5502465644439449, 19.634593611594081], [-0.13254462250509755, -18.138671574857227, -1.5409247159396056, -12.988409153354526, -3.5728985088008423], [3.194823417326921, 15.159710057829548, -16.955924190966513, 14.820701887422453, 15.258008418721479]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_array_rank3(self):
      arg0=0.90619838224
      arg1=numpy.array([[[-2.490998863890094, 2.8989605588785041], [-4.9678932367143922, -4.6731236739913049]], [[-0.88107963059512961, -4.2181879540314799], [2.1677389145622152, -2.1186733833545688]], [[-3.7370105055128278, -4.1712582760771078], [1.2468397651519609, -2.5555804959544526]], [[-4.9277180078231648, -4.7613745768072508], [3.9375682265649559, 1.8972118223798198]], [[4.1012510857912421, -3.3554713501913334], [-0.45783895236852867, -2.9927594335493524]], [[-4.1412388353565959, -1.6126661650824281], [3.0046133820271717, 0.93880366707822827]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[-2.2573391406190546, 2.6270333686334686], [-4.5018968142519658, -4.2347771133786916]], [[-0.79843293586998465, -3.8225150999278763], [1.964401497495124, -1.9199383924910052]], [[-3.3864728745098693, -3.7799875016865769], [1.1298841780932953, -2.3158629111181996]], [[-4.4654900868246106, -4.314749938741727], [3.5682179568730632, 1.719250284207327]], [[3.7165470991043525, -3.0407227091962885], [-0.41489291796284899, -2.7120337571161302]], [[-3.7527839330698973, -1.4613954698909934], [2.7227757860498873, 0.85074236434733541]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_array_rank4(self):
      arg0=-4.25230891466
      arg1=numpy.array([[[[4.7209408414847758, 0.97438757279976773, 1.3714114369019006, -2.4742532058738198], [-0.82835138434201561, 1.321213301016658, -1.9676351641985992, 0.79337224291968322], [1.2639838951714122, 4.9115262084638793, 3.6651554785290958, -3.6497506372765951]], [[4.8032737739823084, -2.6457647374669859, 0.20000078803398846, -1.6580149148101619], [-3.7144208059498354, 3.4802924414312901, -2.975719336499405, -1.1009728089553672], [2.8253906355659808, 4.6655996173770866, 2.2584643164464051, 0.16961837611989772]]], [[[-2.0945702659010266, -2.968934400245804, 2.2641188801803285, -0.53835303144950775], [2.1323670623008262, -4.0963819023173498, 3.3016184990694839, -1.981080460519812], [-2.5286862195964863, 1.8305753219838206, -0.051001324848437335, -0.40514652751457181]], [[-2.7052150745984935, -2.3817621269106448, 0.47261360631634108, -1.7487322566829242], [-4.5628886789561349, 3.8236872499087795, -2.0855416900291002, -1.0365549061146693], [-4.9416791949100194, -2.5084953000654275, 1.6410408557162608, -2.1340796461729439]]], [[[-0.28045541045862343, 4.8864719266773768, 3.3912347871293775, 1.1079830985127135], [4.7281907670955032, -2.7385044156216685, -2.3332093504355376, -4.3910240577194273], [1.6147436522707892, -0.028280464592316257, 2.0435640258032883, 3.9396501997452269]], [[-4.642011182884147, -2.8869219032689521, -3.7214198783307051, 3.2107705226665626], [-2.5968043503073988, 3.5518980048078657, -2.2158848378441807, -3.5112532638264069], [3.2303676856855095, -1.6605521105386565, 4.0064649328006094, -3.1005125671502065]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[-20.07489882582302, -4.1433969621493043, -5.8316650788031295, 10.521288964460618], [3.5224059761075974, -5.6182070980790533, 8.3669925495180397, -3.3736638612102987], [-5.3748499854226823, -20.885326680831803, -15.585373314960197, 15.519867171173283]], [[-20.425003888652288, 11.25060897922104, -0.85046513389573508, 7.0503916028846758], [15.794864705934998, -14.799278574318278, 12.6536778621193, 4.6816764903179626], [-12.014433787011008, -19.839570845201759, -9.6036879462640776, -0.72126973286460805]]], [[[8.9067598140704067, 12.62482621720272, -9.6277328980383459, 2.2892433948663875], [-9.067483468346822, 17.419081281071467, -14.039501776395918, 8.4241661029249642], [10.752754953965262, -7.7841717606263945, 0.21687338831242478, 1.7228081906933128]], [[11.503410177784827, 10.127988324859087, -2.009699051328071, 7.4361497644443819], [19.402812206121364, -16.259499379654692, 8.8683675204035399, 4.4077516678048321], [21.013546493900314, 10.666896926848182, -6.9782126600817325, 9.0747659040133293]]], [[[1.1925830420575265, -20.778788135040681, -14.420577917011643, -4.7114864070970066], [-20.105727749128132, 11.644966739380795, 9.9215269206225472, 18.671990745122034], [-6.8663888274399545, 0.12025727169660191, -8.6898655245995631, -16.752609665014361]], [[19.739265534924584, 12.276083745194617, 15.824626923814513, -13.653188116459054], [11.042414288437175, -15.103767549803663, 9.4226268498223096, 14.930933555394203], [-13.736521307466546, 7.0611805428991872, -17.036726550016319, 13.184337229304788]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_constData_rank0(self):
      arg0=0.280837930666
      arg1=Data(-1.43732681323,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(-0.403655887916,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_constData_rank1(self):
      arg0=1.69576010201
      arg1=Data(numpy.array([-4.9953018082390948, 1.1771651446750191]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-8.4708335038889846, 1.9961896858117119]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_constData_rank2(self):
      arg0=-0.875069245067
      arg1=Data(numpy.array([[1.8742887846766152, -4.1401689925863581, 1.5781950384607581, 1.009012799273985, 2.4929087249653588], [-1.1947885741591113, 0.49649423675401572, -1.1942936563207738, -4.1059123599126801, 4.3980079107302821], [2.5524900691552235, -0.47192022473657413, 1.7127587250031917, -4.3879100408324403, 2.8445828555383752], [2.2065716798909465, 2.8854140148255354, 1.8521837828405676, 3.3445816680003748, 1.6369830670683791]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-1.6401324718443764, 3.62293455479205, -1.3810299408742277, -0.88295606852355424, -2.1814677559761955], [1.0455227356040053, -0.43446683693641736, 1.0450896482248411, 3.5929576290997596, -3.8485614622411268], [-2.2336055578564933, 0.41296287479204913, -1.4987824844703379, 3.8397251268528385, -2.4892069719262935], [-1.9309030141082346, -2.5249370636589163, -1.6207890645755032, -2.9267405552817762, -1.4324735366868715]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_constData_rank3(self):
      arg0=-2.8794703124
      arg1=Data(numpy.array([[[0.35294359302427125, 2.4573305140186914], [-4.2354410212336013, 2.4199309182432982]], [[0.86183611183907249, 1.1860728994169181], [-1.9053114391234183, 0.94963064045270684]], [[3.932595135373063, -1.6303334525967648], [3.6948084531865266, -4.2167494283035412]], [[4.482854919869709, 1.3193451754549894], [-0.89876967273067354, 0.86457613678018141]], [[2.5806186396626787, 4.7097400635945803], [2.9039855806720567, 4.6698204739969125]], [[3.3492006972871113, 0.019051413995413213], [-1.7363948559953304, 1.790427280185054]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-1.0162905980665569, -7.0758102628810624], [12.195826680579854, -6.968119237149911]], [[-2.4816314981982255, -3.4152617022178449], [5.4862877248394533, -2.7344332369326811]], [[-11.323790943010772, 4.6944967760713521], [-10.639091250969617, 12.142004793646208]], [[-12.908247656578636, -3.7990152645359703], [2.5879805903169526, -2.4895213186713949]], [[-7.4308147605448474, -13.561556692259899], [-8.3619402671942176, -13.446609419130066]], [[-9.6439239781207124, -0.054857981009108714], [4.999897438449417, -5.1554821998109412]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_constData_rank4(self):
      arg0=2.93108936339
      arg1=Data(numpy.array([[[[-3.1846920975243656, 0.66015484688723269, -1.9008782367700574, -1.3222601232507527], [0.19054999903579173, 2.6330323185770173, -4.9129088011855035, -0.55776287711273387], [-0.98595451145030033, -1.3008223827251744, -3.0749376369728401, 3.5682514013752122]], [[-2.7598533375082166, 3.5785684913818745, 1.9813784455668015, 4.7175283931149821], [4.5093922148252936, -0.62788980506437131, -4.2032196685274412, -3.8050947666511092], [-1.221731439287975, -3.5695492060939982, -1.1107446812337631, -4.1051637733931017]]], [[[2.2965632064709425, 4.024579477133992, -0.23366268391865574, -1.6703004064429305], [-0.0031983279793115216, -3.9345987642936198, 3.1137767154913796, -3.6487975897032046], [-1.7281351984720104, -3.0914253699964278, -4.8282387169924377, 2.4833328204299585]], [[1.4429847975145025, -4.3671206984535687, -3.4280257001955383, -4.5872724824514037], [1.6911174563093043, 3.5565844235380055, 1.2596580098180068, 1.7986210760522692], [-2.0035753161678125, 3.4469218906929413, 2.5671146568997205, 2.44502612955639]]], [[[1.2461158822092173, 3.9571158804618243, 0.30363431418485209, 1.6019597550082318], [-1.3299024455481225, -3.5944471824369217, -0.79132523332449001, -0.52279078287122793], [1.5239788818860269, -4.7244097489104275, 2.80317672190629, -3.8282761702640333]], [[1.5794535957285669, -1.1860216658611868, 3.6324360035846404, -1.689738378681116], [4.8490682123716553, -2.8574450127975251, -2.1457187466944605, -1.7217748351549469], [-1.5406133082530662, 4.1113396002569473, 3.230803242361981, 2.2515405328479865]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-9.3346171327221654, 1.9349728499007566, -5.5716439808940494, -3.875662582893499], [0.55851907536756307, 7.7176530224401532, -14.400174730454252, -1.6348528363982915], [-2.8899207812972163, -3.8128266496638865, -9.0129170008151078, 10.458863728468209]], [[-8.0893767620835266, 10.489104041247863, 5.8075972866487673, 13.827497294544175], [13.217431556222865, -1.8404011290044715, -12.320012462407552, -11.153072797217609], [-3.5810040266147229, -10.4626677100752, -3.2556919206050119, -12.032601871161718]]], [[[6.7314319868371495, 11.796402097540467, -0.68488620745486062, -4.8957997549889312], [-0.0093745851207889253, -11.532660587223905, 9.126757810744623, -10.694951804337903], [-5.0653186987391727, -9.0612440197069422, -14.151999147278717, 7.2788704157166615]], [[4.2295173915265583, -12.800421027872501, -10.047849667266727, -13.445705580279633], [4.9568163884293943, 10.424666773826679, 3.6921701940851159, 5.2719191047837972], [-5.8726582979679094, 10.103236090242232, 7.5244424654383648, 7.1665900815505204]]], [[[3.6524770078933386, 11.598660266918721, 0.88997930866708541, 4.6954871984816213], [-3.898062912490909, -10.53564590370385, -2.3194449743786052, -1.5323465029515813], [4.466918290725352, -13.847667163321898, 8.2163614732787256, -11.221019562775876]], [[4.629519634406261, -3.4763354895544385, 10.646994533297608, -4.9527741886621239], [14.213052259629499, -8.3754266834793167, -6.2892933952601675, -5.04667590547324], [-4.5156752809158558, 12.050703771592467, 9.4697730188893825, 6.5994665070695762]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_expandedData_rank0(self):
      arg0=-1.17083185491
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.23433155461)+(1.-msk_arg1)*(-1.87754387387)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.44519470366)+(1.-msk_ref)*(2.19828817652)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_expandedData_rank1(self):
      arg0=-1.16665744991
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.1057121744861291, -4.8397191741966044])+(1.-msk_arg1)*numpy.array([2.1831485554385086, 3.5158637586579076])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([3.6233022456534902, 5.646294430069096])+(1.-msk_ref)*numpy.array([-2.5469865264717724, -4.1018086469216088])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_expandedData_rank2(self):
      arg0=4.42524560372
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.6920919574131994, 1.2240450320359351, -4.6665809339157116, 0.63103581663606612, -4.6241857616964008], [4.8030910214540583, 3.2387156610344512, 1.1784501465575445, 2.0384664633684499, -3.7122761428795013], [-1.897926252666795, 0.69501952689657998, 3.308869367446226, -2.1873024790312878, 3.6521889673789723], [-3.6549648580140683, -0.72659879571344543, 3.272013716146386, -4.5927559920609795, 3.6201976768991031]])+(1.-msk_arg1)*numpy.array([[-3.2888647105427369, -0.76120026416540298, 0.52975351423414985, -3.5270115801554414, 4.1674173546320272], [-0.023361659524956835, -0.67547065021336827, -2.9422634278507296, -4.0560470285868373, 1.1077521374248178], [-2.272375160294442, -4.2368419889078179, 2.4916666260482465, -3.9256794927894623, -2.0195082723727986], [-1.741445021929855, -4.9784413742482663, 1.3282315072935358, -4.5295472507331542, 2.5112549199978265]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-16.338413703083809, 5.4166998967760014, -20.650766762228077, 2.7924884733605051, -20.463157712745495], [21.25485742697099, 14.332112240701537, 5.2149313302604998, 9.0207147553580072, -16.427733681083293], [-8.3987898058042045, 3.0756321059007306, 14.642559621585118, -9.6793506793456281, 16.161833171859442], [-16.174117169688817, -3.215378126401351, 14.479464312698154, -20.324073262840319, 16.020263854305977]])+(1.-msk_ref)*numpy.array([[-14.554034101568966, -3.3684981225507364, 2.3442894099214819, -15.607892489362982, 18.441845327464318], [-0.1033810811084888, -2.989123525300625, -13.020238299091409, -17.949004281747641, 4.9020752761539326], [-10.055818188102329, -18.749066385083335, 11.026236782863325, -17.372095917092107, -8.9368201039999597], [-7.7063219274203956, -22.030825804784836, 5.8777506383770941, -20.044359058162495, 11.112919794548137]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_expandedData_rank3(self):
      arg0=-4.50616408665
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.58501595631213732, 2.1514227593173096], [-4.8293886289765089, 1.2805562510292843]], [[-2.6309342857935683, -2.1390446359994151], [-4.7359785735158813, 4.6375650934806441]], [[-0.50950331523989334, -0.80807807015187194], [-2.6322647732602764, 4.7058824897715485]], [[2.1054399880074071, 2.0609937135606557], [-3.210116528746727, 2.3858202482200541]], [[3.5267345997076269, -3.8780009385187189], [-1.933818656902635, 3.2060305076411186]], [[1.9483214298607852, 0.96851830140896311], [-4.2601391249561313, -2.4332004200678234]]])+(1.-msk_arg1)*numpy.array([[[-3.654047674014004, -1.886321865161861], [-4.4667161124468793, -0.66563275415400192]], [[-1.2202082059304642, 2.6015952664738471], [2.0655179829694736, -2.6911420535427091]], [[3.9446622551457295, 4.558075802342417], [-4.8501169957582206, -3.4368684216259848]], [[1.9005566816883421, 0.46526961828663893], [-4.8704486940396432, 4.5509594698612545]], [[-3.7804461039185187, 4.4394158610026331], [3.3609330092800604, -2.9167810391110169]], [[3.147115050945585, -1.7318071078084376], [-4.8713950877842072, -3.480543144384014]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.636177892452316, -9.6946639732420987], [21.762017600381029, -5.7703965893262943]], [[11.855421592985248, 9.6388861184868482], [21.341096563132147, -20.897629273754887]], [[2.295905541164303, 3.6413323789296794], [11.86141698782547, -21.205478671414557]], [[-9.4874580605606695, -9.2871758548632251], [14.465311815807512, -10.750897519737132]], [[-15.892044796356654, 17.474908557157043], [8.7141041818328784, -14.446899534244116]], [[-8.7794560564937676, -4.3643023870745763], [19.196885929019761, 10.964400348536966]]])+(1.-msk_ref)*numpy.array([[[16.465738399557349, 8.5000758446593974], [20.127755731179395, 2.9994504116682363]], [[5.4984583958023165, -11.723215157789122], [-9.3075629551915799, 12.12672767375393]], [[-17.775295388110639, -20.539437484754359], [21.855423022347736, 15.487093052080457]], [[-8.5642202636711122, -2.0965812445336858], [21.947040990964133, -20.507370122899065]], [[17.035310465002315, -20.004736318564753], [-15.144915624062117, 13.143493967070501]], [[-14.181416819133982, 7.8038069942156039], [21.951305596467719, 15.683898519267185]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_expandedData_rank4(self):
      arg0=2.8935266097
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.5951124067042599, 1.0271905581041807, 1.6348119035804025, 0.44083281188346213], [3.8720786930389561, -1.9554549819716947, -1.2369402218836711, -3.7825106553767229], [1.7373953816264764, 1.0700062734830826, 1.3055846167652616, 3.6627851304949939]], [[4.0464473959694907, -3.3236947470680835, 4.925693699928761, -1.1583687379673222], [-4.0051477231114108, -3.6064933808243014, 4.1524526663166696, 1.3760834748646236], [4.7253093762196041, 3.7344081555360003, -3.3092080591147277, -0.61382761773075956]]], [[[0.0061932269980857768, -1.4266206503140131, 2.373862175368366, -3.269188504089283], [-3.8638835538493534, -1.2887495110229343, -4.827180158791978, 0.86441864641853705], [-0.41218614327818948, 1.9986103998833151, -1.1792600902886652, -3.1937132819168634]], [[-0.8151392298641813, -0.60967483201078387, 3.8391360289651431, -4.8020482472719825], [-2.1152740999202155, 0.27052741207230113, 0.31700722558265948, -4.9177656372988281], [-3.1319952763663084, 1.3080522714398866, -4.2254255587665304, 0.29715747426081496]]], [[[4.9017206874493251, -2.5254960515025795, 4.4742170581081808, 3.4769810898376381], [1.5116404520176028, -1.9210731995398369, -2.8467100667199396, 0.17556144337441104], [-2.2003319243819606, -3.2286041588686922, -4.8176166205861604, 0.55747344726509063]], [[-0.12446184332428167, 3.9405111434776607, 1.9072756650424179, 4.9251902256682829], [-2.5692466182250753, -2.1597873246316746, 3.1524520235141367, 0.79361295695951561], [-4.5308703016801664, -3.0945170880141348, 2.321224688416569, 3.8805695370528319]]]])+(1.-msk_arg1)*numpy.array([[[[-3.0371886859790598, 1.1875477124718952, 2.993708007078224, 1.6113025080436199], [2.0331761331214562, -2.6504429448708402, -2.4076201594973012, 0.42681897185080508], [-4.970739387705196, 3.2153252181909, 1.4880715821077217, 2.7218168330862946]], [[4.7679072865744541, -2.7117529858711276, -2.9485861522342915, -4.080096306366114], [2.6096547596008648, 3.7902058849700602, -1.8383835483364153, 2.3937511836132401], [3.7412867699241019, 0.037085206691212846, 4.3187641804448926, 2.3612416748434857]]], [[[-4.2678704561792511, 2.4134454330075696, -3.6845942297234724, -0.14936940304468216], [1.9829156276765856, -4.3074288357727974, 2.2365643229603913, -3.5495188754554841], [-4.9709111144061779, -1.7014597132970311, 1.3480777047517547, 4.0974474904771832]], [[-3.7094391421450723, 4.6039619685394868, -4.651984013288077, 2.216817617341369], [-4.5018513191775877, -3.0162944752470864, -4.6076109136004657, -3.0003721328081578], [-2.0591122746057944, -2.7621836649482781, -0.60313787330780322, 1.9977802646399523]]], [[[2.4712321284544982, -0.23408420845569911, -3.6814578071954038, -4.8618596735127015], [-0.53613585107038197, 2.7632213355463975, 3.13750628580358, -2.8289944247616514], [1.0600035508576777, -0.60445501982743899, 1.6728872553244747, 0.90631877173829789]], [[-3.2119794934854715, -2.7886369973133638, -1.4500564705854924, 1.4498377435962944], [-1.137101824704124, 4.1303029561468492, -1.0799965816899237, 4.9239658536644253], [-2.8787096963819336, -3.4262520820485989, -3.1127850311278782, 3.2315534293536867]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-13.296080023364031, 2.9722032131076324, 4.7303717448651472, 1.2755614716139259], [11.203962733162847, -5.6581610244066587, -3.579119446629337, -10.944795232808513], [5.0271997683070966, 3.0960916248698513, 3.7777438298260129, 10.598366240702861]], [[11.708503214991325, -9.6171991931635255, 14.252625791978353, -3.3517707671537207], [-11.589001512604542, -10.435484565124108, 12.01523228550939, 3.9817341516900218], [13.672808419159054, 10.805609369526264, -9.5752815760840608, -1.7761265456730657]]], [[[0.017920267118877214, -4.1279648136319373, 6.8688333721900614, -9.4594839287095596], [-11.180249879847532, -3.7290310033834659, -13.967574239283239, 2.5012183553333904], [-1.1926715737252953, 5.7830323744866812, -3.4122204510081562, -9.2410943649806008]], [[-2.3586270522228427, -1.7641103496879316, 11.108642258070843, -13.894854384547493], [-6.1206018949295782, 0.78277826548463614, 0.91726884269077835, -14.22968573179527], [-9.0625116736224225, 3.7848840542905924, -12.22638129159988, 0.859833059045082]]], [[[14.183259242454422, -7.3075900277164498, 12.946266115212248, 10.060737304870914], [4.3739718722127403, -5.5586764220511418, -8.2370313281566467, 0.50799170804129923], [-6.366718973372878, -9.3420520458765068, -13.939901887001819, 1.6130642538630504]], [[-0.36013365555119298, 11.401973849474254, 5.518752888834598, 14.25116897580836], [-7.4341834567174718, -6.2494020951157667, 9.1217038158425794, 2.2963402087655163], [-13.110193783013637, -8.9540675383420378, 6.716525403027271, 11.228531216255814]]]])+(1.-msk_ref)*numpy.array([[[[-8.788186281561936, 3.4362009063264769, 8.6623737801545211, 4.6623466833014904], [5.8830492433950541, -7.6691271884769323, -6.9665129975569862, 1.2350120525753456], [-14.382966688211733, 9.3036290776766783, 4.3057747199679284, 7.875649433266144]], [[13.796066606288452, -7.8465294235530969, -8.5318124924845549, -11.805867232611384], [7.5511054890368614, 10.96706158440459, -5.3194117159471821, 6.9263827467871595], [10.825512823296105, 0.10730703238727021, 12.496459077138995, 6.8323156180935802]]], [[[-12.349196731709599, 6.9833685814677313, -10.661471449654064, -0.43220434238487804], [5.73761913347332, -12.463659955700161, 6.4715583827928445, -10.270627317764907], [-14.38346358399062, -4.9232189557584718, 3.9006987106432787, 11.856073345546577]], [[-10.733360864861643, 13.32168646601845, -13.460639530350727, 6.4144207646302798], [-13.026226584955991, -8.7277283268202801, -13.332244785649728, -8.6816566052844752], [-5.9580961589329453, -7.9924519354081021, -1.7451954857343432, 5.7806303560703602]]], [[[7.1505759224300824, -0.67732888607726194, -10.652396127609833, -14.067920337939157], [-1.5513233514866152, 7.9954544628958644, 9.0784579260754796, -8.1857706467424123], [3.0671484807837883, -1.7490066842377838, 4.8405437883103293, 2.6224574828959071]], [[-9.2939481342127888, -8.0689953565217305, -4.1957769832076224, 4.195144090844118], [-3.2902343877204623, 11.951141509735859, -3.1249988475054562, 14.247626222835025], [-8.3296231080841903, -9.9139515709496209, -9.0069263178461512, 9.3505858385040419]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_Symbol_rank0(self):
      arg0=-2.21126432461
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-0.550544750439)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(1.21739996575)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_Symbol_rank1(self):
      arg0=-1.23524190914
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-1.2899234771375356, 2.7052987241998663])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([1.5933675385441561, -3.3416983608752355])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_Symbol_rank2(self):
      arg0=2.49896878295
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-1.0853675364924111, -3.3645054943097588, -3.3976014519590048, 4.3672559323322524, 2.2973939719839001], [2.7169032432993943, -0.65537111622792033, 3.275817818758993, -2.980467347291238, -4.5086847712198441], [-4.6189759328512388, -0.40747991292448216, -4.8670312379598926, 3.0666780478381934, -3.4197723238002764], [-2.1211738364437869, 0.078646347801520022, -4.3681739438901657, 4.0425825812108602, -3.8211546670819461]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-2.7122995917174224, -8.4077942003300272, -8.4904999653371913, 10.913636242033558, 5.741115818115837], [6.7894563912896357, -1.6377519606979771, 8.1861664676966299, -7.4480948594703582, -11.267062495421934], [-11.542676665373628, -1.0182795820757915, -12.162559129284274, 7.6635327088930962, -8.5459042819592241], [-5.3007472004745999, 0.19653476804870387, -10.915930324259167, 10.102287672926769, -9.5489462278457875]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_Symbol_rank3(self):
      arg0=-2.17987893882
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-1.9616769441584845, 4.8145719375821034], [1.9650406410012735, -2.5189164845835856]], [[-1.4656747744859908, 3.4546721342631201], [-1.0649914865833887, 0.73193120490696373]], [[-2.5570571241346007, 2.8600958152627323], [-3.7717244768994584, -2.114834353308348]], [[1.6867665286846343, 1.8679032101803417], [4.720989833533908, 0.37275859594037719]], [[0.66224874069098583, 2.869149105839254], [2.1603281472307376, 3.5247343136670981]], [[2.6179725560223055, -3.4640850575508164], [0.2756059870994596, 0.041777913057110538]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[4.2762182553481827, -10.495183966189458], [-4.2835507072523678, 5.4909329934009614]], [[3.1949935720679843, -7.5307670260231756], [2.3215525116302511, -1.5955214182449422]], [[5.5740749702715053, -6.2346626307105852], [8.2219027502410178, 4.6100828658788569]], [[-3.6769468305933137, -4.0718028676343216], [-10.291186308523939, -0.81256861255612445]], [[-1.443622082095158, -6.2543977081654001], [-4.7092538290974844, -7.6834940953140327]], [[-5.7068632372928958, 7.5512860592507929], [-0.60078768669197824, -0.091070792781225637]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_float_rank0_Symbol_rank4(self):
      arg0=-2.12910647742
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[-2.6851448766629229, 4.7780370646014507, -4.6762834480624456, 2.4269647530912373], [-1.8365758507522334, 0.467661862255496, 4.5113833566294588, 0.72573031661233678], [-1.4496277671338764, -2.3875306121311168, -3.5591146365902757, 4.8432515113205401]], [[-4.1351342821106707, -1.3861222133256481, -3.4635862282321273, -4.6834994013966389], [-3.3440230824151884, -2.8439652319277711, -4.7681931741304018, -4.907886058276965], [2.8514870056534258, 0.017199063533782422, -0.91037740220389196, -2.780908962184979]]], [[[-4.0718918232440853, 4.3582804794114374, 2.9263360140956109, 4.1559299248007733], [-3.8063058790623616, -2.5777643109093429, -0.5291928566930526, -1.0482361531178128], [-2.7381772603843202, 1.8425612933137909, -0.55915232328442954, -0.34372353323186022]], [[-4.4636242918530931, -3.433583866368334, 4.0501435214663868, -2.481282285439026], [1.1914903537547596, -2.2901268307673406, -4.6817616337143146, 1.4489141515429793], [4.243035491586328, -1.4816773555251106, -4.8713557406503813, 4.673996997253056]]], [[[-0.30683640269498547, -2.957179582362901, -2.5089558581201143, -4.8259109153641653], [4.3867077884334584, -2.9581914426725788, -2.2194713331256377, -3.6935518081419927], [-4.8938828797574985, 1.7933660717189825, 4.71264143312969, -1.9980978159239058]], [[1.2400838091513391, -1.8308416759354107, -0.22935139834583929, -2.2567589112241446], [1.957733696867372, 2.5549515819292061, -1.8079885880470625, 3.0453694782576655], [-0.68765910867011204, -0.1365746116510973, 1.438506348146543, 1.7846684160950907]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[5.7169593497252214, -10.172949663615482, 9.9563053795409555, -5.1672663762865856], [3.9102655401172957, -0.99570190017240345, -9.6052155267431534, -1.5451571179623842], [3.0864118688586015, 5.0833068913367372, 7.5777340266593516, -10.311798164546724]], [[8.8041411850603701, 2.9512017828930963, 7.3743438736460014, 9.9716689125255762], [7.1197812054259524, 6.0551047968664093, 10.151990972650518, 10.449411997137023], [-6.0711194540274196, -0.036618637575405143, 1.9382904239328504, 5.9208512845148284]]], [[[8.6694912562392954, -9.2792431991459949, -6.2304809627304483, -8.8484173226140665], [8.1040305021691861, 5.4883346916298077, 1.1267079389917529, 2.2318063834732778], [5.8298709414196894, -3.9230091846453576, 1.190494833371625, 0.73182400104705869]], [[9.503531392572075, 7.3104656506637768, -8.6231868060514234, 5.2829141862459563], [-2.5368098299676158, 4.8759238695095179, 9.9679690200968825, -3.0848925052816316], [-9.0338743490768891, 3.1546488551011551, 10.371635061255903, -9.9514372823123729]]], [[[0.65328737248740953, 6.2961502037152091, 5.3418341690947289, 10.274878189373613], [-9.3397679669205171, 6.2983045620547928, 4.725490791814944, 7.8639650794166904], [10.419597739046701, -3.8182673196895363, -10.033715401053716, 4.2541630024105768]], [[-2.6402704706128932, 3.8980568713721162, 0.48831354782440622, 4.8048800158719329], [-4.1682234950717918, -5.4397639625904768, 3.8494002139198913, -6.4839158823081107], [1.464099462529233, 0.29078189031803509, -3.062733183654522, -3.7997490847623037]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_float_rank0(self):
      arg0=numpy.array(4.19775929166)
      arg1=2.5079501433
      res=mult(arg0,arg1)
      ref=numpy.array(10.5277710171)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_float_rank0(self):
      arg0=numpy.array([-2.1437601280394314, -3.4995964748586026])
      arg1=0.949690700915
      res=mult(arg0,arg1)
      ref=numpy.array([-2.0359090585923636, -3.3235342291297059])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_float_rank0(self):
      arg0=numpy.array([[-1.954929502941245, 2.4683690942483816, -0.90570104955162467, 0.6944042946914788, -0.92819013805677564], [-3.9113066080117367, -0.50981783313509244, -2.4957034528250066, -2.2841067256694081, -1.1006029351516808], [-4.4702174359410565, -1.9382235040788376, 2.4357263842537993, 3.5064747477439688, -2.9520256581363435], [-3.9327566453323826, -4.2062699828507348, 3.0384634066296439, 2.5616025243685723, 1.9062520846897844]])
      arg1=-2.08267556715
      res=mult(arg0,arg1)
      ref=numpy.array([[4.0714839112771513, -5.1408120033001969, 1.8862814470436167, -1.4462188582782294, 1.933118922200777], [8.1459827081398402, 1.0617851447680022, 5.1977406040514618, 4.7570532703155131, 2.2921988421743906], [9.3100126336840194, 4.0366907356215735, -5.0728278287489053, -7.3028492839561263, 6.1481117118015582], [8.1906561767820119, 8.7602957221212385, -6.328133498668044, -5.3349869902531397, -3.9701046416128749]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_float_rank0(self):
      arg0=numpy.array([[[-4.0706945402901207, -4.0265993368078723], [4.0116214775492143, 1.8984274399202565]], [[2.2020192138679224, -4.8191506700331388], [-1.3575873912738903, -1.9550594085828865]], [[4.7832399868765254, 1.5803425138666283], [-3.7053240914972276, -2.6314702021564895]], [[0.55209101084953538, -4.6153509303834097], [-2.7866446645661336, 1.5919034450366514]], [[4.7793499000478192, -4.2205416417027752], [-2.3027006407253769, 0.68054656771940092]], [[-1.4168435612081365, 3.6678088387443371], [-3.6562155444235245, 2.0198208217407583]]])
      arg1=1.97269113178
      res=mult(arg0,arg1)
      ref=numpy.array([[[-8.0302230198109328, -7.943236802947518], [7.9136901128149315, 3.7450109750563296]], [[4.343903775203902, -9.5066957894805117], [-2.6781006073807969, -3.8567283574122779]], [[9.4358551032813391, 3.1175276622778036], [-7.309459975663132, -5.1910779313344237]], [[1.0891050410377034, -9.1046618504146508], [-5.4971892172084802, 3.1403338086720138]], [[9.4281811634925017, -8.3258250678904435], [-4.5425171331004437, 1.3425081789026019]], [[-2.7949947283132657, 7.2354539692510631], [-7.2125839803562934, 3.9844826228302783]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[-0.27189798420511657, -3.7946201285224559, -0.20373264680109315, -2.4438264346644853], [0.65798856027649766, -2.7602192762177857, 0.83805616893583323, 4.507959797191246], [-0.67446700525874093, -2.3886371286841124, -2.3796485912325438, 2.9564878759089712]], [[4.8548405186856165, 3.1259102504021943, -2.0343680007502476, 1.3853951583984303], [0.43815658754806996, -4.7763514425536311, -1.7779861258570118, -4.9256237299857819], [3.0813918058539365, 4.5220563607579205, -4.0524621885510026, -3.7520122826001767]]], [[[-3.9241574290788641, -1.6777111412525691, 0.83792463884423007, -0.92307934477725784], [-0.42248059204129618, 4.9188799134017085, -3.3826461990738697, 0.4181331774325594], [4.3230821391341543, 0.41798542344484169, 0.044211159779807829, -0.77748368461909489]], [[-4.8506642836934644, 0.71085849074759722, -2.3794098652717057, -4.1181693901846543], [-0.82520166449058863, -0.45036574702177834, -2.139464731145162, 1.5357003435103254], [0.25379098082110385, -3.0010437503019993, 0.84662328201442616, -1.5428464641540929]]], [[[3.5376781628340765, -3.3539227213802558, 1.2254242792791592, 0.80452917103704635], [3.4559649551979721, 1.1713557775762986, -0.42454172028622228, -2.3331412829672549], [3.8402433799692446, 4.3856370313246753, 0.85637640528115, -0.54451793422383155]], [[-4.2448206241310409, 3.3610538544001205, -1.5299069476661273, 4.1481066994531854], [2.1136149960694075, -4.7674114637620004, -3.3879326840289026, 3.3240760247391936], [-3.8779204721505947, 1.5565207991711763, -3.2847462863880805, -3.5492210633731904]]]])
      arg1=-1.87618128468
      res=mult(arg0,arg1)
      ref=numpy.array([[[[0.51012990930920143, 7.1193952676225987, 0.38223937900753835, 4.5850614197358341], [-1.2345058223275542, 5.1786717476664244, -1.5723452996721718, -8.45774980360234], [1.2654223724039495, 4.4815162767407077, 4.4646521509973844, -5.5469072211783468]], [[-9.1085609212880865, -5.8647743094094151, 3.8168431691695353, -2.5992524680802647], [-0.82206118931910788, 8.9613011855970441, 3.3358242937624101, 9.2413630575993562], [-5.7812496369246897, -8.4841975123445064, 7.6031537150527688, 7.0394552245224782]]], [[[7.3624307265951394, 3.1476902443254842, -1.572098525375933, 1.7318641909503294], [0.79265017993049369, -9.2287104351369695, 6.3464574914130463, -0.78449364200481575], [-8.1108858015992418, -0.7842164287383222, -0.082948150553091057, 1.4587003382302353]], [[9.1007255473553652, -1.3336993964000254, 4.4642042578174923, 7.7264323370268464], [1.5482279190081045, 0.84496778582541332, 4.0140236878180522, -2.8812522433783077], [-0.47615788843838991, 5.6305021188373194, -1.5884187568940074, 2.8946596611882454]]], [[[-6.6373255603479002, 6.292567040133223, -2.2991180985820914, -1.5094425736827966], [-6.4840167694694664, -2.1976757875962281, 0.79651723016895948, 4.3773960096289759], [-7.204992758133538, -8.2282501195925821, -1.6067173842342599, 1.0216143573660585]], [[7.9640530118393107, -6.3059463384436913, 2.870382782520652, -7.7826001563902887], [-3.9655248986548579, 8.9445281647027066, 6.3563758955474476, -6.2365692264855932], [7.2756818133455363, -2.9203151926278088, 6.1627795074596792, 6.6589821343103655]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_array_rank0(self):
      arg0=numpy.array(-2.77114015802)
      arg1=numpy.array(3.1308073462)
      res=mult(arg0,arg1)
      ref=numpy.array(-8.67590596408)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_array_rank0(self):
      arg0=numpy.array([-0.92072140982667694, -2.2472115253571214])
      arg1=numpy.array(2.01772635654)
      res=mult(arg0,arg1)
      ref=numpy.array([-1.8577638556357032, -4.5342579234280294])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_array_rank1(self):
      arg0=numpy.array(-2.09317664238)
      arg1=numpy.array([-0.71680737286581309, -2.8771230033550066])
      res=mult(arg0,arg1)
      ref=numpy.array([1.5004044499660163, 6.0223266678669605])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_array_rank0(self):
      arg0=numpy.array([[-4.7688943271361932, 4.730622189990429, 3.1759093360955362, 3.3168978362786472, 4.7224122515899829], [4.0274244088670024, 3.2646319379204218, -3.4308358271346409, -1.2186371788676063, 4.6038024416189121], [-0.87965675689562151, 4.6903687274740165, 3.197279061970498, -2.5473025720045452, 2.8266427790961988], [4.6418513414531883, -3.0353016529126799, 0.69837993735169857, -1.550860716436222, 0.37042755791200577]])
      arg1=numpy.array(3.23495911736)
      res=mult(arg0,arg1)
      ref=numpy.array([[-15.427178183290261, 15.303369384289761, 10.273936862707437, 10.730028896817545, 15.276810569208283], [13.028553310937999, 10.560950852396651, -11.098613639150694, -3.9422414525302649, 14.893112683034163], [-2.8456536458658328, 15.173151078717028, 10.343067052262104, -8.2404196799778244, 9.1440738297538857], [15.016199318458531, -9.8190767560243462, 2.2592305457163993, -5.0169710143890782, 1.1983180057884268]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_array_rank2(self):
      arg0=numpy.array(3.91599762955)
      arg1=numpy.array([[4.0696297762207827, 1.3942838384412557, 4.9540649809475852, -1.0736512872275084, -0.19350257554202965], [4.3524823498115399, 3.5892345998804771, 4.8150552880658406, -0.36113340969230467, 3.7371533438139863], [3.0163551457117137, -3.6395254947697486, -1.1300784399794104, 3.5039913439453603, -3.3563973350274292], [-1.9205729412141235, -4.1032549900214201, -0.89851335658652864, -1.5720732489220546, -0.58549414211013229]])
      res=mult(arg0,arg1)
      ref=numpy.array([[15.936660556830132, 5.4600122062570149, 19.400106722031609, -4.2044158957471396, -0.75775562713457201], [17.044310564523894, 14.055434185033834, 18.855745094222108, -1.4141975763066803, 14.634683635643595], [11.812039600490573, -14.252373210208212, -4.4253844921658914, 13.72162179685672, -13.143644007798196], [-7.5209590851740078, -16.068336814366567, -3.5185761745126221, -6.1562351162590661, -2.2927936726191853]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_array_rank0(self):
      arg0=numpy.array([[[2.4941751641530887, -4.3849024119766353], [2.5446778771683718, 2.0719522065353546]], [[-4.3275368984766569, 0.93312403040831882], [-0.6609205636158979, 2.5015716444976066]], [[3.1737416343220808, 3.887358201548027], [1.6977146009765711, -2.4055032122304354]], [[-1.4926494929887548, -0.16622474231382434], [4.1534891346986065, 2.3366965522212393]], [[1.6429698641382071, 1.9202700914996349], [-3.4387172364934369, 3.6133040768628835]], [[-1.6458746464566554, 0.4634488452754324], [1.7686981158662194, -4.5171772502405636]]])
      arg1=numpy.array(-4.89509934803)
      res=mult(arg0,arg1)
      ref=numpy.array([[[-12.209235219911596, 21.464532938030036], [-12.456451017466318, -10.142411895354881]], [[21.183723050297043, -4.5677348328803413], [3.2352718200539985, -12.245441726023742]], [[-15.535780604977024, -19.029004597946212], [-8.3104816363767924, 11.775177205866711]], [[7.3066675599624906, 0.81368662772640266], [-20.331741955301503, -11.43836176931576]], [[-8.0425007107713924, -9.399912872936131], [16.832862502409164, -17.687482430875782]], [[8.0567199088045882, -2.268628140351761], [-8.6579529938337938, 22.112031412576204]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_array_rank3(self):
      arg0=numpy.array(-2.19724983286)
      arg1=numpy.array([[[2.2461416182542395, 0.90079023269443947], [4.1194060996801483, 3.620528707003059]], [[-3.7260948885667555, 0.11998178185056307], [-2.5232279480582323, -3.2914706081684795]], [[0.93943087463987851, -3.2854949584272699], [-0.48322524329693639, -1.485039884829511]], [[-2.2728488145733525, -4.6779622013835578], [-2.5643221405091641, 3.3921928269870545]], [[3.5516836922655628, 1.9346591510995328], [-1.7238006071516976, -4.4542945150621529]], [[1.3397124180131526, 3.3952362600279091], [2.0604644161653365, -1.2540854587570136]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[-4.9353342952891452, -1.9792611882298288], [-9.051364364004904, -7.9552060963275082]], [[8.1871613711240148, -0.26362995011740148], [5.5441621871387747, 7.2321832436619804]], [[-2.0641643322860501, 7.2190532482668779], [1.0617665850679536, 3.2630036387321608]], [[4.9940166779374771, 10.278651665115685], [5.6344563946331032, -7.4534951221263883]], [[-7.8039363992022972, -4.2509294963946278], [3.7876205959481317, 9.7871978787297831]], [[-2.9436828865599418, -7.4601823048667271], [-4.5273550940333802, 2.7555390646460758]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[-4.0874693199367478, -3.1485982774701982, -4.9175834409726003, -3.9159470078930134], [-4.075592951756434, 4.5038863839290784, 0.066068050856054938, -4.1842357530053373], [-1.3254700503415018, 2.7005552482693318, 4.1499893221134876, -3.0323384500709727]], [[0.15710847207857093, 3.8579814809125672, 1.07361656611209, 3.8915234997845296], [4.5107932113848346, -0.46067717049518997, 4.4068414637891369, 2.9544944975338829], [-3.2790315470744491, 3.4664216634186609, 0.87721995680479203, -4.0070375843794306]]], [[[0.93991510749546947, -0.91097602940960609, -2.8222872021101475, 3.3441229002003645], [1.4285175761855999, -3.8850978347917895, 2.127912789713946, 0.79779457245133933], [-0.80234381935224608, 4.8517089678113479, -2.0281390470264848, 4.1809612431426775]], [[-4.4666923419365538, -3.5342996700681417, 2.9560040325888339, 0.12230895584747614], [3.2721839263179611, 0.22679655675346844, 1.4270855381293455, -1.9138792146503016], [0.29763946506986461, -1.1373573416296234, 4.0651364718931529, 0.57417101039905205]]], [[[-1.2951324841825018, -4.2646639673006046, -3.3154751275248104, -4.315069984555552], [-1.913938801514854, -4.0142788440898691, 3.8117527752265747, 2.6515712370833739], [2.9029536033076297, -2.4688125756141357, -1.9382711066008151, 4.3010596221554671]], [[1.2596183724416612, -3.7645056131981205, -1.6217666908393644, 4.0510681831550492], [-2.0177100442218809, 1.5589182867561515, -2.3417037168715247, 0.0026989774579728731], [-2.6134265410489976, -3.5799441642658012, -3.74311528167767, 2.2908302605240918]]]])
      arg1=numpy.array(-0.139189817178)
      res=mult(arg0,arg1)
      ref=numpy.array([[[[0.56893410736153249, 0.4382528186071587, 0.68447754010515593, 0.54505994810626535], [0.56728103784577932, -0.62689512236830758, -0.0091959999199425473, 0.58240300948928947], [0.18449193398157601, -0.3758897912849386, -0.57763625503446347, 0.4220706344863473]], [[-0.0218678995056871, -0.53699173700324687, -0.14943649355611258, -0.54166044447780692], [-0.62785648241915237, 0.064121571139174507, -0.61338745767600278, -0.41123554896431874], [0.45640780155726657, -0.48249059759212898, -0.1221000854123058, 0.55773882879402281]]], [[[-0.13082661197487075, 0.12679858698680768, 0.39283363968472829, -0.46546785509871308], [-0.19883510026443232, 0.54076605734222216, -0.29618379217041474, -0.11104488068487864], [0.11167808952931216, -0.67530848422916256, 0.28229630316661025, -0.58194723106015922]], [[0.62171809046326765, 0.49193852492805823, -0.41144566087264073, -0.017024161203607942], [-0.45545468247606863, -0.031567771271051509, -0.19863577514919073, 0.26639249798741244], [-0.041428382727948618, 0.1583085604471641, -0.56582560232528667, -0.079918757966190412]]], [[[0.18026925369428784, 0.59359779793297818, 0.46148037685745386, 0.60061380225935124], [0.26640079187219567, 0.55874673840925515, -0.53055717191045104, -0.36907171572333375], [-0.40406158131979042, 0.34363357104578579, 0.26978760096862309, -0.59866370247828982]], [[-0.17532605097385112, 0.52398084806554435, 0.2257334092028459, -0.56386743978782672], [0.28084469217289149, -0.2169855513285921, 0.32594131223573325, -0.00037567017894202975], [0.36376236245600924, 0.49829177373059991, 0.52100353173184211, -0.31886024514753536]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_array_rank4(self):
      arg0=numpy.array(-2.79747122963)
      arg1=numpy.array([[[[3.2597865557951398, 2.7458761604618012, 2.1176351847596555, 0.059308008441297133], [-2.2653439612424631, -4.5321343364930264, 1.7979862992958759, 2.5205972245676476], [-4.2943710270147397, 2.2882422797728905, -3.0102320787795245, 2.9852296045988194]], [[-4.01434145017822, 0.12819856389679085, -2.3174108701505824, 1.268907806702976], [3.0212221149926748, 2.8836273564445767, -2.6330851308078107, -1.8006672151174827], [-1.9472309965751022, 2.3950539952009011, -4.1135370502944753, 2.9224596731355827]]], [[[-0.74071850933682715, 4.5072546650011613, -0.50660805089055749, -3.349452025519124], [-2.176815740838749, -4.442650999907519, -0.63195208574124617, -0.86405562876127906], [0.73116254087527288, -2.4741214053899885, 0.18713691229966756, 0.61665256899956944]], [[0.14987561740479993, 4.0194043656574738, -1.1974998712733864, -0.21442423599258653], [-3.9515955401709513, -1.4087497781767455, 1.6157348919029086, 0.9582709891479908], [3.9905645680023625, 1.2880613783752706, -1.2162984131385679, -4.0441863355742536]]], [[[2.4155840505638784, -0.53139100513643989, 2.2713770263059132, 4.4760656663587355], [4.3648248970716921, -0.72553754583459362, -0.40789572530670348, -0.89369451273551803], [-2.7271420435498896, 2.5057714499622206, 4.9431536449694189, -3.5143839396275247]], [[-3.8367225850792295, -0.40624874083354534, 2.2651646702688675, 2.2719340780928965], [0.00051591714823917556, 4.4251353566298519, -3.3928137765371167, -3.3419125049596099], [3.3287864527300144, 4.7376664753346471, -0.26512476393565443, -1.0668028484294667]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[-9.1191591045783529, -7.68150955902449, -5.9240235042217506, -0.16591244730130528], [6.3372345567965604, 12.678515415166919, -5.0298149435528678, -7.0512982172184655], [12.013379397439303, -6.4012919440923826, 8.421037634901289, -8.351093932711148]], [[11.230004712793093, -0.35863179418142233, 6.4828902364828993, -3.5497330823071205], [-8.4517819450201923, -8.0668645666337149, 7.3659798986068727, 5.0373147284328779], [5.4473226903666525, -6.7000946449898917, 11.507501550224406, -8.1754968553567657]]], [[[2.0721387191257352, -12.60891524996573, 1.4172214470663194, 9.3699956764226453], [6.0895794072066431, 12.428188355537477, 1.7678677783671217, 2.4171707622613354], [-2.0454061722832657, 6.9212834501953813, -0.5235101281605018, -1.7250678204550067]], [[-0.4192727277132729, -11.244168073184365, 3.3499714373754181, 0.59984563112510036], [11.054474834770675, 3.9409369742000204, -4.5199718748110858, -2.6807355223325797], [-11.16348956897578, -3.6033146480050604, 3.4025598174022975, 11.313494921040164]]], [[[-6.7575268842105745, 1.4865510485544635, -6.3541118827380609, -12.521664923582508], [-12.210472071939865, 2.0296704104901426, 1.141076556235413, 2.5000846874576723], [7.6291014059508537, -7.0098235393027739, -13.828330105452899, 9.831387960989046]], [[10.733121047838766, 1.1364691645561025, -6.3367329954561944, -6.3556702190855621], [-0.0014432633790729228, -12.379188847399705, 9.4912989273619495, 9.3489040845721849], [-9.3121843311012444, -13.253485660341099, 0.74167889937299025, 2.9843502761709861]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_array_rank1(self):
      arg0=numpy.array([1.6553495507528835, 1.3754789922231225])
      arg1=numpy.array([0.66593530598299555, 3.9576899796058633])
      res=mult(arg0,arg1)
      ref=numpy.array([1.1023557095894356, 5.4437194246798235])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[1.525314714952489, 3.7592351201058136], [4.2234183506463356, -4.8855620621877369]])
      arg1=numpy.array([4.8544240889013501, -0.45052977337039035])
      res=mult(arg0,arg1)
      ref=numpy.array([[7.4045244954210592, 18.24892152288562], [-1.9027757123650415, 2.2010911686644179]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_array_rank2(self):
      arg0=numpy.array([-1.5870284640140486, 2.1488438967517451])
      arg1=numpy.array([[3.3989515027669874, 4.2186502247273552], [-2.3036750901516969, 1.7091834865523667]])
      res=mult(arg0,arg1)
      ref=numpy.array([[-5.3942327826945347, -6.6951179863615753], [-4.9502381575715004, 3.6727685035069215]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_array_rank1(self):
      arg0=numpy.array([[[-2.9395503378092034, 0.92496826867662385, 4.0137291328368967, -1.3279413125176465, 3.6911629590399251], [-0.053340356268214428, 2.3868113746977491, -4.404456309937979, 2.9917540090841879, -0.79309928453277578], [-2.3973618176531675, -4.7823275155792206, -2.1630862250742933, 1.5129402946549266, -1.8940505479383161], [-3.3691560071320179, -4.8814376170549991, -2.7771335728716409, -2.8966158796417094, -4.8871951230231447]], [[4.5962768732413934, 2.5126682816448076, 0.73750120106691153, -3.4941654736611616, 3.6224825920879802], [-3.7799883296474954, -0.9974896655302059, -0.80866406281953385, 4.0205188229976372, -4.7619642793981187], [4.1325622384600571, -4.2082583388162318, -3.5850646563587993, -4.8608600738383334, -4.1030535147080869], [2.9402977648687809, 1.615958446741935, 0.64373639403404503, 3.1493613289794524, -2.4463828213415608]]])
      arg1=numpy.array([-1.5265049520521403, 1.998757980712349])
      res=mult(arg0,arg1)
      ref=numpy.array([[[4.4872381474722909, -1.411968642625961, -6.1269773974714656, 2.0271089895928061, -5.6345785358058773], [0.081424317987654746, -3.6434793830904906, 6.7234243682176213, -4.5669273101888566, 1.2106699853082916], [3.6595846865082802, 7.3002466348668893, 3.3019618342916792, -2.3095108519499696, 2.8912775408649094], [5.1430333291232415, 7.4515386955680558, 4.2393081514988129, 4.4216984844659359, 7.4603275569398999]], [[9.1868450819548375, 5.0222157808203436, 1.4740864114174321, -6.9839911264097916, 7.2404659909274072], [-7.5552818408824729, -1.9937404296565908, -1.6163237492758156, 8.0360440840707472, -9.5180141073141193], [8.2599917549125284, -8.4112899396082366, -7.1656765932669249, -9.7156828657103862, -8.2010109578126418], [5.8769436232021581, 3.2299098419249739, 1.2866732550505369, 6.2948110904445302, -4.8897271880340369]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_array_rank3(self):
      arg0=numpy.array([2.0627775654661953, 1.4865562513728623])
      arg1=numpy.array([[[-0.67547318911140586, -1.8052513722959751, 3.0647416900440536, -0.56376379913473418, -4.458384352098836], [1.8788624952150359, -1.3996431266619149, -2.5960637591553262, 0.57407141076272428, 2.0177304887761007], [0.3857710090510782, -3.6493095551375374, -4.5492079218034238, -1.9129063411405633, -1.9556295955234635], [-3.0623139633153906, 1.9364277446933285, -3.4512031399115619, -3.7051860464329422, -0.23085595440761075]], [[2.6976651282477437, -0.83262875499855049, 0.20054960104600461, 0.30840472209809455, 1.7765995970936874], [-1.9282336100269548, 2.9910014616545766, -2.1387208887633635, -3.7129866826430291, -0.35899047293334618], [-4.5561772704419372, -3.6818689286735284, -1.6792946027048741, -4.7675057593946981, -3.550574189809975], [3.0025019892071327, 2.1959566950888076, 1.066637631520102, -3.1303930730034688, 4.1175887091790759]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[-1.3933509405729128, -3.7238320307991994, 6.3218804021718258, -1.1629193170771202, -9.1966552197350175], [3.8756754037254129, -2.8871524413371583, -5.3551020809054428, 1.1841816270968766, 4.1621291854044813], [0.79575978287782068, -7.5277138797791334, -9.3840040417371959, -3.9459002853427787, -4.0340288560075299], [-6.3168725419408576, 3.9944197088996991, -7.119064410876061, -7.6429746524602615, -0.47620448360630629]], [[4.0102309605072577, -1.2377494808158986, 0.29812826314527169, 0.45846096758783278, 2.6410152372461293], [-2.8664277270928316, 4.4462919206879796, -3.1793289071329021, -5.5195635643471812, -0.53365953172236613], [-6.7730138037384062, -5.4733052726551366, -2.4963658895476377, -7.0871654900843133, -5.2781282578251538], [4.4633881018153172, 3.2644131528283573, 1.5856168390857512, -4.6535053919276113, 6.1210272362124698]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_array_rank1(self):
      arg0=numpy.array([[[[-0.56543895168140512, -3.4027089814372702], [3.9989800841151304, -0.74442886528798091]], [[-1.5539061590649839, -4.8337671801833277], [0.71995919243266293, 3.1839951125890806]], [[1.3616634410495267, -1.9985846512234273], [2.1942421411967619, -0.087511501893228427]], [[4.5483787404512928, -0.14341267626065335], [4.2384743175301978, 3.5898218086644498]], [[-2.1660132486956885, -4.5227969768942131], [4.0644396066482358, -2.1027137836161702]], [[1.7448776071193315, 4.4799996256423], [-2.8783898929881504, -2.2182112383836761]]], [[[-1.2060021604490223, 1.5655687460348142], [2.3496958011646552, 1.1819097161491676]], [[2.8542985663211056, -2.5020278116521033], [-0.92526386860964482, 3.37196146872291]], [[-3.6373070336053304, 2.3847900063293856], [-0.53350964491312158, -3.3119356072060668]], [[2.9900510702868317, -0.92140228490800169], [1.3621291457432854, 1.0050179475683843]], [[-1.8151440291339718, 0.70361213347272233], [-2.4176442540596552, 0.25114060340651445]], [[4.2288561966136182, -1.9848309934112827], [-4.6463706912312386, 2.888691059296975]]]])
      arg1=numpy.array([-1.8081004081114416, 0.94358354225593466])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[1.0223703992972544, 6.1524394980211961], [-7.2305575221180938, 1.3460021351371356]], [[2.80961836037228, 8.7399364112051678], [-1.3017585096610818, -5.7569828624971517]], [[-2.4620242234720791, 3.6136417235223419], [-3.9674101109931885, 0.15822958228759151]], [[-8.2239254568553868, 0.25930451847524139], [-7.6635871432962146, -6.490758277293545]], [[3.9163694389414636, 8.1776710597276203], [-7.3489149115249823, 3.801917650297951]], [[-3.154913913536979, -8.1002891514629489], [5.2044179402157233, 4.0107486453989107]]], [[[-1.1379637905247986, 1.4772449030287118], [2.2171342872868416, 1.1152305565907379]], [[2.6932691518653047, -2.3608722653415564], [-0.87306375866411834, 3.1817273470080876]], [[-3.4321030550417437, 2.2502486017088343], [-0.50341092057482917, -3.1250879319710605]], [[2.8213629804273972, -0.8694200318362042], [1.2852826443504994, 0.94831839499736525]], [[-1.7127400327149425, 0.6639168292764468], [-2.2812493291603162, 0.23697214016661175]], [[3.9902791096916372, -1.8728538595423838], [-4.3842389154661277, 2.7257213422144879]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_array_rank4(self):
      arg0=numpy.array([-2.6440750676341294, 0.18656085686736112])
      arg1=numpy.array([[[[3.3096018471136261, 0.54667858282384252], [4.5161052515735349, 4.2733266157811318]], [[0.37093496374357571, 0.48649449766658215], [1.5357808633860612, -0.40144364505020924]], [[1.7915238606920152, -3.6152839983688176], [-4.6472149988330607, 1.9431264665052979]], [[4.0425329877221774, 2.3109695364224443], [-2.0255978278053046, -3.6261623134560428]], [[3.0092097983552346, -0.43036994764415049], [3.2543042255124952, 1.4818607578119627]], [[1.4949353232539666, 0.78647508228533347], [-0.88184330540480715, -2.0439363364205887]]], [[[0.46710767253223118, 1.480026851044264], [-4.5037444420431676, 0.12607107669844453]], [[-3.9225836003008951, 2.1860045539601805], [0.42818806700072454, 1.7725432501129372]], [[1.6241892025015572, -2.5392894810543232], [-0.48917352398733094, 3.4300783513657187]], [[1.8109483307348597, -4.1110004741315498], [4.5984414015396844, 0.34149865351279018]], [[-2.7780805852636337, -2.9497555640806814], [-0.45465871026878091, -1.9897795825138354]], [[2.2182070133715275, -0.7517410396861095], [-3.5820249872420518, 3.9063393159506781]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[-8.750835727749001, -1.4454592108540814], [-11.940921298497141, -11.298996360644221]], [[-0.9807798893481583, -1.2863279718214], [-4.0607198902287012, 1.0614471329374233]], [[-4.7369235731273962, 9.559082282503617], [12.287585312349865, -5.1377722433466619]], [[-10.688760682924714, -6.1103769333165872], [5.3558327135538564, 9.5878453642036181]], [[-7.9565766011114016, 1.137930448424904], [-8.6046246651739846, -3.9181510834360274]], [[-3.9527212159413807, -2.0794991563861505], [2.3316598973809195, 5.404321106961123]]], [[[0.087144007636931767, 0.27611507751752012], [-0.84022242221918853, 0.023519928095052618]], [[-0.73180055760599338, 0.40782288270276484], [0.079883132680034205, 0.33068718757552679]], [[0.30301012933340643, -0.47373202141977128], [-0.091260631791903093, 0.63991835635297389]], [[0.33785207232441272, -0.76695177103610979], [0.85788916812559257, 0.0637102814183962]], [[-0.51828109443336356, -0.55030892558415812], [-0.084821518569953042, -0.37121498389096119]], [[0.41383060112378212, -0.14024545250620152], [-0.66826565094017543, 0.72877000999841979]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[1.6661982159482092, 0.75531209295602153, -0.72495338778651863, 2.6608552110323931, 0.53310666179101851], [4.602265547678714, 1.5910793869451165, -1.3284764918398819, -2.638877176623903, -3.3559077271892779], [-4.5653893417783724, 0.4876263200529678, 4.3178560909806194, -0.011084695418974277, 1.5567820333443949], [-1.2655598199797478, -2.0726359631493638, 0.53053016820059362, -4.8530783530508783, -4.976875470468193]])
      arg1=numpy.array([[2.0382267001265797, 0.97874340646391023, -0.35931124501953526, 2.8871136900896044, 4.8419498274900015], [-1.1880137803941784, -4.6088975185758088, -4.7867928425521411, -2.4801560635675446, 1.7647959756310456], [0.97975651898355842, -0.0073786579594035118, -2.2761131788053657, 0.77921445182994731, 4.0881109403195186], [2.4622222823760946, 1.3834717599869713, 2.4719165659747491, -3.600458688325717, -4.2920505228559138]])
      res=mult(arg0,arg1)
      ref=numpy.array([[3.3960896914489127, 0.73925673080316212, 0.26048390434670399, 7.682191507117885, 2.5812757090927927], [-5.4675548916756727, -7.3331218383484664, 6.3591417626379245, 6.5448272306137749, -5.9224924515327668], [-4.4729699693054172, -0.0035980278276734751, -9.8279291528660071, -0.0086373548645979695, 6.3642976622080862], [-3.1160895884340141, -2.8674333237505416, 1.3114263115244174, 17.473308121367495, 21.36100096521178]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_array_rank2(self):
      arg0=numpy.array([[[4.5032168949417866, -1.1259576249136738], [0.63332686063389154, -4.24165039551918], [-1.8094309507924899, 3.9303443823445079], [4.2287548086967615, -1.5237755394301242], [2.8120467280185091, -1.1429862894892384]], [[-0.14506107514637456, 3.5709073621818437], [3.0579205960930818, -1.6896472966494072], [-3.2057923803736021, -3.6607331601573048], [-1.9406437507970242, 0.99146030899027604], [-3.9951467877245603, -0.257922661733895]], [[-2.2845290474132263, 0.48995017325840706], [4.3266889498043177, -3.3510743998082582], [1.3379178278546675, -3.5787325187070698], [1.8390251051701618, 2.5090203518357992], [-3.1993054625292441, -1.4871244331477107]], [[-3.0997954555365781, 0.54142477800775435], [4.4048993136134804, -2.6448693677809643], [-3.5092928612235195, -3.2739847758314813], [-0.35962282551231706, -0.90472297015907532], [-3.0733362103545225, -4.3008948151276343]]])
      arg1=numpy.array([[-3.5550715228355112, -2.9861520590703572, 0.24463203205963069, -3.0303716345820564, -2.3270494487191176], [0.17589020270626854, -3.2425415461221805, 2.7718358192435, -0.53253871779120132, 3.2892871052741839], [4.6956997119142301, 1.3736695225881865, 3.6233752265187231, 2.3969105591347049, 2.0072432884263556], [-1.8610495332071642, 3.7650254413566593, 0.1212505980449583, 1.744478730914877, 4.8157187939921666]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[-16.009258144359301, 4.0028598882501099], [-1.8912103089464605, 12.666213062436194], [-0.44264477036395644, 0.96148813294709101], [-12.814698621877136, 4.6176061721590207], [-6.5437717882078701, 2.6597856148494419]], [[-0.025514821912285074, 0.62808761977947114], [-9.915434577574521, 5.4787515576787316], [-8.8859301489774332, -10.14695129801647], [1.0334679347389548, -0.52799100169054991], [-13.141184812539974, -0.84838168539929604]], [[-10.727462389797976, 2.3006588874218292], [5.9434407440652794, -4.6032687709421038], [4.8477783125663443, -12.96709075062015], [4.4079786930961724, 6.0138973743990993], [-6.4217844172876024, -2.9850205374905907]], [[5.7688728855640372, -1.0076183303781236], [16.58455798236924, -9.9580004587602335], [-0.42550385813825459, -0.39697261205965584], [-0.62735437025774909, -1.578269978812642], [-14.800322948460938, -20.711899992193615]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_array_rank3(self):
      arg0=numpy.array([[3.0965277512706599, 3.3023473285529406, 4.0673667637106981, -4.2614884730547731, -1.8079099818806732], [0.90060678652718806, 2.0827561520061346, -1.1561512102003046, -3.265287250293305, 1.8473809864427668], [-1.8818531566332575, 1.7583270314836863, -4.4343794639060654, 2.7598384807724639, -3.8440142264033583], [-4.8730627119825884, -4.6855253722262216, 1.440767855978943, 3.5753286043463, -0.84265174012551469]])
      arg1=numpy.array([[[0.6342665112042658, 4.0838469802769541], [4.5315589615179448, 3.036640915308304], [-0.57744456344610562, 2.0644660498604992], [-0.027150141405697603, -0.25135275654094968], [0.27246146063922083, 3.3476726294990939]], [[3.0023540839453631, -4.7773400222093798], [1.1537714578484337, -4.3129856783843383], [1.094203317674828, 4.3018147460623197], [-4.1602042175407572, -1.6622033193760064], [-4.6478090468583044, -2.7306689149415164]], [[-3.8637127495925605, 0.18523679105642987], [1.7839110770968949, 2.21724135317295], [-2.5240538659841505, -1.5899655940192914], [-0.74179657643701269, -0.45169378715799979], [-3.3670366178972779, 4.2479706906482875]], [[-1.1337461907264035, -1.4861740638934027], [-1.4268096712895515, 1.2744189843812297], [0.49815996247377647, -3.8392027507498403], [2.8264307633845887, -2.2097627147056609], [3.8052440036212705, -0.16731690367232854]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[1.964023853645632, 12.645745506370472], [14.964781630748922, 10.028043014442934], [-2.3486788252461235, 8.3969405960117065], [0.11570001464218745, 1.0711368746697998], [-0.49258579436743549, -6.0522907629401326]], [[2.7039404635588129, -4.3025048455497146], [2.4030246018429118, -8.9828974551693328], [-1.2650644899349408, -4.9735483247174672], [13.58426179015227, 5.4275713061536841], [-8.58627406178271, -5.0445858337332581]], [[7.2709400341449228, -0.34858843997415773], [3.1366990686226486, 3.8986354066074651], [11.19261262911283, 7.0505107784363545], [-2.0472387365561397, -1.2466018953244948], [12.942936660018185, -16.329259768196515]], [[5.524816286881137, 7.2422194142745697], [6.6853529161649483, -5.9713224861650245], [0.71773286106789358, -5.5313999158663076], [10.105418756533268, -7.9006278427050818], [-3.2064954812536439, 0.14098988003190077]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[3.1034763596018955, -1.9357849233578484, -2.8902826716308496, -2.9327023312674316, 2.3703860800549066], [1.0950933891672978, 2.5634821031403572, -0.076171567318665367, -3.2692354898735756, 2.660986975252106], [2.9561331926330539, 1.0265891947500076, -2.8561268866462184, 3.676033625891522, 3.2122901658399261], [-4.1246764466451573, -4.2640070754870916, -3.2280719321500873, 2.57365735585691, -1.7082822436460932]], [[3.3755243330283928, 2.0166316821948369, 3.8217427834912847, -0.22395885079105859, 3.8018179780814627], [2.1722075214803631, -2.9911915999329186, -0.29412977885027392, -2.4914586702227415, 3.2821076505586451], [3.750495576117391, 4.5404352037688227, 1.963704843561759, -2.769102502878622, 2.232280705457498], [-1.7034438004888521, -4.1538418429283706, -4.4784552879848043, 0.85303989232789235, 0.35996410395909706]], [[-4.9633076756402259, -3.4959501276765326, 0.88628679039426839, 1.4411120407239704, 0.12920247546933883], [-3.4328109121227399, -0.19630611913672702, 4.5044915013720015, 1.8107735055585774, 4.8207637603424853], [-1.1857535369032943, 2.6113092090991339, -1.9573907216956012, 4.5543023781456959, 3.8364724596156741], [-3.5842751664677186, 4.3489695553886314, 3.7224863190001507, 2.6116202659376695, -2.8460749977347035]], [[-2.2269686792819456, -2.4199843864424397, 2.8473560915521254, 3.1294315450897265, -0.69048842287042866], [-1.3595783894202484, 4.4378975547014026, -0.63086615524936818, -4.6232664955216451, 4.0209395046867478], [0.20572848016708445, -3.2153352123847534, -0.57257695217535165, 3.2503353435203515, -1.2135504453767787], [1.51465165575011, -4.0261245709949725, -0.28529491542941798, -1.7420939110949418, -3.0046383130283836]], [[1.2646183164302025, -4.9454880719127035, 2.1135739132434681, -4.6168833067438424, -2.1880731423271649], [3.0489161093714738, 0.34788452068129239, 4.4597350821866026, -1.5932572649517516, -0.45090950760560045], [2.0403224569991218, -2.735361646206754, 3.0919059704984218, 2.0531903277967212, 3.2224158539739776], [-2.872019280087259, -1.9076651537492917, 4.0875703477510807, 0.40127965795609555, -2.9829847475953186]]], [[[4.3052233616127218, -4.348099338440754, -3.5336507654173599, 4.4527763977832731, 0.8432391536756807], [-2.8229257903323091, 2.5390046924217913, -2.4289439771550136, -2.6578299524100455, 4.2399602790960458], [-1.8316249548008168, -2.2449798068052687, -0.60073092233392344, 1.6929960364309622, -0.99046933706049778], [0.41432349396154589, 4.2793105155420221, -4.1710598874480862, 3.228997417102681, -0.2239112946139743]], [[-0.3712940652337835, 3.4397820137585207, -2.5127575061557774, -4.4355920082650711, 0.077926893439363099], [1.6633009545095723, 3.5270419115585216, 3.4785817046959835, 1.5341835330667486, 3.3146447661331795], [-3.9590367258819894, -2.4703870655219049, -4.1378069131096087, -3.0864781433071222, -4.7442576783428425], [2.0149018396508644, -0.52805212515639965, -2.6214433069955909, -0.9893307279228587, -1.0445882472515811]], [[3.019710383314731, 3.3894648939090892, 0.4438809612892749, 1.1008035626845878, 3.1703914378179547], [-1.0038896087844895, -0.88053625876752761, -4.429489369949076, 2.4644125103636956, -1.6419334563410093], [2.0203908785430222, 4.1532581458687705, 4.9336152652908254, -2.9772248424120953, 4.1739589095477072], [1.2310839968995104, 0.37324275712368582, -1.8007297071631276, 2.6964056126085474, -0.074891884867148484]], [[-4.5861288066038295, -4.1391934016082796, -4.6338734161164901, 1.5985421915596429, -3.3142127428942159], [3.6294611803679029, 1.49204982564315, 4.0006790184683307, -0.33076481153818804, 0.85176056185644988], [-3.3072823065329207, -1.5999855027486918, -0.71002707321456437, -2.1815020337565247, -4.6700108012352093], [0.086731520527942862, -3.8541692553579465, 0.93703988371767011, -2.0945180601051181, 4.6012675766248616]], [[-1.5123676126255123, 4.9684681897387257, -0.8999251706918221, -0.13939750281011687, 0.92779010802503947], [0.65100907339497827, -2.5891058209849005, 0.8363566772521791, -1.9454099589842322, 2.2217461901487221], [-1.6558684708702645, -3.4442294610447277, 2.8731461107221703, 2.1193961521619489, 2.7398533055023826], [-2.6456718934654679, 2.5775330306421136, 1.0764636774500067, 4.2000803896878089, -2.2918116657686962]]], [[[1.8595634487671466, 4.5801772769914386, -3.5511578801481836, 2.8281245789530853, -3.7265725990183474], [-1.6568847946268441, -2.8138463016097823, 2.5938907268686604, -3.1025651717345362, 3.2819191992289358], [-2.1716160174994039, 1.0304005861925569, -3.545099014543255, -4.1130531098861658, -3.6550309109481063], [-1.2702367084474209, -1.7633035653754972, 0.78357115504306574, 3.3278283794973635, 0.56533674595986572]], [[2.3722697380014228, -0.59081874252364486, 2.9054808386644657, -0.3366077709704749, -3.5098310486852258], [-3.059892393616229, 3.4098091171918394, 4.8220344196211133, 3.8210199274860344, 1.7286842293970919], [-2.7094130623684287, 4.4132143905052068, -0.66764197507871437, -2.5628978116716175, -2.95454874968729], [4.8970007464853733, 4.4694191447218454, -1.481354770876453, -2.2027256889658018, 2.3014190277091364]], [[-3.7278821936583109, 1.4802465038127597, -2.0731696353338815, -3.4229436793411026, -4.0985363265399064], [2.7179820148701985, -0.13160171288630007, -2.6316169738661213, 2.7247641987132143, -1.6441936248393052], [0.80222943722708528, 0.095549251176231031, -1.2487685580368888, -4.5730464372461359, -3.0432641444247821], [0.25700841651885575, 0.198704135911135, 4.5358679730508342, 2.8740129499867653, -0.47754505045317952]], [[1.9271431472804874, -0.21188797354994016, 4.9172648463399771, 2.2616691314698976, 1.5421028072831096], [4.3724733047990334, -4.4615645601149376, 2.5259957406661719, 3.328533445885455, 4.3593459836102291], [-3.0884958624194825, -4.160317581035252, -2.0600132512572502, -2.8091222098714921, -0.55104462001921029], [3.6506840342207365, -1.3051691968005343, 3.4665780553130894, -2.641621055073057, 1.6492832472442007]], [[3.6055218748802105, 3.9558313178349742, 0.82331665851084601, -1.7457484460922377, -0.81713637892603241], [4.4827380953429223, -2.1599428824566322, 2.59689862113453, -3.4771094921311709, 3.3002267669568361], [-1.0264104646551142, 3.9859486332545409, 0.74977148528406268, 4.3518426504564864, 2.5812705850165134], [4.4814331635789166, 4.177111308887568, -2.4534881142505363, 2.8733177646196175, -0.15708553718963625]]], [[[3.1980805238734646, 3.4232856082223257, -4.8850754428283611, -0.96374996409658298, 0.30452490085859374], [1.772665291394131, 2.5401520429833138, 2.4198458842737711, 4.2615012567324779, 4.0246213303532468], [1.4201029398871858, 3.1358694201413453, 2.3640627821013185, -3.3472287654950925, -3.9269148423594249], [-1.5994401714016826, -1.7054239080833278, -1.0263566979527452, 1.8062848676446697, -3.2457329521577738]], [[-2.3207878419759123, -4.900980761204341, 2.3042050451028953, 3.5822455806847326, -0.33252820558520302], [-4.8264417408996554, -3.3965856880047598, -0.70901793663587576, -4.9938033288854324, 4.8857750365076917], [3.4451924611168234, 4.7073919505863895, -3.9029337921408569, -3.8216085645343201, 1.8393213331712754], [-3.6312118038170471, -3.0665947400373517, 1.3699129068434033, -3.1682324060074807, -0.8810506448687434]], [[2.1130619473511301, -0.59261659245219711, -0.26688078731187748, -3.904255832175334, 1.0291142480987689], [-3.640796869787799, 4.3262041008378915, -2.5153054695775512, 3.0865319465048646, 1.5026826397159896], [0.52654149672079065, 1.3582801614488451, -1.8984745362096325, -0.99123630981767796, -4.7752558113354793], [-3.9944574774793526, 2.3683524551131532, 0.94647146067406762, 0.26396527168206951, -2.3712623655524445]], [[4.8401788083576474, 2.7908438789654948, -3.8481328341485819, -0.77274135430949542, -3.0459962232503157], [-3.4128614007434557, -4.0784541339367406, -2.6443731204611876, 1.9009776188199261, 3.5186146499612594], [2.3877852697866251, 3.5648878079952553, -4.5326692309422025, 0.72055102817031624, -1.489151136807072], [-4.2241063965923962, -2.6574811022884393, 1.9435215012473019, -0.79231681765592565, 2.5225967801956015]], [[0.17255112248172111, 1.0742737046893378, 4.6725333774021145, -4.4000754672852356, -2.9710413741955199], [-4.3839157349408424, 3.0554917009262184, 2.1772743732347113, -4.5147089000781619, -2.1774671541542947], [-4.7758232529549494, 4.1458350453790018, -2.7496149382519564, 3.9114012622273329, -1.6193093397735261], [-0.46436614202749738, 0.086852673914292744, 3.1584520092465596, 1.3556720699468698, 1.315280252861978]]]])
      arg1=numpy.array([[3.1610327956653403, -1.6378133683154972, -0.88321408174971783, 1.3141175269473067, -1.9032915101487626], [3.8172187289756092, -3.3460534427719244, 3.1993679423304666, 1.0591937213619396, -0.08053692161595194], [2.5666976377289243, -1.5205143046462108, 2.5933466745629632, 1.0617178308928379, 0.28820968631942101], [2.3288496711146456, -3.5423723787797479, -1.6151598208373463, -3.1565400885487049, -0.36862912072745591]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[9.8101905532736726, -6.1190796280886763, -9.1362783137683543, -9.2703682490605512, 7.4928681374421684], [3.4616261174741361, 8.1032509991278303, -0.24078082239153145, -10.334160600243417, 8.4114670976102222], [9.3444339702679695, 3.2450821122804472, -9.0283107572702406, 11.620062849411676, 10.154154563413261], [-13.038237519353723, -13.478666206563753, -10.204041244293208, 8.1354153066690351, -5.3999361964180697]], [[-5.5284788777081539, -3.3028663280672732, -6.2593014210653051, 0.36680279977817154, -6.2266683084040135], [-3.5576705174360113, 4.8990135895631548, 0.48172968382065939, 4.0805443166963578, -5.375479786335517], [-6.1426117923731951, -7.4363854747028766, -3.2161820442113411, 4.53527309745051, -3.656059181231039], [2.7899230286147985, 6.8032177002163667, 7.3348739400647425, -1.3971201393610344, -0.58955402157791859]], [[4.3836632311819086, 3.0876723818586376, -0.78278097374497835, -1.2728104477464834, -0.11411344573144254], [3.0319069375708971, 0.17338032875519507, -3.9784303251336803, -1.5993006589686365, -4.2577664379232045], [1.0472742212775232, -2.3063450652790736, 1.7287950488877979, -4.0224239929245069, -3.3884265005775389], [3.1656822998901033, -3.8410711524200485, -3.2877523360616054, -2.3066197950590928, 2.5136935157150866]], [[-2.9264985734071001, -3.180143897162834, 3.7417605453688281, 4.1124408427842001, -0.90738293864823383], [-1.7866457907959392, 5.8319189594297063, -0.82903227177105532, -6.0755155335132454, 5.2839870779036771], [0.27035140157979703, -4.2253283576056448, -0.75243340837969941, 4.2713226433763891, -1.5947479101043349], [1.9904302880409779, -5.2908008644176991, -0.37491104871474779, -2.2893161421580461, -3.9484478692879867]], [[-2.4069373052402261, 9.412705460813422, -4.0227472851481902, 8.7872748010731012, 4.1645410353758177], [-5.8029761461225222, -0.66212565472487539, -8.4881759194383548, 3.0324330258655063, 0.85821223767109822], [-3.8833284103722923, 5.2061905984118582, -5.884798383827917, -3.9078197196150546, -6.1331967370374461], [5.4662899127736413, 3.6308428913376609, -7.7798379400104576, -0.76375216618323605, 5.6774895450014196]]], [[[16.433979248371415, -16.597646230142502, -13.488717883410143, 16.997221461558858, 3.2188282904163503], [-10.775725197364764, 9.6919362648694172, -9.271810441228622, -10.145518272771978, 16.184855787478078], [-6.9917130819247815, -8.5695789647091161, -2.2931213278078446, 6.4625361783457418, -3.7808381039033878], [1.5815634010046258, 16.335064247029276, -15.921847922045732, 12.325789416378221, -0.85471838742963813]], [[1.2423697852562847, -11.509694449521641, 8.4078209043235344, 14.841727909986975, -0.26074755007730177], [-5.5654938852025824, -11.80167073097126, -11.639520288961425, -5.1334600926619887, -11.090978531285865], [13.247148466697919, 8.2660471455688018, 13.845323067135876, 10.327520817453093, 15.874539738016205], [-6.7419692374112588, 1.7668906313426023, 8.7714894024040149, 3.3103534882063355, 3.495248100995243]], [[9.6611645954995957, 10.844145323227275, 1.420138517759737, 3.5218756292562365, 10.14324873079376], [-3.2118122319837692, -2.8171594783604319, -14.17156629110865, 7.8845623823357567, -5.2531492636572859], [6.4639738077874327, 13.287800968115418, 15.784450519563688, -9.5252377179231331, 13.354030327811566], [3.9386906739963532, 1.1941409118485569, -5.7611968978998398, 8.6267936764997302, -0.23960669558465905]], [[-4.857598837311901, -4.3842076624862587, -4.908169627936589, 1.6931658526321287, -3.5103933285312854], [3.844302494172577, 1.5803698072804011, 4.2374940975461026, -0.35034401162871398, 0.90217943922206967], [-3.5030526538511033, -1.6946945987815407, -0.75205621794586075, -2.3106332572932131, -4.9464461193607745], [0.091865481987371253, -4.0823118763413593, 0.99250676149947814, -2.2185003785425308, 4.8736337274673209]], [[0.12180143187252525, -0.40014513314833838, 0.072477202932229451, 0.011226645757277822, -0.074721359206068191], [-0.052430266715284869, 0.20851861256006582, -0.067357592158836757, 0.1566773293776054, -0.17893259876654749], [0.1333585492448047, 0.27738763813151163, -0.2313943431104086, -0.17068964177981702, -0.22065935090445221], [0.21307426990555553, -0.20758657565135094, -0.086695070813210567, -0.33826154512498396, 0.18457545648453774]]], [[[4.7729371111576873, 11.755930197233623, -9.1147485421787966, 7.258940676001993, -9.5649850867257307], [-4.2527222883576945, -7.2222926552740985, 6.6577332011807533, -7.963346697191068, 8.4236942558781127], [-5.5738817021700147, 2.6447267504949346, -9.0991972661433103, -10.556963700998427, -9.3813592049567021], [-3.2603135589285595, -4.5258670958482785, 2.0111902326415616, 8.5415292404231575, 1.4510484903765444]], [[-3.6070700711104822, 0.89834834946028852, -4.4178251770647892, 0.51181693081568258, 5.3367483164172969], [4.6526101551716099, -5.1846635388032594, -7.3319723125302918, -5.8099154580807424, -2.6284890990145899], [4.1197013185264924, -6.7103556102336759, 1.0151591734894343, 3.8969227839931651, 4.4924336376741012], [-7.4459596848941825, -6.7958157430091992, 2.2524211193735568, 3.3492759192841817, -3.4993405526167161]], [[-9.6676908900862646, 3.8387923481962729, -5.3764475795980324, -8.8768796080355621, -10.62892555300777], [7.0486696197855716, -0.34128886448047613, -6.8246951278991537, 7.0662581737011312, -4.2639640693146363], [2.080459043269379, 0.24779233279486004, -3.2384897872837524, -11.859494770654273, -7.8922389487607099], [0.66651192231386747, 0.51530871008704904, 11.763078124168029, 7.4533119264990697, -1.2384398685467557]], [[2.0460822421506357, -0.22496523966972148, 5.2207477665816846, 2.4012544444615083, 1.6372780475623792], [4.6423328728080682, -4.7369226471535901, 2.6818947186246356, 3.5339633102197685, 4.6283953618300568], [-3.2791111277695175, -4.4170833579620856, -2.1871528007353502, -2.9824951393776558, -0.585053898691964], [3.8759963340879553, -1.3857214085752108, 3.6805277335077253, -2.8046561766330158, 1.7510734317920089]], [[1.0391463285770361, 1.1401089032457599, 0.2372878358909648, -0.5031416120408605, -0.23550661945045934], [1.2919685403109025, -0.62251646062069199, 0.7484513370005198, -1.002136636025406, 0.95115732128758668], [-0.29582143805322164, 1.1487890052756162, 0.21609140458496609, 1.2542432051995416, 0.74394718561315765], [1.29159244633653, 1.2038839400557921, -0.70711903979657487, 0.82811801163703991, -0.045273573398742807]]], [[[7.4478487762208712, 7.9723175628400629, -11.376606338401061, -2.2444287869230788, 0.70919271521075611], [4.1282709808595692, 5.9156322498828855, 5.6354572917391001, 9.9243958001960806, 9.3727380615541467], [3.307206264505214, 7.3029684677546465, 5.5055468325910297, -7.7951926096687272, -9.1451943391239663], [-3.7248557171363608, -3.9716759074509116, -2.3902304584735643, 4.2065659199536505, -7.5588241181585989]], [[8.2210947484233294, 17.361098877421203, -8.1623523068174393, -12.689647799023415, 1.1779387306302167], [17.097053910752578, 12.031971323346667, 2.5116055547983356, 17.689910977302112, -17.307234538256463], [-12.204154613840457, -16.675335221847345, 13.82564486148587, 13.537560621514498, -6.5155610863262678], [12.863104395340493, 10.863020504019575, -4.8527416425359453, 11.223058964595804, 3.1210094686891212]], [[-3.4129327563018652, 0.95717050929032932, 0.43105512461958195, 6.3059971503994774, -1.6621839845803681], [5.8804688199116324, -6.9875110404151215, 4.0626203315940748, -4.9852423857255426, -2.4270726231390682], [-0.85044866950698039, -2.1938395422126384, 3.0663397917686139, 1.601005060572593, 7.7128013206891097], [6.4516872236679497, -3.82526772708025, -1.5287026748499886, -0.42634610091729286, 3.8299676975040282]], [[-15.278218444324814, -8.8094105848353532, 12.146785557050544, 2.4391890629573409, 9.6148091882575724], [10.772833828107204, 12.873803973078511, 8.3470697638163713, -6.0005120612389558, -11.106648198757485], [-7.5371399269275665, -11.252711277115543, 14.307552135600289, -2.2744482062645903, 4.7005652612394], [13.333561179138913, 8.3884456339340598, -6.1348035316434704, 2.5009797977622639, -7.9626778639313018]], [[-0.063607368560972405, -0.39600857118025723, -1.7224318704814314, 1.6219959506398061, 1.0952123694145868], [1.6160390027145013, -1.1263432191024705, -0.80260673778793423, 1.6642531721762324, 0.80267780244881337], [1.7605075264865213, -1.5282755274591338, 1.0135881370268967, -1.441856408107123, 0.59692457810647204], [0.17117888263119727, -0.032016424817854182, -1.1642973870284257, -0.49974020313928474, -0.48485060312269684]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_array_rank4(self):
      arg0=numpy.array([[-0.66550981062547265, -3.6917559457624547, 2.1633392956686563, 0.57615768112512367, -0.078776389824145809], [-2.2376103634304645, -4.3865220962773943, 4.2826991060271311, -2.0436755116531193, -0.55710048996902373], [-0.86923077853439867, 3.8565284562312492, 4.6956932159676814, 1.5619698871682921, -3.1324993573199365], [2.1959497523813756, -1.3145391506774695, -4.5531102233270149, 4.8839784659554866, -4.8942858649588414]])
      arg1=numpy.array([[[[-4.5026331167831728, 1.8250392588898308, -1.9092280781804858, 4.0679204297013136, -0.62038047653088757], [3.9302518869761158, -0.53329670402836804, 3.1166289521649144, 1.3468710118563472, 0.074899265035403673], [0.83178778579400259, 2.0596399859636509, 1.5665997433562442, -3.4109544293045593, 0.47728463067736016], [3.6119897822067113, -1.9050750510303671, -1.6245207221090521, -3.9677593626180041, -3.7482661246669533]], [[3.1256417679405519, -3.7051423620514088, -4.9139960909174469, -2.2518896418767098, -1.5770197927530982], [1.9784453846599783, -3.8666561516851483, -2.7294827010321732, 1.6036132701362344, -3.2569437999147266], [-1.0525092166723296, -1.4652759340790098, -1.4221054975189409, 2.0733240403288065, 2.7146413103514231], [-1.4059025811428794, -0.5427073073839388, -2.7970965181410747, 1.5727528051423576, 0.49713435954902074]], [[1.502201713640444, 4.3520646042405549, -4.0973635855303412, 3.9700493869847175, 2.7282355205404896], [2.0116390010459266, -2.2948105641008167, 3.8097660507281734, 4.2341872043300395, 2.7391174970628418], [3.9059199440211714, 0.83340454118098428, 3.5251732118082231, 4.3040779821650315, -4.0369726122257763], [1.2178638399657435, 2.473536910541501, 3.0520236042252353, 3.675161782243519, 4.2574292474704016]], [[2.1036881658446083, -1.1101286503693428, 3.8630949770858631, -0.31182191075319476, 3.1223293775787937], [1.8751177472837268, -3.4943776781204914, -0.032247230173760855, 1.6119559541632675, 3.276562737887037], [-1.5592652127982234, 0.035361131982682537, 1.1832564086647004, 3.3867969666963926, 3.1724653589585277], [-0.6584976561476541, -1.7537222733244553, -4.8369610013425977, 0.3115519845026764, 1.1679947081717952]], [[4.3272843885267758, 4.3562820957610953, 4.673270490757309, -3.673214766636077, 1.7096492414278428], [0.65931297745318851, -2.4338109749215917, 1.2450305452097812, 0.78095647728248974, -0.87917694480424213], [-0.38590510705432379, -2.1511819411219415, 2.2038884786909998, -2.3754081197463304, -3.0850883783473169], [-2.9076527919710173, 3.8514998632600719, -2.1654581633615, -1.8426603747933168, -3.0894978331370515]]], [[[-3.0351716021017836, -4.7187048469277091, -4.9058776587223445, -3.1514984318384123, -3.5726359179109179], [-3.7602990574020332, -3.7751329291179712, 0.15935411108317865, -2.5599488020963879, -3.3529656940446362], [1.9195126051726739, -0.11770425269718743, -1.7686799381460947, 2.007932992061189, 4.0525068415449024], [0.73263544233289046, 2.6795471617538311, 3.3390913863729264, 4.347360147172413, -3.9334130026597025]], [[4.0728292235837049, 3.4270535475975024, -1.6214496169735249, 4.0259745641708964, -1.1588428039252685], [-0.35105270701958702, 0.026732223257920573, 3.5503868497193771, -2.3510507985823939, 0.76834294474884679], [0.036155992976635076, 0.7412041489697927, -3.7408663902623331, 2.6996899325898793, 3.1218379582669069], [1.6038686202801848, 4.8840076417266491, 1.4960616446761694, 2.1730312123230942, 4.4507566077334815]], [[-4.1300462931476156, 4.4194499274573822, 4.8150947195279041, 0.26669319773586153, -1.551589264919393], [-0.42578534071326324, -0.12350309840157436, 1.1178101731658128, 3.5494567557419749, 1.9412413327829672], [2.0738792235982642, -2.4939041629484104, -0.87827547611310042, -1.8133992694942922, -4.2012610058075293], [2.2243811770716757, -2.7107281628470692, 2.8292761907370707, -4.96541053114231, 0.13735546551302846]], [[-1.8987326814354413, -4.7771510466371394, -4.3756564609162218, -0.14167577448499369, -1.417157166185433], [-2.3057349253436521, 3.6944779624655695, 2.7225289778326776, -0.033319937960538226, -1.8746329625727443], [3.5032642826504343, 3.2145526371323268, -3.4565074267636495, -0.91255764744636014, -0.13522531116066627], [2.9414492139841482, -3.3263609051206178, 2.2386871730994526, 2.3750303604851464, 2.6049332014891746]], [[3.2929746118813377, 2.3984199581931795, 1.1407875689889622, 1.0937947824592467, 2.1995680908698114], [2.1652366136914445, -4.7531303234861069, 0.91222526922405578, 3.3827024058272457, -1.552068391821666], [-4.8887682715464598, 3.5090051512798741, 0.37786330998990092, -0.53988467748145919, -2.9387368409821466], [-3.0274603263361586, 0.011646505257676409, -4.6528338926298467, -3.0455351314329473, -0.056055694198757777]]], [[[-3.9862778645268149, 2.8748198663383917, -3.5350729767657496, 0.56231255549151893, 0.6519853226482919], [-2.175892327928175, -1.1111864891735292, 0.38121231813503798, 1.1523387815514639, -3.7681989269267535], [-1.4819554830107542, -1.3833960192003438, 3.5853652010262671, -2.7967605987509225, -3.6211135324001531], [1.0143995091228852, 4.320953861579591, 4.9376461512382193, 4.5107596378675492, 3.7400748038058467]], [[3.4532090643555549, -1.8565253186127926, 3.9274435079986283, -3.8526496496491411, 3.2224592771538614], [3.2404190073270875, -2.9379889111985658, -1.2154104468442375, 2.4904443670167185, 0.45128306051772604], [-3.0708292158984518, -1.9115385020919828, 0.44850655090406377, 1.3583879126535416, 0.586879424964315], [0.52464397724801515, -0.85213879037054241, -4.9532705811965307, -4.4306333957943931, -3.9687485255997226]], [[-3.6294383231034431, -4.0139523071173446, -1.7145446665193931, 2.0468625882989864, -4.2370570744371587], [-3.6335075660219696, 3.9710554327681642, 0.076266577655587753, -0.38565614402336301, 3.9461299153659013], [-4.3295848790842424, -2.1542417852197859, -4.199232181900495, 1.6714259439640742, 4.2865935066301102], [-2.2794510245603328, -3.3422427010371001, -2.0880520394041389, 0.41149878899276882, -2.3756759055269572]], [[2.9083061747422514, 2.7139524921463511, 1.4551963052114525, -2.0368287660765114, 4.5213464827470631], [0.97604310551771878, 4.4830614409473508, 2.9403354476187618, -4.9623680382291688, -0.72751287552633936], [1.6020655793829306, 3.3832695048195731, -4.3861007965652012, 4.1518965435550186, -4.6188798698905966], [-4.2187840626448505, 4.7061810281522796, 0.23862281088700588, -1.8855280843414524, -2.0186117114474067]], [[-3.1957167306125589, -3.4273588545659841, -2.5654020748132167, 4.337692198733583, -3.7826466065984055], [-1.1888468911581085, -2.1035488611720812, -0.047444978035380281, 4.1780796359689969, -3.2184267595986347], [-3.9931002154239117, -0.51578883551152011, 1.5461482729698108, 2.7301502241346984, 3.3480980533796121], [3.0944210804428476, -2.4713665679448171, -2.1877621846342921, -0.88461089745553689, 0.85970724467979664]]], [[[1.5031857274015392, 1.4282951984049683, 4.3483628980665134, -0.41283505063964032, 1.6641604327852155], [-1.4374619826348365, -1.9568062602940159, -2.7484127753259049, 4.0665718817566514, 1.8812299214788286], [-4.3181788050906018, 1.2566928237879296, -3.1407095172610555, -2.1759695697141312, 4.5069051575034909], [2.0586823317827658, -0.32716665054664329, -3.978931282042244, -0.79647762963565683, 0.21193429605517711]], [[3.5979679757381131, -3.7065473292727855, -4.5217972922250098, -3.0243720981666877, -1.509108370024471], [2.738695983893348, 2.4543506852102883, -0.10345639114878757, 3.2768132640270924, 0.12004752131409369], [4.6789708461699053, -2.3399889313253706, -2.0092322839081156, -3.2168370540354396, 3.5925981722988691], [-4.2245945632980959, 2.9534512074764141, -3.3978493120307594, 4.0957244948775013, -3.1633596067286254]], [[2.357987769563473, -3.8263731562905856, 2.1191402110619553, -3.333421752064194, -3.073356958539033], [-4.8449360850183352, -4.8417428041340909, 2.7755388948362452, 3.5948977795498767, 3.7605077775579812], [-3.4756593191159455, -0.17686686187599854, 1.8001889136903468, -2.258139231867534, 2.0504077153656866], [2.2009630218127541, 1.5704599235221375, -2.2355409474606516, 4.6417353850749556, -1.9139718797542224]], [[-1.7641236603411112, -0.17007029738527635, 4.5169817589920704, 0.89781978872139234, -1.123710748621678], [-0.91102382560365847, 0.66516354895056296, 4.3290614414169397, -2.618306161820827, 2.6319467313966003], [4.0263059005415371, -2.8055734212474439, -2.5548438744202793, -0.33487505996598443, -1.0596489398158049], [0.47144259727148885, 4.546003101084688, -1.5624559287773554, -0.28309020529081774, 4.3981934250743375]], [[-3.7371426099532732, 4.79413516840998, -4.4689836145754294, 3.3835450814869468, -4.8999864604272281], [2.2611657291453824, 1.2295792784138238, 0.71330067747092762, 1.2459293241141376, -1.6468484628895608], [2.3282847925877199, -2.0392093458369676, -1.9676322269264146, 3.3843877206612323, 3.6529457821986728], [-1.1313600718644889, -4.2854687500534432, 1.7512862176667552, 4.6790224144707491, 2.6094085954704553]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[2.9965465128663511, -1.2145815315678243, 1.2706100167507302, -2.7072409548100125, 0.41286929345181145], [-2.6156211890118812, 0.35491418850510797, -2.0741471437451375, -0.89635587203745637, -0.049846195689698586], [-0.5535629318043479, -1.3707106170153205, -1.0425874985269281, 2.2700236362985944, -0.31763760417653863], [-2.4038146359375308, 1.2678461364385323, 1.0811344781279513, 2.6405827820233538, 2.4945078788009782]], [[-11.539106581118004, 13.678481344999634, 18.141274286097946, 8.3134269745992277, 5.8219721964813242], [-7.3039375121847616, 14.274750838202618, 10.07658399039129, -5.9201488247290168, 12.023841638349355], [3.8856071586198562, 5.4094411418188191, 5.2500664259670238, -7.6542063533761064, -10.021793198102248], [5.1902492130970064, 2.0035429288433884, 10.326197701718772, -5.8062195195988782, -1.8352987277079074]], [[3.2497719971391668, 9.4149923756422513, -8.863987653219608, 8.5885638446093004, 5.902099109424273], [4.3518576996622942, -4.9644538694348528, 8.2418166048446455, 9.1599835643445839, 5.9256405168496213], [8.4498301006369179, 1.8029367931255302, 7.6261457331432165, 9.3111810304398706, -8.7333414875661664], [2.6346527017718167, 5.3510995978612748, 6.6025625943287345, 7.9506219014670583, 9.2102639895817564]], [[1.2120560954433941, -0.63960914894736376, 2.2257518439639035, -0.17965858902356596, 1.7989540538946485], [1.0803634931115578, -2.0133125400012961, -0.018579489359622173, 0.9287408046265444, 1.8878167891219817], [-0.89838262926489687, 0.020373587805101817, 0.68174226859269549, 1.9513290867733963, 1.8278402846673283], [-0.37939848259236142, -1.0104205583360986, -2.7868522342262074, 0.17950306894099249, 0.67294912262667705]], [[-0.34088784187052573, -0.3431721765596229, -0.36814337793357499, 0.28936259836433237, -0.13467999510527492], [-0.051938296127970636, 0.19172684211870766, -0.098079011572414512, -0.06152093189009708, 0.069258385728300495], [0.030400211148440132, 0.16946234717648473, -0.17361437792630585, 0.18712607603257814, 0.24303212473463007], [0.22905438981357482, -0.30340725463581969, 0.17058697642484433, 0.14515813199822494, 0.24337948566405815]]], [[[6.791531431652797, 10.558622867455005, 10.977442690879101, 7.0518255514164885, 7.9941671546813806], [8.4140841404405968, 8.4472765655219781, -0.35657241041497001, 5.7281679694222802, 7.5026307852210978], [-4.2951212980697848, 0.26337625565506478, 3.9576165591872545, -4.4929716721100572, -9.0679313065137332], [-1.6393526583805382, -5.9957824984410593, -7.4715854905894572, -9.72769811887758, 8.8014456984034908]], [[-17.865555383614225, -15.032846111662277, 7.112524572854884, -17.660026384786388, 5.0832895655302419], [1.5399004562994127, -0.11726148800348907, -15.573850366626736, 10.312936277452284, -3.3703533046596577], [-0.15859906210486005, -3.2513083773084772, 16.409393080107179, -11.842249542403135, -13.694011184935293], [-7.0354051423849686, -21.423807438821594, -6.5625074617651169, -9.5320494287557072, -19.523342204975535]], [[-17.687745567513961, 18.927174253453401, 20.621601850758115, 1.1421667195268912, -6.6449899577915774], [-1.8235104980321499, -0.52892660911600331, 4.7872446293252588, 15.201255274698116, 8.31375252049253], [8.881800696912526, -10.680641129176498, -3.7613895963951283, -7.766243430333458, -17.99273675375855], [9.5263552785084435, -11.60923307970771, 12.116938612773499, -21.265359242780875, 0.58825212936058746]], [[3.8803934842250745, 9.7629466094803909, 8.9424219565812368, 0.28953931090947144, 2.8962093966968996], [4.7121740032881547, -7.5503141402329961, -5.5639658017626417, 0.068095141259753147, 3.8311414789476559], [-7.1595354253017245, -6.569502505427292, 7.0639795839240085, 1.864971717057907, 0.27635665697472689], [-6.01136772739072, 6.7980023247153119, -4.575150153915299, -4.853791387156174, -5.3236381933755874]], [[-1.8345177697346489, -1.3361609338609057, -0.63553331363432231, -0.60935360923360804, -1.2253804611438019], [-1.2062543783863735, 2.6479712321007343, -0.50820114444684605, -1.8845051677057538, 0.86465806154928482], [2.7235351994235502, -1.9548684890818462, -0.21050783513669091, 0.30077001835168926, 1.6371717340011749], [1.6865996311636544, -0.0064882737854783388, 2.5920960413285674, 1.6966691139391701, 0.031228654703681719]]], [[[3.4649954116370836, -2.4988819105634761, 3.0727942357700067, -0.48877938038956026, -0.56672570959857593], [1.8913525822120327, 0.96587749708121196, -0.3313614800794219, -1.0016483362233592, 3.2754344869250276], [1.2881613182507587, 1.2024903987909028, -3.1165097850182031, 2.4310303926265955, 3.1475833349296316], [-0.88174727505979733, -3.7559060891120444, -4.2919540081681742, -3.9208911118051519, -3.2509881334890447]], [[13.317399022002885, -7.1597427209440205, 15.146297648837391, -14.857853005761266, 12.427505901390248], [12.49676811186953, -11.330437840129134, -4.6872649742555401, 9.6044695700607949, 1.7403859647017395], [-11.842740255338674, -7.3719026284993889, 1.7296782763676513, 5.238661639748952, 2.2633172027515132], [2.0233044276473104, -3.286297493722472, -19.10242894779752, -17.086863770009568, -15.305591624601144]], [[-17.042728911569956, -18.848288617748739, -8.0509757590486846, 9.6114387698935992, -19.895920160102438], [-17.061836827936606, 18.646858055881072, 0.35812445130241577, -1.8109229391867607, 18.529815472910784], [-20.33040234467213, -10.115658536410656, -19.71830606882332, 7.8485034660644812, 20.128528048694122], [-10.703602712158535, -15.694146377377411, -9.8048517960174966, 1.932272071852261, -11.155445232920812]], [[4.542686667613002, 4.239112067937941, 2.2729728086588481, -3.18146519792966, 7.0622070555051843], [1.5245499393969006, 7.0024069730850549, 4.5927154273540074, -7.7510694447603541, -1.136353204099356], [2.5023781922649606, 5.2845650867029521, -6.8509573663197036, 6.4851373756710542, -7.2145512692169111], [-6.5896136663167661, 7.3509130495365733, 0.37272164499695726, -2.9451380891514645, -3.1530107071660987]], [[10.01058060482041, 10.736199409232739, 8.0361203506196333, -13.587818024784651, 11.849138064137943], [3.7240621225045794, 6.5893654557126284, 0.14862136320388722, -13.087831774504396, 10.081719756024009], [12.508383858529504, 1.6157081957526351, -4.8433084713992622, -8.5521938224888228, -10.487915000455764], [-9.6932720457644841, 7.7415541857891164, 6.85316363733578, 2.7710430677576814, -2.6930323914427565]]], [[[3.3009203258706279, 3.1364644872648979, 9.5487864292735214, -0.90656502722647081, 3.6544126902975766], [-3.1565942848246102, -4.2970482227509699, -6.0353763534187301, 8.9299875167845837, 4.1310863802438682], [-9.4825036777772116, 2.7596342952165558, -6.8968402867312442, -4.7783198378031546, 9.8969372646261355], [4.5207629567102776, -0.71844152525534533, -8.7375331635431746, -1.7490248535757256, 0.46539706494348732]], [[-4.7296697669915133, 4.8724015781680903, 5.944079572057146, 3.9756555292566742, 1.9837820350122284], [-3.6001230926309584, -3.2263400652009979, 0.1359974765528833, -4.3074993250228406, -0.15780716670916414], [-6.1506903621688283, 3.0760070623791322, 2.6412145000023264, 4.2286582488795599, -4.7226109501391846], [5.5533949491945345, -3.8824272418433923, 4.4666059487669383, -5.3839901989051784, 4.1583600507164613]], [[-10.736178220079514, 17.421898736170721, -9.6486789596495566, 15.177436657984131, 13.993332987857292], [22.059528020062945, 22.044988660222938, -12.637334517320673, -16.367965831884128, -17.122006406899995], [15.825059978668623, 0.80529431697537601, -8.1964585468434716, 10.281556822311881, -9.335732330820095], [-10.021227235780371, -7.1504771331140065, 10.178664342549254, -21.134332835763537, 8.7145249328693737]], [[-8.6159419683885581, -0.83061967012833537, 22.060841642031008, 4.3849325144239852, -5.4881790982309946], [-4.449420746220655, 3.2486444494130775, 21.143042857678552, -12.78775091161148, 12.854371159682925], [19.664391315594379, -13.702360174029577, -12.477802466546928, -1.6355225816594203, -5.1753026035329528], [2.3025154930080762, 22.20258125186448, -7.6310011101530835, -1.3826064665632718, 21.480681977170072]], [[18.290644251229697, -23.463867989451039, 21.872483335549195, -16.560036865772574, 23.981934471758688], [-11.066791466485597, -6.0179124821870698, -3.4910974232115266, -6.0979342797495466, 8.0601471536495719], [-11.395291349960706, 9.9804734770218353, 9.6301545956834378, -16.564160982772542, -17.878560907275983], [5.5371996079051877, 20.9743091281094, -8.5712953806236332, -22.900473264969776, -12.771191604713152]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_array_rank3(self):
      arg0=numpy.array([[[0.24891356953890309, -4.8831885442578615], [2.1820821120027398, -0.50840354796168796]], [[-3.0840244321429511, -0.37664930125040641], [-1.7725697666913498, 4.0459742134790524]], [[4.1836393596561958, 0.93427790273309341], [2.5411957985681033, -1.544313814315875]], [[2.1188737704770144, 4.3741088239213646], [2.6674041216911331, -1.8587499871815871]], [[-1.2240328896028552, 2.8885663226998517], [-2.956491176006788, 3.1352164685936739]], [[-0.65953899908719471, 4.6360467202840141], [-3.4960916350457047, -1.5779330892286469]]])
      arg1=numpy.array([[[3.7659686329022115, 1.3300154269666464], [-3.1622418106987427, 2.2245546011600714]], [[1.1953446157559124, 4.8894209071934007], [3.3275826269772715, -2.0357942597279566]], [[-1.7202029647000003, 3.5922774843503866], [-3.7834251031652375, 4.8406183413972368]], [[2.7929336437930186, 3.0395584718044404], [-3.699521432788011, -2.8816387771627436]], [[1.6533319438996061, -4.9599586131514561], [3.1727481984656194, -1.2163476475558532]], [[-2.0287825939960435, -3.6760729715323359], [-0.95775577271894008, -2.8792877901087022]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[0.93740069518723246, -6.4947160966497561], [-6.9002712889528803, -1.1309714518642779]], [[-3.6864719998217619, -1.8415969682135225], [-5.8983723607472909, -8.2367710788079886]], [[-7.196708829716199, 3.3561854741141919], [-9.6144239763605945, -7.4754337744505515]], [[5.9178738405158198, 13.29535953234474], [-9.8681187181034264, 5.3562460401132137]], [[-2.0237326767641406, -14.327169411934358], [-9.3802020524550365, -3.8135131761522847]], [[1.3380612414096731, -17.042446043197195], [3.3484019454194218, 4.5433234774245488]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[2.4099680189302948, 0.88906684895291477], [1.279478936912648, -3.6873406235594053]], [[4.3966126187841486, 4.849723747353762], [-3.9112344210049974, 1.5414719159556292]]], [[[-2.0510264989715576, -4.5016651227893556], [-1.7762189279046039, -2.6129970026104612]], [[4.7945661446989281, -3.6897675477074854], [1.3289158592425556, 4.8895388555239983]]], [[[0.73438837825948955, -1.6298660354564722], [3.0967138698588386, 0.6417537303184524]], [[2.6155635413764875, 4.3938239564604249], [-4.3859990404736351, -2.3746069911319312]]], [[[4.1858368990189785, -2.8829508176260221], [-0.15850516992769315, 0.81097896352379184]], [[-2.9462603181482736, 4.188068365071608], [0.64939672493270795, 1.3627367775965329]]], [[[-2.0737031552793717, -3.06586488796601], [2.417814168885287, 0.23617344414005181]], [[-0.2421384751641531, 4.6676347021316182], [2.4416479872466548, -0.063604501168873284]]], [[[2.4650876137472313, 0.53047757422037911], [-3.8705307636115327, 1.7577786323620694]], [[2.9801038402521538, 1.0705374458320591], [-2.9648386813741814, 2.3028560979088422]]]])
      arg1=numpy.array([[[0.6406635552004003, 3.452482042486281], [1.3656049575302331, 4.2181928984938608]], [[-3.0789774227800382, 0.1085633797294161], [-3.8774974534432296, 2.0509977820758767]], [[3.8417320486583169, -2.9102432814233614], [-3.645902087041172, -1.8372145005554277]], [[4.0986367889569308, -4.9121230066757775], [0.39500721352950308, 4.60393731210028]], [[-0.3551716239547611, 0.12009206696899533], [1.408233579512542, -1.5947443334089115]], [[4.144889771931954, -3.645275656445758], [-3.1156729017816867, 2.8805911737818066]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[1.5439786789271484, 0.56959272826099172], [4.4173780534303546, -12.730477287369013]], [[6.0040359885516139, 6.6228067920383973], [-16.498341259028027, 6.5022258891117604]]], [[[6.3150642838570112, 13.860525277984754], [-0.19283232995268387, -0.28367578582622555]], [[-18.590918016435218, 14.307064270033244], [2.7256034798719395, 10.028433348053541]]], [[[2.8213233689216879, -6.2615085834328017], [-9.0121907342472234, -1.8676594819876557]], [[-9.5360885742933341, -16.019451932950563], [8.0580210365803548, 4.3626623972278775]]], [[[17.156225106892581, -11.816168281875477], [0.77859689187887504, -3.983628424655294]], [[-1.1637940786042968, 1.6543172149579974], [2.9897818122734163, 6.2739546969479791]]], [[[0.73652051726068657, 1.0889082110847694], [0.29036030108835764, 0.028362557069965381]], [[-0.34098753161812406, 6.5731199244397667], [-3.8938042918408771, 0.10143291781836115]]], [[[10.217516437137046, 2.1987710716253233], [14.109151570117531, -6.4075876579699695]], [[-9.2850287795691759, -3.3354445103215267], [-8.540488137253357, 6.6335869501258227]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_array_rank4(self):
      arg0=numpy.array([[[0.51310697192654864, 3.3127196263697627], [4.1917288581924588, -1.8434253897711397]], [[4.8142064611780171, 3.4436079734240028], [-0.97601346154803981, 4.6809775705955712]], [[-3.001254227827058, -4.4068744618618876], [2.2517675915112214, -2.7017385288335714]], [[-3.5468655256602108, 2.2912650530856569], [4.091227046750376, -0.56051997553464972]], [[0.087571539179491786, -0.74680048022867762], [-2.5551526810426397, -2.5243970759507284]], [[2.5565790956037784, -4.6955536261727744], [0.86268103189302092, 3.7488505688787672]]])
      arg1=numpy.array([[[[-0.78000850411842482, -3.6120960746461384], [1.9614558239096516, -3.2556658670247582]], [[-0.91014557503024207, -2.0227185832036874], [-3.1745104295293372, 4.6021692239514227]]], [[[0.33918803996439451, -3.492840046776049], [-2.0330747787219616, -3.5637409782902485]], [[-4.9673315205328414, 0.021737423722473892], [-2.1609778441889826, 1.7448159409711783]]], [[[2.7359160869957533, 2.0160542963760744], [-4.8106405526613507, 0.6450431360439346]], [[4.0888418632653352, 0.67295229398989242], [-1.6819558040825369, 4.5496252007694551]]], [[[-3.0832544271806772, 1.2170404973542395], [-1.0084298419509574, 3.5649603888198307]], [[-3.425746377236929, 4.8620254726544836], [3.7684683036897955, 2.2099839810440312]]], [[[-4.9027509815454682, -0.32149050639034193], [-3.0432811878591171, -0.37497517698248028]], [[-2.743902064210145, -2.8882951654248821], [1.9138904783611288, 0.34984488067772102]]], [[[-1.1127599715233369, 0.30353068730054034], [-3.4894762927757683, -4.2589154693571531]], [[3.9761574822314731, 2.4073666189376706], [0.93721673856342669, -3.741634016752684]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[-0.40022780162516181, -1.8533916791694527], [6.4977532041227759, -10.785108214595047]], [[-3.8150834720104356, -8.4786878572170608], [5.8519731258876666, -8.4837555954553956]]], [[[1.6329212535508955, -16.815253121050581], [-7.0011125185741871, -12.272126848058155]], [[4.8481824320119467, -0.021216018172508219], [-10.115488819202598, 8.1674442845036914]]], [[[-8.2111797230760661, -6.0506914805275978], [21.199888996720464, -2.8426241230313187]], [[9.2071215945152378, 1.5153321662395716], [4.5442047996850397, -12.291897696671009]]], [[[10.935888834606365, -4.3166789833981092], [-2.310580055350921, 8.1682691545375334]], [[-14.01550623385884, 19.891650115713304], [-2.112301761387307, -1.2387401669867681]]], [[[-0.42934144966770077, -0.028153418476196485], [2.2727238525640892, 0.28003164224434968]], [[7.0110887158849851, 7.3800351355778817], [-4.8314195272647744, -0.8831473938191704]]], [[[-2.844858881621219, 0.77600021002680863], [16.385023059987187, 19.997965975703305]], [[3.4301556397406032, 2.0767895189699623], [3.5134855035262049, -14.026826812239447]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[4.1382894871164257, -3.4087964636052472, -2.4646571032335185, -3.9312641551704273], [-2.5353578168789759, 1.2208050361505336, 0.97600053911507878, 2.1577184592931218], [-4.5518274763127877, 0.078660345808561516, 2.8711215605438989, 1.0904502585365368]], [[-2.4344740200292838, 1.1897204591627339, 2.8322065691212668, -2.0580454748976509], [2.3729997983805866, 0.78353026153793781, 2.5296689163528621, -2.184371637924615], [3.8646623852461417, 4.9481846415888668, -2.4766372203813534, 4.3294089771337916]]], [[[4.0146928088633285, 2.5484208512240709, 4.882124815286101, 2.3500912441682109], [4.3002439232920313, -4.7542362763293493, -2.4995962440265251, 3.2666456793537186], [-2.4716302308880866, 0.12916604701719159, -1.9022185085430388, 4.7498404359202429]], [[4.88726296597976, -1.860329901568142, 0.1346280691444246, -3.4926302425472757], [2.66884405853241, 1.9528152290211942, 1.629020456484688, -3.3963776709844593], [2.2616321990040325, 4.0897528996062942, 3.3211729567263077, -2.0645746102864129]]], [[[-0.014847639821546466, -2.1603300709997031, -4.5505653129588532, 0.24510902499269704], [0.082776073975603381, 0.23705135048879722, -0.47748554305366397, -3.9570565292484039], [-2.4914136813485035, -2.337832987504628, -1.9971801212038676, 0.30150618992565903]], [[2.7050637080567297, 4.3204223239276889, -1.228418737832456, -1.2716692020326303], [-2.4648180899619101, -0.56111944493602195, -2.4367313394895707, 1.1954196912797173], [-0.22373173119896883, 3.247881180693966, -0.79870795409590656, 2.1588427350825121]]]])
      arg1=numpy.array([[[[2.3054799445427747, 0.088058357906913187, 4.2955154326577372, -0.13815619457337469], [-3.1009697538153391, 4.9712152294443133, -2.0629247179447132, 2.4773522921019016], [-3.6064588112594231, 4.7545411290087163, 2.0570829710089091, 2.4888719138345881]], [[-3.6626101039214798, -4.3682697996318112, 3.585287159073772, -4.0360703181218796], [1.4435364773628843, -2.7360934455536068, -2.2438802387152301, -2.1727227166505925], [-4.5622889116377809, 0.15562836355115994, -3.9346707649177723, 4.0479588432521769]]], [[[-2.466275262924118, -0.89342290887696052, 4.309423209734577, -4.9874766192440525], [-4.2761536520538304, 1.8557833775210497, 2.5802532350156238, 2.8808908631270338], [-1.2592944517152227, -1.4507334351248549, 1.2142551549923128, 3.9839898839518053]], [[-4.7090904173431012, -3.6705246264378086, -4.4697078470151101, -3.1302064186686906], [-1.6475137619516111, -0.12902982436711952, 2.0546749331697374, -3.5824052208956871], [-2.4694556863537032, -2.4135718027139164, 4.353752309206568, -2.8814155862753497]]], [[[-2.514655409965374, -4.988385933328888, 1.6011049034189391, -0.34383806049490229], [-2.7490493958527216, -3.2100298449194087, -1.1296526441136558, 2.5988633030750039], [3.6917616254909547, -1.1292189816389619, -4.672208488275496, 1.5270723942264368]], [[-0.59157014250760476, -0.98313986821169053, 0.98745189728956984, 2.5333344341636757], [-1.6453205306973953, -4.5768836121510823, 3.5723435583348877, 1.7278228053105229], [-4.0150209992102806, 3.9061569586192881, 3.5640998715573815, 2.2025154538904159]]]])
      res=mult(arg0,arg1)
      ref=numpy.array([[[[9.5407434172591241, -0.30017301902397081, -10.586972623149093, 0.54312849554105902], [7.8620679052409939, 6.068884587893848, -2.0134156368678617, 5.3454287708403987], [16.415978309280998, 0.3739938493688541, 5.9061352698913794, 2.7139910219052514]], [[8.916529143493598, -5.1970199517646618, 10.154273844114861, 8.3064162545794566], [3.4255117697371467, -2.1438120129868548, -5.6762740918963575, 4.7460338793260739], [-17.63170634743209, 0.77007787831945818, 9.7447520663417251, 17.525269355044092]]], [[[-9.9013375627389717, -2.2768175699433093, 21.039141991815058, -11.721025133379118], [-18.388503757307511, -8.8228326544195781, -6.4495912948823442, 9.4108496907235306], [3.1125102364489825, -0.18738550309074886, -2.3097786299201739, 18.923316247091481]], [[-23.014563200131111, 6.8283867170044896, -0.60174813708332742, 10.932653603257869], [-4.3969573149349364, -0.25197140602204099, 3.3471074975598114, 12.167201100668262], [-5.5850004942711378, -9.8709122785572294, 14.459564429621567, 5.9488974611076255]]], [[[0.037336697802469142, 10.776560137522317, -7.2859324359065587, -0.084277811763285485], [-0.22755551615369266, -0.76094190984749011, 0.53939280623661634, -10.283849002057018], [-9.197705422025555, 2.639925385391948, 9.331241914903794, 0.46042177932386691]], [[-1.6002349232672692, -4.2475794341651136, -1.2130044133387174, -3.2215633783747064], [4.0554158078486697, 2.5681783919869905, -8.7048415040183098, 2.0654734045103602], [0.89828759895352972, 12.686733674736365, -2.8466749166050795, 4.7548844865382858]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_constData_rank0(self):
      arg0=numpy.array(3.64144768148)
      arg1=Data(-1.76415045635,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(-6.42406158907,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_constData_rank0(self):
      arg0=numpy.array([-2.4816542133922512, -0.64652668187649454])
      arg1=Data(-4.40746937526,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([10.937814945511438, 2.8495465506591517]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_constData_rank1(self):
      arg0=numpy.array(-2.53792283209)
      arg1=Data(numpy.array([2.7314645196869911, 0.63939050246702411]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-6.9322461695457349, -1.6227237548298366]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_constData_rank0(self):
      arg0=numpy.array([[-1.702364212701247, -2.9561995955849851, -2.5241863030428524, 0.70872519944397272, -3.8208705079726624], [0.85695547797479854, -4.3956226414894264, -1.7563955253828789, -2.3754141712762769, 2.8263571074594465], [3.1210190975785075, -2.0785720057931591, -2.1994927683893906, -3.6685439198155012, -3.7876217827023897], [-0.46734236036753085, -3.8244174555734824, -0.10190575657306056, 1.5739649765438362, 4.6229905491234842]])
      arg1=Data(1.13269679786,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-1.9282624925119609, -3.3484778157433706, -2.8591377426495153, 0.80277076397029601, -4.3278877894044276], [0.97067072580751834, -4.978907690599975, -1.9894635873704209, -2.6906240253872511, 3.201405645217871], [3.5351683378757128, -2.354391855075777, -2.491358415662877, -4.1553479507704267, -4.2902270647579837], [-0.52935719509093393, -4.3319054055940409, -0.11542832415343511, 1.7828250888692594, 5.2364465915123724]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_constData_rank2(self):
      arg0=numpy.array(-1.08019291391)
      arg1=Data(numpy.array([[-2.2903092576199011, -1.2463701908637006, 4.4520175029126854, 0.48656666885032962, 2.1057897751321075], [1.0075366840633428, 3.1680944638998714, -4.1586037442865331, 1.0023477914501564, 0.29566936831945334], [3.2636239819440647, -3.8483925305565814, 1.9017664544718009, 3.930908051911544, 4.1863822082104321], [-1.7324976600719322, 4.7312577238862925, 1.9713919313512598, -3.1859815609499753, 2.0729087421011236]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[2.4739758307401862, 1.3463202482778258, -4.8090377592431537, -0.52558586783621775, -2.2746591932787972], [-1.0883339866281481, -3.4221531904975717, 4.4920942963319082, -1.0827289815963517, -0.31937995651849282], [-3.5253434989580086, 4.1570063414458414, -2.0542746480294407, -4.2461390229009419, -4.5221003962217683], [1.8714316957728581, -5.1106710672171038, -2.1294835947821364, 3.4414747059814883, -2.2391413343967352]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[-0.71948751986650805, 4.4630675745825634], [-2.6654000660714496, -3.0092792497036722]], [[3.6813075854780184, 2.6963113827469893], [-2.370224854520453, 1.5259454427976209]], [[4.1955237263749545, 3.6020225269017363], [0.61945718165683239, 0.65898270419168536]], [[-1.4110667461849258, 0.59024194774528205], [4.7525440489584572, -2.0480588726543592]], [[-0.86493577541786593, 3.0389085599194239], [-1.7432724230906436, 0.16555215464041417]], [[1.8202135179975398, -1.9547513857013588], [-3.1880112157904685, -3.8170481164431269]]])
      arg1=Data(-1.0880537574,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[0.78284109939132529, -4.856057444043274], [2.9000985568564475, 3.2742575946982546]], [[-4.0054605505150862, -2.9337317311113709], [2.5789320588377991, -1.6603106726194643]], [[-4.5649553547324127, -3.9191941446256853], [-0.67400271404855017, -0.71700860735569438]], [[1.5353164751250998, -0.64221496901784247], [-5.1710234096661312, 2.2283981517627574]], [[0.94109662035088448, -3.3064958770075976], [1.8967741101111288, -0.18012964390174604]], [[-1.9804901575228511, 2.1268745899902322], [3.4687275819660264, 4.1531535452627972]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_constData_rank3(self):
      arg0=numpy.array(-3.95678694013)
      arg1=Data(numpy.array([[[1.6187188342692895, -2.6237881438829556], [1.5829541450697473, 2.3910332205363876]], [[3.9649301516446336, 3.8057857150768442], [-0.59450175667981053, 4.6914924585408819]], [[3.4094173311575471, 4.5201964624977453], [-0.16941210194251255, 0.64187681317710776]], [[0.4075825773935815, 2.3983192898769579], [3.6088674312929321, -0.81568822119453088]], [[-2.370812868387647, -2.6335188819720425], [-3.7691175930154319, 4.5615334718052729]], [[3.3500757280491538, 4.4671415564275758], [0.29829072476031904, 0.52587848078954025]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-6.4049255431804859, 10.381770661386126], [-6.2634122880379, -9.4608090204372779]], [[-15.68838384255834, -15.058683214352435], [2.3523167867154959, -18.563236089676725]], [[-13.490337969379807, -17.885454329636545], [0.67032759246624229, -2.539769791551961]], [[-1.6127174192557765, -9.4896384444489339], [-14.27951952080328, 3.2275045008410475]], [[9.3808013951302964, 10.420273118774858], [14.913595267860718, -18.049016068208633]], [[-13.255535889194091, -17.675527370188231], [-1.1802728440937831, -2.0807891048838814]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[-2.18199196547469, 3.7691980134317848, 2.3721747508550983, -2.5128601220261446], [4.5024694848917584, -3.8634643780883637, -2.3612711255385896, -1.3637604694154626], [2.9654830547742526, 1.4060656572272965, 2.3147695964061228, 0.42838639092270192]], [[1.9651922950345027, -1.1202208387930557, 1.2597157121740201, 4.1676801892290705], [0.27896741448014062, 1.4121106152201, 1.9927923309906346, 2.8296094374177825], [-0.0081402267591688116, -2.7732264462760248, 1.6754322194620572, 2.8295913540800113]]], [[[-3.7326402675202406, 3.2226753295634172, -4.7517892633878045, -4.5140582313285824], [1.8689481529879561, -4.2670559142422011, 2.7446741446852405, -2.5209090828356437], [3.3278997021620551, -3.3656606133596445, 3.3662018625639973, -2.8659871380460542]], [[4.9999319461749216, 1.0347042993880509, 3.4455228191014147, 1.2318852674531113], [3.7538571300580728, -1.5649188870690525, -2.1027961227439427, 3.4956999677997871], [0.88114299899422299, -1.6546923055738114, 3.4908655525136929, -3.9618074229523303]]], [[[-2.8604207878358343, 4.954449361060572, 1.2383361670787174, -4.3850162505649068], [0.89953356644362792, -2.1686063366282751, 4.4437455583138625, -4.2696078714180157], [0.2570743651161429, -4.012389848366043, -4.4640021911736891, 3.909533051267184]], [[4.7049145880113326, -0.79269171929174753, 4.1521787930148442, -2.9061203682158308], [-2.5069362702072584, -0.19734062272090824, -1.0624544108777867, 0.93448342865358924], [0.90289978309033714, 1.4419702619228669, -2.7806115353349292, -3.7544389600917505]]]])
      arg1=Data(3.69291872498,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-8.0579189870628234, 13.91934192196903, 8.7602485563634591, -9.2797881978921275], [16.627253869419029, -14.267459945145337, -8.7199823542619299, -5.036256573895268], [10.951287901594103, 5.1924861941294909, 8.548255986588325, 1.5819961245661136]], [[7.2572954245242229, -4.136884511694463, 4.6520277416420877, 15.390904210542589], [1.0302039885936618, 5.2148097326928466, 7.3592201141168125, 10.449517675827289], [-0.03006119582453801, -10.241299872069371, 6.1872350156903986, 10.449450895530624]]], [[[-13.784337137549135, 11.901078069083852, -17.547971547935564, -16.670050168134651], [6.9018736301906571, -15.75789068625229, 10.135858542883417, -9.3095123559821769], [12.289663124977896, -12.429111101011873, 12.431109890333511, -10.583857567649241]], [[18.464342307667497, 3.8210788820300183, 12.724035736014061, 4.5492521712076543], [13.862689286500476, -5.7791182611360581, -7.7654551765016784, 12.909335868008512], [3.253989480372991, -6.1106441993379716, 12.891482765274151, -14.630632816995274]]], [[[-10.56330148872811, 18.296378817438104, 4.5730748192280375, -16.193508621063643], [3.3219043512699518, -8.0084869476502316, 16.410391181355021, -15.767314856692209], [0.94935473665038883, -14.817429602960601, -16.485197280148167, 14.437587810962501]], [[17.374867181510314, -2.9273460933110558, 15.333658814199701, -10.732066324837241], [-9.2579118945861403, -0.72876288084574647, -3.9235577883708155, 3.4509713518606757], [3.3343355157569321, 5.3250789811228563, -10.268572405740663, -13.864837937526602]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_constData_rank4(self):
      arg0=numpy.array(-0.670498696443)
      arg1=Data(numpy.array([[[[-3.5530341018916864, -3.6676652688620228, 1.4867517409047721, -1.7353348257634673], [-2.4077689697156601, 1.795356622253462, -0.67378515571433351, -1.4862429003246702], [2.1124047123420659, 0.71911666942004082, 3.4476116110158568, 1.7689938256193969]], [[-1.056820465149606, 4.5752938625964052, 0.7503188056837331, 1.7549765526652603], [-3.8496590429106803, 1.6493873745706917, 3.39029579302216, 2.3321293899683742], [4.6978840559891104, 3.4341877893772725, -4.8095929091012977, -4.9524314386040116]]], [[[-2.0438523520707408, -1.1053606804507075, -3.530416397349927, -4.2582421288083472], [-0.4538041722352375, 3.3863435399107615, -0.56495789905900828, 0.88665540309261104], [-3.2411310298106635, -0.19127410428961511, 3.6247248458248862, -4.5160252965185324]], [[-4.6524001052919361, 4.4622240048700661, -3.6050347484181833, -0.026015695857639187], [3.7799346845216046, -3.9254810329135181, -1.095191544939178, 0.23125199502933924], [4.4613406106190041, -3.6172871757496505, 0.36638128754221011, -3.671150164127476]]], [[[0.58029738050952417, -4.09555555501636, -4.3002398665443238, 2.7289750392883274], [-3.1253586982798387, 0.31106438392216695, -1.1336214630234522, -4.5072490553320605], [3.8909318587119479, -4.9992829815098823, -2.180800247740514, 0.36017265139512489]], [[0.42650383202922981, -3.5878957576303216, -3.9315812904174918, -3.1257858880657317], [0.71759482882526182, -1.3539256744088868, -0.64317789468593212, -3.6497980757461344], [-1.1400064137516064, -0.946513599085538, 3.3321771708844121, 2.0076287083606745]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[2.3823047337348422, 2.4591647817601583, -0.99686510421056751, 1.1635397385660282], [1.6144059555295378, -1.2037842748707188, 0.45177206858890362, 0.99652392726491201], [-1.4163646059847761, -0.48216678943635483, -2.3116190910268557, -1.186108054092994]], [[0.70859674425678076, -3.0677285707131845, -0.50308778112738806, -1.1767094908495639], [2.5811913700204707, -1.1059120845786994, -2.2731889097765348, -1.5636897159095087], [-3.149925135579652, -2.3026184361169055, 3.2248257759724828, 3.3205988238058453]]], [[[1.3704003377847822, 0.74114289534121747, 2.367139592323066, 2.8551457965033933], [0.30427510592398616, -2.2705389292173308, 0.37880353486407264, -0.59450129196747414], [2.1731741304880421, 0.12824903758943237, -2.4303732840890602, 3.0279890744179427]], [[3.1194282059281329, -2.9919153785007122, 2.4171710994450359, 0.017443490159596876], [-2.5344412786102919, 2.6320299154790652, 0.73432450323678777, -0.15505416121694615], [-2.9913230638069304, 2.4253863359990437, -0.24565817569805065, 2.4615013994928843]]], [[[-0.3890886371807506, 2.7460646608471362, 2.8833052249089079, -1.8297742064674949], [2.0955489331124917, -0.20856826392956512, 0.76009171321669344, 3.0221046161427467], [-2.6088647392137405, 3.3520127222505609, 1.4622237233119362, -0.24149529325474298]], [[-0.28597026340341569, 2.4056794284634315, 2.6361201301834445, 2.0958353633070668], [-0.48114639730136194, 0.90780539977146479, 0.43124993996767896, 2.4471848520668651], [0.76437281435677162, 0.6346361343521435, -2.2342204493941291, -1.3461124318967779]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_constData_rank1(self):
      arg0=numpy.array([-4.4315888711038252, -2.1709373908643537])
      arg1=Data(numpy.array([3.3352605940763542, -2.6140996114264414]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-14.780503730939904, 5.6750465898896394]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[-2.6774936874946729, 2.2173046940204886], [4.8078354652670789, 2.2422744699252126]])
      arg1=Data(numpy.array([-2.2496979595460753, -1.0070199145097671]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[6.0235520854542628, -4.9882658458298277], [-4.8415860592102806, -2.2580150450115211]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_constData_rank2(self):
      arg0=numpy.array([-4.7078008609050848, -3.2724708352430198])
      arg1=Data(numpy.array([[-0.96629531809589775, -4.5518231639838733], [-4.1216070665708857, 4.9183678822106067]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[4.5491259304204199, 21.429077010090985], [13.487838919684759, -16.095215451530187]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_constData_rank1(self):
      arg0=numpy.array([[[2.176629488451213, 4.8511282552907495, -2.5460944701290824, 3.5882742372988581, -2.9777733385312555], [3.9196357386806806, 0.93684797654242757, 0.51555253234624665, 2.8210931284044669, 3.807614182705656], [1.5619169593300066, -0.66994196054524924, -2.2758636407029984, 3.7503361822426911, 2.6164769125677481], [-2.0736113062223915, 3.750322107459457, -3.0484115373212628, -3.5757640130466162, 1.5007695695506609]], [[0.38263684218792537, 1.2818723989266099, 4.3290147018889975, 1.481455813448024, 2.9832068689922036], [4.9747316107933042, 1.3603994747166146, 0.59986147221726505, -1.2527306198738861, -4.7655795736643078], [-3.8169974076200885, -3.2200977840506964, -2.9340342150854659, -0.91449947673906173, -2.9642574384261589], [3.925900280680084, -2.5163220180447423, -2.3708189498644519, 0.55510393113406842, 1.9445674722548469]]])
      arg1=Data(numpy.array([-0.55086428586918945, -4.0230422817600751]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-1.1990274487574966, -2.6723133020105858, 1.4025525120431492, -1.9766521252324458, 1.6403489836103322], [-2.1591873420556862, -0.51607609156603951, -0.28399947755896737, -1.5540394515490041, -2.0974786676215484], [-0.86040427038829992, 0.36904709966956339, 1.2536919991715108, -2.0659262628005024, -1.4413236859348542], [1.1422784113724749, -2.0659185095050874, 1.6792610445418754, 1.9697606894836712, -0.82672035718473569]], [[-1.5393641946811811, -5.1570268607029695, -17.415809184060425, -5.9599593760606666, -12.001567369192724], [-20.013555610629869, -5.4729446068691363, -2.4132680659289036, 5.0397882514081518, 19.172128121943661], [15.355941960224213, 12.954589536637874, 11.803743703419563, 3.6790700615687095, 11.925333008810249], [-15.794062823149725, 10.123269873117836, 9.5379048777027098, -2.2332065857235901, -7.8230771606165606]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_constData_rank3(self):
      arg0=numpy.array([1.0185938701503625, 0.71908423218788986])
      arg1=Data(numpy.array([[[0.45350136752361259, 0.023078221172157853, -2.2232653578106589, -0.92245516227123492, 4.3814163556749435], [2.5683377249848007, 4.4157464243709903, -1.6635815222587693, -0.04332887641457539, -2.2480152787871455], [-3.617095730498523, 3.6624096067769241, 4.5854905080435611, -1.023531019273892, -4.8274682067356354], [-2.1329462740332494, 0.26202878699001975, -0.52076604780264102, -3.8367113161120772, -4.8050878386914349]], [[-1.6959775206650662, -2.8790228471303592, 3.781787547706843, -3.5349898517376022, -2.3610984409784286], [-3.1760002594588266, -4.7955719109409403, -3.2311799552307532, 3.4359843082876669, -0.97617077780943884], [1.3013323744212082, 3.9756486352577234, 1.9657953092337044, 4.1374133768656733, 3.2053727876258122], [-2.8385864293691809, 4.028185852816792, -1.256513631968442, 3.3406833724029834, 3.1034053874715397]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[0.46193371306435849, 0.023507334619934303, -2.2646044651835897, -0.93960717377803782, 4.4628838424670381], [2.6160930631454455, 4.4978522400026719, -1.6945139410681913, -0.04413452791638911, -2.2898145829769447], [-3.6843515388328432, 3.7305079754427743, 4.6707525231258424, -1.0425624221411389, -4.9172295237266814], [-2.1726060000903233, 0.26690091623096918, -0.53044910407420076, -3.9080506281282914, -4.8944330180251496]], [[-1.2195506932553604, -2.070259933480127, 2.7194237950404982, -2.5419554633287165, -1.697828659550997], [-2.2838117080014895, -3.4484201454807777, -2.3234905571680065, 2.4707621381346749, -0.70194901424535561], [0.93576759128191811, 2.8588262463331322, 1.4135724105788738, 2.9751487213473573, 2.3049330298658632], [-2.0411827430619014, 2.8966049310828832, -0.90353914027764393, 2.4022327378272497, 2.2316098802177331]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_constData_rank1(self):
      arg0=numpy.array([[[[3.3349037951520977, -2.468527002924902], [4.7832775784854373, -2.7597141752452004]], [[1.887855618503675, -0.58614665709245095], [0.45384135374649404, -4.6463362714946079]], [[3.3024824713885472, 0.29343300163249886], [-1.8663048477109956, -3.3806729381795595]], [[-4.0253640314153136, -4.3549190031705294], [-1.6716674932547839, 4.9519204047930803]], [[0.27381875411783696, 1.4152536152856676], [-2.0304076086030145, 3.3987030898704855]], [[-2.7270334959844389, 3.8551634249205549], [-3.7336638691664223, 0.99077438852874522]]], [[[2.5586827269760484, 3.7927913378381319], [1.6380714368303213, 1.6073231368712459]], [[2.7633092701331963, 1.6652732797370193], [0.35051097945703091, 3.6408761691288731]], [[1.8986146375204891, 0.42319525013971049], [-3.8837500575917785, 1.0359900387562471]], [[-4.4290919168090772, -2.8964993049594598], [2.4563659953665216, -0.63395185104336704]], [[4.7454079242838532, -1.9479767909395695], [-3.1544317291313915, -4.8604816132894682]], [[4.2213952481311612, 2.0193443681181904], [-1.8832847821406364, -2.5091795930841942]]]])
      arg1=Data(numpy.array([-3.3372214725676597, 4.5769151682572087]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-11.12931255412896, 8.2380213197740737], [-15.962856644173041, 9.2097774037776325]], [[-6.300192307177964, 1.9561012101226802], [-1.514569110861975, 15.505853174001764]], [[-11.021115416296171, -0.97925091380795637], [6.2282726121382508, 11.282054321021226]], [[13.433531280540704, 14.533329208673639], [5.578724653483218, -16.525655105321405]], [[-0.91379382583376978, -4.7230147540603395], [6.7759198694947322, -11.342224930397837]], [[9.1007147392105221, -12.865534161902357], [12.460063235532234, -3.3064335638682216]]], [[[11.710873783854394, 17.359284204185897], [7.4973140059175778, 7.356581645436763]], [[12.647432113058382, 7.6218145333217935], [1.6042590185175756, 16.663981364232139]], [[8.6897981331426895, 1.9369287594988445], [-17.77559454831162, 4.7416385225468405]], [[-20.27157797564886, -13.257031603715415], [11.242578782984248, -2.901543842985121]], [[21.719329508222724, -8.915724521864318], [-14.43756642829328, -22.246012020899837]], [[19.320967942380413, 9.2423678683749149], [-8.6196346855274513, -11.484302139468499]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_constData_rank4(self):
      arg0=numpy.array([-4.3583895514854998, 2.3042591505894308])
      arg1=Data(numpy.array([[[[-1.494487010330011, 4.3822460593536068], [-0.58549984516639597, -1.8080284871037486]], [[3.3928864285275431, 0.90819800392468419], [4.0534855191007217, -4.125216297758115]], [[4.2324238208347253, -3.7781544326406848], [-0.83225164217994507, -0.23486672144978726]], [[0.029790327271348005, 0.37445375897867272], [-3.1925675904012421, 3.5475335498259284]], [[-4.7755716861599957, -2.9426144693689316], [2.0800637024774229, 0.51905623532682466]], [[3.0475366496424137, 3.5993906155269002], [-1.0267728450530456, -3.2380839254012885]]], [[[1.5930513960218837, -3.0178232625532408], [-3.085887811581455, -1.9439596004839919]], [[3.4459086294449399, -4.5065004220939873], [4.2485104181774105, 3.400326382330638]], [[0.1942774694294318, -0.32834418520406583], [-2.1752536795914592, 4.2759692295822891]], [[-2.8920232037756808, 0.21562891367008952], [-0.48542811010296116, 4.2759880695795154]], [[2.7805747170159547, -2.0599098784071401], [0.55339756336863299, -0.92894902990656636]], [[-1.1258150825678728, -1.6749525887591989], [-4.7879051587400392, -3.3822513500932749]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[6.5135565706531215, -19.099535437125265], [2.5518364075695983, 7.8800924669811137]], [[-14.787520759471398, -3.9582806909853305], [-17.666668933546362, 17.979299609766667]], [[-18.446551758184405, 16.466668803119788], [3.6272768614837214, 1.0236406647584082]], [[-0.12983785111477669, -1.6320153506471169], [13.914453228416011, -15.46153315710559]], [[20.813801739329715, 12.8250601573476], [-9.0657279073018433, -2.2622492726818315]], [[-13.282351891570622, -15.687546450427403], [4.4750760396282336, 14.112831147302128]]], [[[3.6708032566426927, -6.9538468675999558], [-7.1106852275289611, -4.4793866977914121]], [[7.9402664914935865, -10.384144834745202], [9.7896690074598265, 7.8352331814760277]], [[0.44766563668612663, -0.75659009329929949], [-5.0123481960519491, 9.8529412249038284]], [[-6.6639709310170741, 0.49686489745596218], [-1.118552164658082, 9.8529846371398335]], [[6.407164735581631, -4.7465661867092139], [1.2751713993060669, -2.1405393025933801]], [[-2.5941697058786164, -3.8595248294518401], [-11.032574274181076, -7.793583623045885]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-3.4128790124745167, 3.8993470962158092, -4.2940091638008369, -2.0434123668724178, 0.6990377141662858], [-1.8463830639450132, 2.0895131781526359, 1.8551667914375578, 1.3205770402985735, -3.2035795438048509], [-2.8452001644303895, 0.37843588344059231, 3.8676931677508577, -1.276366079289164, -1.7579021754744515], [-2.038340239826737, 1.2329064012182389, -2.9282788658013317, -2.9985381194573026, 4.4052008815849657]])
      arg1=Data(numpy.array([[-0.19953293442819575, -4.5189076937472841, 1.0733098964636749, 3.1100246072429165, 4.3739180125727373], [-3.5118922470743619, 1.9213679056708877, 1.9482880745639006, -4.6185082918826126, 1.6461396085204498], [-0.59702739300960062, -4.3913234099521929, -3.96540842263152, 4.65103192445528, 2.860293432674248], [-2.6470458669709496, 4.117210081550148, -2.2646054991756595, 3.8743595705539047, -0.59823995392458507]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[0.68098176420744327, -17.620789593680751, -4.6088025310131471, -6.3550627437177098, 3.0575336494595899], [6.4842983673978978, 4.0147235589788508, 3.614399336084769, -6.0990960106887604, -5.2735391761030384], [1.6986624367603624, -1.6618343541186125, -15.336983063553635, -5.9364193820657212, -5.0281160477933469], [5.395580107313938, 5.0761346647034449, 6.6313964226135589, -11.617414860790108, -2.6353671724279315]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_constData_rank2(self):
      arg0=numpy.array([[[-3.6830106912919214, -1.3066522502717195], [-3.8159929674253656, 1.1103185046029873], [1.6009173101322416, -4.9359955056327092], [0.99391409443967405, -0.85516368670184484], [2.2228114663624785, 2.5403634353168272]], [[-0.66139486766955269, 1.1808161762960694], [4.6140636758073192, -1.7849347578815156], [1.4970060229442028, 1.1672253009950504], [-4.5536275164762072, 0.13866251862801882], [-4.1667292018072235, 3.3446274396771685]], [[1.7231641155489292, -1.0184304449623047], [-0.54036190161084008, 2.3745142236854422], [-4.4570404366570395, 1.8768363891814595], [1.3870592725851303, 4.2410300121804685], [-1.2622865849452856, -0.94723250836899631]], [[-0.92010803017032572, -4.7380823871285287], [-0.94864420526428095, 1.709668152270619], [-4.2985226103350236, -2.9008918823997543], [3.5368428677229744, 1.1668273516880925], [-1.8465407383013179, -0.92816525648325943]]])
      arg1=Data(numpy.array([[-4.9458059938789569, 0.7704692282631953, -0.76879866880732006, -3.1269313627053084, -4.9666840231996154], [-1.4318676197232239, 4.0099660643709569, -0.86764211912083589, -1.8676659315549324, -2.4282696499076897], [-2.9847771607574902, 1.6730128790344541, 4.1750556043071505, 3.5675322649088841, -3.501683184422447], [4.7065322270692818, 0.14499312259976094, 3.064628527137204, -0.97543584145306728, -2.8450940837674166]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[18.215456352511865, 6.4624485313092972], [-2.9401051566700018, 0.85546624136780869], [-1.2307830969002629, 3.7947867739693417], [-3.1079011537382626, 2.674038152194695], [-11.040002196567432, -12.617182487308575]], [[0.94702989486715905, -1.6907724476837316], [18.502238758834068, -7.1575278062210677], [-1.2988654780839628, -1.012733833646801], [8.5046549775137095, -0.25897526202515192], [10.117942060132574, -8.1216573020165299]], [[-5.143260896327325, 3.0397879319435752], [-0.90403242073448398, 3.9725928776762434], [-18.608391653688564, 7.8358962850196487], [4.9483787082884989, 15.130011404900738], [4.4201277084249435, 3.3169081462940091]], [[-4.3305180963818728, -22.299937449529772], [-0.13754688555743666, 0.2478901240070806], [-13.173375016176992, -8.8901560169430294], [-3.4499632987646391, -1.1381652196243286], [5.2535821299765972, 2.6407174799789881]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_constData_rank3(self):
      arg0=numpy.array([[0.10871893184145076, -4.2142375969665693, 4.308454060506401, -4.3150476062305962, 4.9382386970847385], [-0.97509160388221261, -3.1960768145989951, -3.1456538679335475, -3.8668175508083946, 3.5478211473106356], [-1.0892749387089671, 4.2521170181536387, -3.1875017631843416, -0.026230179626233863, -3.6944760038757538], [1.5817035327062232, -3.4718200266255872, -2.9578129479433213, -4.4388985218643455, -2.6441168367673451]])
      arg1=Data(numpy.array([[[-1.4878281638340796, 4.5782412746772021], [2.6866274502201826, 3.2407707674246282], [4.1696737553970173, -3.9856198876729652], [-4.0593720846855881, -2.4858492891140784], [-0.29289924569429715, 3.5085529127179989]], [[-2.3704377543452759, 4.9216191178390183], [-4.2678087008265209, 3.1512266661807029], [4.2613195207700851, 2.0472422784566389], [-2.3230169135366463, 1.1182408539375768], [4.8973643286158204, 0.55138802746821192]], [[-3.698261229662565, 2.7723381633303967], [2.9825321856637235, 3.1545940051617745], [1.7803387032642153, -4.4020614938532709], [0.51860549354544538, 0.51965433636169145], [3.4618996784885567, -2.2827374322493399]], [[-3.8281908081710947, 0.11614684896916661], [0.23089722623211983, 0.30951991437105164], [-0.44057227768879326, 0.37057014305430069], [3.3654219307301911, 2.276943511321317], [2.9652777800326433, 0.27036976786620404]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-0.16175508873566813, 0.49774150109534737], [-11.322086409760324, -13.657378011231069], [17.964847822427252, -17.171860188679652], [17.516383796821852, 10.726558024441733], [-1.4464063894345087, 17.326071764353394]], [[2.3113939517874855, -4.7990294793110087], [13.640244437855502, -10.071562485326231], [-13.40463623301115, -6.4399155918242146], [8.982682572088251, -4.324033360036788], [17.37497273114796, 1.9562261042256199]], [[4.0284232742704393, -3.0198384829422484], [12.682075863851686, 13.413702854713829], [-5.6748327557200104, 14.031578773303199], [-0.013603115250848698, -0.013630626586318518], [-12.78990529000116, 8.4335186665941411]], [[-6.0550629251577117, 0.18370988132722699], [-0.80163361412497247, -1.074597437352854], [1.3031303874527931, -1.0960771672472194], [-14.938766433768098, -10.107121186772806], [-7.8405409038764082, -0.71488925536790882]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[-2.6527180807921868, 0.13425014473676988, 4.2079691770219174, -2.1351056589077277, 1.7330624317267826], [-2.5169002946067378, -2.9132849262797, -2.3810333413570972, 3.4544430296326194, -4.1108552273397834], [3.5762532515758423, 4.1584348947768479, 3.4236030623710771, -2.6002561296215476, 3.2442041402562154], [-2.0040726589437341, -1.9168644599755069, -0.67547125198471836, 0.84104607248263541, -2.8363875279188333]], [[-4.5059360044776042, -2.5500105648100146, -0.23492391683037628, 0.66745084144255351, -0.9126993462512214], [0.2751797195925505, 2.2768149210391533, -2.1766669275342321, 2.4230455744378769, 4.5775773781274669], [1.4759444983567169, 1.6052428197987858, -1.1745926132779561, -4.1814063806682444, -3.365507383015375], [3.4066868572635229, 0.46658987966384924, -1.4788815166801927, -0.51639743605644828, -0.0032498532864781993]], [[-0.38450511336414372, 4.899462510322099, -4.7330136154112186, 4.7697349241263449, -2.0124998744012448], [-4.7982388316456879, -2.8386765580035944, 2.4300502934610249, 3.5635664250047547, 4.7345937673454017], [-2.1009590030744696, -3.6228217328300105, -2.2561234751851944, 4.4844010449723548, 4.7784422932304569], [4.9807083779159349, 4.2625929367457598, 1.4739228279487184, -2.6036116045097089, 0.72132141188831866]], [[4.6846302586033826, -4.3455551811381623, -1.0869329010850604, -2.5047810510676061, 4.0351611802631044], [3.8274499937790534, 3.3275848603201812, -2.4549915088109406, -1.9735001861096291, 2.4599402176580742], [0.01449110683580912, 3.6308698095853824, 2.7662753851562663, -0.8472610119489099, 3.9399527336898021], [3.876631597570503, -2.2934936747183543, 0.66649386635777041, 1.2853088446525565, 4.4883134356320031]], [[-2.4509619048654119, 2.4448787896443127, 3.8535789332744912, 3.8819285966356123, -0.50097071803798876], [2.4376282443138422, 2.8694547919320019, 0.34150326420630872, 3.7086164570070963, -1.7878068592104834], [0.78669574095905581, -4.5784215624564659, 1.1840697929067918, -0.91303008142776854, -1.5369392503345392], [-2.1496704595518179, 2.3609565729217552, -0.85644694884299621, 1.1594183219691629, 2.993815790289009]]], [[[3.350272341374577, 4.4791098207373157, 1.2524506107192899, 0.95488927770854914, 3.2132559868085604], [1.0928840702005784, 4.6271315564599504, 4.0504834680871173, 4.5470115187975662, -2.2956136872933941], [3.5931659503029696, 4.5230339539030808, 4.9383665866773399, 1.4618570808026812, 2.2430888127339159], [-3.2121807135414651, 1.4420834808286207, 4.5526294691305864, -3.1215230807140459, 0.72965651483005267]], [[-3.5653387876509246, 1.0181850574643612, 0.88337530786136753, -3.9055462490168722, -1.038807213091264], [0.57681583104043721, 4.0766727699271659, 1.832578050051346, -1.019809733577465, -4.0475948269372273], [4.7508843890383652, 1.5668164459238989, -1.041263227941621, 4.8973068634946646, -4.1464302203511778], [1.3885415546920141, -3.2346838257662567, 0.70232302333373298, 4.2261466834166175, -2.0815662691863182]], [[-4.9758460969273743, 3.2924883857697154, 4.0525877680614393, 4.3363798361754, 0.010799791399612957], [-0.79637545687595601, -1.0044145123863091, 0.84606590534192971, -0.20724788821488183, -4.1729444319339768], [-3.1974052582692303, -1.4492912284121839, -0.67694255851062035, 3.2643601869284815, 4.3174644040791783], [1.8887101318467581, -0.087538996598967245, 1.5108131527697548, 4.9853254742486524, -1.5975142011609966]], [[-3.007163592404678, -0.89319093445548958, -0.78781273006662733, 2.2470106333238142, 2.3475941861737546], [-4.7663384314088217, -0.94948134936875306, 0.62675048209918138, -1.434881730222993, 3.5178652792527654], [2.6090502878998851, 2.8901731033973217, 2.2936150351180453, -2.4612760243500489, 0.92335210910863719], [2.2581782191179762, 1.2686414766735048, -0.9467674174949261, 2.0435172581077952, -3.814109235696308]], [[-2.3014331064328983, -4.4325215817980732, -1.4192423985560678, 0.1086025153050576, -4.789405093570883], [1.7019318175408751, 4.5819057132649252, 3.7862108314899707, -3.142642432023762, -3.2443858471992137], [-0.15367409318201108, -2.5680116056441116, -2.3745347903662504, 0.16406380170927459, 2.6645078439704859], [1.7308165735555505, 4.2977084418093341, 2.254014573744124, -4.7783889772423471, 1.2712378467890666]]], [[[0.29313670361541178, -2.1115858302329804, 4.5300463604731167, 0.12051489835088702, 4.3746080427226506], [-3.242280196080177, 4.6671735649950197, 1.6771001851269363, 0.69383176389371659, 4.6553734617624656], [-2.8227028797178688, -0.10073831706571479, 0.82890036615776452, 0.76588633188340083, -4.2348411360267937], [-4.8225622568697588, -2.6893040591448147, -1.9574282088695414, 3.9256886985214585, -1.3669381226971566]], [[-1.8741321743854957, 0.22160859103707242, -3.5736774722303521, -4.0566726833181024, 2.5543968299502309], [-1.7048010496625721, 2.3121347460314272, 3.9415158691537346, -0.29574814300065722, 4.6121317035425715], [2.1945359129527811, 4.0975343186135529, -4.2714891758903386, -3.9147126935846801, -2.5750707822597274], [-2.2394055544741409, -0.20325748951610301, -2.9753560972213933, -4.7729121114922712, 3.8674835351830552]], [[-2.3881652124130537, 0.87144474543696759, 1.3283670744911023, 3.4099091348654795, 4.2655467209227744], [-2.3487240594623291, 0.00047243810453245771, -2.4791602561199086, 4.0040990462877879, -1.2910009923304369], [3.6889092859953703, -0.47919425399422089, -2.2478351513572639, -1.4479382664594018, 3.9410611712737076], [4.2683410807028235, -3.1761930578809494, -3.3495441853986585, -1.1689573003864995, 0.47983947723625064]], [[4.9248396940598358, 0.52693875335051565, 0.18750498299256613, -1.5431719338432757, -3.6919189866014457], [4.0247585254247706, -4.1649369856001375, 2.2852862184228684, 2.8367778090775104, 2.9723168026290931], [1.9664172313434669, 4.8533739371080795, 2.0438226614157662, 2.6102438974527029, -1.4837921692349854], [-4.6892878308214945, 1.1941135350395147, 0.075209983693030225, 0.41104412414580604, 4.440891122818158]], [[-2.8320590790644817, -0.15550035123095629, 0.82970617863314899, 0.59717985771857141, 1.6733382966997992], [0.63116151444181501, 0.62819072206565352, 4.450643119946502, 0.072627609368995039, 1.6122877985443935], [1.2854022963885452, -2.2683556989187768, 1.5588506894930898, -0.45309553697873106, 0.06017148259612437], [-3.0569558253087314, 0.11540725324446122, -0.74430355416408034, -0.99747447666206934, 0.76930285980379676]]], [[[4.8905847221773442, -3.8775608747043422, 0.33055779979149325, -2.0855043953013164, 0.41182638962458196], [-0.92243962752390729, 3.2879802283058766, -1.0736094967762631, -3.882376786606887, 4.4192021403328283], [-1.2409805830090823, -3.2066786475852513, 2.2600952890817965, -0.097632980096708621, 3.4581725507592296], [-1.2535949401359234, 0.79997216015610384, -1.0378793852813137, 1.5256515351308515, 3.9502060623325974]], [[4.3969796463028334, 4.7183599204992746, -4.6200483480004504, 3.1935213200040149, -0.60025591413124069], [0.18854893484487434, 4.8374428280115431, 2.689524907238412, -2.2540605903663913, 0.4925203818330619], [-0.69080619508487739, -4.0742316922972392, -2.8027882512892468, 2.268876158243299, -4.5826145201688382], [-0.56211255458518661, -3.2681079642021804, -4.2608387722199641, -1.0187158465340094, 1.1981371284058095]], [[-2.8016611144695402, 0.60103866522312543, -0.93934060766301197, 4.2166992984004619, -3.5958262458135337], [-4.9499797649948611, 0.57130979351793254, 4.6561234010852068, 4.1366411805156904, -1.202453368659786], [-4.5227258196793363, 2.6534902576199535, 0.55435277991915655, -4.9827767391021496, -4.2902066920735891], [0.071287079867041747, 3.4234500147502107, 2.7169904006273962, 1.6028387755708673, -0.94536704711020469]], [[4.3551454291968277, -3.613965598759219, 0.26113996939838202, -4.2670635672664483, 0.4401360586362415], [-2.5370826691640347, -0.13671449131544477, 4.0239587313852692, -4.258632186630904, 4.392320997312062], [0.75486935172626346, 4.172835303110789, 4.580877661305788, -4.8230578765210232, -3.3827730350579008], [2.5115836119837818, -1.8628133494316201, -4.6569383332869707, 1.9873704655190885, 4.4227140145961457]], [[-3.349963088499913, 4.6530533984416529, 4.5888398514958624, -4.1640790717092866, 2.1140723054983948], [3.1572773056494281, -2.3636470497496664, -2.0075408610274148, -0.6744881001646803, -0.18376382126125446], [3.1999151230518006, 0.43244722179512252, -4.2182633630665434, 1.011394812172064, 2.8151792211075932], [2.4163825094127009, 0.36666478769552402, 3.1991769727474608, 3.9406596836467909, 4.3060310562082673]]]])
      arg1=Data(numpy.array([[-0.068360461376269832, -2.194754957844669, -4.0912196481818528, 4.3580398113020351, 2.1259357358320869], [-4.0449854212925977, 0.40922487602616187, -0.58101220336206083, -3.088914708342354, -2.2143346903081831], [-3.4064654546998407, 4.771899923289622, 0.53387057475655642, -1.8452463899343607, 4.4606025613508713], [3.2364216769411556, 2.203976005112219, -0.57330102529330329, 1.8932735679685599, 4.4821099141331473]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[0.18134103190412693, -0.0091774018340365915, -0.28765871439834073, 0.14595680793001686, -0.118472947426723], [0.17205646537738606, 0.19915350168101253, 0.16276853776745254, -0.23614731930372523, 0.28101995999199819], [-0.24447432227610977, -0.28427252801012542, -0.23403908491289713, 0.17775470871740268, -0.22177529182671968], [0.13699933159696151, 0.13103773887969997, 0.046175526432081969, -0.057494297553612635, 0.19389676005042888]], [[9.8894253855580203, 5.596648329673064, 0.51560043117975707, -1.4648910433736404, 2.0031514152064562], [-0.60395205387405604, -4.9970508360454007, 4.777250530782279, -5.3179912875811146, -10.046660645562859], [-3.2393365052719676, -3.5231146372979416, 2.5779429614395202, 9.177162384734963, 7.3864640145358313], [-7.4768428698033915, -1.0240504516723805, 3.2457825407986967, 1.1333658330031653, 0.0071326316127658194]], [[1.5730948746217754, -20.044777287760155, 19.363798298482603, -19.514033238204881, 8.2335790281138834], [19.630648984697977, 11.613649308937539, -9.941869506677822, -14.579332975580616, -19.370263047122847], [8.5954847534028271, 14.821759455214366, 9.2302966904019907, -18.346669665518132, -19.549656997767595], [-20.377171977593637, -17.439203975015438, -6.0301420336075573, 10.651946952604399, -2.9510843329717642]], [[20.415805168223688, -18.938102481609938, -4.7368968551427102, -10.915935539147583, 17.585393068607118], [16.68017944865684, 14.501747296761271, -10.69895073180653, -8.6005923786777387, 10.720517401976881], [0.063152820500287213, 15.823475179827737, 12.055538257535879, -3.6923972206373987, 17.170470868068442], [16.894514835963662, -9.9951367413919883, 2.9046068035757817, 5.6014271148144639, 19.560248638086083]], [[-5.2105875007164633, 5.1976551886827442, 8.1924611650979333, 8.2527307275361519, -1.0650315520824205], [5.1822409952604263, 6.1002764845228681, 0.72601399327949856, 7.8842802564463685, -3.8007624907612914], [1.6724645889317591, -9.7334300133303806, 2.5172562864598471, -1.9410432779969733, -3.2674340760891747], [-4.5700612502237945, 5.0192419491220139, -1.8207511743896809, 2.464848843452716, 6.3646599750737849]]], [[[-13.551802778219981, -18.117933925250941, -5.0661444612485385, -3.8625132072796999, -12.997573621521786], [-4.4207001311242555, -18.716679688283428, -16.384146577599072, -18.392595303985669, 9.2857238980215229], [-14.534303885260474, -18.295606403549378, -19.975620848108328, -5.9131905798602, -9.073261546173212], [12.993224156832481, -5.8332066562386542, -18.41531983118028, 12.626515353716673, -2.9514499650387291]], [[-1.459025323367716, 0.41666665391254371, 0.36149915084414069, -1.598246679568371, -0.4251057529923552], [0.23604738694745045, 1.6682759088726744, 0.74993652534052757, -0.41733151179351125, -1.656376491257521], [1.9441800751188527, 0.64118026583895904, -0.42611081536501094, 2.0040997940756755, -1.6968223928743418], [0.56822574557601357, -1.3237130875830274, 0.28740805215406606, 1.7294443525895404, -0.85182869844801135]], [[2.8910273043662844, -1.9129759315600572, -2.354602948439513, -2.5194896032310816, -0.0062748105969397585], [0.46270385890296706, 0.58357708893039939, -0.49157461585223139, 0.12041355217386257, 2.4245316389054032], [1.8577314741484448, 0.84205588993307079, 0.39331188746980633, -1.8966331047747058, -2.5084995063513102], [-1.0973636352165332, 0.05086122529406991, -0.87780087875913704, -2.8965349382702206, 0.92817524591873313]], [[9.2888718509704411, 2.7589906147976131, 2.4334863293221498, -6.9408241950755976, -7.2515182108911089], [14.722812885716134, 2.9328669053618865, -1.9359787826168227, 4.4322272812175285, -10.866385803050749], [-8.0591338090988085, -8.9274982087394541, -7.0847812172512947, 7.6026717129052601, -2.8521559108046035], [-6.9753199150918599, -3.9187253169099523, 2.9244838012793832, -6.3122505153206072, 11.781458117366741]], [[5.0961431649980913, 9.8150863041151748, 3.1426776770788933, -0.24048231709471443, 10.605345844632716], [-3.7686466641200167, -10.145872768603782, -8.3839379889888335, 6.9588621564646926, 7.1841561301981232], [0.3402858755345794, 5.6864371834917735, 5.2580147596516573, -0.3632921675486897, -5.9001121515021104], [-3.8326071813844007, -9.516564891528736, -4.9911426631018259, 10.580952476093968, -2.8149460637777093]]], [[[-0.99856005437048612, 7.1930441853223304, -15.431446435140414, -0.41052983800895942, -14.901951175386793], [11.044715482404548, -15.898565520243837, -5.712983844705616, -2.3635139350774019, -15.858368876220249], [9.6154398486406798, 0.34316159704895688, -2.8236204627044739, -2.6089653317875823, 14.425840036017101], [16.427891731166135, 9.1610213746608693, 6.6679115735690768, -13.372722937418926, 4.6564274936801162]], [[-8.943171179184759, 1.0574940185701271, -17.053231255597868, -19.358036066336759, 12.18932603689076], [-8.1351399981088939, 11.033275617222637, 18.808519273659535, -1.4112805408978844, 22.00863092233643], [10.472105754675697, 19.553023700668607, -20.383118870763557, -18.680617202217643, -12.28798006833054], [-10.686219193609507, -0.96992439862993307, -14.198101532090076, -22.775858938698075, 18.455244384863896]], [[-1.2749711345645707, 0.46523870711501486, 0.70917609354625011, 1.8204501496982657, 2.277249879549986], [-1.253914663569706, 0.0002522208024036413, -1.3235507108483473, 2.1376706592238408, -0.68922744178673501], [1.9694001207391465, -0.25582771179993397, -1.2000530442130934, -0.77301163452669275, 2.1040165926586418], [2.2787417060118376, -1.695676013348687, -1.7882230794312635, -0.62407190582321304, 0.2561721775030027]], [[-9.0875426664693535, -0.97233183233655152, -0.34599289296173635, 2.8475324399723307, 6.812500181956441], [-7.426671139397599, 7.6853349369827519, -4.2169161445115453, -5.234554011246181, -5.4846568497925761], [-3.628524297241253, -8.9556707364501982, -3.7713563876434799, -4.8165431286227962, 2.7379621436937307], [8.6528914411864921, -2.2034336897034223, -0.13878095089658615, -0.75847768618377975, -8.1945383124717566]], [[-12.632689981972018, -0.69362526499176369, 3.7009895055796678, 2.6637820029266086, 7.4640970922656287], [2.8153606679452552, 2.8021091438629075, 19.852550100492, 0.32396290037612979, 7.191775083821879], [5.7336687756370361, -10.118233240651941, 6.9534133783164478, -2.0210791127839762, 0.26840106938855174], [-13.635864984308594, 0.51478588942071235, -3.3200423401268537, -4.4493372054809468, 3.4315543068953662]]], [[[15.827994407771996, -12.54942206855204, 1.0698244287271634, -6.7495716323092365, 1.3328438545174113], [-2.9854036061878988, 10.641290484243068, -3.4746530479365836, -12.565008390227677, 14.302401601757916], [-4.016336459513667, -10.378164286029255, 7.3146213855369133, -0.3159814931693522, 11.19210460588006], [-4.0571618383596526, 2.5890472400786564, -3.3590153405748051, 4.9376516997560387, 12.784532528517584]], [[9.6908376354182568, 10.399152048263598, -10.18247570145134, 7.0384443611031493, -1.3229496316719551], [0.41555732818757024, 10.661607919039636, 5.9276483607051267, -4.9678954552366088, 1.0855031035887766], [-1.5225202781499403, -8.9795088890908641, -6.1772780532519365, 5.0005486113394255, -10.099972443130964], [-1.2388825824780838, -7.2028315352177481, -9.3907864156246088, -2.2452252817885383, 2.640665481840462]], [[1.6061951894497661, -0.34457608301333631, 0.53852493347283936, -2.4174380311265375, 2.0614908735014685], [2.8378284744526585, -0.32753249038393611, -2.6693603197342917, -2.3715406300601458, 0.68936774912004173], [2.5928833495426589, -1.5212486852993108, -0.31781101710184534, 2.8566310133348849, 2.4595798952859798], [-0.04086895597794063, -1.96266740349667, -1.5576533823917491, -0.9189091134146411, 0.54197989738678287]], [[8.2454817257574433, -6.8422255436784996, 0.4944094016020753, -8.0787186647471998, 0.83329796612585627], [-4.8033915572793893, -0.25883793276579881, 7.6184547047280287, -8.0627557546484425, 8.3158652462442308], [1.4291741908928965, 7.9003187828657309, 8.6728545942478821, -9.1313679943998238, -6.4045147737119068], [4.7551148663118976, -3.5268152765378673, -8.8168582540717821, 3.7626359721286624, 8.3734075425189989]], [[-15.014902770945557, 20.855496768146267, 20.567684592758884, -18.663860090542546, 9.4755044396686756], [14.151263913318893, -10.594125875194544, -8.9980187962383713, -3.0231298007129448, -0.82364964513406025], [14.342371297415065, 1.9382759801472547, -18.906720040025185, 4.5331827147392403, 12.617942696987976], [10.8304920017766, 1.6434318800936338, 14.339062826617864, 17.662469836298072, 19.300104487596304]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_constData_rank4(self):
      arg0=numpy.array([[-2.4773558530257, 3.4104367364575445, 0.3961449104860133, -0.5272701029953506, 2.7465725457597587], [-3.5268785762149046, -3.9403575468036345, 4.0392340505563009, 1.9726910189069198, -1.5713942077791088], [-2.442034844665665, 1.5152394301569387, 4.976013508409304, -2.4599332870782908, 0.10512214135109588], [4.7848672925142051, -3.1950911498203127, 0.53679294879571327, -0.28325115322592431, 0.64151908519894008]])
      arg1=Data(numpy.array([[[[2.8258556675952384, 4.2135213169992358, 3.7715102531156415, 4.9275733922172229, 4.850879370081385], [4.193335819360227, -1.2645004575145902, 0.15289687688448073, -2.0740964982125965, -3.5736839543122203], [-0.56449677670868947, -4.1267558938805626, 4.7815414602036217, 0.15993662162826361, -2.3795060195186823], [2.3057655464628679, -4.6675676936129875, 0.24794471799235307, 0.62654207473012224, -1.8130202054984212]], [[-4.7754234087051568, -0.50588116660275606, -4.0266627283634282, -2.0949968755818826, -1.2039763186817911], [-1.2406756758316275, -1.1363572383246421, -3.3784272908410129, -4.3602944335603837, -1.831771641698603], [-1.2779589488803742, 3.7873258305065303, -1.7568383592587509, 1.4286782051751823, 4.1822767848287263], [-1.611678400520141, 3.2830762221387904, -3.968810976796775, -4.2000745227669576, 3.5170414143655151]], [[0.53163924908149607, 0.4407551740804907, -4.3496980905496709, -4.6044448672327665, -2.0934586441386918], [4.6124133950804431, 0.59719246278203553, -1.2580577247595248, -4.0736805865204397, 1.1055713371683584], [-3.968363879288066, 0.86847094138216541, -4.016177274539448, 2.4664416540131313, 0.69702387323713566], [-4.9976374061276658, -2.7773107381492643, 2.3796134910973468, 4.8801462839159431, 4.5037240098426441]], [[4.806331849723195, 2.6181622867363341, -4.5026653628618227, 0.66744105204330051, 1.0297442580580141], [0.51340626058920513, 4.207264685834982, -2.0668854694015772, -1.8359382344024535, -1.335982418428423], [3.7010591756993456, 1.7006521729330979, -4.7302268964065206, 1.9982413519939746, -0.80105136640030139], [-3.2638323060142813, 1.557584319513353, 3.7879943674360721, -1.6921028222852041, -4.0950570631756769]], [[4.2214670380708306, -0.63140097364606707, 2.4855170258696617, -0.33992706518935289, -2.2582071795950753], [-2.0179745502250834, -1.5056746629425213, -1.2695978498114391, 4.0172566908922249, 2.9904340583435483], [4.5728477811489814, 0.27597876902822538, -0.32971669248130908, 3.6685156108285799, 3.0111886950154183], [-3.1066022106358195, 1.0751687754935135, 0.12522873767966125, 1.0093498894512942, -0.43221935409594092]]], [[[-0.80873537985008426, -2.4037593920766174, 2.8899458976615575, -2.407859673972137, 4.2470624004730411], [1.6704139177757069, -4.408850193393766, 2.402421736388991, -3.8172428517426016, 2.2872301867348472], [1.0122884716438749, -1.6941893327980018, 0.88637365195694429, -1.6803763819785544, -2.2796712511361719], [4.3713829847848018, -1.4767937679291085, -4.5409945844345003, -2.4706422834475306, -3.863717778280451]], [[-3.2825221362820098, -4.1641342899876275, -1.1807017906247053, -3.2576647962893235, -4.039979966367687], [-1.0369310835627763, 2.9451669192249588, -0.61358182251916915, 0.94609287727692859, -2.2444176629796808], [-4.4176833769411141, 2.2317517593370271, 2.4834670598964692, -1.5534406859790306, 4.8234142094776331], [1.0153895917288054, -2.7348616300627904, -3.5661051572411839, -1.7530149435602338, -0.66246866038431573]], [[0.80231778217353078, 4.8618415236296624, -0.13253803899759031, -4.6014987264063834, -3.7528903205216881], [-1.1088702273297013, 2.5735669625176802, -0.48955991302318402, -3.8277083867756567, 3.7450611909860143], [4.4108707340593192, 1.7420484205269222, 4.5161760472737864, -3.7175708193922419, 0.17856394297340827], [-4.8880826053579893, -0.68769661602817322, 2.1650685770174576, -3.4594169315597743, 0.5650018085700248]], [[1.4462372262321317, -0.4275672700022346, -4.5560882488282086, 0.99134854869273425, 0.89115752665427905], [3.364823968244778, 1.2770963209962449, -2.7026681478179837, -2.9319609923955769, 2.6354612058443934], [-0.80734663345676605, -1.2123319552119902, -4.3621347341568848, 0.94301070583961355, 1.2435276003649776], [0.36413008583864048, -3.9490934747889259, 0.22766228368322849, -0.80702112905210832, -1.5485210754465681]], [[0.099718449141908749, 4.9061339254350074, -1.5241518223104897, -0.91017515158188367, 3.9664178841095499], [3.2060393054489555, -3.4700189159470183, -3.9091302632479676, 3.4500611182123144, -3.1961423104306084], [-4.5619114640517457, -4.4199531101880893, 2.1204248770887411, -2.8994653912063537, 1.3994400667650462], [-4.4487313536743684, 1.0587906897624464, 2.6949693432139608, 0.44475566083008378, 4.8004858408755773]]], [[[1.9265068426706078, 3.3768054060158139, -4.8044100907548204, 4.6754558814657106, 4.4150356015619625], [-3.9571950716171225, -0.28803701174810392, 0.86509147981093637, 0.24629877240911835, 4.0432208343557683], [-4.375867665452259, -2.7520041897418981, 3.5984333250882834, 3.2426337220603099, -2.2326317142682273], [-2.4272518871711659, 2.1291682902268452, -3.6613663653033579, 4.4542825652760207, -2.6306061086736179]], [[-0.71482188942867975, 2.8316899535916615, 0.73692098479556378, 2.2867739068893487, 0.43109777806850236], [4.904816837312481, -0.077041999466723965, 1.2652554902157043, -1.9393519500809542, 4.4121057544250224], [-2.38685396591793, 1.2532373328720512, -4.7522470377543682, 2.461789217572087, -1.9621056025084549], [-0.327226612650521, 4.0461816371428814, -2.6937168555632041, 2.0335372121937159, -0.90431292092630766]], [[3.5238402841655532, -1.6988182816280553, 1.8525849289018925, 2.5301854964707502, 0.61714742921606103], [-2.756725703673256, -1.1408875128434834, -4.6150718691819073, -0.10656789733263139, 1.4341488676100465], [0.54145624539797588, -2.1786357795911258, -2.5448154242099186, 4.7654093392551982, 2.7778305137995529], [1.2290111867089104, 0.7428568572383778, -2.691967766274753, -3.0712016804939202, 4.8422634134017368]], [[-1.462914095888932, -1.3125437434108624, -4.1440568894934966, 4.6699415067770911, -2.9732655914900832], [-0.90994322868912647, -3.5966320281392261, 3.7257082486304043, 4.115685541319964, -3.0211206054581172], [-4.427488432395414, -2.8131925465320573, -3.3180094423100037, 4.4034889420271863, -3.0081445112709337], [4.7970051160483234, 1.9191634644100848, 0.62883969866805245, -2.3159634771648765, -2.3224067378623339]], [[-1.6598950962512751, 0.99740456717550696, 0.27893516639072136, 2.2870120894937118, -4.0847912212066966], [-3.8292737048474734, -0.69037214631272903, -1.011091077810542, -2.3713499095731927, -1.6607153255579297], [-2.5705582896833192, -0.79587401977741745, 2.2770670208204766, -3.1942230985344988, 0.18025809805930493], [-1.0347485202894777, -3.9997081139789548, -0.62043568440473074, 0.31537996572787641, 1.177849054296165]]], [[[1.4981415552679502, 2.4230054330895658, 4.9799660397079251, -4.1400152011902698, 3.0081265240299828], [-3.8930296320331301, -1.5432269957666946, 3.5372636197162262, 0.51619131602842483, -3.620831089824057], [4.3136024331201916, -4.0947806393643607, 4.9746880465679766, -3.580797513301579, 4.2111043048852039], [-3.0516560727286555, 4.9304826652648632, -0.58280015011575692, -2.3710816034906665, 2.6790261168349172]], [[0.16468379423298352, 2.6167676600314227, 4.2241186730616036, -2.0348628773693345, 3.4161764588005994], [-4.1715468856127336, 4.0080598782736683, 2.3884243097911186, 4.420234060999201, 4.9369891779860033], [2.5808492542772221, 2.6946120835973808, 0.87213583884734458, -4.9020791067413896, 1.1189619204342618], [1.9340136418108251, 0.36359710610709417, 4.5424795305034582, 1.4590420687831331, 0.1477370067827497]], [[1.58944761013705, -3.6726692662859897, -4.0771677209755754, -2.973939336999857, 3.0845664412923703], [-2.442395535967794, -2.4080311596275781, -3.6437669348023913, 4.3957595159219931, 3.5313136527626234], [-1.219017195505737, 2.9923990524213178, 1.1624978762735338, 4.9288193841476851, 3.5528407438530447], [-2.3938017526421653, -4.85838600071793, -1.3127448465080596, -4.415409684006784, 0.92612796638359463]], [[4.012785220198289, 2.8110410401620154, 1.6638195774979732, -0.10829887483449241, -4.8492588052289261], [-2.8616256000116769, 2.3788728663494298, -0.65113790778587788, 2.5080467280898997, -1.9911416846450458], [0.36787779278403132, 4.197311015526509, -0.68904099868293933, -2.3144924040296546, 2.2411564492902167], [-1.1107051877459275, 0.95198525269184575, 3.2689762093865351, -1.2368989156207775, 2.0314720907490607]], [[-1.4608404317644719, -1.527996252444519, -1.8963387565955783, 3.5742414229871002, 4.8621552907679177], [2.3425674273825026, -2.9176152605192449, -3.8893953736379041, -4.3374022324307449, 3.0097150718056938], [4.6352282797486737, 2.6069700156373763, -4.9728749522917246, 1.5045722984531631, -3.6029123345753042], [-0.082952345715803233, -1.3358489425913667, -1.0979334563152809, -0.038914387313210952, 4.0749319930470076]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-7.0006500779229111, -10.438391696516613, -9.3433730003024742, -12.207352784423041, -12.01735439979274], [-10.388385035794379, 3.1326176095774456, -0.37877997285911819, 5.1382750995870845, 8.8532868610794075], [1.3984593937934136, 10.223442867713317, -11.845579722920494, -0.3962199257039356, 5.8948831647644937], [-5.7122017722347875, 11.563226145165801, -0.61424729834516245, -1.552167675999534, 4.4914962177453717]], [[-16.286279425187377, -1.7252757148640387, -13.732678494135001, -7.1448543072482282, -4.1060850670572959], [-4.2312459028854743, -3.8754744713218003, -11.521912544134928, -14.870508317985673, -6.2471412996500622], [-4.3583981469462971, 12.91643514529405, -5.9915860804338417, 4.8724166355056706, 14.263390389013434], [-5.4965272244890251, 11.19672375657238, -13.535378755323674, -14.324088448303822, 11.994647243194754]], [[0.21060618273824058, 0.17460291898236319, -1.7231107607219824, -1.8240273997677077, -0.82931298718849278], [1.8271840915186308, 0.2365747547117113, -0.4983731647610995, -1.6137678312957497, 0.43796645839846138], [-1.5720471537364995, 0.34404034333354161, -1.5909881869183904, 0.9770683082480065, 0.27612245987013939], [-1.9797886228919959, -1.1002175137559838, 0.94267177342206809, 1.9332451128005317, 1.7841273447328232]], [[-2.5342350894333832, -1.3804786985860096, 2.3741208296297511, -0.35192171225419622, -0.54295336100511993], [-0.27070377189932798, -2.2183648842289125, 1.0898069143309632, 0.96803534194648377, 0.70442358736473221], [-1.9514578527628814, -0.89670304638170129, 2.4941072228596437, -1.0536129234754317, 0.42237043646645323], [1.7209211961517028, -0.82126764457374868, -1.9972961802638256, 0.89219522938504303, 2.1592011594724769]], [[11.594565469595109, -1.7341885795822687, 6.8266528252720606, -0.93363434480976448, -6.2023298421134108], [-5.5425134976901118, -4.1354446920840076, -3.48704259844772, 11.033686936474282, 8.2134440845513268], [12.559658171642221, 0.75799571022549739, -0.90559081544787656, 10.075844260392868, 8.2704481998315025], [-8.5325083423289172, 2.9530290408286222, 0.34394981285110821, 2.7722526954325724, -1.187121811705927]]], [[[2.8523114850202851, 8.4777675022903853, -10.192488272882699, 8.4922286986641353, -14.978873392076213], [-5.891347060014346, 15.549479292821413, -8.4730497531033429, 13.46295203402047, -8.0667831444671485], [-3.5702185235901114, 5.9752000618970955, -3.1261322436083132, 5.9264834615776767, 8.0401236964451925], [-15.417336997467881, 5.208472301596859, 16.015536514549943, 8.7136553389817681, 13.626863456757972]], [[12.934310872248806, 16.408177975456542, 4.6523872112126217, 12.836364065015161, 15.918965549412409], [4.0858792206318562, -11.605010696764477, 2.4177317649449366, -3.7279442089553103, 8.8438080765013609], [17.407252033718883, -8.7938998874959431, -9.785748171701286, 6.1211117305092886, -19.005976581675078], [-4.0009980407144594, 10.776332663481606, 14.05172936903066, 6.9075056625171145, 2.6103633854662323]], [[3.2407493051221388, 19.638115830653458, -0.53535216011302567, -18.586530339292114, -15.158802370654353], [-4.4789863798782354, 10.395239306388165, -1.977447070470626, -15.461010051464161, 15.127178684047642], [17.816539261614668, 7.0365412979101665, 18.241892068455041, -15.016138639043632, 0.72126155865978425], [-19.744109701493947, -2.7777675879133392, 8.7452187180783909, -13.973394665027236, 2.2821745438019372]], [[2.8529791873969814, -0.84345811351195821, -8.9877543698107623, 1.9556243786125662, 1.7579784492622004], [6.6377580223592165, 2.519316442708361, -5.331529182286336, -5.7838531174841741, 5.198950651446836], [-1.5926454529648992, -2.3915563599805592, -8.6051440133332111, 1.860268750142881, 2.4530957290028645], [0.71831615004769189, -7.7903412305400348, 0.44910734236574423, -1.5920033333492163, -3.0547536181215298]], [[-0.15669699339031104, -7.7094704330171524, 2.3950433453546771, 1.4302439612602442, -6.2328060887212153], [-5.037951594494646, 5.4527676254030863, 6.142784653121879, -5.4214060576427459, 5.0223995138483959], [7.1685612510120276, 6.9454887160048209, -3.3320233698879762, 4.5562031213976519, -2.1990720150486029], [6.9907106811292161, -1.6637775571431557, -4.2348592160686866, -0.69888646930536347, -7.5434556448775067]]], [[[-4.7045968382884586, -8.2462764651460052, 11.732536849686602, -11.417626177236286, -10.781670779453748], [9.6636082520282542, 0.70339641924224328, -2.1125835375216901, -0.60147018442144529, -9.8736861621749696], [10.686021314680213, 6.7204901240156154, -8.7874995660717179, -7.9186245377591957, 5.4521644415486481], [5.9274336852524803, -5.1995031548911737, 8.9411842431576769, -10.877513232390807, 6.4240317799713278]], [[-1.0831263124016191, 4.2906882716613577, 1.1166117330723202, 3.465009991572773, 0.65321635158243996], [7.4319718695935224, -0.11673707537010999, 1.9171650079973819, -2.9385825437144129, 6.6853966091271211], [-3.6166552431853134, 1.8989546221124487, -7.2007920934519287, 3.7302000912004254, -2.973059775052648], [-0.49582666608476078, 6.1309339581758486, -4.0816259932277301, 3.0812957666073357, -1.3702505949879353]], [[17.534676855484673, -8.453342717713884, 9.2184876316913069, 12.590237209219755, 3.0709339444591945], [-13.717504340457266, -5.6770716754846671, -22.964659963328948, -0.53028329668994967, 7.1363441382974981], [2.6942935913129111, -10.840921069149276, -12.663035927276908, 23.712741245233723, 13.822522160738133], [6.1155762670496872, 3.6964657564326497, -13.395267969185591, -15.282341049187101, 24.09516815636319]], [[3.5986710806132263, 3.2287700451627277, 10.194103486011175, -11.487744561229515, 7.3140349998309793], [2.2383996376038757, 8.8474748473915863, -9.1649937387480929, -10.124311862239814, 7.4317551416445422], [10.891326173003559, 6.9202659881747515, 8.1620818739784546, -10.832289027773841, 7.3998348156172264], [-11.800312563252129, -4.72101408944686, -1.5469037069898242, 5.6971156491354629, 5.7129656406024614]], [[-0.17449172693611745, 0.10484930389485224, 0.029322261989116857, 0.24041560814342311, -0.42940200014540592], [-0.4025414516730107, -0.072573398349546139, -0.10628805920043168, -0.2492813803870616, -0.17457795119723188], [-0.27022259187932113, -0.083663981204706556, 0.23937016122860894, -0.33578357207107906, 0.018949117263869955], [-0.10877498021270778, -0.42045788172082077, -0.065221527715258021, 0.033153417336549598, 0.12381801477597607]]], [[[7.1684085273579772, 11.593759446374479, 23.828476621229946, -19.80942332668694, 14.393486216575511], [-18.627630155103937, -7.3841363769690149, 16.925336998980576, 2.4699069447442734, -17.325196253417694], [20.6400151951465, -19.592981951314933, 23.803222124484495, -17.133640902512926, 20.14957525381098], [-14.601769330401694, 23.591705241334108, -2.7886213763612542, -11.345310812424625, 12.818784442234735]], [[-0.52617973347263514, -8.3608111917024068, -13.496444188089853, 6.5015723705806572, -10.914995169738292], [13.328472535281733, -12.806116645022078, -7.6312333742292919, -14.123050728432847, -15.77413042934174], [-8.2460486113615055, -8.6095312205008643, -2.786553500142265, 15.662589569668478, -3.5751853289654507], [-6.17934987058152, -1.1617258958230539, -14.513636146151528, -4.6617724011845088, -0.47203320287250711]], [[0.85320426960176632, -1.9714629654010452, -2.1885948836771769, -1.5963896662477217, 1.6557735157776308], [-1.3110607018776388, -1.2926141469684485, -1.9559483976568932, 2.359612712748584, 1.8955842687890101], [-0.65435983500820505, 1.6062987113227374, 0.62402066297362446, 2.6457554912981074, 1.9071398594944313], [-1.2849759016331346, -2.60794734771319, -0.70467217717343733, -2.3701607844191499, 0.49713896203722696]], [[-1.13662604126911, -0.79623061639129267, -0.47127881408617106, 0.030675781189940007, 1.3735581488720612], [0.81055875130413513, -0.67381848277133616, 0.18443556328946548, -0.71040712807597028, 0.56399317841191898], [-0.10420180905228449, -1.1888931855957592, 0.19517165749688517, 0.65558264257404164, -0.6348101488211717], [0.31460852532305073, -0.26965092067903829, -0.92594128117684671, 0.35035304427348046, -0.575416812450951]], [[-0.93715701740716861, -0.98023875805561655, -1.216537504358491, 2.2929440879548424, 3.1191654142286209], [1.5028017130312574, -1.8717058728907732, -2.4951213620731778, -2.782526312288812, 1.9307896595742509], [2.9735874057126259, 1.6724210195727562, -3.1901941902029098, 0.96521184451933983, -2.3113370249287266], [-0.053215512938708305, -0.85697259161518491, -0.70434526650468943, -0.024964322150248328, 2.61414664442741]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[-3.2601447082749626, 1.646845170737925], [-1.3235447075251172, 3.8506613168499513]], [[-4.0357615556243251, -0.74135515648516126], [3.3803484539796784, 4.5124900400093217]], [[2.1452344097476672, 0.36251439526259688], [-0.97357976558454684, -1.7065383494802076]], [[0.97511452993603509, -1.7357638496797789], [0.79609778166073486, -1.2913982447935171]], [[1.7142248112203724, -1.2471747473302432], [-0.80641459737590626, -1.0307882544314619]], [[0.32745803185629718, 3.2112896152464785], [2.2091444425357771, 1.8391815219066379]]])
      arg1=Data(numpy.array([[[-4.6374534389137203, 4.8037196435265077], [-2.7081280524944731, 0.43814309007484731]], [[-2.3935558762640063, -2.1107403132734195], [0.33231844932954591, -1.672049824174624]], [[-2.8834161478224796, 0.26837481006281116], [3.3574067169099173, -3.1650201477078568]], [[-4.0097858830758852, -0.61220384510750048], [4.9891398840623111, 4.1322510668648142]], [[1.5833952792114259, 4.9935157684282458], [3.5706883218256475, 1.6993312098376778]], [[-3.3839662302118301, -1.4463412110612572], [-3.8263333515475164, -3.8360658852274589]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[15.118769288746092, 7.9109824965205355], [3.5843285511793628, 1.6871406481963183]], [[9.6598207866649712, 1.5648082152463543], [1.1233521564200546, -7.5451081779873288]], [[-6.1856035379308496, 0.097289731973634289], [-3.2687032444211401, 5.4012282589409688]], [[-3.9100004765196914, 1.0626413029725581], [3.9718431940971017, -5.3363817747953597]], [[2.7142954735934355, -6.2277867667790829], [-2.87945518539988, -1.7516506514894843]], [[-1.1081069216133392, -4.6446205111840309], [-8.45292305886049, -7.0552214929267718]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[-2.6361322028685352, 4.3034852838405033], [2.6322441098272691, -1.4599590751539449]], [[0.24542472638108848, -1.5429900282566757], [0.77974837556663168, -1.6368096631393181]]], [[[2.5341653595447911, -0.85575704114259743], [-4.3420604297779963, 4.9580324940753062]], [[2.3932559878499831, -2.5028966218134299], [-4.8859775360847193, -3.4797881542923967]]], [[[-2.6386162506028485, 0.63091799098622747], [1.7425362471589558, -1.4833090348534093]], [[2.7255541779025707, 3.0235934071801616], [-3.0586877906497367, 2.876887197108017]]], [[[3.5572414654071878, 2.8061615985627189], [-1.3580470987495707, -1.0838262827012466]], [[1.9730365906511196, -0.42459578290488054], [4.52916817124148, -3.51451682774095]]], [[[-1.8512803609629547, -2.5745989374689362], [4.1775689298440426, -4.4774252690595198]], [[-4.8385375386114973, 2.5152066054311009], [-2.8312805568074606, -0.39228816060812299]]], [[[-1.7862510623533301, 4.8217996377329584], [4.9373382716380938, 3.7050380182206784]], [[-1.9950980989169951, 3.6687443384382323], [3.145847489769185, -3.6921511096832713]]]])
      arg1=Data(numpy.array([[[-2.2472637136525266, -2.5427195995367899], [-3.0292923033959793, -0.87278263070947837]], [[-3.8238822526349248, 2.4657984937768518], [-3.3761091855456558, 4.0130942171676214]], [[0.37070860404620909, -4.9235470701792172], [4.4339515770901201, -2.443481339631294]], [[-4.1609954663149651, 1.3035192673746989], [-3.7636532205208741, 3.970439810094593]], [[-3.384697395809483, -1.9463865108279723], [-3.8750194528855983, -3.4629172341486214]], [[2.6433409397274499, -3.3751580704253605], [-4.0594665954116298, -0.15372735863726383]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[5.9240842438973598, -9.6710663206124075], [-6.6930586888230676, 3.7122665549155407]], [[-0.74346323468929554, 4.6741678168146921], [-0.68055083851848719, 1.4285790437654291]]], [[[-9.6903499436055291, 3.2723141621925533], [-10.706646067634653, 12.225509055987578]], [[-8.0798935239424701, 8.4500522753755121], [-19.607888195272491, -13.964717718959207]]], [[[-0.9781577468746242, 0.2338867277061431], [-8.5794592343805647, 7.3031418527228658]], [[12.08497524555567, 13.406466756245766], [7.4738465402107019, -7.0296201823576157]]], [[[-14.801665610146911, -11.676425689366628], [-1.7702405592223758, -1.4127884419881722]], [[-7.4258255186096118, 1.5980312857495356], [17.982789613710498, -13.954177526110028]]], [[[6.2660238166645525, 8.7142383189049699], [-8.1311638131024928, 8.714800146937753]], [[18.749427085636754, -9.7464745240718678], [9.8044902348784611, 1.3584614321223314]]], [[[-4.7216705617502068, 12.745660385582516], [-16.664297113939313, -12.505088968430307]], [[8.0990340871227886, -14.893145088995544], [-0.48360282527788367, 0.5675846377812519]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_constData_rank4(self):
      arg0=numpy.array([[[-4.100789882509587, 1.0879071440266515], [-4.0000660827929391, -3.2562101620751771]], [[-4.5841825486449554, 0.22742587195880493], [-2.9761916291812298, -1.9499457278681867]], [[-1.4889991245730338, -4.7592736065448022], [-2.2233968945415006, -0.19243908168777057]], [[0.32866825291641177, -4.2904066400755614], [-0.92604103750612232, 1.7898821728721899]], [[-3.529022904183071, -2.9591644469324629], [1.8151188567315977, 2.1497173222905968]], [[1.2926973400606325, 4.1938721142582658], [4.8787088157769514, 4.9909017354824279]]])
      arg1=Data(numpy.array([[[[-4.1301014726744913, 3.1610130702329311], [-2.3049997371675133, -2.4479739071579418]], [[-0.97999786439352832, 4.7306776149775231], [0.67873398638570315, 3.5016931790847394]]], [[[2.8979895718970248, -2.8872209340241985], [-1.6012361619522677, 1.4692522554553546]], [[-1.9996965584902537, 2.7647628328320861], [-0.72548936791668517, -3.6006563575223161]]], [[[-0.035533262089549211, 1.2103498939798527], [-1.7596811974807447, -0.65511974844293075]], [[-3.212625678790304, 2.7012833987104337], [-4.4833644115652129, 3.8648101903267946]]], [[[2.6748719050857943, -3.0820576604519245], [-0.76085884284589866, 4.0464241587586045]], [[-1.2716015180321341, 3.3625736305259384], [2.9007121651712673, -3.9476841718800113]]], [[[1.7441632951368398, -3.2188539592367116], [3.4379821518205169, 3.2696048100233792]], [[1.6280537719054955, -0.25254313587890742], [2.9910425375867042, -1.6379736741821862]]], [[[-1.5838969237207601, 2.0662124490825695], [-3.3266936332244788, -4.2020546538434642]], [[4.2787199228009811, 3.09485515121888], [-3.0583776584344391, 2.391296960868444]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[16.936678332881499, -12.962650416891771], [-2.5076256810440918, -2.6631683019879597]], [[3.9200562185700667, -18.923023076299383], [-2.2101005038149215, -11.402248914205062]]], [[[-13.284913221645406, 13.235547819836119], [-0.36416253034396467, 0.33414597532437484]], [[5.9514801582812069, -8.2284639997462392], [1.4146648935829313, 7.0210844818720659]]], [[[0.052908996144562945, -1.8022099325630649], [8.3748042791032606, 3.1178941278907106]], [[7.1429419575466424, -6.006025119969288], [0.86277453023324147, -0.74374052392402612]]], [[[0.87914547581974201, -1.0129745066483775], [3.2643938315062515, -17.360805079300086]], [[1.1775551890528375, -3.1138811735029686], [5.1919329930735421, -7.0658895233777459]]], [[[-6.1551922171733251, 11.359409347366716], [-10.173554552855638, -9.6752983093405529]], [[2.9551111011586682, -0.45839580807193497], [6.4298959547581616, -3.5211803808454198]]], [[[-2.0474993402240451, 2.6709873369292025], [-13.951727661060657, -17.622879835343273]], [[20.874628607609623, 15.098897109804259], [-15.264062363241127, 11.934728152052173]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[-0.3108872506930549, -0.059454261627768012, 3.9841466583644589, -4.2249051767520616], [0.67288821778681651, 3.2965683644377073, -3.8234961493972266, 2.6364929771979266], [-2.3706685224050563, 2.814572929462507, 1.1075486240313035, 0.89033655113791532]], [[0.3514729600354114, -2.2231563878109482, 2.0047353542868667, 4.5873131793439352], [-1.8246429947131881, 4.4094540652422474, -2.5901419776358914, -4.1260977238068346], [-3.6117296845760514, 3.1810505612578304, -2.1637002192212549, -3.7498702996084585]]], [[[4.9955122497815605, -0.049489304806530576, 1.712585587910386, 2.1344447971771814], [-2.6038185274676162, 0.70070310997077634, -0.61664291177497788, -0.057018587945384169], [-1.0020122716805693, 3.8641619985856739, -0.46004698283630319, -4.0531998939994951]], [[1.5802178816686521, 3.1046815280584976, 0.73562431677243634, 3.5012132425567355], [0.17165529931879853, -4.5201161390825693, -0.61930266742430895, -1.1196915514264583], [-3.2855663211155415, 3.8864226748934794, 3.5541794788673222, 1.7769108946461358]]], [[[4.3776688207657344, -0.22145372952243925, -2.0373658549561249, -1.8370422385047922], [3.5681673779985363, 2.8408489527705783, 1.6511249474012555, 4.5224281228566134], [-2.0429324985812833, -0.47152767434068732, 4.5161552959356648, -2.7875676643055778]], [[3.989474901635008, 1.2017117102925114, 1.1395835077172878, -4.9919802155497361], [-3.0439769822923166, 2.1148416399667171, 3.7634835883073201, -1.0676832034025816], [-0.18804346591546306, -1.9021010091203392, 0.85690673654323124, 1.6480776915730999]]]])
      arg1=Data(numpy.array([[[[-1.1385427074721188, -2.7393322046033672, -2.1373510349017319, 3.3475012581381485], [4.1567072225989641, -0.82296570058218421, 2.0114878956758648, 1.6106174626556555], [-0.72784364666771673, 0.3240033764145327, 3.2369862895023278, -2.6295990676720127]], [[4.0203577193166176, -2.3564098427218925, 4.7177463347718156, 1.400842997055233], [-4.4770732649402447, -0.12448971084422134, 4.6007670684903772, 2.0718877923460592], [-3.8544926282349392, 4.9728991121386006, -0.98146573706809104, -3.251001390144034]]], [[[-1.8515793153735718, 0.028656655135589126, 2.6338810915320297, -3.3321864975869087], [4.3395154282576964, 4.8608647583074038, 3.5933305169383392, 0.8113496252770771], [4.4348554676799132, 3.5541062651149051, 3.8402767115096932, 3.0376975426120936]], [[1.7617605479125684, 3.1808733121589334, 4.6491293204420323, -3.2844943615551534], [2.3350380056982729, -3.2346532684535934, 3.7927917984091124, 3.9188257997978493], [-3.5327836129995238, -2.6514228981558707, -3.8567722929239192, 3.5756391931406242]]], [[[-3.320845240003206, 2.7389397231752755, 2.6318063127768365, -4.5116922584595729], [4.4691851639108151, -0.033238906285237846, -2.8778274665113592, -3.3086532170236724], [-4.3333997230666865, 2.808724890945272, 0.40933876115319645, -3.6119158760681946]], [[-2.102597311971687, -1.3571160436118026, -4.6184876740132879, 0.76915833770448394], [-3.5079297356195638, -1.2996658404867514, 4.4396736956044407, 3.1614253588305488], [-3.3601556697737154, -1.8493416958790529, -1.3468011132580813, 4.3135896671846563]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[0.35395841212263407, 0.16286497357785912, -8.5155199834555528, -14.142875394691902], [2.7969993148762051, -2.712962693556543, -7.6909162236757993, 4.24638162924398], [1.725476022387664, 0.9119311323107947, 3.5851197109464978, -2.3412281647865774]], [[1.4130470280094272, 5.2386675941477669, 9.457832869874343, 6.4261055425831275], [8.1690603696909196, -0.54893166156288364, -11.916639913421749, -8.5488115039822432], [13.921385444375693, 15.819043511747061, 2.1235976304523789, 12.190833556886924]]], [[[-9.2495871513908323, -0.0014181979407408001, 4.51074679762743, -7.1123681329984318], [-11.299310672328957, 3.4060230532933438, -2.215801792934744, -0.04626200996331551], [-4.4437796017449429, 13.733642368592276, -1.7667077143865546, -12.312395357717865]], [[2.7839655210298027, 9.8755986153540913, 3.4200125799368712, -11.499715153779833], [0.40082164778890744, 14.62100844307327, -2.3488860777398051, -4.3878761395456847], [11.607194858660117, -10.30455007212476, -13.707660938174262, 6.3535922376152936]]], [[[-14.537560665750338, -0.60654841663432213, -5.3619523185095064, 8.2881692459253156], [15.946800708101613, -0.094426712111657332, -4.7516527242734563, -14.963146357447862], [8.8528431235960667, -1.3243915156902246, 1.8486374140137523, 10.068459902319653]], [[-8.3882592043562774, -1.6308622418341459, -5.2631523839011205, -3.8396232044459064], [10.678057370724725, -2.7485874375037231, 16.708639090847022, -3.3754007544343563], [0.63185531815974361, 3.5176347059398658, -1.1540829467347733, 7.1091309010872648]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(3.39417802494)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0686847757019)+(1.-msk_arg1)*(4.14836824227)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.233128356336)+(1.-msk_ref)*(14.0803003273)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([0.86551351388293174, -2.1006656642950894])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.90481612072)+(1.-msk_arg1)*(-2.30685953317)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([3.3796711217123274, -8.2027131501854669])+(1.-msk_ref)*numpy.array([-1.9966181005897261, 4.8459406136854639])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(0.87564560084)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-1.9786482186138521, 3.1553362812224144])+(1.-msk_arg1)*numpy.array([4.4377657140488296, 3.4903113416804779])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.7325946082393322, 2.762956333823587])+(1.-msk_ref)*numpy.array([3.8859100250659098, 3.056275771904839])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[-3.2480309099409768, -3.6400680031259869, -2.0933913390882819, -2.0459191239632046, -4.5602287625984097], [2.6147758781005308, -3.0442035089014849, 1.7367640225221077, -1.1648899850328078, -0.89208773551806608], [-1.9778429184559174, 1.11419317055406, 3.8535031058643572, 1.0659309816318983, -1.225441765840205], [-4.8125551820877419, 1.3390215896542408, -3.7876527371698208, 2.1146042262993543, 4.4839304639680631]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.14720708727)+(1.-msk_arg1)*(4.30934456753)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-10.222225899428082, -11.456047817571447, -6.588336058801711, -6.4389311669118108, -14.351984281207915], [8.2292411751724277, -9.5807388582974085, 5.4659560405916841, -3.6661500167814438, -2.8075848436863051], [-6.2246812504650411, 3.5065966429520752, 12.127772285581177, 3.3547055399292356, -3.8567190104851137], [-15.146107776929414, 4.2141782369631686, -11.920527538526596, 6.6550974077737912, 14.111857735012086]])+(1.-msk_ref)*numpy.array([[-13.996884356908719, -15.686307274693991, -9.0211445948048041, -8.8165704624471548, -19.651597044776519], [11.267970225588972, -13.118521853526365, 7.4843146055292005, -5.0199123287658711, -3.8443134368108112], [-8.5232066360665844, 4.801442286701036, 16.60607267519881, 4.5934638850524339, -5.280850816442217], [-20.738958529846006, 5.7703054131757234, -16.322300746595467, 9.1125582350693666, 19.322801386062405]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(4.05270654232)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.9745355330228316, 3.4300580574601085, 4.7924281402746018, -3.2390860393607026, -0.36816310412781483], [-0.87365306433489831, -4.7119498923847463, -1.6621226265526623, -4.8418039834123867, -2.1136069979982439], [2.1481982294422775, 2.2460283330245714, -3.2056042582154163, 3.1654177663177325, 1.6845006476905162], [4.4315374407835719, -4.8245072501962607, 2.8068915676993615, -2.6979984321925943, -4.4998689357654511]])+(1.-msk_arg1)*numpy.array([[3.0667091999578115, 0.82607258923077964, 1.5942923673169833, -0.26068993934592655, -3.9465408378869626], [-1.8464983306742058, 4.2682194550464931, 2.1016287452213147, -3.2836375223068148, -0.43099985137955965], [2.4196410004551003, 0.35267774183229772, 4.2177300318632476, -4.0007776697039006, 2.4520141249321927], [4.3330498281839258, 3.4813717051028892, 2.8524751669572392, 4.1538275852060469, 0.23697339962589137]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-16.107626157363587, 13.901018730004846, 19.422304877687719, -13.127065182853395, -1.4920570207395094], [-3.5406594895476613, -19.096150155950077, -6.7360952427675116, -19.622410680204769, -8.5658289086800998], [8.7060170186602281, 9.1024937194840003, -12.991373349357378, 12.828509290730759, 6.8267867954370596], [17.959720778798104, -19.552312096339019, 11.37550781999709, -10.934195897315112, -18.236648275557648]])+(1.-msk_ref)*numpy.array([[12.428472438060913, 3.3478297868065217, 6.4611991073958368, -1.0564998227041518, -15.994171873236207], [-7.483315865105685, 17.297840909522975, 8.5172845652854807, -13.307619269259147, -1.7467159174247426], [9.8060949126092734, 1.4292993916542771, 17.093222093870292, -16.213977836375403, 9.9372936859729144], [17.560579386878075, 14.10897788551703, 11.560244770931968, 16.834244230232422, 0.96038364701958123]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[4.6679373717194128, 3.8890376548718848], [0.11237713009688477, -2.9291877565021576]], [[-2.4938639135981044, -3.2574938769183612], [-1.4223722833403807, 4.0764479478995597]], [[0.89682341188354542, -0.042413581993498539], [4.8444236216753644, -0.79802066139290595]], [[-3.7852083673770065, -2.6552404416969244], [3.9114781507200611, 0.067206547805385597]], [[1.620675747450889, 0.09513497753404021], [-0.56362653646980654, 0.17485745821843235]], [[-1.8418496952425381, -2.2071183245432744], [0.668719853074351, -0.032600788679193471]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.393061072854)+(1.-msk_arg1)*(3.41911224093)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.8347844713419807, 1.5286293129924369], [0.044171075320102976, -1.151349682160703]], [[-0.9802408254300341, -1.2803940380759369], [-0.55907917568715759, 1.6022930038337238]], [[0.35250637243527505, -0.016671128041933523], [1.9041543460935959, -0.31367085732652644]], [[-1.4878180618560632, -1.0436716566979638], [1.5374497983658904, 0.026416277783179315]], [[0.63702454804104214, 0.03739385633544387], [-0.2215396511136454, 0.068729660123810571]], [[-0.72395941724731905, -0.86753229656007225], [0.26284774288798274, -0.012814100974121015]]])+(1.-msk_ref)*numpy.array([[[15.960201807546996, 13.297056251215611], [0.38423002111499932, -10.015221714242916]], [[-8.5268006342003684, -11.137737189330656], [-4.8632504851306431, 13.937833078183075]], [[3.0663399055248939, -0.14501679737571851], [16.563628105127471, -2.7285222118846577]], [[-12.942052263374688, -9.0785650968220555], [13.373782825262682, 0.22978673027213528]], [[5.5412722866899831, 0.32527716622737074], [-1.9271023901576843, 0.59785727581279313]], [[-6.2974908389595328, -7.5463852806299148], [2.2864282354003618, -0.11146575563704825]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(-2.72329386892)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.30021346820094497, -2.1032146892316641], [-1.0493199589928501, -0.43034816223145445]], [[3.0967100083635017, -3.3990375098953143], [-1.8318456518379653, 3.6762436643802427]], [[-0.49446855785739707, 4.3590547459635776], [3.7449719208515724, -2.2964804306073541]], [[-1.4427209317954013, 1.2160669541192624], [-1.4729138243498694, 1.5913732174539055]], [[1.8719973655797038, -3.7044030142468354], [-2.956493509684722, 4.1312546046464451]], [[4.6918149371686049, -4.5836258713318765], [-1.9403594631365193, 1.2221756184688903]]])+(1.-msk_arg1)*numpy.array([[[1.1390724318611749, -2.2477541156511336], [-4.8422256807165001, 4.5946938003172004]], [[-4.366719544283411, -4.976053575753987], [-2.6571286504007174, 2.7026076981024039]], [[4.1628162961494439, 3.6776036264480965], [3.4186960756365394, -1.065658712273827]], [[-4.8152674755867562, -1.2901791364751269], [4.4819613113750165, 0.28766681583664599]], [[-0.34581856651393572, -4.5644582478654625], [-0.35895755299639287, -2.8579322139803853]], [[-2.04583203185146, -2.5031073051333266], [1.4446530131845403, -2.741149557808483]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.81756949731782169, 5.7276716681999202], [2.8576066108570455, 1.1719645117044457]], [[-8.4332513795889934, 9.256578010915451], [4.9886540324518611, -10.011491831850204]], [[1.3465831919850819, -11.870987063954413], [-10.198659071319904, 6.2539910767599576]], [[3.9289530681160585, -3.3117076803450698], [4.011177187294499, -4.3337769262503016]], [[-5.0979989483112318, 10.088178016694574], [8.05140064841612, -11.250620335767131]], [[-12.777190852482578, 12.482560232805602], [5.2841690294539863, -3.3283433685156809]]])+(1.-msk_ref)*numpy.array([[[-3.1020289699394579, 6.1212950019847829], [13.186803508205747, -12.512701455952939]], [[11.891860562225297, 13.55125619425135], [7.2361421625589104, -7.3599949743290782]], [[-11.336572096729885, -10.01519540821155], [-9.310114062470225, 2.9021018374928707]], [[13.113388393458921, 3.5135369321668248], [-12.205697759988981, -0.78340127585869834]], [[0.9417655819449281, 12.430361161337814], [0.97754690327638172, 7.7829892761120236]], [[5.5714018291742686, 6.8166967773099376], [-3.9342146935173488, 7.464955784563287]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[1.404918094700248, 2.5197915366696666, 2.5172174071634688, 0.70735540798226637], [-2.5960222366335572, 2.6856501881053525, 1.7161014026196506, 0.74282045103081806], [-0.5978258779014034, 2.4254022700413529, 0.16452453709132797, 0.44077579201099937]], [[2.3560193707285082, 1.9884538890659549, -0.85511162761038406, -0.88154697797663673], [-3.393750753515683, -0.50567180236943532, -4.4036081553008497, -2.0787638939074848], [0.40149720034344227, -1.518331276444381, 3.4941027811587375, 3.1165905342130671]]], [[[-4.6365029491561778, 3.1337876874184865, -4.9044029716233686, -4.1754617141439248], [-0.60685046880651683, -0.13188465807462002, -1.7507506212604165, 4.759372779514905], [2.3737578799290961, -0.51664595129214597, 1.4448644975448826, -0.418454637705997]], [[-2.0714149045213115, -0.11344489556723847, -3.8567577461731117, -3.0883696132734939], [-1.3759493014972968, 3.1422775623296921, -4.7533069985161678, 4.3452194683694962], [-2.7349047608737829, -4.9192620559018927, -3.7884620472418828, 3.9755918897849511]]], [[[-3.0337362069403961, -0.4378344597981414, 2.0501243099893625, 0.27131723185556211], [-0.4695843622222915, 2.0494045455639043, -3.8043407145893515, -1.2490206254394698], [-4.0019358912317617, -0.56540417518973562, -2.3508974560073548, -1.9084347041656202]], [[2.9945951767230303, 3.5456693371490964, -3.6933215880348733, 0.75266104955017354], [3.841015248790109, 2.2063884735563803, 4.270150641883518, -2.394950036937511], [-3.6987225266352031, -4.8708148346572884, 1.0398994479239416, 4.7246638897208548]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.66835760781)+(1.-msk_arg1)*(-0.304478887452)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.3439057916443851, 4.2039133803004143, 4.1996188117552586, 1.1801217763334133], [-4.331093448533923, 4.4806249232444006, 2.8630708308354897, 1.2392901507148064], [-0.99738735154305014, 4.0464383292253734, 0.27448576312788736, 0.7353716459404358]], [[3.9306828413048094, 3.3174521736044027, -1.426631989451365, -1.4707356073500499], [-5.6619898886419415, -0.84364139853850884, -7.3467931677143961, -3.4681215572432103], [0.66984090870776813, -2.5331195362332526, 5.8294129574194837, 5.1995875281858783]]], [[[-7.7353449688624893, 5.2282785295688301, -8.1822980094783446, -6.966163316915253], [-1.0124435964369773, -0.22003077265233456, -2.9208781183595152, 7.940335785111909], [3.9602770180808347, -0.86195020338296313, 2.410550676734911, -0.69813197834056351]], [[-3.4558608148910661, -0.18926665458691794, -6.4344511273116192, -5.152504940036911], [-2.2955754851151404, 5.2424426769663031, -7.930215893235351, 7.2493799576623825], [-4.5627991644420884, -8.2070882757795243, -6.3205094784189386, 6.6327089748741273]]], [[[-5.0613568809404619, -0.73046445196601484, 3.4203404895288716, 0.4526541678964271], [-0.78343464322260015, 3.4191396650738266, -6.3470007738899872, -2.0838130627646962], [-6.6766601901080955, -0.94329635716585525, -3.9221376559132146, -3.1839515577051003]], [[4.9960556453997622, 5.9154444134146065, -6.1617811694902995, 1.2557077881199861], [6.408187012036743, 3.681044995644116, 7.1241383098850628, -3.995633114451747], [-6.17079186649348, -8.1262609856387886, 1.7349241553022865, 7.8824289447653353]]]])+(1.-msk_ref)*numpy.array([[[[-0.4277678984354345, -0.76722332369600099, -0.76643955560779664, -0.21537478765555543], [0.79043396241068919, -0.81772378135941814, -0.52251664582434942, -0.22617314450641371], [0.18202535819340021, -0.73848378480560728, -0.050094248012113408, -0.13420692276725796]], [[-0.7173581568146421, -0.60544222789229052, 0.26036343702202958, 0.26841244309094853], [1.0333254537196472, 0.15396638780126426, 1.3408057119003041, 0.63293971769221913], [-0.12224742087564101, 0.46229981783527307, -1.063880527449951, -0.94893601850045017]]], [[[1.411717259626724, -0.9541721885757769, 1.4932871604158846, 1.2713399373207235], [0.18477315559189805, 0.040156093962540163, 0.53306660136716899, -1.449128528875758], [-0.72275915836109317, 0.15730778445598315, -0.43993073473127592, 0.12741060253782768]], [[0.63070210558002093, 0.034541575589414585, 1.1743013077264508, 0.94034334388989971], [0.41894751251017448, -0.956757176243347, 1.4472816266257356, -1.3230275894636658], [0.83272075887787056, 1.4978114378655643, 1.1535067092981175, -1.2104837955646881]]], [[[0.92370862511188823, 0.13331134920746041, -0.62421956904374265, -0.082610368901922318], [0.14297852417427334, -0.62400041597225164, 1.158341428266294, 0.38030041043833929], [1.2185049878162453, 0.17215363422245397, 0.71579864191872167, 0.58107807549902535]], [[-0.91179100777758182, -1.0795814550476237, 1.1245384481271001, -0.2291693989954483], [-1.1695080496375589, -0.67179870771523664, -1.3001707166928922, 0.72921172274972224], [1.1261829199033249, 1.4830602818408691, -0.31662742696577112, -1.4385604047265736]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(-0.0682480919814)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.0909908543036444, -3.2025099474487009, -0.84070577218558729, -4.5530576123554001], [-3.7912050883068336, -2.0827611308996516, 0.25347057384358695, -3.9080709625467049], [-0.60254518137788704, 2.5657284181795799, -2.4993539144356367, 3.0288485148363673]], [[-2.4558221833087082, 2.437906871711812, -2.104514602022205, -0.86574822669421181], [2.425724930592593, -1.6391636669985443, 4.7683339382393921, 3.9100279422187612], [-0.74242025756384589, -1.9108869117758442, -1.9823053926761922, 3.9668693702859219]]], [[[-2.6712557459922817, 3.2277017729213817, -1.4208985704185229, -2.1966164238833983], [-3.7265315769452156, 2.1586595251391882, -0.13294817584548202, -1.6559208463866373], [3.2868010500780969, 3.6061742745260048, 2.3101748558909385, 3.6221558762423438]], [[1.9176934030120227, 0.331643007209248, -1.3547605173383905, -1.1140353997425745], [-1.4083707152128975, -3.8904352950733734, -2.0423239223316125, 1.2118627832663078], [2.0785705446516047, 3.2127263488166218, 0.32117175594092373, 1.7812151222504466]]], [[[-3.5621975214466097, -4.4916620025414691, -0.17535031557782332, -0.71768825334600628], [1.3530857380900958, 4.9796933086049702, -2.59732822244949, -3.0880987184719699], [1.1449554104486825, 2.0267058076822995, -2.9406388182479057, -0.21728102388700066]], [[1.7097637999640565, 3.9469781961755341, 4.4913216989628086, -4.531434291646983], [-3.4499630793552383, -3.9570637865174065, 1.5044483065655969, -0.090659609480693959], [3.4018088693635988, -4.5050117531517788, 3.5310591995505618, -1.3421611197062813]]]])+(1.-msk_arg1)*numpy.array([[[[-4.4406985942413542, 4.0118804395532113, -2.0176982100225418, -1.6408279792004867], [2.896425706561029, -4.6453269286274743, -4.964509368856886, -1.4690482045837214], [-1.0137003835871585, 4.4452790960809541, -0.85679792319512504, -3.5474654407761594]], [[-4.6663465852800776, -3.9245220708215447, -4.8751091169865033, -2.6573521517262169], [-3.8591890250430518, -1.6627973811504049, -4.2037031409102301, 0.46572098743209001], [1.2195008700301022, 4.5971636305396615, 2.2299034844804737, -1.7724684203887051]]], [[[4.1362711043892926, -3.7128662209852994, -0.87335816303058778, 0.7975797175702759], [-0.35418687926634185, 1.0207504155641365, -2.0511339330037668, 1.6508190441841375], [-1.7071113829243068, -2.4779387372268924, 2.52054707814544, 0.30133313505412573]], [[-1.4605659790909886, -0.95145188053993124, -3.7785880831440046, -1.9952755552842505], [3.072728408325764, 1.4952332131670607, 0.62546001718281374, 1.4050486899384618], [-3.7145497736380948, -3.5213372605563711, -4.4781297244915184, 2.4056906897028041]]], [[[4.9238880639097324, 1.6707879568789998, 0.7770351718978441, 2.5503897154885866], [-4.4948792617493991, -2.5884736620362805, -2.8850391295649747, 1.0872606400790543], [1.8284358159986809, 4.7556138091091853, 1.0018144192360197, -1.0880443461377673]], [[4.5630830764399217, 2.1075302279371559, 0.19281363029015086, 2.4322823114295113], [4.4228408259709049, -3.7105284411480688, -3.9912578881784464, -2.0099642202015566], [1.3148716027876368, -1.251859727018052, 1.5469835260764011, 2.7311314299224492]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.2792023201195955, 0.21856519346483852, 0.057376564869418792, 0.31073749472466061], [0.25874251358712963, 0.14214447323693133, -0.017298883038256243, 0.26671838652173951], [0.041122558961629474, -0.17510606908322079, 0.17057613584648415, -0.2067131320382897]], [[0.16760517825642379, -0.16638249242268333, 0.14362910613501817, 0.059085664608163509], [-0.16555109818466685, 0.11186979271789126, -0.32542969321500986, -0.26685194665040374], [0.05066876602707461, 0.13041438572093783, 0.13528856077459689, -0.27073126566148564]]], [[[0.18230810785833379, -0.22028448748687751, 0.096973616330168044, 0.14991487974505571], [0.25432866983496161, -0.14732439382823206, 0.0090734593338622598, 0.1130134382381187], [-0.22431790039030347, -0.24611451358882169, -0.15766502605797028, -0.24720522741276857]], [[-0.13087891576089516, -0.022634002461006011, 0.092459820400084228, 0.0760307904321708], [0.096118614115764819, 0.26551478586586613, 0.13938471090710861, -0.082707322701198599], [-0.14185847372121863, -0.21926244336511533, -0.02191935954128503, -0.12156453350201531]]], [[[0.24311318409961569, 0.30654736149882511, 0.011967324466523412, 0.048980853928331035], [-0.092345519911898058, -0.33985456696485139, 0.17726269543162795, 0.21075684538592918], [-0.078141022166907242, -0.13831880438194619, 0.20069298855186868, 0.014829015304053545]], [[-0.11668811708642085, -0.26937373098118178, -0.30652413642888704, 0.30926174434400916], [0.23545339757228226, 0.27006205327852068, -0.10267572640775555, 0.006187345366836519], [-0.23216696461948103, 0.3074584565064063, -0.24098805304270765, 0.091599935551577763]]]])+(1.-msk_ref)*numpy.array([[[[0.30306920612147303, -0.27380318525702096, 0.13770405302833158, 0.11198337885013519], [-0.19767552803867863, 0.31703469950865831, 0.33881829204828406, 0.10025973699154545], [0.069183117020640381, -0.30338181663234304, 0.058474823471696358, 0.24210774770294133]], [[0.31846925096930273, 0.26784114327247682, 0.3327168954354735, 0.18135921407799127], [0.26338228755476095, 0.11348274861518963, 0.28689471862335603, -0.031784568787935327], [-0.083228607549215974, -0.31374764631063351, -0.15218665811847548, 0.12096758778882126]]], [[[-0.28229261079238177, 0.2533960353644506, 0.059605028243221121, -0.054433293927237988], [0.024172578714775544, -0.069664268251477007, 0.13998597732581894, -0.11266524997213158], [0.11650709468431897, 0.16911459086254371, -0.17202252883272776, -0.020565411518218647]], [[0.099680841285910413, 0.06493477545896853, 0.25788142705824707, 0.13617374962528542], [-0.20970785104528822, -0.10204681386587502, -0.042686452783382865, -0.095891892229270581], [0.25351093462075414, 0.24032454925599453, 0.30562380934175409, -0.16418379946964293]]], [[[-0.33604596549184607, -0.11402809016249915, -0.053031167884469724, -0.17405923189109049], [0.30676693330117594, 0.17665838857809235, 0.19689841588449858, -0.074203464171874914], [-0.12478725575237049, -0.32456156867211622, -0.068371922632316173, 0.074256950615056341]], [[-0.31142171351965731, -0.14383491684984323, -0.013159162375310538, -0.16599862691518194], [-0.30185044750996881, 0.25323648635108703, 0.27239573547390478, 0.13717622297964574], [-0.089737478090786041, 0.085437037797342658, -0.1055786739813781, -0.18639450904264929]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([-4.2420906621066301, 0.29168025607277581])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.7734055768495463, -3.500975945309206])+(1.-msk_arg1)*numpy.array([4.8875524867397715, 0.7150639549250144])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([20.249219224001173, -1.0211655602324177])+(1.-msk_ref)*numpy.array([-20.733440764554825, 0.20857003748094002])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[1.8993647676214342, -0.20775388007919648], [-3.1734606297775017, -1.5282232233957149]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.9767520261538314, -3.18383530557931])+(1.-msk_arg1)*numpy.array([1.1808549927384604, -3.356395312009508])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-9.4526674556651749, 1.0339395436254613], [10.103775993951562, 4.8656110534534944]])+(1.-msk_ref)*numpy.array([[2.2428743688772963, -0.24532720655230653], [10.651388380631948, 5.1293212627094364]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_expandedData_rank2(self):
      arg0=numpy.array([4.2276836779211049, 4.3123771246161198])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.9140487996137168, -0.69577796210517651], [-1.0586489427819257, 4.3197586098955814]])+(1.-msk_arg1)*numpy.array([[-2.2551128248069743, -3.258310024301232], [-3.7454841656150584, -3.8487962470466099]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[20.77504390263471, -2.9415291338492637], [-4.565293483851816, 18.628428213177234]])+(1.-msk_ref)*numpy.array([[-9.5339036813070006, -13.775104107345037], [-16.151940236410272, -16.597460893072174]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_expandedData_rank1(self):
      arg0=numpy.array([[[-1.5415600027397893, -0.80024559887162372, -1.5278699748759994, 2.776821459086114, -0.90098796490741062], [-3.9782809690324541, 1.3143704376279697, 0.91229996334774377, -0.52937709993626392, 4.1965785700176674], [1.2243976592408989, -3.2947749779907012, -2.1721457590323103, 0.50106539458687926, -1.2046421607231128], [3.1689053848514455, 2.6361169414026673, -1.3055136846936808, -0.17462833331514016, -4.9197254863531361]], [[1.242910611528723, 4.6669927434748466, -1.4707201364149283, -1.3623759480802877, -4.139137811061917], [-3.429830237739794, 2.2288844250133852, 2.023560374540847, -4.685146540595678, 0.14259306519852633], [2.4182791926550316, -1.6579641023264591, 1.0315271217768593, -0.38378439119743746, 3.8834496649404464], [3.5917502905643879, -2.8079718624424741, 0.91288456970887566, -3.9123558390741522, -3.7460205686952652]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.5670538888810768, -3.4649565484067759])+(1.-msk_arg1)*numpy.array([-2.6279431732635516, -0.095047259426342734])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[5.4988276027164886, 2.8545191755149917, 5.4499945355860664, -9.9050717843615494, 3.213872624058026], [14.190742601648793, -4.6884301809711717, -3.2542231320856332, 1.8883166430122367, -14.969421908176509], [-4.3674924319321358, 11.752639898229795, 7.7481609769727404, -1.787327264344859, 4.2970235041174822], [-11.303656276530534, -9.4031711873756745, 4.6568376659740576, 0.62290867546059159, 17.5489259283233]], [[-4.3066312625007193, -16.170927067870075, 5.0959813675446126, 4.7205734626926823, 14.341932663197078], [11.884212742180068, -7.7229876840920006, -7.0115487708617756, 16.233829186082346, -0.49407877501702813], [-8.379232324465903, 5.7447735733794261, -3.5741966554599225, 1.3297962394558687, -13.455984346943499], [-12.445258689533016, 9.7295004925120221, -3.1631053677522707, 13.55614298429747, 12.979798499967133]]])+(1.-msk_ref)*numpy.array([[[4.051132085376171, 2.1029999584888861, 4.0151554701097369, -7.2973289967770878, 2.36774517157105], [10.454696313893145, -3.4540908187038499, -2.3974724606482916, 1.3911729358595617, -11.028370004142047], [-3.2176474699619928, 8.6584814108502322, 5.708275618782336, -1.3167713830631971, 3.1657311424977586], [-8.3277032728384643, -6.9275655200835331, 3.4308157752929032, 0.45891333639391463, 12.928759006192431]], [[-0.11813524733772482, -0.44358487002991276, 0.13978791834937587, 0.12949010017339674, 0.39341370532938641], [0.32599596439476897, -0.21184935617558198, -0.19233386788385115, 0.4453103386944296, -0.013553080060321736], [-0.22985080978960945, 0.15758494415338642, -0.09804382594883379, 0.036477654593923846, -0.3691112477727384], [-0.34138602166191528, 0.26689003007144063, -0.086767176523424763, 0.37185870040464775, 0.35604898880919483]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_expandedData_rank3(self):
      arg0=numpy.array([-0.67896494257042228, -2.7807860280891425])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.6678101909857013, -4.509481423211259, -4.4637505713752521, 2.9208932339448692, -3.2471397610234085], [-4.7286691537743897, -1.9916926011053993, -2.0976255884785888, -0.036817614830769507, -4.5872897009081903], [0.84654217001397747, -4.9106251525693985, 3.5439597038259869, 3.5853953491121064, 2.7968283017598283], [-2.1907519311343759, 3.2278243008916068, 1.3903917973279425, -2.2994388421960954, 0.72752927315799987]], [[-3.935831112421968, -1.5760845822824687, -1.6010529047311595, 0.31771948383573445, -0.5447487836769227], [-2.7644230236248948, -4.5977764551547251, -3.1139876934145816, -0.048802582017668072, 4.0633819020629627], [-0.85956940863919229, 0.96909205125577014, -4.1546446755970212, -4.4541552904359847, 2.1310358239947078], [-1.4492454281227038, 0.96237650283040121, -1.7439242595175664, -4.7272731753783308, -4.8467475675493734]]])+(1.-msk_arg1)*numpy.array([[[-0.17758443222566989, -4.3243248123780917, -1.8729092142074135, -2.5526698338812315, 2.1024861695336963], [-4.0696862302471635, 0.38906797169833585, -3.2832694462891796, 1.2356369969765568, -2.4077908188764829], [2.456883021007342, -1.710132152467021, -3.1351891554581925, 4.8802169986028261, -3.2637915686776209], [3.0576330717387652, 2.3883931486963377, 0.39504942572490087, -4.5104972970546333, -3.2550139385544776]], [[4.2406150363244812, -0.84768452858578236, 4.2246563021854868, -4.0993808304203174, -0.22860000107065126], [1.6219141074324517, -2.1034307392582718, -0.073959135014333377, -2.9373929043418667, -3.0619496815432656], [2.2270050848778968, 2.0647687139297464, 2.5994886305456042, 0.44864476973692025, -2.7099906180971125], [-2.7479184621524224, -4.9470231749257536, -0.40225294068305395, 1.8045075973732541, -3.4208176027941919]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.1323846505409716, 3.0617797955330186, 3.0307301503424875, -1.9831841068397131, 2.2046940613613932], [3.2106005804269557, 1.3522894525274625, 1.4242142372156132, 0.024997869739153345, 3.114608888331019], [-0.57477245584698089, 3.3341423246991528, -2.406224396780102, -2.4343577473021605, -1.8989483672836935], [1.4874437591086933, -2.1915795410822834, -0.94402728682315273, 1.5612383614358702, -0.49396687116802246]], [[10.944704166341555, 4.3827539854978017, 4.4521855477079448, -0.88350990150210451, 1.5148298064673413], [7.6872689198240494, 12.785432526771485, 8.6593334694888053, 0.13570953820940582, -11.299395620046971], [2.3902786017167128, -2.6948376360642929, 11.553177865575146, 12.386052798583723, -5.9259546447219167], [4.0300414377956821, -2.676163132832071, 4.8494802149121528, 13.145535197052657, 13.477767917516335]]])+(1.-msk_ref)*numpy.array([[[0.120573603827503, 2.936064947892143, 1.2716396970639512, 1.7331733271624197, -1.4275144013525531], [2.7631742775994037, -0.26416351306015129, 2.2292248510429551, -0.83895420269007692, 1.6348055550600613], [-1.6681374392604955, 1.1611197786876035, 2.1286835248830824, -3.3134962541875663, 2.2160000549890295], [-2.0760256629545344, -1.6216352170401989, -0.26822471064978565, 3.0624695382587439, 2.210040351856565]], [[-11.79224304351585, 2.3572292933186749, -11.747865218596145, 11.399500937049286, 0.63568768899843009], [-4.5101960887088337, 5.8491908107826189, 0.20566452929741674, 8.1682611474020508, 8.5146268931475131], [-6.1928246245119309, -5.7416799909314262, -7.2286216639977949, -1.2475851072596984, 7.5359040470571097], [7.6413732658816595, 13.756612925466726, 1.1185793572092071, -5.0179495143562525, 9.5125617944914822]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_expandedData_rank1(self):
      arg0=numpy.array([[[[-3.4762865697286518, 1.1943539679612334], [-2.5226186302835698, -3.5600230077800896]], [[-2.7700903161571331, -2.0886760805759197], [-4.1899526212226252, -0.56970503292502883]], [[-3.6500726551173868, -4.9134662299610179], [-2.9780486599669276, 1.2284301758621021]], [[3.3095901839859145, 2.1787548110829515], [3.9346966692368319, -1.7738673524801629]], [[3.3937748899123257, 0.54820621281853832], [-4.9987554028192722, 2.8287936355157903]], [[-1.4681539025180799, 0.10145858158992205], [1.5352182183098941, 0.36155844328038178]]], [[[-0.60987756114806935, -4.1198959537113797], [-1.0356426119458506, -2.3054796227137886]], [[-3.2854537587201351, 2.1753323030175276], [4.0183811389018693, -4.9304159921026685]], [[1.8077863351183021, -0.99376718493820348], [-0.29620745411462668, 1.7385264932355096]], [[4.0103868805184177, -4.1063917729745132], [2.4349017474930257, 1.3622625304644576]], [[4.6606087133896441, -1.8891147822618057], [3.7994278496021892, -1.6344262667501361]], [[4.0479485502914372, 4.5877540699563575], [-1.5941522431962563, 0.79184682531159201]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-1.4501090962638727, -2.7044442423675719])+(1.-msk_arg1)*numpy.array([3.2672877717184061, -3.0176275895181237])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0409947759834539, -1.7319435530994345], [3.6580722221789159, 5.1624217464905797]], [[4.0169331649319258, 3.0288081835919147], [6.0758884089495862, 0.82613445043189337]], [[5.2930035592097484, 7.1250620742518294], [4.3184954509344786, -1.7813577721426632]], [[-4.7992668307035986, -3.1594321700800636], [-5.7057394310994924, 2.5723011833969975]], [[-4.9213438385337867, -0.79495881583653094], [7.2487406796264056, -4.1020593823147982]], [[2.1289833287567705, -0.14712601205757625], [-2.2262339031211935, -0.52429918743188708]]], [[[1.6493798587960729, 11.142028891168197], [2.8008376990274693, 6.2350410915440673]], [[8.8853265013355678, -5.8830649221319424], [-10.867487734741607, 13.334035142319062]], [[-4.8890573454414659, 2.6875879415599542], [0.80107654382665883, -4.7017479648342588]], [[-10.845867708684482, 11.105507587326487], [-6.5850560117382528, -3.6841630571076811]], [[-12.60435640085476, 5.1090055960594096], [-10.275340772147645, 4.420214706686731]], [[-10.947451150235837, -12.407325079891866], [4.3112958555694645, -2.1415055875509754]]]])+(1.-msk_ref)*numpy.array([[[[-11.358028600263347, 3.9022981146230951], [-8.2421210034345425, -11.631619640356067]], [[-9.0506822165357743, -6.8243058171464304], [-13.689780963400166, -1.8613902875623787]], [[-11.925837751948773, -16.053708129902972], [-9.730141970292328, 4.0136348920041378]], [[10.813383537536449, 7.118618951823974], [12.855786312818642, -5.7957351094089402]], [[11.088439197775521, 1.7911474555220683], [-16.332372401442722, 9.242482854035595]], [[-4.7968812926979796, 0.33149438296464651], [5.0159997116032358, 1.1813154804915342]]], [[[1.8403833547484405, 12.432311695863541], [3.1251837186884108, 6.9570789165729634]], [[9.9142759063999009, -6.5643427739556905], [-12.125977789949541, 14.878159325570383]], [[-5.4552259208068454, 2.9988192748272824], [0.89384378575722112, -5.246225511095667]], [[-12.1018540952939, 12.391561107498134], [-7.3476266910008459, -4.1108009960963203]], [[-14.063981437473156, 5.7006448867197479], [-11.465258303343083, 4.9320897955783192]], [[-12.215201226309333, -13.844133255424364], [4.8105577909612283, -2.3894988267325981]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_expandedData_rank4(self):
      arg0=numpy.array([-1.8105500736839772, -3.2801345629769338])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.922232930432652, 2.7555767995863043], [-3.0029046625684295, -0.41215409841245965]], [[4.1697760962634884, -0.9418377250440555], [-1.5058236578878437, 3.9712770028491864]], [[-3.5855164235189108, 1.9502925820358454], [-0.70839370496705101, -0.78668732736546954]], [[0.97668338815848621, -4.9365914336811638], [-2.6686470884361047, 1.5388622349274508]], [[-4.4470938340436295, 1.6540525327989002], [2.5389844452396062, -1.5371997424124673]], [[-2.145210338118384, -0.46941788555700104], [-3.9054497412666755, -0.49618218686764859]]], [[[4.7074611151833423, -3.8578933464321539], [1.3920638947289179, 4.8681808703558929]], [[-0.73668000727241889, 2.9910261176493638], [-1.3592786083246953, -2.0620809894839445]], [[1.7759342653887238, -2.3168025884048458], [3.8030060496575295, 3.8379902011076243]], [[2.3477617180884902, -2.6834854901939176], [-3.1091392856735389, -4.433391025499934]], [[3.0990390054543067, -1.9071118592848069], [3.3453376889113535, 0.6602869440407968]], [[2.4003908111263508, -1.7811691397775142], [0.29453260640062151, 1.0397835977476078]]]])+(1.-msk_arg1)*numpy.array([[[[-1.3926809929295745, 3.6526679808435585], [0.79866700423631976, -0.28381449095055711]], [[1.6840732434739039, -4.3403382194072195], [-4.4952104013829057, -4.6253056156030103]], [[-4.8974700195838219, -0.26380518732774583], [-3.6621536790367806, -3.0231017845255259]], [[-0.51869551014518489, -0.69456141932042748], [-4.9553460991400611, -2.1458561589220682]], [[3.4937870131011444, -3.415732404284423], [-4.7399861820507159, 2.5490750090641336]], [[2.4603491119178464, 2.6406161594758757], [-4.5502997074185068, -4.4769646064642989]]], [[[-4.8219585481506488, -0.49996770261850809], [2.6508888636555348, 4.4286961267021407]], [[-2.7618174535392672, 0.84287792507793835], [2.146642207334434, -1.8166232289695627]], [[-0.32839343771800245, 0.38226352712422695], [-0.15191994943324971, -4.8112099003327513]], [[-0.036090996519887675, 2.3039414078813927], [3.2007478722372156, -0.36638412990627955]], [[-3.1625212944318726, 1.1144208151069552], [-4.7091278042516924, -0.81097583964076847]], [[2.4279296970985449, 0.23998622603261222], [1.2534353819777388, -0.84841849166646988]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.1013991212005596, -4.9891097775328417], [5.4369092580792291, 0.74622563324983204]], [[-7.5495884183355457, 1.7052443624768643], [2.7263691347439116, -7.190195870128079]], [[6.4917570247972742, -3.5311023781103144], [1.2825822747253599, 1.4243367985278019]], [[-1.7683341803962638, 8.9379459839991213], [4.8317191826045205, -2.7861871328373859]], [[8.0516860689072534, -2.9947449351362181], [-4.596958474411041, 2.7831771068918836]], [[3.8840107357478701, 0.84990458728380502], [7.0710123168194494, 0.89836269499389809]]], [[[-15.441105907782822, 12.654409305910853], [-4.5661568949726075, -15.968288331677496]], [[2.4164095537083603, -9.8109681472683903], [4.4586167438810191, 6.7639031252639619]], [[-5.8253033654766035, 7.5994242458211581], [-12.474371586692037, -12.589124311019912]], [[-7.7009743571361655, 8.8021935056321681], [10.19839523204719, 14.542119133934087]], [[-10.165264953804334, 6.2555835251032974], [-10.973157778227508, -2.1658300266306343]], [[-7.8736048642277803, 5.8424744578921182], [-0.96610658217835987, -3.4106301169884334]]]])+(1.-msk_ref)*numpy.array([[[[2.5215186743669156, -6.6133382818594093], [-1.4460266033690301, 0.51386034750311171]], [[-3.0490989350608912, 7.8583996829611236], [8.1388035234488001, 8.3743474231409447]], [[8.8671147048225585, 0.47763250135446561], [6.6305126134220913, 5.4734771587268538]], [[0.9391241941129127, 1.2575382289286479], [8.9719022449276462, 3.8851800266515668]], [[-6.3256763340063999, 6.1843545562619111], [8.5819823311729575, -4.6152279454870522]], [[-4.4545852658711649, -4.7809677821101477], [8.2385454705507577, 8.1057685981144942]]], [[[15.816672895031019, 1.6399613417311416], [-8.6952721842871679, -14.526719234117767]], [[9.0591328859870934, -2.764753014418428], [-7.0412752986227742, 5.9587686412498231]], [[1.0771746653136329, -1.2538758074856475], [0.49831787694171042, 15.781415883818267]], [[0.1183833250971638, -7.5572378430654936], [-10.498883723100169, 1.2017892478318184]], [[10.373495404016538, -3.655450233333251], [15.446572872201653, 2.660109881344924]], [[-7.9639361159310553, -0.78718711464796609], [-4.1114367188833763, 2.7829268183839457]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[2.483246455841412, 1.0466480036536217, -4.9324735778739015, 0.48820288947660551, 3.2093078766146981], [3.9932412364684247, -0.56633403589432252, 1.7834708690293137, -0.67587098812692936, 2.0319532928787085], [-3.4390422110681538, 3.7241801800917003, -1.8323863227934178, 4.0332296950060211, -3.4061396757480464], [1.4539012688222597, -4.3994448413110474, 0.25860934218840015, -2.48272227666817, -3.3094651770745487]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.6919610312513544, -0.912667200793158, 4.5719154474843755, -3.9667301430731983, 3.8024142953378419], [-2.0296058610915191, -0.16592855993146305, -1.2740998208858656, -0.47601625972323891, -2.4472616875494402], [-4.6326083041537576, -3.7665105756160622, 3.6023889403562315, 1.2310410222728141, 1.6580702320374083], [3.78771491601319, -1.2224811233035759, -4.0104061091912619, -1.5739746017714698, 3.7374141294184593]])+(1.-msk_arg1)*numpy.array([[-1.8201658848509252, -1.5938973217119878, 3.5790862183905272, -4.180153439397964, 1.2246263537141147], [-1.5664773066333315, -1.4051336941118144, 2.8172506116068323, -3.9825397827637641, 2.8705721702240572], [-1.3515427356346308, -4.8467140859320086, -2.3144282510548733, 3.4156012969032208, -2.1689916509418259], [-3.5051403835164652, -0.24043958786125774, -2.6006351702264565, -1.7222028060964911, -0.36870662175698321]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.1680491459595306, -0.95524130371029792, -22.550852144990216, -1.9365691176222841, 12.203118148180064], [-8.1047058182886591, 0.093970991016118444, -2.2723199147854078, 0.32172557982363054, -4.9727214445519898], [15.931735505329629, -14.02716403381512, -6.6009682236910319, 4.9650712068012828, -5.6476188025193865], [5.5069635223285758, 5.3782382715180512, -1.0371284858062935, 3.9077418067279397, -12.368841913616782]])+(1.-msk_ref)*numpy.array([[-4.5199204825995078, -1.6682494497987064, -17.653748205143895, -2.0407629875696567, 3.9302030028846455], [-6.2553217768402121, 0.79577503595744237, 5.0244843965558026, 2.6916830982313518, 5.8328685737327532], [4.648012517910022, -18.050036537399247, 4.2409266723196408, 13.775904576971147, 7.3878885186392109], [-5.0961280509947304, 1.0578007044631648, -0.67254855064428198, 4.2757512716361914, 1.2202217252615331]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_expandedData_rank2(self):
      arg0=numpy.array([[[1.9252468006199495, -2.6761935349349484], [2.6503061247693385, 3.8671073065794523], [3.1717919747891923, -1.9516755987776992], [-2.2729122866052731, -1.1877583737859201], [0.076807883101346341, -1.6536281474968439]], [[-1.6728491544723911, 2.0148774420204107], [4.2849741927016876, 2.1398007040691702], [-4.3237959315832386, -4.6381973925960107], [2.6716856850215356, -3.1454950510817192], [2.3120442672969199, 3.8655897480684569]], [[3.2316819368150966, -3.7956013917281117], [-2.5048795377007194, 4.1775091998613991], [-2.9720877452314278, -3.0902070523949856], [2.2640895024850103, 4.2752553333673795], [-1.5931667249373982, -2.4484414602693763]], [[3.9420819439750137, 4.1986028634624617], [-4.7335858272741538, -4.8231284907698715], [-0.33185922145677793, -3.9769099747767225], [-1.8741067927279573, 0.90114449890255433], [1.4506972230424449, -3.5061151580947056]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.2208657207104814, 3.8906982733683897, -2.412355517070881, -2.3087702464901652, 0.55159026063827188], [2.0094617836303605, 3.6184886818643118, -3.2706015857877757, 4.1763037000495231, -0.74022745405271007], [0.77464048588052759, -2.8189446766198234, 4.1938894557942525, 3.3481910617282136, 3.9664673796394538], [0.99513516531932567, -2.2849101104500082, 1.1917416500549747, 1.0969396612214624, 2.5579599328126452]])+(1.-msk_arg1)*numpy.array([[1.2534863011418924, 4.7043110282090161, -3.4393927097555208, 4.6336447451820089, -0.039662982253587664], [3.7939294316647079, -1.6664102928127145, 3.6792025269503199, -2.0927171281838008, 0.15270076602224592], [4.3987403173530808, -0.20517346229978628, -1.3420598552698348, -1.2121022726355246, 1.7141001387629293], [2.1881659526751225, 2.2512858574353096, 0.48900622731869436, -1.124708590755632, 4.6438883031916482]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-8.1262082246442713, 11.295853553593933], [10.311541463537733, 15.045747720638959], [-7.6514898693838527, 4.7081353982439982], [5.2476322601961813, 2.7422611934164762], [0.042366480258945544, -0.9121251808765668]], [[-3.3615264456906315, 4.0488192184389131], [15.505130618371723, 7.7428446291195785], [14.141413830458873, 15.169695747421239], [11.157770811724784, -13.136542620320048], [-1.7114386416383625, -2.861415657624971]], [[2.5033916657457711, -2.9402265062970714], [7.0611168383753666, -11.77616732047963], [-12.464607456421399, -12.959986773260367], [7.5806042351729896, 14.314371693786534], [-6.3192438447912123, -9.7116631831152702]], [[3.9229043670199037, 4.1781773546419112], [10.81581811542158, 11.020415052559569], [-0.39549045616485978, -4.7394492554604986], [-2.0557820703078469, 0.98850114133775235], [3.7108253711851433, -8.9685020942333296]]])+(1.-msk_ref)*numpy.array([[[2.4132704908943627, -3.3545719352454544], [12.4678643308823, 18.192075549609381], [-10.909038194951014, 6.7125788262437593], [-10.531868073088148, -5.5036503472390566], [-0.0030464297043843358, 0.065587823868200362]], [[-6.3466716418882259, 7.644302828478537], [-7.140525099154944, -3.5657859178287583], [-15.908120917498564, -17.064867567333629], [-5.5910823941680388, 6.5826313700160934], [0.35305093069358195, 0.59027851565779399]], [[14.215329628330258, -16.695864870395908], [0.51393480739394459, -0.85711402632477307], [3.9887196492145396, 4.1472428294910371], [-2.7443080314123152, -5.1820467056717483], [-2.7308473042876757, -4.1968738468006475]], [[8.6259294924614842, 9.1872398346328339], [-10.656654827898523, -10.858240959863521], [-0.16228122588549809, -1.9447337431516489], [2.1078240097746184, -1.0135249594278821], [6.7368758655594156, -16.282007172318941]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_expandedData_rank3(self):
      arg0=numpy.array([[1.175121919864667, 4.996980247387846, -4.3505357866030172, 0.73261963349544601, -3.0470380000082962], [2.1376695842255442, -2.2683550118966589, 4.084750760615373, 1.0698380602131667, 2.8424003368092432], [1.1290427595385921, -3.8556843972781385, 2.8592192810043233, 3.7379423107680729, -4.2204570583628866], [3.0322827037890256, -3.6003854152623669, 2.4279365700660627, -4.5533730572391065, 1.3242621419706868]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.4197691878302661, -3.2939532407981833], [-3.907701802877491, -0.67542124705914564], [3.9937814354795513, 1.612822860720815], [-4.6863395760827995, 0.42292314245077289], [1.0676858138365288, -1.371983282847522]], [[2.6181973970014374, 0.3555756299355064], [2.5995972145890791, 4.0945797707715972], [-3.3252008862840965, 1.0812397461871628], [2.0698925108332826, -2.5271733390709161], [-2.0057221570165886, -4.8501116814420442]], [[1.79585354019249, 2.0479421357501177], [4.552932904452053, 3.3259700200155873], [3.21824998981063, -1.8373246223652462], [4.2131670621625918, -1.3192138224865566], [0.68029767406157582, 3.1534286836989995]], [[4.8645588625873319, 4.4908052996414085], [-2.5277603540800229, 0.37855926351717883], [-3.6000935374617047, -1.1052287399414249], [4.622038789997994, 1.9328850653792564], [1.0519356798011925, 1.2132198148065036]]])+(1.-msk_arg1)*numpy.array([[[-2.6396086316446987, 1.3215991022785527], [-1.8254034281036069, 3.4069234402365502], [4.1016450331773022, -0.96369319618594318], [4.3838419092928866, -2.4002347577821528], [2.4445397873819887, 0.13849026138180154]], [[-1.8169113179156282, 0.15600501934166466], [-2.3198185309796604, -3.0770381196047514], [1.3226640840768145, 1.5471100527103756], [-4.9835706627149783, 0.47079331962621573], [2.7103747002968479, 1.9524629527065134]], [[-4.7774190378269443, -3.9430073561323207], [-0.46270219023791093, 4.2673014511200087], [-1.988854322760333, -1.1485124251715528], [-1.7799346663903726, 3.1871852917709695], [-1.9184588207612006, 0.059936639327861307]], [[3.0620642098406154, 3.4922636096262174], [0.6709901471805928, -2.4612888244907905], [-3.3517146739254411, -3.694322887316531], [-2.9947748460977621, 2.4546829085794357], [3.1627297145351339, 2.0039998427695851]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-4.0186457334971353, -3.8707966562712026], [-19.526708721660697, -3.3750666302206169], [-17.375089058924559, -7.01664357301736], [-3.4333043826649843, 0.30984179761902753], [-3.2532792468296869, 4.18048519821253]], [[5.5968409410684652, 0.76010320900496997], [-5.8968093706257321, -9.2879605446404252], [-13.582616849447875, 4.4165948756455862], [2.2144497886396404, -2.7036662228940602], [-5.7010653346497131, -13.785959076893311]], [[2.0275954367460787, 2.3122142403226711], [-17.554672361550018, -12.823890711988957], [9.2016824219585196, -5.2533139857306992], [15.748575423991971, -4.9311451640225821], [-2.871167120281032, -13.308910346161429]], [[14.750717700587183, 13.617391236186736], [9.1009115121081514, -1.3629592511797137], [-8.7407987552617694, -2.6834252759918193], [-21.045866895890907, -8.8011467794377545], [1.3930385965489178, 1.6066210706369406]]])+(1.-msk_ref)*numpy.array([[[-3.1018619629096649, 1.5530400743609931], [-9.1215048737477833, 17.024329135224686], [-17.844353500780372, 4.1925817373127883], [3.2116886528881308, -1.7584591085493915], [-7.4486056246851202, -0.42198508906143073]], [[-3.8839560615433864, 0.33348718483319428], [5.262171991438457, 6.9798148404025087], [5.4027531234714035, 6.3195589645643961], [-5.3316135707342385, 0.50367261183022805], [7.7039699610030121, 5.5496813543805628]], [[-5.3939103739403391, -4.4518239062486034], [1.7840336154867342, -16.453367623565775], [-5.6865706267451399, -3.2838488705235389], [-6.6532930999034283, 11.913514754368393], [8.0967730712601487, -0.25296001250582284]], [[9.2850443413911083, 10.589530540541409], [-2.4158231396937553, 8.8615883864448968], [-8.1377506292506272, -8.9695816397478509], [13.636327096718942, -11.177087019990926], [4.1882832262446348, 2.6538211242949705]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[1.7552375426036111, 1.7612701821461032, -4.6608667321674249, -3.2847163875592686, 1.3856742949764911], [0.075246270180300279, -2.3074737915298127, 1.7148027971132462, -3.7021841263316091, -4.9068858234759816], [-0.3093358050865378, -0.45091511192676936, 3.8101814560742646, -1.1200344339986543, 3.6965252925781353], [-4.3887095218142278, 4.6712117394846437, 4.1605659819855152, 0.89906964273436341, 2.3308041998092142]], [[0.98403868293449381, 3.3674511807470626, -4.4103174355578929, 0.0036271827276577184, 4.1882390099605189], [3.8998713466428718, -3.5400359003401261, -1.8536186516670594, 2.2630121000826922, -1.7217445319069125], [3.8953521228432599, 4.1691047409696917, -0.33366409569433042, 4.4485590099391388, -3.4754592682069605], [-0.76217160147654184, 3.9675560036420983, 4.5079638268148123, 1.7460089193795341, 3.6334050880301945]], [[-2.7431570297625338, -4.2372680785598273, -3.289779226708569, -2.0463045400550373, -2.4038510513009061], [-1.5446916464856555, 1.1269716541981438, -4.3499934759579775, 0.52312194902254028, 3.1929380673442065], [2.4140949605130446, -4.9035092521692505, 2.59332628216724, 2.3492148586554773, -4.256220220118589], [0.51049414884359745, -2.7572045253349109, -3.9953182066457558, 1.4809092369528365, 0.072182540067430523]], [[1.9564870370063927, -3.278528094546036, 2.8420569861646383, -0.74882284686590594, 1.4686694586393152], [-0.28660312819679135, -2.4626653179806524, 4.2477925263746066, 0.48462080179316391, 3.1393309943267624], [-1.5162851037176317, 2.6692893914406701, 0.6067051223985942, -1.3388852122053887, 0.16904470942125993], [4.8280790650052268, -1.5043146929946092, 2.997855777385726, -2.4948076767128735, 0.03552907089468782]], [[-3.6565273476146984, -3.2691250969641992, -1.7899894761331869, -3.660539677655068, -3.3105194420866582], [4.0811159471054808, -1.8965843057769574, -4.9348859176378257, 4.844476970260807, -2.3464884496711882], [4.1808399785496171, 2.1430816700819868, 3.7787761615298283, 4.5254364005833896, 2.3860441914181667], [-2.5418309510068737, -2.0074550024573581, 1.9358479046608332, 3.2087133660469807, -1.8031667995834191]]], [[[-4.5378709668746122, 4.6770030276444938, -0.94323224260522398, -0.98603855645810512, 2.2851503970007396], [-2.2138700934927744, 0.10562729187588271, 0.77005243444952853, -2.7408659619605946, -3.9781226084091372], [-2.1869424122969106, -4.6177127197459189, 1.4682033219824326, 0.03597842339238877, -1.5650101171767004], [2.8496407653734011, 1.2214028208271817, 2.4221503255252186, 3.5322248290714704, -2.9894857486529327]], [[3.8563967483921129, 0.85613930527168502, 0.6006104818573732, 1.8275585536609675, 3.9824039602530128], [-0.5795302246399423, -4.8423224547953829, 4.5033971761177014, -1.6196092372835658, -2.9323536254056659], [-2.231214694696908, 2.9845658094239456, 4.0341064181691912, 0.34216014466646705, -0.34353776924711177], [2.2132133929023841, 2.2808376874541425, -2.9183361479383487, -4.3266008796693525, -1.238543622888959]], [[4.7265762246228622, 3.3106782256457059, 0.51820948522321419, 4.2506547412101501, 3.4626919420474724], [0.68631536265615001, -0.80058856524291233, 4.0465088613021223, 4.3483854278282301, -0.40694060201197946], [4.7237381075319664, -1.9047902134034453, 1.2763838127970022, -0.98663384820120648, -0.069172664914356474], [2.0677414758631052, 3.8846804098288725, 2.8122172196291819, -3.3657220165787471, -1.0571662937455786]], [[4.9313342315998767, -2.3451878009965252, -3.5910670400517022, -2.2812724219343785, -4.5729379318120174], [4.0639140737705777, -4.3410381117603851, -4.7308759768887985, -2.9182885987692719, -3.975893586930753], [-2.2029532534723284, -1.6984667541252882, -2.5909710527752949, 3.2670874692683487, -2.4604282542200542], [-1.8693497447445218, 1.26941896416223, 3.0171514121124527, 0.30962538567646281, 1.7500964020265917]], [[-3.1685241000362119, 2.0771588825344978, 1.8163481704809383, -3.6951178526141062, -3.8411816992657641], [0.4703110402970534, -2.414080739874831, -2.4543771282191793, -4.9433009456503774, 3.8040120757864582], [0.33707918601464915, 3.2518836915083646, 2.2855264940258362, 1.2085256687474741, 0.14704201945484474], [4.01904058660263, 1.3603153352246089, 0.25449782135670862, 1.7680107958959379, -0.57312270949105937]]], [[[2.6672788982459341, -4.2302846930051006, 0.031838253445685361, -1.5786413272372393, 1.7969397622853265], [4.6022304027118341, -4.0007557072300415, 1.8502530069897283, 1.0111312034449824, -0.35234955098370868], [-3.8867332670904076, 2.5539671903585255, 4.576302714091451, 2.4096511433416374, 2.8990057185011171], [4.8284073419686937, 2.7354067323583653, 0.040499035207953504, 3.9963344659394409, 4.1045842999169597]], [[-1.6632493409424853, 3.517827239772803, 0.7078335306144945, -1.4418328452595919, 0.23719451621735033], [1.5470921469063743, -0.13403747696967638, -4.5041233681165505, -4.3658656934602549, 0.12045322885745868], [3.6657116257299034, 3.6421422862887596, 4.753021178804465, -4.1223822119416287, 4.4956885991178979], [-1.9455017236250938, 3.0041615469684135, 1.5610489272150199, -3.5763047930921035, -3.0695462382763217]], [[0.46800281184629888, 3.9796392485938128, -3.4688188321043367, 1.0186007982576655, -0.37705689960761735], [-1.3784476736009443, 2.3562262751422605, -1.0697544891117627, -3.4272169136535799, 4.3644296762255888], [0.028713042910899489, -2.0969088422841278, 0.36625780064720459, -4.2852757429101977, -3.3401618032867098], [-0.5118557329239497, 4.1337434445328611, 4.023978978476352, -3.1699789041232398, 3.5292445548777724]], [[-0.60935437730054698, -3.3821869276171168, 0.50421040394352623, 0.12731833682842275, -4.0243044093948379], [-2.2560560304887769, -2.6132865511899785, -1.2992059263118936, 0.2440489238702348, -2.8890148462914156], [-2.8162207095300285, -4.3343719326072936, -3.64454837701612, -2.7583037083014075, 1.2765130971691354], [-2.8302224638971518, 4.4088453483738039, 2.89316899597467, 4.9831642937673237, 1.3303289499515998]], [[-0.38267037467488851, -1.9994336822455292, 4.7317557310515372, 4.9385202381025621, 1.646589764918879], [-3.2881962464987913, 4.3758105127751676, 2.3266225607106383, -3.2196198512369878, -4.4437642201299834], [-0.07141505427705841, 1.8950282022571772, 3.6220399969163548, 4.2872642855030278, -0.047166620394221148], [2.4259492160833185, 4.846597464325562, 1.4660998857023699, -0.84699913295623297, -4.6026143012570353]]], [[[-4.7391002473486274, -4.4786268359753976, -1.1042788437787832, 2.4803599117154445, -4.480418171553266], [3.9621655804545046, 3.946832541513043, 2.874482666298432, -4.0529164320422133, 2.1554163652186187], [3.6967696633335443, -2.2935400155226593, -3.6553388225747039, -3.8850386339102516, -0.036969843296123628], [-3.3602313393484318, -4.106266910168439, 2.0056017225071701, -2.5417342896611772, 3.3115737035343393]], [[2.1082775538249363, -3.7035649098447889, -1.3834086914373334, 2.193387748075958, 0.8134657362445461], [-1.4624249256656863, -3.7970574075498398, 2.9871919348055282, 4.0177557660588565, -3.4396827709062219], [1.0095673365347571, -1.9562528781944035, 4.6942815618633897, -3.3814061022872521, 3.5504255396447224], [2.9529009435368465, 3.2939925102495717, 0.23235390163368308, 4.144942776668266, 1.6629350086749648]], [[3.1037941540523626, 0.2885646744717798, 1.2656940711468039, -2.763289545145736, -2.6386317411614968], [3.3552404497954171, -4.001354953044661, -2.5354214394698307, -4.766856776413622, 4.2795682584049128], [-2.5857384473055536, 0.84342770112277421, 1.1240977282584801, -1.2435382347222736, 2.2926785139764903], [-0.54383070014436452, -1.3433477077744458, 2.5596103002143344, 3.8221029157997926, -1.9348224641620169]], [[4.5374956819015964, 2.7964482595859605, -3.1497895111324126, 0.50208349854558776, -4.4359043247652385], [-4.8660398765050363, 2.9780258112918681, -3.3402957405283216, -2.8222303596152623, 2.9767873923057611], [-3.0347515051437766, -0.91991936953128661, -4.1235418557004859, -1.1486812831681958, 2.09706592512795], [-0.22390326115628323, 2.0778991480050042, -4.7172476728409807, -1.0271395268790453, 4.036446108972136]], [[4.3990254480424298, -1.3370832242418151, -4.7131131567179327, -4.7503122663091668, 3.004517416372682], [-3.82913791100603, 4.1822779126838334, -2.5836234598719621, -2.9232377482109606, 3.8577895992130706], [-3.2334151113509746, 0.27562150990949341, -2.3578550949656218, 4.6899964692365739, -3.1371212007196201], [-2.7110859248270494, -4.7766336891505778, -2.5836007992059216, 2.8282714265394553, 4.8851154929614644]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.010301318263755, 4.3445103056562946, -2.0109439523953574, -3.6496708394310318, 4.8251829782320321], [4.5876013922179357, 2.1873185923174017, -2.7053235888332061, -3.4887938352463688, 4.195233788620305], [-4.0945598875149791, 4.02493854209734, -2.2484753899880228, 4.1820187038118366, 4.7703069642112208], [-2.0013879847521721, 1.2813597029065482, -4.8173281353505502, 0.04884476163450735, 3.5533386082748422]])+(1.-msk_arg1)*numpy.array([[-2.2305870658953819, 3.6849324666665808, 4.5087117673320201, -3.6966645622357741, 4.5284092576590957], [-0.69757483521786945, 1.9172096982459959, 3.7904616317577098, 2.3843748940378964, 4.1765923345245444], [-2.3440609303094018, -2.8141752837108114, -1.6429202328308778, 2.460280378559335, 3.9765473887333851], [-4.0163848909538089, -1.8846327896856963, -3.8499905742100271, 1.9702639666486021, -2.6937245485247221]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.2837938883656843, 5.3019539511330587, -14.030613268095278, -9.8879860715922252, 4.1712971568519306], [0.22651394631818861, -6.9461913965012601, 5.16207312071238, -11.144689755951191, -14.771204862979479], [-0.93119398183818469, -1.3573903558582026, 11.469794260044472, -3.371641133166948, 11.127654961243273], [-13.211338058994064, 14.061754857259752, 12.524557260294332, 2.7064705307341774, 7.0164229553003743]], [[4.2751661991733556, 14.629926358550071, -19.160669549996907, 0.015758332740807467, 18.19584754132519], [16.943031256223648, -15.379722451420937, -8.0530653349242645, 9.8316793906341502, -7.4801368625769538], [16.923397441852668, 18.112718512503342, -1.4496071023715065, 19.326810464000751, -15.099168607613825], [-3.3112623772933985, 17.237087946091599, 19.584895303122739, 7.5855537440121958, 15.785365849571196]], [[5.5163350394717785, 8.5209086172577813, 6.6155616406654723, 4.1150037395828409, 4.8340097340727795], [3.1062883248159561, -2.2662768325307492, 8.7475930734369545, -1.0519689197521498, -6.4208194968987522], [-4.8546096613518159, 9.8606822761644359, -5.2150338037121475, -4.7241394128905467, 8.559020311710313], [-1.0265751213502476, 5.5445837657393513, 8.0343609855493465, -2.97802547409673, -0.14515504241713498]], [[-7.1405336866870535, 11.965548382920053, -10.372572506406323, 2.7329569081062259, -5.3601600959588689], [1.0460070794695431, 8.9879177983121359, -15.503044515462474, -1.7687064084861963, -11.45752478531641], [5.5339415273018977, -9.7420276539436177, -2.214273993351584, 4.8864903163314359, -0.61695754653486456], [-17.620899374017018, 5.4902534683500699, -10.941186811544531, 9.1052268276876553, -0.12966941399641993]], [[-17.643413517150364, -15.774126771582795, -8.6370267514523267, -17.662773743764205, -15.973862061062746], [19.692131200164663, -9.151366309016991, -23.811727529302999, 23.375487815339532, -11.322236125971488], [20.173317899209586, 10.340761195440678, 18.233286413162702, 21.836058689166606, 11.513099817740351], [-12.264799438341706, -9.686337707423986, 9.3408203580155984, 15.482629115875499, -8.7006097482630445]]], [[[-20.817943165339322, 21.456225601029381, -4.3271735493605714, -4.5235518543877671, 10.483359142707961], [-10.156353523097103, 0.4845759112660098, 3.5326936203614676, -12.574000502973176, -18.250040816751405], [-10.032820055353758, -21.184225301968848, 6.7355316039856064, 0.16505466524472906, -7.1796425923949849], [13.073015942547999, 5.6033092812856928, 11.111860205540619, 16.204439543475036, -13.714568982535871]], [[8.4351683071104411, 1.8726494200344603, 1.313726473707346, 3.9974528029713339, 8.7107862243798664], [-1.2676172351648263, -10.591701935369981, 9.8503643719119331, -3.5426013969993497, -6.4139916040991505], [-4.8803773851623422, 6.528196284947831, 8.8238759718484303, 0.74841324597897518, -0.75142654983745283], [4.8410028030612633, 4.9889186798266731, -6.3833309150174973, -9.4636545456375991, -2.7090894937411725]], [[-12.786918154890428, -8.9564558988757916, -1.4019243443314742, -11.499396539381527, -9.3677021916836924], [-1.8567051399722991, 2.1658511305017831, -10.947115874903227, -11.763789671242284, 1.1009060098769936], [-12.779240129776557, 5.1530738960989773, -3.453031237164597, 2.6691638230800048, 0.18713444209526367], [-5.5939097902612458, -10.509317547788296, -7.6079575811857589, 9.1053671649057524, 2.8599769117892881]], [[-17.20440846674504, 8.1818767426116654, 12.528492551288803, 7.9588891621622126, 15.954037665470045], [-14.178158367541748, 15.144987002879169, 16.505050943484783, 10.181307272855999, 13.871073035679583], [7.6856497300501907, 5.9256003411632152, 9.0393638362242434, -11.398194621994076, 8.5839269253889103], [6.5217758653840603, -4.4287410565140188, -10.526219246582802, -1.0802191367838228, -6.1057255384772233]], [[-13.292699364529659, 8.7141671283415203, 7.6200052167003065, -15.501883268220803, -16.114655252989692], [1.9730647674153643, -10.127633088380396, -10.296685858321972, -20.738303154511168, 15.958719992659013], [1.4141259906092851, 13.64241233927922, 9.5883179725240915, 5.0700477199443537, 0.61687564836392894], [16.860814866751724, 5.7068408575126366, 1.0676778592859182, 7.4172186295881168, -2.4043837558825114]]], [[[-10.92133318557295, 17.321154016747304, -0.13036363544723883, 6.4638414552790078, -7.3576774709341999], [-18.844108000045786, 16.38133383857075, -7.5759717441741135, -4.1401372666405729, 1.4427163378418075], [15.914462128898426, -10.457371611671352, -18.737945526244786, -9.8664609144312756, -11.870152528651214], [-19.770203023007834, -11.200286682352985, -0.16582572504554327, -16.36323080132923, -16.806466229363735]], [[-6.6944763774774083, 14.159038441801455, 2.8489864587591165, -5.8032885901472016, 0.95469335029734592], [6.2269508102595861, -0.5394926071407351, -18.12881974269359, -17.572341099228712, 0.48481684334845698], [14.754264006614587, 14.659398863886153, 19.130618133975023, -16.592335050100345, 18.094870315857225], [-7.8305248711354469, 12.091565597079937, 6.283125993247439, -14.394407000003861, -12.354734961188273]], [[-1.052292804881598, -8.9481209114936142, 7.799553776313596, -2.2902988271045155, 0.84780315939291218], [3.099405670477966, -5.2979167929005202, 2.4053166420970089, 7.7060128865007806, -9.81331271832663], [-0.064560570356827565, 4.7148479269241372, -0.82352165114637887, 9.635337047246221, 7.5102716132681824], [1.150895018703783, -9.2946204035564577, -9.0478177029332212, 7.1276195527023063, -7.9354195268919048]], [[-2.5483314031205024, -14.144368991082674, 2.1086173399483483, 0.53244766595467929, -16.829716309921658], [-9.4348685163515515, -10.928813235496419, -5.4333034839395218, 1.0206171642704729, -12.081914122780779], [-11.777487681316821, -18.126424491440758, -15.241569479628486, -11.535277698910035, 5.338401648022101], [-11.83604327996631, 18.43787370911306, 12.099286854454583, 20.839686280702249, 5.5634605509199506]], [[-1.8254551533089378, -9.5379124188943329, 22.571927316781505, 23.558257484718709, 7.8547386227914453], [-15.68570545436638, 20.873959363160054, 11.098703804448903, -15.358574998468498, -21.198119406598703], [-0.34067173076737406, 9.0398662306040816, 17.278242621941676, 20.451566678549138, -0.22499925774486013], [11.572522440305006, 23.119757636800671, 6.9937464949952899, -4.0404458626219837, -21.955883054864596]]], [[[9.4847782935795895, 8.9634699377097977, 2.2100904097548772, -4.9641625251682484, 8.9670550952120021], [-7.9298305863202607, -7.8991432264130825, -5.7529550707080688, 8.1114582502939285, -4.3138244154867431], [-7.3986703865920882, 4.5902634296153604, 7.3157511996991644, 7.7754696422059704, 0.073991000171032467], [6.72512662855965, 8.218233256196541, -4.0139871896241104, 5.0869964677604766, -6.627743820874878]], [[2.7014619000136646, -4.7455988325738359, -1.7726441498584782, 2.8105186732334726, 1.042342214118968], [-1.8738923682741147, -4.8653963516571705, 3.8276673701072488, 5.1481903347482474, -4.407470893421169], [1.2936189024063314, -2.5066636068132606, 6.0150632274689606, -4.3327975186331829, 4.5493722146709823], [3.7837282757228397, 4.2207892643097864, 0.29772892636651349, 5.3111626448762923, 2.130817908668651]], [[-14.951994904653006, -1.3901107252011775, -6.0972636596818797, 13.311672471950578, 12.711154925526289], [-16.163294219665698, 19.275839794826325, 12.213957035329008, 22.963513266203769, -20.61608457836714], [12.456350572862689, -4.0630679947527746, -5.4151476132232137, 5.9905317255117652, -11.044584710692636], [2.6198109326728356, 6.4713467082205067, -12.330482714755581, -18.412323912487714, 9.320674693515965]], [[0.22163289500008987, 0.13659184866270918, -0.15385071787013413, 0.02452414880707875, -0.21667068937663836], [-0.23768055787189607, 0.14546096089396165, -0.16315594923486607, -0.13785116919307744, 0.14540047061378161], [-0.14823171388871018, -0.04493324232772198, -0.20141341903160434, -0.056107063470370568, 0.10243068524472235], [-0.010936501420367502, 0.10149448858485034, -0.2304128381508522, -0.050170385355787626, 0.19715924804327867]], [[15.631226963312701, -4.75110944317505, -16.747286944933947, -16.879467977237926, 10.67606773483123], [-13.606223575586602, 14.861049577674585, -9.1804889892076709, -10.387253551884418, 13.708032725484934], [-11.489418751742717, 0.97937655243240995, -8.378257541658888, 16.66514552681101, -11.147253881354557], [-9.6334062870384614, -16.97299690524504, -9.1804084681881388, 10.049806054603211, 17.358469487021559]]]])+(1.-msk_ref)*numpy.array([[[[-3.9152101601056093, -3.928666487842301, 10.396469048634733, 7.3268458892243071, -3.0908671599182633], [-0.16784335702104716, 5.1470211942789774, -3.8250169398020297, 8.2580440277584817, 10.945236051670935], [0.69000044584436615, 1.0058054164806203, -8.4989414746336873, 2.4983343218348533, -8.2454215063799303], [9.789398695330723, -10.419544488153115, -9.2805046662212085, -2.0054531164224532, -5.1990617012290681]], [[3.6261160912011379, 12.408830185849563, -16.251721906592977, 0.013365923395678173, 15.433377905963013], [14.370762541067037, -13.044793222328591, -6.8304595503466787, 8.3390467600540337, -6.3445123249294362], [14.354109506563717, 15.362869416932783, -1.2295296591849831, 16.392639525606874, -12.806832693993105], [-2.8085508794521714, 14.620175931138679, 16.611542263988426, 6.4339249541110775, 13.388852373434009]], [[-12.368104379729889, -19.104620447143031, -14.832666311385358, -9.2261973592910831, -10.838271521913843], [-6.9645694034093477, 5.0811903587328029, -19.61286677286925, 2.3586060873075887, 14.396037436597181], [10.884458355922092, -22.108509866476933, 11.692560724938835, 10.591932677211179, -19.190070190805162], [2.3016709760452714, -12.431440488318609, -18.013738212539582, 6.6769929029999364, 0.32545026779793901]], [[-7.2324762961752036, 12.119618623402708, -10.506131344609425, 2.7681468814017003, -5.4291783413899557], [1.0594756274309951, 9.1036476096261723, -15.702664099978978, -1.7914805441110762, -11.605053635856139], [5.605197409158964, -9.8674674996906209, -2.2427853256978008, 4.9494095168611851, -0.62490158675101548], [-17.847788783277252, 5.5609468160437601, -11.082067214955591, 9.2224671280982431, -0.13133905730555492]], [[-16.558252291822058, -14.803936353538369, -8.1058049148338789, -16.576421764321651, -14.991386889205648], [18.480963236452627, -8.5885099282113231, -22.347183074922633, 21.937774360645328, -10.625860018481148], [18.93255446365534, 9.7047508747187852, 17.111844952493175, 20.493028091349277, 10.804984605601737], [-11.510450809943951, -9.0905778174619627, 8.7663115728860799, 14.530367311981626, -8.1654772283370782]]], [[[3.1655045919575113, -3.2625596163225845, 0.6579750762075206, 0.68783568353972857, -1.5940634116358399], [1.5443400656619914, -0.073682940724828686, -0.53716920007024904, 1.9119591217689291, 2.7750382230374848], [1.5255559928889872, 3.2212001895602191, -1.0241816903982239, -0.025097642769344335, 1.0917116746038353], [-1.9878376873354737, -0.85201987147316216, -1.6896311142011633, -2.4639911530919982, 2.0853900285027387]], [[7.3935212463016819, 1.6413985791164638, 1.1514962406851565, 3.5038129831912324, 7.6351034949303376], [-1.1110809671063779, -9.2837475723680658, 8.6339567411064877, -3.1051305370888524, -5.6219368093145485], [-4.2777064515418912, 5.7220385148809987, 7.73422794867039, 0.65599274770780358, -0.65863394291435773], [4.2431941811603764, 4.3728441345120519, -5.5950623655692633, -8.2950011669417396, -2.3745478455034434]], [[17.915905829011169, 12.548998789405742, 1.9642531709515072, 16.11194370640607, 13.125200948927535], [2.6014520494340148, -3.0346002393772133, 15.338136581333275, 16.482388124277239, -1.5424927383307925], [17.905148055071692, -7.2200342204533401, 4.8380838698036523, -3.7397977462001335, -0.2621963323243009], [7.8376947286531609, 14.724732045097157, 10.659601471172758, -12.757640167003927, -4.0071482748301159]], [[11.758149535936408, -5.5918069145000571, -8.5624500931062588, -5.4394086893213593, -10.903598396606156], [9.6898946890258362, -10.350662287743138, -11.280181906100658, -6.9582940684624841, -9.4800208500439656], [-5.2526664303185218, -4.049781486894374, -6.1778463294163508, 7.7899613383492579, -5.8665833579437878], [-4.4572305995449879, 3.0267707081640132, 7.1940200785519188, 0.73826299616375879, 4.1728859231382582]], [[-13.233633467967524, 8.6754458663831517, 7.5861458456583675, -15.433000898392871, -16.043050040669353], [1.9642974857469373, -10.08263111308456, -10.250932699752589, -20.646152836851297, 15.887807676168521], [1.4078423444365566, 13.581792498719214, 9.5457124353010645, 5.0475190441668492, 0.61413457130811344], [16.785894106147573, 5.6814826016352873, 1.0629336498316262, 7.3842603374956131, -2.393699915202296]]], [[[-6.2522642556170007, 9.9160450729591574, -0.074630805991319743, 3.700431458148592, -4.2121362908924977], [-10.787908479278915, 9.3780151450303002, -4.3371057848721106, -2.3701531494121104, 0.82592881627297221], [9.1107395979204409, -5.9866547082114945, -10.727132397370648, -5.6483691007825119, -6.7954460414820046], [-11.318081005927882, -6.4119600498265505, -0.094932206146188705, -9.3676514860575324, -9.6213956925967139]], [[4.6806751859286386, -9.899782470533248, -1.9919676268370703, 4.0575703563719783, -0.66750694497061047], [-4.3537884814470145, 0.37720495477902038, 12.675392657337889, 12.286311326536811, -0.33897649949382208], [-10.315955054340471, -10.249626801831813, -13.37583472434555, 11.601106130855236, -12.65165573889807], [5.474982865042521, -8.4542371737529454, -4.3930653076317867, 10.064348555736306, 8.638241155964721]], [[-0.76889128860402689, -6.5382298408826465, 5.6989926432889906, -1.6734798606352017, 0.61947440929383557], [2.2646795728576454, -3.8710918205589544, 1.757521294323374, 5.6306440097416619, -7.1704098198385369], [-0.047173239144457972, 3.4450539633905657, -0.60173235111543055, 7.0403662212865354, 5.487619407548606], [0.84093813991123512, -6.7914107423550432, -6.6110764802249262, 5.2080224792311238, -5.7982672858168973]], [[-1.4991826180617775, -8.3211281346362735, 1.2404989634877339, 0.31323880592977688, -9.9009171757839329], [-5.5505303847419984, -6.4294176254456996, -3.1964108482131572, 0.60042877880645962, -7.1077865396973827], [-6.9286925533491779, -10.663770219172029, -8.9666108606830299, -6.786200491641404, 3.1405801259392296], [-6.9631407948840183, 10.846995702706666, 7.1180069126526924, 12.259981335093233, 3.2729822125953647]], [[-1.521706879159054, -7.9508427880790356, 18.816050896437218, 19.638259757033719, 6.547742230003287], [-13.075668197657686, 17.400617868168187, 9.251924868362071, -12.802970912150613, -17.670839005704732], [-0.28398534760168959, 7.5356694492618992, 14.403213691625609, 17.048509599326966, -0.18756030116401892], [9.6469020204159222, 19.272724491005658, 5.8300156721120731, -3.3681321904165493, -18.302513881010597]]], [[[19.034050630166487, 17.987889156231848, 4.4352088635530462, -9.9620800735414345, 17.995083849381427], [-15.913581972794701, -15.851998586857807, -11.545028750229642, 16.278072321952767, -8.656981722978637], [-14.847649821149247, 9.2117394651431734, 14.681247618305926, 15.603810470008961, 0.14848512003548089], [13.495982381468522, 16.492348376224101, -8.0552684555487311, 10.208583197814365, -13.300554588155268]], [[-3.9733290076968255, 6.9798598678228387, 2.6072173814189803, -4.1337304705188203, -1.5330841998122877], [2.7561339673632195, 7.1560588945873924, -5.6297598694191553, -7.5719942576632944, 6.4825389361668186], [-1.9026637058290574, 3.686818319162191, -8.8469969555047268, 6.3727088156138612, -6.6912483893519763], [-5.5651339428833717, -6.2079662937954394, -0.43790178183024397, -7.8116950682798905, -3.1340218444651065]], [[-11.949578237389781, -1.1109712767663371, -4.8729102437487102, 10.638638702624197, 10.158707332283155], [-12.917644105920568, 15.40517885329055, 9.7613486436088674, 18.35235365780164, -16.476297456547336], [9.9550686494988518, -3.2471886993503127, -4.3277656582860526, 4.7876104823505301, -8.8267906685033388], [2.0937430695218429, 5.1718760128182621, -9.8544755294760851, -14.715060199489862, 7.4490482497935826]], [[8.9400642408743423, 5.5097412404594142, -6.2059167763119083, 0.98923702543323744, -8.7399024505856477], [-9.5873830289530879, 5.8674969477378376, -6.5812643355127607, -5.5605387831316779, 5.8650569354338975], [-5.9792615383173926, -1.812483986009594, -8.1244659332539779, -2.2632053413899755, 4.1317734279662153], [-0.44114852747133648, 4.09400981764409, -9.2942231115555582, -2.0237359985302765, 7.9528643218267563]], [[-11.849762838976858, 3.601733904560763, 12.695828610225941, 12.796032764915109, -8.0933423209531679], [10.314642790563616, -11.265904682149177, 6.959569938001481, 7.8743972835299951, -10.391822546443597], [8.7099296610169183, -0.74244842734465233, 6.3514121511729851, -12.63355862167683, 8.4505403900757923], [7.3029187088664722, 12.866935427775118, 6.9595088964090825, -7.6185841715603662, -13.159155525768746]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_expandedData_rank4(self):
      arg0=numpy.array([[-0.59047674729401578, 1.4509076062248569, -3.2603618949792192, 3.4382348130687941, 3.382893103206392], [2.2404309676889769, 4.521004853567641, -3.1992324316557785, 0.87741007988628361, -1.818256202158012], [3.9635513689260868, -0.087418465546853952, 4.7140746108938902, 1.0816280303081633, -3.5861590954964795], [4.345188225099724, -4.6113432855820342, -0.19052796536299432, -4.6343219244970992, 3.6445164152134506]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.8805315874295712, 2.4646221980262615, 0.15974634058175141, 4.3236730873510307, 1.0813481424910805], [-3.7102111119889791, -0.28781729685155177, 4.2828014103097907, 0.93981314042503783, -0.22533220342697025], [-4.9323022935239091, -2.4531723578621634, 1.4162652863149123, 1.3649283776135066, -4.8429947040320691], [2.0213297321542338, -0.060678969064159816, 4.755385872429704, -4.2945676330941822, -1.2874811992557564]], [[3.5197251365418261, -3.4026113966130955, -0.53386681270127845, -1.2955712593900115, 0.73223857321303232], [-0.20520840924666128, 2.7686013088941728, -1.9305608879490221, 4.4937657712073094, 3.2067752066735], [2.0748533422724904, 1.6820697430199267, -3.8624476822730704, -1.8106622719336682, 1.3502146140394764], [0.36422295030587115, -4.3201081828531258, 2.3645481773517041, 1.9402444633361569, -4.215784085648405]], [[-1.2128202881214065, -4.3211192365702713, -0.56603855907148848, 4.9946470988075031, -1.0945437563649039], [-0.73059955569886625, 2.0150265871254689, 2.2398550134767543, 4.5309457230281431, 2.7913053290960734], [-3.3089882108613047, -1.568731299700703, 1.2557888182432659, -0.89132926698916926, -1.1744849118689373], [3.5924858177366499, -0.81519682225033474, -0.30001930688123224, -3.0771003462548476, 0.84353001789538951]], [[-2.4275932097271968, 3.3662968647197218, 4.9430824390074761, 0.43796169631652759, -3.8662575018438683], [2.2783485282876192, 3.8622566840576908, -2.1320124481572589, 0.43971331106581601, -0.28134371824068882], [-4.5544253048312822, -2.925152663529401, 0.016296178506289394, 2.6579578002619195, 2.651698645623334], [-4.1733825502080357, 0.53021360239755921, 1.0030510719892032, -2.8647892359950058, 2.0489271313400632]], [[-0.97008127454793058, -0.18982224809282489, 1.1687754054872652, -1.1426342527780617, -2.4754886651433941], [4.3119514263855248, -1.2788146714292492, 1.7525451683639472, -3.1311880421423743, 1.495085375626819], [4.2206766453822784, 0.77131988375516247, 4.2987066723886915, -4.495700944365824, 3.7520724605743165], [-1.459799037798569, 0.43340351722532588, -3.4590424471638603, 3.9918338864500793, 3.0497953172807843]]], [[[2.0424743103310554, 3.7374941372273423, 3.2862805508639177, 0.614638703545344, -2.4012856745455071], [-0.4499501742044707, -0.31722358975503617, -1.1318973055859525, 2.0624279134640275, -2.4081972941272767], [0.44512211911148825, -4.3810818359784376, -0.099788323619712571, 2.1626383985026623, -4.3512498618108868], [-4.9011683842962075, -2.2275908440594283, 4.2420391541929909, -3.7088853885251538, -2.6091099808298646]], [[0.49830630913827179, -2.8895858774185723, 4.2342726505085508, -2.0159259393846263, -1.3467746417636639], [-4.2681142058096029, 1.2949726899504919, -0.1986884012779484, -4.7390920129983218, -3.948457485530664], [4.3237405194615466, -4.307865787401254, 0.83990858227088339, -4.4105100061583302, -3.0657407826280023], [-0.55490887899206864, 0.85420832899700549, -1.2356989974702923, 4.6282299603886283, -0.74411991225455143]], [[3.1532895795780664, -2.1298117899101365, 0.33537247246501067, -4.4277602630641315, -3.6389862256283392], [3.1982476316711246, 1.9537452176759418, 3.9481369169373259, -2.4009947200468451, -3.6393479416534733], [-3.2196831011784099, -2.5665178868409222, 4.1775643353321623, 0.93320759714230306, -0.51661317608751745], [4.6957379551416594, 4.4196882267234869, -4.3084113982061485, 2.3400100169050067, -4.3714332964636338]], [[0.092014816876146988, 3.607815474898926, 0.85789962907725581, 0.039592325286498031, -2.6590801880352655], [-2.3153638614536343, 3.8546152658101427, 4.8250591704542085, -4.2853862396215963, -1.9369180018114305], [2.5014483964948164, -0.47322545839155605, 2.8821254499080222, -1.9838638289397084, 0.39789218404961524], [3.3690249439593529, 4.8031235025607515, -3.6221594674005217, 0.10415226077423867, -2.3476458982542114]], [[2.9862507055059959, 3.3497865044367856, -0.7227025434875376, -0.95980792675464599, 4.7450575623170277], [-2.2061358708734788, 1.1600890425444259, 4.0555530154191075, -4.1862134960817867, -3.2370508491511387], [0.10177093680210891, 4.5394961185644185, -3.1430715528558837, 2.6322309632008514, -3.6787435101200385], [-2.5153282103150341, 2.8603192197331042, 2.7205350067594658, 4.2807726399967816, -0.17246045672774635]]], [[[-1.1394777517413477, 1.8421630498094288, 1.8469973440824292, 4.0297690891143958, 4.0717531120632628], [-2.0070779363212932, -4.7804052449945029, -0.35057047448626921, 1.3396957432621264, 4.0077964409783693], [-1.458875895790154, 0.22875793306298053, 2.9334856578323976, 4.9509474417445816, 1.2196680555708994], [1.9076336505672415, 1.7958783082504564, -1.401075246686819, 3.416181678859191, 3.9708535697640759]], [[3.1148253790196598, 2.0908140794162069, 3.0705149647570913, 1.3852658608705379, 3.8161494222058803], [3.041272018243502, -1.0817131717006867, -1.3226420744808931, -2.75639647972885, 4.3924845311074208], [2.598503291632877, 0.73587855645741485, -3.9703025062376174, 2.7724877466582605, -2.6246757737230064], [0.33389375303792601, 3.7205416727789409, 2.9987627158582333, -2.1353685898369248, 4.4457223164737645]], [[-2.1286795267896119, 4.9347722222698795, 1.507091502882286, 4.0642178543506642, 3.1442105390804116], [4.6447354741348565, -1.6736058783319696, -1.9431832073471345, 4.7198141027805445, -3.5701144121427584], [-1.7069429149920978, -4.567665073455645, -4.9019020806346782, -3.4880359609485279, -4.3429148182156503], [-2.1564433345820033, 2.175071514060992, -0.45304147715252441, -1.0250710124624729, 3.0349769744838451]], [[2.5372272352077738, 2.1612837824184936, -2.9957223868172567, -3.4343242843737141, 4.7886068798641208], [1.3891730923113954, 1.3144518777626715, 1.1569806631954673, -0.52519673551626322, -3.7811005187899385], [0.33624406001864493, 1.6988327712811433, -2.9009170667825215, -2.2917960429156459, 3.9454376634240909], [-2.2598726348326235, 1.7298793509048993, 0.94809598677921247, -2.7814241376516238, -1.0438338303621233]], [[2.988248405306666, -0.17810470847121262, 0.022415599955452947, -0.73814156781750917, -4.3239669854669627], [4.6627447137202491, -2.4460954988212436, -2.7657903016422614, 3.6299057751856498, -2.4271339949429027], [1.2779892962098369, 0.3026981998898215, 4.908159460345372, 0.83789957114923208, 2.2711927089001946], [-4.9161318449511064, 3.0091889688550673, 0.89993797750229021, -4.7409406457908885, 0.33777525374492878]]], [[[-4.3578550581699504, -2.4984342303140741, 1.5723798913921803, -2.7684039722418996, -0.84388041828836791], [1.1267282089814614, 1.6589757831190681, 0.64625542800636282, -3.8950314984168877, 0.78158258080189036], [4.8173955507317885, -3.6542856397110048, -3.4568531346881892, -4.0895614467262469, 3.3301595346266755], [-3.3006507271689323, 2.0977149345378399, 2.5508159653489564, 1.284932809188744, 2.4504468915716853]], [[1.4919741161424591, 3.5416486571983032, 0.69841329716803902, 4.9135164995225615, -0.87864199146451938], [2.3616096815095773, 2.6586886846244404, -2.2403952764342439, -2.3138512574804326, -2.1636771718379286], [1.682303436078155, -3.8761750555377459, 1.3962481902815496, 3.5839489389500372, 0.20129116865688435], [4.0939918712743655, -3.7473383224066223, -3.752931002536676, 3.4802248928587005, -0.97400463395920944]], [[0.73728117798300374, 2.6702614014482222, 4.0773172045253165, 1.8863430881328007, 2.3399531210604412], [3.6833889617529927, -2.6847725280702903, 3.0780385064545932, -0.88964524854867122, -4.8409146625440345], [4.6426701733152171, -3.2614276007884335, 3.8923565852314095, 0.12794429646727146, -1.5697037752501686], [-2.690251415730903, 4.0312961252076622, -4.1346986124444731, -3.8883218989580417, -3.7879751721325783]], [[-1.6249567033465286, 0.62119942619752155, -1.9380631005372759, -4.5310062050756708, 1.5125332399731475], [3.9589975684931549, -4.0009180352132612, 0.78307195988320721, -3.3000048682732022, -2.0857753012233129], [2.007514685935325, 1.4582988689237695, -2.6508633297213082, 1.1122449561479826, -0.43506459017891608], [1.9609399210666592, -3.5331573292562735, -0.92910705966339258, 3.3271690466709245, 2.7120491112765786]], [[-1.7708357088806115, 4.6522116689800015, 4.1087767179447816, 0.025697273333799231, 3.624772301974172], [0.25670685650150027, -3.9394399309255022, 1.5518265241677884, 4.9190349300654788, 3.4452417273068789], [-1.0092328508854664, -4.9376612249747476, 0.15418207412535345, -0.71809886327762129, -4.2444252275660466], [1.0081464831954055, -2.814236739899052, -2.6600351579544359, -2.1834514775193927, 1.2354720834894337]]]])+(1.-msk_arg1)*numpy.array([[[[-3.7681488011888389, 2.6092448288795014, -4.300764369033967, -3.0601241840468143, 2.089353277464455], [3.468925736988675, 4.6746323167306958, 3.4577216607540073, 0.71440062941862781, 0.66187794116274645], [1.4497667483832988, 1.4729629308477277, 2.5632582065474852, -4.4440082056871946, 4.1758826859746545], [-4.6626790508781735, 3.2613392072993772, -2.3270162283973783, 4.9748432257244275, 1.7556866727424989]], [[-4.6279866737315167, -3.103758448079085, -4.6347715879975198, 4.9484043462961615, -3.918495496624371], [3.1150019625444649, -0.47751340595484137, -4.869785853671055, 1.9221068418347551, 4.1939638053290391], [-2.9287760214974714, 0.34570831870826524, 1.4933376435996131, -3.7663130975571657, -1.2656623234565121], [-3.0931523739524214, 2.5604084825501765, 4.3211812355940609, -2.6040744497547639, -4.4722451040620967]], [[0.7227534502797619, 1.4496301928063007, -3.2767923740140512, -0.46900959818048094, 4.8633952661227493], [1.204696142349694, -1.9516970225674903, 3.8562790703827083, -4.4859445196582666, -1.9124108844305097], [-4.5123427768164639, 3.8723665553236231, -3.8395691085840919, -3.8602346341183482, 3.7338011082823748], [3.5649231269293811, 2.6576773320207545, -2.0881089506326878, 3.4554084280732091, 4.6776959706255177]], [[-2.5052664981740005, -0.84588209548686777, 1.679714392736674, -2.8417748559438682, -4.1858745007622744], [-4.8556760934773644, 3.4427163035471864, -4.4280022167546722, 0.5040913061845238, -1.2613990684151064], [-2.9125694703397009, -0.91148797664845382, 0.051351318327438555, 4.3733850061810031, -2.6393966228189383], [-2.5277438225664453, 4.554805236639357, 1.6464890244037544, 0.85326260251795283, 0.88898264975777153]], [[-1.5722006361799723, 0.69834689726172794, 2.817393142451408, 0.8736326860805228, -4.6081570498427782], [2.9518201676429809, 4.0930453144975338, 0.071318199111334302, -0.52304958010940972, -1.6207453754445766], [-0.66578700177922112, 1.9760508716426965, -2.6663093488444112, 3.1690562780456624, -3.4290697013620139], [2.9451708666518384, 1.8708332973383213, -0.36957944425651057, -3.8099938874962893, -0.58204733104030382]]], [[[-4.5343274751437503, -2.9056213305015142, -4.399201619520686, 0.5046376144545448, 4.7625600327839805], [2.9714061317939375, -2.2315117326129377, 3.1338618312540021, 2.3611146404779397, -3.9684322967423125], [2.2657535339836734, 2.56468926750152, -0.94108412209502301, -1.9324714599497272, 0.93284301162151806], [-0.90706274626103678, -0.38961655088788838, 4.4504749071566554, -3.3458159797545282, -3.6918044506223602]], [[-3.3300368944495129, -1.0496537652439342, -1.0588711811446574, 4.876845571641903, 3.518823127506403], [-4.3136713910212272, 2.5325351468762305, -0.066376036151385875, -2.8688896553664489, -0.29774203536987986], [1.7558451132566901, -0.87447015457462385, 2.4364862397592102, 2.3210305420110702, 0.40573943016639369], [3.284192659697668, -1.4634313045322287, 2.5272448774771288, 2.3037504466001, -4.2599777525896929]], [[-2.0479986187947743, 0.12569132592175158, 0.32199236037697343, 2.5162969349646245, -4.6244925655457969], [0.51211939668903206, 1.0092866243203567, 1.1723710989746108, 3.7072374878794321, 0.76549177697726645], [-1.5060870400393513, -0.097666365599505589, -1.347404818585721, -3.7539641435511273, -2.5443880595834543], [3.6995541509910481, -3.1622754752359814, -2.3978194598799618, 2.961698539993149, 0.68227002516021606]], [[-4.2105106786128719, -0.035507909112419966, -2.4754971854359731, -1.545994588813925, -0.32118391280618752], [3.2487638235298082, 4.9655646554065065, -4.6826328166889724, -4.8267117998578453, -0.43730560333956259], [0.22377211580643852, -2.4355676309354504, -4.3737170686105911, 0.35241167240357729, 2.6726230479236488], [-0.55609470713294584, 2.4107232311833657, 2.2100921035717791, 4.9028350691320224, 0.39304322013430593]], [[-1.8042667928493628, -1.1487088902515952, -0.9106933215296662, 3.5847806444260399, 4.41467936190638], [-3.3036226230309995, 3.2726664225809348, 1.4546698657878192, -4.1683222120150454, 4.419056951713344], [-4.8288309200791657, -2.1202904370277276, 2.8999734885293336, 1.8159264254704643, -0.56219123653723191], [-0.6529706057979805, -2.0542386037997131, -0.30087061471097432, 3.8553422765636718, -1.7629632870121394]]], [[[3.0445559209252462, -1.3156750391177852, 2.9588549680367571, 1.2769209899536937, 2.1031273613016124], [3.2997897803928335, 4.1630982893390431, 1.2706790139721775, 2.8642733908763907, -1.9489207036717526], [-0.38608996762135739, -2.871945770218586, -0.71554222153691782, -0.70066809505533456, 1.9252465184026342], [0.065338701147323874, 4.8716776071777765, 0.096702108933723885, 3.2975533934861154, 3.8619888344382218]], [[-4.48137792022548, 1.9377719042206234, -3.6915989320005904, -0.19766548421891539, -0.48626185421854018], [-0.52490720008298108, 3.9649435730815075, 1.0275075119402253, 3.396165389045235, 0.31216623850650382], [-0.014517189122481788, -2.5135512500672519, -4.7000708535788256, 4.503682622659273, 1.0071022979502704], [1.052188557459691, -1.7704385947326209, -4.5505234522080142, 3.0991143255598317, 0.62482727297027907]], [[0.61190300368132178, -4.0933733505904799, 4.341292666734887, 0.93028185915314143, 3.3835585092308342], [-2.7288335763378599, 1.5483551942336149, 1.5554521456238817, -0.05298771383026768, 3.8780579537746007], [-4.2441776738278501, -3.2328953054398268, 0.050143952094253486, 2.6016727448452865, -0.94051364833571238], [-1.4153612719846609, -4.2474188111562485, 2.1669217952362452, 3.4711079655926547, 0.65270093969469478]], [[-3.2363509872793994, 0.52918705874995897, -3.5275287405464804, 3.84191200429769, 0.62379322722212116], [-0.087562859492332556, -4.532421719233227, -4.2845943333027758, -0.083841220958468021, 4.4812973197687498], [-3.863434996456534, -4.1983227311929312, 1.7515211729438924, 0.8360488645720725, 1.7295490374518945], [2.0559569935199251, -3.8440719689722034, 1.9772657109223211, 1.1161570641574716, -1.6358517562778463]], [[-0.92355542076452046, -3.8080497945307981, -4.1673431070904075, -1.1852638807772689, -3.0674221487335895], [-2.2856715214723557, 1.1001467826773172, -2.130028534444163, -4.5405209704592462, -3.7519159907156761], [3.0564315652883494, 0.55258807528670317, -3.8338819174530814, -2.4477384873708763, -2.4883263508227751], [-2.0691533787643457, -4.2074130550299769, 0.78586763428574269, 4.8051621711772068, -1.1492729664611923]]], [[[0.52196843384745506, -1.6255320767710879, 1.9731899258800336, 0.89470935624269821, -2.653211222988221], [-2.8727130982323015, 2.7531286739814682, 1.6235602085251362, -0.40227569676558694, 2.5728660183303926], [-0.010700529058008001, 4.7194421311021912, 0.25792607382293475, -3.0395902668231489, -4.1714624664341322], [3.7383729905952947, 1.1559092811784391, -1.7647460452464792, 1.2628008063133276, 0.74306425572876389]], [[2.6122385370995733, 4.0064060548836231, 0.31470855588620861, -3.7097661839383624, 0.21551025048135752], [-4.0815396594335773, 4.5434666836418209, -3.6247056546076162, -3.9455985861929332, 2.628903394748642], [-4.6952218170057485, -1.2656345230662245, -3.8037965162802312, -0.73625963883233414, 4.3078394811779415], [-1.7103704751354476, 2.0606790698402495, -3.4396607670714086, -0.67486102064687792, -1.1462616948211313]], [[4.5960422502178329, 4.4712685125157172, -3.4895532310746145, -2.744185383470723, -4.8196095348801204], [0.59171258491527379, 2.9707781673792786, 2.5861392010429149, -3.2484584025633731, 2.452327596255655], [-2.6296006281235771, 4.301239619988305, -4.8294449843455105, 2.5803213682581401, -4.1419598635077328], [-2.6572840029083569, -1.0598029120595545, 0.68634158824694236, -0.272427145827141, 4.4681962814489271]], [[0.86989857697883544, 4.7882346405621199, 3.8984702536610953, -4.7466953975524415, -1.8877074251504777], [2.6244249998103752, 0.79981001222694825, 4.772948758046871, -4.9484101740200312, -3.8160206460006418], [-4.2351015105623091, -2.4450547429098588, -3.6675643111983849, 0.54834444148950645, 4.7643327099449095], [-0.78708877890398465, 2.0536050781299053, -2.757430976243346, -2.2575957957855932, -3.0507471466888458]], [[-0.96369764629244159, -4.5831933451078433, 1.4081510724196704, 3.9889686433449985, 4.1329755959588805], [-4.6535651786111156, 3.4922497044625906, -0.77840130315169631, 2.9927253998994772, -2.4627465310623551], [4.4603674694586264, -2.4498335200699319, 4.7606807225881127, -2.9194261812360933, -4.8351550599777031], [4.5377012194724635, -4.2566487280815082, -3.5305603978774949, 3.4248727043052778, 1.7942638926831478]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.1104101749290651, -1.4553020987991745, -0.094326499578834599, -2.5530284209817116, -0.63851093387055913], [2.1907933891813656, 0.16994942125986046, -2.5288946460659485, -0.55493780622235045, 0.13305342654015087], [2.9124098149508115, 1.4485412344220414, -0.83627171956865742, -0.80595846870252152, 2.8596757599990008], [-1.1935482054511162, 0.0358295202821593, -2.8079447820802073, 2.5358423270236128, 0.76022771073873763]], [[5.1067959724293583, -4.9368747563733235, -0.77459141925930597, -1.8797541946552847, 1.0624105154460253], [-0.2977384418372841, 4.0169846976786499, -2.8010654766054501, 6.5200389380375956, 4.652734538815869], [3.0104204961042229, 2.4405277843483022, -5.6040547208555678, -2.6271036626529396, 1.9590366535458359], [0.52845384896044656, -6.2680778222158455, 3.4307409358047094, 2.8151154497900954, -6.1167131960689742]], [[3.9542330528487515, 14.088412502575407, 1.8454905490856248, -16.284357079820492, 3.5686087556395507], [2.3820189518893313, -6.5697159020339022, -7.3027379362177749, -14.772522783580026, -9.1006655322372669], [10.78849907362766, 5.1146317530053977, -4.0943260111413284, 2.9060559779712465, 3.82924585288551], [-11.712803868401833, 2.657836656173139, 0.97817151591364626, 10.032460715956667, -2.750213127617267]], [[-8.3466354856534632, 11.574119071403681, 16.995478125664508, 1.5058151510661482, -13.293101139127975], [7.8334972262625451, 13.279345388134795, -7.3303594211503151, 1.5118376138762364, -0.96732576649335422], [-15.659183636592369, -10.057361721287695, 0.056030088260307614, 9.1386830405282833, 9.1171625971495178], [-14.349069172379092, 1.822998866125904, 3.4487251149992515, -9.8498180833027824, 7.0446925924145827]], [[-3.2816812532178607, -0.64214837390835, 3.9538422584201238, -3.8654095332102942, -8.3743135323791851], [14.586870741680556, -4.3260933322571553, 5.9286729631160826, -10.592474432605764, 5.0577140059127039], [14.278097914528001, 2.609292715121295, 14.542065154731004, -15.208475718773609, 12.692860049607493], [-4.9383440970361061, 1.4661577693269476, -11.701570838208784, 13.503947323617542, 10.317131545020315]]], [[[4.5760226955748822, 8.3735976066001321, 7.3626847146695109, 1.3770555853631932, -5.3799147875196685], [-1.008082304204746, -0.71071755416864668, -2.5359377756784811, 4.6207273659509687, -5.3953997940675498], [0.99726538006071963, -9.8155114173057711, -0.22356885045137342, 4.8452420399186584, -9.7486749385534921], [-10.980729426035371, -4.9907635103711696, 9.503995887203132, -8.3095016800609169, -5.8455307991576211]], [[2.252845242177504, -13.063831776609875, 19.143167204277876, -9.1140109563908016, -6.0887746920753454], [-19.296165040046212, 5.8545778165037179, -0.89827122652519986, -21.425457992269056, -17.850995456189615], [19.547651874052725, -19.475882133359057, 3.79723077699978, -19.939937144550456, -13.860228958041457], [-2.5087457352109213, 3.8618800013533661, -5.5866011651118601, 20.924250114344158, -3.3641697349391544]], [[-10.088106289388366, 6.8137629516033522, -1.0729344905946467, 14.16543423319149, 11.641962751378836], [-10.231937547708547, -6.2504850635812517, -12.631007669283349, 7.6813401766081535, 11.643119965017494], [10.300514596944023, 8.2108872600061336, -13.364999306923171, -2.9855480102452163, 1.6527656275598834], [-15.022757156646184, -14.139609912740998, 13.783609474056529, -7.4862359364818838, 13.985231174866387]], [[0.080734727826021882, 3.1655336640460368, 0.75272978208308805, 0.034738705292509962, -2.3331037602080564], [-2.0315235906438471, 3.3820782883053657, 4.2335555522042725, -3.7600410828499653, -1.6994713787025479], [2.1947960373999331, -0.41521278725155836, 2.5288059212460889, -1.740662120633498, 0.34911461299310076], [2.9560164452182582, 4.2143089760855155, -3.1781192276527501, 0.091384243446261795, -2.0598481751319335]], [[-5.4297688664850163, -6.0907700875973925, 1.3140583820115856, 1.7451767157020579, -8.6277303422797136], [4.0113202300189696, -2.1093390966619521, -7.3740344234664201, 7.6116086528082834, 5.885787783169917], [-0.1850456370398656, -8.2539669722519768, 5.7149093448066246, -4.7860702743523058, 6.6888982034242952], [4.5735111188683231, -5.2007931614314824, -4.9466296492283881, -7.7835414027024745, 0.31357729507222826]]], [[[-4.5163786027752382, 7.3015078778572162, 7.3206688515407592, 15.972196789615394, 16.138602621247401], [-7.9551565020476067, -18.947381752819407, -1.38950408405512, 5.3099528971510521, 15.885107070016915], [-5.7823295538523363, 0.90669381874447863, 11.627021094826642, 19.623334510207844, 4.8342169912934567], [7.5610039671152585, 7.1180559270907615, -5.5532337119739967, 13.540211569742564, 15.738682102243441]], [[-0.27229325508029645, -0.18277575856632283, -0.26841970665771725, -0.1210978159317441, -0.33360192678675138], [-0.26586333314543059, 0.094561705631894588, 0.1156233406188274, 0.240959950696646, -0.383984257647703], [-0.22715717047299525, -0.064329374234341138, 0.34707775285212106, -0.2423666245603199, 0.22944512869686687], [-0.029188479546255697, -0.32524404403746021, -0.26214723515944316, 0.18667064550049331, -0.38863822315354152]], [[-10.03475411196853, 23.262884443546859, 7.104541790031301, 19.159026200336108, 14.822043073583961], [21.895629572937324, -7.8895029796875074, -9.1603106220704849, 22.249555830056689, -16.829785708268542], [-8.046656257809456, -21.532313953844032, -23.107932143407872, -16.44286176539233, -20.472824481825253], [-10.165634773384381, 10.253449401313455, -2.1356713251265798, -4.8322612342126376, 14.307107900061848]], [[2.7443360968620114, 2.3377051205142925, -3.2402573046032193, -3.7146614111466332, 5.179491427387549], [1.502568555593875, 1.4217479954793051, 1.2514227158367459, -0.56806751056073324, -4.0897443065359358], [0.36369100034078677, 1.8375051442237815, -3.1377132130313137, -2.478870839766893, 4.2674959685930416], [-2.4443415867613298, 1.8710859949900303, 1.0254871947230741, -3.0084663114597077, -1.1290399299036089]], [[-10.716334198293351, 0.63871182023478801, -0.080385907661258069, 2.647093097192792, 15.506433533558843], [-16.721344365086001, 8.772087621550801, 9.9185640464703475, -13.017419611477218, 8.7040886519531977], [-4.5830729385500515, -1.0855239027252948, -17.601440690864649, -3.0048411681894183, -8.1448583906477214], [17.6300309304313, -10.791430390727271, -3.227320763402544, 17.00176741811195, -1.2113157984510077]]], [[[-18.935700485451342, -10.856166998746806, 6.8322865894608844, -12.029236342504806, -3.666819256938846], [4.8958461465539473, 7.2085620385345681, 2.8081014761800303, -16.924645003313593, 3.3961234270434275], [20.932490422687568, -15.878558932823271, -15.02067753674619, -17.769914244136679, 14.470169997563406], [-14.341948674861285, 9.1149662331696604, 11.08377549703067, 5.5832749125312411, 10.647652979489507]], [[-6.8800048227357191, -16.331757755262025, -3.2206234684570467, -22.657911318669903, 4.0517198477703387], [-10.890192947994716, -12.260126214295845, 10.331231715034756, 10.66996246001794, 9.9774581985219566], [-7.7576786542705847, 17.874373816094554, -6.4385797172608896, -16.526818875496108, -0.92822267903288447], [-18.878801926828473, 17.280263411834021, 17.306053179800152, -16.048511691999423, 4.4914697289335876]], [[-0.14047268274153338, -0.50875947180526759, -0.77684295111774038, -0.35940111055849, -0.44582650720043421], [-0.70178860452330982, 0.51152424723569501, -0.58645241394374348, 0.1695022991008337, 0.92232962115040118], [-0.88455850197320851, 0.62139316495693231, -0.74160278065139285, -0.024376966485708973, 0.29907246652102554], [0.51256812855412393, -0.76807464851153873, 0.78777571401824131, 0.74083406008485009, 0.7217152023919583]], [[7.5305724766773467, -2.8788381203121918, 8.9816083178787238, 20.998141396214582, -7.0095659555381893], [-18.347269230698533, 18.541542168704673, -3.62900755214566, 15.293284911985662, 9.6661542080337401], [-9.3034693227799856, -6.758226420722746, 12.284954047772843, -5.1545011856879102, 2.0162293688384962], [-9.0876268688208306, 16.373788473669965, 4.3057812168030951, -15.419172459495178, -12.568508656701921]], [[-6.4538398096615355, 16.955061794645179, 14.974504194996603, 0.093654134491258173, 13.210542155955917], [0.93557235241756143, -14.357353495005334, 5.6556572408931371, 17.927503549631986, 12.556240029548263], [-3.6781656918247512, -17.995387387183424, 0.56191910008150769, -2.6171230949614102, -15.468877415010542], [3.6742064069453662, -10.25653199485888, -9.6945417982098459, -7.9576247516414895, 4.5026982888152043]]]])+(1.-msk_ref)*numpy.array([[[[2.2250042474458307, -1.5406983994504988, 2.5395013555051769, 1.8069321745117171, -1.2337145272253027], [-2.0483199857815695, -2.7602616851786306, -2.0417042392900884, -0.42183695992390891, -0.39082353380343848], [-0.85605355392039206, -0.86975036029162656, -1.5135443682768515, 2.6240835102420901, -2.4657616254957118], [2.7532035596384925, -1.92574496694858, 1.3740489734444725, -2.9375292462234293, -1.0366921557884439]], [[-6.714781066424333, -4.5032667402026023, -6.7246253501404603, 7.1796775047172421, -5.6853749210101485], [4.5195800408611211, -0.69282783277421722, -7.065609335777542, 2.7887994367948843, 6.0850539853836478], [-4.2493834065196561, 0.50159082914902908, 2.1666949457605833, -5.4645723206699932, -1.8363590920152786], [-4.4878783065800416, 3.7149161423746948, 6.2696347225995481, -3.7782714263249959, -6.4888144383855728]], [[-2.3564378087568936, -4.7263190424370416, 10.683528993993907, 1.5291410222871551, -15.856428605888931], [-3.9277453975454035, 6.3632386029234427, -12.572865337481669, 14.62580257488467, 6.2351515751407414], [14.711870446617118, -12.62531636036908, 12.518384814766902, 12.585761906758512, -12.173542856875033], [-11.622939521570721, -8.6649899024705022, 6.8079908552078585, -11.265881970479933, -15.250981698925271]], [[-8.6136944900367975, -2.9083412684545307, 5.7752525011199412, -9.7706892406097658, -14.392019431657811], [-16.694954585579758, 11.836867046375451, -15.224511373991707, 1.7331842778889504, -4.3369861901973641], [-10.014097748403298, -3.1339096930063501, 0.17655789037037684, 15.036724579204607, -9.0748653540722781], [-8.690976809267541, 15.660489931361484, 5.6610158830406636, 2.9337171846669063, 3.0565310946113127]], [[-5.3185866889899298, 2.3624329023922823, 9.5309398306198521, 2.9554059884774757, -15.588902702405049], [9.9856920870249759, 13.846334765524945, 0.24126184390683306, -1.7694208171871213, -5.4828083526451126], [-2.252286256523389, 6.6847688652650579, -9.0198395072204853, 10.720578626673589, -11.600176243151559], [9.9631982125608971, 6.3288290588146801, -1.2502477530622009, -12.888802045269708, -1.9690039019159316]]], [[[-10.158847692955028, -6.5098440092332401, -9.856107541481645, 1.1306057388846527, 10.670186982927058], [6.6572303152520513, -4.9995479905073097, 7.0212010951999533, 5.2899143587906012, -8.8909986107985688], [5.0762643826877607, 5.7460092574099635, -2.1084340103420836, -4.3295689030464972, 2.0899703712290973], [-2.0322114663602355, -0.87290898613339329, 9.9709818029164943, -7.49606973323068, -8.2712330178263258]], [[-15.055112962365563, -4.7454897672333756, -4.7871617492578968, 22.0482424994929, 15.908616438302515], [-19.502129295502847, 11.449603690858076, -0.30008638160099677, -12.970264056261712, -1.346093187018335], [7.9381842791465207, -3.9534838131319199, 11.015366115602161, 10.493390345710781, 1.8343499330660349], [14.847850954544377, -6.6161800306530303, 11.425686357228058, 10.415266950487673, -19.259380095548174]], [[6.5520236012344819, -0.40211576626668433, -1.0301284020634085, -8.0502187620148575, 14.79482659564515], [-1.6383889827675422, -3.228942501362067, -3.7506876417755013, -11.860314403073975, -2.448986119071483], [4.8183225033903483, 0.31245740430788854, 4.3106611941887092, 12.009803835321675, 8.1400887989371018], [-11.835733622517321, 10.116854258204642, 7.6711817813033161, -9.4751620219336505, -2.1827403916391672]], [[-3.69434451088377, -0.031154997370923299, -2.1720261832316474, -1.3564712356749882, -0.28181000259346617], [2.8504981259349571, 4.3568364809807294, -4.1085892337692043, -4.23500558590134, -0.38369634436088507], [0.19633991000604992, -2.1369915896275202, -3.8375434425696211, 0.30920955363648156, 2.3449864019846114], [-0.48792310140985751, 2.1151928628563166, 1.9391570891509593, 4.3017969096764004, 0.3448600831768035]], [[3.280619286446099, 2.08864706417401, 1.6558737801351961, -6.5180496401036425, -8.027018130325251], [6.0068323239156349, -5.9505460204520579, -2.644962505561065, 7.57907771458936, -8.0349777101422664], [8.7800517696063221, 3.8552312375019873, -5.2728947816122673, -3.3018194857743008, 1.0222077026327039], [1.1872678538190524, 3.7351320820712433, 0.54705986124532269, -7.010000005803886, 3.205518930786698]]], [[[12.067253788155282, -5.2147456023571799, 11.727573659015841, 5.0611419377414171, 8.3358533319129151], [13.078886301244326, 16.500653923683615, 5.0364015452950746, 11.352694719386683, -7.7246473229665673], [-1.5302874196942595, -11.383104589031362, -2.8360883516970641, -2.777133987319405, 7.6308134735349435], [0.25897329837632804, 19.309144448896038, 0.38328377624230087, 13.070022266858755, 15.307191131514877]], [[0.39175518132166315, -0.16939704644677214, 0.32271391404989647, 0.017279613321993459, 0.042508265149752776], [0.045886581985749658, -0.34660928313864575, -0.089823130031680215, -0.2968875670536687, -0.027289093565771798], [0.0012690703971408371, 0.21973079335425574, 0.41087298198135302, -0.39370502418290448, -0.088039337535523163], [-0.091980709159083962, 0.15476902529645414, 0.3977997776269972, -0.27091981889471367, -0.05462144143488705]], [[2.8845564139838298, -19.296467384928235, 20.465177538714762, 4.3854180932089903, 15.950347262839056], [-12.86392507956908, 7.2990619097823624, 7.3325174681459666, -0.24978803645657591, 18.281454539463958], [-20.007370216314559, -15.240109679051937, 0.23638233145739987, 12.264479432329784, -4.4336515108185663], [-6.6721186375053714, -20.022649179504782, 10.215031018715793, 16.363061932271876, 3.0768809283213447]], [[-3.5005279437568966, 0.57238355602028845, -3.8154739634927259, 4.1555197138257984, 0.67471223967983551], [-0.094710443240842129, -4.9023943767001743, -4.6343373293997994, -0.090685014683939275, 4.8470967932067248], [-4.178799585440907, -4.5410235463381987, 1.8944943963343464, 0.90429388662846721, 1.8707287187004724], [2.22378071329925, -4.1578559921622276, 2.1386660163007805, 1.2072667668191883, -1.7693831130189566]], [[3.3120166723697633, 13.656272406760122, 14.944755387546824, 4.2505448466128577, 11.000263838408317], [8.1967817160453649, -3.9453013910794499, 7.6386212024639715, 16.283030576504927, 13.454967655643706], [-10.960849857621357, -1.981668752252304, 13.74891050933385, 8.7779796398818632, 8.9235341755666582], [7.4203132092330302, 15.08845259580638, -2.818246364550117, -17.232076025502753, 4.1214757018830248]]], [[[2.2680510926277058, -7.0632428395076321, 8.5738816318193187, 3.8876805596323263, -11.528702164830856], [-12.482479128528743, 11.962862296368693, 7.0546747008238739, -1.7479636208296154, 11.179587127608432], [-0.04649581286519381, 20.50686437710479, 1.1207373389216182, -13.207591836527675, -18.125789590595044], [16.243934299765517, 5.0226433978600395, -7.6681537360963059, 5.4871071942391083, 3.228754054485115]], [[-12.045928638392754, -18.474913660502803, -1.4512291861010864, 17.107005383383452, -0.99379174653131042], [18.821380503365809, -20.951484584877385, 16.714762082586063, 18.194509548042749, -12.122776017817966], [21.651279600167737, 5.836275259942254, 17.540611525069178, 3.3951459419745378, -19.864926666895094], [7.8871054063735997, -9.5024985924472656, 15.861456582914689, 3.1120158362610191, 5.2858061699333065]], [[-0.87567457865636178, -0.85190169228124168, 0.66485747714250909, 0.52284405769154518, 0.91827039852479675], [-0.11273779488358512, -0.56601631977557898, -0.49273184012018634, 0.61892217000672223, -0.46723698731811258], [0.50101245739363698, -0.81950643333507067, 0.92014432669986812, -0.49162338027688102, 0.78915918540931396], [0.50628691446576235, 0.20192209252048332, -0.13076726635269595, 0.051904989804092923, -0.85131634634696118]], [[-4.0313900473818443, -22.19022077439352, -18.066766168541381, 21.997714549786753, 8.748243907410826], [-12.162430315819517, -3.7065770750956393, -22.119381073917815, 22.932525760865538, 17.68466814409436], [19.626823782869693, 11.331170801662777, 16.996673696889776, -2.5412046673709368, -22.079451533296375], [3.6476227846003861, -9.5170670378359983, 12.778822828491979, 10.46242569306165, 14.138144387997086]], [[-3.5122118912153693, -16.703523380342581, 5.1320296985339136, 14.537861700442575, 15.062697403148734], [-16.959994682713923, 12.727561373938233, -2.8368963269598986, 10.907036846159883, -8.9755201589667344], [16.255882460326042, -8.9284584784350169, 17.35037904106261, -10.63989664051886, -17.621801986191116], [16.537726581701484, -15.513426163290513, -12.867185324967062, 12.482004790857067, 6.5392242101085172]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[2.7642028629859983, -1.7802757423295681], [2.9931106274579786, -4.4304778561898059]], [[1.935964836678906, 2.1207938684273717], [-1.6648395460686647, -2.3700538220306653]], [[-4.5691859195473059, -3.6241430951217621], [-4.6695264009257587, 3.1508645799035584]], [[-2.5894027333622138, -4.1251826135598932], [1.1371651434600594, -1.9410972479862885]], [[-1.5355714415958035, -1.855485648988775], [-1.2271467141455306, -1.8053102059251005]], [[1.2912476306876801, -4.7697773527572016], [0.84627897131291441, 2.6270934177409577]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.77184838429346492, 2.9976676161285321], [0.20028577493283173, -0.024211732208451409]], [[4.0954840916768358, 3.7705450941428076], [3.3016709224095244, -3.2329040745336393]], [[-4.4445436844603945, 2.5684812963042436], [-4.5773407742499366, 2.3039887343388905]], [[-1.4094311395758821, -0.056549658394193791], [-2.9520662861176303, 0.82553233859153607]], [[-1.3251267375584641, 4.1969066759562654], [0.42708248430664231, -3.4796143018780978]], [[-0.49585408360886785, 1.7867139409445274], [-0.72843833970188676, -0.73769323873512338]]])+(1.-msk_arg1)*numpy.array([[[-4.3324900843217486, 1.6447001276896911], [-3.2638945680563047, -2.4233822407337478]], [[3.0820845153794778, 4.2576654624463277], [2.8428529697177733, -1.8523555738177189]], [[4.0679072080882488, -4.8142410252743026], [3.5158702460025886, 2.9521213970942632]], [[2.9078237578175514, 0.76084527655707923], [2.5019964006270801, -4.1578952624961776]], [[-4.3733139433364734, -0.057483025947407285], [-4.5605156040917851, 4.9037336218114085]], [[0.81676459375422361, -1.9276646266926791], [-1.6032256017468685, -4.0144358093682584]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.1335455136551129, -5.3366749405605294], [0.59947748148011548, 0.10726954340954148]], [[7.9287131906642028, 7.9965489162869732], [-5.4967523197323818, 7.6621566581069622]], [[20.307946421849337, -9.3085437549504171], [21.374013591394032, 7.2595564955252394]], [[3.6495848453036088, 0.23327766761047949], [-3.3569868817565598, -1.6024385505637155]], [[2.0348267746897948, -7.7873001073820332], [-0.52409286728600624, 6.2817832118634733]], [[-0.64027041062676149, -8.522227691372775], [-0.61646204878779998, -1.9379890517930516]]])+(1.-msk_ref)*numpy.array([[[-11.975881494940626, -2.9280197407323003], [-9.7691975185516942, 10.736741354654503]], [[5.9668072454472156, 9.029630806571161], [-4.7328940476448933, 4.390182407486491]], [[-18.58702433722182, 17.447498369999778], [-16.417448935938427, 9.3017347456797221]], [[-7.5295267866283515, -3.1386257064624319], [2.8451830958556457, 8.0708790514465569]], [[6.7155359965202166, 0.1066589297058636], [5.5964217383706538, -8.8527603545941922]], [[1.0546453465147267, 9.1945310801099058], [-1.356776113028868, -10.546297890734946]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[3.8529321243463581, -3.7864321960314582], [3.3718915207662388, 1.0500178166491114]], [[-2.2169233403476927, -4.6278633407774574], [-4.2423554221865309, -3.3378544842268223]]], [[[-4.944869899874532, -4.6420744223957389], [-2.0747988807396665, 4.9604191303967564]], [[-0.87973979631576604, -0.93056155537730945], [-0.61473721014612792, -1.7528331027670441]]], [[[-3.6557890219137379, 0.080347829589877406], [0.93701820162962512, 3.1203247305796857]], [[-2.34901927828954, 2.9760761100276243], [4.7800910040980522, -2.0505296822686594]]], [[[-3.9045317852395298, -1.3959134990983646], [2.7151124882279181, -3.0230615438018282]], [[-1.8098350214491132, -1.5111878653143354], [-0.028990047838929556, -4.4746987237134999]]], [[[3.8445040998613091, -2.9673633525534693], [0.73350679156344878, 0.66124067110710083]], [[-1.0083535340110075, 1.4742460439961196], [-4.4465538324393439, 1.8263628118873854]]], [[[2.9887899707567644, 3.8491791382273419], [-4.7361232395535708, 2.6567639492379014]], [[1.0730159988339718, 4.7127735095302334], [-3.6628042834146282, 2.9294951177433015]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.5997513419266047, -3.3908923046257722], [-1.2770602422669031, 2.7299313261078906]], [[-3.4276243427930799, 4.3096089055949811], [2.0663542776150647, -0.073237560576039584]], [[2.7008967236864496, -0.96209783380606773], [3.0888412855256711, -4.4385732600920038]], [[-4.9468032400538506, 2.8707835299189597], [3.4529803278787181, 1.3498765790165299]], [[3.6839165681643458, 3.3354687771531157], [-0.53844408470151528, 3.7007424959957049]], [[3.9505884830109732, 3.046780563700219], [-0.96412842437588075, -0.44024712386666298]]])+(1.-msk_arg1)*numpy.array([[[3.2797758983520033, -0.47553028611273618], [4.2438328214408454, -1.7658133787088848]], [[-3.3511204108531767, 2.1237635115368727], [4.7067309027311097, -0.66046953793609298]], [[-4.271474458319199, -3.889021562385333], [-1.541385238480907, -4.1796943620148044]], [[2.8368187961234845, -2.2690629471601067], [-1.8937170149858695, -0.52925484472940543]], [[-2.8141792596835637, 1.0813626040277304], [2.1297069058515161, 3.5295065529848344]], [[2.171266171907563, 3.2659648283976406], [-3.3297260773444473, 3.2518409542337583]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-13.869597584967927, 13.630214378778343], [-11.433721009799131, -3.5604973341954271]], [[2.8311446581115765, 5.910060279151379], [-11.581338963510676, -9.1121135184804984]]], [[[16.949156440754727, 15.911287291260761], [-8.9415717337541665, 21.377466459841575]], [[-1.817854091305289, -1.9228698505380313], [0.045021853666422619, 0.1283732205435888]]], [[[-9.8739085917757041, 0.21701118969461705], [-0.90150318202471957, -3.0020576640622174]], [[-7.2557477272764466, 9.1926267575199656], [-21.216784111595953, 9.1014262167426239]]], [[[19.314950486116153, 6.9053094201746976], [7.7945002130819923, -8.6785552898776714]], [[-6.2493247257697462, -5.2181019706594336], [-0.039132986602339777, -6.0402910052960115]]], [[[14.162832349854831, -10.931519018235424], [2.4465890010896421, 2.205547612661507]], [[0.54294199567609513, -0.79379906178432047], [-16.455550728440844, 6.758898471057857]]], [[[11.807479236610376, 15.206522772527039], [-14.429928233560736, 8.0945767628774732]], [[-1.0345252242859091, -4.5437188981837737], [1.6125390510597835, -1.2897017999679197]]]])+(1.-msk_ref)*numpy.array([[[[12.636753919417369, -12.418649057288025], [-1.6034365396110786, -0.49931527277462251]], [[-9.4082520343858125, -19.639878338734253], [7.4912079617351557, 5.8940281044311673]]], [[[16.570854450483047, 15.556150345589831], [-4.4063821566924473, 10.534757151066096]], [[-4.1406984856817877, -4.379902829587909], [0.40601520113733597, 1.1576928694636377]]], [[[15.615609432108258, -0.34320370187454491], [-3.6440839904851399, -12.135010158868603]], [[3.6207436404625706, -4.5872797845922602], [-19.979319419746314, 8.5705873521223239]]], [[[-11.076449158429083, -3.9599536520047436], [-6.1607611444096504, 6.8595169360253587]], [[3.4273153744355018, 2.8617621733859315], [0.015343123267690698, 2.3682559782298571]]], [[[-10.819123701598125, 8.3506924027010605], [0.79318681419707671, 0.71504093399741864]], [[-2.1474974849230244, 3.1397119808228138], [-15.694140889794493, 6.4461595126843347]]], [[[6.4894585584407567, 8.3575924524453331], [-15.468011923338656, 8.6768976155658013]], [[-3.5728493527252749, -15.692244851300929], [-11.910856976150521, 9.5262521991055138]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_expandedData_rank4(self):
      arg0=numpy.array([[[-1.4250890218213694, 2.7878079140370353], [-3.3764042479372245, 3.912303953133268]], [[-4.8361547671249392, 4.4985007823472234], [4.8668968586927761, -4.3040466080083162]], [[-4.282923723504771, -0.048735344137170955], [4.018252890306476, 0.1715083087703011]], [[3.6417404571112542, 0.93104431249593489], [2.2730283708529431, -2.3114989353687196]], [[-1.9467728922332119, 4.4655355332681488], [-1.4114207496585838, -2.8006464246386917]], [[-3.1167646102779489, 3.3612350679915775], [-0.81806581558226021, 4.3801991600963355]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.0451148876017076, 2.9681329358350128], [-4.217218124471164, -4.5862078979056946]], [[3.2303852637561992, -3.5978872074974442], [-3.4224526969361211, 0.34982687157105286]]], [[[3.4794832683473658, 4.7439875733707861], [-2.3572423140191265, 1.0322111364279118]], [[0.64111769074683878, 3.3804184335960148], [0.33095901046849008, -3.3045073538487459]]], [[[4.294044794259694, -2.2595527213889066], [-3.0405949996905344, -3.3107166802432553]], [[4.4879081972073678, -2.8853420984890055], [1.0538749508081571, -4.226936315765947]]], [[[2.1494656948889856, 4.8034426083474742], [3.9062575750815061, -2.3641529961115371]], [[4.2872689848496197, -1.2024795350844339], [-0.56563330267849921, 0.93479924967627337]]], [[[1.9269627571974999, 0.52121917911165294], [-3.273687216173121, -2.8910977023370532]], [[-0.62386449451467918, -4.1847283554171879], [-2.6138200459292116, 3.1187347760068054]]], [[[0.55022084102164559, -2.7922634871774807], [-0.55991361686531071, -4.831230926270111]], [[4.9368765284314335, -4.545095680750876], [-2.5208601312036949, 1.6679169076483644]]]])+(1.-msk_arg1)*numpy.array([[[[0.29306793122890618, 4.5202332311775937], [-1.1946093085496101, 0.08401269941511913]], [[3.6604812426082098, 1.036460988605481], [-2.7342200304223327, 2.5459964437336646]]], [[[-1.3534414455308177, -2.6506568891655178], [3.122664794517771, -0.71690594726182688]], [[2.3594587520951995, -0.24248663723498964], [0.53376838222151157, 2.6152460905576893]]], [[[-4.0048936910779487, 2.6580953864636641], [-4.8165291716141798, -0.32996375515245457]], [[-0.062034743749803667, 1.7046730487552306], [1.9242124311023456, -1.332614440739416]]], [[[3.8430144142440827, 3.9628872470567913], [-1.0165686062236468, 2.4469622202028276]], [[2.235687478155425, -4.7177402597478135], [0.97227457671197115, -4.4936159819860935]]], [[[-3.7696172444778178, 0.6226433429597158], [-0.93772336162002556, 1.4939215458513919]], [[-1.471331464474912, 1.7414712644526738], [-1.478875357249164, -4.8603727019812704]]], [[[3.1704731362980869, -1.3023989801208522], [-1.8293318254086719, -1.028932575217957]], [[2.1371507102249403, -3.0009510320754185], [1.1883705660413524, -0.10818071837912768]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-5.7646488183273759, -4.2298536621649081], [-11.756794062621134, -12.78546667320065]], [[-10.907086527020242, 12.14792165099337], [-13.389675215634801, 1.368629052559674]]], [[[-16.827319595349579, -22.942658118138599], [-10.60405639379702, 4.643402604768478]], [[3.1202536751481564, 16.4521478555356], [-1.4244630063966934, 14.222753667471231]]], [[[-18.391066319127006, 9.6774919549463139], [0.14818444369167941, 0.16134891675232721]], [[18.033550084858632, -11.594034226776399], [0.1807483104684913, -0.72495469879678498]]], [[[7.8277961822499744, 17.492891280231007], [3.6368988984237984, -2.2011312008998707]], [[9.7450840360410815, -2.7332700986169751], [1.3074607769504436, -2.1607874704101837]]], [[[-3.7513588600550611, -1.0146953688066132], [-14.618766588626759, -12.910299519936013]], [[0.88053529253328189, 5.9064124325204599], [7.3203857662805873, -8.7344733998198105]]], [[[-1.7149088451336345, 8.7028280194060681], [-1.8820012840536826, -16.238902810944527]], [[-4.0386899236601783, 3.7181874049728734], [-11.041869429418762, 7.305808237991843]]]])+(1.-msk_ref)*numpy.array([[[[-0.41764789144221426, -6.4417347538233249], [-3.3303412845569138, 0.23421126830908373]], [[-12.35926441703689, -3.4995112847487611], [-10.697099833757457, 9.9607119514824571]]], [[[6.5454522988283328, 12.81898695055038], [14.047310021146325, -3.2250019646267054]], [[11.483242388787303, -1.1801574530339458], [-2.2973639949625833, -11.256141065171832]]], [[[17.152654199632334, -11.384419790023809], [0.23473520672533998, 0.016080897160148089]], [[-0.24927128837207019, 6.8498074051882574], [0.33001841977315283, -0.22855444897409788]]], [[[13.995261069614385, 14.431806814576959], [-0.94647041908644591, 2.2782302580122682]], [[5.0817810662079506, -10.723557456721913], [-2.247411648955794, 10.386988558316718]]], [[[7.3385886656442718, -1.2121451816034416], [-4.1874369916898821, 6.6711597469142729]], [[2.0766677585854421, -2.4579486775826744], [4.1418069817661394, 13.612185430215343]]], [[[-9.8816184690508138, 4.0592710497027662], [-6.1488142825566738, -3.4584842544214784]], [[-1.7483299387823725, 2.4549754535772026], [5.2052997552575384, -0.47385309178287327]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[4.2789345789340523, -0.46738415127566491, 3.7206318757369239, 1.1609776795318698], [2.4905592841487358, -0.33055026754904571, 4.7729502040842604, 3.53486938605249], [3.2084965098284588, 1.6984063208088163, 1.2675559145950572, -3.2794599138671652]], [[-4.8443369302383257, 2.3451486855371231, 4.5681354537543601, 2.2629582592258703], [-1.5274089877671515, 3.1601609730214477, 3.4430286023938255, 2.9622041604422584], [4.9424449801146224, -4.9949512914471192, -1.4458142362408868, -4.4375964162832124]]], [[[4.4317298707596038, 2.099266599308935, -0.44647873168381302, 2.4044065646189487], [-3.465599029013775, 1.5295552454764749, -0.42034517572639629, 1.6189442246188266], [3.6979605682643335, -1.7950389741491133, 0.26474138160844429, 2.1855581840339404]], [[-0.088035051244102824, -4.2364229305860572, -0.34305028732196519, 1.2356275900121014], [-1.0972871438731846, 2.0396145633696205, -0.29388049465003796, -4.4821925725999971], [1.3639715966177253, 3.8373053900139542, 2.1781486789421383, -0.080860241300621638]]], [[[4.1078285450085854, 4.0567381546898567, -4.885420862824108, 1.0881736718198844], [3.1715201320574664, 4.7961593390493, -1.0892529114421534, -1.8660377996366719], [-0.64691047920921374, 1.6626224462592845, 2.543466145793392, -2.8523457053132861]], [[1.9300811565247624, -3.8404261438896037, -4.1954611411478684, -0.57995740798697781], [-2.9395019691216175, 2.4284499615902373, -2.480006896545305, 1.9228580859635338], [-4.9020326709282021, 0.4669289858515242, 1.2950043295309595, -4.4964017715994693]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.8945946414363766, 2.9958451916114113, -1.8341297045638507, -0.50406437625297507], [1.2490655637633523, -0.79450973475085807, 3.2918209895668209, 2.9776725675914264], [-2.3034096643916548, -1.3906614109441309, 4.3036976792917283, -2.9114858595165627]], [[4.0322913401048499, 0.44609529697885453, -4.2257467616229683, 4.5422889174310122], [0.20948365993480511, -2.457469024454416, -0.45661156826599214, -3.3592070142064601], [1.1212071528232155, -0.85798303021232769, -4.4470702615039981, -2.6584420808205422]]], [[[3.4396806123964954, -2.9856575665573217, 3.6139848006147997, -1.1372077809368708], [3.2106080066051614, 2.2074036020625787, 3.4105694127301565, -4.4233693514541024], [-0.55512509748334793, -0.76512067225989355, -4.6043948665324566, -3.5322036076778085]], [[2.644780545246384, 3.0597017738279053, 1.6021909533350076, -0.099386176273002391], [-0.20851684982186836, -4.7130453472754059, -3.6837396508679676, -4.8461017472225034], [4.684947651599721, 4.8533194042953802, -1.4474384849651289, 2.0258093380915128]]], [[[1.9016465201482635, -1.4119169398332101, 0.63510495624503349, -1.8460569289076911], [-1.6982164890430185, 1.6299998986806905, -4.5744812061132016, -0.74486983246449512], [2.8147159864507927, 4.513809169482716, -3.9287925307802629, -3.4237002746815701]], [[-1.8218226685288776, 0.96416321528420834, -4.6848297046562379, -4.1767907091253074], [1.6492041458182216, 2.0146473336646284, -4.5532878212568448, 3.7129155358113692], [2.9628046623291731, 0.1753155505052213, 3.1947876405626854, -3.901592527539377]]]])+(1.-msk_arg1)*numpy.array([[[[3.4007018978517163, -3.2592827559531568, -3.4943232477325701, 2.1830176640586929], [3.9012352784777011, -0.4110774948369933, -2.6576644826774198, 3.6350848399026088], [-2.0617002339230117, 0.57606296669331591, -0.50485582875293478, 3.0110291955237631]], [[-3.6249560796681157, 2.4376194572685712, 4.7900054570308725, -3.0991802442593275], [2.8401787792429003, -1.9664416094766723, -2.770614142250106, 1.507471475308253], [-3.7914632910533221, -0.41044000904566502, -1.0817473225976668, -4.5626423343122635]]], [[[3.1652641141996636, 2.1746701989473856, 3.8549440830753685, -0.44280330656435218], [1.2413939326032111, -1.2490395139274968, 3.1392686678047905, -4.9181153968765665], [-2.9260797798541294, -0.24238099597080431, 4.7833839855465019, 4.0996065881537493]], [[4.9064254982209956, -0.49036892423013079, -3.6825152023075258, 1.9033307857816428], [4.9979866045526382, -0.53377903730434539, -1.9166587950046332, 3.6704402994104228], [-4.7722392850391957, -0.85696042025228092, -0.76099969318301319, 3.9111159530461332]]], [[[-0.22013887418591871, -1.7488121093094797, -1.3016711613553222, 2.7626004838749925], [4.1736396744666191, -3.3853485457336472, 1.3036165855928363, 0.40046802088988809], [1.4325203291308881, 3.5920131697512367, 4.0365425305555771, -0.053589319122167467]], [[4.024287866695845, 0.9875971813227018, 3.7372253364357118, -0.004453618324037123], [-1.6868485285787802, -0.29992389699410893, -1.4789485691594653, -2.0867176692223754], [2.8718954455387227, 3.5411607755642933, -1.5774208970328916, 2.4471693468059197]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[12.385781103239326, -1.4002105622345811, -6.8241214430362103, -0.58520748987685833], [3.1108718363412917, 0.26262540539221746, 15.711697663961809, 10.525683600867247], [-7.3904818689057663, -2.3619081304524188, 5.4551774480152524, 9.5481011660756554]], [[-19.533777852350113, 1.0461597993342533, -19.303783600357555, 10.279010221490646], [-0.31996722497477886, -7.7659977034899352, -1.5721266897237118, -9.9506569932691935], [5.5415046641397092, 4.2855834447987791, 6.4296374936459637, 11.797093050745724]]], [[[15.243735315830236, -6.267691206447779, -1.6135673501030736, -2.7343098538203594], [-11.126679990234699, 3.3763457584184824, -1.4336163991211299, -7.1611882648925436], [-2.0528307209473149, 1.3734214266336793, -1.218973858436631, -7.7198365024344442]], [[-0.23283339083017163, -12.962190755399373, -0.54963206688622768, -0.12280430146872784], [0.2288028585904717, -9.6127959281243491, 1.0825792307590365, 21.721161257464573], [6.3901355284229338, 18.623668709561976, -3.1527362238768055, -0.16380743190713232]]], [[[7.8116378579812809, -5.7277773210743259, -3.1027550033224789, -2.0088305467180216], [-5.3859277835918808, 7.8177392367068066, 4.9827669720962184, 1.389955263187783], [-1.8208692676327172, 7.5047604433129429, -9.9927507958855415, 9.7655767747678954]], [[-3.5162656030572452, -3.7027976189541341, 19.655020978780492, 2.4223607133684042], [-4.8478388341161978, 4.8924702400557409, 11.292185198672721, 7.1394096605345183], [-14.523765252316005, 0.08185991220140465, 4.1372638264606767, 17.543127552887306]]]])+(1.-msk_ref)*numpy.array([[[[14.551380943364366, 1.5233371046585762, -13.001090459642372, 2.5344347819959441], [9.7162577424612167, 0.13588177590175959, -12.684900234982681, 12.849550116275248], [-6.6149580048545005, 0.9783889838158063, -0.63993299175357188, -9.8745495462038804]], [[17.560508607228194, 5.7165800660531048, 21.881393751939587, -7.0133155305762953], [-4.3381145942811425, -6.2142720299936629, -9.5393037379639498, 4.465438275906136], [-18.739098710155357, 2.050127853244212, 1.5640056790271699, 20.247165271526171]]], [[[14.027595523742086, 4.5652125131627628, -1.7211505449235101, -1.0646791771383053], [-4.3021736074532804, -1.9104749403351893, -1.3195764398207745, -7.9621545177822455], [-10.820527645496153, 0.43508333436067292, 1.2663596850972876, 8.9599287300588859]], [[-0.43193742016125808, 2.0774101550553428, 1.2632878982191016, 2.3518080318312107], [-5.4842264464259998, -1.0887034981073589, 0.56326863475130728, -16.451620248189109], [-6.5091988370567435, -3.2884188396627008, -1.6575704763819528, -0.3162537797180211]]], [[[-0.90429275124697051, -7.0944728092194147, 6.3592114482217772, 3.0061891123096398], [13.236782251524653, -16.236671043557397, -1.4199681612612762, -0.74728846452621955], [-0.92671241259500337, 5.9721617232873676, 10.266809272523298, 0.15285526424877754]], [[7.7672021799408855, -3.7927940347833853, -15.679383674729298, 0.0025829089393718781], [4.9584945713672273, -0.72835017613533815, 3.6678026511512849, -4.0124619433872235], [-14.078125301520723, 1.6534706096714324, -2.0427668911502042, -11.003456586382054]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(-0.182467669988)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(3.06751268781)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-0.559721892803)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([-2.2685969675991959, 1.7977246762021899])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(1.36567345051)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-3.098162648561114, 2.4551048616187092])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(-1.98293101785)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([4.8485190339404181, 1.5067061264581927])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-9.614278783037193, -2.9876943129387681])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[-0.58441876752599153, 1.6222370274331643, 1.9195498780333695, -3.1156097624319976, 3.7440099440500081], [3.991542817572503, -1.0736281091848507, -2.6316405494587047, -4.3738235512447252, 3.2758456627505854], [3.9097262493485587, 0.23510252490026229, 4.8959785064312467, -1.1992773587997929, -4.1319141021007884], [1.8386363636487051, -3.2568963542085672, -4.1118682932879391, 2.5502555145807388, -3.5144688286672867]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-3.82840790274)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[2.2373934281061936, -6.2105850559428957, -7.3488199227669373, 11.927825036349175, -14.333597257738942], [-15.281254066920452, 4.1102863378072998, 10.074993476719266, 16.744780648776509, -12.541273423431548], [-14.968026870556821, -0.90006836428233861, -18.743802805667542, 4.5913229180065205, 15.818652601926333], [-7.039049984858206, 12.468727740857823, 15.741909069050402, -9.7634183660276737, 13.454820237603933]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(-2.99652269288)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-4.5497584716043127, -3.072603261970992, -3.0383468695271731, 2.0091201472147375, 1.985177329181802], [4.4772495188765973, 3.0287866179105709, 2.6144662353638619, 4.2921015272303222, -4.9500080657726908], [2.1364641676218898, 1.5038300480451614, -2.5568511462799695, -0.05336661523653774, 3.0321391187714948], [-0.071849593759015562, -0.029615064777956945, 1.9977916738497381, 0.70841314290510837, 1.3129732279868209]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[13.633454507294861, 9.2071254007196135, 9.1044753433854364, -6.020374113855568, -5.9486289162883308], [-13.416179785009147, -9.0758278324666239, -7.8343074040418221, -12.86137962649954, 14.832811499037254], [-6.4019633608084403, -4.5062608652052916, 7.6616624821495147, 0.1599142735985925, -9.0858736773742894], [0.21529893817324958, 0.088742213658321106, -5.986428086341637, -2.1227760586510809, -3.9343540728091604]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[0.643958481755873, -3.1189220869789214], [-1.3048925828050875, 3.017696379364974]], [[3.0089955148024963, 1.6524992071023368], [-4.3494065892076792, -1.980881764805642]], [[0.40890794613076942, 0.81243893908613352], [2.0868106341037285, -0.25183112670002217]], [[1.2983089255571265, -0.014232255341715572], [-1.3468872743735396, 3.0968931092214049]], [[-1.0945090913350599, -4.442132096725163], [2.4144910360520662, 1.1962321736587276]], [[-1.6832901765509298, -2.2440852302869585], [0.92586567643463713, 2.4234015639933713]]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(3.87951657712)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[2.4982476049466045, -12.099909939169137], [-5.062352406348543, 11.707203128450493]], [[11.673447980145115, 6.4108980676251868], [-16.873594963450397, -7.6848636438710356]], [[1.5863651555289311, 3.1518703320794987], [8.0958164483081561, -0.97698303066663106]], [[5.0368109989170264, -0.055214270527938689], [-5.2252715084392429, 12.014448154781924]], [[-4.2461661636389492, -17.233325106986005], [9.3670579996628902, 4.6408025477889918]], [[-6.5303521440264873, -8.7059658513602809], [3.5919112399112438, 9.4016265405220416]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(4.77981955406)
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-4.2871892518510748, 0.22373289270689245], [-3.2624200988482244, 2.3435663312877075]], [[-3.4444437426857641, 3.3423062259289953], [-3.3719779993157073, -2.5494219674884033]], [[-0.30495555991389889, -4.1257565557115781], [-1.4671487941498462, 3.9174627748742203]], [[-4.8095952429567195, -2.1387078057829756], [1.1165933559646817, 4.8640000063229003]], [[-1.3314031193169051, -2.7411684637957077], [1.6237056550008235, 3.5683371081246431]], [[0.24520006221466772, -3.9690300052697314], [4.294262185725378, -3.2342252181524724]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-20.491991017943405, 1.0694028554462791], [-15.593779382025321, 11.201824176520052]], [[-16.463819554140812, 15.975620654343922], [-16.117446376981295, -12.18577697174511]], [[-1.4576325483950427, -19.720371860271598], [-7.0127064949894864, 18.724765173636605]], [[-22.988997389387016, -10.222637390497123], [5.3371147567708013, 23.249042341158564]], [[-6.3638666640442478, -13.102290624216799], [7.7610200398068647, 17.056007484883573]], [[1.1720120520298127, -18.971247229829665], [20.525798365580357, -15.459012939951446]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[-0.50391513050655146, 4.8995783021395081, 1.1960910130297497, 4.4422181625059274], [-3.5011543901297482, 0.37207308191004529, 2.9700846550243263, 4.4758287669290358], [-3.0488607930673806, 3.52357567859322, -1.437024963199296, -1.8387471579764481]], [[-1.6481980585989122, -4.0239344336900107, -0.4975882228240982, 2.8759297004895199], [0.62713728296917282, -2.6941794527085494, 0.11788895933243282, -2.2676225088430613], [-0.15182431225211435, -0.3430397118719144, -1.9876882097559943, 3.2730762901746111]]], [[[2.8308625658197535, -3.5411909978084499, 0.96291601348250211, -4.9111229845538755], [-1.3952419763978217, -4.1490705461642374, 3.7230626506387878, -3.270604982342995], [2.903702505894211, -1.4020575165788287, 3.2662429037278677, -3.8534533380890492]], [[4.8028500262921838, -1.0573415620083937, -4.1458722214770773, -1.3152440583324667], [4.2612886981751998, -0.35387282825311672, 4.9101553163165121, -2.7508344198076786], [0.038007166593249764, -0.46243113082366083, -3.1754482214065805, 0.81438525099992631]]], [[[-0.15366846468716222, -3.4535010064469471, 0.95556135078769877, -0.26548028793294076], [-4.1814510105754987, -1.5346001848750355, -1.5525642945065075, 2.2260445775350313], [3.1516038618138715, 3.9684153412846861, -1.7605383795197262, 1.0755657565614225]], [[-4.4116039353930523, 1.2659727662797362, -1.5395017993242597, -2.4907745255968869], [0.41207945564471693, -2.5992224534943298, -3.9780479049399764, -0.88679632550189602], [2.3970837415874406, -4.0182450214586343, -1.880272878879957, 0.77313519652221885]]]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-1.79496904804)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.90451206209886903, -8.7945914007943546, -2.146946347028567, -7.9736441063443051], [6.2844637626962347, -0.6678596656378053, -5.3312100258305986, -8.0339741009697381], [5.472610755342096, -6.3247092815053687, 2.5794153302051877, 3.3004942357413278]], [[2.9584645002265364, 7.2228377598205027, 0.89315545863905177, -5.1622047967208999], [-1.1256920118022802, 4.8359687274802514, -0.21160703310749704, 4.0703122160146803], [0.27251994123267775, 0.6157456650590355, 3.5678388136683052, -5.8750706327407318]]], [[[-5.0813106849047678, 6.3563282342680711, -1.7284044400642515, 8.8153137483976085], [2.5044161621618297, 7.4474532085039336, -6.6827822218146098, 5.870634711674799], [-5.2120561227995905, 2.516649845832418, -5.8628049155755235, 6.9168294699406347]], [[-8.6209671395780223, 1.8978953770125333, 7.4417123146848958, 2.3608223753267867], [-7.6488813179919868, 0.63519077365712107, -8.8135768138627686, 4.9376626398409771], [-0.068221687638626363, 0.8300495666791321, 5.6998312710820853, -1.4617963187260785]]], [[[0.27583013777345838, 6.1989274139511785, -1.7152030481682969, 0.4765288997046771], [7.5055751398833452, 2.7545598329688925, 2.7868048537330021, -3.9956811162351857], [-5.6570313836428099, -7.1231827073776097, 3.1601118991264054, -1.9306072421606999]], [[7.9186925162469928, -2.2723819311351403, 2.7633580791906809, 4.4708631790957538], [-0.73966986821590675, 4.6655238529958609, 7.1404728609921415, 1.591771956192515], [-4.3026911217120904, 7.2126254409636354, 3.3750316194607208, -1.3877537477085831]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(-2.7066536757)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[3.1685061832826058, 3.6537137058170206, 0.23239402313561541, 1.465132518678768], [4.9300659765044568, 0.2405390462325645, 0.10267825709963496, 4.890648733800413], [4.2175031312932774, 0.73559033764454007, -2.4023729197506549, -2.8716383886608088]], [[-2.7018428023395957, -1.8173066676629523, 0.99770339641287009, 3.0596364966765925], [2.603234282589403, -3.4878047048577088, -4.7561241289438296, 2.866575946514633], [-3.73755709641968, -2.16313576986387, -3.23537188491831, 1.1531530294090864]]], [[[-4.7804668723735197, -1.2225441301096063, -4.2593121508421881, -3.0569378124950228], [-1.9842956268205003, 4.0174623330148016, 1.4358677185073132, 0.60794826725745299], [-0.8572161370862732, 0.076266451883909347, -4.997675806398858, -1.7111271168370079]], [[2.3841565918848815, 0.18226099383093164, -0.62580283113246971, 0.73434397638609106], [-3.9778635509187223, -0.99976645173068679, -0.028175241451656952, -1.7238841141834991], [4.2811886700412209, 1.5084791207484693, -4.5674251797459986, -1.3740796043481183]]], [[[0.32015145011639579, -3.2228504719436248, 1.5147184089995864, 1.1674921967630292], [-0.79547690227205958, 2.7281007434753271, 3.1561576438178722, -4.0766772505899995], [-3.5432139976515975, -3.6872939255524919, 4.4467447645014815, -2.2985635795144397]], [[0.12486650492883733, 3.1359584648711376, -4.4097658361868062, -4.616917028893007], [-4.3130527218218608, -0.23404501707555347, -4.8607839433576583, 3.2490711058217521], [-3.8894316433980682, -3.0534706848586524, -2.2014802529798807, -3.0659978804453916]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-8.5760489074617077, -9.8893376318070274, -0.6290101369308464, -3.9656063170702565], [-13.343981196751889, -0.65105589363486938, -0.27791448199325053, -13.237292371901008], [-11.415320352593426, -1.9909882911953851, 6.5023914936465141, 7.7725305999515122]], [[7.3129527521174751, 4.9188197719050031, -2.7004375651597932, -8.2813763700371776], [-7.0460536396802285, 9.4402794245287041, 12.873180855693777, -7.7588283223085446], [10.116272653164911, 5.8548593825413295, 8.7570312045722805, -3.1211858856952999]]], [[[12.939068231674382, 3.3090035634672672, 11.528482889032919, 8.2740719665775782], [5.3708010520101856, -10.873879190642921, -3.8863966381175468, -1.6455054122081505], [2.3201872083143669, -0.2064268723242205, 13.526977591349057, 4.63142850037773]], [[-6.4530862028708524, -0.49331738888932192, 1.6938315331484943, -1.9876148229139532], [10.766699001529304, 2.7060215414189353, 0.076260620838877094, 4.6659572741365123], [-11.587695050134515, -4.0829305568913412, 12.362438151246641, 3.7191576118139582]]], [[[-0.86653909923839612, 8.7231400761193854, -4.0998181493699821, -3.1599970457203339], [2.1530804814695377, -7.3840239050088305, -8.542625687729954, 11.034153464954134], [9.5902531905372488, 9.9802276569848729, -12.035798061740003, 6.2214155613241156]], [[-0.33797038453751532, -8.4879535057876421, 11.93570890949362, 12.496395446657607], [11.673940003009296, 0.63347880574693904, 13.156458727075101, -8.7941102511848168], [10.527344453989317, 8.2646876528164732, 5.9586446187101165, 8.2985944327975396]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([0.30728729403532018, -0.045004500836494543])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([3.6638496537220036, -0.96374141457601681])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([1.1258544458444792, 0.043372701298450785])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-2.3805846488444615, -4.7011601632769215], [-3.6426104039216147, -3.4903475328673181]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-1.5927526597178199, -4.536845359985926])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[3.7916825311304283, 7.4877853538187775], [16.525960109268237, 15.835167009227417]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_Symbol_rank2(self):
      arg0=numpy.array([-3.917433212557162, 4.3101228248807164])
      arg1=Symbol(shape=(2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[-2.0924810794335591, -0.77380619450629951], [-1.7420441178498614, 0.64190208221283207]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[8.1971548772204859, 3.0313340864414449], [-7.5084241142938799, 2.7666768158839856]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_Symbol_rank1(self):
      arg0=numpy.array([[[2.3079975578693546, -2.398402030672937, -3.1928254594206242, 4.5418089100887435, -2.3478528433406218], [0.07470491477219543, -2.5266723001870339, -2.9623555780237787, 1.9885327887406223, 4.9899206092137387], [-1.2180642392651473, 2.0626869015370843, 4.07612836596976, -1.9837778302616571, -4.8599918145304875], [-3.1686627002723808, -4.2488433619883716, -2.0013076570144461, 4.5050366173993908, -3.9013760796686938]], [[-4.9521065951944587, 0.096231521755706595, 0.61718419366214228, 4.0611388898101168, -2.8455682311474364], [4.9254135755683759, -0.14899098806236566, 2.6634360877595658, 0.81175340695156617, -1.5775707725708532], [1.1654402924968963, -4.1192810174973529, -0.58657808640222875, 1.2084642677354243, 1.7874274425803005], [-0.53250848311037835, 2.7165147049496277, -0.46038034128088867, -0.98182415710277748, -3.9367671587277941]]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-3.922518952984305, 2.6055805467182891])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-9.0531641641840341, 9.4077774221906392, 12.523918378148219, -17.815331530656085, 9.2094972768216792], [-0.29303144407501375, 9.9109199854640906, 11.619895900277047, -7.8000575524658258, -19.573058163527879], [4.7778800644699491, -8.0909284653516842, -15.98869077031333, 7.7814061377114312, 19.063410003844421], [12.429139497432839, 16.666168615660943, 7.8501672153917781, -17.671091515637414, 15.303221615220057]], [[-12.903112609714023, 0.25073898106776693, 1.608123128748091, 10.58162448881035, -7.4143572274373319], [12.833561797043131, -0.3882080201316368, 6.9397972576937903, 2.1150888858852954, -4.1104877160819573], [3.036648554491586, -10.733118485657023, -1.528376451060887, 3.1487509874155837, 4.6572861730576527], [-1.3874937445548665, 7.0780978700909225, -1.1995580613330104, -2.5582219240450783, -10.257563925740572]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_Symbol_rank3(self):
      arg0=numpy.array([0.46150238758287276, -1.1743274890903188])
      arg1=Symbol(shape=(2, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-2.6603612515755137, -4.6005061792567359, 4.3831130424286719, 2.4992670049355139, 3.1915268334649536], [3.555049506769981, 1.8648032940246839, -4.216325882493436, 3.351454802849247, 3.1340851567118015], [-2.9906968846326332, -3.8213058921842991, -1.6753161805070125, -3.014709243677677, -2.0196681701297381], [3.4301649634889699, 3.9362125984971907, 4.2764835089884841, 4.8453620733937903, 4.953501303286755]], [[3.9282880737882149, -4.6963388919036273, 4.853911227503378, 4.9568389208328743, 0.59035584946120512], [4.5121865663877507, -0.96157333726723504, -2.5722572410744879, -1.2938034123417674, -0.16779086211781458], [1.9716856250961348, -4.8057520087335543, 4.0711044171151052, -3.6933510955648154, -1.4978813810122227], [2.8424811359299564, -4.7925031045827478, 3.2273675830871529, 2.5712554582098415, 2.0695491493485871]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-1.2277630694350592, -2.123144585816743, 2.0228171341264618, 1.1534176899848352, 1.4728972536788816], [1.6406638353496603, 0.86061117256479747, -1.9458444615981838, 1.5467043933910136, 1.4463877827105385], [-1.3802137527946197, -1.7635417929275539, -0.77316241726020529, -1.3912955138254046, -0.93208168264000579], [1.5830293204532773, 1.8165715122402373, 1.9736073498569671, 2.2361461655747332, 2.2860526783617092]], [[-4.6130966701151594, 5.5150398588463965, -5.7000813840613489, -5.8209522037268346, -0.69327110236755929], [-5.2987847208131944, 1.1292020027292304, 3.0206723872053942, 1.5193489125917941, 0.1970414218031131], [-2.3154046293946196, 5.6435266896068308, -4.7808098279752871, 4.3372037183836074, 1.7590032811192227], [-3.3380037351432228, 5.6279681372622159, -3.7899864702182273, -3.0194959660493401, -2.4303284561035317]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_Symbol_rank1(self):
      arg0=numpy.array([[[[-1.4271027893733557, 4.5357086921326868], [1.0547367871369744, -3.4360299209156508]], [[-4.9312603815072436, -0.67474199843511329], [-1.4405008406988804, 4.2231993939797583]], [[0.16323935240304799, 0.36878568416635815], [-3.8584178681948456, 4.3696080983896017]], [[0.77199852526883195, -1.9937541817961013], [-0.93962610298270821, -2.3924831413138969]], [[-2.6076279253465007, -4.44495554908658], [3.6501028545599006, 3.0640713935821964]], [[4.7787213233730341, 0.21218322391673361], [-1.2055651905993026, 0.97443200310419975]]], [[[-4.7469928706558449, 2.1013993996263727], [0.63300120036467433, 4.8276277342486136]], [[1.2689177603717594, -3.3322416059632785], [1.0024746969933362, -1.9848408445625321]], [[3.1687406963489497, 0.35633143731030525], [-0.92766003172757117, 3.7089696186751624]], [[3.1301013301658482, 3.988329916399282], [-0.02470884696443143, 1.1347020117911679]], [[-2.1444815277569629, -2.2615502350509464], [-3.7942192298009814, 0.54457806588535718]], [[-4.1270266951325301, 2.7869857458319363], [-1.6001873932511899, -2.4603623345648549]]]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-4.1773287566902084, 2.7938595742727736])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[5.961477520802128, -18.947146351615608], [-4.4059823116463219, 14.35342659748893]], [[20.599495798397335, 2.8186191534096183], [6.0174455858878542, -17.641692273708305]], [[-0.68190444101673919, -1.5405390435238009], [16.11787991613766, -18.253289564869302]], [[-3.2248916397279244, 8.3285666773882117], [3.9251271405264223, 9.994188626107066]], [[10.892919119298366, 18.568040637409087], [-15.247679619230091, -12.799633544962751]], [[-19.962290004334864, -0.88635908295460886], [5.036042138755179, -4.0705228280064158]]], [[[-13.262431480686431, 5.8710148320171998], [1.7685164641650037, 13.487713966355267]], [[3.545178033779405, -9.3098151146105881], [2.8007735301610301, -5.5453665969886883]], [[8.8530165328822896, 0.995539997743775], [-2.5917518613122597, 10.36234027982244]], [[8.7450635697277992, 11.142833722290664], [-0.069033048660817511, 3.1701980795893321]], [[-5.9913802481748952, -6.3184537768959288], [-10.600515722069341, 1.5214746433127544]], [[-11.530333045475343, 7.7864468093543016], [-4.470698869265429, -6.873906864604133]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank1_Symbol_rank4(self):
      arg0=numpy.array([-4.7558359863195951, 3.6176404088763547])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[-3.1041142857445325, 0.33676765253290686], [1.3037778296978511, 1.0455063319113123]], [[-4.6347608491539489, -1.4086320625937021], [0.70737415982494234, 4.9295329837635347]], [[-0.14136604222522031, 3.1334184971871775], [2.3542268199962297, 1.5952919049767029]], [[0.93051423938359168, 0.94115798955751906], [-1.4179569533072076, 3.4306511517856944]], [[-4.2921370055886126, 3.8551885294147468], [-2.3012601595931717, -3.4831311603421145]], [[-4.0204713844006665, -2.5387126898616961], [3.524914968319699, 0.18590238565940176]]], [[[-1.9718064915094677, -0.043445362134314713], [4.3894735713784971, 3.4887335659552203]], [[-2.3549438586994018, 4.6959930406401966], [0.68605477460219699, -0.24103323119966014]], [[4.0888213703288585, 1.7090984875571742], [4.8510130857726903, 1.968917749224981]], [[0.50563212395205603, -1.6476036520329984], [2.6523274591483563, -1.9065138591066599]], [[-0.90634995956893061, -1.0719157581101593], [2.9281337504642675, -0.18972156790314187]], [[3.0344818605795769, -2.3321486357861732], [-0.49926641195447008, 0.85751470932879315]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[14.762658425792594, -1.6016117209443719], [-6.2005535206427007, -4.9722566372288179]], [[22.042162434391514, 6.6992230547667253], [-3.3641554850880495, -23.444050359932028]], [[0.67231371085827818, -14.902024449122244], [-11.196316630496813, -7.5869466503725436]], [[-4.4253731054432919, -4.475993035549851], [6.7435707055905114, -16.315614204171172]], [[20.412699629392353, -18.334644342237173], [10.944415880876781, 16.565200517426156]], [[19.12070249190085, 12.073701169370471], [-16.76391745505142, -0.88412125566164668]]], [[[-7.1332868421693618, -0.15716969763536359], [15.87953696551366, 12.620983524002906]], [[-8.5193400638661654, 16.988414183622119], [2.4818994753034671, -0.87197155706992746]], [[14.79188541397887, 6.182903751336295], [17.549220963079264, 7.1228364113501721]], [[1.8291952036349357, -5.9604375494068318], [9.5951669937874424, -6.8970815767870546]], [[-3.2788482383200139, -3.8778057614506443], [10.592934978274206, -0.68634441048178529]], [[10.977664198834983, -8.4368751443259242], [-1.8061663466811997, 3.1021798636737037]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-1.5781145209417713, 1.7105045476702072, -1.1056149826732344, -0.11983922447724105, 4.6053670180211306], [2.7826598349219234, 1.8494171226233718, 1.1376657323920467, -2.7945657521374878, -2.7684269513268234], [1.3898360185164886, -1.3286880768152596, 0.85442300919184788, 0.84377398795138348, -2.0403044630052203], [-2.7210374451485264, 4.2480425859386735, -4.7371351690555112, -1.2507180618476021, 0.71235378970253826]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-1.1216595911240859, 2.5554777831341013, 0.56497529335388297, -0.84925200293063252, 2.7007969184075575], [-0.091039832092403294, 3.3054061376203379, -0.42192272371329853, 1.7404184290810658, 4.8941313893829559], [-0.19407910687979335, 4.2191803847825824, -1.8902736983788158, 3.8357427565141347, 3.5143581078149158], [0.067132974379001453, 3.0238150595664628, -0.55491833657274015, -4.6030815408524015, -2.0294593326588495]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[1.7701072883065299, 4.37115636952106, -0.62464514917225888, 0.10177370141695063, 12.438161050407272], [-0.25333288414156657, 6.1130747081394379, -0.48000702448613697, -4.863713736298874, -13.549045241702366], [-0.26973813318304807, -5.6059746711934366, -1.6150933415650313, 3.2364999624195638, -7.1703605319733539], [-0.1826713370894596, 12.845295145041021, 2.6287231681325105, 5.7571572233013901, -1.4456930466667157]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_Symbol_rank2(self):
      arg0=numpy.array([[[-0.69472505675049767, 1.820890260013579], [-1.894991010525866, -2.5424724756891659], [-3.9126015191231698, -4.2526756304395432], [-1.2107982955545249, 3.7698596305317462], [2.5436951617256849, -1.464981995361736]], [[-0.99982283949934558, 3.1782128789021105], [0.23348500769071734, -4.8154091229586697], [-1.5925764283940635, 2.5794520533580396], [1.2363189031481454, 1.9217905544983536], [-1.4422044440616633, 3.0877815274144318]], [[4.049080118986863, -2.1374053910888255], [-3.0588945148641447, -4.4864778436754875], [4.8605284767061026, 2.1693360655879292], [4.7598861946637587, -1.5286677838536757], [0.90811932305835796, -2.8372360709508762]], [[-0.079654913238956127, -4.9571090203183594], [-3.9113951672966643, 1.6080944332423801], [2.9433049712284012, -2.0559613560293135], [4.7083614035592181, -2.9912736715148447], [-1.1117781140441405, -0.56037468960452763]]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-1.8155121893830173, -1.0684716211237335, 2.9624975193742165, 0.15361814254634609, -4.4273143012750413], [4.0753876897666821, -4.7127171658537055, -1.0120938338795114, 1.3076688419182991, 1.8312894089469705], [-1.0310447004353551, -3.3440340803575506, -1.4113420887217032, -2.1202180331171516, 2.7416475188275244], [0.21617255362379417, 0.71205104379973161, 1.1172975609927107, 3.8291345295431061, -1.646568562706582]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.261281808800337, -3.3058484625834645], [2.0247441170314739, 2.716559687762075], [-11.591072294702181, -12.598541005880328], [-0.18600058516136789, 0.57911883410274145], [-11.261737967592254, 6.48593573917546]], [[-4.0746656920432018, 12.952449642135587], [-1.1003488037135281, 22.69366123437586], [1.6118367831594869, -2.6106475179915161], [1.6166957083214371, 2.5130656288103879], [-2.6410937239463772, 5.6546216082961482]], [[-4.1747825983195623, 2.2037605011640911], [10.229047505924475, 15.002934810019886], [-6.8598684126057092, -3.0616752939461898], [-10.091996545511478, 3.2411090019717954], [2.4897430888622782, -7.7787012342504234]], [[-0.017219206003546917, -1.0715909155137644], [-2.7851130115868155, 1.1450453197187747], [3.288547465611213, -2.2971206085868183], [18.028949227936646, -11.453989302910674], [1.830618891290295, 0.92269534723927404]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_Symbol_rank3(self):
      arg0=numpy.array([[1.4092272523375202, 2.2432086115203163, 1.926247095585798, -2.0032565434535021, -3.1694411380206224], [-0.56376726445784175, 2.0125040963618881, 0.58487228537673452, -3.6595637649600099, -3.7304387742754122], [-0.36545919198961485, -0.69818335964585998, -2.7213127114975189, 4.6194253571945225, -4.041952543918331], [2.9963001289488602, -2.9919020366300075, 2.6030249627466482, 1.9835574722037128, 2.4433304567675478]])
      arg1=Symbol(shape=(4, 5, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[3.8724395024388585, -4.0927177800589032], [-4.3703169159937172, 0.41062179232480567], [-3.7058112365805851, 0.18058447105437203], [1.9299155441392211, 3.5169240359652481], [4.6946252529597938, 3.9978343445826319]], [[-4.4943993968655338, 0.94310699927064157], [2.3882670311904075, -3.4993810620567976], [-1.4294388251336896, -1.5197738694445828], [-4.1491293576967863, 0.028439377864667215], [-3.6145472591934413, 0.14015944740456288]], [[-3.5654612345505852, 3.0499826234063452], [2.9754541558182535, -0.54894756789523047], [0.39627508589052152, 2.9455940706523638], [4.321993288329022, -2.7798685247705626], [2.3179600984077675, -3.3262160490261614]], [[-0.33258705798122978, -1.7571400731233231], [3.3636937785429595, 1.963867366281856], [-0.4442474377499277, 4.8157877028749336], [-0.51666726959745901, 0.17773928094443558], [-0.21406793094177878, -4.6008158135095156]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[5.4571472798651861, -5.7675694317853239], [-9.8035325410300178, 0.92111034062091102], [-7.1383081312525665, 0.34785031287638174], [-3.8661159421095208, -7.0453010878762834], [-14.879338404321242, -12.670900634711906]], [[2.533795253351856, -0.53169285306985337], [4.8063971834767401, -7.0425187221205192], [-0.83603915246217542, -0.88887361627789607], [15.184003453558958, -0.10407571673154192], [13.483847247146132, -0.52285623717899665]], [[1.3030305818491514, -1.1146441851324487], [-2.0774125789814244, 0.38326605722251583], [-1.0783884284836474, -8.0158825873779982], [19.965125389731622, -12.841395152992066], [-9.3690847164604616, 13.444407420983273]], [[-0.99653064471588082, -5.264919027680623], [-10.063842266622366, -5.8756987728498942], [-1.1563871700992994, 12.53561560587179], [-1.02483922325313, 0.35255607882145018], [-0.52303869548726023, -11.241313403125561]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[-0.26968443801195541, 4.7974870029034697, -1.885023727746332, -0.9921370511935681, 3.7235289552562261], [-2.3786595194320315, 1.336509081269651, 0.67105313640381681, -2.8217603656238319, 4.8613114141176776], [-2.0307371636180305, -1.3700216466302075, -0.27764788133100637, -2.504953459291368, -1.7555598363609635], [1.1264870829910585, 2.7957742879632006, -2.4329057487975634, 4.8623393571648865, -2.327139739866074]], [[3.2696838338479104, -1.0955343448688115, 2.8392226568770642, -2.921264653124811, 1.7394657873862549], [-3.7393165600738962, -1.4835557704064497, -3.7183882885236708, -0.8418672177348876, -0.090654603894298802], [-4.8407753772508588, 1.6044659275872233, 0.63854186055981721, 3.2085107745373609, 4.5927789293923134], [4.6785001265166155, 4.8893631294000208, -3.2346034665397259, -2.8159804894268872, 3.4415540796888688]], [[-0.99105981045759606, 3.0964666386370538, -4.2329903397880706, 2.7567902020855586, 2.3672531115978455], [3.4164140650692012, -4.1986759069739632, 1.8626223759801714, -3.0577337443901365, 0.064670805521651609], [1.425097488541919, 2.5391646685422966, -0.91551007758754288, -3.8866187442085751, 4.4602880184752447], [-0.0030207982300911596, -0.06310088088864152, -3.1332694967751151, 1.4480892547575301, -4.2361617976023052]], [[-1.8974622542530337, 4.3820759285210595, 3.4415983986018368, 0.1132941789165649, 4.6093499653573371], [2.9462515192119634, 0.14788884528444601, 3.2658776522809561, -2.5982039036946314, 2.3796195011021197], [2.2614852044785581, -3.1789157116541524, -4.823280890409082, 0.9596455433036688, 0.61897799761979311], [-1.8200796102218328, 1.6703693782889983, -2.7998620837442689, 0.76092121604786023, -0.25885962333267809]], [[-3.2237188692314112, -4.8798448549425899, 4.2310916944357828, 1.3420040359529448, -4.9436247783548772], [-1.7093816349648208, -3.5067097147958561, -2.5798737102530058, -1.2758711478647147, 1.5088103397415855], [-4.6450006987549353, -4.8945238844087591, 3.4035811691470776, -2.753641478805263, 0.39280234524387048], [-4.0859787808209571, -1.5799459909492688, -0.65096206874125873, -1.6764901331884441, 0.3515750152134034]]], [[[3.5476906838194093, 4.5347933626316035, 1.2594573949741221, -0.2089543098211788, 4.2824913119309791], [3.0728083522154019, -1.898189042172318, -4.0944769354642734, -1.3816458953886732, -0.045837969692943403], [4.3352211664969182, -4.0279592826673261, 2.7395755722979693, 1.6938355352690069, 1.4410583778705819], [-2.7885045736030367, 3.6005779277044105, -1.7534731983722507, 3.9844551296672641, 3.6886338430540775]], [[4.1788732296209137, -2.4845361966924284, 3.5672283464182897, 1.3846479243781911, 2.041443657649527], [2.3535539265875425, -0.27768785832781973, -1.4471144167976604, 2.053918860470155, -1.0315474222445831], [2.4860984370284234, 0.69556382988005083, -4.2519396467265338, 4.9507506049362942, 1.783911957562772], [4.880000301861223, 4.5605407094330062, -3.7928129667203017, -2.5463240278767896, -0.92776517809218451]], [[1.458870349452809, -4.0103732600333224, -1.9665474954117022, 4.1283084107124708, -2.4488936925838258], [3.0255875387628777, -4.9360360932338327, -1.2758194021941192, -2.1718420740683007, 2.0417355672322568], [-3.8207790891124835, -1.7013424289493373, -4.1492348077948584, 0.78226328018252467, 0.87137415146960073], [-3.3813502904280899, 3.634585224231877, 0.098225212219746005, -2.5986257679704208, -3.9820944373984757]], [[-1.7789588958513125, 2.0043845230036395, -1.5410573711276898, 1.7185471651062869, 4.3634175895764074], [1.8427909655158006, -2.5121502995479861, 0.61069517117028127, -2.8089170697965238, 4.4205863395133562], [-0.084299541419726509, 2.4314369973182135, 4.9745660966541259, -1.9257960442785551, -4.3431156399854451], [-2.1075238713921518, 1.5966751360343014, 3.5035404971720361, -1.4899674700939647, 1.5109773761607412]], [[-3.8345880310049587, -1.4830305197280245, -4.6185630026491076, -4.12103654086589, 3.5110854240695026], [0.99817060789425405, 1.2305616373398571, -1.7021874865938047, -1.8169692956349817, 2.9600262003867517], [0.099357669004761284, 4.0544104358365125, 3.624959767494893, 2.4842722279450049, 0.95957644964151179], [0.72862299156700061, -1.3946737181399405, -4.8566198769779998, -2.2899190135641856, 1.319824613591952]]], [[[-4.7351786366528188, -3.2254905268726599, -4.6973706521642553, -2.9546589083374619, -1.8884980259243225], [2.1804292383974069, 2.1816104639028042, 2.2073632659931528, -1.26880918160164, -3.172857595731402], [0.67703954791351961, 2.6779823408051247, -2.5716124866107402, -4.5180424740414153, 2.2305614055341998], [-0.98424021544034535, -0.018095711914357437, -1.0564716791988129, 1.1588305890956967, 2.2132029930951491]], [[-0.75593435904502737, -2.9257486844428247, -4.5931592819880196, -1.3953174664232515, -0.76852805988791051], [1.9369125106101803, -3.3707810649981997, -2.3643554681781298, 1.8096291453455535, 1.5643757469946307], [1.6611736496234091, -4.7976684337404034, -3.9553798180513011, -3.3171721877078264, -2.1300546712598933], [2.2420079624515585, 4.1430127296575101, -0.43961513779233563, 3.658102564570882, -1.611604703679077]], [[-4.8303750796239893, -2.8183763811234721, -0.16805052332468584, 1.3239037033604619, -0.32774999136809146], [-3.0027823776941198, -1.2488426832518251, -3.3997258568448139, -4.9159625150812403, -1.7960192321039337], [-4.3213719496173599, 1.7651043717720913, 4.0386229704527388, 4.5653824107491694, -3.8674420933704434], [-0.70415468417311544, -1.5755097298262202, -4.0422853499113609, 4.0157803332313833, 1.4939112586463619]], [[2.8966443464752691, -2.6258403440784539, 1.8704051733450191, 2.7368331407751203, 2.7955790898701842], [-3.1887159266410645, -1.9900897099067838, -3.0105863874459624, -1.5614422787564286, 4.1179249090391341], [-1.0376438256122391, -4.7108299158972269, 1.8460255051178152, -1.2395099696151899, -0.88121073576542841], [4.8746654350538989, -2.0792378422682436, -3.4634566950787793, -3.1543890719580503, -4.6712524272609794]], [[0.34236854013061802, 2.9261285201432399, 4.2819757297315437, 0.9130226581719505, -1.0069781380672449], [-1.7430738487077626, -2.4225226043007506, 4.643663785157413, -4.6369872395694802, -0.79378074266319398], [3.4591959870257334, -0.71526483605608782, -2.6600023467763778, 0.0069122756478599001, -2.9766443363459647], [3.5720781405519002, -0.8131518125620838, 1.6193517654589096, 3.9601053352492634, -2.8998133481884745]]], [[[-0.47655090640691711, -0.33019932946717567, 3.0533751016342556, 0.88057728754615106, -4.6036034252152023], [4.8366464073222346, 0.093870603429636823, 4.4352425636153257, 0.34261446020496056, -1.6938875973221359], [0.58639907626000376, -3.4850624497194485, -1.8888843093002148, -0.26951486765472676, -4.4999123143551936], [-3.1508229741510787, 4.7812383390075226, 4.7436201493031422, -3.8562068457477618, -0.17207662678395774]], [[-0.61266916913982339, 0.29223211016042772, -1.9713022333065089, 1.7743761392730173, 0.88567848280681361], [-1.0182660038134674, -1.0278448132962668, -1.8005419276589842, -3.545035314820959, 4.6786848336064875], [4.4950268578973471, -0.68691086289718228, 3.4265247097865235, -4.3342063713578298, -3.1742085650788185], [-3.2556005911377763, 2.495473633980434, 2.551037051808887, 2.2809346430668338, -4.7826099164104523]], [[-4.849080177409264, -1.9854332305960054, 4.0503178076510142, -3.5389938876305993, -3.5596099579663312], [2.8636406186569356, 3.8449549202001538, 3.1254606649867949, 4.6811286885847387, 3.1970478788437831], [-0.57893926992296763, -0.89915699840955288, 1.2249845298467763, 1.0671078455115683, 0.85515683292712996], [0.46777719793043548, 2.9869176113081073, -2.6443829793375331, 1.1027799884660316, 2.5609504724681873]], [[2.2784884786949258, -1.211468070952153, -4.7900249180490633, 2.1806538107063851, 1.9249496823298218], [2.2256168660846001, -0.53369465679710082, -3.5807184188172569, 4.7665344910979623, -0.75622683425438453], [4.2951297324075846, -1.4424484544701888, -0.70973640781493685, -1.0785772646644363, -4.5831704835864153], [-0.59695419353053225, -4.8381936456237202, 3.2763438492130899, 4.2043014753046286, -4.8686680266866773]], [[3.6316392163625082, 0.054160716815145449, -2.9253960056986337, 4.6739782463946042, 1.8108410238160495], [4.3021086551151377, 2.4385506785756661, 0.30743854454417097, -0.93606471811864189, -3.6679731170255527], [-4.7711808466016556, 0.66703674357543097, 3.0839958181482086, 3.2143490471199048, -0.31915174110454991], [-0.73344775903077775, -0.3050039928426207, -0.30089896409145922, -4.576216821391875, -1.5050151728164862]]]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[0.54268297510301622, -4.2821159202742933, -3.4884282759923115, 1.3981361115142992, -4.6916141691014666], [3.4764367964073113, 4.7989176740311468, 0.46226776243405077, -4.2628211685968811, -1.6472999202711511], [-0.16202306642225395, 2.3649946335280845, 1.1642446698943543, 1.4585557815745585, -2.5550942296110146], [4.7017456287256856, -2.6982507998328198, 4.0176639593643415, 1.722464345884581, -1.3766088074624818]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.14635315315931291, 2.6035145197537077, -1.0229702847131574, -0.53841588665165907, 2.0206957713206743], [-1.2908580247624857, 0.72530072447561311, 0.36416911251583345, -1.531321310244516, 2.6381509411156321], [-1.1020464856044934, -0.74348742314881422, -0.15067477827175974, -1.3593955957828319, -0.95271243496773195], [0.61132536161270601, 1.5172191083083866, -1.3202965299026932, 2.6387087883067282, -1.2628991175109803]], [[-14.001165199183625, 4.6912050593700059, -12.157880540216754, 12.509193878480314, -7.4485941409391412], [16.012186972837736, 6.3527577830722528, 15.922569688048693, 3.6049730158195872, 0.38819352258193685], [20.728761309397701, -6.87050909205891, -2.7343102668647612, -13.739215068018037, -19.666811771851151], [-20.033879874762096, -20.936819696405969, 13.850946999844178, 12.058354884956669, -14.737133515120648]], [[3.4572410659998587, -10.801801777888366, 14.766483193319008, -9.6168648919338207, -8.2579926909287078], [-11.917915427085239, 14.646779755615638, -6.4976245638652124, 10.6666848543864, -0.22559946661292918], [-4.971350375075259, -8.8576938271435921, 3.1936912416122998, 13.558190725298923, -15.559394842718762], [0.01053783796191753, 0.22012289713195993, 10.930185908854513, -5.051555502456802, 14.7775465964343]], [[-2.6529104979064932, 6.126738599062846, 4.8118230024150108, 0.15840068276761135, 6.4444986371732771], [4.119260642614111, 0.20676873508233515, 4.5661414814415444, -3.6326427028328845, 3.3270319561545141], [3.1618641300367707, -4.4445568519238474, -6.7436031888577803, 1.3417150883466185, 0.86541549070504464], [-2.5447190288820147, 2.3354037473535376, -3.9145882865425352, 1.0638714301738872, -0.36192098719440668]], [[15.124445124085847, 22.894349264465546, -19.850649744382451, -6.2961651500681901, 23.193580056850838], [8.0197590990027834, 16.452128984862, 12.103772053515373, 5.9858951552698478, -7.0787559684184203], [21.792551093764867, 22.963217607097683, -15.968289638857366, 12.919023378588287, -1.8428770486024288], [19.16983594274754, 7.4124969975526467, 3.0540628652540924, 7.8654448632257088, -1.6494543228772671]]], [[[12.333322435501211, 15.764922509956151, 4.3784240313953351, -0.72641645143023958, 14.887810377091476], [10.682424023949341, -6.5989342327449956, -14.234190280489043, -4.8032046303343101, -0.15935280451315159], [15.071122383773714, -14.002945864695091, 9.5239613260752787, 5.8885121818714499, 5.0097483706003221], [-9.6940599066236768, 12.517181596203597, -6.0958387483353089, 13.851706426409141, 12.823302420466506]], [[20.054068599163422, -11.92308466607742, 17.118835158931631, 6.6448113966090441, 9.7967200492331052], [11.294511535086361, -1.3326011711932313, -6.9445829511155681, 9.8565875205361397, -4.9503111562108], [11.930581728836911, 3.3379535566281699, -20.404708319589712, 23.758244577749174, 8.5608466220634867], [23.418719697879155, 21.885659413636599, -18.201397180288563, -12.219599381188104, -4.4522687104972389]], [[0.67438873212293171, -1.8538662734409537, -0.90907151042425427, 1.908383891657726, -1.1320446077095854], [1.3986315815922623, -2.2817703601129176, -0.58977018032222372, -1.0039725759396814, 0.94382853214647278], [-1.7662230002788384, -0.78647575776452316, -1.9180574904128078, 0.3616150961642966, 0.40280817924272194], [-1.5630892327619208, 1.6801515789815324, 0.045406349067431767, -1.2012609191631531, -1.8407938853772736]], [[7.5833836392987095, -8.5443327746678772, 6.5692519836653762, -7.3258592346472389, -18.600468868274287], [-7.8554883370998398, 10.708847475610151, -2.6032843032245707, 11.973911145961745, -18.844169025687734], [0.35935386966701977, -10.364781102277718, -21.205685661401567, 8.2093241439507612, 18.513925287794148], [8.9839973722937163, -6.8063405692593246, -14.934966596381397, 6.351464872037293, -6.4410263443689804]], [[6.3167165577471787, 2.4429960569076585, 7.6081584660311634, 6.7885831652028807, -5.7838107391348927], [-1.6442863628012112, -2.0271040870786838, 2.8040133109525258, 2.993093375834635, -4.8760509238976146], [-0.1636718802298707, -6.6788299877000101, -5.9713959359804676, -4.0923414430256413, -1.5807102089885365], [-1.2002605959160477, 2.2974459046961937, 8.0003095361331464, 3.7721834084716761, -2.1741469807419254]]], [[[0.76720816276763759, 0.52260386587983987, 0.76108239718555548, 0.47872289656066491, 0.30598024109263194], [-0.35327983132188762, -0.3534712171004083, -0.35764376506405199, 0.20557635430780818, 0.51407611698154188], [-0.10969602364208497, -0.43389491068189184, 0.41666054073042957, 0.73202709587017678, -0.36140239876778379], [0.1594696178017446, 0.0029319227334579073, 0.17117278105205944, -0.18775728550919174, -0.35858993555618657]], [[-1.787780702440982, -6.9193799377591336, -10.862797052841376, -3.299918320158993, -1.8175647373506587], [4.580787693206485, -7.9718791295188236, -5.5916879939940589, 4.2797632174182478, 3.6997402464637901], [3.9286667667176252, -11.346460099243144, -9.3544520432566181, -7.8450944224176249, -5.0375678666510755], [5.3023367995251709, 9.7982028722785515, -1.0396874416965831, 8.6513929341054592, -3.8114364755696362]], [[-5.6237384400427466, -3.2812796794791415, -0.19565192605372234, 1.5413478300908141, -0.3815811805082211], [-3.4959733780830748, -1.4539584375125008, -3.9581127079335912, -5.723383155583778, -2.0910058180047559], [-5.0311342589729851, 2.0550133566428799, 4.7019452670625057, 5.3152221377441577, -4.5026488433316025], [-0.81980833782969209, -1.834278805316871, -4.7062091728263367, 4.6753508484312114, 1.739278220174193]], [[4.2249173587167625, -3.8299346153473568, 2.7280902794693418, 3.9918238006824089, 4.0775080443790994], [-4.6509200506011998, -2.9026568522365754, -4.3911081813389723, -2.2774506632751423, 6.0062231841689169], [-1.5134614010618743, -6.8710082098462912, 2.6925311734236841, -1.8078944325015405, -1.2852950134362362], [7.1099714535395249, -3.0326843761089566, -5.0516447868402663, -4.6008524182400201, -6.8132822349756914]], [[-0.87478388128808915, -7.4765340969182095, -10.940851478371481, -2.3328589253992607, 2.5729240299200611], [4.4537179326190666, 6.189773527351095, -11.864998541709348, 11.847939338603886, 2.0281845951550728], [-8.8385717055430302, 1.8275690552505783, 6.79655664700008, -0.017661515621327568, 7.6056067674018824], [-9.1269962446438022, 2.0776795040751175, -4.1375963516344694, -10.118442290747186, 7.4092963529053675]]], [[[-2.2406211410639858, -1.5525132539304456, 14.356193036968707, 4.1402504124752371, -21.644972280892169], [22.740681103319108, 0.44135569934113728, 20.853382335816463, 1.6108860406068837, -7.9642286062620071], [2.7570992934942526, -16.385877138804446, -8.881053544420821, -1.2671903508721933, -21.157443053668416], [-14.814368145603298, 22.480166460324277, 22.303295301321132, -18.130903680456605, -0.8090605277873345]], [[1.6531350756644376, -0.78851552497720678, 5.3190678277315113, -4.7877118369976897, -2.3897826746282029], [2.7475370592322577, 2.7733830895806673, 4.8583136964383815, 9.5653943736512446, -12.624265094444388], [-12.128709814591524, 1.8534577852261744, -9.2456230388284073, 11.694775808156768, 8.5648107995601102], [8.784426898973706, -6.7334137288494196, -6.8833377654464876, -6.154533725021472, 12.904681032242879]], [[-19.481974664845247, -7.9768035342898829, 16.272815879771073, -14.218488194744157, -14.301316637515747], [11.505145706149776, 15.447736808268756, 12.557050670128355, 18.807202021273369, 12.844664039192883], [-2.3259834394302112, -3.6125106663202811, 4.9215761963442652, 4.28728073166676, 3.4357327872554837], [1.8793715891375506, 12.000431236543212, -10.624242190840906, 4.4305994147681993, 10.289038414952518]], [[3.9246151670608094, -2.0867105583926553, -8.2506471372382233, 3.7560984396590924, 3.3156571954349685], [3.8335456994301018, -0.91927001792211416, -6.1676598090649373, 8.2101857143453465, -1.3025737594043458], [7.3982078250208456, -2.4845660336012183, -1.2224956574374275, -1.8578108826662088, -7.8943477490881939], [-1.0282323144826258, -8.3336160530721966, 5.6433854651277953, 7.2417593905621658, -8.3861070879160415]], [[-4.9993465307707741, -0.074558119786210561, 4.027125906760304, -6.4342396198748579, -2.4928197022995517], [-5.9223206652920712, -3.3569303415708736, -0.42322260817295226, 1.2885949353170079, 5.0493640984329886], [6.5680495754281392, -0.91824865610703132, -4.2454558054402867, -4.424901208523897, 0.43934709772150921], [1.0096706448953887, 0.41987118285837544, 0.4142201641246398, 6.2996603811860181, 2.0718171422638441]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank2_Symbol_rank4(self):
      arg0=numpy.array([[-2.3514090094789708, 3.1147049738800643, -1.9412312034827597, -2.4297791310688188, 3.2619246872330088], [4.1185507343999408, 2.7921272637438079, 1.1467284212765252, 4.2874835793623536, -4.8898313755566312], [-0.0074004926232831281, 4.1614053719378656, 4.7929313118964441, -3.0768790419076986, 1.413347107242739], [-3.967597166845013, 2.1060054059748516, 2.5711369448468684, 3.2670175740414358, 1.2402596080495449]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[3.9833777908616153, -3.2064216914639498, 0.60423340529277425, 2.7762596780316926, 3.97088089894525], [1.5457609915081392, -0.77166155614162957, 2.3122877677602798, 4.2168588466465433, 2.7261531721699583], [3.0162523817497462, 1.5945865599954505, 4.6518142470718775, -3.6116891489674896, -2.6060878057829084], [-3.9930791955244072, -0.71451128666599839, -3.3442007438434418, -4.7329392541110105, -2.3045837049397289]], [[-4.9595559036509842, 4.9037976703774557, -0.65884725519143572, 0.56382999791617117, -4.8386291600440909], [2.4164393424580899, -3.7862342801555804, 2.2503141783409566, -0.83409350605313293, -0.51842316552098744], [-1.4114196799771594, -4.343952419674892, 4.5112832003881405, 4.5976150638648683, -2.326950099016738], [-3.5331313505082083, -1.3827311587828461, 0.86398656623638193, -2.1795728673539285, 4.0418529207234588]], [[-4.1475770310598268, 0.099627273519048209, 1.9585774120764254, 3.7467369811254496, 3.1625948655607665], [0.5008623792226139, 1.4662448194256958, -3.1214272667608469, -3.0858481867005647, 2.8239282265207741], [-4.2850107342610064, 0.42893817948879054, 0.68744911760236427, 0.68367468553917021, -2.8388235097622969], [3.6237580535143294, -1.6796178886303883, -1.9401795352517128, 1.0887174393317132, -2.0067930434730097]], [[-1.3467410052338513, 0.61024854513913684, 0.43616637081762377, -4.7305978669841426, 1.8954240883769859], [2.9811019083140398, 1.5109212930975824, 2.4284991261625297, 0.73582753504297926, -3.4425589541315169], [1.9731151153509776, -1.9468793998178269, 4.9007342786007229, 1.3094133980392115, 4.9016314220137343], [-4.3509127587462801, -0.092648365114035514, -2.7860279178048186, 1.3100409054303679, -3.6438132660104863]], [[0.28658716934071204, -4.3242947419424773, 0.52081967476747337, 0.80758259464708004, 0.046251255669325531], [3.3640532317288958, 3.2781083770698185, 1.547036332610884, -1.2641916447249866, -3.2034100920029984], [-3.9129061554514646, 4.9861087684453711, -0.37269681673115596, 2.0016511162501915, 1.1713201435583409], [2.3003076670344047, -2.6054204490179678, -2.5435763430963254, 1.4016240564471154, 1.6342949109073901]]], [[[0.48042850893599276, -1.0775819013544972, -0.9491432213659543, 0.7710466060471326, -4.5331817636184155], [0.99713951002344015, 3.5788820251290208, -2.9408502404623151, 3.0953908049797345, 3.8866575453369894], [-4.5753559708475446, 3.9733224527192466, -4.8794389200311503, -1.8226123933968816, 2.9709852863404862], [-0.25769957785461983, -0.98587185056802973, -4.9820589345492374, -4.5702370493907711, 3.043609950867566]], [[-3.3175862732519681, -4.2540135421755751, 0.90128968739367821, -3.5675789962613225, 2.6934005833752304], [1.9355757887565836, 1.2537777395137892, -0.040566844031904914, -2.1496809009341087, 3.0506347093698185], [-2.1833740918968014, 3.0460166951064132, 0.40995574440408244, -4.2737521073649738, -0.017332722215110508], [-0.10903507984233052, -1.884858167648745, -2.3130914220559582, -0.99904390409786359, -3.4818023809691221]], [[-3.9866883008307541, 0.95277130504205587, 1.3600861471829218, 3.2762272989730619, 0.16021729886077374], [3.8336749894711772, 0.41373378820194961, 2.5916567916264981, -4.8590016715441013, 4.7189103264585306], [-0.8866303405770406, 0.067821871920897614, 0.13162215248339937, -1.4730403911977277, 1.1721324868608356], [-2.49462288580487, 4.8608573274547435, -2.2303416989494051, 2.6039866399229528, -2.4554571651455559]], [[-3.5138376516484451, -0.51753540444570501, 4.6418039022518318, 4.9036945920631485, -2.8009904778294779], [3.812938249071923, 0.54093642583282087, 4.7263875634912935, -1.0721925422348977, -3.8691089325521277], [2.5091253358520236, -0.78913163614482151, -0.088392190934278858, 2.824954887623397, -3.5516026497078554], [-1.4776418105178433, -2.1910080796967057, -2.1409241560748979, 3.0933446385876593, 2.1319160136002067]], [[4.6401788238316719, 3.2396822293099099, -0.15376760999833206, -2.7816716646811757, -0.13351793754341834], [-1.4661271984571234, -3.7306090842382345, -2.1035633066313553, 3.1242092208894405, 0.58939923412648199], [-4.4989333044067799, 0.47961610621346829, 0.93061481068175311, -0.29388049054020549, 2.7057711092529129], [0.63984916627853217, 4.8971228523385708, 4.025173993261042, 3.6251558736423242, 0.78939756723680343]]], [[[-3.458402538402904, 2.8586557899140868, 4.4289333398315058, -0.0084493749423275588, -3.2678465074611496], [3.9371191045420844, 1.9533439858214354, 2.1710009459571067, 0.35328023782879825, 2.9003988632090998], [-3.0084696898092647, 3.3424362177155551, 2.0899835757863281, -0.23772271027671721, 2.6836328822133213], [0.98298870831364127, 4.6928671080785929, 2.3424762560944954, -2.7299797435449746, 3.8159365284454854]], [[-1.4123545249978617, -2.3215916039569873, -0.98346503162354271, 3.2518803874247233, 3.17675441091599], [-3.4150729858183873, 1.9178522702779635, -3.3014667458661364, -1.355847178974936, 3.3953542778652626], [-0.53993914071459947, 0.55786127996026558, 3.7921795179425324, -1.1682667104266766, 1.8402748076378543], [4.3626799982547375, -4.4492172166225119, 2.0536999418261734, 3.0724107001089855, -0.13198833077278316]], [[1.0947658335496611, 2.0557117436164072, 2.9751553728445659, 4.4776783469222732, 2.5803475911063689], [-1.4097049031047049, -4.5655213745479228, 0.2037581219575042, -2.2614492732392142, -2.3251789686429825], [-4.4529784700894979, 3.7829142933065771, 4.5946378835358512, -4.5445796040816857, 3.2704284053965438], [-1.2126664608554005, 4.8691943547172336, 3.8827142687802496, 0.31560915748924856, -4.1444086741797292]], [[-4.1402915835278895, -0.82600931415015388, 4.8733643127104749, -3.905386721648231, 2.9495600590458473], [3.7525873605389322, 4.6243777719708437, -3.4437742769503021, -0.97132031027102972, -3.1582542004950165], [3.1853014195438529, 1.1915049584931037, 1.9364395775336574, -3.006378160406463, -2.1223558166893799], [-4.1441351498588546, -3.6553319434629215, 1.7078736999085811, -1.6792454732053255, -0.28616870405767614]], [[1.2315198093175503, 1.0500300239340765, -2.8473107184678295, 4.9742602988789049, -1.6379118685492235], [1.6315299516705162, 1.9846492836836207, 0.53558112826900928, 4.4591771428770812, -3.8653851974208644], [-3.8451974628119681, 3.1983373145369622, 0.91110744380416353, 1.2464062603686346, -1.1187699608604049], [4.2149914126871444, -2.3288084960086284, 3.7418505277624945, 3.145303387646937, 3.7794268518747103]]], [[[-1.3094890103101986, 3.0081105499532299, 3.3083134071240003, -2.4107805845304684, -1.5321464088900525], [-1.8911239278929273, 0.44755179518925203, -2.6421029248104588, -3.6340168429328603, 3.7879784740502913], [2.2204693809922027, 2.3061218942200661, -0.55424328963735903, -4.5645700624289614, -4.9241255305437894], [-1.1254556069943131, -0.48393714362424589, -0.93666719155739209, 1.2948282172193686, -4.8089104812571808]], [[-0.72106233603143188, 1.076213373589642, 4.2324821372073238, -3.724563675483882, -3.8520363421710844], [1.5592952552358881, -1.3715090732699764, -0.017797060275674959, 1.9016362678385699, 4.2159696747908537], [-2.6216211350965359, -2.734812742745647, -2.696708620925433, 2.3859127173818031, 0.4115179118962109], [-2.8050659658810262, -1.1100612894233275, -2.4885421082123891, 0.56483593480716721, -1.8794749837880662]], [[-2.0347986694086382, -4.7195946175251038, -0.94839488779156333, 1.6547183373716488, 0.92469650899149869], [0.57352824151209347, 1.496228694252288, -0.092493866878885278, 0.59400893332823834, -4.0449025306954445], [4.3105996155119222, 1.3436376997629456, -4.4654195493187343, -4.2311682442243388, 2.552451401119356], [2.8126107837855301, 1.5655844205113603, -4.4873134102326082, 4.1600792940050617, -0.67371678464800233]], [[-4.7810526530585262, 2.8011830880386137, -0.31352997514342817, -2.332823338780762, -0.47228566834452934], [-1.0970167681472454, -4.0939032251036176, -4.4099749308193985, 1.4391291086116969, 2.532935446095868], [3.9394636783290142, -0.68627579964241647, -2.9776592237357544, -1.260931429340423, -2.1452042859772833], [1.5228804372901541, 2.229595760087193, -2.0150326067164137, -2.0928852405009959, -2.668941791631676]], [[0.81518152538946875, 2.9885256930693016, -0.17512229592400175, 1.2203093448038107, -2.8009563108734445], [-4.5861318556609598, 1.4255593829945115, -3.0213143074087547, -0.096871701013742317, -1.6875993036618784], [1.5759042791749822, -3.8746641250917668, -4.3868944540543007, -2.7014251191056049, 1.2953725921267232], [2.5976208523153357, -2.788429241290169, -2.6319569348252534, -4.3240522996999475, -3.0521493627503373]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-9.3665504255904413, 7.5396088534971328, -1.4207998730335878, -6.5281220195769087, -9.337165121347816], [-3.6347163219333853, 1.8144919353799904, -5.4371342896195403, -9.9155598837057841, -6.4103011302601161], [-7.0924430253087571, -3.7495252035673818, -10.938317930987449, 8.4925584043195919, 6.1279783460112132], [9.3893623959191324, 1.6801082768408402, 7.863583758579745, 11.12907600343331, 5.4190188868937046]], [[-15.447553441337957, 15.273882994826133, -2.0521148227719928, 1.7561640989322846, -15.070902311550448], [7.5264956390336843, -11.793002744675791, 7.0090647640714074, -2.5979551919847546, -1.6147352122228675], [-4.3961558974570671, -13.530130207859726, 14.051316222830517, 14.320214507405815, -7.2477630473781423], [-11.004661790799505, -4.306799617799876, 2.6910632552220166, -6.788726450881315, 12.589179395869023]], [[8.0514059515417191, -0.19339957207308803, -3.8020515867592684, -7.2732827390035188, -6.1393278370009234], [-0.97228967919755316, -2.8463201954141053, 6.0594120096380601, 5.9903447892338288, -5.4818975897178577], [8.3181965446060371, -0.83266817838872886, -1.3344976778963988, -1.3271706325999006, 5.5108127783310152], [-7.0345522073539648, 3.2605266553371406, 3.7663370541893038, -2.1134522650065701, 3.8956492749219405]], [[3.2722831894718549, -1.4827691797441829, -1.0597879454866861, 11.494307974676738, -4.6054618944635406], [-7.2434192044108849, -3.6712050266560197, -5.9007164965685774, -1.7878983887132409, 8.3646579042228595], [-4.7942339304762509, 4.7304869363851427, -11.907701877057638, -3.1815853484975847, -11.90988173740015], [10.571757022302775, 0.22511506408172788, 6.7694324932572627, -3.183110052861208, 8.8536614312639941]], [[0.93482576271669549, -14.10552377361406, 1.6988745547206878, 2.6342736024589981, 0.15086811268329861], [10.973288285742472, 10.692942642589374, 5.0463160053898584, -4.1236979353221344, -10.449282462435944], [-12.763605187293134, 16.264311285020931, -1.2157089473485139, 6.529235191324009, 3.8207580929262641], [7.5034303673308926, -8.498685283273419, -8.2969544674077618, 4.5719921119445184, 5.3309469161080862]]], [[[1.9786691883050016, -4.4380757311996488, -3.9090945114074769, 3.1755945655919997, -18.670139081719043], [4.1067696613062363, 14.739807192925875, -12.11204091761631, 12.748524073104109, 16.00739628770873], [-18.84383569387531, 16.364330105654627, -20.096216747553949, -7.506521611351161, 12.236153632949028], [-1.0613487856276993, -4.0603632341811879, -20.518862483711548, -18.822753156150178, 12.535261998372581]], [[-9.2631230833690346, -11.877747191443794, 2.5165155087030229, -9.9611345810210068, 7.520317201025458], [5.4043739308296823, 3.5007070091715331, -0.11326779122552449, -6.0021826518474768, 8.5177603437546381], [-6.0962583289369379, 8.5048662602254268, 1.1446486108790266, -11.932859777456297, -0.048395166251708012], [-0.30443981913225393, -5.2627638781822581, -6.4584456230543763, -2.7894577223086992, -9.7216353548719905]], [[-4.5716487813332432, 1.0925699344684514, 1.5596494403591434, 3.7569429582944336, 0.18372573018380428], [4.3961840683635822, 0.47444029377357794, 2.9719265011524385, -5.5719553157897641, 5.4113085888052828], [-1.0167242107057777, 0.077773268115869612, 0.15093486312230664, -1.6891772822747253, 1.3441176361848535], [-2.8606549635193081, 5.5740832491626078, -2.5575962153434544, 2.986065488624011, -2.8157425184994955]], [[-15.065521231987882, -2.2189245482996145, 19.901658009524823, 21.024510041678724, -12.009200679644199], [16.347910132018512, 2.3192560432371812, 20.264309068171364, -4.5970079187469004, -16.588741015081453], [10.757833676027602, -3.38338893192627, -0.37898006717458249, 12.111947693124737, -15.227438041042255], [-6.3353649987745113, -9.393911163949868, -9.17917716383133, 13.262664343253164, 9.1405549008905336]], [[-22.689692000965575, -15.84149981171285, 0.75189768391419964, 13.601905382454857, 0.65288020019941773], [7.1691147755725853, 18.242049350044709, 10.286069857235656, -15.276856272108525, -2.8820628677607205], [21.999025228424944, -2.3452418843849188, -4.550549499829331, 1.4370260433074704, -13.230764465099563], [-3.1287545288925185, -23.946104973320526, -19.682422084322418, -17.726400932219647, -3.860020992062597]]], [[[0.025593882473794335, -0.021155461085764802, -0.032776288510435764, 6.2529536932048408e-05, 0.02418367397248777], [-0.029136620890150771, -0.014455707757805997, -0.016066476485696261, -0.0026144477940037306, -0.021464380391757715], [0.022264157746804344, -0.024735674572998325, -0.015466908035389616, 0.001759265163789718, -0.019860205348419722], [-0.0072746006846457125, -0.034729528415383654, -0.017335478253443191, 0.02020319495381695, -0.028239810129677443]], [[-5.8773797074068543, -9.6610837721524518, -4.0925966657112536, 13.532392513128631, 13.21976287091311], [-14.211503068744523, 7.9809607401179488, -13.738741451521564, -5.6422297341130996, 14.129445531540716], [-2.2469056406892496, 2.3214869272227827, 15.7807962173188, -4.861631364625751, 7.6581294703460889], [18.154879980783143, -18.514996426171358, 8.5462779702639207, 12.785546392232911, -0.54925694871097175]], [[5.2471374428145809, 9.8528851842123135, 14.259715344263659, 21.461204753564473, 12.367428764990278], [-6.7566187706244829, -21.882230351203233, 0.97659868278333628, -10.838971031973687, -11.144423084572031], [-21.342819940492678, 18.131248366609704, 22.021783778824588, -21.781857883809057, 15.674938707540653], [-5.8122270511204928, 23.337714086433628, 18.609582793983964, 1.5126930132514755, -19.863866103771251]], [[12.739176400743801, 2.5415307471291606, -14.994752517359776, 12.016402554384056, -9.0754395285262017], [-11.546257402569969, -14.228651048440907, 10.596076897809223, 2.9886351056522145, 9.7175661585200714], [-9.800787179953323, -3.6661166351165329, -5.9581903520339079, 9.2502619538036672, 6.5302321318424505], [12.75100258943373, 11.2470142480568, -5.2549207934740716, 5.166835202723842, 0.88050648796495024]], [[1.7405649600110893, 1.484056896845251, -4.0242383673677518, 7.0303564040929025, -2.3149380013325946], [2.3059181375734097, 2.8049983239856195, 0.75696203833280662, 6.3023651155682652, -5.4631309871536828], [-5.4345987108424145, 4.5203607914873256, 1.2877110700879411, 1.7616046825412499, -1.5812102878521257], [5.9572459201743619, -3.2914147511561089, 5.288533619147838, 4.4454054443315858, 5.3416420081326539]]], [[[5.1955248873214241, -11.934970895551029, -13.126054901140556, 9.565006217068051, 6.0789397511039329], [7.5032179384607911, -1.7757052346092759, 10.482800078990898, 14.418314930287474, -15.029172661711831], [-8.8099280250907626, -9.1497626939067889, 2.1990141057080455, 18.110375247558711, 19.536946504174736], [4.4653544777204708, 1.9200676399746262, 3.7163180954997839, -5.1373567661905462, 19.07981960104728]], [[-1.5185611777270507, 2.2665111827622186, 8.9136302616506171, -7.8439512354666183, -8.1124093606238965], [3.2838842370377166, -2.8884055226501291, -0.037480705151031746, 4.0048562602658695, 8.8788549265355758], [-5.5211482829312315, -5.7595304205512434, -5.6792829340079489, 5.0247450809902254, 0.8666589471089029], [-5.9074840882615103, -2.3377950764889421, -5.2408831328913452, 1.1895475321927531, -3.9581844762521641]], [[-5.231746034241799, -12.13472408581922, -2.4384531343047886, 4.2545074505318308, 2.3775213570389666], [1.4746196506648008, 3.8470088737320469, -0.23781439830405005, 1.5272783140493138, -10.399998334975651], [11.083141925885409, 3.4546765303495737, -11.48120517749485, -10.878912992588054, 6.5627020973441299], [7.2316074976656841, 4.0253319438534341, -11.53749729215585, 10.696133566308891, -1.7322181153719203]], [[-15.619783039959637, 9.1515143767298088, -1.0243079387823544, -7.6213748449307674, -1.5429655784494825], [-3.5839730605551896, -13.37485378283843, -14.407465600069139, 4.7016600891489997, 8.2751446163076849], [12.870297069398807, -2.2420750980711137, -9.7280650134512889, -4.1194851393163487, -7.0084201021967942], [4.9752771517908405, 7.2841285312131321, -6.5831469384090484, -6.8374928611686903, -8.7194797373543214]], [[1.0110367191687726, 3.7065477047321265, -0.21719711010343884, 1.5135003896855712, -3.4739129762877976], [-5.6879940977655945, 1.768063721604124, -3.7472140987012645, -0.12014605793039675, -2.0930612509043662], [1.954530423613164, -4.805589409109948, -5.4408879961401091, -3.3504684593971126, 1.6065983033892128], [3.2217242201539431, -3.4583761578764349, -3.2643098763896505, -5.3629474104115902, -3.7854575723534016]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-0.74081943270460648, 2.63034633870768], [4.8419676172794226, -0.052720610651687316]], [[4.7072093605755185, -1.1550908276761804], [2.0539179363739235, 1.9327102051605856]], [[-3.5795135865245631, 3.8845777889676114], [-2.6294109863605319, 0.86259884899085559]], [[-3.8425658428081713, 4.4394013018892462], [-1.2132536859016763, -3.3518725873255226]], [[3.3871876381115129, 1.0434540871893114], [2.5737657301882031, 1.6052680298840869]], [[4.6189288536631121, -2.5559092092251765], [3.2637873353810356, 0.30302350199404593]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-1.3503591980240239, 1.9591430901808806], [4.02018780986349, -3.8477265289245812]], [[-2.1309693200658231, -1.2639756578308594], [-4.8024687007780464, 1.620919107612627]], [[2.0858964716474437, 3.1034436075549561], [0.21487294472522755, -3.6129270033530703]], [[2.4891892962732403, -0.77911571958508574], [-1.1854525323058307, 0.76852705170255398]], [[-3.667942481804328, 1.5046076802399844], [-1.0342834426916414, -1.6802441183426655]], [[-4.1617160382129859, -0.81315705818087736], [-0.84747866579163134, 4.5752743550894213]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.0003723350276048, 5.1532248542617296], [19.465619190740505, 0.20285449222560115]], [[-10.03091873051309, 1.4600066887663921], [-9.863876603402403, 3.1327669010227135]], [[-7.4664947603456726, 12.055568107221498], [-0.56498928153215267, -3.1165066745803394]], [[-9.564873766143263, -3.4588073398484065], [1.4382546542815251, -2.5760047572198954]], [[-12.424009431671683, 1.5699890335628404], [-2.6620032801008211, -2.6972421655762551]], [[-19.222670289654495, 2.0783556135509569], [-2.7659901364163435, 1.3864156576627464]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[3.5036138537560042, -0.76485991865055958], [2.2742719286524329, 1.0210957065350872]], [[-4.7630020968419799, 4.6703684474986975], [1.7365954795122835, 4.3810474908159858]]], [[[-2.7485903353975063, -2.1080364780681027], [1.3290132775399899, -0.8005982835013139]], [[0.656123325623299, 3.3380119042883578], [-1.6033718012789331, -4.4497353577141574]]], [[[-4.5302215902589218, 4.8518776848133811], [0.2559710610189132, 1.0250279921335945]], [[-2.5101832713880357, -2.1027431914123151], [4.5390349776075478, 2.140199760337083]]], [[[3.7621784993748904, -0.81544155143228814], [-1.8684441986861335, -0.59866882156041878]], [[-2.1722859727132482, -4.5969164324320122], [-0.77926848182500486, -4.1891668738542478]]], [[[-0.80176615316886135, 0.12266655241777613], [-1.1034305986755442, 3.7512544665750163]], [[0.14641127156915879, -3.4743689413282555], [4.4394888264813694, -0.24549775143099417]]], [[[4.9081910086412357, 4.5700190364997884], [2.5164245507900604, -0.48960110046808047]], [[-4.9468291737401646, 3.2633780143046547], [-2.7300741955766616, 0.21667305372764112]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[0.7104248438422287, -4.0060262326450804], [1.5856678826602781, -2.6369372292403339]], [[0.92058657002345168, 3.9926500736526549], [-4.8113310024975338, -3.6672913510515071]], [[2.6244685968288914, 2.052403908334969], [-4.1534482037858229, 0.85338496581825662]], [[-4.2996983195649126, -4.2618310593917048], [-3.7434634058210725, 3.0835179602065157]], [[-3.4741872454668155, -3.22657300001429], [-1.8978874514230206, -3.7552887853525996]], [[-2.4141159952179523, -0.43033778326169347], [-1.5983008240526928, 3.8689593319345885]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[2.4890543249380785, -0.54337548826850357], [-9.1107930063499669, -4.0905361864208221]], [[-7.5525394500058871, 7.4056532473886296], [-4.5792932720564101, -11.552547231602622]]], [[[-2.5303153492631991, -1.9406300708290318], [5.3062849604553968, -3.1965087955877101]], [[-3.1568264980331628, -16.060280161808407], [5.8800315393501066, 16.318475991813212]]], [[[-11.889424300310782, 12.733600619447584], [0.5253560060558663, 2.1037714572077353]], [[10.425916199719858, 8.7336349313943487], [3.8735442092134886, 1.8264142993195027]]], [[[-16.176232571665459, 3.5061526683968145], [7.9629935187008094, 2.5514253780156229]], [[8.1318730458304778, 17.208388444426795], [-2.4028883595302672, -12.917371293831756]]], [[[2.7854857431862512, -0.42616657185522439], [3.5602993770761144, -12.103696378033955]], [[-0.27787211505799453, 6.5939612153607809], [-16.671562602783659, 0.92191495277809254]]], [[[-11.848942421545742, -11.032556054464674], [-1.0829125629322973, 0.21069385225791942]], [[7.9065211448368062, -5.2158597694585698], [-10.56254603585014, 0.83829923319832156]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank3_Symbol_rank4(self):
      arg0=numpy.array([[[3.7270593938106789, -3.8289987773443448], [0.94397517760008931, -3.8922616324395563]], [[-0.032631279684091297, 1.2535624901095543], [-2.7769446883242619, -2.8110424269712575]], [[4.3074009869614152, -4.5774251755483295], [1.2549719362652603, -4.3739077493698897]], [[-1.2011707074280462, 0.90244159990658268], [2.9591422323291621, -0.95407908435115196]], [[-0.26362892060033527, -2.5214419823837106], [-3.8894705219879908, -3.4702075353137829]], [[-1.8144959155830187, -1.970699129000093], [-2.6941009844502584, 0.44395877269478312]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[2.2641233141739949, 0.035480032164088904], [-4.3271493039348297, 2.0528572951658699]], [[1.8772405373538481, 3.7589920843792548], [1.4925885026921302, 0.29537612611676334]]], [[[3.1334461059330589, 0.17619410454172701], [1.1443880634559545, 0.91161768954384215]], [[4.2761361344553279, 1.9116881181324583], [-2.8730315183983492, -3.1257821544438746]]], [[[2.1956095321051468, -2.6436201517892108], [-4.148053829872226, -2.3983254027119072]], [[3.2979770285195844, -1.5942264205879475], [4.9934510910706145, -0.54092231929744372]]], [[[-1.7242881023765397, 4.2414301918881172], [2.7881563128697451, -4.0876396193255147]], [[-1.1504529303207276, -4.3871749180738071], [-3.4767390984708602, -0.22048668844742014]]], [[[2.0336289374503025, 0.67194489778317124], [-1.0072626987992352, 0.55599983585485813]], [[-4.2093272248994582, -4.4238230972062622], [3.5078121361664607, 1.2563050867679131]]], [[[1.2154328906424894, -3.9157853717666988], [4.2945732279613562, 3.6380072352231547]], [[1.0469477016452426, -1.1413359539440493], [-0.87053124614973854, 1.5665358969496532]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[8.4385220668379546, 0.13223618716987257], [16.568649394152896, -7.8603880732525342]], [[1.772068469646686, 3.5483952204492368], [-5.8095449620489834, -1.1496811628229056]]], [[[-0.10224835625772841, -0.0057494391039891145], [1.4345619504774969, 1.1427697409324975]], [[-11.874593525127164, -5.3086521653805345], [8.0762134922434132, 8.7867062536113547]]], [[[9.4573706655716006, -11.387132050967933], [18.987406030386794, 10.97815507753057]], [[4.1388686172395719, -2.0007094178904916], [-21.840894423333292, 2.3659443241822231]]], [[[2.0711643597413918, -5.0946817040969234], [2.5161482437758114, -3.6888560379056519]], [[-3.4043538524189039, -12.982274580687434], [3.3170840555969279, 0.21036173782553233]]], [[[-0.53612340168162997, -0.17714410810548004], [2.5397544560415102, -1.4019213283228913]], [[16.372054158647956, 17.206329531073372], [-12.172836107389989, -4.3596393787550483]]], [[[-2.2053980157360589, 7.1051765633704074], [-8.4633117197705623, -7.1694176897503077]], [[-2.8205828336703838, 3.0748743171091379], [-0.38647998363309805, 0.69547735419208934]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[3.0115763876702566, -2.5408656928843012, -3.2272302579707937, -3.9843017736844333], [1.7088286306394025, -3.5330988307007418, -3.9474195138779864, 1.1183691850237052], [1.1636231119191471, 2.181858865797512, -2.5327836332838114, 1.9887316766124119]], [[0.15125450409669661, 3.8760890387798668, -2.6415557082902508, 3.8113804900752992], [-0.11246026436472345, 3.599535963360065, 0.88454536016060104, -0.49191959832143795], [-2.6190381401703142, -0.55621313770519976, 2.9905519903715838, 1.7848419939914546]]], [[[-2.7761239388877779, -3.3779460309929741, -2.3496329927289814, 3.2709606491467227], [1.5659844279756721, 3.9317193644873782, -3.702698884754867, -3.5957592690396423], [-4.5993841356780152, -0.17580450818879356, 0.31293810072431061, 4.0944891937637227]], [[-3.2687954484317072, -3.1489102983000485, -1.2549831369668318, -0.67036883378702949], [2.2733693602353933, 4.2336299563066699, -0.17599933901042952, 2.864707442875031], [2.4212468519193431, 0.21094019375253303, 0.45003292315417553, 3.4350584209566541]]], [[[-1.4494887215822949, -1.4384412265239934, 1.0633758122315502, -1.9423721463822119], [-1.0363473067195916, 1.0404134034623436, 0.10873103636000359, -2.8113777479524371], [-3.5854267922210967, -4.1138665971372195, -2.7470168090773495, -2.4443531432647037]], [[0.44963013872478008, -1.6440464713809044, -2.9133948634548412, 1.3895222674874299], [3.3402855375554061, -2.9483377998840012, -4.3705759275846319, 1.2245046114778821], [-0.23581783924746436, -0.14840360470320757, 2.7330521538447394, 4.5574432752408178]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[4.1319588831325049, 4.5300773878485554, 0.53137806428650158, 0.42510417688857416], [-4.6691989372239098, -3.8209997249822178, 1.46563151463452, 4.5332117142762911], [-0.63296953391732114, -1.7936603445950139, 3.3709430067312081, -2.9382869936217015]], [[-3.1298788883764637, 1.3033922142823107, -4.9806209663523351, -4.3136014388876553], [-2.092225498317668, -1.8145944541851087, -2.5502213890934766, 2.4079298557584385], [-2.8620354693946473, 0.58998244175535319, 3.4377535085310598, -1.0806459929246115]]], [[[2.6051129244963578, 0.36589266988266012, 3.0276574310893949, 0.24240158147201729], [1.154538653366604, 1.9524537270508011, -1.4759645051598014, -0.24760348772886154], [3.8528212413769563, -1.1266432283502201, -0.91329878948139243, 1.5631473804843479]], [[1.1150827980128852, -3.3192571935402517, -2.9497199268373309, -4.215632406890947], [-3.2724895688392674, -0.57798049558959796, -1.9825725206658529, -1.4020733992175858], [3.3164995954175467, 2.9956659230049922, 2.8274324062336671, 2.1171415230022177]]], [[[-3.9155201289254347, 2.1593081958197677, -4.8362527255189649, -2.6727094049401634], [0.24976531221460796, 2.2195627891738763, 4.4454004683589474, 0.83997414553486838], [4.1068329626129803, -1.4647161269730669, 2.6397052436060395, 0.92383011520517222]], [[-1.8619620140914672, -0.49456980356431757, -3.7481675000534898, -0.86199263401127446], [-4.9619835082948622, 2.4677616804603026, -1.1664004225285129, -3.790177697625865], [-1.9224990907087069, 2.0853018577044917, -4.4790669508779608, 1.1473553264180421]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[12.443709807266217, -11.510318220895325, -1.7148793674873475, -1.6937433259778072], [-7.9788608260792868, 13.499969660442529, -5.7854624410228537, 5.0698042904350888], [-0.73653797880688532, -3.9135137250840515, -8.5378692761813255, -5.8434644191937295]], [[-0.47340827914410205, 5.0520642750106841, 13.156587744498117, -16.440776366137147], [0.23529223265142021, -6.5316979967530262, -2.2557864971049577, -1.1845078874308892], [7.4957800528648288, -0.32815598511972027, 10.280780597344457, -1.928782348810439]]], [[[-7.2321163532002872, -1.2359656919995543, -7.1138837907687149, 0.79288603428590188], [1.807989552668104, 7.676500126911189, 5.4650521271929655, 0.89032253604759715], [-17.720604895192448, 0.1980689586643451, -0.28580598857411893, 6.4002900576532324]], [[-3.6449775747690119, 10.452043159445417, 3.7018487669558873, 2.8260285802822924], [-7.4395775174891234, -2.446955540289097, 0.34893145317743124, -4.0165301021957127], [8.0300642047965098, 0.63190635021653374, 1.2724376707981815, 7.272504816945764]]], [[[5.6755022660058705, -3.1060379296382981, -5.1427541701557775, 5.1913963035295492], [-0.25884360862558686, 2.3092628756827649, 0.48335299995991365, -2.3614846216120911], [-14.724748935329321, 6.0256467490426981, -7.2513146751954105, -2.2581670459443561]], [[-0.83719423869621734, 0.81309574040146337, 10.91989194202421, -1.1977579593688084], [-16.574441750345763, -7.2757950436063741, 5.0978416086276619, -4.6410900690634938], [0.45335958152614225, -0.30946631257764179, -12.241523577311801, 5.2290068166956392]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_float_rank0(self):
      arg0=Data(-0.413950098176,self.functionspace)
      arg1=-2.11628145739
      res=mult(arg0,arg1)
      ref=Data(0.876034917054,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([3.5468122132869411, -4.455878267895975]),self.functionspace)
      arg1=-4.48121208267
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-15.8940177451599, 19.96773553302339]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[1.6378639322016451, -3.3360447623805345, 0.61928036026330346, 1.9613535634341117, -4.6873736424863299], [-2.5968748876023784, 0.057519677999939844, 4.9232126083468959, 1.0720533003976449, -1.4345700351600641], [0.0083776326707960536, -4.8293168354474574, -2.1808617186581625, -0.023647653213894948, -2.2122332907361697], [2.2499034085112957, 1.4527298971028744, 4.6428703008648835, 3.283105907241783, 2.866417532376202]]),self.functionspace)
      arg1=-4.66911280908
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-7.6473714653758247, 15.576369331701283, -2.8914898625181968, -9.1577810461684912, 21.885876315085447], [12.125101801287187, -0.26856586532378113, -22.987035051465629, -5.005537796905088, 6.6981693266908175], [-0.039116112012996224, 22.548625095502221, 10.182689385423027, 0.11041356052572186, 10.329166794453485], [-10.505052823876946, -6.7829597706991507, -21.678085192673809, -15.329191845074812, -13.383626816594438]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[0.40501025729360318, 4.8892767051813024], [-4.692088626938121, 2.4552206173801769]], [[-4.4352475064019075, -2.745055491187304], [2.1538357221089344, 0.99732905568296992]], [[1.2400293932652229, -4.7749269519327839], [-3.1833887815589113, -0.38924876870408376]], [[-2.789972283209198, -2.1490223534624553], [-0.88153086148501103, -3.3134281332407767]], [[-3.4995274868218007, 1.2342154554530671], [-1.9890139377733949, 3.4545542298934109]], [[1.5679594882803327, 3.5452237270761362], [-2.3426042296124363, 0.58499948893002873]]]),self.functionspace)
      arg1=-3.58951765324
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-1.4537914682989741, -17.550145044827143], [16.842334956964621, -8.8130577486868429]], [[15.920399220721743, 9.8534251447423351], [-7.7312313466905973, -3.5799302514639653]], [[-4.4511073976629563, 17.139684586897861], [11.426830228534332, 1.3972153267655412]], [[10.014654762631867, 7.7139536749625046], [3.1642705891769891, 11.893608777012371]], [[12.561615691948152, -4.4302381652513789], [7.1396006421795351, -12.400183392279965]], [[-5.6282182627486153, -12.725643153027821], [8.4088192367503289, -2.0998659926511656]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[2.7628158753647734, 3.561696131430093, 2.928084195988145, 3.9751116931635604], [3.6107688739051902, 2.2909805685122775, -3.9374744601399603, 2.0651840445284435], [-1.6095925878749959, -2.4147895377131134, 1.2872131401151607, 4.1133580826904552]], [[-0.90424732564670407, -0.56588406258107327, 4.3186291144415652, -4.1749600024456877], [-2.7473813266913316, -4.2358127664598655, -0.10444049359189123, 2.0005917853512436], [-0.078256744980508941, 3.4576901046532562, 4.2214255586517773, -3.9420466395813114]]], [[[-3.1747992222892241, -4.2010980301826573, 1.9303262074828256, 4.7613622375296654], [1.216084530325082, 3.4116055890418142, 1.1403153895137113, -1.9875641319287896], [4.5504383607068846, 1.8022581847787382, -0.29695050791061206, 4.9189485336215526]], [[-3.5264578468344885, 1.2551924891116562, 1.2327729194805439, 4.5083880871709265], [2.8250986936189273, 0.074082211174254731, 4.3632523875770683, 0.79863379214183094], [2.8102533375761158, -4.499530565959371, 3.8779207330275707, -4.1132279477043987]]], [[[0.503835440415906, 0.66497057889350675, 2.8400670102595527, -0.045434758374644169], [-4.5927722143659802, 0.49977001479484695, -1.1513999338112746, 4.2889354724231072], [-0.88218175147574041, 1.3861054870134417, 0.27002541811377601, 3.5713836020978178]], [[-3.6507891869474927, 4.4769712365239798, 3.7565080285947783, -2.765900348318171], [-2.7378866146630898, 3.1561279560917335, -3.4843617115506245, -0.75851651233082684], [4.7315078994383022, -3.3406518283316258, -0.096816753110839926, 0.25802693464901694]]]]),self.functionspace)
      arg1=-0.493151954274
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-1.3624880482341539, -1.7564574077433281, -1.4439904435290789, -1.9603340999392891], [-1.7806577265964856, -1.1298015445645808, 1.9417732249202515, -1.0184495474937763], [0.79377373029479081, 1.1908581796825681, -0.63479167561439709, -2.0285105771057412]], [[0.44593133578931221, 0.27906683135411769, -2.1297403875695511, 2.0588896842200373], [1.3548764703925158, 2.0888993437165766, 0.051505033520136363, -0.98659574864959598], [0.038592466722225577, -1.7051666323821029, -2.0818042640694783, 1.944028004147043]]], [[[1.5656584408981087, 2.0717797036793799, -0.95194414160562457, -2.3480750924420843], [-0.5997144626916584, -1.6824399634465699, -0.56234876282690227, 0.98017113590471405], [-2.244057570383974, -0.88878714592915331, 0.14644172329864452, -2.42578908232653]], [[1.7390795788297579, -0.61900062899490726, -0.60794437441735205, -2.2233203958119785], [-1.393202941773855, -0.036533787217490274, -2.1517464419224073, -0.39384781534364793], [-1.3858819254294557, 2.2189522919164806, -1.9124041880105023, 2.0284464007830478]]], [[[-0.24846743207338376, -0.32793154051575324, -1.4005845963773647, 0.022406239884402761], [2.2649345930478879, -0.2464625594834047, 0.56781512750947749, -2.1150969099786492], [0.43504965476473328, -0.68356062974998322, -0.13316356264634374, -1.7612348028350846]], [[1.8003938221839366, -2.2078271145183161, -1.8525292755458325, 1.364009162099026], [1.3501941346005237, -1.5564506694840545, 1.7183197874471563, 0.37406390040471199], [-2.3333523672687848, 1.647448977689282, 0.047745371003031466, -0.12724648707737882]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_array_rank0(self):
      arg0=Data(-1.79975212775,self.functionspace)
      arg1=numpy.array(-3.903612086)
      res=mult(arg0,arg1)
      ref=Data(7.02553415769,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([-0.69110498776625029, 1.2336798325440457]),self.functionspace)
      arg1=numpy.array(-2.05344291784)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([1.41914464260985, -2.5332911150148654]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_array_rank1(self):
      arg0=Data(1.33117238006,self.functionspace)
      arg1=numpy.array([-1.1963779602114677, -1.2538461782413424])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-1.5925852967460625, -1.6690854013186993]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[4.7909907860593552, 3.3615456866924802, -4.2788081197407664, 0.34359794954010603, -4.1758347664764619], [-0.55121543946773244, -3.9768308292308205, -0.41426087176369819, 2.0375858553524129, 4.1084760252909049], [3.4377544148637789, -4.1244660532789457, -1.3171955596766063, 2.9488359441721981, 4.4364982556421335], [0.28532825966689401, -2.1191142271356767, -4.1678037877714171, 0.11914459097769381, -3.7138651368350226]]),self.functionspace)
      arg1=numpy.array(-2.73283516883)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-13.09298811366827, -9.1865502742113208, 11.693277310289336, -0.93899656043998569, 11.411868109036616], [1.5063809385777518, 10.868023150596722, 1.1321066794246928, -5.5683862850111563, -11.227787772196823], [-9.3948161667294094, 11.271485883033066, 3.599678349706759, -8.0586825753344318, -12.124218459457673], [-0.77975510267783976, 5.7911898866776479, 11.38992076799137, -0.32560252839932857, 10.149381258222588]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_array_rank2(self):
      arg0=Data(0.473116093275,self.functionspace)
      arg1=numpy.array([[-1.9903700158844675, -2.4392195592704247, 0.47730474072663753, 3.7567047875344795, -4.5018455453922988], [-4.3039301000395973, -4.7700298337574427, 1.6548292206683186, -1.0779033925836168, 4.1529456222587253], [-4.9944376776815114, -1.3039286170596842, 2.6073040386815149, 1.2402801915156978, 0.49932686446126695], [-2.1446302481895838, -0.22499848194486471, 3.3899891206101138, 2.0007586692212573, -4.9626253093016945]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-0.94167608608773201, -1.154034028522938, 0.22582055423440892, 1.7773574926672608, -2.1298955769652146], [-2.0362585946610858, -2.2567778797543716, 0.7829263359205505, -0.50997344202744799, 1.9648254083881749], [-2.3629488421720803, -0.61690961321325777, 1.2335575007621404, 0.58679651877675743, 0.236239575381364], [-1.0146590845436825, -0.10645040277064741, 1.6038584089891266, 0.94659112516882638, -2.3478978987263837]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[3.425441390987876, -2.9586474615067093], [-4.3443588238150035, 4.234142889951805]], [[1.0611310092939021, -1.4888079760745745], [-4.8512078096029363, -4.8515112670991982]], [[2.1546318785454845, 4.6977027041078916], [4.6978711152085104, -2.9245729694586631]], [[2.8887195261357768, -1.5204723556879829], [-4.5426446571972647, 2.8555896730199679]], [[-4.0309699841552575, 1.6756042307496575], [-4.9862451818413058, 0.20878660681349448]], [[2.6427653832515787, 2.6260505048768916], [2.1489683634658991, 2.8420967658933582]]]),self.functionspace)
      arg1=numpy.array(2.18600625697)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[7.4880363135686574, -6.4676218630092395], [-9.496795571363581, 9.2558628503213161]], [[2.3196390257767581, -3.2545435511194034], [-10.604770625632703, -10.605433985618255]], [[4.7100387679581077, 10.269207504544362], [10.269575652264056, -6.3931348101892835]], [[6.3147589587516899, -3.3237620830772503], [-9.9302496438048546, 6.2423368925481784]], [[-8.8117256070041687, 3.6628813326168861], [-10.899963166269979, 0.45640882886492845]], [[5.7771016634802068, 5.7405628347687205], [4.6976582885577143, 6.2128413131447537]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_array_rank3(self):
      arg0=Data(4.92808726765,self.functionspace)
      arg1=numpy.array([[[-4.4975422776256337, -4.2245290319910929], [4.6849701071676169, 1.1715357578181962]], [[0.28127931117437033, 2.0069322652921651], [-4.8954038151217185, -1.0942584738899996]], [[3.8633556460765366, -3.2468417088360644], [0.48452890830638662, 3.64763007635724]], [[0.88890696007441594, 1.8376448442972739], [0.54785898609189232, -4.6910288890660716]], [[-1.1554492271383356, 1.8133332851516748], [0.18199147200831511, 3.30625827558171]], [[-4.5190000616110453, 2.1734866591626876], [2.6367049714772754, 4.1197246747703282]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-22.164280834089514, -20.818847734377826], [23.087941534458846, 5.7734304517018611]], [[1.3861689920520925, 9.8903373436245428], [-24.124977211312068, -5.3926012526966556]], [[19.038953769837857, -16.000719285393622], [2.3878007438336022, 17.975839336397406]], [[4.3806110720691933, 9.0560741596461245], [2.6998968938277077, -23.117799740390097]], [[-5.6941546246777612, 8.9362646745639509], [0.89686985602526337, 16.293529311460382]], [[-22.27002666614003, 10.711131931429215], [12.993912198489577, 20.302362715963817]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[-2.7841792006105193, -1.0967892614405752, -2.4836571455393019, -0.29009052589122142], [3.6946717002956238, 2.8549374781630954, -3.0318541093319276, -4.3177192552157031], [4.9533899194323858, -3.0377577273341005, 0.64723102123800036, -3.9041175108693515]], [[4.0554679705131349, -2.7166552277097034, -1.0377988878753017, 3.236582266475212], [2.031887647377645, 3.8371260857060463, -4.9325303524745667, -1.2810082787375698], [3.6687470105142204, -4.5967038380177225, -2.5056407079075949, -4.7485387537634995]]], [[[1.2110049194415575, -4.5873432427301672, 0.68466342102152389, 1.649779599627057], [4.7770424921012662, 1.8538425712652096, -1.589234172323498, -3.4626012776216419], [0.53499558348623566, 2.2483402365597973, 4.5154079276505978, 4.6230018242062396]], [[0.068181919061571783, 0.65512958636807817, -1.075309564415269, -1.4813794694495455], [-3.4843011993676907, -2.7100016946902281, -1.133184620975344, 1.0756191949855323], [3.39676353728, -2.7863815428080549, -3.2344221105648998, -1.8034431786418494]]], [[[0.2536122375729768, -4.4839369190214828, -4.9063589040896574, -2.2861715127449611], [4.5216084406762675, 1.1499287748223015, 4.2883163427403517, 1.6036602773481992], [-0.55196652780655953, -4.9815868163294708, -4.4619226792363786, -4.2222687822902474]], [[-4.895005350328681, -3.3827211216052957, -1.7097050694053384, -1.9613555658792379], [-3.7723829582481017, 2.7441471100098278, 4.5534692647816737, 0.35228873098935587], [3.3951177936321368, -0.68858299208684848, -2.265266396420019, 4.080330222676686]]]]),self.functionspace)
      arg1=numpy.array(4.07448890156)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-11.344107252838654, -4.4688556730885916, -10.119633474777311, -1.1819706281911626], [15.05389883775822, 11.63241106942005, -12.353255919618643, -17.592499185423485], [20.182532251820923, -12.37731014564754, 2.6371356127788568, -15.90728346841888]], [[16.523959236483311, -11.068981574665116, -4.2285000506980728, 13.187418523735559], [8.2789036684548112, 15.634327650091379, -20.097540177759967, -5.2194540145212809], [14.948268976967535, -18.729218771756351, -10.209205255663647, -19.347868450831648]]], [[[4.9342261039978395, -18.691079130145219, 2.7896535102555386, 6.7220086686987077], [19.464006616341816, 7.5534609818574863, -6.475316997110216, -14.108330476193002], [2.1798335672976878, 9.1608373407911756, 18.397979487223374, 18.836369624614811]], [[0.27780647250336055, 2.6693182287395985, -4.3813368859501347, -6.0358642072693653], [-14.196746566511974, -11.04187182822111, -4.6171481615812526, 4.3825984722722557], [13.840075333867267, -11.353080671679947, -13.178616992453355, -7.3481092159682966]]], [[[1.0333402472906097, -18.269751211843179, -19.990904901777931, -9.3149804557394358], [18.423243408730436, 4.6853720305966755, 17.472697344869157, 6.5340960019260841], [-2.2489814915798214, -20.297420195286499, -18.180054436162518, -17.203587292840169]], [[-19.944644972985582, -13.782859667049607, -6.9661743302310173, -7.9915214851857046], [-15.370532495811771, 11.180996943979936, 18.553059982942433, 1.4353965245603948], [13.833369769639237, -2.805623759060075, -9.2298027912876606, 16.625260206991459]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_array_rank4(self):
      arg0=Data(-0.531147038743,self.functionspace)
      arg1=numpy.array([[[[4.299449072853216, -1.2528381369825494, 3.1671579410083694, 0.35376130164463859], [4.7387369909938553, -0.32803776894919068, 0.24567091356204607, -2.0650111609176536], [-3.824257684177732, 0.92332975784634286, -0.61219305622467779, -2.4004413201461938]], [[-2.9553059668751813, -3.1055513686667102, 4.2080092190883605, 2.8603210197838145], [4.2891382921204873, 3.3021309886299886, 2.1863953239059066, -1.0436647064940443], [-4.5495542191674501, -2.9887427929768187, -1.3137358463708217, -2.3995480908993025]]], [[[1.6093100138673453, 0.19214809760495744, -0.43605188900193248, 2.9764797760177908], [-3.9679091844530392, -1.2869725074455554, -3.1670121646717342, -2.0159443063822966], [-1.7975550916634129, 0.53714246014062006, 1.5775273614691079, 0.65040866643857065]], [[2.8028262005168596, 3.19819720140055, 1.1153002206473754, 2.4499181606945841], [2.3974150574609308, 4.7775455577742569, 1.0521449622333172, -3.8117511664257622], [-3.1698177215440748, 2.1813894305440931, -1.8396697310738128, 4.8754108544548505]]], [[[3.4334043259042968, -4.8580490158574055, 4.5351824698784942, -1.9421140171690809], [1.1419940282744099, 0.95134118814479063, 4.1266125156628188, 2.8811694102769394], [4.8321753111084984, 1.457467258607962, 0.23779161489639744, -1.2801113807789601]], [[2.7841022972710299, -3.5182959053137211, -2.0047815728226914, 2.3160065499699982], [0.58035049866990462, -0.51017011523509037, -4.9832704931786242, 4.8302818132808554], [-0.71008333178469751, 1.0363242283488638, -0.84322777455673847, 0.91400855742540266]]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-2.2836396432703698, 0.66544126648200919, -1.6822265615965342, -0.18789926779025831], [-2.5169661201461486, 0.1742362895730741, -0.13048737824365672, 1.0968245630917186], [2.0312431443394288, -0.49042386666295695, 0.32516452895248649, 1.2749872988708983]], [[1.569702012883929, 1.6495044131301835, -2.2350716357201157, -1.519251039511232], [-2.2781631026170577, -1.7539170961508139, -1.1612974018131716, 0.55433941829441991], [2.4164822511094468, 1.587461884052763, 0.69778690448979419, 1.2745128628014939]]], [[[-0.85478024828436594, -0.10205889304288682, 0.23160766958146975, -1.5809484189089258], [2.1075432133215819, 0.68357163627277584, 1.6821491329270117, 1.0707628486048524], [0.95476606391360741, -0.2853016270865763, -0.83789898657965844, -0.34546263715133507]], [[-1.488712836514551, -1.6987129728385997, -0.59238840950576144, -1.3012667762145129], [-1.2733799084071638, -2.5375791754694013, -0.55884368101811421, 2.0246003444704885], [1.6836392961517792, -1.1586385363777836, 0.97713512992415141, -2.589560037996959]]], [[[-1.8236425405099141, 2.5803383488388003, -2.4088487390330675, 1.0315481091197471], [-0.60656674637962404, -0.50530205491692071, -2.1918380177322341, -1.5303246003842035], [-2.5665956071801195, -0.77412941847383532, -0.12630231209002937, 0.67992736916137619]], [[-1.4787676907518266, 1.8687324515274075, 1.0648337957303962, -1.2301400207249047], [-0.30825144880127964, 0.27097534596206163, 2.6468493657049326, -2.5655898814161011], [0.37715865893788281, -0.5504405450646529, 0.44787793544127869, -0.48547293866184882]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([3.8605521687579767, 1.08504727051288]),self.functionspace)
      arg1=numpy.array([-1.0860437687883118, -4.0763839713430698])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([-4.1927286269618032, -4.4230693016682521]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-1.1777347133745106, -1.590926929526848], [0.20984742798309952, 3.3616070603285628]]),self.functionspace)
      arg1=numpy.array([-0.10504146117371427, 2.8227897408348355])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[0.12371097516786415, 0.16711328929811087], [0.59235516685127032, 9.4891099226134177]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_array_rank2(self):
      arg0=Data(numpy.array([-3.2228341799211799, -1.9262362541277755]),self.functionspace)
      arg1=numpy.array([[2.4356896484051269, 1.4111946302508347], [-4.0440652957948773, 0.84205037565780128]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-7.8498238505602442, -4.5480462888936213], [7.7898251868200594, -1.6219879613939694]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_array_rank1(self):
      arg0=Data(numpy.array([[[3.5493484861541766, 4.4683626537737595, -3.8602237438199225, -0.4683230125428679, 0.37191688787990085], [4.5607669113329372, -3.6529226012078899, -4.1074457212439857, 0.17042992813933999, 2.3343276348738851], [3.0842110081842602, 4.3670625716173532, -3.0897026865801482, 1.1615455157411958, 0.025372520122255438], [-1.2275262495181272, 1.5381341141211999, -4.5329708452881299, -2.5686274158067626, -4.4125290303984794]], [[-4.4386529227101024, 4.3814642072700831, -1.8959073268544193, 0.54169036642184398, -1.0940373223943265], [-2.7748830011757839, 4.4850791790379478, 0.27726341804008037, 1.7932509273101696, 0.71236669729825675], [-4.6002948131530914, -3.4993988212293892, -3.3105342130694337, -4.1245727015005631, 2.7417397398707699], [-1.5702774806879294, -1.4849135068706634, 3.7013066608638319, 1.8453205870170617, 4.0971420910210234]]]),self.functionspace)
      arg1=numpy.array([4.871150507877811, 3.015846468250917])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[17.289410680965258, 21.766067010312291, -18.803730850230401, -2.2812718803990575, 1.811663137284514], [22.216182056451753, -17.793935784112147, -20.007986311118181, 0.83018983101352484, 11.370861244169134], [15.023656018919095, 21.272619063668049, -15.050406810926326, 5.6580630289259197, 0.12359336427966455], [-5.9794651137735695, 7.4924827711856699, -22.080783235220583, -12.51217074105598, -21.494093027451139]], [[-13.386295740746872, 13.213823355263283, -5.7177654158249371, 1.6336549784588632, -3.2994485948776195], [-8.3686210989054928, 13.526310201927316, 0.8361839000713539, 5.4081694758160568, 2.1483885881465175], [-13.873782865160763, -10.553649576006075, -9.9840629145092805, -12.439078014864617, 8.2686661113524487], [-4.735715794306639, -4.4782711553539736, 11.162572621079782, 5.5652035751461142, 12.356351505127931]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_array_rank3(self):
      arg0=Data(numpy.array([-1.6171952064263295, -3.2502925981603168]),self.functionspace)
      arg1=numpy.array([[[4.0452943626662723, -2.5506869229165865, 3.882539825437183, -3.1177472463708722, -1.8646198330574384], [-1.7337869262060113, -4.0188539258063098, -2.0168009437665124, -1.7686769853626205, 3.9711088141817577], [-4.6172452811908329, -2.8378354777146884, 1.1119376319524852, 2.5513524255521913, 4.8769943852300504], [1.5640495996741706, 4.5507443946086887, -3.6002196435710374, -4.8948346035579444, 4.1556442354830505]], [[3.5281373897893644, 4.3581112890020517, -0.61358908267345313, -0.85925232117192962, -3.96285089217145], [-3.7188051822713106, 2.8919537326135316, 4.5279744415234209, -1.841665444156396, -1.033021068935577], [3.4473230964131503, -0.32642469595882595, 4.2417685972153851, 0.16304560064845042, 1.575302717001331], [2.9509610226582765, 1.7399888398190395, 2.453706963619533, -2.665745961839705, 1.2821458801946211]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-6.542030651887349, 4.1249586648350283, -6.2788247944563302, 5.0420059016798628, 3.0154542558279522], [2.8038719060250017, 6.4992713041416001, 3.2615608185753011, 2.8602959424450014, -6.4220581384920843], [7.4669869356364051, 4.5893339311867667, -1.7982202082386034, -4.1260349125071922, -7.8870519415621612], [-2.5293735152060881, -7.3594420206326605, 5.8222579496649907, 7.9159030571236304, -6.7204879372363981]], [[-11.467478843325031, -14.165136864602285, 1.9943440537255035, 2.7928214594571941, 12.880424922437872], [12.087204957936668, -9.399695811335862, -14.717241811942669, 5.9859515614291663, 3.3576207341049642], [-11.204808743738766, 1.0609757731317038, -13.786989074638036, -0.52994590895026139, -5.1201947609312626], [-9.5914867694057957, -5.655472846945381, -7.975265581906994, 8.6644543683433479, -4.1673492641583207]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_array_rank1(self):
      arg0=Data(numpy.array([[[[2.1175715643349227, -0.17041531942997867], [3.4291521785177999, -1.0802842782926891]], [[-1.7080345661595508, -2.4708145231621814], [2.2029619503397058, 2.6806558646889345]], [[2.1613204406073896, -1.1624424247664344], [-1.7108986630551204, -3.9559465853105413]], [[0.19440836148145024, -3.7024096858150211], [2.1582702088465684, 3.0234353074433393]], [[-1.6812930504956922, -3.6645050724458281], [2.1897720836244572, 4.6737238132331509]], [[-4.9029237845526152, 2.2111836971109096], [-3.3723079281581683, 0.95653247464262137]]], [[[2.2943810494949215, -2.7698419072372538], [-4.8194092300582394, -1.3443615328290983]], [[0.84541957618715458, -0.084902658974578493], [1.9715308395007387, -1.1656494717123822]], [[4.3446196757018019, -2.0304110619811255], [-0.41514273163480908, 2.1330240956818605]], [[-3.6009665713844976, -2.9230796632191112], [-2.5705711562299505, -4.6258694114639631]], [[-3.4314657443543406, -4.9091692548355645], [0.53649836225264913, -2.824175522229968]], [[1.6793243261428339, 3.3450833903151693], [-2.1719767036454041, -1.4049687157876569]]]]),self.functionspace)
      arg1=numpy.array([-3.8836938799710174, 4.4360747959489739])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-8.2239997248081931, 0.66184093312351422], [-13.317777329198861, 4.1954934402342241]], [[6.6334833913728, 9.5958872421484713], [-8.555629844343331, -10.410846776000831]], [[-8.393906967843181, 4.5145705308840709], [6.6446066669577668, 15.363685542862793]], [[-0.75502256370070164, 14.379025837945214], [-8.3820608014211881, -11.742097200005988]], [[6.5296275306479226, 14.231815922980612], [-8.5044044397036878, -18.151312570128393]], [[19.041455096031331, -8.5875605919613278], [13.09701166196562, -3.714879317763081]]], [[[10.178045945967376, -12.287225873458418], [-21.379259816825204, -5.9636883124264921]], [[3.7503444739256997, -0.37663454558617859], [8.7458582665453495, -5.1709082423745354]], [[19.273057841364768, -9.0070553374704616], [-1.8416042085265854, 9.4622544300061531]], [[-15.974157048373561, -12.967000020557315], [-11.403245917345096, -20.520702705546601]], [[-15.222238701692575, -21.777442000423655], [2.3799468628568792, -12.528253853500392]], [[7.44960831742622, 14.839040118124666], [-9.6350511124297107, -6.232546309202422]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_array_rank4(self):
      arg0=Data(numpy.array([3.9920134633483109, -4.3788986837923849]),self.functionspace)
      arg1=numpy.array([[[[-3.0155967496700886, 0.2504996069475327], [-0.98043558288568633, 2.8447837595913636]], [[4.0377241905998726, -0.89286434822464145], [0.31791229245174879, 0.53823580667096937]], [[4.5684534212718191, -4.7480184166365271], [0.62829936147322574, 2.5100794154369304]], [[-4.5679061168640516, -1.2856881721207145], [4.8434204241380172, 2.3871835871626885]], [[-1.4395809274089277, 3.8254771458511314], [3.8282779712388564, -1.0767207320935857]], [[-3.9585215046827518, 3.6194050216560267], [1.0411431904958217, -2.3700789560781486]]], [[[1.9327914370404686, -2.6658701986454392], [-0.39885102678497208, 3.8011331058384741]], [[1.1544761598562614, 2.3279142428814312], [-0.55697230515887597, -1.217242525742618]], [[-1.5465758455935044, 0.9027449037836206], [-2.4144394930841351, -1.0586502197019287]], [[-1.2847245428564569, 4.1404771963457243], [-0.19078920067017968, 4.1502666374021207]], [[4.9847190722362278, -3.7300108061254011], [1.3893033025648922, 4.7801168145424597]], [[4.4183772490300015, 4.5548039118103141], [1.6071499089026142, 1.0313862184263201]]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-12.0383028247124, 0.9999978034980106], [-3.9139120468254087, 11.356415068603349]], [[16.118649330161851, -3.5643264990564831], [1.2691101516313068, 2.1486445866866482]], [[18.237327564396754, -18.954153443438745], [2.5081795100142643, 10.020270820497684]], [[-18.235142717832396, -5.1324844927735729], [19.334999541815151, 9.5296690194375682]], [[-5.7468264437958867, 15.271356269968987], [15.282537202625273, -4.298283658783844]], [[-15.802471141647359, 14.448713575761344], [4.1562576337327357, -9.4613871018624796]]], [[[-8.4634978797017002, 11.673575504009857], [1.7465282362179555, -16.644776754075753]], [[-5.0553341368642704, -10.193700614135045], [2.4389252939690125, 5.3301816938304682]], [[6.7722989346544908, -3.9530284709783792], [10.572585918362472, 4.6357220536492951]], [[5.6256786097499125, -18.130730145350675], [0.83544657969645097, -18.173597115907594]], [[-21.827579784490016, 16.33333940947389], [-6.0836184029898197, -20.931647227573823]], [[-19.34762632027569, -19.945024854358589], [-7.0375466207507085, -4.516335754348618]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[-4.3865767713141679, -0.14079859794502791, 0.82840688837124077, -0.016416940280961079, 4.6940682049183096], [-0.41659296425237358, 0.89838767687735999, -4.5515030747996699, 0.8752225408162726, -0.83318979357682288], [-1.5672667117901327, -0.2403145578274648, 2.1031077028839071, -0.26070760218902933, 1.6808040128492561], [0.72213403374675433, -3.1016247350285409, 2.6592598306100177, -1.1902806161334722, 2.7780218238390226]]),self.functionspace)
      arg1=numpy.array([[-4.9974001410095878, -1.4686376135233625, -2.3484695710980361, 3.6364883951278362, -1.6853656217344981], [2.4125899441850915, 4.5071581933849227, -4.1177697269776106, 4.9066582899872841, 1.3419712585354979], [0.78864263620958397, -4.2778265764329237, 2.3311773633392772, -4.490848760852427, -1.5035926503826866], [-4.7008725365901167, -3.3037722766525803, 1.4987184914037286, -1.8156415126448309, 1.9536161273964581]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[21.921479375514807, 0.20678211687342121, -1.9454883698278664, -0.059700012815221679, -7.9112211786462865], [-1.0050679963735358, 4.0491753786738398, 18.742041573655591, 4.2944179354798981, -1.1181167558852207], [-1.2360133512296965, 1.0280240021780556, 4.9027170696274309, 1.1707984122354098, -2.5272445604538682], [-3.3946600469771582, 10.247061812167198, 3.9854818815823805, 2.161122898348399, 5.4271882373112366]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_array_rank2(self):
      arg0=Data(numpy.array([[[-0.57897981021654488, 2.2395658305858737], [3.6492568453802523, 4.6518756555430603], [2.6934011513877216, -3.3912021731830433], [-4.0418025098757084, 3.9764172336371004], [-2.0858016200074458, -1.3454421891750004]], [[0.19060083940672001, -3.556613439790608], [-0.40213007938848744, 0.13987198060835926], [-0.068141151032451219, 1.3346307269106994], [-0.6690506945328174, 1.9937124348399653], [3.5690783175422052, -3.5664191260006781]], [[-4.8219118532650072, 2.3942495046292276], [1.321341738550224, -3.3769527372263983], [-4.5481034365060289, -2.7811286886794049], [1.6779143515398944, -2.3791859048604636], [-1.077854252270952, 0.82005756316571787]], [[2.9493239330404695, -2.3436216181660372], [-2.6695533857619616, -2.7140614077094796], [2.8866347417885274, -4.0253927999613524], [-4.306530289330821, 0.66891561773967911], [-3.4755360509058661, 4.6960138160499163]]]),self.functionspace)
      arg1=numpy.array([[1.2942108963820704, 2.7707096026178739, -2.2454385495032736, -2.7591839398412521, 4.9676119905334772], [-2.9192617838413981, 4.810271709429891, -2.4293705300612132, 0.41448928739606927, 1.5611342548679241], [-0.29258276553396456, 3.0289565280082034, -2.0098957450798141, -2.8857483716366117, -1.7101817050652501], [-1.4315678891615766, 3.6848062471153771, -4.7513846870900043, -3.8957430720293686, 1.4547840906642389]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-0.74932197916747556, 2.8984705011091996], [10.111030983914075, 12.888996548997474], [-6.0478667746024923, 7.6147360888244817], [11.152076573259118, -10.971666569159467], [-10.361453137423139, -6.6836347515153429]], [[-0.55641374644812935, 10.382685694677422], [-1.9343549443932373, 0.67282223126231688], [0.16554010420268719, -3.242312556471028], [-0.27731434560875273, 0.82637244638949936], [5.5718104198215146, -5.567659064815782]], [[1.4108083051892801, -0.7005161414427441], [4.00228668471141, -10.22864303819707], [9.1412137452163478, 5.5897787178961389], [-4.8420386077019515, 6.8657318507718612], [1.8433266229605669, -1.4024474416264014]], [[-4.2221574372764641, 3.3550534529113922], [-9.8367869928636829, -10.000790430182645], [-13.715512109356018, 19.126189709258725], [16.777135539145178, -2.6059233835816005], [-5.05615455338787, 6.83168618912888]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_array_rank3(self):
      arg0=Data(numpy.array([[1.986679394094236, -2.831327573445972, 4.0617932378224761, -1.9636795758609282, -1.2776008041062239], [3.9550604146874928, -3.1531771851380719, 2.6119615547347621, 0.24969593264393986, 3.7260389685530289], [-2.4142355003614959, -4.8031211953100446, 0.8897164409476197, 2.211972567885053, 3.9490499605648992], [-3.9179784334882148, -2.3948253154079637, -0.83009937111062282, 4.7923632380752874, -1.4602596044965388]]),self.functionspace)
      arg1=numpy.array([[[2.4287728987255166, -0.11366897985379154], [0.23598020590307733, -4.5467293822431021], [-3.6204255419770117, -4.6581152237861705], [4.2172386352469964, -3.3570384510732443], [-2.2056120704139968, 0.75773611858043033]], [[4.1207413209207466, -3.2711591837983764], [0.076846479257294042, -1.0463421045585299], [0.35205523367916047, -1.4779274060600422], [1.1025356527247236, 4.460313450162694], [-3.7464292043509904, -2.361411858642104]], [[1.4281920088349871, -3.8854795228686712], [0.041761758891489364, -4.1388492032765605], [3.5765607325182263, -1.2681598092274129], [3.1621147763784023, 1.1279664870995223], [-0.95201635546222718, -1.7887872968946583]], [[0.57131341477518394, -0.89395390251154705], [3.6794301788034982, 4.4948735178843844], [0.62309767565599383, 1.4042288778411036], [1.3578147833756411, 1.812599363077795], [-3.4117081510570779, -1.6466324329506579]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[4.8251930708325101, -0.2258238200232405], [-0.66813726376084082, 12.873280268941866], [-14.705419984441999, -18.920300916972597], [-8.2813053745661414, 6.592147841752336], [2.8178917547073157, -0.96808427439868683]], [[16.297780877540696, -12.937632197982408], [-0.24231056515228566, 3.2993020519433114], [0.91955473551312994, -3.8602895653177018], [0.27529866808029485, 1.1137221268226831], [-13.959341208336909, -8.7987126061037166]], [[-3.4479918490620252, 9.3804626000371929], [-0.20058678928514029, 19.879394332449738], [3.1821248857691278, -1.1283026320186262], [6.994511141853005, 2.495030926957813], [-3.7595601509952474, -7.0640104042608431]], [[-2.238393637851678, 3.5024921105728675], [-8.8115925384746685, -10.764436890186374], [-0.51723298870253132, -1.1656495083912757], [6.5071416519645826, 8.6866345529727056], [4.9819795953202259, 2.4045108252917009]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[-1.5577254326373491, -2.465302890102091, -4.2634259281488793, 1.2306365372658989, -1.4350802064101345], [0.75542093853135128, 4.637518587487051, -2.0432099030814066, -2.4884502426896771, -1.4799395835450992], [1.3102650128670676, -0.29586518938144302, 0.17480464365435733, 2.894892904085542, 0.32079759161661059], [3.9858855502577271, 1.7623279790686022, -4.8648880266035697, -2.6313579424881195, 1.724309247272088]], [[-2.2696286223700746, -3.7498942314443493, -0.50925692411712653, 4.8766145142169837, 1.3278654286873532], [-1.1077726040780735, 2.7720241127001453, -2.2317662200152522, 0.44627540269446975, -2.6848013688183658], [-4.1232229050912572, -0.77631967604558305, 3.9007817967847096, 3.2022429311857703, 3.1679814833805526], [0.60189265436940342, -2.4461796971804874, -2.7757643599323156, 3.4052193432628215, -4.7588434526513277]], [[4.6265934759091305, 0.64860428318603169, -0.74598978516979386, 4.8796519960486471, 4.9540230443062718], [2.6314002495413868, 4.3536333012012456, 4.8254426586211157, -2.8856602843286296, -3.5560489566167375], [2.2936613100686341, -0.26730671535576889, 0.22274903076827179, -3.2172035412363185, 0.18942933962142749], [-4.1457980871237776, 0.35826745399071491, -2.489222723077813, 3.1951986580775138, 0.68580802142182762]], [[-3.1554027395015849, -1.1526839507378162, -2.2587689968406597, 3.0598479618047367, 2.1337746135306279], [4.1579519693012585, -0.69762806380118469, -2.1061566537610164, 2.5788004402308795, 0.70693053588313592], [2.6994957759437881, -0.52353390745917139, 1.1286192515061613, -2.4715693541830541, -0.8944938900038748], [-4.6364187671526285, -1.4950629389424486, 0.35644902773715703, 2.6489697037432034, 1.4980382966926626]], [[2.3433440213229453, -1.4437912535326092, -1.5132170767030138, 1.5830919331739777, 4.5898817106823575], [0.099956641646902433, -4.9318341306857914, -3.3483970128868803, 0.28660513544803123, 0.0078777901780613391], [3.2187888031980432, -0.11877836208351766, -4.3595915681655582, -1.8303914378809329, 4.9627614639865953], [3.9060367493486297, -0.50961406132164466, 0.69281975593069678, 0.89947998493459025, -2.0003230079177392]]], [[[-1.2092607189640669, 1.9454597722833791, 1.1686546913299543, -3.9049320144095589, 1.6702311025450012], [2.712221092322733, 1.4686137195590385, -2.3813230773267211, 1.6633278785199312, -0.92443594787251548], [2.6029141783254985, 2.5273937251043925, 2.6930220016008608, 0.98217793213398163, 3.4621943687767445], [-1.2860099055097574, -2.3471022447572287, 4.1229216442065812, -0.22461299307779115, -3.938553294115966]], [[-0.90965852686695037, -2.8969351963710488, -2.8737181476475504, 3.1714536792758743, 0.86465882459387], [-4.8063149351642025, -2.022594797356426, 1.7909424413945647, 2.3735471570594537, -4.587279926505933], [-4.2317170484586777, -4.8313628529480193, 3.4574420315053693, 4.4512389168706186, 3.349767517775323], [2.3716385424989976, 4.96168917831152, 2.3246085973675514, 0.89075821050426107, -0.7387431756073628]], [[-4.7417763730520699, -1.0949102438230049, -3.1888247174149309, -0.71839356175643854, -4.2952981744825074], [-4.90897743107978, -3.8995032833658891, -3.311588996455499, 2.0289726429756332, 1.2397366645323551], [1.4136869236532421, -1.7480288976022762, -4.829164407044563, -0.28745706230423451, -1.6250000642890052], [-3.9186117933228792, 3.3669656655045284, -0.96594002835794868, 4.3858576751181104, 4.5325399267231372]], [[-4.5793975361840191, -4.810571558850131, -1.8379050381590991, -3.2158772149774419, -4.6994422298962419], [-4.718923266602971, 3.2842318475494015, -1.1791713864653608, -0.61510191486466415, 2.2735843628533647], [3.1991447655305034, -1.9299497874501115, 4.1870525786056501, 1.3197108937662296, -3.6198398871064974], [-0.45871499799402837, -0.18445181653551135, -2.6650972202392542, 4.3797757571506164, -1.1660772349492285]], [[-2.1216839841051116, -4.6036013107002747, -0.35584466995724906, -4.2116955574210895, 2.2054779497216979], [4.157562070022653, 4.0413952800464852, 3.1534833395545121, 4.0624028990744634, -2.466259159921278], [2.6019284126140469, -2.6857271093090884, 2.2923091329180778, 4.6835790337166099, 3.924432367578925], [-3.2896433055686636, -3.6482120151988773, -4.2609669548119156, -3.6633193841394607, 0.10036785896972944]]], [[[-4.0685686846598434, -4.1454384372104265, -3.2954365669154164, 4.840096660398018, 3.5610762139268068], [0.5461358532661702, -4.4551357400460114, -0.8185754605043094, 3.4817969043564183, 2.1685820409144307], [0.097325263502574089, 4.4706006344077025, -1.5538010374204769, -0.10233388588006687, -0.7800261250089191], [-1.0896439172631767, 3.0928797176947889, 3.6568356525849115, 0.94984135437115746, -3.9964608293373969]], [[-0.73988456928014923, 4.9777531508443289, 4.9530156396597995, 2.0580535583906236, -0.10198281389962371], [2.8698951608979604, -1.560315749319571, 0.99445832060927497, 3.0369097941693433, 4.9470553465291154], [3.6656560229397961, -4.2075397383731055, 1.0980865180221198, -0.013031832039906099, 1.8101454785454161], [-1.7836842466668723, -2.5777526615928039, -2.5898869382616097, -0.45401068851988668, -3.6632574702103993]], [[-3.428073391928713, -2.9166959570788729, 0.82057298846918769, -1.347973755511588, 4.702748003321668], [3.5286124605258244, -3.6537014799965259, -1.5878931601278978, 4.9327759929944452, -1.7639120019936607], [0.39991146924880283, 4.7630196257880613, -2.0496238323713287, 2.6297293610702255, 0.38548793887289534], [-0.97888520861848516, -3.2036189598426867, 1.4155744822214436, -3.7490172709766534, 2.046236274149634]], [[2.1987654999499817, -1.7378773622670907, 3.8372048846250255, -0.27227687444974258, 0.42882547491792433], [-3.9401763884084726, -1.3542629573993681, -3.0984694910370933, -2.3607060026914795, 2.9160300690390546], [-2.98075463568623, -0.81841994287399888, -3.0441139446250287, -2.4804778630505164, 0.2091004406753445], [-2.9642970131604054, 0.17264085499881698, 2.8047634270417365, -3.4493675217314843, 0.50794124679264119]], [[-1.9782470350307713, 2.0252303360654782, 3.4145422750478147, 4.2940025882353332, 1.4488286113869711], [3.4309400764365972, 4.5721534909255688, 0.10918874338938434, -1.0533366670331299, -4.1300783259336349], [1.7049609574269553, 2.8309219925779292, -0.39327320877288408, -4.8006338392911445, -2.8291635155197925], [-2.0210466519190828, -0.17919099102094194, -1.9092499608640114, -1.4550109843619694, -1.100563570255698]]], [[[-1.6113118850942056, -4.5084409647157013, 1.3814786915071728, 4.5974038333351235, -0.17188200000509557], [-0.33986718806154759, 0.43548592119845964, -2.4818480287298952, -3.254917260559651, 2.2907958202869825], [-2.2986563059511731, 1.8362373621338479, -1.1563271971914322, -0.91290901021881687, 4.7125706537871555], [-3.8664656080748792, -3.6193622257920497, -3.4543475605174248, -2.2436027685479454, -4.8501576379424796]], [[-2.859391383632941, -0.08233023119420313, 1.7476145699339121, 2.5460486137423803, -0.79301305916526932], [4.1265391626046934, -0.82102647984676835, -2.1354424822880591, -2.6666440262870306, -3.5613950698293708], [-2.0600067905638197, 3.0864560748331336, 0.32291947435757251, 3.871687724911979, -2.1314023126066473], [-2.9050450378122781, -2.0029929575024576, 3.0629854336307591, -3.219579819274502, 4.4582058553471207]], [[4.2070270857672867, 3.638465972765383, 0.062010236366684346, -2.9862125499718539, 3.4288363242292146], [-3.7018893397074559, -0.030262289072522819, 2.6089285533271962, 3.5834627823636431, -0.75706746520144197], [2.4358856235605479, 0.092917024709433527, 2.1826249924383401, -4.1405273078543079, 3.4173817318540554], [-4.0304445548226404, 2.4697225595253105, 1.7892966864572024, -3.7369550281066988, 4.7881534801527685]], [[-0.093050557538621348, -1.0224899520617461, 1.3865859149249378, -1.4229696510536796, 4.585386193959387], [0.46160466250144516, -4.0774878624999182, -0.74701947670614555, -3.0627985897001517, -1.4744222668440798], [-2.9529401879809889, -1.6759362283542933, 1.6726747185808852, -3.5696888505912074, 1.2844973039650478], [0.54341069477045423, 3.3345868461195476, 0.42472343749286345, 2.0957093893314118, -3.68307980600179]], [[3.4132241525677181, -1.8891347334658368, 0.85196726162879344, 1.3414968257966011, -4.476615951092052], [3.1392249347615788, -4.8518180249112426, -4.2764677529754191, -0.11378101654045825, -4.0676200376434055], [-0.85932719339513675, 4.9231630796082655, 1.9305135792252672, -2.0281524382047, 1.9540745560044561], [-1.2727404601792971, -1.2501481999696673, -0.3626154214806343, -4.4523121672931589, -2.1465289248043327]]]]),self.functionspace)
      arg1=numpy.array([[1.512455877910412, -2.1544357002082459, -2.6080778968201477, -3.6518786502945599, -4.3033935822195595], [-2.0299054211045373, -4.8557003260226459, -3.3692736229525888, 3.2127728837521747, -4.9935380411429051], [-3.0696898850992227, 1.4217713195789825, 0.90563822527669036, -1.7993819017742929, -2.7773644362639285], [0.57584862399220604, -2.8337213070072123, 3.3108212160053316, 2.1310195518768396, 1.8396900556958062]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-2.3559909867628983, -3.7286618469644339, -6.4482436050644267, 1.8612834643591247, -2.1704954934578953], [1.1425408387783422, 7.0140422465635819, -3.0902648277202367, -3.7636711964435934, -2.2383433220850728], [1.9817180203311582, -0.44748304474904071, 0.26438431078106772, 4.3783977887053203, 0.48519220306004662], [6.0284760291654766, 2.6654433107482851, -7.3579284912125535, -3.979812787002404, 2.6079416563719477]], [[4.8897689302485485, 8.0789060042286689, 1.0971612978961791, -10.506352405582762, -2.8608006846363607], [2.3866248459384565, -5.9721477102392795, 4.8081968189196704, -0.96147165968977688, 5.7842319169502527], [8.8832186266449611, 1.6725308248467043, -8.4039835617154459, -6.8990264916861213, -6.8252124053937386], [-1.2967390222665454, 5.2701368687302388, 5.9802058324038718, -7.3363261201650998, 10.252622226094291]], [[-12.066516182090902, -1.691610494760365, 1.9455994699549497, -12.726512515068791, -12.920478002192846], [-6.862896828515912, -11.3546147837231, -12.585130340322781, 7.5260268052892414, 9.2744526837624619], [-5.9820473655815478, 0.69715673599097561, -0.58094682368484063, 8.3907174454699494, -0.49404647367588211], [10.812564355706773, -0.93438942790321278, 6.4920867643217033, -8.333326996081361, -1.788640742132227]], [[11.523147897466805, 4.2094619102366169, 8.248750275509666, -11.174193444862043, -7.7922859556930257], [-15.184336025641487, 2.5476530320418775, 7.6914285180456874, -9.4174662710493617, -2.5816245312329165], [-9.8582309907294672, 1.9118822993554359, -4.1215805488867767, 9.0258713572634086, 3.266583139724081], [16.931638709589709, 5.459788427570567, -1.3017085943115771, -9.6737159063769091, -5.4706540730155622]], [[-10.084331622293737, 6.2132020145169635, 6.5119686563887926, -6.8126676652844509, -19.752067496897389], [-0.43015277016350029, 21.223623346564615, 14.409470215980544, -1.2333747005182252, -0.033901231694341449], [-13.851715078202636, 0.51115004109676099, 18.761038375542167, 7.876894766726438, -21.356715834206462], [-16.809213479060642, 2.1930698809004108, -2.9814760913070821, -3.8708163945024618, 8.6081771946393246]]], [[[2.4546848889539299, -3.9490993382988298, -2.372258493329924, 7.9266426650946249, -3.3904111695535062], [-5.5055522985399854, -2.9811469508413908, 4.8338606240668502, -3.3763982776819179, 1.8765175420503306], [-5.2836696012527922, -5.1303702238549969, -5.4665799602033793, -1.9937283089280136, -7.0279271180975149], [2.6104784787883903, 4.7643955705193273, -8.3691409963641714, 0.45594313229912414, 7.9948906830351323]], [[4.4170292054771307, 14.066649177485379, 13.953914146429405, -15.399628664425583, -4.1985241364788122], [23.338024997644329, 9.821114216935305, -8.6962797965673815, -11.525233704363714, 22.274456634691997], [20.547949851836389, 23.459650180193396, -16.788302399585021, -21.613882259853352, -16.265467228161704], [-11.515966044020255, -24.092475760750283, -11.287602724112665, -4.3252549331528893, 3.5871154786436761]], [[15.976342059664134, 3.6890522040134384, 10.744023008605369, 2.4204644785249299, 14.472034842000319], [16.539688174206663, 13.138493555261705, 11.157649455817547, -6.8361640076702015, -4.1770120432160862], [-4.7630980629778588, 5.8895876568502414, 16.270776257556726, 0.96852149775309626, 5.4750698539052065], [13.202875353833718, -11.344228606171416, 3.2545162589005123, -14.777154578799614, -15.271367220087726]], [[-14.712564228173536, -15.45527385962313, -5.9047714695090594, -10.331883113755989, -15.098240564970499], [-15.160828711449259, 10.551491023762024, -3.7884098557323673, -1.9761827528212317, 7.3045101898982558], [10.27812555389411, -6.2004903441229917, 13.452048987388853, 4.2399313738844899, -11.629723432820288], [-1.4737471069256476, -0.59260179452412187, -8.5623520817479744, 14.07122478948865, -3.7463413208055951]], [[10.594709685912514, 22.988258271237161, 1.7769238961694651, 21.031261983694783, -11.013138040837157], [-20.760944355070961, -20.180861070207509, -15.747039018175824, -20.285763414977556, 12.315358934384045], [-12.992828508718816, 13.411280488463703, -11.446732857285729, -23.387630073563219, -19.596802317397877], [16.426958988148215, 18.217485480050211, 21.277300580906143, 18.292924701556597, -0.50119072187341007]]], [[[12.48924413813177, 12.725210440006375, 10.115968296446361, -14.857595761326323, -10.931399633958554], [-1.676467704661196, 13.675885117963281, 2.512772811300517, -10.688036739272682, -6.6568743560028567], [-0.29875837693846824, -13.723357547759493, 4.7696873280263166, 0.31413329438893944, 2.3944383060530208], [3.3448689111826675, -9.4941815852362321, -11.225351414210119, -2.9157183979620882, 12.267895384012258]], [[-1.0519466604015648, 7.0772266658143792, 7.0420555818944512, 2.9260815234772575, -0.14499623989244581], [4.0803346299632297, -2.2184121818699554, 1.4138923187589478, 4.3177912454984835, 7.0335814080649612], [5.2117246008577585, -5.9821593260077366, 1.5612279177401993, -0.018528285035908958, 2.5736129256614451], [-2.5359911050958024, -3.664974803221035, -3.6822269697725796, -0.6454993757198817, -5.2083144073786043]], [[-3.1045943027845637, -2.6414713502406082, 0.74314226498722524, -1.2207765596610698, 4.2589883556517343], [3.1956463264398232, -3.3089317240348706, -1.4380567434672249, 4.4673104959829537, -1.5974661350297927], [0.3621751132782795, 4.3135726408567452, -1.8562176900335787, 2.3815834315176438, 0.34911261282641826], [-0.88651586308284769, -2.9013197892546874, 1.281998361825998, -3.3952533478189575, 1.8531497878176617]], [[-3.9564188468557022, 3.1271050731666494, -6.9045970227941842, 0.48993008015653816, -0.77162079858707899], [7.0898820831006022, 2.4368362557877532, 5.5753299253719506, 4.2478116566529831, -5.2470517312585168], [5.3635159450836278, 1.4726500332586243, 5.4775235388970289, 4.4633269745248727, -0.37625154860424409], [5.3339023969644268, -0.31064682999171123, -5.0468405493773432, 6.2067294911716777, -0.91398028664334818]], [[5.494312961239026, -5.6248027106311032, -9.4834282808375256, -11.926010077790076, -4.0239250595078255], [-9.5289709512476488, -12.698536502836644, -0.30325693273002419, 2.9254997984305939, 11.470732661432539], [-4.7352979283761236, -7.8625020640233574, 1.0922630237812074, 13.333109696772388, 7.8576181323801029], [5.6131830950703439, 0.4976786857604531, 5.3026829412420025, 4.041095762340305, 3.0566661198758331]]], [[[-0.92787173185378591, -2.5961795258816305, 0.79552260357895876, 2.6474086713625242, -0.09897801319196263], [-0.19571205258534249, 0.25077396849011124, -1.4291687723018791, -1.8743396257017559, 1.3191516209593557], [-1.3236780708129905, 1.0573947583078545, -0.66586942538745053, -0.52569739736459242, 2.7137273264493844], [-2.2264989001231075, -2.0842047574517202, -1.9891812895147927, -1.2919755670534383, -2.7929566019544652]], [[8.1027182888734988, 0.23330093034584326, -4.9522526432579728, -7.2147922054379592, 2.2471780024915948], [-11.693461949272619, 2.3265602295589152, 6.0512488619480447, 7.5565259954930593, 10.092001092045926], [5.8374851350002395, -8.7461563423964979, -0.91506379493462231, -10.971284000161353, 6.0398001470379032], [8.2320880214642251, 5.6759238214601062, -8.6796470863322082, 9.1233919334885858, -12.63331292332145]], [[13.928714531867415, 12.046310336345107, 0.20530480617232391, -9.8868158659481953, 11.35226404846782], [-12.256293765207413, -0.10019302870619486, 8.6376960053977783, 11.864204606615045, -2.506515025736312], [8.0647818022466389, 0.30763165673608417, 7.2262811315483324, -13.708545656293483, 11.31433994101145], [-13.344081342039962, 8.1768098477233888, 5.9240414312505454, -12.372389990313458, 15.85272012757955]], [[-0.19829255742784294, -2.1789460794411935, 2.9548416950620786, -3.0323761481227551, 9.7715476322335793], [0.98368856102808944, -8.6892063575278282, -1.5919131104936015, -6.5268836781118331, -3.1420226783673049], [-6.2927732761103572, -3.571452870321727, 3.5645025292259569, -7.6070767347266255, 2.7372888690826049], [1.1580188152548154, 7.106069766512082, 0.90509394943763277, 4.4659976837171103, -7.8487150777125718]], [[6.2792745313395759, -3.4754223830266473, 1.5673556989968784, 2.4679383701654962, -8.2355858483932725], [5.7752008950731923, -8.9258413724748813, -7.8673751986526685, -0.20932180465644107, -7.4831601336015741], [-1.58089569227802, 9.0570941601240662, 3.5515466340864421, -3.7311718720003899, 3.5948915287695957], [-2.3414479680735569, -2.2998852116302091, -0.66709998493986633, -8.1908744190226681, -3.9489479172259419]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_array_rank4(self):
      arg0=Data(numpy.array([[-2.6250055157910679, 3.6236030886516559, -2.7721158828031389, -2.8415862728640562, 1.5916292997256711], [-3.9636625823129479, -3.7365774932527298, 3.4773970264391565, -2.8556183987241237, -2.7797417951767911], [0.26564273121106563, -4.4724915053650784, 3.7536627100351581, -1.7044777444784587, -1.05845581801118], [4.3507167486038849, 4.7060511015895727, -1.4052979099922522, 4.3570466022927956, -4.6525581776928524]]),self.functionspace)
      arg1=numpy.array([[[[4.5911841733606291, 3.0154321307547178, 3.785530246227701, 3.9340993003512814, 4.2826799600006016], [3.7425765960293109, -0.4840246319066015, -3.4013206679230268, -2.8867226664804035, 2.4182450078914544], [-0.94305999454198819, -3.2272073174830096, 1.7294187284073814, 3.1995191192254229, 3.1220319996485006], [-3.5897147382856733, 3.8402702196117282, 0.066496213799018378, 4.6433945809013917, -1.8023956205779812]], [[-4.231559510107072, 3.0491789458039094, -1.3811237947453412, 2.2419432623215947, -0.63889325539058461], [-0.089565974501882195, -3.5867006576584402, -4.6085195359851063, -3.1041697567580515, 0.94056639297600686], [1.8546741715633939, 1.2982350010745245, 2.1946552648959381, -1.8660520483819076, 1.3640072692453966], [3.835268999166896, -2.8268284936312051, 3.4869080313940763, 0.33917656336017821, 1.2599912725572207]], [[-1.6182743823608514, 2.1172774671348424, -2.7781503924189019, -3.40918174705237, 1.2626663158477101], [-0.96470859066390702, 3.7715823742855079, 0.8531407996609941, -0.62782373227917709, -1.9650339894700561], [1.8663443573577476, -1.3076535561356231, -4.7884679796410099, -2.5985477198296234, 1.2801908889133431], [1.9786068045309682, 4.4917002106486432, -1.855874297736305, -4.3833283228766096, 1.2705987215568904]], [[4.9753690999973976, -0.46550869495813352, 1.770561582862392, -2.3490206045334663, -1.2588617557191761], [1.7932489458748915, 2.6630701489817756, 0.25417194136586296, -3.1084493710364667, -2.7992988823138241], [-1.1517748178776466, -0.8085309578666493, -4.8888742796580384, -0.52052359731492803, 4.6333101505860252], [-4.6801827559436866, 2.5982311266140998, -3.7514066947454818, 3.091210589967984, 1.5464462136289514]], [[3.1903561117105372, -4.9338449424543267, 1.2155435369182923, 0.0033248156935119511, 2.8287678980538189], [-3.7702760790945664, 2.7836734615180392, 4.2478338389184991, -3.1011165418304474, 0.67537526977856821], [-3.9674576870435341, -0.86371273917338343, 1.3774265703061239, 2.9123536297289823, 2.5371785850789133], [1.0964755853099115, -0.050490939417040082, -0.82569805750952074, -0.6209915742818648, -0.020437908786248293]]], [[[2.4919734207731628, -2.7595036229430558, 0.88472963589336828, -2.2046895628523555, -1.5127862571260522], [3.0469491880534054, 4.7788842146469683, -2.3837497100807914, -3.4563074950292902, -3.4391194958907931], [1.799732094088581, -0.41931916488461063, -0.6690178358941381, -3.0138112185807362, 4.267271640120768], [-0.12313353644872116, -0.075267641157389598, -1.5112685765373834, -3.896085522217815, -0.73703086872340684]], [[-1.3111396572335012, 2.5627135498634814, 2.2613318312862054, 1.8408817150897008, 0.49689674855203236], [-4.6721376850806182, 4.1337537603348888, -0.79330432181218047, 0.95699253677696827, -0.33537696859935906], [-0.22976480172215741, 2.3863981152132494, -4.6479208685937259, -0.53609753470888855, -1.6637192931666354], [-1.8355902289443957, -2.9709259033452531, -1.8716618462754662, -1.9883318366371592, -1.4062596160959195]], [[-1.1693105477507917, -1.0718103820960447, 2.7799144717477802, -0.038426766165211568, -4.3753913506850317], [-4.8551454141663406, -1.9445493803158342, -3.122194254603694, 1.282006487436739, -4.325704377631336], [-0.28790444757102485, -1.2183407158229, 0.21538953670776273, 0.22694411182139884, 3.90159375208464], [-4.1016237877470862, -0.4664073200156853, 1.2801612662251731, 3.7227214033084586, -3.0354145613222769]], [[4.4211676944055576, -3.6039802905578791, -2.4778082332244686, -2.2791563897463938, -3.8535996210633519], [-1.8449426164463576, 4.9047547050395188, 3.4241018771712071, -4.4917796424233805, 1.067726214335206], [-3.3334626205417495, 4.1448695655079977, -0.081710012950058974, 3.5829320138876817, 0.83955121227426055], [-1.7121568653363317, 1.9294275125743008, -2.5322063788202276, 3.5691578706165803, -1.5792020342474808]], [[-2.6346452214857665, -0.82514223198819359, -4.8047895096992974, 1.8149512203723823, 3.7524630717832892], [1.5327047307004289, -1.3274708568200722, -4.7033316562832352, 2.3324496721745023, -2.2968763948359481], [2.1100678061344809, 0.93715898213780324, -3.9305750911096493, 2.8170610747685396, -4.0938656716938979], [3.8231811085989413, 3.2384221624119576, -2.3083224576474981, -4.3826049820623414, 2.3654557890230068]]], [[[0.33449954350656164, -3.1557006295778112, -2.7616714793216435, 3.3570813361271128, -3.6670571068420066], [-1.2171828841962995, 1.3761287723265845, -4.7546310234746789, 3.8234239778325474, 3.2597984485563511], [1.7817645284808901, -3.6375466282562554, -3.0114745301215571, -2.9505162394357551, -2.9663836905245966], [-3.910301192792133, 4.680063819840818, 3.2135901336364689, 2.0741422136993384, 2.806997046479478]], [[-1.6732447797280114, 4.0671141263983159, 1.6670358858766807, 1.2545308818728875, 0.60690876151108242], [3.1811286653445823, 1.5239049440216847, 3.022518634703161, -4.3401648101157688, -4.539509126090449], [-4.1201150682933241, 0.2795537458761892, -2.4595667455580084, -2.2452031175756151, 1.7018823727194743], [-2.8828437772624649, 1.5595659142550113, 4.2058893279406959, -4.6090342064355996, -1.642313591639939]], [[-4.6752886323157972, -4.9564755517408194, -0.34403264299911829, 4.4179361722271544, -2.4312198740953894], [0.41061959291217853, 2.5473371011364252, -2.797743438233228, -3.7998429357997776, -3.7501301473662041], [-0.68749460509246152, -1.5280683061303755, -3.1404897620401453, -2.2005094382419421, 2.3091285335042482], [-1.5286398003182331, 2.1828847427685814, 2.768422976776308, 4.9373201168136465, 4.8118368109011591]], [[-0.48351340436980017, 2.6437337459176913, -3.478007496761645, -4.001921540366097, 2.1769600601338235], [0.34202388870400391, -0.72667113640157854, 3.276688962335836, 3.7692069753313557, 4.5441128983269099], [-4.2399232878584758, -3.1831016046626548, 2.386341552980439, 1.3682238276018355, 4.7554963213365493], [-0.97879840320314049, -4.3808780932342826, -0.99550972897896717, -0.029735591629621361, -3.7027950561215359]], [[-2.5799271601947571, 2.3086815113815398, 0.7862250478727475, -0.6986135068314816, -2.9920607337830942], [-1.2176741047951678, -4.6639754291150766, 2.0005568181372517, 2.0879221420871659, -2.3253072389152898], [-0.13956524908764845, -0.69940873679145277, -4.5811259313567776, -3.6516814273253928, -3.976114714328248], [1.9262220056839787, -0.030682753778270566, 0.5705534563476391, -2.8575677493202054, -1.3014148670222858]]], [[[-1.0859257628794392, -1.7170180830963688, 3.6389952240871182, 0.79904535213226424, -2.5975564551895602], [-4.4454060899589534, 1.8616609007229235, 1.9008544096890345, 4.0734374549475767, -0.15824855499927271], [-3.0214330082405905, -3.5183088740987478, -2.7889752521679134, -3.2571449093154312, -4.4875874084115006], [2.118814859509305, -3.7888728278536754, 3.2567179730053635, -2.3182331015831457, -4.7768994301016754]], [[0.34086168709337983, -3.1377110810787645, 4.2629068646046235, -4.9546725862875673, 2.0630439172385611], [4.9888406404941712, -4.209748832082651, -1.0029052034331212, 3.6271474854099317, -0.68511045786858027], [-2.7138061975198502, -4.4877078954007965, -2.7325603019159885, -2.5468217371427349, -2.1567016378974149], [3.7519713190238217, -0.94589284203396762, 0.67610189832388912, 2.2684696785999892, 3.6747404123102427]], [[-4.4782182130924051, -2.3745161424510872, -4.608568325503704, -0.85076467735334305, -4.4667637061212773], [-1.5419749826044615, 0.5022429456573887, -1.3123408827946612, -2.0056833684644317, 3.6036916459658084], [2.5640355730629878, 0.51843429446359313, 3.307687405539955, -2.0759588763130399, -1.3201225685089613], [-1.4251138032827981, 4.7518305730166546, -1.8877515189455094, 3.3442740275686607, 2.5977890875242791]], [[-2.5035630303195489, 4.1995743644298766, -4.9526670494219873, 4.8489165681252437, 1.4060089560405533], [2.6297073103898914, 2.7200041372660824, 3.0597083223718951, -0.75539663492123488, 0.095799465337293555], [3.4438320319957594, -4.9969217286630032, 1.9223249010698851, 1.044612416600387, 3.469576452665903], [-2.9142752883453427, 2.9251758963466141, 2.8207108360230846, -3.3004675616408141, 0.55188517457643105]], [[-3.2573459036793198, -1.0116116216171323, -1.5679696741086433, 3.3905405736593384, 0.33595938287643179], [3.7400368247869018, -0.548435638584027, -2.7264006885131344, 0.36305512561069442, 0.6487929276765092], [-2.0308923027560635, 2.9825458052692513, 0.26142748558745676, -0.98158963144642719, 0.87848693887865714], [-2.7518101067182874, -2.630298415817427, -1.7175561133676851, -1.1185058142983029, -4.727885494065851]]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-12.051883779084307, -7.9155259757247469, -9.9370377765416347, -10.327032363091895, -11.242058517369449], [-9.8242842078474997, 1.2705673285335704, 8.9284855142721042, 7.5776629220701581, -6.3479064842492825], [2.4755376873946133, 8.4714370089941955, -4.5397337011817509, -8.3987553358457152, -8.1953512195535314], [9.4230209881163827, -10.080730508608962, -0.17455292800164537, -12.188936386860506, 4.7312984456548657]], [[-15.333492110637273, 11.049014245866646, -5.0046444484495138, 8.1239125299302994, -2.3150955735520338], [-0.3245515418431158, -12.99677958116005, -16.699445624707128, -11.248279118287535, 3.4082392866698057], [6.7206030565195656, 4.7042883596893326, 7.9525595964025397, -6.7618319661014299, 4.9426209537809296], [13.89749259119111, -10.243304460610544, 12.635170712403839, 1.2290412425901958, 4.5657082669124751]], [[4.4860441180759558, -5.8693384949456977, 7.7013548276402108, 9.4506468683664284, -3.5002573488419619], [2.6742840064560487, -10.455263403057229, -2.3650051610076126, 1.7404001398518525, 5.4473019324579584], [-5.1737228358114296, 3.6249671921675666, 13.274188140657101, 7.20347540636158, -3.5488374961765472], [-5.4849273486626622, -12.451513494729308, 5.1446986172409321, 12.151094063387095, -3.5222468966972187]], [[-14.137940536984599, 1.3227831174918936, -5.0312034891222277, 6.6749447045171246, 3.5771642844851557], [-5.0956715884260309, -7.5673435790206502, -0.72225149953244394, 8.8329270626301337, 7.954449277626658], [3.2728675119116186, 2.2975104710594971, 13.892158042834433, 1.4791127088319171, -13.165950521826943], [13.299143073784647, -7.3830979031147379, 10.659945767719082, -8.7839415789850239, -4.3943603323706242]], [[5.0778642639573572, -7.8528521707136241, 1.9346947084513271, 0.0052918740739813482, 4.5023498686658581], [-6.0008818755417339, 4.4305762422208916, 6.7609767983888602, -4.9358279498412898, 1.0749470676896988], [-6.3147219001203307, -1.3747105022146735, 2.192352487519869, 4.6353873682390567, 4.03824777464812], [1.74518266801311, -0.080362858546834795, -1.3142052210587254, -0.98838838450978661, -0.032529574449313516]]], [[[-9.877341804036984, 10.937741256016407, -3.5067697532539022, 8.7386455258937712, 5.9961742824077868], [-12.077078486896101, -18.941884546802186, 9.4483795314465713, 13.699636691015392, 13.631509261965304], [-7.133530759526634, 1.6620396838998446, 2.6517609630335794, 11.945730757243453, -16.914024928511893], [0.48805979104966363, 0.29833553291450315, 5.990158708646578, 15.442768401905955, 2.921341676368574]], [[4.8991749337297996, -9.5757777720736925, -8.4496416255600142, -6.8785971843446605, -1.8566932071099851], [17.45780451945015, -15.446091263516184, 2.9642430741835142, -3.5758767741316548, 1.2531620326236925], [0.85853398685668947, -8.9169614872465619, 17.367316508006994, 2.003169982381507, 6.2166160659367895], [6.8588251363082549, 11.101094864561407, 6.9936095297727574, 7.4295559898962722, 5.2545980311742371]], [[-4.0661570217325442, -3.7271102356074022, 9.6668663178109089, -0.13362512239857949, -15.214972872379734], [-16.883268226151738, -6.7619702328743863, -10.857109016924305, 4.4580455472882239, -15.04219154003005], [-1.0011580698820899, -4.236654382392306, 0.74899493447368171, 0.78917477961560778, 13.567390511872718], [-14.262974363083828, -1.6218834277320002, 4.4516289805340028, 12.945380338126238, -10.555341569552203]], [[-12.625167811989224, 10.291592426356193, 7.0756747793059072, 6.5084009201294517, 11.004409979224819], [5.268452080114443, -14.006107776939562, -9.7779283195559081, 12.82680858991867, -3.0490186224556712], [9.519097190678151, -11.836165791576303, 0.23333261634017483, -10.231486580235341, -2.3974378884415208], [4.8892666461562504, -5.5097087039116941, 7.23101512472563, -10.192152883283722, 4.5095983842996699]], [[7.3236334376267989, 2.2936823492230456, 13.356074217138138, -5.0450957634762341, -10.430878435493495], [-4.2605233995931702, 3.6900262225819005, 13.074047581548591, -6.4836078388898688, 6.3847233130804746], [-5.8654436713690155, -2.6050599913737913, 10.925983859838317, -7.8307024090997608, 11.379889511447036], [-10.627456318102816, -9.0019774352833206, 6.4165404122679588, 12.182510240388721, -6.5753563213901458]]], [[[0.088857372325937711, -0.83828893412552907, -0.73361795447470535, 0.89178425502649972, -0.97412706536845917], [-0.3233357857412672, 0.36555860557896463, -1.263033170976678, 1.0156647880493148, 0.86594176307210358], [0.47331279572066015, -0.96628782123759449, -0.79997631915405099, -0.78378319232631644, -0.78799826537091433], [-1.0387430887111899, 1.2432249353446074, 0.853666860092125, 0.55098080256725801, 0.74565836192820312]], [[7.4835730637299926, -18.190133381666779, -7.4558038387222032, -5.6108787123946504, -2.7143942803899566], [-14.227570933226994, -6.8156519171208307, -13.518188918517541, 19.411350245127213, 20.302916004966782], [18.427179644068552, -1.250301753724244, 11.000391376386624, 10.04165187117613, -7.6116544551184129], [12.893494305100951, -6.9751453035624609, -18.810804291720402, 20.613866336220294, 7.3452335877552395]], [[-17.549456597775084, -18.60493745177045, -1.2913825030606283, 16.583442265004532, -9.1259793812882357], [1.5413274539242614, 9.5618442864248578, -10.50178521634162, -14.263328732102144, -14.076723691947173], [-2.58062286248592, -5.7358530191081787, -11.78833931101728, -8.2599702214091923, 8.6676896688930665], [-5.7379982155301414, 8.1938130592351115, 10.391726093529755, 18.533034409989817, 18.062012403854176]], [[0.82413783690533793, -4.5061853322433727, 5.9281863733594591, 6.8211862007029644, -3.7105799731165892], [-0.58297210637595198, 1.238594779551361, -5.5850434118796466, -6.4245294037852627, -7.7453393035957232], [7.2268548824507057, 5.4255258435611644, -4.0674660677793213, -2.3321070636124599, -8.105637643667329], [1.6683400945908058, 7.4671092111910609, 1.6968241774564317, 0.050683654151589555, 6.3113317655240238]], [[2.7307389127532025, -2.4436373776566351, -0.83218447618702818, 0.73945153084697501, 3.1669640915155166], [1.2888542406620007, 4.9366119280080429, -2.1175010034193082, -2.2099733388465266, 2.4612349756934013], [0.14772364988900102, 0.74029324662476326, 4.8489193950864671, 3.865143452275932, 4.2085417524605955], [-2.0388208886973715, 0.032476339249214996, -0.60390562535754644, 3.0246092096290846, 1.3774901376459847]]], [[[-4.7245554043000269, -7.470259331783109, 15.832237469525372, 3.476419996415931, -11.301232375037356], [-19.340702729930126, 8.0995592609962177, 8.2700791168916332, 17.722372559630806, -0.68849463867769878], [-13.145399193736957, -15.307165345303099, -12.134041341048684, -14.170914909588529, -19.524221698599817], [9.2183632964579214, -16.484312470473149, 14.16905743063373, -10.085975582225723, -20.782936356939711]], [[1.6041125180354805, -14.766228689580529, 20.06145754614634, -23.316942382714263, 9.7087900993481977], [23.477738991852423, -19.811293128637978, -4.7197231374063549, 17.06954141934126, -3.2241648249629686], [-12.7713106453389, -21.119382684763142, -12.859568418991673, -11.985473241632837, -10.149548118827164], [17.65696875902454, -4.4514200512396451, 3.1817700833939395, 10.675534229898023, 17.293516165408338]], [[6.2932306953479955, 3.3369025722293779, 6.476411435886849, 1.1955778229798857, 6.2771337006414774], [2.1669342203143893, -0.70580096184068064, 1.8442298997887248, 2.8185826458092862, -5.0642603383322902], [-3.6032338319712034, -0.72855463047799529, -4.6482861979129941, 2.9173406701125795, 1.8551654864592471], [2.002709449254426, -6.6777375728975912, 2.6528532641588241, -4.6997013013836106, -3.6506675752985496]], [[-10.908140794879646, 18.297741215615119, -21.579001139971556, 21.126955457951336, 6.1260465447097339], [11.457757301758802, 11.851184784497532, 13.331291749997455, -3.2912983415669776, 0.41740273494932134], [15.004936653874218, -21.771820839794181, 8.3756591787093768, 4.5514249804615821, 15.117106294483063], [-12.697633243230932, 12.745127700285797, 12.289968564144852, -14.380290975424698, 2.4045894247440054]], [[15.154991321737533, 4.7065819228039159, 7.2950701296485647, -15.77468727277817, -1.5630705741743867], [-17.400738914034712, 2.5516287152323165, 12.684737819009207, -1.689135093613342, -3.0185468412906302], [9.4488445912011922, -13.876467876648968, -1.2163065859436022, 4.5669028669245879, -4.0872115914762572], [12.802956615470009, 12.237616404283925, 7.9910297408951756, 5.2039133731105718, 21.996762318611488]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[-4.5379270918378207, 2.3487790662171353], [0.20727730295426294, 2.3539197529249245]], [[-4.5080229156863618, -2.9553241413013343], [3.6272839597224529, 2.4933632383308559]], [[4.0416940227590388, 2.7138724583294538], [-1.3685432275943974, -4.3972901904873387]], [[1.7408647080074342, 0.24782593487753246], [-3.8816924059842242, 1.3235394862761662]], [[4.0293852126692702, -0.067430472483628989], [1.4640562398560251, 1.1999358427098521]], [[-1.5515545418710728, -2.225605073607364], [-1.1906568538871323, 1.4591520219381779]]]),self.functionspace)
      arg1=numpy.array([[[2.1430591459562915, -4.9382395150095739], [2.0679088597184228, -2.1463401026384696]], [[1.6736582176091011, -2.8297788831579296], [3.2108782947669017, 2.4252608757877088]], [[0.56746846289589126, -3.8085055719651106], [2.3598998386509162, -0.24938434859119241]], [[-4.9032110657311909, -2.5152838632831265], [-1.3519616108184174, -3.371776671731789]], [[0.32478985729560339, 0.39634599052654806], [-1.378759313945964, -3.8276587078161892]], [[0.38528053492023417, 0.41845527449835718], [4.9194869844225337, 1.1616646848920018]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-9.7250461578458776, -11.598833596820747], [0.42863057119765996, -5.0523123640956031]], [[-7.544889598008619, 8.3629138479413569], [11.646767335228965, 6.0470563110511693]], [[2.293533894590583, -10.335798379150377], [-3.2296249419868226, 1.0966153497211253]], [[-8.5358271002429493, -0.62335257490051232], [5.2478991178960497, -4.4626795639418537]], [[1.3087034482118667, -0.026725797408197076], [-2.0185811768422006, -4.5929448771691224]], [[-0.59778376385000576, -0.93131618200130595], [-5.8574208956112299, 1.6950453737743407]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[-1.1801556228622854, -3.3091309053374776], [-1.0590857395353881, -2.78104487871465]], [[-0.86525853329423796, 3.2690517477383203], [4.5657720317754098, -2.1524348803927884]]], [[[-4.2548096243305551, 0.38865174223653831], [4.1620698160861487, -1.6305724093681864]], [[0.92909211411877468, -4.200273048577241], [4.3268401697935701, 4.5808104365705979]]], [[[2.8464631015198005, -3.3483465955376079], [0.79205503838133406, 1.5115972897564127]], [[-4.5052866739983957, -0.89580317816107247], [2.9074189447321164, 0.99231962109260152]]], [[[1.4540922559882254, -3.9413553331366211], [3.4976517078146916, -4.403961888603912]], [[0.5890956979625015, -0.52542712949400716], [-1.3814383662954, -3.1718795608095816]]], [[[0.075605769766276865, 3.0709562247253004], [-4.9823496025097649, -2.884486394775406]], [[0.52473263522935554, -4.2769290189412548], [-0.90491579410858858, 0.19644497872763189]]], [[[-1.8183673618468674, -2.8717617203670178], [-4.1557370289047624, -2.1959110504127297]], [[0.87143750569596179, -0.60789903093862119], [2.6748566605900068, -4.7798142701403741]]]]),self.functionspace)
      arg1=numpy.array([[[0.31529162071440098, -0.17223868986483559], [2.3590713152747789, -4.1912700428990659]], [[-4.4527372081081822, -2.0166067140101926], [-4.6417228251662488, -2.6262155440982591]], [[-0.96338437697853418, 1.2531639042057385], [4.1164071338236603, 1.0754300296016961]], [[-2.2139388265001747, 0.28112128210581133], [0.7296950552091932, 4.1721657303600956]], [[-2.9587773346853097, -4.4261472329021281], [2.8106733055218438, -1.7745103238607784]], [[2.8315185269567946, 2.9794393411096287], [3.0053522837903479, 0.46761356834227108]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-0.37209317902746331, -1.0433412462999663], [0.18241554023210577, 0.47900352636512189]], [[-2.0412065861911639, 7.7119262062383536], [-19.136383539486676, 9.0214358334813287]]], [[[18.945549127673459, -1.7305640736527044], [-8.3932579352984948, 3.2882232684116608]], [[-4.312588072787082, 19.496503281511604], [-11.363214910740625, -12.030195573089237]]], [[[-2.7422380816500391, 3.2257447988501946], [0.99257478424377865, 1.8942791612179593]], [[-18.545594204767667, -3.6874905930841462], [3.1267256417977918, 1.0671703194859603]]], [[[-3.2192713028455633, 8.725919601064696], [0.98326433246044675, -1.238047412469462]], [[0.42986021784824574, -0.38340157826453747], [-5.7635898104623049, -13.233607204439368]]], [[[-0.22370063795589584, -9.0862756735279859], [22.052612906499615, 12.767161474578998]], [[1.4748520103752807, -12.021050223149913], [1.6057824188703651, -0.34859364282279381]]], [[[-5.1487408738829545, -8.1314465162245284], [-12.381766395224892, -6.5425837731770562]], [[2.6189766979239231, -1.8269507409453245], [1.2507992678625841, -2.2351060068736484]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_array_rank4(self):
      arg0=Data(numpy.array([[[0.25937635468622489, -2.6440875674071318], [-2.7030556000563974, -2.4518265008643523]], [[0.23002157791258426, 2.810940654359614], [-0.73784841406022839, 4.7560708921705768]], [[1.2169983460546927, 4.6596027910476288], [3.6638951614646444, -3.0377421680965044]], [[-4.2059523572998518, 2.7105319813949134], [0.35999820774457802, -0.81202290492399065]], [[-1.9488359429332194, 0.00050506944462469505], [2.3464255827418059, -4.2096207096068481]], [[2.8125867251150662, -1.6322005695490214], [1.9003952346066963, -4.6647290457749229]]]),self.functionspace)
      arg1=numpy.array([[[[-0.72501108034964457, -0.0078701370371376811], [-4.0167585376384078, 0.86741615378212877]], [[3.6732617019403762, 1.2741595588656685], [0.27342110395288977, -0.70433692079242149]]], [[[3.0749591558898466, 4.4049805212673725], [4.5157697265053933, 2.1427926764941416]], [[-2.1142303884857161, -3.8447535093828469], [0.16211411870075665, -2.6602135959454087]]], [[[-1.7594663059249491, -4.5135428924975676], [1.0214295420556958, 1.6583449843419764]], [[2.8460259982651515, 0.39938388525592305], [-1.3264410160403428, -2.657126205326108]]], [[[2.8235844572106314, -1.2634563534237331], [-3.8009219131010075, 4.2078989442753354]], [[1.298816996895451, 3.6983687756212564], [4.6495462528750551, 1.5288965579094338]]], [[[-0.85890608487781606, 3.2841372265203876], [-3.8619525558191361, -1.7119380396692687]], [[2.8146975491380912, -4.0000604382064093], [4.7615113889539771, 1.5006049091821705]]], [[[2.4857162961747186, 3.4293609126717168], [-2.1862547282432585, -2.2612762502492001]], [[2.4662733173020985, 0.1249657529150916], [0.72307682412087715, 2.7968051472557764]]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-0.18805073112821249, -0.0020413274555738182], [10.620661310646167, -2.2935242679834396]], [[-9.9290306139026274, -3.4441241309572339], [-0.67038110856728206, 1.7269119279360552]]], [[[0.70730695705453073, 1.0132405701761189], [12.693560709960405, 6.0232630482214313]], [[1.5599815391021261, 2.8368452793506309], [0.77102624116255447, -12.652164450632378]]], [[[-2.1412675842496229, -5.4929742350164537], [4.7594559450212213, 7.7272289175597093]], [[10.427540884446273, 1.4633006847561272], [4.029385807918521, 8.0716643198733689]]], [[[-11.875861703840277, 5.3140372280280248], [-10.302520404245019, 11.405644662936188]], [[0.46757179107055752, 1.3314061308021616], [-3.7755380548380577, -1.2414990242819088]]], [[[1.6738670498139383, -6.4002446685679475], [-0.0019505542325344927, -0.00086464759492764671]], [[6.6044783369782785, -9.3858441447209167], [-20.04415695196953, -6.3169775026309685]]], [[[6.9912926570232035, 9.6453749786089595], [3.5684062126178873, 3.6908563835644199]], [[4.6868940594385569, 0.23748432132887795], [-3.372957463803341, -13.046338205776831]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[0.4809230226397796, 4.661300259289888, -4.3413364017748659, 1.163491866379176], [-1.2774503808024882, 2.0181595125469087, -1.2822677833849481, -4.6696873681773274], [-4.1109612406112293, -1.818054934625227, -3.2454060025567655, 1.4258601490730971]], [[-0.90533407072203342, -2.2879807980723221, 0.11845407955765452, 3.4364246393940618], [1.8824906605669351, -1.9092870782045779, -3.6800321852294928, 0.4506602428211135], [3.3276433313180291, 3.2364006070170657, 1.5092747315595387, 3.6020445934588636]]], [[[-1.5166924674613336, -1.2000236413763421, 2.7677256423012722, 4.9795512322447788], [2.8921860695883126, -1.9434146886114045, -1.188797834328903, -0.24163821608736669], [-3.5100275985092591, 2.0914997061227281, -0.11882811203671384, 0.8926763569011742]], [[0.89640044064808322, -1.1644857050219315, 2.361974564677749, -3.5165403816466889], [1.8808443021335419, -0.46579510530595236, 1.0603849654977617, -0.81712714443285961], [-2.1128445629095252, -3.4057419447857007, 0.59457434726605918, 3.5662198538623908]]], [[[-3.7231176160906143, -4.6949740923184002, 3.9947613146606535, -0.84541236096214156], [3.7679244585138054, 4.5793127407076621, -2.5678668881065461, -0.47316418485714973], [1.9214170956151193, -3.1728018020937587, 1.6807688452773197, -2.9234617703433043]], [[-1.7635262844454469, -1.9044564177132361, -3.1606422062785189, 0.14012469897268609], [-4.3634962080361408, 3.2977075271346479, -0.95395604276613177, 0.14826834701221436], [-2.7473218828526145, 2.2428117178089781, -0.81536821013734162, -2.1881781724744931]]]]),self.functionspace)
      arg1=numpy.array([[[[-4.0464424743815863, -1.0871041232178227, -0.012035635795579758, 0.28619314283387176], [0.41482069406426447, 3.9098101194642805, -1.5181160398181848, 3.5171396832053858], [0.62439092533506901, 3.3734942504834038, -4.1333952767770441, 1.0741959611374163]], [[-0.035247379977580984, 1.497186379959313, -2.1486920179963507, 1.6037654882926287], [-0.80925172241386267, -2.2292257749840019, -1.7657215437068121, -1.8145989660232145], [-1.5610687066406594, -3.9719360171799356, -0.20623720626064124, -3.3009515630234274]]], [[[-0.32152956777351971, 4.9377481762065258, -0.56407416481256689, 0.24678547569193388], [2.3951051061423625, -0.51775726231737806, -3.4756312546766646, 2.5283485792531231], [4.8785110824502915, -1.2847339903952779, 0.85295672735525851, -2.6261846628334395]], [[-4.2705742346654887, -1.8795036462286072, -0.95159561856895003, -1.0935167716947216], [1.6285850047002111, -3.4594610030215178, -0.27169186561831449, -1.0674855559076155], [1.0186772877082433, 0.56813283905828449, -2.030980824937453, 3.9372765219808308]]], [[[-3.6452518746821525, 1.7605365024992556, 1.6076752360651172, 1.1309443468241929], [-3.2939894825835694, -2.4331172718694214, -0.62598249633107539, -2.636693774773351], [1.0914304095770833, -4.3499952382145288, -0.22841193415683847, -2.3394410085942763]], [[0.42633481766968728, -3.0526628139489418, 0.0092668332192946679, -3.5127817901820526], [-0.080186981082233544, -0.83028735279996368, 2.9886765580340242, -1.6088887774569272], [3.510468641113091, -0.24123053853587706, 1.2350567991444343, 1.8237913900644296]]]])
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-1.9460273457175814, -5.0673187314303432, 0.052250743797855, 0.33298339390070358], [-0.52991285359714713, 7.8906204848490029, 1.9466312892987994, -16.423942750779396], [-2.5668468930418489, -6.1331978690211839, 13.414545842192002, 1.5316532132811154]], [[0.031910653997389689, -3.4255336884823198, -0.25452133524463699, 5.5112192397786384], [-1.5234088094918024, 4.2562319665775403, 6.4979121109941733, -0.81776761065096326], [-5.1946798713820508, -12.85477613703409, -0.3112686041166185, -11.890174730858122]]], [[[0.48766147350819566, -5.9254145466107468, -1.5612025301114154, 1.2288809195818833], [6.9270896231847781, 1.0062170687228207, 4.131822908485467, -0.61094564033775267], [-17.123708539033803, -2.6870207633576033, -0.10135523756063944, -2.3443329573678935]], [[-3.8281446257744949, 2.1886551285698106, -2.2476446469186491, 3.8453958856724118], [3.0631148266305197, 1.6114000022042434, -0.28809796954969891, 0.87227142402211355], [-2.152306768693784, -1.9349138401909833, -1.2075690982970686, 14.041193702834301]]], [[[13.571701469716459, -8.2656732678148543, 6.4222788395708639, -0.95611433036542792], [-12.411503537513866, -11.14200492270751, 1.607439724862846, 1.2475890606585538], [2.0970930476356195, 13.801672730906326, -0.38390766282034861, 6.8392663525987487]], [[-0.75185265693475067, 5.813663287139609, -0.02928914419144657, -0.49222749090599349], [0.34989558788619185, -2.7380448530131414, -2.851066062410041, -0.23854727956004101], [-9.6443873167978769, -0.54103467852163534, -1.0070260517363516, -3.9907805108858989]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_constData_rank0(self):
      arg0=Data(-1.83359228247,self.functionspace)
      arg1=Data(1.72429180706,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(-3.16164815015,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([0.055770144059566817, -2.7665845733676662]),self.functionspace)
      arg1=Data(3.93760807724,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([0.21960096971766843, -10.893725762454135]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_constData_rank1(self):
      arg0=Data(0.943140501997,self.functionspace)
      arg1=Data(numpy.array([3.51482835508844, 4.8769286377212264]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([3.3149769792510675, 4.59962892358348]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[-1.4038383478834016, -3.7444810841337839, -3.3923787252930069, -3.6023732401095767, 2.4826650426930907], [-0.37046343778318924, -1.4313929588638699, 2.8562674076568406, 0.12879802829397136, -2.9105120086511507], [3.9020383389129503, -1.6463352572918732, 2.3686646659179242, 2.5337638731858618, -3.0964880168042552], [-0.64538040886386661, -2.7612064176164441, -3.9509645188834561, -2.2450801209019766, 4.116800615796592]]),self.functionspace)
      arg1=Data(2.29211981252,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-3.2177656907633394, -8.5827992805612858, -7.7757384878261977, -8.2570710757584482, 5.6905657322155152], [-0.84914658555829281, -3.280924160518039, 6.5469071149544389, 0.29522051246653158, -6.6712422396159106], [8.9439393858473739, -3.7735976612941333, 5.4292632099740494, 5.8076903739847587, -7.0975215325576126], [-1.4792892217711728, -6.3290159362847778, -9.0560840523088437, -5.1459926258213349, 9.4362002556749562]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_constData_rank2(self):
      arg0=Data(-1.11625991253,self.functionspace)
      arg1=Data(numpy.array([[-2.3434671875811008, -1.3749804016888545, -0.62974905683459426, 1.0733751052656881, 2.8675887666839479], [-2.8711473248946815, 4.686057056283957, -1.7422316301397167, -0.98959990091727423, 4.0783649312995589], [-3.8944421775449367, 3.1885479959787908, -3.3519027934887791, -3.9933556466941935, 0.43156861428957072], [-0.85286805939608445, 1.8632274370407211, 0.90479363453506512, 1.3015130135436301, 3.8011924246818865]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[2.6159184778356779, 1.5348355029252232, 0.70296362710057991, -1.1981656011200956, -3.2009743858822262], [3.204946661759287, -5.2308576397770619, 1.9447833270738024, 1.1046506988416136, -4.5525152814943519], [4.3472096844751968, -3.5592483071018814, 3.7415947190823968, 4.4576228248961849, -0.48174274363931413], [0.95202242538455173, -2.0798460959021035, -1.0099848633474702, -1.4528268026601305, -4.2431187235004675]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-1.9476203477815246, -1.0120412905901954], [4.2219266679886331, -1.5004686941455745]], [[0.66076344712367874, -1.6545964895744802], [4.9891548917210962, -2.3429286486196057]], [[3.1119960904456931, 3.7658497193097329], [4.2334738008925079, -0.42284907063853616]], [[-2.4352550585186181, 4.5557193271698289], [-1.8101296015467705, 3.8493171003844413]], [[1.2849240735215801, -0.23487976907268138], [4.305713976007512, -2.6656240119550367]], [[3.7316927004339444, 4.128555769749914], [1.0128773556867134, -0.28771867140479301]]]),self.functionspace)
      arg1=Data(-2.53919431978,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[4.9453865241785042, 2.569769496451372], [-10.720292213892479, 3.8099815851850143]], [[-1.6778067916559556, 4.2013420078585986], [-12.668433761570309, 5.9491511162291957]], [[-7.9019627960431968, -9.5622242164235516], [-10.749612628171764, 1.0736959582904282]], [[6.1835858118106239, -11.567856638070268], [4.5962708023166412, -9.7741641163355233]], [[-3.2626719088370222, 0.59640537546103789], [-10.933044470483832, 6.7685373498304804]], [[-9.4754929081134662, -10.483205359451789], [-2.5718924281954183, 0.73057361612624661]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_constData_rank3(self):
      arg0=Data(2.89960067434,self.functionspace)
      arg1=Data(numpy.array([[[-4.8469768761334509, -2.8296169116320113], [-2.9749206166968865, 4.8546690903148519]], [[2.3586347370076552, 3.8918759101363456], [-4.1099978309832075, -2.5617603490082228]], [[4.2650341699765377, -2.4714574660362443], [3.3803977395452343, 3.7224543726659824]], [[-0.18986821935290354, 1.1114359622701357], [-4.8040610460270727, -0.40199071918081231]], [[-1.6542827922285253, 1.9926405417820217], [1.3820759696107867, -3.6729389924940872]], [[-1.7567168190705784, -3.2349609352469102], [-1.678980233564995, 1.9642651013534982]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-14.054297418529185, -8.2047591050816813], [-8.6260818262713617, 14.076601767956713]], [[6.8390988739405048, 11.284886013464691], [-11.917352482239789, -7.4280820354723316]], [[12.366895955331486, -7.1662397351122671], [9.8018035651103883, 10.793631209168527]], [[-0.55054201687071858, 3.2227204656801405], [-13.929858648613026, -1.1656125604136323]], [[-4.7967594998888297, 5.7778618586610726], [4.0074684134674836, -10.65005637943208]], [[-5.093777273195033, -9.3800949092936463], [-4.8683722174424391, 5.6955844124599357]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[-1.5919833201710762, 3.3817678546845933, -0.87282744431641746, -4.6689636817019995], [-3.192235421807641, -0.34694495351237187, 2.8029564962260203, 1.6266634134704292], [-4.103500231769301, 4.1592934785553428, -4.0079287962581498, -0.29569721292541562]], [[-2.6471274245063201, 0.78623948561007317, -1.2254520247985745, -1.0102956268706587], [2.3958031959655433, -3.4180961297528456, -1.2819905372591425, 1.8983467106974414], [1.6944828225212794, -1.2403404059633871, 3.1797112533538847, -2.6205355636778882]]], [[[-2.8258246532516229, 3.0236184404907522, -3.3171455809356223, 0.79928384466376912], [1.6191105410225584, -3.6379694646843177, 0.37381163131271045, 4.9513073570605144], [0.90059749323451221, 4.338803272322453, 3.2368928254588827, 0.092464611567375954]], [[3.2453848911647558, -3.7657956323529831, -1.179941134014725, -1.9241306115611092], [-4.9728309956409262, 4.7309894463488185, -1.920666236609363, 2.3515483454678998], [4.4842255402831661, 2.8294054683001661, -0.7104387128414551, -4.7134852464538906]]], [[[-2.2592155410877099, 1.2428976903520503, -3.3608470303241731, 1.1665257389447881], [-2.9015562912055612, 2.7836474741912918, 3.9148684384969155, 2.7394022967260403], [-4.7273135307425429, 3.3891231164429758, -1.5551019395385679, 0.18890829056314296]], [[-1.160339864544234, 0.72762628494266757, -0.92947792937064833, -1.8647004283594457], [-3.1994584131729753, 4.4184958673372776, -3.9981146754318453, 0.16498337558555143], [-1.6544687668928857, -3.5771972584943912, -4.2714452425831739, 4.0671960035329455]]]]),self.functionspace)
      arg1=Data(0.736443333278,self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-1.1724055028291842, 2.4904803912750793, -0.642787952468582, -3.4384271767048404], [-2.350900494642957, -0.25550529802850536, 2.0642186251128916, 1.1979454263369391], [-3.0219953887897328, 3.0630839534272272, -2.9516124422557719, -0.21776424112769971]], [[-1.9494593441140762, 0.57902082753717565, -0.90247597391448953, -0.74402547904844585], [1.7643732915140762, -2.5172341072585698, -0.94411338448950932, 1.3980247793426583], [1.2478905779992666, -0.9134404229666121, 2.3416771542803421, -1.9298759454875307]]], [[[-2.0810597268987427, 2.2267236428747346, -2.4428897485914076, 0.58862725879914979], [1.1923831637756082, -2.679158358934373, 0.27529108378188294, 3.6463572941157238], [0.66323901985910672, 3.1952827443050587, 2.3837881418434024, 0.068094946752900135]], [[2.3900420670182481, -2.7732950879323912, -0.86895978180519695, -1.417013161239598], [-3.6622082342561453, 3.4841056375704378, -1.4144618454024105, 1.7317821018998882], [3.3023780040548396, 2.0836967942689428, -0.52319785377443395, -3.471214786253459]]], [[[-1.6637842236712694, 0.91532371800593804, -2.4750733896481756, 0.85908010354265629], [-2.1368317867881164, 2.0499986245633051, 2.8830787621900793, 2.0174145585893348], [-3.481398534028505, 2.495897124761552, -1.1452444559402986, 0.13912025118610105]], [[-0.85452455757987544, 0.53585552666360048, -0.68450782451371461, -1.3732461990252645], [-2.3562198184802794, 3.2539718246153209, -2.9443848984012586, 0.12150090705161952], [-1.2184224934943355, -2.6344030728372281, -3.1456773723608498, 2.9952593819352789]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_constData_rank4(self):
      arg0=Data(-2.57343614762,self.functionspace)
      arg1=Data(numpy.array([[[[-2.1515725178707146, -4.1979287018982454, 3.6656992283344518, -0.15622258668361866], [2.2364818182424715, -4.4349865449102657, -4.8035994443275163, -1.5597474917045662], [-1.5943503065952545, -2.4531848406287917, -3.6382877568573146, -3.3102200567028648]], [[-1.4092720288850034, -3.9082900757226344, 0.27452523343926849, 3.0784679765176524], [1.9186889827191891, 4.0618560531010814, -0.12668556648413443, 1.5033082598098249], [1.6505382356915508, -3.1207584382494078, -2.1778912296585462, 4.4394293680339914]]], [[[0.81501386624051086, -2.52243326184584, 3.1982746688917771, 1.4446514309805893], [-3.2357693111572017, 0.95658834286632199, -3.9320488140607459, 0.97348286828397601], [3.3762115311327641, -1.3183380863835237, -1.7328289727773969, -3.0433260076669058]], [[-1.9530519917658795, -0.31427091847646871, 3.2311401418466072, -3.0340242465208656], [-3.3176857423276851, 2.5270272660278383, -2.8849643615828082, -0.83161779728995811], [4.2107579792274734, -0.94482321277950643, -2.2721669330540841, 3.9002170350188354]]], [[[-4.2111887495012601, -4.9772439963120618, -0.90348915946983865, 2.6312094570410993], [1.3391892852200291, -4.4392401877346579, 2.6831509606234789, -2.4560108971074524], [-1.8679880854973288, 4.1339030569405892, -2.3242657209323792, -4.2915392876395684]], [[-0.83563423147085913, -3.4393027151133557, 3.5396305930719922, -1.2063213537872497], [2.82137952958717, -3.9933591713880947, 4.4158864390723913, -4.5482309261904952], [-1.2747362840453693, 3.0480931585636828, -4.751262484228338, 0.089895903902177388]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[5.536934491715221, 10.803101466598292, -9.4334429005002303, 0.40202885164639179], [-5.7554431545610614, 11.413154688882358, 12.361756448721888, 4.013910576312842], [4.1029587109619579, 6.3131145456686193, 9.3629012289414977, 8.5186399504973327]], [[3.6266715809630639, 10.057734956250853, -0.70647315916655296, -7.9222407700624773], [-4.9376235841706499, -10.45292719348121, 0.32601721617204399, -3.8686678168109823], [-4.2475547587583016, 8.0310725729825343, 5.6046640159888312, -11.424588010506437]]], [[[-2.0973861441952204, 6.4913209359942181, -8.2305556429448909, -3.7177182131970432], [8.3270457106928326, -2.4617190199245278, 10.118876552312004, -2.505196002331211], [-8.6884647962300061, 3.3926588862841172, 4.4593247161893474, 7.831805157123414]], [[5.026054593792411, 0.80875614175322086, -8.3151328390554937, 7.807867668753663], [8.5378524157510167, -6.5031433124184916, 7.4242715726939226, 2.1401153005504652], [-10.836116792625175, 2.4314422088776597, 5.8472765189492515, -10.036959501482485]]], [[[10.837225352419059, 12.808619615636273, 2.3250716619628906, -6.7712495287103147], [-3.4463181150912017, 11.424101167085714, -6.904917671590967, 6.3203872215660217], [4.807148062543126, -10.638335557489548, 5.9813494229224649, 11.044002331744936]], [[2.1504513374561345, 8.8508259296818323, -9.1090133174346395, 3.1043909774825331], [-7.2606400675959746, 10.276654842081729, -11.364001786095795, 11.704581873183811], [3.2804524320457094, -7.8440731155623409, 12.227070623746092, -0.23134136862487661]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([0.17454031129135572, -2.2388362477513635]),self.functionspace)
      arg1=Data(numpy.array([0.45182574531524011, -4.1410343612543601]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([0.078861806236770818, 9.2710978311601764]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[4.3791064973912004, 4.9812370951509806], [1.1802809250844533, 4.983855971916892]]),self.functionspace)
      arg1=Data(numpy.array([-3.5666670604145558, -0.74904910813725323]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-15.618814898292554, -17.76641426739009], [-0.88408837428592191, -3.7331528708488713]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_constData_rank2(self):
      arg0=Data(numpy.array([-1.0034837739904887, 4.8827997755598371]),self.functionspace)
      arg1=Data(numpy.array([[-4.8107691098825578, -3.7384915382371431], [1.0447009400072504, -1.8280590209546475]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[4.8275287421818129, 3.7515155978217156], [5.101065515394553, -8.9260461772274891]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_constData_rank1(self):
      arg0=Data(numpy.array([[[4.2613265156855942, 3.8323682885429999, 2.750895270986069, 3.0957340840618173, -4.9271854510797564], [-2.5107869331882551, 4.0575728315821422, -4.5075229640299499, 1.1476548414662648, 3.3048942016993532], [4.5843787681160215, -3.5471983058260923, -2.4861432846752898, 3.816580868167188, 2.5658141066455507], [-1.720841914636837, -0.7550198993326962, -3.1606081157284494, 2.4239704715009838, -2.0621022449508466]], [[4.2889217806546132, 0.38653229405041767, 2.5200531496548253, 1.5751519397048863, -1.4913653313021213], [1.0510589250138391, 1.7139166193246105, -3.9417613778493465, 0.32904580976158027, 2.234090408072249], [-4.5111229364975447, 2.4694581770109565, -2.9413502526746251, 1.2128849719841082, 2.6644628140815687], [4.6479537350315354, -4.9834950051909956, -0.22630527364225195, -4.0265659370263363, 0.22509099133705313]]]),self.functionspace)
      arg1=Data(numpy.array([-2.1978733796700176, 1.4774824668052586]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-9.365856110907357, -8.4230602424802044, -6.0461194863604204, -6.8040315338966124, 10.829329739625605], [5.5183917624777887, -8.9180313126066864, 9.9069647308927209, -2.5224000251081176, -7.2637389885408021], [-10.075884056766633, 7.7962927287857546, 5.4642281434331981, -8.3883614915025468, -5.6393345221780633], [3.7821926348006887, 1.6594381378644694, 6.9466164411285733, -5.3275801724181937, 4.5322396303352477]], [[6.3368067324163802, 0.57109468731350665, 3.7233343440323727, 2.3272593734682636, -2.2034661286001], [1.5529211332871304, 2.5322817546182548, -5.8238833241025478, 0.48615941469847346, 3.3008294071845534], [-6.6651050442781745, 3.648581159042565, -4.3457934270599763, 1.7920162803581072, 3.9366970912601174], [6.867270150031108, -7.3630264935812777, -0.33436207395199347, -5.9491805733916987, 0.33256799313631036]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_constData_rank3(self):
      arg0=Data(numpy.array([3.8851474095629932, -2.6519897157621997]),self.functionspace)
      arg1=Data(numpy.array([[[-3.522214434012235, 0.29078245031838357, -0.56969479513155896, -2.6937079692025079, 0.67972872576362953], [2.5700641018885717, -1.6755412038471942, -3.6740880459130065, -1.8126807907974065, 4.3965226314700487], [-3.7031616748379403, -0.33524011406041776, 0.75434415191939586, 4.4049731481332675, -4.3583364694897941], [4.2682866473905055, -3.5040052880620207, 4.9154005460965884, 3.5430927496822822, -0.29520775315485714]], [[4.2931556931011947, 3.9970938705691967, 3.6199309162383404, -3.9354593392830326, -3.4336482356573903], [-0.70641213686494986, 1.277413873093149, -3.1163372216635112, 2.9495023348905267, -2.873999425419933], [-1.3324140404526963, -3.9404720151676074, -0.112912720647663, 4.6474348417979847, -4.6640305422205834], [0.63260999626461967, 4.9643200299987562, -3.6766993354741375, 2.2783413026560204, -3.2738954779124696]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-13.684322284228019, 1.1297326836008477, -2.2133482575468966, -10.465452538666314, 2.6408462981061196], [9.9850778878632251, -6.5097245677429854, -14.274373654085277, -7.0425320787311421, 17.081138512740935], [-14.38732898818958, -1.3024572607234344, 2.930738227748634, 17.113970015664506, -16.932779644442196], [16.582922811381536, -13.613577068009189, 19.097055698631682, 13.765437618269541, -1.1469256374525048]], [[-11.385404746270307, -10.600251837685635, -9.6000195616337152, 10.436797694578903, 9.1059998085084217], [1.8733977220554465, -3.3876884542149908, 8.2644942626985785, -7.8220498587462721, 7.6218169193201337], [3.5335483324177099, 10.450091259473245, 0.29944337393633247, -12.324949405123181, 12.368961031969784], [-1.677675204182135, -13.165325665308997, 9.750568825627127, -6.0421377036400195, 8.6823371379042413]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_constData_rank1(self):
      arg0=Data(numpy.array([[[[0.41574816353600408, 1.5067703719077583], [4.2158415950338473, -3.7926179710752894]], [[4.7897084765728586, 2.2290728683117935], [4.1835162927413236, -4.8329328280664585]], [[-2.5797107600350389, 3.57285437095166], [-2.2401221837500018, -3.9030203653385023]], [[-3.1732544703900665, -4.8347979078496559], [-2.4006146629335312, 4.0920277463533878]], [[-3.1972826624375927, -0.90131120994514546], [0.93124181385150795, 0.86513310905223406]], [[2.6657190298806013, -4.2395668413943453], [-3.4293608765672525, 0.58555100230342827]]], [[[0.15996677142588567, 2.8155038238431302], [-2.6065815060128408, -2.861410759852383]], [[-4.5391126328253248, 3.5620476274202932], [-0.022278696675046561, -1.4077806322848829]], [[0.56043486928377639, 1.3864495807128758], [-3.6685908058046146, -1.7187644235747266]], [[-4.744071029810434, 3.792633553941501], [-1.1187304022725564, 3.6330980156880344]], [[4.8066354840858967, -3.538831426571373], [-2.4793567486381693, -2.6914949706097016]], [[2.0064403156474775, 4.3169312529445598], [-2.0998585713569931, -1.298945236746949]]]]),self.functionspace)
      arg1=Data(numpy.array([-0.57809610467004191, -2.1145790571627829]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-0.2403423938638875, -0.87105808263210538], [-2.4371616039950035, 2.1924976755802223]], [[-2.7689118128118504, -1.2886183421967252], [-2.4184744726574139, 2.7938996420371889]], [[1.4913207415516492, -2.0654531944004875], [1.2950059084108239, 2.2563208696500321]], [[1.8344460484592944, 2.7949778373947542], [1.38778598545566, -2.365585300368624]], [[1.8483366526842329, 0.52104449956473098], [-0.53834726509342101, -0.50013008036417905]], [[-1.5410417873187787, 2.4508770764983443], [1.9825001642513691, -0.33850475351725062]]], [[[-0.33826238469912373, -5.9536054212604164], [5.5118226634025795, 6.0506792667240941]], [[9.5983125114754522, -7.5322313135593317], [0.047110065409935585, 2.9768634421089941]], [[-1.1850838374912354, -2.9317572471875688], [7.7575252872543761, 3.6344632542875792]], [[10.03171324532982, -8.0198234846575538], [2.3656438792568433, -7.6824729765935809]], [[-10.164010730063533, 7.4831388214573202], [5.2427958559054835, 5.6913788973102353]], [[-4.2427766709152399, -9.1284924186880581], [4.4403169579952584, 2.7467223940264511]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_constData_rank4(self):
      arg0=Data(numpy.array([-0.46798773920893666, 4.9990946463718107]),self.functionspace)
      arg1=Data(numpy.array([[[[0.017992445067295471, 2.8897939654231006], [-4.1350173043898693, 3.1627377402643866]], [[4.1390066136212997, 3.3751625314367359], [-4.1761896068268856, -3.221264727226326]], [[2.5740916595229715, -4.513012279404486], [-1.9165600607073463, -0.12149394607734454]], [[1.3270634322872414, -0.82984833914526579], [-1.6803551911871093, -2.0380096235096756]], [[0.68330167459585667, 0.53556349683698379], [0.089660579927244299, 2.3300079882978899]], [[4.4293107177479829, 2.5378553219623337], [-0.029014983286375973, -3.6147610276601583]]], [[[-2.3611753358537579, -3.5087262391433849], [3.9653359317369414, -1.8645682122410356]], [[3.8564951981073516, -4.3600653063648593], [-0.32363004700937203, 3.1405234152469763]], [[-3.6259127756873379, 2.5130729804161582], [1.3930524087586473, -4.5713772162085577]], [[-2.6409351934798209, -0.084169828848086325], [3.4613269140274348, 0.10253409956044734]], [[-4.5624555647732414, -1.0903657239428388], [4.9245909222718378, 2.7515739062471276]], [[-2.0183899814595483, -4.8674647768858668], [0.87998181596531211, -3.9834351134296608]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-0.0084202436898845913, -1.3523881446579848], [1.9351373998712464, -1.4801224847771115]], [[-1.9370043476794689, -1.5795346825497896], [1.9544055326067722, 1.5075123970881403]], [[-1.2046433362567353, 2.1120344136606755], [0.89692660986857342, 0.056857677152308932]], [[-0.62104941546295789, 0.38835884812288385], [0.78638562699165582, 0.95376351619234923]], [[-0.31977680589179547, -0.25063715008757254], [-0.041960052096313226, -1.090415170782292]], [[-2.0728631090527911, -1.1876851745645205], [0.013578656431376174, 1.69166384111525]]], [[[-11.803738980611683, -17.540454557685994], [19.823089627411921, -9.3211529676092191]], [[19.278984498617056, -21.796379130880037], [-1.6178572354096092, 15.699773791966475]], [[-18.126281145149722, 12.563089682340067], [6.9640008387407093, -22.852747368094271]], [[-13.202284987139874, -0.42077294078049993], [17.303500845257208, 0.51257766818318651]], [[-22.808147188167187, -5.4508414531499687], [24.618496115100363, 13.755378383816387]], [[-10.090122550604926, -24.332917107533497], [4.3991123850967355, -19.913569149715705]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-0.24516749416719286, 0.71579045465148816, -3.6685912651486907, 3.7219897590727076, -3.4595493886286057], [-2.2802659670357581, 2.8026513993055699, 0.07942435239090706, -0.69691791761499999, 0.08230241980416686], [-1.7526669315523371, 2.1450532430304641, -4.3930904150691887, 2.9686549523813355, 2.8599391052761094], [-0.57291351160844517, 3.1789395260875413, -0.027738542101269914, 0.83786346401012501, 4.4745914630992054]]),self.functionspace)
      arg1=Data(numpy.array([[3.722160146074792, -1.6437121195301563, 2.4260451367017142, 0.39128146667702612, -4.7409346766495828], [-3.4999965731266789, -0.59861174925695337, -1.0020968376806607, 0.6555777198290329, -0.36860240573503145], [-0.37735120640702302, 3.5215895143041269, -2.7240081550014503, 1.3734368290145014, -2.0843370470072617], [4.4090619323685161, -4.621148708551801, 2.3863063978743346, -2.0641568662593657, -2.3851707277311718]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[-0.91255267590214928, -1.1765534453546518, -8.90016799736037, 1.4563456118868401, 16.40149766213122], [7.9809230704425458, -1.6777000566957554, -0.079590892365762389, -0.45688385933803949, -0.030336869937630399], [0.6613709810509697, 7.5539970082801444, 11.966814116307177, 4.0772600442368168, -5.9610770293117961], [-2.5260111545723634, -14.690352285543716, -0.066192660483966992, -1.7294816222243565, -10.67266457634002]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_constData_rank2(self):
      arg0=Data(numpy.array([[[-2.0726667184113756, 3.9902979553743609], [2.1859456268781061, 1.0662656424174131], [-3.5635742541289983, -2.7571725491389429], [2.111844695611234, -3.7831666962410115], [1.3042088453624725, 2.4057662168596137]], [[2.6145253688631804, 4.3153707341002239], [2.0786998246001884, 2.776097310140976], [-0.76506124214917115, 0.31603255910115902], [-2.326362501458008, -2.874274399181278], [-4.4888405869423593, -1.2939305791527667]], [[-2.2907457740756101, 0.68133421871974598], [-1.3980985483262485, 2.3760430432342758], [1.7909208086500472, 0.76593048748897807], [4.2104828961791814, -3.1529695473479871], [-0.29739127533181442, 2.7617621924264313]], [[4.7324187151936119, 2.3315805843665514], [4.8115190563794563, -4.4498896649999491], [-4.9724289901289254, 2.1432379979683827], [3.6901804684173314, 0.038468288206887635], [2.8496202089182514, -0.5261526857296408]]]),self.functionspace)
      arg1=Data(numpy.array([[-0.11271915746662842, 0.9131377671756189, 1.6151008077144144, 2.4882813169335662, 2.6468647587275642], [-3.184147322573545, -3.8024470696405732, -1.0917892418309449, 0.85123141541996628, 0.37178499114080754], [0.2937412258031733, -3.3635288391773077, 3.5082769928195496, 2.4853762960795747, 2.7914719489396127], [4.7897841385873221, -0.83684078236456205, 2.1810706799183723, -1.1910146108292752, -0.62217429011256176]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[0.23362924620845182, -0.44978302357060801], [1.9960695088947824, 0.97364742793311343], [-5.7555316561940373, -4.4531116111223179], [5.2548637003546874, -9.413583009101794], [3.452064430810696, 6.3677378171430465]], [[-8.3250339530663062, -13.740776168897462], [-7.9041460567133601, -10.555963081982632], [0.83528563352028451, -0.34504094809494767], [-1.9802728448960336, -2.4466726651204525], [-1.6688835578488623, -0.48106396890713132]], [[-0.67288647168040883, 0.20013594858838557], [4.7025447873072652, -7.9918892990451011], [6.2830462689487439, 2.6870963073566436], [10.464634385212214, -7.8363157752394335], [-0.83015940294813695, 7.709381689800348]], [[22.667264099188156, 11.167767700837068], [-4.0264753715025838, 3.7238491486945362], [-10.845219078346322, 4.6745535574557913], [-4.3950588544818601, -0.045816293307994672], [-1.7729604305741229, 0.32735867373465705]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_constData_rank3(self):
      arg0=Data(numpy.array([[0.029877512895187586, -2.6212879305317327, 0.73874274128979955, 1.4591239827227902, -2.5598105570078644], [-3.1984103794474117, -2.6462712782194311, 3.6914018573567482, 1.0325845844970321, -2.0211759520215367], [4.7400970054128777, 3.3598197240674779, -0.90323096399586866, -3.7084811617530877, 1.1689067201761318], [2.3453252002327574, -3.780591751010709, 4.561968646070687, 0.76509616220079923, 4.8663955647121497]]),self.functionspace)
      arg1=Data(numpy.array([[[-2.3521270585600726, 3.5899308667735834], [-2.8564798313504145, -4.1302836660290607], [-4.4296518104590294, -3.6406369166703878], [1.9788943324053587, 3.5298479855593285], [-0.046152980196496607, -1.6544368121847608]], [[-2.5847307065250291, 3.7034281283368902], [-1.2716936370183696, -4.6864205195610165], [-0.93954743601740365, 2.8141225706772115], [-4.8708231668645672, -0.68273837267635695], [1.948711537630369, -4.6628363725720767]], [[1.7718204464068554, 0.85149765682177225], [-1.2309329741782538, -0.029298596623045015], [-4.8095239450102554, -1.8401251246904229], [-1.9132227750831787, 4.0663591405195803], [-3.3509331657828523, 0.86936813142212443]], [[-3.2776248723999624, 1.8942576059196439], [-3.7779873239555792, 0.88510618890411941], [1.2654089668647117, 4.1433376557214849], [-3.720517168044617, -2.8949977219249003], [-0.66487653873622143, 2.7142184098115161]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-0.070275706523248216, 0.10725820576485968], [7.4876561057261606, 10.826662723434335], [-3.2723731214178269, -2.6894940958619258], [2.8874521796868642, 5.1504858510953451], [0.11814288594436692, 4.2350448177329882]], [[8.2670295198260941, -11.84508296521021], [3.365246346336118, 12.401540018572501], [-3.4682471503894146, 10.388057284227406], [-5.0295369159153669, -0.70498511887019588], [-3.9386888972854135, 9.4244127444540151]], [[8.3986007921424441, 4.0361814932169651], [-4.1357128856491405, -0.098438002821603435], [4.3441109492128263, 1.6620579902471486], [7.0951506196329328, -15.080016269539341], [-3.9169282963446563, 1.0162102511262878]], [[-7.6870962101493072, 4.4426500988959123], [14.283027712369487, -3.3462251565394401], [5.7727560312935156, 18.901776475485438], [-2.8465534066731224, -2.2149516466247978], [-3.2355522391871139, 13.208460431166825]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[0.80686620312221891, -3.6141990755541444, -2.6301923612626421, 0.60077132518615883, 4.4867956252786101], [1.0895842602210859, -1.7614928081460892, -2.1890874372689719, -2.3746849785584292, -4.6930570682429531], [2.2618285108896439, -1.4285391134284331, -2.204251255101136, 1.9689005679749343, -1.19846704057742], [3.4062249230357828, 1.7692263964212938, -0.73297011323344741, -0.091144359419660148, 2.6789747994280022]], [[2.3640817963803524, 2.1283514380626665, 0.90788725706345019, -3.7794588009955312, -0.70121237513589385], [-3.6683894347023527, 3.0505181418980136, -3.0194702064661505, 2.2123276043520645, 1.3198948993595661], [1.1015952716800115, -0.071620623381310367, -2.8191823564257068, -1.2779578217649035, 0.24314681526113713], [-2.5782868222825686, -2.3133696395767132, -4.5344270793198485, -1.8071224975111644, -0.67250806721523659]], [[2.3766793386031502, -1.3202268894192093, -2.823073531845087, 0.26509295660739163, 0.6616803556513462], [4.1865545271551543, -1.3276935746200405, -2.1482123349445983, -1.1665191558262364, 3.4755485979999605], [2.6484531281953281, 4.8368454369320553, -3.1042597206781188, 2.3656328966961215, -1.0186319613649442], [3.326952136524266, 1.7597069708995425, -4.866208048284868, 2.4372771576121188, -2.2921191209210812]], [[1.967630913861365, -2.1741261456271355, -3.0070623121534421, -0.67163126000758488, -1.7713975447616881], [1.0858037947003076, 3.8723022763138548, 2.2646155450440331, 1.6438299789333355, -3.4714275417687377], [2.3020228445612965, -3.1134578908321897, -4.1355834227668344, -0.42856027208833325, -1.4232341647270452], [-0.68255979576661474, 3.4086312153791685, 0.80852541403028422, -4.7512731325708941, -3.0183874222448837]], [[-3.9588090712074999, 4.6497595596316135, -3.5622962438929506, -0.98660535870266575, 3.9721629607094364], [-1.8015511048572774, 2.1560427879949486, 4.5349996936954167, 1.5174106495459805, 1.71309495975696], [2.5365468430910445, -4.0463947718164386, -2.5936333660556787, -0.84486707456919863, 2.6920899033062629], [1.8570429391439172, -3.0649996220925524, 0.69911264652553662, 3.3163047610092065, -1.5628770127030069]]], [[[-0.90327043737951485, -1.5013454521506731, 4.6335961231287719, -1.4698919195269444, 1.3246907087308779], [1.6499452255942764, -3.1066586359209145, -0.68464351740615736, -3.6244465649308069, -1.54152118159013], [0.55154069406997408, -4.3318589609451594, -3.6470613600736934, -4.1411402923735192, 1.1845943615746046], [0.30352619329013741, 1.7238575935111244, 3.2380166156228238, -1.9729029109065097, -1.1032617873061792]], [[1.5313962088170721, -3.1796841769041837, 1.1103648319631576, 3.6187552836600041, -4.0174690419710588], [4.9691172765899871, -1.84053971803674, 3.0468892926262257, 1.5209197257539051, -1.2975993003335038], [-0.55981387946875216, 3.9975505234579494, -1.7944577666535011, 4.5274428007030068, -2.9675367435105358], [-0.18057837428074208, -4.2572276065472936, 3.5140290424208107, 2.5581246564137778, -1.5778588912483338]], [[3.928877235611294, 1.3966730096680102, 3.0867147321949382, -4.95843588042076, -3.6233612989502548], [-0.057531505033768582, -0.92082500616567486, -2.2839896524412948, 2.0063398456453649, 1.7621964855257124], [-1.9765242927007454, -3.3305279031891413, 1.4613394973498224, 2.0339650018339759, -1.5009593886490338], [3.5254972974246748, 1.3901327287486209, -4.5760533032565478, 3.7141760414067413, -4.5978817773100431]], [[-4.0804979305335385, -0.23056100893333653, 4.7260589133883393, -2.7670400627588707, -0.36753828370852926], [-2.3578911812294012, 0.84322959912455175, 2.9006601454315506, -2.0388241844535404, 0.30237789202039522], [-4.7805976873466358, -2.5667037833688457, 2.1154308561185173, 4.0469120970217016, 1.6649387456665687], [-4.2403748623192179, -2.5085014837257766, -0.15576808164075828, 4.9890994495080871, 1.978543472698199]], [[1.2412221246566641, -4.3819968856114153, 1.2572219815719032, 4.5573847984629818, 2.2202798334022775], [-1.006653939260953, -1.2535044121046814, 4.551097013706805, -2.4745612194905195, -4.2882126698002061], [3.5274895648558928, -0.1571857806317567, -0.94132655906962182, -2.8902914415742775, 2.1473592542402899], [2.5184944464110561, 2.3138245290456894, 1.5539291426954591, -0.34360418850970476, -0.70360500065001474]]], [[[-2.1894183321587999, -0.30836777651704317, 1.5444090158054733, -2.6179033193511634, 2.2808635902180256], [2.682656906324774, -3.7326569313999358, -3.7120591388557056, -0.11627913547813762, 0.42734298102562995], [-2.8484779249672201, 0.10141016423127169, -0.79373641904727599, -1.1673431270708488, 0.47463602421702866], [-1.4269863333944421, 3.0199571326389005, 3.9802075528465704, 3.2966247782975469, 4.0345742100274968]], [[-3.424441374146503, 2.594573917497951, -4.3444653622452556, 0.64092203226825095, -3.0611725210986229], [-0.97082613892232139, -4.9196960342023788, 4.8218492179655001, 4.5919108951005878, 1.4354351783076069], [4.8853518177047466, 1.0183174815782561, 2.7876929588066668, 4.5689933855644167, 0.16725259942084669], [1.4509270232431248, -1.2098975617189356, -0.9394843899366947, 3.3357273442196451, -2.7417192395742607]], [[-3.7041394409552364, -3.9855043538545045, 0.24359152688836794, 4.8612840740529819, 4.9553154548396172], [3.1221561872819485, -3.693426907541645, 2.6020557354796789, 4.900729449226743, 2.7426719257971595], [3.5398864379033981, 2.4559591859461092, 1.8294064996809478, -0.60915338465294422, 3.7432071198163399], [1.3501595430697098, -1.6497570629314517, 0.8102052753193556, -1.4731564587970802, -1.9958099748539295]], [[-0.26836499702799355, -1.6518628997135174, 2.3771017533302965, -0.4674545246774553, 4.7533416071307411], [0.46560304966106791, 4.1861525003179363, -1.7009994652036253, -1.5368575322029319, 2.6220501572076973], [-1.2372990222834748, 0.1420561802557696, 3.9574157199399487, -1.2006076447248626, 2.4928662187764772], [3.8012182109167689, 1.0909679915372692, 3.6955319287227475, -3.7852471188115313, -4.9905130599215521]], [[1.9152459795411616, -3.7514076710622324, -4.324304552415942, -2.2667214395397908, 4.3710524132699788], [3.2233090179834925, 4.4467825983149876, 2.2070790271481444, -2.0664852522331922, -3.944443196020587], [2.2308481762923051, 1.4327499908491319, -3.1789644689682737, 0.19600910863770515, 0.15818402443436952], [-2.1623195702890174, 3.436576779711185, -4.9798572065068303, -0.44284672337010367, -0.95284125445133849]]], [[[-1.6285395812228112, 3.6115747769991593, -3.7488648495820409, -1.4069905051410956, -3.6230557205232161], [3.1939305960552495, 0.938953017114593, 3.5694701620277112, -3.9889503417366647, -4.3865152020359455], [4.4935546214986548, 4.5639952592926196, 0.40871926344489307, 0.87813572922623262, -3.7569700040106255], [0.30156704614739205, 2.348712226189777, -3.3318719728863666, -3.3584090404583677, -0.088497029879094136]], [[-0.30232575987598409, -1.7588572290669182, -1.0779859745902742, -2.9008011614080678, -2.8936854698717109], [-4.4801264966821508, 4.7506419457093116, -4.1230563754210294, 4.32490172947894, 0.67667653507186731], [0.080917455605964861, -3.6117547725968535, 0.85834507614097699, 0.3494539629989788, -2.1722217359992371], [2.3751974512994591, -0.65583074508105987, -1.7025772643067407, 1.8193687985681812, -4.2828204584524299]], [[-0.487039757538545, 1.352653617903302, -4.3182694378339281, 1.91574861200698, -1.6424832782414125], [-3.0954356339332478, -2.6238488117568548, 3.345635712769008, 4.6460562264327656, 2.5694816818691972], [2.4497489761289994, 3.5858676815818864, 1.1630167196283043, -1.1863446327238227, -1.2080210358639141], [-4.2289223633953945, -3.6654608145343861, -1.0828392624165142, 2.4275575373137315, -0.16993025080320123]], [[2.7779163062423038, -4.7369066149037344, -2.9242147628124426, 3.8199282745106071, 2.73398924503209], [1.7662107447928079, 1.9607172107701833, -3.7756337578185328, -2.8489167013296059, 2.3761881396091997], [-3.4016662377521856, 4.4484655499159746, 4.0375551755472312, 2.6004361328006063, -0.69360245600543546], [4.0710986701799996, 1.0579861202784189, -2.1421101687450497, -2.1624550735419468, -0.60669407180902546]], [[-4.7886819145847799, 4.7488552424474744, 3.7930006132533087, 2.3207480516269241, -2.824933151668283], [-1.1591318773564785, -4.4861924544424445, 3.986623240468635, 4.7350257445946689, -2.6210129037890875], [-0.42931632362359995, 4.6126545846321463, -1.2946628403744045, 3.6166858262780721, 1.4302350808118387], [0.34891091799987883, -1.7234727726490728, 2.0511290509126967, 4.5440764442440624, -2.3804440610961244]]]]),self.functionspace)
      arg1=Data(numpy.array([[-2.1601580810410015, 0.92702618701266104, -0.67860700260013207, 2.2637266256117599, -2.6040266563710079], [1.5203976832823196, -1.23665374960337, -0.67554290567769115, 4.5975555713493304, 1.2891035304160159], [3.05172740034795, -2.3394161794062382, 2.1032838425720808, 3.1004854919741049, -4.7184449599736125], [-3.7350743813339884, 2.0014961981922799, -1.7294202322573948, -1.2854223996244318, -1.7109488285559604]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-1.7429585489933312, 7.8072413395492024, 5.6816312838738092, -1.2977610329585922, -9.692187827925002], [-2.3536742446916601, 3.8051029242123811, 4.7287749177219061, 5.1296949463596686, 10.137745150751606], [-4.8859071357271988, 3.0858703099555775, 4.7615311613514892, -4.253136472677272, 2.5888782625646076], [-7.3579842933390101, -3.8218086974205079, 1.5833313132627693, 0.19688622454168439, -5.7870090618895951]], [[2.1915657334845204, 1.9730375182501476, 0.84163526215291384, -3.5036572812583309, -0.65004223440831932], [-3.4006930701296532, 2.8279102014966631, -2.7991279522986479, 2.0508856234853492, 1.2235771358107586], [1.0212076643366974, -0.066394193404645988, -2.6134558703706916, -1.1847003666737244, 0.22540346503580386], [-2.3901394018855999, -2.1445542361276542, -4.2035326456288367, -1.6752498783325718, -0.62343258928579515]], [[-1.6128312421111481, 0.89591521218086567, 1.915757467565163, -0.1798939366937489, -0.44902092282794942], [-2.8410252188947727, 0.90098215704436047, 1.4577919335653848, 0.79160806781087867, -2.3585316164798447], [-1.7972588388515749, -3.2823171839965881, 2.1065723843417015, -1.6053350492792229, 0.69125078205455825], [-2.2576930171608374, -1.1941494729766964, 3.3022428576752332, -1.6539533464329297, 1.5554480862507045]], [[4.4541784890847715, -4.9216272432948172, -6.8071670208954078, -1.5203895658723448, -4.0099597866203327], [2.4579629602533712, 8.7658337653086988, 5.126470506090465, 3.72118169129021, -7.8583629551838712], [5.2111504059999287, -7.0480175251978601, -9.3618303065558983, -0.9701432986057803, -3.2218130731729255], [-1.5451287832490108, 7.7162092391451971, 1.8302805072241264, -10.755583495754525, -6.8328039741473887]], [[10.308844348907682, -12.10809783899664, 9.2763143769875605, 2.5691466533802214, -10.343618233136958], [4.6912870998629916, -5.6143928922153119, -11.80926008901722, -3.9513777800789787, -4.4609449401019434], [-6.6052355945428083, 10.536919848010289, 6.753890422062252, 2.2000563832683855, -7.0102738695567579], [-4.8357893155563234, 7.9813407176960718, -1.8205079673585796, -8.6357459983180593, 4.0697734017081206]]], [[[-1.373330280369222, -2.2826421472563299, 7.0449088108709228, -2.2348202691241679, 2.0140566846200407], [2.5085728985362619, -4.72335659280317, -1.0409304177385801, -5.5106001605013599, -2.3437252332202574], [0.83856119349991121, -6.586148328526777, -5.5449836426445094, -6.2961801066717662, 1.8010545229673274], [0.46148052109382648, 2.6209490914829483, 4.9230729608225987, -2.9995970150832019, -1.6773966654742261]], [[-1.8938068637620176, 3.9321683599230641, -1.3731368328749547, -4.4751472904351512, 4.9682181546689685], [-6.1450775123138941, 2.2761103436040639, -3.7679470683525818, -1.880851081699296, 1.604681040240137], [0.69229593312504145, -4.9435858440631879, 2.2191229256369414, -5.5988791156041566, 3.6698154409480783], [0.22331292365156047, 5.264716482551691, -4.3456371915248351, -3.1635144483069308, 1.9512651142072681]], [[-2.6541251437957882, -0.9435125432327337, -2.0852082391851048, 3.3496361822759608, 2.4477360202129486], [0.038865000078522741, 0.62205680028583787, 1.5429330063479723, -1.3553686491042003, -1.1904393342070552], [1.335226963833605, 2.2499144971610208, -0.98719753022127565, -1.3740306273856544, 1.0139624667121792], [-2.3816246882611121, -0.93909430285650108, 3.091320345017925, -2.5090852752103747, 3.1060664158065334]], [[-18.760315994403882, -1.0600170511573841, 21.72831848777372, -12.721620456683846, -1.6897776839483194], [-10.840535736896687, 3.8767949413817453, 13.335946212219785, -9.3736074882361287, 1.3901991621112342], [-21.97906353184025, -11.800563279230841, 9.7258109183519732, 18.605903258423126, 7.6546484060946991], [-19.495359072865369, -11.532974972241506, -0.71615241158586562, 22.937661970101782, 9.0964635660604571]], [[1.6000638229253739, -5.648847655513662, 1.6206892949609597, 5.8749408331629125, 2.8621705717503594], [-1.2976811470084841, -1.6158969630361972, 5.8668352276352289, -3.1899656042757902, -5.5279500918141347], [4.5472992515613875, -0.20262874474359496, -1.2134673905710098, -3.7258849012645969, 2.7681683957126606], [3.2466000822016219, 2.9827593691559735, 2.0031755438650491, -0.44294137247359067, -0.90701969035629715]]], [[[-6.6815079150731185, -0.94105439298143378, 4.7131153108779733, -7.9891272911257953, 6.9605739147243471], [8.1867375867639769, -11.391051433651882, -11.328192585757972, -0.354852223827404, 1.304134284542289], [-8.692778132908737, 0.30947617685835743, -2.4222671786606345, -3.5624130064899679, 1.4484597602953195], [-4.3547732935418741, 9.2160859295503617, 12.146508448093741, 10.06040016459661, 12.312420665478097]], [[8.0111935561064609, -6.069788201260133, 10.163512559306534, -1.4993833720262735, 7.1613565238119028], [2.2711663767853669, 11.509216500173752, -11.280312075165808, -10.742390642390097, -3.3580802806216941], [-11.42887108443016, -2.3822683921763863, -6.5215740110491645, -10.688777049789481, -0.39127343713287921], [-3.3943221533126975, 2.8304539313094357, 2.1978449821175028, -7.8036545191552396, 6.4140223484493939]], [[-7.7908566367951284, -8.3826469119628602, 0.51234212269176682, 10.224660247108616, 10.422434931011889], [6.5667806626965737, -7.7683251383533083, 5.4728617859064208, 10.307625067375781, 5.7686175470052188], [7.4453859493822545, 5.1655792738169319, 3.8477611322752838, -1.2812224715886333, 7.8730270545104828], [2.8397687518330241, -3.4699073746328941, 1.7040916647458648, -3.0984661773686022, -4.1977548729544605]], [[-0.83206177983896779, -5.1215769552920367, 7.3701694991467921, -1.4493359719201013, 14.737666691305739], [1.4435955004930399, 12.979105094426886, -5.2739241637195517, -4.765004481826316, 8.1296284716508858], [-3.8362276678236582, 0.44044312592827195, 12.269910025384068, -3.7224665840226372, 7.7290955447488123], [11.785621914775206, 3.3825304299694312, 11.457943130131961, -11.736103775411934, -15.47301333979407]], [[-9.036982739275718, 17.700810618329939, 20.40399302073795, 10.695400352060657, -20.624570229174228], [-15.209006190341704, -20.981898939117716, -10.413980911910626, 9.750596923259506, 18.611638118165548], [-10.526134333892752, -6.760351973224326, 14.999768876538543, -0.92485819076050013, -0.74638261284069363], [10.202785878282523, -16.215298385790589, 23.497182137430425, 2.0895478899264943, 4.4959290147208524]]], [[[6.0827164688137039, -13.489500425841571, 14.002289058757377, 5.2552141905326737, 13.532382603871818], [-11.929568345084759, -3.5070593595009703, -13.332236557125785, 14.899026229834075, 16.383960554456543], [-16.783760747884571, -17.046861769513637, -1.5265968500507174, -3.2799022655669416, 14.032562413420338], [-1.1263753483196888, -8.7726148651673554, 12.444789647812602, 12.543907569056511, 0.33054298912555302]], [[-0.60510385900737429, -3.5203460571404448, -2.1575848298470333, -5.8059424962699975, -5.7917004667124701], [-8.966956150529823, 9.5083917933099631, -8.2522816603376317, 8.6562743691073152, 1.3543655123522673], [0.16195597976273124, -7.228913446155425, 1.7179744066332285, 0.69943077838568168, -4.3476935462331072], [4.7539486687318604, -1.3126427429373515, -3.4077019216385538, 3.6414597334438707, -8.5720488651326558]], [[0.84229641060089577, -2.3393065340381338, 7.4681025341287617, -3.3131344095238933, 2.840543812535151], [5.3533090129746537, 4.5377372214368297, -5.7860100914256121, -8.0349836381982698, -4.4437136070393484], [-4.236645443269329, -6.2014721187256319, -2.0113446453788155, 2.0516884102625474, 2.0891760204155889], [7.3135838959017541, 6.3391220932024375, 1.872684128705794, -4.1982671199993025, 0.29388081381162962]], [[-3.5707958443258203, 6.0889258677264024, 3.7588511574315588, -4.9102213690146401, -3.5143310158965377], [-2.270326853814026, -2.5203498220531317, 4.8532842050781095, 3.6620613425532227, -3.0544054603755719], [4.3725779780528278, -5.7181572618196101, -5.189963862367966, -3.3426588538946329, 0.89157213338390628], [-5.2330814217306081, -1.3599590574976279, 2.7535163933681583, 2.7796681897123166, 0.77985814962267486]], [[8.1931897120859425, -8.1250483140473371, -6.4896299559577884, -3.9706811603046135, 4.8333160665957458], [1.983215327704938, 7.6756457246048893, -6.8209083631737784, -8.1013867508965625, 4.4844189573679953], [0.73453826098374997, -7.892015958109651, 2.2151018701135197, -6.1879643777254136, -2.4470590360746547], [-0.5969687264222775, 2.9487737214120244, -3.5093768468761772, -7.774682269148113, 4.072817977775407]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_constData_rank4(self):
      arg0=Data(numpy.array([[-3.8721883118668634, -1.680875872622658, 1.49005149102641, 1.7528317864578833, 4.1403865863106795], [-3.4260914068588963, -1.4918186358633898, -1.3548306832398325, -1.0207399079730619, -3.1316909016187577], [4.8660793264315689, 1.7679343785041448, -1.8055216781876204, -3.7021983632202349, -0.81652127158915278], [0.94194756549006087, 2.9784216054163153, 1.0293537186509907, 3.4649830377133526, 2.7818196008266396]]),self.functionspace)
      arg1=Data(numpy.array([[[[0.89372288977299519, -2.0238679510034641, 3.9409882210588094, 4.1501097950252799, 4.7097852052438576], [3.2950693762411696, 0.91531418234116479, -3.1072964789328807, 0.045643283915376642, -3.3485145581398843], [-0.24643364615476759, -4.9274661474817094, 2.9806709497650292, 4.1872381720171497, -2.1165264078554227], [-4.9793733698950051, 1.6094778017749833, -1.1966726096736666, -2.4489719727271799, -4.1316484736163712]], [[1.8270088761467012, 3.7605780144342056, -1.1064233754683848, -0.10504213875334489, -3.1627413732466705], [2.871882940843923, 2.0099118784815042, 1.9947643400523347, 1.2042155229877309, -4.3272718325630235], [-2.2604539602459006, 0.85362527725276571, 1.5924585367167632, 3.763481102370708, -4.8380963564458579], [0.97124324713439325, -4.4518080036703047, -2.2637236889953005, -2.6901672466645787, -4.6055554492180146]], [[0.17860115381824659, 2.6091346550256951, 4.7943236377956637, 3.8978426142849969, 3.0966503805526067], [1.7084495868989702, -0.36292084634446464, 3.1882455824450258, 0.51074306767148503, -3.9401868597975431], [2.8969559499540782, -3.417457595427138, -3.9607221106019432, 1.7606240713865429, 2.7675585740166353], [-4.5901500700642615, 2.436517100591554, 0.14108573175857053, -3.5472752675779762, -2.1679988139400086]], [[2.0682034409854024, -0.56150655135991023, 3.1607513230457798, 4.1601919979435831, 1.1739168422688557], [-4.9910397962513002, 1.2587316693893245, 1.5870596170994897, 4.4707964998369221, 1.6038924900749008], [-2.8782294117277942, 2.4889277040272351, 1.361551558652641, -2.208766732120294, 1.3483826229731442], [2.3885380531551341, 2.0987069327272332, -1.3219934570331091, -4.1404226541025482, -1.3053908522492952]], [[-0.028890259201972412, -3.1941347189361844, 0.5185287652022561, 0.14133112356754385, -3.9567781837749849], [-4.2571533299185145, 1.4719678399083609, -1.7105814077058512, 4.0796893409399484, 1.4254235926056049], [3.4676993762207022, 1.8647909188280778, 3.0936292196452939, -1.7217041239702837, -3.9377090701757513], [3.1593334255833234, -1.7133523427015431, 0.39902784639838895, 3.8972020261236668, -0.44898219306828135]]], [[[0.94740382255503963, -2.6517809758294497, -4.6798940060143233, 1.0087211776812266, 0.3248003017728065], [1.54442418255673, -3.2064684148745028, -3.4534987935038908, 3.7856732458828741, -3.0605903874119287], [1.8637942895511506, -4.714577379184024, -3.0179646806459881, -0.91057202432434625, 2.6917872957426257], [4.187431709734712, -0.9405392882569652, -1.5880266688461941, -0.13435185864371313, 1.4259312269735682]], [[-2.6407759083169067, 2.5392504529017854, -0.0013606392492491182, -2.4208298200167224, -1.2269706098497601], [4.4845475851826109, 4.4797561514019932, -1.1136015653000086, -3.9684917731091418, -2.3136802229754991], [-0.72989761134091857, 0.2473435181595427, -4.3240211695806501, 1.9626698575764037, -2.831546436254655], [0.15347255598248744, -1.2216754318533365, -2.3374583249296266, 1.6299190154559353, 0.91195965056815176]], [[1.1966869115942744, 2.4279643892980962, -3.5138953447908894, 3.8094221379985367, -3.906946563296799], [-2.6912334982752761, 2.3802311558120257, 1.8479482790247106, -2.8049305629879742, -0.51472621790265194], [-2.961731329399635, 3.2131269589856508, -2.4451951278276516, -4.0998421588719101, -4.1306013601273985], [1.3612767511633406, 3.1993143145443543, 0.59982565214396466, -4.4602011451279298, -3.9593853856598771]], [[-1.6678320244461267, 0.63778651267729281, -4.0012354702065531, 4.355831430411591, 4.9517485981852598], [-0.17706927646240089, 1.678005453265623, 3.1614079166130455, 1.0198842777360495, 3.4010537519229942], [1.8819143720966238, -2.7971604944307824, 1.8606794193017206, -0.24930561383978045, -1.5646939864311973], [-0.62725669872174539, -4.8847496254076468, 2.3599112666977442, 1.4909824789386708, 3.4263060946144623]], [[-2.4668524035203179, 4.1198483328742572, 3.9528111391212555, 2.0557079011609058, 1.9073052635248224], [3.7990887574648333, -1.4454540555337978, -2.0346316541285612, -0.22791653846941884, -2.95630345369988], [-1.7605118156477086, 4.8271782863452763, 2.5798023019399396, -4.9305031899486131, -3.6402307314115756], [3.138926483594437, 3.610774602556841, 2.1073726829273189, 1.4738881490595448, -4.0198031249833166]]], [[[3.498153995192455, 0.24473810386506845, 2.4699927715877905, -3.3606524163828579, 3.6468065016444822], [-4.3956831395204068, 4.6884095265339312, 3.9043236232253697, -1.0832443073239473, 1.1941639398949802], [-0.91674596526074126, -2.5779761326342676, 1.3587786378989311, -1.773956818716349, 2.2240028346164689], [3.5100248844280948, -1.8399356792587662, 3.1361558887706629, 2.9580672188975452, 1.8947881845209515]], [[-4.5685162467949256, -2.6577467787606324, -2.4373002907490173, 1.1684933671481161, 4.2138396736772208], [0.28613847221489497, -1.37133010497522, 2.4059836641645447, -4.6236888356486734, 3.6548774685737904], [-1.2569321187815738, 2.8955919840285507, -3.1964534484800633, -0.8889738160660805, -0.71297248486769771], [-0.53368812915333308, 4.1535347860084038, 3.9890474070576527, -2.3197182712499851, 2.9717634889361904]], [[4.7196371130097408, 1.6896374778046255, -0.79332071456930819, 3.1630714460525127, -2.0635471449878628], [-4.0114186371925804, 1.6441653940905168, -1.5776611059525223, 0.2889463103279386, -2.8483474072967185], [-2.8315660900443218, 4.9836103366422151, 2.8085061387092578, -0.34046358207327643, -3.4453649758004765], [2.8033297005896287, 4.6458628026209485, -4.4116194455576663, -3.5447903924862012, 3.6154455149001468]], [[-3.9597292352519875, 0.75816390192001037, 1.0696497870610155, -3.3781761800607102, 4.2444115712481256], [-2.9772696257894902, -4.4316372012364464, 3.0740867453890619, -1.4867762880130195, 1.4647512855540796], [-3.295645059773106, -1.377008976413471, -0.97806822873312171, 1.0861363395399071, 0.83465775252013685], [-3.1390190919344407, -4.0000915283245639, 0.92164985346487782, -3.8106867445744932, -2.6641276112731394]], [[-1.0641143819457222, -3.1917604949700342, 1.0973584515548493, 1.2982910911230316, -2.1322543533651572], [0.12311545986338857, 3.5868990079663003, 0.069073826336410349, 2.7624039028251239, -4.1858063149280644], [-2.0466114022017114, 3.0514620671676607, -4.1608136884725582, 2.892418658480552, -2.970276280989721], [0.52126099300154571, -4.859530938340562, 3.5716397132770705, 2.4937681272660983, 0.77324223747675891]]], [[[0.87671834459066478, 3.3258953105314557, 3.4605711113666899, 1.5530503802489406, -2.6877866268833572], [1.6055760480179924, -2.2984924194581713, 2.1188837461677368, -4.2052674426603289, 1.5526273747096608], [1.8415403249764744, 0.69190615951883938, 2.6375847043059082, -3.0246255738012806, 0.75273528545599522], [2.0361329220720705, 1.1766301202807563, -0.75294820544459462, 2.6459383930762748, 4.7460198510638669]], [[-0.43413921324141214, 2.5980541164627731, 1.8685604908513822, 2.9352026412963053, -2.9706615784322334], [1.8967730236532789, -4.6954615476910018, -1.8342368636802044, 4.7665412701744163, 2.4186264593915299], [1.7606940688151003, 0.60571833469779701, -4.9957934325072486, -1.4351430939221501, 0.25624586939817107], [-0.25659152544433184, -3.1915461309239426, -0.39941464988007791, -2.8891458767428513, -1.3966840082053968]], [[-0.84195365997484206, -1.3734417944202049, -4.1155407561419866, -4.3601422752093324, -2.6770313996793762], [2.3674879913314033, 3.0879370830168416, -2.5157331790683459, -3.8688093994975703, 0.40977019140584758], [-1.1797385820957498, 4.5543807067945519, -0.65158882325653344, 1.5158334495168386, -4.5562316915759773], [2.2895069070863103, -2.4765030785073217, -2.3002584659576044, -2.6456670557661188, -3.4060170923554054]], [[3.0947943272801997, 3.1442146322486746, -3.4263568016023473, -4.4725458925499648, -3.3562858254929608], [-2.72415472482191, -0.97118462706982545, 1.7136303791475456, 1.3185283160286989, 0.27945473626609818], [-1.0354967336779075, 1.078338194571379, 4.2311845247172055, -0.78565715800359204, -3.2636017573769269], [3.0477267669624393, -1.4123991644642842, 1.0750795285266692, 0.19086422667162495, -0.013959841285608121]], [[-1.9039596111638946, -4.5244395637181398, 1.9204684653936885, 4.8372709200848583, 3.5224256170756867], [-0.54437390073939529, 3.0806705242172825, -1.8721378899587715, -0.64378189736891933, 4.7335848851440794], [2.9819642093100107, 4.0305230226095698, -4.0861624593493042, 2.8425914591227084, -3.9664271300860374], [4.5294477673830151, 1.0345950241425426, 4.0188472124306802, 4.0674885839388875, -3.9138862972644994]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-3.460663327826869, 7.8367978246375509, -15.260248526788905, -16.070006641261074, -18.237175223148743], [-12.759129125471492, -3.5442688785474332, 12.032037107228961, -0.17673939049234225, 12.966078934145294], [0.95423748429122546, 19.080076823398318, -11.541719213201249, -16.213774708687577, 8.1955888182553256], [19.281071363328554, -6.2322011322422632, 4.6337416923095889, 9.4828806488837216, 15.998520928279879]], [[-3.070975138982428, -6.3210648515776784, 1.8597603567305281, 0.17656279663917893, 5.3161756656357806], [-4.8272787442611547, -3.3784123826372441, -3.3529512507620285, -2.0241368180277526, 7.2736068176348203], [3.7995425229516715, -1.4348381327950011, -2.6767251325191905, -6.3259445820462465, 8.1322394349734335], [-1.6325393405558872, 7.4829366629178562, 3.8050385311165584, 4.5218372182382174, 7.7413670346163679]], [[0.26612491554591555, 3.8877449830097146, 7.1437890849605905, 5.8079861992016397, 4.6141685167299116], [2.5456778543022649, -0.54077074822013615, 4.7506500838805756, 0.76103346951529893, -5.8710813053639974], [4.3166135326669046, -5.0921877855857369, -5.901679886443695, 2.6234205227065068, 4.1238047797164121], [-6.8395599559342335, 3.6305359386477902, 0.21022500496941016, -5.2856228015356708, -3.230429865254798]], [[3.6252127322207843, -0.98422653152799666, 5.5402653881234523, 7.2921167717632418, 2.0576787557871157], [-8.7484532023455568, 2.2063448807268031, 2.7818485438556628, 7.8365542156988042, 2.811353738664371], [-5.0450520015944518, 4.3626715938145759, 2.3865708509076242, -3.8715965369311558, 2.3634879218547828], [4.1867054227345486, 3.6786802221438206, -2.3172321529769775, -7.2574644374812598, -2.2881305795739109]], [[-0.11961684167488525, -13.224952545152609, 2.1469095440596608, 0.58516548824727566, -16.38259131710868], [-17.626260543062461, 6.0945158998372833, -7.0824683152577457, 16.891491023542418, 5.901804722635025], [14.357615982662105, 7.7209553065897403, 12.80882092403815, -7.1285206604823417, -16.303637814949578], [13.080861736968162, -7.0939410573454476, 1.6521295427923277, 16.135922993105233, -1.8589598496722639]]], [[[-3.2458920952810919, 9.0852440141611766, 16.033744639016128, -3.4559709587702363, -1.1127955228489887], [-5.2913384204026883, 10.985653882566, 11.832002540021247, -12.970062576894941, 10.48586242622695], [-6.3855295995838786, 16.152573045793723, 10.339822858564872, 3.1197029878637528, -9.2223093230357556], [-14.346523797530553, 3.2223735733103709, 5.4407245239967041, 0.46030174839474669, -4.885370723505905]], [[3.9395587131662317, -3.7881011467634362, 0.0020298269887170065, 3.6114390397547624, 1.8304176214305405], [-6.6901316609915815, -6.6829837107851509, 1.6612915680411946, 5.920269983394765, 3.4515912740634125], [1.0888748588705559, -0.36899166985042059, 6.4506553626482246, -2.927947469579824, 4.2241537419172621], [-0.22895321910826213, 1.822518176215262, 3.4870638896840398, -2.431543562205273, -1.3604784018730338]], [[-1.6213081460594359, -3.2894806524347224, 4.7607332308163075, -5.1611219979735008, 5.2932510817329179], [3.6461657192262167, -3.224810203097543, -2.503657029462921, 3.8002059910932848, 0.69736687348250481], [4.012644480583325, -4.3532429931788545, 3.3128253856894467, 5.5545919532798997, 5.5962654629327844], [-1.8442995108571281, -4.3345291986731036, -0.81266219811898577, 6.0428173648407562, 5.3642968072633792]], [[1.702422707147665, -0.65101414625667997, 4.0842207256371879, -4.4461709734244979, -5.0544474084173601], [0.18074167696108775, -1.7128071319446481, -3.2269752258689093, -1.041036583799468, -3.4715912937493139], [-1.9209451029870903, 2.8551733456711612, -1.8992697392254085, 0.25447618932798521, 1.5971455957157836], [0.64026594492872102, 4.9860588831100499, -2.4088556091936475, -1.5219053183413065, -3.4973673677043076]], [[7.7254192277409439, -12.902091540111519, -12.378982680203313, -6.4378417304514013, -5.9730905403902534], [-11.89757169619473, 4.5267153144231296, 6.371837439379938, 0.71376414985312053, 9.258228628376024], [5.5133788352562485, -15.117230319839129, -8.0791433969604363, 15.440811980364334, 11.400077461354627], [-9.8301475095228597, -11.307829970623345, -6.5996398574433961, -4.6157621064134879, 12.588780872808902]]], [[[17.022294836680004, 1.1909150276078717, 12.01918076225876, -16.353201246682922, 17.745649725148446], [-21.389742850764065, 22.814172670911582, 18.998748466675369, -5.2711527293437452, 5.8108964602930335], [-4.4609585891448464, -12.544636363045617, 6.6119246390768351, -8.6322146015379406, 10.822174215452407], [17.080059525375908, -8.9532729708049086, 15.260783334813546, 14.394189740072271, 9.2201896126642069]], [[-8.0768369314634754, -4.6987218995295716, -4.3089869747533358, 2.06581959483522, 7.4497920245986453], [0.50587404204136588, -2.4244216368633893, 4.2536212339958697, -8.1743784480490902, 6.4615835259118057], [-2.2221735042399997, 5.119216614685099, -5.6511199408560309, -1.5716473710132439, -1.2604885669251289], [-0.94352559092973765, 7.3431769404971137, 7.0523740484200417, -4.1011096801870517, 5.2538828368737125]], [[-8.5214071207179227, -3.0506770944545059, 1.4323577479101797, -5.7109940655040763, 3.725779104237759], [7.2427033097370455, -2.9685762615563203, 2.8485013276307352, -0.52169882712942073, 5.1427529908837286], [5.1124539587959825, -8.9980164984474236, -5.0708187167625729, 0.6147143780667107, 6.2206811530761268], [-5.0614725455217862, -8.3882060040176167, 7.965274544868417, 6.4001958982650393, -6.5277652534584183]], [[14.65970309354522, -2.8068731567409291, -3.9600556908761644, 12.506678324490347, -15.713653571907836], [11.022442735463171, 16.406799992803474, -11.380878917176403, 5.504340739956457, -5.4227998119030483], [12.201131746046846, 5.0979603786175236, 3.6210025955334775, -4.0210921784786615, -3.0900685652291306], [11.621271344276755, 14.809132308894329, -3.41213057895984, 14.107918228508733, 9.8631288818652507]], [[0.86887202826262644, 2.606140337960956, -0.89601651825266926, -1.0600822926166464, 1.7410310359612249], [-0.10052639183993733, -2.928779339046514, -0.05640024851373409, -2.2555615473776087, 3.4177998948909689], [1.6711017445746004, -2.4915836872898032, 3.3973928837571661, -2.3617213609907317, 2.4252937659248266], [-0.42562068883544657, 3.9679103811006646, -2.9163198003433108, -2.0362147223238147, -0.6313687349909648]]], [[[0.82582271030765297, 3.1328189908299149, 3.2596765335570881, 1.462892024758903, -2.5317540697495211], [1.512368449639701, -2.1650593388059844, 1.9958773862591597, -3.961141429848511, 1.4624935757209896], [1.7346344258633655, 0.65173932250634847, 2.4844664909947722, -2.8490386957610947, 0.70903716959374075], [1.9179304489599507, 1.108323877280736, -0.70923772905864613, 2.4923352277948809, 4.470501844477111]], [[-1.2930496124766628, 7.7381005125135189, 5.5653609369790722, 8.7422709631119506, -8.8478826275826972], [5.649389754219758, -13.98506412104441, -5.4631307042361819, 14.196769502196007, 7.2036893020832986], [5.2440892550872551, 1.8040845748607095, -14.879579095576524, -4.2744611978017479, 0.7632082337142001], [-0.76423774315012816, -9.5057699510267177, -1.1896252227226172, -8.6050945004903703, -4.1599138259784123]], [[-0.86666813082691541, -1.4137574184371273, -4.2363471815944642, -4.4881286648341172, -2.7556122262054323], [2.4369825677385442, 3.1785795193636788, -2.5895793030076804, -3.9823733421247303, 0.42179847031593742], [-1.2143682965163072, 4.6880687166912995, -0.67071537825053584, 1.5603287981157166, -4.689974034759226], [2.3567124486864217, -2.5491976531121376, -2.3677796057918834, -2.7233272221652722, -3.5059963598048713]], [[10.723409849237397, 10.894650367671785, -11.872268198705909, -15.497295653080155, -11.629473455050867], [-9.4391499136146031, -3.3651382592849131, 5.9377001966565466, 4.5686782497841927, 0.96830592097068868], [-3.5879786178015305, 3.7364235531082692, 14.660982607580351, -2.7222887259405257, -11.308324731162541], [10.560321551109809, -4.8939391473492559, 3.725132330537777, 0.66134130792345691, -0.048370613263802702]], [[-5.2964721655179892, -12.586174661106652, 5.3423968198016194, 13.456415060000772, 9.798752624035016], [-1.5143499872553052, 8.5698696479565157, -5.207949877737537, -1.7908851007582238, 13.167979215670517], [8.2952864864220999, 11.212187945878334, -11.366966821579881, 7.9075766381299477, -11.033884735723895], [12.600106580226534, 2.8780567170774352, 11.179707948267168, 11.31501946893979, -10.887725617137184]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[3.4865093128580469, 4.7138242299532394], [0.73217571062152587, -2.5346911390045515]], [[-1.8471575069723292, -3.4291234331786482], [-0.36139078408825664, -2.8943328798723766]], [[2.6450217791764272, 4.4277866941390869], [-2.6651445101225679, -3.6462153725803881]], [[-2.2325330969839694, 2.1895763402889035], [2.2377440000621318, -0.69685545100166202]], [[3.3410244266055287, 0.031697060310148295], [0.69745127174734467, 2.1878856212740931]], [[-0.97735604622835126, -4.5133670451676036], [2.0125454680166763, -4.1450337187794961]]]),self.functionspace)
      arg1=Data(numpy.array([[[-4.4577923456917095, -4.6767931774252194], [-4.7190208243622749, -3.6115364418280227]], [[1.5548254606532801, -1.8606449942303604], [3.4887935212148342, 0.017426298149919361]], [[-4.7254727237561163, -2.3646689384686201], [-0.53649812349398562, -1.1877398104802972]], [[2.4569195574579359, -1.9948937277929488], [-1.9002334941543753, -2.1158563155264307]], [[3.0373702139537535, -0.94773671202151188], [-3.8693403254079861, 4.6093576447221878]], [[-4.8125905520357755, 2.9803272724520262], [-1.7236926621517901, -4.0578778029083971]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[-15.542134528041464, -22.045580998226999], [-3.4551524255152275, 9.1541294172935164]], [[-2.8720075216774159, 6.3803813505418798], [-1.2608178261538587, -0.050437507709770776]], [[-12.49897827123908, -10.470249661795355], [1.4298450285210553, 4.3307551555989763]], [[-5.4851542286520489, -4.3679721077661728], [-4.2522361002610536, 1.4744460070108858]], [[10.147928077463551, -0.03004046771908751], [-2.6986763307790844, 10.084747314197495]], [[4.7036144740536034, -13.451310895299224], [-3.4690098554671849, 16.820040319742166]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[1.5913818394780979, -3.3813249612331586], [2.9836070665437218, -1.4425708301028486]], [[-4.2294750223071285, -2.6045037156297846], [-0.97125137605510048, 0.58027199641245364]]], [[[-2.0928937712873563, -1.8516520407028181], [4.5028667458567053, 3.5849304663789709]], [[-4.2870947272335531, 1.2954520370345426], [-0.79846268542092602, -0.65212061636346341]]], [[[0.91684943877961445, 1.2312794054195288], [-2.633568791319604, 2.1561481698440508]], [[-4.4865342364842187, 0.95410326067084128], [-3.060507048801834, -2.7683458006870021]]], [[[-3.5027716927781052, 0.95454990324609046], [3.2147807072300534, 1.8264244024236573]], [[-3.551565808264904, 4.9593909223060866], [2.5156209075381408, -0.62353674924528679]]], [[[2.3220577590803293, -1.8661048583108188], [3.0191405409167267, 0.44288074890867524]], [[-2.2537749224499546, -1.9565048209475702], [4.9329989043516207, 3.8586475690547299]]], [[[-0.6840252088797536, -1.6662144607527463], [1.2471876311567023, -4.5506575186194107]], [[3.8576875707866378, 4.2526751746233042], [0.13946267755447739, -0.11806658507979861]]]]),self.functionspace)
      arg1=Data(numpy.array([[[-4.2591115479954524, 4.3364468873985373], [3.1093368750596557, 4.6952341856432351]], [[-4.354784989494509, -3.7754490983107578], [-2.7204489611847658, 0.14088851083015363]], [[2.6737628996497689, -3.6243369295746697], [4.7884625224224813, 2.2875825048935985]], [[4.936577168215031, 4.1184421833201519], [-1.1006545734999227, 2.6334563203643597]], [[-0.64078491953032302, 4.7112328083745929], [3.8020640349171746, 4.1074341179487703]], [[2.1223458192999223, -1.0083612415566314], [-4.1276344864325942, 3.0171194250933233]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[-6.7778727697914123, 14.401440189913421], [12.938253576933803, -6.2556317860514223]], [[-13.150862649003315, -8.0982794442375763], [-4.5602526637069412, 2.7245129145272009]]], [[[9.1141023798087328, 8.0635465126195083], [-17.000344195458194, -13.534722496797251]], [[11.662822397203206, -3.52421114841531], [-0.11249421870239969, -0.091876302521090275]]], [[[2.4514380139736454, 3.2921491933135627], [9.5449406269549666, -7.81460743760063]], [[-21.483601046970044, 4.568687706243411], [-7.0011623809426142, -6.3328194211472466]]], [[[-17.291702764038309, 4.7122092582865172], [13.239888474780043, 7.5220233035868906]], [[3.9090471499527162, -5.4585763004101935], [6.6247777785970436, -1.6420567932794474]]], [[[-1.487939594297051, 1.1957718514678428], [14.223873969460698, 2.0865143144560609]], [[-8.5689965754452171, -7.4387566138668229], [20.261968003537749, 15.849140674275482]]], [[[-1.4517380423617012, -3.5362832948356653], [-1.2576156682072464, 4.5887066653740884]], [[-15.923124255061305, -17.553488710370907], [0.42077555352514034, -0.35622098729869395]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_constData_rank4(self):
      arg0=Data(numpy.array([[[-2.1928400425018446, -1.7527121605462348], [-3.4007870386468309, -4.2123699223214182]], [[3.8086282004369192, 0.1911305842279285], [-4.5004431155051874, 1.3703928269416057]], [[-2.6492516944422828, -4.6883980873598947], [-1.7612035935078874, 0.86271514641160074]], [[-4.2510294374522299, -1.3512442811900947], [4.2791391942037968, 3.344508658141331]], [[4.4590467637386801, -2.4682858042646716], [3.2207337199651711, 1.7595640816305824]], [[-0.75905900023424255, 0.3108166266907153], [4.4775009552052722, 3.35008727932839]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-0.76412669812902312, -4.2662932031940066], [-0.27765124168500677, 2.191680347661995]], [[0.98789779846109038, 2.3112485329334742], [-4.6993134683097315, 4.4587005531544577]]], [[[-2.2820078940724677, -0.46033629487164163], [-0.26632046357310202, 2.2211053984542781]], [[3.0736421600769326, 3.2551096212660404], [-1.7855364879715694, 2.640304665033506]]], [[[-2.073713056840166, -3.9801105684917717], [-0.01056277679390849, 4.3895932623350138]], [[-3.004202647487114, 4.6475511110647787], [4.7318391683388814, -0.882055868207968]]], [[[1.1540620275813431, -1.450665843499185], [1.3418374407818181, 3.078328839037427]], [[-0.79881905612884196, -4.21277707021199], [2.3545957389256147, -4.2606322741774596]]], [[[-0.96141522936432278, -1.7142578680747178], [-4.5728493025585824, -4.740185182779511]], [[1.9811959499856489, 2.7683908809474298], [3.312135175978451, -3.9637046798910625]]], [[[-3.0087106672911568, 1.0539269960079123], [1.0037742521618807, -4.7911093886058955]], [[4.7360472689884308, -4.5873133310013161], [2.2249431946917122, 4.9729836537390462]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[1.6756076212020412, 9.3552985690172754], [0.48664270769207302, -3.8413847973773785]], [[-3.3596300285142151, -7.8600640538916622], [19.795246709467857, -18.781696102745705]]], [[[-8.6913196189840658, -1.7532497943327794], [-0.050901985794579739, 0.42452117243837212]], [[-13.832751698844724, -14.64943568524145], [-2.4468863953587454, 3.6182545739023757]]], [[[5.4937878296208957, 10.544314667644464], [0.049522502517770045, -20.580160655419359]], [[5.2910124983802138, -8.1852837178188622], [4.082229320909625, -0.76096295748424869]]], [[[-4.9059516518940969, 6.1668232046215055], [-1.8131501681431841, -4.1595742393718664]], [[-3.4182579321578102, -18.027059477587166], [7.8749658352594034, -14.249721530142903]]], [[[-4.2869954671060642, -7.6439559988521397], [11.287099018546954, 11.700131796240404]], [[6.3809046019772095, 8.9162498603114724], [5.8279140891568701, -6.9743923849273592]]], [[[2.2837889111081262, -0.79999277190964446], [0.31198972701595118, -1.4891564582726999]], [[21.20565617079302, -20.539699821384271], [7.4537538937649748, 16.659929278699199]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[4.954539393225792, 0.92460594510057614, -1.2614415773065843, -2.0399583939874422], [-0.89426349018870699, 3.0178335362682152, -3.7583303091941378, 1.2934797080261129], [3.5470798939938035, -1.219895593632101, -0.49382964395891538, 0.72834873773590836]], [[3.5458235776235743, -0.77970054212756423, 3.8288693544179271, 0.63213753337912415], [-1.3139171550040194, -0.98058929374539883, -3.2153267406240715, 4.8095397319089255], [4.6780860795269881, 0.7164318784393986, -4.7831925884136837, -1.7199764886355151]]], [[[-4.3686706431172793, 2.5720454179026353, 1.0732557103636928, 2.8894188201395679], [-3.1057377326566882, -4.6505182541854744, 0.18360296266450327, 4.8271976954514209], [-0.34632493343498627, -2.377560243605803, -0.24902513648249336, 0.62616997261816909]], [[-4.6384254702483174, 2.1180523285209221, 1.1872253590982895, -2.241760391888501], [3.3587436394133796, -2.3612784288120792, -3.0298621730690201, 4.2885823908000944], [0.95038920025881701, 0.56729263125552176, 0.57068457394812633, -2.778486289537879]]], [[[0.12268930539372214, -4.6722516711327167, -3.6937082808814981, -2.2804238453640502], [-2.6396856391716583, 4.0203429664324801, 2.4210231945957013, 2.5455977756001937], [-1.644256048700016, 1.6664331503409535, -0.47520634929873573, 2.3698285917582309]], [[4.1079613500804051, 1.5019039998765074, -3.089414412213376, 2.8962179746440784], [3.7295783077095024, -4.7096951058083691, 2.5775526031196598, 1.8805276193352336], [-2.0198908376676163, 1.049806155931333, -0.77093286018046658, 4.5055362787439996]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.8564501276340888, -4.5587716300362437, 4.8747671705972184, -1.4730347257103849], [1.2332175032347354, -2.3742536504118119, -4.2394914336160241, 2.6038832659500191], [0.97318862047543675, 3.0912607038881905, -2.7328018182352998, -1.1649130148852436]], [[4.9659468366222139, -0.35362652443448539, 4.4457912012946572, -2.9633685708912938], [-4.774901987671254, -4.2992075801475114, -4.9803740566599615, 3.0810097956777778], [4.7506254207125505, -3.2232277941724155, 3.9524301202379917, -4.3149470162468964]]], [[[-2.5361921258724585, 4.587218067504212, -2.5154994626562122, 1.0186166478017631], [4.8280374383005782, -3.6184410276966394, 0.63999460799959351, 0.52115325582738148], [0.63765915668057005, -0.94271462151196861, 4.3483372310391157, -4.1533663819667108]], [[-2.1931666732480029, 4.8128878232371548, 3.7826919826560363, 0.23085015781519314], [4.2109637201051111, -2.9730618020341515, -4.5420380372635289, -2.9392544016787547], [2.4816057019286788, 1.6825228216334374, 4.5577376242391168, -3.1069766662998521]]], [[[2.1487472413965625, -2.7090570553533198, 3.5059016054446186, 4.4295224913978117], [-3.6105489607900121, 3.4930378784842606, -2.6880551594923441, 3.7914484769780294], [-3.4613577682527739, -1.0640812903901198, -4.6876846343421743, -4.1429001177247669]], [[-4.405137848049562, -0.88515630115644228, -3.9409056691305366, -0.86985514321469815], [0.8962850908187896, 3.3298309314346426, -2.5407586953048735, -1.9839476644365384], [4.321197487878333, 1.7188407477886161, -1.3311092433114236, 4.0383601338750967]]]]),self.functionspace)
      res=mult(arg0,arg1)
      ref=Data(numpy.array([[[[9.1978552889221419, -4.2150673514873551, -6.1492339886805105, 3.004929553347889], [-1.1028213886044975, -7.1651022898199974, 15.933409150528011, 3.3680701665751118], [3.451977788751988, -3.7710153114412703, 1.3495385489094147, -0.84846292396379863]], [[17.60837137852025, 0.27572279281225459, 17.022353686777976, -1.8732564988964426], [6.2738256350640516, 4.2157569246817133, 16.013529882689159, 14.818239026712872], [22.223834649682424, -2.3092231432170229, -18.905234457345365, 7.4216074176526297]]], [[[11.079788085604212, 11.79853321144439, -2.6997741627125804, 2.9432101126658918], [-14.994618046809244, 16.827626050996869, 0.11750490611803277, 2.5157097955069405], [-0.22083726499160791, 2.2413608051727483, -1.082845272431423, -2.6007133136693192]], [[10.172840157693306, 10.193948260917448, 4.4909078474670334, -0.51751074025130961], [14.143547610703544, 7.0202267006684096, 13.761749237745423, -12.605234669121176], [2.3584912584137174, 0.95448279863189756, 2.601030554256246, 8.6326920692282449]]], [[[0.26362830651362085, 12.657396354068425, -12.949777791986525, -10.101188712959946], [9.5307142413235493, 14.04321026624643, -6.5078438894836124, 9.6515028092980142], [5.6913584471644114, -1.7732203369636743, 2.2276175017495237, -9.8179631517826937]], [[-18.096136021563968, -1.3294197892227551, 12.175090771385278, -2.5192901011150077], [3.3427654322411988, -15.68248844094706, -6.5489391889819872, -3.7308683782885406], [-8.7283472135177647, 1.8044495980941049, 1.0261958561587323, 18.194978089807723]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_expandedData_rank0(self):
      arg0=Data(-2.17713914518,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0807509716743)+(1.-msk_arg1)*(4.47271135282)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.175806101443)+(1.-msk_ref)*(-9.7377149713)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([-1.5170512712291782, 2.5072881008769716]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.09575094702)+(1.-msk_arg1)*(-1.73639682737)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.6964129095798715, -7.7619395127323552])+(1.-msk_ref)*numpy.array([2.63420301431962, -4.3536471036647475])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_expandedData_rank1(self):
      arg0=Data(-4.61534240776,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.0166258895690827, -1.6707935048434486])+(1.-msk_arg1)*numpy.array([-1.2608622917915313, 4.0424004183423214])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([9.307418988717771, 7.7112841175162909])+(1.-msk_ref)*numpy.array([5.8193112056526983, -18.65706207992779])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[4.1774962459786664, -4.0760802309809367, -4.4998675705099558, 3.1924034159639909, 2.2734702028564202], [3.3961672414495201, -2.0403170995273268, 3.7698815842384406, 4.2423668964350032, -3.9321343287440924], [3.2332919530856188, -4.3444546793564687, -3.2401792186490286, 4.662546816836219, -0.11475457024859814], [-3.4476657030964475, -2.3118601282139828, -1.0755252793753489, -0.76348552566559391, 4.0515845717196868]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.418354709376)+(1.-msk_arg1)*(-3.63516583387)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.7476752279049292, -1.7052473604245015, -1.8825407896903108, 1.3355570032959485, 0.9511169659905544], [1.4208025592882496, -0.853576267207244, 1.5771477145552792, 1.7748141700236095, -1.6450269143283682], [1.3526629153602689, -1.8175230747785398, -1.3555442353434433, 1.9505984185086098, -0.048008114885897887], [-1.4423471832438512, -0.96717757205647636, -0.44995106567940757, -0.31940776520246467, 1.6949994860132926]])+(1.-msk_ref)*numpy.array([[-15.185891624516131, 14.817227591788789, 16.357764849272794, -11.604915825653103, -8.2644412057529379], [-12.345631122237446, 7.4168910104694579, -13.704144732772189, -15.421707196676152, 14.29396036605133], [-11.75355243879471, 15.792813217208149, 11.778588791259631, -16.949130887398304, 0.41715189304853151], [12.532836570513396, 8.403994950777701, 3.9097127490524359, 2.7753964975564571, -14.728181808164088]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_expandedData_rank2(self):
      arg0=Data(-3.17441264094,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.0696466215321694, -1.5021619775798447, 1.7819238776610602, -2.5540083939519374, 4.3064186640706534], [-3.3640665971587236, 3.2138109995628721, 2.5877877263173668, 2.177104340681594, -3.825182606213243], [0.049554958681299333, 2.6971817826355711, -4.6445679546257503, -3.0039183733907171, -0.19258969290580552], [3.8311564631103039, -3.7667403784887088, -2.3577467998563506, -3.0464513906999944, 2.4468406142974164]])+(1.-msk_arg1)*numpy.array([[-4.0367912794397052, -1.0257521932039761, 1.3202289016896938, -3.2698187840248494, -3.5370035748000994], [-4.5355708941110535, 0.26033107221017726, -3.0682646174732531, -1.3405462119188138, -0.48068487528500903], [2.163045592643237, -4.6532262273928824, -2.9004408342147556, 4.1220994435072651, 0.45924653344174171], [-4.280445492249056, 2.868756193333124, 0.22142206828670385, -1.5497566766859938, -4.7803740355440425]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.7443250386000368, 4.7684819703637755, -5.6565616824340275, 8.1074765308192056, -13.670349844391174], [10.678935530973215, -10.201962262593462, -8.2147060704824231, -6.9110275396975878, 12.142708019054115], [-0.15730788725900735, -8.5619679457022606, 14.743775226853016, 9.5356764568331922, 0.61135915567428634], [-12.161671505903291, 11.95718827260086, 7.4844612455918078, 9.6706938046369384, -7.7672817763827871]])+(1.-msk_ref)*numpy.array([[12.814441266276019, 3.2561607285751402, -4.1909513144536037, 10.379754081580414, 11.227908858883367], [14.397773580130231, -0.82639824645256477, 9.739937987445586, 4.255446840874753, 1.5258921444117641], [-6.8663992722088842, 14.771260157373677, 9.207196048420009, -13.085244580667174, -1.4578380010637766], [13.587900279435477, -9.1066159238818205, -0.70288501255163904, 4.9195671848479092, 15.174879766836101]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[-2.8711700875767665, -1.3492269982925209], [-2.668371497743375, -1.6082098938631031]], [[4.2822805520505209, 2.9543583606472676], [-2.1520006734133954, -3.2687480471344199]], [[4.5141929659303557, 1.3657301454726483], [-4.5334900512923717, -2.8269931573092153]], [[0.93350876727818566, -2.4613743275335978], [1.3708327518990178, -4.9916641738194087]], [[0.060259208884040127, 0.97350790711938018], [4.9104924352416717, -1.6627436280991583]], [[3.5544576600475644, -4.1137477713301953], [-0.27557263842748458, -0.093166344285318559]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.724464168613)+(1.-msk_arg1)*(1.01896132301)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.0800598504439187, -0.97746661558872272], [-1.9331395386643042, -1.1650904437133514]], [[3.1023588199105432, 2.1403267735323226], [-1.5590473787198778, -2.36809083637386]], [[3.270371054023121, 0.98942255439007798], [-3.2843511009265751, -2.0480552473857458]], [[0.67629365297949562, -1.7831775058429549], [0.99311920991251912, -3.6162818356832918]], [[0.043655637665476373, 0.7052715965697971], [3.5574758195796687, -1.2045981801480583]], [[2.5750772135578326, -2.9802628590418911], [-0.19964250239096426, -0.067495678155411604]]])+(1.-msk_ref)*numpy.array([[[-2.9256112710291342, -1.3748101272233897], [-2.7189673516275734, -1.6387036811314175]], [[4.3634782568251094, 3.0103769038161188], [-2.1928054533036025, -3.3307278347003333]], [[4.5997880368949664, 1.3916261959079108], [-4.6194510205257178, -2.8805966877171101]], [[0.95120932854889662, -2.5080452412109198], [1.3968255545029327, -5.0863127305856395]], [[0.06140180320812607, 0.9919669050008143], [5.0036018684533001, -1.6942714471173623]], [[3.6218548798715009, -4.1917498716114689], [-0.28079786023792269, -0.094932901433141259]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_expandedData_rank3(self):
      arg0=Data(3.42495667189,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.2437523050863852, -4.9715537131981966], [3.9731400787333122, -1.9164152162646255]], [[-4.7915374968967539, 4.4361755158655942], [-2.8526647940015293, -0.63098449903650877]], [[-2.4250954292113001, 2.3810065541473175], [-4.1813943480693521, -4.1579549146608077]], [[3.4878400179691784, 4.9852824109169873], [-3.2996340001969293, -1.4832953099792014]], [[2.7943047573021218, 1.712995656294769], [3.3603423068101463, 2.1749730520685837]], [[-3.0392165121056891, -3.8961553458588796], [-2.6203484299831015, 3.5851252982353383]]])+(1.-msk_arg1)*numpy.array([[[-2.8179936419093146, 1.9128330866736709], [1.4220985292046144, 2.4900797873317213]], [[-3.8531022854022012, -4.0349457894364038], [2.8580026086160455, -0.21683798473432514]], [[-4.0944263374718801, -3.9452242595640907], [-2.984576254378184, 3.656167782123374]], [[1.3656060896855973, -1.3442331332094861], [3.6180689369936996, 1.8857070503384588]], [[2.827787287512753, 0.14441353233846321], [1.3401054157371384, 3.4098367734170623]], [[-1.2373388803426755, -3.0323135334335394], [1.2055802243712357, -4.5173663828739912]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-14.534667771164726, -17.027356059690021], [13.607832621021089, -6.5636390810618082]], [[-16.410808318619551, 15.193708930749953], [-9.7702533188883383, -2.1610945698358277]], [[-8.3058467702532113, 8.1548442834465895], [-14.321094470233653, -14.240815426395679]], [[11.945700940037142, 17.074376254538386], [-11.30110348377776, -5.0802221683000974]], [[9.5703727218228121, 5.8669359019496143], [11.509026803551993, 7.449188465868656]], [[-10.409184870462186, -13.344163246528941], [-8.9745798379536197, 12.278898809761655]]])+(1.-msk_ref)*numpy.array([[[-9.6515061252079075, 6.5513704424196844], [4.870625845687834, 8.5284153811663987]], [[-13.196708379872449, -13.8195145022547], [9.7885351026656497, -0.74266070253554761]], [[-14.023232802096626, -13.512222149906121], [-10.222044355204442, 12.522216238941798]], [[4.6771416880456931, -4.6039402381647685], [12.391729345123521, 6.4584649432914016]], [[9.6850489370595554, 0.49461009109418069], [4.5898029846681636, 11.678543207179111]], [[-4.2378320536216227, -10.385542467603075], [4.1290600329619025, -15.471784132407096]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[-0.32788557350298397, 2.0733457697201523, 2.0861544174632707, -3.8411009155865061], [2.8055825271039714, -1.1721030128278365, -3.8747143192108813, -2.0963856526775224], [4.6809048744460515, -2.1889110518074006, 4.3011918449133244, 0.88918557655744834]], [[2.2558263981971027, 2.8921346900126466, -3.2494961436432632, 4.2624790524765697], [-0.5376629289634689, 2.7342794397868184, -3.6401370374051156, -4.54292652438318], [-1.5172533679010645, -1.1909929148417553, 0.5176432308071286, 3.9744791173320841]]], [[[1.368172190061947, 3.2623946452691186, -4.4664718437178692, 4.7390659910937298], [-1.0714168531156054, 1.8134221777493771, -3.674140453774013, 1.3869920143868653], [-1.0854909264268575, 2.1012608404980151, -2.3244772178727047, -1.1763579171311891]], [[-3.4780546169709869, 3.4162398461109973, -2.3524673563114784, 3.3372100667565991], [-1.6973090509055422, -2.3340281089940715, -4.2589795788072804, -4.4365347925799004], [-1.0977824313198301, 4.8485459450720949, 2.6606051484129836, -2.3720513092233917]]], [[[2.5718336405756634, -2.5498154740344345, -0.39550725883060611, -0.96625572438458995], [-0.46834215782855093, -2.3156314271096914, 2.527575380524282, -0.22248954863971981], [-4.1052426216336633, 0.84837237327560899, 1.003511464931349, -0.54015301552670092]], [[4.1641417288033491, -0.53363471945261409, 3.9766578560708759, 1.8519206326244309], [-3.829974815897581, -1.6117914038863379, 1.3463121144956913, 0.18497770648350897], [0.99193511667381529, -1.5017994384271804, -0.71370968941280388, -1.2102807177645869]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.73492562007)+(1.-msk_arg1)*(2.80463523972)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.89674265542323228, 5.6704464648623967, 5.7054771637334474, -10.505125303295191], [7.6730595325852047, -3.2056145591389966, -10.597055462045104, -5.7334588310458221], [12.801926666212728, -5.9865089156329265, 11.763439773470925, 2.4318564143198191]], [[6.1695174108496911, 7.9097732603962365, -8.8871302555545153, 11.657563165611361], [-1.4704681193817331, 7.4780508922917441, -9.9555040441491531, -12.424566141611393], [-4.1495751080035266, -3.2572770361173928, 1.4157157339879767, 10.86990476440746]]], [[[3.7418491752617595, 8.9224066981115264, -12.215468276685897, 12.960992994224091], [-2.9302454013559931, 4.9595747739220313, -10.048500858746211, 3.7933199949731136], [-2.9687369450336187, 5.7467921071187194, -6.3572722964190023, -3.2172514059291633]], [[-9.5122206799416009, 9.3431618794181279, -6.4338232431443894, 9.1270213111136478], [-4.642014008490893, -6.3833932732412864, -11.647992365416476, -12.13359266853945], [-3.0023532966745718, 13.260412525243131, 7.2765571852732611, -6.4873838977053406]]], [[[7.0337737141571148, -6.9735556663766358, -1.0816829350977641, -2.642637536154516], [-1.2808809664021379, -6.3330797166314969, 6.9127306648430631, -0.60849236677161411], [-11.227533222491413, 2.3202353390273687, 2.7445292154703664, -1.4772783209196976]], [[11.38861789968878, -1.4594512659875041, 10.87586345280361, 5.0648651844927572], [-10.474696248204546, -4.4081296046903402, 3.6820634945390358, 0.50590026860273407], [2.712868764034035, -4.1073097603547133, -1.9519429148641809, -3.3100277424858224]]]])+(1.-msk_ref)*numpy.array([[[[-0.91959943404214173, 5.8149786098807086, 5.8509021947142132, -10.772886987173155], [7.8686356234573829, -3.2873214143584706, -10.867160323504997, -5.8795970775419644], [13.12823076464667, -6.1390970725107428, 12.063274221038492, 2.4938412026634147]], [[6.3267702110733453, 8.1113828696250057, -9.1136513957948573, 11.954698959142419], [-1.5079483976618038, 7.6686564720668917, -10.20925661251491, -12.741251821741974], [-4.25534226319858, -3.3403006992215576, 1.451800446723982, 11.146964191999235]]], [[[3.8372239382520861, 9.1498269879943113, -12.526824330106534, 13.291351481978191], [-3.0049334626775113, 5.0859877442049726, -10.304623792333977, 3.8900066807590838], [-3.0444061046526461, 5.8932702011035696, -6.5193107191711741, -3.299254868909288]], [[-9.7546745444263045, 9.5813066597371837, -6.5978128478011895, 9.359656955572575], [-4.760332776864721, -6.5460974849808844, -11.944884211969061, -12.442861821511698], [-3.0788792924246629, 13.598402818948795, 7.4620269582184644, -6.652738692270951]]], [[[7.2130552590548707, -7.1513023332593253, -1.1092535956812208, -2.7099948551898154], [-1.3135289200922751, -6.4945015026740407, 7.0889269832660924, -0.62400202858426734], [-11.51370812423267, 2.3793750544933281, 2.8144836180091062, -1.5149321821869965]], [[11.678898635788793, -1.4966507393146866, 11.153074759444193, 5.1939618674223036], [-10.741682335904963, -4.5204869704167585, 3.7759143999760321, 0.51879499416615893], [2.7820161837387607, -4.2119996280039835, -2.0016953458564841, -3.394395950995698]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_expandedData_rank4(self):
      arg0=Data(-1.77973059716,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.64900947777075135, -2.1182967199019509, -4.0733030829783345, 1.2650099166476858], [4.0963411448454305, -4.4807178441322382, 3.0001985564027365, 0.080392314571470003], [1.6665626992217746, 3.1469225651584516, -3.4140046284542294, -2.9645634586748337]], [[0.17398705996680164, 4.6505084532165633, 2.7291472220729442, -2.294329629974766], [-0.28945621298936164, -1.591169232096834, -4.5154216834966485, 2.3378174550583255], [0.9497218436524415, -1.434711056736643, -0.91716234787179118, 3.8424426009185471]]], [[[-1.475298149960762, 1.8301410245846039, -4.6050566364709944, 3.4650984937029037], [3.1340052890891137, 0.81518923083524264, -0.9501280002136383, -3.5433904210177456], [0.080852616452861348, 3.5751677220853182, -2.1276474651729482, -3.6677729826005279]], [[1.4523644493545689, -3.1618505366345264, -3.0408185482442618, -3.1342002143236094], [4.0889041419747638, -3.26370522103654, -2.0940055120262557, -3.0308863503359529], [2.07692200145872, -1.8316732206988484, -0.61617257389155711, -4.7167614165509777]]], [[[3.1551084566397911, 0.2837823486392459, -1.4008744304391385, 2.5818527876975672], [-0.73487267192445849, 4.5872410538483592, 0.29179742066148506, -2.0087903780035941], [-0.98310171996728712, 0.98157552360889611, -1.0249123410459204, 2.2624237658218593]], [[0.87975385809563189, -0.21314761910066959, -2.22742929627334, -3.3291644466147794], [0.81408767716853525, -3.5398828854376472, -0.36048181015966829, 2.6310467525134316], [0.37206804449212427, 1.3458221601796714, -3.206627339407234, -0.57763740310339262]]]])+(1.-msk_arg1)*numpy.array([[[[-3.4118096100019613, 1.3404629977374904, -4.4284709259534116, -4.5605708027423066], [4.8992116902561982, 4.5943910370784184, 4.2399396931390516, 2.0900773483171537], [-0.41749010442884327, -2.4749457751723059, -3.7514044728827201, -4.7900125709483241]], [[-3.743063496705108, 0.60508323859268032, -4.7507175233637504, -1.3442273508011393], [-1.3081541073005853, -0.41151910097103617, -4.9756255861895831, 3.4227601753522876], [1.2506737616761789, -4.6829650950522357, -4.6964023966906279, -0.70072558139394747]]], [[[3.8639962013093214, -2.3347127080700503, -4.0199085192572994, -4.6595398730472741], [3.6148394825138102, 0.5204103729355678, 4.7513982098045382, -3.5265954249513976], [-2.7043324008536662, -2.8216193859329097, 3.9867080173058387, -1.4272243257865536]], [[-0.12575745222089285, -3.8644198793163986, -4.9242716741641503, -2.869928034650342], [-2.1043757627368134, 1.9453612161208174, 2.5226349151732848, 1.5497135719023927], [1.5584344599477795, -1.0839497554074748, -1.7618109448358696, -2.7954295020403483]]], [[[1.0940294889520032, 2.4864277475034759, 2.0903046721651908, 4.9459620303278982], [-2.2212123872805312, -0.49841853068820363, 3.2469072970809041, -2.6121998659463816], [-4.2989989203818819, -0.071444835573995391, 3.4723180206550612, 0.46957907162601487]], [[-3.7037099221795056, 0.9914111710762743, 1.7750950382744168, 0.72392339836288588], [-3.0558917411887876, -0.97402582283842687, 1.7870948146294463, 3.0387959431992186], [-3.0333961968622569, -4.72882080891985, 1.7438934990665675, 2.1446010184341198]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.1550620254345885, 3.769997486270392, 7.2493821282773618, -2.2513768543670496], [-7.2903836718814672, 7.9744706444370639, -5.3395451683812798, -0.14307666201925151], [-2.966032627888366, -5.6006743761016056, 6.0760084961013749, 5.2761242946221918]], [[-0.30965009413260058, -8.2766521865346476, -4.8571468152738593, 4.0832886424338648], [0.51515407879484942, 2.8318525676202317, 8.0362341291927848, -4.1606852553389615], [-1.6902490239382111, 2.5533991657560802, 1.6323018930693285, -6.8385126646807537]]], [[[2.6256332574167769, -3.2571579785685727, 8.1957601975761083, -6.1669418114115437], [-5.577685104649059, -1.450817216591739, 1.6909718731974099, 6.3062803499642923], [-0.1438958753614934, -6.3628353849693751, 3.7866392937354232, 6.5276478005661449]], [[-2.584817448741858, 5.6272421436910882, 5.4118378107179792, 5.5780320190530501], [-7.2771478103213854, 5.8085160419852935, 3.7267656803720755, 5.3941611742035258], [-3.6963616339081482, 3.2598848748739413, 1.0966211828848276, 8.3945646125333369]]], [[[-5.6152430576359667, -0.50505612880682049, 2.4931790866297869, -4.595002403624818], [1.3078753792397182, -8.1640532600763969, -0.51932079772323014, 3.5751056990109658], [1.7496562111451146, -1.7469399927888138, 1.8240678527649663, -4.026504799772149]], [[-1.5657248592211999, 0.37934533942498755, 3.9642240715853108, 5.9250158286131995], [-1.4488567478266876, 6.3000378815717681, 0.64156050726031177, -4.6825544080031598], [-0.66218088300763411, -2.3952008768059638, 5.7069327896286159, 1.0280389603663955]]]])+(1.-msk_ref)*numpy.array([[[[6.0721019546005461, -2.3856630114324706, 7.8814852055469595, 8.1165873981490488], [-8.7192769471064953, -8.1767783040001039, -7.545950401987195, -3.719774607228338], [0.74301991286298885, 4.4047367223827836, 6.6764893227073419, 8.5249319332914908]], [[6.661644632193874, -1.0768851535512647, 8.454997334788418, 2.3923625457583548], [2.3281618905616632, 0.73239313531338923, 8.8552730957472416, -6.0915910108107072], [-2.2258623607186503, 8.3344162650906135, 8.3583310419597119, 1.24710275741862]]], [[[-6.8768722667751465, 4.1551596421274919, 7.1543541895010963, 8.2927256807431498], [-6.4334404308471109, -0.92619026379219438, -8.4562087732741578, 6.2763897815858529], [4.8129831186868879, 5.0217223546809118, -7.0952662403370548, 2.5400748016115107]], [[0.22381438553824498, 6.8776262994876847, 8.7638769672317824, 5.1076987349107172], [3.7452219328618614, -3.4622188788560564, -4.4896105439947096, -2.7580726607467709], [-2.7735934920355412, 1.9291385454813605, 3.135548844933457, 4.9751114169812869]]], [[[-1.9470777556817644, -4.4251715398562963, -3.7201791824361536, -8.8024799577596742], [3.953159648431058, 0.88705070925667318, -5.7786202627527032, 4.6490120273186024], [7.6510599157558081, 0.1271525598800112, -6.1797906244253102, -0.83572424155819047]], [[6.591605871503095, -1.7644447955293732, -3.1591909524815547, -1.2883886220655267], [5.4386640333982283, 1.7335035593282173, -3.1805473216196618, -5.4082381186333484], [5.398628024860562, 8.4160270821153613, -3.1036606184748985, -3.8168120512048893]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([3.2350033200181727, -1.1825000013160833]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.081623330139542816, -4.5176127316710257])+(1.-msk_arg1)*numpy.array([1.6027170081574784, 0.90406164924485921])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.26405174399236037, 5.3420770611465427])+(1.-msk_ref)*numpy.array([5.1847948424390351, -1.0690529014218666])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[1.5528852455250846, -1.5670094110512478], [-4.3328496435452202, 2.2083335996105475]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.7965850900904723, -0.9098418804964945])+(1.-msk_arg1)*numpy.array([-4.3842022650392476, 1.6500655516425677])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.4485462153071031, -7.5162939770798678], [3.942208067591749, -2.0092343950332534]])+(1.-msk_ref)*numpy.array([[-6.8081630107771041, 6.8700862092686981], [-7.1494859372607467, 3.6438951992521953]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_expandedData_rank2(self):
      arg0=Data(numpy.array([0.096136284117739557, -2.7746165175489224]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.6898343145226296, -3.5931254032935844], [4.1493919081798154, 2.8939885971637827]])+(1.-msk_arg1)*numpy.array([[3.7046326975979955, -3.2754214281746217], [4.8439132547489798, -2.9505127858743387]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.45086324412607187, -0.34542972464169958], [-11.512971326219557, -8.0297085632888656]])+(1.-msk_ref)*numpy.array([[0.35614962156814883, -0.31488684502432768], [-13.440001726200681, 8.1865415109262276]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_expandedData_rank1(self):
      arg0=Data(numpy.array([[[4.8661343247570894, -4.1423900314449567, 1.8950769847323539, 3.869949988643457, -3.5730829501162065], [3.341362333184744, -2.1965452024197774, 3.6138343562458282, -4.22104705183642, -0.27482499987816311], [0.32001816074927358, 3.6838671331120345, 2.2124554936854937, 0.17425345367039657, 4.2043271648802509], [-3.7284332511801175, 1.9783993736356305, 0.73892858210903611, 0.43904997170526094, 3.3150314364576499]], [[-3.1381896583790612, 4.8430016062290075, -0.16726928451286849, 3.8637929619304003, 1.465538182429559], [2.7632761942961901, -1.7602409815536979, -0.8798368170019657, -3.5302883924427508, -0.023974009366103388], [3.1636943536119837, -2.7243424661180971, 4.1500297498414067, 3.3393002171714272, -2.0991752591314605], [-3.5335149929775564, -1.2219900651529492, 0.5062694938307013, 1.3802089776468121, 0.73541003202131527]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.12745442110762806, 2.3037523457018692])+(1.-msk_arg1)*numpy.array([-3.4343087224474909, -1.878560030671248])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.62021033339387333, -0.52796592345982618, 0.24153594004345147, 0.4932422355180236, -0.45540521897659697], [0.42587140188689498, -0.27995939721115043, 0.46059916585416971, -0.53799110845987097, -0.03502766126537523], [0.040787729422226526, 0.46952515288821173, 0.28198723417407606, 0.022209373063565283, 0.53586008494688753], [-0.47520530166759345, 0.25215574688642323, 0.094179714672587608, 0.055958859981014512, 0.42251541268729848]], [[-7.2296117867481096, 11.157076310587996, -0.38534700656039411, 8.9012220993535323, 3.3762370254877503], [6.3659040144319823, -4.0551592902548919, -2.0269261310031448, -8.1329101650940672, -0.055230180313039265], [7.2883682882173657, -6.2762103468147812, 9.5606407709296821, 7.6929207083114362, -4.8359799272634314], [-8.1403434536447694, -2.8151624790204868, 1.1663195339697761, 3.1796596698126218, 1.6942025863217918]]])+(1.-msk_ref)*numpy.array([[[-16.711807556114405, 14.226246216770951, -6.508279418375813, -13.290603001433793, 12.271069941612501], [-11.475269805713864, 7.543614347920431, -12.41102285113546, 14.496378707983084, 0.94383389422820618], [-1.0990411608028334, -12.651537027584292, -7.5982551999909607, -0.59844015585684274, -14.438957454371176], [12.804590835591133, -6.7944342253614982, -2.5377088748028198, -1.5078331474177018, -11.384841377414142]], [[5.8952776608967623, -9.0978692459384671, 0.31422539224485191, -7.2583670250713253, -2.7531014529347573], [-5.1909802123101807, 3.3067183522963024, 1.6528262779329059, 6.6318586707856051, 0.045036615770099965], [-5.9431897619557823, 5.1178408667097957, -7.7960800141486644, -6.2730759183900613, 3.9434267391783213], [6.6379200335852326, 2.2955816942736846, -0.95105763585851943, -2.5928054193809271, -1.3815118923099055]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_expandedData_rank3(self):
      arg0=Data(numpy.array([0.0086223167042742688, 4.9554926099602916]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.3909614567270436, 3.3940795361629306, -0.53416185858040865, 2.0441951398192861, -3.1623843908302685], [-1.2099127261908493, 1.3196549167215341, -1.9754890862436225, -1.1666381019023495, -4.3000078949173135], [4.5791538628165167, -4.9087735790292317, -3.4009278276473509, -2.5677015185262841, -1.1632138049804084], [-3.556956582340367, 0.015910306178771094, 0.70585922962666814, -2.0500738728819812, -4.1862722024162942]], [[3.1701772492439133, -2.3152812939404841, -3.3659811037175489, -3.3206557120450455, 2.4454295467842391], [-0.69142361492923943, -4.5388217659550296, 0.37891651923126357, 0.23023306869167293, -1.7363990766667259], [2.2968668943064463, -4.9795568108816566, -4.9631259371891225, 2.9394063982954961, -1.4121410668890313], [4.2399017856939292, 3.995915429951463, 1.483213164251131, -4.8695255908722732, 3.7861113454231781]]])+(1.-msk_arg1)*numpy.array([[[3.6958399325732465, -3.7370957492801873, 4.6235172998045009, -0.085516736458142617, -0.36866766137634066], [-0.52187923033931138, -1.2808206618532738, -3.5303398756411033, -3.413816731055852, 1.33538368983226], [4.3137109800053963, -2.4665871622116944, 2.6983689985192081, -4.4164477186653484, 0.8796641809737924], [0.46597154082583447, -3.9303105208483311, -1.3381812764197907, 0.75205422263135269, 3.6324607631854704]], [[2.5321640422345197, -0.23829766267250818, -2.2479042246954761, 0.72695158517038827, -1.5205510106017739], [1.9009812642847477, -3.0791102620042921, -4.60501387773059, 0.76674389177729285, -4.0545450391902698], [0.10979526537137829, -3.5932010222513311, 1.7810746627462537, 3.0314436075010072, -4.6676267656053554], [-2.3804460163390151, -0.90650686290907068, -3.9012038232469104, 2.0003168229807713, -1.2196314784716855]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.011993310203339259, 0.029264828680293099, -0.004605712716024047, 0.017625697900860104, -0.027267079758392033], [-0.01043225070974938, 0.011378482632325753, -0.017033292547429899, -0.010059123193875455, -0.037076029900856788], [0.039482914842804896, -0.04232500042796393, -0.029323876818354955, -0.022139535694779586, -0.010029597821325007], [-0.030669206156291661, 0.00013718369873533612, 0.0060861418264761876, -0.017676386199146549, -0.036095364739533044]], [[15.709789930892457, -11.47335934210137, -16.680094484738298, -16.455484841261654, 12.118308047267842], [-3.4263446141338765, -22.492097719117069, 1.8777180108424034, 1.1409182704700653, -8.6047127923638342], [11.382106920798041, -24.676156977201487, -24.594733904042943, 14.566206684423328, -6.9978546211900365], [21.010801965963712, 19.801729383150775, 7.3500518744423005, -24.130898079580071, 18.762046792731375]]])+(1.-msk_ref)*numpy.array([[[0.031866702386950191, -0.032222423104490923, 0.039865430446605413, -0.00073735238525806344, -0.0031787693350109518], [-0.0044998080053684436, -0.011043641387877107, -0.030439708481505831, -0.029435009025513853, 0.011514101095456104], [0.037194182240311853, -0.021267695691286358, 0.023266192090227997, -0.038080010938202222, 0.0075847431617620739], [0.0040177542001790124, -0.033888382056895469, -0.011538222773021424, 0.0064844496863143124, 0.031320227116034938]], [[12.548120198500342, -1.1808823063444247, -11.139472773376951, 3.6024032081107786, -7.5350792961047439], [9.4202986068360399, -15.258508148615167, -22.820112239858524, 3.7995936894345683, -20.092267978458544], [0.54408962615649425, -17.806081111868238, 8.826102329026579, 15.022296394482607, -23.130389943010197], [-11.796282642377404, -4.492188060024187, -19.332386716048902, 9.912555233860461, -6.0438747784413822]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_expandedData_rank1(self):
      arg0=Data(numpy.array([[[[0.20843845413257078, -3.6949302279410814], [-1.6217173902139037, 3.9930359695318494]], [[1.4227577978949189, -0.74101495997050471], [0.4216012592032925, -2.4194959727748708]], [[2.8850763700897879, 1.3251118099126096], [-0.94900703687844956, -4.7681835492237088]], [[3.5429206179719408, -2.43169181084264], [2.3478333138451042, -4.2868133527019445]], [[4.7863521455456333, -2.3500890004966903], [4.0738325758786065, 3.1856026862807489]], [[-3.9351963802615431, -1.1540425408656221], [2.9274648462297703, -1.4108913385376418]]], [[[-2.6742690989875486, -1.2133489337902037], [-0.15530467025764061, 3.0200989615270508]], [[1.2412369906922196, 1.0001195513972334], [1.504109709409482, -3.2341765034792616]], [[0.30054432847747758, 4.9678791681914287], [-1.0174980986102256, 4.3011691589795245]], [[4.02281464714663, 0.46881419607510288], [4.9153696060581176, -4.4937724887265693]], [[0.022988487311801364, 0.85988495151830513], [-0.23740614155971063, 1.9075352941088326]], [[3.1585188489470912, -0.26751931815652341], [1.2008099641719063, 2.8839450076810911]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.673098187800786, -1.2643914675581094])+(1.-msk_arg1)*numpy.array([-0.76660954353225375, -2.5537704235479142])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.76561490814234301, -13.571841524300732], [-5.9567272071197097, 14.66681318351079]], [[5.2259290891272636, -2.721820706600933], [1.548582821154143, -8.8870462729906787]], [[10.597168786643669, 4.8672657876234258], [-3.4857960273684268, -17.514006353755125]], [[13.013495301394777, -8.9318427836961121], [8.6238222903427655, -15.745886357249724]], [[17.580741391980069, -8.6321076488949533], [14.963587051863518, 11.701031454031135]], [[-14.454362692978886, -4.2389115654985314], [10.752865821537076, -5.1823424187664378]]], [[[3.3813230307141695, 1.534148039055063], [0.19636589994568648, -3.8185873581359098]], [[-1.5694094602487469, -1.264542627324706], [-1.9017834828486564, 4.0892651755760987]], [[-0.38000568454990435, -6.28134403212092], [1.2865159141393689, -5.4383615851378]], [[-5.0864125154199851, -0.59276466938747463], [-6.2149513897943489, 5.6818875918932443]], [[-0.029066447209109503, -1.0872311957813636], [0.30017429973399079, -2.4118713499371567]], [[-3.9936042827301632, 0.33824914328407141], [-1.5182938728577173, -3.6464354606187777]]]])+(1.-msk_ref)*numpy.array([[[[-0.1597909081771387, 2.8325687754254387], [1.2432240282501985, -3.0610994819106812]], [[-1.0906997060011783, 0.56806914021355992], [-0.32320354887045943, 1.8548087032670699]], [[-2.2117270791302239, -1.0158433597263041], [0.72751785135028491, 3.6553350141483891]], [[-2.7160367577144799, 1.8641581491211958], [-1.7998714250166139, 3.2863120275228082]], [[-3.6692632334813613, 1.8016006559309383], [-3.123038931421124, -2.4421134212048061]], [[3.0167591007820787, 0.88470002546979687], [-2.2442224894749234, 1.081602765009952]]], [[[6.8294693296025306, 3.0986146205568184], [0.39661247354282403, -7.7126394041355519]], [[-3.1698343154434081, -2.5540757303702626], [-3.8411508896611828, 8.2593442991189452]], [[-0.76752121703085141, -12.686822887487084], [2.5984565502470329, -10.984198584878365]], [[-10.273345065298402, -1.1972438280759903], [-12.552725520757583, 11.476063271863215]], [[-0.058707318978984818, -2.1959487568413798], [0.60628078268381824, -4.8714072159689081]], [[-8.0661320186596832, 0.68318292243583401], [-3.0665929708038449, -7.3649334637546326]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_expandedData_rank4(self):
      arg0=Data(numpy.array([0.61968027099593126, -0.93316798785507604]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.37225949823829563, -1.2291645489814904], [-0.67259198368709328, -3.8100725326080975]], [[-2.2670341490154025, -3.4565523131988076], [-4.2183398581493261, 1.9368721944397249]], [[0.42792865001873981, -4.6166024307525975], [2.3676885075111382, -1.3269478019373668]], [[1.1639704356396958, 4.7666088332240779], [4.8898276455048251, 1.6041368170990191]], [[-0.15915810313328027, -1.1080429584009654], [0.031298812189684, -2.6953184687299925]], [[-3.2885826148672526, 3.0766834118530575], [-0.056837623199879062, 0.62837464839442614]]], [[[4.2423711958337513, 4.0876942916815793], [-0.81271757915179688, -3.7451033382489394]], [[0.63639409552098058, 0.75720612403154863], [4.0379882829800948, -2.3025748922728106]], [[1.7929501896668798, 1.129677573090957], [-4.7252945271721423, 0.46005511938094212]], [[3.4380515371421971, -4.5893670775788475], [4.0388732713724913, -3.8519446753578857]], [[-2.7565603272324046, 0.97810085777380174], [-1.5331749121740335, -3.0845984075982602]], [[-0.81386738516327828, 4.6610062165210149], [-0.54847682275193144, 1.0966498337361017]]]])+(1.-msk_arg1)*numpy.array([[[[4.8062214647378649, 0.24371724394206673], [-1.2161841186296685, 4.0552068986777954]], [[0.67168885769938491, 4.9609301110380262], [4.2929772716403161, -2.2948370572417454]], [[-4.0655566667214815, -0.054004050809631998], [-4.0780562269651472, -1.1541850133901588]], [[0.48374893767344584, -0.065028347502923189], [-2.3151914218899905, 1.4609017302196392]], [[2.2744051309265965, 2.4205333697170106], [1.8151427498242558, -4.2209448600984238]], [[-0.45336128125904818, 4.3369541149976882], [-4.9093833733141707, 0.78308846595697457]]], [[[4.4030432948755625, -3.4818636565642413], [1.5030314047783468, -2.8710260023931222]], [[0.98062659752278591, 2.5500274229457078], [-2.0556129122268496, -3.4163173008446601]], [[3.418904592987948, -1.8143383960831407], [-4.842730895585408, 3.2244123957530118]], [[-3.9877946491746377, 4.4018018684019484], [1.8262578199943853, 0.61128320318852047]], [[1.7485551761970806, 3.6817436358751543], [-4.4392543150333772, 3.0534410671551289]], [[0.43874431779692635, 4.0143102977555731], [2.2505880718312676, 0.22908840905542149]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.23068186674911642, -0.76168902081144163], [-0.41679198272090895, -2.3610267795207398]], [[-1.4048363358188951, -2.1419572741546502], [-2.6140219864509127, 1.2002414863348927]], [[0.2651789418105357, -2.8608174453692445], [1.4672098559684541, -0.82228337350200276]], [[0.72128951498845884, 2.9537734535038962], [3.0301297204898265, 0.99405193753447074]], [[-0.098627136480829491, -0.6866323607370437], [0.019395256419554138, -1.6702356791229402]], [[-2.0378697659734475, 1.9065600104257892], [-0.035221153747265685, 0.38939137240403099]]], [[[-3.9588449925505147, -3.8145054571351795], [0.75840202803153078, 3.494810546463091]], [[-0.59386259760016447, -0.70660051515406141], [-3.7681214010109083, 2.1486891791078371]], [[-1.6731237208158192, -1.054178947806294], [4.4094935859438307, -0.42930871005514054]], [[-3.2082796350570355, 4.2826504413125841], [-3.768947243848316, 3.5945114620327923]], [[2.5723338539645928, -0.91273240936810252], [1.4307097478233257, 2.8784484893594402]], [[0.75947499019368858, -4.3495017924509165], [0.511821013072565, -1.0233585187291216]]]])+(1.-msk_ref)*numpy.array([[[[2.9783206197352219, 0.1510267677724014], [-0.75364530421338072, 2.5129317099172264]], [[0.41623233336410237, 3.0741905155999194], [2.6602733190694448, -1.4220652495230701]], [[-2.5193452569832826, -0.033465244840590797], [-2.5270909878624073, -0.71522568187705615]], [[0.29976967279147476, -0.040296784003029031], [-1.4346784477242447, 0.90529198008093092]], [[1.4094039878871298, 1.4999567745009319], [1.1248081511073946, -2.6156362547646745]], [[-0.28093904162966959, 2.6875249012786866], [-3.0422480191982446, 0.48526447279800611]]], [[[-4.1087790519178125, 3.2491637023817708], [-1.4025807916799982, 2.6791495577327926]], [[-0.91508934884750759, -2.3796039592455109], [1.9182321651116423, 3.1879979415036961]], [[-3.1904123197070411, 1.6930825103611105], [4.5190814455570454, -3.0089184273598031]], [[3.7212823087495357, -4.1076205924733609], [-1.7042053351887583, -0.5704299167290372]], [[-1.6316957154254077, -3.4356853004878496], [4.1425700167366601, -2.8493734566712079]], [[-0.40942215222140582, -3.7460258631824792], [-2.1001767424814193, -0.21377796971916826]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[4.3222117252349133, 1.8668461678136552, -2.3742261552298496, -2.518935731833305, -1.6011379176289928], [4.3799130520189244, -1.9685093374342131, 4.1057754814037644, 0.075345732227047435, 2.5672188430073781], [1.4409816256154038, -2.51047806739219, -1.5587895772107729, -3.0880325132407949, 0.26703907738918975], [-0.52926897812885976, 0.080081374952491124, 3.7041342993520381, 3.9260119361560957, 4.7128536885295631]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.5973429999305973, -4.0169493776018488, -4.9831732983951387, -2.0556630258390785, 2.6324757002855836], [-4.6355783745060002, -3.5722286012285256, -4.3314123648926239, 1.7614092172727016, -3.9865886511021262], [-2.7318972995428714, -1.4934814161801024, -4.8996537197782626, 0.6819521684738632, -0.35786873200758595], [-2.2955381347619919, 0.50923470078650634, 3.4227523922476362, -3.2579364823933377, 2.5715591488257674]])+(1.-msk_arg1)*numpy.array([[-4.1780774464029129, -3.9314642339467398, 2.0182018482558934, -1.9176272720488408, -4.0023074619257448], [0.88535106779019124, 2.7848305199176515, 1.0106380644478037, 2.5957543208339153, 0.2176070323232695], [-0.35155894440919599, 0.81966237261098129, 3.7994168095356944, -2.636193042969416, -2.8986141468242508], [1.5000937767025215, -4.3809042380934233, 3.9236909238438891, 0.79658341354281159, 2.0729369889606808]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-15.548478093991765, -7.499026551877459, 11.831180381092738, 5.1780830483946252, -4.2149566609641838], [-20.303430226155498, 7.0319653569679108, -17.78380668762523, 0.13271466722688219, -10.234445504428745], [-3.9366138117096185, 3.7493523393779746, 7.637529150332349, -2.1058904687223539, -0.095564936021744956], [1.2149571228413083, 0.04078021501250384, 12.678334534313709, -12.790697517114648, 12.119382019815461]])+(1.-msk_ref)*numpy.array([[-18.058535327782216, -7.3394389390399191, -4.7916676146623658, 4.8303798559018514, 6.4082462352987664], [3.8777606974331502, -5.4819648816296711, 4.1494529855831495, 0.19557900998475355, 0.55864487375121308], [-0.50658997921439863, -2.0577444091065136, -5.9224913221836486, 8.140649827868744, -0.77404324747520137], [-0.79395310029280552, -0.3508288349217169, 14.533878131066436, 3.1273959897130457, 9.7694487345127108]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_expandedData_rank2(self):
      arg0=Data(numpy.array([[[-1.5931644434126957, 4.1069024273129351], [1.3864060963524771, -1.0808387630142557], [3.9846914151680366, -0.76556854101218086], [-4.8531285053758735, 1.6049680987573254], [-3.4703943970414652, -4.5446435046282048]], [[-2.3551902062301719, -4.8687765392590467], [-0.18730916531064867, -0.7383690294266243], [3.9979762518855662, 1.5910951130525941], [2.0500102449315598, -3.7031262202482651], [2.049883962799508, 4.4374898661510525]], [[-2.9720501777448751, -4.4715660315280381], [2.8045595657843183, 0.90368985144743785], [-0.5647868429703875, 2.1166698684035792], [1.4755330277561596, 4.5958111198179115], [-2.1361386573989338, -2.4587656019705451]], [[-0.83203243759925272, 1.9339459793417646], [-3.2856251072116049, -1.2362477865885446], [1.9723679207321414, 0.27360037957091343], [-3.0364137615985785, -1.3816325180979359], [2.1998721207737475, -2.8563435505449952]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-2.2696381501833409, 4.2171579837604281, 0.36855743229397397, -4.0275812564721534, 4.2151419597314597], [-1.1211341526841045, -0.98282433730093643, -4.5998688644623389, -4.3109514812018457, 2.4341219698779915], [-2.4675156009943136, 2.418496069042547, -4.4332615581980361, -0.78019732508641404, -0.75236451784326164], [2.4897943954248269, -3.2594748770348616, 4.8837359024812539, -2.1278850244098768, 4.5381422553807376]])+(1.-msk_arg1)*numpy.array([[-4.6241705430361799, 4.1117739604883372, -3.5833163973874393, 3.485217867693752, -1.0879703116952575], [4.2229411551148779, -3.1773444798124206, 1.5216623331933201, 1.2425042770835057, -0.051220589749468637], [-4.754026742133016, 3.8347309335337947, 0.51413217426810931, -4.6737165270307131, 1.5228500667343887], [2.4885657903712382, -1.764337322995372, 1.564542505934293, 0.066176523424964273, -4.0881747684635705]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.6159068002850625, -9.3211824281100029], [5.8466935379669787, -4.5580678186033134], [1.4685876364581729, -0.28215597572049328], [19.546369403502585, -6.4641394317907519], [-14.628205039786438, -19.156317528379379]], [[2.6404841762717655, 5.4585516599504382], [0.18409200626682984, 0.7256870520297577], [-18.390166481908256, -7.3188288709288125], [-8.8374947018666656, 15.96399746425665], [4.9896675895508418, 10.801391574309225]], [[7.333580180523402, 11.033658943671664], [6.7828162852450467, 2.1855703533592719], [2.5038477995166497, -9.3837511589896838], [-1.1512069213320133, -3.5856395422843317], [1.607154931020301, 1.8498879966161661]], [[-2.0715896999462764, 4.815127860419504], [10.7094124923112, 4.0295186021753162], [9.6325240273818586, 1.3361919966429685], [6.4611393712176772, 2.9399551444983056], [9.9833326277173811, -12.962493362612488]]])+(1.-msk_ref)*numpy.array([[[7.3670640894416186, -18.991017227504262], [5.7005884856444, -4.4441646812484414], [-14.278410086500585, 2.7432743063329261], [-16.914210181149866, 5.593663494867501], [3.7756860738546782, 4.9444372102741756]], [[-9.9458296500128895, -20.560556822694817], [0.59514574241806173, 2.3460527597131393], [6.0835698714956754, 2.4211095020600997], [2.5471464973924682, -4.6011501672385453], [-0.10499626549256863, -0.22729084795154755]], [[14.12920602396032, 21.257944493097899], [10.754731321851233, 3.4654074276660496], [-0.29037508757438657, 1.088248081650125], [-6.896223098003631, -21.4795183858045], [-3.2530188969738738, -3.7443313610450639]], [[-2.0705674606886921, 4.8127518046159166], [5.7969510060241056, 2.1811581103485866], [3.0858534493266756, 0.42805942347845066], [-0.2009393064223122, -0.091431636698600424], [-8.9934616979936788, 11.677231633401698]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_expandedData_rank3(self):
      arg0=Data(numpy.array([[4.393519141071339, 3.7293652046380519, 1.0069625865850789, 4.5593379538186802, -3.6038014711382993], [0.28237118722683974, 1.3584955430695844, 4.7712205302079695, -1.7637441294736709, -3.2304188371208795], [3.7804696272492002, -3.3522701336556193, 1.1003414852120041, 4.6252975002499834, 4.4698416627843276], [-2.1874835512824009, 3.2932647318120321, -2.9778177498288336, -1.9114357604246113, -1.2414957256794681]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.7070326559848343, 4.0365815156637215], [4.2565342887156916, 2.4847905660908722], [0.78730023613192124, 4.1902629223260277], [-3.2098060501785772, -0.30025052315346468], [1.0663002160807888, -1.2962414002166192]], [[0.026175855518076219, -0.6119893622550352], [-1.9130280674588462, 4.908599634501984], [-2.526948337310829, 4.198504411828953], [4.2242669584432981, 2.9561148474884336], [-4.2558027233394693, -3.2780743670872567]], [[2.2917362877477574, 2.7086908143911481], [-1.1604237672974405, -0.20642521516471568], [-0.48277923787341859, -4.3825884896040908], [3.7184926279382022, -4.6082241995342423], [4.5184818516668308, -4.4979283324110053]], [[0.18715672591609511, -4.5653279033794414], [-1.8036074903050627, -3.8258706394511477], [3.9517769330020087, 4.0179600448095325], [0.91418027174420224, -2.2544130999513334], [2.7413791091467319, -3.3227034848705461]]])+(1.-msk_arg1)*numpy.array([[[-3.9876084287738967, -1.2716190892000387], [3.3626170149667534, -1.9578314453555801], [1.3192209343052497, -2.7334679683122607], [-4.4240918296522791, 3.0900396172438089], [-2.3613846579234132, -2.6040173958905708]], [[3.4153949920180811, 0.50107843793025264], [-2.6776765002003069, -2.6631675135792721], [-3.7474797647631619, 4.3245655106111514], [4.1078049747913941, -3.2933016099237697], [1.3636702502973499, 0.56897181498499094]], [[-4.3542551520224269, -0.79637821391605446], [-0.16737561228168918, 1.1738355041069672], [1.4172165938277361, 4.167372277859231], [4.8396712655992697, -3.5411887380153848], [-2.1375104178957596, -1.7481063285489098]], [[-1.220704514611668, 4.5250396598208305], [0.64306004040548359, 3.3660264687429233], [-1.7169747077976529, 0.60234517836212564], [-1.6366538022210095, -0.66296306629549662], [-0.30104844928900221, 1.4700726145318379]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[11.893399789574556, 17.734798153563318], [15.874170868685081, 9.2666914779921861], [0.79278188219444279, 4.2194379907369681], [-14.634590548976014, -1.368943605867506], [-3.8427342873870334, 4.671396665051021]], [[0.0073913073993174061, -0.17280816279015079], [-2.5988401034098629, 6.6683107261839361], [-12.056627785752321, 20.031990445887036], [-7.4505260492839662, -5.2138302083076802], [13.748025284546161, 10.58955318492178]], [[8.6638394294952299, 10.240123353414637], [3.890053937495348, 0.69199308363011136], [-0.53122202363115689, -4.822343927723999], [17.199134656700558, -21.314407870697213], [20.196898433115276, -20.105027456428747]], [[-0.4094022594533267, 9.9865796948530985], [-5.9397569378536748, -12.599604845379611], [-11.767671494457531, -11.964752739536882], [-1.747396862886557, 4.3091658180166821], [-3.4034104464726558, 4.1251221741670561]]])+(1.-msk_ref)*numpy.array([[[-17.519633958915524, -5.5868828085520725], [12.540426892140882, -7.301468468855326], [1.3284061242851986, -2.7524999757191746], [-20.170929790112762, 14.088534905703046], [8.5099615041478067, 9.384361722180163]], [[0.96440913874474843, 0.14149011341213577], [-3.6376115913042799, -3.6179011976451481], [-17.880052390176932, 20.633455748457237], [-7.2451169093110614, 5.8085413810892383], [-4.4052260641819041, -1.8380172689183707]], [[-16.461129351514135, -3.0106836495126101], [0.56108826615422935, -3.9350137022423746], [1.5594222117195089, 4.585532601650959], [22.384919406807978, -16.379051417855951], [-9.5543331205460049, -7.8137584983248649]], [[2.6702710466891908, -9.8984498247585773], [2.1177669515049993, 11.085216255856865], [5.1128377608870261, -1.7936741636505524], [3.1283586050001468, 1.2672113127579645], [0.37375036301472836, -1.825088867379717]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[-3.0607156657384662, -3.4367457320754955, -1.8809703774685427, -4.2465591889632996, -0.82506236251002818], [4.4365900051053604, -2.6347036575420679, -3.6469005399341348, 1.8504509228771617, -2.3499163556857128], [-1.2707380680193281, -1.5978932353730069, -2.0778740236705353, 0.34455012766140669, -3.2022796805053124], [-4.7758492259038903, -2.6414309921985391, 3.0827654134388265, -4.7215447446171659, -3.1359726281660203]], [[3.1378789176301272, 4.7915607032133085, -1.1581225562978057, -1.1607164688015894, 2.0303851503230943], [-3.7492535864074505, 2.221739436179238, 2.3657365328739086, -2.4516546510790049, 3.5247465146780943], [0.25757954333744149, -3.0644030641213869, -1.382910632874077, 3.3699587428751485, -3.2274229958978928], [-0.74430204809477551, 1.3563578846586202, 1.4328669733153232, -4.1901711344165493, -2.8846406937625346]], [[4.6612533010000305, 1.3260957726408495, 2.6893264556682936, 3.3488245580009419, 0.87098328701723915], [-4.3030778347140668, -3.4873584780955138, -1.6432752071686108, 4.0923973149304622, 1.7543610150922992], [-4.2675416764167329, -2.2063437764377349, -0.6983538302938328, 4.5321930958977248, 1.823121726008524], [-3.5512930667304654, 1.6724549947715319, -4.441242077673123, -0.51711843880038977, -3.0512057680394591]], [[-2.1479317134336018, 1.1902886595018272, 3.5944135757646674, 3.9633520225773662, 1.3928099992786604], [-0.9033135859639021, 1.4954688416506068, -2.3334085943659346, -4.1632060658423349, -3.9357266629879639], [-3.3790843978650678, 2.7196340046736651, -2.5249902643289799, 1.2222430784611182, -3.3119380466978998], [0.36148724039225222, -1.8619469819359304, 2.0844611292988011, -3.0006819635504378, 0.07880787036814052]], [[2.1559916454957637, 2.0610271710432606, -4.4314054504449309, -2.1348973883193665, 2.1694541679444725], [3.0407652113756019, 1.7574051852990538, 3.0377077977833089, 4.789229745780931, -3.7949536675806317], [-4.2304840607743133, -3.8214500887257197, -2.5416370365087504, -4.0208408984980917, 0.39159974512409867], [-2.1741814527925718, -2.2336805101381141, 3.9781274048878839, 3.0983126068643596, -0.027646982023071587]]], [[[3.233581384697505, 2.2084679518713548, -0.39010774003382931, -1.7794019641650074, 3.1650971539209944], [2.710045912002391, 4.8245728767585678, 1.991582524459929, 1.8609585581442465, 1.8671627177574326], [0.2825017680452131, 3.3200178648092518, 0.69011787059870677, -1.3552381272351797, 2.3926030240947966], [2.6445577755145786, -0.64248448279748427, -0.48801123621474929, 1.2806186874346368, 3.6318976400748646]], [[4.3499912680887949, -3.804346969544655, -2.019321421168395, -1.3489082568938238, 0.46910474417418158], [-0.45912695523603464, -2.9053670333274741, -2.1834394443904315, -2.8539339653061599, -2.4411818410498674], [2.3182974295870657, -3.0840906022996926, 2.6050226275950195, -4.0346149021378146, 2.8222022512515235], [3.9671609405270463, -2.8270939239751591, -1.2151845307057307, 4.0955036192970908, 4.3157985982722646]], [[1.3596025118684532, -1.8582779677982519, -3.4102329429845382, -2.3542592366998836, 1.4321200440656625], [4.9416032874031917, 0.9077361982052885, -0.68470883367029955, -3.9522962906015047, 4.2977784066127498], [-1.5663805190301314, -0.1759059581967124, 0.21095292188047488, -1.6768616758946475, 2.4254091475248174], [1.8730980477558559, 3.6238040757887759, -0.86696617491969086, 2.9719692862670497, 2.7445902019046544]], [[-0.25740680849725983, 1.6888113986716471, 1.9498201455522315, -4.861041202771311, 2.0393387116670549], [4.4757013363010092, -1.9118081718937727, 4.5090034354846313, 0.48743776412864648, -1.019170176419756], [-4.40028491051069, 2.6720171959620718, -0.96225821425623081, -3.9026321955750367, 2.4776904032600253], [-2.3118933603112257, -0.071714034038539154, 1.1639112144892625, 1.9833207491910816, -4.165246031462595]], [[0.5729885751123156, 0.45310405914034479, -3.2400615875278471, -2.1235702938272292, 3.7793929277469225], [-2.1646168445356038, -3.7448294683787431, -1.0067593097803575, 3.4476092246075272, 1.6291538540693926], [0.13704150727536302, 1.2490026913227812, 0.14312323143680228, -4.2427699008331219, 1.2453872531920407], [-4.3275262553379923, -4.5163799834039366, -4.6127467510194711, 4.8243674936090581, 1.7124072966062887]]], [[[4.283864298177507, -1.0389561062241914, 3.7772520281795163, 3.494253697065, 0.074856299070629184], [-3.8634051752906484, -3.3023883692405684, -2.0672440558685556, -0.17216666319065066, 4.0894685100667729], [-2.9071203481483412, -0.73275752189501198, -1.2646063932048426, -3.8212511232199899, -3.1691395175931625], [-0.30581868962059744, -4.494885850014775, -3.2466783848724678, -1.2341709358227657, 0.52550460454899905]], [[4.9103059980405543, -0.80184443087831436, -2.077652667739649, 0.6383232958721452, 2.8839824649994235], [1.2147038535234955, 3.7704035042983914, 2.2844917552826036, 3.5757294637696209, -4.3040746561627898], [3.9703928635826458, -0.22639509506230837, 4.090520105635461, -2.5723412128161094, -0.55205019392510568], [4.7983900306502107, 2.6000241240622568, 1.4353163764171528, -2.8079777274156905, 4.9514154975494513]], [[4.3752603694486947, 2.4518651525418464, 2.6186754567461783, -4.7791052146119757, -2.1416400562068971], [-3.744649751707386, 0.99416075698221729, 4.218563988128551, -3.879670599684558, -3.1225875336696141], [-1.7798802246424064, -0.90528033793424889, -2.7111911279061509, -2.5536595404385922, -4.2971645329265504], [-3.3230852202049563, -4.009272761013146, 2.8496360908940925, -2.1862824427826553, 0.91203078507634672]], [[4.9749728447501145, -2.5528459416830396, 0.15319897907672075, -4.2479059570840816, 4.6131985139437042], [-2.0095140326719232, 4.1798570016612473, -2.5299972590649213, 3.7090153806349893, -0.045530721068360869], [3.9293341143426535, -2.4908913179124346, -1.4640606521242718, -2.5534302364830976, 3.1178360641789471], [-1.0051946593362668, -2.9624470786414503, -2.6381967673818174, 2.3536399208491154, -3.8043006569918667]], [[-1.6843760744268641, 3.9197724086543673, -3.8113362497979928, 2.8227879974059453, 3.7896654802467786], [3.7961887746165051, -0.44081999583031628, -1.5096983642354034, 4.0516742188120816, 1.3136022425306475], [-3.7063976937181673, 4.6051893082698374, -4.5669814537396327, 0.28542079985941982, 3.7591871473264327], [0.63197601241035795, -0.34940633979960189, 0.79968048893956567, 1.8082902098635927, -3.170139135316894]]], [[[-2.3143861726717061, 3.6921665840199775, -1.4946967261305586, -1.2441348702687649, -3.1939531756852269], [2.3907872265574373, -3.2449003480232097, 0.20147998480360396, 2.4104550499578004, 0.76276538477595413], [-1.9629058074194203, -4.0231104675306888, 4.2451302477827522, -4.0726448140585445, -3.0755925611055099], [3.6407990857010599, 4.3596366452072548, 0.96616933627695278, -2.8927496505514636, 4.138464776163973]], [[-4.5040127414448712, 3.4978009810112791, -1.4649026113150185, 2.0339672239715565, -0.30758373960418695], [2.1069411748295739, -0.076576823704602504, 2.0648935053631536, 1.7254920859409939, -4.0896012913791253], [4.7002549912324874, 4.6676293634733703, -3.8747094132600468, -3.56741356559069, 0.86235681821395094], [1.2300820191064288, -4.6804732574820065, -2.1595102040819834, 4.5567022242271253, -0.57554516237488329]], [[-3.2430690123908978, 0.13224499301424864, -1.0859125169615238, 2.3651437511531306, -2.2038591297931132], [4.8620149127435894, -1.2361861032427504, 3.1991655651271493, -4.5630296613769294, 2.0631096264275763], [2.8229039151435575, -3.3212234020261064, -0.075209568678188177, 2.6373176377127834, 4.6444448871613861], [3.976743951816248, -0.55243984516041156, 1.544545643505165, -1.0494947770333409, -2.0426084832480731]], [[3.2592812525642163, -0.34397488419522837, -2.3490382739518423, 3.4462111136719944, -0.34946255440857943], [-3.9902628410103294, 3.4115092332765968, 4.773713794968474, 2.0620779386256336, 3.1320026680087949], [-3.6782682350982698, -4.704991829286838, 0.11408506711281419, 0.54031552749145817, 0.63522548351046204], [2.9592687861172617, 3.3197413458090086, 3.8819548703738889, -4.212687616115387, -0.22114336725482531]], [[2.5929176030970256, -1.3513894662938575, -3.2080904979574854, -2.0221661435649341, 2.8261297681607616], [-4.7443156168177634, 3.5845728808414048, 3.7120858676227559, -0.22785261953900093, -1.7076532240825557], [0.39146368936686038, -1.9961368170527005, -3.3679268727867395, -2.2103197396600738, -3.688430526171893], [3.391100708071944, -3.4384876987957225, -3.6611169546640507, 1.2646145435762559, 2.5711440969947938]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.0129421467116382, 2.3724280275167056, 3.3268643377259117, 1.2250134360296023, 4.7812204616275569], [3.3945783300809236, 4.8064443090323081, 1.8321918837412534, 1.5292291266276594, -1.5716944302593028], [-1.5075674147965348, 4.0791461873340591, -3.1041767960329669, 2.1662310945990084, 0.56299537995902593], [2.3890913742608566, 2.4752483984369871, -0.663830996903787, 1.5904252752043311, -4.4153858994328736]])+(1.-msk_arg1)*numpy.array([[0.63044532320400037, 3.7137452903237307, -2.1081581737849575, 4.8501870313977964, 2.3159683805193234], [-4.0481406851303028, -3.6500285206629091, 0.014062328367088206, 3.9113997445305984, -4.5639366381029163], [2.0334936795188598, 3.2389340412722358, 0.20171125616633567, 4.1266771559380757, -3.7708428841776866], [-2.6155398247142383, 1.3271928170263987, 4.7789670934580784, 1.6316922121775779, -1.1889570545386241]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-6.1610435626655287, -6.9179703316261083, -3.7862845495225286, -8.5480779699698175, -1.660802803161912], [8.9305990089561824, -5.3035060363617346, -7.3409998016988496, 3.7248506530808858, -4.7302456736067882], [-2.557922214547026, -3.2164666394277455, -4.1826401978037167, 0.69355947362452097, -6.4460037344474221], [-9.613508193162092, -5.3170477718267799, 6.2054284291359423, -9.5041964140247313, -6.3125314741694467]], [[7.4443918911394977, 11.367632907850908, -2.747562411860208, -2.7537162825851103, 4.8169426372802286], [-8.8948342906605617, 5.2709169082307872, 5.6125396563102568, -5.8163742080115206, 8.3622074213141335], [0.61108892792870007, -7.2700757169296502, -3.2808559449813255, 7.9949845731719655, -7.6568287721200941], [-1.7658030398381324, 3.2178614609073817, 3.399373767196304, -9.9408794393812911, -6.8436024311974712]], [[15.507357376204187, 4.4117407344079309, 8.9470242778656708, 11.141084995314072, 2.8976432363329452], [-14.315756190769065, -11.601968553642076, -5.466953683798411, 13.614850682847431, 5.8365210966072008], [-14.197532213029881, -7.3402064265942117, -2.3233284531188456, 15.077991582429734, 6.0652786535910694], [-11.814670256518871, 5.5640308785569861, -14.775409883418446, -1.720382892425516, -10.150947656754077]], [[-2.6312452086302476, 1.4581196006434027, 4.4032049249589242, 4.8551594793723734, 1.7062109629527398], [-1.1065712797538614, 1.831969424185619, -2.8584568798452183, -5.0999833676168018, -4.8213180427002067], [-4.1394237888627066, 3.3315881968082341, -3.0931469996469376, 1.4972641932090534, -4.0571686065025636], [0.44282672643377174, -2.2809100700462821, 2.5534928902724694, -3.6758757226009759, 0.096540700065851304]], [[10.30827137054241, 9.8542252821623961, -21.187526413435183, -10.207415076507788, 10.372638658339298], [14.53856884763427, 8.4025416313222046, 14.523950679207141, 22.89836325596313, -18.144510126365059], [-20.226876953963384, -18.271195357303853, -12.152127004986063, -19.224526776848005, 1.8723247141554766], [-10.395240849382972, -10.67971895981103, 19.020304147211284, 14.813715632458493, -0.1321863161509591]]], [[[10.976645297047217, 7.4968174521007009, -1.3242512807156794, -6.0403193480579667, 10.744170211301013], [9.1994631264077107, 16.377390539340816, 6.7605828800995358, 6.3171695946550992, 6.3382301004343846], [0.95897438001582791, 11.270060699363023, 2.342659168735961, -4.6004619788119943, 8.1218783780782822], [8.9771585174087996, -2.1809639027175902, -1.6565923672905907, 4.3471604454622934, 12.328761026070181]], [[20.907990774845622, -18.285381841352216, -9.705755952881864, -6.4834524147540096, 2.2547258279760518], [-2.2067681411175699, -13.964484842986918, -10.494580091607054, -13.717274665899801, -11.733404567227147], [11.142767487082979, -14.823509723963381, 12.520896183304471, -19.392151835517243, 13.564757949466053], [19.06793812561148, -13.58826950199022, -5.8407167720346553, 19.684810063591723, 20.74364561159534]], [[2.4910526873596011, -3.4047218103351473, -6.2482011198033192, -4.3134546657044055, 2.623918721280273], [9.053965435849225, 1.6631468949298713, -1.2545179677766627, -7.2413651857807393, 7.8743547147142969], [-2.8699096738174186, -0.32229346890974464, 0.38650623132090872, -3.0723323527309292, 4.4438149548467631], [3.4318750405498659, 6.6395044159286689, -1.5884483891660575, 5.4452180050267742, 5.028615892125476]], [[-0.39363398894627782, 2.5825795802294786, 2.9817217582638547, -7.4336457930150388, 3.1186161569405866], [6.8443728455578405, -2.9235927409847364, 6.8952993856072791, 0.7454040263237891, -1.558544718771341], [-6.7290438506131309, 4.0861265229151664, -1.471513288577347, -5.9680188239881984, 3.7889563314310619], [-3.5354146642450206, -0.10966718963970147, 1.7798869300055533, 3.0329518571079928, -6.3696155508828687]], [[-0.9005629521062406, -0.71214112607876168, 5.0923867508146321, 3.3376036030723673, -5.9400508143012374], [3.4021162382020758, 5.8857276177217761, 1.582317999793488, -5.4185882160262437, -2.5605320384763415], [-0.21538737369902777, -1.9630505733308945, -0.22494598568993529, 6.6683378220112317, -1.9573682093578624], [6.8015489123156199, 7.0983692648505698, 7.2498283767739977, -7.582431519229389, -2.6913810104114937]]], [[[-6.458214225342636, 1.5662963711474782, -5.6944620751575608, -5.2678230127275159, -0.11285091727114469], [5.8243437524244763, 4.9785730964701482, 3.1165097770592616, 0.25955285134047495, -6.1651494696132021], [4.3826799077603971, 1.1046813629559784, 1.9064793909389948, 5.7607936771211152, 4.7776914696674613], [0.46104229130778795, 6.7763434407122993, 4.8945865393579755, 1.860595887135347, -0.79223361814360982]], [[20.02985599055069, -3.2708406530523244, -8.4750489582146251, 2.6038140386433715, 11.76418607644068], [4.9549545928403562, 15.380027079269759, 9.3187758335571242, 14.585923209073908, -17.556949723687595], [16.195812911701505, -0.92349868885454711, 16.685829493116202, -10.492955850781103, -2.2518934437666229], [19.573334398868568, 10.605878492645131, 5.8548653244801665, -11.454151640906669, 20.197547648735618]], [[-13.581581715445264, -7.6110229135222305, -8.1288315891725187, 14.835187512998647, 6.6480293679321889], [11.624054868520679, -3.0860507533507682, -13.095168444528941, 12.04318345179211, 9.6930637655990264], [5.525062893052902, 2.8101502189203784, 8.4160165888567207, 7.9270106903976876, 13.339158431846441], [10.315444031800327, 12.445491473704035, -8.8457742304915321, 6.7866072284601913, -2.8311048003017256]], [[10.776940871083383, -5.5300542585946868, 0.3318643921368154, -9.2019459711678984, 9.9932540664627894], [-4.3530717826069676, 9.054536207975973, -5.4805587318366955, 8.0345844478774904, -0.098630063737797494], [8.5118457395577121, -5.3958462261286195, -3.1714937090104995, -5.5313199761589855, 6.7539534300866251], [-2.1774839271790789, -6.4173449778571037, -5.7149438711730802, 5.0985279820329028, -8.2409943763792182]], [[-0.94829594801584494, 2.2068137565632719, -2.1457647001066298, 1.5892166011433382, 2.1335641569691397], [2.1372367415614084, -0.24817962104602515, -0.84995320419623088, 2.2810738662902974, 0.73955199364857038], [-2.0866847778541171, 2.5927003043926211, -2.571189458813969, 0.16069059166506316, 2.1164049963461316], [0.35579957523195954, -0.19671415503556938, 0.45021642071635043, 1.0180590337783402, -1.7847736870107127]]], [[[-5.5292800418385708, 8.8209233382162999, -3.5709670555344593, -2.9723518869762562, -7.6306459818226458], [5.7118091406614093, -7.7523634317983019, 0.4813540937804987, 5.7587973678977029, 1.8223162013529952], [-4.6895613329922794, -9.6115785156761309, 10.142004057591626, -9.7299205956954786, -7.3478716584780299], [8.6982016910652149, 10.415570303976191, 2.308266827374605, -6.9110432380286086, 9.8871704994157348]], [[-11.148550324801201, 8.6579262762994915, -3.6259978425236596, 5.0345741136089197, -0.76134615884052304], [5.2152027685978464, -0.18954666023220884, 5.1111243420930821, 4.2710215222411412, -10.122779046732013], [11.63429863929367, 11.553542106434913, -9.590868269580648, -8.8302347147907376, 2.134547333165298], [3.0447585477393231, -11.585333934509485, -5.3453241740622603, 11.278969882672449, -1.4246172413965856]], [[2.1528497355232297, -0.087788325548183013, 0.72086238868486885, -1.5700557341487449, 1.4629900031660747], [-3.2275562064876557, 0.82061865327424277, -2.1237052663586224, 3.0290805290133966, -1.3695561200332176], [-1.8739311201533511, 2.2047310419071771, 0.049926442952345489, -1.7507331965948176, -3.0831264795090396], [-2.6398859019652852, 0.36672669314220974, -1.0253172742914349, 0.6966871640833604, 1.3559468257187006]], [[5.1836432830777603, -0.54706634985957403, -3.7359698433153654, 5.4809412588740063, -0.55579407926887348], [-6.3462148770514695, 5.425750511196048, 7.5922350761094473, 3.2795808730314531, 4.9812162052085869], [-5.8500107700815152, -7.4829379249276489, 0.18144377425960209, 0.85933147150777567, 1.0102786644289308], [4.7064958735641325, 5.2798005435154893, 6.1739591430451854, -6.6999648612101916, -0.351712000725868]], [[-11.448731823105891, 5.966905994116015, 14.164957548786067, 8.9286438766071612, -12.478453528304522], [20.94798427695633, -15.827272553556613, -16.390291597385563, 1.0060572434613479, 7.5399479667352018], [-1.7284632541704059, 8.8137143553533122, 14.870696824423623, 9.7594146117332308, 16.28584413629715], [-14.973018249977695, 15.182250100636024, 16.165244177798272, -5.5837612239243395, -11.352593391280882]]]])+(1.-msk_ref)*numpy.array([[[[-1.9296138771220346, -2.1666802738283044, -1.1858489775603061, -2.6772233803908851, -0.5201567077960908], [2.7970274196922866, -1.6610365989258709, -2.2991713895916193, 1.1666081301464331, -1.4814937763626459], [-0.80113087200007227, -1.0073843172202213, -1.3099859604301674, 0.21722001659347512, -2.0188622481657745], [-3.0109118087985531, -1.6652778155976713, 1.9435150374375547, -2.9766758025423186, -1.9770592771230251]], [[11.653283051955011, 17.794635994858687, -4.3009721890686556, -4.310605319413094, 7.5403332895556314], [-13.923772848750026, 8.250974367437145, 8.7857429071072701, -9.1048209139449234, 13.090010768470757], [0.95658481595316058, -11.380412447034409, -5.1357778497747129, 12.515168409937862, -11.985826950898305], [-2.7641482256902794, 5.0371677061444089, 5.3213029738101998, -15.561228316089904, -10.712820790736792]], [[-9.8266592465853293, -2.7956196423144855, -5.6695255494932422, -7.0598518645214829, -1.8361705357554823], [9.0715687096853355, 7.3519032805153275, 3.4642840597706761, -8.627420849846267, -3.6984705137365057], [8.9966528671058956, 4.6513216664767816, 1.4722403355279767, -9.5545799202885409, -3.8434289684898095], [7.4866875061336788, -3.5257996675150833, 9.3628407878042808, 1.090167463571958, 6.4324243797921952]], [[-10.417870540823703, 5.7731226199356298, 17.433578110653972, 19.222998580768969, 6.7553889957025328], [-4.3812398399275567, 7.2533035816332578, -11.317468103345817, -20.192328069585134, -19.08901041995075], [-16.389191324523782, 13.190733579616664, -12.246675034454112, 5.9281075283678346, -16.063518962887102], [1.7532807253662794, -9.0307911049359166, 10.11002633677785, -14.553868744961608, 0.38223291083163385]], [[4.9932084796320151, 4.7732737595273829, -10.26299490449145, -4.9443548470009366, 5.024387255945256], [7.0423160821290507, 4.0700948409133106, 7.0352352089231305, 11.091704658271233, -8.7889926996525816], [-9.7976673190442973, -8.8503575732215296, -5.8863510113111035, -9.3121403840204859, 0.90693262752683868], [-5.0353354981791627, -5.1731334336621444, 9.2132172833977304, 7.1755940304622543, -0.064029536182219945]]], [[[-13.089992362073952, -8.9401889677768231, 1.5792110140151798, 7.2032694863371391, -12.812758561177706], [-10.970647114947935, -19.530549750782505, -8.0622062450607554, -7.5334220525651503, -7.5585373635123316], [-1.1436069008450709, -13.43989939389377, -2.7936942295061145, 5.4861946009005287, -9.6855936452039444], [-10.705541925238256, 2.6008675743773964, 1.9755381401216612, -5.1841246107423196, -14.702432601015792]], [[-15.877592193158716, 13.885974941335498, 7.3705807796501999, 4.9235536094201464, -1.7122456954140404], [1.6758264812166492, 10.604672534639064, 7.9696162451654509, 10.416940369456073, 8.9103833439564042], [-8.461851737372303, 11.257018658702327, -9.5084068876940524, 14.726459462694615, -10.301118708147129], [-14.480250578983609, 10.318973453102149, 4.4354581949442897, -14.94870501691255, -15.75278797313077]], [[0.019119176970612127, -0.026131714980504383, -0.047955815452510167, -0.033106366447724203, 0.020138942320740177], [0.069490448087346238, 0.01276488448985503, -0.009628600454917733, -0.055578488242463031, 0.060436771202769621], [-0.022026957206411766, -0.002473647345889461, 0.0029664892574799442, -0.02358057951261637, 0.034106899857033461], [0.026340119811294711, 0.050959122851734366, -0.012191563034879125, 0.041792808000388024, 0.038595328652276169]], [[-1.0068209249966187, 6.605616473324643, 7.6265260191936122, -19.01347531867242, 7.9766689158258783], [17.506257063403027, -7.4778459951368133, 17.636514885642178, 1.9065639460873538, -3.9863819676814387], [-17.21127327483336, 10.451327377667413, -3.763776533414291, -15.264754572769087, 9.6912376103371791], [-9.042739098903315, -0.28050225441760068, 4.5525220270095996, 7.7575602717082317, -16.291942263369883]], [[-2.6150835511694819, -2.0679382163837703, 14.78743578902824, 9.6918402675850661, -17.248909852731227], [9.8791741242307669, 17.091164414181211, 4.5947856996577778, -15.73467005404788, -7.4353549636938725], [-0.62544875599487659, -5.7003691440171886, -0.65320535971810501, 19.363732997452562, -5.6838685134695073], [19.75055562908938, 20.612472077851866, 21.052283899267955, -22.018107559755116, -7.8153184003362082]]], [[[8.7112109742604567, -2.1127106753044185, 7.6810181252528409, 7.1055428076170859, 0.15221981103229795], [-7.8562100053739865, -6.7153858761672911, -4.2037277216316404, -0.35009982142204044, 8.3159083678121917], [-5.9116108535603189, -1.4900577893934095, -2.5715691076611895, -7.7704900069221932, -6.4444251785391442], [-0.62188037242222483, -9.1403219661638033, -6.602099975068664, -2.5096787974414703, 1.0686102919084475]], [[15.904157250116793, -2.5971212229763347, -6.7293799514820227, 2.0674870523373805, 9.3410289803188462], [3.9343456612418133, 12.212088259404188, 7.3993181131905867, 11.581551882583543, -13.940613920022754], [12.859840603082183, -0.73327878017437442, 13.248924816651197, -8.3316435199576055, -1.788054165594964], [15.541668813574296, 8.4213066435542707, 4.6488950715730306, -9.0948546484609309, 16.037308207495823]], [[0.88253926517628212, 0.49456879986968005, 0.5282163158722244, -0.96399931619046686, -0.43199290599363493], [-0.75533800531985384, 0.20053341512215822, 0.85093184126347676, -0.78257323017397296, -0.6298610539058378], [-0.35902187593823953, -0.18260523414740221, -0.54687776811697419, -0.51510187372301586, -0.86678645589003955], [-0.67030369411532598, -0.80871544493743464, 0.57480367551117373, -0.4409977778680943, 0.18396687532011921]], [[20.530106789842559, -10.534771030172625, 0.63220272726893878, -17.529736473672148, 19.037180923298962], [-8.2926156531642246, 17.24892040384329, -10.440481893569157, 15.305909042289377, -0.18789058652619325], [16.215093327705997, -10.279104299553731, -6.0417056480290343, -10.537182226176357, 12.86630286200714], [-4.1481138379539289, -12.225062685105161, -10.886986332824224, 9.7127120946719447, -15.69912061552855]], [[6.3515175345316859, -14.780845894770351, 14.371950176759231, -10.644290033560392, -14.290233109602381], [-14.314831427757859, 1.6622629444799857, 5.6928353340317646, -15.278226897013726, -4.9533876688865437], [13.97624336928974, -17.365445333380478, 17.221369517005559, -1.076276992146197, -14.175304104788095], [-2.3830822493685875, 1.3175564101198995, -3.0154694813334944, -6.8187782703923041, 11.954096600262915]]], [[[6.0533692043908109, -9.6570087399833806, 3.9094388130644671, 3.2540843005036368, 8.3539117292772236], [-6.2531992034790793, 8.4871660874837964, -0.52697892413664571, -6.3046411788481755, -1.9950432407949876], [5.1340583114683511, 10.522605647051234, -11.103307224174811, 10.652164703086036, 8.0443348281663205], [-9.5226550024343091, -11.402803266823152, -2.527054376450093, 7.5661019139455492, -10.824319435233967]], [[-5.9776933582410114, 4.6422563373860601, -1.9442082233805069, 2.6994666897221742, -0.40822292983679515], [2.7963171931309723, -0.10163221037144531, 2.7405118282424388, 2.2900607022967847, -5.4276894584202591], [6.2381446625562358, 6.1948441637433582, -5.1424865013433063, -4.7346456596144968, 1.1445137748472956], [1.6325560201113816, -6.2118904876142684, -2.8660864311528207, 6.0476224613224554, -0.76385940537823738]], [[-15.49852009202969, 0.63199446988968766, -5.1895401849333593, 11.302944158058812, -10.532170259898443], [23.235409275904065, -5.907692708787275, 15.288706962266863, -21.806568598193504, 9.8595330148939766], [13.490564918465036, -15.872017348305652, -0.35942405382623666, 12.603654205625986, 22.195649283123881], [19.004728484838285, -2.6400918411366829, 7.3813328046552158, -5.0155010041984589, -9.7615587262608585]], [[5.3181438371054126, -0.56126113972603831, -3.8329074577142808, 5.6231558356984106, -0.57021532847616219], [-6.5108808022181313, 5.5665330477093224, 7.7892316224147296, 3.3646765133586398, 5.1104643619093464], [-6.0018016335100111, -7.6770985262064695, 0.18615171553373522, 0.88162863832643223, 1.0364924744207573], [4.8286158320477304, 5.4167961004004708, 6.3341555300138932, -6.8738095755524027, -0.36083791012442445]], [[-3.0828676760395886, 1.606744039379268, 3.8142818291448797, 2.4042687018406923, -3.3601469248963438], [5.6407875215732437, -4.261903214184227, -4.4135106793632044, 0.27090697939600028, 2.0303263474785807], [-0.46543351506844521, 2.3733209504590831, 4.0043204145700013, 2.6279752472548199, 4.3853854942676813], [-4.0318731095130609, 4.0882142064274536, 4.3529108307387876, -1.5035723828571317, -3.0569799123573005]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_expandedData_rank4(self):
      arg0=Data(numpy.array([[-0.77250494109964674, 1.5257736145655301, 1.6199440196719053, -1.2162235025283819, 3.151557323822944], [2.8968248339744074, -2.9155810149552428, -2.063365779769617, -2.858178512476055, -1.4434045125587547], [-4.8377351668694093, -2.3881526346646975, -0.45671832350298658, -2.6376293329091958, -0.31663490477729184], [3.6960449782465048, 2.7592753230760101, 2.6847645668614319, -4.1506078818181162, -0.66196989097319126]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.74471610175301, 0.65066667380862953, -4.657567131365969, -1.8523206903717027, 0.86715422985092072], [1.3382690403379911, 4.771853672539244, 3.817150403232187, -3.4332656565787723, -2.5565821046551318], [0.22866584467532736, -2.5226736551758009, 4.1354630860463235, -0.74037426141502127, -3.5889841532762032], [2.2898566980328976, -1.3714845478295556, 3.6591799763312434, 2.3277019061838686, 0.32240976981914038]], [[-3.8374568616691809, -4.7050443601058198, 2.4696878366957176, 3.9607763376287402, 2.665567497042046], [-2.9107075533967954, 0.051596358821390353, -2.7820912450215438, 3.6421157062995473, -0.90255669193266463], [-1.5908910169387047, -2.043525970841531, 2.275369075980235, -4.0829360340303511, -0.29647588217905341], [-2.6716229195637098, -1.4730233512407231, 2.6967360327474568, 1.38836391341433, -0.49742631901287115]], [[1.4608750150726335, 4.2966779879032195, 3.9294722682684515, -1.8647638242751841, 0.11651233814877759], [-3.5876185492099397, -4.2187408770376535, 2.3104985043143564, -4.3407131328097659, 4.813588346302959], [-0.19168535410504983, 2.0276720485112669, -4.2016048033495279, -0.73698668522732103, 2.2956666528056484], [2.7919770790079346, 2.5302534634309382, -3.1499839972764199, 2.636239701085934, 1.513973616126302]], [[1.4206752299598238, -3.7973883459811955, 1.0879151348595322, -4.7130672489861141, -1.9967507107492533], [-3.4814453152051108, -1.7839103336521358, -4.6383055704896368, 0.57800906450286238, -4.6020840892870378], [2.5725432647867619, -0.32164627943383817, -4.7615059671081479, 3.7798204874083581, 0.59086820338726564], [4.0431788680988046, -1.493239949827637, -4.0608571369449509, 1.9757597964546711, 2.6938299509446759]], [[-1.7702580335076625, -1.6444394831267406, -1.7002799519819609, 3.84406367274911, 0.40739241349528754], [2.875308130617654, 4.9961626726142665, 3.5282298216308146, -0.66969246086410106, 3.3041994694216967], [-1.8006690950219606, -1.9716704124249782, 4.263415582359471, -1.1640497477838352, -2.2362809443964728], [2.615854739374158, -3.7786685715913402, 2.2622015903364376, 1.7653462350172884, -2.0757073307617691]]], [[[-3.3948440412303036, 4.9044119634825343, 1.8795779172233313, -2.8945539759510241, 4.0106155154834813], [-0.57156329029139386, -4.2737953009475991, 3.76767601924875, -1.722165475174271, -2.8823220118908575], [4.3476258309264075, -4.5328519670674607, 3.135983558487105, -0.41050907280000892, 3.4956654346303502], [1.547374767402232, 2.4672658430562713, -2.2449474577051296, 4.3604154307255847, 4.5228781717635371]], [[-3.5000093466745641, 0.29956642157818347, -1.8076666003361908, 4.3168355307291968, -4.2619027293201706], [2.5212052427206828, -3.4002909851514742, -1.349785413050455, 2.2161435495904547, -4.289463590223356], [3.5534558800006071, 0.14304671720567708, -1.9179715289064503, -3.1395604214330097, 4.9066026750328344], [0.49941592944253355, -4.8602777254841154, -4.8231280987134815, -3.9726519729966272, -4.8539055011729024]], [[3.3538577807679761, 1.5743221912909675, 1.9868080084637993, -0.39565190266443029, -3.4863634947618438], [1.8933968336130942, 1.2042464205061387, -2.088286384432585, 3.9329031494251918, -0.31877173277635684], [-3.1169325586685126, -3.1950325225901621, -2.9917346470297579, -0.96144173593348725, -4.8867592583897812], [-2.1286496513347619, -1.3688934167712441, 3.8081774138006512, -2.0196334062384813, 3.0522211972718178]], [[-2.1609146302102147, 1.0733291372595506, 1.6331065145194659, 1.9357913477267461, 4.5053507017527501], [0.63445757909198441, 0.38644547367331672, -0.26263758510031998, 1.7970338276338929, 2.1480553639963649], [-3.9298347997966232, -2.8214203899890409, 1.992379524678304, -3.1636874309551031, -4.7908595271690828], [-0.99120471152883827, 1.0231576657920796, 1.2434456950302186, -4.1281437437458637, 4.7519756270151596]], [[-2.7132159858927629, 3.6192741595747062, 1.6871950469513575, -2.9296648110133869, 4.2073209849203508], [-3.9138824749154399, 2.423239689917482, 3.7972992222860906, 1.5077596528335189, -0.97561887165792971], [-0.031501700362098539, -1.532380359580614, 4.7743718147194674, 0.18220177324577413, 3.1143182111048748], [-4.459473072975582, -3.8315574438482636, 3.7678940654767157, -1.3503522179428904, -4.9758469481346523]]], [[[2.2648675065272013, -2.1831173998020392, 1.9307535342203694, -1.6690067732164793, 2.7082224397261907], [4.1320561694470719, -2.8341483399807501, -4.6406398969235019, 3.4339246132000696, -3.9825734485455175], [0.79086813618698493, -0.54213106090930108, -4.0449109667922656, 4.5596069925518918, 2.1470284631354852], [-2.6716318346429646, 2.8710503131133871, -0.46494254531630119, 0.43882986599291218, -1.4156480970825225]], [[1.265068646457407, 1.8130187588942022, -0.052383536758472182, 3.461261302897686, -4.2386214905681738], [-1.5362458268317214, 4.0159617813433783, -4.3724705393370202, 0.14499141160302287, 3.5834013306576153], [-4.4474290061632118, -1.2030677041916849, 3.9559992820413417, -2.3384832571615566, -1.2880072938294362], [-1.4644579103503896, -4.3794428775321839, -0.656075087211482, 4.7037553445641329, -2.992953014093799]], [[0.20693507968023628, -2.525082651170294, 3.3415682071810728, -3.4842869653529238, -2.6820800419715853], [-4.7932132247381354, 1.7823469023144058, -1.7799828500421109, 3.8052010336536561, 2.9692935746461782], [-1.7603284490846152, -1.6841417562361638, 1.661725239068339, 1.5083986891081747, 0.032595619450654922], [3.7118055736715565, -4.235567332292157, -2.1827491827054923, 3.9591813899721018, -0.67959581999793439]], [[2.5746062745512699, -3.5158537439916016, 1.2023678747337216, -0.05785027429315992, 1.0910857638291915], [-2.4672103955817848, 4.734124510328968, -2.7393869228661405, -2.6524449845975226, 4.8565483394833215], [3.5901482104283531, 3.1128268165872157, 0.69281353810371282, -1.2184450072067845, 3.4680974074616593], [2.140607001976548, -1.5867624094081889, 0.63899888052468601, 4.8131236307220888, -2.5439732417130658]], [[-4.2701414167960641, -0.2498010446437533, 0.40277406090172896, -3.3970360486075748, 0.17735508028567182], [1.2165135781381053, -2.0562084946284118, -4.7188731382307845, 1.359799913982771, 0.0854935750288357], [2.6156957810912704, -4.6486510591087722, 0.25193524433557357, -0.11319378546657344, 4.0663297955818294], [-2.8481732719758357, -3.8120439805801709, -2.7819449309258424, 0.89167761114115951, 4.4585162443181616]]], [[[2.8764215635600774, 1.1114544590975317, -4.8494645633861202, 3.2481058391930464, -4.7417546438599576], [-4.52841047664461, 0.79048445377744514, -2.5814716226314092, 4.378261668539233, -0.5387225218998557], [-1.6412498817158605, 4.1866249514701206, 2.7517946825230402, 3.8563375002187374, -3.1823803936762083], [1.1361327364852514, -4.1859834669194216, 3.0825473711243916, 1.2710422331775719, -2.7563981688330097]], [[2.7213938316685438, 0.70918302382111875, -1.7790144909126147, 4.010912538048375, 3.8535175887935971], [3.2802651739234641, -1.4038041690542293, -2.5630026053095492, 3.8546578796780118, 4.5841842821024059], [-3.2068221057887971, 3.3990311523980967, -4.3010673696253559, 4.3838870196238702, 1.0517930813806515], [3.6725013168195186, -2.068618648703163, -1.59542076084462, 4.4793508516338729, -3.1423547342296185]], [[-3.6247610354080493, 1.9068925172679387, -2.6733583278919717, -0.17489935351002117, 0.18198620527655152], [-0.52995661514797643, 2.7148893826766853, -1.5272938988560956, -3.1903569607485158, -0.22104160111813087], [-0.5299559228400339, -4.1151086827216936, 1.2689015208160139, -0.0058278792195327611, 1.6975669243091982], [-3.8770807566073975, -1.9915827169549729, -3.4505849334417182, -1.0974394395016707, 4.6716353674240469]], [[2.9160476328489224, -0.92690724831455285, -4.0452116944167003, -3.0893302824072597, 1.7551162466568515], [-0.63969521519344141, 2.6161836390951718, -3.488569218361147, 4.2010189925205275, -2.7944095108322897], [1.9409538488087188, 2.1793851625735812, 0.67977625292035437, 1.888859658770321, 2.4553287769170762], [-3.0462511105049215, -2.0303095183689912, -1.7371893457167475, 4.4283778907495517, -4.4454468505283966]], [[-1.7679601371403599, 3.3089371222387243, 4.8390542936271963, 2.5283721948690721, 1.1174561481530629], [1.8706359908024215, -2.1173151084029005, -3.6293793786102526, 1.0718654397236795, -2.6586287378019158], [1.3850862300219351, 3.9479379282216307, 0.76970347159068453, -3.3664841039891202, -4.221341035096132], [3.0657509161345171, 1.7363510934467392, 1.5961618687228052, -1.1245357868013315, 0.23974937831819609]]]])+(1.-msk_arg1)*numpy.array([[[[0.55539863381000121, 4.243019486539449, 3.9526907793396422, -1.4579251527763892, 3.1784694773469209], [-3.7014135305983018, 3.1212224817625636, -3.4496273986266202, 1.8680255717596932, 3.3324652304943534], [4.1608386641981383, -2.4346565764921491, -4.2521566845554046, -2.6301430047305385, -3.9999707896327563], [1.5100715044046895, 1.064463292900979, -0.014859372341023303, -1.7263305087228566, 2.6172944943780063]], [[3.4292770895304159, -3.7798330488999001, 3.5036920161957923, 2.0460533028783345, 4.8670034938541242], [3.4427635858710008, 3.2985096888136063, -0.72511281741685352, -4.4597025280767308, 2.2043082994782024], [-3.2647514131070068, -4.787308177868999, -1.4221829097915863, 3.7240314772628498, 0.71404423256289107], [3.981443036414186, 1.2199860353716998, -1.6263004589715466, -2.0609944111705181, 1.5879406973002519]], [[1.4518048113651325, 2.2166655093073038, 1.4769030048905361, 0.37108491984581438, 3.078793507709614], [-0.43456315845609961, 0.042506087712839147, 3.2016643565945895, 3.5837805130445055, -4.1159991866107806], [2.2801507332466713, -4.5331428724441718, 0.12764943346834734, -4.6028724056927164, 4.2316987921138818], [-3.3674636593214355, -1.7327884663977047, -2.4755690686391061, -0.51269652950261513, -4.1475341075836791]], [[-1.2654298310539858, 2.1153483748949062, -2.3857045192523598, 3.2499779983317651, 2.6192616129751025], [4.6382537199565625, 4.1944969469638966, 0.8977735121917938, 1.7395716084695749, -4.9612086468722101], [1.1415962434005849, -3.7111413245839153, 2.0334065605883023, 3.682897743916465, 4.6912857936371495], [0.94421363942424996, 1.8705649832176103, 1.2074279802975383, 4.9301404360542787, -2.8400539691748863]], [[-4.0628540080619509, -2.8836492353661991, -2.0022627116065506, -4.3079525725426233, -1.7081163211217354], [-2.073648812241029, -4.4996922262632193, -3.7460613781083563, -1.8967712170959885, 4.0818204700106602], [-3.9915353674616205, 3.1250006392650658, -4.1137114848371219, -3.0136341658457297, -0.42604633194162744], [-2.3973808580693734, -4.5777182832851482, 2.7437006751872426, -0.422507626386178, 0.043930555331281873]]], [[[3.3158862777649958, 2.1243602578851277, 3.2802129656421322, 1.1638016262838615, -0.038537238454870781], [-2.4909690758022771, 4.0575348417333199, -3.7142443872548792, 1.1679455371821534, -1.6446604977998636], [4.3970609280478072, 3.2238692068810675, 0.82106720350381757, 4.8665663021502787, 2.7854246426535409], [1.7489776838115088, 0.38237581541682708, -0.53002507840250423, -3.6299997814217342, 2.8694128737053219]], [[-0.1651016631166593, -1.3997410519463194, -0.36155509639154015, -1.4793517934258626, 0.72344285329698277], [-1.7356700423228286, 0.65952850643130834, 0.71351976980681719, -3.4508153516334286, -2.6103215021069373], [-0.67376431684544613, -2.0873691381166948, 2.6961555904913981, 1.0185783938579611, -4.1174696702724773], [3.647701287498391, -0.39154941444687452, -0.17426942417135649, 0.4954236072543825, 3.8166269539234623]], [[-3.029790810200649, 0.091011755349575729, 4.0615816847349269, -3.0031138651723897, -4.0518183066877054], [3.329199096288356, -4.2437310542684052, -1.7087826602104639, 2.4493140104943034, -4.4479869671234962], [-0.79163500228060268, 2.814685601723367, 4.8353489792098578, -2.623235948438869, 2.7237937329257464], [-2.4778485082121016, 2.6065311704385685, 3.4459002377302657, -1.2432213426628547, -4.9170485668707045]], [[2.7849228024368653, -3.441540498830018, -4.6563448016143196, -2.1553632489266814, -4.1119152566286887], [2.7891512333764421, -3.5127973617228783, -2.9227054094105287, 3.5715748691683338, -3.4039638787383728], [2.1556742200084429, 2.969716026098645, -1.5672145949776874, -0.11300336676700606, 2.1359317579302415], [-0.444955728065346, 0.94939203877716682, -4.2311729389297259, 2.1771664416482199, 0.68525952825151482]], [[4.2594618686914085, -1.1778874870426934, -1.0251262960987617, -2.5523339438501602, 3.8740168424399073], [-2.2538481657851248, -4.9306130380627806, -3.1741942583209406, 4.2871181842689374, 3.2359283215283696], [0.73881703744763172, -4.4594778214685018, -1.5194563413007725, 4.4014445852366411, 2.0895414818738125], [0.18896680497884599, 0.46936342055458358, -1.8659176147859258, 4.7622971423342051, 0.3572619442313627]]], [[[-4.1113071971950053, -2.6343819331999887, -4.6328312488838499, -3.6063084440464745, -4.6569990192715824], [-3.7659542080810207, -1.8466101114797495, 4.1846261109486012, 2.7431171852340377, -3.5870307480614083], [1.5424868524905477, -4.7423818422252246, 0.41196687706566504, 1.7433674783062845, -2.1721291939330478], [-1.7773719517630369, 0.4305314834680809, 2.7989441144275462, -4.8643351317056807, 4.4916980295524063]], [[1.9734074841524238, -3.5089680442299431, -0.83417855563935461, -3.273295953476544, 3.5722429083326084], [-0.38133182354184303, -0.76862891695431923, 4.1583094802762322, 0.75188258565571431, 2.5749814010915983], [0.88079632664996499, 0.5402134257520359, -4.3739343719692059, 1.7645366350157774, -2.5411860355189306], [-2.1667663902301171, 0.70125887366701711, -0.8849261112438489, 1.1733470253549747, 1.9483297669224084]], [[0.81506411709464377, -3.9156531331721736, 2.5392207667970377, 1.4936435992523469, -4.7393264837186946], [-1.8321840843017476, -0.97769125087224662, -3.1852873535387225, 0.24788630035113357, -0.73384128821840644], [-0.93529619035723854, 1.9787709539210319, -4.3016022462016243, -2.1982508674297927, 0.35742607046572772], [0.0084503161180116138, -3.1392869626217337, 2.7295794971365082, 3.6367563994743097, -3.7148966422175334]], [[2.5456245410752114, -3.2551499699230546, -0.50012376791539825, 1.5562754223135089, -4.0007993769799981], [-1.5366991282969469, -1.0220226356001474, -3.2487746698699196, 0.38294203474713129, -2.5148806469678298], [2.5138823764881675, -3.5927699021978912, 1.9363621853305899, 0.65101154622516333, 4.6455929042086375], [-1.7430382950445757, 1.4401386098000026, 0.73166272634821006, -2.6271098880338153, -2.2460325135742885]], [[1.9443016586607236, 4.4345416842273959, -3.073947164436448, 0.54235636420872702, -0.039284939571194499], [0.056452590205444686, 4.6134356514111801, -4.9541195097547419, 3.1623099032217112, -4.188628475288338], [-3.0526405530967304, 2.3335338537489658, 1.6029579463913386, 4.1468568072447916, -0.015891829281647674], [-3.1873074063905493, -0.23143856665104501, 2.2649066471954393, 1.3673453273368574, 3.8248612140916514]]], [[[0.37320820685280953, -4.8642844367428681, 4.2011868495852074, -1.0660051273577507, 2.6157436520120552], [-1.5898021672950313, 3.9919767255719165, -0.57149998992084861, 1.6744826494782021, 4.8944282775321106], [-0.19815804735416886, -4.6167000640112255, -3.0875904467664741, -0.73237897670255592, -4.609027410208121], [-1.9326948789336216, -0.3273870396511871, 0.1483043423125423, -0.613369104767159, 3.7525323358106171]], [[-3.099871810827588, -3.3169783156581847, 4.9300557382652741, -2.4458199310485518, 4.4197062428559857], [2.3051718772511132, -3.9377036944786448, -4.553763746094929, 2.6446039735890556, 0.27651075215843512], [-0.79333737914480817, -4.4560459040187039, -4.2226054638722497, -2.3332276406660188, 2.5736284425467453], [1.0714104010621242, -1.3668189152551546, -1.1431902780421268, 2.6606790746469065, -0.9366485745767239]], [[3.5873409429570859, 3.7878912297434262, -3.5185774356482344, 3.8741940807960873, 2.0319874637697755], [-0.77992699920113395, 0.63756690254855819, 1.621365636064219, 0.52420948042741955, -2.9820798178710026], [-3.3493165089236854, 3.8901176030739393, -2.0049831572970742, 2.2342958272013629, 4.7445856806985116], [-0.86179170515630865, -3.3710362266437519, 0.75740409513785956, -4.9102377439137657, 1.4171572269313915]], [[4.9162860845030067, -2.7724233751073091, 1.0713614391837343, -2.6212217528643533, 1.3630082218160302], [-1.8594640861156564, -1.958753578659731, -2.8480804176171448, -0.94532646786290453, 0.2915770517357803], [-0.39637868811456034, 2.5783189712871764, 2.6423437901773514, 4.8223183771417624, 1.5973224394219301], [0.49538113333041256, 0.94705495885455093, -1.4457041091762202, 3.2449300434466455, -4.554023772661278]], [[0.73065842841331019, 0.056293814803610509, -3.5218480645243266, 1.6030087681550773, -4.2462661818565266], [-4.7141913431540328, 2.8457724308090366, -1.3459411110396271, 3.7906049166861262, 2.6698526769555393], [3.802117240334308, 1.480638670116484, -1.2361482127599963, 1.8704892868340401, -2.8860530661976913], [-0.0022846411277486212, -4.4078780677085572, 1.6098876125705779, -3.6261515696498146, -2.7934590228301781]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.8928116916196078, -0.50264322052603838, 3.5979936224835183, 1.4309268858132491, -0.66988092725529502], [-1.0338194461817807, -3.6862805402410617, -2.9487675474173733, 2.6522146838148246, 1.9749723081730235], [-0.17664549487241474, 1.9487778634052126, -3.1946656677059786, 0.57194277520610548, 2.7725079919341988], [-1.7689256136405351, 1.0594785898401464, -2.826734612088774, -1.7981612239341049, -0.24906314024408571]], [[-5.8550904265682817, -7.1788325400098181, 3.7681845374437497, 6.0432480291494253, 4.0670525548302354], [-4.4410807846894196, 0.078724362897332839, -4.2448414149676363, 5.5570440458665491, -1.3770971862004093], [-2.4273395372943996, -3.1179580069894168, 3.4716980995289934, -6.2296360706823393, -0.45235507838383854], [-4.0762917587388356, -2.2475001629619884, 4.1146086842141951, 2.118329026502527, -0.75895995274029482]], [[2.366535744155017, 6.9603778109597352, 6.3655251014480747, -3.0208130052350959, 0.18874346540210304], [-5.8117412136566386, -6.8341240543025554, 3.7428782345249236, -7.0317122806064809, 7.7977436547558545], [-0.31051954304116697, 3.2847152088417082, -6.8063645742108196, -1.1938771733118196, 3.71885146537273], [4.5228465721999385, 4.098868966339074, -5.1027977384501391, 4.2705607381958099, 2.4525525053848516]], [[-1.7278586041370512, 4.6184729546097083, -1.3231479557724972, 5.7321431572136969, 2.4284951431034925], [4.2342156151197861, 2.1696336741909747, 5.6412162467378106, -0.70298820892282465, 5.5971628300028193], [-3.128787579904754, 0.39119376454824528, 5.7910554646260621, -4.5971065121243289, -0.71862779585631253], [-4.9174091643078661, 1.8161135218946738, 4.9389098903625648, -2.4029654997988628, -3.2762992981537926]], [[-5.5790696705574767, -5.1825452966316954, -5.3585297352180721, 12.114787021094182, 1.2839205444209787], [9.061698397295725, 15.745693061888305, 11.119418534491112, -2.1105741797452682, 10.413374037227834], [-5.6749118741980924, -6.2138323284429449, 13.436398603085852, -3.6685695079223963, -7.0477675884383935], [8.2440161619315866, -11.908690611098271, 7.1294579899887109, 5.563589856051995, -6.5417106403752276]]], [[[-9.8342685261059799, 14.207222371857389, 5.4448079880024389, -8.3850158408142867, 11.618050624775618], [-1.6557187335042329, -12.380436363108132, 10.914297458929616, -4.988811716698164, -8.3495819835565133], [12.594310475856235, -13.130878146930764, 9.0843950511608789, -1.1891728766588738, 10.126330442303139], [4.4824736536761582, 7.1472369661822093, -6.5032195464479301, 12.631359706171086, 13.101985809005379]], [[10.204560803330262, -0.8734101714714303, 5.2703984213088839, -12.586083718078285, 12.425922685191821], [-7.3507781404820474, 9.913823841631098, 3.9354087245534273, -6.4613460596014525, 12.506278607996972], [-10.360388501210846, -0.41706429293654357, 5.5920013769043271, 9.1536427600349644, -14.305597607254342], [-1.4560876024488778, 14.170533463831337, 14.062220717306202, 11.582588671493454, 14.151954727606528]], [[-6.9202353750507122, -3.2484025358416995, -4.0995116556364275, 0.81637459665852485, 7.1936431309295985], [-3.9067702340014057, -2.4848008544824189, 4.3088986639970148, -8.1150177736720934, 0.65774268496859956], [6.4313719794063626, 6.5925207723635362, 6.1730428928323366, 1.9838059771674541, 10.083171827733826], [4.3921828476826743, 2.8245278323176932, -7.8576629589278237, 4.1672424581120318, -6.2978487707381188]], [[6.1762797633619764, -3.0677662769297096, -4.6677099483842017, -5.5328372347096488, -12.877096566918626], [-1.813393019638287, -1.1045301490967048, 0.75066510230233585, -5.1362434723357913, -6.1395256849833411], [11.232169382359348, 8.0641231333284882, -5.6945763461327843, 9.0423834353464478, 13.693131756845865], [2.8330400079567521, -2.9243672552420787, -3.5539897669662248, 11.798971744786886, -13.581994628944658]], [[3.9162681975841647, -5.2240766541174253, -2.4353049443363695, 4.2286914085013141, -6.0728660954171785], [5.6493156259175734, -3.4977151034383707, -5.4810388329835931, -2.1763070867539227, 1.4082126818885363], [0.045469696455926786, 2.2118447259750655, -6.8913498219994098, -0.26299086169915736, -4.4952209594526851], [6.4368235571672114, 5.5304873045786707, -5.4385952969524434, 1.949104484922491, 7.1821599387392645]]], [[[-10.956829184626473, 10.561343818426829, -9.3404742710552799, 8.074212760532598, -13.101662936368262], [-19.989793442413802, 13.710859092449434, 22.450186826124057, -16.612417861656411, 19.266635626669029], [-3.8260105947882423, 2.6226864984131475, 19.568208030906685, -22.058171094971954, -10.386755100380118], [12.924647279380109, -13.889381065600061, 2.2492689020504444, -2.1229426749865015, 6.8485305831678787]], [[-3.0211770210689592, -4.3297655257497087, 0.12509988132280037, -8.2660202997780718, 10.122475080046792], [3.6687895188408222, -9.5907297088279204, 10.442127038511476, -0.34626162162351265, -8.5577093288709669], [10.621139298552871, 2.8731093074453811, -9.4475301081386824, 5.5846549517096546, 3.0759580122261152], [3.4973490169588404, 10.458778046342029, 1.566807448061972, -11.233285718938987, 7.147628626035754]], [[-0.094511042665514453, 1.1532515151489733, -1.52615542945462, 1.591337701419296, 1.2249551002700823], [2.1891483081947452, -0.8140304891257768, 0.81295078313530089, -1.7379050366821294, -1.3561307834005927], [0.80397425808053791, 0.76917839944955624, -0.7589403653098914, -0.68891332046358822, -0.014887016669044456], [-1.6952496187763146, 1.9344612110884911, 0.99690154735276659, -1.8082306868722824, 0.31038386356909403]], [[-6.7908570304484961, 9.273518965370867, -3.1714007753453535, 0.15258758039248141, -2.8778798153955107], [6.5075865098450159, -12.486865674088069, 7.2254873019395927, 6.9961666953023061, -12.809774356912655], [-9.4694802293172806, -8.2104833196967935, -1.8273853103389557, 3.2138062915453713, -9.1475554513072073], [-5.6461278186441559, 4.1852910754127093, -1.6854421909680504, -12.695236071310989, 6.7100584444784781]], [[1.3520758208927919, 0.079095729984042851, -0.12753232642038209, 1.0756201857758871, -0.056156808958022643], [-0.38519066097404153, 0.6510673808989258, 1.4941599467798248, -0.43056011628010432, -0.027070249988325648], [-0.82822058457219838, 1.4719251854437629, -0.079771492100238095, 0.035841103482589683, -1.287541947617117], [0.90183107276129648, 1.207026182797851, 0.88086086829937382, -0.28233625549572411, -1.4117218664676898]]], [[[10.631383475316184, 4.1079856720971177, -17.923839146687648, 12.005145275762608, -17.525738439515639], [-16.737208801641174, 2.9216660957660574, -9.5412352273126757, 16.182252053453595, -1.9911426717362544], [-6.0661333833635762, 15.473954127682656, 10.170756917504718, 14.253196852107145, -11.762221072917084], [4.199197695307773, -15.471583171930423, 11.393233731251273, 4.6978292630751879, -10.187771609963107]], [[7.5090748440942825, 1.9568312171740392, -4.9087907841698089, 11.06721198925305, 10.63291598979754], [9.0511547475526513, -3.8734822021025583, -7.0720298418101617, 10.636062366296034, 12.649026566038083], [-8.8485051019976737, 9.3788627811786807, -11.867829055894688, 12.096351272401382, 2.9021866944357093], [10.13344225746425, -5.7078883902214796, -4.4022051353217124, 12.359762268312856, -8.6706218745108607]], [[-9.7316299912034872, 5.1195574631741634, -7.1773377132482912, -0.46956358707067647, 0.48859011558405646], [-1.4228087423231075, 7.2888388175586716, -4.100424542832493, -8.5653573238573433, -0.59344465848427608], [-1.422806883639274, -11.048097980155026, 3.4067018419234181, -0.015646483628549614, 4.557567528261278], [-10.409049038199852, -5.3469307104543313, -9.264008164250237, -2.9463665214503556, 12.542241103756767]], [[-12.103370288659798, 3.8472285305687248, 16.790087542468772, 12.82259861969896, -7.2847993268809565], [2.6551240021432339, -10.858752432712022, 14.479682893997841, -17.436782542023302, 11.598498140688008], [-8.0561383431106766, -9.0457732332953622, -2.821484673244008, -7.839915787340372, -10.191106973926852], [12.643793869258916, 8.4270186894726784, 7.2103917905423884, -18.380460177014175, 18.451306736006686]], [[1.1703363792277524, -2.1904167460455137, -3.2033082431657482, -1.6737062661771278, -0.73972232456020537], [-1.2383047028820064, 1.4015988514653586, 2.4025398715589774, -0.70954264827181579, 1.7599321757009272], [-0.91688538067608882, -2.6134160399137993, -0.50952052317057228, 2.2285111152806594, 2.7944006647632449], [-2.0294347997045277, -1.1494121440201195, -1.0566110982140005, 0.74440883218432929, -0.15870686982618665]]]])+(1.-msk_ref)*numpy.array([[[[-0.42904818889821927, -3.2777535185338103, -3.0534731576788872, 1.126254384273218, -2.455383376384908], [2.8593602414402768, -2.4111597894328822, 2.6648542103917849, -1.4430589842848558, -2.5743458565996611], [-3.2142684272115156, 1.8807842352209352, 3.2848120491489419, 2.0317984669530125, 3.0899971992455599], [-1.1665376985663996, -0.82230315338520676, 0.011478938555079926, 1.3335988479594736, -2.0218729292199114]], [[5.2323005002395835, -5.7671695334742488, 5.345840831875444, 3.121814143526418, 7.4259455129208707], [5.2528778405089822, 5.0327790505805581, -1.1063580043979078, -6.8044964461506661, 3.3632754417116537], [-4.9812715642341994, -7.3043485025863042, -2.1699291588460321, 5.6820289678191491, 1.0894698496771522], [6.0747807328564321, 1.8614225029085489, -2.4813663296545974, -3.1446108923309977, 2.4228380174355135]], [[2.3518425219018448, 3.5908740354153448, 2.3925001904078904, 0.60113679669465536, 4.9874731306188771], [-0.70396798971069308, 0.068857482590063232, 5.186517027462104, 5.8055238099231588, -6.6676882673245608], [3.693716544273455, -7.3434376865342585, 0.20678493636155604, -7.4563956269147518, 6.8551151513377082], [-5.4551026163802296, -2.8070203134974139, -4.0102833080266684, -0.83053967687430197, -6.7187730739654334]], [[1.539045501328377, -2.5727364095824035, 2.9015499064028947, -3.9526996242712391, -3.1856075329707183], [-5.6411531849008671, -5.1014457681810343, -1.0918932454751105, -2.1157078745517972, 6.0339385572730135], [-1.3884361816219024, 4.5135773001632682, -2.4730768491828954, -4.4792267935599588, -5.7056520392990135], [-1.1483748196756318, -2.2750250955958657, -1.4685022872482421, -5.9961526690947382, 3.4541403857595134]], [[-12.804317304731043, -9.087985867054778, -6.310245712981212, -13.576759480678597, -5.3832265017727083], [-6.5352231012549638, -14.181037990629017, -11.80592717166766, -5.9777832208554216, 12.864091196792508], [-12.579552520621975, 9.8486186516272003, -12.964597558132988, -9.4976408266941572, -1.3427094377185371], [-7.5554832012414677, -14.426941582085503, 8.6469299572643106, -1.3315570043084075, 0.13844966339391046]]], [[[9.6055417160646002, 6.1538995513499142, 9.5022023795969677, 3.3713294528388924, -0.11163562938886319], [-7.2159010794463141, 11.753967694249498, -10.75951538044999, 3.3833336368388416, -4.7642933734833557], [12.737515292867442, 9.3389843799784522, 2.3784878654717772, 14.097590120251926, 8.068887278003066], [5.0664819885322174, 1.1076757580106789, -1.5353898097456067, -10.51547351414415, 8.3121864714754459]], [[0.48136727452046812, 4.0810584369081688, 1.0541431748994874, 4.3131700033524352, -2.1092562484777337], [5.0604866236230022, -1.9229087921729093, -2.0803246946439913, 10.061131725338525, 7.6106038144724382], [1.9644144507488717, 6.0858938302965235, -7.8608600530021624, -2.9697478273758753, 12.004816400300458], [-10.635168622058105, 1.1415940391781494, 0.50809662460118932, -1.4444476636715202, -11.127685088025705]], [[6.2515666776284817, -0.18779054154507893, -8.3805286600210778, 6.1965223821483759, 8.3603832398634861], [-6.869355489321328, 8.7563694359230659, 3.5258436661419643, -5.0538307131642259, 9.1778240968238656], [1.6334325737736384, -5.8077259514062485, -9.9770936169455702, 5.4126952882702577, -5.6201827796699284], [5.1127078192980449, -5.3782272209857895, -7.1101526310326184, 2.5652203751297713, 10.145669750346249]], [[-7.9598065128296458, 9.8365371035720806, 13.308664658653628, 6.1604129246628192, 11.752587831618582], [-7.9718921232826334, 10.040201937958907, 8.3536137994747044, -10.208198546756408, 9.7291364154546649], [-6.1613017355267115, -8.4879785339509262, 4.4793790798040893, 0.32298379473090744, -6.1048742546314232], [1.2717629009595106, -2.713531925148732, 12.093447576619102, -6.2227303416028947, -1.9585940591179578]], [[-6.1481264823411248, 1.7001681140839153, 1.4796719217315948, 3.6840503321102043, -5.5917733921063801], [3.2532146131165214, 7.1168691088208487, 4.581646316198535, -6.1880457330464793, -4.6707535416107255], [-1.0664118458072021, 6.4368304111633199, 2.1931901396695501, -6.3530649761078637, -3.0160536041153683], [-0.27275553903027644, -0.67748127925849844, 2.6932739052448733, -6.8739211853908539, -0.51567350246906307]]], [[[19.889415409673582, 12.744442121207005, 22.412410654896924, 17.446365182341733, 22.529327927606484], [18.21868910925339, 8.9334106758022251, -20.24411289713602, -13.270474473850532, 17.353104794538559], [-7.4621428907272298, 22.942387412855904, -1.9929866487659345, -8.4339501585787549, 10.508185788473609], [8.5984547956513619, -2.0827972980179905, -13.540550372468296, 23.532365130090913, -21.729645516523707]], [[-4.7127982825456431, 8.3799512797819702, 1.9921457154309168, 7.8171303553323019, -8.5310613131967994], [0.9106785990729459, 1.8356031731039304, -9.9306777410728735, -1.7956103778921992, -6.1494486172294947], [-2.1034760680921014, -1.2901121159909665, 10.445622894268737, -4.2139828138753082, 6.0687401258976719], [5.1745688635309701, -1.6747132267298852, 2.1133386240505829, -2.8021317899774685, -4.6529088660714057]], [[-0.37225471710690766, 1.7883505344016117, -1.1597086516155111, -0.68217440056149858, 2.1645372461773067], [0.83679204333114876, 0.44652950900191035, 1.4547790999834702, -0.11321421551572752, 0.33515876287238255], [0.42716690803868818, -0.90374095267121912, 1.9646205662618872, 1.003981450811521, -0.16324303567936752], [-0.0038594142104885301, 1.433769878563381, -1.2466489718003111, -1.6609732857566646, 1.6966613664204659]], [[-6.7144139601134878, 8.5858790436875356, 1.3191411203387253, -4.1048777039797573, 10.552625791807278], [4.0532426966520188, 2.6957168825561149, 8.5690633652612895, -1.010059143652966, 6.6333229632080037], [-6.6306898957086693, 9.4763952804304594, -5.1074056991641159, -1.7171271503860617, -12.253352112895522], [4.5974889353936064, -3.7985518406635577, -1.9298550688123528, 6.9293421014537842, 5.9242012404713149]], [[-0.61563377054836876, -1.4041306839162728, 0.97331896770176096, -0.1717289557365885, 0.012438983100306835], [-0.017874860524132456, -1.4607747581807424, 1.5686471592265163, -1.0012976950828936, 1.3262659784203761], [0.96657255084908278, -0.73887826957639069, -0.50755243671762473, -1.313039610287019, 0.0050319078513314892], [1.0092127771184285, 0.073281528513346544, -0.71714850056418322, -0.43294925751898078, -1.211084566310267]]], [[[1.3793943187787094, -17.978614065186107, 15.527775558084659, -3.9400028977556403, 9.6679061893993303], [-5.8759803168362099, 14.754525529827008, -2.1122896678148808, 6.188963187764811, 18.090027056560249], [-0.73240105582250892, -17.063531087659008, -11.411873165653109, -2.7069056390147956, -17.035172614100219], [-7.1433272017653486, -1.2100372238457595, 0.5481395196564226, -2.2670397994862124, 13.869528295480459]], [[-8.5533997923155098, -9.1524564135738569, 13.603381139984652, -6.748690580429737, 12.195186371157508], [6.3606038763477981, -10.865208633760162, -12.565087931717906, 7.2971904836330417, 0.76296929499595667], [-2.1890362531480658, -12.295457501452741, -11.651331055548628, -6.438017452008606, 7.1013494522857794], [2.9563162805376901, -3.7714297039770686, -3.1543767237820433, 7.3415461132979223, -2.5844712982238742]], [[9.6311658529024609, 10.169596156740326, -9.4465520249865396, 10.40129899326563, 5.4554079430357207], [-2.0939203721937689, 1.7117170289659647, 4.3529850096319631, 1.4073790386643774, -8.0061822305726604], [-8.9921262863623408, 10.444049901656836, -5.3829077378651453, 5.9985582687565717, 12.738095519977492], [-2.3137078340187518, -9.0504386148994094, 2.0334516774218705, -13.182832309725296, 3.8047335085370051]], [[-20.405575771610906, 11.507242312457182, -4.4468012337520078, 10.879663667431883, -5.6573126684525104], [7.7179062917893635, 8.1300180421245205, 11.821265029413553, 3.9236794884030517, -1.2102220090918183], [1.645212507073019, -10.701591044065731, -10.967332961983269, -20.015552664800946, -6.6298591068696036], [-2.0561328365052014, -3.9308537767366309, 6.0005508703236581, -13.468432214278049, 18.901966964794973]], [[-0.48367388019540219, -0.037264810448011068, 2.3313573792973132, -1.0611435394846862, 2.8109003614467141], [3.1206527294544371, -1.8838156657571714, 0.89097249053123784, -2.5092663234211576, -1.7673620854787411], [-2.5168871350513928, -0.98013821902769982, 0.81829289762743995, -1.2382075892720519, 1.9104802335737301], [0.0015123636382486235, 2.9178825639041546, -1.0656971273724367, 2.4004031592133539, 1.8491857647809704]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[-4.7830218353477916, 4.5557279044228025], [1.8901400768708125, -0.70873947661554748]], [[-1.2865746082413754, 3.1989750971337951], [-4.2600507785417872, -4.9205477725322879]], [[-3.0729819125045088, -0.056603481459200466], [-3.8686513902825013, 0.76649766968540334]], [[4.4960117167595541, 4.0415853141195743], [0.21651511278161983, 3.1698005394452604]], [[1.9522445375912554, 4.502297791197968], [2.1902762347033944, -0.35531649226967943]], [[-0.53304102755107774, -2.1563350311995477], [0.51940435837120358, -0.38507102331942633]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.747935051569768, -4.3493254363152278], [4.975883107719838, 2.3971000137858915]], [[0.53639516303371693, -3.0016553578735161], [-1.539194068267097, -4.9143827171263075]], [[0.35894797131777256, 0.46401257398982843], [0.89085556513392206, 1.9710964455415887]], [[1.0861996253509716, -2.7502335854644477], [-0.40495402865944463, 3.2694156709649622]], [[-1.8738369455172563, -4.5989155520585321], [-1.1449726887593026, 0.39027320966130219]], [[1.4291865245871005, -0.68486511511433612], [4.3283031682223516, -3.3551512097942862]]])+(1.-msk_arg1)*numpy.array([[[-4.5806166467096912, -4.5467483990596982], [4.0108007235042304, -4.1794755430362542]], [[4.8840033420260447, 0.73849143348645807], [2.034726783147665, -1.1364788092183451]], [[-4.5860062704288582, 2.9921818737638484], [-2.5898245475632855, -4.6904184844600003]], [[-3.7549239262300049, -0.18421343735157691], [0.026708661775570519, -4.7132989104341236]], [[1.3169490048258012, 3.6931470940878928], [1.0024025793744897, -4.6310354787494346]], [[-4.3041657739446482, -1.5552883904893622], [-3.4684817031862156, -2.532857559401549]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[22.709477024471344, -19.814343255637166], [9.4051160797257527, -1.6989194091657345]], [[-0.690112396742673, -9.602220740015607], [6.5570448888481474, 24.181454932127025]], [[-1.1030406233897023, -0.026264727128669139], [-3.4464096205962509, 1.5108408322328093]], [[4.8835662423178059, -11.115303669411533], [-0.087678667186570969, 10.363395557495526]], [[-3.6581879412227463, -20.705687331939114], [-2.5078064695739468, -0.13867050788368307]], [[-0.76181505362806168, 1.4767986393675538], [2.2481395299265783, 1.2919715097468971]]])+(1.-msk_ref)*numpy.array([[[21.909189440570035, -20.713748555985973], [7.5809751878377964, 2.9621593088989959]], [[-6.2836346864167263, 2.362415705169818], [-8.6680394166680355, 5.5920982732294746]], [[14.092714319660143, -0.16936791121414771], [10.019128336518454, -3.5951948381879313]], [[-16.882181967870888, -0.74451432306361953], [0.0057828289165837892, -14.940217428860842]], [[2.5710065009574103, 16.627648004281113], [2.1955385472093276, 1.6454832818856846]], [[2.2942969468936352, 3.3537228400301733], [-1.8015445135656958, 0.97533005232109915]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[-3.96469343606599, -3.964772144732688], [0.84178100308003145, -0.68367246967770523]], [[2.2864583277283987, -2.7466177398049574], [2.0136259534258807, -4.556779124278048]]], [[[3.7768753192700686, 4.7738070638761645], [2.4900300015172725, 1.2547844602587759]], [[-4.4618906111407508, -2.4070725408095339], [-3.0232273228160644, 3.8585476076260363]]], [[[-3.1928014295096441, -1.9885132659585594], [-3.6549586904731091, 4.2204809607011207]], [[4.2619448127833941, 3.396292010198227], [-4.7651227818653199, -3.5504033491781515]]], [[[-0.81996758437910522, 0.28572432900876699], [3.0909317232440223, 1.4489367928770793]], [[-3.282520976973303, 1.9819773866486283], [-1.4242187797058694, 3.5701869857747841]]], [[[0.073649124461956816, -2.7654498206496769], [-0.13536369203665988, -1.6545376738758675]], [[-3.1272219410512756, -1.5909530632147941], [0.02711560618194131, 4.2972192999848211]]], [[[0.60236903128166297, -0.21916306872872049], [-1.9254608432284268, -0.72659704793057855]], [[1.0079680442588863, 3.8642147735200201], [-1.5630010760898294, 2.0487106391070666]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.0607260814152806, -4.8225437965092173], [-3.1261148743713352, -2.4880791358453234]], [[-0.62734534365756112, 2.8355282141730385], [0.950660309798387, 1.0649298062849564]], [[1.3062978029862791, 4.931719491240564], [4.609558635523733, 1.3829472038292216]], [[-2.7819304568377778, 0.82938307542689049], [4.276069954103491, -4.0062233609648565]], [[2.2643770612101823, 3.1252805602228193], [0.37271844439482837, 1.4093725326571027]], [[-2.0734496111966081, -3.43293081917012], [4.0709662700575358, 3.6300504233433077]]])+(1.-msk_arg1)*numpy.array([[[1.1326610620742974, -0.68519954746277456], [-0.74934680492614447, -3.043599428543966]], [[-4.9253864712446918, 2.913026188145623], [1.9137047469639938, 3.2302979499234166]], [[-3.7559518619108632, -3.4529213440832605], [2.0972642433827229, -0.41146479251480095]], [[1.4164682149641354, -3.784593549509859], [1.6652500392778435, -1.9256249264696503]], [[-0.90928438480169049, -0.53466171462787671], [3.5527942266566193, -1.8684264773965809]], [[2.8095676802335952, -4.3590134592928091], [-4.7607084982381584, -4.3304624203351461]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-12.134840604583141, -12.135081510252137], [-4.0595257544229124, 3.2970404274883531]], [[-7.1477313879419562, 8.586242570616454], [-5.0100607221155808, 11.337627065771734]]], [[[-2.3694051451192419, -2.9948256330422853], [7.0605503234395597, 3.5579767397696469]], [[-4.2417423106735805, -2.2883083273531821], [-3.2195248872418989, 4.109082356330477]]], [[[-4.1707494927398994, -2.5975905105307366], [-18.025231013485321, 20.814228216299419]], [[19.645684515891276, 15.655407164369496], [-6.5899132270835663, -4.910020384211828]]], [[[2.2810927966039332, -0.79486521312902669], [2.5635664585586655, 1.2017236533755675]], [[-14.036289323349978, 8.4750739527607575], [5.7057385463825145, -14.302966505423646]]], [[[0.16676938800986874, -6.2620211378069417], [-0.42304951528216156, -5.1708944283205316]], [[-1.1655732971460071, -0.59297755082660508], [0.038215990559175211, 6.0563828482025892]]], [[[-1.2489818337078415, 0.45442357964422098], [6.6099738698241532, 2.4943573989589121]], [[4.1034039094737871, 15.731088003258021], [-5.6737727179459307, 7.4369229227985461]]]])+(1.-msk_ref)*numpy.array([[[[-4.4906538780935001, -4.4907430283355163], [-0.57678796237319796, 0.46845206683592111]], [[-1.7133502424800509, 2.0581692276763133], [-6.1286708011483091, 13.869010338653741]]], [[[-18.60257060111077, -23.512844728748004], [7.2535226036880998, 3.6552199932119849]], [[-8.5387412429741296, -4.6064261476338872], [-9.7659250230451917, 12.464258426596288]]], [[[11.992008473878414, 7.4687601037115021], [12.620284874077202, -14.572988791501924]], [[8.9384244631210859, 7.1229217930751716], [1.960680256747765, 1.4608659774134425]]], [[[-1.1614580205739253, 0.40471943028287349], [-11.69792026176472, -5.4836368399700968]], [[-5.466218185835138, 3.300487920964426], [2.7425111829478097, -6.874841051965471]]], [[[-0.066967998827573538, 2.5145803388693868], [0.072373783682680423, 0.88461794963089002]], [[-11.110376057640879, -5.6523288578711837], [-0.050663516540997551, -8.0290383192712405]]], [[[1.6923965618625798, -0.61575347460102725], [8.3931097309739933, 3.1672463114118141]], [[-4.7986420342557761, -18.3964001113142], [6.7685174229504002, -8.8718644327939522]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_expandedData_rank4(self):
      arg0=Data(numpy.array([[[4.2422045601143559, 3.6602054935287711], [0.27909549036524073, -2.9984614480342122]], [[-2.0218439872178973, -1.2508003731627171], [4.5787882710025674, -2.9291653909945445]], [[2.2705719071888524, 1.8144291465301361], [0.031739174944799942, 4.6563641934194102]], [[0.33755687939147094, -1.1868187460932789], [1.5521327232135622, 3.0454716374593076]], [[-2.6015002350339103, -4.0349928238758999], [-3.492257189429302, 1.0793703010259224]], [[1.2904613320383405, -2.2665032387790229], [3.9070091851913329, -3.0724532020855442]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.75623488505624259, 4.12288449122161], [3.01221787490155, -4.3046840644246949]], [[0.99975609101862517, 1.1262774935815898], [-0.82856536597807739, 1.3725451353695419]]], [[[1.7169633726167994, -3.9174820274889202], [1.6174854919230661, 0.9807321149674495]], [[-3.6221423702653013, -1.3110274830534929], [-3.2886819067915574, 4.6776479893799809]]], [[[4.3043119196325499, -0.77627975740616684], [-2.8794701907256135, 1.2764718169685487]], [[1.6919902948515038, 0.50415580512719949], [-3.9477211452906849, 3.8201201686217345]]], [[[2.5793319605502854, -2.2239000787291694], [3.8832315173668448, -4.6100612130698799]], [[-2.6575510263383864, -3.781474103522541], [0.68007265973809616, 1.3898520455288974]]], [[[-4.9666690118924803, 2.7175934848764971], [0.98434442261018251, -1.7759342782191787]], [[0.72753177195230201, -0.14670961770357493], [4.0071757178178764, -1.2681820037563152]]], [[[1.2903875163612097, 2.3475551112082629], [-3.9957188721280632, 2.9509078781788602]], [[-0.67031541227760449, 3.2828510916145106], [0.69089283213655506, -0.056276936194906924]]]])+(1.-msk_arg1)*numpy.array([[[[-1.5588105390195395, -2.5302319082830413], [0.025552295896024368, -2.4614633206315695]], [[-3.3213932174021421, 0.78667031939180809], [-2.5760272449273716, -1.1523102218106462]]], [[[-0.31930817222151386, -0.59214141719033719], [-0.34758293311500932, 0.7653681123905578]], [[3.3151470686769997, -0.38855649732411912], [4.5369132081630674, -1.1844523146032673]]], [[[3.8343174782158354, 2.8094096593636744], [4.3206050666075519, 2.5784888348499262]], [[4.1478745115692597, -0.185976094494956], [0.27998075339507622, -0.76236400458574582]]], [[[-2.7572894795464578, -0.55341924064878167], [3.5860243115727055, -3.5867655168137826]], [[1.1857125098483445, -1.2462023428733504], [-4.6306115582592042, 1.0501038769906037]]], [[[4.0669045023243022, 4.8925602650497204], [-3.2836035515368045, 1.0238173628985878]], [[4.6293593486379478, -3.2207199936442632], [-1.5210954957573151, 0.82415928854661757]]], [[[4.1048893935129946, -4.5217834865413211], [1.0980146086688434, 2.8790653901444907]], [[-2.3295331018052288, 4.9782041458859307], [4.7313967920953122, -3.0801266510434901]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.2081030779031479, 17.49011938948507], [11.025336413420215, -15.756028260513027]], [[0.27902741646847945, 0.3143389693584881], [2.484421307061623, -4.1155236740924703]]], [[[-3.4714320711986382, 7.9205374823126506], [-2.0231514568826521, -1.2267000953739466]], [[-16.585023000872201, -6.0029172623673501], [9.633093223363776, -13.701604601747057]]], [[[9.7732497244957894, -1.7625990092858199], [-5.2245946406172434, 2.3160676694320159]], [[0.053702375973195514, 0.016001489298368653], [-18.382027386536212, 17.787870767729565]]], [[[0.87067124751803893, -0.75069277065426498], [-4.6086919602312193, 5.4713070683088549]], [[-4.1248719115895964, -5.8693496980620052], [2.0711419966438864, 4.2327549849230595]]], [[[12.920790601773927, -7.06982008963283], [-3.9718226814543525, 7.1658820682896121]], [[-2.5407280611386662, 0.51234771718373395], [4.3252264608048474, -1.3688379911501114]]], [[[1.6651951932091327, 3.0294290958432293], [9.0563097649287201, -6.6882422632309213]], [[-2.6189284727439159, 12.826129368553287], [-2.1227358943959089, 0.17290825281560565]]]])+(1.-msk_ref)*numpy.array([[[[-6.6127931769830077, -10.733761339465167], [0.09352665381090107, -9.0094615682952419]], [[-0.92698586870663546, 0.21955613854643721], [7.7241183830005085, 3.4551577762749748]]], [[[0.64559130807560461, 1.1972175639289677], [0.43475686244524536, -0.95732272058495405]], [[15.179356514706789, -1.779117932569517], [-13.289369151297285, 3.4694567272192725]]], [[[8.7060935492800802, 6.3789666483361618], [7.8394317634985224, 4.6784852959542365]], [[0.13165011477177335, -0.005902727798726054], [1.3036923549554229, -3.5498444533048978]]], [[[-0.93074203229463537, -0.18681047186860023], [-4.2559608769207316, 4.2568405531955449]], [[1.8403831868592988, -1.9342714361191347], [-14.102396164769655, 3.198061573760941]]], [[[-10.580053018657139, -12.727996679444418], [13.249316766904425, -4.13109571225535]], [[-16.166913467732623, 11.247582552942873], [-1.6418253031447478, 0.88957305937187259]]], [[[5.2972010346228346, -5.8351867412310847], [-2.4886536667746149, -6.5254110314190799]], [[-9.1015072259602849, 19.449889323733906], [-14.536995224210514, 9.4635449918275949]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[4.6154146384300159, -3.9626103474758665, 1.6699472578010424, -1.3985466504703337], [-3.1972370737650779, 2.0937952215237274, 3.1390017737607199, -2.1299464886426867], [0.56334918609467444, 3.2299320396631437, -2.0581359507743602, 4.4014671557640916]], [[-3.8708952999257216, -2.4485778244493162, -1.7251651593278297, 0.72212111134709467], [-3.4147739904947971, -4.9842092269493108, -4.3818677332954721, -1.3948071967997233], [-4.349412647430948, 0.66025718831805769, -3.6114067711596007, -4.5979586792958891]]], [[[1.012503490244546, -2.2611408324328797, 3.8779015888186414, 1.8971332414848527], [-1.8719950704817965, -0.57800751987950871, -2.0553570557690204, -1.8812985558934825], [4.9288933818367902, -4.1380588490785897, -1.381195405417218, 0.96355439266607856]], [[-0.6194003151182681, -2.9857585254122565, -0.033358354587599059, -3.6235822908973447], [-0.8342385846246998, 0.1033525029771436, -3.0200979486962067, -2.4346428269289264], [-1.7854831587240581, -2.4111388760167598, -0.09778001918921575, 2.0857893640394671]]], [[[-1.6463861528627524, 2.4336408320423555, 3.117986893666469, 1.0707917678727776], [-4.2367234836860312, -1.26920724854519, 3.6199048776147293, -4.2315991853075747], [0.81755472261750484, -3.7761947442205743, 4.0280954088514278, 1.6833907904596082]], [[-3.6160408940617539, 4.8481086997205836, 0.25142070922012039, -4.1377569123557576], [-2.8576783077988752, -4.0688554382133493, 0.58125034727648295, 1.8886210372655494], [-1.4388363416757177, -0.54539988816811658, 4.6505558553808317, 4.410601659352233]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.097934205458648727, 4.2810935762485496, -3.2698505009796608, -2.8858033010156547], [-0.48783690510795985, -1.6292366326065508, 4.8806029580991002, -2.9692498472278164], [-1.450245180226136, 3.8297971289138069, 1.700081673822301, 3.5124483805594124]], [[-3.7285074681293375, -0.61581553205600592, 4.2873928413976099, 4.3135777505722768], [-2.8966226839144738, 4.1701262308078242, 2.9206943790714952, 2.0121039468170165], [-2.1941413723415981, -1.088783813525156, -2.3020908395099293, -3.1238426484374826]]], [[[-3.4168702261965853, -4.0925871803652489, 1.2042134966107039, 1.8056599035821712], [3.3770055809186221, -0.15941071219063652, -2.3598580960416959, 2.4888835470617483], [-0.41218811114647647, 4.041180623542731, -3.9463220459484938, -4.915680815547276]], [[0.46910145551088167, 1.9519082961080603, -2.3672291111752184, -3.1094564516741672], [0.46204129478805989, 3.4464398076556932, -2.7359995757089206, 1.6827960417128818], [-1.9023103610335799, 2.0710979134838681, 4.3302360193524727, 2.0425458238933629]]], [[[4.7775542246896983, 0.82450414411359141, 1.1770496612531236, -4.0391121380393047], [-1.1315285964394062, -4.5205588595663819, 1.6895833554194812, -3.7765833263478923], [-2.6774734974050718, -0.20884080077244249, 2.6409402976509275, -0.42024354186168988]], [[4.1172532615324471, 4.8882706357464301, -0.91459430523195273, 4.2608733474067577], [3.5212190876217004, 1.6121358819456795, 1.1045380897781945, -0.014755924088886374], [1.0099127046274257, 4.3564679057371549, -3.2698575006923947, 2.4397806372291511]]]])+(1.-msk_arg1)*numpy.array([[[[-0.74602650821635397, 1.0144821182579102, 1.2208890442270715, -2.651830024526225], [4.4991557715279757, 4.7937145851903402, 0.84683337807489067, 2.13059953251304], [-2.6562301439654759, -1.2991385085186802, -4.5011505529735949, -2.8071289165155298]], [[-0.43272388850488142, -2.8278616814495452, -2.369985741754439, 3.7188809116059485], [1.3813233503495423, -4.0694516069676867, -4.2756937419773706, -3.6051884783542598], [4.3702405249135214, 3.8106836716969426, 4.7504084056306635, -1.4333506732120949]]], [[[-4.462380230015274, 1.4321398926803184, -1.376771676660332, 0.015912934094430398], [3.6349964231330141, -4.5709105519682769, 3.596985962868386, -1.1518251737042764], [1.1163664632553196, -2.3913377858533424, -4.0333070509786957, -0.3415700110225881]], [[0.4911922372629558, -1.708990257471604, 4.7927609473591684, 0.52699186657948704], [2.5087555889939308, -0.55508810553825505, 1.0659517968907668, 4.8993395419342498], [3.3198682519694493, -3.786878017780646, 2.472823092910188, 0.48285540753659362]]], [[[-2.7071480436593518, -3.0115125509352003, -0.54300677070674119, -1.5344792760596802], [-2.5516430601856266, 2.5682564454679593, 2.704108925914559, 1.3081796834812955], [-1.6991813337388506, 2.6116174073629415, 0.5722983462998732, 4.2255126664769787]], [[-4.5955971531732143, 4.7386773151228816, -4.0559699517226004, -2.0064014623687343], [1.9031533597468906, 3.3731747115563557, -2.0448431206559414, 2.16253048666601], [4.0435730909315364, 3.959362255085356, -1.9015116393932452, 3.288673108238358]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.45200696547686009, -16.964305703754967, -5.4604778775303489, 4.0359305405516759], [1.5597302389619856, -3.411287876083005, 15.320221342494891, 6.3243432860057212], [-0.81699444191811821, 12.369984452088623, -3.4989992121463271, 15.459926183349026]], [[14.432662034119804, 1.5078722557437931, -7.3964607543307039, 3.1149255591253531], [9.8913118013083761, -20.784781637135708, -12.798096458470839, -2.8064970657295025], [9.5432262351140444, -0.71887733940433185, 8.3137864456306492, 14.36329941813778]]], [[[-3.4595930297367139, 9.2539159838152116, 4.6698214317835003, 3.425577425902071], [-6.321737800469176, 0.092140590395535973, 4.8503509883129468, -4.6823330228743156], [-2.0316312531016751, -16.72264323997593, 5.4506418781607353, -4.7365258427649497]], [[-0.29056158936587834, -5.8279268359275518, 0.078966868080669897, 11.267371332603007], [-0.38545267580215475, 0.35619818048128127, 8.2629867062322031, -4.0970073121406578], [3.3965431122917393, -4.9937046952381499, -0.42341056106611802, 4.2603203550400073]]], [[[-7.8656991200800617, 2.0065469513029708, 3.6700254169817965, -4.3250480269275018], [4.7939737769971265, 5.7375260720368297, 6.1161310294196412, 15.980986927019911], [-2.1889811024867241, 0.78862353425571341, 10.637959488018424, -0.70743410812009555]], [[-14.888156164910463, 23.698867395750934, -0.22994794887010081, -17.630458145904726], [-10.06251140370388, -6.5595478503935523, 0.64201314826367861, -0.027868348658564291], [-1.4530991013379548, -2.3760171085970332, -15.206654946105948, 10.760900527018341]]]])+(1.-msk_ref)*numpy.array([[[[-3.4432216666785904, -4.0199973391380306, 2.0388203114863335, 3.7087079984178146], [-14.384867633373366, 10.037056691820132, 2.6582114758568642, -4.5380629929798992], [-1.4963850896830908, -4.196129092624675, 9.2639797729228466, -12.355485728038746]], [[1.6750288661791275, 6.9242394038073121, 4.0886168297784815, 2.685482416856384], [-4.7169070492367497, 20.282998248072044, 18.735524445424016, 5.0285428354279649], [-19.007979411374134, 2.5160312866441559, -17.15565708186806, 6.5904871683701574]]], [[[-4.5181755576887248, -3.2382699890955102, -5.3389850723616066, 0.03018895624010157], [-6.8046953853239645, 2.6420206717342598, -7.3930904782836606, 2.1669270359316148], [5.5024512724436887, 9.8954964858864258, 5.5707851674486433, -0.32912128452381562]], [[-0.30424462654432194, 5.1026322310923291, -0.15987861913560433, -1.9095983951843656], [-2.0929007117316019, -0.057369745080219504, -3.2192788351988404, -11.928141872459474], [-5.9275688530741295, 9.1306888074042014, -0.24179268947629401, 1.0071346734087694]]], [[[4.4570110528302465, -7.3289399101639372, -1.6930879942357726, -1.6431077767760849], [10.810606075072933, -3.2596496967108384, 9.7886170905196384, -5.5356920828553715], [-1.3891737239817079, -9.8619759275989018, 2.3052723412237839, 7.1131891077177674]], [[16.617867238508122, 22.973622716615818, -1.0197548418375937, 8.3020015198769315], [-5.4386000725632382, -13.724960269159824, -1.1885657740071931, 4.0842005708455336], [-5.8180399134543057, -2.1594357311406149, -8.8430860886550615, 14.505027068263168]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_Symbol_rank0(self):
      arg0=Data(-3.80448251704,self.functionspace)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-1.59120026824)
      sub=res.substitute({arg1:s1})
      ref=Data(6.05369360163,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-4.514189732111813, 1.9618527334246938]),self.functionspace)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(4.21768499453)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-19.039430295567975, 8.2744768352336866]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_Symbol_rank1(self):
      arg0=Data(-1.54065833383,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([4.5433045655596906, -0.10519143794699559])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-6.9996800420468457, 0.1620640655203551]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[-4.2168763853472591, 1.4394190215912399, -2.2679130482985119, 2.9844055925637756, -0.33938610791446777], [2.1153306636700311, 3.0616253427938123, 0.90422319642560112, 0.54045817247557615, -0.29504772894926212], [-1.4002839385399204, 4.8636751180782483, -3.8782698093855004, -0.064295998199151327, -3.9347862891384819], [1.9861878589777779, 4.6720851207100615, 4.902137939910796, 3.1254719794384087, -2.7678514579662261]]),self.functionspace)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-3.86021519729)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[16.278050307808069, -5.5564671824138703, 8.7546324151727308, -11.520447823290013, 1.3101034115203116], [-8.1656315751912203, -11.818532676658888, -3.4904961245836579, -2.0862848508894474, 1.1389477272156503], [5.4053973400719864, -18.774832605483724, 14.970956057378377, 0.24819638937325261, 15.189121831418133], [-7.6671125578976138, -18.035253985994423, -18.92330737485236, -12.06499443373017, 10.684502261880709]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_Symbol_rank2(self):
      arg0=Data(1.51318622689,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-3.5842410587629514, -0.40585576455428019, -1.8410640036394765, 1.1454295582205889, 2.4374526715383791], [-3.0200359722223591, -4.2454338145054491, 1.3557322994532122, -2.9957923011577714, 0.2992601936144208], [-1.136456609556411, 1.8530704531542295, 0.12322649882714209, 4.7831439299744059, 3.6707503994983259], [-4.3317336346283222, 1.425060507517645, 1.2091014571689502, 2.8714260155539373, 4.9203064028325727]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-5.4236242039731408, -0.61413535302738087, -2.7858726931299147, 1.7332482313719046, 3.6883198112677102], [-4.5698768378787289, -6.4241319752820241, 2.0514754428822872, -4.5331916487345474, 0.4528364032337272], [-1.7196704890386811, 2.8040406871694872, 0.18646464081308795, 7.2377875160689928, 5.5545289468712298], [-6.5547196744750265, 2.1563819324603402, 1.8295956719004864, 4.3450022982693781, 7.4453398808441218]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[-1.1832035588177261, 1.735602496286079], [-4.9054630216548434, 3.1750475225042667]], [[2.8260091629608652, 0.78149643222047338], [1.3670066503050924, -2.8756432210365568]], [[4.9663955912532654, 0.67854216718289706], [-0.41375069902140105, 4.4969833389685459]], [[3.1362741452271674, -2.9542398807637236], [-1.3683065379980128, 3.0483168336175179]], [[2.1875393798297749, 3.0265631827714881], [3.6832929827455647, -0.33545115718515017]], [[1.601831302596266, -0.46872390048831125], [2.2679075480447874, 3.3006120151634146]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-0.285062722773)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.33728722807118205, -0.49475557324277508], [1.3983646454147729, -0.90508769169845948]], [[-0.80558986657482634, -0.22277550080608574], [-0.38968263778464957, 0.81973868631215185]], [[-1.4157342496100491, -0.19342707769339007], [0.1179449008122368, -1.2819223148708012]], [[-0.89403484720074811, 0.84214366413483421], [0.39005318730969246, -0.86896149646551579]], [[-0.62358593178724608, -0.86276034152509529], [-1.0499695264318163, 0.095624620224523477]], [[-0.4566223925409742, 0.13361571130193814], [-0.64649590064288909, -0.94088144785947569]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_Symbol_rank3(self):
      arg0=Data(1.39685846075,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[3.281073455704572, -4.5880313858850155], [4.1630988164642808, -2.4310486678801055]], [[1.5616429262583678, -4.1601399362771252], [-3.9674328280014293, 4.6072327906152886]], [[-0.3361785807287001, -1.3750552394205884], [-2.16412307530398, 2.9167444306467374]], [[-4.9396089975654478, -3.4863826733829617], [3.5510048200071669, 4.2810712246339495]], [[-3.4359741476275851, 0.078146673659574617], [-1.9609517629378859, -1.4023337066145469]], [[2.4044837481635231, -1.5871773226670771], [-2.5136362261221548, 2.732556082536032]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[4.583195216936641, -6.4088304595508996], [5.8152598047081554, -3.395830900218503]], [[2.1813941342112808, -5.8111266678843876], [-5.5419421132431985, 6.4356521042066284]], [[-0.46959389481314245, -1.9207575451805288], [-3.0229736278383661, 4.0742791357885304]], [[-6.8999346210362891, -4.869983134720254], [4.9602511269839731, 5.9800505611947745]], [[-4.7995695590250227, 0.10915984228069041], [-2.7391720611785106, -1.9588617028766466]], [[3.3587234673533031, -2.2170620718748801], [-3.5111940297014286, 3.8169940833588925]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[3.5743560173583546, -3.8342796220697242, -2.3936507977203481, -3.1682473751071241], [1.1373354774869355, -0.62164013035765375, 2.5016720645012569, -0.89917433138458236], [-0.043066884425332574, -2.3832893165175886, -3.0919926263367357, 2.3366455750596291]], [[-0.43279815000340349, -3.5721915092852985, 3.2711384510178299, 2.7383387266615777], [-4.4311378956865575, 4.7920517320435057, -4.0279175064907022, -0.59725822590032873], [3.6364048962390498, 1.8885512259463599, -4.986519580385858, -1.2488265492519735]]], [[[3.6561891133277822, 0.96560554463240855, 2.5006093520379391, -1.2666375977210897], [-0.47107053324834069, -0.23006121718637029, -4.4647393993056497, -0.3999705604240944], [-0.29516011083489335, 2.0845196935657615, 0.84709922785601144, 0.99278524217902397]], [[1.9451726904867552, 3.6754388236246101, 3.7130960901357906, 4.457035436036378], [-4.5499287405411231, -1.8220699875050048, -1.9094256191153258, 4.8584743817888469], [2.4290537816428239, -1.2581807797034328, -3.4171127993070671, 4.0028338164147339]]], [[[3.3365633954939327, 3.5698117760011865, -1.6389739672640422, 3.1085590270266312], [3.3535335861239677, 4.3874920750748601, 2.9026722628952797, -3.6433858774167494], [-2.7581523347450601, -1.28625953628588, 4.0202005377013013, -0.14873645694210591]], [[-0.89446412350778459, -1.016400045178746, -2.4052428555138752, 4.7605910944040239], [4.0695913369694594, -2.3226617783268222, 1.6357400214644056, 2.7169276437069501], [-2.9205151360079595, -1.1218254402781023, -4.1005090842715495, -1.4128636324356569]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(4.90816477513)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[17.543528298157089, -18.819276179025145, -11.748432529322697, -15.550280165385448], [5.5822299281022181, -3.0511121906260241, 12.278618705901216, -4.4132957799990677], [-0.21137936511082972, -11.697576672265185, -15.176009293534564, 11.468641503461155]], [[-2.1242446345863013, -17.532904535877439, 16.055286519845165, 13.440217680563093], [-21.748754933333661, 23.520179511796339, -19.769682822470063, -2.9314417860180995], [17.848074419815362, 9.2693206032105007, -24.474659754924772, -6.1294464792803955]]], [[[17.945178617233719, 4.739351120830916, 12.273402738022673, -6.2168660399845699], [-2.3120917978892157, -1.1291783623167007, -21.913676649788151, -1.963121415760851], [-1.4486944590220399, 10.231166133015423, 4.1577025911991088, 4.8727535549277858]], [[9.547228080983702, 18.039659367243985, 18.224487436261686, 21.87586432864105], [-22.33179997365627, -8.943019730485906, -9.3717755644645457, 23.846192821546939], [11.922196207945337, -6.1753585836806577, -16.771752674190342, 19.646567938409024]]], [[[16.376402927737331, 17.521224412798166, -8.0443542934734982, 15.257319917851328], [16.459695419614839, 21.534534054025873, 14.246793754477203, -17.882338225727562], [-13.537466133826587, -6.313173747667955, 19.731806668087181, -0.73002303874025409]], [[-4.3901773036146485, -4.988658899182556, -11.805328259056109, 23.365765518331241], [19.974224849270463, -11.400006724914673, 8.0284815546150554, 13.335128557407893], [-14.334369515775895, -5.5061041098129326, -20.125974247504821, -6.9345675127769262]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank0_Symbol_rank4(self):
      arg0=Data(2.96884412572,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[4.2758313760628788, 4.6770101773065278, 2.5740531597968506, -1.4837000633954203], [4.4751552546496729, -0.75184940856855853, 2.7028769711218512, -2.0349254816608275], [3.3105762388863926, -3.1260216093691784, 1.1470643599709369, -0.4334187377132821]], [[-4.1251799775164377, -1.5443686481636032, 2.5658504518858338, 3.3293591378445964], [0.85871661921105602, -3.8627966927738022, -0.56672199461470996, -4.4108444625705854], [-3.4689631431997414, -4.3642940012594398, 0.066060761800770251, 1.9873185045487229]]], [[[2.3839105323769951, -1.9013327817477443, -3.0561594129602065, -0.17773198900336418], [-4.5736618195351362, 2.624091052423668, -1.3526860765653459, 4.5417577305247399], [4.8742706829837843, -3.1694512169945224, -1.3210332521587445, -0.59411621769148226]], [[2.9475456790262546, 4.1907593843218756, -0.18963478468595696, 3.5758222176084722], [-3.1787508441000636, 3.1782392226326195, 3.6531943406336111, -1.6468532380096113], [0.6710438159820491, 3.8249060536283199, 1.052684785219391, -4.5442359375674251]]], [[[-4.6121960783985871, -1.1316785114353767, 3.4401052207757381, -0.80040409131635126], [-0.5528798220655764, 3.0035475491128398, 2.4267093906166748, 3.1432009985480551], [0.89960679801954679, 0.6352190888521223, -3.9902824737988651, -2.289698984155677]], [[-3.693912842055215, 1.1704347592692059, -1.6829762293472794, -4.0877155502661155], [3.9281583891123244, 0.26425616431384658, 1.5007013591779401, 0.65973058998625245], [3.3896222188911569, -4.5847461897167276, 2.5343899610142087, -0.23200816033320315]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[12.694276863372961, 13.885314190806628, 7.6419626027414944, -4.4048742175347435], [13.28603838943013, -2.2321237000512024, 8.0244204182459633, -6.041376562496894], [9.8285848195501426, -9.2806708918344185, 3.4054552869169661, -1.2867526734349688]], [[-12.247016343767582, -4.5849897890392173, 7.6176100415449142, 9.8843483187861079], [2.5493957905987483, -11.468041270173552, -1.6825092646254751, -13.095109672146041], [-10.298810850011042, -12.956908608533114, 0.19612410461248694, 5.9000388681545655]]], [[[7.0774587802780049, -5.6447606601215057, -9.0732609204160823, -0.52765857150431383], [-13.57848902593472, 7.7905173063297886, -4.0159141123477502, 13.483770758689911], [14.470949884322158, -9.4096066273150374, -3.9219418105459174, -1.7638384428854821]], [[8.7508036744542768, 12.441711380429792, -0.56299611654616755, 10.616058785348764], [-9.4372157706186641, 9.4356968462304529, 10.84576455828606, -4.8892505615798685], [1.9922244911758091, 11.355549868726893, 3.125257040828342, -13.491128169110892]]], [[[-13.692891234000266, -3.3597771008730244, 10.213136176542195, -2.3762749847029512], [-1.641414011965844, 8.9170644974898998, 7.2045219191501948, 9.3316738204815017], [2.67079235775378, 1.8858664604807764, -11.846526682282024, -6.7977593787666111]], [[-10.966651442039515, 3.4748383595892505, -4.9964940922159649, -12.135790299002178], [11.662089958394954, 0.78453536110719047, 4.4553484146482241, 1.9586372866352999], [10.063260012948687, -13.61139679323559, 7.524208748028574, -0.68879606392321735]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([-1.1428607771484964, 4.4477636587426002]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-2.3096307513567949, 3.9718332036513395])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([2.6395863954216923, 17.665775381787626]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[3.9181413442072728, 3.1908774432745819], [-0.35990695105371806, -4.2352168944742505]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-2.1136550287140077, -1.5236491657627385])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-8.2815991553959645, -6.7444141539874156], [0.54837192572520832, 6.4529846880899475]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_Symbol_rank2(self):
      arg0=Data(numpy.array([1.6557993252759751, 2.6783777232739112]),self.functionspace)
      arg1=Symbol(shape=(2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[4.8516177366361273, 1.287469301116265], [-2.4178024498495478, 1.0007249087371575]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[8.0333053748190526, 2.1317908001018426], [-6.4757882209541169, 2.6803193026869203]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_Symbol_rank1(self):
      arg0=Data(numpy.array([[[-0.26893382577354874, -1.1038490163641956, -4.2469635418341332, -4.2983849576026847, 0.46118390498541473], [-4.6912171569893353, -2.4781639661313823, 0.25671435022272249, -1.1225182471455817, 2.0178721438292779], [4.1005511795129905, 3.9908115382386224, -4.4132595345226688, -0.99998622564093331, -3.4521365669270585], [-0.41342049572381168, 1.3217497400717448, 4.0460692576297888, 1.9072665646179612, -0.051178305915483335]], [[3.4250419227727527, -0.081512747329969493, 2.7690329328605587, -2.9639930420085125, -1.4039726043042444], [4.7510785353570171, 0.81311120087008959, 1.1913983129380465, -0.91899593437421778, 1.0787934283966818], [4.7757056571670891, 2.6730196506622264, -3.4171273788342358, -1.0532465983925232, -3.8374262880767529], [-4.7383524886859139, 3.1063406891375482, 2.1221536333081232, 3.2771665023038192, -3.4255966664358195]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-0.54146201280260975, 0.13784764984667319])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.14561745061405207, 0.59769231023073832, 2.2995694276608103, 2.3274121709440099, -0.24971356546557019], [2.540115884317582, 1.3418316491563966, -0.1390010687869094, 0.60780098950710404, -1.0926011125761181], [-2.2202926952592192, -2.1608728482105639, 2.3896123905829527, 0.54145455451042446, 1.8692008139978162], [0.2238514937484678, -0.71567727468057318, -2.1907928041749862, -1.03271239302916, 0.027711108532825317]], [[0.47213397968055471, -0.01123634065198198, 0.38170468214286885, -0.40857947500276515, -0.19353432395245335], [0.65492501033593897, 0.11208546810394807, 0.164231457469801, -0.12668142977213342, 0.14870913877451791], [0.65831980119994515, 0.36846947683776327, -0.47104297839902193, -0.14518756829741217, -0.52898019527122309], [-0.65317075471048835, 0.42820176362070628, 0.29253389096510346, 0.45174970049882357, -0.47221044979077581]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_Symbol_rank3(self):
      arg0=Data(numpy.array([-0.64923061105392588, -4.8059110359332733]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[0.2786161451712621, 3.8674084470700709, 1.0140586534702356, 4.9731974287343483, -0.0050198474597342724], [-3.2046387397129728, 0.11827789546919654, 4.5426162781310442, -4.5551763564563821, 4.5617534895325633], [-0.84801283435518293, 2.5832618514562924, -2.5243376166768772, 3.6354448151881833, 3.6324925227771381], [-1.6499741798784084, -1.5670422567983868, -2.4213656666809422, 0.04279825531284942, 2.0461723102857698]], [[1.3984319645416594, 1.280448606736563, -0.82934603793563433, 4.1347670543520287, 2.1262936751875188], [-0.46990336403269417, -4.3776604290797039, -0.28490582516787821, 4.0511625535413405, 1.2675939609462983], [2.2674998606707488, -1.1407329884322079, 3.8043656199791247, -4.0163418708808631, 4.8521150225071032], [1.3466780227491899, 3.4201967219207727, 1.6694219834402624, -4.8443898823708373, -3.2843643830937284]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.1808861301790278, -2.5108399492864169, -0.65835791923700238, -3.2287520055490138, 0.0032590386336807794], [2.0805495671909364, -0.076789630349638843, -2.9492055420345284, 2.9573599293605728, -2.9616300054866049], [0.550555890629987, -1.6771326703332645, 1.6388772533815399, -2.3602420588174509, -2.3583253402114179], [1.0712137450256594, 1.0173718019285396, 1.5720247113642647, -0.027785937448803158, -1.3284376993284537]], [[-6.7207396113926086, -6.153722090060632, 3.9857632763224, -19.871322617523727, -10.218778239218816], [2.2583137630268952, 21.038646567682537, 1.3692320493759815, -19.469526824423948, -6.0919438059941857], [-10.897402604374712, 5.4822612581594914, -18.283442717782805, 19.302181721347228, -23.31883313428451], [-6.4720147713791318, -16.437161170941845, -8.0230935338451719, 23.281706798049498, 15.784363034736327]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_Symbol_rank1(self):
      arg0=Data(numpy.array([[[[0.40236948901436254, -0.45873996764531633], [-4.2716081886583854, -1.3330374283552624]], [[-4.6549862447183212, 4.4223931872826014], [2.8136155811413674, -0.1776953074710157]], [[2.1246828736122367, -1.3244438463088324], [-2.4294914170724446, 3.2835647383564677]], [[1.9461745174684744, 1.914690237949447], [-0.16157889815145676, 2.6031516445081637]], [[0.052946799385108712, 4.573871441567686], [-3.8935780012003054, 1.7371489407900924]], [[-0.71975653098559356, 4.1098274243056192], [-1.6689684607370747, 1.2220165201752184]]], [[[1.2365855184108563, -1.1541418667211767], [-0.42251401282008594, -1.6600688876671441]], [[-3.2624836407662059, -3.5159594401437566], [4.5848422318249007, 3.4306381403482984]], [[-2.8214007126714846, 3.7199945078899876], [1.3968758361717457, -1.0412994326168725]], [[-4.8581583739124472, -2.8692613680240067], [-0.46450231202851455, 2.473875695521385]], [[0.6724864702360458, 2.7990412871629839], [-1.2395838287562135, 0.36503404341881041]], [[3.4649904972752967, -0.12809908364194467], [1.3328824627451556, 0.8146800119722295]]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([0.67967922021851379, 4.5563302054878889])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.27348218053300377, -0.31179602349223484], [-2.9033233227463495, -0.90603783982659769]], [[-3.1638974209380564, 3.0058087530319062], [1.9123560441848251, -0.120775808018389]], [[1.4441027987483961, -0.9001969606823963], [-1.6512748318833712, 2.2317707209031323]], [[1.3227743784421151, 1.3013751678894807], [-0.10982181949934879, 1.7693080798498506]], [[0.035986839319136772, 3.1087653747844546], [-2.6463840597157833, 1.1807040374796272]], [[-0.48920355772747076, 2.7933642989847063], [-1.1343631819630682, 0.83057923552683421]]], [[[5.6342919492042842, -5.258651448759875], [-1.9251133588540548, -7.5638220160684897]], [[-14.864952757333162, -16.019872198397284], [20.890055148260302, 15.631120182967752]], [[-12.855233288930142, 16.949523340548204], [6.3646275656654767, -4.7445040577896567]], [[-22.135373742101208, -13.073302238567283], [-2.1164259148144811, 11.271794556126446]], [[3.0640704171184274, 12.753356363108404], [-5.6479532411962623, 1.6632156380605034]], [[15.787640864463935, -0.58366172409311201], [6.0730526253708383, 3.7119511463563044]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank1_Symbol_rank4(self):
      arg0=Data(numpy.array([-0.25833934391757296, 4.4198733077914358]),self.functionspace)
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[-0.84107094495487811, 2.178881639472011], [2.5922510137477159, 1.4421370781868728]], [[1.616950564217758, -4.6161303782334837], [-0.1673936106146634, -4.1201946553382669]], [[-2.1393229839796701, 2.6534548870656209], [-4.3796893217914796, 1.4058206951473187]], [[-4.3965112522380991, -3.7934677750058809], [-3.7169428173811427, -1.6334141089481768]], [[-3.2667152023826231, -3.423830429080498], [3.9646733285137135, -1.2726666813692447]], [[3.4531010622468212, -2.3015583148978958], [-4.5815629435002219, -3.772702545029305]]], [[[-2.8200597978443529, -1.6889236258170595], [-4.2956727047868482, -4.5387747708715551]], [[4.256156902422374, -2.0818549210976842], [1.2285024904967665, -4.1060620187116985]], [[1.0724355513008508, 1.4221689341746284], [-3.3323358618322563, -3.3546836427388449]], [[4.9224279823336587, 3.3677142660991759], [1.3029704153804902, 0.75261009979988458]], [[-0.23699855981501194, -4.8131157347650841], [2.5522539551989398, 1.6533674050693925]], [[4.2237520291171844, 0.30847766367360308], [-3.3848015101423803, -2.9265014797047484]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.21728171610777633, -0.56289085321524501], [-0.66968042616124834, -0.37256074661800231]], [[-0.41772194790716499, 1.192528093350816], [0.043244355542185822, 1.0644083840727785]], [[0.55267129610909238, -0.68549179463941012], [1.131446065954411, -0.36317879605010467]], [[1.1357918324294176, 0.98000197616747453], [0.96023256882137964, 0.42197512925137903]], [[0.84392106214908846, 0.88451010673367814], [-1.024231106535733, 0.32877987549068555]], [[-0.89207186290191809, 0.59458306505875713], [1.1835979649409116, 0.9746375002790284]]], [[[-12.464307026867967, -7.4648284526471516], [-18.98632912689563, -20.060809459852376]], [[18.811674286788929, -9.2015349964538995], [5.4298253663019604, -18.148273916640054]], [[4.7400292675212237, 6.2858065113286354], [-14.72850232830856, -14.827276688625961]], [[21.756508048642193, 14.884870393000172], [5.7589641597821482, 3.3264412912797585]], [[-1.0475036085113834, -21.273361763399159], [11.280639131288913, 7.3076744616385989]], [[18.668448852224959, 1.3634321917208221], [-14.960393846850449, -12.934765775359157]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[0.047521260799689991, -2.058189067208497, -2.5333088272281037, 1.0983477065183518, 1.6289141309198998], [-2.6124146421583183, 3.5411627821389917, 2.9538098297168949, -3.9321159627986479, 2.515087279931695], [1.9727895538582736, -4.7995635561268433, -0.29954595900276004, 4.5898810855634267, -0.28875136213271801], [-2.5264353594386124, -0.11449933320425387, -3.6262460004547039, 1.826989707826991, -1.0498296269981235]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[2.9887495807256359, -2.4467604025261025, -0.062521363554757237, -0.41874275499766966, -3.4521697673384533], [-4.3509841557969233, 3.4155535576340945, -2.5226748278970201, 2.8256272319795341, -4.3717790239823753], [4.5062173539519232, 4.4381504357738688, 0.72230885505163833, 1.0389082548140616, 3.093993145506273], [3.5594765522779355, 2.8911395701765397, -0.78740796412180458, 3.5071887206615244, 0.33779113407215089]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.14202914829062704, 5.0358955105578858, 0.15838592218360395, -0.45992514457286654, -5.6232881163520689], [11.366574716402733, 12.095031138696282, -7.4515017038215943, -11.110693943785284, -10.995405813890272], [8.8898185232912255, -21.301185088148728, -0.21636469868262861, 4.7684653484067701, -0.89339473519422918], [-8.9927874227676181, -0.33103355298564696, 2.8553349806228749, 6.4075976960555172, -0.35462314028623931]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_Symbol_rank2(self):
      arg0=Data(numpy.array([[[3.5543047945483242, 0.32287231497467062], [2.887725648067673, -0.11494808965239756], [-2.5757507703939653, -1.226311097710334], [0.29337751655915678, -4.4058622550850082], [-3.7596938232256316, 2.0892130629837773]], [[1.0834824846864608, -2.6576388147750354], [4.4770302043741435, -3.994909756235324], [-0.96825928853066756, 4.263861904641649], [1.143023197885733, 0.93543768536755678], [-3.7851382134369649, -1.0068989115903229]], [[-0.61956890390509578, -3.4217967267026639], [-4.8259666380309234, 2.8899024016504455], [-0.22531788909672557, 0.02733078206354822], [2.4848243655646538, 4.1133664462182153], [2.2805091814491876, 2.528619238236959]], [[3.5411295680003558, -0.82212661382746166], [-3.2598180292693901, 2.7428174054163197], [-0.50721369767468882, 2.5942042807564425], [-0.58709422929837096, -1.7873735290097512], [-2.6141535394943061, -4.3103924264614113]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-1.7802675340192184, -3.2901271421081679, -1.0370946806931958, 1.2242727597185183, 4.2522003626102993], [2.5413181432902654, -2.8384481640095958, -2.758043431251914, 0.63591539814822795, -0.71716686313728939], [1.5882284584421527, -1.7213086944220546, 1.6179444278746669, -4.2293606224617211, -4.8297975890829541], [-0.70958685747860351, -2.8489679113883981, -3.0471768394091949, -4.5798796867994351, 2.7286536021810708]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-6.32761343174323, -0.57479909998303325], [-9.5009845336693495, 0.37819382969883625], [2.6712974227669823, 1.2718007163104212], [0.35917410183724419, -5.3939771419725773], [-15.986971438423733, 8.8837525439897913]], [[2.7534736962709201, -6.7539057383002348], [-12.707818163821292, 11.339344262970178], [2.6705011704806592, -11.759916317862174], [0.72686605197616672, 0.59485922813336667], [2.7145756990716721, 0.72211453392158276]], [[-0.9840169651478845, -5.4345949403533762], [8.3069783330334008, -4.9744141299920885], [-0.36455182316452928, 0.044219686549174735], [-10.509218325452576, -17.396910073390629], [-11.014397746444827, -12.212719100545641]], [[-2.5127390020819371, 0.5833702403553539], [9.2871169623538581, -7.8141987748286779], [1.5455698321854092, -7.9049992010172199], [2.6888209350007788, 8.18595571823478], [-7.133119472195534, -11.761567821277936]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_Symbol_rank3(self):
      arg0=Data(numpy.array([[1.6617375326008581, 0.79807304085202091, 4.1226859139931999, -3.7732180387951098, 2.006791933347305], [-0.75379928382007932, -3.1753108001075736, 4.2912381501423358, -2.2282383455908903, 3.6354517073736154], [1.2527611252477122, 3.1502229711168646, -1.2859030865822407, -2.4849261177266504, -3.971256912865214], [1.9760344081235903, -2.4753667060623541, 3.2054195488054411, 0.667341291322046, 0.55161790266087785]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-2.4121354284925669, 2.2220160694461839], [-4.397480361315246, 2.687629882305929], [-0.73001209009135337, -3.7797340879653607], [4.286700240099389, -1.8060960291745598], [4.3244897335907453, 4.0192335309226266]], [[0.034142463471212636, 2.8359052003675362], [2.9708191786890694, 1.8278628749187487], [3.8641814504469387, 3.4855679409738034], [4.7717580844901804, -1.0383594814093722], [2.226560341456576, 0.29128921423022458]], [[2.93110646084041, 1.0104282292287579], [3.1199965912542655, -2.2361180190981123], [-0.23222445682904969, 1.2565963319511404], [3.9596606388816866, 1.0535067970028935], [4.6173011155860202, 2.2408317982712243]], [[4.0787235089981326, -1.5335343914094479], [2.86340545007464, -4.2061079672858153], [2.4084111201587168, -4.7236263819930482], [-2.2026497408626167, -3.2245440494631392], [4.7668234244128023, -2.396083429538336]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-4.0083359752423515, 3.6924075006409587], [-3.509510524041902, 2.1449249528566519], [-3.0096105608643575, -15.582656483094727], [-16.174654672850341, 6.8147941170776676], [8.678351113213143, 8.065765428094533]], [[-0.025736564512453305, -2.1377033090186872], [-9.4332742232581133, -5.8040327278451818], [16.58212285923025, 14.957402123219854], [-10.632614339744356, 2.3137124129842341], [8.0945525949186887, 1.0589678712127888]], [[3.6719762281032713, 1.265825205430672], [9.828684931575502, -7.0442703498912129], [0.29861814581635931, -1.6158611018438933], [-9.839464138891298, -2.6178865550750388], [-18.336488974051246, -8.8989187694527878]], [[8.0596979950028977, -3.0303167234659387], [-7.087978517072254, 10.411659624322912], [7.7199680861171611, -15.141204346093634], [-1.4699191223974286, -2.1518713898935506], [2.6294651397293336, -1.3217225160024202]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[2.0016551647687457, 2.450144667427482, -0.64915374101394807, 1.989573025883967, -3.5351915939344614], [-3.9193286792516737, -0.96890928811126997, 3.7466127218427996, -3.0254661201064459, -2.708297672305211], [0.85061116994278052, 1.7167766773807012, 4.0612977892900766, -4.7833042500887153, -4.1831664223633203], [-1.5057536078887299, 2.5009120692276268, -3.4835739957343161, -4.0668080157525992, 2.5749097746589147]], [[2.6451940089566772, -2.4891756268760612, 2.7416453915777232, -1.5914298879581992, -0.067539753897310817], [2.5104969572879892, 4.4231502870611639, 0.29019246528376197, 3.5236496810067806, -1.0539401396614489], [4.3695766434033487, -1.8210715030559612, 4.7968584367441842, -0.94801582649359784, 1.1099961425356959], [3.7962567686762831, 1.8559835153274662, 1.794393385890924, -2.8249333804885901, 3.1175032933167621]], [[0.68050447014202042, -2.8564520317161026, 4.7402184896196111, -2.6728826152718543, -4.0786611927994718], [-0.76403553347261255, -4.51861764547448, 2.6443074374153284, -4.9526037295283896, 0.19119514959147299], [-0.95555988768168376, -3.2415863761533101, 0.2326034613487673, 3.3010230328962908, -2.3877704783930764], [2.099825115580618, -2.9655705045427627, -3.3616220556852836, -1.8120325288142194, 4.7577062233152532]], [[-0.46136647178052392, -0.26296380880995329, 0.78231237786091956, 1.6188948548046334, 4.150283863683379], [3.7655874258731536, -4.1940249477948663, -3.5098772562515834, -2.4503483143189566, -2.7763981364008785], [1.9881563108718057, 2.1698981029233453, 2.8595105899879014, 4.8531265307826015, 3.8505682049191421], [-1.3622950239132381, -1.968957414115807, -0.41624068179704921, -4.131566271472864, 0.12731656222367427]], [[3.7987770478367811, -0.083571965987148999, -2.5998241546549461, 2.9951693043901493, 2.5259451833383375], [0.27232751062473337, -4.9000588826788114, -3.9180586570483369, 2.1053715080786706, -2.6232309271250234], [-0.61519473045214035, -4.8331164084358749, 3.1859626816473146, -0.24328146741609302, 1.1193797590856445], [3.7074014206363817, 1.147617599795554, -0.24588012666983605, 4.7680408426336189, 3.092748819434668]]], [[[1.4189007748252616, 2.0842600729830796, -1.2501019526988566, 4.5310750515669724, -4.606520702948905], [0.73556593337646525, -2.6771347298823143, -1.1548396432527088, -1.6395448032255153, 3.0660247992957199], [-4.3473931042048051, 1.3990905944350596, 1.6560626977924695, -0.34247093804854334, -1.9588112958175783], [-1.6299312014887981, 3.1121499808460591, -2.5061811673147592, -2.4064207008154637, 2.1669908035374883]], [[-0.6766038999837054, 1.3702204517991081, 1.2007123100549606, 0.8974160308449699, 3.130063463385854], [-2.1180201310268143, -3.1434870345362667, 1.7501802254612828, 4.0354176727975091, -4.2138018376538655], [-1.6523798246306853, 4.6967703001008942, 3.5307283526704438, -1.9987649315865386, 1.5435097705408278], [3.264573836273259, 1.7909141156409802, 2.6587070570178994, 1.9966540916158992, -2.2130232900763538]], [[0.28003078175385721, -0.72792274005158841, 1.3638666064355025, -1.9263678915669824, 2.1460006920267318], [3.1278814487630378, -4.3418907012246466, -3.8487716531341776, -3.857856469211538, -2.7414825854400759], [-3.0666346042548986, -1.659332776473935, -2.5587603977598827, 1.7701264958086895, 2.5468999577323501], [-1.1697841097216246, -1.5530225693608237, -3.8602047876552081, 1.028806806707756, -1.8529654060603584]], [[-1.4231897013350512, 0.63038534660178591, -0.51632082279218761, 0.95620479495468125, -2.6141163642280709], [-3.0186190834727777, 1.9625216483378418, -3.3052705176377728, 1.3109255324763227, -3.9695479070732711], [0.24614224977791199, -1.9569433244910184, 1.346456987256226, 1.78916679225508, 4.2904319582500712], [4.8380029339311736, 2.0012751386930709, 2.6435911784375845, -1.6468020010398101, -0.69412231970243354]], [[4.2549401471838095, -0.58221240463400026, -4.9029754701853445, 3.5717653399114475, -4.275395257317566], [-4.8778354323638062, 1.4979945829340995, 2.4767795502622754, -2.1579510728975606, 1.8291561165136736], [4.3243958541942824, 2.5396641851892143, -3.8116920929441687, 3.4228631667366933, -0.80783741658961983], [3.5364095509314328, -3.7938357593802499, 1.5312015456780461, -0.13673491817064143, 2.3195691254196271]]], [[[-3.659975201580151, 1.9737739020170384, 3.8803435263521919, -4.3608028933362055, -1.1408006650063651], [3.6902804441093693, 3.6495091191121833, 2.4237293699307187, 1.8437361669744927, 3.2545868401337223], [2.8499270834859036, -1.2394396395306764, -3.1304519171495127, 1.2065123877750326, 0.73522036001991342], [4.2619953901459091, -3.2872111663287127, -1.6285493660756512, -2.9346289367577638, -4.0872276296436869]], [[-1.9755463607636381, -1.2035887598667836, -4.005996104325944, 3.6949656426705495, 1.2818399192323877], [-2.1806768313150195, 2.6213424901039426, -3.4366735130923542, 1.6136826292528585, -2.6663975694778563], [1.9931641719829143, 3.9411314333747729, -4.7086066702789839, -3.1383826617154922, -2.1722036933440836], [2.9522053271100841, -2.8499936814946238, 4.5998965389488813, 1.9489950476212261, 3.8403947754467467]], [[-1.1634152479961934, 4.648488751474142, -1.9857086336587662, 3.0687086504024492, -1.6869557284771508], [-0.1752628367808704, 4.9700008769881023, 0.5145193523247773, -0.62621941054063157, -4.6404149300616133], [4.2766318873448306, 4.7647185582326088, -2.6623806106715575, -2.1278549700952807, 1.6546102654717396], [0.50474611766265909, -0.64608725436865821, -4.6724004342631931, 0.10274930247660485, 2.9345938100879216]], [[-1.1435163223018052, 3.8853294546332346, -4.2065186559019372, -3.1701422653506119, -0.68367538561759567], [-0.62702215240045867, -1.4994374906948869, -2.2720938548439529, -2.4093563939636309, -2.2665402037607443], [-0.69433113130416579, 4.7473979773613522, -1.3979492454571529, 0.31188929626070205, -4.1619339277222416], [3.9309406989737159, 2.0301031087777321, -2.5152493981561008, 1.3171890912073518, 2.9287726719816911]], [[-1.8785693828974424, -0.037300671563336607, -1.5450654914572706, -3.4834521064259141, 3.9376099096731956], [-4.0504109035019988, -1.5129588486924481, -0.031363762427983666, -0.40020038969266025, -3.6826316367890701], [1.3228721344148919, -1.2606071939578687, -3.1634514578883399, 2.8427859710976788, 3.152256226258622], [4.5001298691533442, -0.91380976719217344, 1.313227010459177, 0.29175482354956372, 1.559563057570279]]], [[[2.4336751400623289, 2.8373028448726636, -4.3104767188953721, -4.2330796065084133, 3.543007659591856], [0.36724274071015017, 2.0469412291954434, 2.7639218887485386, -1.0389728373562122, 2.9409241879655479], [2.8019376700112826, -4.1325331514079124, -1.1762095535978476, 3.5977262170698356, 4.0140682443810221], [1.5944958138329568, -2.7033980838912761, -0.19590965267586924, 4.8891472709028001, 3.1156241343926663]], [[-0.97875555188160668, 2.0913139556207447, -2.9308191005081872, -3.2023889936095804, -2.7083078314797415], [0.386105735953997, 2.9129385863349473, -1.3625243042891224, -4.5721360510175106, -0.97040442265382865], [1.3384669422281998, -4.2317720026504961, 4.0950386069249163, 2.9935489827962938, 4.5764025702556825], [-3.4622339373335089, -3.3372508225429964, -2.9694341873953722, -4.855606240230367, -0.66298014082814127]], [[-2.3899770399125755, -4.1092872074368136, 4.0184560005076513, -3.9362676384443041, 4.8055485804820126], [-3.6092832291011643, 0.06416202152120043, -4.3440696440046889, -4.7851115245430966, 2.4606366573534171], [-1.2756224881431901, 3.7820193643569837, -4.5033166681084325, 0.27544588314854579, 0.94530823032114153], [-4.5110783342410059, -4.705990498835682, 4.4840528427821038, -1.8617533662721, -1.9222312414201195]], [[2.261490650712461, -1.320718639459936, 0.79618170813222644, 3.4956474962069546, -4.3260485458586722], [4.0830430460557476, 0.4406962536458181, -0.68896770230982174, 4.4265973618319983, 1.5655096999071496], [1.4084140092268225, -4.5619303895766796, 1.015952613100783, -2.3012285558740708, -0.87506696704483744], [1.230703278585338, -3.8175771488355306, 0.074228198894448205, -3.7388669442093803, 4.6901981204652294]], [[4.6038687168734764, 0.10032873151114341, 4.8450399927401602, -2.0306982938092588, -1.2998195129863999], [-0.59337371065153022, -4.4168056911045808, -1.8042050745823204, 2.467700155325911, -1.8032458161877276], [-0.63117631952945175, 4.1707444242725735, 4.1239802479550409, -2.371804537369929, -4.771380071736715], [-4.2399963240071052, 0.31014337831565175, -2.6441847343020575, 1.7174449733404797, 3.8648744343031272]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[0.53476988022020855, 3.403239892820487, 3.7778836038135406, -2.50806487988666, -3.1410518572387867], [-3.1597135512842875, -2.8747683798540038, 3.6586984021258591, -1.0009571010889964, 1.5501226693188368], [-2.9314016796616702, 1.7214398843404757, -3.6433657562735409, -0.90861219593488141, 2.899098327069507], [-3.3261008295783134, 0.22376742531771754, 0.40485016088917281, -0.70753152402395347, 3.9134144711773242]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[1.070424892705544, 1.3102635703223773, -0.34714786832652927, 1.0639637287413268, -1.89051398524382], [-2.0959389283470458, -0.51814350394751141, 2.0035756364913833, -1.6179281546596231, -1.4483160218193274], [0.45488123346427217, 0.91808045812772521, 2.1718597323172522, -2.5579670408767567, -2.2370314066284309], [-0.80523167653180294, 1.3374124477021319, -1.8629104484370735, -2.1748064354626013, 1.3769841917721919]], [[9.0022297755311165, -8.4712617936210552, 9.3304769685847528, -5.4160176813261813, -0.22985398481460614], [8.5438233958469354, 15.053041508866942, 0.98759457444962295, 11.991825162726458, -3.5868111279406385], [14.870717547566915, -6.1975431868786126, 16.324859992140325, -3.2263252797481972, 3.7775831531543358], [12.919572478548922, 6.3163571397796368, 6.1067511542772186, -9.613925975039006, 10.609611573814853]], [[2.5708666800713602, -10.791343295700139, 17.907993710427714, -10.097839407153794, -15.408707245787703], [-2.8864373146371145, -17.070811514740583, 9.9898857112535691, -18.710360425971093, 0.72231302077030302], [-3.6099940321345416, -12.246336020814942, 0.87874880281978462, 12.470880791789742, -9.0207189399912178], [7.9328948750278894, -11.203580185065153, -12.699816846391402, -6.8456479801840269, 17.974060332824337]], [[1.1571370446299518, 0.65953029355747417, -1.9620902000135947, -4.0602933295647148, -10.409181200064598], [-9.4443375749752683, 10.518886676932787, 8.8029998791175483, 6.145632550632854, 6.9633866584898163], [-4.9864250190226, -5.4422452248747311, -7.1718380844126388, -12.171956209502028, -9.6574748823659196], [3.41672430552115, 4.9382729403363106, 1.0439586355952577, 10.362236264405364, -0.31931819834110209]], [[-11.932155701343797, 0.26250387897703109, 8.1661824894731776, -9.4079821062992828, -7.9341248094082522], [-0.85539483302503416, 15.391339054017694, 12.306825421492185, -6.6130810856281332, 8.2397043756122788], [1.9323585506502101, 15.181069270968759, -10.007273998281763, 0.76415970505909636, -3.5160298712314693], [-11.645140117819622, -3.6047263932377436, 0.77232222853439669, -14.976663544144719, -9.7144844232583285]]], [[[-4.4833200061431544, -6.585664797005415, 3.9499640804295266, -14.316899242322314, 14.555285889379277], [-2.3241776475526925, 8.4589788846229492, 3.6489624703458965, 5.1804919326893915, -9.6877601069083745], [13.736516904115787, -4.420725510710847, -5.232683747991282, 1.0821100638730241, 6.1892825958035376], [5.150115705005236, -9.8335024681084295, 7.9188145963379188, 7.6036000984576528, -6.8470702074458289]], [[1.9450794973590573, -3.9390664282613432, -3.4517697822474576, -2.5798632290472048, -8.9982074714779632], [6.0888173005701196, 9.0367971293658904, -5.0313627712018469, -11.60089112526231, 12.113704281858027], [4.7502092713569981, -13.50212674616745, -10.150026226111008, 5.7459862240860318, -4.4372330823464807], [-9.3848936382170471, -5.1484632706788869, -7.6431669788097532, -5.7399180480835064, 6.3619293781919763]], [[1.0245481737488926, -2.6632497658978238, 4.9899765736783905, -7.0479991267826785, 7.8515693028791915], [11.443974858628444, -15.88566857077574, -14.081494697469317, -14.114733299535162, -10.030257954805474], [-11.219891126491264, -6.0709981778802513, -9.3617325787070111, 6.4763589817758982, 9.318338805729768], [-4.2798872530707284, -5.6820411929858423, -14.123325088472701, 3.7640938197978744, -6.7794415703475268]], [[1.4245518377480475, -0.63098868910350581, 0.51681499401395348, -0.95711997960523598, 2.6166183378470369], [3.0215082070848349, -1.964399979944645, 3.3084339956496316, -1.3121802207310489, 3.9733471656979544], [-0.24637783279322245, 1.9588163170779929, -1.3477456827050156, -1.7908792057403435, -4.2945383353495776], [-4.8426333918078068, -2.0031905613076955, -2.6461213624333282, 1.6483781570283667, 0.69478666493051744]], [[6.5956791787444509, -0.9025006467817952, -7.6002134234484853, 5.5366744228840359, -6.6273871086662002], [-7.5612432809137857, 2.3220753615229639, 3.8393121277668665, -3.3450888773794145, 2.8354163619310531], [6.7033440446949522, 3.9367910259189536, -5.9085903217361189, 5.3058577887350102, -1.2522470925795346], [5.4818686128944618, -5.8809108142877689, 2.3735502272515818, -0.21195589634376741, 3.5956166843650323]]], [[[10.728857453432115, -5.7859241316451149, -11.374845530813104, 12.783264926199225, 3.3441449855588092], [-10.81769429228482, -10.698177161706036, -7.1049243460602307, -5.4047312967219971, -9.540501329772761], [-8.3542810394438636, 3.6332954411594796, 9.1766120080321762, -3.5367724400563429, -2.1552261982838319], [-12.493620445384014, 9.6361363343785857, 4.7739323471261121, 8.60257619439545, 11.981305938697091]], [[-3.400784298782205, -2.0719056955785726, -6.896081470499249, 6.3606612285608222, 2.2066103623064062], [-3.7539040722828823, 4.5124835129813059, -5.916026854893679, 2.7778576386632756, -4.5900431236076864], [3.4311123016898479, 6.7844208388392824, -8.1055833218898456, -5.4025370861996711, -3.7393180746341934], [5.0820439968497197, -4.9060927934431922, 7.918445365986317, 3.3550778093572435, 6.6110087380668148]], [[4.2387472748758208, -16.936144735543635, 7.2346628378090703, -11.180428012856677, 6.1461967334831362], [0.63854661787478206, -18.107531003867919, -1.8745821892001346, 2.2815463562775391, 16.90672885108696], [-15.581334170539639, -17.359612433345724, 9.7000263470873911, 7.7525539323616046, -6.0283503811984085], [-1.8389747207041476, 2.3539321781315619, 17.023263741792139, -0.37435329012425422, -10.691798596246633]], [[1.0390128766940228, -3.5302577277047784, 3.8220941531801049, 2.8804299251461991, 0.62119579343263043], [0.56971997479239667, 1.3624071910873694, 2.0644521868199135, 2.1891706039090417, 2.0594060717137435], [0.63087773392022861, -4.3135437011871129, 1.2701937337203342, -0.28338641836402129, 3.7815839254035919], [-3.5717006605843058, -1.8445764436407646, 2.2853862789825037, -1.1968140726233827, -2.6611185688833543]], [[-5.4461573552419713, -0.10813831452783829, -4.4792967814965987, -10.098870174166118, 11.415518301785873], [-11.742539474286735, -4.3862164671692838, -0.090926631185572898, -1.1602202802505561, -10.676311217428433], [3.8351363918090811, -3.6546242070950425, -9.1711568293296786, 8.2415160530259453, 9.1387007520408083], [13.04631897525798, -2.6492243673266058, 3.8071742290846902, 0.84582592086699937, 4.5213266511614014]]], [[[-8.0946489022854298, -9.4371553460958744, 14.337080190595904, 14.079649590878674, -11.784400715770792], [-1.221486384532644, -6.8083329205250172, -9.1930828870561729, 3.4557284162398312, -9.7818103813191364], [-9.3195272086512535, 13.745221943157739, 3.9121915719797387, -11.966400155191627, -13.351195717619682], [-5.303453849148946, 8.9917746095111966, 0.65161525828760791, -16.26179679368035, -10.362880018057663]], [[-0.21901360985996884, 0.46796793938026543, -0.65582184419270584, -0.71659033996581245, -0.60603107041803261], [0.086397886434828389, 0.65182076757280305, -0.30488855550359123, -1.0230951123385048, -0.2171448991741734], [0.29950530153528249, -0.94693272556470287, 0.91633624564824123, 0.66985874844279902, 1.0240498203634991], [-0.77473517400474312, -0.74676802419988142, -0.66446264276387124, -1.0865265067329919, -0.14835335914989095]], [[-0.96758258913003514, -1.6636455870706137, 1.6268725583315844, -1.5935985867270208, 1.9455271159688787], [-1.4612188959961996, 0.025976004735832562, -1.75869729428907, -1.9372531705839076, 0.99618914661932723], [-0.51643596955861748, 1.5311511481458919, -1.8231684776185926, 0.11151431010894905, 0.38270818913537336], [-1.826310789401133, -1.9052210105965444, 1.8153695148358875, -0.7537311498712187, -0.77821562735512972]], [[-1.6000759266645097, 0.9344500717839308, -0.56332365735478873, -2.4732808004418239, 3.060815720652994], [-2.8888816690312282, -0.31180649197367244, 0.48746636841854968, -3.1319571776574056, -1.1076474638495877], [-0.99649731040494016, 3.2277095610283761, -0.71881850068331499, 1.6281917472650229, 0.61913746481625254], [-0.87076136631876033, 2.7010561781946221, -0.052518790689342079, 2.6453662271592449, -3.3184630241470461]], [[18.016846460013241, 0.39262790977057305, 18.96064962102222, -7.9469640895882554, -5.0867324920396388], [-2.3221172660798848, -17.284791307947028, -7.0606022478420165, 9.6571334983789505, -7.0568482721590184], [-2.470054542710999, 16.321851585530428, 16.138843981196707, -9.281854199347519, -18.672387820221559], [-16.592862972108065, 1.2137195848402951, -10.347790803683839, 6.7210740121213872, 15.124855540485132]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank2_Symbol_rank4(self):
      arg0=Data(numpy.array([[4.8366449730166341, -4.7814285454811944, 4.8942628850283949, 1.1245635963102405, 1.4780111639947826], [0.31986635463564816, 4.9057909130430044, 0.89588826357124951, 0.020764560530447618, -0.33939617015053791], [1.4195871664739936, 0.27218794891656817, -0.76580035710170602, -1.8011145428163644, 2.3291272343411169], [-2.0382425937767779, 1.1398595188200051, -3.6666592232271755, 3.6462942808241081, 3.0631606688820554]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[3.6596262195636715, -2.4912885189439127, -2.8168597721378474, 1.03050522685204, 0.82215696371170743], [1.8378403023200267, -1.6065061672465033, -2.8136679795185318, 1.9597764043313077, -0.5327092679774994], [-2.6161479266476952, 3.3500786653633323, -0.2033441724302687, -2.367770530523873, 3.4110787851004112], [-3.8207686780102579, 3.7847071093154021, 2.0969717566538124, 2.2073968103335959, 2.5984120595528815]], [[-0.96866899317703226, -3.9865133630611069, -0.80412911897119521, 3.9705988797970768, -1.2094063962665014], [-3.4587861318496191, 3.3004634398722494, -1.4759983501538088, -4.289131992009013, 3.2607364795018352], [2.9609706252709511, -0.80957723752731159, 3.4310882717064928, -3.6057769063433187, -2.4614506855224381], [1.4140323170039526, 2.3536157545149301, -0.76507493441889096, 3.6535972155945533, -2.5189728873041051]], [[2.3124379948841325, -2.2956464528078646, -0.72923633015622524, 1.4568976543481584, 4.1268136871755061], [2.4418542886546177, -3.5782426538851286, -1.9099500745801343, -4.4954503819691185, -0.80369454889074454], [4.272697986021889, 1.8736748975843156, 0.6182939083135679, 3.3976901065001393, -1.5964130277688335], [-3.9091562796656252, 4.9717464421144424, -2.5843770031472046, 2.0063818217843687, 4.2364909733635407]], [[2.1251479179960855, -1.856665933725905, -0.77330008122484717, 2.9071310950138383, -2.4202435367729969], [3.8944574417170816, 4.9529165899744285, -0.27421085888611607, 3.7524020984862734, 3.6463553972485823], [-3.0431125106537094, -0.23472717360557382, 2.2107891229338259, -1.783315524900535, 0.56837627048156225], [-0.25687347221667078, 1.1589542485141111, -3.9907935723906576, 4.1463436282875072, 2.8760581042142874]], [[0.57754497598129184, 0.19493315100410857, -0.88594131935315712, 0.085690821399355244, 0.56994777851758549], [-3.3344507822932279, 3.4199147237985503, 2.4265319430378689, -0.82925096971458867, 4.9189119312377763], [4.8641863685257398, 0.24320508100857907, -1.8565242322888889, -1.1506051406868147, -0.61601927816032909], [-0.11674241462331292, -2.5757472119708549, -3.3079907461729152, 1.6904828832834529, -1.7442903752850523]]], [[[2.0368650506870098, -0.61209593017475772, 2.3271371179158482, -4.4611116965702378, 0.25845747862028645], [2.3925071059709158, 1.9664015845944522, -3.2519928952847978, -1.8094656541106993, -3.9275542885477011], [-3.1859753785401814, -0.78946582925627595, 2.5069989568561883, 0.81952597939077076, 4.4982877732367506], [3.4114748406087347, 2.2211378341710075, -3.665816152544441, -2.5465698673703741, 3.3624937784065274]], [[0.11860769603047938, -4.1684637099661659, -4.8799050234848451, 0.84774936137909762, 1.8422511325694968], [4.901604335865283, -2.3016488033447979, -2.3670224514703451, -3.4059337199828588, -1.1059646834689385], [-2.2414220388976238, -2.1657437482202813, -0.0043524951152384972, -0.81828091614393728, 3.975913794673513], [3.1955409786694489, -2.0756077114365423, 4.3350251935133279, -3.2823583761780095, 2.9036208159446915]], [[-0.084373945066417377, -0.88507644167324173, 3.9734295995254278, -4.6741514144315293, -2.5319558765105588], [-0.32731307819737943, 3.0584956202908078, -1.1372154559522576, 2.7894448324410401, -3.0577218653464899], [2.7746402771702989, 4.6424925610543326, 3.4861224509649151, 1.5604803060068928, -2.3287567327260064], [1.2623315484505317, -1.980002574517008, -0.19820197407297613, -0.54496880053276087, -3.136960580127115]], [[-3.8384293957717555, -3.038669886843631, -2.7480958666062727, -0.8632087261294572, 4.0544596016228098], [4.5266431260593869, -4.2529122263551482, -0.09279297495781158, -0.11284448585829132, 0.025358297023224452], [4.5066431011481498, -0.67357493622063203, 0.14999739565816927, 1.6796320385378039, 4.9754669245554499], [2.7272393714830407, 2.5825008495237336, -4.9525454094952979, -2.8329038015962538, 2.9532294744282064]], [[-1.6951571464485737, -2.9390029579292887, -4.5932603889949286, -1.4553605670844973, -0.40514306127653033], [-3.9880807136621401, 0.72390551222964561, -1.3620407686001457, -4.1657474652466275, -3.971277382477715], [-3.4215499198474686, 3.97315631590258, -1.7940641988488482, 0.094361674669362294, 3.0888339557923654], [-4.0924109053551625, 0.86429503184164602, 3.0144064702475841, 2.8298600294456255, 2.6366082184439188]]], [[[3.167068468847491, -2.4987016235260286, 2.046516328932463, -0.39734073292862782, 4.0102780383046639], [2.9846642736369358, -1.3289254021316288, -3.7120006912661196, -0.037253214769722653, 1.2512199686506715], [3.9875814400288618, -3.8366569162694528, -4.2553598920986682, 0.63881294566062596, -0.16690485875634131], [1.9139423627016718, 1.8330022506632337, -0.37779773810055506, 2.2597555811943417, 4.4931233733527307]], [[-2.9600863388173182, 4.3594018537474994, -3.7048206342920476, 4.3571899288993787, 0.59588364416270601], [-3.5270370760035883, 0.026245959806288077, 0.48036865154498365, -2.5116447472125292, 3.8589784931164086], [-1.1542006874122568, -3.5384313090102659, -2.5823957633281402, -3.8203580696422978, 2.9470914407757309], [3.7699986345655763, -3.0627849384789774, -2.0198591359609286, -0.47853205378822139, 2.0580830892013093]], [[-0.91307010652610998, -0.11192040099032763, 2.3782805001478922, 3.3823759549887669, -2.0350647956413495], [-3.4921303144423002, -2.4316897420607453, 4.9788428992470344, 2.2434302181359751, 3.2713275533851913], [-1.9165079116680572, -2.3131752956866922, -0.35453765530119163, -0.56408496126708751, 4.9185528714708244], [-4.4412366348697097, -0.79678652388651461, 1.1033396853775193, 3.2414437163563505, 4.9379695740468179]], [[0.18197148454226575, -2.6985308606161187, 4.7378949766920577, 0.73792092359732919, 2.5405112729777581], [-1.8871804621059796, -0.91206022469065573, -0.72942447961541479, -0.70199159440544712, 1.8096122470621721], [-2.1744659372743955, -4.0656005412748497, 4.6591794630830634, 2.7859889687193604, -3.4707843425841411], [-0.10068046607404657, -4.1390904252466214, 0.97670269339587978, 2.3973747046933136, -0.28984506669147159]], [[-0.92842032826084342, 3.8028626336203146, 2.9412339016444689, 4.7311760701713084, -4.1665584044081427], [3.0578335582466849, 2.8486975363551092, -2.2106479039769278, -3.7464513244071873, 4.5500880546010496], [-0.63199469967509181, 4.1922534525127517, 2.123884613717002, 3.10280691367727, -2.9556279225944992], [4.6076468325239954, 1.0848101912606198, 0.14498652943604107, -0.70836138281503569, 4.0091438795283132]]], [[[0.25416765790917317, 1.4493145314048377, -3.622060524694394, -2.060842422064991, 0.83754193200428517], [1.0792642056675259, 4.5652056178852778, -1.8509134488844259, 4.6623689656282004, 4.9175460779821769], [2.4735472402938727, 2.3688275779554289, -1.5325744104015362, 2.5793314866791057, 1.0315575735325968], [-2.5961923910991436, 0.28724256972697937, 3.3411105297031565, -4.8500002824661701, -4.3440298926767573]], [[1.2088056054956056, -3.0847361215580502, 2.5186371552225815, -0.057509144427197789, 3.1276244448705093], [0.49236116015198128, 1.3110609990776485, 1.3794339581221093, -0.71114256054188996, 0.81738946542419288], [1.5741662814305215, 1.2972823249586458, -1.5276038310435314, 4.6442012557853296, -1.4116757751390629], [-2.3735340514361636, -2.0351312150973877, -4.4745314764676269, 2.9868535465711012, -0.84108222208728378]], [[-3.6056724840751642, 4.9069953103434312, 4.2480660854667391, -2.243860208789533, 2.8893681344461353], [-2.199821276769951, 4.9873864254414606, -1.5639951916470252, -0.053234786836346082, 0.28966401375344031], [-3.1450234180577521, 2.8740201365823994, -0.9472541226045772, -4.4472530322761719, -0.65555259031609392], [4.9988814931283638, -4.0604966020690902, -1.1012732260247016, 1.3098803111453794, 3.5735259169531872]], [[4.0020985961100983, 3.4842589353677962, -2.207241528791327, 4.1918832277269189, -4.2290253020969084], [2.2399424406992861, 1.810007256049623, -3.7301778430058286, 2.0369875660296453, -1.3338995492525307], [-0.60789769192690457, -1.7842608221634682, -0.083075321962155613, 2.4353688831119786, -0.50261369263342104], [-4.8486013417204079, -1.1425828124822281, -4.1380116953066608, -4.5649509197968055, -0.35488552282048147]], [[1.3403304274334875, -1.3552261252960052, -0.82591532685016045, -0.14810055289431467, 1.0589157079363449], [4.5048196028916152, -3.4097642376748905, 1.2987093298682906, -4.6824322745741114, -0.50932870518125384], [-2.0411433576323201, 1.952248030215733, -3.404823228490188, 2.4452891377362036, 2.2182316752319871], [-3.5839924179844243, -0.43453833785504958, 4.3617118890505999, 0.0003489250564516766, -1.3335752923680699]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[17.700312757972501, -12.049478091484131, -13.624150656603302, 4.9841879251212857, 3.9764813455668491], [8.8889810594235286, -7.77009997793302, -13.608713088876176, 9.4787426942456339, -2.5765256030427435], [-12.653378718088465, 16.203141136039836, -0.98350356937708672, -11.452065433715219, 16.498177058519591], [-18.479701619557723, 18.305284614610656, 10.142307905377523, 10.676394686152939, 12.567596625662244]], [[4.6316215750991905, 19.061228791082613, 3.8448859237015163, -18.985134826517395, 5.782690266196191], [16.537938743540252, -15.780930104722229, 7.0573806445085685, 20.508178141928514, -15.59097848238193], [-14.157669470001826, 3.8709357132848967, -16.405503404403159, 17.240764628626817, 11.76925057105124], [-6.7610944847556125, -11.253645553731946, 3.6581511308626382, -17.469414020334408, 12.044288868649032]], [[11.317679452290891, -11.23549723112462, -3.5690743050979261, 7.1304401169611191, 20.19771106257026], [11.951076815609708, -17.51286021453549, -9.3477977622747659, -22.001915955958179, -3.9334924015356099], [20.911707171922505, 9.1702575098562953, 3.0260929274982447, 16.629188583071805, -7.8132650309848062], [-19.132438491343148, 24.333034085412688, -12.648620447424275, 9.8197600835548915, 20.734500533690994]], [[2.389863985352898, -2.0879389195775144, -0.86962512036921524, 3.2692537991540895, -2.721717775660057], [4.37956506633454, 5.5698696926462965, -0.30836754961629054, 4.2198147986758165, 4.1005585389551209], [-3.4221735489574203, -0.26396563450162225, 2.4861729667700256, -2.00545172003803, 0.63917526279014758], [-0.28887055571267795, 1.303317757668061, -4.4879011718994297, 4.6628271021650498, 3.2343102448724315]], [[0.85361792220944788, 0.28811337341675325, -1.3094311606482332, 0.12665199068013008, 0.84238917954301706], [-4.9283554820205273, 5.0546721416843905, 3.5864413015999221, -1.2256421909916615, 7.2702067490765696], [7.1893217564322835, 0.35945982487093536, -2.7439635415498209, -1.7006072432848998, -0.9104833703569738], [-0.17254659212496426, -3.8069831349213592, -4.8892472532350002, 2.4985525740350325, -2.5780806479199563]]], [[[0.6515245986480086, -0.19578889387231599, 0.74437286664505087, -1.4269595360043736, 0.082671851514591999], [0.76528252642680117, 0.62898570661398956, -1.0402031127157754, -0.57878718261879802, -1.2562924729113598], [-1.019086330292577, -0.252523556913614, 0.80190461740496155, 0.26213878755693515, 1.4388509121273465], [1.0912160211967448, 0.71046726215959899, -1.1725712494788676, -0.81456202030074754, 1.0755486273839427]], [[0.58186455740329257, -20.449611389701545, -23.939793720724861, 4.1588811135915869, 9.0376988657026214], [24.046246010220095, -11.291407784465214, -11.612117233391995, -16.708798693918666, -5.4256314943084014], [-10.995947870718286, -10.624685999998752, -0.021352430985401082, -4.0143150827354326, 19.505001764951651], [15.676655895413131, -10.182497449807375, 21.266727202150175, -16.102563895204671, 14.244556613783981]], [[-0.075589627136208662, -0.79292959645846084, 3.559748944341441, -4.1875173943441624, -2.2683495536460656], [-0.29323594527041086, 2.7400703304026033, -1.0188179801394548, 2.4990308872633986, -2.7393771324291087], [2.4857676599489498, 4.1591545991654089, 3.1231761891917063, 1.3980159916856472, -2.0863058255617584], [1.1309080189925536, -1.7738610683506459, -0.17756682238863239, -0.48823115240980175, -2.8103661670215407]], [[-0.079703299530352087, -0.06309664479741299, -0.05706300296561885, -0.017924149844125693, 0.084189071816151373], [0.093993755190794767, -0.088309853354832213, -0.0019268053453117885, -0.0023431661571317302, 0.00052655389348781382], [0.093578463462914929, -0.013986487534945706, 0.0031146300015535565, 0.034876821133097355, 0.10331338412217168], [0.056629927010179514, 0.053624495209867964, -0.10283742893525559, -0.058824002465180582, 0.061322512182066496]], [[0.57532984330796044, 0.99748634798230307, 1.5589349845290488, 0.49394380265659343, 0.13750400336031909], [1.3535393204681543, -0.24569075840160509, 0.46227142045178449, 1.4138387355190165, 1.3478363342183894], [1.1612609387751107, -1.3484740370267565, 0.60889851809349216, -0.032025990991772592, -1.0483384148268649], [1.3889485879598376, -0.29333842368719187, -1.0230780112790314, -0.96044365605593374, -0.89485473152729889]]], [[[4.4959297537203398, -3.5471247576052827, 2.9052083165319948, -0.56405980518285059, 5.6929392371698038], [4.2369910990884181, -1.8865254460673515, -5.2695085432639761, -0.052884185596997708, 1.7762158099324858], [5.6607194375348593, -5.4464689204998029, -6.0408542915514278, 0.90685065943727328, -0.23693599551265668], [2.7170080154622069, 2.6021064711594728, -0.53631682053045093, 3.2079200224314683, 6.3783802781958752]], [[-0.80569982917863947, 1.186576649074617, -1.0084075295517314, 1.1859745897870493, 0.16219234689757711], [-0.96001698747010666, 0.0071438339670202404, 0.13075055798784674, -0.68363943215085066, 1.0503674409545043], [-0.31415951774483519, -0.96311836038167165, -0.70289700611112194, -1.0398554271027967, 0.80216277453431994], [1.0261481957606668, -0.83365315037715026, -0.54978131531759677, -0.13025065821144885, 0.56018541474957884]], [[0.69922941363658775, 0.08570868304535903, -1.8212880563012799, -2.5902247141826216, 1.5584533472272557], [2.6742746418456065, 1.8621888728306741, -3.8127996701966724, -1.718019662181288, -2.5051838085790297], [1.4676624431436431, 1.7714304674757133, 0.27150506303565414, 0.43197646477403762, -3.7666295453959791], [3.4011006009564029, 0.61017940452611985, -0.84493792506658816, -2.4822987555107745, -3.7814988631624122]], [[-0.32775148718695812, 4.8603631772944507, -8.5334915448566644, -1.329080106939633, -4.5757517999491544], [3.3990281752179867, 1.642724934614701, 1.3137770381215823, 1.2643672696184975, -3.2593189350422778], [3.9164622224837302, 7.3226122601722148, -8.3917158885502463, -5.0178852476864053, 6.2512801544076311], [0.18133705162349487, 7.4549759589436597, -1.7591534250832317, -4.3179464452032139, 0.52204416478158855]], [[-2.1624090714682502, 8.8573509284232603, 6.8505079828875139, 11.019511035498974, -9.7044446531798751], [7.1220834185945581, 6.634979014325129, -5.1488802386917687, -8.7259618118101265, 10.597734006621495], [-1.4719960669724914, 9.7642916895080241, 4.9467974964063322, 7.2268320855476365, -6.8840334890939072], [10.731795723857221, 2.5266609605559056, 0.33769207432208326, -1.6498637884700333, 9.337806196201397]]], [[[-0.51805534631096184, -2.9540546096889719, 7.3826380386695787, 4.2004968037149641, -1.707113639885228], [-2.1998022739302119, -9.304996539722806, 3.7726106289105137, -9.5030390136463758, -10.023151873003213], [-5.0416893428859737, -4.8282452667018356, 3.1237584414127433, -5.2573032996189326, -2.1025645843071592], [5.2916699131774534, -0.58547004036342543, -6.8099937921570657, 9.8854771555519516, 8.8541867558933323]], [[1.3778685758271458, -3.5161658312058481, 2.8708925358341983, -0.065552345694535857, 3.5650524947797844], [0.56122255509649688, 1.4944253595523236, 1.5723609277490427, -0.8106026168717051, 0.93170916274696169], [1.794328420094071, 1.4787196067010595, -1.7412537678008762, 5.2937370087227293, -1.60911206977987], [-2.7054953817729226, -2.3197636875764807, -5.1003372957113564, 3.4045934463803613, -0.95871557695647203]], [[13.220772269670642, -17.992279613003237, -15.576210693155183, 8.2274707301905963, -10.594328319465619], [8.0659949739199224, -18.287046436642946, 5.7346373945355191, 0.19519382215002098, -1.0620992276660552], [11.531729123086913, -10.538052441540481, 3.4732580653880385, 16.306561348820448, 2.4036879515929717], [-18.32919493259875, 14.888457316859236, 4.0379936314966178, -4.8028847241848878, -13.102901762837753]], [[14.592829222290444, 12.704633428941891, -8.0482521628292769, 15.284839839143167, -15.420270772496403], [8.1674893108970004, 6.5998191059838778, -13.601326135408961, 7.4274561121237159, -4.8637902976333587], [-2.2165738773992478, -6.505940031353175, -0.30291707134822943, 8.880071630188203, -1.8326774329131292], [-17.679427342311222, -4.1661931745218732, -15.088408378579949, -16.645154431097843, -1.2940170522075951]], [[4.1056474486201324, -4.1512753644481473, -2.5299113450342787, -0.45365578866555117, 3.2436289482120095], [13.798986227986475, -10.44465570300633, 3.9781553395627189, -14.343042378179359, -1.5601556572438406], [-6.2523500526491818, 5.9800493820592999, -10.429520598007164, 7.4903135107580541, 6.7948000220389755], [-10.978344612341385, -1.3310607456389703, 13.360624307535049, 0.0010688135093102266, -4.0849553845747595]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[2.1149371243852757, -4.3124551526798607], [-1.2322256213848659, -0.15014174903963529]], [[-4.2031138799677468, -3.8911250056127011], [-4.1585247277711046, 2.7155049490193859]], [[3.6275372307240676, 1.5371545430668281], [-4.7091437452540692, -2.1867634564884386]], [[-1.4899852414491388, -2.5228417312009799], [2.9867836648107531, -3.5418442460723587]], [[3.5507013144291015, 0.23428971295438927], [-1.8937372271181543, -2.5930139413559039]], [[1.3220763117063168, 0.17048779376520518], [-1.0002520826054506, 0.63432962114609026]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[3.3560423688784731, -4.2526375667861842], [-4.5925992577301011, -4.2921780122563522]], [[-2.5430661117436584, -4.6662192047126618], [-4.5008069210609758, -3.6935222580047777]], [[3.5846694122351348, -0.2276422070517059], [-3.291387137883043, 2.4980959787057238]], [[0.41916901885720748, 4.2136965148467969], [-4.1036224434883186, 1.4126777197417875]], [[-2.8065702460171913, 4.1633621590845813], [2.6596035676282046, -4.8329773240548075]], [[4.6729385244811024, -0.60501325229696334], [2.3596475382099253, 1.7572604184604126]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[7.0978185969509866, 18.339308787367024], [5.6591184741281477, 0.64443511394963382]], [[10.688796471945379, 18.156842229127651], [18.716716876155399, -10.029777970925231]], [[13.003521752720712, -0.34992125276328928], [15.499615153571625, -5.4627449970343971]], [[-0.62455565176995487, -10.630489410271629], [-12.256632480761697, -5.0034844532220699]], [[-9.9652926615708477, 0.9754329251770929], [-5.0365902853937863, 12.531977579531066]], [[6.1779813292763341, -0.10314737458282074], [-2.3602423643093022, 1.1146823354970135]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[-0.88729757483062066, 0.68419261458506853], [-2.9004237256757448, 3.0682650166394119]], [[2.8678308232303813, -4.6126429020527739], [0.71301822918534175, 2.525978895109855]]], [[[-1.44077406091357, 2.6998445987493636], [-4.672602628489253, -1.1808701128029608]], [[2.3877733682300475, 3.9053866111747073], [-1.8058851293704894, 4.3512989513343197]]], [[[-2.6127508931454679, -3.0825534978491396], [-1.2924110182577619, 2.3565449915024894]], [[4.6374437436783325, 0.73679939544137962], [0.49179063115908583, 3.3161574492546571]]], [[[2.2362512922792614, -4.6731147875200172], [-4.8967755651459646, -1.4513535761371466]], [[-4.1165039613144563, 0.98208630969241639], [2.2126351559249846, -2.4185023879960488]]], [[[2.6686960915188029, -3.7673839019803901], [2.6762723447694894, -4.7967284149320255]], [[-0.78364351021450318, 0.31811486196310312], [2.0387081103950235, 2.1582809706819441]]], [[[2.9503596114058599, 2.2948305665083399], [2.3907773821951928, -3.6395718354891948]], [[-2.8838582287474832, -2.1861867902584518], [-3.0653399554435912, 4.6963879835931071]]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[0.98421961043042749, -1.7895054986568648], [2.1279357655479947, 2.2669035495510528]], [[-4.4010680539599818, 1.4791218597633113], [2.3899455356800878, 3.3239798348945229]], [[0.34119465979557972, 1.046004572229684], [-3.4059022310515816, -1.2645386867561328]], [[-1.1442410794527369, -2.7329882834718933], [1.9238725075689587, -0.32926775576331391]], [[3.4243573281844384, 3.893283996637642], [-3.4663400542951783, -4.4928625435171927]], [[-0.42702877990683596, -2.6352664675605606], [-3.0962614025925008, -1.6111302691961793]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.87329567343565651, 0.67339578858629179], [5.1903242055315753, -5.4906771186127248]], [[6.1025597782928775, -9.8154078049791931], [1.6163435546348572, 5.7261505234155763]]], [[[6.3409446924609059, -11.88219981421223], [-6.9113486897859611, -1.7466507973880268]], [[5.706648301617208, 9.3336612964817789], [-6.0027257541633938, 14.463629969832962]]], [[[-0.89145665211736502, -1.0517507920003115], [-1.3518678342976407, 2.4649568357765657]], [[-15.794679992970231, -2.5094667047712513], [-0.62188827888488007, -4.1934093859570511]]], [[[-2.5588105926051998, 5.3471699088784517], [13.382830246335379, 3.9665323187578538]], [[-7.919628798471595, 1.8894088512770941], [-0.72854941211442981, 0.7963348536036744]]], [[[9.1385690176895817, -12.900868672830633], [10.41948829053495, -18.675025974071897]], [[2.7163748877450051, -1.102694287889286], [-9.1596353063585152, -9.6968597315628351]]], [[[-1.259888465145051, -0.97995869690896964], [-6.3003354667012097, 9.5912416143425165]], [[8.9291789242196078, 6.7690057775348311], [4.9386619875916375, -7.5664928362560646]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank3_Symbol_rank4(self):
      arg0=Data(numpy.array([[[1.4666648493044319, 0.26283308034277653], [-1.9109325983263572, 2.4424984453035385]], [[0.086545863122671207, -0.46906312603277378], [-2.6030517015592904, -2.882742717455903]], [[4.5478334771011024, -4.4923032440828319], [-4.3918727988620612, 2.7125879021176527]], [[-1.3432716742009045, 3.6426325368128118], [-1.4322701382966496, 0.72175061986321598]], [[-4.9183602201531347, -3.9119619535798944], [2.4329847467635259, -1.0742575338477103]], [[0.061352426707301788, 0.6335221579931547], [-2.563010345531922, 1.1712391255773884]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[2.9393975941102255, -4.8398618699223386], [1.3739181705018142, -0.67915909095765148]], [[-2.8527246260505699, -0.17359795233145547], [2.2536538736908307, -4.7264466657186883]]], [[[4.7869980243242978, -0.63662569025231441], [4.2729795591099275, 3.7630993103348658]], [[1.7573363295339561, -3.1418088334872296], [-1.8120535881142033, 4.3573087176377303]]], [[[1.8552623817412854, -0.81503246731913137], [4.7262255911835052, -4.6061362499585652]], [[1.0285544034860195, 4.9073354157812883], [-3.9551048628828269, -2.3997676968770412]]], [[[2.6444864514594517, 1.7392724968111466], [2.7051045231295943, -4.7044434059046019]], [[4.3808963591283252, 3.2964973219345115], [4.7306195170713359, -0.41858706699326653]]], [[[-1.7413814796828611, -0.91478514426336144], [0.77198714715826977, 1.8447525756955709]], [[1.3343057654045936, 2.4428540042369598], [2.9269473081215338, 3.7260049911177457]]], [[[4.0240571076093214, -0.36144688748302567], [-1.2772652139496921, -4.7548020874839807]], [[-1.094253262647833, -4.011154252667275], [-2.7915976073994031, 4.2714782185194284]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[4.3111111294114837, -7.0984552801039129], [0.36111114489190388, -0.17850547591919949]], [[5.4513644819684011, 0.33173398611288329], [5.5045460827421513, -11.544338632827991]]], [[[0.41429487578166818, -0.055097319848952879], [-2.0042971494702462, -1.7651311260774472]], [[-4.5744373228053226, 8.1782908299829433], [5.2236842847760583, -12.560999973477287]]], [[[8.4374243684893422, -3.7066319397982559], [-21.231638555540961, 20.692160818376394]], [[-4.5172801068200421, -21.552392927462282], [-10.728569602662654, -6.5095808224414045]]], [[[-3.5522637430535466, -2.3363154786830962], [9.8537017514313661, -17.136558617942583]], [[-6.2746270341520152, -4.7214746751816783], [3.4143275687832642, -0.30211547506911562]]], [[[8.5647413977835871, 4.499242863531963], [-3.0199843483358344, -7.2166018898895876]], [[3.2463455747480077, 5.9434265308787246], [-3.1442951969248329, -4.0026889328624096]]], [[[0.24688566876059773, -0.022175643672884687], [-0.80917581467099731, -3.0122724792932081]], [[2.8045824327984552, 10.280629847110591], [-3.2696283406544064, 5.0029224135815564]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[0.49855251833949232, 2.2191075525436945, 3.5659607429410691, 3.9630075854976639], [0.90806184753108976, -0.22732828117510895, -2.6049465959200191, -4.1681213697043074], [-2.7841162334085534, 2.0638240301124808, 3.8369745485995246, -0.52243200443811233]], [[-0.56310447670673547, 2.4644927969104593, -2.5247512838752098, 1.9078800489924213], [0.80048756793010067, 1.7258863525849035, 1.9550407393404807, 0.20037804296539719], [1.9829042092944826, 0.043860909627177591, 4.5886620776247842, -3.3503470476189898]]], [[[4.1624683388382984, 4.1085435482524435, -0.5581427324784638, 1.5819284702726186], [2.1279716888065323, -4.8604016279632125, 2.5420360430304729, 4.3687302497804801], [-0.34740372899276917, -4.3174470574241042, 2.3483423207541696, -2.6028764625223033]], [[2.7687864562093214, -0.23227498060413687, -0.38393792754334211, 0.58225933359315363], [0.91946852555657266, 2.3362675996162787, 1.746616011073181, 2.6317413319653244], [-1.3364131928801193, 3.6291430734053201, 0.71651392089301158, 1.9410010211896616]]], [[[-4.3750547314926269, 2.6099246535085419, -1.4924069972894838, -4.7156262412782057], [-3.0095732758179174, -1.0232334593261139, 2.848536753881266, 0.65159347074979124], [1.5356227287962092, 1.8306279761431181, 2.6606824091739387, -3.4713776264019303]], [[2.9131526391234921, -1.7336432071985133, -0.23184337545387557, -4.5630739783674414], [-3.9407776028826835, -4.73869997420839, 4.5070058777064776, 4.9993668893003242], [-1.7705129217497984, -1.6975265983414936, 2.5439369190909114, 4.3674397324396264]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[-0.32184322514491637, 2.7751471635181559, -4.6425212299544238, -0.48640860625312943], [-3.7452531549323864, 4.5808445338289161, -4.9358027195115586, 4.5962412341115737], [-2.6659289991769475, -3.2296454174364699, 0.87160580907958618, -3.6232662935589701]], [[-1.4019622811879739, -0.30136963910891801, -0.87208265085938397, -2.8709649345393484], [2.5448607762572886, 1.1633709972122483, 2.3249555066770515, 4.6030993632877912], [-1.7097392702355787, 3.6266415142919168, -1.4658939234415582, 2.7186516289233378]]], [[[0.073345307355815237, -3.7177761182583389, 0.53007005560107867, 3.0766299750202322], [-2.4124136264605376, -0.93464648491723867, -3.8804861296689399, 1.5770833467926675], [1.0807174878739998, -0.2651379423706075, -4.3039661084066463, -0.74912876477455548]], [[-4.3757375509845051, 3.053524132985002, -1.2656350939240668, 1.1345435954843266], [1.1787569782455591, 3.01579980563851, -4.2538904758570197, 0.78445650006868917], [-1.9201093726588017, -4.8384658543413286, 0.61015050965251749, 2.2629358764973375]]], [[[1.5283854062328359, -0.86506440980311883, 2.2646934050026761, -2.5243688838714973], [-3.0139522634377389, 1.3320963820908771, 2.9247637845552932, 3.8027636538758038], [4.594934635799266, 0.46935644382090214, -2.4781425982073237, 0.81856586017616717]], [[2.4604982792002783, -1.5103786033453046, 2.1461294362776666, -2.8701688539098837], [0.0170008954263281, 3.0406169802634277, -1.5159430608522384, 4.4788153751057305], [2.9265717158055917, 1.8769755571978362, -1.910607284334719, -0.096445006405433631]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.16045575040650228, 6.1583500299833513, -16.555048454287963, -1.9276409962324983], [-3.4009214993395456, -1.0413555142057207, 12.857502492324407, -19.157691308216549], [7.4222562037231574, -6.6654198212480411, 3.3443293058498686, 1.8929102723570628]], [[0.7894512367109352, -0.74272330479143311, 2.2017917924025259, -5.4774567199644553], [2.0371294135069049, 2.0078461270817094, 4.5453827327076244, 0.9223600419908734], [-3.3902491957462058, 0.15906779570852825, -6.726491856316887, -9.1084264584678625]]], [[[0.30529751967094465, -15.274645084517312, -0.29585474923819727, 4.8670085499786406], [-5.1335478987991214, 4.5427572968618408, -9.8643356060982672, 6.8898517235581656], [-0.37544528527512533, 1.1447190290994611, -10.107185759462956, 1.9498896292300976]], [[-12.115482867092442, -0.70925725876335521, 0.48592531498732933, 0.66059859783908448], [1.0838299407769652, 7.0457153728423219, -7.4299132144835838, 2.0644865943596287], [2.5660594973939919, -17.559484841190987, 0.43718133400599463, 4.392360847168054]]], [[[-6.6867698030832496, -2.2577529300179764, -3.3798442843413405, 11.903980151450607], [9.0707101866331428, -1.363045589202649, 8.3312971367266222, 2.4778559676700929], [7.0560860640662844, 0.85921703684158923, -6.5935504184748259, -2.8415512127519977]], [[7.1678070556111013, 2.6184576059875648, -0.49756589266753737, 13.096792810796893], [-0.066996747925024427, -14.408571605951897, -6.8323642855293869, 22.391241289592802], [-5.1815330392612786, -3.1862159327801725, -4.8604644085031179, -0.42121775297048508]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.8058379579)+(1.-msk_arg0)*(3.18551400532)
      arg1=2.41464387868
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(9.18974332827)+(1.-msk_ref)*(7.69188189338)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.4111623532295168, -2.8810948343929113])+(1.-msk_arg0)*numpy.array([-0.33341658100287752, -4.8085719595835723])
      arg1=0.999337612076
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.4102276163277352, -2.8791864319663043])+(1.-msk_ref)*numpy.array([-0.33319572988591284, -4.8053868195851814])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.7618687563419027, 3.8336715894444513, 1.1963334137322441, 0.31203783361051141, 3.8846179480500211], [-3.6529958372284388, -1.5206561693159006, -4.7622832990732524, 1.6331499489945909, -1.5276491843758877], [-4.8501842680068661, -4.1708322831483731, 0.5060396484683638, 2.551247460483137, 1.6479835084262682], [-2.9834742458316601, 1.2938682464073912, -3.0389210892738072, 1.4578842065133299, -2.498817408913653]])+(1.-msk_arg0)*numpy.array([[0.89906512359125834, 3.7395011280385635, 3.4209440882851734, -2.3336948645707736, -1.2513120541259557], [-1.1857863424646622, 4.9531727538746715, 1.3609389593468944, 3.0567771077481645, 2.8994058620605383], [-4.2368482400313496, -0.011792606820821838, 0.10962537038014197, -1.0425877121650071, 0.049304043922039575], [4.4543474496006841, -4.7100027357191117, -0.40702421670241939, 0.60601996301544592, -3.9055432992525043]])
      arg1=3.35382162582
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[5.9089935368804936, 12.857450682980177, 4.0122888746692746, 1.046519234437741, 13.028315682228445], [-12.251496437936364, -5.1000095460920924, -15.971848716725269, 5.4772936171489972, -5.1234628712299797], [-16.266652887265575, -13.98822750890171, 1.6971667165568221, 8.5564289057931191, 5.5270427295588798], [-10.006040445754742, 4.3394033057661634, -10.191999268374611, 4.8894835797495046, -8.380587864996393]])+(1.-msk_ref)*numpy.array([[3.0153040545231549, 12.541619753003426, 11.473236264020503, -7.8267963048684068, -4.1966774277800241], [-3.9769158789629677, 16.612057898379739, 4.5643465132820031, 10.251885169284996, 9.7240900822152074], [-14.209633252745205, -0.039550299780494381, 0.36766393791972318, -3.496653215875821, 0.16535696874623948], [14.939086805398142, -15.796509032737967, -1.365086620210044, 2.0324828576413636, -13.098495577619266]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.05335991834408027, 2.0656418293398779], [4.166464580537161, -1.8609292355601523]], [[1.8951981876592932, -2.16678442624195], [0.44379909857517408, 1.8028431676541823]], [[-1.7424380794335672, -1.2500169302816397], [4.7729409304097921, -1.7552649797467068]], [[-3.9005639936997305, 0.59631262325776291], [1.733946814492259, 4.9708179909056422]], [[-2.1941090910067675, 4.5593293472719871], [-2.8323036202920369, -0.91980388146378367]], [[0.8678406825050704, 2.0895015245493926], [-4.1511667613932506, -3.4855790530079411]]])+(1.-msk_arg0)*numpy.array([[[-1.5328032621280019, 0.050600684796981987], [-2.2437152916593583, -2.5966088569306689]], [[-1.0172708207699834, -4.9069940008030724], [-4.2475905730070398, -1.4231197608729609]], [[2.2927979823064408, -2.4592273293366085], [-4.160406635605379, -1.252800503659337]], [[-4.0564778946341438, -4.1871522825556244], [0.88877072173187877, -4.4409867651274713]], [[-2.6635531056620931, -2.4270384674461356], [4.2872569228124284, 1.1733299794737997]], [[-0.036544369967246304, -1.0400462832352186], [0.92006326230107227, 3.3891479283577848]]])
      arg1=-4.79153672866
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.25567600858405998, -9.8975986935423617], [-19.963768066312664, 8.9167107816271649]], [[-9.080911724262954, 10.382227161430885], [-2.1264796809699855, -8.6383892538321607]], [[8.3489560550250221, 5.9895020328936663], [-22.869721771792175, 8.4104166189903147]], [[18.689695638308365, -2.8572538361042916], [-8.3082698471859437, -23.817856974917692]], [[10.513154296249882, -21.846194025519772], [13.571086823351337, 4.4072740811994873]], [[-4.1582905048500471, -10.01192329947343], [19.890468004016192, 16.701280053142082]]])+(1.-msk_ref)*numpy.array([[[7.344483128299081, -0.24245503970018253], [10.750844228646141, 12.441746707952069]], [[4.8742905007154027, 23.512041982171475], [20.352486238881237, 6.8189306035073187]], [[-10.986025743623188, 11.783478072645451], [19.934741200671819, 6.0028396269698288]], [[19.436762821144558, 20.062893950365744], [-4.2585775565376336, 21.279151196609636]], [[12.762512534521365, 11.629243958643425], [-20.542549010865706, -5.6220536914888131]], [[0.17510369092386921, 4.9834199656298379], [-4.4085169140080671, -16.239226777594684]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.076343554080184006, -3.8878219335232633, 4.9927299526374451, -2.3121999588828923], [0.80463504888614867, 0.63401403947370394, -3.7894134291272197, 0.0037816585532288727], [-3.0653700600135103, 1.1273136570280906, 3.3661518211271542, 1.4395957301823472]], [[1.6012373263137167, 2.8650266870129206, -1.9650893808379077, 0.59719831896948783], [-3.1296038187380981, 2.3347415692958311, -4.5186685111087845, -0.48766622873194265], [-0.13728180485692754, -2.3670466158271877, -1.2752086885088287, 4.9236899759783057]]], [[[3.4624646486105259, 1.8460990207366645, -2.4340571524473553, -2.9074446791571376], [-4.9776296076932969, 1.7700052855775352, -3.8009465657352504, -0.73017450752028257], [0.34040996243511579, -1.6418262634709579, -3.7489077682741612, 1.4991446343854467]], [[4.6265731672847448, -2.5792523959082017, 3.2855481116813028, 3.6498826773759188], [-4.4140347713126973, 0.88537381718270947, -4.8399116325934539, -2.5245669003959801], [-0.49658091592279607, 1.6328209422224136, 1.9577637263620407, -2.8280707036450492]]], [[[-0.67612703331667134, -2.8195066080113298, 3.3739278486094832, -0.1754315755607756], [0.10045468875662689, -0.391065205841854, -4.256295150511626, -1.3616339255943677], [4.267466534683491, -3.1014201030441981, 1.9203626911759315, -4.722758146975564]], [[3.6092852612549962, -2.1113011390485195, -0.086530947884220666, 2.63875554695856], [1.6647268380455298, 3.9161750651085363, 1.1131362307238319, -4.43735505218649], [-4.3134538787397689, -3.3967871590574559, -1.0044632400491471, -3.7896822874877945]]]])+(1.-msk_arg0)*numpy.array([[[[1.9006211108070996, -4.0608309884608316, -0.90543172955276674, -4.7861264171217019], [-4.9196739097873525, 4.9656806236571072, -4.9162217952137421, -3.4448154358418459], [0.22589584403065821, -1.1482640173380476, 1.3534546946526538, 0.78832178599895553]], [[2.3657638259708396, -3.0558069937538024, -0.26150740784987558, 1.559587327279508], [-2.049178361313726, 4.3733261948239086, -4.4591267991418784, 1.1102516204929982], [0.53317421438367685, 2.5540073639440184, -3.2899444336649877, 0.29727453259717596]]], [[[2.2880086523793262, -0.97279333839418314, -4.5379812197609306, -1.6353001816412158], [2.0965450287649832, -1.1321318863794447, -2.5588020144055967, -3.1228595526361493], [-3.6770649958768264, -0.86284953006875753, 2.5953304457089281, 3.4654805501337833]], [[-2.6181130877174863, -3.9654277800055606, -0.20775140423625693, -2.6269224213487261], [1.5650441379802853, -1.6382330840529171, -0.80381139518375999, -1.0491795645181687], [-4.5948429120911811, -2.1360430476306735, 0.093283250890369374, -0.086198182586223027]]], [[[-0.20127323625082916, -1.4459152301473646, -3.5744389600288673, 2.0812211512734216], [1.5473201098495917, 3.9747615350435446, -1.3789472882802167, -0.60170738436110316], [-4.1132205570847882, -4.6459538960764313, -2.0815231901663536, -4.9984714136972483]], [[1.5346719892259291, 2.8401777860985629, 2.1219699533426484, 0.21523599034586205], [0.19369260900184226, -1.0391397291061444, 1.1633791990239217, 2.4887951413781408], [3.1726205039337749, -0.52586329685989863, -4.1492284320734942, 0.38890328481627279]]]])
      arg1=-3.58557378609
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.27373544624658891, 13.940072409813352, -17.901801639185983, 8.2905635607609049], [-2.8850783386526468, -2.2733041199477579, 13.587221456122913, -0.013559415776387447], [10.991110531839007, -4.0420662973372892, -12.069585729821037, -5.1617767127039551]], [[-5.7413545825337833, -10.272764585391933, 7.0459729712494577, -2.1412986375919516], [11.221425413303697, -8.3713881681537146, 16.202019361446414, 1.7485632461009428], [0.49223404080164906, 8.4872202961548524, 4.5723548453070695, -17.654253708684944]]], [[[-12.414922479309292, -6.6193242552734404, 8.7274915196517195, 10.424857426082662], [17.847658238193379, -6.3464845532014538, 13.628574368416023, 2.618094573433384], [-1.2205650378300594, 5.8868892116099021, 13.441985420380075, -5.3752937026047674]], [[-16.588919468027619, 9.2480997784693848, -11.780575182170654, -13.086923650290499], [15.826847366893361, -3.1745731497777117, 17.353860276802461, 9.052020899281608], [1.7805275148036284, -5.8545999678058331, -7.0197062965948609, 10.140256180189041]]], [[[2.4243033667247262, 10.10954898338324, -12.097467250121564, 0.62902285858257945], [-0.36018769869524497, 1.4021931507170937, 15.261260317521806, 4.8822389098572936], [-15.301316139762749, 11.120370621117134, -6.8856021252590471, 16.933797809822288]], [[-12.941358619264472, 7.5702260187070527, 0.31026309841888333, -9.4614527170650984], [-5.9690009114908067, -14.041734655178969, -3.9912320892265654, 15.910463954678608], [15.466207155102682, 12.179430994431829, 3.6015770626077868, 13.58818546761276]]]])+(1.-msk_ref)*numpy.array([[[[-6.8148172321926426, 14.560409141953105, 3.2464922745754099, 17.161009418127932], [17.639853807027475, -17.804814274262849, 17.627475995505769, 12.351639924660848], [-0.80996621674222469, 4.1171853600737389, -4.8529116738023355, -2.8265859308787884]], [[-8.4826207584728728, 10.95682145214359, 0.93765410645395886, -5.5920154378061939], [7.347480215342296, -15.680883762166262, 15.988528159839159, -3.9808891063998106], [-1.9117354865114038, -9.1575818536296918, 11.796338519030551, -1.0658997713515668]]], [[[-8.2038238463105326, 3.4880222934258089, 16.271266503327876, 5.8634894636753225], [-7.5173168964898025, 4.0593424141948571, 9.1747734266381755, 11.197243349562157], [13.184387858952409, 3.0938106563516383, -9.3057488123662626, -12.425736216752501]], [[9.3874376563299435, 14.218333898607334, 0.7449079890521938, 9.4190241720710084], [-5.6115812352105374, 5.874005601679869, 2.8821250675285488, 3.7619107434340515], [16.475148296779739, 7.6589399575369743, -0.33447397907344356, 0.30906994388946368]]], [[[0.72168003974177886, 5.1844357461196964, 12.816414635045987, -7.462372003054857], [-5.5480304245612624, -14.251800765997283, 4.9443172492526823, 2.1574662242598777], [14.748255805875546, 16.658410501138341, 7.4634549857917332, 17.922388071455849]], [[-5.5026796548097963, -10.183667017660349, -7.608479839568707, -0.77174452480650146], [-0.694499141395718, 3.7259121727640734, -4.1713819592985448, -8.9237586178650385], [-11.375664912105654, 1.885521652285904, 14.877364698527732, -1.3944414233601803]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.55810783584)+(1.-msk_arg0)*(-4.93672812378)
      arg1=numpy.array(-3.710847871)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(9.4927490164)+(1.-msk_ref)*(18.3194470478)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.1670474422113557, -0.39503408141001284])+(1.-msk_arg0)*numpy.array([1.5207707589334021, -1.0864524335386347])
      arg1=numpy.array(-0.171185898664)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.54215386250785613, 0.067624264229253245])+(1.-msk_ref)*numpy.array([-0.26033450903061167, 0.18598533619147523])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.10111642642)+(1.-msk_arg0)*(-4.77250770553)
      arg1=numpy.array([4.2201538701909573, 2.619622498366204])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([13.08718848888893, 8.1237543607151359])+(1.-msk_ref)*numpy.array([-20.140716864011793, -12.502168559034443])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.60046228872201013, 2.7938411889913484, 0.74945024238258284, 0.44225252316652064, -3.0632732514315908], [3.6226405525878853, -4.6776459796079912, -2.8046460516317095, 0.19378649033537343, 0.42353023159067948], [-0.47763392064339794, -2.339880322157073, 4.1094130397632718, -0.34523282405308908, 2.2887533589367424], [-0.27191956776708714, 4.8493544059426359, 0.90872071771640961, -2.6783210070931132, 3.4077841404881752]])+(1.-msk_arg0)*numpy.array([[1.8402685532044574, 0.27721945315576679, 3.1381501869394839, -0.46856927866041964, -4.6084770492358951], [4.5026865112790162, -0.10746479415758081, -2.9713621919320108, 3.3968499671572925, -3.8872315154046557], [4.082995420863341, -2.3399420138253135, 1.3125735828453911, -3.5051964376870961, 4.0794905810339372], [0.92902122418051469, -1.2196760816062167, -1.5249495754389364, -2.4905393583034376, 3.1215470951072142]])
      arg1=numpy.array(3.19259707113)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.9170341442973902, 8.9196091971737363, 2.3926926487876585, 1.4119341101609064, -9.7797972105887379], [11.565631617945735, -14.933838854275473, -8.9541047699933056, 0.61868218146910881, 1.3521613769110501], [-1.5248926561180407, -7.470295063311382, 13.119700034808121, -1.1021893029295342, 7.3070672702784272], [-0.86812961563590441, 15.482034673279655, 2.9011791018557802, -8.5507998027891237, 10.879681665962885]])+(1.-msk_ref)*numpy.array([[5.8752359930516125, 0.88505001420512319, 10.018849095586363, -1.49595290667235, -14.713010329756385], [14.375263768122077, -0.34309178707698851, -9.4863622312260016, 10.844773256211491, -12.410363950881797], [13.035359222081997, -7.4704920199507194, 4.1905185762336785, -11.190679880692121, 13.024169680707866], [2.9659904393355201, -3.8939342858622745, -4.8685495481659755, -7.9512886608514055, 9.9658421132309698]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.86508956949)+(1.-msk_arg0)*(2.4848410984)
      arg1=numpy.array([[-2.8253810293834727, -4.1364881160797795, 1.9373024456445176, 4.7377880402176906, 3.2032670334527147], [3.2096966730179783, 4.8024317781031627, -0.21499986311599706, -3.1063167916696943, -0.21408637980647516], [2.7284310058458781, -4.8739015552714271, 3.6513375343370971, 2.5835208521379833, -2.260898461302391], [0.35881462148445564, -0.23041126873207673, 1.2376966359496597, 4.8143669124582207, -3.4598827610843799]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-13.745731775892706, -20.124385187865304, 9.4251499212555281, 23.049763176924689, 15.5841810327468], [15.615461805131355, 23.364260751844242, -1.0459935914877387, -15.112509422688548, -1.0415494133666792], [13.274061227617999, -23.711967619279449, 17.764084152996244, 12.56906035030031, -10.999473521761658], [1.7456652723650679, -1.1209714602017309, 6.0215049937534149, 23.422326249505513, -16.832639532615087]])+(1.-msk_ref)*numpy.array([[-7.0206229004441889, -10.278515673867155, 4.8138887369631442, 11.772650437828229, 7.9596095738645793], [7.9755862065042278, 11.933279854480075, -0.53424049602042833, -7.7187036285825732, -0.53197063515022835], [6.7797174974673862, -12.110870894081076, 9.072993569441568, 6.4196387919589357, -5.6179734159474517], [0.89159731817045462, -0.57253539007933441, 3.0754794683558249, 11.962936766840416, -8.5972588803788756]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-2.5539243008441073, -3.2949285466660116], [-3.3497676933037357, 3.0725120991294421]], [[1.1667144717787004, 0.060803703123565533], [-2.4025959370759784, -4.0710757642329556]], [[4.0896293130893682, 1.8494523375053182], [-3.6953316002355518, 1.77612194937638]], [[4.0752135804874605, -1.1557575744485362], [2.8405340292488335, 2.8815338822230476]], [[-3.7683958804114126, 4.0460022545231809], [-3.5317868614562373, 1.3024497324365605]], [[-3.1458754468886418, 0.69857738574209538], [-3.1176420971289218, -1.8053050128264738]]])+(1.-msk_arg0)*numpy.array([[[0.69721151096448963, 2.5848882587780428], [-3.9587684314269445, -3.2587623979266445]], [[3.9090573083979319, -0.26526103248723665], [0.71359275710798808, -1.6515891005283621]], [[-3.7702382988956886, -3.466101557404655], [-4.6765776670866677, 3.2748242439035717]], [[-0.20275043588000674, 2.7095533628704267], [4.9339632511974543, 2.7064510740092373]], [[-3.1321918256992474, -2.309863914487055], [0.3933393744800906, -1.068744110122033]], [[-2.1394980202646474, -0.45226175999825458], [0.62414570401566838, -3.886303951791239]]])
      arg1=numpy.array(-4.47610025081)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[11.431621203555538, 14.748430494129506], [14.993896012148776, -13.752872177527019]], [[-5.2223309397511439, -0.27216347080150838], [10.754260276538499, 18.22254324934562]], [[-18.305590794035208, -8.2783340717668672], [16.540674702636821, -7.9500999030710053]], [[-18.241064529720212, 5.173286768863508], [-12.714515080752236, -12.898034532933249]], [[16.867717745457171, -18.110311706245035], [15.808632056368236, -5.8298955740254179]], [[14.081253876832161, -3.1269024115296968], [13.9548785728915, 8.0807262206993453]]])+(1.-msk_ref)*numpy.array([[[-3.120788619095082, -11.570218983429667], [17.719844368804942, 14.586547186686429]], [[-17.497332398546781, 1.1873349740459773], [-3.1941127190665597, 7.3926783871084307]], [[16.875964595296733, 15.514618050428481], [20.932830468574455, -14.658441619492208]], [[0.9075312768941346, -12.128232487124718], [-22.084914146167371, -12.114346331175065]], [[14.020004616594337, 10.339182446970192], [-1.7606264727633931, 4.7838057793678859]], [[9.5766076251119721, 2.0243689773595124], [-2.7937387422859001, 17.395486093332817]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.72505534735)+(1.-msk_arg0)*(-2.9970844293)
      arg1=numpy.array([[[-0.87069760453054279, 3.9298064125543384], [-4.1448933113610265, 2.6400169919927094]], [[1.0816332293038142, -1.4407440946401415], [-1.2410800274954337, 2.9152228499098145]], [[-2.4625083088658304, -4.8417663335223864], [0.73968621713649974, 4.9684292374182331]], [[-1.3379354002854562, 2.0055815939872286], [3.2343800548551478, 1.224809095525857]], [[1.7165910063042134, -4.8228475914426649], [0.20423550799837731, -2.3015316537085031]], [[-2.9077290747241236, 4.6756289727044891], [-1.928027088013311, 3.1468618772579795]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.2433967676847977, -14.638746391151493], [15.43995699369713, -9.8342094131278071]], [[-4.0291436446939253, 5.3668514939079257], [4.623091792916088, -10.859366465785058]], [[9.1729797438442642, 18.03584757132603], [-2.7553720985083543, -18.507673898794639]], [[4.9838834172475242, -7.4709024412368414], [-12.048244718713223, -4.5624816707763829]], [[-6.3943965072532567, 17.965374209976755], [-0.76078857118891163, 8.5733327937513035]], [[10.831451738457732, -17.416976707016044], [7.1820076140473033, -11.722234663264215]]])+(1.-msk_ref)*numpy.array([[[2.6095542331659387, -11.777961609223761], [12.422595204583374, -7.9123538197846477]], [[-3.2417461097582483, 4.3180316926496429], [3.7196216259198414, -8.7371690113997218]], [[7.3803453095198082, 14.511182488501332], [-2.2169020439464666, -14.890801905537298]], [[4.0099053556027133, -6.0108973670266641], [-9.693710100839791, -3.6708562690636488]], [[-5.1447681764680944, 14.454481421192284], [-0.61211106093179313, 6.8978846828672387]], [[8.7147095344740251, -14.013254791269274], [5.7784599647503034, -9.4314107334827426]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.7987785557255211, -3.2873183786508129, 3.6560296976435325, -2.7768101237025746], [0.77471512421461597, -4.0421311759255083, -4.8016641499074311, -1.7867842558704403], [-4.0875482699131362, -3.0229152908348422, -0.55389992751347616, 3.7308599465123518]], [[-2.2235928260487059, 0.73346089983424889, 0.066545147067875732, -1.2117675348124433], [-4.3346056431956592, -2.9529767457243041, -4.2298754037561146, 3.5708025401606562], [-4.3805401559322448, 2.8960992656691218, -3.3114125641066838, 1.0517624499876295]]], [[[-1.1318591568843606, -3.1669581203531774, -4.3182746471174251, 2.7625384034603844], [1.4100045102635015, -3.1430018026515505, -4.106477812963151, 1.2045762954200825], [-1.2403882359769902, -4.7740913193819532, 3.6134194652421581, -0.86339216498545834]], [[0.69504580977481911, 2.4698588987133165, -4.4919237470341127, 1.5984001061333339], [-3.3380345245771794, -4.8937704053650553, -1.9574257775153958, -1.2866591877738163], [4.885468308868127, -1.5058049062640197, -3.704862303421832, 2.3431002428713441]]], [[[2.8384236011533854, -1.7742610086901833, -4.4125168035340625, -2.1387075899544774], [-0.18560158907244606, 4.850524665357165, -0.84669243213562062, -3.0024513759837603], [-2.7604305765957085, 2.698291267682837, 4.9772449797004175, -2.1310548449624034]], [[1.0661292327250669, 2.5609959586077657, -1.7374316974790496, -1.4309471783689007], [1.0849402545641764, -1.5346972099498446, -0.53268250124794836, 2.9575901379991194], [-0.1938819515750998, -0.83702791347917227, -4.1013744547549695, 4.9086482211854108]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0199781894736111, -4.2651153078332458, -2.7215544093239119, 4.3460304689389613], [-3.9465830462962281, -4.240722724066889, -1.939405021372024, 1.4438121000285342], [0.7159177134098087, -3.5061634526613137, -3.8710068032715625, -3.8998917079715936]], [[2.4021666086453237, -1.9465045218227397, -3.2559274243447032, -2.4310178362518964], [2.8338910158094874, 0.65646925919358967, 1.5125682960226854, -3.7383026197012512], [-3.1346687177307864, -0.37307296766811504, 4.0755973857718786, 4.4040247010137161]]], [[[-1.4647578694413421, 3.0466280121945601, -2.4131111580480402, -1.4065327257536708], [-3.5246046626834779, 3.8772205026553426, 4.1505799115512403, 0.09508724665952073], [1.5392590321048862, 2.9310719497986284, -1.3778507078981841, -4.9916279208769563]], [[4.368257349311115, 0.10733693499599628, -1.1478339290773452, -2.710636347715857], [-1.3994121123738337, 2.7139786507913977, 0.70823992706433536, 3.0015685289977938], [4.8849419301081713, 1.6997196075939698, -3.4800528343126658, -0.22265328028273057]]], [[[2.6017003386877953, 2.8818204892294164, -0.62897594316405581, -3.2409863385236024], [-3.9925654186442072, 3.1951838371644179, 4.2982010351407638, -3.7010968779315401], [4.2624465382927994, -1.1933904615704574, 1.688802170387258, 2.7800965184865145]], [[-0.29506566408603074, -4.0202704286684519, -0.7448281726260273, -1.2798901833390195], [-0.95840022253564161, 4.0148981427738892, -0.66873076566587564, -0.78784017569925346], [-0.90679452422518825, -4.2526014080556607, 4.3100089995912914, -0.39842835023398138]]]])
      arg1=numpy.array(-0.634810646045)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.4115050691407434, 2.0868247037058283, -2.3208865743195775, 1.7627486285710017], [-0.49179740850325443, 2.5659879031865596, 3.0481475210922508, 1.1342696678115529], [2.5948191579623168, 1.9189788087131672, 0.35162157082892376, -2.3683896129476758]], [[1.4115603984442648, -0.46560878767228198, -0.04224356780129554, 0.76924293163024005], [2.7516538087058939, 1.8745810757081225, 2.6851699377468639, -2.2667834674173224], [2.7808135264119525, -1.8384746458488994, 2.1021199491409899, -0.66767000036216828]]], [[[0.71851624261333247, 2.0104187303778032, 2.741286718534917, -1.7536887886238883], [-0.89508587408626705, 1.9952110048607823, 2.6068358334152273, -0.76467785630571361], [0.78741165742675745, 3.0306439947330923, -2.2938371451607478, 0.54809053804432217]], [[-0.44122247953379129, -1.5678927231313691, 2.8515210158381041, -1.0146814040123653], [2.1190198530662405, 3.1066175526240603, 1.2425947224090321, 0.81678495023000219], [-3.1013472933833199, 0.95590098536269086, 2.3518860323417443, -1.487424978924573]]], [[[-1.8018615199966095, 1.126319777178477, 2.8011126427344051, 1.3576743468796335], [0.1178218646659962, -3.079164696470972, 0.53748936984514328, 1.9059880977059489], [1.7523507176901729, -1.7129040228544239, -3.1596081010861949, 1.3528163028872]], [[-0.67679018699330429, -1.6257474990015759, 1.1029401383351571, 0.90838050275615456], [-0.68873162391974985, 0.97424212733120807, 0.33815252275389895, -1.8775097062385555], [0.12307832693578985, 0.53135423051313257, 2.6035961672940942, -3.1160621484967139]]]])+(1.-msk_ref)*numpy.array([[[[2.551924951545212, 2.7075406040206196, 1.7276717128286234, -2.7589064097169471], [2.5053329332882366, 2.6920559321612001, 1.2311549545594449, -0.91654729198622054], [-0.45447218616450108, 2.2257498865221272, 2.4573563296281198, 2.4756927746416855]], [[-1.5249209367410637, 1.2356617930271581, 2.066897391722808, 1.5432360031771737], [-1.7989841865661971, -0.41673367453714633, -0.96019445718484131, 2.3731143011230205], [1.989921073838687, 0.23683069162719711, -2.5872326094798002, -2.7957217656471864]]], [[[0.9298438893990677, -1.9340318966790069, 1.5318686532180699, 0.89288194831865353], [2.2374565629701424, -2.4613008521482649, -2.6348323151118578, -0.06036239648253891], [-0.97713802060059096, -1.8606756780550677, 0.87467429803394736, 3.1687385452664962]], [[-2.7730162700055514, -0.068138629049262781, 0.72865719806957741, 1.7207408110856586], [0.88836170713876483, -1.7228625406603195, -0.44959824565433826, -1.9054276570404289], [-3.1010131425426484, -1.079000102191515, 2.2091745880195961, 0.14134267270024439]]], [[[-1.651587072817027, -1.8294103265524828, 0.39928062482652771, 2.0574126313801062], [2.5345230327851236, -2.0283367159018209, -2.7285437759475628, 2.3494957001536192], [-2.7058464407045042, 0.75757696989308487, -1.0720695968251703, -1.7648348669669525]], [[0.18731082484405173, 2.5521104680972884, 0.47282485345699682, 0.81248791415166965], [0.60840266443720281, -2.5487000838178244, 0.42451740938229904, 0.50012933091558665], [0.57564281775315773, 2.6995966472182724, -2.7360395974888747, 0.25292655841454437]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.05492199546)+(1.-msk_arg0)*(-2.61459247952)
      arg1=numpy.array([[[[-1.0677014447925606, -2.6090665833155411, 4.1510006095322556, -4.3245479774005], [4.9409646973599735, -2.8401275182574937, -4.7754450980918861, 2.2539194195492041], [-0.27875481569789606, 4.7673018136114713, -0.077326175692339127, -1.9562295757130599]], [[-2.9219822832429165, -0.57357189300373168, 3.5851361635391985, -2.0773558302558834], [3.752284972428745, 4.3221898088648416, 4.9463252086506628, -1.3977021891345687], [-1.5026581081659085, -3.7077580879320071, -1.7919810566458496, 1.4101254710824609]]], [[[-2.9215958527569743, -3.9767322129973204, 3.3848228232593343, 2.6940003094644638], [-4.1423483699538632, 3.2898489111343885, -4.9914374409309623, 2.637855913696959], [-1.0339702599565284, 3.0246950606142065, 0.91249515307212015, -4.4154636795257822]], [[-3.5655551141956812, -2.6863050600054739, 3.5832973441725908, -2.8936546407210728], [-1.4721512342984902, -4.4197300948032137, -2.6094027361515506, 0.041902629798073576], [-2.317018094137925, 0.12360073331180743, 4.5008876284506769, -2.9362154840186596]]], [[[4.7171137980419449, 3.4560002082831858, -4.930813471865112, 0.58590150061251656], [2.158807110863707, -3.4028628917028723, 3.3887587592737312, -3.5493450924272354], [-3.7487048291749447, -1.6721075457854608, -2.0197394039645089, 1.8756159648868218]], [[-2.4262213337644423, 3.3177290379540789, -4.0381296482261408, -2.9927776056649833], [-3.3664366109197918, 4.3081414944991021, 1.691506530135066, 0.10900940028931938], [-2.6439475264583701, -1.6711178229367341, -2.1122138241494892, 2.6461161118960081]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.126341738697527, -2.7523617263627389, 4.378981846169018, -4.5620607817876495], [5.2123323380430335, -2.996112988924867, -5.037722072095181, 2.3777091716799181], [-0.29406458642048339, 5.0291315421814922, -0.081573083562756776, -2.0636696075917174]], [[-3.0824633809413085, -0.60507360590803649, 3.7820389956413947, -2.1914483577367911], [3.9583679506541416, 4.5595730979303788, 5.2179872593105001, -1.4744667824225266], [-1.5851870899625466, -3.9113955608091664, -1.8904002321057654, 1.4875723758051758]]], [[[-3.0820557269219711, -4.1951422815505355, 3.5707240469958332, 2.8419601822337275], [-4.3698544083277708, 3.4705339781002151, -5.2655771454073497, 2.7827322242167001], [-1.0907579698810244, 3.1908173490052061, 0.96261120772764452, -4.657969755692422]], [[-3.7613825159947045, -2.833842294318877, 3.7800991846458798, -3.0525799277654495], [-1.5530047177070423, -4.6624704910103567, -2.752716341383282, 0.044204005841661703], [-2.4442733513880177, 0.13038913222577717, 4.7480853583524594, -3.0974782975054573]]], [[[4.9761871006486427, 3.6458106360369151, -5.2016235869876164, 0.61808038016995115], [2.2773731052084782, -3.5897549119965495, 3.5748761524701491, -3.7442822074842637], [-3.9545911787888053, -1.7639430288259672, -2.1306675223421432, 1.9786285363975582]], [[-2.5594742508456663, 3.4999453371185583, -4.2599117864383311, -3.1571469237401244], [-3.5513280271856273, 4.544753222106805, 1.7844074441059761, 0.1149964140772531], [-2.7891584005065453, -1.7628989484234345, -2.2282208222128133, 2.7914460889837471]]]])+(1.-msk_ref)*numpy.array([[[[2.7916041679296399, 6.8216458673095524, -10.853174976175195, 11.306930619044383], [-12.918609139302179, 7.4257760501201551, 12.485842839842306, -5.8930807638024403], [0.72883024475432212, -12.464551469481203, 0.20217643743540392, 5.1147431368783138]], [[7.6397929030641007, 1.4996567579128814, -9.3736700512527396, 5.4314389310786746], [-9.8106960699364443, -11.300764969325606, -12.932624691809208, 3.6544216323229897], [3.9288385889036741, 9.6942764125947196, 4.6853001941525232, -3.6869034518749326]]], [[[7.6387825448216979, 10.397534137176557, -8.8499322982090298, -7.0437129489563253], [10.830552895642505, -8.6016142218163427, 13.050574795063739, -6.8969182340152884], [2.7034108657319758, -7.9083449583299155, -2.3858029648228443, 11.544638130091629]], [[9.3224735868980257, 7.0235930075928028, -9.3688622879656069, 7.5657276619638942], [3.8490755459161887, 11.555793067390519, 6.8225247699865532, -0.10955830074224693], [6.0580580838499305, -0.32316554778048351, -11.767986944521748, 7.6770069227718887]]], [[[-12.333330261410975, -9.0360321538044506, 12.892067821465378, -1.5318936572422703], [-5.644400836803344, 8.8970997254915716, -8.8602231669121529, 9.2800909858893554], [9.8013354543094469, 4.3718798141630257, 5.2807954562003001, -4.9039713962649003]], [[6.3435800529168853, -8.6745093917272307, 10.558063409587783, 7.8248938206541876], [8.8018598456991644, -11.264034352234978, -4.4226002527538721, -0.28501515819368201], [6.9128453189294357, 4.3692920922459333, 5.5225783797641279, -6.918515286105885]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.9635679456103663, -1.6865827247985754])+(1.-msk_arg0)*numpy.array([-0.11826509975540578, 1.2300763493903544])
      arg1=numpy.array([-4.7330365531023286, -4.4872666566538735])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-14.026675414176237, 7.5681464246770833])+(1.-msk_ref)*numpy.array([0.55975304009862881, -5.5196805877578576])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.777773327340527, 0.43937330337571101], [-1.6988320586682137, 1.2255794020942208]])+(1.-msk_arg0)*numpy.array([[-0.42888501429911496, -4.3494007663285323], [-3.5755303674891281, -0.32198052442583247]])
      arg1=numpy.array([3.544834286406557, -4.6808370643947335])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.846746130622325, 1.5575055503379303], [7.9519560663961828, -5.7367374906813646]])+(1.-msk_ref)*numpy.array([[-1.5203263036134691, -15.417904961804336], [16.736475069012034, 1.5071383727456904]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.9003420523696479, -4.8988095293305349])+(1.-msk_arg0)*numpy.array([3.5173526687562884, -2.3914190180737771])
      arg1=numpy.array([[-3.321118694258943, 1.0105655286814681], [1.0521207743124297, 2.9325338980061471]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.3112615156112453, -1.9204201708285593], [-5.1541392752083519, -14.365925004637333]])+(1.-msk_ref)*numpy.array([[-11.681545702508092, 3.5545153592608711], [-2.5160616290012525, -7.0129173348379261]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.9867688287218197, -4.983696303589662, -0.29091984293407869, -3.9171672015108685, 0.98099314846396801], [4.0313440083070553, -3.1885111928815526, 0.69813889717880695, -1.3333470597852282, -1.9686420913002713], [2.4889813437800035, 2.3430389506658367, -4.0500388204939943, 3.8234705953649986, -2.8569527304792341], [-3.1366845885466796, -1.3160340135133843, -3.8477177290315803, 4.041672631054924, 4.34792032170777]], [[4.7769797435400516, 0.90158594629532196, -1.4716497253408978, -4.2777591943802262, 4.8312160675147968], [-0.0040555522759300189, 2.7598691034949727, -1.3867024906371563, -3.4483272088130867, 4.555921947867855], [-4.0177054372239009, 2.8708739330608699, 1.8405171061780319, -3.7112024088478011, 1.836809466220739], [0.41885847655817177, 3.2818894672296643, -1.4798238847774972, 2.9363212928960341, -4.9602892438662334]]])+(1.-msk_arg0)*numpy.array([[[-2.9887820982308009, 1.0615538179885418, 1.6463510191519148, -0.51934694791157376, 2.2031920835409231], [-1.1550674169150357, 1.7004495123478547, 4.3770440473683827, -3.3187197934303359, 4.2856180934008563], [3.1245112591457183, 4.9084784715186895, -1.0212207124789261, -4.8157671070725359, -2.1994886584794369], [1.7273322893118994, -3.0314304227915523, -2.1737929906438467, 3.0956407332962037, -2.3387938449924786]], [[4.6354251559441586, 1.7848457732272838, 3.6313590365038415, 4.5713609750703501, 1.7429400515226066], [-0.49874309107482873, 4.3527158744914374, 1.4197716902266677, -0.051627724049929213, -3.3133403757699167], [1.9466074984755366, -3.8008931500619103, -1.2579657193694791, 3.4742979760729629, 3.138265208620826], [1.8188730259342307, 1.1611996771681916, -3.0139037408032729, -0.70772947191129987, -3.3278027394175504]]])
      arg1=numpy.array([-1.1507587515797999, 2.3700778482207507])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.2862916170175827, 5.7350321365717027, 0.33477855526461187, 4.5077144385399857, -1.1288864508347332], [-4.6391043981881337, 3.669207159718594, -0.80338944574678217, 1.5343607979410461, 2.2654321152921466], [-2.8642170638736895, -2.6962725777710626, 4.6606176169213942, -4.3998922490243002, 3.2876633574487841], [3.6095672412155753, 1.5144376584272157, 4.4277948502918445, -4.6509901512070098, -5.0034073613768753]], [[11.321813871563519, 2.1368288795816861, -3.4879244143706138, -10.138622306623219, 11.450358181584987], [-0.0096119746114829881, 6.5411046261822969, -3.2865928551316674, -8.172803931024788, 10.797889686864337], [-9.522274657440434, 6.8041947137819498, 4.3621688226239126, -8.7958386194736633, 4.3533814272919553], [0.99272719683001354, 7.7783335265900293, -3.5072978085791227, 6.9593100515518049, -11.756271657655017]]])+(1.-msk_ref)*numpy.array([[[3.4393671561041312, -1.2215923463232645, -1.8945528434613887, 0.59764304541550195, -2.5353425715460509], [1.3292039386796506, -1.9568071579538968, -5.0369217435594349, 3.8190458463310648, -4.9317125269097719], [-3.5955586758715552, -5.6484745580411717, 1.1751786721796829, 5.5417861440338561, 2.5310808227457255], [-1.9877427488120392, 3.4884450888326315, 2.5015113081062328, -3.5623356655875158, 2.6913874852660649]], [[10.986318479188469, 4.2302234296164229, 8.6066036113540036, 10.834481383235048, 4.1309036068904641], [-1.182059952109596, 10.31627547373097, 3.3649694325371589, -0.12236172512479093, -7.8528746282276982], [4.6136113113172783, -9.0084126584157236, -2.9814766852986838, 8.2343566712087171, 7.4379328527940931], [4.3108706674929671, 2.7521336322174181, -7.1431864927474926, -1.6773739439099418, -7.8871515559418679]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.7903426307423693, 0.94013921204392137])+(1.-msk_arg0)*numpy.array([0.63149262219082658, -1.3453281383747075])
      arg1=numpy.array([[[2.176629634610995, 0.31352788667419418, 0.39525494060090871, 2.0980923327727652, 1.8807588337247836], [-0.056894420589164163, 1.8541581806802379, -1.9239937467761714, -3.7774289123724811, -4.364685392051503], [4.9021175378802635, 1.2046347929052823, -3.6747140588942195, -0.25562227064105514, -2.0242788810971648], [-4.4439299500655158, 0.87821455100227652, -3.4864663916986882, 3.7716459666279363, -1.0002141117592678]], [[3.3683517426132585, -4.3615723358723635, 2.2741392463203933, -1.2842720768923965, -1.2135708446996185], [1.4123328847025629, -1.5273204814879837, 2.9936062220607065, 2.1386487778611798, 3.0211928535673778], [-3.1882239871045059, -0.89852026296672882, -2.5478818313162477, 4.4769650639454923, -2.5600571359651259], [-1.0698265630954418, -4.2267575500087871, -3.5980095170527351, -0.30596596882262084, 0.064937336824656633]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[10.426801730014237, 1.5019060014619547, 1.8934065919720759, 10.050581144915082, 9.0094792193371305], [-0.27254376839965949, 8.8820529770522558, -9.2165892664636324, -18.095178753536679, -20.908338503322785], [23.482822622617647, 5.7706134029296789, -17.603139412109307, -1.2245182604190099, -9.6969894206312119], [-21.28794708783165, 4.2069486026044736, -16.701368586804751, 18.067476462005313, -4.7913682994305331]], [[3.1667195531871983, -4.1004851791196097, 2.1380074791138117, -1.2073945384196281, -1.1409255376953755], [1.3277895253679859, -1.4358938740046556, 2.8144065947779331, 2.0106275768571051, 2.8403418687855608], [-2.9973743870559595, -0.84473413203103742, -2.3953636172746804, 4.2089704075658796, -2.4068100985936716], [-1.0057859020522051, -3.9737405125659566, -3.3826298322884885, -0.28765060484115379, 0.061050136674563403]]])+(1.-msk_ref)*numpy.array([[[1.3745255554987579, 0.19799054728583521, 0.24960057887394724, 1.3249298288211417, 1.1876853276174244], [-0.03592840684587903, 1.1708872114743358, -1.2149878562304377, -2.3854184890135404, -2.7562666232645996], [3.0956510582836461, 0.76071798415406011, -2.3205548168526065, -0.16142357797749304, -1.2783171786695611], [-2.8063089769992215, 0.55458600965856697, -2.2016778038739941, 2.3817666014413303, -0.63162783218712848]], [[-4.5315383792810975, 5.8677459910057914, -3.0594635186570751, 1.7277673623722671, 1.6326510052855592], [-1.9000511705422793, 2.0547472200617913, -4.0273826857518715, -2.8771843789573244, -4.0644957573607705], [4.2892074412928922, 1.2088045926689819, 3.4277371209234282, -6.0229870750463927, 3.4441169008608483], [1.4392677785130024, 5.6863758661145614, 4.8405034454310369, 0.41162462724215032, -0.087362026461326653]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.3389532903086554, 1.2699398217476556], [1.2886326179432936, 0.26915799520196604]], [[0.90218471194921257, 2.7524257887557724], [2.6165968175771335, -3.5342811949989783]], [[2.1386063249834786, -3.9259407014082157], [-2.3645508721212529, -2.9105179577890814]], [[-2.3207880688823836, 4.8952825304793333], [-2.8339961218853018, -1.3245142526844345]], [[0.83193128845555275, 1.0835412817490573], [-3.0036570713712898, -3.9967340387105788]], [[0.27025295124622684, -0.77897506771948066], [4.5095646354436774, -0.036248736710230212]]], [[[-4.2594401310429708, 0.90171827150309447], [4.554476953964274, 1.673421045371799]], [[3.361291806606344, 3.4748763917621286], [-2.0901648009819995, -1.6661448921829605]], [[4.0683397502574188, -0.50568735350752192], [-1.1777388745966286, 0.1991636784375892]], [[-2.5036588621816858, -2.2835026794329085], [-0.55503522804732608, 2.727118124171767]], [[-1.8134421652192301, 0.83166330247178433], [-1.6545404760119542, -1.351121200582206]], [[-0.37820462524928011, 4.6876723664211024], [4.707389156497845, -0.47192349471738737]]]])+(1.-msk_arg0)*numpy.array([[[[-3.582155919079236, -4.3150958945556441], [2.0030240710922094, 2.847189789962032]], [[4.230451886889707, 2.8414208829348215], [-0.41820185173341784, -4.746588368886794]], [[-2.502508020638754, 3.8198545659241407], [-4.1362361009671051, 4.7073960865911104]], [[-0.8731731301135266, 1.4399226361331676], [0.92347374607182076, 4.6900820698819601]], [[-2.2970273812939048, -4.9448932611651149], [4.0865736307115839, 1.1244472957744822]], [[1.9892696979874369, 2.2221484919821242], [-0.81132169363838713, 1.7281120091497781]]], [[[-2.8514644817221058, 4.8648141828152127], [0.80152073446479477, 3.726121556139109]], [[-0.21481485803375389, 2.5590918990736462], [-3.0164967490823278, -4.704038973822291]], [[0.73813650934372355, -0.5304755260133831], [1.3242114042694659, -3.1497690804367631]], [[2.7090158791766648, 1.8346450990547023], [1.7940652407498945, 1.4323213879004584]], [[2.3477783829912271, 2.330658179071472], [2.4840434410169294, -0.71606173532182815]], [[-2.0385781455269236, 4.647098061318184], [1.9820704737663295, -2.9647623339766449]]]])
      arg1=numpy.array([2.329330722895576, 3.7811299297819669])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.106857204324799, 2.9581098430253454], [3.0016515475006709, 0.62695798753691956]], [[2.1014865672699963, 6.4113099522389092], [6.0949193566132074, -8.23250977086321]], [[4.981521416962817, -9.1448142920563633], [-5.5078209922815624, -6.7795588986173962]], [[-5.4058829501772303, 11.40273199549951], [-6.6013142352743488, -3.0852317416909272]], [[1.9378431095376207, 2.5239259971037304], [-6.9965106973876949, -9.3097153876110674]], [[0.62950850229103639, -1.8144905576086481], [10.504267452222345, -0.084435296085291947]]], [[[-16.105496563601001, 3.4095139446116121], [17.221069125136523, 6.3274223997823364]], [[12.709481052690146, 13.138959127184553], [-7.9031846871698068, -6.2999103191863401]], [[15.382921194220019, -1.912069587459525], [-4.4531837082050432, 0.75306374546583987]], [[-9.4666594577590359, -8.6342203259410866], [-2.0986603128531041, 10.311587961356723]], [[-6.8568604468390451, 3.1446270044773765], [-6.2560325138845023, -5.1087648102843231]], [[-1.4300408281120256, 17.724698285686689], [17.799250030765091, -1.7844040504432153]]]])+(1.-msk_ref)*numpy.array([[[[-8.344025836513504, -10.05128543942903], [4.6657055074944553, 6.6320466516731633]], [[9.8541215518637539, 6.6186089592971538], [-0.97413042161447061, -11.056374116586809]], [[-5.8291688167664457, 8.897704597400045], [-9.6346618271324864, 10.965082329335077]], [[-2.0339089983803338, 3.3540560349377748], [2.1510757685125599, 10.924752258277726]], [[-5.3505364505802628, -11.5182917946712], [9.5189815093914127, 2.6192096323243503]], [[4.6336670236473401, 5.1761187532100355], [-1.8898365471435674, 4.0253443955173793]]], [[[-10.781757695549679, 18.394494509470402], [3.0306540384256602, 14.088949737923343]], [[-0.81224288907329101, 9.6762589726499364], [-11.405766141045193, -17.786582554780313]], [[2.7909900477443395, -2.0057968884260351], [5.0070153740418855, -11.909686141941268]], [[10.243141021009496, 6.9370314945635361], [6.7835937777809159, 5.4157932688572696]], [[8.8772551124232386, 8.8125213969782816], [9.3924910017076986, -2.7075224589969773]], [[-7.7081288402512689, 17.571281566281939], [7.4944659912949918, -11.210151595689332]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.082466209713593308, 4.4079586773370227])+(1.-msk_arg0)*numpy.array([-2.6503908424265399, -2.1460430124759533])
      arg1=numpy.array([[[[1.3492347729117302, -4.7443687332892512], [-0.36137230429515732, -3.2633064259882794]], [[1.4672729016557708, 1.090581335349845], [0.38999041903401555, 4.737224077570394]], [[-2.728888573239352, 3.8696717707976429], [3.9036965642335968, -1.6881039986135571]], [[-3.7061914947920105, 3.2940203561712487], [-1.4043365849738931, -4.1424157263266022]], [[-0.93373675029992498, -0.56240571936262818], [-2.2747679456798875, -4.584228153763199]], [[4.2531226028834581, 2.7188596696966201], [0.7007315842202928, 4.0020983487603417]]], [[[-1.609713790860968, -1.4516926543704258], [0.54964590159368498, 4.2284274670348054]], [[0.28805546470642795, -2.5735281537515728], [3.5764180299049269, 1.0535094810928882]], [[-0.14231332066007152, 3.1962105760948951], [-1.3175757530265839, -0.84714509394642867]], [[-3.8156412295520701, 1.5997513196796609], [0.77300722798555199, -3.7099820098248393]], [[3.7522694228140097, 1.6386671780291078], [-2.1885282392524963, -2.161254359361152]], [[4.4027478320403155, -0.39079892280607709], [3.9782667696527927, 3.2581481332206437]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.11126627773581119, -0.3912501069180464], [-0.029801004230688901, -0.2691125120852661]], [[0.12100043481501736, 0.089936109110690957], [0.032161031682351258, 0.39066091424120375]], [[-0.22504109736578484, 0.3191171637733704], [0.32192305952432149, -0.13921153836802103]], [[-0.30563556504825379, 0.2716453734928635], [-0.11581031532492853, -0.34160932400813654]], [[-0.077001730667522722, -0.046379467997082799], [-0.18759149045819742, -0.37804392030319461]], [[0.35073890050701106, 0.22421405170303249], [0.057786677777249135, 0.33003788172329585]]], [[[-7.0955518724546769, -6.3990012326585335], [2.4228164213926151, 18.638733544806279]], [[1.2697365852070477, -11.344005756700373], [15.764702888704003, 4.643826258840221]], [[-0.62731123670420841, 14.088764143493858], [-5.8078194736023923, -3.7341805678246476]], [[-16.819188867408954, 7.0516377111633144], [3.407383918243152, -16.353447392971649]], [[16.539848561999396, 7.2231772066607771], [-9.6469420428101564, -9.5267199072784585]], [[19.407130510368873, -1.7226255028770088], [17.536035528052555, 14.361782335879358]]]])+(1.-msk_ref)*numpy.array([[[[-3.5759994864087017, 12.574431443804635], [0.9577778460104619, 8.6490374674710164]], [[-3.888846661889072, -2.8904667841325367], [-1.0336270352418437, -12.555495313715085]], [[7.2326212845160045, -10.256142624518565], [-10.346321625456673, 4.4741353789889962]], [[9.8228559980758732, -8.7304413867628874], [3.7220408244993668, 10.979020706579711]], [[2.4747673322320378, 1.4905949683270203], [6.0290241318754063, 12.149996318327906]], [[-11.272437198399647, -7.2060407704067693], [-1.8572125738165057, -10.607124814044786]]], [[[3.4545150329633585, 3.1153948771743214], [-1.179563746451173, -9.0743872193914381]], [[-0.61817941723874326, 5.5229021117687038], [-7.6751469227704829, -2.2608766604765602]], [[0.30541050738479619, -6.859205373230191], [2.8275742381904427, 1.8180098094170183]], [[8.1885301987953749, -3.4331351412977211], [-1.6589067602117999, 7.9617809685960896]], [[-8.0525315757571843, -3.5166502471830556], [4.6966757354541206, 4.6381448160901932]], [[-9.4484862206437707, 0.83867129757111125], [-8.5375316027786585, -6.9921260349097336]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.40660382148361052, -3.8774876194653416, 2.3713772849394585, -1.1530131266969446, -4.6314694255191027], [-3.4626797211285929, -1.3197954661473119, -1.0598746938675019, -3.9240361738766394, -2.0354352696514466], [-2.0190131665070208, -3.2604844108796338, -2.5481423680366144, 1.8296240548897398, -0.11231102295359996], [-3.6628356012953867, 3.6858602187174139, 1.5499809535482392, 1.2572788050643968, -4.707735379131142]])+(1.-msk_arg0)*numpy.array([[-0.032248166122366584, -2.6645753813179649, 4.3001272847796965, -2.5874840666017027, -1.3191025380546408], [-0.6371021358481439, -0.19625264203335924, 2.8052565487624159, -1.7810254567780559, 4.8532603955395164], [-0.49595053613062579, -2.1380610773257658, 2.3299979054048823, -0.93676256568311089, -0.68543603565263655], [1.8764271118258655, 3.368418822878871, -1.0732482094783489, -0.2215380712321231, -3.3492396508998201]])
      arg1=numpy.array([[-3.316295999187806, 0.24326247464931949, -0.063654806472408509, 4.4262361549940756, -1.107043345613099], [-0.29279707959690526, 1.5376686487669744, -1.4297243683914385, 2.8735130494915184, -3.142295420190353], [-2.7251660428939575, 0.43908194939854361, 1.4517323738466201, -0.22368930298356826, -0.34205025014284107], [4.8474866226900666, -2.9884341526608473, 3.779919282736298, 4.9343793618534235, 2.1594450104122229]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.3484186264405704, -0.94324723373323782, -0.15094956214588676, -5.1035083885687804, 5.1272374079314451], [1.0138625099258782, -2.0294081110795164, 1.5153286772637833, -11.275769152311293, 6.3959389259196566], [5.5021461215207363, -1.4316198511125917, -3.6992207688489422, -0.40926732956025574, 0.038416013495077228], [-17.755546578392313, -11.014950559549099, 5.8588028941909833, 6.2038905878054935, -10.16609567480584]])+(1.-msk_ref)*numpy.array([[0.10694446429274805, -0.64819120114906226, -0.27372377011937504, -11.452815526063555, 1.4603036869347399], [0.18654164478128735, -0.30177153489238423, -4.0107436473552918, -5.1177998915283363, -15.250377913895043], [1.3515475600182341, -0.93878402576534759, 3.3825333902710826, 0.20954376537875416, 0.23445356745190166], [9.095955323028841, -10.066297850756868, -4.0567916021694161, -1.0931528865526019, -7.2324988528103917]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.5007975002196172, -2.1489741550914898], [-2.6950752417097998, -1.9509622864310847], [4.323463699705524, -4.6389801599953531], [0.99043531913079264, -4.6553490891362959], [1.2855474604864394, 2.7497309937390693]], [[-4.0268296752084431, -3.6943801408533061], [1.3046305168090297, -2.8946342977873996], [0.85814670955065875, 1.6863457451081203], [-4.3356241352046778, -3.3892562064330543], [1.1352524857484072, 1.45145239880652]], [[-1.5410782863424921, 3.6464415090490423], [-0.94379639606473109, -1.1734047226106172], [3.5976292059008905, -3.2514426871678124], [-4.2007499859561728, 4.4850251592822445], [-3.6595785487274233, 0.64926915829651222]], [[3.7697280943820672, -3.8982291576668606], [-2.1634613084977006, -1.9970475493574655], [-1.0475473854467729, 3.545291278991618], [-2.8735777987770597, -0.48173724544028484], [3.9643909581091439, -2.0110125021371426]]])+(1.-msk_arg0)*numpy.array([[[-4.7232364429895615, -2.5937866050104077], [1.1431898214624905, -3.0973744182713316], [-1.6123765939753798, -3.4631506035124513], [-4.3806703671396274, -3.1336869595173811], [3.4380284818957492, -2.0890127262292655]], [[3.3066416175900688, -4.9522356271402526], [-1.2743401256662477, -3.3074054519899656], [2.9843522969524461, 1.0536281283903275], [4.1609406125345156, -0.0048725392420543301], [0.12213337721717199, 1.7730350217585942]], [[1.9571788281980034, -0.88090730489568436], [-2.9212806117016346, 3.3814821563308488], [0.74789360542385364, 0.68449841175157111], [1.5922097084407945, -3.4748735685520469], [3.3997438729806042, -1.2569237466237504]], [[3.8890321644195396, -1.4324725315981279], [-3.5974963880651734, -1.2050057170063422], [1.3578690904693813, -4.1912367073161541], [3.3062375311487813, -3.0373142536697548], [3.7019133984066901, -4.9229264573242055]]])
      arg1=numpy.array([[0.83678779539102432, 0.33627451279117171, -1.9005179428449717, 4.9612195458456778, -1.9292130990380887], [3.3011115101578188, 3.3624999852019197, 4.1492728906872838, 4.0624347794878872, -4.2220244761854984], [-2.5149138339545618, 4.7709200548024846, -2.4970942327715164, 1.5774278576926335, -0.85454837950259854], [-4.2945555805916484, 0.098615332348193085, 0.14574774190495177, -0.47464160802562283, 1.2741201762834162]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.9294246223191824, -1.7982353455912969], [-0.90628511384151222, -0.6560588923435634], [-8.2168203365292527, 8.8164650305730063], [4.9137670641675903, -23.096208893757865], [-2.4800950002055888, -5.3048170519524334]], [[-13.293013790275662, -12.195560805869313], [4.3868200934643351, -9.7332077834751001], [3.5606848781710427, 6.9971086845029715], [-17.613190277642577, -13.768632289608817], [-4.7930637814802042, -6.1280675537792826]], [[3.8756791015297227, -9.1704861958135844], [-4.5027771536355345, -5.5982201235029398], [-8.9836191417054838, 8.1191587823138658], [-6.6263800510492059, 7.0748036287041529], [3.1272869184774907, -0.55483190708330066]], [[-16.189306825041626, 16.741161783483296], [-0.21335045595995744, -0.196939507795031], [-0.15267766596730328, 0.51671819830834664], [1.3639195871982732, 0.22865254082161093], [5.051110506402404, -2.5622716037311299]]])+(1.-msk_ref)*numpy.array([[[-3.9523466102397786, -2.1704489749214289], [0.38442560024012556, -1.0415680734360309], [3.064350647473471, 6.5817798607498057], [-21.733467449360081, -15.546908994119343], [-6.6326895821393137, 4.0301507154987677]], [[10.915592703793445, -16.347882029766311], [-4.2849686536949703, -11.121150783373007], [12.38289208200511, 4.3717906299955667], [16.903549859743851, -0.01979437288134106], [-0.51565010797009647, -7.4857972589988719]], [[-4.9221361105581369, 2.2154059675137856], [-13.937196256072998, 16.132781034595595], [-1.867560808830601, -1.709257036326111], [2.511595949383175, -5.4813623689938114], [-2.9052456173794634, 1.0741021508356607]], [[-16.701664784808351, 6.1518329046189866], [-0.35476830193047126, -0.11883203926405314], [0.19790635373844298, -0.61086328588047467], [-1.5692778982991229, 1.441635721440957], [4.7166825517638724, -6.2723999256362104]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.71568360748626514, -0.39141171149356957, -2.4917946873955077, -4.3840710734399222, 1.7675770263336581], [-2.007756215076645, 4.5094449738182174, -2.3422141145222417, -2.7426199772981299, -3.7916951205359153], [-2.8095117235406519, 2.7373248059364661, 3.843084012874975, 2.6722405131124827, -3.4235160932640429], [4.4942289290474502, -1.1036071092574384, 0.36183126325311488, -2.0755404604318786, -0.27165821816881319]])+(1.-msk_arg0)*numpy.array([[-3.4908364019258933, -0.18813474878190739, -2.7942085190538481, -3.163789419079853, -2.4908576582278599], [-3.9056411264050261, 2.2317120400531918, 4.5045051348931651, -4.5810301208495021, -4.7641818959646312], [-1.7102051057212906, -4.9474653270741156, 4.0629119094271715, 4.4946911659950892, 3.1310068844366157], [4.152177160024829, 4.0204433042180305, 1.1818296094298821, 0.34463623871548066, 0.52681828745779313]])
      arg1=numpy.array([[[1.0522298812661992, 0.55852327465448681], [3.2403375481944661, 3.0935509487243174], [-0.90783646433292908, 4.140753202344813], [-3.0818437460433934, -0.16358281013233977], [-0.51329175724464093, -1.5640499794964056]], [[-0.40840883003207473, 4.3286560971553083], [3.7201109819211169, 1.7108941125363595], [-2.3692518877328994, -1.6211332958987192], [-1.299749149169489, 1.672649561021105], [1.9776154326816364, -4.880502314325021]], [[3.9248844993575265, -0.79334007939500673], [-0.18229359656381661, 1.538870843779133], [4.8450494427800344, 0.66313539982872349], [4.1379447842613999, 2.041511853422354], [-3.1094850281848387, 0.71483249194286191]], [[0.96130531804679453, 2.0378650762493571], [3.4748128333149992, -1.991514195893509], [-3.3698388765139953, 2.5623245665257777], [-0.079494971555112315, -3.3811024189870951], [-4.2518314418852778, -3.9634038841111661]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.75306367732943791, 0.39972595206976519], [-1.2683060655556728, -1.2108520714327409], [2.2621420788487141, -10.317906831418741], [13.511022019890571, 0.71715866601320577], [-0.90728271791206028, -2.7645788117954755]], [[0.81998536678907918, -8.6908861819929832], [16.775635769470135, 7.715182856512266], [5.5492952123064629, 3.7970412871759418], [3.5647179819884878, -4.58744210107543], [-7.498514786395484, 18.505376810990423]], [[-11.027009014487954, 2.2288982538149429], [-0.49899678383750973, 4.2123893338090008], [18.619932055136758, 2.5484850534532217], [11.057583693525805, 5.4554106827145663], [10.645372035754392, -2.4472405401544273]], [[4.3203261700130637, 9.1586321791753491], [-3.8348281461854157, 2.1978492247751875], [-1.2193130576485163, 0.92712913477051218], [0.16499502986351691, 7.0176148714718138], [1.1550449534566904, 1.0766912370409927]]])+(1.-msk_ref)*numpy.array([[[-3.6731623727182088, -1.9497133784867362], [-0.60962009059814759, -0.58200443058228069], [2.5366843825467953, -11.57012787329138], [9.7503046349895062, 0.51754156384004513], [1.2785367044380493, 3.8958258692797489]], [[1.5950983229602311, -16.906177275113642], [8.3022164686874582, 3.8182229902035139], [-10.672307294148171, -7.3024032557220613], [5.9541900018939415, -7.6624580206633794], [-9.4217196415621132, 23.251600769120749]], [[-6.7123575101675934, 1.3567742543546746], [0.90189124834711976, -7.6135101424425482], [19.685009082834483, 2.6942607135268699], [18.59878386719517, 9.1759652928517159], [-9.7358190302993144, 2.2381454534920824]], [[3.9915099854043041, 8.4615768248148377], [13.970287989112173, -8.006769914135214], [-3.9825753632721677, 3.028231041689752], [-0.027396847993548034, -1.1652504203915257], [-2.2399425587732011, -2.0879936467310101]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.3430441157209954, 0.63909230898927483, 3.9559623950261145, -3.9638310321742543, -0.7907702312816669], [2.7267420036490506, 3.81363347709895, 1.0252962821271696, 3.750248445420155, -3.7185238741077886], [-4.7308807257719696, 3.0031163414932251, 3.9187695150261721, -2.3214050965662167, -2.2956445278398285], [-3.3337284110667023, 0.26145795785346415, 1.7555825074981248, 1.450559149642805, -4.4341813041284723]], [[-3.5191531097452309, 3.0378083891662087, 0.58382158045019583, 3.5007874424380567, -4.2941495103757958], [-2.8704016481936412, -4.5594312415238978, -0.22282759305601374, 0.55627634267337811, 3.7818440922534986], [4.7497678652854045, 0.23003520898815388, -2.6381241842060255, -4.2643469422685287, 2.6287150059121034], [4.6061712938518031, -0.25745682167369299, 0.48096052212279972, -0.088555931715674774, 2.9923106929968881]], [[-0.80996170822834479, -3.0051298299457474, -4.0957639395633212, 4.0093971658871936, -0.040614446290565098], [-2.6616577223535942, 1.6043651890022792, -2.8420111099224963, -0.7012952057540458, -3.6489857336196607], [3.9766693716373016, -3.9140859548811315, -1.5804593729431593, 1.9665091472177219, -0.74255947762976415], [2.3609370073669931, 4.6632745059183893, -1.6935602451713461, 3.7725739868628505, 2.4282123674536713]], [[4.0715540674568942, 1.6470962383502492, -3.4683641653723862, -4.1015732039513431, -3.9487307424989426], [2.6894158556314816, -0.49045225876660226, 2.7595261540430531, 4.9682933484556422, 4.4466955947871085], [3.9849510345049168, 0.016636734119534147, 4.6762336175417261, -1.4874581023208555, -4.4145527772998703], [-3.3065062791262854, -2.081434743858269, 1.0407804598738748, -0.26950672074466375, 0.46898486000825734]], [[4.6766709448721837, 3.5329645740570079, -2.4238596670234469, -2.9576142863104149, -1.2613957610958657], [3.7448047079269369, -2.6948726112988854, -3.2859863597330974, 1.9682551352243625, -0.37266764469841718], [-1.5367740612613634, 2.6892355204014082, -1.9674127342687986, -3.6679392090854304, -4.1752276760483191], [-4.954252692948474, 2.2245270485650064, -0.35471264206592679, -1.4634367358991263, -1.3115534042856423]]], [[[3.552926893378622, -1.2270509028646561, 2.599584105650651, 0.36084879145455595, -3.0588277776261252], [1.7868765769542936, 0.66302467198165438, 3.6848178659694817, -4.9937313638647849, -3.0952342908560593], [-2.1607065327562616, 2.4106504105795494, 1.6797036492798636, 1.4902991050026007, 0.60316779325177272], [2.017842456799734, -2.2645855635756948, -0.42171780269183934, 0.7369340192803806, -4.9881326063336653]], [[-0.60599234860006312, 2.093507608340901, -4.0490583426106728, -2.2601366102015352, 4.4912700333390454], [4.5413801018768005, -3.8596120165921444, 0.42143939981488376, -1.1174654006650564, -2.0822109410257159], [1.8425040250784512, -4.7817582774682199, -4.6713545484255317, 1.9262926935789491, -3.3705473761914995], [-2.110269658644147, -2.9991937922577327, 1.069146729105686, 0.98817016001019198, -3.5933522915957705]], [[1.5045723242463858, 1.4234444980772123, 2.1739682544704468, -2.9616244236149267, 3.0234304161436896], [4.3312601146101795, -2.3690918130920302, 4.5908172285896907, -2.126891488556264, -1.2656158751239985], [2.8891949776983852, 3.2312946403670519, -0.61712178967238351, -4.1294136425141197, -1.7346834739501391], [-3.6940561943847792, -4.0452709720442837, 3.4698203210085712, -2.2634695279935113, -4.0742947733165202]], [[-3.7427160650113223, -2.8509676494734979, 0.98864597167014256, -4.2625861623087049, 2.876689880629427], [-0.99521387465273481, 2.8674723249856973, -2.0537099919974802, 4.5224637616473622, -1.9446363301497538], [-4.7551006257228465, 2.7344397292692868, 4.4875736047700769, 3.0409667178263895, 1.5812466694320566], [-0.13474722060277333, 1.0516270333790683, 1.5796018175440807, 1.1348982198136852, -4.1129872277738997]], [[-0.86694951508261564, 0.39934421697883948, 2.4169091454857643, -0.15658039758192999, 4.6249320309330866], [2.5922212342280515, -3.2747592808078672, -3.1308665069341055, -4.6956581536410615, 0.65927075088013787], [1.7323166728262498, 4.453039493755556, 2.7083643225126064, 3.3549650991152848, 2.6977875900843786], [1.3854196547149638, 4.323752718127766, -4.4083370126270713, 3.378263302586614, 2.1128210300147643]]], [[[-3.4394051372823951, -2.8867575852813445, 2.5562151627705259, -3.2025103801319976, 0.98493416432924175], [1.6414295692104481, 4.3026666954142616, -0.54720323636228763, -1.2446051245970589, -4.4109599970257776], [3.5275141102445673, -3.6424589825259157, 3.9021854003667986, -1.5971406529627705, 2.1439207818199097], [0.34799732803232875, 0.86916400284698092, -4.360575088147943, -4.6229451960871701, -0.33968004101750804]], [[2.8625789778521469, 0.64910129612564127, 3.1068458459591923, -0.58809200056160904, -4.7092778998631433], [0.007630347244317548, 1.6743559687209295, 2.5607064075612289, -0.3364475562541962, 1.4031875578633848], [-2.6759386663524909, -3.8250907154181246, -3.0157476979798337, 0.88366622289353991, 2.7427815992967588], [0.075983705971016668, 4.164582092069665, 1.2001713311395505, -4.5488604101751751, 0.55402037736502585]], [[0.08517935521271891, 4.5232795310866756, -3.5856291358731274, 3.6679501687247154, 3.8128368386195906], [-4.1450343168454262, 0.69178397674661518, -3.3346560634146005, 0.73507322421468491, 2.5882982078409835], [0.31526297241914492, 1.7860635796258242, 4.6259236380775732, -2.7036643188929812, -4.1159642025319148], [-1.3550534270080794, -2.8652520806239101, -2.3022634377186026, 4.5623341813911438, -4.8499128897128667]], [[0.32638013573557956, 0.10428243188823316, -2.3466815592796317, 4.5147573837408803, 2.4354028268243937], [4.0886061842338286, -4.849099798765252, 3.8936385523207875, -4.3617993713600081, 1.7194635021802851], [3.8493866929640763, 2.428819628074816, -2.8403405454136865, -0.99927418418626157, -1.254644347484116], [-2.8566699454797719, 2.6852350272114167, -3.4212916190094775, 0.63081422691708333, 2.7767920790742648]], [[-2.8092461575648775, -1.6517038244842741, 3.6369120324533029, -3.2305779529198553, 2.7697284958915329], [-3.5685030489552814, -2.3408052507120125, -2.9056225121603108, -1.7775866061185042, 2.7456952746096599], [4.1786838506187287, -3.9575195420019438, 2.9684903790341419, -2.5303072523578996, -0.076286257898583898], [4.5998647203400935, 0.036615270970390412, -2.2244340629281489, -2.1011874565598978, -0.26859598793183537]]], [[[0.80333187276476981, 0.80834411577327359, 3.0820858667387903, -4.1761028728335683, 1.9506222693070852], [-0.13573639589724351, 4.309885688848432, 0.59764638821367733, 0.18090855533027383, -1.1569364081065734], [4.1355966039971612, -4.0510118051210675, 2.4397045304836427, -0.93590875967795739, 1.9238710212409629], [-2.8009019574744833, 1.2619003912516042, -0.9775288939362099, -4.7690683478134677, -3.7101375663188616]], [[4.0896667826335271, 4.8030510849032773, 0.69133294910565635, -0.50426070615955254, 0.46083630446983648], [4.016247880506377, -1.7800769264856333, -2.3785063730679057, 4.7932870994367054, 3.9500822080554645], [3.6490250359540841, 0.83355142800438919, 0.43675951841127958, 3.1459706601304607, 0.7537456213408964], [-4.4530297767278268, -2.235134595992613, 1.9688805696939333, -0.24850388099980059, 1.54490215964156]], [[4.1061433657602464, 4.6297905929485186, 0.17910930988487372, 2.3556656184224956, -3.5198994522751823], [3.7233522404172259, -2.2208099998798159, 0.068816061741681978, -2.5481266544713543, 3.0512419317074126], [-3.312234560082663, 3.4461726746871584, 2.3604143355985761, -3.3895230309731428, 2.9505187906779637], [-3.235304551828432, 3.5767255612468283, -0.94781198400263911, 3.6505338287226543, -0.90790979941474692]], [[-1.5841880301122, -1.3820115163637512, 4.9268045573224981, -4.0422322102073602, -1.9034345751492463], [-3.1110807724309852, -3.0113533165886128, -2.3359870321504483, -3.8329026831843671, -3.8837539744670879], [3.2209119071520647, -3.2694912214142469, -1.6695058000729168, -1.9271062121257287, 1.6185605304950288], [2.1058576990816489, 0.03773274989555464, 0.38939159984457028, 2.2028125408587895, 3.5584932280426891]], [[0.67237015796194655, 2.0673547488193567, 0.60316949633224226, 4.581333874648605, -3.3750276786510156], [-0.26288370152787266, 2.9197951035325085, -1.78090301054156, 2.8518901997161077, 0.87339379030024489], [2.0938258071925855, 0.43537118448338141, 1.905446850097503, -3.4885964596174688, -0.75922478350721256], [-3.0632753471630947, -0.60087838834990581, 2.0401719665618225, -1.0016843067815184, -2.0176512080567708]]]])+(1.-msk_arg0)*numpy.array([[[[-4.5642333508367567, 4.0825210061951527, -2.7472027785430821, -4.7736391733879024, -4.6221984969595038], [-2.7450922930759925, -2.4293487892144605, 3.3046949331175206, 1.7934495655840497, 3.2664539928356824], [-2.3173623623481965, -2.6178929252961227, -3.3947633270629018, -2.8985999274025698, -2.2549794622527894], [2.349524648882662, -2.0646243781464468, -0.20863096719855179, 0.26544963840176816, -1.1874461588657081]], [[-4.6569593878656379, 2.9227272706268295, 3.4193738899174555, -3.4487138842106448, 0.99201047610933202], [1.1327210970131425, -1.7853113885861136, 2.2487013575060661, -2.7559186500165653, 2.4158445661886585], [4.4264581339602884, -0.093448859597984324, -0.34937553028809454, 0.40973295632350304, 2.7429284127985865], [2.0350092152247514, -2.1581117251711071, -4.4340740200754638, 3.7627319206143817, -3.2173090732777077]], [[1.61908769813021, 2.7728333493489012, 1.6197450468838248, -4.9848613398967325, 1.2734219646039708], [-4.793448728380012, -2.372081856619638, -1.3151498546915361, 4.2636591553402994, -4.9919080230902573], [-2.5971808210978753, 4.4499969543854245, 3.6738563740609624, 0.5455669924864246, 0.28663522186543222], [-3.7179003027748161, -3.35332851042353, -3.3420453759497057, 4.8168130941807128, 4.4354270493683519]], [[-0.94598878299671529, 1.5026800800582851, 3.2569798530188034, -4.1542863248916948, 1.6484063942418175], [-3.5382558556250832, -1.2255806163259395, 3.8341091750573302, 4.5389317435296022, -2.5516746901961742], [4.1493022118946907, -2.681217438283686, 2.0429555642956467, -4.0100034819961188, -0.25232665204719584], [-0.12150762649189684, -4.846328276249122, -2.6652775039229071, -2.8213808665994677, 4.9365761154700003]], [[-3.7890733169354816, -3.3245797094496679, -4.2341268096482523, -2.580380414921073, 1.5690692796606944], [-0.69858336815573185, 3.4945214350509275, -4.9642478405135506, -4.7525593262013386, -3.7146465329653133], [2.5693708137293036, -3.5540706457521964, 1.5572786151874718, -2.1529813698229594, 3.1347742192689374], [-2.0242163873429506, -0.78366015566834069, -2.1492556609076754, -1.5029148157668635, 4.6177543150269962]]], [[[-2.8482564810654476, 0.61667884918475302, 3.9127412655553311, 1.348802524835186, 0.19641382600028301], [-3.1676940307115844, 3.354085933357613, -1.4267605336563092, -1.6135633582209028, -1.7644861677516124], [0.034480559886825191, 2.1948373374543486, 0.9555647697356342, -1.814752466741071, 0.94269982290583876], [-3.8871473850801932, -0.90297733242592582, -1.3999571075347248, -1.0169725011947905, 4.5757201872933901]], [[3.023064351547653, -1.399361863518279, -2.8404219141429152, -1.2744557516016597, -0.50588768693219954], [-1.1023235344092788, -3.9241025928140347, 3.5274521039890772, -2.5438432225045791, 3.4015678507727554], [1.5284321391807989, 3.3769916992927449, 3.8993804658565843, 2.3490221481709552, -3.094838218360294], [-1.3288898577613475, -3.8456692433690733, -2.0443718799104782, 2.4481774028750616, -2.8883026421216949]], [[-1.5919521036503195, -2.3450251695746571, -2.9333719025079752, -4.9848398243718286, -4.7978578513491836], [-2.2650853304039211, 4.7557818180342757, -3.925276771548095, -3.5287438670787719, -1.271246077770777], [-1.2769183816651508, -0.2550310512085332, -2.4769231670606171, -2.6847547121544446, 3.7163595230995732], [0.93696874788771556, 2.6752741278541663, 2.2547840679370346, 0.5069449506120165, 2.7149278750025365]], [[-1.2893540220487996, 0.58529404528922235, 3.2864936200183532, 1.5415839267399107, 2.5417111232248937], [-0.76347139739238834, -2.2813221862343758, -4.5806501930576671, -2.1739193935910182, -2.0662908664812196], [-0.66141576118704304, -1.0874373766110015, -0.73877132145793301, -3.3715004980926966, 1.3291402670567498], [-4.7699031665610301, 2.2083036747174445, -0.11284656146654903, -2.3593469077064455, -4.23322678606762]], [[0.82531748077643918, 3.7126060625761301, -3.7833476435088533, -3.1858158454192154, -4.006966156356949], [-3.3551494380129174, -1.0568085104098834, 1.4622105693104368, 0.80510885732238524, 4.9832860994673585], [4.9402724801797717, -2.0510232716153052, -0.68732889353630444, -0.5082304598298526, -3.970489580778882], [-3.4486042716184806, -4.2599445404940406, -2.649407091120338, 3.595022812782549, 3.1842760365869154]]], [[[-0.23549300124861539, -0.50091459377978254, -3.689150855756397, 3.9259470440462234, -0.97252667057420616], [2.2838242003813409, 1.6608332647426352, 2.9233986021742489, -0.89468580965600175, -0.28739572539849689], [-3.0274023000112482, -0.17832075662430924, 2.9297524778739366, 4.6394236631435, 3.1390943235639455], [-2.3753758868756245, -0.7751629967107041, 2.5245331957844428, -1.3698806652928477, 0.11650851067441259]], [[-3.8690775867163918, -2.1408278913016621, -0.90659712455668728, -4.8342088225160111, -0.74660993384586405], [1.8051625403008451, 0.90543488553950269, -4.6555009132082095, 3.8416399378802186, -1.4516926248261717], [-4.3869937993060502, -1.6385917874331959, -4.6272853187806646, -1.4807921615991648, 0.21484161135312618], [-0.45774259784221893, 0.76623343703677715, 4.9546188552069381, -2.1527586409624542, -3.956438172121107]], [[-2.2168124193462955, -2.3355102193250299, -0.92603252412141579, -3.2532585239999321, -1.522475150312903], [3.4498828982016612, 0.27227485301162702, 2.5191824943258103, 1.9271569358464173, -4.467953629297293], [1.1208742975620014, -1.5478302434482782, 2.974132033704274, 2.4619881524155129, 3.1546634801103153], [3.8659696344602086, -2.6721897230942337, -2.0068132728931012, 0.23860364857133654, 3.817687564842851]], [[0.68515420363769053, 1.7616977067855562, 1.0665389985186664, -0.001145839127567605, 3.1639209203762704], [4.8694163123829579, 0.47840833401389204, 0.0081600019282710434, -2.3322634524478012, 1.0695532335633162], [-1.9219192670605945, -0.57862863628802241, 0.39617353683853018, -2.6252089832751646, -1.6191540746047273], [3.9109447591067745, -3.0102563619214506, 3.9233148951921688, 0.058248170414064937, 4.4634558864615919]], [[4.2896165682755374, 1.6626453415656686, -1.1653155777344768, -3.3796761964601174, -3.8067993909665923], [-4.542708270432394, -2.2299827089267978, -1.6775628322986105, 3.2656464840124464, 3.7822056247952602], [4.4495745380585898, -3.5221744819223511, -4.8499874008764632, -4.1859526630834596, 0.90983906589403052], [2.2342420161848286, -4.5347703280173421, 2.9326857083766349, 2.4869509128767273, -3.6892637832142992]]], [[[1.7114408399295158, -3.990566213113774, 3.7881727452127532, 0.76564693332895484, 4.0981011898331872], [-4.6829778823345887, -4.5069799026804294, 3.252004911053664, 2.9857385821714821, -0.025550294635695359], [-4.4181012405372329, -1.2509929360326488, 3.909239421830911, -2.3818640501242672, 1.5748487705619008], [0.056812128174845711, 3.627993933391414, 3.3902614207707806, -1.048287864215415, 1.615629730805094]], [[2.2633444347280864, -0.34443446058723648, -0.37175213923827677, -3.3272987355641606, -2.227035343390952], [-4.8205052050800443, 3.5467329176533102, -2.0760660506614306, -1.9785893738386608, 4.6406670178694913], [-4.8084231579169732, 0.78814354462605341, -1.9583299497557349, 1.282790873468926, -0.49391553730200854], [3.6751388321689564, -0.22663731306198454, -1.3790900806749482, -0.82860210298115966, 3.6667040751695428]], [[3.4221107381235303, 1.2792802195826098, -2.6713372345594788, 1.9686045131262357, 3.7767494586894124], [-3.4784410085774931, -2.1016440935633183, 0.16725101588033642, 1.7221571333768315, 4.1450910413574533], [-2.4409385875231382, 2.271284374213395, -4.1060556737994425, -1.2333585980878703, -3.2015653825982051], [2.2511648302191674, 2.7740199049772363, 3.2603203506062499, 0.46614404832548928, -3.9850851444696356]], [[-3.541483542002164, 1.2568751582098656, -3.8278936479660897, -0.75876462688849777, -1.7071535066686705], [-1.9063548139208573, 4.2741710969633235, 3.1578833727784321, -4.1921702399108742, 2.587928881337664], [1.9655176654082176, -1.1262403453781209, -3.4760820397857817, -2.0024205581445251, 2.1434031358681214], [-0.21341006582650657, 4.0856386957232509, -1.0616334897327828, 0.31962645272964529, -2.432209789690579]], [[4.3080901495189234, 4.0838797741680253, 0.68301401443723186, -2.2410702480755731, 1.4091579273772856], [4.2138374169557409, -2.8934830074003051, -2.6683189769040503, -4.6040308789391657, -3.1272523920645998], [1.2356578504018456, -4.0040394265857451, -4.0850083515724123, -3.0287030107095503, 0.62300607596694135], [-4.9920769649340695, -2.1013453269303328, -4.0328267715007771, 0.72561113398818744, 2.7113591495324094]]]])
      arg1=numpy.array([[1.5926342433070495, 2.9225640598965041, -0.42374815821114797, -1.2519093298790041, -1.6545908114414356], [1.8060898958277321, -0.92045047099275656, -0.92143959741918202, -3.2939038294769785, 1.9842842311262467], [4.3067768454246096, -1.0710390117196358, -4.6065046669358622, -4.4292854239622717, -1.6766185466245864], [2.4270601210711584, -2.1241340778128492, 0.61807307736062711, 2.5270115996559204, -3.0747983809025206]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-6.916880778890441, 1.0178402959304889, 6.3004011755535592, -6.3129330365238445, -1.2594077489270181], [4.3427026876751533, 6.0737232670499184, 1.632921968451136, 5.9727740950851675, -5.9222484564588562], [-7.5345626448657459, 4.7828659220970975, 6.2411665212584406, -3.6971492493788647, -3.6561220854981542], [-5.3094100253504299, 0.41640689686255833, 2.7960008183923688, 2.3102101736634859, -7.0620289859869159]], [[-10.284950399814429, 8.8781896190292535, 1.7062559684157179, 10.231275560606466, -12.549927026846472], [-8.3889326944784255, -13.325229880047042, -0.65122791501874955, 1.625753246467887, 11.052681624151994], [13.881500855934464, 0.67229263429955977, -7.7100869263043146, -12.462827112403554, 7.6825879999893401], [13.461830677138259, -0.75243405399871843, 1.4056379361851519, -0.25881038332288003, 8.7452196873967072]], [[0.34321978208331633, 1.2734182306248907, 1.7355724258575929, -1.6989746645816948, 0.017210296812392552], [1.1278725576358146, -0.67984679393779612, 1.2042969734452782, 0.29717255180058499, 1.5462509839600858], [-1.6851063220459896, 1.6585867144610018, 0.66971674841220963, -0.83330462923888493, 0.31465821100784469], [-1.0004427085243028, -1.9760539831159185, 0.71764303471097812, -1.5986212786484204, -1.0289505184540244]], [[-5.0972165241560941, -2.062015147999289, 4.3420774580476955, 5.1347977612084055, 4.9434528577144734], [-3.3669048015895764, 0.61400175861014095, -3.4546765382916238, -6.219852796507416, -5.5668597022458481], [-4.9887973792076945, -0.020827682662961155, -5.8542204944943332, 1.8621626760995973, 5.526619809144977], [4.1394460601417071, 2.6057675753704821, -1.3029627680718643, 0.3373979781653399, -0.58712652181633596]], [[-7.7379767735206517, -5.8456107213828306, 4.0104959332804926, 4.893641421917132, 2.0870938359003959], [-6.1961194603785383, 4.4589114606603237, 5.4369628373362744, -3.2566568613146503, 0.61661246063952269], [2.5427322410245896, -4.4495843818580978, 3.255263032434025, 6.0689385122785193, 6.908293348465528], [8.1972609833115335, -3.6806820143585957, 0.58690427826439728, 2.4213889763445411, 2.1700842114457579]]], [[[6.4169053627457426, -2.2161642373301516, 4.6950825865700123, 0.65172535616772198, -5.5245179422477415], [3.2272597307283943, 1.1974821607505624, 6.6551123156929872, -9.0191277587542285, -5.590271377934644], [-3.9024302366600567, 4.3538513489206982, 3.0336957889493301, 2.6916141553063095, 1.0893752568807371], [3.6444048725982063, -4.0900451046114128, -0.76166026233240414, 1.3309690861140144, -9.0090158993480838]], [[0.55778594268693482, -1.9269700641243017, 3.7269576585331441, 2.0803438073679752, -4.1339916175425779], [-4.1801154537296341, 3.5525816985215424, -0.38791409405451438, 1.0285715543602605, 1.9165720413733911], [-1.6959336976895103, 4.4013716586691354, 4.2997504942724358, -1.7730570170746494, 3.1024219199188656], [1.9423987012207287, 2.7606093386821819, -0.98409661036569374, -0.90956168920236879, 3.3075028092422283]], [[-1.3863725167416328, -1.3116181252568162, -2.0031804332013303, 2.7289580166025549, -2.7859085054763515], [-3.9909945763241641, 2.1829750065046003, -4.2301607789367299, 1.9598020369695686, 1.166188582461583], [-2.6622186571159228, -2.977442832562577, 0.56864045343432623, 3.8050052443354883, 1.5984060418863244], [3.4038496525977466, 3.7274728559319881, -3.1972298397070347, 2.0856504506449269, 3.7542165356918518]], [[12.328146779185802, 9.3908132583157347, -3.2565047520812711, 14.04054888350422, -9.475539814022941], [3.278138792867265, -9.445178072189643, 6.7647232072756349, -14.896560703161107, 6.4054450548203317], [15.66284416061686, -9.0069814957140952, -14.781635881811965, -10.016651917160383, -5.2084744597899695], [0.44384438595485431, -3.4639583124288271, -5.2030564758572426, -3.7382455923109035, 13.547784380154349]], [[-1.7202742519609804, 0.79241243254256954, 4.7958347054522141, -0.31070001382530199, 9.1771796990112104], [5.1437037186693395, -6.4980532016413797, -6.2125290394706596, -9.3175204290293454, 1.3081805550142178], [3.437408657206213, 8.8360960480415542, 5.3741646173066853, 6.6572043421533653, 5.3531773739325113], [2.7490663743432719, 8.5795543378701744, -8.7473936196460844, 6.7034345999150942, 4.1924374530502115]]], [[[-14.81275040728227, -12.432620726643552, 11.0090482749434, -13.792497552384452, 4.241891653200816], [7.0692708620708498, 18.530625297389761, -2.3566822281065103, -5.3602365323114247, -18.997020381284823], [15.192216091909897, -15.687258006351497, 16.805841728853689, -6.8785283830664019, 9.2333883815666145], [1.4987468346392658, 3.7432954023379468, -18.780023822370936, -19.909993328175155, -1.4629261355070853]], [[-3.0659337594081686, -0.69521281070834151, -3.3275531044213893, 0.62986947508172919, 5.0438203477825425], [-0.0081723995716315133, -1.7933005620057378, -2.7426164600585174, 0.36034845814598088, -1.5028686152312889], [2.8660347046325318, 4.0968213795793824, 3.2299834340400877, -0.94644099805792037, -2.9376260934736025], [-0.081381513349993079, -4.4604298881155868, -1.2854303163979439, 4.8720069581645964, -0.59337743744557692]], [[-0.39237909731397724, -20.836508269806231, 16.517217348300765, -16.896429570318585, -17.563850691366124], [19.094119925157759, -3.1867061173947326, 15.361108718745328, -3.3861182378845376, -11.923007773841219], [-1.4522603537608632, -8.2275102149905308, -21.309338827693264, 12.454442302808488, 18.96020830790421], [6.2420599354601514, 13.198797081341731, 10.605387270366545, -21.01641369869931, 22.341146360694715]], [[-1.4456307778844302, -0.46189665553788956, 10.394122425198328, -19.99714907252952, -10.787094242329799], [-18.109603776148898, 21.478047058009317, -17.246036485972024, 19.319654377812682, -7.6159946272422561], [-17.050032370340116, -10.757935376065248, 12.580678976889891, 4.426070578557999, 5.5571779205680505], [12.653006550584651, -11.893672365940462, 15.15387709920296, -2.7940562605118662, -12.299204681217534]], [[4.7100342098071284, 2.7692772656610947, -6.0977141660533274, 5.416446912181919, -4.6437781653263634], [5.9830183955648089, 3.9246374973799751, 4.8716205933778998, 2.9803346720497377, -4.6034836207900423], [-7.0060588444280034, 6.6352506627496979, -4.9770260249652907, 4.2423600679619522, 0.12790295484535211], [-7.7122185020863174, -0.061389842398641384, 3.7295274056488168, 3.5228898596032674, 0.45033301491546879]]], [[[1.9497347523727826, 1.9618997674958398, 7.4804076968787543, -10.135652744145053, 4.7342775211085515], [-0.32944039346012655, 10.460351681779329, 1.4505237153356283, 0.4390759402027028, -2.8079542187307713], [10.037341594398821, -9.8320492021978296, 5.9213095731334837, -2.271506827575541, 4.669350633738385], [-6.7979574440164638, 3.0627081163708607, -2.372521395667373, -11.574815601640784, -9.0047269309005085]], [[-8.6870005798911087, -10.202324486919029, -1.4684838763101808, 1.0711173500554774, -0.97887809861771757], [-8.5310489879272229, 3.7811220606764917, 5.0522664413285803, -10.181584472654214, -8.390504228292837], [-7.7510184296623272, -1.7705749938336868, -0.92773577686652753, -6.6824634869824964, -1.6010567602924179], [9.4588322982629212, 4.7477255638463642, -4.1821663132304607, 0.5278555621004255, -3.2815793241813043]], [[2.5378966661593587, 2.8615489193189734, 0.11070264234448209, 1.4559734980110166, -2.1755550864677078], [2.3013037773322602, -1.3726228708589716, 0.042533355052520296, -1.5749284828337493, 1.8858904905021849], [-2.0472030074905145, 2.1299865501599955, 1.458908552249552, -2.0949729305382907, 1.823636228764685], [-1.9996546405474436, 2.2106777745142434, -0.58581706971179259, 2.2562966775276836, -0.56115460369014236]], [[-4.0032615281295918, -3.4923591327092671, 12.450092265591605, -10.214767683696788, -4.8100012505882841], [-7.8617371993996006, -7.6097247616817514, -5.9030663268899906, -9.6857895407591972, -9.8142913436881152], [8.139281750843141, -8.2620422414870056, -4.2188605224770983, -4.8698197518106996, 4.090121235306178], [5.3215268328040537, 0.095351096672982291, 0.98399708961580556, 5.5665328426176925, 8.9923536645609161]], [[-2.0674026730685653, -6.3566990344208953, -1.8546245907321672, -14.086677980143401, 10.377529641817334], [0.80831437982356436, -8.9777812569088642, 5.475917693357613, -8.7689873685988537, -2.6855098123055083], [-6.4380922018476747, -1.3386786131411137, -5.8588648895756101, 10.726730745654059, 2.3344631350690439], [9.4189540777156893, 1.8475798956176064, -6.2731174595470032, 3.0799772846672764, 6.2038706677589737]]]])+(1.-msk_ref)*numpy.array([[[[-7.2691543289866969, 6.501962753486751, -4.3752892184159853, -7.6026612127295312, -7.3614716056200811], [-4.3719279869910963, -3.8690640706394692, 5.2631703141662634, 2.8563091917933097, 5.2022664831771479], [-3.6907106524266564, -4.1693459181378687, -5.4066163226033463, -4.6164095020286604, -3.5913575095379087], [3.7419334113044997, -3.2881914842025539, -0.33227282257468338, 0.42276418399212995, -1.8911674146929496]], [[-13.610262135373738, 8.5418576780133755, 9.9933392380212602, -10.079087250860104, 2.8992141645179532], [3.3104499680171515, -5.2176869000056971, 6.5719737688877089, -8.0543487985369069, 7.0604605034392343], [12.936607454948884, -0.27311027849938346, -1.0210723682272675, 1.1974708123062141, 8.0163839981141116], [5.9474447939740482, -6.3072197652263196, -12.958865369993362, 10.996825078212938, -9.4027918671403565]], [[-0.68608543006500367, -1.1749830248130455, -0.68636398038865032, 2.1123258117191956, -0.53961021212655436], [2.0312150701305995, 1.0051653178686519, 0.55729232869719736, -1.8067177143155506, 2.1153118307439494], [1.1005505894815417, -1.8856780134660416, -1.5567898720406192, -0.23118300824691762, -0.12146114734392068], [1.5754534057134977, 1.4209667801689032, 1.4161855727167714, -2.0411156771064181, -1.8795040430497456]], [[1.1842921833944724, -1.8812192120482958, -4.0774434652221876, 5.2007898091206721, -2.0636553443835393], [4.42957551715606, 1.5343158080973036, -4.7999570480289631, -5.682330997408684, 3.1944653514727079], [-5.1945501515585519, 3.3566411264216294, -2.5575951314699457, 5.020160771958234, 0.3158900898750176], [0.15211653125665889, 6.0671635846927074, 3.3366857738777114, 3.5321130300379835, -6.180145696614745]], [[6.2693658940793702, 5.5008190391600582, 7.0057473137218391, 4.2694737245518466, -2.5961676126416173], [1.1558696219762836, -5.7820030568204039, 8.2137988626317107, 7.8635409919630357, 6.1462200211971929], [-4.2512573395823106, 5.8805326336753136, -2.576658887543434, 3.5623031917136641, -5.1867686191458837], [3.349249834866824, 1.2966368928616017, 3.5561386679763296, 2.4867090445470503, -7.6404938591377078]]], [[[-5.1442072511781571, 1.1137774384832562, 7.0667624647076961, 2.4360586115717631, 0.35474102653997741], [-5.7211401819420145, 6.057780713975113, -2.5768577836024429, -2.9142404775606359, -3.1868206389039839], [0.062274990814077988, 3.9640735381617414, 1.7258358754284824, -3.2776060936095006, 1.7026006249488277], [-7.0205376157865276, -1.6308582362559438, -2.5284483865106844, -1.8367437587425675, 8.2641619964055693]], [[-2.7825810062234493, 1.2880432863647013, 2.6144676886909934, 1.1730733968211753, 0.46564455970617924], [1.0146342164334208, 3.6119420797795754, -3.2468449505211363, 2.3414816922860715, -3.1309747303576012], [-1.4068460823894329, -3.1083536001526362, -3.5891865863776475, -2.1621585426563725, 2.8486452957361164], [1.2231772954739295, 3.5397480663414211, 1.8817430597479468, -2.2534260435501738, 2.6585395273105372]], [[1.4668877054981704, 2.1607990481907211, 2.7029250249276888, 4.5932288009682836, 4.4209362070216534], [2.0871393149674842, -4.3821656838229686, 3.6169054481341432, 3.2515243282764712, 1.1713764741218189], [1.1766031595386899, 0.23499570915498164, 2.2823350858945806, 2.4738393011368434, -3.424400822829814], [-0.86336010584801171, -2.4651035153558962, -2.0776473238270867, -0.46711915120562358, -2.5016420481644528]], [[4.2470081507780852, -1.9279022971482416, -10.825393920530111, -5.0778291997487495, -8.37215200221471], [2.5148013595669281, 7.5144558855082026, 15.08822121240711, 7.1606814155237251, 6.8061633979157934], [2.1786399086504318, 3.5819141391353768, 2.4334416848580536, 11.105398401751074, -4.3780602155702821], [15.711602306569743, -7.273939930799874, 0.37170572095797511, 7.7714618143589282, 13.943841921672655]], [[1.6376644627775276, 7.3668656663535188, -7.5072370698832627, -6.3215641453374811, -7.9509597587156406], [-6.657570122921121, -2.0970084625263499, 2.9014413752688317, 1.5975648099248803, 9.8882560263637007], [9.8029047798876743, -4.0698131355392153, -1.3638558850415399, -1.0084736872184179, -7.8785798649905976], [-6.8430110755671674, -8.4529407771746694, -5.2571767127441458, 7.133547077943537, 6.3185087269525999]]], [[[-1.0142158050370855, -2.1573273740260417, -15.888349484850034, 16.908177825661465, -4.1884553463868786], [9.8359211852227322, 7.1528382487045414, 12.590425409790726, -3.8532121289564381, -1.2377492556202558], [-13.038346127473652, -0.7679877056881721, 12.617790134532846, 19.980962408541448, 13.519378748329029], [-10.230213868775886, -3.3384540456636134, 10.87260111311023, -5.8997703302780957, 0.50177615606746606]], [[4.1439330347433181, 2.2929101889615642, 0.97100088831305786, 5.1776262397138924, 0.7996483656863369], [-1.9333995031571243, -0.96975608498471055, 4.9862230971423829, -4.1145462424499124, 1.554819434214507], [4.6986415032289219, 1.7549957286243616, 4.9560030947716225, 1.5859861733213527, -0.23010374709990636], [0.49026017961490886, -0.82066590315040955, -5.3065900821283121, 2.3056884872873331, 4.2374996297984326]], [[10.21175675544009, 10.75853872499715, 4.2657731440996987, 14.986150573554562, 7.0132888852102662], [-15.891901670948171, -1.2542353810873357, -11.604625916974971, -8.8774574188943376, 20.581649245011004], [-5.163312682767816, 7.1300872400689652, -13.700353093342185, -11.341159914042862, -14.531972043740296], [-17.808607163373281, 12.309454430371638, 9.244394707250903, -1.0991288206917862, -17.5861955843516]], [[-3.0347435273389007, -7.8030619740930236, -4.7240056402260482, 0.0050752485459408386, -14.013908815191909], [-21.568034695641952, -2.1190070605498059, -0.036142977600394963, 10.330260514766971, -4.7373565475737118], [8.5127289956237444, 2.5629113845977045, -1.7547656720784819, 11.627799884475502, 7.171695541795839], [-17.322690615433274, 13.333284626248378, -17.37748147888874, -0.25799777218748826, -19.769920098402928]], [[-7.1920506962788773, -2.7876220161279703, 1.9537897103001687, 5.6664277925706719, 6.3825504621737688], [7.6163889381118492, 3.7388303684388058, 2.8126329577599209, -5.4752434618146388, -6.3413160976795648], [-7.4602391950975582, 5.9053430608388577, 8.1315788272050504, 7.0182458702183075, -1.5254530523215208], [-3.7459716019833933, 7.6030800366367348, -4.9169952500851295, -4.169668025074067, 6.1854880823274812]]], [[[4.1537698121654554, -9.6853441163423906, 9.1941230016345266, 1.8582711387031343, 9.9463379699583943], [-11.365868866072544, -10.938711188264842, 7.8928114331459076, 7.2465670447319459, -0.062012101191914547], [-10.722997331762931, -3.0362350667865647, 9.4879591044450766, -5.7809272498696433, 3.8222526477487317], [0.13788645068635119, 8.8053593952223927, 8.228368294358809, -2.544257670640091, 3.9212304900539743]], [[-4.8076470436339882, 0.73162497530643578, 0.78965138745585095, 7.0676286312754373, 4.7305216653903619], [10.239399378384739, -7.5337362552879901, 4.409842646000282, 4.2027891149690868, -9.8573989564387166], [10.213735490275917, -1.6741225613484121, 4.1597553818776811, -2.7248198090426565, 1.0491428243444398], [-7.806487634103398, 0.48140803997890053, 2.929372236735329, 1.7600619638896731, -7.7885710793228728]], [[2.1151145149808577, 0.79068866212400246, -1.6510816252322045, 1.2167414495339515, 2.3343071603522478], [-2.1499307385888948, -1.2989696324254658, 0.10337335007685064, 1.0644189591247741, 2.5619691758717678], [-1.5086784244387283, 1.4038197227311793, -2.5378424661192849, -0.76230574420935882, -1.9788013683937262], [1.3913843742595744, 1.7145470193289147, 2.0151162322806835, 0.28811108644187605, -2.463073838786467]], [[-8.9493699906300037, 3.1761381041157004, -9.6731316506595242, -1.9174070135558303, -4.313996713745011], [-4.8173807278379099, 10.800879940940389, 7.9800079133716588, -10.593662823987122, 6.5397263022248469], [4.9668859398151897, -2.8460224167710013, -8.7840996358942824, -5.0601399778206977, 5.4164045870776176], [-0.53928971182691565, 10.324456376095741, -2.6827601431379366, 0.80769975360468838, -6.1462223513447798]], [[-13.246508616522883, -12.557106917412396, -2.1001303857253313, 6.8908391702715823, -4.3328765135356297], [-12.956700467041971, 8.8968768663234137, 8.2045428699160432, 14.156466692187356, 9.6156705917937657], [-3.7993987577650841, 12.311613945935706, 12.560577065388127, 9.3126511135643142, -1.9156180736755839], [15.349630169120045, 6.461213208962465, 12.400129227470929, -2.2311079399517206, -8.3368827230274878]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.8390979152443663, -2.4153509310307397, -4.1739877743171894, 0.79191204530283965, 0.43878672648455819], [-0.48356051654220611, -0.89128842038101297, -4.8860526221012837, 4.2454593588205984, 1.2422326245781532], [0.086936915683594407, 2.9632235029563612, -1.5992050524029988, -2.3259142160215642, -3.6873491305516115], [0.77440593406138714, -3.1523251552226483, 1.2037658221656962, 3.6398000237563437, -1.9477342076728679]])+(1.-msk_arg0)*numpy.array([[4.8183216960011919, -0.63339815885980233, -0.66560827720936278, -4.8201448612450637, 4.2864172897311867], [-1.336516230356275, -4.8026934851873291, -2.7987982976019463, 3.0350084230714867, 2.5196062599023081], [0.46402567629136193, 1.616196730458201, 1.1625116366022192, 1.7939989435872237, 0.38370943126169443], [-1.510453541794774, -0.50701810448876294, 1.3282507826222814, 1.3198864211484551, 1.3595520491036144]])
      arg1=numpy.array([[[[3.9914102889716094, -1.3329107969433016, 0.26571388172129229, 4.2989851332044573, -2.7329253224501695], [0.17587573391533962, 4.3961635512469801, -4.6063586040400031, 2.8876092834124991, -3.4828808067136894], [0.17077612117010332, 4.3364257631136311, 1.2560837646262266, 4.2934765388183109, 2.6184416281320377], [-0.95155629172883494, -0.59313583306202755, -1.3791523976283293, 2.7189190832942618, 2.4842105200108024]], [[-1.1637586429445248, -0.1218909770704073, -2.6994320121210769, 0.87586671872947264, -0.23849761819371196], [0.67242353698567747, 4.5606230084088395, -2.5336097869252994, -3.1520665189796593, -0.85767423058618419], [-2.3552786899100808, 4.9128288657019468, 2.6128535960628394, 1.5425612581792034, 2.8886884286412249], [-3.8468687483974948, 4.4684575222919314, 3.2661489826923358, 2.5723259380422263, 2.4385733472581306]], [[-0.41916003356465037, -4.9988594926339509, 0.22291359719183212, -0.82979001917603323, -3.6862996095128375], [-1.9675119230778004, -0.25286662115035252, 2.8031275493962369, 4.2480522310710143, 0.055400893084741831], [-0.75511502235906214, -0.0081524284550962278, -3.1844753612976637, 3.3582233712574219, -1.1605417972074195], [-2.1472495432889049, -0.94003278988278449, -0.18962450312655577, -3.7962025437628477, 4.161464301451856]], [[-2.5671643631569516, 2.7107761950632732, -3.9047871337047626, 3.2167687902759319, -4.739047698834109], [-1.1388415395990172, -4.2808528925699463, 1.4451799865257868, 4.0575090990422407, 4.252842201919675], [4.418846861526994, 4.0226662807315474, -4.2401809306227101, 3.0590428101583793, -1.1845866751938017], [3.3957879322616797, 2.3542758683316105, -2.016931050732611, -2.7860027852392588, -4.683403309522058]], [[-1.4332083729064813, 4.6014595974389181, 0.90543238205070598, -2.0959716198498421, -2.1874956824537084], [-3.4455071343834343, -3.4063663265066735, -1.3192425524570228, 3.8258735565460249, 2.6544956748262871], [1.8448757026356564, -4.4682318636421492, 4.2039414908460451, -4.3381958478192404, 3.1193969204347187], [0.55117651560301795, 0.059212696399190357, -4.9995835352655842, -0.99945678859500564, -4.1014419311363062]]], [[[3.8340208452593316, -1.057160075473369, -0.52357456027622185, -1.98053249338236, 4.9628294301360896], [3.7410634327911634, 2.6930006822864243, -4.6012985704966489, 4.9722890056003308, -4.6798420669009202], [-4.9931497168666974, 0.87844158722421817, -2.2916652449057597, -2.1269824611585699, 3.5924874879339015], [1.1872132114272747, 2.5083937065648101, 3.297400251744838, -2.6137238313502795, 3.4520944275323586]], [[-2.6393063763741544, 3.0378437134529293, -4.1725205855919798, -1.965291718358817, 1.006643575641184], [1.7479554890928526, -4.8582963407450457, 0.048562568072468615, 3.0525600535138011, -1.1211910706184911], [-3.6756661857890527, -4.369956975507602, -1.8646939763945189, -1.3172605425355388, -1.319408019138911], [-2.8410679003817938, 4.4834515584745365, 2.016741240267681, -3.7179603499036307, -0.77283108171604464]], [[3.4296349372913024, 4.3056143559835842, -1.245876438255852, 2.8170333518886483, 1.6190176080685781], [0.49061080736406826, -1.9924363814361112, -0.13716251582368333, 1.4376538356168034, -3.2797470752116542], [3.1614606621791186, 2.9260360430004102, -0.59420668251985198, -0.7463012296186804, -1.6099231617857024], [3.7395114741906337, 2.1776386715727636, -4.8902991077257187, -3.7357934512621096, 3.417184040798535]], [[-1.2158127771285097, -3.371721993832665, 1.2580143732041265, -1.0349685243800364, -3.0493303438766217], [1.8209065131145508, -2.8840980714032627, -4.6367788631183737, 3.4274524117615321, -3.3798746943078117], [4.8868869313260728, 0.50517254213815121, 2.5499214052062644, 3.6477228255969791, -1.6353826965806171], [2.4060426702130142, 2.4188324376305159, -1.2468497838269252, -1.9051019438383374, -2.5494608809408694]], [[-0.50025538469377473, -2.0261415860453824, 1.3987839174158312, 2.3323749023082314, -4.7160784558081073], [2.9835920048132314, 3.6329358456940586, 2.281196412521532, 0.119305673636946, -0.093312453036706877], [3.5595783975630031, -2.2380875275031475, 3.7297943933354496, -1.4182958289082546, -4.7224611632445157], [-1.5289025393811482, 4.0067691191228914, -4.0977477643029072, -2.8636926819999675, -1.4865811927227179]]], [[[0.071852392064429615, -4.8077990581617369, 0.76746587858423609, -4.7945349994106774, 4.2750478458365553], [-0.76858733355437181, -4.7621086033836164, 3.5074042103264613, 2.5081069604666775, -3.1706050499495966], [0.37286239240656194, -2.838134699609256, -4.9822212083557034, -1.9751308036506088, -0.90710547739697844], [0.75204636972041605, 0.39908550784708474, -0.96806853391767689, 3.9589519018229229, -3.3058571046025262]], [[1.9402879812581446, -0.98338657552433695, 2.1705188695677808, -4.1812221622047403, -2.6702789993058964], [-1.6698333997928447, 0.27026678875887367, 4.8504472059123565, 3.7101210215531459, 1.1448559178250637], [2.1909087927355948, 1.9592613547082376, -2.3633607441003548, 2.9697594983105047, 1.0415964755123301], [3.7292203489814462, -4.9144331607888709, 0.41936956764485966, 1.7377520355526848, 3.1196666567103701]], [[3.6716919074375429, 2.334450243635465, 1.9772866981069503, -4.0602960580397234, 0.4162191586806081], [0.24313501700880114, -1.2130094572431394, 2.6477240469332584, -2.0625147109484541, 2.0292426007366862], [-3.1898253830312981, -0.89271457368153051, 4.9548206333833509, -4.9596427532943022, -4.3106497535705781], [1.7511406976008708, 4.2436309364508631, -2.5514019999711, 4.7857939915445726, -4.4165845925227964]], [[4.1657879175377186, -2.568639241330998, -3.1859111427811602, -1.5228078047801885, 0.063253662483187867], [3.1662446023172066, 4.586273192660645, 2.6466134644776336, -1.1824445305120634, -1.2687596233303831], [-1.0323958810688136, 2.2454958228461175, 2.3344838794039742, 1.09384318664305, 4.58286546434584], [3.4690936480195464, -4.8855128061388422, 3.2757423339507579, -4.1504225131924706, 3.8514793808553911]], [[-4.5786580026748922, -3.8117445447681364, 1.7621048610570735, 1.000587393917824, -2.2275828696796585], [-4.608736003684097, -3.9825010431892052, 0.39390647504198206, 2.4914793855150466, 2.7473260849703713], [-4.0367186365670982, 0.49430618563839523, 1.1791430952153465, -4.3076034706949065, -0.041792226364362861], [-0.66103592960553659, 0.7801840507840776, -4.8237487123629643, -2.580884519903627, 2.8193250510289216]]], [[[-1.8218432711641022, 3.8667318054567819, 4.8976810001790909, 2.4239750911114371, -2.0135935805072389], [0.19561618277807646, 4.2227073401623034, -1.0814846723777691, 1.0086487490045535, 2.5812249927767672], [4.5908980871209497, 1.475562242996431, 1.4806395287004337, 1.2372379151914545, 4.6272968408093043], [-0.64909507477233586, -3.6407508690680226, 4.7019489093252265, -3.0208959971413121, 3.7288808038729329]], [[-3.9504555130398069, 2.7796575551961791, -1.2963191241120429, 3.8489553964659819, 2.0771671419978937], [2.8150629450889557, 0.62123020825445874, 1.7407524748159817, -0.9310641215564095, 0.18463161441196618], [2.890986239873186, -0.26183212653203025, 1.7298030695719504, -4.978388267302619, -2.0928681775818969], [-1.3337674848238645, -0.31836995032431048, 2.4964376409857261, -4.5452939544008553, 2.8152688588212449]], [[4.2213733870675121, 2.114564681575728, -3.7271927670968275, 0.87839256653560316, -3.9707894365188334], [-3.7890774369691158, 4.0181710743001613, -3.6142471254929029, 0.8693685158270803, -2.4584224847625444], [-1.9605420337997472, -0.35226830972941681, 0.80347062884011411, -2.6007311234696315, 4.4855630572346143], [1.3087830679881858, -0.0077930233706249297, -1.6298565023212408, 1.7730544845485436, 4.2558532611085411]], [[0.79627441428694468, 3.3651429367838794, -2.7063248357208245, 3.837560430239078, -0.39653495551722528], [-4.5460373616780059, 0.68153494122508995, 2.2629874209921184, -0.92221725528156284, -2.0412789236208373], [-3.1907664820497983, -2.5238858192921976, -4.566955365810637, 4.5338592714702859, 4.248418606630942], [-2.9287416116665854, -4.1497069843522656, -0.46633842467354825, 3.0089382847290516, 4.4024460837051933]], [[-1.4538624842992167, -0.82555399035882893, 4.3799411031456401, 4.0524997574575909, -0.64058527500430351], [-3.4248434084527011, 4.9379187220773204, -1.6184204892294076, 0.15234932370163534, 4.3525916942321867], [-2.0810254921639579, -1.2119668682259501, 2.8170762099656335, -3.4742335939486901, -0.54818494652047534], [1.0861009490648312, -2.2947695427898851, -3.2716385874855702, -2.2068010759705849, -4.5375132076005054]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-19.314825208247427, 6.4500858586950374, -1.2858154910889936, -20.803209995756212, 13.224893230387153], [-0.85107989733179279, -21.273465875912532, 22.290620317677927, -13.973424063401703, 16.854001250812832], [-0.82640237192776622, -20.984388869895131, -6.0783123267750687, -20.776553368146285, -12.670895423882808], [4.604674067542665, 2.870242373227188, 6.6738534921675177, -13.157115667687385, -12.021337948412397]], [[2.8108855217311284, 0.29440948495125485, 6.5200756237308264, -2.1155254945420707, 0.57605544415279608], [-1.6241388161053392, -11.015505029440504, 6.1195567577186161, 7.6133468012883432, 2.0715842513674132], [5.6888245765111742, -11.86620577476789, -6.3109583658973953, -3.7258267711150883, -6.9771962855763068], [9.2915380129949448, -10.792893036739128, -7.888895986231037, -6.2130698493648122, -5.8900104046866728]], [[1.7495688555812334, 20.865178407783539, -0.93043862940777389, 3.4635333952911886, 15.386569502576814], [8.212370712750042, 1.0554621852144679, -11.700220121031595, -17.731318077151272, -0.23124265042196612], [3.1518408715299766, 0.034028136702567227, 13.291961225670763, -14.017183295054735, 4.8440872731278679], [8.9625933420960582, 3.9236853724280216, 0.79149035776121546, 15.845303006497941, -17.369901117517468]], [[-2.0329683814561834, 2.1466963209908059, -3.0922479655243515, 2.5473979519737546, -3.752908955971435], [-0.90186233289969264, -3.3900589697956436, 1.1444554389603661, 3.2131903294574231, 3.3678769664724419], [3.4993380559918759, 3.1855978819448865, -3.3578503532235282, 2.4224928484614683, -0.93808845679121411], [2.6891653668520474, 1.8643794180976045, -1.5972319936204675, -2.2062691638782295, -3.7088434938217012]], [[-0.62887281031789488, 2.0190593938111756, 0.39729171097314514, -0.91968452587844907, -0.95984406970296732], [-1.5118427965752979, -1.4946683296150929, -0.57886612103175006, 1.6787425338206645, 1.1647574676244448], [0.80950697033039887, -1.9606008326215354, 1.8446337251009493, -1.9035427549135071, 1.368749963323562], [0.24184893899661325, 0.025981745219324722, -2.1937508932252805, -0.43854837253037166, -1.7996582788298046]]], [[[-1.853981100367188, 0.51120087216369992, 0.25317998481552828, 0.95770731552859745, -2.3998283627474697], [-1.8090305659776538, -1.3022288009749368, 2.2250063135142741, -2.4044026399452285, 2.2629868472065544], [2.4144900562606302, -0.42477966767029834, 1.1081588295684504, 1.0285247375940509, -1.7371851053367298], [-0.5740894337635043, -1.2129601564376986, -1.5944925689801344, 1.2638936459864152, -1.6692965645300186]], [[2.3523832111000553, -2.7075949247278519, 3.7189192817395349, 1.7516417512439166, -0.89720976241992567], [-1.5579324867698896, 4.3301432712855075, -0.043283254586955963, -2.7207114282144964, 0.99930461827685169], [3.2760787085798277, 3.8948920498331594, 1.6619801487146606, 1.1740590681867364, 1.1759730892163613], [2.5322109211264903, -3.9960484574075603, -1.7974981143554263, 3.3137750073048453, 0.68881539404404302]], [[-16.757376778202339, -21.037458313810522, 6.0874178379542139, -13.764173195542298, -7.9106052291316242], [-2.3971502217524336, 9.7351490058859049, 0.67018327009431677, -7.0244522931894497, 16.025016796666918], [-15.447063158110343, -14.296766080265019, 2.9033251191962277, 3.6464670798557655, 7.8661692860246202], [-18.271449843846984, -10.640057141227258, 23.894258778162815, 18.253283388168036, -16.696541042746343]], [[-5.1616837332338941, -14.314508694058135, 5.3408488942502874, -4.3939168079139703, -12.945808046546638], [7.7305845976395524, -12.24432114899542, -19.685256219207435, 14.551109918425228, -14.349120652590008], [20.747079858096352, 2.1446894968396073, 10.825587693989906, 15.486259008314212, -6.9429507744514476], [10.214756371977543, 10.269054809757316, -5.29345008379146, -8.0880328769757845, -10.823632556937422]], [[-0.62143355948750145, -2.5169391802000973, 1.7376150169491786, 2.8973521963945679, -5.8584665178749891], [3.7063153268095341, 4.5129514305205829, 2.8337766067048902, 0.148205400089088, -0.11591577344161405], [4.4218244151963866, -2.7802253433258648, 4.6332722783699758, -1.7618533499729483, -5.8663953252856329], [-1.8992526142196469, 4.9773393189267239, -5.0903559601092603, -3.5573724763460701, -1.8466796566844632]]], [[[0.0062466253505698852, -0.41797522134307152, 0.066721116376513442, -0.41682208498580842, 0.37165947411682448], [-0.066818612212695069, -0.414003034128481, 0.30492290410143558, 0.21804708334752779, -0.27564262389344674], [0.032415506370232564, -0.24673867707861344, -0.43313894510783563, -0.17171178014104305, -0.078860952404587764], [0.06538059183453708, 0.034695263146246463, -0.084160892509141924, 0.34417906768418516, -0.28740102036484139]], [[5.7495069485678858, -2.9139942130854859, 6.4317325279135211, -12.389895782127102, -7.9126334901940254], [-4.9480895762876829, 0.80086090051883652, 14.372959160408508, 10.993917809878747, 3.3924639631979052], [6.492152427467861, 5.8057292947055696, -7.0031661028826058, 8.8000611435215799, 3.0864831568346465], [11.050513385804944, -14.562563845757701, 1.2426857592698957, 5.1493476740599737, 9.2442695585534622]], [[-5.8717882492413223, -3.7332646242052472, -3.1620868776618782, 6.493245970269105, -0.66561978146895395], [-0.38882274761656382, 1.9398508526358478, -4.2342536732245817, 3.2983839464042783, -3.2451750196495097], [5.1011848688269827, 1.4276336565852927, -7.9237741906572818, 7.9314857491821682, 6.8936128650498105], [-2.8004330510718245, -6.7864360341058898, 4.0802149690648992, -7.6534659310379949, 7.0630243947276963]], [[-9.6892653383318468, 5.9744345272426136, 7.4101560179762078, 3.5419203214068316, -0.14712259278507653], [-7.3644133319311349, -10.667278017368, -6.1557958813426108, 2.7502645431749526, 2.951026044618303], [2.4012642563400619, -5.2228306563748248, -5.4298092421788748, -2.5441854179113994, -10.659351933636255], [-8.068814232638772, 11.363283688353738, -7.6190956625597259, 9.6535267259303161, -8.9582106446454866]], [[16.883110605256441, 14.055232933035636, -6.4974958273595682, -3.6895150570037911, 8.2138757577449528], [16.994018696126464, 14.684871759024601, -1.4524706982647027, -9.1869543459661696, -10.130350450757261], [14.884790954827176, -1.8226794838400202, -4.347912266938244, 15.883637912427968, 0.15410252954844952], [2.4374702602943517, -2.8768109813289029, 17.786845620531032, 9.5166222905207523, -10.395835775653872]]], [[[-1.4108462401192896, 2.9944200555696332, 3.7927932296783977, 1.8771406945736884, -1.5593388175327212], [0.15148633274177933, 3.2700896220262643, -0.83750814788577954, 0.7811035766127209, 1.9989159515538897], [3.5552187213375346, 1.1426841570533666, 1.1466160372314713, 0.95812438337000161, 3.5834061321862349], [-0.50266307767371665, -2.8194190774454291, 3.6412171370349227, -2.3393997863685234, 2.8876674219267948]], [[12.453120288343376, -8.7623844341496024, 4.0864193841345831, -12.133158917609677, -6.5479062333218945], [-8.8739937353390683, -1.9583196126647349, -5.4874178153784987, 2.9350168515075472, -0.58201888256020951], [-9.1133286473547823, 0.82537999891235836, -5.4529017297930125, 15.69349856748334, 6.5974010027563947], [4.2044687936283101, 1.0036056030743088, -7.8695831741239912, 14.328244470339241, -8.8746428423771686]], [[5.0815450059517131, 2.5454406924395498, -4.4866672656543489, 1.0573789500399664, -4.7799006106979549], [-4.5611619161626162, 4.8369370068573518, -4.3507071625289679, 1.0465161062195563, -2.9593649636008181], [-2.360033493207359, -0.42404855148435155, 0.96719048211170888, -3.1306712390753355, 5.3995675014680993], [1.5754683258733406, -0.0093809751848968025, -1.9619655525288342, 2.1343423893371525, 5.1230506998748826]], [[2.8982796320381898, 12.248447341249456, -9.8504812013490408, 13.967952545150601, -1.4433079405118172], [-16.546666897032832, 2.4806508952618609, 8.2368216686874192, -3.3566863876823425, -7.4298470746884471], [-11.613751917165802, -9.1864396650180389, -16.622804248971718, 16.502341084005465, 15.463394145342194], [-10.660033787720231, -15.104103580227243, -1.6973786092052767, 10.951933640238174, 16.024023360056184]], [[2.8317376939218422, 1.6079597473027281, -8.5309611141891999, -7.8931924041861503, 1.2476898530574132], [6.6706846625662664, -9.6177532096982894, 3.1522529492707756, -0.29673598928950196, -8.4776917348888343], [4.0532845381270066, 2.360589327809838, -5.4869156997714983, 6.7668836163801123, 1.0677185724692515], [-2.1154359714795388, 4.469601137217686, 6.3722823919881879, 4.2982619451971997, 8.8378696922109441]]]])+(1.-msk_ref)*numpy.array([[[[19.231898792994294, -6.4223930117461494, 1.2802949612263972, 20.71389333810561, -13.168113374712705], [0.84742586452441349, 21.182130218142973, -22.19491760140771, 13.9134304598409, -16.781640155574703], [0.82285428979283726, 20.894294337508935, 6.0522156550934021, 20.68735115826037, 12.616494106541282], [-4.5849043254034845, -2.8579192531185083, -6.6451999195846421, 13.100626808708414, 11.969725446002453]], [[0.73712258179824408, 0.077205520458018373, 1.7098152664447017, -0.55477236704982436, 0.15106395225634525], [-0.42591183030072433, -2.8886902167798119, 1.6047837743076607, 1.9965131297253425, 0.54324927855478666], [1.4918291857907724, -3.111776758328904, -1.6549766571164162, -0.97705546085916761, -1.8296899322209674], [2.4365995826102855, -2.8303127675629436, -2.0687727521991417, -1.62930651314326, -1.5445878683978853]], [[0.27899638781598562, 3.3272822549037535, -0.14837313539339722, 0.55231510510928361, 2.4536315323653866], [1.309592221508695, 0.16831011606763877, -1.8657848989517323, -2.8275387270185677, -0.036875293001995109], [0.50261080912732481, 0.0054263238590691867, 2.1196131590490008, -2.2352612726268708, 0.77246622626868811], [1.429227069247119, 0.62569360579419109, 0.1262156388427482, 2.5267838350917895, -2.7699050843576343]], [[12.374104113042437, -13.066333946619682, 18.821639636782852, -15.505291554262033, 22.842896412730475], [5.4893811948706199, 20.634331071867095, -6.9659768856264819, -19.557781633203543, -20.499315485269264], [-21.299481992218219, -19.389834201571961, 20.438286323490367, -14.745029481713571, 5.7098793751347783], [-16.368189751569137, -11.347950728691872, 9.7218998396744016, 13.428937008685448, 22.574682395530871]], [[-6.1433291494138436, 19.723775976461685, 3.8810610171046394, -8.9842089901102451, -9.3765193144818983], [-14.768881352713308, -14.601107517096315, -5.6548240862008843, 16.399290561104227, 11.378276156092051], [7.9079071091824487, -19.15270631484351, 18.019847491380791, -18.595317688332436, 13.371036893285597], [2.3625725461745675, 0.2538103256170931, -21.430301307017771, -4.2840888589728401, -17.58049160645113]]], [[[-5.1242310872133814, 1.4129115989548222, 0.69976589761082031, 2.647013822153506, -6.6329020818666669], [-4.9999919967177515, -3.5992391202363283, 6.1497102201838976, -6.645544958006905, 6.2546848779171365], [6.6734256371911798, -1.1740514387450951, 3.0628477943599357, 2.8427465810215637, -4.8014178349755019], [-1.5867297259659483, -3.3525089009474041, -4.4070289544378429, 3.4932843222686358, -4.6137802311194509]], [[12.675779539225529, -14.589832211617667, 20.039337433232621, 9.4386937322545013, -4.8346005426375926], [-8.3948944398636751, 23.332908184805671, -0.23323112930561121, -14.660510282153817, 5.3847370505096341], [17.653098044212442, 20.987563896819285, 8.9555536122980115, 6.3263986259297589, 6.3367122978223671], [13.644778296138485, -21.532643591038635, -9.6857900159422048, 17.856223950666969, 3.7116708013079238]], [[-9.5988564238870548, -12.050546129657356, 3.4869568544128549, -7.8843081495538536, -4.5313037252499111], [-1.3731206924356707, 5.5764275524435707, 0.3838902157821249, -4.0237031076652174, 9.1793505306673406], [-8.8482907192424385, -8.1893846958714835, 1.6630646514602618, 2.0887466109550021, 4.5058502044757667], [-10.466138347827691, -6.0947714067900147, 13.686960817467059, 10.455732351584892, -9.5640088759794804]], [[-3.690002019462963, -10.233204651537525, 3.8180842190195206, -3.1411381891072776, -9.2547432783930201], [5.5264666049283919, -8.7532619396731324, -14.072662905484096, 10.402346939372931, -10.257948166150374], [14.831742999172601, 1.5332029204937245, 7.7390329429712938, 11.070869500716956, -4.9634002590675346], [7.3023597703659089, 7.3411768222071521, -3.7841995962195805, -5.7820004463592163, -7.7376352479467911]], [[-1.2604465988242721, -5.1050790236483365, 3.5243847145716014, 5.8766664042948547, -11.882660799444517], [7.5174770923218954, 9.1535678986342361, 5.7477167610559405, 0.3006033221375109, -0.2351106407981268], [8.9687360131127694, -5.6390993445062101, 9.3976133015965306, -3.5735470489105712, -11.898742709056418], [-3.8522324090052762, 10.095480554525293, -10.324710918438292, -7.2153780080035483, -3.7455992790371995]]], [[[0.033341354820849038, -2.2309422094364728, 0.35612387334059442, -2.224787345604144, 1.9837319678422376], [-0.35664425724154197, -2.2097406652579958, 1.6275256107239064, 1.1638260285416222, -1.4712421525556689], [0.17301772380007008, -1.3169673733921663, -2.3118785656404217, -0.91651140692787492, -0.42092023261673167], [0.34896882531197965, 0.18518592267682513, -0.44920865614753724, 1.8370553336483553, -1.5340025786857909]], [[3.1358870914567567, -1.58934616813892, 3.5079855003932776, -6.7576775878746709, -4.3156961880893867], [-2.6987792811550975, 0.43680430034352891, 7.8392769154556667, 5.9962854646384356, 1.8503123912345907], [3.5409396275513925, 3.1665517955925591, -3.8196559075082543, 4.799715591416625, 1.6834248181798137], [6.0271537351820044, -7.9426908065223358, 0.67778372408129151, 2.8085491582073328, 5.0419950506947675]], [[4.2683845684143424, 2.713825573295114, 2.298618795448109, -4.7201414155212982, 0.48385961534299249], [0.28264728653820981, -1.4101376093536917, 3.0780100150714333, -2.3976973521408405, 2.3590181368453487], [-3.7082091265030153, -1.0377910800691685, 5.7600366435849235, -5.7656424140944962, -5.0111804998422862], [2.0357214382887401, 4.9332703450693014, -2.9660345146165787, 5.5635412055515481, -5.1343309828458219]], [[7.4734191232710874, -4.6081360854044977, -5.715521224512166, -2.7319155930620371, 0.11347700367286184], [5.6802394716958178, 8.2277692626356007, 4.7480217593565968, -2.1213042385891323, -2.2761534239208312], [-1.8521171200012527, 4.0284171340154584, 4.1880616134721329, 1.9623535212877141, 8.2216558016388088], [6.2235503397522143, -8.7646048130949357, 5.8766782865716056, -7.4458536041079224, 6.9095499405025462]], [[-1.7568742581481886, -1.462602331387848, 0.67613625405967681, 0.38393481984782929, -0.85474455601307497], [-1.7684154708089193, -1.5281232102812348, 0.15114562950865776, 0.95600413801621442, 1.0541749295543987], [-1.5489270122006438, 0.18966994536044618, 0.45244832644123462, -1.6528680778412437, -0.016036071409429666], [-0.25364572059248591, 0.29936397840580337, -1.8509178749701238, -0.99030973128433197, 1.0818016118721552]]], [[[2.7518096215247949, -5.8405187507226968, -7.3977196133014793, -3.6613017615915799, 3.0414395554123792], [-0.29546915610951946, -6.3782032579109407, 1.6335323537897621, -1.5235170753607958, -3.8988204325088578], [-6.9343382757106911, -2.2287682160726003, -2.2364372202469149, -1.8687903910437147, -6.9893169021361823], [0.9804279546514183, 5.4991850449761959, -7.102075383428363, 4.5629230582757501, -5.632301217140415]], [[2.0029524660886264, -1.4093367047634358, 0.65725726511982141, -1.9514900693779773, -1.0531613470421131], [-1.4272878784355569, -0.31497496264033514, -0.88259302016532215, 0.47206636606902591, -0.093611571167855262], [-1.4657823634435989, 0.13275362848853192, -0.87704147347321404, 2.5241329826968708, 1.0611220563424251], [0.67624426198414067, 0.16141932873961351, -1.2657390807069817, 2.3045463251045555, -1.4273922804257904]], [[5.6070425051132933, 2.808672193208396, -4.9506467098804672, 1.1667256139505093, -5.2742041766844281], [-5.032845071070656, 5.3371388741494021, -4.8006265730262792, 1.1547394115344907, -3.2654015894020634], [-2.6040914907583934, -0.46790065809112608, 1.0672104915708982, -3.4544231501386631, 5.9579526412734696], [1.7383921343380981, -0.010351089391026292, -2.164858174770202, 2.3550610067335489, 5.6528404247930082]], [[1.0509917869252776, 4.4416064674846769, -3.5720414018847397, 5.065143902209182, -0.52338110329789234], [-6.0002529837123486, 0.89954871446120666, 2.9868863681972595, -1.2172220325949332, -2.6942563330636777], [-4.2114493527131547, -3.3312426214129154, -6.0278623733245356, 5.9841792878116573, 5.6074300302466202], [-3.865606284291168, -5.4771419003914597, -0.61551375438637801, 3.9714567840875992, 5.8107288057206796]], [[-1.9766017196438714, -1.1223836192380114, 5.9547579017348005, 5.5095843492433678, -0.87090902325770314], [-4.6562528738208764, 6.7133575169073216, -2.2003268924431154, 0.20712683521810818, 5.9175749568047422], [-2.8292624721083666, -1.6477320391422807, 3.8299617337398209, -4.7234014017175561, -0.74528596732966745], [1.4766107708344716, -3.1198586341205528, -4.4479629455424616, -3.0002609247998699, -6.1689853792279807]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.15717163466314688, 4.5169890011818019], [-1.4504866059380852, 1.2470437067189302]], [[-0.11979036068329751, 1.9704622394216633], [4.1759418533879575, 3.2261219087215824]], [[-1.2221978755241181, 2.8996371760000104], [-3.8685796281730358, -4.8699147569105738]], [[-1.1777340014435786, 3.8865613195758613], [-4.0065848726352744, -3.9904783423799275]], [[4.6565082071868442, 2.6313058596063632], [-4.9684669420575176, -1.3901358300398146]], [[3.277773516667807, 1.7602042214582454], [-3.1168260579833018, -1.7593516445736812]]])+(1.-msk_arg0)*numpy.array([[[3.3258297475085001, -1.3942523120997965], [-2.3559032483863254, -0.58311191804156515]], [[-0.91173886268259263, 0.21991383079525839], [-3.8685139771270638, -1.6495439200142412]], [[-4.890224625068714, 1.7392011279791326], [3.0786390108587316, -2.5821925867054074]], [[4.2243889958586003, -4.9955952884522761], [-4.6540592119480308, -4.3190570379682622]], [[-2.7658959240436798, 4.6271802478624764], [-1.4341993684293239, -3.7311388439768089]], [[3.2476055037800169, -0.49577069764444648], [1.0713520453953702, 4.4209968846425713]]])
      arg1=numpy.array([[[4.4176625252528456, 3.357163229649192], [4.1029218425928597, -3.9967363662372444]], [[2.9748485854872282, 3.6190196014770457], [-4.7448307356789918, -1.48569407855633]], [[3.1440819127972262, 1.8700744157782889], [-0.066030377918638195, -2.1983599651187271]], [[-1.0755040058409984, -0.86466063876525201], [1.3758228853111749, -3.1001941761681717]], [[-2.7149827229452383, 2.5180456270070675], [0.050197342720167626, 1.9995194227499935]], [[-3.8501587652432256, 3.2171398640965663], [-1.4395955302015517, -1.2269950567453014]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.6943312404841151, 15.164269383497375], [-5.9512331778917522, -4.9841049329308404]], [[-0.35635818503371247, 7.1311414684373551], [-19.814137256363477, -4.7930302164884999]], [[-3.8426902342945755, 5.422537297877227], [0.25544377485661041, 10.705825635133104]], [[1.2666576363677169, -3.3605565931847852], [-5.5123511597131696, 12.371257717171471]], [[-12.642339331764989, 6.6257482130998753], [-0.24940383788428444, -2.7796035924252931]], [[-12.619948435680669, 5.6628231698043816], [4.4869688614884842, 2.1587157709686231]]])+(1.-msk_ref)*numpy.array([[[14.692393440939435, -4.6807325950348062], [-9.666086896839726, 2.3305446084230748]], [[-2.712285065985045, 0.79587246428394653], [18.35544402007627, 2.4507176342837544]], [[-15.375266793194141, 3.2524355333265174], [-0.20328369736206453, 5.6765888048395352]], [[-4.543347287276557, 4.3194946131258281], [-6.4031611733913927, 13.38991547564736]], [[7.5093596472432456, 11.651450988503587], [-0.071992997226094729, -7.4604845875085868]], [[-12.503796796430773, -1.5949636748429143], [-1.5423136158234649, -5.4245413233428126]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.210962050830628, 0.91555424316146183], [-1.7714877126095807, 3.9284299833539542]], [[3.9199556545792991, 2.3798615483991901], [-0.43680898442435279, -3.869512032930297]]], [[[3.9941831773318555, -3.9629286719894719], [-3.5465283763581756, 2.2849456156833217]], [[0.12416692279820829, 4.3161500731061366], [1.5044433797995458, -2.3152558574879887]]], [[[-1.1256809841269599, -3.6662314587716374], [0.47578438950113622, -0.62131093991804942]], [[3.463457087318778, 3.8147205333017062], [-2.2494919580486981, 0.57249599968049836]]], [[[-3.9769159484743821, -1.8628598432180956], [-0.47119326654051807, 4.0578430339879805]], [[-2.945466735840514, -4.5814812404796834], [-4.6176220384726028, -4.140508745699182]]], [[[3.4940426867603538, -2.2497429541067704], [4.4297856688135813, 1.12775970496957]], [[3.7292166295569675, 1.2548963871481984], [3.2633783269691108, 2.5547614874950941]]], [[[-4.5220974122822195, 0.86221986836013009], [0.22786584396066711, -2.8466873430127193]], [[-3.049569816392105, 2.3517415478845338], [-2.0616803837128463, 2.0060083397271562]]]])+(1.-msk_arg0)*numpy.array([[[[-3.161921895217692, -1.2822436383143421], [2.2696623026418692, 3.5154596015696935]], [[0.043243542627521059, -2.0146163182366128], [3.0807481062545055, -1.6523894232376768]]], [[[-0.43669912645935227, -3.1233467314378673], [-1.8842144064868194, -1.0049203907899731]], [[-3.1514176560550289, 4.6417517943906059], [1.8148831939066525, -2.842459646912908]]], [[[1.2075399947698742, -2.7678875420768567], [2.0161038880816982, 2.3591459958973546]], [[-2.5251564308881558, -4.9234910459408123], [1.0010140141211537, -4.3877085688802939]]], [[[4.0177907545350813, 2.9913305497967224], [-1.6204720554843965, -1.6186476428630958]], [[0.027461709841070991, -1.6463912082918974], [4.8340968223270995, -1.1503960094820531]]], [[[-0.39597920933178976, -1.5035317266567914], [-0.8862209017341911, 2.862436178083386]], [[4.6161425748521836, -4.2534100401829047], [0.98655713101096332, 2.2338168365756008]]], [[[-3.3116637534569273, -4.8256725321559903], [-2.8394941253374584, -0.72360272815990978]], [[0.78120585266681974, -0.065675918337607442], [-3.1641290002223856, -2.99106007752723]]]])
      arg1=numpy.array([[[-3.6348057715135562, -1.1077861132872204], [4.1014026471947336, -4.5195708050584518]], [[3.0769097380227421, -3.3209323904618349], [-2.5683907256999672, -0.91920382523866806]], [[-2.8518075252138599, 4.8596758373215465], [-2.6671665803177227, 1.8519910702819136]], [[-3.9786649264281104, -0.52357270382530352], [-4.9718351651166106, 1.2073314440782053]], [[0.12718753551558315, -4.5701109559418427], [4.0209361684106639, -4.3591367488134161]], [[-4.9274443168943005, -3.0673756077882053], [2.5091905642497769, -0.25457588631865402]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.0364176229566162, -3.3278618471770072], [1.9624294878878359, -4.3518601825806567]], [[16.077316498577503, 9.7607704545613956], [1.9741891333915369, 17.488533613854148]]], [[[12.289741113779003, -12.193573821933938], [11.777780958739886, -7.5881499055665023]], [[-0.31890917295362198, -11.085559818495037], [-1.3828901095667328, 2.1281920406091919]]], [[[3.210225501523408, 10.455386463300743], [2.312157901433455, -3.0193697621832838]], [[-9.237616995661206, -10.174495119674111], [-4.1660390189771661, 1.0602574791804003]]], [[[15.822815999547606, 7.4116951210632056], [0.24670393258689596, -2.1245758490037598]], [[14.644375094933107, 22.778369539738961], [-5.5750002839164736, -4.9989664031634318]]], [[[0.44439867831529606, -0.28613926187638783], [-20.24461201751911, -5.1539869833511718]], [[14.994942025624123, 5.0458582706920616], [-14.225512390372295, -11.136554684593092]]], [[[22.282383194592445, -4.2485403902644752], [-0.69895013161302366, 8.7318593189566318]], [[-7.6519518083119946, 5.9009677015060369], [0.52485411098948054, -0.51068135104865242]]]])+(1.-msk_ref)*numpy.array([[[[11.492971953812349, 4.6607065770315117], [-2.5143003807181592, -3.8943773284411312]], [[0.17735918020659316, -8.2627527006973516], [-13.923659198766975, 7.4680909958523776]]], [[[-1.3436837947888058, -9.6102559731826762], [6.2573486530769005, 3.3372726756099866]], [[8.0940718806188645, -11.921832259714014], [-1.6682475742003664, 2.6127997805288987]]], [[[-3.4436716440814323, 7.8934825214404745], [9.7976113504406523, 11.46468479297625]], [[6.7350128425392679, 13.131770776226885], [1.8538690153794299, -8.1259970885657395]]], [[[-15.985443156795862, -11.901501941829135], [0.84843493556331273, 0.84747972291428531]], [[-0.13653509468206565, 8.1855857049244811], [5.83635709731404, -1.3889092753897718]]], [[[-0.050363619750319548, -0.1912304948829667], [4.0501278524000863, -13.081650938143177]], [[18.561214637763477, -17.102690269652499], [-4.3005374445938216, -9.7375130624348341]]], [[[16.318038741436183, 23.778232693764963], [8.7097950185180242, 2.2195613580867066]], [[1.9601943542482856, -0.16479339459116349], [0.80551094465817041, 0.76145177026883659]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.17329474151845492, 1.5772241956649182], [3.0044688029219326, 4.1868388317191503]], [[2.3081257105122441, -4.0976951314236647], [-2.9106259239164278, -3.604813903666634]], [[-4.7571155698569694, 0.58239584495519736], [0.05508462664372793, -4.5308694283924922]], [[-1.0698099370267444, 3.2200641699859531], [4.5595010397707476, -1.6635213404440554]], [[-2.9862394762960509, -4.3692743535557117], [-4.178205030760739, -3.1611508178774663]], [[-4.1924341362865061, 4.3707882058370231], [4.7147819649377372, -0.62528375356875188]]])+(1.-msk_arg0)*numpy.array([[[2.4745774292193996, -4.954845381206832], [-4.1039987166061529, 4.3421418574604562]], [[-1.1572705516855155, 2.376485242832719], [1.8852502950821162, -4.5644127881796779]], [[1.9190807301371402, -1.0285448466805267], [-0.11165049537166638, 0.53762698708289491]], [[3.1348619234119397, 4.1803754954845402], [2.1077488027318072, -0.32856803346572949]], [[-4.3005694621389221, 2.8009272603391171], [0.0011033916524505472, 4.4242374877574591]], [[-4.1272647035783239, 2.3171259388848489], [0.89975295637565011, 4.6920542959359288]]])
      arg1=numpy.array([[[[2.7721822423003832, -2.0734219926907826], [-3.186424168941552, -4.2422701411003763]], [[2.2096326480528994, 0.93564696635488964], [2.7077771394085817, 1.6524308180634453]]], [[[2.0454588591452607, 3.6070586077090105], [0.64200868499953501, -4.0329437190610289]], [[-2.4462277050241052, -0.93661648781168871], [1.5292841695400137, 3.0679864088627511]]], [[[-1.4586522324052331, -4.0586802425648489], [4.8526202824340547, 2.207545925434987]], [[4.4075836093128267, 4.0063136300353914], [0.73382216473575035, -0.099495729832865365]]], [[[-4.5782730690189091, 0.7433315569343657], [1.4575425693780275, 3.6587254483135219]], [[-3.4719506023641298, -1.0718237563452773], [-1.5224193563742516, 0.91210578817689125]]], [[[2.1066063901000245, 4.6068155186065756], [-2.7186385367091992, -2.4902059540704826]], [[-0.49917009273567903, -3.360090297401582], [-4.5390566149878167, -1.5096210479285546]]], [[[0.99926677962255184, -0.57914516216891343], [-1.5572743461429326, 0.53087357332155616]], [[-3.7436919152603023, 4.8839997930363328], [0.82396936123553388, -2.6266785295142689]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.48040460512149569, 0.35931312828202888], [-5.0257052969060947, -6.6910111110903401]], [[6.6387723569927148, 2.8111221209618131], [11.337026474917248, 6.9184615157974747]]], [[[4.721176182588219, 8.3255447117776669], [-2.6307558628543037, 16.525773842902026]], [[7.1200537740457488, 2.7261402301922559], [-5.5127848370151229, -11.059520062928712]]], [[[6.9389772457815608, 19.307610974976104], [2.8261458896349096, 1.2856655745211123]], [[0.24279009752001185, 0.22068629052817787], [-3.3248524120780103, 0.45080216055532851]]], [[[4.8978820236583589, -0.79522348611394567], [4.693380603883452, 11.781330723930164]], [[-15.830362381511923, -4.8869815315072804], [2.5325770884336714, -1.5173074433748037]]], [[[-6.2908311631342118, -13.75705436167622], [11.878477635031732, 10.880393010191892]], [[2.0856349926735187, 14.039146184413639], [14.348642530860861, 4.7721398103443882]]], [[[-4.1893601581466715, 2.4280279477421369], [-6.8065163453740913, 2.3203359530644136]], [[-17.650691124352488, 23.026994140967343], [-0.51521465501900143, 1.6424194103531318]]]])+(1.-msk_ref)*numpy.array([[[[6.859979606479353, -5.1308432643597213], [15.788239076045867, 21.019792614462855]], [[-9.0683295517801543, -3.8398939491169073], [11.757552457700539, 7.1750890216709093]]], [[[-2.3671493023730612, -4.1743427049053938], [1.5257241656718346, -9.5842312335234379]], [[-4.6117515027347427, -1.7657565100257615], [-6.9802842202091773, -14.003556398574586]]], [[[-2.7992713911804041, -7.7889350432945355], [-4.9911375843949486, -2.27055998541675]], [[-0.49210889337181435, -0.4473069014077104], [0.39452259948152923, -0.053491589457657113]]], [[[-14.352253919049701, 2.3302417943040572], [6.0930752406534818, 15.294846208835535]], [[-7.3179997252769713, -2.2591352391762665], [0.50021833403404947, -0.29968880513399038]]], [[[-9.0596071100108784, -19.811930137027119], [-7.6147087884772438, -6.9748857406147939]], [[-0.0005507801134775138, -0.0037074955856329821], [-20.081864435082576, -6.6789220325532108]]], [[[-4.1242385089945373, 2.3902853860679008], [-3.6084007814077319, 1.2301009270118655]], [[-3.3683978685150766, 4.3943932527225034], [3.86610898110477, -12.324518278450094]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.1728359598217555, 0.84381999171972666, 4.4123393667262203, 2.8181722755821852], [-3.7572209218226558, -4.2729386278744386, -1.5237779655830588, -3.4377789408970969], [2.5732414064966207, 1.9689019587375309, -0.46956164517169796, 0.41689762869153402]], [[2.4372956660116207, -3.2855207313192736, 4.9479293715063939, -0.27294926309362744], [-0.49685406947239663, 1.7347124622516441, 4.1346090589856015, -1.2897149493466307], [-4.8909450275704414, 3.1887532324960262, 3.2623366285351025, 4.2745445393475947]]], [[[-4.5955929930949182, -1.0357151177905477, 3.8408121041808876, 4.8507680029248093], [-2.1340695046285485, 3.778513463463538, 3.8133360915893402, 2.8573285268717372], [-3.892414545799646, 2.7388208062840285, 3.6235575941814346, 4.850684275547394]], [[4.4920905323225462, 4.7043943326760811, -3.3066773451522735, 4.6198915236697342], [-4.1495338559249149, 4.0970073893224779, 0.45331794825412697, 3.5857168291759134], [3.6810118339453499, 0.71482023466978273, -4.8041168969020829, -2.1297617263117385]]], [[[-2.9248001403567239, 0.1630402996305671, -4.62132550915726, -0.22149418456826986], [0.92281638426753432, 0.1278964668831577, 2.9652692255819213, 0.39714355219878783], [0.88023603901496461, 3.3019826040687565, -3.5614115570213567, -4.2259564307222828]], [[-2.3227781686677074, -4.6967775366726636, 0.039735234746713921, 0.74199605279772918], [-4.1333646628035661, 2.0346285695173973, 2.388729059109779, 3.143045816057322], [1.6837516511230453, 1.8225393301193549, 1.1787706838494536, 0.44442159895872457]]]])+(1.-msk_arg0)*numpy.array([[[[-3.385902169537097, 4.30675303127326, 2.1603954472604814, 0.12644583808124832], [0.23940726756039687, -2.1488914254634741, 0.068061059283150982, 2.1708787359569559], [2.5972517360764726, -4.937734416239552, 2.6622455822020257, 0.21210316823605613]], [[3.2759579292700085, -3.5149536326283961, -3.4215108186217402, 1.5295523827725299], [0.24025228606478422, 3.5694357079399666, -4.1886782090016386, -0.97599798798110893], [-3.7266046581252974, -0.30096088514504071, -4.2984001340590394, -2.7321742681401684]]], [[[3.4408884931491901, 3.5281183426330749, 3.42328500165371, 4.8979895326836242], [-0.0495167981689697, -1.9621726567103712, 1.0343095103273301, -1.0227832635804224], [4.7914914114925562, -4.3135922705586784, -3.7816328699726331, -2.441406712667554]], [[3.3749023264971498, -4.1552554126990469, -4.7487102941094941, -3.6063844802519816], [0.58126694975109938, -2.8301272209198611, 2.4095905023925956, 1.5137404359408579], [2.7034184267957357, -2.6706101892226606, 2.775340739138481, -3.078407487571817]]], [[[1.8084020996551429, 3.9793823617266657, 3.0900653338498039, 4.8857425655061775], [-0.5993248676999805, -1.4275041603805572, 0.79197165663717239, -2.4072382332613418], [-4.9035691881295858, 0.12959640616823087, -1.3038887359272811, -4.6224340144585865]], [[-0.33881108433247498, 4.5267078717199674, 2.5036619355018814, 4.1445366650362665], [0.76149225519376085, -4.7328872625904204, 2.7274571537242593, -3.0216787449556537], [3.5388403688726662, 4.0538559477902218, 4.9854386071024059, 1.5163135633042923]]]])
      arg1=numpy.array([[[[1.846467019927065, -0.12979052277041436, 0.5523103616649303, -4.3738822327017797], [0.40921464676598163, -4.5629915909607526, 1.2242269286463214, -2.4013313769285971], [4.4403157056683025, -2.9359100564636154, 3.4358032705140999, 3.6246196695342405]], [[4.3622606170862053, 2.5742498754876717, -3.5564947295338669, 1.440752806989094], [1.0388463619891199, 4.3496228436359203, -0.83128822795580781, 2.0775635635429683], [4.0954257780983774, 3.5478456477610774, 3.8333411429541737, 2.0465035754024381]]], [[[-0.036872205477733289, 2.3851426067080714, -3.5910686268332626, 3.7754119813738765], [-1.926826410706497, 2.7146164047369039, -2.7317623458193219, 0.90786550405234845], [-1.1377398951658013, 4.24946239146343, -3.4755461888597394, -0.57779890047013804]], [[-2.6393396358671728, -1.8967236803936305, -4.3007630346302168, 2.9810015559513605], [-3.4712173231254306, -2.0219255390516269, -0.65585686226305917, -0.063849151549520755], [3.6318587999560688, -2.1359366716630621, -3.1887134391141481, 2.0395296399956759]]], [[[0.75385939045563344, -1.0130816098928919, -1.4243053812608664, 0.51205684551073549], [4.2259974555719477, -4.3273612294209638, 2.0787614672545294, -0.86825766934842097], [0.031636382074372804, 0.029229745362115978, -4.8025769779669059, -2.274600614810848]], [[-3.7746306596147337, 4.5627581523955421, -2.0428956256262731, 0.099656784765040918], [-3.6125822149310904, 1.1913703137518059, -0.51293690476440634, 2.7803806246664404], [1.2996366065872635, 1.0465132830550514, -0.93956943414522431, 0.86307960719952881]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.8585369594495056, -0.10951983784943004, 2.4369807514249682, -12.326353644861664], [-1.5375098323454139, 19.497383027682439, -1.8654500187446881, 8.2552464377205599], [11.426004231742937, -5.7805190608484276, -1.6133214361889012, 1.5110953451375166]], [[10.632118896037387, -8.4577513335108048, -17.59728473186831, -0.39325241696774843], [-0.51615504251088873, 7.545344952949665, -3.4370518379341708, -2.6794647861192251], [-20.030502345174064, 11.313204277695093, 12.505649220330016, 8.7478706829918202]]], [[[0.16944964913342717, -2.4703282558539041, -13.792619848885433, 18.313647637107355], [4.1119814838016184, 10.257214633437377, -10.41712794695758, 2.5940700032915638], [4.4285553172799297, 11.638516013261526, -12.593841786571051, -2.8027200409390725]], [[-11.856152589862564, -8.9229361326963144, 14.221235693480081, 13.77190382038598], [14.40393380358203, -8.2838438741543499, -0.29731168714947953, -0.22894497723971993], [13.368915221856847, -1.5268107528779846, 15.318952112227029, -4.3437121669411498]]], [[[-2.2048880510138709, -0.16517312922715438, 6.582178791250799, -0.11341761344900089], [3.899819691874705, -0.55345421217009894, 6.1640874061753763, -0.34482293502887251], [0.027847483645909946, 0.09651611070706638, 17.10395315281604, 9.6123630954847616]], [[8.7676296909368912, -21.43025999544145, -0.081174937247294965, 0.073944940930173231], [14.932119668668806, 2.4239960772343299, -1.2252672899005626, 8.7388636894046989], [2.1882652822012565, 1.9073116178601601, -1.1075369044114103, 0.38357121906028252]]]])+(1.-msk_ref)*numpy.array([[[[-6.2519566887497477, -0.55897572737202317, 1.1932087908157054, -0.55305920458265811], [0.097968960427936652, 9.8053735042774974, 0.083322181566627129, -5.2129992241605292], [11.532617675274626, 14.496744128784201, 9.1469520782414335, 0.76879331555893915]], [[14.290582258085834, -9.0483689511385883, 12.168585193471326, 2.2037068889163796], [0.24958521333797043, 15.525699094145432, 3.4819988858380788, -2.0276978579208], [-15.262032781667834, -1.0677627665081542, -16.477234082768252, -5.5914044083713943]]], [[[-0.12687314754536499, 8.4150653805224138, -12.293251370147491, 18.491928366337589], [0.095410274485593929, -5.3265460828321674, -2.8254877742350213, -0.92854964312674615], [-5.4514709361993781, -18.330448125846448, 13.143239708900104, 1.410642114179727]], [[-8.9075134775042617, 7.8813713393500899, 20.423077695074099, -10.750637746989996], [-2.0177039053362953, 5.722306506743073, -1.5803464662380762, -0.09665104250102545], [9.8184340033214834, 5.7042542388777102, -8.8497663130118678, -6.2785033148873408]]], [[[1.3632809045447136, -4.0314390893974288, -4.401196683449931, 2.5017779260706212], [-2.5327453659611119, 6.1773261584679489, 1.6463201629750888, 2.090103057977903], [-0.15513118836378964, 0.0037880699521427447, 6.2620260249947304, 10.514191251210077]], [[1.2788867067386731, 20.654273245203356, -5.114720016083802, 0.41303119837833968], [-2.7509533779207476, -5.6386213829842751, -1.3990134303088593, -8.4014170364411065], [4.5992064882556916, 4.2424140969541924, -4.684165731040963, 1.3086993146079866]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.244476539344)+(1.-msk_arg0)*(3.64214849659)
      arg1=Data(1.00355644593,self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.245346006937)+(1.-msk_ref)*(3.6551016008)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.42843838027687475, -3.5202322782183915])+(1.-msk_arg0)*numpy.array([0.61612642085628799, -2.278232520434611])
      arg1=Data(1.16759179775,self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.50024113865355879, -4.1101943342306786])+(1.-msk_ref)*numpy.array([0.71938415537028211, -2.6600456042319962])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.54916602757)+(1.-msk_arg0)*(-0.667085736821)
      arg1=Data(numpy.array([0.067997146336198355, -1.3411412447221558]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.30933030808415435, 6.1010741886593243])+(1.-msk_ref)*numpy.array([-0.045359926465376288, 0.89465619541588215])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-3.6831650057850363, -0.83648107489310064, -2.6914272635716063, -0.39807856562135946, 0.066859348456391743], [0.40574299997446683, -0.90663541681981208, -1.1722125430383223, -4.6618861369088629, -2.4692624341238711], [-0.15374503606008005, 0.46572360780890332, -3.5424225745258084, -4.2717491263300733, 1.1277777009852832], [0.33111096403225204, -3.8271803978371288, 3.2609030884959243, 1.6226273301850878, -2.3103674390928686]])+(1.-msk_arg0)*numpy.array([[-3.0058016517304029, 1.4591368171697292, 1.7567256194016156, -3.7972719622753468, 3.0899497589411258], [-4.557295268721572, 3.1485553176988326, 3.2493687566635572, -2.4520436849038951, -1.4792303598071435], [2.4752749128219858, -4.1996214560252954, -3.5650722527107237, 1.6764483475799521, -0.8459296031754473], [4.4405815577710843, 4.9375892824776528, -2.8985551216036254, -4.2435068158098668, -3.9431304323174308]])
      arg1=Data(-1.98204105489,self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.3001842534072825, 1.6579398320782721, 5.3345193326544171, 0.78900806013403302, -0.13251797354389505], [-0.80419928368441607, 1.7969886179559418, 2.3233733853612502, 9.2400497165849931, 4.8941795197359381], [0.30472897345692174, -0.9230833109096428, 7.0212269764861857, 8.466782144585018, -2.2353017041444869], [-0.65627552443677484, 7.5856286729908895, -6.4632437974228747, -3.2161139852165572, 4.579243116167639]])+(1.-msk_ref)*numpy.array([[5.9576222765916889, -2.8920690763347703, -3.48190229983449, 7.5263489258198479, -6.1244072797747906], [9.0327463218709898, -6.2405659032774485, -6.4403822781903761, 4.8600512518680565, 2.9318953027803469], [-4.9060964993572647, 8.3238221408472377, 7.0661195685287783, -3.322789451309232, 1.676667203042191], [-8.8014149550983323, -9.7865046700651863, 5.7450552508857093, 8.410804725648962, 7.8154464016469856]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.82351374714)+(1.-msk_arg0)*(0.353830927212)
      arg1=Data(numpy.array([[-3.2659959171270367, 2.787303069714171, -0.62491753523763727, -4.4983750487019902, 4.4610174680279187], [-1.5024205001523852, 2.1520264242813179, 0.27925582950489769, 4.9474896180944974, 3.5636542958622091], [-1.8636657580869431, 0.32185624116958778, 4.0289142337184618, -4.2717808936059498, -3.9783098492973865], [-0.44295064360686087, -1.958409313813533, 1.8469452471949968, -3.8113552792286534, -1.746668247867488]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[12.487580287229106, -10.657291604489776, 2.3893807868081853, 17.199598838490914, -17.056761615223813], [5.7445254363133529, -8.2283026174420755, -1.0677385030801712, -18.916794568602757, -13.625681190273601], [7.1257516461142494, -1.2306217627138165, -15.404608958659171, 16.33321297146027, 15.211122399159775], [1.6936278751340914, 7.488004933887523, -7.0618205428597358, 14.572769305354598, 6.6784100574093408]])+(1.-msk_ref)*numpy.array([[-1.1556103636281732, 0.98623402957825201, -0.22111515092426798, -1.5916642144302497, 1.5784459470219401], [-0.53160283863146862, 0.76145350508851795, 0.098809349083117518, 1.7505748389428883, 1.260931103768506], [-0.65942258319744751, 0.11388269224205433, 1.4255544589748537, -1.5114881944317629, -1.4076490627141443], [-0.15672963693663655, -0.69294578336756329, 0.65350634932508922, -1.3485753723844178, -0.61802524567498407]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-2.1526121582694446, 1.6264359969521793], [-0.10932217756408669, 0.90984506049895852]], [[0.16333218401049532, 1.4106385756380879], [2.5549740807024204, 0.88964769441716562]], [[4.0676239173910798, -3.9633913442123747], [2.095522147633532, 4.1354091670988229]], [[-4.3662145472677754, 2.3430068187625608], [-2.8456754862988163, -0.92495229448953697]], [[2.0409495365575649, -4.2321206610076425], [1.8508319949153185, -4.1557104722361]], [[2.8132421290201428, -0.13568256963493752], [3.6391381873746589, 0.63949438348796406]]])+(1.-msk_arg0)*numpy.array([[[2.170013318365501, 2.3482699592992624], [4.3330250362209615, -3.0683996151355366]], [[2.6910931034243015, 3.0416578726782788], [0.38028160798374344, 0.097227799870079679]], [[-0.46847508834841634, 2.3007068989200476], [3.9020425219493866, 4.2068087478746392]], [[-1.3394821170229498, -0.75871070272714203], [-0.14967482721960046, 1.5824639648838783]], [[2.5094454486427207, 1.727635391700975], [4.0749338606770191, 2.3260739116028581]], [[-1.3590007205125021, 2.330830541482567], [-0.89010785675171178, 3.2865421896046101]]])
      arg1=Data(1.1714187357,self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.5216102129006996, 1.9052375992529309], [-0.12806204702651275, 1.0658095504560707]], [[0.19133038049332082, 1.6524484568090299], [2.9929445073725014, 1.0421499774159892]], [[4.7648908666289458, -4.6427908775367897], [2.4547339048202605, 4.8442957781409586]], [[-5.1146655247721169, 2.7446420853803071], [-3.3334775803835597, -1.0835064473972962]], [[2.3908065257495941, -4.9575854340636578], [2.1680992754839119, -4.8680771073380056]], [[3.2954845380055451, -0.15894110417880628], [4.2629546545059744, 0.74911570219517531]]])+(1.-msk_ref)*numpy.array([[[2.5419942578602024, 2.7508074268136418], [5.0757867097030296, -3.5943807977962092]], [[3.1523968808746092, 3.5630550196564106], [0.44546900043573873, 0.11389446639907438]], [[-0.54878049570184506, 2.6950911667580169], [4.5709257177245606, 4.9279345847831495]], [[-1.5690944480209223, -0.88876793215359817], [-0.17533189686827455, 1.8537279370411532]], [[2.9396114147668029, 2.0237844663035585], [4.7734538711510277, 2.7248065606834979]], [[-1.5919589058433581, 2.7303785660433975], [-1.0426890201961418, 3.8499170965839511]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.63651413685)+(1.-msk_arg0)*(4.58181989281)
      arg1=Data(numpy.array([[[-2.3764820686422503, -1.8353813176319811], [-3.7628334943211152, -1.0332113232596516]], [[-4.4070491761298438, 2.4113403525137347], [4.0945778389207526, 0.075547115500381068]], [[-1.262159623063499, -4.0520133533247558], [-4.5432897526272544, -3.9116485827746916]], [[-1.9816843038076817, 4.5147363067689419], [-2.5065546729288668, -1.5428618958694642]], [[-0.15067628712087355, -4.6332359741324964], [4.9833698245255604, -1.6699796722602667]], [[-3.2741661144682155, 4.4991506548941711], [0.2778545744898695, 0.043821566537466161]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.2656285699528107, 4.8390087904524934], [9.9207637024013664, 2.7240762601306039]], [[11.619247454672434, -6.3575329281664121], [-10.795412356759323, -0.19918103801521653]], [[3.3277016891718927, 10.683190488757605], [11.978447660620851, 10.313116786886241]], [[5.2247386817685282, -11.903166096959545], [6.6085668299717542, 4.0677771996715704]], [[0.39726016108269563, 12.215592145175927], [-13.138724991527994, 4.4029250141712319]], [[8.6323852472003111, -11.862074305459643], [-0.73256751363179906, -0.11553617967507127]]])+(1.-msk_ref)*numpy.array([[[-10.888612817018307, -8.4093866320234358], [-17.240625357623312, -4.7339881943906521]], [[-20.192305583796593, 11.048327195489996], [18.76061819503812, 0.34614327664428274]], [[-5.7829880688576214, -18.565595388207033], [-20.816535367400732, -17.922469290250625]], [[-9.079720564461196, 20.68570862115876], [-11.484582062848713, -7.0691153263577968]], [[-0.69037160970561251, -21.22865275437681], [22.83290299525494, -7.6515460829553223]], [[-15.001639435644517, 20.614297971356478], [1.2730796167067588, 0.20078252529558829]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.8553599919200767, 4.0495934569778917, 0.98187339984612443, 2.5179806236202475], [-2.3484118086544612, -1.0148538662440076, 3.36824574828902, -4.2694164875560192], [-2.3084309691463476, 3.1629317968103425, -2.1017039374549129, 3.0700143717163346]], [[-4.656054949194135, -4.9405646167266459, 2.0752706342348537, -4.2977832573640598], [-2.5939705917622047, 3.9943533256903798, -2.4362170762572832, 2.3486291479905326], [4.5120900163306548, 0.6256529689318322, 1.0096393197046796, -1.9401818944949731]]], [[[3.1500698173961901, 0.37377418416340724, 0.13467539303428921, 3.8454381466081049], [-1.4178479195186808, 0.63171620334595868, -1.1011182566956759, 3.0599398301915048], [-4.8787908726029325, 3.1003939174686383, 2.7439912778055806, -4.6542875535850117]], [[-0.13345679211296169, 2.4123362848861998, -1.3113350080824535, 4.2614763129682025], [-3.3793050110380465, 4.5310582522937342, 3.4148520205948021, -4.3443520391528185], [2.9398083157542567, -2.9496802698035984, -3.2015413536676163, 2.3450662633899864]]], [[[-3.8547977780295173, 2.496823025493339, -1.8500337308351558, -0.10995636714687684], [4.8093721503048918, -0.51227870664727426, 4.4151830929249805, 2.4747443667733107], [-4.9344923495132775, 3.7657285758940144, 0.055988170265948156, 1.4788979767126564]], [[-4.0141603796001171, 4.3957604271773594, 1.5242862613980055, -4.0013150987254242], [-1.9790922470825612, 3.8596444867324884, 1.2304015929088576, 1.2771179190685666], [-2.1460796226852397, 0.68241373607105515, 0.79292197134832954, -3.8172522460366833]]]])+(1.-msk_arg0)*numpy.array([[[[-3.6764724043696031, 1.4558240539376381, 0.15490670487125868, -2.7496862917441134], [2.3604507355334174, -1.533671984314835, 1.1202167729296422, -2.3388373978498658], [1.7121850989920961, -2.0818101218518379, -4.652314158380138, 2.7169908302370338]], [[-1.8738619141888879, -4.6205760762181054, 4.438629329847263, -3.2355621787375788], [2.5217709400254815, 2.3908142954371536, -3.1455008591875289, 0.46002706415353511], [1.2192905383864039, -0.45047211414511246, -2.8869299626430855, 2.0825593732837042]]], [[[-4.4069017051803812, -4.9304441981065565, -2.6782519515147394, 0.71902389070798201], [0.89103371168096679, 2.47068735389563, 0.94925775574835214, -4.672366424697322], [-0.426576979697324, 1.8672847389785527, -3.2443100343366451, 0.20629392234387112]], [[2.7942359440484008, -0.12498776220640639, 1.9808366681347653, 1.7565412452412454], [-3.3788947337259492, 4.816799614704447, -2.9773805959346324, -1.1533902949411545], [3.1594547644002535, -3.0338085295120241, 3.6145197492135122, 0.72526121194955628]]], [[[-0.093129205705537288, 1.6109678820887696, 3.4077912259628818, -0.028748273625827103], [3.9801065345127959, 3.7519390629063825, -2.0370969946664621, 2.1282710704746961], [2.8403580406761995, 3.8292952718215894, 0.72991510028167816, 3.7797313379525619]], [[-4.7015271519891773, -4.7431534476509931, -2.1712513205552799, -4.4421989389429566], [0.51631971775431751, -0.55593848069715435, 3.4899980101398036, 1.8227163870100327], [4.4524231032538548, 0.0588505534682362, -1.0764391689404453, -2.3798324295662789]]]])
      arg1=Data(-4.94786309803,self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[14.127930335599054, -20.036834027785456, -4.858175162031241, -12.458623409153256], [11.619620127008169, 5.0213579946769888, -16.665618843039724, 21.124488288879053], [11.421800406577903, -15.649753519008625, 10.398943355107555, -15.190010820222458]], [[23.037522465495542, 24.445237350511086, -10.268154989546112, 21.264843182422432], [12.83461136834296, -19.763513420657905, 12.054068570392404, -11.620695492288865], [-22.325203686770642, -3.0956452371477576, -4.9955571322821122, 9.5997543992284164]]], [[[-15.586114205697781, -1.8493834928166204, -0.66635540740640786, -19.026701501340895], [7.0153173995983709, -3.1256452909600907, 5.4481823888664414, -15.140163367982229], [24.139589321534512, -15.340324653585052, -13.576893184757413, 23.028777633981548]], [[0.66032593687655183, -11.93590968401578, 6.4883060956398291, -21.085201392044116], [16.720338561086997, -22.419055921526997, -16.89622029791802, 21.495259139355024], [-14.545769080788281, 14.594614157934419, 15.840788320613839, -11.6030668270513]]], [[[19.07301167626202, -12.353938510138278, 9.1537136269012649, 0.5440490513989491], [-23.796114987163971, 2.5346849085241563, -21.845721496508546, -12.244696329403554], [24.415192603644712, -18.63230945784521, -0.27702180158484002, -7.3173847247207737]], [[19.861516011778491, -21.749620805390613, -7.5419597435980776, 19.797959320554828], [9.7922774969276958, -19.096992527400328, -6.0878586373052359, -6.3190046235861725], [10.618508170508273, -3.3764897422915237, -3.9232693616478427, 18.88724152401895]]]])+(1.-msk_ref)*numpy.array([[[[18.190682140488562, -7.2032181136955762, -0.76645716866919078, 13.60507133406662], [-11.67918708905238, 7.5883990156665488, -5.5426792325675196, 11.572247253102777], [-8.4716574682917205, 10.300511479006181, 23.019013544669534, -13.443298666602836]], [[9.2716122160101762, 22.861977859137919, -21.961730266963865, 16.00911870554188], [-12.477377375824553, -11.829421826624756, 15.563507625980726, -2.2761509348181757], [-6.0328826606534403, 2.2288743502680251, 14.284134228745167, -10.304218672517054]]], [[[21.804746323686604, 24.395162904684177, 13.251623998113919, -3.5576317754325721], [-4.4087128211227355, -12.224622785097866, -4.6967974201815474, 23.118229413212102], [2.1106444963114601, -9.2390692534977124, 16.052401897447353, -1.0207140857121286]], [[-13.825496914732859, 0.61842233632583454, -9.8009086534793148, -8.6911256074884982], [16.718308565114512, -23.832865064178428, 14.731671579401427, 5.7068172779598099], [-15.632549638656108, 15.010869269646824, -17.884148884217055, -3.5884931870342869]]], [[[0.46079056025883153, -7.9708485358909327, -16.861284452716006, 0.14224252220516279], [-19.693022248325068, -18.564080835393966, 10.079277047008349, -10.530393892196468], [-14.053702734641099, -18.94682876688865, -3.6115199893751231, -18.701593207505127]], [[23.262512699691051, 23.46847391190364, 10.743054285514086, 21.979392204082824], [-2.5546792782594054, 2.7507074934136786, -17.268032366552323, -9.0185511494528701], [-22.029979969384865, -0.29118448180384843, 5.3260736412694092, 11.775085057734792]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.30737531309)+(1.-msk_arg0)*(-2.16805392193)
      arg1=Data(numpy.array([[[[4.3079960058483735, 3.6147983154581826, -3.4579717393972476, -1.4074595926624021], [0.97954194513739079, 0.41630377771151661, 4.1184190850798696, 2.859610526541708], [-2.9074734714850301, -2.735194624408873, 0.45000678441488873, 0.68285774986479808]], [[1.9149332994904436, 3.5438085187336448, 4.8519741413897783, 0.23867388332909112], [-3.2381067767415694, -4.3350617347375353, 4.5943592261913384, 4.6837078162820784], [-1.8135827297263885, 2.5020623858272906, 2.3965257131208357, 1.528002043851437]]], [[[-1.9180808041392314, 1.4399479434088462, -2.3330805208060101, -4.7517006922779794], [-3.9356990870399144, 1.8345644101918701, 2.0496793592070777, -3.9646600697797862], [-2.5237183775568726, -0.30095451240504367, -4.9444614353462395, 0.55312697697970759]], [[-2.773633243199245, -3.3395605539481235, -1.1459645674463226, -1.4719037551438885], [-1.470508991975227, -4.986353164462777, 2.6254205709631044, -2.5013381204988105], [1.4916006899197809, 2.7087421282519211, 3.7403127500349669, 4.4042079881717466]]], [[[-0.32546911897876285, 4.4944950014775227, 0.82808906071079402, -3.2896509753066896], [4.4433095558367377, 1.2836329023674153, 4.2482053427717332, -1.8403573995804479], [0.32551230573392154, -0.93841515979169721, -0.84387837842354596, -4.7426200993586658]], [[3.3288386988368455, -4.9034612194815175, -1.8758508354608683, 3.8488600247079532], [-3.3105307965356601, -4.299302227041105, -4.4617675961931704, -2.371234132355835], [3.2759580798435444, 1.8158880577386061, 2.304869633750922, 2.9252025957006911]]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-18.556155644482981, -15.570293025805045, 14.894782103643692, 6.0624567036061858], [-4.2192547926212685, -1.7931766148608261, -17.739576696033048, -12.317415787078961], [12.523579454539627, 11.781509801676126, -1.9383481139118488, -2.9413246141200351]], [[-8.2483364204397258, -15.264513327912471, -20.899273636374925, -1.028057992931126], [13.947741191287101, 18.672737896930951, -19.789629510365309, -20.174487421581588], [7.8117814782703974, -10.777321752524335, -10.322735693882858, -6.5816782820372302]]], [[[8.2618939042617523, -6.2023962235744392, 10.049453438771712, 20.467358257112345], [16.952533087267831, -7.9021574507345624, -8.8287382715993488, 17.077278909364392], [10.870602236680824, 1.2963240370966191, 21.297651123137513, -2.3825254856466693]], [[11.947079359323064, 14.384740686646376, 4.9360994874945154, 6.3400418981517221], [6.334034129711422, 21.478094522956756, -11.308671753845966, 10.774202069928313], [-6.4248839887489515, -11.667568972760055, -16.110930802737577, -18.970576761966161]]], [[[1.4019176482623787, -19.359476814172115, -3.5668903771458242, 14.169761399719524], [-19.139001889229473, -5.5290886747278796, -18.298614818393357, 7.9271100302159159], [-1.4021036698254017, 4.042106292716463, 3.6349008944722714, 20.428244735343469]], [[-14.338557632629525, 21.121047805490438, 8.0799935797039932, -16.57848465396723], [14.25969862622293, 18.518708276271084, 19.218507596588797, 10.213795363266664], [-14.110780959836845, -7.8217113912387983, -9.9279385603102455, -12.599945446508876]]]])+(1.-msk_ref)*numpy.array([[[[-9.3399676361587929, -7.8370776648322309, 7.4970691915397225, 3.0514482898364026], [-2.123699755854711, -0.9025690379837048, -8.9289546495783245, -6.1997898172746373], [6.3035592627743542, 5.9300494327044948, -0.97563897384794374, -1.480472422717912]], [[-4.151678650203702, -7.6831679576262459, -10.519341566366087, -0.51745784881503232], [7.0203900969580424, 9.3986475958269615, -9.9608185391212221, -10.154531100286791], [3.9319451499364213, -5.4246061685182676, -5.195796971349095, -3.3127908238964174]]], [[[4.1585026100018148, -3.1218847860894225, 5.0582443733230331, 10.301943321753326], [8.5328078412118913, -3.9774345645583904, -4.4438153734376042, 8.5955968134241534], [5.471557526320975, 0.65248561094371449, 10.719859006757524, -1.1992091117687658]], [[6.0133864309267153, 7.2403473565258061, 2.4845129748502557, 3.1911667090501892], [3.1881427872922026, 10.810682534365261, -5.6920533656047221, 5.4230359222323354], [-3.2338707257411556, -5.8726989946664512, -8.1091997269758505, -9.5485604017720984]]], [[[0.70563459987055366, -9.74430751506946, -1.7953417357852979, 7.1321406988101348], [-9.6333347089019785, -2.7829853483021574, -9.2103382545804031, 3.9899940779220229], [-0.70572823108445104, 2.0345346675894116, 1.8295738279771037, 10.282256106661116]], [[-7.2171017965013835, 10.630968327951843, 4.066945760785508, -8.3445360715459458], [7.1774092771149007, 9.3211190549192615, 9.6733527356879723, 5.1409634604796057], [-7.1024537630986204, -3.9369432253746557, -4.9970816490019905, -6.3419969600625858]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.46901113224895852, 4.6669893328597354])+(1.-msk_arg0)*numpy.array([-0.60929061871013879, -1.4663010232893283])
      arg1=Data(numpy.array([-2.8455916429268822, -1.3387247930404067]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.334614158367311, -6.2478143287544352])+(1.-msk_ref)*numpy.array([1.7337922927153204, 1.9629735339379426])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.9544031033834006, -4.8197505532824341], [-0.13979045243376831, 4.5065917599033494]])+(1.-msk_arg0)*numpy.array([[0.7918830361827256, 4.0868196756342776], [4.7545353156726637, 3.0581712228533551]])
      arg1=Data(numpy.array([3.3102939842473074, -3.1679277115801985]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[13.090236804418954, -15.954791262103473], [0.44284604807926825, -14.276556920976796]])+(1.-msk_ref)*numpy.array([[2.6213656509031695, 13.528574586955681], [-15.062024182206137, -9.6880653636342462]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.5558487345190803, -1.3114411582373964])+(1.-msk_arg0)*numpy.array([-4.2396861746879058, 2.6252447960936749])
      arg1=Data(numpy.array([[-1.8226268606330418, 2.465075160589306], [-0.6635695890934814, -4.7197151299859685]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.4809854158824862, -8.7654343902759031], [0.87023247049186847, 6.1896286766193622]])+(1.-msk_ref)*numpy.array([[7.7273659026407273, -10.451145077917049], [-1.7420326106136803, -12.390407584040247]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.8753805861367283, 2.1915346606783892, 1.4832851321328171, 4.1751339036893071, 0.13794618897489208], [3.4762834983359348, 0.48144856334173269, 2.56702525203589, -1.6058742419267, 1.5752804106231286], [4.9976456194839489, -3.9071306635593817, -0.7441917116176473, -1.8706004059065551, -0.053945853530231958], [-1.3979035376180771, -3.8188256368420648, 3.6263872978121121, 0.97365843508443639, 1.8745358346934289]], [[1.6207859700841691, 0.35414619006553849, 2.1852625975571627, 4.3461640888897151, -2.8771146524403788], [1.6064751623828535, 3.318254889503379, -0.13136448488976704, -4.556647874619129, 1.2148307661328595], [-4.2221026914490007, 0.33650280651794517, -0.1591370041241138, 4.2103634029960055, 0.93423129767333535], [3.0380905923345285, 2.3279120380175486, 3.6670589069129758, -0.65120763725277175, 0.94890584014216817]]])+(1.-msk_arg0)*numpy.array([[[2.9272038143530619, -4.8406869278124223, 4.2718847355280758, -3.4941860915082223, 0.52340868468243507], [4.4000991371561344, -0.63216338855877829, -4.9546109131658458, -2.3272132907701226, 3.2994925875994188], [-0.051440703373366148, -3.7652637021917581, -0.15275362873453346, -2.9377357590150699, -3.6708238079135427], [0.23090038190797202, 3.4812205097430109, -4.3202817479502826, 4.1708925993758541, -3.6159387757358594]], [[0.91737471326794573, -3.3841381215211697, 0.29848327348042325, -4.9829748841406287, 1.6583301884579784], [0.72041344570924704, 0.17361454984564872, 0.51705973702298902, 4.0997066934167172, -4.6839529755907048], [-3.6128955894139314, -1.8081581638805488, 1.2547700496603653, 2.579005791227643, 4.2523340382112202], [-1.3315228094038103, 3.403395544451616, -2.4628549080539885, 4.2637117110210845, 0.78645510168547261]]])
      arg1=Data(numpy.array([-4.9542141298318469, 2.3870331510034903]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-24.15367938814645, -10.857331981949118, -7.348512160181901, -20.684507379597562, -0.68341495857586454], [-17.22225282675717, -2.385199275294855, -12.717592775271365, 7.9558448600862635, -7.8042764687564175], [-24.759406543939615, 19.356761940505169, 3.6868850929998951, 9.2673549622114439, 0.26725930980531437], [6.9255134582094025, 18.919279929407057, -17.965899191063496, -4.823712376725279, -9.2868519191143211]], [[3.8688698412722631, 0.84535869598802338, 5.2162942640169465, 10.37443775988063, -6.8677680546130695], [3.8347094688715866, 7.9207844247239896, -0.31357138029637099, -10.876869534165456, 2.8998413116181037], [-10.078299091429825, 0.8032433545640485, -0.37986530439563881, 10.050277020723334, 2.2300410782512614], [7.25202295965435, 5.5568032073679863, 8.7533911774838948, -1.5544542183090218, 2.2650696976001741]]])+(1.-msk_ref)*numpy.array([[[-14.501994497965617, 23.981799575860617, -21.163831717766175, 17.310946106811951, -2.5930787013304215], [-21.799033317959839, 3.1318727919602796, 24.546203393825305, 11.529512968265813, -16.346392798760483], [0.2548482595008193, 18.653922635941381, 0.75677418585971368, 14.554172007024745, 18.186047177288419], [-1.1439299346320446, -17.246711838429249, 21.40360088054992, -20.663495049838936, 17.914134975357463]], [[2.1898038524629078, -8.0780498836457095, 0.71248946881781128, -11.894526239061458, 3.9584891351590601], [1.7196507773366259, 0.41442368597811141, 1.2342387333230216, 9.7861357865766063, -11.180751030476454], [-8.6241015430453487, -4.316133479440472, 2.9951777054255877, 6.15617232029037, 10.150462318350726], [-3.1783890873641973, 8.1240179905835799, -5.878916311636524, 10.177621200529142, 1.8772943994990441]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.1661963904739991, 2.0309368993850683])+(1.-msk_arg0)*numpy.array([-2.3569684504103829, -3.0510397601986581])
      arg1=Data(numpy.array([[[1.9242176209818584, 2.4453184889306581, 2.1115125053443791, -3.4668139127506725, 3.3653361874524528], [4.6688431868482496, 4.4649679833466269, 0.18659486814657367, 4.3779845798910539, 3.5642862624432805], [3.95931098040389, -3.6280776381062694, -1.1955275554798339, -0.74372054903101059, 2.141364180993345], [-0.76874755666505568, -1.6608292220980925, -3.6885833684357197, 0.78209746616196529, 0.71282112758388028]], [[-1.1958742930086519, -0.49175827697774199, -0.39375808432889681, 3.4772424162627065, -3.2582656528806475], [-3.0136472875170162, -3.3128548183577022, -3.2406169358877088, -4.2053237857712835, 0.084678759897963474], [0.87478197963857518, -0.61324971221060665, 4.5075223986648716, -3.236453106457394, -3.2671331554430783], [-0.80299397954630258, -3.9167309561770178, -2.9603304443425227, -4.4034234778425692, 4.8893967463792301]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.244015644075509, 2.8517215953502673, 2.4624382621733258, -4.0429858714948761, 3.9246429145385804], [5.4447880721915523, 5.2070295457608076, 0.217606261713506, 5.1055898146197745, 4.1566577738774146], [4.6173341741110869, -4.2310510459189636, -1.394219919912786, -0.86732421980130547, 2.4972511785647504], [-0.89651062576849405, -1.9368530440045353, -4.3016126102321612, 0.91207924203694457, 0.83128942604192724]], [[-2.4287452286973021, -0.99873003029211893, -0.7996978228947339, 7.062059931294824, -6.6173319424342871], [-6.1205274779500298, -6.7281990929082749, -6.5814885118665236, -8.5407472503846069, 0.1719772180709426], [1.7766270013650998, -1.2454714690657949, 9.1544935642531797, -6.5730320370337525, -6.6353412805937193], [-1.6308301030446446, -7.9546334238636662, -6.0122443337882245, -8.943075224769002, 9.9300562679548747]]])+(1.-msk_ref)*numpy.array([[[-4.5353202243779647, -5.7635385296147525, -4.976768357743687, 8.1711710157971087, -7.9319912188497934], [-11.004316091314793, -10.523788668840472, -0.43979821722995949, -10.318771531186369, -8.4009102688099535], [-9.3319710661753703, 8.5512645286558957, 2.8178207298622171, 1.7529258699879802, -5.0471278154401835], [1.8119137373896044, 3.9145220780048229, 8.6938746261114481, -1.8433790528896543, -1.6800969085011601]], [[3.6486600161688569, 1.5003740554658751, 1.2013715711871202, -10.609204867866771, 9.9410980562284941], [9.1947576974292531, 10.107651770575053, 9.887251118966546, 12.830610074897329, -0.25835826329300221], [-2.6689946013825856, 1.8710492548849453, -13.752630058312549, 9.8745471098199697, 9.9681531591201349], [2.4499665587959174, 11.950101877296989, 9.0320858890155975, 13.435020111889934, -14.917743876588984]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.6350766985658831, -2.5796065853861236], [3.331109228311206, 4.2270084159090935]], [[1.2233826136763346, 3.2816772423487226], [2.4424629019487041, 2.3622748467340795]], [[3.811315897660398, 4.2596164220770074], [-0.52926805700642099, 1.5537258651275518]], [[-4.8538122729001394, -1.472410892761177], [-1.5532873738026631, 2.1135842337560637]], [[1.3906667115461993, -0.17690908462478916], [-3.188860597446209, -2.3517495287193926]], [[0.51525865423212736, -2.377976488464264], [2.9279151730729067, 0.13875994658254776]]], [[[-1.4112601399425495, -1.8070335079054956], [2.1447733989286846, 4.4853690233738597]], [[-0.29715655667170804, -2.9637940875935422], [2.2057653071833849, 1.9031675748028825]], [[-0.19725623275606452, -2.218685275089384], [2.0365441369738182, 4.9779260345455967]], [[-4.5593892534707301, 0.20803366743242702], [4.5436286384707518, -4.8567798663618174]], [[4.8189853814593739, 2.7136284613212354], [3.9006524787988788, 4.2075193969237183]], [[-2.0980033474076287, 0.41467259116046584], [4.9885338193553412, -3.6667243196692665]]]])+(1.-msk_arg0)*numpy.array([[[[-4.5722301438765491, 0.21265386241792683], [-0.96454851536155672, -2.6237701867107619]], [[-0.16197492509525091, -2.6134488742561879], [-0.53951635915248808, 2.2691916983611451]], [[-4.8422318856970392, 0.0052870551323502823], [-2.2503380491668858, 3.1094944704443197]], [[-4.6261065467116591, 2.2788981757031435], [0.91315801380740425, -2.7959475286818725]], [[-4.5177531751156241, 0.402894035531979], [-3.0170709845769315, -0.85699074592533098]], [[-2.9785986177781396, 1.1192035523703261], [0.80979195503613965, -4.2777151830102582]]], [[[-0.56272395493662586, -0.82895035582597743], [-2.1627795178573415, -1.6827180570722646]], [[3.7789757986250958, -0.74983037363239013], [-2.6369542878751915, 3.4478880514050942]], [[3.9523107445353585, 3.0473322906125038], [-1.3295819896045136, -3.0386646047422836]], [[2.5396564674418993, 1.7942724878382297], [-3.6216346464865765, -0.93717029802915608]], [[1.4817318982819794, -4.4344149277070688], [2.3976349719832379, 3.4932105606072259]], [[3.6578749978327387, -0.93370210157250533], [3.5797522034264677, 0.79978187392536615]]]])
      arg1=Data(numpy.array([0.83609806253682883, 3.2155888732177971]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.039280584843707, -2.1568040681485825], [2.7851339718895503, 3.5341935468684631]], [[1.022867833036025, 2.7438039841989701], [2.042138500137392, 1.9750934225338481]], [[3.1866338377496732, 3.561457037648645], [-0.44251999702570044, 1.2990671855465044]], [[-4.0582630372892883, -1.2310798946957424], [-1.2987005637993256, 1.7671636828518329]], [[1.1627337431582403, -0.14791334289995012], [-2.6662001672248099, -1.9662932245341844]], [[0.4308067625088155, -1.9882215347631029], [2.448024203478441, 0.11601692249538204]]], [[[-4.5380324032150536, -5.8106768415526355], [6.8967094771685939, 14.42310272383676]], [[-0.9555333172372581, -9.5303432905744874], [7.0928343787087291, 6.1198044774050482]], [[-0.63429494722326107, -7.1343796837495903], [6.5486886666699506, 16.006963568386013]], [[-14.661121352129278, 0.66895074625040396], [14.610441693900277, -15.617407297941279]], [[15.495875772819984, 8.7259134862716952], [12.542894709115094, 13.529652556595964]], [[-6.7463162198976629, 1.3334165701639866], [16.041073843189714, -11.790677923485591]]]])+(1.-msk_ref)*numpy.array([[[[-3.8228327647676688, 0.17779948235860199], [-0.80645714491657228, -2.1937291696507617]], [[-0.13542692105168727, -2.1850995403046549], [-0.45108858259431917, 1.8972667825244096]], [[-4.0485806979853498, 0.0044204965526834679], [-1.8815032829613403, 2.5998423022074784]], [[-3.867878720794558, 1.905382349424112], [0.76348964613434944, -2.3376863116855482]], [[-3.77728467673378, 0.33685892251593191], [-2.5225672047408549, -0.71652830228016096]], [[-2.4904005333991788, 0.93576392172116585], [0.67706548466362715, -3.5765893765992529]]], [[[-1.8094888881873272, -2.6655635406439466], [-6.9546097528454194, -5.410929461084244]], [[12.151632530218196, -2.4111462062530573], [-8.4793608672754246, 11.086990454198814]], [[12.709006453627046, 9.7989678066908699], [-4.2753890518030548, -9.7710960924500423]], [[8.1664910785017888, 5.7696426474134261], [-11.645688072102306, -3.0135543826527611]], [[4.7646406052074175, -14.259255300765753], [7.7098083379471642, 11.232729010495499]], [[11.762222142652528, -3.0024020887166216], [11.511011354215041, 2.5717696947956865]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.4877107941718242, 1.0813810090773837])+(1.-msk_arg0)*numpy.array([1.5457820602725043, 2.8423320396566245])
      arg1=Data(numpy.array([[[[-0.30200237176660316, 3.0484010167705939], [3.845546715744474, -1.7660326108153903]], [[0.94181623742315868, -2.6363593334822144], [-4.601126798140676, 1.5997538421237145]], [[1.1611281991954439, -3.8446784817960178], [4.1406072293458234, -4.548945080735507]], [[2.9748343672850952, 3.6648872376365507], [-0.69202590668574082, 4.0417529374257306]], [[1.3315059696718663, -3.5392673763206783], [0.73836247104644492, 0.9141500818745163]], [[3.0880653156362801, 3.8690008927922381], [-3.6736908294972292, -3.3433783274776774]]], [[[-1.7855800570807565, 0.65385606917444328], [2.7012967307985267, 4.7158506000154681]], [[-1.1472047678587538, 0.59003333444652384], [-3.5436789764038137, -3.7775814789492879]], [[3.4553240653989672, -3.1045425373972737], [-4.1644993006273801, -3.3729700782752561]], [[-2.6959319840304099, 0.055705801905604702], [-3.7368697114896188, 3.5374012756991]], [[-1.9761146576178112, -4.3143709154139174], [-2.5058942212010207, -1.7451184930990324]], [[-3.4722212099519432, 0.8304441825171418], [3.0767651548635975, 1.5689530930193198]]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.053296931875874, -10.631941131155164], [-13.412154789994009, 6.1594109996002846]], [[-3.2847826573870442, 9.1948589047015545], [16.047399599228481, -5.5794787431927277]], [[-4.0496793537512419, 13.409126641080112], [-14.441240528215317, 15.865404860176048]], [[-10.375361933653535, -12.782066778127557], [2.4135862245944018, -14.096465347235398]], [[-4.6439077429287901, 12.343941031853822], [-2.5751947602800671, -3.1882911080468075]], [[-10.770278734452274, -13.493956176451913], [12.812771160487529, 11.660736681744035]]], [[[-1.9308923639144409, 0.70706753587523108], [2.9211309845683484, 5.099631280502912]], [[-1.240565449485485, 0.63805084259307532], [-3.832067147349866, -4.0850048715782163]], [[3.7365218245305027, -3.3571933418143249], [-4.5034104560144952, -3.6474657868331182]], [[-2.9153296492947978, 0.060239196276147655], [-4.0409799394013559, 3.8252785610271172]], [[-2.1369328625073574, -4.6654787740444172], [-2.7098264215635441, -1.887137997027035]], [[-3.7547940757577263, 0.89802656807282977], [3.3271554078605297, 1.6966360789243142]]]])+(1.-msk_ref)*numpy.array([[[[-0.46682984843656261, 4.7121636042404456], [5.9443771251376551, -2.7299015276546439]], [[1.4558426438820682, -4.0752369621287841], [-7.1123392616049257, 2.4728707900068501]], [[1.794851139992836, -5.9430350246760124], [6.4004763737574129, -7.0316776989658054]], [[4.5984455972314064, 5.6651169448602348], [-1.0697212317986322, 6.2476691827263924]], [[2.0582180410645163, -5.4709360168242389], [1.1413474617220709, 1.4130767969582683]], [[4.7734759658603103, 5.980632171256544], [-5.6787253792244323, -5.168134239318884]]], [[[-5.0752114056125386, 1.8584760547384582], [7.677982246568348, 13.404013254657881]], [[-3.2607368677317763, 1.6770706509627873], [-10.072312292890151, -10.737140870031018]], [[9.8211782984800671, -8.824140722521145], [-11.836889791300807, -9.5871009222848738]], [[-7.6627338549446851, 0.1583343855510653], [-10.62142450898935, 10.054468983041769]], [[-5.6167740053821857, -12.262874683843657], [-7.1225834329100453, -4.9602062059326677]], [[-9.8692055938216985, 2.3603981071149258], [8.7451881781678793, 4.4594856450071729]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.1790394433535543, -4.7788164792433587, 0.17519107118251753, -2.0044873639070104, 0.83165618219904847], [-2.5645235412454115, -1.3629909560856799, -3.9128172033284461, 3.7939593494588966, 1.7817602649005977], [2.7888257611539267, 3.7700162579446879, 0.64958389875549916, -0.55211554619249004, -1.2610356824628566], [-3.6427975269284429, 0.77088198005684472, -4.057957590862058, 2.3813919173275124, -1.7325378624904966]])+(1.-msk_arg0)*numpy.array([[3.8882157897655425, 3.2079005850185567, -3.7824154761043638, -1.4869515366794825, -3.9590913776374217], [4.3441190425945475, 1.2902890393229125, -4.0551444849343721, -3.8766491268684322, 0.53349509734523259], [2.6069272913314645, 0.23213447935939513, 0.56400861332873919, -2.9584207797721538, -2.9376736270048265], [-2.061371789025924, 1.0767896660446983, 4.1176441461910365, -2.6047696124624595, -4.5057313632290601]])
      arg1=Data(numpy.array([[-2.1913758977968065, 4.8958813696644228, -1.283863214221026, -2.2812871719881169, 1.6364473415859448], [-1.8813947940346831, 3.7287515364492663, -3.47432853650814, -2.9946103111863409, 0.51188066144677613], [-1.4002049753768553, -4.8603360763741748, -4.3489720326773558, -2.9237706740627112, 1.5191421004559356], [-2.2080917888994369, 0.11386583366091418, 1.6147791235349738, -4.3541157752131587, 2.1408582300474475]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.1578463121071607, -23.396518569772891, -0.22492137175121152, 4.5728113096933392, 1.360961548473149], [4.8248812396785068, -5.0822546216709332, 13.594412467663993, -11.361429788111433, 0.91204862293690103], [-3.9049277062268737, -18.323546027005733, -2.8250222085651848, 1.6142592426517186, -1.9156923954065082], [8.0436313078338699, 0.087777119313348848, -6.5527052019143275, -10.368856114200833, -3.7091179417815923]])+(1.-msk_ref)*numpy.array([[-8.5205423671251843, 15.705500709927954, 4.8561040906707005, 3.3921634659949214, -6.4788445600305948], [-8.1730029514043139, 4.8111672378389576, 14.088904203671092, 11.609053448171732, 0.27308582330768988], [-3.6502325637680255, -1.1282515846008043, -2.4528576855558235, 8.6497439174355613, -4.4627436841821186], [4.5516981212170853, 0.12260955300163695, 6.6490858054152771, 11.34146846041866, -9.6461320713518379]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.3319171621458441, -4.790148750950439], [-0.20150325100092026, -1.9808389997501665], [-4.9790980879587101, -2.1758974243046616], [-2.7960627653242041, 0.8684856720020786], [-4.8841129674960806, 0.1370640686804494]], [[-2.4994550807685867, -3.357109111366495], [0.28763795924897018, -1.294830688415145], [2.0467414357543721, -4.538558522746734], [-3.7637768151359419, -4.0379517087795751], [-3.4728240470133001, 4.3119569436035032]], [[1.1266090445069254, 3.3177181314072506], [3.8479819878984074, 4.9196254394107726], [-2.8544649647473506, 1.4667388054621791], [-2.0323612012884782, 0.028475161713624964], [-0.30012894389450651, 2.5730234582873424]], [[-4.446390250263863, -4.4495731635454492], [-2.6060828500908952, 4.8951283760156752], [-1.1150933042733726, 4.1145903698847555], [-0.51972091800833375, -3.1261048198451991], [-0.24102766517186236, -2.8603878824202433]]])+(1.-msk_arg0)*numpy.array([[[4.3840627075135217, 4.8020783775014149], [-0.88794679671560317, -0.65035253127700532], [2.0071818715065612, -4.0943446280353797], [3.9916921192058084, -1.9650278678570796], [2.4130013188226673, -4.7079078689262079]], [[-2.5126578198668534, -1.1122178123475344], [2.5390369340349404, -0.12102102309136775], [-4.9525956291440334, -3.2610235957367815], [3.4561455901819347, -2.5503960701549677], [2.7835530368131263, -4.6932372871610193]], [[-3.6817830785312955, 1.8095274095641534], [0.36591219022042321, -4.176596096040849], [0.58161204807226063, 4.5173001184200317], [4.1998733340241525, -1.204131434989316], [0.94046817733813803, 0.56059852601451787]], [[3.2621842458483403, 1.3253053465478901], [2.5196017254943719, -2.77936174462867], [-3.8716253050400038, 3.4763980456559853], [-1.8695729249749227, 4.94000135623555], [3.5297246121639549, 4.2075859385273215]]])
      arg1=Data(numpy.array([[4.4928378324394647, 4.0464813612106809, -3.9608737935245761, 2.8330884262542568, 2.6186058000220402], [4.463302112202209, -2.5625080149297719, -3.1153803375033262, -2.7630607016088948, -1.934425935901082], [-3.1745190926782896, 3.9847238672060481, -1.4123340629396508, 0.86015916090774613, -3.8307498258293027], [-3.3089171363437906, -1.4271159531446131, -0.66192425177804548, 0.49205424559268174, 2.5033951433094623]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[19.462601313082651, -21.521361531282778], [-0.81537914939858136, -8.015428092048257], [19.721579131983979, 8.6184550853259587], [-7.9214930595204747, 2.4604967057167397], [-12.789566544648094, 0.35891676522124405]], [[-11.155823141348975, -14.983792187655357], [-0.73707457597352921, 3.3180140170408432], [-6.3763780249024977, 14.139335982373318], [10.399543807528808, 11.157105681523328], [6.717920907363486, -8.341161346195376]], [[-3.5764419217712797, -10.532159552277255], [15.333145667757758, 19.603348906134148], [4.0314581011805126, -2.0715251763896494], [-1.7481541055617564, 0.024493171206304027], [1.1497188995502134, -9.8566091646889475]], [[14.712736893970051, 14.723268890270989], [3.7191824105812978, -6.9859157981028526], [0.73810730109386058, -2.7235471519591181], [-0.25573088422932666, -1.5382131487725756], [-0.60338748639445949, -7.1606811328320745]]])+(1.-msk_ref)*numpy.array([[[19.696882792103743, 21.574959408777879], [-3.5930601626564176, -2.6316393960285884], [-7.9501940736879515, 16.217182338843465], [11.308816744092303, -5.5670977096929715], [6.3186992489298692, -12.328154851539571]], [[-11.214750954653125, -4.9641641110796701], [-6.5063024936672491, 0.31011734164663085], [15.429219042640238, 10.159328790292765], [-9.5495400592705835, 7.0468991549829534], [-5.3845771883675306, 9.0787199316223095]], [[11.687890677897448, -5.7443793103860923], [1.4580590376729599, -16.642582147573574], [-0.82143050690854735, -6.3799368297659287], [3.6125595229130334, -1.0357446847430503], [-3.602698306536074, -2.1475127058902781]], [[-10.794297352998319, -4.3853255720803599], [-3.5957638180237126, 3.9664714853194192], [2.5627226832035515, -2.3011121752534978], [-0.91993129517903893, 2.4307486405693082], [8.8362954513111198, 10.533250203566483]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.3069859044248568, -0.78387234539427997, -2.8010194888531394, -3.177816948357429, -2.7772382415257013], [0.73160368478016569, -1.7394683525685748, 2.8334588746996161, -1.0369063940146814, -2.8699157386284], [1.1711053094691142, -2.9676397159688497, 3.8065147621132205, 2.2978179322089094, 3.1283135547080594], [4.9226769948594882, 0.34642138226181363, -2.1117154406370977, 0.076843313016551562, -0.90959350671262396]])+(1.-msk_arg0)*numpy.array([[-4.241568921931913, -1.5077717674634528, 2.1157850786469918, 1.1814590819248858, 0.8903218595790765], [-1.5649893221852995, -1.5014403680409294, -1.7378395465609566, -4.8918854715843887, 0.46852810726718896], [1.1362405260406607, 3.6179856527546335, 3.3551077790241184, 1.0278773921788984, -2.5735349176012825], [2.0437724793475907, 2.9912296715195072, 0.6520467058307764, -2.9509554854833087, 0.69942386403001677]])
      arg1=Data(numpy.array([[[-0.63031404741824115, 0.3062925832973491], [0.2362225161557312, 0.84114000603381722], [1.3668881314315549, 3.8456345856586456], [-1.6062616535689447, 0.45807487436267369], [-1.782501568732143, -0.37679562451733339]], [[3.233838553305489, -2.995974472100067], [-1.9241297084194455, -4.8543683077946218], [3.074752071578521, -0.046905088361986813], [-4.1455858587769416, 0.7734563077917409], [-3.6702968423584812, 2.9965908230862368]], [[-0.59468543359072612, 4.9045979792658443], [-4.6887625278181488, 4.7119252998644772], [-3.425614608496419, -4.3291900994530685], [-2.0457571808982777, 2.8911831728371009], [-3.5131025323486109, 0.30846404672789784]], [[2.7438722218154563, -3.7223376663257723], [0.46161461042452867, 0.73216918019222899], [-1.7948460943417355, 1.3755719313509047], [-1.299626152361375, -2.7591434135492254], [-4.6632620873761681, 3.6760633256291992]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.82381157533662197, 0.4003200889995116], [-0.18516829777393121, -0.65934638933468714], [-3.8286802952218366, -10.771697421437533], [5.104405506208022, -1.4556780993664045], [4.9504315222624609, 1.0464512176490974]], [[2.3658882015824561, -2.1918659632957205], [3.3469627340326249, 8.4440200431206112], [8.7121835447151899, -0.13290363888784121], [4.2985844839026548, -0.80200179104024361], [10.533442673322725, -8.5999631654046222]], [[-0.69643926874204165, 5.7438007343297182], [13.914557896299637, -13.983296658556254], [-13.039652576552319, -16.479126021562507], [-4.7007775352132084, 6.6434125398457411], [-10.990086271025369, 0.96497225851898305]], [[13.507196663164937, -18.323865997120834], [0.15991317141551381, 0.25363905945169085], [3.7902042109886316, -2.9048164871406992], [-0.099867579230401668, -0.2120217209849197], [4.2416729147765198, -3.3437233312567338]]])+(1.-msk_ref)*numpy.array([[[2.6735204745863297, -1.2991611023322778], [-0.35616964069879087, -1.2682471535818278], [2.8920415126625518, 8.1365362742653691], [-1.8977324185567144, 0.54119672051738188], [-1.5870001113762227, -0.33546938110153174]], [[-5.060922805594247, 4.6886680583763445], [2.8889660175677783, 7.2885445386613803], [-5.3434257458593795, 0.081513517490396772], [20.27973123375661, -3.7836596749917204], [-1.7196372326589595, 1.4039870265948224]], [[-0.6757056898918451, 5.5728029879789842], [-16.963875554819609, 17.047678131761252], [-11.493306220904994, -14.524899379549186], [-2.1027875561329767, 2.9717818200073127], [9.041092036112639, -0.79384299507883871]], [[5.6078505337927576, -7.6076112812755481], [1.3807953195087681, 2.1900861763631081], [-1.1703234832887635, 0.89693714647063627], [3.8351389233883659, 8.1421093914482281], [-3.2615967881373211, 2.5711264156306082]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.355651795111398, -0.55266182907681394, 3.2947026687448879, 1.3627730543155705, 2.5764718942119238], [-2.9489273714358601, -0.15473712505210813, -1.8898653805979224, -0.94473006051119768, -0.1307254991832334], [-2.9021270624637676, 3.8592082073739729, -4.7405618459359768, -4.3921139051538507, 3.1082285044521605], [3.7026471107224594, -4.3461639579715605, -0.99061022626435236, 3.5224452812830194, 0.082394067564726114]], [[-4.0275095797579796, -4.5576248329404168, 4.2066425863711494, 3.0528885745116625, 1.8626929239350885], [-2.1925692044954594, -3.2603639245779323, -0.65288714245815971, 1.5353067689587601, -0.74383396936177171], [2.2946443494121436, -4.4089013814523659, -3.1502946006144183, 0.91553841797519731, -2.563155934849509], [-2.0536598115360949, 0.14584906095995809, 3.7458557657594014, 3.5852274313889261, -1.8392021111893051]], [[-0.25710716420986124, 0.65222534699602974, -1.0030323313834777, -3.6146311782659724, -3.3243446986860334], [0.76714868591039398, 1.7948130522812367, -0.54104887169818561, -3.4083279567050595, -1.9089039803146313], [-0.12493450992645538, 4.1844539291946106, 3.5168401503352911, -3.9838018328706681, -4.6049465014934565], [2.273375285077285, 2.2156673106776559, -4.6191828264352068, 2.0271139618401914, 0.28561845838655309]], [[4.8399029942815535, -4.9598243560835105, 4.8586670775305283, 4.9942254152905168, 4.0302781498422906], [1.6062433526175113, -2.9493412068229641, -4.8225550824350787, -1.3405906657172526, 0.98870985499760877], [3.7752119313489327, 3.4181946288725982, -2.3527709183601697, 4.2862454909016368, -1.5994213956270533], [1.4527409227003609, 2.3801453377383375, -2.7082494249073163, 3.9216307187182444, 3.4784188168922334]], [[3.5125075514945099, 2.6467118483398311, -2.2350868234213594, 2.9358544982583457, -3.6545187127455669], [2.4722101494930424, 4.649941771723725, -0.5951990574034447, -3.6887394285095052, -1.9482452734818709], [0.86450691086532672, 1.7303717781001406, 0.5334077717496708, 0.049721670994157918, -1.4052297064390098], [4.0584709463459419, 3.4832721917229357, 4.1657518988092974, -2.8584612337487227, 2.667747731019805]]], [[[-4.5007118395596182, 1.060409131915911, 3.7528703165571038, 0.073772023383986252, -1.7192516442259933], [0.53375852159748938, 0.5091698331532406, -4.7556131544042799, -4.073475806950916, -0.25302601093532395], [-3.9134429066734775, -1.3816760971878184, 4.0248926435621364, 3.7595382459824762, -2.3268125986631683], [2.6129305212384519, -3.8649415430065979, -3.6888048694118947, 4.1941527141026995, 0.40055968938087716]], [[-4.4198174602502096, 0.56585017005843241, 2.4169573508736173, -1.3089035234798327, -2.1486871581596025], [-2.163282043772635, -2.7727504394168214, -3.5597480212031063, -3.5650197498778136, 2.074532847097097], [3.2442075387321587, 4.9810282941929511, -0.30612037769926914, 0.49977952199749875, 1.383469344368816], [4.850202277584156, 0.0063132314781260135, -0.81696335520390662, -4.2472951774663183, -4.6083258869334296]], [[-2.4974810139380041, -1.6267399139465177, 3.0533300298086399, 2.0691599933762497, -1.5940778438815393], [-0.37612777304234868, 0.28651018357219638, -3.8847079723206002, -1.8927693539310733, 4.2270013250911926], [0.38921661719252931, -3.4591977456293388, 0.80740380687862068, -1.7948309194311318, 4.6871371538802169], [-1.8670502707681456, 4.3751508470815601, -1.5932870489158302, -0.67970614508989868, -2.3219417312546922]], [[-0.54911221225982576, 2.6276824220260089, 3.7539763958895414, -1.3046625732533546, -3.0454971839285228], [-1.377261980776967, -1.3209890551806893, -1.8916874657089311, -4.2913288349963903, -0.27989915916168417], [2.1490443250312001, 4.978004225286627, -0.040106253065151698, -4.9740925397877964, -0.35909896839419897], [-4.2740696459337304, -2.3918599880475422, 1.9955887687542546, -2.8027490363819387, 2.181746309211924]], [[-0.59028243104425648, 0.082922649083946531, -4.5741562103443698, -2.5055408225336242, 3.0823619891469267], [3.5767898401253078, 4.4534285799014999, -1.6353785500154139, 3.9263376172122246, 4.4827642805368626], [4.2876446984617242, -3.9208323810679149, -4.1969258797567752, -3.0041085803028542, -0.12602300500817787], [-4.9004038999758741, 4.9352597928403981, 2.061743035319231, 4.8780506912073491, -2.8178530979598393]]], [[[-2.3304678279522983, -2.0741412594237896, -0.31262072783912043, 1.4115363440070228, -3.2511500235348501], [3.532610407131946, 3.1882200889341554, -1.3705316869990734, -4.405315441917609, 1.8325657752341069], [0.91494665281001009, -0.45535891078258839, 0.22494972537538693, 4.1105098161279034, 2.658070281154016], [1.6823185766600925, 1.1267479090020904, -3.1566687934493274, -1.0002639742777939, -0.24376300058001288]], [[0.01607313660314702, -3.2734057682114748, 4.392335542865073, -2.8644259554321794, -1.6749279813877758], [1.7536357385034052, 4.042639065243236, -2.8079292439898786, -4.1446232877737996, -2.9595931425245703], [-2.2723197493836333, 0.89215330778578839, 4.1269805410835669, -2.0189078097106661, 4.36210077295034], [2.1884169726136609, 2.2789676047681509, 3.7375645217066431, 3.6990401579093195, 2.742246558989093]], [[-3.873474061885338, 1.46365538902542, 3.9552870547694372, -4.1097663220327476, 2.5074173576288086], [-2.6744657717635123, -3.1868729994268974, -1.7748530221575001, -0.59801664961302325, 1.7098043572686166], [-3.5189836786204989, -4.369102458709885, 1.1284674618202821, 3.9446514404686841, -3.0716300125566631], [-3.1053596542170672, -0.12918429815356625, -0.060109913996965858, 4.9812295823393011, -2.3305365272298939]], [[-3.6269844935803439, -2.381984523351004, -1.3042097215872852, -2.8514691223533228, 2.323182862570718], [1.0929073655163615, 0.75795199093651977, 1.8205276331158302, 3.9376671683131033, 2.9644197115008373], [1.9475975886040651, -3.0948452711450889, 2.7648905296931954, -2.7482247724723985, -0.74277629569527903], [4.2981892234760704, -3.1397619812374602, -0.20892687460926318, 4.6154916810532054, 1.9738911523963996]], [[0.23145408380169741, 3.0214264071596837, 3.2087698435813579, 2.4570456596112065, 2.3842967981482497], [-1.8562965786914001, -4.1458335269773166, -4.5819555026710512, 1.5606998882504532, -4.4180169431547007], [0.67925535312982976, 1.6352504893974, 0.80634107620073436, 4.1671410523012113, 4.3038015010054895], [0.041759249063321846, 3.7562316830946472, 3.5533514017378582, -2.98628780124342, 4.4815065832411758]]], [[[4.441177596381495, -1.5681824817844268, -4.3365924816519499, 4.2159431408447308, -0.16970842003669961], [2.1469249411926441, 3.3998819982391382, -3.8170008482821318, 0.11867220326726979, 2.1525806476652187], [0.085382566055702824, 0.068719676015510878, -4.8243916578334121, 3.589121467468015, -3.5832226399873068], [-2.2922701272486066, 3.1910608237992726, -1.2809387174693478, 0.27894398042877722, 1.9017241305244417]], [[-4.3797365363612002, 2.919766963573843, -2.7544741505890169, -1.7704762206602966, 4.0125551575829839], [-2.5826971722844361, 1.8479841190339634, -2.5579534056774689, 3.3542980956499697, 1.1963765365156167], [-1.8445220848678456, -3.5937045817798818, -1.3768215631477263, 4.2660497907971759, 1.284919583419212], [-4.3759970179742602, -3.3924803532737648, 1.6178775132670609, -3.152268092544448, -2.8403475939989953]], [[1.3105885307669638, 4.270965804049645, -1.7099888456558245, 4.1710646157008693, 1.9949522863970435], [1.4342355424790698, -1.4886951987208019, 1.2648554193269259, -0.88097185056581218, -1.6278702821249666], [1.7978093017448735, 0.6670217953458355, -3.1884574001715658, -1.6999182227051381, 3.1769368922359309], [1.3554016192810705, 2.9239358539672802, 2.2469918567150851, 4.7639641748022914, -2.9301442685755807]], [[-2.5046706980707745, 1.2520035986165032, -2.318824162274332, 0.92677646239976763, -1.0624168133299094], [4.5018803673570247, 1.2168303739067641, 3.1493762858027896, -0.49641407367734125, 1.0746353011834886], [-0.55473819278112657, 4.6159898917858513, 4.108123449247536, -4.2379968699871124, -2.0343617473034525], [2.6203870688725992, -4.4400483709902572, 0.80926378852166447, -0.047360880696293073, 2.8714583248675529]], [[3.0804493419296453, 3.9787228605408185, -1.3189298148339468, -4.5659487001185166, -3.5961726967160157], [0.33634299985656746, 0.5302513681714025, -2.0635105534388565, 3.1774941453893994, -3.8794234468089006], [3.5181091593299296, -3.3928889363423176, 2.3847139424411665, 3.6935840991119129, 1.4447848907147662], [-4.4154638547393175, -3.2560296412450218, -1.8045834760489776, -1.2428115794337371, 4.5368958205821492]]]])+(1.-msk_arg0)*numpy.array([[[[2.6379582401896204, 1.8714640732765364, -4.505113319587922, 3.1379510050892794, -1.719899051592396], [4.4249167047292275, 3.2606531899428433, -4.684643296209785, 1.7521319611655448, -2.3321903469658567], [3.4187356144474759, -0.60195771057184988, 3.4456047134333652, -2.1163625132138097, 2.8659440007533243], [-0.1728740489227123, -0.70149623224888558, 3.5575400960392063, -3.2214275149883664, -4.2955894075253216]], [[-0.3412168186329847, 0.58580945180429467, 4.3690241141644908, 0.56126783236042499, -2.2628388419854497], [4.9716631460236833, 0.44595360410092511, 4.6244167572761619, 1.8035577179616897, 2.935087540674191], [-1.3943604119138922, -2.9390837314446649, -0.85754054222193954, 3.9695272592823763, -4.641172315063252], [-0.91896450246491934, -4.1640140590963162, 3.4129757533502136, -1.1775845317398739, 3.6797170071131422]], [[-0.21289984788773619, -2.1905959190434987, -1.346504912859273, -3.9381742960823574, -0.73914066653582911], [1.7643041375988595, 1.650523818750762, -0.14666095358479403, 2.0918080490999893, -1.7263007668700281], [-2.8806300159468856, 1.0938326614722458, 2.4956773823932332, -3.4669306033619796, 4.6314494767254182], [-3.607553828084332, 4.0879052701408956, 4.9395231217791782, 0.95241615898439935, 0.75157005247127184]], [[-2.6777856928976806, -2.9770347313210688, 0.75789182262999155, 1.7604676981558693, 0.29672990411849476], [3.5511165542435847, 1.8645207534266195, -3.5351713661869977, 0.10932592505928618, 3.2422919482330208], [2.5440955646274501, -3.5392544901076963, 0.64805965593121151, -1.2029501119365071, -3.8807875278623225], [-2.2739805088459528, 3.2494168533372498, 1.5072054659100349, 0.40714409382742289, -0.23451679895673649]], [[-4.0332575572932399, 1.5202232062631342, -0.88845160092966502, 2.9171769898485964, 0.35394227633542474], [-1.5001848944765559, -0.19764789861038068, 0.460982452187638, -3.5091165139953393, 3.1818999072630341], [-1.9727807779131377, 2.8436029042078967, -0.23623350484805528, 3.3720854997103515, -2.4581820417418876], [2.0802729012452925, -2.8541122877566991, -0.31932578999842853, 4.1995672164069937, -3.5621157635955503]]], [[[-3.4400899878591495, 1.0099215306032141, 3.3884839179435886, -1.2110386855800059, 2.5982297590378742], [-4.2755139609052364, 1.5850611151607286, -2.3520553471583181, 0.04823236633193595, -4.3223847537639521], [2.322519076344185, 3.7884728843113855, -3.3145640398237863, -3.0537249544924494, -2.4222920287733207], [-0.25820175874726736, 1.6051449441653984, 2.2388361289020375, -2.0530531165751875, 1.4411178519724324]], [[-3.7143278836534224, 0.84943868648135101, 2.0187131651489292, 2.4530476480604957, -0.30036798396263276], [2.0373387794006241, 3.498291132331719, 3.1339115453711628, -3.710036250411922, -2.8090307116351667], [1.9505009380692773, -3.3646227312492574, 0.23203484409091324, 1.0704014505690331, -4.570785870347005], [-2.9280720105978864, 2.0160654167219469, 2.2313248639318122, 0.01746812433307543, -1.1189992110127731]], [[-2.0118658171682249, 3.1611230420348804, -1.6368137556429518, -1.5820159654179964, -2.6645263549525291], [4.1801143737698325, -3.7524810829077748, -4.1139632564718989, -1.6543620193980759, -0.61369840492747318], [2.5303263583920197, -4.4846808292919249, -2.854817974354261, 0.023360568244962465, 2.1051120697368972], [1.1079272053495304, -2.5394682958358272, -4.8469813784617912, 1.4734886524687472, 3.4063256467475718]], [[3.7011042638381983, -3.2343512853992329, 1.5087465676322491, 2.8726147091981051, 0.69295428770269485], [2.5155155851290978, 0.55910708907187789, -0.5694960503270563, -3.1268106618552105, 0.21904681248509217], [-3.4527294905699235, 4.2390277900684055, -2.3553256945532395, -3.8962636184789687, 0.49769146277352316], [-1.5529695172158462, -4.613292090330944, -3.3748305345181882, -1.7424770708118089, -4.4666314059127901]], [[2.9363767726032837, 0.33202724871858269, -3.4773986794311309, 1.2464284352443569, -3.8946510108903345], [0.12722137315844062, -0.28184920234397914, 2.7660533728755361, 4.5694015997998356, -4.5463821912427891], [-4.5358961358500158, 2.4505613082203928, 2.0367397238692693, 0.75566906392270461, -0.046080582374874446], [0.58495555965447732, -0.65802572909788459, -3.7055197452088007, -3.9011757980091213, 4.0929302958994249]]], [[[-0.7136691816038363, -1.7221340779670324, 4.6934872395355836, 4.4118000164755813, -4.7108710847693605], [-0.45418555065891741, -3.0512602566505267, 1.3310839463941946, 0.16923968268299205, 3.379675876161377], [2.5113436683654475, 1.1615850727843622, -4.3681629946030522, 2.3177353428550918, -0.52058965394566314], [1.7781247051104607, 1.5119779197789818, -2.4894120681840159, 4.6826731994719708, -3.9583211329091963]], [[-1.0199282301684964, 2.2680178749985878, -0.85111941039461403, -3.5541062331803266, -4.7267207453298994], [-1.9650619715550945, -0.81456718483746648, -2.5014236562409584, 3.5871445802506106, -3.2190402908904847], [-2.6497458611394942, 2.3175749051010577, 1.3085460659819805, 3.8204901723812714, 0.95729822836007372], [2.8268139610620384, -3.4844694860926664, 3.321433808193289, 4.0123145917727943, -1.9982417506200356]], [[1.8924229388386475, 0.8464405519892253, 1.4720614327414658, 4.441340013679639, 3.5388394331929547], [4.6504264473145493, -4.6848450807049371, -2.902753774052973, 2.3723285025695979, 0.16920966394525649], [4.3428943305010392, -4.953880998133668, 4.2279103126444806, 2.4590392590541645, 3.1355982338337629], [-2.3278499912363015, -0.42135507276459894, 2.4990176983471528, 2.0817997377107744, 1.1975148793881774]], [[-0.52334508074414465, 3.0328353917170787, -3.4940552318704778, 1.0370685739634258, -4.25317192367869], [1.960339447032192, 1.0362934282337202, 1.3040447045851273, 0.31516795922455998, 4.5817132589577998], [-3.994274372527812, 1.3843540325908901, -3.7465551191753632, 0.46265774479996935, 2.6952944317861958], [-4.4923648052778571, 4.2037638391497421, -4.9007724410801492, 3.7751324930249517, 2.5697407615334757]], [[1.9661462873301545, -2.8413465942418012, 1.5587749755943818, 0.57953017538231766, -1.1603900376082654], [-3.6556131012154989, 2.8898062955687189, -2.1880515863017225, -0.03333605338339396, -4.1171590903954138], [-1.5761239714142805, -3.13925399702629, 2.6007438988713485, -3.342942995755843, 3.1119158651321843], [4.8561560483905311, -4.7365796920075613, -4.508318844432166, 4.8588933785460693, -3.0201532387331911]]], [[[-2.5879214650985016, -3.9419226629552262, 4.065365112019462, -3.2915523153030968, 4.7822352325246875], [0.77007822817106408, -0.64962848816254315, -4.2632942672307585, -3.4163440172363844, -2.4551003534893292], [-2.9873401628213889, 4.6746846931524981, 2.1240517476735512, 1.6775119778866578, 2.578319721834907], [-0.51676071957428782, -2.2419050047443969, -0.94081369392194336, 2.3277558090272068, -4.2372555858762553]], [[1.009288496708578, -1.5298151081319435, 1.3237541188566562, 0.66061606930216143, 0.73550959755000367], [1.372634175495727, -2.9636981756022243, -4.4044627866238608, 4.7945524834889905, -3.6733192910580428], [-1.8563230059198976, 0.65437880586093211, -2.5677364203900508, 4.1205976792459804, -0.79729878575753776], [-1.4073483200483548, -3.0250233767875221, -0.72763318097014107, -1.1297178374248262, -2.1473162346789518]], [[-0.6355160914343525, -2.5455995427745282, 0.23699885776004059, 3.7990912861871955, -4.0204461415746717], [2.8068346484341697, -2.0956340653388161, 3.4845217068245109, 0.54215874147234189, -1.4681481254484199], [4.3898276566430834, -1.5438386933250059, 0.72079514673262235, -4.3034220321941792, 1.3058200711564325], [0.15981874840053667, -3.7202582059348934, -4.6820790310495122, 2.858114802981242, 2.8118055746905712]], [[-3.3856628521849319, -2.0956327296890054, -0.46716102871730847, 1.942788166896471, -1.324887489110651], [4.6725280416545498, 2.541081631612947, 1.6351904283321916, -3.9539218536327914, 2.9320784720529369], [1.7760797906652384, 3.2551245696203921, -3.0067908524836406, 2.6098871761381215, -3.4353599170478488], [3.6699009414617443, 1.4957692561769234, 2.4805506098443324, -3.0801845016626817, -0.83766139968769604]], [[-4.239588972056076, -0.22794138373586126, 1.525047149332643, 4.4120348926104125, -3.7845370646123899], [-1.94952752359809, -2.9023492814845895, 0.11184912384997325, 1.9101757745239833, 1.516992431000781], [-0.57808424536232561, 0.61197134024692623, 1.0619508127106814, 1.9422707751123527, -2.665342607690512], [2.3293400023848143, 1.1189256345886971, -2.1551085770400782, -2.8120418714727702, -1.1985642305864719]]]])
      arg1=Data(numpy.array([[2.3238901894071269, 2.3194190159439012, -1.1781409772492757, 4.7639552578869218, -1.7220831043747675], [0.046237916510458277, -1.6744052220196606, -4.6924427813209322, 4.6553481378367465, -0.72778216449991007], [-2.0102431196803252, 0.064484649663835825, 0.5767214636307072, -2.4839921427755551, -4.1646625784126297], [0.30724497286996133, -3.2641858483554764, 1.4958447870805394, 3.5345570939243132, -4.3835624417744654]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.1503859069115383, -1.2843254026514064, 7.6565272089097238, 3.1669349313123401, 5.9874377582422866], [-6.8529833877539419, -0.35959208684565785, -4.3918396172716783, -2.1954489192599738, -0.30379170505726549], [-6.7442246088724733, 8.9683760919958395, -11.016545166048356, -10.206790414945658, 7.2231817279319621], [8.6045452954445683, -10.100007783484958, -2.3020693863421027, 8.1857760318970367, 0.19147476527901497]], [[-9.3414823061868884, -10.571041705060148, 9.7569668081086789, 7.0809278132802191, 4.3203653886391908], [-5.0854867066797604, -7.5621500855635437, -1.5143188534827305, 3.5610197152303376, -1.7252626532427264], [5.3222417388547472, -10.226089703561954, -7.3068532024904798, 2.1235172164788683, -5.945032616119418], [-4.763297619156587, 0.33828508544808805, 8.6882090940854599, 8.3156446808471838, -4.2658803508566434]], [[0.3029084856999959, -0.76841340769665045, 1.1817134911087497, 4.2585451087579731, 3.9165467120234125], [-0.90380930251396918, -2.1145428033943716, 0.63743184644211837, 4.0154908296985257, 2.248958000842912], [0.14719046561691349, -4.9298766413959099, -4.1433334915455093, 4.693480184545705, 5.4252761714501343], [-2.6783565800153033, -2.6103684506610478, 5.4420485692294456, -2.3882260239980542, -0.33649880968396523]], [[23.057081317270264, -23.628381319359654, 23.146472570323645, 23.792266426245753, 19.200064782687956], [7.6520714651481097, -14.05052954954682, -22.974436641415892, -6.3865139506178341, 4.7101695122404745], [17.984940729987187, 16.284126274698448, -11.208495387125373, 20.419481742974963, -7.6195719672743394], [6.9207927570458825, 11.338905896253596, -12.901979087456441, 18.682473281928647, 16.571031611866562]], [[-6.0488299084174795, -4.5578577561745348, 3.8490052554245926, -5.0557854283533574, 6.2933849298405651], [-4.2573513289057869, -8.0075861614118988, 1.0249822404942595, 6.3523158462772544, 3.3550402686411283], [-1.4887527448164022, -2.9798440033531768, -0.91857251147230057, -0.085624849540320305, 2.419922335224133], [-6.9890242462982197, -5.9984841893045333, -7.1737709619565972, 4.9225077951489284, -4.5940832943233278]]], [[[-0.20810353827518871, 0.049031108908455433, 0.1735249043715445, 0.0034110646580363321, -0.079494613986189594], [0.024679881958370356, 0.023542952234983511, -0.21988964398938221, -0.1883490342691681, -0.011699395568601811], [-0.18094944638721341, -0.063885824026266178, 0.18610265001658377, 0.17383321553561248, -0.10758696667247003], [0.12081646328865177, -0.17870684438334086, -0.17056265157523914, 0.19392888302679259, 0.018521045475048098]], [[7.4005654358166248, -0.94746247962655217, -4.04696600970159, 2.1916348948345652, 3.597772998109023], [3.622210750794264, 4.6427078151168342, 5.960460675776635, 5.9692876857986352, -3.4736086324306932], [-5.4321180441686767, -8.34025978682436, 0.51256955898628709, -0.83683344149110173, -2.3164882947152616], [-8.1212040214385617, -0.010570907754793097, 1.3679277081521242, 7.1116932246085245, 7.7162049298497184]], [[11.71928675533947, 7.6333839662849714, -14.327576457365979, -9.709414874316451, 7.4801190713855652], [1.7649580534668869, -1.344432642678288, 18.228769882255676, 8.88171189155935, -19.834961854558181], [-1.8263767057152367, 16.232087490640033, -3.7886961651986235, 8.4221413915762255, -21.994122902786362], [8.761026565429276, -20.530145009578028, 7.4764083112572184, 3.1894821939465734, 10.895578715473908]], [[-2.556308514807196, 12.232776470205133, 17.476067024087477, -6.0736584809003018, -14.177849643988704], [-6.4116339975234027, -6.1496639381381453, -8.8064637208571863, -19.977629700845579, -1.3030280293854175], [10.004549496662625, 23.174342700331554, -0.18670857052246326, -23.156132442528772, -1.6717307138130308], [-19.897282167182155, -11.134940941323348, 9.2901604585080442, -13.047772507344394, 10.156788617821926]], [[0.42959702533165789, -0.06034962503638109, 3.3289893075251316, 1.8234879230664061, -2.2432880802335986], [-2.6031238518076836, -3.2411258913264742, 1.1901993409069422, -2.8575184896121324, -3.262475891032], [-3.1204713392530379, 2.8535118769349435, 3.0544478010150753, 2.186336644965563, 0.091717295361634699], [3.5664265572482425, -3.5917940544027624, -1.5004998088872445, -3.5501582905871669, 2.0507832268759891]]], [[[4.6848069167774593, 4.1695281960017576, 0.62844366720804734, -2.8375312237188384, 6.5356019658594597], [-7.1014057654481064, -6.4090974978064805, 2.7551018940937562, 8.8557550571363652, -3.6839027410260048], [-1.839265213685866, 0.91538211738582531, -0.45220363770985023, -8.2631240762495572, -5.3433674943166078], [-3.3818693438413487, -2.2650372316856453, 6.3456717231411037, 2.0107737721560328, 0.490022894748602]], [[0.0010364705828529118, -0.21108442417069634, 0.28323821868766835, -0.18471150422404228, -0.10800714409194645], [0.11308258623537409, 0.26068816383954679, -0.1810683335795267, -0.26726458070066889, -0.19084832694318785], [-0.14652974296321883, 0.057530193498998856, 0.26612689436124143, -0.13018856281277441, 0.28128854014205013], [0.14111930179738411, 0.14695842758870528, 0.24101553877823498, 0.23853130867524241, 0.17683280864827097]], [[-2.2339156303060927, 0.84412147820971239, 2.2810989393062191, -2.3701904484229144, 1.4460814084247269], [-1.5424218143216817, -1.8379380606346623, -1.0235958326680574, -0.34488903744035454, 0.98608087144611711], [-2.0294734176265843, -2.5197551647396863, 0.65081140624062228, 2.2749651522600769, -1.7714749565736863], [-1.7909275648798137, -0.074503357509230386, -0.034666677579046082, 2.8727820154072981, -1.3440704370288499]], [[9.009400984022351, 5.9168308402168694, 3.2396467009543106, 7.083026895292762, -5.7707679768564857], [-2.7147733087241734, -1.882746790087404, -4.5221763363655008, -9.7811343069550176, -7.3635952712570578], [-4.8378171073811158, 7.6875713366304836, -6.8679663513924396, 6.8265687414025757, 1.8450504823470055], [-10.676668259277124, 7.7991440915792607, 0.51897271494406338, -11.464845070782101, -4.9031301132468421]], [[-0.96392816142970994, -12.583221491325656, -13.363443690302228, -10.232766112033968, -9.9297916510770659], [7.7308488957114694, 17.265997746130878, 19.082298617925957, -6.4997884207294359, 18.39954983394934], [-2.8288693503662583, -6.8102665195242906, -3.3581385054901651, -17.354736399485883, -17.923881056153668], [-0.17391318187662916, -15.643437526432164, -14.79850961076772, 12.436881054208603, -18.663962762134368]]], [[[1.3645294901109124, -0.48181618407100485, -1.3323962393732316, 1.2953273359301389, -0.052142058909979777], [0.65963189531057731, 1.0445966523100538, -1.1727543220750629, 0.036461437873270844, 0.66136958269230395], [0.026233364191352096, 0.021113774993018173, -1.4822700840250944, 1.102739527899206, -1.1009271428099312], [-0.70428847305712095, 0.98043739623460391, -0.39356198149695282, 0.085704135699078685, 0.58429517888913285]], [[14.296274021515661, -9.5306620029935782, 8.9911155420136399, 5.7791634243292282, -13.097725761128155], [8.4304035603585614, -6.032163609336326, 8.3496353075650891, -10.949052374986355, -3.9051953597988152], [6.0208628864047604, 11.730519639016126, 4.4942014621374744, -13.925179355499983, -4.1942163204718055], [14.284067538517345, 11.073686359980211, -5.2810528831788899, 10.289588897906098, 9.2714224207420468]], [[1.960437021755306, 6.3887019337869058, -2.557877900740134, 6.2392652619722382, 2.9841389780814205], [2.1453937596629462, -2.226856952558339, 1.8920273854107519, -1.3177971502335661, -2.4350412755599584], [2.6892436721799733, 0.99776107543717041, -4.7694373808750061, -2.5428138118966963, 4.7522044891349662], [2.0274704466021114, 4.3737542049148415, 3.3611510554796822, 7.1261509767164508, -4.383041029542702]], [[-8.8529015838104179, 4.4252782011087302, -8.1960163923298435, 3.275744319657178, -3.7551728842596943], [15.912153188440364, 4.3009564301947272, 11.131650292421256, -1.7546038856401132, 3.7983598271795906], [-1.9607538145652841, 16.315479817494602, 14.520396880254697, -14.979441900841984, -7.1905677455396795], [9.2619077031111825, -15.693604467050704, 2.8603890645753145, -0.16739973683958576, 10.149333392068634]], [[-13.503342039071661, -17.440980097696198, 5.7816111996426391, 20.01512123290847, 15.764047567459121], [-1.4743805417250035, -2.3243899822156844, 9.045527360259813, -13.928743994687224, 17.005694917170736], [-15.421851176901418, 14.872940510462298, -10.453542472461011, -16.191056532402357, -6.3333047833804752], [19.355461516647974, 14.273009244666065, 7.9105043486551088, 5.4479421618081325, -19.887766121347454]]]])+(1.-msk_ref)*numpy.array([[[[6.130325274442348, 4.3490769997152432, -10.469388645557746, 7.2922535555672097, -3.9968565327661909], [10.283020519063964, 7.5773999591672263, -10.886596596933785, 4.071762275099279, -5.4197542671439578], [7.9447661545912354, -1.3988836180358968, 8.0072069901227518, -4.9181940816865835, 6.6601391467408622], [-0.4017403062945788, -1.6302002120292485, 8.2673325276079996, -7.4862437979676448, -9.9824780818692673]], [[-0.79142477769722597, 1.3587375822345533, 10.133597611510577, 1.3018152834143835, -5.2484714401175285], [11.531370041754812, 1.0343532695804039, 10.725960164475962, 4.1832060673927307, 6.8076978552997369], [-3.2341060544724525, -6.8169666961641138, -1.9889958405724104, 9.2069970094872211, -10.764823323830086], [-2.1314637419945601, -9.6580933913257478, 7.916120863275947, -2.7313119557988581, 8.5348055995904009]], [[0.25082603484667965, 2.5808308168201828, 1.5863726139069747, 4.6397245137644472, 0.87081190719720269], [-2.078599000835661, -1.9445497447962292, 0.17278727918069989, -2.4644447791845621, 2.033825672506429], [3.3937882620812605, -1.2886890807340878, -2.940259790191678, 4.084533009100304, -5.4565004125899303], [4.2502069924986401, -4.8161287098662591, -5.8194545978383143, -1.1220805042938817, -0.88545547608979358]], [[-12.756851231174279, -14.182460261188986, 3.6105627333276504, 8.3867893469697403, 1.4136079868975853], [16.917360379958012, 8.8824934467260288, -16.841398217477838, 0.52082381550953794, 15.44613377438913], [12.119957441673739, -16.860850037148456, 3.0873272052978846, -5.7308005107355839, -18.487898148101699], [-10.833141401449055, 15.480076503522367, 7.1802594040380185, 1.9396162465067575, -1.1172275374527549]], [[6.9456046950065344, -2.6179506983841807, 1.5299874910156896, -5.0236212066891106, -0.60951801400118011], [2.5834430602163203, 0.34036610681211366, -0.79385009232558046, 6.0429902600338563, -5.4794960701093105], [3.3972924462795251, -4.8969205168874392, 0.40681372738607074, -5.8070114655583414, 4.2331937615611741], [-3.5824028157231975, 4.9150185487342259, 0.54990554774741895, -7.2320037490606568, 6.1342593723149204]]], [[[-0.1590625936470948, 0.046696667414145646, 0.1566764364949062, -0.055995905634783447, 0.12013673067338136], [-0.19769085756363508, 0.073289923506775656, -0.10875413876988327, 0.0022301641275578933, -0.19985806537061537], [0.1073884431459491, 0.1751710929269249, -0.15325853534193953, -0.14119787949172488, -0.1120017365903694], [-0.011938711363809638, 0.074218557915503899, 0.10351911800877002, -0.094928898595739677, 0.0666342869212323]], [[6.2192900046825246, -1.4223045724298955, -3.3801438654852043, -4.1073957917755406, 0.50293772087454991], [-3.4113306912515662, -5.8575569401213015, -5.2474378569171796, 6.2121040715719635, 4.7034556923755266], [-3.2659289562574445, 5.6337418713298097, -0.38852035463634293, -1.7922857784902086, 7.6533477300427046], [4.9027790649947081, -3.3757104616924711, -3.7361420041897349, -0.0292487186021902, 1.8736581223556674]], [[9.4405652307573753, -14.833388999463841, 7.6806548920335738, 7.4235193968601427, 12.50313745993637], [-19.614947518292119, 17.608302769533942, 19.304537185451117, 7.7629991156160214, 2.8797446501100921], [-11.873411654822714, 21.044108183939265, 13.396069995743899, -0.10961812982862912, -9.8781179355084703], [-5.1988850169714782, 11.916309673188197, 22.744182780560013, -6.9142611906352807, -15.983988191908999]], [[17.229928842598799, -15.057031233593205, 7.0237405240943742, 13.373021537187846, 3.2259434528627295], [11.710600794930059, 2.6028381459620906, -2.6512023773954438, -14.556392192035739, 1.0197391706015488], [-16.073657804378712, 19.73415012873317, -10.964861086137464, -18.13846358080713, 2.3169270244399676], [-7.2296137500880207, -21.476480742019152, -15.71101104438384, -8.1118373868269842, -20.793724197919236]], [[-2.1370426433524781, -0.24164350974536011, 2.5307887377455174, -0.90712838449637401, 2.8344575426775305], [-0.092589446327900671, 0.20512482254447426, -2.0130843108336345, -3.3255289867716762, 3.308775871786521], [3.3011443076957026, -1.7834748131363687, -1.482302844760526, -0.5499624669872869, 0.033536625982202528], [-0.42572022334159176, 0.4788993894194899, 2.6968111807652164, 2.8392061663697423, -2.978761669896941]]], [[[1.4346485620470004, 3.4619081814002475, -9.435050430583809, -8.8687906285255824, 9.4699961858585961], [0.91302337827030855, 6.1337749372857449, -2.6758023449558643, -0.34021290769036627, -6.7939701768029828], [-5.0484113304843889, -2.3350684004881339, 8.7810696055429904, -4.6592115262143681, 1.0465117700210309], [-3.5744629543819109, -3.039443210344269, 5.0043234821160869, -9.4133115809499852, 7.9571878229159418]], [[-0.065769714604671597, 0.14625233810060131, -0.054884137001387195, -0.22918529531468856, -0.30480093132138353], [-0.12671633280345679, -0.052527079541901028, -0.16130342813352949, 0.23131576155098804, -0.20757868541184507], [-0.17086793355377924, 0.14944800582513926, 0.084381134633838617, 0.24636297031013402, 0.061731040879610059], [0.18228610794392558, -0.22469479407501164, 0.21418149550296431, 0.25873270079156507, -0.12885591923238299]], [[1.0914009270953491, 0.48816043401960973, 0.848969424044974, 2.5614161131709468, 2.0409246574651032], [2.6820007472021969, -2.7018507118272699, -1.6740804051313898, 1.3681727662147825, 0.097586845050968429], [2.5046403746800596, -2.8570094998959976, 2.4383266236076859, 1.4181807206070876, 1.8083668027744682], [-1.3425210540585288, -0.24300451427302264, 1.4412371446298111, 1.2006185917185801, 0.69063253396029944]], [[1.2999850685286938, -7.5335392833568466, 8.6792057423900868, -2.5760701892445992, 10.564845640291459], [-4.8694677836009417, -2.5741447333425045, -3.2392368000175265, -0.78287473436841348, -11.380939735701757], [9.9217461574488457, -3.4387245397754258, 9.3064134785071353, -1.1492382028773818, -6.6950901910236151], [11.158998878791634, -10.442116346531963, 12.173480237174068, -9.3773994506106728, -6.3832158606192255]], [[-8.1883358665288206, 11.833249833339003, -6.491771809023982, -2.4135476344756466, 4.8326329659899674], [15.22439508378713, -12.035068138016271, 9.1124965611071751, 0.13883341403778657, 17.146578393141162], [6.5640245226881513, 13.073933645547664, -10.831220791664466, 13.92222959619097, -12.960079550684572], [-20.224251369664195, 19.726256192973111, 18.775626782959112, -20.235651426127728, 12.577919174423826]]], [[[-0.79512586033377974, -1.2111359216351643, 1.2490629935489068, -1.0113129018153584, 1.4693177342748209], [0.23660266432216648, -0.19959508722105457, -1.309875731471976, -1.0496545248902476, -0.75431724150086144], [-0.91784524727940353, 1.4362733717232627, 0.65260422158835418, 0.51540712213482132, 0.79217577298525221], [-0.15877213326586376, -0.68881404235972277, -0.28906027786473559, 0.715191270392459, -1.3018754775256423]], [[-3.2945052278641134, 4.993600826564693, -4.3209794614741703, -2.1563736246123359, -2.4008400196523536], [-4.4805330506222392, 9.6740616435977245, 14.376985097705933, -15.650310365802367, 11.990396846362835], [6.0593832859004291, -2.1360140375550105, 8.3815688857441515, -13.450396631361148, 2.6025314133807598], [4.5938464700086934, 9.8742384974543267, 2.3751299321166139, 3.6876089775568706, 7.0092392651830018]], [[-0.95063343247787557, -3.8078218060538824, 0.35451350592439895, 5.6828508960862179, -6.013963402612541], [4.1985889770572911, -3.1347432922654663, 5.2123036306224284, 0.81098532720154848, -2.1961217201140846], [6.5665008163715362, -2.3093430615034416, 1.0781976627929457, -6.4372514134652041, 1.9533041463044885], [0.23906404167267906, -5.5649288439413098, -7.0036635112945156, 4.2752961289172138, 4.2060247111848907]], [[-11.966818651826275, -7.4071335309822466, -1.6512073280577424, 6.8668956972961341, -4.6828904732876229], [16.515317136190369, 8.9815981072583106, 5.779673928378684, -13.975362536580153, 10.363598763337469], [6.277655423471427, 11.505423639159083, -10.627673937592785, 9.2247952327610907, -12.142475764984715], [12.971474406643123, 5.2868818352940377, 8.7676477548635656, -10.887087980947557, -2.9607620425727155]], [[18.584502986466227, 0.99919528867062235, -6.6851394057497879, -19.340430447045442, 16.589754535938255], [8.5458756316501709, 12.722629303226952, -0.49029761845412334, -8.3733747823907834, -6.6498310449911653], [2.5340683861518252, -2.6826145825488079, -4.6551276976104123, -8.5140652215386883, 11.683695749533342], [-10.210807348576916, -4.9048803869216719, 9.4470530162588986, 12.326761132485213, 5.2539811452531682]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.4309072273787997, 0.3418510477047878, 4.7466543125431322, -0.096616018314319874, 2.0020070280476876], [-0.49219978460475744, 3.0897629420009096, 1.2226973455393217, -2.8756539946198933, 2.3975579176485242], [3.7419362813937305, -4.4249186160750149, 0.010668211850619613, -1.0058405759144486, 4.0198208105233579], [1.3600734843699165, 3.0829497368032719, -4.4936653250100971, -4.8584130021741281, -4.1043328644261656]])+(1.-msk_arg0)*numpy.array([[-4.7558671014932763, -0.90297813889773337, 4.4696491329601358, 4.1908524987368789, 1.1152359815749149], [-4.342642994355411, -0.12843617702105803, -1.7223625486282002, -3.9805304303388214, 4.0408082411898576], [-0.99640426776984192, 1.1090021666777448, -1.2269056568562373, -2.0701712490758659, -1.1947993860495765], [2.5569689150914456, -1.5256764231534845, -2.5830987211117549, 4.3077071633843182, -0.092874278161603208]])
      arg1=Data(numpy.array([[[[2.2455611946430052, -2.6411289980945507, -4.5022811382812105, 3.2049385833090707, 2.639389987679337], [3.0955173636106199, -3.3857694033071195, 2.0003933588729392, 2.8052255659052712, 3.0622364265608102], [0.31769066043823635, 0.31404350317567964, 3.073471621134324, -0.34895864375339336, -3.9145006042920985], [-2.7360448314415864, -3.3123197687318928, -3.7483141966021316, 4.0516262493030091, 4.0279776449080451]], [[4.1017992058292911, -2.8729775620860742, 0.53031364423699845, -3.0272181879136162, -2.9866532258335452], [-0.71841317445712871, 0.77781160138592931, 4.8286117513485287, 2.373612668866536, -2.7851367120356487], [4.6845009327428162, -2.2453242925169601, -4.6832498707146986, 0.80303833027885663, 1.8475132382882906], [4.8230402817299982, 0.74672362770280465, 2.858338015493338, 2.3617114927682135, 1.345128494759992]], [[1.8994050469445112, -2.65821927202186, 2.7043771881893131, 0.82683510149424855, 2.0373808502271276], [-4.4541551391585266, 3.4433091397654021, -1.5386897544465317, 3.0401163889575837, -2.2778541424684939], [3.6079303764047701, -4.4530346409448773, -4.7400177176550704, -1.2496950381266823, -3.2767023397868322], [-4.7383764687954013, -0.73334499931489106, -2.3441008025420738, 3.8993940958762199, -1.446589379306511]], [[-3.3522872404390016, 0.064240402184410961, 0.033960624678366358, 4.8280829742204254, -4.1599375544644968], [4.9170856665264715, -3.9597458728793744, 2.5811940139711655, 3.1390059833444823, -0.56420688580730705], [-3.6075455200456261, 2.0792202624390788, -3.2982331374945839, 0.36050354888597536, 4.1993961286008936], [-4.8312672177092715, -3.5294845889747126, -0.79879531366475298, 4.1927854137977327, -0.14779383977284866]], [[-3.0454155258283624, -4.9247837826136553, -1.9968013218101657, 2.3312593372630825, 1.0345647042471997], [0.83814773304586421, -2.753923287539247, 4.9903907383722661, -0.46082860692507843, -4.8559004727729], [-2.9693129739532012, -0.30525208907090473, -0.67421406535499884, -1.3295451145797479, 4.6431014327586748], [-1.3439044084182572, -3.5833403074354111, -3.7600147173139553, 0.16243615877285222, -2.9621745647176745]]], [[[-1.2722184588843275, -2.9981215044253817, 4.6803629201451482, 1.1632965542655072, -1.8185872595393526], [4.6675411367619173, 0.7380653489971678, 4.7725172790156893, 1.7736859266605496, -1.0896700573784566], [3.1870128506488733, -4.1081112342884518, 2.8021153590581633, 3.2071597462500616, 2.8205028249039454], [3.2484160766697485, -0.60683281994801952, 3.0205437213607738, -4.132539273785671, 2.8282216561917926]], [[-1.9541559657960219, 2.2269990695329742, 2.3285190360732333, 0.54364602509288318, 0.60362997084904535], [-0.66791089366165934, 0.57213502118215942, -3.8610605910462126, 1.8029143277443724, -1.755739025801982], [-2.9550056056032248, 2.1978243525186478, 1.879584928071024, 4.0130842797165815, -4.3234165637406505], [3.567903248671481, 0.88806655428511316, 3.1854809987273764, 0.037450748023434954, 1.0299999662474724]], [[3.5457735117543141, 0.016061271508844399, 1.3733452235816399, 4.0855260585753808, 2.1649451486758844], [0.84607379327321564, -1.3992009125541416, -0.11468369096265274, 3.4219364776060814, 3.6267218062771871], [1.3472146396058609, -1.7408023159589203, -1.5706332874103692, 4.5679659572708804, -4.9355809848263918], [1.9077342899420859, 1.0406149566857978, -1.8191521353255933, 3.2264560390267487, 1.538511218162717]], [[0.50797000610872978, -0.097858428682143561, -4.0714361243487049, -4.1127767455791675, -0.82417353198202647], [-3.7586189640621646, 3.1006345452796715, 4.7392382036225449, -1.8112976383976118, 4.6812617899041093], [-3.5347659519001908, -2.4375827621627364, -4.934343979905921, -3.6746477608678574, 3.7489934407265384], [3.4038214566957912, 4.6846500000158997, 4.1560471903245748, -2.9167254952038557, 1.9492679864725222]], [[-2.0711479400955399, 0.8492425219480646, -4.0221176513947734, 4.8415521706562732, -0.71302957270458123], [-0.49520788816709782, 3.1727257502124946, -1.1900633906470723, -4.4191379603157523, -4.2842271479017926], [3.5135325452734953, -4.234005304795355, 3.3552051967134311, -4.9607817251201229, 3.4714635748454157], [3.8464184378178672, -3.0944879571112294, 4.8000144264954265, -4.5448674161271878, 0.67086898702728437]]], [[[0.41865248080849682, 1.5021440907691535, -1.545788488637533, -4.4873567539695713, -0.56935858521296367], [4.7419195487854147, -3.3302046354582773, 3.8548921345732037, -1.9894672166886327, 0.26216935482974879], [-3.3066678311885553, -0.95190439606568766, 2.3832718176056487, 4.6883192200394426, -3.607914575096772], [1.3751899945967549, 0.7493599073656334, -2.8402649637353106, -0.89482610061595125, 2.6040842678679388]], [[-4.9934248370126202, -3.0983009498604219, -2.3215285329840709, -0.40157868314305567, 1.0959070956879291], [-1.6660029908417662, 4.0633975560872493, 3.8590124204486784, 1.0028205305271563, 1.0660114765000595], [2.2036506110237886, -4.4862074378109398, 2.8883567942758237, -2.835619020007778, 3.4950641780160812], [-4.4867045772700767, -1.1167137565495255, 4.2531564606231349, 0.89704497391732652, 3.8279521147335434]], [[-2.1054226042297817, -0.9527589527400151, -3.5124945332278914, 2.6093540465721698, -0.38223311499975932], [-3.3947674073401437, 3.9841161244020054, -4.5291773493117669, 4.9143637887277993, 3.2714118890027439], [3.8011043040796864, -3.5943423837665756, 3.8968810749647815, 1.7138220566308435, -2.525658977865274], [-2.9142525087749869, 1.9931355209095276, -4.4786109260052296, 3.8119778845432801, 2.6889249686800838]], [[1.8880237576320624, -2.4634970963815208, 3.290147375504116, -2.8645223783436293, -0.82936325436677549], [-2.4494207510447752, -2.4779338678641327, 4.5202744123566454, 3.1905081567786748, -2.6293653363165759], [2.3577631849711533, 0.89976366434804333, 0.68450545555706643, -2.2848784319249651, 1.9713924898995838], [-2.7227429419095825, -4.3992868585121911, -4.665472564220881, -1.3938981498370318, 4.1567457350691388]], [[0.13051546715244111, -4.0528856769445101, 2.4588141891033741, 4.4327535347357543, -1.7444127710271129], [1.8588620642390214, 0.022171423572383553, 1.7821447081127122, -4.757252734351507, 0.61275817479185157], [-2.2642670634340134, 2.871203517167813, -2.0140652474087628, 2.7691886715326808, 3.8012694133155165], [2.1189806994945783, -0.040722853643312185, 0.15690413187511165, 2.8049550843673963, 2.194710912548973]]], [[[0.12676759016706374, 0.88332534882539626, -4.7453441763563653, 3.8303430732814601, -1.578450737086329], [-3.0661009422669605, -0.26071042716670956, 3.1774665595828555, 3.8512708587296203, -0.6964639089182576], [-4.3515506316835424, -1.6795569791807918, 2.0085885287397627, 0.781469433687489, 1.8175306087669654], [-0.32667837148430046, -0.96248304542734608, -4.3530491774792566, -0.21803218951960623, 2.8388695013754273]], [[4.3242118058950574, 4.3205152047518549, 0.11749440877178685, -0.031572910433677492, 4.2102316179801829], [2.2954388055513579, -3.9223205830283634, 2.9742589416802474, 2.8830806721792612, -2.4125917077335348], [1.5355985613220913, -2.6231025550098677, 1.3609324486161034, 1.4145877338511204, -0.58423537949167326], [4.6333353685195888, 3.5453872817404939, 0.13508814360663379, -2.832790215241868, -2.795485324749174]], [[1.4867423610831505, -1.0768904054012642, 4.3548671472262974, 0.358661787805441, 0.029913083469560675], [2.6384733085369918, -2.3591576496900144, 0.45296979963413264, 0.76919210206067667, -4.2090702410539924], [-3.51152701632095, -2.4280088207953678, -3.6427347581065082, -1.1141798578192241, -1.9058508345967686], [-1.0153911715627828, 1.8154101989386255, 0.46660389210401476, -4.9578552980845885, -4.0097702557113966]], [[-4.4299843312636238, -0.51401196891696088, -2.5623853573492728, -1.3936124927114246, 0.03557292813913282], [-2.0237250283885921, -3.4968831906676945, -3.614154898121984, 3.0390738869147995, 3.551037371183952], [2.0387596328094872, 3.4892979875887846, 1.1562366634940595, -0.2860293094888835, 1.9650095218252774], [4.3333289252396838, -2.8144341229858227, -3.2470876939361384, -4.8932362239073761, 2.5485987104920227]], [[0.41165462768665595, 0.1978815887597607, -0.83278227857869069, -1.4367127888387978, -2.9416705664198317], [-0.8560891239845656, 1.2681510143621235, -1.7634950442515915, 1.5666138319249212, 1.0798164272136717], [0.87093985402604979, 4.7368966228270217, -3.2925971351486449, 3.9810431054642823, 1.0602625015790679], [2.5373704510295081, -0.57524122275443101, 0.23007686339313604, 1.2462344375863319, -0.5429743983169093]]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-9.9498733268650632, 11.702597566096873, 19.949190035301463, -14.200785532089332, -11.694892172279616], [-13.715950258898864, 15.002030119351522, -8.8635573914306587, -12.429694234397449, -13.568485514390924], [-1.4076578434065257, -1.391497627932476, -13.618267619227712, 1.5462033766632144, 17.344789019136538], [12.123160818066935, 14.676581602663818, 16.608432464210946, -17.952380030674362, -17.847595258543294]], [[1.4022043559874098, -0.98213038963147148, 0.18128827489456203, -1.0348577091692588, -1.0209905343820815], [-0.24559029637309196, 0.26589571085071873, 1.650665986158145, 0.81142197769738289, -0.95210190301045439], [1.6014015518321874, -0.76756646183393429, -1.6009738749671316, 0.27451949455293057, 0.6315743361573174], [1.6487613734317947, 0.25526825447612367, 0.97712584529082158, 0.8073535481792522, 0.45983358523126744]], [[9.015819157345355, -12.617647971227827, 12.836743643062073, 3.9247004002697135, 9.670742599023388], [-21.142334700022975, 16.344198177686629, -7.3036283586095623, 14.43038156827857, -10.812186188692316], [17.125598280517067, -21.13701608234496, -22.499225541038296, -5.9318703420877705, -15.553373292069338], [-22.491435100060588, -3.4809352035799681, -11.126636183422152, 18.509075801496088, -6.8664597157643437]], [[0.32388464541711542, -0.0062066518739683236, -0.0032811403358907875, -0.46647015306033657, 0.40191660294856885], [-0.4750692388102013, 0.38257487977416615, -0.24938468812665096, -0.30327825957557014, 0.05451142281222416], [0.34854668403447081, -0.20088598295531901, 0.31866215321707342, -0.034830417481544702, -0.40572893326998793], [0.46677780198757218, 0.34100474768849059, 0.07717642265442666, -0.40509023232749497, 0.014279252330237203]], [[-6.0969432860339259, -9.8594517444078136, -3.997610279878864, 4.6671975774024856, 2.0712058088729712], [1.6779776521000573, -5.5133737763577653, 9.9907973309253659, -0.92258210978943234, -9.7215468739914339], [-5.9445854423274893, -0.61111682764619002, -1.3497812972493106, -2.6617586634951231, 9.2955217003211548], [-2.6905060706776207, -7.1738724793722541, -7.527575889625278, 0.32519833147232019, -5.9302942968688841]]], [[[0.62618565143306248, 1.4756747586970642, -2.3036736211675355, -0.57257431344093912, 0.89510825743022548], [-2.2973627421480605, -0.36327560580064111, -2.3490319767540053, -0.87300783105881208, 0.53633536753193001], [-1.5686470386219695, 2.02201146464916, -1.3792005761661106, -1.5785633362973288, -1.3882508828948319], [-1.5988696932434814, 0.29868298326951276, -1.4867109690430254, 2.034034940428008, -1.3920500899921107]], [[-6.0378786860065459, 6.8808991969134903, 7.1945718274027559, 1.679737341898087, 1.8650735146104698], [-2.0636863277945054, 1.7677615862695417, -11.929761931034717, 5.570577877467044, -5.4248173777477433], [-9.1302668135977996, 6.790756237439262, 5.807471857097295, 12.399479090594706, -13.358332281478775], [11.023975238389799, 2.7439151294605817, 9.8423811423158938, 0.11571393339302313, 3.1824557259736279]], [[4.3354078607056392, 0.019638074039850385, 1.6791855593823775, 4.9953618669518454, 2.6470726865242362], [1.0344921811655456, -1.7107992416561455, -0.14022344451668742, 4.1839926477731328, 4.4343831255446906], [1.6472357637178001, -2.1284743708316753, -1.9204091513323571, 5.5852398504690921, -6.0347217688415808], [2.3325816523065313, 1.2723571452682414, -2.2242724869947921, 3.9449792344173198, 1.8811335825300224]], [[-1.4607459772136604, 0.28140698134703207, 11.70804155482309, 11.826922877404538, 2.3700379095041009], [10.808487638259448, -8.9163521159899233, -13.628409271702379, 5.2086652893036716, -13.461689165999225], [10.164763829628173, 7.0096446072298662, 14.189465976645085, 10.567015512360701, -10.780807963629048], [-9.7882127689201557, -13.471432485941806, -11.951353704685648, 8.3874933214926539, -5.6054202718843849]], [[-4.9656971423974934, 2.0361081325003827, -9.6432600208154255, 11.607901740465346, -1.7095296975554128], [-1.1872895931570302, 7.6067937429493204, -2.8532459047495369, -10.595139205936182, -10.271682719456697], [8.4238977728362414, -10.151272941877956, 8.0442987847157603, -11.893761502787855, 8.3230349796990772], [9.2220109801794958, -7.4192141026400344, 11.508312593071251, -10.89658285819853, 1.6084472515521107]]], [[[1.5665709072328067, 5.6209274731302923, -5.7842420289935648, -16.791403045235938, -2.1305035471313927], [17.743960803050332, -12.46141354988691, 14.424760739218794, -7.4444595587705971, 0.98102102070702368], [-12.373340328041975, -3.5619655960563841, 8.9180512827217573, 17.543391788221147, -13.500586448723856], [5.1458733345912453, 2.8040570251933086, -10.628090516572607, -3.3483822514329047, 9.7443174017416698]], [[22.095498519268489, 13.709729551240283, 10.272574823350535, 1.7769529908585968, -4.8492997091982204], [7.3719276486123837, -17.98020349044419, -17.075815898908061, -4.4373992341118367, -4.7170140273147263], [-9.750974612044244, 19.851102807143821, -12.780743748857844, 12.547383389728807, -15.465374545680277], [19.853302608791342, 4.9413674901830573, -18.819871199691029, -3.9693510045433045, -16.938376573928178]], [[-0.022461094377006566, -0.010164234350404961, -0.0374720358046184, 0.027837141762103466, -0.0040777438471396822], [-0.036216097885083343, 0.042503394852590158, -0.048318223471485719, 0.052427474009161812, 0.034900115082516968], [0.040550985982224126, -0.03834520601368293, 0.041572752864394581, 0.018283416774402442, -0.026944265038286137], [-0.031089863149811253, 0.021263191984057919, -0.047778770155123468, 0.040666987642184502, 0.028686021216299842]], [[-1.8990509037167951, 2.4778853381879609, -3.3093637310204715, 2.8812528387529821, 0.83420721341455883], [2.4637267788876782, 2.4924064287303764, -4.5466754182161537, -3.2091425618740081, 2.6447223441701526], [-2.3715338798412695, -0.90501880233473053, -0.68850336163410175, 2.2982234378619091, -1.9829065573940163], [2.7386453287573347, 4.4249812273787681, 4.6927216109089906, 1.4020393177981645, -4.1810235240918709]], [[0.52464879095456052, -16.29187418685359, 9.8839924465678592, 17.818874906851761, -7.0122267591175058], [7.4722924097204251, 0.089125149875195542, 7.1639023850355557, -19.123303542465337, 2.4631780628465942], [-9.1019478621746597, 11.541723649559033, -8.0961813952856208, 11.131642250092602, 15.28042189405163], [8.5179227129256478, -0.16369857453928327, 0.63072649456867513, 11.275416820723361, 8.8223445993470708]]], [[[0.17241323806369593, 1.2013873850092287, -6.4540167884714936, 5.2095480500100901, -2.1468089938952666], [-4.1701225919789096, -0.35458533908819601, 4.3215880151607449, 5.2380113760847147, -0.94724209534034676], [-5.9184286300459465, -2.2843209128722308, 2.7318279989485332, 1.0628558556039285, 2.4719751880146621], [-0.44430659097294251, -1.3090476692413393, -5.9204667624478118, -0.29653979970473282, 3.8610711344071649]], [[13.331327648865768, 13.319931213344265, 0.36222935659883632, -0.097337695911619296, 12.979932458532819], [7.0767224614225759, -12.092317209105349, 9.169490821437897, 8.8883927994776535, -7.4378989703708571], [4.7341731804634239, -8.086893331575661, 4.1956863342680482, 4.3611028817614486, -1.8011683094350137], [14.284340054898758, 10.930250787107523, 0.41646995677731424, -8.7333498484988006, -8.6183407461728745]], [[-6.6809225952229943, 4.8391850735877275, -19.569315494516456, -1.6117060392674396, -0.13441938595129754], [-11.856416017537347, 10.601264926644335, -2.0354946818926734, -3.4564918773016906, 18.914152992756218], [15.779627191077617, 10.910659046826799, 16.369230870712258, 5.0067513929069278, 8.5642558100690529], [4.5628280989730552, -8.1578458616401832, -2.0967617304625636, 22.278942439420312, 18.018565559347174]], [[21.522693474438849, 2.4972824330592864, 12.449126336726307, 6.7707450545814831, -0.17282797659656882], [9.8320919907483422, 16.989302760624078, 17.559057148907158, -14.765076086754728, -17.252406135366346], [-9.9051363083493644, -16.952450711361372, -5.6174752395099707, 1.3896485162236794, -9.5468278102318944], [-21.05310159308172, 13.673683336876861, 15.775693071418941, 23.773362492941029, -12.382145112378659]], [[-1.6895676172074594, -0.81217190801154915, 3.4180156748822266, 5.8967475159724483, 12.073595182072049], [3.5136747264476589, -5.204913885301842, 7.2379706663744825, -6.4299046362340633, -4.4319260497603175], [-3.5746270658176433, -19.441800484458259, 13.513914631106024, -16.339526052454257, -4.3516702301496677], [-10.414212931384252, 2.3609814555237039, -0.94431203176853762, -5.1149609589652414, 2.228547667554114]]]])+(1.-msk_ref)*numpy.array([[[[-10.679590609992609, 12.560858512837772, 21.412250747225308, -15.242261970666076, -12.552588010414903], [-14.721869191696948, 16.102269318430849, -9.5136049655094439, -13.341279981156738, -14.563589478074888], [-1.5108945604298798, -1.4935491651909141, -14.617022570325938, 1.6596009336084756, 18.616844642728342], [13.012265602063756, 15.752952617737826, 17.826484173680278, -19.268995986606775, -19.156526366968539]], [[-3.703835013011934, 2.5942359321074306, -0.47886162750519956, 2.7335118453596059, 2.6968825713960864], [0.64871139123091071, -0.70234687223253212, -4.3601308526924196, -2.1433203501971865, 2.5149175648097026], [-4.2300019339128045, 2.0274787508788346, 4.2288722522510094, -0.72512605693874532, -1.6682640654984853], [-4.3550999374253534, -0.67427511161404252, -2.581016741570815, -2.1325738483532293, -1.214621624776687]], [[8.489674121215641, -11.88130746441043, 12.087617154387534, 3.695662794494774, 9.1063775507272648], [-19.908510655809842, 15.390383711066141, -6.8774033268565846, 13.588253582002164, -10.181208792893958], [16.126182878678119, -19.903502421940722, -21.186216081932667, -5.5856983436275094, -14.645709771996662], [-21.178880275390075, -3.277794840348454, -10.477308119653339, 17.42892343970302, -6.4657469649666881]], [[-14.048941358077546, 0.26922205001440075, 0.14232396879199696, 20.233783596620651, -17.433684694716916], [20.606780752065756, -16.594710885719572, 10.817403383175733, 13.155111068849237, -2.3645078371901058], [-15.118691156990245, 8.7137054322671617, -13.822408585685952, 1.5108171986523025, 17.599049758733031], [-20.247128291402468, -14.791549308957981, -3.3476333362512389, 17.571345228081768, -0.6193821827099607]], [[-3.3963569732506791, -5.4922960758473618, -2.2269046821390477, 2.5999042952982796, 1.1537837834438873], [0.93473250976819411, -3.0712743407608487, 5.5654633135509588, -0.51393264378189041, -5.4154749301829783], [-3.3114846691098281, -0.34042811318278376, -0.75190778496779598, -1.4827565509064777, 5.1781537839145146], [-1.4987705520651904, -3.996270045079688, -4.1933037039997547, 0.18115464897230057, -3.3035236582791621]]], [[[5.5247905777636621, 13.019771347419189, -20.325145246209161, -5.0517816317388942, 7.8974752222625755], [-20.269464818424833, -3.2051543171990322, -20.725338727157631, -7.7024847635992213, 4.7320480408334129], [-13.840059028790998, 17.840060471615505, -12.16858663338963, -13.927549803831507, -12.248436833128766], [-14.106711318101373, 2.6352582942922056, -13.117143030711587, 17.946142726203941, -12.281956961745546]], [[0.25098432154973449, -0.28602724672026858, -0.29906608311400518, -0.069823817115624104, -0.077527925791184088], [0.085783921772621938, -0.073482834860498614, 0.49589986156064231, -0.23155942375197805, 0.22550040832068333], [0.37952962305947457, -0.28228015760127728, -0.24140670254784266, -0.51542520295010408, 0.55528309511636864], [-0.45824785324037837, -0.11405987317264384, -0.40913100144976605, -0.0048100309027089309, -0.13228925799664415]], [[-6.1071075025635242, -0.027663332530182737, -2.3653983794344389, -7.036757074734818, -3.7288204439136541], [-1.4572458149095846, 2.4099312497896546, 0.19752689425252346, -5.8938152328134166, -6.2465298134250453], [-2.3203920402207725, 2.9982927135728792, 2.7051999518644121, -7.8676934882119296, 8.5008598439864667], [-3.2858100937300607, -1.7923162289379748, 3.1332395081418212, -5.5571270464149585, -2.6498741028078139]], [[-2.0219900670151958, 0.38952845323441376, 16.206475388150775, 16.371032988967741, 3.2806478239342822], [14.961297162498022, -12.342170160845507, -18.864681886143831, 7.209925368042537, -18.633905007095684], [14.070243435664279, 9.7028723612581302, 19.641306365774689, 14.627047232910918, -14.922982473952626], [-13.549014887817812, -18.64739188055005, -16.54327231101113, 11.610114590604015, -7.7591205370391565]], [[-8.369111665061455, 3.4316261814565978, -16.252606152791195, 19.563783911338511, -2.8812157735967547], [-2.0010401156078341, 12.820376358493922, -4.8088179564650346, -17.85688908899883, -17.311740366370884], [14.197511264629917, -17.108803528858644, 13.55774080976267, -20.04556767760943, 14.02751862222576], [15.542639322599056, -12.504232439357823, 19.395937852412928, -18.364937710201996, 2.7108529315385423]]], [[[-0.41714711859001807, -1.4967427828476334, 1.5402302471479317, 4.4712214206611058, 0.56731132419759622], [-4.7248688758310307, 3.3182301113175381, -3.8410309746811357, 1.9823136252967426, -0.26122666403082773], [3.2947779390935237, 0.94848160274872506, -2.3747022103178566, -4.671461279514677, 3.5949414803754394], [-1.3702451796105926, -0.74666540979473051, 2.8300521314630189, 0.89160854556557978, -2.5947206781359182]], [[-5.5377189633894606, -3.4360224664149226, -2.5745801730835409, -0.44535162969724429, 1.2153633435954279], [-1.8476009265351216, 4.5063166937738126, 4.2796531355139127, 1.1121301411435418, 1.1822090371419076], [2.4438533022261177, -4.9752137686981461, 3.2031939429902736, -3.144707637061249, 3.8760337460976051], [-4.9757650974354695, -1.2384379755722674, 4.7167597300505051, 0.99482481968169612, 4.2452071891781546]], [[2.5831549032025101, 1.1689453487371491, 4.309499412493909, -3.2014312404801091, 0.46896397103098542], [4.1650593357768049, -4.8881346106009689, 5.5568733107757451, -6.0294607322395875, -4.0137137525242155], [-4.6635963729759586, 4.4099190033213445, -4.7811054349703053, -2.1026979761253726, 3.0987452872326471], [3.5755128885235128, -2.4453892454850026, 5.4948330799739677, -4.6769372303570229, -3.2990572549355757]], [[-3.9085325006220768, 5.0998608611109022, -6.8111685019910375, 5.9300518699814013, 1.7169239642300926], [5.0707204157027075, 5.1297474503636833, -9.357742126394033, -6.6048982561052476, 5.4432365225592703], [-4.8809735576568247, -1.8626648688964671, -1.4170435139298168, 4.7300896374046104, -4.0811200532342031], [5.6365441569654582, 9.1072771709292244, 9.6583271658023246, 2.8856078739326665, -8.6051755104588583]], [[-0.1559398000237103, 4.8423853185424228, -2.9377896835506987, -5.2962512018113692, 2.0842233078402348], [-2.2209672531036313, -0.026490403272128978, -2.1293054031045702, 5.683962646285849, -0.73212309103816331], [2.7053448972432368, -3.4305121995354875, 2.4064039210677781, -3.3086249246026891, -4.5417543612384126], [-2.5317568388070244, 0.048655640531216154, -0.18746896043302519, -3.3513586126988031, -2.6222392508698187]]], [[[0.324140787498234, 2.2586354588588464, -12.133697550353444, 9.7940681725165284, -4.0360494687329229], [-7.839924799909209, -0.66662845810548865, 8.124683221595923, 9.8475798693691772, -1.7808365655870646], [-11.126779697661362, -4.2945749868901748, 5.1358984311968339, 1.9981930500330252, 4.6473692688443622], [-0.83530644111805197, -2.4610392284602716, -11.130611432678844, -0.55750153109096001, 7.258901069018119]], [[-6.5973480009760408, -6.5917081837660545, -0.17925844931547316, 0.048170045058998405, -6.4234511155677128], [-3.5020968664213026, 5.9841920375760038, -4.5377567436749882, -4.398648207593399, 3.6808342871846564], [-2.3428265204375252, 4.0020057236922213, -2.0763425503581301, -2.1582031540187705, 0.89135414406257474], [-7.0689705323134984, -5.4091137866996917, -0.2061007957482133, 4.321921243134403, 4.265006051241377]], [[-3.8404022915365568, 2.7817142289695247, -11.249051758611845, -0.92645880539189029, -0.077268447654731356], [-6.815437028969404, 6.0939371078152895, -1.1700657101371759, -1.9868991351221963, 10.872443956736113], [9.070620945008022, 6.2717864798445744, 9.4095434950142582, 2.8780365658213145, 4.923000853476684], [2.6228556366919906, -4.6893837631716, -1.2052839169596479, 12.806629679939439, 10.357632419480062]], [[-19.083075237464602, -2.21421304056887, -11.038005759204548, -6.0032745178348801, 0.15323775736749803], [-8.7176148015096704, -15.063548769957439, -15.56872094422059, 13.091440352717106, 15.296829121294529], [8.7823794746722097, 15.030873936318693, 4.9807289579009435, -1.2321305054231335, 8.4646855932851413], [18.666712052555454, -12.123758032459289, -13.98750291930577, -21.078628733857435, 10.978616921778523]], [[-0.038232126398281691, -0.018378109717533991, 0.077344052988771048, 0.1334336631889472, 0.2732055304454763], [0.079508659432065767, -0.11777861005878712, 0.16378332927643108, -0.14549812879801022, -0.1002871712245111], [-0.080887910264841448, -0.43993585457119566, 0.30579758220389308, -0.36973650475022241, -0.098471114495971609], [-0.23565644906794714, 0.053425113332115776, -0.021368222609323299, -0.11574312381096212, 0.05042835530391377]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.306834532285821, 3.8209742544625911], [2.3576450094058341, 2.7620368134173487]], [[-1.5778910201163985, -1.9885303581077443], [2.1041598479267751, 2.817219616078539]], [[4.7197856262239597, -2.2327195830539228], [2.9655030963192273, -0.38200028743116121]], [[1.8512019974211311, -2.3077360593578735], [3.2228051869569398, 2.7684938068521987]], [[-0.16719334977478439, 2.8411142683000268], [-1.1089590236248101, 0.20481994526078484]], [[2.4631248161160535, 1.5791386294089058], [3.5264826659659789, 1.2637455244324425]]])+(1.-msk_arg0)*numpy.array([[[-2.1663885640743672, -2.894770192576126], [-0.01150928360812209, -1.1458565914147609]], [[1.3101810333618378, -3.9694450572598052], [3.0489862360588802, 1.6829798479975846]], [[-2.199936728327907, 1.4996852723381968], [1.9474887000068506, -4.3989338982815536]], [[-0.0083790292224961149, -4.9105061222406938], [-2.8736535242744186, 2.7840687686899743]], [[-1.418616832876316, -3.3332771221767143], [4.2792083966727805, 3.6574302273258059]], [[4.1655512147376612, 0.80465531859313622], [2.3755222746226403, -4.3071937889516434]]])
      arg1=Data(numpy.array([[[3.9107822819826392, -4.8496466411505619], [-3.2957847841646037, -2.4976749946253642]], [[-4.9969685246960669, 4.6529934120275378], [4.6420290199702059, -0.34909413997207039]], [[4.7985548357386474, -0.12546874849058121], [3.1053658312077399, -3.9168006886491549]], [[2.1600470077163632, 3.8165817534061013], [-3.8086126669927789, -1.3273628078516397]], [[-0.80993198990605109, -2.9779326736918144], [-4.1100007498633158, -0.673125530897563]], [[4.9776405071086813, -1.2011375193820184], [3.7504797699166463, -0.23050644187154035]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.932309898311736, -18.530374959077278], [-7.7702905484613618, -6.8986702831072346]], [[7.8846717629222116, -9.2526186558920944], [9.7675710767321853, -0.98347485898738396]], [[22.648150140366742, 0.28013653181618797], [9.2089719876504841, 1.4962189888745474]], [[3.9986833352080691, -8.8076633358225589], [-12.274416658294232, -3.6747957129832098]], [[0.13541524248214953, -8.4606470092626616], [4.5578224186656602, -0.13786953439207558]], [[12.260549858763889, -1.8967626560885336], [13.226001897667127, -0.29130148426800606]]])+(1.-msk_ref)*numpy.array([[[-8.4722740122718463, 14.038612541329575], [0.03793212179228387, 2.861977355803301]], [[-6.5469333853628706, -18.469801700835145], [14.15348258927505, -0.58751840262704258]], [[-10.556517026036937, -0.18816363425002999], [6.047664865664454, 17.229747322111301]], [[-0.018099096999620697, -18.741348066132783], [10.944633213099992, -3.6954693380603816]], [[1.1489831543657345, 9.9262748525994589], [-17.587549719146526, -2.4619096634894775]], [[20.734616460913955, -0.96650169333250735], [8.909348233958589, 0.99283591474244159]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.2411017822618371, -3.5004834267883869], [4.1157122889625537, 3.0546233579954514]], [[3.5626038356124692, 1.6788029581542094], [1.4169266505234841, -1.2435196360336898]]], [[[1.0165287564943153, -0.57162187440316181], [-4.1747900918930494, 3.9324153411995262]], [[1.4614647823687488, 3.8301698593659665], [2.5389777620723386, 1.1445390649292388]]], [[[0.44074753844405734, -0.66571536647716556], [-4.9118824830375152, -4.9750018357429475]], [[0.78470410821869141, -4.2238862856454071], [-2.9256412620131487, 1.4631828905127966]]], [[[1.4836764379384526, 3.7023837404467272], [1.9678626301276747, -0.92234486162065288]], [[-0.20277513645988066, -4.7306057489996336], [-2.6411521835201923, 0.8665323259162534]]], [[[-2.8727269845657455, -0.08359307854943232], [-0.87640284807053703, -3.6741243221771205]], [[-2.052084364900304, 1.0389485952042428], [-3.2947519313122386, 1.7941625603709852]]], [[[1.1504721555520518, -1.3533562993883841], [-3.1002636944268769, -1.9608418867191557]], [[-0.76604636304644114, -1.032467625757322], [-0.81500820871513469, 3.3818938905585298]]]])+(1.-msk_arg0)*numpy.array([[[[4.6905184461935399, 0.44236525986372754], [4.098958457482782, -1.7016286101483722]], [[3.3704126727079711, 3.2454052618379503], [2.5453677866467315, 1.7356538669228581]]], [[[0.15659783472192235, -3.1004837292133383], [1.6271393886013694, -4.2391448686813842]], [[-3.1090296951974752, 4.2878631031449483], [3.8176577779912826, -4.6583688935048775]]], [[[-2.8138777142242475, 2.2896982600694642], [4.0904983249291096, 3.7207062185278534]], [[-1.6331611694562511, 4.4254595567641726], [-3.681780164739592, 0.19331282798654481]]], [[[2.6788991738135053, -4.4544770762936849], [-1.4709331990981656, -4.8858330281271689]], [[1.8153569418247155, -4.1722550232105746], [4.1842543185937657, 2.4800561205396505]]], [[[1.5719027358049953, 2.2034494585298212], [-4.4384196229861947, 2.0586303548153806]], [[2.9246927677410364, -1.6036076055827051], [-2.7232634192618899, -4.2335375279740717]]], [[[-0.85860309056033035, 4.6382563744421308], [4.6193754401209866, 3.0791739995610747]], [[-2.8596102751421224, 0.41835991998032185], [0.81199539171994406, 4.9443585150756437]]]])
      arg1=Data(numpy.array([[[-0.22464861303353789, 0.19651448658172033], [-4.5541741247659093, -2.6950176506589916]], [[-3.448090391278873, -2.8119709672704696], [-2.502317980548999, 4.7909943058832312]], [[0.55759126475778054, 2.6365133614529501], [1.9418836853244237, 2.1836470912365167]], [[4.5378432039081158, 3.0104767205146707], [-3.4666175239739605, -1.1219088555555343]], [[1.5352148461968751, -0.15639189238779139], [-2.2077226753840273, 1.3516332013761154]], [[-1.9649000365852256, -0.50907146088890798], [4.0951139565843668, 2.9935773381842532]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.95275763311918726, 0.78637874677489694], [0.80879708738355316, 0.60027774089700658]], [[-16.224718204938089, -7.6455609926063666], [-3.8186423328499139, 3.351307368051839]]], [[[-3.5050830377267097, 1.9710038925743609], [11.73938853285167, -11.057837770702065]], [[-3.6570496028604498, -9.5843029076482882], [12.164228000852724, 5.4834801429369016]]], [[[0.24575697739990043, -0.37119707316269213], [-12.950243796415103, -13.116658813189236]], [[1.5238041055569278, -8.2022958667603945], [-6.3885680317965434, 3.1950750628153068]]], [[[6.7326910406976088, 16.800836894846089], [5.9242046371701367, -2.776697734195301]], [[0.70294384147803346, 16.399200788494092], [2.9631320235611396, -0.97217029007057909]]], [[[-4.4102531157757134, -0.12833333522839005], [0.13706229990380131, 0.57460325561329118]], [[4.5304331841914314, -2.2937103721907879], [-4.4532961006597001, 2.4250496852634025]]], [[[-2.2605627805345101, 2.6592098421810815], [1.5782557680627332, 0.99820864384428321]], [[-3.1370471527021757, -4.2280725839603344], [-2.4397901040437691, 10.123960910919791]]]])+(1.-msk_ref)*numpy.array([[[[-1.053718463345604, -0.099376742082606967], [0.80550471679202917, -0.3343946726760737]], [[-15.349446183829754, -14.780140667841524], [-6.8598111124317516, -4.677617806791635]]], [[[-0.53996348919973747, 10.690748155016999], [-4.5754687204492734, 11.920352296785639]], [[7.7797809083534162, -10.729596941132231], [18.290376676167064, -22.318218843485436]]], [[[-1.5689936335480308, 1.2767157487458221], [10.784653488676508, 9.8096916591897649]], [[-3.1714090304724505, 8.5937277133434016], [-8.0397085473059136, 0.42212699453152369]]], [[[12.156424409844682, -20.21371852762379], [-4.4282101533171989, -14.708686591498543]], [[-6.2931481867973362, 14.463612377950161], [-4.6943519739268336, -2.7823969239081374]]], [[[2.4132084167853129, 3.3827683215794471], [0.6941328440499186, -0.32195309691652779]], [[-6.4569105418735564, 3.5403208732632239], [-3.6808532535674146, -5.72218988208152]]], [[[1.6870692440541808, -9.1137101198329979], [-2.3515922036967329, -1.5675196062876979]], [[-11.710429948126567, 1.7132315471869348], [2.43077100336287, 14.801319602588793]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.0566771903127776, 1.4189803342697527], [2.7276266029149587, -2.9445976587880471]], [[-2.9176498444513808, 1.9975274369583671], [0.81828621263678514, -1.2492133427022836]], [[-1.1043190794158688, 1.3042240604607169], [2.5779864195683935, -1.7501109059040152]], [[4.4423740392119768, 1.5910420516545463], [-1.1606923225440271, 4.1183769983278964]], [[2.4728416689308581, 2.904964829574233], [0.56955617706179762, -2.2074259898161221]], [[1.4740375984407539, -4.3809675309167186], [2.0581179475679896, -0.22288749472120895]]])+(1.-msk_arg0)*numpy.array([[[-2.2423308342138251, -1.1481092124167658], [-2.5417657674531502, -1.2241467479004866]], [[-4.3792552704821386, -0.61521830062593175], [-2.9148298738241918, 0.48874736966585175]], [[3.5420649072240433, 4.2390761090068629], [4.2784583047947677, -4.2382495410185053]], [[-1.0991740576227746, 1.454478352476217], [3.9570814708999045, 2.797250776610066]], [[-1.446634825356087, 0.8679638961210534], [-4.6642509404156431, 1.451040969865419]], [[3.2052744594432721, -2.0888583014051365], [2.1917319170660177, 1.9936117042468524]]])
      arg1=Data(numpy.array([[[[2.1713535916304982, -0.11163075461808525], [3.9604467210624019, 1.2169323837732806]], [[2.85312205053099, -1.651250181000373], [-3.0270750235332269, -3.3511710433026476]]], [[[4.2459799672174086, 3.673155953960018], [4.5703742803420191, 0.56558068980899812]], [[1.9137146837954511, 0.94928719615678947], [-2.8656324288487376, 2.6084938439277519]]], [[[4.8737076402844774, -0.75392216029833925], [3.8120337217526217, -3.8300947770740734]], [[-1.2252554889502454, 0.45218544318195342], [-4.256603914799264, 2.627694022763837]]], [[[2.5026140770903638, 2.0367512302913973], [2.8785752358589285, 2.7015704515568935]], [[-1.2365938357537685, 2.436980897138036], [4.1812958756074323, 3.2846273142443252]]], [[[-1.3421802482173151, -1.7709144437105095], [-0.11551961069982664, -1.1727527940426219]], [[-3.2631065523439897, 1.0187735330749828], [-1.8140305174166746, 0.54649694552032102]]], [[[1.9412438859918861, -0.0069577530043076763], [-1.1922740713490643, -3.2352598934753587]], [[0.58704403514365566, 1.2496422831983107], [1.9071174065245922, -4.9497333727224611]]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.808480587271168, 0.45284993599658918], [5.6197960121106734, 1.7268031207102967]], [[7.7822516063916058, -4.5039939217647582], [8.9135180272717118, 9.8678504083072731]]], [[[-12.38828279089555, -10.716982897717109], [9.1294480221520349, 1.1297629457073133]], [[1.5659663406703825, 0.77678862444773222], [3.5797862653981953, -3.2585653141913156]]], [[[-5.3821283346610409, 0.83257062601188514], [4.9717460991973832, -4.9953017621049325]], [[-3.1586920110153645, 1.1657279316495912], [7.4495289334039176, -4.5987559666177846]]], [[[11.117547806232674, 9.048010789779557], [4.5799342491029593, 4.2983121939343789]], [[1.4353049712646686, -2.8285850174945737], [17.220152757304952, 13.527333579063365]]], [[[-3.3189992450077388, -4.3791910285188589], [-0.33558040620910362, -3.4068056204787305]], [[-1.8585224932983453, 0.58024875878992799], [4.004338110465155, -1.2063515608966819]]], [[[2.8614664756952766, -0.010255989529013628], [5.2233139945341334, 14.173568547392629]], [[1.2082058647418914, 2.5719112110902835], [-0.42507262087947573, 1.1032336709840693]]]])+(1.-msk_ref)*numpy.array([[[[-4.8688931104940005, 0.25031308312668987], [-4.5470253657375164, -1.3971712806983985]], [[-7.2519679584054071, 4.1970911835675659], [3.7055840457089886, 4.1023251343172165]]], [[[-18.594230149798413, -16.085687570682257], [-2.8117778979764827, -0.34795559085113403]], [[-5.5781527303029979, -2.7670106781966153], [-1.4005703120289865, 1.2748945050092555]]], [[[17.262988800721349, -2.6704412267712874], [16.159501076610052, -16.23606326472667]], [[-5.2422045221945508, 1.9346565646891312], [18.040549588195553, -11.136822985915902]]], [[[-2.7508084697792903, -2.2387441141675732], [4.1868253665309325, 3.9293757394789002]], [[-4.8933025544902771, 9.6433319530019475], [11.696133135279355, 9.1879263056445737]]], [[[1.9416446889762451, 2.5618665069977249], [-0.10026685138140885, -1.0179070843040854]], [[15.219947805446902, -4.7518154097155554], [-2.6322326013577593, 0.7929894578562956]]], [[[6.2222194473201995, -0.022301507999822088], [2.4904915914875931, 6.7579994856891004]], [[1.2866431485475751, 2.7388808770009887], [3.8020515830203294, -9.867846384760746]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.7085086277348154, 0.72857652482800273, -4.0786447104860173, -3.5607616809227127], [3.8999148268537951, 3.8163984301620886, 1.3970514523691264, 0.57407792830467663], [1.1271544656191281, -2.481557445918583, 1.1213749430595543, -0.41129786587545958]], [[-1.1105763638374491, -0.88814218116518617, -3.4311792018696954, -1.3996642261342327], [1.4525774143975765, 3.7396302394131595, -2.1627443205663646, -1.0987427056834296], [0.37678311102784168, -3.2941855480564852, -3.7361412586426637, -0.11569599286376597]]], [[[2.0433785635852564, -4.1871114266869514, 4.8044601613066931, -3.5574932318253163], [0.70362054129357166, 4.7923299875095555, -1.5996296252452824, 4.2866039306859456], [-3.5699699703926138, -4.8543999343877964, -0.004924068137689197, -1.4195170039616811]], [[-2.4397375804737531, -0.79495034723378843, 3.1681501500349825, 2.74720929492418], [-3.9645320968459732, -2.6311695510389752, 0.79993054638467953, -4.5871345671483787], [0.70796606769243375, 4.2035713686793432, -3.0382141398384634, -3.0548472242198841]]], [[[0.63477536311611704, 4.1316119169672287, -3.6202669685260824, 0.22429257515216428], [3.5299490183764046, 2.3572243052378354, 2.6977998140618915, -1.8826355936930703], [2.5000932180651878, 3.2046784771724006, -4.937269185149848, 4.2799953123406862]], [[-2.9416577617796236, -3.6143923404557796, 4.0735414029542021, 2.0448507975963839], [3.4163225205256591, -1.5075079426373295, 0.65538742165793984, 0.81001711995531966], [2.379432221005251, -4.4233808653391717, 0.3766723916497563, 0.18106085097322477]]]])+(1.-msk_arg0)*numpy.array([[[[-3.2073436989832107, -4.8449966701319882, 0.25483832475643098, -0.49510209762480173], [1.6147750327974357, 3.9840899356180532, 4.644643642505109, -1.2662975722770478], [2.5449910977835533, 2.358583994964504, 3.9610626593045382, -0.73015426519264981]], [[-2.0277819924873777, -1.0815299045900129, -0.73379778122156214, -3.9845181712355893], [-2.1900442969591563, 3.1501620254141542, 1.5628048211398093, -0.29380417680174542], [0.82302742374891835, -0.085722249020117935, -1.0395450849098395, 2.589623746971748]]], [[[-1.6938429687211265, 0.97766228811663947, 2.9298102913261612, 0.66467351869895719], [-3.8909343606639202, -4.6540564922241749, -1.2881701782016117, 2.5248585326374258], [2.0988634567811006, -1.242642374945151, -2.9681890819969317, -0.089929423725538804]], [[-0.16441501373957657, -1.6412983037699513, -3.7688857346640283, 3.6792207791345195], [3.8729907748716901, 0.82391344892760454, -0.55922310816266663, -1.0271528830419463], [-1.1623490699356722, 2.7870519735523516, 2.34958823398037, -1.4399238019580562]]], [[[2.8881080970547393, 0.92391062375640853, -1.9767191479749426, -1.022964611880639], [-3.1890993388585587, -2.6694547636771748, -0.22248143248098806, -3.9376923614051531], [-2.6316492012525892, 4.9445367218783041, 1.953204811963368, -0.57089819016926224]], [[-4.2571243593194117, -0.71726776985891405, 1.8526975198632378, 1.3683987664671484], [0.63440426986385035, 4.1815338652648695, 4.1248597916271521, -0.61350292718373911], [2.0339492786057791, 0.92487739750263476, -2.0414054303804408, 2.2530288949888844]]]])
      arg1=Data(numpy.array([[[[0.36018966362490978, 1.6029289505386739, -2.1914005425066807, 2.8735483948910616], [2.0832671064697053, 0.11717702449263978, -3.8880992992828975, 1.537745510486265], [-4.6874369688418032, 0.58839879445042165, -4.4961898257374378, -0.32429016105823294]], [[1.6207511897425793, -1.7306258062623758, -1.5970317863647465, 0.34782172788276355], [0.23606044067242848, -1.3608704405348515, 3.7910625698965603, -4.5446538634130409], [-2.0555093686700818, -4.8087523148749076, -2.1555428415001412, -3.4478002453847645]]], [[[-2.0569032385363926, -3.0793704553137466, 1.5632543023152836, 2.3890473920570132], [1.0923909168937094, 0.28485882563383846, 4.3510253559474492, 4.410428026959865], [0.61022327554640654, -3.983014305829462, 3.5682154534438943, 4.3732159160623745]], [[3.5432887639518693, 0.82438968531777146, 3.2456454298611828, 4.5525145753384137], [2.0790272988806988, -4.7971027358386378, -2.1039087561183578, -3.2695830850969045], [3.2236871918177528, -4.1223548518151141, 2.6212401064701085, 4.8625838626241489]]], [[[-1.7288212135377523, 1.0555900210584106, -2.6865899704509988, 3.684097155265599], [-3.878579709598815, -0.48374924374589057, -0.672325080157405, 3.2907095053311046], [-1.8646939378776861, -4.1055496266238016, 0.033433185503321639, -3.6875129843482091]], [[4.5455608786950972, 4.4813365293171366, 1.4409493983461674, 3.5949483162338076], [2.2818502257663651, -4.2524158029530685, -2.0321973168879079, 3.1020642291068352], [3.9808746899567815, 0.89660198324962526, 3.0701736085522491, -2.5853087961041066]]]]),self.functionspace)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.61538714792405935, 1.1678564043296644, 8.9379442312510626, -10.23202101280506], [8.124564276818008, 0.44719421232477508, -5.4318747730185546, 0.88278575691977235], [-5.2834655117382283, -1.4601454095379616, -5.0419146098212666, 0.13337985116766027]], [[-1.7999679629895331, 1.5370417783546253, 5.4797022500995247, -0.48683362958969989], [0.34289606455350868, -5.089152251347639, -8.1990990419555132, 4.9934052822810964], [-0.77448121467438824, 15.840922379844089, 8.0534125449005209, 0.39889667258572631]]], [[[-4.2030319849943556, 12.893667220446389, 7.5105930174650695, -8.4990199277527481], [0.76862868824893282, 1.3651374922917998, -6.9600290595669394, 18.905758116373615], [-2.1784787689352889, 19.335144384884195, -0.01757013602271329, -6.2078543548464005]], [[-8.6446947558837692, -0.65534886659931602, 10.282692055575062, 12.506710356647496], [-8.2423704566315159, 12.621990651744389, -1.6829808808252695, 14.998017589811649], [2.2822611446616787, -17.32861282662639, -7.9638887553891635, -14.854450815273783]]], [[[-1.0974131135862728, 4.3612883104366169, 9.7261729279972151, 0.82631563806528396], [-13.691188638592978, -1.1403054749982351, -1.8137984762377934, -6.1952068432404541], [-4.6619086678552719, -13.156966525404483, -0.16506863654694853, -15.78253828720575]], [[-13.371484440455239, -16.197308426568547, 5.8697670337250605, 7.351132931768479], [7.7955363147521926, 6.410550598348248, -1.3318765598153492, 2.5127251327775375], [9.4722215050674539, -3.9660120565315449, 1.1564496359133383, -0.4680982106511728]]]])+(1.-msk_ref)*numpy.array([[[[-1.1552520480662365, -7.7661854278180371, -0.55845284312273658, -1.4226998379369467], [3.3640077101754375, 0.46684380396679626, -18.05883569184288, -1.9472434067086868], [-11.929485357123912, 1.3877879792471735, -17.809689627873542, 0.23678184425668014]], [[-3.2865300768624954, 1.8717235631279614, 1.1718983813747588, -1.3859019950994318], [-0.51698282183231725, -4.2869623832815202, 5.9246908614770195, 1.3352382871889406], [-1.691740580188303, 0.41221706341177539, 2.2407839661940612, -8.9285053902634068]]], [[[3.4840710879345824, -3.0105843653008155, 4.5800385428832158, 1.5879365364171021], [-4.2504213538188989, -1.3257490668085201, -5.6048611081305566, 11.135706836452862], [1.280775333521617, 4.9494623564364346, -10.591138151124898, -0.39328078715884363]], [[-0.58256987080843381, -1.3530693921575023, -12.23246676038131, 16.749706222897856], [8.0520535492713545, -3.9523974599448595, 1.1765543938871579, 3.3583616922024668], [-3.7470498091729039, -11.489217225434425, 6.1588349125996196, -7.0017502428096554]]], [[[-4.9930225451783832, 0.97527083478711651, 5.3106338373479245, -3.76870101656684], [12.369175987591802, 1.2913467231426983, 0.1495798469263146, -12.95780168274562], [4.9072203121963573, -20.300040892335147, 0.065301858804351742, 2.1051944889900476]], [[-19.351017943462249, -3.2143182583705885, 2.6696433765643692, 4.9193228414474941], [1.4476155264159729, -17.781620689235758, -8.3825290010835136, -1.9031254848690125], [8.0968972038576013, 0.8292469088636143, -6.2674690767092747, -5.8247754200914779]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.93774598126)+(1.-msk_arg0)*(-2.99055872847)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.53293792831)+(1.-msk_arg1)*(-4.49791161981)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(7.44112821967)+(1.-msk_ref)*(13.4512688545)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.5208250868812616, -3.4389968009321423])+(1.-msk_arg0)*numpy.array([1.28569538036332, 1.964894991402689])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.54318998369)+(1.-msk_arg1)*(4.57248338084)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.4333020083963142, 5.30702541713719])+(1.-msk_ref)*numpy.array([5.8788207595344009, 8.9844496932850966])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.37886151919)+(1.-msk_arg0)*(1.60853362479)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.6624250583031959, 1.26480292809463])+(1.-msk_arg1)*numpy.array([-1.23093320940169, 1.3803106874815629])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-11.091263557308602, -3.0087910150042201])+(1.-msk_ref)*numpy.array([-1.9799974571888461, 2.2202761534661137])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.5352313543415486, -3.5564508372596517, -2.8170388470301289, 0.074872627383681056, -0.18093772185077839], [-1.0320376455164784, 1.7194511367007292, -2.5497351937091119, -0.40713267729866409, -2.1719183101359461], [-4.9280619863646411, -4.3629746440785198, 3.7761516078396618, -0.42290687904907731, 4.1442551737127591], [-0.93066883871365924, 4.7174197673145528, 2.5531595562348519, -3.3654139790581414, -3.0887518250892465]])+(1.-msk_arg0)*numpy.array([[-2.8563606417016709, 1.8674519005038936, 1.9949048361463664, 0.9788416065765162, -4.0178942387140815], [-0.57290689806770878, 4.6000609472041614, -4.9558453506707281, 2.2396767839418716, 1.5850210591409368], [-1.9171763532256514, 1.8144072733946404, -1.252624256540209, -3.7846103784937646, -3.9308384962251983], [-4.4142203232839874, 4.7881001003505705, 1.3625556947614017, 3.5638039933783414, -0.083117021129619495]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.70124298281)+(1.-msk_arg1)*(1.36219878813)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-6.8482759057220637, 9.6068378678662913, 7.6095064178511569, -0.2022491593249249, 0.48875675147554681], [2.7877844479500165, -4.6446553173023135, 6.8874543000375379, 1.0997642876267988, 5.8668790944973281], [13.311892859533925, 11.785454641507213, -10.200303032714105, 1.1423742394145757, -11.194640206977175], [2.5139626698977966, -12.742897043440761, -6.8967043352808002, 9.0908008951908617, 8.3434691931725133]])+(1.-msk_ref)*numpy.array([[-3.8909310045839876, 2.5438407157546856, 2.7174569502302828, 1.3333768502482934, -5.4731706628048418], [-0.78041308225829631, 6.2661974475987918, -6.7508465308359735, 3.0508850008851747, 2.1591137659199506], [-2.6115753049926171, 2.4715833889897323, -1.7063232442394478, -5.1553916711227856, -5.3545834358868571], [-6.0130455749096843, 6.5223441541355411, 1.8560717161615805, 4.8546094809075191, -0.11322190545561939]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.32688553383)+(1.-msk_arg0)*(-1.77146319575)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.8752300170986542, -1.1343729745980848, 1.9947008644549129, 4.8011283224137244, 0.099789650569065635], [-1.4891775367223792, -0.21274447710553623, -3.1205941744176267, 3.2904197245780296, 4.1807679502729194], [-4.921598872936511, 0.43215939470138576, 0.58194793015193724, 3.7199922431379697, 1.5397179796150144], [-3.6630651436893569, 4.6103243978544626, 1.3189336052515364, -0.38461428978817125, -4.4487660506842612]])+(1.-msk_arg1)*numpy.array([[-0.79216641385536235, 0.14126985687280413, 1.4269665081343961, -4.7373390944395135, -2.6497296655172367], [-1.5580860535097418, -0.010903261552322796, 0.15107921468708518, 3.116520575558031, 2.9210787772597744], [-3.4213012673148633, 1.9168496146445921, 1.6523864291103338, -1.5205697784311614, -4.7754164452702872], [-3.4762400662262696, -3.138890470983986, 0.84413334080682922, 1.7818325539387123, 2.7176224220175369]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.6903311332323776, -2.6395560645646197, 4.6414405858264747, 11.171676039505588, 0.23219909433551678], [-3.4651456675099759, -0.49503204617996827, -7.261265441419285, 7.6564300573629405, 9.7281684638072221], [-11.451997220770089, 1.0055854438411476, 1.3541262201152293, 8.6559961365327371, 3.5827474929504164], [-8.523533292342627, 10.727697147649872, 3.069007526147431, -0.89495342701396507, -10.351769366749387]])+(1.-msk_ref)*numpy.array([[1.4032936470517239, -0.25025435211863017, -2.5278186507238085, 8.3920218515733964, 4.6938985811430047], [2.7600920995993228, 0.019314726553544007, -0.26763126846054308, -5.5208014983895577, -5.1745835457935723], [6.0607092766111217, -3.3956285441248673, -2.9271417443208971, 2.6936333990560941, 8.4594744771616615], [6.1580313369012272, 5.560428944829348, -1.4953511455423245, -3.1564507902864518, -4.8141681005411048]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.048752785272911, 2.2949012898246792], [-1.8347545174858437, 3.9303737738736864]], [[4.1065018308113306, 3.017772837164971], [-2.1284129757967616, -2.8591131896334043]], [[4.8390090222814734, 3.3982524459917034], [2.9283817729433217, 0.45007010686916082]], [[1.0906345579886443, -4.2693015285938323], [-4.4977527821126175, -1.7534151596542111]], [[4.9947633200964567, 3.945630659961644], [-4.8745910840519606, -4.4517154581141583]], [[-0.97118876049024561, -0.54994959698069135], [4.0369580691379028, 1.745967331297634]]])+(1.-msk_arg0)*numpy.array([[[4.1596533021381141, -4.4334528324889755], [-2.0201410048218293, -1.8058760034006349]], [[2.1312989915784355, 2.4029367228043519], [4.7478819159842995, 2.1803025580154163]], [[-0.28117495050288532, 1.3994282282189596], [0.66729999079365054, 3.632440979262233]], [[1.3595626519671269, -3.8433699179001177], [-3.8103400632136921, 2.9980820230568197]], [[4.7054520271230853, -2.4243244237037711], [-0.5429998339105957, 1.1903739287233082]], [[-1.9333849853772014, -3.1145287101296981], [-2.3691207651966826, 2.9574101351063291]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.55244353964)+(1.-msk_arg1)*(1.46116639629)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.2854601050929046, -3.5627046814978525], [2.8483527974946252, -6.1016833736173099]], [[-6.3751122377593505, -4.6849219451552369], [3.3042409739597995, 4.4386118003434332]], [[-7.5122882948963836, -5.2755950558427251], [-4.5461473650028701, -0.69870842979372594]], [[-1.6931485736566596, 6.6278495768369989], [6.9825072494847005, 2.7220780369105109]], [[-7.754088048310285, -6.1253688278595702], [7.5675274368190175, 6.911036903261019]], [[1.5077157169932256, 0.85376569895982291], [-6.2671494742272333, -2.7105157038940018]]])+(1.-msk_ref)*numpy.array([[[6.0779456252858042, -6.4780122983534705], [-2.9517621520058182, -2.6386853320289196]], [[3.114182466933415, 3.5110903917641902], [6.9374455091719556, 3.1857848315093173]], [[-0.41084338915229651, 2.0447975010881025], [0.97503632278987939, 5.3076006953914927]], [[1.9865472607003329, -5.6158029725335172], [-5.5675408587914905, 4.3806967054008519]], [[6.8754483813697842, -3.5423413816122449], [-0.79341311049923702, 1.7393343836658721]], [[-2.8249971717177615, -4.5508446915106155], [-3.4616796508496197, 4.3212683094540711]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.98071137892)+(1.-msk_arg0)*(-2.76229348119)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.93181983881782493, -3.454260773171046], [-1.3855037950477387, -3.7428858836457515]], [[-1.7477527837205429, 3.1167029816226961], [-2.643044793535827, 1.1100617848980754]], [[0.53849527178480106, 2.9379518230476709], [-2.9874359508555228, -0.4829781509423805]], [[0.66336764031148121, -3.4354653899457297], [-1.8434805051980927, -0.59286162773319884]], [[4.8241384581494309, 4.0189857609830639], [-1.3287948678551587, 1.8141822220903672]], [[-0.25619294082166899, -3.9338347998426859], [-1.4417667775796383, 4.5294035013704921]]])+(1.-msk_arg1)*numpy.array([[[1.8032662071320038, 4.977432544678873], [2.368429750109974, -3.2262662244797191]], [[3.5416608782815526, 0.34423097017149296], [-0.52215478802251347, -4.1957360460220139]], [[1.1362696773421774, 2.637775923500687], [-2.279228609950037, -3.3023699313590784]], [[3.7549409505519176, -2.2627380562870369], [3.0557801993915827, -2.3010391696316645]], [[0.64790006034962211, -1.6378137094114518], [2.5053679790269268, -3.7513355220617783]], [[1.2395377946385064, 0.91102223648159075], [3.4737076697970064, 2.2915396388009555]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.6411256743009162, 17.204675938681035], [6.9007945175275101, 18.642234310663692]], [[8.7050521774114653, -15.523398005273942], [13.164243278152277, -5.5288973633432006]], [[-2.6820895276717751, -14.633090075764645], [14.879556234213018, 2.4055747721671987]], [[-3.3040427545049775, 17.111061559579589], [9.1818443290525327, 2.9528726553742333]], [[-24.027641311977899, -20.017408111435341], [6.6183437185732279, -9.03591803699514]], [[1.2760230955487981, 19.593295750357708], [7.181024194835973, -22.55965155898431]]])+(1.-msk_ref)*numpy.array([[[-4.9811504888169607, -13.749129471245993], [-6.5422980593931355, 8.9118941604745547]], [[-9.7831067566745862, -0.95086696492957157], [1.4423447671284755, 11.589854328734498]], [[-3.1387103225999482, -7.2863112383346715], [6.2958983314143282, 9.122114933882056]], [[-10.372248909975459, 6.2503465825297546], [-8.4409617247390312, 6.3561454982440662]], [[-1.7896901131685281, 4.5241221329163244], [-6.9205616364565943, 10.362289658360238]], [[-3.4239671698227063, -2.516510785055269], [-9.5954000518515539, -6.3299050061560038]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.6840159492987392, -1.2234605161137191, -2.694961332062956, 0.80551058735035852], [-1.3220815684692577, 4.6389778193729807, -4.9535934791811709, 0.15617416499676295], [-4.8780014843901585, -3.8218016578030269, -4.1581182095612768, -4.7743770180566623]], [[4.8003848994148814, -1.1801819870613928, -1.7212360877182862, 4.821957961514574], [3.6091862875718501, 3.2916618647682654, 1.0682695039876124, 3.5245175729933518], [-1.0433735753662656, -0.41346116532904009, 0.7023667556329567, 4.5366236390181118]]], [[[-2.9599946300668312, -0.92576652194567366, 4.935360239881998, 2.0346544116600995], [2.8404567140102586, -0.29670420131765152, 4.1058823702497662, 2.4983691444622984], [3.0499719048671849, 4.9473511309547256, -1.899970922826113, -2.1121460628676703]], [[-0.89488161745448291, -0.11139803315992758, 1.6630453250895849, -4.2307646476719993], [3.4622268126963291, 2.6832484006117108, 1.3577904421569791, 3.5671803675501508], [-3.5692991609551274, -0.4348972462299292, -3.2443420092301323, -3.5469507470886574]]], [[[3.4370524940269238, -2.9514856952423174, 2.5234221635609533, -3.1458978058034557], [-4.359346104178349, 4.3886956155659291, -3.9969298062489278, -1.1965665507315384], [-0.52408564744997577, 3.6718906395853637, -4.7835909073100158, -2.0558626957023893]], [[-2.7761574941126765, -2.0753002341107818, -0.24724314430631011, 4.6338803679896312], [-3.4578842631845816, 3.0105525004892328, 4.6524554452322988, -1.0480747967138173], [-1.3901589785096458, 3.5690769693795836, 0.41463883050293759, -3.3631970731358529]]]])+(1.-msk_arg0)*numpy.array([[[[2.2509328624156879, 1.2963650991046771, 4.3202443466111777, 1.9208489322663258], [-2.5784752361659247, -0.61420651901860079, 1.2185506766537602, -2.9688899762373557], [4.3927095826364653, -4.2627701085403711, -2.0323000509661506, 2.7103651620305289]], [[-0.60272308172818256, 4.3044801586160251, 1.1379998265243083, 3.1459767221365471], [-1.047504745627541, 2.1920393826820872, -1.5798947293391352, -4.6005810809610539], [2.1073620157267579, 1.7567631474323777, -2.0144621003682039, 4.6304833983558567]]], [[[-2.2873092770635441, -0.31630802752119003, 0.68869822593539354, 2.9866764804005346], [2.8758045465478013, 2.2428968080458329, -1.4200151154630616, 3.9297782897788398], [-4.8203517362469741, -3.410837351084691, -4.1690738284720208, 0.60064272809938313]], [[-2.0980815554209209, -2.3102298248842867, 1.4541174623153914, -3.6161378966033664], [-4.7843406881449733, -0.39211282319236229, 3.4117002694816776, 3.9284877103416562], [-3.4937780737725865, -0.56275701825070001, 2.4623450051554272, -3.0467263700060485]]], [[[2.4779117804316728, -4.2465997345022819, 0.39115574574363077, 2.3064250228213776], [-4.89445568368442, 2.794933739933513, 0.79120130635847019, -4.2753160544110624], [3.2695315005424987, 0.39839296657863787, 0.67109719969040427, 2.0792341404708301]], [[3.3466895014035067, 4.3583733271646885, -3.9934749237491363, -1.0373095803613852], [-3.7441092570369552, -2.2688906236878772, -1.8432201062137645, 3.5980699014984374], [-1.2141287162516257, -2.5262928776227733, -3.159173747716173, -1.2044331934887351]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.99043970935)+(1.-msk_arg1)*(4.09734824192)
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.3423719260054412, 2.4352243940911706, 5.3641580504933426, -1.6033202593617428], [2.6315236528772443, -9.2336056624608798, 9.8598291649255714, -0.31085525958369903], [9.709367856784775, 7.6070657809401174, 8.2764836004704616, 9.5031096041346714]], [[-9.5548767239460606, 2.3490810913032836, 3.4260166581559033, -9.5978166034015029], [-7.1838677052144257, -6.5518544853786196, -2.1263260410215845, -7.0153397335779983], [2.0767721960925822, 0.8229695217438906, -1.3980186809372006, -9.0298758374648624]]], [[[5.891690851139507, 1.8426824468649181, -9.8235370013944685, -4.0498569357667336], [-5.653757836247892, 0.59057182423280041, -8.1725113116537518, -4.9728531537455583], [-6.0707851918409759, -9.847404147136082, 3.7817775713981465, 4.2040993954731656]], [[1.7812079065462563, 0.22173106874469639, -3.3101914535025387, 8.4210819556286776], [-6.891353730757376, -5.340844166619922, -2.7026000130413483, -7.1002574539755727], [7.1044747845047409, 0.86563674838179949, 6.4576671658749465, 7.0599916141039936]]], [[[-6.8412457672220315, 5.8747543293805471, -5.0227196777985554, 6.2617199142194355], [8.6770155925446169, -8.7354340254603944, 7.9556478018312902, 2.3816935774526691], [1.0431608837833708, -7.3086769374110085, 9.5214492951820731, 4.0920707464916219]], [[5.5257741156836966, 4.1307599947916476, 0.49212257229114059, -9.2234594928109939], [6.8827101477693873, -5.9923232440482854, -9.2604320641589908, 2.0861296937451801], [2.7670276331311445, -7.1040325255696963, -0.82531359327033149, 6.6942410047298928]]]])+(1.-msk_ref)*numpy.array([[[[9.2228558064879262, 5.3116592596966914, 17.701545578231119, 7.8703869956059984], [-10.564910975726169, -2.51661800087368, 4.9928264726717844, -12.164576124575602], [17.998460885659295, -17.466053609916287, -8.3270410408702009, 11.105309931593823]], [[-2.4695663592806421, 17.636954210263962, 4.6627815885091053, 12.890162191552127], [-4.2919917278947679, 8.9815487108411922, -6.4733788916686965, -18.850182803863813], [8.6345960502167713, 7.1980703935933557, -8.2539527453483341, 18.9727030114706]]], [[[-9.3719026450924954, -1.2960241404675927, 2.8218364652464585, 12.23745362613842], [11.783172702889191, 9.1899292932436669, -5.8182964368354959, 16.101670166741602], [-19.750659711924119, -13.975388423925342, -17.082147321504245, 2.4610424259971193]], [[-8.5965707724984863, -9.4658161114095467, 5.9580256277560695, -14.816576253170505], [-19.603109907293863, -1.6066227867396066, 13.978924101102152, 16.096382213353607], [-14.315225448213761, -2.3058114793549103, 10.089084977862109, -12.483498935740769]]], [[[10.152867477172522, -17.399797956280214, 1.6027013071376699, 9.4502265123662514], [-20.054289390675908, 11.451816845585878, 3.2418272816088374, -17.517458719172726], [13.396409145633967, 1.6323547212023379, 2.7497189313056496, 8.519346349988087]], [[13.712552344811504, 17.857773289668064, -16.362657457955709, -4.2502185854154506], [-15.340919481858558, -9.2964350080652629, -7.5523146616575998, 14.742545385192393], [-4.9747081609922805, -10.351101680690403, -12.944235001109316, -4.9349822278453033]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.36012767748)+(1.-msk_arg0)*(2.02513478448)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.65970406346879251, 4.4475054318612646, -2.1617273131652812, -2.3595256380659091], [-3.4717035770453397, 4.1581975467447876, 3.3406590274922685, -4.5522672840598961], [1.1080475355222212, -2.3509494697136137, 0.74463973731315036, -2.7716336306587408]], [[-2.1755872880678706, 1.224168477506308, -3.1153327462944311, -0.66598378819679471], [-3.2520285909259474, -2.1126664782620264, -0.9786470801564171, 1.4590531769292667], [-1.7257018875234795, -0.37278343961693672, -3.2456071013724941, 2.110531188073633]]], [[[-0.37745614169925812, 0.7058452459725153, 4.1771701836412767, -0.68395757409795976], [1.9084342243160899, 3.0477893991578888, -2.3675012997967073, 0.93481417995209171], [2.6311988892032714, -3.6375136444563894, -0.44369083721198699, 0.1700015829762167]], [[-4.7047154447344743, 0.53700331647249744, 0.92277879680391273, 3.6711377477531144], [-3.3318614630877077, 0.38250214479523059, 1.3231783804965644, 3.901438620759647], [0.52408386972215926, 1.142066887897192, -1.4694083634729682, 2.4578756158738058]]], [[[2.3189252524165589, -4.2502574509724509, -0.38920294810293132, 2.9150031494916471], [1.1762817476395497, -3.6976621527241429, 4.1079565361320203, 2.3533228984189396], [-1.9209284261629422, 4.5165932069965447, 0.10523655334559123, -0.048211631041449543]], [[-3.8976687540819679, -4.6605155208672482, 4.6170846917877544, 4.4113670704973167], [2.3352640840299106, -0.080136389292638555, -2.0236231583052144, 4.0798300581969933], [-3.6742188995914393, -2.4699506211886222, 2.3607728500093819, -1.5087516448437608]]]])+(1.-msk_arg1)*numpy.array([[[[2.422170272734526, 4.3582470666547408, -1.8134621048799113, -4.0242318667517498], [-1.8404975978432638, -2.1033438433852227, -3.4624575538327562, 0.18841827001155487], [1.0284410442269589, -1.2111594348807033, -1.1274765552568891, -2.7178307471964769]], [[-0.90948501915102486, -2.8691832855221042, 3.4145197346609777, -0.89588968106216083], [-3.4274505911849831, -3.5008890380301061, -2.6689802061258572, 3.4113568466368225], [2.2184512072319817, 3.6115746104998241, -3.8351586120768424, -4.4898865383111772]]], [[[-2.4208420581771914, -0.37593077118698659, 1.6052128806612886, 2.1830674601779751], [-3.8010968575870652, -3.7423167974868643, 2.7205771795501255, -2.8058358589222667], [-2.8118041839101493, -4.7836039966819683, 2.322131103433561, 1.6638240100349062]], [[0.56955854840271236, -4.3636983293201297, -4.6367875150425242, -3.1118498955416962], [3.1255478973188886, -1.9353208492538423, 2.5679707618898266, 3.4984646003638016], [1.1594208809366435, 2.678527926963838, 0.42194320227755, -1.5871369841714689]]], [[[1.481864598169615, 3.3824543025226586, 4.8563120897359937, -0.52504150272581462], [1.8299687005885845, -1.8601364241317917, 0.50527119711146984, -1.9268170359998171], [1.8379370576708443, 0.35752648247810459, 1.6860556954551322, 2.6993496656833678]], [[-2.8558564148393737, -4.2393925583645604, 1.119443924112864, -2.0087422986436545], [1.2564312449206021, 2.0880679867786736, 0.66396625974209122, 3.9980720955113007], [-1.6491705337874252, -3.0980174047589193, -1.6292418803866235, -1.7724871730435368]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.2166898826066901, -14.94418609733413, 7.2636797761284422, 7.9283074021859772], [11.665367277232038, -13.972074665241413, -11.225040859296028, 15.296199296450691], [-3.7231811920703377, 7.8994903815387119, -2.5020845910964251, 9.3130428742073601]], [[7.3102510614077936, -4.1133623831659722, 10.467915785379814, 2.2377905594721978], [10.927231276322507, 7.0988291068897995, 3.2883791405173453, -4.9026049627133332], [5.7985786753449702, 1.2525999531625986, 10.905654251543307, -7.0916542592283305]]], [[[1.2683008287580195, -2.3717301470090471, -14.03582514759206, 2.2981847749477797], [-6.4125826577721892, -10.240961515236762, 7.9551066439138403, -3.1410949993566275], [-8.8411642125632639, 12.222510273944524, 1.4908578623597175, -0.57122702417358662]], [[15.80844458051407, -1.8043997065771209, -3.1006545753313697, -12.335491554062253], [11.195479919645861, -1.28525604342144, -4.4460482985480194, -13.109331891599023], [-1.7609887159735966, -3.8374905595553797, 4.9373997116242805, -8.2587758846977124]]], [[[-7.7918849226492837, 14.281407697422829, 1.3077715980769864, -9.7947827625446191], [-3.9524568567567293, 12.424616941334062, -13.80325845493695, -7.9074654050220001], [6.4545647712058045, -15.176329842741614, -0.35360825557899039, 0.16199723583876843]], [[13.09666465823495, 15.65992719298535, -15.513994062139494, -14.8227565890964], [-7.8467854829709722, 0.26926849963540683, 6.7996321830083195, -13.708749897957471], [12.345844617632723, 8.299349444261729, -7.9324981935569214, 5.0695981602811147]]]])+(1.-msk_ref)*numpy.array([[[[4.905221273239416, 8.8260377340248208, -3.6725051889221265, -8.1496119341574289], [-3.727255706137679, -4.2595547809537289, -7.0119432320398367, 0.3815723926312693], [2.0827317324472623, -2.4527611011237105, -2.2832919907323719, -5.5039735844671132]], [[-1.8418297482429395, -5.810482874549141, 6.9148626869431276, -1.814297356172464], [-6.9410494142829648, -7.0897721675069452, -5.4050446545045077, 6.9084574123860074], [4.4926627074291838, 7.3139253704550571, -7.7667131093011061, -9.0926254070863664]]], [[[-4.9025314597381096, -0.76131048128581091, 3.2507724411167653, 4.4210058504649998], [-7.6977334654635623, -7.5786959211210316, 5.5095354801596983, -5.6821957974347432], [-5.6942824599727642, -9.6874428488410587, 4.7026284716779063, 3.3694678779687255]], [[1.1534328281662272, -8.8370772756678164, -9.3901196849385773, -6.3019154675307902], [6.3296557674076022, -3.9192855709463941, 5.2004869154214912, 7.0848623544561171], [2.3479835558330859, 5.4243800760859733, 0.85449185600563493, -3.2141663143746362]]], [[[3.0009755436374537, 6.8499258649405492, 9.8346865371977135, -1.0632798104638161], [3.7059332700650498, -3.7670269763808668, 1.0232422768644771, -3.9020642029249761], [3.7220702671674628, 0.72403931603790739, 3.4144900374307627, 5.4665469034401717]], [[-5.7834941651613248, -8.5853413349945349, 2.2670248299917382, -4.0679739020323771], [2.5444426183917184, 4.2286191123771326, 1.3446211683224119, 8.0966348714644507], [-3.3397926135064528, -6.2739028092906395, -3.2994344042967154, -3.5895254291687344]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.2203308083326578, -1.8418552270103761])+(1.-msk_arg0)*numpy.array([-3.4373236650305463, -2.9501048613517722])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.6450219576979759, -3.1544599146252903])+(1.-msk_arg1)*numpy.array([2.698606850502471, 3.8677633183826838])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([10.313485358058491, 5.8100584821472951])+(1.-msk_ref)*numpy.array([-9.2759851898456933, -11.410307368118819])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.1477964895058825, 2.8036517024908321], [-3.6062975993118886, -1.4914001630714271]])+(1.-msk_arg0)*numpy.array([[0.32205560896968777, -1.4480627832240298], [4.5470706830851828, 1.0671629505922411]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.2731468203135448, -3.3864134605414598])+(1.-msk_arg1)*numpy.array([1.5865953976835803, 4.5883212663042254])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-9.1778497398126557, -11.980415357765356], [12.212414733028131, 5.0504975872788087]])+(1.-msk_ref)*numpy.array([[0.51097194698948933, -2.2974897474201215], [20.863421114588224, 4.8964864608143452]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.27121939882773471, 3.1377575597961016])+(1.-msk_arg0)*numpy.array([-1.6821934687119655, -4.4820159541509579])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.1054108976246839, 4.0364454809688741], [-0.052221068517473768, 0.77110779212694691]])+(1.-msk_arg1)*numpy.array([[2.684291285051823, -2.775030432760861], [1.5201400834623593, 3.3978471699937352]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.84224767676686285, -1.0947623167493046], [-0.16385705252133351, 2.4195493041640086]])+(1.-msk_ref)*numpy.array([[-4.5154972678346255, 4.6681380694672594], [-6.8132921066226633, -15.229205225678603]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.0543559401107601, -3.2896124169505905, 1.2404222462520229, 1.581099810255667, -3.5244086729435189], [1.7517251194838011, 3.5901820142015666, 0.053082049688940813, 4.5900753809964829, 0.8805587243873596], [0.21512358612028404, 1.8374628444894059, 3.442482289975775, -0.76801380805759933, -4.9593658819885711], [4.01805488041032, 3.214881770379689, -2.4300686611461817, -2.8916814402280311, -4.165549288397]], [[-0.3302595861668598, -4.8331521274527987, 3.8360118512246508, -4.5521634261736201, 4.1205444918644289], [-0.55939573041897805, -4.5916472540011988, -1.7288584218446723, -2.5458415414291737, -1.1364952633298908], [-1.9351931597540508, 0.59267393725259332, -1.2313815593223909, 4.7397146804827344, -0.78944137108529677], [1.3758309568461868, 2.6852828635173012, 2.6504391370931462, 3.8567615074690274, -0.23137392300224668]]])+(1.-msk_arg0)*numpy.array([[[-1.0451599841371149, -2.8248618075164256, 4.6361356181161497, -2.9252044649890663, 4.1429573855321546], [3.2749386834766057, -0.16154409419556348, -3.1684145784668747, 4.8499090969379548, -3.1318835387147068], [-3.8192710535375154, -4.4950098748765734, 0.5482609279331383, -4.5540452856480726, 4.1454456751818789], [2.0582351065867091, -2.4724252613256859, -3.2245533575802732, -3.9401343620720386, -2.941459734571791]], [[0.46857824492512634, 1.4945064297133133, 4.0677729135516056, 0.40544905312353308, -2.952743537324658], [-1.4888925609176304, 1.0296274902701485, -3.9336422462895682, 3.6677808959354934, -2.6063405414655625], [1.3525158761248068, 2.7209680554680471, 1.7295938357875764, 4.6067180087213675, -3.713577775641308], [-1.9914227435669618, 4.2524738625370446, 0.74942827650561306, -0.58183966924419117, 3.1364716443173641]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.8282557372694739, -1.7584078924071376])+(1.-msk_arg1)*numpy.array([1.71278761582073, 1.749218882026887])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-8.6384997112813551, 9.3038651916334079, -3.508231334598972, -4.471754609551267, 9.9679290497347992], [-4.9543266192991151, -10.153952879507257, -0.15012961157877014, -12.981907030802669, -2.4904452642512394], [-0.60842451666667707, -5.1968148319466492, -9.7362202870725429, 2.1721394589410816, 14.026355008952661], [-11.364086768184098, -9.0925078117193987, 6.8728556328454378, 8.1784146236805846, 11.781238673787589]], [[0.5807310628589214, 8.4986528461173485, -6.7452735145607408, 8.0045600961108097, -7.2455979555091705], [0.98364586734758652, 8.0739887705852684, 3.0400382938262203, 4.4766278592670119, 1.9984222407226082], [3.4028589254438297, -1.0421625288889727, 2.1652710524771002, -8.3343517019188145, 1.3881599375090976], [-2.4192720131363989, -4.7218225805544609, -4.6605530970093518, -6.7817598738655871, 0.4068497323043519]]])+(1.-msk_ref)*numpy.array([[[-1.7901370773814409, -4.8383883203190967, 7.9407156719747265, -5.0102539813767768, 7.0960061028125043], [5.6092744196309754, -0.27669072394713862, -5.4268212517839221, 8.3068642390916292, -5.3642513393033537], [-6.5416001619616484, -7.6989972466804844, 0.93905452760226105, -7.8001123671447976, 7.100268014509127], [3.5253196010091754, -4.2347393686409669, -5.5229750574166463, -6.7486133400266999, -5.0380958058098955]], [[0.81964591373005036, 2.6142188661651162, 7.1154251881819928, 0.70921913942360648, -5.1649947492711537], [-2.6043989808664865, 1.8010438474344987, -6.8808012923483712, 6.4157515983078577, -4.5590600881237426], [2.3658463087586501, 4.7595687000166897, 3.0254381957969398, 8.0581581250287169, -6.4958603650271822], [-3.4834342651451169, 7.4385075756756072, 1.3109140919884852, -1.0177649357542178, 5.486375423181852]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.8002589979103476, 2.9070389014653228])+(1.-msk_arg0)*numpy.array([1.3745911893665426, 1.6657970582928403])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.4171277249462086, -3.6936279258197899, 1.9964179711645329, -2.2482663892530788, -2.016794630383838], [-2.0248934510627938, -4.1292628255462862, -0.71449207040565632, -3.4806591682727395, -0.46978089013803803], [-2.5031574489494379, 1.177754971599656, -3.3900004231052803, 2.9353673712298107, -3.0793105944461607], [4.3670754238108831, 1.4859669421099175, 0.068714732664409084, -1.5113416610972119, -1.9502405595560335]], [[-2.4591719957807969, 4.7955489117415198, 2.7393780436529358, 0.47110205416301643, -2.2133542186804132], [-0.084988119158790454, 0.5834345791731419, -3.4724949580892184, 1.5365569939971602, -2.8926187087809243], [0.84498208302809985, -0.40317711722414717, 3.7630911991319618, -3.2768529605318566, -4.8912203375218564], [0.10885104223208675, 0.37905656386386077, -4.4662351566112513, 4.3589508865219599, -0.92647270431143625]]])+(1.-msk_arg1)*numpy.array([[[1.0584208664709047, -1.5151554537090584, 2.5933740847221189, 0.4756422645425209, -0.71081884682027674], [0.38651927446901446, 2.4391824148554742, 0.69164692692041729, -1.7695651258909062, -4.4604994550703116], [3.6472132479987653, 1.8187952928907105, 2.1570197233403912, 1.5351512381765211, 0.97892475172451832], [4.2523457376029157, 4.57984232374538, -4.7349921431690749, -1.2427537097548411, 0.59415128787037919]], [[3.9471006185019171, 4.6158764375536148, -4.5079482822971908, 0.51449661967501026, -4.9208231184113593], [-1.2134154554677368, 0.20703687815389138, 1.0639013243865145, 1.061866271601315, -1.8326267535464069], [2.0774688599107023, 0.39023713964147166, 0.71611289454608151, -3.9846767361515822, -2.7843684556603812], [-3.2326331509693631, -4.7446744674640771, -0.80292665895366344, -3.7699807628516036, -3.7327516642890668]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.3514559359329796, 6.6494869083900108, -3.5940694161788711, 4.0474617969522635, 3.6307526802857781], [3.6453326550855305, 7.4337425564264077, 1.2862707786833765, 6.2660879863421455, 0.84572727451733543], [4.5063317206575375, -2.1202639849559266, 6.1028787646151663, -5.2844215222289099, 5.5435566050123617], [-7.8618668262686873, -2.6751253581307037, -0.12370431576810653, 2.7208064243070269, 3.5109381154304606]], [[-7.1489086571288931, 13.940847240312292, 7.9634785387190554, 1.3695119980121122, -6.4343068164263464], [-0.24706376855697415, 1.6960670181163733, -10.094677928307554, 4.466830955868363, -8.4089551135325387], [2.4563957864038874, -1.1720515639512405, 10.939452505638403, -9.525939030647919, -14.218967796814383], [0.31643421423372092, 1.1019321770080177, -12.983519343360976, 12.671639796696093, -2.6932921925791247]]])+(1.-msk_ref)*numpy.array([[[1.4548959976926075, -2.0827193371891379, 3.5648291675905464, 0.65381366613049952, -0.97708532407483839], [0.53130598920545569, 3.3528786567181417, 0.95073177189725055, -2.4324286310599366, -6.1313632511139149], [5.0134271964400332, 2.5000999848689109, 2.965020306993559, 2.1102053663425848, 1.3456213387733531], [5.8452369850493398, 6.2954109069083923, -6.508678481720013, -1.70827829998159, 0.81671512545740754]], [[6.575068599086344, 7.6891133911200464, -7.5093269875869231, 0.85704695555624233, -8.1970926750290438], [-2.021303896205223, 0.34488142258688548, 1.7722436964769128, 1.7688537115338567, -3.0527842550063626], [3.4606415155342285, 0.65005587925137587, 1.1928987531404336, -6.6376627853292218, -4.6381927826424416], [-5.3849107934246803, -7.9036647704588079, -1.3375128665099112, -6.2800228645787994, -6.2180067417104317]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.9615030247632275, 3.6493817992368314], [0.43575031890137783, 3.795726800920594]], [[2.0344636586888001, -4.180671352624028], [-3.5618645756050418, -2.194098347005502]], [[-0.21876022571801546, 2.1448706213427968], [3.3829410164023042, 0.069979818716543463]], [[-2.8539484021311479, 3.8101593372341842], [2.8423247510956511, 4.6367783987777571]], [[-1.3312779164022706, 4.2370615178614131], [-1.6343640776708304, -3.2868476877885877]], [[2.0731317778407723, 1.0442475913834217], [0.48707639586322937, -2.5222898702814445]]], [[[-3.6872265026798576, 3.2770686288031428], [-3.0608205874425733, 2.1466600013020267]], [[2.5419288660372779, -0.87524462527641056], [2.4873414460853311, 2.3859136569188859]], [[-3.7367974645527169, 0.72661965510739357], [-1.7156007647633587, -3.8158111843558782]], [[-4.9696828872622199, -4.0156027569325312], [4.9533721740437837, 0.39267722606685851]], [[-2.707700525856521, -3.9268046067629658], [-3.0498714874657242, -2.7225625263009281]], [[-4.1716034714495729, 3.906684084832758], [-1.1297509602032751, -0.68994258801377573]]]])+(1.-msk_arg0)*numpy.array([[[[0.021023584968997255, 0.031673002248588489], [-3.8259296629297812, -0.34889770723657065]], [[-2.9226789433465425, 0.96623524018679152], [4.5432815432520783, 4.4892491087115243]], [[1.1472037527953045, -2.4650288770907691], [0.90186034560769901, 2.300243873935008]], [[-2.7803633576234121, 3.5929907318484098], [-1.0847095907397231, -1.8322454702670421]], [[-2.5479661405300713, 0.022862077717920926], [-3.7883561585563985, -1.8529289780161049]], [[0.2353373199051223, -3.5441179131884324], [-0.96361572215628044, -1.8402898016948965]]], [[[0.09303096407372724, 4.8648047375877574], [3.0277444071825261, 2.9723755364329305]], [[3.3922600524468773, 2.4905172575982473], [-3.7311796631250873, -0.20081209197774808]], [[-4.9675335115798198, -4.8243639479945077], [-2.4325974032784803, -4.299236646237051]], [[-2.9904679678972634, 2.008927906497421], [-1.9793749887906662, 4.60686721097664]], [[2.3808686599021343, -3.8764725949598366], [1.7944000855908975, 3.9484631824820884]], [[3.3361800616655444, -4.3806977772536175], [-1.0146369471200711, -3.8263709115263964]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.1751970238426566, -0.5289543753052639])+(1.-msk_arg1)*numpy.array([-2.8058222768036534, 2.9777194432923704])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-12.578552614171882, 11.587506227802345], [1.3835931157141435, 12.052180441602879]], [[6.4598229541847205, -13.274455236516067], [-11.309621799791715, -6.9666945414299626]], [[-0.69460681763499044, 6.8103868134151977], [10.741504247115849, 0.22219971211781744]], [[-9.0618484726473252, 12.098006587952291], [9.0249410904732308, 14.722684972017053]], [[-4.2270696780679424, 13.453505121351808], [-5.1894279552957689, -10.436388996090441]], [[6.5826018510336555, 3.315691844315503], [1.5465635225289336, -8.0087672893861228]]], [[[1.9503745913340371, -1.7334197893810441], [1.6190344417521774, -1.1354851999815105]], [[-1.3445643954051663, 0.46296447400237356], [-1.3156901407849582, -1.2620394679278271]], [[1.9765953685047764, -0.38434864575185768], [0.90747453079863538, 2.0183900213038029]], [[2.6287355070970477, 2.1240706477673426], [-2.6201078839758067, -0.20770833681079903]], [[1.4322500401681706, 2.077100477716137], [1.6132428674137682, 1.4401113603290285]], [[2.2065879082618793, -2.0664576396077279], [0.59758671340484548, 0.36494815063932379]]]])+(1.-msk_ref)*numpy.array([[[[-0.058988443044286944, -0.088868815282341793], [10.734878677732272, 0.97894495929008918]], [[8.2005176871866912, -2.7110843615488283], [-12.747640563847563, -12.59603515534374]], [[-3.218849845625817, 6.9164329363055748], [-2.5304598482719238, -6.4540755035679798]], [[7.8012054464283729, -10.08129343576933], [3.0435023335600886, 5.1409551570478529]], [[7.1491401576407023, -0.064146926954958966], [10.629454102143857, 5.1989894038526137]], [[-0.66031469475306004, 9.94416499244298], [2.7037344595043313, 5.1635261213701185]]], [[[0.27702011055057157, 14.486023654935902], [9.0157733905871389, 8.8509004276029266]], [[10.101198714875062, 7.416061661805494], [-11.110406229304649, -0.59796207073035623]], [[-14.791921122637655, -14.365602329461987], [-7.243592585444862, -12.801920552815149]], [[-8.904774612550705, 5.9820236873500079], [-5.8940233896885843, 13.717958066791235]], [[7.0895589005160353, -11.543047817401936], [5.3432200239095087, 11.757415589600985]], [[9.9342082359458299, -13.044488946515767], [-3.0213041653222485, -11.393859060500501]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.3006651436306322, -2.5642089690084835])+(1.-msk_arg0)*numpy.array([1.1762327971786535, 1.16109681273156])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.158717317308164, 4.3729992419919252], [3.340058898512325, 4.6533469048175267]], [[2.7092429222867498, -2.6448526039872577], [-0.43780904255466346, 2.3751293456568812]], [[2.3494110718162915, 4.6120969579826721], [-3.0433951006577131, 0.07242505038169611]], [[-2.8690438237624138, -1.7930468477530082], [2.696571416448136, 4.5834370262440842]], [[-3.2528862427684535, -1.2693141428239176], [1.7983426849969355, 4.9502706969561103]], [[1.2030091472228932, -0.70996666810984976], [0.59746946772940479, -4.8412569412127642]]], [[[-2.175707075551979, -0.86346683781004963], [-3.708199604689284, 3.5155127223811906]], [[-3.3180558915797542, 2.8195316820204175], [2.8194577405498134, 2.2504387114765763]], [[2.5179120078525399, 0.097492707257497813], [4.7802433959856074, -2.9495974089370147]], [[0.046529916308012176, -2.7023129835370421], [-2.238701445466007, 3.7132726616187846]], [[4.1836156198679415, -1.1045808980042002], [2.6473923719712662, -1.7979628448013463]], [[2.8294476555693588, 3.7919096409568702], [-0.56190187386129242, -0.59621395082761452]]]])+(1.-msk_arg1)*numpy.array([[[[2.7593326461268717, 0.44255654256832422], [-2.9027471882336311, -1.7635020415854363]], [[-0.92912820110284677, 0.34968861155437292], [0.14692040893866132, 0.64627119658716303]], [[2.677974285076095, 1.6809473065430325], [3.9086331782142754, -0.35106318667635605]], [[2.4574901282134878, 3.7271839318604023], [-3.6216191146234644, -2.7294024091167657]], [[-1.8345150778712638, -4.0648946032870388], [4.8106363965814385, 1.9722619642754999]], [[3.6698224542094078, -3.3889877365073984], [-1.1778118568633755, -0.70887804718053093]]], [[[-4.7201436184374153, 0.62233384337420006], [-2.7139272262590985, -3.4929359447261685]], [[3.0439660731663807, -0.98459090242102398], [-4.3066906829660105, -3.8770791300620902]], [[-2.2896689836776849, -2.7207758849650432], [1.037972384809235, 2.7866198246767713]], [[3.2031907871386238, 3.4462259668984103], [2.7079844364471262, -1.590911710598506]], [[-0.62195028690067389, 0.38643770229490837], [0.18633582384069847, 1.1753126146811077]], [[1.887276218090344, -4.4962917712096973], [0.30287832043885654, 0.27750369445515766]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.425868147821516, -14.433806171165925], [-11.024415983992954, -15.359139929952699]], [[-8.9423036792198687, 8.7297728000214541], [1.4450610463264779, -7.8395066428238991]], [[-7.7546192328039174, -15.222987668258279], [10.045228127037154, -0.23905083932055676]], [[9.4697529446413462, 5.9182472312751351], [-8.9004792815810436, -15.128390830549888]], [[10.736688237701445, 4.1895809475362986], [-5.9357270166725069, -16.339185940979149]], [[-3.9707303597074151, 2.3433622345697587], [-1.9720466465179933, 15.979368037220823]]], [[[5.5789675970656027, 2.2141094099539229], [9.5085986852179758, -9.01450925339328]], [[8.5081886768602466, -7.22986842744033], [-7.2296788260582252, -5.7705951281721317]], [[-6.4564525537096422, -0.24999167436259437], [-12.257542990029867, 7.5633841309604772]], [[-0.11931242872421892, 6.9292951894537573], [5.7404983253961914, -9.5216070632968908]], [[-10.727664695349361, 2.832376245657815], [-6.7884672646933639, 4.6103524525836201]], [[-7.2552950557509766, -9.7232487110113457], [1.4408338246577996, 1.5288171601601521]]]])+(1.-msk_ref)*numpy.array([[[[3.2456175567001857, 0.52054951997485377], [-3.4143064447185152, -2.0742889392043038]], [[-1.0928710629207719, 0.41131521371011964], [0.17281260356855324, 0.76016537729771427]], [[3.1499211841075598, 1.9771853522850347], [4.5974625363562671, -0.41293203405078205]], [[2.8905804875474783, 4.3840359817714925], [-4.2598671815092359, -3.210412630301569]], [[-2.1578168015109318, -4.7812623494607269], [5.6584283049604238, 2.3198392070088367]], [[4.3165655304637625, -3.9862385249162506], [-1.3853809349485919, -0.83380560829369743]]], [[[-5.4805437110028956, 0.72258984199676557], [-3.1511322523948424, -4.0556367924970544]], [[3.5343393056164869, -1.1432053586455413], [-5.0004848254125402, -4.5016642206231419]], [[-2.6585273591584699, -3.159084208189801], [1.205186427705379, 3.2355353967267777]], [[3.7192146135177526, 4.0014019861184824], [3.1442320980854279, -1.8472025165132393]], [[-0.72214449579785178, 0.44869158445392554], [0.21635393115914442, 1.3646517308694301]], [[2.1913104015687708, -5.2206300446627196], [0.35167105250704445, 0.3222086551531162]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.8010724537391676, 1.9567431036351577, -1.7497019751505425, -2.9534339168036885, -2.3453955934613377], [1.2133081144887656, 3.9255264126334684, -0.60086932913390534, -1.9726063008062433, 0.32572163520084185], [-3.6422910152393451, 1.7338965415100835, -4.3387732978260463, -1.4770242813794598, 4.0150986292163022], [2.5512291372009246, 4.2133572986296723, -2.4978823528401648, -0.97117360769554395, 3.2583364504933563]])+(1.-msk_arg0)*numpy.array([[3.6202352090216365, -3.7707876954944486, 4.0748779487105686, -2.7859678250263453, 4.8106930629207323], [2.5119628810154495, -2.0655409733998811, 3.8788476589060235, 0.44411188459786644, 3.5080621290517229], [1.6794781232406351, -1.8083433530055224, 1.7664176740307891, -1.5477310026931947, 4.8130390164647743], [-2.3396558092517026, -0.68786091033237984, -1.5979465467857579, 2.7449514036468425, -4.5653763163948327]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[1.972881179132008, -0.25132488350281257, 3.3302330802691635, 0.98150957022639673, -2.8112922215693095], [1.8766193183598325, 0.49436103408036303, -1.3206823427140604, 2.3007520456125539, 4.4119738012038319], [-3.5836750171032792, -4.9999259662741071, 3.0847568817365794, -0.17806621383024357, 3.5492116988084934], [-1.7254103571939017, -1.6405005136730519, -1.0899463955035857, -4.0156307078815958, 0.53287200253005818]])+(1.-msk_arg1)*numpy.array([[-0.60596525926565725, 0.40257451965059587, -4.8066498050506041, -2.0378429736310935, 3.5073089572163028], [4.623634431674752, -2.110063796808924, 0.74323532549242977, -4.7154354792644266, -0.094286016255727745], [-3.6530816688372525, 0.56946836094963427, 4.3519830324390636, -2.0039732546844546, -3.5939765448674263], [-2.2243603368023912, -2.5713421276187987, 2.2898221415312978, -2.2409027180719363, 0.43886108639317989]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.5533019462351079, -0.49177823256603792, -5.8269153982586301, -2.898823654374052, 6.5935923884007934], [2.276917446772361, 1.9406272966592593, 0.79355751326559187, -4.5384779817681773, 1.437075320991386], [13.05278731633298, -8.6693543407291376, -13.38406078876381, 0.26300812152057673, 14.250435026684444], [-4.4019171769013372, -6.9120148126900798, 2.7225578668701536, 3.8998745617463806, 1.7362762692910765]])+(1.-msk_ref)*numpy.array([[-2.193736767037457, -1.518023045218055, -19.58651129777466, 5.6773649569922373, 16.872586870000216], [11.614398067751941, 4.3584232287965534, 2.8828966023025675, -2.0941809373957678, -0.33076120292587363], [-6.135270745223556, -1.0297943252702206, 7.6874197455824707, 3.1016115348431157, -17.297949334706185], [5.204237583868788, 1.7687257366798652, -3.6590133838135062, -6.151169061407586, -2.0035660100067298]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.8516642531473533, 4.610922495677384], [1.2611828713067403, 4.7280411004436083], [-1.1773337885795554, 2.4954455848492882], [-0.38698551186082497, 4.9308583559497681], [1.544593494834591, -1.6425079109714935]], [[3.1005068201692367, 0.062568104206275343], [2.7775509332852755, 2.6127366060707056], [-2.6661357977161648, 3.1490986228973323], [-2.8161958056865477, 4.6020390947949892], [3.3676252501642008, 4.2428508410618644]], [[-3.3753730169491436, 2.9966851686063851], [-3.1668830113506621, 1.0633142285546757], [-2.7711445463360218, -0.88521656508900204], [-1.1768539080457763, -0.14322612263222734], [-0.19937353766020038, 2.1421824260131581]], [[-3.2164763061403456, 2.5483597701475649], [0.31020556051380321, -4.1559724701928928], [3.7605421496246745, 1.4175152906243067], [4.0277952967702983, -1.902114927577462], [1.9607511916673328, -1.2589472280650496]]])+(1.-msk_arg0)*numpy.array([[[-2.4294881342719443, -1.5784755270640671], [3.3078493036623353, 3.3762127781883162], [2.1622662920158797, 0.72180865177354114], [2.7486230590055474, 3.2961197264948474], [-2.9328098165232888, 1.5605273629516141]], [[-2.6042882840620694, 3.4225582317950725], [-4.457664476845169, -0.80033297163047656], [0.52959539364233699, -2.412989596245485], [-4.6596743700574113, -4.7208429268809109], [-0.98086538798217138, -1.462444241883345]], [[3.3880098053409657, 0.54067050511132742], [4.8974427523025881, -2.5172132379555001], [3.9944957345123075, -3.3617007455199821], [0.21105575545136901, -2.8864915614774498], [-4.8843723893008981, -0.58367225867736217]], [[-0.54197047302258916, 0.37086460101686924], [-0.9064175206281444, 1.0030747348563054], [2.2082589582175549, -0.68625483790903719], [0.81541183305204434, -3.9797251585027995], [4.0861747074268528, -0.96306967572602353]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.3658182990560492, 3.2481292838517124, 2.3248484959757096, -0.8985225106417829, 3.5970779802745732], [0.94025592082831988, -3.3566066271707893, -0.4970973275138526, -3.9286127380053184, 4.842522656406711], [-2.7466534678859378, -1.8312176797379598, -3.2809856449213415, 2.7708554247907724, -0.080944028591933836], [-1.7644028037117474, 4.7990368189244652, -4.2572589795543712, 3.6311153852755655, -3.0459782452842266]])+(1.-msk_arg1)*numpy.array([[-1.2462264527594034, -0.027735890280721875, -1.0523550471425192, 2.3795345336429161, -3.0368688711883127], [2.9949955756456461, -1.8668710995047433, 4.3385142558369516, 0.32838487998118726, 2.4705674115255185], [4.4085216350183654, 1.8130526850394544, 1.8664989389910778, 1.4564394502432885, -2.9060020400606557], [-2.3950921809920445, -3.8085477871677287, -3.4497062007869115, 1.0998570839316208, 4.108726371151878]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-9.1123377719160601, 10.908604815802741], [4.0964850165836086, 15.35728875360536], [-2.7371226876405634, 5.801532914726093], [0.34771519369918391, -4.4304872296069995], [5.5560232487448546, -5.9082290389823484]], [[2.9152698952327118, 0.058830030434953696], [-9.3231458699697658, -8.7699290069886455], [1.3253289798337191, -1.5654085095198174], [11.063742714937321, -18.07962940861006], [16.30780157220746, 20.546101325596346]], [[9.2709800024119851, -8.2308557105150832], [5.7992521600471223, -1.9471598144462519], [9.092085476530551, 2.904382842703594], [-3.26089203529486, -0.39685887886725552], [0.016138097332842258, -0.17339687554034727]], [[5.6751598126264309, -4.4963331233145878], [1.4886879063408429, -19.944664902892153], [-16.009601834482343, -6.0347296996659541], [14.625389470843192, -6.9067987780888407], [-5.9724054742338186, 3.8347258686470207]]])+(1.-msk_ref)*numpy.array([[[3.0276923795947863, 1.967137956860582], [-0.091746145351540784, -0.093642267180202327], [-2.2754718456690513, -0.75959897776502305], [6.5404434888709311, 7.8432307162161328], [8.9065588369150817, -4.7391169711853429]], [[-7.7998318884716893, 10.250546761615828], [8.3218849831111772, 1.4941184947176862], [2.2976571651428612, -10.468789762497288], [-1.5301666087627173, -1.5502534379538246], [-2.4232940626420865, -3.613067085170135]], [[14.936114526500008, 2.3835576191995944], [8.8793217318892221, -4.5638402198920787], [7.4557220502716079, -6.2746108747185616], [0.3073899284402738, -4.2040001829301081], [14.193996127724349, 1.6961527744432252]], [[1.2980692422649631, -0.88825490610223778], [3.4521344424383784, -3.8202580618008382], [-7.6178446211063449, 2.3673775696548223], [0.89683648090395918, -4.3771289076801967], [16.788973777538519, -3.9569897739122006]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.2039367266765035, 1.3289506293022129, 3.9530484166288744, 0.17907031654473116, 3.9736264910937695], [-4.1555706098723677, -0.38689321128441012, 0.53686423547498396, -0.30404081881130196, 2.366136020382724], [0.84988731197980538, -2.7664391627872531, 1.8756158079785497, 2.9873198291177561, -3.948145261632888], [2.2223108222965937, 3.4734710497561601, -1.7738798098061959, -4.4183053442154687, 0.45542592615892108]])+(1.-msk_arg0)*numpy.array([[-4.2959566738934676, 3.0076602046647896, 1.1275386551186095, -2.2226064771247533, 3.934539144571179], [-0.68846818702208434, 3.5431582427002368, -4.5948867846342436, 2.1399817390268128, -3.2015957651229177], [2.9213263418743711, -1.88594653669686, 3.7452076322839325, 2.8314097696598708, -4.4169857305189151], [3.6839811689960342, 3.4801907757343518, 3.9095518941492138, 1.3994116484698722, 4.5566168992508]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.2576122902888702, -3.7159982984291084], [-3.9104662661998257, -0.58724789656784715], [-4.0817845335545586, 0.75356870752467309], [-3.2283999694600576, -0.64175909189865443], [3.4173162025344102, -2.8811999119452913]], [[3.04235502883874, 1.97753448178951], [-2.7425559145637135, -4.7968178329436739], [-2.8647908183163051, 1.6884371709853934], [-0.950570791818941, 0.27881175661839919], [4.6049580418452187, 0.28216940672704194]], [[-4.1389825082823082, -1.843318654791485], [-1.7546875665626493, -3.330356192668793], [0.85871811639628959, 2.7505055206681517], [-2.6074624932671742, 3.4996447592286337], [0.23277206228323344, 1.1457078380851726]], [[2.7359564150566751, 3.5464200087360425], [-0.89366921962890089, 2.8394847497770668], [1.7416432339536776, -1.2447094830419845], [-1.7746795066764633, -0.018201918396682615], [-2.4702398850816074, -2.5283891014847693]]])+(1.-msk_arg1)*numpy.array([[[2.4808199084323395, 2.4689606393187233], [-4.7709701408513201, 2.5855741525337717], [-3.2054063980555836, -0.16260423063690599], [0.81858177644559404, 2.0615031117956857], [0.27989237251535659, -1.2518472764944031]], [[3.0180846713313851, 4.806166020232796], [0.54424342982112162, -0.20415337261563238], [-0.076665024061881581, -4.7715496811647675], [-0.51786108514906992, -0.031022387234555637], [4.9629370026767603, 4.4637983507262309]], [[4.4255493919617095, -1.9942079406867421], [3.3775734314317525, -1.4615267701318304], [-2.8185143410254678, 1.0740541724988759], [-2.7849898135734752, -0.87159837141348895], [3.626566773775421, -0.26361965289332367]], [[2.1505391118474018, -4.9483081550822856], [-1.8753196741611267, -1.3550319318427317], [-1.8518816185236062, -2.3087424900737896], [-0.57949970294021735, 4.9660252372470417], [-0.80979732450842512, 1.3499154577236725]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.5140856241985232, -4.4738268277461977], [-5.1968166053313327, -0.78042346170024124], [-16.135491887388078, 2.9788935861014765], [-0.57811060446421292, -0.11492000373175126], [13.579138190834694, -11.448812296242846]], [[-12.642721142639667, -8.2177841725336709], [1.0610762649126075, 1.8558562553339031], [-1.5380037324711369, 0.90646153094861803], [0.28901232188273845, -0.084770154776475523], [10.895957095161068, 0.66765119710687726]], [[-3.5176687182954836, -1.5666131366429661], [4.8542364025947782, 9.2132277974299992], [1.6106252737104449, 5.1588916344974569], [-7.7893244098178531, 10.454558184111733], [-0.91901791474406358, -4.5234209721516345]], [[6.0801455505122401, 7.8812475658232879], [-3.1041341624391667, 9.8628680745747559], [-3.0894657685959976, 2.2079650210424839], [7.8410759486182897, 0.080421633327036654], [-1.1250112874979978, -1.1514939482338233]]])+(1.-msk_ref)*numpy.array([[[-10.657494842357689, -10.606547936061553], [-14.349457030282482, 7.7765284847857137], [-3.6142196191721792, -0.18334255552893319], [-1.8193851583842642, -4.5819101688899257], [1.101247495928569, -4.9254421123920489]], [[-2.0778552819506619, -3.3088924064768195], [1.9283405944061549, -0.72334770495813072], [0.352267105905606, 21.924730572209729], [-1.108213265571619, -0.066387342182967574], [-15.889318090341742, -14.291277896047765]], [[12.928474016003848, -5.8257321883032231], [-6.3699229154480426, 2.7563613504198732], [-10.5559214217103, 4.0225558843291935], [-7.8854473665551597, -2.4678521440397851], [-16.018493690540055, 1.1644042451141601]], [[7.9225455912352842, -18.229474061712647], [-6.5264702315687035, -4.7157696300245737], [-7.2400272894390767, -9.0261485751707564], [-0.81095863457937079, 6.9495135635988712], [-3.689936173823174, 6.151047587223565]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.8105409124957905, -2.9375682770695954, 2.7421881097994794, 0.26548489088166338, 1.1595914869395942], [-3.8204078231111849, 4.3820450782337304, -4.0745178821825991, 4.5162176443285489, 4.3812312283592671], [4.1296835615160212, -3.8387900272376543, 2.6827385475473795, -2.6879163548122187, 1.422449892321862], [-4.5225418614956068, 1.4341155759654542, -0.77232708054148169, 4.5136648161142414, 3.2412817809698229]], [[2.1026983930761514, -1.1145219809021247, 3.7355651363576943, -2.0590331418260686, -3.5481213458988261], [1.126335632898753, -1.2229925405242961, 0.14779068355942027, -0.7703133734558163, -0.88089441493008369], [0.41739196511088039, -0.93295079214826337, 0.9456603961572565, 2.4444398670088496, -0.52647410196711064], [-4.0974237458825513, 2.0115672960619069, 2.8928205881495384, 1.1909144960823674, 2.961687472283197]], [[-3.42263546356837, 4.8799172103753783, -4.4167190652674044, -1.6248130797013838, 4.1602744563766123], [-2.2410238724682028, -4.0969193194085545, 4.9182923956272244, -2.2759283868787881, -2.5797403884637848], [3.6070594410609864, 1.222293491525174, -0.5483035853016025, -2.2243258979863425, -1.9062845081966726], [4.5902038339559805, 0.73570454920003669, 4.3585787542120915, 3.6949368591187888, -1.788299750826833]], [[2.8728666732140304, -0.29539450105486154, 0.78330977331414342, 4.8205542287005017, 1.3299198011489359], [-1.6489052436272713, -3.5318532859054286, 1.5601524600798689, 1.8444724638460883, 1.9832865409665796], [-2.38265989573335, -2.5801307364642803, -3.7762424739837308, -3.6501064461451662, 2.5762125377950076], [-3.5881982137565762, 2.8273862285339622, 3.0533419406497195, -1.255989325885698, -2.420133574178295]], [[-0.085015107703009463, -0.59497372809225801, -3.8117251282353513, 3.5569409377354528, -0.16428883628806457], [-1.8256096144483633, -1.7175290271371746, -1.7095052419078751, 2.6188108027628552, -2.0040895742065681], [2.0694485172815611, -3.5784880748612338, -3.9877965554233219, 2.6388962733013068, -4.73937059272677], [-2.9206149918762159, -4.8433697710098613, 2.729790327669205, 2.5393238504524636, -2.8573239191361512]]], [[[-1.4683568799219771, 1.4873233344312844, -3.6192985635138575, -4.9869689459344455, -4.4226127205564127], [-1.0308346893878872, 4.1142034440750148, 1.1238560051662354, -2.8456664420626101, 2.4848181376806107], [1.4203248503227304, -4.1546364851338868, 3.0852140919243336, 4.6895388665990581, -0.54651046163449202], [2.2530900815673327, 0.94741514824514717, 2.3774757628787313, 1.8544491955353539, -1.9484346872057925]], [[-2.0735100050238309, 0.48536060327257413, 0.48793402163348354, 2.6620817392600209, 2.1755579356837531], [-0.067424307941514883, -3.6207338148620671, -0.36219946898280497, -3.064866581793745, -2.4484263341107129], [1.9213573572868912, -4.6023739478525902, 1.3823889272931433, 1.5183925660005002, 4.8017998840523664], [3.6964235328385868, -3.2953417435752619, -0.20406514305843793, 1.2904142158541685, -1.7977454093831113]], [[3.7349559993037929, -1.4417244692193831, -3.8085705168502293, 4.795439063752827, -3.1008876328416859], [-3.9476255482181619, -1.2637187546403506, 0.44152644681840769, 4.0091770356203611, 3.2390555577263012], [-2.2103940267600941, 0.61937093066254523, -1.2794017787563683, -4.8969169907827128, 2.2811179664883632], [-2.2779166485853519, 1.0321727126857887, -3.951472140225988, 4.3090952090943659, 0.87281713084469192]], [[-1.8371466525055178, 3.9407647449294601, -2.8664272212459183, -2.6344758520087552, 1.5544732054387147], [4.1768038456600998, -1.6515316938662847, -0.35359882447436775, 3.4100339015063881, -1.5738880670910707], [-4.6585025206305026, -3.9102130743168209, -2.4220553345986797, -3.371796265601239, -2.0430427300222407], [2.960336402649232, -2.9065235578030277, 2.5397826654847719, 4.0177728171781766, 1.6388212844760419]], [[3.5891685471271408, -2.2872666648454487, 0.74152462263148866, 2.2965114988406201, -1.4546725695794582], [-3.7284947677180966, 1.603388215331103, -0.5506803875112789, 1.2314433675166292, 1.242165794570484], [4.3196409585479962, 4.8168884527648661, -4.3356589930153833, 3.971917234759804, -0.89759747585922955], [4.1871641074828485, -0.13811906324034062, 2.9052527479540204, -3.2616263458103734, 0.76921470880219278]]], [[[-2.7832243129071719, -1.9211224008792716, -2.9402358944611162, 1.5353323683653555, -4.1083691142124836], [2.6149976900884573, 0.96952542113016271, 3.7073606683995219, -4.3080163776672258, 0.63904956693620463], [-4.675424351370121, -4.2892317890011746, -3.9547105978282158, -1.3718177779870788, 2.2360452812056515], [-3.6966424651372529, 2.3609642172563028, -2.7092971675502753, -1.2868199399262559, 0.15876788036695544]], [[0.45059519161250705, 4.1613765894568058, -2.1337190585255628, 0.4987649728201804, 4.9967614115820211], [-0.89366477349946827, 1.622476648957015, -3.6211805301105149, 4.9023655911093211, -3.1883736783901515], [2.3627816301841591, 2.1385775152200583, 0.36829007405087921, 4.9388637266100091, 3.5081148113362168], [0.96188260286712435, -4.6839584053480019, -1.2200853426592397, 2.4077837836493421, 4.1788143472446428]], [[-4.1893374811421165, -3.0636735496597955, 4.7629330627455175, 3.0278011984023632, 1.2323839764199995], [-0.87918918218919728, -3.7112566462565288, 0.42835652871027996, -1.5845483198356813, 3.7265105972326005], [-3.8325037214840529, 4.3005620303910366, 2.5273817840653159, -1.855391939428217, 0.67653499504209424], [-4.0020571342948505, -1.1989596057388052, 1.8306814846584061, 1.9647733800388689, 2.1864405174349422]], [[3.020256007438018, 3.0844081148897846, 2.0015563858064329, 1.9955207445380125, 1.9296483562399356], [-0.13927342953120636, -1.7828186609904675, 1.0008641052202272, -0.72654921264372518, 1.6199852867583324], [-1.5921680941974383, -2.7231245140455016, 4.2803723314695681, -4.9055727212906275, 2.8848722936688], [2.113670998579332, -0.018184163707174683, -0.12973360059351968, 4.1253156706302168, 2.2430557276750278]], [[-0.97159887149925339, 3.7458484364508848, 0.17942606508145786, -2.340262492062041, 0.20385004339311141], [-3.4903747676651009, 3.5404973564542992, 0.46925099408466675, 0.71592817949230714, -0.054037503332176939], [3.8716887356334162, 2.7102296149054812, -4.8806794838596783, 3.225145298450018, -1.4013147105274237], [-0.48580814831072594, -1.876577890861145, -4.2813783241436489, 1.479582724095815, 2.6216418611051768]]], [[[3.3789618826225389, 1.744690451646064, -4.5169784126018762, -0.70548182803549508, -3.4547548614023462], [3.3205521085428185, 0.38269788322902798, 0.58894440943127258, 0.84092724636404803, 2.2192451864076679], [-1.4993363711172538, 3.0637318702801686, 2.8835924452351733, 2.280629234813917, 3.8147245758060624], [-4.9837066828169334, 0.61856230337714813, -0.40017672831588236, -0.52613814787581425, -4.2965486510219435]], [[-4.1889554238638818, 1.7300231828887167, -0.01810884777003352, -3.0747482522853788, -1.461305905771948], [3.8455020177321337, -2.4491493415247456, 4.414148708069753, 1.2631715027520105, 4.4470058275516919], [-1.6724411903437684, -1.2305380523104326, -0.86840427694642841, -1.7402561399249397, -1.1927340866945846], [4.9426811704979539, -3.1314812268423822, 3.9854055046912436, -4.5514293856599863, 4.6280712417349985]], [[0.81918821469619907, -0.37090733208877147, -1.6668126717508458, -3.670684379749205, -2.9937231179603749], [-2.6825357571247865, 4.5910801389096054, 0.41752809651273015, 3.4546869833874503, 0.21496814516912366], [1.6532901176768844, 0.29016623687880028, 3.7232602772206569, -2.7829024068701602, -4.2578816196445075], [-0.8565423293796437, -0.23674581846480969, 2.7205809055903067, 0.39193989276850782, 4.4694340562399066]], [[1.982439908431374, 4.714133943688509, 4.1614755555887708, 3.7390351344928057, -3.7158641079438604], [-2.8656111036447762, -0.7123257889714063, -3.4142136009427571, -2.3357254786061454, 3.2705437816705931], [0.21767413076696851, -2.9493345579279353, -1.5183540869884569, -1.7958717432397666, -4.680185678475115], [3.2952518910874051, -3.922172387446369, -2.8974428459799908, -1.6972742320580059, 0.39077222342841633]], [[-1.1152282296774176, -0.70564891516952777, 3.5929202470451855, -1.7588223513665815, 4.9661438169762988], [2.162834511222206, -1.6543528546056727, 2.0852978463170917, 2.2177373637079754, -3.5388983401769005], [-4.4445360896019288, 4.3716189102546466, 1.5974033285860685, 3.5612461988539685, -2.3978913495779617], [2.3962921104250388, 4.3977131079221436, 1.1009041428037358, -1.0658380436254133, -2.1343647829394286]]]])+(1.-msk_arg0)*numpy.array([[[[1.6040514509424018, 3.8531500595735704, -0.70051217155756351, 0.40958255448712233, -4.9748465081706552], [-2.6827424199235828, 2.5509047778222378, -2.3868450979334388, 3.118164224950414, -0.94999475925704591], [2.2685197999279669, -4.9208134167139894, 4.0975334277014923, -1.5655609877851475, -1.7756395650932633], [-4.8293779915702517, -1.7876743203751202, 3.4122740161479328, -4.9495873678827156, 3.421066693388866]], [[-1.8727935870933687, 4.2261140410436635, 2.0024810188503288, -4.9780895771073279, 4.5229824276954691], [1.1541262608402878, 1.5214105856719424, 3.9290806319533225, -4.1890524090412065, -3.3823240155260512], [0.031490664728362816, -0.060421416100982839, 1.5699253420943258, -1.6269463834493014, 2.1886617937779071], [-2.5137122690047664, -0.12430235563455305, -1.3319235019565365, 1.3471205884305535, -4.0791323007376015]], [[-3.6391151786374709, -2.3636817757214756, 2.6260742382058684, -1.3288714558759129, -0.19789742231352037], [-1.4068938380389131, 3.1597976520782414, 2.6915362954176185, -0.44265827031314675, -4.0459236641566809], [-0.37905776501332067, -0.8194544179823513, -0.16944978071139438, -0.99096091334217462, -4.2331878216054495], [1.7265021032348944, 1.7454079858637588, -2.6753164946302621, -0.30785740922678961, 2.8680886589277019]], [[-2.7955694603012304, 1.6052653858117178, 1.1485832848502762, 4.5829257048115242, 0.19208714281664108], [-1.1126590017911786, -4.8346836064434715, -0.55465892186210475, 2.8467360577781182, 1.6594807693604423], [4.8316471907778809, 1.2030457042201128, 0.98349261791271214, -3.2818163761563324, 2.7020891451544724], [1.4829322444962862, -4.1037812540904506, 2.6582007715601979, -0.48817740274726606, 1.0775143136746603]], [[-4.9412301975066271, -4.6031737857060717, -3.7281501328122557, -4.0456349777295362, -2.0148860863624698], [4.3557816972240406, -1.3242612093088555, 3.7870763265433105, -4.5433689099040242, 3.5603174099794863], [-0.34029215723879513, -0.068991513972839869, -0.89953772971630652, 3.8936241105607436, -4.9429121801923346], [-0.23084592421364736, 4.5934302264295148, -3.5865396586432077, -4.8503382037317007, -1.9989112701021163]]], [[[-0.60725041548171355, 3.9861767998355244, 0.62405923154855536, 2.1427017594418469, -4.0913760644811772], [-4.990897847860877, -1.8020906907565459, 2.3047703842155425, -0.31880102545621192, -1.7073803299880739], [-0.52355753713920272, 4.0386945841180211, 1.1624758825927293, -1.342572407998317, 2.3720150800721598], [-3.0552249642333673, -3.0252141182178329, -2.4575294362014768, -2.6687009739043188, 0.7575464906591689]], [[-0.63379397773712931, 0.15135009897431306, 1.4235263769187281, 1.8300689179635645, 0.45738063001318796], [3.0992614237528464, 2.979901436186414, -3.6629141524826094, 4.7294262174004409, 2.0384792984103361], [2.9974764797151243, 3.5028350167493389, -1.1784880567395417, 4.3457153512136131, -2.3378188164389071], [-0.5738023732416444, -1.2037939207804182, -0.34021873992893781, 2.4667980011452428, -2.2361942825841838]], [[-0.284752756680966, 0.5576263537376791, 1.8840273744580918, -1.4332402961698412, -1.3551622859121815], [3.7284931713151863, -4.107597294133992, 3.1942495290872799, 0.24174444925971272, 4.2311048072099471], [-2.1281408757401499, -3.8434568014913992, -0.89597124342817125, -1.9794842281295635, 0.77146817372282683], [-3.5447074378673946, -2.8022767472286225, 0.96734939552566601, 0.28497911056408931, -4.5717497145810686]], [[-1.4982818521873007, -0.70548991718680565, 4.8930775671415692, -0.21173265878830883, 1.4423668295820651], [-2.7644975856980745, 1.6304871305218924, -2.8179610400128907, 4.3649910149180577, 0.064465589778945009], [0.19576490194930596, 4.3363912969541705, -4.828409431048569, -0.5413931153172733, 1.7916469663899273], [-0.27614441032193682, 1.3872039638508804, -3.6483898951657654, -3.0674416073905295, 4.0663227256039445]], [[-1.093433167550768, 0.8097686043671759, 2.0015847581990336, 4.1800357885252097, -1.7923319058675191], [3.1569993193178831, 1.3203124378954563, 4.6123244565041652, 1.2999824222796397, -4.5507179532969424], [-1.0316662368293872, -4.4714498729796679, 3.5674534995230474, -1.4784383894308051, -4.0361554168907032], [4.6119841048653356, -2.0071527155420146, -2.3673459808272588, 3.5660631392773823, 4.3097937095812924]]], [[[4.2647718268969435, -2.3764861156332797, -0.8462025031288265, -3.2612777030122664, 0.2712277601567834], [-3.1461732976818779, -3.7926669653572653, 0.045696361459388157, -3.8027068355091362, 2.7472844771353859], [-1.1795750464787624, 4.2596223510779012, 2.400442895070249, -2.8868872102470169, -4.7409939885498513], [-1.0163907414500004, -4.7411689364238825, 2.799497501984149, 3.5506991944895763, -1.5235846323383404]], [[-0.19637798655730521, 3.3565472488413626, 2.1988231234077373, 2.3422815997155952, 3.4757074691934307], [-4.366432216134867, 4.817496532666004, 4.0769373724525373, -3.6844625549990981, 4.4421261419132421], [0.66962513274808622, -3.7418288405660904, 3.2516052730323999, -4.0610193867398685, 3.0159724760965663], [0.66899889349532504, 1.9079563361663077, -4.9132035117175645, 4.3857589638491117, -2.657698788839574]], [[-2.7333911027981905, 4.1957428448285672, 4.7608281028088761, -3.9014431729769274, -2.6430532575345236], [-4.8416543993051873, 1.7739035858772398, 1.5631926030067556, -0.53680402885166245, -3.5213558240649565], [-0.97611190548361737, -0.52360014967918289, -1.3333665321122679, 1.4791338512697392, 1.9777560851732732], [-1.3301234097468981, 3.7194445739729272, -1.2915553677117178, 2.7881061033682739, -1.1840016418316015]], [[0.57171992502612401, 0.44924034149949055, -2.4631008153021137, 0.35163197132076629, 3.6697357248211482], [-4.9631624778950965, -2.5921113688671649, -4.333938462566671, -4.8830829534818312, -3.9786885415078044], [-3.0780217016830091, -3.2950515965562257, 1.1185049798291331, -3.9325470345010469, -4.1387337824451009], [3.2518360315358361, 2.4417424025397114, 1.0529070044304607, -2.1732930411644027, -0.8720327573523301]], [[0.59132958914411304, -2.3075265533711109, 1.1954982440248276, 0.1991053768533968, -4.3854723539677307], [2.9040678188461104, -4.7930903840790071, 4.7007599758209224, -1.2479366376682788, -2.0042699710780489], [3.9420440402108596, -0.66671749151368775, 3.48740138494761, -2.6058747280728278, -4.6220707237798662], [-2.4036483191104674, -2.9803284614898939, 3.8197289315797249, 1.1157716188250069, -2.6638910677655456]]], [[[4.8821124453411358, -2.235150356618175, 3.815959124811485, 0.52004695917012, 3.2110918012686636], [4.8276162968323462, 0.094325822312750773, 1.316419760936415, -4.6181849681736686, -2.4744900167659378], [3.3054747840044509, -2.0593188040357546, -4.5096429179261381, 3.6328770886438342, 3.548351401904986], [-2.7649862439127615, -2.3788410811194005, -2.3566717138734159, -4.4517808575347289, 1.2335046579973454]], [[-4.1452776182046538, -1.2170627477247766, 2.0302264696596728, -2.8971062043949036, -0.4889233338774126], [0.45379851654108982, 3.8430573822347469, -4.1125939174302406, -1.0032020422094359, -0.36303795422064322], [0.50478388403598728, -3.1961445998348359, -4.924380223087919, -2.5598767053452187, 0.50973576344414351], [1.1641101713686144, -2.1928754307975193, 1.0689439979022204, -3.3135232793938654, 0.59277387505055845]], [[-4.9163053111999266, -2.0494014236662004, -2.2460340594839581, 3.7843365449177195, -0.32724058476541096], [-1.405098073929433, -2.1207318554501264, 1.9306658389096532, 0.070604578017126052, 3.5301554677509053], [3.4967441407541795, -1.5124871719131572, -0.0065547742530220887, 4.7255907901594156, 1.0306896314336758], [0.94843992044651326, 0.55329985233332124, 3.261196879333669, 1.9362766943576881, 2.4552057865735089]], [[-3.2533544513627453, 1.3118507768483543, -2.4053909984704323, 1.1218844119599938, -3.4713374588052295], [-2.7427897404417423, 4.3370927337162417, 4.8406175201447414, 1.4736967215116561, 2.1411769176599238], [4.5060863751965847, 4.1600438556810619, 2.9045251510100156, -2.3502165256129937, -0.87914549611768322], [1.887603496607082, 2.8793724545766741, 3.8094396683336171, 0.32526071455687244, 4.1971616638004967]], [[1.8061870227711463, 0.035652985317958574, -0.51603038511458177, -2.6258216435935089, 3.203674414638483], [1.0893373432076681, -4.6989556277922837, -2.4650904595338865, -1.0942393780875967, -4.0279687876155421], [-0.47994968352920164, 2.1209439912834167, 1.1852583314067733, -3.7098745192401763, 2.9221138522620915], [-2.535028775473628, -1.5230579954434411, 0.75306806256012315, 2.8011493193286103, -3.3911900298105357]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.7819552336438638, 2.2475962846297382, -2.0856460942865995, 3.0361600725066165, -3.836836736129924], [0.65234302576923664, 3.531275911683796, -4.1252317427791549, 3.9072963226504207, -1.1472310526013296], [0.35076325856506152, 4.627290574521238, -0.11025378459803292, 4.0374856449903103, 2.7166607387440713], [3.9746793855314415, -0.1288633349692212, 2.4297079994235666, -3.336094641248748, -2.4186104387934595]])+(1.-msk_arg1)*numpy.array([[-3.1344446081787458, 0.88903630450988125, 4.0986670130450626, 3.5422039150895728, -1.7898631883002123], [0.92466608314645349, 2.3798403826942778, -1.6908071309584161, 0.44898453983940545, 3.5781601399571556], [-1.4401530495287429, 2.2415874885397535, 4.3399552696048627, 1.9709150356797087, 0.21692781896233893], [1.025959538239456, -0.79867803333191212, -3.630051801869131, -4.5807893588299864, 3.0673197195940869]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-6.790213322035938, 5.2346151655103537, -4.8864564538931567, -0.47308219075995017, -2.0663401190408801], [6.8077957150469368, -7.8086081612219305, 7.2606084647307947, -8.0476976675860197, -7.8071579171787304], [-7.358911235736505, 6.8405519798960084, -4.7805199953001907, 4.7897466160545701, -2.5347420302190926], [8.0589671394655582, -2.5555297562418251, 1.3762522832557793, -8.0431486419889406, -5.7758190333136792]], [[4.7260170959748784, -2.5049954634137914, 8.3960423214699347, -4.6278752394977687, -7.9747443544576679], [2.5315477837493221, -2.7487934902122921, 0.3321737912710423, -1.7313534761798928, -1.979895014147943], [0.93812863001752012, -2.0968967341748077, 2.125462792924536, 5.4941139630899016, -1.1833012355350558], [-9.2093543877992872, 4.5211911809114307, 6.5018928060253165, 2.6766949967064257, 6.6566777589381543]], [[7.1384062867581761, -10.177780270261366, 9.2117128680361233, 3.3887850536249724, -8.676860171102188], [4.6739826868163377, 8.5447237771317646, -10.257817325499403, 4.7467811509697455, 5.3804254654728876], [-7.5230494351084509, -2.5492716466714098, 1.1435672311676266, 4.639156621555748, 3.9758348391194414], [-9.5735406982696656, -1.5344193195879399, -9.0904527553630015, -7.7063306288566968, 3.7297603907256835]], [[8.7224830868473529, -0.89686498974078421, 2.378253858140611, 14.635974276533391, 4.0378493998843386], [-5.0063402640479158, -10.723271928617358, 4.7368726063174709, 5.6001136495673967, 6.0215754080224873], [-7.2341368417883753, -7.8336899238999385, -11.465276623613009, -11.082307452184976, 7.8217936455441448], [-10.894344148847278, 8.5843971766298832, 9.2704348879105449, -3.8133846427486571, -7.3479129280528692]], [[0.32618908836094879, 2.282817056976552, 14.624967000042941, -13.647401658147807, 0.63034944240608104], [7.0045660345474676, 6.5898784666894006, 6.5590925127588076, -10.04794949301442, 7.6893645008107381], [-7.9401360946354957, 13.730074505430432, 15.300524320060573, -10.125014164238806, 18.184191196307925], [11.205922892922464, 18.583219064071812, -10.473759811133348, -9.7429710343469011, 10.963085379964314]]], [[[-0.95787236995737823, 0.9702450042800943, -2.3610241760848814, -3.2532144116080968, -2.8850605639332856], [-0.67245782034318557, 2.6838719233381094, 0.73313962693906887, -1.8563506571451012, 1.6209537824208491], [0.92653901043476805, -2.7102481356835058, 2.0126178958718075, 3.0591879736996663, -0.35651228815718683], [1.4697876011402899, 0.61803966446584924, 1.5509297328493357, 1.2097369993508595, -1.2710477793655628]], [[-7.3221359333760008, 1.7139422068167565, 1.7230296570853205, 9.4005451207822155, 7.6824953327525627], [-0.23809383449582197, -12.785810103041394, -1.2790262600436415, -10.822889532812907, -8.6460689351774214], [6.7848429535236354, -16.252252258612906, 4.8815967195286794, 5.3618630927973143, 16.956480263280167], [13.053091380894019, -11.636760919853403, -0.72061032409656967, 4.5568086365401594, -6.3483350594947057]], [[-15.407559046211446, 5.947447544765228, 15.711235990723377, -19.782297446356313, 12.791880093789835], [16.284870220115526, 5.2131327205877165, -1.8213989136917879, -16.538784369762347, -13.361854803357778], [9.1183876032401763, -2.5550486237237986, 5.2778288294938838, 20.200917412131425, -9.4101402443816315], [9.3969340661494023, -4.2579516384018845, 16.30073830356773, -17.776016339213658, -3.6005729338019501]], [[-7.1782763595043395, 15.397735596293302, -11.19998054071924, -10.293677808665143, 6.0737874392693021], [16.320010306579842, -6.4530237141843543, -1.3816153865622087, 13.324012923469176, -6.1496470568083188], [-18.202149767917277, -15.27836116605771, -9.4636879021332554, -13.174607149310143, -7.9827733460335777], [11.56691153987952, -11.356648809100587, 9.9236834691799327, 15.698628953805111, 6.4033603783144777]], [[-4.1176056102842544, 2.6240233434905766, -0.85070007335132647, -2.6346293041259816, 1.6688455431889229], [4.2774449769877823, -1.8394567500028687, 0.63175764061147255, -1.4127500707350285, -1.4250511720104633], [-4.9556262437348346, -5.5260840099286277, 4.9740026302774591, -4.5567067900788523, 1.0297516970422804], [-4.8036446864420554, 0.15845447830552559, -3.3329961681081963, 3.741839025896263, -0.88246700005556489]]], [[[-0.97625282931282398, -0.67385915343474767, -1.0313267232911394, 0.53853818450824542, -1.4410649378892262], [0.91724511091553618, 0.34007389597727944, 1.3004059087237607, -1.5110938625822088, 0.22415510848313452], [-1.6399670806610227, -1.5045049190509003, -1.3871671759760074, -0.48118327396422989, 0.78432252913472367], [-1.2966463568215247, 0.82813950220033061, -0.95032190291102603, -0.45136915531503025, 0.055689939072981143]], [[2.0850348830731451, 19.255898669426813, -9.8733380881916659, 2.3079304576321622, 23.121466982944924], [-4.1352465831957463, 7.5076709050995989, -16.756254535620204, 22.684670092597401, -14.753531470066358], [10.933277167003085, 9.8958195790608254, 1.7041851883453623, 22.853557570987331, 16.233066600834427], [4.4509103020429999, -21.674036580516336, -5.6456894061986151, 11.141515207565684, 19.336588241679255]], [[0.46189031225430871, 0.33778160362288201, -0.52513139595479352, -0.33382654113432009, -0.13587499747827791], [0.096933934714008477, 0.40918009086438539, -0.047227928447584312, 0.1747024491403382, -0.41086189668957013], [0.42254803977966232, -0.47415323974921247, -0.27865340681732947, 0.2045639832346452, -0.074590543616402327], [0.44124194523356536, 0.13218983411286872, -0.20183956207713502, -0.21662370102675454, -0.24106334184568376]], [[12.194240274226745, 12.453253487159129, 8.08125517533216, 8.0568863603526015, 7.790927538197888], [-0.56231447246081523, -7.1981047513698586, 4.0409744574127391, -2.933432016428053, 6.5406673403822788], [-6.4283558247337371, -10.994576134979928, 17.281941843522088, -19.806179442666959, 11.647630473318051], [8.5339163149963877, -0.07341829993387157, -0.52379755006924211, 16.655902801223075, 9.0563053014012187]], [[-2.6395045080100679, 10.176199380591985, 0.48743974651413513, -6.3576992305403053, 0.55379140947734107], [-9.4821640948187387, 9.6183301639065686, 1.2747957522464406, 1.9449339769872691, -0.14680156372227701], [10.518064780732976, 7.3627743877951843, -13.259150332195267, 8.7616256090441933, -3.8068966567143656], [-1.3197759230777062, -5.0980254792976298, -11.631052400910939, 4.0195242962751028, 7.1221115151123717]]], [[[13.430290139356316, 6.9345851722911505, -17.953540981459213, -2.8040640787597195, -13.731542929680439], [13.198130014408102, 1.5211013873569363, 2.340865203390468, 3.3424161908549017, 8.8207880938544392], [-5.9593813662572677, 12.177351907598274, 11.461355448350446, 9.0647700056552214, 15.16230713293653], [-19.808636215727748, 2.4585868358999963, -1.590574192606554, -2.091230450303692, -17.077403352149844]], [[0.53980276595650734, -0.22293655692110692, 0.0023335665160964638, 0.39622231398027824, 0.18830875242799172], [-0.49554421463583193, 0.31560555198655083, -0.56882192357194761, -0.16277649248270684, -0.57305600156587244], [0.21551634932759206, 0.1585712372272523, 0.11190547122885194, 0.22425520989139139, 0.15369969214293236], [-0.63693037931994012, 0.40353311428441763, -0.51357264453920581, 0.58651236951305952, -0.59638869468511657]], [[1.9903881582808651, -0.9011965118209414, -4.0498680820935977, -8.9186912008357773, -7.2738730077675848], [-6.5177785878258474, 11.154984139503329, 1.0144713560810754, 8.3938805990409584, 0.52230982193866626], [4.0170122242874555, 0.70501922690705454, 9.0464352794990361, -6.7616402395875257, -10.345409031848831], [-2.0811477495386157, -0.57522320895402768, 6.6102171893917792, 0.95229949275285841, 10.859419679342221]], [[-6.6136071551155657, -15.726796987668061, -13.883076300687355, -12.473775075622241, 12.396474338120072], [9.5599498467722483, 2.3763862474107951, 11.390139698183724, 7.7922012526061293, -10.910843584000681], [-0.72618150119016289, 9.839259213953131, 5.0653729331203259, 5.9911980989922329, 15.613542362009968], [-10.993272175421495, 13.084738283813639, 9.6661435517983687, 5.6622674702982971, -1.3036531205283981]], [[2.6973026379349521, 1.7066898323523003, -8.6898744152558614, 4.2539060989984723, -12.011167276288472], [-5.2310541262247776, 4.0012350835970381, -5.0435231390960373, -5.3638427383663965, 8.5592164673806987], [10.749601381905487, -10.573243130768775, -3.8634963654816841, -8.6132672316617374, 5.7995650491817949], [-5.7956971126724079, -10.636354829639325, -2.6626582518960809, 2.5778470183756235, 5.1621969442104385]]]])+(1.-msk_ref)*numpy.array([[[[-5.0278104216477049, -12.077485428733992, 2.1957165991021896, -1.283813829516238, 15.593380814052372], [8.4089075132618749, -7.9956697268223147, 7.4814337477753377, -9.7737130423116838, 2.9777059509513131], [-7.1105496554309431, 15.424017081872796, -12.843491559291117, 4.9071641969381465, 5.5656438608754319], [15.137417806534476, 5.6033661346793995, -10.695583891543322, 15.514207437969608, -10.723144051312621]], [[-1.6649814897792929, 3.757168809486779, 1.7802783248498781, -4.4257023611506563, 4.0210955828815109], [1.0260601458752567, 1.3525892447279977, 3.4930953251531305, -3.7242196731322097, -3.0070088434183027], [0.027996344196663341, -0.053716832483671623, 1.3957206244919504, -1.4464144003774833, 1.9457997929622783], [-2.2347814662371461, -0.11050930689521606, -1.1841283480692988, 1.1976391096674759, -3.6264967062546467]], [[-14.915521339352992, -9.68794452348539, 10.763403853941835, -5.4466016007757716, -0.81111563680307386], [-5.7663893648264564, 12.950958404470327, 11.031711028441704, -1.8143088505840792, -16.5828938595774], [-1.5536315574986843, -3.3586707916783043, -0.69451822656951168, -4.0616188067325778, -17.350427284438343], [7.0763572184817827, 7.1538461359652112, -10.965231465996403, -1.2618050079193572, 11.755340376835624]], [[-9.902477087183863, 5.6861773343800408, 4.0685162084030901, 16.23365737414802, 0.68041182932347588], [-3.9412650723043687, -17.125435198963441, -1.9647150045593089, 10.083719609088307, 5.8782192782444147], [17.114679595504946, 4.2614332035201761, 3.4837314016321024, -11.624862816226035, 9.5713507488872089], [5.252848402267313, -14.536430024910391, 9.4158891801146556, -1.7292239072696249, 3.8167754204634359]], [[8.844126035434499, 8.2390513083838286, 6.6728786831772036, 7.2411331199378459, 3.6063704345984671], [-7.7962533161331313, 2.370246390235843, -6.7783485081630657, 8.132008762704876, -6.3724810707866375], [0.60907640550898701, 0.12348537116508582, 1.6100494689063629, -6.9690544645708306, 8.8471365543270046], [0.41318262191914806, -8.221611670311697, 6.4194153083842869, 8.6814418016655459, 3.577777699034201]]], [[[-0.56150386317253254, 3.6858824882331791, 0.57704640528738838, 1.981283643254107, -3.7831566802229619], [-4.6149139643655817, -1.6663321404965421, 2.1311430037245325, -0.2947844955116683, -1.5787566821713714], [-0.48411589716831044, 3.7344439021212055, 1.0749020211092355, -1.2414311698443061, 2.1933218932546454], [-2.8250629008089314, -2.7973128893718355, -2.2723941179895317, -2.4676572766292324, 0.70047754631915515]], [[-1.5083285025272584, 0.36018907746384604, 3.3877655576216648, 4.3552719140833123, 1.0884928935675351], [7.375747492773586, 7.0916897742851033, -8.7171510184205001, 11.255279499142615, 4.851255353643217], [7.133515572602338, 8.3361882267756631, -2.8046134679516665, 10.342108884512603, -5.563635626783852], [-1.3655580595262797, -2.8648373851151159, -0.80966629623224828, 5.8705854990749744, -5.3217854572438998]], [[0.48146199155624408, -0.94283861531000801, -3.1855269196546034, 2.4233329131409196, 2.2913180566262241], [-6.3041628417894762, 6.9451547960272482, -5.4008598818413356, -0.40874323867793727, -7.1539821798630125], [3.598275768385534, 6.498544167492283, 1.5149145675220308, 3.346926048541182, -1.3044038894380217], [5.9934166131075273, 4.7381095071331094, -1.6356012560831095, -0.48184471231594911, 7.7299470183707744]], [[-0.67270538795404733, -0.3167540658294582, 2.1969161798815748, -0.095064690375042685, 0.64760040725952561], [-1.2412166764017973, 0.73206351401144454, -1.2652209408355601, 1.9598134822361235, 0.028944053162375503], [0.08789541441841546, 1.9469726510265708, -2.1678811865555874, -0.24307713875294812, 0.80442178875924819], [-0.12398457099761877, 0.62283313337298674, -1.6380706582357378, -1.3772338585784831, 1.825716037793804]], [[-3.912478975837252, 2.8974817427353647, 7.1619907985335631, 14.956837442095283, -6.4132505831485975], [11.296249126255121, 4.7242893375671793, 16.503635522812754, 4.6515452860459581, -16.283197588674529], [-3.6914670063625121, -15.999563703312335, 12.764919913144032, -5.2900893144437608, -14.44201043139047], [16.502417690145126, -7.1819138415591999, -8.4707430260838734, 12.759944981532811, 15.421132063061867]]], [[[-6.1419241520499011, 3.4225037265919847, 1.218661115399835, 4.6967390293532096, -0.39060948590664207], [4.5309710690024581, 5.4620208960061891, -0.065809754308105567, 5.4764798456222783, -3.9565101176695041], [1.6987686003343985, -6.1345081187456332, -3.4570051555550232, 4.1575594194827667, 6.8277569504075064], [1.4637582258119981, 6.8280089021218009, -4.0317048646305702, -5.1135502729034146, 2.1941950544771895]], [[-0.44019843769148326, 7.5239943176953288, 4.9288544029426866, 5.2504291285593574, 7.7911023767681646], [-9.7877398252448273, 10.798839953707759, 9.1388118056497447, -8.2590451652791916, 9.9574143822280892], [1.5010233195798817, -8.3876367132701599, 7.2887576976993165, -9.1031302480334713, 6.7605661681983236], [1.4996195495060598, 4.2768510518305431, -11.013375520515673, 9.8310624211152415, -5.9574643533700451]], [[-11.862795120480055, 18.209336269320637, 20.661781012468303, -16.932088857625132, -11.470732912883253], [-21.012563523870114, 7.6986622152988895, 6.784185974826511, -2.3297054737598932, -15.282526764804482], [-4.2362820079276693, -2.2724012287660647, -5.7867511073553981, 6.4193747522690394, 8.5833729438408302], [-5.7726761013558381, 16.142223078817018, -5.6052925240869156, 12.10025577553062, -5.1385141646878685]], [[1.1268113964316635, 0.88541454369523298, -4.8545624312738846, 0.69303673930179444, 7.232737317020975], [-9.781971552204805, -5.1088312710566068, -8.5418244795832532, -9.6241416134886215, -7.8416570687443024], [-6.0665192519954854, -6.4942667349930945, 2.2044782822278677, -7.7507160788157634, -8.157092640496602], [6.4090925281190145, 4.8124668144222129, 2.0751902462044769, -4.283375931769001, -1.7187024730709424]], [[0.1282758380609284, -0.50056670242047829, 0.25933682664961188, 0.04319149514448193, -0.95133095286585423], [0.62997309806100354, -1.0397546431076183, 1.0197256090202897, -0.27071217301257433, -0.43478191343767125], [0.85513901589642849, -0.144629571298106, 0.75651437628292517, -0.56528672124991652, -1.0026557211992457], [-0.52141818741712576, -0.64651615294238585, 0.82860546615493491, 0.24204190373178691, -0.57787207928363604]]], [[[5.0088498300552926, -2.2931738277717382, 3.9150196616322295, 0.53354713809300958, 3.2944502616741009], [4.9529389866953863, 0.096774477104046755, 1.3505934100596193, -4.7380709174518536, -2.5387266349793252], [3.3912833830593718, -2.1127777692763514, -4.6267111657003337, 3.7271849003457271, 3.6404649658097656], [-2.8367640100431846, -2.4405946971303085, -2.4178498233475572, -4.5673470329395798, 1.2655258693351745]], [[3.3107421757224857, 0.97204128179435767, -1.6214972840061783, 2.3138550856798021, 0.39049232675129375], [-0.36243890671997681, -3.0693655120249339, 3.2846384218659685, 0.80123543410639009, 0.28995043930178405], [-0.40315979975950633, 2.552690483240498, 3.9329943119544217, 2.0445172925972939, -0.40711475706650929], [-0.92974922225036016, 1.7514014364112322, -0.85374208998649692, 2.6464382561858004, -0.47343547273591663]], [[17.846442953460073, 7.439433330732653, 8.1532199846891817, -13.73733769375777, 1.1879002743723881], [5.1005787950703834, 7.6983664931579971, -7.0084170073411638, -0.25629827565127805, -12.814647216587339], [-12.693362368820036, 5.4904067837073027, 0.02379417008802822, -17.154139362714357, -3.7414567537536452], [-3.4428860421814806, -2.0085071259364971, -11.838313608075172, -7.0287847032703308, -8.9125241895106821]], [[14.902931451304632, -6.009312078959792, 11.018589489618792, -5.1391161761435766, 15.901465692202921], [12.564142056523593, -19.867308242866216, -22.173849226445029, -6.7506942600432325, -9.8082804397889696], [-20.641432517469301, -19.056284626369877, -13.305017904200739, 10.765846851474384, 4.0271803334791922], [-8.6467140109479956, -13.189798700033007, -17.450240695807466, -1.4899508200875589, -19.226313486826477]], [[5.5401530722208712, 0.10935910492817279, -1.5828301761716876, -8.0542345075313264, 9.826693707179663], [3.3413459141111121, -14.413199258624884, -7.5612205771115395, -3.3563820223644552, -12.355068092162639], [-1.4721591287020614, 6.505613328618213, 3.635566252737179, -11.379371270085025, 8.9630574419425564], [-7.7757437527487099, -4.6717058235091082, 2.3099005184871793, 8.5920205447041997, -10.401864051328715]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.656475402390905, -3.8549778701956736, -0.14929182199206004, 4.0902596180094726, -1.4340524237059213], [0.5960890494312574, 0.43453271034418961, 1.0112785562103177, -2.6245491532008192, -1.3492885789442663], [1.4922238566492476, -1.8173094652462307, 2.0277396394614975, -3.8809387968725084, -1.9952151304665788], [-1.204225114215288, -2.4552852955473981, 4.0905772832346123, 2.9261408473517339, -4.312547733388282]])+(1.-msk_arg0)*numpy.array([[-0.25445255230663744, -2.4231016756541868, 1.5723527382444598, 4.5244305693310967, -4.9440481907210065], [-0.43473272628939519, 0.72957379376250664, -0.45481601373428937, 4.2176337964718584, 4.1062359014845811], [-1.1015644451347759, 3.4510770978195016, 4.483915827883461, -2.5670457966301843, 0.83145330706002607], [-1.2193221317072145, -2.232244562700394, -1.7614612831351071, 4.7671319350421175, 0.43972427231737576]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.3175612803490742, -2.0067070048281446, 0.93187235152063508, 3.1557275282959516, 1.2160410166686813], [-1.1822109856659013, -4.5130257626084669, 3.8494980801849152, -4.1589260664421612, -1.4383954292384904], [4.681243267553592, -2.8676069767453374, 3.1416294062722727, 0.49042629909485669, -4.9577868555642324], [0.19652350713941225, 2.6122844601104598, 3.3283041478420206, 4.47303099514804, 3.5770798405791542]], [[0.19449345775668192, 1.0802655115727049, -2.6908710476035793, 1.6111923313884189, 3.1806525154135876], [0.76320983615098736, 4.1818636797421718, -4.3675822760099141, 1.6262599963194777, -4.7585731126669382], [-3.5360872731326776, -1.4853095939445926, 4.1613354882879179, 2.0347466661295419, -0.86650901770844158], [3.1227532983959367, -1.1812139779223276, -2.5803397029932573, -2.6374391244391648, -3.1339523977457961]], [[1.3615326615876757, -4.8089640673101002, 4.4813224625444388, -2.4464330990694858, -2.7439766844587044], [-2.8693258526799967, -3.4791096758185489, 3.855872166717095, -1.8054826976447504, -3.2555470898026928], [-2.9920704205774209, -0.68122099304886419, 3.2448234796267705, 0.0060282290831707996, -0.9026358565389927], [1.0614802943365653, -4.0078448924783592, -4.7848215929155016, -4.3981667050431845, 0.86361964484558218]], [[4.7928513166661659, 3.8797063337081319, 3.3985172902713963, -1.2356115581755942, -4.2970387844176461], [-0.97971909082100339, -3.7633779711801063, 1.1855274015159454, 0.86922674156974722, 2.6804428829727813], [-0.61799596088439834, -4.3253200534075589, -4.9527225641855113, -4.2414261003310827, 1.5803897680093693], [-1.4086759647718505, 0.7254992249045058, -1.2050592222214696, -2.9028889221698195, -4.7322254234004992]], [[-1.144196333528237, 1.6854803771676687, -0.88610098636743473, -0.076957734955623636, -2.2088508396396955], [-2.5757681820581233, -0.28849678559196157, -1.6358092868853378, 2.9091430779201506, 1.2305996711850709], [-0.0015373190527681047, 0.90308751824614308, 2.5708905763754464, -0.49518458159498913, 3.3422361476688529], [-4.3345711797450974, 0.67345540103631585, 4.9486077964243655, -1.3633530556681892, 0.0094558789654470843]]], [[[-1.575133749218117, 3.5844484182288241, 4.5959502827160019, -2.8888745950368735, -1.6200931033176893], [-2.9825529304464569, 3.8383472873970526, -0.94746609928955206, -4.58177613656148, 2.2322197916876041], [-2.9396867885635034, -4.7451002756419882, -2.5031455312798769, -1.9330189008120859, 0.5602625150645304], [3.7417673091588632, -2.9418189771933276, 0.84273679566414295, -1.238232631067171, -4.4308461654462308]], [[-2.8085958838925507, 4.0966015246712715, 0.14963317696806655, 4.5438818732836097, 1.947795717958388], [-2.5643977930002491, -3.1646112459363076, 3.7692308116840749, -2.732454181336923, 4.8719500226450805], [3.5147933781493848, 2.2481547383617926, -3.0125835977275308, -2.2467908464305841, -2.6892581116351555], [-1.0528281889469273, 0.42035772867274535, -1.4157425342324159, 4.2873836734176898, -4.7668588868268689]], [[1.9498761615601579, -1.2232103543373696, -1.1259581246349648, -3.2338374900626423, 4.7044114600414115], [1.0831549079694467, 0.98210108535964569, 2.6635128288229781, -0.044939312731364645, -1.5332068760452278], [3.6599557891649734, -4.8843936398783079, -0.43440625591425341, 1.4026976711913566, 3.7513280533774989], [0.72226977321868358, 4.5376318434381098, 3.335228142961741, -2.8386299717806618, -4.7422576966623078]], [[2.0050744474397693, 0.4820070772267453, -4.2071374214780066, -0.57405291590297747, -0.052623122939782441], [2.6841644685981647, 3.7035746771763343, 0.11421773579612804, -2.8214807263477528, 0.12308271463891529], [1.2089722367018831, -0.8499773269459201, 2.7430364516518271, -1.5786783616863276, 4.826775516169203], [4.1661962755187183, -3.7191348132169311, -4.153796158783452, 1.7917575707740108, 3.334367471417087]], [[4.8944280893455332, -1.4821140881165116, 3.4671113778300242, 2.6287225813111084, 4.59957014383345], [3.539612627070488, -4.4037157324836604, 0.30258193221252316, 0.64579520566968451, -3.4435139344261776], [1.5537971846791514, 4.9847565549822406, -0.46427570674544061, 3.7268422582813869, 1.5544831477073116], [1.2188589794602498, -4.6226948613034526, 0.79874604821996176, -1.2555503517102586, -2.4918910536680738]]], [[[4.5169457515145517, 1.4844148042902683, -3.6809550497812706, -0.028132164192641085, 1.0903268957512866], [2.2171555177300197, -0.38925865154071015, 4.1057848517925848, 0.12844694723941608, 0.89336660479766294], [0.5759307982865014, 4.8482689312676932, -3.5582228999325061, -0.40160715870683017, 3.1679038033041955], [-3.5838242990390556, -1.5787055063460151, 4.626065556064253, -3.9625976696890208, -0.53020841112077122]], [[-0.10586205369016444, 3.7580024583969003, 1.3826022716842434, -2.802184850840761, -1.8736274853560353], [2.5315359077112909, 3.4528981229902058, 3.0543548511097143, -0.55002803558036817, 0.057032801521695653], [2.4774276683806633, 0.85807858172219476, -1.228007778619439, 4.5594999188637519, 1.1693057479575302], [-3.8263370410920294, 4.3802540579589522, 2.137670846739212, 0.65551005384505867, 2.0349515565759564]], [[-2.2098715374540467, 0.84775936349345571, 3.8789173888433801, -2.6283002866674621, 4.67809989704255], [1.469888901564234, 0.023784156659919731, -1.2829331019372714, -3.6984021126120914, 0.079962700825468858], [3.1212550825716949, 4.515762515653293, -4.9888886827842827, 1.3688556388171227, -2.2347365621400939], [-1.8891236383075318, -1.0775399020450793, 2.7123267133976441, -0.13443379344773199, -0.14374396691978752]], [[1.2769575810905112, 4.6812214968230403, 4.954673742540173, -3.9298821870802536, 4.5871657699692339], [0.93801832382337746, -2.9978365905033613, 4.3231136950490683, -4.9148275819813136, 1.7935638225438923], [4.1892299068427725, 1.5582702666048762, 3.8652298762841646, 0.26044924336887743, 2.7856594854041328], [4.0731072205306056, 4.5431291496715414, 3.0795522122505066, -1.2346809284411875, 1.6287841846036404]], [[3.0486763710392388, -2.6878162469086786, -2.9708384055464601, 3.9132671093641633, -4.3868276312804966], [0.84622046627436553, 3.623276445822178, 1.5770053085573421, 2.3412428112376862, 0.19135268902312141], [-4.2251004061738815, -3.0036705929738385, 2.0344693258570494, -2.8114665901927394, 0.89393650506558942], [4.505366196965662, -0.023272997821049302, 3.2245490897155786, 2.2034291977357228, 4.5753637601397372]]], [[[-1.9144356682428567, 3.3028051665443776, -2.9869226996731637, -3.5603860923150279, 0.048386378489794879], [1.9376810332723773, 1.2513361424461031, 1.8432859952488041, -1.2380708954255413, 1.7053732320852184], [-1.6040872312481271, -0.55536272128915964, 2.4763675894085146, -2.983224117948037, -1.8700953345558613], [1.7945400096966067, 2.0906401457854376, 2.6114120639339289, -3.6508861066287315, 1.6747824230592121]], [[-3.9320105740991664, -1.4190287238138302, -0.080646029630849547, 4.9898153305795105, 1.6172540334482557], [-1.9608121093462696, 1.7358747329116451, -4.7446038703729432, -4.7616463696738043, 1.5573101685234043], [-1.9373784688011861, 0.62986247865274869, -3.3941137771477035, -1.3505361074135172, -3.4162731509014046], [-1.7604542340436948, 3.3251388749284576, 3.9200583649454419, 4.1416164991985625, -0.33839440557876799]], [[-0.7519415069670119, 1.1406836476611097, -4.9638627712056049, -2.1410720731695019, 0.40138087767519437], [-1.0512021077944755, 0.33218492077483575, 3.8887883987014362, -3.772224919599223, -0.8719852859869377], [2.8276493648180354, -4.8590316109420186, -1.2232535870663499, -0.81999703452155703, 4.9501768045085335], [0.88517075453070948, -0.096832444574722842, -4.4889494404339194, 4.571669574624849, 3.5638527856373088]], [[-4.8358831574350187, 3.698039155777332, -3.6405612257935127, 3.8384673680670858, 4.7804893327781777], [-3.975951618491572, -1.3835244217406304, 2.990744795356111, 4.055082260067925, -2.8475822755448297], [-3.129750953313045, 4.0905631709425947, -1.5925958437530765, -2.0390510552205434, -4.4947012317113977], [0.59417090825735297, -3.1799646389311365, 3.6512301294072049, -1.1741760917376154, 2.9599446129410403]], [[1.3840253851833504, -3.0093552372411492, -3.3018916756624881, 1.458258105810085, 3.0736719073521925], [-0.026251572322316896, -0.61168135125820289, 3.6501536007560755, -1.8821028191759002, -3.8453899039272352], [-4.0920529639067169, 4.7303440438171869, -4.0904835677625488, -1.7700217349101219, 4.8850163116394949], [3.750601865814799, -2.1936945751745416, 3.1522036133605784, 3.0299128116803171, -0.7115023306585968]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0242634379262681, -3.6506349163539689, -3.8492453694131834, 0.90913212389383435, 0.85483754618654739], [2.5100010039183847, 4.8394553315933422, 1.1594341920628803, -1.8026233836413708, 0.19077376706729865], [-1.4713695120118029, -0.081959878190741797, 4.3255593285079215, -2.4202864647685249, 1.9974093284924361], [-0.6239741622379924, 1.0649557073743932, -1.7659695187165014, -0.24055793746621212, -4.5714139755419856]], [[4.8948256389006399, 4.5564919133882995, -3.4168621143880906, 3.0097717468906247, 1.2888550023831771], [-3.2143269433746182, -3.4266950395966731, 1.2242598144105701, -1.3397403297349095, 4.3003470720483463], [-1.9837063355612283, 2.0525668763656864, -4.1572939600639174, 4.4807273000486028, -1.1871106119345241], [-0.42940024834606927, 2.5851564545325587, -4.1045071497091925, 4.9748269936652481, -1.0708109893989279]], [[1.402716206655068, -4.1521679694034406, -4.1505862596209573, -0.21063490129220774, 3.1129108557252678], [-3.6845099047273511, -0.398684675926817, -4.9187250946167449, -3.9356809382099902, -3.471427782256904], [-3.5351513128337873, 2.2891744597651087, -0.72455259488397949, -2.3210539434128554, 0.25605988103443167], [1.6495694698204488, -2.7435810967594634, 1.5464197234105903, 1.6544362682747105, -1.7252859096324622]], [[-2.9657586735674957, -2.747628135142437, -4.4859984543232407, -4.527481693190671, -2.2163133232349388], [3.8426532662017525, 0.29357243509922437, -2.4387986848344512, 1.6771608266532958, 4.5888463214079849], [0.15193017638992679, -3.8749300976050636, 1.9823031545437324, -3.8571283347243321, 0.91852312731161145], [3.7637242842714222, 0.7617046796131266, -3.5703458135568455, 2.9014661269117514, -1.0921555214826473]], [[-4.8623578062207162, -0.22766972747572201, 4.0273179871871339, 0.27472158203461294, -2.8076989013947995], [2.5243811401297069, 0.47952631844994364, 1.5246852092479575, -4.2290055690448565, -4.439021304156606], [-1.6405543830877756, 3.0544805444936536, -2.1856627809500715, -4.9950918278275278, -0.3012483213424435], [-2.8380735761302578, -2.4943860088868783, 4.8859714648838786, -4.8462259168751478, -2.1431639933239777]]], [[[3.548023098228084, 1.5158281275748822, -3.6023827166923725, -1.4401952778036842, -0.64939636624009278], [1.7955448847906759, -4.5561170541628275, -3.7899840913249117, -2.6404859900223254, -0.2111979988998911], [0.30699626972924055, -2.1858603593404435, 0.52483308741635071, -4.0351581665878298, -1.0469483748869846], [-2.8056786814118082, -3.8247201195125324, -4.3581470316306339, -1.713665500628899, -2.5322862222490694]], [[-4.6057653340510019, 0.2837987262492927, 1.7310245063352045, 2.3972410349936908, -0.099871807458038475], [4.015489340614895, -2.9985942470858795, 3.9366153051391901, -2.0466733185374073, -0.068586561356785758], [3.4493962963899918, -4.223011074266342, 2.0056308307545265, 2.84747020165387, -1.4191524516819367], [-0.1425046889062509, -1.5239183935688869, -0.33048081733181878, -3.2902490035383982, 2.4825343401285984]], [[-0.20706744711090597, -1.9945100398706073, 2.0159826274101444, -0.010574596147239212, -3.8044782344377372], [-0.031143916992598442, 1.4665890820743721, 1.2085692711741389, 4.1709134446342606, 1.5262825112293754], [0.97975726812357955, -0.2210475359812305, -4.9292453802751366, -2.0432398721336642, 3.5951208048255108], [-3.723204429803133, 3.8054115142134695, -0.78860499390962246, -3.0745318952473442, -2.4231243559868276]], [[0.8260858615278277, 3.3012797015993094, 1.5092247183881691, 2.855676109729524, -0.63837432901349977], [0.33988004198005761, 2.2527684296725941, 3.4574548856170697, 3.5575312825500287, -0.67159225754403717], [-4.7983045108716302, -3.5163751111653152, -3.1770787336527526, 1.5028738324747888, -4.012202937857138], [-1.7799706935210313, 0.20078337705806071, 4.8753191029954852, -4.5924505471177604, 2.3314716532484026]], [[-1.8055332766248808, -4.4158061969650344, -0.41751560564028534, -3.1573168933974802, -4.5705146597084845], [2.0103590628243504, -4.5350930686577495, 2.1135409352851404, 4.4704893193661643, 0.32625704613163276], [2.0318047626043807, 2.7701960878193326, 3.4757403655485781, -2.8974972649255051, -1.2003175992488444], [-0.26674016101319964, -0.69134822435575138, 1.6951594468415632, -3.454332821997701, -0.64932645295580294]]], [[[-0.18642779103007623, 1.507916454379937, 2.4299466494436341, -1.8881133186233203, -1.2479473262020004], [-4.4635330495521348, 3.2241479800518924, 1.0981244573693827, 0.88420486710761192, 1.672939768072891], [-4.6544814340070388, 0.2594635651552224, -2.5672901476035772, -4.3062359771095684, 0.099458648779039649], [-0.7694146237520707, -3.0659274761755939, 3.7684509950237288, 0.58811460284928074, 0.044752030375278196]], [[-3.2669502323781794, 1.2133827930536176, 4.9735772452100608, -4.054429897126405, 4.0822955920644173], [4.4152595667008789, 2.4729430029506236, -2.852554034993442, -2.189644114561704, 3.3358616705604316], [-2.9751319477709268, -0.39017889691170904, -2.0089406689164058, -4.6899783314727506, -2.1557409886498782], [-2.5573037542622314, -4.5575803329547835, -4.1643124410800434, 2.5772245754957135, 2.6370251274254244]], [[-3.5047930719071907, 4.9957085055487216, 1.9994563302944002, 2.7926506786094123, -1.9053056072836028], [4.9677394285953618, 3.9975736728094837, 3.8164709727542405, 1.0874701828634041, -1.8456194935375212], [0.5138282126502256, 3.0412022044437386, 1.0769992058536273, -2.9292062502262741, -2.7988737144250688], [4.3025102435802083, -3.7318411639139537, -3.6485571722029131, 0.36464219225518413, -0.99441747772393452]], [[-0.86577736570638919, 3.6962263258990014, 3.7577475322873575, 0.058427311195339726, 2.5604188052741357], [2.8066450695833902, 1.0907262918301619, -2.329343022415804, 3.6994187110594439, -1.8945699740618762], [0.37350808934901725, -3.9182565253122759, -3.0483638826942192, 3.2837033317022986, 2.8499137565438906], [2.4652480294579826, -1.7165960854932747, -0.091312129284180088, 1.2387813193973667, 3.8972692142680394]], [[4.3417060524669076, -0.26978443728363022, -1.6985136427304237, -2.3864417002044904, 2.8404676775533355], [-0.69952166151337369, 2.2486288755223658, -2.0544977306530541, 3.5687400542561463, -0.89697589073688988], [-2.0327797473552267, -2.7952881626113815, -4.613917344211373, -0.082721291494539884, -0.87727031724401883], [2.5923597949915846, -4.4574253075311692, 0.15513208278540436, 2.7245219727331618, -3.6790612006969257]]], [[[0.69368953277653223, -1.5121147861312609, -0.30533286395516157, -1.4085336755872624, 3.5931648813028545], [1.0475066088696945, -4.4764639506759831, 0.78150273636533729, 4.4583295133518916, 2.8030320239453923], [-4.2766355734429373, 3.4098960096893123, -0.95378460211478, 4.6392741465605134, -1.7330879826008152], [-2.5045555071049419, 3.6741213638843213, 3.0555615113817716, 3.794855105108951, 3.0127095461804512]], [[4.0891434327630893, 1.985290406312834, 2.4329741481835319, -3.8352074905944922, 0.34507268619215203], [-3.1332390293109222, -4.5317716703202198, 1.0348891860028653, 1.8136000360391522, 4.1461113968469938], [-1.4220126013748002, -1.7354225067644014, 4.2530829204921403, -4.8642331746746841, 0.60469716137445673], [1.2660582744534317, -4.2163365274300357, -4.3701599582206407, 0.082756043589764516, 2.0624291489391382]], [[-4.4980569514905326, -3.1232172310239328, -1.636738765453325, -1.3666289669547549, -3.5957381075517425], [-3.7199428043403824, 2.7261212328128535, 3.128557833267859, 0.45140751944251889, 3.0775433944589867], [3.373833819824636, 3.5131273840156663, 4.6853360312835406, 1.488895654711837, -4.2344204327678492], [4.0637926531066366, 2.7885282166091123, -1.994176583381936, 3.1472610077798695, 4.9228479958295175]], [[1.4842732937174485, 1.378272653195908, 0.075281361838722916, -2.1368206328099393, 1.7544817140192936], [-2.0181008991018459, -2.3329020703164649, -0.68697179604265557, -1.2402142059560548, -0.61736996529686294], [3.7097414876083104, 2.753308106642665, -1.7865994464451109, -1.795279918589193, 0.71866784600092615], [0.44711866902004793, 4.5948039911301812, 0.85358554939434317, -3.739115777435924, 0.65714668388678543]], [[0.35983488125630281, 2.7745145942633309, 4.2751623578392657, 1.1432460396068267, 1.4401408860205542], [-1.2198328615519802, 4.5619622863655209, -1.7942901191154292, -2.5644221073281823, 1.8288734404055713], [-2.0765819912235783, 4.5811290914627598, -1.3881840460243069, -1.8575932445312846, -4.1747159414145241], [4.852837638076096, -1.8621720656265204, 4.5109962665729864, -1.3740972003321081, 0.69970622462826793]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.4741058151299615, -7.3374748029596377, 3.4073683315033731, 11.538840083861995, 4.4464240657474612], [-4.3227253895236748, -16.501767691334315, 14.075595041747155, -15.207010862308126, -5.2594575059220476], [17.116850860417735, -10.485334374193874, 11.487290647462508, 1.7932316993259485, -18.128025687667567], [0.71858336984685423, 9.5517538724419015, 12.169862248259971, 16.355527807890919, 13.079504449466057]], [[-0.74976797554984587, -4.1643996410483854, 10.373248340062048, -6.2111107821313292, -12.261345059701585], [-2.9421570286777223, -16.120991941581121, 16.836933020277073, -6.2691962969960846, 18.344194043039192], [13.631538185007036, 5.7258356150457264, -16.04185621780983, -7.8439033693838089, 3.3403730875910331], [-12.038144859396883, 4.5535537448563739, 9.947152452626284, 10.167269458701234, 12.081317139556713]], [[-0.20326569175012299, 0.71793900750307249, -0.66902479536720449, 0.36523245474176547, 0.40965327872657198], [0.42836688443551796, 0.51940262241315649, -0.57565018113766742, 0.26954380150652446, 0.4860265566175927], [0.44669164461655253, 0.1017007232315054, -0.48442560931609668, -0.00089996530321209429, 0.13475615161806995], [-0.15847032715017401, 0.59833846625966625, 0.71433473351330623, 0.65661032082071236, -0.12893135028713276]], [[19.604006195683148, 15.869006146501954, 13.90081803350407, -5.0539720599513949, -17.576004216924009], [-4.0073054341781047, -15.393192942824406, 4.8491148564643733, 3.5553630399366929, 10.963707282604458], [-2.5277639228984161, -17.691681949419515, -20.257921103492325, -17.348533900955623, 6.4642044488040815], [-5.7618504135668349, 2.9674801825240724, -4.9290050739623803, -11.873569333918256, -19.35603055265284]], [[1.640837525291597, -2.4170672199860657, 1.2707152671484274, 0.11036142633602998, 3.1676079001901649], [3.6937866043850467, 0.41371951460952, 2.3458362725785733, -4.1718636817986958, -1.7647444410746607], [0.0022045961136313917, -1.2950748443594469, -3.686791862133922, 0.71012064941809672, -4.7929418481620605], [6.2160023060392922, -0.96577035011397194, -7.0965630044323804, 1.9551197538478406, -0.013560226148669231]]], [[[-0.93891997929851989, 2.136650450357394, 2.7395956352575004, -1.7220265112816386, -0.96571975794677734], [-1.7778671411882396, 2.2879967859315542, -0.56477416649385048, -2.7311465819497518, 1.3306017737487033], [-1.7523151034204445, -2.8285023127634301, -1.4920976403287216, -1.1522513991177303, 0.33396635003678143], [2.2304265185094607, -1.7535860777140047, 0.50234617544818272, -0.73809691202759475, -2.6411788789369757]], [[-1.2204267816893648, 1.7801073637155471, 0.065020509945345722, 1.9744653058817605, 0.84638095252126511], [-1.1143147233930564, -1.3751271018824065, 1.6378540805139108, -1.187340721307647, 2.1170216480014026], [1.5272926929070623, 0.97689677173348222, -1.3090661158589938, -0.97630411607599765, -1.1685706160639213], [-0.45748828646987288, 0.1826591831542955, -0.61518644054956328, 1.8630084478956166, -2.0713561119211659]], [[1.9718679494514726, -1.2370064010758062, -1.1386573066341241, -3.2703105079693464, 4.7574704291299517], [1.0953713314834617, 0.99317776765508847, 2.6935534079797603, -0.045446163296058384, -1.5504992359787495], [3.7012348062603482, -4.9394825480989937, -0.43930573128969597, 1.41851807572197, 3.7936376176908588], [0.73041593345494393, 4.5888097792460538, 3.3728447010463687, -2.8706456194776826, -4.7957435166579252]], [[-5.262416443132647, -1.2650512663722562, 11.04183895693958, 1.5066300943256206, 0.13811197275038861], [-7.0447215831110404, -9.7202137827991457, -0.29977006176424276, 7.4051148511084266, -0.32303663447922321], [-3.1730070600792275, 2.2308072736758104, -7.1992339963817829, 4.1433189573403073, -12.668109593652328], [-10.934386906981059, 9.7610521246681827, 10.901842191103924, -4.7025558151160869, -8.7512113235680715]], [[-6.6039959214179351, 1.999799611788005, -4.6781337840337711, -3.5469053561759689, -6.2061474631275102], [-4.7759588915931204, 5.9418833427573876, -0.40827034532924572, -0.87136409534706882, 4.6462940231566767], [-2.0965207952833338, -6.725875088455104, 0.6264419085929005, -5.0285856946259333, -2.0974463573628084], [-1.6445925003293791, 6.2373493803010982, -1.0777389203400607, 1.6940997498521086, 3.3622801386877259]]], [[[6.7402942096004788, 2.2150791841252624, -5.492808940537131, -0.041979486547432748, 1.6270118053863871], [3.3084923574582494, -0.58086104623616408, 6.1267501061139908, 0.19167159898442387, 1.3331029604128128], [0.85941767698216298, 7.2347025626890034, -5.3096650985549543, -0.59928778322345255, 4.7272216308604058], [-5.3478681170653459, -2.3557820191930539, 6.9031253851824461, -5.9130827770126722, -0.79118964007050707]], [[0.1923841121815405, -6.8294534380632914, -2.5126161950027162, 5.092437052802512, 3.4049609634830165], [-4.6005841666944374, -6.2749844414410445, -5.5507079811424251, 0.9995711552109936, -0.10364625003488714], [-4.5022527512110795, -1.5593943284888059, 2.2316701594811041, -8.2860223593405173, -2.1249904035300431], [6.9536385219988004, -7.9602771597120157, -3.8848094633600945, -1.1912646254166914, -3.6981367250830361]], [[-4.4810441146132938, 1.7190352660803285, 7.8654345475542087, -5.3295086756836296, 9.4859685985939297], [2.9805519913063163, 0.048228077250481408, -2.6014543055755035, -7.4993965664116828, 0.16214353814220381], [6.3290926558012952, 9.1567906553845528, -10.116167338942546, 2.7756828395298703, -4.53146391080538], [-3.830650885239907, -2.1849703724782663, 5.4998923919267275, -0.2725967318571455, -0.29147533965669536]], [[-4.9557942184146366, -18.167534123774132, -19.228785553269667, 15.251632246977941, -17.802509604359152], [-3.6403917051034655, 11.634420330768497, -16.777739662406795, 19.074145042850379, -6.9607114235775507], [-16.258144874484721, -6.047551533679731, -15.00072058570194, -1.0107875732063663, -10.810973971780806], [-15.807479835978775, -17.631606176162695, -11.951553657517552, 4.7917211169459737, -6.321211733760622]], [[-6.0827652233934311, 5.3627716437460897, 5.9274617369175031, -7.8078097461605909, 8.7526648646797085], [-1.6883918780210974, -7.2292159865675787, -3.146464852459725, -4.6712830810775401, -0.38178978039439787], [8.4299842581386155, 5.9929690140389233, -4.0592039814201257, 5.6094806795538341, -1.7835956405832776], [-8.9891748044785569, 0.046434637383873288, -6.4336691327327564, -4.3963152742341496, -9.1288350016192616]]], [[[2.3054115112475753, -3.9773209289127465, 3.5969273291661521, 4.2875063486685869, -0.058268092163337393], [-2.3334041636052256, -1.5068904090588764, -2.2197312881599318, 1.4909160654504463, -2.0536532751875169], [1.9316821292610611, 0.66878173647535144, -2.9821040431945058, 3.5924734041657764, 2.2520157678490094], [-2.1610301481408003, -2.517601368341535, -3.1447279909540167, 4.3964887387419926, -2.0168150546942365]], [[9.6542077445225658, 3.484120359539487, 0.19800901069690466, -12.251420208668851, -3.9708200474902222], [4.8143531394091728, -4.262067706630229, 11.649356116123961, 11.691200314056742, -3.8236407573819551], [4.7568168663576857, -1.5464920820531307, 8.3335176484455946, 3.31595144563823, 8.3879252329815959], [4.3224173943316417, -8.1641645852648601, -9.6248616611381195, -10.168850090278722, 0.83085480811305146]], [[-3.0758748467204597, 4.6660546164797303, -20.305064288987658, -8.7582207842752009, 1.6418795001429209], [-4.3000234622324243, 1.3588280907546326, 15.9073894830344, -15.430577563364094, -3.5669232021730042], [11.566718256677436, -19.876244326238304, -5.0038133348888643, -3.3542612417336293, 20.24908078451751], [3.6208593802669613, -0.39610059805743592, -18.362394606627717, 18.700767708415249, 14.578215245720369]], [[-14.150475239990884, 10.820983428826173, -10.652794910079296, 11.231896156927801, 13.988385106971462], [-11.634194437962426, -4.0483873237639454, 8.751340509696119, 11.865741840556142, -8.332426812666526], [-9.1580921065273309, 11.96956398316776, -4.660159751728477, -5.9665505825164882, -13.152128870752872], [1.7386277649599202, -9.3050244231105061, 10.68401362473978, -3.4358046240172531, 8.6612148378254954]], [[-5.9686755378243017, 12.977988107324473, 14.2395654617719, -6.2888076889063713, -13.255356817230936], [0.11321115871648629, 2.6379050249244442, -15.74146163745969, 8.1166582468407231, 16.583427514175582], [17.647173734400713, -20.399834484310571, 17.640405638616393, 7.633303220934641, -21.066866022325691], [-16.174649575261473, 9.4604125679151387, -13.594028547976515, -13.066643628376069, 3.0683877633822116]]]])+(1.-msk_ref)*numpy.array([[[[0.5150789983213474, 0.92891337200599522, 0.97945030870169003, -0.23133098930874027, -0.21751559543471005], [-0.63867616173925523, -1.2314117608978903, -0.29502098940198396, 0.45868212081517368, -0.048542871943426075], [0.3743937277175749, 0.020854900192375361, -1.1006496112926254, 0.61584806827355965, -0.50824590163598715], [0.15877181815485303, -0.27098069783493484, 0.44935545133313792, 0.061210581165898156, 1.1632079537268905]], [[-11.860660207655217, -11.04084319043594, 8.2794043148530907, -7.2929829632273018, -3.1230267159499574], [7.7886410025914374, 8.303230492402589, -2.9665060077343361, 3.246327037922152, -10.420178196174923], [4.8067221457042386, -4.9735782375139745, 10.073545960817908, -10.857257828897229, 2.8764897129654123], [1.0404804612936844, -6.2640969368060793, 9.9456381521949346, -12.054511624439943, 2.5946839027214601]], [[2.2055646685139778, -6.5286726763424383, -6.5261856706348427, -0.33119236381665434, 4.8945939079105294], [-5.7933492377868836, -0.62687294188963572, -7.733970871192378, -6.1882787000510024, -5.4583089790495354], [-5.5585048468427019, 3.5993897301309503, -1.1392522565679539, -3.6495155235383048, 0.40261645509903926], [2.5937050727966446, -4.3138772500854801, 2.431517286579882, 2.6013573966726868, -2.7127580242651854]], [[-13.418369203947622, -12.431452727792635, -20.296588540712118, -20.484276574758784, -10.02755575085995], [17.385817904943192, 1.3282480996759001, -11.034175322109466, 7.5881977137947842, 20.761916574540837], [0.68739753446245033, -17.531852187625478, 8.9687929900991286, -17.451309347459915, 4.1557941158462537], [17.028709206291424, 3.4462799372441788, -16.153781741939895, 13.127482040478228, -4.9413818278598347]], [[24.039731314483696, 1.125610104208288, -19.911254208010714, -1.3582367406102407, 13.881398673530317], [-12.48066200854851, -2.3708012271355492, -7.538117150201443, 20.908407332185284, 21.946735247387469], [8.1109799294845342, -15.101499009596363, 10.806022117682444, 24.695974713855975, 1.4893862180908481], [14.031572529199899, 12.332364634196963, -24.156478380873605, 23.959974476151825, 10.595906063611819]]], [[[-1.5424417544304416, -0.65898009448677763, 1.566073659565473, 0.62610001950870853, 0.2823138527379821], [-0.78058212294002849, 1.980693188249814, 1.647630116615115, 1.1479056731713582, 0.091814681848614341], [-0.13346132530006727, 0.95026503330398804, -0.22816211893939062, 1.7542153107696448, 0.45514272129887057], [1.2197203422621907, 1.6627310048495847, 1.8946291406308204, 0.74498647503648252, 1.1008676931434112]], [[-3.3602456879434279, 0.20705211337466356, 1.2629101161828453, 1.748964236463505, -0.072863853457079741], [2.9295957920453151, -2.1876957808008721, 2.8720513627539468, -1.4931992175978355, -0.050038957770195118], [2.5165891421475859, -3.0809982105535738, 1.4632556940806278, 2.0774396376463038, -1.0353764381009529], [-0.10396768651427926, -1.1118109237805176, -0.24111014366650899, -2.4004794479348162, 1.8111919966733225]], [[0.094177590869118047, 0.9071351056869682, -0.91690118235626106, 0.0048094956665373128, 1.7303376249258389], [0.014164752178645221, -0.66702820009529651, -0.54967665823717726, -1.8969982265193082, -0.69417772758970531], [-0.44560929511516378, 0.10053595916077016, 2.2418997345748988, 0.92929821374679222, -1.635118513343949], [1.6933729970809088, -1.7307620955131362, 0.35867017974092802, 1.3983463406953267, 1.1020757603723961]], [[3.484127648367338, 13.92358884107178, 6.3653571787446648, 12.04419607217252, -2.6924291448473823], [1.4334895518013651, 9.5013522646119704, 14.582278575355296, 15.004364169288877, -2.8325302028665638], [-20.237491270815557, -14.830782509923322, -13.399754641105863, 6.3385714676788556, -16.922002709009945], [-7.5072645537237541, 0.84683075684982922, 20.562310617378625, -19.369274636149544, 9.8332936402565796]], [[-7.4139455618021772, -18.132341939975916, -1.7144175693102179, -12.964687980032499, -18.767611383956563], [8.2550085586442439, -18.622161975096329, 8.6786976677251424, 18.356883740384713, 1.3396883959380215], [8.3430696610134643, 11.375078629955876, 14.272209873254713, -11.897807293690489, -4.9287872192193865], [-1.0952980255201781, -2.8388388992772033, 6.9607245793615702, -14.184305449363507, -2.666287592910757]]], [[[0.20536222618374789, -1.661067152378634, -2.6767428326014846, 2.0798785001808784, 1.3746944039451339], [4.9168693070706313, -3.5516067806782714, -1.2096548585710309, -0.97400864382086483, -1.8428509673611149], [5.1272112582420801, -0.28581583818290335, 2.8280355469449114, 4.743596444744111, -0.10956011125613736], [0.84755979309203211, 3.377316699116832, -4.1511916293509081, -0.64784613616332709, -0.049297245508997965]], [[-11.274497126676433, 4.1874775679955993, 17.164198525180648, -13.992150162687613, 14.088316824303012], [15.237401171569859, 8.5343169616958807, -9.8443839004584763, -7.5566306561391574, 11.512315812765008], [-10.26740972794337, -1.3465374551844753, -6.9330091333755979, -16.185476809015327, -7.439628354760365], [-8.8254524185022181, -15.728561108532832, -14.371363293576159, 8.8942007084308443, 9.1005770236324341]], [[-15.71519712858095, 22.400336439521944, 8.9653938865688421, 12.522010579566231, -8.5432299694540568], [22.274925452679483, 17.924783864640663, 17.112734601390528, 4.8761247652925395, -8.2756024593231476], [2.3039624555154155, 13.636494700299352, 4.829173785744997, -13.134314268524752, -12.549914148357541], [19.292093780820021, -16.733261662020816, -16.359823253378366, 1.6350248973671442, -4.4588842678902987]], [[2.2224901474541401, -9.4883822532928619, -9.6463100075557087, -0.14998558361240055, -6.5727123316918483], [-7.204786428506873, -2.7999443427166448, 5.979530214602339, -9.4965772522001988, 4.8634478883372969], [-0.95881237077076598, 10.058343943421669, 7.8252896916694636, -8.4294168350269167, -7.315859129494533], [-6.3284045916709593, 4.4065807657773393, 0.23440241766030645, -3.180008378903004, -10.004468554822992]], [[3.6099258556061415, -0.22431316257280254, -1.4122347853347823, -1.9842148437409748, 2.3617162440988326], [-0.58161959882541869, 1.8696299149037388, -1.7082189324988004, 2.9672407201488498, -0.7457935707062997], [-1.6901614434631476, -2.3241515869889771, -3.8362568343461589, -0.068778891377411597, -0.72940930645813762], [2.1554261246352042, -3.7061410129198444, 0.12898508326303421, 2.2653128043866935, -3.0589676021956893]]], [[[-0.84583099984806287, 1.8437550244115679, 0.37229911855807651, 1.7174562838984588, -4.3812254626456966], [-1.2772479913043913, 5.4582515668487384, -0.95290358244000428, -5.4361398460734165, -3.4177989826806838], [5.2145964039453476, -4.1577616714342964, 1.1629706742401109, -5.6567696419583333, 2.1131925333809818], [3.0538599599022413, -4.4799374935624492, -3.7257137756205396, -4.6271508162814516, -3.6734634260634222]], [[-9.1279681938874297, -4.4316537148730797, -5.4309933134733113, 8.5611210677073775, -0.77028662748885068], [6.9941557868199666, 10.116022670471994, -2.3101257584523327, -4.048398819361636, -9.2551346219618367], [3.1742798975103406, 3.8738874547127224, -9.4939212239824915, 10.858158055874439, -1.3498319505584937], [-2.826151699210516, 9.4118942878707585, 9.7552658048690066, -0.18473172833384863, -4.6038462536741926]], [[7.9231531693873016, 5.5014262312690931, 2.8830519659523848, 2.4072640137017283, 6.3337534607458936], [6.5525352253226181, -4.8019570047323885, -5.5108334953503935, -0.79513686841405518, -5.4209735365076996], [-5.9428776493529227, -6.1882378696653175, -8.2530380175838562, -2.6226320504029976, 7.4587676488367709], [-7.1582134211362369, -4.9118844904867389, 3.5126648433619287, -5.5437784131250192, -8.6714061474129522]], [[7.0757266188105978, 6.5704075802454422, 0.35887618413483702, -10.186505878125168, 8.3638458083488061], [-9.6205532442456203, -11.121251960731492, -3.2748851873881835, -5.9122647475060104, -2.9430840773025193], [17.684827116328229, 13.125383002186597, -8.5169552762770575, -8.5583362322463543, 3.4259844393589454], [2.1314736858389973, 21.904036841375664, 4.0691549318082441, -17.824858231434629, 3.1327049427637221]], [[0.15822813131483707, 1.2200214109963823, 1.8798926568395076, 0.50271303284583357, 0.63326490313988903], [-0.53639011739476672, 2.0060055467113904, -0.78899291695428964, -1.1276386450594762, 0.80420004274291534], [-0.9131235049981552, 2.0144336561354228, -0.6104182194806288, -0.81682883771319215, -1.8357239294702499], [2.1339104990773841, -0.81884225648736619, 1.9835945507452053, -0.60422389150937961, 0.30767781046060338]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.6265594807434791, -4.9213948032726638], [-3.4553463124298944, 2.267802160103213]], [[-4.7140975024102971, 4.7288535939101788], [0.71109054619642986, -4.9564006033004677]], [[-2.9094205288126931, -1.2648016461448131], [2.296416963542292, -1.5670803683738734]], [[-0.38519558402415388, 2.4199706132375729], [2.6619047126730457, 2.0514422030148074]], [[-4.5274560266988848, -1.2114171435125485], [-0.5839045279434778, -4.5127055038147565]], [[-3.1833913324469387, -3.5336695794595432], [2.7962541421816223, 3.6990384096821582]]])+(1.-msk_arg0)*numpy.array([[[-4.621529362359567, 4.2314618497522734], [-1.8635304969571664, -3.1316286279995911]], [[4.6250995199762261, 3.0818703243885626], [-0.025406338401640305, -4.1536580227056366]], [[0.15657288095353916, -1.3301646688075275], [-2.4128137335999575, -1.8706977733795638]], [[1.1401110688137353, 0.49588235993367835], [0.013030734704713609, -3.2256063935770452]], [[-0.11590479193061132, 3.3665664254722891], [1.6010582227129291, -4.946493841051014]], [[2.5883396470113418, -1.4807167209038772], [1.8361656673931401, -1.4657034147288028]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.6268996783643974, -3.3744720776147874], [4.7752910356410663, 0.88420629466442158]], [[-4.7790809793287616, -3.8793066956871627], [1.0256322752496541, 3.5691920188237027]], [[-1.3226304479424735, -3.6735629625797395], [4.9705835793888937, -2.6505442463002185]], [[-4.4906104652895671, 0.40297211323772686], [-0.061010802681891185, 4.6920040002246299]], [[-4.7306165800967364, 2.7543200306494997], [-0.51796043246756085, -0.82960003051840481]], [[1.5749638371729544, 2.2800880225849838], [0.13953913812932317, -1.0666454638120859]]])+(1.-msk_arg1)*numpy.array([[[-4.7038415356377916, -1.0257964004058784], [2.1404875582647431, -1.1891750922453714]], [[-4.1329754049929557, -3.8698440192638106], [-0.72468514131353245, 2.7270756044230264]], [[1.0151883641819071, -0.81350308085866541], [1.1471345034586102, 4.7695630045891022]], [[-4.6347213572569776, 2.1032142097313589], [0.30777741254250834, 1.0019676918356577]], [[4.9993143669076687, 1.4373088235616107], [-0.89463237191371903, -4.5479964854797359]], [[0.90395112713111381, -3.4757728116106645], [4.353799168209262, 2.1862140849838507]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[16.780067092642277, 16.607109346562122], [-16.50028427078189, 2.0052049450168332]], [[22.529053708470272, -18.344673409780061], [0.72931741480396362, -17.690345475393016]], [[3.8480881772765603, 4.6463284822874709], [11.41453245041342, 4.1536158538833972]], [[1.7297633208021921, 0.9751806719895425], [-0.16240494318289145, 9.6253750227751045]], [[21.417658545560638, -3.336630503848812], [0.30243944181337074, 3.7437406236852953]], [[-5.0137262281737547, -8.0570776838986209], [0.39018689299057346, -3.9455625401541465]]])+(1.-msk_ref)*numpy.array([[[21.738941772836569, -4.3406183339306814], [-3.9888638431837284, 3.7240547625796596]], [[-19.115422561706467, -11.926357442981699], [0.018411595934852129, -11.327339462836527]], [[0.15895096689047189, 1.0820930561242699], [-2.7678218842313029, -8.9224108926783749]], [[-5.2840971202760985, 1.0429468257676326], [0.0040105658109446211, -3.231953392942732]], [[-0.57944449149214916, 4.8387956284375928], [-1.4323585153576313, 22.49663660454717]], [[2.3397325413140515, 5.1466349202149928], [7.9942965553906582, -3.2043414496890352]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-1.3485306430893065, 2.7184849016718129], [-0.19490567771878187, 0.089835290196486284]], [[4.5930484084101586, 4.462039911019966], [-1.4888762830367943, -2.0850900050874008]]], [[[4.3494277450576373, 1.0844382205438485], [2.8624773369765064, 1.6420509879281342]], [[-0.5119153306877724, 3.0944522589859638], [-4.7773398062706285, -4.8294441189042665]]], [[[1.6939016556964459, -4.2232191347851602], [2.4254255873633079, 4.3034817420809652]], [[-0.44858025387597422, 3.0320739339098424], [1.3746558457151856, 2.3631450046734752]]], [[[0.87158267784406007, 4.8726277979612593], [1.3584285632564272, -2.9547682799961663]], [[-1.6800973515391684, 0.67418772561256191], [2.6538029337652391, -1.2507886986790773]]], [[[-1.520904677763304, 1.3882659017024501], [-2.2528749413607265, -2.9946350314848234]], [[-2.0685877179517966, -0.97395061476521416], [2.1363430200031752, -0.3014433092325941]]], [[[-4.230677672084231, -1.692168673127894], [-3.1218219453352249, -3.144977938771385]], [[-4.5382272596778117, -3.1624659105500621], [-0.045116183336540772, 2.1332464491389738]]]])+(1.-msk_arg0)*numpy.array([[[[-1.6405419951934253, -3.3931621812791013], [-4.2123946459253041, -1.0816496943318601]], [[0.35181070431392936, 3.6420760677282882], [2.7367446756647631, -3.4750983083896592]]], [[[-3.8633518046824067, -2.1683428702782348], [-2.0432837687879566, 4.9152890858342317]], [[3.7371644359427485, 0.32455496247276194], [-3.9536645213065178, -1.7008562789940749]]], [[[-3.4234847064465921, 1.7935093078651674], [3.1538358535816311, 0.86867430929066813]], [[4.646641182248306, 2.1073091592607343], [4.1629947533390954, 0.18557661698695238]]], [[[-2.9932227233181274, 1.3798535070655458], [0.89408775417040509, 2.3650798763731595]], [[-1.2686253531730554, 3.5865645034030251], [-2.2565437017491572, -0.91912996186515361]]], [[[4.0803851628566186, -1.8441399097414743], [1.6622369049899666, -3.2635299885279592]], [[-2.3243560782534542, 4.0452372029668542], [-0.96887801084070091, 3.5783078103754313]]], [[[0.72423079245923194, 0.79153351910701364], [-4.1544086868835137, 1.0722864380490851]], [[4.3356277047936693, 0.67571892870462058], [-2.1318970294645556, 1.0493841016264387]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.736459363390269, 0.4346184280223131], [2.1736323191416567, 2.4154993408321168]], [[4.0366145286295296, 2.1158389528099519], [1.2534995340223709, -0.67674017499916861]], [[-0.87565506964954931, -4.5981144921936936], [3.8674857515299141, -2.9999341522712553]], [[3.8594735420905693, 1.7590724092284651], [-4.8522624657724167, -4.8428424140946262]], [[3.1473147343946479, 0.89781114195358569], [-3.6096352233541773, -1.2364671329783672]], [[-3.0455274348917891, -3.140069554066943], [-3.9612781429593791, 0.85614045681693529]]])+(1.-msk_arg1)*numpy.array([[[1.5867010393394034, -0.82963676191304003], [2.5431992702939912, 0.17662146160559278]], [[2.1802463666055507, 1.659902436113871], [2.2765758243472991, 3.9344963263269648]], [[0.65652495480921402, -0.70998771265654437], [4.4832762101514341, 4.3618546606539219]], [[2.6048625187331247, 0.85789182477470671], [0.19319301869141015, -3.0826302182643959]], [[-1.2114354920940817, -4.9709270506201726], [-2.770869990764171, -1.8527400220631431]], [[2.8353952306854646, -2.9911512674297933], [-4.2148461830377588, -0.024870281582737519]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.3872605912790466, -12.875993266758533], [-0.084709599262760546, 0.039044072606125184]], [[9.9835984639024673, 9.6988341598929608], [-3.5963796802559487, -5.0365335328642518]]], [[[17.556963226924033, 4.3774590764484529], [6.0565410511105915, 3.4743154427584106]], [[-0.64168562847603061, 3.8788944646933787], [3.2330177765260792, 3.2682788581759792]]], [[[-1.4832735722983583, 3.6980832456156087], [-11.152384542992628, -19.78790176515345]], [[-1.7348777402830018, 11.726502736981569], [-4.1238770191803109, -7.0892794062891733]]], [[[3.3638502848835978, 18.805778066686511], [2.389574205532246, -5.1976513570047045]], [[8.1522733177171514, -3.271335795874307], [-12.851949406287051, 6.0573725610332589]]], [[[-4.7867657019341907, 4.3693097276857928], [-2.0226562237816914, -2.6886166973516015]], [[7.4668470893166417, 3.5156064448639723], [-2.6415179290016728, 0.37272474432233699]]], [[[12.884644918516654, 5.153546118475437], [9.8027380437651761, 9.8754494737482368]], [[17.977180451544154, 12.527407089316092], [-0.038625789811582621, 1.826358589468946]]]])+(1.-msk_ref)*numpy.array([[[[-2.6030496888533463, -5.3839339596827074], [3.4947574539452959, 0.89737634992971393]], [[0.89472472649280022, 9.2625251978017911], [0.48336784465723442, -0.61377694245090464]]], [[[-8.4230587350778148, -4.7275216644791724], [-3.3916517054830608, 8.1589003277801631]], [[8.5079382064777711, 0.73887398123743475], [-15.555678534609752, -6.6920127813123385]]], [[[-2.2476031421898841, 1.1774836172960836], [-2.2391847037786223, -0.61674808589678498]], [[20.832175869483766, 9.4476490211478694], [18.158378067129956, 0.80945823171292597]]], [[[-7.7969336821916801, 3.5943286818974931], [0.7670305749339682, 2.0289826908797077]], [[-0.24508956156795889, 0.69289922314388874], [6.9560898038461518, 2.8333377949577243]]], [[[-4.9431234076985975, 2.2340565390479985], [-8.2628583955537778, 16.222769500483775]], [[6.4404885050827936, -11.208826371223648], [1.7950790671814945, -6.629674091543694]]], [[[2.0534805348544607, 2.2443103650037086], [12.426464809192966, -3.2073709382182995]], [[-18.274003882622356, -2.8480513474570337], [0.053020879428185161, -0.026098478095897576]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.7807045854345112, -4.4723900930760401], [1.9299155560525225, 0.73650428425596903]], [[3.1245668373680502, 4.6877982550291968], [-2.0707111468913517, -3.1409219887253315]], [[2.2330311885019469, 2.8461352418518686], [3.5829968654289051, -1.2556610306028446]], [[3.2366428995612573, -3.1954175388209127], [0.40628017675704431, -1.435888067850505]], [[-4.3144830950158051, -3.4523521495122909], [-2.0534609629308607, -4.6059212227491928]], [[3.1186273386903398, -4.9006796182117025], [-1.4560869442731859, -2.3121985338643847]]])+(1.-msk_arg0)*numpy.array([[[0.84710448735210253, -4.5638396567118322], [2.0072004762942708, -4.4806937180659645]], [[4.3180426631161204, -1.5464234445308565], [0.74468908079970308, -2.2842403682787173]], [[1.9713894322122103, -0.95694558323992229], [-0.057399469455651442, -0.38090132564066881]], [[-3.398561189035564, -1.1571245744113257], [-2.7479300167954723, 0.79987198292553252]], [[1.6329993570279981, 0.5462911789863476], [1.8058126133469079, 3.6276042006664415]], [[4.6740431434422689, -3.3843266299575103], [0.96255958215783721, 4.8535917811942575]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-2.9006544470096642, 1.872663579246538], [0.76843606825922972, -0.5538177880230073]], [[0.5379809325742162, -1.3217821002952492], [-1.0220090048816797, 0.93210591877742299]]], [[[-3.1651916754819465, 0.20882856648388071], [-3.7712010815505339, 1.6510680537981637]], [[-3.3657350686243879, 2.4452061485884746], [-2.7921177539780659, -1.4532229316690115]]], [[[2.7400364593907671, -1.6558644183827353], [-0.28094483198175979, 4.0615363971920182]], [[-0.84795850642626647, 0.59435606781676498], [4.9858614001112826, -4.7209817672819465]]], [[[-4.9826563167335998, -2.0760498493135149], [-2.7829083651725375, -1.6102395479234968]], [[-0.76176862849308513, -4.9870615625477814], [-0.081174176491376926, -3.8241306448073686]]], [[[-3.4723199403569871, -2.9394494099779411], [-3.9673518656296469, -4.4448814198143607]], [[-2.9335954108160509, -3.3058698921336616], [4.7309140164651122, 3.6111109480995811]]], [[[-3.2134538118521636, 2.404438560909286], [1.5448198252757415, 4.7826635074561494]], [[2.6420647060662485, 1.178578782992588], [-1.0750430322070095, -2.729651977976959]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0388263839158114, -2.5310099653111262], [-2.6662857677986072, -4.0894654426565857]], [[-0.33589150636221365, -3.7488336053030857], [1.7121604874518326, -4.2216044370992414]]], [[[-2.2558395630565666, 0.26492532339022912], [4.6806604687592763, -2.6574827629331512]], [[-2.6880541987797715, -4.7125390950238222], [-4.1381977588289365, -4.2107783702453219]]], [[[2.0163697362675492, 2.6730141496239455], [3.7523182246539406, -2.0300632961118845]], [[4.6486883590077053, -2.2040789451637233], [1.4765870357616171, -3.8944638274585763]]], [[[2.3680356062273864, -4.5189614659760036], [-0.19067868256212606, 1.1669304012311414]], [[4.4685971820986694, 3.1135710533656766], [3.8513298453767462, -3.6750805654966845]]], [[[2.967712392336761, -2.1322075629326678], [3.6984480667588819, 2.5531369231800731]], [[0.054939349685835026, 2.4054703138965472], [-1.0902978650872699, -1.759726622418154]]], [[[-1.9499585908398807, -0.31496738594804441], [4.126145035822848, -1.1546777427115673]], [[1.7256293766012707, 1.6610646533197038], [-1.2793678987708135, -2.6988381800592922]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.966517568570444, -7.0799877810335907], [-3.4367458588448825, 2.4768891885233844]], [[1.038257770634623, -2.5509278370715771], [-0.7527140106435366, 0.68650000255991828]]], [[[-9.8898529431243052, 0.65249881353064276], [-17.678629849456811, 7.7398739415294839]], [[6.9694651240836487, -5.0633156283294252], [8.7698240485800927, 4.5644598605990883]]], [[[6.1185868714520311, -3.6975968901792844], [-0.79960698731943847, 11.559681876112272]], [[-3.038232670539089, 2.1295759279361186], [-6.2605518641066746, 5.9279528313624876]]], [[[-16.127079188509853, -6.7194320039158058], [8.8925541990037598, 5.1453876931377991]], [[-0.30949149303214185, -2.0261442531301741], [0.1165570314415591, 5.4910235627803585]]], [[[14.98126568315651, 12.682204788004009], [13.696695741178109, 15.345295924023352]], [[6.0240236571438821, 6.7884747720249292], [-21.790217271438284, -16.632492553553821]]], [[[-10.021564909260841, 7.4985478302529573], [-7.5706670315381901, -23.438301571755243]], [[-3.847075924428037, -1.7161131787128876], [2.48571292291017, 6.311497301438342]]]])+(1.-msk_ref)*numpy.array([[[[-3.4213079534511492, -2.1440298991479443], [12.168500723205639, 18.663664561948732]], [[-0.67420159155343529, -7.5246605981123222], [-7.6716667404461862, 18.915716481469975]]], [[[-9.7408114744234826, 1.1439588489388444], [-7.238283084778133, 4.1095936480364612]], [[-2.0017646104290905, -3.5093764069059548], [9.4526383726375727, 9.6184299351892317]]], [[[3.9750499895103681, 5.2695518467223543], [-3.5907643519932551, 1.9426601049117465]], [[-0.2668322454717052, 0.12651296209076959], [-0.56243395934542562, 1.4834064345386047]]], [[[-8.0479139055786995, 15.357967053013303], [0.22063898940901239, -1.350283843892222]], [[-12.279392329656597, -8.5558753569690396], [3.0805708403217826, -2.9395939793349206]]], [[[4.8462724285299528, -3.4818935793192813], [2.0204295548094877, 1.3947561798776182]], [[0.099210170631757369, 4.3438286338659307], [-3.9551691153682333, -6.3835916875086642]]], [[[-9.1141905815114921, -1.4721711506983919], [-13.964222523802249, 3.9078066336779838]], [[1.6610210917006083, 1.5988736986365668], [-6.2095295185977868, -13.099058809509048]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.0506915610245278, -2.2647605172017382, -3.7778556941616803, -2.772215869822646], [-1.1202842123861725, 3.9568123162155473, -3.4267603001676274, -4.2112861754117557], [2.0706268565008488, 4.8719247832408445, 3.5746962873383232, -3.5779291824090063]], [[-3.5930253723482473, 4.6794664631533518, 4.1570894152816464, 0.87274687101384352], [1.3995493260483718, 2.3899484731821277, -3.1963951812823166, 1.1657698961997784], [-3.3948107101975444, 1.6636747512976324, 1.8311382787126593, -3.2699339850274733]]], [[[3.9237318467564108, -2.5583785833088237, 0.98791077175314079, 0.2908698896519688], [-2.387409639410957, 1.9173781868696036, 4.0454047969819253, 0.19712392732234818], [0.3055740969931362, -3.5138592838097074, 1.7060250029707671, -4.2322166324644419]], [[0.990161445924441, 4.603651895867781, 4.3480839014843546, 3.3659773716175323], [-3.2951403757660582, 1.1034824004975272, -0.37666489451278551, 0.35376386258692438], [-4.9565942443422584, 0.73350345455696875, 1.1009961889029558, -2.3759475707038424]]], [[[-2.0522556641205449, -0.25425187508250069, -4.1925584171941956, 1.7761929983242331], [-2.8477627471859357, 0.47061597214384143, 2.9971260291500244, -2.2910991349139742], [4.4426715479898267, 0.74312698513003017, -2.6905536282634146, -2.4128577684225148]], [[-2.470825963813863, -3.937779176316103, -0.22994292213967338, 3.6161179245703572], [-2.8116416937682476, -4.1001732035988923, 0.054957135016486802, -1.4212364589978552], [-4.9253964763160685, -1.4234222800250218, -2.4497077175232551, 3.8249488453033962]]]])+(1.-msk_arg0)*numpy.array([[[[1.2682556634344424, -3.5894816523439377, -4.0661047987575314, 1.6425128986155437], [-3.9973070322147732, 4.7599010108416735, -4.0063482532431678, 2.1238938454936749], [2.2630300312230869, 3.1696116874093914, 2.9767228894580224, 3.3321378350287478]], [[2.794554322344486, 1.3461216826302849, -3.32175091854341, 4.4258196498099611], [4.9628234679447196, -1.8301062902154408, 3.3356374927777583, 3.4206276702208722], [-1.042084941298195, -4.1858032259539932, -1.7351350004367716, -2.7303287037551982]]], [[[-1.2724458825261875, 1.9562825580089829, 1.4440258664259442, -2.7961970618933627], [-2.394927635387579, 3.9804680139992676, -3.4752193298276524, 2.3628274313169246], [-1.8734231730983355, 2.3464592112256533, -0.14743732445938029, -1.9344171651201814]], [[-0.4639755411960147, -3.489276321123004, 2.0653788148927639, 4.7308605417958987], [-4.7707262112317803, 1.3368858521526299, 1.6218546317746316, 3.1377265498983515], [-0.54722619988644183, -1.6183008030463295, 0.75725843258493342, 3.7263016456052824]]], [[[-1.3709142423011667, -0.59580159400144517, 0.057261700306153962, 4.5591055634622428], [-3.0030125771370164, -4.6416082963837706, -0.52615937863302964, 1.5291313030532372], [3.8621435305731282, -0.6440476762314562, 3.9699672516913829, 1.2832080900936385]], [[-3.9726687426593044, -1.0698207920314395, 2.7908783211106467, -0.98186951324375116], [-1.5165462676029549, 4.7570890613701682, -2.008375384247183, -2.8934035556709556], [-1.4836637272484996, 4.5408339570799079, 2.3444603914274165, 0.77811084025784893]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-2.5606616293264892, -1.3127833549481482, -3.8216012905418428, -4.3777881736651691], [-2.6784485843741832, -3.3194352744179598, 3.8606628792499027, 0.66135104535068212], [-2.6268914965438639, -4.9338702068591527, -1.5477231360748167, 1.9771412190223305]], [[0.79645742409913467, 3.1751236713591204, 4.2749962111722475, 4.2588352120298527], [-1.3544970911452134, -4.0345114655878618, 1.3038132183298821, -2.3748006125650645], [-0.81858960408321657, -2.0113824055138618, 0.3548757579494124, -1.1046584046014942]]], [[[-4.9867981069002383, -1.7666573469067615, -4.0207988472922915, 4.3620986566460491], [3.9922134658526751, -1.9678920590567825, 4.5314995435611731, 4.0757964434769764], [4.6663796768703651, -4.1223888184718529, -2.3641659493555025, 3.2137270733558694]], [[1.906982519922245, -4.43428482319844, -0.41768338802830396, 0.59287858576126951], [4.8593847648328552, -0.23475398050844376, 3.1199507556683415, -2.64692303342283], [0.45811860944311356, 4.0421841971830137, 1.3590611213880557, 4.9542752179299878]]], [[[1.2289700855823495, 2.4050971837396373, -2.2551131854435158, 4.6890363374299682], [3.9470228194961461, 1.8157715050840046, 2.0709823082330452, -1.4344366744835488], [-3.9911070876013199, 4.6538420070074018, -2.5485411955400981, 2.6217934061642545]], [[-2.1204619997339034, 0.42951735167660399, -1.8422986970828772, 2.5670112560972456], [2.5438479943919443, -0.14989546048147862, 1.5189442627619094, -0.9460353772904897], [4.4089417965803275, 2.1928096255421972, 4.6418957988254004, 1.8068291928989053]]]])+(1.-msk_arg1)*numpy.array([[[[3.8189256557983224, -4.2378069126141318, 4.2290315195995092, -4.9109516642337736], [1.6548804460815916, 1.5149922216818714, -1.7151008177899216, -1.4459619145568556], [0.21656191260379032, 0.97876660031305018, 4.003333167435585, 3.423989040409678]], [[-2.674134239177727, -4.2830947692220986, -2.2259141311716144, -0.77986607290403676], [-3.6658196067186832, 3.7256526590658741, 1.2655886458011247, 4.120338714762152], [-2.7031588958633614, 1.3843656517939742, -4.2408976636702249, -2.8476647714845695]]], [[[4.0485837617263396, -2.6319470744079307, -2.2391120791623829, -3.4015271912713185], [-1.4326983515612444, 0.99722920537915005, 2.9964116021969822, 3.3471815111337069], [-0.45321627047361002, 3.6808376821998259, 1.1534493417132339, 2.7005213534229995]], [[-2.8080290209827941, -4.2940903803230013, 0.3080964033941358, 2.1606143226488337], [2.883873734584963, 1.7936911891281238, -3.9465959630956657, 2.2546954410960494], [-1.2932380643322117, -1.5521512650104965, -3.5007482592080774, 4.0751350366076107]]], [[[2.2310945314775452, -1.2773704209912573, -4.9269840586384541, 2.8698043943533458], [-3.1453856791593848, -4.5668968180794227, -4.913917306031049, -0.47963388391127904], [-3.9609742401169603, -1.1150332973118027, 0.36514682081002903, 2.4654531621261633]], [[2.7105642205046321, 1.3823089977129754, 4.1561634711315154, 3.5980368407536805], [4.5749623234909862, -3.1714029129879329, 3.9247135145952328, -3.7168924523789837], [-0.98934804946362931, 3.5159732502632117, -2.0390636838799958, 4.922153261875895]]]])
      res=mult(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.372450452552128, 2.9731399099262013, 14.437458196289127, 12.13617384975648], [3.0006236627624907, -13.134382376697319, -13.229566286944413, -2.7851385143794407], [-5.4393120818574312, -24.037444538090739, -5.5326401483542735, -7.0740712652837132]], [[-2.8616917327833193, 14.857884736489348, 17.771541499833294, 3.7168851054626328], [-1.8956854910467635, -9.6422745172174977, -4.167502288361824, -2.7684710636051455], [2.7789567551980712, -3.3462861232577077, 0.64982658456833731, 3.612160059052655]]], [[[-19.566858545389046, 4.5197783203714454, -3.9721904922926665, 1.2688031549096377], [-9.5310489109629017, -3.7731933081493847, 18.331749991043775, 0.80343700190464074], [1.4259247559867845, 14.48549422126065, -4.0333262207726079, -13.601189172057984]], [[1.8882205692788443, -20.413903733135225, -1.8161224154033115, 1.9956159037890377], [-16.012354939983194, -0.25904688593780723, -1.1751759222689013, -0.93638571627395906], [-2.2707080627918157, 2.9649560725893278, 1.4963211151344267, -11.771098168639003]]], [[[-2.5221608191710878, -0.61150046872144448, 9.4546937673568276, 8.3286335114310148], [-11.240184547653923, 0.85453107205619494, 6.2069949819144581, 3.2864366239981369], [-17.731177903066925, 3.4583955799388995, 6.8569867604391916, -6.326014587262347]], [[5.2392925642231933, -1.6913444832985716, 0.42362354586134976, 9.2826154157471166], [-7.1523890836411255, 0.61459735040727537, 0.083476824931124266, 1.3445399697070355], [-21.71578638915938, -3.1212940768500888, -11.371287962321359, 6.9110292350391349]]]])+(1.-msk_ref)*numpy.array([[[[4.8433740912013148, 15.211530159004734, -17.195685355940419, -8.0663014529814436], [-6.6150652445966669, 7.2112130074008123, 6.8712911654885813, -3.0710696111455569], [0.49008611184148704, 3.1023100555982004, 11.916813473631992, 11.409203428272864]], [[-7.4730133966235002, -5.7655667376102233, 7.3939323098180667, -3.4515465896788138], [-18.192815573475364, -6.8183403665143389, 4.221544937368062, 14.094144618397722], [2.8169211793154645, -5.7946822111791194, 7.3585299695047395, 7.7750608642568073]]], [[[-5.1516037376710644, -5.1488321552670051, -3.2333357601372574, 9.5113403381832438], [3.4312088753282532, 3.9694389546376132, -10.413187520074798, 7.9088122921035584], [0.84906586353046398, 8.6369354844242654, -0.17006148484163267, -5.2239348608350342]], [[1.3028567847046071, 14.983267884823123, 0.63633578451490314, 10.221565045058441], [-13.758172015467366, 2.3979603738762156, -6.4008049424897688, 7.0746177474618488], [0.70769375149301395, 2.5118476386158628, -2.6509711396423428, 15.185182392974683]]], [[[-3.0586392691228155, 0.76105933295688821, -0.28212748457895326, 13.083741180344731], [9.4456327544622898, 21.197746159526091, 2.5855036763953878, -0.73342318589373923], [-15.297851036234531, 0.71813460405436491, 1.4496209206750368, 3.1636894433872356]], [[-10.768173753769434, -1.4788229067654808, 11.599346530572921, -3.5328026814639006], [-6.9381420361143968, -15.086646106572383, -7.8822980129353128, 10.75446983775989], [1.4678598146132411, 15.965450726979805, -4.7805040424547247, 3.8299808104761648]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-1.95685942046)+(1.-msk_arg0)*(2.36372856686)
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(1.34364689692)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.62932808802)+(1.-msk_ref)*(3.17601655403)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.288384529534099, -4.1050727979220696])+(1.-msk_arg0)*numpy.array([4.0527762889997305, -4.9007734936302345])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(2.0872950744)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.6892386824234507, -8.5684982311362958])+(1.-msk_ref)*numpy.array([8.4593399856546352, -10.22936037398075])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.05424669139)+(1.-msk_arg0)*(-0.446501418892)
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([0.31469541805389767, 0.44966002919307879])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.2758528574407484, 1.8230326856065719])+(1.-msk_ref)*numpy.array([-0.14051195067977265, -0.20077384105357957])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.0717701098589387, -4.6874627714111261, -2.6225133032012469, -1.1642891040984891, -1.7177259513128984], [-0.76790474318670654, -4.7242620420189416, 2.484823877862695, 1.172023458280159, -4.595909169319996], [-2.7715701452034835, 0.36099200061435255, 4.4883547188088802, 4.6371922919063344, 2.2573851667722042], [0.99732310414850645, 1.2304659205170276, -1.2461223961743051, 2.0585168326997705, 4.5199811622623436]])+(1.-msk_arg0)*numpy.array([[-0.37069519564906628, -4.8110378586881382, -4.2480553466816362, -1.0595905472553948, -0.10315957205958703], [4.6185690722932726, 0.94798901157409521, 4.6534876614474321, -2.0787502654333423, -4.5044055302903914], [3.5242480337513911, 3.3756842864526391, 4.7729248116653959, -4.6895939722906421, 2.6296635396220491], [-1.9655178993215703, 1.6010943214215114, -3.6948124384148073, 3.7953303279162895, -3.2144148823675947]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(-3.49463286638)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.7454430511158892, 16.380961460902235, 9.1647211818837473, 4.0687629691497689, 6.0028215648905254], [2.6835451537887409, 16.509561401427089, -8.6835471907427788, -4.09579169747325, 16.061015233999168], [9.6856201209034545, -1.2615345098468953, -15.685151916317668, -16.205284591016149, -7.8887323958790274], [-3.4852780981566926, -4.3000266467983366, 4.3547402812019236, -7.1937605795474227, -15.795674725056823]])+(1.-msk_ref)*numpy.array([[1.2954436141240935, 16.812811022366159, 14.845393832711515, 3.7028799513434216, 0.36050483100104586], [-16.140203275678545, -3.3128735568131611, -16.262230924984262, 7.2644689985778355, 15.741243609653013], [-12.31595300801987, -11.796777253957197, -16.679619915602629, 16.388409225540649, -9.1897086330822635], [6.8687634504257558, -5.5952368378127106, 12.912012982391046, -13.263286102701997, 11.233199894100213]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.37332335458)+(1.-msk_arg0)*(2.15259406926)
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[4.1681363488632819, 1.2264680671357775, -4.024189485619396, 2.6332329902692155, 3.8585571292960825], [-4.9042187066906395, -3.6965677050536074, 2.0496579830655728, 0.42375316634648907, 1.9667176762291083], [1.9329670643234751, 3.679012280515245, -2.0364674706582142, 0.92645906133675204, -4.5552398064249626], [-0.04545878085149635, -3.5061648492009412, 1.6432859206510866, 4.9101852505150863, -3.1950246024331461]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.5560626441046443, 0.45786917310842734, -1.5023239182371624, 0.98304737331813585, 1.4404894913475463], [-1.8308593791759395, -1.3800150560828561, 0.76518519397980045, 0.15819695357438543, 0.73422164040171356], [0.72162174874597351, 1.3734612061031179, -0.7602608676392556, 0.34586880465931219, -1.7005774054511034], [-0.016970824562599549, -1.3089332232143187, 0.61347701243161468, 1.8330868293317308, -1.1927773025461037]])+(1.-msk_ref)*numpy.array([[8.9723055844216351, 2.6400878874507501, -8.6624464203145592, 5.668281717827921, 8.3059071924157735], [-10.556792102366222, -7.9572097185089099, 4.412081618354188, 0.91216855270673491, 4.2335448057555789], [4.1608934387336856, 7.9194200157643246, -4.3836877995756343, 1.9942902808437906, -9.8055821913581607], [-0.097854302056628442, -7.5473496602306813, 3.5373275268886384, 10.569635649216693, -6.8775910103308675]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-1.3852424730613642, 3.6886682592985505], [2.0767576272063248, -2.2457248046015152]], [[-4.4080937004247387, 1.3830657936945245], [-1.3134408914908988, -1.4982539551368701]], [[1.0149852907563863, 3.8969279164076109], [2.5299506030257728, 4.0049492769110628]], [[2.2625829450784014, 3.7823202251578945], [3.9560040068495663, 3.5275468902287361]], [[2.8582818077737429, -3.7444341487838759], [1.1162133992860293, -2.9763606596026992]], [[4.7540408730135688, -0.27131676753306166], [-1.2293922826075585, 2.7201785655792863]]])+(1.-msk_arg0)*numpy.array([[[-0.67596104425453074, 2.9052294867376993], [-3.9566579766516474, -1.8586418617485001]], [[-1.842573951598343, 4.9857957950548588], [-0.3650601119126442, 3.0550113597875033]], [[3.7790341540079559, -1.8808312759363877], [-0.38015686937497595, 4.685396620618933]], [[-0.40361790627538063, -1.685354432748285], [-0.6238570193086046, 3.7513756276927275]], [[-2.8158268616938411, 4.3064103673629504], [4.2321248119886441, -2.2053892746065626]], [[-0.8905003868952317, 2.553369287807814], [-0.80651025103042429, 3.0466979861342693]]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(1.7007856616)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.3560005360188545, 6.2736340858053836], [3.5321195949661011, -3.8194965475604699]], [[-7.4972225606613785, 2.3522984709618382], [-2.2338814356037724, -2.5482088443287765]], [[1.7262724292509966, 6.6278391245057193], [4.3029037101765946, 6.811560305596263]], [[3.8481686311647572, 6.43291600651939], [6.7283148920726443, 5.9996011715144588]], [[4.8613247154670285, -6.3684799110482651], [1.8984397447888652, -5.0621515335956326]], [[8.0856045514707127, -0.46145166797125753], [-2.0909327667377569, 4.6264407013225473]]])+(1.-msk_ref)*numpy.array([[[-1.149664851866689, 4.9411726546942161], [-6.7294271545351414, -3.1611514285070341]], [[-3.1338233573118073, 8.4797699998832226], [-0.62088900396226332, 5.1959195167445635]], [[6.4273271038245845, -3.1988908659970452], [-0.64656535259081471, 7.9688553912468247]], [[-0.68646754775723662, -2.8664266539283454], [-1.0610470733271309, 6.3802858788467223]], [[-4.7891179519104297, 7.3242810057664292], [7.1979371983219895, -3.7508944564921118]], [[-1.5145502896785812, 4.3427338734673659], [-1.3717010708840771, 5.18178025003564]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.75640901135)+(1.-msk_arg0)*(3.40942738263)
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[4.5552143437382995, 3.6781882063967934], [-2.6511142147904465, 4.4802120516345898]], [[3.7121328695653979, -1.987480404902926], [-0.17363700248082292, 4.9272608289045436]], [[3.5651644606549944, -0.031012208134701069], [2.4641960610762439, -0.73353041443998812]], [[3.9793732896893204, 4.3524518204411464], [-3.5829543190559452, 2.5729379317404222]], [[-3.2313110679050974, 2.4554029195365592], [0.90552279728024576, -0.90845616012246388]], [[3.2965382994176036, -4.9635351327826749], [2.5558942411780272, 2.0144172100453019]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-17.111248209442515, -13.81677932394471], [9.9586693265529487, -16.829508923512346]], [[-13.944289362558445, 7.4657893028559261], [0.65225160082250588, -18.50880697896153]], [[-13.392215906943854, 0.11649453809900651], [-9.256528289556293, 2.7554402589005749]], [[-14.948153684908508, -16.349589239765326], [13.459041891351799, -9.6650072324301064]], [[12.138126013948868, -9.2234976534385584], [-3.4015139956850207, 3.4125329062990835]], [[-12.383146174187761, 18.645068100929738], [-9.6009841596148906, -7.5669749604296861]]])+(1.-msk_ref)*numpy.array([[[15.530672517299227, 12.54051558936316], [-9.0387813983913734, 15.274957648840678]], [[12.656247453464418, -6.7761701149204887], [-0.5920027508962511, 16.799137991436996]], [[12.155169335743437, -0.10573387161033142], [8.4014975268071606, -2.5009186809850648]], [[13.567384259580988, 14.839368418198363], [-12.215822566108784, 8.7722450382882329]], [[-11.016920436717356, 8.371517949262401], [3.0873142206447581, -3.0973153082422118]], [[11.239307945929369, -16.922812596365009], [8.7141358129836988, 6.868009195973527]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.7872822895296299, 1.6577910296764227, 0.54254853248412793, -0.49668709745737782], [2.7114243104864935, -2.9786893458680397, 2.9064915155128324, -4.7575736645232976], [4.2181063758190156, -3.2969510626675969, 0.014766120684297945, -2.6185421407478238]], [[-3.4541674422589921, -3.4815530495038427, -0.88557998100483459, -4.7710228818071156], [-0.044229028195980291, -1.1841738785112588, 4.6736716290057565, 2.0430492748764992], [-2.3305214396353757, 0.31968118920904676, -4.2554311773188491, 0.22218050589038274]]], [[[0.096644472440873486, 4.6276177769410758, -2.0228354862435061, -3.7125870619625898], [4.1240349239484626, 0.81144130777042633, -4.1435985363129895, -1.7561293196318939], [1.0594985682659299, 3.5180556219248054, 3.5316092653391582, 4.1115506994452709]], [[-1.007840133383533, 4.261969718179861, -1.2033732868280236, 0.97821552441224568], [1.4450813301760723, -3.0650860401299918, 2.8636549043686301, -0.24392858655165472], [-0.015934547748988059, -0.33966175231966123, 1.3444960108675161, 2.9474862907484214]]], [[[-2.3641032214682323, -2.2338893071467671, 2.2070918460772146, 0.59054977784255858], [-4.1404183731023769, 4.0906274391366857, -2.1259258402928305, 2.6504152350652221], [4.0236654939218539, 4.0093097792934671, 1.5566582681331385, -0.61272013770228817]], [[0.81892806425918785, 0.69869427477610824, 3.4306011994867518, -0.81570626442060146], [0.075061819601419444, -2.2787505220631332, 4.3945820073276707, 1.0825356020897701], [-2.1437997582441035, 0.9998781235950025, -4.1179383789532427, -4.0275217151816021]]]])+(1.-msk_arg0)*numpy.array([[[[-4.7274807156909269, 3.0618604212137672, -2.8086306006155315, -0.62435680253984938], [0.39480705788055914, 0.13697994758208232, 1.2490254795392204, -4.7123964839763115], [-2.1460470567195458, 1.0361116475306966, -3.6716902894567385, -4.338040429180122]], [[3.5544921239080889, 3.0296757766017333, 0.58385955654329802, -1.8669321092604827], [-1.8144601653000816, -3.9716208437073131, 3.296540624851815, -3.6027526585586314], [-1.3968846594771511, -1.6827368029275389, -2.8979779111395243, 3.9184943667776384]]], [[[0.4965962533216306, 1.3251531703175434, -1.0383932228299928, 4.5243229697998366], [1.6671088251660491, 3.7545009581697624, -3.4038738430608149, -4.372713395401993], [2.71970427882564, 2.8414822137325215, -1.7443227341703227, -0.80289142307707451]], [[2.8701871734935231, -3.9393023613454234, 2.221922812130142, -2.0420959494209736], [1.4781297750854394, 4.8025510077323883, -1.6035951460534879, 4.4780397150093094], [-3.6304645299796947, 0.16306103724254406, 4.8788349333512375, 0.92678303633203107]]], [[[4.9409304828493301, -2.4492205574110404, -0.14334304804134845, 3.1017211991559499], [0.5584226686664433, 0.58683973119633048, -2.1032644207922422, -2.2822875350607874], [-2.3952190393200601, -1.0507395638520634, -3.5245719735933845, 0.81081081790713583]], [[-4.7529034978651188, 2.7616144402710407, -1.5107891652848271, 2.4373704091274373], [-4.4225430130058561, -0.63310213175648666, 1.3626404479183325, 3.2621966593191427], [3.0613336186945119, -0.49246330702891772, -0.1958538425638503, 3.7338987993946144]]]])
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s1=numpy.array(2.08186114352)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.8845958381254473, 3.4512907287647172, 1.129510708254259, -1.0340335686858526], [5.6448089156061076, -6.2012176077893111, 6.0509117501261231, -9.9046077496204958], [8.78151176306522, -6.8637943094653053, 0.030741012893214601, -5.4514411355010424]], [[-7.1910969812620298, -7.2481100128768734, -1.843654551936009, -9.9326071524944783], [-0.092078695217004888, -2.4652855848477926, 9.7299353620140216, 4.2533448996687033], [-4.8518220293247172, 0.66553184612961347, -8.8592168169974421, 0.46254896206152285]]], [[[0.20120037191095655, 9.6340576368909954, -4.2112625985502827, -7.7290907462470013], [8.5856680627010995, 1.6893081288969283, -8.6263967871097602, -3.6560173935435336], [2.2057289008913381, 7.3241033000387183, 7.3523201036162291, 8.559677640800885]], [[-2.0981832125744506, 8.872829151151322, -2.505256087001102, 2.0365088902650581], [3.0084586705244343, -6.3810835285021259, 5.9617318738647942, -0.5078254461364341], [-0.033173515798233855, -0.70712860409531542, 2.7990540026470736, 6.1362571797765639]]], [[[-4.9217346360528156, -4.650647347480894, 4.5948587545351156, 1.229442635806701], [-8.6197761288915231, 8.5161183181686191, -4.425882400917625, 5.517796492084285], [8.3767128463311291, 8.3468262418587837, 3.2407463621705528, -1.2755982465365991]], [[1.7048945163219014, 1.4545844618585255, 7.1420353361356677, -1.698187176425739], [0.15626828559034581, -4.744042167666521, 9.148909523081814, 2.2536888064712213], [-4.4630934161829021, 2.0816074137713576, -8.5729759025658101, -8.3847409635326269]]]])+(1.-msk_ref)*numpy.array([[[[-9.8419584087523511, 6.3743682378166246, -5.8471789139317982, -1.2998241669021209], [0.82193347299026454, 0.28517323031298658, 2.6002976131231774, -9.8105551328657921], [-4.467771979556824, 2.1570405793459981, -7.6439493446715598, -9.0311978085429505]], [[7.3999590377236233, 6.307364276780727, 1.2155145240421987, -3.8866934158652722], [-3.7774541146089842, -8.2683631113212179, 6.8629398349247968, -7.5004307695782444], [-2.9081198945491655, -3.5032243647913575, -6.0331876079900031, 8.1577611633090452]]], [[[1.0338444438095236, 2.7587848945007201, -2.1617905023076252, 9.4190121915759217], [3.4706890851378662, 7.8163496581343797, -7.0863926913234137, -9.1033821096509566], [5.6620466599609802, 5.9155714107821185, -3.6314377220334024, -1.6715084561722351]], [[5.9753311511349452, -8.2010805186743596, 4.6257347664816173, -4.2513602084457123], [3.0772609438351139, 9.9982443327864097, -3.338462424511222, 9.3226568818317386], [-7.5581230379040676, 0.33947043745784744, 10.15705687340771, 1.9294335918161372]]], [[[10.286331185093513, -5.0989371103923613, -0.29842032191146756, 6.4573528425650633], [1.1625584555592177, 1.2217188338532599, -4.3787044722022701, -4.7514057375904741], [-4.9865134481874822, -2.1874938699461608, -7.3376694393750652, 1.6879955365491586]], [[-9.8948851111210558, 5.7492977965929448, -3.1452532592623825, 5.0742667471357397], [-9.2071204543370602, -1.3180307279855565, 2.8368282011142716, 6.791440467567825], [6.3732715081214772, -1.0252402235144562, -0.40774050464339684, 7.7734588243077027]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-1.15920248053)+(1.-msk_arg0)*(4.87307689811)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[-4.5983722150554485, 0.31585265118779038, -4.6434558996682895, 2.0851944600735397], [-2.3951408877277993, -2.9355289500149508, 4.9157002732347266, 3.0700001673038386], [4.431922431057373, -0.45120807195833645, -1.4446938668137133, -0.4611013001159403]], [[-2.8986901415212953, 2.4643007272584248, -0.59698495567308818, -2.6915976852754131], [-2.0106922442484212, 1.1879318513684343, -2.5881656591744848, 0.35073832519391601], [-0.50139447639525336, -1.8805161483966373, 3.1904018702080066, -1.5804209769751578]]], [[[2.1406073407752313, -1.9295310632741289, -0.26665260807060864, 3.1338058108551969], [-3.4394566082348801, -2.7250881198953913, 4.5089422170040603, 1.9606249352276617], [-4.4801112084553782, 0.20077706383025706, 4.7314288106612139, 4.5736350166241238]], [[4.9450126914593238, 4.7498969205036126, 2.0326878017486738, -1.7175795744330991], [2.7772431517825504, -4.0230440841731916, -3.4843437320584583, 3.7228029521685642], [0.11982153231150328, -4.1892616357086698, -2.1928360450197748, -1.9127435950060425]]], [[[-1.1424250577994952, 0.21447461061068473, 0.63548750246448815, 4.5887783157961124], [4.609128600530406, -2.9690088498623912, 3.5471815794369466, -0.24237727054882718], [-1.6534493875965217, 4.4670829403184946, 3.4250925077936252, -3.024942375248564]], [[2.6718345891749875, -2.0632101044882489, -4.3092118433426183, 1.8768708064831667], [-0.22719786400443631, 1.4714673828531222, -1.4617851310060526, 1.3166654134736948], [4.5771913971395293, -2.5379253842605221, -2.0235848634832521, 1.2301631387157563]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.3304444781085589, -0.36613717673996604, 5.3827055971433539, -2.4171625905119405], [2.7764532582812524, 3.4028724405352047, -5.6982919502928535, -3.5587518091768415], [-5.1374954756137257, 0.5230415162508375, 1.6746927140219769, 0.53450977087161566]], [[3.3601688023494614, -2.8566235158184514, 0.69202644145741987, 3.1201067133694611], [2.3307994371222214, -1.3770535488110311, 3.0002080521466601, -0.40657673658294963], [0.58121772076316847, 2.1798989839046681, -3.6983217618438093, 1.8320279167967659]]], [[[-2.4813973392748476, 2.2367171948137945, 0.30910436471617425, -3.6327154694536121], [3.9870266319531802, 3.1589289082550844, -5.226777002533284, -2.2727612883117203], [5.1933560259073701, -0.23274127042626502, -5.4846840137861035, -5.3017690563255178]], [[-5.7322709782092423, -5.5060922925261773, -2.3562967419372312, 1.9910225031965061], [-3.2193871505909826, 4.66352268166915, 4.0390598972334857, -4.3154824166912018], [-0.13889741747681841, 4.8562024797172789, 2.5419409827901727, 2.2172571199555513]]], [[[1.3243019608247915, -0.24861950063136029, -0.73665868920486755, -5.3193232062891482], [-5.3429133068327035, 3.4416824234863705, -4.1119016857860142, 0.28096433324513748], [1.9166826315384693, -5.1782536251660387, -3.9703757310910452, 3.506520704859005]], [[-3.097197283346826, 2.3916782709845408, 4.9952490579470599, -2.1756732945161805], [0.26336832752585326, -1.7057286402274632, 1.69450494986919, -1.5262818133313614], [-5.305891621440697, 2.9419694008437101, 2.3457445933198113, -1.4260081618601694]]]])+(1.-msk_ref)*numpy.array([[[[-22.40822141008141, 1.5391742577089043, -22.627917672049762, 10.161312951443971], [-11.671705727696574, -14.305058310040616, 23.954585439515835, 14.960346892471351], [21.597098812985575, -2.1987716316993335, -7.0401043072060192, -2.2469820932818494]], [[-14.125539963416415, 12.008726943990016, -2.9091535960076444, -13.11636249911248], [-9.7982579246488442, 5.7888832614283725, -12.6123302821957, 1.7091748297830285], [-2.4433338397599016, -9.1638997992678224, 15.547073649386331, -7.7015129521805079]]], [[[10.431344180248916, -9.4027532486199821, -1.2994186642087233, 15.271276700030292], [-16.76073653962905, -13.279563962366641, 21.97242215257948, 9.5542760777094227], [-21.831926430871789, 0.97840207142087499, 23.056616432268559, 22.287675139881845]], [[24.097427107593756, 23.146612951693246, 9.9054439677642971, -8.3698973448294876], [13.533719443375963, -19.604603186648305, -16.979474945756174, 18.141505062415217], [0.5838995410029052, -20.414594097095666, -10.685858672321036, -9.3209466248250745]]], [[[-5.5671251569806754, 1.0451512701973098, 3.0967794672950792, 22.361469601237978], [22.460638103646552, -14.468208436538097, 17.285688608143026, -1.1811230777375927], [-8.0573860128849102, 21.768438678391604, 16.690739173606687, -14.740776806927107]], [[13.020055412070437, -10.054181496121533, -20.999120682839735, 9.1461357678035888], [-1.1071526623791554, 7.1705737096987461, -7.1233913519011427, 6.4162118089344728], [22.305005655612341, -12.36750555915795, -9.8610846495981814, 5.9946795721779038]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.6093742432549405, -3.9464724361383641])+(1.-msk_arg0)*numpy.array([0.64581752189931052, 2.3152421388562763])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-1.8823774154667805, -3.8249697734567123])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-4.911827144003821, 15.095137780009319])+(1.-msk_ref)*numpy.array([-1.2156723177359852, -8.8557311993585248])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.5176087482969649, -4.2724627629443344], [-2.6921489532884491, 0.98392197016487515]])+(1.-msk_arg0)*numpy.array([[-4.6877894446219077, 0.41976948456144214], [4.7020116656533535, -1.842169481684329]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-2.5139487805206153, 0.083820538479056239])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-6.329139442609188, 10.740752552723649], [-0.22565737493046542, 0.08247286936059374]])+(1.-msk_ref)*numpy.array([[11.784862557644658, -1.0552789838130048], [0.39412514974986823, -0.15441163792446438]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.7352657208854723, 2.9545484178308499])+(1.-msk_arg0)*numpy.array([-1.9269037807348433, -0.088917661609251297])
      arg1=Symbol(shape=(2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[3.4314415693862941, 3.4822385219222465], [-0.1119069379899269, 2.01927059349741]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[12.817346067350073, 13.007086182883061], [-0.33063446658243356, 5.966032737190134]])+(1.-msk_ref)*numpy.array([[-6.6120577334211541, -6.7099385733124892], [0.009950503243915788, -0.17954881933011474]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.2714773102530819, 1.3197723242438189, -2.0132615176534054, -3.0657199362255216, 3.2781022418809886], [-0.1511962699222833, 0.82924313947439909, 0.69676229952959723, -0.11124281195909624, 3.6084974980068729], [-4.6951654539004721, -0.53425070228965055, 2.2891993620951991, -2.3096914974100926, 2.518160258561954], [0.48580896737543622, 1.324634520115028, 2.1193928473085961, 3.0578410740896853, -2.5593147756479286]], [[0.42218487995015863, 0.16523999871395745, -2.6097511913085292, 4.4171600926849628, -2.5535143115606704], [-3.5773181555425126, 4.4232548577934825, 0.91330980237886106, 3.9288878634183746, -2.0260315570210405], [4.838911778221064, -1.8544983758994347, -1.4142855462164206, 4.4357066222766264, 0.86098585318484577], [4.2568598235733237, 3.9224276394176449, 0.65283518180174482, 2.2898961191697929, -2.1654715590708538]]])+(1.-msk_arg0)*numpy.array([[[-3.8260301356489403, 1.7441299945634059, 0.085570002454114835, 3.0655032359465704, 4.826857496585891], [2.1572320627882728, -3.2267081557760813, 0.36245545548605396, -0.38091056721413974, -2.7690860660660945], [-2.2811925638798214, -4.1318017559849904, -3.0242519841047244, 1.9775132153205233, 2.1075255697396802], [1.4125501641588887, 1.4271011600389736, -1.9850900826470621, 4.002856047189459, -0.7725992884316728]], [[4.2907522937680902, -3.8336690938685347, 1.6195217216562572, -2.7007490166439077, 3.0394408013702066], [-0.72001756478127898, 0.86186068030376006, -3.5602770013734233, -2.2866203343923086, 0.81670315566722351], [2.9937374879815959, -1.6297096456752969, 3.7927278160514906, 2.2775758292124317, 3.9231261938675388], [4.9326424718466981, -2.9872293265479435, -4.5639405552661696, 0.073490557756803909, 4.523003224979103]]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-4.6913966804497349, 4.3722414762734605])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-10.656401113038202, -6.1915755009068834, 9.4450084007963824, 14.382508331996986, -15.378877975735303], [0.70932167880978192, -3.8903085118159124, -3.2687883390756762, 0.52188415874879823, -16.928893183560618], [22.026883624590948, 2.5063819712496063, -10.739542288221068, 10.835679023812686, -11.813688677857996], [-2.2791225768778349, -6.21438599047677, -9.94291256843246, -14.345545464327202, 12.00676084270065]], [[1.8458942427736154, 0.72246917591653803, -11.410462401393225, 19.312890564577117, -11.164581183263435], [-15.640898813489049, 19.339538349372731, 3.993210998647974, 17.178046472065237, -8.8582992058462917], [21.1568907767663, -8.1083147167892786, -6.1835979244615, 19.393980470498722, 3.7644380577794752], [18.612019079309412, 17.14980081274323, 2.8543530590441137, 10.011978788591803, -9.4679645662601413]]])+(1.-msk_ref)*numpy.array([[[17.949425077684086, -8.1824056667675773, -0.40144282545930998, -14.38149170502766, -22.644703236486968], [-10.120431338324638, 15.137767930787994, -1.7004223206781701, 1.7870025705766408, 12.990881178222091], [10.701979221652413, 19.383921042304369, 14.187965719072428, -9.2772989339001839, -9.8872384618396723], [-6.6268331511037388, -6.6950976448728063, 9.3128450241241172, -18.778985572102776, 3.6245697370661771]], [[18.760205143228333, -16.7617270185197, 7.0809400431512906, -11.808326867575255, 13.289169136428661], [-3.1480906603821213, 3.7682630131933608, -15.566390772427386, -9.997656266520341, 3.5708234110116552], [13.089343214027855, -7.1254841071044579, 16.582721865556387, 9.9581115058405132, 17.152855061482491], [21.566704003036179, -13.060887960673355, -19.954650190981276, 0.32131846473876835, 19.775662297572257]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.9561962737547933, 3.640475115601669])+(1.-msk_arg0)*numpy.array([4.8772025518556852, 0.032985776085920904])
      arg1=Symbol(shape=(2, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-0.29196812653682969, -0.50146385964178553, 4.0612988195065327, 1.068960871435463, 0.060500041707955532], [-0.032258784963785203, -0.18426661913922437, 3.2945641068179636, 1.801650099239227, 4.2315445483291079], [-1.2541027382197112, -1.4837980243055271, 2.7565627384949076, 0.89959135685024538, 3.8660253132794811], [3.8800818973625244, 4.7377730183907047, 0.55276640336332861, -4.1086613481005196, 1.6012201512779844]], [[3.4866175336006027, 4.0266781470835973, -3.3815054370620334, 1.8335445156898844, 2.5087082149980455], [-0.35769837363628199, 0.60960007820747109, -0.28495349294444594, 3.1011043545659494, -1.5818324728432809], [4.6258730260377128, 1.9151407854864066, -1.4774229362100524, 3.2116275216775314, -3.5753355420132307], [3.3289468599003929, -1.3505202178709896, 4.5903979854618608, -1.1395239255926128, 4.8758530500569783]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.1550832142601737, -1.9838894529375286, 16.067295256336486, 4.2290190163626553, 0.23935003956702325], [-0.12762208486958418, -0.7289949120159932, 13.033942243039517, 7.1276814092201839, 16.740820774327027], [-4.9614765798505047, -5.8701962147622506, 10.905503234404863, 3.5589599738729594, 15.29475493863799], [15.350365544209048, 18.743559961253307, 2.1868523852428399, -16.254670715475623, 6.334741195947049]], [[12.69294436869346, 14.659021592994874, -12.310286396896078, 6.6749731827169381, 9.1328898290058671], [-1.3021920281140726, 2.2192339151831297, -1.0373661001680312, 11.289493233681315, -5.7586217544366169], [16.840375639223286, 6.9720223724370971, -5.3785214344918479, 11.691850073248514, -13.015920070625372], [12.118948204627696, -4.9165352462762826, 16.711229636781937, -4.1484084947526352, 17.750421696062929]]])+(1.-msk_ref)*numpy.array([[[-1.4239876918059493, -2.4457408159083176, 19.807776966345742, 5.2135386899989173, 0.29507095780541609], [-0.157332628345137, -0.89870562508764473, 16.068256469024718, 8.7870124615606073, 20.638099869401739], [-6.1165130751343781, -7.2367835105813407, 13.44431482253766, 4.3874892612573353, 18.855388523465361], [18.923945331225553, 23.107078655408156, 2.6959537130637155, -20.038773611666674, 7.809475007895732]], [[0.11500878526059528, 0.13282310372977041, -0.11154158118025234, 0.060480888838114817, 0.082751687444835853], [-0.01179895845906447, 0.020108131681711511, -0.0093994121131665363, 0.10229233385878667, -0.052177971754647022], [0.15258801183878135, 0.063172405123069292, -0.048733942158028593, 0.10593802630143613, -0.11793521762088308], [0.10980789572260387, -0.04454795750620158, 0.15141784009370735, -0.037588081054147522, 0.16083379693703398]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.8382273516008318, -2.9353805044036827], [-3.877881750190193, 4.9297689327655352]], [[3.06307741305646, 2.8747676563997189], [2.3304795221628538, 3.6561304347512689]], [[-4.7348188242955942, 3.6916496829948571], [2.605853147298955, -1.2929286633769568]], [[-1.5687649387487115, -3.3657832145011488], [-0.62001779579562655, 3.9185218671896909]], [[-0.53953789346124381, -4.958014743025597], [-1.8824604797064861, -0.037327167581914011]], [[2.9181995071040561, 0.21442080128434249], [-4.5113275208725945, -2.4705666846557541]]], [[[2.0027634761746711, -4.7571267066534446], [4.2100164778607088, 4.0979881397728732]], [[3.6482603776259523, -4.249582088775437], [-2.6167855295576992, 2.3919037211474432]], [[-2.5640296010939179, -2.6019087282783104], [-2.6944933592247988, 2.2486392162238484]], [[4.2272451001788838, -0.055223529183675346], [3.3893035402825937, -1.7361160207553326]], [[0.3057937312513479, -3.7252421474674593], [-2.5327116542276054, -3.0110862236077485]], [[1.5176238145150025, 1.7095498177745796], [4.5054927348245251, 3.1816476683876989]]]])+(1.-msk_arg0)*numpy.array([[[[0.046377321057516241, -2.7534905024014602], [-3.5555951364516147, 4.8566364185982387]], [[-2.4216701973103638, 2.761770670882151], [3.1569180104265619, 2.2559913919806318]], [[-1.6829197042616206, 3.9261899529038828], [-2.0842129284192987, -0.15223204145575053]], [[1.9654251652526806, -3.2970953389902782], [-1.5574099637799743, 1.9679518316708657]], [[0.34189819871482818, -3.0645769797827627], [-4.8203472298255221, -2.2736475516224717]], [[1.6115702225030057, 2.8732545208975937], [4.9104292091612276, 4.8442588000127991]]], [[[1.7539611334947223, 2.1122398140582952], [1.7747878746706078, 3.5414262130233976]], [[3.3456436363703919, -1.5915958350387016], [-3.0431077257570505, 4.1427378815178386]], [[2.1150328272185739, 0.95819844677109955], [0.64612850709834913, -4.8100321281558607]], [[-2.7705568985601294, 1.247799029401973], [-3.7995425691570475, 4.110152735528418]], [[-2.9089834282148983, 0.21758361083157229], [2.9397637776608878, -0.17583179111472003]], [[-2.3073553872356909, -3.1325730356294468], [-0.6367241447209917, 1.923246015171185]]]])
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s1=numpy.array([-2.6450294250235675, 3.9457172470988837])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-4.8621654348673431, 7.7641678077882625], [10.257111336014951, -13.039383885731869]], [[-8.1019298886594058, -7.6038450412832974], [-6.1641869105356113, -9.6705725816413146]], [[12.52373511241734, -9.7645220384003224], [-6.8925582518960091, 3.4198343590884415]], [[4.1494294239356364, 8.9025956406059485], [1.6399653139176857, -10.364605641315023]], [[1.4270936041202205, 13.114094885003366], [4.979163360267636, 0.098731456606948362]], [[-7.7187235643794994, -0.56714932873421708], [11.932594038626634, 6.5347215773973906]]], [[[7.9023383898021144, -18.770276893077209], [16.611534627265495, 16.169502481508498]], [[14.395003893906207, -16.767649340643743], [-10.325095795934599, 9.4377757659314661]], [[-10.116935819108342, -10.266396144544853], [-10.631708919686696, 8.8724945379573548]], [[16.679513899490072, -0.21789643154569635], [13.373233434546336, -6.8502229260589989]], [[1.2065755994531644, -14.698752190882038], [-9.9933640560142063, -11.880894844990939]], [[5.988114459539843, 6.7454002007679126], [17.777400390475847, 12.553882079349293]]]])+(1.-msk_ref)*numpy.array([[[[-0.12266937885089557, 7.2830634003747887], [9.4046537593852069, -12.845946233833418]], [[6.4053889295885407, -7.3049646896503679], [-8.3501410299651138, -5.9671636143886486]], [[4.451372137723947, -10.384887953662664], [5.5128045236835836, 0.40265822908186771]], [[-5.1986073947751477, 8.7209141887373391], [4.119395181022921, -5.2052905017984665]], [[-0.90433079596327537, 8.1058962867752626], [12.749960261719346, 6.0138646761742285]], [[-4.2626506590122277, -7.5998427533561284], [-12.988229747726653, -12.81320706846321]]], [[[6.9206346951712332, 8.3343010643387547], [7.0028111270297897, 13.973466488054505]], [[13.200963798673282, -6.2799871367229549], [-12.007242638299454, 16.346072309314827]], [[8.3453215045366402, 3.7807801375680894], [2.5494403943002095, -18.979026727164328]], [[-10.931834138717495, 4.9234621512246122], [-14.991920646209365, 16.217500536785135]], [[-11.478026084232361, 0.8585234059441863], [11.599476639913133, -0.69378253078963903]], [[-9.1041719466023885, -12.360247454480014], [-2.5123334394699026, 7.5885849724751457]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank1_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.4002905020658751, -1.8947722391002033])+(1.-msk_arg0)*numpy.array([-4.1340598571830238, 2.6654599419062777])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[3.8039308349141336, 4.5154088089051818], [-3.0300355566736457, -0.70602228265929234]], [[3.1992458443114753, -2.7121545722674423], [-3.420522017146884, 2.3290756120555054]], [[-3.5325187821631809, -1.7871723912020165], [2.913343969818377, 2.6456463010101725]], [[-0.52475889480626758, 2.0729019357310996], [-3.4429499776352457, 4.8706161163550075]], [[-2.576888181581233, -2.3719775994000045], [-3.1234006360838862, 3.0162879926069888]], [[-3.5796325346335633, -4.9904853040576533], [2.0643987958635712, 2.6344120716093125]]], [[[-0.10177816190016564, 2.5431499178316095], [-4.4860535803210775, 4.3649314640074497]], [[-3.027099465038364, -3.8224845993585941], [-0.22545345009899265, -3.4513328837733748]], [[-3.1223904518711776, 0.59197973181679231], [-4.8261107092801181, 3.0319610536902477]], [[3.3920476876119725, 2.412563436890748], [-2.0729166759501636, 2.8553113225374274]], [[-4.3123178097353909, -4.1530309105442633], [0.021166156259665669, 2.9628246361951831]], [[-3.1971817906640396, 1.3847225623397419], [3.7751846129666351, 2.6061252254269593]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[12.934469888474043, 15.353701685864875], [-10.303001124279284, -2.4006808619732602]], [[10.878365258186031, -9.2221134322155205], [-11.630768527011758, 7.9195336822655999]], [[-12.011590063358776, -6.076905307358575], [9.906215829824319, 8.995965989150605]], [[-1.7843326858843374, 7.0484687637804253], [-11.707030108041042, 16.56150971965091]], [[-8.7621684087164713, -8.0654129023528505], [-10.620469517022551, 10.256255412756888]], [[-12.1717905084005, -16.969099780086569], [7.0195556180511307, 8.957766345620831]]], [[[0.19284643571507987, -4.8186898641772968], [8.5000497871084519, -8.2705509635763246]], [[5.7356640313497689, 7.2427377032527271], [0.42718293845693428, 6.5394897360674387]], [[5.9162187478370472, -1.1216667619564413], [9.14438059476816, -5.7448756345652825]], [[-6.4271577921912044, -4.5712582252887648], [3.9277049715582422, -5.4101646279324038]], [[8.170860072064011, 7.8690476774243097], [-0.040105045289271501, -5.6138778699847922]], [[6.0579313003069002, -2.6237338699770438], [-7.1531150021274259, -4.9380137287577615]]]])+(1.-msk_ref)*numpy.array([[[[-15.725677764119224, -18.666970295665525], [12.526348360681736, 2.9187383770185065]], [[-13.225873818227679, 11.212209343686228], [14.140642761697636, -9.6285379921426468]], [[14.603644092085869, 7.388277640334052], [-12.043938355792383, -10.937260169310909]], [[2.16938468171832, -8.569500680382923], [14.233361292831059, -20.135418566371918]], [[10.653009987924333, 9.8058973758169152], [12.912325187534316, -12.469515107939717]], [[14.798415164894934, 20.630964963366562], [-8.5343481911965622, -10.890817192518428]]], [[[-0.27128561350574326, 6.7786642322423969], [-11.957396115591068, 11.634549966478181]], [[-8.0686123642256824, -10.188679578143999], [-0.60093714000343079, -9.1993895478818057]], [[-8.322606672453265, 1.577898261578081], [-12.863804770791049, 8.0815707340313043]], [[9.0413672323655323, 6.4305911983400232], [-5.5252763626546777, 7.6107179518949479]], [[-11.494310378618701, -11.069737529554288], [0.056417541634267654, 7.8972903826713008]], [[-8.5219599900071792, 3.6909225205704002], [10.062603359163521, 6.9465223919670276]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.84137523729844421, -1.3364986810944171, -2.6963137262213688, -1.6388020812486737, 0.37395884436753679], [4.1646153061423483, 3.8320409148127936, 3.0599663704062294, -2.4097409049716809, -0.29328787837586034], [4.5183512416665845, 2.0662828937870179, 3.1901196711227264, 1.5781615848300294, 1.6102339778246506], [-2.414632170085925, -1.4386760627496686, -2.6701084245593698, -2.7887941254721573, 3.6852558818790744]])+(1.-msk_arg0)*numpy.array([[3.0553684560460574, 3.1688175968093777, -0.35965978599140502, -2.0936024936124955, 1.8156523573944501], [0.47918874902853936, 1.2186224430193349, 1.622888553979581, -2.4640634528055951, -4.5966067151319265], [3.1553521303287262, -0.7844665648856779, -2.3154005193663574, -4.0531931812432482, 1.0606594690186997], [2.2252896663521824, 0.46584297292449683, 1.3894011327929867, -3.116654623518742, -2.6201584704513436]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-1.3671025208296816, 0.38280378923095526, 2.11106386165015, 1.9350886942144232, 1.0555730401523764], [1.5803552675067678, -2.7721527625754048, 0.81909075187955516, -4.6702069162274151, -1.4372828504619131], [-2.461720703071494, 2.0863181748467046, 0.99165081175762371, 4.155268525855206, -1.9433140651532033], [4.1904010661992253, -3.3456060808062982, 2.320939121784833, -4.1892359407927762, 3.6756480482291014]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.1502462078743747, -0.51161675942511686, -5.6920904670971879, -3.1712273794793751, 0.3947408742409102], [6.5815717362013704, -10.623002808300267, 2.5063901550621916, 11.253988640714855, 0.42153763783798343], [-11.122918795359423, 4.3109235556826979, 3.1634847614728154, 6.5576851621579921, -3.1291903372942347], [-10.118277220007009, 4.8132433838457551, -6.1971591019671077, 11.68291658189972, 13.545703589453636]])+(1.-msk_ref)*numpy.array([[-4.1770019183240574, 1.2130353834403591, -0.75926477669528203, -4.0513065155686645, 1.9165536787546886], [0.75728846365723079, -3.378207571952438, 1.329293005895859, 11.507686179315895, 6.6066240019771865], [-7.7675956647109681, -1.6366468518805515, -2.2960688045736717, -16.842106055231007, -2.0611944644819675], [9.3248561904843026, -1.5585270829170803, 3.2247154449514066, 13.056401563882693, -9.6307803679654285]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.5873712341789528, 3.9716817164550591], [2.2724618178530323, 0.16971449424838525], [-4.1361448792651387, -3.7858373918885313], [0.12864908555476973, -4.6710320993575518], [4.6023272451534982, 1.7560455615185404]], [[2.953943559259038, 4.9490269322135596], [-4.4474021782192263, 2.4939015591087719], [1.7751110226822533, -3.9608761176711358], [0.17660086808041076, 1.9055594253343404], [-0.42979928521098021, -2.4899799585164906]], [[1.6464547058652386, -0.96213697981852508], [0.15391544538697843, 4.8233685485583706], [-3.2391582931373177, -2.2346469528221058], [4.2475944546001223, 3.5004632203212598], [-4.0791252824931767, -0.14213488017254949]], [[-3.1931637723900463, -1.5638289073488467], [3.7654852643148011, -3.9951188545711913], [-4.9361664183273311, 2.199045394902905], [4.3490456832010338, -4.7708572502813755], [0.33770305108285648, -4.3063315880592885]]])+(1.-msk_arg0)*numpy.array([[[-4.3057337146006427, 4.006180614215209], [0.78041562902633821, 3.9025808116966267], [-0.021716003725495092, 4.4983573603355769], [-1.4974592269154474, -2.673213974173847], [0.8029880034894612, 2.0893800123368695]], [[-4.7468387574975459, -2.8764004728059769], [1.1865736923919679, 3.5796992761800102], [1.7423380711896028, -4.9129438199643181], [-4.1026508151513887, -0.25979896061101471], [3.964117306933483, 1.5022770998271877]], [[2.0744187402669017, -0.86463606726533371], [-1.1166347152372511, -1.2793320566256794], [-3.220954811655965, -2.3762763063462344], [-3.0767407830595306, -2.8320908180335538], [4.2811821152438068, 3.5025431929022961]], [[-3.382920802895506, 1.0100791179622828], [4.3769568973570259, -4.930614023327867], [1.9981078550065501, 3.2996093060674063], [-2.2157830834458658, -3.9840948852038305], [-4.8569456827514381, 1.057874795844242]]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-2.7571011150887048, 2.9735913993658354, 2.1771611189190878, -1.1344181793357722, -0.25358182301507437], [3.0321868763724193, -3.3336583267409514, -4.8493016476236805, 1.1517999566744255, -1.6445888448364157], [-2.2256651917429449, 2.791907804775593, 4.540519660501408, 0.38451540689019037, -4.1656379722749657], [-2.0440433035333951, 3.9482269599795163, -3.7543107305878678, 1.7398152479654652, 2.0466326215687056]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-12.647846345080639, -10.950328089215665], [6.7573729169550285, 0.50466156044472088], [-9.0050538133523457, -8.2423779721697557], [-0.14594186140825385, 5.2989037297721433], [-1.1670665329379692, -0.44530123478740141]], [[8.9569088939300894, 15.006374514671611], [14.826119303786369, -8.3138156985951976], [-8.6080488070080072, 19.207483083455926], [0.20340887220368303, 2.1948232635406364], [0.70684310997664312, 4.0949932636424613]], [[-3.6644569286256301, 2.1413947856707756], [0.4297177332514166, 13.466400296029239], [-14.707461913466174, -10.146458423568333], [1.6332655100150824, 1.3459820394659754], [16.992159170420422, 0.59208245403152426]], [[6.5269650260393082, 3.1965340059383562], [14.866990437973293, -15.773635969940463], [18.531902552293779, -8.2558997231338118], [7.5665359937315424, -8.3004101899061293], [0.69115408074945706, -8.8134787074139087]]])+(1.-msk_ref)*numpy.array([[[11.871343225800462, -11.045445038699505], [2.3206372024033977, 11.604680736991231], [-0.047279238969449973, 9.7936487439261182], [1.6987449698269748, 3.0325425295572392], [-0.20362316178409248, -0.52982879249964199]], [[-14.393302184740019, -8.721783764833706], [-3.9556312699342402, -11.933494299266048], [-8.4491228793372066, 23.824346560835547], [-4.7254330311416659, -0.29923643157582752], [-6.51934310260578, -2.4706281602289955]], [[-4.6169615833112916, 1.9243903984379647], [-3.1175411765542531, -3.5717771537928451], [-14.624808647910518, -10.789529287748744], [-1.1830542340937784, -1.0889825532461441], [-17.833854785484061, -14.590326923887005]], [[6.9148366135423753, -2.0646454571097226], [17.281219224813306, -19.467183216156158], [-7.5015177609229982, -12.387758624516451], [-3.8550531947630522, -6.9315890306188441], [-9.9403834755063833, 2.1650810667101603]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.6109846457957202, -1.8610861268350831, -1.7010080749362269, 3.0394982591129001, -3.8872821419942682], [2.2598087021344693, 1.2574139385771979, -2.6378733271798782, 3.7767658076328168, 1.7135291854268528], [1.9981355126205855, 4.2556693884462877, 3.0286723685296497, 4.5010851101863096, -1.4867089100446798], [-1.7178246073519876, -4.0859100941176312, -0.024976100879925411, 0.82551517821789488, -0.92174320495665896]])+(1.-msk_arg0)*numpy.array([[4.9912879472293721, -2.7299280799893966, -3.2710814186345862, 2.6908397405292925, -1.4895709833645601], [-4.834388024973296, -3.9877325285031939, -2.2265964396850588, -0.28203295025701269, 2.3660639701734718], [3.7222793096766491, 1.9569772131959127, 2.0309601471060725, 3.3544572026511865, 2.2781842180196064], [-0.13422134317592871, -2.5833387120722584, 1.6158446324511511, -3.7592690625095715, 1.6676548018031232]])
      arg1=Symbol(shape=(4, 5, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[4.8354328397120572, -2.0703899001190731], [0.25419209979340796, -1.1601150714104969], [-0.27488298875790651, -4.009793826454688], [3.158163359534333, 2.6082014479245981], [-2.0631663931359432, -4.538458664256857]], [[4.5901364327645986, -4.188075907360429], [-4.8886167588888103, 1.4282448576282896], [-0.50846036071144063, -2.1814656549482758], [1.0822657359402452, 4.5480708706272175], [4.4538363375329055, -0.51138272042537558]], [[-3.6774078505200136, 2.9441557977233508], [-1.2454338787514674, -0.71592664651441495], [-1.3586868510572438, -0.78222759408348086], [-4.0865996693590549, 3.5815991983884174], [-1.3915008643239979, 3.2570204088470263]], [[-0.63504455210036426, 1.3576861854577738], [3.6617768125461474, 1.2513846161435751], [1.4791209690426328, 2.0023775061196272], [2.6826284470771817, 1.5346149007379193], [1.5443614699796964, 1.7386624426226351]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-22.296106579688693, 9.5465360402595802], [-0.47307339047659053, 2.1590740649343676], [0.46757818353980302, 6.8206916776288553], [9.5992320332987529, 7.927623760382561], [8.0201098760000775, 17.642269317744841]], [[10.37283025474591, -9.4642503806528104], [-6.1470148529888746, 1.7958949916830167], [1.3412540234489689, 5.7544300653470408], [4.0874642262716847, 17.176998554875691], [7.6317785514772769, -0.87626921637186184]], [[-7.3479592205137729, 5.8828222541188167], [-5.3001548331365456, -3.0467471139444018], [-4.1150173232816343, -2.3691111001020655], [-18.394132923044339, 16.121082822521327], [2.0687567333253609, -4.8422412620302397]], [[1.090895158362827, -2.3322667384412181], [-14.961690840788188, -5.1130450347245509], [-0.036942674536421825, -0.050011582592537268], [2.2145505005813142, 1.2668478932785006], [-1.4235046909506623, -1.6026002922007609]]])+(1.-msk_ref)*numpy.array([[[24.135037652491889, -10.333912154529752], [-0.69392615093749122, 3.1670307094624195], [0.8991646368247278, 13.116362078271607], [8.498111474918483, 7.0182521073815503], [3.0732327930682195, 6.7603563354764944]], [[-22.190500603550817, 20.246784014222428], [19.494496068806765, -5.6954584774317434], [1.1321360288810745, 4.857243660603066], [-0.3052345984693044, -1.2827058456209743], [10.538061687285982, -1.2099642297677746]], [[-13.688339155233127, 10.958970210330179], [-2.437285721258823, -1.4010521335484751], [-2.7594388468943065, -1.5886730695502156], [-13.708323695233439, 12.014321228043743], [-3.1700953084635737, 7.420092493203061]], [[0.085236532759466924, -0.18223046342354543], [-9.4596097948190234, -3.2327503225753809], [2.3900296785734834, 3.2355309454043213], [-10.084722127305344, -5.7690303192102572], [2.5754618211313707, 2.8994887711543846]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.9346748322739744, 3.0481207946840563, 1.8379307788718924, 0.4052864714572193, 3.2707457565403413], [1.7529470019674429, -0.83873329678873088, 2.6818741709719056, -1.0822692365851703, -4.8009603632757774], [-1.0045087305268185, 4.3582244951233395, 4.4324127942645095, 0.4232638013262946, -2.903434836268588], [4.4787140773440957, -3.530382000925687, 4.8744340640633492, -3.5463707663040056, -1.7066480146908845]], [[-4.3970724852302912, 3.8844246853969562, 4.2745197359444642, 4.8677355121045203, 3.6903014580412972], [-4.7848067984025917, 4.7061366876486339, 2.5236647082774883, -4.1030082334911864, 4.1318683161675125], [-4.5091536125508647, 2.1604315454121847, 2.2094972998309172, 0.77539913950875494, -4.7715012252206259], [4.9866128861618257, -4.0716197079867191, 4.5606977502643371, 1.9308949133105857, 1.5446071203342173]], [[1.6118268902953297, 4.8024776909663185, -4.9459747333216617, 3.6274023201774153, -2.6593978067447877], [1.5172552752422721, 1.4696282380626133, 3.8606626286644854, -4.6036042136814039, -3.903775147208167], [0.91652072758585224, -3.7222841310616026, 4.0700723480117293, -3.5851705033947088, 1.9773701053632822], [0.84230081856180661, 4.3995569218948081, 2.4945408617724096, -2.0264517892726222, -3.1804521694547985]], [[-2.6221435285395867, -2.2738629854484174, -2.3930220271881906, 2.4619335964603621, 2.9848128132038436], [1.6180199705405087, 2.8809713115940419, -1.9178109339545104, 3.5873985984655867, -1.7346745843958211], [0.93526521592777456, -2.8657451723669523, 0.67036927214590847, -1.5140258688856587, 2.3449534379140671], [-3.0840640560719734, 2.3544661250876509, -3.6974168648677894, 4.8914025198638242, -1.0370153952578152]], [[-0.87358747493009403, 1.4854011751833838, 4.6871711663776434, 4.0447015225248233, -3.3724221537842336], [4.747681205231256, -3.4739240165878513, -2.3074550103679528, 2.8650776734361907, -2.2661293421564599], [-2.6913919975572975, 4.221105736908795, -0.07892190643871011, 1.4382870861255048, 4.0401013932397163], [0.77593870610113314, 1.0556653960006477, 2.9724557647016647, -3.847317319929223, 1.0791415476767705]]], [[[1.5617510037509348, -1.1854404561812948, 2.1249247794415238, 1.5318491626165702, -1.3428276043875034], [3.190692820440761, 1.6101645691073161, -1.3107179185877071, 3.3291483477890011, -1.520480380414595], [-1.6213080862687743, -3.5178906815168718, -4.7678714391738772, 0.095254052264341915, 2.3501435485931896], [-1.9433277894376353, -4.8595686992249814, 1.0610174347384094, 0.50992896879129468, 1.9019785854048248]], [[2.997372225522593, 2.9338192003383128, -2.5028884527692341, 4.2064684898581071, 0.98901463949488821], [1.9485960065891987, -0.9969576060130958, -4.0876431104139748, 3.6229485720805741, -3.7497887991975842], [3.1845194180925649, -0.27936648511917461, 3.0244504353942077, -2.9146116273751033, -3.3476097061881882], [2.6411411762917822, -0.97144936353425493, -1.0602286790606397, 3.4865653043808269, -1.3273444137769341]], [[-2.2378250765174821, -0.68276719115844564, -4.2907705261038025, 4.9154016630325668, 0.42099415331003875], [2.3723023532945824, 4.5273366848012095, -4.3585502266513156, 0.87902235459940492, -4.7475950591138467], [4.8410730311105823, 0.75579113345433058, 1.7255084402684719, -2.168252118526004, 3.4465907950723409], [3.0182282371601659, 2.9792242724639504, -0.47357186613746105, 1.6493332645234471, -3.0945117014446999]], [[2.3463776936311191, 2.2909162124064348, 1.8940279261209287, -1.8574225245007323, -2.2392702452148896], [-3.9381111551137513, 2.5400702343320889, 3.4464048404344751, 4.1496240397587307, -3.0703691571988969], [3.367259174459484, -1.6142716036961193, -4.5658724973383649, 0.98698769212446891, -1.062073423484915], [1.6802236027413731, -2.7992619130542682, -3.894957803441097, -0.81535867513033722, -1.5345049825370536]], [[-1.294138693249407, -4.2938525221604449, 4.3611645344646313, -3.1632757996203074, -2.1484712557616437], [1.4097360459964614, -1.7775867466844177, -1.2890190799534307, -1.124141106302301, -3.5275688634506919], [3.9274066964497258, 1.5937710558625673, 3.297507937955551, -0.11859074916434409, -4.4770393696172253], [0.11912339091287105, 4.1154289014723187, 2.6719211145573887, 3.1115836678155802, 2.5555584918539198]]], [[[-4.7757315602540746, 1.5808678490809438, -1.2379102304702405, -1.0721966246052785, -4.9744822687283978], [-4.9259925294272042, 1.3640393605637637, -0.81750836304970775, 3.2851616216437325, 2.8367017159541739], [3.855790226242906, -1.2393662632473399, 2.1566790200415111, 0.00034665726654914408, -4.3719934643824931], [-1.2562439755143218, -4.4507421743671847, 1.4131675252551101, -4.397558764716079, 1.2100384032858011]], [[1.5980050188940904, -1.0164047414773316, -1.4756850444921166, -1.5507245657035487, 3.7230789236721922], [-0.82058237195807759, -4.6078224476211069, 4.2710135788405914, 1.5326385468823602, -4.2060588519341566], [0.029372429810447187, -1.1319209717486123, 3.7703865987644445, 0.036041850550865284, 2.4324065564425732], [0.58962923974684145, -3.262706229478419, -4.5383726116187439, 2.0127583028635243, 1.2398211439239137]], [[4.7836687831497713, -2.6875341440623943, -4.7604119046401516, -4.8391784563427178, -2.6109141709226371], [-4.4956207455294219, -2.0292553069073338, -4.315873961561322, 0.86977680142602321, 3.6606203090320637], [4.8680762120311591, -2.7504537383787877, -0.75167861197502006, 4.8385361530927558, 0.0085917876119410153], [1.4429546275093426, -2.5976482835815862, -2.1977929857374221, 0.010511654910435375, -0.22271653295975824]], [[-4.3148152729387892, 3.5461150646180783, 4.7667205452140227, -1.4970008894899181, 0.55673250501383365], [0.49417346389118322, 4.4848955430699871, -0.417569078109433, -3.9624616416117409, 1.3999633405941907], [0.54252974536660581, -0.92638267517560546, 2.3502922592986835, 0.67681024774191201, -1.581901532981953], [-3.9163746323093385, -3.2512153246287778, 0.35529481251930939, -1.3688574590104929, -1.5843827304270821]], [[4.1651215603502614, -2.7569358449490577, 2.7592042546800144, 1.3659869353641083, -1.3898286764838463], [-3.204059375357561, 3.7614919225757824, 4.314569711899491, -4.6460387832414654, -2.7279457082286438], [-4.3915434078662363, 3.5895012332114451, -3.1660097224161388, 1.755828958202688, 4.8956341300908601], [2.0771312948576126, 0.84969031169437059, 4.7999509440488168, 3.1966964349487519, 0.28816533443049686]]], [[[3.6844282548628033, 3.0607278915072431, 0.99470525231908091, -3.6849330195716146, 2.1102127363984993], [-3.2849459226627511, 0.10968438885144494, 2.6095512606003757, 1.122893892751633, 0.68668578992830653], [-4.2950925114378444, -1.2670773564706295, -4.4595060559805155, 1.5956422413876776, 0.14374687535895081], [-2.494468704598245, -4.58312641205824, 1.7104620055136728, 0.38223509300414449, 4.838446127751121]], [[4.9053552747813427, 2.5959137201419082, -4.1271724637061773, 2.3738409623085079, -4.4070279361022004], [4.4420224550452954, -3.9014158747852679, -1.7226046677612605, -4.7972025953087787, -3.673293211268287], [-4.1483080450472709, 4.8661910108124378, 2.8358881867675159, 0.63683133938584735, -3.7431985053588712], [1.6400134557024773, 4.5011604645714023, 3.5972179713984822, -0.97935672090608161, -2.2658163935181119]], [[4.779545374091029, -1.2911859827899539, -3.0807715454103377, -0.22597320791051523, -0.65228278629124947], [0.58641872760084013, 2.8625793122326124, 3.0081899015661104, 4.9321975332946728, 3.331394721133524], [-1.9001564238109503, 2.4766239349486998, -1.1210365205256245, -2.7169765674477286, 3.9141836059430783], [-2.0988801381912845, 4.6538182429440376, 0.7466695449585945, -1.4232851661715507, -2.8167978897004344]], [[-3.3511451602419795, -0.70730223676734294, -1.6677668468931648, 3.7268554806286645, -1.182881165876597], [1.8304860123072011, -3.5692970939913451, 1.9845750082327571, 2.0633556538545186, -3.5770119279582744], [-3.5283458203613538, -2.3101203542228923, -0.91750620047591802, -0.50105560201067956, 4.997989884391604], [-2.2932774883201867, 0.46961425526901657, -4.0888336716655607, 3.6923887910042037, -4.9400984863697932]], [[3.7791313284533956, 2.763221360705268, 0.21407137411708277, -0.86014475952386249, 1.5202363557726617], [0.72576189173789718, -3.4241181201455486, -3.200506859692088, 1.8694263936907074, 0.65949182284736363], [-3.842197250422573, 0.060757257398597275, -4.6243491404116268, 4.1474663027829504, -3.9047053314158351], [-1.7614823454547048, -1.2544021509556202, -4.3552148554597938, 4.9754892500263193, 4.4001768537350703]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0727056096667487, -4.9957199172923037, -1.6411142636400653, -4.0293952273068712, 4.5305868257797606], [-3.0116628206013427, -3.2179690377472117, -1.7648437688784546, -3.6628532820243676, 3.6236268598291197], [-4.5709589071184062, -1.6346880449411891, -1.7633660417494101, -3.3447122542804197, 1.9155587118414559], [2.1707157268473143, 4.2058374740329363, 1.2763951758474024, -3.7704163876519328, -3.7758959880106446]], [[-0.19018090342213778, 0.12261592668411669, -0.70686980593984838, -1.9701586772191284, 2.0669200845242415], [-3.2977067835424458, 4.4605595639297082, 0.95925760534646987, 4.5048017430794847, 1.0520730614761211], [-1.4764052993096399, 2.5507911468540723, 0.5502057861567069, 1.0132174339810343, -2.4564657937446577], [-3.0384189209593249, 4.6943646919037398, -0.17365795778416704, 1.1783720185007702, 2.5553939352698318]], [[-3.6144604668776181, 4.7566681320149033, -2.2243244684203001, -4.7108636621519384, -3.2749082029312859], [-4.7339455232848646, 4.7197525715846798, 0.77014340906429357, -0.842630139108131, -4.5380093743444592], [2.8977518935584854, 3.9667707555704403, 3.4225107884863863, 2.546458443322984, -3.4424847644275958], [-4.0514358783326703, -4.1738526403774667, -4.2099974439883106, -4.4531477980434486, -2.9198546191855423]], [[0.789634772763117, -2.8249904882493815, -2.9523994061840284, -4.6890973673756609, 1.7055568162105006], [-3.760707678282488, 3.5604612410674523, -2.0162487534218321, 1.5348962638505235, 1.9812495455143262], [2.7730114624087108, -2.6151932192462359, -1.5855623082921255, 0.70315774997401359, 3.6021506318515328], [-1.2120756475010817, -3.0644255144687707, 2.5407294300854684, 0.50983059283477061, -0.77821704208377263]], [[0.13861772756247603, 4.5925563383507573, -2.7679531822434802, 0.37258443621629489, 2.5365935737052414], [2.3579091862003825, -1.3302500074539756, -3.5851156980777277, -1.4129259722598486, 0.11210034730556551], [0.86522315482925549, 4.5681946478159841, -0.74827376217124986, 1.0406717330345074, -3.4477750654668804], [1.7544835768507703, 1.6576710287098608, 2.8994643975385284, 4.524171981267779, -0.60658302818973109]]], [[[2.3778322478187022, -3.9335219849089222, -1.6211341142505544, 2.9962273486285937, -2.8469562475360091], [1.1251402146842633, 2.7666280647393027, 3.3710249820954985, -3.4176146981742415, -4.2013891909733267], [4.0721255596348289, 1.5333044882518285, -0.58038272842755312, 3.5514360946343224, 2.4654775730527785], [-0.44852840987009479, -3.0243211076298007, 0.72617086084505811, -1.745060507780829, -2.1543320227354212]], [[-4.3858533207739878, -2.6856216004698141, -3.9037465856371889, -2.0536560940722373, 1.1476300107067736], [4.4635501944804705, -4.1310605300623546, 3.1438498988370025, 2.575057516191376, 2.9366035465707299], [2.4445627974180981, -2.9717881665885493, -2.9025200149181174, -2.082424849306368, 4.8369794315394063], [-1.2825499742140387, -2.5577818297167298, 3.8391121880671477, -2.9383693162575284, -1.3066987413263842]], [[-3.4654244152958693, -1.8471647916028444, 1.4041775033655801, 3.8425269729699405, -0.93028313075565627], [-4.6892634308929546, 4.0857103144630464, -0.32016524636702748, -4.9795134678547948, -3.5289287098059119], [1.1847937653089113, -4.950698877819887, -4.3391043031997212, -2.6348815035189244, 0.89229486915135414], [-4.0688444735277969, -0.78651669066563556, -4.0996924214999595, 0.65442105595646183, -3.215048966355345]], [[4.4620205342994073, 1.1372680800155894, -1.5562817367995354, 3.7747413637519038, 3.1389112510347079], [-3.580393558691056, -1.8116197233060447, -4.594917447436238, 1.0302330473595802, 1.8857307831106294], [1.9873607699309614, -4.675237874919814, 0.83087166837823556, -3.328774591844359, -4.8527766701190043], [-3.8680587229283825, 2.9614785142823861, -1.8125287577565929, -4.9834825162912999, -4.9711849098501446]], [[-1.0791854484203744, 2.1435360188886712, 1.0199271332016302, -0.30206062394174893, 4.2939893512863119], [-2.4648772067137283, 2.122486045698853, 4.2340025240997754, -2.3012066242508133, 3.0253104770143828], [0.08725567628040487, 2.4366016997615638, -4.0948100923792037, -1.0025152017105521, -0.36484736057614153], [0.52754220188798762, -3.6865766132397191, 1.8791809511817501, -3.9677356846895817, -4.0737306073982706]]], [[[-2.4817984594988385, 4.4082845748088832, 2.4189145979449478, 2.5460061831641951, 1.9231696887536751], [-2.2167363369304227, -1.0421203587488348, 1.9568641207319537, 4.0390377168704319, -4.8248171863523206], [-2.5161466406662281, 2.0854512799849525, -1.7406548937229083, -2.4125415876812797, -0.54225704377298811], [2.0576207891460552, -0.53395188730576582, 1.1758521620687334, -2.6843713597119079, -3.3421733360875572]], [[-1.4763828743319749, -0.70036893224448526, -2.4827620830974242, 4.5383471747671642, 2.4743829511662927], [-2.1271535751155879, 3.3058498762751825, 1.3290572969875747, 0.85971781655969171, 4.8609123478472398], [-0.99986340322068301, 4.2070314979682895, 1.8863519987258206, 2.911902934088074, -2.981453416049912], [2.2446279138059495, -0.62849137074548445, -3.3091238853803242, 1.7210964465116589, 0.050582452704333924]], [[1.767227518950051, -0.45634683161437817, -4.9189546068873469, -0.63934269371543451, 2.6928887095112151], [-2.2323266216446571, -2.8472124913210139, -0.39126388558543468, -3.6029127501884362, -0.53621132800703375], [-3.4558026540741116, 2.6227149101719167, 1.8725555748839504, -2.5523543362504277, 4.9153109069258658], [1.1943355454409659, 2.5271148008175048, 1.2396632290837504, 4.4611880559224595, 2.978303729755134]], [[-2.8184573365504573, 2.7022799967259417, -4.5523065786623977, 0.5154349068962274, 1.6783775891843558], [3.9879286001128715, -0.095097599069907446, -3.1982634112773631, 0.45254227410219805, -4.0411181753483607], [1.3706394914136011, -4.1900054252490273, -0.75919645973090422, 0.71425816545608001, -2.7153824660716976], [-1.1026603069625551, -0.7024129503929375, -4.7686106014455003, -2.7790760419709413, -0.91319494873555307]], [[3.6499597052553838, 3.8563371688955375, 1.0919240391836471, 3.1773311259656953, -3.7544923654940399], [-1.5809538891238208, -4.9030209128483975, 2.1225726489695953, -0.60232991100548006, -3.0167218338819666], [-3.8018011768903834, -2.95586731967169, -1.7418708950242023, 4.2142169325387755, -1.5089306360498811], [-3.0156501164861291, -0.91090912453941897, 2.6476783098444399, -2.7215877551771603, -3.4312291456985466]]], [[[-3.6377032687694486, 4.4969694048941893, 1.0865973669749618, 2.6686676834577883, 4.02249148226751], [0.96470923835632849, -2.0251213410990232, 1.8084198824849276, -4.5425088541017526, 4.0403493559963906], [-2.1577273573741795, 0.2308908865162449, -0.82182548633759289, 1.6993023008970383, 1.8449245863966865], [4.4327475963254805, 3.6856259752803879, 1.9181895431847593, 2.1771345941500311, -3.1616034248003952]], [[-3.3482663451122683, 2.1265170721577071, -4.0081711928271773, -4.3044574017821766, 3.1584764945478589], [4.7930090966980803, 4.0155115927129117, 4.2763880844469924, -4.565059110201477, 4.6922867742103609], [-4.0805148262927116, 1.4541579609190025, -0.31175735774011137, -4.3461928219519308, -0.2535810351940917], [3.0193612146811777, -4.5337660392062284, -0.82826787153976689, -0.89579897658454755, 1.5400880747151806]], [[-0.5144495500848576, -1.9143584390871626, 4.278311372223083, -4.5109325978583232, -3.2254596498738786], [1.1847995782681719, -3.1222704594108643, -2.3526032446382694, 3.0427867587111699, -0.19037067033412924], [0.8480943670310257, 3.8808539825354575, 2.5471989724678092, -4.751933766953643, 3.5155528929098416], [-0.35144464744034654, 0.22158139175217251, -2.1925955021433174, 2.0808198174705712, -1.1815819955529108]], [[-0.2622370053989691, -2.9965725694767418, 4.2344811711812707, -4.6616666130097872, -2.3594326927706311], [-2.2750296536628833, 1.4023440304489609, -1.3548420741083635, -0.81269886903461952, -2.5832739424045204], [-1.2225115682281151, -4.697267086927976, -3.527535845391605, -0.81376311463314455, 3.9161907645304463], [1.7827831965558953, -0.46161008796368819, -3.5042301612402316, -0.10279330335538006, 0.99395155038903127]], [[-4.1639158019376579, 3.3695564838371777, 0.031113953081559842, -3.9109456751406793, -4.2925006652587943], [-2.5026522138965532, -1.6096022269102916, 0.94248869806697666, -2.0373086098549642, -0.41614847135422206], [4.1056055371259248, -4.5126069118754355, -4.5480252783144879, -3.9624858859133685, 1.2392346615921168], [4.0284724569387755, -4.2324469239755071, -4.2255650520653933, -2.1023188065650089, -3.4736051518485231]]]])
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[-2.5681536341372189, -2.3107482601864526, -2.2846184418277558, 3.2571740629499075, -4.8548803096104134], [4.5897374193578226, 4.3254457654053251, 2.1330631166062686, 4.4160591197903862, 3.2194806787765575], [3.5273075059200618, 0.43402072940376346, 4.8685807075967205, -1.9254742700584915, -3.1947072083679307], [-3.875399216885429, -1.9076591157885412, 0.25673417812806143, 2.2773937417915313, 3.1920271598734971]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.5366958355154408, -7.8280424961570869, -4.7200886090524996, -1.0408379245395081, -8.3997776009979646], [-4.5018372135526308, 2.1539959642198698, -6.88746489848024, 2.7794336730511184, 12.329603804295431], [2.5797327468250133, -11.192590075536851, -11.383117025586705, -1.0870064695748574, 7.4564667262437752], [-11.502025833992761, 9.0665633655699303, -12.518295555986544, 9.1076249714816253, 4.3829343011214643]], [[10.160527595159616, -8.9759275836063246, -9.8773390429663248, -11.24811136564333, -8.5273576737324568], [11.056483984737099, -10.874697163183717, -5.8315538339461579, 9.4810191370704491, -9.5477075229036075], [10.419518865115368, -4.9922134348131353, -5.105592041470957, -1.7917522125699279, 11.025738154656088], [-11.522807050921784, 9.4084881563711829, -10.538624391659585, -4.4618120615353076, -3.5691982159838993]], [[-3.6824094386025932, -10.971829099248028, 11.299665088560785, -8.2872302366061117, 6.0757092734454279], [-3.4663493827789424, -3.3575397753086778, -8.8201410391221042, 10.5174790854525, 8.9186366940606394], [-2.0939001565600308, 8.5039989715461406, -9.2985623458407929, 8.1907466491524499, -4.5175362090318467], [-1.9243359836529179, -10.051308879631835, -5.6990740566981497, 4.6296691292470857, 7.2661196796875274]], [[-8.5407778904910927, -7.4063675389044281, -7.794489279025183, 8.0189462550956758, 9.7220548779281071], [5.2701726813795187, 9.3838250322268895, -6.2466440317183691, 11.684781668384959, -5.6501370639524788], [3.0463216032991922, -9.3342308464575492, 2.1835094058322611, -4.9314457907695646, 7.6379215167989152], [-10.045333451913722, 7.6689059947296689, -12.043130312160926, 15.932149419148269, -3.377739648313502]], [[4.2411626307603942, -7.2114449172699784, -22.755655003420497, -19.636541779957025, 16.372705910101018], [-23.049423999584661, 16.865485305215078, 11.202417895147265, -13.909609182269776, 11.001786722265797], [13.066386014383962, -20.492963126902062, 0.383156409566209, -6.9827116539976499, -19.614208702869096], [-3.7670895457149727, -5.1251291445806242, -14.430916963238076, 18.678265101347492, -5.2391030510984606]]], [[[7.1680270216353046, -5.4408604201558957, 9.7528467735234301, 7.0307854224732189, -6.1632261036039466], [14.644442231653311, 7.3902325741560135, -6.0158510771647995, 15.279916746440449, -6.978605697388284], [-7.4413783918552143, -16.146194498168178, -21.883277955063779, 0.43719108802311585, 10.786541785840543], [-8.9193642732598342, -22.304144300772919, 4.8697914228099242, 2.3404400692759526, 8.7295822842497834]], [[12.964971000230236, 12.690075836568193, -10.826108259312569, 18.194851316767682, 4.2779291843270384], [8.4285463451869767, -4.3122860552179754, -17.680878582428377, 15.670867559387188, -16.219508082653508], [13.774466031839514, -1.2083845800549036, 13.082096328454167, -12.606994521430764, -14.479904227861462], [11.424112916828928, -4.2019515358049411, -4.5859616502041254, 15.080949131643177, -5.7413562738058532]], [[-4.7734221321360417, -1.456385512688942, -9.1524843510532961, 10.484861990719882, 0.89800710073252854], [5.060270651250927, 9.6570948988079603, -9.2970627303458144, 1.8750101632683873, -10.126919913177904], [10.326314327459293, 1.6121501906294786, 3.6806184113294882, -4.6250186215312228, 7.3517957030034848], [6.4380713301859069, 6.3548734116909973, -1.0101586807202192, 3.5181319535467752, -6.6007887742581985]], [[10.361742612432437, 10.116821432473087, 8.3641392962839998, -8.2024876786255412, -9.888749788056467], [-17.390931681288333, 11.217100323230325, 15.219527526090395, 18.324985084477966, -13.55893171777131], [14.870015586069652, -7.1287188373208998, -20.163162881671191, 4.358595998927127, -4.6901790276675559], [7.4199667641730995, -12.361706099825184, -17.200363929084787, -3.6006721132095323, -6.7764647224965424]], [[-4.1664545185736079, -13.823975232611543, 14.04068495567444, -10.184105318519045, -6.9169616968314198], [4.5386179622604681, -5.7229061857997614, -4.1499720224844046, -3.6191505719587624, -11.356939798933283], [12.64420997691756, 5.1311151207428489, 10.616263094360225, -0.38180062561624301, -14.413741748604636], [0.3835154554343353, 13.249543833168763, 8.602198403532638, 10.017683498928955, 8.2275711880070528]]], [[[-16.845473778743525, 5.5762070299309165, -4.3664900475929134, -3.7819672017923538, -17.546528644551934], [-17.37549042315473, 4.8113862748869654, -2.8836033851376572, 11.58777524618446, 10.005919254741476], [13.600557806279816, -4.3716259229364409, 7.6072700952527459, 0.0012227667782805274, -15.421365362749823], [-4.4311588040985255, -15.699136278560347, 4.9846764189548285, -15.51154203850758, 4.2681775423615331]], [[0.69356730389128796, -0.44114072726543513, -0.64047789938069355, -0.67304660711098852, 1.6158934300799834], [-0.35614975961311518, -1.9998904596795475, 1.8537084287817718, 0.66519690003020615, -1.8255167308316187], [0.012748243410691133, -0.49127716578574943, 1.636425941729919, 0.015642910265147983, 1.0557148678337021], [0.25591131271271067, -1.4160821375484263, -1.9697477912008301, 0.87357882672230791, 0.53810807721606546]], [[23.289677549175657, -13.084476884789638, -23.1764495591448, -23.559930873167836, -12.711446361744837], [-21.88729243035613, -9.879593237997307, -21.012180705676482, 4.2345785553379205, 17.82202541439025], [23.700621929005422, -13.390806007808242, -3.6596079885746637, 23.556803767956641, 0.041829811411264524], [7.0251410614293972, -12.646860318567045, -10.700132529652608, 0.051176840301859997, -1.0843134156307082]], [[8.3080657880990465, -6.8279533155889149, -9.178197762368784, 2.8824366949675126, -1.0719741137093468], [-0.95151828966815233, -8.6355509720812655, 0.80401851587175777, 7.6296179370171382, -2.6955933913392465], [-1.0446270654447847, 1.7837260052785815, -4.5254272723972555, -1.3031807177389647, 3.0459106995228349], [7.5408785864214165, 6.2601314539925772, -0.68411101979118583, 2.6356998167023504, 3.0506881813623656]], [[-13.306343872579664, 8.8076028168666856, -8.8148497217857056, -4.3639283089441356, 4.4400956911594047], [10.23603158249365, -12.016865359270598, -13.78378695961125, 14.842733591178479, 8.7149878181144089], [14.029695380970933, -11.46740546418618, 10.11447408196579, -5.6093594294312812, -15.640117644933335], [-6.6358263204082286, -2.7145117636503997, -15.33443788076521, -10.212509143694843, -0.9206038711068637]]], [[[-14.278630373565855, -11.86154247384656, -3.8548799558691891, 14.280586538323094, -8.1779167861004023], [12.730476856198209, -0.42507079465944658, -10.11305291175308, -4.3516621126151094, -2.6611815725345114], [16.645198155276692, 4.9104305949995366, 17.282366277042719, -6.1837506927031169, -0.55707652819580533], [9.6670620643452487, 17.76144450817743, -6.6287231166799678, -1.4813135800943906, -18.75091033442903]], [[-9.3577457061180329, -4.9521184720292553, 7.8732381728205416, -4.5284793511800681, 8.4071070158401238], [-8.4738646289045523, 7.4425715580162422, 3.28614249735466, 9.1514272612252405, 7.0073912794401112], [7.913557657233369, -9.2830336409446019, -5.4099079508440884, -1.214857109799238, 7.1407467509538929], [-3.1285866187866977, -8.5866797918666204, -6.862265654616678, 1.8682787762452608, 4.3224052977979426]], [[1.2270726534430383, -0.3314915721020521, -0.79093935071124077, -0.058015045811867678, -0.16746328504556587], [0.15055373006950523, 0.73492194705243108, 0.7723051620317094, 1.2662636800756597, 0.8552828857503777], [-0.48783509778186068, 0.63583401047133992, -0.28780838974868789, -0.69754074603689409, 1.004904711114128], [-0.53885426726785146, 1.1947942017596165, 0.19169559195819838, -0.36540594737891441, -0.72316829116509884]], [[-7.6318770157700628, -1.6108056875690988, -3.7981617798818883, 8.4875173481451895, -2.6938861644504324], [4.1687373888653561, -8.1286948644505888, 4.5196587038651579, 4.6990732531784536, -8.1462645790458339], [-8.0354326901672533, -5.2610536374924504, -2.0895228790187819, -1.1411008923087098, 11.382390884250817], [-5.2226958000917945, 1.0694965660057489, -9.3118842150776366, 8.4090231248941727, -11.250549376692383]], [[12.063089841152049, 8.8202776321138163, 0.68332164033316856, -2.7456054338230267, 4.8526357370534443], [2.3166516700285364, -10.929878038119574, -10.216104821498581, 5.9672598220451025, 2.1051158102432654], [-12.264397976940126, 0.19393881577574748, -14.761048052931573, 13.238825083143295, -12.46392546918219], [-5.6226994883290873, -4.0040857352540744, -13.901964105712189, 15.881896819742627, 14.045484025369056]]]])+(1.-msk_ref)*numpy.array([[[[2.7548728098250415, 12.829776260725916, 4.2146335602016594, 10.348105996383307, -11.635243021400498], [7.7344128175232854, 8.2642388788315504, 4.5323899387296294, 9.4067699675423189, -9.3060304888273926], [11.738924728808025, 4.1981300432963806, 4.5285949084329102, 8.5897349309735489, -4.9194490672188449], [-5.5747314825817451, -10.801236793528187, -3.2779789094477207, 9.6830085481588366, 9.6970810037336808]], [[0.43946019170339268, -0.28333453925647228, 1.6333981742538402, 4.5525407356753442, -4.7761319892588263], [7.6201702126757693, -10.307230251808615, -2.2166028426249782, -10.409462790305819, -2.4310759963949815], [3.4116009767098094, -5.8942362046920538, -1.2713870631061299, -2.3412904227622566, 5.676274059103001], [7.0210212353243584, -10.847495044497279, 0.40127982381729643, -2.7229210916030531, -5.9048720900157763]], [[8.2576630398859674, -10.86717173605563, 5.0817327011617373, 10.762525999488556, 7.4819156757098098], [10.815259245104548, -10.782833765906334, -1.7594838352003823, 1.9250883554463234, 10.367619905814587], [-6.6202574158650158, -9.0625576226792486, -7.8191312647304514, -5.8176859209636884, 7.8647641785223623], [9.2559851235214499, 9.5356607156778317, 9.6182378005834082, 10.173743583594725, 6.670753710447249]], [[2.5719779010473687, -9.2014857464060817, -9.6164787692913265, -15.273206323662697, 5.5552954246482651], [-12.249279508038285, 11.597042006543344, -6.5672731441006746, 4.9994242999326426, 6.4532746318805554], [9.0321810116204446, -8.5181395233313104, -5.1644524257600963, 2.2903071853775732, 11.732831608905434], [-3.9479413613737382, -9.9813673035696056, 8.2755980006478875, 1.6606069834797899, -2.5347883648208609]], [[-0.67297247610600563, -22.29631133783559, 13.438081402397357, -1.8088528430537871, -12.314858194465888], [-11.447366879933751, 6.4582045680474121, 17.40530761027275, 6.8595864816614887, -0.54423376883427876], [-4.2005548577995544, -22.178038246149498, 3.6327795541633066, -5.0523367054773747, 16.738535277300912], [-8.5178077707876536, -8.047794437095142, -14.076552612026221, -21.964313469148074, 2.9448879997021837]]], [[[10.913625644769221, -18.053833044003138, -7.4405799059732693, 13.751896778903934, -13.066781620590552], [5.1640981453606569, 12.698096354179494, 15.472119501913744, -15.685954065157606, -19.283273183095766], [18.68998705737939, 7.0374649849987145, -2.6638043262129298, 16.30015913600116, 11.315894673627847], [-2.0586276264258365, -13.880839755842192, 3.3329335728678458, -8.0093695116052341, -9.8878182984695897]], [[-18.970770674030728, -11.61651057923323, -16.885444138059874, -8.8829780557035978, 4.9640113700636821], [19.306844287389666, -17.868678276391289, 13.598552231994471, 11.138271629085141, 12.702119375188623], [10.573823800359508, -12.854308540852095, -12.554692907531772, -9.0074157462070517, 20.922092199504782], [-5.5476003548848229, -11.063546584178914, 16.605871556791016, -12.709757116203066, -5.6520545373306765]], [[-7.391969003654463, -3.9401190872617318, 2.9951992415973936, 8.196352560606913, -1.984352634215897], [-10.002494868488329, 8.7150779769189235, -0.68293267824466541, -10.621616516925236, -7.5274276720199369], [2.5272398815655022, -10.560153177701645, -9.2555833482628689, -5.6203685517842876, 1.90332127452377], [-8.6791020736893945, -1.6776897434540892, -8.7449026937318042, 1.3959214171912557, -6.8579023682156945]], [[19.70454647318487, 5.022243076399346, -6.8726321567568096, 16.669481024246593, 13.861617656344473], [-15.811229627296393, -8.0002198006977956, -20.291427098434763, 4.5495700443017153, 8.3274986222251624], [8.7763026523672654, -20.646126854729069, 3.6691784085171606, -14.700065394040802, -21.430148670385051], [-17.081575999272637, 13.078064201060014, -8.0042341505733425, -22.007353414384138, -21.95304645730808]], [[-3.4744167000062109, 6.9010727970736987, 3.283635699102613, -0.97247834259965238, 13.824415751338565], [-7.9356245425715795, 6.833302815100315, 13.631289320230403, -7.4086902646481194, 9.7399286280480961], [0.28091796389834545, 7.8445920942564733, -13.183161975674096, -3.2275783220869059, -1.1746190280775115], [1.6984119262176181, -11.868862177154794, 6.0499867642545979, -12.774048375350384, -13.115296981059423]]], [[[-8.7540663343710996, 15.549375268955002, 8.5322556175108222, 8.9805467199939528, 6.783610878298787], [-7.8191107199004231, -3.6758789634868725, 6.902461501123482, 14.246928055411304, -17.018613876112653], [-8.8752229316175342, 7.3560279531215231, -6.1398250718453022, -8.50977605057248, -1.9127073406384845], [7.2578612538920408, -1.8834124998938107, 4.1475921571173764, -9.4686032457886551, -11.788873094467533]], [[-0.64078077199678862, -0.3039746348244865, -1.0775702102419513, 1.9697367510799537, 1.0739334932894311], [-0.92322874622549056, 1.4348073746002958, 0.5768384174579414, 0.3731353538246483, 2.1097367227804193], [-0.43396144356997007, 1.8259388793728046, 0.81871587039922777, 1.2638262354058649, -1.2940125863173251], [0.97421504439010609, -0.2727782831549263, -1.4362283624201841, 0.74699153508921556, 0.021953833017766378]], [[8.6038898046942354, -2.2217613803706509, -23.948327500635749, -3.1126915041658836, 13.110546018831331], [-10.868262323193742, -13.861883805673884, -1.9048998049405779, -17.541071506721664, -2.6105881267298616], [-16.824854130886763, 12.768899213189259, 9.1166879457826866, -12.426343080419665, 23.930587853299009], [5.8147189949308924, 12.303462365142233, 6.035400481034201, 21.719654102025004, 14.500112080049202]], [[5.4268670827854919, -5.2031706041895456, 8.7653491866324487, -0.99245665111867998, -3.2316728634172782], [-7.6786539103477134, 0.18310798015344512, 6.1581739072840618, -0.87135850489753952, 7.7810690688989874], [-2.6391310742429455, 8.0677476377224906, 1.4618132491313538, -1.3752857197648629, 5.2283990717890285], [2.1231440496711982, 1.3524780629374726, 9.1818370170114587, 5.3510394133510397, 1.7583333773376906]], [[-11.660552580631862, -12.319868151367752, -3.4883775989702244, -10.150642651494401, 11.994503823806172], [5.0506847856811845, 15.663716253055487, -6.7809981419477792, 1.9242677085048212, 9.6375429883436414], [12.14564162459339, 9.4431306331343432, 5.5647675043801179, -13.463189212007816, 4.8205915799157619], [9.6341191650538267, 2.9100879463342029, -8.4585569818994522, 8.6946760196703696, 10.961772485325284]]], [[[14.097552399050686, -17.427551710084675, -4.2109985850445364, -10.342152650599765, -15.588760340307816], [-3.7386334268482542, 7.848153659393124, -7.0083489963821277, 17.604035255881058, -15.657966730171959], [8.3620549110201612, -0.89479436079103791, 3.1849018461691943, -6.5854748061479897, -7.1498192973343926], [-17.178666563450534, -14.283272018334211, -7.4337502534960347, -8.4372657012232075, 12.252475436573741]], [[6.3873508153414003, -4.0566696775816089, 7.6462243136377959, 8.2114374010332281, -6.0252964768280597], [-9.1434274953734942, -7.6602272943933496, -8.1578907119448036, 8.7085766256893749, -8.9512836387164025], [7.7842313054875865, -2.7740376899436123, 0.59472676540707281, 8.2910543557713243, 0.48374617337910392], [-5.7599119450449114, 8.648880113544271, 1.5800527554576087, 1.7088790835955581, -2.9379630548476383]], [[-0.13207678242938689, -0.49148124050156122, 1.0983887539236319, -1.1581105731022376, -0.82808573229559501], [0.30417854597315291, -0.801593540290373, -0.60399366047361669, 0.78118735771666015, -0.048874657587920797], [0.21773481029474887, 0.99634785764125477, 0.65395303472516553, -1.2199838101778264, 0.902562582626937], [-0.090227852718103671, 0.056887516499966019, -0.56291420421004879, 0.53421756567088985, -0.30335248251919128]], [[-0.59721691496176421, -6.8243756165505003, 9.6435809189823001, -10.616450370787014, -5.3733572486941759], [-5.1811382956420058, 3.1936895187831764, -3.0855088606902452, -1.8508353183004977, -5.8831319097651917], [-2.78414019475046, -10.69752666729311, -8.0335880582401398, -1.8532590245663079, 8.9187083388034303], [4.0600992948074976, -1.0512679254763417, -7.980511839005632, -0.23410082575962096, 2.2636190404999699]], [[-13.291332331211438, 10.755715813136113, 0.099316583287368698, -12.48384481583884, -13.701778707281125], [-7.9885338384753348, -5.1378940248905147, 3.0084495221036014, -6.503144415701164, -1.3283572231025147], [13.105204382232969, -14.404363824539258, -14.517420212171066, -12.648362568450867, 3.9556706972586788], [12.858993495350889, -13.510085534052857, -13.488118412005003, -6.7106587292683448, -11.087841987376988]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank2_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.9181177405873804, -2.6372249178668463, 4.23433144361295, 0.32431689785033857, 0.62534146309206839], [-1.1946795169008508, -2.7991969099190452, -2.1817611291960768, 3.1326553935318699, -2.5746658283199189], [3.4431996216943936, 3.3137317938707298, -4.3954061543753333, -4.4402548769555352, -4.2664553067719435], [3.5951572233141178, -3.901239997880408, 0.37047315082358523, -2.4306431838892983, -3.9387067791577191]])+(1.-msk_arg0)*numpy.array([[-0.27978181188516338, 2.8195028859224678, -3.0171710214499736, 3.3806875124255225, -0.17420056768566194], [-4.724105154589556, -1.7555779066048105, 4.3048731416366266, 3.02370178427031, -4.9080303064436217], [4.844429956989698, 0.20943685473251783, -0.87326925958057, -2.5587331964453544, 1.4728365629868048], [1.2978818860965182, -4.9851833160019403, -4.0733236553469983, 0.63224653655605056, 3.9256390769606337]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[1.4451130176805318, 4.6607882813351722, -2.6189378397444987, -4.5031652297173625, 1.3339709626109251], [-0.82301274653132417, -2.429849526408506, -2.5515252373240105, 1.1490832843869576, -0.46887204482171096], [1.836385556119966, -3.9185353388358566, -0.82009769145188471, 0.28341357903149955, -3.8340559327478543], [-0.89778008294981149, -2.2072889376464855, -0.71880100335686947, 3.2012699377237439, 4.2123015107214208]], [[1.4385418875162346, 1.0149495249582028, 3.8705040183170727, 2.4760638379728608, -0.62739808398374564], [-3.5442892852085173, 1.0237156633974749, -3.0519866083820602, 1.1551328540013515, 3.6740686839738022], [-0.281740669804174, 3.4046970832073349, 1.2292443514529712, -3.7959731903858049, 4.5454808071845925], [1.2869637036350028, 4.0871017921939199, 0.91623442905729213, 2.5333390742702022, -1.1379382609378785]], [[-0.40385416968715759, 4.0487013896771824, 2.4022004878894681, 3.8532173361078375, 2.8321463739112929], [2.4117802947142746, 4.3667087187830784, -4.9203607752490983, 3.2683249820203439, 1.8315530454748652], [-2.6672819825123097, 3.0093121869915347, -2.3881014242301504, 3.03365008899865, -3.4114027514410603], [4.2622332474115243, 1.7929302309573893, -0.40427886921021816, -3.631780934830855, 1.6344027359769129]], [[-3.767045978787503, 4.3804247056476786, 2.7687337153373361, 3.5797075624496095, 3.8542920606710815], [1.5453489090975623, -0.4242246381514212, -2.3726039718610883, 4.6290844692072461, -2.3518251560743231], [-1.1102008884840133, -2.5876853473746642, 3.0453177475958046, 3.9061194369890728, 1.2265761915774345], [-3.630382132230634, 2.7874155027303118, -1.2887916016977297, -0.53984508449305046, -3.3702804246607188]], [[-0.6312079638218302, -4.3850987707136788, -1.1810965926038319, -0.048248493522756952, 1.2633074453146449], [1.3865720393101206, 3.2759730742287196, -3.0977535983996187, -2.1541348643028679, 1.5268367097587756], [-1.6205241840919324, 2.7077382418162959, -0.45038073446426985, 2.1921767528234763, 4.4575102706263809], [-3.6328161601516538, -4.9064215674144327, 1.4745223820734807, 0.25101338086022729, -1.4703506812054359]]], [[[-2.4727018086132082, 2.929297852469964, -4.3610147515677173, 1.0234429545698056, -0.59771747869175851], [-1.29348990248526, 0.60707612515547904, 0.36055807928060624, 4.9253773903003779, -4.8633684581487033], [2.6659064787466411, -4.5030815805969437, -3.0594777802721418, 4.8686647189943102, 1.3804509669701126], [-0.10367154727886874, 3.3013322634002069, 4.4010441966197504, -0.6374248370420581, -4.5483148291621198]], [[-4.4091557286280727, -3.2038256746082849, -0.22263410102239689, 3.8073012389558301, -4.0138729057763776], [1.3094859692721847, 3.9563063419568429, 3.1263606973447384, -2.6016008060001372, 1.9493514904067597], [4.3651904216691371, 1.2509634292069851, -0.99236081534409237, -1.2772119998593636, -2.1687101666639288], [-3.6985526876653485, -0.38733473813265196, 1.9013883040133486, -4.2498257356408908, -1.2786770526057198]], [[-0.76088523490449145, -2.5205457122335506, 2.1621815027339828, -0.42523843886437351, -3.8781460804361325], [2.0134404204903857, 2.2767358387683245, 4.1272990341652775, 3.2673556951605676, -2.5243522452339318], [-2.2410729595835654, -2.0467900408281126, -1.4529663296434805, 3.3940562800928191, -3.1972056407446581], [1.7229913679733935, 1.8105586443665054, -4.8777243845658713, -0.47826907835779764, 2.7660730223893299]], [[0.12592182884599623, 1.6995074693181245, 0.24255141803760871, 0.48964685026047938, 2.3453890933177757], [-1.8312745732227196, 3.4426812301634033, -3.0648184645753629, 4.1141158565274534, 3.552109564449168], [-4.3112743135320368, 2.7752632271951612, -4.5556553815606806, -4.4933356418873958, -2.2134561403830921], [2.9473686278679345, -2.238270302784092, -0.28514233727624294, 1.0306777438027606, -1.2236214419844851]], [[-3.9202369953301575, -0.32563661273265865, 0.55786996452134385, -2.6339150362016603, -0.43563045052571425], [1.3971567161975198, 2.4367673666234202, -3.0346699641332786, 0.4838664542111708, -3.7514715449276723], [-4.8397574044058347, -1.913192221866733, 4.802946112739038, 1.963321006351995, -4.5791038485665982], [4.7237959439198569, -3.1095529471731274, 3.304192868161369, -4.7792590200153509, -0.61711252391108573]]], [[[-3.8474824710137678, -3.5093042711947362, -0.24316821331664595, 1.9408624014261511, 4.8704864574334898], [-4.4395482863262012, -3.918928138674481, 2.0898313174058742, 2.8144223389521636, 2.4481260557905262], [4.5038174177312325, 0.16686436418235839, 3.8129060765296625, 3.3513481845009281, -0.39066348351341684], [-3.9965968575246134, -2.0581545200138462, -2.6817259076563382, 3.0004052191943806, 1.7484807259292534]], [[0.90400600511209905, -3.7220821512206292, -3.6030359126623832, -1.7618987131901043, -4.4581875011566661], [2.252105248382124, 2.8229424183954883, -0.15789715810004967, -2.7037863312069863, -0.019919709595161095], [2.6869421497020571, -3.2109177215405946, -0.88607192501585441, -2.5797388999991466, 3.4469545914354427], [3.0675191467099108, -1.7892850657909189, -4.6518530110793037, -3.7060079999740028, 0.37984941601906907]], [[-4.4457342252361709, 3.370445429450923, 0.83550741465422362, 2.3784561865315421, 0.67867950538970856], [3.9426728224790875, 1.2562254384121871, -3.5277587632789045, -4.1366928974190564, 2.9836243945447229], [4.50326586525164, 3.384618580009791, 2.8695586824417427, 4.7799079058743619, -0.43425315736391035], [-4.9808161963452582, -4.3939574915537545, 2.2942436414149894, 0.085681479958678253, 3.9305127413535761]], [[-4.0903310336172432, -4.7694390294975211, 0.78831504629867677, -2.661701104422689, 0.21430441531501554], [1.7531981408284647, -0.10514551809623462, -1.5214352837458733, -4.8862058750750776, -1.9539270635991759], [-0.35468286739168597, 4.5177625725541919, 4.9650081443204499, -0.9750358473330154, 4.2061560773173507], [-4.9520220168601092, 2.8579318842237456, -2.8864942793382942, 2.7295172443673295, 0.84698718086489766]], [[0.048460988294748653, -0.70072652217758513, 1.6239523668530085, -1.313264605562118, -4.4623227986952116], [-0.3252319182205845, 4.1254680317190946, 4.0050443200587953, 0.16091550725817161, 4.9928079036127464], [-4.6975057266787825, -0.24021567128973587, 4.6446774309672474, -0.95861007041951218, -0.76108581343958814], [-4.1074197743426764, -0.8593888158656906, -3.2725405272983563, -1.3958144661446825, -3.1866472322489283]]], [[[-3.3468468258058781, -1.4565175117449245, -3.0458863844918591, -2.1970574777720553, 3.6923496277764372], [4.9620381294535321, 1.2579948248221466, 4.6667373988713869, -0.24577470999708773, 2.8773803937226941], [3.1050576430197161, -4.7625335089118126, 2.4360091737776806, -0.4805549867574701, -0.024368145566294963], [1.3742944882810439, -0.38452242462783115, -3.7155128585334762, 0.22844961504163042, 2.322735569552532]], [[1.708678113766501, 0.013247374366946119, 2.8260233924693434, 4.4381318647965582, 3.7594048191490081], [-0.32214284488198253, 4.0129235306725271, -0.60688468756016434, 2.8236438650161055, 4.5145881873724623], [1.6002000380528125, -3.5372708216505675, -1.8156490557485485, 0.47821205238086861, -1.0734926847242865], [4.6698636365174124, 4.7926690909115326, -3.6388342742385316, 3.5543693602516075, 0.58373677467607976]], [[-2.0484976115366447, -1.6169714853822081, 2.2054094727783999, -4.8502475421195479, -4.75183363572076], [2.4669996698875583, -3.8354579634352892, 0.44781564197316737, -3.70019569122783, 0.29875963755446122], [-3.6460326749087004, -3.2332597308026347, 3.0662832079888283, 4.0771465515639491, 3.1680421629767626], [-4.6613479753799751, 4.0466253213431465, 2.8425349493159011, -4.604800391994516, 4.9847682863346012]], [[-4.6062268532281028, 3.1184419476262519, 2.9024942678606793, -4.0570425000223587, -0.31783791501939529], [0.78244917949379733, 0.2373265595980909, -1.5059534761703408, 1.1521106223398743, -2.9429041763300177], [1.7317938677021569, -2.261133722143478, -3.335724589079839, 4.9880695606036074, -3.0548848802961204], [-2.6439179872233751, -1.5820314917524958, 2.8719808614806013, 3.9432057753652909, -3.942315759946565]], [[-2.2507883754241389, -0.46761386884973355, -0.90315424309850645, 2.6825952239349391, 0.068041994046351917], [-1.0047211756095873, -1.6613413503846686, 2.6000933569680305, 2.2901845133096765, -0.41999949971717943], [-4.5188342149536718, 3.3280907703215767, 1.7955752019576154, 3.0260071171831164, -3.3635886450313466], [-0.12143065462969194, -4.5694821654509266, 0.018755989072802848, -3.1334121950994485, -4.3532889506418924]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.1072359694083884, 22.922305531556276, -12.880244651143009, -22.147096805069207, 6.5606262566452154], [-4.0476735894452505, -11.950286062787518, -12.548701535239642, 5.6513268863559105, -2.3059679217031381], [9.0315603821120281, -19.271818167047208, -4.0333370053442694, 1.3938613509581816, -18.856338501251518], [-4.4153881531014783, -10.855706882841453, -3.5351479665614289, 15.744222473128204, 20.716594788582043]], [[-3.7937585111530199, -2.6766501775968909, -10.207389641809542, -6.5299372517310461, 1.6545898605038503], [9.3470880190803758, -2.6997684563224098, 8.0487751326210937, -3.0463451460190099, -9.6893454833299621], [0.74301351478406308, -8.9789519856229543, -3.2417938337988468, 10.010835085239956, -11.987455248392713], [-3.3940127476164328, -10.778606688232051, -2.4163162669173941, -6.6809849320711061, 3.0009991367394386]], [[-1.7100524093405314, 17.143543600109542, 10.171713059732744, 16.315799325355943, 11.992246443966986], [10.212277136994761, 18.490092033042007, -20.83443834455705, 13.839171239314473, 7.7554026510992813], [-11.294155967534159, 12.742425217025909, -10.112012951154595, 12.845479960766207, -14.445009937254614], [18.047708259527152, 7.5918608531471019, -1.7118507278851141, -15.378164208668323, 6.9206028964740769]], [[-1.2217166658999552, 1.4206457518026376, 0.89794712953184719, 1.1609596518650545, 1.2500120445260341], [0.50118276429492625, -0.1375832186369513, -0.76947555998138017, 1.5012903149404753, -0.76273663890441279], [-0.36005690814382485, -0.83923008447332681, 0.98764800486885174, 1.2668205384372071, 0.39779938542947613], [-1.1773942711363368, 0.90400594886543661, -0.4179768942381768, -0.17508088312254005, -1.093038892211686]], [[-0.39472051161170868, -2.7421840810813225, -0.73858867127193695, -0.030171783531509018, 0.78999852618816324], [0.86708098774474385, 2.0486017952884086, -1.9371537675219372, -1.3470698477407896, 0.95479430198323256], [-1.0133809642561293, 1.6932609938077474, -0.28164174743836684, 1.3708590179670523, 2.7874659943814226], [-2.2717505727337448, -3.0681888415134209, 0.92207998376783229, 0.15696907484282113, -0.91947124624342669]]], [[[2.9540862021538876, -3.4995721432475162, 5.210014996600405, -1.2226863345410348, 0.71408082868666467], [1.5453058918172191, -0.72526141192278815, -0.43075135196965331, -5.8842474811984289, 5.8101666800919283], [-3.1849038641318854, 5.3797393272726763, 3.6550954365044097, -5.8164940144403392, -1.6491964943251665], [0.12385427401948262, -3.9440340335681516, -5.257837354676977, 0.76151839637800955, 5.4337785628163768]], [[12.342095090927558, 8.9681389282828121, 0.62319668762449798, -10.657385863216112, 11.235620634657016], [-3.6655090787690452, -11.074480487098716, -8.7512992032997428, 7.2823929369984812, -5.4566186682926867], [-12.219027539544463, -3.501692965457925, 2.7778133278359278, 3.5751678833178548, 6.0706467970356872], [10.352977254485623, 1.084226202085222, -5.3223602651503796, 11.896099066900415, 3.5792688544383235]], [[1.6600698292938454, 5.4992286593130011, -4.7173635569317645, 0.92776869655431238, 8.4611883716396754], [-4.3928460453781275, -4.9672937544723563, -9.0047806013103138, -7.1285896509587525, 5.5075336050502344], [4.8894858709118338, 4.4656069507044274, 3.1700254600468392, -7.4050200622103448, 6.9755389890231312], [-3.7591555925847242, -3.9502064724087846, 10.642029461177675, 1.0434688844574755, -6.0349106007669491]], [[0.39446969629780709, 5.323971240107122, 0.75983000792431821, 1.5338948463943827, 7.3472957931127523], [-5.7367521688439256, 10.784733923882319, -9.6010200932480743, 12.888107227565715, 11.127535185487828], [-13.505736731281546, 8.6939433171435851, -14.271298402118555, -14.076072133507537, -6.9339953165173291], [9.2330902288171117, -7.0117295361987972, -0.89325268079270603, 3.2287581931169766, -3.8331843098739413]], [[10.09330023079211, 0.83840545925262322, -1.4363287342991495, 6.7814510384064368, 1.1216028347441676], [-3.5972116540014252, -6.2738616704104357, 7.8132610568827863, -1.2457944251278261, 9.658785592639811], [12.460758006482008, 4.9258306366477385, -12.36598123173119, -5.0548955050771562, 11.789662203232648], [-12.162195996766691, 8.0060597144381447, -8.5071924678334589, 12.304994883523268, 1.5888585275421312]]], [[[-13.247650188670416, -12.083235138988236, -0.83727670009997701, 6.6827766863513958, 16.77005712770266], [-15.286250979972369, -13.493651884531488, 7.1957064014970022, 9.6906179327683404, 8.4293867091581287], [15.507542428912801, 0.57454731562697192, 13.128596760263189, 11.539360801039788, -1.3451323586432109], [-13.761080787893752, -7.0866368647002815, -9.233717630730359, 10.330994115659976, 6.0203681740595441]], [[2.9956334409900283, -12.333981963898561, -11.939494658247382, -5.8384597834779743, -14.773237665619945], [7.4628727647069812, 9.3544740441034584, -0.52322883295796785, -8.9596227295536863, -0.066008575010157167], [8.9038056297590735, -10.640120141372032, -2.9362047095812782, -8.5485628128122748, 11.422283021668319], [10.164935724759843, -5.9292108108094483, -15.414993223226777, -12.280716537853127, 1.2587190867456188]], [[19.540807574320119, -14.8144765835948, -3.672394432397398, -10.454280960192825, -2.9830720748383324], [-17.329648388612949, -5.5216210232797787, 15.505932579267611, 18.182445420076451, -13.114241026126253], [-19.79368249891542, -14.876773336788137, -12.612875893145608, -21.009636626827483, 1.9087190004342516], [21.892710163228287, 19.313227800438973, -10.08413262111192, -0.37660490432636118, -17.276199893196171]], [[18.162112320381539, 21.177524911068442, -3.500319728905128, 11.81863130991078, -0.95156622525560219], [-7.7846465950829673, 0.46687289951682226, 6.7555604386248422, 21.695999466610903, 8.6759341733616484], [1.574882331708507, -20.060017295710935, -22.045901626942829, 4.3294076763268947, -18.67640503554453], [21.988239911154285, -12.689945986931209, 12.816770301136113, -12.119752256036268, -3.7608389605541817]], [[-0.20675664068154342, 2.9896183891404062, -6.9285201935048759, 5.6029847455962614, 19.038300785022617], [1.3875874434238316, -17.601124976845934, -17.087342593171677, -0.6865388198835255, -21.301591776061503], [20.041698236180288, 1.0248694255438784, -19.81630867359409, 4.089867022066354, 3.2471386076581723], [17.52412289338433, 3.6665439740306325, 13.962147899318326, 5.9551800363520284, 13.595687994838567]]], [[[-12.032440541121929, -5.2364094532332706, -10.950440436600029, -7.8987670612485017, 13.274577435301651], [17.839307223464939, 4.5226891813511187, 16.777654668862603, -0.88359872395396244, 10.344634906714564], [11.163170414109041, -17.122056745839835, 8.7578359771662839, -1.7276707318407389, -0.087607314551435236], [4.9408047565043747, -1.3824185724270055, -13.357852891673113, 0.82131228368024711, 8.3505995607254171]], [[-6.6659634009287236, -0.051681186747225848, -11.024995493647085, -17.314217546811896, -14.666340448688473], [1.2567565514845742, -15.655377786295128, 2.3676028172108676, -11.015712385970458, -17.61249201053586], [-6.2427643930613828, 13.799742412758489, 7.0832827184000324, -1.8656199862167253, 4.1879525990784092], [-18.218258803428984, -18.697352354069203, 14.195965816317484, -13.866447915454168, -2.2772972536000258]], [[-0.75891336460056946, -0.59904452098143945, 0.8170449962363957, -1.7968864892033789, -1.7604267792149624], [0.91395714078398838, -1.4209341965652829, 0.16590367186988589, -1.370823156393028, 0.11068242426371357], [-1.350757213079171, -1.197835919901469, 1.135975601381072, 1.5104733293274113, 1.1736745620599673], [-1.7269042715241592, 1.4991660330004986, 1.0530828789992219, -1.7059549101358886, 1.8467228131638631]], [[11.196093904246739, -7.5798196643522173, -7.0549279084533199, 9.861222699428545, 0.77255056172347925], [-1.9018547648763726, -0.57685618444299702, 3.6604355521078338, -2.800369831276873, 7.1531499770359073], [-4.2093729604315326, 5.4960092695902834, 8.1079562357788415, -12.124217278246846, 7.4253351118582405], [6.4264212344068099, 3.8453540621264226, -6.9807607052183389, -9.5845262405645606, 9.5823629306534777]], [[8.8651954327324454, 1.8417939152666141, 3.5572597399171459, -10.565955994248664, -0.26799746321777546], [3.9573021055367947, 6.5435364392551332, -10.241005331532932, -9.0203652680948441, 1.6542548767789051], [17.798362956327875, -13.108373678717829, -7.0722442204379501, -11.918554746228647, 13.248189398482891], [0.47827974258752731, 17.997850382301859, -0.073874341310856684, 12.341591854733666, 17.146328701525615]]]])+(1.-msk_ref)*numpy.array([[[[-0.40431633846549531, -1.3040037901650912, 0.73273117401833143, 1.2599037271885916, -0.37322081292148013], [0.2302639974291186, 0.67982770310687801, 0.71387035396923315, -0.32149260331273749, 0.13118187024251982], [-0.51378727821098746, 1.0963349170355385, 0.22944841803724797, -0.079293964654291893, 1.0726991157332548], [0.25118253828211051, 0.61755929832881107, 0.20110744710405834, -0.89565710350985317, -1.1785253488762502]], [[4.0559730033723778, 2.8616531146852906, 10.912897249619494, 6.9812691368927426, -1.7689507084143976], [-9.9931338681894957, 2.886369267313214, -8.6050850501299436, 3.2569004154806671, 10.359047257541498], [-0.7943686315945977, 9.5995532517948892, 3.4658579964255445, -10.702757365177094, 12.815996253762147], [3.6285978764763578, 11.523595298149647, 2.5833256169085597, 7.1427568309249878, -3.2084202107159427]], [[1.2184970976718323, -12.215624507438232, -7.2478496997730915, -11.62581568585323, -8.5450699678697752], [-7.276753615315986, -13.175107005425247, 14.845569946160706, -9.8610954244327882, -5.526108773055209], [8.0476459036717767, -9.0796095250871023, 7.2053104134706194, -9.1530411377458609, 10.292785524142674], [-12.859886640750666, -5.4095771363262433, 1.2197784887656342, 10.957704192826151, -4.9312725723680941]], [[-12.735205299219691, 14.808847101503352, 9.3602234966724538, 12.1018726545086, 13.030157038751559], [5.2243417593265331, -1.4341709366617454, -8.0210326196019768, 15.649488059011865, -7.9507859365486695], [-3.7532422799816239, -8.7481555399560271, 10.295267680664956, 13.205369202671571, 4.1466708139042883], [-12.273187539764846, 9.4233807820216757, -4.3570016739784023, -1.8250475357899567, -11.393864945022679]], [[0.10995678562547359, 0.76388669521602115, 0.20574769692318851, 0.0084049149616422444, -0.22006887413533446], [-0.24154163638488899, -0.57067636925358611, 0.53963043539151567, 0.37525151623303593, -0.26597582160328698], [0.28229623281715877, -0.4716895388685749, 0.07845657961836118, -0.38187843480916062, -0.77650081960778417], [0.63283863739606472, 0.85470142234876945, -0.25686263602241488, -0.043726673442548866, 0.25613592336298668]]], [[[11.681303359832574, -13.838311084181473, 20.601892267122146, -4.8348521371115831, 2.8236802220760095], [6.1105823157401584, -2.867891452075253, -1.7033142808584216, -23.268000717816872, 22.975064001808551], [-12.594022537900701, 21.273030906435306, 14.453294752135838, -23.000084094969335, -6.5213955287216461], [0.48975529088437864, -15.595840762541723, -20.790995574827814, 3.0112619583337943, 21.486717529140886]], [[7.7406163839594795, 5.624565570955558, 0.39085150901174343, -6.6840139388999775, 7.0466665933006611], [-2.2989046366632335, -6.9456040056999298, -5.4885697683360313, 4.567312896819109, -3.4222384087652666], [-7.6634318624052735, -2.196163758286374, 1.7421667227984245, 2.2422451690036449, 3.80733965442443], [6.4930973848791282, 0.67999630872624361, -3.3380352984026254, 7.4609001684116842, 2.2448171832371586]], [[-3.2755144116082207, -10.850629538861574, 9.3079170784630421, -1.8305975342587302, -16.694926901012863], [8.6676055884546166, 9.8010589629152971, 17.767498759680894, 14.065551776270196, -10.867016180537668], [-9.6475347921593961, -8.8111714733302762, -6.2548357281845686, 14.610981721374696, -13.763564691130799], [7.4172592632604113, 7.79422527949139, -20.997984695423664, -2.0588877098977862, 11.907593461889475]], [[0.38075005856021937, 5.1388037673579321, 0.73340315549761137, 1.4805460547949487, 7.0917571862730826], [-5.5372281945423873, 10.409641378318989, -9.2670970598011166, 12.439859456076835, 10.740520027948584], [-13.036007834305575, 8.3915683718897878, -13.774943305745669, -13.586506997700297, -6.6928312810804291], [8.9119637789866086, -6.7678619082075064, -0.86218539399318228, 3.1164621329441045, -3.699866337399897]], [[19.240641981521897, 1.5982343641795336, -2.7380426929253838, 12.927334822275299, 2.1380874535898942], [-6.8572875059486771, -11.959728085140561, 14.894252154020309, -2.3748312215398415, 18.412336036265891], [23.753676016658755, 9.3900054069741348, -23.573005081538781, -9.6360390004529819, 22.474380465117488], [-23.184533654214114, 15.261780104216792, -16.217078735270874, 23.456748112579387, 3.028806969841523]]], [[[-18.638859341571845, -17.000578739567679, -1.1780113771788208, 9.4023719598638102, 23.594730499503427], [-21.507080713780926, -18.984972874284534, 10.124041439096263, 13.634271890440875, 11.859775203158659], [21.818428019269167, 0.80836272459905578, 18.471356420328352, 16.235371541299333, -1.8925418826343476], [-19.361233542603124, -9.9705854128688305, -12.991433323485753, 14.535252926973499, 8.4703924079107686]], [[0.18933217436998645, -0.77954117880769236, -0.75460850903631638, -0.36900652484780594, -0.9337087680500753], [0.4716738397477479, 0.59122818119975851, -0.033069484163677505, -0.56627250507676496, -0.0041719213247956959], [0.56274471268182891, -0.67248450840436469, -0.18557611704210794, -0.54029240124694666, 0.72191932803605019], [0.64245156191870068, -0.3747422363991163, -0.97426946331844178, -0.77617465912810413, 0.079554466963017503]], [[3.8823230351639899, -2.9433063846333236, -0.72962294136917016, -2.0770326729572259, -0.59266994916417826], [-3.4430149764547489, -1.0970230584684877, 3.0806832831874362, 3.6124467436413421, -2.6055074658905966], [-3.9325636478427546, -2.9556833613277904, -2.5058973859388969, -4.1741466378262171, 0.37921993320170672], [4.3495936718893349, 3.8371080052776456, -2.0034924460358985, -0.074823002563282406, -3.432395951413834]], [[10.466065800167078, 12.203721973197421, -2.0170878782217807, 6.8105829749015969, -0.54834782161134243], [-4.4859662828840703, 0.26903932761028126, 3.892946966763823, 12.502497177220922, 4.9995780410642041], [0.90753882700553234, -11.559749068052774, -12.704131159494281, 2.4948565902952109, -10.762431184462278], [12.670903124068237, -7.3126851853429189, 7.3857687338925029, -6.9841063834327324, -2.1672142166426789]], [[0.071375115438981382, -1.0320556425177314, 2.3918164224500718, -1.9342241279483317, -6.5722721737679146], [-0.47901346060561123, 6.0761401565490898, 5.8987757109652206, 0.23700224264140371, 7.3535900324103514], [-6.9186581890924108, -0.35379842367794267, 6.8408507436081827, -1.4118759613612133, -1.1209550136043795], [-6.0495580231869051, -1.2657392698289238, -4.819917342461137, -2.0558065808837958, -4.693410556996926]]], [[[-4.3438118707530782, -1.8903876952761103, -3.9532007655399988, -2.8515211031132544, 4.7922336990262595], [6.4401394063379893, 1.6327286959398266, 6.0568739371643554, -0.31898654416584499, 3.7344998924219523], [4.0299980701608389, -6.1812059731443325, 3.161652181010997, -0.62370361258587259, -0.031626974728257415], [1.7836719225022506, -0.49906468972237578, -4.8222968366492944, 0.29650061724825483, 3.0146364219143109]], [[-8.5180736251664264, -0.066040589674931552, -14.088244666769373, -22.124900926600382, -18.741322182518928], [1.6059411356750604, -20.005159433500282, 3.0254314191619813, -14.076382286209524, -22.506049710308641], [-7.9772905319665508, 17.633943484272883, 9.0513433804323409, -2.3839747450401521, 5.3515578217376438], [-23.280126288770752, -23.892333991130357, 18.140255913629957, -17.719182833634804, -2.9100348300519769]], [[8.3441937789940415, 6.5864482014289214, -8.9833465751946076, 19.756628047604192, 19.355756354654904], [-10.048888113086226, 15.623061651549985, -1.8240980476837048, 15.072094638491357, -1.2169446989134822], [14.851471142873701, 13.17011334535924, -12.489963925094173, -16.607537494801875, -12.904461083589917], [18.987178973919089, -16.483214645753186, -11.57856485019904, 18.756842364862393, -20.304574577150252]], [[-2.9122709745449433, 1.9716241208378027, 1.8350919482287043, -2.5650510692998369, -0.20095192095720898], [0.4947007837660769, 0.15004889533865609, -0.95213386952324275, 0.72841795070382154, -1.8606409729009903], [1.0949206748836959, -1.4295939645153051, -2.1090003183505832, 3.1536897037922915, -1.9314403851446673], [-1.6716079903602232, -1.0002339313831174, 1.8157999527263726, 2.4930781944025213, -2.4925154852365501]], [[-8.8357828005337407, -1.8356832764852589, -3.5454575892303009, 10.530900638746958, 0.26710831070268187], [-3.9441727084228231, -6.5218265252406029, 10.207028085859454, 8.9904378188985365, -1.6487664483936761], [-17.739312176528863, 13.064883179646399, 7.0487801784262967, 11.879011786375038, -13.204235023756125], [-0.47669292295522936, -17.938137750168853, 0.073629243631241501, -12.300645357307392, -17.089441217940763]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.2216399020297573, -4.5966427284561382], [2.0022869794948139, 0.75779802929770401]], [[-2.8907386075595198, 3.3882009565802651], [3.5499504698713018, 0.23752961983680798]], [[-1.8733718432903448, 1.6063287988854995], [3.686880561914478, 0.71782961489578323]], [[4.7533156211926517, -4.3211368233384704], [1.2951739471735015, 3.9746352898273951]], [[-2.9544955929525019, 0.59012653399220838], [3.3577738040702361, -3.4802173645672854]], [[4.7204343849796864, 4.5855936947290754], [4.5147532438350471, -0.071518069239516535]]])+(1.-msk_arg0)*numpy.array([[[2.36952161427444, -3.3908949035762959], [-4.4419734396870902, -2.288400919540885]], [[1.3757120050879177, 3.8437294893218379], [4.447535179777617, 0.72529782075058513]], [[2.9635712888632622, 1.5336985357613395], [-2.6055870083194965, -1.1514126444467387]], [[3.2908175377913196, -2.5670849195354051], [-4.5360164845446853, 1.2708964227367918]], [[1.8855133676905886, 4.9696017648303208], [-3.7233123599118176, 4.7368784299652855]], [[4.6267471514817622, 3.0923328426190615], [4.6570887958026681, -0.68893944222672587]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-3.5420367478311618, -0.50715954590227152], [-1.9413918363753533, -0.31967730072797629]], [[-0.10457439976517602, 2.0296147764437507], [2.0454833094324743, -1.9715885477979835]], [[3.831916148021806, 4.8568267863582211], [-2.8805452340897708, 2.7964288845523946]], [[2.3818482335104276, 4.1523401810365232], [-2.0078713298926845, 3.1148067107322994]], [[0.90868168003935157, 4.4549772593657622], [-2.8216790389651294, -0.32762946832913276]], [[-2.4405464742912542, -4.6164829559594143], [1.4995280280431871, 2.4678567170020589]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[11.411166921268585, 2.3312312388387935], [-3.8872235960718964, -0.24225082850286991]], [[0.30229725476355751, 6.8767427270361567], [7.261364435433717, -0.46831067823305933]], [[-7.1786038175536486, 7.8016607381257215], [-10.620226231280967, 2.0073594692816901]], [[11.321676415655238, -17.94283005930485], [-2.6005426357536159, 12.380220673467788]], [[-2.6846960190729394, 2.6290002890836246], [-9.4745599605311899, 1.1402217648229953]], [[-11.520439495385379, -21.169315134671734], [6.7699990288295506, -0.17649634755975921]]])+(1.-msk_ref)*numpy.array([[[-8.3929326325402815, 1.7197247195000811], [8.6236109732046646, 0.73154982894224896]], [[-0.14386425718181578, 7.8012901681801941], [9.0973589783488737, -1.4299888771346883]], [[11.356156677608931, 7.4489081306840559], [7.5055112388209499, -3.2198435769697165]], [[7.8382279391933896, -10.659409859519773], [9.1077374512378775, 3.9585967061862326]], [[1.7133314546897398, 22.139462850423037], [10.505992441482965, -1.5519409615492636]], [[-11.291791447985918, -14.275701862104423], [6.9834351783919955, -1.7002038301068774]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.51495685629892662, 1.6894241017513441], [-1.3077624806734125, 4.8827648323030441]], [[2.051368328343588, 0.66375535814226616], [-2.6945064678412436, 2.138739457706281]]], [[[3.1174953569349206, 4.3227239825259449], [0.93441792616615782, 2.5608588412905684]], [[-2.6246000349549172, 3.6924150661327655], [3.1393845770845417, 2.3709484171614514]]], [[[4.1234880675468677, 4.7760521302913599], [-1.1538828215997166, -0.32786346318010473]], [[-4.0577374255123129, 2.4961379990936319], [-2.0279726832094669, -1.6509651535801293]]], [[[-3.0992715276336877, 2.311310270620397], [-4.5411984022035998, 0.68035966101169354]], [[2.1220460283861673, 3.7386644892099312], [-0.71504537890859954, -1.1949160652432789]]], [[[-2.128024497648755, -4.5550773373992595], [3.9326525808938211, -4.5468075885189219]], [[3.3963245996753493, -4.6885381026788044], [-2.4144941511681104, 2.2571577464728518]]], [[[3.293721013288085, 3.0902753179326865], [-0.22236468151770694, 1.2694303389060533]], [[-0.97629362063204184, -0.67813823333310186], [-4.4177439419091211, 3.2815205257409197]]]])+(1.-msk_arg0)*numpy.array([[[[-0.84460588941928183, -4.5231993982680336], [0.87480702079251405, 3.1328294682334903]], [[2.5515919596389312, -1.98312592652377], [2.8635913995121083, -1.9934364631210864]]], [[[1.2519716758080515, -4.8839728646801293], [4.7992828081348957, -2.8089543064403055]], [[3.4905451610708678, 3.2034583049967811], [-1.9165163399229526, -0.6285987582750181]]], [[[-1.6875222544145716, 2.0099339598044965], [1.2560387336290972, -3.3247835452955785]], [[-1.711717787931768, -0.68472157468378114], [-2.2410029054979663, -3.6452877227445746]]], [[[-0.38369945070970957, -4.9963410426551143], [0.39807005287312336, 4.9049143890948663]], [[2.501288855985794, -1.0378233288217342], [-2.5286179643645221, 4.534173695560785]]], [[[0.37820430302926056, 1.6970828242863822], [-4.1906107651193816, -0.52418600601584942]], [[-3.0922739138361646, -3.0156763026817082], [4.8331762625475214, -0.3695683199294173]]], [[[0.049111709446046703, -1.2719558804697306], [0.30653216274667106, 2.9593186780984109]], [[-3.6917320518831098, -3.7668235076423304], [3.327185124123627, -4.1461739272461271]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[-0.48339531062426033, 2.9348892385045815], [3.785719825405554, 1.3625488229193916]], [[-0.58749676668081463, -4.0610397369408808], [-3.9172543392554324, 2.5341321154347272]], [[4.9293503550750906, -3.9136106031078235], [4.4979310068664109, -3.553398047043701]], [[3.2081621766748256, -0.71458093785161481], [1.384786509261434, -4.4396832555785926]], [[3.9456829706242456, -2.6551837970355443], [1.442520932691127, -3.9020977877585441]], [[1.9378159136427398, 1.5332806047645189], [-1.4676768350286871, 4.8695202540440139]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.24892772950871223, -0.81665968844220294], [-3.8381380310484543, 14.330373960474832]], [[7.7659057498193711, 2.5127918185383407], [-3.6713966161057741, 2.9141369306289513]]], [[[-1.8315184423417179, -2.539586362987607], [-3.7947083290706569, -10.399749515177378]], [[10.281225875737109, -14.464128940140711], [7.9556152794904058, 6.0082965279679668]]], [[[20.326117369910051, 23.542834264308858], [4.5158480453566234, 1.2831299258733093]], [[-18.251422983934116, 11.227456503540727], [7.2061941719744933, 5.8665363524688354]]], [[[-9.9429656901996033, 7.4150581887644131], [3.2450538132169031, -0.4861720446421427]], [[2.9385807121409706, 5.1772521473127027], [3.1745749957193596, 5.3050488466824426]]], [[[-8.3965100214439072, -17.97289108005269], [-10.44191541215929, 12.072609837273697]], [[4.8992693292455032, -6.7633143568341163], [9.4215922858290266, -8.8076502491337756]]], [[[6.3826249946491407, 5.9883846886273373], [-0.34094745335573934, 1.9463929177443016]], [[1.4328835311879329, 0.99528777601027218], [-21.512293602306706, 15.979430664156569]]]])+(1.-msk_ref)*numpy.array([[[[0.40827852627091343, 2.1864933781412437], [2.5674617110922031, 9.1945074923885013]], [[9.659612267950509, -7.5075591363167939], [3.9017830907273163, -2.7161545063902315]]], [[[-0.73552931151319134, 2.8693182665564119], [-19.49007819265303, 11.407275057705293]], [[-13.67335317857191, -12.548760945872493], [-4.8567056067541721, -1.5929523010671143]]], [[[-8.3183884239955859, 9.9076686784397783], [-4.9156465058449577, 13.011908135907197]], [[-7.6991885133430831, -3.079830401840574], [7.9631753478157332, 12.953158274912951]]], [[[-1.2309700649777968, -16.0290723548142], [-0.28445327171271839, -3.5049583242412901]], [[3.4637510635350934, -1.4371637447491308], [11.226262836144395, -20.130295034066123]]], [[[1.4922742778793652, 6.696150799525677], [11.126841803227707, 1.3918101898060597]], [[-4.4606698503233861, -4.3501761928389469], [-18.859526401933792, 1.442091723622221]]], [[[0.09516945211074776, -2.4648163466257067], [0.46999981987599171, 4.5374659324456683]], [[5.4182696136817636, 5.5284796038081527], [16.201795350873947, -20.189877915514227]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank3_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.4867250869746877, 4.5498373125278349], [4.7206228642174857, 2.987044803018418]], [[-3.5532851952749143, -3.9508747789554999], [-4.1165640872557532, -4.90664544094337]], [[-0.39546251144391675, 4.0249676012168027], [-1.0482767198202469, -4.6097611699944157]], [[3.6252874505378045, 4.059724753871004], [-2.0127073820053756, -1.1280934965456701]], [[4.8820117223656201, 0.55218630941159841], [-0.7170479516657231, 4.2725200951280922]], [[-0.75277686368033514, 0.9336872064202506], [0.16281490371411422, -3.0851418993842614]]])+(1.-msk_arg0)*numpy.array([[[-2.1650426321272942, 0.038523958959699733], [-1.6968203300072195, -4.7741175956803836]], [[-3.1757963275502621, -4.312913998522772], [-3.0660652158476998, 4.6551503898351552]], [[-3.113291991065573, -3.326928469308903], [-4.1876927407515474, -4.6067484947617565]], [[-3.5117372780449152, 2.2592240027086339], [-2.2629733446894171, -2.6215417553068532]], [[-0.52709670467129133, -1.0624404133087006], [-1.6752589087406635, -1.5361586628830559]], [[-0.31500973485659411, -2.093395274964597], [-3.5745770575190914, -1.3620777436513731]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[4.0970314883195407, 4.7360504472449847], [-3.4351526712250235, 3.1609575592724255]], [[-0.68105563385253198, -0.71391927843108416], [3.9158849869897896, 0.46930163512039069]]], [[[4.1428594736094819, 3.6103341799465785], [1.5595385154184189, 0.76608783303442429]], [[-1.1695239544412139, -1.2749217742122609], [4.3594436059435093, 3.7173747824695074]]], [[[-4.145374814224116, 0.14306833507307193], [0.34077464213169861, -0.48859763048392235]], [[-0.22138291924214926, 0.56352748169143219], [-0.76139611481646696, -4.7554989199488045]]], [[[2.3807330337718282, -2.5785010513044404], [3.6590036795220762, 3.2729349399164818]], [[0.018080225591141641, 4.6804331358677285], [2.5632426227496783, -4.1666163071730402]]], [[[-1.0768125319391242, 3.6325218647103537], [-3.6229266420250039, 3.2850377371130062]], [[-2.5163250615001709, 3.0202654337761246], [0.3078416940315174, -4.8989976140277793]]], [[[4.0911748408012265, -4.5266185719313921], [-1.9344773783635705, 1.9857431460896882]], [[1.1724207567622091, 2.3809412880999439], [-4.1475039583928694, -4.6618588292821581]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-18.382253960768526, -21.249356354831761], [-15.629385797769274, 14.381842646494597]], [[-3.2150067969683946, -3.3701436689674251], [11.696923899605697, 1.401825010234409]]], [[[-14.720761233680996, -12.828546991599175], [-6.1615413873763343, -3.026717098000379]], [[4.8144203100380345, 5.2482971897825816], [-21.390244094152447, -18.239840028681861]]], [[[1.6393403349094289, -0.056578163096096826], [1.3716068938963373, -1.9665896327290868]], [[0.23207056040739085, -0.5907327400360588], [3.5098542450655592, 21.92171426513038]]], [[[8.6308415904138034, -9.3478075024925236], [14.854547812260858, 13.287214993388249]], [[-0.036390203515613284, -9.4203423235435455], [-2.8915773327925787, 4.700332758723043]]], [[[-5.2570114037170077, 17.734014325265367], [-2.0005304917287421, 1.8139528643342595]], [[1.8043257310738223, -2.1656751427759566], [1.3152598238679318, -20.931065751918265]]], [[[-3.0797417654262418, 3.4075337316556706], [-1.8061967792874523, 1.8540629707406404]], [[0.19088757262466802, 0.38765272657095146], [12.795638239899921, 14.382496003232847]]]])+(1.-msk_ref)*numpy.array([[[[-8.8702478373797433, -10.253751126190931], [-0.13233568052657571, 0.12177259928676355]], [[1.1556290453869293, 1.2113927456259483], [-18.694895419048606, -2.2405011939098323]]], [[[-13.156877901845805, -11.46568602990353], [-6.7261554943835211, -3.3040709391921448]], [[3.5858367158128561, 3.9089933048390479], [20.293865601672302, 17.304938667776302]]], [[[12.905762209088877, -0.44541350175808064], [-1.1337328585265012, 1.6255293668938329]], [[0.92708364383673447, -2.359879944293211], [3.5075604058482086, 21.907387491315315]]], [[[-8.3605089437694939, 9.0550182633438077], [8.266508938775484, 7.3942931755630559]], [[-0.04091506857872499, -10.591695428069771], [-6.7196475645205336, 10.92295862759657]]], [[[0.56758433713386203, -1.9146903045352417], [3.8491436789401483, -3.4901568511530208]], [[4.2154959765655589, -5.0597265746949374], [-0.47289368508311058, 7.5256376242321945]]], [[[-1.2887599018527629, 1.4259289161410424], [4.0496258033921997, -4.1569453193174866]], [[-4.1909083388813642, -8.510858103742013], [5.6492228334328978, 6.3498141554098737]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.65481945816537745, 3.5863749534189182, -2.3371297451642503, -0.48494562095695048], [-0.32644747567060062, -2.8334281670748851, -0.78336934168368177, 1.4854340404191593], [-1.7989782474142491, 3.805623542163481, 4.8141013087697324, -4.5389395216637904]], [[-1.6587641955422292, 1.9240359658309352, -3.0190344684263284, 2.4917628959031859], [3.6147320216599805, -3.8595305340551, -1.1427069704107771, -4.2654144287733864], [2.3968697413489135, 1.1452113383845752, -1.6324568626896605, 0.40617237034892195]]], [[[-2.730568740831596, -2.4862611556734637, -2.317046615710443, -3.8448538059168582], [1.3394367694219254, -0.43841587674708649, -3.5534468652372855, -2.055741760877563], [2.0913517414650551, -0.45396538002954401, 1.3760302540756424, 4.3141268790275866]], [[0.17382597117908993, 1.0818993977228706, 2.2351231454569751, 2.174135030488701], [-2.5834377572764722, -0.22177330117660254, -1.8739905439087035, 1.0050598681793543], [0.57238831835712212, -4.5324816939755639, 0.48618416436271428, 2.0804073037475659]]], [[[1.058238615552904, -4.0006047263580484, -1.9732482947363641, 3.1880242780323105], [-2.4759076078800613, -1.815358764184396, 0.21916326152880838, -4.8628429634253836], [4.1544688218606325, -2.7840893997461946, -0.54900288672126951, -0.70591514405265787]], [[-3.2401769446312469, 1.0742459590443021, 1.5027152194840019, -4.6312791548486842], [-2.0203430369650888, 1.7478842532237149, 2.272743191101128, 0.01906789098995354], [-1.6368807803203844, 3.6276804168533516, -1.1981690901860507, 3.062837973403596]]]])+(1.-msk_arg0)*numpy.array([[[[-3.5421119383182376, 3.6250844098770933, 1.8618077312143475, 4.2979862543060694], [0.44736259009715251, 1.9655770174219898, -1.2198693569544181, -1.1416057780212743], [-3.5812587703790189, -3.13301663598752, -0.60198873245302309, -0.18203026841749192]], [[-2.4007880666231163, -0.92849659406757912, -2.8941503440454475, 0.98583409044758152], [-4.6688078985371355, 1.9934411833267074, 2.2792513138134307, 4.3445098431433511], [-3.6657439697977257, 0.84105191840115889, -3.0143592988742807, -3.6289639365001136]]], [[[1.6999364219680224, -3.0021068037715102, 2.9487520592833274, 3.3678360565949443], [4.2762086022970784, 1.76499874401049, -0.41930146761234965, 3.5589840992932142], [-0.58537579863977474, 0.32706635518915128, -3.4690802551731981, -4.7543973846080103]], [[-4.7871079516687756, -0.073381117061986245, 1.1668334284860844, -0.76040543294201157], [-1.1665201623423971, -4.4824956004877992, 2.2940939473159183, 2.3815500524985218], [3.676733442996559, 4.9308971013347236, 0.63355095655369098, 0.62229453281729885]]], [[[0.57350774695789219, -3.883731138374904, -2.4268008522208184, 0.42968498561144841], [4.2872276267336744, 0.51202850062217475, -0.98571585106275261, -4.5176379646625628], [-1.409325289740988, 3.7923974893243688, 4.4487848567589907, -0.61968040099943877]], [[-2.4712131542627711, -1.9443505999814481, 1.675043005471812, -4.9985667271243415], [-2.0345152572135872, 3.93566334892407, 1.5328253023067067, 3.0518698510957734], [-1.3158281079429432, -0.50538706679592149, -3.9537350001430207, 3.1374038788516128]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s1=numpy.array([[[[1.4048921365198517, 3.0176000703602384, -1.5772144469237057, -2.2657776478522229], [0.96913450524513411, 1.1955336124039402, 3.6732994734645619, 4.4888023486227127], [-0.4953412866778697, 2.1748374978657425, 4.8256121407619563, 0.46823404572588956]], [[-4.3699363066805628, -2.5422412372350212, -0.054766332935520801, -4.9942866483889805], [3.4004358053624255, 2.5363167428989133, 0.7809340312816353, 0.031946228384978959], [2.7768740245161077, -3.7499452284379817, 2.4387671893761702, -0.69859717342081673]]], [[[4.7040971883027645, -3.4492422115341235, -0.87182466423757621, 3.5435977978739039], [4.6199676053195144, 1.3581459695207414, -2.9307631306431836, 4.5220838420106713], [-2.05920273081315, -3.9691809387402222, -4.1531856136031706, 3.502340243877379]], [[-1.608093909004392, 3.6505112954499452, 4.2839247682509782, 0.36933076572362822], [3.3838505329128363, -0.38011225621847355, 4.112598667848598, -4.3262118186329008], [4.9015164971080587, -4.092982954397467, -4.7417835325857096, 3.0560901485696892]]], [[[0.01727908492196395, 2.1685258509483631, 1.0672607066375006, -0.16220781677279827], [-4.9294734598919394, -4.130783122186017, -4.0139143832789319, 3.0655015557606262], [-2.2108604249831498, 3.1641163183594525, 2.5150624124238252, 0.7974849327067135]], [[3.342058409651905, -2.2156231952553096, 1.3026006894503519, 2.4900070924636157], [-3.6311538406040644, 0.39896818742368545, -3.3183831456907429, -1.5254015604436844], [0.55710845618377025, -0.055450760529578424, 4.0648475806969167, 4.6849891370114811]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.91995070761672881, 10.822245311775124, 3.6861547984081744, 1.098778948388075], [-0.31637151282255049, -3.3874586120701125, -2.8775501903349485, 6.6678198093576473], [0.89110819977967315, 8.2766127822577893, 23.230985722457245, -2.1252860155337707]], [[7.2486938823217635, -4.8913635742587154, 0.16534144684164936, -12.444578161960342], [12.291664193242704, -9.7889919132535343, -0.89237876097651248, -0.13626390349817916], [6.6558053249004399, -4.2944797939283124, -3.9811822347995038, -0.28375086984739001]]], [[[-12.844860736213331, 8.575716927046523, 2.020058387764569, -13.624615479794077], [6.1881544841031193, -0.59543275597795764, 10.414311059337034, -9.2962366002109924], [-4.3065172171156787, 1.8018707332612272, -5.7149090551096737, 15.109540185611433]], [[-0.27952848547986753, 3.9494859719278317, 9.5750994029141694, 0.80297495559695575], [-8.7419672317071324, 0.084298749879257451, -7.7069710144398034, -4.3481018801511482], [2.8055707851793734, 18.55137031456054, -2.3053800643790625, 6.3579122659953651]]], [[[0.018285394905840188, -8.6754147685336296, -2.1059703694115752, -0.51712245795829748], [12.204920842189301, 7.4988533438053686, -0.87970256773680622, -14.907052669800127], [-9.1849507050780446, -8.809182701508508, -1.3807765247048402, -0.56295669115148383]], [[-10.828860606565074, -2.3801242642678409, 1.9574378809473978, -11.531917942752123], [7.3361763780134615, 0.69735021233506755, -7.5418326998333782, -0.029086190670445197], [-0.91192012448117454, -0.20115763807277642, -4.8703747275085938, 14.349362633822107]]]])+(1.-msk_ref)*numpy.array([[[[-4.9762852088163818, 10.93905497030692, -2.9364700510655166, -9.7382811857827924], [0.43355452241898562, 2.3499133920966737, -4.4809454665962178, -5.124442697583155], [1.7739453272459487, -6.8138020613828436, -2.9049641359272091, -0.085232769025691865]], [[10.491290936981789, 2.3604623300708654, 0.15850200130744505, -4.9235380354490506], [-15.875981546544578, 5.05599824925575, 1.7799449168002861, 0.13879070366984661], [-10.179309210257863, -3.1538986282770365, -7.3513205550855529, 2.5351839484850598]]], [[[7.9966661428732362, 10.354993511102483, -2.5707947740045483, 11.934256433750177], [19.755945216201141, 2.3971259303870176, 1.2288732819028514, 16.094024489386747], [1.2054074431109527, -1.2981855427200184, 14.407734208220143, -16.651517295497992]], [[7.6981191388250494, -0.26787859670751551, 4.9986266247147437, -0.28084112080888018], [-3.9473298729958888, 1.7038515161907988, 9.4346877116509766, -10.303089983784911], [18.021569626316545, -20.182077785650904, -3.0041614928402161, 1.9017881912517243]]], [[[0.0099096890630896318, -8.4219713716990938, -2.5900291924096792, -0.069698263416084288], [-21.133774802499154, -2.1150786884482922, 3.956579032406816, -13.848826209036355], [3.1158215090162615, 11.999586781676653, 11.188971574194849, -0.49418578289070669]], [[-8.2589387042463045, 4.3079482890274745, 2.1819121737865719, -12.446466602692253], [7.3876378899986834, 1.5702044726300679, -5.0865016484628933, -4.6553270331325276], [-0.73305896581930452, 0.0280240972156467, -16.07133015004808, 14.69870309083749]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=4.06842253835
      res=mult(arg0,arg1)
      s0=numpy.array(-3.66450462612)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-14.9087532128)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=2.24213775529
      res=mult(arg0,arg1)
      s0=numpy.array([4.908226748356693, -4.9887944585440183])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([11.004920504003364, -11.185564408871439])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=-3.3890939414
      res=mult(arg0,arg1)
      s0=numpy.array([[0.023646020576874704, -1.7517908127582915, 3.3870145835424861, -2.1853666605735267, -4.5441123059422459], [0.66131891220476646, 2.941661607540941, -0.4100105337630886, 0.95296251401349608, 2.9250863882464584], [-4.179814930514838, -0.4048393460476154, 1.3687883429065373, -2.3896814375660034, -3.2910962895981744], [4.6791274309276059, -4.4907967538617557, -4.3525531472879351, -4.8472844009380847, -2.4583879106727902]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.080138585075209864, 5.9369836301122012, -11.478910604503543, 7.4064129090784245, 15.400423485091615], [-2.2412719186837298, -9.9695675317540537, 1.3895642158850003, -3.2296794826205861, -9.9133925564658139], [14.165785457164143, 1.372038574928669, -4.6389522799979384, 8.0988548819212909, 11.153834495627841], [-15.858002427176315, 15.219732070553446, 14.751211501077385, 16.427902195442329, 8.3317075736621842]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-2.47230167907
      res=mult(arg0,arg1)
      s0=numpy.array([[[2.4349049623127996, 3.1547307839320187], [-4.4147967333651783, -3.4729098066627664]], [[-3.8042907286730987, 3.5409687615399204], [-4.0113806831823204, 0.32106869625905698]], [[-2.4014324453179148, -0.5668882866579148], [-0.75733122642342288, -4.0091323097960263]], [[-0.92677992643989704, -1.3421794192130876], [3.7806849350025509, -0.87050547086092411]], [[0.93555424244296059, 4.2940932820016275], [-0.0090366868537916645, -4.4439147468224407]], [[-0.31439318774925695, -2.7040006672252992], [-0.35280427067390097, 3.412515166511632]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-6.0198196266905493, -7.7994462141143579], [10.914709376631064, 8.5860807462549662]], [[9.4053543561513422, -8.7543430146731875], [9.9173431984020635, -0.79377867685659753]], [[5.9370654667215508, 1.4015188629468567], [1.8723512626952683, 9.9117845411039625]], [[2.2912795682614426, 3.318272431727507], [-9.3469937128239753, 2.1521521372450581]], [[-2.3129723244484666, -10.616294031155972], [0.022341416081817136, 10.986697890192502]], [[0.77727480595920373, 6.6851053897750026], [0.87223859076852051, -8.436766976002767]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=-0.633767174071
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-1.9581866462635125, 0.11244019462166222, -3.8406939339552624, 1.5213134303767184], [4.3046835254533313, -4.5345922421651874, -4.8721910113152305, -1.5382772244216825], [3.1417015659243681, -2.25045166605753, 3.4977717457697324, 0.42516693080954404]], [[-1.9871261342152637, 2.7279725197714146, 1.8469589934321275, 1.9359615951653684], [-4.4820266235517661, -2.9088413342860897, -3.5790541364116999, -4.9394526875495544], [3.1491155462719949, 4.4827065590653472, -3.861033262546921, 4.9052827452284458]]], [[[2.774421655310789, 3.4240073298043896, -3.1109716476449965, -2.2936616186573655], [1.9717556507568448, -1.0044237346970144, -1.5834389001525553, 0.96242936491677078], [-2.712029082688229, 2.4416273024325585, 4.6334967455521401, 4.0774138737715244]], [[2.4894081303399886, -2.5316788606929066, 0.35733793079998044, 3.8924348213103492], [4.1236933993745541, 3.3510194328740202, 2.1749557972640075, 3.8377022536377652], [-3.2035336644296528, 3.577813840297372, 1.221398635623089, -3.4938752151132588]]], [[[-0.78746434164629786, 3.7625766723991418, -4.2146458366874349, -4.6214314233263432], [-2.6279427154204105, -2.462459987704122, 2.4924306584358655, 0.53881943956290179], [-0.31633322859976865, -0.16981113047383367, -2.0215488626515943, 1.5227956180966418]], [[3.9891177164148512, 3.6379795017010359, -3.1776242170763624, -0.12484383459319925], [-2.0416034279273862, 2.6372985223154561, -4.1695534336553264, -0.12236813595020823], [-0.69915302213719066, -4.1977662310993491, -2.9554207130503443, 2.858612143693863]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[1.2410344171052856, -0.071260904397323369, 2.4341057409930671, -0.9641585136455606], [-2.7281671131949876, 2.8738757108796675, 3.0878347287736156, 0.97490960945895377], [-1.9911073232091838, 1.4262623927798392, -2.2167729148606039, -0.26945684424745103]], [[1.2593753146035183, -1.7288994347978861, -1.1705419818918288, -1.2269489092772394], [2.8405613473177644, 1.8435281522503582, 2.2682870258794696, 3.1304629712438974], [-1.9958060605827146, -2.8409922681267572, 2.4469961397970965, -3.1088071834608919]]], [[[-1.7583373721666995, -2.1700234494072776, 1.971631709741845, 1.453647442330763], [-1.2496340067379768, 0.63657079190840271, 1.0035315970632035, -0.60995613884590028], [1.7187950077327026, -1.5474232355963968, -2.9365581384940764, -2.5841310682965908]], [[-1.5777051558740443, 1.604494957195715, -0.22646905059135267, -2.4668974169560074], [-2.6134615124553524, -2.1237661162283588, -1.3784155893605607, -2.4322097122125239], [2.0302944775457354, -2.2675009669160811, -0.7740823617125776, 2.2143034216377711]]], [[[0.49906905048656935, -2.3845975848905074, 2.671104181625974, 2.9289115333227813], [1.6655038283715116, 1.5606263076692586, -1.5796207349639173, -0.341486073546105], [0.20048161635431638, 0.1076207202861419, 1.2811913099284125, -0.96509787556825866]], [[-2.5281718621673557, -2.3056319881199721, 2.0138739203149085, 0.079121924250274006], [1.2939012350904664, -1.6714332316685352, 2.6425260967842603, 0.077552907717455072], [0.4431002350828333, 2.6604064416929858, 1.8730486334997456, -1.811694540072867]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(4.9941851322)
      res=mult(arg0,arg1)
      s0=numpy.array(-3.56419721137)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-17.8002607213)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(-2.3308373397)
      res=mult(arg0,arg1)
      s0=numpy.array([3.5686513293518836, -0.23992567572445278])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-8.3179457708237319, 0.55922772373132978])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([-2.0972533328632248, 1.2571872975302947])
      res=mult(arg0,arg1)
      s0=numpy.array(-2.64046759321)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([5.5377294601705884, -3.3195623177201519])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-3.7026004169)
      res=mult(arg0,arg1)
      s0=numpy.array([[4.1542599258511483, -4.0583883484667531, -4.4375583087411705, 0.3447070952178235, 2.0868875627714871], [4.4051418188648146, 0.20783848164391028, 4.7252059023350377, -4.7259599112288075, 1.6022085462954951], [2.5334046754611839, 0.96027058142233912, 2.595769684705342, -4.2199464190871758, 0.27099193988086157], [-4.8144259335967767, 1.4825147264844416, 3.455177823191578, -0.47491537637263725, 0.95123133496832413]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-15.381564533384067, 15.026590390991361, 16.430505243980893, -1.2763126344632822, -7.7269107599494928], [-16.310479935050132, -0.76954284878343782, -17.495549343942979, 17.498341137587403, -5.9323380314808611], [-9.3801852075491379, -3.5554982551150047, -9.6110979167767798, 15.624775370624745, -1.0033748695805034], [17.825895468888884, -5.4891596443476214, -12.793142848626612, 1.7584218705514496, -3.5220295374258708]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-3.1709331494361246, -2.0294789782320724, -4.4739014429892512, 4.7910719406231692, -1.5720596603448134], [-0.20052927636160689, 1.8825415703491011, -3.2585344873329225, -4.9417077550900066, -3.0649891572752175], [2.1056265429798762, -3.0366529621491178, -4.8997308131721384, 0.73195305683919987, -2.7108025448069686], [-3.7191927831904339, -0.68617631260101231, -4.225459691282615, -1.9579432814884354, -1.373936690956763]])
      res=mult(arg0,arg1)
      s0=numpy.array(0.330071801078)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-1.0466356157337831, -0.6698737815959378, -1.4767087071349942, 1.5813977445379739, -0.51889256349280855], [-0.066189059417636337, 0.62137388673020488, -1.0755503471102621, -1.631118379125964, -1.01166649142779], [0.69500794543998423, -1.0023135124668074, -1.6172629743033671, 0.2415970637758009, -0.89475947833251557], [-1.2276006605056859, -0.22648745135759421, -1.3947050906860967, -0.64626186533036289, -0.45349775815188159]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(-0.481531114076)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-4.1136167450005114, 0.069339415585833564], [3.5596957344248796, -0.35636609441858447]], [[2.0528257744353997, -4.4648738797226297], [2.3135040588400244, 3.2850741330273774]], [[-4.0157415610001621, -0.61790136066894341], [0.57703670008811692, 0.34144625665387984]], [[3.6190128197873044, -4.7873320661355017], [0.68130367850562301, 1.0173567380114212]], [[-2.1999987153360969, -3.2007032971648486], [-3.8613951752946276, -0.19900958826190873]], [[0.87747318403817776, -1.1532702913937785], [-3.2974891252862504, -0.40409955348106408]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1.9808344541007645, -0.033389086036407993], [-1.7141042527683141, 0.17160136246420557]], [[-0.98849948216729622, 2.1499756935105627], [-1.1140241868720109, -1.5818654070981077]], [[1.9337045077087069, 0.29753873059183927], [-0.27786112505602645, -0.16441699636353788]], [[-1.742667274966609, 2.3052493432567993], [-0.32806891933472054, -0.48988892346711249]], [[1.0593678323610136, 1.5412382245097218], [1.8593819206463553, 0.095829308747513581]], [[-0.42253063988150102, 0.55533552824531318], [1.5878436121517647, 0.19458650818525067]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[0.8409251860165341, -1.1533691397859269], [0.039994626130839706, -1.2504466521500825]], [[-0.65037952880075078, -2.9617571847693203], [-1.5176871376744328, 3.4059501806008328]], [[4.5932805881650456, -0.62961045799713133], [-3.4676029951870859, 3.445360586444469]], [[1.0994162189588437, 2.4453340754013153], [-2.2312018814746573, -4.2823019993533906]], [[4.3133209563152928, 1.3368254758877303], [0.079678004275356429, -1.4925697201110308]], [[-0.96947768950136926, -0.8066071426847099], [1.2452932583310066, -4.4872621435341484]]])
      res=mult(arg0,arg1)
      s0=numpy.array(0.693913594011)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.58352941812302028, -0.80033852501011604], [0.027752814759573353, -0.8677019305123651]], [[-0.45130719630124649, -2.055203572670893], [-1.0531437362877856, 2.3634351308428077]], [[3.1873398412341207, -0.43689525573564003], [-2.4062168569932423, 2.3907825472031936]], [[0.76289985981160902, 1.6968505568190551], [-1.5482613165379679, -2.9715475710113863]], [[2.9930720469192895, 0.92764137053859053], [0.055289650310328668, -1.0357144187940928]], [[-0.67273374783528139, -0.55971566133521222], [0.86412592048601655, -3.1137722012888491]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(-4.58630751735)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[0.71914725737593876, -3.1690490799713178, 2.210854187190165, 2.3213404675126785], [3.0826656709940856, 2.1672574153620801, 1.5563916497829098, -1.8886091814781478], [0.56788685652052528, -4.9827847414965518, 3.4660567992808602, -0.86446988347015186]], [[1.4181767181597014, -0.0055974474588484924, 1.110651909528233, 3.5454038115782556], [4.5680453816112792, -3.0290381587557191, 2.4327126897129245, -0.25968732042266485], [-1.9496757548063037, -2.4040464367748715, -3.0796048177219548, 1.7632020213402297]]], [[[-4.3709062122465214, 0.37938377709987225, 1.5510809591625314, -2.2472961754565093], [-0.0020759310896067973, 3.2228611695431084, -4.583792789186619, 3.0159193115174041], [3.1129110761600423, -2.8129886336022336, -1.4866996740844387, -2.3846813774270514]], [[-4.6472935371290731, -1.8358639121877776, 4.9405620165585304, -1.4875680518611523], [3.9860693220593877, -2.4178476726619555, -4.7000708443538777, 1.683090922624225], [3.2796585364418931, 4.8811312253345367, 1.7330963474915855, 0.20378971150597192]]], [[[1.6559593083323678, 4.1266159537801741, -2.9230135973773042, -3.3021152205335524], [-3.8448501742384344, 3.1647126780237773, -1.1283931123783972, 1.3782210042611904], [1.3618615473504656, -0.48469256411890704, 4.768407572123504, 2.8257417228326966]], [[-4.5129566699423238, -4.0356092263193446, -3.0566118603182701, -2.8437176030197362], [3.2511071891326999, -4.2370736605071402, 3.8331429347076522, 4.3198667194041427], [2.4486290313518904, 1.2651033306284498, 1.788679519264937, 3.8468306517266591]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-3.298230472582524, 14.53423361831307, -10.139657178467663, -10.646381236474481], [-14.138052740346758, -9.9397089761004693, -7.1380907233349777, 8.6617424863432113], [-2.6045037590624669, 22.852583117246027, -15.896402354092421, 3.9647247250789759]], [[-6.5041945434218986, 0.025671615358469974, -5.0937912018247928, -16.260312153070966], [-20.950460873264646, 13.892100477831336, -11.157168496375075, 1.1910059098140868], [8.9418125706567348, 11.025696245031121, 14.12401472597529, -8.086586685073577]]], [[[20.046320018843574, -1.7399706688725256, -7.1137342630204339, 10.306791343200658], [0.009520858361757362, -14.781032409240307, 21.022683327006149, -13.831933410123328], [-14.276767469424581, 12.901230916500722, 6.8184618912903376, 10.936882127770348]], [[21.313917284651762, 8.4198364612923111, -22.658936716459916, 6.8224445388155752], [-18.281339696425999, 11.088992956928729, 21.5559702455222, -7.7191725508094615], [-15.041522600013701, -22.386368831907454, -7.9485128067867521, -0.93464228583775255]]], [[[-7.5947386242249664, -18.925929770024798, 13.405839234958124, 15.144515859077959], [17.63366525718147, -14.514345545462829, 5.1751578138232732, -6.320945352408204], [-6.2459158521988378, 2.2229491504205865, -21.869383493802911, -12.959720505507788]], [[20.69780710081637, 18.508544931742076, 14.018561952588737, 13.042163419940531], [-14.910577341219174, 19.43252278073556, -17.579972256514065, -19.81223720913901], [-11.230165733682522, -5.8021529154815958, -8.2034343253288462, -17.64274833597365]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[-0.28535586384525224, 4.744418088711015, 2.0766835548575093, -3.0234215999667082], [4.5727049081833098, -1.5620938680303578, -2.9770593112517894, -2.1881181363043445], [-1.3899275042293979, 0.65806164204059048, -1.5333199725951219, 2.2120249342664087]], [[-2.2708493454987022, 4.7567849941000269, -1.2794827070490444, 1.4379487519896328], [-1.4560693623439702, 3.313081071415553, -4.5258180332030999, 0.20252525328693771], [1.9106577224584171, 3.964652551590083, -0.14427505970724397, -4.085087774085677]]], [[[1.6453383822154057, -2.2796940149128475, -1.9112369884867295, 0.608956959966652], [-1.3616615764719722, 0.36038425125468976, 2.8629905382743184, 0.88232864194881433], [-3.0977625110340212, -1.8386107710095234, 0.67675621171695255, -3.6193083677568549]], [[-3.3779635476313965, -4.1996414991640574, 3.3992964691784398, -1.1776543649137872], [2.0100904033443703, -3.7764130477195614, 1.3746755155982484, -2.5609426820127879], [-3.7889313719187454, -0.21475949999966026, 4.7126726847835201, 4.3005541715406856]]], [[[4.5620929558140677, 3.2899710926921006, -4.6097385562856497, 4.4335117017098185], [2.2987637328563153, -1.9373924168175414, -4.0226976646627888, 0.23179284313151616], [-3.5779692410957944, 3.6256980080705041, 1.4054063069906251, -1.4223800693269939]], [[3.1317203805693214, 1.9904743382959689, 1.0410845649041551, -2.661168262091993], [1.1472226680368456, -0.55732903106100196, -1.7748302380956158, 1.1066832612143056], [0.58894119949330559, -0.54349941504749211, -1.8318237752264976, 1.7703286025901823]]]])
      res=mult(arg0,arg1)
      s0=numpy.array(-2.30130950683)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.65669216229624794, -10.918374451918261, -4.7790916074673353, 6.9578288711530645], [-10.523209277122149, 3.5948614690562732, 6.8511348953751092, 5.0355370691403563], [3.1986533792850844, -1.5144035129069764, 3.5286438299426846, -5.0905540105682752]], [[5.2259271873707229, -10.946834528860039, 2.9444857175542265, -3.3091651332854646], [3.3508662661634308, -7.6244249665411177, 10.415308065984728, -0.4660732907620157], [-4.3970147809882416, -9.1238926082448764, 0.33202156650248443, 9.4010513307309544]]], [[[-3.7864328609416176, 5.2462815091782478, 4.3983478514061796, -1.4013984412204461], [3.1336047310176021, -0.82935570352357446, -6.5886273436898293, -2.0305112918636072], [7.1289103165385406, 4.2312124466809138, -1.5574255038292502, 8.3291487548616487]], [[7.7737396258831923, 9.664674907296412, -7.8228332810478225, 2.7101471857338058], [-4.6258401548004988, 8.6906952484270246, -3.1635538328501847, 5.8935217405580964], [8.7195037869161638, 0.49422807903088556, -10.845318452061816, -9.8969061995961845]]], [[[-10.498787890248805, -7.5712417528022398, 10.608435163572594, -10.202882627778806], [-5.2901668323740827, 4.4585395872790397, 9.2574723787840103, -0.5334270735132921], [8.2340146296725756, -8.3438532948606614, -3.2342748952338147, 3.2733367758651428]], [[-7.2070578845317579, -4.5806975178180522, -2.3958578066260161, 6.1241718208217399], [-2.6401144324019867, 1.2825865976120241, 4.0844336999355706, -2.5468207100800999], [-1.3553359813567383, 1.2507603708043507, 4.2155934687626333, -4.0740740433506408]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.97697411928279365, 2.7185681544914857])
      res=mult(arg0,arg1)
      s0=numpy.array([0.93374009750809694, -1.7506407080373187])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.91223990940200284, -4.7592360788266816])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=numpy.array([4.9033058160567027, 2.9778419833091387])
      res=mult(arg0,arg1)
      s0=numpy.array([[-3.4289651827170262, 3.0326805366566028], [-1.3829340772320151, -1.8311128639939289]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-16.813264923472328, 14.870160113630282], [-4.1181591553303774, -5.4527647625785587]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_array_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[2.9652366807366786, -3.8550491620523264], [-2.5695661213984389, 0.71781212289640273]])
      res=mult(arg0,arg1)
      s0=numpy.array([3.9426645774192579, 1.0607536679078509])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[11.69093362480476, -15.199165775433499], [-2.725676688205144, 0.76142184223108023]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_array_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=numpy.array([-2.7155542798290466, -1.4431519854060717])
      res=mult(arg0,arg1)
      s0=numpy.array([[[-3.4198557032210521, -3.4780081886976557, 3.7220419045785054, -2.4243053533493275, -4.740915474608153], [-4.5681713143916323, -3.1806646270836101, 1.5255147284863373, -2.8711014901643486, -1.8369035685258819], [0.61377070938786105, -1.5331467256401075, -1.3016555306720532, -3.1811055151571477, -2.9977383398637514], [0.45142558159868607, -4.4075510873506927, -4.4661611143582096, -4.1805320492506581, -2.5595878823170635]], [[-1.9745837763073437, 1.0700884574894056, 0.43197400387436247, -3.075769718242273, -3.4873770490170064], [3.3418703850668159, 2.7958800823083552, 0.84173148919077878, 0.37493394755613974, 0.65294393848711429], [4.1338722479771679, -4.79207487158879, 2.3554640326527378, 1.3667380783721095, -0.32022938771387732], [-0.49111228909155002, 3.2478512955111221, 3.1987163610093834, 3.8092208111621311, 0.49828522854295088]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[9.2868037912797021, 9.4447200220983891, -10.107406823681217, 6.5833327779002353, 12.874213307379925], [12.405117163788479, 8.6372674407777552, -4.1426180498833194, 7.79663193943935, 4.9882113471437073], [-1.6667276767119161, 4.163343152417883, 3.5347162471796429, 8.6384646962727771, 8.1405211786246312], [-1.2258706701346285, 11.968944218820342, 12.1281029285015, 11.35246169830512, 6.9506998284246677]], [[2.8496244971285618, -1.5443002819859564, -0.62340414133509625, 4.4388031755332102, 5.0328151121484606], [-4.822826881178929, -4.0348798917405944, -1.2147464698044819, -0.54108667081177897, -0.94229734118653896], [-5.9658059420833105, 6.9156923651479083, -3.3992925952753907, -1.972410771332789, 0.46213967666465278], [0.70874967505979103, -4.6871430454205578, -4.6162338671415766, -5.4972845764787568, -0.71910131687027778]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_array_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[-1.5849800614643774, 1.4933983067443659, -1.9937472981160642, -1.0566351356524462, -4.3461084805123926], [-2.4639641548093172, -2.9138962025643664, 3.0645036285941387, 0.78705440490203138, 2.2440101162650397], [-1.6408270574074955, -4.2341296241280082, -0.087926408507309262, 0.48849077388619833, 2.748161534201202], [4.4374341981843273, -3.2440908197136151, 0.40607730023930699, -2.2519328454595087, 1.5321087260091062]], [[0.17030609676953379, 4.6907977548858781, -4.6826660427970506, 2.4058653186639791, 0.84547477956806638], [1.0361646216974361, 0.091390952794968783, 2.3635101065986337, 0.97365315082031945, 4.0075727463377735], [0.013443102999541878, 4.2710854677814787, 3.3505665085728147, 2.1221305452311494, -4.09827841839678], [-2.2200464504556896, 4.5770699785841256, -4.8041124222598262, 4.1464644260591061, 2.7397633722293993]]])
      res=mult(arg0,arg1)
      s0=numpy.array([-0.25351658553973699, -1.1121581520795463])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.40181873333101142, -0.37860123957665642, 0.50544800744746066, 0.26787453175192499, 1.1018105823647963], [0.62465577941956207, 0.738721015891324, -0.77690249629532038, -0.19953134536477263, -0.56889378259214107], [0.41597687305516229, 1.0734220850415825, 0.022290802863545126, -0.12384051306329272, -0.6967045286623339], [-1.1249631664809512, 0.82243082779460219, -0.10294733062186373, 0.5709023258456789, -0.38841497289346505]], [[-0.18940731387108509, -5.216908962852763, 5.2078652129628091, -2.6757027269575997, -0.94030166847428243], [-1.1523789309172225, -0.10164119317724153, -2.6285970325760677, -1.0828562889827542, -4.45705469989137], [-0.014950856590185501, -4.7501225212216536, -3.7263598565939589, -2.3601447856558351, 4.5579337525116488], [2.4690427578695555, -5.0904256893208899, 5.3429327939228815, -4.6115242137494716, -3.0470501691938749]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_array_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=numpy.array([-4.1623769452184733, -3.4194708663555531])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[1.3531068876536478, -0.72356133065180206], [-1.8917249807295211, 3.61065762161709]], [[1.4337078236780627, 0.34420215006067867], [-3.4290704708285356, 3.8351448417321166]], [[3.5912358952351529, -4.1323976499158785], [4.1098042890817954, -2.0074054333172797]], [[0.46004672032888827, -1.4912302710433134], [-0.95676824049905296, 3.1019835057538341]], [[-4.7948747848199416, 0.30440006869723568], [4.9762420124750903, 4.2780333162410109]], [[1.4999760446908752, -0.85473430657554061], [-1.8800531849377355, 3.9873307353481771]]], [[[0.60896236991986008, -1.8578678532007729], [-2.8676828844995184, 2.9606356847828206]], [[-2.7103134559506992, -1.8585414316118598], [-4.3213662922532183, 1.700825509709051]], [[-2.660939519841655, 2.6657321110914776], [-0.28161445373083183, -2.8180250144797245]], [[-2.7638273492325305, 0.78261337000121323], [-1.0185871906011776, 4.0634847640952767]], [[0.4523128114512156, 1.2326789842806596], [-4.3963018453374207, -2.8143770781816735]], [[-3.7883972285253242, -4.3418454823894344], [4.9520979927885271, 3.6522414384670672]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-5.6321409135858671, 3.0117350011566617], [7.874072446482419, -15.028918041296341]], [[-5.9676323914569203, -1.4326990939071982], [14.273083871306152, -15.963318470799313]], [[-14.948077495167825, 17.200596706484852], [-17.106554622234064, 8.3555780953461447]], [[-1.9148878624203354, 6.207062500202583], [3.9824300661705019, -12.911624628797734]], [[19.958076259543912, -1.2670278280682934], [-20.712995026553894, -17.806787246398113]], [[-6.2434657068012935, 3.5577263719773287], [7.8254900327693928, -16.596773525774275]]], [[[-2.0823290826477949, 6.3529249975585786], [9.8059580774925603, -10.123807470007478]], [[9.2678379013148504, 6.355228279311496], [14.776786139210797, -5.8159232792044335]], [[9.0990051652326738, -9.1153932913857929], [0.96297242007721329, 9.6361544376746036]], [[9.4508271003373334, -2.6761236183394876], [3.4830292231036779, -13.894967766703466]], [[-1.5466704812368042, -4.2151098743164699], [15.033026079836468, 9.6236804257810977]], [[12.954313953124467, 14.846814133248143], [-16.933554813678182, -12.488733195734634]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_array_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[[-0.45694751196403516, 2.3608679682775637], [1.9832307456536871, 1.2378813294827653]], [[3.399856093157263, -1.9093615346763384], [3.6045298898424392, 2.8502059201911596]], [[-1.1818579817758943, 1.7976798024559217], [1.7295442682820239, 2.0578192942162667]], [[-2.8123369892620387, -3.8976591663203108], [4.867838804663613, -2.9540770913638212]], [[4.7095696446510189, 4.4746555083549318], [-0.73925820239855611, -1.6167832292993287]], [[2.7616610449957557, 1.1092059973100445], [1.9180616619651358, -2.1476413925332838]]], [[[-4.8640969240901946, 3.9283603790426938], [2.5858975670322053, 1.378863083256662]], [[-4.8318739823796157, -0.86280723500535395], [2.9913430415371671, -2.3893297988780549]], [[-0.03584596756720515, -2.9548596057250442], [-3.4266239166908274, 3.4738045263617323]], [[4.9223946094603761, 3.1409119023860903], [-4.973440808637692, -0.49569784200403344]], [[3.1376491142286849, -3.4997484515519863], [-0.97002680571835143, -0.71342983628380541]], [[1.8858763706986732, -4.0268498238892807], [-1.1504598503980992, -4.8162190318768907]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([-1.4718747843468529, -3.9226886928638729])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.67256952062989528, -3.4749020316799317], [-2.9190673260690692, -1.8220063148794408]], [[-5.0041624539261802, 2.8103410970919116], [-5.3054166542836256, -4.1951462241254864]], [[1.7395469620550013, -2.6459595715645028], [-2.5456725968959395, -3.0288523298993608]], [[4.1394078995807408, 5.7368662448852419], [-7.1648491908494973, 4.3480315817951025]], [[-6.9318968050872032, -6.5861326113863727], [1.0880955072320169, 2.3797024669605578]], [[-4.0648192550422326, -1.6326123380869577], [-2.8231465950689008, 3.1610592114893019]]], [[[19.080338005122549, -15.409734840365212], [-10.143671147101431, -5.4088506256983244]], [[18.953937436023651, 3.3845241848766441], [-11.734107525514872, 9.3725969855816569]], [[0.14061257166064076, 11.590994364377833], [13.441578892700026, -13.626653736778509]], [[-19.309021676344297, -12.320819604771472], [19.50926002467083, 1.9444683199062445]], [[-12.308020702559208, 13.728423678770824], [3.8051131825662381, 2.7985631519422074]], [[-7.3977059154788432, 15.796078272031359], [4.5128958467504861, 18.892527938699267]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[4.7168067847627135, -3.7721257780228754, -4.4364414123186515, 1.1923217369903618, -4.4814363228370002], [-4.7876307377387182, -1.5659011724114427, -4.4879240261995275, 3.5770354423956654, -2.3883055341727264], [-3.8932193584585715, -0.7194200059611946, -4.3240808804935371, 3.5155843511740095, -2.4186602703976035], [-4.6575621470442776, 0.512512252467058, -3.257302663754774, 1.8942680565559336, -1.7111345271650444]])
      res=mult(arg0,arg1)
      s0=numpy.array([[2.9888914807309845, -1.208191969293261, -3.9252949735745259, -1.2877344357558074, -1.2291736910018667], [2.4680076386080065, 3.3546696390855839, 2.2436445859042733, 0.40545701463757045, -3.6799088235647992], [1.3165138429232002, 2.7381794074737451, 2.4085146071594243, -3.3339100679230924, 4.7588388980748668], [-4.4590625791046179, 3.6638568274920136, -1.7437805134796647, -3.9064812221590097, -0.8787747993853916]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[14.098023615231382, 4.5574520721713325, 17.414341176332275, -1.5353937592226676, 5.5084636259313884], [-11.815909231573642, -5.2530811208971873, -10.069306443332279, 1.4503341117265276, 8.7887466085708574], [-5.1254771789472899, -1.9699010456475821, -10.41461196320747, -11.720642063011903, -11.51001457599639], [20.768361079739297, 1.877771515374741, 5.6800209115609794, -7.3999225926713956, 1.5037019008308787]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=numpy.array([[0.45854670739451375, -1.0875788033874647, -3.0599866564686562, 1.5485979954171993, -4.2237242804217354], [0.56662745336512632, -2.6617701739782564, 4.7199171589816658, -3.2823194588302562, -1.7009083637349143], [2.2566178729898381, 1.9989317961997299, 1.9104226505393251, -3.859863713836591, 2.3975422343654982], [0.78771350869871704, 3.1542447758089018, 3.3740971534689681, -4.5898078624432328, 0.81346017655242697]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[4.7322835002186672, 1.3290214139132885], [4.089657023053805, 2.9203033256096838], [3.9554694584337273, 4.044950995696535], [1.4575245201209395, -3.8405979544285165], [-4.608721932723495, 2.9487671062305552]], [[-0.90751476259748109, -2.4314905666933937], [3.2289417176188611, 1.5115343999166031], [3.0064086583155181, -2.0479792414636067], [1.3109784137737135, -4.8556840184361194], [0.14416470999951603, -1.2125583187567877]], [[-4.1614503009682382, -1.5180913983330435], [-0.09287199686868064, -3.567228784748008], [0.93872047761136379, -4.7645347265581979], [-1.5149057763616902, -3.4782314945935155], [2.473833300569348, -2.9573747052878927]], [[2.9703569145386997, -4.8677566725300769], [1.7571166942775527, -2.3015112331958809], [3.6498353109866901, 0.901957771676269], [4.7187015738700655, -3.4039322060427391], [-3.8512886462785034, -2.344678253847098]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[2.1699730174826546, 0.60941839340673964], [-4.447824291397998, -3.1760599963950136], [-12.103683762876507, -12.377496072901002], [2.2571195501307022, -5.9475422934313968], [19.465970728956414, -12.454779223894935]], [[-0.51422277882186795, -1.3777493076868055], [-8.5947007574720065, -4.0233571826401358], [14.189999813294461, -9.6662923630223325], [-4.3030499576358832, 15.937906139643967], [-0.24521096099359524, 2.0624505858897662]], [[-9.390803126723867, -3.4257521823104815], [-0.18564478751736749, -7.1306470421517156], [1.7933528629538429, -9.1022750608979717], [5.8473298360599379, 13.425499534305123], [5.9311198188948095, -7.0904307587719408]], [[2.3397902672387745, -3.8343976880102586], [5.5423761534115776, -7.259529783773611], [12.314898933430717, 3.0432931499621128], [-21.657933584272083, 15.623394802518701], [-3.1328699421560691, -1.9073023863330965]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_array_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[-4.1118837680698466, 2.0951123156752409], [-1.6511211747203802, -1.7513611677132959], [2.820353445560305, 4.1481615785623696], [1.3151337847615938, 0.71619564202185515], [4.4439303178068439, 4.6599015783105493]], [[-3.2541466522947049, 0.87318275102265552], [-0.70520398920165128, -2.8477612977485611], [1.8912481600639355, -2.7042578008657312], [-0.30439194747749898, 2.1931152166453538], [2.7968530172735484, 1.3632693411926242]], [[-4.5992439486704892, 2.9161178883667516], [-3.5656453677370967, 2.6457175709530931], [-1.2175655101877849, -4.9275972748882682], [-2.740657925921349, 2.4027658464853099], [1.8787041828939133, -1.6932500719561738]], [[-4.5311230576811932, 3.51803834625394], [2.7640114865730423, 0.011579960277293466], [-0.40706163090127134, 1.5845716936070833], [-1.8316422024214116, 2.7684103751224427], [-4.230963001970264, 2.69964489668825]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[-2.9537358806794423, -3.749268742660897, 0.77051952691424486, -0.069051454766273501, 1.246896460885857], [-0.73027195980900483, -1.527006015337582, -1.6692063798880841, 4.4033053841339278, -0.86995680056321856], [1.8308839133986332, -1.6835788151572184, -4.2159433082848219, 4.3622460129067449, -1.2212425343162323], [-2.8384771241171212, 4.7612468686837381, 1.0851770108391134, 0.19022930910831715, -2.0409606490337762]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[12.145418622931292, -6.188408420863353], [6.1904970107246635, 6.5663236832175489], [2.1731374026040866, 3.1962394970777241], [-0.09081190105006326, -0.049454350978874337], [5.5411209856967156, 5.8104147860718429]], [[2.3764120532771664, -0.63766087886073297], [1.0768507335509807, 4.3485486319076117], [-3.1568834947303217, 4.5139643740671982], [-1.340330701214683, 9.656956041480532], [-2.4331413025528805, -1.1859854343698621]], [[-8.4206817594168086, 5.339073331384677], [6.0030450034856457, -4.4542740533458423], [5.1331871650745873, 20.774470756987718], [-11.955424110091673, 10.481455733779043], [-2.294353457547869, 2.0678690091069005]], [[12.86148914578769, -9.9858713676086364], [13.160141035451781, 0.055135049609745586], [-0.44173392384873611, 1.7195407739288062], [-0.34843203070026152, 0.52663279298783938], [8.6352289945391245, -5.5098690005055726]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=numpy.array([[-3.7761046661110242, -0.69558885517264102, 3.9679100531264861, 4.2415579278370821, -3.8451041284883782], [-1.8349545565320327, 4.2926740499587446, 4.9752386448316539, -2.1578748270408288, -1.0785696134103215], [-3.9024350835977661, 3.8628968712677878, -4.7044359532507842, 2.9920153863751651, -0.38004081416393909], [-0.26915015956694965, 1.24481306672463, 4.2932743363770438, 3.0341940963224303, 0.18503219010491456]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-2.6970369203574132, -1.3145546394633589, 2.5762947143794346, 1.5919366381059685, 3.5470900097628224], [4.9441263917628753, -2.2682048098487941, -0.06602981867696478, 4.1799812473047311, 1.4409487993708954], [0.40241688389671992, -4.376102600760893, -3.4034715838183072, 4.7629762403511187, -2.7733664498238584], [-2.8426532183247213, 1.7676496027181319, 1.1914735687439926, 2.9901194167267162, -2.2985633255480065]], [[-4.1800322200969724, 0.65538648393501653, -3.0136236194428747, 1.7462207973059556, -2.6357936531669388], [-4.9006854338081949, -3.4857421614665682, 2.4479661796341414, 3.9141590077308361, -3.2631703861983485], [-3.8795463462494686, 3.2410411427584194, -3.6201060489272185, 4.0107599074133962, -2.3937367018587539], [0.21505205652797255, -1.8245871711214945, 2.6128767927320462, 4.4957459796398034, 1.1681985532737693]], [[-1.999898601007466, -1.9114823237809886, 1.4487352591789637, 4.876882195175039, -1.6225135351504907], [1.7430117551078101, -2.7401084789166275, -1.9221230340524684, -4.5446581989531083, -3.4249690025506241], [-3.8886447136493354, 2.1214987267628969, 4.4355529047047177, -3.3373168253936392, -2.5596289218027755], [1.7107608080314174, 1.0017957463266409, 3.0921130967816381, -1.5784397405110671, -4.6419123773213435]], [[0.9895772172937658, -1.7835952220130391, -1.7009587831842721, -4.1760061311679326, 3.7083550714212432], [1.010318829026442, 3.6730911358832401, 3.8329976670599955, -0.23854608703933877, 2.0736679331809587], [-3.9472800623218673, 0.73158535738678498, -0.34898437287310013, 3.6395752043855918, 3.502146568703882], [-2.1328599624670028, -4.1622619795835281, 3.7692022891223829, 3.801849330651244, -2.7389278611180625]], [[4.0262855332404612, 2.2877475956899582, 4.4877890110306158, -4.2552807271579018, -2.4067257055833871], [-0.19575752722855722, 2.7414725161571845, 2.7421065323929472, -0.48693367560077139, 2.4458481172508897], [-1.364297019630405, -3.552522853787401, -3.0601901352858905, 3.4056877837061137, -0.58240088918478961], [-3.6161396109370356, 2.4761570576885603, -4.9330005011273439, 4.8736188770007285, 2.0609881065854632]]], [[[-3.2089935272763626, -4.2505799222940102, 1.669490365846686, -1.6064705530387702, 1.6582778333064132], [-3.648571214859937, 4.6900546385086912, 0.95361527228790699, -1.0595184756804876, -2.0349938028294812], [-1.4217578984511556, 0.59552917111561676, -4.1443159239231395, -0.029761615872838654, 0.87486726533960191], [-3.6631375207993444, -2.120937707483125, 3.6937968556495999, -0.84256520422146952, -3.3831346732829379]], [[1.3774397117015047, 0.11919605100142583, -2.7864800534026548, -0.30644614627399491, -1.7056794314099033], [-3.4829962575028084, -0.20304956174479827, 3.299514443354651, 2.9979844813533996, -1.928007730225386], [-4.1123154927069896, -3.6838934774509822, 2.8640093951529053, -2.9173834257140694, -0.32647167123161758], [-0.71714002780927366, -0.49029201282448387, -4.0179718436425045, 0.61003562452465321, -1.7139931563629887]], [[-2.7302448013684586, -3.3522007998911163, -3.6190060992095461, 1.0917106825227885, 2.2400041359547966], [4.3771488115546138, -4.6233103025020554, 0.093119786655908143, -1.791163326690798, 0.27097200663831256], [-4.0122030545927672, -2.1237510713053673, 3.1330926739131577, -0.3210768044386807, -4.024686211598695], [1.0416048680528274, -3.3685698387851835, 4.340011872827569, 0.15765729388455796, -4.4399819525320394]], [[1.4067148209538418, 2.8586486507977202, 2.3258521610738985, -2.0519156839088026, -3.0739305208947441], [-1.9649684037334469, -4.4696064205602104, 0.5721570333064907, 1.8383291855004744, -4.3228127628667004], [1.1162372528496523, 1.6481075885069076, -1.6844919391583235, 3.1507720903877239, 1.7138959115890575], [3.7338288769691133, 0.95976205290358241, 0.3853399186892954, -0.81170516443588703, 1.0825153850812512]], [[0.1760343443472836, 0.72175721752293942, 2.8059953952272823, 0.56131986870032691, 2.8094300355018742], [2.5694593472018603, -4.3767584793863765, 4.3305505416124355, -2.1141773817568432, -0.19409431404609823], [-4.9263973978128961, -0.56255465231931012, 4.0928263862152754, -3.827702832901223, 2.0772301251058423], [-4.4369964008364455, -2.2688079793748219, -3.0731610462475922, -2.6053289182594028, -2.6582207564241633]]], [[[-4.0410456368706225, 4.0656510514774062, 2.0871577522962159, 3.882170545519152, 1.6352966362136439], [-1.0855866100547407, -2.0698798000610341, -0.018215231969831436, -4.8837233207511677, -1.7734774565326141], [-1.0754225085887468, -4.910103815447159, -2.6194590879336168, 2.133360860286416, 3.5108522373190016], [0.12297593267312656, 3.3560676663075011, 0.85706189817523892, 0.94851068598839028, -1.7213436375936753]], [[0.57587238977761146, -3.1395503267046507, -4.8415656672799852, -3.2282876982644426, 1.4006548710849831], [-4.6049678877694724, 4.9607966287546716, -3.8425153567639816, -0.35470713418570998, 1.5836328871866208], [1.5860365350857899, 2.9389534888448079, -3.7400698975176825, -1.8115561450306306, -4.9168558764042487], [2.7488744213817853, 1.0987998729593871, -4.0332297798730412, 3.8373072833949315, -3.7072947465371122]], [[-3.4926847937562155, -4.4466671874194166, 4.833268175659768, -3.0301620844660402, -1.9381659628536605], [0.39759438989029405, -2.9329756087413292, -4.905482451607015, -1.8738477095631079, 1.4619977826237172], [-1.4161418850614282, 2.427360785037779, 4.5995351521702066, 4.5036428245993108, -3.9262936376473787], [-0.98820757843357665, 0.53341040015789254, -0.23406758008403017, -0.013432882313752614, 0.068144941949872972]], [[-2.2920222676659705, 2.7778853770256386, 3.649751549383085, -4.6750006679237552, 2.5750664508900032], [4.1272981917202909, 3.5016195947742386, 2.4933845883507058, 4.1885216242151397, 4.0507311851749463], [0.20210805022280098, -1.5952719992903686, -4.9560144046022252, -3.6927706273583905, 3.8081201065177179], [-0.047797090477926574, -3.3258444472380511, -0.97706910167732008, 2.6633054658254807, -2.4619944815120887]], [[-0.09157079730147899, 3.2241668479609977, -3.206020919197996, -3.4733928705917116, 1.4695699498104586], [1.5809132689138465, -1.1411470466733187, 2.7663390131531207, -1.5117734281188158, -2.3833312671845217], [1.7302439140241344, 1.0450451701593462, 2.9307806574533135, 0.39325227501453508, 2.7783180554388052], [-2.0304341529494763, 2.1620956980187849, 2.5452664159395377, -2.0528518914410467, -2.4560654616373947]]], [[[0.24889943298026296, -3.6292683042282246, -4.3910786892958065, -4.611519632161035, -0.43179001273915052], [2.3066006169772235, -2.171943463484125, 3.7966371077865766, -1.6107813631846102, -4.1913029957452297], [0.31220415567618609, 1.5870016283077639, 4.9123376068750861, 2.4714360006737621, 3.8154546213936893], [-4.4147096704911268, -4.3548286322027305, -4.4431476337600024, 2.1908138405425319, 3.7222078690601688]], [[2.4834990072868219, 0.51195320192064386, -4.7479825154217803, 1.5330199625414744, 2.7163576968663588], [2.3352140239004768, -1.3317852207763847, 2.7701796295009009, 1.2335270237061557, -0.4556555771459081], [1.4631749524420732, -3.5873654575128535, -2.4016589077330552, -0.11280110819707545, -2.1130494689817905], [-1.9225854603098846, -4.2157513028103182, -4.8121912696860667, -4.3369074155025888, 1.9967377501678421]], [[4.3799715836928002, -2.7557574737215687, -4.0392060404899874, -3.3905210792965237, 1.038328740064868], [-1.0375133936829437, 0.16829269388417689, 4.9153292006373626, -1.9290712004255264, 4.9042211764170531], [-2.0512077592820823, 4.6322886503106524, 3.5442650694211615, 2.8576152953062204, 1.4602701599250469], [-4.635623024155584, -2.3036304098816185, 2.4877176847520852, -0.15388344769433537, 2.093135865213287]], [[3.8309395713279635, -4.2600257017745689, -3.8267996379064604, 0.18741847313251725, 4.7881551019656872], [-0.87119626560239194, 0.65226827781975505, -4.6228919209124886, -0.41096678406467024, 3.1359512124882247], [3.9675749234865556, 4.3602269012162989, -1.875495114427844, 1.0789220623509674, -1.0781865841040492], [4.5521462554999275, 1.1130936692175331, -1.7310587121366297, 3.62029917744551, 2.6113610931867228]], [[0.66438181664201856, 1.4446904708263739, -0.088984474262577606, -2.1019430236173009, -3.6332142136951253], [3.7766692212353945, -2.6982157656492167, -2.8594104178522972, -2.7085710489868187, 2.6434619495531555], [1.5195634864186705, 3.6096181945072718, -0.87412296488175656, -0.95192387116372235, -4.177795402753012], [-4.5763771481353945, -3.3874848708362162, 0.54752772315549691, -0.27662803150625948, -2.6252530668272991]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[10.184293699635335, 4.963895907935485, -9.7283584922453521, -6.0113193673050445, -13.394183136981193], [-18.669538737778456, 8.5649787661655008, 0.24933550640855157, -15.784046692203974, -5.4411734849315163], [-1.5195682730042623, 16.524621450113795, 12.851864928632587, -17.985496805765802, 10.472521992015638], [10.7341560818515, -6.6748299128732356, -4.4991289024821448, -11.291003881730928, 8.6796156989534996]], [[2.9075838265620062, -0.45587953405598064, 2.0962430033695001, -1.2146517252767062, 1.8334286896777041], [3.40886217046388, 2.4246433995215373, -1.7027779923930562, -2.722645383151173, 2.2698249531689743], [2.6985692015768703, -2.2544320980587571, 2.5181054221768369, -2.7898398923700118, 1.665056572030664], [-0.1495878138028145, 1.2691625015230881, -1.8174879769636461, -3.1271907991246541, -0.81258589428603667]], [[-7.9354177641711194, -7.5845899289041618, 5.7484511992150154, 19.351029890148602, -6.4379877674574262], [6.9161138658099208, -10.872503980150411, -7.6268113101627728, -18.032794955649749, -13.589968936867216], [-15.429792452326364, 8.4179161256175394, 17.599874961752235, -13.242172981947592, -10.156377331094541], [6.788145008682652, 3.9750354130288295, 12.269226642123934, -6.2631069146282252, -18.418690787705625]], [[4.1973490912193308, -7.5652224539817468, -7.2147152117393656, -17.712771912351606, 15.729202852421622], [4.2853258389001825, 15.57962882707367, 16.257881642099363, -1.0118070466362219, 8.7955826616852324], [-16.742617041734569, 3.103061672513443, -1.4802374334511501, 15.437469062120975, 14.854557542933385], [-9.0466490827682176, -17.654475297237379, 15.98728985104872, 16.125764168865889, -11.617321183099181]], [[-15.48148712633593, -8.7966277251268199, -17.256016054098598, 16.361997491871875, 9.2541109466777876], [0.75270807612920143, -10.541247290013413, -10.543685148459073, 1.8723106863525467, -9.4045406932969229], [5.2458641026652604, 13.65982029164725, 11.766749723147186, -13.095224157470813, 2.2393920634397371], [13.904433347204353, -9.5210817253039188, 18.967900592719989, -18.739572064634395, -7.9247138773972106]]], [[[5.8883572947575606, 7.7996209963169676, -3.0634389538967066, 2.947800461233026, -3.0428644662216695], [6.6949623755388554, -8.6060371293157178, -1.7498406890632299, 1.9441682546797845, 3.7341211510164052], [2.6088611340483552, -1.0927689660863456, 7.6046313883110255, 0.054611212655621355, -1.6053416748956213], [6.721690884994211, 3.8918243104667636, -6.777949371177928, 1.546068860661528, 6.2078983841020365]], [[5.9128997058037038, 0.51166979499137966, -11.961450615969232, -1.3154734198202396, -7.3219258327616785], [-14.95136765068573, -0.87162558455739136, 14.163740028452583, 12.869370185284764, -8.2763087516583749], [-17.652830000786604, -15.81375393346611, 12.294258809410916, -12.52337612534253, -1.4014364711426275], [-3.0784483875635615, -2.1046638003537019, -17.247843466669075, 2.6186840949473553, -7.3576139441262827]], [[-13.583619445619078, -16.677998964853863, -18.005419000668791, 5.4315211766629181, 11.144555141785043], [21.777359921425461, -23.00207208405655, 0.46329316116895314, -8.9114650021572821, 1.3481503990945121], [-19.96166768812154, -10.56616840196109, 15.587883749091681, -1.5974337254023796, -20.023774373266935], [5.1822327921812024, -16.75943883973838, 21.592594788719921, 0.78438266117403388, -22.089969792592505]], [[-3.0355145009615416, -6.1686059629106289, -5.0188978297998768, 4.4277772015170713, 6.6331572911112708], [4.2401558543470053, 9.644851181706942, -1.2346432592864374, -3.9668842732059439, 9.3280888430008684], [-2.4087002689294734, -3.5564098774940209, 3.6349227518629377, -6.79897177959048, -3.6983728437862209], [-8.0571353420897776, -2.0710463739096685, -0.83151531039359039, 1.7515581413152372, -2.335932699351241]], [[-0.1898652947295891, -0.77846540307982615, -3.0264613686614323, -0.60542255378364407, -3.0301658672946026], [-2.7713407747850476, 4.7206387011021107, -4.6708002235207831, 2.2802874813223242, 0.2093442292658417], [5.3134625368646695, 0.60675435387421617, -4.4143981731357727, 4.1284439647318649, -2.2404372929996823], [4.7856094927531529, 2.4470673452165546, 3.314618121598925, 2.8100286041737754, 2.8670761336157025]]], [[[15.769918267743595, -15.865939300951577, -8.1449976375638098, -15.149918537343817, -6.3816389652495369], [4.2364312733615872, 8.0775715505885088, 0.071083760294941845, 19.058413225483942, 6.9208806463426056], [4.196766527207445, 19.161361393508244, 10.222269044801152, -8.3253022671560224, -13.700872944241382], [-0.47990559410176592, -13.096836203926474, -3.3446284202539487, -3.7015013781684782, 6.7174318022733566]], [[2.2245356527214395, -12.127759134215156, -18.702468868173394, -12.470542449178003, 5.4105853192401678], [-17.78851604595333, 19.163045776212211, -14.843240549442012, -1.3701970788623425, 6.1174105251499711], [6.1266955690993008, 11.352874236860158, -14.447504305443891, -6.9978545647647579, -18.993307181436609], [10.618618401863749, 4.2445505914042592, -15.57995069777564, 14.823122299119376, -14.320897277265717]], [[16.43111191711904, 20.919060988636446, -22.737800577276641, 14.255203454339378, 9.1179776390156846], [-1.8704573426107094, 13.797995903770314, 23.077528013380842, 8.8153965357853181, -6.8778749321479395], [6.6621487989873227, -11.419363348642776, -21.638218538110337, -21.187099224644914, 18.470996951968136], [4.6489592612578123, -2.5093950643406773, 1.1011559392377188, 0.063194134512604375, -0.32058351494116999]], [[-6.8577658907710806, 8.3114757896472877, 10.920112792200788, -13.98767392974205, 7.7046384420013778], [12.348939693785507, 10.476899704797292, 7.4602450524960187, 12.532121145816795, 12.119850032113147], [0.60471039597690512, -4.7730783673302541, -14.82847135366681, -11.048826535410575, 11.393953951865644], [-0.14300963013392221, -9.9509777588266548, -2.9234057857703024, 7.9686509323669146, -7.3663253698549163]], [[0.034800640360095114, -1.2253149938994787, 1.2184188003586267, 1.3200310544508955, -0.55849656019682581], [-0.60081156584059259, 0.43368245269850264, -1.0513217308121798, 0.57453560445368401, 0.90576315520317829], [-0.65756330578793265, -0.39715981730545019, -1.113816267194482, -0.14945191476834518, -1.0558742557953356], [0.77164784859318702, -0.82168460937540921, -0.9673051209777932, 0.7801675041812377, 0.93340511768060641]]], [[[-0.066991322102761069, 0.97681814319429938, 1.1818595298949985, 1.2411912448422635, 0.11621635082815758], [-0.62082192411664427, 0.58457892976714554, -1.0218654833785594, 0.43354206092860653, 1.1280898700982627], [-0.084029798317710278, -0.42714174149204359, -1.322156450737157, -0.66518739394084692, -1.0269302201685671], [1.1882198122544425, 1.172102821244086, 1.1958738946060194, -0.58965789476350428, -1.0018328418989]], [[3.0914920154682832, 0.63728603530233041, -5.9103506757771092, 1.9083232809213302, 3.381357554957265], [2.9069049305499162, -1.6578236448931898, 3.4483557999771159, 1.5355105572673651, -0.56720601635717904], [1.8213792997040819, -4.4655993966285807, -2.9896163901617099, -0.1404162934247383, -2.6303515896240737], [-2.3932595028885317, -5.2478223077996669, -5.9902785720834038, -5.3986390199925669, 2.4855652422312695]], [[18.804419594329016, -11.831222839208046, -17.341419632974798, -14.55643713668916, 4.4578301324432079], [-4.4543296267464347, 0.72252670365269456, 21.102856711941079, -8.2820318778309687, 21.055166916628171], [-8.8063976315032253, 19.887685981069378, 15.216502263863473, 12.268526410576703, 6.2693404017834053], [-19.902001362725709, -9.8901173192424832, 10.680454492097445, -0.66066385677930928, 8.9864064926705645]], [[11.623814230691289, -12.925744834506215, -11.611252869144597, 0.56866402472044786, 14.528191942660412], [-2.6433785658289257, 1.9791085577790997, -14.026751374369333, -1.2469529899936376, 9.5150846552869393], [12.038392409559826, 13.229774722296739, -5.6906162038785251, 3.2736589519773265, -3.2714273682225534], [13.812095294034137, 3.3773422397937107, -5.2523681247524712, 10.984690391126117, 7.9233764123132424]], [[0.12293202259915446, 0.26731424184070413, -0.016464992158139138, -0.38892712113565536, -0.67226158308031414], [0.6988053773070072, -0.49925677249368344, -0.52908297202401944, -0.50117283324879691, 0.48912555398482754], [0.2811681598955062, 0.66789555997222794, -0.16174088661307273, -0.17613655869457207, -0.77302663318163345], [-0.84677708646557504, -0.62679374459808868, 0.10131025375861893, -0.0511850905140145, -0.4857563245346988]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_array_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[[0.51716435171210051, -0.92190593168084689, 4.8206874680288081, -1.8497043559476003, 2.5264616556692667], [-0.17855329100062711, 3.2189901163621251, 3.1913816629210814, -3.5046511800289473, -3.2608624741297607], [-4.8189702499156928, -3.6457110239844157, 0.6656005229859252, 0.97043863611729542, 3.0514274598651276], [3.9502993221637812, -0.66089503535904282, -4.5094830226452629, 4.6879160699270201, -0.28735648562186444]], [[1.9276171518901108, -3.2998676488174028, 4.4076977392437442, 4.0360524517618437, -2.8722037668622513], [-0.012312062340365415, -3.5657295891228169, 3.8575593255106071, 1.0431669618648565, 3.5395198336667804], [0.92198291065323801, 0.44127622623133611, -4.158243861901, 1.1350999720947472, 1.6973490016166313], [-4.2248850014412662, 0.88343861120514067, -4.7283795596419544, -1.6373735933543667, 1.7884896829003019]], [[0.75289247091544098, -1.9249031201740907, -2.1010441030157256, -4.7204662432744504, -4.1023388955494475], [2.1268487653697115, -3.5063505786946125, 0.606868710877424, -4.3428107031800458, -1.1770821547576995], [2.445438098027136, -0.82048786186021072, -4.5930395498321364, -2.6550161550773002, -0.19927619320568368], [-4.2510707636290395, 1.3768706186356017, 0.80188910799789159, 1.0106841805951525, 4.4835769297223251]], [[-3.3167665605311392, -3.8933854815297622, -3.9336475536972912, 4.3526892836845796, 0.39999537299200938], [-1.4542775247812498, -3.5792795831302704, 1.0389175814020621, 1.9368762090901992, -4.832447661962064], [2.3681948643828328, -2.1218835168637806, 2.7514202964337944, 2.7780348133283503, 2.7877444756894176], [1.070531314965474, 3.0369654225616465, -2.8879758007277045, 2.6985580346546136, 1.8347618030443833]], [[-2.4394288307487733, -4.4800297193699894, 3.8879669600485283, -4.160756165257391, -1.3535660318386791], [-1.3030797904086153, -2.9770138802960053, -2.1266308896971053, 0.42182952174089205, 0.87640954963377382], [2.5280519307946143, -3.9809371710340971, 3.3761898215954353, 3.6301308654447535, -4.9626270587733554], [-2.5718530191898137, 0.45599829322026775, 0.18409292222827212, 4.6873151571102678, -0.95080329741052338]]], [[[-4.158960377591244, -2.3876071887519181, -3.3556413064588275, 3.1192829926582437, -0.19124728388174894], [3.5469271060937881, -1.8863582041931126, 4.8372310818060615, 4.2316412652111985, -2.749914929974786], [-1.7171923995896865, -1.8373241878149402, 0.54766251881897787, 4.474391526330006, 2.1773763580797665], [-3.8173033199608595, 2.3731340977325139, 2.2066039475683699, -2.8503027081269918, 2.6293068679257203]], [[-1.023382790877474, -3.4025274513585293, 0.13085263779627532, 3.4842815835877001, 4.3166425993093487], [4.1622883944123128, -0.049341430081573101, -0.72448408144665155, 0.7691843021061846, -0.70410204104074392], [-0.79623277883918675, 4.2579241503252128, -4.3192442076522441, -4.2735737958028697, -1.8355258681312203], [-4.6007932945514458, -0.690385471633153, 0.55317433122610637, -1.3007516105402583, -1.8381424417305317]], [[4.7229084306428195, -3.4471088946564543, -0.58919867185306618, -0.5059925092100368, 1.6100884716600481], [-3.2844743886989569, 1.8431981900554284, -1.8278884381614748, -0.9464316100659218, 0.36597774926318394], [-0.20510100673443432, -0.33859313165784677, 1.3460155542567209, -1.8395784070212007, -3.4537619006388431], [-0.040783135040145524, 3.4977973778329225, -1.4947719209016319, -4.5491259963451807, 1.5312892697950673]], [[-0.36815204692149628, 3.2589884733914616, 2.9801150715533193, 4.1641241517651437, -4.9790892199260073], [4.6047880512646451, 4.9757209196580767, 4.7385229103058002, -0.94867622123197304, -3.3000743069094418], [0.43151139210608314, 3.7055529091513186, 3.400156997850015, -4.411685014970165, 2.0824550575093648], [4.5263162774231507, 0.15267483309482355, -1.2124058551684458, 4.182552610851717, -3.8102229133289942]], [[1.6337385863867162, 4.990151439964654, 1.2302233657844317, -4.6539712849583825, -3.093070457278678], [-2.2626803818287589, -4.939677227115137, -4.837270211222239, 4.28450351622174, 3.816492026209465], [-4.7365857492570775, 4.5066728262847064, 2.7486101251503845, 3.9676475721332753, 4.3199994093883216], [0.079805349027771477, 0.34762223625914679, 4.7235026078029385, -1.2967516558866912, -0.38892983172065598]]], [[[-1.6871474347162483, -1.3632580757409274, 0.46749106530613638, -2.5999273765148811, 1.9433117734277321], [4.4260596560158163, -0.75289705430116349, -0.064830592978420754, 2.6775448957904127, -4.319249466277121], [1.3251628992287801, -0.87218453381233374, 2.2953725198658681, -1.1002190265318514, 3.3594046316911594], [-1.2218410876858044, -1.6600697923845518, 2.9788474621531345, -4.6265985566910039, 3.1852515897007532]], [[-1.1080238259074626, -3.0544595782352255, -4.3474013658536332, -0.74038157883506894, 2.050838068960056], [0.91262867415304783, 0.6778077469356516, 1.4849012651247566, 3.6853812055471042, 4.0338163135890888], [2.0423362639974574, 1.7284330165167736, -2.5787275689400935, 4.7251112493488812, -2.1732139008834936], [2.1757070751653149, 1.5490827516160488, 1.9676306620679664, -1.9769205569670421, -2.6479824214989134]], [[-2.5603648769306506, -2.604857004164006, 4.0593757117864957, -1.9587297852835373, 3.8660056894707679], [-1.7107557622631973, -3.3398294162983322, 3.4168425488519922, -3.0830853253097237, -2.0489754573249486], [4.1298069186105373, 1.2196644101833165, -4.7704323438242788, 2.3541418363868036, -4.9790675386718855], [-0.14478448393229826, 4.9489640941780948, -4.4901191227782249, -3.5024174324465673, -0.21518294801981419]], [[0.21050915370268797, 0.16446245329077946, -3.6718595738467705, 2.4591526419451393, 2.3423853507303249], [-0.36048716446593332, 1.9361713971405168, -3.9911467648118948, 0.46906609408468292, 3.1564124552800461], [3.2048682689706496, -0.46675225687223332, -0.53026018900340954, -2.7671921319140536, -2.1005936902693922], [-0.40945398232453112, -2.5534911108021907, 1.8467254486451878, -2.5965450307205087, -3.7333847654517616]], [[0.12203116506642964, -4.9888025029268857, 1.1182654826587051, -3.4047112826577219, 0.079699767362781948], [1.636931882174113, -4.6736733355936657, -2.6939348236973526, -1.8317560251482821, -0.67416420055616744], [-4.9038204229479954, 2.1478217978386045, -0.78408907899913416, 1.961130926531375, 1.6292572600425412], [-1.9526891544018032, 4.5634169294352134, -0.227104918024855, 4.7691940604846188, 3.3500553670707429]]], [[[-0.0030148006833687901, -3.4659752755050253, -3.7942961895748128, 3.9839713006404924, -3.997901377309395], [-1.9359159590365893, -0.36636033750039232, -3.0353885085764718, 3.1609158087874878, -3.6040317348225406], [-1.4968180647575458, -0.34665522451058717, -0.3674521517441498, 1.5155071709078216, -3.4053931183877886], [-0.12625177125228326, -2.2529729296039323, -4.9542031379351386, -2.1182840047791309, -0.94461263169039]], [[-3.3678555943853117, -2.4286900115414154, -4.1555363649269941, 1.3054223246335592, 3.8681226350516944], [-4.649438854615676, -4.4149867296340126, -0.052058357535720745, 3.3239621045384133, 0.91014113877698843], [4.817279551240107, -4.3443396285325777, -0.50373793806991962, 4.4401645552522488, 1.4113215076187045], [3.126859540057831, -1.5135012008188697, 2.1096740858522258, 2.5916410517065414, 4.149671507269046]], [[-2.4873688771978895, 3.2064778069083495, 0.63874168919387486, -4.8912841938117646, 3.9859663440292099], [-1.8075951759030229, -4.218302502980535, 0.85905216178963961, 1.011582687932937, -4.9418976932317751], [4.6951618476320149, 2.5602958190855674, 2.7710196735166583, -2.9539912515381261, 4.1758692743247448], [0.69742109757845938, 2.1957802753302857, -0.78902895190816125, 3.7084099092048746, 1.9603259672810296]], [[-0.16587571565302994, -4.1330425066793399, 0.088973709970429304, -3.2381806795005774, -2.0497700680769846], [4.8528420160349324, -2.0932913965178788, 3.0047555776683019, -0.18599406846763777, 4.4830000255293641], [2.4417092103306803, -1.2711135691658493, -0.7878520744203783, 1.2405105651253736, -0.9940680023607591], [3.5745507684980815, -3.4454008954417628, -2.5473661817743332, -4.8258119534002377, 4.8982908799057867]], [[4.29419963385069, 0.81652632140638204, -1.4828546527585318, -0.056534389024939458, -2.7760113926625052], [-3.8732370177457076, -2.0767127216860182, -3.4994481875887065, 2.5675372126277418, -3.7968589028784883], [3.0961072295682008, 2.9421969886149393, -1.7665744381840112, 4.9212225588376644, 1.6763172498148133], [0.59442844046436782, 0.68058987170564222, -1.3865284101280717, 2.8297125896084445, 4.7750223546053263]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[4.2157830631332427, 2.119279201985627, -0.27165733694474703, 4.5943645637087283, 2.7589822382144664], [-4.0341429904067923, -3.1389571566377672, -3.8837762085927654, -1.6201705822341914, -2.1993351779357817], [3.9009682786378228, -4.8983251982760079, -3.7508399405909887, -2.0809512474163352, -1.5478344873544572], [0.87319702240401664, -3.7170676308086801, 2.3189829672709905, -3.424277850372579, 1.9984895923985588]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[2.1802527148041566, -3.8865554125821866, 20.322972580374525, -7.7979522956076766, 10.651014257626064], [-0.75274194006714501, 13.570564012952753, 13.454172762536698, -14.774849086955969, -13.747088789643007], [-20.315733161337548, -15.369526787991651, 2.8060274116166921, 4.0911587659534181, 12.864156203679098], [16.653604976684797, -2.7861900965754982, -19.011002150354802, 19.763237168988486, -1.2114326051661473]], [[4.0851589393914809, -6.9933408774439325, 9.3411421474183349, 8.5535220191419743, -6.0870017069759443], [-0.026092697651486908, -7.5567765581327411, 8.1752452489803336, 2.2107620464787239, 7.5012307685056339], [1.9539392071335799, 0.93518752858277499, -8.8124797333111839, 2.4055937630348634, 3.5971564376371949], [-8.9537109143354918, 1.8722530749581212, -10.020756459843152, -3.4700518022763807, 3.7903089879364789]], [[-0.2045287636546391, 0.52291405550312786, 0.57076404582871676, 1.2823492887855117, 1.1144304596098176], [-0.57777407168455897, 0.95252586060285105, -0.16486033787205265, 1.1797563904810355, 0.31976300352666126], [-0.66432120137327899, 0.22289154754843432, 1.2477328925892979, 0.72125461823358084, 0.054134839962742924], [1.1548345628111369, -0.37403700557601394, -0.21783905960370586, -0.27455977299266293, -1.2179965687152721]], [[-15.238434751798346, -17.887632289198383, -18.072610926826361, 19.99784140179516, 1.8377245673219431], [-6.6814811256530158, -16.444515280339864, 4.7731661206076126, 8.8987154193345077, -22.202026294095603], [10.880350564877485, -9.7487064381966047, 12.641027909804389, 12.763304703104964, 12.807914431782228], [4.9184111378178814, 13.952926318625932, -13.268413679711706, 12.398159407528627, 8.429564610753447]], [[-6.7303408154241495, -12.360322422414741, 10.726831785538584, -11.479452357486476, -3.7344646400933525], [-3.5951739967135992, -8.2135284186546063, -5.8673368519125422, 1.1638201580376242, 2.4179983808411216], [6.9748503743461283, -10.983334946330819, 9.314847750622274, 10.015466580156184, -13.691799910038187], [-7.0956967992429449, 1.2580911916508308, 0.50790910260879985, 12.932219263380681, -2.6232494095913808]]], [[[16.777840854639305, 9.6319488043484167, 13.53713685477037, -12.583633619927376, 0.77151888970589544], [-14.308811122532104, 7.60983872684199, -19.514081861645789, -17.071045947967885, 11.093550038972769], [6.9273996819845536, 7.4120284933784939, -2.2093489114021074, -18.050335212279741, -8.7838475724249605], [15.399547430476678, -9.5735622856629679, -8.9017558476868963, 11.498528690528001, -10.606999870870983]], [[3.2123547354047783, 10.680387894098319, -0.410740823875548, -10.937010612543784, -13.549756179749533], [-13.065244943630852, 0.15488063507329589, 2.2741244923271058, -2.4144365698696344, 2.210146140728102], [2.4993405794868417, -13.36544148408411, 13.557922516876234, 13.414565050755046, 5.7616370599642446], [14.441693038143312, 2.1670904170216256, -1.7363905258704972, 4.0830035769134456, 5.7698503723896719]], [[-18.342719398292779, 13.387799513495242, 2.2883157838773944, 1.9651616689960967, -6.2532232999627819], [12.756163488561276, -7.1585692782585193, 7.0991096280933244, 3.6757285702341722, -1.4213756754626823], [0.79656641031362052, 1.3150199491256631, -5.2276231860180573, 7.144510851029918, 13.413638299845269], [0.15839256958074313, -13.584662238505665, 5.8053596236702649, 17.667787314496273, -5.9471848345034708]], [[0.59646911621151, -5.2801172524291626, -4.8282947706034305, -6.746591451460791, 8.0669738804435056], [-7.4605421380824879, -8.061516659437272, -7.6772154225202032, 1.5370173057051384, 5.3466833112415664], [-0.69912206338919913, -6.0036278143192936, -5.5088343428943194, 7.1476822793380697, -3.373932423001484], [-7.333404478568764, -0.24735927322774826, 1.9643043002724037, -6.776448698748764, 6.1732110759302934]], [[-3.5931387445913812, -10.975015605141159, -2.7056735250882595, 10.235642764111963, 6.8026986645269112], [4.9763925601811563, 10.864005893242599, 10.638778540721919, -9.4230593032160233, -8.3937451695538865], [10.417339661650402, -9.9116840822952277, -6.045114938673712, -8.7261868790442101, -9.5011266697295369], [-0.17551871150422094, -0.76453781283744493, -10.388565448412404, 2.8519915338380755, 0.85538706065188252]]], [[[-6.5815086242132619, -5.3180265090621965, 1.8236678163058408, -10.142234222546607, 7.580797583644995], [17.265918317476334, -2.9370275259086971, -0.2529020866940993, 10.445017703107014, -16.849255155470395], [5.1694184339192004, -3.4023641995204317, 8.9541753876537165, -4.2919195220545374, 13.104930903336191], [-4.7663633245986574, -6.4758796004170129, 11.62038945676016, -18.048214207643142, 12.425565410903335]], [[5.427461026732713, 14.961736319185112, 21.294985657380387, 3.6266297439471926, -10.045671790770752], [-4.470352031273098, -3.3201227664015898, -7.2735292839125183, -18.052195624384193, -19.758944094070269], [-10.004027185291626, -8.4664269983363241, 12.631446230428292, -23.145131497343055, 10.645108411941315], [-10.657320790349655, -7.5879110764556259, -9.6380948529080239, 9.6835997791815025, 12.970679019820048]], [[9.6035188428778149, 9.7704016907465405, -15.226068553633763, 7.3468819114667028, -14.500768550618959], [6.4167710416929822, 12.527165569412473, -12.816029502944769, 11.564159578421673, 7.685378982625104], [-15.490244737253402, -4.5747659838329344, 17.893128169103189, -8.8300092257358394, 18.675685390950576], [0.5430634251111186, -18.5627721889939, 16.841718143727938, 13.137007194242726, 0.80711679596683361]], [[-0.43805928599016558, -0.34223834732859831, 7.6409607605340497, -5.1173767578429139, -4.8743897175320194], [0.75015621457296155, -4.029078284091387, 8.3053818388569827, -0.97610367360622896, -6.5683404361754691], [-6.6691746221195043, 0.97128869117266359, 1.1034456017618668, 5.7583919187472175, 4.371233060080975], [0.85205377527781911, 5.3136905122903419, -3.8429456259936949, 5.4032836206505293, 7.7689916847519855]], [[-0.18888404582186427, 7.7218405646304689, -1.7308898800772214, 5.2699295427824513, -0.12336204855824109], [-2.5336996206791347, 7.2340727714608173, 4.1697652268039116, 2.8352551481438293, 1.0434945997605829], [7.5903023704320276, -3.324472651386245, 1.2136401176328533, -3.0355060823026623, -2.5218205758664745], [3.0224396162661238, -7.0634141035570046, 0.35152082436667748, -7.3819230437041323, -5.1853312316989912]]], [[[-0.0026325149798592223, -3.0264792902969293, -3.313168134855633, 3.4787918770623354, -3.490955578531481], [-1.6904360510551661, -0.31990475583227318, -2.6504922075283441, 2.7601022723030182, -3.1470297794966249], [-1.3070170772268315, -0.30269830984344059, -0.32085812477894049, 1.3233363490686449, -2.9735791310913458], [-0.11024267073072677, -1.9672892536870079, -4.3259954284295992, -1.8496792855791928, -0.82483293731727059]], [[12.51854701512757, 9.0276050271689545, 15.446409710718497, -4.8523430674304233, -14.378073438749031], [17.282278667916113, 16.410804263172462, 0.1935044357090927, -12.355391944814434, -3.3830561664152947], [-17.906153888471167, 16.148204210457848, 1.8724279840100058, -16.504391943792154, -5.2459774926335925], [-11.622748382434281, 5.6257863227538882, -7.8418012560772006, -9.6333050639733493, -15.424609638158838]], [[-5.7681660595418736, 7.4357674191529028, 1.4812310977264966, -11.342804733531301, 9.2433880599191589], [-4.1917824246403201, -9.7821716552084474, 1.9921273311874974, 2.3458430233026868, -11.460176576600285], [10.888000353239237, 5.9372823956345604, 6.4259474248579513, -6.8502553977844309, 9.6837697207093552], [1.6173076462998868, 5.0919770583605386, -1.8297447001587075, 8.5997394151050646, 4.545962528423737]], [[0.56800453902537051, 14.152685910270426, -0.30467070431721494, 11.088430376318254, 7.0189822424727115], [-16.617479426865831, 7.1680113634716562, -10.289117970393029, 0.63689536895441301, -15.351037690639908], [-8.36109076598607, 4.352646040202651, 2.69782440780779, -4.247852851311988, 3.4039650422480641], [-12.240255021600261, 11.798009971915079, 8.7228895930380173, 16.524920982091661, -16.773108964743397]], [[8.5819132759323065, 1.631819355250135, -2.9634695905777049, -0.11298338807895282, -5.5478298766158458], [-7.7406238688576288, -4.1502887606911925, -6.9936107820340299, 5.1311963975325474, -7.5879830012084692], [6.1875380752419851, 5.8799500605333375, -3.5304806289080775, 9.8350120657140767, 3.3501025773130793], [1.1879590516937455, 1.3601517752955963, -2.7709625972058718, 5.6551511598116511, 9.5428324791492045]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[-2.1978657820430767, 3.9279916676184872], [-1.7741060385440779, 4.0196874937344642]], [[-0.84496212629192158, -4.7804894681462153], [2.6177102255607441, 1.065798054937062]], [[3.3236033413787318, -1.0594762010774361], [1.1831104126521144, -2.3244762564082944]], [[4.9479116465869595, 0.62586393256073514], [4.0243919325581388, -4.2824310751392183]], [[0.33155759215251468, 2.5075655206588738], [-1.1280590183357009, 0.73867648076891168]], [[3.9055987153284448, -4.2641749460424192], [-0.47672129232214111, -4.2753965769432734]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[0.2077586792167283, -4.1396745659017427], [-2.3149031974570056, 1.9543975869875734]], [[1.043428767504083, -2.8497324473920616], [4.4765362852676081, 2.5465906497428339]], [[-3.54847482835287, -3.6840303412457054], [-0.86429368737676793, -3.2538445867589596]], [[4.2213749298390617, 3.6977635605910812], [0.61933581336876653, -3.1249193092660068]], [[-0.28692381088720786, -1.7593939219729227], [0.86284084170860975, -4.3363702005127314]], [[-4.5552851721614642, -2.1386366007455635], [2.5437379972910792, -3.9524665626652524]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.45662569197291125, -16.260607201514222], [4.1068837412534673, 7.8560675381987632]], [[-0.88165779002440903, 13.62311595179229], [11.718274809038725, 2.7141513612168215]], [[-11.79372279631192, 3.9031424705970106], [-1.0225548611249455, 7.56348448396386]], [[20.886990179960904, 2.3142968437113205], [2.4924500508655969, 13.382251557303329]], [[-0.095131767868986111, -4.4117955359960899], [-0.97333539287776416, -3.2031746790259241]], [[-17.791115916348527, 9.1195206115885554], [-1.2126540653975384, 16.898362012501767]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=numpy.array([[[-4.0329738311624492, 2.7825691199738758], [2.4362729637772862, 4.7989777645077751]], [[-2.2719212148949444, -0.40765075750040225], [4.2029216294481841, 1.7576206295727488]], [[-0.93744818126144658, -4.190988061738004], [1.6574303787962998, 3.5762582973501935]], [[-1.933368704282362, -4.2192333811115743], [2.1552549730879322, 0.15289352372730658]], [[1.5847715624963694, 2.4465396469291019], [-2.3680228379039594, -2.910973789596353]], [[-3.6391181212828148, 1.5726508337132286], [0.32954650590666112, 0.042827360390489844]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-1.2314671215151165, 3.5306863283122158], [-3.6844787636244725, 4.9856883140284687]], [[-2.71001837868344, -0.19518712151887474], [-0.75832069137197866, 4.0238230788612057]]], [[[-0.51297273501993335, -2.4180670758353964], [-4.8251113370180141, -3.6546411781929109]], [[0.8835706470358673, 0.2839688464952328], [-1.6805573424185907, -2.9311686955200256]]], [[[-3.0570533854877571, 3.7665087793854131], [-3.4832376956741786, 0.56776760400371629]], [[-2.8147693371949876, 2.3993078864400648], [-3.3988032460212625, -0.50492548056246012]]], [[[-1.4110519693477519, 0.036000342220332548], [4.6875645291752104, -4.3370954855143449]], [[-1.2548750494648866, 1.738269366657855], [4.3150590335475449, 4.8552171316944133]]], [[[3.4629042848911755, 3.0554294354845393], [-2.7628924883241957, 1.8547997490838029]], [[4.4001648004941689, 3.5648107711193209], [-0.78731693200448394, -2.466676674957391]]], [[[-3.1270153669093483, 1.6551732039507669], [2.0401673337547628, -4.4213320131959613]], [[2.2987408217152936, -2.0992229932883499], [2.7225611572866981, -2.9971002310485826]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[4.9664746750074125, -14.239165568126198], [-10.252316830860982, 13.873022344430233]], [[-6.6023445073260199, -0.4755291070339463], [-3.6391641362602885, 19.310237483768141]]], [[[1.1654336393544693, 5.4936578886294196], [1.9669602915591722, 1.4898172446825027]], [[3.7135781835725736, 1.1934988070242651], [-2.9537822542148691, -5.15188256800384]]], [[[2.865829136244646, -3.5309068049401264], [14.598207598766278, -2.3795072502211654]], [[-4.6652842087712978, 3.9766857790713059], [-12.154998309644311, -1.8057439394050319]]], [[[2.7280837176529382, -0.069601934992245942], [-19.777928737610608, 18.299218049550433]], [[-2.704575690963162, 3.7464136970557522], [0.65974458073043019, 0.74233125572594516]]], [[[5.4879122343423612, 4.8421576805702333], [-6.7595260128877452, 4.5378411232476736]], [[-10.419690738111312, -8.4415533188165757], [2.291858953170467, 7.1804311482096477]]], [[[11.37957828724964, -6.0233708003589719], [3.2084708583439223, -6.9532114766756159]], [[0.75754200578128206, -0.69179160255709804], [0.11660010786826652, -0.12835789172153803]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_array_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[[0.0013078540955859808, -4.8314761114980467], [0.55805264907414465, -4.4933654032350825]], [[-1.6239200059669647, -1.0276921232573977], [2.0083484278304837, 4.447034462319774]]], [[[1.886478998007779, 3.752745476305865], [-4.739634347503582, -3.9098392330739418]], [[-0.37957377596325426, 4.9340658332452207], [-2.2581820030567457, -3.1380041464616006]]], [[[1.7652233105701569, 4.7202006279747994], [4.5714627953100635, -4.9478737572199272]], [[-0.16904681637644536, -1.3968951384913697], [-4.167737197993012, -2.6661163252818487]]], [[[-0.33066954719185304, 1.8397138137528701], [-1.9078739134160685, -0.74511872216711428]], [[-0.38228903756063204, 4.1371375446384828], [-2.0457234357467655, -1.028945335575584]]], [[[-3.8451230480948384, -1.6158988997063819], [2.4500883507559603, -4.997636265437527]], [[-0.21192049721819561, -4.0298814480188208], [-0.50560545857539996, -2.6958381520371222]]], [[[-4.6751699619005418, 3.9798986907581479], [2.7245956794776589, 0.7156608761855594]], [[-0.51362028065661125, 4.4706727792721175], [-3.7310208734199124, 0.51807651075132277]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[2.7625068818334064, 1.9341109256469924], [-2.3773285558423893, 4.1453504478448675]], [[1.9049781031667647, -0.32078629291835004], [0.65815322341022409, -3.5466968325731028]], [[3.7729595373899585, 4.5269747086768746], [1.9749136289006515, -0.31292301366010111]], [[1.6088839901549159, -1.0954221308359346], [3.0335836682208654, 1.7272368478172551]], [[4.5212946411447188, -1.4418934328888042], [0.21087620168883348, -0.21927156543530124]], [[0.8880508747972673, 3.2419063679038338], [-0.36576373065249879, 4.3814548102587647]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.0036129559394902774, -13.346986007427061], [1.07933572566055, -8.6906671193211764]], [[3.8605914025890087, 2.4431618312341081], [8.3253080547356308, 18.434516299958833]]], [[[3.5937011832887977, 7.1488979591208039], [1.520409732124157, 1.2542228334845145]], [[-0.24981770417220603, 3.2473713326685951], [8.0090869576149437, 11.129549366856622]]], [[[6.6601161252387504, 17.809125977711592], [20.694896456025944, -22.398899360660632]], [[-0.33385286158410782, -2.758747247151669], [1.3041808841392788, 0.83428915527559055]]], [[[-0.53200894050874781, 2.9598861014138356], [2.0899273076005231, 0.81621953836204908]], [[-1.1597057808838063, 12.550352888598674], [-3.533448898665128, -1.7772322979958395]]], [[[-17.38493423189324, -7.3059550358741117], [-3.5327663029523801, 7.2060589111012989]], [[-0.044688989513382092, -0.84980609301450516], [0.1108649003944613, 0.59112065175738937]]], [[[-4.151788774491683, 3.534352513932272], [8.8328840832618951, 2.3201055517656024]], [[0.18786366999174559, -1.6352099542731449], [-16.347299353021533, 2.26992882011346]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[3.4603503679912766, 4.0442489412040512, 4.7520796983331035, 3.0849958567239373], [3.1251798929977852, -2.4008890625383019, -3.0593022388611213, -1.3070687843882087], [1.1709008880179459, 4.6621617852981814, 1.569719957146944, 3.4681012628505687]], [[-1.3093398197458961, 0.82456177058912061, -2.6996921788966919, 3.5564654698231841], [-2.1808891856971035, -2.0130574152214842, 1.0019531064433949, 2.7600232177003221], [-0.42690323591055801, -1.8318744779534093, -0.79116143406472972, 0.34206879863109219]]], [[[1.6486089891750648, 1.0966915493286402, -1.5033419954034977, -1.6870597228726156], [-2.7835695998965724, -2.9047375590378657, 0.31375384464550571, -0.36200934974368959], [-2.6115930100284315, -0.85099355685653855, -2.3844787431313885, -1.931678302138844]], [[-1.3015280195855059, 4.2008479865210795, 4.970402392733309, -3.2811509297113775], [-0.49176602842015971, 4.2159965946661675, -2.7048008484457964, 1.4089523468124927], [-2.579957089369139, 4.2561114856052615, 0.89519162604255609, -4.8266675518713793]]], [[[2.9500808340623159, -4.6100830183253114, 3.8114461931790871, -4.8906318079510207], [3.4953332294694803, -3.076974984124087, 4.6074632398784381, -2.5069452117598976], [-2.5425255456022686, -2.9783916455234594, -1.1978116391759661, 1.9655706637162504]], [[-0.40602112448523009, -2.4580098517894831, 0.025965227016339298, -3.5395694390250485], [-2.9753496179991332, 2.4204483404978836, 0.84925939687208007, -2.0808786688545302], [-2.9873807036204147, -4.8342374395009733, 4.8939384464342286, -1.0662158205300676]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-0.98314009739664421, 3.5800954288566125, -0.48767474694468937, 1.936017177278023], [-3.0983137526300517, -4.7340246142491722, 1.6513480408144829, -3.2777128937586553], [-2.2500637858856285, -1.5705993823792577, -2.5719114102758955, 4.7175170813955312]], [[1.7811283750199891, 2.2639107587736156, 1.0729344511603376, -1.8231158855766836], [-4.9498761303271932, -4.4357158145389119, -2.9744491209168409, -1.6327797328085358], [-1.0347223589594146, 4.1022028615878838, -3.4998147706115645, 3.2225145467715599]]], [[[3.6593322659289527, 0.27290081291328239, -1.6535692105580924, -0.032963106499020789], [-2.2324029478215435, -4.5871860493142647, -4.8257394696556393, 2.5985022002948464], [-3.615884549891212, 1.99164623414873, -3.7496565569557641, 0.43118109554150763]], [[4.4566560341418349, -4.36222832574812, 3.0407889791144864, -2.591412635321324], [-1.2789070528527438, -4.6081063405175273, 1.9597790101223964, 1.4760612184092601], [-4.3671424611583589, 3.2102590497279966, 3.3045979954462634, -2.631538070349376]]], [[[3.9586393395551802, -0.022401388963032787, 4.108281618134793, 2.7606654994351265], [-2.7835817117592807, -3.2149315586604024, 0.60332373113353022, -4.8479444207495064], [-3.6715382759377224, 1.422846231588105, -1.8273268136222809, -4.7170881875333306]], [[-4.3766437321273353, 3.3095683480476517, 2.3811095889049758, -2.5058723894621893], [-3.7372595031256219, 1.8124437571638365, 2.0062096358182675, 2.0393706995690506], [2.8216153245518232, -2.616520609553199, 1.4466393003747431, -3.8311645385591611]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-3.4020091978134572, 14.478797147562819, -2.3174692643455921, 5.9726049704490736], [-9.6827878419179516, 11.365867918137942, -5.0519727584026741, 4.2841962076186837], [-2.6346016849905038, -7.3223884205415013, -4.037180668724015, 16.360826947506972]], [[-2.3321023054929735, 1.8667342637101321, -2.8965927462663781, -6.4838486945395903], [10.795131323170802, 8.9293506122727617, -2.9802585366604539, -4.5065099719420871], [0.44172632330878003, -7.5147207255302861, 2.7689184728779686, 1.1023216795853661]]], [[[6.0328080679888298, 0.29928801532691307, 2.4858800365381888, 0.055610729315258525], [6.2140489802755425, 13.324571607737667, -1.5140943118620205, -0.94068209183628393], [9.4432188155656913, -1.694878112798158, 8.9409763541042508, -0.83290316654998608]], [[-5.8004627020904174, -18.325058078964208, 15.113944817587718, 8.5028159776503731], [0.62892304209992511, -19.42776063948147, -5.3008119293453211, 2.0796999177166344], [11.267040152950498, 13.663220413315559, 2.958248452960512, 12.701559415669557]]], [[[11.678306044586842, 0.10327226284537751, 15.658494333947477, -13.501398502650421], [-9.7295456540557499, 9.8922639816691174, 2.7797919129440429, 12.153531052476085], [9.33497985822817, -4.237793329026549, 2.1887933259350993, -9.271770159577974]], [[1.7770098095895748, -8.1349516046717731, 0.061826051026700038, 8.8697093278370396], [11.119653634988449, 4.3869264842729576, 1.7037923853139774, -4.2436829866201782], [-8.4292391736057706, 12.648881891927983, 7.0797636902266694, 4.0848482420655534]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(-4.59297809178,self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array(-0.137924156916)
      sub=res.substitute({arg0:s0})
      ref=Data(0.633482631042,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(-2.07075229481,self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([1.3780191084555407, 2.4339465000203315])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-2.8535362311275572, -5.040100300364017]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-0.31940961061215134, 2.4880646430347699]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array(-3.06344216847)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.97849287016383024, -7.6220421453518394]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(0.0282724384901,self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[3.9072889273704838, 4.805223386892127, 4.991005980209053, 3.4300814364243148, 1.7340482793782508], [1.8560404273232045, -0.057168997477235806, 4.6054240139493388, 1.2062712616046127, -3.1037518660245267], [4.5770814240016211, -0.16057992331825943, 2.9051300630331225, -3.9261516978005306, 2.7436459268961233], [-4.4115661948830178, -0.45436119764657246, -0.092497573536442168, -3.5823684581071369, -4.8373868002500755]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.11046858586220575, 0.13585538263718983, 0.14110790957927744, 0.096976766427406075, 0.049025773317618593], [0.052474788816669816, -0.0016163069647169277, 0.13020656715530046, 0.034104230046114875, -0.087750633720771049], [0.12940525302435293, -0.0045399860047635439, 0.082135111012900011, -0.11100188237894248, 0.077569560706836668], [-0.12472573388991924, -0.012845899012759603, -0.0026151319582943368, -0.10128229188077711, -0.13676472076298468]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[4.3299078603055445, -3.3508041989911619, -3.4709265436267334, -3.6820906718648692, 1.9692091708931443], [4.4867344919449064, -0.0028664996988458924, 4.7352955985834608, 1.0698718126717903, 2.9795848828308173], [4.5594030915875656, -1.0555616128405401, 2.1836439885761187, -2.9776201465267049, -4.7424886163373827], [-2.3467028223135724, 1.3544939117816561, -1.7158592768435152, -3.540559246347359, 4.8830999790977927]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array(0.291460484971)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[1.2619970448456646, -0.97662701688180664, -1.0116379337051791, -1.0731839329300061, 0.57394665995843219], [1.3077058109596871, -0.00083547139239568708, 1.3801515516455614, 0.31182535737843559, 0.86843125496300111], [1.3288858362537213, -0.30765449959558328, 0.63644593591504262, -0.86785861196696379, -1.3822480320885264], [-0.68397114267501413, 0.39478145241854429, -0.50010517697130186, -1.0319331150099944, 1.4232306880711498]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(-3.94149588698,self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-3.5182383516970828, -4.7196309784459478], [-3.4779190693891957, 0.63319370001131858]], [[1.6382866876937143, 1.9937806338546018], [1.0172393528190051, -1.7521403716929065]], [[3.9556975819746487, 0.90550610872977266], [2.1391346660574353, 3.2621795068671045]], [[3.9368826217093336, -1.7096609714896758], [0.23538823973933898, -0.25129637746189726]], [[-1.559186024055923, -0.83790458096869269], [-1.8568662590749052, 2.9575272508792469]], [[0.44376292518165972, 2.6640391529349872], [4.4589917719396439, -0.55115704322508119]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[13.867121992616909, 18.602406089591412], [13.70820370723453, -2.4957303642540216]], [[-6.4573002412330709, -7.8584781678712421], [-4.00944472520671, 6.9060540684330052]], [[-15.591365749475825, -3.5690486031904625], [-8.4313904879541557, -12.857867108895604]], [[-15.517206660976461, 6.7386216872507445], [-0.9277817787752346, 0.99048363817815321]], [[6.1455253008476083, 3.3025974595668406], [7.3188307228091141, -11.657081494961364]], [[-1.7490897443961566, -10.500299364037517], [-17.575097729162007, 2.1723832189497672]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[4.6035428873307911, -1.7586853055395588], [0.1272583169410213, 2.6014457435303209]], [[-3.4245327845394016, 3.5740269498968242], [-2.0641410544564867, 0.36222439391099837]], [[0.5412651091087497, 4.5022088021569235], [-4.3939058066109098, 2.0089019669919326]], [[0.91317283925531267, 4.072454616190484], [1.0652987534951013, -1.5679033756815719]], [[-2.0547451650719784, 0.3304912624123979], [0.81752199970882344, 0.0013648408701119052]], [[2.1538182004548112, -0.027005761870010581], [3.3366669722161078, 2.0484456139160567]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array(-0.238545276494)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-1.0981534109110116, 0.41952607247613666], [-0.030356870400876385, -0.62056259417497583]], [[0.81690611995124063, -0.85256724696068953], [0.49239109855825358, -0.086406918198425817]], [[-0.12911623510898418, -1.0739806435449293], [1.0481454755272606, -0.47921407516573689]], [[-0.21783306742710862, -0.9714648124290457], [-0.25412198570136468, 0.37401594426807522]], [[0.49014975352711759, -0.07883712957106688], [-0.19501601146059355, -0.00032557634273136374]], [[-0.51378315814563036, 0.0064420969322169083], [-0.79594614545619724, -0.48864702535483917]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(4.46204438043,self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[2.7536107109591423, 0.14027142434455975, 1.1894900591399216, 4.5768844288312618], [-3.1574975831332095, -0.32003797174797377, -0.89346842266114557, 4.8483834199931835], [-4.7112220422907267, 2.5600700483609078, 0.15155499244985915, 3.7496587795542649]], [[0.44324378011933518, 4.8550743524774234, 1.473279593361382, 2.0336461654703779], [4.6844418271461024, -3.6010414056468343, -2.3679456320335799, 0.088522097715757475], [-2.4258535598853683, 2.2687354890992761, 3.6500761371603883, 1.0977533157881894]]], [[[3.5286187567571776, 1.4518891561454783, -3.3397449551441225, 2.2739410682890995], [-1.5365951385817711, -0.093403970592395247, 4.2442378482696181, 2.6970420278698484], [3.8705621530811989, -3.9648567032119342, -1.4943540621500451, 2.8524402084046852]], [[-4.1637530500638356, 2.1876878037737244, -0.66934751727386388, 2.6274605920647716], [1.20250884591802, -0.3479182931587923, 2.8076785765822656, -2.0312446271753117], [4.2488793866047239, 3.9108225386019289, -4.9466753228237748, -3.1586044006187519]]], [[[4.2687212653022364, 3.3558920049098422, 4.9352104459396333, 4.3365404569548467], [1.3889796576450539, -3.6581455684305575, 2.5958360314977931, -0.57057143334283644], [-0.59380304885611146, 0.72599497422901305, -2.334483442523374, -4.2640404450598099]], [[-4.9378232209876902, 4.4437171445037276, 0.52721487379771137, 3.2412393558102526], [-0.49672970331794808, -0.10948652148002935, -0.39445879392992023, 4.8387556313180244], [2.2523185696942303, -3.2251861626092468, -3.5194456448099078, -4.9720205815703489]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[12.286733198724713, 0.62589732073143323, 5.3075574339616054, 20.422261445540137], [-14.08889434703811, -1.4280236333619842, -3.9866957544260471, 21.633701993346371], [-21.021681838757207, 11.42314617279373, 0.67624510238687385, 16.731143885836872]], [[1.9777734182416462, 21.663557231037505, 6.5738389303590745, 9.0742194444183575], [20.90218733026445, -16.068006567759088, -10.565878500577149, 0.39498952865639431], [-10.824266244630518, 10.123198439815567, 16.28680171595499, 4.8982240138101387]]], [[[15.744853494265202, 6.4783938501849292, -14.902090209167381, 10.146425965186399], [-6.8563557031035183, -0.41677266209156533, 18.937977640076088, 12.034321224237853], [17.270620104257652, -17.691366571773592, -6.6678741453880574, 12.727714802422234]], [[-18.578850898532004, 9.7615600709619006, -2.9866583280060373, 11.723845769621619], [5.3656478383448247, -1.5524268648376853, 12.527986414690169, -9.0635036739644708], [18.958688390120795, 17.450263731224336, -22.072284826013298, -14.093833015779635]]], [[[19.047223733460186, 14.974139061835022, 22.021128036540102, 19.349835976458962], [6.1976888759254951, -16.32280787640731, 11.582735576860193, -2.5459150577807996], [-2.6495755572300985, 3.2394217949783615, -10.41656872591628, -19.026337705801669]], [[-22.03278635476061, 19.828063112849456, 2.3524561649077333, 14.462553853218884], [-2.2164299812820811, -0.48853371790269867, -1.7600926447658545, 21.59074237299242], [10.049945416840325, -14.390923792708394, -15.703922661649839, -22.185376495373969]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[-0.5674010034990129, -0.76985125155829692, 1.8436417628370494, 0.1823659011901384], [1.1450860574424393, 0.33307792924557678, 3.8856203815320587, -1.6821667208325786], [1.4654471440736714, 4.302489121185392, 4.9180284999203856, 1.4131851035245138]], [[2.4906282840903637, -4.7293571328683619, -4.0934507572872603, -3.997577325073939], [-2.9576390241143189, -2.4873316088409991, -0.96306236373727216, -2.3988892153095476], [4.5719538162615354, 2.1293014424868826, -2.1646530263379802, 0.95897254199545401]]], [[[-2.918302073995962, -2.2324321622287391, -3.0637503033335944, 3.4626367261459166], [-2.9258992015408216, -4.5696788127804115, -2.9930023419604188, -0.55233252721344606], [0.42155455909835826, -4.1217150298193497, 2.3157634983292361, 1.2175667283250204]], [[-0.45650855743357432, 3.9956118057507748, 0.45692169751079348, -3.783659854106677], [-1.343330523724843, -2.9988499813311567, -4.4459407773500894, 4.1413940856273044], [1.9353990803937693, -2.8805764701445371, 4.2499142520068585, -0.2264218335535686]]], [[[2.2156090221690352, -4.6196876025406954, -4.9020440976646178, 2.5323642401962898], [4.5125610627972996, 1.6919109742962322, -2.0933894072513981, -1.416547119938476], [1.5270796913999565, 3.222569861977103, -4.3935335634803785, -4.839345805420729]], [[1.2806617871134556, -4.3583544244849168, -2.9164739251761551, 0.945298568151407], [-0.5549237517940746, -1.3467122574737078, -0.19185776918046038, -3.7478017458280775], [0.23153565035319801, -1.678759099923969, 3.3540251773055569, -4.6735801629208105]]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array(-4.60221533072)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[2.6113015969673974, 3.5430212322933046, -8.4848363852788822, -0.83928714625728618], [-5.269932608551942, -1.5328963522974832, -17.882461689233484, 7.7416934714377259], [-6.7443033128113727, -19.800981393762751, -22.633826159236989, -6.5037821485814833]], [[-11.46240767215818, 21.765519901322822, 18.83894183072276, 18.397711651182149], [13.611691659505942, 11.447235662785159, 4.4322203748282769, 11.040204723389401], [-21.041115944529174, -9.7995037423310656, 9.9621993434957172, -4.4133981345081814]]], [[[13.430654544607588, 10.274133521794923, 14.10003861549089, -15.935799825772643], [13.465618161463954, 21.030645888630911, 13.774441263042283, 2.5419532243954155], [-1.9400848546161324, 18.969020099081508, -10.657642274325765, -5.6035042632684142]], [[2.100950681624322, -18.388665908020265, -2.1028520412214338, 17.413217386788396], [6.1822963305066398, 13.801353358602798, 20.461176804980674, -19.059587351414912], [-8.9071233188438921, 13.257033192202002, -19.559020524818862, 1.0420420335892981]]], [[[-10.196709808701353, 21.260797107536295, 22.560262498123148, -11.654485529191007], [-20.767777704002569, -7.7865386241145451, 9.6342288232130695, 6.5192548720639421], [-7.0279495669875462, -14.830960423097761, 20.219987521869321, 22.271711456348545]], [[-5.8938813101170444, 20.058085549062984, 13.42224101008224, -4.3504675624512839], [2.5538785978857148, 6.1978597974100689, 0.88297076663948848, 17.248190651138113], [-1.0655769196630311, 7.7260108662508511, -15.435946090606617, 21.508822275129255]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([3.5593939762667262, -4.4904422386032428]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([-3.1596254867448712, -4.0262180884003493])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-11.246351924778518, 18.079499765981332]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Data(numpy.array([-2.55087901605655, 2.996306816764486]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[4.136181195879411, -2.0878275532462984], [0.9798612182640376, -3.1790108670673778]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-10.550897819176477, 5.3257954947206718], [2.9359648477676896, -9.5252919315623625]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_constData_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[3.354752498498021, -0.082241853576001667], [-1.3557587505391409, -1.832531249241669]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([4.0291787624244293, 2.6501136500297839])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[13.516897520138519, -0.33136712981084554], [-3.592914770951102, -4.8564160777214793]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_constData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Data(numpy.array([0.57613119948198399, 0.48605839761545866]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-0.47417479531622497, -0.8903639016333873, 0.83329273200813514, -4.9660447698933909, -1.9403167546793441], [2.8591262733364706, -1.8186881681907332, -3.3313492329946195, -1.6609548583677389, 4.4595359649006578], [-3.9893843679693219, -4.7309532785619401, 3.2390626640676281, 3.5310373601688081, -1.782621486784012], [2.9322208904030456, -4.2910998122473565, -2.9023466256957766, 0.2074525123639841, 4.7920334407846266]], [[0.40773275315192148, -4.4861012201844153, -3.1099327365067841, -4.3538145837960416, 4.1066992713115997], [-3.3704736464354834, -3.6218755986433635, 2.4447698251258165, -1.0283508511898152, 1.7916313261937535], [0.2520489703143749, -4.7530478691940043, -3.9002321638511406, 1.7914005031941773, -2.3892492366390616], [-0.94921900659910641, 4.3444341959590744, -1.3369665942754558, 0.26080406454822835, -1.244778566991914]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.27318689358966092, -0.51296642262350267, 0.4800859412114663, -2.8610933299599126, -1.1178770192484009], [1.6472318493277955, -1.0478029958234194, -1.9192942294985775, -0.95692791483683426, 2.569277804591263], [-2.2984088011128421, -2.7256497870711152, 1.8661250578465931, 2.0343407897297539, -1.0270238554032305], [1.6893439387340377, -2.472236481926986, -1.6721324427745965, 0.11951986478381327, 2.760839974197026]], [[0.19818192865236231, -2.1805071706235908, -1.511608922598346, -2.1162081401147192, 1.9960956673022878], [-1.6382470197915631, -1.7604430498391233, 1.1883009037392795, -0.49983856691581452, 0.870837451527395], [0.12251051863163137, -2.3102588310900081, -1.8957405958897584, 0.87072525807008816, -1.16131465546474], [-0.46137586933369912, 2.1116487238336714, -0.64984384047892507, 0.12676600570591051, -0.60503507565815662]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_constData_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[0.99232793020433885, 0.32634836108849097, -4.4013088147739055, 0.37824946400707837, 4.5100514390538624], [-2.3612466499611751, -4.4023510038054203, 2.6208014065035226, 4.7474072294681626, -0.2719132170642844], [-0.31082893919834209, 0.85309251049179124, 1.2214359297333433, -4.4852978087292561, 4.4421862731207504], [-4.1773010571378748, -1.7422036245227068, 3.2896792342811878, 2.5711979103746971, 0.16363450452273032]], [[0.83455843792930651, 4.3828021589067241, -1.7879090981298629, -2.7870038558653976, 1.1942594733584722], [0.045598581721119302, -4.0191551667011343, -0.60371723795681298, 0.27568596178559002, -4.8162736508428718], [0.85152400096727199, 2.4828281816093458, -3.5450757964986392, 1.5693548121836001, 0.1772774934729604], [3.0993719249204617, 0.36858899511680399, -0.67367042582728054, -2.3021175096017608, -3.6403401276108802]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([2.0012308342220155, 2.6812679969041664])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[1.985877251584635, 0.65309840290810828, -8.8080349110586926, 0.75696449039891567, 9.0256540037619626], [-4.7253996031057417, -8.8101205718836493, 5.2448285850672764, 9.5006577302201993, -0.54416111422154989], [-0.62204045729224233, 1.7072350364400408, 2.4443752446090015, -8.9761162754974269, 8.889840141127026], [-8.3597436793725368, -3.4865516128881957, 6.5834075183433827, 5.1455605391290584, 0.32747041599352977]], [[2.2376748311661818, 11.751467165439088, -4.7938634461893921, -7.4727042459804025, 3.2021297059156955], [0.12226201787305649, -10.776432123067782, -1.6187277093129799, 0.73918794653144748, -12.913720404337784], [2.2831640523893388, 6.6571277451609046, -9.505298279751349, 4.2078608336954355, 0.47532846982043597], [8.3102467527924961, 0.98828587661775258, -1.8062909532314892, -6.1725940036079212, -9.7607274820090826]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_constData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Data(numpy.array([0.35041924800756963, 3.5604546075241412]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-4.7660453028839456, -2.3920430134694115], [-1.3776601343589712, 2.0629710814646494]], [[0.87794589087308061, -1.007262784149682], [-2.4906199539159712, 2.8883501960268756]], [[4.8674974164688738, 1.1912370318669101], [4.0555847389759574, -1.5809935687910439]], [[-1.2211088204576273, 1.5727472931389075], [-3.5536423080175705, -3.4574772106380625]], [[-3.508188820724004, 0.72281536810679814], [-0.15312558736685133, -1.3131194228535579]], [[-3.4019141756655111, -1.7529357313572183], [0.091716422567943212, 0.96766446042524823]]], [[[-0.29858382869973887, 3.2522355645638221], [4.1230736755257755, -1.1695948157029692]], [[4.3413148064520382, 4.0486911297820818], [3.7368989025178525, 4.5290973264749059]], [[-3.417915123341607, -3.1541242670867051], [3.9581194368918045, 3.5061629647303025]], [[4.9188415789679674, 4.2370568561987838], [0.35188552242340965, 4.9685835887714944]], [[-0.94554769053063126, 2.2318176478397191], [-2.1611025133125872, 3.4223523783297942]], [[-2.3530854409401778, -3.681908518724696], [4.7604674979817236, 3.0231330282686546]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-1.6701140110066017, -0.83821791398171197], [-0.482758628292078, 0.72290477502820505]], [[0.3076491388710807, -0.3529642673677425], [-0.87276117132388231, 1.012133503674254]], [[1.7056647843578108, 0.41743238490557188], [1.4211549544629305, -0.55401057748056137]], [[-0.42790003460017212, 0.55112092376767663], [-1.2452646652634012, -1.2115665641550992]], [[-1.229336888426668, 0.25328841774029881], [-0.05365815317580945, -0.46014232070047761]], [[-1.1920962072229995, -0.61426242078779547], [0.032139199826203148, 0.33908825254586611]]], [[[-1.0630941686261841, 11.579437100605137], [14.680016665187244, -4.1642892505059859]], [[15.457054305344935, 14.415180987474734], [13.305058915321595, 16.12564544397285]], [[-12.169331649028068, -11.230116279452565], [14.092704586212285, 12.483534082504509]], [[17.513312163517821, 15.085848605994713], [1.2528724296334683, 17.690416331510299]], [[-3.3665796313835967, 7.9462854274046189], [-7.6945074008558025, 12.185130293995519]], [[-8.3780539000934322, -13.10926814997573], [16.949428437157948, 10.763727919657541]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_constData_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[[-2.2187352212677567, -0.41363863211920382], [2.2212455930315622, -4.0414395009300064]], [[-0.59470172344871131, 2.1274354690207877], [-4.1272065895148646, -0.38822998825408739]], [[-1.2724971095901596, -0.37656419216753356], [-0.79982102026583668, -1.0588079234978922]], [[0.70953271218030167, 2.3331833924680208], [0.45532674292593001, -3.8765336841751208]], [[-4.045806059728517, 2.6665626726282285], [-4.3099111090023348, -1.2376423087106412]], [[1.2138944714652267, 2.9913931279189887], [3.9651953242852791, 3.6891028199187108]]], [[[-3.3784044998979965, -3.6086866568154639], [2.3348193567408995, -0.16823835389373976]], [[-4.7250850318479962, 0.70238725344174213], [3.7177474948854865, 3.3154766445098058]], [[4.7643009207415261, -2.6173833106378455], [-4.3603972919892744, -2.9158758324856491]], [[-2.1993350083882044, 4.9776878401073024], [-1.4955141486855537, 1.91083915863085]], [[1.5412817338632721, 3.6246092780472239], [-0.79303259319713071, 2.4944271714952198]], [[1.3265793879194305, 2.937679772247991], [4.8716944185580235, 1.2248855649067991]]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([-1.0668016340084985, 2.7434126397197449])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[2.3669503594806502, 0.44127036863380681], [-2.3696284281802469, 4.3114142633386212]], [[0.63442877032275535, -2.2695516345990128], [4.4029107335850997, 0.41416438584056059]], [[1.3575019957818737, 0.40171929551341501], [0.853250371333939, 1.1295380228886966]], [[-0.75693065673642745, -2.4890438555263765], [-0.48574331336114968, 4.1354924685670031]], [[4.3160725153998669, -2.844693416345863], [4.5978202135150701, 1.3203188372505625]], [[-1.2949846056729863, -3.1912230768257706], [-4.2300768511103941, -3.9355409163146402]]], [[[-9.2683576071062266, -9.9001165870955337], [6.4053729347453077, -0.46154722655772923]], [[-12.962858000122367, 1.9269380690701112], [10.199315468755261, 9.0957205332438082]], [[13.070443365390721, -7.1805624573953768], [-11.962369045043122, -7.9994506146944628]], [[-6.0336834609903311, 13.65585173712965], [-4.1028124183836621, 5.2422203002593166]], [[4.2283717900496649, 9.9437989074402129], [-2.1756156398867348, 6.843243031140358]], [[3.6393546604098486, 8.0592678186341598], [13.365068044724215, 3.3603665409755727]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[0.017641197345915849, -1.4831716712794618, -3.7863915058034858, 3.8717524716336946, 0.25872158192888861], [0.52872984551942981, 0.61617902848501416, 3.6409791838484757, 0.28776399844138911, -2.5523246338712866], [0.3442489622254552, 0.47179518804008946, 2.9309620281400504, 1.4305843353711776, -1.3190912331588098], [0.35225735725034202, -4.8589239826246358, -0.81036029458247416, 0.63382030237684273, 4.9478378482218783]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[-1.729089492621009, -3.0702221880191427, -1.9573972200420995, -4.9046741960662672, -3.6019335055467838], [1.9897465451053495, -3.2788536298173563, 0.18226642101841684, 4.5681836200836106, -1.1510187784122761], [2.7368354255723482, -4.3723601673729009, -0.4034879251215262, -1.5907439143649826, -1.6158965406470371], [4.4550617449354188, 3.8506412040851128, -1.2130436375575271, -4.8522670509942669, -3.9348227393220969]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.030503208968076727, 4.5536665738036382, 7.4114722074507622, -18.989684441177573, -0.93189793455773118], [1.0520383834163707, -2.0203608441654208, 0.66362824484261795, 1.3145587841297195, 2.9377735821900881], [0.94215275503514284, -2.0628584873446947, -1.1826077873442091, -2.2756933254775742, 2.131514960459155], [1.5693282766580479, -18.709972895011759, 0.98300239947251367, -3.0754653694743772, -19.468864875661961]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Data(numpy.array([[-1.2232662906276879, -4.4714615892405156, 4.8460926777353066, 0.94782486329170901, -4.2134733403251996], [4.5132723449284846, -1.2423492789447521, 0.72566033151938036, -4.8121476357744424, -0.94318375312638913], [0.59528895471204635, 2.7130899176903398, -4.2517022486402372, 0.83342014767083938, 2.447556139788178], [-2.6564246783972214, 1.5436933352275801, -4.582227447751964, -4.1929731639263519, 2.9680869183851142]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-0.050949560862242116, -2.64440755354478], [3.9693998092720335, 3.9037967506614297], [-1.045375795258261, 1.8344802272481093], [-1.3964916543223937, 0.79219790217992525], [-0.12219476524159756, -1.032242250988876]], [[4.7199370029928787, -1.0931076722725477], [3.999375384099487, 0.93238116982320118], [-2.7617368361893724, -4.5896402803233549], [1.6224044486820022, -2.7782355753865673], [-3.3226198637204876, 2.003921905876985]], [[3.4243712921270362, 1.7626910305018759], [3.1774981654455488, -2.7547971751471922], [3.7874491660834195, -0.48456204725491059], [-0.52252204418839732, -4.9502015426417758], [-4.5462172349877239, -4.1422042091999343]], [[0.82629616515543169, 1.6301696709386704], [-2.777760901950963, 0.64738627859578557], [4.6716417624912587, 2.8943852037199136], [1.4240547420267911, 2.848308566057705], [3.8992250026207405, -0.77821998293059558]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.062324880325064537, 3.2348146189325617], [-17.749018779498527, -17.455677222784516], [-5.0659879868827815, 8.890061196717264], [-1.3236295113461354, 0.75086486833366628], [0.51486438567276771, 4.3493252052989027]], [[21.302361145412394, -4.9334926272968387], [-4.9686211246653889, -1.1583430740315184], [-2.0040828681184646, -3.3305198873741477], [-7.807249731995034, 13.369279755720918], [3.133841073276181, -1.8900665841572415]], [[2.0384904070362428, 1.0493105010277615], [8.6208382361498703, -7.4740124411736764], [-16.103106136047465, 2.0602135459194204], [-0.43548039922876297, -4.1255977006689255], [-11.127121906305037, -10.138277344483734]], [[-2.1949935247838748, -4.3304229438561617], [-4.2880109911974529, 0.99936588358609957], [-21.406525110151808, -13.262731324852549], [-5.971023317280399, -11.942881380061506], [11.573238722118782, -2.3098245509621878]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_constData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[-0.63775877346837984, 4.8573049544817408], [-0.1288789095765166, -3.6737614700639409], [-1.448655969674113, -2.5316843606076143], [-1.7280878470886507, -0.689359816503611], [-4.6492099201936341, -4.3940963215860034]], [[-2.8930882107203582, 2.8133890555937047], [-3.7417005553104365, 0.94122581130554117], [-3.7470873624719072, -2.3730361755443816], [-4.0985585727863754, 0.4333567482725309], [0.82478291924922598, 3.3180512283222257]], [[1.4486697173873164, -1.2811591388115762], [2.3184762138244261, 0.54505068045009075], [2.223649693568448, -0.97797412864307098], [-4.4778569617618711, 4.6110483690064257], [-2.3284392712420363, 4.1348985332150825]], [[-2.5979767435684407, 4.2708116536335847], [3.8730498947781129, 4.5301452622542051], [-1.4273145247913321, -0.88916875035609166], [1.733234889326817, -4.8634295073753409], [3.7105688049676626, 0.87545643450415245]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[2.831462994457592, 3.8909892515363023, -2.2335840940534837, 3.5216272735275318, 0.33281365551366537], [-4.0427663401585185, 0.33660094708445332, 3.2529521771415428, 2.1041544481653691, 3.6740106224126592], [2.86721183947372, 3.9639829412772496, 1.9338164623768357, 0.23545119925040048, 0.9218838826906337], [0.31031495724350933, -0.048579150694584072, 1.6542180553364991, 4.6119831284132236, -3.9425693734424838]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-1.8057903664663799, 13.753279231410568], [-0.50146645191194505, -14.294566392726999], [3.2356949316197245, 5.6547299190171314], [-6.0856812933588671, -2.4276683310730514], [-1.5473205487900399, -1.4624152594661883]], [[11.696079637409699, -11.373874575724592], [-1.2594599506239179, 0.31681749950577809], [-12.189095993692552, -7.7193731936727366], [-8.6240002519947581, 0.91184952952012599], [3.0302612065061787, 12.190555458565228]], [[4.1536429651799613, -3.6733546510505062], [9.1904001613570898, 2.1605715994357171], [4.3001303839818705, -1.891222469748612], [-1.0543167917185872, 1.085676868284166], [-2.1465506359819582, 3.8118963143321265]], [[-0.80619104210007231, 1.3252967356923873], [-0.18814947448606892, -0.22007060936340311], [-2.3610894575538568, -1.470879001080039], [7.9936500671524406, -22.430054834242107], [-14.629174928516584, -3.4515477264592271]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Data(numpy.array([[2.7345686778481113, 0.86574139709459352, -4.5614832340147533, -0.17091820806295566, -3.2239301569092604], [4.3113668923357515, 3.4350299681613059, -4.289825034884891, 0.67258013514791681, 2.5419535849865902], [-0.69571176293878878, 1.3068290832574005, 2.84443523575891, -1.5299575632830766, 3.8830423960516463], [1.2550124441934134, -0.71514619525156942, 0.82214090015905228, 4.114903097398475, 4.8132216696572687]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-4.8372440688543588, 2.998004698332787, 1.8772196397494314, 4.6695489310676148, -4.6184575741384926], [4.1795902336032036, 1.8287330929270968, -2.1842921382979661, -1.9394113678522884, 0.059041682244219018], [4.0100558323472981, -2.4056048426274304, 4.4124821527362972, 4.3521509770256017, -1.9029180421113443], [-4.9854267214303425, 4.2226823812405687, -2.4210564576203875, 2.9316583588762724, -4.9509216283862791]], [[0.13479225475627565, -3.4566758860229827, -2.0469978976746086, -0.26681142070548525, -3.7847395182216759], [1.2195529544294237, -4.8350375712121441, -1.5763367459025721, -4.8148154660293585, 2.7788810506033883], [0.29054817959764279, -4.7758769887800243, 2.6687640939722446, 4.8595484548187091, 1.1999139212342191], [4.537982558230496, -1.1484111285148924, 1.8190377963828066, -4.1853159292127735, 1.0825796665153575]], [[0.44768722754785983, 4.8544808084386659, -2.7956858946719452, 0.95981718505519353, 3.686799167294307], [3.5059189704210443, 2.9387065998493291, -4.7030800125413634, 3.567793929033634, -3.8543088323583641], [1.5621668455110775, 2.4085297794876537, 2.7626175687091656, 2.1969630438345256, -4.5998733288322686], [-3.6977999443858001, 0.71088553215178241, -2.358424582097518, -3.4016235469589464, -4.2312649339701593]], [[1.9065594248202764, -4.2035827363235683, 1.7946047704471884, -2.6223977459557526, -4.8030939396177619], [-0.69271383309481749, -4.8117110516781114, -1.4669309951759937, 4.0386758974389387, -0.09925441729632567], [-4.854001228953674, -3.3391723671536253, -1.0235329492771017, -0.93530851966750994, -2.7491556361215777], [-4.9652487750946941, -0.58864983907961843, 4.3385720776768224, 4.0626373482125047, -2.4717944169021355]], [[3.2384361159214432, 3.8833251875681682, -2.1274126060099907, -2.3309544685293391, 2.3729680512490168], [0.96124835602620085, 2.5915693737714998, 2.1423114619577817, 1.5212417329756978, 0.10188574572366349], [-0.57476090168465532, 1.2678978893398902, 2.1064350564993672, -4.0489323945020361, -3.8970240876188935], [-1.1580934528119204, 0.085730530347479572, -1.7203062226943224, 0.51575450987518234, 4.4751378383707063]]], [[[3.4768755976229464, 0.86560855239781187, -0.88416508126940041, 1.5518133002925776, 4.2911509295863546], [-3.2799775074368211, -1.3584825807318532, -1.2749207534280926, 4.4916276642196564, 4.1618771418821261], [-2.4640691667021608, 1.3864687369146935, -4.0198528773466666, 3.5031228221332249, 2.2087455943733705], [0.46329527725591202, 2.1735099832826155, 3.9604276739057482, -1.2841974061073058, -1.5248513290806534]], [[-4.865936611479281, 4.4309721093689838, -0.37022487154025807, 1.8005696759764938, -4.2462057845499359], [-0.18122716195270705, -3.1858463850654228, -4.2264154569547996, 4.5314518739004015, -4.8540014814497514], [-4.3653170501760465, -2.7643109030877722, 3.7614870333916794, 0.65996018956537128, 4.4214849006129811], [3.7800997199777164, 4.9868109224817108, 3.8873680573399803, -1.4819558726020876, -2.8284882374812836]], [[2.5673910162905056, -3.6975909735637269, 1.6723504361799151, -3.5733549401660767, 3.6223240970460111], [3.5372179759914388, 3.5510356298989585, -2.6247932626441086, -0.81808418082144918, -1.6010075156419843], [-4.2837148139608505, -4.9062496633386772, 3.8496930720591926, 3.698120815989018, 1.5759628865543851], [-4.2292385555822074, -3.9886785541799474, -2.8650165038319866, -4.3357295379413019, 4.1442824722156324]], [[-2.5861566468673125, 1.238229199743456, -4.4316194276357157, -4.1781139785621306, -4.7609634363667421], [-4.4165213564373706, -2.4890384445771083, 4.5694379613589771, 4.1353742966430449, -0.35945367464701228], [-1.4085355459640669, 2.6691337810853399, -2.2745580274725317, -3.1283860572763844, -2.1192766398364791], [-2.0693929148473433, 2.3553779193328817, 4.1719211406464396, -4.7681223749625996, -4.1933263097257667]], [[0.15825903493891857, -3.9947980082936949, -1.8149828232209342, -0.07896091248303172, 2.9222988564544892], [2.7123224692204921, 4.1516612965104809, -2.4254606535359002, -2.9770403215790884, -4.5973638719103018], [-3.4770950996643926, -2.4145722181499618, 3.1423638471115218, 2.5612139277526724, -2.3320511395049848], [4.6441339213335713, 4.4870894198825155, -1.836441891215034, 1.7494803906428675, 3.9953163543048511]]], [[[0.15587834881211293, 0.67974694363440591, 1.6428594341111289, 2.5702993042290405, -0.6053478589568897], [-2.4231122204980315, -3.8523363166677047, 3.1131955176314854, -3.5634875414485512, -2.6595818682273253], [-1.8151306522359132, 1.5203141989594595, -3.5963675204343835, 4.3744210382754325, 3.8484343284795912], [-1.1435043961165436, 4.3853122863649077, 1.472677005242363, 0.025080711367447606, 4.3788920532671014]], [[4.2208565354599674, 3.448580040861339, -2.8529528357798406, -0.22098691580635066, -1.5564422260747399], [3.7364843413283779, -3.3110699445211789, 2.5288846565076026, 3.0523732601865952, -2.7798463517334482], [2.1498146965134701, -3.2168380855148184, -2.0836177039577075, -2.4893302433457762, -4.805251011088064], [0.83840893758592827, 2.4870845287648313, -3.8361097940950204, -3.5937671993217393, -2.3561713362272343]], [[-2.576838817635918, 1.6616533475426092, 4.1952441148354769, -2.9885413737563091, 4.5156708745530612], [-4.7181363897041866, 3.4457314228920719, -1.7375793742123049, 4.9314469797157159, 0.52070837550164306], [2.9117105532374143, 1.1576549041162467, 2.6623414195320318, -3.114524366355095, -0.60863489263971893], [-1.1849329809985729, 1.7610516631070041, -3.6600113743278206, 2.8659271493040848, -2.1132372907914032]], [[1.2109924548353561, -0.83549357759071974, -1.417743435273958, 2.0178882475729418, -2.8745808977023692], [4.0416466274336127, -2.1104996811931942, 2.9059790112885402, 1.0548023028221776, -4.0880949558202424], [0.6013552269508704, -2.4262971270275635, -0.41084709925423724, 1.0751259424776247, -3.6592077109873635], [1.2448212106139369, 3.5863607799978752, 1.3807267679777393, -2.7636395273529315, 2.3186526416479527]], [[0.17413036339369192, 2.4509308511250802, -1.2455375116847023, -0.58241800058595139, -1.3179060639996565], [3.076892589809713, 1.1392956989156575, -1.6632773566220682, 0.94357629077277316, 3.0455909493377025], [2.8395768299243676, -4.968753289079765, -2.6841801893594064, 0.99709939078519216, -0.37000006077811065], [0.66904210197291469, -1.5779448425043405, 3.2795459240843599, -2.815949585659554, -3.9214841062602614]]], [[[-4.7953154574718413, 0.17528246900380395, 1.8496985835514579, -2.576093770091167, -1.6859495252328074], [2.6709918756188191, 0.69464046462070783, -4.9075715997239309, -2.4722262048030776, 3.1055963322417632], [-2.7139255729176304, 4.58243902059173, 0.51970703554544162, 1.1483996112594568, -0.33032650758863102], [2.4050012482847363, 1.3567675435366988, -1.8592148099861374, 2.7736413395404256, -3.1179065798808736]], [[0.49744014860793229, -2.4234078143061701, -0.76119158652815067, 0.90736392207939964, -3.0078563707372075], [0.28529889820057441, 4.7807288716380842, 3.3468733180461232, 3.3065547362198497, 4.6958976213229988], [-2.363970182692742, -4.3074182914899559, -4.0702224564470466, -0.6421693293623898, -3.8293051812431522], [-3.7358849348226011, -4.5990174501763716, 0.78000304490480676, 4.9463621988697355, 2.7349969172939579]], [[-2.7537556556942944, 4.8878577930913369, 2.8785479999228638, -1.0738158439929091, 4.0462589471074413], [-1.815211519268817, 1.9711112411732348, 0.48630405642596042, -1.1571075886262863, 2.676208607456144], [-3.2898632614246157, -2.3177015283270972, -3.6864287038546206, -1.7318674801300959, 4.5697294918010041], [-2.6206093888510029, -3.4614960907649648, -0.17372067745222619, 0.52180661112163573, 4.7006557477187663]], [[-0.99617009151617353, -2.277231608792011, 3.75472288539261, 3.6346936569175199, -4.3857770040234385], [1.6070899071375866, 3.0349699484063954, -1.8622333787093757, -4.7402078503517995, -2.0312066550622152], [4.5660140156166467, 3.2301241437072132, 3.9354582049774312, 1.8203206646765269, -0.94729687997392276], [-0.83017428478283151, -2.9508629781783702, 1.5396116440210426, -2.2768924830227855, -2.786201254512374]], [[-0.82505789763992965, -0.9622925535904967, -2.4519315179500776, 2.5748056846948222, 2.2936809034866057], [-3.805079187609496, 4.0254346578125837, -0.99089670271980257, -3.6117414795546186, 3.384396473523049], [-0.86228690643859984, 0.14174965917766613, -4.9161027968151227, -3.7504026130130907, 1.6730866519748044], [-1.6513630723143233, -2.9021718633206817, -4.2437480975338904, 1.7145570121445264, -2.5081442944612062]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-13.227776117795683, 8.1982497441023146, 5.1333860283001105, 12.769202246576629, -12.629489422209494], [11.429376539051191, 5.0007962360627385, -5.9730968646594933, -5.3034535799914293, 0.16145353495250231], [10.965773075559058, -6.578291653928706, 12.066235486436485, 11.901255743040265, -5.2036600744697354], [-13.632991758130416, 11.547214976241536, -6.620545156310615, 8.0168211223346511, -13.538635211465886]], [[0.11669523495022845, -2.9925874108687291, -1.7721708197825115, -0.23098969212236015, -3.2766056781443527], [1.0558174785985683, -4.1858921819060519, -1.364699976689238, -4.1683850683129133, 2.4057923631090694], [0.25153958692815415, -4.1346744166183385, 2.310459555231418, 4.2071122685236224, 1.038815154562565], [3.9287193599533672, -0.99422705483946172, 1.5748163232083217, -3.6234012598389236, 0.9372340329552048]], [[-2.0421177825421104, -22.143632817539359, 12.752474336117613, -4.3781899973485014, -16.817272588792534], [-15.99219060338986, -13.404860884901218, 21.453020625437325, -16.274432189706545, 17.58136511751766], [-7.1257978745324957, -10.986468207758183, -12.601633721661459, -10.021410090201208, 20.982245068060024], [16.867452449056515, -3.2426924362140115, 10.75791418992608, 15.516448777883031, 19.300844054979425]], [[-0.32586572045582107, 0.71846882873680007, -0.30673063154606534, 0.44821552356709127, 0.82093620931750999], [0.11839740705298755, 0.82240903066954263, 0.25072521704748907, -0.69028324733731272, 0.016964387146620817], [0.82963719198814656, 0.57072535740723551, 0.1749404175838343, 0.15986125616758651, 0.46988075501207505], [0.84865142322597065, 0.10061097567203559, -0.74154096506849698, -0.69437869556611931, 0.42247467243693138]], [[-10.440491855343234, -12.519569181286327, 6.8586296567045277, 7.5148344054741338, -7.6502832618039047], [-3.0989975632723183, -8.3550386578243856, -6.9066625276980584, -4.9043770988892561, -0.32847252819770745], [1.8529890039535188, -4.0876142413244727, -6.7909995022191723, 13.053475249921936, 12.563733478276347], [3.733612407039522, -0.27638924215506394, 5.5461471104628837, -1.6627565179485553, -14.42753183344904]]], [[[14.990086340361652, 3.7319560545306025, -3.8119600587442419, 6.6904364859676964, 18.500726047834394], [-14.141186433169052, -5.856916822382142, -5.4966511266816305, 19.36505480421599, 17.943379319479543], [-10.623506225745039, 5.9775754095925766, -17.331060607453026, 15.103247755130969, 9.5227126291738013], [1.9974359197366518, 9.3707989820859012, 17.074856752767534, -5.5366461799144879, -6.5741935359324968]], [[-16.714638083604608, 15.220521983769375, -1.2717335286994562, 6.1850107967417483, -14.58584412090892], [-0.62252073235237115, -10.943477806658091, -14.517863752539897, 15.565672986128586, -16.67364055427927], [-14.994994887880232, -9.4954907934215402, 12.920820684550586, 2.2669830289504667, 15.187933137378304], [12.984755820761617, 17.129844964278803, 13.353225774235829, -5.0905628338808091, -9.7159418603399619]], [[-11.013658256021573, 15.862018327158072, -7.174090768225267, 15.329067480854038, -15.539136595974785], [-15.174046227252937, -15.23332154490879, 11.259903849487889, 3.5094379995311509, 6.8680421216398475], [18.376387051236531, 21.046952633185825, -16.514509717142449, -15.86429125845863, -6.7606050447904584], [18.142693434236968, 17.110733117829611, 12.290419523496841, 18.599521116350498, -17.778246700945267]], [[-1.7393975870637004, 0.83280836250755053, -2.9806191935633635, -2.8101164643647181, -3.202129431465834], [-2.9704645307963076, -1.6740778134420322, 3.0733132016008424, 2.7813706033234005, -0.24176140107350283], [-0.94735302786515696, 1.7952063592102481, -1.5298225455192545, -2.1040903171978091, -1.4253833688370421], [-1.3918325663421678, 1.5841803993093286, 2.8059512846024335, -3.2069443913541513, -2.8203479761146713]], [[0.40228712121950211, -10.154591118479448, -4.613602094175536, -0.20071497456005488, 7.428348054566702], [6.8945978242747108, 10.553330316314891, -6.1654084034994998, -7.567498318087595, -11.686285575690222], [-8.8386143539312076, -6.1377305061353189, 7.9877430464973864, 6.5104869255684914, -5.9279657544367588], [11.805172870491702, 11.40597303702576, -4.668150048993609, 4.4470979508583772, 10.155908729980769]]], [[[-0.10844640085606254, -0.47290794450814605, -1.1429566331660743, -1.7881874602255279, 0.421147626146119], [1.6857876747212086, 2.6801156903020091, -2.165886741944536, 2.4791601996715817, 1.85030239022447], [1.2628077460312808, -1.057700471578958, 2.5020351878172056, -3.043336172375128, -2.6774010312206902], [0.79554945935049559, -3.0509133417840606, -1.0245587155565803, -0.017448945921205893, -3.0464467100971078]], [[5.5159380767961572, 4.5067046933385928, -3.7283217389587704, -0.28879212859509357, -2.03400396744436], [4.8829464063837964, -4.3270025001997441, 3.3048200173275366, 3.9889301493690512, -3.6327840594322516], [2.809440369017985, -4.2038575662808215, -2.7229322139219407, -3.2531291598364827, -6.2796417736419121], [1.0956571833002298, 3.2501943947094087, -5.0131398454919314, -4.6964394945301446, -3.0791132273192012]], [[-7.3296511297549332, 4.7264653313669438, 11.933100182848229, -8.500712387035783, 12.84453334866898], [-13.420433393990921, 9.8011598722358944, -4.9424319969373967, 14.027181552380236, 1.4811212508316551], [8.2821720939595718, 3.2928744001173547, 7.5728577433373063, -8.8590628502901243, -1.7312225343367578], [-3.3704651231651837, 5.0091974025333919, -10.410665316416447, 8.151944166598625, -6.010966611446765]], [[-1.8527670653540926, 1.2782697181093576, 2.1690872915923229, -3.0872833862342559, 4.3979867857087962], [-6.1835478257595948, 3.2289749495480495, -4.4460245670627794, -1.6138027609711967, 6.2546117970765751], [-0.92004797769329527, 3.7121316402678204, 0.62857862685693311, -1.6448970671754879, 5.5984325130488717], [-1.9045236261139884, -5.4869798000195429, -2.1124533614949401, 4.2282511970616845, -3.5474401457155702]], [[0.67615558349758531, 9.517068404709633, -4.8364749637443714, -2.2615537884988819, -5.1174851205242202], [11.947704374328262, 4.4239335005287899, -6.4585764921562046, 3.6639467409798341, 11.82615877730948], [11.026197217442254, -19.293879677017788, -10.422785473924511, 3.8717792074961692, -1.4367259225430895], [2.5979188467043364, -6.1272267220753918, 12.734615863017943, -10.934451626260115, -15.227289040051295]]], [[[-6.0181805729601923, 0.21998167984872022, 2.3213947403640098, -3.2330297388735407, -2.1158876344491504], [3.3521280422411239, 0.87178242733928279, -6.1590634284237105, -3.1026746518889166, 3.897562043604835], [-3.406010366626365, 5.7510179956000984, 0.6522387969443979, 1.4412558030374967, -0.41456387767068192], [3.0183064948980372, 1.7027601510162858, -2.333337722961295, 3.4809543968525229, -3.9130115575830215]], [[-0.35574242964233804, 1.7330908779439795, 0.54436326696311277, -0.6488978565836242, 2.1510570393959081], [-0.20403042155760562, -3.4189200630813046, -2.3935037193896807, -2.3646700389986823, -3.3582533171800377], [1.6905842818408718, 3.0804338025160574, 2.910804103555602, 0.45924495260076498, 2.7385130308231616], [2.6717038970360405, 3.2889698313892062, -0.55781620984831159, -3.5373721068578781, -1.9559226394275453]], [[-2.2639751535905885, 4.0185078058615504, 2.366572043807623, -0.88282792458538273, 3.326594973051531], [-1.492359632430746, 1.6205311701317899, 0.39981045470103765, -0.95130547449408531, 2.200220553547398], [-2.7047311431478289, -1.9054772207988508, -3.030763812959206, -1.4238390890703467, 3.7569615178726461], [-2.1545101619152276, -2.8458375119585488, -0.1428228741368136, 0.42899855697648615, 3.8646013477673296]], [[-4.0991433951156244, -9.3705874005119583, 15.45032083097499, 14.956412186944492, -18.04704737835505], [6.6130192366782827, 12.488607241208767, -7.662909898130037, -19.505495965725185, -8.3582185563719058], [18.788705215625789, 13.291647843922409, 16.194029157343873, 7.4904431413358914, -3.8980348655606063], [-3.4160867358334372, -12.142515208904664, 6.3353527227729467, -9.3691919308337646, -11.464948172168485]], [[-3.9711865516423779, -4.631727371491607, -11.801689914712954, 12.393110516729839, 11.039994627940793], [-18.314689600563902, 19.375309324772921, -4.7694054819228899, -17.384112354592297, 16.289850445072783], [-4.1503780235319985, 0.68227253122047493, -23.662292511893252, -18.051519126893851, 8.0529369284994576], [-7.9483765241351048, -13.968796501604718, -20.426100303616931, 8.2525429647168558, -12.07225446872792]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_constData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[[-4.7132302641617274, -0.69168344689024508, -0.40909130024405105, 3.1613428874957101, 2.4470242064979688], [-3.1262104357247003, -2.7177988898267849, -1.2508443999272645, 3.883386664804263, -1.1596541692743143], [-2.2370866878633358, -3.6152715613379049, -0.026594647981752928, 0.1525516392339652, -3.5640296625875023], [-1.5539788781466468, -4.5169839236645331, 0.30600070068645202, 3.7746618300448613, -3.6795513836981595]], [[-0.6645212542982728, -1.8966916200798134, 0.91596415482942639, -1.1773310258463185, -3.1595850573322712], [1.1642573025774219, 4.2121221905200255, 1.392305515062934, -4.8806182458536984, 1.0529073762244003], [4.8898351520217656, 2.5394661812493329, 2.700334333385829, 3.3466235366147998, 1.1656773050688649], [3.0762518973369684, 2.0759119255296623, 1.4971411134289045, -2.0020274152531004, 4.383854826174975]], [[1.7532919347990195, -2.8557826723746271, 4.8190672037158269, -3.460894586990845, -2.4352501961949047], [-1.8256004299138686, 1.3780963653227545, -4.5194173991042019, 1.5483004902877431, 3.1992389655201077], [-0.54615776631699475, -4.0761465798232868, -3.8883324210859884, 0.77820728342924106, -0.30147013534149281], [2.4381050217646472, 2.4209974679054334, -2.5827452117547889, -4.6001963563796258, 3.7321769533270324]], [[-4.3484097914598516, 0.82806666446779253, 1.7448217009883429, -3.4512001823103953, 2.7025018578854016], [1.7137011839493628, 1.1477599376387726, 3.1242336501124228, 1.5317357053087077, 2.8525473820766312], [-1.4110685650340047, -1.2445517148268204, -3.6123563491809985, 4.6591337693005155, 2.5530995999226711], [-3.1369877999114637, 1.9503459640816168, -1.0326769346212283, 3.3171384701714661, -1.1698519898434698]], [[-4.9490560937110404, -0.54229228584417388, 2.6538524384613069, -2.2176884698503265, 0.73320887504017307], [-1.3722693584107737, -4.5545734912695046, 4.1098835236714883, 4.9494870014248313, -4.5253147181405939], [-0.44153279397101919, -4.6383283768511463, 2.9910858655033943, -4.4894375068336787, 4.5027680882929584], [-3.1950049145546622, -3.289700107782866, 2.8600947353563919, 2.052888295670579, -3.8105874796678241]]], [[[3.158076637753144, -4.2236591254854741, 2.198602700466747, -4.1398900891956778, 1.550199859585546], [-2.1367376070000068, 3.0611612432572031, 3.9670515370008079, -3.7039654333861973, 3.3120877273579215], [1.3947275167787527, 2.6266560454714147, 2.8127816760845095, 0.93173851297846078, -0.47801025975846123], [-3.0067860568909586, -0.18602790758014542, -4.75025569945723, -0.40980669966763728, -3.9102516375065233]], [[4.8929121980486876, -2.834946578523688, -1.6373888995429233, 1.3179924063785862, -0.53564721705263985], [-2.6838321789568012, -1.3580044119075971, -4.8802259499552294, 2.9787253390899151, -2.5553724695657176], [3.6902515749058775, -4.9832868552586076, -2.8049934623814377, -2.0146042505964337, 3.2226228679783393], [2.5975228162228916, -0.20580344841885889, 2.5854492578476549, 1.9895854749686235, 4.4907025652152086]], [[4.2093954084029441, -2.0430657488867556, 3.5374455908392672, 0.85236799086624426, 0.17731668379339638], [-1.0467098404032216, 1.8323575200842814, 4.6124354611031979, -4.5424419482918594, -4.2197856093828321], [-2.9489916711142872, -3.8888300053778169, -3.110811244559982, -0.88623227980499308, 2.0798708802147479], [1.6033512332933739, 1.4546549509486253, 3.2469707850358205, 0.02832974837519675, 2.1776662009148424]], [[-1.0233966025765051, -4.3184570391430146, 3.1802262925741616, -3.2925151773619765, -3.9946885574916635], [0.42294511738838114, -3.5764991607478089, 1.5432617826322907, 1.6884667330665728, -1.2075924683257333], [4.9955495961627321, 2.1950314594886811, -0.90243274060019285, -2.7582043602959274, 3.1159200572077079], [-1.3710337636722878, 3.5424488853093479, -4.5732711016847594, -2.0746033260383303, -3.9252119201185423]], [[1.7857652692785484, -3.9651663494323119, -1.4244855188638281, 1.2082323797224888, 0.12945684113432065], [-4.3906750576809506, -2.474143991357546, 3.9474206956583568, 2.0001095288939554, -3.6002459535695519], [0.88390087599592793, -2.4448273153469255, -2.1347110684081061, 0.75002199257956725, -0.045949671619096577], [0.61442528736005286, -0.7250539091652648, -0.099363475833662562, 3.6631564505415106, -1.2729612061113391]]], [[[-0.82598715975992132, -0.045209661337143103, 0.0067481949508163552, -4.6844106947334296, -2.093033992257797], [-4.8029508010137612, 0.89240865180956419, 4.3352433248699036, 1.2855042295637968, 4.8260414704475139], [-2.689893803660631, -1.7526996569124966, -0.38469089766754472, 2.8517944089646949, 0.74170729452643158], [-4.5547095488079457, 1.9018273556720313, 1.0003636315959197, 1.4337734567580647, -0.69016919665098264]], [[-3.9951376502719285, -3.8128036210092917, 4.6258183685004166, 3.7883667551476066, 2.4057272724789813], [3.3527575624750998, 3.0922312556558413, -0.28316590116344109, 3.4788137710176503, 1.6875523322673214], [-1.233759958588454, 4.2489839735912831, 2.431100597858836, -1.8948626016138439, 3.9531634701366976], [-1.7527607191203165, 1.86785573762665, -0.014456473628542632, -4.3547443795604215, 3.9369102171022234]], [[3.9850454581590018, -1.3021444957028416, 0.065504477771797909, 4.1964668390573152, 0.02337131407512949], [-3.3092612276674691, 1.1105216983362389, -2.6081252575053493, -1.888917805576761, -3.2549925476035813], [-2.0838670585117915, -4.177218168651252, 3.5486509310341852, 0.4040014891488255, 4.4424119153009087], [1.3174498685502218, 2.0105897377322126, -3.7147180600491749, 3.7381374288182894, 4.5358227333295247]], [[-1.9317465003269541, 4.2446929281892327, 4.1270674443277979, -1.8638191712531516, 2.2101004437231158], [-2.3971271653773552, -1.4775060645882698, -1.4993391479796569, -4.0482091644509275, 4.2036809679163429], [-1.5513912843047475, -2.5085581718592742, 2.775634614727398, 4.7377706913842381, -4.1384376608626106], [-4.4285350390071105, 3.5878350233819454, -4.3649339104850462, -1.8825138633092009, 1.265055649447743]], [[-3.3465187202902094, 0.87801886656171302, -0.51026942902196737, -2.5495467431432428, -4.4433404093190116], [-2.6835129954701822, 2.7254261192195264, -0.53842726122199203, -0.98138689276651458, 3.0047237605855734], [-2.8934409083625399, 3.8416751756470742, -4.8540059601185135, -0.89628996458817767, 2.1704422949291322], [2.5765896129923815, 4.011948810112143, 0.016342848289822243, 3.1050546833344868, -0.83835391423843575]]], [[[1.3816608079416977, 1.3605973937681597, -3.3074013055763265, -3.1457865613535754, -1.5004240797115465], [1.9671343503063383, 2.7095223266497612, -1.7954856001881039, 4.9947192396706956, -2.8892702744882035], [2.0008445403730803, 0.34831686883598145, -4.9607664422661095, 4.8983791328716855, 3.3768281809702643], [-0.69197875274346998, -2.3296394815339747, 4.6097950557630245, 2.7150994121102219, -2.0887840424315627]], [[3.629219773201882, 1.6778017684054172, 0.49805692615968233, 2.9950720120272125, 4.1070039104226339], [1.0644857918522685, -0.3769261806321218, 4.5796643694179515, -2.8004552638468603, -3.0919666960144863], [1.7188929001252244, 1.2460091972043772, 0.1641767858439005, -3.4519179172054324, -3.2189448588686851], [0.44095441260111734, -1.2520651085210965, -4.1448583827323766, 3.3982603030821465, -2.3260328065686586]], [[1.2402073844312991, 1.4177528215936297, -4.0683970534640235, -2.1628942061667997, -3.9031842298643982], [4.4155513220084028, 2.9996756677587291, -1.1840233684421655, -1.591479919533251, -0.039049702709342604], [-2.338079137100757, -4.3127145011834145, 2.9223763021391758, -4.5170228957123326, 2.6559225543480895], [2.6073946607258822, 1.9413509201761041, 1.3939148218498474, 4.508239489055919, -2.0437317533784305]], [[-4.0399131397476822, -1.8033440777960053, 4.3510559773348607, -4.866808316969486, -0.88336432570910439], [2.8800070455314986, 1.7024924786636211, 1.3033786886738756, 2.435263441397697, 2.609758521648649], [-4.2590286814442688, 4.8702853655772866, -2.2469000965865318, 3.1951403996667445, 0.23288196641083658], [-1.1032394794715095, -3.0002279049500702, 1.1276212090624549, -4.4807381314511376, -0.96624775997191303]], [[-4.3775052452806085, -1.1319198937556054, 4.8662655439953344, 2.3494381737084149, 3.1436049046498109], [-1.3057103464635977, -0.11114952640345344, -1.6907246420340929, 0.40116273435484651, -1.8597706116689672], [4.4596717406690374, -1.1917144960300052, -0.7851087950536515, 2.5326371509290677, -4.7285171408015225], [2.0542792074873271, -0.57485053805826603, -3.184517303338823, 2.1770469853652532, 1.5585510781524858]]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[-2.6962993577191483, -1.5535166430415401, 2.7468823381016332, 1.2030998694977431, 2.3329717500614819], [-2.6340332200772201, 4.2403418187757111, -0.057471646266153087, 0.83254910254952019, -2.8370109090108384], [-4.0544154947968636, -4.7406250544751449, 2.5286036192866472, 0.71163471817502533, 2.6941458532022775], [2.9243748792774777, 0.98676317385305623, 0.69678000184069333, -1.5195073742040686, -2.4854542821970105]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[12.708279734041717, 1.8649856335951345, 1.1030326100965262, -8.5239267970846804, -6.5979097963036821], [8.4291991899394088, 7.327999401049774, 3.3726509521304768, -10.470772970086839, 3.1267747917906661], [6.0318553996479691, 9.7478543888156963, 0.071707132271967261, -0.41132488688554358, 9.6096908901266751], [4.1899922510559264, 12.179140852204398, -0.82506949272248997, -10.177618267956944, 9.9211720325599515]], [[1.0323448282072063, 2.9465419985114121, -1.4229655589569921, 1.8290033430214254, 4.9084679716710422], [-1.8086930963366752, -6.5436019254974482, -2.1629697898487916, 7.5821216732659273, -1.6357091325458064], [-7.5964402903953721, -3.9451029770119828, -4.1950143286913679, -5.1990353621256302, -1.8108990938402922], [-4.7790085207010957, -3.2249637257987405, -2.3258336366935453, 3.1101829094211277, -6.8103914331408015]], [[4.8160866492354666, -7.8444989842025459, 13.23741058801183, -9.5066702150366975, -6.6893457527863207], [-5.0147095773611543, 3.7854685661071303, -12.414307832108552, 4.2529992708455007, 8.7879330097537238], [-1.5002311221131919, -11.196695047629966, -10.680791652149065, 2.137643842233834, -0.82810299023465561], [6.6971876227222076, 6.6501951851782106, -7.0944972059857925, -12.63619812313868, 10.251850955763988]], [[-5.231571252628056, 0.99624689595663263, 2.0991947607559052, -4.152138488948224, 3.2513796325393347], [2.0617536707676063, 1.380869831187945, 3.7587650967307131, 1.8428310271619397, 3.4318993831125235], [-1.6976564064447786, -1.4973200056913398, -4.3460254522790027, 5.6054032298179779, 3.0716337954817057], [-3.774109612689494, 2.3464609748620431, -1.242413485276129, 3.9908488605692338, -1.4074487763123535]], [[-11.546008056097488, -1.2651525831507238, 6.1913627677620067, -5.173804550597886, 1.7105555923630831], [-3.2014656466473297, -10.62569128871065, 9.5882421567687217, 11.547013351620645, -10.557431397559442], [-1.0300835350601043, -10.821089070702252, 6.9781188262276164, -10.473730877109423, 10.504830747065817], [-7.4538562069636258, -7.6747774176316383, 6.6725202200860325, 4.7893303998313232, -8.8899929412030154]]], [[[-8.3184787753915543, 11.125258446811038, -5.791192550780897, 10.90460802240986, -4.0832779279073703], [5.6282378394263217, -8.0632004067523564, -10.449345534218523, 9.7563679975569606, -8.724149101670827], [-3.673758612151043, -6.9186992814883679, -7.4089603756310805, -2.4542301956106156, 1.259094903741528], [7.9199743595157788, 0.49000368842755793, 12.512331316231496, 1.0794444607347649, 10.29973271205353]], [[20.747620208983633, -12.021142530909115, -6.9430886243309997, 5.5887383175959506, -2.2713272945791392], [-11.380365823006462, -5.7584028978937001, -20.693826180669578, 12.630813621989828, -10.835652745247675], [15.647928074876321, -21.130839647308377, -11.894131079928485, -8.5426106520873599, 13.66502251323147], [11.014384622853983, -0.87267696877873735, 10.963188608374038, 8.4365224915381898, 19.04211388296541]], [[-0.241920883906103, 0.11741835201851276, -0.20330282168247726, -0.048986991659656393, -0.010190681728061396], [0.0601561376909555, -0.10530860322740933, -0.26508425924598372, 0.26106161683676499, 0.24251802586145338], [0.16948340616411198, 0.22349746245827609, 0.17878344344812272, 0.050933228094598972, -0.11953360350697445], [-0.092147234920237076, -0.083601414780227659, -0.18660875639411206, -0.0016281572774284327, -0.12515406158473527]], [[-0.85202792302729735, -3.5953275323371749, 2.6476945457870062, -2.7411805560433877, -3.325774373504522], [0.35212257790939816, -2.9776111665497007, 1.2848412121294863, 1.4057314632992954, -1.0053800257501491], [4.1590403330269003, 1.8274714716652649, -0.75131956829799451, -2.2963405648125477, 2.5941564472443268], [-1.1414529295104541, 2.9492626402918458, -3.807472751423302, -1.7272091372394616, -3.2679316614113714]], [[-5.0662355498759188, 11.24922018938215, 4.0412809567446448, -3.4277684418928263, -0.36727047054415074], [12.456393036562648, 7.0191734939449759, -11.198875576037912, -5.6743325526886803, 10.213937045398946], [-2.5076364276846839, 6.9360017642869085, 6.0561985886599787, -2.1278205749462784, 0.13035971964884271], [-1.7431312430125891, 2.0569858499228095, 0.28189526489733552, -10.392414811599687, 3.6114048284854636]]], [[[3.3488951390338775, 0.18329875143983168, -0.027359986170499791, 18.992547304719359, 8.4860294493465513], [19.473158148377202, -3.6181954655874762, -17.576877710067212, -5.211968266970362, -19.566777316314639], [10.905947116919736, 7.1061726467111734, 1.559696736210608, -11.562359439681522, -3.0071895475318251], [18.466684968986165, -7.7107982992652291, -4.0558898083737578, -5.8131133191083579, 2.7982326849332475]], [[18.939449640956063, 18.075072373550203, -21.929270455164414, -17.959226354993451, -11.404650982148013], [-15.894166502250474, -14.659108964793219, 1.3423833656284414, -16.491751722739433, -8.0000528670844275], [5.8487933708926425, -20.142839881270195, -11.524936404159101, 8.9828331239985442, -18.740465790965935], [8.3091813795616449, -8.8548037079380499, 0.068532721082828416, 20.644210311578956, -18.663415212413984]], [[10.076600368522667, -3.2926072846683914, 0.16563485957324992, 10.611201237456724, 0.059096789357857386], [-8.3678099174449354, 2.808069185709368, -6.5949149656809452, -4.7763243997163896, -8.2305859366214804], [-5.2692737862651358, -10.562528979801495, 8.9731315877979707, 1.0215596276589154, 11.233098847392004], [3.3313085058448086, 5.0839844877302633, -9.393049531269817, 9.4522678319008087, 11.469297779939689]], [[-1.3746978763457633, 3.0206708556914674, 2.9369644776335346, -1.3263584306639458, 1.5727842064073978], [-1.7058789147630116, -1.0514446118751641, -1.0669817920212858, -2.8808461878575904, 2.9914853209008645], [-1.1040238993853995, -1.7851770876567314, 1.9752379568083769, 3.3715621107411171, -2.9450559184728751], [-3.1514992844120497, 2.553227965722896, -3.1062385132406369, -1.3396622225766213, 0.90025752057046815]], [[-9.0160095329336603, 2.3655108883806029, -1.3747402662154273, -6.8688507855847396, -11.971007138132926], [-7.2297754087604131, 7.3426954773044626, -1.4506015730722892, -2.6439994275339735, 8.0951640595999752], [-7.7953518247507674, 10.350033243819697, -13.077400028872432, -2.4147358913620551, 5.8474881084981565], [6.9417082212474854, 10.808775250023441, 0.044030016949538529, 8.3654701990719182, -2.2586477215613794]]], [[[4.0404941584269247, 3.9788968391460129, -9.6720812937169427, -9.1994591955910732, -4.3878024869714745], [5.7526382781996777, 7.9236590268960265, -5.2506729852945355, 14.606431473536887, -8.4493094101564452], [5.8512195112065273, 1.0186091012525322, -14.507140765725717, 14.324696885346951, 9.8751115040657016], [-2.0236052815167644, -6.8127391775709629, 13.480768859690908, 7.9399685155161812, -6.1083875819225231]], [[3.5811804220149579, 1.6555929980879998, 0.49146423321682542, 2.9554267645064312, 4.0526402136755513], [1.0503953784896283, -0.37193687430886285, 4.5190441483486135, -2.7633861243870257, -3.0510388704072025], [1.6961402136410508, 1.2295159900834898, 0.16200360627232077, -3.4062254798618636, -3.176336245395242], [0.43511757570278864, -1.2354917403549486, -4.0899936129164454, 3.3532781222481876, -2.2952435146960215]], [[0.86415170360688198, 0.98786181363965742, -2.8347777064013338, -1.5070614289541278, -2.7196607148694807], [3.0766678582766906, 2.0901140173024104, -0.82500380484255598, -1.108911381261805, -0.027209051925694266], [-1.629126785452752, -3.0050132180729645, 2.0362533651837333, -3.1473712215888932, 1.8505937223074007], [1.8167804564999941, 1.3526944977337376, 0.97125197213430636, 3.1412511194826696, -1.4240314148809061]], [[6.1386778069905148, 2.7401946244382653, -6.6114616431350113, 7.3951511264728262, 1.3422786070237889], [-4.3761919434447849, -2.5869498758563352, -1.9804935288203829, -3.7004007573333784, -3.9655473185370305], [6.4716254884011972, -7.4004345274728447, 3.414181265863069, -4.8550393989109528, -0.35386586528041036], [1.6763805245700167, 4.5588684258644552, -1.7134287424793078, 6.8085146326173627, 1.4682205965854846]], [[10.880089157222564, 2.8133351470388548, -12.094880534630969, -5.8394211696007039, -7.8132862717973977], [3.2452833719268912, 0.27625706636363301, 4.2022188015596438, -0.99707163596011505, 4.6223748306767876], [-11.084310225038855, 2.9619518973140289, 1.9513520166566332, -6.2947538520278874, 11.752513176047108], [-5.1058170530776579, 1.4287647314401728, 7.9149721683139536, -5.4109507523201614, -3.8737074512168634]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[1.0023130356440539, -0.40420028464498614], [0.75895970283950565, 3.0528816932396019]], [[0.34611117676776271, 3.7708006267406162], [0.70884201005376823, -0.81636059711064135]], [[-2.086287064738388, 1.0222557288247982], [0.43298532874069817, 3.2633686573324532]], [[-1.2712949654482664, -3.3071422160706723], [-3.7111958793061808, -4.318631868220101]], [[-3.210400448771753, -0.77934933873441814], [-4.5397205741647468, 3.3122637124076153]], [[0.89825214159873124, 4.2640484279077846], [-1.7926124278503242, 4.0459952180189358]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-0.5267948314619364, -2.7329215433414875], [-1.8820217480956867, -2.7105701930913351]], [[-4.5408436299647494, -4.9537172715177391], [-1.1611844670829607, -4.9691024635342842]], [[1.0866547139098959, -1.6790317653640949], [3.4429610282706804, -3.9635768347368994]], [[2.857950403500336, 4.9029244601276751], [-4.9517887392324216, -1.7980350609285245]], [[1.3912459087428308, 0.6082746503483456], [2.7464455001737722, 3.7074780795099667]], [[-4.5776594170036242, 1.089712367672413], [4.6152240757666565, -1.3362775285459048]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.52801332668421119, 1.104647665731044], [-1.4283786666721894, -8.2750501207294693]], [[-1.5716367322854987, -18.679480192134907], [-0.82309633169029961, 4.0565794542348073]], [[-2.2670736734672094, -1.7163998410222605], [1.4907516126671927, -12.93461241340937]], [[-3.6332979594708186, -16.214668464293744], [18.377057964234112, 7.7650515143029972]], [[-4.4664564897798495, -0.47405844651789253], [-12.468095142961062, 12.280145107307538]], [[-4.1118923748331051, 4.6465863082452223], [-8.2733080355333346, -5.4065724904428922]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Data(numpy.array([[[-3.6566775744031355, 2.8607013035023954], [2.3081609235297371, 3.7368071867394743]], [[2.0846931736258414, -1.3283097178151659], [-1.7035717037935019, -0.025768786178618441]], [[-1.9005841322942576, -2.9537166297711459], [4.2181833333360448, 3.812694099922318]], [[2.8772805908029175, -0.1065746562510137], [4.5276509577831252, -0.68227411153722706]], [[-0.45305469491554096, 0.36409374520960824], [-0.071800078297314052, 2.5342210843384096]], [[-3.0355620209813061, 3.5291601267283319], [3.5921037522814014, 1.282198234998102]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-0.47752544627563154, 0.070341156240405489], [-1.2031244772394256, 3.1794379360347449]], [[-0.55739496190502447, -3.1680909699255846], [-3.467463172503209, 4.6361227517957158]]], [[[-1.0735560794480992, -1.6759542836739705], [-1.4004135027020359, -3.7764012440695183]], [[2.6990625250810787, 1.1177332937905424], [-4.249324366000133, -1.6774251640091142]]], [[[-0.98029278219333804, -2.7435380184079405], [-1.6272973490843698, -4.8578344855443003]], [[-4.0887256195651664, 1.4949117711872919], [-0.55364526717399265, -2.832950713847775]]], [[[2.7332213189856338, 4.0786735128846114], [3.7232059851325197, 1.1357272685084023]], [[3.7122909801896924, 2.8562273120123987], [3.5553601569066995, -2.1112626277294932]]], [[[0.54474374233541933, -4.1682940269757021], [-3.2605025518202266, -0.12673284749605074]], [[-3.7352701052385862, -1.7734174309950621], [2.4300707886880684, -0.2181431669529319]]], [[[3.6497085663603919, -0.23401484430780162], [-4.2477066711507163, 1.9798820416859257]], [[3.429526953794479, -0.35082186378631164], [1.6520825474993437, 0.39438452852930084]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.746156590602951, -0.25721492858187794], [-3.4417797603144629, 9.0954222480195597]], [[-1.286557270041524, -7.3124637789696578], [-12.95724130276445, 17.324296817516618]]], [[[-2.2380350303299736, -3.4938504544841131], [1.8601828645986895, 5.0162304708668231]], [[-4.5980465444975644, -1.9041388116894771], [0.10949993099105079, 0.043225210381984833]]], [[[1.8631289068392491, 5.2143248241321629], [4.8065752415730048, 14.348666504627959]], [[-17.24699426303388, 6.305811918030102], [-2.110880043604197, -10.801174472058131]]], [[[7.8642446514861142, 11.735488134844847], [-0.39679939801721509, -0.12103974323618572]], [[16.807957812025517, 12.932000324879258], [-2.4257301922483747, 1.4404598335558914]]], [[[-0.24679870999092346, 1.8884651787097484], [-1.1871285853577112, -0.046142637085915233]], [[0.26819268601774698, 0.12733151039926699], [6.1583366291281711, -0.55282301309647386]]], [[[-11.078916711693736, 0.71036657372661594], [-14.990837013863043, 6.9873207571434506]], [[12.319216639275352, -1.2601885332891647], [2.1182973264748264, 0.50567914639082812]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_constData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[[-4.794385567384567, -4.6234933875055262], [3.4258425985180292, -3.5754588652628829]], [[0.79394234473512793, -0.20850574996494764], [-0.59115062756726289, 4.0426023171879137]]], [[[3.1477225115973955, 2.1684766886902009], [0.16459678195934213, 4.1248476992007514]], [[-3.2826865863438282, -4.4935247136319312], [-3.4088212048474995, 2.3853146196546779]]], [[[-0.34222068150175833, 1.4510252768017198], [-0.34439924675498634, 4.093689036874121]], [[1.4877819395892367, 1.3556067326451142], [-2.6056511158950668, -1.3699625980467465]]], [[[3.434740169475873, 3.7851147969209613], [2.8363652043734859, 0.10520597377119856]], [[2.1799788358031265, 3.2035541178990847], [-4.067899826773866, -4.2178007529603434]]], [[[-0.38702414137855534, -2.0942157237550263], [-2.5175781692290622, 1.5554932678382807]], [[1.8251457162218738, 3.9860865541528625], [-0.27346372145524622, -0.65201567691829432]]], [[[-3.7401613393611122, -0.020461539562192321], [-4.222705349947856, 0.65194423792806067]], [[3.4500935588699129, -1.5972716384526695], [4.2312644735188485, -2.514195147053794]]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-4.2736946197517511, -1.6002209475654174], [2.6397649118633524, -2.5541806377925056]], [[-4.7121183131356474, -4.7839996159649081], [-4.774668537335863, -2.4797335714028423]], [[4.6615849597445749, 3.3973797539218147], [3.5584124567520146, -3.2176626509955364]], [[0.16852110251376473, -1.0876853490664562], [-1.963085122945285, 3.7625555085194158]], [[-0.7290269840473238, 3.6307115029221713], [-3.1468264666691415, -2.8068701591598133]], [[3.8157136605451836, 0.003184658616723901], [4.7193761077882268, -3.3698755248936707]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[20.489739804346872, 19.759398814640164], [-5.482105089210493, 5.7215241733521429]], [[2.0958211436743084, -0.55040616267922216], [1.5099054869511914, -10.325536564856487]]], [[[-14.832440891567423, -10.218118716384843], [-0.78743094168255245, -19.733269808890128]], [[15.673740361750344, 21.455091071919526], [8.4529683805702298, -5.9149447407157068]]], [[[-1.5952907818021351, 6.7640776065481054], [-1.1700550281913138, 13.907816252727832]], [[5.2941417867650129, 4.8238078839012726], [8.3841062771403987, 4.4080774849958271]]], [[[0.57882620020838926, 0.63787171871828507], [-3.0850728773989253, -0.11443099630520254]], [[-4.2794840209006999, -6.288849429397799], [-15.305698901333187, -15.86970945688828]]], [[[0.28215104254271328, 1.5267397730336103], [-9.1406000185256975, 5.6475473002584433]], [[-5.7434168453347985, -12.543522667042225], [0.76757715936552184, 1.8301233468463463]]], [[[-14.271384715243165, -0.078075376023242951], [-0.013447874978597556, 0.0020762198349410957]], [[16.282289111364722, -7.5381256081612831], [-14.258834588663271, 8.4725246908630236]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[-0.036691835968081676, 2.9833539168204588, 2.411529435348049, -1.790383270043904], [4.2308661697922787, 1.1712799986845699, 0.13186843099498802, 0.7582328421868203], [-4.3174982551266128, -0.96186945533325829, -3.3777499091262539, 2.2146753673900887]], [[-1.3056429424349481, 1.3931331747821805, 1.2021762963640601, -4.2470516800720404], [-3.0761037667498679, -1.0853259504658519, -3.2979672547022809, -0.12392430560210688], [1.0070694637276612, -2.8515009451295659, 3.4214435836265569, -0.23869249898523748]]], [[[3.7625137191405162, 3.8021341282998105, -1.9002627976957367, 2.9652361052313578], [-1.8609716124236666, 1.6023991185623103, -1.3562608423349309, 1.2336750395747123], [-1.0401336031013422, -2.5714480458076561, -4.3803086190183818, 4.8471095998378537]], [[1.8547434976296469, -2.9186129990894596, -0.48496565028150407, -1.0560548412806225], [1.7725389172561714, 2.5225365963813751, -2.7697533143048734, 4.6909796651388351], [-0.2603367196359887, -0.72568032394905835, 2.071140087903764, 1.8270969376472435]]], [[[-0.86726421718235525, 2.8482771691792319, 1.817899608030662, 2.4086264015760523], [2.8832459480826289, -2.4854829651088481, 1.611485036011179, -3.0375403490824326], [0.016483805118033601, -4.7409630333124184, -1.1749755888651925, 1.7428768216251989]], [[-2.1392838726328089, 0.08292792125027848, 1.2119563532452435, -4.5283107936796574], [-1.3731838657224706, 1.1760139794538302, 4.9096497023042573, -3.6595321721430674], [0.12293732695602166, 4.7319038109173484, 4.7518176134683756, -1.0109558360721547]]]]),self.functionspace)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-4.3105358642248532, -3.1172581342347727, 2.9715106910369924, 3.0403397451789758], [0.069710981063034971, 1.4749200018746569, -0.58065229689667319, -0.13826086769642743], [0.52689528380087758, -0.34025981708910891, 4.9241059842842354, -3.5740476436138846]], [[1.412853698296102, 1.518446371446057, -3.4998864227654449, 0.40457444674564158], [-0.94869138591760471, 3.4781624860374265, 3.991257086473059, -4.0676487406796831], [-2.6007215394489522, 0.41141977611006997, 2.3263847492388203, 2.7270594683429508]]], [[[1.1131723055083222, -4.9352260555539242, -2.0309184070859554, 4.2346495295752398], [-3.2646405730450034, -2.6949591789910796, 3.3774161919292354, 0.17704947809898375], [4.6874097234763905, -0.49725715876304921, -1.2245946699716814, 3.3130384232829186]], [[1.1164323992163494, -2.9206329315281963, 3.5239623627107353, -3.3355388696433308], [-4.1651904404295728, 0.32225279517240679, -4.471307711799807, 3.5506414177152035], [3.7352537653098548, 3.3898478320179475, 1.4353597123875659, -2.3448612469197396]]], [[[-3.8409939776443345, 2.7298672031026863, 3.966698033836197, 0.64068993112501182], [0.10312137441911862, 0.37970980562277123, -2.234773662368883, -2.2408111381386542], [1.7357361283226727, 0.83796129133106323, -0.87544255682415617, -0.16472355002223793]], [[-0.4672150773002528, -1.5325411484915308, 2.7332620301454726, -1.9688026329217765], [-3.8399304060392101, -1.3380428426407054, 3.379392913612234, -1.3908420551073322], [-1.7476727100265474, -1.0918669290276437, 2.0740696800212941, -0.16571781397298313]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.1581614748646715, -9.2998842645097444, 7.1658854988871292, -5.4433734150179847], [0.29493783144262486, 1.7275442978555939, -0.076569707345400237, -0.10483393067667809], [-2.2748694684447304, 0.32728552493529528, -16.63239854094412, -7.915355278190261]], [[-1.8446824598734208, 2.1153980141891275, -4.2074804974150215, -1.7182485837652932], [2.9182731457042963, -3.7749400060332401, -13.163035176286579, 0.50408054562201421], [-2.6191072460378337, -1.1731638804228588, 7.9595941733298385, -0.65092863938013201]]], [[[4.1883260712423409, -18.764391416696032, 3.8592786941409267, 12.556735678097485], [6.0754034312032825, -4.3184002129767132, -4.5806573293815793, 0.21842152190044592], [-4.8755323648917637, 1.2786709491651103, 5.364102587680927, 16.058660346126302]], [[2.0706957329895901, 8.5241972395269485, -1.7090006987995572, 3.5225119715665349], [-7.3829621534447902, 0.81289446910858743, 12.384419354034454, 16.655986688701745], [-0.9724237122687428, -2.4599458728767973, 2.9728310408879048, -4.2842888034547535]]], [[[3.3311566352238549, 7.7754184294885462, 7.2110588008868204, 1.543182683331646], [0.29732428495463542, -0.94376225356018983, -3.6013043157793536, 6.8065542467694904], [0.028611536075601098, -3.9727435055473088, 1.0286236337221126, -0.28709285730957751]], [[0.99950567991932204, -0.127090451674917, 3.312594282518798, 8.9153502132846096], [5.2729304790701788, -1.5735570880536112, 16.59163541228542, 5.089831247034863], [-0.21485421136465011, -5.166609282480529, 9.8556008370859036, 0.16753339117710694]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.22450432416)+(1.-msk_arg1)*(-0.618809867859)
      res=mult(arg0,arg1)
      s0=numpy.array(-4.51120276111)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-10.0351900492)+(1.-msk_ref)*(2.79157678449)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.75740398627)+(1.-msk_arg1)*(-2.3741906724)
      res=mult(arg0,arg1)
      s0=numpy.array([-2.2645344301096912, -4.1426959236190628])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-10.773305124854639, -19.708478100939196])+(1.-msk_ref)*numpy.array([5.3764365212949841, 9.8355500204457087])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.8825072874193722, 3.1437955915256985])+(1.-msk_arg1)*numpy.array([-2.2366499681989627, 3.3548396554124764])
      res=mult(arg0,arg1)
      s0=numpy.array(2.94380944674)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([14.373171076462775, 9.2547351609405126])+(1.-msk_ref)*numpy.array([-6.584271305426002, 9.8760086698879768])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.80698153217)+(1.-msk_arg1)*(3.95415909129)
      res=mult(arg0,arg1)
      s0=numpy.array([[-0.98980571812178209, 2.5784627061952348, 2.5659266057757542, 3.9566014168411776, -1.3223334682849339], [-4.7278161437232837, 3.9012604770728547, -3.3770115466196051, 1.1534461607000379, 3.7063718199607063], [3.8255088720364263, -0.76232121979982814, 4.8666887836748618, -4.0479556436509334, -2.5799340944246807], [1.0394764831996293, -3.4749542508966522, 0.40523253353168798, -3.9531737940712275, -4.6249939709605279]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.7783663712038669, 7.2376971976784796, 7.2025085953155719, 11.106107107229882, -3.7117656248457935], [-13.270892602925297, 10.950766111327281, -9.4792090452852626, 3.2377020714371167, 10.403717249984116], [10.738132754957809, -2.1398215855592402, 13.660705538593037, -11.362536734770513, -7.2418273572651861], [2.9177912914661266, -9.7541324074016966, 1.1374802378578102, -11.096485833415388, -12.982272662882673]])+(1.-msk_ref)*numpy.array([[-3.9138492789215005, 10.195651751252615, 10.146082015809609, 15.645031463011154, -5.228716905335145], [-18.694537186648322, 15.42620458290574, -13.353240908455266, 4.5609096226449353, 14.655583827596551], [15.126670685171181, -3.0143393817543327, 19.243661698444217, -16.006260609478666, -10.201469854396896], [4.1102553862253712, -13.740521942997823, 1.6023539065505699, -15.631478097273845, -18.287961957432341]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[1.0784544497242496, -2.7690159848610696, -2.4073361320052857, -0.4616387532123678, 0.86253465624952064], [0.58918505431477719, 1.1224858294586406, -2.5977298228308445, 3.152122871394571, -2.0083046510146509], [2.2148192035122953, 4.6174889582312133, -0.37495405650013947, -3.4132544843333665, 4.299304596268863], [-3.7155757616643048, 4.2189992328739816, -3.0400836236450859, 4.2514371229864754, -4.9219449059097355]])+(1.-msk_arg1)*numpy.array([[0.73566221234751517, -1.737433621293575, -1.1296618608334197, 4.4165616442710061, 3.8123936138031631], [-4.4024169945779521, -1.7690902760598082, -0.88944782789157184, 4.4486822946731159, -0.59930477500953039], [1.821149105847498, -3.1342298154925405, -3.5026134587682431, 3.7946095276355472, -2.2020621782276839], [-0.74745917555767605, -3.7053334305478449, -2.3356332000987279, -3.7245519312912476, 4.0898114460353909]])
      res=mult(arg0,arg1)
      s0=numpy.array(4.55344065409)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.9106783349597523, -12.608549957294263, -10.961662211535081, -2.1020446663810843, 3.9275003693289854], [2.6828191790997291, 5.1111726094980474, -11.828608583622616, 14.353004429298142, -9.1446960437301783], [10.085047802734366, 21.025461942226372, -1.7073310442840741, -15.542051731722012, 19.576628332970994], [-16.91865372651743, 19.21096262654719, -13.84284036374186, 19.358666634118357, -22.411784031765563]])+(1.-msk_ref)*numpy.array([[3.3497942253817121, -7.9113008849827358, -5.1438482424949994, 20.110551342322655, 17.359508070488282], [-20.046144519372429, -8.0554475837678261, -4.0500478992144311, 20.256810817699467, -2.7288987267192644], [8.2924943757274967, -14.271529461127914, -15.94894251872166, 17.278529289536802, -10.026959445178148], [-3.4035109972576749, -16.872015879619081, -10.63516716637424, -16.95952618221477, 18.622713705944307]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.63720978815)+(1.-msk_arg1)*(-3.17929764177)
      res=mult(arg0,arg1)
      s0=numpy.array([[[-1.2650625299070195, 0.53029731350616949], [-2.8982723907138652, -3.37899024431215]], [[-1.9203811595129894, -2.5481216854382662], [3.2509186638453986, 0.34330074453545212]], [[-0.2851689170033902, -4.3599319488735251], [-1.1433250271456008, -0.28628815825148735]], [[2.1782246826973362, 4.6221670350767869], [4.2699813748836117, -3.7505470530521237]], [[-2.0538990588256256, -2.6461701494159162], [2.1396128452202543, 4.9047344962158661]], [[0.97997712476531884, 3.5790417001135015], [-4.4981030273544995, 2.8579503711232217]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.336235286488995, 1.3985052658066106], [-7.6433523175072606, -8.9111061463537489]], [[-5.0644479908410389, -6.719931450227822], [8.5733545207633561, 0.90535608376710031]], [[-0.75205025919666435, -11.498055211224761], [-3.0151879526219902, -0.75500193317144415]], [[5.7444354539931455, 12.189624147355618], [11.260836677049108, -9.8909793992155279]], [[-5.4165627018011691, -6.978505819142474], [5.6426079382602383, 12.934813821683488]], [[2.5844052655914047, 9.4386838037261604], [-11.862441331833637, 7.5370146927649548]]])+(1.-msk_ref)*numpy.array([[[4.0220103180212234, -1.6859729982655574], [9.2144705769950903, 10.742815715295427]], [[6.1054632917334866, 8.1012372654493152], [-10.335638041540108, -1.0914552475184294]], [[0.90663686533413701, 13.861521363318341], [3.6349705625772368, 0.91019526639478066]], [[-6.9252245969383832, -14.695244754472945], [-13.575541715556618, 11.92410540110491]], [[6.5299564341518392, 8.4129625157523389], [-6.8024660731032052, -15.593610817312516]], [[-3.115638961752015, -11.378838836956726], [14.300808347293311, -9.0862748751992743]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.9560595889528685, -2.7175015524035762], [2.8995450103647809, -0.072758554706258138]], [[0.15948116439660254, -2.9315039957964597], [-1.7483604653823748, -2.556450705922253]], [[-2.5937483216824733, -3.7920834596148181], [-1.2116278377198944, -3.4907024460450806]], [[3.1105179082188918, 1.7013219399233641], [-2.1978088841546164, 0.85945753233714672]], [[-1.3126841214661953, -2.2681410295645312], [4.1637037323059367, 2.2004467496556313]], [[3.4277859989818964, -1.4374674582399285], [-2.5777653917441579, -0.73067358399664428]]])+(1.-msk_arg1)*numpy.array([[[-0.099424883310867074, 1.6552092983461595], [-1.5779797335516332, 3.7979225628649598]], [[0.92715629856420989, -1.7130816534211135], [1.0127340987907232, -3.0528129800363613]], [[-2.1028682547808266, -2.6476311679399558], [1.4987435881598135, 2.0049340871462924]], [[0.16253805596488036, 4.5548513995420929], [2.9174625913581487, 0.051164968305092096]], [[-1.533085291282096, -4.18809336809045], [2.9133191474536835, 3.8803335792290312]], [[-1.037374286189956, -1.6026512983693317], [2.9193455868220521, 3.278768646761689]]])
      res=mult(arg0,arg1)
      s0=numpy.array(0.424498996194)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.6793433243926055, -1.1535766811499], [1.2308539463180455, -0.030885933437304575]], [[0.067699594198146168, -1.2444205035531593], [-0.74217726253941418, -1.0852107584824469]], [[-1.1010435589330754, -1.6097356220888892], [-0.51433480087233163, -1.4817996843567223]], [[1.3204117296811728, 0.72220945569963646], [-0.93296766514903684, 0.3648388597481575]], [[-0.55723309188169312, -0.96282359027568887], [1.7674880548114653, 0.93408743640631142]], [[1.4550917157343322, -0.61020349308383237], [-1.0942588212180278, -0.31017020295176423]]])+(1.-msk_ref)*numpy.array([[[-0.042205763162130067, 0.70263468563827736], [-0.66985081290653137, 1.6122143155572453]], [[0.39357691805509182, -0.72720144227495553], [0.42990460834770416, -1.2959160455922643]], [[-0.89266546328187335, -1.1239167730814315], [0.63621514872545282, 0.85109250742795661]], [[0.068997241600352824, 1.9335298469166866], [1.2384599414639477, 0.021719477685789561]], [[-0.65079316722844083, -1.7778414307195189], [1.2367010536857177, 1.6471977092790888]], [[-0.44036434316470097, -0.68032386740617001], [1.239259271148212, 1.3918339993014242]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.43682823987)+(1.-msk_arg1)*(-4.26217066876)
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-1.5254061105044361, -0.10772842119364334, -0.24707397490888638, -2.7771643871317639], [-0.99355115567479402, -4.0429781936467943, 2.3824064194948482, -0.47544575272334733], [-0.13061633151305507, -4.3805834268729154, 1.0834687617107512, -3.5902807231491893]], [[1.4018883909582494, -0.5803903934398793, 3.6142374983071068, -2.6048981833147398], [-1.061701170797742, -3.7649977343219709, 2.4125495685005589, 0.51671774594626818], [-2.0791761363740568, -3.1905790836650927, -1.7305422465396267, -1.6920300984171921]]], [[[4.035835601387177, 0.8685294966788053, -4.8651619778645667, 4.2317852072729103], [0.50095823205236734, 0.03646825897134498, -4.4221177608073994, 1.221724065632098], [-3.0888240734863515, -2.796524676441392, 1.0304181034377553, 3.2284970910210227]], [[-2.8796559630532927, 3.0110028029547493, -2.0092259152708514, 1.3930458356730622], [1.2168964470704839, -0.55104370356783594, -3.6794619157643718, 4.4822902972899357], [-3.2237164197369883, -0.57077178631053371, 0.54174453527552835, 1.9483379867473456]]], [[[-3.1476897247618716, -1.5636835264233073, 3.3858927509234462, 1.8008188793126827], [2.064130574445703, -4.516876704922649, -2.0322727608435667, 3.8417568814937688], [-2.9719345408032716, -1.6080968319491085, -2.0667718503169885, -0.46236675669593641]], [[-4.6395307969541602, 1.4014765076653104, 0.25797870514327848, -0.4394114253096193], [3.7621537442128794, -0.93449701743696689, -0.6475019917938436, 2.2620522697529912], [0.98482093549492689, 3.9594294303890027, 3.8944358767599159, -4.4637576128063126]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.7171526873479781, 0.26251565900131596, 0.60207683939498879, 6.7674726053248726], [2.4211135139041451, 9.8520434354584623, -5.8055152418734171, 1.1585796367626608], [0.31828956521927809, 10.674729401711884, -2.6402272755541007, 8.7488974552320311]], [[-3.4161612202334459, 1.414311700883752, -8.8072760014730687, 6.3476894550882896], [2.587183395303335, 9.1746528020436049, -5.8789689186091518, -1.2591523953640114], [5.0665951247807959, 7.7748932126147121, 4.2170342166564128, 4.1231867265335946]]], [[[-9.8346381649343471, -2.1164572046672805, 11.855564099203788, -10.312133698148164], [-1.2207491668607249, -0.088866883320278117, 10.775941439567641, -2.9771317044614944], [7.5269337302628623, 6.8146503050466336, -2.5109519333307535, -7.8672928837392533]], [[7.0172269718792686, -7.3372966605688656, 4.8961384506113301, -3.3946134318018872], [-2.9653676272192309, 1.34279885825684, 8.9662167038620222, -10.922571575732912], [7.8556432089487815, 1.3908728074027445, -1.320138382354838, -4.7477650269180449]]], [[[7.6703792116494087, 3.8104281754083456, -8.2508390726225063, -4.388286300000793], [-5.029931674589065, 11.006852710567975, 4.9522996547428537, -9.3617016595402056], [7.2420940160754874, 3.9186557725396072, 5.0363680102215014, 1.126708369893914]], [[11.305739665766016, -3.4151575313936817, -0.62864979397832332, 1.0707701701161545], [-9.1677224866318596, 2.2772087221650015, 1.5778511389755276, -5.5122328509968765], [-2.3998394668295591, -9.6484494497456339, -9.4900713228527493, 10.877410606822613]]]])+(1.-msk_ref)*numpy.array([[[[6.501541182142172, 0.45915691700357386, 1.0530714488710677, 11.836748593163106], [4.2346845936315898, 17.231863071405311, -10.154222762240988, 2.0264309418448714], [0.55670909703622318, 18.670794194082202, -4.6179287766833337, 15.302389190827718]], [[-5.9750875808200572, 2.4737229113506292, -15.404497055223917, 11.102520632035228], [4.5251515891642979, 16.047062911182092, -10.282698007797245, -2.2023392208009445], [8.8618035436431857, 13.59879258676256, 7.3758664042545119, 7.2117210561360574]]], [[[-17.201420124177446, -3.7018209456989348, 20.736150680829958, -18.036590786939072], [-2.1351694829284145, -0.15543394372847938, 18.847820613924323, -5.2071964778576589], [13.165095366979161, 11.919265450397347, -4.3918178170336599, -13.760405605532902]], [[12.273585181851029, -12.833407830313581, 8.563663762983694, -5.9373991010466272], [-5.1866203436243774, 2.3486423105527541, 15.682494654197352, -19.104286233985196], [13.740029568609099, 2.4327267661695884, -2.3090076682134, -8.3041490199491363]]], [[[13.415990819243248, 6.6646860615475845, -14.431252770559434, -7.6753974071591813], [-8.7976767908971141, 19.251699406135188, 8.6618933521912052, -16.374223496816906], [12.666892229292053, 6.8539831496624153, 8.8089343594438159, 1.9706860285999872]], [[19.774472079595515, -5.9733320639299397, -1.0995492702268548, 1.8728464884735172], [-16.034941339956873, 3.9829857777653124, 2.759763997388625, -9.6412528353474674], [-4.1974749052491269, -16.87576398323662, -16.598750365300123, 19.025296769765678]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-1.881844002900519, 4.6378212275917789, 2.866557686461034, -3.7517429520472145], [1.5757188004891365, -3.3356725105452254, -1.4815677115124171, 3.2205435377500784], [-2.9226693157717842, 1.5556847309384576, 3.7837451131760798, -2.8169805069780884]], [[1.378277807346282, 2.2508270928143261, -2.8509290986591838, -3.8763306658434207], [2.9411187668289163, 2.0034276048851805, 4.7681338764525965, -3.2125046053650639], [3.0019542618088479, 1.9294382069178093, -0.62306332065779646, 4.8016520635385849]]], [[[-3.4006558771036755, 1.2947142980082846, -1.3880048699656546, 0.082477400949722224], [1.6642780804150252, 1.0288155341057266, 1.3301477989171229, -0.3097636729128439], [0.3023617962309002, -2.8068171947450962, 1.2711036188532443, 0.15681870817064958]], [[-2.2722099646209051, -1.8642111127442815, -3.759714094252625, -3.334027260180763], [3.8733089789745829, -0.6187322871926435, -1.997621336094193, -0.9285836306100208], [0.36260609675468203, 2.8148408511881176, -1.277949854099206, 3.9732495538330568]]], [[[4.5495543977655011, 2.9630514940291297, 4.539353403464812, -0.41521940337704066], [-0.26420559414976008, 4.3742507098286261, 0.8264650998953158, -4.5553391045487688], [1.2245827053035461, -0.067765556923522041, 0.6284859809018295, -1.3880287278997381]], [[-0.57688794562519696, -3.3628025441896527, -4.8120218191350617, 4.5275234815914462], [-0.61960721542742103, 2.0376019163609218, -0.88567783881055373, 0.53832526254697299], [4.8808577497220202, -4.6404830514240256, 0.61238680705043613, 1.2987243269913531]]]])+(1.-msk_arg1)*numpy.array([[[[-2.332947790666553, -3.0816947463831812, -2.4092669645431308, 3.1915855119181487], [-4.5611787658091751, -2.0263001820068647, -2.596439661505646, 3.6759054574428074], [2.981574425721826, 3.6797723297498397, -3.773782028215483, 0.91555783285383185]], [[2.5034528135810294, 2.1976997571902128, 0.35499526896295386, 0.6298471641074368], [4.7272361061186725, 3.6727153698512254, 0.59810388268988302, -2.9004290286191203], [-3.126954871428639, 3.9738583498818656, -1.956275000094263, 4.2507296862639716]]], [[[-4.1085832208551896, 3.7904606284376605, 3.1117108693411026, -4.6680905837539832], [2.4160586176997541, -3.4009763122056178, 3.1648534763141729, -0.90959143349554061], [4.3420933985203334, -3.7753462136348235, 1.0900627425194038, 2.3305509901842569]], [[-4.3969298002741164, 0.43800774338868997, 2.4821776628932115, -3.0252126392516967], [-0.60762582215776018, -3.7483801841423956, -0.27317461732494497, 4.7356930832088846], [1.4906647004084324, 2.158358514023079, 3.5244093614395453, 3.5821446283703615]]], [[[3.9397585260309125, 3.1200232692811625, -1.1466858401614521, -1.0893119824894129], [3.3856649894678998, -0.98873775648049644, 1.8501143665982411, -2.9922692911557727], [-1.8605255955520352, 4.3753750863160796, 4.0110000884742156, -3.4914263663688372]], [[-3.864491172551813, -3.546082287456426, 4.9299384187189759, 1.8790758672831851], [-0.38630591947711146, -0.98694601939303084, -4.1046019881725542, 0.86587239914622049], [4.9731791719872742, -3.8397483660486911, -1.8487288667453585, 4.7522620102897601]]]])
      res=mult(arg0,arg1)
      s0=numpy.array(4.49662023155)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.4619378160716909, 20.854520762321414, 12.889821287858309, -16.870163261766937], [7.085409037519999, -14.999252496757343, -6.6620473460044094, 14.481561228448614], [-13.142133975442507, 6.99532343505805, 17.014064826952453, -12.666891539571894]], [[6.1975918732156572, 10.121114643279554, -12.819545463757885, -17.430386896226061], [13.225094150327077, 9.0086531005811707, 21.440487255616404, -14.44541320244606], [13.498648267850461, 8.6759508767605649, -2.8016791332092814, 21.591205813792282]]], [[[-15.291458017538581, 5.8218385065067375, -6.2413307797835236, 0.37086954975654068], [7.4836264873266352, 4.6261927451972014, 5.9811695035682186, -1.3928895986204772], [1.359606170180979, -12.621190984165418, 5.7156702489374442, 0.70515417584635998]], [[-10.21726529725375, -8.3826494054543783, -16.90600646107638, -14.991854430682528], [17.416799517918221, -2.7822041205063357, -8.9825445148657757, -4.1754879400912079], [1.6305019107520564, 12.657270320058149, -5.7464551688544159, 17.866194328780008]]], [[[20.457618349549406, 13.32371729518872, 20.411748352194959, -1.8670839697591266], [-1.1880322199436482, 19.669344239706366, 3.7162996888628617, -20.483629979104659], [5.50648336787948, -0.30471597426485697, 2.8260627769714457, -6.241438059852606]], [[-2.5940460076380849, -15.121245954925625, -21.637834666603673, 20.358553686161475], [-2.7861383405079945, 9.1623220009624298, -3.9825568886348659, 2.4206442667255246], [21.947363704738862, -20.866489973218325, 2.7536709061199596, 5.8398700839611219]]]])+(1.-msk_ref)*numpy.array([[[[-10.490380234671239, -13.857210944061356, -10.833558575980172, 14.351347983626885], [-20.509888718073626, -9.1115023936143231, -11.675203111936408, 16.529150849218365], [13.407007884585797, 16.546538705467004, -16.969264617549939, 4.1169158743685941]], [[11.257076570290112, 9.8822211910635875, 1.5962789085248954, 2.8321835009126315], [21.256585514107417, 16.514806236813627, 2.689446019474536, -13.042127850276259], [-14.060728538023442, 17.868931853409972, -8.7966257439078461, 19.113917106123239]]], [[[-18.47473843392217, 17.044261948742989, 13.992182049826768, -20.990630561636504], [10.864098060969944, -15.292898892500876, 14.231144171499418, -4.0900872423045689], [19.524745023085103, -16.97629816535245, 4.9015981816763681, 10.479602733131546]], [[-19.771323496636807, 1.9695544804990484, 11.161410297277902, -13.603232358413102], [-2.7322625651294281, -16.855042171572105, -1.2283625110104643, 21.294613328389055], [6.7029530503204571, 9.7053185611037538, 15.847930438928598, 16.107544008283895]]], [[[17.715597895589337, 14.029559755570013, -5.1562107481068802, -4.898222298936469], [15.224049688906595, -4.4459781994918561, 8.3192616915350079, -13.455098632869833], [-8.3660770342839825, 19.674400133767733, 18.035944146599434, -15.699618435996394]], [[-17.377149191159663, -15.945385356533084, 22.168060833928795, 8.4494905614511016], [-1.737071013089984, -4.4379214382547314, -18.456836342494906, 3.8934993479453963], [22.362498079902675, -17.265890186852292, -8.3130316248657206, 21.369117501116069]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-4.294499659730679, -1.5615076373946946])+(1.-msk_arg1)*numpy.array([-4.836200315213607, 3.0599935911124394])
      res=mult(arg0,arg1)
      s0=numpy.array([0.41653199172264088, -4.1939981237740671])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.7887964967198233, 6.5489601014922254])+(1.-msk_ref)*numpy.array([-2.0144321496655873, -12.83360737988624])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.5593765039031693, 0.36716537481379685])+(1.-msk_arg1)*numpy.array([4.2294957470960632, 3.0404578594495444])
      res=mult(arg0,arg1)
      s0=numpy.array([[2.0113348301139471, 2.3694570800510197], [-0.070137868988819463, -1.1572383856816759]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.1364282755617618, 3.694875697638571], [-0.025752196955920875, -0.42489786562772575]])+(1.-msk_ref)*numpy.array([[8.5069321099531212, 10.021608643002445], [-0.21325123501209861, -3.5185345450025545]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_expandedData_rank2(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.5641807799062377, 4.6671356747783328], [-3.457959697529839, 1.1398725256130664]])+(1.-msk_arg1)*numpy.array([[-2.095773107081409, -1.2369081116697886], [-0.68467397058420421, 1.7062819692533635]])
      res=mult(arg0,arg1)
      s0=numpy.array([0.95813410726591641, 0.33652301982868238])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.4149631696898006, 4.4717418732426486], [-1.1636830398586184, 0.38359334453905619]])+(1.-msk_ref)*numpy.array([[-2.0080316949853616, -1.1851238493447034], [-0.23040855217909084, 0.57420316097237289]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.8039105935144084, -2.8838919098744209])+(1.-msk_arg1)*numpy.array([2.9743820943815811, -4.5334516844486465])
      res=mult(arg0,arg1)
      s0=numpy.array([[[-1.9789023423817986, -1.0104760605743888, -3.9785773440998984, 3.9792497592161489, -4.1911331789472701], [0.20805159018717845, -0.64194798284186749, 3.8514556886912406, -4.9668990979444096, 3.0773193372485661], [0.10395476685318261, -2.8057334744007401, -1.6798476867825785, 3.3483470080079041, -1.6392362862819168], [-0.74249769407160215, -2.4624232734560159, 2.6084245737796081, 2.0942116390644898, 1.438120729402744]], [[2.9935938937594733, -0.54335823532658445, -2.0711231426551024, -4.1799097439804456, 2.5558726231014726], [-4.7255052116140348, 1.7380308700588145, 3.5776135253422261, 2.7322918868829245, -1.4689368879082778], [-2.186320407236976, -2.012210144124551, 4.9805872433017466, 2.6775407965018472, 3.4181336897778785], [4.9152173692339804, -2.4588510667095242, 3.4683040768499023, -1.6900526957877493, -1.3174593614035377]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.5697628989530035, -1.8228084701628469, -7.1769978181382266, 7.1782107948896696, -7.5604295403326995], [0.37530646753616953, -1.1580167667336505, 6.9476817172814602, -8.9598418996990805, 5.5512089520894268], [0.1875251051727766, -5.0612923370494824, -3.0302950376777673, 6.0401186385077317, -2.9570357020971674], [-1.3393994559957836, -4.4419914287037336, 4.705364721024341, 3.7777705607696057, 2.5942412185222778]], [[-8.6332012116624117, 1.5669864190219787, 5.9728952754567359, 12.054407894670469, -7.3708603804318518], [13.627846249843129, -5.0122931652746159, -10.317450702391753, -7.8796344679971826, 4.2362552071547919], [6.305111734824064, 5.8029965556080354, -14.363475257381651, -7.7217382413903906, -9.857528094819628], [-14.17495560640811, 7.0910606988696872, -10.002214068211904, 4.8739292966437464, 3.7994103939399833]]])+(1.-msk_ref)*numpy.array([[[-5.8860116937101905, -3.0055419013736997, -11.833809213402963, 11.835809232884731, -12.466031482629315], [0.6188249245603582, -1.909398585689225, 11.455700837747306, -14.773455741525879, 9.1531235354063298], [0.30920119715371824, -8.3453234078645835, -4.99650888085442, 9.9592633863948503, -4.8757150583774926], [-2.2084718463661863, -7.3241876933560528, 7.7584513467949741, 6.2289856010789206, 4.2775205470945012]], [[-13.571313280219067, 2.4632883072003482, 9.3893366997703485, 18.949418869691463, -11.586925048435551], [21.422849561462503, -7.8792789754918795, -16.218938062768977, -12.386713256994765, 6.6593544088365348], [9.9115779329329197, 9.122257467346099, -22.579251627689743, -12.13850183408127, -15.495943933594191], [-22.282900461985033, 11.147082510182644, -15.723388959375297, 7.6617722405259485, 5.9726383611475065]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_expandedData_rank3(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.3354781479977369, 3.7901129900025907, -4.5582971640332568, 3.8607031059156487, -1.1762124282360995], [4.2634943637324536, 4.8482455432019336, -0.47087204906022251, 0.63999597365529937, 3.0583842860354657], [-2.8314562590271519, 0.99140793121386039, 1.52662079941401, -3.0543955469987836, -1.9198400785947092], [-2.9756526892761794, -1.0252959062639642, -4.5320725769286598, 3.6994128252170775, 1.7294882298959511]], [[-4.0513995742527795, -4.774747313865503, 3.6372501610823988, 3.4503518535015463, 4.245438823755169], [4.1585364897214241, 4.278154185306537, -3.8398021576720276, -3.8278475099635303, 2.8936519002905321], [2.6655749876862354, 2.9300521892372497, 0.18486064803569402, -3.8983021330022281, -2.8726814277504422], [-1.9400623810517281, 0.99806504379036376, 1.5549313615306879, 4.8514882246885662, -0.37079650721235069]]])+(1.-msk_arg1)*numpy.array([[[0.85597157327108242, 2.2256404687849987, 4.5629598079076654, 0.55934048289223703, 1.1608996548077162], [-3.1943180371541979, -0.14250000560717879, 0.67857909767425983, 3.6766714697443899, -2.7884242908929604], [-0.53514575881052107, -4.8625437396927707, 4.6454809779644179, 0.18914501151876983, 4.6727039200274145], [4.9289679286780306, 0.48513352944669652, 1.5442273085244453, 0.62388993458383801, -2.0226904351351616]], [[0.19647746046422476, 1.1826786311587236, -2.2023297254646557, -4.9147096187952819, 3.6157913447544328], [0.84438782922438893, 1.136385420113573, -2.7756581305243335, 1.5662193744797657, 4.6682749468362381], [-4.4257094395590499, -4.177267265283076, 0.3906032580000538, -0.99742669628578362, -1.7353807891924209], [-3.2145816978512554, 0.73946360862899141, -1.1317568819183754, -3.1228105555823982, 3.6767888787224638]]])
      res=mult(arg0,arg1)
      s0=numpy.array([-3.6701727492225733, -2.8154843721903577])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-8.5716082551860975, -13.910369412381996, 16.729738034293398, -14.169447332170565, 4.3169028014090438], [-15.647760830234885, -17.793898674199529, 1.7281817628314233, -2.3488957821218475, -11.224798663257902], [10.391933602497145, -3.6386383725042379, -5.6029620564056799, 11.210159301941712, 7.0461447393236254], [10.921159411332299, 3.7630130950594634, 16.633489269342491, -13.577484139236208, -6.3475205714653047]], [[11.406652186807369, 13.443226443346212, -10.240620986274354, -9.7144117220916382, -11.952966661372892], [-11.708294497994018, -12.045076250551327, 10.81090296722841, 10.777244843430093, -8.147031703826924], [-7.5048847207321012, -8.2495161484996213, -0.52047226557747872, 10.97560873354411, 8.0879896661128541], [5.4622153149255555, -2.810036533221254, -4.3778849482183269, -13.659289278476201, 1.0439717713191425]]])+(1.-msk_ref)*numpy.array([[[-3.1415635423286998, -8.1684849981016558, -16.74685074278058, -2.0528761978480832, -4.2607022776571721], [11.723699012313476, 0.5229996373435315, -2.4905025124761111, -13.494019436099967, 10.233998845705621], [1.9640773808484102, 17.846375525323229, -17.049717692356836, -0.69419486692757881, -17.14963059247011], [-18.090163773626141, -1.7805238595094324, -5.6675809863517381, -2.2897838364238563, 7.4236233151462185]], [[-0.55317921942467374, -3.3298132033508705, 6.2006249244560188, 13.837288125571746, -10.180204024257263], [-2.3773607372490075, -3.1994753911147389, 7.8148220890343651, -4.4096661722695378, -13.143455157905201], [12.460515762933852, 11.761030703866854, -1.0997373686257899, 2.8082392757980821, 4.8859374917706306], [9.0506045334293557, -2.0819482338984123, 3.1864438141600742, 8.7922243165533303, -10.351941627886406]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.0203157551373696, -3.7153886214580134])+(1.-msk_arg1)*numpy.array([-4.2457798980044235, -3.6557520874216087])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-1.6980546244745218, 1.4237608803400352], [-0.78208971244606662, 3.8666798577646198]], [[1.8130543661544563, 0.46537344585478735], [-3.3094443064341905, 0.97763223319713966]], [[0.86701853681734598, -4.2053093194324651], [-1.597665161322106, -4.6429106635630424]], [[-2.6289553385707984, -0.14238739605565787], [-3.7292210287020735, -3.3281397192572806]], [[1.357981641813665, -3.2005435210908839], [1.602536053101824, -0.95678888796382822]], [[-0.20668844714011136, -2.2557172569166264], [3.5387156834563367, -1.3593131592347563]]], [[[-1.0983371663932919, -3.6542310141641119], [-0.75790967997630165, -3.6271255336392505]], [[-3.8092776068551593, 1.7007408201908163], [2.7850219208283145, 3.3328704455534073]], [[-4.3142481350948447, 4.5415503566568169], [-0.2206547222595141, 4.2609893397113403]], [[-1.4290208365251567, 1.7269760782897308], [0.21565109110877412, 2.4818605682643744]], [[0.49201825159843615, -4.6833843209435368], [-1.520210232603131, -0.31237381872902947]], [[0.16063887527801946, -0.94275556427181151], [-4.0486075408936522, 4.4819050171965618]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.4306065109097461, -2.8764465380992243], [1.5800681679856434, -7.8119142367141849]], [[-3.6629423008624453, -0.94020130468299445], [6.6861224730386599, -1.9751258034583121]], [[-1.7516512099282335, 8.4960526732754182], [3.2277880968531378, 9.3801455632917143]], [[5.3113198900670815, 0.28766749958423016], [7.5342039986763876, 6.7238931101139459]], [[-2.7435517061434593, 6.4661085006627452], [-3.2376288362572714, 1.9330156646936858]], [[0.41757592616204442, 4.5572611132839098], [-7.1493230482385419, 2.7462417917675301]]], [[[4.0807494105420732, 13.576888330204318], [2.8159290010768356, 13.476180936283097]], [[14.15294667648447, -6.3189130913861282], [-10.347438755156659, -12.382908930202829]], [[16.029108431277841, -16.87362451890132], [0.81981804435397687, -15.831231308917408]], [[5.3093677558519792, -6.4163872708078493], [-0.80122761011054477, -9.2210765153747758]], [[-1.8280390135384956, 17.400592815948482], [5.6481718004377131, 1.1605901317472243]], [[-0.59683584937176648, 3.5027032963117173], [15.042150390185384, -16.652018903347688]]]])+(1.-msk_ref)*numpy.array([[[[7.2095661903073749, -6.0449753253128033], [3.3205807795395694, -16.417071612115627]], [[-7.6978297818077417, -1.9758732214753061], [14.051172109823478, -4.1508112833495883]], [[-3.6811698748162955, 17.854817773337022], [6.7833346256833922, 19.712776763586344]], [[11.161965729255309, 0.60454554390230653], [15.833451678878641, 14.130548717772648]], [[-5.7656911566715019, 13.588803344535972], [-6.8040153600870736, 4.0623150271508282]], [[0.87755365401723473, 9.5772789849982924], [-15.024607913571899, 5.771344486571814]]], [[[4.0152483887350119, 13.358962657951235], [2.7707298946504082, 13.259871740941907]], [[13.925774562829139, -6.2174868035757154], [-10.181349700583048, -12.184148088437656]], [[15.771821625527762, -16.602782196478511], [0.80665896149965399, -15.577120673130954]], [[5.2241459060958153, -6.3133964031348668], [-0.78836692647564854, -9.0730669531218666]], [[-1.7986967503305131, 17.121292007486968], [5.557511731158586, 1.1419612398745087]], [[-0.58725590361867908, 3.4464806220150117], [14.800705468772835, -16.384733622241711]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_expandedData_rank4(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.56538391049153169, 2.194002989222394], [-0.84013030532688759, 2.6238726613746692]], [[-3.8353214415262773, 2.5829154129352414], [-3.1701845811093818, 1.6082174448629774]], [[-3.9980289791038448, -4.5100023281014776], [3.2603367199560633, 1.835958087059069]], [[2.2445409670944656, -1.5453858150046873], [3.4514614225259557, -2.9887509732945339]], [[-1.6509588676972475, 2.7844625568101851], [4.2331119744022612, 1.543603499985049]], [[-4.4915674022778438, 0.013945999832288614], [4.2118879187122751, -1.9742617067541612]]], [[[1.8632135295989158, -4.6495705621042527], [0.02922271095905149, 2.2532822394447649]], [[2.4192957069572856, -0.27159690905107325], [4.3929591162053345, 2.3532290782492558]], [[1.6577610871920481, -0.94428541229826024], [-0.10925313418789795, -0.11445985576095552]], [[0.74841496423941756, -0.99230930233058423], [-2.2707580738547604, 3.7707959637875259]], [[3.4812852882276708, 3.4467096028353943], [-1.6538436669701859, 0.24110633670868786]], [[-1.7028420496477215, 2.6453609767847492], [-4.5808714608713874, 2.1284650707391553]]]])+(1.-msk_arg1)*numpy.array([[[[-2.000000507235141, -2.2040233801911002], [2.6650173770746939, -3.4568328544514659]], [[2.4056680163957145, 3.9414719301123604], [-1.6707893949469765, 3.2339971837618435]], [[4.1912697024777312, 0.79187429281073829], [-2.439814305867074, -4.0634299622657313]], [[1.4354464462232848, -2.7554968906011812], [2.5513923384136117, -0.60755790666566689]], [[-1.1811169763002329, 3.9849524897305617], [0.87824917648315903, -3.2459127914988972]], [[0.42312349476280886, -2.8120933075776433], [4.3364970144113073, 3.0353112329792218]]], [[[-3.8085497874945329, -1.4007757311008562], [4.5679305992358756, 0.9969343244520541]], [[-4.260755084158073, 4.3711621722162626], [-0.34801672064921707, -1.4293954401974718]], [[-3.5836966961696417, -2.0719374986479702], [-0.99901660775325229, 2.5330219163457031]], [[-3.9277860309829151, 3.669287135922227], [0.079488701404565987, 2.6411062263817477]], [[3.4729699758886525, 0.47455448360069319], [4.2718499189892345, 1.8355580247106271]], [[-4.9512045301505996, -1.0550146163008209], [4.6592684186580406, -0.48260595729534828]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([-3.7917981250236341, -4.6800457400712805])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.1438216517203199, -8.3192164208297221], [3.1856045165140254, -9.9491954377012437]], [[14.54276465084228, -9.7938938198624932], [12.020699950629389, -6.0980358920617368]], [[15.159718786756113, 17.101018371547408], [-12.362538661675107, -6.9615824321325563]], [[-8.5108462305675285, 5.8597910357728944], [-13.087244950525324, 11.332740336700775]], [[6.260102739025565, -10.558119902111374], [-16.051106047553588, -5.8530328570232282]], [[17.031116854374403, -0.052880416015651882], [-15.970628712982901, 7.4860018379763886]]], [[[-8.7199245420425804, 21.760202902336836], [-0.13676362393724326, -10.545463945891747]], [[-11.322414567318182, 1.2710859572210025], [-20.559249598104074, -11.013219723072295]], [[-7.758397714169079, 4.4192989212379254], [0.51130966524550781, 0.5356773603632331]], [[-3.5026162651942858, 4.6440529232053551], [10.627251650276436, -17.647497587001791]], [[-16.292574383142732, -16.130758594012562], [7.7400640083476837, -1.1283886840176864]], [[7.9693786804680666, -12.380410370352267], [21.438687966265242, -9.9613138872032998]]]])+(1.-msk_ref)*numpy.array([[[[7.5835981733805244, 8.3572117205168652], [-10.105207893547227, 13.107612336029165]], [[-9.1218074739985955, -14.945265874433332], [6.3352960950693173, -12.262664457719872]], [[-15.892448599323426, -3.0026274587341737], [9.2512833103926102, 15.407706112084057]], [[-5.4429231433612904, 10.448287943290012], [-9.674364684996398, 2.3037369313381597]], [[4.4785571361688072, -15.110135378868607], [-3.3301435806923934, 12.307846036795748]], [[-1.6043988740950661, 10.662890131064417], [-16.443121248415384, -11.509287442073788]]], [[[17.824187208813168, 6.555694493133795], [-21.37812414189511, -4.6656982382826753]], [[19.940528681101039, -20.457238903241446], [1.6287341709479453, 6.6896360407734905]], [[16.771864456616253, 9.6967622642413769], [4.6754434193760694, -11.854658429100899]], [[18.382218282213074, -17.172431629571168], [-0.37201075839223707, -12.360497943853634]], [[-16.253658341053146, -2.2209366894071505], [-19.992453015589412, -8.5904955142006241]], [[23.171863669552941, 4.9375166607315935], [-21.805589314589216, 2.2586179545731171]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.63328760428132203, 2.1056948322202604, -4.685277980929504, -1.748495179274927, 2.9416429179488004], [3.6662832754402679, -2.9186676143047539, 1.2904669925962509, 4.7546113290124747, -0.37702307599075002], [-3.0113479492538966, 3.8089527163626578, 2.6708531318137032, -0.77518992652560481, 0.91164459187835511], [4.7194104515824868, 2.8349272643579928, 3.7062718781656638, -1.1306796001887975, -4.987262564933113]])+(1.-msk_arg1)*numpy.array([[1.9280683209031331, -3.8617042627037712, -3.9534623580416026, 2.8875766669512801, -0.64364404705206901], [2.0226686751313663, -3.9500265184904091, -3.2608232168648832, 0.31240388855676393, -3.9512369003433956], [-3.5858774522838823, 0.79007276822464334, 0.93091305836840998, 0.096789495181623941, -4.8296631219659361], [3.6955413407101734, -1.2590236439005364, 4.491426214243802, 0.74512792864552502, 3.0554514927467373]])
      res=mult(arg0,arg1)
      s0=numpy.array([[-3.2188091293394749, 4.2244505753608088, -3.1370720158902685, 0.67345875697904933, -2.111677101201952], [-2.9264416473771995, 2.4556082163972999, 0.19930216120601507, 0.042364340392544442, -4.4378671572503912], [2.1491085722918326, -0.55709752000474211, -3.8244580006389084, 4.5549172855060078, -3.756024365458317], [3.7523165112337988, -3.8137656413074894, 0.86631726794789721, -2.5858836042865763, -4.2449198625915896]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0384319221582441, 8.8954037455071617, 14.698054440640806, -1.1775393900183524, -6.2117999897453746], [-10.729164068330892, -7.1671041746194586, 0.25719286058945945, 0.2014259727765326, 1.6731783264648681], [-6.4717136918949798, -2.1219581121009625, -10.214565628496402, -3.5309259958816095, -3.4241592997334052], [17.708721760762323, -10.811748196414348, 3.2108073277645994, 2.9238058398295128, 21.17052992184405]])+(1.-msk_ref)*numpy.array([[-6.2060839133132371, -16.313578794452233, 12.402296129287866, 1.9446637928067154, 1.3591683954848059], [-5.9192218497496931, -9.6997175737922703, -0.64988911443192154, 0.013234784674773266, 17.535064470549791], [-7.7064399718912888, -0.4401475798012302, -3.5602378939763009, 0.44086814465817936, 18.140332363059539], [13.866840790693873, 4.8016211147016215, 3.8910000871132575, -1.926814093780481, -12.970146730745748]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.8622696911324272, -0.63215830400268658, 3.6823300079627526, 0.41895006405023594, -1.0784383855608981], [1.2310202486872814, 0.25025577288782053, 3.1055465996662974, 0.43303365310680064, -1.9633018341049988], [3.8329442950314316, 3.3100138848167759, -2.3900826562026234, -2.0257428723434034, 4.1196517061243316], [-0.66234155656466775, 0.12357316680700503, -0.94455677671005844, -1.0477525455127621, 1.0995425888016532]])+(1.-msk_arg1)*numpy.array([[-3.7607482088215374, 3.9354587865030961, 0.27512705293091066, 3.5240092956042286, -2.054326542159056], [-4.0351180547425294, 3.8591509059074038, -4.2022629437249046, 4.9758658432511478, 1.1591370629292879], [0.34143245353171281, -3.9587553878989912, 4.4214775064046528, 1.2450118365164382, -2.2597560750459467], [-1.7617598575012403, 0.26961479574418856, 4.9045139925623964, -4.6718459597222042, -0.052133699321625038]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[-3.8303470348802526, 3.9200491583968287], [0.96282196232001649, -1.8416847537244085], [1.4675056405413658, 2.9958547845131704], [-0.72692960859759292, -0.16973448783656941], [-1.4222307064527016, 4.7136164421955513]], [[4.352634507537978, 1.4288095026863648], [4.9800563281786339, -4.6197643050761705], [-2.5718700163969324, -1.7154857764868714], [-2.796705868434465, 4.3093896590357588], [4.0595290286211281, -2.8121654475967803]], [[0.66064996709621049, 0.49232189897634626], [3.2950489387943716, 2.8029427576529473], [4.6723364697202658, -2.940827905027561], [2.1124924489372141, -1.1172355763761832], [-1.6863554528272742, -1.4771776644205192]], [[-0.25244382065767823, -3.183017534079231], [-3.4678343670941389, -3.7722131025323957], [3.1641982711212844, 4.2832453662075327], [3.8091682817667802, 1.6355000584653343], [2.6919893551321064, 2.9261048095141327]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-10.963486224456709, 11.220237893828422], [-0.60865589875676018, 1.1642363104220277], [5.403840057020072, 11.031725972511634], [-0.3045472060819745, -0.071110274550664751], [1.5337881869619872, -5.0833449060746752]], [[5.3581812139142446, 1.7588934293237197], [1.2462878454332256, -1.1561226867264021], [-7.9870621842051976, -5.327521019944701], [-1.2110677588734038, 1.8661107467129248], [-7.9700807874943447, 5.5211295810734642]], [[2.5322345223941229, 1.8870424140004272], [10.906657738560153, 9.2777794461778793], [-11.167270360221401, 7.0288217706830691], [-4.2793665213138228, 2.2632320055725272], [-6.9471971183719496, -6.0854574854787478]], [[0.16720403311953844, 2.1082447880946682], [-0.42853127470398877, -0.46614431895080571], [-2.9887649198418598, -4.0457684369632814], [-3.9910657635076183, -1.7135993494433253], [2.9599569445684493, 3.2173768573581376]]])+(1.-msk_ref)*numpy.array([[[14.404970750590797, -14.742317850933249], [3.7891461514504616, -7.2478744460135145], [0.40375050204163432, 0.82424069787207699], [-2.5617066979478609, -0.59814591292069352], [2.9217262893394098, -9.6833073667596583]], [[-17.563394087061852, -5.7654150210774446], [19.218788890360475, -17.82836760301339], [10.807674065981992, 7.2089223090179235], [-13.916033204363092, 21.442944809655742], [4.7055505551120795, -3.2596851973985581]], [[0.22556733919130448, 0.16809467389488594], [-13.044292739843071, -11.096164743831061], [20.658630603222278, -13.00280443228648], [2.6300781034784291, -1.3909715167656131], [3.8107519792132911, 3.3380612010964512]], [[0.44474538950893988, 5.6077125172633755], [-0.93497945455876375, -1.0170444651428237], [15.518854695956083, 21.00723683214289], [-17.795847447074102, -7.6408043402667012], [-0.14034336361747252, -0.1525486683227707]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.1551295926911553, 2.1284383433048966], [-2.293942242077839, -2.5829179304449168], [1.7042124505955982, -4.8002441505904123], [-1.5062308538961355, 3.7593511626433074], [4.5510885867334387, -1.6140152782167769]], [[0.10096868511237922, -2.7470529127741496], [0.90863274270737282, -3.9988439608313762], [0.031957480807786709, -0.054697742412512795], [-0.51496664462570951, 0.0064318305258703745], [-0.76233290792722386, -1.4398109008773794]], [[-4.7251674790957621, -1.3469922649072452], [-3.1093262070922134, -4.0466299087831992], [-4.33670060531926, -1.9522679867529202], [3.065265070452579, 1.9303145712640779], [-1.0061872983572782, 1.8612736821896707]], [[1.4844856602378718, 3.8393652493259296], [-3.3576898908173414, -1.5007060611964116], [2.442113986429014, 0.23653236656485355], [2.1940533423914435, -2.1257377497350207], [1.1609406172239183, -3.7302126137039449]]])+(1.-msk_arg1)*numpy.array([[[4.2507389068581265, 4.4885669401816735], [1.9606230681181724, 0.85624433313209991], [-4.1646377592260375, 2.1626083707216406], [1.3471213226839049, 4.4855524798333306], [-3.1285493953078758, 1.864476960657182]], [[-3.9560468160014208, -3.0406475988719537], [2.3242566979467352, -3.8749580398704198], [3.7130914647078956, -0.40837310090225198], [0.97205232000561548, -0.036343612468044739], [4.4152549271380028, -4.6247365074125089]], [[-2.5396440333670753, 1.7562792993752474], [-0.94269856685267683, -3.2896733332224404], [2.5927275815856472, -0.94620226885774095], [1.4742780515934149, -4.7511921311259098], [-0.50470843756519912, -3.1388255387455444]], [[0.35609173959662499, -0.38248614486654642], [-1.4175015375719378, -3.2822165456915817], [-4.0782832608297337, 3.8605486889086809], [-1.6989053279540078, 0.90463601639161162], [-2.2333726520215103, -3.3634458384072188]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[0.89961454617941428, -2.1879241724385148, -3.9090703864573104, 1.5032270536259134, -4.2850184553612554], [4.4765603353318451, -4.6646926842429277, 4.8615101965885508, -1.0941055507299193, -3.5763672031703164], [-4.0591414946578341, -2.332843025602461, -2.1485460741100315, 2.3662336344440851, 2.0375025780997502], [-3.8757913683709755, -3.0821822558505287, 2.2414167878103788, 2.2560614830091854, -2.923214116976256]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.8384004766660937, 1.9147740942830989], [5.0189716816199068, 5.6512285754452956], [-6.6618864228550949, 18.764492256837908], [-2.2642069685827315, 5.651158371765451], [-19.501498586136758, 6.91608525439392]], [[0.45199241088468778, -12.297348108382568], [-4.2384925075706681, 18.653378169519133], [0.15536161880433799, -0.26591363246880501], [0.56342786432575054, -0.0070371014797089128], [2.7263824098083798, 5.1492924846649668]], [[19.180123383605363, 5.4676321954681368], [7.2535699565380236, 9.4401523598992085], [9.3176010601492933, 4.1945377185486814], [7.2531333081915106, 4.567575263582575], [-2.0501092144541766, 3.7923499260106692]], [[-5.7535567084204322, -14.880578693360917], [10.349012202125909, 4.6254495930669179], [5.4737952869285191, 0.5301676172789811], [4.9499192374368999, -4.7957950601557995], [-3.3936780012400858, 10.904210171702269]]])+(1.-msk_ref)*numpy.array([[[3.824026552620353, 4.0379801108874585], [-4.2896946037763142, -1.8733976739732177], [16.279862134912435, -8.4537883394926574], [2.0250292167747697, 6.7428038381442663], [13.405891897403544, -7.9893181860118867]], [[-17.709482261227798, -13.611642434832202], [-10.84194321521476, 18.075488420331862], [18.051232016543352, -1.9853099940487831], [-1.0635278389180398, 0.03976374813486485], [-15.790572915052499, 16.539755968414532]], [[10.30877447750048, -7.1289861803026557], [2.1991677769277027, 7.6742914919183711], [-5.5705946666526387, 2.0329591700683038], [3.4884863122030305, -11.242430624376199], [-1.0283447427277901, -6.3953651273993843]], [[-1.3801372906768044, 1.4824364987952512], [4.3689980867450684, 10.11638959698961], [-9.1411325662698193, 8.6530986414792643], [-3.8328348736761253, 2.0409144727239812], [6.5286264648579779, 9.8320723565170205]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.4737933434178156, -2.2967593147507124, 4.4274014670619195, 4.0305282649044187, -2.7004551496004012], [-0.80380222878986185, 4.5764038136685805, 4.9894424983175618, 1.3808229830494199, -3.5314681160134622], [1.2506000339974346, -4.0932112697906851, -1.5785462249410731, 2.9612257805922599, -3.1448117202733519], [-3.3124690468212883, -1.9355206451199471, -1.0315830381343636, -4.2758464470941888, 3.9863982027870559]])+(1.-msk_arg1)*numpy.array([[-2.0399276587843929, -0.085488251496560252, -0.28425929349163503, -0.3016463453506848, 0.3564013848790859], [1.2212574831453749, -1.0445977432602707, -1.5144486484224915, 1.3736995762393214, 4.8401236399732053], [1.8118492871837031, -3.2347598378192819, -3.4868895687010539, -4.6121974670095334, -1.8477488466362435], [-2.1558773414954691, 2.8583962327201018, 2.9234198445283885, -4.4467656847238368, -2.4736386851484138]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[0.84219707958701573, 4.498805200781117, 1.2259752902059775, 2.5626237731849564, 0.89841807814728813], [2.8346592433586189, -3.1289608776474154, 1.2049162037395647, 2.2685991252024582, -4.2046575449989199], [4.773739393475946, -1.5086720345963709, 2.6015413885588634, -3.825437042413439, 0.070602814165030559], [-2.238790475174175, -4.6070597102731199, 4.6987923893865595, 3.0566095674209279, -0.42643449068183514]], [[-2.9835455436437295, -1.4630914834076334, -3.9372227836137688, -1.5578557261569861, 0.51099776240648964], [-2.8560284152775539, -1.7349529544193656, 1.6580592104773659, -0.048507334095685195, -0.076576188232449383], [-4.1989398682148424, -4.6912338121670523, 0.46581855869242084, 4.4335858887453696, 1.1467126160051411], [-1.0653918711659816, 2.4953998676177056, 4.699302022125913, -0.2180567099945927, -2.444670033736033]], [[0.13333463081285934, 1.2425843007166222, 4.3137619428585392, -3.5347209104982547, 3.415824929119637], [-0.7518971314365448, -2.5672090655391577, 0.46849486661078465, -3.2157575410462691, 0.60408704648377132], [3.5285594841969843, 4.2821930863545159, 0.93227837252531032, -2.8412861318668803, -3.7054820265344643], [4.5503848674948628, 4.2264780103241364, 0.071384319892229264, 4.6279011557258496, 2.6980504047832143]], [[-4.759414353949011, -0.18620523788402998, 2.4018915121480813, 0.6078935496379847, -2.1761343620017346], [2.2476528527291659, 3.160270914208775, 2.327462229363773, -0.071296342324650652, -2.9431498548302026], [-3.9001719531793766, -0.58126701853039808, 4.9593867298662637, 3.4464807279977343, -1.4976521051465319], [0.9105592316634219, -0.38397919508934031, 2.0492907068552979, 0.78908090217192317, -4.2171981819532807]], [[-1.0515161988747046, -0.72994044890293797, 0.56514499424210474, 1.3531036787159767, -2.4357957403406241], [-4.2721193098682706, -2.433585889375034, 3.5165319542340026, -0.81607928406717622, -4.094516199753703], [0.42537421541738851, 3.5503633626250473, -1.653811984075777, 3.8656063171025057, -0.40199938195643092], [-4.287713704635209, -0.96618978719199688, 2.926762690590559, 3.4677229308314033, -1.471639764413526]]], [[[2.8406263822655013, -0.26216575335077597, 3.2940915811019202, 2.8558464260814631, 2.981227454099125], [-3.8028690731208736, 1.8001388305002113, -1.412579226360994, -4.1927367983403565, -3.1087479319578391], [-2.2580253033091902, 2.1415853420966151, 4.3741380408285959, 2.2135839596296218, 4.698880505510326], [4.3025051088331594, -0.67858383757038965, 1.3244759291058683, -0.81617240935278534, 4.3510804881238538]], [[4.0241599501303753, 0.34592672733264607, 0.64388132863861713, -2.6172203306588058, -0.50019839710060765], [-4.6354713014872004, 2.816675120280884, -3.7427308442813145, -4.5823088033288695, 0.33226081161070553], [-0.54297644903332376, 2.64713413184157, 2.3446531387748628, -2.492458469506067, 2.7068628947860063], [-1.7649975577356569, -4.7724557715569782, -4.1577166573373061, -0.064912308296677601, -2.6300442281180336]], [[-2.603483484672223, 1.0195437128165805, -1.5900404208999506, -0.84740988114606974, 0.39245364059165144], [4.2142280942495685, 3.9554520828552615, 3.7268657558173643, 1.1470377629206983, 0.092843296283069421], [2.7475759583820647, 4.2724119273168757, 2.2506268936214378, -0.76604797832426286, -1.6001722969010301], [1.4954328617895882, 2.5282533062417469, -3.4515018713075585, 1.0721990909948884, -1.0805641422638121]], [[-2.0725679315779635, -1.3564905031095451, 3.4230238142804783, -1.8006594725902545, -2.7794607357765813], [4.0416710708143402, -3.0418961299483147, -2.5452193801464009, 1.5158605322196621, -4.6890634179205612], [-2.2804001563915719, -2.709702533184033, -1.4170402032708376, -0.18557906791526246, -0.31636181235411254], [0.74092311586269943, -4.3126374410744468, -4.2238299500930498, 4.7810334164853145, 0.35902452663741702]], [[-2.0719203427864796, 0.52404047717172642, -0.66139520736472157, -2.8683849032459721, 0.59982297091011816], [0.14162256849915433, 4.5623821831663776, 3.8619622986841957, 3.0318052353299212, -0.18468079496681611], [-2.457409071702612, 2.3198551827856999, 2.4354169938407004, -2.2142439159234897, -0.24965298683789516], [-2.9139623765745437, 0.86137986913479736, -2.5686063207630672, 2.074635308947208, -0.70371832713968985]]], [[[-2.509650279693699, -1.3359995042647252, 0.9686895812129066, -4.433908219333297, -2.6474413064088473], [2.2999182840569565, -1.4817506702832119, 3.2122097655266284, 3.2464817256526413, 0.33753069744404929], [-0.77696676711694401, -3.3103233047068956, 0.82104253902835733, -1.1736250260995638, -1.9522609909105313], [-3.7463495384752363, 3.0859577223969108, -0.37543037168544569, -1.1100531793965294, -4.5367242732070796]], [[1.4335256603380042, 0.21994337200649738, -1.9200878866930902, -0.60422063711155349, -1.751044949987449], [-0.74964096570242944, 2.6878230754371222, 3.5828607806263371, 3.8516201745731031, 3.0573432652692727], [2.0766500852352223, -0.83035650657523963, -3.6714025292895225, -1.0703321872091029, -4.9026088378937498], [3.8797294618096529, -3.065304179174384, -4.7714455857102545, 4.5930888845649882, -2.0854105930100486]], [[0.81660690270553449, 1.0148953433905072, -0.21219704496213332, -1.0939619565448941, 1.4448669432844863], [2.7047959147168417, 3.4985781653699242, -1.1844483334216771, -2.9569111989505723, 1.5369478919910069], [-3.2651042268344801, 0.49946067883341438, 3.9786869414823585, 0.34956725643528674, -3.6863223057000205], [-3.0931684661817673, 4.8682279314294732, 2.1574107068367629, 3.7539767042336933, -0.55461663311506282]], [[-1.2715724581877454, 0.78840432455593401, 3.6858769850694326, 3.7525078165942851, 1.4745741873033547], [4.0609766536474989, -3.5560685609953477, 1.3622505246420369, -3.5827400990508416, -2.7034142511561376], [-3.4885797494241908, 4.0642083762060892, -0.031387318159828759, 3.4863913662275223, 4.1202739670708759], [1.356824823783727, -0.31139531320607716, 1.5961667088092071, -4.2976317270655109, -0.95861424553637864]], [[4.6365873847141437, -0.35637200409890823, 0.44202352855572435, -4.066445961836469, 4.842859143934799], [-2.0981518903333765, 0.43614939473071068, 2.5120302919354085, -2.7037635302755425, -0.82505089519194819], [3.5211309595335756, -4.18002726299994, -3.1029424284964326, 1.2326589586349437, 0.39770996043368534], [-4.7335556279256181, -4.4346573990333304, 4.5642532722221159, -4.3355088557709553, -3.8964052301132535]]], [[[1.5997752553860414, -4.3087786732203215, 0.54924115970442067, -3.435007706654333, 1.3569146874289872], [0.1728613162021766, 0.94246984379308429, -0.91299390656441215, -1.4215666499098756, -4.2545217620138134], [-4.586281323263127, 4.288457874162459, -4.4406850827471587, -2.4467099872076603, -1.1795706259610936], [1.2019743019293454, 3.0781226260094634, -3.2283479619626068, 0.76816028963880711, 1.3257502557120757]], [[-1.8545991826188324, -1.6871233814086253, 4.1835895078777803, -4.8573915068373017, 3.8631280956011604], [-1.4973549688328882, 1.8414002272812766, -0.68534835169438946, 3.8811389490600412, 0.73553405700967023], [-2.4487322018674176, -3.0839253956265509, 0.3926897185194349, 0.34642087158816182, -1.5121725051597021], [1.5257841711208551, 1.0282910855843657, 2.9418422249796752, -2.4403429501759444, 4.8635727237293978]], [[-0.79308851315085604, 4.6515834974877421, 4.6975592952486309, 1.3380985070209377, 2.5737658904424876], [-3.637209425639627, -0.014043169293541879, -3.3494921471597108, 4.2882804395249927, -4.7973340483305336], [-4.8840595264440427, 4.8328166217476216, -3.1277061573324669, -0.47186406040255591, -0.4467203270077702], [-3.7085231386509965, 4.1402816437752161, -4.2567980559716503, -3.6159180157057591, -1.9116421854805141]], [[-3.2351885662590174, 4.3617513442404423, -3.1079101074741686, 1.8507290533100731, 3.3138230478452151], [-4.3833021116728608, 0.66346290018734155, -3.2017673250711831, 0.94702340090845105, -3.1580449831812398], [-4.0016725292640585, 0.70755623794476818, 4.3089717295823853, -2.5304965022644801, -4.4675908816208469], [-1.4374380011793653, -4.5854126649929139, -3.9412655100916183, 3.6965219327826837, -2.4346991492950609]], [[3.6906044955738579, 1.0571085686025778, 1.2305300650187263, -1.9973329543225504, -4.3972584625633351], [2.644642454728122, -4.2696985083845025, 2.0654946823314244, -1.2555200706063099, 3.0148163950946358], [1.2612934421261954, -2.1699578720610413, 1.0557651967322563, -1.0655683408032268, -2.4813463264811508], [-3.9658777927747102, 3.2329084072322569, -4.6882882367329595, -4.7298605494152337, -2.691756529266438]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.7678156885023153, -20.126724760588012, -5.4847600925182265, -11.464649178159105, -4.0193368176215643], [-12.681679653795571, 13.998324346233773, -5.3905460916663293, -10.149243665214238, 18.810768936167662], [-21.356723521744087, 6.7494869057778564, -11.638758546760585, 17.114214776013178, -0.31586240003807875], [10.015885925141433, 20.611033064548295, -21.021426113739881, -13.674639536154956, 1.9077797858161607]], [[6.8524860183467133, 3.3603689928489193, 9.0428531025136518, 3.5780196500887929, -1.1736388706238765], [6.5596098659814377, 3.9847693587169459, -3.8081629360721023, 0.1114096714179898, 0.17587707361098201], [9.6439542544005672, 10.774634955768171, -1.069873113660569, -10.182879687723243, -2.6337228821519645], [2.446948703960159, -5.7313328899786598, -10.793165692144548, 0.50082377982397552, 5.614818671475172]], [[0.59032594007101291, 5.5014195559408821, 19.098755954367771, -15.649628544794416, 15.123228302410958], [-3.3289504628018074, -11.366065183022727, 2.0742148597435661, -14.237449654943683, 2.674535875835351], [15.622349436948978, 18.958987952768393, 4.1275706342286576, -12.579514388570113, -16.405656560450261], [20.146380638043112, 18.712314943414025, 0.31604704261607319, 20.489576366278179, 11.94535232034421]], [[-19.182954077983293, -0.75050547436483395, 9.6808916289468563, 2.4501321338689745, -8.7709710542777355], [9.0592283526179518, 12.737561244473795, 9.3809023009481383, -0.28736192292380569, -11.862448677742469], [-15.71975329527695, -2.3428131476434899, 19.988948391317869, 13.891137988643226, -6.0363291407867017], [3.6700347200890726, -1.5476389989428341, 8.2597241169862343, 3.1804128795002149, -16.997536471066226]], [[2.8395723341394357, 1.9711714441415673, -1.5261487099719808, -3.653995797131806, 6.5777571503775603], [11.536666590041083, 6.5717895469576826, -9.4962368245455746, 2.2037855051414148, 11.057057356747153], [-1.1487039905311172, -9.5875970255534053, 4.466045088868289, -10.438896485347303, 1.0855813011404225], [11.578778553694363, 2.6091521863139437, -7.9035913794636006, -9.3644302459510591, 3.9740971801672274]]], [[[-2.283301817224292, 0.21072941685572691, -2.6477981547276435, -2.2955357223658415, -2.3963172721344024], [3.0567546367705942, -1.4469556040872451, 1.1354343304912258, 3.3701311832352481, 2.4988185164535848], [1.8150057714638308, -1.721411071120958, -3.5159419062525452, -1.7792837203637777, -3.7769706231464326], [-3.4583631958598606, 0.54544720105985678, -1.0646167037938199, 0.65604120171456037, -3.4974081939980337]], [[18.416180942589016, 1.5831003942150126, 2.9466609679317601, -11.977457102437903, -2.2891098520821318], [-21.213788542277282, 12.890242762318845, -17.128247709304034, -20.970495482961347, 1.5205596453878505], [-2.4848794920883264, 12.114354736252029, 10.73007956601929, -11.406496445258119, 12.387697674776653], [-8.0773415543371918, -21.840684793517983, -19.027390366791831, -0.29706493524294603, -12.036144435676407]], [[-12.989931142091487, 5.0869547296195226, -7.9334152500809569, -4.228102874484434, 1.9581248729874319], [21.026648751052623, 19.735500722256759, 18.594982387599558, 5.7230789614916358, 0.46323628815863549], [13.708872254107078, 21.316953640473663, 11.22937347089124, -3.8221523388013274, -7.9839676627884266], [7.4613762739936238, 12.614574492674457, -17.221070119724523, 5.3496757112673547, -5.3914126535691285]], [[-2.8618494338540494, -1.8730732629819304, 4.7265899542839733, -2.4863919843982702, -3.8379432644437541], [5.5808323045064006, -4.2003200882817175, -3.5144974170089487, 2.0931350619864348, -6.4747665364409777], [-3.1488289464949739, -3.7416195350477461, -1.9566816805813942, -0.25625184215028357, -0.43683966145772646], [1.0230836670558037, -5.9549888961950339, -5.8323614715809677, 6.6017608242102117, 0.49574931785938409]], [[7.3169206294701361, -1.8506322366324324, 2.3356960868926264, 10.12960983026751, -2.1182556970215529], [-0.50013558516269607, -16.111907212919952, -13.638396723049297, -10.706723522530309, 0.65219433906533053], [8.6782617847200143, -8.1924946117762811, -8.6005974629457889, 7.819531790160597, 0.8816415630855553], [10.290565224135815, -3.0419355436253857, 9.070951324365419, -7.326508445902804, 2.4851588349481459]]], [[[-3.138568725106611, -1.6708010254540211, 1.2114432231978216, -5.5450457698397262, -3.310890187801117], [2.8762778842329513, -1.8530774386319064, 4.0171896419744932, 4.0600501564732436, 0.42211590169870583], [-0.97167466537132696, -4.139890437408944, 1.0267958272222037, -1.4677354975403545, -2.4414976616045756], [-4.6851848601834041, 3.8592988325442223, -0.4695132355934879, -1.38823254389226, -5.6736275303097603]], [[-5.8677233884296527, -0.9002746890127602, 7.8593253768007365, 2.4732027212651184, 7.1673969231986927], [3.0684388491099566, -11.001827703582688, -14.665406125350774, -15.765495105515791, -12.514351909018838], [-8.500167532296599, 3.398824610657794, 15.0278262088259, 4.3810957711040137, 20.06741374664211], [-15.880552356818221, 12.546937611533075, 19.530534844622231, -18.800483185451736, 8.5360261414496073]], [[-1.2890517435266435, -1.6020592130193594, 0.33496284426862671, 1.726869516733093, -2.2807892588638734], [-4.2696453804123076, -5.5226673556059591, 1.8697064453605339, 4.667621010589408, -2.4261432928335442], [5.1541179513087094, -0.78842176907899197, -6.2805412516993213, -0.55180807300892998, 5.8190301595788396], [4.882709405397998, -7.6847228233106843, -3.4055725269246242, -5.9258257549848281, 0.87548799249331055]], [[-3.7654131450766255, 2.3346432114054592, 10.914713952279277, 11.112022888372969, 4.3665470988385735], [12.025468761164259, -10.530321900373043, 4.0339313731953315, -10.609302346471019, -8.0054199761440739], [-10.330472291647, 12.035038621320478, -0.092944935718536528, 10.32399199490721, 12.201061494393421], [4.017864647935923, -0.9221118294214371, 4.7266100082489224, -12.72625786567763, -2.8386732175253231]], [[-14.581194349520608, 1.1207228552675497, -1.3900807732386247, 12.788206920641571, -15.229880195479128], [6.5982926556340908, -1.3716077283392674, -7.8998623037601625, 8.5028272388581794, 2.5946297250216594], [-11.073293910158542, 13.145398727744352, 9.7581697164690393, -3.8764803402151156, -1.2507229448413046], [14.886141217266369, 13.946162563876957, -14.353717184780107, 13.63435906297741, 12.253460834594547]]], [[[-5.2992060153368836, 14.272695984646013, -1.8193443407611214, 11.378356703885057, -4.4947379012857036], [-0.57259775931249723, -3.1219021851270865, 3.0242640554310625, 4.7088955258198979, 14.092971645698324], [15.191914923323687, -14.205383966760168, 14.709631883280995, 8.1046510991738856, 3.9072911870357343], [-3.981502670215582, -10.196185920976609, 10.693802696369724, -2.5445071824258241, -4.3915066858616587]], [[3.589615006381329, 3.265462135580969, -8.0974238632046429, 9.4015815429138865, -7.4771641837789513], [2.89816145524899, -3.5640681558314737, 1.3265058838034172, -7.5120245624848447, -1.423641352531049], [4.7395717310844123, 5.9690012712448901, -0.76005905732070711, -0.67050474885933331, 2.9268411027193535], [-2.9531867632016411, -1.9902786253413423, -5.6939963611337614, 4.7233341612384585, -9.4135454158205025]], [[0.81813665790562529, -4.7984946364740741, -4.8459224896089026, -1.380359723195715, -2.6550532367092567], [3.7520835496322702, 0.014486695244867136, 3.4552792853742074, -4.4237173641773557, 4.9488484325222375], [5.0383129647182265, -4.9854516534086626, 3.226488620172582, 0.48676696101648548, 0.46082911213105199], [3.8256495663611805, -4.2710443168175747, 4.391240671303688, 3.7301196922865265, 1.9720176535238034]], [[13.833169536718362, -18.650178988378798, 13.288946390931542, -7.9134332471298672, -14.169398505427798], [18.742326760736859, -2.8368654845448509, 13.690265441327883, -4.0493266440894562, 13.503315421099131], [17.110537266688141, -3.0254018261354676, -18.424501460564144, 10.820014478591849, 19.102732598248892], [6.1462641702609613, 19.606520452070647, 16.852246128380113, -15.805760172894582, 10.410399707256531]], [[14.712219128353457, 4.2140556980281136, 4.9053828396660899, -7.9621644994787761, -17.529223232352653], [10.542597928542534, -17.020718460266554, 8.2338842895122113, -5.0050029530280709, 12.018258659138207], [5.0280179108789653, -8.6503161613077584, 4.208700482818589, -4.2477797187247681, -9.8916345363767224], [-15.8095681055902, 12.887660264365833, -18.689383801059964, -18.855107593622286, -10.730413390608051]]]])+(1.-msk_ref)*numpy.array([[[[-1.7180211167969939, -9.1772371605564746, -2.5009009034773961, -5.2275671139784148, -1.8327078867645712], [-5.7824997937560862, 6.382853837567251, -2.4579418905258286, -4.6277781021945721, 8.5771972217597803], [-9.7380830245802148, 3.0775818114076614, -5.3069562339935805, 7.8036148297575387, -0.14402463340326035], [4.5669706125308531, 9.3980685286573493, -9.5851965579952481, -6.2352623986869498, 0.86989551220151096]], [[0.25505809178645672, 0.12507713269602719, 0.3365862915235609, 0.13317836211306491, -0.043684305226785528], [0.24415687544656994, 0.14831809450210295, -0.1417445827814772, 0.004146807176599608, 0.0065463644382635702], [0.35896002747288402, 0.40104537596370404, -0.039822014097262888, -0.37901950548866475, -0.098030456511326025], [0.091078488224628357, -0.21332737146738559, -0.40173511312579419, 0.018641286864530246, 0.20899056667013041]], [[-0.037901607952831387, -0.3532161354255044, -1.2262269221680713, 1.0047772687083427, -0.97097998104266237], [0.21373374736053927, 0.72975303521548152, -0.13317401978723944, 0.91410896665820995, -0.17171735704092533], [-1.003025826021043, -1.2172531813218987, -0.26500879151157603, 0.80766198845205994, 1.0533177029086389], [-1.293489187549117, -1.2014156531726703, -0.02029165633894596, -1.3155239128757512, -0.76694590186849643]], [[1.4356599458783097, 0.056168129492872525, -0.72452179656829863, -0.18336886761055318, 0.65642297728986732], [-0.67799626864279383, -0.95328417158914447, -0.70207047542933942, 0.021506281099102215, 0.88779039752892908], [1.176472615915801, 0.17533707181258337, -1.4959808822448424, -1.0396183159220842, 0.45176128412421085], [-0.27466686445659877, 0.11582592088939712, -0.61816105228402218, -0.23802337032618187, 1.2721024192057593]], [[-0.37476182950173703, -0.26015178686826873, 0.20141845860536917, 0.48224802497935981, -0.86812097513997677], [-1.5225892384057365, -0.86733338119546399, 1.2532968584605568, -0.29085178701267456, -1.4592912440020716], [0.15160395946661187, 1.2653544192635351, -0.58942088145423566, 1.3777074448126758, -0.1432731364498086], [-1.5281471022970243, -0.344351378211257, 1.0431022761389148, 1.2359012549252748, -0.52449445008011242]]], [[[3.4691362261619179, -0.32017188810407976, 4.0229339935869, 3.4877238185659616, 3.640846337276991], [-4.6442823129709829, 2.1984330174489464, -1.7251229507290682, -5.1204111898321409, -3.7965816751162196], [-2.7576302987979533, 2.6154271248299388, 5.3419488146727723, 2.703355975268245, 5.7385429797604077], [5.2544665604337011, -0.82872558957434395, 1.6175261396664646, -0.99675666245887928, 5.3137896058890872]], [[-4.2036284024245534, -0.36135427870509307, -0.67259698282332414, 2.7339424510210879, 0.52250611679369952], [4.8422028604812795, -2.942292474142763, 3.9096481935668685, 4.7866694348790082, -0.34707889398236891], [0.56719197330368543, -2.7651903402289397, -2.4492193774923319, 2.6036164924159859, -2.8275828712084254], [1.8437124656705566, 4.985296528777873, 4.3431414373701864, 0.06780725075652437, 2.7473382653667984]], [[3.9428420445521266, -1.5440465978827191, 2.4080345663690594, 1.2833587491615295, -0.59435088556251281], [-6.3822320414803508, -5.9903290607800797, -5.6441468067496743, -1.7371297897448097, -0.14060640457098342], [-4.1610626966098501, -6.4703484688291741, -3.4084588571482968, 1.160140325399962, 2.4233787722848787], [-2.2647562763438205, -3.8289098025075092, 5.2271223440294321, -1.6237904641970327, 1.6364589047852391]], [[-2.8470856893358554, -1.8634104292942459, 4.7022063631341986, -2.4735651544485524, -3.8181440349101221], [5.5520418372763833, -4.1786514246740314, -3.4963667839432193, 2.0823369707480621, -6.4413644301567787], [-3.1325847284911843, -3.7223172215695217, -1.9465875267472315, -0.25492988695408431, -0.43458608756914807], [1.0178057702865078, -5.9242682252777987, -5.8022734125497761, 6.5677035782119111, 0.4931918401013427]], [[-10.028350631262226, 2.536420701861712, -3.2012345785309693, -13.883337578743085, 2.9032173413010232], [0.68547074174648148, 22.082493859336147, 18.692375018546635, 14.674312191314879, -0.89387788156793124], [-11.894163741032422, 11.228385911515428, 11.787719365080852, -10.717214322128125, -1.2083513233840157], [-14.103938184850952, 4.1691850675963584, -12.432372174909919, 10.041491403158496, -3.4060837110712106]]], [[[-4.5471080703434099, -2.420629749479823, 1.7551195272228848, -8.0335734466369964, -4.7967646438775615], [4.1671053035493619, -2.6847088957366116, 5.8200399739539517, 5.882135600478656, 0.61155475356661881], [-1.4077466831662613, -5.9978069199807891, 1.4876053390860269, -2.1264316669594496, -3.537202684777796], [-6.7878207408273523, 5.5912902996038865, -0.68022325132538752, -2.0112490618256049, -8.2198606405592507]], [[-4.6371112325447417, -0.71146398636116348, 6.211023180958108, 1.9545086501100319, 5.6642098784356731], [2.4249084886382803, -8.6944621355879086, -11.589694157667916, -12.459066251243565, -9.8897712049203061], [-6.7174642929228856, 2.6860038785415075, 11.876105450213878, 3.4622675723092753, 15.858762169356565], [-12.549993044666083, 9.9155228494928966, 15.434480548995632, -14.857539455324988, 6.7458024316317973]], [[-2.8474180907732047, -3.5388279861916336, 0.73990766258765117, 3.814524534832187, -5.0380914726996524], [-9.4313246604913807, -12.19915571011366, 4.130040538473394, 10.310422815196077, -5.3591675722205157], [11.385057869270868, -1.7415642310005799, -13.873241993381937, -1.2189024200236478, 12.85379879461542], [10.785536858964242, -16.97497319216054, -7.5226528890730764, -13.089702211139226, 1.9338869526370122]], [[5.864743270772605, -3.6362764286962408, -16.999992494245973, -17.307307046629635, -6.8010273315981733], [-18.730026235537846, 16.40129040953498, -6.2829684191864104, 16.524304809795776, 12.468680361459812], [16.090018683755005, -18.744931578136654, 0.14476450931298454, -16.079925428318486, -19.003517154309616], [-6.2579440154309625, 1.4362166748077094, -7.3618360512947687, 19.821526165711354, 4.4213181951021401]], [[-8.567248992433715, 0.65848595954720435, -0.81674846507492227, 7.5137708358919451, -8.9483873976273109], [3.8768577354311504, -0.80589454107476632, -4.6416010746389569, 4.9958759446437719, 1.5244868400071225], [-6.5061656693333338, 7.7236405541161925, 5.7334582934329479, -2.2776441691135503, -0.73486812068708818], [8.7464219519880597, 8.19413309429072, -8.4335937195041151, 8.0109314878320017, 7.1995782699691908]]], [[[-3.448919224571894, 9.2891983111146015, -1.1840965712234548, 7.4054552826383917, -2.9253416289705605], [-0.37266779482135615, -2.0318493812762846, 1.9683028760856476, 3.0647233299663226, 9.1722270656249592], [9.8874599865468316, -9.2453891608646739, 9.5735723506115313, 5.2748066226316634, 2.5430095852031487], [-2.5913091625893094, -6.6360548237583341, 6.9599222216582604, -1.6560593630689009, -2.8581549367714882]], [[-5.3011793168034504, -4.8224671175524136, 11.958356488565192, -13.884349583990362, 11.042350794981537], [-4.2800338019566535, 5.2634514725907406, -1.9589971465841742, 11.093832950656477, 2.102447777593774], [-6.9994469007582261, -8.8150807328487826, 1.12246281204387, 0.99020811428321587, -4.3223881919714113], [4.3612957266758157, 2.9392633651740145, 8.4089507331388251, -6.9754670953279785, 13.902017951068355]], [[-2.3185306978127262, 13.598531505036433, 13.732938064578638, 3.9118237293588183, 7.5241982792898465], [-10.633090213620587, -0.041054079792812039, -9.7919718120986996, 12.536444135810283, -14.024621557721193], [-14.278156541464439, 14.128352016983644, -9.1435982481993641, -1.3794567581005741, -1.3059510689287264], [-10.841570137425027, 12.103781519349083, -12.444407910977388, -10.57084648330193, -5.5885327006713537]], [[14.386125500051508, -19.395686202866464, 13.820148017122504, -8.2297584459806661, -14.73579461440506], [19.49151741596441, -2.9502640576404264, 14.237509071596566, -4.2111911617901647, 14.043086062024603], [17.794500084633459, -3.146336798905089, -19.160987623552071, 11.252525011583385, 19.866329825776695], [6.3919499775624233, 20.390255688988567, 17.525884224660999, -16.437566883527072, 10.826536629711594]], [[-9.1292220518341427, -2.6149046496972024, -3.0438867720685141, 4.9406800629340308, 10.877228641592904], [-6.5418898844013453, 10.561691404260385, -5.1092875501833452, 3.1057030166320359, -7.4575664635257759], [-3.1199842517673591, 5.3676917374725237, -2.6115816330702351, 2.6358310694802709, 6.137954264434681], [9.8101487287785272, -7.9970473016712527, 11.597131149508893, 11.699966030390852, 6.6584330817942892]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.49641509426428509, 2.8400527939288569, 2.6536389846342381, 0.65550986533642419, -1.6515573429202899], [1.5788749450739834, 0.97163515687365809, 4.3783669951794142, -1.2778511718008154, 1.5643929319476815], [2.2255941889737043, -0.62192911042515764, 0.62026701474828005, 1.4745149648647891, -2.5583286041723827], [-4.0682925725350998, 2.6200913640353818, -4.186032429250071, 4.2572642834115388, 1.2613370101555121]], [[3.6725457155162609, 0.79711876551619554, -0.38200736787290701, 1.4103973199173669, -0.42897577617206117], [-2.5581268341628984, -3.5822786097698325, 1.5740052147886949, -1.1747137290368528, -4.6637462822001705], [2.2417986121474573, -3.5548661225790434, 2.1230352760352318, 0.370810455389881, -4.0279169970967343], [-0.55451397744741815, 1.9917744547672651, 2.9539032976833859, -3.6913520355566245, -1.7267797534763107]], [[-3.819248327322653, -4.6661597843386513, -2.4711080760915536, 0.44615700327478791, -1.6213162677403128], [4.9718013412650865, -2.3636095924777303, 1.5920187494195908, 4.1102688665342573, -4.3282071274635294], [2.6235284944269317, 2.7661191737480504, -4.2833350476972658, -3.9887576853322515, -1.0088549955975292], [-1.6923577394345388, 3.5799819723286586, -4.1025326458873135, 1.0409154438153854, -4.6807960289479418]], [[-4.4094552401473894, 4.7776816183705932, -2.9681827860454311, 4.4482851569774322, -4.8889688611820512], [-3.0268636904938973, -4.5267111086427239, -3.9464392473028855, -3.9172668746163564, 4.9423994190644223], [3.7731047991692641, 4.2849785014811097, -0.47985033417582379, 4.1408169393916356, 2.1279718017934837], [-0.66965184283490053, 0.79645225178905488, -1.1639093000444767, -3.4638206246778394, 2.9652198116933102]], [[-3.3599147173663493, 2.5823578842974095, 0.84008525700242664, 0.73231682061727721, 1.889101121781394], [-1.4848629684522896, -3.9123554008926051, -4.5148720658939556, 3.30595782367973, 1.2184870277737057], [2.0353577990373903, 2.2134708118066531, 2.6406880773404007, -1.6475921730547229, -0.1101688382877235], [4.1615727001267455, -4.55020521735677, 1.4951400828601367, -1.867053928173783, 3.2980531228851451]]], [[[1.3041859237018292, -0.45354815858357611, -1.3730837078077407, 4.0335972536519353, 0.17020315092788341], [-0.39290545370818819, -3.2896382195085705, 2.7456959914593995, 0.44924562851652805, -1.5176096229945135], [-0.36784923034450934, 4.5936613896085063, -1.8137389539067539, -2.6232056548616232, -1.3692987504591967], [3.6892675221251388, -3.7013984299569671, -1.4716812555483427, 3.2094301211386718, 1.5483109461165077]], [[4.9985123617170792, 1.2314158139489173, -0.69276352424828236, -4.122136083475338, 0.9433126389612676], [2.2087138476758437, 3.7328220632790341, 0.67372943174355626, -3.0108765708209519, 0.92403312022336959], [-2.9341678324588614, 1.4135547254920429, -1.2184421343443419, 0.82927310919128061, 2.6471266913005049], [4.2241510872418253, -1.8574587319089364, -0.59753418880047882, 1.0087116449129478, 3.7823973560782473]], [[0.49059406224970914, -1.6344731672576507, 0.72334176653872184, 1.3707635227199795, 0.74989939828190977], [2.9677120507558108, 4.6559390285860776, 4.6779417943410184, 0.91059065887470592, 1.3253904870608775], [-1.8677593702508934, -3.7437227339727874, 4.0845095806668414, -0.60143143779198471, -1.2084919141415709], [2.7502788318594895, -1.0279246988179116, -2.2944581000773479, 4.8397500814307328, -4.1372414437446388]], [[3.8554242722281131, 1.789171289199734, -2.9838682556426157, -3.248089473719479, -4.8367295266301804], [-4.3579769204538348, 4.2712676841410353, 0.43385122815231103, 0.83313210497773849, -4.6111519200775692], [0.68570627333335299, 2.3539621857144608, -1.0596853436651079, -1.1059387511214003, -2.0764010801918387], [0.90276083139145058, 3.2845167198177752, -1.6824819755958185, 0.50895311497783524, -2.3594541819331303]], [[-0.753886406611187, 3.1467038119873116, -3.4379996443946927, -2.1673013249513327, 1.6213692609556052], [2.5537408237446213, -4.1690924992499667, 4.4724517289220209, -0.4187328373685073, 1.2854912227286448], [4.4993675725139219, 2.0050398351153795, 3.4711672564696947, -0.73323039146923996, 4.4294169226386124], [3.8771505526674517, -0.574390986221605, 0.03206752147133507, 1.6175716144487211, -3.4841965513820927]]], [[[-0.9151321417456435, -3.4904615383974527, -3.550526236795446, 2.6205268402386093, 4.926070153251791], [-3.4035709857189831, -1.3466783215475364, 1.2571057050880698, 1.4078643387386736, -4.6342754280560259], [-2.961037194503545, 3.2973739900859105, -4.0924946281846077, 2.2096041706677383, 3.6331516761863334], [-1.6793685218718792, 3.0207145115815983, -2.3122871175442752, 4.8896698864219967, -4.3523715253164843]], [[4.0513028745117978, -1.0392854338704796, 3.204969802486243, 3.8855403186327671, 1.5346706994683785], [-0.38281277024393301, 1.942769917316288, -0.91804881077968048, -0.85262406730884166, -1.3678989835009006], [-1.3601913870402385, -1.8733261514131563, 0.74236034059431155, -4.3822110645421821, 2.2051950767933057], [2.9881207275384902, 0.25253500943853613, -3.9943730770970953, -3.4258839561526711, -3.6213741148091794]], [[-3.8348354693324738, 2.857439661968816, 1.2271851217052232, 2.0626011738372343, -2.2945156600620455], [0.55543482931555754, 3.1524601607585918, 4.0567045916661151, -3.6458535386304645, 0.58748017750952819], [4.2602317707994182, -1.6509561372239245, -0.21956466859669099, 3.7185611106244476, 4.3561622478693316], [-1.5235073968422306, -1.4008643954208457, -4.3505636327032571, 4.1997140094413385, 2.6281944372332209]], [[3.239603176088675, -2.9613189404055582, 2.6613000979497148, 1.6068047060612214, -4.9241722187953538], [2.8083265989948005, 2.8513101309836086, 2.2748398545416322, -2.1483756522866635, -4.6680430461260922], [3.1679259476763697, -1.1839974449755166, 3.0494495147759544, 3.8662832522045036, 2.5021689295457925], [-1.7688816184898615, 2.5793370940334892, 2.9548895502229664, -0.72840407921613526, -3.6955630004273434]], [[3.07864313755754, 1.9020374078346958, 1.3662434697152381, -0.4438000431530531, 2.626055779742007], [2.3700818634209151, -1.8100260652597178, 4.4857406351137037, 3.783602301997508, 2.8447413282533249], [-1.9422128234418654, 2.74311941665737, 3.7564420834058545, -0.15498492120971896, -4.8189865811919272], [0.038453790020540168, -3.8068527369081218, -0.97377825747670244, -4.1675925172106254, -3.7312151430378937]]], [[[-4.1024396573824724, 0.19664889820438436, 2.1717931085222641, 4.4674484851639722, 0.37656989391375006], [1.2083930040960933, 2.0326264846151023, -4.1583231168878712, -3.0499629206651679, 3.783966844318897], [2.4731233421296528, 2.116794892699585, 1.5849539294369368, 3.1760983560187857, -2.4556763612255086], [-4.4810490025337728, 1.2546174371125209, 3.6672844780707319, 2.5701862021099187, -0.62191773777513859]], [[1.362330380734261, -0.39771789834166427, -2.8363855575185362, -2.0498860147690587, 9.4048837220306325e-05], [2.4659927498202716, -3.1889127067598597, 0.49853716518892899, -4.0676207965263478, 0.98756438861388673], [4.995839180084868, 3.1246427932196248, 3.2593664663207775, -2.8681020286769088, 4.7094016943083936], [-4.0870260486759724, -2.2684840005730211, 4.2554474953352042, 0.052552952092003835, -1.4059430105884707]], [[-4.596682201364116, 0.28643394189844607, 1.076789452998165, 0.17847563497332875, -2.6712070890724169], [-1.4449018558283147, 3.8619171485959427, -4.6962619109750339, -2.0105983604458624, 4.3998109245184924], [4.1879492010682, 2.3705854664905086, 1.0983606924913083, 0.4220467048227281, 4.5720086588074551], [4.0134009717309933, -0.7919331867269177, -4.4231357158661355, 4.3312610962385936, -1.2301194092144732]], [[1.0144638836905164, -2.7434453083985524, 4.1452661137430287, 0.35436973838313257, -4.746743612702538], [2.0180048921695759, -3.827015957073201, 0.7186962375291861, 3.3342879298570303, 2.3763180296753657], [0.79953664195686081, 3.8029158913740702, -4.5525335887522296, 1.4767700002110455, -1.7921474433350615], [-2.7016941926372118, -0.4260573538711343, -0.4973479201255806, -2.6805832410930543, -4.9593596094034185]], [[-3.2438880347368935, 2.7317748024840292, -0.44330261684072259, -1.58297457148619, -2.201600940939985], [-4.3516904352890808, 1.1220638608436753, 0.057443111804805547, 0.57390052725992646, 1.5820596310900381], [-4.9200528796039436, -0.91110377256952013, -0.44865920409776905, -3.2320086492845834, -3.9935033227333494], [4.9098000314399552, 3.826403038810561, 4.8901413002524698, 3.7509470739998552, 0.67006448777562078]]]])+(1.-msk_arg1)*numpy.array([[[[-3.1175778385983333, -3.3436155146677948, -3.9050299352708384, -4.7027696804616772, 4.4344333292580025], [-1.6402377922964204, 0.26486048233325121, 4.3515986935351378, -3.976320913454896, -1.3911876569589063], [-0.20176018870901391, 4.4931274992315107, 0.12196140833846147, 0.35830569290074354, 3.5218696073281031], [1.1498300091133569, 2.500983975475318, 1.4775995566894773, 0.58590334366795993, 4.6812268132470525]], [[4.6935859853517918, 3.0240149244574113, 2.2922191501543798, -3.130984221589177, 4.1362022424825238], [-0.37640247682710637, 2.1655838411331354, 1.3278641644509523, -2.6365314066102528, -2.5395360042738924], [-3.8631162183662915, 1.9724637598941328, -1.0800324264418961, -4.8414620661957875, -3.521324748278424], [0.77545213982355232, 1.7653486144938837, 0.084774886527673843, -1.5515026507675476, 4.6905932021464789]], [[2.8618058973420428, -0.11546685622177044, -3.1679926356674102, 3.3977938924324231, -1.917130928313886], [2.8878041508928254, 4.0688668365088496, 3.8043679703166404, -3.0102918715135294, -3.0366913039431798], [-2.7192577628201873, 4.5884266542307923, -1.7956799412309699, -4.9708884566018128, 3.1698915975144146], [-2.3370283888467638, -1.260370629976304, 1.2964445118753432, 2.6524485026837183, 3.2142498181980059]], [[2.6298548634775489, 3.4658082960021535, 0.65060372589384308, 3.3368832320702726, 4.091366638432028], [-3.5423617078690448, 2.50272201652884, 1.8843460377684043, -4.333062841870766, -4.1270481116154176], [-0.65816673687762339, -0.68591120142758655, -3.642041986086376, 4.9906762855328477, 2.6906864302118771], [-4.2070510417349425, -3.4016201893485243, -2.9476465889317369, 4.6719705305123149, -1.9416888006339694]], [[4.2631310794038608, -3.6233297889767133, 4.1225389296912773, 4.7803305649160279, 1.2156341330422729], [1.8394257096179114, 4.7734167498842837, 1.9233799449158848, 3.4268514166296988, -4.3984492665258035], [-4.4149254196144874, 1.5713510892848301, 1.4229850060638629, 1.9345321375118267, -0.3956852518462437], [-3.3781666529579413, 3.36452206031122, 2.6402121803621759, -0.015936422824431418, -3.0611802393568723]]], [[[1.4529790169624199, 4.6442497812081847, 4.5265939966330055, 4.7140070190898182, 2.5901156130959899], [-3.8893639934940003, -4.8313013976421617, -2.8950376207050676, -4.7451018818187274, -0.84524652738562089], [0.24663326675804864, -2.0230475653941316, -3.9688543990873502, 0.55645523941722708, -1.2301168449694688], [-1.6089140949871039, -1.0452268655425523, -2.1163117125536077, 4.1057047697193152, 4.2601920636490611]], [[0.95307956221305723, 1.1182792040648168, 0.72710959556238208, 0.56636281309129188, -3.3456961922403661], [4.2879750880197882, 0.83119691106274196, -3.786771182777382, -2.4066459372324509, -1.6261032323888571], [-2.9917754455215064, 1.9121390762825357, -4.3691716040166391, 2.9674012345702874, 2.7109006953891734], [0.91303732482502475, -0.046290698931020557, 1.5604408350592731, -0.66837914351846184, 4.1987956421526]], [[1.6690115552939044, -0.28948974654890591, 2.9414198195701502, -2.3944386884621127, -3.7932733936814298], [2.3614086632306188, -2.6354694675071211, 4.1839443417437501, -4.5164898929862529, -2.4397774290066296], [2.7011449223416211, 3.0104298609729483, -0.053185738094915003, 2.4610239750942906, 4.0837995391059554], [2.6020091611295717, 2.7019363243325216, -2.7667081638664937, -3.5914885644842567, -0.87875867586607903]], [[-0.96863847783199652, 1.8665412568138162, 1.9195090609338816, 4.1201122804475112, -0.62744127493043145], [-1.1123625006296276, -0.14251617175670805, 0.77100405537114902, 3.9072252383496942, 2.4333171857830678], [2.0621506571539356, -1.1576468224911283, -0.76894018765922301, -2.98680554166602, -1.3676152730867521], [-4.1624471481736371, -4.0001556747170985, -1.5606235401460489, -3.2726852795038939, -3.4230686760812246]], [[3.6319587151104074, 3.6112929453177074, 3.6124035093314895, -4.8856898175507366, 2.3221742767907045], [1.3883688744382985, 4.0183855793688501, -0.32471597840303978, -0.20844940622168906, 0.72657995667514008], [3.9561852569102882, -0.94649873971396836, 1.242173020600573, -0.70360540288210949, -3.6589321975567977], [-4.0815432516645531, -2.6779296260415641, 3.9789425462979811, -1.8272967889772493, 0.094084240151430798]]], [[[-0.1075879958441428, 1.5884283467823499, 3.4596519311008826, 0.036946349693561586, -3.0630643726415183], [-0.38632129582441799, -1.7188605263721, 1.5175478743381552, -3.2407724897677026, -1.190952680695474], [3.7125177472239876, -0.10579910397025927, -0.88025422778425266, -0.1181697430674209, 1.4038791840409912], [3.5177246118948382, 4.7778957583625399, -3.1370842358228526, -4.5046876626537902, -4.7926686709672541]], [[4.4893553489559324, 3.9430759767630814, 1.0169791383502735, 1.0267876271274741, 0.4617681319582978], [0.21061155779949914, 3.2620082431522963, 4.5394617093273801, -3.7059691273992756, 0.23429833912924991], [-0.39894460495688211, 0.30998562349660652, 2.5780464395255667, 4.2302312226118275, -4.4248181049548299], [-1.0039956583850209, -2.953696197337278, -0.053021896283590664, -3.2344636173098316, 1.4787370713461385]], [[-4.5490547851125438, -0.83151375910150183, -3.8645324996291741, 2.5147109170710298, -4.8502682307328495], [3.3426390567555693, -4.6732161023171948, -1.0479439120346532, -3.5379394971012426, 2.1310816471453728], [2.5719794040064166, -0.99060965897344477, 4.5105197240525392, 0.99379101690993732, -0.52234059681880307], [0.32534358335044278, 1.0542375875543399, 1.2383505534872743, -2.5647908698956345, 4.6057209458280184]], [[0.72413399074197038, -1.8695165966691341, -4.1279610377662088, -2.4204206816873053, -0.51245931498011377], [0.74503934702402486, -4.9996390331357157, -3.0149260752151221, 3.7998520338705504, 1.3858754566884999], [2.3231623216827488, 0.015996799067401923, 1.8896301074569966, 3.6885210601157077, 4.2031103224120852], [2.686087539683947, 4.0387809386698432, 0.049090893133737268, -2.3542853350692949, -4.8816377097212058]], [[4.1181698870193291, 2.3076009732729457, 2.2650335705206484, 2.4352108481627361, 3.6603765431338218], [2.0004263335288126, -2.0014009783744879, -4.4123732175643928, -2.7369418396873013, 2.2067914516169527], [0.33499920052376009, 1.968630814898332, 1.5386689406682708, 1.0675138526574282, 3.6766200430454248], [0.16680945984751183, 1.4317755192263748, -2.4740816264281573, -1.4630021829150017, 3.8009823531168401]]], [[[0.90974820109919285, -1.1934040232923415, 4.4629149542271787, 1.8861177330186543, 2.3910159679606879], [2.8772102581999768, -1.7357058505987597, -0.74936070856663051, -3.1493660799879941, -3.5755260351935703], [3.4684764370381291, -1.5946186868373413, 3.4288927734303645, -3.0002164972401326, -3.233544679083896], [0.48403654591103162, -1.0506662482617779, 0.033532272624922754, -1.9021998931753705, 4.6004905513892087]], [[0.40116088329388155, 4.2437216663787396, -0.86695088774752538, -2.314625825779002, 3.0265089441877695], [2.4897335714335869, -3.8520253317689712, -1.3952633955287688, -1.4247954880722249, 1.4744824662922165], [-4.2838273024900131, 1.7267729866431889, 1.7649841475780059, 2.7657606012290312, 4.7465133622139284], [-3.2434410220388954, 3.0782725734201009, -1.1007334289244088, -3.6948859790365196, -2.744937012898955]], [[2.1698839676517512, 3.4999295686615319, 0.014478416955540752, 1.1164428194991407, 2.9490980746474555], [4.151536540157263, 0.4508696879769527, -0.10072670345029877, -2.0800789566519171, -0.90236423381661712], [-4.6539258817838514, 4.2731060357306827, -4.3026069572149179, -1.9082079460927548, -3.4599747317829244], [-2.5820226786492704, 1.1916597103574702, 1.8290164917572369, -4.7208359780295908, -4.5617538543597238]], [[4.0986650514435521, 2.7832414887535384, -2.1787880993503528, -3.8839145343369665, -3.5311899840235816], [3.2552802823245877, 1.8383732242721162, -4.364421124652873, -3.0792204749946861, 0.28533637652854971], [2.5936829904131216, 2.3594857991108409, 0.54266212495414479, 0.82342106903393386, 0.1943554496251334], [-2.2252417458245222, -0.029263263254413197, 4.2054639448148592, -0.36520048355893042, -0.57575646465113284]], [[-4.6623666787803355, -3.6278750333350986, -4.4121632637154491, -0.21779419967463998, 1.042724871250007], [-4.996228445911016, 1.6343749021419995, 4.6719036756009054, 1.6509915244209878, -0.85238463050785551], [2.3973502985718476, 4.1402685941541169, 4.138320474721457, 1.5341747876297323, -1.4411412828036885], [1.3460778188203193, 4.3299704194971227, -3.7514661585454223, 0.86815922178031713, -2.5991232177727785]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[-3.8723220655583135, 4.2046875791737559, -3.7724738475427353, -2.6116807613391568, -0.027609940446616577], [-0.40618016556430359, 0.96018377180333658, 2.9970708743477044, -1.058093048031945, 1.3863835187484526], [-1.9266003169988624, -4.2425348975222352, -3.4603628306083136, -2.767440171031581, -4.0641962264300719], [3.3213800934442297, 2.9999805098486521, 4.5018003661441206, -4.2615551361930049, -2.9648733361660415]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.9222791231958014, -10.997599101281251, -10.275744794224918, -2.5383453157333942, 6.3953619415250964], [-6.113912288567156, -3.7624842576340796, -16.954447126545496, 4.9482512890638448, -6.0578332695844717], [-8.6182174869412336, 2.4083098175123907, -2.4018736477477489, -5.7097968344418639, 9.9066723048857188], [15.753739097774663, -10.145837602732989, 16.209665742927719, -16.485498423567805, -4.8843031365305398]], [[15.441907353979015, 3.351635372492265, -1.6062216348480718, 5.9302800927565071, -1.8037091178370868], [-10.756124125555822, -15.062362375639045, 6.6182001761767451, -4.9393042255661399, -19.609596065184839], [9.4260627795053775, -14.947101431233675, 8.9267000552930647, 1.5591421160055967, -16.936132567635493], [-2.3315580334513952, 8.3747893104754993, 12.42024050584973, -15.520982054262699, -7.2605693814105638]], [[14.408014432096044, 17.60296575487321, 9.3221905915070291, -1.6831156267521759, 6.1163732186459256], [-18.755990535100434, 8.9166553734233798, -6.0058490969830975, -15.505881805369608, 16.32804819510423], [-9.8971926335087659, -10.435112242151039, 16.15876944770115, 15.047484052101014, 3.8058790868545205], [6.3843753127033409, -13.505388365284325, 15.476697115300192, -3.9268262892968808, 17.658180604887999]], [[11.516089418679069, -12.477779166502206, 7.7519458784529114, -11.61750076542849, 12.768425917535371], [7.9052016676589512, 11.822324314582447, 10.306839457974728, 10.230650533466704, -12.907969477624377], [-9.8541452145068096, -11.190995915070104, 1.2532158860891642, -10.814491936836424, -5.5575830154162631], [1.7489168347272224, -2.0800790233227247, 3.0397595268698838, 9.0463936862008936, -7.7442075353411353]], [[0.092767045252195465, -0.071298747397302256, -0.023194703915917582, -0.020219223805298617, -0.052157969470020854], [0.040996978130354023, 0.10801989962464355, 0.12465534886342487, -0.091277298630823483, -0.033642354271806854], [-0.05619610761897894, -0.061113797294305737, -0.07290924055345889, 0.045489921778352496, 0.0030417550641969782], [-0.11490077441476479, 0.12563089507110445, -0.041280728647117744, 0.051549247767499695, -0.091059050312636683]]], [[[-0.52973445441584321, 0.18422226614486195, 0.55771936777099596, -1.6383672003080634, -0.06913314402345383], [0.1595904022383097, 1.3361857966466519, -1.1152472524002235, -0.182474663769883, 0.61642292792989184], [0.14941306128403445, -1.865854143777532, 0.73670478858827204, 1.0654941072009114, 0.55618199316851058], [-1.4985072929477969, 1.5034346270993741, 0.59776773603650801, -1.3036068579711686, -0.62889319643862662]], [[4.7994904528791089, 1.1823854808957472, -0.66518029368048803, -3.9580081725179834, 0.90575348766758901], [2.1207711930956519, 3.584195168189976, 0.64690406694644642, -2.8909948222051574, 0.88724160664728102], [-2.8173403364743699, 1.3572723079733797, -1.169928364278858, 0.79625458183836395, 2.5417280908942055], [4.0559613236150209, -1.7835017311733652, -0.57374263118389079, 0.9685485518744622, 3.6317965598181794]], [[1.470345175096528, -4.8986319244907488, 2.1679065406924201, 4.1082754295623083, 2.2475016452815808], [8.8944433507709366, 13.954179255314077, 14.020123103713305, 2.7291047421664669, 3.9722892259076739], [-5.5978072088689625, -11.2202023676232, 12.241564700210747, -1.8025326451234205, -3.6219359177584085], [8.2427805833011032, -3.0807631758497989, -6.8766535441529895, 14.50507400817798, -12.399605831191304]], [[-4.079397619658188, -1.8931097028405912, 3.157210257538658, 3.4367808915283198, 5.1177098873382345], [4.6111450830158676, -4.519398642873135, -0.45905496838808157, -0.88153128836916572, 4.8790277900532306], [-0.72554104080591353, -2.4907110240345531, 1.1212456952333931, 1.1701861041106851, 2.1970255478770055], [-0.95520495973083275, -3.4753243073838758, 1.7802224818169881, -0.53851975273225072, 2.4965220670533452]], [[-1.0451756891342441, 4.3625383033221379, -4.7663860444518429, -3.0047108370742119, 2.2478396211942098], [3.54046418919464, -5.7799611290979493, 6.2005333653755113, -0.58052430448647463, 1.7821838446867893], [6.2378490473245343, 2.7797541818380771, 4.8123690751888679, -1.0165385301784302, 6.140870619211662], [5.3752176259246092, -0.79632619661530268, 0.044457883254971069, 2.2425746266670332, -4.8304326749163291]]], [[[1.7630938743830047, 6.7247243063488691, 6.8404449733228843, -5.0487078411077322, -9.4905683188135352], [6.5573209400143231, 2.5945108811889797, -2.4219402499237539, -2.7123918813053223, 8.9283965087527779], [5.7047351975759515, -6.3527217745633191, 7.8846014479766069, -4.2570240956504728, -6.9996311710455377], [3.2354719265962735, -5.8197095355761714, 4.4548530936531865, -9.4204395532004099, 8.38528036037156]], [[-17.187793825548447, 4.4092047216820465, -13.597196232552831, -16.48454039752918, -6.5108939986994541], [1.6240965369770473, -8.2422691720707402, 3.8948541173615818, 3.6172873600251081, 5.8033591737877632], [5.7706594268273852, 7.9476515718113383, -3.1494896515078592, 18.591683369628271, -9.355617069139825], [-12.677206464591576, -1.0713885903890965, 16.946267173307699, 14.534432238839242, 15.363806059061638]], [[13.26992211957648, -9.8877779969828747, -4.2465057814242941, -7.1373484363154427, 7.9398567043274033], [-1.9220060381888282, -10.90865596526254, -14.037669783759501, 12.61597607091865, -2.0328945699731453], [-14.741947669450942, 5.7129072522143467, 0.75977341812682198, -12.867570650550409, -15.073901926626194], [5.2718883681896846, 4.8474990846368815, 15.05452868680263, -14.53253425745582, -9.0945063422133714]], [[-8.9654079677092966, 8.1952729949150189, -7.3649887982363218, -4.4467358905564156, 13.627352007371973], [-7.7718758434347093, -7.8908301965533578, -6.2954831961221513, 5.9455010826042889, 12.918529845953975], [-8.7670455264528755, 3.2766420916239984, -8.439169086723739, -10.69970758473737, -6.9246028103321162], [4.8952740488082025, -7.1381610886601408, -8.1774800422484777, 2.0158147095660026, 10.22724950196063]], [[-12.512209822186191, -7.7302532554506067, -5.5526815540013992, 1.8036904606721416, -10.672805990422345], [-9.6324777656456355, 7.3563011041686162, -18.230930161973149, -15.377302198090405, -11.561586971456833], [7.8935340279565249, -11.148575581825943, -15.266917740181192, 0.62988913193410179, 19.585307078497383], [-0.15628374829341371, 15.471796527916981, 3.9576259194164649, 16.937913781745628, 15.164390504333349]]], [[[-13.625761412586311, 0.65314573589378289, 7.213350397725212, 14.838094467111196, 1.2507317494355348], [4.013532468862036, 6.7511251434081245, -13.811371622540339, -10.130086130440311, 12.567992150973765], [8.2141826371816915, 7.0306804185168161, 5.2642344302580524, 10.549029854501738, -8.1562345821159656], [-14.883266954763794, 4.1670613805135446, 12.180445662461141, 8.5365652881329108, -2.0656251940062136]], [[4.0869645901774767, -1.1931459434429603, -8.5091013909718107, -6.1496180917185024, 0.00028214467863484744], [7.3979301868888978, -9.5666759678882887, 1.4956017790019849, -12.202783111034092, 2.9626739180622601], [14.987420170592875, 9.3738674798979265, 9.7780358734166057, -8.6042501862881053, 14.128113295973401], [-12.260998489271666, -6.8054077886225617, 12.766259546689875, 0.15765783201102146, -4.2178016297233496]], [[-20.693345617149138, 1.2894684245145283, 4.8474911537672662, 0.80346167887073572, -12.025241051632976], [-6.5046597036102263, 17.385580033567472, -21.141633590336095, -9.0513124352239522, 19.807070430962252], [18.853311246761802, 10.671902521022902, 4.9446005676156819, 1.899970010300877, 20.582270254233492], [18.067529964021755, -3.5651251099689185, -19.912073985191306, 19.498472788912686, -5.5377520068027044]], [[-4.3231937740236237, 11.691343444870453, -17.665280097908422, -1.5101661787180101, 20.228509622903839], [-8.5998391130878673, 16.309039508157888, -3.0627636424050908, -14.209251853028569, -10.126810304591096], [-3.4072694831057677, -16.206335749395169, 19.400872897838237, -6.2933367793751263, 7.6373351419596931], [11.513418763055924, 1.8156669047023331, 2.1194755834860763, 11.423453279072998, 21.134584415681271]], [[9.6177171396994776, -8.0993662722951534, 1.3143361085236895, 4.6933190988282698, 6.5274679266710294], [12.90221093883739, -3.3267772224909367, -0.17031155053647273, -1.7015423708845885, -4.6906064164435382], [14.587333595264683, 2.7013072817716597, 1.3302177112549534, 9.5824962665218845, 11.840231519462598], [-14.556935199123515, -11.344800343194148, -14.498649551202885, -11.121082965072203, -1.9866563333176945]]]])+(1.-msk_ref)*numpy.array([[[[12.07226545549992, 12.947556136191219, 15.121533585015021, 18.210638802890372, -17.171554029132977], [6.3515289958720826, -1.0256250900334665, -16.850791641430842, 15.397595212912384, 5.3871266613743423], [0.78128043068912389, -17.398836758641021, -0.47227385265559207, -1.3874750408347101, -13.637813392475806], [-4.4525121159307686, -9.6846154338408255, -5.7217413674278452, -2.2688064459698372, -18.127217882819789]], [[19.735062694392692, 12.715037992102141, 9.6380653893983439, -13.164810467105022, 17.391438193916905], [-1.5826548190851717, 9.1056034784718864, 5.5832539590968571, -11.085790857475642, -10.677955494034986], [-16.243196780269436, 8.2935938715972259, -4.5411989285651337, -20.356835414774334, -14.806070431323443], [3.260533980559801, 7.4227393922740319, 0.35645191240877477, -6.5235839247374647, 19.722478976022153]], [[-10.796087904466427, 0.43559569535460618, 11.951169367263287, -12.818088598541751, 7.2323262893794613], [-10.894165636068539, -15.349693729863578, -14.351878674448763, 11.356247358755265, 11.455838527186094], [10.258328794966722, -17.309719554453675, 6.7741556168509094, 18.752546701582411, -11.958333151168592], [8.8163784778893515, 4.7547152397965684, -4.8908030158400395, -10.006292608328215, -12.125673378620968]], [[-6.868341352058529, -9.0515848491584698, -1.6991692341725242, -8.7148737400331555, -10.685343537177786], [9.2515179221461032, -6.5363109415483107, -4.9213102945454095, 11.316576861787453, 10.778532154227083], [1.7189214044566599, 1.7913810887554551, 9.5118509870512415, -13.034053240997704, -7.0272139845806931], [10.987474267671008, 8.8839460059044004, 7.6983118875400072, -12.201695552082507, 5.0710712851234394]], [[-0.11770479521846085, 0.10003991969209886, -0.11382305433763461, -0.1319846422124725, -0.033563586018171526], [-0.050786434298525973, -0.13179375219118714, -0.053104405735344251, -0.094615163532549632, 0.1214409223062418], [0.1218958279118097, -0.043384909995880246, -0.039288531273851582, -0.053412317108767406, 0.01092484623907927], [0.0932709801069148, -0.092894253716520492, -0.072896101066231372, 0.00044000368511465257, 0.084519004105002724]]], [[[-0.59017125767125478, -1.8864021450531212, -1.8386126989947769, -1.9147361514851915, -1.0520535885580169], [1.5797825108172341, 1.9623788015853445, 1.1759068600928719, 1.9273662679766193, 0.34332237443614416], [-0.10017754112544924, 0.82172179505624965, 1.6120699369219145, -0.22602108127561349, 0.49964906375313761], [0.65350899348060354, 0.42455042129833198, 0.85960384179069926, -1.6676558431227424, -1.7304055177487079]], [[0.91513152887440607, 1.0737535440881889, 0.69815883398148659, 0.54381238208314475, -3.2124831891734158], [4.1172440934135848, 0.79810178517550612, -3.6359962372353687, -2.3108223734070306, -1.5613579350167304], [-2.8726542316694479, 1.8360049104775129, -4.1952076704007304, 2.8492505098635759, 2.6029628546830645], [0.87668362234772046, -0.044447577899000001, 1.498309966683161, -0.64176680701824029, 4.031615436713496]], [[5.0021459213211239, -0.86762128780402481, 8.8156436704627765, -7.1763024536011146, -11.368709206640686], [7.0773091270008344, -7.8986887812982465, 12.539577726532071, -13.536240312554879, -7.3121858723666939], [8.095522774142264, 9.0224716555886317, -0.1594014265749549, 7.3758632768265082, 12.239436655329039], [7.7984058716073426, 8.0978946619990939, -8.292020455744284, -10.763945772168613, -2.6337020330185803]], [[1.0249096394502808, -1.9749743276995082, -2.0310191930084671, -4.359462161052555, 0.66389125105218982], [1.1769830288076391, 0.15079537056789943, -0.81579403099264958, -4.1342078617927704, -2.574675997933721], [-2.1819472743290862, 1.224898054954134, 0.81361026691460303, 3.1603181794601038, 1.4470642128354025], [4.4042563902829208, 4.2325369104636961, 1.6512849184235374, 3.4628055426395532, 3.6219251690974574]], [[5.035287703403875, 5.0066370207610262, 5.0081766884062491, -6.7734398407694751, 3.2194241450042398], [1.9248117254645967, 5.5710235392134253, -0.45018088073225282, -0.28899082127865083, 1.0073184769873789], [5.4847900372960359, -1.3122102532556272, 1.7221282031946163, -0.97546693425812159, -5.0726832949108012], [-5.6585842951667038, -3.7126374979122314, 5.5163403682345225, -2.5333341521000277, 0.13043683991991506]]], [[[0.20727906689859782, -3.0602665564408542, -6.6653665071646868, -0.071180849031566573, 5.9013007913190707], [0.74428673099873499, 3.3115572349853193, -2.9237082157608394, 6.2436733061076479, 2.2944898121585453], [-7.1525378686656369, 0.20383258724729711, 1.69589807428873, 0.22766586445336723, -2.704714081001478], [-6.7772493523912951, -9.2050954826487885, 6.0439074831884421, 8.6787326788496557, 9.2335569807560276]], [[-19.046246735323656, -16.728637434998948, -4.3145694845031288, -4.3561823404323574, -1.959067414396731], [-0.8935268837858964, -13.839183807578815, -19.258824717787348, 15.722703352131454, -0.99401888018734219], [1.6925364087077945, -1.3151248254145418, -10.937451987120163, -17.946903586518829, 18.772445225459069], [4.2594866176592641, 12.531159193882125, 0.2249472453159379, 13.722324771202965, -6.2735936294458199]], [[15.741380092804336, 2.8773393051342322, 13.372684619394631, -8.701812187157536, 16.783687904108302], [-11.566743948136605, 16.171023299858678, 3.6262661617669822, 12.24255433271021, -7.3743157207733896], [-8.8999819307139276, 3.4278688435532856, -15.608034799837075, -3.438877496307585, 1.8074879861495492], [-1.1258068430027899, -3.6480445626032156, -4.2851422265505965, 8.8751069944704177, -15.937465569097441]], [[-2.0039974951887398, 5.1737753300324076, 11.423885200367419, 6.6983694252970922, 1.4182004942952928], [-2.0618518179534249, 13.836201900957274, 8.3436275332409107, -10.515863162509417, -3.8353274108864928], [-6.4292127328518314, -0.044270184347048616, -5.2294382677672155, -10.207761353460203, -11.631856349520705], [-7.4335865602287408, -11.17708461167156, -0.13585610969012293, 6.5153438103413128, 13.509640298305069]], [[-16.737050514621913, -9.3785431676822668, -9.2055408900474518, -9.8971747396645675, -14.87648853391763], [-8.1301251559791439, 8.1340863038830484, 17.932750580426319, 11.123468696815708, -8.9688334901797599], [-1.3615024866257568, -8.0009019291437582, -6.2534525023891421, -4.3385857716321476, -14.942505304962385], [-0.67794637724509621, -5.8190166623347892, 10.055153210009292, 5.9459279510621075, -15.447938136264757]]], [[[3.021619565177557, -3.9637483663994368, 14.823036887704717, 6.2645138923403163, 7.9414728390919151], [9.5563088762389352, -5.7649388602534044, -2.4889117402424694, -10.460241805040612, -11.875680996883496], [11.520128592558811, -5.2963347630957234, 11.388656200226388, -9.9648593499563507, -10.739830928371761], [1.6076693480884043, -3.4896619618304019, 0.11137342278436332, -6.3179288589444162, 15.279977737462385]], [[1.2034748311953143, 12.731082288358662, -2.6008357662385628, -6.9438323649293476, 9.0794678454459312], [7.4691521890166372, -11.556000918750202, -4.1857629926915569, -4.2743586947369723, 4.4234186609902215], [-12.851398415027566, 5.1802853048627133, 5.2949180429258549, 8.2972278985943841, 14.23944757637798], [-9.7302598509602785, 9.2347577242619572, -3.3021788333121029, -11.084585923222614, -8.2347575394590429]], [[9.76838444006491, 15.755984213679119, 0.065178942751640601, 5.0260026936002058, 13.276250792242836], [18.689388716540662, 2.0297253264179309, -0.45345151047304527, -9.3641002086642811, -4.0622636381910056], [-20.951045238622143, 19.236670316225037, -19.369477575364357, -8.5903712303994837, -15.576115514389775], [-11.623750640135709, 5.3646141204064559, 8.2338671122763643, -21.252261134399948, -20.536105171815958]], [[-17.466687101514037, -11.860937061663106, 9.2850256154626916, 16.551515932338361, 15.048360813288989], [-13.872556406888162, -7.8343288561365316, 18.599221260273701, 13.122267830684269, -1.2159767009379421], [-11.05312306945147, -10.05507882597526, -2.312584565815746, -3.5090542859910956, -0.82825646459708813], [9.4829903911895812, 0.12470700982361259, -17.921816474100257, 1.5563219964507289, 2.4536179191303615]], [[13.823326649344841, 10.756189953277723, 13.081505215401274, 0.64573221538696279, -3.0915471677263144], [14.81318450067587, -4.8457145686597975, -13.851602636925248, -4.8949807489919133, 2.5272124631504842], [-7.1078399776853693, -12.275371959373203, -12.269596032011643, -4.5486339208615929, 4.272801363032781], [-3.9909502334249085, -12.837813843154709, 11.122621985000571, -2.5739821282031232, 7.7060711257845949]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-3.7100272149590374, 1.4231892493493783], [-0.47274567780448962, -1.4967428338537458]], [[3.405370914009012, -3.8297176493023355], [3.4228643833062318, -3.6369935879448279]], [[0.82782245041277491, 2.426462732748905], [-2.1935839704990565, 2.5975638936940815]], [[-2.4582167907843413, -4.0869541096983051], [0.38742419610956613, -0.49758143190127058]], [[2.2550110113713204, -1.9207510346673429], [0.31856789386794571, -1.0947937610657341]], [[3.8120756015811228, 2.5620154764321104], [-4.4820327244360847, 1.2597080848059496]]])+(1.-msk_arg1)*numpy.array([[[-0.83715136744182939, 2.9616350362152009], [-3.0055859967015586, 0.10657999605450641]], [[-4.0096337132437316, 2.3477680966738381], [4.3627133934386908, 4.5563087682294263]], [[3.2799219909291857, -0.64416967540887082], [-0.73793885678830407, -4.7582073617825982]], [[-1.1342861526698576, -0.73626371806429347], [0.076217147845013322, -2.4500715901634229]], [[-1.1156501541135921, 2.0036204723173174], [4.4970852565936941, -1.6570667627565925]], [[-0.13392280116009481, -2.3863816463817136], [-3.250530215061934, 4.2078895763189124]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[-2.7445934604562785, -0.75410399733444233], [2.8575705419235922, -0.18362900366132884]], [[-0.69811354711499352, 0.44440990117433632], [1.9888297812640801, 4.4815188694680899]], [[1.4418647479967071, -1.6778518801110662], [-1.3019920434200216, 2.4084823889522085]], [[2.21720998681712, 1.6091045438057474], [4.5392368218343275, 2.7385982690092234]], [[0.54078061078377004, -3.9298889321403649], [-3.0665193100910493, 3.4607118254146467]], [[0.34598768436137295, -2.6234216105372763], [2.9211243134575762, -0.31256033204780032]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[10.182516432291393, -1.0732327018977705], [-1.3509041227158114, 0.27484539531779717]], [[-2.3773355680210591, -1.7019644420520625], [6.8074946227475435, -16.299255392509195]], [[1.1936080088504322, -4.0712450581621855], [2.8560288761634709, 6.2561868921403221]], [[-5.4503828182885723, -6.5763364282411159], [1.7586101766501061, -1.3626756480959503]], [[1.2194662320535097, 7.5483382325363451], [-0.97689459812109147, -3.7887657153103631]], [[1.3189312100015405, -6.7212467674029543], [-13.092574765062748, -0.39373477727024619]]])+(1.-msk_ref)*numpy.array([[[2.297640168492876, -2.2333808194556188], [-8.5886740053924324, -0.019571178485717371]], [[2.7991796141844443, 1.0433713878230801], [8.6766943239905441, 20.419183719943085]], [[4.7292038948999684, 1.0808213009953092], [0.96079052006883869, -11.460058633836137]], [[-2.5149505856079766, -1.1847252941765685], [0.34596768395327532, -6.7097618157702259]], [[-0.60332197176255553, -7.8740059183696767], [-13.790398778470324, -5.7346305413733063]], [[-0.046335639856569792, 6.2604851821073124], [-9.4952028428458988, -1.3152193631947171]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.9772852556633405, 2.60971816003432], [-2.5081856148901438, 2.7545711421579133]], [[2.5055086402332014, -3.8299095076938494], [2.3745336479035668, 0.20112328320634365]], [[2.9906579870570784, 0.004759977978795682], [-4.8028699156907093, 4.6228721051647561]], [[0.53973686367646856, -4.6962938415575985], [0.082930527397627074, -2.6026134602509221]], [[4.9269064049933569, -3.9256298764059694], [-3.3698449281049192, -0.6636484632453552]], [[0.95815381895417318, 2.6332819114039454], [-0.99002418575373774, 2.8465193527544939]]])+(1.-msk_arg1)*numpy.array([[[-3.8833528070013412, 2.7216040153575891], [1.5845372920656056, -3.49822511063861]], [[2.9687435898003081, 4.3150320227286514], [0.14445496102070265, 0.30415737520959851]], [[-4.3740785671868965, -4.0588898556887525], [-3.0965807312677027, -0.18541810546676629]], [[2.7935867526728195, -1.0623244369731113], [2.3221593657165727, 4.7804005422984766]], [[-0.25673032812687424, -3.6300636818989576], [-1.29397708503034, -0.29781906331275465]], [[-0.59797015547838406, -3.5443793121604559], [4.5570536380160167, -4.1211679532084702]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[1.6409065891583117, 3.7228882290587073], [2.3766358729520096, -2.2078516574657101]], [[4.4399146630658475, -1.7809381443387204], [-1.5911475119337704, 2.0104485694248844]]], [[[-1.3296789941462106, 3.8670693140747421], [4.1739485505155809, -1.4618415736995747]], [[1.0323887467580199, -3.3857768098762389], [-2.1811427870970421, -3.6851285356251209]]], [[[4.3256685487775464, 0.51003298875813741], [2.4907208312672724, 3.4764325090356643]], [[2.6649538613894723, -2.0429152431945283], [-1.8193399889286601, 1.5763268857820538]]], [[[-0.20239620435870265, -0.94190344889822253], [-2.3290324590484368, -3.5659137097135316]], [[-1.83689894425242, -0.096287031339560514], [0.85375771880199203, 1.7862764332854661]]], [[[3.0678502315018807, 4.2850686536412326], [3.7225534054684228, -0.053408844243070241]], [[2.6538760408967912, -4.2666554696151486], [0.28127439679410227, 0.6670336018121672]]], [[[4.8803313145588874, -1.3700239950352344], [-1.6133748030570914, 3.5389868963542046]], [[0.31438987242153438, -2.1735270732259449], [-0.17632780259046399, 4.9454568987248138]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-3.2445404046635522, -7.3612120038003868], [6.2023497974318786, -5.7618705651501365]], [[-11.136130089241579, 4.4669234346395248], [-4.3829290192891275, 5.5379236121304469]]], [[[-3.3315222085699232, 9.6889755787949454], [-15.985845238244584, 5.5987209418541397]], [[2.4514418168939129, -8.0396409593427265], [-0.43867859848279211, -0.74116515012230966]]], [[[12.93659519476317, 1.5253342314921168], [0.011855776308159892, 0.016547742187779182]], [[-12.799426727571285, 9.81185616184497], [-8.4105760846290583, 7.2871575889030868]]], [[[-0.10924069256058776, -0.50838001339437555], [10.937820794216924, 16.746578594453467]], [[-0.15233499822299756, -0.0079851342905415991], [-2.2220013307471862, -4.648987088997762]]], [[[15.115010955146969, 21.11213219546125], [-14.613366865023625, 0.20966335462490951]], [[-8.9431507162352144, 14.37796729425372], [-0.18666732118267024, -0.44267582477565892]]], [[[4.6761080868062379, -1.3126937229018631], [-4.2484706852051417, 9.319150178865117]], [[-0.31125357745335108, 2.1518443708842208], [-0.5019205025024297, 14.077338770433403]]]])+(1.-msk_ref)*numpy.array([[[[-6.3722192090349266, -14.457288454467383], [6.4682617348690785, -6.0088979362725849]], [[7.0352103572167337, -2.8219629045668206], [5.5661921809768629, -7.0330016692096011]]], [[[-3.9474759903636842, 11.480337237472865], [18.010721656696571, -6.3078932026697103]], [[0.14913367617114184, -0.48909225709547105], [-0.6634106650807845, -1.1208590227057282]]], [[[-18.920814087962313, -2.2309243646852441], [-10.109561515383389, -14.110456644911455]], [[-8.2522447768961005, 6.3260519776892492], [0.33733857394707972, -0.29227954475803614]]], [[[-0.56541135528773245, -2.6312889971389146], [2.4741880957507316, 3.7881572739661262]], [[-4.2655720872706411, -0.22359383162220559], [4.081303861952553, 8.53911683037283]]], [[[-0.78761019657758491, -1.100107081495497], [-13.513105921120205, 0.19387750577896751]], [[-3.4340547834314892, 5.5209544074013666], [-0.08376887738707961, -0.19865532248983259]]], [[[-2.9182924749528043, 0.81923346132033603], [5.7184122747165045, -12.543511941444782]], [[1.4326915118739445, -9.9048794563705975], [0.72667648929548967, -20.381058484998448]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_expandedData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.1048579480050762, -4.6973468441199318], [-4.4655029874369658, -4.946225904272481]], [[-0.85838306873632853, 4.5967628057186065], [-3.1836167408689553, 4.1159448621062662]]], [[[1.8192136131828276, -0.24022096086781541], [4.1307907861967479, 4.7313283654303024]], [[2.5389264566747736, 0.94784116632347004], [-0.32550189755989134, -1.6219619903733342]]], [[[-2.1198413791716741, -4.1131404378801228], [-4.0984430526109259, 2.3664967249248923]], [[3.6023022315000972, 3.9465173371893556], [1.9240384105941608, 3.1897868473286319]]], [[[-0.30998276174020756, 0.77190756905426294], [3.4630627101309006, 1.8165281140446501]], [[-2.1411299082012203, -2.1021589594958634], [2.7192956009707032, -1.9411923253066665]]], [[[-3.2989235036326727, 1.3860643429625714], [-2.6673350662258954, 4.0844398710819956]], [[-2.9767065618359636, -4.0609644201604045], [0.66014562603649374, 4.9517974832942979]]], [[[-1.8506028601798574, -4.6994824184728596], [-1.9991048720862556, -0.055225136522849994]], [[3.6271569603375458, 1.3861252639385624], [0.22851173090213628, -1.0165732487619463]]]])+(1.-msk_arg1)*numpy.array([[[[2.1614483270124616, 0.86837342474314916], [3.4713402619511839, -4.3561122901827867]], [[2.539008017088074, -1.481650116373693], [-2.6701198460224917, -2.1817611296206918]]], [[[-1.1189074653805511, 0.23637836725902517], [2.4638968803889849, 0.39089764380014991]], [[-2.3356848475642753, 0.53528439323719201], [3.0165475486440236, 3.7733618899938968]]], [[[-0.39368520421793463, 1.8713277657409568], [-4.8766568145081957, -0.80680102752202743]], [[3.1752638398962425, -0.95302718565438838], [4.7396680200827763, -3.8711490775969848]]], [[[3.3210135994158829, 4.7440355576971243], [-0.84690483058227173, -0.16610675915321238]], [[-4.8043734969766891, -2.4847777532718283], [-2.9861112048950931, -2.022617946820684]]], [[[-1.6059575904657128, 3.1502788056754234], [-1.0719896833603682, -2.9576401750435508]], [[-0.24032480873102458, -2.4697847500655659], [-0.78599960633844468, -3.2146597062394013]]], [[[2.6852546035803568, -0.46533900549928653], [3.1651854913894297, 0.023546878502137325]], [[0.63409501111331679, -0.82658019527636561], [0.79929677637710128, 0.61789634018710071]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[-3.5428037739644278, -3.2524783047090033], [-3.1186737628622065, -2.6673793914081623]], [[4.3588505485034279, 4.1062704309072835], [1.5099256032570763, -4.6008814876572979]], [[-1.1341166116672285, -1.5460448167539695], [2.9328446542331275, 4.2956453633255638]], [[1.1885476285640983, -2.5964108735405764], [0.39357835061665813, -1.457662234648569]], [[2.5374411402943569, -3.2601410837389224], [-0.9344432604047368, 3.4109700164700758]], [[-4.6383087872932922, -3.9768413092555752], [-0.14206736399080278, -4.2495716709130695]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-14.542706229780261, 16.641778126967989], [14.523951586251972, 16.087492443835917]], [[2.6770167549531338, -14.335803556295481], [8.4919136847358718, -10.978786501354564]]], [[[7.9296802556668711, -1.0470872670406977], [16.962144061623956, 19.428113765879342]], [[3.8335900617200087, 1.4311696448528564], [1.4975956546806262, 7.4624548951924581]]], [[[2.4041473222181637, 4.6647808967200657], [6.3363766382504387, -3.6587099954353741]], [[10.564992842587126, 11.574522275214159], [8.2649866773290945, 13.702193080724106]]], [[[-0.36842927636207362, 0.91744891067012224], [-8.9915336763367684, -4.7164533473976862]], [[-0.8427023777258329, -0.82736425601241215], [-3.9638145023809788, 2.8296027427891675]]], [[[-8.3708242168015445, 3.5170566869282958], [8.6958886335005214, -13.315850227775721]], [[2.781563384910172, 3.79474083316232], [2.2517369369143476, 16.890432743148832]]], [[[8.5836675081623319, 21.797650597332996], [7.9501228368467043, 0.21962160423334864]], [[-0.51530062813604793, -0.19692316240880731], [-0.97107697811302895, 4.3200008793468321]]]])+(1.-msk_ref)*numpy.array([[[[-7.6575872901688475, -3.0764766463904438], [-11.290458890259094, 14.168160716695764]], [[-7.9183376865893731, 4.6207833436763712], [7.1222226498703298, 5.8195846741256254]]], [[[-4.8771504191985953, 1.0303379757813467], [10.117426904745988, 1.6051314362478835]], [[-3.5267103524769006, 0.80823961037276515], [-13.87877777319429, -17.360790865904473]]], [[[0.44648492987116495, -2.1223039050009391], [7.5395299911583198, 1.2473505467522072]], [[9.3125555786194472, -2.7950806867853153], [20.359932954171033, -16.629083585921521]]], [[[3.9471828380148679, 5.6385122119246764], [2.19891291097785, 0.43128139563398632]], [[-1.8908973966864713, -0.97795472978169173], [4.3527415318365126, 2.9482937962029387]]], [[[-4.0750228596156965, 7.9936470449181911], [3.4948376080674151, 9.6423242455762583]], [[0.22456989782676337, 2.3078737143491654], [-2.6810210901777176, -10.9651078711371]]], [[[-12.455040023906536, 2.1583859982776623], [-12.587440413613891, -0.093642199131321754]], [[-0.090084206748587706, 0.11743006946991627], [-3.3966689375442685, -2.6257947828199679]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.2932531153253546, 0.61441133097337541, 2.5675941968330456, 1.3783487989283758], [1.8243254506582325, 2.1488485535737514, 4.5550381838279108, -1.9084986325978615], [-1.5029593060018365, 0.39619130827128668, 3.0636267705427329, -0.95623028799289767]], [[4.7264805310071072, -3.4557951789142205, -4.6522474034927095, 4.6710104244729429], [-1.5505751267321353, 1.8005549643455794, 2.0900981044227374, -3.661839214773809], [1.8325434334607662, 0.078727374774883785, 2.3960989289460457, -0.64280728697782674]]], [[[3.3913338880934454, 2.6881354779109152, 3.3408323617185971, -1.2294933144150622], [4.1057416641607762, 3.5065035353479495, 4.4763824938954588, 0.9888509059593158], [-0.9581510974855556, 0.76547907156817807, 2.4507031313697558, -4.1518763543669444]], [[-4.5634643591353798, 1.7577368792809098, 3.8335104882828102, -4.3683781927779037], [3.9456721559370926, -4.1412348066351976, 3.5072208969525125, 2.2385809434701995], [3.8179506908060592, 2.1842140830320114, -2.9001098925646938, -2.455319947381851]]], [[[1.1216330577956022, 2.2724432896608633, -1.7235652310573313, -0.82072021836832487], [2.0644004586595441, 3.4643515647882754, -4.9153032299106778, -0.89346685808566662], [1.085982115714387, 1.5476136964598064, 4.4564373905683752, -4.2937039444838128]], [[-4.1298379936766434, -0.1547820401758857, -3.5383094564369886, -2.602922838754699], [2.8419741828770908, -4.9098890332430898, 4.3055128415591852, 2.9692881972549134], [1.2778624337125679, -4.745563649102607, 1.0665382630039666, -0.98619228722444596]]]])+(1.-msk_arg1)*numpy.array([[[[-2.1196681321816992, 4.0660377961800283, 1.9549269984875473, -3.6541843043854416], [-4.0910352658657825, -1.7418084531663833, 3.9378674030717562, 3.0430406566194623], [2.0595321244260942, -3.3996868202870365, -3.6714467102620221, 0.43148501683187401]], [[-0.13008147445352591, 1.4529666629354043, -0.45915401938620981, 1.3754016663249402], [3.3147782401930144, 0.23289302218320174, 1.3822925466549476, 2.0757293013580735], [-2.1987820383228307, -4.4509030257894144, -3.9152476988732743, 0.084902637768883515]]], [[[-1.3253768465034952, -4.7246760308912847, 1.4748811900669239, -3.3067557188400998], [4.8364923769202992, -4.4949163298956618, -4.456119709991075, 4.5561585045396829], [4.8962236492356688, -4.1552845996382697, -0.85330265325088117, 0.19777000219523533]], [[3.5119503063972672, 2.7156815160565753, -1.2507982084187583, -1.0778066833512012], [-2.3793231580824803, 4.6089663095390208, 3.7999421365149892, 3.8826431802048234], [-4.2425274641326185, -2.9795582899141659, -1.2626663843498953, -2.6736304693930935]]], [[[-3.5887525323861524, -0.23837410562261319, -1.7799488311873493, -2.1151940930804134], [-0.76096609901407231, 2.9721615327829074, 1.2037031752340166, 3.3328369721634648], [-3.3002669186654234, -3.582634610959877, 1.7518112565791002, -4.676117951285871]], [[-0.085618499572452933, 3.5225695177885754, -2.6427806293718268, 3.5990168471787474], [1.737555891930799, 2.9593140387071779, -0.79735167352912217, -3.1782367848993234], [-2.5069991884956133, -0.32966804241887004, 3.7856045701793484, 3.4061107151700227]]]])
      res=mult(arg0,arg1)
      s0=numpy.array([[[[3.5788098519882503, -4.4426880037872287, 4.4516895209567018, 3.474064325858814], [2.9131707375183211, 3.1525801524011854, -2.0365746722358025, 4.8349567218538851], [-4.8734757196802247, 2.0003165843185258, 1.0344539962438679, -2.7405619502224097]], [[-3.2959520550574406, -0.92594212148176336, 4.7387381788572007, -3.6740166170159307], [1.2934906251014633, 0.81847891456288568, 3.0803666958386842, 4.4878520688347479], [-0.27352543621644898, 1.2418267359597968, 3.3424925108295973, 3.2649635175489031]]], [[[2.8296664025874341, -4.6575614719308414, 4.3797455906562206, 1.3961420783128169], [-1.6494935104583552, -2.2648752567645447, -4.9627429914362295, -1.5639874497961568], [-0.050676025323980411, -0.014457737570277907, 4.1162269954003481, 2.3194428918505352]], [[2.8962157379566591, -1.316312657501201, -0.67622858307793887, 0.13553229558338842], [3.0295097736618466, -3.3198549692337984, -1.2543870016851066, -3.0388723871083281], [-3.6642080415869591, 0.83361441866650843, 1.4581191532968596, -3.5368126705081417]]], [[[4.4880736029840183, -1.6627212075592412, 0.70202552877051083, 2.8841985059480884], [1.3116755979649044, -3.3655920508770643, 1.4686094672412544, 0.27308870312799627], [-4.5873443608937254, -2.0422123210613607, -3.1707762427181363, -1.220272362251519]], [[-2.5179125591736371, -0.33770673359971326, 4.8873649023295496, -0.18786537078487786], [-2.0753352142965276, 4.6338492882767319, -0.14988452201198132, 0.74997391837525562], [4.3972699552434751, 1.4414253008961122, -4.2108983051919093, 0.85312569963804386]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-15.364736546205627, -2.7296378495063593, 11.430132180110908, 4.7884723909474136], [5.3145715185674867, 6.7744173005126038, -9.2766753962508925, -9.227508292327979], [7.3246356854673911, 0.79250804449790824, 3.1691809557876254, 2.620608342923552]], [[-15.578253219361859, 3.1998663193702832, -22.045782388420182, -17.16136991776823], [-2.0056543899435302, 1.4737162728283852, 6.438268591899365, -16.433792695762847], [-0.50124724202294524, 0.097765758847377565, 8.0089427252089767, -2.0987423407971924]]], [[[9.5963435630942353, -12.520156233248278, 14.631995805358633, -1.7165473512591585], [-6.7723942306516882, -7.9417930949669708, -22.21513584856752, -1.5465504066399296], [0.048555289280377639, -0.011067095532272699, 10.087650387056355, -9.6300400979787231]], [[-13.216777296532186, -2.313731302754122, -2.5923293657059023, -0.59205632444360301], [11.953452360076831, 13.748298951571829, -4.3994123051756127, -6.8027618154184983], [-13.989765623634048, 1.820792353069931, -4.2287057810142779, 8.6840067000515138]]], [[[5.0339717189266899, -3.7784396506948053, -1.2099867927034906, -2.3671200276193116], [2.7078237060512804, -11.65959408789494, -7.218660857808338, -0.24399570556246017], [-4.9817739345538303, -3.1605557591535334, -14.130365805175009, 5.239488255143927]], [[10.398570951630877, 0.052270937207697946, -17.293009450970885, 0.48899906422707839], [-5.8980490998464257, -22.751685802211224, -0.64532973427354601, 2.2268887040806664], [5.619106086698582, -6.8403755108293778, -4.4910841641057262, -0.84134598501599811]]]])+(1.-msk_ref)*numpy.array([[[[-7.5858891943973985, -18.064137340034474, 8.7027280334023516, -12.694871331978668], [-11.917884222675683, -5.4911907587369502, -8.0197610157189132, 14.71296987759693], [-10.037079802292002, -6.8004499281092743, -3.797942721426951, -1.18251141922051]], [[0.42874230305000072, -1.3453630343206864, -2.1758106816411718, -5.0532485771492306], [4.2876345779799907, 0.19061802800577701, 4.2579679246219415, 9.3155660394407356], [0.6014228161771451, -5.5272503765896515, -13.086686111526733, 0.27720401485907425]]], [[[-3.7503743333182231, 22.005469048834378, 6.4596043889374091, -4.6167008017742095], [-7.9777627891113392, 10.180424776807582, 22.114576859759051, -7.1257747203820898], [-0.24812115364053852, 0.060076014271387403, -3.5123874165580196, 0.45871622581300331]], [[10.171365748309476, -3.5746859533273208, 0.84582550019544145, -0.14607761398970653], [-7.2081827621108445, -15.301099705754279, -4.7665980231999354, -11.798857149318902], [15.54550325072827, -2.4838027517297734, -1.8411180392446764, 9.4561301204061241]]], [[[-16.10658550824434, 0.39634968075168553, -1.2495695193987515, -6.1006396430527499], [-0.9981406629553039, -10.003083228656743, 1.7677698788970355, 0.91016012646515843], [15.139460838783942, 7.3165005443631355, -5.5546015140872163, 5.7061374985823434]], [[0.21557989537108191, -1.1895954457302969, -12.916233292548263, -0.67613063445625743], [-3.6060109293323985, 13.713015252050598, 0.11951067446236587, -2.38359469509532], [-11.023952209391535, -0.47519185723945201, -15.940795868694964, 2.9058405869240636]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s0=numpy.array(-0.10592948978)
      s1=numpy.array(2.26378668473)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.239801768484)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s0=numpy.array([0.76626641075429625, -4.5147321833311587])
      s1=numpy.array(4.04385630994)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([3.0986712602229498, -18.256928227247673])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s0=numpy.array(-2.31337146759)
      s1=numpy.array([4.7721577944047482, 4.8323549276862749])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-11.039773680400364, -11.179032010964399])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s0=numpy.array([[-2.14189300222254, -4.4029797844639269, 1.7483287754508581, -2.9744998451191673, -4.8001287719926955], [2.0699763663882917, 2.3596046116059313, -1.8554826043472508, -2.3802956515864357, 4.7540182810708167], [0.43431035012110897, -2.8722927082944452, -0.80111014006963366, -4.1470643141531625, -1.5883595688231491], [1.5983830383546005, -4.1223994602396319, 0.69604980685959994, 1.9208060138915819, -3.2343436783541391]])
      s1=numpy.array(3.10328827256)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-6.6469114348677625, -13.663715529429675, 5.4255681854294862, -9.2307304860790502, -14.896183324885339], [6.4237333822816325, 7.322533319066614, -5.7580974060031735, -7.3867435807851081, 14.753089179165656], [1.3477902161806883, -8.9135522769993241, -2.4860757027040847, -12.869536051648533, -4.9291376225315702], [4.9602433379788495, -12.792993899754357, 2.1600432027425227, 5.9608147767654929, -10.03710080645323]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array(-4.84713237952)
      s1=numpy.array([[1.7320172319018114, 0.35064445937638133, 3.1089187704898293, -0.13358854357027106, -4.8412813474382119], [4.3521122219952666, 2.5515270651641657, -0.30607338626572123, -2.42065070990372, 3.3617125179044791], [-1.842951599879572, 3.5869736031888131, -1.8089770961514273, -2.8171367917997205, 0.36001918988634252], [4.4476872874157074, 1.8084952543634003, -4.0615706423065978, -2.5640636106729584, 1.1179116950001822]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-8.395316806631687, -1.6996201127412911, -15.069340837727658, 0.64752135507190212, 23.466331577516684], [-21.09526407052245, -12.367589454769753, 1.4835782210768165, 11.733214435473752, -16.294665596160506], [8.9330303736578802, -17.38653589648721, 8.7683514565591878, 13.655034961059458, -1.7450606725453646], [-21.558529064796272, -8.766015905626638, 19.68697057201765, 12.428355750432704, -5.4186659742754779]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s0=numpy.array([[[-2.9133674195370785, 2.9771756937077409], [0.83841753531001295, -3.4519065755399438]], [[-3.3497052645939842, 3.5554372634483382], [-1.4900187374251539, -0.96952520290287758]], [[4.3445875425798768, -2.9903342524317065], [-1.4813802327464476, 2.2609171894473032]], [[-1.5333221185083579, 2.8139757756129509], [-3.0780804308384968, 3.2707335149761825]], [[0.25608455317646417, 3.7571605244973121], [4.121718023691173, 2.178687538258802]], [[4.893909172459491, 0.60060610982266205], [2.7782058707391579, -2.8538096748090047]]])
      s1=numpy.array(3.06880062166)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-8.9405437482021224, 9.1363586196441631], [2.5729362535708029, -10.593213044932476]], [[-10.279577598366952, 10.910928084346741], [-4.5725704276967649, -2.9752795453843022]], [[13.332672951529513, -9.1767396128364318], [-4.5460605791685307, 6.9383040764997954]], [[-4.7054598704849226, 8.6355306095398578], [-9.4460151396795613, 10.037229044046189]], [[0.785872435985698, 11.529976553257304], [12.648730833414584, 6.6859576718135605]], [[15.018431510795876, 1.8431404031971457], [8.5257599032264082, -8.7577729041558872]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array(1.41305363507)
      s1=numpy.array([[[0.64709771662155191, 3.4123858448378446], [3.1474331602583554, -0.15823017129706241]], [[2.6633988370776827, 3.8166916124802412], [-4.8603621556046157, -4.4790844045900506]], [[0.70445334115060376, -2.4107822833003723], [-1.6793812946571549, 0.34228894084651262]], [[3.6570466228843692, 2.1281109096665993], [-3.4703105672205914, -4.5874538483908802]], [[-4.2775223751774494, 3.1729793841064922], [-3.8692092495768593, -0.7064304533338337]], [[-0.73716671297162861, 3.4555479724765412], [0.20245917123807367, -1.9760366959187459]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.91438378071538795, 4.8218842222979665], [4.4474918682322615, -0.22358771872852662]], [[3.7635254083648051, 5.3931899569434512], [-6.8679524117172939, -6.3291864996761396]], [[0.99543035444768035, -3.4065646687717765], [-2.373055843078165, 0.48367263210626499]], [[5.1676030240748334, 3.0071348567293015], [-4.9037349618211312, -6.4823183361690493]], [[-6.0443685413232604, 4.4835900527030965], [-5.4674001949479365, -0.99822412000512795]], [[-1.0416561034146652, 4.8828746236550353], [0.28608566787053352, -2.792245836193]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=mult(arg0,arg1)
      s0=numpy.array([[[[1.98128587450765, -0.62304850663361044, 0.4454550996627713, -1.527210306799379], [-4.2232248405048525, 3.3667847674282996, -1.8544009015825758, -0.52196931572940475], [-4.784142550819138, 1.0849982819827941, 0.55830854701128363, -1.1980875140663705]], [[1.0186339332931951, 1.6553388235369653, 2.5420618075543917, -2.6484886247971229], [-3.3672679053216736, -1.4182726464351711, 2.8645587768755307, 2.1202971369773413], [4.4027966858000109, -4.674194212260387, -4.7518903823400986, -0.64311446693304664]]], [[[4.4963402282119862, -3.4897364853522763, -0.45861363973975866, 1.1566192392908814], [4.159072497619265, -4.0958687659955908, 2.2376086767471461, 0.85263670935383917], [-3.8594809831459767, -2.5268945536895728, -3.775521568663387, 2.8289163201979957]], [[1.4345826248133822, 4.8002533341124884, 3.7926837220248064, 4.6216221976979472], [-1.9385382352705061, -0.66849543143269852, 3.0894196208782958, -1.4158070002060619], [-3.4951326606982445, -1.0993225073361748, -4.4106575974507747, -2.2958358819426685]]], [[[2.6046025203684398, 0.6137193941979957, 4.3043748107237541, -2.0984433132313605], [1.301738482690503, -2.285759772682423, 3.524715975993562, -3.4422721338775633], [-3.4990832913871381, 1.6678749430790702, 4.4553763617614131, 3.5068638732900315]], [[3.1661236358785345, 1.7902859880177164, 1.9094247424485777, -0.33690446779463556], [4.7068598369309438, 4.8338022268210672, 1.4454090561789794, -4.7594050814810807], [4.3073602538077402, 1.6824179975523137, 3.7690831201425219, -0.55373404949717653]]]])
      s1=numpy.array(0.500648327441)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.99192745925387626, -0.31192819276046013, 0.22301635059603758, -0.7645952857491044], [-2.1143504528042101, 1.6855751626653568, -0.92840270978159622, -0.26132306489522378], [-2.3951729663048495, 0.54320257515057468, 0.27951624025697236, -0.59982051004475523]], [[0.50997737497744433, 0.82874261335121768, 1.2726789922026509, -1.3259614002500473], [-1.6858170448435976, -0.71005582829247516, 1.434136560497931, 1.0615232153047289], [2.2042527968066477, -2.3401275145005416, -2.3790259720994871, -0.32197418222286167]]], [[[2.2510852148580685, -1.7471307345999403, -0.22960415167714096, 0.57905948763656079], [2.0822326896371468, -2.0505898471117519, 1.1202550414799592, 0.42687114245242852], [-1.9322426990007087, -1.2650855319233629, -1.8902085585671109, 1.4162922241764502]], [[0.71822139168811838, 2.4032388030144229, 1.898800761942784, 2.313807423339675], [-0.97052592516776737, -0.33468111964844083, 1.5467127659547897, -0.70882140663181148], [-1.7498323207614752, -0.55037397461562643, -2.2081883490767602, -1.1494063943726363]]], [[[1.3039898954699449, 0.30725758822306465, 2.154978049666155, -1.0505821349981226], [0.65171319412402184, -1.144361807124386, 1.7646431580842228, -1.7233677864210739], [-1.7518101974082052, 0.8350188006325675, 2.2305767236340905, 1.7557055327244031]], [[1.5851145027726354, 0.89630368554135287, 0.95595030368051603, -0.1686706583086387], [2.3564815048566574, 2.4200350000364561, 0.72364162644345476, -2.3827881936556388], [2.1564727067528215, 0.84229975653047395, 1.8869851600838277, -0.27722602572765359]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s0=numpy.array(0.793477607541)
      s1=numpy.array([[[[3.1825233740974888, 3.6683931650111266, -0.82579496204942515, -4.7345353442548808], [0.2119187128022082, -3.4179611097579068, 0.97787071408791615, 3.1660547308129807], [-3.342088235491584, 2.379401393209772, -2.2846240479964197, -3.2342486935972712]], [[-0.74312285454628224, 4.617152220395683, 3.4834130885777146, -2.1969828853390325], [-1.4182500519578491, 3.3314520334748838, -2.5552964873793425, 1.193241941960296], [-4.3500396135335819, -1.3930727747103533, 0.30214022317161326, 4.1730204856466688]]], [[[3.3051142248281327, -2.9777772256348278, -2.3424975955575356, -0.46795937561265077], [4.5991193380076041, 2.3225807056984484, -3.5035760321416443, -2.8258051147372099], [-4.3458774085360528, -3.3508232570134409, 1.4407184896661187, 4.3874081421364917]], [[-2.4808554491737844, -4.8150622785105757, 1.8208752797123466, -2.4705289753752711], [-1.0285362892960439, 2.1971820569259464, -3.5099551409899044, 3.2298900914488442], [4.1696990494851462, -2.6861658709110636, 0.80202015978594421, -3.8591407339564876]]], [[[-4.7979089220729021, 4.0762497340527979, -0.048165079825138513, -4.6951415124803635], [0.30768121750164301, 0.43589766429027854, -3.1498192621102392, 1.583421888654069], [-4.8738006213989387, 3.083337810962675, 2.937423052407615, 1.8084212397006425]], [[-0.16766068428710756, -3.4929353146028674, -3.3541424145584822, 2.2997359186365003], [-0.68539904998366374, -0.95812542203241957, 0.28256993859920332, -1.284664335709611], [0.60601343430498389, -3.1477833148984669, 3.1644330076612892, -2.6104735582089944]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[2.5252610328237379, 2.9107878320945741, -0.65524981080679134, -3.7567477777799758], [0.16815275322756776, -2.7120756040405518, 0.77591851469936568, 2.5121935331508922], [-2.6518721772904135, 1.8880017248649721, -1.8127980237359478, -2.5663039155897445]], [[-0.58965134473478487, 3.6636068974944336, 2.7640102836033487, -1.7432567236684098], [-1.1253496581231046, 2.6434325891608745, -2.0275705433649276, 0.94681076132481423], [-3.4516590252573236, -1.1053720524083528, 0.23974150142426282, 3.3111983111725354]]], [[[2.622534127767965, -2.3627995487882516, -1.8587193877946804, -0.37131528578773448], [3.6492982091200639, 1.8429157816796247, -2.7800091278234498, -2.24221308182018], [-3.4483564087937872, -2.6588032212693999, 1.1431778603210572, 3.4813101159305062]], [[-1.968503246466677, -3.8206440969158355, 1.4448237605775898, -1.9603094207426928], [-0.81612051410022424, 1.7434147618626845, -2.7850708078506141, 2.5628454623847854], [3.3085628259534885, -2.1314124687101073, 0.63638503758699261, -3.0621417567456941]]], [[[-3.8070332926883639, 3.234412886717839, -0.038217912306695676, -3.72548965439164], [0.24413815634865577, 0.34587503579397272, -2.4993110522873261, 1.2564098119380542], [-3.8672516567018453, 2.4465595094848709, 2.330779415961608, 1.4349417587048767]], [[-0.1330349986469028, -2.7715659567282565, -2.6614368984572927, 1.8247889546969154], [-0.54384879839224598, -0.76025106759896233, 0.22421291884284089, -1.0193523835927365], [0.48085808999031904, -2.4976955737646485, 2.5109067321443934, -2.0713523135179872]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s0=numpy.array([-4.326793300489518, 0.65945613750033605])
      s1=numpy.array([3.047674382279693, -3.4532200825114945])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-13.186657099321305, -2.2772471775516219])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s0=numpy.array([[-4.0197731139525388, -3.7322677803660387], [-4.3332551959161423, 3.9470901269350556]])
      s1=numpy.array([-3.658206955309943, 1.7679632123690405])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[14.705161964229086, 13.653407953214245], [-7.6610357761867389, 6.9783101403262249]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_Symbol_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([4.386604190085988, 3.1973582384601649])
      s1=numpy.array([[-4.6176685783713509, -4.2763926885899304], [-1.6675032819292479, -4.4310671424624415]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-20.255884334312174, -18.758842086221673], [-5.3316053561358441, -14.167709033122428]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s0=numpy.array([[[4.7134986464946955, -3.3051878881830388, 1.9324366567891405, -3.8232036818004369, 3.2399026780315392], [-1.8711660280449238, -0.15403622216764923, 1.8833511897294297, -3.9953060640835636, -4.4291888637015635], [1.9606310716861604, 2.8847037018303903, 4.1198470196956976, 3.0547038809296616, -2.8330967503216442], [2.3345477159184309, -4.068187600345988, -3.7329835793787645, -2.6454807686789317, -3.6367883344284424]], [[-3.674885294810263, 2.5533665294383372, -2.7368547719272707, 3.5109920183566476, 0.89219522893565184], [3.8716780558204871, 0.71312112532142535, 0.78067940106278844, -1.2505578595787825, -2.2159268235588025], [-0.89430904018294655, -4.337900457945512, 3.7003113997657096, -3.8303388414635222, 2.8358188346470641], [1.1437992067601712, 3.5774401585618349, 4.2987013465034778, -1.7374030777695326, -1.8796846349230401]]])
      s1=numpy.array([-4.8627250301079084, -0.26624248586152444])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-22.920447847689502, 16.072219873077159, -9.3969081000665984, 18.591188238691696, -15.754755847577609], [9.0989658800616482, 0.74903579307789059, -9.1582189707808066, 19.428074800761056, 21.537927550596798], [-9.5340097870955844, -14.027520895335579, -20.033683222889739, -14.854185021364433, 13.776570480506434], [-11.352263612177801, 19.782477671377062, 18.152472688426929, 12.864245550524151, 17.684701663029639]], [[0.97841059614624548, -0.67981465211327619, 0.72866701791989208, -0.93477524280724489, -0.23754027562561983], [-1.0308051900371604, -0.18986314112594399, -0.20785002439984282, 0.33295163324792226, 0.58997386599152724], [0.23810306198674164, 1.1549334013432584, -0.98518010553535962, 1.0197989348431997, -0.7550154559893657], [-0.30452794413426776, -0.95246656083634906, -1.1444969324693683, 0.46257051436882385, 0.50045190983762222]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_Symbol_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array([-0.85077472917798502, -2.8513057675341189])
      s1=numpy.array([[[2.2550072568909894, 3.7797721116392946, -3.4830898886860853, 1.0820378130252761, 1.2088733422017821], [0.33368483167864138, 4.1584082953834614, 1.6042004323805461, -3.4495745135083631, 2.2903011721063704], [4.2085071476052338, -3.8277417848095121, -0.25413693587615604, -4.5564647124412714, -2.4694602192444961], [-4.5205393630617339, -2.1195423143044803, 2.4277894027273881, -4.9396851886016329, 1.4540343076742577]], [[-1.0514005644119551, -0.46907253064834009, -3.3112171675532034, 2.8273925097661703, 3.617540679251622], [-0.89555056772528907, 1.2652842106919344, 1.4929988765263671, 1.1472684024052127, -1.9822370717292594], [1.4388828735292769, -4.2498856497924811, -4.6698226103107254, -2.2255340429859416, -2.3607875966705527], [3.9089463493795265, -0.12827686386285198, 2.1498787458849167, 3.8228591272349615, 2.7425697200708301]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-1.9185031882758223, -3.2157345946344216, 2.9633248567494821, -0.92057042733691852, -1.0284788903222069], [-0.28389062230219764, -3.5378686913163508, -1.3648131884057655, 2.9348108225093572, -1.9485303594348189], [-3.5804915287474572, 3.2565459803345695, 0.21621328279415961, 3.8765250317362683, 2.1009543492435436], [3.8459606523472676, 1.8032530384336738, -2.0655018716065756, 4.2025593285570579, -1.2370556443270655]], [[2.9978644932964356, 1.3374692120294369, 9.4412926074024384, -8.0617605701790485, -10.314714603039445], [2.5534884988735711, -3.6077121675157677, -4.2569963075615904, -3.2712130126876371, 5.6519639952415801], [-4.1026950361000933, 12.117723464613787, 13.315092142140205, 6.3456780526093413, 6.7313272903097578], [-11.145601270967283, 0.36575656177333882, -6.1299616676406812, -10.900140277955494, -7.8199048607023922]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-4.8004114386377204, 2.8414584319935132], [0.45300782491349256, -2.1580875750652115]], [[-2.9809945222164291, -1.3757348526733102], [-2.5164957207591487, -0.67591934034710288]], [[1.8675257040289397, -0.21233261221908162], [-1.1692723975201638, 4.4952918639129997]], [[0.29535631231550852, 1.9176856630644226], [4.0555481088214904, 3.1946034067741849]], [[-0.50319083911217533, 4.1214130594262812], [-3.327799435546499, -3.4036098310759288]], [[4.1818475913629207, -0.087889621453975053], [-3.6329680263336939, 0.83835685983213093]]], [[[2.0731540255222232, -1.0237228709531543], [-2.3489793464683917, 0.28521723886040462]], [[3.6520721075910672, 4.7759879349182199], [2.495265129133867, -0.024669167856664842]], [[0.10760532088799035, -1.9027401293972512], [3.3324344087431808, 1.7146277697974668]], [[3.146710331160115, -0.52169030499264224], [3.4578966082162133, 0.7946616798278221]], [[3.242495106769713, -2.8743860507016317], [-1.7463547521376297, 3.4205774768250432]], [[3.9953732830932278, 0.76395462856780316], [-0.23607427784745205, 0.44519225169933563]]]])
      s1=numpy.array([-0.86368720048113445, 4.6211524374848061])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[4.146053916594628, -2.4541312784119915], [-0.39125706009558231, 1.8639126161011927]], [[2.5746468135427047, 1.1882045835097372], [2.173465144085224, 0.58378288281544444]], [[-1.6129580471393146, 0.18338895941834493], [1.0098856036140544, -3.8825260452886394]], [[-0.25509546652821319, -1.6562805617349192], [-3.5027249925245925, -2.7591380730442907]], [[0.43459948714054764, -3.5596117073222722], [2.8741777782498552, 2.9396542465320361]], [[-3.611808239023016, 0.075909141104930369], [3.1377479841016203, -0.724078089272568]]], [[[9.5803607783234597, -4.7307794404141124], [-10.854991632533874, 1.3180323385724451]], [[16.876781921864733, 22.070568286845358], [11.531000533667809, -0.11399998517154857]], [[0.49726059090787134, -8.7928521868642626], [15.399687390721787, 7.9235562977787009]], [[14.541428116899187, -2.4108104245289406], [15.979467339628798, 3.6722527587121108]], [[14.984064166181415, -13.282976104472171], [-8.0701715195539823, 15.807009944635675]], [[18.463228985827943, 3.5303507939339034], [-1.0909352245022184, 2.0573012590897339]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank1_Symbol_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([2.0608385579963127, -3.3077184815799199])
      s1=numpy.array([[[[-0.055298207479803629, -0.28466292150957173], [-0.37459740741594505, -3.8269021935375305]], [[1.4249772396750862, 1.8682482980579334], [0.86215947275294802, 1.8344922939413504]], [[0.86246045498722879, 2.5855799174316738], [2.7945384074770416, -3.5218849395492882]], [[1.9086159577003459, -3.2522431296816512], [4.7246216965076968, 4.988164415034408]], [[1.6357304659582912, -0.21251465980886319], [3.5662331318752045, 4.8855111503479041]], [[3.8258042704807647, 1.2639255297657801], [-1.9489155164833862, -1.535844977514822]]], [[[0.71679572312628359, 3.4465817921279189], [-0.31622657756249417, 1.5364448719043509]], [[3.333941486485072, -0.20470412297469576], [4.3598576131257971, -2.4713257445950996]], [[-4.2754160533036174, 4.2073367606370091], [-3.8090867378466808, 1.7375044175270915]], [[1.847462072143041, -4.0485881048486236], [-1.6819586529563715, 2.1514525753609757]], [[-0.7073065943520005, 0.90669708219552358], [-1.8797354961884816, -1.4349000489773802]], [[-4.2532745754765209, -0.57994650162682593], [3.2036791097604453, -0.91184016588351113]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.11396067816245942, -0.58664432467880334], [-0.77198478092823342, -7.8866275981228098]], [[2.9366480397895707, 3.8501581285487769], [1.7767714845910465, 3.7805924537014404]], [[1.7773917603847242, 5.3284627886241154], [5.7590925019302981, -7.2580362802496854]], [[3.9333493580359322, -6.7023480416265491], [9.7366825641090138, 10.27980156012803]], [[3.3709764147361212, -0.43795840507357459], [7.3494307449723708, 10.068249754157881]], [[7.8843649559537141, 2.6047464661772359], [-4.0164002426462604, -3.1651285487675249]]], [[[-2.3709584609022514, -11.400322292098359], [1.0459884949702281, -5.0821270987267138]], [[-11.027739871352704, 0.67710361081900983], [-14.421181603993116, 8.1744498394014684]], [[14.141872695955856, -13.916685561389627], [12.599386600716434, -5.7471754736811143]], [[-6.1108844400454725, 13.391589698712416], [5.5634457216370565, -7.1163994457642143]], [[2.3395710942814634, -2.9990986959727213], [6.2176358412244417, 4.7462454112224126]], [[14.068634920537676, 1.9182997617586712], [-10.596868600406129, 3.0161105689397898]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array([[-0.92964581149018066, 2.4747158105775497, -0.93508990837288586, -3.1659815060428764, 4.3041770360778084], [-0.36479777649600464, -0.46284804614702768, 3.8551035893914882, -1.905079535045795, -3.6289074810279285], [-1.7924132541146398, -4.0327211642101402, 2.8872047452169616, -4.1797485938348728, -2.7354747030752105], [2.4144235731691506, -2.7389038851960712, 1.1149133694930651, 0.49729995915312664, 3.6498358516917797]])
      s1=numpy.array([[-0.18715376609396461, -3.2194017776597219, -1.0511098402009553, 0.81254966800764983, 0.6703765234252117], [3.9222539536801051, -3.8484925826880803, 3.566965925952033, -4.6051070643132697, 3.4599364944347428], [-4.6781404947857226, 2.5880296585438494, 3.7373738283571072, -1.4920200540263338, -0.37038743494743187], [-1.8421337238240634, 0.27558954383466983, -1.2684097801497787, -0.7328562484822676, -1.1997419895987829]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.17398671475386721, -7.9671044797759834, 0.98288220416335004, -2.5725172216534986, 2.8854192376524734], [-1.4308295211551656, 1.7812672725085064, 13.751023144374816, 8.7730952249180287, -12.555789428535784], [8.3851610274643473, -10.436801977613325, 10.790563451882322, 6.2362687227899993, 1.013185458635615], [-4.4476910877306883, -0.75481327232819018, -1.4141670218847477, -0.3644493824353453, -4.3788613264176641]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array([[[-0.84046681222821906, -1.218326806305523], [-4.6113563648944211, -2.5864974373338701], [0.29333567710704322, -0.82645367358008226], [2.7789176130560449, -1.5975690910818163], [-2.798980709202552, -1.8883370881127481]], [[-4.9069519966313049, -4.6429612772860498], [2.7728317460619447, 0.57477170853077109], [1.1648612923796939, -1.685421067020505], [3.8896204849670681, -3.5705844172753198], [1.4917822097951046, -1.7097149527684308]], [[0.047211941196874996, -1.9142403465097093], [0.030724216463714704, -2.42220097425181], [4.5486850631368636, 1.7478010665509576], [2.3116697648837725, 2.5052477980117613], [-4.4125312892275144, 2.6431610375271495]], [[2.9780832437094285, 4.8202788652291133], [-2.269873730611133, 1.8222686683250853], [1.6330337625606415, 3.7610080251015123], [-4.2194194897357864, 2.3504028425122065], [-1.1761599162689294, 3.496459371056357]]])
      s1=numpy.array([[1.4868319623380266, -4.6900311149843441, 1.7367643222646763, -4.1355687788497466, -3.2187841796962835], [2.6005002793047973, -4.1688083900345152, -1.5817492130294672, -4.7955601803229619, 0.716259547581517], [-4.3942237946175311, 3.396280919168273, -3.4057854003488588, -3.026535697909293, -3.9479650382267542], [3.1615406741147929, -3.0504812196760653, -2.9489252953261946, -3.7635971003985427, -3.304505866753126]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-1.2496329197052687, -1.8114472361882616], [21.627404833635932, 12.130753459923119], [0.50945493844686385, -1.4353552542784636], [-11.492404919550241, 6.6068568551333264], [9.0093148260562579, 6.078149545151061]], [[-12.760530037774942, -12.074022098383731], [-11.55940424713709, -2.3961131208775512], [-1.842518432510069, 2.6659134463829686], [-18.652909114276561, 17.122952451967191], [1.0685032506779972, -1.224599658563271]], [[-0.2074598353973918, 8.4116004792498718], [0.10434807013210996, -8.2264749512422242], [-15.491845178816456, -5.952635355173415], [-6.9963510651983194, -7.5822218927912459], [17.420519259951853, -10.43510736656034]], [[9.4153313058870758, 15.239507692997741], [6.9242071862653098, -5.5587963499297857], [-4.8156945705367864, -11.090931701146665], [15.880194956934703, -8.8459693228474325], [3.8866273435505425, -11.554070504519677]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([[1.8808525339500459, -2.4729889924489168, -0.68745060736297692, -3.738412173126775, 1.9008159969062213], [-4.0059814226417672, 3.4337069876450794, -4.9538114461607554, -1.9544150547032233, 4.1238178362981159], [2.1724633667122104, -1.4890565238725548, 1.7270979236597448, -4.4712732467361676, 4.7889156409673959], [-2.482623526618013, -0.56215989013488787, -2.5849587036624655, -4.1601321748487727, 2.2422969714671916]])
      s1=numpy.array([[[-2.5426745959761354, -1.5071628809880711], [-1.9661244029675098, 1.4495534784288964], [0.34896625099743694, 3.6285977816801935], [-4.1819560546712564, -4.2239378844838082], [-0.72827804683872088, 3.3504923249671705]], [[-2.1945798657510851, -2.6258189051082512], [-4.6390140652065979, -0.83396959174978846], [3.6615330470694456, 3.459372066322759], [1.2597351298622383, 4.0851290898143571], [-0.78955469057769978, 0.11594661606008216]], [[4.1119672913968124, 0.61200151599999586], [-0.94569218710344316, 3.925755294836641], [3.3995914012335646, 1.0743949964808541], [0.20834355695736306, -0.39449795046126468], [-1.0388402878385214, -3.6050496973218902]], [[2.8935277128219274, 4.4528404575745881], [4.4570289063463804, -0.42456502178294286], [-1.119015577281143, 3.0444502201106953], [-1.788795072464997, 1.485070937142055], [-2.4056478538561823, 0.22008118724248771]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-4.7823959568521239, -2.834751123781865], [4.8622040063238501, -3.584729796120699], [-0.23989706119736909, -2.4944817488919999], [15.633875422264246, 15.790820805885627], [-1.384322561626659, 6.3686694088091151]], [[8.7914461727025106, 10.5189817530852], [-15.929015011483701, -2.8636072146747629], [-18.138544319068487, -17.137076938678469], [-2.4620453027412785, -7.9840377935392555], [-3.2559797157371584, 0.47814272336697639]], [[8.9330983056784081, 1.3295508738823276], [1.4081891207816868, -5.845671532903725], [5.8714272503620117, 1.855585367612502], [-0.93156097235331037, 1.7639081317897027], [-4.9749185028969674, -17.264278881969577]], [[-7.1835399747729269, -11.05472648025119], [-2.5055628803197005, 0.23867342600061545], [2.8926090560267688, -7.8697780943422506], [7.4416239351725757, -6.1780913875374823], [-5.3941768971182666, 0.49348737963073414]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Symbol(shape=(4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-0.72125805379004682, 1.8836903509544811, 2.6176555755068733, 2.3494720025251423, -1.9408488748850452], [-3.8497632783760736, -2.8762863491361834, -0.05368874863915174, 1.2100053437730818, -0.5099328377831398], [1.2173853909788015, 3.8170640104337146, -1.0303320615500775, 4.9513066095229377, -3.0466201045561201], [3.7950551211927426, 3.5871024502656503, -0.012713958185096352, 3.3972563684979544, -0.37396128758483727]], [[0.10921638256705091, 0.15134291113872234, 4.5630206259487487, 0.30362339200798782, -1.7053681193901893], [-0.87253616181155991, 0.41979886696433866, 0.74410001283794713, -4.6490867799864732, -1.2989265401432104], [1.6111311778005692, -3.3545301421205007, 0.80757496298079978, 1.6500907039262405, 4.2201666960963458], [0.27453426682254012, 0.15810827741913691, 3.0209088135079867, 3.1949379779877791, -1.1927025568210614]], [[0.80173634143668604, -0.33019510109770422, 3.3139380993402057, 3.994610367412168, -1.370866061587761], [-3.9226278874837872, -0.74593578821624451, -2.395921179061137, 0.41656091089142588, 3.4894104602466633], [-4.1880635228673588, 4.0388256271768856, -1.9947361629991911, 4.1697820887093187, 1.6378012771370365], [3.427623165328173, -4.6496275346871432, -3.9641516606803648, -3.4128023887769574, -2.7335122118087654]], [[0.98992465643557548, -4.2692660842785015, -3.3611100087286818, -4.208603726665741, 3.6366225297837467], [-1.0217940294367498, 0.17827340261806945, -0.01261823212779678, 3.5090410727009118, 4.0674616692458905], [-1.5405891250943462, -3.9558489996361645, -1.2996747387945984, 4.6685213288065235, -3.12673279037605], [4.9366430090727818, -2.6927203281681877, 1.2167932830688857, 0.39002913478728019, 2.8275853910687871]], [[2.9649032079609841, 4.573884591207074, -0.87332068671065777, -0.96751576255131688, 0.011344328010645555], [1.5978699054785439, 3.752075795950935, 4.7332734678723245, -2.5847196708966322, -4.2240193073160972], [-4.8844829660451028, 1.8336190980054523, -2.3194271581785273, -4.7151334143683155, 2.8854368061592153], [2.5751282818386967, 0.76088445790663961, -4.819309302740403, -2.0931115903363704, 4.5729840588660355]]], [[[2.5244028492259414, 2.2647049117015872, 3.1880089947051253, -1.4415487175280886, -3.8374960263584192], [-2.8897063865008654, 2.1431077277332982, 2.6805797328757652, 4.4923824618850681, 4.9262725292329321], [-3.3544639914976648, -3.749440449403032, 4.0564945356714652, 0.89799167395634605, 3.5323470799251631], [2.2595387451791922, 2.7877845914762398, -3.8068060004756097, 4.9363543402058276, 3.8571965964693646]], [[-1.4074862959730194, 0.2404322253632083, 0.38190844163046833, 0.65814266801891197, -3.3614719215877606], [-2.6604056356587726, -2.243568983815126, 0.27321042459666955, 1.8373351983558823, -2.3471292442147265], [2.8270678919415779, -0.77456558807416798, -0.82360217531187185, -4.5330184890115142, -1.4852803152683922], [-2.1127631616211517, 0.42199884105256569, -3.4261537386856533, -4.8640253926513068, 2.3427214939409069]], [[3.0760441934799374, -4.5578132121102799, -0.76580663671484039, -4.4061250922967385, 2.8666052817857759], [-2.0598971014771172, 3.3970257377942268, -3.7621829358759715, -0.50620652667046517, 3.8827759618024196], [4.8639964830384308, 2.9638481783407009, -1.5789601064942005, 2.9008506913602599, -4.9813477197823985], [-3.5839521278176178, -4.6107168795888995, -4.6528361133249696, 4.1919168118217573, -1.9422971066311812]], [[-4.3805303065657322, -2.2886567227535815, -0.49014367946103832, 2.8855443480091321, 2.2935539530849791], [3.6727505191728458, -3.4526803447432886, 4.8644002532628257, 2.8004564725650667, -3.6813120914745956], [-0.01022779977800603, 0.07988001584817539, -2.359536463032291, 0.39290153075215795, 3.6084605070276421], [4.0616338243031986, 4.866772687246673, 2.8685878358258865, -1.364719265625074, 2.6454245861845287]], [[-1.4552190411709462, -1.0505079961895247, -1.8286499426634109, -1.4473811945759008, -0.075186027149209522], [1.4688016114470868, -2.7656477775145683, -2.846476395237806, 3.8053263513332976, 4.0644724505732306], [1.6214033386890589, -4.8687040203604246, -2.7781199448488336, 3.8195828782554226, 2.4579025092580267], [-0.92924829208708282, 1.3422869968538143, 0.24850119148343097, 1.5742288380692271, 4.8016182776253586]]], [[[1.6246347056275567, -3.7235230570900422, -1.6774425445837506, -3.2091713871148353, 1.0362972768279164], [0.29202082414958586, -3.6444674747125037, 3.091545094259482, -4.1066388555809059, 4.3371265830180867], [-0.53933908749857196, 2.544729698487612, 4.3070464088837994, 4.9858406079935342, -3.2939383734006578], [-2.0767559010813583, 2.7349088564149433, 3.9668001464705522, 3.858284375904919, -3.8106882695747069]], [[2.5231170625014805, -2.7794171207987475, 3.7228733461944472, -2.1386186783219463, -1.5827271246956673], [1.2463786417135374, -1.1475493758714039, 1.157609058780416, 1.9997715749352443, -2.4482357008578104], [3.3102666960928389, -4.6575246998721118, 4.4995167093875423, -1.1661911811472194, 4.5161888123338265], [-3.3489005543331132, -2.9713880856934827, -0.48546859571212408, -1.4303133148840166, 1.3573666214353555]], [[4.2173321185484074, -3.2176754386274329, 0.4761727094991226, -3.2004776838229274, -1.6016646850859351], [3.3454150056667569, 3.993120957490607, -4.8566916252748848, -4.9428002437781799, -1.3846604451432833], [-0.97205876679849812, 4.5769930348082024, -0.36948274957568117, -2.5482176800510348, 0.48472005423634101], [0.65389133732845384, -4.8402803908538363, -2.5602777473506046, -3.6851646748747582, 0.49712669228996287]], [[-0.24199672326411292, 2.5383117926022987, -3.9923009059701542, 4.7009065837188118, 1.1565243132165097], [3.1984785667802669, 0.52132153831166317, 0.47813656778198066, 1.7398707987783606, -3.8665953401010631], [-0.18332198464315752, 3.4141363562903617, 2.6158352029898158, 4.3747841016065276, 1.2378213905539184], [-1.3462596797778126, -4.8617372770218807, 1.6825170467158603, 2.1536520394450722, -1.3787349233705157]], [[0.485735024146142, 1.5438016834241166, -3.8272174484028056, 4.0393103588502566, 0.75152457485820978], [2.8198304667175824, -1.7756191383237061, 2.2843705524242317, 1.9432418876101343, -2.4666675814065586], [-4.9292171996722578, 0.10704220991123314, 3.5702784836669483, 1.4713023183447955, 2.7012054108083579], [1.6161958343045306, 1.8454319628612943, -3.464945751556451, -2.1074528380027102, -3.2792371874745685]]], [[[0.093759740404619762, -1.898367378410557, 1.8178442386406948, -4.2556112625751821, -3.1896068035834677], [-0.58367743697662, -1.7537489678646736, 2.2666718180870413, -2.3491082111664308, -4.5664096048130958], [-0.6107149885954577, 1.1424052521428205, 3.6382759523930819, 2.6284634177045954, -0.66385904776997773], [-3.9903227753151183, 4.7876216993188088, -2.9013088569922818, -2.780113752642055, 3.4683516532161782]], [[3.037278121126306, 4.317427092341493, -3.0143709258776887, 1.7134228091861816, 2.1088518601971771], [4.2323229015686294, -0.36210332892123098, 3.0127908515570603, 3.7343387646689674, 0.76752075105215312], [-4.392181362177733, -0.79803623753327724, -0.73692881295537216, -2.0229194891010982, -1.3356821986755598], [0.64717298936925438, -3.8983789759529461, 0.490040236306303, -0.45181454418045597, 2.7761832189215934]], [[3.4711455704945529, 2.6947856407702053, -4.3317211497728545, -2.089950772964948, -2.8600552009120985], [0.20901298922822598, 0.2924555430306377, 1.1991084835249683, -1.2535651165998454, -0.19248409797063815], [-0.5300677213301821, 4.5173443765725114, -4.5941804125433841, -3.660487515157349, -4.239989589076977], [-3.2840110350854901, -3.1643033573985169, 3.743581109959246, 3.2817186811926238, -2.3610017430020291]], [[-0.20868166150340883, -4.598298289541515, 3.3688814196175958, -1.3417193027332686, -4.6881293853928447], [1.198620328814477, -1.5584440603783123, 2.4533126677919466, 4.8774610407206573, 0.432989452552766], [-1.1292595451885701, 2.6286505698414109, 0.24634427928206293, -3.5688599091862083, -0.36190248685820237], [4.5238526396622341, 0.41114849564489475, -0.85462922186447265, 1.9729189186291887, 2.3631746605734048]], [[-2.5845775941355686, 0.18644843606991035, -3.3192071730701866, -3.7924248201199742, 3.6522020211568744], [-1.942305800728489, -3.918729462625735, 4.5446889764613037, -3.6966418518768265, -0.49057689663402471], [3.2039236009060712, 0.52529081608740125, 4.419022202941548, 4.7492998407898561, -1.8253773068035342], [-0.58769824155195671, -2.5307921221667709, 3.4936991726738924, -2.8291945556493214, 3.7023086360513489]]]])
      s1=numpy.array([[-1.3868960858678481, 2.9936646937526659, 2.7577966356121415, -4.944643339780237, -3.3352810011311407], [0.0060427010152981353, 4.5589944876027424, -3.2321063264418912, -0.2422690696441574, 3.1874146391349694], [2.6578482606323259, -0.2770736719867628, -0.27872379458680108, 2.6011369927807788, 0.0073377342331681916], [3.0625016508797636, 0.38363672012626271, -0.96645451234932445, -1.2130414480738363, 3.1227548765213715]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.0003099717020778, -2.612482774725803, -3.6304162718206316, -3.2584735241582146, 2.6917557078390861], [5.3392216222975515, 3.9891102794520954, 0.074460715342782299, -1.678151675158067, 0.7072238567769209], [-1.6883870337411997, -5.2938711355775494, 1.4289635033079533, -6.8669477566789681, 4.2253454981351766], [-5.2633470932349464, -4.9749383478803972, 0.017632938842797621, -4.7116415601594328, 0.5186454460175115]], [[0.32695722847036446, 0.45306992972574012, 13.660153744767959, 0.90894662885173849, -5.1053003288697907], [-2.6120807016377299, 1.256737046508513, 2.2275859370538678, -13.917806951437774, -3.8885505230050339], [4.8231865239857132, -10.042338450595256, 2.4176086542342365, 4.9398182818334693, 12.633764039854467], [0.82186354181191223, 0.47332316789972206, 9.0435880580451169, 9.5645730234315458, -3.5705515345037444]], [[2.21102578506208, -0.9106109389028596, 9.1391673409873135, 11.016323031830657, -3.7805698125215943], [-10.81780999086115, -2.05713920712545, -6.6074633668066785, 1.1487902785839035, 9.6230844275380623], [-11.549827493093535, 11.138259726452512, -5.5010766792530417, 11.499411015478326, 4.5167228518897877], [9.4526876334882743, -12.822727172009779, -10.932324112880593, -9.4118149457781737, -7.5384707811309166]], [[-4.8948243593284078, 21.109998109377344, 16.61949021892897, 20.81004438683204, -17.98180137118996], [5.0524070422816365, -0.88149839291539778, 0.062392657450511355, -17.350956569145861, -20.112147252648096], [7.6176637567356211, 19.560262409227274, 6.4264280410613299, -23.084172895105159, 15.460578467205412], [-24.409938975684398, 13.314541636567684, -6.0166088030158944, -1.9285549636461734, -13.981401271608176]], [[-9.8887853397050414, -15.255190378419428, 2.9127698942808578, 3.226936941132315, -0.03783652168450595], [-5.3293451380217993, -12.514227117039155, -15.786797070552673, 8.6207664115914717, 14.088291344102499], [16.291123236998914, -6.1156349408888042, 7.7359413341804348, 15.726294894741249, -9.6237425595473489], [-8.5887764338920825, -2.5377634765119823, 16.073750756004632, 6.9811153204962837, -15.252186850011459]]], [[[0.015254211660039102, 0.013684934669289855, 0.019264185189084248, -0.008710847899008705, -0.023188841134678579], [-0.017461631715622285, 0.01295015924226728, 0.016197941873435991, 0.027146124063540437, 0.029767992014031151], [-0.020270022967203975, -0.022656747610407597, 0.0245121836492533, 0.0054262951999452845, 0.021344917286249186], [0.01365371706959978, 0.016845748781345971, -0.023003390484117002, 0.02982891338343311, 0.023307885789689942]], [[-6.4167222647173974, 1.0961291900729269, 1.7411184801622588, 3.0004687955543812, -15.324931960749998], [-12.128774627755615, -10.228418629769646, 1.2455648196918212, 8.3764010411829588, -10.70054928606613], [12.88858693544036, -3.5312402463169081, -3.7547977772244514, -20.666006303604807, -6.771384769853463], [-9.6320756074409726, 1.9238903901333928, -15.619816008347421, -22.175064952657074, 10.680454376865056]], [[-9.9421018981613507, 14.731336917602073, 2.4751684753572225, 14.24106478590665, -9.2651730666715473], [6.6578064535035049, -10.979548378210653, 12.159775268276455, 1.6361133173377864, -12.5495447502981], [-15.72095380461962, -9.5794724478282536, 5.1033669493992679, -9.3758578716088294, 16.100245479315578], [11.5837143459842, 14.902327195951697, 15.038461037774935, -13.548720847407225, 6.2777107661724214]], [[1.0612670019197155, 0.55447073495635646, 0.11874665321498985, -0.6990781446091292, -0.55565718239257733], [-0.88979385131510147, 0.83647765489962511, -1.1784937237347883, -0.67846398418729748, 0.89186805547133752], [0.0024778795367242407, -0.019352457122698001, 0.5716427036902989, -0.095187888317090602, -0.87421836988527135], [-0.98400824784917695, -1.1790684911088474, -0.69497010617808441, 0.3306292668084444, -0.64090455330870588]], [[-4.6383864749762278, -3.3484045655828338, -5.828665597098678, -4.6134040079998853, -0.23964904359378969], [4.6816797583114775, -8.8152662127410277, -9.0729005321331204, 12.129152918925811, 12.955158989317898], [5.1680847376798216, -15.518578468112098, -8.8550201814840062, 12.174594381540615, 7.8343544395756091], [-2.9618996095895356, 4.2784252236923619, 0.79207633557677004, 5.0177200438102876, 15.304748389641105]]], [[[4.3180325265151129, -9.8965592807111289, -4.4583877494325845, -8.529490589314193, 2.7543209147150933], [0.77614703953439512, -9.6864415385957123, 8.216857751443964, -10.914822939350836, 11.527424344816845], [-1.4334814555991053, 6.7635054029047224, 11.447475806314513, 13.251607787745634, -8.7547883763730105], [-5.519702059447007, 7.2689727470104009, 10.543152869572813, 10.254734417523768, -10.128231189101143]], [[-0.69908930935973967, 0.77010330764258483, -1.0315101883717424, 0.59255493018213912, 0.43853201619247956], [-0.34533870694544361, 0.31795571935880773, -0.32074299264143019, -0.55408405340205991, 0.67834165552575931], [-0.91718774874193232, 1.2904774709626115, -1.2466976168358022, 0.32312087279904017, -1.2513170176188706], [0.92789217370758115, 0.82329340780081106, 0.13451056644821544, 0.39630216224647341, -0.37609055403336011]], [[-1.1754708111146048, 0.89684270800298771, -0.13272066447027395, 0.89204928452550247, 0.44642205868282558], [-0.93244676484706313, -1.1129778255158624, 1.3536755189345542, 1.3776760398304198, 0.38593781348458506], [0.27093590804344375, -1.2757168664591008, 0.10298363399609863, 0.71024890121699935, -0.135103012829073], [-0.18225507478762462, 1.3491013174028659, 0.71361032893770771, 1.0271430818583276, -0.13856103806544348]], [[-0.62946662901401707, 6.6024967029495309, -10.384521572831186, 12.227702014517714, 3.0082781741578475], [8.3196809206685991, 1.3560287384358491, 1.2436987140589442, 4.5256422973614363, -10.057544175250651], [-0.47684559584530689, 8.880636374744638, 6.8041457135150276, 11.379412762117964, 3.2197430094251414], [-3.5018058549592737, -12.646044680442907, 4.3764573311968897, 5.6019439893783467, -3.5862784124178209]], [[0.0035641945149259245, 0.011328006461683823, -0.028083104488923885, 0.029639385898526421, 0.0055144876000042574], [0.020691166547364245, -0.013029021336346465, 0.016762104003764617, 0.014258992522043257, -0.018099751153933093], [-0.036169285788756576, 0.00078544728805963093, 0.026197754651546789, 0.01079602538865833, 0.019820727413707637], [0.01185921550088018, 0.013541289288870091, -0.025424851057266457, -0.015463928834199945, -0.02406217096921032]]], [[[0.2871393597752061, -5.8137532303586195, 5.5671509818793945, -13.032816517139009, -9.7681761016316955], [-1.7875131143221679, -5.3708591093142442, 6.9416861848941993, -7.1941477747924027, -13.984636953333315], [-1.8703156607906051, 3.4986179706611003, 11.142226110559957, 8.0496735559973889, -2.0330694297470244], [-12.220370086945669, 14.662099357951631, -8.8852631642509436, -8.5141029570998281, 10.621832663806103]], [[1.1652114165001537, 1.6563235690901577, -1.1564233752476822, 0.65733190670571395, 0.80903301087821289], [1.6236744764730564, -0.13891613345414233, 1.1558172007177607, 1.4326294755179623, 0.2944491435624938], [-1.6850020519855662, -0.30615600470916959, -0.28271295276873909, -0.77606619787824038, -0.512416737830927], [0.24827932299592945, -1.4955613241437671, 0.18799742898644881, -0.17333264983473254, 1.0650458245766505]], [[-3.3547042996258307, -2.6043877423365305, 4.186411451436979, 2.0198423551199327, 2.7641132544896512], [-0.20200154657923974, -0.28264497922353182, -1.1588838046990613, 1.2115136634616277, 0.18602712503921268], [0.51228634113027871, -4.3658078565743494, 4.4400663902494344, 3.5376946764221859, 4.0977570706776021], [3.1738472834633473, 3.0581552581999136, -3.6180008560658057, -3.1716318276996853, 2.2818007881889311]], [[0.25313950485654918, 5.5779264158208841, -4.0865927956419696, 1.627561125896182, 5.6868952584144408], [-1.453976139355851, 1.8904572397433772, -2.9759699511162294, -5.9165624037595066, -0.52523415252530492], [1.3698386339467448, -3.1886620937205401, -0.29882582126501916, 4.3291749922118976, 0.4390027167199963], [-5.4876207568885231, -0.49874016653046249, 1.0367006688566958, -2.3932324219862182, -2.8666288123133596]], [[-8.0710022858347212, 0.5822327629570957, -10.365070385889641, -11.842813100870334, 11.404931671608839], [-6.0653449109206363, -12.237231539182488, 14.191949663517457, -11.543706369701354, -1.5319513962726214], [10.005068048731346, 1.6403544575288231, 13.799523133691933, 14.830899237888696, -5.7002058863121841], [-1.8352375497294078, -7.9030434409581547, 10.909966128566079, -8.8348810952816326, 11.561402347616538]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank2_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=mult(arg0,arg1)
      s0=numpy.array([[2.0888765848718291, -3.6009685597210392, 2.4512272367762042, 1.8126087467275207, -2.2482225648128984], [-2.7029557990624231, 4.5527071456571715, -4.9954551547648336, 2.3946225660901916, 2.8479458756694491], [-4.4755923857193034, -1.4183613662196395, 2.3664964273763474, -0.22392550650502052, 4.4259656814993988], [1.4368359500266461, -1.1118470071963826, 4.1800290051559461, 1.857919050883849, 3.5604524405754105]])
      s1=numpy.array([[[[-3.3765070812153262, 1.2238770806358845, 3.6302944466099394, 1.8640385390284964, -1.4547122745009489], [-3.2260521524470431, -0.26948879062286135, -4.079334010892353, -0.21842403501960739, 2.888122825535576], [-0.1578222560678979, -1.7332713605964623, 3.4031474133846888, 0.2329311965320402, 3.7996547966575918], [2.0712717116823445, -1.8371254918125244, -3.6644418593701067, 4.413616475205, -2.3887981150176407]], [[-3.3251219823582066, -0.99208321483392581, 2.4396279088164077, -4.8252556472487571, -0.64057589331103237], [-0.18066547771676156, -4.3591919508231056, 2.9172613966383381, 3.0564921729384391, -3.032278327600376], [-1.6959550602906557, 1.1493447718534302, 2.7147606402179836, -2.3003523634940857, -4.4389093627056582], [1.5520511788864173, 3.5402107899614723, -1.9836506348478125, -0.058110612843390363, -4.7336856842939055]], [[0.4948152343237231, -2.4922085140267622, 1.6757505849368837, 1.8293784143756575, 3.2323459778192909], [3.0931508396054177, -1.4467219437168755, 3.2213175074851979, 0.8121426794057971, 0.95395033366075221], [2.9232107380393018, -0.21664336219523328, -4.1880934320969221, 2.4412583753863704, 0.12873099297221025], [-3.8172909670438071, -2.5479795448206231, -0.68021348335112464, -2.7785305392268276, 0.70758577232185971]], [[-2.8890242827969024, 1.3814429647844779, -3.906867809714786, 0.98667703121967509, -3.2662153784778383], [4.5265799764123944, 1.8876532068906631, 1.4054917248779191, -2.9940945755825341, -0.95239992003035034], [-4.3104230524552403, 4.9752139668717561, -2.7454753655850226, 3.2919801873073649, -4.390371810750981], [3.9245889876463149, 0.82305651018008241, -2.5637908715711744, 3.1437748590723587, -2.8087995025718437]], [[-3.2857489488496281, 0.74159851828118661, -0.40034411317386187, -1.9699749313958237, 3.6799522145975221], [0.062925809865069127, -2.7926575739999295, 0.46398050667735014, -2.3333211256925201, 1.5011225919346511], [-3.8737221319091897, 0.15314705062198009, 4.8923396623473412, 3.8133412294538402, -3.48065625325366], [-4.8590254200980088, 3.1625682783313209, 3.9907272575200938, -4.4901042357851937, -0.75487970922150716]]], [[[-3.1174537204124819, -2.9575056063519378, 4.3694949701020285, -2.8178242905198294, 1.4158660763988262], [-3.4387228239790679, 0.31294796438325889, 3.7950192819535964, -1.0862832521165378, 0.4408227763731638], [0.77757928747769434, 0.28197302997346441, -4.2180508674072499, -3.7318580733003315, 3.3880404584991659], [-2.8068093533276759, 2.4463310111039718, 4.2154247370479094, -3.2634110975999118, -3.5202873596132367]], [[-0.42710404287054615, -1.4088806473695836, 0.030311363309452055, 1.1002593724951604, 1.5311539965869612], [4.8154731611502886, -4.7242827083771219, 4.9329973701925294, 0.35793559721952661, -0.071461713921689984], [0.98432296240933681, -3.5099509587168543, 3.4711300651083583, -0.53107897986623609, 1.6006848234070539], [1.3400093497502432, 4.7589063522948347, -4.4721490207028491, -2.0557251812264399, 3.1711689992109164]], [[-1.8176795453128172, 4.6526553240963473, -0.51045284965947069, -1.14769420277708, 3.3460391725281653], [-2.4390667482778339, -0.96388861032886908, -1.7044643205986665, 3.5945449960714875, 1.4775849829271381], [3.1067348479207055, -0.13256089224703871, 3.5079410607318877, -2.6031144699302979, -3.3159696250580009], [0.56618529178856392, -4.0322706252263387, -2.7612957863749568, 3.0954080040046978, -2.0827588559763912]], [[-2.773266521333182, -1.353250282152457, 1.6979035626658279, 3.6083737597082042, 2.9130701405005857], [-4.6129855336002556, -3.1428847616915832, -4.0088975276019152, -0.95538449433560757, 2.035641862887374], [-0.074221525896999374, -2.3705558089849719, -2.8892407151658315, 2.6884475025414574, 0.16126460889927152], [3.7557438878767684, -4.1890704614055903, 4.7285027449530741, 1.973320853489021, 4.033239070043285]], [[2.2715699688338677, 1.5667118752187275, 1.1098582184215857, 4.0166503703431218, 3.7079443778486869], [1.1983013004839895, -2.8036704411517608, -1.0732485844417674, 3.6496107185202735, -3.0252207412133103], [1.0996892700972403, -2.5170073569935845, 0.26363383510379013, -0.87204453170325635, 1.2429156669512134], [0.80777417339922941, -0.034472632414765059, -4.4563335076261996, -1.0083162589201331, 1.7773270486283224]]], [[[-3.6661449889409967, 2.1221709486967129, 4.1112365747362034, 1.0384134529528213, 1.8900616680305991], [3.2435231516311767, 4.6970631893949122, -1.0269580483578276, -3.4202420938551215, 1.2072856636391034], [3.5220599162303277, 0.10724264330660738, -1.695147302681157, 1.8429144471055494, -1.6592473121952898], [4.2150192849407855, -1.2556555411309489, -1.6483655449019388, -1.5239801044553203, -3.0741956946999127]], [[-2.543820545863662, 0.10742859239341485, -1.7336397459317632, 1.4851856130634298, -2.1207627650855976], [1.5654115568797184, 1.9551934010808516, 0.31161084924170357, -2.8700878999438197, 4.9261380508386416], [-0.87100226650527102, 4.7999086913582474, 2.4778503116869723, 1.4766046342017001, -2.3957436781437913], [4.9659265917467668, 2.7510628404739057, 2.5094288493696304, 1.1655619172035596, -4.9270716988686605]], [[-3.9296258154554984, -2.1384089391384054, 1.2169560328191888, -1.8713355591297596, -1.9868786655352446], [-3.7290831413906602, 3.3618765606684988, -3.933598054900822, -2.7157723142189614, -1.7362336337246562], [-3.4277500856214917, -4.2624201557624355, 2.0734224374800618, -1.3774131128231479, 4.4235873668437886], [-4.1435023525144707, -3.8330280757233606, -3.5595234738895796, 3.1086658046240014, -2.3711555914317439]], [[4.9680455892927231, 3.4241211894177166, -3.8556947070471925, 3.2118330399342412, 3.3477534625564527], [-0.56113077531727651, 1.0135862944632956, 0.49912805348717448, -1.713462469497876, -4.3966043837095503], [-3.3147257364077642, -3.9321125338313454, 3.7723642352727715, -1.3450224247934797, -3.1990591594209263], [-0.45245650683687266, -4.5976608199672517, -3.302608747104776, -0.3641580073490216, -1.3282972674364544]], [[0.92525953463572641, -2.0665015951672294, -3.6792174443365342, 1.389436769313094, 4.4126466401115803], [1.7271805116571093, 1.5928353325923368, 2.111539640749708, -3.4403488429404758, 3.0780199920847942], [2.7870690071881041, 1.6356108720393259, 4.8346366805049854, -2.3678814790632208, 2.1401072133373944], [-0.78329551471170333, 2.4727534280387742, -3.921605595516966, 4.8839272793392965, 3.7137930980132481]]], [[[-1.2491964568668159, 1.5869952030668983, -4.683657718456665, -2.6204400011339377, -0.49567808999183338], [-0.20793429142951503, 1.5496023964745014, 2.3582444815054426, 0.24121877690864935, -1.14609930690597], [2.9589228041009239, -2.2728630028591947, -1.3311393582915034, 3.2170383425624536, -0.94110130739286646], [1.1051897044921137, -2.4246328085482816, 2.3278539604956876, -2.4266880519030254, 0.7734310948213956]], [[-4.9843742090520715, -3.8576692477990071, 1.2616358585060086, 2.1071505305183278, 0.50694251258540124], [-4.4987676639494678, -2.6163818629159783, -4.0214148731981867, 4.0197811982375136, 3.4502570288315013], [1.1773890350920553, -4.9246163266992715, 0.61489747425363817, -1.2449408950665153, -4.2017726756886997], [-3.3686085236892325, -2.319943662946705, 4.6348428817943947, 3.2129542261857686, -4.9829831374307894]], [[2.5532682649036156, -2.2748703372191805, -0.85797928169581184, 2.8614274818071692, -2.8426555567621694], [0.77097933224470161, -1.3885755290039947, -0.78644894053792314, -4.2240160962659088, 4.5782551851555624], [1.7215644104584538, 3.8560140784744306, 1.461485948887093, 4.4314921587102791, 1.3562172833675925], [1.4350736150146162, -0.69758306611054799, 2.0221361798327653, -4.3610555337478987, -0.7107312917000268]], [[-1.2865960784900388, 4.0108805213089589, 4.3964961934290603, 0.91821770427544536, -2.7588510626566762], [2.1068990336125726, -3.9112923088749163, 2.6229115940539813, -4.6312047729404568, -0.52773810944832], [0.29366440990483866, 3.3104633956257583, -2.4330207246659921, -4.4875534227707101, -3.3877880830358595], [-1.9350248365707179, 1.9747957337586293, 0.040751370794955832, -3.7023248430083888, -2.5740815725569801]], [[-4.8463425349604794, 2.5620136687850605, -4.7230776520355287, -0.47724287939350685, -2.2507660961253571], [-2.3088702939587069, 2.8643124869500109, 1.1196416683646202, 2.2205115833612989, 2.9837215956986665], [-4.9409350632059601, -0.3358387263665108, 4.2944744279819531, 3.3951791546863692, -4.148628184970617], [0.65274411611004624, -0.084959660924523561, -3.6536966395973547, -2.9509471458550482, -0.2446708456186899]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-7.0531065806046183, 2.5565281765015908, 7.5832370657137371, 3.8937464574753191, -3.0387144079306729], [-6.738824802821993, -0.56292882461752203, -8.5212252972243192, -0.45626085232568231, 6.0329321444951312], [-0.32967121527187787, -3.6205899603788869, 7.1087549466864077, 0.48656452232195696, 7.9370099353339745], [4.3266309794406439, -3.8375284233183256, -7.6545667966624045, 9.2195001096602613, -4.9899044484463122]], [[11.973659715709198, 3.5724604652439402, -8.7850233970658707, 17.375593878359169, 2.3066936519282462], [0.65057070508504034, 15.697313160703025, -10.504966569782544, -11.006332217784761, 10.919138922012447], [6.1070808508064509, -4.138754387723953, -9.7757677125931188, 8.2834965372221863, 15.98437305455443], [-5.5888874982479635, -12.748187749436445, 7.1430635695576523, 0.20925448983517031, 17.045853320743927]], [[1.2129045795461098, -6.1089693893079504, 4.1076454758409451, 4.4842221956880772, 7.9232144995146587], [7.5820155854979845, -3.5462442324806158, 7.8961812126517508, 1.9907462559078948, 2.3383490404009839], [7.1654537799186064, -0.53104211007972801, -10.265968690919507, 5.9840790217550985, 0.31554891619072789], [-9.3570475891175562, -6.2456768590129466, -1.6673578172126939, -6.8108097359672737, 1.7344535174706686]], [[-5.2366706845058673, 2.5040156010735433, -7.0816227641972125, 1.7884594169839261, -5.9203705637248687], [8.2049184580067607, 3.4215767135982702, 2.5476065939668664, -5.4271220162303253, -1.7263284254296043], [-7.813110526976307, 9.0181163531926707, -4.9764726615843493, 5.9670720815670313, -7.9580263455531712], [7.1137443263182165, 1.4918794294034461, -4.6471497585900847, 5.6984338072966363, -5.0912545461656329]], [[7.3870949291139958, -1.6672785228315745, 0.90006266892748499, 4.428942092879832, -8.2733516062913459], [-0.14147122564777451, 6.2785157736622885, -1.0431314447453401, 5.2458252057365566, -3.3748576837379072], [8.7089895067733671, -0.34430865494287888, -10.999068423618409, -8.5732397993894836, 7.8252899289219968], [10.924170592463817, -7.1101573661059545, -8.9720430703705691, 10.094753661254247, 1.6971375959911918]]], [[[8.4263396118976441, 7.9940069294485987, -11.810551768411367, 7.6164545067995313, -3.8270234218979673], [9.2947157984425335, -0.84588451513451024, -10.257769375710186, 2.9361756157327839, -1.1915244797566409], [-2.1017624443186609, -0.7621606365459781, 11.40120505279871, 10.087047420505053, -9.1577236047584307], [7.586681618439691, -6.6123245928897214, -11.394106738514838, 8.8208559509823488, 9.515181133032744]], [[-1.9444796279158023, -6.4142209906576051, 0.13799876033355299, 5.0091587072349926, 6.9708957413629946], [21.923439070489248, -21.508275644533139, 22.458492376783564, 1.6295759511464061, -0.32534425561218655], [4.4813341845954229, -15.979778810656462, 15.803038650924266, -2.4178470665453342, 7.2874492334702818], [6.1006701418553524, 21.665906955605998, -20.360384802997583, -9.3591147220769972, 14.437403762794041]], [[9.0801366543435122, -23.242131022101145, 2.5499443190958013, 5.7332549213564805, -16.714988632450883], [12.184248560500006, 4.8150623270864612, 8.5145750764473487, -17.95638832965945, -7.3812095195664806], [-15.51955461053303, 0.66220199249569522, -17.523762254444328, 13.003741597256234, 16.564777556539603], [-2.8283532344172131, 20.143027080193733, 13.793929269877193, -15.462971869705592, 10.404328463219372]], [[-6.6409265937668831, -3.2405236632101926, 4.0658381862045232, 8.6406932318849723, 6.9757034950462273], [-11.046359255806776, -7.526022772967659, -9.5997964847387234, -2.2877852694287126, 4.8745939413479817], [-0.17773254080260226, -5.6765864343716039, -6.9186410154026641, 6.4378170573345921, 0.38616787158190474], [8.9935890663650202, -10.031242657823677, 11.322979376884046, 4.7253586459011663, 9.6580852915622692]], [[6.4693083240349925, 4.4619106233915238, 3.1608161357315976, 11.43920285622486, 10.560024898105889], [3.4126972465227152, -7.9847016696145019, -3.0565538796290062, 10.393893793608827, -8.6156649329281212], [3.1318555212913823, -7.1683007213795396, 0.75081489337075868, -2.4835356274643852, 3.5397565474986505], [2.3004971256046338, -0.098176191309099117, -12.691396633651605, -2.8716301309620413, 5.0617312378567849]]], [[[16.408170597447306, -9.4979721391817193, -18.40021910978006, -4.6475153432641374, -8.459145609977675], [-14.516687520344771, -21.022140245698296, 4.5962456216834493, 15.307609472574629, -5.4033185235712473], [-15.763304543127623, -0.47997435780746317, 7.5867883605524016, -8.248133866997696, 7.4261146364864583], [-18.864708217341004, 5.6198023789719267, 7.3774122816451673, 6.8207137514879399, 13.758846843409994]], [[3.6080567848487726, -0.15237256507817665, 2.4589276385724443, -2.1065298952343992, 3.0080079729145486], [-2.2203192745119305, -2.7731707835806603, -0.44197678985932476, 4.0708217949347718, -6.9870438959740477], [1.2353959647008188, -6.8080050492044055, -3.5144871533720936, -2.0943589663325741, 3.3980302764440919], [-7.0434784252163816, -3.902001248970651, -3.5592769312228869, -1.653187993298423, 6.9883681462694733]], [[-9.2994454532013027, -5.060537114740681, 2.8799221039407032, -4.4285089151028956, -4.7019412636194406], [-8.8248619314903642, 7.9558688701022842, -9.3088457436573453, -6.426865479166767, -4.1087906913000527], [-8.1117583315622284, -10.087002070588738, 4.9067467907385245, -3.2596432105173134, 10.46840369982297], [-9.8055835140509853, -9.0708472472325674, -8.4235995841219342, 7.3566465205497176, -5.6113312358766718]], [[-1.1124721249224061, -0.76674807167493553, 0.86338839020426927, -0.71921134027683475, -0.74964738975688994], [0.12565149307847601, -0.22696782437424035, -0.11176750218798052, 0.38368795135965517, 0.9845118635243546], [0.74225163945033579, 0.88050029077292369, -0.84472857210487973, 0.30118482773249078, 0.71635094261285615], [0.10131655246493899, 1.0295335278494548, 0.73953833648334821, 0.08154426624348865, 0.29743963839994275]], [[4.0951669467778293, -9.146265140973922, -16.284090143407425, 6.1495994575931512, 19.530222593717482], [7.6444416703489377, 7.0498345183333635, 9.3456019850837766, -15.226865911240711, 13.623210851936351], [12.33547177778515, 7.2391575879333612, 21.39793603043324, -10.480162164191853, 9.4720410809606204], [-3.4668390665864064, 10.944321811309608, -17.356891782134102, 21.616094529294454, 16.43712079999597]]], [[[-1.7948903778721517, 2.2802517602863572, -6.7296477874983163, -3.7651423985171069, -0.71220809934080931], [-0.29876746516924474, 2.2265244315020079, 3.3884104499789682, 0.34659181048380477, -1.64675668646312], [4.2514866582858586, -3.2657312719936069, -1.9126288844886323, 4.62235634320787, -1.3522081910791479], [1.5879762990135946, -3.4837995849362451, 3.344744256852112, -3.4867526324743947, 1.111293601907849]], [[5.5418615470813828, 4.2891380079188464, -1.4027460534515446, -2.3428290110690728, -0.56364251543869293], [5.001941363234077, 2.9090163439660266, 4.4711980914604244, -4.4693816948446683, -3.8361579515645881], [-1.3090764749729384, 5.4754199244310282, -0.68367191648152237, 1.3841838083160909, 4.6717283743840179], [3.7453773054800981, 2.5794224185115073, -5.153236186948555, -3.5723135406436164, 5.540314888262464]], [[10.672735405241308, -9.5090239925450621, -3.5863782833113578, 11.960849870104306, -11.882382678933594], [3.2227159711586157, -5.8042859870864598, -3.2873793825226825, -17.656509800637089, 19.137239466955858], [7.1961891699605331, 16.118250692312795, 6.1090536569759095, 18.523765759530104, 5.6690275817703375], [5.9986493352950934, -2.9159174498477083, 8.4525878840761983, -18.229338624162061, -2.9708774141780636]], [[-2.3903913650190951, 7.4518913313588584, 8.1683340349101741, 1.7059741656321823, -5.1257219478609901], [3.9144478528375695, -7.2668644942341825, 4.8731574193770166, -8.6044035761902844, -0.98049468742145951], [0.54560470172876341, 6.1505730099867328, -4.5203555555521744, -8.3375109960247258, -6.2942360198295981], [-3.595119507798143, 3.6690106153543067, 0.07571274814958015, -6.8786198583858411, -4.7824351921826702]], [[-17.25517210646446, 9.1219278198133296, -16.816293353217077, -1.6992005746838477, -8.0137456401139158], [-8.2206228730973443, 10.198248384731791, 3.9864309106987363, 7.9060258863047057, 10.623398837402876], [-17.591964304516281, -1.1957378129313809, 15.290271958097035, 12.088373907493843, -14.770993346218569], [2.3240643812752531, -0.30249483208917927, -13.008813117576578, -10.506706967468649, -0.87113890942071392]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([[[-1.7903401615921153, 1.6136470990503788], [-1.9826326718780676, -2.0511215746383762]], [[2.4347691798865148, -1.4711445573812689], [-1.7860339215306009, -1.9419306634086664]], [[-2.9050205625464063, 0.95079041891800919], [2.1607902173573743, 0.41004466503415404]], [[0.21829643467305537, -2.9911793831195812], [3.5580746482410479, 0.79304183595075273]], [[-0.048645665128834281, 1.1708821605959105], [-0.66399095881989911, 4.5379254941518692]], [[0.9652080271982495, 0.52552794098657785], [-0.14471489676633897, 1.5928393065250575]]])
      s1=numpy.array([[[-2.1739488837701195, -1.9874568124700231], [4.0733189281512434, -0.98623617490628135]], [[-0.28635650972062443, -2.4817880863384811], [0.0021889387657347825, 3.9083882400468344]], [[-1.6449922285104543, -0.57290679353176532], [2.7641500594377932, 2.2659821069900135]], [[4.3503969975445251, -2.5584418546333234], [-1.9850691205239901, 4.9137877972323611]], [[-4.449969325618504, -3.9782986586963087], [3.5320942033739762, -2.0890224198450182]], [[-0.32503248451732158, 4.1773646906930306], [-1.3329208616890673, -2.4092077189266781]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[3.8921079958619944, -3.2070539199301655], [-8.0758951899320053, 2.0228902960391006]], [[-0.69721200432764951, 3.6510690357905311], [-0.0039095188877556466, -7.5898189678527794]], [[4.7787362490519065, -0.54471429022304052], [5.9727484077409887, 0.92915387403410665]], [[0.94967615397633465, 7.6527585284894215], [-7.0630241127425624, 3.8968392961895568]], [[0.21647171764762227, -4.658118928990147], [-2.3452786167404942, -9.4798280968695376]], [[-0.31372396315630952, 2.1953218646499408], [0.19289350489703297, -3.8374807522899856]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([[[[-3.2232665037832575, 3.8133829281259164], [0.96556132913708304, -4.3224582132459357]], [[-3.6751398588318027, 3.0866460343346773], [-1.7789142713950987, -1.3147706810114066]]], [[[-4.2055608752428935, 0.089130531320204653], [-1.5228925373279645, 1.5320680241989137]], [[3.9731467655901227, -1.4628956522290206], [-2.2289583398827304, 4.9114695326026094]]], [[[-0.67932489381549921, 0.06356518352367857], [4.4777436037221054, -2.3253843412157602]], [[4.8754271548339894, -1.3669569718355055], [-3.1752074198498681, -3.7520440953098855]]], [[[-3.8831242517185629, -2.2174484446917511], [-1.0909412575563469, -1.3352638201143883]], [[4.7461038106457405, 3.3260206910321148], [-0.84465483587059165, -2.0906315074170068]]], [[[3.6525433195041259, 3.2702672945088889], [3.8567875222224224, 4.5761730072982463]], [[2.1306388270099283, -2.5618301779339414], [4.9742941432810905, -3.3368117482111117]]], [[[0.93755866299788959, 0.14976657883219691], [-1.2434382860108317, -2.6412778357416067]], [[0.60335004176831042, 2.7196505356230993], [-1.9181321394063144, -4.744248955459601]]]])
      s1=numpy.array([[[1.7672547479115073, 3.4996700459098982], [-4.2465049183858881, 1.2522295294481687]], [[0.8883406629569679, -3.1376722307600433], [-4.8121552248925674, -2.3246720493607431]], [[-1.222387613952912, -1.0754474477129028], [2.5044139370753973, 2.6587128802521089]], [[1.1145276516496772, -4.4667430299767457], [3.1929649707534171, -3.7834047616638844]], [[-3.060767509793092, 0.29609946932909192], [0.51407808541216493, -0.17923912589069335]], [[-1.2031549957289966, 3.2607061195951488], [-3.7862288562233504, -4.5352000059042199]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-5.6963330325950858, 6.7392190853352121], [3.3791460610699975, -15.12717753359402]], [[15.606499486285269, -13.107457566118503], [-2.2276089809977164, -1.646394671215162]]], [[[-3.7359707360191581, 0.0791782752826974], [4.7783376248056566, -4.8071272951643378]], [[-19.119398967299514, 7.0396809563465021], [5.1815971519149064, -11.417555943728159]]], [[[0.8303983360499434, -0.077701293017988393], [-4.8155779301357136, 2.5008286547120391]], [[12.210087715762095, -3.423426091647221], [-8.441964864626911, -9.975607963474264]]], [[[-4.3278493533317999, -2.4714076077165261], [4.8729542583038779, 5.9642803616760673]], [[15.154143214951159, 10.619867558466616], [3.1956711279952232, 7.9097052000460479]]], [[[-11.179585920450037, -10.009527883371764], [1.1419927386451225, 1.3550023990191253]], [[1.0953147288940848, -1.3169807530233864], [-0.89158813416489802, 0.59808722101115597]]], [[[-1.1280283891749097, -0.1801924075151983], [-4.0544868285344222, -8.6124308025536873]], [[-2.2844213385467405, -10.29721933681947], [8.6991128899605918, 21.516117890811472]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank3_Symbol_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=mult(arg0,arg1)
      s0=numpy.array([[[3.6858140535426394, 2.0649143646412496], [-3.8390032595304757, 0.40667238187906207]], [[1.7170818563051293, -2.9887310995993719], [3.2005253721230336, -4.583912811852672]], [[-1.3225809750248829, -1.3350163087098044], [-0.44923278112917941, -1.1699710352948465]], [[-4.5718138153073538, -2.2334596632233437], [-2.8623272328315941, -4.4298874528679892]], [[-4.5839280199339774, -4.8190284370298411], [4.6243316246560369, 4.504334556532184]], [[-3.7963961998921683, 4.7552669630061946], [-3.9834322560400137, -4.8548160344145437]]])
      s1=numpy.array([[[[4.4434344148381513, -0.16212498156787447], [-4.8758939675162836, 4.0867160142504133]], [[0.29342982040058363, -2.8607834438185176], [1.1662272411313239, -3.1060330842152641]]], [[[-0.8742366677282476, -2.3907330446025354], [-4.1651864755835746, -3.5985673342816304]], [[-2.6265355230258605, 2.114709643194967], [-4.4822849741649149, -1.7821343523775179]]], [[[-0.83847930757970346, 2.7727930672343684], [1.5770511179525766, 1.8852316429125455]], [[1.3354240215692537, -3.3386492545442783], [3.8107048640839309, 1.7249028341532]]], [[[-2.8994950088132212, 3.6146341367942334], [-0.99451164953457472, 1.0282589673476501]], [[0.82661336901541205, -1.1608384889529031], [-3.6904505193283876, 1.9404120874493191]]], [[[-1.8491119954133772, 3.9310761140907182], [-0.1186777073608436, -1.2455987118116632]], [[-2.2916312099342218, -0.20036876766034073], [-4.0380767444370687, 3.2321669572067329]]], [[[-2.6422915099048758, -2.3982248009988516], [3.7841677770095892, -0.61621781999000547]], [[-0.43823985611493832, 0.12985640669394272], [-0.6770581217828866, 4.6940990538876175]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[16.377673012205474, -0.59756253549321314], [-10.068303493991989, 8.4387186020351113]], [[-1.1264780369612826, 10.982556965630108], [0.47427240996312275, -1.2631378725529909]]], [[[-1.5011359202728298, -4.1050843341561354], [12.448622355207329, 10.755150105969918]], [[-8.4062935822267093, 6.7681818677187398], [20.546403519449274, 8.1691484903060694]]], [[[1.1089567801569529, -3.6672433584050665], [-2.1053889621357191, -2.5168149889840263]], [[-0.59991624719626913, 1.4998306898337879], [-4.4584143150353839, -2.0180863546572341]]], [[[13.255951338707002, -16.525434283877448], [2.2212016538411832, -2.2965749269186659]], [[-2.3660379571554859, 3.3226996198489722], [16.348280451002978, -8.5958071595851226]]], [[[8.4761962877714083, -18.01976994787362], [0.57191124661341097, 6.0025756133481423]], [[-10.597262676147599, -0.92657162888507139], [-18.18884862189687, 14.558761317827768]]], [[[10.031185447210211, 9.1046115209991925], [17.994728012486291, -2.9302802414141711]], [[1.7456987787305798, -0.51727419907810179], [3.2869926258621529, -22.788987353943746]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_mult_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=mult(arg0,arg1)
      s0=numpy.array([[[[0.77184318415333841, -4.5955448304894002, 2.3682516251392771, 4.7383897652852198], [-4.3940703498952747, 0.89062727414341403, -3.8705061844175512, -3.9244199079718367], [1.4532078832387976, -3.0843288862140428, -0.41797408925805879, -0.037367547708742954]], [[-2.566363865401009, 4.7441172697199256, -0.34805935051750669, -3.9118072635652492], [2.3470456365625196, -1.436512996975754, -4.2912040143067207, -1.8135938567031129], [0.85623873323901556, -2.2735916212941607, 3.5898386363798451, 0.78885998938523105]]], [[[-4.0250015812292013, 1.6517476592918321, -1.2928284832532491, 4.4821999349806028], [1.9774159763167907, 3.6379887728698588, 0.11683175496148035, 2.4030900279149501], [3.2596814161802961, -4.8199483164610673, -2.4819079967286637, 0.86142704143355964]], [[-2.415277379693217, 3.27038456490925, 2.5650880076373275, 1.2367219389227166], [-2.470192103270815, -0.4546382865213241, 4.8926301136997861, 4.2472752926038719], [3.9171046333355193, -1.9016604884864141, 3.9868718600871365, -0.2868646017135692]]], [[[0.52326238445261986, -1.1869340783997782, 4.8218078088450724, -0.84172798065254462], [2.1937890071062247, -0.75871909137902893, 0.67560566424225854, 1.2073115066494244], [-1.1048039285768674, -3.6621248993131053, 4.2580619745441908, -4.0996164881575616]], [[1.3550243093280381, 0.9447724758685192, 1.881095865692556, -2.4856920906516344], [-0.72101395700450954, -2.8845738525093791, 4.4282486604378821, -1.4707585032536019], [-1.0700453817327094, -2.8013161487931182, 3.3678571104474138, -2.4053802489998777]]]])
      s1=numpy.array([[[[1.9272385848621667, -1.300239859647454, -3.4083411170420161, -1.3873201420926895], [0.33724817494189452, 0.01297104297820173, -4.9576650434317751, -4.536540017837746], [-3.9178712890681053, 0.56152660909960961, 2.9969981861563353, -1.3248898580117121]], [[3.7554028842744192, 1.2329960642993276, 4.7631276983469633, 3.5128898123139578], [-0.39446627867501061, -3.8975771060740771, -1.5768725800054839, 2.7560528350373961], [-2.4225593022144585, -2.1518460907766004, -1.488502546644618, 1.4802643068081078]]], [[[2.5989036373853942, 3.489876658586768, -3.5178758100724794, -3.4924900882534358], [0.51293437040306067, 1.5194790042443369, -3.9360552508145332, 3.2492540110881531], [3.0736413758798626, -0.2559674968571386, -4.9725193010955815, 2.366160548255241]], [[-0.29864891831012308, -2.8175867283783318, -0.68290820996579793, -1.6354593517807672], [-3.5329291147178399, -2.4698778815514211, 3.4602772618651372, 3.0823044012586251], [-2.4086021558152613, -4.7302423734675809, -3.3799810279989497, 1.968771664689541]]], [[[-0.68420477020885784, -4.783485932393452, -0.85049290887124229, 1.5846212303610452], [-3.9715988938864344, -2.9123137434934856, -1.5783076708496226, -1.6891185956356658], [-2.4450651195066686, 3.6053245572652362, -0.87703784060163681, -0.44940559857512952]], [[-4.4519043553342765, 2.476124207718863, -3.5175765523643552, -2.3901389570772977], [-0.79123409476413897, -0.40489773839681575, -2.4941955871006694, -2.8996558727949981], [2.9611859871243071, -0.12669636485528013, 3.117229128355671, -4.4414678767001226]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.4875259659631888, 5.9753105653991208, -8.0718093894637732, -6.5736635624660371], [-1.4818922060684732, 0.011552364650472878, 19.188673210873393, 17.803287959313362], [-5.6934814427887206, -1.731932740823747, -1.2526675873667483, 0.049507884978082332]], [[-9.6377302622245971, 5.8494779221391404, -1.6578511331185906, -13.741747883914105], [-0.92583035813523851, 5.598920169590559, 6.7666819453697284, -4.9983604903730194], [-2.0742891081245012, 4.8924192423042729, -5.3434839522946422, 1.1677212853559804]]], [[[-10.460591249938535, 5.7643956020378946, 4.5480100478092984, -15.65403884648995], [1.0142846188370065, 5.52784755805237, -0.45985624257801161, 7.8082499122085931], [10.019091672958425, 1.2337501055453186, 12.341335417276749, 2.0382746806403218]], [[0.72131997686428773, -9.2145921467816478, -1.7517196597003424, -2.0226084605635997], [8.7270136005915599, 1.122901047985456, 16.929856733152011, 13.091395327749929], [-9.4347466644058802, 8.9953150225874943, -13.475551248157405, -0.5647708994561258]]], [[[-0.35801861951334379, 5.6776824667037253, -4.1009133493627168, -1.3338200283309534], [-8.7128499940433013, 2.2096280372740358, -1.0663136023430113, -2.0392923166064554], [2.7013175496572353, -13.203148831266018, -3.734481479302179, 1.8423906017889193]], [[-6.0324386242813128, 2.3393739982845259, -6.6168987099096634, 5.941149501165385], [0.57049082558277298, 1.1679574291196375, -11.044918267448615, 4.2646935314224885], [-3.1686033899739789, 0.35491657286248113, 10.498382284826441, 10.6834191071819]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_float_rank0(self):
      arg0=1.87348049808
      arg1=-1.95690527584
      res=quotient(arg0,arg1)
      ref=-0.957369026091
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_array_rank0(self):
      arg0=-3.68510487457
      arg1=numpy.array(-4.46156511129)
      res=quotient(arg0,arg1)
      ref=numpy.array(0.825966848548)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_array_rank1(self):
      arg0=-2.40551396076
      arg1=numpy.array([-4.9602758758898711, -1.5254085997921631])
      res=quotient(arg0,arg1)
      ref=numpy.array([0.48495568007677509, 1.5769636811332681])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_array_rank2(self):
      arg0=2.68098465563
      arg1=numpy.array([[-4.4763901941813593, -2.9966536569232671, -1.2838145136047907, 3.8765986006818167, 1.87522396359455], [3.886853309991146, -3.0847632599158592, -3.6021241045652808, -2.9086471165204495, 2.3671821762097736], [1.80597609325274, 4.1802163169190827, -0.1852143109215838, 1.5255488577662399, 0.7362253921756583], [-2.5958627822717064, -2.0064015994122899, -4.8549083690580241, 3.6568076597452777, -4.0310732330479029]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[-0.59891665813894324, -0.89465949774711806, -2.088295954917323, 0.69158170132793806, 1.4296877107340025], [0.68975709701561694, -0.86910548062549275, -0.74427881377745786, -0.92172908855036273, 1.1325637217823714], [1.4845072787183571, 0.64135069871240136, -14.475040520816517, 1.7573902284261305, 3.641527016206703], [-1.0327913609049997, -1.3362153700487192, -0.55222147398535026, 0.73314893893319621, -0.66507962039626267]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_array_rank3(self):
      arg0=0.759735359002
      arg1=numpy.array([[[-1.661753141502337, -4.0515688648204016], [-1.1880013417386746, -4.4934977582250042]], [[-1.229411717309814, -1.6020202156974062], [1.8245951289524784, -0.77926155374567685]], [[-2.2651162930332247, 0.92261322869706852], [3.8353674257897357, 0.701695208334284]], [[1.5379294299472388, 4.9435582888272336], [-2.0037437590124041, 1.8074459746178784]], [[-1.8231894114926703, -1.9632425122009032], [3.8252028991130587, -0.91344872294504587]], [[4.1496278239021152, -4.9023174263545286], [1.665165020254971, 3.2838191777126742]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[-0.45718906137597265, -0.18751633857165226], [-0.63950715568227601, -0.1690743825589742]], [[-0.61796658377754221, -0.47423581273054805], [0.41638572138368052, -0.9749426945936277]], [[-0.33540677860060913, 0.82346029232091067], [0.19808672146856729, 1.0827141898339616]], [[0.49399884299508545, 0.15368188551939987], [-0.37915794152066734, 0.42033641374129976]], [[-0.41670676355015768, -0.38697988367739755], [0.19861308773403508, -0.83172195649096536]], [[0.18308518046507791, -0.15497473805301573], [0.45625229317253863, 0.23135724529485716]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_array_rank4(self):
      arg0=-4.7709537676
      arg1=numpy.array([[[[-0.11887735843278069, 1.9463205918231505, 1.4758821115063032, -1.4049003296197613], [0.62230197071628268, 2.481067626922612, 1.010830526490758, -3.3994689744076547], [-2.7223427970333889, -3.3697087715800089, -4.8544732455124793, 3.7166860978702019]], [[-3.6199397913884734, 2.2519230525733294, -1.5225677505863491, 2.8042940293363126], [-3.5838404901674314, 0.54416186111818288, 2.9819783087374816, -3.490771993162177], [2.0287735917249918, -1.0817840254498923, 1.4082641135332103, 2.5827482588621589]]], [[[-1.0774747609045754, 1.8150326401795249, -1.1702288988358744, -3.7316669914667235], [-1.7288366893491078, -2.329385604156744, 3.771818793004563, -2.148578744079781], [3.5253025823008617, -1.8756218183827391, 4.7022513159487431, 3.5241569060903402]], [[0.8590098144152245, 3.6958418260753589, 4.3162623448300526, 2.3601620521599278], [0.66998181087564301, 4.0046502257369028, 3.3225047113414465, 0.57332635124140996], [3.5481736981059306, -0.35566346048568676, -3.0226888284355722, 2.45054479498065]]], [[[3.6304748082966576, 3.4088199382993309, -4.8435119508850386, 3.0724736463628428], [-0.64048226759901539, 2.0001955563812635, -1.9270237663373146, 1.297829001293934], [-4.8282051005901803, 0.17936443442241767, -2.2505505823953511, 2.8787040234330838]], [[-1.4737518591306475, -4.9457971627166595, 1.8113252752849176, -1.8490779475593824], [-4.1086987923497098, 3.7822580964694357, 1.5100246674853341, 3.627223695893413], [4.1244548346186622, 1.3793511028495491, -1.3264713356412683, -3.914132006117069]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[40.133410015984289, -2.4512681968446337, -3.2326116905981204, 3.3959375387798376], [-7.6666216597521721, -1.9229438632906428, -4.7198354645688827, 1.403440891361933], [1.7525176376755172, 1.4158356377375847, 0.98279535725327749, -1.2836579797077421]], [[1.3179649503976847, -2.1186131391781018, 3.1334919354246105, -1.7013029724023083], [1.3312405450771116, -8.7675269225892016, -1.5999290650842644, 1.3667331401035054], [-2.3516442579200509, 4.410264577179138, -3.3878259921216149, -1.8472391768068714]]], [[[4.4279030383918627, -2.628577394138774, 4.0769406501121157, 1.2785046946873204], [2.7596324146707794, 2.0481597203512543, -1.26489474426725, 2.2205161345591158], [-1.3533458919393107, 2.5436651039353362, -1.0146105444041564, -1.3537858542435777]], [[-5.5540154344426984, -1.2908977148154159, -1.1053437873892173, -2.0214517741414091], [-7.121019839873628, -1.1913534263087866, -1.4359509412625222, -8.3215323301806752], [-1.344622381408983, 13.414236483795095, 1.5783807194170156, -1.9468951464882678]]], [[[-1.3141404415469131, -1.3995910179932107, 0.9850195098059239, -1.5528054319514546], [7.4490021175525349, -2.3852436589907655, 2.4758146998197357, -3.6761035258445292], [0.98814231545729747, -26.599218417871711, 2.119905148953372, -1.6573269529495334]], [[3.2372843081020815, 0.96464808617005526, -2.6339574855485046, 2.5801799074490868], [1.161183627401285, -1.2614035441032914, -3.1595204173353357, -1.3153183171474601], [-1.1567477300405513, -3.4588392743107845, 3.5967258691598101, 1.2189046665119598]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_constData_rank0(self):
      arg0=-4.45871167875
      arg1=Data(-2.96219792285,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(1.50520383677,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_constData_rank1(self):
      arg0=-3.05116019759
      arg1=Data(numpy.array([4.0935236078087271, -4.1211273163457438]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([-0.74536279496963576, 0.740370283026345]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_constData_rank2(self):
      arg0=2.83385447154
      arg1=Data(numpy.array([[-2.0824352535058255, -1.1077410609329172, 2.3128395637789732, -1.20014472192435, -0.45808278188535034], [-1.8219102369283648, 1.0942615384942469, -1.4173091041710348, 2.4170957295552018, 1.7568066017699069], [1.7575759004192157, 2.2671096775272339, -1.9311118269783769, 2.5004745395721439, -4.9917931461888356], [1.8586888682026377, -4.1698517509435788, 0.5186446235316744, -4.0009075023424288, -2.232750473312243]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-1.3608367735660531, -2.5582282461882007, 1.2252706655150816, -2.3612606211334093, -6.1863370194305505], [-1.5554303467325525, 2.5897414574586843, -1.9994611360367696, 1.1724212809984833, 1.6130713925403555], [1.6123653441453905, 1.2499856092679185, -1.4674730028323755, 1.1333266652760936, -0.56770270492972685], [1.5246524149474387, -0.67960556892695567, 5.4639619172066975, -0.70830292124522354, -1.2692213059245729]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_constData_rank3(self):
      arg0=1.96350564402
      arg1=Data(numpy.array([[[2.7957780979985429, 0.78760364818579554], [0.54846692814137832, 2.8694657594986941]], [[3.1224242633585622, -3.1596533196008361], [1.0878923380728933, -1.5607873520928353]], [[4.5421825078137985, -3.8750148806853169], [2.7789485168500558, -1.2091576716587129]], [[2.2083121246792423, -3.7914972800285565], [3.9067911070875212, 1.6976785375054231]], [[0.88952415085523917, 2.3082234201683072], [-1.6953013197703424, -2.5627788629199557]], [[0.42184287968447176, -3.0874044716381697], [-4.1004805237123882, 2.0055423606939007]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.70231097576258572, 2.4930123781725197], [3.5799891356710756, 0.68427568355583179]], [[0.62884011857793598, -0.62143072211133454], [1.8048712867113323, -1.2580225239448468]], [[0.43228241944115081, -0.50670918808801235], [0.70656423899736753, -1.6238623713378102]], [[0.88914317051364389, -0.51787077742707344], [0.50258782468786745, 1.1565827102380686]], [[2.2073663116770881, 0.85065666818242891], [-1.1582045156945984, -0.76616272766658511]], [[4.6545899873660952, -0.63597292225828483], [-0.478847694231542, 0.97903972636193359]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_constData_rank4(self):
      arg0=-2.60579738081
      arg1=Data(numpy.array([[[[2.1725261142014771, -1.611784333668973, 2.226053240449934, -4.5073473700110158], [-4.8282803812057571, 0.64106642804528402, -3.4988584319470553, 3.3382456499350575], [-1.1267702708094927, -4.1240577081477507, -3.3873241635252747, 2.689153433063324]], [[4.9527346244309935, 3.08609223352396, 3.3367712027899241, -1.0199138670652932], [4.3478928290359224, -4.3103114114468921, 1.7158079436584615, -2.8655479125182595], [-3.4121445993980282, 4.5437776857514862, 3.4848073609342194, -3.3291778230676083]]], [[[-1.9243164036967917, -1.1519601717907877, 4.6564843066635326, -4.2267749289145531], [1.7419583541028336, 1.8349548501567625, 4.4823413513952381, 3.123261616700006], [-4.3180099797327163, -1.5851131632392956, -3.9836848556543458, 0.42760149041752715]], [[-4.8449649217963202, 4.5573587660765078, 0.19427086099184177, -1.7052237820838476], [-4.2867738997334968, 3.5786461895900938, -2.9653626316600734, 2.0179629250880895], [-4.6036003646279786, 1.1235792362586192, 2.2167112013768815, -3.3783853397594021]]], [[[-4.3329873655463462, 3.7381795943030909, -0.98293693239915658, -4.2972771430785963], [-4.4313763754317179, 4.9480943340248142, 3.5100988878707469, 2.1349001788309039], [-1.433130750682662, -0.89293167608922364, 1.1822619543492188, 0.23137330018516877]], [[2.0462212665584492, 3.8549950216416491, 2.9169688508471756, -3.4257603863836761], [-1.3361448553786746, 0.466005819122568, 2.1928513547357564, -1.6105891528034166], [-0.68911702189335156, -0.64804225495744205, -0.6733521375175231, -3.8299277157786502]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.1994320177667397, 1.6167159131494504, -1.1705907718006436, 0.57812215631413466], [0.53969471013961112, -4.0647852809156033, 0.74475644885052694, -0.7805888643510418], [2.3126252514060019, 0.63185279285967144, 0.76927901051421432, -0.96900286490505017]], [[-0.52613305141641531, -0.84436795261692266, -0.78093378971529903, 2.5549190622395894], [-0.5993241975528194, 0.6045496791455478, -1.518699916525335, 0.90935397360626324], [0.76368316315414297, -0.57348698836629908, -0.74775937689394711, 0.78271498829298014]]], [[[1.3541418530765688, 2.2620550993154143, -0.5596061769345293, 0.61649778486709905], [-1.4959010786165921, -1.4200880095687343, -0.58134737551759508, -0.83431927920302129], [0.60347182916166164, 1.6439188325724789, 0.65411734994775161, -6.0939857301788347]], [[0.53783617071916512, -0.57177797811405984, -13.413217852146341, 1.5281263422348177], [0.60786909731105976, -0.72815172072265855, 0.87874493088517502, -1.2913009195616723], [0.56603466296279803, -2.3191932502104589, -1.1755240733159351, 0.77131443537260569]]], [[[0.60138587098765917, -0.69707656228683978, 2.6510321211019825, 0.60638336650095659], [0.58803341446115542, -0.52662645554051879, -0.74237150121668194, -1.2205710630623381], [1.818255158900441, 2.9182494591526118, -2.2040778452030834, -11.262308048172301]], [[-1.2734680375941538, -0.67595350089399953, -0.8933236911497392, 0.76064788161041264], [1.9502356876336642, -5.5917700463784126, -1.1883146457602078, 1.6179156405417983], [3.781356863959294, 4.0210300499289797, 3.8698880357230614, 0.68037769226614131]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_expandedData_rank0(self):
      arg0=-3.20459365093
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.97816626317)+(1.-msk_arg1)*(-2.66160300259)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.805545429461)+(1.-msk_ref)*(1.20400888029)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_expandedData_rank1(self):
      arg0=3.95344243959
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-3.8915500966441572, 2.5911008941434011])+(1.-msk_arg1)*numpy.array([-4.5838176526004037, -1.2093009944548108])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.0159042904276341, 1.5257771121652337])+(1.-msk_ref)*numpy.array([-0.86247812177957861, -3.2691963851209795])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_expandedData_rank2(self):
      arg0=3.89815297743
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.196454530139039, 1.9144309626597424, -0.53782730390699207, -4.081588810087414, -2.6069191849825546], [-4.6213936351743783, 1.219994074786066, 0.7606003813500859, -1.2302731333641868, 4.0898533221639362], [-0.48723179530520433, 0.15322234012567826, -1.3908615372186697, -0.35862024506728929, -0.12122391355231343], [-1.8694682622507841, -0.36393249714014342, -4.3790843686811067, 2.8462793641645909, -2.5707926762400235]])+(1.-msk_arg1)*numpy.array([[-0.11594662439032621, -3.6391726521062271, 3.5603654462542575, 1.7331919197847681, -0.72690525117661231], [-1.1818438101765985, -0.27756118318590595, 4.8116845705646121, -4.9929893797032214, -0.56969619966783203], [4.7454341413419847, -4.2258781544582744, 0.34034849132755696, 2.4747381661806642, 2.2572015760033644], [2.8064101873598251, 1.1611153392272229, -2.9350101846230192, -4.1851245894940163, -1.9298724231096398]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.2195239884292739, 2.036194072003636, -7.2479640752900485, -0.95505773824984985, -1.4953102496938615], [-0.84350161123656231, 3.1952228768910609, 5.1250999513156392, -3.1685264610863011, 0.95312781910855282], [-8.0006128807463401, 25.441152864723133, -2.8026894648501006, -10.8698631241428, -32.156633647582723], [-2.0851667054966323, -10.711197840425001, -0.89017535384960578, 1.369560917493468, -1.5163233556154836]])+(1.-msk_ref)*numpy.array([[-33.620236879903651, -1.0711646162684438, 1.0948743987865344, 2.2491179037530151, -5.362669991884359], [-3.2983656079270709, -14.044301629947523, 0.81014308404064095, -0.78072526916924556, -6.8425118154217017], [0.82145339316111632, -0.92244802972264006, 11.453416356343814, 1.5751779443576914, 1.7269848731587858], [1.3890175409800771, 3.3572487122789942, -1.3281565419605763, -0.93143056892824072, -2.019901901674316]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_expandedData_rank3(self):
      arg0=-1.27190300056
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.1230776710483594, -4.5956001432878866], [-1.1906802976086004, -1.0337927633438126]], [[-4.4332297509624983, -4.2193007285271893], [-3.8912967020440759, -4.3478668570430106]], [[-2.9555107449505789, 0.83108498872491054], [-4.409503816269293, -3.8563676601428076]], [[2.7165616594243165, 3.6890204735863144], [-2.7655425212469309, -2.6699198913092923]], [[2.4878405918672906, 2.5261864139999801], [-3.8522993154722651, -1.9163487687006797]], [[4.2291188693660136, 1.6054676540699635], [3.4996207807552597, 4.8977389656851056]]])+(1.-msk_arg1)*numpy.array([[[4.1507116003996973, -3.9686550141586006], [-3.4595970598022174, 1.0669534614657374]], [[0.34164407886175141, 1.3278715866199686], [-3.1945390324679588, 4.6557279548631527]], [[2.628808514089279, -2.880785614719815], [-2.7652929515567024, -2.9707538829771227]], [[0.21831286601512989, -1.4944988395788208], [3.2766635883058264, 0.087724203888184249]], [[2.5776544644234392, 4.2845215561378218], [1.9081134898064693, 1.5555060535587648]], [[0.58254004942708182, -2.0024953855807128], [1.4197994208522768, -3.8650476213729981]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.30848388074016059, 0.27676537577269028], [1.0682153749521437, 1.2303268562686767]], [[0.28690211696832502, 0.30144876660689651], [0.32685839655711107, 0.29253494699309257]], [[0.43034964522836461, -1.5304126747729063], [0.28844583280896396, 0.3298189157905656]], [[-0.46820325102712301, -0.34478068356221653], [0.45991084598642296, 0.47638245802737444]], [[-0.51124778842982577, -0.50348738854317632], [0.33016723167113887, 0.66371164859540421]], [[-0.30074893608955755, -0.79223209345497891], [-0.36344023545419951, -0.25969187199836502]]])+(1.-msk_ref)*numpy.array([[[-0.30643010717382096, 0.32048716656410076], [0.36764483798896941, -1.1920885460278767]], [[-3.7228890510723156, -0.95785090469152012], [0.39814915004356211, -0.27319100533559559]], [[-0.48383250196476846, 0.44151254923626321], [0.45995235327312867, 0.42814149224757431]], [[-5.8260560807714992, 0.85105653271470483], [-0.38817015121644782, -14.498883366092494]], [[-0.49343425122055529, -0.29685998398958396], [-0.66657617974657479, -0.81767794966029073]], [[-2.1833743479250844, 0.63515901695286858], [-0.89583287743180195, 0.32907822235480694]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_expandedData_rank4(self):
      arg0=-2.99311882056
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.8526421407048783, -1.5400316582756366, -2.4177011363501286, -2.7100096677802963], [-3.4161194534084718, -1.675239526910679, 0.91442160874590606, 1.0720067925928571], [2.6720297025826811, -3.3503682154377112, -3.1427193670829947, -3.9206133222408068]], [[3.3463954932046089, -2.0412572267117, 0.035911546786685733, -4.4715208268978852], [-0.58460995360733126, 0.67613870691768607, 1.8096654790598148, 3.6350194814241323], [-1.4040291586258014, 0.64483448103090879, -3.249637068595479, 3.9268421269317795]]], [[[0.10800922223141907, 1.9976117588337194, -1.2378774668501871, 2.2500280937633041], [-4.8594960096647108, 3.4890429055426413, -2.5538098765389297, -4.4661991288958225], [1.2517761826918852, 3.149613144183002, 1.9256578159023272, -4.140923038897963]], [[-3.9444401195119738, 0.27232068566507284, 3.0254065762511111, 2.4324717344863247], [0.1380379190713672, 2.9257183123207886, -2.9995838045184966, -2.2787322194584192], [-0.32894851544758374, -0.012507904594849961, 1.0483961595717775, -3.3532692243897322]]], [[[-1.1302007836730232, -3.230924129943249, -2.047218679934224, 4.3910248209760638], [0.38615724718903444, -4.2566621031344107, -3.6043153153277672, -2.8261375382181519], [0.85512271729660494, 1.0083796165158629, -3.426192939622632, -0.87977999851976385]], [[-0.093635407269392346, -3.9958300757616838, -3.5360555023765894, -4.2263582695996718], [-3.1107492631218516, 0.5995153668840576, 2.9949277968760253, 3.9505072752813923], [4.5026436537651797, -3.797019211455952, 1.4662383925508884, 2.9026138216666206]]]])+(1.-msk_arg1)*numpy.array([[[[-4.9411832158954327, 4.9510431179201912, -2.9034861100837386, -4.7127195743019579], [2.7859537540717341, 0.91133101449001774, -4.2497086078013666, -3.8556088441724325], [0.57791558184128533, -0.60366994535270102, 0.15996666102123314, 0.8418916525451392]], [[4.1579000601970471, 3.3625866693066158, 3.8868439867942648, -3.6838790218132345], [-2.6283103540281827, 2.5031674384164138, 2.7083347632287893, -0.014263039276753098], [-4.8108147879908074, 1.0106045264998507, -3.0238445130945912, 2.0931956842916941]]], [[[-2.4784615661234923, 4.1370134018167004, 3.9276346793157462, -2.9257372362408063], [-4.8202163917399732, -2.8107487909800257, 0.51361424577416681, 1.900346210229225], [0.78143051930426033, 2.6085255248349624, 0.5490993667972468, 1.2263996260903003]], [[-0.60742592121970063, -4.77612182149538, 1.7988396257939874, -0.40732986250086611], [-4.4804682349304645, 1.4819540888353808, -2.569814610175285, -4.7521179871077273], [3.3559897838321273, 1.1744824427612368, 1.1327525098155489, 0.28907111535921892]]], [[[3.38666778556113, 4.3253173861654162, -1.787132623683938, -3.5084441768404195], [0.89871742933957055, -0.3926968775175439, 4.5460538659715439, -1.7937678237989072], [-3.6017996320660881, 4.7023933135998934, 0.061160490048011162, -3.7380436232065808]], [[2.8989075115141922, -3.3050082679648729, 3.6086344640118355, -0.41885083464055661], [1.5061708611940006, -1.2192783910983485, -4.8753628230345258, 3.5226300322703743], [-1.3279786113086454, 4.4381351023780216, 2.8262462093796259, -2.7407340551008152]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.61680188519332713, 1.9435436956580017, 1.2380019910458824, 1.1044679493733849], [0.8761751049334453, 1.7866811118497086, -3.2732371937932423, -2.7920707604062014], [-1.1201667472716781, 0.89337010981770959, 0.95239773932901328, 0.76343127325921789]], [[-0.89443068717769847, 1.4663114385525158, -83.346975788440716, 0.66937378498845679], [5.1198560717044499, -4.4267822414724405, -1.653962489305141, -0.82341204382836042], [2.1318067378920231, -4.6416854380525638, 0.9210624932491317, -0.76222030929825324]]], [[[-27.711696822909591, -1.4983486192045878, 2.4179443448239057, -1.3302584215956097], [0.61593194327200751, -0.85786242863351658, 1.1720210059692457, 0.6701713770866593], [-2.3910974357403028, -0.95031316023154189, -1.5543357681919372, 0.72281440452752777]], [[0.75881968793220655, -10.991154833667641, -0.98932779615499866, -1.2304845224388274], [-21.683308765382609, -1.0230372513822421, 0.99784470633779054, 1.3135017774342608], [9.0990494864607161, 239.2981812307512, -2.85495019533256, 0.8925972298272814]]], [[[2.6483071537325058, 0.92639712360188586, 1.4620415737177226, -0.68164470541298894], [-7.7510362484279334, 0.70316100926859015, 0.83042646347470439, 1.0590846270144656], [-3.5002213834498899, -2.9682460568740248, 0.87359902763824504, 3.4021219231981807]], [[31.965673112779271, 0.74906058661285901, 0.84645696837724149, 0.70820281424907816], [0.96218581678664461, -4.9925639706483143, -0.99939598666700391, -0.75765429905255732], [-0.66474699103768953, 0.78828118949855197, -2.0413589193692254, -1.0311805167511967]]]])+(1.-msk_ref)*numpy.array([[[[0.60574941057168308, -0.60454307289747167, 1.0308707212892532, 0.63511498474820427], [-1.0743605546864663, -3.284337713701432, 0.7043115415162251, 0.77630250928574018], [-5.1791626919264493, 4.9582041372071091, -18.710891391038416, -3.5552304284129619]], [[-0.71986309849244057, -0.89012391795764467, -0.77006404957971386, 0.81249107335835224], [1.138799615489875, -1.1957325645178962, -1.105150981035687, 209.85140421184613], [0.6221646337387684, -2.9617112748562322, 0.98983886492635986, -1.4299278576853067]]], [[[1.2076519004637314, -0.72349749199288871, -0.76206650183585778, 1.0230306342893327], [0.62095113109121503, 1.0648830767660808, -5.8275619206080362, -1.5750386979193194], [-3.8303070415271829, -1.1474370452038118, -5.4509602478936223, -2.4405738202132876]], [[4.927545427342352, 0.62668393571632575, -1.663916436816441, 7.348144823408294], [0.66803705854234319, -2.0197108959748853, 1.1647216918699024, 0.62984943317391162], [-0.89187363888145643, -2.5484576964117545, -2.6423413716759141, -10.354264613522192]]], [[[-0.88379463533929947, -0.6919998125753456, 1.6748162844151053, 0.85311855332150344], [-3.3304337079062414, 7.6219572701380409, -0.6583993302322404, 1.6686210895546196], [0.83100647629263569, -0.63650967091563526, -48.938764522745721, 0.80071800178398789]], [[-1.0324989012815899, 0.90563126560600904, -0.82943253200209277, 7.1460256802972433], [-1.9872372369377729, 2.4548280707731522, 0.61392739970322308, -0.84968299058821495], [2.2538908345864153, -0.67440912714702894, -1.0590439044630704, 1.0920865579733356]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_Symbol_rank0(self):
      arg0=2.42790243501
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(1.90014564031)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(1.27774544409)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_Symbol_rank1(self):
      arg0=1.92983179025
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([3.8943436629686516, 4.6055252497591876])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.49554737775259694, 0.41902534143146103])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_Symbol_rank2(self):
      arg0=-2.92009770269
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-0.56466523022513737, 4.4876366887208956, -4.1589241692233188, 0.91180492353070353, -1.1311157765263657], [-1.9738929009481012, 1.747574484209605, 1.2937623205210897, -3.4755375076886095, 2.1589740393431516], [0.11184076941878196, 0.91647365199472119, 1.1727894825792209, -4.5448935170254403, -4.4791565045126367], [0.51845614376322935, -1.4098938476015133, 4.8008761494336536, 0.0029476699242341198, 2.4353086961641566]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[5.1713786264521806, -0.65069833082216078, 0.70212814272886948, -3.2025465396476838, 2.5816081459446512], [1.4793597470685429, -1.6709432010319609, -2.2570588556875308, 0.84018592699031847, -1.3525395162117786], [-26.10942072253474, -3.1862320278726299, -2.4898737122582659, 0.64250079605784305, 0.65193026851038827], [-5.6322945302400305, 2.071147205624448, -0.60824266483763623, -990.64609598230095, -1.1990667578555474]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_Symbol_rank3(self):
      arg0=-2.44472523509
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[3.0269819670616478, 3.6317184712677708], [0.3239334404340255, -1.9161388263697443]], [[-3.1169884211450336, -1.0427593551719836], [-3.5288234973596544, -1.786745887524106]], [[1.6491947666577866, 3.9258977543945619], [-4.3256365377620352, 2.193464036497077]], [[-4.7400371417731266, 0.53436317113168208], [0.52884519537175478, -1.3458590101347223]], [[0.21015441727695361, -0.69084185818165444], [0.31611879965944212, -4.9656634611716353]], [[2.8092620041813259, 0.80854905786792663], [-4.8741506257921436, 4.5668812743315907]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.80764446623397546, -0.67315934713242287], [-7.5469986421030928, 1.2758601837420478]], [[0.78432284781773354, 2.3444769140279371], [0.69278762083639844, 1.3682556944205591]], [[-1.482375086625934, -0.62271750005480986], [0.56517120977348845, -1.1145499513142492]], [[0.51576077612186477, -4.5750256888211425], [-4.6227615500389296, 1.816479450431181]], [[-11.63299476054112, 3.5387624622544829], [-7.7335648424614423, 0.49232600118868047]], [[-0.87023753265043524, -3.0235954284999118], [0.50156948826136416, -0.53531613550544599]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_float_rank0_Symbol_rank4(self):
      arg0=4.2498744183
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-3.28799472386895, -3.7765388840709626, -2.1117109099459075, -2.0195205103881362], [1.948843942772359, -2.780398372094198, -4.2687430479530795, -2.642493211496749], [-2.8446513426575537, 1.212190246687884, 3.9322250900537146, 2.2349064975378328]], [[-4.8931203973194535, -0.95836155635981601, -4.3433760294309121, 2.0360079992016473], [1.9257985779339615, 4.13715315877851, -1.4391676388473371, -2.030358020675159], [1.0723643496984412, 0.38957370413081982, -4.3364367222378259, -4.5569961178883132]]], [[[4.5155232106587491, 1.428176611572086, -3.438630795165146, 3.9160734888810502], [4.7802230658367293, 2.899582186984575, -2.4106353602251387, -4.6468313337964524], [-2.6402806076597618, -4.4789486404699339, 2.4723549099885958, -3.0541864551604849]], [[-3.9268742690210887, -2.8166787569038609, -4.1315804138818244, 1.7841901517344381], [-2.7152866166535183, 4.2821003820940504, -4.5231015335546152, -1.0225668067852665], [-3.5876328661055581, 4.7788248955427637, -1.7941114242536904, 3.420050110700787]]], [[[-2.3436601805942936, -2.5121334269177553, 4.3915830166569769, 1.7936561171068561], [3.7353153585755461, -2.5041709450983363, -3.3602006595440792, -2.9743678146863739], [1.8787693913665162, 2.4940733494344656, -1.1325576272326323, -1.1859058277489942]], [[2.5524675740392988, 4.9603199016470327, 1.7529820750837422, 4.3674290085245229], [-1.3137466294448332, 0.82650713335122461, -2.5208852218301105, -3.1109496888420973], [-0.23650868667222635, 2.8751669751639852, -4.5763682891264104, -1.6984907914274316]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-1.2925429555732377, -1.1253358031673659, -2.0125266191893885, -2.1043977500788005], [2.1807156155628644, -1.528512770311319, -0.99557981601554979, -1.6082820571908261], [-1.4939878060164771, 3.5059467190986617, 1.0807810644025799, 1.9015893608886167]], [[-0.86854074153314276, -4.4345209697697561, -0.97847259585664315, 2.08735644455485], [2.206811484334017, 1.0272460929518874, -2.9530086027383216, -2.0931650354382154], [3.9630881234478639, 10.909038195431346, -0.98003837955362316, -0.93260435347225012]]], [[[0.94116987556770626, 2.9757345022060497, -1.2359205368236332, 1.0852386785808537], [0.88905357757711401, 1.4656851036591749, -1.7629685884562283, -0.9145747097359449], [-1.6096298272120786, -0.94885535857649062, 1.718958067520487, -1.3914914759437864]], [[-1.0822537537872876, -1.5088246779587726, -1.0286316596960088, 2.3819627152225995], [-1.5651660462778636, 0.99247426241336567, -0.93959297326648616, -4.1560848544062265], [-1.1845901118951732, 0.88931369346942712, -2.3687906786874375, 1.2426351312811501]]], [[[-1.8133492446934532, -1.6917391300800206, 0.96773177284348222, 2.3693919797479217], [1.1377551853933596, -1.6971183323636463, -1.2647680447974849, -1.4288328421637737], [2.2620521910928373, 1.7039893470905532, -3.7524575492756282, -3.5836525286039]], [[1.6650062322136874, 0.85677426104869747, 2.4243684397604364, 0.97308380056177424], [-3.2349269813881145, 5.1419694359646844, -1.6858658940502904, -1.3661019442203186], [-17.969210679304215, 1.4781313415915236, -0.92865655685898674, -2.502147459231816]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_float_rank0(self):
      arg0=numpy.array(1.48299586238)
      arg1=-2.69134968237
      res=quotient(arg0,arg1)
      ref=numpy.array(-0.551023106399)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_float_rank0(self):
      arg0=numpy.array([-0.10741480847023421, -3.1689363926684075])
      arg1=3.10028502087
      res=quotient(arg0,arg1)
      ref=numpy.array([-0.034646752716972928, -1.0221435678770814])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_float_rank0(self):
      arg0=numpy.array([[-1.7462325116846933, 3.8610721003773065, 0.37031651467380211, -0.41172658006642671, 4.7480098636403731], [-0.77475737356855845, -0.0049544880579492911, 2.2001882122090155, -1.7393929848644296, -3.8510614575256308], [0.87162102833365118, 3.4158953069479008, -3.83872097598596, -4.7444036342675355, 3.2514280623043543], [-0.22856932680708653, 3.0449152690323853, 1.4157920899580159, -4.4915492011804723, -1.824104563597071]])
      arg1=0.883462342273
      res=quotient(arg0,arg1)
      ref=numpy.array([[-1.9765783193338737, 4.3703867336818307, 0.41916502487366764, -0.46603749856177573, 5.374320598006082], [-0.87695574162816725, -0.0056080353636832354, 2.4904153883326283, -1.9688365894456854, -4.3590555853434418], [0.98659669646035753, 3.8664865988056221, -4.3450872689261049, -5.3702386703445599, 3.68032445382907], [-0.25871994296782519, 3.4465705252341614, 1.6025494491541699, -5.0840301688746301, -2.064722485968105]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_float_rank0(self):
      arg0=numpy.array([[[-0.73055146992086684, -4.9771547751081471], [-1.435242501983268, -4.9215103846390189]], [[2.4191711385331498, -4.128136473300783], [2.8191677397571233, 2.9800335717105657]], [[2.6659347090706733, 1.0815371679168306], [2.426481645682955, -3.0421595972799684]], [[-0.85230096242149589, -3.6002357927841655], [4.1995103404042489, 4.2587537911784263]], [[2.6087717058233864, 1.1719252638023034], [-3.0243264060957098, -2.8467883340994984]], [[0.45301103734540149, -2.7179137554510655], [-1.9526366507271433, 1.0976352976200285]]])
      arg1=4.63156666949
      res=quotient(arg0,arg1)
      ref=numpy.array([[[-0.15773312186846369, -1.0746158115120583], [-0.3098827253070171, -1.0626016499032751]], [[0.52232242590142541, -0.89130455586289359], [0.60868555737889196, 0.64341804498682553]], [[0.5756010653222845, 0.23351432573365058], [0.52390083503843721, -0.65683165424751644]], [[-0.18402001379697558, -0.77732569769511517], [0.90671486347513341, 0.91950609698245067]], [[0.56325902053993893, 0.25302998907945973], [-0.65298129594398036, -0.61464911060253702]], [[0.097809460528635095, -0.5868238437231178], [-0.42159312173765834, 0.2369900674970665]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[-0.0745539458461284, -4.6268506066631385, 0.51232561665592868, 2.2319546581712064], [-0.6374394506754566, 4.7934592146454928, -2.8215842476366371, -4.2769675535055196], [3.5931109608107796, 2.7655681997151422, -4.1336408023474842, 1.416937003900081]], [[2.7950737697781669, -3.3955843917320507, -3.9274224590911153, 2.5799644436664062], [-0.27530410676032524, 1.7802087854434774, 0.35089449882073787, -0.55889469924017199], [-3.5338038869851065, -2.6934499317708793, 1.0971693140508378, 3.2624017707291415]]], [[[3.6086086114148159, -4.0330559948718747, 4.0340805262015262, -4.2693282492273275], [4.7912141318000998, -1.6859759760441118, 3.7520367643417494, -0.34233484160746119], [0.6463226153568522, -1.9934583970979745, 4.0452954860443331, 2.8965336466179288]], [[-1.1395111957209716, 1.3873662450742721, 3.0655967957228629, 4.618972857628119], [-3.6183019785898471, 3.6499307157104521, -2.8655668842938686, 3.7246970270171893], [-3.9689555754305559, 1.7645220198789815, 2.7373757287616005, -4.8268833618717242]]], [[[1.0979886305035889, 1.5932499964360867, -4.1509263239628833, 1.8892936797338304], [-0.96509040009594482, 3.5509000165839772, -1.7381765582911326, -1.6775038978857459], [4.9644511608998556, 2.88765244456222, -2.9176788895519401, 1.8719811389067562]], [[3.8152077151119936, -3.4768085529302608, 4.1763204986667795, 0.81461216607153197], [-4.7833649314809659, 0.28249365259467396, 3.2982134897149749, 4.8805734981736144], [1.6579522010322467, -1.3831868919671719, -1.5834681240451562, 3.0102403400603244]]]])
      arg1=-1.98776650817
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[0.037506389980706233, 2.3276630266375995, -0.25773933434806834, -1.1228454896458087], [0.32068125106972295, -2.411480017867357, 1.4194746898265338, 2.1516448415444316], [-1.8076121848501401, -1.3912942935419739, 2.0795404215490669, -0.71282869395074011]], [[-1.4061378729773382, 1.7082410724651818, 1.9757966757879541, -1.2979212764971724], [0.13849921790529826, -0.89558244296991552, -0.17652702034064774, 0.28116717780592132], [1.7777761484837866, 1.3550132375689847, -0.55196086136856903, -1.6412399330178173]]], [[[-1.8154086994509819, 2.0289384987065033, -2.0294539170544024, 2.1478016817735122], [-2.4103505678861032, 0.84817606550460112, -1.8875641323638503, 0.17222085199638712], [-0.32515016864372903, 1.0028634595167154, -2.0350959075984094, -1.4571800232613819]], [[0.57326209644624826, -0.69795231953542403, -1.5422318381550295, -2.3236999107471621], [1.8202852114231751, -1.836196907787188, 1.4416013513214263, -1.8738101339882127], [1.9966910394713486, -0.8876907889451614, -1.377111304323863, 2.4282949441118635]]], [[[-0.55237304079174687, -0.80152773974462788, 2.0882363732840048, -0.9504605656477183], [0.48551497176744046, -1.7863768214142219, 0.87443698801858194, 0.84391395618565901], [-2.4975021666213943, -1.4527120930396018, 1.4678177127722314, -0.94175102116354981]], [[-1.9193439971067681, 1.7491030956777871, -2.1010116034730042, -0.40981280382885787], [2.4064018142035857, -0.142116114459691, -1.6592559921669103, -2.4553052273021332], [-0.83407794336770069, 0.69584978229580574, 0.79660670281664359, -1.5143832676941991]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_array_rank0(self):
      arg0=numpy.array(-0.442055328297)
      arg1=numpy.array(1.50935966986)
      res=quotient(arg0,arg1)
      ref=numpy.array(-0.292876069981)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_array_rank0(self):
      arg0=numpy.array([-3.9369874561588079, 0.3061952811767954])
      arg1=numpy.array(0.740564070842)
      res=quotient(arg0,arg1)
      ref=numpy.array([-5.3162010029480733, 0.41346224213735105])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_array_rank1(self):
      arg0=numpy.array(-2.23314667879)
      arg1=numpy.array([2.5355727580016936, 1.9004180396504546])
      res=quotient(arg0,arg1)
      ref=numpy.array([-0.88072672012358921, -1.1750818147360353])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_array_rank0(self):
      arg0=numpy.array([[4.0176524465654655, -4.155143142660692, 1.4181287106034919, 3.1976207151102169, -4.2028616597454374], [2.7155803266760952, -3.5622324058659949, 2.8971682419917864, 1.6293238378608113, -4.9845860530458452], [3.1800078608395381, 3.2672274632464333, 1.0606609522897852, 2.6292800432902368, -2.3994723792626504], [1.3882892163390403, -0.074537064495025618, 0.59403719509955089, 0.043217888028558527, -1.5226918808732757]])
      arg1=numpy.array(1.40580012922)
      res=quotient(arg0,arg1)
      ref=numpy.array([[2.8579115644307045, -2.9557140138954612, 1.0087697967377884, 2.2745912798377645, -2.9896580405696849], [1.9316973090548446, -2.5339536765100696, 2.0608678159724398, 1.1590010585424479, -3.545728834030343], [2.2620625754333217, 2.32410525176706, 0.75448915549685736, 1.8703085798946275, -1.7068375008617229], [0.98754381045118445, -0.053021096630982548, 0.42256163074242215, 0.030742555168671963, -1.0831496236396929]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_array_rank2(self):
      arg0=numpy.array(4.48824581683)
      arg1=numpy.array([[-1.0102772318801421, -0.47804633912243233, 2.220852040974143, -3.1874929673316545, 0.79426512438866226], [3.7539156760390142, 0.13143584981459, -3.9993974128176877, 2.4079022387398439, 1.7735016900915817], [4.6485475353061894, 4.6664141682986049, 4.6021990083372124, 4.8206097355690734, -1.034355706228042], [4.8271773465720518, -0.045409707445741354, -0.38497627887866415, 4.4903077469996653, -4.5125874213512187]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[-4.4425883066586431, -9.3887254216263294, 2.0209567022148316, -1.4080802256926785, 5.6508156773076728], [1.1956171113491829, 34.147805360299543, -1.1222305146394758, 1.8639651330624905, 2.5307254241197894], [0.96551574072234392, 0.96181900169195989, 0.97523940375099027, 0.93105355194332984, -4.3391705482066234], [0.92978680802383251, -98.838906244816641, -11.658499661088925, 0.99954080426517289, -0.99460584311300759]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_array_rank0(self):
      arg0=numpy.array([[[-1.8160210496655758, 0.080915820511328285], [3.1901526850103465, -4.7960839234525547]], [[-1.5872387405454145, -2.4682923845698435], [0.52443631009326896, -4.7675642689621736]], [[3.0322590715974673, -0.63770515654109694], [1.5667286942772609, 2.9323587223009699]], [[1.4856697518984365, 1.9594479149245183], [1.6404519086844349, -2.0999621183747585]], [[4.3292786874449423, -3.1546081099239123], [-3.6942474168584405, -0.87599803528576192]], [[3.2585028766378556, -3.3824441434336694], [-2.7106874692899896, 1.3451345675832016]]])
      arg1=numpy.array(-0.458995253914)
      res=quotient(arg0,arg1)
      ref=numpy.array([[[3.9565137856649928, -0.17628901349487103], [-6.9502955810693789, 10.449092626894775]], [[3.4580722284391672, 5.3775989261803447], [-1.1425745813736048, 10.386957660907253]], [[-6.6062972236430948, 1.3893502189906524], [-3.4133875697346219, -6.3886471533109654]], [[-3.2367867406680739, -4.2689938473560298], [-3.574006255393158, 4.5751281750032726]], [[-9.4320772394217975, 6.8728556189274324], [8.048552540263417, 1.9085121857270424]], [[-7.0992082137048236, 7.3692355521989032], [5.9056982532499847, -2.930606702602661]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_array_rank3(self):
      arg0=numpy.array(-3.08119276551)
      arg1=numpy.array([[[-1.2046885311344369, 2.3765388912482974], [1.344195631063684, 0.90774043098564849]], [[-0.78864216368420514, 2.7766870864986304], [1.6213774431820127, -2.02874064765868]], [[-3.4421459693442378, 2.5366851978778469], [-1.8905237321808874, 4.7309098919450712]], [[4.9570590876980312, -1.83365498979013], [4.9591982417001006, 3.6321738638554244]], [[0.41915749399486657, -3.0570077960538677], [-4.94938720661417, -4.013176287491051]], [[2.7792615199216106, 4.7157923678038713], [3.9179813320787265, -2.3266542471455267]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[2.5576675513043834, -1.2965042469356209], [-2.2922204880794701, -3.3943544435554656]], [[3.906959210901241, -1.1096651043227335], [-1.9003550212615117, 1.5187711495143643]], [[0.89513715947905792, -1.2146531891654391], [1.6298090910272041, -0.65128967490105472]], [[-0.62157676779720206, 1.6803557826675701], [-0.62130865017705006, -0.84830541736251353]], [[-7.35091894968917, 1.0079113208309252], [0.62254025334560659, 0.76776910476498972]], [[-1.1086372201483201, -0.65337752920321857], [-0.78642354425826988, 1.3243019538856167]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[1.3363114706438104, -1.0866347105687515, -2.6738302392255999, 3.0233504358112757], [3.7889149178870714, 3.0930390200360698, 0.61874369922093297, -1.1801536359184328], [-1.775983537052277, 2.6145650513991905, -1.163531313968412, -2.050622370814188]], [[-1.0202737545176976, -1.4779480618120742, -2.4725971705836347, 4.2199217853575295], [-3.2812720518134917, -2.6204255982552063, 4.5142465192918841, 4.9075081098276048], [-1.7768518000253297, 3.14885758854256, 1.6344128145620909, 1.5130791772410728]]], [[[-0.30299861753378377, 4.6863032780234768, -2.712672124944604, 0.32677059407953646], [-3.0891484965151972, -3.2830302868031191, -4.2699937948738453, -3.600616583338867], [-2.7473097923778687, -1.6556759803579792, 0.41006023774856093, 0.21647156924104216]], [[-0.86726949012948573, 3.3347300703464988, -4.8361512614869433, 2.9122716868485776], [4.9346738110233073, -2.3917489850611973, 3.1500032168967529, 1.059487196859811], [-3.1058523387789205, -1.4488257246962122, -4.1951870334322727, -4.2143590147104648]]], [[[-4.704525551537726, -2.7532251394882303, -3.4999646094413359, 4.4544957259811078], [1.6256540094937648, 3.3529401805212693, -0.052223103234791068, 4.5482575928889446], [4.5580688055356635, -0.67889085152245876, -2.9161874020167633, -0.5075036871219254]], [[-3.9530575984327134, -2.7174781345428354, 0.79887890208561174, -2.8000569694191482], [0.58770748379090154, 1.7894088703292175, 3.2364203813894203, -3.4592717484918802], [-4.2541829534083337, 4.5917755077513256, 0.36334632104599329, 0.30226072833780204]]]])
      arg1=numpy.array(1.72184526588)
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[0.7760926589175543, -0.6310873178325026, -1.5528864830105751, 1.7558781243102455], [2.2004967536637707, 1.7963513222250724, 0.35934918861805815, -0.68540051728351015], [-1.0314420071547028, 1.5184669047820902, -0.67574673347414116, -1.190944628675723]], [[-0.59254671411744175, -0.85835126482987012, -1.4360158950261444, 2.4508135945614375], [-1.9056718491661979, -1.52187054793873, 2.6217492411994665, 2.8501446715716763], [-1.0319462702207614, 1.8287691994945856, 0.9492216559447012, 0.87875444281867787]]], [[[-0.17597319778812043, 2.7216750371700753, -1.5754447735193817, 0.18977930279463542], [-1.7940918140114368, -1.9066929833090072, -2.47989402966699, -2.0911383006836006], [-1.595561370590896, -0.96157071321320264, 0.23815161900639872, 0.12572068671340192]], [[-0.50368607871665272, 1.9367187844467557, -2.8087025921034821, 1.691366666072023], [2.8659217577791121, -1.3890615100274595, 1.8294345486853012, 0.61532079441321197], [-1.8037929425592196, -0.84143781871894685, -2.4364483363031288, -2.4475828915724489]]], [[[-2.7322580284964784, -1.5989968402161692, -2.0326824243677288, 2.5870476367668918], [0.94413478475954049, 1.9472947116427637, -0.03032973070783291, 2.6415019299404126], [2.6472000102738313, -0.39428098736518769, -1.6936408048960929, -0.29474407322056312]], [[-2.2958262724062446, -1.5782359706690086, 0.46396672100259106, -1.6261954688379454], [0.34132421503594379, 1.0392390685648958, 1.8796232422949715, -2.0090491387550493], [-2.4707115312283729, 2.6667759285537906, 0.21102147111903366, 0.17554465219775736]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_array_rank4(self):
      arg0=numpy.array(2.90213346276)
      arg1=numpy.array([[[[3.7073476215478962, -1.0830543009184512, -3.8981152503724958, 4.6791445227267623], [1.2767349356051003, -3.269122599902925, -0.57457499926721223, 1.2109734168890354], [3.9575313368129699, -0.62849473313943172, 1.2542339947389101, -3.4667855559893646]], [[1.6013560937790121, 2.5116757279386412, -2.3876210869867243, 4.2968851406360145], [-0.28339881990027749, 1.2809025117084039, -1.5597330272582823, -2.9641438491187735], [-1.9625750893987437, -0.70876896735183248, 4.1379867781077042, -4.3827693977918436]]], [[[-1.5103053756449549, -4.9722847411729543, 0.53754627248961295, 0.5808214026732692], [4.4429224767542408, 0.34486923387098578, -3.5743132691449384, -4.8972118616184579], [-2.0358464356222639, 1.0512781616688569, -2.6997391867304166, -2.2445220212954862]], [[3.6167419707694339, -3.2672564212966995, 3.2162986036165027, 3.0985377334683655], [1.3528978777764777, 0.12250418429090182, 3.6500118818294052, -1.21372663788198], [-1.7476198066015494, 1.1491006056280471, 0.55462327179041981, -3.3266753337751886]]], [[[-2.2442490824552541, 3.9008136000906148, 0.40789028572040475, 0.11255501536943857], [-2.2779783561366962, 3.981484256921954, -3.5557177902099468, 0.70434171507519139], [0.33530401607059801, 3.4777776260608242, 1.2868107295688826, 1.6246787012080404]], [[-2.0342124074570567, 3.1496994282450466, -1.6015075468238891, 4.6653567969089877], [1.7448190987052223, -3.8090925316842847, 3.2610709170895866, -0.58189773369612308], [-1.4842229708837076, -4.4537797257793423, 0.030170833203065683, 0.60919008023972587]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[0.78280586527602158, -2.6795826029255743, -0.74449657754153875, 0.62022736178996862], [2.2730900375882284, -0.88774078489775177, -5.0509219274525616, 2.3965294549737681], [0.73331913654584269, -4.6175939267910504, 2.3138692420539355, -0.83712517428456279]], [[1.8122973859712288, 1.1554570641754831, -1.2154916366681168, 0.67540401192460897], [-10.240457119002844, 2.2656942556027007, -1.8606603899811027, -0.9790798323190667], [-1.4787375415296349, -4.0946113563747275, 0.70133947216040726, -0.66216887072063935]]], [[[-1.9215540840709024, -0.58366196101637435, 5.3988532918733609, 4.9966021386394148], [0.65320371398533683, 8.4151706726337849, -0.81194155191077078, -0.59260933461140652], [-1.4255168818161641, 2.7605761905657729, -1.0749680847058043, -1.2929850699748346]], [[0.80241650806717491, -0.88824784116975153, 0.90232090375612384, 0.93661388448424099], [2.1451238193488278, 23.690076217097332, 0.79510246999805301, -2.3910931606718173], [-1.6606206062680984, 2.5255695180655007, 5.2326211509232436, -0.8723825355903887]]], [[[-1.2931423189398428, 0.74398157930370878, 7.1149854859601085, 25.784132792653871], [-1.2739951874198805, 0.72890743137294545, -0.816187794980615, 4.1203486896342447], [8.6552302497735649, 0.83447930684744853, 2.2552916260941474, 1.7862814725193981]], [[-1.4266619612219955, 0.92140013003774646, -1.8121259987318292, 0.62206034588557035], [1.6632861623983755, -0.76189628858439717, 0.88993264376932768, -4.9873599684448076], [-1.9553217540063195, -0.65161135966529493, 96.190033706789166, 4.7639210763621866]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_array_rank1(self):
      arg0=numpy.array([3.8336218270979181, 1.3627783080410349])
      arg1=numpy.array([-4.8582369181717269, 2.2150224809165442])
      res=quotient(arg0,arg1)
      ref=numpy.array([-0.7890973395633829, 0.61524355611828241])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_array_rank1(self):
      arg0=numpy.array([[-2.1307163225068049, 2.9386937618032611], [-0.25447190484708493, -4.7996731123826457]])
      arg1=numpy.array([0.47773664563312135, -4.4816895664771188])
      res=quotient(arg0,arg1)
      ref=numpy.array([[-4.4600227802978551, 6.1512839524980381], [0.056780350596017597, 1.0709517116678569]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_array_rank2(self):
      arg0=numpy.array([-3.6534601402637099, 2.0101881413322706])
      arg1=numpy.array([[4.0835346338863054, 0.0050703626121411105], [2.9510827933680694, 3.3366869487768014]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[-0.89468082624946588, -720.55204326321905], [0.68116968654682974, 0.60245032638413598]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_array_rank1(self):
      arg0=numpy.array([[[0.9173012784132526, -3.6353393853599281, 3.8448919873067791, -4.5759608568416263, 1.2019517514457654], [4.5656681039957654, -0.0048300792701692075, 2.4234350043852491, 4.9398578851588972, -3.6367028441925173], [-0.17293608172341379, -0.96838556498389572, 2.7248760979921327, 1.3790737528615242, 1.8529848374170035], [-2.5929276667142487, 3.8824888019579795, -1.4287376814070218, -2.0327352862134029, -3.0576704262803247]], [[-0.18350869759204969, -0.28802438106523098, 2.3949860855355638, 3.104456284099923, -2.4802300817114942], [-4.7103451331929058, 3.5150485286739421, 2.5073958966137635, 2.6249655146942406, 3.3767124465716787], [-3.3893178137182831, -4.6233791338344066, 0.60051737711758335, 4.4028101274223648, 4.3242385353735866], [-3.6744712672105218, 3.1656987258027822, -0.1929462625590741, -4.0158128372424251, -1.2564984110263056]]])
      arg1=numpy.array([-4.3562032727018423, 1.9291481873051577])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[-0.21057357083438305, 0.83452014467295188, -0.88262455781179994, 1.0504470453701955, -0.27591727846535508], [-1.0480842647097153, 0.0011087818836271737, -0.55631816347315788, -1.1339824098922398, 0.83483313714534946], [0.03969880900809155, 0.22230036211861051, -0.62551628732928399, -0.31657699756654906, -0.42536693570493767], [0.59522650904810526, -0.89125519607581316, 0.32797773473065633, 0.4666300351389806, 0.70191178759752182]], [[-0.095124210156397798, -0.14930132530024792, 1.2414733618163043, 1.6092368147397544, -1.2856607377456819], [-2.4416709738471796, 1.8220728463499432, 1.2997425045487898, 1.3606863028812086, 1.7503644711133544], [-1.7568986333044989, -2.3965909743267786, 0.31128628742432213, 2.2822560529021283, 2.2415274077074137], [-1.9047117745492739, 1.6409826609665334, -0.10001629933292074, -2.0816507843558383, -0.65132290992197861]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_array_rank3(self):
      arg0=numpy.array([-3.3296070622697105, -2.1715867869097858])
      arg1=numpy.array([[[3.6042083485484149, -1.2882987497076592, -3.4373618842831788, -4.6925458242829388, -4.8367237508131726], [-4.6062325208449835, -3.6945745373137262, -3.5885048456746405, 4.9344990155014461, -1.9387851785785681], [0.034601031739373234, 1.9087066908427266, 0.57709389439786207, -4.5396276669878723, 2.9796758502805982], [-4.4644928082153896, 0.49143328887475235, 2.2357708464505865, 4.4447548496169862, -3.1427858573078171]], [[2.6369133483918858, -4.1374931981035772, -3.0965212848179089, -1.4655929422738301, -0.88610605234149098], [0.65804781409495838, 1.1850865598562876, 2.6957557348726962, -2.6738441845273888, 0.58091990678134486], [2.5149725157584415, -1.8925067255579284, -3.2659818160152385, -0.87713970267718722, -4.6319536135550106], [-0.94601840870581633, -2.5396750095250553, -1.529056067051199, 1.2335137250267261, 4.4001931685131215]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[-0.92381092885784488, 2.584499180042878, 0.96865188314731687, 0.70955238093567319, 0.68840132986919533], [0.72284823816469335, 0.90121528978289922, 0.92785357842919203, -0.67476091327811405, 1.7173677099753939], [-96.228548540096895, -1.7444309690137001, -5.7696106207185087, 0.73345377782468379, -1.1174393556789606], [0.74579738512350036, -6.7752981689408927, -1.4892434381437845, -0.74910927034741404, 1.0594444589749836]], [[-0.82353361676981984, 0.52485567538942035, 0.70129884059152725, 1.4817120936326456, 2.4507075436077614], [-3.3000440703483882, -1.8324288372430184, -0.80555769902213947, 0.81215906277411642, -3.738186213899465], [-0.86346342685772826, 1.1474658227539889, 0.66491086271854904, 2.4757593120932899, 0.46882740374489618], [2.2955016170146059, 0.85506483261253741, 1.420213969719053, -1.760488548161663, -0.49352078505307784]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_array_rank1(self):
      arg0=numpy.array([[[[-0.68259869128854689, 0.54590444598251686], [-3.9545894233755083, -3.7192887442044809]], [[-0.17896275809695084, 4.5665467895196148], [3.8889888059695981, -0.98246344667720109]], [[-2.8879317433748177, -0.20209488145670473], [-2.7092353964855889, -2.7103872703237442]], [[-4.3267103951476296, -0.77184567896089629], [-1.3001696526347861, -1.7711491445395455]], [[-3.2024982345040587, -3.4066266553320323], [-3.7105147132506144, 1.3842890016660121]], [[-1.3890931595628766, -1.2874698896019021], [1.3314090056270995, 1.9146723114024811]]], [[[-1.9390529937934575, 2.1755821206377179], [2.8987780396840037, -1.636371148645793]], [[4.3811329045571661, -1.2106158991655303], [3.7865656139055108, -2.3601302240007715]], [[4.947322083660211, -2.5907159869099248], [-0.68525848353197816, 2.4310565664575261]], [[4.5257163283708763, -2.3046240120797714], [-1.2955625378678914, -1.5718949916965719]], [[3.5380226922270435, -3.2396985873627751], [3.0263107597318406, -0.88531474592517867]], [[0.45596801921594121, -0.79891114608978775], [-3.9966250296491515, 4.8947584284692738]]]])
      arg1=numpy.array([3.2955663220622622, 1.6812874062844676])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[-0.20712637057821309, 0.16564814439568218], [-1.1999726410909703, -1.1285734774340899]], [[-0.054304098478880423, 1.3856637503996621], [1.1800669220141777, -0.29811672734366523]], [[-0.87630818534631716, -0.061323263350452038], [-0.82208492614714979, -0.82243444842210567]], [[-1.31288827846745, -0.234207296571079], [-0.39452085789649066, -0.53743392529609779]], [[-0.97175960716215704, -1.0336999235992532], [-1.1259111031723041, 0.42004586355882145]], [[-0.42150362754454462, -0.39066726740800162], [0.40400006418136519, 0.5809843056668752]]], [[[-1.1533144104603952, 1.2939977498824002], [1.7241418860622459, -0.97328460471970313]], [[2.605820330409288, -0.72005291578369113], [2.2521822264008788, -1.4037636963072846]], [[2.9425796358003184, -1.5409120280245443], [-0.40757962081352495, 1.4459494298062923]], [[2.6918159925865415, -1.3707495836020303], [-0.77057767340980554, -0.93493532742885066]], [[2.1043532943875647, -1.9269153954601324], [1.7999960913403761, -0.52656954582302196]], [[0.27120170978000718, -0.47517821349493589], [-2.3771218500240985, 2.9113157037715292]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_array_rank4(self):
      arg0=numpy.array([-4.560787113316791, 1.0955334540628412])
      arg1=numpy.array([[[[0.46751265443367185, -2.5249180902822199], [-2.1351916408141869, 2.5286823367264502]], [[1.9742631293208399, 2.7843167842447372], [-4.2462714025973192, 2.4057720035161161]], [[4.7165641185027098, 3.1058665325584602], [-1.8634221559930566, -3.3675931372111956]], [[1.7426297602642489, -3.9860256584124953], [-1.4081302996283163, 2.5482025262786205]], [[-0.33477199698726245, 2.0186783888092172], [4.6170267517075203, -3.3223597782144076]], [[-4.3859130663994863, -4.5008968352964995], [-0.6174836804748951, -3.5698597912853658]]], [[[-0.85453793987609217, 1.30645980084818], [-1.8271765372371949, 4.1379222236186433]], [[-3.6272035949111681, 3.7956140702269696], [2.0622470691508976, 3.7426116902069406]], [[3.75171986916639, 2.3827986845491296], [2.9251528684881904, 3.3719438058383986]], [[2.9540191471259813, 3.8871014828836365], [-1.4472186695698062, 0.50025521861427968]], [[-1.5527294046959916, -4.2187601117916529], [-1.2131941893028833, -2.5307579533239002]], [[-0.61620001470710051, 0.82943764116427765], [4.1283155369315043, -0.13293719917433666]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[-9.7554302970506033, 1.8063109179145744], [2.1360083217531147, -1.8036220078244536]], [[-2.3101212019725725, -1.6380273750186556], [1.0740686783532234, -1.8957686375313407]], [[-0.96697235502962464, -1.4684427246008664], [2.4475329428968027, 1.3543165481961146]], [[-2.6171865173616697, 1.1441941181917039], [3.2388956579661952, -1.7898055850283363]], [[13.623562168762644, -2.2592935747467524], [-0.98781907894080745, 1.3727553358980202]], [[1.0398717540156042, 1.0133062987693087], [7.3860852643250023, 1.2775815802207267]]], [[[-1.2820185072434507, 0.83855121554570511], [-0.599577233910499, 0.26475448180482936]], [[-0.30203252323631186, 0.28863141346647253], [0.53123288205904073, 0.29271897400669578]], [[0.29200833011721161, 0.45976752512356567], [0.37452177828540184, 0.32489671155431615]], [[0.37086200173370759, 0.28183814054942613], [-0.75699234476327937, 2.1899490765883427]], [[-0.70555336348340447, -0.25968138150371917], [-0.90301574448880984, -0.43288748836053892]], [[-1.7778861212516899, 1.3208147299959117], [0.26537057166834915, -8.2409849227087708]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_array_rank2(self):
      arg0=numpy.array([[3.1126950084573544, -4.4935477824775898, -4.2670360505833349, -2.1731744986806278, -2.1626791173784845], [1.9281953952138728, -3.2411219699570051, -3.0278869126493335, 1.8164488152449607, -1.3882421607405826], [4.8671050066607275, 1.4311354552384135, 2.3594674345658149, 0.27904925526930313, 4.3287344137453783], [-2.0673529146532488, -1.5647915806808141, 3.6615033392204062, -0.042496735285393328, -4.6610318506722193]])
      arg1=numpy.array([[-4.9330898394139444, 2.1347463502482338, 2.9763526414486252, 2.6150671702018542, 2.0294807940467976], [4.5876895581210881, 0.17910052158311274, 4.4064100943576765, -4.0809186844502499, -1.2650333765579003], [1.7188294089430656, 2.0545930981811553, 2.6800084611701038, 2.3374991347987244, 0.068322804308462715], [-2.6944389736030763, 2.5395789041866639, 1.0244246934352654, -0.33686505469064709, -3.1610386302789637]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[-0.63098283424474289, -2.1049563016960158, -1.4336459971713966, -0.83102052729027331, -1.0656317239968005], [0.42029770558485113, -18.096664048255949, -0.68715504181657638, -0.44510781902277935, 1.0973956786167398], [2.8316393595182809, 0.69655420165936377, 0.88039551693641327, 0.11937940473006048, 63.357095153794866], [0.76726655712254965, -0.61616182828623745, 3.5742044902705978, 0.1261535878941771, 1.4745254316176706]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_array_rank2(self):
      arg0=numpy.array([[[4.6623159650797223, 1.1152427053702576], [-2.4242616040299536, 4.1049782397106842], [-1.2363119457685223, 3.1081214265831267], [-1.40284542173849, 0.29454285824049009], [2.5574230200683754, -1.2952299935670863]], [[-4.2936786920816052, 0.67417286945159471], [4.5084915679002631, -4.0824848191696939], [-1.2564628191713401, -3.9348894775470908], [3.7973683408075516, -1.4144400002611048], [-3.7519807851950593, -0.99287239243330916]], [[0.13539998400403697, 0.92721374579818594], [3.6267860514930916, -4.4927392733992066], [-4.6300713980013377, -1.7761367154728647], [2.5116997485449657, -3.7519945484086215], [-1.4461408932742703, 0.43906777155003951]], [[-1.5373235679076922, -4.1318455019218217], [-1.5794326574851514, 2.4667193300555006], [0.69753303534778865, 2.8564781957491157], [-1.9228510423841771, 2.631895398282027], [3.9997782694721415, 0.71266153745780247]]])
      arg1=numpy.array([[0.061308352116094866, 1.7798968848448196, -0.82206726327795021, 4.30649875376427, -3.0450327684177383], [-0.23400370446441077, -3.9242996343373173, -4.9220063181575044, 3.0928598440628328, 0.65617897666366964], [1.4616374943518089, -4.6141000254775424, -1.0999622076581228, 4.7563426455002684, 4.7962132115609197], [1.3281335480105296, -4.4251056769674086, -1.1533302351381258, -3.6745033303242414, -1.1397415139284628]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[76.046995297656295, 18.190714101374134], [-1.3620236232063034, 2.3063011541079117], [1.5039060682684202, -3.7808602354382232], [-0.32575080174173415, 0.068394971200916505], [-0.83986715893282982, 0.42535831042636513]], [[18.348763759569557, -2.881035028888308], [-1.1488652722772013, 1.0403091505674715], [0.25527452383313526, 0.79944827844513433], [1.2277854582053302, -0.45732431198792139], [-5.7179228817600025, -1.5131121656496087]], [[0.092635817381027621, 0.63436642079941763], [-0.78602241639044934, 0.97369784976306939], [4.2093004339294549, 1.6147252179275804], [0.52807376081728596, -0.78884025564436377], [-0.30151722400256392, 0.09154467330428491]], [[-1.1575067659502447, -3.1110165902450828], [0.35692540987350169, -0.5574373834493328], [-0.60479905416183799, -2.4767218518354515], [0.5232954958880125, -0.71625881423538851], [-3.5093731522384402, -0.62528347765573578]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_array_rank3(self):
      arg0=numpy.array([[-0.19128129052136345, -2.1509707193186345, -0.9509423847887728, -3.2953180462316278, 4.4858179086329901], [-3.3563146574324265, -3.4081172782902769, 3.94894190837568, -3.6753021377550521, 2.8938464141442379], [-0.13976181441182334, 2.4539469349163614, -0.30166107848493606, -4.4582872792889203, 4.3113433443052553], [-4.0795988420915421, 0.21139869424117208, -2.8978215821651898, -3.4729710491802877, -3.3084397560998635]])
      arg1=numpy.array([[[-2.5845049170223833, -3.9863997379328451], [3.8204474468637724, 0.84836791229553121], [0.34877242245394235, -3.0811586848580874], [-0.32506666759482883, 2.4775657961387223], [0.80169532971001622, 2.6260655766586307]], [[2.198161207758079, -1.9961109805769839], [4.6711269423511705, -3.2964837509931044], [1.8532390861336099, -1.0338715445568756], [-3.325973355166786, 4.3477266619202464], [-0.76462802816588038, 4.3616080879574799]], [[-1.5918466457884115, -0.67268503646787003], [-1.2946881413827338, 4.4252791365737085], [3.1349393294803534, -1.7173587714167504], [-4.1729521155429747, 1.8897768331774234], [-2.2857741223958761, 4.7692782609287114]], [[-3.3369832772747143, -3.9118489543664059], [-4.0326103515500886, 1.8324380667902274], [2.5680697251254436, -4.3510842145295907], [2.2759568132214909, 1.0735518029633218], [-3.9971051056789642, -3.3394637979291475]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[0.074010805420226963, 0.047983469570603753], [-0.56301539263009071, -2.535422059396959], [-2.7265412158965949, 0.30863142150452777], [10.137360654704205, -1.3300627782993169], [5.5954147946147694, 1.7081896006346813]], [[-1.5268737550216149, 1.6814268796128111], [-0.72961350020062421, 1.0338644251662219], [2.1308324100881713, -3.8195672655525339], [1.1050305415242114, -0.84533882268757743], [-3.784646007661701, 0.66348153153288292]], [[0.087798541889443096, 0.20776709282204894], [-1.8953961625813094, 0.55452929841988241], [-0.096225491717869802, 0.17565408201576777], [1.0683772916260312, -2.3591607225880042], [-1.8861633361157495, 0.90398234458765181]], [[1.2225409908027216, 1.0428825063753788], [-0.052422296183392197, 0.11536471440558238], [-1.1284045576385737, 0.66599988400327537], [-1.5259389057846353, -3.2350288449927209], [0.82770897152525058, 0.99070987328908233]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[-4.3698508540570122, 2.6228132159481534, 1.6459301268402982, 1.6188715259874851, -4.0683439590246762], [2.4728070625496716, 0.23687557395272485, -0.21700866694764365, -2.0166821250433875, -2.9284968776538065], [-0.23008910966974128, 3.6560025766014519, 1.7719010165441436, 4.7628791995401532, 3.5750022200025917], [-1.7789793031784984, 3.2843095356473917, -4.0405467723495834, -4.8477177164975274, 4.7552624283307168]], [[-1.7370051899199579, -2.3364466392936691, 3.7725959614648445, -0.57376982018192102, -0.67164249439189838], [4.2786149757711449, -3.9081345162170611, 0.87712310638003022, 2.6608593685970572, 2.8249483526262527], [4.6038723534180228, 2.4659437353288087, -4.1976842566650756, 0.82446744832397556, 4.6422946155003437], [-0.5466698097665148, 2.5453989864420477, 4.9630985390675715, -1.3392469405218366, -3.7204577346047563]], [[-3.1239877398687388, -2.2584685874052002, 3.3769932353110246, 0.61937092015698436, -3.3585692411901413], [2.3191980021373535, -4.2231757472163043, -2.3492023226592362, -3.453439723295022, -0.033688936366815803], [-0.050511848405308868, -2.1919691038858558, 0.94460540121935743, 3.6876396475824365, 0.038627014069751553], [0.090901734795963307, -1.0164071438268465, 0.24678560976640984, 1.9092345864720883, 4.5033001428228552]], [[-2.201814968088267, -3.2238876758610679, 0.14912896812283449, 4.0209387839636967, 3.6617441363601504], [2.8252360058760022, 1.6105171174417467, 2.3315405661050539, -1.3352074013324411, -0.28863410653614885], [-3.6910748164729377, 2.0799129709259709, 1.212267738155969, -3.8323851096541892, 2.7376992255596431], [-1.857736523700436, 2.3167114903176405, -3.2251170462542724, -0.69038079399411334, -3.4327780190003834]], [[3.4719524572613398, 0.88255452565919956, 3.8624034183491389, 4.5321320513386212, 1.0220938463487759], [0.82623109171749842, 1.9114713359666267, 4.0728135336558751, -2.7124988939133141, -4.2353067304841074], [1.4374708157226559, 1.801191500599173, -2.5734209859863255, 2.8221798559892095, 4.2935113201901114], [-3.5412471614855368, 4.6094039472680279, 1.5853437147854468, -0.12008617041919223, 0.70499731489981698]]], [[[1.5187160733549092, 4.4937042415948767, -0.52967872988841691, -1.7212268190118305, -1.4302729316131524], [-3.2745416713536937, 1.0199164612173846, -4.9308450912166322, -0.25195859001510001, 4.5739512993423794], [3.9440253040353781, -2.1987874878957614, 2.0518022455100517, -0.19053114726777665, 4.0524698193667454], [2.9961631621890543, -4.0957949764778494, -0.92386759187982026, -3.8946443645727582, -3.9625372987276508]], [[2.8469515585120941, 4.047023856421033, 3.7526360518622219, -4.6284664780859428, -2.3475807304064067], [-0.088014903070447126, -0.31860695618365042, -2.6721064628153037, 0.56228673722305267, 4.4128681648130534], [-2.2700658640060825, 0.050746920159642883, -3.2890514574048879, 4.9134899529341922, -2.6165270789800998], [-3.8218464243013606, -4.8195443998835943, 1.4498574147776901, 0.65768397535433731, -2.5205585927600662]], [[-1.4131508999944109, 3.7810246960922473, 0.32709987535875129, -1.565149554533305, 3.9565796608944357], [-0.85885083329319833, -2.6777641827827248, 2.9455631054597253, 2.3151416623198902, 1.19271990944537], [0.65105849927113102, 2.672904195907555, 2.6985443061925523, 4.09103688344217, -3.8864690385754033], [-2.5661662720917344, -1.4721648860722238, -1.8319435406535334, -2.3458042810088617, 4.225329703054868]], [[-1.159348946395069, 2.6138216197032786, -4.4789711461464634, -2.1386849554238574, 0.36174949149941327], [-2.8491810672083884, 0.82759372324896496, -4.3474445835844815, -4.4499100236842679, 4.713080425838287], [2.4614761140005816, 4.4395015696400293, -1.922374612050354, 3.7571900316879017, -4.9396512345241295], [1.1103327208534193, 2.5425813187854329, -3.0345041867598663, -2.4633191774522523, -3.3981109139184573]], [[3.708029412015275, 4.8615222974707812, 4.460570518031, -2.1961803845165062, -1.0788726291499051], [-0.37321039496161479, -1.7036866101960646, 0.44400821912887967, -2.1701312716328358, 3.3389245339748523], [2.336020578627668, -2.0785766723058052, 1.8161215809425366, -2.2744868281282171, -1.3005817560192092], [-2.0638881157337696, 4.2175578358591004, -3.6312216513606099, 2.0289090717975071, -2.8214846424879769]]], [[[-0.24780425786178117, 1.2470814633614635, -3.1832194834868721, 2.1187558946550293, -1.9139815258437567], [-2.3348385966164473, -3.5089775528396028, -1.7853757273965032, -3.3743184803248938, 2.0712668217848149], [-2.8337283990950892, -2.5733544699606394, -0.71147030125700805, 2.4320166260354235, 2.5933916922741931], [-0.12187975369667736, 0.96490488822125364, -3.9945854204311768, 4.1048351780461658, 2.3520046812169255]], [[-3.7406159102169778, 2.7985157202701956, -2.456877250010888, 0.32291702830320101, 4.7260430252603332], [4.9058694011002615, -3.9598865665014849, -2.904843785878902, 0.49968201908879983, -0.033545949391694307], [-1.2698094232349577, 2.5411693940745375, -2.3012888396222819, -2.6155889117700837, -2.7814890391686911], [-0.025469489111586974, -1.6258698232980837, -3.6462197318048064, 2.8832185343228733, 1.1274869408252037]], [[2.1969023486880515, -2.5533526642993665, -2.9346210117679283, -4.0941718682555006, -2.8542999759200494], [-4.6071266685761856, 3.1791562767105486, 4.3948306381933406, 3.1553825143565408, -3.4002020456408411], [0.19875878110598144, -1.4735159612811, -0.68121374930200229, 4.296535987525413, 4.1391258643592188], [-1.9799423665607585, -4.7339831581421743, -1.3414883705950418, -2.2584679306002098, -3.1137939146035731]], [[4.3459509214088143, -4.9238340643762823, 1.9353038896866934, -0.55275167637366707, -0.61770889353602509], [0.92303756173510365, 4.0881610959404817, -0.24095143660186835, -1.5479846338443783, -1.9204916173215372], [3.6073999870795213, 4.8405235517124723, 2.5886872831143002, 2.781209697646565, 1.1255817004317228], [1.4945642639441168, 2.1426013293582606, -4.7793318453801863, 0.54498053041928696, 0.0030475931797777633]], [[4.4103343927312562, 4.6445482437409797, 2.4952036934139841, 2.2037386769776921, 1.2572093089803316], [2.0983659491391524, -1.6850283517873796, -3.8111111784176401, -0.78871697359351245, -3.6712167246088976], [-0.28137117663334621, 3.5592601142535347, 0.53395182793409024, 0.78468828871821472, -0.88993233637378388], [4.3327248006394541, -2.2312518547996705, -1.2553914489477291, -3.1799281045835626, 0.82033468490147232]]], [[[2.6276240738848227, -4.7447886988866887, -3.7224583951256776, 0.44107491693841983, -0.10721550775338251], [4.3752173662389833, 4.8370503650921215, 0.95259100146704334, 1.7969155058558117, -3.9533757894233368], [0.62528484951705998, -0.77527642927370088, -0.12666573017059868, -1.3265307956052377, -0.49583239824166458], [-1.797646328197291, 4.4874091925881441, 3.3307092057604599, -2.0375925053849651, 1.9831327579568301]], [[4.9417236831693572, 2.8168805080460659, -3.8753109932803866, 0.35527814875949471, 0.37928454625320995], [-3.8154399155112118, -2.1699412705556709, -3.0264790463857283, 1.9291365856161669, 0.48657282294151116], [0.72623762606012754, -0.060428821721808745, -1.3470785171329713, 4.8141843509845526, -0.23902035613921058], [3.4277300452688415, 4.1816860012941355, -0.52969180196967613, -2.4823682563869056, 1.0526407153936645]], [[0.98268792632919677, 4.247941496246268, 4.573805025693666, 1.0919111426877457, -3.884217070242145], [4.8721792457224673, 3.7267632826828532, -2.5512198749174719, -0.091438054262279245, -0.83822320013539642], [2.6705779018331013, -3.6577051821369242, -0.22617974490551518, -0.38705154754142157, -4.2275213923776631], [4.9243653835121677, -2.9909298483712954, 0.046134966959836987, -0.25350424660759874, 1.5762417724361191]], [[-3.4032501902657697, 2.9999193246665978, 0.1485872877305372, 1.9638368778465836, -2.4223644550576982], [-2.5718949122014512, -1.3807530568082926, 4.9915586182186438, -4.3675290772208637, -1.9121694888026122], [-4.2986089769083735, -2.9075814947111289, 0.24454608686821899, -2.1527980554902646, 1.7330284311879227], [3.1126485111371807, -1.2802861638220966, 4.00944037482064, 1.7084230875541406, -1.8008037440882232]], [[4.4772469486722368, 1.6442398005463339, -2.8411308445038133, -3.0061535112416027, 0.086802814296639497], [4.51780040989998, -1.1988718912254925, -4.5739329938444868, 3.7170898318028378, -2.6228918114177482], [-0.064962949756512955, -1.0920136536558092, 3.3826023373657002, -4.6756852748610065, -0.95426256153510725], [-4.8880936150380352, 4.5327308438502811, -3.4746214960663524, -3.7024542113218937, -3.1089587367231442]]]])
      arg1=numpy.array([[1.6144289894192667, 1.1060158715420769, -4.5337514112698631, -1.7877804484156865, 1.5867532295993723], [1.0499612793660358, -4.8665413743666726, -4.2010463435396508, 4.7174660738110745, 1.9558546638237448], [2.7580196212180885, 3.0261659029817007, -1.8237839615390206, -0.8518258587392129, -4.49298453711239], [3.4154622798448866, -4.7360288206799446, -3.9825413870628279, -4.3714603258253693, -2.8132361305825428]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[-2.7067470187269804, 1.6246073584763967, 1.0195122471334976, 1.0027517695713679, -2.5199894115430359], [1.5316914393609693, 0.1467240587880749, -0.13441821744399224, -1.2491612441677085, -1.8139521136245385], [-0.14252042745621635, 2.2645793655604316, 1.0975403862027537, 2.9501942982660569, 2.2144066065665555], [-1.1019247763993774, 2.034347473423904, -2.5027714435448947, -3.0027444677150656, 2.9454763631575105]], [[-1.5705065674131022, -2.1124892502998609, 3.410978141032329, -0.51877177800526053, -0.60726298028205705], [3.8684932882614338, -3.5335248044570049, 0.79304748597964514, 2.4058057728295705, 2.5541661971699665], [4.1625734963450514, 2.2295735520419204, -3.7953200895864181, 0.74543907509613871, 4.1973128369557342], [-0.49426940772948708, 2.3014127120012233, 4.4873664716471993, -1.2108749747456826, -3.3638375635762441]], [[0.68905139618004285, 0.49814565963876334, -0.74485628544092564, -0.13661333936778508, 0.7407925438614722], [-0.51154061874066603, 0.93149697990023472, 0.51815860852442408, 0.7617179262873931, 0.0074306977403024027], [0.011141292016970325, 0.48347800861712992, -0.2083496238614419, -0.8133749103260961, -0.0085198791388812548], [-0.020050004190790546, 0.22418678300276723, -0.054432982177398961, -0.42111585159392956, -0.99328342785373869]], [[1.2315913679666279, 1.8032905990878498, -0.083415705913436472, -2.2491233683235619, -2.0482068363624584], [-1.5803036711692975, -0.90084725944339039, -1.3041537444775404, 0.74685199880985875, 0.16144829573001179], [2.0646130344159048, -1.1634051445014792, -0.67808535395398739, 2.1436553426067566, -1.5313397279770933], [1.0391301266029305, -1.2958590594111752, 1.8039782508597964, 0.38616643033876002, 1.9201339974617564]], [[2.1880859559604917, 0.55620149951232767, 2.4341550697989307, 2.8562299208194499, 0.6441416518224683], [0.52070547348191465, 1.2046431041134482, 2.5667592525929126, -1.7094648640470536, -2.6691653443512759], [0.90591957773143617, 1.1351427978841693, -1.6218155022353853, 1.7785877497169242, 2.7058469080753969], [-2.2317567063529093, 2.9049280387674541, 0.99911169878993633, -0.075680432331315883, 0.44430179926453756]]], [[[1.4464496007623315, 4.279876153440787, -0.50447453663075625, -1.6393240901713093, -1.362214930894164], [-3.1187261241965554, 0.97138483224182115, -4.6962161254116586, -0.23996941122174717, 4.3563047411654274], [3.7563530975321027, -2.0941605477331358, 1.9541694401806182, -0.18146492733790995, 3.8596373971177464], [2.8535939572915781, -3.9009009731776785, -0.87990634515365118, -3.7093218970173218, -3.773984218846834]], [[-0.58500510722208621, -0.83160165404896191, -0.77110945190527358, 0.95107924129963606, 0.48239202131758691], [0.018085719672300393, 0.065468868273027614, 0.54907710779774255, -0.11554134527341364, -0.90677707746548031], [0.46646389897414708, -0.010427717809395392, 0.67584989099839354, -1.0096472165663297, 0.53765639243550301], [0.78533112744751554, 0.99034283881143526, -0.29792357718655443, -0.13514402216295299, 0.51793633277968165]], [[0.33638069767250905, -0.90001975386600752, -0.077861525108325433, 0.37256183972837731, -0.94180814429215831], [0.20443736228093154, 0.6374041045513763, -0.70114987186213695, -0.55108691335436089, -0.28391020043840481], [-0.15497531948732859, -0.63624725302493612, -0.64235052068453402, -0.97381379515923383, 0.92511929666093706], [0.61083979138625133, 0.35042814710580666, 0.43606839602488257, 0.55838571850468355, -1.0057803122197306]], [[-0.24575671096633281, 0.55407322041251361, -0.94944427285050093, -0.45335460222951623, 0.076683008598124075], [-0.60396429409965746, 0.17543183359459355, -0.92156350794322395, -0.94328394821700168, 0.99907033820611146], [0.52177929326623473, 0.94107758279086307, -0.40750152348151075, 0.79644240634731267, -1.0470984119942084], [0.23536633936116907, 0.53897182915644604, -0.64324875670136983, -0.52216998255214231, -0.72032545878453835]], [[1.8958614260049285, 2.4856255361870208, 2.2806247317530604, -1.1228750403278527, -0.55161186007588214], [-0.19081703864027358, -0.8710701473417839, 0.22701493487293808, -1.1095565083502548, 1.7071434783641701], [1.1943732946192889, -1.0627459753283179, 0.928556510120222, -1.1629119843094775, -0.66496850715714184], [-1.055235930311313, 2.1563758871600762, -1.8565907367889394, 1.0373516546627954, -1.4425839990441334]]], [[[-0.089848620348950817, 0.45216555160354005, -1.1541685414409752, 0.76821639641536477, -0.6939695102670953], [-0.84656344670429073, -1.2722815769127309, -0.64733974829663687, -1.2234570248759198, 0.75099785579844169], [-1.0274504130770337, -0.9330442938705813, -0.25796419132898873, 0.88179815956541863, 0.940309369927112], [-0.044191039381673708, 0.34985425078125448, -1.4483527925979565, 1.4883270396144794, 0.85278750851603979]], [[-1.2360908258636201, 0.92477273553072559, -0.81187791045762259, 0.10670830306594518, 1.5617263483815387], [1.6211501809158833, -1.3085490661962, -0.95990896699243777, 0.16512049739125667, -0.011085297524052223], [-0.41960998304283526, 0.83973234632334826, -0.76046354145845318, -0.86432436146112379, -0.91914624919541654], [-0.0084164219438503766, -0.53727055139181357, -1.2048975002369047, 0.95276287776622548, 0.37257935518812224]], [[-1.2045847507257224, 1.400030221860648, 1.6090836818695979, 2.2448776579878396, 1.5650428099561837], [2.5261361903239954, -1.7431649492233618, -2.4097320356325063, -1.7301295443424318, 1.86436667793237], [-0.10898153799875321, 0.80794435764072459, 0.37351669039086866, -2.3558360409638284, -2.2695264086357962], [1.0856233020549002, 2.5956929427909596, 0.73555223583774243, 1.2383418092427878, 1.7073260760424513]], [[-5.101924151306295, 5.780329411065364, -2.2719478046265653, 0.64890220307680557, 0.72515865443471716], [-1.0835988979030187, -4.7992920783026793, 0.28286466550628137, 1.8172548038580911, 2.2545589542961939], [-4.2349031202443568, -5.6825271292854733, -3.0389864977165817, -3.2649979677337249, -1.3213753596277247], [-1.7545420212485927, -2.5153043986355663, 5.6106911950924729, -0.6397792751043182, -0.0035777185542224615]], [[-0.98160462300761608, -1.0337334138091645, -0.5553555042986269, -0.49048436707819604, -0.27981607739703712], [-0.46703164273246256, 0.37503542197150219, 0.84823598811381951, 0.17554411039669757, 0.81709979063679639], [0.062624559312234249, -0.79218169678835493, -0.11884123426724665, -0.17464744920367084, 0.19807153330328112], [-0.96433111773495361, 0.49660795321447526, 0.27941147773336439, 0.70775407266976265, -0.18258123929104278]]], [[[0.76933189670715862, -1.3892083443246732, -1.0898842060392284, 0.12914062015594893, -0.031391214122338869], [1.2810029822486237, 1.4162212809774601, 0.2789054375123432, 0.52611194580003351, -1.1574936174094945], [0.18307473433594978, -0.22699018924867473, -0.037085969567888538, -0.38838982454389226, -0.14517285146659059], [-0.52632592044873383, 1.3138511934589248, 0.97518547501327524, -0.59657883426471403, 0.58063377530461091]], [[-1.0434319279458864, -0.59477689319501448, 0.8182616998356893, -0.075016044498751155, -0.080084932042021786], [0.80562007960150772, 0.45817737871032121, 0.63903307200550807, -0.40733210431333555, -0.10273856882307879], [-0.15334316017862887, 0.012759386399412381, 0.2844320776197416, -1.0165023341841462, 0.050468518074789664], [-0.72375616261066744, -0.88295197508822953, 0.1118430275712784, 0.52414551312433011, -0.22226231200226068]], [[-0.24674895520770493, -1.066640891679264, -1.1484639031126058, -0.27417446212481, 0.97531116258073636], [-1.2233844603723649, -0.93577515472636075, 0.64060096982420245, 0.022959724802688344, 0.21047444801410992], [-0.67057128659312804, 0.91843494558998817, 0.056792817179566198, 0.097187074765562376, 1.0615134863659286], [-1.2364881880471672, 0.75101036189786896, -0.011584303206416163, 0.063653888803541392, -0.39578792013473996]], [[0.77851563015688741, -0.68625106968120253, -0.033990309108543185, -0.44924046690868558, 0.55413163439847413], [0.58833769964865357, 0.3158562480027986, -1.1418515201270172, 0.99910070129625084, 0.43742121540164686], [0.98333477980193251, 0.66512819012309177, -0.05594150893318392, 0.49246656609740541, -0.39644153258115455], [-0.71203860475377545, 0.29287379237060046, -0.91718558009871065, -0.39081290008770142, 0.41194557650439523]], [[-1.5914934761431219, -0.58446562045463901, 1.0099155252621805, 1.0685748979838077, -0.030855146979313481], [-1.6059087116034121, 0.42615402176611444, 1.6258617412600032, -1.3212861129553075, 0.93233972893509665], [0.023091893727051251, 0.38816992352138019, -1.2023883457892521, 1.6620308633292018, 0.33920457339551729], [1.7375340668704717, -1.6112159212570893, 1.2350977076875715, 1.3160836984399238, 1.1051183023443416]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_array_rank4(self):
      arg0=numpy.array([[3.8096386339595139, 1.1181339867051046, 0.050149361374446144, 4.2891992598299336, 3.4939924892024834], [2.5429703612299779, 2.2677184386958533, -0.8953956778387262, -1.9185806298460428, -3.0966091407654828], [4.4519449385835799, 0.20632216003435389, -2.6965706506370246, 4.8510570639808979, 0.8570511555496152], [-2.8599751393014894, 0.5656324530002923, 2.8612240236898687, -2.0884763665737571, 2.593651362065077]])
      arg1=numpy.array([[[[-2.4302904396901548, -4.3628081971842017, -4.5557186172936159, 1.5743628361800868, -2.8829351106717094], [1.7683375634231489, 2.3090655520747747, 4.6591346412000476, -1.0412476717462829, -0.11072204532779395], [-1.1244739303384641, 0.86568809611888398, -3.0183308201446737, -1.9575104189981953, 2.0082310603023785], [-2.0002623832112754, 0.38152711022667773, 1.2769717976338466, -0.095766206059493264, -3.2044616610760004]], [[2.0498214689050123, 2.9831362479097958, 4.0783362736375466, 2.0908785207754725, 4.6115160936454185], [2.2421696595710578, -4.2181587243051641, 4.0384654234639665, -0.97500514627684076, 2.7395156243747554], [2.9390824755231506, -0.046567240558458067, 3.6683036401552087, -0.97369838544503828, 4.5355047465110481], [0.31804534727410516, -4.5080397300479236, -3.6695787238044608, 0.73198100415952982, -4.0123514707711214]], [[-3.3935587935290181, 3.2753713280569308, 3.8042857538815316, -3.360142761985021, 3.8778788732152698], [3.6451226637720051, -2.283300882569236, 0.73846669894430583, 0.67587694431219258, -3.2832849619347604], [3.0904238338895738, -0.58588336664559737, 1.9388034633268258, -1.9896111042216749, 3.2657065024970695], [-3.795731903281756, 1.3115648162029778, 0.51129246710270237, 4.3817089570370129, -1.9549031913808523]], [[-4.1704671677738805, 4.3940867552554543, 4.6658928742509982, 4.5275442304443967, -0.065067480213373941], [3.7765564743067603, 0.84173755645480597, 2.4854197238728197, 4.5374068952208919, -1.1292392722611764], [-2.7437507805442052, -4.0595450329827898, 2.3993116262540912, -4.0458508441651286, 0.72044063079340148], [-4.944243473273918, -0.074762546406768493, 1.7189186540641401, 1.8582039540676822, 3.6800461457371476]], [[-1.3323372672416491, 1.2183467506126835, -4.4055525793188908, 3.1264341982215349, 2.9766435941712475], [-4.6486713982129082, -3.8344023107563685, -3.5913823685511446, -2.0864908447886732, -1.1784626746108975], [-3.4378890470235799, -1.9862424862424053, -0.77350281374575225, -1.5489158264780611, -0.56729106485015102], [-4.0934529107379287, -3.9036409502261806, -3.3273805501324172, 3.1297492436048149, -4.831515111558419]]], [[[2.875989656741984, -1.8610442491782111, 1.1196296199309552, -1.8854636004297873, -3.6047481428262564], [1.0310517416301366, -2.9319639129602493, -4.668558168167829, -2.8872920418780557, -0.68630964339557377], [0.79769148631861242, 2.6758863558846926, 3.8047112229127613, -2.2719349700655354, -3.6750845662900389], [3.4731361750383254, -3.6447321464710445, -0.8322895659574101, 3.6789476087651014, -0.35160182947795349]], [[-3.5988644625862323, -0.24476757444595432, -2.5982464417349007, -0.59025464736676625, -4.9233202550574475], [2.9638087360886098, 2.2711768262191292, 0.79303160367647063, 0.87681102804711664, 0.749452341312697], [1.9476356464773676, 3.6204411941780918, 2.8216642430233199, -4.379987513914636, 1.9375455108443838], [4.921250495114041, 0.34854533213207262, -2.4292005225662416, 2.1607004240126866, -1.1020322372209499]], [[4.9889298696341058, -1.0261949372126713, -4.2969873888944035, 4.0920745393169007, -2.58806653489117], [2.4212208772273716, -0.46801469230319448, 4.2692511718001391, -4.9315696308572097, -2.0351651522081191], [4.9365399537490759, -1.7677805300927218, -1.7403984506037729, 0.94108616639068998, 2.8957575963508724], [-3.5816822891533451, -1.0482119566632955, -1.1224399941537158, 2.4112530038088575, 2.6205973732458396]], [[-0.59502424961869771, 1.888503606143118, -2.3773997655268464, 4.9992872711102869, 1.4343348434971581], [4.0483188323915904, -4.4130433756440652, 2.5723329690300272, -4.5371465396970798, 0.27483159858378681], [-0.78014739312922643, -4.2494490174256452, 1.5026091563845378, 1.9320793249678907, 0.88259224965944583], [1.3519246238394764, 3.9576965409231786, 0.12729180940136953, 4.9103916358425721, -0.83699847308980058]], [[4.6425103387934659, 3.5454754791544101, 0.21194707710940186, -0.77289774400493272, 1.6930972623346063], [1.4454738347707385, 3.3675553871081796, 1.6432941323358996, -0.83352762799315361, -1.2264575013244827], [-3.8481266324021179, -2.120851594217493, 3.3511645964820342, 2.6989539094617632, 0.65296968113542064], [0.025959872129655004, 2.506623936234984, 4.4664629604731445, 0.28443888691598662, -2.3863290231251835]]], [[[-4.6911461482125389, 0.94098814504538097, 2.7000857357508288, 0.96604755929042518, -2.3212156654762928], [0.27851578015437362, 1.9107359857725719, -1.7494224610762652, -1.8788174344824382, -0.1622519301503651], [3.5611539397975225, -3.1692322378081714, -2.4236554974501514, -4.9199038851524524, -2.4484193255015518], [0.59753990200273943, 3.1127499602415742, -4.8482730959630942, -4.356183767789811, -1.6040906852857164]], [[-2.3706711864028165, -2.4415348799521541, 1.9425669903387952, -0.48996261273519082, -3.0642219270666713], [-2.9998463286565769, 2.5493637593059058, -2.5089139721620137, 3.3472589971907158, 1.1459278818816809], [-2.6449953591774147, -2.2637796550681886, -2.4781376177777625, 4.2099028427441141, -3.0301507057124399], [-1.0748732061389301, 2.180738299866201, -4.136781871972806, 2.3866899561121517, 0.88489896174336202]], [[1.5567247622184519, 2.5811617326235838, 3.8594104519100298, -1.2988146305835713, 1.0445991505690362], [0.29088539914734213, 4.7094687017974817, 1.1984270903527303, -1.0433285338074629, 2.4848051233462236], [-3.0735290207403478, -0.97552486446467945, -2.0855341814709649, 4.107577617322276, 4.9778776908049043], [-1.9093241917244819, -0.18305381722177039, -4.8321349178178181, -2.9655364390770114, -1.8189099408380525]], [[-0.66556503961082747, -1.1354505103065993, 2.9380698226781803, -4.3209530785336883, -3.1142225027932948], [-2.0287316069029524, -0.94497151899268417, 4.1886149817104936, -3.0869345031363782, 1.1477383922300994], [3.6937681774253655, 4.1658453739403747, 1.534144955181576, 0.77932819090407524, 2.476014181909882], [0.69207757880023113, 1.1795513656713306, 2.5047010609291167, 4.0937321910029052, 2.4029774128834127]], [[-3.1474630114162094, -3.3749003281958636, 3.8448226058438255, 0.19701213345099511, 1.4696018202368375], [1.7382853570771974, -3.0737433323663299, 4.5152169158180442, 1.5800341360475141, 2.6410901705679919], [2.6526590901326577, 3.6103945037831746, -3.2105172923678262, 2.8439484914105968, -2.833325904852122], [0.078107424403467185, 2.0767599919831712, 2.9519551886548809, 2.2876783598548966, 3.3050639218065019]]], [[[2.2206491386702645, -4.746773534489714, -0.24018557600279156, 3.771067487649276, -1.294198546369143], [-2.3448395240659305, -0.21586812505610364, 3.439847948606916, -1.9725080672870088, -1.4543929572548975], [4.5451598374373745, 3.3886997439174777, 4.5793165864269039, 4.8226259431444554, -4.2867417405386412], [0.691738343851652, -4.6292387218351996, -2.2194367367005974, 0.41119545570594163, 2.3022627116127818]], [[-3.8327823438307131, -0.28648688697690261, 0.60829791670950684, 4.2958563965048988, -3.6486498765859334], [3.6833876177934464, -2.9888522421776251, 4.4928309672093008, 0.90526794301477942, 1.0487867031075995], [-4.3348809848226306, -3.0351130109848592, 1.3860631819477476, -3.6675663392832583, -4.325961829745081], [-3.1286079771616802, -3.3121206720531426, 3.1613485551525837, -4.1770909414339377, 2.4848610993157791]], [[-3.7537546257238086, -2.6237722218860093, 2.7066094647106329, -4.6600796974005849, -4.4637683412175981], [-4.7566924451722681, -1.6268485406483277, -3.3039392424096645, 0.36174746506252387, 4.9830071320877334], [-4.5279145879464098, 1.216113559168134, -0.93790184583473923, 1.0884979252543818, -4.6075586098028243], [2.6499751246041745, 0.99160844977261053, 4.4335452451235771, 3.4641037166512891, 0.99042608780783059]], [[-1.5631743916605534, -4.9234386695442565, 3.1676461165290561, -2.5097039405701729, -2.9898978025586622], [-2.1163520373466858, 1.432246828720853, 3.1970698864047939, -4.7163108318024785, 1.967748136815727], [-4.0413218457837408, -1.6293353835497504, -3.775149452393658, -0.62212452096324711, -0.6518555844450642], [2.3690575586662526, -0.23827840546781687, -4.3842683671240064, -1.9656118405296317, -4.0298033450629305]], [[-0.47039659689603219, 2.858901157563599, -0.51417790708566891, -0.79852445775138747, 1.7803456163415889], [-0.096305450017892369, 2.3772543359180895, 0.92032381487914616, -2.7272784304413067, -4.5971930403086239], [-1.1162991674193004, 2.8839545903169439, -1.7045660827918852, 1.6287977749550908, -2.8460393224533407], [-3.3202308450410944, -4.3321156893019905, 3.3731582800989592, -3.6466416425555339, 3.8077631384014285]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[-1.567565164945971, -0.8732079114590211, -0.83623220703272483, 2.4197971054772411, -1.3214444611872957], [2.154361651733967, 1.6498616206614067, 0.81767086108038933, -3.6587247562055487, -34.407227780890722], [-3.3879297075502866, 4.4007058096780627, -1.2621673570483276, -1.9461651886937037, 1.8970121064584562], [-1.9045694534551096, 9.9852370430402253, 2.9833381136674668, -39.78061563379503, -1.1888544900488234]], [[0.54547871786238877, 0.37481827639905863, 0.27416424534012723, 0.53476755134028875, 0.24246559352701208], [0.49868393407794626, -0.26507631878866511, 0.27687100654833208, -1.1467980358614682, 0.40815024990423199], [0.3804364103481237, -24.011171228869742, 0.30480955133195992, -1.148337106663734, 0.24652911841074199], [3.5156432763075403, -0.24803108527466727, -0.30470363790039406, 1.527545087032635, -0.27867299134943774]], [[-0.014777808320301707, 0.015311046092656789, 0.013182332931557126, -0.014924771037055628, 0.01293216292051581], [0.013757935191830044, -0.021963536105682503, 0.067910118961543503, 0.074198952629580719, -0.015274142194740947], [0.016227340995920519, -0.085596151434661982, 0.025866139772823597, -0.025205609914438182, 0.015356358979626689], [-0.013212039904896195, 0.038236281390675114, 0.098083513059801702, 0.01144516029388634, -0.025653117553623192]], [[-1.0284697342718634, 0.97612985330795476, 0.91926655313929939, 0.94735667759758835, -65.919246384898941], [1.1357434448579975, 5.0956491449603352, 1.7257444360932472, 0.94529747031230815, -3.7983086181915087], [-1.5632612445140512, -1.0565714199451566, 1.7876791046632057, -1.0601476487982149, 5.9535776807956493], [-0.86751376282644199, -57.370962680875444, 2.4952892620536335, 2.3082499907723832, 1.1655286618615939]], [[-2.6224534696354551, 2.8678145096586181, -0.79308836435284658, 1.1175646975682498, 1.1738027676690246], [-0.75161098513990066, -0.91122219476058675, -0.97288234185212896, -1.6745783945945696, -2.9648732747145541], [-1.0163191544030417, -1.75909664273291, -4.5171037869694768, -2.2557665364858175, -6.1590825339817679], [-0.85355629230204577, -0.89505990273004954, -1.0500730038418467, 1.1163809676899663, -0.72316704150294708]]], [[[0.88420706078294409, -1.366421224187919, 2.2712603489239562, -1.3487241867996356, -0.70545021745574565], [2.4663848171280267, -0.8673266236290319, -0.54470144092216888, -0.88074580761005672, -3.7052814071625475], [3.18791212498195, 0.95032823633843355, 0.66837407946119065, -1.1192971606738482, -0.69194880154746508], [0.73218274005680661, -0.69771117849967923, -3.0553913748813075, 0.69122222756620533, -7.2325288096643137]], [[-0.63012054559737862, -9.2647829020202792, -0.87278804745775107, -3.8419323741245566, -0.46060754149932759], [0.76513656602841418, 0.99847727068920777, 2.8595561995042553, 2.5863251785811188, 3.0258340840244089], [1.1643442872887495, 0.6263652182343119, 0.80368117656198101, -0.5177454117144431, 1.1704078309404871], [0.460801261985609, 6.5062367205553553, -0.9335245969320819, 1.0495293162781081, -2.0577605283258076]], [[-0.17947650122097139, 0.87253955887833623, 0.20837754380031068, -0.21881216220176591, 0.34597088821612471], [-0.36981164595940397, 1.9131785658956644, -0.20973131863337538, 0.18156403434642121, 0.43996217057236719], [-0.18138122778864865, 0.50650839433770767, 0.51447740460128466, -0.9514491975509547, -0.30920947214886735], [0.24999304950925283, 0.85421242540390463, 0.79772253528245507, -0.37134040949844066, -0.34167617161643576]], [[3.2243738487557505, -1.0159263787503965, 0.80700799994437344, -0.38377083088084818, -1.3376099998855282], [-0.4739203381154195, 0.43475227105965936, -0.74585236551607836, 0.42286062684105763, -6.9809317405004752], [2.4592540419195403, 0.45148926883898383, -1.2768327822934231, -0.99301338462277045, -2.1738018100502696], [-1.4191476329480996, -0.48477204101113613, -15.072302286131231, -0.39071845427596619, 2.2922152089042109]], [[-0.66701179206641359, -0.87339742129707776, -14.610294149831986, 4.0064926632076183, -1.8289611646383377], [-2.142279622278064, -0.91954215589743682, -1.8843912844523665, 3.7150647882194945, 2.5248401493091897], [0.80470562343019492, 1.4600781823718336, -0.92403970369471644, -1.1473367996058228, -4.7423475089699778], [-119.28445276231164, -1.2353704502705229, -0.69330232180800411, -10.886729217443865, 1.2976455093816452]]], [[[-0.94901007087146461, 4.7311381785462094, 1.6488161392940366, 4.6084117658281674, -1.9179367969973098], [15.984533932389718, 2.3299634129115518, -2.5448083797007448, -2.369546320401251, -27.438471360296248], [1.2501411098327035, -1.4047392568688899, -1.836872007291187, -0.90488453484201103, -1.81829349744722], [7.4504563187533703, 1.4302288958147069, -0.91825374735810239, -1.0219828124565908, -2.775369858712577]], [[-0.087031116427167099, -0.08450510444413438, 0.10621109133454908, -0.42109776271003851, -0.067332642656161221], [-0.068777576392305154, 0.080930843737469435, -0.082235645511814531, 0.061639138234452648, 0.18004811934199627], [-0.078004734230807657, -0.091140566429438616, -0.083256942049639113, 0.049008769974337042, -0.068089735485893615], [-0.19195023083279483, 0.094611150749731296, -0.049875039685367821, 0.086446988854156262, 0.2331590034051722]], [[-1.732207719747584, -1.0447120056658112, -0.69870014714358419, 2.0761782221574041, -2.5814405929471524], [-9.2702165819987794, -0.57258489680753455, -2.2500915344323107, 2.5845843981629786, -1.0852241993954124], [0.87735324197052089, 2.7642254430047473, 1.292987990604443, -0.65648684014275915, -0.54171090937370925], [1.4123168094369085, 14.73102659951701, 0.55804953638479737, 0.90930282127179018, 1.4825201567674071]], [[-7.2886296233609746, -4.2723632777891787, 1.6511033967051694, -1.1226821897420602, -1.5577104910229611], [-2.3911773481887475, -5.1335484366259019, 1.1581530136245382, -1.5714803987749468, 4.2266226317960047], [1.3133084782170026, 1.1644832269404175, 3.1620591311117296, 6.2246651931753325, 1.9592202255639015], [7.0094122574965834, 4.1126289241502967, 1.9367808556688209, 1.1849961936060256, 2.0187693142566632]], [[-0.27229903971579406, -0.25394858283348865, 0.22291045476245519, 4.3502455434441476, 0.58318596489727736], [0.49304399422122813, -0.27882977297581057, 0.1898139494798422, 0.54242572106292919, 0.32450658637122493], [0.32309133078489732, 0.23738435083798978, -0.26695111021112777, 0.30135959147576485, -0.30248943620707364], [10.972723298651886, 0.41268666521796143, 0.29033338949164339, 0.3746379607332459, 0.2593145475628692]]], [[[-1.2879005014786156, 0.60250928731297515, 11.907355915778428, -0.75839935208485942, 2.2098426453383926], [1.2196890703813787, 13.248714410977481, -0.83142487168938206, 1.4499180950043487, 1.9664390734534127], [-0.62923532759938838, -0.84397419524553074, -0.62454191260295411, -0.59303275290654711, 0.66716758610751425], [-4.1344753615607441, 0.61780679527533433, 1.2886040372356424, -6.9552693241501791, -1.242245346230717]], [[-0.14757750434504591, -1.9743746702301779, 0.9298609077275708, 0.13166931126014592, -0.155025138649247], [0.15356310866330639, -0.18924737898323887, 0.12589666896630033, 0.62482324417297719, 0.53932077068130191], [-0.13048396368451537, -0.18636289685198612, 0.40808562002595333, -0.15422555467962773, -0.13075299211172736], [-0.1807936491658001, -0.1707765232628628, 0.1789212556389537, -0.13541300894112651, 0.22763141696573802]], [[-0.76222990285044545, -1.0905001584448422, 1.057124812794433, -0.61398607094335178, -0.64098846646450347], [-0.6015154556805169, -1.7587525526805468, -0.86600382566450895, 7.9094514821142958, 0.57419625295440824], [-0.63190768467820158, 2.352760564274154, -3.050664668586252, 2.6285985092908661, -0.6209848351364351], [1.0797173139945033, 2.88543731585384, 0.64535802963484079, 0.82596372906980464, 2.8888819255789064]], [[1.3360482219486578, 0.42419059254110281, -0.65931492652411638, 0.83216045239952952, 0.6985109540488319], [0.98682843388953523, -1.4581818752840152, -0.65324701704357013, 0.44281991604348597, -1.0613534971774368], [0.51678051050367035, 1.2817964844191252, 0.55321687072535508, 3.3570069916873391, 3.2038942618735295], [-0.88156421482200764, 8.7648579084345002, 0.47635687227416612, 1.0625070135978727, 0.51825763883302922]], [[-5.5137545194408153, 0.90721966906873686, -5.0442683871147738, -3.2480550055645061, 1.4568246402598728], [-26.931511784465037, 1.0910281339600187, 2.8181943356596357, -0.95100351072163647, -0.56418152105506469], [-2.3234375136740191, 0.89933848846768327, -1.5215903849365413, 1.5923716264510424, -0.9113195807250124], [-0.78116597402822319, -0.59870316216854713, 0.76890888203117058, -0.71124382823848553, 0.68114829305111169]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_array_rank3(self):
      arg0=numpy.array([[[3.4385774668455813, -1.3593485483586711], [-4.1887191364408309, -1.7219160268076426]], [[1.2151186775273848, -0.10278130281459141], [-1.9841485658031677, -2.2166103915776167]], [[1.7220134383141961, 0.93323230214847541], [1.498331211357077, 0.20253027634011023]], [[3.2399396378857972, -2.2462904814681539], [-2.8089662525830361, -3.0432987092045662]], [[4.7442619244159765, 1.6083569573012788], [-2.9658385562293321, 4.3300646896620592]], [[-4.3004782184797694, 2.5487323154305912], [-3.5041369180059831, 3.3590094136429016]]])
      arg1=numpy.array([[[-0.040353793163339091, -4.0225523716812042], [-0.54455915574858871, -3.730527140035421]], [[1.3783635347693206, 3.4872634295354494], [-0.21262080655541382, -3.8518610675947462]], [[1.8509796868724893, -3.070004503974352], [1.32172558542511, 4.7692314168936463]], [[3.290930989667288, -1.5942575809036641], [3.8046901928309609, 1.2728847641222369]], [[-0.91031287591603149, 3.5611531264465253], [-2.6385998010941458, 3.2188916850312026]], [[-2.1604771549707502, -0.30485761142363543], [0.12677457207062837, 4.5693004708258673]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[-85.210762044780594, 0.33793184594152065], [7.6919451123408766, 0.46157445373559008]], [[0.88156618110964746, -0.029473340598270566], [9.3318645430218208, 0.57546478252440081]], [[0.93032541120091861, -0.30398401726783658], [1.1336174678612769, 0.042466019917319235]], [[0.98450549344802696, 1.4089884271993875], [-0.73829040216621811, -2.3908674178398086]], [[-5.2116827630740818, 0.45163937078610489], [1.1240198513618815, 1.345203602158513]], [[1.9905224216721662, -8.3604024302638411], [-27.640692141747213, 0.7351255263446892]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[2.1429866339494357, -1.2378453277452763], [0.40807761628921302, -1.0682616623254759]], [[-4.4219111290863573, -1.481074799267541], [-4.120725860148772, -0.49516238496250331]]], [[[2.6911351850274254, 0.89645908845738198], [3.477253580359168, -0.35370633364193615]], [[0.73090558769329217, 4.4942656396975273], [-1.1491244673855827, 0.54555788051103171]]], [[[-0.54073115405226613, 4.6372724477677707], [-4.1222406894807841, 3.5878730237352059]], [[-1.049740453324739, -4.6063194332320361], [1.1621514113920703, 4.5440497391135359]]], [[[-3.6116037270525658, -0.67877110704707011], [-4.3892549938050349, 4.3550937450127556]], [[4.8964751692461164, 2.988069450011591], [3.1421009011231789, 4.492814903057635]]], [[[3.7315409601769325, 2.642643235785], [3.1595844030063489, -2.1837225373125504]], [[0.020069534140824352, 3.6943680523477447], [-0.43279165233984607, 0.47535769709831843]]], [[[-4.617586895073881, 2.4774131123801251], [3.1415781692199012, 1.0614070355260843]], [[2.6971815105328858, -4.3898506828282899], [3.6329281109441816, -4.0767526616310255]]]])
      arg1=numpy.array([[[2.3062531236450337, 2.2537731968224364], [-4.6225289021111156, 1.6766455463323542]], [[2.112604507408248, 1.6415111322582296], [1.82119383388331, -0.027131462753930968]], [[-4.3364930311561061, -0.25166150662219344], [2.6946101697625551, 4.067433411545375]], [[-0.99402475627641618, -4.2908048176204714], [-4.0788299292953498, 4.3211481584502724]], [[2.5914678015681876, 3.8976728573650838], [-2.4550725711604935, -1.4755819607108034]], [[3.6470299430720701, -3.1673823084112254], [2.4625005809629865, -3.2471015456871521]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[0.92920703802124061, -0.53673437449435801], [0.18106418909611491, -0.47398809420202676]], [[0.9565999959604069, 0.32040357791838403], [-2.4577203387816939, -0.29532919825878895]]], [[[1.273847128315049, 0.4243383393880768], [2.118324702175796, -0.21547604928840278]], [[0.40133322115130976, 2.4677579926319311], [42.353944489008086, -20.107942039799827]]], [[[0.12469319105722339, -1.0693600599495203], [16.38010017824972, -14.256741413860707]], [[-0.3895704340109577, -1.709456709145404], [0.28572106628551397, 1.1171786429779746]]], [[[3.6333136617054831, 0.68285131005159694], [1.0229444545648572, -1.0149829531113319]], [[-1.2004607336231892, -0.73258005403716442], [0.72714491285808058, 1.0397271137930455]]], [[[1.4399333682320292, 1.0197476635387268], [0.81063355459295794, -0.56026316656775466]], [[-0.0081747213408594442, -1.5047897547897926], [0.29330234704913699, -0.32214930092350386]]], [[[-1.2661225619618204, 0.67929607133778558], [-0.99185316558635839, -0.33510543792185643]], [[1.0953018778489485, -1.7826800597592523], [-1.1188218353594424, 1.2555051341236396]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_array_rank4(self):
      arg0=numpy.array([[[-2.080763780909697, 3.0296342701528509], [2.4279492205283884, -3.7389115926529115]], [[-3.6243324969098198, 4.3626261374739244], [3.5066439836675514, 3.4340906216379317]], [[4.7791985538048483, 2.4592945175546674], [-1.8213344137227141, -0.44972381321182553]], [[2.7553624392565901, -2.5052062396889041], [-4.6508084765031663, 0.12366075531366238]], [[4.2675483224922921, 0.61164875479143532], [3.0190651446904848, 4.6644196233729094]], [[0.12322787343180863, -3.4179201245383437], [-2.2291499900848799, 2.7762077506473757]]])
      arg1=numpy.array([[[[4.0909752578686938, 3.0675348378741134], [-4.3079421297580662, -0.83967978991496217]], [[3.7194038004314418, -1.6548168731558697], [-3.8224300033413594, 1.9224881773791838]]], [[[4.2340050149962654, 3.1055842622794021], [4.3797075934218661, -2.1631989144675612]], [[-1.8441432110435443, -1.2265359580456812], [-3.0545651788860315, -1.2078577441546856]]], [[[-3.7391352170249248, -4.9625061940575685], [-3.0576289178276559, -1.7988232915337599]], [[0.37809187387427645, -3.4902470604827607], [-1.4243530342343824, 2.0048128149933744]]], [[[1.4005709522094012, 3.8615494152020204], [0.99949390244615799, 1.189309385480648]], [[4.2176897859618876, 2.4933612080087588], [4.2212619909035318, -3.4784559589127717]]], [[[1.9895778096007035, -4.3220334718050823], [-0.55419402753843361, 1.4377611566639059]], [[-1.2780649881396942, -3.5103057879515154], [-4.315822987887568, -0.099717425448523578]]], [[[-2.305177258929044, -4.1815564055330618], [4.5281423570417125, 3.7057131388315661]], [[-1.0358775626485475, -1.4773221704977804], [1.5326022621269466, 1.5540317881178245]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[-0.50862291990338959, -0.67831789723102998], [-0.70326717000792094, -3.6080828746154228]], [[0.65277914171264551, -1.4672011507219482], [0.97815044078885927, -1.9448294333596121]]], [[[-0.85600571659054037, -1.1670372435007359], [0.99609986384168725, -2.0167475622775628]], [[-1.9015030734425711, -2.858981802094831], [-1.1242485985813238, -2.8431250602621803]]], [[[-1.2781561180361536, -0.96306147880032378], [-0.80431425252934707, -1.3671684868266072]], [[-4.8171741832471815, 0.52183538361630843], [0.31573900739682903, -0.2243220962318678]]], [[[1.9673137122471409, 0.71353804988468361], [-2.5064747604339264, -2.1064377951380995]], [[-1.1026909783604442, -1.8652766641129312], [0.029294735929715099, -0.035550473191074655]]], [[[2.1449517087993475, -0.98739363087578436], [-1.1036725846869888, 0.42541749855773553]], [[-2.3622156719001657, -0.86005759243336444], [-1.0807717639170289, -46.776374363784491]]], [[[-0.053457005509875075, -0.029469379695261966], [-0.75481728599436304, -0.92233802145193433]], [[2.1519435022661901, 1.5089125680241926], [1.8114339377227282, 1.7864549308928857]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[-4.8640066679570531, -2.3281325035508726, 3.732696670709494, 0.82713306133281073], [1.7967636258069071, -3.4372393667404966, 2.9678604355799285, 3.2932552720008861], [-3.9627823759850922, -2.2856564373486208, -4.9351974415410123, 4.5752247236233039]], [[-2.6205863316468747, -4.8883695483972334, 2.491544312547008, 2.2474344950251632], [4.5627851133126089, 1.3204973992291089, -0.72889096111745744, -3.6695037674949802], [1.0922972863000258, -3.8220755465391432, -4.7699704573956367, -1.1645894807993318]]], [[[3.9122172059191449, 2.1152831487935781, 3.3983887809378484, -4.612497316057242], [2.8883731248377469, 4.0759274897087217, 4.0184571800643329, -2.360043357294809], [0.84810949020344584, -4.2310132797360946, 1.2037609323447898, 3.9416630534826762]], [[-3.9724003208051419, 4.5986293641451574, 2.176081222864279, -1.0989640758841359], [-3.4919462299888959, 3.2174520132793045, 2.4102869810764975, 0.68617149368866848], [2.1732449986071334, 4.8664376405819283, -3.4238889251794391, -0.89334478531055517]]], [[[-4.7508172510015756, -0.58148278182498903, 1.3679277685083449, 2.3684342432913912], [-4.8202970130665044, -1.4708075287035163, 4.0746955047879254, -1.9176168365070745], [2.3566950208276873, 2.668233195570231, 4.3007000840420737, 1.6326728139216593]], [[4.4909452572770334, 2.7268834191063487, 4.9504101214040865, 3.2395967314143945], [-2.5156511913305857, -0.070432933094409123, 0.29640887034162322, -1.6557864466179817], [1.3462450726310156, 3.7579526690311873, 0.58384090963514623, -2.6049141331563486]]]])
      arg1=numpy.array([[[[-2.5245263646493097, -0.20349243991665489, 2.4167634625340861, 3.7078519013214919], [-0.31825484774285062, -4.9412597587917144, 1.8143067173558247, -3.2873739025549344], [-4.6523850771686428, -3.8655232350705147, -1.4427691893418548, -2.9150083826001651]], [[-0.82054048425013537, 1.0725688215891536, 2.1151709132471055, -1.1114065839458886], [3.9565241172557677, -4.8788275092474445, -1.5618258223540247, 1.5588934881369383], [-4.9943928811193814, 3.8481328204080221, -2.2160303218726729, -1.3872774550167666]]], [[[1.0218109431960531, 4.0625608055431197, 1.9027062048159422, 1.4043711591522277], [3.0316575989134353, 2.0463053774912581, 4.0355138545984381, 4.8362807831081476], [-0.34083356811415033, 4.1266401292743389, -1.0369467225682936, -4.2202107615778726]], [[0.30233413346716365, -0.33180667433660016, 4.9545249414187467, 0.85685878681262206], [0.76224652760065936, 3.4325353352680406, -4.7508291585723619, 0.0039288682323377344], [2.2157706935899588, 0.91546333052864615, -2.4522820288962399, 0.88507481793830678]]], [[[-0.66685062274124896, 3.3385530945337916, -4.9222402111125483, -0.35529418128434376], [4.021445447340291, -3.7869837611642563, 3.876047757649232, 0.20037298892031519], [-3.5550541568442293, 3.0296864904797349, 2.3133030179012515, -1.1719673461226088]], [[2.2691450949220648, 2.8636952748875224, -2.5721446212819687, 0.40340983079395709], [3.0727134776730818, -1.1732734885686282, -3.6245825662577582, -3.1031119454912259], [2.2096401949119464, -3.2608049788274696, 1.720007891680444, -2.6023513599414114]]]])
      res=quotient(arg0,arg1)
      ref=numpy.array([[[[1.9267006817861965, 11.440879594860693, 1.5445022769400825, 0.22307607837249852], [-5.645675591589697, 0.69562005126826287, 1.6358096495973382, -1.0017890783404286], [0.85177437169426429, 0.591292898361514, 3.4206423854894568, -1.569540846240119]], [[3.1937319144487324, -4.5576278649928152, 1.177939946574867, -2.0221533032906565], [1.1532307090996179, -0.27065875904122599, 0.46669158025499535, -2.3539156429991057], [-0.218704717930643, -0.99322859290856913, 2.1524842915347557, 0.83947841622298736]]], [[[3.8287094417704961, 0.52067728953309489, 1.786081725248061, -3.2843862436207063], [0.95273725036526469, 1.9918471282647716, 0.99577335745863704, -0.48798724952815342], [-2.4883390884767578, -1.0252925254425105, -1.1608705694766395, -0.93399673053507604]], [[-13.139106310126847, -13.859363659092926, 0.43921087260510389, -1.2825498119382155], [-4.5811244834142757, 0.93733980833967423, -0.50734027695510653, 174.64864004369684], [0.98080771845848091, 5.3158192996891973, 1.396205201862738, -1.0093438059750841]]], [[[7.1242600501326745, -0.17417209352669874, -0.27790756034621872, -6.6661216762115254], [-1.1986478683316615, 0.38838495791472227, 1.0512500772846953, -9.5702362221570549], [-0.66291395766518835, 0.88069613933807733, 1.8591166184289563, -1.3931043551027851]], [[1.9791353436705983, 0.9522254141420311, -1.9246235536074869, 8.0305349154196222], [-0.81870672602889394, 0.060031129809585984, -0.081777381235835375, 0.53358901506077283], [0.6092598585647393, -1.1524616447262921, 0.33944083190498325, 1.0009847913907348]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_constData_rank0(self):
      arg0=numpy.array(3.95019602353)
      arg1=Data(1.14176453167,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(3.45972914199,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_constData_rank0(self):
      arg0=numpy.array([0.87638358328190247, -3.5532081472679655])
      arg1=Data(-2.07860974919,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([-0.42162006775049077, 1.7094157037636188]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_constData_rank1(self):
      arg0=numpy.array(-2.88491680496)
      arg1=Data(numpy.array([-4.1951220623644803, 0.77558740396145076]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([0.68768363877628291, -3.7196540199366877]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_constData_rank0(self):
      arg0=numpy.array([[3.2489755850502799, -0.38993378468109263, 2.9889297607362586, -4.546966787319187, -2.6748514077506655], [-2.5991450496867543, 4.8727431106617178, -2.3701833834516162, -1.607678257511119, -4.596442643767543], [2.0626640147560824, 3.9427430298857065, -4.9850372803364653, -2.6236603378131784, -2.0998732331906766], [3.8631536752524056, -1.2816220051996687, 4.2150233893767837, 0.47874999053453049, 0.7708497011187001]])
      arg1=Data(2.15357895988,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[1.5086401035534585, -0.18106314741394763, 1.3878895626362118, -2.1113536452657571, -1.2420493780708506], [-1.2068956365669954, 2.2626257042015694, -1.1005788167521104, -0.74651465651278004, -2.1343274286142981], [0.95778425271529122, 1.8307863808706943, -2.3147687515504276, -1.2182791467995351, -0.97506210466672127], [1.7938295958556794, -0.59511261443071051, 1.9572179464473358, 0.22230435914003766, 0.357938907965623]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_constData_rank2(self):
      arg0=numpy.array(0.23874651927)
      arg1=Data(numpy.array([[-2.7710790198180737, 0.33931045430829698, 0.79223761825474437, -1.3607535335274026, 2.2650577571330928], [-4.984743998378903, -1.1288397520666091, 4.7201583078461162, 4.1418174085608968, -0.55052953068415089], [0.72318311127380142, -0.062830742113462179, 1.0201975002175567, 3.7060118857911011, 1.1309021697606179], [-1.9942253665512033, -1.0194983472064778, 1.1288938453552309, 1.7314037641606745, -2.2096537009329511]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.086156517935073618, 0.70362264480521242, 0.30135721123216475, -0.1754516989212391, 0.10540416398593122], [-0.047895442443604976, -0.21149726418951839, 0.050580193226486499, 0.057642936836613527, -0.43366705319834231], [0.33013287443886563, -3.7998360554030128, 0.23401990224397676, 0.064421412188616939, 0.21111155823584496], [-0.1197189260927657, -0.23418038874182601, 0.2114871298597038, 0.13789187953290505, -0.10804702979913074]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[0.28545566558628277, -1.3815042632509709], [-2.8513397258577533, -0.3004159616860651]], [[3.381665953927488, -0.48528582194395131], [-1.9632921884777064, 3.0119966140532366]], [[4.5848121878260208, -3.9316962996574745], [-1.8494894947695482, -0.3409276333283966]], [[-1.783819151900726, -2.4366423500693468], [3.6895324414382333, -0.037627978805737428]], [[-3.1139847100588645, 0.66733482811219957], [-0.82986455636240652, -1.9703216801975043]], [[-0.62255568842090359, 3.5865028145112507], [4.4756282086639558, -4.3771137305304499]]])
      arg1=Data(2.77290914251,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.10294447128117111, -0.49821476011330967], [-1.0282845846431536, -0.10833963402575621]], [[1.2195372369352302, -0.17500963681229964], [-0.70802615144522452, 1.0862226128794972]], [[1.6534303694054953, -1.4178958262225683], [-0.66698524896358058, -0.12294944255546519]], [[-0.64330241642424635, -0.8787314062028303], [1.3305637696084984, -0.013569856375349841]], [[-1.1230027923820096, 0.2406623491126911], [-0.29927578355907325, -0.71056121168648367]], [[-0.22451355469139836, 1.2934079806403211], [1.614055123571337, -1.5785276421175529]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_constData_rank3(self):
      arg0=numpy.array(0.854524916829)
      arg1=Data(numpy.array([[[-3.8531346523013577, 0.33123673494803629], [2.4541354880466582, -1.7890139142146131]], [[-3.5073357126112112, 4.8048155084202016], [2.7794266039988251, 0.97348760089758457]], [[-3.0819514439604303, -3.8178016395781156], [-3.5434870844630084, -1.184203644133146]], [[-0.27770572810379157, -1.9637928034115681], [3.2281378906561304, 4.8746203099123004]], [[1.1726777817679599, 1.467861908198028], [0.1810081951843916, 1.9431188623316462]], [[-3.7104424199015886, 4.6706053392312263], [-0.14772135231213213, 1.9790146254663714]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-0.22177395651587931, 2.5798011713989544], [0.34819793812991906, -0.47765135309439449]], [[-0.24363932821050518, 0.17784760212578327], [0.30744647676606546, 0.87779743269602151]], [[-0.27726748210257668, -0.22382643141295783], [-0.24115367051177117, -0.72160301233900814]], [[-3.0770878320159576, -0.43514005924902005], [0.26471140508053814, 0.17530081575618817]], [[0.7286954098686893, 0.58215620424290337], [4.7209183869206104, 0.43976976056108141]], [[-0.23030270251491286, 0.18295806533933401], [-5.7847081918375327, 0.43179312867783026]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[-1.906729003167591, -4.6204085645506368, -2.1354895491244941, 0.74822410903335612], [-3.0239668670475641, 3.3568055396924503, 0.9049250774115416, -2.6289671375195676], [-0.74434740092168106, -4.372764530812578, -0.88394399056580042, -0.091992573270816003]], [[-1.3128486656355856, 1.3665591253951348, -2.924231794100971, -0.051883326903321425], [0.29017173252665884, 0.24243136336683868, -2.9081772569266882, 2.0113171562844006], [-1.7656382543843154, -4.9003401010964263, 4.4154884439858169, -2.5158132193729199]]], [[[-3.3391382045288873, -2.0630764044403693, -1.4008241468764702, 4.1754661582853618], [4.8426966937817966, 4.9668913670536554, -3.3982856409301565, 3.8932044155486025], [-4.1758394049863785, -4.2789351016164954, -3.8465205898823349, -4.3198083880680995]], [[-4.365502168078188, 1.6056048645115739, 1.9588693249119249, 0.59414300983871904], [-0.11544935199713979, 4.6287293679788242, 3.9937415586648921, 4.7585329619060506], [-0.61961378273975498, 4.1851094825427264, 3.5138613051010541, 0.47870864282768011]]], [[[2.582614842180158, 2.5334761987601908, 4.9773128981611965, 3.1321681546431464], [-4.9873584312412689, -1.2473040714173833, 1.1878193690927095, -2.7524160426602471], [-1.4771231889259804, 1.9053822470352033, -4.5607038705413387, -4.3801037541864183]], [[-4.5629265334751583, 3.7777790876591872, -1.71864060273451, -1.3714285847359609], [2.0506166317157035, -2.5192103103964714, -2.8968583686914529, 4.3511605598670968], [3.3286495651844401, -4.2841915535838817, 0.59896962292894074, -2.2222450819486985]]]])
      arg1=Data(-2.66142860626,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.71643064130474254, 1.7360633133978658, 0.80238468321256473, -0.2811362691729723], [1.13621941987776, -1.2612795743610954, -0.34001478577483574, 0.98780299097212099], [0.27967964241899484, 1.6430140265765563, 0.33213139307498513, 0.034565110277441684]], [[0.49328720017099759, -0.51346826369202292, 1.0987451578546827, 0.019494540180916869], [-0.10902856151934781, -0.091090688210411275, 1.0927128573313707, -0.7557283902170816], [0.66341747820445707, 1.8412442436274641, -1.6590670264852494, 0.94528675819374874]]], [[[1.2546412842630532, 0.77517630929117609, 0.52634293611448679, -1.5688815204242745], [-1.8195854220535519, -1.8662500866550638, 1.2768652267961014, -1.4628250430597685], [1.5690217634126227, 1.6077587396319863, 1.4452841533449321, 1.6231163886613493]], [[1.6402852805499615, -0.6032868440418514, -0.7360217442265905, -0.22324213711457877], [0.04337871462180614, -1.7391897558688849, -1.5006006733659591, -1.7879619053911573], [0.23281247570684949, -1.5725048842953522, -1.3202914017080165, -0.17986905292225913]]], [[[-0.97038666981647059, -0.951923411660892, -1.8701658524520646, -1.1768747609006962], [1.8739403414826756, 0.46865960202160373, -0.44630893584749609, 1.0341874421087947], [0.55501138954214124, -0.71592461377906658, 1.7136299879767256, 1.6457716520700922]], [[1.7144651270176881, -1.4194553552093994, 0.64575867212590521, 0.51529790485894267], [-0.77049469855949149, 0.94656317455733807, 1.0884599203152794, -1.6348965926184598], [-1.2507003033478454, 1.6097337886545406, -0.22505567931476184, 0.83498203811445404]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_constData_rank4(self):
      arg0=numpy.array(-4.49762385744)
      arg1=Data(numpy.array([[[[-4.0411344087239121, 3.9478615111484903, -0.99562193757480788, 1.7510195663316424], [3.7941583228684799, -1.8145359977243025, 4.9464712322522342, -0.29799182781270783], [-2.4275981141644722, -4.9807036379779577, 1.4118722460240987, 4.681707923190098]], [[4.7428930002136998, -2.8523211712018428, 2.1565959999133533, -2.692416503060644], [4.3199943184703287, -0.2633516461301415, 3.9766260982689658, 0.76541197567496511], [-0.50579645206568458, -2.5995642338382829, -2.1233253257984153, 4.6264794135265781]]], [[[-2.2337539584405661, -2.8614948910655036, -1.3316547457764552, -1.5631177486915582], [4.7944947633380472, 1.7742443623014807, 0.13452586630782992, -1.4159116671495795], [2.8197848010061222, -1.7633688522589375, -1.3886902756446884, -0.76621177075292835]], [[2.4480317371669535, -2.7412233371499859, -4.8642137751722201, 4.4080238559989908], [3.2194927569524072, 1.7998779757264192, 1.1094381040648358, 4.8500725904671569], [1.9626008812590001, -4.5964672002010909, 4.1690406030119931, 4.1801862273396129]]], [[[-4.4916550444027035, 0.65214307245260983, 0.54292198813394954, 3.9837460426102353], [-0.96021924468438158, -0.83192593157345662, 0.18863695919018397, 4.305137173038041], [3.8790712663296425, 3.7873723900298675, -0.28503281797346869, -1.133417313401571]], [[-1.3207920349140845, -2.0529649823890095, -2.4271431792197373, -0.7016658046790889], [2.4195302535457248, 1.4018068692749033, -2.7540858841603955, -2.1646722477510636], [1.5455655231425505, -0.15888174619892137, -0.043537507958280486, 4.3242331345377139]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[1.112960719081644, -1.1392557324358916, 4.5174013224281753, -2.5685743003211488], [-1.1854075330305727, 2.478663340424013, -0.90925907505805403, 15.093111413329632], [1.8527052856056967, 0.90300973202751389, -3.1855742402363019, -0.96068014733697349]], [[-0.94828701748833266, 1.576829391741968, -2.0855198922837856, 1.6704784910975696], [-1.0411180029124454, 17.078396598350221, -1.1310150228598974, -5.876082424073374], [8.8921617363477701, 1.7301453062377889, 2.1181981879049729, -0.97214824825311019]]], [[[2.013482210269403, 1.5717742049730545, 3.3774699273245075, 2.8773416853616904], [-0.93808087805847995, -2.5349517535480834, -33.433152901221348, 3.1764861903378905], [-1.5950237960833573, 2.5505859716628914, 3.2387523239125104, 5.8699487910754602]], [[-1.8372408286846509, 1.6407360161011397, 0.92463531935943721, -1.0203265690856027], [-1.3969976629785679, -2.4988493209511757, -4.0539655533375276, -0.92733124577950499], [-2.2916650554817508, 0.97849580156738047, -1.0788150765884976, -1.075938633552542]]], [[[1.0013288671942147, -6.8966827179871242, -8.2841070277840494, -1.128993617899398], [4.6839551303903324, 5.406279197154479, -23.842749993146519, -1.0447109294464281], [-1.1594589396895942, -1.1875314582951455, 15.779319340896336, 3.96819759523598]], [[3.4052475624835994, 2.1907942395613156, 1.8530525499879273, 6.4099231107549199], [-1.8588830831306413, -3.2084475800617058, 1.6330732034560624, 2.077738956607238], [-2.9100182361043228, 28.307996135741572, 103.30457732557306, -1.0400974502314297]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_constData_rank1(self):
      arg0=numpy.array([1.6051137744842139, 1.7173171527021527])
      arg1=Data(numpy.array([4.9794473934178249, 1.3834969291296257]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([0.32234777228613026, 1.2412872891467401]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_constData_rank1(self):
      arg0=numpy.array([[-2.4133026845340524, -4.6716600238888102], [4.8584828300954577, 1.677290389682577]])
      arg1=Data(numpy.array([-4.5063522018571387, -2.0182693573654253]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[0.53553352610554772, 1.0366832894161149], [-2.4072519420487772, -0.83105378554230758]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_constData_rank2(self):
      arg0=numpy.array([-1.2953603138883008, 3.357152949372459])
      arg1=Data(numpy.array([[-0.82429772977928195, -1.1912219995869187], [3.2289859890587085, -2.3258950713175364]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[1.5714714078311884, 1.0874214162746274], [1.0396926343898794, -1.4433810840274717]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_constData_rank1(self):
      arg0=numpy.array([[[-4.1752919269322932, 4.4052813313192232, -3.9077865103466367, 2.0990096047098126, -4.5903650481101828], [-3.4211593191282983, -2.0439914973647388, 0.94745279181022823, -0.46906463017239375, -3.8213031892795346], [0.96846507857895592, 3.8509412716986962, 0.022486981307200615, -1.3180995971896916, -0.48646627750926186], [0.89833540404335555, -3.4779537252260253, -0.22605258050146571, 3.3367281464778173, 0.11281009939349573]], [[-4.2705397975734494, 0.48732435484446945, 3.5145764256152194, -0.70519678754292769, -2.3994727799771773], [-2.7168307673441108, -4.3396491987595267, 2.3402178220069931, -2.7043313810287462, -3.0690726562361705], [-2.7986535178277547, -1.8809384092155579, -3.6495966415964185, 3.9024485074251398, 2.9497405624641395], [2.5836403674748256, -3.5833874790991995, 3.4976481870249483, 3.8105665585071851, -3.6616562505311814]]])
      arg1=Data(numpy.array([-1.7237703458734241, -4.5880677812717687]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[2.4221857261482809, -2.5556080262461491, 2.2669994989190885, -1.2176851804735374, 2.6629794735123329], [1.9846955409798615, 1.1857678734629007, -0.54963980212234109, 0.27211550035960358, 2.2168284762685735], [-0.56182952729021474, -2.2340222297695038, -0.013045230393383171, 0.76466079158695499, 0.28221060808583076], [-0.52114564227996063, 2.0176433209632498, 0.1311384553299797, -1.9357150182248419, -0.065443810228871024]], [[0.93079265633466657, -0.10621559621104547, -0.76602539307756523, 0.15370234729781038, 0.5229811097760344], [0.59215140160615309, 0.94585551165432369, -0.51006609613738252, 0.58942707692063157, 0.66892487263678846], [0.60998521627158586, 0.40996308225729389, 0.79545395046120815, -0.85056470249953853, -0.64291564621272868], [-0.5631216648588101, 0.78102322152396764, -0.7623357704744792, -0.8305384183864285, 0.79808242273094865]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_constData_rank3(self):
      arg0=numpy.array([-2.2094294651673216, -0.33881347637330084])
      arg1=Data(numpy.array([[[1.6503666090549887, -3.2665218422207598, 4.8997243648188284, -1.3996920326803739, 1.2542984267042403], [-0.044328305269526247, 1.2452138372073041, -2.5631885535194563, 0.38215161502184891, 1.8588067169032465], [1.1660693771893049, 4.3034483005110733, -2.1337201464330247, -4.9109240819294584, 3.3415386500305306], [-2.9604423992228943, -3.6017641326106875, -1.3691349346420054, 2.1463230754201126, -1.2986062745183666]], [[-1.3817378843702768, 3.6493558386651372, -3.0955197560003711, -2.1567233960284882, -4.001406792590382], [-2.4437798605462548, 1.9364850851250139, 4.2965554740128411, -0.45596090365092845, 0.78551483655038279], [-0.44962686576891286, -4.7885612089396261, 4.9642450896324828, 1.7627742137902578, 0.13164890034905774], [-4.1337404906586164, 2.5449280550092093, -4.0798712959221097, 4.6471528627439724, 3.8635084909890924]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-1.3387507072943363, 0.67638594562870913, -0.45092933819533687, 1.5785111392942071, -1.7614862764141044], [49.842407728729633, -1.7743373861974634, 0.86198475805987973, -5.7815520812099699, -1.1886278681240237], [-1.8947667337709642, -0.51340908752288994, 1.0354823095525725, 0.44990096126659251, -0.66120122990261831], [0.74631732937863915, 0.61342980379057976, 1.6137412093315933, -1.0294020925693383, 1.7013851761857268]], [[0.24520821221292211, -0.092841994958001176, 0.10945285544262577, 0.15709639771016118, 0.084673589548730666], [0.13864320671566804, -0.17496312208953987, -0.078857000316316234, 0.74307571912500514, -0.43132664159624617], [0.75354366513195647, 0.07075475525733696, -0.068250755201609947, -0.19220469287714137, -2.5736141773684467], [0.081962928524165465, -0.13313283088942754, 0.083045138387563791, -0.072907753710783674, -0.087695802186929211]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_constData_rank1(self):
      arg0=numpy.array([[[[4.460354262987039, 3.2194614286214449], [2.9506883133479276, -2.4125338784544392]], [[0.34559416461620973, 2.3464014789764329], [4.2762737182214732, -0.3467544458040841]], [[-2.0738707275482895, 1.9414523666261507], [4.6323182065481774, -3.1066241883513488]], [[-4.3081996219035847, -1.6834879456771112], [2.2017039150587276, 4.7042658299587856]], [[3.9241243098201686, -3.7380957222347844], [3.606387813250798, -0.20224425359260501]], [[1.7699113727363098, -2.0845570497508792], [1.5742045509072007, -3.0285774987877465]]], [[[-3.7441991546983031, 2.3122549265917005], [-4.7328963354350551, 1.5574012147684435]], [[0.30057640204820046, -3.0730815363774475], [-2.32288744329189, 4.4091234012737619]], [[0.30929562851234405, 1.3566950284964232], [-2.9614167161935723, -4.2147094141983388]], [[3.530591863656138, -2.0333769686825254], [4.3935843787045652, -2.3567460025955045]], [[-4.3936829227131629, -4.7563646421358863], [0.0060851274652105047, 4.3615912172551106]], [[0.22691512591031326, -1.7547428276066368], [-3.5788788192929832, -1.7788015738100862]]]])
      arg1=Data(numpy.array([-3.8500003988483567, -4.9541182511429893]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.158533454781266, -0.83622366106363943], [-0.7664124695235256, 0.62663211130474061]], [[-0.089764708782780034, -0.60945486646658731], [-1.1107203312240246, 0.09006608048866907]], [[0.53866766563677304, -0.50427328973963059], [-1.2031994095205376, 0.80691529000376916]], [[1.1190127728797867, 0.43726955097996606], [-0.57187108752438554, -1.2218871019769151]], [[-1.0192529618942336, 0.97093385324140591], [-0.93672401029609509, 0.052530969517068607]], [[-0.459717192046458, 0.54144333345379103], [-0.40888425657776284, 0.78664342468475568]]], [[[0.75577508749098998, -0.46673389882412047], [0.95534585480334566, -0.31436496583607543]], [[-0.060672028161389363, 0.62030847480647877], [0.46888009642401351, -0.88999155404829322]], [[-0.062432023789699594, -0.27385196713530474], [0.59776867770774933, 0.8507486500197976]], [[-0.71265797154550714, 0.41044175080265694], [-0.8868549671156718, 0.47571452337694359]], [[0.88687485844720693, 0.96008298571365791], [-0.0012282967738621448, -0.88039707494846864]], [[-0.045803332582535862, 0.35419881776172607], [0.72240480300753462, 0.35905513022417057]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_constData_rank4(self):
      arg0=numpy.array([0.57850987683604416, -1.1514326521835039])
      arg1=Data(numpy.array([[[[-2.04677552899313, 0.44765376017781833], [-4.0821149586103544, 3.3088549904839031]], [[4.1308617347183052, 3.8966656123684213], [1.4880315118154446, -0.33935099248035172]], [[-0.086712277090975398, -3.5228795086792775], [1.333339481803467, 3.1611529826100302]], [[-2.5403415236842406, -0.11661127580044539], [-3.3127552474365118, -4.5162715693424307]], [[-4.0831078444425302, -0.35012366419519747], [0.54848901839136932, -0.085327100260049527]], [[3.4694808721482975, 4.0298974553668021], [-3.9650797554226589, 0.79757982447326192]]], [[[-4.5563621415327891, 4.0849345548610447], [-4.6559400491012104, 0.43457151349953804]], [[-3.0311899513875273, -3.5892200803281282], [-4.5317994585538468, -4.7003065089134433]], [[-3.7113173394066146, 3.4517004898159502], [-0.78392422488369107, -4.9709286942112332]], [[1.4604046360583665, -1.6449708912391636], [0.55722778786777472, -1.5981238629129191]], [[-3.0718517712558748, -2.1839516816394822], [2.2084884489985521, 2.233680363154499]], [[2.0522175754815155, -1.450119040588759], [-0.12822308087230461, 0.35545176849759486]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.28264451506347177, 1.2923154640904766], [-0.14171817371673975, 0.17483687816474547]], [[0.14004580980619394, 0.14846279726948952], [0.38877528617001139, -1.7047537495253964]], [[-6.6716028715183251, -0.16421506197154234], [0.43388040685149082, 0.18300597282653275]], [[-0.2277291739880058, -4.9610114705033919], [-0.17463103478100556, -0.12809457269202154]], [[-0.14168371222999832, -1.652301560837999], [1.0547337456868713, -6.7799078495921261]], [[0.16674248919488399, 0.14355449071430232], [-0.1459011955673456, 0.72533163337989892]]], [[[0.25270876554956068, -0.28187297414919577], [0.24730401165834989, -2.6495815220634058]], [[0.37986159582524198, 0.32080302305626224], [0.25407846545595825, 0.24496969506137109]], [[0.31024904282844246, -0.33358417266525348], [1.4688060601193171, 0.231633306976295]], [[-0.78843398860415959, 0.69997144527957256], [-2.066358995823677, 0.72049024415715446]], [[0.37483340275652688, 0.52722441703432232], [-0.52136684378205622, -0.51548675950994227]], [[-0.56106753296532952, 0.79402629712110651], [8.9799172220030954, -3.2393499040680536]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-0.25226245126182256, -1.8217373720941055, 4.9615454415181954, -3.5673510604899281, 1.6958602914438163], [4.8849063521974703, -4.3047279153653246, -3.0362975512546306, -2.8233537857882549, -4.2089355752172626], [-0.48454255689325798, 3.7285167207336176, -4.3978861009561694, 0.051910763686702666, -1.5750071756521935], [0.76632477933439436, 2.5336724311697045, -3.1138483567245192, -2.0605410520683232, 3.0086412606582371]])
      arg1=Data(numpy.array([[2.6604640715228003, -0.70548455235821983, 4.272617525240527, -4.4127499780361292, -4.8525442043135705], [-3.2028959032407145, 4.3542807463695858, -2.0971977124811794, 0.99109385275994466, -4.9520509238065973], [-2.0857829189669728, -4.4693098632714969, 3.6452155175177658, -4.2197595744176466, -2.6810616693803233], [1.2461067096893705, 3.8528164421827267, 0.56683994679895022, -3.7502399788906473, -0.35839533557892622]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.094818965594010901, 2.5822498394962623, 1.1612425901002887, 0.80841903081886335, -0.34947858691041206], [-1.5251530176971673, -0.9886197436751003, 1.4477879377726428, -2.8487249496361335, 0.84993786210540245], [0.23230728015225943, -0.83424887394233505, -1.2064817786002786, -0.012301829706462999, 0.58745652650959967], [0.61497524519824132, 0.65761566095640656, -5.4933467097881783, 0.54944245266081571, -8.3947556287201479]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_constData_rank2(self):
      arg0=numpy.array([[[1.2512793522961427, 4.8871294756580532], [-3.4072651279768094, 0.97286399543267521], [-4.2682220992300799, 3.1877773078016851], [-2.6802305589848818, 2.8822392836605815], [4.6544427354138183, 4.782856914942899]], [[-0.30375415479553336, 0.60028200206674764], [-4.5384103353121628, 4.9969049208689604], [1.3459644803677335, 3.5801914490311653], [-0.63655038310076684, -2.8908286986491727], [4.2756095924235069, -1.0220962941571363]], [[-0.91862365999582174, -1.2989453727565858], [-4.8211559020570256, 4.6658293690729344], [3.6220078406128806, 2.7353129373993434], [-1.5497528669417981, -0.6971850010227385], [1.0360875160941863, -1.6293742132813591]], [[-2.1113510240283317, -3.6212641730326212], [1.8782587978949161, 1.4241070056260421], [1.2710191218397568, 4.7151861771004207], [-1.63352620785688, 2.3329948057073002], [-2.0103516152359271, 2.4037565321624346]]])
      arg1=Data(numpy.array([[2.2585516298220201, -1.310450473424817, 3.9745858152092737, -3.7500053238703157, -3.7514007246411705], [-1.7778456820371433, 4.4772823145992025, 1.0166134125746247, 3.6643014735170425, 3.6124769604343001], [3.924275612136757, 1.8857955767461441, -1.1582391855840521, 1.4946007033820035, 2.2619223164240747], [-1.2048969668353049, -3.5543420917492066, -1.4236369667961304, -2.5411990284563646, 0.85364839968265827]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.55401848502119333, 2.1638334103715717], [2.6000716525150622, -0.74238898391186714], [-1.0738784612215866, 0.80204012594299445], [0.71472713436541524, -0.76859605113463469], [-1.2407212870757831, -1.2749522820973462]], [[0.17085518606287406, -0.33764572939700532], [-1.0136529297055132, 1.1160575924764471], [1.3239688397962512, 3.5216842555363796], [-0.17371670636308148, -0.78891671974645661], [1.1835672972456781, -0.28293503470102621]], [[-0.23408744715961316, -0.3310025852259938], [-2.5565633738391278, 2.4741967934422746], [-3.1271674155856259, -2.3616131896107784], [-1.0369009351025966, -0.46646907059868131], [0.45805618900836564, -0.72034932475367874]], [[1.7523083567666815, 3.005455464415328], [-0.52844063666661989, -0.40066683759333727], [-0.89279721690576963, -3.3120706240944719], [0.64281710702886397, -0.91806851001531486], [-2.3550112856572687, 2.8158625179359853]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_constData_rank3(self):
      arg0=numpy.array([[-0.59029181070011738, -1.8511059271793782, 1.8752413180436402, -0.80087004844485499, -1.3613665235352199], [-4.2965526910302234, 2.0316754851279413, -4.1197476680605778, 0.64263416095420656, -1.1428591139334312], [0.67243778504434992, -4.7073096418191991, 4.4343808012389676, 1.0841660035212106, -0.634971070286098], [-0.64297732080747494, 0.68595698006839889, -0.79784513653338607, 0.24945894477176722, 4.8753105435535673]])
      arg1=Data(numpy.array([[[-0.094414993985702189, -3.5820088768128966], [3.9710986938727917, 0.088308742733759438], [-2.1037402336141806, -2.9838514194191368], [-0.19297714466983251, -0.86142066556719943], [2.1543306402651519, -0.69233559916516452]], [[1.0965435168703719, 3.2057591444516689], [0.83783897890447001, -0.32095870260486947], [-2.4927550784670949, -0.58068055826174003], [0.35142257011561728, 3.7398935973747811], [4.433981937205699, -2.9966994617692979]], [[4.2608525202594532, 3.9313549194238462], [4.8317267644712896, 0.83241532573198462], [-4.6920100829009783, -4.4253313464324071], [-4.9394961666636625, -4.8686528771070261], [-4.9677984429252708, 4.348932080430771]], [[4.8168069031652259, 0.37507263456760498], [-1.1706939881315046, 3.00117351769069], [0.60701741306771062, 4.0505461611128695], [4.1196118820032694, -0.75560165786413247], [0.29283182718245726, 4.9856799914313381]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[6.2520981655679471, 0.16479350861501252], [-0.46614452822226321, -20.961751576061349], [-0.89138444380179771, -0.62846336980434891], [4.1500772011891645, 0.92970842290801659], [-0.6319208844226738, 1.966339048832372]], [[-3.9182692022045318, -1.3402606051881449], [2.4248996958633886, -6.3300214907371624], [1.6526885066438186, 7.0946884813794879], [1.8286650192751743, 0.1718322043721521], [-0.25775006080734342, 0.38137261627786639]], [[0.1578176625093336, 0.17104479214583304], [-0.97424996720283197, -5.6550011710558374], [-0.94509191644730572, -1.0020449214077169], [-0.21948918815610716, -0.22268295376305952], [0.1278173979039692, -0.14600620532643563]], [[-0.13348621477538594, -1.7142741473227407], [-0.58594046524764853, 0.22856291914644827], [-1.3143694387633478, -0.19697223653271037], [0.060553991957723277, -0.33014610565693514], [16.648841044576297, 0.97786270918561602]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[4.6278621436348359, -0.60383965791597571, 0.27601094110139979, 1.0181097751040715, 4.8454655473080859], [4.6164800517009024, -3.8819192573976635, -0.50094031278580342, 1.1688197559209463, -4.7446425049635792], [3.6904476055626105, -0.60572188187032783, 4.3749942104115629, 2.186404477671636, -3.5090226662530832], [-3.4974757307091897, -1.4597831381324031, 1.736313419274035, 0.47175843367402859, 1.3737755767743307]], [[0.16485101874499808, 2.7760610186384396, -0.86608425534835476, -4.3184485194986273, -3.4654017798308114], [-4.5295933480097972, -2.5396474460966099, -1.9265072074218423, 4.341118781430449, 0.83254287196637211], [-2.325116855360283, 0.26850015858831888, -3.8077679750480788, 0.10391670958792876, -2.0358856621814803], [2.4604642817333788, 4.0971390500423404, -0.71122737580408124, -3.0423419018674389, 1.9408192881054651]], [[2.4872077985417542, -3.0110876312666166, 1.6226145647446257, -1.9109959175022961, -1.9686711161403716], [-3.0889136309292686, -3.8125550315010615, 2.9023360196236201, -2.5956563546280864, -0.375151399471358], [2.3074943796469194, 3.1549305410348101, 3.1856821640103501, 0.51025666247342727, 4.0451798456472901], [-2.9070075180343302, 3.4787957597981567, -4.5355703581778952, 0.20637020410209317, -2.3877042031398688]], [[-0.38376211282952255, -0.14077564278773824, -4.8409037369791541, 4.4664906368543029, 4.4660372388722944], [-4.627403956578064, 1.141631293360355, -4.6771424503067847, -3.463116486702984, 3.3942697771709067], [4.4423930976202701, 4.3993345542264546, 3.5182217362992478, 1.0198151955270296, -4.1499321071195769], [0.64546618207822881, 1.9454848167034546, 1.8573953947987167, 3.4718391257325276, 1.7828101732293336]], [[-4.742873248791911, 4.8616534820020565, -3.9901254742526326, 4.5223196683457516, 1.9885071722934775], [-1.0245719969303604, -3.3825928806254204, -2.9916011465904369, 1.6105398962371407, -1.8384277192188847], [2.0298377033719959, -2.7508690673639045, 2.0936531023787346, 2.5010195466807881, 2.2412674430817079], [3.1111768084812805, -0.24894082214299829, -1.9614728757982149, -1.5133541079819457, 4.4209321120456337]]], [[[-4.2583357284382783, -0.16896887406252947, 1.1881791974531843, 2.3458541294502631, -2.5312125079849999], [-0.7251795879121925, -2.5867464208112034, 4.4153231504010737, 1.5644158505836447, 3.7741623589910525], [-4.7839239026381941, 1.5315116330105489, -1.8305876037807267, -2.5162345558645907, 1.6550906406837189], [-1.0954643422265842, -4.7218014068858691, -3.4153974859761638, -4.0533799695634309, 4.0962743642809336]], [[-1.4147302814836813, -2.4749216880383185, 3.3976941035298456, -2.4165777272678088, 3.8299145816005602], [-1.729133520819933, 0.65895152205052021, -1.98991889619625, -2.3389061529436237, -1.6135988939566923], [1.4322327628930474, 3.8211135267730452, -1.0847143729531994, -0.35465876306188981, -3.7170365626154753], [2.637930773090349, 1.1095169315526849, -0.82523584879338685, -4.9162018482115011, 4.2569091692438903]], [[-2.6033468361035617, 1.296755414547186, -0.24440716063659274, -2.9275922967796753, 4.456676239719437], [0.88132844541104127, -3.203636328058852, 0.68189590423114765, -0.042961423722003289, 3.0235734460004551], [-2.8269075097262286, 2.0072460237421232, 1.8379047871424357, -1.395003056365872, -1.8685475744431979], [2.3575053697899211, 2.220039999617053, -3.1655033494579232, 2.5278990169251978, -4.8327151573503873]], [[3.0958060656977242, 2.8644864391679956, 4.5637571194336353, -1.3880401313243729, -4.0177500223017528], [2.9959183891733367, 3.5127138974435095, -3.3431051329751549, -1.9126197635495581, -2.731536703823755], [0.59134161948732089, 0.59179185329392325, 4.7429365384781725, -0.36994653645472919, 1.798544692347499], [0.16036168829803543, 3.8618707949535356, 2.6794703532880586, -4.5759700053972177, 4.7293483269414907]], [[-2.0158258687973163, 3.1161411994760488, 0.7237627107829061, -0.49914500554748997, -4.5973884632245845], [-1.4325678952679177, 3.066462626773049, -4.6289957881180666, 2.5265612124966665, 3.5646416320503391], [-1.2755173594374911, -1.302110427520804, -3.7339505221171878, -0.36059452384931312, -0.11579740028483076], [-3.7729057741822061, 0.38216496632666619, 0.71229235474119523, 0.57930834959446109, 2.3249516960818086]]], [[[-1.1761806485875415, -0.052218690430071213, -3.6497572087706853, -3.9783624654205916, 3.757711185513795], [-1.2915472853502674, 4.341701466575083, 0.84379094381728859, 3.899920688390349, 0.93192520699695525], [-3.3107782988655909, -0.89741467742713255, -3.430306571970382, 3.6256825078056281, -2.866079986561143], [-4.3359467603682162, -3.5994401657704502, 4.1707322010014796, 0.53575834449041704, 3.3680451022375113]], [[4.3890398939444211, 3.303800893429429, -3.0720976305652758, -2.1161565062812282, -2.3926699858490927], [2.954794512848947, -1.658256120455639, -0.53549359075601632, 1.2354834693775754, 2.4482181906348908], [1.8227413340617424, -3.8122464521971757, -4.0879717231120285, 3.4327287161079525, -2.5472475885696024], [-1.5251107794285055, 3.335040276329801, 0.63784145120750324, -0.63578315731296975, 4.6452153908561868]], [[1.1000314146691546, 2.0716745034683681, 3.2245143233036195, -3.4423557567246865, -2.4766732096196553], [-2.2640039097003664, -3.7068907752692337, 0.81275326092717481, 3.0933874289117345, 2.5660577633240047], [-3.3203761857586889, 1.010209832275395, -4.0736559619185977, -2.6680004453960793, -2.8095878952042943], [-2.4462639497365868, -3.7806728833387817, 3.4457857401161824, -2.335399069975006, 4.5861965141270673]], [[-4.1620928302475342, 1.0425682733462915, 3.5798849492383038, 1.1853131137172817, -1.4553729558055215], [-2.3084288109220683, -0.042793453571196238, -1.1192285195979226, -4.0496969111429415, 4.0444246743929497], [3.2663715465785845, -4.1141743849385843, 3.980571492966714, 1.0422530762746351, -3.1720584145810715], [1.0824663138567772, -3.3294742095120142, 2.0513884233833979, -1.1405438654717628, -2.5543980606793082]], [[-1.908773033666229, 4.7148602316667425, 0.30952637678105166, -3.6534222069071651, 0.093897484819867927], [-3.559563617495983, -4.2392794846160626, -0.52477973181341486, -3.6010294589262637, 4.0467399695593489], [4.3552257061056547, -3.2801490494823438, 4.8891223229134457, -0.9538460266682236, 0.091784728472767974], [3.5197588285102306, -1.3230654258809524, 2.6648419032437083, 3.8846134753914967, -0.23498106518619188]]], [[[-2.3433003568399977, -1.9493453009610437, 0.046940933838772736, -4.8473069716821549, 3.2689825687676546], [4.007780244736221, 1.4607958018174134, -3.738411686765116, 3.555289407028404, 3.9235459542016908], [-1.9743198861067821, -0.87325137921226403, -0.71215274448057286, -1.592173609307924, -0.17243784905847193], [-0.24787703701679131, -3.3363824097121753, 4.4828798487747861, -2.6846985308049831, -0.95935765409833618]], [[1.629378145516414, -3.5160635698190958, -3.9217478052320045, 4.5577368948035755, 4.1093914188641936], [-2.5316728094342311, 2.80512846103972, 4.9402459179764264, 1.7625473566147463, 2.1276699135244215], [-2.3761654443318294, 2.3129701945222862, -3.6065910384190545, -3.6087255041406765, 0.91652609644609129], [-0.54848253036599104, -4.9039814496118028, -0.49182231513424313, -4.9843806851098371, 4.8302351945891591]], [[1.825096857569763, 3.4271037512005567, -3.3332026031838344, -2.1221790170145702, 4.0761051644504516], [-3.3344583820101126, 3.7050165776663029, 2.2771923613142153, -3.6567673478784943, -4.4091854410409681], [-1.3226286939376664, -3.4980823632087734, 2.5171249501875161, -0.12744908114710274, 0.80013503956883092], [-2.3236073253950185, 3.6554857405836394, -1.6297751043665798, -0.29236969624598519, 0.94879245599576301]], [[-1.5480612874335078, 3.5057104764762563, -0.93987486230017048, 1.239173899203835, -1.1368274749226428], [1.0488863863100892, 4.3057043041063867, 2.4169898237053822, -2.2298683000003972, 4.9660289496341612], [1.8018167112561247, -3.7274124798730801, 3.2844765718603544, -2.4221502358487279, 1.4902216331970433], [-1.3522658162199019, 0.52247574305073563, -3.4076239586247481, 1.6468514745441816, 2.1703259407834627]], [[1.1227982027125396, 3.7449685330079046, -1.2022859301328737, -1.4988823935526141, 4.2510671466751475], [-4.84267417251393, -1.1986165495143308, 0.99001783809397725, 1.3409108123243012, 3.3232230362681303], [-4.9277906272711114, 2.7172851902161677, -4.7119952511413468, -4.7754781706220868, 3.6950203077029382], [-1.5564992918267526, 3.8272392437855345, -3.7948209593656514, -3.1382914622647307, -1.8924407378263073]]]])
      arg1=Data(numpy.array([[-4.4054756509923916, -3.6616540629282213, 4.0519034390616557, 2.0245803281034478, -3.0865220452480435], [1.8005681978622903, 3.2067649449922762, -3.5896287974830954, 1.9507149033667748, -1.4247960813843386], [-3.947777649525992, -1.7836837723458476, -1.3837436755523971, -0.023577899335468722, 4.3877347238284532], [-1.7230721955907979, 1.802714540215903, -3.3363284642471083, -0.93639095012001849, 1.0257861350817841]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.0504795645828502, 0.13706571225287623, -0.062651791308669394, -0.23110098789780595, -1.0998734146258602], [-1.047895940739334, 0.88115780563290824, 0.11370856462978293, -0.26531068345767711, 1.0769875674820233], [-0.83769560835758761, 0.13749295873054729, -0.99308101031634066, -0.49629248936584625, 0.79651391682590045], [0.79389287509087403, 0.3313565330462257, -0.39412620947817689, -0.10708456272315539, -0.31183365557016973]], [[-0.04502091566049439, -0.75814398928183457, 0.23652814833516742, 1.1793709742326581, 0.9464033795315806], [1.2370347581080572, 0.69357929570923371, 0.52613031551135014, -1.1855622368539263, -0.2273679756903601], [0.63499085806617384, -0.073327560161049063, 1.0399038001976109, -0.028379717964079509, 0.55600163947584513], [-0.6719543243158661, -1.1189312205986663, 0.1942366383009195, 0.83086546396315797, -0.53003895363981868]], [[0.61383688825460869, -0.74312916794580075, 0.40045736260694126, -0.47162918520705088, -0.48586328518141553], [-0.76233643703133325, -0.94092939006068155, 0.71628953238227866, -0.64060173043738455, -0.092586460934576467], [0.56948404974361666, 0.77862925128478189, 0.78621867769585696, 0.1259301141173278, 0.99834063335528977], [-0.71744244692749592, 0.85855840646680859, -1.1193678295621594, 0.050931668833126144, -0.58927964080329021]], [[-0.1895514381437346, -0.069533246388703021, -2.3910652838921607, 2.2061315991538586, 2.205907652503921], [-2.2856114387483184, 0.56388540257614428, -2.3101787493352557, -1.710535481665528, 1.6765300591212073], [2.1942291130437588, 2.1729612271534759, 1.7377535914294833, 0.50371683522300881, -2.0497739948935889], [0.31881480478617402, 0.96093239161614941, 0.91742242528784035, 1.7148438506190655, 0.88058258221811547]], [[1.5366400042708126, -1.5751235243846631, 1.2927578082248794, -1.4651830124810667, -0.64425497149937716], [0.33195032528854729, 1.0959237714932906, 0.96924664808283312, -0.52179763261911583, 0.5956308402362771], [-0.65764561976710945, 0.89125203936226383, -0.6783211238040846, -0.81030347751162668, -0.72614658512882646], [-1.007987878548023, 0.080654153281122132, 0.63549614972556734, 0.49031048079241157, -1.4323345329258299]]], [[[-2.3649955239095926, -0.093841974029718153, 0.65989124925334142, 1.3028410322004793, -1.4057854131768854], [-0.40275041443759585, -1.4366278510762862, 2.4521832361824059, 0.86884565241182454, 2.0960952012103156], [-2.6568968108610762, 0.85057130012005289, -1.0166721848992317, -1.3974669545157852, 0.91920463920706341], [-0.60839925059609801, -2.6223952041870944, -1.8968442795063616, -2.2511671451132886, 2.2749898443970085]], [[-0.4411705584136878, -0.77178144656445336, 1.0595394928573505, -0.7535874218163594, 1.194323452855939], [-0.53921430179039764, 0.20548793982532054, -0.62053781001433606, -0.72936625947470468, -0.50318589657670676], [0.44662854542227665, 1.1915789252779949, -0.33825814849544955, -0.11059705626872632, -1.1591234862474238], [0.82261432263994727, 0.3459925970830261, -0.25734216974090518, -1.5330720936963911, 1.3274777672406368]], [[0.72524123885147251, -0.36125056035220665, 0.068087029168019064, 0.81556964854761205, -1.2415423686271629], [-0.24552077530384023, 0.89247008779991799, -0.1899627907791652, 0.01196820789718595, -0.84230810944030321], [0.78752084664251176, -0.55917927367574172, -0.51200413492088692, 0.38862042151656251, 0.52054061293283271], [-0.6567546403246235, -0.6184594911801623, 0.88184698977160203, -0.7042229599611135, 1.3462994170146212]], [[1.5870110288051911, 1.4684290534839948, 2.3395305544428675, -0.71155458387523918, -2.0596295313925395], [1.5358053521827439, 1.8007315632750085, -1.7137845859511445, -0.98047119045870434, -1.4002746885817838], [0.30314097588874389, 0.30337178040345042, 2.431383761046912, -0.18964664483581464, 0.92199259319922022], [0.082206624874431547, 1.9797207620079498, 1.3735837813426819, -2.3457912775974936, 2.4244180012051078]], [[1.4148171062056336, -2.1870787266963787, -0.50797634850293449, 0.35032732899048846, 3.2266992612428722], [1.0054546850494059, -2.1522115808977098, 3.2488830146279688, -1.7732791699159138, -2.5018609179405638], [0.89522800918864998, 0.91389248225308661, 2.6206911788311937, 0.25308500532859257, 0.081272963757958588], [2.6480321103328928, -0.26822432439268989, -0.49992582380569761, -0.40659035855264453, -1.6317785586712694]]], [[[0.29793487703867139, 0.01322736361211759, 0.9245093145529385, 1.0077473501827723, -0.95185482038609293], [0.32715806208218007, -1.0997836889563397, -0.21373821393375644, -0.98787749326728436, -0.23606324614276367], [0.83864355918401701, 0.22732148492073376, 0.86892091614690037, -0.91841102252579054, 0.72599833146764792], [1.0983259811729345, 0.91176365168454554, -1.0564759647753357, -0.13571137790770596, -0.85314964550800143]], [[-2.4606603266744345, -1.8522346531663338, 1.7223331165506679, 1.1863966803365165, 1.3414205045451104], [-1.6565685906100325, 0.92968055558118923, 0.30021778470953464, -0.69265835600034886, -1.3725629108656785], [-1.0218971335174101, 2.1372882970076152, 2.2918702218923315, -1.9245164245640527, 1.4280825043441072], [0.85503428526611824, -1.8697486225059141, -0.35759783269689854, 0.35644387596620153, -2.6042819152561658]], [[-0.79496761871740207, -1.4971519220431817, -2.3302829709529678, 2.4877120073199119, 1.7898352515547762], [1.6361439981263619, 2.6788854328742824, -0.58735824798094904, -2.2355205545397268, -1.854431430228306], [2.3995601529547579, -0.73005560937585812, 2.9439382697032923, 1.9281030819027958, 2.0304251031772149], [1.7678591728775392, 2.7322060798792949, -2.4901907781010153, 1.6877396523909702, -3.3143396390203814]], [[176.52517601457447, -44.218030559572988, -151.83222637026907, -50.272210295435038, 61.726150201013617], [97.906466478523427, 1.8149816046937295, 47.46939087632137, 171.75817291962471, -171.53456365422844], [-138.53530800621047, 174.49283018820708, -168.82638424783917, -44.204662232430195, 134.53524291747561], [-45.910210169928099, 141.2116559723969, -87.004715483599156, 48.373430102656307, 108.33866174145015]], [[-0.43502471179496405, 1.0745545317636844, 0.070543548382747026, -0.83264427702671717, 0.021399991277945597], [-0.81125315032494449, -0.96616585811211986, -0.11960151760394622, -0.82070354877430662, 0.92228455553220534], [0.99259093364367457, -0.74757232511547511, 1.114270262594069, -0.21738917384594283, 0.02091847712996709], [0.80218131907462198, -0.30153724168778628, 0.60733888235580014, 0.88533462478835423, -0.053554072881863431]]], [[[1.3599548311651208, 1.1313195732304544, -0.027242580989288074, 2.8131769429545788, -1.8971825888275118], [-2.3259502735821549, -0.84778560385076807, 2.1696198779896796, -2.0633432633444504, -2.2770641672715319], [1.1458137918764557, 0.50679906590498269, 0.41330406601819353, 0.92403186203234378, 0.1000758119710401], [0.14385760367504541, 1.9362986752671811, -2.6016784788508063, 1.5580882435889274, 0.55677159468608139]], [[0.90384700914503668, -1.9504272536671239, -2.1754680054681947, 2.5282632347646765, 2.2795574824464611], [-1.404366999298194, 1.5560580438341405, 2.7404482560975825, 0.9777185002366785, 1.1802589184583823], [-1.3181041098427302, 1.2830485043102122, -2.0006445601681944, -2.0018285888504987, 0.50841443611827919], [-0.30425367862196406, -2.7203316666123274, -0.27282318090990698, -2.7649306498148509, 2.6794232180601725]], [[-0.5470375225724734, -1.0272081385050118, 0.99906308353725626, 0.6360821603017649, -1.2217337735570604], [0.99943947897905261, -1.1105071390212757, -0.68254441543066036, 1.0960453645572628, 1.3215681514247926], [0.39643239810206665, 1.0484826061627499, -0.75445957349872095, 0.038200399784636761, -0.23982501967155478], [0.6964564041866228, -1.0956612275310109, 0.4884936006246503, 0.08763216792923377, -0.28438220821578253]], [[1.6532211115828177, -3.7438534364593385, 1.0037205743815716, -1.3233509989016963, 1.2140521806377285], [-1.1201372526887963, -4.5981908555977808, -2.5811759750514391, 2.3813432837156232, -5.3033713632085604], [-1.924214144770604, 3.9806156599391875, -3.5075911097169175, 2.5866869340613317, -1.591452408853417], [1.4441252513670495, -0.55796752732794908, 3.6391039001263183, -1.758722117437276, -2.3177562112334482]], [[1.0945733855361781, 3.650827794342653, -1.1720629564144163, -1.46120360013748, 4.1442041389419026], [-4.7209393916479794, -1.1684858164112029, 0.96513084378455249, 1.3072030966936328, 3.2396841043315288], [-4.8039161953365914, 2.6489782784981037, -4.5935454672193128, -4.6554325578219542, 3.6021351637866901], [-1.5173721291355295, 3.7310303901508628, -3.6994270341381612, -3.059401326392972, -1.8448687042113572]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_constData_rank4(self):
      arg0=numpy.array([[-3.9626176303936846, -3.4282004102293575, 2.8313160974240761, 2.7547726695322563, 2.6333302762262427], [-0.42619255971332137, -1.916768306991071, 2.786266676299114, 0.34088995088184149, -0.040508308703145701], [-3.8997755227104722, -2.315285506682081, -0.97444214970210741, -0.29268836708568635, -4.0099271785206874], [-2.8327173697229457, -3.3743584862771989, -3.1453471741764885, 1.0941321485956115, 4.9811343210896837]])
      arg1=Data(numpy.array([[[[4.0269966661756822, 4.3104251847932655, 2.6942206542765152, -3.1903289778951249, -3.656995595866448], [-0.97938048156793389, 3.5762098360365755, -3.5641849517064883, 2.1285714544260834, -2.9471607580574211], [1.795708757593566, -2.2000806931424544, -1.8782304857377072, 2.8028411924731582, 4.0094538627000844], [-3.9457491323610006, -1.2063162090566291, -4.4880877969205502, 2.4209016591833699, 0.13355151086266392]], [[3.2052514859766639, -3.8333250589772758, -2.1654876767896925, 0.92269636240102315, -0.33892306843069964], [1.4819417079497939, -1.2027682462969214, 4.609774268481198, 3.8768401822798886, 3.2230517828714493], [0.66400154578427717, 3.2640545725790258, 4.8824946395193631, 0.52063343181565447, -0.22060323905330748], [-4.5025049491311133, 1.9801577333757168, 3.2633523826377662, -3.6752493841974401, 4.7904113698029498]], [[-0.79411727915285901, -2.888522018346583, -3.5094615165238494, -4.5121262257262913, 0.73050839940424339], [-1.839881152234637, -2.5394310260125987, 2.4077150542805184, -4.8786769581101508, -2.7051534795163343], [4.7441462604599369, 4.4006590964448762, -3.4241902462424099, -3.323076894227921, 2.1282243999975048], [-4.0208485466131823, -3.0688480102985229, 1.7523132793138689, 2.0382145811902683, -2.7294822027003307]], [[-2.6155337091526087, -4.7805751798011054, 2.6848289159898995, -3.5984813694457052, 4.1560596355940014], [2.8322921011154731, -0.95960680053377967, -2.3989775010457004, -1.9929694015915489, -3.4323850505861864], [1.531227971468482, -0.34372218820419409, 4.233711463803628, -3.7299891168497465, 4.8607541028194614], [-4.8060175781545205, 0.26463986068686918, -3.2420862774785872, -4.1755040807080812, 0.82120338423931827]], [[3.0987617158413023, -3.936840805553711, 2.9009623574372245, -0.41315092362698369, -0.34183713314202357], [-2.0094656318653934, -2.0979857671778457, 3.6459804920711996, -3.7429682200142511, -2.9758076489916596], [-1.6309280267604476, -1.7773231244407639, 4.1753440691295403, -3.7195559756147945, -0.83846604302465355], [4.93422688173548, -2.5782914943066526, 2.3108357665010795, 2.6531193840725802, 4.2431696913128327]]], [[[-4.5711325128727491, 2.5506810433097993, 3.3804630555863149, -1.1577423989093125, 0.68052635493383384], [4.5813174549801143, 3.6183204550119132, 0.18744769735005029, 4.815711930676299, 0.89742205375638928], [4.9352037433236511, 3.1614439740240474, -3.2918826139973199, -2.6122552559775025, -0.38732307269477939], [2.7875597614090255, -3.2385844952268803, 2.2622376434645117, -2.1822589643367696, -4.4335410017415757]], [[-2.9156732340566762, 2.1264641723333266, 1.2215430636548223, 0.609077966207618, 3.8344727735749515], [-4.3032517924209728, 4.4735580828950585, -0.8029003359118132, -0.52278021076713888, -1.3255721315811506], [2.1041526903942476, 0.25043880704677868, 1.3189548240133444, -2.2156647473521662, 1.2886436213853081], [-1.6607207344678088, -0.6453851593123705, -2.951661116827009, 3.5692695980817248, 1.3755621276192187]], [[-2.6945581561733434, -1.3362411216491377, -3.5558765669422332, -4.7227871277884681, 2.1478837503944774], [3.114316805079941, -4.0069165220489467, 3.33636031242904, -2.9216907570914543, 1.6590925265466963], [0.50487345098080283, -0.35319954254656771, 1.1053201538153932, 2.8312309891932399, -4.643578107324279], [4.8172102857043892, -2.5248577963459451, 0.37138389915237191, -1.2703199415193067, 4.8733486555172014]], [[0.71703812616002249, -3.1684303440789883, -3.4053833130257685, 3.1653033780278115, -4.3772163854492474], [-1.6577550209572891, -4.9346421057863576, 0.34500417982765974, -0.49282125301518143, 0.91300614464021734], [4.2582172831768297, 0.9699022281327796, -1.0111437489872088, 0.3546697688106768, 1.1032441474519841], [-4.4736819948324005, -3.1326003465021168, 0.81971388258370226, 4.202874308681249, 2.9277964316366418]], [[4.0852121917938025, -0.85959157276106168, 2.3416832455340275, -0.85528394009164366, 4.4664124122975633], [-2.616993820340312, -4.3270190042261483, 0.21341717413795536, 3.04359385939639, -1.3668579789428903], [-0.23143824254519529, -3.0734198022554282, -4.7847440501252763, 3.5697136706074879, 2.2498661479739557], [-0.98880853725040119, 4.9509327645186758, 2.6181631849976963, 4.4597504530373264, 4.6699677392425443]]], [[[2.6526418548276176, -3.7221292298491857, 3.7060666765304191, 4.8632891845734019, -4.8854703342344594], [-0.84478901940677087, 4.8558721154921649, -4.4384987690514848, -2.3420616235271918, -2.9990465846035708], [0.73848204313267107, -2.6108142935191925, -1.1919893399031367, 2.9542841908734205, 1.3991761494458563], [-2.816775404470433, 2.8844386403141051, 2.3597622615374565, 3.6717787145719285, 1.7249312242215673]], [[2.5245337126703271, 2.5269491186848336, -3.1809223957064736, -1.8037379282754742, 0.92169854613727953], [3.1601239516916237, -0.16845920995878494, 3.3005574021703925, -1.5887616063718224, -4.268920295969524], [2.9080314451829556, 3.4394283033875741, -4.409170642726612, 4.4356301999265835, -3.4714058540234505], [-2.5435910472751355, 0.53495122183444543, -2.3051972424897569, -3.3277224639270351, 3.7084216890762747]], [[0.27572976871511035, 2.6438567858788842, 2.2294532713376078, -3.3407262085310565, -0.49837231242610525], [4.5668700696839828, -4.3024957209001036, 3.2650808457926281, 2.4703487038120588, 4.905021916923948], [-1.0065927788198472, 0.20556857457883293, -4.7177051391779621, 2.0031365241613575, -0.6171107264181197], [2.8181137239722762, 0.30320472946696242, 3.391384255711964, -0.84598816376138686, 2.4481247596250002]], [[1.4353757887564766, -1.4116618014410545, -2.7813920061538875, 0.91662580118506476, 2.3682223367823907], [2.6048662460677887, -1.1264631544020243, -2.154769572646785, 2.1816589780718285, 0.26820120305074013], [1.0457138747800983, 1.625110413182874, -0.36894963791273572, -3.4396399502649864, -1.5804820418502152], [1.629565562828649, 4.5090242325985539, 3.4302829982554002, -3.8693930861840062, 2.3346908065279841]], [[4.7435143031929989, 4.7425847813597564, 4.3790064182746633, 2.1387822415374753, 2.9363781358294219], [-3.1091800662620974, 3.3410707900523988, -2.0085015312199417, -1.9346114061131656, 1.1815661097154386], [-3.0616085871133567, 0.64118416499419073, -3.9413678262464682, -1.0876707931508633, 3.5287230421973241], [3.9563121780758905, 4.5950785403151588, -1.6784870566214902, -2.9619188236477179, 0.42568431475465385]]], [[[1.6468800845251783, 0.061471943478278668, -0.25027369719891368, 2.5023358882331106, 0.72216207640937213], [-2.0749330163664084, -0.77350489417858803, -3.1631784844578128, -3.1351111119592314, -1.656006184896317], [3.8825338373456049, 2.1332427273014556, 0.48896341851975755, 0.15804551087124619, -0.93557130637148767], [1.2866499018486746, 2.0771760168660141, 1.8757148605368705, -3.5591351715452024, -4.8998060774627064]], [[-1.0863507550310225, 4.8353071982919573, -0.82872451866395203, -0.44947815627600196, 4.7339696118633796], [3.471854520064678, -2.9483312829154529, 3.9188934421769108, 2.5556950979252848, -2.350426933088019], [2.8181140513930849, 3.8561072392257874, 4.9753961558093707, -4.10402529920463, -1.3608553205790717], [-4.5327148386707563, -4.0036659523130211, 2.269672367264052, -2.3974994888640566, 2.0449030883102246]], [[3.2053404181710548, 0.38288059008272413, 3.1336696916554807, -3.0015184926657437, 0.08563364523835304], [1.1019841571278945, 4.1171866254501168, -2.8175256219174871, 0.63605735646774963, -1.1200539995186665], [4.2095193514027773, 0.21713268204456071, -4.0332310750865759, -3.7747071261860201, -3.3758115268972091], [-0.55542534170156621, -4.1360726068566098, 2.6125656433215099, 2.7167723475622143, -4.5358558938012621]], [[-3.5929981873792527, -3.7595354162646677, -0.59500130812638297, 2.9323349011273514, -1.5001915545339584], [-0.29522896174112834, 4.7390374804977409, -3.018474931257995, 2.3010044169824306, -0.72541693704384258], [-0.72685966126756618, 0.75475457353527364, 4.8520524660965521, 1.4546613261705303, 3.5780243652480479], [-3.608107113880199, 1.8283327489810368, 3.6425000224728112, 3.1771160472837572, 3.0342455459179849]], [[1.3672336518916453, 2.3920465312907799, -3.569738499410092, 1.951034420162669, -1.9937245633923952], [1.7492372101377178, 1.5520370047636423, -3.2442589416125758, 3.4531071140269951, 1.3083519688389158], [-4.7353445920442674, -3.5800886555256084, 2.6481334830508088, -0.29582163113640458, -3.4535178490498009], [4.1664293073008558, 3.0283209380847822, -3.6945900224268415, -3.3658416005800973, 1.4965559969409314]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.98401313904162324, -0.91931015166981433, -1.470784371021673, 1.2420717919216251, 1.0835718902348928], [4.0460451325819289, -1.1080495306688589, 1.1117878797216827, -1.8616324211968287, 1.3445542865485178], [-2.2067150998940379, 1.8011237691167297, 2.1097611078532243, -1.4137859972356008, -0.98831855062802521], [1.0042751065683162, 3.284891308467583, 0.8829189199713493, -1.6368354391274091, -29.671080505173727]], [[-1.0695573889375358, 0.89431508089846035, 1.5831077899790316, -3.7154155472213621, 10.114981037150409], [-2.3133166384608566, 2.8502584939235707, -0.74368075540472423, -0.88427694953711111, -1.0636504285932196], [-5.1629404057789978, -1.0502889378839753, -0.70214115187780979, -6.5846720566404429, 15.540118200172722], [0.76139847683919293, -1.7312764293706333, -1.0505149331922117, 0.93278035089800293, -0.71563799965896757]], [[-3.5653626633643345, -0.98019543539597043, -0.80676653215690997, -0.62749044591906011, 3.875815938233043], [-1.5388581452586145, -1.1149411298915228, 1.17593487335242, -0.58034506521637796, -1.0466378779847636], [0.59680202548173222, 0.64338455567062391, -0.82685712352900553, -0.85201642560302537, 1.3303653963498376], [-0.70415885219276264, -0.92259899738360096, 1.6157590830634454, 1.3891158092739457, -1.0373088692877348]], [[-1.0532353912673367, -0.57624293435897145, 1.026051475058912, -0.76553756618631319, 0.66283280584796822], [0.97263014236678258, -2.870730665935171, -1.1483111735443399, -1.3822453407123789, -0.80258264411849523], [1.7990610940122571, -8.01453256167372, 0.65067558171697615, -0.73854710650171196, 0.56673771420248586], [-0.57319238324344024, 10.409515264942632, -0.84969135111193861, -0.65974613275077976, 3.3545559144084698]], [[0.84980082939720436, -0.66889427495046216, 0.90774369046021885, -6.3737731798072028, -7.7034646646541152], [-1.3104629581456011, -1.2551707058377846, 0.72225572296748819, -0.70354064513436254, -0.88491279909121012], [-1.6146207760356486, -1.4816271954233544, 0.63068581478010488, -0.70796898702162625, -3.1406522638971253], [0.53368650030540155, -1.0213469974365297, 1.1395575204435511, 0.99254119209066238, 0.62060451685859697]]], [[[0.093235660640579129, -0.16708971152280488, -0.12607520115003937, 0.36812382453543158, -0.62626899990487395], [-0.093028384062324554, -0.11778740026273271, -2.273661217173693, -0.088500426489062978, -0.47490760665996956], [-0.086357642334397064, -0.13480946150402334, 0.12946772703896553, 0.16315119234159248, 1.1003541739667344], [-0.15289091398632262, 0.13159840675500556, -0.18839424803338845, 0.19529880123225865, 0.096129157155850187]], [[0.65740161983934176, -0.90138753896231394, -1.5691369088995966, -3.146999913534382, -0.49987792851219848], [0.44542322863071732, -0.42846617199851722, 2.3873053992614097, 3.6664897934418064, 1.4459932140431604], [-0.91094544409319123, -7.6536393444528903, -1.4532478839257639, 0.86509852597587611, -1.4874308731924821], [1.1541785847608597, 2.9699603087144171, 0.64938630524481344, -0.53701976113578609, -1.3934436464229758]], [[-1.0340347154562846, -2.0851526204046245, -0.78356675881330673, -0.58996236775207667, 1.29721484032243], [0.89466385428556094, -0.69536429345784068, 0.8351216341710318, -0.95364872874939199, 1.6793919758643989], [5.5187427084674692, -7.8886474659908643, 2.5207779544065625, 0.98411845834347178, -0.60002580163438135], [0.57839839057216835, -1.1035341001507049, 7.5023895291593146, -2.1933582125513436, 0.57173555049149494]], [[0.47541398210918084, -0.10758953609912252, -0.10010325403836909, 0.10769582253889293, -0.077878249751378206], [-0.20563349021557528, -0.069080987754332621, 0.98807484318632477, -0.69171114028911473, 0.37337092732949118], [0.080054616336421802, 0.35146836556722877, -0.33713302507510612, 0.96114746972925402, 0.30898867822607495], [-0.076198967936390485, -0.1088201216802142, 0.41586455728597799, 0.081108766488142678, 0.1164322584720426]], [[-0.0099158395699780397, 0.04712506495733839, -0.017298799391592207, 0.047362410077295775, -0.009069540598537755], [0.015478947022457251, 0.0093617126857038803, -0.18980810174611654, -0.013309367338249055, 0.029636077286152498], [0.17502858757335765, 0.01318020684106307, 0.0084661391035294967, -0.011347775323462306, -0.018004763856562824], [0.040966786973530719, -0.0081819549224041773, -0.015472033575012378, -0.0090830886458136677, -0.008674215961439604]]], [[[-1.4701477757403099, 1.0477270620903414, -1.0522680413190519, -0.80188024497509969, 0.79823952575930579], [4.6162715578961704, -0.80310507154186295, 0.87862489675622046, 1.6651037203868837, 1.3003384284629125], [-5.2807993897420511, 1.4937008474294247, 3.2716530191682547, -1.3200407512445584, -2.7871941100875528], [1.3844822404090995, -1.3520050203896175, -1.6526137341350864, -1.0620943760128323, -2.260829572767677]], [[-0.91711411698007639, -0.91623748557592077, 0.7278660774016974, 1.2836041591117893, -2.5119769542711614], [-0.73265654831124638, 13.743893891278111, -0.70148318134372911, 1.4572894368774343, 0.54235856988663955], [-0.79616935040962644, -0.67315998545505418, 0.52510680449652969, -0.52197442129427352, 0.66695903735905848], [0.91024282742399543, -4.3280310656035965, 1.0043763127971765, 0.69575679215442132, -0.62433177799118955]], [[-3.5340476809702799, -0.36856843188583621, -0.43707673187403034, 0.29168572605971721, 1.9552493696097735], [-0.21337198887498382, 0.22648300263695539, -0.29844349825449812, -0.39445530430518599, -0.19866213978371017], [0.96805994460298639, -4.7402291507762593, 0.20655003247445416, -0.486458180932062, 1.5790393976102757], [-0.345778149906804, -3.2138092021690707, -0.28732873547457677, 1.151838987166907, -0.3980361482277443]], [[-0.20391062004693139, 0.20733603954353927, 0.10523089389705129, -0.31931063549300331, -0.12358990224007023], [-0.1123621481630844, 0.25982950790881226, 0.13583279196121475, -0.13415862425225009, -1.0913014697786929], [-0.279893357202739, -0.18010368077848879, 0.79330167862886813, 0.085092733924996403, 0.1851893025896367], [-0.17961128644473137, -0.064911686428664378, -0.085324845569460031, 0.075641931581145072, -0.1253649375186239]], [[-0.84534944393895628, -0.84551512801232265, -0.91571621402203973, -1.8748646312109498, -1.3656031318282604], [1.2897056757930014, -1.2001922229423336, 1.9964770333459003, 2.0727300406943456, -3.3937391615661814], [1.3097452089071433, -6.2539398154928207, 1.0173948119781329, 3.6867103573723505, -1.1363677825006404], [-1.0135517618508234, -0.87265694010219508, 2.389012868882046, 1.3538275075284834, -9.4199552098409765]]], [[[-1.7200507774308675, -46.081467567783939, 11.318478135844797, -1.1320292303856603, -3.9225507157719575], [1.3652090681382854, 3.6621841581637407, 0.89552878019416915, 0.90354608451267615, 1.7105717331003223], [-0.72960532693247726, -1.3278926647537774, -5.7933114470985405, -17.923428220815811, 3.0277941942333979], [-2.2016224970389087, -1.3637348721158795, -1.5102068173155914, 0.7959004739044846, 0.57812846568610066]], [[3.1061408763699334, -0.69785813969982513, 4.0717493090674468, 7.5072802519132313, -0.71279682020370805], [-0.97191816845319234, 1.144497738714243, -0.8610487975918969, -1.3203290521692144, 1.4356364108897979], [-1.1973818038376212, -0.87506863189694073, -0.6782090070028356, 0.8222070382780432, 2.479586503612552], [0.74444535038668969, 0.84281718966282515, -1.4867161159232762, 1.4074490951720626, -1.6501312485500474]], [[-0.98128334711206799, -8.2149559305080295, -1.003726456094624, 1.0479186391362214, -36.730273076916397], [-2.8542580706189269, -0.76395545315670976, 1.1163508681904728, -4.9450684630765194, 2.8082102965822848], [-0.74719864944398817, -14.485830251620019, 0.77985791431723794, 0.83326919653090026, 0.93173068138298998], [5.6629522242190111, 0.76046710808757623, -1.2039303901193548, -1.1577514682077938, 0.69344071941856567]], [[-0.30451786823574101, -0.2910285520551632, -1.8388735178431057, 0.37312659893485112, -0.72932829496931062], [-3.7060461214337157, 0.23087644972174703, -0.36247846131344724, 0.47550197666742061, -1.5082804008606772], [-1.5052866555939575, 1.4496528897740781, 0.22549882884424668, 0.75215593410733605, 0.30579225765550661], [-0.30324270152250815, 0.59843163078788109, 0.3003794486877806, 0.34437903189939462, 0.36059446476491119]], [[3.6432209770422208, 2.0823735056699744, -1.3953779308800429, 2.5530735232617667, -2.4984064562128387], [2.8476036824631223, 3.2094172405691155, -1.5353689118951106, 1.4425079085602739, 3.8071821954073588], [-1.0519053522437125, -1.3913438465837047, 1.8809982023078073, -16.83830320979084, -1.4423363476925972], [1.1955403425089239, 1.6448502067419346, -1.3482238329160425, -1.4799075275055111, 3.3283982231680485]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[0.012252643974369981, -4.6445050230774987], [1.5698784609279235, -2.1351253291170802]], [[-2.1124514389462901, -2.8588282737043915], [4.9606998494956915, -1.1996984335792948]], [[-0.050369732081758656, -1.7666868306659342], [3.1072653935823311, -0.24684380783394921]], [[-3.8721882268193966, -1.7070977994771432], [-0.62212628173993778, 3.624445813231528]], [[-0.47579796191990731, -1.59775857983138], [4.3171966101484536, 3.2540114858425682]], [[-0.83476924627340043, 2.4860741135818918], [4.4183547181244105, 1.9699345780406388]]])
      arg1=Data(numpy.array([[[0.19963456649204048, -3.351996670977214], [-4.3977089898131361, 2.0176894620204653]], [[2.4328499751642543, -1.8847866801421151], [-1.5834958738831362, 3.0068693624726279]], [[0.6132600490057083, 0.050098760564765143], [-3.049328836560429, 3.1254975146141177]], [[-2.1247838724535892, 3.9057826906107316], [2.4865335785715859, 3.9970778642839004]], [[-0.43562589779932193, -1.5132268929339587], [2.7282480289079096, -4.9975962161783007]], [[-4.825752760840639, 1.6014203224686181], [-1.9800666191026748, -3.8477730592656609]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.061375362942762218, 1.3855935667512096], [-0.35697643126555073, -1.0582031424097431]], [[-0.86830320838162967, 1.5167914246342362], [-3.132751989640989, -0.39898588497132154]], [[-0.082134377028838232, -35.264082599050546], [-1.0189997734344891, -0.078977444928291743]], [[1.8223915745124686, -0.43706932379543401], [-0.25019822257832708, 0.90677388239492518]], [[1.0922168868369053, 1.0558618719321891], [1.5824062051559822, -0.65111532526550031]], [[0.17298218281037359, 1.5524182369245596], [-2.2314172035922293, -0.51196745434269364]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[-1.7628976558934006, 3.5645424296925921], [1.8445192529180021, 4.3666922180914032]], [[-2.6063683522707226, 4.5843203439888391], [4.3932419069768827, -3.3263914894557125]]], [[[-1.3619869814651189, 2.3263778575855119], [-0.69796993966785958, 0.4143570779490986]], [[1.245594342123935, 4.6878249558574154], [4.3442882059959658, 4.2322501278321312]]], [[[-1.0243372613125246, 1.9466321863555871], [1.9601380512147815, -2.4447014486372098]], [[4.0639261857941413, 1.9291896272055586], [2.3309618484711967, 1.3870187848190003]]], [[[-3.6691066563556851, 0.085266631883778032], [4.0298258907272739, 1.6999433152956378]], [[-2.5962053931280282, -3.9493825319906959], [0.54556213396199649, 3.7585801150583436]]], [[[0.14397023360269046, 4.2866314221709523], [-0.13780528207272091, 4.0862247679871331]], [[1.3211795764354832, 0.21494141018484569], [-2.4375894804572606, 0.79842059072608507]]], [[[1.1093080709359651, 2.4401699334879439], [-2.4593420971913882, 0.78404887994561445]], [[2.62940243152077, 3.8390965731053122], [-3.4227319794996589, 0.92301707676843314]]]])
      arg1=Data(numpy.array([[[-0.077173824075464026, -3.7096140387913712], [3.9877298658111275, 0.2766955234524282]], [[1.82915588928934, -3.64470207312525], [-0.19405027727523816, -3.4589174140931487]], [[-4.2999207136187723, -0.35921198900718387], [1.9298214577654011, 2.9713500399844257]], [[2.8995525537074549, -0.1119210577547971], [3.1919448927957532, 4.3347952476295184]], [[-0.24271407524005006, -2.7216333504961856], [2.8520103692628602, 2.9610980146308066]], [[0.76547804164840016, 0.40061253121223217], [2.7429616705352036, -2.6177592337428646]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[22.84320722748636, -46.188490364388613], [-0.4972267286110888, -1.177128448520244]], [[-0.65359701884936283, 1.1496065426328375], [15.877531563072813, -12.021847870724997]]], [[[-0.74459863669371307, 1.2718313792759082], [0.19150260451037801, -0.11368750302100734]], [[-6.4189258557832503, -24.157785403255421], [-1.2559676008150491, -1.2235765186494727]]], [[[0.2382223602561388, -0.45271350706309188], [-5.4567723550440315, 6.805734561906057]], [[2.1058560466520486, 0.99967259636517147], [0.78447904726950801, 0.46679750488982119]]], [[[-1.2654044334062009, 0.02940682408903179], [-36.005966808820205, -15.188770990888671]], [[-0.81336159624424775, -1.2372965902088366], [0.12585649443542621, 0.86707212229083297]]], [[[-0.59316804540610357, -17.661239538462574], [0.050633301523732945, -1.50138694003076]], [[0.46324501154494735, 0.075364876825605706], [-0.82320459113920363, 0.26963666409591414]]], [[[1.4491703361564134, 3.1877726083862825], [-6.1389544898896453, 1.9571251991871654]], [[0.9585997718326571, 1.39961728752711], [1.3075045005593759, -0.35259815527370175]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_constData_rank4(self):
      arg0=numpy.array([[[-3.605348627805911, -3.955969128414043], [4.7908445446210592, -3.9380141980525227]], [[2.6225825924712387, -4.1750382072668391], [-3.0436362408371864, 3.8985360063756627]], [[-3.867189557700855, 0.31737007564040454], [4.6665525889659918, 2.4694931303674563]], [[-4.8741774362335333, -4.6351889003204239], [-0.25892125822756284, -2.794003365042923]], [[3.7861584908388615, 2.0420828791550161], [4.9966032641197753, -4.0166163185419403]], [[3.0760593059169477, 4.9209482931593289], [0.71582082858857277, -2.192230667914953]]])
      arg1=Data(numpy.array([[[[3.1195711928882517, -2.3693285611081638], [-2.7388853125593871, -1.4228001162338568]], [[-1.9418843407091559, 4.2088354555884617], [3.5583160432244991, -0.40005024219246987]]], [[[-0.065336566121679596, -1.6418746088742733], [3.4846300131384336, 4.679152001576675]], [[1.7691935384982003, -0.36523039547550074], [-4.8885138505612202, -3.0110684215906991]]], [[[2.1664873431550316, -4.5921990954639416], [1.5123379605570477, -3.359362496100462]], [[-4.0250463070108635, 4.9748502889724975], [3.187191094804561, 3.6386434702344186]]], [[[-1.647180685518749, -3.4553109066272758], [-2.7066058552575658, 3.040126381662148]], [[-3.1241593517554511, 4.8455388710271592], [-1.473988258900282, -4.9496064703233813]]], [[[1.8268777103364258, 1.7405147282452083], [4.3456000520826681, -0.10233081415610989]], [[-0.48175461693987209, 4.6723600842083339], [-3.3503998209063734, 3.0523979610846492]]], [[[3.831978477995639, -3.9449484196734086], [-0.40316674638969552, -4.6974238082831636]], [[-3.1905266896249351, 2.826271918367631], [0.5724374078391854, -3.4488667967913367]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.1557192975833011, 1.521675248839125], [1.444371953171466, 2.7804110242030826]], [[-2.4671111683569644, 1.1382826901108263], [-1.1067072598992489, 9.8437990600137866]]], [[[-40.139584127930299, -1.5973099153225672], [-1.1981295550819724, -0.89226385589953672]], [[-1.7203523382867492, 8.333469170534439], [-0.79748899676904783, -1.2947351107737792]]], [[[-1.7850044542928691, 0.84212149284223481], [0.20985393735901853, -0.094473304387009974]], [[-1.1593786090951914, 0.93802874818366011], [0.77481803158680262, 0.67868510629549528]]], [[[2.9591030778135319, 1.4106335342746945], [1.7125466906519089, -1.5246698059263566]], [[0.082877097188424828, -0.053434977020971997], [1.895539769853718, 0.56449000173954789]]], [[[2.0724750591771288, 2.1753096537459795], [0.46991965544006503, -19.955698544914672]], [[-10.371676966706481, 1.0693960170166079], [1.1988468640304957, -1.3158888093067205]]], [[[0.80273397243241207, -0.77974639429420123], [-12.205739528931305, -1.0475844833250973]], [[-0.224358201082067, 0.25327387076117192], [-3.8296425738319595, 0.63563796373768366]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[1.2202956960887885, 4.975084425901473, 1.8326455390427876, -2.4517933820952531], [4.4247202410245414, 4.5901555926458695, 3.3486832081393949, 3.5813113479925267], [0.7585558118499991, -0.87210894004311701, 4.6105048229128229, -4.2210833540449748]], [[-1.4383369527300927, 4.2308807427798865, 0.24765193962077348, 1.7907168113795988], [1.202990189641068, 2.0303275466292199, 4.5448832759859066, 1.3920320773722308], [-1.2554533755392572, 0.42086332983618391, 0.19164060966007668, -0.29878836471421533]]], [[[3.5722392391501288, 4.1008228678666043, -2.5994977673349959, 4.5211717219935164], [-1.4086568050071624, -0.8386373065409094, -4.3756623632116547, -2.4298761288139428], [4.5057518300882879, 1.5395914936668547, 2.0486710561607557, 1.7603528534216837]], [[0.20921239474655717, 2.7504439195099648, -0.19870015729294455, -0.27416741217767271], [-1.9636221119340092, 3.3185055401523993, 0.10954116548836179, -4.7213945805329374], [3.8723156083224115, 0.57579966724129861, 1.4010825721523714, -0.35267533017901886]]], [[[-1.6773296497015044, -4.1648901049787366, -3.6493113417444523, -0.14477958658520507], [-0.87221635531205521, -0.97316579339138531, -0.7813342965506731, 4.7929386875951323], [-3.8186920407581026, 2.3755653343521583, -3.7684706069856566, 4.9751733752192049]], [[-4.4336354272551235, 3.6382572114468399, 4.664525166277949, 4.4943350809264206], [3.159545999994398, -3.7038993404488774, -4.2757117920436301, -4.6224643971774615], [1.9490051106345909, 3.4586621306541314, 1.273918540230178, 2.5026497346668428]]]])
      arg1=Data(numpy.array([[[[-0.98496959359348768, -3.9958170266473236, -1.4645731302117051, -0.47057893249429217], [4.4854902170653883, -4.9789342605791198, -1.7236601165536314, 0.55203359556704434], [1.8668580554706118, -2.0811501664380803, 1.1009994800979008, 3.2031394982054273]], [[4.1845647202783063, 4.9572876523779676, -1.526559564900527, 0.25396516896345833], [0.13369133724685689, -4.5712236890936442, -2.547493724466027, 0.21054917355307445], [-2.5762876331622531, 4.4998773224346014, 0.25268157672408087, -3.1712692414816415]]], [[[4.2340193153934198, 0.70300233921735078, 3.7494145102539136, -1.7420431716777673], [-2.4794797613064645, -2.3363859965059444, -1.225640474568749, -1.4697673345675986], [-2.7094855772518791, -3.6567199688006102, -4.6388550625558409, -3.8407941816683824]], [[4.3606061048946163, -1.1459814705883611, -2.7956895656989103, -1.7938974254614437], [-4.2836518908363672, 3.7331833969510857, -3.4696494510204801, -2.9057151348528052], [4.4260610959140507, 3.8645857661253551, -1.3235435586971169, -1.5392151999094716]]], [[[4.946491062654518, -0.34802757147996655, -3.353322772445372, -1.6880577462900312], [-2.9836276386369232, -2.1286413613694579, 0.9793704031043422, 4.7897695489187502], [2.0337982290889443, 3.2020949325188237, -1.4058811937502802, -0.063198544890705755]], [[-4.3243948888723862, -0.14548002568433205, 3.7051746660010281, 4.3049566598503262], [4.2318541146875042, 0.7417528782637115, -2.6740344765446169, 4.0890089027837249], [-2.0628964088429882, -3.5655771904778888, -2.2603199856003506, -4.4362407656653762]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.2389171239659846, -1.2450731334101652, -1.2513171935483189, 5.2101639338156982], [0.98645187636132992, -0.9219152839571676, -1.9427746665246932, 6.487488038320989], [0.40632752427381341, 0.41905142363453018, 4.1875631244647424, -1.317795667784639]], [[-0.34372438924410564, 0.85346686322516907, -0.1622288086982776, 7.0510330951613893], [8.9982658144841796, -0.44415405692644666, -1.7840606366708702, 6.611434535131683], [0.48731102823261097, 0.093527734131312087, 0.75842731450635703, 0.094217280830630162]]], [[[0.84369932516904467, 5.8332990362905921, -0.6933076511615025, -2.5953270248975295], [0.56812595407712707, 0.35894638462783462, 3.5701026965116056, 1.6532386260501601], [-1.6629547202308006, -0.42103073432003452, -0.44163290909805064, -0.45833043119665767]], [[0.047977824576203781, -2.4000771304773827, 0.071073755731269964, 0.15283338293835208], [0.45839908610095281, 0.8889211129736202, -0.031571248633242753, 1.6248649166953213], [0.87488977770712806, 0.14899389018311193, -1.0585844061918015, 0.22912672003223547]]], [[[-0.33909485096721698, 11.967126878102759, 1.0882672469621033, 0.085766963187958359], [0.29233418541146416, 0.45717696322752122, -0.79779243284670676, 1.0006616474224939], [-1.877615972981113, 0.74187848405965195, 2.6805043155410697, -78.722910216100161]], [[1.0252614622831595, -25.008637401132066, 1.2589218017385242, 1.0439907845861285], [0.74661033068898941, -4.99344114325371, 1.5989740706592153, -1.1304608297699155], [-0.94479058777736924, -0.97001465566100176, -0.56360097169685464, -0.56413749092165855]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(-3.120612559)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.17752912959)+(1.-msk_arg1)*(0.827093170783)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.982087789513)+(1.-msk_ref)*(-3.77298794045)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([2.2226480464519396, -0.65923039152497687])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-4.01369549685)+(1.-msk_arg1)*(3.58805209648)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.55376598653239073, 0.16424524282992758])+(1.-msk_ref)*numpy.array([0.6194581312335885, -0.18372932549457471])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(0.82382699172)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.2244477856127123, -1.1502779272164751])+(1.-msk_arg1)*numpy.array([3.2713551543007604, 0.71997153751979148])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.6728151264590605, -0.71619820934327505])+(1.-msk_ref)*numpy.array([0.25183049618946785, 1.1442493887432144])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[2.5356609751498436, 2.8461278614860186, 4.5666915580662408, 2.8866035902632312, -3.2465253916285897], [2.6780143367640266, 3.9329276057654798, -1.3288021247322424, 4.7011576114542777, 2.2768740402630421], [0.57814250446892679, 3.0285633106433547, -1.0101832496036778, 1.4508401115534557, -4.6027501529238455], [3.4133051041026548, -2.9122468632977005, 4.6045361443094173, -4.3370307695296519, -2.1131338880781225]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.780874707552)+(1.-msk_arg1)*(-0.561747617946)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.2472059225714491, -3.6447945284431658, -5.8481745072538427, -3.6966283609207089, 4.1575496814403463], [-3.4295057976185119, -5.0365667727846457, 1.7016841650532859, -6.0203737756992783, -2.91579944675152], [-0.74037806433916054, -3.8784241330300628, 1.2936559986303724, -1.8579678628615557, 5.8943516910054283], [-4.3711303120601563, 3.7294675254966139, -5.8966388586756047, 5.5540674164293806, 2.7061113231622849]])+(1.-msk_ref)*numpy.array([[-4.5138793546142049, -5.0665597335206956, -8.129436444723467, -5.138612960778187, 5.7793309449174659], [-4.7672909527527221, -7.0012359289506305, 2.3654788774914626, -8.3688073812300097, -4.0531974992410147], [-1.0291855025268206, -5.3913238149856495, 1.7982866634975396, -2.5827258811680491, 8.193626471891637], [-6.0762253279927458, 5.1842620605077698, -8.1968058202831848, 7.7206037568762804, 3.7617140163494502]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(1.98159459178)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.70133132351628369, -4.61955611368289, 1.1546938405634588, -4.4418757698297107, -4.0361539340284605], [-3.9658993779042415, -3.8984919229000714, -0.55503594982737692, -3.4817682053659094, -1.271960622583804], [1.8223632397162923, -3.6590305253925912, -3.4596760523878265, -1.969978330797252, 3.2988295671999808], [-2.4134389596046857, -4.0711247849459928, 0.58890078451662298, -2.1805851606178237, 2.1880748488830415]])+(1.-msk_arg1)*numpy.array([[-2.8675220946223168, 2.5703833753656093, -4.2095133926966692, 2.3072527665711391, 3.8799406718538147], [2.8278720993472648, 1.3483136549220101, -2.8687291490522702, 0.70207950842853162, -1.4184798404169974], [-0.39573476798329743, 2.1915803312806563, -0.4081786388235269, 0.88203476324025942, -0.10676435691508868], [2.4499312756069749, 0.71870415458629289, 3.7889142772921129, -2.7393218931682317, 0.23013744367765465]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.8254756708144311, -0.42895779226622277, 1.7161212108037063, -0.44611661704607142, -0.49096110410177718], [-0.49965831276902134, -0.50829772921556926, -3.5702094474991126, -0.56913455316221806, -1.557905611692618], [1.0873762972105614, -0.54156273855155435, -0.57276882626269932, -1.0058966440373949, 0.60069626254056696], [-0.82106679511791925, -0.48674376160190141, 3.364903976824996, -0.90874441758280178, 0.90563382362669176]])+(1.-msk_ref)*numpy.array([[-0.69104771520037467, 0.7709334766038487, -0.47074196158000797, 0.85885457392699871, 0.51072806503212576], [0.70073699310260185, 1.469683693064701, -0.69075694804788235, 2.8224646467901597, -1.3969846700061705], [-5.0073805793555666, 0.90418524180557602, -4.8547238961029331, 2.2466173379558438, -18.560450781821661], [0.80883680758938881, 2.7571770374918847, 0.52299800068096658, -0.72338873234188683, 8.6104831969494455]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[-3.7873893220711397, -0.8794596993530126], [-2.9030879587374536, 4.1486513350425831]], [[-4.4747689713854557, -4.6975226191666444], [-4.7648836143257602, 3.9717818661841235]], [[2.6488396857598655, -4.8731404131757881], [0.20802996790965977, 1.761304698434202]], [[2.9684688520448681, -3.5855211048428415], [-3.3313641481968057, -0.51755808635389045]], [[3.4797072292835853, -1.1512382598710325], [-2.349231749850238, -0.76880717818302458]], [[-3.0770033047844434, 1.3709599214518491], [4.666572646996233, -2.0606729749054908]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.42249892296)+(1.-msk_arg1)*(-1.74742786038)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.5634224998733155, 0.36303822099504113], [1.1983856550874485, -1.7125503321079525]], [[1.8471706752801544, 1.9391226863502815], [1.9669290950635079, -1.6395391669915176]], [[-1.0934327609630805, 2.0116171639926979], [-0.085874121940025055, -0.72706108627849519]], [[-1.2253746839319219, 1.4800919293970154], [1.3751767303695108, 0.21364636386379132]], [[-1.4364122915831585, 0.47522756314186165], [0.96975553945009363, 0.31736120536405654]], [[1.2701773675207579, -0.56592797976476794], [-1.9263466343664803, 0.85063937712279636]]])+(1.-msk_ref)*numpy.array([[[2.1674081133460907, 0.50328812953833146], [1.6613492462623336, -2.3741474135271088]], [[2.5607746521887682, 2.6882498131564736], [2.7267984689759639, -2.2729303774085707]], [[-1.5158506658960278, 2.788750553696981], [-0.11904924525159613, -1.0079412938127237]], [[-1.6987647498089329, 2.0518850512469298], [1.9064387284493753, 0.29618280564773386]], [[-1.9913309774802777, 0.65881876212145141], [1.34439412528396, 0.43996504554788474]], [[1.7608757274352553, -0.78455880928610899], [-2.6705380821762259, 1.1792606845923019]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(3.2470116659)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.3591539945213054, 0.10549803491210508], [-2.5869709220810089, 2.9137397051061296]], [[1.619096018356311, 1.8876766424454363], [1.9574770550971596, 4.0708785896579833]], [[0.19798811821138784, 0.57378311326188935], [-0.18390311836686557, -0.018723881080670246]], [[0.63399949072298334, 3.0941670616352699], [-4.0026516842255848, 3.8380133408772483]], [[-1.4390851473725497, 1.44556407229146], [2.9520047935070277, 3.0567526230908051]], [[1.0627882882531985, -0.1410603444721934], [2.4774071810029756, -4.878173579939185]]])+(1.-msk_arg1)*numpy.array([[[-4.8463617972094495, -1.395563750347899], [3.0417556462074486, -4.1966763685407713]], [[4.2921571294984844, 3.3907324364598264], [-1.3487737681916014, 4.0869280396355325]], [[3.2381853334035355, 4.6920614298624255], [2.0071710961222262, -2.1941360871815849]], [[-0.21372557622608124, 4.9547644638948682], [-4.4160071752103462, -2.7219424944720805]], [[3.1126563509654641, -2.8856717685164082], [1.0644336554299159, 0.82544134543170244]], [[-1.7453116073803643, 2.5811485490708073], [3.2700185050263109, 3.1845549082350537]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.3889946827099933, 30.777935044957665], [-1.2551403798862524, 1.1143794554487074]], [[2.005447255186124, 1.7201101040743008], [1.6587738065385804, 0.79761938224943352]], [[16.400032967779804, 5.6589529925977908], [-17.656099008707571, -173.41552490672061]], [[5.1214736185240559, 1.0493976573389665], [-0.81121514487305768, 0.84601364755894304]], [[-2.256302673836756, 2.2461900708063331], [1.0999344150921739, 1.062242211347892]], [[3.0551820167610639, -23.018600146233975], [1.3106491701460625, -0.66562036235207633]]])+(1.-msk_ref)*numpy.array([[[-0.66998953065474509, -2.326666671505333], [1.067479457116786, -0.77371028422295673]], [[0.75649878788916614, 0.95761365036681123], [-2.4073804981015825, 0.79448711462632227]], [[1.002725703313168, 0.69202241156306654], [1.6177054722283226, -1.4798588313937731]], [[-15.192433789303353, 0.65533118467209994], [-0.73528224413285337, -1.1929023748626999]], [[1.043164197964966, -1.1252186410531406], [3.0504594150435849, 3.9336673451912834]], [[-1.860419452987577, 1.2579716371087375], [0.99296430919412193, 1.0196123977950209]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[-4.366104149245893, -0.96220936219835096, 0.61428725897932157, -4.5042542952934586], [4.4582264077784721, -1.2817226210183872, -0.10277337844417822, 0.17499149000038905], [4.8362187909539145, 3.2429647289791248, -2.7715440756600018, -2.5365570965526016]], [[-2.749618991403012, -1.4273289012684129, 2.9789743908989239, -3.9957030479882261], [3.9456034410132705, -0.6158560665249011, 2.6110641679148401, -2.2714386960303012], [1.1782090447729674, -3.9542677685525076, 2.3918204907157943, 4.0896409723067393]]], [[[1.25953383498815, 4.7951260000317717, 3.9429483681443465, -2.3397750906527657], [3.6727499928426504, -1.9530647670403645, 2.139252645794695, -1.7701921796256324], [-4.4515236746177642, -2.1072222287692832, 1.401129713343698, -2.050579356190668]], [[-2.1160238183328142, 1.507251046505969, -4.6633716846380846, 2.9621205737944445], [-2.2206854031317347, 2.7047545834466931, -3.1497609585269917, 1.894457198777082], [-3.1359846930730715, 0.40918760280953226, 1.618172919802924, -1.8246153355525951]]], [[[1.8241906440214564, -2.6533179093654158, -2.2975011792909052, -2.7306898884227513], [1.0283337935331103, -0.99361559029740221, -2.88380272870545, 1.7628968147980553], [-1.0030437764655984, -0.81017883599384533, 2.322251462584779, 4.0415620687374325]], [[2.6308046666210503, -2.7266961255898456, -3.4890628563415738, 4.9663478463831119], [-0.52728103596659448, 3.5805869167490325, -1.8505243134967442, 1.2773455089098897], [0.60842702480555655, 1.0894648663649544, -3.748237362835849, 1.4634997625591142]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.99763921694)+(1.-msk_arg1)*(-2.5816094907)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.87363332159847118, -0.19253277806386779, 0.12291548715584727, -0.90127640267149012], [0.89206647664102112, -0.25646561614009933, -0.020564385299327293, 0.035014830483810259], [0.96770066445798386, 0.64889932790404425, -0.55457065933564997, -0.50755106290081775]], [[-0.55018357109152993, -0.28560062847882212, 0.59607631955534424, -0.7995181073584815], [0.78949345275653671, -0.12322939687951479, 0.52245951629811227, -0.45450233548894969], [0.23575312134964818, -0.79122713683478485, 0.47859006760803746, 0.81831456709493167]]], [[[0.25202576262785958, 0.95947822399392302, 0.78896218734227375, -0.4681760705578667], [0.73489698503918577, -0.39079747101798318, 0.42805263704182017, -0.35420567647729789], [-0.89072529676193168, -0.42164352753330603, 0.28035831570123881, -0.41030960162952712]], [[-0.42340467698437034, 0.30159260824531969, -0.93311491330372798, 0.59270396385440605], [-0.44434688194469718, 0.54120645089360764, -0.6302497683007392, 0.37907042036081917], [-0.62749321368446664, 0.081876178941162225, 0.32378746235183037, -0.36509544934095167]]], [[[0.36501047091159661, -0.53091425654974034, -0.45971729441841058, -0.54639596215087261], [0.20576391149795537, -0.19881699081628057, -0.57703299568536903, 0.35274591427540686], [-0.20070351878657477, -0.16211230959763892, 0.4646696893832134, 0.80869424408194979]], [[0.52640948104132512, -0.54559683226983435, -0.69814220372594926, 0.99373876960733465], [-0.10550602255948056, 0.71645566262815108, -0.37027969270456401, 0.25558978018666334], [0.12174288667002159, 0.21799590147927231, -0.750001590777297, 0.29283821801271603]]]])+(1.-msk_ref)*numpy.array([[[[1.6912333817250174, 0.37271685189627407, -0.23794739723138014, 1.7447465666375175], [-1.7269174225780304, 0.49648199142312727, 0.039809808111763295, -0.067783873057076255], [-1.8733347581733153, -1.2561794263092907, 1.0735721594016465, 0.98254871842262281]], [[1.0650793628202917, 0.55288334909348524, -1.1539213818484817, 1.5477565690651838], [-1.5283502230800605, 0.23855508307667644, -1.0114094239749971, 0.87985371304802562], [-0.4563854638040532, 1.5317063958747676, -0.92648423370549493, -1.5841439175998879]]], [[[-0.48788704857441567, -1.8574172497080677, -1.5273217666545278, 0.90632417454450809], [-1.4226590063589404, 0.75652989891690126, -0.82865075198365212, 0.68569324136908727], [1.7243210836720648, 0.81624360166102272, -0.54273495600029298, 0.79430268736599918]], [[0.81965294362201491, -0.58384161196226625, 1.8063815234030467, -1.1473929672430934], [0.86019415838592528, -1.047700898680777, 1.2200764561319739, -0.73382794942590923], [1.2147401473278885, -0.15850096782025805, -0.62680778236745094, 0.70677433675670021]]], [[[-0.70660983026078394, 1.0277766327270683, 0.88994915287132803, 1.0577470753273659], [-0.39833049779160712, 0.38488221935878858, 1.1170561384658519, -0.68286734347327283], [0.38853427680673691, 0.313827028802455, -0.89953630514321792, -1.56552030169572]], [[-1.0190560098654433, 1.0562000703108068, 1.3515068289428893, -1.923740931491974], [0.20424507961663643, -1.3869591546092035, 0.71681031548915675, -0.49478649405055031], [-0.23567740473449733, -0.42200994003572651, 1.4518994357365069, -0.56689432225617187]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(3.28837679378)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.8511021026743437, -1.8887577572043144, 3.8148442435067516, 4.1456615488140045], [-0.4780586985315729, 3.0134066087423825, 3.1662984180128344, -3.2083453164017408], [1.8350240774279536, -3.069583621372626, 3.0559577270096927, 0.96265959821024794]], [[1.211694564671185, -1.7803829317097564, 3.6564238559791047, 2.4281350013189691], [-1.8084097686577882, 0.63489127095123354, 3.2297646672425682, -4.486889497732852], [-4.2895064993646868, 2.4524196022008873, -1.6259025331818879, 4.5864780978683868]]], [[[0.32367393380389409, -1.5467381708760151, 2.5855706754937753, -3.9277068641153869], [-4.3696507037604251, 4.2029115240616619, 4.2682724119415383, 0.12389074533316524], [-3.5542815874761224, 3.0579524715988988, -3.3948517452347371, 3.7079509253695306]], [[4.3453246742226952, 1.7100950356184246, -4.3972139288727652, -0.6233374117199002], [4.0389817917621258, 0.61609558001832898, -1.7814551257294231, 3.5393372000738665], [-0.93373466883891076, 1.3612665921015648, -0.708163821666556, -3.3232630633546298]]], [[[-2.1379059809147352, 1.276948072534009, 2.8586782774655459, 3.8019143010899796], [-3.3608557657621918, 3.0026093751190377, 1.5824385532343737, -1.670012413711055], [3.02197945916358, 0.78172526975799705, -0.76803996640998484, 2.539132983429651]], [[4.4613698802625095, 1.6829427653585816, 3.4864278423724961, -2.264429660174232], [-4.3994633200060775, 0.80075769975621647, 0.74968537914510414, 2.4716561988118704], [-2.723187471682138, 1.5809732664296696, 2.388869500505387, 2.7676656528404173]]]])+(1.-msk_arg1)*numpy.array([[[[-2.811411151058155, 2.3059790678316086, -3.4827820004118215, 3.3412440275833255], [1.0569377097517929, -3.370748686005288, 2.1110474933294343, 3.6363472498461764], [-3.7379316072362179, -2.4821693384310861, -1.5409474970366297, -1.8270889022430312]], [[-1.4931584787164209, 2.226388973282849, 0.40439499541022705, -3.9437470963565779], [4.9372547095587223, -0.12577630461946576, 4.3973763845748692, 1.4492549918007827], [0.4988513343084815, -2.2297597141797256, -3.3831718742371129, 3.3013314228981407]]], [[[-3.5624934119480756, 0.70297936137734052, 2.8424453241515781, 3.3773019436469838], [-3.6592934903332788, -1.8336089406762923, 1.9885717087358215, 4.8059628853768182], [1.6253934482653776, 0.93152753551647205, -0.45361895914624561, -0.36493676715614853]], [[-4.3807632978283797, 4.0400563172701407, -2.2752850518649725, -3.2387813178935776], [-4.6650731972579749, -2.2580439941511257, -4.8112660037460087, -4.1667335335725744], [-0.99391559849668276, 2.0541052827491573, 4.8107104976229351, -3.0292407146411291]]], [[[-2.6588080399717406, 1.7261252680169044, 2.868182004463689, -4.1895140996262583], [-4.9839880812782766, 2.9401845524694616, -0.55191552447091574, -0.45885578625506884], [-4.7699801886100648, -2.8143724648432142, -1.5435553841227789, 1.804461369424236]], [[0.54269606107581492, -3.1737710887720327, -0.73730922665579879, 2.9381046450524053], [3.1791161187993104, -1.4126703364060624, 1.365809626484122, -2.4245800755317237], [3.500799487175918, -2.0901428245079465, -4.5540469526623939, -4.2956524945038357]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.7764426873233254, -1.7410262280812716, 0.86199503410442446, 0.79320917905742017], [-6.8786046648359269, 1.0912489486963206, 1.0385555496213941, -1.0249447828990823], [1.7920074369780454, -1.071277801617343, 1.0760544115910933, 3.4159289533894799]], [[2.7138660927120934, -1.8470053465558336, 0.89934234194632756, 1.3542808748271471], [-1.8183803531570839, 5.1794329899319909, 1.0181474914059736, -0.73288562052759043], [-0.76660958417284286, 1.3408703758661711, -2.0224931855843309, 0.71697209135545126]]], [[[10.159535416209989, -2.1260073978275553, 1.2718185679285974, -0.83722561472908952], [-0.75254912044909328, 0.78240447721983142, 0.77042336486880014, 26.542553965121552], [-0.92518747118168809, 1.0753524864515929, -0.9686363471985121, 0.88684474524348056]], [[0.75676204664106128, 1.9229204958164727, -0.74783188786715493, -5.2754362756944841], [0.81415984605120428, 5.337445845150909, -1.8458937002065836, 0.92909395400812345], [-3.5217464912957892, 2.4156743527423039, -4.6435255419374206, -0.989502405044413]]], [[[-1.5381297508591505, 2.5751844295901001, 1.150313702561901, 0.86492659575254427], [-0.97843436998520372, 1.0951730255136176, 2.0780439070215388, -1.969073263639598], [1.0881532578963402, 4.2065632531012724, -4.2815178084486716, 1.2950786017292415]], [[0.73707782184407888, 1.9539445199635301, 0.94319370497768462, -1.4521876530850828], [-0.7474495306805119, 4.1065815474341196, 4.3863424381232239, 1.330434546424943], [-1.2075469749989003, 2.0799698917179348, 1.3765409927536785, 1.1881409123279951]]]])+(1.-msk_ref)*numpy.array([[[[-1.1696534647900911, 1.4260219616290113, -0.94418105795764284, 0.9841773802325674], [3.111230457050207, -0.97556273104470503, 1.5576991063324741, 0.90430769336558092], [-0.87973166427623761, -1.3247995384002149, -2.1339966482365504, -1.7997902509000432]], [[-2.2022958986989045, 1.4770001258746359, 8.1315961649052273, -0.83382040314467154], [0.66603345122507873, -26.144644682745305, 0.74780426013102907, 2.269011880165182], [6.5918973602487974, -1.4747673360823919, -0.97198041247200329, 0.9960759380219697]]], [[[-0.92305484208221711, 4.6777714602343163, 1.1568830421623657, 0.97366976617841627], [-0.89863707365138756, -1.7933904666563272, 1.6536375225186939, 0.68422850367618682], [2.0231266449941891, 3.5300908114986385, -7.2492049273551311, -9.0108125290035357]], [[-0.75064014424490244, 0.81394330562367512, -1.4452592615106845, -1.015313005424938], [-0.70489286121330386, -1.4562943867800417, -0.68347432697015065, -0.78919776541726727], [-3.3085070792312798, 1.6008803547711039, 0.68355324965190067, -1.0855448950923796]]], [[[-1.2367860877311099, 1.9050626595383724, 1.1465021357316378, -0.78490648690692355], [-0.65978825393604823, 1.1184253012358565, -5.958116139126572, -7.1664712362518319], [-0.68939003177346048, -1.1684227424986058, -2.1303911914074232, 1.8223592089610199]], [[6.0593341828683025, -1.036110261832708, -4.4599696774441648, 1.1192170433146762], [1.0343682554843654, -2.3277736560608631, 2.4076391980408247, -1.356266525065587], [0.93932166233187808, -1.5732785124666333, -0.72207792935955872, -0.76551275923554185]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([4.4671748846543711, 1.9617048751826429])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.7212039366877185, -2.5768665164473115])+(1.-msk_arg1)*numpy.array([2.3226776121238144, 0.17395509602172066])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.2004649464685477, -0.76127531739099041])+(1.-msk_ref)*numpy.array([1.9232866676532294, 11.277076211309714])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[-1.6716363115177089, 4.6175021256648918], [3.2057068052427713, 0.62035728884162822]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.1772702675330518, -2.6133647075651556])+(1.-msk_arg1)*numpy.array([2.2571300167226349, 1.4716307262379758])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.52612342380795574, 1.4532922089911124], [-1.2266587958285755, -0.23737876579025535]])+(1.-msk_ref)*numpy.array([[-0.74060257899761306, 2.0457404276469329], [2.1783364182927367, 0.42154412637706162]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_expandedData_rank2(self):
      arg0=numpy.array([-2.2786023395938573, -2.0284244525488102])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.35462595523854823, 2.2986738573949896], [-4.4383930742228763, -2.0021925102398628]])+(1.-msk_arg1)*numpy.array([[3.9364154124324617, -4.8717702324748435], [4.289862809694732, 3.6062825446969029]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.4253682110241961, -0.99126821852671232], [0.45701775814526513, 1.013101608449132]])+(1.-msk_ref)*numpy.array([[-0.57885210295572476, 0.46771547730327478], [-0.47284133375191861, -0.56246964218919604]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_expandedData_rank1(self):
      arg0=numpy.array([[[-3.6874944996126446, 4.7147517841006366, 2.0074166131746098, 2.7831202568373259, -3.9983914085173078], [1.4120546681601756, 3.1309667801774204, -4.3047065417325143, -1.7853555640868715, -1.026323261091294], [2.1131105416085845, 2.9661282425233582, 4.4785383133377543, -0.9520635605489236, -3.3307103459085319], [3.4030636768719056, -4.7109852529077942, 3.6307159275104279, -4.1315209352937998, -3.0396604771797877]], [[1.9279034520986249, 3.2023034870167919, -2.0465387717300612, 4.2857293726872463, -0.92064525610626013], [2.9772470999229528, 1.9861208615620631, 3.8070431237572606, 2.04013150691992, -4.1375908068632175], [-4.4590168907225518, 0.18351766834601957, -3.1035890742121488, 2.390285388634549, 3.9375713602118818], [-4.461338213198573, 3.0052793029301643, -3.4175119202066107, -2.0842675422114834, -4.1945358318033144]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.0553506786066045, 2.0222419370881148])+(1.-msk_arg1)*numpy.array([3.0673870461532697, 0.18046489554995482])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.7940950602709453, -2.2938916619800058, -0.97667840046425025, -1.3540853567256477, 1.9453572814289579], [-0.6870139888330189, -1.5233248577804808, 2.0943903084464539, 0.86863793252897736, 0.49934216665502312], [-1.0281021937536896, -1.4431251432647019, -2.1789655458570776, 0.46321222478412366, 1.6205070894113986], [-1.6557095157985224, 2.2920591128038255, -1.7664702988649215, 2.0101294529917908, 1.4789011475357976]], [[0.95334955562966384, 1.5835412312870349, -1.0120148010959222, 2.1192960615080474, -0.45525969925830145], [1.4722506962791888, 0.98213810382250133, 1.8825853889862125, 1.0088464043316028, -2.0460414409272194], [-2.2049868558967876, 0.090749610608052173, -1.5347268876646369, 1.1819977346906327, 1.9471316898321798], [-2.2061347514247402, 1.4861126395477455, -1.6899619464561129, -1.0306717035117376, -2.0742007941162317]]])+(1.-msk_ref)*numpy.array([[[-1.2021614632027071, 1.537057995342741, 0.65443864206574742, 0.90732607752502725, -1.3035170809407917], [0.46034447134116857, 1.0207276529070188, -1.4033789922699631, -0.58204443626566116, -0.33459203082257888], [0.68889595926884462, 0.96698857949573147, 1.4600499532506577, -0.31038259803009915, -1.0858461275975879], [1.1094340641294678, -1.5358300670975702, 1.183651059641663, -1.3469186878372694, -0.9909608508622173]], [[10.68298322631372, 17.744744634450839, -11.340370466474212, 23.748271704736645, -5.10151990114562], [16.497652304344228, 11.005580090850863, 21.095754452163945, 11.304866249487217, -22.927399781847786], [-24.708500105430439, 1.0169161586067008, -17.197743997546816, 13.245154307436428, 21.819043245016676], [-24.721363119419653, 16.652985578008259, -18.937267050148282, -11.54943478541805, -23.242946053418002]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_expandedData_rank3(self):
      arg0=numpy.array([2.1794216334854326, 0.40723622407515414])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.4163769171374412, 1.1675204911009356, 3.7870446462572858, -4.2953678073842561, -4.634110902610086], [2.864705426526017, -3.2768004890145042, 0.68408608620766831, 3.8413475265348964, -4.9257123490766874], [0.33811875358935239, 4.539252243965846, -1.2927231844118547, -3.168095260313557, -1.5400646514673388], [1.3201414174499861, -3.6625188090526386, 1.011561859456938, 2.4742352341902976, -3.7342930958219203]], [[4.4119001076608804, 1.2855331877781806, -2.5192638992193492, 4.4444835963096683, -1.564938236380077], [3.7057464490926844, 1.6328012081798571, 0.20874821603675997, -4.9171541418341391, -3.6096862643485039], [2.8048456989769122, 3.7526910045225819, -2.4779926452193637, 0.44429903970931495, -4.5818147260856943], [4.1939628272686509, -3.2545173687507711, -0.65688540097625392, -0.40216516977123273, 1.6081801876475179]]])+(1.-msk_arg1)*numpy.array([[[3.8298545840052753, 3.7800502252264661, -1.6706967547523996, 3.0193017869690841, 1.8635054558421702], [-4.9581743533673572, -1.4827317660793158, 2.1073329198142119, 4.7679036674579827, 1.2235658404102354], [3.9870686030318563, 1.7337230615920349, 4.045801451734425, -0.11090792424049223, -1.4103258943631749], [3.4733227801507631, 4.9718437632268486, -3.9364804527594166, -2.3220014898346841, -4.8402117033900884]], [[1.6351041297453586, -0.37288441419999874, 0.03996451113072208, 2.4759346323645364, -1.3577660145562898], [2.9784923744381189, -1.7208418337789411, -1.4917531814484186, 1.2100638739585854, -0.63322401420058583], [2.6594607398504868, -1.0563173553276983, -3.7744456365819188, -2.1671908144514376, -3.3047998345974605], [1.9849045520121056, -4.208875000524972, 2.7489713068729813, -1.7592932861989672, 0.051745811412262199]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.63793360227698614, 1.8667095353763818, 0.57549404273312232, -0.50738882703798815, -0.4702998437646172], [0.76078385348275857, -0.66510660041462955, 3.1858879714501671, 0.56735862049206176, -0.44245816219738449], [6.4457283435167145, 0.48012789692015867, -1.6859151748539234, -0.68792806226089553, -1.4151494428554858], [1.6509001268176637, -0.59506087125028861, 2.1545114746175442, 0.88084657568893454, -0.58362361431240051]], [[0.092304044547160957, 0.31678390565629094, -0.16164889442560801, 0.091627343256096033, -0.26022510959739154], [0.10989316988345545, 0.24940955582040214, 1.9508488829597519, -0.082819495246340122, -0.1128176229877015], [0.14519024138251047, 0.10851845344697193, -0.16434117545135155, 0.91658137353074332, -0.088880988957635465], [0.097100580250104346, -0.12512952857015158, -0.61995018228434573, -1.0126093821272639, 0.25322798228901727]]])+(1.-msk_ref)*numpy.array([[[0.56906119688914814, 0.57655890891102157, -1.3044986334509443, 0.72182967694436317, 1.1695279059435277], [-0.4395613139350924, -1.4698691181671553, 1.0342085073475606, 0.45710269868925341, 1.7812050332777509], [0.54662255668943127, 1.2570759896820682, 0.53868724392076139, -19.650729633705748, -1.5453319280289743], [0.62747454568297645, 0.43835279974102292, -0.55364726426056277, -0.93859613916121887, -0.450274030774101]], [[0.24905828116193107, -1.0921245527219343, 10.189946343722388, 0.16447777689762369, -0.29993107774776395], [0.13672562252303153, -0.2366494212782298, -0.27299169134651879, 0.33654109740746779, -0.64311557196590186], [0.15312736825663714, -0.3855245036173986, -0.10789298966932298, -0.1879097222817615, -0.12322568520243143], [0.20516665330951916, -0.096756549915205284, 0.14814131491914145, -0.23147716601306792, 7.8699359998566267]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_expandedData_rank1(self):
      arg0=numpy.array([[[[3.151972567179369, -1.6021843864412553], [1.9142312927570879, -3.1201986027522977]], [[-4.8725811559512797, 0.16320343253043745], [-3.0314548052799184, 0.21870767867463847]], [[4.1614734912401605, 4.6972954541984144], [-1.3516187397083135, 3.8956838218707279]], [[-1.7742633633689486, 4.0793196298584427], [-4.3622421593880105, 4.2506475262010763]], [[0.41970167105672829, -2.548805576169475], [2.7774349231823345, 4.6406790650087295]], [[1.624039585864038, 2.5024570043082548], [-3.528324409732766, -0.096675490043040213]]], [[[-0.40891484658005606, -1.9895895384029441], [0.97512567683751605, -1.1402832070693467]], [[-1.8463279082013004, -4.3293491329407852], [-0.42418979476310703, 1.9514179674267904]], [[-2.2121431307799866, -2.5772282234961841], [-4.7549946235790692, -0.31352507444599365]], [[2.2971754986157507, -1.6946763139607821], [0.99418959530694462, 3.5552061735024392]], [[1.6864309218508655, -1.7001795534180419], [1.8511915088062345, 4.9963126961858677]], [[-4.434424191670562, 2.153779175805024], [4.7936568160000945, -2.4481065775990105]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.443865536331435, -4.9610516196617036])+(1.-msk_arg1)*numpy.array([-0.76091121419911367, 4.4438162442909466])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.70928621521285529, -0.36053844864170081], [0.43075814898246273, -0.7021361418887867]], [[-1.0964735805156167, 0.036725555981868781], [-0.68216618628440528, 0.049215638251554575]], [[0.93645351264962018, 1.0570291598148118], [-0.30415383378681649, 0.87664304646957203]], [[-0.3992612622643989, 0.91796648582352525], [-0.98163234772157226, 0.95652028429963099]], [[0.094445177880698561, -0.5735559627831589], [0.62500426722524172, 1.0442888127618215]], [[0.36545650911047567, 0.56312617558948908], [-0.79397641105169892, -0.021754818918947125]]], [[[0.082425033627837999, 0.4010418941253861], [-0.19655624484391282, 0.22984707567850365]], [[0.37216462350117663, 0.87266762469929826], [0.08550400747332533, -0.39334764421577589]], [[0.44590205875157446, 0.51949232160416958], [0.95846505703226581, 0.063197301395419278]], [[-0.46304204727714487, 0.34159618643039696], [-0.2003989620601325, -0.71662349962503924]], [[-0.33993416137158922, 0.34270547532298767], [-0.37314498028393184, -1.0071075810589063]], [[0.89384762176148203, -0.4341376266413231], [-0.96625820158810927, 0.49346524996769697]]]])+(1.-msk_ref)*numpy.array([[[[-4.1423657693058615, 2.1056127923250703], [-2.5157091353580392, 4.1006079875382291]], [[6.4036132797436114, -0.21448419931912149], [3.9839796663670324, -0.2874286442273507]], [[-5.4690657905735565, -6.173250395767246], [1.7763159676006888, -5.1197613455744309]], [[2.3317613543604092, -5.3610980541955513], [5.732918740039108, -5.5862595357791323]], [[-0.55157771790560262, 3.3496754005027829], [-3.6501432379409526, -6.0988443571477795]], [[-2.1343351964833346, -3.2887634688656555], [4.6369725454059099, 0.12705226081441662]]], [[[-0.092018846887603992, -0.44772092927087315], [0.21943429323619718, -0.25659999072515427]], [[-0.41548250573441514, -0.97424125907608816], [-0.095456196081030925, 0.43913111167317359]], [[-0.49780256634642983, -0.57995832451605012], [-1.0700250330305399, -0.070553114082695281]], [[0.51693755374493144, -0.38135607342854566], [0.22372428126032404, 0.800034469937833]], [[0.3795005979415676, -0.38259447734867397], [0.41657697056769499, 1.1243292750020264]], [[-0.99788648942618841, 0.48466882008724466], [1.0787252560585949, -0.55090184720039637]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_expandedData_rank4(self):
      arg0=numpy.array([-2.3837562890184505, 3.3397891325146212])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.6736353514438838, 2.2440613945319932], [1.1795682079905925, -0.98849135941729838]], [[0.083743117143707302, 0.82521496794156235], [3.2673281894350108, -1.7016059447783327]], [[-0.65162459240602288, 0.57380325343410643], [4.8814073940189822, 3.4421366744340585]], [[0.26372358003283747, 4.257717983503893], [1.3909240782322083, -1.9405454972780189]], [[0.63481923076840374, -3.5062462978869382], [4.4043695901092743, -1.7930816969402388]], [[-0.1170368741869332, -1.8818128258500755], [-3.7387027386765315, 3.6421718552826423]]], [[[-4.8457451343744697, -0.42782299239980759], [-4.1087145651131198, 4.1852947095755795]], [[-1.5085707268481539, 2.3176844142830157], [2.8890335752208527, -4.4053536745439708]], [[4.9766169909621816, -1.1399221176899843], [2.6489095153084037, 0.93769289857513805]], [[-3.1092306460854671, -0.19570543829077192], [3.3556849189497662, -3.7913945777050362]], [[-2.4157759199013427, -0.4591819290710113], [-4.9630558710582262, -1.8526184405840631]], [[-4.7340488462581689, -4.0711341368483662], [-0.75372488718574271, 3.020645067629351]]]])+(1.-msk_arg1)*numpy.array([[[[4.8061908301254324, -1.0559323035396186], [4.6048798734516794, -3.2223317390106496]], [[-0.39944967320086544, 1.6571810472945137], [0.32764383816814568, -2.6184039521431979]], [[-1.4923412912536858, -1.0257169745228301], [3.3537166678954655, 3.7076544105372218]], [[-2.1999274362656518, -2.890503052835478], [-3.7419345340944643, 4.1542406201416409]], [[-0.95901549892374405, 0.26786720200510405], [2.5765642552961712, -0.39291715214989864]], [[-4.9835907116103852, -3.6957552707011576], [-4.55025702970023, -1.0643724484626671]]], [[[-3.7909822214082656, -4.1420129596315132], [1.7667315654350046, -4.3994784406179157]], [[0.43080417618656597, 3.1403368368324536], [3.564365413773853, 0.98743319858928036]], [[-4.6049910046670615, -1.7218062123304123], [-1.0190375494883295, -0.62657699693919344]], [[-4.546698120499042, 2.8203621816144278], [0.7066091127532701, -2.3359253952069037]], [[3.8248920063289287, -4.8537095532900025], [-4.127371097126078, -3.0887087821917714]], [[2.5459518230565319, 0.72443882383575886], [3.9886451186935989, 2.3522320257322384]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.64888211838486909, -1.0622509236275113], [-2.0208719367565915, 2.411509484942429]], [[-28.465101017529687, -2.8886488752919184], [-0.72957356923200656, 1.4008862018456225]], [[3.6581742260782386, -4.1543094688852138], [-0.48833381371511497, -0.69252226581339271]], [[-9.0388439620061192, -0.55986711620029284], [-1.7137932445946871, 1.2283949499571736]], [[-3.7550158745712259, 0.67985990900155435], [-0.5412253082419699, 1.3294186723818298]], [[20.367566252761296, 1.2667340004666143], [0.63758914672694189, -0.65448759249540256]]], [[[-0.6892209639386554, -7.8064741536694537], [-0.81285498897212183, 0.79798183025761193]], [[-2.2138764017331947, 1.4410025419909454], [1.1560229556208292, -0.75812054587429833]], [[0.67109627656294779, -2.929839750177492], [1.2608166165033314, 3.5617088895410904]], [[-1.0741529055489749, -17.065387460273261], [0.99526302772188757, -0.88088671966615095]], [[-1.3824912753708607, -7.2733461860563144], [-0.6729299889591025, -1.8027398731180218]], [[-0.70548260927945805, -0.8203584112559088], [-4.4310453181196161, 1.1056542750769918]]]])+(1.-msk_ref)*numpy.array([[[[-0.49597620512214224, 2.2574896904165143], [-0.51765873476123025, 0.73976129153925463]], [[5.9676010495063432, -1.4384404726991848], [-7.2754497760312375, 0.91038523183839326]], [[1.5973264982944384, 2.3239902899407401], [-0.71078046390672367, -0.64292839220499398]], [[1.0835613255793772, 0.82468561542603203], [0.637038480309841, -0.57381276314639063]], [[2.4856285343601048, -8.8990226170840785], [-0.92516857831843291, 6.0668165692828877]], [[0.4783210393792891, 0.64499841423921045], [0.52387288750048733, 2.2395884941041491]]], [[[-0.88098253630795553, -0.80632030007258582], [1.8903772354869872, -0.75913296941751596]], [[7.7524530102704423, 1.0635130261642085], [0.93699403535019254, 3.3822937463375644]], [[-0.72525421420580738, -1.9397009422996088], [-3.2773955524912384, -5.3302134435661914]], [[-0.73455264545868915, 1.1841703006394959], [4.7265016431804643, -1.4297499138318159]], [[0.87317213845211239, -0.68809002595773439], [-0.80918072398194174, -1.0812897453364578]], [[1.3118037436014995, 4.6101741411802104], [0.83732421239032229, 1.4198383050562204]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-2.1620181400089455, 4.4757840098556088, -3.7860489950179299, -2.7120266373775594, 2.6626148910266547], [-3.5061964866697171, -2.464864693407347, -3.1680128654510864, -0.46661407381987097, 4.7486805863563433], [-0.22440254849984775, 0.0016138150895059766, -4.6660011920457114, 4.5083241575962134, 0.50937026779615291], [4.8934748541386615, -3.3277520595493559, -1.9721332735990726, -2.7805146726198302, -3.2130231714625679]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.0973779322035853, 4.0435895248135818, 2.0228753000382005, -0.024664796801905808, 4.7600494930724899], [-0.381525087411557, -4.3909799125427877, -4.0280479602206221, 0.0090645390024697292, 3.7322431545889749], [-4.2061262401976851, 4.1706718899777968, 1.631543575094847, -3.7177848431854663, -3.2755203913850996], [-0.16002850800011093, 2.0916430850132084, -1.2806806936805195, 1.1858228146176071, 4.5985095862319305]])+(1.-msk_arg1)*numpy.array([[-0.73317304492883295, -4.581738489411685, -3.4232872189962604, 3.6854570290003217, 3.7661015053442508], [-3.6042493585198732, -4.259955948907769, 1.908408601055676, 1.229671112924998, 2.8264815349774652], [-3.674377446549709, 4.8950973601397525, 1.1887879454512982, -0.6010062624693111, 2.7755790877965927], [-2.9970250099257756, 0.13879494043557017, -0.37903460563633917, -1.2516360039688812, 4.2864942761519682]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.69801560782439254, 1.1068838670171282, -1.8716175905387906, 109.95536104185564, 0.5593670601328149], [9.1899500251933084, 0.56134729434003716, 0.78648836774961584, -51.476867570732175, 1.272339552828333], [0.053351358396057313, 0.00038694367048724325, -2.8598691835579451, -1.2126371879372659, -0.15550819623527318], [-30.578769466095814, -1.5909750967519116, 1.5399102081654741, -2.3447977542213709, -0.69870968217233931]])+(1.-msk_ref)*numpy.array([[2.9488510999730035, -0.9768746121584776, 1.1059688401278915, -0.73587254336084207, 0.70699498865027832], [0.97279520307928347, 0.57861271876281395, -1.6600286037794181, -0.37946249929376963, 1.6800677901453911], [0.061072263741049473, 0.00032967987575632264, -3.9250071553126009, -7.5012931463861738, 0.18351855655481214], [-1.6327774502822228, -23.976032909457011, 5.2030427941748822, 2.2215042263109592, -0.74956898679144701]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_expandedData_rank2(self):
      arg0=numpy.array([[[2.8986209390432478, -1.5120327567406475], [2.7972331023222798, 4.6691597644067144], [-1.7602038196088299, 4.0866540186558176], [-3.4701664233095308, -1.0935631535360679], [-2.4826465085418215, 0.6354587729915071]], [[0.33820319532884469, 0.005636722086050483], [1.1724793710167249, 3.478035798037773], [4.3784269758121521, -2.2180982690243201], [3.7321145668091518, 2.6080702282029211], [1.4661987181220768, 2.8840308077693013]], [[4.1558337270365016, -3.9614484100923208], [-4.6794932754909251, 2.9385889447752938], [1.4454841506856955, -1.313251162382171], [-1.6623031424813286, -4.0589346424450365], [0.096320596020565752, 4.6289768185597211]], [[-2.2791932041356064, -4.6418081683554115], [-1.7450830371221082, -4.2735484653475417], [-1.6517429142947848, -0.44848630206984197], [-3.681563796358521, -1.5117315937328413], [0.35883715544599681, -4.6632044881517203]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.8930644843788915, 0.71422129995290007, -0.67556841567575265, 3.1789040536299815, 0.73283217799703682], [3.360484953495817, 3.660377504762657, 4.3298423388943554, 4.5040896465888842, 2.9499842209642164], [4.5228299161043104, 4.856516697606617, 2.2834886923904909, -1.0308571244401077, 4.6406113454515197], [-4.693610426664665, 0.82252363046393562, 2.3258776948385922, -2.3448720420803184, 2.7620884888924371]])+(1.-msk_arg1)*numpy.array([[1.3891489232716436, 3.4071034345915603, -4.6459882865780529, 1.800924587762224, -0.95448240437691112], [2.469149122505943, -1.8596086446285467, 4.6960915243044958, -1.8379088238245069, -4.2576876436243634], [-3.1643793424070177, 4.3330931196481099, 4.8235363809084895, 3.5886254699597959, -2.4890579980915231], [-2.4346373219644279, -2.205309176163107, 2.9333991346345911, -1.6201270790475197, 0.31660647803174946]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0019206121033106, -0.52264053044958791], [3.9164795316336067, 6.5374132145241628], [2.6055152650204141, -6.0492082279602268], [-1.0916235170252961, -0.34400634152117027], [-3.3877422185899975, 0.86712727971134007]], [[0.10064118721228658, 0.001677353764130609], [0.3203165163951443, 0.95018499963797931], [1.0112208789870634, -0.51228153253975561], [0.82860574714262669, 0.57904491980485118], [0.49701917308657428, 0.97764279119657194]], [[0.9188569555178151, -0.87587826285196035], [-0.9635493022801852, 0.60508161049327513], [0.63301568144507747, -0.57510736390263539], [1.6125446515046205, 3.9374366691694322], [0.02075601442361125, 0.99749288918512813]], [[0.48559488260623029, 0.98896323861585067], [-2.1216205498409972, -5.1956543338907997], [-0.71015897265802275, -0.19282454235022248], [1.570048911109162, 0.644696839146782], [0.12991515546624866, -1.6882893169080209]]])+(1.-msk_ref)*numpy.array([[[2.0866164098638054, -1.0884597982335795], [0.82100034707564107, 1.3704191416678984], [0.3788653158454876, -0.87960919541314508], [-1.9268804740022223, -0.60722317911984169], [2.6010395761695579, -0.66576268989090104]], [[0.13697155519938858, 0.0022828601296991597], [-0.63049791385053788, -1.8703052430327372], [0.93235554570257428, -0.47232858591971899], [-2.0306309640774178, -1.4190422258138922], [-0.34436502647572637, -0.67737021810136022]], [[-1.3133171713462533, 1.251887963305627], [-1.0799429290527101, 0.67817350415352629], [0.29967311046038914, -0.27225899395721498], [-0.46321444140559803, -1.1310555187277622], [-0.038697610137818902, -1.8597303968444985]], [[0.93615307034585393, 1.9065706939093872], [0.79130992424303959, 1.9378455010026521], [-0.56308154413515898, -0.15288962786365218], [2.2723919894746341, 0.93309445492485399], [1.1333853864165484, -14.728708386327117]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_expandedData_rank3(self):
      arg0=numpy.array([[1.6004896542026881, -0.91205104216791177, 3.7845829941674793, 0.62635929467683393, -0.78839261794904836], [1.4543497215506349, -1.3321936219988406, 2.5073884002282245, -3.0608528618182884, 4.6164018817413996], [-4.7641414445612167, 4.6072139780638608, 2.064762313451074, -4.2253828630692345, -0.7927805343469938], [3.1045280012955185, -2.9858933739881577, 2.3922366672129245, -0.54051030675204981, 0.3097180192234319]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-4.5013271107849082, 0.29129297257155518], [-0.32873605971276998, 2.4055034933306345], [2.011461681202757, -0.77024951594008328], [3.592415853290726, -1.356656459999559], [4.387492327724253, -1.6440159181380487]], [[-0.61378686612204891, 2.2040186490007274], [-3.032343631695932, -2.1033881252159081], [-1.1110939026383813, 3.9586316144629627], [0.64337844776940134, -0.31765592237795026], [3.1952123985589971, 2.6092596968822681]], [[3.3231156910569268, -1.1977971183262892], [3.6222245179900057, -3.682094782494707], [-1.5942300554934796, 4.7287020958120412], [0.63759231885766887, -2.8647528549283785], [4.5608016996729912, 0.33249969137533952]], [[2.069860539271291, -0.38360856505477958], [0.48289236352699749, 4.42776527499473], [4.2103037072787224, 2.3598235475789853], [3.2202618042034175, -3.6817426793053176], [3.6868898283852918, -1.8865916610233615]]])+(1.-msk_arg1)*numpy.array([[[-2.3394470269326484, -3.1214962712084295], [0.058295829742180061, 2.7806602859763832], [0.74170346879162796, -1.4520342712781789], [-3.6813605218452219, -4.0460006496644674], [2.4667812023413305, -2.7174824860418836]], [[2.3243383292332016, 3.2206337387020039], [0.47338821294210209, -4.3223225835869243], [-2.1492494858774669, -1.8627223048704944], [-4.8446721488529256, -1.7142499725906446], [3.4710501261668369, 4.175244301394196]], [[2.3494228209670309, 3.8368456620130349], [0.13864562841184203, -4.8788933435183388], [-0.38175788478472494, 2.3629936187389937], [-4.1856997068628674, -3.9834312091514312], [1.6244551737209791, 3.4898247397450319]], [[4.5496014497342827, -0.73202643452530758], [-2.7814197625623773, 1.0401663254758597], [-4.6138317154742436, -0.53694980829280592], [2.6433020992123915, -1.3981393296059208], [-4.5331677853400567, -0.87951663286783255]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.35555950829878846, 5.4944327701195501], [2.7744173942001003, -0.37915182609238102], [1.8815088696616291, -4.9134506622161602], [0.17435601006578236, -0.46169337127325333], [-0.17969093939315889, 0.47955291019441743]], [[-2.3694702539650687, 0.65986271132960583], [0.43932805242582951, 0.63335606302431413], [-2.256684510889881, 0.63339776074828891], [-4.7574687533135309, 9.6357493948324837], [1.4447871709008584, 1.7692381817177532]], [[-1.4336369502218467, 3.9774193573100813], [1.2719294331927364, -1.2512480667166215], [-1.2951470249454966, 0.43664461655973713], [-6.6270918549325808, 1.4749554593514396], [-0.17382481996615554, -2.38430457203664]], [[1.4998730312470663, -8.0929579892257877], [-6.183351818155689, -0.67435674398790513], [0.56818624819802299, 1.0137353996942664], [-0.16784669682648787, 0.14680827907669933], [0.084005227614592393, -0.16416802089299423]]])+(1.-msk_ref)*numpy.array([[[-0.6841316070751815, -0.51273156049072843], [-15.64521932703525, -0.3279980106766836], [5.1025553383662405, -2.6064005988206005], [-0.17014342685537398, -0.15480948939758019], [-0.3196037886135788, 0.29011874851026992]], [[0.62570483103052577, 0.45157252874608905], [-2.8141672850687875, 0.3082124474137018], [-1.166634407361294, -1.3460881386732255], [0.63179772908740739, 1.7855347299160824], [1.3299726923965232, 1.1056603035659236]], [[-2.0277922739340206, -1.2416818043344668], [33.230142420201609, -0.94431537106352059], [-5.4085649458567255, 0.87379089688482936], [1.0094806505448306, 1.0607395085327318], [-0.48802856931475042, -0.22716915417503594]], [[0.68237361790818774, -4.2410053173949809], [1.0735141146899054, -2.8705922320857278], [-0.51849239736890418, -4.4552333016355332], [-0.20448298622889241, 0.38659259152977116], [-0.068322646301563775, -0.35214572146695727]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[-0.28111320724498956, -0.54218894553301222, 1.6954157928875535, -1.2212275671898167, 2.7583985860742564], [-1.5147350401972104, -0.067376458229628788, 4.7262145354795262, -0.43465333674140716, -0.023137140003289147], [-3.420056669228301, 2.552030300964832, -0.25880085677567344, 3.3243559136908267, 0.091537804992652738], [4.4139957512587724, -3.9999615527576706, 3.0268308705996798, 1.8074411890252895, 0.25423151213546369]], [[4.8551469880958429, -4.9678273739849041, 0.32863208740970862, -3.4454822147034592, -3.4305950115595127], [3.5280224890645222, 1.2462618068925782, 1.2019074753128756, 4.6167789289172685, -0.96698077359934853], [0.59274965644235245, -0.48317337020364981, -4.5456698546916483, 3.8057456589627829, -3.9739393736524931], [-1.744611687004892, 2.0705378371102743, 0.8846217953991955, 3.5344707047967709, 2.0761501617054652]], [[0.90196385942227764, 0.12654862291202562, -3.8707906767432911, -1.3560149143875568, -2.2017939101370967], [2.3644104871913774, -3.8027421652380857, 3.6660199391347277, -2.2111887951009801, -1.1986556694176365], [4.0172993295823911, 0.70183539856772814, -3.752326214115941, -4.2771468816499372, -4.9259062251106203], [-1.9502445025334971, 3.5029533536725275, -0.9001780482416919, -1.7012073043106399, -0.21982847378216164]], [[-2.808112952048325, 1.9568747730529523, 0.88817511922877479, -2.3111178171483502, -3.1505647821104232], [0.3074378513401328, -2.2923884526393556, 4.5051852840947841, 4.4952931408210812, -4.2375726449495108], [-0.72687738652940048, -4.1400736917791567, -2.1197586495197038, -4.2412465258030334, 1.155085699063231], [1.7393142865834257, 2.3529194314768986, -4.5841732884577144, -3.9936091077644842, 4.160671703497357]], [[1.5614598428986239, -2.1416763447585407, -1.606206446405567, 3.8902875584529166, -2.0173449518354025], [2.0751491316157731, 0.69192874805873217, -0.3807108961551231, -0.49249717475479482, -0.049743913641685289], [-2.3877759330320689, -4.8337198248183881, 0.52099483090728071, -0.0023457844292433805, -2.9645212402954932], [2.2190563712791871, 3.1138290177802084, -4.4016575178764512, 4.2512236787594517, -0.76983632543004088]]], [[[-4.9410608458340155, 3.3158669637061351, -1.5695589373406671, -0.17334739969775903, 2.2032349446588313], [-3.608296819038622, 4.1658843535704921, -3.7791074065972574, 3.4546940426206003, 0.46455463942858799], [0.50564303623081663, 3.1717016816028405, -0.6832996041766437, 1.7316691884281905, 2.7944162545971976], [3.5863850446386571, 4.4916314814955154, -4.1357467385365076, 0.86195983448125713, 3.1067121598891756]], [[3.7547046077453849, -1.7162917785867569, 2.7167385513946734, 4.619785230534184, 1.0530870083864174], [2.2153278962101517, 3.1341491080705737, 1.9085002321760767, -2.9549826757846045, 3.9232323372615454], [1.6522239139509223, 1.1776741531661212, 1.7389092980468206, -2.5445440025600909, 0.27534718589479645], [2.7869974234472696, 0.79314632456544576, 3.7282653324814454, 2.5434735475146866, 0.80100964106405748]], [[-4.2127233135607174, 3.2807185673590471, -0.89375719945385512, 2.8497777036969545, 0.76579366363402812], [-4.0453649738448805, -2.8043704335941291, 0.2779225476988243, -0.29871308841848432, -3.3589046292285305], [-3.2572215924757506, 2.1584858308259234, 4.5972206401670341, -3.8395286670018947, 0.36700432228541668], [-1.8756402589396814, -1.2440244824212621, 0.54591735407395614, -2.4857125490492948, 4.8175576521140169]], [[-1.4384855665424636, 2.9298148875320589, -4.4238172614506936, 1.4047754249764113, -4.812892529093288], [0.29902395134354531, -2.768161313849411, 1.6908374671081123, 3.8937052420178091, -4.2046037170316097], [-2.902669439162755, -0.87954622348868217, -2.0966535505964314, -0.13682670432771182, -2.7284764580584442], [0.78954101104717722, -1.7366922828897993, -2.1363600248440928, 2.5055192897711418, -4.5116809604035275]], [[-1.0516040385121848, -0.61654789070676408, 4.188151453987544, -1.5784702793518157, -2.3666464598100956], [1.3719398392157061, 1.2442048500836433, 4.613907403103239, 3.4758957228281098, 0.87007436368937618], [4.4237554151995209, 2.4264565211589959, -1.5078471414517303, -1.764675902994969, 4.2025987902539672], [-4.1458107245883404, -4.2198299801612613, 0.034820499309438979, -3.3881730518320752, -0.64172037056079567]]], [[[-1.4322093799765199, -2.7574040800197519, -0.30715532199519835, 0.069876171374874119, 0.68386254718090367], [-2.5228753326813713, -1.9396103687521582, 3.4427542683090664, -2.9055520539366131, 4.4459099531810899], [0.95340658454531102, -2.9554367996588149, -4.1773277685051946, 1.7952034969415118, 1.6389166271773519], [0.75343018091986735, 0.36253970085846454, -4.2740477226589482, 4.5930533037925123, -1.188182402340999]], [[-0.62972322317924778, 4.2677213977083426, -2.1623047150456145, -3.2618992827448157, 4.6525611861944256], [3.5201126697360774, 1.2505871200206684, 1.2483559387653749, 0.093516850307137744, 1.7389216400447083], [-3.7328051992422404, -3.5946554627350249, -0.12393923639267168, -0.52322224318196131, -3.2296137332985575], [2.159175139144641, -2.2927332467928552, -2.2403539786254889, -3.6133584130546117, -0.14561231805850028]], [[0.17074267049882597, -1.7560368648760605, 1.5054508112851135, 4.5284874963451429, 0.38749755077953374], [2.2900625551979035, 3.443326311631191, -2.8119025046670352, 3.6764075865245225, -1.7203644274955554], [-1.9521061068440582, 4.5893834330404015, -2.7030926033810445, 1.1460622700675982, -3.4027214016635954], [-0.76322578065365398, 1.7619558971804858, -2.4491363247123656, 3.5475631447229752, -4.1039139985475224]], [[1.6079343080465236, -2.9413113253066738, 3.8907837909641056, 3.7467435727645046, 1.6815675126538068], [1.0835829091465046, -3.9334420460449451, -4.9519932570445526, 0.11781423441059857, -2.444037968862911], [-1.6326052583201092, 3.0611053544504134, 0.12957014383556498, -0.15693158358951376, -3.4759204878236893], [2.0643682145851869, -1.9586023507948305, -4.1916577051725596, 1.3920625289038338, -3.0327350667196051]], [[3.9564830733755407, -2.0781639834279253, -3.4419796130500968, -3.0754770378472984, 2.2332440656694841], [-4.5408941624388284, -0.029756032323210668, -0.98796305757716496, 0.084900223173636213, -0.17988752218802695], [3.0606938431447013, 4.374614362668046, 3.736995648365923, -1.9908304954885003, 3.0616226746292625], [3.5527360847263854, 1.0244423428064842, 1.3235636685290073, 4.2606311931403127, -0.17613159706867965]]], [[[2.0251020340300094, -0.63278810122271878, -1.2122834694294404, -2.8658064237128755, 2.0572054422360946], [2.4406133009400319, 0.22759090328363918, 3.1230185902392833, -2.5273806784988864, 3.1278826883401596], [-2.3686122360694384, -0.97922239545774836, -4.7118753148541987, 2.659010590554054, -4.8248256586481366], [3.2037346340735446, -3.8897397500832733, -4.8055574272823893, 1.7996229298773594, -4.539990198827077]], [[4.1767968739415693, -2.9980773671700378, 1.3417912399087042, -3.8607172951241631, 0.4308608986879916], [-4.1874569937356663, 2.9796707672153069, -2.5113465648348141, 2.2380792536979204, -2.1027066070068425], [2.1529211417810661, 0.43002894984614493, 4.0006405528134952, 0.28962592636786511, 3.4541387184961909], [-0.061431761181783173, -1.8807528382328487, -3.2399622045449741, 3.5785692692200328, 2.1738142020030624]], [[-2.8686291098066174, 4.0052257573051868, 3.1885799507345514, 4.9268688677094232, 1.3282653786836631], [1.6717714378601558, -1.2271082047287041, 3.280098144759414, 2.5989570285117587, 2.8007689955199266], [2.3010563163149822, 0.47023565276342438, -4.1096686330315659, 4.4816736524185838, -1.5753805992639092], [4.6768931148262674, 3.4576807211898188, 1.2362313039436978, 0.60827478419193071, 1.0065907168471542]], [[1.6928498520841249, 0.25200626413111582, 4.9241157812886556, -0.40342473811534596, 1.0485323492575747], [3.2946596512557171, -3.8234309281870651, -1.0155339681197573, 0.78940917265388855, 3.5207715315157291], [1.2775237632234813, 2.6700395090880562, -3.1656027422574997, 1.7317266396377207, -2.2271608953294475], [1.8669544842266186, -4.6477261160079593, -0.55113988098768463, -3.5646780215713303, -2.2708977247443194]], [[-0.99061433244609276, 1.0866339032194814, -1.1045582883312899, 2.63209873747061, -4.5044144860887041], [1.8373461023186639, -1.8221088083042867, 1.1528225492326785, -0.50227766836334542, 4.1378311043250253], [2.8412677151063681, -4.4956954008967189, -3.5111717959661837, -0.81091966792807302, 1.1086880329578408], [2.0433423037255594, 4.585906131141833, 4.7969813789406786, 1.202753030785999, 1.8183131896589222]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.19160434412165905, -3.5337416511709883, -1.4618864293835632, -0.97515092523833058, -0.36530064534933704], [-1.2130604298061334, -0.79220110320927706, -2.2254735444729845, 4.2957121248548251, -1.3636280775207754], [-4.1912562835906932, -3.8283220583959365, -1.268434241770926, 4.8240264978705305, -4.072117936229354], [4.5212923738238349, -0.22775523554770327, 2.8755423340716675, -1.1349555356430727, -1.5669103620416469]])+(1.-msk_arg1)*numpy.array([[0.54734679908627548, -4.6596138043869439, -0.48122626088702525, 4.8717263185314525, 4.7158307854586905], [-0.1483525605257654, -4.8569635057042078, 4.9126129407176649, -4.8854342861119484, 4.7463686488677368], [0.12497360681897085, 1.9680018241170494, -1.3622297368839167, -2.8391741913190494, -3.5574116914621423], [-2.3592673795602659, -4.4551007605653128, -3.4932653028101415, 4.2953097311987971, 0.51100182553170281]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.4671546646484013, 2.8297320085224671, -8.8485248111652961, 6.3736945672505163, -14.396325922145131], [7.9055359999323942, 0.35164368813500463, -24.666531216424922, 2.2684941655885646, 0.12075477781755425], [17.849577914876178, -13.319271609752345, 1.3507045362778831, -17.350107216671606, -0.47774389151913416], [-23.037033797397143, 20.876152735962489, -15.797297730774806, -9.4331952509263353, -1.3268567229041506]], [[-1.3739394294676228, 1.4058264198059578, -0.092998334301209781, 0.97502380049818238, 0.97081092796433055], [-0.99838155624518532, -0.35267485003596682, -0.3401231878155539, -1.3064845664049578, 0.27364218130629803], [-0.16773995242293135, 0.13673137934221619, 1.2863616821522121, -1.0769733711862213, 1.1245698655801946], [0.49370097172405741, -0.58593356320322765, -0.25033572986470454, -1.0002063120900593, -0.58752177342038692]], [[-0.61698627286841334, -0.086565290140484891, 2.6478053280619727, 0.92757883726942492, 1.506131985277086], [-1.6173694752665455, 2.6012569025909875, -2.5077323829325007, 1.5125585344091175, 0.81993761302174217], [-2.7480242300876783, -0.48008886631752412, 2.5667699888959175, 2.9257723415994024, 3.3695546562996266], [1.3340602001181863, -2.3961870657419162, 0.61576469289838875, 1.1637068859227264, 0.15037315441450344]], [[2.8796700893884828, -2.0067404156691793, -0.91080785162737909, 2.3700103823246685, 3.2308483749224903], [-0.31527207059255347, 2.3508037507928194, -4.619987703948186, -4.6098434862505151, 4.3455556829973085], [0.74539988397360013, 4.2455722336184083, 2.1737749456594391, 4.3493231827334382, -1.1845199231912984], [-1.7836359906630153, -2.4128771973444381, 4.7009885032281797, 4.0953753971862676, -4.2666951297620654]], [[-4.274451367052472, 5.8627773370355136, 4.3969439059423276, -10.649550193738733, 5.5224237282860953], [-5.6806610063097693, -1.894135027867363, 1.0421851179349799, 1.3481968373853261, 0.13617253151610278], [6.5364678749708709, 13.232168862433575, -1.4262083506834577, 0.0064215173422431451, 8.11529154967827], [-6.0746029319414516, -8.5240172921196269, 12.049410735825955, -11.637602432084417, 2.1074047780393226]]], [[[4.0732190453394619, -2.7334722015753692, 1.2938835516970146, 0.14290087734991302, -1.8162614907906474], [2.9745400397037813, -3.4341935910284929, 3.1153496674532692, -2.8479158644822964, -0.38296083856500973], [-0.41683252029878393, -2.6146279308688105, 0.56328570892865248, -1.4275209592855478, -2.3036084484626955], [-2.9564768221908113, -3.7027268973016874, 3.4093493093311658, -0.71056627790506055, -2.5610530881678155]], [[-4.7395851792363617, 2.1664849640247992, -3.4293546681378304, -5.8315814151470171, -1.3293177756509911], [-2.79642111988441, -3.9562544098636789, -2.4091107983119091, 3.7300915939320296, -4.9523186995930484], [-2.0856117307305131, -1.4865848436656532, -2.1950351886690695, 3.211992500707102, -0.34757233331200948], [-3.5180428456321198, -1.0011931583436826, -4.7062107303030905, -3.2106412591586269, -1.0111190678971491]], [[1.8929559167410073, -1.4741665096432119, 0.40160315617928688, -1.2805264348230263, -0.34410369223929649], [1.8177546904080881, 1.2601230154179315, -0.12488243160160292, 0.1342245065821363, 1.5092988355536503], [1.4636083185824142, -0.96989956864981541, -2.065726933301157, 1.725263675471431, -0.16491066505682825], [0.84280501271195862, 0.55899315698936336, -0.24530390641117916, 1.1169364629036458, -2.1647337323234122]], [[-0.33486544831983539, 0.68203240868499138, -1.0298216297723157, 0.32701805524826366, -1.1203945677006792], [0.06960986738692386, -0.64440102907104413, 0.39361051624595417, 0.90641670783500139, -0.97879084883363909], [-0.67571321233748916, -0.20474980583537281, -0.48808055327201155, -0.031851925909103122, -0.63516278064155751], [0.18379746782353579, -0.40428507134855812, -0.49732383426794263, 0.58326052047908505, -1.0502754442736315]], [[0.77118098097841725, 0.45213786726048893, -3.0713297291458388, 1.1575519053711822, 1.7355512832450011], [-1.0060953289477892, -0.91242243438235993, -3.3835526557152051, -2.5490056857348282, -0.63805841052441969], [-3.2441070172465141, -1.7794122614214269, 1.1057612895395648, 1.2941035257966691, -3.0819245067868875], [3.0402796722445586, 3.0945607895030824, -0.025535187991102711, 2.4846753361020135, 0.47059779799159995]]], [[[0.34171362547878631, 0.65789441004009774, 0.073284786520392678, -0.016671891825954025, -0.16316409708905499], [0.60193773942165085, 0.46277541565424712, -0.82141344631868296, 0.69324132368431457, -1.06075831501581], [-0.22747513395399377, 0.70514342232655391, 0.9966767684572212, -0.42832109884808622, -0.39103231019155782], [-0.17976237431951925, -0.086499053345378579, 1.0197533706999484, -1.0958655336288803, 0.28349075359406817]], [[0.16449066028763035, -1.114775959966261, 0.56481787113585169, 0.85204411566971161, -1.2153003627244059], [-0.91949230394973647, -0.32666716669721962, -0.32608435751312803, -0.024427634060213109, -0.45422553628450862], [0.97504994154182589, 0.9389637046996987, 0.032374297277539418, 0.13667142816119054, 0.84361077360658698], [-0.56400039134882329, 0.59888724402499938, 0.5852052007255093, 0.94384912187053704, 0.038035545556872954]], [[-0.1346090044529572, 1.3844130086115993, -1.1868575931719381, -3.5701397417517597, -0.30549281785276355], [-1.8054247353025017, -2.7146273714779148, 2.2168295462768288, -2.8983824824783202, 1.3562898026890748], [1.5389888120007096, -3.6181484872506502, 2.1310466986503918, -0.90352517483880124, 2.6826155346554517], [0.60170701445908259, -1.3890794170934151, 1.930834286918178, -2.7968049331197813, 3.2354172281078108]], [[0.33331788470820256, -0.60972122076963231, 0.8065427900699994, 0.77668387070809608, 0.34858173216836619], [0.22462208895926059, -0.81538566336260465, -1.0265269602541591, 0.024422385420686494, -0.50663858706866194], [-0.33843206687210153, 0.634554009146027, 0.026859335016663179, -0.032531244108793357, -0.72054340691496299], [0.42793467562760296, -0.40600986575414044, -0.86891266186512095, 0.28856859088944303, -0.6286729701958198]], [[-0.97160326280704745, 0.51033983199225219, 0.84525538477828455, 0.75525244750035114, -0.54842322856135006], [1.1151185288713779, 0.007307261918539561, 0.24261651382621446, -0.020849156262957108, 0.044175420507245135], [-0.75162210207958813, -1.0742847901696073, -0.9177031969330085, 0.48889313292628828, -0.75185019750784121], [-0.87245412347170403, -0.2515748214687229, -0.32503077986846896, -1.0462936633621951, 0.043253068753645103]]], [[[0.44790335740161413, -0.13995735044392738, -0.26812764342513967, -0.63384673822567905, 0.45500385114361341], [0.53980435219585443, 0.050337577061214597, 0.690735818882251, -0.5589951875554956, 0.69181163917846489], [-0.52387946636289084, -0.21658019753975377, -1.0421523151508072, 0.58810852532972202, -1.0671341863626465], [0.70858824627703076, -0.86031590715146944, -1.0628725218267936, 0.39803285898880003, -1.0041355045100586]], [[-18.33897194019384, 13.163593626993006, -5.8913738544011052, 16.951168151370716, -1.8917716541262468], [18.385777098233181, -13.082776165605273, 11.026515192046213, -9.8266863034599936, 9.2323085436445709], [-9.4527844183416683, -1.8881188342916291, -17.565526180738718, -1.2716543076227245, -15.166012364939565], [0.26972710872728239, 8.2577809186692672, 14.225632164957915, -15.712346900014522, -9.5445191271915135]], [[-0.99759585376882243, 1.392859256442915, 1.1088621137493857, 1.7133703125605351, 0.46191821380799697], [0.58137604793770703, -0.42673974581732615, 1.1406885253937158, 0.9038145596805478, 0.97399678743527163], [0.80021646318688233, 0.16352937920325697, -1.4291803616789112, 1.55854900806579, -0.54785512304846695], [1.6264386232157988, 1.2024447285023496, 0.42991239923539487, 0.2115339346545578, 0.35005247703025705]], [[-1.4915560997065369, -0.22204064936193957, -4.3385979685086271, 0.35545422304739283, -0.92385324035048633], [-2.9028975565892483, 3.3687935853986435, 0.8947786377766328, -0.69554193786684737, -3.102122877017238], [-1.1256156942744269, -2.3525498799168334, 2.7891865741364485, -1.5258101178884631, 1.962333171111875], [-1.6449582610025253, 4.0950733046775518, 0.48560482210909295, 3.1408085247600135, 2.000869332257686]], [[0.63220868049870182, -0.69348823617684374, 0.70492755366814774, -1.6798017303561947, 2.874711020622494], [-1.1725917109416812, 1.1628672912278928, -0.73572973742453152, 0.3205529049593428, -2.6407580194527083], [-1.8132930791294684, 2.8691465126562408, 2.2408249259333575, 0.51752779710478392, -0.70756315090880295], [-1.3040581983663271, -2.926718874439318, -3.0614268021626496, -0.76759530086892813, -1.1604449327207864]]]])+(1.-msk_ref)*numpy.array([[[[-0.51359249330455869, -0.99057662607715324, 3.0975165940822715, -2.2311769598881326, 5.0395811041172527], [-2.7674137178217979, -0.12309646889705951, 8.6347714892447147, -0.79410958000851484, -0.042271444798642477], [-6.2484272767058142, 4.6625472282383233, -0.47282793506367066, 6.073582451272955, 0.16723913457695055], [8.0643492546724787, -7.3079107422114973, 5.530005611894655, 3.3021864602891222, 0.46447976412736902]], [[-1.0419633883659645, 1.0661457327875075, -0.070527752128364651, 0.73943514620451989, 0.73624020263861101], [-0.7571491194705775, -0.26746032165138767, -0.25794143587206753, -0.99080720478822792, 0.20752380222776257], [-0.12721003957115268, 0.10369386616306069, 0.97554648207368189, -0.81675130573691335, 0.85284736900536684], [0.37441121952260742, -0.44435825028265208, -0.1898487369417482, -0.75853297143834741, -0.4455627116030102]], [[-1.8743030726538561, -0.26297114932747762, 8.0435981810477593, 2.8178323267065029, 4.5753818714685632], [-4.9133031161540384, 7.9021917013187144, -7.618079554464253, 4.5949046733758534, 2.4908359473321386], [-8.3480467632366171, -1.4584312112852338, 7.7974261155229296, 8.8880163642068126, 10.236154228222901], [4.0526560186858651, -7.2792231812446655, 1.8705921131204051, 3.5351505987534266, 0.45680897251318026]], [[-0.57641024319584744, 0.40167994774444521, 0.18231219513507255, -0.4743940168307772, -0.64670397639663357], [0.063106552223731599, -0.47054951422853736, 0.92476157105903278, 0.92273104991992971, -0.86982978268510314], [-0.14920324726872436, -0.8498165580506486, -0.43511447707076661, -0.87058390568243726, 0.23709987457001144], [0.35702216685844734, 0.48297446893243573, -0.94097512641875602, -0.81975235196880469, 0.8540446304774284]], [[0.33111023570086534, -0.45414613928948011, -0.34059882966079275, 0.82494214390572618, -0.42778145434223491], [0.44003892972888581, 0.14672467684640023, -0.080730397988207886, -0.10443486994347095, -0.010548282138339467], [-0.50633197874588698, -1.0249985728332767, 0.11047784676960277, -0.00049742760840287771, -0.62863180957141696], [0.47055470652629622, 0.66029277966922184, -0.93337902018219243, 0.90147926678542856, -0.16324511214521067]]], [[[33.306205355153729, -22.351262101271558, 10.579924820832945, 1.1684826947604494, -14.851344235990997], [24.322443820657512, -28.080973721023, 25.473826627622746, -23.2870536940992, -3.1314231300234661], [-3.4083876573400844, -21.379487285977714, 4.6059171594680395, -11.672661275889739, -18.836319674522048], [-24.17474313842925, -30.276737156251667, 27.877825120640399, -5.8102120477492853, -20.941412462844628]], [[-0.77305596456216175, 0.35336723789896235, -0.55934917942126383, -0.95116737548234154, -0.2168200372824772], [-0.45611376194372966, -0.64528982035580595, -0.39294102785303192, 0.6084012515873668, -0.80775413129086682], [-0.34017630810083005, -0.24247127897564286, -0.35802395797386116, 0.52389605142630347, -0.056691219847836608], [-0.57381477546086379, -0.16330086146085795, -0.767612383354914, -0.52367565548465245, -0.16491983934475118]], [[-0.85753210448232398, 0.66781539823077929, -0.18193112509354126, 0.58009408396026441, 0.1558831670386302], [-0.82346503228766654, -0.57085108626214087, 0.056573263770750816, -0.060805337612217104, -0.68373077011392658], [-0.66303240898109828, 0.43937632719556335, 0.93579948097751897, -0.78156547510152363, 0.074706541450383926], [-0.38180094413578536, -0.25323071396696017, 0.11112565973785127, -0.50598583259974206, 0.98065076777863114]], [[0.29444374487478286, -0.5997040827794532, 0.90551156813765465, -0.28754361285133478, 0.98515142098525854], [-0.061207240509527396, 0.56661519769462299, -0.3460976789545066, -0.79700288940261188, 0.86064072727049723], [0.59414767841915483, 0.18003439857721742, 0.42916421096005447, 0.028007070879384352, 0.55849210085883483], [-0.16161122324204466, 0.35548370547666053, 0.43729173287976114, -0.51285497727268547, 0.92349639687695584]], [[-0.22155970517862086, -0.12989886296628977, 0.88239067881645528, -0.33256377583067964, -0.49862255439738495], [0.28905041742659143, 0.26213826656310246, 0.97209208648466505, 0.73232738119852892, 0.18331369264731165], [0.93202946135564579, 0.51122377983383904, -0.31768437156929069, -0.37179495179244848, 0.88543455031806517], [-0.87347002125031703, -0.8890649446641522, 0.0073362399521464759, -0.71384532102038389, -0.13520238692666242]]], [[[-11.460094786662683, -22.063891330382738, -2.4577615211196138, 0.55912742821044192, 5.4720557771170455], [-20.187265110591348, -15.520159961148913, 27.547850749766951, -23.249325420729985, 35.574791080657249], [7.6288634761606717, -23.648487667799174, -33.425679828191377, 14.364661008319414, 13.114102000363857], [6.028714382959599, 2.900930125059265, -34.199602871749342, 36.752186487229494, -9.5074666770410783]], [[-0.31998101600427897, 2.1685556107769717, -1.0987310522518137, -1.6574676114481133, 2.3641041025364968], [1.7886734791596994, 0.63546034596880951, 0.6343266167070013, 0.047518680705031555, 0.88359757533501337], [-1.8967488512958954, -1.8265508795185081, -0.06297719589171491, -0.26586471453943228, -1.6410623677889804], [1.0971408220687815, -1.1650056512633049, -1.1383901941405117, -1.8360544023762564, -0.073989930433031856]], [[-0.12534058380592811, 1.289090097895655, -1.1051372397205286, -3.3243199540658983, -0.28445829678181117], [-1.6811133197227015, -2.527713364639768, 2.0641911041372705, -2.6988161298946962, 1.2629032981109833], [1.433022678912792, -3.3690230867654956, 1.9843147820017015, -0.84131350170728258, 2.4979056832566857], [0.56027684610638684, -1.2934352036763914, 1.7978878733881805, -2.6042326405514982, 3.0126445543137046]], [[-0.56633873080520458, 1.0359742400800609, -1.3703927722576577, -1.319659633502025, -0.59227345676616228], [-0.38165425441652234, 1.385417653510548, 1.7441667623584274, -0.041495951453356709, 0.86082705891583133], [0.57502821183423714, -1.0781675051181898, -0.045636560177157746, 0.055273672206989546, 1.2242716556284328], [-0.72710164134948829, 0.68984930786683618, 1.4763651057370175, -0.4903054321781844, 1.068175061605019]], [[-1.1121802637775036, 0.58417865675079428, 0.96755166721662134, 0.86452660096342049, -0.62777217239975724], [1.2764601222110623, 0.0083645174930485917, 0.27771962968140451, -0.023865728944838859, 0.05056696772537194], [-0.8603709968375115, -1.2297183295279559, -1.0504816345363641, 0.55962892916401341, -0.86063209438964205], [-0.9986856717351611, -0.28797407543950165, -0.37205805324854191, -1.1976772897457753, 0.049511164954958387]]], [[[-0.85836054513137039, 0.26821381362067642, 0.51383894845161338, 1.2147018386050934, -0.87196790836803273], [-1.0344793142500566, -0.09646676983515913, -1.3237238887359049, 1.0712565690498186, -1.3257855872712285], [1.0039609145576853, 0.41505358991580749, 1.9971773253324199, -1.1270492753770265, 2.0450525025050004], [-1.357936222841633, 1.6487066212937111, 2.0368854623752215, -0.76278888330697969, 1.9243220324070547]], [[-0.93753140465706786, 0.67295388551203239, -0.30118089624047983, 0.86658360890456543, -0.096711819068558885], [0.93992419448764952, -0.66882230668947051, 0.56370140650110601, -0.50236333003025679, 0.47197734013540643], [-0.48324858571950224, -0.096525078322039576, -0.8979910372006602, -0.065009960926476137, -0.77532224390316395], [0.013789084575943021, 0.422157194486034, 0.72724779498227365, -0.80325215108399572, -0.48793827992505934]], [[0.82118844723845097, -1.146556419314388, -0.91277921209405788, -1.4103906919826632, -0.38023604379980702], [-0.47856984595911078, 0.35127827358018371, -0.93897767859794501, -0.74399073738288346, -0.80176246369460136], [-0.65871215520444659, -0.13461206407230089, 1.1764547713354498, -1.2829468316687318, 0.45097651128776317], [-1.338831353880841, -0.98981337558538807, -0.35388989864274473, -0.17412785215672191, -0.28815180915042632]], [[0.39411589804297065, 0.0586701029498942, 1.1463936454972159, -0.093922153083650234, 0.24411099894417512], [0.76703657185070928, -0.89014091356805758, -0.23642857713926194, 0.18378399278637558, 0.81967814938763484], [0.29742296671745055, 0.62161745629059983, -0.73699056421106968, 0.40316688388252864, -0.51850996428792095], [0.43464955988297543, -1.0820467921671402, -0.12831202299207992, -0.82990011073693826, -0.52869242659027627]], [[-1.9385729814474695, 2.1264775367266591, -2.1615544859981375, 5.150859754232548, -8.8148696560952846], [3.5955763962425884, -3.5657579234836652, 2.2560047569950545, -0.98292734637634649, 8.0974879101842117], [5.5601909291615526, -8.7978069280259419, -6.871153135926221, -1.5869212738805825, 2.1696361491551213], [3.9986986379147278, 8.9743439299265688, 9.3874055615151057, 2.3537157221200173, 3.5583301248815462]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_expandedData_rank4(self):
      arg0=numpy.array([[-1.0469673139890459, -1.9785720041783836, 1.3123112229705356, 2.2436054993860397, 2.7147710136850725], [-2.4780120074828735, 3.9521404229009303, -3.212579441597466, 1.3932724023714949, 2.5202362152404358], [-0.57676820425044983, 1.3467692628332362, -1.6882011534292287, -2.4453907638299297, -4.0534621101514379], [-1.6988587242597077, -4.4871230080251472, -3.4412042583868341, -3.7819171202680959, 2.1902855831489685]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.7106176319153521, 2.3204652560517189, 3.6556568086419468, -1.8275248998741356, 4.7681636012999533], [4.0973830727684977, 0.19818500265134897, -3.9373750984685616, -2.0470951581447316, -3.6663352267165736], [-4.5764409487173872, -2.3310123935624572, 3.0839035438726885, 3.2350125012490185, -4.9757833420967543], [4.0900277338846678, -4.1320641465317891, 0.32829923142401718, -4.5814673535763317, 0.74936410516817986]], [[1.4898597148444201, 1.5683196689103074, -3.267172877826293, 4.2067369606712965, -3.6012667592768386], [-3.9781925695164189, 1.9710367999077292, 1.5929253705607138, -4.7561142925895483, 3.6438516961538454], [2.5892611178905325, -4.6740897694148309, 4.1987011346063579, -4.512370068984306, -0.11727872751328494], [0.57629394593970318, -4.646711083237637, -0.26410359606443112, -2.7794113104409366, -4.409022147199348]], [[-4.2835034180078093, -1.7964062426363023, 2.8122547351744842, -4.8739742136857123, -3.6816531793123874], [-4.0942458684640046, 3.4050338436413625, -1.5173721504829061, -1.5381769646269472, -4.1979088590615943], [-2.8275267166188121, -0.16922458970516274, -1.1622768731225728, 0.9731580059374707, 1.0330535809425614], [-2.5519325841635645, -2.5285996196335558, -3.55803521605222, 2.9244431469582288, -0.33234104321942759]], [[1.4435985881632538, -1.7978882653877895, 1.9582403287504881, 4.9867236962094914, 1.4102518225095135], [-0.80396041696417342, -3.8394857029648541, 2.1094873776407805, -3.0224307566786415, 4.8879714708975577], [-4.0302682802900165, 2.3785414573243591, -1.8375513887621073, 3.5172891041697909, -0.86356452885561463], [4.4591584951761547, -4.5782935090897778, 4.5336427297388013, 0.64191515121097531, -3.5945304493311436]], [[2.3475512842853234, -2.4637749271786005, -0.81185759788214718, -3.0046606857987133, 4.9481537780909477], [3.3971663735720039, -2.6514627591104101, -0.36068268777977686, -4.2716867008908066, 1.2738424085721682], [-4.490485023892818, -0.075495959751724406, -0.61422112678379825, 1.6733354788054076, -1.2782352067888434], [-1.855149189120918, 2.0689954724401645, -1.885133530503861, -4.208164843984429, 3.1938949627940296]]], [[[1.1970271564600576, 4.0317626735470764, -2.0341258421148845, 3.9474718610223718, -2.411563348668444], [2.4311695939811004, 0.48051389382203347, -1.0764409636802075, -2.2909778254968503, 0.59078856086659748], [0.5547882920341376, -3.5851477369246867, -4.8049209530527568, -3.7859619794983121, 0.18623981107369492], [-1.5592877719783349, -2.9486430671233164, -0.44918090151107037, 4.663451786880815, 3.0162164920086525]], [[-4.7927232537014195, -3.8820157526195578, 3.1163684771430606, 0.81929543452767817, -0.024942178001477799], [-3.7139363994977459, -3.2357806781013831, -4.3043413163469859, -3.7080656127768661, -1.3493116928110926], [2.081366237891852, -4.2738169325849764, -4.7643693435272976, 1.9019039823958757, -3.2190449241581245], [2.387754388494379, -0.68305157397321015, 3.9394796979959104, -2.9752774597567266, 2.2879334031910989]], [[2.8834181459355968, -3.4521555183839148, 2.9712927007780374, 0.95666471862168923, 4.3610693860821002], [2.6997928985891413, -1.9186769893065438, 3.0038600675993479, 1.0277276974747043, -1.6375985481281843], [2.7551112332150653, -2.8644470886563589, -2.7391213156811158, 1.1913378180376979, 3.966107415227615], [-0.71049979293314269, -3.7916645166380403, -4.4147381306671374, -1.5897189753733585, -3.7743907229107556]], [[-0.020391779580844371, -4.0829781778949314, -4.0639683360551047, -1.1280974052969519, -2.3435000715777257], [1.7959242933898771, 2.9526971160663731, -4.6343068478895644, 0.78368444290604344, -3.9680256228952615], [-4.6755337275674709, -3.4361304157593722, 0.855021421030977, -3.7313165125466274, -4.2623078596881641], [4.2845561536654024, 0.24573794500390456, 1.6715537926957964, 3.1619998593748289, -3.7455272828224806]], [[-0.044841226194841433, -1.4921595559166412, 2.8102744940328179, 2.4399181154758498, 3.2933191091601746], [2.8440291414237784, 4.3993821021810913, 3.0463926674417152, 4.8319104424761576, 0.045158178738995858], [-1.9066941982094141, -4.131052090824002, -0.19509543634462911, 4.7893033582570528, 1.0831983579390023], [4.2311796707561875, -1.8543254367809716, -2.5048224784528972, -3.592606572119672, 2.3338738237611274]]], [[[0.79086333237225137, -3.4596832051900828, -3.6716005825198419, -4.2804308779365696, -3.8468772215782332], [-2.9480175033014708, -2.6417416029083962, 2.9312023810637751, 3.290574234021955, -0.17407947535445167], [-0.94410630799851791, -0.23711834402262433, 1.5436761236881269, 3.2664240320931199, 2.6001130066443547], [0.94033732466024134, -0.15264379793964267, -0.46479690983790345, -1.9720098055167523, 1.7168767240843916]], [[0.93207924341497517, 1.0631009861443648, 2.3702121448267333, -0.59466906736152758, 0.22827098656331035], [2.8860871135080028, 4.3060670438533144, 2.3466837998629231, 1.1333183134617109, 4.0583564219694761], [-2.7996947288826024, 4.2793639793672025, 3.3773745001664714, -3.8688354791976156, -3.5533691885121677], [-3.0753312881690653, 0.96181019183043581, 0.014927530035301118, 0.81161209007764779, -0.65249140201376221]], [[0.97257801715814374, 0.97219098096537238, -1.5358304026549319, -4.1888273143151018, 3.3438670511964279], [1.397313760678407, -2.7195773221579387, 3.6209802464285996, 3.0494346489744864, -1.6061532722378646], [1.365460811176753, 2.8523270281342796, 2.3804876374522497, -2.6412418041580454, -1.1545287235686521], [2.3295510378551851, 4.3597302435323453, -1.2114096485106662, -2.3551490947006015, -2.3903904445612678]], [[0.11392441864442482, -1.1919762079400273, 2.9275884850577816, -2.1417561482244993, 0.75385897013738123], [4.4483645637756482, 0.53971504945435367, -4.0095585640072535, 4.2734542174180952, 1.6838635247053793], [4.4755235392281829, 2.7271152567254662, -1.0192066349544184, -1.8288147362030038, -1.7983808460281892], [-2.9115222916061914, -1.4987715628375309, 4.8829144747792501, -4.4530009172784411, -2.2351099957802978]], [[4.6270125369985511, 0.079209961439003607, 2.4567160416986429, -1.3155379995536718, -1.534825835903888], [-2.9918949716493093, 3.1128056763842018, 1.5636010498805275, 2.2301690205973763, -3.2530874732284722], [0.14314501032181326, 0.55635785576280927, -4.0808613903949551, -4.5894578207153671, 3.8497104525090382], [-3.7822003168338023, -3.2526171632191203, 1.9884057975226153, -3.4157449274848939, -4.3558815093889427]]], [[[2.5724808085006838, -4.864898511415003, 2.779017337971144, 2.9542714112356849, 2.6185735744832828], [-0.24798509786864731, 3.3819745592613089, 2.667407980204759, -0.62546694251469148, 2.1870116296894988], [1.0318472185973837, 3.9953563370907723, -4.7794942324740166, -2.3656791902405008, 1.2462840454794097], [1.9663507428308913, 1.33087521467528, -3.0034424540004769, -1.499585523680441, -2.6099924886818791]], [[-2.2265219800786329, 3.177435306015278, -1.3037022618779437, 3.4590117886682155, -4.6904840836709045], [1.5653055014275221, 4.4477614990121932, 3.40279710023494, -3.2416096585385068, 4.4637259354493857], [-0.29745527529428362, -3.6639492014457087, 4.2720464067258099, -0.65975255801572175, 3.7800185007042586], [-3.9538503278274781, 2.4129892425339712, 3.4817332715681779, 3.3724226828377049, -2.0875977176630922]], [[2.8794338252312857, 0.37842581150166588, 1.8737786440842816, -4.4122308114048305, -2.0747477932069147], [3.9046726487763639, -4.6245740555216468, 2.493011481511914, 0.62251046152987932, 1.3163249916332038], [-2.3810411082341609, 0.70317060922827945, 0.99580869803881278, 0.9795636933569174, -0.35238775199624683], [3.7728855562272123, -4.349137427120966, 3.8720221146894964, 3.570248817196541, 1.2913058209925907]], [[-1.5261938790420304, -2.3685108325255912, 4.4152731182574207, 2.5332648135383096, 2.3422321432076574], [-4.9413038049865454, 3.7248784620279771, -2.5312350448221919, -2.5158859763813601, 2.3315961588154357], [-2.5454856366415957, 1.6400048561896385, -1.1015184504013042, -4.6218729378080958, 1.4853751056914639], [0.95792522526240198, 4.3196777251970069, 0.1895669429431166, 3.9942321767759843, -3.0819280187027864]], [[4.4926299354039205, -3.3673513880380055, 4.1495694037189583, -4.043814474006691, -0.96344971436182902], [-3.6031976467559179, -4.3954326327779656, 2.5973753349568254, 1.0903927756856069, -3.971706982337353], [-3.9676430328747481, -3.9274811327134538, -1.5317617270736172, -0.27781325284791158, 1.4191774237976809], [-3.1329524716215609, -2.0320700304186881, -0.70393165725697138, -0.26455854972843795, -3.5545249997270179]]]])+(1.-msk_arg1)*numpy.array([[[[-2.9011774791741471, -2.0836942595627872, 4.3642361066125268, -1.2862808336490539, -1.9709475628922926], [0.20034447993347726, -0.33029330004493751, -2.6565080248159303, 3.7088792481737585, 0.081245890579078939], [-3.8160740665441506, 0.72790529776807578, 1.025421449792506, -2.3208430205513664, -4.7218614075884897], [-4.544737722805503, 3.9990257646567571, -2.5568183078403561, 4.6009122826800706, 0.48139534658602745]], [[-3.4290668517105649, 0.62550762971848695, 3.4040113806073364, -4.1722093694251772, 0.16214661426837029], [4.278624398001309, -3.0985244390701103, 2.8404409787329996, -3.5914142944743768, 1.2295716726656227], [2.7734760955151883, -0.0044859257410712061, -2.0754428795420496, 4.3158366873720606, 2.2086294544053899], [-4.961298476645708, -4.7211049217535495, -4.8872024769242648, -4.3157426298291144, -4.6451398731158609]], [[1.6655205649640017, -1.0041223092558105, -3.120413548975062, -2.8813326798273309, -0.38199053833350316], [0.6707735008556126, 3.6298818016222878, 0.59532767755677174, 3.5200976599522988, 3.3204026004348624], [3.2361944028756877, 3.6975043947914408, -1.0497056696350713, -3.800285648064837, -1.7093596607011783], [-0.52973759062424541, -0.96133511683179584, -1.6632075513671962, 0.38615974507388273, 2.3202261685856236]], [[-0.9863346087772733, 4.801837056326514, 3.4734062417946472, -2.6265225012996876, -2.8235080230052678], [-0.62960947954451107, -3.4251049553125803, 3.0858118143585376, -2.1781019635893584, -0.27149210973475668], [-3.4987885378089825, -4.1300840924005371, 1.4553998957539278, -0.32422619389010165, -3.8128030413757252], [-3.3271900496378048, 3.3791446188006233, -3.7167612209022893, 4.4848927638094498, 3.5479050194068549]], [[-3.7927225449477819, -3.4251203735390945, 0.29210937824533101, 1.0267135799284404, -0.21556340057407652], [-0.23503112338583154, 1.4270340098299439, 4.3207119277672987, 3.939843062053086, 4.2067015495549072], [-2.6860561609275968, 4.1265004565141759, -2.734526197038718, 1.0797686812396812, -3.9360553018849496], [2.5312381746327333, 4.7640661376866156, -4.7178326690082208, 1.3379165708432028, 1.6687716861522661]]], [[[1.8122425104082183, 4.9370437241211924, -3.9698730574332721, 4.1107308971095886, -4.9589670270598756], [-1.9185264963544038, -3.1667949740392642, 3.5442285867159207, -0.8961243104406158, -1.6985078410966006], [3.7910463384173809, 3.950999181158684, 0.07018357326270408, 2.8535905161356361, 0.14515079375355633], [4.2476793987004484, -3.6384566800906493, 2.8300450941375397, 3.3391392504010131, -1.9936404102159431]], [[-0.93610477028764727, 0.72427792543589131, -2.3444602227707279, -4.511347332307734, 1.6697772027648377], [2.9330146783668454, -0.68965776048172245, -0.55838805385341139, -1.3259407293308922, -2.4784397860336007], [2.8170633497631279, -2.2592265769099362, 0.91206944071354279, -4.6966292170708499, -2.2258393690581579], [-2.060112068216422, 3.3744455222552112, -4.1329351414625002, -3.3284704073429605, -0.45396608086487955]], [[-3.1894555546707117, -3.2037873207671042, 2.0394417513938343, -4.1785269871797937, 4.6862103730382216], [-1.792814414075008, -0.65748381625392316, 3.9899495891513901, -3.3793934841576045, 4.9121193562239132], [-0.615902269822862, -1.8924778898929628, -1.6280366450330819, 3.6836542328429935, -3.3002639291902049], [-4.3762145438907041, -0.0087518373867165877, 4.3796707141179461, -3.109894426443669, 1.8923217133691042]], [[2.0542929772518246, -0.60896883471347074, 1.9640703573279943, -1.4916699321532101, 4.8046002215266075], [-1.3894412205838425, -4.1626809938511435, 1.380429175850006, -2.0731129353618227, 1.2260785811282373], [3.1720285545882998, 3.2972439885824585, 3.7630731043880257, -2.4115007216789652, -0.50774193023370273], [-2.9443686371974498, -3.461128404778778, 4.212888893644763, 2.756527476383682, -1.6619101250306434]], [[0.73476917199223468, -2.2787262818631557, 4.0217864663871268, -2.1777471558745276, -2.8231894785387257], [2.3183313509861065, -1.3851083512154516, 0.82130114780365471, -1.3291755992398233, -3.4509520746729705], [-1.0579856071023643, 4.587363564279844, -4.3168219959194785, 2.7883957354892974, -2.7536269332611054], [4.9297747721771863, 0.037246775473532168, -3.599556461476241, -4.736165460812277, 1.319649212111762]]], [[[-2.0564810208895787, -3.7526161129953906, 4.1670038496284327, 4.5450286132233213, 0.27298229351610814], [-2.9452652482363852, 3.0370060476950727, 0.84687341723839626, 2.1520025236775151, 1.38922992632682], [1.0423415149693014, 0.74308715966441774, -2.2977529704633426, -2.8383927921095853, 0.65042746644309002], [4.7072808079917259, -2.2151298421175669, -2.9604010774140144, -1.7118012972285968, 2.7368701802857931]], [[-0.61509643712310602, 4.6561441425330869, -1.7865373275373941, -2.5715510262665475, 4.2043864190092073], [-2.6077491300213937, -4.6392425933454708, -3.7944143868541258, -3.6472016265151899, -4.877563805442712], [3.5736105083145269, -2.4810043264225889, 1.4739441696348763, -0.73936221007641123, 4.4296482178559717], [4.7117579724175904, 2.0394974267088282, 0.56869487265861363, 2.7428276527141637, 4.4872867438615831]], [[2.1513865557035325, 1.2536670759187798, -0.31225513388445769, -0.19488124458696721, 3.0064560566613245], [-4.4131549897633047, -4.6933472936788441, 1.6224355624124707, -3.1140050937693253, -0.51995542777634629], [0.49789014640629681, -1.4369781659027572, -3.142764544496063, 3.9066728841886889, -2.6437609224065053], [-1.9233787368287381, 2.8095222191858928, 0.20813078777064931, -1.045051405428139, -4.2429798375508367]], [[-0.43991397098941487, 0.010645159057187215, 2.7029834255587817, 0.39032853404837198, -4.8128225787683112], [3.1251764960734629, 1.7311480972742359, 4.2814381621736501, -1.17614122123853, 3.0881355494092873], [3.0324092225176855, 0.96378166941756582, -4.7260779260064636, 1.2605247058316085, -4.0603129257743973], [-4.6086781823342129, 0.015397265321763776, 2.9999423608851208, 2.9290476527980651, 4.0936943394032692]], [[-4.4271583193218653, 0.81487574206261648, 4.4738155832117101, -3.4822959438049708, -3.6988816710556085], [-1.1513163749000057, -0.078874572338403226, -3.6904948910349522, 0.02109647146618876, -1.9127494802565179], [-0.81070447631700482, 1.8860838379811966, -4.97498240866181, -2.14497109850871, -2.8430096721096842], [-2.4022560061064135, -4.200473110846219, -2.6853595349281925, 2.6982137044630425, 4.0352096613876327]]], [[[-3.1408597877891253, 0.6932656589045898, -0.76430901629883774, 1.798536400965788, -4.3460055524668508], [-3.1888180862558912, -0.41121485446739925, -0.70398361627949591, 0.021282664610033919, -2.210825695886165], [4.5481058237453986, -2.9040094567990105, 4.3842732833515594, 0.91853377930537494, -2.2228499954527878], [2.6192184913893568, 0.10179035442601325, 3.8485591390603222, -2.8324371894864164, -2.2922379954290326]], [[-2.5578557280100966, -1.7085435405448623, -1.6645175512579415, 3.8441384134265579, -1.2445429230220251], [-0.62724840342997634, 0.42635111264055681, 2.9661309493777228, 0.13833988194251656, -4.4727558122455244], [4.5257271043020921, 3.1232710293977295, 1.9091736418015106, 4.0619439086364597, 2.5142926481723009], [-3.9988875270901802, -0.44304848421523246, -2.0902933103479935, 1.0879620572813344, 3.9349787499809636]], [[-3.9280508697463992, 0.74257578569001925, -1.8253034236408814, 1.970329177368054, 1.116639059036757], [0.67335771466776784, -2.3334246875796758, -1.6254041275718158, -0.82899853015682279, 1.0820805324264882], [1.7594676613098024, 3.121789096614485, -3.1711232625050787, 3.133233390600795, 3.6728881910445104], [3.0964105616298792, 0.80332889861732237, 3.6294212505791617, -0.47371147214067122, 2.864098260134158]], [[2.4861408193502736, 4.1852709998443309, 1.8048462374642824, -2.7980671724548634, -2.4600894690297439], [-1.562058893930256, -0.98521908339050857, -2.9948589673176129, 1.5138677706336185, -4.671214486492131], [-3.4999055595570807, -0.93758023310657546, -4.4039968959193185, -2.7293035748376582, 2.3697143769310189], [3.76885434433275, -3.2547668047751275, -4.240827755436003, 4.886431372962905, 4.680785248765396]], [[-3.284733692975319, 1.202152603951169, -4.1337869831541791, -2.0167605275550828, -2.797196542988055], [-3.5377934057250302, 1.0690562238346875, 0.820296533184532, -4.641349571874124, -2.1404329190042368], [-1.8965104065061733, -3.8241611632962802, -2.1947874565268686, 3.9762681732539988, 4.7781579260771814], [-2.668468172563724, -0.19191168873378484, -1.8029835868980926, -3.091182144540169, -4.4565045831517169]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.61204052527903197, -0.45118853266972203, -0.28639649966977826, 0.57288812538814227, -0.21957453676790981], [-0.25552097409374924, -5.2827777076093483, 0.26590489547116375, 0.51144047203839083, 0.28556235293483212], [0.22877325977132368, 0.44914703880616386, -0.33949418297119976, -0.32363624980887035, 0.21041256059751637], [-0.25598049258082822, 0.25337634578296864, -3.1890641639572643, 0.22852226878180729, -1.3971409983055898]], [[-1.3280257090413359, -1.2615871900357698, 0.60559146337391312, -0.47033413847264888, 0.54941000943115248], [0.49735450700389167, -1.0038229647822949, -1.242099624216495, 0.41600598355282925, -0.54298916892441151], [-0.76414541218242349, 0.42330637659663295, -0.47123430335888417, 0.43847733539810096, 16.870681035947101], [-3.4332687652169067, 0.42580052186067829, 7.4916511310799727, 0.71186729245355751, 0.44875528816184129]], [[-0.30636399575487466, -0.73052029759408055, 0.4666402394337561, -0.269248700431298, -0.35644618302032255], [-0.32052574885124369, 0.38540328326579643, -0.86485785478064192, -0.85316010650881735, -0.31261069904788052], [-0.46411983139095214, -7.7548494888181096, -1.1290865828250376, 1.3485078630230751, 1.2703225149010942], [-0.51424212031081773, -0.51898735283394348, -0.36883030753883222, 0.44873883916515744, -3.9486884023051116]], [[1.5541754597035631, -1.2479115318671445, 1.1457253057481647, 0.44991574349536334, 1.5909254386876741], [-2.7906914968004211, -0.58435052841934676, 1.063578537215641, -0.74231824647375699, 0.45900544075271699], [-0.55668887114993515, 0.94326945299910392, -1.2209756489572117, 0.63787918278489442, -2.5980750996792779], [0.50314549299226208, -0.49005278821280651, 0.4948792026925557, 3.4951745493987323, -0.62417206670304359]], [[1.1564267123184675, -1.1018746005317543, -3.3439004829996808, -0.90351999695546958, 0.54864321834647201], [0.79912807179666734, -1.0238767278013374, -7.5267571903607378, -0.63552671433486474, 2.1311670858313003], [-0.60456075440412615, -35.959156259657512, -4.4198593882633634, 1.6223710356175227, -2.1238430918399325], [-1.4633707249019123, 1.3121203259489413, -1.4400948101323436, -0.64511993097557407, 0.84998756856743407]]], [[[-2.0701385044688916, -0.61462248850643797, 1.2182196185593315, -0.62774659192658133, 1.0275541834101598], [-1.019267439679133, -5.1570038647012515, 2.3020417199758754, 1.0816394553908266, -4.1944143330195907], [-4.4665903067226136, 0.69118825479936674, 0.51572378228376359, 0.65452638481362702, -13.305490341709628], [1.5891947926577492, 0.84039063090142396, 5.5167350151057155, -0.53136863437807957, -0.82156304563955185]], [[-0.82461269171940876, -1.0180639839583476, 1.2681877807094448, 4.8238282020689276, -158.45209759415437], [-1.0641378843847185, -1.2213869900539354, -0.91817542625895621, -1.0658226783482625, -2.9290047984889438], [1.8988202801367262, -0.92473320342023091, -0.82952016057910527, 2.0779915597644005, -1.2277369580154374], [1.6551704153260878, -5.7860058793392612, 1.0032138063591141, -1.3283266775476048, 1.7273843798900246]], [[-1.1141566290431542, 0.93060101854895472, -1.081205981745335, -3.3581038153324778, -0.73664946763976735], [-1.1899355107113205, 1.6743722155955858, -1.0694837207130372, -3.1259052855063665, 1.9617625121062325], [-1.166043462371775, 1.1215356200223643, 1.1728503674539217, -2.6966150095772492, -0.81000817811009684], [4.5215768865111636, 0.84727417932163673, 0.72769422477885315, 2.0208473896104597, 0.85115179573141042]], [[-68.325199222941137, -0.34123924783987625, -0.34283544731648791, -1.2350639189749224, -0.59452628965933652], [0.7757968459470187, 0.4718643151003693, -0.30064310545297251, 1.7778487438196287, -0.35112485018554307], [-0.29799216165559977, -0.40547715999993172, 1.629517539679298, -0.37339968284292907, -0.32688216061273162], [0.32518476883062014, 5.669748733144802, 0.83351933300602699, 0.44063012787323907, -0.37198298054355117]], [[-56.203552603348875, -1.688985742340565, 0.89679361236483002, 1.0329183587166908, 0.76525721671809643], [0.88614992671233839, 0.57286140569398891, 0.82728541273599754, 0.52158173154155507, 55.809075689408019], [-1.3217831247439689, -0.61007127478214296, -12.917966009151176, 0.526221879617501, 2.3266617759979624], [0.59563441199603429, -1.3591121414024587, -1.0061536244265339, -0.70150631989560508, 1.0798510997389645]]], [[[-0.7292893482877657, 0.16671127673921257, 0.15708903822392639, 0.13474536108581842, 0.1499315343404235], [0.19564612611849483, 0.21832877356947525, -0.19676846879509305, -0.17527889153422502, 3.3132464529552612], [0.61091446944484906, 2.4324065125700196, -0.37363291133404608, -0.17657481042988088, -0.22182428332021362], [-0.61336308697396991, 3.7785236743029116, 1.2409036980293093, 0.29247735109476874, -0.33594037134963178]], [[1.4449085443625045, 1.2668309787931571, 0.56820621131855997, -2.2647373753753213, 5.8998705140292254], [0.46664193070605375, 0.31276086719450386, 0.57390316620922899, 1.1883415690332766, 0.33185090780658039], [-0.48104146817847915, 0.31471248281909064, 0.39876219316716394, -0.34810714233642004, -0.37901191556094466], [-0.43792656355896248, 1.4002443249953287, 90.220502631604262, 1.6593755555124732, -2.064041393766642]], [[-1.7358002377661419, -1.7364912722733428, 1.0992106618744486, 0.40302476725643194, -0.5048649146577745], [-1.2081761454990565, 0.62075865233707328, -0.46622766172069413, -0.55361119281469584, 1.0510834691866275], [-1.2363600182522545, -0.59186802101492864, -0.70918291146264878, 0.63916948110223493, 1.462242661413389], [-0.72468948994719318, -0.3872260573767547, 1.3935840411249327, 0.71681285793239402, 0.7062449388844847]], [[-21.465027365751681, 2.0515432670053482, -0.83529183705669108, 1.1417689945034786, -3.2438305581006595], [-0.54972804696437694, -4.5308923038225348, 0.60989027215653002, -0.57222814131547395, -1.4522499762905623], [-0.54639211310050317, -0.89669505452666054, 2.3993081284633648, 1.3371451549581588, 1.3597735814583953], [0.8399010960279778, 1.6315967185821325, -0.50080556939111298, 0.54915568383140811, 1.0940807246384403]], [[-0.87604303591985433, -51.173640745587853, -1.6499514153653507, 3.0812200875434028, 2.6409915804969999], [1.3548143061709583, -1.3021892567543412, -2.5923889667771438, -1.8175582535289956, 1.2460353874618217], [-28.317173620223269, -7.2857102100119189, 0.99328590764979019, 0.8832115401203573, -1.0529264889284349], [1.0717206310068519, 1.2462155571176166, -2.0385487284344612, 1.1866992987489007, 0.93057217038947204]]], [[[-0.66039704500258323, 0.34920743367482465, -0.61131634590663642, -0.57505167527892265, -0.64877257634242325], [6.850648441623532, -0.50232747008918144, -0.63689496952367131, 2.7161447053131886, -0.77679455435767542], [-1.6464246776465703, -0.42520831208180432, 0.3554473845196639, 0.71812726394528481, -1.3631392702345038], [-0.86396525668351309, -1.2764973797143047, 0.56563718142722841, 1.1328855189867328, 0.65090559901100709]], [[2.0153059561831399, -1.4121839080501397, 3.4418311137710096, -1.2972268619393095, 0.95664390454841897], [-2.8666116639422756, -1.0088497346410539, -1.3186572328145401, 1.3842268134307649, -1.0052416015037908], [15.085034224341353, 1.2246684550797355, -1.0503450994728722, 6.8012210843420791, -1.1870637689178365], [1.1348742708960069, -1.8595702496017139, -1.2887612743534889, -1.3305339899592552, 2.1494193876817138]], [[-1.1950975321026609, -9.0934713061233285, -1.8365052186132453, 0.77992389915140758, 1.6586132876747408], [-0.88130416245398435, 0.74411269385514689, -1.3803403168844928, -5.5279460684560124, -2.6142512527375392], [1.4452519305468507, -4.8938397214347606, -3.4556880906584624, -3.5129969411116018, 9.7653912171825006], [-0.91208816358266354, 0.79123833542892574, -0.88873569325230717, -0.96385558390548354, -2.6649026144261292]], [[2.4780056925938858, 1.596748922712488, -0.8565533816310571, -1.4929024001188984, -1.6146636580133378], [0.76536826504202238, -1.015312891097409, 1.494099541646382, 1.5032148339678291, -1.6220292291909992], [1.4857350070369262, -2.3060401961582859, 3.433367020670667, 0.8182650564300572, -2.5461023991697762], [-3.9480295753064913, -0.87550909138612065, -19.950298620382018, -0.94684458811824446, 1.2271270118307116]], [[0.48752860009424426, -0.65044758647096323, 0.5278344257083577, -0.54163849435426503, -2.2733782059396557], [-0.60787272802560732, -0.4983094421275846, 0.84326880049678166, 2.0087124859863286, -0.55147209824124122], [-0.55203695619814908, -0.55768201275501073, -1.4299127236541158, -7.8840212289945608, 1.5433486655162698], [-0.69911229199570823, -1.0778593012848487, -3.1115031701854563, -8.2790202221672153, -0.61619642098935279]]]])+(1.-msk_ref)*numpy.array([[[[0.36087668593342209, 0.5024572627122017, -0.2398970377433797, 0.8139492454527999, 0.53119998405876412], [-5.2258355924589628, 3.1698109342411804, 0.39411411680624991, -0.28228670817594548, -12.886403318700811], [0.27435717853798969, -1.4383290205460617, -1.0210117159153436, 0.45111509254094928, 0.22172766703962715], [0.23036913851713858, -0.26180559356283839, 0.40948052928851952, -0.22755646047204761, -2.1748596479254672]], [[0.57700012561475356, -3.1631460755624206, -0.58124717662529413, 0.47422644191295216, -12.202363972297514], [-0.46243180520885213, 0.63855297677502498, -0.69657212348096065, 0.55091722701626056, -1.6091554873649474], [-0.71339068232021419, 441.06213931796276, 0.9533252028670427, -0.45844459545181454, -0.89583700888887097], [0.39880124396709943, 0.41909087744728346, 0.40484756126240701, 0.45845458681967949, 0.42594454811351024]], [[0.78792856154189828, -1.3069236793903469, -0.42055682760433477, -0.45545286462692613, -3.4354547855968112], [1.9564148275037736, 0.3615300152153797, 2.2043511035070105, 0.37280534511883934, 0.39522653752851128], [0.40551062748406391, 0.35491809687072923, -1.2501706534811408, -0.34531910085200684, -0.76772095021373465], [-2.4772854450900899, -1.3650923595670019, -0.78902432946013534, 3.398363603952181, 0.56559625123549984]], [[-2.2746900285364253, 0.46723899063381286, 0.6459381204505491, -0.85421141386598887, -0.79461630039853925], [-3.5634874827633931, -0.65504722589771991, 0.72707139461530279, -1.0300736773997192, -8.2639804949691005], [-0.64125210058880389, -0.54323482263093203, 1.5415732170461678, -6.9198773623654937, -0.58843991547397323], [-0.67432441967968637, 0.66395663769559932, -0.60364531538062516, 0.50025844931916064, 0.63237473582681469]], [[-0.7157842371837535, -0.79260601602739145, 9.2936797510316307, 2.6441366577368988, -12.593840171639734], [-11.550687307180389, 1.9023870454275893, 0.62831567090563101, 0.68905562250248165, 0.64534433491539489], [-1.0106903396791076, 0.65788700190241856, -0.99277564670068297, 2.5142153693217395, -0.68971871721034694], [1.0725071393485004, 0.56984326733199786, -0.57542757536073641, 2.0291033632793161, 1.6268079307748782]]], [[[-1.3673732920682269, -0.5019222324031507, 0.62420434397593461, -0.6028154285714149, 0.49970326359521355], [1.291622509353719, 0.78249840226383705, -0.6991682242987034, 2.7652547516141577, 1.4589346881572269], [-0.65364856724947085, -0.6271861607311604, -35.307578287691747, -0.86838388110380493, -17.071983854874098], [-0.58338018830729221, 0.68106129201492538, -0.87560866525275327, -0.74211101174809568, 1.242958356374039]], [[-4.2218996722840245, 5.4566628142400146, -1.6857357546592175, -0.8760443680755684, 2.3668669187463616], [1.3474669772541172, -5.7305821080594823, -7.0777667889335794, -2.980631287263718, -1.5946082068129577], [1.4029291969004691, -1.7493333618208768, 4.3331573742993044, -0.84148444346768436, -1.775573061488817], [-1.9184104029459741, 1.1711969853523176, -0.95625512804500656, -1.1873743609623468, -8.705805542501011]], [[1.0072501047687876, 1.002744289788954, -1.5752249062283163, 0.76883060740161135, -0.68553888661951956], [1.7919196858169937, 4.8861726512166399, -0.80516792751778588, 0.95063787530450272, -0.65401086753459259], [5.2160539082303226, 1.6975519020616758, 1.9732844782079129, -0.87211753289831484, 0.97343106809816438], [0.73410007881864492, 367.07485521537143, -0.73352077160542217, 1.0330188106324956, -1.69769200390232]], [[0.67822477991205299, -2.2879206996316177, 0.70938008772097272, -0.93403531997210709, 0.2899871660765978], [-1.0027573543456862, -0.33470554299730176, 1.0093037924336687, -0.67206777720883082, 1.1363646864211638], [0.43923703030861549, 0.42255665859003855, 0.37024856114191207, -0.57776155314664523, -2.7440562211007498], [-0.47319903655055195, -0.40254860248692431, 0.33071662641596777, 0.50544477220279471, -0.83835604668802699]], [[3.4299699977982616, -1.1059846175029913, 0.62664595355914765, -1.1572675957546474, -0.89269113334359074], [1.0870905982306838, -1.8195227925879545, 3.0685896665067598, -1.896089739144925, -0.73030171405068445], [-2.3821082237053468, 0.54938663132449583, -0.58381749759955726, 0.90383017846575286, -0.915242433460561], [0.51122745596091368, 67.663205289578272, -0.70015187765851661, -0.53212588033362374, 1.9097773803141522]]], [[[0.28046366506263953, 0.15369763036860848, -0.13841316808523699, -0.12690089619510833, -2.1128410814543028], [0.19582895109220355, -0.18991341972736159, -0.68105597898119952, -0.26801465049623729, -0.41517116304530544], [-0.55333899299543543, -0.77617840215530243, 0.25101401746165269, 0.20320239180912555, -0.88675253430570633], [-0.12252683189650572, 0.26037670265824453, 0.19482772407118279, 0.33693642199257384, -0.21074006666630485]], [[-2.1895253842344928, 0.28924561216452199, -0.75384333821317639, -0.52371866203585116, 0.32032480571816985], [-0.51644893572342498, -0.29029938308573083, -0.35493468175198861, -0.36926098437832749, -0.27611515021708605], [0.37686515072075727, -0.54283229113718245, 0.9137179620357373, -1.8215283990428059, 0.30403526343342369], [0.28583158785259349, 0.66034369310607111, 2.368175497234875, 0.49101490627766614, 0.30012997602071206]], [[-0.7847037757829457, -1.3466104246153152, 5.406480055037, 8.662717425718494, -0.56152530474833529], [0.38253837840392135, 0.35970088037229936, -1.0405351020036619, 0.54213178931758188, 3.2468189834060004], [-3.3907101106829178, 1.1748272823398433, 0.53717073917795799, -0.43213271330236364, 0.63856044588650995], [0.8777268465662309, -0.60088549643804345, -8.1112514468044488, 1.6154240304931247, 0.39788102184423868]], [[5.5587931393267125, -229.71857448939602, -0.90470061366521315, -6.264955160892443, 0.50809908817701521], [-0.78248085089030006, -1.4125832259413844, -0.57116106112073928, 2.0791642361236367, -0.79186639469167575], [-0.8064184562133806, -2.53728706555265, 0.51742497735247572, -1.9399784490670711, 0.60226657613182266], [0.53060566763014538, -158.81981070842568, -0.8151459160396759, -0.83487571856125087, -0.59735548408980388]], [[0.91559005072407962, -4.9743315464162663, -0.90604139458996102, 1.1640199958773108, 1.0958615253552129], [3.5207195854427846, 51.391240421063408, 1.0983519093870622, -192.13934029905937, 2.1191808713014679], [4.9999256554819338, -2.1491420627887536, 0.81476913427755293, 1.8897513877784204, 1.4257644460082068], [1.6873564265622572, 0.96500132322828636, 1.5094671895619478, -1.5022761553122035, -1.0045232962585464]]], [[[0.54088970506243039, -2.4505161945336051, 2.2227380392376133, -0.94457844909196453, 0.39090118587064687], [0.53275498266331034, 4.1313165266367866, 2.4132077579277444, -79.823591424673594, 0.76842725657698419], [-0.37353104569160728, 0.58500454269604929, -0.38748924039721688, -1.8495332044777273, 0.76427052105855453], [-0.64861283235617095, -16.689781009599788, -0.44142721025576004, 0.59978690103548193, 0.74113540027144365]], [[1.7542517972723735, 2.6262854305686485, 2.695749891392885, -1.1672636428367964, 3.6054385309021093], [7.1536619041009208, -10.524478241031586, -1.5127865507645641, -32.435498317756633, 1.0032121574221173], [-0.99147007864432468, -1.4366742321720358, -2.350295913257562, -1.1046737987899529, -1.7846462746836347], [1.1220928264742258, 10.127837399044815, 2.1466475474095681, -4.1243377726221837, -1.1403169605545784]], [[0.87605898510397939, -4.6341455305995263, 1.8852779290375519, -1.7465123583986917, -3.081751646190229], [-5.1105143424171082, 1.4747440861081285, 2.1171376398112352, 4.1510378284215497, -3.1801738921133529], [-1.9558212600651579, -1.1023179823770761, 1.0851688734635943, -1.0982917099983369, -0.93692050489786649], [-1.1113527065918105, -4.2836804007795353, -0.94814132083392277, 7.2643464656582131, -1.2014965779231483]], [[-1.5211998816931294, -0.90362538540724424, -2.0954234448146107, 1.3516177014971584, 1.5373087718470984], [2.4211104555427561, 3.8386559741140012, 1.2628030773867869, -2.4981819374390946, 0.80962180846209508], [1.0805769058370547, 4.0336997162761241, 0.85874654538752448, 1.385671112270114, -1.5959379565253764], [-1.0034659805717854, 1.1619625451259907, 0.89178748545501207, -0.77396300727639544, -0.80796638155224365]], [[-0.6668076586644085, 1.8219696700319563, -0.52984964926221911, -1.0860414775195197, -0.7830288467356803], [-0.61911065230788809, 2.0488029855833489, 2.6701143971021111, -0.47190704971281794, -1.0232909257291389], [-1.1549030132579128, -0.57274928791469304, -0.99794883401373902, 0.55083950269796245, 0.45839539358783199], [-0.82080258841710563, -11.412986867033819, -1.2148117149070681, -0.70855921156816393, -0.49148060823937506]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[0.59619899874328297, -0.73770392664902218], [-1.1319267727414664, -1.7432510775579626]], [[-3.2393471443425681, -4.1504200525592534], [-3.6318073722246327, 2.8543728508611919]], [[3.2868888376063943, 0.093326553630252107], [-1.7528143630796889, -1.0831992500813148]], [[3.4512766226383711, -1.6067417245066631], [-0.91390243855859588, -2.9563821306728046]], [[1.4433274447573661, 4.8857312776191968], [-4.1381061533108996, 0.40617722060185901]], [[3.8864295536534375, 0.21555204135575146], [0.95399108470228633, -4.0940928246977943]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.509319241698881, 0.31327239033268306], [-1.5284494598374354, 1.461503338936109]], [[2.987718496564721, -0.6983155739909277], [-2.6464294722546153, 4.3177817064212807]], [[4.8511892511147074, 2.3998567868038148], [2.7534490666003419, -1.6532463926450882]], [[2.7907567836199583, 0.73720106367167748], [-0.27378163699369296, -2.6449806929379225]], [[0.5406379145011968, -3.1316221905132888], [-4.9588942641129226, 4.3154554530179894]], [[-3.4066912263773008, -1.311340050593218], [3.8155663154683008, 1.0333548423366832]]])+(1.-msk_arg1)*numpy.array([[[-3.1921019160633115, 4.1567295378213629], [0.09924285367620822, 1.9361957649677439]], [[3.3348860327056453, -1.4152317347077146], [-4.5537343668711339, 2.9333806059436043]], [[-3.956270449474184, -4.3462284798710051], [2.0872452641936601, -4.2628826346641855]], [[1.9456999606036849, 0.97554286773031773], [0.75093470546600294, -1.6748693375930355]], [[3.4326134023919881, 4.1657764859920334], [2.5809383740529075, 2.228785651542883]], [[-2.864508443776864, -3.8770751367983056], [-3.6299375459300656, 4.3191902678231475]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.39501185850662374, -2.3548322463579039], [0.74057193416251865, -1.1927794012615462]], [[-1.0842210027709003, 5.9434734196736878], [1.3723423995616737, 0.66107391362936452]], [[0.67754290081573965, 0.038888384566708498], [-0.63658862782011527, 0.65519529024845802]], [[1.2366812625504529, -2.1795162862410726], [3.3380706193222491, 1.1177329719518641]], [[2.6696748526950951, -1.5601279402156747], [0.83448162693405392, 0.094121518579875801]], [[-1.140822368508664, -0.16437539695233208], [0.25002607891646589, -3.9619428457314712]]])+(1.-msk_ref)*numpy.array([[[-0.1867731715403845, -0.17747219777875414], [-11.405624997790914, -0.90034856448877953]], [[-0.97135167816047829, 2.9326787626172282], [0.79754484553301774, 0.97306597209979262]], [[-0.83080489051077, -0.021472997579966624], [-0.83977402806892087, 0.25410018124194622]], [[1.7737969329904066, -1.647023188478516], [-1.2170198446101397, 1.7651419512649498]], [[0.42047480317812524, 1.1728260731338096], [-1.603333963690398, 0.18224149115491736]], [[-1.3567526959456846, -0.055596560229099569], [-0.2628119830248633, -0.94788434193272963]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[1.7244206466751679, 1.5541370985360228], [-2.9856536197430139, 1.4051465920103894]], [[1.1123311966094125, -1.4923151530164547], [-0.93972493679888114, 0.76695701312263331]]], [[[1.8882345982672319, 2.2453271137016486], [4.4176484566410288, -0.51174339803225966]], [[-3.109066282849021, 1.2587059644044558], [-0.19724918825562554, 2.4968438365477787]]], [[[4.8147532177215151, 1.9362726777000026], [-1.1025588714451229, -2.1123082282233216]], [[1.9281718691139123, 4.6716214707373496], [-0.50687389197683341, 3.3375323499779217]]], [[[-3.0730230808869541, 3.5216867396117646], [-1.8445583253821116, 2.4161429966300494]], [[-3.0920506039796756, 0.33241879177112921], [0.90176452449660616, 1.7794444968038334]]], [[[-3.5221840928089163, 0.97767172908311117], [-3.4379895218445702, -4.2305930027117231]], [[-4.2553856570569142, 2.464805936669733], [-3.6071555105315567, 4.3704269074540427]]], [[[3.271432377263201, 3.3520387271201475], [3.9454338818463075, -3.0942730623781358]], [[-1.8423090454541216, 4.3824897391410236], [2.1450995103132318, -4.8146651693331819]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.1927447313871555, -0.39829562288376774], [3.3223951363427418, 0.52333543242431979]], [[1.8023406989286626, 2.9690409574411172], [-1.4749836383852442, -0.27064853333087768]], [[-2.8984191550413971, 1.6758963956067143], [-1.0453711120665421, -4.4337941922123347]], [[2.6440892207863325, 0.86222665834878676], [2.1127404641093097, -0.85474894687115288]], [[-1.3781434052524446, -1.0836044733679797], [0.16984150803145148, 0.92945843581416909]], [[3.6526979988145687, 4.4514453273883792], [4.6559703470109, 0.49469856713297666]]])+(1.-msk_arg1)*numpy.array([[[4.5615450710293199, 0.18504887281118698], [1.0597720128442329, 4.036201028946433]], [[-4.5594119073927981, 0.042254356118220926], [4.7506125269876822, -1.435281518146263]], [[0.99813679721957005, -1.4270204728064284], [4.6405663036299742, 1.6661171300729096]], [[-3.9592207503182619, 3.0544107264051874], [4.9491423725228039, -0.26112440213253052]], [[-0.43509402423257804, 0.77088108358077534], [0.1506288012125605, -2.4591784430676125]], [[2.0671729997359893, -1.8562300713203306], [-0.45598151023147793, -1.4113453836227006]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.4457583431701067, -1.3029922142087937], [7.4960743935021839, -3.5278986543631814]], [[0.33479798487601181, -0.44916847387971431], [-1.7956455431379108, 1.4655170768196437]]], [[[1.047656860542308, 1.2457839491924605], [1.4879041818434195, -0.17235983112651576]], [[2.1078649294392906, -0.85336944197051512], [0.72880198472932878, -9.2254105567063842]]], [[[-1.6611652629147586, -0.66804439735092336], [-0.65789202383597845, -1.2604050189263734]], [[-1.8444855103200675, -4.4688641352469114], [0.1143205728554392, -0.75274859528664539]]], [[[-1.1622236710957354, 1.3319091927482087], [-2.1392963294762262, 2.8022132849117667]], [[-1.4635260016583362, 0.15734009804714491], [-1.0550051308019226, -2.0818329210203421]]], [[[2.5557457078740886, -0.70941218842463194], [3.1727347074886687, 3.90418562001918]], [[-25.055039291507562, 14.512388433416977], [-3.8809218051497161, 4.7021219444048841]]], [[[0.89562082009651445, 0.91768843967062264], [0.88632648312476436, -0.6951164924659462]], [[-0.39568745248493065, 0.94126238195536427], [4.3361749009000503, -9.7325229730026361]]]])+(1.-msk_ref)*numpy.array([[[[0.37803433262713543, 0.34070409791771045], [-16.134405870115188, 7.5933809845149316]], [[1.049594802587889, -1.4081473514396323], [-0.23282411605850489, 0.19001952767521879]]], [[[-0.4141399453744416, -0.4924598082618929], [104.54894743351799, -12.111021088582762]], [[-0.65445587599215338, 0.26495656239145843], [0.13742891952679959, -1.7396195833222849]]], [[[4.8237408250388008, 1.9398870807024875], [0.77263003051160994, 1.4802227918070316]], [[0.41550357067533872, 1.0066921071859447], [-0.30422464473110156, 2.0031799023828958]]], [[[0.77616866415946351, -0.88948986724942625], [-0.6038998977563893, 0.79103408580340751]], [[-0.62476493324307347, 0.067166948685228503], [-3.4533904802927093, -6.8145469449488596]]], [[[8.0952251620126709, -2.2470355248099203], [-4.459818245733782, -5.4879969074613157]], [[-28.250810089445693, 16.363443888738853], [1.4668132443581208, -1.7771898252337937]]], [[[1.5825634224523126, 1.6215569415565392], [-2.1255090857567742, 1.6669663476452514]], [[4.0403152411133467, -9.6111128210357979], [-1.5198969261564454, 3.4114010824017416]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_expandedData_rank4(self):
      arg0=numpy.array([[[0.46011252672322556, 2.7203063200770998], [4.3999337602866859, -2.7308978011877305]], [[-1.9799560072048195, -4.586782479563535], [1.6197507722703168, -2.7141765040664199]], [[-4.8126132773144707, -0.38439892049291124], [4.5183603069840181, -0.15763628100527427]], [[-3.6320170381479144, 4.9627397167146725], [0.66959848728458926, 0.38246432699021948]], [[-0.028225132206422643, 2.0411958620962771], [-2.0171183671366144, -0.7345555929846137]], [[-0.93695509443552449, -3.9406518605427463], [-3.4214255966350491, 0.53856179299498308]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-1.1173667868651949, -3.8848546481098567], [-0.66152820702394788, -0.080840955869470932]], [[-3.8768443519801874, 3.8180973544861772], [-3.9042878729373163, -1.9874980208560111]]], [[[-1.1247824688692818, 3.5156014939901947], [0.10278424596275393, -2.8695751033790398]], [[-2.86850129074399, 0.9737186671935163], [3.1170211269225252, -4.0766004150817743]]], [[[4.5181916072484469, 1.2292040777868305], [4.2714383428136919, -2.2462869293559873]], [[4.1626805478781783, -0.85503944896801087], [2.4322250985314131, 2.7571508437178069]]], [[[-4.7515973513689138, -4.9668591539757436], [-3.8418426791810401, -0.49864042776986306]], [[-4.1323827902361545, 0.99174392384617605], [3.3166913645149627, -2.6302052453821423]]], [[[2.9219849505352391, -1.864561144055715], [2.8175316854787269, -0.64445434312374061]], [[-0.50201111351109606, -3.3340298825489398], [2.2888219939412737, -1.3107869446993892]]], [[[2.7920860564687686, 3.9523098583526064], [-2.9175009337963709, -0.41964649289410882]], [[0.87960135178848198, 4.4836945724546613], [-4.9541012393898756, -4.4714435029804411]]]])+(1.-msk_arg1)*numpy.array([[[[-2.8904042247947315, -1.8564568729527817], [1.5920013033183888, -3.4293997643402161]], [[2.5561404160278265, 1.2544010441988043], [2.7471453370513421, -0.42705042644462132]]], [[[0.89171849471811537, 1.5160679262484607], [2.7808869850488991, -3.2269945219431193]], [[0.42581045114317106, 1.5585431474355103], [-4.6469459926502719, 0.0062326879405025792]]], [[[-4.9093086271813835, -4.2670801443057833], [1.0467733679355682, -3.1367772815773809]], [[-0.067861471281701391, 2.4085796847403431], [-1.0381724701369266, -4.2653842290350843]]], [[[-2.4443497535220979, -2.6604193987793909], [-1.3779119820739751, -0.19396102836864326]], [[-1.6735100669799783, 1.2963310854475916], [-0.080319604771760034, -2.8473429092097238]]], [[[-3.542139741493302, -2.2388103317614352], [-2.8096137056177986, 0.76265691212954678]], [[0.79395691354085063, 1.8871389235089602], [3.2226726668278545, -3.4605615172094262]]], [[[4.7811598470186354, -2.9988089673310556], [-0.73551445691055406, -2.185919401728571]], [[4.8659486277512016, -4.124978134154544], [-3.7293264708536955, -1.4194851530168795]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.41178289182380717, -0.11843751398706498], [-4.1121546915060305, -33.6501008779438]], [[-1.1349265951415664, 1.1523890963955814], [0.69946117962177612, 1.374037997789572]]], [[[1.7603012689157791, -0.56319125207720189], [-44.625345417484056, 1.5984186906843507]], [[-0.56466796005875586, 1.6634689534491676], [-0.87075973936248585, 0.66579409991351213]]], [[[-1.0651636087309111, -3.9152272305991143], [-0.089992852440355972, 0.17112636656934954]], [[1.0854448846157889, -5.2843881208492185], [-0.064811550995200098, -0.057173615061522645]]], [[[0.76437811741383066, 0.73125025807116972], [-1.2917602648353557, -9.9525418324186106]], [[-0.16203689766269777, 0.67517276504982848], [0.11531501878111941, -0.14541235048546611]]], [[[-0.0096595748041934503, 0.015137681215982288], [0.7244624337736445, -3.1673242392973533]], [[4.0180751239325492, 0.6050090845599988], [-0.32093172598351954, 0.5603928204770714]]], [[[-0.33557529226750216, -0.23706519175246654], [1.3506942928086778, 9.3904081822914396]], [[-3.889745723648911, -0.76308177137095712], [-0.10871029213389871, -0.12044472721974563]]]])+(1.-msk_ref)*numpy.array([[[[-0.15918622135141028, -0.24784444682056903], [1.708733726792093, -0.79323103371136461]], [[1.7213192720938486, 3.5075973355052157], [-0.9940856657110646, 6.3947900109212643]]], [[[-2.2203823504083662, -1.305981066497633], [-1.6493955001493457, 1.4213790721905613]], [[3.8039244173593683, 1.0392723325854147], [0.58407747978117897, -435.47447425188426]]], [[[0.98030367263293672, 1.1278469385527421], [-0.36722267901314437, 0.12254581246508194]], [[-66.582115324728861, 1.8759438749775554], [0.15184016677352591, 0.036957111608427073]]], [[[1.4858827108986714, 1.3652046890855989], [-3.6016376817080622, -25.586272450991885]], [[-0.40011619917706792, 0.51653354208766289], [-4.7617804902930994, -0.13432324071440063]]], [[[0.0079683847240096285, 0.012607201157686224], [-0.72650409485650058, 2.6764274074389487]], [[-2.5405892092314777, -1.0688764573759995], [-0.22793366529144038, 0.21226485624707359]]], [[[-0.1959681592782925, 0.31244240785014588], [5.3576810401457129, 1.8027434394088713]], [[-0.70313639916421833, 0.82944090498465273], [-0.1444126163810214, -0.3794064290495463]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[-2.7283726157810873, 3.4863053269351507, 1.9184768473220011, 4.9142874263444778], [-1.1602722905411191, 2.1153985698774616, -1.4382340681030392, -3.7686589975231453], [-1.3649787825248438, -4.0823654207943321, 0.51308652235537267, 1.1181695823143789]], [[2.9606346632347691, 4.091085928719183, -3.9282963404969964, -3.3022009481627803], [2.4303502582288754, 2.9159278994644158, -4.8670776608110078, 2.3085987619561541], [2.2074615231784804, 4.8138927663771032, -2.6748767713501143, 1.4496009580415308]]], [[[-1.2334518473388743, 0.92579278349226435, 3.6063813491292684, -4.3949408954941251], [-3.5307882408647853, 1.8580660376762541, 1.3366035578662929, -3.3706350534695053], [-0.90624145188815319, 1.1337002175177444, 0.16300418883366063, 4.9743538276040304]], [[2.7395989664639577, -0.79244958887890515, 3.736410961294288, -2.212138810137195], [2.7722788648447665, -1.2492467261476747, -0.86361099807256281, -4.6202752023917917], [4.4074540906397921, -2.7252428367925696, -2.8155191012347225, 4.9985798225158398]]], [[[-0.70523164821764261, -0.6989853140184854, -0.71838157294277227, -2.5695756470202036], [-1.0782296058549212, -1.8642572709941998, -2.1153097364893672, 4.7348098713709845], [4.4325149828414627, 0.84503532841849172, 4.631648990277009, -1.2054565857241561]], [[1.0098875418171591, 3.4523690014995498, -0.35085574312319423, -4.500401911098681], [1.9606286810077336, 2.6683863708452025, -2.8408073629688877, 0.44221566136076973], [-0.87821449460451984, 0.54277259886698914, 2.5994074823987265, -4.2784712688655455]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-4.8687980377650586, -2.9920569561224166, 4.5679137607812272, -3.4127311169387586], [-1.1487420755443987, -2.2942259449173816, 0.2442499359424648, -4.5043377827692135], [-0.89752883726233179, 2.2849275749633708, -2.9517222377677124, 1.6222660538498346]], [[-2.4644369065092886, -2.6571561125641443, 0.44642462951960127, -4.5981218310928051], [-2.2285447871939312, 4.272817494589475, -3.8272576119989834, 3.5644185700373345], [4.3415365965762547, 3.1915382380027246, 0.76761418746953947, 1.7763510573444705]]], [[[-0.98314954983969383, -3.401647667199752, 0.17757677528511184, -4.7934710694919316], [-4.0377050499412181, 3.9147378093365983, 2.7730745593529935, -0.22599496909395089], [-3.1685112008279104, -0.62909016601815715, -2.8397022105553891, -1.1311062205399383]], [[-3.1100412960725365, 2.2923628012836534, -1.9224627610541534, 2.9348074395124355], [-3.5606800741362479, -1.8733172344376978, 4.5789822312663535, 1.6539958806270718], [1.4215815862566181, 4.6222820780030869, 0.3513188681739905, 3.1729072103200657]]], [[[-0.16145332690034131, 0.82270987607836155, 3.1156619103715197, -3.4218806883212185], [4.544760467130919, 2.0889125123048755, 4.5972453976185097, -0.29086457452406655], [-3.4419524062016391, 3.3165386702973816, 2.3081542602821612, 1.3187215657907316]], [[1.9695255521702579, 2.5950943914785753, 0.66437144656254166, 4.4921429693580013], [3.720767076639234, -3.1371094709915259, 2.7627206730069531, 2.5171430524596339], [-1.8182292979767301, -1.2994099537192083, -3.1189488279142963, -4.4149445583446187]]]])+(1.-msk_arg1)*numpy.array([[[[-3.7906389303562573, -0.28578839507567988, 4.7929569686889266, -1.6256170736353246], [-0.61172152600478924, -2.0265771285142864, 2.9081985667844092, -0.53969354681416348], [-4.3187329636131722, -0.89724782202700126, 4.1067615151750516, -3.393732924247054]], [[0.4182663201065111, 4.7665054831134785, 0.65143328716352578, -4.0291986920828986], [3.6669162329368081, -2.0765312428145699, -3.5073301647820054, 2.7118671294891605], [3.6748690674893751, -4.8117401822632901, -1.0755373877887466, -4.6088859723578679]]], [[[4.4971581418522248, 4.3926800255161584, -1.5265868063797061, 2.9135760577156304], [4.8668662347459026, -0.64143784489935829, -2.837418846243025, -1.366351426478972], [-0.93887925624838786, 3.850645855997719, -3.2396290362350855, -1.6926632079438875]], [[-1.7402293098349695, 4.0603202021248155, 4.5106658076212955, 0.0124936465114871], [-2.692308609408439, -2.6159733671013474, 0.66876558507367267, 2.4304657556596627], [1.6918651204877424, -2.0137734243636176, 3.0037097668270825, 3.5266183840313339]]], [[[3.7383758995976031, -1.9301835624390251, 3.1254721131799048, 1.048404148170345], [4.1392200054089017, 0.01951406410354295, -3.8248697442652189, 3.9145422462489634], [1.7340133603809456, -4.0016718584628492, 0.60777997775413972, -2.5770230992703427]], [[-4.6238765653811456, -4.4116002737324926, 1.5615690926658621, -2.4253176147131263], [-0.32299999444802463, -3.0835648202696286, 4.8988629108961455, 3.9877600259830981], [-1.549309959422609, 1.285006159547776, 2.3115099618803701, -4.6571754086943482]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.56037909040759915, -1.1651868189879846, 0.41998972568034948, -1.4399867021321693], [1.0100372531329596, -0.92205328536359132, -5.888370298045146, 0.83667326458945501], [1.5208188593564762, -1.7866498113664611, -0.17382615335222146, 0.68926399566879093]], [[-1.2013432583381944, -1.539648313990593, -8.799461500868011, 0.71816299555898633], [-1.0905548195372134, 0.68243680034468057, 1.2716880216142348, 0.64767891777984232], [0.50845166776189088, 1.5083299673669752, -3.4846630182382592, 0.81605544807600705]]], [[[1.2545922922306105, -0.27216010418103653, 20.308857075138192, 0.91685979361923065], [0.87445422515846905, 0.47463358420704216, 0.48199337207079845, 14.914646405550121], [0.2860149118776536, -1.8021267518669541, -0.057401860035802936, -4.3977778013010145]], [[-0.88088829235920896, -0.34569117437918528, -1.94355440167043, -0.75375943932617995], [-0.7785812842276395, 0.666863413832123, -0.18860326475512976, -2.7934018799612295], [3.1003877183340052, -0.58958817112475448, -8.014141443266686, 1.5753942650001447]]], [[[4.3680217791545042, -0.84961337446240692, -0.2305710932727969, 0.75092496818784249], [-0.23724674020842321, -0.89245349434821741, -0.46012547809284915, -16.278399936184801], [-1.2877908988093643, 0.25479435412182921, 2.0066462064415105, -0.91411001154086724]], [[0.51275676048190622, 1.3303442883757826, -0.52810177941650271, -1.0018385304735438], [0.52694206345715744, -0.85058758564833337, -1.0282644172915769, 0.17568157714701887], [0.48300535888502627, -0.41770697331773543, -0.833424216240509, 0.96908833447950615]]]])+(1.-msk_ref)*numpy.array([[[[0.71976589327242135, -12.198904458706027, 0.40026999196005392, -3.023028919937945], [1.8967328125903407, -1.0438283054286177, -0.49454465885845356, 6.9829610151348254], [0.31606000973555554, 4.5498749850088567, 0.1249370143504674, -0.32948072440392762]], [[7.0783482219673974, 0.85829879839913414, -6.0302358167811851, 0.81956766109633183], [0.6627776867110009, -1.4042302082159437, 1.3876873382730184, 0.85129493877195572], [0.60069120358799355, -1.0004473608366777, 2.4870142142148426, -0.31452306842382716]]], [[[-0.27427362090292379, 0.21075807436793209, -2.3623821023855078, -1.5084352728172639], [-0.72547468341281141, -2.8967203174109333, -0.47106318463911862, 2.4668873528060673], [0.96523748485971439, 0.29441819889822035, -0.050315695720240527, -2.938773528164814]], [[-1.5742746952835549, -0.19516923528942534, 0.82835020829545525, -177.061101264813], [-1.0297032276154623, 0.47754565924037434, -1.2913508370461759, -1.9009834603235476], [2.6050859712559009, 1.353301619646603, -0.93734725382900363, 1.4173860844001736]]], [[[-0.18864653185185401, 0.36213411388460442, -0.22984737886906931, -2.4509399848375057], [-0.26049101145770243, -95.534034381681039, 0.55304098647043765, 1.2095436895356098], [2.5562173188029433, -0.21117057027836678, 7.6206014673135751, 0.46777096645562422]], [[-0.21840711522841316, -0.78256614092070209, -0.22468153652056744, 1.855592803102204], [-6.0700579402741353, -0.86535763844000446, -0.57989117365385123, 0.11089324795860823], [0.56684234762926944, 0.42238910283356434, 1.1245495478133944, 0.91868372852742231]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(-4.64989438247)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-4.47226787903)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(1.039717322)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([0.014031738343240008, -2.309428083292091])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(0.181704780636)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.077222725203784898, -12.709781631574311])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(-1.15990739748)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([2.8950945058587019, -0.54522714970126085])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.40064578034710224, 2.1273837851139672])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[-3.7798835681922736, -0.30192794799769906, -2.8399688659898858, -0.93114160629996512, -4.7788814125372046], [3.3792447415082307, 0.63724492320910997, -3.1079982252493354, -2.198291418065399, 0.47101675567444978], [2.8297476933494607, 1.0302387725246298, 4.1119721587072124, -0.60407347845052506, 0.4043418322334027], [-1.8116474341459243, 1.9477763227009657, -1.2704668233177108, -0.70298060349202984, 0.59338802250993794]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(4.10806327292)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.92011327895365158, -0.073496421047903313, -0.69131575570248249, -0.22666194370441253, -1.1632930398232295], [0.8225882896660327, 0.15512052295063616, -0.75656045653912829, -0.53511625114399242, 0.11465664581647335], [0.68882767994478644, 0.25078454348956897, 1.0009515154784687, -0.1470458068241158, 0.098426388634063527], [-0.4409979383934251, 0.47413493739061852, -0.30926174669548062, -0.17112214607954457, 0.14444471350332061]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(4.67234364753)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-4.9914836654370962, 0.85154250917812568, -0.18770854920122737, 2.7661609401099838, -0.49965130072574659], [-3.8402664299571763, 2.953375447689802, -3.0537425960153808, 4.0333585504011129, -0.7130368503095541], [-1.8742641637108703, -3.652812606415532, 1.2279508875875358, -1.9305896057257419, 0.24464893554047329], [-1.9533547953622388, -1.8683408445064495, -2.1096956133770659, 3.7802718540889835, 2.3368980115373761]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.93606309480427863, 5.4869176784182034, -24.891480262442418, 1.6891076653509098, -9.3512088145221366], [-1.2166717421182038, 1.5820351087393467, -1.5300384693951197, 1.1584250666406519, -6.5527379763130416], [-2.4928949387173058, -1.2791084982919438, 3.8049922800361018, -2.4201640958165411, 19.098156455134411], [-2.391958521118327, -2.5007983212872076, -2.214700366208215, 1.2359808574296443, 1.9993785028129045]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[3.2613033041365309, 4.9054829578411194], [-0.6284019102533307, -1.7793142540778986]], [[-3.346880043803766, -3.1039001435292715], [0.37528846910705127, 4.5119207642908883]], [[0.74489302414602498, 4.6537132397694361], [-0.30287142841025982, -0.6901946889677486]], [[-1.50789230690454, -2.9290132354757228], [-0.65946984911967377, -0.96161791735046265]], [[-4.4542273575852311, 2.9636972027153021], [3.9075760068062486, -0.8449668097971994]], [[2.943522209530645, -1.0509212468737763], [1.8184704805045744, -3.3009927301384225]]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(0.38274953063)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[8.5207245029686582, 12.816431021535603], [-1.6418097475375781, -4.6487692647210013]], [[-8.7443086822266611, -8.1094812537652299], [0.98050667361914856, 11.788181051110541]], [[1.9461631289800909, 12.15863865884492], [-0.79130450640121797, -1.8032541746874973]], [[-3.9396320210337508, -7.6525586606380509], [-1.722980164168076, -2.5123947657579651]], [[-11.637446949331931, 7.743176582971004], [10.209224817015841, -2.2076233729326411]], [[7.6904658895014038, -2.7457153119030742], [4.7510717453073328, -8.6244200605741526]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(1.40885745774)
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-1.8738367688237769, -0.99697180631170745], [-1.859055020706526, -1.9000469427178626]], [[-4.6454219460567847, -3.2009847875783159], [2.19217139602089, 3.1967972868925791]], [[-4.5368961757597113, 3.9412009785455897], [1.7461186655450556, -2.2017224661025692]], [[-4.0860481033053233, 0.33889299269718443], [-4.8495315777883938, 1.2158917271384855]], [[-2.7628045662612397, -4.2404748362192626], [0.82570949492582457, -1.9122440513661045]], [[3.479142311680171, 3.892337610047516], [1.574781595249986, -2.8891740010111966]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.75185708871550394, -1.4131367094000113], [-0.75783526686610991, -0.74148560546649211]], [[-0.30327868471276431, -0.4401325064721015], [0.64267669046918185, 0.44070903823413216]], [[-0.31053332568272757, 0.35746907234754072], [0.80685092344289955, -0.63988875956279534]], [[-0.3447970807285029, 4.157233959082733], [-0.29051413216664002, 1.1587030541375236]], [[-0.50993742914014561, -0.33224049479138656], [1.7062386546281301, -0.73675609383092389]], [[0.40494390039927303, 0.36195664376572745], [0.89463673056978099, -0.48763330185126219]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[-4.7913110449000538, 1.7694057286795477, 2.9960912932667094, -1.2291182910565404], [-4.5273491269297619, -3.980660222586172, 0.15156149938133368, 0.25867286194129502], [2.1182279610512387, -3.6473085860907895, 2.1242315649818755, 1.7739611662391095]], [[-3.1247815112826824, 3.3842641535918112, 3.4221927794347895, 0.10265246043580145], [0.028405879112072263, -1.1136950337201732, -3.5054372660956945, -2.4831598746276695], [2.9761398553147753, 3.3408257108673087, 2.6358591157582776, -2.7800972796285239]]], [[[1.6545203710720591, 1.0931737430676094, -1.7228209304653666, -0.58353476194143106], [4.8364215236726391, -4.42906928835842, 4.9254456043959802, 4.2281789957217697], [0.41991161459387527, -3.0468703243851358, 4.7049948277171829, -4.9653497081183779]], [[-1.6744087085637647, 0.28270624300094216, 0.78480661935767504, 4.3231580674701906], [1.4559754575686599, 4.9650822421606051, 3.0887052938192223, 1.7852109695776726], [4.7535249399505393, -1.5816169298179417, -0.22064141981726237, 2.0838502138282147]]], [[[-3.2154729270156004, -0.57878141540550843, 2.0773503133149553, -0.19730274118670721], [1.0563927686331578, 1.4658049496925525, 0.72300572613983949, 4.9657361108387423], [-2.9004531644318963, -1.3941845318968515, -2.7921014759489959, -1.216613224147105]], [[-4.2246430567364524, 3.2784975658466351, -1.0081233560097203, -3.9754914870875089], [-2.9553855840865717, -4.400555961544816, -1.83367262258853, 0.50891160802987834], [2.774377345131481, 0.58046456924754075, 0.70725231237592823, 0.13686712103582277]]]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(0.970134500689)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-4.9388111045375398, 1.8238767175294455, 3.0883256817868587, -1.2669565819822231], [-4.6667231437623071, -4.1032044729439106, 0.15622730587730077, 0.26663608165410252], [2.1834374094992608, -3.7595906376886168, 2.1896258338119412, 1.8285723938055953]], [[-3.2209776160542365, 3.4884484070885935, 3.5275446621089266, 0.10581260677034844], [0.029280351427441068, -1.1479800305305032, -3.6133518224580792, -2.5596037176937241], [3.0677600406950769, 3.4436727159925322, 2.7170037905929854, -2.8656823127646422]]], [[[1.7054546249991396, 1.1268269938763282, -1.7758578106860394, -0.60149882467505866], [4.9853103051540799, -4.5654177696090361, 5.0770749838249998, 4.3583430882180725], [0.43283855413394051, -3.1406679405997746, 4.8498376507313639, -5.1182075316253703]], [[-1.7259552230889625, 0.29140932808804249, 0.80896681728144937, 4.4562460817523935], [1.5007975250173353, 5.1179318317532569, 3.1837907956310669, 1.8401685212812886], [4.8998617579029649, -1.6303068581664926, -0.22743384516317827, 2.148001346563047]]], [[[-3.3144609584861011, -0.59659914681333481, 2.1413013472241511, -0.20337668750723584], [1.0889137206051354, 1.5109296171312712, 0.74526338938159109, 5.1186058297171195], [-2.9897433421557609, -1.4371043715134708, -2.8780560571392018, -1.254066547764753]], [[-4.3546982956838374, 3.3794258048927017, -1.0391583386564123, -4.0978766184093178], [-3.0463668511801654, -4.5360266627126835, -1.8901220617196679, 0.52457840399264843], [2.8597862906223743, 0.59833411638800327, 0.72902500825741123, 0.14108056247721765]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(0.0777099501438)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-3.8338372125706579, 3.9651954028703535, -4.5934059895850528, -1.4178957396085923], [4.1467193230868631, -4.3124807518410435, 2.4125407554532803, 3.4447800675507096], [2.4262018734781963, 2.4204738985573773, 4.4611494271352612, -3.211531826858268]], [[-2.2731209363115559, -3.6419791140633642, -1.7677417883769864, 0.57640117483621633], [4.5311806196364088, 2.0659411361537732, 1.2044467828863104, -3.7185696701962057], [-1.5189533916220643, 4.2478077067950899, 1.6423290975115981, 3.8189725114059776]]], [[[1.3496159567639729, 4.98377973388809, 2.5625021810557227, 1.5384983155227392], [-2.8039219061823504, -3.6872125869219596, 2.7132708742887797, 0.58186895774899572], [-2.5872442014703569, -2.9990302033690188, -1.8864469219860904, -2.7534308471789668]], [[4.268451452057878, 2.9967915190864662, -0.035387451305657791, 3.2724363486806549], [2.4326973977475141, -1.628446070899189, 0.98621145092048224, -3.2788565894859936], [1.6619580669139324, 1.9798146646967565, -3.4236625602407234, 0.4551770294250872]]], [[[-2.6424445991587731, -1.0020838949133148, -3.1842747706171073, 4.070751090705226], [-3.7737451165190885, 4.1392423468331678, 3.6608076193097023, -4.9879269871031457], [4.7270742948748765, 4.2380114569313836, -4.2381258919251366, -1.6614363902054938]], [[-2.1709045458148712, 0.69455123297087518, 1.4622082134378633, 4.7363730131996604], [-0.19507289394426497, -2.8624339273307884, 2.7291246397954838, -4.7777706614008562], [4.1773452392575177, -4.0256174198779435, 1.2327443868451935, -2.6620836479457752]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.020269496547474709, 0.01959801277070174, -0.016917718642762727, -0.054806533352891457], [0.018740103703458012, -0.018019779012487375, 0.032210834145755388, 0.02255875516576417], [0.032029465887921459, 0.032105262605846237, 0.017419266360168228, -0.024197160213044504]], [[-0.034186456559533451, -0.021337286049693968, -0.043960011951252626, 0.13481920845470552], [0.017150044694094896, 0.037614793947351302, 0.064519206035453525, -0.020897806693421651], [-0.051160193968029455, 0.018294130880611839, 0.047316917334975388, 0.020348392116384107]]], [[[0.057579305990208655, 0.01559257316598053, 0.030325808390830471, 0.050510260141154152], [-0.027714734127380229, -0.021075527464678227, 0.028640690054269224, 0.13355232154746297], [-0.030035800292688902, -0.025911693072141799, -0.041193817455497959, -0.028222953274239265]], [[0.018205653974655376, 0.025931049807385302, -2.1959747672293273, 0.023746817925155751], [0.03194394428823507, -0.047720309276727498, 0.078796438706172284, -0.023700319920354655], [0.046758069105846678, 0.039251123617514809, -0.022697899917542428, 0.1707246744018098]]], [[[-0.029408355493439717, -0.077548347536813084, -0.024404285352774675, 0.019089830945747746], [-0.020592262525525533, 0.018773955142596322, 0.021227542724147393, -0.015579608591847666], [0.016439333358486585, 0.018336418137021377, -0.018335923029526115, -0.046772750736589742]], [[-0.03579611563004572, 0.11188512301876462, 0.053145611842155958, 0.016407058719237994], [-0.39836365049253752, -0.027148207475393267, 0.028474313342319344, -0.016264897512053504], [0.018602711936156178, -0.019303858772087977, 0.063038169934526531, -0.029191400579672511]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([2.1776746456960474, -1.0582522462424535])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-4.1315853352106826, -3.5597178495579138])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.52707967257440203, 0.29728542849930628])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-1.9865273920015278, 1.3420366519642162], [-3.3322262932241333, 4.701274529556958]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-2.9012476368506657, 1.7794044892497247])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.6847148677587287, -0.46257225164722954], [-1.87266375540569, 2.6420493811045866]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_Symbol_rank2(self):
      arg0=numpy.array([-2.7409605915932822, -4.4713860176502793])
      arg1=Symbol(shape=(2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[4.3487358945744319, 3.3838952553309589], [3.4100989333804534, 4.4456670397566409]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.63028904445840417, -0.81000160607075433], [-1.3112188546441277, -1.005785178616311]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_Symbol_rank1(self):
      arg0=numpy.array([[[-1.0333699247583561, 1.1349217930286493, 2.3452659471944575, 0.54166477177589556, 0.4400766578135169], [2.8792941603677686, 1.4185321784627938, -1.7325993945723814, 3.8722218264319199, -4.2318775307583376], [4.7789337490787318, -3.0030217055908381, 4.6860541378727465, 3.2286834719082602, -1.4210945275627553], [-2.211097434600664, -1.2260020468325736, 1.3672947269916067, 1.7094684024156237, 1.5848685496863411]], [[-3.9715591002884434, -1.0614302919227345, -3.7910113823603533, -2.4102884819830428, 0.52570084855033627], [-1.5494065296687607, -3.9021199598567802, -1.7312667645582849, 2.2130558093680586, -0.49631910577403016], [-1.5599519530839832, 3.7545734469944545, -0.48419127730180378, -3.7567372142060842, 1.1400089883643103], [-4.6373166710745366, 0.99593019907278801, 4.0340811863067092, 2.2906547015752965, -1.8427143057139519]]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-3.9006010356447574, 2.6900355153134594])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.26492581920456348, -0.29096074749953249, -0.60125758203999247, -0.13886700198918447, -0.11282278135907164], [-0.73816679379818451, -0.36367015377883033, 0.44418780047982737, -0.99272440094398262, 1.0849296024090351], [-1.2251788135745056, 0.76988691695161993, -1.2013671982984941, -0.82773999247902286, 0.3643270651308364], [0.56686070028568725, 0.31431106017483773, -0.35053437008730043, -0.43825769074920368, -0.40631393346906786]], [[-1.4763965299639001, -0.3945785421346194, -1.4092793053398038, -0.89600619332424736, 0.19542524459535932], [-0.57597995299635085, -1.4505830639199131, -0.64358509569958111, 0.82268646520459776, -0.1845028078434853], [-0.57990013299218768, 1.3957337832980055, -0.17999438094607567, -1.3965381471063314, 0.42378956778623406], [-1.7238867831580165, 0.37022938671377953, 1.4996386342641403, 0.85153325617278153, -0.68501486141131018]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_Symbol_rank3(self):
      arg0=numpy.array([-3.3956334394580256, 2.4999496823560907])
      arg1=Symbol(shape=(2, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-1.3804552532689494, 2.5906803751340268, 3.443496829253462, 2.5135579195984619, -1.0949321267075485], [-1.6466200472967296, 4.2960439879244348, -3.5420187322281018, 3.9786786974712989, -3.0459949741280381], [-3.4461129826145842, 1.3516373381571256, 4.6228590517605159, 4.9995668648181297, -2.3885471125072346], [3.3860678769193946, 2.7752616288779457, 4.9969670498867469, -4.3189705401268839, 4.1012972728966819]], [[2.4698426025696616, 2.236144474563206, 3.425645514507794, 3.260631748029553, -4.3962948789613474], [-2.8128065227484491, 1.6329049585527775, -1.2930519576220725, 3.9136332184320022, -1.6566287436455509], [-4.0998209471738836, 1.9539724468547028, -3.7285661363895639, 0.57012238332254839, 3.8581576145432575], [3.9878461728457175, -2.4661035748996216, -4.8893897150047803, 3.2899789722492159, -4.755676226592831]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[2.459792471662583, -1.3107110672741153, -0.98610035316750588, -1.3509270715355046, 3.1012273333039064], [2.0621839537497837, -0.79040937406662171, 0.95867179034426142, -0.85345756660776972, 1.1147862909491757], [0.98535174458550123, -2.5122370798721518, -0.73453103402857844, -0.6791855237206732, 1.4216313430358349], [-1.0028249766060018, -1.2235363340611962, -0.67953888940191942, 0.78621361454303129, -0.82794131064285015]], [[1.0121898779116958, 1.1179732395619986, 0.72977477435089788, 0.76670715233845299, -0.56864922649290528], [-0.88877413435224129, 1.5309829694998072, -1.9333714067867063, 0.63877975855839042, -1.5090584972313961], [-0.60977045450713474, 1.2794191066410603, -0.67048554079741651, 4.3849351568814603, 0.64796463289435724], [0.6268922054664241, -1.0137245279561491, -0.5113009655753421, 0.7598679819667612, -0.52567701484319951]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_Symbol_rank1(self):
      arg0=numpy.array([[[[0.88211780913070648, 4.0988877572644515], [1.0029490062214199, -2.8753373344049846]], [[-1.5698660695565012, 4.0483950196677085], [-2.4777393730451367, -1.6754622017574006]], [[-1.4383227824156686, -3.6428471312322594], [-3.4018367405744598, -3.6802790113301875]], [[0.76403231550211359, 4.9810342084478805], [2.7841673202692849, 2.6019943463479507]], [[-1.9026857172708009, -3.9660387818746212], [1.4160224199729035, -0.91676736140084714]], [[1.4804793874497566, -2.7547858210611964], [4.9818880071711167, -4.3997348703116828]]], [[[-0.77582032221584107, 4.3920577634886513], [3.6596374317331239, -0.70923854940193998]], [[1.5924851763304844, 1.1144494219882839], [1.8145449995797698, 0.98397838183449693]], [[-4.8196347859373816, 3.7045902362637442], [-3.7219070250814967, 4.5786044306679941]], [[-2.2638898662523168, 2.2822684167309131], [-1.0528599793768976, -1.7839366512663855]], [[0.54562623472123128, 2.8285746511050931], [2.7487677211501076, -0.1466364707516945]], [[3.2450042591758947, 1.2904980154133057], [-3.9076706889719004, 1.6125970200451176]]]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-0.91235094336215816, -0.40420152969893675])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.96686238508173428, -4.4926656645516241], [-1.0993017692571168, 3.151569421092403]], [[1.7206822451142489, -4.4373221172422204], [2.715774440824573, 1.8364229400399985]], [[1.5765016662505118, 3.9928134647483229], [3.728649337543458, 4.0338414051152016]], [[-0.8374324825998789, -5.4595594433124361], [-3.0516407535122241, -2.8519665215220673]], [[2.0854756945382245, 4.3470539606822109], [-1.552058920172358, 1.0048407009066203]], [[-1.6227082332966685, 3.019436589728699], [-5.460495266012515, 4.8224149953722426]]], [[[1.9193898716655002, -10.866009752016543], [-9.0539920382264469, 1.7546656736559243]], [[-3.9398296624869835, -2.7571628014826275], [-4.4892086403812117, -2.4343757000805959]], [[11.923841034266278, -9.1652058789165167], [9.2080478464633764, -11.327528705985593]], [[5.6008938608880081, -5.6463626410093637], [2.6047896952817178, 4.4134831765607689]], [[-1.3498866150448083, -6.9979315843062571], [-6.8004881703379123, 0.36278059328700318]], [[-8.0281840140310354, -3.1927093803294437], [9.6676296398048471, -3.9895866332970971]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank1_Symbol_rank4(self):
      arg0=numpy.array([3.5946705251977065, -0.23922747533054078])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-0.0088585622301637912, -2.2139608845060654], [4.0699058244735582, -2.675336462457143]], [[-4.1486465582714152, -0.15340530626673399], [-4.3287212631240424, -0.36532655702178118]], [[-0.68292044046113087, 2.323725315504217], [-2.2762808358933349, 3.7978600641536993]], [[-0.44957623378261946, -3.0883845624713646], [-3.5290219374877809, 4.7587401908565035]], [[-1.6154364116590472, -4.2892541120428351], [4.0472632627569745, -0.84368845388780223]], [[-3.6658199065995611, 4.9833441425718377], [1.5622089296058039, 1.9099328179299571]]], [[[-2.0927445161287928, -3.9762609493416079], [4.3744650197740942, -2.9634656506653565]], [[3.3431386011538073, 4.0703048371787673], [-1.8705354890117176, -1.6994327697520308]], [[3.2527645769649443, -2.3039174725975675], [4.4622221413614671, -4.8795555007384142]], [[-2.9260505638579182, -3.2994796475709096], [2.3323775171519125, 4.118997615002689]], [[-2.746974878142423, -1.5518295395402895], [-0.17125773379088027, -4.2632124939432945]], [[4.8743586637612655, -0.44765214114311025], [4.7596179521993793, 4.3940614283455108]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-405.78487025328968, -1.6236377753348055], [0.88323186831053435, -1.3436330628470587]], [[-0.86646825047816811, -23.432504472481945], [-0.83042319121361707, -9.8396091280694602]], [[-5.263673939486222, 1.5469429631865552], [-1.579185866926198, 0.94649894005474089]], [[-7.9956862820640406, -1.1639322929140681], [-1.0186024878487039, 0.75538280742969466]], [[-2.2252008802413914, -0.83806424877114105], [0.88817314116330448, -4.2606610397867826]], [[-0.98059114107767142, 0.72133700229311171], [2.3010177813442398, 1.8820926534440721]]], [[[0.11431279522503268, 0.060163927463098781], [-0.054687253012459781, 0.080725577256760009]], [[-0.071557749728945408, -0.058773847389856797], [0.12789250818060377, 0.14076901398426439]], [[-0.073545892938171584, 0.10383508878936629], [-0.053611735980842533, 0.049026489255904339]], [[0.081757806336444805, 0.072504607054224762], [-0.10256807638184733, -0.05807905167490237]], [[0.087087609440503047, 0.15415834615534449], [1.3968856765479398, 0.056114368136800354]], [[-0.04907875924459415, 0.53440484997939941], [-0.050261907096975249, -0.054443361621509402]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-4.1342672334801698, 3.7450438683564027, 3.5440433301320855, -0.55264217669000359, -1.4040602909294986], [0.82355895701376713, -4.6279875006190485, 1.2860245592470658, -1.7094961516969209, 4.1949377658513498], [-4.5830885131416803, 1.3295029448461868, -3.0225745729833173, -4.3566303797141801, -1.5554665395205483], [-0.39790349328995056, -4.7873694415018768, -3.8963799648353339, 4.898200849736563, 2.2736145326008925]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[2.893846010143343, -3.8415614291384004, -2.3149632820399679, 2.3886542133571105, 0.22679698744555221], [-0.7184480741002508, -0.73441967805753183, -2.0334420172423719, 1.7223397947936512, -4.9411604472217308], [2.6978938320177059, -3.7258851465299525, -0.84385665128763065, 0.10912100835200977, -3.7406585660178084], [-1.2056499635296269, -4.7650153661393961, 2.7680274364310637, 4.6313655379312788, -3.7156319894813974]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-1.428641060716076, -0.97487543475163296, -1.5309285281661316, -0.23136131366343649, -6.1908242553996677], [-1.1463026858902114, 6.3015570509488832, -0.63243729024105277, -0.99254290986276084, -0.84897825331902343], [-1.6987653327017955, -0.35682875144028514, 3.581857852718477, -39.924762843652189, 0.41582692247062014], [0.33003235211409077, 1.0046912913484667, -1.4076377688868227, 1.057614824314747, -0.61190519917938058]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_Symbol_rank2(self):
      arg0=numpy.array([[[2.7953537428220212, -4.7030736998458949], [2.1224902317532637, -2.1942774048490366], [3.766233374666605, 2.4024463224199311], [-2.1184665925669979, -2.8057686218326019], [-2.220619328960395, -4.6273254995615059]], [[2.8968889631769947, 1.0985629322368666], [3.562679458655154, -4.2971838078165856], [4.0412222376345994, -2.9938753739201585], [-4.4186589354545909, 1.3407411531404456], [-1.326262348663072, 3.2904544491263561]], [[4.1488793511017263, -4.2404455851507947], [4.1309703908133137, -4.4586444281877293], [-4.5519258916946059, 3.5708496015157021], [3.5900208354153236, 3.9380776367395711], [-3.7721990596833166, -0.56857069472759214]], [[2.1822409521752073, 4.0676126124160366], [-2.7261533628266066, -3.1022528549906569], [4.9814489929395833, 0.099162389790551586], [-4.587990174773692, -4.4645669144978779], [1.9895395973193377, 1.8628907333571121]]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[1.7272766871243208, -4.7055415530829361, -3.1201305971373539, 0.61165320719954419, 3.1177519788789212], [-4.4969071625386121, 4.9275621680708106, -0.83058054625104383, -0.099290956537618769, 1.2521356582158329], [-1.8891997645612477, 1.624886478505136, -3.4745204631599815, 0.97753334870543629, -1.6327920264015852], [-2.4427686009184555, -4.0084553495190658, -4.435598110646616, -1.6782680813484916, -4.8592639883124145]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.6183589830508871, -2.7228258998133463], [-0.45106184013244316, 0.46631771924560073], [-1.2070755557866826, -0.7699826169533156], [-3.4635093344256345, -4.5871886042727068], [-0.71225015459981633, -1.4841865327675603]], [[-0.64419585694573944, -0.24429299794943987], [0.72301055514637547, -0.872070947305567], [-4.8655392374349393, 3.6045575440377058], [44.502128789347253, -13.503154767498625], [-1.0592002072306304, 2.6278737671403127]], [[-2.1961041012861191, 2.2445723658745034], [2.5423132295455657, -2.7439728788251077], [1.3100875185391061, -1.0277244412220592], [3.6725302928740469, 4.0285864844967518], [2.310275282270116, 0.34821991137513808]], [[-0.89334738925115853, -1.6651649324813889], [0.68010071838612107, 0.77392725738179047], [-1.1230613930019449, -0.022356035717603778], [2.7337647815402848, 2.6602227404042567], [-0.40943229306014506, -0.38336890892072728]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_Symbol_rank3(self):
      arg0=numpy.array([[4.3238024777905597, 2.0194284186653721, -0.88035376570350454, -1.3024949562518886, -4.9903518181204785], [1.8850960005470174, -1.8079382039644432, -1.9424943061644107, -0.35491954012596771, -1.5208134874036894], [-1.4681046402037392, -3.8887567584087535, -3.9901541126396376, 1.5774680206239893, 0.38007339779540317], [3.0123025415968048, -1.2237936000931535, 4.3687193863764886, -1.2132082786978904, -4.2423413409217741]])
      arg1=Symbol(shape=(4, 5, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[3.3756658604600052, 4.9006158217940943], [2.0781224738900921, 0.50576558114264447], [-1.8615837751394926, 3.900150876255589], [1.441107153491112, -4.6556276700002019], [-1.5016090175639354, -1.7760257235868004]], [[-0.40134253453609237, 1.1129017941098436], [-4.7688740623996848, -0.80231856744663954], [-0.94188833182033882, 1.2518738575282109], [-4.6317409173922606, 0.21701577798673899], [4.4473642680071386, -3.692988441312143]], [[4.9677792928359281, 1.6095502459444866], [-1.4657988311859826, -3.5201614425843664], [4.641068739269766, 3.0178799629187623], [4.1584557963160194, 0.34932656159676867], [0.037548064913453416, 1.7952547716687111]], [[3.5546823666046183, 4.9715390259408885], [-4.9997066389351543, 3.2646133336060803], [-1.2941755081512554, -2.2137090340548116], [-2.3751477569372437, 4.6755968333775062], [-2.0273852890120225, -0.92626633348287246]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.2808739539171543, 0.88229778358909072], [0.97175620977003874, 3.9928150391392867], [0.47290580067369664, -0.22572300242617901], [-0.9038154817957621, 0.27976785270971466], [3.3233363410512413, 2.8098420827160857]], [[-4.6969753722365368, 1.6938565563683134], [0.37911217203641007, 2.2533919534209024], [2.0623403439028198, -1.5516693590836781], [0.076627675523309, -1.6354550043253331], [-0.34195838158432773, 0.41181106076338936]], [[-0.29552533509709339, -0.91212103747792794], [2.652994855550777, 1.1047097759112374], [-0.8597489795566905, -1.3221712465927684], [0.37933985543899973, 4.515740267254218], [10.122316520743615, 0.21171000561782122]], [[0.8474182025085164, 0.6059094630212043], [0.24477308139699153, -0.37486632413565357], [-3.3756776873464824, -1.9734840122029871], [0.51079276022066267, -0.25947666617386816], [2.0925185577276904, 4.5800448397709346]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[-4.1665140225127617, 0.42861923008852809, 2.6705390378337484, -0.79650776291624403, -1.4368511562051034], [3.6364200910330773, -4.0032399901598765, 0.51276002854704128, -3.7578609483856207, -2.1537781695449807], [-1.9717504757596305, -1.8110966214578363, 0.3379691089930752, 4.4583184476881623, -3.175936337896367], [-3.7389513151976574, -1.0193822650394679, 1.1142380131981247, -4.6125773632583904, 1.6553493178785672]], [[-2.7873642772859943, 1.1131925789819057, -2.7415643337674753, 4.9358399590962847, -3.3478067497561081], [-4.3447712728780443, 2.0252949646344316, 0.84125505763165709, 3.2159791645778881, -4.5939758931339858], [3.7622678412657713, 1.1490411519649726, 0.5671438472058119, -0.65598100182134189, -4.5069983398622888], [3.8934966478802142, -2.6750415870682218, 4.9017681052048587, 0.65134551039052013, -1.0136658214253114]], [[0.011875485700635124, 0.16688097965478477, 4.7752912043215936, -2.612898386148613, -2.0532200984294668], [0.1804110036632034, 1.9524245286915871, 2.4170956160064989, 1.1760414176616694, -0.039830844148035283], [1.2925621716365763, -1.272392731348889, -1.2447351124698414, -0.29701977144519098, 0.068376377859138415], [1.9293029868848794, -0.17226447581622839, -0.74761155256666711, -1.3376597426971326, -0.32395146130927266]], [[-3.1817883749048437, 1.5588507317957809, -4.5926880853679455, -3.8864010402286633, 0.35989573516433992], [3.7324015983765619, -0.54087418705185719, 2.8270725434677608, 1.0412448764123265, -2.6559552849171841], [-1.3554464263703316, 2.0315700322017811, -3.3497691092573523, 2.6946090851054265, -1.5776444826186689], [-2.4484197202800839, -3.3009093423838429, -0.52078027961951623, -3.0523116769154059, -0.67277521162832521]], [[1.372338578538681, 2.1207700395371241, -2.6586270819718338, 0.55293621612878674, -3.9099427049344948], [3.2667148606428729, 2.9118373297409859, -4.7489796749862458, -0.43894607105291605, -0.11782027480219082], [-4.3309218170616814, 4.1631123454562946, 1.0268253961043694, -3.3252205519935609, -4.6377985303578253], [-2.6956292392766512, -4.5322977490331944, -2.6606120514325928, -2.2985050847396113, -2.6233867610280392]]], [[[-3.9622521345238439, -1.6702882715182166, -4.1611980542420195, 2.476056075725853, -1.4885194376363584], [-4.9612764753382024, -2.7805143176981915, 0.61399523558444358, -1.2415339973004782, 4.1637191947753323], [-4.4674718845399468, -3.6549215295197923, -0.64697437784410194, -0.86280065446778931, -1.2339538293617327], [2.822963316315521, -3.3850540086258709, 1.5454346040766085, -3.7267262899612694, 1.0887036035851896]], [[3.1907997285578045, 4.3975641984038809, -3.7533070037760705, -1.3081806634873203, 4.7316213176178294], [-1.0346744433353452, 4.6777965026740791, -3.88447802307173, 4.9165812015924537, 4.2477998416869074], [2.5356592310846571, 2.8395749823348915, -4.2939010949624903, 0.25284638612081878, 0.96161595691753732], [3.257846025842607, -1.5463528068609991, 1.7548067689757927, -3.3862832318315492, 1.6688272114290781]], [[-2.869790574987876, 3.7298453780946161, -1.0154151674822565, -4.2044213188807609, -4.5692215298956071], [3.7012503478975329, -4.9789120022394826, -2.8725011509490761, -1.7161326774181096, -2.0934243599457556], [1.5657201605847373, -1.284029058431214, -0.36756629528284357, -0.70209501190987744, 0.23158189692966591], [3.9765755219122756, 3.0932455191712744, -1.3150741022543908, 1.6474211653053041, -0.89878344129691534]], [[-1.8006723072515429, -2.4452749235344995, -3.5947526215295134, 4.5368252084039131, -3.9576030333835188], [0.09269625317723218, -3.3644612862738454, -3.6393567720231399, -4.6969126270695085, 0.59848388919528173], [-1.3042863812182217, 1.5613216984917209, 1.4044858966607965, 1.0924012312669964, 2.5765711575576979], [0.72221889401039352, 0.48837965549020002, 0.090615213078607759, -2.1543737842470181, -1.4880877279303628]], [[2.4828135743592883, 4.0648188558204801, -4.2657908543230487, -1.1070629357159669, -1.432694596296944], [3.5561577067479835, -4.3188983892907018, -0.96154485865922368, 0.2845102101847452, -3.8257335808696791], [1.934407447893026, 4.4665477032975627, 2.2156672557047576, -4.2168608141400803, -1.7280826959987285], [-3.2200734821925292, -3.3698934707961179, -1.2892252476025989, -1.3549253450854648, 2.4511788539198642]]], [[[3.115573775842325, 1.7257165102324903, -2.931128295051991, 3.4496569261104266, -1.071587529809209], [-0.18457185068438164, 1.075957902985138, 2.8090243602446137, 0.37770637187738387, -2.7212603192028464], [4.3778321705892882, -1.7666619842207663, 0.094939150452203691, 3.8865746934735199, 0.034116318815530455], [-3.6075858185853149, -2.4281342321245858, -1.1157244794044594, -3.1503797106911566, -2.0742954958446793]], [[3.0660180249467981, 0.4528267331272362, 2.557718428963593, -0.53558917247453675, 2.4754495788373116], [-3.3110968143008215, -0.65431956382863277, 4.0661281740658168, 1.796169127106829, 2.7504978347937055], [1.1765650254049458, -4.7468418309560025, 0.014816812051704709, 1.5604976837486939, -1.766860841186654], [-4.1596215221019648, 4.4237108416646365, 2.8396067402635925, 1.9499844066855658, -1.4122683327164443]], [[2.8680364975251491, -3.3671026543091012, 1.9946728312815107, 4.5272483712700442, 2.0184959244528606], [4.0566238145616165, 0.36357870750328924, -0.4145441378901662, 0.53733330272384805, 1.9881904760137692], [3.7508598432681612, 0.71718106985690699, -1.0674305842774592, 0.72289254838701567, 2.162238873835963], [4.2651816052031712, 4.4048675286855783, 2.4377083542188007, -0.49057280648359214, -1.9554389262099847]], [[2.0856812104458458, 4.5477606590987598, -2.398846474673527, -3.1742878638293814, 1.9575996009380923], [-4.4391853021307064, -2.2811086713629622, -4.6960152200803833, 1.9764938968354642, 3.5112607570790964], [-3.3427293756503653, 2.693327239742203, -2.928273103320842, 4.83331220220383, 0.60618823445439496], [-2.5829138184727816, -1.727940633163616, 0.484306512701286, 2.5882831837170457, -0.22859365212996074]], [[4.5143824433319342, -2.562979189426613, 2.2780666447570503, 2.4099208754171739, 2.6274373470820533], [-4.3012882007747439, 3.432584586507053, 0.59868756713562554, -1.9532207602237941, 1.5663383001728812], [-3.8923865343729211, -4.738720672210964, 2.3743488617457045, 2.3316642709177158, -0.34108583258650516], [4.0805363732677176, -4.5482188538297832, 1.3477408935952013, 1.3518889438298842, 0.96056698509765681]]], [[[3.0450060460545423, -1.7583556403472689, -0.50503326040886787, -2.0916852101376424, -0.46377121416470501], [-0.61198000336112113, -0.71730430974979598, 3.7731579845868808, -3.6235282200506234, -0.33950267417212299], [0.34592253236288606, 1.8007052376099377, 3.3169964589731311, 0.072486886732808564, 3.9264437153837264], [2.8940983587987699, 2.9498845185403315, 0.5966865992362127, 0.68559880076150392, -3.4630170896019132]], [[1.3437211769237347, 3.6906205539349166, -2.2902489864222053, 4.8712925689062985, -0.3472274030473157], [1.5138962901472155, 4.2366965920346296, 2.40331239109031, 2.644051329256607, -2.135951217368369], [1.2611061929039211, 3.3503384575319437, -3.085101227538527, -3.477788447669238, 1.9959142031307833], [-3.3415727466334166, 1.1576922789156701, -1.5113984882987888, 3.5684316259143003, 0.49830986502602492]], [[-3.1136282083764821, 0.67421193046957928, -0.078471028100011608, -0.96764691994596674, -4.8962813068547417], [-0.64704208048668832, 2.3517671442303634, 2.0959085341657913, -4.27590856971264, -4.5714889066350093], [-4.3921860852235426, 4.8290169408690033, 3.2519708694448077, 1.7723335036920362, 0.40496288861440277], [-1.3214982604808521, 0.4962759545709714, 2.1978515002329289, 2.7724560003115597, -3.1059308288573217]], [[1.8225352525484428, 2.8794588453449457, -1.5912341408799877, -1.7390810908732379, 3.4556270511736766], [-3.9341586569417841, -3.2074443186116044, -2.0573817246769845, 4.6887224389289202, 3.7291555520212754], [0.70261190378315685, -4.6873793306083709, 1.4799233560992935, 2.837367887407936, -2.0038713151995182], [-1.857888542400008, 1.3088707349447581, -1.6542219577904413, 1.8789027710356052, 3.1730324838552004]], [[-4.787479142258122, -3.8268165634317386, -1.3573543214372084, 3.3510163953137813, 3.967353571394753], [-0.9281319135378352, 3.1748082237956581, -3.6888951824138183, -0.071777934233815621, 0.7314603701548954], [3.782488987437528, 0.54972268091621856, -4.6870815540177482, -4.5502346396427154, -4.2051397207520393], [2.8658118838398288, 1.9064230326998564, 2.658883262025026, 4.2072443708946299, 3.5556028886775284]]]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[0.077878189480512283, 4.0502107311203499, -2.7533122448913616, 1.4933419339230278, 2.9213519162770387], [3.6706450063561604, 1.137127304719872, -3.6151130083376772, 4.7781249812533435, -3.0461437837547809], [2.006553799477877, 2.1023518478864531, -1.5346039112150436, 4.6390711739546191, 0.7322428230105329], [0.40739338420570981, 0.76566888889333384, -2.0638206332022877, -2.9622280061089459, -4.3848215009267548]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-53.500396584789151, 5.5037133367845295, 34.291231674074886, -10.227610172108031, -18.449981513304845], [46.693690689137433, -51.403865663333384, 6.5841287781266526, -48.253059983192891, -27.655729850832341], [-25.31839130971359, -23.255504956378488, 4.3397145111809046, 57.247330445500175, -40.780818854181142], [-48.010249597973356, -13.089444834802578, 14.307446290555383, -59.228102168613084, 21.255621489413166]], [[-0.68820228435742825, 0.27484806418306529, -0.67689424471232806, 1.2186625059212552, -0.82657594184736505], [-1.0727272137951731, 0.50004681215048885, 0.20770649071855901, 0.79402761438249891, -1.1342560173068383], [0.92890668931319309, 0.28369910314447522, 0.14002823182707119, -0.16196219045617105, -1.1127811956133908], [0.96130717791150921, -0.66046972976348428, 1.2102501402066443, 0.16081768422216097, -0.25027483474789863]], [[-0.004313163435302159, -0.060610989532489237, -1.7343805495297224, 0.94900183987367226, 0.74572729709066521], [-0.065525079473985323, -0.709118456257265, -0.87788648762642285, -0.42713695834671772, 0.014466519088759183], [-0.46945716891894962, 0.46213165023682018, 0.45208643326937853, 0.10787725656481457, -0.024834225753365784], [-0.70072073752790232, 0.062566269458125143, 0.27153169930283949, 0.48583655746967891, 0.11765881690692692]], [[-2.1306495870950641, 1.0438672459298459, -3.0754430589804018, -2.6024857080247115, 0.24100022037075552], [2.4993616757091233, -0.3621904500002715, 1.8931180322787871, 0.69725817829073033, -1.7785312422989132], [-0.90765979015238463, 1.3604185257590815, -2.2431360381459773, 1.8044153344215379, -1.0564522744460656], [-1.6395573342322578, -2.2104176326933462, -0.34873478591163642, -2.0439469404685804, -0.45051652025931954]], [[0.46976147272512953, 0.72595500313424288, -0.91006737913314439, 0.18927408678426078, -1.3384018142933334], [1.1182202467431461, 0.99674308785496191, -1.6256102691791854, -0.15025443138405162, -0.040330736651659753], [-1.4825060250122122, 1.425063622859156, 0.35148979839887018, -1.1382471702454839, -1.5875521550543696], [-0.92273348659477916, -1.551438470586296, -0.9107468486108542, -0.78679500129132629, -0.89800436106693871]]], [[[-1.0794430209575514, -0.45503944637139054, -1.1336421928670324, 0.67455612608635973, -0.40551985688041453], [-1.3516089043607211, -0.75750019761742116, 0.16727175592334248, -0.3382331974763601, 1.1343290314278158], [-1.217080888182863, -0.99571642672905147, -0.1762563191819935, -0.23505423514770479, -0.33616811956072956], [0.76906464978967526, -0.92219596358793765, 0.42102535151192877, -1.0152783185265799, 0.29659735596876552]], [[2.8060180380101318, 3.8672575886190756, -3.3006920053693434, -1.150425865298861, 4.1610304299072745], [-0.90990203035379069, 4.1136964025557727, -3.4160449819017051, 4.3236858188042886, 3.73555346358809], [2.229881580153692, 2.4971478308089812, -3.7760953212008928, 0.22235539070368796, 0.84565373896674534], [2.8649791560894475, -1.3598765946807851, 1.5431928876319474, -2.9779279925617037, 1.4675816898444707]], [[0.79383149803869624, -1.0317368694954563, 0.28088061566550326, 1.1630124173667431, 1.2639221842740274], [-1.0238270115930521, 1.3772493393032035, 0.79458128814344497, 0.47471065868760542, 0.57907577304433044], [-0.43310407087514424, 0.35518365690638365, 0.10167491152700095, 0.19421108283215716, -0.06405937971940559], [-1.0999865046378752, -0.85564282832575378, 0.36377122906569831, -0.45570391893857592, 0.24861835279395575]], [[-0.37685751509564136, -0.51176453799939803, -0.7523354109893079, 0.94949906630819536, -0.82827532743720855], [0.019400131545516241, -0.70413840145958639, -0.7616704850337559, -0.98300330056194296, 0.1252549674910961], [-0.27297033592371545, 0.32676451633589809, 0.29394080359371172, 0.22862550384365418, 0.5392431482363278], [0.15115110986924191, 0.10221156989537218, 0.018964596663781409, -0.45088267734719389, -0.31143758980118275]], [[-0.81506775471343229, -1.3344146384351054, 1.4003905124481317, 0.36343095215005355, 0.47033058778694803], [-1.1674293661754016, 1.4178248618215521, 0.31565970844422536, -0.093400125004620832, 1.255926788903559], [-0.63503484576444036, -1.4662957563322712, -0.72736791596017525, 1.3843275674079421, 0.56730174892422014], [1.0570983219391425, 1.1062818139996897, 0.42323190864400229, 0.44480019371092772, -0.80468258491017697]]], [[[1.5526988494666949, 0.86003999029656564, -1.4607773266855324, 1.7191948339526495, -0.53404375705652418], [-0.091984501353718426, 0.53622180639517947, 1.3999247670187296, 0.188236354278498, -1.3561860738102025], [2.1817666547133889, -0.88044585930388075, 0.047314530254263652, 1.9369401879405581, 0.017002444103122388], [-1.7979013667732411, -1.2101017340060394, -0.55604015187371536, -1.570044975375648, -1.0337602193294939]], [[1.4583753085998152, 0.21539055585889405, 1.2165986542808862, -0.2547571535245054, 1.1774668361653844], [-1.5749489399833572, -0.31123218717477602, 1.9340854758225161, 0.85436180861569988, 1.3082956773191174], [0.55964230087736078, -2.2578722185480613, 0.0070477318373708096, 0.7422628544872264, -0.84042109457697267], [-1.978556313627396, 2.1041724514913636, 1.3506810209329709, 0.92752524209776488, -0.67175641134295994]], [[-1.8689099360201304, 2.1941183843609209, -1.299796525151693, -2.9501087141668587, -1.315320461326525], [-2.6434337778728385, -0.23692022732785872, 0.27013103176698228, -0.35014461959660137, -1.2955724024185449], [-2.4441875951550043, -0.46733952951355967, 0.69557400217513221, -0.47106132279739571, -1.4089882464355123], [-2.779337113657006, -2.8703612029751473, -1.5884935105428684, 0.31967389298204929, 1.2742303808294997]], [[0.4495902589629569, 0.98031706963917509, -0.5170962860284396, -0.68425073571859651, 0.42198093703083206], [-0.95691252314770625, -0.49171667901322802, -1.0122748808954405, 0.42605379885788291, 0.75688874462468958], [-0.72056005400771306, 0.58057467513399919, -0.63121969754661222, 1.0418706721594935, 0.13067017334369635], [-0.55677391478151284, -0.37247555994934584, 0.10439730164528484, 0.55793133725746225, -0.049275737223728341]], [[6.1651439952276563, -3.5001765929083692, 3.1110808780494961, 3.2911498749950994, 3.5882049840784291], [-5.8741281793524278, 4.6877681537312617, 0.81760796872571573, -2.66744951106977, 2.1390968281984106], [-5.3157045887726389, -6.471515354330486, 3.242570342968798, 3.1842773976689096, -0.46580973123665403], [5.5726546509408692, -6.2113532709413253, 1.8405655217679271, 1.8462303778844114, 1.311814817314829]]], [[[7.4743630213616639, -4.3161124076069983, -1.2396697638905585, -5.1343131509505904, -1.1383867096146256], [-1.5021844415914891, -1.7607166379206574, 9.2617065737171043, -8.8944208731208896, -0.83335343020861141], [0.84911180636211669, 4.4200649971789598, 8.1419988334868059, 0.17792848274680603, 9.6379663185720812], [7.1039404933915655, 7.2408748715733013, 1.4646447938754963, 1.6828913461572481, -8.500425445920575]], [[1.7549637923331243, 4.8201260459585695, -2.9911741480477763, 6.3621398747793751, -0.45349550972246477], [1.9772205872636912, 5.5333273344280389, 3.1388403341866984, 3.4532568419727872, -2.7896539200588188], [1.6470646923197727, 4.3757014371765637, -4.0292889946169872, -4.5421571884628742, 2.6067589163973937], [-4.3642530016639274, 1.5120012001388101, -1.9739583392023432, 4.6605414921219586, 0.65081639368456179]], [[1.508671906019895, -0.32668145652921943, 0.038022213189260322, 0.46886192742657873, 2.3724354859547656], [0.31351662546503273, -1.1395210932557105, -1.0155478147893671, 2.0718411769524798, 2.2150611507075331], [2.1281820786957106, -2.3398433290087577, -1.5757042143720357, -0.85876334172608249, -0.19622000192238115], [0.64031643022696916, -0.24046467342510008, -1.064943079293996, -1.3433609276450209, 1.504942231359546]], [[-0.61525826127828898, -0.97205847740507922, 0.5371747676405787, 0.58708549351594963, -1.1665634934404792], [1.3281079811643277, 1.0827810391357295, 0.69453861094894986, -1.582836442454618, -1.2589022669189238], [-0.23719035210462322, 1.5823830309286384, -0.49959805695148246, -0.95784925453290115, 0.67647436695182572], [0.62719295697985444, -0.44185347388705365, 0.55843843025552764, -0.63428701881177951, -1.0711641633633591]], [[1.0918298820707442, 0.87274169829328763, 0.30955748623982171, -0.76423097145585661, -0.9047924916798169], [0.21166925799412123, -0.72404503196416226, 0.84128742336128193, 0.016369636533356029, -0.16681645307575177], [-0.86263237548850724, -0.1253694547885317, 1.0689332628539403, 1.0377240301984927, 0.95902187121260496], [-0.65357549520182856, -0.43477779706583813, -0.60638346657054509, -0.95950185657623843, -0.81088885554999979]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank2_Symbol_rank4(self):
      arg0=numpy.array([[1.3266963385999375, -1.4086485345864319, 4.4325949629385715, 3.4391238120963017, -0.9876087740222772], [0.67889099214008652, -2.1850485306974141, -1.3724608057427479, -4.779519903249998, -3.7431907817209198], [0.96524525917373971, 0.88345775327418252, 4.0322047855422802, -1.1900906580276738, 0.46441240765654168], [-3.1984328567845255, 0.04275352867055382, -3.6218380531909422, -0.67237914235987706, -1.6742100016934591]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-2.5419319838037802, 3.6224037950079584, 1.567659287782952, -1.4400397774054294, -1.2168619386030533], [3.6657489060460584, -0.28700725046578235, -0.42422036304827326, 4.6404868764652658, 1.1652997349817049], [3.8330561603888302, 3.0331723200220928, -3.3055761928901219, 3.6742371070846165, 0.78084051598436943], [0.032350987631720507, -4.221431990552257, -0.2040180349910079, -1.4594433923381156, 4.4375821838775771]], [[-3.5933144934941073, -1.0851540888189404, 4.3171036183873177, 4.1666808364717447, 3.7601860561318023], [2.6325571482857875, -2.6361805663621798, -4.6569290274218762, 4.5252558270969274, 4.4361268001080933], [-4.5114171583037734, -1.6532149034827137, 1.7019331202742807, 0.97372410311669277, 1.7544566792429741], [-0.62361468842974688, -3.2533015933444052, 0.50089796561936151, -1.7313058765416378, 0.50076895666290255]], [[-4.2903084642301481, -4.198892953324358, -1.6177805398972964, 4.3764061602886954, -3.8652327716896151], [-3.048435030154486, -2.4228663906520351, -1.3780790713428526, 1.3323333736528848, 2.3029969068836174], [-0.17316877580985857, -2.1439249774153524, 4.5462505957136337, 1.1737623375391451, 0.26697076420417432], [3.243333652120258, -1.2837104355493767, 1.0232896821745276, -2.4012521277730969, 0.39880294857737297]], [[-4.1111989271919844, 0.99554161246643247, 0.69499628972738225, 3.2962875562290588, 4.17057676594772], [-1.8262858155107442, 0.53919225690747297, -1.7558176570315087, 0.19207902138841426, -1.38894194650514], [-0.14902923287687386, -4.1099976888795444, -2.3721587780164031, 4.0466117130058379, -1.1551243172151482], [-4.3648490735197187, 2.5210657497640874, 3.9732777176985383, -2.6767685968559007, -4.3615603038484654]], [[3.929695600723786, 0.53547207203430869, -1.5445242428206054, 0.71581249194383467, -1.358028181536528], [-4.199041597816711, 2.7881649894679583, 1.6968602258015935, -0.92627469901570247, -0.29585285963120889], [3.1735960012826219, -0.71292277852037955, 3.9063313382077016, 2.4450014220338288, 0.35540070080474351], [-4.4004152589572065, -4.7464341388833358, 3.2104969190852266, -0.46695322859632604, -1.9831482276577574]]], [[[-1.3996675750224732, 0.45978173041121728, -2.3344721366182597, 4.4597060459072928, -1.3308368371950263], [4.6368936568778487, -3.5358905890022649, 4.2123361950433207, 2.1162135729634333, 2.8200861462502269], [3.6680796255731494, 2.4195249843450259, -1.9285687375543423, -2.9429210589300014, 2.3383620922984072], [3.3319901653202617, 1.5500886940310457, 4.653481097407429, -3.2974423201198508, -1.3727350613320652]], [[3.2603441390790131, 0.12049534114136051, 4.4892296661044249, 1.2648886260306771, -3.2145656528176447], [-4.5601121661822823, 4.7571296034668009, -0.21593414721899684, 4.5391850811858969, -2.070699012521795], [-0.23318182408894383, 3.256710790349981, -2.527604955711682, -1.8142257518731633, -3.0564183411290466], [1.6589744466608547, 3.2207713079922549, -3.9298644477742939, 0.85204227896092366, 4.7660670322737815]], [[4.4899052614916659, 0.38645589314569762, -3.9341499012926731, 2.3436366605630399, 3.6708378208979102], [2.0571607873257571, 2.0439091162609557, 0.75638591963212143, -4.1327807864340285, 1.4682851292090202], [0.73509798800609261, 4.8190838465759711, -1.588988986443618, 1.8994404710834232, -4.2948385769117126], [-3.2414848386973603, 3.9398585908939392, -4.2997305960307175, 1.5502054400201537, -2.0370176715824662]], [[4.6569279941552839, 2.5583798184527149, -3.5252202006257951, -2.2859158235782893, -0.34719155338458929], [4.6371681958276803, -3.756638282001866, 3.8686488811071467, -1.0505296944632683, 0.02681509014722927], [2.6977367018845646, -1.6995067849078707, 0.81513549318364298, -1.9359322371553187, 2.9045291395501742], [0.55275348150847137, -3.4322413748370071, 4.5066025912954064, 1.2385100079605849, -2.7514701818973308]], [[-0.65586825059043097, 2.8614883173984174, -3.8313465407228753, -1.0831300287226666, -0.11484389046721866], [-1.0378619928715196, -2.3526578654754191, -0.17924171722013238, -0.15652191565160845, 1.9813605586047043], [-2.2220254059268263, 4.3664586121731155, -1.4862190191032632, 1.122658773526938, 2.8773872925372421], [-1.3839482069896247, -3.24969430571727, -0.9705199306395631, -2.587989017464567, -4.6526334952779518]]], [[[-1.3094377202394125, -3.6527327699033432, 2.8944329123993393, 1.7029271843596128, 2.4145045411105661], [-4.5488562247255002, -2.1928227858908542, 1.4455800656472464, 3.0221420998237551, 1.9154153359726491], [-0.20862292483962719, 4.1264153588844579, 4.2805667083460222, -1.4257427867041783, -0.51075010487218542], [-3.0993578191706304, 4.0325700006064231, 2.9182749440177025, -2.5988545545912833, -1.7816714484252891]], [[-1.0223742607663699, 4.5583926480773744, -1.9960420767697631, -2.7176778180677963, -2.0182270860659446], [-4.5762257447687249, -4.3790999958719725, 2.0800023024582837, -4.2121266498005348, 2.1261366827235539], [4.3727572067499665, -3.4581503584813778, 2.3504840839440906, 1.7702879114967711, -1.4947730455206907], [-3.3232567329342033, 1.7341906932785554, 0.86515116363579025, -1.7479450391223104, -1.6489376069622361]], [[2.0276990753358559, 4.6268045541219021, -0.19538531571822393, -4.5112815790998955, -3.0787306064665945], [-2.5763472080181882, 2.3979398912824843, 0.25570449237921178, -0.14901897227075445, 1.0974186092530651], [-0.65691623965353418, -1.4425247237625927, -3.4125933899481975, 2.2071558545164933, 0.260029449387571], [-0.30933595911913336, -1.6223929608743926, 4.9860278183132269, 3.9753363415456775, 2.9697505453460256]], [[4.9701622081729564, -2.8358594856520258, -0.076839989949122334, -2.6122208428162508, -3.4691381070083329], [-1.9915926522860072, 2.0126577046328489, 4.010822276906298, 0.86661625950577026, -4.0476314271211038], [0.16257587399893669, -1.2658675817558485, -2.8982610493026986, -4.0025747822920739, -3.8017108281482095], [-4.3230940193859482, -2.9265344926769177, 3.146339064406245, 2.8373442900691135, -2.5151470024900546]], [[-3.5034721981459951, 1.9662739056933702, 2.5919070902128958, 2.8282205212758171, 0.96002323454998084], [0.10346571950901229, -4.0112814280536817, 2.2687239998145223, -4.0432741663816216, 0.23359012945743629], [2.7792631113798869, 2.0081392140575804, 3.2782680951565908, -2.222036446295502, 3.1685852513084303], [2.920898324951068, 2.2087085839130189, -3.8627066652628472, 0.57555893550155091, -0.80395821223596542]]], [[[0.78628720190482149, 1.2740069111148022, -0.72491576373119759, 4.8755429875448559, 1.1815542320757562], [2.5065163743988981, 3.1673051555086946, 4.5757554166734842, 4.4168981261396034, -2.5470655424132382], [-3.3235613413372231, 4.288705342777325, -0.65362099730087841, 1.0331256745044124, 1.8196107316898971], [2.8234028022840647, -4.2632609710391991, 1.1528742347963465, -2.2940519471128109, -1.921189292073493]], [[0.31425564941185957, 2.96390791021018, -1.0619338844860984, -3.4197271399582387, -3.4676199793021825], [2.8908352857009127, 1.0623809894114196, -0.34397457591068381, -0.60261575287719182, 1.7197625000271586], [-4.2057934899191043, -3.1106609252614859, -4.8730724755106634, -4.9498874195326312, 2.3466120452625372], [-1.3062301182134228, 2.7082201897321827, 3.6154352621847661, -4.3165542189645016, 4.0555176205977599]], [[-3.9948353212155832, 3.2158402589649366, 0.31039140226282047, 1.4123376818442495, -2.0562269230451924], [1.7300048976064497, 4.7282992980917236, 2.9450095325687577, -4.2341115852684528, -2.297006396802467], [-0.48546255390158066, 0.72268663637626762, -3.2995855916862715, 1.773389947124123, 2.5583214249909636], [0.64140516116957969, 0.59933134905444074, -4.5537735819346983, -1.9172867709013452, 3.4723377063246126]], [[-2.0042435462199348, -4.4256268632221429, 2.5301681139856393, 0.49205149719298813, -0.040793211100186078], [-0.97047081824556969, 4.7048988849302269, 1.099808700043468, 4.5841616928249813, 4.3892249081083019], [4.7519842846311722, 3.2964350077510147, -1.0588796983006086, 2.5833713176680124, -3.7961512224601837], [-0.31999675741465428, 4.2679247685032724, -2.4121777085585885, -1.9911941629066066, 1.3870408202043389]], [[-2.6233030370988808, 3.7816956209842303, 1.0486077332605284, -2.4036163283289982, -3.5109097924908128], [-3.4632662543208848, -1.1461914106293825, -1.9785678402751614, 3.8886731244357424, 4.7204905492309646], [2.7582856242782405, 3.9693246032643366, 2.4017377814344147, -3.391781652086058, 2.8710274224538281], [-0.59098874214241537, 3.7306435193003935, -0.92984004213532501, 4.3595806478005716, -2.6392986903643454]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.52192440515841487, 0.36624750129410205, 0.84629125023474061, -0.92129145278909808, -1.090260362751565], [0.36191686135724327, -4.6225185476912172, -3.1273754259857838, 0.28589593590457552, 1.1385022228815374], [0.346119723553791, 0.43739563685266464, -0.40135100847274197, 0.36108076314449572, 1.6990618589090922], [41.009453983379998, -0.31427637388666685, -6.502838529242827, -0.90904268405675581, 0.29896828579761986]], [[0.39201927277360976, 1.2981092262386222, -0.32629481687368944, -0.33807449859280442, -0.37462203028207175], [-0.53508754235541878, 0.5343520669869396, 0.30248443261465685, -0.31128594457611419, -0.31754018720837918], [0.31224080708068747, 0.85206619636620096, -0.82767561063704809, -1.4466608457956771, -0.80289730219742217], [2.2588443805475289, 0.43299045421065202, -2.8122464678901915, 0.81363354313812608, -2.8129709636427749]], [[-1.0331646313766754, -1.0556580061011525, -2.7399235270934654, 1.0128390283241375, -1.1467860345706797], [-1.4540559070776522, -1.8294838626019669, -3.2165026340754763, 3.3269413275940374, 1.924707301902848], [-25.596964246057933, -2.0675140266719438, 0.9750001390415608, 3.7763990385240525, 16.603297279205467], [1.3666786826082047, -3.4529554642450173, 4.3317107952453373, -1.8459514982499261, 11.114749724771883]], [[-0.83652576122004363, 3.4545254251863446, 4.9484060029231598, 1.0433324621807714, 0.82461587571684392], [-1.8831246362905725, 6.3782885752501928, -1.9587021455922091, 17.904734141381571, -2.4760745549875827], [-23.076840333316778, -0.83677025449468456, -1.4497865168081598, 0.8498773927438934, -2.9772759181345725], [-0.78791356909920995, 1.3641547478157294, 0.86556341047520902, -1.2848043032691934, -0.78850768360619872]], [[-0.25131940851611401, -1.844370277370879, 0.63942588056676219, -1.379703183637327, 0.72723731911428868], [0.23519861640231993, -0.35421460987885589, -0.5820212879088098, 1.0662158591525288, 3.3381755216203306], [-0.31119549357357745, 1.3852955800794988, -0.25282258173097283, -0.40392973399612719, -2.7788599509961789], [0.22443535800671616, 0.20807383924948464, -0.30761866431059481, 2.1150057726146487, 0.49800048238891104]]], [[[-0.48503730761154928, 1.4765506048552721, -0.29081134938005088, 0.15222774441896458, -0.51012338490041287], [0.14641073148897768, -0.19199999973179363, 0.16116733344763443, 0.32080457323095402, 0.24073413255222181], [0.18508076744217558, 0.28058854383926302, -0.35201804266567244, -0.23068610355009669, 0.29032757346523502], [0.20374939854446822, 0.43796912702757218, 0.14588884706511729, -0.20588411448404384, -0.49455354588328859]], [[-0.6701895375114143, -18.133883932773792, -0.48673128648227804, -1.7274631819200346, 0.67973367685994102], [0.47916552292325143, -0.45932079065179143, 10.119050455143503, -0.48137462818029736, 1.0552226651406758], [9.3705782568368576, -0.67093723433225061, 0.86447390671545177, 1.2043972633733049, 0.71490492688584417], [-1.3171080091651979, -0.67842399281044163, 0.55601117029238289, -2.5644836936520434, -0.45845946267670884]], [[-0.30567700782327417, -3.5514034850681315, 0.34885828963756266, -0.58561159621603853, -0.37388216878702507], [-0.66716263220577077, -0.67148817666289973, -1.8144980890314071, 0.33209136333770467, -0.93473725125998253], [-1.8670447044284015, -0.28479703807558798, 0.86373210730334216, -0.7225605785686503, 0.31956050993880253], [0.42340497458389842, -0.34835280862995177, 0.31919692992154686, -0.88534123949720167, 0.67375989167366701]], [[-1.0263246305823439, -1.8681823038068712, 1.355807476197243, 2.0908556010467225, 13.766233241151614], [-1.0306979823484512, 1.2722864285733284, -1.2354493907656396, 4.5496285620864132, -178.23993419406247], [-1.7716776807429564, 2.8122982183381477, -5.8634668017985732, 2.4688466938662481, -1.6455403521929219], [-8.6467477151055228, 1.3925360664580215, -1.0605594361663344, -3.8590886408098402, 1.7370785751907314]], [[5.7072297345559795, -1.3081272283942507, 0.97699092001598931, 3.4559015837971532, 32.593730206217508], [3.6066363422408338, 1.5910476557816473, 20.883479804669523, 23.914803023831087, -1.8892022279664815], [1.6845850509794698, -0.85726010806226205, 2.5185997040863066, -3.3342195063967073, -1.3008991842805504], [2.704718834719356, 1.15185935339685, 3.8568922322431791, 1.4463704275639064, 0.80453162397596045]]], [[[-0.73714484030386573, -0.26425290870628937, 0.33348337597971822, 0.56681534480097051, 0.39976949421257635], [-0.21219515664775429, -0.44018388781088846, 0.66772175551657131, 0.3193910899259273, 0.50393522545520786], [-4.6267458857445503, 0.23391858919279657, 0.22549473584695121, -0.67701219895704412, -1.8898581712778917], [-0.31143395357688436, 0.23936230717100629, 0.33075884818613049, -0.37141180427680454, -0.54176389256664648]], [[-0.8641236259331726, 0.19380905101424398, -0.442604774496488, -0.32507817792114141, -0.43773951869622069], [-0.19305379641380235, -0.20174413786097323, 0.42473883429362264, -0.2097414979950849, 0.41552255809936189], [0.2020367725677614, -0.25547118016642478, 0.37586204446521865, 0.49904749817063521, -0.59103136487615615], [-0.26584095791303824, 0.50943518305012414, 1.0211599896155246, -0.50542650569710712, -0.53577391257498042]], [[1.9885617321566367, 0.87148802988665075, -20.637194615778331, -0.89380472374477626, -1.3096971774902941], [-1.565085937560406, 1.6815287156283747, 15.769002523281792, -27.058331728500427, 3.67426317682613], [-6.1380805377393548, -2.795241370300348, -1.1815661360123213, 1.826878141519185, 15.506723546271576], [-13.03503413254769, -2.485344107613185, 0.80870082006609723, 1.0143053163583364, 1.3577587490845837]], [[-0.23944704582693166, 0.41965783708569254, 15.487907518151191, 0.45558577533767397, 0.34305081588520775], [0.5975572648662133, -0.59130305927742011, -0.29671986836216452, -1.3732614002723424, 0.29402149861113497], [-7.320216885535288, 0.94013834873386459, 0.41062231378847025, 0.2973312736823292, 0.31304081552340468], [0.27528678596648104, 0.40665526444524874, -0.37824615645874748, -0.41943822686343252, 0.47316942383465305]], [[-0.13255775453343241, 0.23618907127426644, 0.17917787617086053, 0.16420657588858881, 0.48375121657783521], [4.4885630705548758, -0.1157765706511098, 0.20470202972882967, -0.11486047904393047, 1.9881508209924801], [0.16709911550114584, 0.23126504597167102, 0.14166395004199875, -0.20900305592682472, 0.14656774895507957], [0.15899643054652432, 0.21026422907895517, -0.12022978908364496, 0.80688940612457971, -0.57765739635262847]]], [[[-4.0677666494331284, -2.5105302246639938, 4.4121441646156843, -0.65601572275237774, -2.7069708439582278], [-1.2760470625497349, -1.0098278188388929, -0.69899558991501898, -0.72413552802042458, 1.2557324511383181], [0.96235108315998386, -0.74578050976877108, 4.8934059187088899, -3.0958797518209051, -1.7577566460130172], [-1.1328291004730429, 0.75023154306335738, -2.7743120283623335, 1.3942286096920897, 1.6648192190018583]], [[0.13604696924484427, 0.014424715600398676, -0.040260066370557082, -0.012502029232389553, -0.012329358155087532], [0.014789334031595573, 0.040243122850155792, -0.12429269970712943, -0.070946583235547506, 0.024860135437235465], [-0.0101653894260453, -0.013744194464705248, -0.0087734235198448501, -0.0086372729411673382, 0.018219257314760175], [-0.03273047227622445, 0.015786577779992747, 0.011825278443713123, -0.0099045503662896113, 0.010542064582190669]], [[0.90663012664283193, -1.1262493661164257, -11.668615904908961, -2.5644278275302388, 1.7613999761403476], [-2.0935420808356904, -0.76599170755807822, -1.2298221833026892, 0.85539504102637132, 1.5767644610100775], [7.460592014940886, -5.0116300356012511, 1.0976645256048598, -2.0423246782606479, -1.4157087603656899], [-5.6467242118642265, -6.0431313311160526, 0.7953487339728873, 1.8890434692188804, -1.0430546679241555]], [[0.33547776348238961, 0.15192856585978456, -0.26574484859059977, -1.3664812447388253, 16.48262355980135], [0.69283808407079484, -0.14291043416756624, -0.61136008683446719, -0.14667439488713249, -0.15318858259411081], [-0.14149439520128046, -0.2039716059254589, 0.63499106030550534, -0.26027197010409953, 0.1771212744059564], [2.1012061115625711, -0.15754240733621813, 0.27874361825591254, 0.33767633256738, -0.48475800608436465]], [[0.63820686288114592, -0.44271410750337609, -1.5966027605839703, 0.69653795489789139, 0.47685930446697461], [0.48341937314367894, 1.4606722630857421, 0.846172654590719, -0.43053503036113167, -0.35466864814848786], [-0.60697484950694658, -0.4217871222516304, -0.69708276008946879, 0.49360783606567499, -0.58313967627050201], [2.832896605820642, -0.44877244181385179, 1.8005354962437741, -0.38403005631702347, 0.63433896580395777]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-4.323641641100707, -1.0705832775272937], [3.9340167831319981, -3.959390191141706]], [[-1.8792781315922991, -3.3224410723968303], [-0.60850268139970787, -0.27226667624534073]], [[1.5190663990296969, -1.7036294734411483], [4.0070461078346238, -4.9794621874336702]], [[1.5814644714644768, 1.0132361902610274], [2.7041200059289814, -4.080828241554137]], [[-1.3151850738879323, -3.8962057049536289], [-2.9199809777935704, -3.0392920764140707]], [[-0.81224294544638553, 2.0362395297645985], [-0.030660296030269762, 3.1915411568824208]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[1.4136355199958537, -1.0961836633110744], [-4.4347124376396359, -4.3928146985815495]], [[-0.75906155730099645, 3.38615514445695], [-2.0896929047060264, 1.8185255330236547]], [[4.7263856697625268, 3.8337427966424364], [-1.0009548721112536, 4.9092713818342464]], [[-3.6276057252579941, 4.494002555446869], [-4.359472186224794, 0.92097065073070805]], [[-0.48223089383025375, 3.2068217314316421], [3.743678539548382, -2.239487100321127]], [[0.63395414053742094, 0.90392170398299321], [0.63894306522132815, -0.9347830617954358]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-3.0585264588664187, 0.97664589736134766], [-0.88709625222641675, 0.90133330514036991]], [[2.4757914737171896, -0.98118394776907436], [0.29119239483914061, -0.14971836870095745]], [[0.32140127894091663, -0.44437761315995805], [-4.0032235413199038, -1.0142976014443101]], [[-0.43595268924987807, 0.2254640885846747], [-0.62028610125637462, -4.4310079135707134]], [[2.7272932753057502, -1.2149742116204947], [-0.77997641809967522, 1.3571375677842741]], [[-1.281232968614771, 2.2526724613340066], [-0.047985959468312127, -3.414205164086344]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[-4.5017509065034114, -0.96808242860407656], [-0.082818599643119661, 4.118345789303369]], [[2.5061424871160103, 0.37392071535990112], [3.2951353773234064, -0.21370918759497393]]], [[[4.7518094356327261, -1.2656286911072225], [3.3790826073294546, -2.4763946358215838]], [[-2.1147144931009976, -4.1675611521086928], [1.1368544237316156, -4.7489114992811254]]], [[[2.988970837941328, 1.768099034757741], [3.3765516766059829, 3.450575571598355]], [[0.73694480969221043, 1.9637724444208535], [-4.8830970000742688, 2.2202030494426115]]], [[[0.40344765728185372, 1.4348721027287432], [2.0367159603446563, 4.4252613315824387]], [[-1.3018330444292667, 4.9744982162621252], [4.1419081576471264, -0.37829666562321762]]], [[[-3.6171673559075854, 3.15235677842327], [1.6433751618053902, 4.8260753230237228]], [[2.9218183434516263, -2.4239083860712274], [-4.9619659370380838, 0.87760988664251904]]], [[[-2.7580302597491158, 3.145829106129348], [-1.9044976925645218, 3.1791117799520556]], [[-0.55592283911732299, -1.6107469458043711], [2.0301504274826545, -2.7807154783846331]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-1.6662367468995165, -0.90593900002018923], [3.6319406855390231, -4.2750570639043017]], [[-0.89263984071030222, -2.9699260149331685], [3.1145410183590396, 3.8304596326990339]], [[-0.9854362351355741, -0.64327105476590773], [-2.4727975446957351, -0.43087517534563524]], [[0.71306857816556235, 3.6316942341122349], [-0.70472788321205648, 3.3615675001257266]], [[-2.7545908191862356, 0.94103424596505025], [1.5052306660841879, -2.4855914497432585]], [[4.5659528085065375, -4.8104340008141309], [-4.9971619564328797, -3.7392260578848391]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[2.7017474646865969, 0.58099932701967794], [0.091417412917728466, -4.5459416022619514]], [[0.69002847350852836, 0.10295342015047441], [-0.770781612518183, 0.049989785960844864]]], [[[-5.3233221495598473, 1.4178492079180793], [-1.1377665942986439, 0.83382367889636111]], [[-0.67898110207428852, -1.3380980142956853], [0.29679321354198968, -1.2397758897500581]]], [[[-3.0331448462823292, -1.7942297753182275], [-5.2490340605092847, -5.3641082495994654]], [[-0.29802068158511036, -0.79415011092729204], [11.332973630140547, -5.1527755054851578]]], [[[0.56579082241957945, 2.0122497984977685], [0.56081702617305562, 1.2185115393295742]], [[1.8472847115055018, -7.0587503840333667], [1.2321359477363505, -0.11253579337885343]]], [[[1.3131414403596151, -1.1444011054079324], [1.746350006763117, 5.1284800141088187]], [[1.9411100300345649, -1.6103235475377018], [1.9962918433560812, -0.35307889666790132]]], [[[-0.60404265559004555, 0.68897538762743082], [0.39590974374499255, -0.66087836968847591]], [[0.1112477129947089, 0.32233234781010967], [-0.54293332257933768, 0.74366070286683739]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank3_Symbol_rank4(self):
      arg0=numpy.array([[[2.8515951340791466, 4.505037739321466], [-4.8299503128922749, -1.2495543127830744]], [[-3.4852670917967199, -3.7230400674926969], [4.4497038767889432, 0.74781576720351595]], [[2.2458856934356932, -2.8435889248021393], [1.8572996860707045, -0.2315808110857791]], [[-0.41804736388429475, -0.32746955056878324], [0.5618960100393755, 1.3961218591883711]], [[-2.9094910807359051, 3.2263561838679475], [3.7457494436130467, 4.0573292979860387]], [[-3.5695827195125673, 4.0107658733335576], [-0.7381522908569309, 3.646922859954703]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-1.6269311367594828, -4.3956339241699265], [-0.92877448010699126, 2.576326942977996]], [[-2.626511697277405, -4.7409538485528548], [-0.74846620598932656, -1.8114420077716975]]], [[[3.4828927901840228, 3.9516960987272682], [3.5258368259330677, -2.0459274318959308]], [[-4.540942501233789, -4.7417228494545158], [3.3329938439506517, 1.329055253567045]]], [[[2.5688013667634895, 4.8068222607616882], [-4.0970813964667405, 1.522407284128958]], [[-0.2872280173927475, -1.6948294707666332], [-4.4720769888990999, -4.2991704527709054]]], [[[-3.6938524223626712, -1.3258237131638086], [-0.31771801374737052, -3.1380886799996546]], [[0.68273950234710945, 2.5007326790534385], [-3.7580643073825923, -2.4583189952598117]]], [[[-0.048790020391155764, -0.54296479641936823], [4.3965457614010148, 0.21610443588933848]], [[0.88310569216498003, 2.6469871031713925], [2.2016475390527104, -4.4632058299537842]]], [[[-3.5204384147725252, -1.635365914663117], [-0.20743919711881453, -0.20645895560569816]], [[-2.7362396857869631, 3.5640191177404379], [-2.4519455521083056, 0.27311452174156692]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-1.7527448271467385, -0.64873353497417174], [-4.8505184367280449, 1.7486281202004814]], [[1.8389220645386484, 1.0187718478564363], [1.6694866151390322, 0.68981193293633725]]], [[[-1.0006817039041191, -0.8819673893747112], [-1.0559309041499507, 1.8197322199461445]], [[-0.97990755786490225, -0.93841500611973427], [0.224367581284554, 0.56266717670048461]]], [[[0.87429324917611384, 0.46722877851527023], [0.69405233863657345, -1.8678240405484479]], [[-6.466290102650694, -1.0958622788348023], [0.051783726367105279, 0.053866394373016872]]], [[[0.11317381315870281, 0.31531142468911627], [1.0306924266156547, 0.10435318563681231]], [[0.82300204998787918, 0.22469255300493007], [-0.37150025784437379, -0.56791728896062954]]], [[[59.632913809220554, 5.3585261879274944], [0.73383887236961876, 14.92961572302052]], [[4.2415641489413858, 1.4150992421252115], [1.8428605060607299, -0.90906165939204553]]], [[[1.0139597115330357, 2.1827425211120999], [-19.33465771676854, -19.426456273436958]], [[0.26976887101344443, -0.20711232641336505], [-1.4873588268788824, 13.353090259351289]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[3.7073432339913861, 1.2926650828871633, -1.0786189516627074, 0.28613692951424241], [-1.446887436375329, -0.66860100483336282, 3.0996229120521068, 1.0723992022643483], [0.45212001262005153, 4.6336400985217328, 1.8253716910951754, -4.7700393291846233]], [[-3.9332832886515181, 3.908341708314758, 2.3026186492088554, 4.1707979315214665], [0.9293989100822051, 3.4450121982581301, 4.2831173462570966, 0.47389957213456402], [-2.2246126080614772, 4.3963267868360649, -1.993574154782233, -2.8965839200690024]]], [[[0.7841962816237027, -0.38808403060280305, -3.7052366920144522, 0.86640094109190269], [-4.3967994984959704, 1.2321653518871232, -2.7970836642936794, 0.54092242806829915], [-2.2495548642672389, -4.2385909266029493, -4.661154283360097, 4.6687511241799058]], [[-2.886246308741617, 2.1537715966322448, -4.0195062239935826, 2.8853508726727348], [-1.2759430965236795, 4.4187765494652709, -4.7136011601722778, -0.35867834914717633], [0.74813584989173343, -3.511975655756665, 1.1614183050322886, 4.7399546827351742]]], [[[0.44039224884813599, -2.0538290744557974, -4.6574589819435595, 1.1776800834346783], [-3.3569245681378668, 3.2873932545793565, 2.6723346107197221, -4.9722240797942145], [0.33966525480914811, 0.36206095124682758, 1.970916241007286, -3.1350869659528566]], [[-1.3105406969502678, 2.3413739549871426, -3.8511753151110737, 1.8266194198461942], [3.1495338830199682, -3.3501858113725183, -1.63974326482517, -4.4863297682840866], [0.44726462804035361, -3.3697835812697532, 4.2486379281473248, -1.0035997772507823]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[2.4694276674508053, -4.4691510507971701, -0.034243119254605325, -1.2950025835968813], [2.2655157612954548, 1.7911022386513764, 4.3108931283065921, -0.28393381394535666], [4.7033387694517295, 1.412380262467793, -1.2397828102503414, 0.21216203982094761]], [[-4.8748619251393297, 0.27289670640429975, -2.1220577616605265, -2.480398382798815], [-0.39819334989817001, -4.5213527649430398, -2.2169037492738197, 0.063328008296634763], [1.8696065582927615, -2.6676981453594339, -3.0539936138577906, -4.4118371590332304]]], [[[-4.3487956861309129, -1.5435830802585815, -0.3264148730653238, -0.0079547430097912653], [-2.4674905589728846, -1.4901769265542231, 4.6729803225386046, 0.80026178809739967], [4.1224145955316551, -1.3500134582160808, 4.8177656621965976, -3.1658653610065226]], [[1.549575400848866, -3.0525525653677255, -4.9644742483992186, 2.4980575946483574], [3.1506740457233811, 2.2324921553214434, 0.43121035982779965, -4.5918083869598121], [-4.9281789864284988, 4.2604769273279004, 4.460260145410988, -2.7260742894550818]]], [[[-3.5977614637896482, -1.254650981407277, 4.9989652512030389, -2.3185235859188671], [2.1996219479560466, 2.5218982648283914, 2.4749621311374659, 3.5535760776125684], [-0.54053143008958937, -0.69914671725033983, -0.76189989590177731, -3.9043495935544179]], [[0.30088935149544049, -4.0770645722622403, -1.9954490598862851, 2.18122062198319], [3.0066569950483526, -2.5043400552213599, 4.3873164281383765, -4.1167876413269902], [0.22827969414120552, -3.6922316726284485, 4.2124040790743305, -4.4163910871194698]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[1.5012965485312162, -0.28924175267170449, 31.498852182329884, -0.22095471710913081], [-0.63865697210951111, -0.37329025133528437, 0.71902105197159061, -3.7769337415751845], [0.096127460678907331, 3.2807312744696437, -1.4723318278034438, -22.483000885597907]], [[0.80685019371889177, 14.321688817029923, -1.0850876403133525, -1.6815032457871748], [-2.3340392558536709, -0.76194280281988358, -1.9320267502186761, 7.4832540116336954], [-1.1898827580562676, -1.6479850969959433, 0.65277613736197682, 0.65654823957821085]]], [[[-0.18032493090550214, 0.25141764998991251, 11.351310855473615, -108.9162704597087], [1.7818911130205817, -0.8268584286406131, -0.59856525626758017, 0.67593184644530802], [-0.54568865215681217, 3.1396656831879732, -0.96749294386288276, -1.4747156280504523]], [[-1.8626046252157302, -0.7055641305140935, 0.80965395787674022, 1.1550377696871699], [-0.40497464288811524, 1.9793021619057098, -10.931094424666922, 0.078112656043266102], [-0.15180776752467651, -0.82431514491484814, 0.26039250338956865, -1.7387474365867885]]], [[[-0.12240729500289199, 1.6369724368701515, -0.93168460829422783, -0.50794397373703892], [-1.5261370578963442, 1.3035392031577671, 1.0797476765802254, -1.3992170059673379], [-0.62839131251415103, -0.51786119038185552, -2.5868440875353169, 0.80297291285813255]], [[-4.3555569196343829, -0.57427934080719867, 1.9299792675892919, 0.83742992406949257], [1.0475201821181861, 1.3377519576016179, -0.37374629609766824, 1.0897646804142609], [1.9592834558631043, 0.91266851055173659, 1.0086017030638135, 0.22724431723852753]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_float_rank0(self):
      arg0=Data(-2.15705444425,self.functionspace)
      arg1=3.43019640462
      res=quotient(arg0,arg1)
      ref=Data(-0.628842838662,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([-1.7772192236914974, 3.5961694660198784]),self.functionspace)
      arg1=-3.69197241491
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([0.48137391723635486, -0.97405101173059139]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[-2.5193545606194343, 4.78133775745947, 1.8191578253275917, -2.7568879853055583, -0.18269025174915487], [-1.008042760029241, 2.7608454238388358, 1.6525799685235683, 4.2559992937363145, -1.0843638810770817], [3.6970252611596983, 0.13573726454413837, 4.2047310659759436, -4.9782927182011871, 1.8039897028362262], [2.1377446577542631, 1.6107063956808156, -0.15033611951901449, -2.0963476192970179, 2.1723645056638299]]),self.functionspace)
      arg1=3.18317871049
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.79145872404707451, 1.5020638777523012, 0.57149095001539818, -0.86608017835098494, -0.057392395578363578], [-0.31667802901131675, 0.86732341314651629, 0.51916028562166028, 1.3370280718794867, -0.34065441487878056], [1.1614256054724708, 0.042642049626921114, 1.3209220871317895, -1.5639375514145961, 0.56672586332976138], [0.67157544460460628, 0.50600564472614895, -0.047228300133925122, -0.65857050764621194, 0.68245131776732326]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[0.1479652146242918, 4.1612595599710858], [-2.1187636466565731, -1.0657876668805253]], [[2.3827024272799395, -3.5265008494876859], [-1.8889008409871497, -0.79954368633192718]], [[-3.4136189627771718, -0.71187258310490797], [4.0930700158745061, -4.4824490066171085]], [[0.71459538336689743, 2.8722058051086847], [-1.4294225261192706, 1.9944699333369789]], [[-0.73685802009854662, -4.5139916383963303], [3.4238854647667498, 4.144736854756605]], [[-0.0060205783595161932, -2.9232736501672663], [0.096623355485614404, -4.1091183811356133]]]),self.functionspace)
      arg1=1.89180080015
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.078213950756732001, 2.1996288190841162], [-1.1199718524758251, -0.56337203515156709]], [[1.2594890683490667, -1.8640973453507981], [-0.99846709063647732, -0.42263629779072903]], [[-1.8044283322593002, -0.37629362618377216], [2.1635840388454892, -2.3694085583785283]], [[0.37773288990634646, 1.5182390264791985], [-0.75558828710070003, 1.054270583446921]], [[-0.38950085021692038, -2.386081895115018], [1.8098551731765771, 2.1908949686632941]], [[-0.0031824589349187357, -1.5452333300309238], [0.051074804217270106, -2.1720671546470336]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[1.5084573242708252, -2.5803708796582745, -1.1777426987284523, -2.564900761139838], [-3.0758559953252238, -0.78994909954369863, 0.76428847495055408, 0.3382666633936271], [3.1890669638980818, 1.5475633838348255, 1.7743670008441832, -3.9189421585397364]], [[2.8886381379600454, -1.9753731057058745, -4.3118525748516232, 2.7197474435711904], [-2.9767046770821026, 2.5027626740392561, -3.7047696981893887, -0.18847711833061354], [-2.4537083306501906, 4.2012488617251407, -2.3724531369513127, -2.1225156208118188]]], [[[1.8931348644030876, -3.1492338360736696, 2.3662370077878228, 1.3456758179477468], [-1.1948873512146294, 3.4230678648886261, -2.9123833291197734, 4.6774587640729397], [1.6544401705031913, 1.7607343079902815, 1.2838451591295144, -1.7187912285633535]], [[1.270694520483783, 2.9477883968927854, 1.8060703594957195, 4.1066761502508768], [-2.6025328734561959, 2.6901891658506658, -0.016177737861177377, -3.5552001956310431], [0.33222873141185616, -4.9435887813956914, 2.7237717319960177, 2.0318246416162156]]], [[[2.5369226557860172, 0.2162836919054314, 2.9925418536456903, -1.858662794685527], [1.7696566455846003, -1.9353247757315231, -3.5160104586937733, -2.5422554254372343], [1.4428157541473849, 3.9171163259232546, 4.935836817099025, -0.35259299736052707]], [[3.1602209574717968, -3.6880575234602131, -3.3444550745686552, 4.1472765711728421], [-4.1199084355585383, -0.85098036479261641, 3.7856366769354945, -1.4011176928899349], [-0.84519334457415685, -3.3869699074718342, -2.5397539988389428, -1.3507166442431942]]]]),self.functionspace)
      arg1=-1.12397825294
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.342069849065449, 2.2957480475424532, 1.0478340623156062, 2.2819843306039584], [2.7365796333606083, 0.70281528800058968, -0.67998510910156529, -0.30095481163446691], [-2.8373030844390628, -1.3768623901683086, -1.5786488717277947, 3.4866708037299241]], [[-2.5700124805922009, 1.7574833859513677, 3.8362419945296606, -2.4197509484427404], [2.6483650100045848, -2.2267002653311105, 3.2961222234608001, 0.16768751338227667], [2.1830567666593854, -3.7378382106151964, 2.1107642703529681, 1.888395629779547]]], [[[-1.6843162752106042, 2.8018636729376767, -2.1052337993253909, -1.1972436427773367], [1.0630876069828548, -3.0454929674543916, 2.5911385042476405, -4.16152069833577], [-1.4719503390580457, -1.5665199067604658, -1.1422330955032103, 1.5292032777977951]], [[-1.130533012684545, -2.6226382843194744, -1.6068552525618713, -3.6536971596409313], [2.3154655053669515, -2.3934530395256322, 0.014393283694695135, 3.1630506963492819], [-0.29558288209223749, 4.3982957574862045, -2.4233313454960776, -1.8077081440931513]]], [[[-2.2570922961889379, -0.19242693650027737, -2.6624552973586479, 1.6536465806435512], [-1.5744580831184258, 1.7218525097580473, 3.1281837077443901, 2.2618368449711643], [-1.2836687457060489, -3.4850463660580098, -4.3913988586570536, 0.31370090696992015]], [[-2.8116388811051176, 3.281253452925855, 2.975551409319602, -3.6898192294549212], [3.6654698832439858, 0.75711461727096341, -3.36806932611591, 1.2465701086563337], [0.75196592315402444, 3.0133767255942892, 2.2596113334083228, 1.201728450452467]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_array_rank0(self):
      arg0=Data(-4.38958461836,self.functionspace)
      arg1=numpy.array(-4.59772210786)
      res=quotient(arg0,arg1)
      ref=Data(0.954730302394,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([-4.6973442207100549, 3.0132705240692079]),self.functionspace)
      arg1=numpy.array(0.665363920485)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([-7.0598120458423841, 4.528755514534109]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_array_rank1(self):
      arg0=Data(-3.48386296437,self.functionspace)
      arg1=numpy.array([3.9115720552735045, 2.8697534288179494])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([-0.89065544879196001, -1.2139938328453239]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[4.989790678921624, -1.1321060304279138, 1.7049250216805483, 2.3591563192226559, 1.7527132633134093], [-4.0223878001585938, 3.4070437909748073, -4.6708304675191323, -4.1195163161743107, -2.0833985352483597], [2.5863194866688621, -1.5396513098294182, -1.2597569930986974, 0.16700219317410259, 3.1918897921019038], [-2.9008135915381974, -4.225297939736584, 4.078435335357506, 2.9540931402550052, 0.36206863804872214]]),self.functionspace)
      arg1=numpy.array(4.04438336631)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[1.233758085470368, -0.2799205534914464, 0.42155376166475034, 0.58331669022206345, 0.43336971413626685], [-0.99456145371122928, 0.84241365923895262, -1.1548931059385596, -1.0185771088108189, -0.51513379087783562], [0.63948425567560785, -0.38068876522844708, -0.3114830813502687, 0.04129237464613688, 0.78921543854964415], [-0.71724496142111682, -1.0447323008342932, 1.0084195700475418, 0.73041867515970205, 0.089523817416805027]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_array_rank2(self):
      arg0=Data(4.66332680527,self.functionspace)
      arg1=numpy.array([[3.899682266161415, 2.1067265204761778, -4.2866315574291303, -1.2156950554981782, 3.3563954389467128], [-4.9366599672944229, -0.52136835524176117, 4.0093732224652836, -2.9521355629998025, -1.6495490680888016], [4.9394683215506205, -2.9098799868695457, 2.1509707109009035, 0.034845312592946875, 1.9631505847600828], [-4.457093968289044, -3.0019540880757556, 0.49739201589516835, -3.4528224245821324, 3.3581708452133903]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[1.1958222457601431, 2.2135416058739832, -1.0878767495634583, -3.835934664849951, 1.3893853957612246], [-0.9446319649655599, -8.9443994028171101, 1.1631061880551894, -1.5796452113240773, -2.8270312750836597], [0.94409489072463859, -1.6025838956640142, 2.1680103692899562, 133.82938645859824, 2.3754300059677051], [-1.0462706953118761, -1.5534304218027954, 9.3755562137028807, -1.3505840242671943, 1.3886508519717147]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[2.9664762466993766, -2.8982283168035003], [1.703180022675709, -3.7533027692976617]], [[2.2897453846524254, 2.4559098400450621], [-3.4283411831813417, 1.6098200461697889]], [[3.6254841413605892, 0.6130266489254188], [1.875694973034264, 3.2766939436132674]], [[-0.47360512114693165, -2.1672344089988629], [0.63430276760174387, 4.539246926227186]], [[-3.4389603190645524, -2.9530004746045337], [1.381431985785591, -2.1555274603075967]], [[-4.9329371971720573, 3.3243165166265065], [-1.958800356392767, 3.2917796874963887]]]),self.functionspace)
      arg1=numpy.array(-0.159094003316)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-18.646059467138784, 18.217080822606061], [-10.705494784051908, 23.591729990209192]], [[-14.392405351075912, -15.436847328328602], [21.549154032959436, -10.11867205937412]], [[-22.788314240572056, -3.8532354214945252], [-11.789853381878258, -20.595961351858453]], [[2.976888577035715, 13.62235133835453], [-3.9869684235740332, -28.531854322660415]], [[21.615901588881478, 18.561356261399105], [-8.6831178862212202, 13.548766234911019]], [[31.006430753818325, -20.895297417458512], [12.312219917550387, -20.690784183446496]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_array_rank3(self):
      arg0=Data(-1.62544361144,self.functionspace)
      arg1=numpy.array([[[4.4977575764422149, 4.2773601865279147], [-2.6222328106398085, -4.7098715195287708]], [[-2.3740993938671515, -0.74765217278245366], [0.065186454062073906, -0.051469274654937003]], [[-4.7063508044722093, 3.1628111358350921], [-1.4981812983469434, -3.818338086792743]], [[4.7199598630757862, -2.77480150734812], [-0.40404655009656754, -1.920028222637681]], [[0.60762320889049093, 3.2260366112860943], [4.1396631509538935, -4.0539972382183977]], [[-0.012395316948378365, 2.2753626891525567], [-4.1711068363296473, -0.96831914363224314]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-0.36138977786370613, -0.38001092743014558], [0.61987006067499217, 0.34511421483486265]], [[0.68465693375523806, 2.1740639171635405], [-24.935297291787371, 31.580853282519275]], [[0.34537238700750394, -0.51392370319515768], [1.084944534569166, 0.42569399945423986]], [[-0.34437657492621654, 0.58578734627713647], [4.0229117438244941, 0.84657277027014965]], [[-2.675084801983262, -0.50385156998799607], [-0.39265117768355279, 0.4009483766075736]], [[131.13368687582485, -0.71436682124757822], [0.3896911959381883, 1.6786238526051269]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[4.569266124159526, -2.3908867024571121, -4.0972624546433947, -3.4528591225339547], [1.7828209220766267, 4.4056658070866135, -4.8286270996660088, -0.50994509413660438], [-0.59532050216155508, -0.27922262988707125, -2.2062376513821178, -3.5852566634239191]], [[-3.1397706282966711, 1.6202550666658517, 4.6833282958122862, 1.8356678158217354], [-3.3636197872159612, -0.57757829352942025, -4.1073506946704885, -1.7357454689846943], [0.46610642454390216, -3.1038028824087585, 2.6196545302193597, 3.8653713637755374]]], [[[-3.9079503274596536, 1.1094673797634798, 0.43299455792331987, 2.6733316913866387], [-2.6925951938420378, -4.5303637836181334, -4.8054507329676621, -4.596162990485654], [-3.6178859808155126, 0.40706345951645773, 1.0129500479230629, -1.5433399383335678]], [[4.5487175164397566, 4.124576998311273, 4.2908430735551342, 3.2381994452788145], [-4.4830129838302426, -1.7870761162528814, -2.6286345763219208, -3.2033054595909327], [-2.3836589691643093, 3.6821837868887428, -2.760477858257838, -2.6100754109906532]]], [[[2.4276318048333838, -2.5914351429943516, 3.5540267107423791, -4.7795242459563783], [-3.7880507349801498, -1.8740339247162319, 3.8146987544352502, -4.7230898032308355], [-0.89414592817533389, -1.4074562009972666, -1.7746422961252026, -0.18479601136659607]], [[-0.020690487669377866, 4.7925787768113519, -2.7290757036008895, 2.1532346942370308], [3.3812769364934585, -0.99557318007945916, -2.2832285555288365, -1.9028024772340113], [0.0051784919781008298, -1.6068923751778286, 3.4489554117580958, 1.4574859215293756]]]]),self.functionspace)
      arg1=numpy.array(-0.74390711087)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-6.1422535924117119, 3.2139586616697309, 5.5077608410729519, 4.6415191790512766], [-2.3965638935653306, -5.9223332358477503, 6.4909005830330182, 0.68549565757004971], [0.8002618787520841, 0.37534609604774943, 2.9657434633244133, 4.8194950835095547]], [[4.2206487643683372, -2.1780341160763941, -6.2955821061281929, -2.4676035340969706], [4.5215588587181728, 0.77641184643905181, 5.5213219965985401, 2.3332825343686134], [-0.62656535706323591, 4.1722989833777149, -3.5214806955629423, -5.1960403487163465]]], [[[5.2532772847019995, -1.49140579993452, -0.58205460278119259, -3.5936364262757876], [3.6195314636721641, 6.0899589712513942, 6.4597456628021117, 6.1784098086020451], [4.8633571691313069, -0.54719662383735779, -1.3616620047346202, 2.0746406584679957]], [[-6.1146310473122991, -5.5444785216392098, -5.7679823338954179, -4.3529620808348], [6.0263074761977471, 2.4022839547316774, 3.5335521571356963, 4.306055706130179], [3.2042427533430127, -4.9497897426794575, 3.7107829968594812, 3.5086039276321874]]], [[[-3.2633534071149208, 3.4835466755587241, -4.7775140992909302, 6.4248938827445503], [5.0921017955480607, 2.5191773237998909, -5.127923498373228, 6.3490316656734276], [1.2019591090211372, 1.891978421004699, 2.3855697441185488, 0.24841275028352933]], [[0.027813267768325215, -6.442442486144393, 3.6685705294719337, -2.8944940339660525], [-4.5452945496660062, 1.3383030831841993, 3.0692387828633234, 2.5578495613648058], [-0.0069612077938699113, 2.1600712665582571, -4.636271600800149, -1.9592310655904255]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_array_rank4(self):
      arg0=Data(1.44257501897,self.functionspace)
      arg1=numpy.array([[[[4.5176163558739919, 1.0836846042577157, -1.0181192915661263, -1.6115927369593752], [1.7694704607838876, -3.658750433798359, 4.4057412764737052, -3.5328149400565403], [4.1370676235656489, -1.7392790908361331, -4.6349591756610202, 3.7659667658062528]], [[-4.4160605082327749, -2.3309315609379953, 1.5546136525084711, 2.2028341684495922], [-2.8413835611285245, 2.1711922523196083, 3.9812332134267887, -3.6043107416764855], [-2.8219407024955112, -3.9524901506838308, -1.9186309487180733, 2.820289408955464]]], [[[-3.7087274654844746, -1.0085863949949312, 0.57985815547805686, 4.7258746797018869], [-0.16430600394919814, -3.8910693773407132, 3.2684247090106098, 0.66078785631353831], [2.1027885196530551, -3.2972988011227979, 1.1821266598289357, -3.8619135467312207]], [[0.31803477140738678, 1.0886361484994538, -3.6103294768256902, -4.164286630973729], [2.9310221952289437, 3.9836307784276741, -0.25353563736665841, -2.0099123728122716], [4.9578807629423043, 4.2004621186468061, 3.8833581334110807, 2.6214679089951023]]], [[[4.1089457296453169, -2.3702307962736802, 1.820540030783909, -0.26688800044948913], [2.9113701673740602, -2.5677071710214419, -0.18304876343096232, -0.33304248061572395], [-4.3373009059225698, 0.88527528916984988, 0.58402050577127884, 3.5784754903773095]], [[2.0648885641972745, -4.5218326624214953, 1.4598443910428758, 3.8977930697243686], [-1.3058016303999951, 1.2349410906978147, -4.5705698129716108, -4.5166596853911827], [4.1977612453952755, 2.069484214397538, 3.4433545308362277, -3.6310400196544967]]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.31932216136381419, 1.3311760758641691, -1.416901762809301, -0.89512380261277225], [0.81525803958949983, -0.39428079205516653, 0.327430715614971, -0.40833585779251536], [0.3486950541376157, -0.82940974026008918, -0.31123791263263212, 0.38305569557020808]], [[-0.32666559171479576, -0.61888347266175903, 0.92793152603703932, 0.65487227301613915], [-0.50770161364530719, 0.66441606791338537, 0.36234376175328992, -0.40023602912195388], [-0.51119962148548936, -0.36497877641026316, -0.75187727996051179, 0.51149893141799241]]], [[[-0.38896765329769417, -1.4302939501553071, 2.4878067253893983, 0.30525037516678294], [-8.779807093453698, -0.37073998920988271, 0.44136706438218226, 2.1831136955484718], [0.68602952959255059, -0.437502060316455, 1.2203218724285896, -0.37353892092981145]], [[4.535903456676861, 1.3251213648918483, -0.39956880063992284, -0.34641588027116327], [0.49217471683373615, 0.36212568363064529, -5.6898313544930827, -0.71773030430761942], [0.29096605746405457, 0.34343245534015954, 0.37147617330443483, 0.55029283937455831]]], [[[0.35108154594553964, -0.60862217351918724, 0.79238851910827834, -5.4051700209100293], [0.49549694337609645, -0.56181446048473682, -7.8808236227953259, -4.3315045465166788], [-0.33259740337599586, 1.6295213891296441, 2.4700759728720207, 0.40312558318461417]], [[0.69862124474063436, -0.31902441480390498, 0.98817040214790441, 0.37010046279142583], [-1.1047428532680799, 1.1681326581781006, -0.31562257617771594, -0.31938979676421236], [0.3436534225362699, 0.69706983456752736, 0.41894466748966103, -0.39728976027850721]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([3.0387994188598491, 2.7130358246142938]),self.functionspace)
      arg1=numpy.array([-3.0202934064771569, -1.5176810540468475])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([-1.0061272233826704, -1.7876192216935642]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[3.6635196459082167, 1.6872506938280036], [0.47912752284165983, 2.2392774263152626]]),self.functionspace)
      arg1=numpy.array([0.38622350516006598, -0.75168484581362272])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[9.4854911649924372, 4.3685862493758414], [-0.63740479206156242, -2.9790110027979502]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_array_rank2(self):
      arg0=Data(numpy.array([3.8775236963086464, -4.90677166488077]),self.functionspace)
      arg1=numpy.array([[-4.7374254653036107, -0.78731555730199965], [-2.3013692500360516, -4.912029469291892]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.81848753604826263, -4.9249931115247865], [2.1321096841821032, 0.99892960650093987]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_array_rank1(self):
      arg0=Data(numpy.array([[[2.9744468247249314, 4.3268795569571612, -4.3705033863581892, -3.7609548733276075, -4.1037980272770991], [2.8176115840234885, 1.1644198602349611, -0.41099601567885191, 2.8062486638449267, 2.7210593283667279], [-2.2277418128477002, -1.9322372828510548, -4.9312889812451086, -0.9407045393809419, 3.7317032831658246], [2.5770075778961754, -0.54051971385333175, -2.7632414140806292, 1.1633613222050361, -2.3671936930434012]], [[4.1186396144846285, 0.92344762896435384, -4.8585953411032587, -3.4387804913205002, 1.4382411746322088], [-4.4864623960087471, -4.223794697947695, 2.6214889212899166, -3.6182516003080787, 3.0302583133129914], [2.1193229965521256, -3.3839794494725428, 3.5015509385580241, 0.61420865222928178, 1.3082127283496261], [3.659126908713823, -1.6786974764910156, -4.8650173994029666, 2.0412530328652645, -4.9714614906999843]]]),self.functionspace)
      arg1=numpy.array([2.38510893543212, -3.9443639060476618])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[1.2470905544555508, 1.8141224045069635, -1.8324124828983408, -1.5768482594050657, -1.7205914439851768], [1.1813345471002608, 0.48820405765827146, -0.17231750280805941, 1.176570437583182, 1.1408532700305123], [-0.93402099155864804, -0.81012538008079849, -2.0675319722248604, -0.39440736873953752, 1.5645840018999957], [1.0804569718444699, -0.22662265266949058, -1.1585388713409022, 0.48776024647036303, -0.9924887110510644]], [[-1.0441834761163289, -0.23411826366945651, 1.2317817160972038, 0.87182130585061379, -0.36463196826921523], [1.1374362261884401, 1.0708430557007174, -0.66461639537633466, 0.9173219526627413, -0.76825018824122038], [-0.53730412483054413, 0.85792780029349869, -0.8877352652957049, -0.15571804905920361, -0.33166633695836739], [-0.92768491849940571, 0.42559396558648332, 1.2334098767975543, -0.51751133553765938, 1.2603962537730187]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_array_rank3(self):
      arg0=Data(numpy.array([3.9124222237513901, -0.38171181128285792]),self.functionspace)
      arg1=numpy.array([[[2.3091539981802658, 1.6751282394786617, -1.7479915339226828, -4.4315265157287254, 2.4240956587645277], [3.8706405366096241, 3.6106718324199853, 0.15301960175648688, 1.001120894822316, 3.1151818963291316], [2.3700037115547721, -2.7013075507535445, 4.7656123533810728, 3.3575576582601627, -0.11696074877821516], [-2.4414029004902704, -3.2016823677200303, 1.4486380097564346, 4.9424346421116425, 1.8026368322452608]], [[4.5720660183695312, -2.4838414368524262, -3.4569996942177541, -3.8898805010564885, -1.0321218600885009], [-0.084392375319273683, 0.75774746595583942, 4.4710675751982549, -3.626282844842077, 2.9965518886272822], [4.1242354223983479, 3.6864572037113668, 1.5274596359796817, 4.2739536844995829, 3.0570918773356901], [-2.3509931593738309, 3.7657536184448066, 4.0233163076572591, -4.836110350766111, -1.4161836659005989]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[1.6943097891412109, 2.3355956466765946, -2.2382386572385107, -0.88286106601531344, 1.6139718783809909], [1.0107945149508415, 1.083571813040999, 25.568111397764326, 3.9080417200220223, 1.2559209554863267], [1.6508084796140505, -1.4483438669025297, 0.82096946491580092, 1.1652583877826093, -33.450728253888443], [-1.6025303414547909, -1.2219894962714521, 2.7007590560247703, 0.79159817115554565, 2.1703884852271114]], [[-0.083487817050153221, 0.15367801085022995, 0.11041707985144361, 0.098129444125387724, 0.36983211580280589], [4.5230604049093746, -0.50374541444537624, -0.085373751316190322, 0.10526255882819348, -0.12738368146787532], [-0.09255335163696421, -0.10354434900222546, -0.24989976971668759, -0.08931117168331007, -0.12486108582890434], [0.16236194042543448, -0.10136398977703127, -0.09487491961702739, 0.078929508137131149, 0.26953552739934655]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_array_rank1(self):
      arg0=Data(numpy.array([[[[0.34856690345269215, 3.1505495541585145], [0.87502746571102108, 2.33231720313381]], [[-0.24051062407320689, -2.2072750475050085], [-2.3507324653800463, 1.8345720048812844]], [[-2.8658451072730551, 1.7177851274364331], [4.1053134902033737, -2.211084447342623]], [[0.12428165783863765, -1.84941338350436], [0.87962915383641871, 4.2193772658346056]], [[4.2375430813814017, 4.5892211617061314], [-2.4904804475650213, -1.472795687167836]], [[4.7148167098728777, -3.4558307837012303], [-1.4100125414568874, 0.41535729857456616]]], [[[0.85236281597149421, 2.7183042516755922], [3.5863961511734495, -3.0297332283616174]], [[0.26192491502089776, -2.1142349012353301], [2.4249774632029677, 2.0968268214046191]], [[-4.9771333052130764, -3.8857055152809874], [-3.3674333326359629, -2.8337202417980589]], [[-3.6195352097122013, 0.65617922744900614], [-3.2830401080787199, 0.45288813861461907]], [[4.4607153452770572, 3.1448414954620389], [4.2651783130793106, -2.087844946138909]], [[1.9094673489046965, 4.3613038351770612], [3.3674100389397257, 4.5215098195168117]]]]),self.functionspace)
      arg1=numpy.array([-0.72233659098341452, 3.9950730487171331])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.48255468129911688, -4.3616086925199857], [-1.2113846600512481, -3.2288509709282636]], [[0.33296198347887546, 3.0557430913197217], [3.2543449891963472, -2.5397744317280582]], [[3.9674649506144948, -2.3780951275052811], [-5.6833802156059336, 3.0610168098121329]], [[-0.17205504939108265, 2.5603207792457301], [-1.2177552194038246, -5.8412896681451461]], [[-5.866438353361362, -6.3533001359632131], [3.4478115585621842, 2.0389326881014291]], [[-6.5271741300741253, 4.7842388532420053], [1.9520159425085286, -0.5750190475732152]]], [[[0.21335349956747307, 0.68041415476707567], [0.89770477471621835, -0.75836741691982379]], [[0.065561983930432777, -0.52921057398793669], [0.60699202082967108, 0.52485318687174842]], [[-1.2458178472634673, -0.97262439707547654], [-0.84289656073179609, -0.70930373668836955]], [[-0.90599975659380705, 0.16424711624727698], [-0.82177223496149698, 0.11336166650570932]], [[1.116554138280262, 0.78717997321022359], [1.0676095933837584, -0.52260494881547703]], [[0.47795555315761995, 1.0916706107733198], [0.84289073012595905, 1.1317715006409517]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_array_rank4(self):
      arg0=Data(numpy.array([3.151416490152549, 0.29260741460854156]),self.functionspace)
      arg1=numpy.array([[[[-2.2483875573228129, -3.2535137157857141], [-2.9403276717021076, 1.4699813374438842]], [[3.5417151987172186, -1.2646617172115384], [0.32397228496801311, 2.2168860533078103]], [[0.76017899576336134, 3.8220456463613655], [-4.6143215523887795, -4.607417531785023]], [[-0.22279035880291875, 2.6812291382391837], [-1.838726599584759, 0.98162479843022332]], [[-2.6136036733106014, 0.19268511182214265], [1.359008397378501, -3.5332436811375976]], [[4.9403468736652343, -1.0782689177851457], [3.3167529499861281, 3.5238544959108395]]], [[[-3.3400063109073352, 1.9611291204918055], [3.0531601424240815, -3.7356048881194992]], [[-1.6366421263112629, -0.60634954380918948], [2.51564846449355, 4.2611675187662019]], [[3.019414524246768, 3.0734910316556263], [-1.4056947668192441, -2.2738910766618368]], [[1.1487922146800802, 0.74406064477283351], [-4.2461367137041908, 0.55659447472033108]], [[-0.67927614830097838, 4.8245606925771636], [-1.1802044077470599, -4.7817698879516]], [[2.2705266149901249, 0.52143884658239337], [2.9396202889287046, 1.3549998837024333]]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.4016340198506461, -0.96861939596633639], [-1.0717909165301449, 2.1438479590716932]], [[0.88979952179496735, -2.491904710376724], [9.7274261916067886, 1.4215509567802675]], [[4.1456242644377985, 0.82453659159009163], [-0.68296421356268466, -0.68398760659566604]], [[-14.145210354189091, 1.1753626145589879], [-1.7139124929525877, 3.2104083914670589]], [[-1.2057744341018279, 16.355267204356991], [2.3189087692405477, -0.89193295865115318]], [[0.63789376955519705, -2.9226628331509561], [0.95015110792792978, 0.8943094823607286]]], [[[-0.087606844829299979, 0.14920354379071299], [0.09583755877810704, -0.078329326406851321]], [[-0.17878521510871359, -0.48257216913256457], [0.11631490597294135, 0.068668366901769792]], [[0.096908659695056701, 0.095203601245232847], [-0.20815857148749523, -0.12868136807946887]], [[0.25470873746304762, 0.39325748064242327], [-0.068911444528897525, 0.52571023949809492]], [[-0.43076356403859928, 0.060649545783253017], [-0.24792943721258567, -0.06119228266207679]], [[0.12887204786622339, 0.56115384675757218], [0.099539187326529652, 0.21594644997976992]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[4.3116571995618589, -3.1981365339881949, -2.1736877462707827, 4.4814990486963886, -0.83284918451909817], [-1.7711974895621827, -2.4727472327168254, 1.3585813935915061, -4.4161342193344293, 2.9783580922530799], [3.2547841377745765, -1.6788664792449093, -4.853265756526346, -2.0837832851294613, 4.1291428463852711], [3.5920737072909006, -3.1872602865348041, -1.0724946362863119, -4.6284858794312029, -4.1238522724401854]]),self.functionspace)
      arg1=numpy.array([[-1.157811468619653, -2.8646074981343803, 0.92155877461254843, -3.9907883604286543, 4.0746855074888781], [-4.362062164967301, -1.8666651209616667, 1.0920272966835345, 4.7700868305107811, 3.4722791729064486], [-0.74628969878454754, -0.94190834065963003, 1.6040557346345539, 0.036598990068569037, 2.6467388748918852], [1.9446725730756098, 1.6232396513304428, 1.9525706940429979, -1.1693749769626471, 1.9842643725500686]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-3.7239717487875916, 1.1164309721562309, -2.3587076659160102, -1.1229608398013435, -0.20439594245700726], [0.40604590732041068, 1.3246871144423151, 1.2440910568055314, -0.92579744902915095, 0.85775306187724087], [-4.3612877721285912, 1.7824096111827381, -3.0256216487590111, -56.935540604411379, 1.560086975544174], [1.847135480298276, -1.9635180079062606, -0.54927314004984973, 3.9580852768487524, -2.0782776375410275]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_array_rank2(self):
      arg0=Data(numpy.array([[[0.11720169606754993, 1.5791913802755433], [-1.5898989715678002, -4.8494229182518431], [-2.7691770933060078, 1.3318709224096352], [4.0874777405673655, 0.71954134514966217], [-2.7565993513383411, -1.8038440717292268]], [[-3.4105293790363636, -0.1808189348469984], [3.7095032702358104, -1.7728511290554225], [3.1825018099332762, 0.58041671126619221], [0.01581530325655045, -4.8474138820983379], [2.8032529377793871, 2.3058239037331116]], [[2.2497896952778431, 4.5670167909199328], [-4.7368771935661735, -2.9962273805477126], [-0.63048845761946204, -2.4007847576016115], [-2.0465021893956092, 0.45183000946287954], [-4.0724365342857567, 1.8181443465548952]], [[3.5496520220560939, -4.8200638217585325], [-0.82969060432685016, -3.8259666343954812], [-4.7523562229876717, -3.9172705390237841], [3.7185801804593392, -4.7928431391611532], [-1.079287333077886, -4.3438657225608326]]]),self.functionspace)
      arg1=numpy.array([[0.44802781566251948, 2.2736497351968801, 4.5365881904507557, -1.732385187340765, 3.3471559585444801], [3.1951611722426296, 4.1838218640862497, 0.56239244455230164, 2.8931793959998888, 0.36307401417330709], [0.41687173000682343, -2.7078732493612714, -4.0265260681860635, 3.6342465823243071, -1.169342815940948], [-1.6198124513402345, -1.1812557482766364, 1.5029730642439842, -3.0080240676488179, -0.21764603430282747]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.26159468669202685, 3.5247619122493989], [-0.69927172464413367, -2.1328803831043577], [-0.6104096243813707, 0.29358426784541369], [-2.3594508718016112, -0.41534720477157161], [-0.8235646577212542, -0.53891844122902277]], [[-1.0674044892209837, -0.056591491038958969], [0.88663030854110447, -0.42373963009120963], [5.6588630248522271, 1.0320492689553094], [0.00546640947271251, -1.6754626031141984], [7.7208856275825424, 6.3508370572410797]], [[5.3968392033708259, 10.95544855211261], [1.7492979757023339, 1.1064873074300865], [0.15658372675170459, 0.59624219909326381], [-0.56311594247596564, 0.12432563372568643], [3.4826711882680392, -1.5548428756470949]], [[-2.1913969232173196, 2.9756925363614823], [0.70238016241386048, 3.2388977915893995], [-3.161970321389739, -2.6063477997153757], [-1.2362202219232634, 1.5933526565521849], [4.9589110894443937, 19.958395917827207]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_array_rank3(self):
      arg0=Data(numpy.array([[2.6711719967190684, -0.44174715656421082, 2.0764381865675139, 1.2994515835863343, -2.2194530212820682], [4.9491016036354694, -3.9183772542159079, 4.2111758429139687, 2.2072779639800455, -1.787204815485449], [-0.12188442415463907, 4.5161013274555817, 3.5494472014253873, -0.082961469995747272, -3.5974298564874072], [-0.51738868679261962, 3.4793899892542939, 3.5238708744141984, 1.4069959481607963, 4.8653048149443414]]),self.functionspace)
      arg1=numpy.array([[[3.6857596456909025, 3.3444561714759136], [-0.3836338005634321, -0.16587282802165593], [2.5045706640912471, 1.0872131866749051], [-0.19255452198188294, 1.0065370244755272], [0.38640362352030966, 0.91271510235773867]], [[2.4971893239329246, -2.7424596709095472], [0.44686798882039902, -4.2942196014562857], [4.9321108531405713, -4.7228629197693923], [0.93239954301219985, 0.75010203322447033], [2.8619648407475591, 0.089661692011512528]], [[2.4776789796797063, -4.2327666463140625], [4.6645045617324215, -2.9945493317426197], [-0.94080658273047924, -3.5841398006927472], [3.544117006143523, -2.1761583409690211], [4.9487903098108976, -0.22271869300182168]], [[3.9150819353955058, 0.11812392375098213], [2.3735894811259257, -2.3546408809040589], [3.0039321425789556, -4.583330505551789], [1.4375903447641249, 0.31534368280924241], [-2.3946557429360391, 1.4462481737919264]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.72472766905514052, 0.79868650081315795], [1.1514813238964587, 2.6631676919774798], [0.82905953357116569, 1.909872150206364], [-6.7484864557405562, 1.2910122052027198], [-5.7438721745460342, -2.4317040613755001]], [[1.9818687979335619, -1.804621470329254], [-8.768534225419188, 0.91247714785873557], [0.85382830360198825, -0.89165743627375738], [2.3673091439419172, 2.9426369563239283], [-0.62446777473989601, -19.932758075276702]], [[-0.049192984706354199, 0.028795450904617503], [0.96818456658948537, -1.508107173117607], [-3.7727703723372303, -0.99032052286000272], [-0.023408219833582902, 0.038122901460748197], [-0.72693115514625062, 16.152348094364861]], [[-0.13215270978494922, -4.3800499540070339], [1.4658768994897236, -1.4776733120841723], [1.1730860442769062, -0.7688450287723595], [0.97871827901824948, 4.4617857431820376], [-2.0317345527833948, 3.364087093149422]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[3.4992886578516362, 0.21539362884639246, -2.9131565576687599, 4.0911293094068277, 3.7949662044912387], [-4.9220749706251237, -0.038820493668174194, 0.1923040441388304, 3.6767130922025775, -1.0505334041380889], [-1.0675130546847011, -2.1056868453794508, -0.43373663076363655, -3.5237467966655913, 1.4970599129355691], [-3.3245180597112514, 0.44436868354867087, 0.64004543383139989, -1.0651706831430987, -3.3623630449487574]], [[-3.0897230719985922, -4.3083580630882148, 4.6528567540414318, -0.83715172151664596, -1.6299571485467013], [2.2893903269387685, 3.4153308535795368, -4.0193409497567076, 3.795061248033587, -1.8033320643901982], [-2.1273184470372932, -4.7317602381002626, -2.1964872774780653, 1.5246368049391377, -1.5210566849249374], [-4.6735582439855161, -3.0211561612976237, 1.9246906099721404, -3.2135363065561959, 0.27012268568076436]], [[2.964750599816961, 1.3838906876536932, 1.5278927354143264, -0.69536784820641184, 2.868093523266106], [-1.6783107507903736, -2.2597243491116883, 0.29389102120837318, 4.669488364297063, 1.1123005562291084], [4.0317300116172881, 3.7019846408843886, -3.2838773812870636, -1.7456224244145124, -3.7029722224584125], [-0.95490710622938302, -2.372843535996052, 1.6327099346130245, -0.14416630445541578, 4.1271630811041362]], [[-2.0547774185573022, 2.6463617504437051, 1.8165422029674758, 4.4945986411982446, -3.3745178064554251], [-1.7363266845126146, -4.7824843180092982, 2.2824713082528891, 1.6288504588940578, 0.58808192504561951], [-3.4864003622488848, 0.80756382403152305, 2.0116704527390521, -4.1794114858553089, 2.0291631262209258], [-3.3211130292524285, 3.3461397184482422, 4.1154712111561906, 4.3260371462682699, 1.6869769704835926]], [[-3.9938250091943184, 1.7071519338952772, 3.8368726188540805, -0.95028026723541181, -0.2197152823642714], [4.0245816132882801, -4.7893867233123819, 2.1172632226446355, 4.1307401656944105, 0.061781112012255335], [-2.165493611452094, -1.6979693297288789, -1.642479612935055, 2.5112282500998742, 3.498029276696764], [-2.2712475243311325, -4.7863755893184994, 0.93745892221624771, 4.3625884711387286, -2.9480412080919125]]], [[[-3.1113703623624058, 4.2227468316769592, -1.151378222564936, 4.1314607255873099, 2.6697968429209631], [-4.9948789394748641, -3.7786275643107068, -2.4855479153629578, 1.5366416391218207, 3.0798420224531657], [-1.1330792849236593, -1.9752805680886087, 2.2287296593495318, 4.3357121128830141, -1.7495568107288797], [-2.1711552078634986, 0.32565106310445557, 0.17905846109166479, -1.5446264566010157, -3.9425337132006764]], [[-4.0704519179953236, 3.7054732861488322, 2.985556946038237, -1.3303252797629916, -0.74003206810398314], [-1.8877061689292365, -1.4755184449395653, 3.0504190374268987, 4.3934255954780621, 4.6062999742514616], [-1.8667117788576415, 4.4799832087380551, 0.46902632188290472, -3.6595780448680602, -2.429144753719906], [1.2078539840339229, -4.3221363160602539, 1.9048107157926273, 3.5593760980535674, -4.762418300426833]], [[-4.7729624986768515, -0.88263079256207266, -1.8583649179608841, -4.8858110559152994, -4.7294026174249577], [0.29973443801841526, 4.9394373355037384, 0.87064587274213423, -1.6744810497246396, 1.2629470155352651], [0.82060680622968007, 0.6266751796894976, 0.75705718194842042, 0.55897020719593815, -1.6794664704308548], [-2.7285321396835585, 3.0191436456486116, 4.8214810155038244, -0.74079164107439777, -3.2376987682279323]], [[-2.5899265439345074, 2.5927340730582777, -4.3742359921067351, -4.0320766899049456, -3.2090535847997268], [-3.332380619580789, 3.1412324148961712, -4.9930927635026725, 1.7727627267456247, -0.6744418885575918], [-2.3669523071284182, 1.6607865189053435, 3.1467574655583626, 4.0454755477978903, -0.55412936536095803], [1.3899136587561243, 1.2442825357351674, 4.7845638952540437, -0.42763577285742649, 1.0561770837596995]], [[-3.1600823123131336, 3.5182825587192461, 0.32738744704615641, 0.12743694630637314, 2.3772286827507747], [-0.65141070583600769, -1.9805550968646979, -4.5027148263346559, 4.654749740453596, 1.3231660892851584], [-0.79780942597979987, 1.5247378680857402, -2.8802802662519458, 0.82341270645915898, 1.0025657697754298], [0.36129496377959924, 2.1585223088351393, -2.1003526082782855, -0.57379214940316281, -2.4228961671916585]]], [[[-2.2272253901654917, -1.2885409264630465, -0.22533097618939824, 1.0842432052747437, 3.8213135208070419], [-4.1242544748175884, -4.9523612885165065, -3.8790671919899089, -0.84859213305707115, -3.153125971878187], [0.62647430915966495, 0.0039594547793884161, -4.7239675915452581, 0.13422329164095803, 3.4642020948727374], [1.8527543566395384, -1.6109499242879979, -4.0645528494809247, 0.13045498795439414, 2.1534144582338266]], [[-0.42980955667225462, -0.10645392524418984, -2.1982119221807226, -0.79476738857814588, 3.9645079243997596], [-1.0520305053267807, 1.5093648430246152, -2.749769427307788, -3.1558686848348483, -1.3769389932363163], [1.204691570838456, -1.4665367146195765, -3.6406849840227817, -4.2469372878807787, 1.6873042934118772], [4.6412053965289175, 1.511047264008929, -1.9216893783884426, -4.9457996398534911, 0.99675751096246135]], [[4.9490266179021365, 1.7413676345690376, -3.9816593827468507, -2.6106763536819466, -0.7847693658406083], [-3.2440051331540087, -1.9923117139790349, 2.3804871083067631, -4.028303638544223, -3.0776439373724873], [-1.7066463774032448, 1.8261853008014164, -2.6323720116687834, 3.7305066122816815, -1.4040563682235705], [1.3487027037191055, 3.0386453518896008, -0.10591045680789701, 0.69238702105574923, 3.2792604400896668]], [[3.1012953981142672, -3.0910031069659727, 2.2856926109493694, -3.867714012541339, -1.7800747461799449], [0.42002922398354325, 1.1163628456385846, -3.1328349897372987, -2.6840597786889244, 0.39594993029274583], [-2.9008765797199878, -4.2103673057661322, -4.2347377831039834, -0.20778299814619405, 4.8485785650203415], [-3.9573375456653204, 0.314754321887591, -4.9768758819454364, 0.59200138172527872, -1.8608465500528562]], [[-4.6607855096242909, 1.3315286778693363, -0.96100061627168287, 3.1042001752419708, -0.96309574598404968], [-3.2789740231513473, -3.8108999246388064, -1.0465176420239408, -1.5269639046868324, -3.4219249815301755], [2.7166219350559171, -3.0119177486377247, 0.25685973396270256, 0.6919435149109141, -1.9525237515864458], [-4.4767309692874484, -0.11900153260445201, -0.77079154835095487, 2.3317317817338949, 1.494833128083811]]], [[[4.4539697043161954, 4.9963723124513759, 3.3668978254651645, -2.8063074292687618, -2.289394060552592], [-4.1405804578464345, -1.1691705030268915, -3.4911549174017353, 4.0482175839786407, -2.7449557692602422], [2.2804597907110233, 3.1295807301533358, 0.040002051082280943, -2.334182448216735, -2.4717543894421556], [-2.5003292896153964, -0.10525292647795048, 0.74102965214136596, -3.771724689237077, -2.6367762667772956]], [[-2.3128192941663741, 1.285320813764212, -1.0581974985472922, 4.8580528180974927, 3.7501775308584264], [4.4717022994774531, 4.5873548613967401, -3.0906739499857538, -1.061253398303621, -3.6062791816113036], [-1.5937100607933874, 0.14915106062011052, 1.6978962406278884, 4.2995087375154419, -0.55029875771125525], [2.5155518674364554, 0.0067647608409089344, -0.74909154963302971, -3.1175487809128812, 3.1001201286407891]], [[1.7098571653891437, -2.5637506594548598, -1.5351164214893762, -3.8662318573283008, 3.0682000767145965], [0.39178352488695278, 4.1843543304162765, -0.80996649349107308, -3.0763420031643429, 0.34253765836147121], [3.817881056957841, 4.5461155314127097, 2.7963034046895983, 3.117527683899862, -2.1699148265144483], [2.7361410086904536, -0.99731326212133986, -2.8424881750781394, -1.0131777420577146, -2.3560461169221081]], [[-4.410122437455481, 1.1689233680753075, 1.1008045580446, -3.1336623524013598, 4.8432229761416288], [-2.4538948865801657, 0.20610531994082937, -0.55236721336813233, -0.26220684739084277, 4.7653880033297487], [4.7774440074365288, 3.9741977961238728, -1.0503602310113713, 2.4026042044401432, 1.8551717195117838], [-2.4195468332493575, 0.018729929129951195, -0.1811364378411966, -4.9463114397783947, -1.6653726374194755]], [[0.3203119842710942, -0.93400070359139864, -2.078781137057347, -0.28428906357823891, -3.2003311082099364], [3.4075702367956424, -0.61364709437854792, 0.39494467970215741, -0.12142748966489236, -3.8791743299663386], [-2.7445744838396879, -3.8442535391414498, 1.2091474053746252, 3.1783147873838207, -0.20775427542066005], [2.0018499527498115, -4.7732105372722025, -0.58663271442487286, 2.3174764962462451, 3.6196301778729669]]]]),self.functionspace)
      arg1=numpy.array([[1.2014321076760739, 3.9213191461247749, 1.2013137646628333, -4.8635361684772871, -3.4097044329616111], [0.2284351890337275, 0.18819679474660411, -1.4870506142547968, 2.9255254966400788, 1.01569176216569], [2.3899393105472129, -2.6112676937449066, 3.9889680227481978, -0.67173512658335888, 0.74903252327197123], [-3.6602220979479219, -1.4169675324582021, 2.0212662491446611, 0.61640504572907773, -1.7772403206613241]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[2.9125979200108931, 0.17928073294380956, -2.4247367279901226, 3.4052105676785063, 3.1587021690570838], [-4.0968398789889813, -0.03231184968351191, 0.16006234801798619, 3.0602753736242585, -0.8744009731603829], [-0.88853381548923982, -1.7526473880013693, -0.3610163470681767, -2.9329554072610584, 1.2460628473058932], [-2.7671293604279108, 0.36986582987882, 0.53273541612720643, -0.88658416596128331, -2.798629255424649]], [[-0.78792950965289232, -1.0987012029729661, 1.1865539581596145, -0.21348727056402875, -0.41566551657941403], [0.58383167542005543, 0.87096477647189752, -1.0249971501882988, 0.96780218763475001, -0.4598789328770479], [-0.54250071666306621, -1.2066756266896568, -0.56013989058981173, 0.38880712028906211, -0.38789413160316583], [-1.191833173947124, -0.77044383502507485, 0.49082733086752367, -0.81950389315594419, 0.068885667198935296]], [[2.4679236074923878, 1.1519810463855817, 1.2718515181944596, -0.57883949111461086, 2.3874641310474614], [-1.3970627825624029, -1.8810442497059989, 0.24464135003968601, 3.8869848174990524, 0.92590344749882403], [3.3561007375528185, 3.0816134383704541, -2.7335717594218467, -1.453094500173689, -3.0824355229940341], [-0.79488567792893972, -1.9752071488684106, 1.3591036602092617, -0.12000720269435872, 3.4355413235962433]], [[0.42248630366423851, -0.54412297118214881, -0.37350235302890994, -0.92414212324146194, 0.69384038476513377], [0.35700910291703186, 0.98333479023075399, -0.46930283423131253, -0.33491073212354266, -0.12091653164979765], [0.71684474865135706, -0.1660445807447056, -0.41362300660527035, 0.85933595249972838, -0.41721970515462037], [0.68285973707320613, -0.68800551749487182, -0.8461890831264639, -0.88948390562143154, -0.34686222370826209]], [[1.1713112053308827, -0.50067446239393665, -1.1252801215738921, 0.27869872181560751, 0.0644382194070207], [-1.1803315191729382, 1.4046339844044495, -0.62095212774956476, -1.2114657580764325, -0.018119198665731071], [0.63509716282686213, 0.49798138317052065, 0.48170732837051955, -0.73649440867185789, -1.0259039589711403], [0.66611272882628292, 1.4037508773630374, -0.2749384706644461, -1.2794623571960044, 0.86460315433595936]]], [[[-13.620363725586188, 18.485535654725631, -5.0402839747905031, 18.085920750928253, 11.687327395635087], [-21.865628323740399, -16.541355035072151, -10.880757583263483, 6.7268166766326969, 13.482344972684746], [-4.9601783758296749, -8.6470065161325333, 9.7565076062798237, 18.980053516373363, -7.6588761045504459], [-9.5044691540187198, 1.4255731110515313, 0.78384797827810826, -6.7617710876101409, -17.258872111067696]], [[-21.628699487023397, 19.689353855032618, 15.864015909825209, -7.068798815379381, -3.9322246114785999], [-10.030490537689133, -7.8402952979420499, 16.208666260943009, 23.344848148947225, 24.475974633115154], [-9.9189350242179142, 23.804779538196112, 2.4922120619239081, -19.445485507845479, -12.907471442277251], [6.4180369578569447, -22.966046376505805, 10.121377031725448, 18.913053768243277, -25.305522906695348]], [[3.2096839562308497, 0.59354455329308609, 1.2496984972446035, 3.2855714587520732, 3.1803911528559472], [-0.2015630370245472, -3.3216336338215564, -0.58548502949137315, 1.1260417323211083, -0.84929658979238154], [-0.55183515501313951, -0.4214215532963162, -0.50909980782853392, -0.37589185051112328, 1.1293942884872705], [1.8348616472956458, -2.0302897673470182, -3.2423113035194198, 0.49816168593940524, 2.1772619823370536]], [[-0.88528592449766663, 0.88624559110354473, -1.4951966739413067, -1.3782401467824239, -1.0969152682091732], [-1.1390707834910265, 1.073732708364302, -1.7067336344315451, 0.60596386146065573, -0.2305370058583246], [-0.809069109070089, 0.56768827371791197, 1.0756212752793866, 1.3828201300737446, -0.18941190770593766], [0.47509880202801813, 0.42531932713087167, 1.6354545194526733, -0.14617400304613978, 0.36102132248469643]], [[-3.1112611424307572, 3.4639274332770533, 0.32232952874215554, 0.12546813024716089, 2.3405020807513224], [-0.64134684369896755, -1.949956837930531, -4.4331508771261721, 4.5828369529438655, 1.3027240532736644], [-0.78548380098966786, 1.5011816821617672, -2.8357818518784885, 0.81069152781494691, 0.98707679546177218], [0.35571319689472986, 2.1251745748460831, -2.0679035574728375, -0.56492744233713688, -2.3854640329320804]]], [[[-0.93191713293151979, -0.53915215368712255, -0.094283137314397025, 0.45366976495586819, 1.5989165515387478], [-1.7256733075256534, -2.0721703127191913, -1.6230818811469057, -0.35506848617958131, -1.3193330717491027], [0.26212979818982274, 0.0016567177090709632, -1.9766056697329415, 0.056161799192392704, 1.4494937505670618], [0.77523071337544636, -0.67405474154870759, -1.7006929136415114, 0.054585063051046469, 0.90103311357340266]], [[0.16459804473583109, 0.040767143674772272, 0.84181791374602166, 0.30436074803129176, -1.5182311388052774], [0.40288114000982733, -0.57801995813764473, 1.0530400364139807, 1.208558085559168, 0.52730671640240834], [-0.46134357412845994, 0.56161867974415403, 1.394221279091288, 1.6263890898868754, -0.64616289530701365], [-1.7773763324405891, -0.57866425094161267, 0.73592201327795825, 1.8940224518921511, -0.38171402853492126]], [[1.2406784385532643, 0.43654589975111485, -0.99816778676598372, -0.65447412433337138, -0.19673493529284844], [-0.81324420618419813, -0.49945542371293128, 0.59676765888605132, -1.0098611008089569, -0.77153888419796002], [-0.42784157899251635, 0.45780895970764557, -0.65991303932670076, 0.93520594575023708, -0.351984864309904], [0.33810817635733198, 0.76176227399189012, -0.026550841271204287, 0.17357547543806318, 0.82208240863019544]], [[-4.6168426741182369, 4.6015207254200297, -3.4026694756534019, 5.7577962793365627, 2.6499652552545898], [-0.62528995040044222, -1.6619092875442323, 4.663795096843919, 3.9957115125731724, -0.58944353901315671], [4.3184827842407074, 6.2678980734285705, 6.3041779646735119, 0.30932281180982613, -7.2179916951516718], [5.8912172209803817, -0.46856909729959106, 7.4089856030892429, -0.88130180825345661, 2.7702087867842575]], [[-6.2224073919577654, 1.7776647027993771, -1.2829891712495729, 4.1442795590263657, -1.2857862857236078], [-4.377612348296875, -5.0877629558617725, -1.3971591479799519, -2.038581579898096, -4.5684598134434893], [3.626841092545618, -4.0210773965873674, 0.34292200402817175, 0.92378300462618457, -2.6067275998341275], [-5.9766843630926854, -0.15887365222089689, -1.0290495064005158, 3.112991371253516, 1.9956852094405546]]], [[[-1.2168577712301345, -1.365046212701835, -0.91986161914950249, 0.76670413821120265, 0.62547954722095278], [1.1312374897052893, 0.31942610905561686, 0.95380958422141304, -1.1060032630938559, 0.74994240671875689], [-0.62303863800766279, -0.85502481718470402, -0.010928858963150847, 0.63771606906733291, 0.67530175035769757], [0.68310862639105663, 0.028755885206244673, -0.2024548326061468, 1.0304633402851888, 0.72038695910163097]], [[1.6322316786991011, -0.90709263573202348, 0.74680433694306048, -3.4284856263922872, -2.6466220608121449], [-3.1558255196714327, -3.2374452881347575, 2.1811889681226151, 0.74896098463351768, 2.5450683230229076], [1.1247329415011835, -0.10526074677332821, -1.1982605117862666, -3.0343029314556786, 0.3883637028412209], [-1.775306638870118, -0.0047741113934863444, 0.52865823138056023, 2.2001554089982958, -2.1878554431396169]], [[0.84593366465833164, -1.2683883978866028, -0.75948253830438772, -1.9127771311495323, 1.5179593870985408], [0.19383073608078291, 2.0701648445309808, -0.40072231643596012, -1.5219875186983198, 0.16946686687437779], [1.8888560864128874, 2.2491423548661582, 1.3834413976253299, 1.5423636966278469, -1.0735423042029673], [1.3536766914543326, -0.49341013958125146, -1.4062908220433576, -0.50125892246331272, -1.1656287824125673]], [[-7.1545852325709509, 1.896355937016571, 1.7858461180222485, -5.0837714163986041, 7.8572085184882177], [-3.9809779358274451, 0.33436669827556337, -0.89611079142740935, -0.42538076092597055, 7.7309360725516054], [7.7504946472101244, 6.4473803770104308, -1.7040097875399702, 3.897768555087616, 3.009663422397046], [-3.9252547493142949, 0.030385749207808028, -0.29385943398135267, -8.024449952267906, -2.7017504950007174]], [[-0.18022997821245906, 0.52553427509671291, 1.1696680031903719, 0.15996095759995635, 1.8007306445866982], [-1.9173379070803778, 0.34528087577385447, -0.22222356487793143, 0.068323618507433084, 2.1826954322772001], [1.5442900163430975, 2.1630465471945715, -0.68035109901439361, -1.7883427190089558, 0.11689712021802047], [-1.1263811255446357, 2.6857428800040144, 0.3300806917359283, -1.3039747462987421, -2.0366576966507695]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_array_rank4(self):
      arg0=Data(numpy.array([[2.2854908013485904, 1.1441259308531473, 2.8614777455199469, -4.5178505473090711, 1.0160860274626078], [0.38955827055461967, -3.5205562070709204, 4.6015815202434336, 4.7950720646487675, -0.45785887638035927], [3.0199268308450087, -1.8308317304886623, 1.6203038866761075, -4.9593734941655221, 3.2594131567079518], [1.9201067433330348, -2.7748880920334029, -0.98394045685615517, 0.17881157601229525, -1.8398312953975959]]),self.functionspace)
      arg1=numpy.array([[[[-3.3716811009485936, 3.9385604267950125, 2.702175986533037, 2.7742103188865883, -2.5263947964844871], [-0.90126882655136598, 4.0319026795226307, 2.1712190974773362, -0.1410601913645575, 0.31293125516757136], [-3.7407883333611602, -0.72683378005853072, 3.9585972798526168, 3.7059993074619069, -0.29438877323344137], [0.99209187607107285, -0.30841230530450403, -2.3456238246106231, -4.2572893340680178, 4.6885365474590177]], [[3.7556019876603166, 4.7060670510666149, 0.77301946641165209, 2.6004721093704877, -3.4606948861940645], [-2.7601439504666194, 0.5108468967362505, -1.0265122991568196, -3.869346373053979, 3.6810374225801912], [2.2881222661902436, 1.8025242117208524, 1.1841998892928611, 4.5217401371175576, -2.1397435370063911], [-0.33748706206027101, -1.7301640035256893, -2.1866685891904036, -1.6017693733335845, -4.8116915341699258]], [[1.8952539821013268, -0.671788950473581, 0.043981417415008828, -3.921408132003136, 3.2345938538145163], [-4.9877347051587577, -0.06177787683199476, -3.0857512956773583, 2.0630332982595778, -2.3097184638033341], [-3.9214473349268064, 3.9831359974948572, 3.4261726456503663, 2.8783675431123257, -4.5269566168370892], [2.5209477017401589, 1.9524401038801056, 1.6075235780097676, -1.3195008373646044, -0.49794976173014582]], [[3.2151640560543413, -3.0534557036352847, 4.4662269426806471, 0.12112115545367708, -0.77265845975186842], [-3.1534481767784381, 4.2358108937860752, 2.1693864437251564, -1.4944444355100375, 3.701386057055732], [4.995088093121387, -4.0451756901645535, 0.20395180480804687, -4.9186618475622463, -0.89459785235903588], [3.6729429709961643, -4.0972095075321757, 0.78271831398430258, 1.0586307188531316, -1.1298751338447897]], [[-4.7902471465604091, -4.8348042121178523, 0.07819902686213176, 4.2890544830627277, 4.1585079494730994], [4.8627787486637573, 1.7250670248474345, 0.89776343995742813, -4.3528334633195751, 0.66246911027972732], [-2.8390362219929832, -2.899584019107909, 0.42075270635973716, -0.84345704116276288, -4.0665705411028332], [2.7626829740583485, 1.6244568874771943, -2.117595272737903, -0.82030496939221376, 0.51055538277296364]]], [[[-1.6658684883916952, 0.47757478295293954, 0.82163571964578352, -3.8120284283645511, -1.7324939342412868], [-1.0358659783854698, -0.24496005633501561, 3.5035301207574534, 0.64488482151664073, 0.58751767125190835], [-4.9102796392583876, -3.0707184222769799, 3.2448462478838138, -2.2590552002129236, -4.9624223729856265], [-1.9083634906963853, 1.1082231467657033, 0.67114276120407723, 1.3648108911883181, -4.7081223509576979]], [[0.23421483233240803, 2.0555999029415819, 1.3718219930698661, 1.9065840864786683, 1.7637058949032527], [4.7008889342502016, 2.0253719178077931, -1.6895803783243322, 2.7489265090400288, 0.33037358211349499], [1.2153450996279416, 1.3351650212784794, -2.1237668926164632, 3.7258322437470159, 0.66551857257829816], [-4.9482239433345487, -0.099235567429226457, -0.92860732463763274, -0.042488886551129923, -0.18915593885482362]], [[0.76126477654468783, 4.5797174736490494, -4.1415069558839264, -2.5431452139903232, -1.3168454040015964], [1.3636075599110615, -1.2876723308847273, -2.8896888784931285, -2.3502308158602525, -1.558451082275949], [2.591721241080549, -0.97043732379772329, 1.0590739297003084, -3.7875498188522903, -4.8776200717649152], [4.0979438179624328, -1.3438325205350421, -0.10871477574456723, -0.7796927017566766, 3.5793700688576493]], [[-2.6218812097524813, -4.410867882486067, 2.2362688404901219, 4.6442222544831431, -2.498708290452937], [1.9747153819658889, -4.2378796362229787, -4.2108887301257081, -4.4576217276772381, 4.3071682700043183], [-0.94689384650292219, 4.922208033083054, -0.86129483856380951, 2.0001041557311279, 2.1494832612338275], [-4.5370460936366896, 0.14869315413707973, 4.3684334992495941, -2.3902968028987157, 3.7311981285477707]], [[-0.92140984490338607, -3.3192964766663957, -0.36095053938515953, 0.69017235372145436, -1.6376104865308427], [0.40085982043151169, 3.7199656232611105, -2.4393897330931216, 3.6514164446322201, -4.5166904857748644], [0.29150791005304821, 4.020755972166528, 4.7840525414075579, -0.90898618123129005, 0.99137337438485318], [4.2475733389228143, -4.5040595974566777, -3.6798813080059443, -1.1361542459524818, -2.1802236706181533]]], [[[-2.0146007689536507, 3.2808827689548981, -1.3571534088923243, 0.14974723223409292, -4.0282358415409716], [-3.8877522262428243, -4.8982863492582887, -0.47062094545039024, 1.4510819351000457, 0.98217781652241953], [0.41889792294906947, 0.083900243404299246, -1.2668987376392691, 4.8159820885782025, -2.4459137826851043], [-4.0969070484466199, 0.80966888403983184, 4.3459833204156535, 0.95531977493343323, -0.97205310145334423]], [[2.512235273541032, 4.0382426595336813, 3.849351449519661, 0.55091155172893647, -2.281123643744194], [-2.5647030440752561, -0.057055426361078965, -3.2254668952357903, 2.74700037727664, 3.5827662515216669], [-3.7558795052642848, 4.4325326740113553, -3.0177137969818348, 3.2725483826150086, 1.3649546255833958], [4.1528715461341097, 0.78588989366300765, -2.1745501994415575, 0.44324209239559398, 2.5525721042302196]], [[3.54731387802887, -2.0358763785739811, 0.64207959192076469, 4.7456476402748553, 4.6619559034234399], [-1.06500955675495, -3.0403844471286878, 4.9807096131376056, 1.919981202054891, -2.8742864157872958], [3.5140822715744644, -2.1385833408613317, 1.0301187982142554, 0.77961240612170801, -0.64899468298907692], [3.3294721532811593, -1.6730749193215955, -4.4669333704388281, -4.3001799316582332, 2.8047697872494801]], [[-3.1816443681560189, 4.8006506148603716, 4.9208058265470349, -3.086063086758033, -3.0846383293067205], [0.00015883339904299021, 2.9669473408483569, -2.3745961562986526, 0.76739483786528773, 0.15180788036576853], [-1.1108360034298959, 2.6449959662844167, -1.4352763628206011, -3.4168364679989094, 3.7803294039019164], [-1.4096813812172493, -0.54729905562522418, -2.2021317612746341, -0.75109798469154399, -4.1325379255517447]], [[1.8219114204172646, -2.2382267109407206, -2.5374147437026737, -0.19729825361687681, 4.8916653880283185], [2.6154238814851771, -4.6772690495017919, 4.400182929137177, 1.0131164290524586, -1.7339503833011625], [-2.6645152140884365, 3.0909098965554875, 0.66379294122656063, -4.8520466597373293, 3.6608867061034225], [3.1026031608467495, -2.0951520123192466, 4.0709797915493144, 0.27411431450237966, 3.1481620269834014]]], [[[-3.7542664924867553, -0.10330665148303098, 0.50990124803647774, 3.1434789154718725, 0.54017598612084328], [3.7339903443887845, 3.6234685849728958, 1.880238668846669, 4.8976718661118674, -0.92795546752942304], [-4.8641871718702214, 2.5962128423352215, -3.166508857869287, -2.0988756936006436, -2.2003352706125732], [3.1913533284762039, 2.3715158735584723, -0.53896214834185141, 0.81143787203891193, 0.89225714869137462]], [[-0.075596842389654029, 3.5720460328698032, 1.3121966446289246, -2.1539257627346431, -2.5736826539308777], [0.97894670678708628, -4.1009352697568424, 1.4594601348857505, -1.6300676043687234, 4.1651061126980533], [1.8791576046212457, -0.16777118588557993, -3.6114945620326102, -2.4603091269506638, -4.1088915626916762], [3.5691073217357747, -2.7715799448808651, -1.8916377713264199, 3.5265424614658212, -3.3097068458136549]], [[-1.6673109473257739, 4.5183077169169188, -1.4521237124478423, -3.9105812574176682, 2.9097844574479605], [1.4200787639998005, 1.4393111795555749, -1.713744562789743, 1.0785198373709566, 4.2006307241110736], [3.12594215595729, 1.64858506864367, 2.3455890721662698, 0.056816934050437418, -2.8154446395386437], [-3.9155433880494717, 2.7577437487303005, -3.206092967113797, 2.6935323917792777, -1.4332456923481987]], [[0.24097463049293921, -1.5410790502613092, -1.8609359854144736, -0.64236947608391404, 1.5390664750578233], [1.6073661607823952, 1.5211768744381269, -4.1842836339260838, 1.566167391862229, 2.7379407982042192], [2.5780560757754252, 1.4770307898427504, -2.270811925249566, -3.6499404079430633, 3.077577114192021], [-0.48122866065766079, 3.5345356011771951, 4.5586044516195408, -4.2918631584335518, -3.7377487777337959]], [[0.35858876690030872, 1.8658521227294429, 4.1852359258308365, 3.4656997365411861, 4.8625500360214335], [-0.91015179943371294, -3.653669663574437, -3.6893196433020616, -4.2067390989333084, 2.6819763935938372], [4.8611231874413754, -2.4089273228372154, -1.1186269062009546, 0.86436813159944847, -3.201943012920677], [-4.5347041520452089, 2.9249219542632225, -2.6193181718851708, 2.5590513667892694, 3.1544532081410068]]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.677849041152079, 0.58028582875098844, 0.84579642952157796, 0.82383472723360696, -0.90464515068226159], [-2.5358591510302655, 0.56685167847830786, 1.05263020392738, -16.202238060502435, 7.3034916251006452], [-0.61096501530602221, -3.1444476908661891, 0.57734865149851311, 0.61670027750594292, -7.7635120940439721], [2.3037088161629691, -7.4105045811711756, -0.97436373955998046, -0.53684178405715932, 0.48746357807261342]], [[0.30464514999522635, 0.24311721835621419, 1.4800738927884538, 0.43996854522316448, -0.33060583740492988], [-0.41451676122172021, 2.2396650310745803, -1.1145759595797691, -0.29568971618069878, 0.3108161638984866], [0.50002831918511648, 0.63473540239487902, 0.96615946446030843, 0.253027793760498, -0.53470236552453243], [-3.390132717587913, -0.66128177936985921, -0.52322786201303173, -0.71428880455617982, -0.23778039858295336]], [[1.5098122850781919, -4.2594891498330449, 65.061062460061976, -0.72970668933107075, 0.88464823555682015], [-0.57370287608929016, -46.318810102550948, -0.92731962861959016, 1.3870245080067081, -1.2388859466482549], [-0.72969939441335285, 0.71839820365652518, 0.83518200670730436, 0.99413216090738843, -0.63209745259701977], [1.1350801698681521, 1.4655905396704876, 1.7800533595051009, -2.168606236912348, -5.7465189572089184]], [[-1.4051695243363072, 1.4795860774827532, -1.0115586613244152, -37.300259648174574, 5.8471508210237335], [1.4326699834732994, -1.0665845715484483, -2.0825476071249232, 3.0230970385775358, -1.2205834456789399], [-0.90445863277776661, 1.116849030387921, -22.151559539083916, 0.91851212531476978, 5.0501468736992754], [-1.2300355826335505, 1.1026652503377243, -5.7720005608552514, -4.2676359818875156, 3.9985396721985791]], [[-0.21211557491187039, -0.21016073927376644, 12.993589156218158, 0.23690210312671084, 0.24433908502960783], [0.20895172903801515, 0.58901249216822216, 1.1317970661745711, -0.23343094469957421, 1.5337862727419334], [-0.35789822602168936, -0.35042475774687798, 2.4149245200430625, -1.2046683801013316, -0.24986312599093644], [0.36778958606676082, 0.62549276333249104, -0.47983013588280227, -1.2386686237137556, 1.9901582898685177]]], [[[-0.2338469532674316, 0.8157010890439057, 0.47412528598752091, -0.10219185871122929, -0.22485404586724825], [-0.37607014679813722, -1.5902930313742527, 0.11119021590440852, 0.60407418124442158, 0.66305796338777667], [-0.079335251589348524, -0.12686225729083841, 0.1200544619975992, -0.17244300649134314, -0.078501635144015994], [-0.20413211238518561, 0.35151609284784113, 0.58044024769890201, 0.28543021825935005, -0.082741747455942399]], [[-15.031311945583326, -1.7126660699063923, -2.5663360296423097, -1.8465255385473973, -1.9961129671588691], [-0.74891286654753819, -1.7382270269064823, 2.0836867261458654, -1.2807021924716198, -10.656288509961687], [-2.896754352445805, -2.6367948163439996, 1.6576942692300962, -0.94490464861358003, -5.2899443413455263], [0.711478754273692, 35.476757963637745, 3.791221664598365, 82.858283491010823, 18.611925316143164]], [[6.0446531378078427, 1.0047741038001115, -1.1110886856548363, -1.8094057291456511, -3.4943976766447027], [3.3745643948641368, -3.573565580213875, -1.5924141711210784, -1.957927489159877, -2.9526634313881206], [1.7754924593375354, -4.7417606551194247, 4.3449105781931259, -1.2149230347649427, -0.94340712325681408], [1.1229000993311367, -3.424222475588945, -42.327103089051697, -5.9017886275912286, 1.2855841759083664]], [[-1.8288670161000338, -1.087103987786221, 2.144228805512415, 1.0324811780960754, -1.919020352623704], [2.428234523536819, -1.1314790594011273, -1.1387316008479806, -1.0757018781733565, 1.1132771612482091], [-5.0640017171491563, 0.97417094775763591, -5.5672829441825593, 2.3974111802672362, 2.230802235648182], [-1.0568709168227242, 32.248102425940914, 1.0976639716439454, -2.0060571803609402, 1.285129306846825]], [[0.49691120505486658, 0.13793852992613415, 1.2684809313771153, -0.66339788012596324, 0.27958960946219852], [-1.1421919909246332, -0.12308148051620353, 0.1876940245213703, -0.12539212749984643, 0.10137043435284473], [-1.5706567835399003, -0.1138738285909076, -0.095705235763495322, 0.5037027909050884, -0.46184302323477322], [-0.10779304789978562, 0.10165471092764847, 0.12442218595046589, 0.4029900675999486, 0.21000546070144432]]], [[[-1.4990199931341768, 0.9204616694692157, -2.2251919429726077, 20.166829034436489, -0.74968967797321373], [-0.77677965443891117, -0.61652721288993118, -6.4168984828223063, 2.0811552799303494, 3.0747251465499525], [7.2092189180230859, 35.994255896166571, -2.3837160312215016, 0.62706355117208623, -1.2346824537411771], [-0.73712358985299453, 3.7298294282684115, 0.69487768548457762, 3.1611685532788614, -3.1067508825699237]], [[-0.72876603149834707, -0.45337338165311708, -0.47562082976786169, -3.323277075499556, 0.8026008303011446], [0.71385719867962238, 32.088652162585291, 0.56761758528444717, -0.66648397489618771, -0.51101065544286461], [0.48745752570670786, -0.41304415898006125, 0.60669495308659416, -0.55945138663639626, -1.3413132540623069], [-0.44085922479181322, -2.3296288007410482, 0.8419358315843134, -4.1305457263626559, -0.71724976052764122]], [[0.45676924636182997, -0.795875380120595, 2.5235249758196017, 0.34142945483880549, 0.34755881871088928], [-1.5213984479286002, -0.53292730404745625, 0.32531587113656174, 0.84391653675668854, -0.56372387865608375], [0.46108877409695354, -0.75765290775318439, 1.5729291509726424, 2.0783454367235352, -2.4966366122037682], [0.48665488464269485, -0.96845865535604003, -0.36273294278327911, -0.37679909036998988, 0.57769585726501693]], [[1.558745390843232, -1.0330627850345586, -1.0078376731328069, 1.6070227194789581, 1.6077649840006212], [-31223.744653498266, -1.6715407873560233, 2.0885123902061022, -6.4626099231542069, -32.668748698791667], [4.4645415514554889, -1.8750022901291035, 3.4553439481295261, 1.4514518153308076, -1.31188924675364], [3.5180811495737712, 9.0615422102273264, 2.2520784547855328, 6.6028315815575054, 1.2000793661206108]], [[1.7890074787288299, -1.4562479934564043, -1.2845409544486668, -16.520233184816913, 0.66631972920407012], [1.2462274967287839, -0.69686244734096159, 0.74074492110878754, 3.217214787205056, -1.8797614903504642], [-1.2232668590046079, 1.0545157464280162, 4.9102859555649818, -0.67176047249397297, 0.89033434202535278], [1.0505414285139858, -1.5556929223001419, 0.80064586993872044, 11.890707578059212, 1.0353384383557769]]], [[[-0.51144657609566557, -18.586477402652328, 3.7656443296166882, 0.61082221162116646, 3.5545947851585646], [0.51422381051907518, 0.52990848362699339, 1.0212037307534052, 0.39204479104014717, -2.0691798373095454], [-0.39474359754021898, 0.73957986495666206, -0.60637971643794986, -0.91482632782271689, -0.87264280538414363], [0.60165909120750372, 0.80965375974983644, -3.5626003593023285, 2.3663016103849754, 2.1519656593944378]], [[36.706402070745305, -0.77683435949565327, -2.1146892147541743, 1.2882932829172258, 1.0781780293678465], [-2.834565020541941, 0.67664762048242111, -1.9013113312962309, 1.7023147289084579, -0.66622266442952605], [-1.4766659726727374, 16.539717934198059, 0.76834896034611466, 1.1278615608245262, 0.67533738715061475], [-0.77747398491897501, 1.0011935961503287, 1.4669236013867741, -0.78685798408904162, 0.8384090257248229]], [[0.59013614613057785, -0.21776747368759336, 0.67758721135234989, 0.25160977156267944, -0.33814891489217885], [-0.69287738243812891, -0.68361899138445703, 0.5741465083071855, -0.91230631348853819, -0.23423636150839564], [-0.31476604740781994, -0.59683935974603131, -0.4194854369556883, -17.317732350406192, 0.34947959659309435], [0.25129090890914763, -0.35679183655449265, 0.30689704476720847, -0.36529742870706283, 0.68651206287185029]], [[0.74203485921533385, -0.1160301127849188, -0.096086903264686871, -0.2783625042434873, 0.11618184068727586], [0.11124507929497385, 0.11754818194849459, -0.042734095404645789, 0.1141714333610802, 0.065308781011472389], [0.06935907162473666, 0.12106150883376786, -0.078743454719458358, -0.048990272724223773, 0.058101412045117824], [-0.37157299768456487, 0.050589835890389998, 0.039225069406662087, -0.041662925729803103, -0.047839377830178581]], [[-5.130755520596348, -0.9860541856372933, -0.43960037809155522, -0.5308686370025153, -0.37836758115973168], [2.02145542814102, 0.50355709869995824, 0.49869121498805313, 0.43735331622161144, -0.68599831817767409], [-0.37847864052299007, 0.7637554184202856, 1.6447229055538928, -2.1285274504430491, 0.574598388532649], [0.40572245370578558, -0.62901893594663205, 0.70240847986536736, -0.71895051395781573, -0.58324887833154815]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[0.24675036172314968, -2.0329249578143171], [-3.2068772028437853, -4.9239674903560289]], [[-2.9451633011799672, 4.2372784785484043], [4.8022874820117565, 0.76376532910105244]], [[3.8779121704133583, 1.2923264166907771], [0.73231643654210998, -3.7293028535776216]], [[3.2740079891185676, -2.0523828087770966], [1.1413934284652711, -0.037568914208128135]], [[-0.60852206492388117, -1.4817563493240216], [4.7723517923399026, 1.52682460323798]], [[-2.5354624121886782, -1.2079654898239376], [-3.4499676930900067, 4.8343149431392227]]]),self.functionspace)
      arg1=numpy.array([[[2.0103703525379171, 1.4313261363678533], [4.8174594290189638, 2.3902575191715494]], [[-3.8139372817138795, -4.6883647005497222], [-2.6958949017481881, -3.5809270909245616]], [[4.6330725717221881, -2.3747930668799078], [1.8406360915730584, 3.5752882824216048]], [[-1.434175186387864, -3.3939372193519199], [-1.2742508528954923, 1.7380025125333329]], [[4.3114017829329612, 0.5662377833347696], [-2.6832260215270312, -4.0623194645810141]], [[-2.7703356867975693, -1.3368196272933641], [4.2681667615478105, -4.1560581204776508]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.12273875876235883, -1.4203086956639293], [-0.6656780923834037, -2.0600154798645502]], [[0.77221073227939685, -0.90378602117953266], [-1.7813333445965014, -0.21328703704600013]], [[0.83700656753837022, -0.54418485328857891], [0.3978605221829874, -1.0430775252203439]], [[-2.2828508122250639, 0.60472032218940208], [-0.89573683695927842, -0.021616144934892667]], [[-0.14114250899388822, -2.6168447124765279], [-1.7785873251273645, -0.37585045109086612]], [[0.91521847849406368, 0.90361142607524747], [-0.80830199142427805, -1.1631971457087373]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[-2.2942669563620677, 1.5204737122656056], [0.1789512399045261, -0.87474453363918681]], [[-0.95096126968534378, 4.262050659351118], [-1.7383267515448608, -1.951405017641652]]], [[[2.6841869596380361, -0.69243651493516634], [2.3753496943393291, 1.066437692645092]], [[-4.2953652047576298, 0.62141283647599899], [4.0159562311339041, 0.38612133679853322]]], [[[0.82927607125968628, 2.5660995054889675], [-4.2386546274989776, 4.9409471127582432]], [[3.7251496564745512, -1.3536227095229494], [-4.3021750671509533, 3.0627660300762809]]], [[[-1.8914333578394382, 2.7648229480681161], [1.8200340213542345, 4.4143735392647283]], [[4.8533820473239331, -4.3456806594594148], [-4.356877497824506, -3.8502406021672906]]], [[[2.6732145969327252, -4.2608031717885773], [0.075679022730751555, 1.2021292126888845]], [[-4.3453867033580416, -2.9436798174552301], [-4.2733317745437684, -4.7562757219699758]]], [[[-3.054008919397476, -0.8034206596867044], [3.0407750816914447, -0.024408497730140866]], [[0.77925700977268164, -2.1280219876941908], [-1.4828080515084983, 4.2710709322527016]]]]),self.functionspace)
      arg1=numpy.array([[[-2.1361329256933423, -0.21460220167904609], [0.25539479494212181, 1.8247078084787898]], [[0.82283126723873146, 1.4654528490797079], [-4.846966872591592, 4.6416688769745047]], [[0.70558602847462648, -0.2743696488727263], [-3.6707246583784436, 0.67982248961755953]], [[2.8984601458945525, -3.3267334973269258], [-1.5275389052971344, 3.9450358816544124]], [[4.9458729118608211, -1.1383743062943088], [3.5533225110616584, -3.5885632408959767]], [[-2.2795554853546274, 4.4941111468138466], [2.492239758896905, -4.8266056418018319]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[1.0740281790363764, -0.711787966927242], [-0.83387420308092319, 4.076120966118669]], [[-3.7234951084294923, 16.688087399420159], [-0.9526603346943846, -1.0694342450742764]]], [[[3.2621353447659662, -0.8415291719028305], [1.6208980697202431, 0.7277188708697151]], [[0.8861965261299527, -0.12820653674980451], [0.86519662164086819, 0.08318588573043835]]], [[[1.175301150807166, 3.6368343503576712], [15.448700849069464, -18.008358916733656]], [[-1.01482677213936, 0.36876171205957936], [-6.3283800298680646, 4.5052437612048823]]], [[[-0.65256490089004993, 0.95389372594419719], [-0.54709342447077769, -1.3269393363825914]], [[-3.1772559314159405, 2.8448903293982553], [-1.1043948974165936, -0.97597099688549149]]], [[[0.54049399258157704, -0.86148658643667109], [-0.066479911143730544, -1.0560052225722782]], [[-1.2229080500941445, -0.82843023910478653], [1.1908196923615653, 1.3253983287145443]]], [[[1.3397387951372317, 0.35244619613271544], [0.67661323504365001, -0.0054312180835682181]], [[0.31267337221102276, -0.8538592565572739], [0.30721549709102558, -0.88490157456871854]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_array_rank4(self):
      arg0=Data(numpy.array([[[2.931122467877163, -4.1181981132885745], [-4.622389076901344, 3.7464097121634676]], [[-1.9235780472025166, -1.5190766182674786], [2.8767119092284279, 1.5971913127195467]], [[3.9712135887983138, -3.1796283355519286], [-4.04445121858907, -3.9295016871455388]], [[3.9999696286828303, -2.5507190396522805], [2.9562060723936421, -3.8019978077321124]], [[-1.1327968590223159, 3.1577499128346531], [0.92532631828107537, 1.3927887015211047]], [[4.0353284544143868, -4.7134814897555124], [-4.4332900220040772, -4.2083819004604397]]]),self.functionspace)
      arg1=numpy.array([[[[-4.5386978762884773, 0.21720999652221273], [-4.396628578201959, -0.20887135590473616]], [[4.52657574965286, -4.698812874832706], [1.7686660806311405, -2.4911912468035933]]], [[[0.29039431757261092, -1.099238416077446], [4.5323436019475594, 1.2025866739690105]], [[4.8805289203475049, -0.42425188176413009], [-4.0289082070031164, -3.5626171080991922]]], [[[-4.0422548065740536, -1.6101284273511851], [-2.7253312644108716, -4.002766277546483]], [[1.8934595861016952, -0.0078725075121983679], [-4.1153799895188978, 0.60934875254413612]]], [[[3.0318836908390718, 0.20812693031600027], [1.1101009250396814, -2.2386657905271101]], [[0.39133103802505076, 3.8672472727488376], [4.8480883516609978, -0.32984679693754337]]], [[[-3.7172860987666012, 2.0255981241768524], [-1.2775233392939245, 4.65118420184141]], [[1.0689206394579411, -3.0664825082451985], [3.9586352173828097, -0.27138437228822099]]], [[[-2.6618239072956795, -2.0107962391783838], [-3.8654607149838274, 4.3617737054121335]], [[-1.2461577419027705, 2.7876766650470808], [0.84282651229230954, 3.5880455190243694]]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.64580691373845978, 13.494417912655392], [0.93667182479461319, 19.716433090839118]], [[-1.021166844994438, 0.98373550937925291], [2.1182119978388334, -1.503862747177851]]], [[[-6.6240209632254263, 1.7499188702544328], [-0.33516360445724541, -1.2631743317543396]], [[0.58942626018157052, -6.7806697692569902], [-0.39643278790598452, -0.44831966620508268]]], [[[-0.98242534892649447, -2.4663955504042239], [1.1666942573453585, 0.79435773039961222]], [[-2.1360113774151861, 513.74371028827034], [0.95483325893434967, -6.4486907878931268]]], [[[1.3193018059264146, 19.218895039722415], [-2.2977361626476469, 1.1393925124713213]], [[7.5542335903455804, 0.76442127019521355], [-0.78422617987758292, 11.526556701570827]]], [[[0.3047376039735491, -0.55924067340981243], [-2.471774734526504, 0.67891310595363985]], [[0.86566418883100271, -0.30175496380398248], [0.35183557590889247, -5.1321625109713676]]], [[[-1.5160012814349317, -2.006831112864639], [1.2193841400287606, -1.0806341199927396]], [[3.5575672909874498, -1.5903171546364412], [-4.9931769339036718, -1.1728897747107587]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[0.237891944132544, -0.2166080112520099, 2.5920477865534401, -2.8229910852579154], [-2.1918722574343641, 3.6636625180134672, -3.0034290993018233, -0.85774815669976512], [0.22369702642310507, 2.7616137715536855, 0.88381553343919084, -3.6140439716730679]], [[-0.96725821998373895, 0.77762893704465963, -2.4772799177740348, -3.0233466659935715], [-2.9990794159473264, -3.0895450759778411, -0.57799319608287547, -3.9827507259706341], [-0.37350349457087795, -4.0644820182377375, -2.0810361667830648, -2.0294386824750079]]], [[[-3.8352000081175577, -2.1922138045962436, 1.7813515208141677, 4.6504351527256986], [-1.0958221944388846, 1.3708774006946909, -4.6509446904648595, 0.7381891813373791], [-2.3688984707625025, 3.8061329895475353, -0.55023281590565531, 2.1848756186274398]], [[-4.17044619635762, -1.6495653635385654, 2.9957529778400618, -2.5795407023686656], [-4.7166197300437886, -2.3895719688876769, -4.3625776182319491, 1.0744536836952658], [3.0974257447902929, -4.1498832059906468, 0.98898281996532944, -3.8202921478680407]]], [[[-1.3383968676163702, -3.3703928120863678, 2.3335748731505026, -2.8959909703543341], [3.8275843069272995, -3.6829758678073943, 2.1330121078951638, -1.3363278021599401], [3.0209427210391429, -4.5675784014347665, -4.3161786055247404, 4.4530872288622341]], [[-0.84980243355592577, -3.0532819622913676, 0.50514276587608187, 4.7014507555806535], [-3.1907693922699911, -3.6959806288079888, -2.7799934015446137, 4.8741578594112109], [-3.4967799976121783, 2.4501320498024999, 3.5761713699047473, 4.8079541367803138]]]]),self.functionspace)
      arg1=numpy.array([[[[3.6588843067064118, 2.1235224378933548, 1.9697647589241711, -1.9885160710962402], [3.4421018585791057, -0.35602745452951368, -3.3065982525737159, 1.3560757799714303], [-4.1127333690044034, -1.3064232579918555, 3.4898435352131667, -0.3642067651087979]], [[-2.3137966474435112, -4.7657062052217034, 4.0750094698090322, -0.33394018797632619], [0.51868722227087183, 0.23744771644460894, 1.8398770139533855, 2.4278117543780509], [4.8823194445792737, -2.2397176354582751, -2.5959670928595422, -3.9447523143901089]]], [[[1.0132621287912311, -1.2689742723011066, -0.36562527501363462, 0.64264410067910305], [0.10627113741666605, 3.0496014104854634, 3.0160170102356965, 0.97011733484358853], [2.7304197234377625, 0.54943061893753065, 2.1824297284934921, -0.64175446024203886]], [[-0.53965823151979286, -0.78666187116486075, 2.3985061868779276, 1.0594075090663768], [-2.7122708045144628, 3.6494278764687031, -3.3412712446910806, -0.59633543117264232], [-0.40557312273548529, 1.1791121051435862, 3.4864020249966305, -0.49575767026732009]]], [[[-1.9791849323797228, -3.151307521072908, 1.5800342448630911, -0.18936569461213892], [2.8254382324423197, -0.045402231057379616, 0.95834609933991466, -2.9038832903173937], [-2.0493516683338662, 3.7200061921648082, 3.282090922608095, 1.5659279297633262]], [[-4.9189876151874987, -3.7615791223770598, -3.9649798735594688, -1.8886009854361596], [-3.4352861475143834, -2.079879916967923, -4.0475604309180717, 4.0932406629673572], [4.5019537461673416, 4.150369450203689, -3.0624268758616315, 0.11239093504474695]]]])
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.065017618539211272, -0.10200410760288287, 1.3159174336986017, 1.4196471058449336], [-0.63678308995166277, -10.290393258730443, 0.90831388329806362, -0.63252228921737408], [-0.054391327215373771, -2.1138737041459668, 0.25325362713867505, 9.9230555769425646]], [[0.41803942496521984, -0.16317181621322455, -0.60792003948155926, 9.0535574179167195], [-5.7820576393168404, -13.011475209105919, -0.31414773471240254, -1.6404693316063637], [-0.076501240611277541, 1.8147296578330028, 0.80164196707545154, 0.51446542665601447]]], [[[-3.7850028133319769, 1.7275478726774907, -4.8720688709163742, 7.2364083756645892], [-10.311569265908991, 0.44952674666964498, -1.5420817172716796, 0.76092772989815394], [-0.86759498930806034, 6.92741332273709, -0.25211937352295649, -3.4045351516581746]], [[7.7279395602152734, 2.0969179059053009, 1.2490078175447838, -2.4348899552750343], [1.7389929214270086, -0.65477988599130754, 1.3056640119127156, -1.8017605990347498], [-7.6371573241810538, -3.5194984326662437, 0.28366861104214891, 7.7059667998844699]]], [[[0.67623638686816145, 1.0695220284115177, 1.4769141116638933, 15.293113022852092], [1.3546869519135518, 81.118830111075965, 2.2257221158038107, 0.46018647051544548], [-1.4740967925213078, -1.2278416124831031, -1.3150697854814191, 2.8437370227729906]], [[0.17275962048209662, 0.81170217692028845, -0.12740109205714623, -2.489382771604816], [0.92882201227361705, 1.777016354961511, 0.68683184574814438, 1.1907821359024942], [-0.77672499425146246, 0.59034071043536951, -1.1677573097638685, 42.778842749783074]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_constData_rank0(self):
      arg0=Data(3.47999815809,self.functionspace)
      arg1=Data(-0.444898270994,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(-7.82200872643,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([1.0568443980959508, -4.3799555039373903]),self.functionspace)
      arg1=Data(2.92212859646,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([0.36166936642544306, -1.4988921121544785]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_constData_rank1(self):
      arg0=Data(1.60305152811,self.functionspace)
      arg1=Data(numpy.array([3.0154257726763198, 1.8260402341188353]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([0.53161697516542517, 0.87788401271432592]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[4.6868344174265957, 2.976097645716961, 0.80792083978395102, 4.5853222901196116, 2.2737026296443199], [-3.3897782267273446, -3.5319834204334244, 2.7964534385648125, -0.097791255012900535, 4.063897471968021], [1.1210330726469007, 2.1648813971309808, 4.4877946942301321, 3.7339054067223891, -3.2110409526815742], [0.33413411126235548, 4.5200565210241503, -4.4385033282198574, -4.8370855971455482, 4.9695019979078019]]),self.functionspace)
      arg1=Data(-4.90132316945,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.95623860239252034, -0.60720290069189931, -0.16483729226836857, -0.93552743444918596, -0.46389567695055761], [0.69160471765207587, 0.72061835107101002, -0.57055071495724652, 0.019952011249224195, -0.82914293374906922], [-0.22872049728017815, -0.44169325757293371, -0.91562921665823949, -0.76181579496690488, 0.65513757033970688], [-0.068172226092983332, -0.92221148550252274, 0.9055724698765677, 0.98689383048593504, -1.0139102903651107]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_constData_rank2(self):
      arg0=Data(1.5164145294,self.functionspace)
      arg1=Data(numpy.array([[1.5647179651007974, -3.1371028185915995, -3.3607363221278752, 3.7459228297535248, -1.7258612339000869], [-2.0547052562239019, -2.8594035884754754, -2.9077878804109081, -2.5491900961445504, -3.751396969328411], [3.3479749033382884, 2.9368473720961035, 4.3435037286737597, 1.1324246447352913, 4.6679726258920891], [-2.9902929140804444, -3.7311465739834917, -0.39246413515595968, -4.7926506240433557, -2.8387377524659785]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[0.96912962157857685, -0.48338056387840361, -0.45121496721147331, 0.40481734363308119, -0.87864221039866319], [-0.73802046536940991, -0.53032546210233678, -0.52150108321555, -0.59486129798197251, -0.40422662325357978], [0.45293485559981628, 0.51634093885956101, 0.34912241916236525, 1.3390864782440377, 0.32485506041405648], [-0.50711237091689887, -0.40642051962498055, -3.8638295669812859, -0.31640414633771113, -0.53418619880545326]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-4.9570081830715926, -3.4532998342448007], [3.283841996235088, 4.6059847729940113]], [[-4.5985766160167776, -1.9912050839725048], [1.4710700577808975, -1.9772405577527596]], [[-1.1969699297043004, -0.15405058357916879], [1.8800804804933913, -1.5149712327672105]], [[3.0813297281636736, -0.29070005971011081], [-1.1965981932445824, 2.371692375602656]], [[1.2648541020502178, 1.2760988489211531], [-3.2324771468973621, 0.4909891669041988]], [[-4.5310216643273957, 3.8589297395926501], [-3.4769370670977393, -0.061777269015776959]]]),self.functionspace)
      arg1=Data(-2.01531290701,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[2.4596717293045582, 1.7135303516567499], [-1.6294452265030355, -2.2854936109305628]], [[2.2818176770587209, 0.98803767744838178], [-0.72994622952347021, 0.98110846751220693]], [[0.5939375099229709, 0.076440032236896133], [-0.9328975535040257, 0.75173003035838737]], [[-1.5289584646869696, 0.14424561997253812], [0.59375305397152778, -1.1768357992223972]], [[-0.6276216947013441, -0.63320134778242632], [1.6039579440273979, -0.24362924744673817]], [[2.2482968518536377, -1.9148042600097432], [1.7252591669542336, 0.03065393408682909]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_constData_rank3(self):
      arg0=Data(-3.60190733122,self.functionspace)
      arg1=Data(numpy.array([[[-1.5620280792777508, -2.5040712972610022], [-2.7463151353014439, 3.2049044592440001]], [[3.8086541979508262, -2.1119100607114447], [1.8792323050506434, -3.7547448009631976]], [[-1.1440580286986179, -1.3040770529840753], [-4.23407251868825, 3.7478131086507389]], [[2.827314428900694, -0.32385766989235343], [-0.11167119966822714, 3.2793725094755803]], [[-0.19941249675049644, -1.9943248454151119], [4.139024778774381, -3.3945050004861543]], [[4.6736068955633332, 3.4722906690045523], [2.5618958851476954, -1.5462158944562976]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[2.3059171464345054, 1.4384204376123606], [1.3115418856777943, -1.1238735435091163]], [[-0.945716556036161, 1.7055211764109588], [-1.916690832494844, 0.95929484483066119]], [[3.1483606957559158, 2.7620356657427565], [0.85069571088371976, -0.96106908930564461]], [[-1.2739677251317261, 11.121883673207293], [32.254577204506766, -1.0983526027650699]], [[18.062595824801313, 1.8060785531000081], [-0.87023091760404825, 1.0610994329667696]], [[-0.77069111966563175, -1.0373288628660022], [-1.405953829779101, 2.3294983217626775]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[4.8165353204105461, -2.9259346603589775, -1.3991795151986794, -1.0951400849038584], [2.1585263752350849, 2.4502552310165058, 4.7359490254822525, -1.7216549370060177], [1.4874970204874369, -4.0677731440690295, 4.3260217902130176, 0.91679089933901814]], [[3.1792086064475988, -1.5954180114096541, 3.4733943326613321, 4.5243398542675273], [4.2373720265434329, 4.2944757015168413, 3.4632642500703952, -1.1230366786841719], [-0.81152323962129014, -4.5344536867465326, -1.4580295734569182, 3.0838950744717142]]], [[[-2.0355451215541409, -0.51887400497391667, -3.8169792240816101, -4.6415923040456297], [-2.2456100390403178, -3.2033947113519767, 3.2791414352361574, -0.10223362617477783], [0.3378915443625079, 4.9955323088802821, -2.0187876331610424, -0.026817663725929641]], [[-2.5509720697758329, 0.53245875588354608, -2.4224717715853119, 2.0479576561671635], [2.9378984348224169, 4.2629077547947301, 2.3688192660068275, -3.1560912282080054], [2.9707004432319204, 3.8208130114233043, -2.7252845096146796, 0.43513293821076182]]], [[[-4.1553584479576937, -1.1480750748899471, -1.2286156228842779, 1.3543097107737569], [-1.2670710176680542, 1.5379068815178165, 1.0058224934192017, 2.9753296518791297], [4.899880783675993, 2.0833644537318809, 1.8902421697175429, -3.9903292791564771]], [[1.9410796835349631, 4.8195908641701806, 2.5317867730503298, 2.7941850005576594], [1.2115206860073808, 0.87373947779799011, 1.0574210774984403, 4.0311740324240048], [-3.6632537335259716, 1.6477729330070643, 1.8406217898793145, -3.4006295668226172]]]]),self.functionspace)
      arg1=Data(-4.53902813685,self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-1.0611380179169607, 0.64461699115770543, 0.30825530774710225, 0.24127193132204397], [-0.47554813721229089, -0.53981935276453008, -1.0433839321306257, 0.37930034472077639], [-0.32771266791891446, 0.89617711576655301, -0.95307225683148178, -0.20197955855247243]], [[-0.70041614869810498, 0.35148890099532309, -0.76522864100788712, -0.99676400274605215], [-0.93354169632452966, -0.94612229138842974, -0.76299686753446727, 0.24741787114422994], [0.17878788479680158, 0.99899219613328571, 0.32122065109459425, -0.67941748354289244]]], [[[0.44845395538018995, 0.11431389921578028, 0.84092433644334619, 1.0225960633200082], [0.49473366794257423, 0.70574462522989867, -0.72243249796396836, 0.022523241339860126], [-0.074441385727276432, -1.1005731091023172, 0.44476208833556014, 0.0059082391466551004]], [[0.56200842842624965, -0.11730677577438321, 0.53369833773807318, -0.45118857923330968], [-0.64725274799875721, -0.9391675103713657, -0.52187807490628757, 0.69532312491799331], [-0.65447940697078966, -0.84176896380103239, 0.60041145977640642, -0.095864780982026346]]], [[[0.91547316356531594, 0.25293411723278791, 0.27067812444446249, -0.29836997479210059], [0.27915028932741248, -0.33881853893590241, -0.22159424068173808, -0.65549927477236225], [-1.079499980159216, -0.4589891031554324, -0.41644204722372097, 0.87911534338321873]], [[-0.42764213505852694, -1.061811189280413, -0.55778168733827738, -0.61559102880871797], [-0.26691191362543332, -0.19249483621918742, -0.23296200103120393, -0.88811391136650453], [0.80705684632856001, -0.36302329118169846, -0.40551010797546144, 0.74919772785978123]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_constData_rank4(self):
      arg0=Data(-2.14493594914,self.functionspace)
      arg1=Data(numpy.array([[[[-3.0885635216139038, 2.1953506211752458, 2.8953139468244071, 2.244082900055723], [2.5329480831449267, 0.026028214718101239, -1.7444148774427628, -3.7906040672753569], [0.35681737296255811, 3.0377535562994318, 3.0965325294500037, -2.8889626176662278]], [[-2.3284343773649372, -4.7601082736997782, -3.9385395001799148, -4.4088357647228706], [3.089083139398344, -4.2284740504327338, 3.4084250657028488, -3.1536090016250826], [1.3899077807477624, -1.0842707558900209, 1.2857839844012373, -4.2718152509486043]]], [[[-4.0833769596507272, -0.88886345236166342, 1.7976154686788846, -2.9795916087994803], [3.7047379842318424, -2.315848947657658, 4.1798036140625676, 3.6074510008070035], [-2.1566423021975165, 4.933540628466254, -1.7279630212791295, -1.0887507460994295]], [[-2.3811995069295544, -4.4389615514838123, -2.9878036559809686, 1.6098564682140815], [-4.612166642994973, 0.12062316539153528, -1.1673694992743222, 3.9305068407934822], [-3.9904752095272054, -0.84008657214058235, 1.1734361015543904, -0.85271888680796071]]], [[[-3.401496910053643, -3.8657462334350279, -4.7397827516520818, -0.86896877355742586], [-4.287375154801234, 0.47298865741905871, 4.5523006218350464, -0.53954083876763903], [1.5585133611167015, -4.5972877649039141, 4.8529633702838115, 0.56198911611838476]], [[-2.6192999922381266, 3.1723612545352253, 2.2136214606941573, -3.0267095453687345], [-4.8440981973180559, 2.9418810167960627, -0.63641433523184965, 0.22641559813485923], [-2.0736336028462587, 2.5837228752949937, -1.7228321017220538, -4.2900578964400271]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.69447687707405525, -0.97703570830385766, -0.74083017887847546, -0.95581849898771976], [-0.84681402015634766, -82.408108752982642, 1.2296019581532427, 0.56585597204749105], [-6.0112990892972711, -0.70609281147487646, -0.69268962258124478, 0.74245887988263415]], [[0.9211923556817907, 0.45060654628095798, 0.54460186295891877, 0.4865084715329821], [-0.69436005841948178, 0.50726004784533596, -0.62930412369003175, 0.68015278622998787], [-1.5432217725850177, 1.9782290885219025, -1.6681930830971752, 0.50211346304336624]]], [[[0.52528482438191004, 2.4131219969009732, -1.1932117777733859, 0.71987581882051788], [-0.57897102528291511, 0.92619855509347615, -0.51316668130503285, -0.59458491568019578], [0.9945719542594903, -0.43476604545609893, 1.2413089416389682, 1.9700890739406991]], [[0.90077960410006341, 0.48320669693980162, 0.71789722354798968, -1.3323771351583829], [0.46506037512608756, -17.782122879738246, 1.8374096209200579, -0.5457148495134676], [0.53751391413590666, 2.5532320361577341, -1.8279103108332733, 2.5154080463311321]]], [[[0.63058588787647718, 0.55485689427402518, 0.45253887393628728, 2.4683694217851224], [0.50029117389768307, -4.5348570531045498, -0.47117625291421106, 3.9754839578677705], [-1.376270491257378, 0.4665655183714828, -0.44198478032399197, -3.8166859243648283]], [[0.81889663478465324, -0.67613231187616407, -0.96897142859405216, 0.70866923865136167], [0.44279365565355755, -0.72910356907339247, 3.3703451201391572, -9.473446029357433], [1.034385219351742, -0.83017260467238563, 1.2450057942334039, 0.49997832218422494]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([-2.0091676487239498, -4.1597968201567328]),self.functionspace)
      arg1=Data(numpy.array([-0.27875252091714486, 0.85734050881001078]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([7.2077111342829641, -4.8519774551776793]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[4.630332459274042, 2.308576541848848], [-3.0422969101423369, -3.9140018394468012]]),self.functionspace)
      arg1=Data(numpy.array([-2.9180160560700452, -2.7579160201554211]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-1.5868084240462088, -0.79114593528248633], [1.1031144124435266, 1.4191881880530319]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_constData_rank2(self):
      arg0=Data(numpy.array([0.48956578374837267, -4.0643578089544743]),self.functionspace)
      arg1=Data(numpy.array([[-1.0566255731485552, 1.6681153539154456], [3.7560154222506448, 1.1491468641552967]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.46332948604447854, 0.2934843700102937], [-1.0820929501186838, -3.5368480180660473]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_constData_rank1(self):
      arg0=Data(numpy.array([[[-4.5510046161128432, -3.5734732250045029, -0.17356228355855308, -2.4065728718774473, 0.31226796955436864], [-4.248245486423353, 1.4717841710461386, 4.3180086877402104, -1.1424296111527408, 4.8938037153930871], [-4.4799262954864183, -2.440104034022117, 4.4143500975599395, 3.765558690240006, 1.6769409589306434], [-0.15736547553655278, 1.7258453032938856, 4.4258121655522054, -1.968786854397202, 3.5152991752712062]], [[1.9659968544290605, 0.23040243587557097, -3.8540820015819008, -0.9161786969051402, 1.7059122332266412], [1.3396997532845569, -1.2994053907887002, -0.0053897188813865071, -4.5011489754106631, -4.7052970185510921], [3.7019377241329714, 4.8546267180088201, 3.5535883625244189, -2.4715425875900565, -1.3277670819467313], [-3.8664727519003783, 0.45926255473713518, 2.0115041207896098, -0.63934299009493323, -2.7576157913828023]]]),self.functionspace)
      arg1=Data(numpy.array([-4.6140347372545598, -3.3798741251176034]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.98633947841077141, 0.77447904675524593, 0.037616163172153751, 0.52157666964367178, -0.06767785405538021], [0.92072247573739363, -0.31897986358071478, -0.93584225815983113, 0.2475988318701971, -1.0606343458751235], [0.97093467010871737, 0.52884388024222506, -0.95672233715920474, -0.81610974010147708, -0.36344350539685272], [0.034105828087065568, -0.37404254661524222, -0.95920651177967708, 0.42669528222248893, -0.76187098178694623]], [[-0.58167753639661157, -0.068168939832205747, 1.1403034133550158, 0.27106888096705717, -0.50472655787655518], [-0.39637563521332081, 0.38445378220808357, 0.0015946507715576452, 1.3317504761376415, 1.3921515548711063], [-1.0952886371187454, -1.4363335847129817, -1.0513966588624868, 0.73125285028301423, 0.39284512759791945], [1.1439694523433896, -0.13588155586153819, -0.59514172609005644, 0.18916177538791837, 0.81589304491830761]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_constData_rank3(self):
      arg0=Data(numpy.array([-0.32355451658856182, 4.1125233953067237]),self.functionspace)
      arg1=Data(numpy.array([[[-4.1075031701656037, -3.9868937425291517, -0.53604619062542636, 2.7843314405489448, -2.4107912154644664], [-0.60442920575642933, -1.2922108208035454, -1.4845035849977029, 3.9332682158446239, 3.9013183597369014], [-1.2027956938676274, -4.1059139128552333, 1.7081799634539641, 4.3907343759029853, 0.28552099592117486], [-4.7724896219127739, 1.4845517263502881, -1.2705717520749049, 3.3019060194804126, -4.9861000466675689]], [[0.86995844639956577, 4.3245519611834347, -3.7307428860538261, -0.63075080691431218, -0.88795999036702877], [2.7626170475712399, -3.2340961904685841, -3.2800896141329927, 4.8478573789244965, 3.6204784175439428], [3.4096457282142687, -0.084723863320120607, -3.719451591736529, 3.4167692634903428, -2.9544171687357021], [-0.27176695216322155, -3.7541106374466491, -3.9174041761391196, -4.9079175949756335, -0.81146546320077118]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.078771580491687601, 0.081154537212047614, 0.60359447048967541, -0.11620546026832619, 0.13421092399584897], [0.5353058943993958, 0.25038833554060741, 0.21795468859649975, -0.082260984716264057, -0.082934661248814823], [0.26900205765466462, 0.07880207024690479, -0.18941477099071577, -0.073690296175573267, -1.1332074390700397], [0.067795750692252715, -0.21794762071646223, 0.25465269164073712, -0.097990225851272511, 0.064891300527515003]], [[4.7272641725899982, 0.95097097507907191, -1.1023336426318899, -6.5200446043431093, -4.6314287129162839], [1.488633178066521, -1.2716144335555046, -1.2537838532176089, 0.84831773582808911, 1.1359060657228206], [1.206143899724319, -48.540319505591562, -1.1056800428438103, 1.2036292410057696, -1.3919914353417515], [-15.132536765679911, -1.0954720817988062, -1.0498082940627045, -0.83793652108519989, -5.0680202446141704]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_constData_rank1(self):
      arg0=Data(numpy.array([[[[-0.078197371537694238, -2.4205486735050128], [-1.6563021797613011, 2.4497807814017127]], [[-2.9180198497793697, 2.8624133122504798], [0.9978299518384901, -1.8939710353953556]], [[-3.2189952977312988, 2.3809239129974449], [4.0657320163666881, -1.8223773291424736]], [[0.71617085499183997, 2.5619333998342331], [0.81348458998474449, 2.776332658292918]], [[0.17059529770815729, 4.5837191696779627], [4.1225893349585121, 3.4309546424918089]], [[0.85527784840969989, 3.917822385017848], [1.1292621196843688, 3.3049005135953671]]], [[[4.2704485123743403, -3.8464907685242888], [0.15790744333196649, -4.6302658739997096]], [[-2.5662571775089038, 4.6549249681220815], [-4.5579492559143979, -2.2304257752858536]], [[-1.0952412672874079, 1.3871915002133859], [1.9872940269198933, -2.9685291662793389]], [[4.9395287256117086, 3.3782499374524129], [2.8173796607675818, -4.8581476869308506]], [[0.22450286326500724, -0.62819256647006938], [1.26186721591076, 2.995363376963267]], [[4.2352415490061546, -1.0992479019114221], [0.96470966581521456, -0.73646813672180844]]]]),self.functionspace)
      arg1=Data(numpy.array([-0.96274808575979876, -0.69239542682436728]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.081223087009288666, 2.5142077240223446], [1.7203900005203863, -2.544570919056516]], [[3.0309277088579973, -2.9731695700973213], [-1.036439299747872, 1.9672550518764602]], [[3.343548894403539, -2.4730497502037876], [-4.2230486629926887, 1.8928911478481489]], [[-0.74388187895137636, -2.6610630939996733], [-0.8449610048746492, -2.8837581703440542]], [[-0.17719619517448726, -4.761078456012199], [-4.2821059796810435, -3.5637096487022424]], [[-0.88837138298199414, -4.0694159177952667], [-1.172957013768724, -3.432778067782027]]], [[[-6.1676440180440135, 5.555338206328142], [-0.22805962779996988, 6.6873143504661261]], [[3.7063462265759006, -6.7229285286756344], [6.5828702491857527, 3.2213178898589439]], [[1.5818147042228028, -2.0034671612082446], [-2.8701720865409319, 4.2873321389402168]], [[-7.133970754640278, -4.8790760403293918], [-4.0690327399898285, 7.0164352604298266]], [[-0.3242408233322352, 0.90727428595425497], [-1.8224661328256355, -4.3260877540762115]], [[-6.1167959592553229, 1.5876013320207223], [-1.3932929485681342, 1.0636525144303424]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_constData_rank4(self):
      arg0=Data(numpy.array([-0.34022088051528421, -3.0059792528249787]),self.functionspace)
      arg1=Data(numpy.array([[[[-0.29225966318842467, -2.1917241163300396], [-3.9692154361155705, 3.3940419631339349]], [[4.086952703796193, 3.4595489940973394], [-4.1234023462882323, -0.42796158571765464]], [[4.5758119972036955, 2.2179226625705395], [2.6572365562890727, -4.5653316417421683]], [[1.2117753234236117, 4.3408615573211637], [-1.9779508947791777, -3.3207719768204038]], [[-4.3312992641899362, -2.3139188806955402], [2.8318795683878273, -0.68569328258268225]], [[3.7940021710384269, -1.9166508584534627], [0.29878078018802334, 3.1372872166481596]]], [[[4.3565667239300758, 2.9210788155218737], [-4.0241070539109085, 0.21283419459651043]], [[4.3988234190234525, 3.1357183310258758], [-3.1730128731374476, -0.10821316769319544]], [[-2.5080939524547139, 1.8569249106178534], [4.5613356234084712, -3.3636984221032984]], [[-0.11958810314306145, -4.7574138377862454], [-4.9878725449954517, 1.2474351143955431]], [[-1.0376166119573926, 3.1033740938678864], [-0.16984370491346379, 0.54639023246190277]], [[-4.8734747346634837, 4.808037599982347], [-3.2236219840445193, 4.0013099454378036]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[1.1641048128353524, 0.15522979282856614], [0.085714894036650641, -0.10024062289469651]], [[-0.083245612360345597, -0.0983425530598835], [0.082509746064810097, 0.79497995116725995]], [[-0.074352023361797887, -0.1533961874581205], [-0.12803560138823886, 0.074522708800505252]], [[-0.28076234425542101, -0.078376349031790277], [0.17200673758549861, 0.10245234628878111]], [[0.078549382012954541, 0.14703232829537105], [-0.12013960067834735, 0.49617064824353108]], [[-0.089673348927516564, 0.17750801039987374], [-1.1386973429187197, -0.10844428865482458]]], [[[-0.68998811295911311, -1.029064754039489], [0.74699286389599351, -14.123572852208701]], [[-0.68335983659292054, -0.95862540429182874], [0.94735803887637315, 27.778313091688542]], [[1.1985114233392158, -1.6187941879808168], [-0.65901295168864427, 0.89365301986417667]], [[25.136106132806294, 0.63185153852912301], [0.60265759112890882, -2.4097279434702745]], [[2.8970037855835837, -0.96861646772287135], [17.6985025989426, -5.5015245043469392]], [[0.61680411133444468, -0.62519878231318637], [0.9324850332027842, -0.75124878947513751]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[1.6614925957632245, -2.4335880526451192, -3.6253501258809671, 1.2250293628411315, 2.7386244842718286], [2.0502877834900302, -2.9727256976379834, 3.6786210579386278, -3.8731462576630946, -1.1028470878832328], [0.53929851092005077, 3.8877503855040416, 4.9484505164928212, 4.1529254372802065, -2.5161088672848688], [-2.7830215575478325, -0.35971285031583111, 0.97159850021323813, -4.7919740451251682, -2.6590219889721967]]),self.functionspace)
      arg1=Data(numpy.array([[-1.7373564182667267, 2.4683557301262704, -1.7836985274942463, 4.3584613165389055, -1.9278176737119668], [0.2381970103312554, -2.7835492037952547, -2.9533166306045744, -3.6984855784983175, 1.5517917927475953], [0.27228269053108001, -2.7006827914713813, -2.3128129417462473, -4.6377918808327667, 3.8286718360249221], [0.62000847795106218, -4.091455372540449, -1.8414184011561332, 3.196433855001672, 0.54158523040701922]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[-0.95633375989758751, -0.98591464064242773, 2.0324903956577725, 0.28106922922375244, -1.4205827250243395], [8.6075294590756606, 1.0679623315387399, -1.2455897954922552, 1.0472249182693025, -0.71069269281965786], [1.9806566104814214, -1.4395435101750413, -2.1395809523430733, -0.89545316909186168, -0.65717537962125061], [-4.4886830688910333, 0.087918067670986172, -0.52763592435223883, -1.4991625863387879, -4.909701815490525]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_constData_rank2(self):
      arg0=Data(numpy.array([[[-2.2466353418190907, 1.6352666067423227], [2.2034679434434619, -2.3668514826040985], [-3.3346352764554652, 2.9520473432906726], [-3.9457896831765868, -4.4865963847179655], [-2.7835147047019779, -3.7788351133191025]], [[-0.85632939082277115, -1.1578468143886411], [4.6278579392566215, 1.5843855537482598], [3.1647697350864199, 2.4395116782970669], [2.2887087699301469, 3.2752557063643284], [-2.0181355164097079, -4.7442482816520339]], [[0.7601963399420919, 2.1678354290289494], [0.76130777122432924, 1.2878764848832569], [1.1277447212542615, -2.1489010232923023], [3.1050715611059871, 0.49621277251245566], [1.6609161884410586, -2.3904779168344614]], [[-0.67982712214514862, 2.6513352419655867], [-3.9576823054228303, -1.459743383148191], [-1.4300476152253028, 1.0817853542699618], [4.5545132242962065, -0.11496549393967648], [-4.7760924728135468, -4.8394448173117341]]]),self.functionspace)
      arg1=Data(numpy.array([[3.9692310079379407, 1.5279429836314931, 3.8004244677568018, -4.3151526386107042, -0.64772341952924073], [3.8937318899394917, -4.372839111979018, 0.070536563699353572, -0.05306911341419962, 0.35496428273179781], [4.5664012137875627, 4.1841962869085219, -4.4373080195397847, 2.6768674988047927, 0.26761893867722186], [1.5733935681267299, -3.4937991267543156, 2.779976027468579, 3.4980567048668938, -3.1065631050775302]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-0.56601274587599337, 0.41198574823989942], [1.442113983995944, -1.5490443740111], [-0.8774375874976228, 0.77676779747529534], [0.91440326997261523, 1.0397306330657305], [4.297381599579972, 5.8340257575764731]], [[-0.21992510399478954, -0.29736172061056676], [-1.0583188223365001, -0.36232422761861316], [44.867081256971176, 34.585065537002109], [-43.126945650419714, -61.716797128326874], [-5.6854608043327026, -13.36542438901288]], [[0.16647602879194962, 0.47473608374215914], [0.18194838841722186, 0.30779542750247019], [-0.25415065086494165, 0.48428033704885232], [1.1599646088169793, 0.18537068896163597], [6.2062729814660385, -8.9323944286231658]], [[-0.43207696784635102, 1.6851061906413192], [1.1327732825611685, 0.41780976243596168], [-0.51441005285484109, 0.38913477799124252], [1.3020124053333528, -0.03286553181934513], [1.5374200720427182, 1.5578131374192563]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_constData_rank3(self):
      arg0=Data(numpy.array([[4.9817805567109872, -1.4155652700399282, 4.3308523504900549, 2.5159206055992023, -4.0322516070367342], [3.8481068335040209, -0.06912498830377789, 0.47906766302386572, 0.77364182552728344, 2.6165570324302845], [-4.2833893338356415, 4.7455536090381916, -4.4789266743747902, -4.2710525413386966, 0.15032015027942691], [0.15368800127373827, 0.87083927145838924, 1.3206389482378045, -1.8694909532413586, 4.3674697068548962]]),self.functionspace)
      arg1=Data(numpy.array([[[-0.75373856190371136, -3.8557950249878381], [-1.753733341398076, 4.4112053835764424], [4.7277718960529587, 1.8917307183977448], [4.2760813984243704, -1.9380111906100659], [0.1667278102104337, -3.8259227232195481]], [[-0.55511668382871981, 0.94764138905499617], [-2.9218582635490273, -4.0632922742099273], [0.88285915405736493, -0.83242439803324331], [-1.1228375209077681, -4.426107331604082], [2.433485757441578, -2.2067655847903178]], [[-0.99944288750784871, 1.5302026453724871], [-2.5553558539301671, -0.21056982642036015], [-1.0406044115765525, 3.932605934877266], [-4.427401848436797, 3.4974478332902379], [-0.80995623232029423, 2.0180776086716747]], [[2.0247157251861729, -3.1982851565462034], [4.2575997565224846, -2.2946705947443649], [1.8126420129587117, -4.2899721285291461], [-4.7773595986907216, -4.3739077102942865], [-2.6491329934773122, 2.2276273860156373]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[-6.6094277359626457, -1.2920242192404148], [0.80717246837049905, -0.3209021450940106], [0.91604511505847452, 2.2893598482971158], [0.58837060644501682, -1.2981971506610426], [-24.18463723566855, 1.0539291822505914]], [[-6.9320684202158604, 4.0607205193321256], [0.023657885519681371, 0.017012064020725325], [0.54263203912221958, -0.57550891607183996], [-0.6890060325930556, -0.1747905704868899], [1.0752300581291163, -1.1856977698330855]], [[4.2857769937374268, -2.7992301194806553], [-1.8571008815619456, -22.536721854747849], [4.3041588374481883, -1.1389207941361088], [0.96468599136685473, -1.2211912071096389], [-0.18559046067069876, 0.0744868034972994]], [[0.075905965149555318, -0.048053251586767341], [0.204537608337725, -0.37950513396255209], [0.72857130023273153, -0.30784324668575341], [0.39132305505193904, 0.42741892995167358], [-1.648641165848777, 1.9605925722912789]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[-3.0773266614140162, 0.9984904171465816, -4.3859628088913336, 1.0093752351131542, -0.62582896985097225], [1.9533718352093921, 2.555444973495832, 4.0116152636527023, -3.0568807014157215, 2.5273057225868918], [3.8415409094297246, 1.7450636826258323, -3.7893320414460452, 0.61097517562511694, 4.679288887375785], [0.28062990383703124, -2.8064740668656665, 0.89521061766793153, 1.4696064217439737, -2.5185832538577699]], [[-3.3957305509661073, -1.3554224896758971, -0.53742522434193507, -0.82994815521273058, 4.0834886851598888], [4.2261533526697512, -2.4832355233024259, 2.2890747739462203, -1.5431958357891862, -4.5597624385516822], [4.5903991642639355, 0.58006634548257718, -1.2177998623130826, -4.0471119313213855, 0.9542166331692945], [-2.0733090257804818, -4.4811389220719047, 2.4629308437716873, 3.9127713781280669, 1.7733424153112489]], [[-3.270499905503911, -0.39290158881396753, -1.045475639583997, 3.7778942134582909, 0.30446612180702015], [-4.1870196668561821, -0.60924008530906448, 1.2909812882712242, -0.92229605978106566, 0.1763811836107676], [-4.1406164552572431, 3.0049652824407183, -3.3006800644998116, 3.5521758600483402, -1.3136519209149586], [-3.5702928490622865, 4.5007488228712766, 3.1558264743811968, -4.2112496154108223, 3.1861863914678494]], [[4.629025328618674, -2.618959010709867, -1.1775480844100841, -2.7511047283706827, 2.3787878370908135], [4.6161129868756916, 0.15406164824798729, 1.3847966187802729, -0.27032730707044283, -1.3910939484097948], [-2.4534949471467637, 4.7567970974430516, -4.4760518941364467, 4.2004239376328787, 3.2762909386037791], [-4.7226218475541053, 2.1613355870882476, 4.7408086928944915, -3.2036592579586323, 4.2308871170517666]], [[0.55326577149439249, -4.6571192474765244, -3.2194556007146269, 2.2523056351245803, 0.37322582456747], [1.5352038110987571, 1.0271851906099174, -4.1449718510791644, 0.16577465437090044, -3.0990638378652791], [-0.72026225249142861, -1.2582666877088466, 0.64688013021870461, 0.71350649147887601, -2.0976696942413642], [4.5899363817199443, -2.5677059309973393, -4.7797301997100048, 1.4911614131107065, -3.4363100327080898]]], [[[-3.4827527595667407, -4.450808509113056, 0.010641533579024198, 0.89358246246317741, -3.9198411384705132], [2.8957622134799212, 1.1094894628843299, -1.2650024435864102, 2.3036370454822377, -1.4995550254477941], [1.9245980379449517, -4.1693755019889007, 2.1022710966879101, -3.3044090869537834, -3.3619751017627242], [-3.3402121519027892, 3.0724095318566942, -2.0692751355646299, -3.0054736794515016, -0.60729313071098012]], [[1.6691275516868682, -0.53328858572130855, 2.0402033560444686, -3.1075230936069618, -4.7220399991203488], [-1.1039022211020555, -1.9062644560260535, -1.1228244387906217, 4.8588056145271441, -4.7322143649711013], [-2.3111221662656201, -4.9683425013646065, 0.029055088347233138, 4.051929230898665, -4.594562449499513], [-4.1074024107696969, 4.1671626897127521, -0.3334318129227718, -2.5576428521566319, -2.5242950527767349]], [[-0.79017031039932739, 2.0776531928347559, -0.64884098671549406, 3.7917702375979072, 3.0967454713853719], [2.2657076408963794, 4.2081191443323114, -3.8523599720599986, -2.499369853389124, -1.7176649099864285], [-1.2634851790073443, -0.46916987916943231, -1.3082446666484682, -0.84258649607533798, -4.1776088856128144], [2.6460395580838272, 4.58458111329921, 1.2810798020702707, 3.5860067271114282, -0.42324638292408778]], [[-0.34692092217387671, 4.4380408028426288, -4.9101471406417101, 0.99789995065827508, 3.5230891294439317], [2.9613917015561873, 0.025717035796146703, -3.5104399375805908, 0.8858052511258645, -0.66615239477807009], [-4.6411801671906137, 2.6549507613294843, 0.33992396395030688, 1.9277867506263471, -1.9589924942929771], [2.65126958727447, -0.41084157611361771, 1.5384405314753016, 1.8314786584178258, -4.5035258363958031]], [[-3.2571313532957689, 0.37324876661492734, 1.5902708961264747, 2.0482344625910276, -1.9316243559125423], [-2.3742298380350277, 0.12387062822586969, 2.4621643416253258, -4.8907352662901307, 3.8074581711074931], [3.5139410738961079, -0.29962112322277612, 4.0790773023230269, -4.2622079695636899, -1.0348560360712922], [2.7734509792832061, 2.6984754561620381, 3.9861947505221558, 4.3334752290558249, 4.4898088237043225]]], [[[-1.2472860621827078, -1.7829808635427624, 3.7735424720589865, -4.8267215736644333, -0.84068146535531607], [2.5485336854807343, -1.7707986232698216, 0.089991919505880702, -3.094646897092681, -1.1892063927326046], [1.8325438563079155, 2.3570638772970174, 1.5683722549499715, 4.120569356632549, -3.3033936683468212], [3.4785054194014737, -1.5979930733911738, -3.1496085747873801, 1.4436214861028045, -4.4305905019124197]], [[-4.0437235424608575, -3.5188994255456709, 1.8337526804346655, 4.0085460384505165, 4.6879332269113547], [3.4590528668372649, -3.1035331936139086, 0.95388850172540618, -1.3425144360419319, -4.3465582620771448], [-4.7426371849251456, -2.7205041336122191, 1.9591877009978944, 3.1106382361929459, 2.8262929532122438], [4.3922607342169648, 0.61869772606121831, -3.0093284371777629, 0.069441557340804927, -4.2630273898411311]], [[-1.5264137150928314, -2.27405222766457, -2.6651668666228225, 0.3627100848628757, -1.1646623335085806], [3.8435513908349996, -2.6674313304657571, -2.9845327972982894, -4.5633540541487347, 4.5329426221155291], [0.90147226396106461, -3.1613955772969335, -0.91928241514991793, 0.32228838044336428, -1.8887940369256029], [3.883377895194899, 0.99631692005169992, 2.3983577399289215, -2.0266306029806778, -4.0071455274938632]], [[-4.6362731724366082, 4.486929124835271, 2.7781055234045029, 4.6473222768043456, -0.62218086577525167], [4.2208361449825897, -4.992355065685663, 4.1776734949232424, 3.6474479245593567, 2.0336383912075107], [-3.6827632553736289, 4.7329452298961492, 1.8611072469961387, -3.8302913483089496, 4.0294539120297976], [-0.21546734592833694, 2.5662075168229412, -0.43665918268205406, 3.5517659535252495, -2.9204410362575386]], [[4.1870484759154376, 1.1734010848239107, 2.1992943567855505, -2.807312852045285, -4.7639285834289105], [-2.7459899068987248, -3.2284580282622697, 2.4238476346696896, 0.051434137327845519, -1.8709463961370956], [4.9587426137648674, -1.3777483324792685, 4.8505863169087693, -0.15146259444668875, 1.7907877769018299], [-2.6597991957533984, 1.4756595081279569, -2.8585780995207468, -2.9418667044400912, 0.3800287054526823]]], [[[1.5416870786632408, -1.6460280845500841, 3.3998504321921867, -4.4594873158443686, 0.82925918670239351], [2.140276575303977, -2.2018285569991258, 0.40172505189167396, -1.1529014201544454, -3.4001343695997899], [-1.4073069554224027, 1.1866070304131382, 2.7884787256273738, -4.5685231862057361, -1.0058324341392231], [-2.9058143965446437, 0.34365423123546623, 3.5390247654454381, -2.1737465636064215, 4.5577129150940507]], [[-1.1448497770035826, 4.5753318952152551, 3.4492853933235281, 4.2496084278072619, -1.4472170876613974], [0.38788260918400219, 0.16500994808030711, 1.1985669156379082, -3.228591656614598, -0.65440413752131654], [2.6910983546759741, -1.8151958846388938, 0.41451619260055317, 0.050124184987119236, -0.27470305510200177], [-2.5081467663811319, -0.63452386520595994, 4.0807350609284594, 0.65959567693181498, -2.5008214196560976]], [[1.7052203889662039, 2.0294759449424236, -4.8436972507155893, -1.3221456283338773, -4.8026996426912039], [4.418208849485854, -2.1048690134124803, 1.9818050894264427, 4.0887303953902538, 1.2718878491886816], [0.47146660162815213, -0.44706353537184551, -2.2218808919502706, 0.43742688694131093, -0.63959417646252881], [-3.1265221072340701, -2.3489832133253321, -0.41360115284442411, -2.5863490949814096, -0.36287854781232731]], [[-2.9208802479651585, 0.023641566540116621, -3.7506085176612292, 3.8299641809057547, -4.584885318433404], [4.6611883709273556, 3.0641994168508209, 1.8061429587386915, 4.8545195113784025, -1.8716782554166524], [3.4351769282969169, -2.8639465662306831, -2.4646242654445416, -2.1286119858699362, -2.1315957762961988], [2.8648394224687967, -0.31720918114413443, -0.15585952808295467, 0.95347840375699278, -3.1830529176043343]], [[1.6584922435669007, 2.6941309656360115, -0.092366249179038462, 0.11681232177065848, 1.1827599279815537], [0.67797638304562291, 1.3238757572025817, -2.4262110362791067, 0.48000802506531848, 2.9230949840484826], [0.94750033150934243, -1.5504249016442637, -3.7182814199788181, -4.9298236122096553, 4.9062250036557664], [2.7467336497457273, -2.0496679923680441, -1.132241987643503, 2.2686609573419592, 4.4414554597859137]]]]),self.functionspace)
      arg1=Data(numpy.array([[-3.2990335638118085, -1.0381013228069347, 1.5017270960672242, -3.9005523741683765, 0.056244374048426415], [-1.4949658453911652, 1.510900615534351, -2.8979394619492895, -3.4083166898874797, -3.4842556244964786], [-3.0769754025436837, 2.6000797619006217, -0.95959452998239669, -1.9516870558868007, 3.8686790559235984], [-4.9230729704918197, 2.5034473732221638, -1.3498947098238867, -1.6875558280060323, -4.5363227766107723]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[0.93279640897571681, -0.30266149095885342, 1.3294689866154779, -0.30596088690497891, 0.18970069802135311], [-0.5921042624835875, -0.77460411483149305, -1.2159971052302827, 0.92659884850753205, -0.76607457114402988], [-1.1644443244133249, -0.52896208809998591, 1.1486188206790264, -0.18519822966553173, -1.4183817159984227], [-0.085064276676464767, 0.85069582124013832, -0.27135541374534444, -0.44546573816786017, 0.76343062449710841]], [[3.2710974125187997, 1.3056745617190371, 0.51770016330273483, 0.7994866560507059, -3.9336128328191458], [-4.0710412941605778, 2.3920935931263196, -2.2050591051717028, 1.4865560826148649, 4.3924059611276531], [-4.4219182303437394, -0.55877623189432879, 1.1731030830596179, 3.8985712111206552, -0.91919412123392408], [1.9972125843886188, 4.3166681552387383, -2.3725341540959981, -3.7691613450104051, -1.7082556166254437]], [[-2.1778257275032269, -0.26163314882105548, -0.69618217738890475, 2.5156995724136384, 0.20274397565600752], [-2.7881361918695458, -0.40569294308171161, 0.85966437687119812, -0.6141569012082202, 0.11745222156054909], [-2.7572362955298839, 2.001006234961217, -2.1979226939060692, 2.3653937318910363, -0.87476075004253206], [-2.3774578339914725, 2.9970484215527549, 2.1014646953136733, -2.8042709134298711, 2.1216813626203765]], [[-1.186761485187239, 0.67143285347328441, 0.30189264787430142, 0.70531157243010645, -0.60985922220772304], [-1.1834510971948884, -0.039497392540674255, -0.35502577223450837, 0.069304878165641445, 0.35664024347484508], [0.62901217873529136, -1.2195188376254613, 1.1475430823027393, -1.0768792557306546, -0.83995563302807996], [1.2107571939887103, -0.55411013101677897, -1.2154198272764543, 0.82133476252621107, -1.0846892212167307]], [[9.8368197860648348, -82.801512618253057, -57.240491252381531, 40.044994245741712, 6.6357894612912309], [27.295242183279459, 18.262896653903745, -73.695759286259332, 2.9473997564301886, -55.099979158750784], [-12.805943077458426, -22.371423080030702, 11.501241522605278, 12.685828645982385, -37.295635869914214], [81.607031091998792, -45.652671479400659, -84.981480913889385, 26.512187900372332, -61.096066777264269]]], [[[2.3296537310893957, 2.9771974542659065, -0.0071182452842190446, -0.59772767733657961, 2.6220272192538738], [-1.9370089439884366, -0.74215037507698145, 0.84617481227834745, -1.5409295487145258, 1.0030697557879178], [-1.2873859586011953, 2.7889436503467162, -1.4062335291264401, 2.2103575791654078, 2.2488641544068497], [2.2343066647310637, -2.0551703848811229, 1.3841621478805048, 2.0103962165537665, 0.40622542152598734]], [[1.1047235897091472, -0.35296073099599845, 1.3503226718343233, -2.0567356063376434, -3.1253147629768709], [-0.73062530371108836, -1.2616742864664707, -0.74314910408155421, 3.215834029433339, -3.1320487372344394], [-1.5296321561483113, -3.288331773964817, 0.0192303107487698, 2.6817973262031165, -3.0409428669632135], [-2.7185126331536091, 2.7580653862126976, -0.22068414659083915, -1.6927935734886745, -1.6707221023164274]], [[0.27266625848278486, -0.71694154419541578, 0.22389735715150283, -1.308436662457878, -1.068602540545259], [-0.78183401366581984, -1.4521073333608334, 1.3293445300160691, 0.86246448078246984, 0.59271938994579509], [0.43599433169575774, 0.16189775022209979, 0.45143961211959949, 0.29075365691337629, 1.4415790738439924], [-0.91307620218676944, -1.5820141081261259, -0.44206575702880851, -1.2374332777467036, 0.14605080212386234]], [[0.10178658667582025, -1.3021210194493822, 1.4406370027791668, -0.2927838113221865, -1.0336742298322756], [-0.86887222374102591, -0.0075453774211913715, 1.0299629573730962, -0.25989523032119061, 0.19544908979689388], [1.3617221019869004, -0.77896246238113909, -0.099733679372831091, -0.56561256656287706, 0.57476833068515421], [-0.77788240603956249, 0.12054090435099239, -0.45137839920799461, -0.53735577560965708, 1.3213343260495198]], [[0.93481411937634962, -0.10712439236396921, -0.45641625285638737, -0.58785424587985702, 0.55438652156633539], [0.68141666223991115, -0.035551532830996192, -0.70665433509378106, 1.4036671798433007, -1.0927608595473592], [-1.0085198827522648, 0.085992864908146741, -1.1707170029789384, 1.2232764839633818, 0.29700921734777785], [-0.79599526503857099, -0.77447688889129762, -1.144059213823674, -1.2437305686152318, -1.2885991464398252]]], [[[0.40536107670915972, 0.57945892647331598, -1.2263804478058105, 1.5686578351176497, 0.27321682996241664], [-0.82825936254605881, 0.57549976571341199, -0.029246876472098508, 1.0057431380616135, 0.38648550513257524], [-0.59556662519725778, -0.76603273310162712, -0.50971231477945012, -1.3391622673441406, 1.0735846850176187], [-1.1304950363028095, 0.51933891706451074, 1.0236053795502076, -0.46916900437663134, 1.4399174261353291]], [[-1.5552305747362736, -1.3533813374145895, 0.70526785651153179, 1.5417011805516019, 1.8029959294343114], [1.3303641363327816, -1.1936299951603291, 0.36686893829292649, -0.51633586619687877, -1.6717018938295463], [-1.8240352678482235, -1.0463156451875804, 0.75351061521503315, 1.1963626200140542, 1.0870024045517217], [1.6892792284980842, 0.23795336401870956, -1.1573985080280715, 0.026707471962338619, -1.6395756208359231]], [[1.5906861360712767, 2.3698053256996854, 2.7773885566768639, -0.37798265155755884, 1.2137025557346037], [-4.0053911008699767, 2.7797483698814851, 3.1102019697351122, 4.7555023622659114, -4.7238104016690086], [-0.93943039043542864, 3.2945118782147786, 0.95799047037792273, -0.33585891787990546, 1.9683251393275991], [-4.0468945725088048, -1.0382686529798966, -2.4993449472590452, 2.1119655642658315, 4.175873665690208]], [[2.3755207877473956, -2.2990002988960319, -1.4234380020224078, -2.3811820971947326, 0.3187913061669343], [-2.1626603159822366, 2.557969040491102, -2.1405447570717251, -1.8688692500970869, -1.0419899978705722], [1.8869640213401262, -2.4250533483943255, -0.95358896877578325, 1.9625540563769099, -2.0646004183282902], [0.11040056103176524, -1.3148662891843164, 0.22373422079373601, -1.8198439871864656, 1.4963674772800903]], [[1.0822940893751221, 0.30330794254623844, 0.56848715672551353, -0.72565152380547482, -1.2314096141251456], [-0.70980039109063653, -0.83451172392265449, 0.626531071622075, 0.013295012738027778, -0.4836137526767339], [1.2817663450712455, -0.35612887824584571, 1.2538094390336427, -0.039150984679066109, 0.4628938588637464], [-0.68752128499281906, 0.38143756222643332, -0.73890288085381051, -0.76043183265243941, 0.098232161406822935]]], [[[-0.31315543927621792, 0.33434972311321326, -0.6905951734963901, 0.9058340882968593, -0.16844340753688844], [-0.43474402840105808, 0.44724678472095875, -0.08160046667996905, 0.23418328898733945, 0.69065284832861484], [0.28585945482782704, -0.24102974656794393, -0.56641019589616237, 0.92798201724589435, 0.20430987721856567], [0.59024402318667035, -0.069804821763820996, -0.71886498263540588, 0.44154262523337373, -0.92578617916336325]], [[-0.45730930446125462, 1.8276125730282031, 1.3778142213886386, 1.6975026011182455, -0.57808967871319694], [0.15493939011178898, 0.06591308842571128, 0.47876657143195461, -1.2896582892649777, -0.26140119601517292], [1.0749570306374312, -0.72507850736345703, 0.16557815316366456, 0.020022064583129169, -0.10972991005935707], [-1.0018771687430839, -0.25346003754385704, 1.6300462732221062, 0.26347495217479067, -0.99895106500174347]], [[-1.2632247363860509, -1.5034327715879396, 3.5882037431997342, 0.97944352156648595, 3.5578327759487149], [-3.2730025662981324, 1.5592838449504636, -1.4681182724873394, -3.0289254159116514, -0.94221263327612992], [-0.34926175960024453, 0.33118400429183947, 1.6459660711168704, -0.32404518941953597, 0.47381041781101069], [2.3161229423900549, 1.7401232823794019, 0.30639512091901178, 1.9159635756471971, 0.26881989030068137]], [[1.7308347371336372, -0.014009353733826287, 2.2225092974214911, -2.2695333199323726, 2.7168791943616899], [-2.7620943221978513, -1.8157618053272888, -1.0702715304374719, -2.8766571338350113, 1.109105977031986], [-2.0355930578935713, 1.6970973752107748, 1.4604697661212613, 1.2613579654932323, 1.263126079102717], [-1.697626457699956, 0.18796959240094577, 0.092358146318106599, -0.56500554703638783, 1.8861911794440238]], [[-0.36560278561262605, -0.59390195502112841, 0.0203614808133314, -0.025750443150329042, -0.26073098988454929], [-0.14945505785903537, -0.29183896790335767, 0.5348409175794594, -0.10581434538570189, -0.64437543975484424], [-0.20886968987185903, 0.34178011089471777, 0.8196686177514162, 1.0867444524952607, -1.0815423075606978], [-0.60549784153540709, 0.45183468930740744, 0.24959467026494006, -0.50011012643085029, -0.97908717666344347]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_constData_rank4(self):
      arg0=Data(numpy.array([[-2.3581196984925956, -3.3400997054166801, 2.0153462296851092, 1.1754871083092722, 2.3495399133865948], [-3.9540602505306532, -3.2643180621187264, -2.59528792702144, 3.9658332851333515, 2.2525837721447237], [1.1203021681853444, 4.4210290163233985, 3.2308567095505687, 0.92788849186787647, -1.763308695029818], [2.3652356870341311, -3.6568229803469112, 1.6653142608216189, 3.5935112617813392, -1.2948215874497127]]),self.functionspace)
      arg1=Data(numpy.array([[[[2.9521473932012476, -0.69742872282115265, 1.0270022532663123, 0.34954954817798534, -2.8902438662558927], [0.70528720609376627, -1.9810479387081701, 2.945827669267949, 0.97751783029551476, 4.199239769990891], [3.2534183934261147, -2.0076476160833776, 0.382277355397874, 1.5287801101084533, 2.9799085616780641], [-0.49220077249369254, -4.2495081917124118, 0.90933371203738922, -3.1561997357832738, -2.8044049082141465]], [[-2.3721400049841046, -1.7971920999014799, 0.8715254377303614, -3.0080504862949753, 0.30118979275567881], [2.0735053873439524, 4.7372024650208537, 2.9787778837008974, -2.4122239453894845, -2.9565829858051651], [2.3598997931330725, -0.34150769586149021, 2.0010472546121862, -2.6389080566127232, -0.40410360956483071], [-1.5582924434266765, 0.82858834001972692, 0.55994357885189228, -2.9223379438626274, -2.7429089616844604]], [[4.2248411948256361, -3.4264244166095628, 4.0125482279384919, -1.739530737551819, -3.8050594969596521], [-3.5332335640728152, 1.6785894928381238, -2.9071744740963257, 2.1558561207344962, 2.6828273357505488], [0.55015466916461175, 3.0512877463352908, 1.6248762141855879, -0.46465260347806669, -2.1895523419469356], [-3.2292709461338234, 3.2816714894808303, 4.547343594633201, 1.0521330540286957, -4.0888108977805189]], [[4.9946533532115449, 4.4848605034934561, -0.29374919048957615, 0.54256029480605328, -1.9485477671331362], [0.78646883980589966, 3.288880123012186, 4.8850172851375717, 2.728871147559115, 4.5094118725284886], [-3.9398155786057742, 1.4735217412460386, 1.1658135739695563, 3.3186175900412724, -0.20191846791161172], [-3.9419654575099514, -4.6550586054290433, -1.7569675131780249, -4.9318070461971155, -1.9635423986161191]], [[-1.470303777784995, -4.3903554010865147, 0.52945617967604353, 3.9795952435697242, -4.6632325537728105], [-4.1074766612918214, 3.8276084966007637, 0.27114065221565031, 0.49984192668767857, -1.8822987057073162], [-4.1486892246618634, 1.0533226286682709, -2.7068598149103353, -0.76622793574148051, -1.5373723792423046], [-2.5480835183823345, 4.586657658258483, -4.6639468322214519, 1.4095128708750746, 4.2649946329655606]]], [[[4.4644498083060711, 3.7446725534495169, 4.4117907739818953, -3.2826785475853191, 0.68195015921420321], [-4.9045664536316016, 0.56937380825917749, -3.2047571117382012, -3.7123286938859135, -1.5725306787675928], [1.5469358613007698, -3.5222570290404018, -0.99835872451659924, 2.8607907415599279, 2.4693797109362547], [4.4014037559364017, -1.2103544783315847, -4.4935783665427644, 3.3980363012137378, 4.2453037850666568]], [[-1.209068684814214, 3.8158024424971888, -0.65148079150524474, -0.19734054267260603, -1.5387955037482213], [-1.6611210248680761, -1.127766113047548, 3.1622114831688393, -2.3143207892178195, -0.52323667386193051], [4.142247663363257, -4.4840072132519389, -1.1526620421743883, 1.4865357714549159, 2.5346701647230363], [-2.1944855328242827, -2.9327751431347582, 3.1548070358109861, 3.4472569988950692, 2.0741446710950449]], [[-4.4118979068301387, -0.91696325486580399, -4.927377520517088, 2.9723878959649754, -3.0600729408580793], [3.1168495922493005, -1.3037708126969938, -3.3058818777069741, -1.0247093941899399, -0.5824718868571912], [0.53696173215187937, -0.40728121351715085, -2.5131330443670183, 3.02894583413517, -3.6847246446935014], [4.2522514357695904, -0.85888167441832053, -4.1997105385038456, 1.2113225510086263, 0.90654671106392382]], [[3.7735914799443808, 1.84832747465253, 1.3333127176095747, -0.63259442176999414, 4.121087005968862], [-1.3881198955938787, -1.6942932451729611, 1.6474310890177435, -0.62594957112963368, 0.9636133769440578], [2.0141906455037359, -3.4888275264840605, 0.4028880669774173, 4.5506647135504004, -0.24685935259619818], [-1.5345228565105904, 1.8680037001643477, 2.9277953325295991, -4.9534795769654707, -4.5624044889928319]], [[-2.9925187489977709, 0.48035960279598644, -0.66636209803753665, 0.89018948555942856, 4.2317154298533168], [-4.9215169363576825, 3.5318490525087025, 3.0446960824921767, -4.4138230795357671, 3.1764214655057845], [0.30286715890631122, 0.39943870365918421, 1.300328901201099, 0.53401209816268036, 4.5902215731639036], [1.9197760857844761, -0.51580783611818415, -4.6453211377420534, -4.4796332692516581, 2.0592646273991742]]], [[[-2.4427623000954437, 2.9695246333287724, 3.5981480474892287, -3.7333895950264506, -2.0185286478124018], [2.1169537327029779, 2.8956361563107489, 4.1131721462950122, -0.07267733923694486, -4.8253173503734974], [-3.6875595355866873, -3.77304799202585, 0.2781742486880141, -4.7346755459824719, -1.798068878820934], [-0.45981329153655892, -2.0885319144954084, 2.3011591740528701, 2.7297525873792381, 2.9791071445461315]], [[-0.0035894552690418635, 3.9069563402015728, 4.6790543900277157, 0.89767543658027193, -0.80120854720603596], [-3.6578240525509695, 0.70709463074434797, 4.9983934785397039, 2.4911286030248183, 2.2150463460998289], [-1.3166041342565582, 3.2397679107249679, -4.3159556189809356, 2.9252185589611379, -4.8360560819325809], [-3.4417173844799551, -1.3911852864518695, 3.7244664206200415, 4.9370463701463478, -3.6710440641988598]], [[-0.79126105013397918, 1.4008208323116644, 4.720589178010643, -1.6792558413891499, -0.67710118652280116], [-3.6047395137751401, 3.5705081142377963, -2.8574624986649031, -4.3896588857823771, 1.4909765143647169], [1.9985707764277185, -2.5602991166821862, 0.98176994842426613, -0.73028675806021237, -2.2743963967176164], [-3.8858768746392847, -0.047416475476938835, -1.831160661475181, -4.6437710985618486, -2.1199762780471665]], [[-3.6340827197185419, -0.75650686058580963, -2.7515492895324343, 2.3882344566230547, -1.6940229389581365], [1.9092081149898155, 0.42873591874183514, -4.9236438423632318, -4.8862278517128672, -2.8114817564262671], [-4.9318802412770602, -0.83074013071954855, -0.59647339984027248, -4.8155453852083276, -1.1873433513160956], [2.0685731442623148, 3.0697547250362458, -2.1550519596341342, 0.34809361790123106, -2.1270474216052437]], [[-4.336398838632352, -4.1939394209668883, -3.3926756866703678, -1.2722056851946997, -3.3498337956411559], [-1.438775997426438, 3.561321877739168, 4.6551898794633377, 4.2128120129295272, 1.3451899734102266], [-2.4911892960207349, 2.9410220580307396, 4.7894821420959506, 3.0253675816512633, -1.5069355220983551], [-3.5484220279840786, -2.8603300921507202, 2.6827701213395638, 3.9160842543609, -1.0831131724443432]]], [[[0.027324866768055678, -1.0078834974941051, -3.8455271982607799, -0.26260521335637943, 0.33414912978645983], [4.9539463447786751, 1.8182189541712344, -4.9357545520656547, -4.9075633568897912, -2.1509242190891387], [4.0827454011565205, 3.3583199349203277, 2.2426482486258825, 4.5550519393737066, -1.1821347182812647], [-0.15900674976603835, 3.4722411793999104, 1.3304984344436548, 1.6144067358739802, 4.9433298742499439]], [[-2.8901168209274863, -2.9285913253257299, -1.6389278667215548, -1.8614022231379046, -1.9364373140532001], [0.60502972033214242, 3.132238864071681, -0.7333131008546987, -0.2320860403350995, 1.6646986331435141], [-0.8270333005329551, 0.85776279695658619, -2.6246751959316059, 3.5053810637674943, 2.4240707383939473], [2.4185738008715427, 2.8853959275600456, 0.34231145532419127, -3.2303526093356751, 0.96652709460587261]], [[-3.5999748126628872, -4.5427754576194488, 3.6391564703198576, 1.7286930556058149, 2.1079425007445423], [1.1529860732315296, -1.8508366677915422, 3.5727563091018446, 2.4646839448900906, 1.8686621479175471], [-1.6622832913134236, -1.3450593931994828, -1.6839781651860464, 4.280172850811379, 2.0306529740164354], [-4.8471835907124516, -3.3407548557345921, 1.3160097713857732, -4.4097712988158424, -1.7186210102256516]], [[4.2676792568323378, 2.0737104790146574, 0.10160803356141468, -3.4585448958407516, -0.18143571824666793], [-3.2790030352772548, 2.0312669303598598, 3.7879868256143361, -1.1104935167210073, -2.6118825095593579], [-0.067343450706914609, -1.3594255632019916, 4.8334460678822477, -4.4145586173974909, -1.808238498131499], [2.8119469700410011, -3.6587706458947356, 3.8750070745900995, -1.6901641574306825, -0.018685591592573658]], [[-3.0055267304870137, 4.0366003002766568, -2.8375723804812578, -3.3409008747708455, -1.5574837047705694], [1.9602003964576689, -3.2416321803024459, -4.8171066313849318, 2.2360838911847933, 0.72472555483644374], [0.83952958793013543, -1.6116440952111253, 2.2420402501478565, -0.45423296249238199, -2.0217617039332669], [2.5093433500927853, -0.12883103938525586, 1.3690569144711038, 1.3576177302399328, -4.6259482399122023]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.7987811529747163, 3.3811622913289558, -2.2961193035290357, -6.7461672051479145, 0.81588952614831545], [-3.3434885506473928, 1.1903395432370565, -0.80049478898356552, -2.4123546654690782, -0.56155871720983219], [-0.72481292392562624, 1.1745685246761266, -6.1686094276713472, -1.5424845488899697, -0.79133961652993712], [4.7909711448549475, 0.55491590840829774, -2.5932390576493183, 0.7471389315947019, 0.84086277683569355]], [[1.4080533604250993, 1.8585101200922154, -3.832475290813099, 1.1103868504317196, -11.089684264719173], [-1.6108468903932611, -0.70507852051495046, -1.121298678794697, 1.3846557289179791, 1.129716203283593], [-1.4153565821463399, 9.7804522296076417, -1.6691758266668268, 1.2657128000526094, 8.2654537756135156], [2.1434357328152212, -4.0310725412056359, -5.9650647521759552, 1.1429546375467692, 1.2177216787266159]], [[0.47702295465055577, -0.58817764078370904, 0.50226093624313251, -1.1585574121682194, -0.52964907153105667], [-0.57039711446700603, 1.2006188757190446, -0.69323195000587823, 0.93482408696109298, 0.75120236134066942], [3.6632357092330659, 0.66049038872378207, 1.2403075459475728, -4.3373182773530745, -0.92043756665486998], [-0.62408706587409146, 0.61412186934163315, 0.44319198401097987, 1.9154860898706667, -0.49289299017938831]], [[0.23534908735026386, 0.2621011528438027, -4.0016692687736448, 2.1665557165945373, -0.60326317277751196], [1.4946391373870351, 0.35741257338156762, 0.24063110521341136, 0.43075947699498618, 0.26067415031888858], [-0.29836094732262941, 0.79773991479437378, 1.0082976683028126, 0.35420987095251705, -5.821592846196876], [-0.29819822648872224, -0.25251821898404025, -0.66904316641748052, -0.23834815460099612, -0.59865634128284739]], [[-1.5979962432839319, -0.5351593888743349, 4.4376475402066315, 0.5903967035800961, -0.50384360768919589], [-0.57201540194452449, 0.61384018649587146, 8.6653915382555784, 4.7005658948147468, -1.2482290437020209], [-0.56633307200254146, 2.2305985359463394, -0.86799467798239982, -3.066372033424936, -1.5282828969156892], [-0.92208120198438936, 0.51225534767264413, -0.50376644458176678, 1.6669162530796322, 0.55088930129623614]]], [[[-0.88567694123790075, -1.055916156644525, -0.89624836106221017, 1.2045225242779836, -5.7981660347243462], [0.80619975035772162, -6.944576995243124, 1.2338096500505287, 1.0651158818568152, 2.5144566677894562], [-2.5560595946142253, 1.1225927630863133, 3.9605606215793738, -1.382156406299955, -1.6012362266601308], [-0.89836344716105787, 3.2668613379950759, 0.87993575008524849, -1.1636309621290126, -0.93139630300180487]], [[2.6998615571788815, -0.85547355014072679, 5.010612906293872, 16.541548015981334, 2.1213462439729329], [1.9651295801147144, 2.894499155766971, -1.0322896110817885, 1.4104864275198346, 6.2387027232347645], [-0.78805477784210898, 0.72799126024405825, 2.8319819189681117, -2.1959229806652032, -1.2878670004289963], [1.487509492904963, 1.1130475071571944, -1.03471243250844, -0.94693202832426504, -1.5738140678467378]], [[0.58824750296320949, 2.8303074449818122, 0.52670774995724012, -0.87313231578709838, 0.84811309311264049], [-0.83266383256836229, 1.9906013401640741, 0.78505162102814863, 2.532706288960183, 4.4556449600077359], [-4.8332828423746292, 6.372225972834249, 1.0326902241958758, -0.85682876787476492, 0.70433700677172806], [-0.61033265935078318, 3.0217060211222977, 0.61796828691579686, -2.1425242391965984, -2.8628286831194925]], [[1.0509439896212094, 2.1456334656708465, 2.9744209537305566, -6.2691562692522353, 0.96232699755898254], [-2.8569818051895659, -2.3407006410678934, 2.4072832615401967, -6.3357073285893035, 4.1155855450142704], [1.96894633285397, -1.1367238004826234, 9.8435114122048315, 0.87148439508724718, -16.065153065602054], [-2.5844080903111539, 2.1230328852049039, 1.3545459414701961, -0.80061565279791524, -0.86924193036834929]], [[-0.75273839901559181, 4.6893697118436055, -3.3804200130509736, 2.5304542557353567, 0.53230984206865839], [-0.45770111152189114, 0.63779163227394842, 0.73983862793324029, -0.51034754487296841, 0.70915770989667537], [7.43753063316296, 5.6393728286949152, 1.7323184696302896, 4.2182261036687247, 0.49073530247736702], [1.1733575539484089, -4.3670987806175976, -0.48491454204168, -0.50285004078492956, 1.0938777572213771]]], [[[-0.45862103248505676, 0.37726650104582904, 0.31135521757285284, -0.30007641572628513, -0.55500929818334843], [0.52920484320406735, 0.38689327930367151, 0.27236938507289798, -15.414738348261505, -0.2321717074419217], [-0.3038058524544216, -0.2969223213044328, 4.0273396026740693, -0.23661646026325028, -0.62305853873627559], [-2.436428413023096, -0.53640653533226501, 0.48684253606508882, 0.41040428841974885, 0.37605299636043238]], [[-1231.6712940968082, 1.1315788125995037, 0.9448552309513143, 4.9249749254201198, -5.5179503909941472], [-1.2086499932221095, 6.2523866312907046, 0.88448999369593762, 1.7747092666975224, 1.9959081326256589], [-3.3579030334883493, 1.3646128791164234, -1.0243453377695464, 1.5113499819628804, -0.91418067562125338], [-1.2845415594724718, -3.1778865542770043, 1.1870234597490061, 0.89548055352624667, -1.2042974529885437]], [[-4.083174205280935, 2.3064025284510796, 0.68441810708724304, -1.9239812242533993, -4.7716010160052651], [-0.89628021586696716, 0.90487308981799253, -1.1306733547894774, -0.73601543846948081, 2.1669400412569133], [1.6165835844580196, -1.2619059579793699, 3.2908490575985452, -4.4240932399381991, -1.4205336915822173], [-0.83143568717690786, -68.137850336892015, -1.7643764293995887, -0.69573987196551268, -1.5240060669578335]], [[-0.25532949121745391, -1.2265433933399568, -0.33722401244920125, 0.38852487422022364, -0.54774257805419657], [0.48600699137130277, 2.1642424889215026, -0.18845564821002814, -0.18989873579935598, -0.33003539494680373], [-0.18814092120525805, -1.11694193834621, -1.5556242610590052, -0.19268606515848144, -0.78148287168944885], [0.44856450662216052, 0.30226795786002758, -0.43056432478101608, 2.6656291415579987, -0.43623310060836162]], [[0.40662973140771902, 0.42044209942910848, 0.51973983306384364, 1.3860248508164459, 0.52638692024788114], [1.2255616567025562, -0.49512758339867285, -0.37878340963249746, -0.41855859924868549, -1.3108250357825724], [0.70781802805849137, -0.59955643318448992, -0.36816270375698013, -0.58284114159357581, 1.1701288271275683], [0.49692755853834697, 0.61647035070136358, -0.65727163166308145, -0.45027343144270215, 1.6280004157371903]]], [[[86.559825052805962, -2.3467352059189408, -0.61506148964538809, -9.0068116196318186, 7.0783835006413165], [0.47744475261162755, 1.3008530582128011, -0.47920447868386407, -0.48195723927100126, -1.0996369216744177], [0.579324805892655, 0.70429135188700942, 1.0546619107492048, 0.51925548127983312, -2.0008173776276585], [-14.875064678161939, 0.6811841588270382, 1.7777064788679362, 1.465080412807916, 0.47847012989255761]], [[1.2652855254388562, 1.2486627781498958, 2.2312287530151478, 1.9645528166299984, 1.8884282769230125], [-6.0440385942353867, -1.1674789628251114, 4.9867143735530881, 15.75632457284796, -2.1966876812060527], [4.4216151610707684, -4.2632100544831548, 1.3932478144401228, -1.0432026971745694, -1.5085463152654184], [-1.5119749411943355, -1.2673557016624757, -10.682736214257456, 1.1320197583937872, -3.7834665999074542]], [[-0.46259053117924248, -0.3665852024511671, 0.45760996384836644, 0.96333716122785895, 0.79001882652558908], [1.4443489817306834, -0.89976295034650866, 0.46611470717415437, 0.67567051113155263, 0.8911799613843836], [-1.001823377233011, -1.2380971942512875, -0.98891677769327524, 0.38907640389007431, 0.82008806139228618], [-0.34356327332277642, -0.49848442425011047, 1.2654269725277374, -0.37764186575136138, -0.96898283618851277]], [[0.8420293666700257, 1.7328895707219598, 35.366408893341315, -1.0390240317836841, -19.805974791004715], [-1.0959158082869818, 1.7690984912281871, 0.94865991546803841, -3.2359587946015429, -1.3758318946695611], [-53.360961222801862, -2.6434042135541289, 0.74346774771313828, -0.81401371535073586, -1.9872993886009009], [1.2779441789149182, -0.98216357612177208, 0.92735605190126336, -2.1261315038440087, -192.31455659180375]], [[0.4308135323886807, -0.32077032431498587, 0.45631314864648787, 0.38756659834707763, 0.83135482155201801], [-0.66055572164438892, 0.39943507326883249, 0.26879653836465889, -0.57905769660710227, -1.7866371329239636], [-1.542317991009825, 0.80341658018490192, -0.57751933194077254, 2.8505672075073774, 0.64044223655571397], [-0.51600016689698336, 10.050540565598348, -0.94577630320791317, -0.95374534274893263, 0.27990403703139738]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[-0.20738294827179971, -4.0244565239446839], [2.8515317378942315, 3.9816259788923212]], [[-4.987675160509867, -1.5095018402055906], [-1.5476495175886407, 4.5645401800381311]], [[-4.5054038047936817, 1.7449390200071155], [0.51643275490126594, 2.7805495146979169]], [[-3.7478587170090591, 3.1478466436904267], [0.057454563297762995, -4.692011132322814]], [[-0.93667452931336115, 1.7720154121568683], [0.64740055968805255, -1.1221943281484337]], [[2.695800562435128, 2.0871193567063839], [1.1987766966184878, 1.650182900349737]]]),self.functionspace)
      arg1=Data(numpy.array([[[-1.8158211270472369, 3.6934339029470866], [1.7580810575651418, -1.3643182481281224]], [[0.18097877679782304, 0.63853559491762635], [3.1221560745068793, -0.73441995072619637]], [[-1.515145338317919, 4.1654785822051963], [-1.5103682160940624, -4.4061782138569727]], [[1.6852022077498674, -3.7305290184212083], [-0.67951381269267941, -1.8314102974574897]], [[-0.90264851634218424, -3.2951108985178279], [1.3183368946844176, -0.85267361070267178]], [[2.5980414568377244, -2.3578262769868461], [0.81176957864456156, -3.9016751206127687]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[0.11420890812578635, -1.0896246229649502], [1.6219569203728676, -2.9183997094191243]], [[-27.559447846648631, -2.3640057848307148], [-0.49569895951888954, -6.2151636478893337]], [[2.9735786335820973, 0.41890481143305947], [-0.34192506793926314, -0.63105697948244066]], [[-2.2239816087194146, -0.84380703866569096], [-0.084552458279089762, 2.5619661191359682]], [[1.0376957501786643, -0.53777109989042782], [0.49107368708134863, 1.3160889630718782]], [[1.0376280006387557, -0.88518792799849166], [1.4767450372064015, -0.42294215928736051]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[2.800720304554007, -2.0862429026986806], [2.431024694541164, 4.1155119375226512]], [[-3.5244784834120146, -0.58867366370559182], [-0.45910229803041958, 0.55044865422231393]]], [[[2.9483772376518207, 4.7242225034362555], [-3.8333295716317437, -0.0062924162760715774]], [[0.35078623601652303, -4.0780152883765952], [2.2903179643948164, 0.41904660821419526]]], [[[-1.038581410983884, -4.7996849470935867], [-2.8929466124034544, -1.7751375900170405]], [[3.5479099837747352, 3.6595097237284921], [1.9134237016720874, -1.2678744861980782]]], [[[-4.6374453444388761, -4.4008005263051855], [1.6905387519034072, -3.9353176814652349]], [[-4.5002088511913509, 2.1896075892344689], [-4.9633112219476283, 4.1649407410357888]]], [[[-2.6720891303379988, -4.0458159710591985], [-0.97745567215012841, 4.2958889239781612]], [[-4.3021105307389185, -4.2427055667093541], [-4.9164128833110023, 1.4496885669314752]]], [[[-0.42890675220085939, -2.9736483484276413], [3.896288966124871, 4.4286321766033705]], [[1.2575628276880026, 1.3644848580810649], [-3.6186169383304034, -2.6813534030742483]]]]),self.functionspace)
      arg1=Data(numpy.array([[[2.4914368052068276, -4.7058201967747557], [2.4725430168452078, -0.4298420904063267]], [[1.7450948267902158, -3.6286678045468079], [-3.6571889173759162, -3.5749699365870802]], [[3.9742866112316335, -3.5765259889509196], [3.8888740006619891, -3.2465263306052372]], [[0.069500550958082385, 2.0162848745486484], [-4.8986384103545211, -4.9290052262652004]], [[1.2907336436874672, 4.7666017888511885], [-4.1593491482370881, -4.4241690961233173]], [[-4.6001173505067747, 1.1610240132288006], [1.7721070743837171, 0.91704179840073685]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[1.1241386089748739, -0.83736537019067214], [-0.51659957093288944, -0.87455783804560017]], [[-1.4254467806626892, -0.2380842960850478], [1.0680719926623132, -1.2805834200693997]]], [[[1.6895226507976211, 2.7071437213103215], [1.0564013511593675, 0.0017340844119671215]], [[-0.09591690337621854, 1.1150682615823488], [-0.64065376912828431, -0.11721682018233889]]], [[[-0.26132524213245584, -1.2076846530215801], [0.8088705691894118, 0.49633012467993576]], [[0.91232320285275048, 0.94102038870520022], [-0.58937569168440263, 0.39053263614273948]]], [[[-66.725303332283531, -63.320368912750411], [0.83844241121029073, -1.9517667027811076]], [[0.9186652441378429, -0.44698289724878959], [1.0069600242052132, -0.84498606713623681]]], [[[-2.0702095613655569, -3.1345088050085996], [-0.20506342158397664, 0.90124770523646469]], [[1.0343230100224547, 1.0200407360626593], [1.1112624261172599, -0.32767476455675398]]], [[[0.093238219706201331, 0.64642880210437403], [3.3559073040094285, 3.8144191042934348]], [[0.70964268799916808, 0.76997878841806988], [-3.9459672881225738, -2.9239162356071007]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_constData_rank4(self):
      arg0=Data(numpy.array([[[-4.7325335595165932, -4.5061370238276517], [3.9448945089556133, -4.1422853891190989]], [[-3.449491047669444, -0.68396473398316804], [-0.35951319048896391, 3.9588990122140046]], [[-1.1353744772170837, -2.1286757911311094], [-3.1403687511711196, 1.3292719327366997]], [[1.0277517746554024, 0.89207606361461167], [-3.9541459266451753, -0.27242476385769976]], [[-2.3539347549844001, -0.55094886469263571], [-1.117030814138694, 0.82390176537891691]], [[-2.067473053322181, 0.093955745319723505], [4.1488205965159572, 1.9641513887743622]]]),self.functionspace)
      arg1=Data(numpy.array([[[[2.2503256069723072, -1.0006650409501949], [-0.32223789407519021, -2.2796122187703558]], [[-1.804078113930264, 0.051160216664386304], [-3.6857620432091043, 3.4407902208972008]]], [[[2.3369902901548105, 4.7784975039849584], [3.295236061258251, 1.6507754440132816]], [[-2.5457264609441843, -3.0315181825889512], [1.0226335921331451, 0.59528548300293327]]], [[[-1.7842080141411865, -4.5434966336546445], [2.3075723452650703, -4.8216067933331086]], [[-3.3395479982231802, -4.6350645083746889], [1.6338081535032778, 4.1219714699832632]]], [[[-3.465470057832567, 3.9753531399672184], [-0.23228598074089923, 1.1524926227434484]], [[1.8657029492311263, 2.1509232515298233], [2.8639816324040055, -3.2411145552215359]]], [[[3.4909464090660851, 0.40961801684688481], [-1.8403046618059093, -0.15149506123915657]], [[1.1254480267780185, 3.2324446036739278], [1.2260786704318027, 2.38466731153287]]], [[[-3.4660302278160637, 2.3689389461566961], [-0.74725343388824683, 0.5144230522112947]], [[3.9988554088598995, -0.026323349887675818], [-0.19652248863058119, -1.6360169476047579]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-2.1030439083364314, 4.7293883226126816], [13.983883046281951, 1.9767120858205893]], [[-2.1866539361543977, 77.108635697036377], [1.1238613183808552, -1.2038761805243041]]], [[[-1.476039957119778, -0.72187775441815993], [-0.20756168033740299, -0.41432936046125551]], [[0.14122223891864016, 0.11859179752038812], [3.8712780830483053, 6.6504208909030247]]], [[[0.63634647317935444, 0.24989002276509326], [-0.92247412979227261, 0.44148680769125637]], [[0.94035742347226792, 0.67752428159242761], [0.81360343923270362, 0.32248450587701355]]], [[[-0.29656922654186668, 0.25853093762228063], [-3.8404214527680334, 0.77404058473803605]], [[-2.1193866517039681, -1.8383482180653481], [-0.095120988478207658, 0.084052803199694076]]], [[[-0.67429701838766876, -5.7466582478580301], [0.29937916048747282, 3.6367447241259194]], [[-0.99252101168685536, -0.34556843228468648], [0.67198115850816786, 0.34549966839999613]]], [[[0.59649596726826304, -0.87274222777095822], [-0.12573477893682689, 0.18264295294669652]], [[1.037502028036271, -157.60990201548665], [-9.9945375333941158, -1.2005690965793574]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-2.7158225957952853, -0.53369886118058396, -3.7018700025142715, 0.71780441627105418], [1.7725997997251106, -3.9278712567383858, 4.7046191518795251, -1.0530902456790061], [-1.1419394315618705, 4.0412488665610269, -0.78095097319176965, -3.6435213416881593]], [[3.9942851995164901, 0.82142551712873679, -2.990207155781075, 1.2176799834278595], [-1.4974029515320209, 3.7091675255144665, 3.5180302508332684, -2.4454444834575693], [-0.99749594784949824, 1.4484066954737882, 0.69652811266204573, -3.4714686445343923]]], [[[2.3235393947898757, 4.539721250824952, 0.94539438704467393, 1.8293521606576402], [2.9468098345521332, -4.7549588191215095, 0.37158746806350074, -4.7989725716142875], [2.1469278948389947, -2.8457534431844991, -1.7948139862455594, -2.2674731979001397]], [[2.8494629088806285, -2.9761945930553999, -3.3414942248070556, -4.346606693340588], [1.9264966279278095, -2.7634225811398916, 1.8088562491213818, -4.9475542942754425], [3.2044808307345178, -0.63021125276344936, 1.2077120402068608, -2.923721362611059]]], [[[-4.5065278668388995, 1.1078534634943518, 3.9397955168183376, -0.08296117696301053], [0.69918706331875935, 2.4875177537031838, -2.0941040754989313, -4.6492542870617175], [-2.979670717695091, 1.9072194314564967, 4.5078430272086454, 2.8855401856346603]], [[-3.041531853745294, -4.7587872613433833, 4.6071791326393026, -3.8571221875841988], [4.7344000525262171, -1.845701360340517, 0.13411126271190543, 4.8951618903814325], [3.7414625890504336, 0.78541848275422943, 3.6251230985039271, 3.5916534770410014]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[3.1040365264797725, 1.0401019579279858, -4.4085198645640036, -1.94760123885098], [3.3309779498675613, -0.56414640264341198, -2.2156776224377195, -1.1971118210632383], [1.3587955191509344, 2.0137457122925104, 4.3534452741971883, 1.6064280811052232]], [[0.59796635159553002, -3.1226869065715634, 0.89204036917565066, -2.5588525984495512], [0.24374660154973338, -0.1098691861355956, -2.0801676454766849, -1.3264209166587726], [-4.1718319435774109, -1.6712845626963482, 0.56008139088385711, 1.4379278503983652]]], [[[-2.5469026617158219, 0.70558209738015609, -1.6945285094473284, 3.3452373614502431], [3.4107650787039372, -1.4486597908926746, -3.8426428883934785, -2.6582069423055907], [-0.75139809521923873, 0.27495919794870982, 3.2692111942775064, 2.223106162938457]], [[4.9451738457901477, 1.1924580576764487, 0.4696671292726462, -4.8253797738939337], [-2.8673281422883354, -4.0986182995073275, -2.370554777744156, -4.3698572658995074], [-0.30745124755527176, 0.19751481370506951, 1.9892761388507418, 4.0983937986835013]]], [[[-1.8304773311093179, -1.9336248761065331, -1.8077449704280624, -4.1319624788933984], [1.3877607337341615, -3.7372607599187635, 3.8238252084912503, 0.38460378304392684], [2.958035183079474, 1.3381855307554495, -3.3675100435098173, 3.7903938817246203]], [[-3.2278236632959834, 0.43310149857657443, -3.030613036103925, 3.0784179715972098], [4.5151588931612014, 0.95635141011450742, 2.3520124323437859, 1.1765837831088124], [-2.9161229590159374, 4.6721221415439551, 3.1887831482321118, 4.3377585935681289]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      ref=Data(numpy.array([[[[-0.874932550769706, -0.51312167726688962, 0.83970813702579994, -0.3685582048071272], [0.53215596932894393, 6.9625034181439798, -2.1233319794526051, 0.87969246243319477], [-0.84040565005353418, 2.006831767234575, -0.17938688188420687, -2.2680886773228064]], [[6.6797825477281396, -0.26305087307987274, -3.352098468978904, -0.47586953002516474], [-6.1432772478121915, -33.759852566276216, -1.6912243868820906, 1.8436413756333052], [0.23910261998572499, -0.86664277753934227, 1.2436194524564792, -2.414216153872152]]], [[[-0.91230003789172343, 6.4340085550371109, -0.5579099919381203, 0.54685272313967304], [0.86397326305214106, 3.2823157300386376, -0.096701015123175554, 1.8053419751631179], [-2.8572442603977803, -10.349729939623037, -0.54900521244612099, -1.019957227280158]], [[0.57621086694583878, -2.4958484484180783, -7.1146009940740109, 0.90078022808824632], [-0.6718786732202634, 0.67423272410413715, -0.76305186705818617, 1.1322004342988581], [-10.422728338932615, -3.1907037297186487, 0.60711130879224662, -0.71338224344137591]]], [[[2.461941369198942, -0.57294125514411021, -2.1793978582528815, 0.020077911497693168], [0.50382392751335414, -0.66559919510600452, -0.54764639106639312, -12.088425782672843], [-1.0073141572958213, 1.4252279580244818, -1.3386279384367643, 0.76127713258173224]], [[0.94228562989080278, -10.987695209976298, -1.5202135930102672, -1.2529559738708793], [1.0485566874948931, -1.9299405436329355, 0.057019793291765576, 4.1604873028652989], [-1.2830263475285733, 0.16810743789645, 1.136835880644228, 0.8279975474814516]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_expandedData_rank0(self):
      arg0=Data(-3.85384219668,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.2823600439)+(1.-msk_arg1)*(-2.1254746587)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.17410708915)+(1.-msk_ref)*(1.81316779333)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([-3.4962304545003975, -2.1847391300877241]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-2.01262809122)+(1.-msk_arg1)*(-1.78941847896)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.7371468031031654, 1.0855155702225032])+(1.-msk_ref)*numpy.array([1.9538361180476314, 1.220921296930735])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_expandedData_rank1(self):
      arg0=Data(-3.67050599506,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.4139884801916445, -1.5660456832964318])+(1.-msk_arg1)*numpy.array([2.3223897811863834, -1.1546593699919261])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.520515124732891, 2.3438051866636265])+(1.-msk_ref)*numpy.array([-1.580486628384705, 3.1788647721171879])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[4.6362997412315146, -2.7917106255995883, 1.1005362735990278, -3.1534112444636522, 3.4053916196349938], [-0.37153469181735677, 0.38174416542191114, -2.4884206765787398, -1.2401796785757249, -4.8882077592896103], [-1.9049063402284148, 0.015850181753497949, 1.2271537558508001, 4.8051549581894371, 2.0391977398281753], [0.005989465386331716, -4.2989729530039522, -3.1708479209334071, 0.12223503700910499, 2.796230474382206]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.217023658275)+(1.-msk_arg1)*(-0.645686890459)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-21.363107497461577, 12.863623476735777, -5.0710428639168947, 14.530264900708698, -15.691338201074345], [1.7119547922546858, -1.7589979288679061, 11.466126303243929, 5.7144907077433862, 22.523847391258464], [8.7774132800322846, -0.073034349708446133, -5.6544699578010462, -22.141157311475784, -9.3962001932511896], [-0.02759821410221033, 19.80877562918494, 14.610609488982208, -0.56323369525889921, -12.88445000237143]])+(1.-msk_ref)*numpy.array([[-7.1804148570114057, 4.3236290946122642, -1.7044426483804911, 4.8838086866228263, -5.2740603378404867], [0.57540999717845609, -0.5912217997028838, 3.8539123425730466, 1.9207137343202443, 7.5705544459984164], [2.950201356687546, -0.024547783124763373, -1.9005399892475221, -7.441927394206961, -3.158183587059658], [-0.0092761142820649775, 6.6579839493812294, 4.9108135348354818, -0.18931008018786513, -4.3306291574126758]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_expandedData_rank2(self):
      arg0=Data(-0.606616856422,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-2.688757294852282, -2.0127886973192233, -2.1953667774926302, -3.0276339357342499, -3.2288275336514181], [0.8287415809591101, 3.7598733200823489, -1.2979341676645006, 0.63373329569594095, -2.964706602994954], [1.0753603672804761, -1.3601253639238675, -0.18240391412526513, -3.6397056678439013, -3.0285633505678811], [-3.4793245283911411, -4.922027482504701, -4.505948837060366, 4.3903143745998783, 2.679896834293432]])+(1.-msk_arg1)*numpy.array([[-0.16885240785963784, 0.99195720605498217, 3.5848358165960565, -3.4187938512249905, -0.95958135583221171], [1.8641264350489006, -3.8895824240714338, -1.1069911547830835, -2.3364606312354095, 1.4182470616643865], [3.1537224043731111, -0.80692335376367108, -3.4291970375918313, 0.50133476075115446, -3.4104168136866839], [-2.9435322082343554, -2.2070580698456341, -2.4367552545044893, -2.4384386083513654, -1.2886973592855231]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.22561235169253877, 0.30138129115591128, 0.27631686087320712, 0.20036003998448307, 0.1878752736402991], [-0.73197347684672587, -0.16133970609648254, 0.46737105127114187, -0.95721159128293776, 0.20461277881908663], [-0.56410564763158511, 0.44600069413604976, 3.3256789435203706, 0.16666645926384524, 0.2002985528793203], [0.17434902995455706, 0.12324532087199958, 0.13462577547104213, -0.13817162158857421, -0.22635828687861773]])+(1.-msk_ref)*numpy.array([[3.5925863546248942, -0.61153530890169128, -0.16921747255864533, 0.17743592706086531, 0.63216823955066781], [-0.32541615472889268, 0.15595937822731981, 0.54798708535386587, 0.25963067740685275, -0.42772297776538021], [-0.19234947742417866, 0.75176515042314174, 0.17689763806867498, -1.2100035822634612, 0.17787176452673395], [0.20608466750427185, 0.27485314714194181, 0.24894451558100644, 0.24877265900584941, 0.47072095868840103]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[-2.2428652403549831, 4.9469968889969493], [0.74056907170076958, 2.389788596040292]], [[0.92886859947603906, -0.90858954317492646], [-4.5102015617897271, 0.2374596434451508]], [[4.655195650858591, 3.8703748111806853], [-0.99679490005947979, -1.7735052326206424]], [[4.0350648724503166, -2.6714161838540198], [-0.20026375635468252, 0.85138247727722138]], [[-0.82763938032999107, -0.50045345293199262], [-0.64996297606498477, -2.4412901288597344]], [[-1.9728917751338493, 2.2032031790707425], [-2.4211154163672255, -1.4678812467754287]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.8560473947)+(1.-msk_arg1)*(1.48036308576)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.2084094656009623, 2.6653397446278477], [0.39900331953643736, 1.2875687349736886]], [[0.50045521581524066, -0.48952927913977606], [-2.4300034442427489, 0.12793835121005032]], [[2.5081232646111244, 2.0852780064994181], [-0.53705250356637391, -0.95552798796437033]], [[2.1740096098734698, -1.4393038623300747], [-0.10789797551880614, 0.45870729363381513]], [[-0.44591500340724166, -0.26963398368062008], [-0.3501866266574219, -1.3153166971036481]], [[-1.062953338784054, 1.1870403661924147], [-1.3044469787170601, -0.79086409698909366]]])+(1.-msk_ref)*numpy.array([[[-1.5150777953976027, 3.3417456410506561], [0.50026177957729934, 1.6143259846418589]], [[0.62745998492779642, -0.61376128053815115], [-3.0466860496503489, 0.16040635282654714]], [[3.1446310000912279, 2.6144767107621458], [-0.67334487711203916, -1.1980204381515664]], [[2.7257264864788522, -1.8045682235382248], [-0.13528016084812697, 0.57511733808365817]], [[-0.55907863975641148, -0.33806128898199667], [-0.43905646007993965, -1.6491157827089638]], [[-1.3327080323179676, 1.4882856782029286], [-1.6354875636006712, -0.99156839352417403]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_expandedData_rank3(self):
      arg0=Data(-0.803488697602,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.7527065686814609, 0.16642355351772053], [-4.2973420716842723, -3.0806385699165895]], [[0.54243976042135422, -3.2163246120852773], [-0.17263927651043254, 3.7592354509703085]], [[0.26349389383813637, -2.035980822610286], [3.364042355283555, -4.2112584561115582]], [[-0.0088193931613309573, 4.9240567590744426], [3.6888969076118769, 3.0522804201715754]], [[-3.0652057702826543, -0.81464166817157668], [-0.68265031948768318, -1.6148619066312242]], [[-2.4523438564441582, -4.254366000776165], [-0.55008646608181522, -0.37596414299786929]]])+(1.-msk_arg1)*numpy.array([[[2.9558607595624036, 4.3998372018929786], [0.0947319564843907, 1.345886200976441]], [[-4.9323195616156852, 2.8590438621070362], [1.0198471821398858, -4.2258707125544506]], [[-4.9027703464402963, 2.9145909392522462], [4.1986369426905785, 0.20359684962674951]], [[-0.019822594113060532, 1.9130674757931585], [-1.4109174021729309, 1.0186085908793219]], [[4.5843547353732035, 2.9308768479021925], [-4.152737599852677, 1.5165003423479329]], [[3.0583445390428974, -2.4316050575237869], [1.5898564659552328, -1.6558629745646627]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.16905918469619141, -4.8279746503307717], [0.18697340919072031, 0.26081887873762361]], [[-1.4812496358625895, 0.24981579738016119], [4.654147734180099, -0.21373726335607154]], [[-3.0493636338119559, 0.39464453136222871], [-0.23884618941838356, 0.19079538954334757]], [[91.104760033202183, -0.16317616488087561], [-0.21781272768657686, -0.26324209672594739]], [[0.26213205827530084, 0.98630935415433907], [1.1770135817185658, 0.49755876604810445]], [[0.32764112401698886, 0.18886214713424584], [1.4606589093613822, 2.1371418327148151]]])+(1.-msk_ref)*numpy.array([[[-0.27182900784557679, -0.18261782441764654], [-8.4817069911768872, -0.59699601423849191]], [[0.16290280618766753, -0.28103405766198081], [-0.78785205437897843, 0.19013565540812039]], [[0.16388462865385933, -0.27567803316085943], [-0.19136893915067305, -3.9464692065449087]], [[40.533983242492255, -0.42000018701295128], [0.56947961401856784, -0.78881005402468019]], [[-0.17526756631674811, -0.27414618194436491], [0.19348410013433689, -0.52983087122652006]], [[-0.26272013742865324, 0.33043552657344361], [-0.50538442608324485, 0.4852386398776527]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[-0.21183748932531277, -1.74666416049158, 0.83219165279681739, -3.1760260397713589], [-4.0884789499008445, 3.5869816796440404, -2.6316110715973728, -0.011923546613902047], [-4.7590395228192737, -4.5154499925167073, -4.6717629662609319, 0.82102456791936262]], [[-2.4469014949671863, -0.36462287671763782, 2.2254558533274151, 3.2500413365094261], [3.9964701321373575, -1.5174568234391161, -0.67466299828040022, 2.4503108956801531], [-4.7726960987812861, 4.7992468355442561, 2.312184982319188, 3.7686116999432357]]], [[[4.1787871835028021, -1.011884288161772, -0.99320721810839085, 3.0221659494124502], [2.3565976628898042, -2.5872696429533972, 3.4574793383108844, 1.5686063067575295], [-1.0822120424366277, -1.1213911364268672, 3.7587721750466923, -3.9244156904015046]], [[1.9168690387269818, -2.9203831799571067, -1.8320512286522153, -0.5694217449827752], [-1.5797540997515558, -2.2375494497612003, 0.47182251494661998, 4.622710897562742], [-1.7239083687904722, 4.9931655201086826, 4.8910590961590188, -4.557210790342026]]], [[[-4.1209259724446099, -4.5153170734566572, 1.6042046853364225, 1.4523079943483816], [4.7127598266371287, -0.54447236810497124, 2.6369861405459183, 0.92347206431267903], [-4.5270302381589778, 1.4509979356052618, -1.3668462431337183, 4.4054770520289512]], [[-3.161183579068684, 1.6541379001439926, 0.051131355548590385, -3.8570949085725328], [-0.52667596276226103, 1.4644487417344791, 2.1674594153405629, 2.7450674537391375], [-1.9498598589119522, 1.405520659527423, 3.886288829448004, 0.1219402296054648]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-4.27805057939)+(1.-msk_arg1)*(0.994692115915)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.049517294242793566, 0.40828506537661624, -0.19452590317785984, 0.74240030145287672], [0.95568738004069931, -0.83846172762027749, 0.61514258019158574, 0.0027871448438078813], [1.1124318038089924, 1.0554924278521098, 1.0920307928955724, -0.19191558226878785]], [[0.57196647154061397, 0.085231081295314801, -0.52020325894408659, -0.75970147528532461], [-0.93418019679076192, 0.35470754617743161, 0.15770337114066238, -0.57276342348161957], [1.1156240465626892, -1.1218303165144443, -0.54047630793737278, -0.88091798589224113]]], [[[-0.97679704948616664, 0.23652929515049237, 0.23216350524063228, -0.70643530115576214], [-0.55085783095735708, 0.60477771240400657, -0.80819038348120054, -0.36666380577959873], [0.25296850103871465, 0.2621266662504515, -0.87861798389040846, 0.9173373754171793]], [[-0.44807068152975366, 0.68264344372760466, 0.42824440586967988, 0.13310308852488761], [0.36926961718531853, 0.52303015315895196, -0.11028913898761414, -1.0805648067441977], [0.40296586886904684, -1.1671590663657332, -1.1432915542700317, 1.0652540697612742]]], [[[0.96327191461821615, 1.0554613578454368, -0.37498497401224529, -0.33947892092354515], [-1.1016138634122425, 0.12727113857132977, -0.61639900969114281, -0.21586282050080371], [1.0581993256382309, -0.33917269295386415, 0.31950212316749638, -1.0297861070765892]], [[0.73893085656735513, -0.38665692923598771, -0.011952022211915003, 0.90160105332850082], [0.12311120520625043, -0.34231683673608387, -0.50664651460211263, -0.64166316007633462], [0.45578232952753361, -0.32854231932132499, -0.90842517107411658, -0.028503690487640707]]]])+(1.-msk_ref)*numpy.array([[[[-0.21296789824294982, -1.7559847238603452, 0.83663240060071531, -3.1929739755188962], [-4.1102959242204467, 3.6061225601887656, -2.6456538958045543, -0.01198717313942944], [-4.784434747871817, -4.5395453731546764, -4.6966924654490478, 0.8254057258349472]], [[-2.4599586704430658, -0.36656858025070382, 2.2373313487858493, 3.2673842332798717], [4.0177961282621535, -1.5255542887688274, -0.6782631404086179, 2.4633862644275917], [-4.79816419816453, 4.8248566151829744, 2.3245232824546576, 3.7887217960659534]]], [[[4.2010860613463024, -1.0172839132545635, -0.99850717846945358, 3.0382928557059277], [2.3691729583292358, -2.6010758520717219, 3.4759291674181889, 1.5769767163723141], [-1.0879869510585192, -1.1273751128458354, 3.7788297654173646, -3.9453571890336572]], [[1.9270978507385883, -2.9359669522159968, -1.8418274351821937, -0.57246029788705599], [-1.58818399630915, -2.2494894791663813, 0.47434025805307528, 4.647378644706583], [-1.7331075025201836, 5.0198100901946736, 4.9171588051227015, -4.5815290152858807]]], [[[-4.1429160908273852, -4.5394117448108666, 1.6127650553058506, 1.4600578119712948], [4.7379080936019582, -0.54737778594330133, 2.6510576472393383, 0.92839990338409406], [-4.5511874133970629, 1.4587407624821529, -1.3741400190711035, 4.4289855941756686]], [[-3.1780523123578113, 1.6629647241366652, 0.051404203100112469, -3.8776771695061827], [-0.52948641527898843, 1.4722633449114306, 2.1790254297392457, 2.7597157048069958], [-1.9602647168040874, 1.4130208101972273, 3.9070268752174897, 0.1225909280413448]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_expandedData_rank4(self):
      arg0=Data(-4.50076585879,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.75834898848841803, -1.4002867519560946, -1.1709692413941819, -0.74170730489047187], [-1.8893054696961187, -0.17661727817214157, -0.35893251035774032, 1.4886417793347366], [-2.634975310962393, 3.9118775696020478, 4.6382533979459488, -0.74862634031944619]], [[-0.54812032157735313, 0.92022980371129659, 0.54407252760868552, 1.7290023601165174], [0.72811891353865565, 2.4723036616819076, 2.9261952718042226, -0.27474093555944012], [4.2932603480978742, 0.81545989063514668, 1.2182151077241663, 1.9203340670596738]]], [[[0.88173837431634894, -4.9599716389785806, 1.241650880310825, 3.1123082388131653], [4.7657942437290082, -3.567255767412032, 1.7503008462485052, -2.1802003458655705], [-0.82174173945158913, -2.6644090094630202, 0.61117466881768223, -0.64350499198238342]], [[-3.2987463143665896, 3.2430677394517016, -0.17740257165377837, -0.17198366281376742], [-0.77598064514557752, -4.5731586503506652, -2.8720572999772074, -1.4234958285027179], [0.76115359347677014, 2.1575054297180127, 1.5184300861880784, -4.6170036734332349]]], [[[-0.55265108586518608, 4.3434832696664358, -4.5207328197828245, -0.75081950456428981], [-4.0630258529005587, 4.300153925339723, -2.6582910599601597, 0.21223125010892474], [-0.83409448228094352, 1.3206730321450566, -3.17548839910922, 0.83593483435000948]], [[1.2989592917129897, 4.8825787521966042, -2.4099287565484651, 0.75917298810940093], [0.73327384672719464, 4.1571936538488679, 2.0628216349259283, -2.8664857120421274], [-3.501381482599105, 4.2380957087093467, -3.2367960802491957, 4.1804586937647645]]]])+(1.-msk_arg1)*numpy.array([[[[4.2278640661360072, -1.3417406042779909, 1.2019674255799151, -3.5649017145122972], [0.98887408417241307, 0.13318848147466866, 1.4728206249694473, 3.2292356871653389], [1.9442531692754494, 4.3853598126730713, -4.7991267545987659, 3.7957931644667795]], [[2.2175444355373743, -4.0987704020288565, -2.4985250297910779, -2.707789385636159], [1.9366702225943673, -2.5196365280716537, 2.4462329518437702, -4.2144565774798686], [-2.8496726550066489, 1.9701125043033016, 2.3959173559341442, 2.5048446210971251]]], [[[1.2234245173377669, -4.7326082333501684, 4.3732667198213058, 0.19792434852267071], [4.9324655228225094, -2.7644427092981241, 1.8368667201497537, -2.9836276190478737], [1.5159566135576705, -1.5018097831750685, 2.5558710474706556, -3.1172493365632583]], [[4.4222610208098558, 3.7610820483031091, -1.3240060107841511, -0.93081824596481599], [0.18303984980930199, -2.5656752378543537, -1.7211055640462702, 1.2853984214699965], [-0.21277160952606256, -4.3192001846692483, 0.35523690986480005, 3.4802923962918122]]], [[[-3.5116729579663932, -1.462870493717964, 0.37634429020857585, -2.2178106879911184], [-1.2037378872280682, 0.02196340440305633, -4.8379053546086856, -2.9865359698305372], [2.0229105242697729, 0.7919887533856409, 0.51697877692991856, -3.239254847643501]], [[-2.6652663271271315, -0.74421473041812369, 2.8884290691656735, -3.9853888366370924], [-1.7419860066619783, 1.2336846395906385, -1.8935071323710426, 4.9142345018366242], [2.8814555146936174, -2.6069423496544939, 3.2702211001699037, -0.36825662153201488]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-5.9349533356201967, 3.21417441999384, 3.8436243239274392, 6.0681158579908665], [2.3822330115401904, 25.483157171105468, 12.539309560752026, -3.0234042341636407], [1.7080865388259323, -1.1505385275264119, -0.97035790687647716, 6.012032460510297]], [[8.2112734770377802, -4.8909151177693904, -8.2723637574118332, -2.6030998931026517], [-6.1813610045093199, -1.8204745349653646, -1.5380948435537842, 16.381853871280128], [-1.0483328505310887, -5.5192976509095688, -3.6945575787528506, -2.3437410896349418]]], [[[-5.1044232505851674, 0.90741766009782932, -3.624823958298852, -1.4461182869566422], [-0.94438946136119861, 1.2616885786293783, -2.5714241459919438, 2.0643817745137869], [5.4771050838883308, 1.6892173246706648, -7.3641236923321332, 6.99414288136051]], [[1.3643867790596549, -1.3878112393527968, 25.37035295958, 26.169729061225979], [5.8001006686824956, 0.98417006776015192, 1.5670877662605327, 3.1617696158109574], [-5.913084950745751, -2.0860971179024062, -2.96409159679681, 0.97482397180913205]]], [[[8.14395551534564, -1.0362111649480767, 0.99558324683513488, 5.9944711497677341], [1.1077374404547491, -1.0466522680203749, 1.6931049901138338, -21.206895103730403], [5.3959904476116298, -3.4079334924279694, 1.4173460246480769, -5.3841109065554855]], [[-3.4649013926049972, -0.9218009759225505, 1.8675929097753086, -5.9285115899631489], [-6.1379058845228291, -1.0826452250129317, -2.181849260542176, 1.5701337145640701], [1.2854257330031296, -1.0619783431375562, 1.3905002808972236, -1.0766201004465177]]]])+(1.-msk_ref)*numpy.array([[[[-1.064548383862054, 3.3544232353432153, -3.7444990296821459, 1.2625217240832891], [-4.5514044010570078, -33.79245569106925, -3.0558818789537958, -1.3937557660104902], [-2.3149073021548969, -1.026316209171015, 0.93783016972424771, -1.1857247388836034]], [[-2.029616988352648, 1.0980770858903681, 1.8013691298381287, 1.6621550710954367], [-2.3239712194078681, 1.7862758412373458, -1.8398762290407502, 1.0679350412206348], [1.5793974970725322, -2.2845222539131904, -1.8785146522880525, -1.7968243702167033]]], [[[-3.6788259471760716, 0.95101171212041979, -1.0291542106028793, -22.739829093225541], [-0.9124779155512599, 1.6280915656726549, -2.4502408418758908, 1.5084877985644234], [-2.9689278825935999, 2.9968947527283998, -1.7609518536733662, 1.4438260699914356]], [[-1.0177521945480519, -1.1966678208533508, 3.399354551364862, 4.8352789368974864], [-24.588994492087988, 1.7542227451037335, 2.6150434655573167, -3.5014558782843461], [21.153037610697947, 1.0420368740418069, -12.669758501464528, -1.2932148642421604]]], [[[1.2816586033683297, 3.0766673318802136, -11.959171364866847, 2.0293733289157831], [3.7389916081795587, -204.92113955554322, 0.93031292034392921, 1.5070188018023176], [-2.2248961606535804, -5.6828658734778692, -8.7059006281096547, 1.3894448169361433]], [[1.6886739658940115, 6.04767102132588, -1.5582054296710117, 1.1293166221119157], [2.5836980558862117, -3.6482304426575207, 2.3769468737927046, -0.91586306211301938], [-1.5619765204919713, 1.7264539276789297, -1.3762879392343541, 12.221819230485334]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([-4.1702418010911515, 0.64455120410923428]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-1.2890928384546632, 1.4292639528115094])+(1.-msk_arg1)*numpy.array([3.3919739781249767, 4.8459158873478234])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([3.2350205328037878, 0.45096722886023655])+(1.-msk_ref)*numpy.array([-1.2294439249785718, 0.13300916051640302])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[3.3939883826545714, 1.2873010778142424], [-2.3971038592917537, 2.1949588431489273]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.9409488331589939, 1.7527229169989278])+(1.-msk_arg1)*numpy.array([-3.7960636616264654, -0.21138176954591614])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.1540453694357371, -0.43771624426103989], [-1.3676456421281677, 1.2523136554334617]])+(1.-msk_ref)*numpy.array([[-0.8940809968398632, -0.33911472318740937], [11.340163650068494, -10.383860670028787]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_expandedData_rank2(self):
      arg0=Data(numpy.array([0.45623391123536994, -3.4465151251394111]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.1445966010048609, -3.1493598763322792], [4.3031091854540104, -0.4760521072667494]])+(1.-msk_arg1)*numpy.array([[-3.9598013557993972, 4.5793691634042553], [4.7099751951244109, -3.8924119110614153]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.11007920798003734, -0.14486560099530338], [-0.80093601547221205, 7.2397854615696566]])+(1.-msk_ref)*numpy.array([[-0.11521636320649885, 0.099628113601614596], [-0.73174804162601825, 0.8854446044996267]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_expandedData_rank1(self):
      arg0=Data(numpy.array([[[-0.92125789006553482, -3.1568509326356908, 3.0935659405930078, 4.7815423224716085, 0.31484505865106094], [1.6186219765291687, -1.8796378503697131, -4.9493316830351377, 2.6745632626364015, -0.99266605180886902], [0.42355354593764893, 1.476062939783561, -2.7459050850900066, 4.1885055061821834, 0.34667735862418425], [-4.746454696027115, 4.092562943894487, -3.2850504675923711, 3.4357655849351012, -1.7494718744561819]], [[3.0993648418101412, 2.6282229362873233, 0.93259466948845393, 0.73600892014836194, -2.7693004994941872], [0.46507036158204862, -1.0963575563450734, 0.2452797124429642, 3.6467541867424185, 0.34361987623521273], [-1.6340610439572325, 1.7148068368642413, 3.5452162537811009, 2.3862413555258453, 4.3114038928783653], [2.2650995736371105, 0.71309269832472655, -0.70370060690044145, -2.4469730923979305, 4.8664590273476325]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.32919656862285507, 0.16274493095049003])+(1.-msk_arg1)*numpy.array([1.6362705820520347, 3.2375894421601732])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.7985039270593868, -9.5895620839606792, 9.3973213437019751, 14.524885063275356, 0.95640443631647942], [4.916885930191901, -5.7097735199151645, -15.034578591568957, 8.1245174390032062, -3.0154204096401731], [1.2866280706069393, 4.4838345246381239, -8.3412324028075151, 12.723417876754226, 1.0531013736700157], [-14.418299424818439, 12.431973276681212, -9.9789936490981415, 10.436820770362571, -5.3143684995710547]], [[19.044309544443042, 16.149338237065439, 5.7304068645441637, 4.5224690922770998, -17.016201262432308], [2.8576641918483561, -6.7366617807506719, 1.507141949125179, 22.407789695470314, 2.1114014072717766], [-10.040626361839458, 10.536775719213749, 21.78388127406328, 14.662461937151107, 26.491786058700487], [13.918096007095883, 4.3816584280690272, -4.323947927536496, -15.035633233592659, 29.902369302230973]]])+(1.-msk_ref)*numpy.array([[[-0.56302294997578717, -1.929296393434335, 1.8906200322402607, 2.9222198179931294, 0.19241625566366666], [0.98921412771429718, -1.1487328996726649, -3.0247635918676834, 1.6345482782451859, -0.60666375274190532], [0.25885299814317603, 0.90208976190994128, -1.6781485380287089, 2.5597878200128794, 0.2118704341609679], [-2.9007761601841069, 2.5011529198074531, -2.0076450090990536, 2.09975392983374, -1.0691825017486913]], [[0.95730632224393275, 0.81178388527661172, 0.2880521715768295, 0.22733238210008636, -0.85535876273628575], [0.14364710840907177, -0.33863390523461973, 0.075759980326384285, 1.126379441214524, 0.10613448133990203], [-0.50471533625553211, 0.52965543269133397, 1.0950172395594651, 0.73704260473919314, 1.3316709761697658], [0.69962532745529293, 0.22025420797299713, -0.21735325601720543, -0.75580092414845212, 1.503111841166819]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_expandedData_rank3(self):
      arg0=Data(numpy.array([-2.2893146129191457, -2.6517719323311093]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.2008072325515569, -4.8003819672087538, -1.9906722652858679, 2.8106848125744168, 3.6566047901557965], [-2.9974080809370287, 0.64705341631146851, 0.42055082997286064, -0.46891623129025106, -4.7260742660188049], [4.8214373090652192, 2.9059737883528083, 2.524979752580176, -1.1898545774520319, 0.381572790693542], [-1.504229210823965, 4.5404986459557115, -4.078527697598, -3.840649590711458, -4.8523657310983381]], [[4.5928539149231309, 2.2004668523765361, 1.2719562771974635, -2.3000722748074978, -4.4844653521602957], [-4.4063628052207351, -4.2229338206957934, -4.6266288880412398, 1.6495022061324498, -4.8442642888048946], [-2.8676790207474667, 1.9827734462509579, 1.8138376061915897, 4.7756354160608883, 2.2382905845092482], [-3.0984478862806641, 4.2489073984640626, 2.8219152410204282, -2.110276577850303, 3.6058526583565094]]])+(1.-msk_arg1)*numpy.array([[[-2.7886886086919835, -1.5704039723762242, 2.4983675897089554, -1.9468712473555092, -1.8065404696839469], [-1.7802322846767247, -2.5648840204682855, 2.9198146481714682, -2.409406675368635, -4.4876123327071404], [0.08278077954539409, -2.9936163978999888, 1.9237761395893918, 2.2080524070317828, 2.7886811285944972], [-4.7350953375758973, 2.427218931281554, 4.0057095037482711, -2.4902277595751876, 0.20700247756265799]], [[-3.5124628566166516, -1.506160635857261, 1.2428539769735467, -1.7397161363423965, -4.8018145213213099], [3.2177301845162862, 3.070121502802035, -4.0037630655469902, 0.10355395373554455, -4.8466780377115768], [-1.5381232766684603, 1.3145588692380583, 1.8202887138655521, 4.7373889845140624, 0.23483693107550074], [0.60259934452788677, -2.020724037884587, 3.2972820978171598, 4.3750591945734012, -2.3068664081047685]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.0402158712760026, 0.47690259411800479, 1.1500208511672771, -0.81450421003352291, -0.62607657767182578], [0.76376474310547537, -3.538061240707755, -5.4436097844983014, 4.882139836831751, 0.48440089682459436], [-0.4748199481127337, -0.78779602971463714, -0.90666652300074346, 1.9240289160558679, -5.9996799267529424], [1.5219187318301963, -0.50419894188455971, 0.56130907588721546, 0.59607484589477056, 0.47179350028114164]], [[-0.57736910022653987, -1.2050951503619141, -2.0847980232259493, 1.1529080896177695, 0.59132398716241052], [0.60180517346171403, 0.62794541542073923, 0.57315423313620928, -1.6076195124034776, 0.54740447139916792], [0.92471016217146895, -1.3374054092488976, -1.4619676663882175, -0.55527101658827716, -1.1847308614366163], [0.8558388037031861, -0.62410678408517406, -0.93970644255502389, 1.2565992345100172, -0.73540773391992809]]])+(1.-msk_ref)*numpy.array([[[0.82092873538610467, 1.4577870746564126, -0.91632417197096161, 1.1758942025718377, 1.2672368271493302], [1.2859639905557978, 0.89256067512213377, -0.78406162334750507, 0.95015699770520667, 0.51014090415829716], [-27.65514682866408, 0.76473211949436537, -1.1900109195697655, -1.036802661761367, -0.82093093736857836], [0.48347803997778555, -0.94318422760092935, -0.57151288948361345, 0.91931936912858281, -11.059358515292111]], [[0.75496084672775865, 1.7606169416463344, -2.1336150356040986, 1.5242555247583274, 0.55224372381660092], [-0.82411258255631026, -0.86373517462123017, 0.66231989478848619, -25.607635794410982, 0.54713185231160488], [1.7240308189567135, -2.0172333049398721, -1.4567864493868257, -0.55975389418082899, -11.291971497781825], [-4.4005556202665135, 1.3122880129179542, -0.80422962114361218, -0.60611109802130936, 1.1495125695248654]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_expandedData_rank1(self):
      arg0=Data(numpy.array([[[[3.8876113630414615, 1.3502241959260814], [-0.83398446384828517, -0.8243261771454975]], [[-0.83655375834464163, -3.1074566222628319], [-0.14660612976344556, 2.0961648312871901]], [[0.30828208753741748, 2.6972740320018413], [1.6593320188381986, -2.8466908754154896]], [[-4.1476693824946498, 4.5358808919775395], [-1.2172568497704281, 4.6106711509524843]], [[3.4782973798246406, 0.022964500169462276], [0.82812159877558056, -1.4308770863234277]], [[-0.4888147946084942, -3.3331390978262556], [1.5433071452632721, -3.3576909006985121]]], [[[3.1760101706430195, 3.2303487718482291], [-0.16203505346575309, 3.3018754525173648]], [[2.1117096602831236, 1.696986346578722], [-0.48740994841355612, -3.2635398953687709]], [[4.4505669334755389, 1.519397321873134], [0.52669149280075089, -1.3118519813427945]], [[0.68903234323556628, -4.2209238929308022], [0.21644537534321984, -0.0026047135525217868]], [[-2.1982847202186582, -3.7106426573798004], [1.7271676910150067, -4.7921106093816457]], [[2.5332355183669941, 2.483654628195735], [0.40199524913523277, -0.65280014796009489]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.6648157054617201, 2.0228689687918866])+(1.-msk_arg1)*numpy.array([0.6150461047798288, 2.2732119569025926])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.83339012910836285, 0.28944856156807963], [-0.17878186760343578, -0.1767114135249436]], [[-0.17933264916879285, -0.66614777913402223], [-0.031428064691129014, 0.44935640840707408]], [[0.066086659581528734, 0.57821663326244244], [0.35571223465385737, -0.61024723272186077]], [[-0.88913895947452359, 0.972360148476344], [-0.26094425302702179, 0.98839299172187201]], [[0.74564518717258976, 0.0049229169209353075], [0.17752504087267337, -0.30673818145658138]], [[-0.10478758979399201, -0.71452749867989562], [0.33083989651645124, -0.7197906868576216]]], [[[1.5700523462673022, 1.5969144920827387], [-0.080101606167069198, 1.6322735201624732]], [[1.0439181641825743, 0.83890077546258934], [-0.24094983705477022, -1.6133224374477637]], [[2.2001261585091894, 0.75111010417079072], [0.26036856609417747, -0.64851060626347379]], [[0.34062134219552315, -2.0866027202205069], [0.10699920690982126, -0.001287633352780825]], [[-1.086716319313324, -1.8343465220072555], [0.85382084438544681, -2.3689673841028007]], [[1.2522983729786079, 1.2277881892068583], [0.19872530318922013, -0.32271005093818073]]]])+(1.-msk_ref)*numpy.array([[[[6.3208454339095921, 2.1953219204752594], [-1.3559706457239182, -1.3402672917351224]], [[-1.3601480471843765, -5.0523962319462603], [-0.23836608121585764, 3.4081426010129192]], [[0.5012341109741274, 4.3854826671366345], [2.6978985899474939, -4.6284186718563713]], [[-6.7436723040127413, 7.3748632122485711], [-1.9791310607619828, 7.4964642733620597]], [[5.6553441324041627, 0.037337851570790774], [1.3464382464011011, -2.326455001020852]], [[-0.79476122328012744, -5.4193320987203659], [2.5092544010432154, -5.4592507368215619]]], [[[1.3971465181674265, 1.4210504049300361], [-0.071280222230810791, 1.4525154341596886]], [[0.92895414080105099, 0.74651478997628651], [-0.2144146510111119, -1.4356513854588238]], [[1.9578319214631186, 0.66839227959341607], [0.23169484534931103, -0.57709180059490928]], [[0.30310958955821266, -1.8568105275506737], [0.095215659360749411, -0.0011458296023002131]], [[-0.96703904514648609, -1.6323346558654415], [0.75979175006997202, -2.1080790969933245]], [[1.114385972973106, 1.0925750327214911], [0.17684019649578955, -0.28717082275494449]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_expandedData_rank4(self):
      arg0=Data(numpy.array([3.9286081307035197, -2.7147092351673798]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.066882747317055902, -1.7264313618319838], [-3.2521832617581214, -3.3114371485347105]], [[-3.0185326927041944, -1.3103954534668771], [2.6178434744409902, 3.0784104245223975]], [[-2.1400969013982705, 3.870175823510337], [-4.0858191098960495, -2.1551366273211361]], [[2.4169152740351869, -3.0911696660122612], [-4.9119014615500518, -0.73377475510720469]], [[0.25562662799750413, 0.84687262103899297], [-4.8742764566126349, -0.82677340988247217]], [[-1.9871844670376229, -0.24559667584941902], [-1.6045070879169501, 0.88153598838830227]]], [[[4.4842481760556332, -2.8872002843095244], [3.8275381701659974, 0.22730280999799124]], [[3.620320836646659, 3.3322561376047037], [1.774628445681353, -1.4694600170010075]], [[-1.4814365139493035, 2.7611699713422233], [3.891629010545488, 1.7527883538335844]], [[1.3996294637830733, 2.2457681045365083], [1.1419029460715659, -1.3441271977805478]], [[-0.72540660851483096, -2.1046360668919952], [3.1045395011766779, -0.69751031107940342]], [[2.2452533593382391, 1.7665346389031757], [0.78612458444258326, -2.0646363579860836]]]])+(1.-msk_arg1)*numpy.array([[[[2.8894217995579119, 4.1794692806477798], [1.2679037236007904, 4.0502016758100119]], [[-3.5945313458507391, 3.8688246659069456], [2.1164640826180587, 0.76017053027119985]], [[-3.1764962616873014, 0.52421486262324546], [-3.9465926306286381, -4.641028666146604]], [[-3.435226136943287, -0.4850513889477428], [3.1454648854024878, -4.7699139909185986]], [[4.284622640314506, -1.2658078583962187], [1.0759643095185547, 2.2852647655245883]], [[-0.77724449192676648, -2.8288130137511089], [-2.5304826782028056, 2.5260720412335322]]], [[[4.4868663304757526, 3.5724925833919237], [3.3566257669454629, 3.347284517536421]], [[-1.6089148539475318, 3.7768157171586054], [-3.3430339656722463, -2.3387677722604518]], [[-3.1798751199496844, -1.825587670995299], [4.7464307640582462, 4.6028155536056872]], [[4.9140664461776069, 2.0753972625651151], [-1.9596688829260769, 3.9199329096916795]], [[-0.18521724005903017, -0.38487093916326121], [-2.7708623523899045, -0.52446421816675493]], [[1.132705386667066, -3.7235637841809588], [4.7109614375591811, 2.2128024136333346]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[58.738737391872618, -2.2755657812742234], [-1.2079910061955503, -1.18637556881969]], [[-1.301495968620443, -2.9980324796684155], [1.5007039836644274, 1.2761807520558364]], [[-1.8357150688535147, 1.015098101444958], [-0.96152277549151477, -1.8229044418343132]], [[1.625463735906832, -1.2709131348883833], [-0.7998141170901597, -5.353970143234962]], [[15.368540286585002, 4.6389599015300238], [-0.80598795855614946, -4.7517349781023812]], [[-1.9769720405272977, -15.996177949542933], [-2.448482876946235, 4.4565487767392566]]], [[[-0.60538782167833793, 0.94025663890394229], [-0.70925726001307121, -11.943139793086459]], [[-0.7498532195510863, -0.81467604021543505], [-1.5297338672631786, 1.8474195988726372]], [[1.8324843552899495, -0.9831735327208927], [-0.69757657469689283, -1.5487946558008243]], [[-1.9395913742982829, -1.2088110209079908], [-2.3773554876153566, 2.0196817977122752]], [[3.7423276867098978, 1.2898710983207207], [-0.87443217718391242, 3.8919987160710829]], [[-1.2090881520682846, -1.5367427138892202], [-3.4532811832774528, 1.3148607136878085]]]])+(1.-msk_ref)*numpy.array([[[[1.3596520007236761, 0.93997775002060091], [3.0985066591227026, 0.96997839741346348]], [[-1.0929402897650105, 1.0154526167400144], [1.856212993628432, 5.1680616049426993]], [[-1.2367740450658391, 7.4942707863035549], [-0.9954430310882495, -0.84649512280763406]], [[-1.1436243129540378, -8.0993647688054953], [1.2489753578033735, -0.82362242551609222]], [[0.91690878299031398, -3.1036370209307083], [3.6512439083238681, 1.7191041449421278]], [[-5.0545332537058121, -1.3887832499377688], [-1.5525133463840535, 1.5552241054792326]]], [[[-0.60503456872082328, -0.75989219621832871], [-0.80876136443347735, -0.81101837054633996]], [[1.6872920456335778, -0.71878255082292564], [0.81204955230584464, 1.1607433911848268]], [[0.85371567522762815, 1.4870330679256492], [-0.57194750542327011, -0.58979318279238235]], [[-0.55243641185987813, -1.3080431800377814], [1.3852897593158267, -0.69253971884455134]], [[14.656892815712949, 7.0535573329318257], [0.97973442557545598, 5.1761571926804546]], [[-2.3966595966805526, 0.72906210085629342], [-0.5762537586327412, -1.2268195381755453]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-4.8405095914372467, 3.5890733965767421, 4.5161802046673891, -2.6142687644297569, -2.8547615208897126], [-0.87864495610239501, 0.84672953972527587, -1.2334509616515552, 4.9858073509288836, 3.3489576296816086], [-4.7979651107922496, -1.0904827823658705, -4.5443932090661718, 2.6667067755830853, 1.9540600623722746], [1.4496405550472495, -1.5876307798462164, 4.7226407451453962, 2.2910459811183976, 4.0587171413346717]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.7999087146314663, -0.16101156796046734, -3.4337594276514305, 3.1386517226592758, -3.6754663985435378], [4.1046977167380021, 2.9556832185980619, 3.4768298925515069, 0.79751651541450475, -1.5163028446602924], [3.9422179706477447, -2.3883529253969646, -4.9121169706357222, 1.267566849818996, 4.7377210975379462], [2.9572734090342356, -0.76542379570919383, 4.9133638876667227, -1.302861720810701, -2.861711941545706]])+(1.-msk_arg1)*numpy.array([[0.24716830980496685, 4.481705902750674, -0.13108527390412572, -3.4258774735674793, 1.3095356872807296], [-1.2043346179540304, 3.6970695568551832, -3.7039230073984228, -2.9496133007565648, 3.0731024123845589], [0.36541118331141753, 4.3005446976247548, -1.5715700989176029, 3.8143467065614249, 1.5465391148495495], [-2.7437900620698619, -0.41107704023399716, -0.62206239159591892, 3.3701927367373017, -3.6150070490525978]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.7288097880270969, -22.290779737378596, -1.3152290659326404, -0.83292731893641692, 0.77670728319566662], [-0.21405838303743668, 0.28647506417379065, -0.35476310310550591, 6.2516665856600326, -2.2086337445550979], [-1.2170725075366386, 0.45658360235207823, 0.92513945336241454, 2.1037997135724096, 0.41244725515559411], [0.49019497169883336, 2.0741852928353461, 0.96118277683440667, -1.7584720960968925, -1.4182829104533923]])+(1.-msk_ref)*numpy.array([[-19.583860063843737, 0.8008275139995078, -34.452231514353564, 0.7630946478968591, -2.1799799338173576], [0.72956879508709183, 0.22902721377131041, 0.33301204133773604, -1.6903257622448484, 1.0897644075203472], [-13.130318200204723, -0.25356852655621925, 2.8916261592124077, 0.69912542847660553, 1.2635051022051709], [-0.52833508477455049, 3.8621246736195487, -7.5919084788735196, 0.67979672383258682, -1.122741141652368]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_expandedData_rank2(self):
      arg0=Data(numpy.array([[[-0.041967768494254543, 3.8302858526052201], [2.8847537747783907, -0.87367740884449674], [1.9904178211854751, -0.27542359063997157], [-0.39541240994303006, 2.7989149916961811], [-2.2126153796612491, -0.50402187713444935]], [[-1.9132691153881165, -3.5865573583765711], [4.8678344949799381, 3.9222007520578188], [-3.1223799569024759, -0.71831858728478259], [0.62962074165089454, 0.99589969887500729], [-0.89102989329313154, 2.942435989344383]], [[-4.8252695077540189, 3.1918814885655795], [3.6592704852539413, 4.637338261278444], [2.6628524746705917, 2.1066368098694808], [1.0482741801089688, -1.6102753999952335], [-2.9102507109431386, -0.30756155096788795]], [[-0.41404759268513391, -1.2117616675418308], [-1.9458721281995359, 3.8448894082720155], [-3.2794228376211718, 3.2994966347295254], [-4.5795365295640602, -3.2310188432094531], [-3.4383765244429663, 2.8429964851965899]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-3.5222168163598533, 4.7844334446350931, 4.7402125752059554, 4.8130733610341441, 3.5941095835770316], [2.6923912843753186, -2.5592630113129875, 2.0747656694942762, -0.30950192294278445, -4.2559922893244915], [1.3373651175917836, 3.1575430136893488, 0.83283523083911426, 3.9272066498901577, -3.5026438146505057], [1.7607951633038645, 1.5477319236615008, -4.6448402087801419, 0.82942846739176979, -2.293116616624137]])+(1.-msk_arg1)*numpy.array([[0.86017744628095727, -4.1718686726266805, -2.9418267908059503, -3.4238439417973767, -1.1488088154719822], [4.9330228414707538, -2.2856949595207041, 0.94958031262292497, 2.9546632202789658, -0.84024883350449908], [-1.8482034607844069, 3.4484986601200145, -2.9613284816978283, -3.4071461884784027, -2.7198649010530094], [0.78641749110962689, -3.8074093108948324, 0.74580204167896724, 0.16196283666953004, -2.8396873657616286]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.011915157607369402, -1.0874645293879865], [0.60294574230374931, -0.18260833157250278], [0.41990054024085499, -0.058103636972020145], [-0.082153829846896662, 0.58152344287035795], [-0.61562268155973898, -0.14023553411880738]], [[-0.71062075059124552, -1.332108516020811], [-1.9020454222415288, -1.5325508690275638], [-1.5049313774618005, -0.34621673080790499], [-2.034303165758645, -3.2177496327191242], [0.20935890685895844, -0.69136309215715352]], [[-3.6080419956241756, 2.3866941395280712], [1.1588980639026552, 1.4686540266192818], [3.1973340897066316, 2.5294760978674753], [0.26692615733330166, -0.41003072757586417], [0.83087258223928884, 0.087808400523470442]], [[-0.23514807475290683, -0.6881900250499009], [-1.2572410625195007, 2.4842088926976982], [0.70603566327687195, -0.71035740443610662], [-5.521315833256752, -3.8954761865959964], [1.4994337834875791, -1.2397958588699998]]])+(1.-msk_ref)*numpy.array([[[-0.048789663895171149, 4.4529019787321245], [-0.69147760899244703, 0.20942111974354513], [-0.67659245860636652, 0.093623319870751412], [0.11548786003822793, -0.81747738485617605], [1.9260083574064562, 0.43873433973204196]], [[-0.38784923096315643, -0.72705062871901449], [-2.1296955985765891, -1.715977337973515], [-3.2881683785943885, -0.75645901429932494], [0.2130939111197413, 0.3370603092899972], [1.0604357397044344, -3.5018626292786492]], [[2.6107891312497045, -1.7270184567292686], [1.0611198802456809, 1.3447412101117231], [-0.8992087474010616, -0.71138234845925497], [-0.307669269858103, 0.472617055716757], [1.0699982597725426, 0.1130797161464946]], [[-0.52649845325911693, -1.5408630673156158], [0.51107510890186103, -1.0098439895258791], [-4.3971759989265484, 4.424091716485008], [-28.275230440105062, -19.949137157939777], [1.2108292503955851, -1.001165311179977]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_expandedData_rank3(self):
      arg0=Data(numpy.array([[2.4695760757129479, -0.96338597107431756, -3.8192229422657995, 1.0892026584194712, 2.2086969603699744], [-1.4062660385253736, -4.203697454269709, -0.40676810323842716, 4.4304778867191192, -0.29518601053275173], [1.7256703174813897, 1.966447367475455, 3.6954637328308788, -0.95612999202379889, 2.0027869570327557], [0.17545188607092843, 0.17703123206359273, -4.08940906789854, 1.8468881672821249, -1.1824650206363763]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.815322320617013, -3.6115243058837043], [3.0573312093175016, -3.1932468529886049], [0.9535931857463833, 3.2175544281735604], [1.2059581159997315, -2.0341919383096796], [-3.8813113413879794, 1.0765085697003505]], [[-2.7681975065641984, 3.8275967032359421], [2.5115198081486572, -3.3005759532031309], [-4.8518351239679909, 4.1790257332450391], [-4.3538624313827237, -3.0394485292058713], [4.1222296322296366, -0.13397185193511874]], [[-0.00013338099467219422, 3.901776119397395], [-1.9274433494479313, -3.4060037763062212], [4.7671514342802297, 4.1417431042681283], [2.9539423031291818, -2.0609907959861715], [-1.0683132076054025, -0.57630922637223048]], [[-1.3456775392090083, -0.75290095103673593], [-0.076171458927356994, -4.1479856942673159], [-1.4143473916414893, 1.7540517947297136], [-1.8018418560032092, 1.970581688971631], [3.093864100216134, -3.6302429087423507]]])+(1.-msk_arg1)*numpy.array([[[4.2249922730046521, -2.8263743954171003], [-0.6389347329995303, 1.6797362706228451], [-2.6774579601888493, 2.4992943105722194], [1.607352900792451, -1.1427545058556268], [0.17426326939191661, -2.2974120815201546]], [[4.5950634461410758, -1.7454979184580068], [-2.9101730051545296, 2.4757713761332187], [4.4359803834519891, 2.0420076901752164], [0.58028603401799739, 4.2726115364079256], [-4.9357689400959162, 4.2809597528074601]], [[1.7574057603258639, 3.6536398490175888], [4.1114574783690134, -1.3723452100775733], [0.24772355278422165, 1.0512326370548823], [3.225451597105927, -1.7609788704975706], [0.094378278735262278, 3.3289029576197713]], [[-4.6655813901198542, 1.5533613746884214], [-0.27054305199352768, -3.7320832125837553], [0.18225145374688978, 0.82058659860408056], [-0.30471126461167231, 4.2729066395547424], [-3.959440291701287, 1.4409053204501365]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.87719123939304733, -0.6838043625207354], [-0.31510683832301489, 0.30169480012880023], [-4.0050862352549954, -1.1869955979062567], [0.90318448374679183, -0.53544733803465405], [-0.56905946627310022, 2.0517225989058052]], [[0.5080078409111739, -0.36740183137279914], [-1.6737663946072654, 1.2736254259472866], [0.083837989718364286, -0.097335630169132561], [-1.0175971235986121, -1.4576584680237001], [-0.071608337445551562, 2.2033435103644567]], [[-12937.90259791141, 0.44227814838025836], [-1.020236142368955, -0.57734738321636525], [0.77519327501473423, 0.89224841806886768], [-0.3236793051140327, 0.46391764285696219], [-1.8747188958956642, -3.4751950261840543]], [[-0.13038181953609732, -0.23303448591655146], [-2.324115023613023, -0.042678843446412325], [2.8913752675376156, -2.3314072481700507], [-1.0250001470044858, 0.93722994464946197], [-0.38219682000698429, 0.32572614295003899]]])+(1.-msk_ref)*numpy.array([[[0.5845161165127245, -0.87376112652212934], [1.5078002827481689, -0.57353406479523994], [1.4264361939772223, -1.5281205283068], [0.67763753552967532, -0.95313792493335292], [12.674483659563586, -0.96138475902351872]], [[-0.30603843777311773, 0.80565323146743439], [1.444483694551518, -1.6979344275460726], [-0.091697453116753536, -0.19920008391521973], [7.6349896895531861, 1.0369484445206352], [0.059805475927934226, -0.068953231886650715]], [[0.98194188071934518, 0.47231538651665339], [0.47828473912748115, -1.4329101402731601], [14.917692287619474, 3.5153624445907945], [-0.29643290659878369, 0.54295369924208181], [21.220846405248757, 0.60163572880622096]], [[-0.037605578254936677, 0.11294981897314214], [-0.65435512299841991, -0.047434963793594617], [-22.438279551821285, -4.9835191981627913], [-6.0611089308950277, 0.43223227724783136], [0.29864448849367453, -0.82064033205663789]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[2.8294113744888971, -0.15261616977882841, 0.93227461287473901, 4.022226104560918, 4.1486708887344204], [1.9248427878583039, 0.61163798504771449, -0.56873285588587663, -4.0490375539837578, 2.2232290686813592], [1.3151418691594827, -0.56322505829395908, -2.1950433324771379, 1.4835504594617186, -4.0053185301886476], [-3.6271508431530854, -0.99723816108677976, -2.2466990894389474, 2.0486892053650578, -1.4551923430218858]], [[-1.6196831232714759, 3.5937811465112741, 3.4125706868281487, 2.9089293664058733, -1.5183129129518766], [-1.7698831793714853, -0.81746480505016006, -4.1252522036876007, -2.4605713600366852, 4.395743083900733], [1.9068262839592691, -4.8438212840442798, -4.2394949403416424, 2.2936565660697097, -1.4123357406725545], [4.6413328033828378, 1.4147773840693665, 3.7308347642734088, 1.0376073241127335, 0.062174592115487926]], [[-0.80941983420180641, 3.8737762309633137, 3.8002697314624712, 2.7501125991421196, 1.5694098415485076], [0.81573430437641559, 3.3653185030068862, 2.2637166706138823, 0.40764607210197479, 0.24602993243744464], [-3.5963465170161433, -4.9514083507504605, 4.9382328131267279, -3.9914611098347317, -4.6083613337457709], [1.6831085187688499, -1.1009369070899746, 1.1477325008565886, -4.5233336999412437, -4.8345391085573697]], [[-0.37417739541050299, -3.2024149133870838, -3.4195357595495057, 2.3073199774326794, 2.5986993592536916], [-1.2237765453988567, 2.548082691374872, -0.43310400558990203, -2.282233891061253, -1.7638212961663102], [4.4251270227838564, -3.585957491152556, 0.26176322993476386, 1.2148725190618368, -4.8421994656824205], [-1.6815512213345238, 2.4766473609457318, -4.9686773000801256, -2.134564513117474, 2.897155018952108]], [[-0.10739632329628357, 3.5156481432177298, -4.3095716547429355, 2.7975529367384464, -0.93507532142239747], [1.5793992677661421, 1.1611766524483569, 2.7729138422683031, -0.93688980175735814, 1.2045060364507538], [-4.7963688079512554, 3.7160902395592874, 1.2459097721645591, -4.8746431779962016, 1.8905341029396787], [-1.0689316637578514, 0.26694821855874373, -1.5035603431854163, 0.29025974012649058, -1.4578960236716432]]], [[[1.7444266451412389, -0.22698203264911232, 3.2367895124259309, 2.4762572769439615, 2.9163185556459847], [0.47327460603946392, 4.6444707675032042, -1.4874789347645176, 0.89005412398751638, 3.0345139494674171], [4.7904264835414203, -3.9393635792206294, 2.200111520547062, 0.45092332380655442, 4.7764955204666713], [-4.0686783934143769, -2.9062189682007111, -2.6229254022941086, 1.185208242355146, -0.82548122864124451]], [[2.9877328882816911, 4.1241858970493581, -4.0861387841362653, -2.1361900671352507, 3.1656260456244869], [-2.7748226567434164, 0.53551851162911035, -0.034887072695840793, -1.9579666652378269, -0.75306921234883895], [0.13180779268132703, 1.1474595627023199, 1.9484225779421429, -3.6645500036702106, -1.8883179776524726], [0.95586087742702475, -0.34491660240279476, 1.075724618049696, 3.4056863472267764, -2.1058600088681381]], [[4.0788337095179337, -2.1092490070725467, -1.4616216999846432, -0.69030635547431274, 2.2478214399103136], [-4.9812264023456398, 0.95627152641481228, -0.1140971292294175, -4.3057746207160443, -2.602930487511431], [1.3354709894810401, 2.9111518197563377, -0.14381850779463257, 2.6458118591223165, -0.88375206055853894], [0.82873849947732126, 1.2738827871528455, 4.5933255768095655, -2.042785119695937, -1.7604949229591793]], [[1.6324941463221352, 3.0293730479382539, -1.3842583230127534, -0.12188933734052654, 4.0415386738497894], [4.6670926350268722, -1.23270771211783, 2.6479297768097947, 1.7127997655498053, 3.6352939085451048], [4.1915493181525108, -1.3652694098381, -2.5077634093092547, -4.8146970309974577, 3.3882179674747466], [0.23261704816411299, -2.7422245840400574, 2.2809057969799715, 3.2143333396773119, -0.32330921381520028]], [[2.7168660789276728, 3.1144779931644919, 3.6774339385577246, 4.8044369205528721, -0.74912715856674339], [0.25206915417894571, -2.6908560704153048, -2.697352858308788, -3.540290926732145, 4.5561108839497173], [0.85360729227675325, 2.3625874596961296, -3.6310861907171299, -0.28413978165736964, 0.14135894544538452], [-4.7409230046826867, 3.5625699566071152, -0.33497446619513127, -2.7041736338265023, -0.684502157316766]]], [[[4.4772148443486657, 0.33632123083782606, -1.7264490407196718, 1.4119077426792126, -4.5278865011174627], [-3.4113186055709468, -4.2455411325745001, 0.44757709125059009, -2.6202609611422423, -3.8528250828855337], [3.2213812242087272, -1.9081415645715638, -4.8356832002809895, -3.724348776734753, 4.2311616703450401], [-1.8531573593474668, -3.9447759556482884, -0.66775582854124327, 2.9580953993728603, 2.0810005014188686]], [[0.28716872042059283, -2.4594939539338423, 0.91342215103824742, -2.225242200902573, -0.93314681487334106], [-2.2446905906883052, -1.4108593982904596, -4.0466818977992878, 3.0453753492870899, -3.2905566202285739], [1.2111817930543731, 3.6594285653757499, 3.825387651575344, -2.3567594525418412, 1.9645383441198483], [-3.7033867387957695, -2.366875690997662, -1.7256704445086313, -3.3205932107197098, 0.47087024671479227]], [[2.7345829017206311, 2.3187063722990864, 2.4984918346623459, -1.8194864126688195, 4.1546703465128552], [-2.2766356662055331, 1.4334326139833138, -1.113873675310689, -0.81152199445860873, -0.8684999613518567], [3.2471969097894, -1.0814973490205295, 2.335495553996072, 1.7837755590774371, -2.6165282779223489], [-2.5861900570293961, 1.3463191552206233, -4.9724242125468479, 0.63388463852443167, -3.5733685459237341]], [[4.4790474311261033, 0.14566980078935199, -0.87083943119343754, 0.33649672299743649, -1.0084439849958606], [-3.7580566037626495, 3.5250138242251339, 3.8112881303591646, -4.6545973275426453, 3.6806981634345632], [1.5766627867039507, -0.97658017104835082, -1.2728204692015623, -4.4828335546971125, 0.32387296874643834], [2.4704370528817154, -4.3290524833250901, -1.3001127184005323, 0.059644433126879726, 1.9613815816517715]], [[0.87360408386849109, 3.8684112671954889, -2.5718906506074566, -1.2163102202854601, -1.2284663347681848], [-3.3534591054230733, -2.5492619958160057, -1.9735317726405235, -4.2340057319094537, -2.6448628921067483], [-1.0663694842659357, 2.0248326924205218, -2.0417323399312703, -4.4130721747072332, -0.8564378867080702], [1.7042778831067471, -1.7410311382801211, -0.41268638815378811, -1.1870948277614781, 2.0953612970921451]]], [[[-3.7836719914356065, -4.2418784670166154, -0.94087612107504359, -2.8664687833582114, -2.2784023759466274], [4.0290284180816265, -2.028459200171211, 1.7338786397890837, 1.3891418378826046, 4.6292589569218219], [0.58804105170692189, 2.0862575616655228, -0.14173216842050618, -1.8424843916034672, -2.0584829249232706], [-3.9787305950449547, -0.72327815566807274, 1.7700958118479262, 1.769892447056181, 3.7938251178210258]], [[-4.8450512692814129, 4.592835328626153, 4.6270383243699182, -3.5426481021503453, 2.903525250457041], [2.8547428952944021, -1.1378803126140902, -3.6223816789693086, 0.84054577036847888, 2.9357855264063817], [0.41489761083487764, 1.8195666708720957, -0.16539160598377922, -4.1828443662604107, -1.8571963742976063], [-1.7126506925857674, 2.10941687085427, 1.9939102426581821, -4.005562442112435, 4.7533054567587119]], [[-0.12224089906248548, -4.1344008615564789, 0.88196943978212694, -3.5871126245227547, -4.5564658426475688], [-3.8006984793086298, -2.6097863537899446, 4.8363137267362806, 3.613331530518467, -3.2427033835805474], [3.1548714822989226, 0.10098033250824479, -1.6011963858939238, -2.2940868951628932, 2.7876434219326587], [-3.6802226354976995, -3.5403115765986959, 0.17101825571180562, -2.0613105490272754, -0.36792591238862471]], [[1.2735103901854581, -3.5616892898000332, -2.406231916673268, -1.0582030159041009, 1.9924436982058147], [2.5581755343242953, -0.73368463862315014, -2.4444261626277006, 3.8321402762720709, -2.2677211129906416], [-4.1265834771464185, 1.6596105462307476, 0.61881558063071651, -4.3099751967995212, -3.2984620113153196], [-1.4107593689019815, 0.64876067476481314, -1.4371279016141392, -3.3888497152479502, -4.5514035984994718]], [[0.37059927958106265, -3.4154018747032922, -0.04472290235487808, 2.5090584125454516, 3.1298140969920514], [4.68929883927715, -1.5614817199908346, 0.32112100663941856, -0.88081237795344602, 4.8666271253683515], [-1.4662311607952994, -2.158396537098799, -4.7235419160426382, 1.8941876988662223, -3.6529864301397872], [3.3028339461129281, -1.3534851716128973, 0.12708961713292233, 0.88939480753385602, -2.5034913720727547]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.4962943258620101, 4.7978855087137884, -0.48132692895484386, -0.4650024923945999, -2.9040238320263843], [2.8275327848109812, 4.366656738096161, 3.8611752722183272, -0.11489467004924503, 2.1435368352002913], [4.3812921469892778, -2.8707957287987673, 3.840759570613999, 1.0355213124960869, 4.7836950608824829], [-0.64782147734945461, 2.8875251894981648, 0.50218864913839489, -1.8595975764134751, -3.4013791408892335]])+(1.-msk_arg1)*numpy.array([[-0.12648344098965758, -0.0082059935855323118, -3.9090034246439451, 4.9848999515729222, 2.4026882145547956], [3.9405667069310386, -2.2565817442449623, 2.0673208217629613, 3.4359087058270159, 1.1323419408922044], [-4.8705151205382036, 1.4483528830584742, -4.7468273511940335, 3.1866396548464131, -3.8896409493965587], [-3.1007548573879218, 2.2191914315962391, -0.67050077296913635, 4.7243750851006059, -2.7497878895154573]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.8909457354647676, 0.10199608936624602, -0.62305563602111425, -2.6881249464364085, -2.7726302352609573], [-1.2864065275054817, -0.40876849860093667, 0.38009424085614413, 2.7060435129640159, -1.4858233639297966], [-0.87893260465439105, 0.37641328217260134, 1.4669863372051357, -0.99148304836820811, 2.6768253150202899], [2.424089151753948, 0.66647192591088267, 1.5015087944977883, -1.3691752818649598, 0.97253081687892817]], [[-0.33758269561244253, 0.7490343694080126, 0.71126555242519462, 0.60629403538762139, -0.31645459446549074], [-0.36888816462107565, -0.17038022344749637, -0.85980630346335496, -0.51284495129528684, 0.91618340535998721], [0.39743055154112022, -1.0095741707981702, -0.88361736282409986, 0.4780557105633374, -0.29436628658760383], [0.96737047913156249, 0.29487518647535182, 0.77759979005284074, 0.21626346069081046, 0.012958748599266934]], [[1.681642529245944, -8.0481186443793078, -7.8954022782693656, -5.7136063529911576, -3.2605901459873303], [-1.6947614091480523, -6.99175196848919, -4.7030750503183567, -0.84692139080425, -0.511149320009324], [7.4717334532378468, 10.286996328050828, -10.259622963231324, 8.2926195683707409, 9.5742852862032759], [-3.4968093774091589, 2.2872954760303053, -2.384517532290956, 9.3976327270182818, 10.044189962641642]], [[0.8046782577091588, 6.8868768786501962, 7.3538009268296491, -4.9619518500874911, -5.5885708179139009], [2.6317634107654699, -5.4797183521600905, 0.93140147133312801, 4.9080035664078876, 3.7931437465705797], [-9.5163511920033006, 7.7116952055162793, -0.56292865998798192, -2.6126150696648289, 10.413276369222862], [3.6162198027694954, -5.3260948090662179, 10.685270254129561, 4.5904367138447251, -6.2304075060603976]], [[0.036981901495396485, -1.2106127038098593, 1.4840000991781741, -0.9633367694459849, 0.32199299162428563], [-0.54386580796895911, -0.39985093773769248, -0.95485230241151475, 0.32261780754864211, -0.4147714020687866], [1.6516285972090048, -1.279634897819091, -0.42902876981391092, 1.6785823601849537, -0.6510050234747875], [0.36808639514916341, -0.091923563303704464, 0.51775069012992725, -0.099950880886522098, 0.50202619124318448]]], [[[0.61694303051480015, -0.080275650159892284, 1.1447398699719438, 0.87576607077590352, 1.0314004390371512], [0.16738076692932211, 1.6425877685494932, -0.5260695623955266, 0.31478118618773715, 1.0732020388121768], [1.6942072287454157, -1.39321588077854, 0.77810292151719052, 0.15947589581589885, 1.6892803316464382], [-1.4389500327885201, -1.027828566237116, -0.92763748536675217, 0.41916693193510624, -0.29194399904948493]], [[0.68421519425049349, 0.94447220022324929, -0.93575910111904059, -0.48920494448267948, 0.72495417787400507], [-0.63545701509691466, 0.12263810593515843, -0.0079894241265805032, -0.44839033216324897, -0.17245899037101164], [0.030185059322706122, 0.26277759657440031, 0.44620465834730227, -0.83921183263604426, -0.43244021477075595], [0.21889993529552657, -0.078988714499499804, 0.24634970930156194, 0.7799299444617297, -0.48225911381948511]], [[1.0563710326401621, -0.5462712408444329, -0.37854321467901414, -0.17878140897699188, 0.58215990765394399], [-1.2900803644388357, 0.2476633302029343, -0.029549844590159274, -1.1151461192908463, -0.67412906796535865], [0.34587163113001695, 0.75395484911095967, -0.037247339904361756, 0.6852348501657739, -0.228881622369555], [0.21463374259133111, 0.32992099486355947, 1.1896185106796777, -0.52905785821069795, -0.45594794300745056]], [[-14.2086151222022, -26.366523761631708, 12.048063869450569, 1.0608789536388723, -35.176032727345358], [-40.620619155148873, 10.729024345424195, -23.046584978005203, -14.907565031656228, -31.640231065435678], [-36.481668961284015, 11.882791510284433, 21.826629627243818, 41.905312308515526, -29.489774991498926], [-2.02461130759513, 23.867291518960034, -19.852146283220552, -27.976348583442693, 2.813961811079893]], [[1.2674688087055008, 1.4529621987454562, 1.7155916698833433, 2.2413596266023332, -0.34948182194253974], [0.11759497202920353, -1.2553346535628216, -1.2583655265512377, -1.6516118914286544, 2.125510888887522], [0.39822375723111497, 1.1021912107590959, -1.6939695791968243, -0.13255651920290873, 0.06594659029135648], [-2.2117291976648943, 1.6620054752985991, -0.15627184972719696, -1.2615475458222416, -0.31933305090733671]]], [[[1.0218937003379935, 0.076763023225679711, -0.39405019861687318, 0.3222582962538672, -1.0334591598117739], [-0.77861016593361065, -0.96901575839719734, 0.10215641327596807, -0.59805666301956717, -0.87938100305251399], [0.73525825627090069, -0.43552027588089381, -1.1037116535595461, -0.85005716391088848, 0.96573374438237269], [-0.4229705066850914, -0.90036816156143495, -0.15241070582342006, 0.67516506549456978, 0.47497414726130138]], [[-0.10003105325113236, 0.85672899999853813, -0.31817734082405524, 0.77513080383245703, 0.32504814101273571], [0.78190536796832821, 0.49145238169934446, 1.4096028697564451, -1.0608122754039948, 1.146217610406383], [-0.42189758780265785, -1.2747087954275911, -1.3325182329068075, 0.82094292843607675, -0.68431840148441136], [1.2900209867406291, 0.82446677318557893, 0.601112237696796, 1.156680420487163, -0.16402081206656227]], [[0.71199012889095525, 0.6037103676157497, 0.6505202392200059, -0.47373088036800731, 1.0817314310170874], [-0.59275662127467699, 0.37321591930685721, -0.29001390345624284, -0.21129205813028168, -0.22612713589177227], [0.84545695977274893, -0.28158423591394893, 0.60808168568143683, 0.46443301807415027, -0.68125281726605458], [-0.67335380137214829, 0.35053460922715229, -1.2946460514194427, 0.16504147861124677, -0.93038069169023285]], [[4.3254034244157857, 0.14067291424279832, -0.84096717342717975, 0.32495393280349127, -0.9738515014867658], [-3.6291446234978872, 3.4040958710238565, 3.6805501580379754, -4.4949314624176164, 3.5544397966685715], [1.522578789714585, -0.94308070656154763, -1.2291591238556687, -4.3290596732300983, 0.31276320906013433], [2.3856940683594581, -4.1805537279479692, -1.2555151716449537, 0.057598460222039238, 1.8941006408877588]], [[0.18262118984384657, 0.80866594085992072, -0.53763683049918343, -0.25426165439171589, -0.25680281019868367], [-0.70101857721767835, -0.53290645899693379, -0.41255384123009958, -0.88509105994903781, -0.55289119779696649], [-0.22291752937722054, 0.42327796120996602, -0.42681072140802745, -0.92252372246593872, -0.17903270919406741], [0.3562680859495142, -0.36395111229329496, -0.086269376057941466, -0.24815436867384397, 0.43802150229567488]]], [[[5.8406090624170197, 6.5479126817038473, 1.4523694474048849, 4.4247819554953578, 3.517021981532094], [-6.2193498655930579, 3.1312009111994885, -2.6764760052155183, -2.14432816208293, -7.1458868203356261], [-0.90772083400642589, -3.2204204933145988, 0.2187827563241648, 2.8441236606448217, 3.1775465879048994], [6.1417083782461672, 1.1164775805633169, -2.7323821048512147, -2.7320681838114593, -5.8562817851352609]], [[-1.6779251959091843, 1.5905784459751713, 1.602423535974095, -1.2268804147700048, 1.0055410983139015], [0.98864692355828065, -0.39406766623284323, -1.2544935338205163, 0.29109556288046129, 1.0167133907900572], [0.1436862308054824, 0.63014746243246655, -0.057277978590560219, -1.4485914725430897, -0.64317928066988617], [-0.59312060681396728, 0.73052760839148712, 0.6905256618747323, -1.3871956707704352, 1.646152031520393]], [[-0.24341629240767229, -8.2327644574401884, 1.7562512440201943, -7.1429583895955515, -9.0732155146578837], [-7.5682683904335324, -5.1968246559685394, 9.6304720049605752, 7.1951676660112893, -6.4571419309139184], [6.2822437100315502, 0.2010804757962904, -3.1884360362208435, -4.5681775147623478, 5.5509885114197992], [-7.32836682352709, -7.0497642323712588, 0.34054584070193872, -4.1046538040313454, -0.73264481986973462]], [[-0.68483117333462118, 1.9153011033006981, 1.2939530289741841, 0.56904947034025022, -1.0714381022417516], [-1.3756608240252373, 0.39453946807038531, 1.3144920135582017, -2.0607363253629063, 1.2194687397712662], [2.2190733788248855, -0.8924568236056567, -0.33276854545282814, 2.317692414458822, 1.7737504356598053], [0.75863691520982279, -0.34887154242051099, 0.7728166135739245, 1.8223564916576613, 2.4475207196589093]], [[-0.1089555924906906, 1.004122661201007, 0.013148461404154441, -0.73765914019496814, -0.92016031361144113], [-1.3786463211070352, 0.45907311573111237, -0.094409059777872006, 0.25895742328894639, -1.4307805521780361], [0.43106960443462289, 0.63456511247214931, 1.3887137306333182, -0.55688813872452303, 1.0739721386028067], [-0.97102787113860456, 0.39792246484437815, -0.037364143151562008, -0.26148064379007707, 0.73602243924452915]]]])+(1.-msk_ref)*numpy.array([[[[-22.369816573224437, 1.2066098817734385, -7.3707246227667866, -31.800416505823961, -32.800110878337449], [-15.21814059451226, -4.8357158870917143, 4.4965005018513162, 32.012392470527772, -17.577234231500313], [-10.397739489606552, 4.4529548997644159, 17.354392917382949, -11.729206984359534, 31.666742293294806], [28.676883035224144, 7.8843376910367491, 17.762792282213905, -16.197291829944579, 11.505002802231443]], [[197.37806353237713, -437.94588785047779, -415.86319209958043, -354.48837926640601, 185.02487201900314], [215.68176491045543, 99.618016578931091, 502.71209216647236, 299.85050979991365, -535.6746916851979], [-232.36994570908817, 590.27846336417383, 516.63395738160716, -279.50991457190167, 172.11026622816183], [-565.60278228413472, -172.407810135717, -454.64753602185453, -126.44505668906459, -7.5767293097944508]], [[0.20706552189207486, -0.99098819063228627, -0.97218378155005636, -0.70353292141017143, -0.40148592136151906], [-0.20868088762309472, -0.86091469805080023, -0.57910327126921735, -0.10428388717492761, -0.062939298258572002], [0.9200162103576871, 1.2666676932373022, -1.2632971314361361, 1.0210942985291187, 1.1789095155795437], [-0.43057228043287205, 0.28164132580422446, -0.29361255956462379, 1.1571577736218677, 1.2367702412534283]], [[-0.075062167555125361, -0.6424231066817303, -0.68597881457390419, 0.46286184273460368, 0.52131424592256959], [-0.2454967115263185, 0.51116024717223396, -0.086883189190836527, -0.45782942751762207, -0.35383283783052832], [0.8877062861387145, -0.71936398443083149, 0.052511230411388256, 0.24371051191879972, -0.97137345036474121], [-0.33732898105686787, 0.49682990330914406, -0.99674564150727285, -0.42820608916011227, 0.5811861917184411]], [[-0.044698401834123741, 1.4632144620017455, -1.7936458124848607, 1.1643428888491123, -0.3891788022091171], [0.65734674112046443, 0.48328228582230603, 1.154088085782744, -0.38993398980439853, 0.50131599645522129], [-1.9962510237059596, 1.5466385596967094, 0.51854825133664673, -2.0288288544751718, 0.78684121039399357], [-0.44488987679823383, 0.1111039780116488, -0.62578254393444777, 0.12080624459227809, -0.60677703200944988]]], [[[0.44268420632823641, -0.057601367907279682, 0.82140203507601117, 0.62840130902707159, 0.74007592626626262], [0.12010318343476438, 1.1786301598026683, -0.37747843023395594, 0.22586957414577086, 0.77007044294670335], [1.2156694302663544, -0.99969468155220087, 0.55832363316608735, 0.11443108500445587, 1.2121341613289587], [-1.0325109802755028, -0.73751294784300447, -0.66562136803334948, 0.30077101353733982, -0.20948287138225846]], [[-1.3240082686574102, -1.8276253043203112, 1.8107647970463667, 0.94664865236247231, -1.4028412902381628], [1.229657495820899, -0.23731403171848817, 0.015460141333153337, 0.86766928352198913, 0.33372121983589431], [-0.058410378005352988, -0.50849456955358485, -0.86343983900041366, 1.6239385136461544, 0.83680459724905365], [-0.42358796877834781, 0.15284915039415142, -0.47670536234424182, -1.5092235660916848, 0.93320794349186997]], [[1.9730047056942051, -1.0202814119938239, -0.70701251813359456, -0.33391351173333422, 1.0873113724039338], [-2.4095081662737621, 0.46256561456161749, -0.055190818971251027, -2.0827800771842386, -1.259083960317158], [0.64599116664542788, 1.4081761229850032, -0.069567580551908537, 1.2798264455470594, -0.42748665386386259], [0.40087561192877313, 0.61619985332828464, 2.2218736097731018, -0.98813164274807586, -0.85158283340747842]], [[0.47512733488918396, 0.88168030856020385, -0.40287983224500878, -0.035475138537241206, 1.1762648602961061], [1.358328475698982, -0.3587719633025348, 0.77066360125317812, 0.49849978919551591, 1.05802983134562], [1.219924531476634, -0.39735322638890735, -0.72986904601286295, -1.4012878231694956, 0.98611990526075688], [0.067701754639060641, -0.79810752229518556, 0.66384353958873954, 0.93551185868997899, -0.094097149108442454]], [[2.3993336118831534, 2.7504748174482581, 3.2476355469622273, 4.2429205764182152, -0.66157326820949935], [0.22260868830870403, -2.376363511091979, -2.3821009899036918, -3.1265210612464367, 4.0236175305489681], [0.7538423345903551, 2.0864611425013364, -3.2067046707252529, -0.2509310760259289, 0.12483768404268748], [-4.1868298201046752, 3.1461962398037335, -0.29582448030777292, -2.3881245904358246, -0.60450128410630743]]], [[[-0.91924873109806149, -0.069052497018151499, 0.35446949614004997, -0.28988879158293085, 0.9296524883013032], [0.70040201521722978, 0.87168215835563556, -0.091895226721138229, 0.53798436023594509, 0.7910508411397329], [-0.66140462445638748, 0.39177407673476428, 0.99284841143181479, 0.76467245959872998, -0.8687298089893799], [0.38048487962453725, 0.80992992692164789, 0.13710168473256987, -0.60734754459523854, -0.42726497093574634]], [[0.19827261973213403, -1.6981317071984214, 0.63066270777145261, -1.5363950504959456, -0.64428139425719444], [-1.5498229864728903, -0.97411301816941187, -2.79398891329159, 2.1026473485219963, -2.2719301758007577], [0.83624771781910712, 2.5266139268823538, 2.6411986307489554, -1.6271997522904036, 1.3563948172432603], [-2.5569643849331523, -1.6341844026295245, -1.1914709907329684, -2.2926686234832783, 0.32510740457150172]], [[-0.57608644667322151, -0.48847497512540261, -0.52634984376119487, 0.38330579101663337, -0.87525204502493126], [0.47961206459991856, -0.30197698545381962, 0.2346564542800364, 0.17096092493325582, 0.18296430375404135], [-0.68407731512977588, 0.22783582991458198, -0.49201190209890261, -0.37578269170222506, 0.55121623019724497], [0.54482496743406028, -0.28362505219027306, 1.047525819808059, -0.13353859148995215, 0.75279092361024602]], [[1.405570731637048, 0.045712668066440934, -0.27327828857869702, 0.10559610104822306, -0.3164599999445073], [-1.1793164621067822, 1.1061852628564712, 1.1960210576563788, -1.4606600782312125, 1.1550405951412672], [0.49477285086378592, -0.30646081039100709, -0.39942403505391283, -1.4067588557995185, 0.10163463831057111], [0.77524832439856883, -1.3585007883590585, -0.40798862099869088, 0.018717030975300034, 0.61550152954030957]], [[-0.22459761588123514, -0.99454199436984969, 0.6612154396941089, 0.31270501213592972, 0.31583026576238865], [0.86215132683219287, 0.65539776780977688, 0.5073814777034108, 1.0885338227854475, 0.67997610229732741], [0.27415627769739848, -0.52057059218657586, 0.52491537560761903, 1.1345705765957343, 0.22018430437414499], [-0.43815815014266285, 0.44760715987171662, 0.10609883881899498, 0.30519393517431082, -0.53870301252798691]]], [[[1.2202422201873044, 1.3680147777273748, 0.30343453911981882, 0.92444224558046062, 0.73478958535469496], [-1.2993701867408129, 0.65418238250539629, -0.5591795287066661, -0.44800118092947816, -1.4929458050809967], [-0.18964448295738159, -0.67282247633822545, 0.045708924097244329, 0.59420511338183546, 0.66386509724194642], [1.2831490324251691, 0.23325873502859326, -0.5708596432995855, -0.57079405772410519, -1.2235166249217608]], [[-2.1832507102807361, 2.0695985318051533, 2.0850109001375072, -1.5963688628709958, 1.3083707917746277], [1.2863887516188803, -0.51274545152493933, -1.6322979745662458, 0.37876217364622927, 1.3229077422557929], [0.18695890986585439, 0.81992325896973295, -0.07452786795631007, -1.8848506292455078, -0.83687975172189], [-0.77174536103623892, 0.95053398315303983, 0.89848501317616625, -1.8049648106433429, 2.1419087101195742]], [[0.18231283838969164, 6.1661388446255954, -1.3153891469454349, 5.3499008041976888, 6.7956160922387276], [5.6684475731149959, 3.8922943253789133, -7.212987548574981, -5.3890042729075143, 4.8362410817531023], [-4.705246599982881, -0.1506043491360643, 2.3880604623368988, 3.4214530208580265, -4.1575543747523405], [5.4887671780016021, 5.2801006640474952, -0.25506049001926762, 3.0742851196118743, 0.54873301750177195]], [[0.26956165995408865, -0.75389638325556163, -0.50932279366680033, -0.2239879342437025, 0.42173698368900475], [0.54148442666884877, -0.15529771142367421, -0.51740730119773004, 0.81114225844548182, -0.4800044603025736], [-0.87346652262233382, 0.35128678742395114, 0.13098358396273246, -0.91228471896569152, -0.69817953737792149], [-0.29861290509111627, 0.13732200832462008, -0.30419428511221508, -0.71731174053801539, -0.963387435695646]], [[-0.13477376964023444, 1.2420601195189362, 0.016264128053440057, -0.9124552559534238, -1.138202007844161], [-1.7053311119584049, 0.56785533384030751, -0.11678028253153867, 0.32032011680313816, -1.7698190991109153], [0.53321609509803514, 0.78493201069378926, 1.7177841003856402, -0.68884865850507437, 1.3284611675206277], [-1.2011231697928977, 0.49221439107123138, -0.046217971072422211, -0.32344124102262195, 0.91043072144517201]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_expandedData_rank4(self):
      arg0=Data(numpy.array([[3.9581552679952186, 0.3785155803125253, -2.6102642796991296, 0.52491675023995299, -2.0420668296070121], [-0.57901184314576959, 0.90806263341547311, -0.56071157159255769, 4.2933268982401511, -0.37662814284830581], [-0.60019192302048108, 0.012120187197036891, 2.2333890918417589, -2.4423027816940834, -4.4158293133165376], [-4.5987061550846109, 3.4000465327597933, 0.49142360337942836, -0.60817694230205976, -4.0155797431103979]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.3183800045555412, -3.4767078838096968, -1.0325988107220265, -4.0715093102981736, 0.95368243707483735], [2.0026155590694916, 0.53641254621472712, 4.2566467289740331, 3.808373333537455, 1.406919533468086], [-1.8243846494371097, 3.772782874603509, 1.8043373932537055, 1.8485558090936136, -2.2978639654713939], [4.3100876399688701, 0.21647019840802173, -2.0105659440659265, -0.76630982685399474, -2.050220027752776]], [[2.3729524760622143, 0.25118602462127182, -0.60282017996953741, -3.8868902615470935, -4.6780937500135931], [2.0618407668888601, -3.3494289915953921, -1.2556651629741333, 4.8304117004948015, 3.0720687398140338], [2.2082524403400701, 2.1435246711236813, 4.9381509778644688, -3.0387871874636838, -1.6420243474926801], [-2.0522471811084841, -2.0001772931760717, -3.531360138792655, 4.1689554084159202, -4.4726743203792445]], [[4.2428276835531058, -4.3573714380074788, -0.69902571123078427, 4.6111605913350413, 4.3803665192425392], [4.4911391426469347, 4.0000273651701939, -4.0049040320821412, -1.3169173733334372, -3.1587934270092766], [-3.8212119471053088, 3.7865571744801922, -0.77696724832495434, 4.9646336288609323, 1.2008644995708648], [1.9993258867161856, -1.9154560768982765, 0.46758304607086743, -2.063988093979876, 2.4537737757684299]], [[2.9167934407905571, 4.2376744227669452, -4.6793430141037993, -4.7418437356231387, -0.36705306454760844], [2.8442371311137569, -0.56044560102302299, -1.5445466287241141, -1.9693256613489352, 4.2328032961548328], [-2.0263906524846478, -3.9498989256184558, 2.1286309772035992, -4.5294474159808553, 4.6690276111023152], [0.19235003763457392, 4.6139002140193455, -4.192820124296361, -1.8287346330316003, 4.2575119898619125]], [[-3.0481050486093619, -1.2343696268973359, -1.7377325714511915, 3.5015828387849766, 4.3787101028248614], [4.4625577304642601, 1.1702458693862265, -3.0833859447381586, -2.6742931411204873, 0.66273883130850386], [-4.1048796860034749, -3.2049666658013907, -2.91989471309273, 2.7556201386302348, -4.2638520622419431], [3.6459500538188099, 1.5341888785964191, 0.66017242950759503, 2.8913225276975361, -2.8830110840693699]]], [[[-3.9438012089025607, -3.6368590250207458, 2.4615909187788301, 3.1123111320163428, -1.8267427684907656], [1.8189275911019216, 0.9339143935695704, -1.1098988889411143, -0.11664377230693468, -1.157642388894776], [4.6356380323153026, -2.2757622136558195, -4.8842739376579924, 3.9848803418458161, -2.0577432757113776], [3.9358863671207853, -1.5284439228485844, -3.3022648349657757, 2.8403878934187681, -2.9675451941809028]], [[3.5516833844642921, -4.2541313993066243, -4.0417874711611397, -0.32690662906619217, 4.4707667781776799], [-2.2130923975433636, 0.48822575154352776, -0.048912660012896403, 0.31635274459529938, 3.0579792498863654], [1.9674207974449622, -3.4957748698378213, 1.8322963244123338, -3.7213987440623377, -1.5589013351048706], [0.11848379510555773, 2.0966672033377778, 4.2129323496973718, 0.42130236447568059, -0.18109199623449967]], [[4.3080774350779389, -0.48900501115363948, 3.9610587249472751, -1.7377776923708388, 1.9875134360258953], [-1.2313553413268066, 0.034382995425884744, 4.3949310756907636, 0.10446926856979921, -3.3999708886391877], [-3.5943026615316978, -0.071981776867443337, 2.5576863217054999, 3.2851285960394865, -2.5861940180694076], [4.825451157472509, 1.8217044118587218, -0.69542941808014103, 0.68534282077819775, -4.3505444496496679]], [[-1.0302474914621254, -3.4078117538417976, -2.0201707974577365, -2.5597059373292206, 3.366715674794813], [3.2964030778475841, -0.098709673430996681, -3.1373533877120394, 4.5356132790491959, 2.7354245639195067], [0.0020701233437900868, -3.5096855278946224, 0.66070912360625478, -2.3224076285963613, 2.8302347864565904], [2.2568261452595015, -3.4152722397230431, 3.6800084365147701, -2.4610521457262138, 0.78266768533662834]], [[0.27243756422839471, 0.75044794003305082, 3.4485776517281064, 2.5961559542977044, -3.1760317465867516], [-3.7842074635374945, 4.3662202739045277, 4.7076072735677794, 4.2930800537044522, -3.6561120120352317], [3.0746748212503814, -0.74975362110958255, 0.37602783796028572, 1.1521715219328073, -1.5665072037475172], [3.4371358368045684, -2.8284156577452291, 2.2818575897668918, -0.1771097551258034, 3.9307739938328154]]], [[[0.42647148587707573, -2.7273834607238356, -4.7763571908257232, -0.6652283765673932, 1.0801253127019192], [-0.21357748856724701, -2.8221197055240941, 1.4998059024489603, 3.7901054068362541, 2.3556269428022771], [-0.9528004098623768, -0.4291579251138824, -2.2347718325472563, 4.0588691960323899, 0.29921209332065324], [-0.19733348906899817, -0.65425568170863624, 2.7540265710646326, 3.5948859567645322, 1.1305644738392653]], [[1.7582003227086478, -3.5917575960190442, -0.93025256645960841, 4.733197354937472, 1.2543307057204434], [-3.2612643094721894, 0.58614032050230058, -3.4601380123422709, 1.0041979660635043, 2.4190439735797327], [-0.76425602645723778, -3.5952815773901428, 2.8314448425991365, -3.6480351491390062, -3.1436664992347185], [2.8677775206798124, -1.6744007780014911, 2.9531560192847746, -2.4812132214255769, -3.3668608126314825]], [[0.2259041635324035, -4.0189941561497537, -2.7697334676417262, 0.58237493531315643, 4.9730520795654982], [0.01910689665060783, -2.7572797985987485, -4.4032419810763859, -0.79873843234518649, -1.9232253527518504], [3.146045855195128, -0.21379937627542311, -0.99588001631277212, 3.2599484652039941, 4.1005460178928406], [3.7472998860569913, 2.1316795821357513, 2.106609456081495, -4.963645747401638, 2.0600066869876263]], [[-1.8608015329093863, -4.3334048013048374, -4.1927201585793998, -2.6386770320544892, -1.0834597811932278], [3.5231162225729591, 0.87864755323554622, -2.8941353354247932, -1.4584762532185747, -2.4826293192554694], [2.8552353345335035, -0.17142391587890771, 4.2546933044980975, -0.52325181147238187, 0.65090148473647336], [-3.4664839207454623, 3.2163062186234832, -3.8741161353901044, 2.8705899936621426, -0.80835308950140927]], [[0.63383824512798093, -1.0873241127402911, -3.9304420942580109, -3.4899836816307617, -1.1872939265983806], [1.7259373564984006, 0.163079971402464, -1.9367954998604811, 3.4124599734581729, 1.8482809883637943], [2.0043936355085457, 2.7683336016827367, -0.96459124970295207, -1.9206042969198753, -4.6199685628260241], [0.39138183591371245, 0.21931534558134924, 2.6327188423699974, -0.76471028731627833, 0.021484944170571119]]], [[[-4.8553143958079836, 1.6160996788825095, 0.88918402255760931, 0.57376313751433816, -1.6887128042149335], [-1.0525163289189976, -0.18884069584071028, -0.2938972409615257, 2.5941635126082101, -1.8812421544220648], [-2.633462196183344, -4.397500301124257, 3.2738495668802674, -2.1167132500272046, 4.1668517223010362], [-3.0061575854338409, 2.8772939779960316, 4.7774101225836301, 2.5098602205846001, -0.33232321327937075]], [[-4.5737236672233372, -0.79373209486910667, 4.9782579948284695, 0.86910773597780988, 2.0768901859020357], [1.1686043632284902, -2.2552527771671746, -3.2529758103932869, -4.4570971471531218, 3.1445708224355311], [-4.76117359336238, -1.3921686251117258, 2.4537068985773516, -4.3944819494988652, 4.3328839506577701], [4.8864574937316547, 2.8725746163788104, 3.3614806693119608, -1.3952179384152217, -4.0993320479119539]], [[3.7422308143920482, 4.7750668659889364, 1.5231316295588027, -4.9755139383688203, -2.7735374088330178], [-4.7155561866589677, 1.2362821413788048, 4.2131639715312161, 0.75193267641079586, -2.099654963652112], [-4.062615064788206, 1.8190482078105097, 1.6307258003288458, 4.633924513200455, 2.9108156184324692], [-0.72645917865346199, 3.8006835113610915, -3.7776901445677966, -3.3166235267946131, 1.8714703598091713]], [[2.749888904755875, 0.11016966918833493, 3.4222685003542157, -4.7330528816891784, -4.1595865883802059], [-3.3050908317524397, -0.97806766236404297, -0.79581195642758118, 0.52677156021975868, -2.7053795928536948], [4.4044044670540412, -2.3686433364167234, -4.0118555738631825, -3.0646835700884365, 3.3642929793472494], [3.845156479541803, -0.37717069682921078, 2.9918029058353106, -0.29586624730885269, 4.96722833853884]], [[4.7247479272394131, 2.2044341156652951, -2.4310477055954705, 3.8798575872437908, 1.3675873138513319], [-1.4079524061803528, -4.0338271636673007, -4.8601143137354432, -0.78592432948333624, 2.1379921802999577], [3.800454611929883, 3.2990709435258267, 2.0839086160367781, -3.1303586475380984, -2.4179005952848165], [3.967029354329604, -2.2646459760465798, 1.3555026752271093, 4.6667540759225563, -3.6174780321642652]]]])+(1.-msk_arg1)*numpy.array([[[[4.6972372083371603, -2.6983868504961004, 4.20497112952342, -4.5032290685490661, -0.023444538113456481], [1.839426002879696, 4.0144225246272001, 1.0342746516884374, 3.7548578595671103, -3.6845918227231667], [3.8531059812403576, 3.3239029851643007, -3.1104210086425299, -1.0061313248985648, -2.0189376268613204], [-2.1194512096538878, -3.7971024489729324, -4.2533707464042596, 1.2444856339513501, 4.552704817416366]], [[1.1764835616910529, -0.38395136526168994, -1.8140338858193692, 4.424841206522979, 1.0809121033492293], [4.9323275599397558, -2.1252766602082951, 3.2230358795475933, 2.853160472265202, 0.44046890391790416], [-0.70225518439557, 2.0545965036241922, -1.902622979415888, -0.75163641632257949, 1.7243613943775449], [-2.5262495836205923, 0.15394164874260508, 2.9305164601834708, 1.5510708054542413, -2.8692377152049611]], [[-4.0192426895507118, -2.0921780458501793, 1.1041147567665064, 1.4932839146795187, -1.3666249482318515], [2.6242230918771305, -3.3181964319320367, 4.3767976093344423, 1.3425469448891807, -4.6501225230825938], [2.5757041268112548, -4.331916008835953, -2.9278020389226791, -2.7694365690473854, -0.8443396174860851], [-1.9789896792994299, -0.72069680457572449, 1.0768342335500538, 1.7703763460378905, -1.7320664203800842]], [[4.6406649704655969, 0.79662831932619049, -2.1104966305776953, -2.821248700088578, -2.4071228062998165], [3.1841009490062628, 4.926345540425709, 4.7736945227263448, 4.293369599771669, 4.7018013333828304], [-2.7314955210297818, 3.1799747456318599, 3.2618928884173179, -2.6313291427456722, 0.60364167756250708], [-0.26509778928581529, -1.2880495435634209, -2.4705310902137754, -0.25177191641300922, -3.742635099588596]], [[1.0531066212120948, 0.50930214124813045, -1.4307819581518446, 2.8387007956951935, -4.5992669288333063], [-4.7826467394121828, 2.7465090851466574, 1.0709802843933502, -4.0682713178183274, -1.976641101498533], [1.6951067159446831, -4.2956846024182704, -3.4410704472418296, -0.086066419547945472, -0.18367409849696958], [0.2791977502654861, -4.5974584551631059, 2.2834595426999886, -2.6142192222802487, -0.98554499068825052]]], [[[0.70990594049597888, -0.9155176241369789, -2.6352987750315524, 1.9402028950602048, 0.60951952880521709], [-3.2475170595980662, -0.8506790916999023, -0.88260748826108237, -3.8932897582299528, 0.27987649849546514], [-4.4762632058556262, 3.3766357361564747, -2.1952382471898479, -0.11825452913232581, -4.6852164509359326], [4.9677944344391509, 2.9083622799358988, -0.74751238950700216, 1.4608453568198492, -4.0510572522840906]], [[-2.8754625529398226, 1.7708738040035721, -1.0195732000450564, -2.25964855666306, 1.9964407106262883], [-2.2872868326729998, -3.5109984945061958, -1.6775427121167761, -3.2329292972022525, 2.5572508575556219], [-1.4957774040817231, -0.44177733155264765, -3.3204288468180829, -4.3331661195203894, 3.2010767073059778], [0.10980180636002235, -1.1423358443442866, 1.4337793970035868, 3.7470497928226258, 2.1537641131662539]], [[-0.8381111971725872, 0.58556918214331066, 1.3091771674608266, -1.8775274130911859, 3.7995167753513801], [-3.8745807072888052, 4.3167720571934236, 0.019369895514639524, -0.13800997877594501, -4.7247506609294536], [2.0992051258328885, -4.5650675538334538, 3.6905490899340947, 2.7903346461455687, 3.6950059818149974], [3.5812248106937226, -2.7091945546910736, 2.5267721695201661, -0.80263529410706091, 0.5800472540160051]], [[-2.9791329222120755, 3.2801562372320898, 3.0100856580717466, 2.0890131745423322, -2.5137367027024302], [3.873839256202638, -1.3473634517251423, -4.1057291539939698, -2.6435248755672758, 0.96125132198843755], [1.8359568915594977, 1.4678888244012995, 2.7920085406611816, 4.7631915471310862, 0.98145049797927086], [4.4544123234081177, 3.28346582214186, -2.7934312852285439, -3.5960945317511053, -3.8978032276721608]], [[-1.6403901972395429, 3.1478463259559568, 1.3687971861770585, -1.941375681796714, -1.9976265759388578], [-0.20518640298234825, -2.6774325670073127, -0.56762475636883281, 1.4714292094243175, 1.5614951194333839], [-0.63209595516899952, -4.4935218510385742, -1.1963748988499177, -1.9721514701301555, 4.1194939082553503], [3.1906250817553179, 4.6101433779329994, -3.1138393965250968, 1.1908345140780154, 1.6438626924408171]]], [[[2.6155226358902217, -2.2529769574668079, -2.473087822507205, -3.4853823204426515, 1.7996561131427153], [-4.4929471834007515, 4.7628758654150705, -4.343760725738008, 1.8688476914546239, -3.6958735277867416], [0.60510250630252393, 3.3571823639434815, -3.3977929764033918, 3.1531867368132005, -4.3072113787562838], [1.4371924144127144, -3.3388166153497165, -2.5942813444713164, -1.7650171252146762, 3.1331564859776346]], [[-3.616821452815222, 4.1002242819964874, 1.6778284278407067, -0.037253655514708939, 2.1213519593381447], [-3.2966963699177931, -3.8094253747299933, -1.7079318992270478, 2.6290087200979144, 0.30528579652754662], [0.98279884078857016, 4.9201336538048466, -0.93976559668704951, 4.1742093086042757, -2.3007482409900426], [2.7495777725104489, 2.1381614448921473, 3.1849097904480193, 1.4853127757625906, -0.34756863449014208]], [[3.6920032705049515, 4.4272129674226228, 1.9057369347656277, -1.6501080956321279, -0.99165207213973261], [-2.1226166521541376, 4.0246836467121874, -3.5641277736185026, -2.3245312714723698, 1.9561232049680699], [-1.2798801124475823, -3.2130032796805041, -4.9761676849939107, 3.9632777371500207, 4.3662064178392832], [2.2704086106372214, -3.4466432111030745, 0.04797054030023773, -1.6079803887727282, -3.3201818598306652]], [[3.5826666432053518, -4.5756874020863005, -0.46690018537922295, -0.35125448221908862, 3.0147304911193107], [2.6437684679072957, 0.17539063548377065, -0.2908116084181156, -1.6854933807274062, -1.2438235172051382], [-1.7828210731777103, 2.9022669557280159, 1.296649725133511, 1.6508062819592793, 0.93730661570498164], [2.3040165438091087, -3.6150108625711033, -2.7636476411563073, -1.4797744769366536, -4.5092395331452799]], [[-4.4593128260572819, 0.26718039129267446, -0.39274030342720145, 1.9816499028919647, 3.6477975434544021], [-0.30796276351635488, 3.3954823013801363, -4.1899216994083659, 1.8344046142097223, -1.3015040124970345], [-3.2439678315677312, -2.6977520214163131, 0.75035362832276498, -2.6244966187637111, 3.2666554320664858], [-4.678121916171941, 1.4232385156649485, -4.8104164171262784, 2.0561056342236625, 1.0209456514101287]]], [[[1.1200349847856996, -3.9273533534659419, -4.1407147818696881, -2.627392823444008, -0.062880709405368052], [-4.4703650711003782, -4.7077335471376403, 0.3664776473133351, 1.1747357250118942, 1.4266706499697008], [-1.0675573484502445, -1.2438491383310044, 0.80406452423883223, 2.06619696236179, -1.8700784645217339], [1.4707221904997141, -4.7771549040829946, 4.0824686008821676, -4.4240789014930817, -3.8471514980315447]], [[-4.6510532626054921, -0.81810191969326151, 0.16077215379990939, 3.4272528306346111, -4.2747356492483402], [-1.8895026584597643, 1.865234335548843, 2.5551150966748049, 2.2082592146606039, -2.8575593319069914], [3.7177840844302796, 3.078486869729737, -4.496025649928943, -0.20878575594879845, 1.506946662518911], [0.44399960628687296, -4.8292000502474455, -3.822857391966239, 2.5059631543557135, -0.14376076022736139]], [[-2.3695261849429752, 0.94901527127525043, 1.1451369018739204, 0.8312114495722609, -4.8033467996417816], [-2.2973915016987601, 2.484341996196914, -3.8179107169024054, -4.8175934299276904, 1.4521713959850837], [1.5889772927827552, -0.54902739519342436, -0.69583951084606799, 3.475766977608064, 2.3554915052289225], [-4.6724060903686162, 3.0201171051994553, 3.5480180182717689, -3.1112885915703692, -1.8986726564247736]], [[-4.3141501609001285, -3.4493419117346602, 2.699173519632331, -2.429077706718934, 2.259361891396332], [0.14326366856936623, -4.2870251960241648, -1.7373208669208329, -0.42985762755024304, -4.2466833987152599], [3.7108186928766713, -3.3695882922238751, -4.6106020434874981, -1.5432250061066566, -0.4425689418998342], [2.525237381042519, -4.2034836807674028, 3.3691813735098091, -4.5952068683399165, 4.9721920034533049]], [[4.4904975856473577, -0.42862030250116501, 4.4720763152163823, 2.288157502695892, 3.3148598617385474], [1.6240412065587639, -1.1525805392006516, -3.9191611584110797, 1.99554967312692, -4.9996810899366375], [-0.91500403102947203, 3.7497946145168051, 1.7475115954243474, 3.4961929267182796, -1.4384064388883933], [1.3740951072745133, 0.83057420741139776, -0.50044278804061015, 0.68964630380608494, 1.5706688300047791]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.707293567153604, -1.1384779510604046, -3.8331975854470994, -0.97215920837605707, 4.1503912771381088], [1.9764928171407805, 7.3789386469919744, 0.92987638392750549, 1.0393296353429291, 2.8133487195520579], [-2.1695837383944534, 1.0491341271292192, 2.1936890976125039, 2.1412149141096188, -1.7225368113482842], [0.91834681765863269, 18.284989329268068, -1.9686771675793542, -5.165215333652986, -1.9306002352994813]], [[0.15951249935719378, 1.5069133757868729, -0.62790794483962531, -0.097382625914904347, -0.080912354591317334], [0.18358138338862737, -0.11300898787892549, -0.30144627044998518, 0.078360935626615877, 0.12321195011262624], [0.17140956051846773, 0.17658559540352731, 0.076651277372693144, -0.12456139800577888, -0.23051764176975012], [-0.18443956644056733, -0.18924101458600315, -0.10718690969931401, 0.090793866383989466, -0.084628469054377833]], [[-0.61521807492148606, 0.59904562115841575, 3.7341463093012717, -0.5660753356983812, -0.5959008836891807], [-0.5812031640063422, -0.65256160555993281, 0.65176699835727614, 1.9821017875190741, 0.82634852199578923], [0.68309853413823041, -0.68935028824897093, 3.3595550974980446, -0.52577178394894353, -2.1736542970767485], [-1.3055721916282423, 1.3627377370751124, -5.5824613437834421, 1.2646702213605792, -1.0637754407012083]], [[0.17996363503124221, 0.12386906068569895, -0.11217744641882949, -0.11069887147408755, -1.430084096660277], [0.18455449600097307, -0.93660606717544659, -0.33985166940124362, -0.26654644305015512, 0.12401160968590209], [-0.25904025445258011, -0.13289371706081418, 0.24659828587552579, -0.11588979891629493, 0.11242528294152128], [2.7289661946268415, 0.11376855282760392, -0.12519419738476009, -0.28703822892541153, 0.12329190181728134]], [[0.66994634274125986, 1.6543398226185078, 1.1751329653110369, -0.58318392670544228, -0.46636264599695748], [-0.45760009235657928, -1.7449895641828161, 0.66228064413792498, 0.76359124518092947, -3.0812542333986666], [0.49747300428071145, 0.63715696372037012, 0.69936317239468904, -0.7410552713633759, 0.47892534726762748], [-0.5600918277715099, -1.3310400421330355, -3.093232522797317, -0.70627431220313674, 0.70831043310615194]]], [[[0.14681567667222528, 0.15920656785492715, -0.23521854859336716, -0.18603919035904704, 0.31696408116844099], [-0.31832594435218797, -0.61998385198100825, 0.52167981148099796, 4.9639327646414459, 0.50016468703997929], [-0.12490445524638558, 0.25442545784062215, 0.11854614432691005, -0.14530219064936051, 0.28138196342573429], [-0.14711091457890171, 0.37882439420260594, 0.1753377975669751, -0.2038495673380917, 0.19511475150611399]], [[0.25567105372835425, -0.21345429846465888, -0.22466857544951555, -2.7777431005585642, 0.20311116156804015], [-0.41031392743631728, 1.8599236737198506, -18.564981605499508, 2.8704117442607635, 0.29694859226046638], [0.4615497785703751, -0.25976004383188461, 0.49558721551587076, -0.24401110868979806, -0.58250167150853449], [7.664023865934376, 0.43309812447578128, 0.21554170778002218, 2.1553703705071192, -5.0143719893595105]], [[-0.13015354994017503, 1.1466376801942197, -0.14155598554020965, 0.32266012738809102, -0.28211712254570753], [0.45536130211477488, -16.307816251821794, -0.12758142549583193, -5.3672393735381485, 0.16491658015841959], [0.15600009915514812, 7.7896322651929717, -0.2192260899368878, -0.17068177247872279, 0.21680955399128507], [-0.11619878707595273, -0.30779503411338416, 0.80628106464133142, -0.81814758190051085, 0.12888308074583851]], [[-4.1672772162221614, -1.2598486091258028, -2.1252296606024821, -1.677273485062811, 1.2752270500245944], [1.3024277665228723, -43.494489942177907, -1.3684549898190215, 0.946581340625267, 1.5695285312815841], [2073.947386332889, -1.2232796540080948, 6.4980590472347268, -1.8486534600452518, 1.5169507910738844], [1.9023737859729919, -1.2570965348836474, 1.1666622433904628, -1.7445087076662753, 5.4855042295422924]], [[-1.3824383723111113, -0.50187111291386655, -0.10921260324805933, -0.14507146314720867, 0.11858450195060681], [0.09952629354423187, -0.086259537820226162, -0.080004155181549089, -0.087729121781299532, 0.10301329434342191], [-0.12249365046517083, 0.50233587707242111, -1.0015964373576072, -0.32688548161344866, 0.24042541390636918], [-0.10957615896799963, 0.13315869674846453, -0.16505330768112531, 2.1265239883640619, -0.095815262703787152]]], [[[-1.4073436159187387, 0.22006143678131448, 0.12565892772284931, 0.90223439673078443, -0.55566878765122996], [2.8101834469858216, 0.21267415476588361, -0.40017973128419937, -0.1583575807516879, -0.25479073622179188], [0.62992408148436174, 1.3985339379698341, 0.26856966526930193, -0.14787170860474597, -2.0059079710300352], [3.041510723051283, 0.91736600812244573, -0.21793250992072419, -0.16695715253250079, -0.53087810284918935]], [[0.0068935189241489713, -0.003374444648066007, -0.013028920998481491, 0.002560676491630678, 0.0096626728037208353], [-0.0037164075177330384, 0.020677961868670523, -0.0035028045568715259, 0.012069519762670407, 0.0050103211555519106], [-0.015858804873572101, -0.0033711371240733468, 0.0042805662376637068, -0.0033223877242239421, -0.0038554303390602599], [0.0042263345429124424, -0.0072385221962827451, 0.0041041472641097651, -0.0048847826105300448, -0.0035998480102193337]], [[9.8864450168551379, -0.55570846959911313, -0.80635523884663363, 3.8349677440029493, 0.44909827126461399], [116.88915958891263, -0.80999726359898938, -0.50721470712717909, -2.7961457736349002, -1.1612726967466971], [0.70990354071086381, -10.446190867108221, -2.2426286854422903, 0.68509950868257141, 0.54465651210748689], [0.59599956228531004, 1.0477133198433621, 1.0601818412018746, -0.44994933270789728, 1.0841659427366577]], [[1.3125004136661005, 0.56359903901862074, 0.58251032487739363, 0.9255785198510984, 2.2541702277165698], [-0.69322231439485438, -2.7796159821995832, 0.84387994984194792, 1.6745577970874699, 0.98375652085931231], [-0.85537705146574683, 14.247153141801427, -0.57402557761614936, 4.6675476857340845, -3.7521849910710898], [0.70454755814037351, -0.75935020351990667, 0.63041547964543809, -0.85080167738560475, 3.0213316599068007]], [[-6.9668079312962865, 4.0611895400605951, 1.1234943060903071, 1.2652865216989087, 3.7192385258534686], [-2.5585107690556148, -27.077692467941027, 2.2799667355870232, -1.2940310941849831, -2.3891547557526338], [-2.2030749025982481, -1.5951218128596811, 4.5779280235813893, 2.2991874590712524, 0.95581371458843545], [-11.282662883440741, -20.134611655245994, -1.6772886045595996, 5.7745127619686176, -205.53133758500033]]], [[[0.94714899596513769, -2.8455584857640064, -5.1718272465772577, -8.0149906022320767, 2.7232020409903304], [4.369249225622732, 24.352304648165894, 15.64732673242969, -1.7727125266907344, 2.4445051607390633], [1.7462586559053248, 1.0457545969715827, -1.4046785171827036, 2.1725692674836834, -1.103640460847763], [1.5297621712738447, -1.5982746949922382, -0.96259396557681842, -1.8322558831636744, 13.838052749022571]], [[-0.74338695997870119, -4.2836198192546195, 0.68297917389814689, 3.9121116888166827, 1.6370853672665817], [2.9094932722709701, -1.50761216976777, -1.0452111331097558, -0.76283877611496587, 1.0812434270843967], [-0.71411942162744213, -2.442266311300421, 1.3856775374153796, -0.77370815760149503, 0.7847075000113114], [0.69581011133758375, 1.1836233995014265, 1.0114728797341936, -2.4369286253743159, -0.82941476636215639]], [[0.13131835735238143, 0.10291449673294836, 0.32264027208323182, -0.098768410553490962, -0.17718297283979945], [-0.10421328554407669, 0.39750117463587387, 0.11664003743980257, 0.65354734379298851, -0.23404969477682783], [-0.12096238396759046, 0.27015424949673461, 0.30135268803641285, 0.10604911711002883, 0.16882677153012857], [-0.67646416731950865, 0.12929874374186998, -0.13008573614383925, -0.1481698478616203, 0.26258690168597565]], [[-0.22116418639685062, -5.5203664201113458, -0.17771163841735724, 0.12849569981669159, 0.14621091047870008], [0.18401217190741762, 0.62181479431807696, 0.76422192125911315, -1.1545364029302196, 0.22480281285057716], [-0.13808380834488823, 0.25676172218570825, 0.15159492436972771, 0.19844689619441194, -0.18077407230450546], [-0.15816702012983655, 1.6124713489538465, -0.20328108550060284, 2.0555806815881508, -0.12243788705734455]], [[-0.84990348796377591, -1.8215920877719232, 1.6517897751935748, -1.0349812210409048, -2.9362510915679092], [2.8520706562832627, 0.99547640000016424, 0.826231541871709, 5.1093719744625101, -1.8782013237050366], [-1.0566051046906921, -1.2171850232534904, -1.92694617806577, 1.2827858386988629, 1.6607712289501226], [-1.0122384747992363, 1.7731600371906449, -2.962428489812905, -0.86046525653198691, 1.1100495172068692]]]])+(1.-msk_ref)*numpy.array([[[[0.8426560321394585, -1.4668598267396347, 0.94130379164905831, -0.87895934400479658, -168.83059281613029], [2.1518426192728417, 0.98598372336573947, 3.8269866340952969, 1.0541425044652812, -1.0742452511523704], [1.0272635342153356, 1.1908155218915231, -1.2725464678245155, -3.9340344247747856, -1.9605138937098534], [-1.8675378088281618, -1.0424146625450805, -0.93059258268078038, 3.1805552109329951, 0.86940740213450718]], [[0.32173469535643567, -0.98584251694101999, -0.20865959741515858, 0.085543313905712154, 0.35018164672195518], [0.076741776719538995, -0.17810179135709681, 0.11744069705040214, 0.13266536670193368, 0.85934688452620955], [-0.53900005115421556, 0.18422866954404196, -0.1989440810962616, -0.50358866613253328, 0.21951058609100954], [-0.14983301047002689, 2.4588250379558709, 0.12916343772688707, 0.24403501051112528, -0.13192200085292913]], [[0.64944181810303081, 1.2476300881163391, -2.3641240765076899, -1.7480026765434835, 1.9100077772444495], [-0.99468078296345752, 0.78665152387596593, -0.59638679068280231, -1.9442629471065471, 0.5613323663499451], [-1.0134177495497747, 0.60256576405795648, 0.8915439790661559, 0.9425253890530495, 3.0914862048885765], [1.3189883236900828, 3.621861874683622, -2.4240168062764309, -1.4744120850580227, 1.5070232001416746]], [[0.11311239953339881, 0.65892303537983887, -0.24871717046819936, -0.18605830468737913, -0.21806812218560842], [0.16485556163154189, 0.1065529703372355, 0.10996027243489459, 0.12226218545635327, 0.111641626904361], [-0.19217192420731405, 0.16506947137268924, 0.16092396905609138, -0.19948730157422506, 0.869583346795331], [-1.9800872412180464, -0.40752838496239657, -0.21247121816003209, -2.0848899977345932, -0.14025325372961253]], [[-1.9390883966303936, -4.0095390618279838, 1.4272383139669795, -0.71936670208559728, 0.44399832869126865], [0.42697421341597874, -0.74351358990606453, -1.9067268178178718, 0.50194951862308479, 1.033099447370025], [-1.2046833455373152, 0.47537634128386047, 0.59343941396021671, 23.726638569755156, 11.117881325225092], [-7.3140518778007095, 0.44417298155542873, -0.89428640684058236, 0.78113832696319263, 2.0720178671710814]]], [[[-0.8156176897762546, 0.63244205013702537, 0.21971392717656354, -0.29842850179223279, -0.94994797669690945], [0.17829370331850755, 0.6806466137409547, 0.65602416798722341, 0.14872045984294061, -2.0688119447626696], [0.12935160791892999, -0.17147595665881382, 0.26375808816513191, 4.8963185376000293, 0.12358273074664554], [-0.11655310033196618, -0.19908518520551408, 0.77458494504370456, -0.39635396070001205, 0.14292857569941964]], [[-0.31579706454082868, 0.51277659162529543, -0.8906301512979593, -0.40186011702477142, 0.45484077167040521], [-0.39700426743342931, -0.25863372907631721, -0.54130522391865132, -0.28087921198935661, 0.3550932951034203], [-0.60708406941936954, -2.0554758439597691, -0.27347751610026394, -0.2095610019022259, 0.28367412481648946], [8.2700154352477835, -0.7949173948373397, 0.6333349714141564, 0.24234069030917146, 0.42161656787963098]], [[0.66901811297134328, -0.9575496605545929, -0.42829311840204803, 0.29864361376721249, -0.1475744429476043], [0.14471541928078957, -0.12989140129791979, -28.947578533336895, 4.0628335470064512, 0.11867537820128152], [-0.26710661320917251, 0.12282656608700297, -0.15193174726272798, -0.20094778680653846, -0.15174848818976325], [-0.15656977744548287, 0.20696615184821784, -0.22190824260148345, 0.69858823267466008, -0.96666533236805241]], [[-1.4411330445276862, 1.30887878129336, 1.4263138614435464, 2.0551937874593573, -1.7079461399535385], [1.1082873124809829, -3.1864653095221227, -1.0456916998686312, -1.6240917337003848, 4.4663937515935022], [2.3384682494333053, 2.9248311090529957, 1.5377198299054768, 0.90135508004628984, 4.3744711598595876], [0.96383688498672293, 1.3075594907333441, -1.5369366416646593, -1.1938859950240326, -1.1014734832584672]], [[0.22959667979124576, -0.11964629268677185, -0.27515262790698614, 0.19400064932292863, 0.18853781151329327], [1.8355414265959247, 0.14066764836183349, 0.66351606166306698, -0.2559607627985433, -0.24119713098108944], [0.5958401406754914, 0.083815803134740935, 0.31480779412068965, 0.19097323332038466, -0.091425828326521755], [-0.11804211814228714, -0.081695537854870479, 0.12095297633802363, -0.31627244457211934, -0.22911167981377212]]], [[[-0.2294730371607735, 0.2663994946913803, 0.24268928808682955, 0.17220260730084133, -0.33350367252795532], [0.13358535022131965, -0.12601460545690207, 0.13817333893742687, -0.32115614652006214, 0.16239514650813916], [-0.99188470840742515, -0.17877846895259855, 0.17664169865222101, -0.19034455397559835, 0.13934582499960513], [-0.41761417399752965, 0.1797618713951486, 0.23135190186660076, 0.34004878164991215, -0.19156142558044112]], [[-0.0033510604145534782, 0.0029559815179513332, 0.007223734558267708, -0.32534222560391296, 0.0057134258856405672], [-0.0036764645078124444, -0.0031816313498189874, -0.0070964112811067459, 0.0046101738287826939, 0.039701117231450561], [0.012332317351241474, 0.0024633857634465046, -0.012897032238426391, 0.0029035887520191222, -0.0052679328320691943], [0.0044080175939052594, 0.0056685089079642698, 0.0038055040784473687, 0.0081600235282525829, -0.034871349121638452]], [[0.60492608706067064, 0.50446841122756381, 1.1719293734087317, -1.3534804766751878, -2.2521902132697349], [-1.0521867382766472, 0.55492289280083951, -0.62662991724741024, -0.96079115787808655, 1.1417425477953036], [-1.7449986683289664, -0.69510949645337539, -0.44881708841459428, 0.5635207118862634, 0.51151706495521176], [0.9836947769568789, -0.64798963949824617, 46.557513796247378, -1.388940504147794, -0.67267071086150243]], [[-0.68169970162476412, 0.53375647571127938, 5.2308884386293624, 6.9530864524904237, -0.81012309023593809], [-0.92379601744297801, -13.924932622301126, 8.3982300258889691, 1.4490135705190739, 1.9635444642355042], [1.3699090831032803, -0.84151555282461832, -1.8835486055747315, -1.4794605571741628, -2.6056604538709465], [-1.0600196375570956, 0.67560039915261694, 0.88372437402049797, 1.6504560794628667, 0.54162187742342682]], [[0.9902488310560642, -16.527520196941978, 11.243636761448544, -2.2283599675564281, -1.210546709545405], [14.338841692729607, -1.3005013489605493, 1.0539169058791888, -2.4072275435367434, 3.3928664613522268], [1.3612432498082061, 1.6368551587622342, -5.8849976153071477, 1.6825433425007219, -1.3517891326919302], [0.94393207198199003, -3.102662880967233, 0.91797236047904029, -2.147666559448854, -4.3252344600492689]]], [[[-4.1058593861373875, 1.1709428058023328, 1.1106068389980057, 1.750292576751652, 73.133814783139599], [1.0287093071690991, 0.97684078953038467, -12.548394666899721, -3.9146729406207927, -3.2233831649808433], [4.307690038160926, 3.6971574874869075, -5.7193247761278565, -2.2256862433037399, 2.459097969592797], [-3.1268353634632295, 0.96264539195790677, -1.1264523024355635, 1.0394720025297457, 1.1953535381795104]], [[-0.73102721916693503, -4.1560182795251279, 21.14823028987567, 0.99206177681680396, -0.7953817058506647], [-1.799439930680677, 1.822852211091929, 1.3306823388052349, 1.5396953900098906, -1.1898428476341638], [0.91453577064866665, 1.104453803650066, -0.75623379346457442, -16.284858693107413, 2.256248756061813], [7.6577692516307918, -0.70405998869017172, -0.88939925928312535, 1.3567823321145158, -23.650727273440477]], [[-0.20739319383856267, 0.51782475820338714, 0.42913960992371736, 0.5912137081753549, -0.10230858271904855], [-0.21390503230122296, 0.1978083549413529, -0.12871532097485408, -0.10200603486517217, 0.33840606194151762], [0.30927037510951755, -0.89508029595918071, -0.70623124401474235, 0.14138565863169972, 0.20862890071499898], [-0.10517570473859623, 0.16271673788191524, 0.13850651288935664, -0.15794857626222028, -0.25882481728303058]], [[0.14097259474509491, 0.176316804151262, -0.22531969059362394, 0.25037360501881684, -0.26918084465264391], [-4.2451582342915444, 0.14186455980386817, 0.35006598601441508, 1.4148334316365578, 0.14321221649959831], [-0.1638929283905794, 0.18048998558832022, 0.13190835742614421, 0.39409479492326666, 1.3741970678993269], [-0.24083951349198704, 0.14468402603409866, -0.18051178457884501, 0.13235028579285099, -0.12231565914583879]], [[-0.89423937247958796, 9.3686176778793246, -0.897922902041555, -1.7549402689191056, -1.2113874826081918], [-2.4725848869433222, 3.4839905815998944, 1.0246018422825993, -2.0122674956109705, 0.80316717624109268], [4.3885924071749329, -1.0708799163465228, -2.2978844624692156, -1.1485578248336665, 2.7916864347559898], [-2.9223448375965835, -4.8347031575005461, 8.0240535762991954, -5.8226655039675244, -2.5566049738812096]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[-0.32569948677910254, 0.40108063279695205], [1.5436364928257786, 0.52212455636117028]], [[-3.9502864679463445, -0.11970206752521051], [-2.2148407538423021, -4.2058237639089189]], [[0.49394151853283574, -2.6366852931788443], [2.8992191958799571, 0.45569668245095141]], [[0.6120905013641762, -4.7213927846397654], [-2.253868721265774, -1.9792396476562946]], [[2.4815808367829906, 2.4365046680454494], [-1.2039348784865691, 3.3122980088263017]], [[-0.0064412372082021108, -3.5161747133461771], [0.98659291425760021, -4.3502788736099962]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.4785580275839241, 2.0133519479435869], [-3.1489899667362851, -1.9521160468896284]], [[-3.5573293149839778, 2.9097875981899897], [-1.7397760150959583, -2.5516273772001807]], [[0.85808344952247673, -3.5194522560590782], [3.8408711052050144, 3.3505574307980233]], [[0.6875309980525568, -3.5284283475324161], [-0.090104565539088632, 2.8610374742653297]], [[3.8208448974352702, -3.3249635035040539], [-1.5807143167178461, 1.6054411515615739]], [[2.8195919809425956, 2.600079608244819], [1.1764426950432334, -3.1421228975613968]]])+(1.-msk_arg1)*numpy.array([[[3.4352548624024237, 3.3302754510506301], [1.9007250674670466, 1.9438780011099874]], [[-3.0384056952704643, 0.73042885505763344], [2.7138808905645293, 2.6616576075527343]], [[0.15295137754490629, 0.12270619380605474], [1.1218674176030099, 2.9256639984501529]], [[-4.3489377242349807, 1.621573480954237], [1.5921438565413943, -3.5812218336306301]], [[-4.8661469342482775, -0.57658365486691654], [1.8323037036064189, -4.0207966641719528]], [[1.0808870003988673, -1.5313357091098179], [2.9181859339459795, -0.46333239572303953]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.22028184264862449, 0.19921039299989796], [-0.49020051163441886, -0.26746594148082986]], [[1.1104640920668141, -0.041137733764371749], [1.2730608622168764, 1.648290734567927]], [[0.57563342913525961, 0.74917490033840772], [0.75483376465068996, 0.13600622937014253]], [[0.8902733158183892, 1.3381007971840044], [25.01392363173888, -0.69179088546001399]], [[0.64948483997577178, -0.73279140221470362], [0.76163976358889951, 2.0631699926244629]], [[-0.002284457202225689, -1.3523334832504483], [0.8386238602309003, 1.3845030940661964]]])+(1.-msk_ref)*numpy.array([[[-0.094810865517944903, 0.12043467235433027], [0.81213033870430662, 0.26859944711706613]], [[1.3001181751651203, -0.1638791604361873], [-0.81611568198985363, -1.5801520646286171]], [[3.2294022222049965, -21.487793006979743], [2.5842797022079935, 0.1557583791892552]], [[-0.14074483015777117, -2.9116119868101182], [-1.4156187658581563, 0.55267161309852408]], [[-0.50996833229951477, -4.2257609064686861], [-0.65706076788303858, -0.82379147355078697]], [[-0.00595921424332532, 2.296148840798709], [0.33808432244875036, 9.3891100941070587]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[3.5896155670278667, -0.30945507778214143], [4.4955053817460318, 2.2159246764882754]], [[0.94352354367141089, -3.4011097251776867], [-2.6419756615604353, -1.6498662550316165]]], [[[0.83996823792007458, -2.1274692354454441], [-0.70698066656675884, 0.88442904477667472]], [[-4.0359218674390682, 3.1818349053698221], [-2.1115849093129899, 3.9495505950413374]]], [[[1.6203614753017597, -3.6860862136525894], [-2.0886669498995758, -2.2814065651230186]], [[-2.5890749183276629, -0.86688994543306119], [-4.4639818339465211, 4.7834721676693768]]], [[[-2.2222363609047848, 0.11597232782900768], [2.3476791584878187, -2.969927586439761]], [[-0.44530609282278011, 0.78023591456256725], [-2.1432911005993671, 1.8782418141399759]]], [[[0.11254445835066829, 2.3631300172630878], [-2.4532764681604347, 3.4926139540800207]], [[2.2013847460219429, 1.5615658643857531], [4.3611855577631768, 0.36794367413702478]]], [[[1.5376977104301144, -0.6298245009486001], [-4.8590814641156648, 0.25318063774806188]], [[4.0408025920456865, -1.0630970915589399], [-3.4705745453403813, -4.346900174220524]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.380361091871313, -0.17925708634952908], [-2.0421546502509891, 3.5480927152751267]], [[-4.5427489988510539, -2.4400527347003518], [3.6727238330856622, 3.1386904841358962]], [[-2.5186630351940167, 4.9395499435377541], [0.95349464888995961, -3.2988763620999348]], [[0.50468104506087919, 2.8738872285181758], [2.3941847578033535, -4.3158489321605744]], [[4.7729717640775871, 2.4139309047463433], [-0.45419234538148334, 2.3541491108543697]], [[2.059218545766174, 0.561114185750343], [0.40200191215346059, 1.7888490408291622]]])+(1.-msk_arg1)*numpy.array([[[-4.906207761537031, -1.264208953173406], [-1.1870205702818861, -2.1744883807071789]], [[-0.15002407145973873, 2.148209839035891], [3.6991228907304592, 4.4313112817659839]], [[-4.1610793926761662, 3.142001276474037], [-0.020787709298963009, 3.2003378057676244]], [[-3.0678948341394752, 0.30990059847789109], [-3.6358798912918298, -1.0797598505021568]], [[2.8918040829923886, -1.3230399528905865], [-4.9114466740918399, 0.11027442396516207]], [[4.2474383173055266, -2.8677184644547973], [-3.539608087047311, 0.51578111799914872]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0619029948190279, -0.091544976815134307], [-25.07853649356073, -12.361713121720037]], [[-0.46202355123077876, 1.6654515977816258], [-0.74461855243700159, -0.46500088566701459]]], [[[-0.18490307039471435, 0.46832198652919649], [0.28973991279478595, -0.362463086227227]], [[-1.0988906465227644, 0.86634199846618565], [-0.67275984044483583, 1.2583434445045882]]], [[[-0.64334190507422939, 1.4635090768974757], [-0.42284559803512223, -0.46186526934659411]], [[-2.7153533806842178, -0.90917127478615434], [1.3531825215495272, -1.4500307506597205]]], [[[-4.4032491068427557, 0.22979330998059982], [0.8169002371392009, -1.0334182764614273]], [[-0.18599487419314503, 0.32588792991833587], [0.49660938885699263, -0.43519637588419985]]], [[[0.023579535751227801, 0.49510664090839862], [-1.0162993743262199, 1.4468574668863714]], [[-4.8468116391811158, -3.4381157680545433], [1.8525528130970479, 0.1562958235910088]]], [[[0.74673847202458188, -0.3058560745014372], [-8.6597016926562258, 0.4512105453357933]], [[10.05169992948478, -2.6445075493897456], [-1.9401159438986033, -2.4299983257422668]]]])+(1.-msk_ref)*numpy.array([[[[-0.73164768829587878, 0.063074189439787293], [-3.5559828701271687, -1.7528152058454269]], [[-0.79486705394443913, 2.8652491880321946], [1.2149872517144571, 0.75873767350049182]]], [[[-5.5988897631370644, 14.180852544162443], [-0.3291022383940152, 0.41170514570103789]], [[-1.0910483340665933, 0.86015928623055582], [-0.47651468719919687, 0.89128258971401952]]], [[[-0.38940893032556073, 0.88584856615338725], [-0.6647568750333176, -0.72609982122070293]], [[124.54835119600304, 41.702042921886822], [-1.3948470770496686, 1.4946772678336142]]], [[[0.72435219622777847, -0.037801924152832699], [7.5755876884997582, -9.5834845141534686]], [[0.12247546842493817, -0.21459342384529348], [1.9849701761022145, -1.7394995871225203]]], [[[0.038918424319468153, 0.81718192154212677], [1.8542723995601946, -2.6398401245928622]], [[-0.44821513743280006, -0.31794417571977351], [39.548477343585709, 3.3366184189115842]]], [[[0.36202943881845306, -0.14828337786154025], [1.6944067293716922, -0.088286434280847675]], [[-1.1415960447238283, 0.30034316382347287], [-6.7287739396193045, -8.4278001317366922]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_expandedData_rank4(self):
      arg0=Data(numpy.array([[[-0.85253873915565936, -1.1273242326822173], [2.7731905129587826, -3.4413681419988662]], [[-0.67066340414812053, -0.63597694886321943], [1.300399958598339, -3.9676015870535553]], [[-0.57366198274240077, -3.658981774862291], [-4.3602454182325285, 4.2364978623322518]], [[-0.25503799156517282, -3.4090244284610116], [2.1302237811874107, 3.4410099665684655]], [[-4.6300577039525157, -3.2372952664497747], [2.3260386947983882, 3.8291258105741459]], [[-2.3176875232645875, 1.6401814028373991], [4.7298164787813732, 4.7751483096688201]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.35293508954585029, -3.3327674316389588], [1.6050141836569223, 2.310472770019544]], [[-4.5750220578203997, -1.5098813430296456], [3.5799845006614976, 4.3837802615144881]]], [[[-0.91448491880151295, -4.5937603890058698], [-2.3991976594054689, 4.6517101021350911]], [[-4.4960981407647962, 3.70564619901665], [1.7412584985538446, 1.7037283218764721]]], [[[2.5934489154593834, 1.5428199837387], [-2.6766652563555446, -2.3710750368006095]], [[-4.8931440000620166, 2.0571842765516868], [1.7495636173820017, 1.1801552585932829]]], [[[-3.745727957504462, 2.0371296987513601], [-3.3085474902626144, 4.0963274761617932]], [[-0.6902548143890419, 4.3630106098439221], [4.8845312883914946, -0.27740268093997233]]], [[[3.5093367840786964, 1.6761430741283823], [-2.996095257108534, 4.4068999994831195]], [[-0.3100524034707739, 1.4344414826437193], [2.7123302078496261, -0.66309967721481566]]], [[[-4.8521966531106919, 4.0018614440541729], [2.4276653824757943, 0.5605898868766932]], [[-2.4049807310099727, 1.230272541528632], [4.0969488952564621, -2.5824588564688633]]]])+(1.-msk_arg1)*numpy.array([[[[0.73384543904612709, -0.38831484649754877], [-2.0842505986826545, 1.0746805417274778]], [[3.5112551805447705, 3.2977181958573851], [-3.0094218746669501, -4.105265976442757]]], [[[4.8418278348073098, -1.3798566554710479], [-3.3463597816409099, 1.0458754718672996]], [[1.3050532043928085, -4.222080109950074], [4.2804955886604219, 4.6730000237230236]]], [[[4.4656466594633475, 3.5042901106559405], [-4.8286421946554725, 0.26724396013268858]], [[-1.7903161380897501, 0.52690440203556488], [0.48838681879102985, 1.7189592523602943]]], [[[2.3573086234646512, -0.24436857186734073], [4.6619833657082861, -0.64750402684730535]], [[0.60739776743886953, -2.5720451148361234], [4.8683703421325166, 3.0353966286307177]]], [[[-4.9361974000089779, 3.1095797060424157], [2.4433787023216667, -0.23171455145996234]], [[4.9026952218173214, -4.1809326576651333], [2.2597721585706356, -0.32189001189186239]]], [[[-1.9137022362268254, 2.7654996478226757], [4.394964550027014, -4.9076719514707579]], [[4.0286751310097006, 0.1826427957314225], [1.7848615869238413, 0.59036897935498978]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.4155680871876153, 0.25580504990004821], [-0.70237649247041611, -0.48791928964074371]], [[-0.60615893823252232, -1.836694337446577], [-0.96128017910775354, -0.78502295660457178]]], [[[0.73337830986547592, 0.14599442447046265], [0.26507901354856156, -0.13671895601819897]], [[-0.28922855282183368, 0.35092393843303771], [-2.2785827551444773, -2.3287759768433456]]], [[[-0.22119656158362647, -0.37182690708494165], [1.3669926660325971, 1.5431741796748482]], [[0.89109280621564912, -2.1195210696152613], [2.4214597401560218, 3.5897801001048428]]], [[[0.068087697360458679, -0.12519477366683918], [1.0303688970746561, -0.83221481883455861]], [[-3.086141142054784, 0.48824629864093205], [0.70447086187088603, -12.404386125284319]]], [[[-1.3193540514430966, -2.7623284523967087], [1.080504786611497, -0.73459694271017573]], [[-7.5020824504514598, 1.6215640184299662], [1.4117476550209318, -5.7745855444500158]]], [[[0.47765737643356698, -0.57915236588416308], [0.67562087208439769, 2.9258133998378244]], [[-1.9666754156467128, 3.844527386512671], [1.1655376798079156, -1.8490704305733414]]]])+(1.-msk_ref)*numpy.array([[[[-1.1617415518229739, 2.1954832447052497], [0.5408774901610871, -1.0489854323314705]], [[0.78980033360278945, 0.84094223588979866], [1.1435313111026413, 0.83828140776906168]]], [[[-0.13851450878257238, 0.48603846021902408], [0.19005038022282347, -0.60808094842089577]], [[0.99643443977700941, -0.30799983058912545], [-0.92690238895800692, -0.84904805626183788]]], [[[-0.12846112254016526, -0.16370276564659811], [0.75776618505968263, -13.691541515271588]], [[2.4354611598847944, -8.2752115969952023], [8.6744721588093423, 2.4645714297852832]]], [[[-0.10819032731926763, 1.0436611779342237], [-0.73123908024564244, 5.2648698496278685]], [[3.5071313978805549, -0.82822177919812145], [0.70680940946271209, 1.1336277882474692]]], [[[0.93798066178311557, -1.4889657579625843], [-1.3249257118324469, 13.971048628808894]], [[0.47444081052547599, -0.55634445355964623], [1.6944742840783413, -11.895758392964751]]], [[[1.2111014343768991, -0.83807189239360125], [0.37319559331310592, -0.33420762819036032]], [[1.1740376984916991, 25.89654007342617], [2.6753605683781081, 8.0884133087174224]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-1.4821869317147618, 1.7675413288555211, -4.8043974366004516, 0.24335601519388739], [4.075136991901374, 4.6591318470498688, -2.5695659175902961, -1.5928624426891567], [0.92374425601878141, -2.888577131052755, 1.0316999411960328, 0.40280549387157372]], [[-2.3534195875967989, -1.2010750416483118, 1.5841401354744082, -0.16926775468789934], [-3.9743355340556921, 0.49769130305389542, 0.22021186555486771, 4.9506949412054002], [-4.1772137418512658, 3.8258264316097428, 0.76401131952560775, 4.5507526663744748]]], [[[-1.0114376220297574, 3.2744143991152441, 1.5984324079495433, -2.8393319211799861], [-0.61630266398054356, 1.6398402051438676, 0.53937654765638321, 3.2676703072782285], [2.4847094997165451, -4.083815606455083, 2.5424163367484445, 2.2237011601461223]], [[4.3100463074382631, -0.34337844799462669, -1.8587727555530864, 1.7289326028841359], [-0.72377332350811319, -0.97102826864458969, 0.89069755972107867, 4.6647191815735063], [4.0661719951691708, -2.979600017609668, -2.0262080413146091, 1.164914244360336]]], [[[0.46214558117336146, 1.7395166850000523, 1.034043363141782, -4.2925108147331068], [1.446535780011498, -0.19613209498162387, -3.8172995638572091, -1.0917726824942955], [1.8429804236773073, 2.0257662211866663, 3.0581769064543316, 0.26133329910483649]], [[2.5030574378559507, 1.3025655587298433, -4.8318910378415492, 0.67331436141337964], [0.41312152933602686, 3.6350700909723734, -4.5019584386818643, 0.62306044230073798], [3.5678874491604038, -4.9914495904881919, 1.1575250708468658, 3.054254762820543]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.67718855462366, -1.0272064714895013, -2.3488924979489587, -0.39110884299801807], [-3.0281956051450356, -4.212898736505263, -4.292134441228594, 0.035010207990150199], [-0.62578287399790788, -1.4769389201343408, 4.8460818500348548, -0.80451287239161218]], [[-2.9399486095210037, 3.2723703954548924, -3.7351811085278386, 2.6705051354871667], [-4.3939506504788506, -1.1216658711941321, 0.79409260073144239, 3.7952425102442433], [4.4290829988471074, -0.15069756544037816, 2.184508238788279, -3.0065387678878617]]], [[[-4.7194841859289465, -1.7469020439912497, 4.2182001914623477, -0.16319303635225957], [0.099271723967886416, -4.3348430200183508, -0.63369781247245527, -3.2646254368526897], [-4.6271295030135668, -2.2902496239691548, -1.0453058005569194, 0.51995557729700526]], [[-4.6432936925061385, -4.50111304848794, -2.0693721289619127, 0.27899932547317707], [4.8897706298441879, 2.7525313508528857, -0.01426959199627742, 1.7147762806760571], [0.77013459112591587, 1.5939980187696516, -2.9275215932212162, -4.4553225443453961]]], [[[-0.37600447950270421, -3.5025699965518275, 4.2879894555123936, -4.0647522543213945], [-1.5042801943714537, -4.6015403503595529, 2.7268618628093568, -2.8916391983737002], [2.9969146235288626, -2.4576073255346556, -0.53664958229594184, -2.4005877016289223]], [[4.5725695641107134, -4.3753093717787586, 3.4516878228406505, -1.0453874293457313], [-4.6348330152047632, 0.49355357808341616, 3.1547688787176771, 1.0270658812638365], [-0.33446662483860301, 4.4987226034594983, 1.6073410491750497, 2.050831019135523]]]])+(1.-msk_arg1)*numpy.array([[[[1.3222399745487357, -3.322394667309243, 4.5318973147224, -3.4777330301770348], [-3.6932345570214786, 1.5958419456354003, 2.4280088080869264, 4.1639043526389248], [4.6944248776055275, 1.7846397516439447, 1.9727787707936013, -0.81591848083771801]], [[2.1980396595691731, 0.41390334674174589, -3.2662969920560716, 0.48349601235455086], [-1.1884773948930629, -2.1069716048772333, 1.1296491041142094, 4.7948119291528961], [-3.1869767594354315, -4.2505729548633511, 3.9963101662366434, 4.5832314911208432]]], [[[-0.25952981776564599, -3.9426261669189557, -2.3164863207423858, 0.31893045288787114], [4.555432644638886, -1.276845309329099, 2.5465007204422454, 3.0597578031534862], [-0.057370352698676186, -2.7403287737993489, -2.9308791893838402, -0.1464191139551847]], [[4.1550982791234023, 1.1696400061384935, 0.97015472217102783, -2.6805076120274673], [2.7763798402449709, 1.878596821342386, 3.2305540059712605, 3.1249937969770087], [2.7645710682494071, -2.6511681944295216, 2.4051645875962553, -3.3720513950251654]]], [[[-2.6720219638459297, 3.8638474197969384, 4.4177323320873345, -0.43514662019358941], [-4.0256997263438157, 4.7515465499912359, 0.79168659529682106, -3.193409481845296], [4.5661558901602, -3.9543134238176272, -0.48642842256762542, -2.2640808582183949]], [[2.6983159194389383, -0.53353091055053259, 2.5536634064427775, -0.16293924736562637], [3.3156394180610036, -0.73530886185366917, 0.23364563313484599, 4.9485346091761322], [4.7670952053914721, 2.3445831412489273, 4.4484472272353948, 4.8656894517972589]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.88373303504169565, -1.7207264341826982, 2.0453883865675535, -0.62222069265542168], [-1.3457310964250591, -1.1059206827541197, 0.5986685535541556, -45.49708596810661], [-1.4761417967821691, 1.9557864524214807, 0.21289362687685773, -0.50068247220723194]], [[0.80049684541262578, -0.36703517527127311, -0.42411334000850404, -0.063384171196143646], [0.90450163194768041, -0.44370727133210386, 0.27731257708739454, 1.3044475887488933], [-0.94313286586379097, -25.387446840495834, 0.34974064458067494, -1.513618488801808]]], [[[0.21431105226400371, -1.8744121402674629, 0.37893706685253492, 17.39860955250051], [-6.2082397620082874, -0.3782928695620737, -0.85115734509471441, -1.0009326859955101], [-0.5369872397343316, 1.7831312201582461, -2.4322225471186449, 4.2767137371735817]], [[-0.92823038835434701, 0.07628745252465452, 0.89823030354889721, 6.1969060317687221], [-0.14801784752246674, -0.3527764609633644, -62.419273091580997, 2.7203077358490302], [5.2798199717591414, -1.8692620583741451, 0.69212402942009654, -0.2614657486109106]]], [[[-1.2290959453051882, -0.49664009190752878, 0.24114876537592114, 1.0560325811172313], [-0.96161325890215321, 0.042623139220391404, -1.3998874002089809, 0.37756186287290761], [0.61495926817801716, -0.82428392857510635, -5.698647697386753, -0.10886221691776077]], [[0.54740718599493909, -0.29770821856199214, -1.3998632801807167, -0.6440811726947796], [-0.089134069767079938, 7.3650972303517683, -1.427032727833863, 0.6066411645707116], [-10.667394544618881, -1.1095259766960934, 0.72014901345358717, 1.4892766563029598]]]])+(1.-msk_ref)*numpy.array([[[[-1.1209666628182331, -0.53200823678392972, -1.0601293681109683, -0.06997547341392657], [-1.1034059518786405, 2.9195446703180803, -1.0583017281617297, -0.38254059358487924], [0.1967747445327005, -1.6185771545164247, 0.52296788492964408, -0.49368350310928877]], [[-1.0706902295193714, -2.9018249093736368, -0.48499574267960927, -0.35009131484578648], [3.344056480277688, -0.23621168026272207, 0.19493829079565572, 1.0325107667111448], [1.310713587566686, -0.90007311302170956, 0.19117918473407264, 0.99291355350274368]]], [[[3.8971923563059718, -0.83051607240614977, -0.69002454002719038, -8.9026679499252204], [-0.13528960080352556, -1.2842904251302762, 0.21181087573488327, 1.0679506410312807], [-43.309991708903674, 1.4902648344611027, -0.86745859261532288, -15.187232732652259]], [[1.0372910621857903, -0.2935761825796922, -1.9159549637541267, -0.64500193736678679], [-0.26068959045756934, -0.51689019038727191, 0.27571046887770323, 1.4927131010903014], [1.470814782759037, 1.1238819264165238, -0.84244049316376346, -0.34546159233484702]]], [[[-0.17295725387982216, 0.45020325494412805, 0.23406654939938537, 9.8645160401876524], [-0.35932530450433209, -0.041277527836065427, -4.821730703203353, 0.34188308411466856], [0.40361749971104199, -0.51229278108939669, -6.2870029064331048, -0.11542578002734392]], [[0.92763690857088821, -2.4414059859920201, -1.8921409241526923, -4.1323031270820874], [0.12459784591945214, -4.9435961941334172, -19.268318342948053, 0.1259080700669222], [0.7484405692433429, -2.1289283807736137, 0.26020879010545001, 0.6277126382762428]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_Symbol_rank0(self):
      arg0=Data(-2.36328790413,self.functionspace)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-3.75331845213)
      sub=res.substitute({arg1:s1})
      ref=Data(0.629652914951,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-0.95302014339889674, 4.63367368941908]),self.functionspace)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-4.19113982526)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([0.22738925045063926, -1.1055879504415023]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_Symbol_rank1(self):
      arg0=Data(-2.85540279248,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-4.2782612401545155, -3.7969047362895028])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([0.66742132660905862, 0.7520343518743281]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[4.0706461137180465, -0.99816491755884496, -2.2227129164936099, 2.7042926284130866, 2.4127583338918326], [-0.4720372294292936, 2.2861585159798086, -2.496567002959309, 0.26451254653197687, 4.5930265477126149], [1.0798055174176868, 4.8625344417338905, 3.6609139071635433, 3.7169691154918461, 4.9065818317334475], [-0.19984959800162727, -3.9128876995365647, -3.8944378235077179, -4.1412621007219483, 0.56241187424379202]]),self.functionspace)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(1.84704404518)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[2.2038706247086628, -0.54041208176001454, -1.2033892328081863, 1.4641191884234115, 1.3062808871214826], [-0.25556360210293155, 1.237739036025777, -1.3516553703592398, 0.14320857546524743, 2.4866903199711561], [0.58461276017471353, 2.6326034045694322, 1.982039311252197, 2.0123879152687305, 2.6564509084307892], [-0.10819969265108381, -2.1184593349275356, -2.1084704686214071, -2.2421025159232393, 0.30449294141688893]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_Symbol_rank2(self):
      arg0=Data(-3.29972887079,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[4.2595443930312324, 2.8062346419112725, 2.9424101132040059, 4.3294990416160477, -4.2272099469591682], [-4.9919228146572792, -3.2873724320988673, -4.214511195135322, 4.761470110190384, -3.0472476448806263], [-2.9902432101061005, 4.0135910952562277, -2.3482076906035187, 3.5095687847989652, -4.4177021744696425], [-2.0280185107272777, 0.081900203219675838, -3.3098366573852642, 2.3273384333911684, 2.7505290137532423]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.77466709260970545, -1.1758563669302062, -1.121437442042498, -0.76215027167665239, 0.78059261597951335], [0.66101360003059673, 1.0037587583846472, 0.78294462109870544, -0.69300631830691883, 1.0828554995636863], [1.1034984912395789, -0.82213877609293984, 1.4052116786754094, -0.94020920321711032, 0.74693330162928395], [1.6270703907964554, -40.289629830842046, 0.99694613733543813, -1.4178122199371912, -1.1996706285562218]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[3.9005556268051258, 3.7507568898745909], [-3.3927697693821268, 4.284157753905701]], [[4.7701405936230152, 1.2140438394065436], [4.9338882074444985, -0.557351328392377]], [[-0.99735509453998539, 3.2833451172592358], [-0.72038009096878408, 3.7989464932473105]], [[-0.012921345580538812, -0.55603010013268861], [-1.5000784438800183, 2.5159464120586268]], [[-1.8631678473634095, -2.5026574459427096], [4.4452666229244322, -2.7709028097957136]], [[-4.6961228044365422, 4.8465925473520084], [-2.1429620269585659, -0.3967096430348569]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-1.47801317586)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-2.6390533525028257, -2.5377019306232147], [2.2954935888177435, -2.8985923967853915]], [[-3.2274005885212245, -0.82140258235350916], [-3.3381895966948831, 0.3770949660627137]], [[0.67479445435762297, -2.2214586249149457], [0.48739761101795492, -2.5703062430608052]], [[0.0087423750962114365, 0.3762010442213764], [1.0149290063019443, -1.7022489739235551]], [[1.2605894709127128, 1.693257872671595], [-3.0075960725642896, 1.8747483818453994]], [[3.1773213399764715, -3.2791267537391655], [1.4498937235163807, 0.26840737925300817]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_Symbol_rank3(self):
      arg0=Data(-4.30515743852,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[0.95743070823134691, 2.2124073898487415], [-1.0192093819003709, -3.1191587814683452]], [[-4.3763246075984359, -4.8648317985330127], [2.1159563889959774, -1.6649100062854592]], [[2.4815817724464289, -3.8424034360991142], [0.47470070849252455, 0.27930027092093557]], [[-3.8540438717375523, -1.6610204971596119], [0.56371761640816498, 1.7854490352854127]], [[-1.5045189308665741, 2.3677258093825877], [3.5661233349646331, -1.4721977236702877]], [[4.5071174024019633, 4.419076847187565], [1.0004862004879334, 2.6325781934724626]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-4.4965733828106016, -1.9459153220478591], [4.2240166887900683, 1.3802302928906758]], [[0.98373814205731136, 0.88495504404012393], [-2.0346153923150454, 2.5858199075418562]], [[-1.7348440765965187, 1.1204334761081882], [-9.0692037351074948, -15.414082572577536]], [[1.1170494114218907, 2.5918749623382182], [-7.6370816047043455, -2.4112463326797662]], [[2.8614843922494808, -1.8182668877699135], [-1.2072373931400422, 2.9243065447659649]], [[-0.95519088014531062, -0.97422099397490913], [-4.3030652860768779, -1.63533886636048]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[-0.21310175550683841, 3.9762845513412124, -1.6554950425239268, 1.7367120759826253], [3.198592808270929, -2.538136535103539, 4.9546270186788917, 0.53883615786825967], [0.94146260494102929, 3.6885738146172304, 2.9233832537858708, -2.8323079648349649]], [[0.94076507067363568, 0.46425235159379064, -3.8723385496826648, 1.8234025329356633], [1.7850095234257113, 1.9772930432652913, 3.9819327664444266, 4.5420190217835437], [3.8948156322855105, -4.5672745565001893, -0.5264345613064858, 1.110078810620748]]], [[[-2.0571583333595078, -0.3533500737889419, -4.1370177946073792, 4.5702609677972248], [3.632309229755041, 2.514885840581087, -0.72944591362014588, -0.47552727648322701], [-1.4681752311055938, 4.1467044620209688, 4.913809210593076, -2.6542276870992341]], [[3.1527397029130189, 0.77558770629207707, -3.0480513381955832, 2.9639370909875717], [-3.760969212139079, -2.8789559713103676, -1.3822458829825468, 2.0391446235105173], [3.285903012423093, 4.4229214720225443, 1.7930106486549846, 3.9678387953884293]]], [[[4.2985830689943558, 4.7290825547552657, 2.0378124378309606, 2.7618170652071221], [-0.83281782440729391, -3.4764382309665631, 2.3231297534445083, 3.5852931150606242], [3.9344221507673858, 4.3781885913442782, 2.1627618146632059, 3.671748405197647]], [[4.3520994090920908, -0.27700311411235656, -1.9661484245369873, -3.0471676378382293], [1.6368849407610675, 0.65551385121381323, -1.931152455779964, 1.2575942604472834], [4.6437907431772079, -0.95099108538706467, 1.5193073746916053, -2.367932927877876]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-1.2622987517)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.1688203804528958, -3.1500344478619482, 1.3114922598927332, -1.3758328396084789], [-2.5339427801610319, 2.0107256952363493, -3.9250827207227115, -0.42686896199772095], [-0.74583184343227338, -2.9221084229514358, -2.3159202604406417, 2.243769916610856]], [[-0.7452792529567972, -0.3677832612678793, 3.067687854776326, -1.4445094954606998], [-1.4140943425832109, -1.5664224024667821, -3.1545089948704255, -3.5982124007288219], [-3.0854943229936271, 3.618219973961923, 0.41704434912767441, -0.87941052712575596]]], [[[1.629692123669308, 0.27992586803534675, 3.277368205460915, -3.620585825384194], [-2.8775353099807437, -1.99230636740989, 0.5778710567835349, 0.3767153186546679], [1.1630964770668482, -3.2850420365572077, -3.8927466291034212, 2.1026937430852057]], [[-2.497617698403455, -0.61442483821591776, 2.4146830012276643, -2.3480472328765689], [2.9794604542550207, 2.2807247233970154, 1.0950227758075612, -1.6154215638475844], [-2.6031104031462653, -3.5038626680678235, -1.4204328779089503, -3.1433436736364144]]], [[[-3.4053611026809039, -3.7464051583625597, -1.6143661990397171, -2.1879266389929133], [0.65976285192958251, 2.7540534491464284, -1.840396142608776, -2.8402888858428921], [-3.1168708243397658, -3.4684250344505045, -1.7133517812279173, -2.9087792412526077]], [[-3.4477570410637588, 0.219443387502194, 1.5575935743361082, 2.4139829289538342], [-1.2967492351231782, -0.51930167112352899, 1.5298695757898422, -0.99627307620783812], [-3.6788365170537256, 0.7533803579445626, -1.2036036418859832, 1.8758894633247374]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank0_Symbol_rank4(self):
      arg0=Data(0.461078184276,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[2.7256903096658149, -3.4491703768610638, 0.35152672103538141, -4.4679110961304858], [1.2376905485939504, 2.0078899201626941, -4.0744644997703094, -4.5035694213420738], [0.31992906680158484, -2.0389432332582258, 0.80390253109016463, -4.0767192462892012]], [[2.5838070697325133, -1.1977398450703292, -0.30979704327747637, -4.5534119333239005], [4.3056443749058992, 3.5679562631563329, 4.7620202033411694, 2.4663036455852456], [-0.79669310512782765, 1.533132799132475, 0.59064616499169809, 3.0174544402461496]]], [[[-0.56921727557204349, 3.675363616749669, -1.5823297537242054, -3.1390463297492688], [2.7880504578367145, -3.6909923543238854, -2.133803998175301, 0.53943126597150393], [2.9585558144595172, 4.2434834304322457, 3.2377753315524682, -1.6695143373647001]], [[1.4377899746628913, 4.1366674635877505, -4.6650068490484777, 2.9718311310696999], [2.3640098228772723, 2.8919071529272911, 3.1716608977626422, 4.4686955074938979], [1.042151095991251, 4.9512059506825636, 0.47502809953480352, 1.4983520478032828]]], [[[-1.5193789708919789, 2.5322847295030195, 3.0067398452301894, -4.6830799577314242], [-3.792471344167395, 2.5069751796548303, -0.74845371611289835, -4.8427186454024191], [-0.92172660565845188, -4.8560513848092572, -4.3511193272866482, -0.23692735145465882]], [[-2.2827083053463291, -0.41187998749871646, 4.2850481641475682, 4.245615142241272], [3.2012191598179562, -3.8759808882278368, 3.5224680491323479, 4.5206451054297023], [0.7762726302975036, 0.72413333116912604, 0.52752476760134215, -1.2182504989646405]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.16916015096821369, -0.13367799612591857, 1.311644767480588, -0.10319770791209149], [0.3725310699022979, 0.22963319833704152, -0.1131628915411237, -0.10238060994256064], [1.4411887887686681, -0.22613586133973171, 0.57354986014337161, -0.113100303557906]], [[0.17844915345146073, -0.38495687203976475, -1.4883233855227334, -0.10125993233805412], [0.10708691757334077, 0.12922753258970202, 0.096824071420814381, 0.18695110194602743], [-0.57874002085368548, 0.30074249571633194, 0.78063350209367255, 0.15280369377774394]]], [[[-0.8100214172387844, 0.1254510389596292, -0.29139197009375756, -0.14688479743224911], [0.16537655657545849, -0.12491984269095743, -0.21608272581265772, 0.85474872029390925], [0.15584569404511919, 0.10865558728685798, 0.14240586114249806, -0.27617503722877301]], [[0.32068535210351024, 0.11146126400882395, -0.098837622150482501, 0.15514952362374068], [0.19504072267960423, 0.15943740925739841, 0.14537436350806635, 0.1031795931278936], [0.44242930420478799, 0.09312442036714369, 0.97063349458106096, 0.3077235319640988]]], [[[-0.30346489790166181, 0.18207991340932525, 0.15334821368303889, -0.098456184484840975], [-0.12157723616943827, 0.18391812891388473, -0.61604101142055978, -0.095210607519684187], [-0.50023312926536057, -0.094949197967306359, -0.10596771763627869, -1.9460741085602506]], [[-0.20198734248952246, -1.1194478932460956, 0.10760163401042878, 0.10860103161217338], [0.14403205818057346, -0.11895780644224978, 0.13089634252019372, 0.10199389103158528], [0.59396424178830043, 0.63673106102055477, 0.87404082726236965, -0.37847567857967201]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([-1.4702882833190145, -4.8893431524998174]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-1.2799793333815068, 0.58433322499013229])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([1.1486812677160505, -8.3673885779511927]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[-1.8243733668599358, -3.0933793707107715], [1.8478906132360748, -4.758340753539545]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([3.944232746984941, -1.3734077054203384])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.46254201612583012, -0.78427911564686925], [-1.345478553777677, 3.4646236035811597]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_Symbol_rank2(self):
      arg0=Data(numpy.array([3.4559152888462759, 4.2411111955228229]),self.functionspace)
      arg1=Symbol(shape=(2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-3.5001951242520546, -0.59095894778104974], [-3.3514204306692186, -2.0318373681573165]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.98734932372799056, -5.8479786147965935], [-1.265466772450256, -2.0873280814640731]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_Symbol_rank1(self):
      arg0=Data(numpy.array([[[-4.1793936083498897, -4.5480180902686298, 3.3099153054387784, -1.3249783787827942, 4.1848330229495012], [3.6373810458475209, 1.3335543121009756, -1.9057647425217827, -3.5065471637401133, 4.3089624556948234], [2.5646995847581806, -1.0725566109764015, 2.8916786966264159, -2.9213296174404735, -2.450024286342579], [-4.4639993236418984, -3.1542667781848079, -4.687564020455409, 3.3925192063998093, -2.4200887736010914]], [[3.5431868242317748, -4.6394935338611374, -4.95443758953434, -0.49742589992909281, 3.1900621132083664], [-2.2790143761496759, -2.3690028997274215, 4.1556437940939812, 1.7307835326696122, -4.2727839465178281], [-3.3636241814816739, -2.4672993418804756, 3.6531156969584053, 4.5892965554133465, 2.4095463861724369], [0.2300631688008794, -3.7121475990162836, 1.5244574969180213, -0.098957080507147133, 0.56597038688835344]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-2.1391088046363738, 1.8077251010934061])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[1.9538013210414247, 2.1261275164737334, -1.54733377669465, 0.61940672485242132, -1.9563441625218683], [-1.700418902471788, -0.6234158399098666, 0.89091529070011144, 1.6392561033547752, -2.0143727361391988], [-1.1989570512726457, 0.5014034857187758, -1.3518146857976077, 1.3656760287782879, 1.1453481379873323], [2.0868500536141412, 1.4745705180344955, 2.1913630621759075, -1.5859498119248321, 1.13135375271969]], [[1.9600252395060904, -2.5664817792566637, -2.7407029899278594, -0.27516678261989269, 1.7646831983905358], [-1.2607084864679976, -1.310488468791259, 2.2988250766559761, 0.95743735130011987, -2.3636248365049677], [-1.8606945156911188, -1.3648642375923887, 2.0208358531663975, 2.5387137417284857, 1.3329163735763905], [0.12726667824755281, -2.0534912065838902, 0.843301614828466, -0.054741221686468118, 0.31308432158519295]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_Symbol_rank3(self):
      arg0=Data(numpy.array([1.0733338097850966, -2.1819363388006128]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-1.2577202649397368, -4.1045420851425085, 2.051756061663438, -2.5666803084993228, 0.67297255635885733], [4.3210680859867274, -4.5459825189213188, -0.30956836588074488, -4.5352627763984232, 1.5245848485094724], [-2.015841019769935, 4.8101503002032171, 2.0535111075262833, -3.4112113131285291, -0.34076334873356462], [2.7222193357638229, 2.9481060061342692, 4.6499016490498022, 2.0041669087502703, -3.2602550463624613]], [[-1.4484732227052186, -0.022011596021948243, -1.1823856079435369, 3.3352633240270517, 4.6733527867929574], [4.0096340275026137, -3.4422479426346451, -3.8013098971709756, 0.37350299211256477, 0.47470895613541053], [2.835552873441415, 4.8109728885630361, -2.003270956686781, 1.5610019460814781, 4.154063529355982], [-0.43133822112771192, -2.9325151191939813, -0.4048264918295077, -1.7962714883435149, 0.23866193547268821]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.85339629145319129, -0.26149903875277986, 0.52312934750874007, -0.41817978118694865, 1.5949146806110617], [0.24839548658488633, -0.23610601345642213, -3.4671947397835132, -0.23666408380364246, 0.70401710395748296], [-0.53244963231653786, 0.22313934966642329, 0.52268225180338301, -0.31464887726369212, -3.1497924109916902], [0.39428630738306458, 0.36407571761387081, 0.23082935743478147, 0.53555110859224331, -0.32921774355740629]], [[1.5063698138137156, 99.126675622474465, 1.8453678090648817, -0.65420212043890646, -0.46688885653290157], [-0.54417343923021921, 0.63386960357381794, 0.57399591136320172, -5.8418175620478827, -4.5963664906676343], [-0.76949238338570292, -0.45353328512568769, 1.0891868279313186, -1.3977793841179007, -0.52525348333776822], [5.0585276980464444, 0.74404947634177265, 5.3898062079384204, 1.214702985021908, -9.1423725969502474]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_Symbol_rank1(self):
      arg0=Data(numpy.array([[[[4.8996422101793975, 1.3146695711952301], [-3.7328957228221507, 2.4098055974030608]], [[-3.2829690761411956, 0.28868569290364743], [3.515478445895484, 0.11781962705613047]], [[-4.2244781597264094, 4.843156955897026], [2.178785565323488, -2.3877431570168319]], [[3.2538740916393394, 4.1760794659328369], [0.88529407019919226, 3.5899924358139366]], [[-4.205842216336622, -4.4754875709103121], [1.0559528014698198, 2.8755944563440528]], [[-1.6129082168076501, 3.6089644475943512], [-4.9012607923543685, -2.1379428341645657]]], [[[4.9628750624554705, -3.6460599686697215], [-2.0537188965258202, -4.7390479754136265]], [[-0.67403414226186875, -4.2362131209178173], [2.2639385436049348, 4.7731809355893748]], [[4.0736412152966022, -2.4954327840001032], [-4.1205869342678074, 3.8918881685797384]], [[3.1180215774077933, 0.98411700771551303], [2.0721537911922683, 1.6333023350267979]], [[-0.78908753108271767, -3.1533882248249512], [-2.6613448848978125, 1.5703266481653682]], [[4.3989954891072394, 4.1022092738289615], [0.50175961381276757, -1.9602827517897503]]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-3.5771981747102046, -2.9564838828683051])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-1.3696871045105927, -0.36751376551894105], [1.0435249993172546, -0.67365728139964831]], [[0.91774872841847932, -0.080701621437854615], [-0.98274634901385616, -0.032936287368444513]], [[1.1809460794183264, -1.3538967424664357], [-0.60907600275738016, 0.66748976165131457]], [[-0.9096152722662455, -1.1674163023610367], [-0.24748253436389825, -1.0035766151269403]], [[1.1757364313978342, 1.2511153568596685], [-0.29518990838559411, -0.80386780823989712]], [[0.45088589953177943, -1.0088802105258594], [1.3701395765560092, 0.59765848290967671]]], [[[-1.678641000281935, 1.2332419567031117], [0.69464910951361403, 1.6029338102854542]], [[0.22798505554778742, 1.4328551376400374], [-0.76575372411924647, -1.6144789299370563]], [[-1.3778668772394793, 0.84405424919113647], [1.3937457796218795, -1.3163907948667481]], [[-1.0546384492320549, -0.33286736769244546], [-0.70088452137338142, -0.55244756938847561]], [[0.26690067064298184, 1.0666008507936173], [0.90017229598960091, -0.53114669667736436]], [[-1.4879145848207522, -1.3875297266457962], [-0.16971498363994908, 0.66304530295221298]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank1_Symbol_rank4(self):
      arg0=Data(numpy.array([-3.1596158513980619, 0.5824317387910094]),self.functionspace)
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-2.724438003064773, 4.6092258196128135], [-4.3030726901360792, 3.082730560358188]], [[4.0693147786696358, 1.1190601058660521], [2.4503445646893667, 1.286438412644701]], [[-2.4010040613500969, -2.5430513029357558], [-1.4403804975248127, 3.8004813499742056]], [[-3.4669500866477332, -0.031640639302237261], [0.17554500780936433, -4.1427448962175362]], [[-1.9126598707487954, -2.4068773466275006], [2.5990542945078428, 3.5105809110938537]], [[0.30022834915510277, -4.0037270655358546], [1.2150321185018047, -4.9776195201702382]]], [[[1.9932078347222326, 3.0152707521580346], [-4.5181299127761125, -0.83078717883664144]], [[-4.0215529692228689, -3.540252770122275], [-4.7935738144826825, -2.2778119131914289]], [[0.37177170893906641, -0.17480152608336041], [0.50420855857139912, 3.334194508005762]], [[1.9489475274475296, -3.7459463416144589], [-3.7159671159362917, 4.9066677888589574]], [[2.0975225282464782, -2.9006015474319655], [2.6825665718802414, -3.4175703143175342]], [[-0.77860640954835159, 4.8421940937574348], [-0.91639012765303463, 4.4224988194440495]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[1.1597312355222431, -0.68549816716584255], [0.73426969027059197, -1.0249406458120494]], [[-0.77644911324137522, -2.82345500016981], [-1.289457775420499, -2.4560956982794249]], [[1.3159560628237312, 1.2424506921077565], [2.1935980505343053, -0.83137254480133327]], [[0.91135314106963705, 99.859418806833347], [-17.998893223037665, 0.76268655940724139]], [[1.6519486290895466, 1.3127448541677011], [-1.215679048365693, -0.90002650029039344]], [[-10.524042317422042, 0.78916864203758663], [-2.6004381310463018, 0.63476443681456807]]], [[[0.29220823270152124, 0.19316067665703385], [-0.12890991406511837, -0.70106009532621061]], [[-0.14482756866523616, -0.16451699260188507], [-0.12150261189914832, -0.25569790702120249]], [[1.5666381405220651, -3.331960262825485], [1.1551405244711517, 0.17468439150521295]], [[0.2988442380251255, -0.15548320388913744], [-0.15673759229278253, 0.11870209352943652]], [[0.2776760349162592, -0.2007968792910087], [0.21711734757910459, -0.17042275219648761]], [[-0.74804385328507916, 0.12028260898130487], [-0.63557181730304591, 0.13169743228201169]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[4.9939781740323301, -2.6764387943077717, -0.41371381825511655, 3.1708723067060678, 2.3775406228954736], [2.5958848979539892, -3.47223479970455, -1.6169866586837545, 4.5873948053265288, 1.9161981794366003], [-2.2022059996060772, -4.5924435791244029, 1.2339333431831809, 0.86946804541409772, -0.4918066312691689], [2.3604379540785221, 4.7595160409211932, 1.3276484964898376, -0.56363044548816266, 0.96680933374223699]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[4.1757363546230639, 4.152344808235009, 2.1005833778314269, 2.6833101045466652, 0.19143338541901844], [-3.8468780551287218, -1.8778992222808788, -2.661978053267843, -3.716207594962925, -3.4509008693059839], [3.8978117931654293, -4.18752650384641, 4.246795747284235, -2.0851048994322721, 0.0088540756478447946], [-0.94821844064560512, 1.2312690624189271, -4.8064676342818835, 3.0654545469280912, -1.5598124134123359]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[1.1959515040990003, -0.64456082476575827, -0.19695186709618787, 1.1817017724985515, 12.419676002132023], [-0.67480301188469238, 1.8489995408205164, 0.60743801275849829, -1.2344291022773972, -0.55527476795413422], [-0.56498520617837644, 1.0966960029759956, 0.29055631977880347, -0.4169900735693608, -55.545790529684652], [-2.4893398534533788, 3.8655369376135718, -0.27622124968042078, -0.18386521048011617, -0.61982410540456523]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_Symbol_rank2(self):
      arg0=Data(numpy.array([[[-4.8928381034401287, -3.2019357496653287], [-4.9740113593374238, -0.042846811826386499], [-3.0920109554302866, 4.0344972358316706], [-2.042198743432456, -2.1261063168519625], [0.53995401880996674, 1.008401726547687]], [[-3.5849856046141451, -2.2559104789144091], [-1.1081880416585488, 3.1018865460927287], [-4.8097289000740195, 3.1713484420669982], [-2.216299061293836, -1.7607375060495989], [-1.0203766568590011, -1.4568882502758687]], [[4.2318305795257789, 2.7188593354501345], [-1.363225732266522, 0.018857329022692326], [3.5218189153252286, 1.8494431756983154], [-2.3277283815189262, -4.9032493462460858], [-2.409903699412197, -0.74178282825791886]], [[0.17044886664805681, -1.6954984250694913], [4.7862718060867788, -1.9497547242858837], [-0.11533337229445362, 3.0212197507854412], [3.1933995125839054, 2.8773721224478255], [-2.7271881932421707, -0.15772075811969266]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[3.1364823947051921, 2.9742683816861595, 4.4670992021743441, -0.27205980841102217, 3.9976979684571852], [3.9172531850265209, 2.4871174198314003, -1.5228533635237893, -2.9547647845409184, 1.6627284519317982], [-2.1619482974541846, 2.6431105875289198, 4.2655031018577265, 1.4740675577249824, 3.7281729008639921], [-0.38238211623706331, -4.5358261805131237, 0.47540229792893829, -1.1577101466507989, -3.8216656723477991]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-1.5599762688609071, -1.0208683954581192], [-1.6723478587085603, -0.014405832402419572], [-0.69217423108160703, 0.90315819130855524], [7.5064330720513688, 7.8148489821763247], [0.13506623638662452, 0.25224560096941373]], [[-0.91517842612715206, -0.57589090425339351], [-0.44557125965273964, 1.2471813841032897], [3.1583664030163758, -2.0825041451979938], [0.75007630823587956, 0.59589768879798144], [-0.61367606699308164, -0.87620335634674473]], [[-1.9574152557251239, -1.257596834601338], [-0.51576568104969844, 0.0071345213899363553], [0.82565147210686451, 0.43358148652918327], [-1.5791191993340183, -3.3263396379291916], [-0.64640341622935715, -0.19896685266018996]], [[-0.44575533062426115, 4.4340421611620107], [-1.0552149962557258, 0.42985657886592871], [-0.24260162981310054, 6.355080242454032], [-2.758375679631262, -2.4853994160558477], [0.71361244730933049, 0.041270161139657882]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_Symbol_rank3(self):
      arg0=Data(numpy.array([[-1.5826582986437354, -1.5367818601218799, 3.7751616740093077, 4.0272661637437288, 2.0484562263796953], [-3.246279600564248, -0.085403284324286233, -1.5701230599717189, -1.9389097614971753, -4.3340602173897214], [-4.853529667311939, -1.582922793489586, -3.3621797464984962, 4.0573604376591597, 0.3696820689908753], [-4.4345687679051862, 0.8548573381880562, 1.1193239801336849, 4.9478534882872314, -2.6158615935767928]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-0.42486138598841183, -4.7089830403057196], [-0.23214717671575702, -0.5161741439599945], [4.6370494023802049, -2.8007347421712581], [3.4261385815386163, -0.50611193668609822], [-1.5940743577108005, 1.2898524931901445]], [[-4.076705060611622, 4.9499450441122228], [0.7239749180959425, 3.5601054887993442], [-2.8219162855703237, 3.8016808926127883], [-1.5107836212145118, 4.6657149579140516], [2.110394834241748, 0.14108355414036922]], [[1.1458225454081443, -1.0897696414681546], [-0.87301040259323148, -2.9166293071434102], [2.6523743439768657, 1.8513245700169234], [-0.91709698304503728, 4.1163954053432796], [-4.8246471597471974, 1.7070429550711133]], [[-3.2831119976346921, -2.6864548580600456], [-2.3361452817825357, -3.5334764775542049], [-3.0818568081833622, 3.6250108588289507], [4.151605648355682, 4.6594853006916512], [-1.4429452404632093, -4.1372334139549611]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[3.7251168283080984, 0.33609343781815471], [6.6198602191209446, 2.9772546302532086], [0.81413013889209618, -1.3479183219909747], [1.1754533764174704, -7.9572637431026019], [-1.2850443371546469, 1.5881321602234719]], [[0.79629984320651681, -0.65582134177945628], [-0.11796442416665108, -0.023988975774166945], [0.5564031321554207, -0.41300758909636348], [1.283380183813811, -0.41556541258663243], [-2.0536726810871495, -30.719811701636079]], [[-4.2358475898055419, 4.4537207521886817], [1.8131774704947352, 0.5427233380713451], [-1.2676113212049005, -1.8160941635791943], [-4.4241345382987793, 0.98565857701437287], [-0.076623648683616055, 0.21656283920252892]], [[1.3507211362573246, 1.6507140459108611], [-0.36592644509496397, -0.24193095485943911], [-0.36319791924190148, 0.30877810404553641], [1.1917927441511498, 1.061888420927688], [1.8128626923756705, 0.63227314774009258]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[-4.0383079757037388, -1.6019211705616829, -1.0143953450770593, 3.8419882837867121, 0.11860869977450683], [-4.394681747741096, 3.1344656289872557, 1.2640635866165635, 0.34925455182910259, -2.7024036775344653], [4.7316414588326641, -3.9834738000256307, 4.9315428502448562, 4.6880170190928041, 4.1752864163143304], [2.8070522209545494, -1.0290337810955466, 1.304816938264767, -3.8126911088989148, 0.4389492265642243]], [[-0.97615970327453105, -1.0032635029597214, -4.7299352700667541, -1.601657173491823, 0.12049089755466902], [-0.79396441722465827, -3.8133638430409911, 1.0932110219815092, 1.975720136505581, -3.1705327729470509], [3.9593639814743149, 3.4644966663143517, -2.0811105542943298, -4.3215185266672869, 2.6045315990405671], [-0.57666267941667648, -4.5201843241409172, -4.1355126952252306, -1.7566465673697884, 4.5973199279997363]], [[-0.54105580964988498, 4.1206902134048047, -1.0113591764204499, -3.4464788047657491, 0.089430598830712249], [-2.5231794220050165, 3.0820643006796065, 2.9422655572191516, -2.23217848900671, -2.4948436942029364], [4.5185850363065914, -3.9593312484625143, -2.2577235345670821, -1.4469553213470778, -4.5835135215797598], [0.91570447749293571, 0.59838863824383282, -4.1834855228064827, -0.66387549816902069, -4.160606426894252]], [[3.2811627759080864, -0.0049104620728357418, 1.7094412663718197, 4.552745815132802, -3.8960092019237735], [-3.8204385649087014, 0.29165378829892852, -0.164222027740478, -0.3960768187553203, 0.47425575340027226], [1.3132611080245562, -2.5283911122239875, -3.4875105372571125, 0.68501217357805988, -3.8149793234376892], [3.2795390984388053, -3.3407699102834787, -4.7937104092778302, -0.28346995639977113, -0.85848920667180906]], [[-0.77563726067947059, 4.3912571241055698, -3.2407238591483867, 1.0669117587199608, -3.8625720819401699], [2.8752994125414046, 4.6605663364123409, 4.9225395632696038, -3.8444891269611259, -0.90573858425274345], [1.9017442597329026, 0.19095388206546904, 0.41050261458940618, 0.34698235282922418, 1.5222941002746779], [4.9881999052112587, -0.3624817793480144, 3.5816499250781906, 4.5667604987299875, -1.0421480776207437]]], [[[3.9426809541533387, -4.5023853806114467, 0.86882751069473851, -3.9294206170452206, -1.1281134542836426], [1.1060818507011829, 4.6448739260146237, 2.5318820084548381, -3.6401560894624039, -4.4832309550086613], [-2.2994307148557303, -4.1363078569106246, -4.8091937033975345, -2.2421989472528479, -4.6420422452351193], [4.7356572783270838, 0.53189265839531075, -4.3590608367602872, -3.8593148307575831, -3.9084155586666167]], [[-3.5524286083948811, -4.491126961395322, -4.7769766330387533, 3.8849009217169677, 2.8642595951378169], [0.85625558307381056, 1.4616127152782079, 0.34664627769893208, 4.8967753371440814, -2.5353749432960981], [1.5511597954260914, -4.3887397075608252, 1.066038662002228, 3.5996291051345644, 2.2746611106187595], [1.9131747472969298, -0.71474579194608978, -3.4599366467564363, 2.5772460423380625, 4.9410943687377049]], [[-2.4698602153794056, -1.4009260557908085, -0.20311049863814912, -4.8825802521144794, -4.2709024315866078], [4.3125287000543828, -0.8938014160346448, 2.9795667611691119, 4.3058159071740452, -2.1138483952128659], [-2.2059322903695744, 0.32509724208605029, -4.2730802150219027, 0.14949214969599467, -2.3302949085679647], [-0.24037653116739044, -1.1662356681858066, 2.1121612060488895, 3.1203070419064947, -3.5144698646360482]], [[-1.1369851663334964, -0.12735904680537669, -3.9508638086833048, 3.0955159529638649, 0.58561603458316203], [-4.8769217178423823, 2.0999945403111973, 0.7533048353382803, -2.8579085971949167, 1.3073183278981491], [-3.5983167697145291, 1.1745035044468022, -1.1047225139335684, -2.6359230353249083, -2.4273106230038488], [-0.21461422797709329, -4.3537068853826044, -3.232567220248507, -0.14387669674300163, -0.83253628355006803]], [[2.5198535610876647, -1.8895703422602859, -2.5696952784416336, -3.5984127453170633, -4.125517158834815], [-2.0271550478235834, 3.9458314801851646, -1.2333222658934195, -3.586302986368417, -2.5628258942649818], [-1.3934526627569808, 0.71129739245951384, 3.1413394025356318, -0.10596442567294329, -2.2010872695446904], [1.4879458286820118, -0.096333329096161435, 1.3207897701689566, -1.4212381636787552, -2.4685680016528821]]], [[[1.0765063184369179, -4.8662007562224066, -0.95364537254958925, -3.4084410116349186, -3.6014915448166307], [4.4966973832014876, -2.9640166954272784, -0.45975600268453753, 1.6997297905400242, 4.3158812515078289], [-1.8060909240010647, 4.9528705714672636, -3.0271786212619691, 1.5519285084708123, -0.74621807588375511], [4.0236888329012217, 3.9268080739940956, -0.24614568270343895, -2.0114235605443511, 4.0988293388425614]], [[0.34820843946798519, -3.1925897610908462, -3.8521344455391016, -0.81850666771573977, 0.39268424399074053], [1.365275107489202, -3.2454552898248101, 0.14440501415798135, -4.4260839568206602, 3.6270474389755964], [3.3674698877329678, -0.15090224876420955, 2.509788343521083, 3.0486542025903756, -2.9028929280777946], [2.0286510557085098, 3.1777408271595213, 2.4085323292701499, 0.53065775691904538, -0.8410615965432644]], [[-2.098136638293334, -4.7135583443147899, -4.1406907220600377, -3.5376860760745155, 1.3206668181607073], [-0.41461009846364227, -3.1658625793219777, -3.7596548362648341, -1.5428774020282332, -0.61811893257569306], [2.110420787541444, -2.2281854529351683, 4.7442582504948678, 3.7712505406557018, 2.6309881569917106], [3.5403718889681759, -4.2168647800971826, -1.3356640096729153, 2.2582119139496797, 4.0699170551214472]], [[-0.26301936532916592, 0.083815277873359406, -4.8226545770839877, 2.5094836577069959, -4.5467586695957438], [-1.4428562683499244, -2.0534461635769565, 1.6542783149740012, 1.6409731362911559, 4.9459619173134488], [-1.5349840103541057, 3.7777141995256773, 0.26755498199593397, 2.5174264242529887, 2.4468792154115082], [2.9334348091150755, -2.8385401612087247, 1.8709150333066837, 1.7923820051071973, 3.2116774469443126]], [[4.7877582274830051, -0.90259230893714548, -3.768051337452436, -1.2029245338961925, 3.2200698853221095], [3.3887691440730148, -0.88351475590030848, -3.8242634833560221, 1.4734119317761731, -2.9944864007465108], [-4.4460960190845045, 3.1377934892184349, -0.79965269095188773, 2.3206399013112815, -1.7780849466592064], [0.55503247964900737, -3.0197882357594774, 2.6888678801938504, 3.8180383518705518, -3.2997915346120954]]], [[[4.310301046349629, -4.8380169965651865, -2.8362404428646935, -4.3400620746005503, -0.45757736344791855], [-0.86583219430568192, 1.4657138544430213, -0.084702788342362112, 2.3089954059180693, 4.4355384434062959], [-2.6472660086452429, -0.21695060490625284, -3.3636204181248939, 3.6243677502759581, 1.2689053638787362], [-4.2840533084650358, 1.1535996715530308, -0.89722889012632656, -1.490630501211323, -0.5339908778875202]], [[2.6331691584905546, 2.1277387175481275, 2.1164646800140305, -4.0331381540365454, -4.5519998469453942], [-3.2660246457654765, -0.55452078176470998, -4.1601352914260854, 1.103649984592538, -0.94443736498901742], [4.695946795191773, 2.238279354637303, 3.0867359128395329, 1.8144787745755186, 3.0019543806296021], [-1.2368777024191955, 3.8369533695976958, -0.019185378412238663, 3.9636934737803546, 0.65489896459587271]], [[-1.609663824666451, 2.9322426951905145, 1.2869253947562083, -3.6890795217199255, -3.6439015462890092], [0.52355407731110581, -2.8836543107092183, 0.28708846395486454, 4.3454225494395775, 1.3975947290082171], [1.3749923281064564, -3.3506113144258807, -1.6098088009647258, 4.9803688774887611, 0.54729800424729369], [-1.3889098790196819, -0.13641616119626931, -3.7435412826555536, -3.136520421765189, 2.5211162046006326]], [[-1.5862531918940261, -2.3369294103334393, -1.9320890362470902, -2.9710965582187421, 2.1231351124987459], [-2.6428934723496358, 1.9524765295056836, 1.685809629813928, -0.28696243702760338, -0.71182630045411521], [-4.7251482672145881, 2.2652072087389641, 2.1643133354296111, 2.8715339016433363, 3.4929456016799758], [3.669611376210737, 4.3113119452470716, 0.74775396290638696, -4.1251454431281935, 3.9205044918678738]], [[-1.0792471744043719, 4.0517079326137857, -0.6034895871370054, 1.3773953674560468, -2.9170824072701587], [2.7096125858598601, 2.2357566875862265, -4.7515038250660968, 4.4570835112963874, -0.32060570417311052], [-3.1130994403203571, 1.7094197102786621, 4.119767776099577, -3.4822306372960576, 1.2131396683403537], [2.7884803439864623, -1.8235088528498999, -0.64184669644069015, -2.3694637141717112, 3.225668824593825]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-0.86314636701260472, -4.1385370451032655, 2.6041251772277736, -1.3614458864694221, -3.5996368085130559], [-4.1706591371428043, 2.1059439574530137, -4.1367824586465449, 2.3375198669708137, -1.4518924294344462], [4.1975028102787633, 3.4313143895599136, -2.91694183741531, -0.56883656012398109, 1.5093514925474061], [-4.089796245654699, -4.590402309787267, 1.5532648585058952, 1.6227639402092811, 1.8715221181057293]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[4.6785900167552539, 1.8559090691721463, 1.17522981483191, -4.4511434336264859, -0.13741435324001633], [5.0914675838251187, -3.6314416057102887, -1.4644834699259113, -0.40462957984506276, 3.1308753425999201], [-5.4818529506289018, 4.6150617696656067, -5.713449119079522, -5.431311766181997, -4.8372866710488713], [-3.2521161279632165, 1.1921892050094436, -1.5116983493550549, 4.4172011313618116, -0.50854552986586832]], [[0.23587071775267238, 0.24241984354031265, 1.1429003095824002, 0.38701047158365071, -0.029114369701543293], [0.19184663773013227, 0.92142798324180264, -0.26415397761752579, -0.47739578381768055, 0.76609988950043073], [-0.95670618344689007, -0.83713076107740991, 0.50286140527767142, 1.0442140494502825, -0.62933630185146217], [0.13933974086301493, 1.0922179202163282, 0.99926922247038641, 0.42446075708039394, -1.1108562948444076]], [[-0.20776874106561458, 1.5823702521825365, -0.38836811120465964, -1.3234689464638965, 0.034341897084193072], [-0.96891633477123096, 1.1835315474199377, 1.1298479746474193, -0.85717019616659895, -0.95803524193826461], [1.7351642984830935, -1.5204074224563306, -0.8669796499454554, -0.55563969581809303, -1.7600972340581369], [0.35163612160447322, 0.22978489800588181, -1.606484035172234, -0.25493225286341675, -1.5976983223684482]], [[-2.4100574312336289, 0.0036067993018582822, -1.2556072065448292, -3.3440519820727053, 2.861670258541912], [2.8061626267174504, -0.21422356275596197, 0.12062325015821802, 0.29092365895088856, -0.34834711986250066], [-0.96460764329765514, 1.8571366936814164, 2.5616225895699172, -0.50315049638474574, 2.8021527416935443], [-2.4088648186697235, 2.4538396593543288, 3.5210436616832044, 0.20821243004735307, 0.63057167031301664]], [[0.21547653331166822, -1.2199167187423883, 0.90029189930610543, -0.29639427961085957, 1.0730449452026045], [-0.7987748668814002, -1.2947323811641807, -1.3675100642453466, 1.0680213953443858, 0.2516194361916439], [-0.52831559429421393, -0.053048096856290507, -0.11404000915275041, -0.096393711723532532, -0.42290213742522259], [-1.3857508883713723, 0.10069954237904045, -0.99500313937441498, -1.2686725749469243, 0.28951478525724822]]], [[[-0.94533761319424281, 1.079538085602435, -0.20831899278394317, 0.94215817879980257, 0.27048804929584341], [-0.265205526112337, -1.1137026003032917, -0.60706999186448884, 0.87280114959386668, 1.0749454241134631], [0.55133508619239557, 0.99176358481894233, 1.1531015950376062, 0.5376126107464424, 1.1130236474839912], [-1.1354697477318521, -0.12753203771998847, 1.0451731233414945, 0.92534889662584263, 0.93712179062040468]], [[-1.6868580931713328, -2.1325956683230136, -2.2683303684947815, 1.8447313889660546, 1.3600834841787199], [0.40658991899736474, 0.69404160072992749, 0.16460375238008498, 2.3252163571657318, -1.203913776681147], [0.73656271333170065, -2.0839774449025166, 0.50620466809170195, 1.7092710812153113, 1.0801147402658318], [0.9084642259952519, -0.33939449785288822, -1.6429386140650097, 1.2237961191783349, 2.3462610917308551]], [[0.5970486096548292, 0.33865113038821909, 0.049098665609939265, 1.1802845087754363, 1.0324213260621744], [-1.0424838006747248, 0.21606198173811514, -0.7202618921720999, -1.0408610919760097, 0.51098853187084581], [0.5332483185715553, -0.078586980421594191, 1.0329477698520195, -0.036137300230407787, 0.56331096253255264], [0.05810712397142484, 0.28191853931022776, -0.51058068128144629, -0.7542835701656363, 0.84956603344955628]], [[-0.48640663225973463, -0.054484690635130714, -1.6901947506453578, 1.3242736443456784, 0.25052879458178057], [-2.0863658900843367, 0.89838575063431447, 0.3222667092513255, -1.2226243026111574, 0.55927581466603737], [-1.5393737698484586, 0.50245712177361657, -0.47260454533170476, -1.1276580244602561, -1.0384128311813643], [-0.091812793127277817, -1.8625325700545008, -1.3829047042229348, -0.061551004881704424, -0.3561622278868371]], [[-1.7355649151426598, 1.3014534024372093, 1.7698937099924157, 2.4784293053437496, 2.8414757699658386], [1.3962157297102364, -2.7177161339164635, 0.84945843155462553, 2.4700886330576055, 1.765162378636602], [0.95974924485264423, -0.48991053196453721, -2.1636171791041527, 0.07298366154730862, 1.5160126362819297], [-1.0248320044354866, 0.066350183486862058, -0.90970222269389622, 0.97888668255703237, 1.7002416650209136]]], [[[0.25646351344918461, -1.1593085165556409, -0.22719350424598192, -0.81201637394700354, -0.85800813188197711], [1.0712791834683379, -0.70613810863188753, -0.10953083856399, 0.40493833294828507, 1.0282021112503326], [-0.430277478213557, 1.1799564634808035, -0.72118560917912267, 0.36972661570838738, -0.17777667094264493], [0.95859109922406505, 0.93551052887402586, -0.058640981037745156, -0.47919528621131979, 0.97649233939889879]], [[0.10147960808471559, -0.93042764335573303, -1.1226410664261985, -0.23854027197452987, 0.11444134795270235], [0.39788691809855015, -0.9458344300071726, 0.042084460286514913, -1.2899091876534028, 1.0570431698159803], [0.98139357267256, -0.043977972179798902, 0.73143642889656002, 0.88847999817975976, -0.84600027817623202], [0.5912168998214985, 0.92610016640506243, 0.70192703315042448, 0.15465145325465365, -0.2451135340737855]], [[0.71929327194006865, 1.615924693408167, 1.4195314657796148, 1.2128065190388726, -0.45275733688641001], [0.14213862379615583, 1.0853362033873242, 1.2889029147033828, 0.5289366357045262, 0.211906499007675], [-0.7235045829407003, 0.76387723071967528, -1.6264493825830739, -1.292878209733997, -0.90196798689788693], [-1.213727282305116, 1.445645821938544, 0.45789874605674058, -0.77417104619085308, -1.3952684976152228]], [[0.46238125986810585, -0.14734509655126843, 8.4781023498786077, -4.4116075400639509, 7.993084461035262], [2.5365041024006016, 3.6099053885168644, -2.9081786068979851, -2.8847884459703095, -8.6948734733847779], [2.6984622964802885, -6.6411241195578281, -0.47035475697556933, -4.4255707187743027, -4.3015505453415255], [-5.156902728748161, 4.9900803854626519, -3.2890203697506846, -3.1509613318745506, -5.6460461089988829]], [[3.1720631351432078, -0.5980000771151035, -2.4964704086871854, -0.79698104771205946, 2.1334128605706288], [2.2451822261451002, -0.58536050765031378, -2.5337129901409687, 0.9761887400325977, -1.9839556362663877], [-2.9456995544361981, 2.0789017698737808, -0.5297988539450641, 1.537507938190477, -1.1780456410840696], [0.36772910908396295, -2.0007190178497347, 1.7814723034829463, 2.529588615191722, -2.1862313390255284]]], [[[-1.0539158401666613, 1.1829481729573821, 0.69349186915561489, 1.0611927377095502, 0.11188267971395457], [0.21170545970978527, -0.35838309940264229, 0.020710759963251616, -0.56457467981964049, -1.0845377561581311], [0.64728554911699898, 0.05304680034775746, 0.82244205238797707, -0.88619763249251193, -0.31026126673838944], [1.0474979806186506, -0.2820677613899959, 0.21938229589789679, 0.36447549258598855, 0.1305666213701652]], [[-0.57362492016796307, -0.46351900638676991, -0.46106300432567399, 0.87860232760806822, 0.9916341836182867], [0.71148984889667188, 0.12080003980967152, 0.90626812437685411, -0.2404255466322481, 0.2057417414101958], [-1.0229924259970575, -0.48759982319306377, -0.67243254611001213, -0.39527663418669007, -0.6539632428794071], [0.26944864936609797, -0.83586429046030863, 0.0041794546790230617, -0.86347409361687166, -0.14266700833596929]], [[-1.0363099479471947, 1.8877931082604129, 0.82852926705244445, -2.3750485961991679, -2.3459627804842782], [0.33706683985287544, -1.8565116534491364, 0.18482904726951621, 2.7976056534360105, 0.89977876043148297], [0.88522721709488661, -2.1571410027578137, -1.036403284442565, 3.2063874040641336, 0.3523533035916015], [-0.8941874088078523, -0.087825434567218447, -2.4101113613402112, -2.0193081718093118, 1.6231077338772253]], [[-0.97750088758408926, -1.4400920259739414, -1.1906162001590428, -1.8308864799125202, 1.3083450154955587], [-1.6286370474862739, 1.2031796376088324, 1.0388508075897449, -0.1768356012339016, -0.43865055342695991], [-2.911790279617136, 1.3958944690666653, 1.333720377808302, 1.7695327277687762, 2.152466859246025], [2.2613340642370217, 2.6567708576831328, 0.46079034934061469, -2.5420489948748743, 2.4159425747174681]], [[-0.57666813764228309, 2.1649265554578339, -0.32245923320843811, 0.73597600270425045, -1.5586684116897846], [1.4478122164019136, 1.1946194308668705, -2.5388446009258794, 2.3815286328582892, -0.17130746202327185], [-1.663405102297854, 0.91338472238247448, 2.2012925929346863, -1.8606409208888299, 0.64821016893363737], [1.4899531867722926, -0.97434533912726273, -0.34295437399924428, -1.2660623624207958, 1.7235536750475073]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank2_Symbol_rank4(self):
      arg0=Data(numpy.array([[0.17987742736663925, 0.89240441275846738, 1.0829604207606591, -3.8706420684545284, -4.0620979021361503], [-0.33950301510526781, -4.0034813203615762, -0.5688520588658168, -1.8417860767233285, 4.9426020678624578], [4.2273002047353891, -1.5005403719891754, -0.19445252935733937, -2.5720855376705085, 1.5225418665585178], [-1.7110779737184778, -2.0508226145952602, -1.0425701463208883, 0.10011215761504655, -2.2764209858246764]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-3.7463429332972864, -2.2191347553409582, 3.9881461164443603, -2.5638561008346494, -2.9562140397109293], [3.483033013178229, -3.8753975342278779, -1.8624623706188084, -4.159491941721277, -4.3466606395626197], [-0.4346124506174025, 0.075320178696856921, 0.82700688483759244, 3.4519181076608163, -4.2860068471871031], [-4.1281699500836586, 2.6008740308563647, 1.4778643094139294, 0.48757079946011661, -0.99272545834909742]], [[1.6933956433432069, 4.8293894689350267, -4.5681331358627419, 3.2234601106532228, 3.3615002669948186], [-4.2882250399894346, 0.099222180184115771, 1.0729386152160183, 4.3170167615925834, -1.8488924857254139], [-1.5242579772546803, -4.9613160447067672, 0.13638655268958821, 4.3493924536682567, -0.93611059730522328], [-3.2419771651328375, 0.85596195930791907, 2.0438246533285289, 4.0851831910713319, -1.2679387972512499]], [[4.5730957494935041, -1.0044826463259349, -3.407498436331057, 1.8083174385924607, 4.8186512156982051], [0.94981210224187507, 1.8152343309614896, -4.8881575787685723, 2.5887790407082365, -1.0193310679957532], [-2.5596887196775828, 2.6454143355957811, -1.8653831926301945, 3.6512536438544476, -2.1204904084894336], [0.46706959544719684, 1.5866260757227231, 3.4001377729105524, 3.4230198224508008, -1.0511269677686199]], [[-2.4820625121362507, -3.8181638195063505, 4.783208127376211, -4.4377482394250976, -2.4119724739081096], [2.3310539610172176, -3.5865208293101993, -4.1369692667944369, -1.643469626784726, 2.2236816744482528], [-4.0380406378193605, 3.4795584723730411, 0.55580975119783549, -0.37860138523889386, 1.4564226379235041], [-3.3857704553262202, 3.772443084653025, -2.1319737291185525, 2.8230971485887792, 1.4580619123654959]], [[-0.26361563477097061, 2.1698693211713307, 1.7224101351731891, -4.9785623498587031, 1.5320601130055325], [-0.28108433307170344, -0.2077037993015054, -3.6166086889906279, -3.713238968519581, -0.3755176766964361], [-0.21007633635043188, 1.958177504362939, -1.23171396539276, 1.3485150135145094, 2.0286230818462681], [3.5648272356025359, 2.2595925393563618, 1.4584522365393804, 1.4566667637579522, -0.82329381716062855]]], [[[1.8508065981541328, -4.4730590956665486, 3.5820245072964365, 1.0861838688289485, 4.3462130860440791], [3.6790137949226338, -1.4854850025135824, -4.2190140974253199, 3.0219428598983118, 3.2750825884415242], [2.4227029058859593, -1.516433800853604, -3.8583058346609613, -1.2491332088441554, -1.8600436672033061], [2.3722441304354707, -0.16131478313807346, 4.2577298020271641, -0.40791824179336356, 2.4086994030221334]], [[0.20095474883931796, -1.6672750717479676, -0.47986050296914495, -3.3926319210370917, 2.0779474357980483], [-3.6226055848187344, -1.5615438619778099, -1.2414521863983685, 3.6963127326896181, -4.7849253332700563], [2.2492774474173185, 1.5724812882053865, 1.6869732750062703, 3.44027972780545, 2.2949953172383903], [2.1716444304214626, 4.4804178570001643, -0.98351270863755058, 1.854806255059728, -4.6152264135963081]], [[4.2320942004557178, 2.0719656410836951, -3.9242762472348005, 2.9189125418160957, -4.7556565626446599], [-2.0739686714109498, 1.0129913851206123, 0.94732746873247642, -3.3288308592003144, 4.2342908455390518], [-2.8348824511780526, 0.74671386726505951, 3.5750051603864428, 4.1958055963839946, 4.518810337908759], [-0.80941297645629007, -0.180354117992783, -2.9059888998727224, 1.4589179980560294, 1.1013321416220645]], [[3.0810376199953406, 3.0602539032002412, 2.6855981147014418, 2.3755825402883355, -4.7857866856816305], [1.6461034657454547, -0.098052902333573222, 4.0879983071342867, -3.8677565983901041, 0.124810573878122], [-1.4743338859340915, 3.6868174147864323, -2.5886849467077244, -1.9234450967963825, 3.1371564500343894], [4.4818581509437045, 2.6596083861241038, -4.2047428915715077, -3.0679667044922376, -4.7512407747079619]], [[-2.4850685752316179, -4.6561550826382723, -2.6526799330993436, 2.5675465759570093, -4.6261642317923757], [4.6307107534326324, -4.1884669926839946, 3.7176064322192222, -2.5208065114423874, 2.6386487253795945], [0.54162550100767248, 1.7890376218907864, 1.8544690403362729, -2.6587689177963245, -4.3941457333867282], [-4.032350289345997, -3.0469186458805253, 4.9242868047560844, 3.9251686231146863, -0.39427453713569349]]], [[[-3.2337567292182512, 0.51680810173164637, 0.25422935550041004, 0.28234125371049412, -2.3647189614792321], [-4.5114687490104579, -1.5781636789663445, 1.4195990733587349, 1.5899784304566182, 1.6040729820779456], [0.43651335496638577, 4.173284641719432, 0.29762951361347945, -3.9214942483913786, 4.1107475703388818], [-3.782212508689148, 3.9631349118506289, 2.8551226609695579, 2.3393793806519021, -3.9337460711810781]], [[2.4142081620130664, -1.6866846409621115, -2.1496729686014393, -2.0259644425451304, 1.1004721644613085], [4.8562613299199047, -0.36965446310047856, 2.3690589055240698, 0.35522637729410977, -0.12571329619529781], [-0.43902229404694282, -2.6604522185627824, -2.1029697292910319, -4.2025977890633008, -4.6022482690130042], [-1.8101576519109139, -1.5980211410877598, -3.334173175548182, 3.9944486138843143, -0.5069166168209307]], [[4.6801594848719823, -3.0143031402268274, 2.4967221721149855, 1.1151978487325938, -1.93072744449195], [4.3334462165569541, -0.92465955737011107, 1.4370416125838918, -1.0903744082257472, -2.6987455630616406], [0.13503151683033288, 0.64826066676549843, -3.3241434097645426, -3.5401727610860245, 4.1358012329456244], [4.6487045650318972, -2.2196520955155652, 0.065685239199958723, -4.1429652666469536, -4.9605379967749119]], [[3.1178062819735111, -0.21650526470744413, -0.83129524158522816, 4.6980447444622317, 3.4748879816432758], [-2.6755701052611069, -2.1415084148135985, -2.9606854095977795, 0.69839484050206124, 0.51635654690141131], [-4.9363615743569325, -4.0902218565182435, 1.8471127772197384, -0.99247425351824248, -3.551831683574842], [0.86132705551149069, 3.3755054165042448, 3.6442191411470723, 4.145208693868355, 4.8955506015912427]], [[3.150479305593981, 4.9278251177379602, 0.82335290607348455, 4.9527557347459918, 2.5414877512091385], [-0.58466083616054121, 0.091004027421245581, 2.4648321340546921, 0.12596797917185842, -1.1688276424789801], [3.6987047972248757, 1.7685157407882146, 2.9754583162401627, -4.4062193695383369, -2.7131449521689976], [-2.5212371210884434, 2.1753883826481335, 3.713205904603825, -2.6071288342884213, 2.5848604060296623]]], [[[2.0771734006643729, -1.2400287019227383, 3.809244595923662, -1.6582432402657554, -2.2819398967398365], [1.888025928467111, 2.5295694674429363, 0.19931413439019963, 1.6649633443855834, -3.6005850997578968], [-2.9475284685617753, 4.4113854972199409, -2.5542515650029651, 4.5091658492340851, 2.8932021419611411], [-4.8447884915957529, -2.7747360621097448, 4.9854050235927758, -2.8245997935933631, 2.0568585181777044]], [[-3.2512789267432707, 0.34019906173393721, 4.3773454385538066, 4.9941392355932788, 2.8675731243293177], [-1.8415852601929625, 1.8026961764956875, 0.068240501116980212, -3.7109842564711562, 1.0952989344081034], [-2.5970521435564753, 2.3333218360759398, -2.0249779680711812, -0.112969966673786, 1.3561640383822731], [3.2432894996895598, -3.3039076543413115, -4.2676125439644785, 3.9861556870529675, -2.8582649157792215]], [[0.90813224768933143, -3.1758095793985186, 0.16061255384985618, 1.9915449206986002, 1.3318084422193799], [-1.9391510555929425, -3.6141327458436656, -1.4957195345031851, 0.12096846851741372, -4.8825814868920236], [-1.222842750975397, 0.5548134280776349, 3.3474707344117931, 2.2930040044848523, 3.9692891242782409], [0.92321474870345011, 3.8568048334334026, 1.0783864102212481, -4.4358213433283344, 0.34875519561882129]], [[3.7690946932983742, 2.1782268275604375, 0.26934846874161344, 0.22882311689777612, 0.29121123206440913], [4.0967377557503522, 0.018325368471693082, -3.332016424536115, 3.9427862849698236, -3.9313458599603388], [0.86814425821552721, -4.6281822691030605, 2.8562377409600117, 4.8209000553333183, -4.5929855641114177], [0.053329924335240086, 3.0654041908282821, 0.065128123332158339, -4.296560842534153, 1.2287593239726355]], [[-2.1795323250346712, -3.1358363149811899, 0.92112300025088967, -4.4818522208732148, -0.15080897678705796], [-1.2198988295362168, -3.2664323735385414, 3.0735686430941023, 0.75061947149649377, -3.3539298605175238], [1.7334649733561882, -1.5059130893576702, -0.94598720815355719, -3.5879413112828229, -4.1686735252957954], [-1.3569538661770784, 2.9812241256802574, 2.4794539159264151, -2.8221986214805117, -3.8455627685062179]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.048014138206061902, -0.081057460315879765, 0.045103018323463369, -0.070158940397661601, -0.060847227213706216], [0.051643905379611402, -0.046415219542755237, -0.096580435773783974, -0.043245047685367688, -0.04138290110100229], [-0.41388006052543747, 2.3881704807233213, 0.21750414738320295, 0.05210941330486335, -0.041968534764402536], [-0.043573164269313569, 0.069160376562879031, 0.12171444037238609, 0.36892575922474469, -0.18119554188302517]], [[0.52699108815269369, 0.18478617607853851, -0.19535429161477955, 0.27684673677492005, 0.26547801335035354], [-0.20810577906626518, 8.994001251560185, 0.8317385543802025, 0.20671784754184089, -0.48266971695129807], [-0.58546809403337652, -0.17987251864564735, 6.5431994222300904, 0.20517909622200631, -0.95331087515452484], [-0.27526548377829235, 1.0425748516674895, 0.43663452796946978, 0.21844905626482711, -0.70382294058128103]], [[0.23681122812278818, -1.0781275562318273, -0.31781685039501034, 0.59887738604323948, 0.2247434753593682], [1.1401838513159701, 0.5965953829151297, -0.22154777199991152, 0.41832864208618731, -1.0624226561543113], [-0.4230828586442606, 0.40937270437704898, -0.58055654464951112, 0.29659961382946581, -0.51071224676376814], [2.3186275264262841, 0.68255554180739186, 0.3185048645348374, 0.31637573748704884, -1.0302850692334693]], [[1.5594458437403178, 1.0137443680860616, -0.80921464535513254, 0.87220857507589555, 1.6047621232521538], [-1.6604686691875079, 1.0792191799981752, 0.93562263068338569, 2.3551649542967392, -1.7406457556092982], [0.958544605074794, -1.1123946038517842, -6.9639693440297492, 10.223528543119805, -2.6576365731125273], [1.1432086491172333, -1.0260306071153187, 1.815520527100873, -1.3710622995703141, -2.6546486370904292]], [[15.40916913242002, -1.8720472530315162, -2.358380166944213, 0.81591785272136585, -2.6513958999737248], [14.451527261392851, 19.557167060962417, 1.1231787156021724, 1.0939500356896381, 10.817328062614481], [19.336294476118891, -2.0744278254068123, 3.2979230700212594, -3.0122748812039415, -2.0023916411515925], [-1.1394936230197323, -1.7977125660422064, -2.785211473071417, -2.7886253762367925, 4.9339589554376744]]], [[[-0.18343516575090274, 0.075899514816196972, -0.094779646095026468, -0.31256495778315829, -0.078114673253235095], [-0.092280984532814789, 0.2285469153379511, 0.080469751289158217, -0.11234594128516781, -0.10366242863720368], [-0.14013398600399779, 0.22388251627875927, 0.087992769275922561, 0.27179088082960812, 0.18252421762534865], [-0.1431147033939317, 2.1045995196526937, -0.079738036674761689, 0.83228201223040088, -0.14094868570121372]], [[-19.922302625267804, 2.4012122463777588, 8.3430107199695911, 1.1800517750059236, -1.9266518735706204], [1.1051386154592648, 2.563796905000717, 3.2248373028173174, -1.0831013525872437, 0.83668626812731584], [-1.779896617448631, -2.5459643624316817, -2.3731741217694724, -1.1637080810621734, -1.7444398645566903], [-1.8435252402644016, -0.89355088032839913, 4.0705943961899136, -2.1584363916395448, 0.8674506863991438]], [[-0.13441384617680815, -0.27454705212596647, 0.14495719032691756, -0.19488492742296662, 0.1196158829748365], [0.27428189572353512, -0.56155666002834381, -0.6004809082828988, 0.17088644119409305, -0.1343441156067772], [0.20066160366876126, -0.76180727826753025, -0.1591192273424093, -0.13557636210697219, -0.12588535838595816], [0.70279582291393616, 3.15408411627496, 0.19575162826352557, -0.38991366178482784, -0.51651271888605721]], [[-0.59778110619957758, -0.60184093705339037, -0.68580107598417794, -0.77529870904834242, 0.38484499993150162], [-1.1188762523437474, 18.783595721191649, -0.4505349411493354, 0.47618975751730203, -14.756650975115615], [1.2492326835155327, -0.49955988309500221, 0.71147556177730398, 0.95754543750218701, -0.58708773567959571], [-0.41094251863716841, -0.69250273323411993, 0.43802584943189415, 0.60032792208159003, 0.38764317870978349]], [[-1.9889197896286577, -1.0615200697013465, -1.8632485608950231, 1.9250291753792965, -1.06840177309215], [1.0673527955073911, -1.1800503803648716, 1.3295119206343677, -1.9607225090172971, 1.8731565214886334], [9.1254973384136182, 2.7627155557739211, 2.6652383837943234, -1.8589814386573504, -1.1248152354867427], [-1.2257372780637303, -1.6221641081703713, 1.0037193737555423, 1.2592075761424031, -12.535940321607459]]], [[[-1.3072412548971559, 8.1796322282316378, 16.627899624001412, 14.972307975475523, -1.7876543782145822], [-0.93701196659349617, -2.6786196267703737, 2.977812738869789, 2.6587154415178897, 2.6353540343652364], [9.6842402566604573, 1.0129431772939652, 14.203229220826632, -1.077981997925779, 1.0283531480350419], [-1.1176791877832641, 1.0666556397297626, 1.4806019589014294, 1.8070178098079117, -1.0746245762289821]], [[-0.62154556330302646, 0.88963896127805109, 0.69803193039423828, 0.74065484096261447, -1.3635423234205148], [-0.30899086149754718, 4.0593054373086312, -0.63339090830130051, -4.2241806011685972, 11.936210547355785], [3.4179138333889005, 0.56401703496851019, 0.71353398533941248, 0.35705067372712451, 0.32604507281633038], [0.82895562737594297, 0.9389990741723041, 0.45004872062245743, -0.37565644649262564, 2.9601325389560946]], [[-0.041548269879665922, 0.064509944856676465, -0.07788312673677171, -0.17436594733242344, 0.10071464510026035], [-0.044872491693651942, 0.21029634940495873, -0.13531447360644025, 0.17833555876806734, 0.072052931561558245], [-1.4400529144738039, -0.29996040069430985, 0.05849703378805577, 0.054927412440088613, -0.047016894286006403], [-0.041829401425083922, 0.087604958340182268, -2.9603687483787309, 0.046935592466290837, 0.039199887085586776]], [[-0.82496643635037781, 11.880013823895119, 3.0940698430628593, -0.54748000020696397, -0.74019236051867443], [0.96132242343898533, 1.2010625407205735, 0.86874665215441993, -3.6828530059321327, -4.9812199595517095], [0.52104885327521377, 0.62883765915327827, -1.3924897111815739, 2.5915891808303031, 0.72415749585345213], [-2.9861891847146267, -0.76198530895388772, -0.70579881122651322, -0.62049602990439778, -0.52539249350920436]], [[0.48327308922648604, 0.30896832379015438, 1.8491971733231853, 0.30741307427643677, 0.5990750362004118], [-2.6041454675791642, 16.730488855299484, 0.61770610887562138, 12.08673725313407, -1.3026230825011471], [0.41164189899687997, 0.86091507779282295, 0.51169994828979026, -0.34554381860430217, -0.56117232709639653], [-0.60388681961862478, 0.69989427115773428, 0.41003432227412745, -0.58399180222103364, 0.58902285903211982]]], [[[-0.82375307385083907, 1.379869652263169, -0.44919089090512371, 1.0318618717506456, 0.74983481211010938], [-0.90627885344122505, -0.67643051346921645, -8.5848300671375384, -1.0276970838357478, 0.47522220036780427], [0.58051278960280428, -0.38787768033349174, 0.66989406883910096, -0.37946663106417278, -0.59141321268296621], [0.35317908649400942, 0.61666332776079447, -0.34321744484570976, 0.60577713614490514, -0.83188899897423452]], [[0.63077412329231342, -6.0283017952564695, -0.46850828735892813, -0.4106458626501695, -0.71517709424582387], [1.1136180653294181, -1.1376418507648431, -30.052865688657086, 0.55263576260639424, -1.8723862045054569], [0.78967325307023173, -0.87892830851153725, 1.0127629272671526, 18.153697615201132, -1.5122231209150954], [-0.63232795431661593, 0.62072637287561405, 0.48055501605825496, -0.51448633109246866, 0.71750613572365951]], [[-1.1480377984303753, 0.32828484210263814, -6.491212058650806, -0.5234981824839644, -0.78282289950310491], [0.53764256441698255, 0.2884703522635873, 0.69703585616884123, -8.6185281098339086, 0.21352846831533984], [0.85257907894476637, -1.8791364692330441, -0.31145011533732958, -0.45467436789545107, -0.26265915978354559], [-1.1292823774588299, -0.27031965353371851, -0.96678717057180685, 0.23503429593460939, -2.9894039125954284]], [[0.026561327257988668, 0.045960391428641889, 0.371682668488026, 0.43750893254273082, 0.34377848994816268], [0.024437043224093354, 5.4630365424677967, -0.030045517446386602, 0.025391220923305195, -0.025465110723190486], [0.11531742180824572, -0.021630988538065558, 0.035050358791701208, 0.020766279422095333, -0.021796749895601895], [1.8772229449591222, 0.032658713625623358, 1.537157106530876, -0.023300532980698913, 0.081474179411619302]], [[1.0444538764931894, 0.72593743970285374, -2.4713539724929672, 0.5079196889229769, 15.094731323845393], [1.8660735879959243, 0.69691355139204025, -0.74064426409980788, -3.0327230671037952, 0.67873243642412251], [-1.3132200654837938, 1.5116549566586592, 2.4063972178523971, 0.63446438732599308, 0.54607802026500718], [1.6775964478718766, -0.76358599349025513, -0.91811385208751584, 0.8066126063907143, 0.5919604289046454]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[1.7865542024572907, 3.4855054629079021], [4.0303890624734038, -4.6573708057223007]], [[3.5484148590967592, 2.1611487972295427], [2.2395107368710763, -1.0095043825362318]], [[1.8645960660598879, -2.1813146678693105], [1.9510620123778395, 2.9390015942110583]], [[1.5751328572537506, -0.77847452936882799], [0.6749677995883685, -1.0972643328736953]], [[0.91985272686150754, -2.8138942303783141], [-3.6705968517691154, 3.5870543263877881]], [[-0.71361412890608467, -4.0565999188396971], [-4.4845104707953638, 3.7375597483315843]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-3.8369932983058961, -1.1096762242671221], [0.092528531553220716, -1.7295273211874127]], [[-3.5897140561162733, 4.4872426070510603], [-3.378622452285712, 4.7664748847630776]], [[-1.3789424951562337, -3.5248998020432265], [0.31242261936919835, -3.9778109000818507]], [[-1.4598282387531034, -1.4398842529057654], [3.742797504161187, -1.0505040004525315]], [[4.6339936933145225, -0.74663700578375725], [-4.3294893798112497, -0.19216222711792952]], [[-4.0914598964206785, -2.2923694818332061], [2.4865228644955595, 1.77971497864882]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.46561306303195465, -3.141011212716466], [43.558338112771168, 2.692857608357853]], [[-0.98849512903426195, 0.48162067141937148], [-0.66284729013033061, -0.21179265745494644]], [[-1.352192765550118, 0.61883026195663782], [6.2449448004666275, -0.7388489971080785]], [[-1.0789850582690013, 0.54065076953083113], [0.18033778178967719, 1.0445122840094094]], [[0.19850107439476708, 3.7687580558969516], [0.84781287809259864, -18.666802420989953]], [[0.17441552574678146, 1.7696099825913043], [-1.8035267380117721, 2.1000889429886027]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[-0.22737432027613469, 4.9782513662365027], [-3.8086416663168499, -0.22007692267947743]], [[0.36800508673362664, 0.77679005948848445], [-1.9393735670888193, 4.009102525689519]]], [[[1.86047445474928, 1.4929343200991463], [4.9713051335460658, 3.2680554438496205]], [[-0.91791632602425199, -1.0087868077787512], [-2.7938929693575765, -4.7166887320438553]]], [[[-2.1491200472033967, -4.1887196832354645], [4.9485098252512003, -1.6678908164399164]], [[-1.0080313504101266, 4.7550924275274014], [4.4665241341953958, -2.0689734227591297]]], [[[4.4404213475478098, 2.2896653460896923], [4.6115723306345053, 2.1599093904448017]], [[-3.505290559687638, -3.3297721880179463], [-2.078229538750076, 3.0119543182196615]]], [[[-0.71137999818806463, 3.4402175327905784], [-0.17865431459489933, -0.70141597793598898]], [[-4.8274628918890352, 3.0220058121960562], [4.0814740168611436, -1.1628531131567033]]], [[[-3.8503272778905817, 4.152733243929589], [-2.4010300372818048, -4.4619812864655515]], [[-2.8941598667718358, -1.6053619502469854], [0.23531445778350424, -4.0968033514290747]]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[1.1457555149689247, 0.60087107726983202], [-0.0049559860122405652, -2.3850893700493794]], [[-0.42766384422314907, -2.5912156679805642], [-4.3037141599994344, 4.823816482548434]], [[-2.1408512939433963, 4.2405050262237474], [4.1240217070595069, 4.2802757565950103]], [[1.2285759471959796, -2.8551933091358785], [3.1831684435038721, -3.9223283951078525]], [[-4.1653929357782999, 3.0792880563977505], [-2.0783543209799937, 0.80830882773570334]], [[-3.7752940291447601, -0.58241276461106395], [4.473385846139605, 2.8607323386977237]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.19844924794649718, 4.3449508217043347], [-6.3385338559181648, -0.36626313198404775]], [[-74.254666140039049, -156.73774251378552], [0.81312406631063372, -1.6809024332729792]]], [[[-4.350319719285201, -3.4909060942737864], [-1.9185223348932583, -1.2612054967993243]], [[0.21328468664479627, 0.23439911905740593], [-0.57918724301915314, -0.97779191001727683]]], [[[1.0038623669394475, 1.9565673221141595], [1.1669623770397803, -0.39332362681461219]], [[-0.24442920576401839, 1.153023132586239], [1.0435131725598323, -0.48337386196935422]]], [[[3.6142831525249486, 1.8636742411533229], [-1.615152401723088, -0.75648446763084354]], [[-1.1011954352717792, -1.0460559179057141], [0.52984587964183727, -0.76789957770398309]]], [[[0.17078340726938981, -0.82590468314312271], [-0.058018058500150474, -0.22778511301619758]], [[2.322733348764503, -1.4540378325727965], [5.0493992850411908, -1.4386247845568836]]], [[[1.0198748092642784, -1.0999761109654107], [4.1225573736956402, 7.6612010546253542]], [[-0.64697300128255364, -0.35886954657228226], [0.082256719582030249, -1.4320820217993695]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank3_Symbol_rank4(self):
      arg0=Data(numpy.array([[[-4.0467138719993088, -1.352662515091807], [0.3649947817347865, 1.592190335189926]], [[-3.7626129326189472, -1.7865325268378172], [3.6437561967206058, -1.4780037019683281]], [[0.95568883883713696, 4.6675307265429318], [0.44722613165538405, -3.345509117688553]], [[2.5267431811591852, 0.27629899970731131], [-3.3305713443450715, 1.4765824806355194]], [[-0.66154700296336966, -4.0543085601259907], [-0.5956676107638863, 4.6411381564967265]], [[-1.1150837755555743, 1.762839674887088], [2.0256523406546965, 0.61367774778093054]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-0.0039940180046285079, -0.96259922471803616], [-0.81800747403365737, 4.1489418261631528]], [[-2.7194420713997172, -4.1071526194659134], [4.2935750046885719, -3.1150745626524023]]], [[[3.0955332288930499, 4.9621555839921321], [-2.2736627791238604, 0.91335391910023844]], [[-0.6111310879509011, 2.3779449370409065], [4.0236252920218245, 3.9214862128282792]]], [[[-4.0600310653230771, 3.2559740133275668], [1.2168748501593427, 2.8983615110202079]], [[2.6325186261319091, -4.8065569318676253], [-0.74764585474943956, -0.48754030547075367]]], [[[4.241762510533519, 4.8852976119331277], [1.2130627143896362, 0.63577482828248133]], [[3.3041547885349942, -2.0418094245140264], [-1.6278298115892396, -4.8814046284113397]]], [[[3.5063575689650062, -4.5291678144320926], [2.2639029641639894, -4.2824499779078673]], [[4.7135988791897088, 1.9024867094756113], [2.8446286750780594, -1.7798846452120962]]], [[[4.0088491649300835, 2.0168868315763859], [-3.3727575278690116, -0.969505064813311]], [[-0.45748132178619638, -1.6624107917179742], [-4.1261747947906438, 4.7651911480338178]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[1013.1936980027966, 4.2039446615850631], [1.6536065476538067, -0.32602590534336767]], [[-0.13421678864698924, -0.088868083451510441], [0.37083091210733682, -0.51112430960054411]]], [[[-1.2154975102510666, -0.7582617813832967], [0.78575087882040473, -1.9560134242351124]], [[-5.9623152357344145, 1.5323131078278303], [-0.36733134790134708, -0.37689886480624724]]], [[[-0.23538953851849112, 0.29351857076415494], [3.8356703040840605, 1.6104032256831846]], [[0.16988526774927923, -0.093045008723450368], [4.4747243583791976, 6.8620154685636381]]], [[[0.59568237846521432, 0.51721376707679123], [0.22776975702062835, 0.43458625194980005]], [[-1.0079949510542725, 1.6311861941462951], [-0.90708652103744292, -0.30249130998920598]]], [[[-0.18867071881623376, 0.14606369868993699], [-1.7908490886327186, 0.94672642553706321]], [[-0.12637214706447072, -0.31309948595019205], [1.6315444603220028, -2.6075499718374586]]], [[[-0.27815558273193897, -0.55287374487146201], [-0.52267014759311559, -1.818288257448704]], [[-4.4278361633338639, -1.2185028819268791], [-0.14872800555025145, 0.1287834482849953]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[-4.583067409848999, -1.3688987855058121, 1.3730544220139027, -1.2418004568596333], [2.4824856925925758, 1.1946488728901672, -4.8596303442778579, -2.4940785859507897], [-1.9976535470630088, -3.9182161165797735, -3.7431123564199242, 1.5551032683413126]], [[3.669971525537413, 4.4192710106591413, 2.090160533708687, 0.77753751335303534], [-1.3314503173063397, -2.4126086326029439, 2.4833885771978661, 2.4904565482714593], [-2.4085733420759468, 1.5770650349973527, 1.4641543235672376, 2.80394361546543]]], [[[1.4669344624265204, 1.7565357895035474, 3.5000057479790492, 0.65662002333756941], [4.7465485940430323, -0.63149978617604319, -0.0088780085446416024, 3.3288515567843202], [-1.1361185135330611, -3.416053892831604, -3.7042483895520348, 1.5323398970856905]], [[-4.2806013870114512, -4.3544390453613673, 1.1788597824656257, -4.2948135053543179], [-1.5987652657925153, 4.141410823751615, -1.9047573648403038, -3.6861016377610003], [2.0159066725157615, -0.89739181413721258, 4.2704679096361726, -0.80643645511267703]]], [[[-0.96699836801883432, 4.2473801780568632, -2.4664327318386903, 3.1156021621622791], [-2.9587690488447449, 4.0716317719718536, -2.5008944849021808, 3.2276292278891177], [3.6585048119796522, 2.2889645786478621, -0.79864848369138208, -0.41909933591114967]], [[2.1863307167435782, 0.64790167491261119, 3.3004554868949789, -0.20910728350353303], [1.1369501426165085, -4.9211754783872932, 3.3713283436917649, 2.7949500632040305], [-4.8019213987960319, 2.3131640539984577, 4.7526524870456619, 0.73737794415406643]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-0.41388445449058064, -2.5974605654098393, 2.480308615613195, -1.9036783962026096], [-3.1541173636374396, -3.3101864604613986, 4.7650885475720131, 4.9556614874280864], [-3.7341256749786744, -4.7739198398737104, -2.2812176457720259, -3.0237753359232977]], [[0.52585966261881012, 4.172572455415418, -4.2616119541046507, 0.94578746468690245], [-1.9728274315480041, 4.2280454346880401, -0.85828142778471772, -3.6927301477987351], [3.2846532397551425, 3.7383095107307867, 4.1237847699178776, -0.65784619350778417]]], [[[3.6675674497488444, 2.8243331902843174, -4.714655946538139, 2.8308810770385042], [0.79836727388410544, 2.0105758463959722, 4.3747433313106239, -1.8789310155863426], [1.1868055324433646, 4.5578233140632349, 0.72482208511841861, -1.8565635151980828]], [[0.92974541518407694, -1.5055042717382272, -1.5727371024720105, 3.3485104295405428], [-0.96637467637148511, 4.3384556474493117, -3.5138236033522476, -0.65062051147704203], [-0.95779159152816629, 0.1564991959068589, -3.0417376361197035, -0.55761905642425891]]], [[[-2.620812034463158, -0.88250024923996051, -1.9146735876369059, 2.1397618442352169], [1.5587525301947043, -0.9923619444979801, -4.894311289839866, 4.1560194378937698], [2.1714359425112919, -4.8544682699888497, -1.101153401362942, 2.3883068994036494]], [[-2.886391790500257, -4.3395149908868138, -1.9116865613403391, -1.3657971040957562], [0.82473162593266025, 0.41579205867976121, -2.6877297780099241, 3.8339201105572869], [-3.0095162441139811, -2.8389232418133292, -3.0504367668585086, -0.12409625748422748]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[11.073301642821916, 0.52701427068241979, 0.55358208787838648, 0.65231630475858371], [-0.78706192775581618, -0.36090077920373348, -1.0198405120412752, -0.50327864247345488], [0.53497223204048094, 0.82075448436591814, 1.6408396469128457, -0.51429193494180947]], [[6.9789941811865779, 1.059123851743675, -0.49046242506793941, 0.82210596184041695], [0.67489446670031361, -0.57062031850680772, -2.8934432189773225, -0.67442148453659412], [-0.73328085684183097, 0.42186582744697854, 0.35505110117480632, -4.2623087936621946]]], [[[0.39997477415908911, 0.62192937984300711, -0.74236716054519758, 0.23194899590218229], [5.9453195907577525, -0.31408901450200388, -0.0020293781537080143, -1.7716731104923045], [-0.95729121787463334, -0.74949239087248432, -5.1105622546626037, -0.8253635733664626]], [[-4.6040575377980764, 2.8923458585300543, -0.74955933869221192, -1.2826041894525679], [1.6543948272687687, 0.95458180520675728, 0.54207540840215562, 5.6655171067275356], [-2.1047445919830658, -5.7341624596671972, -1.4039566920321047, 1.4462139444874134]]], [[[0.36896898949752893, -4.8128940266190892, 1.288173998829099, 1.456050901438446], [-1.8981647128265857, -4.1029704882845204, 0.51097985738949725, 0.77661552745885343], [1.6848320230660578, -0.4715170542567208, 0.72528358238085866, -0.17547968228697788]], [[-0.75746152131504396, -0.14930278528205002, -1.7264626710463109, 0.15310274335511587], [1.3785698363764955, -11.835664909073053, -1.2543405111908228, 0.72900581718114232], [1.5955791593375317, -0.81480330990595962, -1.5580236045804614, -5.9419837399027644]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.07161502931)+(1.-msk_arg0)*(-0.721128914102)
      arg1=1.8151050027
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.69225197702)+(1.-msk_ref)*(-0.397293221621)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.59394987630841989, -3.3346806616329152])+(1.-msk_arg0)*numpy.array([-1.3409776806270668, -2.980392011723171])
      arg1=-1.21521240538
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.48876218978449515, 2.7441134133099365])+(1.-msk_ref)*numpy.array([1.103492422136779, 2.4525687842884789])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.0201843048097761, 2.3406424683330123, -0.71025000397171034, 1.7090937038847818, -3.507355664695063], [4.4715373763576025, 0.3140686993942392, -3.2307060824277722, 1.4513365703074541, 3.4112043356957642], [-0.14562167961714589, 4.6589307329960139, -4.7540409279807694, 2.8299312336780886, -4.7782739711168487], [-0.53974442802661393, 3.7334222293902624, 0.10828311295914794, 1.5368233721538509, -2.7989207185732869]])+(1.-msk_arg0)*numpy.array([[3.1000800254346075, 4.3792135895360111, 2.7996262841863251, -4.3541481086170162, -2.9180705896057901], [-4.9421528150220944, 2.213413088763545, -1.9645664425006606, 4.0548618819457172, 0.15710717354913672], [-0.04023653614057654, 2.2114796163834116, -1.5609621634023716, -3.731148898507012, 4.9727468174051825], [1.5915162774025502, 3.7963523415556786, -0.89613551929269164, -2.1245477970382973, -0.55046225803908388]])
      arg1=-1.8359491625
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.6450261077501929, -1.2748950331225481, 0.38685711918390064, -0.93090469975772672, 1.9103773330655427], [-2.4355453123091291, -0.17106611980860933, 1.7596925603504743, -0.79051021670638055, -1.8580058780365136], [0.079316836539790922, -2.5376142369102541, 2.5894186097804681, -1.5413995613196125, 2.6026178004939626], [0.29398658691208518, -2.0335106797360609, -0.058979363465528935, -0.83707294490857465, 1.524508834856019]])+(1.-msk_ref)*numpy.array([[-1.6885435004192608, -2.385258632967735, -1.5248931404933703, 2.3716060322143986, 1.5894070757585328], [2.6918789016468696, -1.2055960665888434, 1.0700549245212552, -2.208591591082357, -0.085572725410030373], [0.021915931531486958, -1.2045429478970908, 0.85022079874984857, 2.0322724478014913, -2.7085427630478454], [-0.86686293385048618, -2.0677872890512261, 0.48810475671007225, 1.157193151333618, 0.29982434660140722]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.6251075325111719, -3.2641264921192681], [3.5704446752098242, 4.8855059919019084]], [[2.5497385474376069, 4.7327818219006748], [3.1696367967181374, -0.94273925413867588]], [[-4.0847116891888042, 1.6286820447425265], [-4.9073162595310009, -0.038785115322587771]], [[-0.85901046699259709, 2.6909959251290116], [4.5454208994030036, -4.2415397619406328]], [[-2.536674160862403, 3.6959707902246404], [-2.2939251499346378, 2.0891428754300581]], [[-4.2616201264644644, 1.4914947044485807], [-1.576814511049025, -3.7310181969269718]]])+(1.-msk_arg0)*numpy.array([[[-1.2510232016203271, -1.0796917088478439], [-4.7330135889092437, -1.0217419042971021]], [[-0.77483302170091228, -2.0261330854147639], [4.4674775299074092, 4.7653112174033705]], [[-4.6123939984420943, 1.0850611375161412], [-4.3765253026508049, 0.48813945712868811]], [[4.5606499141013135, 3.7780212771516481], [-1.8196454509299596, 1.0642765770182834]], [[2.9932063398650968, 4.672905267140175], [4.3055309246425661, -0.30767668076516408]], [[-2.6220634609263471, 2.4602972897238828], [2.4612782879288764, 1.0992352966430454]]])
      arg1=1.72623137936
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.3621226794890724, -1.890897437708241], [2.068346525213828, 2.8301570984762039]], [[1.4770549173924408, 2.741684503302984], [1.83615987672364, -0.54612566160707021]], [[-2.366259667176462, 0.94349000036624009], [-2.8427917127520757, -0.022468086136295522]], [[-0.49762185838155654, 1.5588848385609195], [2.6331469545490207, -2.4571096393338272]], [[-1.469486762432783, 2.1410633791164106], [-1.3288630813720954, 1.210233402319012]], [[-2.4687421265913012, 0.86401783809736676], [-0.91344331351201502, -2.1613662232898081]]])+(1.-msk_ref)*numpy.array([[[-0.72471350977626314, -0.62546175545105764], [-2.7418187651472188, -0.59189162965928821]], [[-0.44885814900964155, -1.1737320440603443], [2.587994624202326, 2.7605286721042699]], [[-2.671944244321383, 0.6285722473193579], [-2.5353063065502397, 0.28277753664186178]], [[2.6419690712596871, 2.1885949487009766], [-1.0541144557381923, 0.61653182171569987]], [[1.733954309751109, 2.7069982176309124], [2.4941795034708925, -0.1782360606140895]], [[-1.518952495175812, 1.4252418992864919], [1.4258101882289387, 0.63678328976499876]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-2.8563777382049627, -1.6373406306290974, 1.1522129701298347, -2.3256541855839097], [0.25206749094787284, 4.0233442460919626, -2.5791844246953732, -1.9411319575965913], [-0.33515450655856593, -1.4542973431678341, -4.3765240999792159, 2.2422304484887823]], [[1.7374622713260166, -2.5314422125261582, 4.7235439585980714, 1.3898645450919602], [-0.036265333471833472, 3.8012596727566077, 3.5320823155887719, -1.0964066638827874], [0.7127527345474185, 0.3883908937070748, -1.1913725661321162, -2.795562787542909]]], [[[2.071420686681769, 3.5647802423843586, -1.3623213131802192, -4.2771822646088236], [-4.5459941945732938, 4.3371687017883236, -2.5288882146945744, -3.0350151410373147], [0.32530107169954636, -2.5433119721743855, -4.9161405911038116, -2.1427385583843717]], [[0.033933189727012092, 1.2372619097898934, -0.46540394427049403, 3.710482859938713], [-1.3592520516809161, -0.034131768690967235, 0.74810929125373704, 2.0295390768572705], [-1.2988113880468188, -3.8978783822741092, -2.8967392196439046, -0.62201530702582364]]], [[[-0.5776215773233595, -0.21324924394292566, -2.7216205441395278, 1.0930303995336015], [-2.9885185337952045, 1.6682505663199931, -3.6073165169843566, -0.33327910945681971], [-4.0296246712933312, -4.6554526283656994, -1.1558255735773559, 3.6763504981666593]], [[0.75356024103681918, -4.9978697625136226, -1.3089318913128767, -3.588436300557162], [3.0717809941613634, 1.7491423932784356, -0.54896798470151698, 4.7002544240324013], [3.7007392950182911, 1.2819502755342684, 0.13459650802902345, 3.0977261492855757]]]])+(1.-msk_arg0)*numpy.array([[[[-2.1397404490134111, -4.7392656668629085, 4.4058713899144912, -3.468047446843233], [4.2576522414499198, 2.5811452089540659, -3.5006545841877634, 0.081995239080018045], [-0.073121745199429711, -0.042296883550594799, -3.3736648339381494, 3.427384474461256]], [[3.2734874277996298, -4.734124965570329, -3.2744742972110394, 0.56305161207354359], [-1.1627280446819133, -0.64667131886072582, -4.6674188646875745, 4.6296164331219813], [4.6911827090457283, -3.8792833863588703, 0.06931921923014972, -3.3725193815823573]]], [[[2.3446802783612597, 2.166889500876648, -4.9074409267232078, 1.7210376610238232], [-0.049765540550725795, -4.1729790914718965, 4.0145201485837667, 1.8842608293325096], [1.0835542666824427, -2.2257139182099306, -3.3909660384919285, -4.0631900722169529]], [[-1.8358833940025732, 4.2200945698562826, -1.1073041022244299, 4.5761425136126661], [-2.3874903668872269, 2.9090925768697842, 1.2882397928395726, 0.38344309000399512], [-4.1444938359422103, -3.893948814245114, -1.7589664897549504, -3.219376115710312]]], [[[-4.5871899299425909, -0.86214842035451333, -1.2713986886226714, -4.3121003162191727], [0.0028515877301344972, -0.94235476219354819, -1.5679921942933817, -4.1825022108494103], [-0.83523386021306845, -0.64511487405858237, -3.9205727072355634, -2.8405866753688311]], [[0.37939633022031938, 2.2296701454581775, 1.1551987832868624, -0.28295253381631724], [-1.9579703259392689, 0.9079359902184736, 4.7841054888975325, 4.813425672329835], [-3.0104756940629374, -4.1248519671820727, 0.5236877750751967, 4.4852868153496335]]]])
      arg1=1.2962396453
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.2035876996650652, -1.263146545905905, 0.8888888519234307, -1.7941544945129886], [0.19446056279910365, 3.1038583495535872, -1.989743512354254, -1.4975100974826614], [-0.25855906180108151, -1.1219355529196007, -3.3763232870112434, 1.7297962275848753]], [[1.340386615718473, -1.952912196214325, 3.6440360204372411, 1.0722280792210328], [-0.027977337063621138, 2.9325284769175206, 2.7248682976107914, -0.84583639133150079], [0.54986185396415854, 0.29962892673035496, -0.91909900337514627, -2.1566712588028505]]], [[[1.5980229382659381, 2.7500935149647758, -1.0509795145658982, -3.2996848075985317], [-3.5070630735995771, 3.3459620815554936, -1.9509418832086698, -2.3413997188235895], [0.25095750841954884, -1.9620692681271077, -3.7926170588307384, -1.6530419866040293]], [[0.026178176118903516, 0.95450090133753851, -0.35904159077173009, 2.8624975893830102], [-1.0486116950732389, -0.026331372300392727, 0.57713810402676957, 1.5657128558083448], [-1.0019840025372408, -3.0070661674346284, -2.2347250604032864, -0.47986135070036745]]], [[[-0.44561326249895111, -0.16451374922520373, -2.0996276066730988, 0.84323173071845481], [-2.3055293399082082, 1.2869923955548734, -2.7829086466066557, -0.25711226366576467], [-3.1087034607366411, -3.5915061271577269, -0.89167583923755533, 2.8361657595429266]], [[0.58134330620743091, -3.8556680322414385, -1.0097915891225924, -2.7683432716810952], [2.3697631879223087, 1.349397389300931, -0.42350809643209003, 3.6260690228607628], [2.8549807965154734, 0.98897629013339272, 0.10383612977498417, 2.3897788966056934]]]])+(1.-msk_ref)*numpy.array([[[[-1.6507290582949568, -3.6561647254354424, 3.3989636143933342, -2.6754678113830797], [3.2846181312862526, 1.9912561834608657, -2.7006229880996053, 0.063256234583795395], [-0.056410668709652383, -0.03263045047565355, -2.602655185070359, 2.6440978617527287]], [[2.5253720943227327, -3.6521988682645001, -2.5261334268548667, 0.43437308380003076], [-0.89700083537509512, -0.498882533954768, -3.6007376271857101, 3.5715744769128333], [3.6190705368789815, -2.9927208293772423, 0.053477163332724148, -2.6017715117774274]]], [[[1.8088324075411171, 1.6716735279087669, -3.7859055958600316, 1.3277156483076815], [-0.03839223767850742, -3.2192959894393121, 3.0970508911178114, 1.4536361668632609], [0.83592125160642805, -1.7170543473787598, -2.6160024118863947, -3.1345979016645065]], [[-1.4163148000116261, 3.2556438041008682, -0.85424335402635521, 3.5303213647254874], [-1.8418587763002705, 2.2442552096088404, 0.99382841553173318, 0.29581188277477666], [-3.1973206890903887, -3.004034653899891, -1.3569763092268283, -2.4836272577985232]]], [[[-3.5388440297820352, -0.66511499125909657, -0.98083613877338593, -3.3266227675173727], [0.0021998923890889483, -0.72699115908790013, -1.2096468426781299, -3.2266427168843732], [-0.6443514231650338, -0.49768179548963548, -3.0245739832491747, -2.1914054902307467]], [[0.29268996022107019, 1.7201064274957247, 0.89119229416719326, -0.21828720857435815], [-1.5105002636172258, 0.70043837457785341, 3.690756957047205, 3.7133763727845466], [-2.3224684609629596, -3.1821677281149001, 0.40400536812259064, 3.4602296200451366]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.434074465009)+(1.-msk_arg0)*(-2.09375845359)
      arg1=numpy.array(-0.153501662596)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.82781604882)+(1.-msk_ref)*(13.639972481)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.6183662257152918, 0.24319840580125884])+(1.-msk_arg0)*numpy.array([-1.3247060133160571, 1.5108038129023402])
      arg1=numpy.array(-3.9528644904)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.40941606514616452, -0.06152459979135038])+(1.-msk_ref)*numpy.array([0.33512558210216614, -0.38220480782272276])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.74384491862)+(1.-msk_arg0)*(-4.01292837447)
      arg1=numpy.array([-3.7725137212850957, -1.830587431696272])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.7273253648200142, 1.4988876636608774])+(1.-msk_ref)*numpy.array([1.0637279731629812, 2.1921533519727281])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.9350855697117106, 2.7869272205600071, 4.8091866180243983, -3.7713101353343892, 3.512234115389484], [-2.1782576043176292, -2.7809318883797496, -1.0575301841562093, 3.9791343404202877, -0.47096069985982858], [1.6315480956898387, 4.3434809697034851, -1.4839047534547669, 3.5666892254606459, -4.1819576151048121], [0.97695266266160985, -2.5632362841821008, 2.1335051795849189, -4.4934962341326257, 4.6829374166410069]])+(1.-msk_arg0)*numpy.array([[3.3894460170789635, -3.2241158259771883, -2.1502921938883057, -2.8275298843589161, -3.7310061431459318], [3.4211170997104841, -0.96407167121425985, -0.33960754934534343, 2.6726125050074288, -1.8146089688222888], [3.245811276585064, 2.7386730518571278, 0.087881173571568105, 4.2453861391898062, 2.3203737191171978], [-0.23638428667021483, 1.3883850911122151, 4.4910977131370409, -3.0422542926096465, -2.4949851300400527]])
      arg1=numpy.array(-1.21402427709)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.4176498156674691, -2.2956107823893368, -3.9613595121586735, 3.1064536406042844, -2.8930509724411264], [1.7942455068088663, 2.2906723867629739, 0.87109475824896343, -3.2776398425678992, 0.38793351067940884], [-1.3439171905249037, -3.5777546229393979, 1.2223023719238193, -2.9379060145510665, 3.4447067443694288], [-0.8047225093446515, 2.1113550466515716, -1.7573826321716761, 3.7013232098784297, -3.8573671919338492]])+(1.-msk_ref)*numpy.array([[-2.7919095862035364, 2.6557259906837865, 1.7712102092805018, 2.3290554709060953, 3.0732549699111305], [-2.8179972709634753, 0.79411234965470467, 0.2797370330687482, -2.2014489787827332, 1.4947056686366558], [-2.6735966799421256, -2.2558634975832037, -0.072388316469700495, -3.4969532482344619, -1.9113075108247499], [0.19471133414044356, -1.1436221806414328, -3.6993475319279501, 2.505925416840133, 2.0551361098228162]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.42861468873)+(1.-msk_arg0)*(-4.01508663813)
      arg1=numpy.array([[4.0354181320688305, -3.9773213098350046, 1.6877627416792285, -1.4140679729167616, -4.8716267828815436], [0.50100280673571795, -3.3576204392909137, -1.6214671229136424, -1.1107091846466588, -0.6572651913065588], [0.36657992984149157, -0.10600985914685612, -3.7157533861268788, 0.73724026316369429, -3.2973654195672641], [-4.0845877016674246, -4.7318685300519263, 3.3573145192816813, 0.44579425587899291, 1.3305374705278359]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0974363854725573, -1.113466663549934, 2.6239557133023417, -3.1318258906559113, -0.90906280101131476], [8.8395007556597527, -1.3189741868688716, -2.7312392746947478, -3.9871955233150218, -6.737941925578804], [12.080897856691063, -41.775498282598868, -1.1918483894175251, 6.0070168573317888, -1.3430767067695013], [-1.0842256335737808, -0.93591245416098601, 1.319094372390099, 9.9342120952095581, 3.3284404136105734]])+(1.-msk_ref)*numpy.array([[-0.99496173797191245, 1.0094951665580003, -2.3789402023010582, 2.839387296103423, 0.82417779872530961], [-8.0141000891529028, 1.1958131393119906, 2.4762060120662781, 3.614885600683909, 6.1087772351751894], [-10.952827231601816, 37.874653078865499, 1.0805578898527772, -5.4461033108755705, 1.2176650529238278], [0.98298455838945986, 0.84852032820161882, -1.1959221023431625, -9.0065912361518485, -3.0176426647599426]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.6028958688696102, -1.3161178073681756], [2.7696590088431332, -3.3760241435722484]], [[-4.0753601289076524, -0.017415349167820615], [2.4217911383224369, 3.9115138739043189]], [[2.3176850542952012, -1.1870840807291962], [1.3494772645052135, 1.4912567117419817]], [[2.3632617446859792, 4.8452120033515804], [0.43046102144541276, 3.6639001143636403]], [[-3.1563945924268877, -4.403763939849199], [3.7956999984316759, 1.3209613922073791]], [[-1.5375965673055991, -3.6976564116939201], [-1.6144088280003857, 3.0994915522660023]]])+(1.-msk_arg0)*numpy.array([[[-0.74204569696458034, -0.86036595567355612], [-0.097318574074382269, 4.7105913805811301]], [[4.3029413758104376, -4.8053492098223209], [0.7950285646863815, -2.1443993589679025]], [[4.4924133436625482, -1.2382807872431503], [-0.89170253332968485, 0.80817899822145733]], [[4.5134614233535846, 0.31152497651541289], [4.6194391355851483, 0.032300682369280409]], [[4.4279783008863554, 4.8937498399994066], [3.9316730555836852, 3.0435521722959678]], [[-3.0186456513952953, 3.0441379797687169], [-3.1088206909765659, -2.3370983432807266]]])
      arg1=numpy.array(4.53097749225)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.35376381180682054, -0.29047105389930672], [0.61127185327667155, -0.74509841404995758]], [[-0.89944391378625788, -0.0038436185563942766], [0.53449639563742435, 0.86328256553777316]], [[0.51151987805258059, -0.26199293259774264], [0.29783358377152308, 0.32912472293062162]], [[0.52157878707800054, 1.0693524767311313], [0.095004007894897324, 0.80863348375262711]], [[-0.69662552900010277, -0.97192359648236359], [0.83772210409855563, 0.29154004725614574]], [[-0.33935206474436258, -0.81608359741750658], [-0.35630475559885333, 0.68406686141497286]]])+(1.-msk_ref)*numpy.array([[[-0.16377165815404129, -0.18988528571262686], [-0.021478494263284218, 1.0396413110939851]], [[0.94967176137367204, -1.0605546414742946], [0.17546513220294449, -0.473275217684336]], [[0.99148877948425673, -0.27329219563789514], [-0.19680136016007702, 0.17836747139080542]], [[0.99613415230436753, 0.068754474514206632], [1.0195237437143472, 0.0071288551807026533]], [[0.97726777686696464, 1.0800649194062135], [0.8677317559638662, 0.67172087645572198]], [[-0.66622393436248306, 0.6718501658803], [-0.68612582964567326, -0.51580444777704437]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-3.40619075833)+(1.-msk_arg0)*(0.423291655451)
      arg1=numpy.array([[[4.0374046362295601, -2.9701302080101142], [-0.14957976659336492, 0.82261296669881112]], [[2.1608226820029586, 4.4514894274900243], [0.14591856234952516, 0.57678914960309768]], [[-0.82321044058855453, 2.2432145499661749], [-2.3759954526704821, 3.7869217114213303]], [[3.6838288679365334, -2.407135929957227], [0.61134612038733849, -1.736622655977059]], [[-3.7144423473036814, -2.3831324272804268], [-4.1197769771644506, 4.4602672931674121]], [[4.0860153505377124, 2.7217998878898184], [-1.3706563565347807, 2.9387377333333227]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.84365850471569714, 1.1468152975743393], [22.771734679820966, -4.1406966534977867]], [[-1.5763397833163757, -0.76518001756868981], [-23.343094281416061, -5.9054348728257713]], [[4.1376913974735263, -1.5184418086023093], [1.4335847126749723, -0.89946162553627806]], [[-0.92463327707228804, 1.4150388085454646], [-5.5716240681719702, 1.9613879541478525]], [[0.91701268719548501, 1.4292914314546574], [0.82679008529196107, -0.76367413306182885]], [[-0.83362162550011099, -1.2514479016216127], [2.4850800436551443, -1.1590659212961181]]])+(1.-msk_ref)*numpy.array([[[0.10484251483055276, -0.1425161948487205], [-2.8298724158432842, 0.51456963673880762]], [[0.19589374869874704, 0.09508989347179321], [2.9008759998396294, 0.73387589857076396]], [[-0.51419616975242399, 0.18869869378175302], [-0.17815339460144367, 0.11177723959124779]], [[0.11490535272555794, -0.17584867151991773], [0.6923928055397649, -0.24374417435702961]], [[-0.11395833233435648, -0.17761986308662264], [-0.10274625490581342, 0.094902755289861557]], [[0.10359521909165997, 0.15551902156150557], [-0.30882405603180013, 0.14403859543148581]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.230809997749498, -2.484034026286416, 0.81795825009696799, 3.0580762090792497], [1.5021788982410653, 1.9412692273587009, -3.7611615682691091, 3.9002095553844391], [-3.88092167986657, -1.399582659949794, -1.4714475153309445, -3.7720969246128897]], [[0.99989560015724344, -1.2488555822021374, 3.4356896677962805, -1.0593044453611675], [-0.28667541145351105, -3.3977807720796136, -2.0727321126464315, -0.96616676149851521], [3.248693866932161, 3.4770337630123791, 0.82342392154992261, 0.22406233943265352]]], [[[4.70996717134447, -0.78952241649698074, 4.4914637451261257, -4.7862434129136524], [1.0495955738190368, 1.6361384403639914, 4.4879154494900906, -2.5320351932286691], [2.3621779939637637, 1.8099485542605596, -3.8896695957103522, -0.54420141200138517]], [[-2.47680833965787, 1.0154631436412176, -3.465267236625249, 1.450921778360601], [0.46556454548709425, 3.0474516987121802, -1.883713829703483, 0.59254419025639127], [-3.0613069333552714, -4.3590405090165838, -1.2499165503554801, -1.0537044868217782]]], [[[2.1317071421460589, 3.0816152884261623, -1.218188608310331, -0.72702969407447249], [-4.0126026587960393, 4.5433557220097054, -3.1339610649074556, 1.3031359153783226], [-0.73941316798518741, -3.8890749779439195, -4.391209655228697, 4.9901120625248687]], [[-1.3963185325299374, -4.8159645059371083, 0.69731108197823577, -3.5431277360125102], [2.0338986575023359, 0.30481663269245907, 0.70683227948371652, -1.2062922650226326], [0.7289067177657369, -0.064618103808605554, 2.7470675387811134, -2.438116991719065]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0075527463759952, -3.2046319308160518, -2.195234242085744, -4.134965506853324], [2.3500907263739457, -3.3828212431229598, -4.773250446851776, -4.247518797029274], [-1.6627334346353031, -2.374247195482877, 0.51637360218920936, -3.0914275058853691]], [[-4.2691994344807007, -4.2996952579444079, -4.1554984558862227, -0.56876434186620894], [-0.94299820422726643, 3.2392764784890353, -2.7476004838145727, 0.736616013608141], [-0.031385560939323653, 2.3614623599422409, 0.41302189113575594, -2.3147802930583508]]], [[[2.6267990048016934, -4.5806174488867697, 3.4790739214982551, 2.0934744144562458], [-3.4795552817975928, -2.1330972261045646, 1.1321769580835719, 0.2510948182448729], [-2.3314085737105703, -4.1764616088151021, -1.7574049295455909, -0.44671494787306454]], [[0.99505889094051536, -4.6119839690208551, -3.1900528778071315, 1.3667760021198401], [1.2037607716990593, 1.9038621039960946, 4.5888580716689642, 3.1257511517575924], [0.27552778098910213, 1.773466786099986, 3.3157111853590653, -4.1775035928426547]]], [[[4.4568677800837939, 1.8859431929967929, 4.0259627066818915, -2.5066530079929983], [-2.3970110179504776, -1.0098667685070573, 3.0807981064166494, -4.0075846114007971], [-3.9631914974353908, 4.2372597925507289, 3.2015531766005907, -2.5892250316149221]], [[-1.9924658893369971, -3.1454966279692953, 0.80853099655696781, -2.4699295822858867], [1.7108093365221846, 4.5408650033505822, -4.1177416110889773, 2.8467237227387532], [-1.9214807464966066, -3.2280952357954584, -1.0245360507198518, 0.79234650296351372]]]])
      arg1=numpy.array(1.59674330858)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.6496494301964639, -1.5556877632981649, 0.51226659019118914, 1.9151958819219839], [0.94077669852695123, 1.2157678801139735, -2.3555204822580516, 2.442602724198975], [-2.4305232149790368, -0.87652326609359454, -0.92153040969330835, -2.3623690197061302]], [[0.62620935674773315, -0.78212670470660339, 2.1516856524993244, -0.66341561581560438], [-0.17953756869563561, -2.1279442687008379, -1.2980997643819756, -0.60508583709527275], [2.0345749059821552, 2.1775784149702448, 0.51568960215784942, 0.14032458331194686]]], [[[2.9497334643804161, -0.49445794590429232, 2.812890287994954, -2.9975033477168971], [0.65733519481746416, 1.0246721759046158, 2.8106680800690036, -1.5857496816317607], [1.4793724083705115, 1.1335250597479929, -2.4360018136972998, -0.34081959766299991]], [[-1.5511624982854866, 0.63595891599142695, -2.1702093367195943, 0.90867565911439729], [0.29157131455341573, 1.9085420194570668, -1.1797223884272752, 0.37109545853257969], [-1.9172192029270956, -2.7299569602659703, -0.78279116226070211, -0.65990850323883787]]], [[[1.3350343356325467, 1.9299378127135045, -0.76292075361414169, -0.45532033243398662], [-2.5129916857857562, 2.8453889223119009, -1.9627206502554002, 0.81612110623908951], [-0.46307578933439864, -2.4356294196103869, -2.7501036839373647, 3.1251811331916932]], [[-0.8744790255432715, -3.0161169175147151, 0.43670831637821722, -2.2189714007087615], [1.2737793523687095, 0.19089895730547993, 0.44267120186789993, -0.75547037431787489], [0.45649586495770417, -0.040468686144715202, 1.7204190078765973, -1.526931084425228]]]])+(1.-msk_ref)*numpy.array([[[[-1.8835543134671395, -2.0069800284089472, -1.3748197536132072, -2.5896244466057086], [1.471802458006626, -2.1185754935969952, -2.9893661812781476, -2.6601137291166412], [-1.041327949020058, -1.4869310444106683, 0.32339174331556558, -1.9360829566489639]], [[-2.6736917646937592, -2.692790528596408, -2.6024837139172194, -0.35620274017114983], [-0.5905759549202535, 2.0286770334859807, -1.7207527778887184, 0.46132400220492342], [-0.01965598400862037, 1.4789242248599388, 0.25866517737469852, -1.4496884255720748]]], [[[1.6450978630606767, -2.8687250006138023, 2.178856114694999, 1.3110901440493614], [-2.1791575784911843, -1.3359049101018656, 0.70905382975445108, 0.15725434194438262], [-1.4601022977097355, -2.6156124070617173, -1.1006183148550457, -0.27976628771363476]], [[0.62318024794186699, -2.8883690598470779, -1.9978495357802455, 0.85597728499966053], [0.75388496399552041, 1.1923407436659312, 2.8738858945019516, 1.9575789890342468], [0.17255608932790847, 1.1106774498882634, 2.0765461596375068, -2.6162649753373777]]], [[[2.7912237089922871, 1.1811185823435613, 2.5213587462982763, -1.5698534601794594], [-1.5011874514018722, -0.63245404761083757, 1.9294260322631831, -2.5098490094593378], [-2.4820467235647561, 2.6536887737569304, 2.0050518824139605, -1.6215662327824312]], [[-1.2478310562695822, -1.9699450820096169, 0.5063625394340372, -1.5468545063026675], [1.0714366719616497, 2.8438290481319126, -2.57883755576872, 1.7828311585473609], [-1.2033748544123719, -2.0216745036281796, -0.64164104851071191, 0.49622659992110113]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.903684940616)+(1.-msk_arg0)*(3.84439870516)
      arg1=numpy.array([[[[-1.1671048972320697, -4.3672154678470685, -0.39628581794849271, -3.9776552093840767], [-3.8617490637236829, 4.9988558274193284, 2.4548306686142238, 0.92628552659423846], [4.4159223546633299, -1.2200391546077016, -1.465153575902912, -1.9760214481900498]], [[-4.8394585824677776, 0.6258932614062509, 1.3323135896723155, 4.2855059003118239], [-2.9979975429411221, 4.1697202313527946, -2.3963438469139642, 4.1815858109238384], [0.59641406279939524, 1.6620816034275689, -2.2559713596855091, 3.8052627231508289]]], [[[0.51596695871057463, 0.70620690307720135, 2.7944082199219613, -4.8825681671291807], [-0.24171971419678151, 1.8727967051949239, -3.6559839742291569, 2.158487875782451], [1.1100830189025874, 1.8166006296093329, -1.6297260806279814, -1.18455508216607]], [[3.6494100212053322, 3.7593330526121491, 2.9873072052726277, -3.9424072025836532], [-2.5886466112572357, -1.0397669648279564, -3.3155262197845548, -0.57484319195214084], [1.3216843931545208, -1.3476713638928262, 1.6854085630903102, -1.6864747618763367]]], [[[-2.3081344080857802, -4.5896562883573147, 0.13504259399197505, 4.499777367347491], [3.9979535824269909, 2.1242902475097489, -3.866106195199106, 0.29624368743448404], [4.8425302855662515, -2.6342448300309194, 4.5780175570625836, 1.7787966046492398]], [[-3.9153057598948804, -3.4704546678101869, -1.3797710719393685, 4.6582551937687349], [3.9121674591095363, -0.91764341688963569, 3.049308001109015, 1.8202540726426699], [3.5791894687539667, 0.11687694708315988, -3.2161596667002534, 3.2755786142850045]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.77429624600078462, 0.20692474352803525, 2.2803867806679361, 0.22719036544041105], [0.23400923408123958, -0.18077835645091062, -0.36812516324234379, -0.9756008430128309], [-0.20464239813039467, 0.74070158912768591, 0.61678513125084544, 0.45732547156492631]], [[0.1867326530884594, -1.4438323534360036, -0.67828246114203916, -0.21087007266753938], [0.30142951342427787, -0.21672555722587122, 0.3771098800281511, -0.21611058136249794], [-1.5151972379294465, -0.54370672219241867, 0.40057465124109559, -0.2374829299217624]]], [[[-1.7514395551108033, -1.2796319841653172, -0.32339045318194781, 0.18508393732212153], [3.7385653198325501, -0.48253232083824388, 0.24717967775185881, -0.41866574779269383], [-0.8140696913905533, -0.4974593346971723, 0.55450112221785564, 0.76288975854415353]], [[-0.24762494084384981, -0.24038437881634436, -0.30250820505534548, 0.22922161364348512], [0.34909552222619822, 0.86912257379272484, 0.27256154248559833, 1.5720546981639478], [-0.68373731678790539, 0.67055289948847563, -0.53618152915928929, 0.5358425521947936]]], [[[0.39152180109189649, 0.19689599478469036, -6.6918511700806409, -0.20082881148153855], [-0.22603687661309305, -0.42540558743104423, 0.23374550387108106, -3.0504783019748838], [-0.1866142052450277, 0.34305275284731834, -0.19739656507472178, -0.50803163119041395]], [[0.23080826786825087, 0.26039381784696725, 0.65495280992210414, -0.193996443523482], [-0.23099341990376207, 0.98478877958823585, -0.29635738347430246, -0.49646088103730535], [-0.25248312460265632, -7.7319348525841995, 0.28098261102288324, -0.27588559061745493]]]])+(1.-msk_ref)*numpy.array([[[[-3.2939615918663274, -0.88028601599012246, -9.7010756656984647, -0.96649872922415103], [-0.99550712429113186, 0.76905572752759599, 1.5660545365973189, 4.1503387398231286], [0.8705766081013061, -3.1510453501778675, -2.6238878765950489, -1.9455247860203326]], [[-0.79438611564708672, 6.1422592991718377, 2.8855058861233038, 0.89706998300522856], [-1.282322166745363, 0.92198001109400207, -1.6042767443880768, 0.91936382008915196], [6.4458552286930288, 2.3130023804087352, -1.7040990740669058, 1.0102846990754688]]], [[[7.4508621923560092, 5.4437285849374817, 1.3757469927814969, -0.78737225443017989], [-15.904365591098827, 2.0527581528190497, -1.0515359838173535, 1.7810610605204078], [3.4631632406755068, 2.116259700949398, -2.3589232269512275, -3.2454368421025142]], [[1.0534301935992934, 1.0226278574839964, 1.2869110677255193, -0.97513993547954947], [-1.4850998542803855, -3.6973656936654482, -1.159513890199712, -6.6877346013373398], [2.9087115843033775, -2.852623279058752, 2.2809891852647115, -2.2795471311318862]]], [[[-1.6655870176771179, -0.83762235418667585, 28.468045462678162, 0.85435309156810879], [0.96159163079321208, 1.8097332554570755, -0.99438518009036392, 12.977149786564633], [0.79388222240348827, -1.4593930910804562, 0.83975184831493643, 2.1612356888434752]], [[-0.98188977845366288, -1.1077507338792878, -2.7862583752810188, 0.82528726856873758], [0.98267744040705585, -4.1894254722517097, 1.260744635754522, 2.1120121432170089], [1.0740975683804457, 32.892702976115395, -1.1953382616435357, 1.173654843268261]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.089526126672073048, -4.6362186954045059])+(1.-msk_arg0)*numpy.array([1.5302035341316378, -3.1972038648494774])
      arg1=numpy.array([3.2573346539853834, -0.81281252405411042])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.027484473099052591, 5.7039213326588261])+(1.-msk_ref)*numpy.array([0.46977166815187921, 3.9335071375408996])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.82472276433834057, 4.5464418192074501], [-4.0453319272200634, -1.3079511923161666]])+(1.-msk_arg0)*numpy.array([[-4.0873581367133385, 1.5523500254133316], [-2.6138945955984405, -0.22103855563260311]])
      arg1=numpy.array([0.84445438718408017, -1.2351382801426913])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.97663387964442139, 5.383880868175754], [3.2752056933679685, 1.0589512229878126]])+(1.-msk_ref)*numpy.array([[-4.8402355399479342, 1.8382875960770386], [2.1162768878772553, 0.17895854997471802]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-4.6141041686384145, 3.5035234509817421])+(1.-msk_arg0)*numpy.array([4.2873210590166071, -2.9981712095952453])
      arg1=numpy.array([[2.1805977730994961, -2.0539784745977654], [-0.91889834403759174, -0.46482350390987648]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.1159813265698877, 2.2464228450797195], [-3.8127432416380702, -7.537319910701056]])+(1.-msk_ref)*numpy.array([[1.9661219101964962, -2.0873252139880392], [3.2627887829478897, 6.4501282408829175]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.23139230892018059, -0.35373658752423598, 0.41577323943521982, -0.34676314268519626, 1.6540618824743261], [-0.025056034179515407, 1.3150915412112161, -1.9671428280865166, -3.3841268133684208, 4.0614598146770646], [3.6407732982018342, -2.8236187454733699, 4.4748086266914537, -2.3377931051604097, -1.5122074149434859], [-2.733505205100486, -4.8603293706626447, -1.5702106723595466, 3.5660551918052459, -1.0535245394911961]], [[4.9856662286044777, 0.038966738101597898, 2.1096027478150923, -1.1542771230421369, 4.2988121129821106], [2.1737028738671462, 0.097664883908748834, -0.25731015805749013, -0.67760017642728076, 2.0242598083883747], [4.0842723509441097, -0.4752304252885029, -4.2570115478642911, -0.058154987773301414, -4.8534510339928971], [-3.3623236807822723, -4.6805224623079011, -3.9820200317603338, -4.8487288492092686, 0.84141915498740971]]])+(1.-msk_arg0)*numpy.array([[[2.0954427446715682, 2.0242314862931359, 3.0505519617026895, -2.155888100911616, -0.20304992559410184], [4.7002042283297154, -2.8930126955819615, 3.6539078142328076, -3.9565763544759291, -2.9816898220075974], [-4.5132692731279072, -2.4430751214420976, -3.4521882367786194, 2.8881896219503442, -1.2179279360213902], [4.0868574610460424, -3.9518155826601999, -3.8125616389839534, 1.2560541564278012, -3.1329271292029262]], [[1.3668889330718699, -0.47682412212070702, 2.5196963886143013, -4.6705872746519663, -0.95900104058304958], [-3.652796265209842, 3.1336295780773273, 2.5980421974770689, 1.0373523950838957, 3.1983442483312725], [4.7627110953582612, -3.2355769274160426, 0.68480981830262433, -4.5636069674222499, 0.91291059881925118], [-0.921605739615595, 2.3027746107541844, -2.8433643728687197, 2.5716743863907441, -0.72884382699185846]]])
      arg1=numpy.array([-3.3520057362534672, 0.57826298699862111])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.069031000280687912, 0.10552982761885335, -0.12403715033612354, 0.10344944787378944, -0.49345437109038753], [0.0074749377390745482, -0.39232974066479143, 0.58685544801161049, 1.0095826438384485, -1.2116506158538241], [-1.0861476932527931, 0.84236691928496676, -1.3349644895575095, 0.69743111709986461, 0.45113509162239035], [0.81548345085940144, 1.4499764478610437, 0.4684391364182357, -1.0638571268648906, 0.3142967591304659]], [[8.6217972457164489, 0.06738584169782047, 3.6481718443793856, -1.9961110238668782, 7.4340087635461289], [3.7590212805239247, 0.16889354170091769, -0.44497082442197483, -1.1717854880255314, 3.5005868504483786], [7.0630015110302207, -0.82182404195556114, -7.3617223366821545, -0.10056840759451909, -8.3931552651915968], [-5.8145234199301949, -8.0941069505440471, -6.8861748396319706, -8.3849891108815342, 1.4550804286379411]]])+(1.-msk_ref)*numpy.array([[[-0.62513101395036452, -0.60388664148159155, -0.91006764359308279, 0.64316360726794264, 0.060575649796187549], [-1.4022064990804963, 0.86306913627644277, -1.090066098251006, 1.1803608543039636, 0.8895240809880679], [1.3464384097899496, 0.72883977942493616, 1.0298873296789539, -0.86163027429018368, 0.36334303454464451], [-1.2192274663628464, 1.1789405787464848, 1.1373971105566332, -0.37471718584577701, 0.93464253217674831]], [[2.3637842362460062, -0.82458004894206383, 4.3573537391565917, -8.076925861871052, -1.6584167794667037], [-6.316842591238772, 5.4190388258150781, 4.4928384764201832, 1.7939111068963667, 5.5309510036804399], [8.2362371489109645, -5.5953381076831015, 1.1842532441113289, -7.8919230004827057, 1.5787117960939596], [-1.5937484506816491, 3.9822272262424354, -4.917078278910318, 4.4472401730890594, -1.2604020028582539]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.9954535212533342, 1.4404191824763632])+(1.-msk_arg0)*numpy.array([0.27751962268729713, 0.81634105419514214])
      arg1=numpy.array([[[1.5044242900626914, 3.0354430811501558, -3.8856877653140409, 2.2040453731678404, -4.3613931516204465], [-2.8076737112164416, 2.4067750090090234, 3.1272388711008823, -1.896911429334962, -1.892481766548241], [-3.4303461124026722, 3.2585201775078794, -4.0385681213272289, 0.7984465020981526, -0.08718423786933549], [3.764962532448255, -2.5220489240600052, -1.4754652013872658, 0.11766568008865441, -0.50627552140950627]], [[1.5026477215691845, 2.5338925666760197, -0.91651665117954728, -0.83859966772007244, -0.64996063260297099], [2.14141791331358, -0.45565047270025172, 1.270033152401755, 1.8100729031272351, 0.8349300676181759], [1.7647910042141932, -4.9394589486431908, 2.174955898544864, 3.2813693521546199, 4.2401057182678681], [-3.0624450010827218, 2.379717227897256, 3.3015660926392716, -4.4530125004310159, -0.97144932065141631]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.3263901244044531, -0.65738459523254822, 0.51353933763436643, -0.90535954728794876, 0.45752663240458769], [0.7107141806690892, -0.82909848813618492, -0.63808797584780419, 1.0519487048232505, 1.0544109626445208], [0.58170617653962764, -0.6123802869250482, 0.49409926025899281, -2.499169970698969, 22.887778456513601], [-0.53000620963835832, 0.79120333559629941, 1.3524233030891977, -16.958670699475608, 3.9414378868207036]], [[0.9585874066159451, 0.56846103162373474, -1.5716235822039446, -1.7176481674414166, -2.2161637339599376], [0.67264739569096643, -3.1612371077774313, 1.1341587262918231, 0.79577965063604517, 1.7251974007661264], [0.81619816682924295, -0.29161476944191, 0.66227512173468139, 0.43896892665574266, 0.33971303504781269], [-0.47034940446835966, 0.60529005950389037, 0.436283612703598, -0.32347072511854441, -1.4827527816998975]]])+(1.-msk_ref)*numpy.array([[[0.18446898559164615, 0.091426396498972579, -0.071420978588295811, 0.12591375207871627, -0.063630957595324], [-0.09884326001936318, 0.11530767173852463, 0.088742700550278678, -0.14630078051909504, -0.14664322139994732], [-0.080901347442435684, 0.085167378923381262, -0.06871733108121833, 0.34757447362851845, -3.1831398595606299], [0.073711124691281729, -0.11003736685668446, -0.18808957502106244, 2.3585434808025743, -0.54815927484438354]], [[0.54326842045363355, 0.32216877105647174, -0.89069964320290285, -0.97345740240340717, -1.255985383185207], [0.38121519817304372, -1.7915948805174831, 0.64277145258087365, 0.45099899168965085, 0.97773584382215017], [0.46257095160037576, -0.16526932659686969, 0.37533683084852815, 0.24878060546860245, 0.19252846707997337], [-0.26656513142489946, 0.34304120028431695, 0.24725873457906736, -0.18332332418023234, -0.84033313611021465]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.4058261166389094, -4.7396166458070113], [-1.0280276353381899, 1.0924879149581255]], [[4.2380176920849273, 0.66598837440873204], [1.9436683844766991, -2.8365803710437554]], [[1.9729550000829335, -3.6860914362477848], [2.7589169894291228, 4.6221891285818693]], [[0.94070724270672024, -1.7099583630009207], [2.4729715634225515, -1.32535677405895]], [[3.9970871871926299, 4.9109160945169155], [-3.3506188184250227, 4.2351575924174618]], [[-1.2766452182196519, 0.58079390624070992], [-0.55706974057595282, -2.992233784296412]]], [[[-4.5773146763444537, -0.16303279598927034], [-3.1460056933971101, 1.1931176730804633]], [[4.1517981787002594, 3.7435017397333379], [-1.9124907273124769, 2.8897879836961256]], [[-1.5539229964490242, -2.4251073229152964], [1.9508571577279703, -2.1274798703123832]], [[-2.9724026054166375, -3.519405981120618], [0.69172648804422998, 2.698500197717415]], [[4.1320655559335755, -3.9406236397393468], [-3.6340132986640414, -3.7794140196136627]], [[0.030272376924259881, -4.936590150602294], [-3.3301810209200697, -0.8969993099186544]]]])+(1.-msk_arg0)*numpy.array([[[[-2.9164085950638197, 0.42036327146582941], [0.72565539540162671, 2.4689462210566564]], [[3.9208248670288945, -3.9896161737767457], [2.8989289016123312, -3.1757487047059172]], [[1.7817699855776423, -2.0439148510883998], [1.3355257880937197, 4.1396724065165191]], [[3.0482255304121875, -1.5211038938457055], [-0.7745644349835592, 1.5891895623237264]], [[0.52001071136876398, -0.93291527925536766], [-0.049284074122981636, 1.4242391446291975]], [[3.3446705232703344, 2.2084595007159011], [3.3563360328281959, -3.3732511729922199]]], [[[3.0710434896923484, -3.9756246573674856], [2.7428508327769876, -4.9714030033892156]], [[-1.3325319405828862, -0.14820041927572447], [-2.0891502847960264, 4.7793219217035716]], [[-1.7168835774786451, 1.8162057067070716], [3.4938601894935974, -0.97730331547223326]], [[0.090756876689716215, -0.48129729803209464], [-0.23294558154530964, 0.49273769127391365]], [[-3.8782760129838412, 1.1114707723446564], [3.4315427299359662, -2.0741856803986449]], [[-1.8984238665000932, 0.15900088736468732], [2.9307187038421745, -2.5191694843190926]]]])
      arg1=numpy.array([1.8479158696791949, 2.6809080013633082])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.76076305188231652, -2.5648443868983204], [-0.55631733684751528, 0.59120002857477782]], [[2.2934040242972009, 0.3603997267063625], [1.0518164903330405, -1.5350159699295165]], [[1.0676649475527509, -1.9947290332474412], [1.4929884172205747, 2.501298465164596]], [[0.50906389091730164, -0.92534427083943815], [1.33824899931828, -0.71721705290026916]], [[2.1630244389245594, 2.6575431138915806], [-1.8131879667263764, 2.2918562808558494]], [[-0.69085678583478061, 0.31429672517588042], [-0.30145838872668063, -1.6192478420653831]]], [[[-1.7073747678088078, -0.060812529152945244], [-1.1734851370495698, 0.4450423783560396]], [[1.5486537309706141, 1.3963559129331087], [-0.71337424721024667, 1.077913894183089]], [[-0.57962563268072453, -0.90458431310663001], [0.72768523080087455, -0.7935669068951664]], [[-1.1087298049411234, -1.3127664132192947], [0.25801947985252383, 1.0065620291129576]], [[1.5412933057875606, -1.4698839489215751], [-1.3555158538883303, -1.4097514788615413]], [[0.01129183728381042, -1.8413873762515967], [-1.2421839985656316, -0.33458787450464844]]]])+(1.-msk_ref)*numpy.array([[[[-1.578215027489384, 0.22747965876758558], [0.39268854567908612, 1.3360706845843988]], [[2.1217550708677906, -2.1589814986920146], [1.5687558882838082, -1.7185569737312982]], [[0.9642051431091202, -1.1060648834858651], [0.72272001664533136, 2.2401844556025061]], [[1.6495477853877465, -0.82314564142456059], [-0.41915568110686041, 0.85999021297415223]], [[0.28140388851092002, -0.50484726851624762], [-0.026670085436052635, 0.77072726523878532]], [[1.8099690457504334, 1.1951082497599299], [1.8162818383127304, -1.8254354694068562]]], [[[1.1455236390546213, -1.4829396067846348], [1.0231051686153274, -1.8543728471328125]], [[-0.49704500859606543, -0.055279934708822864], [-0.77926966674486464, 1.7827250764566214]], [[-0.64041122507955039, 0.67745916897688618], [1.3032376298317148, -0.36454190706105927]], [[0.033853036599377558, -0.17952771888753477], [-0.086890554031265166, 0.18379507652755869]], [[-1.4466277884252805, 0.41458743521950248], [1.2799927219400822, -0.77368775032334947]], [[-0.7081271962837582, 0.059308595178883955], [1.0931813782314914, -0.93967024718417513]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.4307601376124364, 4.8596081507273716])+(1.-msk_arg0)*numpy.array([-1.7928769730983318, 3.9709366076287154])
      arg1=numpy.array([[[[0.2202373494502341, 1.7506813765209825], [-1.7989258123504435, 4.7535055812704314]], [[1.2531994465019247, 0.85627298500583926], [3.0394233704637514, 4.9992688342689]], [[3.6464985006849009, -4.7049888828261626], [3.1416539364171463, 1.5294167044656088]], [[-2.1394198244860085, -4.5644717885353714], [-4.3906652730131048, 4.017163391353078]], [[-1.3512504870885564, 4.6124849486520247], [1.6589085326086073, -3.7262584782740173]], [[-1.648593350604266, 2.0786639495039285], [4.0533627476956422, 3.9665311830438483]]], [[[-3.5651631222927427, -2.6643097479245936], [-3.3494719863172637, 1.8778731412462584]], [[-2.8773863212582498, 1.789399892600235], [-0.93406706344662993, -3.2802704699637895]], [[2.1366190102459477, 1.7054822590166685], [-2.0497639066644888, -1.7670468515444449]], [[-2.3504966149627915, -4.173396738379588], [-4.6855166219786772, 0.05025323716995711]], [[2.829736867231154, -0.86827010438618402], [-0.29702765270060016, 1.2423113494897917]], [[-4.1637676880528964, 4.5111790790011828], [0.082278331531984605, 2.9366908697492793]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.4964464074052888, 0.81725901514740207], [-0.79534137972206387, 0.30099052439316765]], [[1.1416858997234156, 1.6709158909207902], [0.47073407130976058, 0.28619387855377709]], [[0.39236548084243139, -0.30409426530951045], [0.45541621278762678, 0.93549399155566049]], [[-0.66876081133639753, -0.3134557959600256], [-0.32586408861693383, 0.35616179832071049]], [[-1.0588415332935004, 0.31019291196398779], [0.86247078093123608, -0.38396695933856867]], [[-0.8678672257703901, 0.688307572733864], [0.35298102505773288, 0.36070815319154897]]], [[[-1.3630815713145199, -1.8239651581475618], [-1.4508579771913539, 2.5878255799016712]], [[-1.688896661120679, 2.715775367386279], [-5.2026330237957659, -1.4814656886451856]], [[2.2744383193370439, 2.8494041055163368], [-2.3708136019602604, -2.7501297696096443]], [[-2.067481450427147, -1.1644251566205566], [-1.0371552472852346, 96.702390221989333]], [[1.7173357024826161, -5.596885262061198], [-16.360793705715309, 3.9117473672949843]], [[-1.1671179841927903, 1.0772368078554164], [59.063037135582455, 1.6547904993290159]]]])+(1.-msk_ref)*numpy.array([[[[-8.1406581471026058, -1.0241023850160569], [0.99663752712280651, -0.37716942632034606]], [[-1.4306397741419512, -2.0938147115386405], [-0.58987404996651616, -0.35862783789672409]], [[-0.49167083786311333, 0.38105870550355009], [-0.5706793330467812, -1.1722619269578189]], [[0.83802017377728422, 0.39278958358369492], [0.40833834091569582, -0.44630421977793822]], [[1.3268279939430894, -0.38870088315893386], [-1.0807569783724371, 0.48114670078624894]], [[1.0875192311319106, -0.86251410360303815], [-0.44231841182178716, -0.4520012298812961]]], [[[-1.1138162466672834, -1.4904185261199225], [-1.1855410715032582, 2.1145925783854529]], [[-1.3800498661897675, 2.2191443198638057], [-4.2512328750531987, -1.2105515822518593]], [[1.8585141237564942, 2.3283365081254157], [-1.9372653575945171, -2.2472163678954034]], [[-1.6894032445528859, -0.95148792615640898], [-0.84749173420962121, 79.018523606727172]], [[1.4032882893151137, -4.5733886121023763], [-13.368912192264354, 3.1964101505307427]], [[-0.95368831912081176, 0.88024362103308873], [48.262240296949464, 1.3521806631175093]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-3.8846190774939773, -2.9165408643585291, -2.3765628901206384, -4.9685585047397005, -0.62889841264881507], [0.93974736680596749, 3.4360598181767603, 4.816789596546263, 3.3376629976688204, -2.3893472530929163], [-2.2963331666506805, -3.2295090359219869, 2.3821315968568504, 0.54158157212409819, 2.1033849968697762], [3.8644393495371308, -0.86286188108164907, 2.018759371580944, -1.6318075830082468, 2.4422372940364356]])+(1.-msk_arg0)*numpy.array([[-1.2322200561770211, -2.8886616522419981, -2.8867753933120985, -4.5015147092275196, -1.7944036347080008], [-0.33277434480717538, -3.662499257658439, 1.4077913994797786, 4.4834001417022193, 2.095196954557319], [-3.4056425836717108, 3.0126406992430645, 1.9555283027789336, 3.163153072657849, 2.4928433677777697], [2.8417258611448233, -3.3738661842349127, 0.42529050480593078, 4.082489191191506, 4.6595412919086172]])
      arg1=numpy.array([[-4.6069184802085985, 3.8143812717093866, -3.3571545414678692, 1.8042907389511811, -3.5495138236474864], [-2.8721697163596174, -3.4965665289341596, -1.5665463541693905, -0.4006662002486312, 4.1782208285864435], [-4.1979800803429637, 1.0604295228701401, 3.3999202172204406, -1.4849619825219227, -1.2876118864339734], [-1.852780671987718, 0.86249557185180148, 4.2513553019615884, 3.6106478655767216, -3.8078926829030593]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.84321419929229657, -0.76461702609280668, 0.70790988641277119, -2.7537460551551027, 0.17717874725799998], [-0.32719075110821344, -0.98269539267830164, -3.0747826795717175, -8.3302834019881189, -0.57185758032355349], [0.54700906690893025, -3.0454725809415923, 0.70064338121567171, -0.36471073232752121, -1.6335551256016105], [-2.0857511134284694, -1.0004247085339362, 0.47485077773892065, -0.45194315362780507, -0.64136190208347044]])+(1.-msk_ref)*numpy.array([[0.26747164324063033, -0.75730805246625643, 0.85988754990406135, -2.4948943161146038, 0.50553504616698985], [0.1158616577954022, 1.0474559049144876, -0.89865926771519855, -11.189863629425368, 0.5014567301523305], [0.81125744250636822, 2.8409626799988588, 0.5751688798090826, -2.1301239424903264, -1.9360207792750899], [-1.5337626865980503, -3.9117489925091791, 0.10003645298940327, 1.1306805158468196, -1.2236535217574143]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.0178073126852656, 1.6357990393059518], [-0.82138217623232812, 2.6924028217804441], [0.32643854630143387, -4.6854313250089028], [-2.7523756821474188, -2.7842826803231526], [3.0317868602237823, -1.9969290883364064]], [[3.5456788318130386, 1.580060872259951], [2.3153014033500998, -2.249634629588213], [4.3635149586346635, -3.1191256330390082], [0.63222150434745661, 1.4310272446542216], [-2.0545241574613216, 4.4972680866201991]], [[-2.9710890855716565, 3.510814389473774], [-2.7523373861811038, -0.11350509219963367], [1.1876017952012488, -2.0204850951419617], [-4.1286459217874327, 1.5043758705176984], [4.8775852704968141, -2.2026133953043701]], [[-4.5201650136146609, 3.0354556671615853], [-0.49282328062240133, -4.9037096824390147], [0.69000930600704802, -3.5443852214437221], [-3.908200308745001, -3.8296044260659965], [-2.6236334961668017, 2.2050457486543786]]])+(1.-msk_arg0)*numpy.array([[[2.551723708067958, 1.9450895604651084], [-4.9891862159622473, -3.16626827614646], [0.5406039714861226, -2.2135182779393805], [-1.3109346036817948, 1.0372513595238395], [2.4573888145883416, -1.9721755444335365]], [[-3.5642630152592969, -4.876844890461058], [3.831222213480757, 2.6379848333150804], [-0.87993608618819863, 1.0901059135730238], [2.32612851445157, -3.7108734682664379], [-2.429516456319365, 1.4943186478223494]], [[-3.3245221554413029, 1.6087201232150896], [-2.0212811075770363, -3.0441661522848085], [-3.6392472411489574, 3.1283143333578671], [-3.4728508081277587, -2.9463012129151798], [-4.545091766463627, -0.60905442839302193]], [[2.092875298844854, -2.7005430024935171], [-0.068914879608915847, -4.8699389483937736], [4.2022477963236184, -0.86125247933553251], [-0.29668917988153609, -1.4282058626433116], [-0.082336018574755698, 4.0903112630086067]]])
      arg1=numpy.array([[0.44513757052876546, -1.3271332561940063, 3.3228681422062927, -4.9759768046691688, 3.2007941063568399], [3.2119161586958462, -3.2517125914272449, 4.5614685330155709, -2.2703047472385576, -4.3189766065423694], [0.48481726072205156, 2.498899302256441, 2.8645816029724713, 2.1134205342453205, 1.9905395219573254], [1.31002602830476, -2.0219479780712168, 0.79476567496770922, -3.841466314307771, 0.80411696502340568]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.7794936048658023, 3.6748168377763206], [0.61891462096874372, -2.0287358554345927], [0.09823999398444011, -1.4100563502642949], [0.55313273959893638, 0.55954494757904472], [0.94719833875056003, -0.62388551777525025]], [[1.1039138808818447, 0.49193714723285697], [-0.71202522924508083, 0.69183070961409943], [0.95660310425291017, -0.68379856409465689], [-0.27847429078252484, -0.6303238569160482], [0.47569698672346039, -1.0412809552632802]], [[-6.1282658978492899, 7.2415210305116249], [-1.1014198866260096, -0.045422035252537601], [0.41458124075394392, -0.70533340472667228], [-1.9535373366956177, 0.7118204096823989], [2.4503835350632053, -1.1065409005989038]], [[-3.4504390874309445, 2.3170956924341564], [0.243736874522616, 2.4252402809674547], [0.86819213227229852, -4.4596606686464257], [1.0173720108357258, 0.99691214570915432], [-3.2627510801123751, 2.7421952832324532]]])+(1.-msk_ref)*numpy.array([[[5.732438412324627, 4.3696369150655947], [3.7593709544061831, 2.3857952932524515], [0.1626919722210754, -0.66614688973775094], [0.26345271594748787, -0.2084518076029902], [0.7677434826900984, -0.61615195445303939]], [[-1.1096998922619812, -1.5183599600685818], [-1.1782167414123008, -0.81126014650551059], [-0.19290631510867312, 0.23898135122119513], [-1.0245886669095472, 1.6345265862567961], [0.56252132800143961, -0.34598905804647356]], [[-6.8572685520519663, 3.3181989453493856], [-0.80886857095516895, -1.2182028101476541], [-1.2704288952259708, 1.0920667542204872], [-1.6432369951245298, -1.3940913155588657], [-2.2833466586960176, -0.30597454693797294]], [[1.5975829896701677, -2.0614422493484015], [0.034083408849447924, 2.4085382023722102], [5.2874047391318868, -1.0836558578986499], [0.077233315511969866, 0.37178664233599379], [-0.10239308726978434, 5.0867118105007876]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-3.4955630504862234, 2.6296443455256489, 2.7540735601986661, 1.3400563987746681, 1.1272279023085083], [3.9557457490542678, 2.0849747741308455, 4.6312807760654806, 1.9742823645181922, 4.321472668330264], [-1.7335206649843227, 0.18085988298441791, -3.758891431340424, -1.0236575406790758, -3.2916425062632495], [-2.8414960754328691, 3.2814888582781307, 4.5566226950827478, 4.5857057245595954, 2.01749768755045]])+(1.-msk_arg0)*numpy.array([[-1.2113104856256296, -4.4104297479553436, -0.26017548458372275, -0.13953901055787377, 3.4703275122866213], [-1.7545693577908383, -1.4423700958697596, 1.3169674169273886, 2.328619891274073, -1.4901755902988922], [3.3567905816276173, -1.7732660951500878, 2.7995454848954493, -1.2799905299359038, 0.71819237018233473], [-4.0184953172994966, -0.42337778196133335, -4.8152200850726707, 1.9070505717586252, -2.3957193452019787]])
      arg1=numpy.array([[[-4.3507437968298053, 4.5526223957598972], [-2.6679253552941375, 1.8111188338937279], [-2.501499103361462, -4.1065057455397449], [0.58348288399203696, -3.9043173320337377], [1.1005390364038794, 1.0574025317556952]], [[4.7231207522348342, 1.4310551190232719], [4.9813489629976999, 3.2511714565765022], [3.0576226823106101, 2.5443177358360378], [-1.8902880569065141, 2.6725028067239762], [0.93672060595000062, 2.8752674080853051]], [[-2.8242434809658779, -1.4855835065679477], [-4.1229601669328311, -4.7537142628792193], [2.6769778362903054, -0.64544770260402284], [-2.1001218412469536, -0.55135998974142897], [0.22292897346515606, -1.0845561117293356]], [[-0.38951602675209163, 0.25154102219838048], [4.6672571590865601, 4.0755187599261493], [0.49891510518739057, -4.5085857130445728], [-3.2166355524737855, -0.51175093462110244], [4.7010547666449547, -3.0008478465556845]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.80344033427876993, -0.76781308586932884], [-0.98565139399701529, 1.4519446743713507], [-1.1009692374056022, -0.67066107558475607], [2.2966507425313893, -0.34322425274705837], [1.0242507217116419, 1.0660348055313205]], [[0.83752797283078817, 2.7642162041628104], [0.41855625647156819, 0.64129954448060178], [1.5146671964657441, 1.8202446615983232], [-1.0444346602650265, 0.73873911733635145], [4.6134062183328668, 1.502981133573247]], [[0.61380000579534555, 1.1668954705812324], [-0.043866512326497666, -0.038046014754550071], [-1.4041548571613913, 5.8236963524316314], [0.48742769137207587, 1.8566046860947236], [-14.765431586117877, 3.0350135605383222]], [[7.2949400801968691, -11.296352581376938], [0.70308721941525898, 0.80517083875172568], [9.133062213803484, -1.0106545566826401], [-1.4256217870354984, -8.9608155341324913], [0.42915851605581196, -0.67230922416346273]]])+(1.-msk_ref)*numpy.array([[[0.27841457511431905, -0.26606873584635271], [1.6531308641013669, -2.4351962253483679], [0.10400782644059492, 0.063356902609063848], [-0.23914842129247121, 0.035739669368828851], [3.153297972624634, 3.2819360726558333]], [[-0.37148517893823285, -1.2260669309427945], [-0.2895541160805894, -0.44364627185444772], [0.4307161326825884, 0.51761122377848223], [-1.2318862634538863, 0.87132551756926169], [-1.5908431829441718, -0.51827373902979978]], [[-1.1885627440590254, -2.2595771740779518], [0.4300953740402646, 0.37302748905148958], [1.0457858286847064, -4.3373699737420068], [0.60948393792995625, 2.3215150786261809], [3.2216196890826696, -0.66219936655667333]], [[10.316636649862618, -15.975506826597325], [-0.090712332217878819, -0.10388316356787061], [-9.6513816378922677, 1.0680112105090784], [-0.59287119745101025, -3.7265209357567564], [-0.50961315366928894, 0.79834748967753877]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.062137760597399172, 4.3582895699915447, -2.7309081115149416, 3.7067156687697729, -3.8011744429158991], [4.4202763210878615, -3.6312937123137079, -2.5523402306460441, -0.76658527495366169, -3.6029259253606161], [-1.615198810181091, 4.2715179721596215, -4.5683037529916648, 4.4980521096070838, 2.9714676247560874], [4.1633330697518538, 4.7609585210100231, 1.6854513489970602, -0.30426662078571987, -1.6575732463662654]], [[-2.003785789789764, -1.4716720416209581, -1.5717227855774905, -0.23723474693414293, -1.7300813272875679], [3.7293627985589044, -1.7435839339782788, -4.4036408848613364, 1.5935138150229218, -4.0557302470367169], [2.0633818447104799, -4.1435202310200347, 4.1680368483557277, -0.47494203316305317, 1.6274724356913461], [-1.9578305414247819, 0.32614214699647537, 4.8719270937515109, -0.027824538242119168, 2.7584156752559039]], [[4.53372755081854, 2.6711862975317278, 4.9300933410819123, 2.8520870187899892, -0.44131954328436329], [-1.6033847218580091, 2.5798466557229274, -3.2465256625915839, -3.2529895246990801, -2.9419619470761482], [2.9860630995257607, -4.8500921395417649, 1.9492413894888152, -1.4998519212531312, -0.91105738937787972], [-2.896301329559019, 0.99727396158285941, -1.1080396778022261, -4.3308532839577341, -0.72144282852946251]], [[0.39835158501634194, -4.5476113939174825, 0.61682457012751879, 2.685185625413701, -4.3312975563849037], [1.5623952613561247, -2.784582888422257, -2.6425695290283002, -2.5837578745829601, -2.4654276192907387], [1.6220354001705006, 4.1409213927596618, -1.9826366273296281, 4.582028085088842, 2.684802646522467], [-2.7927478989742696, 4.3821013955083323, 2.0828960870533573, 3.3227844928358312, -2.6287823314010828]], [[0.34397275221204104, -2.2674804171317495, -1.4415388064367027, 3.6042824705664867, -0.7635789095082206], [1.937368274686718, 3.7407805302943249, -3.6591222201021978, 1.6329235598941967, 4.0848180578315549], [2.6110642746919801, -4.9635465397217935, -2.3792442095462221, 4.9977972543196554, -3.213332542067362], [0.16843349435614297, -2.3719082247842662, 3.0298696381556933, -4.4318979488998593, 2.3252989876905747]]], [[[-4.8368769639150635, 0.805305161861531, -3.0011507582596666, -0.14376183090754857, -1.869773887506867], [3.741682400905356, 2.2709765925011718, 3.9687886595929083, 2.7644758429999161, -2.56284103670842], [-4.6503307034840349, -2.7108359357528498, -4.7988698334915085, -3.7338721310835368, -0.23256589079891477], [4.3104708459754377, 3.8308157289858222, 3.1018549707204848, -0.019459967394485744, 3.1121037871106108]], [[-0.12000341392468172, 3.4296375317282095, 3.0728481706256972, 1.0078429964076987, 4.2561704695077189], [-1.6020743490127618, -3.8130848579168619, -1.2794926987759512, -0.45582575871519371, -0.026307914059838566], [3.7401342332930607, 1.022954676165325, -1.5047662022964281, -2.965045108206751, -4.0556966395335881], [4.4040653651476127, -4.3558988297588694, 2.9274329561317352, -3.4433193110490423, -3.6605161143163301]], [[0.67420269286622592, 0.96759318738833677, -1.6751891458669554, 3.6853769824315297, -0.51014897049478503], [-0.13156388255169738, 0.06276929902596784, -3.5987818279626884, -3.1098796423757991, -2.6482041521806354], [0.41887320803154271, -2.664322028716013, -3.2857358840679494, 1.7890992058651101, 4.8284179439274055], [4.3649234446565792, 4.2949940046407384, -4.2592155698646703, -1.2605824300593236, 0.3035968128303832]], [[-1.9539453222093259, -0.44948739223552003, -0.21826797855054991, 0.94629006448805786, -2.5443087107079609], [4.8598865186921518, 0.29661003677895348, -0.62690370511588522, -0.20933890065733518, 2.9868411857232244], [-4.3033762546528802, 1.0957326374254395, -3.1740140492655069, -1.1326620250460828, 1.4093922505663787], [-0.92957401491327918, -1.8196099431164905, -0.53862770989276942, -3.9603167685222052, -4.059395955170003]], [[-0.20086818932697259, -4.0909313622101342, 0.82528649607482407, -4.9875209369294859, 0.99864916538563264], [-4.6617770234012736, 0.84336042305469761, -2.0222566314981338, -0.96938015307818315, -2.764684265177678], [-0.26332565127917373, -1.3825527168892213, -4.8205570855900879, -0.1827059559139883, -1.7529309503640222], [0.31274602324215017, -2.0826138442204711, -3.7280119060315187, -0.52000394681757545, 0.86476739516086454]]], [[[-0.32806914750132243, 1.3812168879486464, 0.073652273367157406, 2.9560996678600837, 4.4681669287078645], [-3.8655483133516801, 4.4242092680761651, -1.4080889716022558, -1.182122350476237, -1.3597555827748495], [2.5739550727110831, -4.465271066446137, 4.6853803855802241, 0.51837412260552806, 0.034505832019059213], [-3.8197334905344196, -3.3616193811549411, 0.63464146035359548, 1.6139367099027719, 3.1363193764185517]], [[-0.85366949095026889, 1.8545874292993014, 4.8525623780930296, -3.5446521733470115, 4.0548695225200095], [0.4749982237932473, -2.790024023616823, 2.2586853504335078, -2.7766101379773502, -1.2224510023912805], [4.844674656867733, -4.8567988188025124, -2.5483503385201125, 3.0965223689273511, 2.5384233564953131], [2.4098755639365272, -4.1988940168138145, 0.40730103130866802, 2.8494962989706103, -3.7352013284859731]], [[-0.75556504522458745, 2.3455797833889402, -0.60755431087122869, 2.369501245834762, -2.5946368505713147], [3.2928603124029827, -0.028362388745626355, -0.23249172055925627, 0.31680511287523672, -4.8300753558380327], [-4.9738549246952335, -4.2915840827926921, 3.8614211894778077, 0.22424240252053007, -3.3442263230057145], [-0.12913513953608735, -2.8032195844202636, 1.4244984843278576, 4.1321980855607894, 4.6372126899171064]], [[-3.2367116068654935, -0.27573687386611123, 1.2170195893606834, -0.42333943057049872, 4.911808597430591], [-0.91263035503188839, -1.7050099001384531, 4.509160948866958, 0.52622664734981317, 1.1826874266037439], [-1.0572374599303034, 4.9175134547140473, -2.6945557413433905, -1.7204976012372142, -4.697549697746414], [3.6338118362025806, 0.12327293973629594, -2.7790668569439028, 2.518139849014581, -4.1908016605016964]], [[1.9473600756327336, 3.7512810368572875, 1.2543630010822024, -4.4334961340131871, 4.2875635558182186], [0.66197132365369171, -2.0212456410328841, -1.6979846217154559, -4.6364543973761085, -2.5013980026131346], [-3.4434553436684521, 0.25989329180777254, -1.9241243133122179, -0.026087952850351925, 0.23115133252400355], [-3.1731583730417712, -0.47363929911329539, -1.5681105003662141, -2.0432099719954619, -2.1866553981705339]]], [[[2.2822560121919562, 3.7876732339339743, 0.980921502883497, -2.4100537552120969, 0.49113660178187057], [-0.65786450057048995, -0.2397496113823383, 0.77117202121111195, -2.891180052883203, 1.2667129267234358], [0.35613892430869765, 0.13409709554031402, -3.6845784737679086, -3.1834198743780306, -0.5396292304346737], [-1.3405414083379528, 0.93956352294272349, 1.0947342680250287, 1.4097175465393486, -1.6941515093146862]], [[3.0026705355320455, 0.20746672923477227, -0.43796595548172057, 4.5512689711497671, -2.8087966032513667], [-1.1289920082142402, 4.0178100800780143, 4.1836298002450025, 4.9785916358621556, -1.5077578917978531], [0.54502083024492887, -0.29173554011004832, -4.5467452199878302, -0.94884676773743326, 0.35105998448982856], [-1.8957937981898176, -3.4895771603285053, -2.1960106679987823, 0.59273167319405751, -2.2136646297359333]], [[4.8323358304092263, -3.5364087541077471, -2.3946902717664384, -3.2887403516207172, 3.1213106008766474], [3.3344015709765333, -3.1605910657603795, -1.1342081280017293, -2.0228497079277883, 0.13491172020542752], [-1.6822147650017416, -2.731316108090315, 4.6358568401961033, 4.3995434408587837, -2.1720417939209744], [-3.0330275433734046, -4.8408997198209027, -2.787292886061171, 4.5059247283464217, 2.5390455756979389]], [[-2.1738262939240838, 2.2601001238999219, 0.67871720323985141, 3.4052590825546591, -0.22460419967508649], [3.4596191405205872, 0.75123359423683134, 1.4297658513756195, -3.3023778730593834, -2.3638818539302155], [1.6531959432796324, 4.6036365712836584, 0.71707883832139174, 0.59301414973647759, -0.14583518593754086], [-4.5722978807996952, -0.79969522626425782, -3.894256327463137, 2.5585622116893223, -4.909191486478397]], [[-2.6681293619166038, -2.9246994793115677, -2.6292107993453664, 0.48203489596885341, -4.9996180684856304], [4.89120113780438, -0.19598474592941884, 4.5493545352022711, -1.1584435454070166, 0.013177455516632186], [4.9952106248665569, -4.3739801386324508, -2.8531282012623027, 3.3103165093959976, 4.4913328175090861], [4.4994123417638203, -2.9471301870830358, -1.1700262623473821, 0.40872570563259814, -2.9395529500333675]]]])+(1.-msk_arg0)*numpy.array([[[[-4.6517259187346376, 4.4515811533638878, 0.59667479792810507, -4.4410397995643178, -4.3962246620158352], [1.2591288315914273, -1.3039518870418454, -2.5596224137840173, -3.0279928366246622, -1.8567081643612338], [0.9810609508444692, -3.9688841408632216, -4.537191063134669, 3.0006294961275888, -2.3764788695506556], [-1.6114059646545176, 4.0956794283145932, 2.2844250128219645, 2.5006138820255899, 3.6216154343103319]], [[-2.8029925646938834, -1.5054082448286277, 2.2792774647347276, 4.2743309298536118, -3.5487849851190125], [4.6015698477218923, -0.65917661833643848, 0.078645544344080776, -1.4210515772124319, 0.1396228781282991], [-4.4416168633016611, 4.4036134606635962, 1.2811310302144836, -1.828988598198682, -2.3520505966049479], [3.7013327021658444, -4.8220512565236522, 3.6422736913539993, 3.4758752350280613, 2.5918284663547837]], [[-3.5193662585898289, 2.2497954435552661, -2.6911755037696938, 4.813891542100496, -0.91921292838023216], [-1.4090247460444525, -2.2607117694676706, 2.8471418536504487, -3.1462146082726115, -0.2029444894506387], [1.2454647078603713, 0.77380390656273246, -0.82512396575370595, 0.71454419044822881, 4.7364442182159472], [-1.7803144800521875, -2.6803591051683915, -0.74717481019757503, -3.376506750079832, -1.3808957954461021]], [[-1.6047410867281062, -4.49540456316464, -0.66326577139739307, -2.3699386071822293, -0.60158008078386693], [0.3770418283157051, -1.7589370263946682, -3.3527636704436414, -4.3219394482019249, 2.0250757528660781], [-1.1745479359689615, 3.9002715138899688, -4.7410053640203698, 2.9952635190315942, 1.5840833198986317], [0.47113829360861281, -2.2188860804354062, 1.7230034639375633, -1.5681801158252764, -0.62094632828014795]], [[-0.85220473063866464, -0.46150859024599011, -3.7119576008808473, -1.8547553869970224, 0.14733200652812251], [-1.3618042373707717, -2.2796688362803899, -1.405320191587911, 2.0186655366409987, 4.728925872787368], [-0.39887828381538704, -4.7373790741862578, -2.266469108229451, -4.7539359555972061, -1.1128189248139564], [4.8259286079781525, 4.1332349019761665, 4.4080701420000317, -2.0404767374973867, -2.4203778089691941]]], [[[1.8292483771795922, 3.9976732946637803, 4.1397252909735105, 4.2496070142119926, -0.96780411411913647], [-2.4468969260894999, 0.66599700676934415, -3.739816463577994, -0.42465860932009658, 4.710045539461289], [-0.88601694383878549, -4.6927968418624797, -1.6971180911720984, -3.2426624066536167, 0.5620193708699226], [-2.1297004817565544, 4.7355179380234294, -0.65604062107739036, 3.3766196141802833, -4.6130912234222379]], [[3.1843154603534582, -4.9024243879361817, -1.2058444935990842, 1.1671585756180249, 3.7012613353203623], [2.545336149179807, 1.1872061333747155, 0.78946526642073422, 3.7363073665850735, -0.73527953714115224], [-1.9899924549157264, -1.4907004721447672, 0.16570745442974566, -3.6559181913151075, 1.1951113367630706], [4.117838780645215, -4.2389667794093988, 1.1204773859789929, 3.4260314969423575, -0.83204680795920716]], [[-1.3911768815529388, -0.35971279194451888, -4.3271997973335052, 0.23442497051994859, -0.67795583370581358], [-4.3478603106908338, 3.7691925679650211, 1.9535901600262964, -2.2901755612463215, 3.3430253114029291], [1.6225102903678259, 2.3648623824239587, 4.3486641473407133, -2.5434602795395413, -4.8956733235046643], [2.5165241638810478, -0.81172436872043541, 0.91261483759001116, -3.4399492356359396, 4.6538543248719062]], [[0.0065724007883138924, 1.4138101243367824, 2.271710989190705, -0.65203883632541881, 1.1109074393383196], [-0.34953758115237044, 4.30124272553239, -3.848074428364078, -1.8931790463482159, 4.6611504099894869], [-0.92451971268632605, -4.2898791047041538, -0.73500108738880332, 3.3811091240825846, -3.2457521866009609], [2.8270701621021912, 1.8075389138494398, -0.51092764850278716, -0.024940730140276202, -2.7908361455730724]], [[0.34167685577879148, -0.97475564288155514, -3.6045693392798284, 4.1069256841882922, 0.76809369242338921], [-3.6612433825613455, 4.1237189683998281, 2.8518230152698916, 3.6597652958488478, 2.4439872291262175], [-2.9331589300436689, 4.0831951747897897, -3.1655504410971789, 1.7848280034332156, -1.5299138939635935], [-1.1732108352239679, -4.9321806545048998, 2.8999918712953114, 3.4266195092075407, 1.7351323985833194]]], [[[-3.0624501648593383, 0.64258860420901787, 4.3681751118918193, 2.9393724550456666, 3.1820940504603872], [0.60599512693305524, -0.20908324957493196, -2.3806614664595163, 1.7777181364536752, -2.9530930598963137], [4.2241576163802659, -1.0769455300829422, -2.5422242220418467, 0.36304429085454437, 0.17847208187547103], [0.77868282979380066, 3.0349442210545448, 2.0053841757909865, 2.3136012088678894, 4.6506300031219077]], [[-3.504620556207303, 3.87910588197016, -4.1858119601940524, 0.45527017716460616, -1.2178262972019849], [1.217720644965242, 2.0556157313342318, 2.9690027014333067, 1.9602006211551304, -2.3039823593060351], [1.081733500227636, -0.13042740492895799, 1.1549638125728983, 0.72878824870210757, 4.26003947305081], [-3.4544145328471973, -2.0508217432725848, 2.2243961228373719, -0.47212133631119002, -3.9742525779150251]], [[-1.2934193788003312, 4.771664540932699, 0.74203362931980443, -4.724556886810408, -2.0374495310243699], [-3.2393260528346115, 0.21408575564847343, 0.35103145516353074, 1.1957837649612255, 4.2125618412736632], [0.37788643977965908, 3.9679389661110918, -4.4148712997236501, -0.95506203075509699, 2.6272774932029872], [1.3085473130207603, 2.6187181494032172, 0.28891007150597048, -1.0801504268891291, -3.3078563553949625]], [[0.9318215798941214, -3.654688446821023, 2.8860854467663213, 4.8901171025731838, -3.4286448008352566], [-0.36323848382058355, -1.957492328637731, -0.085975478001488526, 2.7890782634608158, -0.96533937802734204], [4.4370169563326858, 1.6913475857338547, -3.0701993259395008, 4.0301955706503954, -4.9176837247690122], [0.64361556057503755, -4.4175292677787592, -2.6863915141789709, -3.2351359033720497, -0.38150585154995564]], [[-2.0990839604344815, -0.019014209912277202, 3.8065058030166838, 2.0063381740937078, 3.5339881715120232], [0.53433768948042104, 2.7435927907658808, -1.1015017155241544, 2.3960571591493522, -1.7862253882670895], [-4.7100460828591109, -1.9533657673115257, 3.5997892226474129, 0.67509362807649254, -1.379015004946579], [-2.263166753405482, -1.7231249744765265, -1.5012444687411577, -0.69935153403271855, -4.1917957480254353]]], [[[-2.4100221698762834, -1.26257463431936, 4.4433799285397519, 1.6914710506611987, -0.49072288014787446], [-4.1672229390039881, -0.87859172465550195, 0.59549813475882907, -4.4566714910676177, 4.0118903893546403], [0.32923800625510236, 4.372240305862702, -1.2519764604669623, -3.7184369556763111, -3.8863523450607582], [-2.4310446707280233, 3.2830460892432285, 0.64862108452969558, 4.8499747450185637, 3.1343560065745599]], [[3.8233721105621772, -4.0053144760048109, 0.76662815605562606, 4.2255137822471394, 4.9971773859156503], [4.6196112959119304, 4.3627307862920155, 3.6410765765451156, 1.2309255571583178, 2.940538571441035], [4.7538475149458286, -4.9630074625549483, 3.7856969089527563, 1.0448249405534007, -4.6856467353301259], [-4.8085058115147845, 3.2361845632578472, 3.7262166820337086, 0.84139996783794668, -1.6162522158780424]], [[-2.7317965654244283, 1.0922353633098769, -3.0939421886001464, 4.8500064177111142, 1.9785250964933461], [-3.3036891143005862, -3.9727703544398398, 3.1884344822334114, -3.2023186283164504, -4.4744435701566889], [-4.155421318963719, -1.5583615676251772, -2.0637447111206155, -4.7775912499114694, 4.6267372919690182], [-1.7827694341256004, -3.3018847854508171, 1.3278491023486279, -4.1716349500492882, 1.9465508022988605]], [[-2.4986863027904893, 2.1607557132808282, -4.4060514789672114, 3.702466667747613, 1.2062002199658934], [1.5467258795333461, 3.5094370659532483, -2.6485109878371524, -2.2429082819379156, 2.843893291729513], [-4.8304505329207608, -4.0959683611984952, -4.581837293887153, -0.43958518585780126, -0.30118716712809679], [-0.49668158660052608, 1.6111145919773957, 4.5119455167354641, -4.7339536164091625, -0.63810281855421369]], [[-2.7613659351628472, -1.610023046097159, -1.1668699406147578, 0.18566477782133184, 3.6951676953788191], [1.1538892545009194, 1.5671591910811413, -1.1479805537964904, 4.8269624593584766, -4.4417583131004754], [-2.8045394941261548, 2.6023843621819491, -2.9926105892372625, -3.5101744760889875, -1.0815822356143769], [0.18780459485292145, -4.0757701867732568, -1.222809403810956, 3.8964251793537894, 3.8266369749415396]]]])
      arg1=numpy.array([[-0.33827426957936169, -4.5831983754465817, 3.0795132007289894, 0.14748685177417453, 0.49290597204881603], [-0.33096553599374623, 1.0013520272681955, 4.7631922671099201, 1.0056425910504485, -0.98147431242146155], [2.7810056151779969, 3.2994611248936394, -3.6775686686845601, 2.8572692309643708, 2.848126334022691], [-0.063869442110324748, -2.1612578194048861, 4.9006853831711936, -0.9022931208736793, -0.99914737053538616]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.18369047304326877, -12.883893224900032, 8.0730589261511945, -10.957722777375325, 11.236960019579955], [-13.067137286511327, 10.734761815698132, 7.5451799328983427, 2.2661648960380507, 10.65090150025538], [4.7748201841942137, -12.627380667974485, 13.504733181959931, -13.297056602029871, -8.7841964109509636], [-12.307566504921843, -14.074255564664124, -4.9824994111816139, 0.89946723161672992, 4.9000866912740051]], [[0.43720250044697601, 0.32110153675762726, 0.34293143277359089, 0.051761832567639415, 0.37748340472366981], [-0.81370311582804211, 0.38042951475090492, 0.9608226666453753, -0.34768597919736599, 0.8849126559226298], [-0.45020565894868697, 0.90406739826449145, -0.90941663592067379, 0.10362676765366399, -0.35509535096934725], [0.42717560555820649, -0.071160381960271676, -1.0629972116964708, 0.0060709871061184377, -0.60185386913066508]], [[1.4722221517820757, 0.86740537332309486, 1.6009326863462867, 0.9261486582083287, -0.14330821611022584], [-0.52066174662880227, 0.83774495758362721, -1.0542333969612661, -1.0563323852383633, -0.95533344243490181], [0.96965426185505321, -1.5749541643119569, 0.63297062309308705, -0.48704188730156533, -0.29584461244141191], [-0.94050622315026933, 0.32384143095953688, -0.35981001073154306, -1.406343471082548, -0.23427171163243624]], [[2.7009294742170007, -30.834012247278746, 4.1822342989053274, 18.206271224265745, -29.367347016239783], [10.593454552466795, -18.880211048818708, -17.917322779894224, -17.518564153359907, -16.716253616055855], [10.99783052291394, 28.076546098496028, -13.442802551412212, 31.067366548067923, 18.20367452573549], [-18.935571987463696, 29.711810529510899, 14.122588298532518, 22.529360772603205, -17.823841920676159]], [[0.69784659086657397, -4.6002291425010053, -2.9245715981991323, 7.3123124387901077, -1.5491370622561618], [3.9305027420013618, 7.5892375877398548, -7.4235704730715018, 3.3128500210836895, 8.2872155937826744], [5.297286749922737, -10.069966324591858, -4.8269737931083299, 10.139453643756395, -6.5191592804420768], [0.34171526398033131, -4.8120906608722507, 6.1469525831909042, -8.9913658998250821, 4.7175305627262389]]], [[[14.61444300957807, -2.4331994551745342, 9.0678648737504037, 0.43437100021878117, 5.6494519343010907], [-11.305353560970339, -6.8616709159230496, -11.991546635441525, -8.3527604609929913, 7.7435284281588954], [14.050800454256075, 8.1906894855785595, 14.499605885194601, 11.281755122545722, 0.70268914888862999], [-13.023926594148119, -11.574666581169973, -9.3721388887424695, 0.058797564332660458, -9.4031053045034145]], [[-0.11984138510416258, 3.4250068291015086, 3.0686992056218068, 1.0064822050215561, 4.250423780654887], [-1.5999112254093164, -3.8079364240360105, -1.2777651254839477, -0.45521030197416112, -0.026272393068009867], [3.7350842974738674, 1.0215734809625983, -1.5027344643238052, -2.9610416990873212, -4.0502206307985409], [4.3981189883465994, -4.3500174874986444, 2.9234803310061817, -3.4386701352598417, -3.6555736790213955]], [[0.14154429530834375, 0.20313964524791828, -0.35169463081182339, 0.77371997092774136, -0.10710232589546911], [-0.027620947292040371, 0.013177989782061281, -0.75553990394476755, -0.65289819683527639, -0.55597255027192938], [0.087939596921980889, -0.5593563894351461, -0.68981802535163639, 0.37560927746270695, 1.013693689685355], [0.91638615446968053, 0.9017049415153543, -0.89419350112628582, -0.2646507550752692, 0.063738097436615002]], [[-1.9429818700979276, -0.44696534955426453, -0.2170432920134748, 0.94098049636064662, -2.5300327704401342], [4.8326180314377254, 0.29494577836955782, -0.62338619176923493, -0.20816431455898193, 2.9700822263338171], [-4.2792303080140721, 1.0895845573534102, -3.1562048758794874, -1.1263067367333315, 1.4014842480907572], [-0.92435823938432082, -1.8094002375295271, -0.5356055070521063, -3.9380957049416914, -4.0366189651233277]], [[0.20465965006399109, 4.1681491919203886, -0.84086408134177659, 5.0816622236647602, -1.0174990346123252], [4.7497697743101286, -0.85927916032156371, 2.0604274670305815, 0.98767755896388154, 2.816868694562916], [0.26829601951527926, 1.4086489064377368, 4.9115468684014418, 0.186154597834784, 1.7860181648964892], [-0.31864921912276373, 2.1219239442775777, 3.7983794979146106, 0.5298192120124251, -0.88109019687671553]]], [[[-0.11796781197089547, 0.49666094897842999, 0.026484043385307344, 1.0629606972839138, 1.6066731057002492], [-1.3899822036512743, 1.5908667152378244, -0.50632367080356711, -0.42507010558501729, -0.48894384655452017], [0.92554831916308034, -1.6056318052994436, 1.6847791892287634, 0.1863980855617042, 0.012407681534598657], [-1.373508010802754, -1.2087783508268042, 0.22820574575250382, 0.58034284472290532, 1.1277644889680718]], [[-0.25872997396742703, 0.56208797712659375, 1.4707136088016359, -1.0743124526012642, 1.2289490219863468], [0.14396236409924643, -0.84559990798702367, 0.68456189205936413, -0.84153443028271968, -0.37050019870462547], [1.4683229998728671, -1.4719975883816709, -0.77235349714940515, 0.93849336352680013, 0.76934482947640159], [0.73038459091231489, -1.2725999361332585, 0.12344471290650459, 0.86362475298522146, -1.1320640514006299]], [[0.20545232823479748, -0.63780720217739328, 0.16520542934920818, -0.64431189715413673, 0.70553049700072568], [-0.89539057161393953, 0.0077122662554582229, 0.063218865915131067, -0.086145261017941954, 1.3133882167768511], [1.3524845822863576, 1.1669623246838681, -1.0499929538661776, -0.060975721386254893, 0.90935795474946779], [0.035114270097987885, 0.76224806032594228, -0.38734789548807813, -1.1236222781501037, -1.2609452352050312]], [[-1.1327989577562683, -0.096503637416431337, 0.42593801668102566, -0.14816224735938355, 1.7190569737709955], [-0.31940649664430193, -0.59672707131031788, 1.5781365298029857, 0.18417118052687106, 0.41392229118169915], [-0.37001674482508251, 1.7210535855083959, -0.94305279745512038, -0.60214752694359175, -1.6440696756325344], [1.2717778908696382, 0.043143620629229079, -0.97263037967406607, 0.88130996607718037, -1.4667157071114501]], [[0.68373374185346758, 1.3171048601481739, 0.44041691061875804, -1.5566360526400249, 1.505397953946259], [0.23242344124487063, -0.709675556483507, -0.59617601980359625, -1.6278963267853308, -0.87826090181897321], [-1.2090247902749871, 0.091250619294228952, -0.67557547933436879, -0.0091596894908468913, 0.081159086857472931], [-1.1141213559020766, -0.16629855686363731, -0.55057617410931925, -0.717387409254994, -0.76775224892573635]]], [[[-35.733144627280538, -59.303371202011519, -15.35822876281124, 37.73406617595149, -7.6896961293869888], [10.300144777124075, 3.7537451942700129, -12.074193788620066, 45.267031578092215, -19.832847835673622], [-5.5760456415686521, -2.0995501308541522, 57.689222764829537, 49.842612823815763, 8.4489422892178432], [20.988775916069088, -14.710689367221502, -17.140188356961719, -22.071862536457981, 26.525227923367439]], [[-1.3893162160351822, -0.095993512376001192, 0.20264401199590193, -2.1058426858129233, 1.2996120028034341], [0.52237729255508913, -1.8590147107874153, -1.9357384217108291, -2.3035621160797177, 0.69762981457391438], [-0.25217760942329559, 0.13498414557055449, 2.1037495754392692, 0.43902525613473697, -0.16243318188965294], [0.87717151612750877, 1.614604758857219, 1.0160799180374813, -0.27425310755255905, 1.024248291832891]], [[0.98605306249679325, -0.72161513698709745, -0.48864395171943353, -0.67107763393956132, 0.63691307579040557], [0.68039494688370894, -0.64492837606220432, -0.23143867425086417, -0.41276873534346714, 0.027529153507529849], [-0.34326112236839701, -0.55733349410055433, 0.94596091724547271, 0.89774043768789646, -0.44321184162927507], [-0.61889864503212755, -0.98780055060143357, -0.56875572866453561, 0.91944786821444036, 0.51810009767550991]], [[2.4092240577199511, -2.5048402471599194, -0.75221365157107467, -3.7740053689619053, 0.24892598034838725], [-3.8342519304266451, -0.83258264621304123, -1.5845913243705048, 3.6599834318383495, 2.6198602197490963], [-1.8322160559961524, -5.1021519113722311, -0.79472936425255369, -0.65723004644241256, 0.16162728337807833], [5.0674196389443775, 0.88629205716422044, 4.3159548015753204, -2.8356219863583778, 5.4407945410521226]], [[2.6704062289498949, 2.927195292266433, 2.6314544549484449, -0.4824462438514534, 5.0038845278716186], [-4.895375078836933, 0.19615199089639976, -4.5532367590223766, 1.1594321113874049, -0.013188700591356347], [-4.9994733231294077, 4.3777127054727512, 2.8555629383616092, -3.3131413913667087, -4.4951655280866492], [-4.5032519470604635, 2.9496451414407834, 1.1710247125210684, -0.40907449459991602, 2.9420614383023684]]]])+(1.-msk_ref)*numpy.array([[[[13.751344211071626, -13.159680039807206, -1.7638787563418881, 13.128517889009636, 12.99603622670582], [-3.7222128456803194, 3.8547179147361326, 7.566707385007037, 8.95129517355819, 5.4887655708192611], [-2.9001938340282334, 11.732740257775035, 13.412758436450366, -8.8704041837377119, 7.0253019022279366], [4.7636078459596511, -12.10757008922819, -6.7531740314231063, -7.3922674790934018, -10.706151073251149]], [[0.61158002230718689, 0.32846237965467562, -0.49731154491270246, -0.93260875478407057, 0.77430316002266053], [-1.0040084392536295, 0.14382458805794307, -0.017159533125471062, 0.31005674657797594, -0.030464070435243684], [0.96910857865035682, -0.96081668300785972, -0.27952772829512351, 0.3990638083651501, 0.51318978667943138], [-0.80758727835017408, 1.0521148904129198, -0.79470129655889055, -0.75839510976641411, -0.56550649874548331]], [[-1.1428320092138966, 0.73056853369639374, -0.87389640126648349, 1.5631988656391993, -0.29849293328654475], [-0.45754788312350958, -0.73411335562143709, 0.92454283130738546, -1.0216597245070527, -0.065901483845757572], [0.40443558013180203, 0.25127474900239294, -0.26793973981289665, 0.23203154001063553, 1.5380496557360837], [-0.57811555398780157, -0.87038402840224582, -0.24262757179306849, -1.0964417198408298, -0.44841366327613508]], [[-10.880570487633815, -30.480036078387574, -4.4971179696272641, -16.068812769906952, -4.0788726150652419], [2.5564436678939706, -11.926059884225316, -22.732627553656428, -29.303896558992871, 13.73055108645743], [-7.9637467465057714, 26.444876048082545, -32.145274693906892, 20.308681641790088, 10.740505345684044], [3.1944426770326575, -15.044636547214855, 11.682420793520981, -10.632677401144923, -4.2101809131495598]], [[-1.7289397551755867, -0.9363014782062401, -7.5307620750702231, -3.7628989952942438, 0.2989048923788068], [-2.7628073397250348, -4.6249568184469423, -2.8510918334921938, 4.0954373675981266, 9.593971550255489], [-0.80923808278769083, -9.611121274296675, -4.5981774146672061, -9.6447116188042248, -2.2576697948868549], [9.7907691966454937, 8.3854429371101684, 8.9430244143267732, -4.1396875939967384, -4.910425002376491]]], [[[-5.527005619141435, -12.078820480992071, -12.508025279863983, -12.840028800739818, 2.9241839674129202], [7.3932076303429231, -2.0122850700138413, 11.299715701059158, 1.2830901200786076, -14.231226599830284], [2.6770670885065426, 14.179110304557973, 5.1277788972087004, 9.7975832949412851, -1.6981205284182286], [6.4348104262940424, -14.308190500272843, 1.9822022226803295, -10.20232999197864, 13.938282756756294]], [[3.180015992018951, -4.8958051259061852, -1.2042163602432283, 1.1655826760566601, 3.6962638857563732], [2.5418994318349557, 1.1856031655656118, 0.78839932902965904, 3.7312625978080391, -0.73428676141704141], [-1.9873055636034977, -1.4886877257457309, 0.16548371593336145, -3.6509819641438952, 1.1934976953344498], [4.1122788674819555, -4.2332433190091923, 1.1189645154419723, 3.4214056631901659, -0.83092337689586282]], [[-0.29206817687354014, -0.075519267703794704, -0.90846632986307008, 0.049215936996426679, -0.14233224184275164], [-0.91280386490233156, 0.79131648621271988, 0.41014304073256369, -0.48080686917891136, 0.70184555313601038], [0.34063506140017491, 0.4964868621309812, 0.91297262497011844, -0.53398228265994241, -1.0278135017369616], [0.52832722736341609, -0.17041604100792501, 0.19159731256108684, -0.72219407547097358, 0.97704523854873593]], [[0.0065355235018921204, 1.4058773334768775, 2.2589645758914996, -0.64838029149534004, 1.1046742145019099], [-0.34757634995079056, 4.2771087499779696, -3.8264831487940008, -1.8825565496094265, 4.6349970173008099], [-0.91933229649771975, -4.2658088896405442, -0.73087704710383694, 3.3621379545498682, -3.2275404954861706], [2.8112076668801014, 1.7973969379731289, -0.5080608687914614, -0.024800789427806801, -2.7751769569125866]], [[-0.34812613173320595, 0.99315451310862091, 3.6726069074459544, -4.1844454125914092, -0.78259174254736574], [3.730350694078223, -4.2015556762009618, -2.9056522205191153, -3.7288447079369749, -2.4901183843482277], [2.9885233805121953, -4.1602669811254289, 3.225301366560716, -1.8185172865398238, 1.5587915797704777], [1.1953556199850612, 5.0252773731147213, -2.9547302813667589, -3.4912982090723252, -1.7678836589237441]]], [[[-1.1012024384795525, 0.23106339688849903, 1.5707178324457416, 1.0569458900058832, 1.1442242450332911], [0.21790503536767178, -0.075182606045025802, -0.85604338713539174, 0.63923572349201929, -1.0618795747045993], [1.5189317106466542, -0.38725039755592616, -0.91413847141014593, 0.13054424948771917, 0.064175376310431506], [0.28000045219037123, 1.0913117918535, 0.72110036917801501, 0.83192971500699708, 1.6722835717195221]], [[-1.06217967830134, 1.1756786139115993, -1.2686350291000883, 0.13798319177931886, -0.36909854400580105], [0.36906652294759079, 0.6230155936146986, 0.89984472889615108, 0.59409720162055824, -0.6982905002041222], [0.32785156705324314, -0.039529911095152671, 0.35004619507681861, 0.22088099271834899, 1.2911318884498557], [-1.0469632470540331, -0.62156263269769385, 0.6741695200027783, -0.14309043763211765, -1.2045156549747613]], [[0.35170502452180669, -1.2975052190227325, -0.20177288207788233, 1.2846957630026656, 0.55402079867978393], [0.88083360085653961, -0.058213938320572645, -0.095452046389413084, -0.32515606714393419, -1.1454746928710555], [-0.10275442114717231, -1.0789571381492096, 1.2004864347788828, 0.25969930592668328, -0.71440610084997969], [-0.35581859399754412, -0.71207865449319097, -0.078560075292710044, 0.29371319048013622, 0.89946827738723334]], [[0.32612312826384154, -1.2790843814138968, 1.0100852294525373, 1.7114652863575956, -1.19997260449903], [-0.12712784636608612, -0.68509201282969345, -0.030090086390798576, 0.97613421697732716, -0.33785383875131908], [1.5528872492128181, 0.59194547276281684, -1.0745222370603358, 1.4105060618631813, -1.7211131773918347], [0.22525548296259354, -1.5460668598904757, -0.94019544433069535, -1.132247485925624, -0.13352114229053302]], [[-0.73700521474752745, -0.006676041608527087, 1.3364947184910785, 0.70444142527201647, 1.2408115922725316], [0.18761024856847658, 0.96329743452455385, -0.38674608719634795, 0.84127488676570161, -0.62715806069747837], [-1.6537349578193206, -0.68584238837207534, 1.2639148691003022, 0.23703078757850987, -0.48418322897877197], [-0.7946159994275912, -0.60500299930262791, -0.52709897408967865, -0.24554793292647067, -1.4717731085000527]]], [[[37.733571646255129, 19.768054841287864, -69.569731341389968, -26.483260144020669, 7.6832185147355023], [65.245958024899366, 13.756057601659778, -9.3236783520074695, 69.777836533618, -62.813925670819387], [-5.1548595913268471, -68.455902563080514, 19.60211987298031, 58.219342972391658, 60.848384088711398], [38.062719673185235, -51.402454456581374, -10.155421170100427, -75.935761841178604, -49.074422807082556]], [[-1.769049521178812, 1.8532330756853874, -0.35471388428184897, -1.955117869005863, -2.3121616222962467], [-2.1374642369987886, -2.0186072883675288, -1.6847025578594346, -0.56954128568393547, -1.3605681585229512], [-2.1995744664349326, 2.2963514199900219, -1.751617449321788, -0.48343373528712036, 2.1680184072719024], [2.2248645063728825, -1.4973616447800513, -1.7240963334303827, -0.38931031748430212, 0.74782943588057715]], [[-0.5574315329046291, 0.22287400188156953, -0.63132846667216203, 0.98965880045388965, 0.40372416137700695], [-0.67412797516962741, -0.8106560702880895, 0.65060991125494394, -0.65344301417779571, -0.9130240405804857], [-0.84792656415637513, -0.31798849462496492, -0.42111348714762487, -0.97488226163580594, 0.94410004524205837], [-0.36377961340827492, -0.67375979629081884, 0.27095171359263781, -0.85123500569421517, 0.39719970781704483]], [[2.7692622773972189, -2.3947380992870642, 4.8831708643648817, -4.1033967588742781, -1.3368163760330398], [-1.7142166373114653, -3.8894645041237861, 2.9353110719414905, 2.4857867471782731, -3.1518507965302964], [5.3535269428226329, 4.5395096853142576, 5.0779920492473849, 0.48718667546989203, 0.33380191000066695], [0.55046589086215736, -1.7855778290954643, -5.000531880777948, 5.2465806364846639, 0.7072012451301265]], [[2.7637223662843535, 1.6113969706335105, 1.1678656973190038, -0.18582321617064823, -3.6983209928269032], [-1.1548739340450007, -1.5684965374441109, 1.1489601911090983, -4.8310815818611248, 4.4455487189246066], [2.8069327677090938, -2.6046051252554263, 2.9951643546173705, 3.5131699082669705, 1.0825052114532601], [-0.18796485923021311, 4.0792482740451828, 1.2238528968511644, -3.8997502212971016, -3.8299024626277731]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.0490405292898672, -0.022008364514584322, 2.0832902150618606, 2.820955205628576, -3.5173913049466785], [4.830975207846544, -2.1976570411388319, 3.6424099644638392, -2.8722232598362529, -1.6813004845841037], [4.6352463682994607, 4.085670059053605, -4.2500305281090505, -3.4984347475702338, 2.3674268445308453], [0.45680473168447655, -2.4940035598622989, 2.2719867946981509, -3.7842817497557424, -3.9021947857299457]])+(1.-msk_arg0)*numpy.array([[-1.6330231470050958, 1.4665587905601996, -4.9210231523748336, -4.7808506655174385, -1.2397232994286433], [-3.0965215321251494, 4.6285180056864643, -4.4118516917550075, 0.88954810414002328, -2.8314859961891834], [-2.5687526650053827, 4.8917605774524056, -3.9026761457417827, 1.9256537231787973, -4.0873190734667952], [2.534114155190073, 0.2707942958627978, -0.14164297302730144, 1.6781031816303429, -1.6643082436268744]])
      arg1=numpy.array([[[[-4.629086296736423, 0.83832625233974767, -3.0007512849138451, -3.1176464739345313, -0.6782390604527162], [-0.62831938027844814, 1.8256138300731113, 3.5384327568601091, -1.4451670589777352, 3.6901429908133512], [2.9694168112377017, 4.0101501184260595, -0.3046419134567433, 0.7941066520246487, 2.8031077396685591], [0.49479445683190715, 2.6288385413203406, 3.3660141136473882, -3.828808046489006, 1.6512946969881392]], [[-4.7427191876891861, -4.6218552474648247, 4.5767194496373431, 1.399329025270017, -1.726008252406209], [-0.61244196820754837, -1.7551253904370654, 2.5595266350033965, 1.4948168561743316, -4.888413310763303], [-4.4981567415776569, 0.25884150161936237, 0.85798515927510177, -3.9064459904391402, -1.1709932372760878], [-2.8130248000735958, -1.8309546990087231, -4.4516363676612203, 4.4139978167818779, 2.0858693860295325]], [[0.81052613570077092, 3.0724941649165576, -1.9025262567114201, -0.10982235483314096, 2.1221165370035582], [-3.1193818878279469, -3.3249355195087391, 4.5425353044696752, 4.870302525864421, 4.6380168244873268], [-3.1209130761130375, 3.7296186990797651, -3.5973552258978825, -0.8078078748401758, -2.411286809223224], [4.1594382556825558, -2.6248400073482103, 2.2162824978836779, -0.072191233243628083, 4.2328590932208563]], [[-0.77198778516429556, -2.3322675258809964, -1.9075444872853131, 3.0046492737492407, -4.928433454378478], [-4.4489533970111692, -0.48909553051010057, 3.2337854942529773, -0.019654207754529196, 0.48458581510208099], [0.091138196379944247, -4.6927307310457138, 3.8726686055776103, -0.20371746185889705, 2.3514689106494266], [-3.10516795682521, 1.0090053758359199, 2.2055038260337838, 1.2033975685853893, 0.33127495726220513]], [[-1.3594263836380263, 4.3679800312273613, 2.5316712745982164, -4.7057517503355459, -3.3864053754729726], [4.1335287308517188, -3.4098102147910483, -2.5916827563469749, 2.1869531312937438, -1.0765449378572933], [0.52005044811342849, -3.1357453777259936, -4.0286861865916404, -2.7670599188457756, 3.9380570349816928], [-2.9955237441890361, 3.5140495187285197, 2.2452151329201522, 4.6422096739805934, 2.5479220975824921]]], [[[-2.5446335686939769, -3.0586110558217774, 0.27919572515452185, 0.54652382018929302, 3.3416812067526145], [-1.2339258914145357, -1.7273972352748155, -3.4467557172528105, 1.7322729843503906, 3.1628821433791714], [-3.1509733263738493, 3.4498954155456012, 2.1412060055658335, -2.2865619838095199, -0.51124114736713633], [-3.6745579579662135, 3.7677439123161971, -4.8441187150594791, 2.6302396659442291, 0.41196656959005651]], [[-4.6690696526848097, -0.27921409212317094, -4.3326480221178016, 1.8191560519915138, -3.3887988189204279], [1.8711027830220299, 2.4611319463883854, -1.6166901232677668, -2.5319758029392894, -4.2903407026008384], [1.4404570030361707, 1.7416887514236388, 1.8471200501170602, -3.8755690733585322, 0.2579246462816176], [1.7640000806238065, -3.9152515060144397, -0.54601618736331048, 2.3966888629969931, 4.2889562771582543]], [[-4.9235403988297648, -3.64756898744126, 1.252175904661228, -2.4975277525280548, 0.77095575145806272], [0.68617430859184392, -1.6276578036173008, -2.2864947027649087, -2.8275082271280185, -1.2869303503211049], [-2.1282018619610521, 3.9692574431614105, 3.2801757530078213, 3.3523816448976245, -4.7186799647545463], [-3.3101335676154662, 3.979386204997386, -2.8860094402639334, -0.95549618588623009, -3.0529642093926759]], [[1.9123560598347735, -1.7852304581089773, 1.1381231060070292, 3.5684883820885744, -4.5960433995119701], [-0.67379597476586639, 4.4175921713581374, 2.4469178543066583, 0.92468438218441484, -4.325408613462236], [-4.9905873766486373, -4.8447256934389946, 3.3222524808713167, -4.5161408608788012, 4.9552091395062909], [-0.18696068630929563, -1.3720359977625698, 0.57360890843710521, 1.2690027753546236, -0.97908326345255059]], [[1.6885264650093577, -0.21327104428732291, 1.2378919425844979, 0.91454247837963187, -0.15684439505497672], [-0.85618762454553909, -3.303581917813939, -4.3865092263231862, -1.8432786722955576, -4.8481767151112578], [4.3997890874157957, -4.6121425881912099, -3.7370106874400086, -3.6768056151392292, 0.87726751996553798], [0.38931549125171472, 4.3672864953755948, 3.7054286370443901, -2.2539902759510722, -1.7538491622297734]]], [[[4.0641939058821421, 0.28337514620491966, -2.6974616072013404, 1.8881935943712325, 1.5431869230629927], [-4.6770315977680763, 3.1436124010764459, 3.6756154881345697, 0.20054415848580476, 4.7888001849684834], [0.23194233660288965, 4.5943087477457354, 4.1388912772957855, 1.075117404399542, 0.24274452361583521], [2.5479384700468461, -1.5349988803859747, 4.574123712571609, 2.4839809406341109, -1.449680360422839]], [[-4.2786642753175865, -3.7232621042281409, -3.4572378900473097, -4.7107299445566566, 3.0705543957026951], [3.9824342297139292, 0.41971949857903201, -4.0524278670724181, -3.595339740542205, 3.155331982048418], [-4.4349802559467966, 2.3879382705510572, 2.0622084313928362, -0.87689311076340637, -3.987554696144918], [-1.1878756882760708, -4.0148276904873912, -4.940197034090211, 2.8495810327682412, -4.9601153423019397]], [[4.0051608592469954, 1.0293994194442089, -2.0513543476676444, 0.28234882788641436, 1.8658875342315042], [-2.2239122455071767, 4.1230181267660235, 2.773036247951401, 0.51969045050264651, 4.1936237132164429], [0.93935695056310475, -1.4606825947767641, -2.4848379274768906, 1.352885788640954, 0.090000247156790891], [2.5317312982749849, 0.64298589617182156, 1.8419409152805333, 0.74169459478872746, -3.4287744435501777]], [[0.3717339358466738, 4.4293021028996513, -4.8752657327288116, -2.4528922325562288, -3.0354892580825252], [-1.0137678749550916, 2.9751059190301019, -3.0170145531703851, -0.61568363450327013, -4.0475872236659285], [-1.5621741937416678, 3.9797798383014378, -3.4437961087049263, 2.4639575179489928, 2.6467004272417958], [1.5275122246320603, 3.025940325063905, -0.27262250843830493, -0.55168349528155503, 0.28081231433380704]], [[-4.8400458706846923, 2.419637613221238, -0.84654401162402326, 4.9674017196370475, -1.9980482612791892], [-4.4348187161072019, 4.6143187623891126, -1.8849826396595395, 1.3241056808206491, 4.5154191816131153], [1.5464753903271289, 1.1635114012671561, -2.3506363519347131, 0.087778764638660078, 4.6114353148546474], [-1.8754791092755232, 3.5934761987025556, 1.9280689502397053, -1.6453500776856247, -2.1214103133375861]]], [[[3.3730060396318589, -2.3871548358510788, -3.7208953950787347, 4.7449342700609947, 1.6808461951915277], [-2.0294651577819045, -1.7808971456410583, 4.2551986427984581, 4.8022476492794812, -0.047664719953464108], [-1.1441893020434399, 4.2158807283531115, -4.5041416081579513, -4.7665868233593294, -4.8418069594418967], [-3.450315441451417, 3.2426815483261819, 1.9255124317264318, 1.252198143897215, 1.7695818860185621]], [[-4.6875062973960571, 4.2068261241511991, -1.6838334881853276, 0.78310975305973329, -4.6077003262763325], [0.93426899859518908, -2.3031542568986199, 3.0398181011393639, -3.74763226252812, -1.8883290461380318], [4.597015224965368, 4.4776775482195426, -0.98375869446620889, -1.6319577928183069, 3.1225984106748133], [4.1663482553285149, -4.1750118432855521, 4.2922772841964054, 2.1833248323364618, -0.89845830753246236]], [[-1.0540564092399416, -4.550173215178674, 3.0441105159161879, 0.27320905171932885, -3.8339292577408179], [4.3694797463186958, 0.38339140585857479, 1.828735989025736, -2.8440422417238254, 3.6321859680548023], [4.0445140964012545, 4.8909954827969209, 2.5209161660011157, 1.121033988463914, -0.15701390427101813], [-3.3126238070012892, -1.2122670925946721, 0.15298147070074641, 2.2222267887643268, 3.9305529701679394]], [[-4.3466391657481633, -1.7053057943603322, -0.69385902775994968, 2.6820828944815753, -4.5529645985769118], [3.6578254793608149, 0.30748262258790149, -3.4673670964388048, 3.4783364838735018, -0.048445157473954836], [-1.7992889020142377, -1.3736027933583017, -2.8847636756574091, -4.7213837843962692, 2.9470072421759399], [3.534355594583694, 2.4610627638806992, -3.8211572204414015, -3.9387861728457105, -1.0072857216105859]], [[-3.5341510656791018, 3.1259935740802707, 3.0558144718224796, 3.9996143513231104, -2.885290112418204], [4.3504843804831701, 0.89318303967601054, 2.9378351639071703, -0.49253845083940639, 1.7020629659291631], [-1.9445574099638243, 2.5875717203590289, -1.8432143167376216, -0.43056197835754162, 3.1500711696938275], [0.67880102250075591, 2.9492142394884162, 2.4996854007299332, -1.4412971934298446, -2.0512104402244766]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.44264470306688219, -2.4442041789470941, 0.68284250667202417, 0.6572395383572589, 3.0211184356179044], [3.2611448788700543, -1.1223844251923794, -0.57908138152895683, 1.4178572065843327, -0.55527401902608509], [-0.69004813387440667, -0.51096354719361314, 6.7260624319207949, -2.5803089850282048, -0.73098885936226865], [-4.1411954014391323, -0.77944708169895127, -0.60874389117446148, 0.5351640783268895, -1.2408690786854655]], [[0.0046404527958796449, 0.0047618030717548601, -0.0048087641719721875, -0.015727798192664195, 0.01275101928620718], [0.035935428427605706, 0.012539482725563983, -0.0085986073415309922, -0.014723117700793185, 0.0045021488805225057], [0.0048927518045681169, -0.085026413372259663, -0.025651218178620764, 0.0056338586450315337, 0.018794612824391023], [0.0078237363972079189, 0.012020157858902586, 0.0049438819114839272, -0.0049860388310363967, -0.010551170970718067]], [[2.5702936935656906, 0.67804529585443118, -1.0950125958644572, -18.969637085519757, 0.98170396334759236], [-0.66785353316021012, -0.62656559889307806, 0.45861838718391146, 0.42775375944271576, 0.44917694219277476], [-0.66752586959471127, 0.55857994694628843, -0.57911718032846793, -2.5789426916320148, -0.86397445840670239], [0.50085855036210813, -0.79368274227370539, 0.93999308168123363, -28.857939135507717, 0.4921709343923964]], [[-3.6541448709945796, -1.2095332865225148, -1.4788411093065339, 0.93886339090370807, -0.57238374662894276], [-0.63407164649638925, -5.7676977801994429, 0.87233838194955249, -143.52932668977735, 5.8213738778844037], [30.952501998924259, -0.60113297934739229, 0.72842669821158867, -13.847390301684024, 1.1996566030930373], [-0.90847105369230363, 2.795778172431965, 1.2790525105102968, 2.3441589706256831, 8.5154496100222321]], [[2.5874084446806296, -0.80526725850400116, -1.3893554586801158, 0.74746639677621518, 1.0386799319486113], [-0.85094154026163393, 1.0315504627468608, 1.3571843607527438, -1.6083523943038882, 3.2672963117986846], [-6.7635578773301974, 1.1217082005227894, 0.87308644606108454, 1.2711655721621991, -0.89317937086785493], [1.1742157984125494, -1.0009509786929149, -1.5666166031813262, -0.75769763797217549, -1.3804940536777139]]], [[[-1.8984954326158727, -1.5794669932455907, 17.303184728823567, 8.8394595612928573, 1.4456720761048294], [-3.9151258932645128, -2.7966787888704552, -1.4016006947243151, 2.788807105745045, 1.5273965291306142], [-1.5331691853469438, 1.400325118865241, 2.2561935634819568, -2.1127680955308774, -9.449503884274181], [-1.3147092148521673, 1.2821930896244835, -0.99728670827738508, 1.8367053278060399, 11.726619498892338]], [[0.47068414151310306, 7.8708672059767304, 0.50723184295608104, -1.2080640573594308, 0.64850619897198292], [-1.1745250240018252, -0.89294564005956989, 1.3593557661481683, 0.86796131250055486, 0.51223368806272074], [-1.5256665325703216, -1.2617966553108235, -1.1897748827963599, 0.56705402472271327, -8.5205391296312882], [-1.2458372679675107, 0.56130673540713449, 4.0248935690922032, -0.91695550268077874, -0.51239903116824026]], [[-0.73979487714360448, -0.99858562703126841, 2.9088644422121197, -1.4584062022041231, 4.724538285855143], [5.3082867120728192, -2.2378229357356076, -1.5930104539753847, -1.2882049040626629, -2.830308542770021], [-1.7114964654280989, 0.91765525835047734, 1.1104313423217826, 1.0865141115444426, -0.7719129060818406], [-1.1003815677104947, 0.91531954347372324, -1.2620921864104266, -3.8120612287797631, -1.1930732608190062]], [[-1.5019291230130081, 1.6088809412756062, -2.5236490188773248, -0.80488513686996799, 0.62493388555496221], [4.2627492110416743, -0.65017845659419959, -1.1738127026949607, -3.1061660769603328, 0.66403512743208937], [0.57552809780981262, 0.59285570362135931, -0.86454093310902247, 0.63599062746624413, -0.57963714123324062], [15.362712431877968, 2.0934022609611516, -5.0072849594719724, -2.2633703531764211, 2.9335842691334593]], [[-0.99572054061633331, 7.8833978152187552, -1.3581964844797745, -1.8384061148947377, 10.719544577890581], [1.9637056602827343, 0.50893258481589632, 0.38328894294686938, 0.91212495964610074, 0.34679026433662508], [-0.38213206387390969, 0.36453783733591727, 0.44990518497442594, 0.45727206183034458, -1.9165197004559691], [-4.3186066888282308, -0.38497599971158036, -0.45373980968776167, 0.74592180033903577, 0.95863459685812769]]], [[[1.1405081734881866, 16.357279141719552, -1.718373435204738, 2.4548575856402031, 3.00368432302368], [-0.99106586547575259, 1.4744967817000101, 1.2610803233533869, 23.113345226794827, 0.96793480397219089], [19.984477332551425, 1.0089105070645963, 1.1199246507697533, 4.3113862256636679, 19.095163504636464], [1.8192144052105927, -3.0197066770067811, 1.0133627027970102, 1.8660555290396772, -3.1974264774805006]], [[-0.954893816423665, -1.0973361382251099, -1.1817729033965019, -0.86731145855106362, 1.3305968670581396], [1.0259227958039854, 9.7342870009273224, -1.0082030311387633, -1.1363794116540025, 1.2948463370251198], [-0.92123748546009721, 1.7109613382555184, 1.9812110147828765, -4.6592566515851619, -1.0246053961350154], [-3.4394761163796677, -1.0176451828142128, -0.82702573011969427, 1.4337792159869054, -0.82370464739182592]], [[-1.0611385353716087, -4.128650597455862, 2.0718168623286681, -15.052410735768268, -2.2777527852767898], [1.9110603562236357, -1.0308056858926913, -1.5326271090934309, -8.1780038944306277, -1.0134506142539299], [-4.5244041954033944, 2.9096194774324546, 1.7103854062725692, -3.1414555195959544, -47.222431741826256], [-1.6787052129129352, -6.609834762181686, -2.3073652867208057, -5.7301624657513868, 1.2395188421051513]], [[-9.4111255664675344, -0.78983882026921948, 0.71758852529502426, 1.426248858851991, 1.1525109957991235], [3.4509228729754424, -1.1759025872634274, 1.1595684031069573, 5.6821954515532154, 0.86432596859560118], [2.2394652027831152, -0.87905233196602117, 1.0158658170055994, -1.4198437765608651, -1.3218098699655463], [-2.2902826511996786, -1.1561479645162369, 12.832523505160019, 6.3413801164828874, -12.45826685296848]], [[-0.48913314207824637, 0.97842207097248535, -2.7965785736161983, 0.47659258867104992, -1.1848697003020201], [-0.53382719702439763, 0.51306096662144873, -1.2559409273702613, 1.7879440280503673, 0.52429835399802049], [1.5308532287927705, 2.0347259527947297, -1.0071429562391956, 26.970382350176848, 0.51338177441299893], [-1.262305099972751, 0.65881244611711021, 1.2278745758737089, -1.4388590468606566, -1.1159683865240595]]], [[[0.13542956233020376, -0.19135948989317006, -0.12276742105909444, 0.096272088438983675, 0.27177069085278499], [-0.22508626469041695, -0.25650259073218035, 0.10735215204525823, 0.095123110061392721, -9.5837074492509959], [-0.39923877182617956, 0.10835333376779907, -0.1014188210373108, -0.095834765758559287, -0.094345919924310934], [-0.13239506341840909, 0.14087252321162141, 0.23723800696259401, 0.36480227503193996, 0.25814274846147744]], [[0.53205337798644359, -0.59284683660784954, 1.4811461925193647, -3.1847433263573, 0.54126861194504006], [-2.6694705310915809, 1.0828643163574605, -0.82044499930029158, 0.66548780273864594, 1.3207462782839565], [-0.542526713054575, -0.55698596716817828, 2.535178162990015, 1.528227979202381, -0.79869494307573452], [-0.59860659911773206, 0.59736442757001307, -0.58104437219023308, -1.1422961544358783, 2.7758701087776272]], [[-2.1554698351831414, -0.49931874837624957, 0.74635489835833035, 8.3159279694444095, -0.59260008256827945], [0.51996734774026787, 5.9260243186991, 1.2423809715193268, -0.79885831559275955, 0.62551499694133261], [0.5617453025370166, 0.46452441076451634, 0.90125440319666128, 2.0266885911383641, -14.469971976345013], [-0.68585717155575177, -1.8741635474368201, 14.851385493230623, 1.0223919566560051, 0.57803235624657578]], [[0.8706224753083166, 2.2191220848899085, 5.453963410943711, -1.4109488403740083, 0.83116871827612471], [-1.0345714335220348, -12.307302825459388, 1.0913992215137613, -1.0879573518262782, 78.114757946452215], [2.1032096321604485, 2.7550044074266959, 1.3118169026075739, 0.80151962275603161, -1.2841100950134061], [-1.0707133587675937, -1.5376616172878663, 0.99034965887077542, 0.96077359462792544, 3.7569099497458538]], [[1.1041392156733183, -1.248305440575977, -1.2769737239324896, -0.97564276026738994, 1.3524445146555675], [-0.89695639484092649, -4.3688635054528255, -1.3282551838409566, 7.9226196027531417, -2.2926265736589375], [2.0067264487719805, -1.5080528029532303, 2.1170597202373056, 9.0630268855034295, -1.2387640073888302], [-5.7486577898099736, -1.3231303217927084, -1.5610743594335774, 2.7074185695483948, 1.9023863710945741]]]])+(1.-msk_ref)*numpy.array([[[[0.35277440132330268, -1.9479565890336477, 0.54420476472510504, 0.52379997560922564, 2.4077397516962131], [2.5990335460946623, -0.89450634088354664, -0.4615102954377428, 1.1299891848906685, -0.44253654968669875], [-0.5499474310325686, -0.40722244773371219, 5.3604677323462644, -2.0564279909273542, -0.58257594736554263], [-3.300407117454573, -0.62119568065405262, -0.48515041585359364, 0.42650953695695665, -0.98893501564780073]], [[-0.30922319718337715, -0.31730954606694678, 0.3204388660258406, 1.0480442869947686, -0.84968237464431939], [-2.394608577939914, -0.83558633391714177, 0.57298047635212568, 0.98109596804624444, -0.30000711832838112], [-0.3260355018322077, 5.665856446455166, 1.7093055453304951, -0.37542021421761357, -1.2524058584417133], [-0.52134584470134449, -0.80098037999202976, -0.32944262950450587, 0.33225181602591425, 0.70309234143936727]], [[-6.0713935499688452, -1.6016379163761498, 2.5865730551761139, 44.808938579505153, -2.3189222017577551], [1.577563546027186, 1.480035664902734, -1.0833208379322317, -1.0104142661038105, -1.0610188230437887], [1.5767895588119858, -1.3194440368901603, 1.3679558573886932, 6.0918236942768775, 2.0408286287437121], [-1.1830980170583885, 1.874789754270159, -2.2203952596629288, 68.166492401751356, -1.1625766518559779]], [[6.1929097291351196, 2.0498723291666585, 2.5062852779497784, -1.5911509896633727, 0.97005482772017038], [1.0746012014262141, 9.7748811168469807, -1.4784068621786683, 243.24820034609229, -9.8658493842010682], [-52.457156882791971, 1.0187779652236915, -1.2345106572330562, 23.46804550720767, -2.0331336909732172], [1.5396431793678182, -4.7381815597926806, -2.1676909416724839, -3.9727940211291899, -14.431669405464238]], [[0.91194588714025127, -0.28382073419879916, -0.4896857312667463, 0.26344851262930397, 0.36608827413507566], [-0.29991887807035922, 0.36357545474260744, 0.47834685645555491, -0.56687236762831572, 1.1515759870611002], [-2.383851997293629, 0.39535202961143368, 0.30772396806549912, 0.44802907627160071, -0.31480582643069988], [0.41385861214873054, -0.35279050361168773, -0.55216236575790634, -0.26705456808149891, -0.48656248187686429]]], [[[1.2168830790495413, 1.0123946705257647, -11.090862979407614, -5.6658491683905812, -0.92663582805802502], [2.5094874446433648, 1.7925937757058623, 0.89838729116352622, -1.7875482444739259, -0.97901894277251711], [0.98271905579367014, -0.89756968230743717, -1.4461576906080389, 1.3542259313548977, 6.0568707117415412], [0.84269225510842272, -0.82185031790591678, 0.63923320510263926, -1.1772773303582318, -7.5164388586347295]], [[-0.99131483357181716, -16.576949861272279, -1.0682884882543588, 2.5443215828677332, -1.3658284994212122], [2.4736845285489424, 1.8806460224445232, -2.8629592889025517, -1.8280261605633696, -1.0788229482289413], [3.2132288544056178, 2.6574886023137947, 2.5058024817570113, -1.1942808702607985, 17.945233510692773], [2.6238763005326358, -1.1821764192099373, -8.4768878886858392, 1.9312135493042808, 1.0791711797895021]], [[0.89607301542678996, 1.2095320765543314, -3.523348177625746, 1.7664875544582959, -5.7225744582761529], [-6.4296369545064138, 2.7105523544015973, 1.929526312228077, 1.5603320441038122, 3.428197719211608], [2.0730419283110972, -1.1115055536032659, -1.3450046655913097, -1.3160350339198141, 0.93497582474519458], [1.3328319240402104, -1.1086764301023417, 1.5287031394296242, 4.6173409762624864, 1.445104295091836]], [[0.46515820083048748, -0.49828194455201363, 0.78159216647564433, 0.24927868859121413, -0.19354649789305292], [-1.3202039451914616, 0.20136492225504421, 0.36353819666417836, 0.96200186926333942, -0.20565643240535192], [-0.17824517176120208, -0.18361165531924756, 0.26775451572745135, -0.19697085001173437, 0.17951777192367965], [-4.7579420128379963, -0.64834166566375995, 1.5507919961768863, 0.70098200052512771, -0.90855204796699451]], [[-1.6768976115358023, 13.276467068706008, -2.2873450410199334, -3.0960683217317078, 18.052835073874956], [3.3070858711513433, 0.85709574232772379, 0.6454986984177763, 1.5361139033106401, 0.5840311033555643], [-0.64355039296946148, 0.61391987390824265, 0.75768742265194233, 0.77009401436142111, -3.2276197758927787], [-7.2729856885105706, -0.64833987859220354, -0.76414533203578272, 1.256210386708273, 1.6144410004959298]]], [[[-0.63204480998989865, -9.0648481329686454, 0.95228516252007189, -1.3604286513114541, -1.664576485593072], [0.5492271350553235, -0.81713402839541605, -0.69886327155212402, -12.808912931698323, -0.53640840414858293], [-11.074962435182176, -0.55911624708848284, -0.62063786963829615, -2.3892764218062705, -10.582124065013563], [-1.0081690335945019, 1.673455725491781, -0.56158355707463137, -1.0341273650632887, 1.7719441713731541]], [[-1.1432915187273747, -1.3138372858298955, -1.4149331729629591, -1.0384294228339142, 1.5931196608334073], [1.2283343039173797, 11.654832796697676, -1.2071184825274492, -1.3605836806717717, 1.5503156578398165], [-1.1029948940343353, 2.0485289078781541, 2.3720980396479425, -5.5785140941451035, -1.2267569852224609], [-4.1180744969632928, -1.2184235425701562, -0.99019544032281182, 1.7166595794962454, -0.98621911787684124]], [[-0.97441183585208591, -3.7912165793224433, 1.9024875688488732, -13.822179376327298, -2.0915923785026855], [1.754869668812743, -0.94655808578822065, -1.4073657164146018, -7.5096168151004123, -0.93062144165254446], [-4.1546252927625575, 2.6718166969999584, 1.570595853591372, -2.8847048128595016, -43.362949203271263], [-1.5415048778679246, -6.0696139199589725, -2.118784654472694, -5.2618371135001523, 1.1382131458320495]], [[5.1801935133871035, 0.43475330389366856, -0.39498436162185552, -0.78505435241727628, -0.6343800156931555], [-1.8995016223650814, 0.64725551815196192, -0.63826464514572945, -3.127667547526098, -0.47575348393226696], [-1.2326754153879187, 0.48385935941638986, -0.55916600820568285, 0.78152878414142402, 0.72756769272356892], [1.2606470129184331, 0.6363819230764669, -7.0634436393742481, -3.4905045005851196, 6.8574404500286104]], [[0.84447940839217428, -1.6892277798679904, 4.8282416712459151, -0.82282837269006759, 2.0456558295794189], [0.92164287541714107, -0.88579035908445614, 2.1683590010171319, -3.0868526075151137, -0.90519150250998781], [-2.6429900527561561, -3.5129170792958115, 1.7388138620857665, -46.563871003336409, -0.88634422785904954], [2.1793466284173548, -1.1374276181215683, -2.1199029593618235, 2.4841637830753212, 1.9266989736824045]]], [[[0.75129250449449381, -1.0615625417890413, -0.68104955558323366, 0.53406728332982745, 1.5076419022986918], [-1.2486610797298547, -1.4229424542526761, 0.5955336913539474, 0.52769335116865246, -53.165405307409181], [-2.214768264887923, 0.60108772483703476, -0.56261866869377675, -0.53164124542351565, -0.52338190605644763], [-0.73445868883340915, 0.78148720971325114, 1.3160726014726185, 2.0237325598512328, 1.4320411930140493]], [[-0.057769372174118663, 0.064370213522298908, -0.16082011538720115, 0.34579354273747936, -0.058769945241130196], [0.28984617521289574, -0.11757540557767235, 0.089082401266477262, -0.072257435333348941, -0.14340418923101364], [0.058906547533750631, 0.060476506614567105, -0.27526495815087232, -0.16593216874509359, 0.086720820370967081], [0.064995597887542833, -0.064860725197295369, 0.063088723755062706, 0.12402840468452428, -0.30139884465703343]], [[0.13437893056353339, 0.031129138678677636, -0.046530167773712078, -0.51844172854423975, 0.036944597436512433], [-0.032416438855595156, -0.36944743899541305, -0.077454030476406893, 0.049803399875470583, -0.038996619191048076], [-0.035021011090883118, -0.02895994762732898, -0.056187101712306113, -0.12635029310876325, 0.90210464917052646], [0.042758544670220777, 0.11684139072366993, -0.92588319603996561, -0.063739206881788274, -0.036036398466663969]], [[-0.38606912551055989, -0.98404824939905089, -2.4185073833339334, 0.62567163195554643, -0.36857373812105981], [0.45877070710426082, 5.4575545359498019, -0.48397044067063338, 0.48244417680994295, -34.639234737394084], [-0.93264799207718563, -1.2216800881189041, -0.58171253187591521, -0.35542613315535088, 0.56942621572633312], [0.47479749468389409, 0.681861188694044, -0.4391609883658481, -0.42604576841447067, -1.6659654213574699]], [[0.47092164785748075, -0.53240936175518105, -0.54463654746496615, -0.41611717966666095, 0.57682526844137461], [-0.38255699781228364, -1.863345103631366, -0.56650838143465809, 3.3790422672392069, -0.9778182575744615], [0.85588023017424641, -0.64319308737689773, 0.90293799723333301, 3.8654324517359528, -0.52833988629807294], [-2.4518352042185101, -0.56432259865786238, -0.66580708241960351, 1.1547294001637038, 0.81137859431172688]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.97115913705939771, 2.5320486079552262], [-3.1451556530087332, -0.81501677476249768]], [[0.016749416106957682, 4.9472288058018705], [1.7674000519075781, 4.7349545816375169]], [[4.7842975375527601, -1.2747079626861102], [1.7172124658762353, 0.9960281739514425]], [[2.3251526886355727, 0.8836679927459663], [-0.31218781317445288, 4.5997608625519124]], [[2.4269033248814766, -3.9603763127354408], [-2.687846324882802, -4.6983478966808301]], [[4.6973087263161748, 4.3731354225464418], [4.1224677096429385, 2.1264719129750755]]])+(1.-msk_arg0)*numpy.array([[[-4.7714645092416692, -4.5676079017914342], [-0.12709878519162388, 4.2912072082845452]], [[3.9871487561384615, -2.8511996790618275], [3.849848480173943, -0.22537885155561455]], [[-0.070006271592022884, 3.1360835682857129], [-0.83806385532153005, 1.0423339812305565]], [[0.038063883885207872, 3.412493538539886], [2.8198172040445124, -4.2756149766899698]], [[1.9845132648997943, -0.83131586943600233], [4.0089493252333135, 1.0583549325137254]], [[-0.72413586535276853, -0.14630073963672796], [-4.0856177162577954, 0.30073428061724439]]])
      arg1=numpy.array([[[0.43452563422455626, 1.2421103234126676], [-0.26174837607265644, -4.7561796506688072]], [[-4.5880094219576648, 1.5536409421258899], [2.7143403398528232, -0.62020292878860417]], [[-4.280286580881695, 3.252504906450536], [-0.31884887825305164, -3.0848974228079307]], [[-4.416823888571769, 2.7213554530723467], [-3.163412675734445, 1.2136355695465557]], [[4.9504350098462417, 3.2206810189139912], [-2.8814788455586315, -0.91728755649124238]], [[0.39382610534931306, 4.8584866546609096], [-0.44457618363093232, -2.8879371059478931]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-2.2349869848127701, 2.0385054050581313], [12.01595096863446, 0.1713595437144371]], [[-0.0036506934852393673, 3.1842806607763827], [0.65113428333140044, -7.6345247044962337]], [[-1.1177516848806988, -0.39191576933767058], [-5.3856625599083481, -0.32287238032207866]], [[-0.52643092577264561, 0.32471612326435556], [0.098687033648549402, 3.7900676100573558]], [[0.49024041726726053, -1.2296704608365325], [0.93280099176355735, 5.1220011254188282]], [[11.927367593242174, 0.90010238442275969], [-9.2728037655414095, -0.73632902482379869]]])+(1.-msk_ref)*numpy.array([[[-10.980858511964909, -3.6772964652946798], [0.48557621292115921, -0.90223825075260178]], [[-0.86903674108785478, -1.8351728522038382], [1.4183366852156385, 0.36339533577474414]], [[0.016355510377438865, 0.96420563795801495], [2.6284045906424924, -0.33788286557736008]], [[-0.0086179310847542684, 1.2539683247505431], [-0.89138455620237389, -3.5229809375869277]], [[0.40087654134488526, -0.25811803918300508], [-1.3912818868729537, -1.1537875173648775]], [[-1.8387198195266403, -0.030112409487916724], [9.1899158494947546, -0.10413463644961751]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-1.2504454378875884, 0.66343885719869711], [0.54015724770562468, 1.5359241834086133]], [[4.3873458828640217, 4.0920211753990241], [-4.8191478938762611, 1.064011443745204]]], [[[-2.2261761586106763, -2.8676103121574048], [1.2098281698038367, 1.1025835405497189]], [[-0.93917727073980917, 3.624383149203446], [3.5803274942722361, 0.089153635754435356]]], [[[2.2916476363116569, -1.5450610672119081], [-3.4027117900807733, -2.1213249619535182]], [[-4.7943904110782229, 4.6960497334278379], [-3.8341310104094317, 3.0597351976264733]]], [[[-2.9075354833899789, 0.451358207085927], [-1.4469535448263429, 3.6292494104363318]], [[-3.61705807988705, -4.1871177215300079], [1.0708284184502768, 1.0303001150404558]]], [[[-2.5613611954004623, -3.182331717721727], [-1.4413621560157583, 0.45847259493782211]], [[-2.1529607471409697, 1.1016324167169609], [-1.3418875826170851, -1.7940889592959328]]], [[[-3.8977858277112674, -0.53473928402073856], [-2.0810587134694805, 0.33560962111448411]], [[-4.7082761404059168, -3.0157567281480979], [-0.2194102313972488, -0.48635475762145308]]]])+(1.-msk_arg0)*numpy.array([[[[1.0295925142014726, 1.6240538927015615], [-0.51953478030150535, -2.4698264845549147]], [[3.0398557659738206, 2.2102845554746056], [-1.0330251908120216, -2.4822433651471609]]], [[[2.3054584514320444, 0.87743082108640991], [1.5821100500732239, -2.5766970619402851]], [[-4.7465114328113875, 0.069246302952208794], [3.3093217026696564, -0.68901472266894714]]], [[[1.2598784542053201, -4.0818854312019663], [-4.5515909162558845, -3.2368336992888223]], [[-1.280040341250126, 0.47362889355830262], [-1.0404781634081206, 3.6432087860110194]]], [[[-0.88512240852868906, 2.0849305795898569], [-1.5675873528894666, -2.5960438460614843]], [[-3.5540081857802952, -0.058329306938569303], [-0.2712023388402045, -1.8044939391580628]]], [[[-4.9244765418016634, 2.9009111959013918], [-0.36120935045176861, 3.0574181319660383]], [[1.8912711265056146, 3.5546991197620788], [0.88959328747591293, -4.2983071660835224]]], [[[4.8631982507198845, 3.2992071908086409], [4.1743612749880228, -1.3868132482386351]], [[4.6668832509415292, 4.3437592294953724], [-3.0800407598816779, -1.7365054736775742]]]])
      arg1=numpy.array([[[-3.2393624629224629, 4.8982121508975922], [-0.41259826492393259, 1.096439493589604]], [[-0.45463450566849417, -2.740087107143804], [0.83676887150026147, -0.72796920046254687]], [[-3.1901094392121356, 4.8659038116615676], [3.8609256436750687, 3.6179562862224195]], [[0.22252496996178905, 0.95377802753999763], [-1.2526671813222978, -1.6701049624594244]], [[2.19947201001116, -2.2444571434895755], [-0.16492667123908156, -3.7566191620528899]], [[-2.6036019303545013, -1.3202301713943743], [-2.8406092015803566, -3.0228330924788804]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.38601590658659152, -0.20480537908072224], [0.11027640924181722, 0.31356832576701621]], [[-10.633456938246896, -9.9176887623447403], [-4.3952702561807415, 0.97042422310214793]]], [[[4.8966282384072652, 6.3075069674724169], [-0.44152909104591581, -0.4023899596750497]], [[-1.1223855269089269, 4.3314029389085773], [-4.9182403486264521, -0.12246896667851843]]], [[[-0.71836019421253061, 0.48432854629385169], [-0.6992969696453667, -0.4359570275247891]], [[-1.2417722726497846, 1.2163015211445114], [-1.0597505074923734, 0.84570817211868632]]], [[[-13.066108868093533, 2.028348581120726], [-1.5170757797370871, 3.8051300256905276]], [[2.8874853064075126, 3.3425620020715683], [-0.64117432288408804, -0.6169074029474283]]], [[[-1.1645345718163815, -1.4468616573600226], [0.64218742612068624, -0.20426881229062394]], [[13.054048389905278, -6.6795285955902717], [0.35720618053914738, 0.47758073999588291]]], [[[1.4970744115174905, 0.20538442447226543], [1.576284771064995, -0.25420538659560149]], [[1.6574881676037994, 1.0616584380809226], [0.072584302435739517, 0.1608936857385721]]]])+(1.-msk_ref)*numpy.array([[[[-0.31783800855449895, -0.50134985241397934], [-0.10606620625982914, -0.50423019838010108]], [[-7.3675922183876708, -5.3569894577285675], [-0.94216342703054956, -2.2639127645982637]]], [[[-5.0710151180498286, -1.9299697012575769], [-0.5773940711404516, 0.94037049231846048]], [[-5.6724283066377241, 0.082754396477554865], [-4.545963895954575, 0.94648883803209216]]], [[[-0.39493267494812762, 1.2795440121985513], [-0.93540503315079826, -0.6652070868173483]], [[-0.33153716475920114, 0.12267236856379167], [-0.28758726780928151, 1.0069797691820557]]], [[[-3.9776318526441243, 9.3694230357513195], [-1.6435557410906367, -2.7218532730904377]], [[2.8371527878847549, 0.046564089654682027], [0.16238640381070865, 1.0804673836192515]]], [[[-2.2389357624863235, 1.318912531142723], [0.16093394854943741, -1.3622082920293634]], [[-11.467345531784751, -21.553209635869653], [-0.23680688648507411, 1.1441956133063578]]], [[[-1.8678731929107615, -1.2671703582426779], [-3.1618435674585665, 1.0504329307774707]], [[-1.6429163322941909, -1.5291646690008422], [1.0189251823215566, 0.57446290302900893]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.3245147118315943, 4.1788340792633942], [1.7723589344589534, 3.7326090703270118]], [[-3.3513393817868566, 0.47727189660178837], [0.00060996163195792263, -0.26966392984855503]], [[-3.7919952380077304, -2.5656980752066305], [-4.8677707043674943, 4.8840197540168777]], [[-3.4601386308782356, 0.7665800542552681], [-3.9989588681508192, 0.21738893103738466]], [[0.8863412820127694, -4.238602088151719], [2.8740805329500718, 0.94225674343480037]], [[4.2741618397812022, 0.97940392133744503], [-4.6124283559599233, -1.3043541923021906]]])+(1.-msk_arg0)*numpy.array([[[-4.2931545073126571, 4.5711942067336295], [4.0074193030150909, -3.0615734862965049]], [[-4.488385296244112, 1.0220317303843576], [2.1733711916499177, 2.0755568693247959]], [[-3.9711339151550029, 3.2073423224583646], [-0.84279576012989033, 0.37602654794122614]], [[-4.4347094579829252, 0.52822688632702341], [4.0780314703450138, -3.0859192910829658]], [[-0.12525235156653203, 2.2536122680385446], [2.8198048651028582, 2.6792932446083393]], [[2.2441730407287555, 1.0432349288764229], [-4.8042657644362272, 3.9305290026674573]]])
      arg1=numpy.array([[[[-0.13928645071565349, 2.4609898807856503], [-2.7576965385414551, 4.5151449027951003]], [[3.0562134951025381, -4.0825182234313253], [-4.8125404605383961, -4.1169043470735645]]], [[[1.9371209449331417, -1.5222152917137799], [-1.1001688223410833, 2.06692120112164]], [[2.8836454196072481, 3.3114023000270176], [2.5900274366928908, 1.3746650119258401]]], [[[3.616810084090794, -4.224306753864326], [-4.6982149515403293, 0.27984479525538841]], [[1.3544160820892888, -0.4820144559663353], [2.4731718155135409, -3.4721400849510853]]], [[[2.301023880524081, -4.4752201896111119], [1.62600997580412, 2.9103987415091126]], [[0.58341313332390232, 4.3082361250252674], [-4.4716156729047434, 4.2043649200337843]]], [[[-0.42800834490025164, -1.3115239506439602], [2.4860000057779716, 2.6396482279438693]], [[-3.6114316938916313, -1.7337792677469608], [-0.80259170425121162, 2.9832091831557523]]], [[[2.2819225040568094, -0.30888584022883325], [-1.4758454896771642, -3.7419853770896316]], [[3.5329928219517335, -3.4866967504454505], [-3.5984356978575893, -2.3734460889190281]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-16.688735335621253, 0.94454460377119165], [-1.5153349981987425, 0.92551494342440421]], [[0.57991987055193917, -0.43413374722655845], [-0.77560055877627498, -0.90665431004737262]]], [[[-1.7300620235163096, 2.2016198365828825], [-0.43381696236963591, 0.23090957523818081]], [[0.00021152449181529375, 0.00018420040112702282], [-0.10411624449541694, -0.1961670134244333]]], [[[-1.048436370681314, 0.89766095573880278], [0.54610061516352204, -9.1682894186584942]], [[-3.5939994871137322, 10.098806465479674], [1.9748000213251407, -1.4066309637635726]]], [[[-1.5037386878793082, 0.77317729279794722], [0.4714485554593027, 0.26339348052967398]], [[-6.8544203750906245, -0.92821255662428803], [-0.048615298571974476, 0.051705533456795613]]], [[[-2.0708504695611327, -0.67581021419972886], [-1.7049887684233074, -1.6057450546936478]], [[-0.79582857341903657, -1.6576969089525035], [-1.1740175464608011, 0.31585339330379952]]], [[[1.8730530209429037, -13.837351160593041], [-0.66362226140060643, -0.26173376500449785]], [[-1.3055300671151311, 1.3228647875301036], [0.3624781159987846, 0.5495613312608465]]]])+(1.-msk_ref)*numpy.array([[[[30.822484780496872, -1.744482795655423], [-1.6576132082869903, 1.0124136224075182]], [[1.3112367017019009, -0.98160475561744964], [0.63616576554537596, 0.74365912544768631]]], [[[-2.3170392679838714, 2.9485877067959825], [-0.9289771802563398, 0.49447058254070819]], [[0.75368877770899112, 0.65632955308154051], [0.80136481950747085, 1.5098637495814633]]], [[[-1.0979658380800164, 0.94006760080154583], [-0.68267253745102152, 11.461146953014868]], [[-0.62225764392121985, 1.7484864814692458], [0.15204222593129715, -0.10829820765901602]]], [[[-1.9272765900077822, 0.99094776795067441], [0.32486079063924339, 0.18149639731259812]], [[6.9899548663073228, 0.94656637937204435], [0.69011281756206144, -0.7339798875160839]]], [[[0.29263997550263277, 0.095501383337325205], [0.90652142510084055, 0.85375477087489648]], [[-0.78079972268955578, -1.6263920774454659], [-3.3383016924003979, 0.89812449617565226]]], [[[0.98345716681397255, -7.2653801128151256], [-0.70687272900405496, -0.27879182405779729]], [[-1.3598289061289979, 1.3778846020441691], [-1.0922882420846334, -1.6560430932128707]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.1239526048911062, 0.9453973539558973, 4.6508427661240734, -3.2888290967333864], [-0.12079452097078747, -0.96707317264993442, -2.7030427381207955, -0.87789477031229168], [3.065183990557367, 1.5541814880188785, 3.0266676955292056, 1.7512608489474255]], [[3.1808174907730766, -0.18086748921125739, -4.9631547823009186, -3.2458221110308294], [0.068442620355504857, -3.7245151777844687, -3.9372818612291525, 4.00380841601004], [2.8303850836941349, 0.01944455118124111, 3.3097338438642137, 1.431485490144917]]], [[[3.7701312504563234, 0.06684075767822506, -0.9062074478581934, -4.4723034597412683], [3.7637874031821816, -0.66423165514632654, 3.6243416191041362, -0.67892997810624323], [-1.2016764933911217, -3.4032589253040868, 1.3196999365294193, -0.89456234960912617]], [[0.56039208512841032, 3.7040534120860986, 4.224331822502311, -4.8282730612234417], [3.4538590940505483, -2.6850276953824648, 2.760228174480595, -3.6409496425912913], [-1.5023759526033587, -0.42559295083487569, 4.2556677636523368, 0.43255216603838331]]], [[[-1.054126427159213, -0.26964077431045208, 4.3960945219076866, 1.0414299835740861], [4.9936686572427735, -3.180746432941719, -0.90974739677105276, -0.38694233156532043], [2.006471114019841, 2.6645316321811556, 4.7384301441696781, -2.1942963154316319]], [[1.0713955230669381, -4.8269030119680876, -0.58539117707223021, -4.0358961759034457], [-4.2414839336410193, 2.3932424440785045, -2.4983438650524024, 1.8162189855469233], [2.5867148148907919, -1.2544024484468954, -3.7864409609826941, -3.2312708527594634]]]])+(1.-msk_arg0)*numpy.array([[[[0.86725983704767007, 3.5132728560119375, 0.61199516863365488, 2.19500211285375], [-0.50454541888794324, -3.116347955707556, -4.0416688696085501, 4.7929697908845981], [-1.4606865313547992, 2.3447083016883337, 1.7846646686906054, 3.9239689421199397]], [[3.7083699240279842, 2.2316982153447658, 1.2519206174587509, 0.6089266396992894], [1.7014723473579263, 2.4219508746504328, 3.9286217115441229, -1.7306697735198662], [0.19228795109517982, -3.3735654555994676, -0.30892788987735198, 2.7733621297872935]]], [[[-1.818783072728726, -4.2858856510086385, -2.6654755975815068, -4.3058236238568979], [-4.0709183560878657, -4.6484156808429695, 0.53440873143246215, -0.36745863315202243], [0.68497269905464453, -2.7163230603196764, -1.4457222267119976, 2.9188250580581112]], [[4.1415701736173673, 1.3359959221952211, 2.7761132713547667, -0.50303319815129832], [-0.31512859089411727, 1.7631070339779455, 0.12161473294471836, -1.8263358322743963], [4.248077504115912, -4.9372508774499604, 2.2976875304811166, 3.7445950807652899]]], [[[-2.4850479573306758, -3.1957407146418393, -2.2865393742375328, -4.8841931172212476], [-1.9694786887225781, 4.8620414151053186, 4.0334332427738158, 3.7061269368629741], [-3.7275635496155104, -4.6848871462953303, 1.4136773956718596, 4.6436147449490512]], [[3.0010387754565553, -4.1541035579465824, 4.9709521668566197, -1.6218697804962954], [-4.3485283544392184, 3.2400317533609204, -4.3510096075555014, -4.8434408940478484], [-2.9535936421418829, 0.50733166270389596, -4.3269886283345906, 0.98602373728230752]]]])
      arg1=numpy.array([[[[1.4648706152716127, 2.53841732963369, 0.62585927176136913, -4.6250963699298833], [2.690183583130354, -4.978630665215694, 1.3253711102513197, -0.96290477077474357], [0.99664713477766931, 1.9775521019226421, -1.4800511698196761, 1.0991408982923021]], [[-1.3121136750686113, 1.3876980232666174, 4.0562809918834368, -1.2450124698317255], [-0.54998827649775883, 4.774024977609475, -0.069246591428083093, 0.99739266765939494], [-1.1621186716978613, 1.9719939407284146, 4.9928488758244196, 4.7517316273386943]]], [[[-4.2421114685978232, 0.23690774261651093, 4.4273269057990099, 1.6660737195772697], [-1.9248453855148284, 4.7180359548516471, -2.9305308308618416, -1.0747721158797177], [4.8471288948219087, -2.229561808311141, 4.2496491762785702, -2.1004675155979422]], [[1.7003389830101412, -2.9031665545045939, -4.0644445019395281, -2.6046070171023414], [0.2309642785613244, -2.996236892539347, 1.8511652991592626, 3.0877328599740537], [-3.1699643832222026, 4.6882458746342692, 4.2626164926183172, -4.0544092417770559]]], [[[-2.7635331006879182, -4.1434725262896546, 3.9746220646781776, -3.1414011208104564], [4.4257738106099023, 4.8992375599926579, -2.7751714026473384, 3.5556292864356855], [4.3933736216294808, 2.3962518961237631, -4.1187265995789648, -3.7957735963753603]], [[3.8661800654562537, 4.1176512755320154, -0.23645841714736981, 2.4147878085966914], [3.8578109361798116, 4.2580028654198081, 2.1779053714316996, 0.30861649512704936], [-0.62537987416518614, -3.2287847160424787, -3.1122906548505158, -2.0579371196065219]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-2.1325792000489208, 0.37243574684085701, 7.4311318469967009, 0.71108336641712944], [-0.044901961980686994, 0.1942448110093013, -2.0394610363947336, 0.91171504904472156], [3.0754957131755001, 0.7859117777518233, -2.0449750368414379, 1.5932996867538094]], [[-2.4241935368951548, -0.13033634564492527, -1.2235727239390279, 2.6070599208291712], [-0.12444378049535344, -0.78016248244462827, 56.85885442200091, 4.0142749649502356], [-2.4355387729541662, 0.0098603503690577708, 0.66289485746105425, 0.30125554269710514]]], [[[-0.88873931728684463, 0.28213834187100434, -0.20468500906748563, -2.6843370777590914], [-1.9553712893025437, -0.14078562806696807, -1.2367525981763008, 0.63169668069638041], [-0.24791511005099304, 1.5264250188614432, 0.31054326646442948, 0.42588725746347483]], [[0.32957668484218244, -1.2758666588862555, -1.039338049882705, 1.8537433975721056], [14.954083443399227, 0.89613331378043048, 1.4910760134355361, -1.1791660119916871], [0.4739409567359949, -0.090778718142225476, 0.99836984420765651, -0.10668685380383428]]], [[[0.38144157813662966, 0.065076037695345035, 1.1060408890130879, -0.33151767110384345], [1.1283153796227583, -0.64923294573747625, 0.32781665157806511, -0.10882527406371094], [0.45670395619019782, 1.1119580693880173, -1.1504599855338935, 0.57808935641656756]], [[0.27711992326474871, -1.1722466738867863, 2.4756622501934129, -1.671325389972393], [-1.0994535511999814, 0.56205749965895058, -1.1471315043454136, 5.8850353569054477], [-4.1362297089329489, 0.38850606614132405, 1.2166090448787334, 1.5701504297552509]]]])+(1.-msk_ref)*numpy.array([[[[0.59203852408962743, 1.384040683538402, 0.97784789048711185, -0.47458516261943884], [-0.18755055307446475, 0.62594479592161978, -3.049462024898189, -4.9776155818900163], [-1.4656004922751793, 1.1856619602632619, -1.2058128158555776, 3.5700326939125611]], [[-2.8262565923139782, 1.6082016245086135, 0.30863754753771427, -0.48909280385086523], [-3.0936520287171243, 0.5073184338183313, -56.733791953127998, -1.735194000955782], [-0.16546326616906185, -1.7107382461597935, -0.061874071809622275, 0.5836529390319487]]], [[[0.42874476217615798, -18.09094799382019, -0.60205077562495068, -2.5844136266367661], [2.1149326521096334, -0.98524380172705428, -0.18235902035376217, 0.34189446090276715], [0.14131514014128801, 1.2183214881928972, -0.340198017940394, -1.3896073309313743]], [[2.4357320599010617, -0.4601857651336852, -0.68302403195074324, 0.1931320905028234], [-1.364404023241395, -0.58844046622885382, 0.065696311939269655, -0.59148116598718425], [-1.340102597555948, -1.0531126159920348, 0.53903219641271538, -0.92358586848623747]]], [[[0.89922858413097351, 0.77127112448927504, -0.57528472821545418, 1.5547817452745942], [-0.44500211104353077, 0.99240776867178593, -1.4533996851243765, 1.0423265864642919], [-0.84845129748673076, -1.9550895938251402, -0.34323166675262501, -1.2233645203136738]], [[0.77622840236294022, -1.0088526880921593, -21.022521536031999, -0.67164070264161824], [-1.1272010024279051, 0.76092756528511096, -1.9977955262102405, -15.694044130900814], [4.7228792677164551, -0.15712774536598165, 1.3902906599005989, -0.47913210169941223]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.722727061511)+(1.-msk_arg0)*(0.0238543021523)
      arg1=Data(-2.66864849022,self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.270821377997)+(1.-msk_ref)*(-0.00893872019477)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.1239088628536575, -1.9106996908385443])+(1.-msk_arg0)*numpy.array([-0.056345512966209377, -2.9219843282411473])
      arg1=Data(4.29407641708,self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.26173471398478082, -0.44496173455084886])+(1.-msk_ref)*numpy.array([-0.013121683801915912, -0.68046863735764496])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.25036899361)+(1.-msk_arg0)*(2.31122467868)
      arg1=Data(numpy.array([2.0107771846430245, -3.0742833657818105]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.62183368856595045, -0.40671884951400766])+(1.-msk_ref)*numpy.array([1.1494185911442159, -0.75179298837651498])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.4846419689310064, -4.5496266730071326, 0.66785772060010462, -3.1979716193125043, -2.2167077313472259], [-2.906398867162272, 1.5731233970826519, 0.97102336728721284, 3.5349046692979833, 4.0935488902635999], [4.1954572821446234, -4.8068604158129782, 2.0035402095681265, 2.4872065843424593, 4.7554721435699818], [4.9336367055627601, 2.1127856942863277, 4.4410334518794521, -3.0071055564487259, 2.5796556826569876]])+(1.-msk_arg0)*numpy.array([[1.0326952659586262, 3.4124047329049194, 3.1426086644393312, 4.335209282361312, 0.094882809730605366], [4.8472179005957887, -1.3633586134948938, 2.2463443301559636, -4.5792744058382215, -2.0982750198612035], [-2.4606211642184048, 3.9042990691355524, 4.1381763474008828, 1.0577477731649099, 2.4488027175569238], [-0.78021773318382692, -3.0124451889190631, 1.0007431131121871, 0.64394308277439638, 1.0392467013084214]])
      arg1=Data(3.33006882689,self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.3467114951855943, -1.3662260179940704, 0.20055372886210587, -0.96033198878349413, -0.66566423896346838], [-0.87277441345776163, 0.47239966464962008, 0.2915925819450027, 1.0615109936331382, 1.2292685536135013], [1.2598710417837591, -1.4434717916335704, 0.60165129122562078, 0.74689344684380954, 1.4280401969986332], [1.4815419626540918, 0.63445706503908594, 1.3336161150843096, -0.90301603743660508, 0.77465536502640997]])+(1.-msk_ref)*numpy.array([[0.31011228885719372, 1.0247249862677905, 0.94370682043077603, 1.3018377420177702, 0.02849274734637788], [1.4555909059464447, -0.40940853909270314, 0.67456393454015784, -1.3751290570519097, -0.63009959521505265], [-0.73891000220465208, 1.1724379501136837, 1.2426699154042635, 0.31763540880113977, 0.73536099247666975], [-0.23429477699794307, -0.90461949752944792, 0.30051724607956848, 0.19337230437243622, 0.31207964619734674]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.225666951027)+(1.-msk_arg0)*(-4.31177999232)
      arg1=Data(numpy.array([[-2.3148368642456219, -1.9841452556837078, -1.2575270453724352, 0.85886299798927013, 2.7049313219616566], [-0.87470171469411717, -2.4065420338525811, -3.6964781464899121, -3.6304376835826311, -0.67247775024212775], [-2.434369200157299, 3.3700424886481901, 3.2283587907151521, 2.2754260366990673, -0.0059944217916720888], [2.0232661614129466, 0.93893118872291126, 0.12285676613780527, -3.0000439775108969, 0.90972311031430841]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.097487194243661021, -0.11373509594656089, -0.17945296036179126, 0.26275081305798598, 0.083427978076516585], [-0.25799303606717605, -0.093772287312112623, -0.061049177645318448, -0.062159709295548302, -0.33557534200328265], [-0.092700380456883211, 0.066962642692858412, 0.069901447037462011, 0.099175691667162419, -37.646158190038271], [0.11153596858927634, 0.24034450419528758, 1.8368296522958718, -0.075221214328442434, 0.24806113911861863]])+(1.-msk_ref)*numpy.array([[1.8626712140784416, 2.1731171041870865, 3.4287771449407898, -5.0203350271375795, -1.5940441656725532], [4.9294289926317942, 1.7916911201485601, 1.1664562379226533, 1.1876749769914585, 6.4117808964913046], [1.7712103784583355, -1.2794438072641303, -1.3355950412694904, -1.8949330467242835, 719.29873174893714], [-2.1310987523790725, -4.5922215004728937, -35.095991273948336, 1.4372389287090921, -4.7396619294723745]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.2115680619734661, -4.8939811685524166], [-1.0228040015921103, -1.9767939029922466]], [[-4.0608145956116406, -2.9981362165315648], [-2.1572452502171102, 2.7527206712837984]], [[-4.4091434796252109, -3.4132318680344023], [0.71599568783169332, -3.9087961621284082]], [[1.6749464899511253, -1.7571133467023081], [4.2482726923641874, -3.6997701571669648]], [[-0.5243786340066583, -2.9466464182876937], [-0.55875743427206892, -0.48427065061653707]], [[2.2539247448090949, 1.8263662588419454], [-1.6978704018453761, 0.63453759803115961]]])+(1.-msk_arg0)*numpy.array([[[3.4088129911802252, -4.4901035588651421], [1.4340203227775641, 1.2762689960014137]], [[-2.52202022824767, 3.8439778672079292], [-2.0425364343290306, 4.8846561464970879]], [[3.1090729612459498, 1.6618784212165814], [4.2238849116585957, 0.92327834920562424]], [[3.7295890660439852, 1.7851003754496961], [-1.6214872839581673, -4.7821157646423611]], [[3.1843490310755964, 3.4447955384360611], [1.649091988174809, -0.12250810051865813]], [[1.280622801172683, 1.4678727215161071], [-3.0661236312377804, -1.4992217354928474]]])
      arg1=Data(4.84688364917,self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.24996846420703348, -1.0097170724099049], [-0.21102301512169383, -0.4078484333599256]], [[-0.83781969808707135, -0.61856987572712552], [-0.4450788189615536, 0.56793619788141536]], [[-0.90968626415847786, -0.70421163681496002], [0.1477228957113815, -0.80645553825021776]], [[0.34557183773899575, -0.36252435046647496], [0.87649570319080594, -0.76332968252698552]], [[-0.10818882233666632, -0.60794659652943372], [-0.11528179232600999, -0.09991381796411844]], [[0.46502555207707202, 0.37681248221304198], [-0.35030145651129113, 0.13091661446003242]]])+(1.-msk_ref)*numpy.array([[[0.70329994237910676, -0.92638979679924538], [0.29586440000944209, 0.26331744031439547]], [[-0.52033851249534557, 0.79308234846226711], [-0.42141230988252953, 1.0077931512411096]], [[0.64145813811266028, 0.34287565815631094], [0.87146406173397772, 0.19048906803519672]], [[0.7694818642246255, 0.36829858207035815], [-0.33454223400545019, -0.98663721079012223]], [[0.65698895652678102, 0.71072379445816181], [0.34023758512479524, -0.025275642946289974]], [[0.26421570928192345, 0.30284876381696463], [-0.63259691240209159, -0.30931663394669262]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.21966752618)+(1.-msk_arg0)*(-1.46987298445)
      arg1=Data(numpy.array([[[2.7062569098601861, 1.1995523283177807], [-1.9908225229494692, 2.0209448554051557]], [[-3.749684469855199, -2.4718875429628726], [1.9944111107767366, 3.8825765577220555]], [[2.5996259412281342, -2.0436623996557857], [0.39945524655416431, 2.8245011712541341]], [[-2.4761592863610926, 2.0122441253580501], [-1.1575427811302639, 1.0483821055816742]], [[4.5701879037877831, 1.4454863250801555], [3.1690153427279348, -0.21501760484490795]], [[-1.1108980477382113, -3.1924930804984264], [-0.43827357688490576, 0.91851647640227707]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.1897124454242354, 2.6840575856254389], [-1.6172549230586124, 1.5931496188848373]], [[-0.85865025499074321, -1.3025137552644543], [1.6143449606646423, 0.82926053828156987]], [[1.2385118470755596, -1.5754400172544571], [8.0601457959306693, 1.1399065997718814]], [[-1.3002667251296045, 1.6000382287626769], [-2.7814674141306504, 3.0710821074055121]], [[0.70449346809297009, 2.2273939713663209], [1.0159835715419603, -14.973971682450562]], [[-2.8982565346399158, -1.0085119826393882], [-7.3462505977664891, 3.5052909870349436]]])+(1.-msk_ref)*numpy.array([[[-0.54313874602785761, -1.2253512829340225], [0.73832447016573888, -0.72731968936183999]], [[0.39199911252997993, 0.59463586385036593], [-0.73699598668914079, -0.3785818418769506]], [[-0.56541710910773502, 0.71923473500234547], [-3.6796937757866659, -0.52040091164063274]], [[0.59361002846096311, -0.73046454251128379], [1.2698217365369882, -1.40203936773144]], [[-0.32162200228845123, -1.0168709028562954], [-0.46382640204745684, 6.8360587753309998]], [[1.3231394072960594, 0.46041540181543733], [3.3537796070162975, -1.600268500580424]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.1477836107364769, -2.7678509997319898, -2.0473318303845636, -4.029118818525018], [-2.2251289022458351, -2.8643755991470563, -1.56227163177563, -4.6459234843574659], [-1.6680593053283097, -0.049529627114981878, -1.2453530334451921, -4.6884000626882791]], [[4.7042293616242343, -2.3137649285360196, -3.1018993308823442, -1.4902463241728392], [-2.7857814207176435, 1.7880202663834455, 4.3098348754761453, 0.6089072417888497], [-1.3447447897159259, 3.694977761783198, 3.1778937869842938, -4.3632414381648852]]], [[[1.5001421936573678, 3.9632667995560578, -3.0584024088162778, -3.9644526088848542], [2.9821505326345878, 4.5970770100029377, -0.44573447118713005, -0.28834319339189474], [-3.353076235723508, 0.096379182044527667, 0.65374317010699023, -3.7055448315580417]], [[-2.8045474658976257, -4.5138990077727854, -2.9222529895881322, 2.7296994263907246], [-0.44032083005939349, -0.78030829578420224, -3.5044512308130757, -3.914384889488387], [-2.2616126865296238, -0.81414563796896999, 3.0980153468468004, -3.8083707709281409]]], [[[2.0595571681668288, 2.821175923661789, 4.1942315632971052, -0.65163888255575753], [-4.8912817605860344, 4.8533690688298581, -0.29377166913585029, 3.62159967270712], [3.4762058258133344, 2.7035052109168776, 3.8861426043835312, -1.5976914973674958]], [[2.7132515266831323, 0.81526601609963389, 3.6731437509855294, 3.0355124819413994], [3.4853207945073201, 2.5064849594258938, 2.4121291089850336, 1.8356228491876543], [-0.90945179570080192, 0.70585094508185442, -3.8230102027940949, 1.484635266367488]]]])+(1.-msk_arg0)*numpy.array([[[[3.0205567978950381, -2.8144127127104923, 3.7360353645080711, -0.19814917671570864], [-1.9973110573263897, -1.5802362500155978, -2.0013773305931681, 0.30703657877921398], [-4.8594892582758202, -0.71444507098194965, 3.3702356371722484, 0.16558921125379555]], [[-0.0039544518558418673, -2.8462252464710778, -2.0563778480971284, 3.9506935088098576], [1.2227107751098787, 3.810802986493659, -3.379055242910709, 2.2161955784493923], [-2.7338107598548067, -4.6258339479666368, -1.7321271055658114, -2.0818981239768664]]], [[[-0.58398464609228018, -4.6363835076956095, -4.2303700032279137, -1.2945885310908931], [1.7118440220026887, -0.69079531522001147, 0.71686130397969094, 2.9725057074550731], [-4.7314205714035396, -2.8906210930096243, 2.5132228896412059, -3.558904359498877]], [[-0.25825951791243185, -1.3264505629586898, 1.5476187744372005, 4.0597456257927043], [-0.045157982217193471, -4.0429028282861417, -2.1021380389173139, 1.6350674236973459], [-4.2705672593307327, -1.815819566273642, -0.69434152654470704, 0.93159742972198334]]], [[[2.8664257443705701, -1.8562112510104667, -0.093478423795709809, 3.6133959212321756], [-3.3103788905031029, -1.347529607003878, -0.76770546052661892, 1.7583659129380003], [1.479948148559326, -2.6517098199745712, -0.68482210895006901, 4.6901520197211326]], [[2.2131760830986122, -1.4684114443559837, 3.2091995798652935, 3.6113403843469545], [1.9715828934117097, 0.24887238789461996, 0.63314904049481591, 4.0487700779560711], [-3.9026772524552888, -3.68462292658982, -4.4659880097252538, -4.353727562078924]]]])
      arg1=Data(4.12689727176,self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.76274823516432977, -0.67068570343990586, -0.49609469186359367, -0.97630703000525476], [-0.53917719674639952, -0.69407484861579061, -0.3785584008760246, -1.1257666906693975], [-0.40419210740821154, -0.012001662230352872, -0.3017649704944686, -1.136059308957116]], [[1.1398949505768601, -0.5606548397439115, -0.75162988720637158, -0.36110574749991048], [-0.6750304738097348, 0.43326018280619411, 1.044328121509593, 0.14754601379494467], [-0.3258488644530016, 0.89534037764183605, 0.77004431603697143, -1.0572692148230241]]], [[[0.36350364326345525, 0.96035024343350228, -0.74109002657941758, -0.96063758020268042], [0.7226132215705926, 1.1139305650917568, -0.10800716418062309, -0.069869244229865485], [-0.81249326429118085, 0.023353908686831723, 0.158410332765274, -0.89790091382168313]], [[-0.67957772661111038, -1.0937754711427483, -0.70809928068420669, 0.66144108918639177], [-0.10669536968435911, -0.18907868173131315, -0.84917336198238236, -0.9485055313292754], [-0.54801768437701681, -0.1972779025881892, 0.75068874819083919, -0.92281695427508248]]], [[[0.49905704759411634, 0.68360701463772688, 1.0163159601771043, -0.15790043697367431], [-1.1852201396097828, 1.1760334094200964, -0.07118463334340297, 0.8775599280100348], [0.84232913903720541, 0.6550938957020479, 0.94166206437453881, -0.38714108739799696]], [[0.65745555268657163, 0.19754938454106738, 0.89004971752603979, 0.73554350449089034], [0.84453781254995963, 0.6073533684930571, 0.58448973893614575, 0.44479489754939688], [-0.22037180375798401, 0.17103671320160857, -0.92636427588304782, 0.35974611641731669]]]])+(1.-msk_ref)*numpy.array([[[[0.73191955093413097, -0.68196820210958786, 0.9052891599887738, -0.048014080232093977], [-0.48397401868849221, -0.38291145767802326, -0.48495932871658703, 0.074398890634018541], [-1.1775164096119954, -0.1731191798428095, 0.81665120676437419, 0.040124384095301661]], [[-0.00095821426981125622, -0.68967678598404136, -0.49828665767153235, 0.95730357424882206], [0.29627846166124089, 0.92340631121938144, -0.81878831005473729, 0.53701253811591187], [-0.66243731787670102, -1.1208987390176035, -0.41971655495769922, -0.50447054697122362]]], [[[-0.14150695004912864, -1.1234550322893486, -1.0250727664533676, -0.313695361391926], [0.41480170435021324, -0.16738854149525634, 0.17370466400648801, 0.72027615705349823], [-1.1464837285349152, -0.70043446750967198, 0.60898605517547721, -0.86236805162451857]], [[-0.062579584832392454, -0.32141593929093393, 0.37500782610434047, 0.9837282971827277], [-0.010942356749766454, -0.97964707189470512, -0.50937493726922467, 0.39619775245860928], [-1.034813075808273, -0.4399963087767752, -0.1682478338621794, 0.22573797416710537]]], [[[0.6945716250287981, -0.44978373067682081, -0.022651017856795751, 0.87557205408569705], [-0.80214715136207559, -0.32652366130510913, -0.18602485353359299, 0.42607455362942853], [0.3586103678150338, -0.64254320991280223, -0.16594115720710792, 1.136483830557119]], [[0.53628087576719163, -0.35581487680964879, 0.77763011011409844, 0.87507397120408792], [0.47773975545864161, 0.060304963149401702, 0.15342011172121808, 0.98106878154327914], [-0.9456686211126768, -0.89283126861597173, -1.0821660234408568, -1.0549638809463915]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.73517827026)+(1.-msk_arg0)*(2.43707402572)
      arg1=Data(numpy.array([[[[-3.3629266374322819, -4.5045267892089091, -0.90479343290543923, -4.4526561179065194], [2.4595335265295617, 1.2121725726657582, 3.4414574408981498, 4.5406329544499187], [-0.4082593301600479, 3.7358579288005309, -1.9491687462731977, -4.9324041927492459]], [[3.4597036989084913, -4.3546905578070589, 4.7576710841934471, 4.7895516040491479], [2.3469034212518132, -1.5132113609238496, -1.3258900228314499, -0.27692588188111511], [1.2610590885245552, -1.8587338335463088, 1.2965551786806442, -2.3078995889164977]]], [[[3.8277611606825008, 3.3824136839936418, -1.6774330203814722, 4.8999535783030428], [-2.6162869799956301, 3.7194927044883812, 1.9182103239656509, -3.1865476856537791], [1.3540773334837297, 2.5466159122983845, 2.7921914588884427, 4.4112823762965512]], [[1.0022584100642398, -4.3002641798023458, -3.64430946315042, 1.4264353990576133], [-3.8558230774823166, 1.8265192792835681, -4.0452937966213298, 3.6969297237714436], [0.26059383426548255, 1.4820772028613334, 3.7169512221438143, -3.1399977187185915]]], [[[1.1483579170413574, -0.45722207833129147, -3.5220241559034751, -4.9798627992904843], [1.9029583944118302, -1.3603982332669586, -4.1618212703173585, 4.9917048496126419], [3.5295910151271599, -2.1156917233614534, 3.1985259440043041, 3.6691366770994041]], [[-1.1118862555253073, 4.365099795742772, -0.63867104527805907, -0.71052740617816568], [-0.77971323733064501, 3.2053788789795963, 3.2382317049543978, -4.0183703388834875], [-0.64798645276979183, 2.9285276931573359, 1.4312467118655245, 2.6907575526427134]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.4080528006623332, -1.0512043754744957, -5.2334357191928964, -1.0634502519104934], [1.9252342849499164, 3.9063565510685372, 1.375922367653253, 1.0428454177556699], [-11.59845696215241, 1.2674942036082211, -2.4293321341788019, -0.96001424157803494]], [[1.368665840300832, -1.0873742249651452, 0.99527230581162107, 0.98864751060523226], [2.0176280912883895, -3.1292246361192757, -3.5713205384459203, -17.099081667965255], [3.7549218060817968, -2.547528960197686, 3.6521224457850967, -2.0517262938989203]]], [[[1.2370621027499509, 1.3999406082900023, -2.8228717407635386, 0.96637206752852123], [-1.809884888952968, 1.2730709928653601, 2.4685396648627771, -1.4859900862544217], [3.4969777228867436, 1.8594002524647977, 1.695864463443183, 1.0734244299803495]], [[4.7245083929552516, -1.1011365981882841, -1.2993348446772786, 3.3195883061979692], [-1.2280590097381623, 2.5924600544680856, -1.1705400171956251, 1.2808407581596841], [18.170722586761119, 3.1949606006466058, 1.2739414609609521, -1.5080196530177341]]], [[[4.1234341662904672, -10.356407738533347, -1.3444479823687672, -0.95086520675492125], [2.4883246444923164, -3.4807295058646819, -1.1377658872644012, 0.94860942562057504], [1.3415657083113619, -2.2381229826503932, 1.4804251561988002, 1.2905428952300111]], [[-4.2586894538249025, 1.0847812173450531, -7.4141113884330334, -6.6643147457582161], [-6.0729740673243029, 1.4772600834525649, 1.462272839529613, -1.1783827449747117], [-7.3075266466114526, 1.6169142881327325, 3.3084290996110175, 1.7597937300623225]]]])+(1.-msk_ref)*numpy.array([[[[-0.72468843018742901, -0.54102775713412765, -2.6935142730760626, -0.54733039363078306], [0.99086839005404292, 2.0105008813693748, 0.70815172570614826, 0.53672561736748714], [-5.9694264054195534, 0.65234654854735308, -1.2503145406861076, -0.49409454912451167]], [[0.70441697839186834, -0.55964344500830721, 0.5122409646629531, 0.50883135357722342], [1.0384210971989551, -1.6105311449871755, -1.8380664940157678, -8.8004559529128112], [1.932561327136286, -1.3111473960026627, 1.8796531499694604, -1.055970561899721]]], [[[0.63668393178474914, 0.72051329417483245, -1.4528592176885036, 0.49736675802555247], [-0.9315010334685685, 0.65521677802336509, 1.2704936446582955, -0.76480073927287828], [1.7998041658715742, 0.9569853129193594, 0.87281766368788927, 0.55246384561813322]], [[2.4315825152917254, -0.56672658325570724, -0.66873410459755422, 1.7085064120859335], [-0.63205027220999777, 1.3342722704094341, -0.60244673149617922, 0.65921567565804162], [9.3520018713623756, 1.6443637490754306, 0.65566478548262086, -0.77613878863289121]]], [[[2.1222251264616134, -5.3301757312568965, -0.69195267205404143, -0.48938577706669278], [1.2806764629604761, -1.7914416279883898, -0.58557873282493245, 0.48822478474571818], [0.69046923999733989, -1.1519041261097422, 0.76193661342176378, 0.66420911516517611]], [[-2.1918375315871685, 0.55830889092000546, -3.8158517498711051, -3.4299507725183931], [-3.1256029897089261, 0.76030763217985331, 0.75259408460106147, -0.60648318103836973], [-3.7609953345461395, 0.83218404641029875, 1.7027630565101828, 0.90572040699940803]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.970503366115091, -3.3038063594549607])+(1.-msk_arg0)*numpy.array([2.4102274346736374, 3.8594884915118186])
      arg1=Data(numpy.array([-2.7309000390423255, 0.10318857543888171]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.4539175031494271, -32.017171914654405])+(1.-msk_ref)*numpy.array([-0.88257622037270111, 37.402284846909069])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.8428396164187522, -1.3157562143036774], [1.277496576322326, -1.6068816105308681]])+(1.-msk_arg0)*numpy.array([[2.9448292127849509, 0.014224504973821439], [1.0017953120543019, -0.49995507433514952]])
      arg1=Data(numpy.array([-1.5246393513093848, -2.9875612149783115]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-3.17638372134344, 0.86299505071391791], [-0.42760515497306728, 0.53785730062188308]])+(1.-msk_ref)*numpy.array([[-1.931492329812873, -0.0093297506466727392], [-0.33532210387246392, 0.16734554988483441]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.9153217144642465, -3.0117748541522262])+(1.-msk_arg0)*numpy.array([-3.1066686337393792, -4.9948775747994407])
      arg1=Data(numpy.array([[-4.2579318146351319, 0.83295442773917294], [0.70389119865051875, 4.0701028336449845]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.68468022537229489, -3.4999774505996566], [-4.2787505511168771, -0.73997512526115428]])+(1.-msk_ref)*numpy.array([[0.729619159954912, -3.7296982047044058], [-7.0960932376700914, -1.2272116403325057]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.235670502717924, -1.6992513165019507, -0.69720144954481622, 1.1795042383094323, 4.6375267379182983], [3.4856801219385769, 2.4151720187621404, -3.5466448437169493, -4.1499982496596957, -1.2698510460271528], [-2.9173587821354752, -3.0981973599312185, -4.7139123205894187, 2.0811078027564678, -3.6552994889710924], [-4.3064964283854055, -0.83073388215256649, -1.1411127156115386, -4.7647949392300459, 1.3752218012278021]], [[-0.39149456763861057, -2.5617898258489422, 4.4410135720793562, 4.150548860086662, -3.063215107925592], [3.0268558692330689, 1.8579842775446762, 4.9491317921190934, 0.97270665126512412, 0.53981798796144709], [0.19905767503764604, 2.6033186671447384, -2.197327563782121, -2.5086220474878895, -1.6187068877930533], [-1.7951249455887073, -1.5044696674437983, -3.1017823486241736, -4.2688330487798858, 2.9433907119685383]]])+(1.-msk_arg0)*numpy.array([[[2.5325211893527779, -3.7374787482701266, 2.4243358388759244, -1.4567214221316038, -2.2772282223476301], [3.6417032367284978, 3.5927848667233562, -3.4268581880153071, -2.1626693685597886, 0.38611495316090672], [-1.4185740347488887, -3.2820670714402866, 1.079998014623059, 0.73978744249482631, -0.017350797758993508], [1.5231533516694604, 0.67955409760942764, -1.024951980975084, -3.0381659548104558, 1.3348362026838734]], [[2.9548948107308259, -4.4400507373603944, 0.054336452943033464, -3.384069688666973, -0.307726241058627], [-4.1446611495280683, -0.2975091657709692, 4.797834156047788, -4.2320765838717511, -0.3968196021374526], [4.2290624065849585, 2.4111597111055714, -3.070412993853525, 0.42289291849136657, 1.3040775146695749], [-4.615062976217434, 2.4978178279566654, -2.7404512157405936, 3.9619001415826656, 4.4075223608073966]]])
      arg1=Data(numpy.array([4.0346998983568749, -3.8883058787256717]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.058410912497829252, -0.42115928304704103, -0.17280131536641682, 0.29234001735538834, 1.1494105769321092], [0.86392549873613023, 0.59860016348321599, -0.87903559944106746, -1.0285766858025243, -0.31473246536732452], [-0.72306710675645658, -0.76788793168804315, -1.1683427365958872, 0.51580237816547292, -0.90596564330836782], [-1.0673647450555663, -0.20589731654908994, -0.28282468197356014, -1.1809539889622276, 0.34084859738585738]], [[0.10068512608038864, 0.65884472717679476, -1.1421461455431652, -1.0674440204912419, 0.78780198972656712], [-0.77845106934464503, -0.47783902179876858, -1.2728247073352907, -0.25016207099013332, -0.13883115289745762], [-0.051193934131253047, -0.66952517326078609, 0.56511180763954172, 0.64517096281274278, 0.41630132460760977], [0.46167276998717754, 0.38692163486296083, 0.79772076718427609, 1.0978645152728894, -0.75698538226452139]]])+(1.-msk_ref)*numpy.array([[[0.62768514465825409, -0.92633376519334409, 0.60087141545849077, -0.36104827095686876, -0.56441080618536899], [0.90259581343622997, 0.89047139991415769, -0.84934648780467903, -0.53601740477415238, 0.095698555750863015], [-0.35159344449053093, -0.81346002283265306, 0.26767740893514447, 0.18335624981578025, -0.004300393634247641], [0.37751341860388726, 0.16842742080673087, -0.25403425454083811, -0.7530091534311496, 0.33083903048836005]], [[-0.75994402263930017, 1.1418985223496738, -0.013974325744362824, 0.87031982416363962, 0.079141469487343727], [1.065929810770549, 0.076513827628312242, -1.2339137675095149, 1.0884114356915626, 0.10205462597698299], [-1.0876362453179647, -0.6201054614293332, 0.78965315220000187, -0.10876019831803016, -0.33538449786182067], [1.1869084172282107, -0.64239231836752619, 0.7047931158746007, -1.0189270765089893, -1.1335328285057371]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.2136644736729085, -0.80755393331548664])+(1.-msk_arg0)*numpy.array([-1.9695895503633567, 4.0469760319444088])
      arg1=Data(numpy.array([[[4.1944438836399875, 4.1249463546716534, -0.1581932361206464, -2.712537949130478, -1.3548629744933258], [-2.606981092264673, 3.2264620199369816, -2.8305602274880295, 2.4027446068698382, 3.5424350500390762], [-3.7925390079685437, 0.80136443012807135, -2.256980636934296, -0.50371403496434475, 3.1541266947987019], [2.6086918963153272, 3.9477982192806937, -0.43907347858737289, -2.3124746859380965, -0.89220056722267849]], [[1.5913912338471654, 0.27532055748139683, 0.97103101877229214, 3.4382709463092969, -4.6606014206806652], [-1.3455546380852912, 4.3575427569826815, -3.2395834118911901, 1.1808730813635222, -2.1258425925579139], [2.1091939238169513, -3.1361786045708531, -0.73411683347967571, -0.61135996690801964, 2.3111199494492958], [-1.3710744499761498, 4.9849682230986048, 4.8049359057853849, -0.33946624059111219, 1.5837205639126051]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.52776113713359885, -0.53665291214414268, 13.993420502407908, 0.81608608439285191, 1.6338659446360222], [0.84912947019109675, -0.68609655405649106, 0.78205877839151905, -0.92130660384948171, -0.62489909974453584], [0.58368930919939244, -2.7623692672747255, 0.98080791542800971, 4.3946849204422147, -0.70183118430954017], [-0.84857260330344908, -0.56073394604150972, 5.0416720244522875, 0.95727079182053665, 2.4811287450352117]], [[-0.50745154060151298, -2.9331407022523277, -0.83164586681948105, -0.23487210459150401, 0.17327247288989284], [0.60016435636134902, -0.18532323796970976, 0.24927709234196141, -0.68386175115705572, 0.37987475467024101], [-0.38287325039040415, 0.25749615539705217, 1.1000346218567456, 1.3209139901647908, -0.34942103870805769], [0.58899349581602534, -0.16199780964973121, -0.16806757658164617, 2.3788932057258299, -0.50990935630741119]]])+(1.-msk_ref)*numpy.array([[[-0.46957108141213794, -0.4774824642586501, 12.450529483202715, 0.7261058047113117, 1.4537186323952194], [0.75550588234315996, -0.61044870145467456, 0.69583029226382576, -0.81972488658677223, -0.55599877557151833], [0.51933270725100822, -2.4577950758914811, 0.87266568358277607, 3.9101343493491769, -0.62444845782869129], [-0.75501041466235363, -0.49890836384292825, 4.4857857429697168, 0.85172372365423654, 2.2075636608195293]], [[2.5430428079969389, 14.699142225214546, 4.1677103549803585, 1.1770381378141554, -0.86833772439466872], [-3.0076638416580463, 0.92872893225416786, -1.249227297895652, 3.4271049919026648, -1.9037044634028604], [1.9187311257851083, -1.290416313039731, -5.5127138452362576, -6.619628780098525, 1.7510887017823289], [-2.9516821876556789, 0.81183587353518782, 0.84225390542080814, -11.921586149177644, 2.5553599063879644]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-3.6552029201093084, -4.1115112538527923], [2.5616877647924552, -1.5802409924256402]], [[4.0062861550065172, -1.1686859749149328], [-0.48690845656033321, 0.38918550949013486]], [[4.5253882352758605, 2.4042528804647656], [-0.27269598679069329, -1.0150163715089522]], [[1.4296882474957497, -2.7672919345273925], [-3.8297125530873775, 4.2957055688416119]], [[-4.7876488907399235, 3.9816762684894673], [4.4118208407855732, 4.2898272001247637]], [[1.9399234285141729, 4.8233061653427853], [-2.4267271624188549, -4.4821033561211525]]], [[[-4.2335271350538397, -4.8527217213224194], [0.70467347968021787, 3.8669247155816251]], [[-2.9103135064137664, -4.2256106067708386], [-1.3510574372149788, 4.832602321309718]], [[4.8154087234522827, 2.3113979157459186], [-2.1447991945199396, 3.2559009834729675]], [[-4.9337556238796063, -4.3821867173438234], [-2.2198711314504482, -3.7567701197430345]], [[1.7510966806896349, -0.65589843424628835], [-0.37024572771659603, -1.7157381910701153]], [[2.3365641891591515, 1.0496549023256518], [0.59044051780099327, -1.744321307712311]]]])+(1.-msk_arg0)*numpy.array([[[[-3.5050266762538218, -0.048007732814113524], [-3.1712230199158675, -0.89630824605012016]], [[-3.0230677135598159, 4.3808865686646019], [-0.91082644066091678, -2.860925341443922]], [[-2.1444646779841037, -4.4530676616192437], [1.5001577436514228, -4.4302458355313137]], [[4.2544906245448306, 3.3284466890970759], [2.2922108939290791, 3.0867104592014663]], [[-1.5295157527456729, -3.2834405657677825], [2.8148470874816081, 0.53429992664956316]], [[-0.57988120583570169, -4.0148926677943848], [-3.2537173320083204, 0.80899365136025914]]], [[[-3.4921127223146629, 1.2334174726500411], [-0.071274841377152676, -2.2892376891818458]], [[-2.8667667167399538, -4.9969097706176333], [3.5296254724053284, 3.8439487176729159]], [[0.14922892126890819, 2.4598603596389337], [3.6156150384073431, 2.4609719183626826]], [[0.0036356710152620764, 1.2049872204014278], [1.2798685075322789, -0.46110990790337869]], [[-0.78215343870226572, 0.82757149177313316], [0.061137758039793155, 1.6785744496198962]], [[3.6742184824548314, 0.25941409932261728], [3.9438449971166847, 3.6020203158751727]]]])
      arg1=Data(numpy.array([3.3140623990554161, 0.32135645480847508]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.1029372654996252, -1.2406257815256188], [0.77297511523096096, -0.47682897970661181]], [[1.2088746899118137, -0.35264452933898743], [-0.14692193384744756, 0.11743457504030753]], [[1.3655108716618312, 0.72547000960212238], [-0.082284505828380872, -0.30627557640382846]], [[0.43140052157836367, -0.83501503632403973], [-1.155594582099279, 1.2962053973594423]], [[-1.4446465739765533, 1.2014487927639312], [1.3312425384757522, 1.2944316321103257]], [[0.5853611655191211, 1.4554059593801065], [-0.73225149988441007, -1.3524498987703595]]], [[[-13.173929048902956, -15.100744511930181], [2.192809477252279, 12.033132235934922]], [[-9.0563405927174578, -13.149294322683689], [-4.2042330782501143, 15.038136776153745]], [[14.984633578691342, 7.1926294964994133], [-6.6742060488506958, 10.131742912752285]], [[-15.352906562340783, -13.636529317438352], [-6.907815599264894, -11.690352141773621]], [[5.4490789106236228, -2.0410308379745992], [-1.1521340933925177, -5.3390500343074683]], [[7.2709421397859213, 3.2663258715348804], [1.8373382857764264, -5.4279952420806588]]]])+(1.-msk_ref)*numpy.array([[[[-1.0576224144882833, -0.014486067862752623], [-0.95689900733907085, -0.27045605607956824]], [[-0.91219396304108802, 1.3219082929498416], [-0.27483684100834166, -0.8632683990076202]], [[-0.64708035630087268, -1.3436885385406352], [0.45266430230130916, -1.3368021787381057]], [[1.2837690158632675, 1.0043403799656154], [0.69166195982984868, 0.93139780955278617]], [[-0.46152291917666366, -0.99076003116405975], [0.8493645407171283, 0.16122204784129923]], [[-0.17497594674167305, -1.2114716575459537], [-0.98179120976590684, 0.24410936003825423]]], [[[-10.866788795003117, 3.8381599441814367], [-0.22179371321367017, -7.1236710977105044]], [[-8.9208312882606187, -15.549430222572429], [10.98352131905658, 11.961635312301]], [[0.46437194285656092, 7.6546163079406124], [11.251104448990173, 7.6580752666984546]], [[0.011313514823994734, 3.749690421247605], [3.9827067058449672, -1.4348860930090703]], [[-2.4339123331704062, 2.5752446524416528], [0.19024904315748251, 5.2234035585820369]], [[11.433467190334252, 0.80724720303883502], [12.272493482252202, 11.208800265181969]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-1.7033171016342594, 4.3919326132319156])+(1.-msk_arg0)*numpy.array([1.7922959022761962, 0.24175610094136779])
      arg1=Data(numpy.array([[[[-2.5456980640218863, -4.9868209281468978], [-4.6019226390057897, 1.7242704145566403]], [[-1.9867293105052388, 1.7437083012276915], [0.41089391906602479, -2.8960431860620242]], [[4.9024251453706853, -3.7608519730733434], [-0.86223954338749031, 4.4552867516281385]], [[4.3330511383693278, 3.5958910639310453], [-3.3214238311385502, 0.62655071262966722]], [[-1.4614248345161105, 2.2030161123044163], [-0.83489174474091499, 0.90382336736603985]], [[-1.3884781714710384, -4.3271276807144998], [1.5020755972855895, 0.96588472631813715]]], [[[0.27784338244015583, -1.8822889774363993], [-2.4221353110814792, -0.020597660965138154]], [[1.293425769577194, 3.4855795390224458], [0.40654871052845998, 4.9597736531032961]], [[1.8443923371010804, -2.0665078365410814], [4.1889802241683665, -3.5333771766064794]], [[4.3069190428521189, 3.2884936425728366], [-4.6693294930472637, 1.3820728575005212]], [[1.5520832134771725, -0.63463145687294187], [3.4088554203740564, 3.3523121674763061]], [[3.5789546704977955, 4.0022220905245369], [1.4949357475527147, 1.4643154783126562]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.66909627881918965, 0.34156371888557302], [0.37013162437738156, -0.98784801226913788]], [[0.85734734602626583, -0.97683603412050402], [-4.145393792894172, 0.58815321188300118]], [[-0.34744377550418815, 0.45290724384515457], [1.9754569535774458, -0.38231368632149204]], [[-0.39309877664524279, -0.47368428891507769], [0.51282738615456358, -2.7185622285630249]], [[1.1655181035692754, -0.77317505401834863], [2.0401652218550033, -1.8845685596712751]], [[1.2267510837636444, 0.39363689433656968], [-1.1339756166149924, -1.7634786587082145]]], [[[15.80722410827217, -2.3332934878116052], [-1.81324824964916, -213.22482298671324]], [[3.3955814987879727, 1.2600293764817263], [10.802967761287398, 0.88551069472372268]], [[2.3812355564949517, -2.1252920194985214], [1.0484491160623326, -1.2429843726590233]], [[1.0197388364011364, 1.3355454170182843], [-0.94059171017414844, 3.1777866046618741]], [[2.8297017679822427, -6.9204458204334092], [1.2883892308785525, 1.3101204165417144]], [[1.2271551381848163, 1.0973735374731048], [2.9378738319835693, 2.9993076480300398]]]])+(1.-msk_ref)*numpy.array([[[[-0.70404889236730284, -0.35940650929733969], [-0.38946676049804435, 1.0394517513872945]], [[-0.90213392071031717, 1.0278645235641166], [4.3619431174599583, -0.61887747769166412]], [[0.36559373149606261, -0.47656645757624538], [-2.0786519430955148, 0.40228519558729192]], [[0.41363368329659206, 0.49842886517169677], [-0.53961674071020782, 2.8605759536268556]], [[-1.2264030690771994, 0.8135645909558985], [-2.1467404769134286, 1.9830156720769241]], [[-1.2908347708321037, -0.41419991147112406], [1.1932128486176499, 1.855600211329836]]], [[[0.87011646208071625, -0.1284372930189655], [-0.099811145907213628, -11.73706574501559]], [[0.18691146150613272, 0.069358939664068003], [0.59465469863898124, 0.048743373760635762]], [[0.13107628787991357, -0.1169877494130479], [0.057712399678220817, -0.068420689005965332]], [[0.056132028147265235, 0.073515757431181702], [-0.051775335474043553, 0.17492283393697761]], [[0.1557623320980035, -0.38093935988075239], [0.070920021863185884, 0.072116225716344032]], [[0.067549360972415456, 0.060405468630473448], [0.16171671681350502, 0.16509837157491875]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.352027170879869, 0.15158521614605824, -1.0804299697763584, 4.4465676110947534, 2.8671318760229667], [1.4634817100898703, 1.6668363435864695, -4.0857681120502312, -2.9048042306999502, -1.4638487379087728], [-2.2975313649736675, 4.2979818483863337, -2.6925575541307687, -3.9275109420687091, -4.1476809938641912], [4.0766471941803069, -1.6803402125080935, 0.3889083692181492, 2.0209326341988305, -4.5039544380486776]])+(1.-msk_arg0)*numpy.array([[-0.64447870184323985, -3.3474429334892974, -4.7329437532272554, -1.7779900897266723, -0.35298891133245114], [-3.0258507236434831, 2.9685621722483058, -0.3115890584527321, -2.7804801048656258, -2.7411578470859741], [-3.5163957665285004, 4.7091546989425161, 2.4218378420534421, 2.6511289316446049, -4.0725614256333644], [0.3796296684290752, 0.53718658362576299, -0.98765129719956857, -2.4906786263153089, 3.0126483149355607]])
      arg1=Data(numpy.array([[3.3679577435720915, -3.0919683904489883, -4.8979030915368842, -4.5285343379455654, -0.66773919635472101], [-3.385192459300729, -2.3713747056452417, -1.108259809855733, -2.4760443505075633, 1.2190709538749385], [1.3690554150807479, 4.1712687359077556, 4.929524377866235, 3.9299973264979595, 1.5955301143530729], [-3.4025479803715966, -3.0953915966522372, -0.10658248470896492, -4.8448937413545812, -2.3712536990082791]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.69835412138671438, -0.049025474068331722, 0.22059031172814336, -0.98189994361663657, -4.2937899881795607], [-0.43231861339788585, -0.70289876147301233, 3.6866518804666328, 1.1731632472998699, -1.2007904324648075], [-1.6781872666842761, 1.0303775950438741, -0.54621041458288788, -0.99936733177590575, -2.599562964404416], [-1.1981160053281867, 0.54285222403699551, -3.6488956912583324, -0.41712630701242148, 1.8993979597933153]])+(1.-msk_ref)*numpy.array([[-0.19135593463821163, 1.0826252117678381, 0.96632041605832031, 0.39261932383475823, 0.52863290527119799], [0.89384894951246752, -1.2518317603630555, 0.28115163581840347, 1.122952464197847, -2.2485630047807561], [-2.5684831510790969, 1.1289502060618746, 0.4912923958602563, 0.67458797332237352, -2.5524817043548151], [-0.11157217197789975, -0.17354398203017257, 9.2665441221084119, 0.51408323056821914, -1.270487555252114]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.8738049640376326, -4.6591222757436901], [2.3376825016559319, -1.5246171012169607], [3.1352739642058562, -3.9075456773891437], [-1.2592087136404273, -1.7987869999521], [-2.1598246087478836, -3.0253905676560846]], [[0.69308086983241601, -4.4192858613521544], [3.1192207874031848, -1.7256307517747302], [1.1212336523759392, 4.7673528697318819], [-2.1276085201051478, 4.5650611763147833], [-3.6866753950367159, 0.64927070462842718]], [[0.53007157273166872, -2.4208477773143819], [-2.0702207983890011, 2.8783071715972275], [-3.0920441406251307, -2.1929561290829191], [-2.565252788760394, 3.666618813594404], [-4.1105182212271094, -1.7013131579329102]], [[-2.0207357901368352, 1.2696251113328998], [1.6890840317050193, 2.6786130874090253], [1.1827707498302846, -4.7771336950293168], [-0.52769794660217961, -3.0716926180921087], [-2.6860824275630826, 4.6470602525812854]]])+(1.-msk_arg0)*numpy.array([[[-2.6143676519504733, -0.87596539294521047], [3.2946606822309157, -1.3531551399987229], [-4.12678740093265, 2.3804721974586318], [2.6598406077646519, -1.7151992324441903], [2.3915019377193261, 2.8933078711229889]], [[-1.0070411804329371, -4.1599982830434366], [-0.37744071962241499, -0.18227781174198654], [-2.0875962831515151, -1.3928820940096145], [3.2499273988036652, -4.2575627687317974], [-2.079336193705311, 2.8511533330933734]], [[1.8196906407119684, 1.7349387098113596], [-0.76530235567975868, -3.5727557091636273], [-1.8172857487136351, 4.7690672880929643], [4.9374684070894599, -0.17306854168737207], [1.1108866883777582, 1.2336909913217582]], [[3.0523851102201842, -0.24261932973219302], [-0.63056118393237437, -0.23433245188843799], [4.7640889428631041, 3.1679482352016066], [0.58690406217008917, -4.3885985586862422], [-4.1020370111055566, 2.2601145638881359]]])
      arg1=Data(numpy.array([[2.3672291319270045, -4.3580532375048691, 3.0481315785191949, -0.93066662161111235, -0.35009288721543896], [-0.96908847716138347, 2.708491622267565, 2.1584173012659029, -4.047097111025824, -0.81140995598330878], [2.8705225540975929, -3.3524874046395228, -2.5974682514051892, -4.4023195754503455, 1.7065660458696454], [-0.97276009828049492, 3.7696795022052356, 2.9581404380494813, -0.77453996378490952, -2.5427194114011598]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.6364300826613369, -1.968175455812766], [-0.53640521908684469, 0.34983902631025565], [1.0285887874069386, -1.2819478348396818], [1.3530180242851766, 1.9327941479604711], [6.1692901730356438, 8.6416796174277319]], [[-0.71518843342618388, 4.5602501376313498], [1.1516449826755435, -0.63711873338933278], [0.51947028580540944, 2.2087262119960998], [0.52571224799838312, -1.1279840960272065], [4.5435422228323672, -0.80017591581262659]], [[0.18466030583003293, -0.84334741556330484], [0.6175178452643888, -0.85855868320755657], [1.1904069044741485, 0.84426676934224876], [0.58270480931588786, -0.83288338130684636], [-2.4086487781563934, -0.99692195450070697]], [[2.0773218326993477, -1.3051780326692677], [0.44807099137126039, 0.71056785751734486], [0.39983590184452905, -1.6149110547906342], [0.68130499557892643, 3.9658284423205314], [1.0563817680862093, -1.827594594883174]]])+(1.-msk_ref)*numpy.array([[[-1.1043999149428723, -0.3700382785641646], [-0.75599367485405455, 0.31049532124886325], [-1.353874429179817, 0.78096110228124827], [-2.8579950607448463, 1.8429792071783382], [-6.8310497729354092, -8.2644006113226602]], [[1.0391633005303327, 4.2926919276026716], [-0.13935458264641756, -0.067298643364228861], [-0.96718844957698791, -0.64532567135775587], [-0.80302678923854665, 1.0520041036654606], [2.5626210011010566, -3.5138259175513782]], [[0.6339231294714649, 0.60439821569587804], [0.22827896523060853, 1.0657029476738], [0.69963732866821848, -1.8360444966027878], [-1.1215606505768883, 0.039313034576702127], [0.65094854726918217, 0.7229084361004523]], [[-3.1378601112604749, 0.24941332417012221], [-0.16727182869617976, -0.062162433636959107], [1.6105012735650972, 1.070925570149897], [-0.75774535803432475, 5.6660711698343817], [1.6132480023995799, -0.88885724227145646]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.8215957940427359, 1.0954256245381702, 1.70978709467395, 4.3342808780117092, -3.836222050538268], [0.029352051305135518, -3.0321536903433977, 2.8867452190893141, 4.8968477464385014, 2.127467559777763], [-1.9429407541344288, -2.4151318801146426, -2.895550242082038, -2.2705796934813227, -1.5909022377868665], [3.4593174530565456, 1.3935934781684969, -0.50677332253916152, 0.53235497430121193, -2.772619088366036]])+(1.-msk_arg0)*numpy.array([[4.8490749683258301, -4.8122886850712128, 2.6454710648101152, -2.1230221584520348, -3.6835364050349795], [-2.7749456889644808, 4.9529680927311368, 1.3552116403176129, -3.6338232972764373, 1.7232076226578101], [0.14295846689310032, 2.3423429408135057, 2.9076126553409063, 1.3327817827083539, -3.1782434647876698], [2.8128995041045712, 1.1603966269808739, 2.7285012967801485, -3.2184785508114313, 2.243668867604435]])
      arg1=Data(numpy.array([[[1.5384588557375132, -0.5130960946837444], [-1.1757114336918519, -3.3135223335063015], [-2.3151287062310266, -0.22113947734295714], [3.7284744590306893, -2.4769910953418273], [-1.0929510960164035, 2.9712102764869037]], [[-4.55634443262886, -2.4759558866021636], [2.188120790617722, -1.1846938212592595], [4.5241662939942646, 4.3913180061127157], [2.6246196834355082, -1.6093292578297858], [3.7989203878634203, -2.2934769670169217]], [[-3.4870225838938183, -2.608984438210733], [-4.803103036278582, 0.76268312561501794], [-1.0948854214018366, 1.4947448899254123], [-4.6702210022198951, -1.0730807686928259], [4.3604513311026913, 2.2377160790321522]], [[-0.2992129366293641, -1.3735949529299516], [-0.9860062794598683, 3.2284076002229263], [0.037699313134350376, -2.2229641741833861], [-0.66928452995207888, 4.6969962413082644], [0.51791284208497856, 3.1090484880918812]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.1340427311794037, -9.3970619616869424], [-0.9317129978896469, -0.33059249773609134], [-0.73852787971233003, -7.7317135557045003], [1.1624810430211487, -1.7498168992866623], [3.5099667903903105, -1.2911311194959032]], [[-0.0064420176611188181, -0.011854836131760131], [-1.385734143811777, 2.5594407904655041], [0.63807230581276542, 0.65737557951188319], [1.8657361206819685, -3.0427879954422772], [0.56001899028325997, -0.92761671051134043]], [[0.55719190437958821, 0.74471151520816148], [0.50282741425132405, -3.1666255604739009], [2.6446148478027225, -1.9371534645129482], [0.48618249380533568, 2.1159448195565287], [-0.36484806662996322, -0.71094910238789411]], [[-11.561389998794109, -2.5184407133104529], [-1.4133718082727666, 0.43166590181248093], [-13.442508109714241, 0.22797188025998058], [-0.79540905321587041, 0.11333945077906513], [-5.3534472657681418, -0.89179023710488281]]])+(1.-msk_ref)*numpy.array([[[3.1519042256097638, -9.4506175715771938], [4.093086574789992, 1.4523181680139596], [-1.1426885501829736, -11.962907286369999], [-0.56940772473575363, 0.85709721058123367], [3.3702664450959983, -1.2397427520311066]], [[0.60902895511861666, 1.1207573220428539], [2.2635716062699074, -4.1808001391164717], [0.29954947547277999, 0.30861159187996812], [-1.3845142289415155, 2.2579737984610584], [0.4536045630656062, -0.75135161479260493]], [[-0.040997287357245712, -0.054794679799294865], [-0.48767284880657907, 3.0711875773109183], [-2.6556319031247528, 1.9452233454271892], [-0.28537873948038928, -1.2420144145644163], [-0.72887947220452998, -1.420306845255503]], [[-9.4009956113258486, -2.0478376817740238], [-1.1768653518277148, 0.359433123283673], [72.375358326993705, -1.227415775957106], [4.8088345192169255, -0.68522059321788631], [4.332135999122988, 0.72165772782187887]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.3095484822201122, -3.0863861980739458, -2.8456045560392118, -3.2001881514889496, -4.7262897108077091], [-4.7108408796280639, 4.4337061256578956, -0.69820389939340011, -0.88879769363575001, -1.9835470059929983], [-4.9605968927940767, 3.8151441991115949, 0.55560639129374234, -3.6672704625875352, 3.1532822188201379], [4.2292545650071141, 0.11729644988445198, 2.9152924823967528, 0.41488515427345618, 3.2515439516230451]], [[3.9646504294331368, 3.9740765535998488, -2.7694760498554927, 3.5652863521269769, -4.1762013023421947], [-0.8008576263358762, -2.243327943083413, -2.1211687734157247, -1.3514415553230497, -3.7534347456053005], [1.472395885345291, 3.1257343949071874, -3.519179169337173, 1.0362631529329551, -4.9933979903142633], [3.0789076064286149, -3.3547031189296428, -2.1289218450126732, -2.9970560603509866, -1.1057413493189716]], [[-2.3943233821695955, 4.3330230017873088, 2.6280778202331287, 0.52481231763981917, 2.9802796970137724], [-3.0431810875099297, -0.8244372314584103, -0.27970960818104817, 2.4146496860394562, -4.4968982881806472], [0.16031790341004637, 4.4350525239221046, -4.746392810403723, -1.5991128924990239, -3.4866871442979894], [-2.0076794450930793, -0.7062216325229258, -2.9493904516071345, -1.1790486553357518, 0.75871010557742924]], [[-4.1588141486847938, 0.95719041837421326, -2.8565031533941356, 3.0121140818268231, 4.2445943036734928], [-0.34308756762571235, -4.4148127814308324, -0.28340612343086491, 0.060676990086280469, -1.6136941371010827], [0.59728273626675765, 3.311051078690161, -1.5709437543786029, -0.64230133567984993, -2.763421613819852], [3.9940118020236923, -3.013921050957503, -2.1888140005025756, 1.722645650540013, -1.8085660947721016]], [[2.6603339854976396, 3.7044173806203702, -4.0476512685961588, -4.3526057012191153, 0.88476266956252125], [3.0912750303930974, -3.7430301223775286, 2.8425548575287376, 0.74904357412503586, -2.0909333114747755], [2.8458862451713696, 0.63206737765309828, 0.68374391329891715, -0.41634483015604662, 2.0009189699777687], [0.87413642208220121, 4.4481805949767175, 4.2686738678275979, 1.6455255100255464, -0.98912767026136361]]], [[[0.27763987435877358, 4.8141979986423973, -0.36098884451798519, -2.7569056708455655, -1.4605934675364685], [2.9933128431890292, -4.5350948417984416, 4.5016747318785661, -4.0259477379348665, 4.9352779146049457], [1.814701756763978, 0.57473858742130535, 2.1632217728973, 3.760141281820955, -3.5842280112941518], [4.8036799055670425, -1.4712461756241479, 4.5039314510837141, -1.2861874256467063, 0.26031644469437865]], [[3.4404546551252206, -1.6456707479830088, -3.5365017737372426, -2.6037407659727254, 0.18237369930516945], [-4.8445535113484786, 1.274113755818914, -3.8262598808299844, 0.51921681698380695, -2.9524007116780151], [-1.1031548678060235, -3.8222043995035904, 0.43586598334843263, 1.0237982848452454, -0.29262627114522655], [-3.9446520229554514, -4.6299790872040036, -1.4467150595992573, 3.5414578784056605, 0.93722374947562592]], [[-3.3613380371969539, -2.3869845495837518, -2.3015663388285059, -1.8357838184828656, -3.8660778171484642], [4.4185071250896417, -3.6830145521515121, 4.4900560235810403, -1.7030545878663639, 4.0954000446955128], [-4.0751962086029003, -0.44155019128416217, 1.4362827285097568, -3.0697183298573893, -1.4957335702873853], [-3.612786254909178, 1.4004214827284303, -4.037379856338684, -1.2033049495256409, 0.15082590561302389]], [[-4.0277059144451126, -3.7182114319182293, 1.3427958991862834, -4.4692834538810402, -1.6916069665647946], [2.4394407288296973, -3.0982301172362492, -4.8360356070886921, -2.8690530176856011, 3.9203709451394904], [0.29443799192192532, 1.9877554155081238, -2.5557469200981453, -1.5708246047757521, -3.4089046642467147], [0.35869120630419005, -0.46519732605407516, 4.9369602908271233, -4.3722137401618211, 3.4421836671514061]], [[4.8021493406056734, -3.9619740672201464, 0.78421089271300115, 3.2107540032978026, -1.7809092947038554], [2.867451852361147, 1.9214367856745387, -3.1475896671323298, -0.667802665452502, -0.24960122040569654], [-0.87604713621740515, 3.5372745842428159, -2.6258604053611645, 1.2728214980331352, -4.3813946116200313], [-3.2952602274253415, 1.3073819181065849, -4.9119207563665732, 3.9325802297756347, 3.2663298894977295]]], [[[-3.2664839203822393, -1.2385048913682861, -1.7319658419908288, 2.9353596967919051, 1.1352695342214503], [-4.6148284508059456, 3.6302767986530249, 2.9605737048881462, -1.7174508877678605, 1.9515639344328992], [-0.68203519182981154, 4.9329858192014626, -1.8723101522661434, -2.5484116062100726, -1.6159947627648852], [-3.7877297053067438, -1.3129494166334545, -0.51153738283079431, -1.3740343973853433, 1.4003496094200605]], [[-2.2033623766963037, 1.7500585468806289, 1.4152359327158273, -2.2416079403823321, 3.4391561277415246], [3.2006229510864586, 0.24802637136311212, -4.4387816582069997, -3.3730355959296454, 0.44455454819271978], [1.2359826190076948, 3.1173225190707008, 0.15040100683404312, 2.8587303536686051, -3.6346106909579614], [2.7702333104794477, 3.8888071921348217, 1.5744982313472295, -4.6878939925704932, -0.5862927875249575]], [[-1.0080238706164959, -4.6256906499392905, -3.9751276139365932, -4.6414964958411922, -0.80121900340720842], [-2.4125359205334385, 2.095887089291951, -1.6798157409837575, -1.9251013931834526, 4.1978692315314241], [-2.8997783173977529, -4.0431492469471939, 4.2190067091189061, 0.59835265984576935, 3.4926795451992305], [-2.4561837354941032, 3.9443392169923026, -2.3005670815128463, -3.5093759201158345, 2.9128800947302427]], [[-1.018221522450784, 1.488467831089439, 1.285278850250819, 4.5189795278901528, 2.862614978664606], [3.8662738876585312, -1.6482064942614247, 3.5120130011870128, -4.6941135090445441, 0.72888181685627806], [1.8845175874378022, -4.5649336457212168, -2.5136147358050365, -3.2055482592531659, -2.2324356376749965], [-1.8277788134588926, 0.51905038251021107, -4.4840668866950626, -0.52685718053530373, -3.6291196866133237]], [[2.5234542226163885, 0.63100372901278234, -2.0059679396287402, 3.4328192213397681, -2.7299643398472351], [-0.68532541056030372, 3.9437728221713044, -3.5987598222191499, -2.2400282106423552, -4.1115474354207846], [-1.7896563569727411, 3.5727395645615605, 0.13020058851353955, 2.068096268686678, 4.6156818483885438], [-3.3431509669817396, -2.0974742766895682, 0.98285968295113513, -1.7015269318898762, -1.5548108386497939]]], [[[1.9371581967270295, 0.24565261086698342, -4.3892092359959278, -3.703784274207055, -2.0485997574397397], [-0.066348184476929362, -4.1272157536946628, 2.7019358924797636, -1.6333904175577141, 3.9035622334279445], [4.7954046692287662, 0.073446977403853531, 1.0183475170141634, -0.67068997341991921, -1.1113357789384279], [-2.0334143414112704, -1.0895104182089854, 0.99097441723810942, -1.3038798707905794, 1.4782779134420156]], [[-0.25623384929794302, 4.9939205144132917, 1.2148355416103414, 0.52919927847510007, -3.4054354136895104], [0.30610661448523757, 1.0966637046656089, 3.2284714403110772, 0.83726834267280914, -2.1248048996319291], [1.8668680586192039, 1.6004787298066372, 2.821736704928953, 1.0424857442511772, -3.8327724192142654], [3.0550895108749927, -1.3350044854636876, -4.9933054814269529, -4.6522180477454089, -4.3397287880499018]], [[-4.3216034325989821, -0.34423042506494106, -3.9153542164127986, 0.23781186928949349, -2.3282301287845177], [-2.6268742706938775, 3.9350573812239329, -4.5017302706440612, -4.5067004592081714, -1.5342922967061243], [1.0917093901098109, -0.83559092532171064, 4.1157114142951379, -4.0061309158620118, -3.6832468396844154], [0.2282324400067548, -2.2305585363700677, -3.2846723491821916, -2.0007908664726548, -1.5525159922139964]], [[3.7999450803864434, 0.29164162999299315, 1.2056268204791838, 1.6186381972097923, -4.3260950452827709], [1.0547757865803673, -3.9166919001709077, -2.9512469633140093, -3.7905736247496593, -4.2413554341031467], [-3.5661375826498301, 1.7505445072112167, -2.2223231619784189, 2.3593708134209903, 2.7157484391437077], [-1.4110589121963066, -3.0382018438646763, -0.33015450774308341, -2.8572918308779807, 4.1879801630077687]], [[1.9477551920385494, 2.4088548084579742, 2.1935512939509083, 3.4475566608168187, -3.0522188265058903], [1.9680767736341567, -2.8152287322273462, 0.32428349893078412, -3.7990862936887995, 3.0724003567891778], [-2.5742507933693739, -3.4087414980976325, 3.16984322427124, -0.054583303898299462, -0.44340490222648743], [4.5206763550963256, -2.0790672865841966, -4.8671868768697415, -3.46749737746301, -1.5350955882533244]]]])+(1.-msk_arg0)*numpy.array([[[[2.4011426109613945, -3.7230716972778999, 0.1300827886652467, 0.25480631918469676, 4.9069272657743053], [2.8321320835777728, -4.7110885148481598, -2.2254839019747994, -1.6086688128136775, -0.91146165939723112], [-0.96664558836517234, -3.261449033546441, -0.38957533204580752, -1.8001542973771856, -3.4749136247280688], [2.6522718374448946, 4.9249057111473089, 0.016222150921471723, 4.6377609278998442, -1.1768942214509783]], [[-2.0264864738144417, -2.0512661490595709, -0.39278616840776248, 2.7038967909179235, -3.3691162203142988], [2.4943568695142204, 4.1074442529324422, -2.3083172808635579, -4.9809667687723458, -2.6481160760459987], [1.60118529296824, -2.2447122979508527, -0.32599400767472986, 3.6947163165183881, 1.6388037316981237], [4.4619960920712405, 4.148133474255804, 4.5519753136115249, -2.9046056480265516, -4.2957708836552779]], [[1.5854993471950074, 1.0544851862952456, 1.2425147537719816, -2.533171000739276, 2.8935667541155201], [4.9561081143694157, -3.8557986472606123, 2.8685221708733968, -0.77999231064322849, 3.5997957141551922], [-3.2616207161408584, -4.9287192436967109, 4.8938950668835783, -2.3888727240972116, -0.77813633976108143], [-3.5079324139404813, -2.7732489698540883, -4.8509924943515648, 4.4190190881588354, -1.4738566244001916]], [[0.38357563218258672, -4.4733211758357978, -2.3278505805366589, -0.074748550320508755, 1.666660098540536], [-0.45427863713053096, 2.8279957609866413, -3.2493357018085769, -3.5182400303050287, -1.4587988836461752], [-4.0267425800208105, -0.26945080244267494, -2.9799363386116173, -2.8397963378744437, -4.638851901032468], [-1.7192328361860367, -4.7194690681706284, 1.3582541138214745, 3.0298361678612, -1.0302071537762671]], [[3.8016956214130833, -0.7276635602858974, -2.4842433519935292, -0.4467938671654359, 0.59424508426767098], [-2.1293618965090513, 3.3615336225117449, -1.0589573724872023, 1.6441225256797534, -1.9684715758821336], [2.4930600894068897, -1.1796704204237916, 1.3084294229659035, -2.2335411789634643, 3.2756580025261925], [-0.4480868498478765, -0.081421649214546754, 1.3457737480772618, 0.8094455461475567, -1.4198394396369816]]], [[[2.789345262849718, -2.2525807328303014, -1.0651830863640677, -2.7734565509534539, -2.3391802077711854], [4.6898221756023535, 2.8885230925669481, 4.9426557068841568, 0.17027902969070574, 0.85966030450619701], [0.92212058207157188, 2.9427099416932005, -2.3516631984587599, -2.9499670138866452, -1.6984483102498595], [-3.3635834284918085, 3.1558936199237291, -0.81807171517004651, 0.12277411364302893, -2.5523428624306952]], [[3.677063117252338, 0.07933068811095545, -3.7381798247341713, -3.833883601967909, 2.029739486419059], [2.5271551981462705, -3.9534612054006657, 2.8032875448444106, 4.8209785739071833, 3.2703420850283607], [-0.52788981445592498, -4.2243304662405921, 4.2623657273142115, 4.6322422718383507, 4.4562466447321238], [4.0824234421430923, 4.4246676222740895, 4.8972157587630285, -1.3893078554924676, -1.887387131607615]], [[-1.5042722949800504, 2.3106611046023922, 3.4305147165486023, 2.2847760670482895, -1.8004270597555072], [0.4789031893493263, -0.57003366686019241, 3.2279532221815419, -0.80622987804173096, 4.4569099714476383], [0.18361401436160296, -4.1364172500107541, -0.73321411367430223, -2.0726749040555914, 3.8057467092351445], [-4.4125230534039979, -4.066499995947388, -3.0037094711059709, 1.5010955410824369, -3.5903574525598581]], [[-3.1171637002002783, 2.5129066511512441, -2.0786399914603604, 2.6991164247160615, 4.7538296810106466], [2.708133775362322, -1.4269150393117149, -1.8002968672596777, 1.2796704443003559, 4.5181844548981793], [-2.6243426891768742, 2.5323872166634054, -0.53603658121553455, -2.3111231592549397, 3.0835650799710592], [-0.48931192986049776, -2.6457386174189654, -0.30528561131886534, 4.2284853849273034, -0.11776204175869243]], [[0.42461141527310264, -3.6665020970992224, -0.26511362309883957, -1.3465765016743347, 1.1469549410509758], [-3.7572196019220092, -4.5316291557940245, 1.8945942860291245, 0.70459596534545899, -3.1450414030915996], [2.0255812118807626, -0.15116977379509589, 4.0364469109352488, 3.6078386675583936, 1.5676876855728397], [-0.32831476433605378, -0.37762145028085037, -0.23646835390177223, -3.595604598141743, -2.8004955239740461]]], [[[-2.9254681513101755, -4.1234975572234509, 4.2449714053002783, -2.6528112115793059, -2.6950168872095825], [3.4868028511158542, 2.0614039284068628, 3.5481946300280605, -4.4679714767892538, -1.2791717427047167], [4.3553848465531004, 0.26526017889687648, 2.3948958537218257, 2.6705436984224793, 0.56166772734315451], [3.7596120352073861, 3.6293608745233072, 4.0947886302800089, 4.9348783026178413, -4.5543019486846505]], [[-3.5950062040080057, -2.1206631128245066, 3.033598093494609, -0.39173098989234489, -1.0452414856792744], [0.55658092020847505, -4.6935132842385583, -4.5217666437800998, 1.5520172903937377, -0.72318227918314726], [-1.2416772119969322, 1.788488387835546, 3.4773786627038987, -4.1236922812575196, 2.3439480593139317], [2.5858816878895849, 0.0019849243673277783, 3.479471774023704, -0.29800536093014252, 0.97445649886796026]], [[-2.7071925134590611, 4.4924626796035803, 1.265934982102106, -0.83743190684919, -2.9600871082023819], [4.4153495121743465, -1.7155736904604, -2.5783482191757559, 0.39748687678755701, 2.0488728966954142], [0.1949266728349528, -0.36293588636099461, -2.1828990736503173, 4.8527282083009631, -3.7395770125478647], [0.035425160624896535, -0.32153646382833756, -3.4051953823240577, -2.5445704605081021, -0.13435750114663314]], [[-4.4206281381845365, 1.0229244353045219, -3.0368648949482315, 3.2623023965442037, 2.4582554653427398], [0.34501526794554405, -4.8379606215210114, -1.8106844690436876, 3.2507318702721104, 4.9012064552653296], [-4.9452630720982018, 4.7075509033145746, -3.6253102672067028, 4.6374862003395414, 2.6711379124180965], [-1.4331568592127377, -1.6711913521106405, 0.94088805821327881, -2.8052276016961155, 0.71824156425251218]], [[-4.9110821109790006, 3.3634815894347625, 1.3362985851678459, -0.84403069759930105, -4.9821844595436096], [1.1253452364404106, 2.1139234097659223, 3.0487471780855753, 0.93721688030730377, 3.1461772438328577], [-1.971651135920057, -3.5697116643605797, 0.57146638476784872, -3.9434969309755354, 1.6112524089043543], [-0.98073011502280849, -0.86152546483264114, -0.90008384920719475, -2.7764302412259001, -4.7737108904605492]]], [[[-4.5661016037938182, 0.15023180138772041, -3.2158571861934737, -1.6141501482354723, -0.084298563493879541], [3.8744035732260969, -2.8258833317167809, -2.615934576059693, 0.00059755832585217661, 0.036789876777173269], [-2.8742424815626908, 3.7695831161345641, -3.0008722684504074, -3.863273104492678, 1.674155504407878], [0.54567054344484234, 1.1783613437417504, -2.0546098121013778, -3.657025058766493, -3.563130752379331]], [[-1.2843954947429701, 1.7263472291415578, -0.22891485436442416, -3.0135316491375121, -0.40355166571984036], [-1.3924495716255789, 3.7088443130322695, -2.8805414242805027, 1.5504665217393079, -1.6079230095985508], [-3.6584118271622543, -4.7368096411156957, -1.2706894235119814, -0.22794294754626154, -3.221450849597022], [-4.1678774048397385, -0.95940908388104873, -0.97067083649991748, 1.3033824100681617, 3.77609406956031]], [[-1.6137698781742116, -1.8404917075064264, -4.1034357661091789, -0.65947251283208352, -4.2943408064378206], [1.1449488559294583, -0.91451105696645207, -2.4613306862970186, 3.8635812638165881, -2.5613061036513285], [0.87531456918394124, 3.3625291113541902, 1.6354599802097045, -3.6508519716988497, 2.4001835922113148], [0.34260450807596676, 4.1661039108944689, -3.2929258142729969, 4.0556438476054382, 1.6837574766595047]], [[-2.9006496718368888, 3.6626637533024962, 4.7532059467672827, -2.0293193075554239, -4.1251139584283045], [-4.9281195725859606, 4.7174398968678162, -1.5756892749743221, 2.1403056812434365, 0.84004192886883189], [-2.8630473485087702, 3.7513968650531169, 4.2717935875510555, -4.6953949748075896, 0.8534527986053364], [-4.6754497999909752, 1.1177879432852187, 1.4733319387153712, 1.1569084820650346, 1.7744512727417519]], [[0.045207227097114, -2.9658585617651423, -2.442926534564994, 2.5014113064164789, 2.3315452783100445], [0.51930109255760293, 2.1434868883454214, 3.0344897522286889, -3.4402192508852938, 1.6255745081477073], [0.85827674776956187, -0.037074026760369527, -4.0341080134768026, -3.2817897899890438, 3.4436587706278434], [-4.5930490854530728, 2.5544527102958172, -0.59679752864238012, 4.5692805811930999, 1.5129716433557885]]]])
      arg1=Data(numpy.array([[0.77248044731134335, -0.21490433800425812, -0.83442465946423461, -4.1443777538106987, 4.0067710914406671], [2.6044920864482695, 3.9023078239081936, 2.9321315449697387, -2.4791458413890668, -3.0783764763249257], [2.7017247547547463, -2.8226806068082166, -3.9483029237487486, -2.2923776117160299, -2.1796092515962973], [-2.5618654722358771, 2.5088881823098665, -1.6949217054833032, -0.97602369865711935, 2.1502374825093664]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[4.284313594912545, -3.9954230671032085, -3.6837237317054172, -4.1427432404630613, -6.1183292434880334], [-6.0983302503310988, 5.7395706792186525, -0.90384669517983729, -1.150576298376554, -2.5677633820983203], [-6.4216471886889579, 4.9388229985501821, 0.71924977936665979, -4.7473958406994674, 4.0820220496134159], [5.4749017657692765, 0.15184390788492849, 3.7739369229908322, 0.53708175490717547, 4.209224923349459]], [[-18.448442996783903, -18.49230495068511, 12.887017896309837, -16.590108814166115, 19.432838541674517], [3.7265772937538748, 10.438728058802441, 9.870292955062153, 6.2885727104134688, 17.465607164853648], [-6.8514014143172712, -14.544771054576172, 16.37556134054141, -4.8219741051128642, 23.235445299458423], [-14.326875087871001, 15.610215922505819, 9.9063698052037026, 13.946000756353289, 5.1452723550748543]], [[2.8694302775123366, -5.1928271206287766, -3.1495687362841824, -0.62895111222718347, -3.571658223676351], [3.64704117141491, 0.98803076120468802, 0.33521253837421755, -2.8937899409514678, 5.3892202695303908], [-0.19212987247162963, -5.3151024165198484, 5.6882221259511621, 1.9164257364179271, 4.1785523770794519], [2.4060643730042268, 0.84635757646037801, 3.5346396084469411, 1.4130079234388788, -0.90926136586684769]], [[1.0034833684889899, -0.23096119013139904, 0.68924777688704175, -0.72679525389721666, -1.0241813260798069], [0.082783855142125509, 1.065253469564031, 0.068383274948881495, -0.014640796204083666, 0.38936946218701057], [-0.14411879701785493, -0.79892598488294042, 0.37905419044732147, 0.15498136845495386, 0.66678806276260016], [-0.96371808731751174, 0.72723125882679096, 0.52814056307728974, -0.41565845414454894, 0.43639026223156174]], [[0.66395956364482378, 0.92453931010279322, -1.0102027733111134, -1.0863125449110946, 0.22081687457827737], [0.77151276173393879, -0.93417618250602175, 0.70943779733288281, 0.1869444390584617, -0.52184995442875759], [0.71026923680536691, 0.1577498098165207, 0.17064711152567277, -0.10391031098468528, 0.49938439813848262], [0.21816480206457173, 1.1101658900552109, 1.065365045920994, 0.41068617908838018, -0.24686403283041425]]], [[[0.10660039084142101, 1.848421050573239, -0.13860239637366822, -1.0585195037413768, -0.56079781356843017], [1.1492885153170085, -1.7412588294644902, 1.7284271107221805, -1.545770770002618, 1.8949099289970026], [0.69675840683343226, 0.22067204212744335, 0.83057337135060094, 1.4437138440104218, -1.3761715882891941], [1.8443826074809819, -0.56488794236671214, 1.7292935826215925, -0.49383426132834696, 0.099949025012923215]], [[0.88164614642818639, -0.42171730735861218, -0.90625904806131019, -0.66723100366927424, 0.046734831677763614], [-1.2414585752737, 0.32650262698724736, -0.98051205939923858, 0.13305378263670831, -0.75657811861729585], [-0.28269293904682402, -0.97947280736956854, 0.11169441341301189, 0.26235713097074514, -0.074988003086891006], [-1.0108510658200331, -1.1864720304322485, -0.37073319811822536, 0.90752909258164594, 0.24017166040401922]], [[-1.1463803671985817, -0.81407826114717741, -0.78494648126445477, -0.62609190287941596, -1.3185212729561777], [1.5069266359041349, -1.2560877626619316, 1.5313283032216005, -0.58082475555643609, 1.3967313477873926], [-1.3898408533526276, -0.1505901711816682, 0.48984252803179745, -1.0469238104694483, -0.51011816739716642], [-1.2321364848405749, 0.47761209251711229, -1.3769436310813101, -0.410385731700881, 0.051438996954886108]], [[1.6246345201653756, 1.4997953609033801, -0.54163650914296924, 1.8027513263910679, 0.68233459215008763], [-0.98398435787984029, 1.2497167635367143, 1.950686210690638, 1.1572748040018772, -1.5813393789463004], [-0.11876590195151712, -0.80179043214108914, 1.0308981736492593, 0.63361524705445249, 1.3750319191938718], [-0.14468338260536345, 0.18764419514481845, -1.9913956687844316, 1.7635968272492057, -1.3884554953099284]], [[-1.5599616803006007, 1.2870336353239324, -0.25474820859117914, -1.0430023838835054, 0.57852225301239557], [-0.9314818620834876, -0.62417212464162852, 1.0224836667443722, 0.21693339673961787, 0.081082097113631554], [0.28458089611679371, -1.1490714704478702, 0.85300171228439514, -0.41347168152502073, 1.4232809551776118], [1.0704539398505732, -0.42469851500014821, 1.5956205467859466, -1.2774851484281375, -1.0610560191770921]]], [[[-1.209036529214746, -0.45841268219075615, -0.64105932291687162, 1.0864762191729511, 0.42020177378302415], [-1.708104588627817, 1.3436886167860462, 1.0958087790689461, -0.63568684587330027, 0.72234002779089757], [-0.25244436563328765, 1.8258654256026399, -0.69300551396698651, -0.94325360187974039, -0.59813449165053079], [-1.401967279842532, -0.48596712685953819, -0.189337341611333, -0.5085767508208191, 0.51831690365778194]], [[0.78059216879935445, -0.61999878507669015, -0.50138011693647622, 0.79414154579715623, -1.2184007356150703], [-1.1338948315181876, -0.087869088257764738, 1.5725412388141962, 1.1949760053595826, -0.15749374800693647], [-0.43787547766706003, -1.1043837235965759, -0.05328304111746849, -1.0127714580152773, 1.2876450428686104], [-0.98141933019192196, -1.3777000425606571, -0.55780247596897414, 1.6607950546241188, 0.20770780304042824]], [[0.25530560599930346, 1.1715642743914367, 1.0067939797695096, 1.1755674742996378, 0.20292743968248628], [0.61103111061266713, -0.5308323929973422, 0.42545259911031413, 0.48757692364588107, -1.0632085006146699], [0.73443663604324838, 1.0240220482141711, -1.0685620608646551, -0.15154679653547415, -0.88460272999597445], [0.62208593994152284, -0.99899609861933225, 0.58267238505817431, 0.88883147719168132, -0.73775496738345114]], [[0.44417704886262732, -0.64931179901691705, -0.56067501430913202, -1.9713067798229504, -1.2487536800369059], [-1.6865781047147432, 0.71899432529687335, -1.5320394786782039, 2.0477051795714498, -0.31795887951926521], [-0.82207991292808358, 1.9913532667525902, 1.0965098956464649, 1.3983508837593095, 0.97385161426517253], [0.79732885372696183, -0.226424468576824, 1.9560768975310208, 0.22983001484685961, 1.5831247295669733]], [[-1.1577553273680898, -0.2895031430751403, 0.92033374246306487, -1.574970017596341, 1.2525017215117205], [0.31442581281869064, -1.8093944220887175, 1.6511032055784762, 1.0277200874431087, 1.8863690509707549], [0.82109045722853147, -1.6391651677680141, -0.059735747780563668, -0.94883808516230617, -2.1176648268528506], [1.5338304168663675, 0.96231665155276103, -0.45093389204111267, 0.78065686803435796, 0.71334384248510829]]], [[[-0.75615141299217703, -0.095888177396211693, 1.7132863858636691, 1.4457372232643286, 0.79965157407419096], [0.025898387404012964, 1.61101970358054, -1.0546751661091867, 0.63757852832614459, -1.5237186635023021], [-1.8718409382533112, -0.028669334201905778, -0.39750233884271718, 0.26179749900550875, 0.43379942896396728], [0.7937240902960454, 0.42528010546085049, -0.38681750778007595, 0.5089571973709508, -0.57703182679293596]], [[-0.10213043813775524, 1.9904914653531995, 0.48421270831284102, 0.21092979838897419, -1.3573484213848928], [0.12200887095869428, 0.43711143142933528, 1.2868136025650652, 0.33372086830189396, -0.84691096024681256], [0.74410173868347862, 0.63792349977635077, 1.1246960804490918, 0.41551702128525647, -1.5276776566764063], [1.2177065253112449, -0.53210999791731828, -1.9902463237041315, -1.8542946953746802, -1.7297418109939158]], [[2.5497363203373964, 0.20309517776031105, 2.3100501950893029, -0.1403084688337754, 1.3736505475458691], [1.5498499206161442, -2.3216750180810624, 2.6560107502785226, 2.6589431503699434, 0.90522900954213947], [-0.64410608854555462, 0.492996769478177, -2.4282604919036963, 2.3636082439098107, 2.1731073640561736], [-0.13465662706919834, 1.316024527359527, 1.9379493097267135, 1.1804621181024604, 0.91598094896737414]], [[-3.8932918182362473, -0.29880589005600355, -1.2352433881861355, -1.6584005075254076, 4.4323668075221025], [-1.0806866554896162, 4.0129065570434017, 3.023745189153233, 3.8836901501111005, 4.3455455435546249], [3.6537407724385873, -1.7935471337629778, 2.2769151661338185, -2.4173294323356851, -2.7824615763738336], [1.4457219779988322, 3.1128361412175174, 0.33826484766438841, 2.9274820220136455, -4.290859093657132]], [[0.90583259192630361, 1.1202738432625574, 1.0201437337939965, 1.6033376261274439, -1.4194798720297142], [0.91528344643000792, -1.3092640953044512, 0.15081287605141147, -1.7668217229918233, 1.4288655935825425], [-1.1971937120011396, -1.5852860559939495, 1.4741828519201401, -0.025384779282425933, -0.20621206068318826], [2.1024079395270396, -0.96690123927980598, -2.2635578239430769, -1.6126113537079529, -0.71391909067729564]]]])+(1.-msk_ref)*numpy.array([[[[3.1083538998542823, -4.8196322770838229, 0.16839622170115284, 0.32985471680424122, 6.3521701848287684], [3.666283196468143, -6.0986508218367703, -2.8809582297140426, -2.0824718844505763, -1.179915507984207], [-1.2513528228832593, -4.2220473604193822, -0.50431740169184069, -2.3303558085420959, -4.4983839226257167], [3.4334485056240047, 6.3754438423505579, 0.021000079649826382, 6.0037259765497, -1.5235262271649945]], [[9.4297141352925546, 9.5450197427793526, 1.8277256385582119, -12.581862311519968, 15.677283444355334], [-11.606824192933681, -19.112895956763129, 10.741138602878369, 23.177599926687627, 12.32230163726862], [-7.4506885614217522, 10.445169784829453, 1.5169261388677533, -17.192376621290823, -7.6257359293773419], [-20.762708345062958, -19.302232392226596, -21.181402645866235, 13.515807428554558, 19.989223686913949]], [[-1.9001108478901152, -1.2637272572604781, -1.4890676344222287, 3.0358295048060642, -3.4677387841982217], [-5.9395513521276104, 4.6209068770047308, -3.4377245906361522, 0.93476661049787779, -4.3141051421784926], [3.9088258947609145, 5.9067276929007964, -5.8649933356785482, 2.8628980423841419, 0.9325423583007546], [4.2040133571709708, 3.323546276346542, 5.8135775822664186, -5.2958874573483827, 1.7663147986859855]], [[-0.092553250443904195, 1.0793710036983575, 0.56168880319759273, 0.018036133470646705, -0.40214965853633028], [0.10961323125355259, -0.68236920690599157, 0.78403463555436204, 0.84891876158490986, 0.35199467092613107], [0.9716157211582066, 0.065015985136711682, 0.71903106223162372, 0.68521657690669968, 1.1193120358700668], [0.41483497362305488, 1.1387642122707422, -0.32773414840685755, -0.73107142925745783, 0.24857945268840556]], [[0.94881777237894382, -0.18160846818535373, -0.6200112996972581, -0.11150970618708031, 0.14831021555913376], [-0.53144086545343971, 0.83896323143907836, -0.26429195687005058, 0.41033602573203048, -0.49128625792654246], [0.62221175917251859, -0.29441922024041345, 0.32655457302289936, -0.55744167260635202, 0.8175306070076499], [-0.11183240560088079, -0.020321013443588297, 0.33587487714287612, 0.20201941355639411, -0.35436000890344516]]], [[[1.0709747506484197, -0.86488292460206029, -0.40897919863394616, -1.0648742476064115, -0.89813296801416342], [1.8006667019663847, 1.1090542787964506, 1.8977426472523582, 0.065378977565992244, 0.33006831119940577], [0.35405006099636965, 1.1298594290244732, -0.90292583751548627, -1.1326457965589358, -0.65212266110818695], [-1.291454654822433, 1.2117117331031719, -0.3141002882775682, 0.047139369046981926, -0.97997720005028321]], [[0.94227910333576115, 0.020329172297716153, -0.95794078617569267, -0.98246570362259189, 0.52013823050644381], [0.6476052920948977, -1.0131084947166575, 0.7183665849397024, 1.2354172944457604, 0.83805333474515242], [-0.13527631296068257, -1.0825210764664615, 1.0922679398073258, 1.1870519909931456, 1.1419515952662995], [1.0461561789491305, 1.1338591986940558, 1.2549537298824434, -0.356022107477176, -0.48365921315694188]], [[-0.51303028936772188, 0.78804824038896915, 1.1699729919804833, 0.77922017890635586, -0.6140335220785903], [0.16332936705070947, -0.19440930876314946, 1.100889633590725, -0.2749637475934088, 1.5200238812933737], [0.062621342714519301, -1.4107202172109383, -0.25006180740157391, -0.70688332780000929, 1.2979454198649951], [-1.5048857753240867, -1.3868750202983668, -1.0244115671614491, 0.5119468612032988, -1.2244871682920735]], [[1.2573539031708316, -1.0136179200104101, 0.83845006483995199, -1.0887283755778341, -1.9175272392797484], [-1.0923656568122484, 0.57556720362696112, 0.72617626490701748, -0.51617392689707831, -1.8224762656022842], [1.0585672877181174, -1.0214756931139264, 0.21621825237808234, 0.93222557570877562, -1.2438014046980537], [0.19737117586690101, 1.0671976505974963, 0.12314144905158692, -1.7056218776375336, 0.047501054513481258]], [[-0.13793355638554602, 1.1910505830906106, 0.086121247721897082, 0.43743074053175107, -0.37258436382682181], [1.2205198522071317, 1.4720841296201832, -0.61545243104604208, -0.2288855735366814, 1.0216558719439868], [-0.65800308294291954, 0.049106980565147666, -1.3112258822072671, -1.1719939699726261, -0.50925794737244112], [0.10665192086187181, 0.12266902803638499, 0.076815930644089536, 1.1680197746424772, 0.90973132932635203]]], [[[-1.0828150225746218, -1.5262463542840685, 1.5712079470085112, -0.98189543805698276, -0.99751719062670663], [1.2905840407981808, 0.76299553638060746, 1.313307221168114, -1.6537478397553647, -0.47346486367773449], [1.6120757078929264, 0.098181792364320966, 0.88643221316570664, 0.98845883309267846, 0.20789228301464813], [1.3915599761192814, 1.3433496021887577, 1.5156202063417523, 1.8265658979261243, -1.6857016765566391]], [[1.273614235821426, 0.75129403862042843, -1.0747224061332572, 0.13877977867828975, 0.37030101215071409], [-0.19718168568771804, 1.6627858188836357, 1.6019405925253254, -0.54983808180433913, 0.25620407687602148], [0.43989291916416112, -0.6336134465662776, -1.2319419541539949, 1.4609135271313742, -0.83039790391459911], [-0.91610849688502471, -0.00070320544327268314, -1.2326834873316264, 0.10557530321048827, -0.34522379064694814]], [[0.68565977984503146, -1.1378211769369952, -0.32062762319669069, 0.21209920389139833, 0.74971124692527469], [-1.1182904649023628, 0.43450913559376403, 0.65302695081149498, -0.10067284209544891, -0.51892494984911008], [-0.049369735959843238, 0.091921996201953557, 0.55287021178652263, -1.2290668426457767, 0.94713528439132344], [-0.0089722499283975462, 0.081436624807666003, 0.8624453209610794, 0.64447194393386076, 0.034029177533082065]], [[1.9284031197963667, -0.44622859256541958, 1.3247664256653129, -1.4231086448720405, -1.0723606149261495], [-0.15050542553819143, 2.1104553616275319, 0.78987181683747287, -1.4180612538083004, -2.1380449844807115], [2.1572637277661566, -2.053566951297606, 1.5814629529961537, -2.0230027446778318, -1.1652259639800504], [0.6251835875067302, 0.72902097087731488, -0.41044200283780824, 1.2237196818530154, -0.3133173001610543]], [[2.2531938270046492, -1.5431580623781183, -0.61309089424591612, 0.38723945449449332, 2.2858154304010081], [-0.51630595512302624, -0.96986347815221097, -1.3987585966855025, -0.42999307312579399, -1.443459299655967], [0.90458926730837819, 1.6377759737192354, -0.2621875385917542, 1.8092678438061345, -0.73923911257226904], [0.44995685089175669, 0.39526601577859843, 0.41295651894851954, 1.27382017634239, 2.1901682088036605]]], [[[1.7823346515571472, -0.058641565303038753, 1.2552794910760177, 0.63006827084746064, 0.032905148380140227], [-1.5123368557852874, 1.1030568788026489, 1.0211053642003407, -0.00023325125082803645, -0.014360581059342188], [1.1219334163765382, -1.4714211799906289, 1.1713621581508671, 1.5079921824001916, -0.6534907951067207], [-0.21299734484832508, -0.45996222538310388, 0.80199754216922636, 1.4274852049802644, 1.3908344489570703]], [[-0.51193811816693291, 0.68809253489813005, -0.091241553122414706, -1.2011422710608943, -0.16084880488707196], [-0.55500662861092032, 1.4782820291407468, -1.1481346377216639, 0.6179894874038725, -0.64089066261939942], [-1.4581805012107203, -1.8880114604209428, -0.5064751121519061, -0.090854167656208795, -1.2840153149556144], [-1.6612447833376476, -0.38240408267129161, -0.38689282501472294, 0.51950597848811753, 1.5050866340658358]], [[0.95212060412787536, 1.0858859742914286, 2.4210178870410379, 0.3890873016131659, 2.5336514321251782], [-0.67551725382086525, 0.53955946991998738, 1.4521795775782902, -2.2795042693225152, 1.5111648493055185], [-0.51643363015069021, -1.9838846245675827, -0.96491771562000062, 2.1539944646928793, -1.4161029293839316], [-0.20213589038808955, -2.4579919517323741, 1.9428188355957281, -2.3928207624487174, -0.99341313006513476]], [[2.9719049607379437, -3.7526381360840331, -4.8699698104739371, 2.0791701168193981, 4.2264485627796953], [5.0491802395437801, -4.8333251573280389, 1.6143965327299572, -2.1928829025240026, -0.86067779914014331], [2.9333789255813643, -3.8435510021063495, -4.3767314189486211, 4.8107386954515938, -0.87441811072781894], [4.7903035617104184, -1.1452467238481487, -1.5095247592271404, -1.1853282698532721, -1.8180411758271486]], [[0.021024294974318996, -1.3793167433319651, -1.1361194074777519, 1.163318622600368, 1.0843198936282556], [0.24150871556362652, 0.99686053553672271, 1.4112347017071734, -1.5999252542423803, 0.75599766136093594], [0.39915439794488988, -0.017241828896547493, -1.8761220778129701, -1.5262452713637635, 1.6015248541798419], [-2.1360659568137099, 1.187986318290166, -0.27754958859051537, 2.1250120595333803, 0.70363002024786725]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.7610833907697021, -1.5868036469912794, -1.4006323326845784, -4.7787762297682708, -0.84281497561357632], [-2.6935175560089855, 1.1040962209913783, 4.7821510492454973, 4.0219949911816926, -3.1764608129014906], [-3.5236605165145631, -0.48422136886666767, -3.3805704686592808, -0.41222847051146871, 4.2643385471293342], [3.631132068338129, -4.2811373727110364, -2.4988828481797674, -0.92937476432798682, -4.7856468667336092]])+(1.-msk_arg0)*numpy.array([[-0.58994609675004384, -1.1977669798768198, -4.2224694971282322, 4.6130652724130279, 3.928651328754281], [-3.3880325989277482, -4.1083939360408896, -3.033262693408961, 3.3009068053387693, 4.5386381810404668], [-4.8399180860702549, -0.59365215313808317, 3.5873484513334724, 3.7353793706452745, 3.8254822801751658], [-0.55368736472044855, -4.2368118587715653, 3.1414781697768799, 3.4386451924230581, -4.2075985163700285]])
      arg1=Data(numpy.array([[[[3.4864684909339356, 4.6664842195399316, 1.601330054889921, 1.2221852516600862, 2.3093176727183371], [0.88572319282653922, -4.5723284385796532, -1.2112559797650455, -0.86605324361421854, -2.1578007269766362], [-1.1675562843825915, -2.3193842015857626, 3.3242228194748602, 1.0251943331264251, 4.547551199285536], [1.1314938857434527, -4.1807829173615465, -0.72612777063851386, -1.8374330597391642, 3.5322105514489284]], [[0.90515640605103531, 3.5657906092389187, 1.4426056425219249, -4.1514294437167987, -1.1186576468270037], [-0.57279312659072978, 0.4430970643829637, -0.14309760131221871, -3.7336401950697971, -3.1144919818378924], [3.8634213883983968, 3.2793093424299897, -3.3988924981526991, -3.4015395617163313, 1.2560063579288343], [2.5442951670965739, -4.040537355896979, 4.41554484531631, -0.4399039060101364, -4.4900730200763412]], [[3.6187987776090917, -3.7672593447337155, -1.2151838099474532, -1.21329514931521, -1.84425237579662], [3.7924789231552989, -2.8989856230220878, -0.43669627914514209, 2.7894614068400534, 0.87595457922835873], [-0.23483203973725697, -1.5675041388344546, 4.8788284406039377, -2.5875919035263206, -3.7368844568537818], [-4.4024196792836232, -3.3385275954765969, -4.7362409549848747, -3.946065020267465, -0.51384437526558724]], [[1.822321832690557, 2.0522269929064239, 4.2554126367937855, -2.1227786910779312, 1.0193184021979338], [1.9078393780594736, -3.206528232490081, -1.3497140528317795, 3.2276317849734841, -0.12135117326991463], [0.86017486433975776, -3.0301934753853033, -3.6102794729348533, -3.9248932155574434, 0.11685571011492613], [1.2038452462695917, 1.2651751945078722, 1.2218499881030098, 4.9056458215220076, 0.0038496255158975146]], [[-3.8234344419797637, -0.58553652866009287, 3.4532663239871493, -2.8801192386121057, 0.74459170506255923], [-0.47806901758357867, 1.5817048149204203, 3.7405768514648923, -2.1200651069880649, 4.5881181502133561], [-0.19496480467558541, 3.465836457528491, 0.074726172323046569, -0.90593777041442713, 4.5341887796614984], [-0.68250413934845078, 0.096005408930332159, -4.3416128420323687, 2.1941003556814351, -4.2576030558002742]]], [[[-0.3711582330506964, -0.14977711120974391, 4.7654848399944143, -1.9274023215057747, 1.8667824251505252], [-0.69191159191050033, 3.0954703685693126, 3.1655096493692287, -0.94466112631649501, -0.64951476895854832], [-0.26453533127517836, 0.61055279979279753, -0.2275345568980649, 4.2916793914479019, -2.725974244634509], [-1.1886907591135332, 3.3339068073454357, 1.5361311925072307, -0.76646750034189814, 4.4190388884232874]], [[-0.75016839928311008, -4.3501982396206849, 4.519467475965623, 2.2632040499616393, -3.204005113648285], [-3.4434133917242251, -0.29198481032614509, -3.8267277350535869, 4.0067692346160655, 0.019033094943637252], [1.4617532489206164, -2.7002048346694565, -4.3226664033785189, -3.5296374729212898, -0.53764039645221828], [-4.4682808710587318, -2.0160801803183146, 1.6859691448715584, -0.069349082104665172, 3.2894609158688262]], [[-3.8132038530407, 4.7459237730889772, -1.9347015554365585, 2.0142175831439442, -0.10374025451405267], [4.8642135301522487, -3.6190609670506846, 0.82415213261518883, 1.0582175535087259, -3.1049625705266628], [-0.16657998336038027, -4.930237681683308, -2.8622468433925041, -1.2608062459356417, -1.5208925804258144], [-1.3979291112020498, 1.0111799813477607, -1.9928008452118307, 0.71003325997162836, 4.4693805118825463]], [[-4.8419653463745673, -2.3761539364267747, 4.6322978180659575, 3.8473290101624382, 0.41483993895453875], [0.68151183883272104, 1.3273750134114382, -1.966946431200296, -3.4797294203556328, 4.6020765362506779], [1.3247395667630002, -0.5334978707519431, 0.35288541643137439, -2.772420496111403, 2.3842536061454291], [-1.4594481879357102, 4.073121929547133, -3.5449845011026992, 1.2263891029081293, 1.0085597348528079]], [[-3.0442838815018449, 2.2565561417912203, -2.1454132606021581, 3.6591203819631541, -0.83399173286782791], [3.9477624970347485, 0.84661283025740808, -4.0553385524962842, -2.46736390408996, 4.3049680579958398], [-3.8285433659941273, 0.42070194491155899, -2.3559810134787695, 4.2706124573810325, 0.2091374690247525], [4.3339827619109617, 4.0774115280612566, -2.0780489075250861, -3.3324636839296886, 3.0429346458648983]]], [[[2.8218076176733113, -2.6775763346274974, -3.0739140979062696, 0.86898774848497773, 1.6286138493414279], [-0.09803556007493075, 1.1263835971104594, 2.8045681924289658, 1.1661871966952493, 4.6490867424866593], [-3.0665583416506004, -4.5530339548197531, -0.41618218794829165, 0.53203102166119542, -4.7386332482621318], [3.2800180474659477, 1.0808166123303682, -0.67404271007391259, -3.9930731631898562, 0.63836056072304892]], [[-3.2416286584888079, -0.075827132410180909, -1.410184552471192, 3.3018330834521219, 4.0442021735886016], [-0.99297968209026344, 1.1099600775669085, 4.0540370705642577, -0.90525545075538894, 0.058603329587723074], [-2.9083630708219124, -0.97682780457101792, -3.1268794232105179, -0.12785337743353686, -1.8098590663836491], [-2.0687833077006457, 0.62448151843442101, 0.62137673151657502, 0.85771900994305561, 1.3115656345842517]], [[-3.8307439746586471, 4.6921612400450083, -4.2006308790989619, -3.5775804828666677, -0.97938739132247132], [3.8849816328537941, 4.1341150893303134, -2.002373758880438, -0.0272473240208706, -0.59861144729735827], [-2.6585127771222004, 0.68477785711314354, 4.6581343817762413, -3.7720557345782191, -2.398554968125064], [-2.136829002484423, -4.7801494272016711, 2.3378484515913156, 2.7158162201921439, -3.1479626821478823]], [[0.44794148362518449, -0.8806981996431027, -2.006946595846463, 4.5393433938562531, 2.3127209954453312], [-0.23085029322711215, 0.32703598990309679, 2.7534231737670485, 0.96566913754984274, -0.64541004012433767], [-2.9907443591801219, 4.6730074419722651, -1.4578256831439362, -4.7504210090782264, 2.4219826425465403], [1.0541690790351002, -2.8486971263959004, -4.9650569126229618, -4.5560941799674728, -1.1332169746360279]], [[4.3203677989267657, -2.0829319050401471, -0.34477587786705044, -1.17902282095642, -0.89993633201783307], [2.9740094331933911, -0.76065507749672268, -3.005389484455617, 1.839035701122036, -2.392534793192771], [1.6038453894240785, 4.067777575705275, -2.833880992351717, -4.3814240778698128, -0.36593857661318552], [0.056376948606880362, -2.0170469609290431, -4.1849380856664329, 4.2104726663726044, -3.8591230895695361]]], [[[0.88924262814999633, -4.2688519708745165, 2.0084634432414106, 0.42851440917399319, -3.8743227191200713], [3.86318190937277, 2.7784103492349903, -4.1823949845036479, -4.7879399916310215, -3.3647557991901014], [-2.6523261192788841, 4.0219992473947865, -3.1736267864098835, 4.9299757665722659, 2.4756750303376087], [2.2072565244152536, 2.1320843352773693, -4.9295666984944368, 4.552753974361508, -2.6770100761481275]], [[-0.13764250285913793, 2.352050496273379, -1.6251515482298942, -0.24076478400126078, -1.1543306138499987], [-0.52487049128397434, -2.6128185834079041, -1.3545343894862851, -0.20353024135879672, 2.3276811133516269], [-0.23739764503989402, -1.7664580734248747, -0.77256666988275136, -0.53801354129379497, -4.4935179096168012], [2.7552390140761096, 2.8199089827182515, 4.3286841334354893, -1.3623538355025677, -2.9727313259137302]], [[-3.3587505264874618, -0.50439718849091175, 0.4083188608077295, -1.7728766157761866, -2.6515057401911046], [1.8010145054993494, -0.064711595387276333, -0.45073709515167693, 1.5046088323626456, -4.5828170586092725], [-3.6447122513536092, 4.1108581809707765, 1.6440085701195972, -0.089530699801078306, -2.506984587907688], [4.5087179110190974, -3.5905362067405164, -1.6508657975498018, 0.15542657647569325, 1.2610300184213941]], [[1.0081544440590715, -4.1832491391725126, 3.7029188094153671, 1.649205542799824, -4.4803100797207289], [-3.0726381824395443, -4.5069105579140025, -4.552209503377008, -1.1701127199329733, 2.752981941780642], [-2.5136884009787916, 4.4351514362577706, -4.4574180844618319, 4.7839322577675389, -0.31139265855413001], [-4.9376677897550874, 0.29030044885444184, -2.6522203405535483, 4.3523235731714927, -0.11032289222083591]], [[-3.8275020504738833, -3.3969632259957971, 4.9301551090544891, -2.8366048114750999, -3.2934280082966714], [2.4446356042827366, 0.41602497820525297, -2.6508201607664592, 3.2144786632863749, -2.9607122692527534], [0.72186735184749651, -2.507471149133933, -2.0912003356569873, -1.0837034862175465, -0.53707781519479436], [-3.8206491781284422, -4.1758778395248886, 0.018018374155599481, 2.5076674206535952, 2.1756702850757668]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.79194273458931463, 0.59168385895493658, 1.7242437824346619, 2.259136564624177, 1.1956273592794984], [3.1173208663064047, -0.60386812274303814, -2.279520957498419, -3.1881219903376063, -1.2795821950798694], [-2.3648396464499131, -1.1904381296043793, 0.83059516185075721, 2.6932292752238718, 0.60715828580522524], [2.4402106149743097, -0.6604225680562712, -3.8024759586618875, -1.502685159676868, 0.7816870910022885]], [[-1.7530712221483309, -0.4450075231226116, -1.099956634175693, 0.38223066741334399, 1.418489071694222], [2.7702910061717221, -3.5811648835926913, 11.08896048878624, 0.42500175809297969, 0.50949036190964636], [-0.41072497340216307, -0.48388348926406788, 0.46685902771379456, 0.46649572001174028, -1.2633723045860474], [-0.62367121060174113, 0.39272094457322915, -0.35936757582123657, 3.6071597121820416, 0.35340263730595189]], [[-0.3870434414178629, 0.37179079126647718, 1.1526094416491151, 1.1544036366378803, 0.75945806065700683], [-0.36931842234716189, 0.48314566362853151, 3.2073374552821843, -0.50211568772741588, -1.5989782642821686], [5.9644004891823243, 0.89354298848999747, -0.28708374351265314, 0.54128795610151026, 0.37481285516219076], [0.31815057053181584, 0.41953594590091409, 0.2957265785243503, 0.35494405831905002, 2.7257909205693713]], [[-2.6223558012871266, -2.3285807302438939, -1.1229877423517758, 2.2511890899666249, -4.6882075507161458], [-2.504810564623591, 1.4903271960456856, 3.540584185029501, -1.4805828384812272, 39.379728279504199], [-5.5555869252658336, 1.5770531712205693, 1.3236582557093641, 1.2175557314084919, -40.89467450985839], [-3.9695934710682086, -3.7771656056117342, -3.9110989698396503, -0.97413804494463596, -1241.3613246363084]], [[0.22043400727885087, 1.4393892342502084, -0.24406312648381553, 0.29263197311918188, -1.1319156121176028], [1.7629566958210814, -0.53285225388656388, -0.22531684525703874, 0.39754202492910562, -0.18369513339022969], [4.3229083167907705, -0.24317794158544712, -11.278711988217824, 0.93032325523642212, -0.18587999233602642], [1.2348862476030775, -8.7788280369200695, 0.19412485780722977, -0.38412781504327231, 0.19795527308854718]]], [[[7.2570599710799861, 17.983505852486729, -0.56521374979594785, 1.3974858938141606, -1.4428663564216906], [3.892863752393088, -0.8701480664645791, -0.85089538632292772, 2.8513055962319354, 4.1469689139291681], [10.182071117022549, -4.411604626042303, 11.837839459329587, -0.62761388033234755, 0.98809354538495453], [2.2659531382389795, -0.80791627110706532, -1.7534423942089874, 3.5141966943249234, -0.60952565116937274]], [[-1.4717978283895927, -0.2538036567932706, 0.24429785740530827, 0.48784652051594396, -0.34459876992336747], [-0.32064004387185202, -3.781348145330266, -0.28852228259607743, 0.27555772652256932, 58.009284578306413], [0.75532325432261693, -0.40889350571307137, -0.25542017772373932, -0.31280725838327461, -2.0535960993204547], [-0.24709642317757197, -0.54764499535780109, 0.65487332573662926, -15.920848372946306, 0.33564655401894622]], [[-1.2541031724365184, 1.0076333455589703, -2.4717771254215082, 2.3741978469779577, -46.097352196082248], [0.98312934241104688, -1.3213789689601887, 5.8025100706478039, 4.5190622980967818, -1.5401638314868158], [-28.707837236961172, -0.96996359161588119, -1.670768214937538, -3.7929309635491792, -3.1443055944862368], [-3.4208823687300041, 4.7292778115242777, -2.3997134790140882, 6.7351085066586647, 1.0699807359278095]], [[-0.83065340279503874, -1.6926491712190623, 0.8682505203995986, 1.0453992836479249, 9.6952935662794335], [5.9015775838472297, 3.0300366893640027, -2.0447913208939492, -1.1558355565389449, 0.87395221689607561], [3.0360646666645832, -7.5389147955039775, 11.39745312190834, -1.4507160789003481, 1.6868989862550587], [-2.755832666365861, 0.98744772701387684, -1.1345592596894611, 3.2795423423482477, 3.9878599672320587]], [[1.0434180702406894, -1.4076586680356482, 1.4805822594803697, -0.86809409948881977, 3.8087437653352616], [-0.80462307833548763, -3.7519639431116403, 0.7832788241431049, 1.2873904848960929, -0.73785932209222449], [0.82967868174497861, -7.5503829999389573, 1.348253994717566, -0.74379514521658707, -15.188386986387123], [-0.73291957707300837, -0.7790385618524619, 1.5285784667525417, 0.95318692540281869, -1.0438807212695296]]], [[[-1.248724574434297, 1.315988818300029, 1.1463106659078823, -4.0549024110614109, -2.1635948373762424], [35.94267747153534, -3.1282953032642693, -1.256400370661984, -3.0215222105850081, -0.75792531129024732], [1.1490603223344917, 0.77391483381855408, 8.4666298043306902, -6.6230358250772792, 0.74360270818739704], [-1.0742808318499488, -3.2601835281909066, 5.2276516960300246, 0.88244326425005848, -5.5198593605523421]], [[0.14937595260908246, 6.3858589066418903, 0.3433744668512525, -0.14665228575407152, -0.1197322359472935], [0.48764479032175317, -0.43625115772461531, -0.11944177135984396, 0.53490025214718118, -8.2626938140406985], [0.16649275110271058, 0.49570801179161511, 0.15485770422496631, 3.7873177743652855, 0.26754645036213209], [0.23406094155161022, -0.77539743703002384, -0.77927180775637273, -0.56454545515881172, -0.36919339459527634]], [[0.88248405297316146, -0.72047193088932582, 0.80477684565905383, 0.94493205250003898, 3.4517194101248139], [-0.87016382267321368, -0.81772529201815247, 1.6882814477899553, 124.06981566592995, 5.6473535277717355], [1.2716021144418561, -4.9367403363638855, -0.72573485253772363, 0.89621434743654094, 1.4094196353989972], [1.5820500679880325, 0.70721020757677178, -1.4460177974146315, -1.2447714405432433, 1.0738915323966574]], [[-0.92027303918205827, 0.46807007290184272, 0.205400817024533, -0.090812356489574433, -0.17824392623377863], [1.785696109581786, -1.260498792911493, -0.14971489832690171, -0.42688375809275736, 0.63870786768680154], [0.13783473978514044, -0.088214811474275276, 0.28276938407509722, 0.086777249789794467, -0.17020290041304351], [-0.39104587557129716, 0.14470772153760331, 0.083025930571598391, 0.090478478764548217, 0.36376835128495161]], [[0.98703136991916529, -2.0472769833765363, -12.368436485494824, -3.616841397242943, -4.7384891524135426], [1.4338685343544562, -5.6061395937342011, -1.4188971410145723, 2.3187905186003555, -1.782351738107304], [2.6588214644932866, 1.0483214649193249, -1.5047698046030302, -0.97327683222177297, -11.653153888820372], [75.639754412123423, -2.1141493627719994, -1.0189729118657336, 1.0127933096887045, -1.1050019520380205]]], [[[4.0833985611918218, -0.85061091204674766, 1.8079154393161037, 8.4737688875795794, -0.93723015132895915], [0.93993297585297586, 1.3069099275914833, -0.86819443926075268, -0.75839130705169433, -1.0791665978292226], [-1.3690368020525934, 0.90281768965773967, -1.1441585015249356, 0.73654156536813919, 1.4667240343911172], [1.6450883837799906, 1.7030902616081292, -0.73660268547479657, 0.79756826061469088, -1.3564132988109108]], [[31.103309543072701, -1.8201723897910054, 2.6343003994759915, 17.781410144635679, 3.7087618757959717], [8.1565594633415639, 1.6385130601479192, 3.1605970331507658, 21.034404244448183, -1.8392284699799917], [18.033613484209596, 2.4235714603803804, 5.541447152205988, 7.9573041273569372, 0.95273624336708695], [-1.5538170557397515, -1.5181828204200527, -0.98901588583070921, 3.1424562849575266, 1.4401359905591673]], [[0.74399179947224814, 4.9541966236094366, -6.1199300057717618, 1.4095074783789883, 0.94243916213421541], [-1.3874862420871656, 38.61568909288706, 5.5439919967955413, -1.6608189413961107, 0.54527222366107109], [0.68561869246377616, -0.60787376702682983, -1.5199938087901697, 27.910904904483623, 0.99676833285413924], [-0.55423357537463447, 0.69596369575346784, 1.5136801864140526, -16.077577624381124, -1.9816204306602987]], [[-0.92185752867993243, 0.22216576957494485, -0.250984375343277, -0.56352876594763646, 0.20743536670254697], [0.30246801255008249, 0.20621105131452677, 0.20415904927893588, 0.79426088486690716, -0.33758839831941023], [0.36972552523459257, -0.20954747040433899, 0.20850069406047095, -0.19427005113188772, 2.9845750655885541], [0.18822140409208951, -3.201423793850144, 0.3504138589533688, -0.21353531020919964, 8.4241334288774414]], [[1.2503316271616622, 1.4088014936725517, -0.97068890549599895, 1.6871038388477395, 1.453089867055785], [-1.9576115386479991, -11.503268114763362, 1.805345733205034, -1.4887785448359221, 1.6163836372865257], [-6.6295377599299394, 1.9085551067602695, 2.2884688688757859, 4.4160113237588323, 8.9105279185622077], [1.2525742730134344, 1.1460217589310748, -265.5981513873935, -1.9084057268990975, -2.1996195377402743]]]])+(1.-msk_ref)*numpy.array([[[[-0.16921021896056557, -0.12642196330157238, -0.3684100569701716, -0.48269777102016564, -0.25546337938669489], [-0.66606147555806339, 0.12902531055562241, 0.48705319652124995, 0.68118917757074293, 0.27340156548034733], [0.50528278991021813, 0.25435462410526799, -0.17746887882901899, -0.57544806646652891, -0.12972830230976401], [-0.52138690644573593, 0.1411090000153257, 0.81245494333769952, 0.3210707969049989, -0.1670189497927991]], [[-1.3232707318532597, -0.33590502391627275, -0.8302802544033554, 0.28851917059306975, 1.0707180908066058], [2.0910987305415838, -2.7031706507574684, 8.3702799270790127, 0.32080407251305282, 0.38457860442780967], [-0.31002752727767058, -0.36524976902278655, 0.3523991948929856, 0.35212495934413202, -0.95363130315036637], [-0.4707657332241264, 0.29643754638940134, -0.2712614234112749, 2.7227923269433769, 0.26675890893561793]], [[-1.1668152214636207, 1.1208332399602006, 3.4747578617845636, 3.4801667999014221, 2.2895291081321489], [-1.1133798190274939, 1.4565334383157307, 9.6691217644307788, -1.5137221424803697, -4.8204205985747199], [17.980806630358291, 2.6937533321397953, -0.86546791889356611, 1.6318143101985796, 1.1299438197463783], [0.95912470975854958, 1.2647699850824348, 0.89152336995948234, 1.070045596167605, 8.2174092008807005]], [[2.5314218321151829, 2.2478338353204634, 1.0840465228981211, -2.1731258617781526, 4.5256371929183041], [2.4179526460477656, -1.4386479512861416, -3.4178093224520745, 1.4292414933728028, -38.014179410959997], [5.3629389367868434, -1.5223665782022235, -1.2777584968132658, -1.175335230555526, 39.476592696036299], [3.8319420928127901, 3.6461869410958676, 3.7754759727706579, 0.9403584034082989, 1198.31533050755]], [[-1.0275189462173795, -6.7094897354130483, 1.1376624216513516, -1.364058569549867, 5.2762491202130741], [-8.2177492877740246, 2.4838081617346166, 1.0502795383593668, -1.8530805095583311, 0.85626638201799388], [-20.150566843546038, 1.1335362694972129, 52.573967147285977, -4.3365576058906274, 0.86645076322728143], [-5.7562307717353169, 40.921145720083011, -0.90488292523918956, 1.7905522500743298, -0.92273781215986039]]], [[[9.1282700940786565, 22.620496359976105, -0.71095234014672037, 1.7578232427783227, -1.8149049151534407], [4.8966264455444977, -1.094513012732748, -1.0702960894789439, 3.5865057897943364, 5.216251824974238], [12.807486178106792, -5.5491230243764997, 14.890189187594837, -0.78944214837649218, 1.2428703629890701], [2.8502220387869217, -1.016234944379087, -2.2055620089309529, 4.4203212757441754, -0.76668992612929865]], [[5.4766289008801623, 0.94441533689718027, -0.90904381055714889, -1.8152998339281541, 1.2822682206529969], [1.1931166748421362, 14.070574189978721, 1.0736049754486541, -1.0253632529038228, -215.85527462596525], [-2.8105933330913393, 1.5215119546824363, 0.95043048726356538, 1.1639705118612633, 7.6415276142777993], [0.91945740534153819, 2.0378127696251762, -2.4368144272022714, 59.242225150727904, -1.2489566044762577]], [[0.79546303064552837, -0.63913009109177132, 1.5678194318319634, -1.5059260324172197, 29.239013414971669], [-0.62358748739264758, 0.83813528454616959, -3.6804645324205514, -2.8663885638180817, 0.97690797377131022], [18.209046682678434, 0.61523660505822275, 1.0597488125146324, 2.4058119185140798, 1.9943964040903652], [2.1698258295806734, -2.9997258147516415, 1.5221103005335845, -4.2720008546221688, -0.67867631438955767]], [[-0.68172871328175255, -1.3891805386576193, 0.71258518665730763, 0.85797362186068971, 7.9570612551375079], [4.8435061832417938, 2.4867929349183915, -1.6781884615558373, -0.94861019538737945, 0.71726464767316223], [2.4917401790938665, -6.1872914332090554, 9.3540470975532557, -1.1906227103603588, 1.3844612824871736], [-2.2617499083730248, 0.81041197941888721, -0.93114844488374848, 2.6915656682788098, 3.2728917200135035]], [[-1.4908721911970337, 2.0113118822905793, -2.1155076573762748, 1.240363176738515, -5.4420661526626377], [1.1496735643163787, 5.3609371590323267, -1.1191761482519591, -1.8394685005795515, 1.0542791769640705], [-1.1854738858004172, 10.788251007478918, -1.9264324097157526, 1.0627604884157065, 21.701697941573997], [1.0472210967076545, 1.113117513354978, -2.18408631510309, -1.3619467791734312, 1.4915332431500323]]], [[[-1.7151835779864231, 1.8075742691173662, 1.5745131229811726, -5.5696045134218863, -2.9718021175046503], [49.369005311654242, -4.296864850026421, -1.725726655224783, -4.1502068448235878, -1.0410470602408088], [1.5782899090271763, 1.0630094425161956, 11.629325392156352, -9.0970599251116226, 1.0213742723907298], [-1.4755766633081313, -4.4780197036709311, 7.1804323579725837, 1.2120784889911456, -7.5817937132398141]], [[0.18313391683020031, 7.8290202236155855, 0.42097479517682534, -0.17979471951907691, -0.1467909188652923], [0.59784924489937274, -0.5348409957585144, -0.14643481122767735, 0.65578412440677503, -10.130007242155887], [0.20411899707223116, 0.6077347003843635, 0.18985450757437647, 4.6432262100130011, 0.32801015513560566], [0.28695714574277931, -0.95063206134005818, -0.95538201388580268, -0.69212894462662777, -0.4526286275610315]], [[-0.9364625970998588, 0.76454074525773574, -0.85400230455453896, -1.0027303280844651, -3.6628493312431347], [0.9233887802703229, 0.86774276328010691, -1.7915478743285276, -131.65874375720989, -5.9927829104000896], [-1.3493816852054863, 5.2387039301429201, 0.77012558189992442, -0.95103272691557927, -1.4956290345672925], [-1.678818682806428, -0.75046784749436757, 1.5344657815144747, 1.3209098703592204, -1.1395778201810809]], [[8.3389895939418217, -4.2413841338145266, -1.8612250960618195, 0.82288979848955712, 1.6151448350240796], [-16.180960043097635, 11.421921396945013, 1.3566310497542517, 3.8681772310989633, -5.7876065422311322], [-1.248979826436682, 0.79935232653272636, -2.5622949395359687, -0.78632596216352812, 1.5422816435702413], [3.5434347724032413, -1.3112588684958524, -0.7523336461961988, -0.8198643889033792, -3.2962613994067831]], [[0.88545291933836379, -1.8365853780041994, -11.095562438536712, -3.2446210643081068, -4.2508365804030683], [1.2863046893793784, -5.0291944316793815, -1.272874048425739, 2.0801566156878604, -1.598924408982227], [2.385193925424975, 0.94043546112815679, -1.3499092906511085, -0.87311390365003472, -10.45389178583072], [67.855433376688566, -1.8965757140394814, -0.9141072584269726, 0.90856361821982456, -0.99128278403834935]]], [[[-0.62265049739276923, 0.12970404420161241, -0.27567709364272319, -1.2921090933388668, 0.14291204033880817], [-0.14332417621264584, -0.19928206964565232, 0.13238524022048057, 0.1156420852575961, 0.16455499232774082], [0.20875538671352578, -0.13766471117046938, 0.17446517879526688, -0.11231036234999967, -0.22365106806645055], [-0.25084867055365551, -0.25969299410870522, 0.1123196821516895, -0.12161592035029728, 0.20683051201552941]], [[30.781275919600791, -1.8013269126170666, 2.6070257037789961, 17.597307165774623, 3.6703625529263877], [8.0721090804841857, 1.6215484250136816, 3.1278732320545957, 20.816620815098577, -1.8201856923051551], [17.846899273409306, 2.39847858407252, 5.4840727977749353, 7.8749167699070135, 0.94287191995032527], [-1.537729335686058, -1.5024640457322456, -0.97877593471090052, 3.1099203073103396, 1.4252252875457199]], [[-0.93531155261542975, -6.228183347285813, 7.6936886127730189, -1.7719666116761899, -1.1847902579122687], [1.7442825475222219, -48.545830943839796, -6.9696463938024564, 2.0879035814537281, -0.68549063373047026], [-0.86192762367184594, 0.76419035429605164, 1.9108648378568645, -35.088278956343466, -1.253090340056193], [0.69675642428177931, -0.87493287600870884, -1.9029276482918416, 20.211975590082996, 2.4912001489936801]], [[3.4108317556764893, -0.82200344230591427, 0.92863099878929467, 2.0850313094299557, -0.76750160842380788], [-1.1191181610888268, -0.76297169607346615, -0.75537938002900262, -2.9387298623845672, 1.2490620226149851], [-1.3679679593875289, 0.77531629795362056, -0.77144327215117503, 0.71879052777969932, -11.042794677271788], [-0.69641080340757755, 11.845125303775237, -1.2965156551455201, 0.79007112743627039, -31.16891810214549]], [[1.0993066655180721, 1.2386352858255034, -0.85344140768361476, 1.4833220684632415, 1.2775741585273508], [-1.721155704759749, -10.113812239163531, 1.5872817698630457, -1.3089520750056314, 1.4211440132384001], [-5.8287696563660853, 1.6780246974420066, 2.0120494649060667, 3.882610483293564, 7.8342437489136687], [1.1012784268329852, 1.0075961697310449, -233.51710204455122, -1.6778933608641633, -1.9339320600333982]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.7675495833982797, 4.789098653628642], [-1.9070012699141978, 0.69065712558000314]], [[2.6282308967508143, -4.9785639913987501], [0.80898922058190603, -2.2735464938221241]], [[1.8397981293364749, -0.51552371845779987], [1.7030735135467356, 0.4452529768835225]], [[-1.5720050276004551, 4.5132550526878266], [-4.7104769818054448, 0.07244380798902128]], [[-3.5372848174795699, 3.3103775798917514], [-3.1121986855422845, 4.9527830007152769]], [[-2.8777806100073224, 1.5846890814949779], [4.5197266326660639, 2.0385859389409653]]])+(1.-msk_arg0)*numpy.array([[[-3.1883638425181382, -4.5246909655667613], [-2.5034875185856298, 3.9009835542888034]], [[-1.0723200033650171, 3.509606424041948], [4.3368871437507615, -4.1285238389472578]], [[-2.0593376644346262, 2.0665706880248234], [3.0678361985341791, 0.72569426972266182]], [[0.50889813651338223, -4.3782175406930426], [4.44365754670015, -0.96821575990824993]], [[-2.8208102324374029, -1.7316731836350128], [-2.7925753223286778, 3.703118537158419]], [[-3.2727131792490249, 1.3159096321614605], [3.1032937384972659, 4.3134183865056208]]])
      arg1=Data(numpy.array([[[0.62776721419568116, -0.57113252860468222], [-4.4843510693264292, -4.1832872046319487]], [[-4.4429321509954338, -4.8404006809921878], [0.017495551217726835, -1.9639455584702201]], [[-3.5885481554296836, 3.5375777247208084], [3.36663766105905, 1.2915404402151403]], [[0.071553711165992517, 2.1332652079399308], [-3.6232581980723335, -3.8235694189307887]], [[2.8016553657266474, 1.4867119715242687], [-1.5369448286659058, 4.726154786871513]], [[-1.2973851076003018, 0.83028293366115236], [2.2757125159459966, -2.5493610214187101]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.5944545614836585, -8.3852668404805346], [0.42525690795226667, -0.16509914136788706]], [[-0.5915532372381519, 1.0285437755079072], [46.239710342033824, 1.1576423205911379]], [[-0.51268592468314866, -0.14572788460739358], [0.50586777818287587, 0.34474567192751149]], [[-21.969580640669623, 2.1156558668325278], [1.3000666042269744, -0.018946643842883137]], [[-1.2625695725291777, 2.2266435216081217], [2.0249254413665101, 1.047951923723129]], [[2.2181390807932018, 1.9086133379946204], [1.9860710001795845, -0.79964584137498884]]])+(1.-msk_ref)*numpy.array([[[-5.0788951229369141, 7.9223135418690056], [0.5582719728858484, -0.93251631156699821]], [[0.24135412536623252, -0.72506526945669036], [247.88513890071337, 2.102157985562032]], [[0.57386373966271786, 0.58417675845918571], [0.91124632567946851, 0.56188273098268471]], [[7.1121137984419081, -2.0523550116495062], [-1.2264258586551435, 0.25322301070683811]], [[-1.0068369817876537, -1.1647670946374331], [1.8169652353446419, 0.78353729493690694]], [[2.5225456651821547, 1.5848930271984825], [1.3636580704954511, -1.6919605933667328]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.6361768517775381, -0.19391466575308236], [-4.5551472792528536, -1.1209551269935436]], [[-1.2144807546195047, -3.7673000266441403], [2.9919827694589625, -3.496915579611708]]], [[[-1.8620663370453561, 3.1312864500882895], [-1.6480260471788712, -3.9316602697833982]], [[-0.71574176234957143, 0.80470134155966733], [-3.0169390773475637, 3.5411029877259121]]], [[[1.8999763477391749, 1.1703426336259382], [3.651356182614899, -2.9759150825138478]], [[2.29262809582639, 4.219719253702241], [0.14930668616477227, -2.9030659015275804]]], [[[4.5152668593509571, 3.0192741207479266], [3.0601953499824504, 1.3380877804247673]], [[2.2761782100101025, -2.4549547048153695], [2.5128643153914441, 0.46478809944495225]]], [[[-2.3969978473250073, 0.7144084238084325], [-0.64609514619497332, -2.1191815679770123]], [[3.0874750132128508, -2.518904063511366], [-1.8830183320666052, 2.5869816354382271]]], [[[0.9812758214507058, 1.1526160820043776], [-1.9205393176861616, 0.73238524608324873]], [[0.6491629873548117, 1.6551240253079769], [2.4565273215503201, 3.2143393940983156]]]])+(1.-msk_arg0)*numpy.array([[[[-4.5436129543213175, -1.3995299700027464], [2.0429699225359297, 2.1579709516899284]], [[-1.2376593148140413, -0.56570575291441383], [1.1864172597230151, 0.26547187736758993]]], [[[-4.90273801524215, -3.8696517563851032], [1.5049974004061193, -1.9423293092929828]], [[-2.4013043688544933, -3.4373853355681714], [3.3241157528510037, 3.172388348248921]]], [[[-1.7668295488257768, 4.0707117457983149], [-0.60767581004086146, -0.74033361872213899]], [[-3.4275798604818295, -0.82499760677958278], [4.8413282834290108, 1.1148915790560316]]], [[[-2.5140020080445149, -2.3129209889275728], [-4.6095035960968627, -0.84874137742238176]], [[-0.81244800085765867, 2.8143145369638152], [-3.3368807084197174, -4.4983301374746905]]], [[[3.7092612590206819, -3.0445851306449678], [0.20360064962390823, -3.2401826503457745]], [[1.0030663846735255, 2.7390857693690016], [-4.9892827603232224, -4.9941967737431936]]], [[[1.6490952564313481, -1.9556269996260136], [-0.82461742633758561, -1.0965952345406138]], [[0.97763392858240117, 3.4205535601945023], [1.4992265208323747, -4.2670916559002912]]]])
      arg1=Data(numpy.array([[[-3.501309574406426, -4.9173241860161498], [2.9496659714505977, -3.6274807728420067]], [[2.7143869790911044, 0.15320618608611447], [3.1261787060073214, 3.4459860511089513]], [[2.6310376534708322, 1.5075752614509321], [3.0520409198255862, 0.21025095925275039]], [[-2.7055315042918471, 3.4109494412040071], [0.66169357568477416, -1.9718768887650375]], [[-0.69133124596165096, -0.24233353713861305], [-0.37008758773769479, -2.7597757552927469]], [[0.93111150564013556, -0.83824887204776743], [-1.8936818780306486, 1.0280649040473291]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.46730425202544901, 0.055383467708923322], [0.92634675017090551, 0.22796038751752579]], [[-0.41173501215876412, -1.2771954733543756], [-0.82481009737092192, 0.96400664775184863]]], [[[-0.68599884665997679, 1.1535888118417004], [-10.756915822266766, -25.662542552775786]], [[-0.22895100685517086, 0.25740733887456235], [-0.87549369980086944, 1.0276022407538044]]], [[[0.72213955023895215, 0.44482169690047452], [2.422005903108766, -1.9739744731879885]], [[0.75117868863875059, 1.3825893441636372], [0.71013557652921444, -13.807622623199062]]], [[[-1.6689019707175041, -1.115963394238205], [0.89716819399769632, 0.39229188338612403]], [[3.4399279268421621, -3.710108115036153], [-1.2743515225056572, -0.23570847758961433]]], [[[3.4672204696762279, -1.0333807823407155], [2.6661400391536048, 8.7448959520813485]], [[-8.3425521836229368, 6.8062376231236312], [0.68230845511825344, -0.93738834775864233]]], [[[1.0538757340089815, 1.2378926423124355], [2.2911326000289804, -0.8737085971784212]], [[-0.34280466792548842, -0.87402432505149064], [2.3894671551176976, 3.1265918926363203]]]])+(1.-msk_ref)*numpy.array([[[[1.2976895809310414, 0.39971614627649926], [-0.41546374516972306, -0.43885065740159052]], [[-0.41959304097249378, -0.19178637797967643], [-0.32706369351573378, -0.073183538105868945]]], [[[-1.8062045143186627, -1.425607986699388], [9.8233461641045423, -12.677877825385158]], [[-0.76812767108934099, -1.0995485731390946], [0.96463412896905709, 0.92060394360215592]]], [[[-0.67153335737897835, 1.5471887072495076], [-0.40308157448538751, -0.49107572779459213]], [[-1.1230451853436172, -0.27031013949404342], [23.026426612442098, 5.3026705943147663]]], [[[0.92920818111210146, 0.85488599384576847], [-1.3513843214485719, -0.24882848369713462]], [[-1.2278311755057794, 4.253199124763051], [1.6922358223436378, 2.2812428925478914]]], [[[-5.3653892843524673, 4.4039455014215498], [-0.84016703601140486, 13.370756225509195]], [[-2.7103486253218092, -7.4011824771339541], [1.8078580300427263, 1.8096386143567043]]], [[[1.7711039402285138, -2.1003145034509347], [0.98373818782853661, 1.308197685803894]], [[-0.51626090946125558, -1.8062978792149278], [1.4582994856941005, -4.1506053159692797]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.54515654280981352, -3.9819918610129532], [1.7584152918232041, -3.9257677133174909]], [[-1.8192999289142362, -4.0751357046253434], [-4.6011412270765124, 4.8456115667316251]], [[4.2166068044289187, 0.26969650907317266], [-3.6006349274006686, 1.4450772380143633]], [[-0.66239565669969913, -4.6315857569885255], [-0.66612450223240582, 3.7337614793166587]], [[-4.7095169373425314, -3.1303141310328533], [3.7370914206583894, -3.5043604059131326]], [[2.3300079880532563, 0.41716046762589887], [3.1901495047424113, -1.8738062443435641]]])+(1.-msk_arg0)*numpy.array([[[4.6885436366976325, 2.7801127878766749], [-4.5813214377733482, 2.8417732714377086]], [[-1.924188611802621, 2.5296715165197403], [3.3659098869162332, 1.8167811382580146]], [[-4.9803325618301226, -3.174001682812444], [4.0212800670236053, -3.704990651895991]], [[-2.9517316172330532, 3.2415112530074719], [3.5568461876969302, -0.051859363250542856]], [[2.0712172629253605, -1.3109657071197898], [-2.8754816171722073, -4.0236870173740797]], [[-3.6212129535076896, 1.7487270855197661], [3.8160803757972275, 0.33648908436087055]]])
      arg1=Data(numpy.array([[[[-3.426328662475874, 4.8421961583942057], [-0.033057462092007661, 0.24255990455345078]], [[-0.45128155604930686, -3.4413965306673999], [4.6190349565856543, 2.3223015676880499]]], [[[4.642821863384528, 1.3132202867097895], [-0.39460164216719207, 1.2135727722283738]], [[3.73415707909923, 0.66140883842243348], [3.5169574008433013, -0.092745122152257942]]], [[[-3.6578734138781477, 2.9138896223494415], [0.62407280515182517, -1.1582451836616015]], [[-2.0975577379314645, 1.9836925258983529], [3.2662600596973057, -3.2691344040009818]]], [[[0.88626504061545219, -1.5688675155117204], [-2.2604266343789883, 0.47310633165946747]], [[-1.6913207872003189, 0.73966921924370865], [-1.1424050436501787, 0.64125721035286443]]], [[[1.1353720814302903, 1.0465141146725561], [-1.3459303073672526, 3.33012611646609]], [[4.6478038840554579, 1.0557566653804837], [-3.2780870553414578, -2.8465655750124199]]], [[[2.9303653780380117, -4.8297421446844355], [2.3982993252871037, -2.3152335022860182]], [[0.85076097344288559, 0.057773279219030194], [2.6098929561082951, -3.7967901047059174]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.15910807062387353, 0.11258456389974114], [120.45667177746485, -16.416529633550695]], [[-3.8964927067196169, -0.51095980255497897], [-0.84991080392675356, -1.6904642221921935]]], [[[-0.39185219300832735, -1.3853730004981912], [10.327214256494946, -3.3579656678870116]], [[-1.2321766678830823, -6.9565765677564499], [1.3777851177753069, -52.246538192883875]]], [[[-1.1527481482631163, 1.4470715610116722], [0.43215552231531795, -0.23284923855290426]], [[1.7165844173383682, -1.8151174541377337], [0.44242565245961551, -0.44203665540510745]]], [[[-0.74740131489301365, 0.42221261524663817], [2.0489874285439797, -9.7897353027231286]], [[0.39384870526841725, -0.90057080232904607], [-3.2683342042911985, 5.8225645170712124]]], [[[-4.1479943133793595, -4.5001943799067563], [2.3257624216487094, -0.93999867318980823]], [[0.80405531599099589, 3.5397279914984772], [1.0690260346207021, 1.2310836738401303]]], [[[0.79512541525223834, -0.48242906520747469], [0.17394011799422077, -0.18018073218705691]], [[3.7497600434495912, 55.218425332027785], [-0.71796287275232307, 0.49352379053587447]]]])+(1.-msk_ref)*numpy.array([[[[-1.3683870108682681, 0.96826800966536464], [-84.099401827608105, 11.461551293874484]], [[10.151802962833241, 1.3312390469821505], [0.61523095151856566, 1.223688306022553]]], [[[-0.41444377329608001, -1.4652443548702583], [-6.4106968805971736, 2.0844827557186649]], [[0.9013841184549668, 5.0890004659515435], [0.51657752175854732, -19.588967010851942]]], [[[1.3615377019156818, -1.7091699437175414], [-5.08594775579152, 2.7403538798049305]], [[-1.9171248515853709, 2.0271690367954038], [-1.1343220025901255, 1.1333246645844783]]], [[[-3.3305292231579751, 1.8814409681178728], [-1.4340263044626613, 6.8515490833478152]], [[-2.102999155816359, 4.8086983953904516], [0.045394900467914043, -0.080871392030050185]]], [[[1.8242629855017527, 1.9791584593901286], [0.97402198311749411, -0.39366848619864847]], [[-0.61867533331961411, -2.7236215611633514], [1.2274497136425064, 1.4135233885685292]]], [[[-1.2357547562660005, 0.74977355830334735], [0.72915297397685075, -0.75531348513793783]], [[4.485490631233584, 66.052687806238154], [0.12892830856274715, -0.088624621082901159]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-1.6597433267362769, -3.3127220077317112, 4.8324766543580591, 0.26673126406537317], [-3.3865968712048753, -4.8794562284762142, -3.4530038009320707, -1.3937966356882736], [-0.089332972923571674, 4.0968634622920899, 1.2153392854258289, -4.1544526989792345]], [[-3.4385910750551449, 1.4517175746376063, 4.2467230768968136, 1.1460205674168042], [4.9698356427786212, -1.9254862070906107, -0.52040146297790102, 0.63812932883304363], [-1.4012643146880599, -0.22220145451308593, 1.4134680484187658, -1.857239943316654]]], [[[2.0710512619600321, 4.1808889080042704, 0.086335758156236508, -4.84265971174485], [0.31045614234950669, -0.53952037877093151, -4.9936616541543017, -3.8885090697886748], [3.9984058836534473, -3.2831552784003906, -1.6290341702664368, 3.1593197354534492]], [[-4.9696884943209119, -1.4053430918584056, 1.1608345361348764, -3.814803742894135], [0.10728625968054395, 4.7528145798011145, -3.5125092449530024, -3.575085836876025], [-4.5828596812841322, 2.236435182042765, -1.2904911840799684, 3.4281082601334862]]], [[[3.1599190585979748, -2.958849623108919, -0.010471490040377951, -2.2167162525545403], [0.099644943117606743, -3.3591242044251182, -4.1142204331325907, -0.35921879307032789], [-0.82106122474344367, 4.0899900450775188, -3.623616729506022, -2.3416609306190814]], [[-3.771772576374377, -4.0677023926160247, -4.2243819419898356, -1.7904561484758297], [3.1697205188205473, -3.2161663143832953, -1.8929641011375042, -3.0190149063503604], [2.774226092456332, -4.4019477326426468, -3.401185888948671, 2.5688465626061427]]]])+(1.-msk_arg0)*numpy.array([[[[4.7489903827034592, -1.5347810942202322, 2.4610467389254067, 4.0703158710526939], [3.7775260956215906, 0.84840523781539723, 4.8387581198703451, 2.4540947721155337], [-1.7166387807697192, -2.5340889306317815, -4.1371971572276145, 4.2920718077639659]], [[4.4854337259325021, 2.7536542661509467, -0.60669962203430039, -4.3557443154228377], [-3.1030179162740676, 3.243046677127607, -4.8871919810851949, 2.0477293830605339], [-0.89559503530505324, -2.8287212006748694, -2.4938874140776024, -1.5233301435012092]]], [[[-1.3067116137900836, 1.6264884122268999, -2.2511481691890123, 3.5338565140484626], [-3.4127692481375371, -3.9611729854657138, -1.4721258018408276, -2.2167673820962683], [2.0670592229921967, 3.9874860174939535, -0.33926786450096724, -1.2080805094982594]], [[0.25597548351997723, -3.3597162541178394, -2.8906009878744054, -1.8063590348429939], [-3.9628974272323383, -4.4976281437863062, -4.4101701022598672, 3.8200235486315677], [-0.77154654710459258, 3.3773304217388294, 1.5576382454744184, 2.2469261991059097]]], [[[-1.1265128208630779, 2.2522626559667618, -4.6468127704403006, 0.43035094918894767], [-1.2698336056251378, 2.6634136134624047, 2.2303308931313044, 3.3972951760303243], [-2.1549062214178214, -4.9383213586907448, 4.0020516494658747, -4.767392674639563]], [[-4.9187690349985722, 2.2797703824151627, -1.8701773404573685, -4.6399369279987237], [2.575929552467545, 3.4912868024000474, 3.318519686707738, -1.0912694438241011], [-1.9524875064860217, -3.6776636229168513, 1.1486376922465302, 3.4923077413860018]]]])
      arg1=Data(numpy.array([[[[-2.7453032048001291, 1.0646568200070643, 0.893789888597043, 2.0686950035809115], [-2.7349313708409784, 1.5976735923286425, -3.7512226852949002, -3.3383909745020777], [-2.1679716356508107, 4.1384954524027595, -1.0643575097437639, -3.0761708504773044]], [[-2.9460861396137137, -0.039518046720582234, 3.5741791615067982, -4.1407476314023839], [-1.0603960717084195, -3.282051969105507, -2.5543367239534618, 3.0471858493813713], [-4.9657686003941333, -2.4977004661658455, -1.966226956555992, -3.7930462827516487]]], [[[-2.1455935517084046, 4.1751611205184851, 3.5015229219025219, 0.035643016721429888], [4.6208294843930524, 3.7862449212278655, 1.2773005013382566, -3.8396991346252451], [0.0013086473973880075, 1.095116491680816, 0.803883273937398, 1.3725808493354847]], [[0.93078029661343997, -3.7557594808366304, 0.35696436171113977, 0.5267407397376056], [3.3642119103311181, 3.0419727484021521, -3.2486739777516274, -3.3898252430862783], [-2.4368085507684754, 0.25424868671470069, -0.13398700796563556, 2.0384656610789333]]], [[[2.6783044007265229, 3.0378971392287113, 3.8874298857084639, 4.8633204616933092], [0.95832368146857672, 1.0579294593490918, 4.0996018813528163, 2.740203688794784], [-0.76107928908534284, -4.6084701748925196, 2.2181206069839856, 0.8431399729382969]], [[-2.3618634657761439, -0.29712779954123469, -2.2770063809334138, 4.3380436741460855], [2.1436097528772518, -1.6219139777168721, -3.3263594117822128, -3.3297961874168172], [3.3190449424637194, 2.3543399696600851, 1.8227935065873471, 3.4101318936623706]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.60457559800106453, -3.1115397426465838, 5.4067255805986605, 0.12893696925049913], [1.238274900537454, -3.054100820033149, 0.92050088480967229, 0.41750551278558945], [0.0412057849164408, 0.98994030787529352, -1.141852501908323, 1.3505272954311436]], [[1.1671726188922662, -36.735559955738054, 1.1881673763400504, -0.27676658164957307], [-4.6867729666063802, 0.58667145591097514, 0.20373252206641429, 0.2094159530711244], [0.28218477892361749, 0.08896241063452319, -0.71887329369879616, 0.48964336442774103]]], [[[-0.96525796337846981, 1.0013718722033591, 0.024656630866584969, -135.86559604628712], [0.067186236453450357, -0.14249484383486924, -3.909543329014848, 1.0127119165986929], [3055.372968787512, -2.9979963806054188, -2.02645610759815, 2.3017367151690835]], [[-5.3392712677768044, 0.37418346383175538, 3.2519619901838799, -7.2422796550623154], [0.03189045831241482, 1.5624119520129203, 1.0812132177646132, 1.0546519600582935], [1.8806810571305961, 8.7962506746488298, 9.6314650477973824, 1.681710084985701]]], [[[1.1798207320052225, -0.97397952843793068, -0.0026936794613003231, -0.455803040333214], [0.10397837916820184, -3.1751873196648419, -1.003565846685325, -0.13109200405037119], [1.0788116777296444, -0.88749409019944603, -1.6336427866441006, -2.7773098249137931]], [[1.5969477622343913, 13.690076791523907, 1.8552350038905616, -0.41273354603288287], [1.4786835684834903, 1.9829450627897125, 0.56907984580153437, 0.90666657549765706], [0.83585071625966911, -1.8697162641630687, -1.8659194673764263, 0.75329830127106467]]]])+(1.-msk_ref)*numpy.array([[[[-1.7298600658753858, -1.4415735337233349, 2.7534958387014683, 1.967576594909823], [-1.3812142183516727, 0.53102538709351077, -1.2899149226300728, -0.73511305022670781], [0.79181791520736189, -0.61232130366617188, 3.8870371274249891, -1.3952644428374321]], [[-1.5225059666858978, -69.680930477688747, -0.16974516234897657, 1.0519221897007134], [2.9262819799725874, -0.98811557758833091, 1.9132919850602423, 0.67200672498405589], [0.18035375938257972, 1.1325301968723116, 1.2683619282922716, 0.40161127229803167]]], [[[0.60902103884010528, 0.38956302889334177, -0.64290544982806241, 99.145831052055101], [-0.73856203949187826, -1.0462009373078591, -1.1525289470241717, 0.57732840630822624], [1579.5387108230527, 3.6411523776560486, -0.42203622777128158, -0.88015253169467877]], [[0.27501171270096808, 0.89455042881751079, -8.0977299078766798, -3.4293133197611154], [-1.1779571361312657, -1.4785234832063376, 1.357529297326443, -1.1269086972618134], [0.31662173331642185, 13.283570764432788, -11.625293146884175, 1.1022634533449247]]], [[[-0.42060671690547852, 0.74138871487221802, -1.1953431719819798, 0.088489120258200757], [-1.325057107718751, 2.5175720270622892, 0.54403597170643403, 1.239796585167195], [2.8313820285499625, 1.0715749850341427, 1.8042534012194804, -5.6543312233500913]], [[2.0825797537717494, -7.6726929823972316, 0.8213316203754889, -1.0695920273121879], [1.2016784067202595, -2.1525721156399706, -0.99764315153476624, 0.32772860031132534], [-0.58826787233459232, -1.5620784042704865, 0.63015239416615088, 1.0240975570113147]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.710929955773)+(1.-msk_arg0)*(0.686158592038)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.56439324129)+(1.-msk_arg1)*(-4.57036866437)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.454444532876)+(1.-msk_ref)*(-0.150132000814)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.96261650059988391, 0.26475247754135101])+(1.-msk_arg0)*numpy.array([4.3470101602579376, -4.1056380247766713])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-4.30482544339)+(1.-msk_arg1)*(-3.55443495517)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.22361336441136026, -0.061501327062616254])+(1.-msk_ref)*numpy.array([-1.222982053428979, 1.1550747380538751])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.30705632988)+(1.-msk_arg0)*(0.189835669272)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.8177387537227263, 0.98579834006073597])+(1.-msk_arg1)*numpy.array([-4.5929329647721957, 3.5755983970135645])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.3694583839716783, 4.3691048715026053])+(1.-msk_ref)*numpy.array([-0.041332122791216883, 0.053091999769982003])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.1516674063743153, 2.1189811151111657, 4.0852727254033603, 3.663138190672699, -0.90162895883529259], [-4.8665586836390222, -0.3624457258153182, 1.144005376140278, 3.8671380378261606, -2.7079124750683081], [-1.9869266854907819, -2.763436226279171, -0.21618678128834112, -0.45703335252908062, -0.18961044357901891], [0.093813780755986897, 0.66503730069094757, 1.2854585694228877, 0.7154600827230766, 3.3155944785642717]])+(1.-msk_arg0)*numpy.array([[1.1990702466179393, 3.274729143819453, 3.1222796077074086, -0.43108890105210129, 1.3264240715364117], [-0.86374313720285834, -0.62786204774054521, -2.5425697658197266, -2.2522814841332894, 2.8293692235652905], [-2.4594083308038304, 2.4726262631674709, 1.9386975247767513, -3.9656862075852173, 3.2493327377545942], [3.9926429835919865, -0.77935420800416022, -3.0160788354656454, 4.0479733441505754, -0.69570134568033737]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-3.18834805429)+(1.-msk_arg1)*(-0.717513520939)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.98849540662010948, -0.66460156765455802, -1.2813132869553883, -1.1489141487363548, 0.28278874937176868], [1.5263574116656144, 0.11367821820059848, -0.35880818425707578, -1.2128970777264361, 0.84931520303205577], [0.62318374645947316, 0.86672978583963556, 0.067805263919441278, 0.14334487475850402, 0.059469807044354464], [-0.02942394593014528, -0.20858365817232499, -0.40317385289609697, -0.22439836258147636, -1.0399098285710817]])+(1.-msk_ref)*numpy.array([[-1.6711465521208648, -4.5639964241121849, -4.3515272069294122, 0.60080944605436237, -1.8486398274416416], [1.2038005027027752, 0.87505256614334226, 3.5435844644306775, 3.1390091174652448, -3.9432974306338555], [3.4276822095058916, -3.4461040677397761, -2.7019665388863894, 5.5269846377156817, -4.5286014032189064], [-5.5645543492543261, 1.0861874867303161, 4.2035149825705949, -5.6416683811768875, 0.96960032860359002]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.32701248512)+(1.-msk_arg0)*(2.89186927069)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.41431354372388451, -0.08645770703625999, -1.0074753136722938, -0.51186447250111478, -1.7891785795647506], [4.3982048807648972, -2.0842348312927692, -0.17303313381725438, 2.5631692400863866, 2.8594845374353444], [-0.078247947257654715, -1.793568263425549, -0.89389850821073225, -2.8154497532030032, 3.7524308846676195], [-3.1238960973986876, 3.3748957480588189, -4.9697011044744146, -0.16983085513965612, -0.46719478628623889]])+(1.-msk_arg1)*numpy.array([[-0.52672216553575346, -0.77868567470340722, -3.3232810791304179, 3.6685213699867223, 4.4019745614696628], [-3.7837525223301549, -2.3474369276573928, -3.4579292133882991, -1.2829435518645935, -1.7132850423997281], [-4.6402927626235169, 3.9101262979729476, -0.30147924764210554, -0.98528247496946975, 1.1110559832199058], [4.1956540163587945, 4.080308686232085, 3.3064521447924573, 0.11081704391372771, 3.7408299828879947]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[10.443811337241055, 50.047735863516394, 4.2949067102635627, 8.4534339020893832, 2.4184352163262792], [-0.98381330620585061, 2.0760676389004336, 25.006843427374516, -1.6881493494242266, -1.5132141574708062], [55.298734813681989, 2.412516196540984, 4.8406082406124238, 1.536881445031433, -1.153122500615837], [1.3851332919552566, -1.2821173772863488, 0.87067861711446703, 25.478364821032155, 9.2616882981758426]])+(1.-msk_ref)*numpy.array([[-5.4903124643421659, -3.7137825500484851, -0.87018497738474232, 0.78829287852743557, 0.65694820138187082], [-0.76428605032163677, -1.2319262922953862, -0.83630088767855582, -2.2540892516146052, -1.6879090163746029], [-0.62320836607111985, 0.73958461960306032, -9.5922664438889136, -2.9350661806664662, 2.6028114823750257], [0.68925351313785366, 0.70873786594727195, 0.87461398019651804, 26.095888940491832, 0.7730555208108294]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.8485354736094131, -3.0826633039267581], [1.8111780315045856, -3.9723870277161089]], [[3.3224069533598186, 4.5352767665847349], [3.4805686835639644, -2.0014335040294498]], [[1.0259953161421755, 0.40664515150693692], [-4.9614893114501442, -2.8593395924333445]], [[3.1256686660197861, 1.5777012284301097], [-1.3666190519306309, -4.9878676446691825]], [[2.8925919102860043, -0.67597591962337233], [-2.4825859968777211, 0.74843996088008069]], [[4.5507359251702066, -0.40328800886345739], [-0.31700129629022467, -0.24390823094368486]]])+(1.-msk_arg0)*numpy.array([[[2.8219318003694491, 0.81062225790016029], [-4.1898494325449853, -2.7496234425935118]], [[-0.80041006230467993, -1.3066397878402638], [0.1316597228124774, -0.24234930183254377]], [[-2.5062888199669775, 3.6579220722539869], [2.1982492479333775, 1.7762684207232375]], [[-0.79188780539299941, 0.95997293102884829], [1.4691958436179808, -4.2985202017632407]], [[-0.38129593475682011, -3.8149258758755966], [-0.050594429291481724, 0.1896344902899525]], [[3.9730798265693359, -3.8202411755423937], [-0.81250074429543773, 4.5916649583256479]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.15909873632)+(1.-msk_arg1)*(3.61992707707)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.5347844047621804, -0.97580467127682236], [0.57332112183800321, -1.2574431378305753]], [[1.0516945593251643, 1.4356236208895341], [1.1017600189405024, -0.6335457265132054]], [[0.3247746910680655, 0.12872188730020578], [-1.5705394878629615, -0.90511244854702921]], [[0.98941784569222879, 0.49941497880102581], [-0.43259776474164569, -1.578889443157413]], [[0.91563833603207323, -0.2139774587769267], [-0.78585261306800969, 0.23691565960729946]], [[1.4405171553674958, -0.12765919729787015], [-0.10034548545304212, -0.077208169576786362]]])+(1.-msk_ref)*numpy.array([[[0.77955487508123711, 0.22393331153952689], [-1.1574402863205917, -0.75957978822549865]], [[-0.22111220620285729, -0.3609574889274913], [0.036370821845118285, -0.066948669592709356]], [[-0.69235892508496022, 1.0104960664604048], [0.60726340645290389, 0.49069176889617394]], [[-0.21875794416104413, 0.26519123468242151], [0.40586338131614025, -1.1874604405680573]], [[-0.10533249058301886, -1.0538681566376016], [-0.013976643234596903, 0.052386273605097025]], [[1.0975579734008987, -1.0553365010416822], [-0.22445224088679905, 1.2684412864031016]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.24943346074)+(1.-msk_arg0)*(-1.73395443346)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.7839213510305942, 0.1383929816536007], [2.2104053927491085, -0.26490748547073562]], [[-2.2268140492572606, -2.5473581843492421], [-3.7555511789939544, 3.7417368573873908]], [[-1.5926240905343327, -3.6180580907669837], [-3.3054287684123898, 1.9164944134595778]], [[-3.2282252019622417, -4.9913913979696645], [-2.0335922180252375, 3.1846379346236997]], [[-3.7486406543100506, -0.43081094526326691], [2.8980137530017078, 3.6812287693314154]], [[0.98417755189913603, -0.81796830694112543], [-2.7121724466340655, 3.7816479878163243]]])+(1.-msk_arg1)*numpy.array([[[-0.4033828866502418, -4.237443071315167], [3.7830156148644338, -0.0079945575474091868]], [[-4.7310701318194344, -1.7468399431858228], [-3.3283462830335218, 1.6371615209384336]], [[1.7934114031605723, 2.1090021140259285], [-0.56584955069563758, 4.5239100958225169]], [[-1.6455538470068367, 3.2153801876303856], [0.23364222852752992, 4.6691727146269066]], [[-1.5571033886214347, 2.1416180352276362], [-1.8467132165464983, -3.0200809511627202]], [[3.3829604095734691, 3.4481011731896611], [-4.3270533933949116, -1.1913410132682731]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.85874762165803731, 23.479756140177408], [1.4700622208929872, -12.26629536332055]], [[-1.459229818413629, -1.2756091705920434], [-0.86523477004221139, 0.8684291773012216]], [[-2.0403015878331905, -0.89811533679652733], [-0.98305959329424775, 1.6955089657020419]], [[-1.0065696342260935, -0.65100754512265935], [-1.5978785874259593, 1.0203462771734353]], [[-0.86682980856074054, -7.5425972725759642], [1.1212622636358742, 0.88270348417622313]], [[3.3016740266715718, -3.9725664590724459], [-1.1980924976843899, 0.85926386358740492]]])+(1.-msk_ref)*numpy.array([[[4.2985324634222133, 0.40919828402953934], [-0.45835243889606953, 216.89185713810434]], [[0.36650364191244783, 0.99262353154846261], [0.52096575476355367, -1.0591223964640406]], [[-0.96684699918782568, -0.82216818177817608], [3.0643382703465227, -0.38328666943584949]], [[1.0537208713096724, -0.53926886783888739], [-7.4214085543646338, -0.37136223897290122]], [[1.113576944296657, -0.80964691412431178], [0.93894082628469022, 0.5741417072901509]], [[-0.51255534310957185, -0.502872261097558], [0.40072406689084195, 1.455464400321594]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.22695727048898195, 0.3724049212365923, 3.7947480229474984, 1.1377686968022482], [4.0320771260361781, -3.1068257495263438, 3.3244270934568689, 3.6378409890964321], [-2.4154898296309781, -0.25644049841235272, 1.6889555135298213, -3.5138478606613788]], [[-3.7571973035511652, 0.1421958226396125, 0.34979864462665056, 0.96355974031819791], [0.059117579300547618, 4.4394802114203564, -3.374030882439242, -2.0097799434043848], [1.3120122176242885, -1.0095412757780799, 4.9381761460625171, -1.1812282301727839]]], [[[-3.2048892602856505, 1.1775212171727727, 1.1106738947626233, 2.4187695328552197], [-3.764324635002728, 0.9344805535801779, 1.145622998721052, 3.0468867338902523], [-0.50220039429457053, -3.5689348115199957, 2.214511572530907, 4.1831556032373953]], [[2.5942243038220347, -1.1255402908783108, -4.4554362121121454, -3.4365413752040288], [-2.9626450255747794, -2.1551411570385568, -4.6142645392347168, 1.0277311860633223], [-0.49528208304665178, -1.260854686740732, -0.12000234585284986, -2.1734453839471057]]], [[[-1.9065066606037995, -2.4384807532949804, 3.6281107356471622, 3.5119999868011238], [-1.3801387591937941, -2.1663367462057961, -1.7665259885106144, 2.4836416340890919], [3.9526436157381291, 1.7081545267885954, -0.33278527976110617, 4.8315702168529757]], [[-3.8421049186925007, 3.4229236294924767, -0.8869832841695926, -2.1926215559360598], [0.62836502141497608, 4.8085133602263674, 2.611878547849745, 4.1583588768180526], [3.9352886732587677, 4.8688748956965764, -4.4720938465238511, 3.387965587713456]]]])+(1.-msk_arg0)*numpy.array([[[[0.71662077647310252, -0.32882469102279543, 4.2316359751814918, 4.3112676914137325], [4.685200134736613, -0.58994595072572587, 3.7174734130812119, -1.9215831961375809], [2.0008875290967998, 4.0405300426888466, 0.11246281123240287, 4.2623110655632956]], [[2.2836510300067392, 3.6232554717754706, -4.9806600032404695, -3.8092125798784195], [3.1111439928052782, 0.014357997817252333, -2.0367207321691438, -1.2012139033201596], [4.5449858210753646, 0.25541746841421542, -3.2191841245841379, 4.4936662361449038]]], [[[-1.9705375179728968, 1.8696439420638464, 2.8164974526721398, -3.3265630237521107], [-2.1691055031571418, 3.1390047128683172, 3.5248608779102639, 2.9134988533585791], [2.8518496360401251, 3.0307616283102821, 2.2713740294678244, 4.7812179894091287]], [[-1.9976146405611725, 1.1475164967002316, -4.1950793854968271, 4.6187478959205421], [2.76595323401774, -0.78883334718575249, -4.2764270490376219, 4.5576531384815162], [1.7217041058271887, -2.0813750714346382, -3.2622762197629198, -2.8716816341096409]]], [[[-3.6277268165692012, -3.5936431012311698, -3.2656916729165584, 0.54327072461644565], [2.6657077072006192, -2.719161182056645, 4.3072305379533518, -4.5832795861973965], [0.5092946118988726, -4.479926934279753, 4.8710798738208254, -1.7406953497221531]], [[2.4480949019600171, -0.53116518991649464, 4.6810531346909094, 3.3074900637589728], [1.6275693378052729, -3.3531637853117333, -1.9966420013398958, -0.19584007799217229], [3.5071147376347156, 0.052409443101865349, 3.8566606385914302, 4.3273743355640484]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.367114698103)+(1.-msk_arg1)*(4.17842136497)
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.61821896987913849, 1.0144102733028975, 10.33668235718137, 3.0992185893950603], [10.983153621652038, -8.4628203816867913, 9.0555543284742601, 9.9092763321388428], [-6.5796598232374937, -0.69852964138231466, 4.6006207930527054, -9.5715259531002843]], [[-10.234396288030972, 0.38733350469001865, 0.95283203433090768, 2.6246830903152114], [0.16103299488132986, 12.09289694571099, -9.1906722881708784, -5.4745286794217529], [3.5738482398073632, -2.7499342330709551, 13.451316908786628, -3.2175999388617429]]], [[[-8.72993992570375, 3.2075022418227457, 3.025413857033207, 6.5885935522360466], [-10.253810742119528, 2.5454730045075635, 3.120613270565225, 8.2995498399576029], [-1.3679659160721132, -9.7215797404948159, 6.032206239552707, 11.394682982861815]], [[7.0665225806112382, -3.0659090924255521, -12.136360203309284, -9.3609473904419911], [-8.0700801163244105, -5.8704845329605444, -12.568999724263119, 2.799482536037889], [-1.3491208213819246, -3.4344979736702492, -0.32687970945539957, -5.9203442280452476]]], [[[-5.1932180063974478, -6.6422858193710672, 9.8827716634357436, 9.5664924475778932], [-3.7594211463716087, -5.9009806945827243, -4.8119184484763036, 6.765301544503898], [10.766781161741228, 4.6529178363417918, -0.90648857558779861, 13.160928292476006]], [[-10.465679904786132, 9.3238534092369232, -2.4160930868524932, -5.9725790529869407], [1.7116313366401195, 13.098122698622847, 7.1146117585137185, 11.327138080553496], [10.719507264594043, 13.262544160858079, -12.181734672099999, 9.2286296495791902]]]])+(1.-msk_ref)*numpy.array([[[[0.17150514844714637, -0.078695914629358071, 1.0127355777620646, 1.0317934250390115], [1.1212847450045327, -0.14118871678960615, 0.8896837078833193, -0.45988257963867096], [0.47886207596811248, 0.96699918217137426, 0.026915143641416048, 1.0200768886789109]], [[0.54653440391493435, 0.86713501471040755, -1.1919956290195319, -0.91163916875818707], [0.7445740199615104, 0.0034362254457244177, -0.48743785134892825, -0.28748031813927266], [1.0877279776481097, 0.061127743256294664, -0.77043070657639823, 1.0754459264981806]]], [[[-0.4715985645907349, 0.44745222627382653, 0.67405778562390795, -0.79612914380637934], [-0.519120814703941, 0.75124178216870241, 0.84358674485657881, 0.69727262975134907], [0.68251844104377579, 0.72533652391360681, 0.54359621279762549, 1.1442642021453548]], [[-0.47807879246227236, 0.27462919520797541, -1.0039866780002598, 1.1053810739738295], [0.66196129887936928, -0.18878740995340057, -1.023455194082163, 1.0907595812849193], [0.41204654950865277, -0.49812474368580606, -0.78074371510638885, -0.68726473069177552]]], [[[-0.86820511856140292, -0.86004803904237404, -0.78156111786578397, 0.13001817604408997], [0.63797005480388025, -0.6507627988062481, 1.0308272339568407, -1.0968926266332772], [0.12188684850435172, -1.0721577703585556, 1.1657703827247381, -0.41659162580330311]], [[0.58588990629008642, -0.12712102096024611, 1.1202922649065525, 0.79156451082008972], [0.38951776176809477, -0.80249536665375931, -0.4778460157419942, -0.046869394177003608], [0.83933965277909139, 0.012542881276000879, 0.92299466753838366, 1.0356481449783084]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.08014547381)+(1.-msk_arg0)*(-0.91035408773)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.5171335700093094, -3.7344527464704536, 0.87319086285380099, -2.153105618959128], [4.7845308126810711, -3.6998307943896878, -1.1553465050687128, 2.3951827177236806], [-4.753858420595626, 3.6478523862005918, 0.075530322730200439, 0.98795898857245934]], [[4.5924349459447225, -3.9469290474929264, -3.6052610032082875, 1.1744825726391497], [-0.98459657459937233, 1.3416584201761639, 0.89599658769127721, -0.3363648809916695], [-1.874555845364875, -2.2625452683114169, 3.3575229959344259, -3.2862588379802506]]], [[[-0.79915056761038361, 2.0263822473969899, 1.9986691728997101, 1.4971509009477071], [-4.0530387768331266, 3.9106875689579965, 1.065260913030488, 0.43242408275320532], [-2.8659464088182385, -3.5962697598928486, -2.9795992333313368, 1.1676563807622911]], [[2.571679984647397, 1.5375375576438808, -4.6620276083255385, -2.0294886791831477], [3.2100836459737305, -4.9192041590243827, 1.3314156405890554, -2.4165118052132648], [2.7145754022007669, 2.0947603932233969, -4.7415544246312189, -2.6055292676792154]]], [[[-2.161788877230697, -4.7148486451600116, 1.410821771947159, 2.3555646219672361], [1.0575801809927441, -4.6238085397468556, 4.644579929771977, -2.302446918183203], [3.4550784844357594, -3.7510122872927254, -1.6831553588449499, -3.8797861499020403]], [[-3.2077488839291757, 0.56522387415505104, 0.61327687704736, -4.7061587794701341], [-2.7392441025224592, -1.5697447715083213, 4.6122598971651207, -1.4210599209223496], [-3.6689716038439526, -0.76678079331569826, -1.6410932856284854, 4.7653213107428769]]]])+(1.-msk_arg1)*numpy.array([[[[0.99947925556973072, 1.5740216720251796, 2.381350184222776, 1.1050446752477638], [0.61291000483921021, -1.4285656441413708, 0.24408124961416355, -3.7226941021116646], [4.5478033253331667, 4.1063309723719659, 2.3998743081082639, 1.9342531166429673]], [[3.3403529596010522, 2.5300967475259375, -2.470594457866031, 4.9865559357428406], [-3.9951238660214528, 2.3883167031323715, 0.21912425594688667, -3.2199037287897703], [4.9137546305802751, 4.8792980294647172, -2.4824385200757693, -2.0499079492259908]]], [[[4.8762500670240545, -1.4223378049566291, 3.8137051800101354, 2.9687831637667452], [-3.6117973502428158, -3.0415701375896953, -2.531046442048146, 3.5567626353160477], [2.8348848116088456, 3.6510088036512371, 1.7290953802650808, -0.5758011465628794]], [[1.4950001041639274, 3.9929459684732205, 3.0171474986998437, 3.661852431737854], [4.3353874476395475, -1.4326826044143126, -4.7655231414554535, -4.3346809408576972], [0.15173112051108628, -3.4016318959549583, -0.46394595197268629, -4.6328205023015681]]], [[[-1.5384232657421903, 1.2938469117311611, 2.9059380058548312, -2.0753669786869819], [0.42426439353951828, -4.0591826510796638, -1.8128720193369774, -3.3982855890351606], [4.1465729731659593, 1.1451360175554122, -0.65171248162680406, 1.2338890848770863]], [[-2.4790660729306744, -2.4423678797883142, -0.21042425907837092, -2.9798264592129362], [0.15341261853066168, -4.2946293708609753, -4.8965533397374896, 2.1635674302077259], [-0.17462041899873881, -3.3306856276412478, -2.9186469204043455, -0.19392474086700595]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.030240141475681, -0.82479165835592538, 3.5274595793942893, -1.4305593960143184], [0.64377168721528366, -0.83250982139070939, -2.6659928084778493, 1.2859751579789687], [-0.64792536952063551, 0.84437229024536287, 40.780250401096509, 3.1176855612841061]], [[0.67069985967532963, -0.78039038370089309, -0.85434743034476934, 2.6225552814222604], [-3.1283325102634061, 2.2957747124702372, 3.4376754511418901, -9.1571553627487319], [-1.6431334822215804, -1.361363026390177, 0.91738626289169345, -0.93728024044091329]]], [[[-3.8542742740250344, 1.5200219394785155, 1.5410982045340036, 2.0573380224154101], [-0.75995953737701494, 0.78762248824543935, 2.8914470024538028, 7.122973943084288], [-1.0747393825424989, -0.85648343407502858, -1.0337448873516635, 2.6378869028252709]], [[1.1977172479473663, 2.0032977136072132, -0.66068795223619492, -1.5176953216871052], [0.95952187341800244, -0.62614711124819133, 2.3134364505816256, -1.2746246333944034], [1.1346693377226003, 1.470404674337721, -0.64960668969918411, -1.1821573113844688]]], [[[-1.4248132675004532, -0.65328618278661033, 2.1832279137301049, 1.3076038946623505], [2.9124462893412884, -0.66614900840635527, 0.66316987120176951, -1.3377704604124654], [0.89148350397493692, -0.82115046230261357, -1.8299828697494254, -0.79389568259863375]], [[-0.96022026201762134, 5.4494256429236856, 5.0224386228984512, -0.65449246788056703], [-1.1244508917539784, -1.962195084015846, 0.66781697963429731, -2.1674986596019217], [-0.83951194132605478, -4.0169830812977221, -1.8768862811071765, 0.64636679731739521]]]])+(1.-msk_ref)*numpy.array([[[[-0.91082839654399705, -0.57836185098917925, -0.38228484569844423, -0.82381654617308009], [-1.485298136010011, 0.63725044170212253, -3.7297174165103901, 0.24454173852564884], [-0.2001744628353897, -0.22169525395165898, -0.37933406956098703, -0.47064889279307426]], [[-0.27253230384321181, -0.35980999091033106, 0.36847572649210969, -0.18256169176892056], [0.22786629858266261, -0.38116975296266215, -4.1545107993440293, 0.28272711373011467], [-0.18526649297139614, -0.18657480691529638, 0.36671767714182091, 0.44409510586721684]]], [[[-0.18669142788346563, 0.64004070239652022, -0.23870594205898904, -0.30664216196056665], [0.25205015659817237, 0.29930399318392165, 0.35967498367706713, -0.2559501943397674], [-0.32112560058936723, -0.24934316422878924, -0.52649153893986056, 1.5810216654894278]], [[-0.60893245772626714, -0.22799058512621373, -0.30172674293247437, -0.24860479899172969], [-0.20998217546282252, 0.63541923725787319, 0.19102920302922538, 0.21001640031882837], [-5.9997849133596191, 0.26762275154231441, 1.9621985790775056, 0.19650104882701697]]], [[[0.59174487801988507, -0.70360262831366671, -0.31327374702949645, 0.43864728362668542], [-2.1457235195599096, 0.22427029428878476, 0.50216125463865346, 0.26788628085493926], [-0.21954372770497735, -0.7949746351293282, 1.3968645889014444, -0.73779247979995755]], [[0.36721654887292843, 0.37273422045186261, 4.3262791643738723, 0.30550574007924547], [-5.9340235271951807, 0.21197500624997134, 0.18591732277103529, -0.42076529486403014], [5.2133312527221731, 0.27332333024004662, 0.31190963229065283, 4.6943679473775282]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.54006506455590664, -1.2689525647104603])+(1.-msk_arg0)*numpy.array([3.0673150091209873, 2.8096990370336128])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.9986567856502031, -1.5915154648947807])+(1.-msk_arg1)*numpy.array([1.7745633388560638, -3.7332254132814335])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.54079146340980633, 0.79732342707354975])+(1.-msk_ref)*numpy.array([1.728490013266176, -0.7526197124442966])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.1750294259323963, 4.0715926518618524], [-4.1554134289620581, 3.1619835013552482]])+(1.-msk_arg0)*numpy.array([[-1.0581267980055897, 1.5506723038549666], [-4.2155871831890748, 3.3298845457354709]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.7072052789544445, 1.7823660266370167])+(1.-msk_arg1)*numpy.array([-3.0717881427343352, 1.6739960452521618])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.5421916684296817, 1.5039837146868857], [-2.3314029592465513, 1.7740371248667175]])+(1.-msk_ref)*numpy.array([[0.34446607280139574, -0.50481095433705403], [-2.5182778628094438, 1.9891830420865031]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.62530064816611208, 3.5357056303911012])+(1.-msk_arg0)*numpy.array([-4.0311597919975029, -1.6196088181192128])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.97570084950313962, -3.524728703702694], [-4.3017345312914612, 2.1958999610301877]])+(1.-msk_arg1)*numpy.array([[3.4291833711986879, -0.17889046969843836], [-2.4487433524887283, -2.8220230999455165]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.64087332555315146, 0.17740390842258008], [-0.82192557552583712, 1.6101396662589107]])+(1.-msk_ref)*numpy.array([[-1.1755451241991737, 22.534234488807389], [0.66140406934567386, 0.5739176331159308]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-1.5299389389781135, -0.53588892639964669, 2.8110456756271907, 3.0367339256777086, 3.5643498887852783], [0.58432724824193194, -4.7288505151368447, 1.9953428398631541, -2.7529185420754643, 2.6243110374537633], [-2.7904233364307385, 0.12204175402186124, 2.3720500775674838, 3.6417630010301387, 2.1935299613302846], [1.5552322111346442, 4.2686099166114797, -4.2846979836267227, 3.0975217549230543, -0.71686120635133666]], [[3.0740903075008141, -1.7876498774642058, 2.7500834120820041, 2.5218865766151737, -4.7632628518841909], [-4.9765075192561428, 1.3627640016611515, 2.9498933477187919, 3.227740505800849, -4.1157226870198027], [-1.8708935905207746, -3.2998569216462736, -1.5375736047459423, 2.6020629058913922, 2.3149808598284398], [-1.5750358310902977, 0.24105974911734229, 0.34865070501622153, 0.29862085184748821, -2.7501569796879499]]])+(1.-msk_arg0)*numpy.array([[[-2.983556766441998, 2.1017370675627287, -0.069046970703628752, -1.0253263863113604, 1.052313262029358], [-0.59536381917541714, -2.5662492747020869, 1.71388586759908, -0.0040530599938009004, 2.6722602638572326], [0.14814167095797437, 1.2160369967639548, 4.6452002469782911, -4.9124803565875785, 0.26956555936056326], [4.6857064116145057, -3.6567094900268007, -0.48923807530862895, 2.5149476902570242, 4.1482919390303206]], [[-4.3264616667074476, 4.2814246916321093, 3.0983241681324607, 3.6064031886581667, -0.28343764383973014], [1.8235081272491005, -4.2610511071993438, 1.3487296994487021, -4.9040677507807606, 3.9060808425179303], [-4.1901604532505141, 4.2939629187978809, 4.0902404275062718, 2.684812639122006, -1.3670616515918477], [1.5101249180197858, -2.4346051746601871, -0.94927760323029542, 3.2603339319171489, -3.4234600384592175]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.1336301189568605, 3.557156252366017])+(1.-msk_arg1)*numpy.array([1.9475120588881323, -0.02529157745747046])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.71705912162793528, 0.25116299289102872, -1.3174943729241697, -1.4232710246715015, -1.6705566054382013], [-0.27386529794939884, 2.2163403455556754, -0.93518685461690254, 1.2902510691128488, -1.2299746868669057], [1.3078289960562548, -0.057199114756370184, -1.1117438099942025, -1.7068389542657045, -1.0280741454862425], [-0.72891369376384829, -2.0006325738869952, 2.0081728063163671, -1.4517613561048921, 0.33598194925267211]], [[0.86419884014262938, -0.50255028192117324, 0.77311290732668991, 0.70896142808957552, -1.3390648354892876], [-1.3990129098057063, 0.38310490318065699, 0.82928416365086333, 0.90739351234681931, -1.1570261172199729], [-0.52595204084059088, -0.92766712720347821, -0.43224798003271186, 0.73150087353082927, 0.65079538136359538], [-0.44277948994865601, 0.067767545762715126, 0.098013885328854758, 0.083949320935469932, -0.77313358890509876]]])+(1.-msk_ref)*numpy.array([[[-1.5319837188301475, 1.0791907849662539, -0.035453937442137762, -0.52648012197507865, 0.54033722524426453], [-0.30570481782552889, -1.3177064876133306, 0.88003864200849502, -0.0020811475725161163, 1.3721405480707891], [0.076067139241515647, 0.62440537464923884, 2.3851971677291255, -2.5224389929540139, 0.13841534799762051], [2.4059960965220699, -1.8776312440984209, -0.25121183361911675, 1.2913643737296552, 2.1300468564999031]], [[171.06333814024421, -169.28262773769731, -122.50418833473347, -142.59305077837013, 11.206799746530249], [-72.099422438772578, 168.47707954810633, -53.327227284129847, 193.90122103009551, -154.44196191741207], [165.67414429948306, -169.77837487671448, -161.72342094455337, -106.15441617418702, 54.052051671773206], [-59.708609340764347, 96.261499653556385, 37.533348990452318, -128.90986880512401, 135.35968818931926]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-4.1744515437091723, 4.1433431512109067])+(1.-msk_arg0)*numpy.array([-1.4317900009847784, 3.8533911481014744])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.7720027980139039, 3.2341213896924224, -1.5269191349137334, 0.86256355811615304, 0.52333209290597793], [-0.38743153229323468, -1.2878585381948984, -3.1176950236044698, -2.9633297879301903, 4.8194793567747478], [-3.8839837603057745, 0.6529963543928643, 0.63980130954192127, 2.5437322412320942, 1.1723717165602778], [-2.387393083667404, 1.3074994382150713, 2.6707628198789592, -3.9064424372267057, 2.4300573393970044]], [[-4.3791331284469965, -1.0809475385385028, -3.6426614373017943, -3.466554377753214, -2.1000666389370313], [0.32983352331768145, -4.833530149957511, 2.9985549116748569, -4.3537174235446541, 3.5341237511750982], [-4.1725002897748462, -2.2636697631139335, -2.8599685241841555, -0.17161169457195236, 2.933966716198424], [2.1657951137007325, -3.4180890291470876, 2.4596664798699681, -0.16210623101321531, -3.8258497634360609]]])+(1.-msk_arg1)*numpy.array([[[3.9748123361359049, 3.3518758716703676, 2.2817831852378099, 3.5183273283646734, -3.7124664425377283], [-2.7472536392985458, -0.47298638650645852, -2.7592611154258617, -4.9498939617738031, -4.3202302040571308], [1.8860035706795122, -1.2882790612966333, 2.9468200983439807, 0.20039700215846246, 0.21410900951454437], [1.976357972850634, -4.4224709720911104, -3.2789176674007479, -3.9209277240895366, 4.6677623820753649]], [[-0.50054682568087294, -4.2086069665436918, -4.8133717386702424, -1.4632466143919687, -0.6733483636509412], [-0.028076671084670046, -2.6425532950313038, 2.2825640860169756, -0.91189724535644334, -2.9946620169052371], [3.2524257307483246, -2.7922975916797377, -4.5372227649539187, 1.0071059411726226, 3.4219148398093733], [-3.0742931066007486, 4.9861020689354092, -1.1503539765601167, 0.28607490050253315, -0.61466716976532521]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.5059333802621342, -1.2907528941287447, 2.7339047944703481, -4.8395871868575275, -7.9766779073859624], [10.77468196509535, 3.24138981099602, 1.3389544237341571, 1.4087029937443849, -0.86616234549093785], [1.0747860447749478, -6.3927639344793086, -6.5246061260767902, -1.641073488806831, -3.5606893997382967], [1.748539682161, -3.1926985371465335, -1.5630184427602452, 1.0686069514114571, -1.7178407587472906]], [[-0.94615601528430593, -3.8330659014339603, -1.1374494233205432, -1.1952338546312784, -1.9729579406623492], [12.561922479966407, -0.85720850448141483, 1.3817799817768297, -0.9516793921451917, 1.1723820225121553], [-0.99301207033217176, -1.8303655501018268, -1.4487373256643974, -24.14371096064033, 1.412198416681252], [1.9130817707548997, -1.2121811678629071, 1.6845142156955959, -25.55943177084372, -1.0829863709780647]]])+(1.-msk_ref)*numpy.array([[[-0.36021574854441712, -0.4271608066056643, -0.62748731353963227, -0.40695190280953125, 0.38567082642935641], [0.52117139113167443, 3.0271272954813204, 0.51890340967741189, 0.28925670166713913, 0.33141521015250147], [-0.75916611359803299, 1.1113974013857708, -0.4858762846735703, -7.1447675642003912, -6.6872010861715623], [-0.72445883825368507, 0.323753397143899, 0.43666543238329675, 0.36516612948208543, -0.3067401216657864]], [[-7.6983629710564392, -0.91559776874723531, -0.80055964037509075, -2.6334529738192467, -5.7227303965040059], [-137.24530007424698, -1.4582075431919821, 1.688185305160723, -4.2256856983872746, -1.2867532717711065], [1.1847745243409074, -1.3800073314475854, -0.84928409904524726, 3.8262023790811748, 1.1260920649668003], [-1.2534234747584545, 0.77282636713536401, -3.3497438411298428, 13.469868000766301, -6.2690694047848154]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.8070000431010378, -2.2349669765403215], [0.19883665012088159, 2.3400998471023202]], [[-3.2573640539180615, -0.10966470278857976], [4.031102410702589, -2.0508734644113247]], [[0.20725052300907443, 2.8450967320608775], [3.1398119834893699, 3.6227754474327707]], [[-1.2828344448709341, -4.9084451578478907], [-3.4998143863004372, -2.7374085932378045]], [[-3.4835281216785727, 1.8739178928303177], [4.716318942069627, -2.1010195113582277]], [[2.9199088629201517, -4.6067565019514047], [-4.2495209710436308, -0.58092961576455249]]], [[[-4.6894845488072363, -1.2953520170523056], [3.5692754714930022, -1.7697840633508855]], [[-3.2030429913666691, -0.96423540848366862], [1.905530627621137, -1.119271965963089]], [[-4.6275991733752182, -2.1750621037534037], [-4.276981436382882, -3.867113532960369]], [[2.5771513089549893, -3.1726779183582288], [-1.7335867006033068, -1.0335785820809784]], [[-1.0552015146277229, 3.3099410407056578], [-2.7793877297983371, -4.4943494208998125]], [[4.1787858447520794, 4.9473404774872201], [1.2598215785016436, 2.7637951083574075]]]])+(1.-msk_arg0)*numpy.array([[[[1.9776520028684228, 2.7084148306555793], [0.74775655160693422, 2.1811212009660359]], [[-4.1420558625537742, -3.6336744950112831], [2.0208276321188023, -0.80223588932134149]], [[-4.5259471020371098, -4.6394828701384885], [2.2712186951965343, 3.4010704281613489]], [[-1.1463151761790358, -0.68752634857484196], [-4.5549705595684822, 4.103007705029917]], [[-2.7160033903692673, -1.0778332077065942], [1.6552147487463227, -0.68440543699399825]], [[-0.30455211848843078, 3.6772918235690319], [-2.5777076313784186, -3.5997588410910621]]], [[[-2.2339277439583229, -2.5444469372656875], [-4.7954088128593479, 2.7149095789841002]], [[0.73739835360076089, -3.6034613942757945], [4.591552763465474, 1.4270090360151286]], [[-0.53775441790321654, -3.4056380120528287], [-2.6977001211445071, 3.3831585007203824]], [[0.03387162115046749, 3.3781885958218041], [-0.67524031330820833, 2.636844480762301]], [[-1.9768003504121689, 1.7790851497339837], [1.3211520317729821, -3.8427126040988013]], [[-3.6215705437269485, 4.1405879864606305], [3.161812521819984, -4.5500050045562919]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.3370488654063815, 3.3936173350302461])+(1.-msk_arg1)*numpy.array([-0.79747549719777044, 2.6423142085925866])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.8473155631031792, -1.6715671613550396], [0.14871307643677431, 1.750197698564347]], [[-2.4362341109560131, -0.082019966230066216], [3.0149252693747877, -1.5338807110748223]], [[0.15500594508644447, 2.1278928584230474], [2.3483150576812002, 2.7095310733700577]], [[-0.95945217715063136, -3.6711037904781603], [-2.6175665503718943, -2.0473511956541683]], [[-2.6053857953948394, 1.4015328394604043], [3.5274095540525763, -1.571385732951238]], [[2.1838460347017139, -3.4454660716915768], [-3.1782839662722688, -0.43448645056662549]]], [[[-1.3818542533952021, -0.38170243995431519], [1.0517613269621016, -0.52150371966882703]], [[-0.9438433020433995, -0.28413203767273743], [0.56150427095934008, -0.32981678706362255]], [[-1.3636184391231529, -0.64092733181834072], [-1.2603016233545856, -1.139525512509177]], [[0.75941128728705654, -0.93489560110640813], [-0.51083741313687503, -0.30456544743921943]], [[-0.31093709468522568, 0.97534303780781384], [-0.81900445907922781, -1.3243536254094941]], [[1.2313662479316736, 1.4578368711223966], [0.37123265652178056, 0.81440976854651015]]]])+(1.-msk_ref)*numpy.array([[[[-2.4798906171006454, -3.3962357968020478], [-0.93765457902400462, -2.7350322469219734]], [[5.1939600365258149, 4.556471650576805], [-2.5340310006009448, 1.0059693271333081]], [[5.6753431521604414, 5.8177121258785425], [-2.8480106325239007, -4.2647961474832599]], [[1.4374299651927169, 0.8621284929640145], [5.7117373205497612, -5.1449953251822471]], [[3.4057515245458512, 1.3515565199106001], [-2.0755681579716758, 0.85821500397054673]], [[0.38189526770237953, -4.6111659060254233], [3.2323345863743302, 4.5139428781701332]]], [[[-0.84544364053819765, -0.96296153159656683], [-1.8148518436093166, 1.0274741626697685]], [[0.2790729244852117, -1.3637520407518671], [1.7377012728214249, 0.54006031204563543]], [[-0.20351645393060513, -1.2888845698130738], [-1.0209611379191053, 1.280377061031815]], [[0.012818922534011961, 1.2784961700755402], [-0.25554883333419731, 0.99792994799312718]], [[-0.7481322031966442, 0.67330567422623178], [0.49999808027247683, -1.4542981268475259]], [[-1.370605559304757, 1.5670308901930672], [1.1966073192726405, -1.721977268925873]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.5955481500591731, 1.0041466317503387])+(1.-msk_arg0)*numpy.array([2.6430199341421377, -1.9994396535947301])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.55348842264522613, 2.7499061020181674], [-2.7931255907046069, 2.3317369460657167]], [[4.0094271356958586, -4.3281209537849996], [-1.2301751817319397, 0.94677077704070101]], [[4.2009490605129507, 3.8274431815100645], [3.1050714969031219, 1.9268460085645582]], [[4.9735492722736296, 1.320955158698319], [4.4175870178683194, -3.3825768814311195]], [[-4.4552880468990841, -4.2360209103154665], [2.426918287431274, -1.8400364024460227]], [[-2.8266186707702468, -0.02665049867220759], [-2.9219773305641663, 4.4028460458684435]]], [[[3.5616949438310339, -1.4888674589960385], [-1.8810164797876308, 3.7876028801859825]], [[1.2970085777653146, -0.89847329543155219], [-4.8418591134336362, -1.1880655099655044]], [[-0.40023281174937697, 1.8725325714555305], [2.4440077189913918, 3.4265247986422054]], [[-2.8564795361399362, 1.8533713910706648], [3.9918444446330366, 3.4084478120090402]], [[-3.243252687000421, -4.6067394178509522], [-4.6099763024632878, 4.8096142043528989]], [[0.38777466116669768, -2.8611567641941416], [0.12680327989266438, -1.3525581934598687]]]])+(1.-msk_arg1)*numpy.array([[[[3.3261443446182248, 1.737845486307056], [-3.4591442956614826, -1.7056891065642734]], [[1.5883145497706943, -4.1685818568706043], [-2.7258388929472988, 2.0364008891387089]], [[-1.2023278197462286, 1.7435405173748002], [-1.899796738958448, 4.0596000972417041]], [[4.4093525070590029, -3.5706053520568179], [-4.2687865611165474, 4.5456205697937797]], [[-4.0801467965774538, 1.217296181367181], [0.48385105108551674, 1.5681387189109692]], [[3.150185962709859, 1.0588778012726028], [3.6645142558381103, -2.8110665640004506]]], [[[1.2240222910095389, -3.5017898365533684], [3.5971790788451248, 2.4061938273503998]], [[-4.8706680201316512, -4.070674676338518], [0.10608719374407993, -1.9059862148275544]], [[3.0416663621584572, -3.7709603718694584], [3.7155870779287721, -2.9893926119872418]], [[3.3913638401918327, -2.6536778919807178], [1.3456295907280662, 1.4832779773179015]], [[4.4653637514911821, -0.71140672378253278], [3.9171673361905199, -1.0623917079924841]], [[-4.2683099804191507, 2.7884312299312128], [-1.6735954310240473, 0.96772572500456011]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.4961578290569593, 1.3075166993594383], [-1.2872848116908855, 1.5420041939661566]], [[0.8967735360615664, -0.83074114343195937], [-2.9227936016373461, 3.7976965885002203]], [[0.85588949027154926, 0.93941254763201987], [1.1579598581369974, 1.8660277645839209]], [[0.72293405638977193, 2.7219305109510743], [0.81391676847922823, -1.0629612499858241]], [[-0.80702933507558583, -0.84880321088674804], [1.4815283104833386, -1.9540635963937936]], [[-1.272031557436573, -134.91485447545423], [-1.2305188382022649, 0.8166418068224699]]], [[[0.28192943179750746, -0.67443654952835563], [-0.53383191616891534, 0.26511402158956859]], [[0.77420199755381458, -1.1176143318405805], [-0.20738865138895823, -0.84519466588967318]], [[-2.5089063222011103, 0.53625055556166357], [0.4108606629788944, 0.29305103297318663]], [[-0.35153293382499712, 0.54179461093885684], [0.25154953948678932, 0.29460525351522543]], [[-0.30961097658999898, -0.21797339520861675], [-0.21782034567374772, 0.20877903904257947]], [[2.5895106934763672, -0.35095827125472495], [7.9189326380226301, -0.74240549249989252]]]])+(1.-msk_ref)*numpy.array([[[[0.79461973393265461, 1.5208601426117514], [-0.76406755782262625, -1.5495320477633265]], [[1.6640406237692098, -0.63403335352188062], [-0.96961707494179394, 1.297887831535959]], [[-2.1982523324628644, 1.5158924658210171], [-1.3912119543857924, 0.65105425924537197]], [[0.59941225608768745, -0.7402162024485982], [-0.61915017213951007, 0.58144314809408804]], [[-0.64777569678600289, 2.1712217409354597], [5.4624660382829369, 1.6854503382058221]], [[0.83900441606582299, 2.4960575535398397], [0.72124700563831023, -0.940219619125929]]], [[[-1.6334993801016884, 0.57097648543142598], [-0.55583545043763871, -0.83095535815434685]], [[0.41050624787617662, 0.49118139192424531], [-18.847134918264405, 1.0490315397037806]], [[-0.6573500889084587, 0.53022027717663478], [-0.53812213565703959, 0.66884478324363483]], [[-0.58956801682524029, 0.75345981501256687], [-1.4858766984404033, -1.3479871501970011]], [[-0.44776635563609546, 2.8105436549204321], [-0.51042998217666213, 1.8820173750912552]], [[0.46843824904169307, -0.71704822128392742], [1.1946971272330158, -2.0661222513076298]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.3605618682722547, 1.8784360335805284, -3.5026289759554077, 3.3783024102659773, -2.760220350855521], [4.387352151545743, 4.6292495211061926, -1.7737258328183012, 0.74697917356140398, -0.63004188890415236], [1.5601236744625977, 4.6691549508537715, -1.1465243675707182, 3.6271529799962412, 0.96031963437344459], [4.4895520827838702, -4.3892729630834557, -2.1968659179210537, 1.2588879765658767, -3.9753232986419529]])+(1.-msk_arg0)*numpy.array([[-2.6947707812670552, 0.68719228684100742, -0.83340383698952358, 0.64693374118262259, -4.4449644412356015], [4.2040451283237488, 0.12500716257406275, 0.79996270802452685, -1.0463919512189013, -4.9094492027163996], [-0.32128870582064195, -3.214765833656601, -0.5305611027926318, 0.024085296867363937, 3.2881887847003206], [-0.21160807903498302, 2.352553907281508, -2.9757204476988219, -1.9174393450261396, -0.56346267361433977]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.5791909956217332, -4.2566600239199062, 0.62290025866419629, -2.8197922877915724, -1.8135677704710096], [2.7935970816355251, 0.94756375760479017, 1.7731829042248135, 2.8839750186855166, -4.5806332214928869], [-4.2590312672654349, -3.6422138997039233, 2.1943464825168482, -0.85817468675746778, 2.0196047202469174], [3.043405323459611, 4.6062488993500335, -1.9718744785700162, 1.8828312351076404, 1.5118835201332823]])+(1.-msk_arg1)*numpy.array([[3.2292925887620054, 1.6267332571232398, 3.240549378776846, 0.62006570336244771, -3.1893995388406604], [4.954239432298051, -3.8633374201619475, -2.4307774874518229, 1.8591572079694103, 2.5214184094442258], [4.0181148103972362, 0.2121862573144746, 4.4228248607735701, -4.181869164099794, -3.9062041555733207], [3.4102253841089354, 3.7095909886860596, 3.1228471629756438, -4.8028293641061959, -1.6155678870855938]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.4947918743311306, -0.4412934138561293, -5.6230976424167203, -1.1980678239643756, 1.5219835706159754], [1.5705028403656358, 4.8854227316669485, -1.0003061887142009, 0.25901027877206395, 0.1375447145490539], [-0.36630951419717916, -1.2819551732624295, -0.52249012483009971, -4.2265905018721739, 0.47549880664570621], [1.475173894248283, -0.95289530787248722, 1.1141002846764358, 0.66861434689015387, -2.6293846355911747]])+(1.-msk_ref)*numpy.array([[-0.8344771206687509, 0.42243698149766562, -0.25717979872415769, 1.0433309529530126, 1.3936681143596503], [0.84857528300235574, -0.032357298620016101, -0.32909746455778044, -0.56283134461866235, -1.9470981826449607], [-0.079960061118532086, -15.150678815603488, -0.11995978124709973, -0.0057594572958259045, -0.84178620823204442], [-0.06205105387492578, 0.63418148104645489, -0.95288699459225845, 0.39923120303962206, 0.34877065712837307]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.9441855500221692, 1.4439948988940241], [-3.715443074974444, 3.424272385414465], [-2.6800358169022829, 3.6373027425358071], [-4.8228128269106341, 3.6695325934229146], [0.26691356487363826, -2.9549184242964133]], [[-0.89215243678586464, -0.22788500627233432], [-0.93371063328295634, -3.1337288382295547], [2.2789846009506931, -2.0636177080866758], [-0.56916673540948626, -4.194527894212289], [-3.3213062892381995, 4.2711449832022801]], [[4.054452941931606, -2.4072428880080987], [0.61628343700042354, -2.3751722207080905], [-4.5635960135810478, -1.822958219313878], [4.3328710472898866, 2.7966180315001976], [2.8470486001343733, -2.1693624104751374]], [[-0.061512843302847564, 4.6435962203804912], [-4.7593094074269207, 1.7453648871198197], [4.2807031470219563, -0.12135901850261455], [1.8107956059856267, 3.7366774137712628], [-3.7832484703395117, -3.9036904369102232]]])+(1.-msk_arg0)*numpy.array([[[0.23105054184775486, 4.5425334077993416], [3.1214444569166044, 4.5695772209996424], [-3.0086378030769856, -4.2576134342618772], [1.0225979665038292, 3.1965456875996328], [3.1836164710527157, 2.8146591477750906]], [[2.7820656721531201, 1.1341000743823226], [2.8045061344990474, 1.8986703752406449], [2.6824005551412782, 4.8770743548128141], [2.1418065422062895, -1.1962699741096507], [4.4832241676615574, 4.710792093425912]], [[-4.574828909138656, 2.5686905233865058], [-4.3611121672787156, 3.6043426850004323], [-2.8825361626822787, 1.1665682850926959], [-1.0898578358823929, 4.9812631064408937], [-4.5141736990504384, 0.58207065628710097]], [[0.46403258310889939, -2.7221088891921488], [2.8428714185808097, 0.99985547275132181], [1.0847415088175048, 0.97188114265759484], [-3.212783926741849, 0.21191837676400205], [0.33182069546089732, 3.8490095943634177]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.1421159145941697, -1.656325747789996, 0.98484292669463258, -1.2652843668415823, -3.791827829338871], [-4.6903172499368644, -4.8570061652885874, -2.030725859888781, -4.9080745802192283, -4.607111350750305], [-4.7477064956964572, 0.8277524374687637, -1.6603660382850807, 4.8610907538277921, 3.0587347541071228], [-0.47625507011558188, -2.4689922192061378, 0.38980210172410423, 1.5051466891465806, 1.9969705998984768]])+(1.-msk_arg1)*numpy.array([[-2.6105203811790014, -3.3502026425230511, -1.9474904260746593, 2.8481423458232689, 3.9956267906513059], [2.0591214159671392, 2.2331268355480614, -0.68462260009524378, 2.8605640860445538, -3.4289981294141003], [-3.7254318374234896, 4.737770593334691, -4.4480107563036517, 0.38697495868059129, 4.4510986205545571], [-3.291697966710776, -3.5736663662693227, -2.6865989986672698, 3.0770324444663189, -3.5835469603270154]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.9370073001913537, 0.45956130777579157], [2.2431837939679977, -2.0673906627264635], [-2.721282495166129, 3.6932820899099732], [3.8116434165304645, -2.9001643342696513], [-0.070391794376427771, 0.77928602175263373]], [[0.19021153351574965, 0.048586267011128476], [0.1922399522479252, 0.64519762413012272], [-1.1222512333966677, 1.016197089349961], [0.1159653803353704, 0.85461779882426581], [0.72090862069077144, -0.92707656881501033]], [[-0.85398137934742835, 0.50703279366366394], [0.74452627271626703, -2.8694234087323007], [2.7485481564622858, 1.0979255039429328], [0.89133720531303251, 0.57530669002590484], [0.93079290262468573, -0.70923521811174417]], [[0.12915945081260569, -9.750229471055377], [1.9276324041868367, -0.70691388718957238], [10.981734393140266, -0.31133495167378689], [1.2030691885668297, 2.4826001616426914], [-1.8944938250627459, -1.9548061634501186]]])+(1.-msk_ref)*numpy.array([[[-0.088507465221705883, -1.7400873176664402], [-0.93171810483852768, -1.3639703948052155], [1.544879380558067, 2.18620506537918], [0.35904032956900628, 1.1223265200516712], [0.79677523398870076, 0.70443494731806222]], [[1.3510935540663223, 0.55076891803859584], [1.2558651348662675, 0.85022952795006246], [-3.918071876049821, -7.123741392899273], [0.74873573105920987, -0.41819373316812963], [-1.307444331685182, -1.3738100505265738]], [[1.2279996276358152, -0.68950141499918394], [-0.92049880452509136, 0.76076766782908056], [0.64805062770974498, -0.26226741548218008], [-2.816352354163465, 12.872313814373769], [-1.0141706764717837, 0.13077011001265604]], [[-0.1409705835109116, 0.82696192564477955], [-0.79550554730395395, -0.27978422445604695], [-0.40376011059172134, -0.3617514720803931], [-1.0441176636013907, 0.068871024465508107], [-0.092595604057778877, -1.0740781792384206]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.28209175688513, -0.28134902902861292, 3.0488582732127423, 3.2730903577496662, 2.9650716637958556], [0.8270945457887553, -1.8519333916088843, -3.1076189268436405, 4.5803045894379277, -1.010509257709856], [-2.5989595332199178, -4.0142243268973576, -2.6440047099458965, 2.2477475962959712, 0.31507157969761046], [-4.806887839401659, -3.8587970865895227, -1.5511743353799434, -3.8104352452762091, 0.10282281718958064]])+(1.-msk_arg0)*numpy.array([[-1.6884334784877018, 1.3211116720725915, 4.4573971901980443, -0.25923252718985346, 2.3233256739121693], [-1.9637556681662316, 4.5541685687895459, -4.012491505855305, -3.7017372885742095, -0.84639776838027903], [1.5718417067725765, 4.1293889422197427, 0.78267605183584887, 3.780095802451978, -3.8358444439846195], [-3.5924115637527763, 1.7488584318296176, 0.0097012118123398494, -3.3568146249603528, 1.7513937827895028]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.0171240939684179, -2.4183518329723022], [-4.4430958218614505, -4.5871050820392032], [4.9939489043445118, -2.2764914639697018], [0.99866205500933614, 4.9037831731295256], [4.7148520250450474, 3.6185105335885268]], [[-1.6896395709776693, -3.3460650250853554], [2.8115884590276909, -4.3087520769461305], [-1.4665261573763724, 3.2742500794279046], [-2.844314324722419, 1.4615796729052519], [-3.4185014990028506, -2.3578426170487554]], [[-2.5630629668492864, -1.5493267915822373], [3.3383543632551493, 2.9943312103292641], [-0.78374774980543993, 4.9005632926702916], [0.63762749814362962, -1.5157662517733339], [-3.6663209474508252, -2.2039116651101009]], [[3.1139904150757225, -0.93383845003868071], [-2.9686314827361162, -3.5803772881508245], [-3.6110093051614269, 3.3946058503060126], [1.1894490261324764, -1.163375279301194], [4.9409501398574101, -0.64819722898273735]]])+(1.-msk_arg1)*numpy.array([[[-3.3395026829317596, -4.7197283762089253], [-4.9560677857636755, 1.3054793536020313], [-2.916022414424865, 1.5224747026105643], [-2.1999451781820776, 0.87088285887804595], [-0.15541359809218136, -2.859591895602589]], [[-2.2050939636493974, -0.72951051931046962], [2.2053862976583574, -3.8378980513119298], [-3.3775634258105534, -0.32876488972317475], [-1.5833280552655529, -0.99983356462624506], [2.1432211818337592, 1.7286569989931913]], [[-2.3040079566123284, -4.6439110001927206], [2.2610186287921428, 3.7876334777496261], [0.19620330726680901, 1.1761821665949403], [0.79697521626000078, 1.6653438936202667], [-0.29650072235050207, -0.75197297014468223]], [[-3.7739686952398985, -0.82163647493707082], [2.4807391769962583, -4.5036286071330096], [3.9794455956840977, 1.3703907393906656], [-3.2006716375117463, -3.6689469210509773], [2.9251148086043433, -4.5096121013489636]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.2773425175531129, 0.11664628489495757], [0.06332274619068215, 0.061334768660572928], [0.61051050613681135, -1.3392794664365673], [3.2774754395960977, 0.6674622923143696], [0.628879050295863, 0.81941772347291009]], [[-0.4895094551497613, -0.24718424166537434], [-0.65867868594443002, 0.42980736847627116], [2.1190340937410874, -0.94910860546932363], [-1.6103369974360786, 3.1338042491610718], [0.29560006277739337, 0.42857366747178477]], [[1.0140053392503103, 1.6774766610507985], [-1.202456027760691, -1.3406079838629286], [3.3735404160359668, -0.53953077473777344], [3.5251735579785985, -1.4829117574469504], [-0.085936715364943211, -0.14296016700010089]], [[-1.5436424647070632, 5.1474511883747684], [1.2998572268165001, 1.0777626981827089], [0.42956808035990357, -0.45695270784975234], [-3.2035296692502544, 3.2753276720518145], [0.020810332887218324, -0.15862890582076061]]])+(1.-msk_ref)*numpy.array([[[0.50559428717255017, 0.35773954429214827], [-0.26656448805391447, 1.0119743896580427], [-1.5285881096621092, 2.9277315298277289], [0.1178359032583121, -0.29766635609733028], [-14.949307540863456, -0.81246756835649281]], [[0.89055418977078216, 2.6918812219765735], [2.0650207964133478, -1.1866309390977099], [1.1879840583281958, 12.204744579732607], [2.3379471337374622, 3.7023534911612836], [-0.3949185345658508, -0.48962736324975986]], [[-0.68222060703458498, -0.33847369312360759], [1.8263400794825386, 1.0902292860377731], [3.9891073333005496, 0.66543778171854473], [4.743053140586972, 2.2698589864430243], [12.937049237438812, 5.1010403249555498]], [[0.95189225291769908, 4.3722639796729075], [0.70497473013151657, -0.38832208079052355], [0.0024378299888962638, 0.0070791574501250813], [1.0487844443705554, 0.91492591667114831], [0.59874360405879024, -0.38836905335286087]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-4.5330592108718477, -4.6395072161654198, 2.0121288499410417, -1.0574253071969721, -4.541464928617537], [3.6358354694123953, 0.45518353021901703, -4.5163281394832797, 3.1107824969379632, 0.5048036944386558], [-2.7832441900184945, 0.20371133333451041, 0.38818619288046197, -2.3298441575270399, 4.261540192328269], [1.0515794914381749, -1.5748481442160922, 1.4858210105216907, -4.6936553235420284, 4.4537028820181508]], [[2.8320062438657292, 0.21231040636825771, 4.2119125351929441, 0.70189818282361927, -4.1754034552790174], [-0.28773387112433024, 1.6441110956936864, 0.89875484213750312, 4.829762143486132, -4.2684626160132488], [-0.69528026847473257, 0.30819055704841247, 3.0519530947429416, -2.6196388028437632, 1.3458157364857692], [0.2804087730885847, -3.8425497806330822, 2.1270006871334086, 3.4916084277832731, -0.45451377258430181]], [[-0.02703073177433879, -3.0692380727238766, -3.8015513339756257, -0.3014600538569816, 1.4491470453886262], [-1.0358983232038632, 1.5425253505734302, -1.2279618164266672, -0.6049045685582648, -3.6626959519129421], [-3.9971790963394915, -0.10065443663090434, 4.3947814965911256, 0.55712394389656339, 4.0074696587129885], [-3.771309756733805, 1.0192482886840626, -1.1671236223068626, 0.047945887113897356, 3.0418964118739531]], [[2.7723487569156822, -4.4305895838049283, 2.2036291196377444, 1.9190671524309311, -3.800794788549815], [-2.3000286767213707, -4.6583126375509316, 4.999103505819205, 4.4260948364118953, -2.2520942000356325], [-1.8117169168125091, -2.1082094380802752, 4.9927255685502132, 0.68690950569459552, 1.1009743989999299], [-0.79953336488655768, -3.6838439562679515, 3.2275102331781813, -2.3953296771033958, 4.1524785537370974]], [[-0.33831201468260375, -4.3933528013042569, -1.7109830733295919, 3.6577485071044347, 3.9551577470244776], [3.1860284402212038, -4.9563328146115389, -1.8861811589988333, -3.5905623376175688, 3.7442751559295857], [1.6058023659800824, -0.14145490930859683, 4.2475339778446788, -0.0074888748559329699, -1.6606052208472155], [-1.1375112703184698, -3.8728015221607812, 4.3962643901270066, 0.14327117470930695, 1.3583827884941693]]], [[[-2.1242186576931257, 2.9751624272464561, -0.7165287546765633, 1.1194666430407327, -1.5582789477227807], [1.8027085310579816, -1.4442692064112039, -1.3626517887548517, -2.4441543976662015, -1.814560866491346], [0.37845705588803469, 3.6835336107542034, -1.9533336810621238, 2.1337264672822842, 4.631436042503104], [0.089806989545365923, -0.29322064404467074, -0.73549356208000294, 0.79491617536172843, -1.932895506271215]], [[3.2296658194912844, 2.0827430245954766, 1.9959271322250194, 2.6916876566124408, 0.16033509059006157], [-3.8886669501599167, -1.5968173011623312, -3.6167780884907796, 3.6534073581735313, 1.2456977137328469], [-3.2525329899016753, 4.9725119432362312, -0.25220690250732059, -0.29802405678665966, -1.1460055235603539], [4.4020587466672119, -4.2326468579304226, -2.0175839046636277, -1.4327738854765948, 0.80099487207361353]], [[0.9393815241789385, -4.0378123578121237, 4.2102341584935008, 4.0901904149235602, 1.6607420807210369], [2.897336657555492, -2.7706292635513763, 0.25951782974798743, -4.9150618798108372, -4.5091370073066281], [1.5284727770026372, -2.9118653998897734, 2.272498731012468, -2.4655584670780106, 4.527616747787512], [-1.7331191338717322, 2.5316397452088637, -0.88453268519629979, -4.3615512801677205, 4.2726743461951422]], [[4.1957001840403585, -1.222635340525684, -1.7343404594931733, 3.8521626589965248, 1.6953342837462335], [-3.465060769048697, 4.1955678148488946, -3.0262439818955711, 2.4926650435094171, -1.8055485259009494], [0.55130822061609219, -1.7900954403067648, -0.84072365613289435, -0.31573038012718602, -2.4808243807738828], [-2.8504460122046869, -1.4702457569773095, 3.8297919843771595, 2.7935161244931974, 0.074061245700381306]], [[2.9524632634895021, 2.9404599818037109, 0.13984582937518386, -1.2063984708834266, -0.0002663312096355952], [3.4540670682632975, 4.9918276631409828, 4.2154198720165752, 3.4803216485586805, 0.36520141246981019], [1.8248570259235111, 3.8118537825158629, 1.843599742062163, 2.5726802738320078, 3.1273467785649576], [2.8496165060446828, -1.2829545984562007, 1.1648941279767389, -2.1305722067463084, 3.4968774715100608]]], [[[-3.8672661274542741, 0.95952017581066951, -0.70762340703795079, -3.2605781731028172, -4.8970006298183355], [2.6303531037034977, 2.8109941052408765, 1.8047063636626817, -4.3194146648428706, 3.5576857918119096], [2.5454587507900559, -4.0519016802077008, -3.7947462630538222, -2.5536653792575192, 3.5111749123191327], [2.8856864723247524, -0.071378225899239034, -0.082448896890730872, -2.1824444294374135, 4.0711949961522418]], [[1.9529241372287887, -0.65954495634105736, -4.6798408266332236, -1.1366022428142886, 2.0643506431088179], [0.91045614438891054, 2.0785420314777747, 1.1444151603109116, -4.0421957306196159, -2.4676999904992512], [4.2827480470896671, -0.4780651414888295, 0.4579908138876041, 0.69744071058333024, -1.4274927291619455], [-2.952885495051687, -3.5644028516655846, 0.32975476424637229, 3.4299049703999867, 4.3477343343763426]], [[4.788080167245143, -3.9380573578428368, -3.2110038748255363, 2.5283983725597947, 1.8620271798599006], [1.7574558702916203, -0.44817586356643702, -3.5208227137947654, -3.3160287006238507, -4.010299383374349], [-4.9407427133262196, -1.5637026796290399, 1.3149245547555779, -4.6795319344633608, 4.2710443668536922], [-1.0170369062805906, -0.40512422698035788, -0.13076905350192103, -0.41858387939543551, 0.89418218380674563]], [[-4.5701572290315813, -4.8853063665285479, 4.2498560805376009, -3.340393987556415, 2.6771607388492065], [-2.6893285583297022, -4.1496432065618922, 0.28533543807977146, 0.60187190542819735, 0.48046564973398187], [-0.10379267127414948, -3.0479327670037248, -4.190858750930686, 2.194773499368166, 2.6926415031727071], [1.2001536379163884, 3.3481225763695157, 3.7216339151235456, 0.069986134324567573, 1.1712418277480676]], [[4.0533139532568292, 4.2921486991791724, 2.3781016958391286, -1.0250553122825767, -4.2338581281713061], [0.38198500840107208, 1.7819570987408149, 1.3212124775927849, 4.4454062504406728, -2.2484071510020378], [1.2119151619932067, -0.13299730240398766, -4.7252679434059175, 3.2840805564570843, -1.6417174874745677], [1.268327075960328, 4.8999329020844264, 2.6772614287297731, 3.3552116420287046, 0.014970132426431526]]], [[[-2.5439446589240964, -2.2971244342760109, -0.41732856969003773, -3.1791938208081172, 2.3422147596295968], [-1.4484074232722923, -4.9729766806443347, -0.39474565217014401, -4.8810926144705054, -2.6468426382063392], [-3.7222372458985586, 2.5218262601574173, 2.9802808712959354, 3.8156236636618246, -0.85230381741076311], [1.706585011416804, -3.7564858238165355, 3.5526570902074734, -0.5312581403439669, -1.4517876052980139]], [[2.8876752405106014, -0.30928247409644971, -1.8148455854733028, 3.8621443434501046, -3.8677068238113197], [-1.4399330747194861, 4.5332589559686927, 3.9656970536664975, -0.54031150621276414, -0.89241344654740473], [2.6589070463045434, 0.62249540336455933, 2.7959758996901805, -4.3683488553200327, 1.0018145577191087], [4.0926654820802799, -2.1344821039998618, 4.734089577218155, 4.5124269937918395, -0.38810434015201523]], [[4.3552162079763885, 2.2080506432829408, -2.5581164073635709, -2.2588454986849058, -4.4764864513660205], [1.3229389461326511, -3.8784343312154714, 1.0538173834977425, -0.49375782619489517, 0.51463673029398826], [0.31782082080987273, 0.98240819469377794, 2.845359978311742, -0.57799718607389927, -0.69612511624667395], [-0.25411499771167989, 1.4406883621431978, 3.0824912782249179, -1.5642192029759991, -0.37012395875404458]], [[-4.5030128270920198, 2.3214236016861829, -0.54956287413896732, -0.1192213059173497, 4.9663633798114653], [-3.9567234705277778, 4.8855391715864442, 2.1144046957197329, -2.9862559695805913, -1.8476072515257549], [0.40876104229301635, -1.8760737043596052, 1.8086999520657798, 2.3499456607066191, 3.7510661208956932], [0.41558991264530221, 0.840790767398917, 2.4019148487631004, -4.6976786185976236, -1.3746040461677755]], [[2.2977706171515653, 3.4244062402528428, 1.3449669892844041, -0.51583344060784153, 3.752753330530993], [-2.8591953928822935, -0.9026182936575653, 2.2935812575691683, 4.1652190091179797, -3.5762268826899577], [4.1476756113308326, -2.4385896205600277, 4.9047845999828503, -4.7147802732750339, -4.5736204879080606], [1.6822838380195124, 1.4333629966383832, -4.5990721704429784, 1.4621564953424437, -2.2519600414013876]]]])+(1.-msk_arg0)*numpy.array([[[[-0.11735831203695124, 3.0999921157329169, -4.6818126977528207, 2.9690866170245487, 0.91352198552959507], [3.1508229328364692, 4.7802517158984124, 2.7157214082486059, -0.71747863940126066, -3.6467089553525711], [-0.84738504585347485, 4.7119075044074439, 0.37707300782192732, -3.4299155444074048, 0.79223237571242855], [-4.0188881320448289, -2.6362158420864414, 4.2286609608395249, -2.8504373390894799, -1.4774994714462109]], [[-3.0633760666487442, 4.5472670078538684, -1.7574711049107328, 2.9856790453308166, 0.51405737372245675], [1.5095611841700745, -4.3100724031571067, -4.8456775751915586, 0.3409582587326252, -1.327028857266682], [0.67836091874695637, -1.1408467147185175, 1.8123781451939838, -1.2535867818360256, -2.2361883730452226], [-2.3814963085550902, 1.05960421348155, -1.4163119970088101, -1.0293743247931375, -3.0971711192476437]], [[-3.345623248739682, 0.41988965696071467, -0.50384546151831344, 1.0658646276153094, 2.8242640305063542], [4.8356378171355257, 2.3278386646354479, -0.67575802761312698, 0.66466150976614902, -1.9514057837372434], [-2.6062030862788821, -2.9027347476329601, -4.1097896591890448, -0.70658171088170718, -1.4308665136389909], [-2.452923040638042, 0.39068806275588397, -1.8224110135990479, 3.2001916769379974, 0.31826688544375425]], [[3.5030473438701346, -4.3587666530327649, 2.8522479096450128, 0.95011724395287622, -4.1009231984391192], [4.2622344415696656, 3.018818273592542, 1.0336483138192474, 2.5286559257349941, 3.0230156839596027], [0.76924244438455158, -2.2031736295352609, -1.6789696453383107, -1.1257062737385057, 2.2367858037324337], [-0.45697189739718169, -1.3036526957029082, -3.4402935840361435, 2.3868583519777786, 1.9693227204900507]], [[1.9395035658120161, 2.6200773017888768, -4.4403870259673663, 0.30200165044758798, 4.2182224265932255], [2.5557094072771385, 3.2921055022777868, 1.1459437818889731, 1.4989287268930385, -2.8797304605364693], [-4.9277228134899973, 2.4841987196163249, -0.22658684133409412, -1.468127817717316, 3.1636411319941331], [2.3054594931845029, 3.2596397099055494, 4.4169661228900399, -3.9557323286326262, -2.3092437653611242]]], [[[-2.057797683937217, 1.2157570499699357, 3.4594300510538556, -4.9247423087807647, -3.302460446504083], [3.331657284465825, 3.2048324985876047, 0.10485539586366954, 0.09536989214369207, -0.78445350715907303], [1.4479393281785367, -2.894562052226155, -4.4239551657722789, 4.2387276385159893, -0.63826759021882573], [-3.8230756435657476, -4.9985162166276265, -1.656998438628726, 3.8399860681830411, 1.6453501835545046]], [[2.6846828113185559, 0.57263301287937551, -3.3961701387963394, -3.4749766502849866, -2.0395118702484361], [-4.3709223615304884, 4.5147392160462338, 3.1595545230616686, 2.6414384031509019, 2.3814795959213946], [2.9097504391519582, -4.9321223375451435, 2.5944220043719817, 1.2763099930865582, -2.1271218879692499], [-3.5624655220674795, -0.29271232207716658, 0.97340132631809517, 1.7093228260685667, 3.4043979664535478]], [[1.8292182472916068, 1.1424012943807753, 0.81896551370051185, 0.69041762600317025, -0.33190963075262125], [1.8211831195012493, -3.57286073865138, -0.36002423643413906, -4.1679817600240687, -4.7101556461575793], [-3.1047380631634267, -1.3019404922182307, -3.8427598116701955, -3.4549791380594099, -4.2784594784269387], [4.473782728767306, -3.6044487513030288, -0.4211060487357976, 1.5694444807424368, -3.4422370740777284]], [[-0.32276701033701816, 0.48684687004924143, 2.3997610465779937, -3.5763859304197454, -0.7191127161930897], [2.9882342326327525, 0.60143213460924194, -3.6787765369928485, -1.9146541515558511, -4.4514697439472775], [-4.2567893541468784, -4.2669354681444496, -3.905681465722207, -4.9068116087795932, 1.1436427180269746], [-4.6104998473100469, -4.2558347046796117, -0.80306313883920932, -2.722735816009564, 3.6317854445988438]], [[2.8651582248445582, -0.67572433135827747, 1.4591939092856894, 0.53329032449262748, 4.9414150776424606], [1.3066699868955194, 1.9410819224525344, -3.3044206410304202, 0.89871681903206024, -2.8597406037755233], [-1.305733917019285, -1.4275070304643012, -2.5227758776666533, 4.5329982762491028, -1.7238116028356485], [2.8327853346682099, 2.24733110352018, -3.3974453939794413, -3.6622255207311727, 1.5406926635850269]]], [[[3.258183536397329, 3.535894511586724, 3.8644229617553929, -4.3642686383493139, 4.4879193345040402], [-4.0138745199809547, -1.916308932883195, 1.5803316610263796, -0.85079099331113817, -2.1583515103145157], [-2.7126664930168598, -3.8494427877102289, -4.6869597690970322, -2.4970392097088556, -1.4728515175377508], [2.328996410912735, 4.1518279232474775, 0.83104190193262184, -0.51695835623888087, -3.3758787956490357]], [[-0.54330185531442154, -1.5973867886918178, 4.029191597575517, 2.2582995186950541, -2.1135365207350034], [-0.21873594522474082, -3.553082614273789, -1.0536194407654453, -0.29857286989220633, 2.3586901481591287], [4.5339386117296492, 4.4307668882757749, -3.7498737563855822, -3.8715969099832881, -1.2204792511427573], [1.9432127310425447, 0.75707930180955341, -3.6018299224700732, -3.8873717419363709, 2.4717594055574654]], [[3.4721937494148758, -3.2092419319173038, 1.2789299640832672, 3.2844641200662537, -0.38707020294642103], [2.1272895907451286, 3.3980684860237513, 3.0335896448170256, 0.5424863087868248, 1.865844401430742], [4.388038147177852, -3.1189213052544353, 4.7540369089988523, -3.7311867867448156, -3.1377517364993901], [1.4323959092959546, 1.1082301817133882, -1.8334970756442193, -3.2542802755523126, 2.3851924521142278]], [[0.29517002459452879, -3.5436471689686213, 3.9111123209191714, -2.3723305292184804, -2.817948583014207], [-2.7063158497080564, 0.19772220846301902, -3.7015727304931803, -1.0961623950314991, 2.8169799877324175], [-2.8409373427656592, -0.11404266846314748, 4.7887916573800489, -3.2617784308759434, 2.9452465317684782], [-0.56157356660485291, 1.9370233449955476, -2.9484677827652139, -4.7969990234916207, 3.6327868418842382]], [[-1.8253202094119967, -2.7204904917424777, -3.4215502407245868, 2.7671900340389417, -2.8291897064412055], [-3.4541144162834083, 4.3692614797141065, -1.6519659123396568, -2.3520180316569119, 2.224069769794812], [-1.4050938027895823, 4.821383911829102, 1.5944727693735956, -0.78981170690568625, -4.7703411123822459], [-3.9221261967469667, 3.4788593702384176, 4.0932597310688603, -4.8577354265032344, 3.1884719112385813]]], [[[-3.5486376455785718, 2.7615068231679825, -3.4013997420844211, 0.52646538422275135, 4.5271288182777294], [-2.2161708154584767, 0.1274786657068363, 3.8603289541611172, 4.9758282784618419, 3.8835315006884912], [-4.9620501632166869, 1.4743386867457868, -1.2150607880678232, 1.5073524374920417, -3.1205259866938806], [4.138566006626661, -4.942345654011925, -4.8886996954664639, -1.2360981843811638, 4.0356400324501855]], [[1.6444205669736363, 4.661416513035487, 1.3784460920409503, 2.2275898032618304, 4.2783758290120595], [3.6931635413449655, 4.059329925234211, 2.8901336530385962, -3.8859646641743484, -4.1311055928679004], [-2.9880118219802787, 1.7467422078626837, 0.42282294206731574, 3.8086627488567935, -4.8708582552174784], [-4.2645551651733218, 0.5854572560095761, -0.30492382309752042, 0.59390683874652073, -4.3661975822776791]], [[-4.9833058613960706, 2.5368559946746707, 3.5552712535051949, -0.96469462286848007, -4.3727666023468856], [-1.6866698153185045, 2.2883149296565968, -3.7637176488345325, 2.3505043496596398, -0.92115745782971548], [-0.018316527755902534, 0.87261865011122186, 4.7859544146779793, 1.8690928500551784, 0.97401200198945936], [-3.607818243182451, 2.916599642247073, 1.5375013215306508, -2.5204308065220991, 1.4868068584466441]], [[2.5701639789155148, -3.8376359946967842, 3.5826479875496808, -2.1273561012591569, 1.7942620831876717], [-4.0673356583697089, -0.32753916062894639, 3.726555190195544, -3.2184812309851942, 0.51078031819266911], [-3.5691946062544231, 2.8030828212675534, -1.6927166742413489, -3.8515064657507461, -4.8831649682463034], [-1.9856660403211746, -2.9091321235732179, 3.5954904947901891, -4.4225081451634916, -0.27823029672933153]], [[0.38646911658450023, -2.8429036381188544, -2.0993329204081599, -0.21060457178764036, 2.6968094410685284], [4.4520823973244408, -0.5971822959379498, -4.999184088258346, 1.332096336816905, -3.0838445550730107], [-2.4446857329683382, -2.0350486080084442, -3.7297260776054788, -3.0676072504156329, -3.9052826477404201], [3.0485099760470327, -1.5578212904057809, -3.0108853068618648, 0.73663460341698705, -2.0982584306329501]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.42064116805735541, 2.0697555833304717, 3.066345438313947, 1.8152654916615587, -2.2394338097676947], [-1.2826735194852912, -1.5425481461229609, 4.9480962575073075, -1.9833848207734448, -0.63692446969401217], [-3.1644349959169658, 2.8822232870863349, -2.8333494934530079, -2.0914542218959067, 3.3321385819297618], [-3.270659368517733, 1.978428097804934, 3.5236851202822699, 1.519598006105209, 3.78266712346114]])+(1.-msk_arg1)*numpy.array([[-0.084181089689349342, 4.6942377726291156, 2.7860574690424595, -1.6509269252260528, -2.4476241342391836], [-0.5450049423369201, 3.1644914266224333, -0.58865829621481858, -4.6708834287098231, 1.7615831277172864], [-0.75503430293632512, 0.31669981504743294, -3.0857449722034378, 3.7009179316051277, -0.74460738707414631], [3.8002194077420572, 0.39755192439033582, 1.1391920319038586, -1.7526761694585935, -1.451211647898222]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-10.776546746023095, -11.029608056653199, 4.7834805595316414, -2.5138416957153127, -10.796529853678749], [8.6435559462801805, 1.0821183583175855, -10.736771582156381, 7.3953353431963667, 1.2000815249966799], [-6.6166709332620357, 0.48428767511109111, 0.92284403515048219, -5.5387925254366932, 10.13105828896709], [2.4999443023959782, -3.7439230009017042, 3.532276732169723, -11.158335607564776, 10.587891105824616]], [[1.3682805190498433, 0.10257752561615327, 2.0349806368998888, 0.33912128971971917, -2.0173413174517538], [-0.13901828478768191, 0.79435036143162618, 0.43423235544136307, 2.3334939557039371, -2.0623027425995915], [-0.33592385210815456, 0.14890190877151729, 1.4745475839383908, -1.2656754372071639, 0.65022930597447592], [0.13547917220127759, -1.8565234521314751, 1.0276579052444554, 1.686966546148835, -0.2195977999744963]], [[-0.008815292444416126, -1.000943349165357, -1.239766168049852, -0.098312489548712315, 0.47259745339893944], [-0.33782831844720657, 0.50305009060609929, -0.40046427942634905, -0.19727215368496645, -1.194482495725238], [-1.3035645124632045, -0.032825537323104059, 1.4332310514263615, 0.18168988299077685, 1.3069204821608507], [-1.229903751094491, 0.33239839058853848, -0.38062365959283906, 0.015636166269727511, 0.99202665618344765]], [[1.5272414804613956, -2.4407391668915035, 1.2139431558414668, 1.0571826331994887, -2.093795539004518], [-1.267048091470133, -2.566188063921746, 2.7539241663451657, 2.4382630842393076, -1.2406417740989686], [-0.99804514829078717, -1.1613780175761383, 2.7504106652632085, 0.378407185532872, 0.60650874709912539], [-0.44044982321276016, -2.0293692427855472, 1.7779824758437726, -1.3195478502215616, 2.2875323597631043]], [[0.15107033447784651, 1.9618140898569343, 0.764024846756725, -1.6333362884629608, -1.7661418389654311], [-1.4226937301405229, 2.2132079961433102, 0.84225805235766016, 1.6033348795381597, -1.6719740228973297], [-0.71705730215203756, 0.063165479011532161, -1.8966999423328759, 0.0033440929681729779, 0.74152904792460761], [0.50794592157937823, 1.7293663716555758, -1.9631142349248754, -0.063976516780448647, -0.60657420753823477]]], [[[1.6560867792339926, -2.3195009346106432, 0.55862130447979519, -0.87276039150628926, 1.2148679488979268], [-1.4054305352630723, 1.1259834903200914, 1.0623527874042757, 1.9055155973337528, 1.4146708721479566], [-0.29505330088977061, -2.8717624202862821, 1.5228611578774573, -1.6634992730952316, -3.6107676444132082], [-0.070015470173114283, 0.2286011518833988, 0.57340667824431302, -0.61973383194245013, 1.5069271150517269]], [[-2.0937212414463198, -1.350196445945782, -1.2939156143952983, -1.7449618434132679, -0.10394170904360271], [2.5209371648682013, 1.0351814983381689, 2.3446776021748081, -2.3684235512233456, -0.8075584006008385], [2.1085455245442994, -3.2235700102678404, 0.16350018191731466, 0.19320243425510772, 0.74293014868983065], [-2.853757762914138, 2.7439317654808724, 1.3079552231381699, 0.9288357637832747, -0.51926733962037641]], [[0.18984705941273844, -0.8160335101981715, 0.85087959881655217, 0.82661900700049584, 0.33563253305780788], [0.58554573451549574, -0.55993843275537458, 0.052448015608880696, -0.99332382072269709, -0.91128724516329174], [0.30890118087004087, -0.5884819632342152, 0.45926728437519931, -0.49828425696796774, 0.91502196242002376], [-0.35025978551694992, 0.51163914634195551, -0.17876222271429679, -0.88146047554154305, 0.86349863136000893]], [[-2.1154241678648091, 0.61643879075816599, 0.87443467416315424, -1.9422164668449602, -0.85476820533753883], [1.7470441100267444, -2.115357428828553, 1.5257976920058562, -1.2567732783884937, 0.9103369688978733], [-0.27796331546043745, 0.90254569943148777, 0.42388327637046452, 0.15918765577930719, 1.2508033513165919], [1.4371623612068993, 0.74128113797098105, -1.9309374279085618, -1.4084589612840901, -0.037340835184722365]], [[-4.6354998182247709, -4.6166541273196033, -0.2195642278312338, 1.8940997375450153, 0.00041815195098964344], [-5.4230403016588165, -7.8373934440595914, -6.6183983699695581, -5.464261170921378, -0.57338260633204796], [-2.8651074228631837, -5.9847814990482835, -2.8945343282977576, -4.0392234813461654, -4.9100747849542987], [-4.4740257936921157, 2.0142962933619915, -1.828936056635367, 3.3450939760092222, -5.4902545559130616]]], [[[1.2221031977095953, -0.3032200620485892, 0.22361761513539991, 1.0303824149682026, 1.5475118421256493], [-0.83122361720098914, -0.88830837380697347, -0.57030919136947789, 1.3649876424752481, -1.1242720411076073], [-0.80439596770811606, 1.2804502811515557, 1.1991860373021219, 0.80698936225660645, -1.1095740367078362], [-0.91191207152244258, 0.022556388736484568, 0.026054855605222967, 0.68967902082153576, -1.2865472039733026]], [[0.67757558756074621, -0.22883201287565655, -1.623691282906855, -0.39434912898899305, 0.71623550207162756], [0.3158867491176573, 0.72115926645606676, 0.39705985495239371, -1.4024575225418803, -0.85617932571555588], [1.4859182028950766, -0.1658667958276438, 0.15890191989621702, 0.24198011087766183, -0.4952748579743142], [-1.0245165627111372, -1.2366851893937998, 0.11440986051421585, 1.1900205600889819, 1.5084654800535964]], [[-1.6899010087915067, 1.3898946695218737, 1.1332890214374085, -0.8923708064967415, -0.6571823151935432], [-0.62027500467293428, 0.15817881436865888, 1.2426362232863595, 1.1703563955968597, 1.4153917095793891], [1.7437816000965445, 0.55189191564340079, -0.46408837236421446, 1.6515900863187927, -1.5074188259241408], [0.35895215490734467, 0.14298420576652274, 0.046153520349003101, 0.14773464423031929, -0.31559191193071079]], [[2.1851576674189532, 2.3358418823530402, -2.0320100894606719, 1.5971633290296654, -1.2800474955757606], [1.2858653706949519, 1.9840946854673367, -0.13642920561804811, -0.28777675319261803, -0.22972802593711039], [0.049627034714659575, 1.4573270287698521, 2.003801329742549, -1.0494006879952649, -1.287449409593973], [-0.57383691469395259, -1.6008586472117172, -1.7794479440003608, -0.033462905184281314, -0.56001313128734653]], [[1.2164301854784829, 1.288106299796641, 0.71368631206865474, -0.30762685496979847, -1.2706128584001828], [0.11463659118878866, 0.53477880794166111, 0.39650586105804253, 1.3341000505045555, -0.67476399787067198], [0.36370490968336106, -0.039913496733069286, -1.4180886620475863, 0.98557742293994111, -0.49269183952240958], [0.38063455188763307, 1.4705069376936593, 0.80346641140575681, 1.0069243999106365, 0.0044926500079002663]]], [[[0.77780788895696451, 0.70234291482242328, 0.12759768678667738, 0.97203452350005248, -0.71612922524888045], [0.44284875313344307, 1.5204813832074766, 0.12069298807752128, 1.4923879452119841, 0.8092688170721648], [1.1380693696590853, -0.77104521627402367, -0.91121713865501153, -1.1666221497688616, 0.26059082324951138], [-0.52178622691308585, 1.1485408294043717, -1.0862204497368806, 0.16243151012840989, 0.4438822395485244]], [[1.4595805850687609, -0.15632737648621076, -0.91731692826586619, 1.9521277259128871, -1.954939291502451], [-0.72781673305039074, 2.2913438001605129, 2.0044686274251959, -0.2731014115763114, -0.45107196341253819], [1.3439492945205342, 0.31464140852792072, 1.4132309901948501, -2.2079896965508707, 0.5063689495871101], [2.068645045337302, -1.0788777749204379, 2.3928539947803147, 2.2808142478356315, -0.19616802884199683]], [[1.2359833694866322, 0.62663108873532425, -0.72597758313848693, -0.6410463539102943, -1.2703991130193339], [0.37544187433713577, -1.1006756275954601, 0.29906684267331041, -0.14012541113643603, 0.14605071472809766], [0.09019557933269963, 0.27880135743090478, 0.80749552845511074, -0.16403201941823847, -0.19755599393367754], [-0.072116261537955931, 0.40885842887907914, 0.87479192180996879, -0.44391571595667867, -0.10503888574595316]], [[-2.9632921397636096, 1.5276563883076455, -0.36165016795956395, -0.078455818866805901, 3.2682086708842526], [-2.6037961715078977, 3.2150207831005768, 1.3914237102344176, -1.9651618109413593, -1.2158526426743919], [0.26899287880792067, -1.2345855264498917, 1.1902489637384763, 1.546425864778294, 2.4684594911451794], [0.27348674516260779, 0.55329815123533732, 1.5806251647561087, -3.0913956189229039, -0.90458400224605529]], [[0.60744721704433391, 0.90528881566494057, 0.3555604935318124, -0.13636765376696799, 0.99209187804430021], [-0.75586756633931085, -0.23861954123831802, 0.60633970230786305, 1.1011328444113275, -0.94542468738769447], [1.0964950062895595, -0.64467465440858529, 1.2966471645263284, -1.2464169115047614, -1.20909938374995], [0.44473483473751263, 0.3789291919842146, -1.2158278855462248, 0.38654114877667867, -0.59533656224575326]]]])+(1.-msk_ref)*numpy.array([[[[1.3941172829911646, -36.825279016614232, 55.615966899810367, -35.270232637534981, -10.851866956114902], [-37.429106043457573, -56.785339005931327, -32.260468690418975, 8.5230381555875319, 43.319811715551545], [10.066216165418522, -55.973467696790792, -4.4793077544306827, 40.744489731181993, -9.4110491873647302], [47.740984903801994, 31.316009947302661, -50.232908322337131, 33.860779773799003, 17.551441504244927]], [[-0.65258221143174644, 0.96869123979356231, -0.37438902544691954, 0.63603063797482473, 0.10950816695306577], [0.32157748654572521, -0.91816235391569267, -1.0322607864998763, 0.072633359290120422, -0.28269314882263602], [0.14450927959003337, -0.24303130134789916, 0.38608571465243863, -0.26704799427616671, -0.47636879113449637], [-0.50732332359493548, 0.22572444447953355, -0.30171287983471107, -0.21928465805357208, -0.65978147449335567]], [[-1.2008450241658295, 0.15071105374758353, -0.180845322509259, 0.38257094100131273, 1.0137134864906523], [1.7356561631865715, 0.83553145995782452, -0.24254992408515477, 0.23856704937051662, -0.70041835296668242], [-0.93544484104795167, -1.0418789920477129, -1.4751273815617088, -0.25361347306470039, -0.51358111939118245], [-0.88042801266446502, 0.14022972142428908, -0.65411824194186219, 1.1486452496035093, 0.11423557804539454]], [[-2.1218669889889163, 2.6401935703095654, -1.7276645416964591, -0.57550532942139865, 2.4840125482099102], [-2.5817220474407492, -1.8285595973178468, -0.62610179652725417, -1.5316582988000869, -1.8311020541055607], [-0.4659457863522477, 1.3345070553220226, 1.016986045647307, 0.68186317427972698, -1.3548666325289731], [0.27679716795133796, 0.78964893950373116, 2.0838557609477975, -1.445768625798479, -1.1928588058011118]], [[-0.79240253382077763, -1.0704573733921359, 1.8141621353752546, -0.12338563189623958, -1.7233946861307661], [-1.0441592610262245, -1.34502085358016, -0.46818617526223111, -0.61240151456463865, 1.1765411282936222], [2.0132677826457717, -1.0149428929325826, 0.092574198041451367, 0.59981751167593678, -1.2925355195427157], [-0.94191729070408481, -1.3317566469080317, -1.8045933038092894, 1.6161518728700643, 0.94346339090945708]]], [[[3.775741326516437, -2.230726651316052, -6.3475205127869252, 9.0361424754498962, 6.0595055016262842], [-6.1130771955572589, -5.8803732767002845, -0.19239347704639403, -0.17498904089705436, 1.4393511805515458], [-2.6567453167854498, 5.311074868082108, 8.1172753164454896, -7.7774113760157855, 1.1711225727275167], [7.0147540812617732, 9.1715062164290639, 3.0403365362591068, -7.0457821019339963, -3.0189637849877609]], [[0.84837733758185807, 0.18095577951701569, -1.0732119892077523, -1.0981153625667883, -0.6444990980510491], [-1.3812400706029779, 1.4266871378017809, 0.99843990616652289, 0.83471182159914925, 0.75256313728213409], [0.91950018087349705, -1.5585829356502197, 0.81985433189847323, 0.403322310292623, -0.67218443699169628], [-1.1257624185981181, -0.09249900935569548, 0.30760118928716412, 0.54015719925428385, 1.0758120365922983]], [[-3.1074364517646611, -1.9406866457614314, -1.3912409269802382, -1.1728665516865777, 0.56384091226924249], [-3.0937865502139235, 6.0694986575837522, 0.6116013971928389, 7.0804773955025526, 8.0015106836083838], [5.27426196679375, 2.2117083893830229, 6.5279973736543759, 5.8692439404584338, 7.2681545574711546], [-7.5999654766348392, 6.1231596912509332, 0.71536586071000297, -2.6661383876422948, 5.8475979973644261]], [[0.069101919425587521, -0.10423014778249706, -0.5137702713426201, 0.76567655455439265, 0.15395646822890649], [-0.63975782702377426, -0.12876196629367984, 0.78759759114968719, 0.4099126387499486, 0.95302522785862986], [0.9113456627888219, 0.91351786728769835, 0.83617618065904553, 1.0505103978017576, -0.24484505671828938], [0.98707234245482867, 0.91114127972471048, 0.17192960413080335, 0.58291667038276518, -0.77753716187303867]], [[1.6264677946577055, -0.38358923897840791, 0.82834235088102692, 0.30273355602791308, 2.8050990043516699], [0.74175891352271439, 1.1018962953896181, -1.8758244155712311, 0.51017565103308249, -1.6233923672289385], [-0.74122753361704541, -0.81035462250033508, -1.4321071983334355, 2.5732525504618686, -0.97855819331638161], [1.6080906373910497, 1.2757451341125983, -1.9286318882844635, -2.0789399393696493, 0.87460684616201101]]], [[[-4.315278820745319, -4.6830912156383437, -5.1182084664586345, 5.7802256419035425, -5.9439939576924408], [5.3161485569212079, 2.5380422126924271, -2.0930594211156719, 1.1268242912969857, 2.8586138430011672], [3.5927725170463272, 5.098368077767808, 6.207611695083874, 3.3071864417257348, 1.950708082811387], [-3.0846233102989866, -5.4988599949711379, -1.1006677427776506, 0.68468194653995462, 4.4711594990058829]], [[-1.7155104913245682, -5.043851346905817, 12.722431167103949, 7.1307257263690627, -6.6736272656757114], [-0.69067279117918079, -11.219086483336389, -3.3268710327716549, -0.94276300681605485, 7.4477155845697656], [14.316202272017714, 13.990430930981654, -11.840467149701221, -12.224815822527175, -3.8537415974176148], [6.1358189639343639, 2.3905265044002748, -11.373009238829576, -12.274625867255873, 7.8047390245152615]], [[-1.1252367842101632, 1.0400217648659671, -0.41446392219834738, -1.0643990834151522, 0.12543817017711151], [-0.68939254860912724, -1.1012149470010455, -0.98309797865467485, -0.17580399990069551, -0.60466578354283074], [-1.4220352578406652, 1.010751482494455, -1.5406447881543941, 1.2091688782953722, 1.0168538763781299], [-0.46419776170715876, -0.35914509841104408, 0.59418295813829813, 1.0546173792283711, -0.77297134844265292]], [[0.07975589571274562, -0.95750493106225232, 1.0567952041084252, -0.64101138502943655, -0.76141882502972247], [-0.73125529928579047, 0.053425180486848779, -1.0001769287782527, -0.29618662593690148, 0.76115710745054621], [-0.76763046229817755, -0.030814698021062563, 1.2939470006845297, -0.88134308600063316, 0.79581514267491282], [-0.15173899475292943, 0.52338997535009912, -0.79668553511707618, -1.2961646575640522, 0.98159075910895965]], [[2.4513861144789253, 3.6535905216201909, 4.5951064952084293, -3.7163075226963751, 3.7995724398574642], [4.6388398453257027, -5.8678728623451404, 2.2185730910230115, 3.1587358284194718, -2.9869026394353297], [1.8870264077163477, -6.4750686006140841, -2.1413603961664989, 1.0607089328097663, 6.4065186502201952], [5.2673748136699725, -4.6720720619066336, -5.4972053757791457, 6.5238883078922667, -4.282084715499983]]], [[[-0.93379809553865589, 0.72667036475369262, -0.89505351589828352, 0.13853552327799848, 1.191280905795798], [-0.58316917463858731, 0.033545080435915982, 1.0158173884109429, 1.3093528937631225, 1.0219229691782283], [-1.3057272833004514, 0.38796146447285834, -0.31973437786050496, 0.39664879202005127, -0.82114363721645633], [1.0890334379628981, -1.3005421855230392, -1.2864256430844185, -0.32527021515202603, 1.0619492190973274]], [[4.1363667639024282, 11.725302349332063, 3.4673359817207796, 5.6032675648041241, 10.761803846310507], [9.2897639648169275, 10.210816942866973, 7.2698268470734968, -9.7747348855967786, -10.39136107617424], [-7.5160290735921667, 4.3937460761670151, 1.0635665837003159, 9.580290058204481, -12.25213099568607], [-10.727039421864738, 1.4726560735617158, -0.76700376577257212, 1.4939101091192155, -10.982710218227327]], [[-4.3744212756367258, 2.2268905712366913, 3.1208708926479214, -0.84682353444506431, -3.8384806774314959], [-1.4805842808606038, 2.0087174642823675, -3.3038482919727521, 2.0633082780006742, -0.80860595231713839], [-0.016078525167781674, 0.76599785257703235, 4.2011831900540244, 1.6407179805598564, 0.85500247079647806], [-3.166997435149661, 2.5602352900702323, 1.3496419203012888, -2.2124722925860572, 1.3051415536692608]], [[-1.466422619136452, 2.1895864516045997, -2.0441015003110192, 1.2137759035751043, -1.023727094858558], [2.320642985421614, 0.18687945117101931, -2.1262086260616457, 1.8363239525185031, -0.29142880304605873], [2.0364255921599921, -1.5993158748392371, 0.96578974698117437, 2.1975003328427105, 2.7861193375811841], [1.1329337814495146, 1.6598229463414551, -2.0514288705715935, 2.5232887981409688, 0.15874598033433501]], [[-0.26630789323130111, 1.9589862321159071, 1.4466069945404632, 0.14512326447534876, -1.8583157356642606], [-3.0678381087778308, 0.41150599693907092, 3.4448345942500005, -0.91792009714514711, 2.1250136460379969], [1.684582491126609, 1.4023100014086771, 2.5700772750874834, 2.1138248544644909, 2.6910496848591374], [-2.1006653167800611, 1.0734625047021644, 2.0747389336507225, -0.50759970434626056, 1.4458665858090656]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-2.6865304440509172, -0.39263196366632336, -0.59127657012447798, 1.5547655293753522, -2.1043667424779535], [-0.56452973896440106, 3.4769813466247399, -0.97414830390188989, 3.2992221870487022, -2.2766979461088779], [-2.5815189555515463, -1.7285959828180419, 3.7277110684587988, -4.0840731170209317, -2.7239962172619969], [-1.8742365241392047, 4.0136480575064777, -0.1171824923566831, 4.1956454327866801, 4.7775103781607626]])+(1.-msk_arg0)*numpy.array([[-4.850498384676543, 0.87086125771472211, -2.6193266907883386, 0.41675575229381678, 3.8526210381142594], [-0.62101328530417987, -4.1518707624737674, -3.9652246226686048, 1.4399597477210344, -0.41377544798233501], [1.4755064002667684, 1.013736830676879, 2.2661557486541248, -2.1370053518807053, -3.5564785684698306], [-2.5559120503197077, 4.1867400123883609, 4.3945421463127445, -4.8751193979142631, 4.3176650717990874]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-1.7961613725823957, 4.4790595663515038, -2.9729608857320255, -3.5880854738955037, -4.6584297317201102], [2.5779203180678039, 2.6137656594841641, -4.2701300395357444, 4.6852073257623985, -3.6731208738351553], [-4.095181321075323, -4.3019367383287674, -2.9968901032974893, 3.458467318242338, -2.579105188881019], [-3.2788745080955159, -3.8182345283060783, -2.3669486511674354, -3.2627405326163341, 3.3401657487534049]], [[4.2320148678464662, -4.4921625195923331, 0.69159966275378437, 2.3808710422641264, -3.0727903246843411], [-3.1457605528263741, -1.0761719268894132, 1.9628959533757264, -1.2136647022317328, 0.96660000938896928], [2.1257219400927845, -4.4696711520225385, 0.091710483449461755, -1.868233462137221, 0.39810733813680965], [-2.8039376153479645, -2.7828682845372765, 2.6311668410799935, -3.8240003340405035, 2.7517505269060294]], [[-3.7779442198417779, 3.8574758206786104, 4.143312137082674, 0.098877633367340678, 4.2492492762041802], [-3.6572280327435838, -2.6092965984165719, -3.3864154010204195, 2.1609389515764708, 2.383557834130726], [-0.0075980990089261624, -4.4287818129730683, 4.0575189073928026, -0.7545577326555275, 4.8997178984580412], [-0.99291973142616108, -1.6554361738803047, 1.4862153541749183, 1.1180017491617411, -0.76198759755226408]], [[4.2113328630423279, 3.7572382984732258, -2.0424473617031813, 3.3429384850535033, 3.7352778044937622], [-0.022887437780517494, -4.4130346194952583, 2.4641021761785726, -3.9052753893857073, -0.42679856014853357], [-2.1739694498651163, 1.3202419336471083, -2.1750060863706788, -2.7607179121522254, -2.3858202986594068], [-3.6094480820606933, 4.1759690658037041, -2.5836892667931055, -3.3083039175570472, 0.2392584832190483]], [[4.0077442750429615, 0.50901343152012668, 2.8309425524054053, 4.7239027580405253, 2.8814682986563591], [-4.7973573522115087, 3.6432061485002354, 4.4937447901265664, -2.765744952361131, 0.35328079630792075], [-3.2764433960193271, -2.1122534490003844, -0.15108131445767725, -3.5336437441402522, -0.75948882445520738], [2.8432876259678386, 2.9045056892925807, 3.1741646271027566, -2.9687106726977195, 2.0230957091558324]]], [[[-4.7493165451350929, 2.2335282078662697, -3.8407337928908634, -4.3879593207323264, 4.592136407479039], [2.4668101188876239, 3.740542200103242, -1.5413920305409001, 0.082482743173190265, -0.59175318240345121], [2.8701379457194323, 4.8422117481205635, 4.4013651380026246, 4.4996923408613281, -1.7290863535393108], [-0.047323571881273452, 2.053964493937074, 1.0649093839786383, 2.9966349773357486, -1.5293119981893977]], [[3.2526777028465528, 3.5058665444295336, -1.3857403569922369, 3.4261150875540718, 3.9548873130123301], [-3.9957047645268409, -2.7102957918073045, -0.088996988924505693, 0.64131233514981378, -1.279464987676354], [4.7023982769324277, 4.9613362458166019, 1.1854600658953647, -0.43314353976556141, 0.00018464290796327276], [2.1950790987120516, 0.83187641771678678, -1.7429294544186003, 4.0753507054422329, 3.9345340492381737]], [[4.2748467319105039, 1.6449735377490278, 2.2920390497228036, 1.7404090821177265, -1.87328244272042], [-0.5038993477236744, -3.8161838985621745, 4.6120574713205222, -2.9742921695227089, -3.5471266290640768], [0.64485700763256304, 1.4394964999952098, -1.3741059430603042, 0.42313557256894807, -4.2172124984634793], [2.3728573037325891, -1.0797309811451683, 1.9894056102378679, -0.444382805469826, -1.4556482945020512]], [[-1.308434164303204, -1.1682957304275501, 0.95284450699792878, 3.7319441822410937, 4.3894953185928891], [-3.4007772069208664, -2.5289862806783256, 0.45334084339161418, 0.53980181221754009, 0.26243205313171725], [2.6459975264763411, -0.65342483431843235, -0.54569135021761461, 3.8882826327836337, 4.1756872679948298], [-3.6902097564377434, -3.5433428881280649, 1.7443104837144396, -3.6860491914169979, -0.3252334269217263]], [[3.567820014494492, 1.0895032441126995, 2.0876871687293095, 1.3137277607021716, -2.8386627568295877], [-1.5935481847369992, 2.2015273910996118, 1.5991553064753354, -2.9565972265823803, 3.9668421841813988], [4.5260716757551727, -2.0007010385799839, 4.7742287337120555, -4.0845238919847375, 4.2676930121364531], [3.692565301532877, 2.6913187533174572, 1.3811524505273853, -1.8374452654184346, -4.4907700370241432]]], [[[-1.2973457254342344, 0.562375209663033, -2.1738571717392041, 1.7139363802749727, 2.6999511463338681], [-3.8068883396964992, -1.5364611895560563, -4.6613903034628166, 0.15467795850534838, 4.9589661545963395], [0.79881538662313645, -4.0151147169770169, 1.3307109121765244, 4.9023631439744157, 4.7790480978270882], [0.84374803294686807, 1.6848036867035443, 4.1897773062465209, -0.13933951289470858, 1.3952576023759971]], [[1.4559161233625106, -0.79807851464535062, 3.9921044379722268, -4.2808331781468318, -2.6727941018104584], [-1.2930847012190805, 3.4374428932522303, -0.26724042330401865, -4.8855822461758001, 4.5283991885247126], [-0.59383208132480458, 0.087113797163594242, -3.6078086204265949, -2.5463040988353414, 0.6221667967375204], [-3.5008888282101234, -1.7925966126324622, -2.1160476332002909, -2.6376537208027893, -4.0146601541635434]], [[2.9922812516405539, 3.1559551314455181, -0.51349347465167838, -0.59718909332725012, -2.9778637571521491], [2.987634102793665, 2.1967370592895694, 2.6448581110135896, 3.1392321892281281, -1.2580392358554549], [-2.5072221866724465, -4.4818282651646744, -1.8797475191551563, 4.218377808280616, -2.6931539219956901], [3.9756838928603759, 1.4835521219167163, 4.4566167036105426, -4.7305154964929894, -2.4093110807828735]], [[0.6445826369375629, -2.729031640746018, -0.6619782804529839, -4.1671811643699916, 0.78482794599610184], [-3.9240767316662026, -4.8038663038770171, -3.585055172905137, -1.0675308397878891, -0.2181906842209802], [2.5853349216080552, -2.9842019111243956, 2.5021030261328754, 4.7598691012646164, -4.520624052025692], [-4.2099492395492719, -1.9737151781460471, -3.3445177223510081, 4.0178174348635309, 0.23992654337746]], [[-0.38319464368352296, -1.194343844404723, -0.68609373933455053, 3.8241909061116584, -1.7266369436443108], [-1.7475468865768806, 3.7483940580369595, -4.562804907214657, -2.7326063273315935, -4.1225907359043745], [-3.9070072798260593, 1.0723844346827693, -1.1927125289188889, -3.52038681069308, -3.2975455312308433], [-4.6182684165134971, -4.7553441132590386, -3.5586405111303501, 2.3602703019640936, -0.70132048904084066]]], [[[-4.7531722671578489, 4.3332704841268139, -0.47127760596694568, 4.9413824748292345, 0.7395937714991323], [-0.82584771776210886, -4.795484233264097, 1.6451642293392954, -1.241986052016717, -3.4145587418698717], [4.3679028489229932, 0.35784526136703931, 2.5231192879998225, 2.0199438943406882, -1.5205585533954027], [-2.2242570996596589, -4.9085848025958336, 2.1999156437199794, -3.2126501764850168, -1.0561149332533537]], [[3.6976419075761253, 0.73381909302327486, -1.4932796721131503, -0.42398606050366361, -2.3455977397755481], [0.30564798307954444, 0.74848119975538729, -2.7680757512295893, -2.6827114253148663, -0.71802477105387297], [-4.9702049307729466, -1.881635229872467, -0.10311015030099746, 4.2745859343004202, 4.3363996027138985], [0.49749512313814837, -4.610758665833159, -4.9792239746818368, -0.99110903856597599, 0.50023469562645495]], [[1.610085443747443, -3.5394939437927455, 3.1234229620314231, -3.4778900866696572, -1.7531228214862606], [2.1656179680577932, 1.8586728193768138, -0.80390468671544468, 3.3926697011420881, -3.4784816066133537], [0.46795483502440227, 0.89109632376978531, 2.9763538834496126, -3.6733858220063542, -1.5960832564056702], [4.0334859454809404, -1.5532326908679392, 1.7972246259248479, 3.4140870797569995, 0.43611067245198498]], [[4.9300628645326849, 1.6896705239184246, 1.0905614902998586, -4.422468143276058, 2.4995321767063192], [-3.9694982231365641, -3.9168476012398168, 4.4757916407273814, -0.77478894874742288, -3.9265389983496988], [2.8606091527288759, 0.14098458616937748, -2.3038084693061291, 0.76406429191735104, -1.615072497539296], [-2.8852693011059505, 0.41686620077587122, -0.35655179307736518, 3.6710948248397681, 4.5034239578133608]], [[4.5288908639995089, 0.10746854034619524, -4.4903891249632899, 3.0815937569228851, -2.1467506606548059], [3.7707782019897671, 4.0154139018170678, -3.9703365745469767, 2.6780831533003351, -1.9953363933485169], [2.9502198069138323, 1.7582905656835539, -3.6687047272721918, -2.8851231345649078, -1.5886146412756963], [1.2872889601033002, -4.575213421194225, 1.0873883764833021, 1.4816422291534517, 2.4215019065982082]]]])+(1.-msk_arg1)*numpy.array([[[[-1.3974775202820213, 1.8257805360804999, 1.5124527536551806, -4.0106773846616228, -3.1259948975253691], [-4.7574689489284143, 4.5333206887649879, -1.512354098936044, 3.1446968704594056, -3.4637759690125511], [-1.0355634545453221, 3.2871603494627912, 3.9981747855300931, -2.9515862138396329, -0.6327079741540107], [-0.60690905157531994, -4.056242339140292, -2.829205363466718, 3.3449801432289608, -1.537037918275781]], [[-0.4948755548905126, 0.69668787921923681, -2.5036476994491288, 2.2802312754328931, -2.5750558290541847], [-2.0217188685754572, 1.2728328447243262, 2.3861615227397595, -4.5967104302086863, 3.3336790544055201], [2.2476609978223072, -0.30736690334563477, -3.5231245396796105, 1.6076971501454951, -1.9105889650659504], [0.93288818333232548, 2.646281235766784, 3.9251365999777903, 4.1443667280801684, -0.50405585669992536]], [[-1.6647069561364649, 1.5241453653321404, 4.7518849351745356, -2.5641572933864065, -2.7627421644684347], [3.5245158347114582, -2.518868670824074, 1.4429987316018922, 0.81089493960463255, -0.024171502752125207], [4.0722771309528518, -1.7303959651827086, -3.1998116827585807, -0.36289971394258824, -3.0641379581546166], [0.35271892289023743, 0.26129708869225077, -4.9669234571120624, 0.63016340090502432, -1.4424545364276575]], [[4.7744636078568305, -3.2668516435695691, -0.21302238822263675, 1.440447158122435, 4.1279165565107672], [1.5843978496402933, 3.6983177257590256, 1.2067212396338389, 3.5543366159332308, 0.17596126368028653], [-1.7000629320196037, -1.8739747803920634, -3.5864740189404056, 2.360684287707981, -2.3081277992302738], [3.0773591821856066, -1.8632375382224211, 3.4985421737314777, 0.36273758782573573, -2.7076715602261747]], [[-1.8214259560314447, -2.6508762991743682, 4.0917126713202769, -4.1378814959657175, -4.4954801107558682], [0.088235159297390453, 0.49573734051000429, -4.2073404571971924, -4.0179254588916971, -2.6564006619972025], [-4.0369947944752251, 0.4717598372498033, -3.4146901288852085, 4.966849867866431, 3.3923957353920731], [-0.3783693713932248, 3.6413508194003956, 3.1031886605355101, 4.0545508116751474, 3.3361199213459152]]], [[[-0.47966012092284149, -3.6663467617360199, 3.370053453830943, -2.8891378164955883, -0.053411090925265192], [-2.1853925016557141, 0.71468234327118463, 0.069259344061397776, 2.0856111012378697, 1.5581853338200666], [-0.3096587685657024, -4.6575538049878791, -4.2229450461028604, -3.3932188087314765, -1.9877321191637343], [-4.6382249926915939, -2.3851995212500254, 2.9468567141561719, 2.5990747014523361, 0.27433908828970122]], [[1.3577109125498623, -4.5723956718980858, 0.81577170252792097, 2.1007871079848321, 4.6822878480078742], [-2.6211885890397966, -3.8049728489941446, -3.4966412095857615, -2.6615940988133815, -2.4183078015216597], [4.6605768748597391, -0.29782208988509318, -2.3590660871684452, -1.9269701310333334, -0.0065048338048612209], [3.1229476856500646, -1.4382090401063086, 1.8677849892963234, 2.761744418936444, 1.5317362412862092]], [[-0.74940983508424885, 4.059046742519719, -4.0907045839696305, 3.4361072510170168, -2.6347068187916567], [-2.5320559323438339, -2.9875857008692543, -4.302129779444491, -4.9885958510180437, 4.2891555336647187], [-3.172163710645898, 4.1332494305097143, -2.2343821030392572, 1.0745286126535447, 3.6622234382813907], [2.7336341372866633, 1.097625384364683, 3.2881554072039414, 2.2511656617162075, 2.6938356890620518]], [[-0.54325254561834324, 0.75361525202975344, 4.2109215333944512, 1.3035699996030861, -1.0751007372938428], [-2.7353640672265001, -2.4779261916700799, 2.9865037292773779, 1.0315160222806004, -4.4585132516391912], [0.52091910125351326, -1.8413781184098408, 2.6920980582052803, -4.6181532042482862, -4.927005021637755], [0.84226985541570709, 2.397899426564174, -1.4138925142127388, -1.5077503820248674, 2.8701513751323136]], [[0.31841609579653074, 3.3154972676376921, -1.6737494957808794, -0.41720709557621483, -1.4222599154324032], [1.1106258037639947, 3.7613819536637614, 0.55981182991551215, 2.200043184442765, 0.40671081451440116], [2.5064321975116073, -1.4126355532027435, -1.2984999515845006, -1.7969231449090439, 2.8626644720343384], [-2.9556846149238849, -3.6373248147886605, -3.1163201538403431, -3.8066402554019829, -4.7386297824602792]]], [[[2.6365432692609172, -0.42223660552414977, -3.4584969281524893, 2.0294677374597843, 2.8957565451903271], [2.7928637143507142, -4.2283491623006952, -1.1889944505454642, -3.1978757614573983, -4.760841402289433], [1.6467956859380797, -3.8576138201080887, 2.8072590087154028, -4.7918053934943448, 0.49801354351024152], [3.4820281239200099, -0.028087694237277638, 1.1364093664491328, 3.3842219618755927, 2.0515655744790608]], [[-1.1885788936767128, 3.187848319665159, -0.62575673828504019, 1.6000665647483654, 2.3786897078322431], [-2.9755944112490775, -1.950739477255957, 1.2250344932156496, 0.13779905837396988, -3.4375662542955174], [3.4229572401511117, -2.8071452532513472, 2.5952102649346358, -2.6715698971849844, -2.0994079593378023], [2.2953258939730414, 4.4214143534993706, -1.5908140884257573, -4.3670059485131709, 0.66831192560514285]], [[-4.9520804081319101, -1.1650563030478067, -1.4584207000584595, 0.16671358676412229, 4.8948658066912625], [-3.7494574422683256, 0.78654166672870218, -2.6564749758970221, -1.4356552598047392, 0.77296251702537333], [2.5471307091687336, 4.5302792177572258, -0.5608569757057138, 1.4082305428396573, -2.003415158959081], [3.7198779481659585, 2.3101594915579167, -4.9275776159804705, 1.0391300125619471, 4.2981194784359218]], [[-1.0470240402249997, 4.4532288171485455, -3.0745180963916638, -4.3888890880337392, 0.21039744406968985], [2.1028068379330911, 3.6933515835375488, 1.1546558146217745, 2.4366510977955365, 4.8512446597337835], [1.9268805509688045, 4.3761975457375026, -0.85810815061914258, 4.6748553655731513, 0.52257471487873275], [3.5650328983149588, 2.923475715016326, -2.8320306321987654, -3.9265788226564458, 4.9900727711503414]], [[-1.4621478237541554, -1.5543050387076809, -3.7659890121354103, 2.1210825115887051, -0.22479841383953136], [-3.1744678693571835, 1.7522581510864219, -2.8612224695063593, -0.74148242985790613, 0.69681306564404633], [-0.68347585249113507, -3.2896014410636445, -2.3278315062905728, -3.3633330145587093, 2.942480886000598], [4.696132132360809, 4.7079303839872431, 2.9842036247177841, -4.3068107971813081, -1.1290197950789715]]], [[[3.9115662452858846, 4.5500920809072163, -3.8757060995017625, -3.3930567694090454, -1.7756900655451702], [2.9005194966450389, 0.56489190336689621, 3.2126303111890326, 2.1482968554192645, 3.6796513288136303], [-2.0824872978282585, -1.558014131882536, -3.2388055222456491, -1.0374041253870714, 0.76273487025524744], [-2.4343925620573192, -2.4562197792752238, -3.6658744068426721, -1.1225674396692287, -1.8207064239256132]], [[-3.0963350228165831, -0.53100146879534904, -0.64109976916602385, 2.8759492508722504, 4.0037782736954224], [0.25900129050199361, -0.15282026697346129, -4.3496105685954127, 4.0504949838844677, 1.2758934434257778], [-1.4273100772300475, 2.0603887081680288, 1.6384164582447696, 3.7617185758732514, 1.7255555943534127], [1.5359440739303327, 3.5844316798826679, -1.9741717878602261, -4.7552240069267562, 1.4099172215700797]], [[4.1910104759387359, -4.0192991028001366, -2.093759017375072, 0.23426455060744011, 2.3241580935076298], [1.2665651741045538, 2.0215072976245647, 3.5265803404086924, -4.091499422339913, 0.44155458673435621], [3.7922382645492263, -1.7255262349415847, 4.8941531433759859, 1.5691619481065624, -0.25110802657906994], [4.4549712654933344, 1.8351078275537507, -0.65034080121560933, 3.8493693320215492, 3.8069696395299033]], [[-1.724957074420713, 4.6265390610141051, -0.98683217460113415, 1.0590865881507208, 4.506824343546441], [-2.5603671733605071, -3.7991813483894243, 1.1894318888986124, 1.8703027437581143, -2.4201624114639078], [-4.8186868872508146, -0.42948068666526229, 1.1906717837344907, 1.8189930824321774, 1.1561308770359897], [1.0719635844093522, 2.6841610010343331, 2.8978431552144048, -4.2755310755903055, -2.9554933197757807]], [[1.4809029179994502, 3.4647897785806805, -1.1580525975254288, 0.23077554525088306, 0.51400850188578229], [-2.5433844036233499, 4.9389611692880049, 2.2930019491732567, 1.4442575384426668, 2.5688900050114913], [2.3208666130138154, 4.6565111803939896, 1.2289145969755664, -3.1373075834868214, 4.8779973095508868], [2.4149452125434943, 0.69661602938872491, -3.545876138146653, 4.6647747807535485, 1.0222600657139269]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.4957066135925252, -0.59979788262545419, 0.90365482335951375, 0.74873646784512393, 0.576703009118681], [-1.0421309088655302, -1.0278390621220088, 0.62914487830047472, -0.57340695027060573, 0.73140267808444714], [0.65602234270434734, 0.62449324745174906, 0.8964394260219678, -0.77679798501501107, 1.0416521418486642], [0.81934530809821915, 0.70360540300356289, 1.1350184731408755, 0.82339690122298381, -0.80431051813927701]], [[-0.092776603090272441, 0.087403775342917683, -0.567715666752868, -0.16491105847250923, 0.12777701117848231], [0.1248130482510994, 0.36484129891883899, -0.20002688527177781, 0.32350941981285009, -0.40619900667549491], [-0.1847052317901868, 0.087843590794964038, -4.2812113609965676, 0.2101621513711463, -0.9862464869496963], [0.14002878006884556, 0.14108894978894357, -0.14922351465373551, 0.10267571374698646, -0.14268443299174538]], [[0.15650749077212181, -0.15328069380366463, -0.14270625783477628, -5.9798819003669328, -0.13914847816427953], [0.16167342173654767, 0.22660381747452124, 0.1746024926375867, -0.27362021018369065, -0.24806470464355826], [77.819013601935538, 0.13350772178310369, -0.14572367587669688, 0.78360679976545811, -0.12067563528719782], [0.5954928192183363, 0.3571726771794162, -0.39784043978789929, -0.52886909217074762, 0.77596613386338331]], [[0.36918609379458239, 0.4138054086181171, -0.76122673148298181, 0.46508948230032066, 0.41623825877284854], [-67.930956024217679, -0.35231210797824625, 0.6309663391420498, -0.3981193064133472, -3.6428556104647258], [-0.7151735869479755, 1.1776368328798519, -0.71483272581076296, -0.56317435495004053, -0.65166916814689502], [-0.43074882752925231, 0.37231251115030067, -0.6017618098886709, -0.46995849478165191, 6.4982670978145327]], [[-0.52507510411337199, -4.134206706871046, -0.74334491199403097, -0.44547207050274779, -0.73031056543611073], [0.43865123816716978, -0.57761396327908554, -0.46828799603875237, 0.76086796820561653, -5.9566406226161819], [0.64227166110503453, 0.99626621202765075, 13.92870289771972, 0.59552317518356768, 2.7707672248995197], [-0.74011743421900178, -0.72451803080836508, -0.66296710778946921, 0.70884871396567539, -1.0401716206278904]]], [[[0.11886546908368753, -0.25275245549896441, 0.14698486523833976, 0.12865427815091598, -0.12293400911283316], [-0.22885009861195496, -0.15092190082732379, 0.36624669634907753, -6.8442163444909854, 0.95399527328525713], [-0.19669080359233235, -0.11658509960526103, -0.12826241887774603, -0.12545963061473203, 0.32649019397374274], [11.929144748006495, -0.27484883045972275, -0.53011997777242359, -0.18838788949407304, 0.36913967825582106]], [[1.0689596892990318, 0.99176089636079012, -2.5091145892377433, 1.0148466288407672, 0.87916066159073891], [-0.87017974338163429, -1.2828789230810069, -39.06852792035685, 5.4216660994248604, -2.7175275448055145], [0.739405967308435, 0.70081550097648193, 2.9330227534898992, -8.02731895414313, 18830.841568614942], [1.5839890911743617, 4.1796849538875644, -1.994906528092715, 0.85317352982200301, 0.88370854162463586]], [[-0.2278791182453753, -0.59219694514655152, -0.42501383386975983, -0.55972375340431346, 0.52002211822751687], [1.9332200136843363, 0.25526765213513958, -0.21121772873809661, 0.32752273427738393, 0.27463025873393271], [-1.5106423476395805, -0.67672849771092292, 0.70893245809878447, -2.3022132079031401, 0.23099341194137502], [-0.41053808940365688, 0.90221390412332447, -0.48966801887394579, 2.1921377063002394, 0.66921955501285901]], [[-2.5215041589850844, -2.8239615202918849, 3.4624979866267664, 0.8840491781062666, 0.7516176570627513], [-0.97013770273880595, -1.3045631019254813, 7.2775754383081264, 6.1119138772344765, 12.571719603903681], [1.246872740445172, -5.0491227357313733, -6.0459492087111428, 0.84850369652444191, 0.7901027963315348], [-0.89404733194178321, -0.93110440937079875, 1.8914191125097981, -0.89505647258614285, -10.144166970397922]], [[-0.63812017894951256, -2.0896660550680961, -1.0905359673665147, -1.7330058892049374, 0.80203185131144272], [1.4286972731135998, -1.0341447284794945, -1.423687828749354, 0.7700399383586588, -0.57393206999453628], [-0.50301853554473641, 1.137950099593483, -0.47687240664288427, 0.55739616325333641, -0.53347275439784703], [-0.61656267667460429, -0.84594139705767057, -1.6484045227878599, 1.2390561988198401, 0.5069727301417456]]], [[[1.9898465805539125, -4.590385406743577, 1.1875292402426745, -1.5061929866599741, -0.95613543195285766], [0.67811785510823663, 1.6801719256556347, 0.55380879683767481, -16.689636846107479, -0.52057603844680456], [-3.2316840646554175, 0.6429502361753624, -1.9399547504492825, -0.52658664397894284, -0.5401742988787448], [-3.0595851542733112, -1.5322372427867239, -0.61614705671892656, 18.526826324577883, -1.8502095606972173]], [[-1.1872909126287876, 2.1659472734787175, -0.43300369759266, 0.40379895942741439, 0.6467374279399789], [1.3368002739405815, -0.50287264006954435, 6.4683177845873736, 0.3538157574096934, -0.38172341060355891], [2.9109171383291477, -19.842964479803896, 0.47912629650894539, 0.67886470575477909, -2.7783481726802943], [0.4937591759238496, 0.9642972493848273, 0.81689842690531944, 0.65535364600169388, 0.43057093662718654]], [[1.2457756323591029, 1.18116732120694, -7.2595101057271334, -6.2420950250276466, -1.2518071249920979], [1.2477133879858666, 1.6969309334019023, 1.4094181661148648, 1.1874594944744643, -2.9631119302284641], [-1.486789279495875, -0.83173893507537888, -1.9830913622560338, 0.88368354800780402, -1.3841433413863244], [0.93762763059535681, 2.5126930246594097, 0.83644417197439969, -0.78801371039210655, -1.5472103615808419]], [[-6.3359961671082567, 1.4965283128430473, 6.1694971536320633, 0.98005653124475467, -5.2037814630026142], [1.04077300121672, 0.85016377614939709, 1.1391939370660835, 3.8257190938225341, 18.717907831869873], [-1.5797075585397171, 1.3685646074404274, -1.6322561758510279, -0.85802214937714627, 0.90343126745761093], [0.9701003229811348, 2.0692312458463173, 1.2211246750847107, -1.0164904660880021, -17.022181287360688]], [[7.1086489912205586, 2.2807470646107557, 3.9702974405567804, -0.71230654644061375, 1.577631144340407], [1.5587542961996281, -0.72671020578038115, 0.59700036987223448, 0.99684912166693096, 0.66074863884455726], [0.69720786836703041, -2.540130320026329, 2.2838665237558211, 0.77377753177234154, 0.8260678105770497], [0.58983064031571453, 0.57282841207365898, 0.76545978969838668, -1.1541035003470703, 3.8840961583590179]]], [[[0.39431277025015155, -0.43252239411426385, 3.9769267633536982, -0.37929395947112454, -2.5341431963930536], [2.2694698838885592, 0.39083363284535, -1.1392397735829118, 1.5090640680673102, 0.54889567461734168], [-0.42909299702060472, -5.2375613889066255, -0.74282517400316295, -0.92786563497643948, 1.2325974030753635], [0.84263483948235485, 0.38182828646416417, -0.85195836008054249, 0.58339265751922609, 1.7746520431877939]], [[1.0854615340882212, 5.4695334254258423, -2.6878073360676886, -9.4664623000543102, -1.7111408275361599], [13.131603281222803, 5.3623899422165664, -1.4499776806048754, -1.4961162127362995, -5.5898462271928162], [-0.80754176405404099, -2.1330638339390116, -38.925828793672615, 0.93895598759634069, 0.92557153980794815], [8.067713372120707, -0.87049623465408943, -0.80607903519000512, -4.0496533694352923, 8.023529940241545]], [[-0.072780294245715985, 0.033107131758817528, -0.037517330755764675, 0.033693558288638785, 0.066842146437485914], [-0.054110417481332926, -0.063046325924092816, 0.14576664907311551, -0.034539905938157048, 0.033687828658887711], [-0.25041410748661763, -0.13150373223507675, -0.039371155764874259, 0.031900404159745897, 0.073418784318666705], [-0.029052411224581725, 0.075444260892553114, -0.065201917816133437, -0.034323228909856528, -0.26869897885744742]], [[0.85103284645121469, 2.4831145323271562, 3.8472341725848525, -0.94871128448166664, 1.6785722832003553], [-1.0569712333745351, -1.0711791368800292, 0.93740856804156913, -5.4152107352197643, -1.0685352761172333], [1.466696500213686, 29.759603845955709, -1.8211780574147913, 5.4912204079817455, -2.5978062527713845], [-1.4541607714671383, 10.06472922241656, -11.767281820614214, 1.1428866953797105, 0.9316567731775085]], [[1.0548963359081025, 44.454966660667992, -1.0639412855338723, 1.5503375055287396, -2.2254612357745902], [1.2669826020633519, 1.1897927573540621, -1.2033011026794085, 1.7839290659340428, -2.3943383151265438], [1.6193743825340334, 2.7171336020354833, -1.3022335492540458, -1.6559121241392829, -3.0073437912697978], [3.711296007523738, -1.0442158514462772, 4.3935639569843481, 3.2244696352171549, 1.9729533828335239]]]])+(1.-msk_ref)*numpy.array([[[[3.470895462917841, -2.6566710997420127, -3.2070412599363705, 1.2093962987964875, 1.5516654836885186], [1.0195543968330225, -1.0699658633675802, 3.207250463425805, -1.5424375017640217, 1.4003499152571686], [4.6839219396808662, -1.4755892226155753, -1.2131781737584655, 1.643353110247344, 7.6662513873989173], [7.9921338659991559, 1.1958107970700269, 1.7144384240573716, -1.4500828635694927, 3.1557441277165998]], [[-1.7597580828323465, 1.2500020219824661, -0.34783698118003403, 0.38191795152419022, -0.33819121429867738], [-0.43075289608804412, 0.68419137777933103, 0.36496324721337831, -0.18945314718795236, 0.26123128336660434], [0.38745222636263832, -2.8332954792319871, -0.24718435238565747, 0.54168240432341996, -0.45580775019532338], [0.93351086794127891, 0.32908870226802717, 0.22186775811054568, 0.21013132158749351, -1.7277078445557343]], [[1.5734461138237825, -1.7185543783204282, -0.55121845888975241, 1.0215156057486128, 0.94808944695434871], [-0.74317347789778765, 1.0398821983566926, -1.8151968074709179, -3.230167760160696, 108.36424684261893], [-0.64320934125017548, 1.5137152094039759, 0.81858776405560163, 7.2177700619591105, 0.85483314607865546], [-7.4261019775325368, -10.024324051590627, 0.52735394724832418, -4.1565833354119412, 1.8158816272125218]], [[0.087288496996396805, -0.12757106773249213, -1.9563941413437331, 0.28932387414824795, 0.10096031414115862], [0.26303731249599527, 0.11268792548327733, 0.34536207585131667, 0.11725275271497967, 2.368451689748277], [-0.24514136767791789, -0.22239133453366181, -0.11620208318613273, 0.17654023219616985, -0.18056008529198367], [0.13542642493809509, -0.22367290468579384, 0.11912268927983599, 1.1489180230586751, -0.15391665607294142]], [[-2.1151675286918707, -1.4533386711836316, 0.9415668566168236, -0.93106123069750146, -0.85699879505561449], [43.663105147566732, 7.771496563383268, -0.91569034579168884, -0.95885826592138046, -1.4503162468035542], [-0.95432895860720757, 8.1664879752666266, -1.1282490922161719, 0.77566689966596436, 1.1356638018143823], [-10.182169407444922, 1.0580197375073717, 1.2415039688400451, 0.95019675842280038, 1.1548209084042664]]], [[[1.2946944267732372, 0.16938203766905269, -0.18427401636559701, 0.21494761577606042, 11.627047389335766], [0.28416556057261244, -0.86893609608673061, -8.966491001613548, -0.29776082652014596, -0.39854905050459943], [2.0054761832859751, 0.13333464546112656, 0.14705691845961416, 0.18301598579678388, 0.31242302688425067], [0.13389028912627232, 0.26036114789203119, -0.21073752324669986, -0.23893629719729259, -2.2636704422097949]], [[-3.0579932179202314, 0.90802963269148784, -5.0895008365795373, -1.9763405566861201, -0.88671839435079203], [1.5839649157005891, 1.0911696159859134, 1.1873882716624591, 1.5599188337262981, 1.7168495920417188], [-0.89084911030433733, 13.940775058276092, 1.759963735249656, 2.1546108554611254, 638.27468725964582], [-1.3294717620636429, 2.8868340044412961, -2.2228847465135479, -1.5033508292822639, -2.7105650767833329]], [[5.291129682362433, -0.97688567641552393, 0.96932558713901074, -1.1539874436384312, 1.5049965310702604], [1.5660098862816738, 1.3272337665543454, 0.92168875090992985, 0.79485786002476122, -0.92447676274417068], [1.2500062999148389, -0.95934801161506755, 1.7746403434197833, -3.6901991961633263, -1.0827369464189238], [-1.4505323037136153, -3.6125482146750074, -1.2059115618383756, -1.7614095177898461, -1.4719623170666467]], [[-2.6506267836849937, 1.9107359409761302, 0.34195834244393375, 1.1046278666734251, -1.3393719283883903], [-0.52642343480846765, -0.58111486635948839, 0.48215568378662321, 1.395964499453336, -0.32296859209549889], [2.764267511511842, -0.7820011182518779, 0.53488384025691882, -0.31180423949478348, -0.29225863204872204], [1.7096180499186144, 0.60050881691242497, -1.0184365029493136, -0.95503855604215326, 0.50170167336719396]], [[-1.2994803134786859, -0.12480041893599629, 0.247214681184588, 0.99177471421203833, 0.29092815138260908], [-0.37256062895353126, -0.1100062299121998, -0.73913309056863408, -0.18807605728300172, -1.0173701638014441], [-0.16508543434493556, 0.29291026057231717, 0.31865649858317174, 0.23026886216843701, -0.14454206981801382], [0.13999309868620424, 0.1137581791705827, 0.13277693804098595, 0.10869833244555969, 0.08731964027109633]]], [[[0.55963670972879054, -3.4945013789960875, -0.42663227145179972, 0.72704107241123705, 0.50954090139845953], [0.52831306901410857, -0.34895566653344395, -1.2409699638125844, -0.46140204008248331, -0.30992555214236178], [0.89598631625407854, -0.38249199351567681, 0.525603941669051, -0.30792285560469718, 2.9627836822803695], [0.42374913348076798, -52.532129828887648, 1.2983933816712465, 0.43599575231437182, 0.71920996268492809]], [[-0.8528982266722045, 0.31800033408846712, -1.6200174423293368, 0.63355916123171063, 0.42617447216380389], [-0.34068380651761559, -0.51966797334868642, 0.82751696894335969, 7.3566310440650513, -0.29489957594566585], [0.29615819291745699, -0.36112731591025748, 0.39061838047346481, -0.37945360581620824, -0.48286795625783618], [0.44165267918542694, 0.22927885731282963, -0.63724406142269951, -0.23213543618414917, 1.5168618003621004]], [[-0.45761691287015965, -1.9451040629760326, -1.5538422819720592, 13.593107752281977, 0.4629658581357427], [-0.60439564484912744, 2.8811642720458424, -0.85306873552945983, -1.5784818348120289, 2.9317796125161082], [0.88968961839955751, 0.50022429959096759, -4.0405234254288658, 1.6092221264314333, -1.1311463520279823], [0.60920164054614367, 0.98095207579191135, -0.45989245127358869, 2.1808202258223481, 0.52724354453701106]], [[2.041027970495763, -0.47987773357872388, 0.6950700190669723, 0.4869125897273705, -10.156992929880204], [-1.0162632693268341, -0.57860869823658889, -1.8507726067102643, -0.8770255839311899, -0.44050661258506296], [-1.1090492095144422, -0.48832469959272934, 2.4903683181878788, -0.45712758679512688, -4.089377635457569], [-0.59943495974210448, -0.73098105139168956, 0.75458412334387492, 0.54424104249483995, -0.42825134018798489]], [[2.4323659418637651, 2.288147101052215, 0.94436774961624703, -1.676728061750886, 15.820745830566954], [1.1203384991860077, -2.02965445831413, 1.2429926740661388, 4.7964434830254516, -5.1039206120262239], [5.2035175134676166, 1.0811274958950332, 1.5278075577459305, 1.0574268301934602, -1.2086666681134417], [-0.75732080534155266, -0.75542293075663025, -1.1917680613386985, 0.82578008088896082, 3.1500586473074774]]], [[[-0.6534242014691749, -0.56172754416216109, 0.65947003841397578, 0.75327712561816651, 1.439390859876774], [-0.88119112913258113, -4.52460379602865, -0.79558237417418076, -1.1897387662567203, -0.69460713038367372], [1.2273362017550671, 1.6404934961863404, 0.78915267766603892, 2.4637573610631804, -3.3509836117291685], [1.0499177865379656, 1.040587683514991, 0.69721757121544492, 2.2768449894402982, 1.4038024014925605]], [[-1.3521598862967645, -7.8846109821250874, -6.5305592261165399, 1.4557767356703388, 1.045697270474488], [16.164938808890355, -27.396497174785257, -0.96255514059511627, 1.0336366367680903, 3.2814182359515471], [-2.9333079610237771, 2.0320146367483023, 2.555357638969034, 1.112985973815557, 2.4263141831470141], [2.7258414440018619, 1.1680345411201725, -2.1207576960292309, -0.88045063834841308, 2.9694934910618507]], [[1.0485638658129148, -1.0933603182831519, -2.098876761768957, 18.758886630170224, 1.89081033626266], [3.4696533870984037, 2.1738937828602887, 1.2461199581812064, -1.0740664222798602, 9.9524323341624488], [1.1588254323031342, -2.5467837331731533, 0.89791676262941589, 2.8005663479256824, -17.500604047513285], [0.98643557599379983, 2.3947051395725287, -6.7572911588793421, 1.1416265282097238, 1.1543412641597521]], [[2.8262265016371257, -1.0537292204003723, 4.9401707031742541, -4.6031358082125715, -1.0817194162215804], [1.9040704195233142, 1.283202603629584, -4.0986957247535338, -2.6065937261677679, 2.0143769586791493], [1.0117111802413965, 11.351195872781902, -4.0944275866046489, -2.6801198118883094, -4.216753911471308], [-4.5478404946007887, -1.8162544631397486, -1.6823268675331615, 1.1402371569106593, 1.649511222134691]], [[2.9155625391243172, 1.2461549899768463, -3.7283842556246927, 18.709370037909448, 8.3999876577109909], [-1.6976061761045897, 0.87420510585255673, 1.8829748807478441, 2.9895395778614366, 1.6807512440688457], [1.8603676090597394, 0.9272317631230873, 3.5133971737540786, -1.3762326316122342, 0.88513067921241051], [1.7878935925223705, 6.198055872455023, -1.2176581763106453, 0.9255891816285311, 4.2236464248299797]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.7067084140413069, 1.3086440215502639], [1.1202004917271697, -2.6851302044912497]], [[-4.0358539314582984, 1.0091529908837638], [0.68118946076863196, -0.27510180914272553]], [[-4.0961613826212409, 0.16520851254676483], [-3.3260638178922921, 4.1038146562463353]], [[0.52110013072264483, -0.62142119340836111], [2.7444874343795611, -4.7712649523279982]], [[-0.55275729889200687, 3.7466470036181789], [3.8783148290923037, 0.61602259614193411]], [[-2.5391598084146461, 1.1541135100406716], [1.3661653396710793, 4.5865155897514072]]])+(1.-msk_arg0)*numpy.array([[[-3.6535158000447989, -1.8933698936724852], [-1.3969120144583949, -2.6655738430985974]], [[-0.44539222485018737, -0.51363817158772207], [2.1140059911872244, -0.051462335375429547]], [[4.4985311188559809, -2.0479960702059428], [-2.5352537393994057, 2.2407051692838342]], [[4.4211926775679409, 2.8048470019806562], [3.3062647578045912, 1.7622129850437513]], [[-1.1563114933300733, -2.0359356553492001], [-3.9957266605597308, 4.4890379614825839]], [[-3.2398863759090037, -3.8747634371664064], [1.7280896763498701, -0.13890890898374675]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.6901114384332754, -4.6306339171146691], [-4.1766432919639822, 3.3818832293719456]], [[-4.2026710380673968, 1.2851731718999915], [-0.91881276516502464, 1.0316689809896165]], [[-0.50548591419182998, 2.4068043292490788], [-3.8142071066112271, -2.4600047030462999]], [[-2.3565225020105873, -1.4373960555828305], [-0.37121833135465998, -3.3682609100423324]], [[3.3176791449158909, 1.1372195347347169], [-3.4050476489855965, 2.4589349711199837]], [[-2.4704231348672723, 0.91938055372873251], [2.8887260565949147, 3.7988143918580946]]])+(1.-msk_arg1)*numpy.array([[[-4.1101778929518886, 3.845686775805536], [-4.0771137740381942, 3.2916339387418638]], [[2.1119023648145072, -2.8160770481124731], [3.1368935519974546, 4.1272133794252177]], [[4.0455434151679857, 2.0433918125251429], [-2.5647062300468804, 2.9208943914751639]], [[1.645404741828643, 0.061780105338050895], [-2.6838719352455609, -3.570718734582278]], [[4.3442853267873396, 0.23978652188778149], [1.6881531624251931, 4.264439923516731]], [[-1.1705016577842065, 0.27800444123962187], [4.7063613964889655, 3.7426301515683509]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[2.784850931725781, -0.28260580408085362], [-0.26820592840247509, -0.79397484252876138]], [[0.96030688457457536, 0.78522724637321728], [-0.74138005760758663, -0.26665705203120221]], [[8.1034135029661041, 0.068642269975602785], [0.87201972124879423, -1.6682141506333117]], [[-0.22113098019562372, 0.43232426511452293], [-7.3931894051791662, 1.4165366281758833]], [[-0.16660963123545819, 3.2945679257014979], [-1.1389898846930084, 0.25052415105607739]], [[1.0278238462784905, 1.2553164251298687], [0.47293004352286921, 1.2073544839625683]]])+(1.-msk_ref)*numpy.array([[[0.88889481068686316, -0.49233596079230629], [0.34262276989999657, -0.80980263683799525]], [[-0.21089621957467106, 0.18239492841007224], [0.67391703165735595, -0.012469027075744874]], [[1.1119720287735895, -1.0022532426980366], [0.98851623226768692, 0.76712981332823604]], [[2.6869940052891716, 45.400489148294263], [-1.2319010882693568, -0.49351772459051008]], [[-0.26616840431730621, -8.4906175681634206], [-2.3669218821469302, 1.0526676520232545]], [[2.767946849423693, -13.937775309951292], [0.36718167832140081, -0.037115318200901283]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.51888145999164514, -4.2001985506673059], [-4.7206790716534988, 3.0322592110035487]], [[-2.2417355251381723, 0.10409933133669114], [3.8069705260493265, -2.3649838324891812]]], [[[-1.9181127974631949, 4.6933009292746224], [0.28185615065035119, -1.539473479835114]], [[-2.3329315121712781, 4.5136818235493159], [0.14200393144118451, 4.7024567018332615]]], [[[2.7660647195337926, 1.5902486445543875], [-0.44798238699518933, 3.9324484943735047]], [[2.0999527850392008, 3.2717741879474911], [-0.85728222623118278, -4.5477170641599765]]], [[[-0.61866362929961305, -3.6103858006431722], [1.0324300210653554, -0.19460450649877092]], [[-2.1309252356635655, 0.71461213429701154], [-1.2749582746983288, 2.0674193934361371]]], [[[-2.2394644667319175, -4.7382754824972197], [-3.7497719666548335, -0.53754066471807782]], [[3.6555280159010408, -1.183363920944914], [-0.28222837675513812, 3.4977156277946335]]], [[[2.496998044953803, 4.1444330308014514], [-4.0898538272020328, -2.4409656577806595]], [[2.6032107768177184, -1.8676143920236532], [4.6712825007783163, 0.90799760502665627]]]])+(1.-msk_arg0)*numpy.array([[[[-2.1402180999827083, 2.3674369414270169], [1.568130266479411, -0.25770016041948729]], [[1.853706110479397, 4.247754600012394], [-2.3407093032703328, 2.076657903966705]]], [[[2.6795073407024788, -0.12860537637267022], [0.75434978494216232, 2.3271999719823029]], [[0.9538298337489941, -4.6608393990633044], [4.6025749780129317, -0.72783617659018329]]], [[[4.1688259148783899, 3.5545342366512571], [3.5421705303476472, 1.7320016614644373]], [[-3.7735812229942645, 4.0598629668825019], [4.7490710357942874, 3.9989203011540635]]], [[[4.2295096710146822, -0.11915180902509537], [-4.6309909060700383, 2.6097594143425615]], [[-4.4430934385199201, 4.3305640834533552], [-3.0326052207175946, 1.6391291912363846]]], [[[2.0182820713933536, 4.3379185919470427], [1.829272497296218, 4.9972039126290806]], [[-0.91322492181643344, -4.3684413607350736], [-3.4697836702342966, 2.1581195225837826]]], [[[-4.1230835852209911, 1.0039813025624786], [4.6037213531756151, 3.5485607103263437]], [[-3.5257219022004582, -1.9265734141610693], [4.1126228502472983, -2.5458756079450962]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.51545141753522472, -0.32002448931706517], [4.7166652636486859, -3.2154909627086106]], [[2.1020255540950288, -3.7805876928136595], [1.6953273043194539, -1.0876468553589813]], [[1.041624713693027, 0.21670896678752971], [-0.79918245826114997, 1.219488609536894]], [[3.5305078650387394, -2.4979984875218078], [-1.772605368974248, 2.8806587410059867]], [[4.1866110411726201, 1.9787912900425519], [1.9634501664866724, 2.5775540128588652]], [[0.81982974515458018, -4.5555516291094342], [-4.8204250362422911, -3.5827969767991941]]])+(1.-msk_arg1)*numpy.array([[[-1.5374960177581656, 0.017384201691458046], [-2.5258081057479274, -3.9895072422407152]], [[-3.3727643837528323, -4.6679919593009771], [2.0263206021815261, -4.6366587058820095]], [[-4.4486377547775193, 0.95716096809492779], [-3.7583695746942913, -1.5466231761422797]], [[-0.26433424009995488, -3.945424162154858], [1.1693044360432268, -4.9566760581227403]], [[-1.7254131004873994, 4.9886260553073019], [3.1185270604871587, 1.5083026612064483]], [[2.154081192325358, -3.0928353812813416], [4.6154613941447327, -2.760626444792619]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0066544437355942, 8.1485827912778497], [14.750993218448581, -9.4750849145151808]], [[-0.47527975801361527, 0.022070536177113124], [-1.1839468902884043, 0.73549696140243737]]], [[[-0.91250688828518423, 2.2327516048182385], [-0.07455352806287717, 0.40720480648059726]], [[-1.3760950503347047, 2.6624250149508555], [-0.13056069692245434, -4.3235142718094846]]], [[[2.6555290818003465, 1.5267001864003806], [-2.0672074332503718, 18.146219571195857]], [[-2.6276262239392101, -4.093901404025023], [-0.70298502136624252, -3.7292001160117367]]], [[[-0.17523360744384706, -1.0226250552775802], [-0.41330290079142501, 0.077904173069308955]], [[1.202143056182136, -0.40314225986494417], [-0.44259261138759065, 0.71768979921382525]]], [[[-0.53491104014875712, -1.1317687351175774], [-1.8949810348994403, -0.27165101616478138]], [[1.861788029202754, -0.60269618304720374], [-0.10949465087721195, 1.3569902358380384]]], [[[3.045751949977114, 5.055236230810352], [0.89777356513058759, 0.53582219158338118]], [[-0.54003760192213723, 0.38743770061395483], [-1.3038088764246853, -0.25343261449267074]]]])+(1.-msk_ref)*numpy.array([[[[1.3920153777720845, -1.5398003728679535], [90.204330018210285, -14.823813310110848]], [[-0.73390615314795993, -1.6817408220148911], [0.58671639406666887, -0.52052992459297953]]], [[[-0.7944543513356912, 0.038130554565917421], [-0.16160048935798182, -0.49854412609802279]], [[0.47072009864683106, -2.3001490455387312], [-0.992649075545319, 0.15697428315499243]]], [[[-0.93710168026186635, -0.79901633546897388], [3.7007051566235067, 1.809519735130553]], [[1.0040474061950682, -1.0802191977654922], [-3.0706064082395481, -2.5855815190410594]]], [[[-16.000612215108202, 0.45076191786595454], [1.1737624944083951, -0.66146485322815041]], [[-3.7997747221029683, 3.7035385738443081], [0.61182235537622442, -0.33069120757856785]]], [[[-1.1697384648483449, -2.5141333346325325], [0.36668863871848856, 1.0017194829250937]], [[-0.29283854335827847, -1.4008027751577889], [-2.3004558431654001, 1.4308265695543918]]], [[[-1.9140799334355962, 0.4660833148441671], [-1.4885116036367649, -1.1473487182031001]], [[-0.76389370446760974, -0.41741729583203951], [-1.4897426118644033, 0.92220938213041892]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.2660013133266572, 1.7675338049744136], [0.47367971191213698, -2.1682691721145462]], [[-4.543822456300318, -3.3766783121878063], [2.6369652613029659, 4.831528903582603]], [[0.83486110507153377, 4.8953980632537029], [-4.1907511077617183, 0.41559047739554078]], [[4.9384011443453488, -3.4018134972007266], [0.51123081587371466, -4.8666862894690963]], [[-2.3810871215055753, 2.1280618134826561], [0.13654875079281226, 0.12431947777118868]], [[-2.2375954398292408, -1.5329268540914232], [1.9661034186067585, 2.9971659691811334]]])+(1.-msk_arg0)*numpy.array([[[3.8027412993657332, -4.6234295484490309], [1.3698162931514037, -4.8798421645988643]], [[-4.519991791609753, 0.10684201610876709], [2.7319585995034323, -2.3930724477932142]], [[-3.1454951920984819, -4.5441746219990753], [-3.2485130352361766, -2.3473555346140973]], [[1.1917465829750959, -1.4406581029828311], [-0.20381359046611003, -3.6509728005194866]], [[2.3223437373046476, 0.61823239765200633], [-3.7027407137625423, 3.8213175438533504]], [[2.8523883757394275, -2.9298396411250973], [0.56155578329650169, -0.84260193849147491]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.45504264316183285, 2.0973975445560145], [-0.41609002186950583, 0.64977006131173187]], [[-0.98624213578520603, -1.2048480757352529], [-0.20097509949317072, 3.8571947739583532]]], [[[-1.2839454150284269, -2.0659061603088191], [-0.057270535401236167, -2.1015362354545184]], [[-0.041504741112028398, -0.69420972795543356], [-3.1922236113572433, 2.8516147846892075]]], [[[-4.4169597007887589, -0.59222632795816654], [-3.0538389592697737, -0.69199835438569934]], [[3.0854946543914004, 3.7698307461293137], [0.97178318142556375, -3.1177211265727864]]], [[[-1.5817859707226276, -4.8199087547012223], [2.2188057943274329, -2.3255036193279843]], [[-1.4832409831384599, 2.3366151766307386], [2.464172629005553, -2.5552910709353016]]], [[[4.3136428919842658, 1.0696257490823253], [2.2272245846458558, -3.2679152785336307]], [[3.6471081794878195, 0.96650552371803666], [3.4761857054194927, -0.46361762144656904]]], [[[-0.23972764417939185, -1.9418676305482183], [0.90295075241418665, 4.2267664347084306]], [[-1.9092934453109303, 3.4569249477390649], [2.8413494865623665, 2.4056979282454094]]]])+(1.-msk_arg1)*numpy.array([[[[3.7225464822012935, 0.85620436022092328], [1.4395502726847145, -1.6840361822055137]], [[4.8476232929111713, -0.0051445131539811229], [2.4685293850205774, 1.8478213858261547]]], [[[2.1631909009159305, 1.0178511523855214], [2.6550457383993269, -4.2043807009259364]], [[-2.535803523096134, 0.81028072824915753], [3.6611613856300789, -4.7763517887956741]]], [[[0.22675163242136875, -3.6685012132103143], [4.6178874515480075, -2.5356902067681109]], [[4.3178649493156289, 3.1831477081507575], [-2.4579978447643103, -2.8935098978937193]]], [[[-0.92474840451384033, 2.2180975610201976], [-3.849436881368915, 1.5998353847684896]], [[0.53358110551798621, 1.9258966659777146], [-4.4977913023102349, 4.0363186943278269]]], [[[1.1321468839766524, 3.9309338433253451], [-2.3108374886096561, -4.6957505119602851]], [[3.6530287447453738, 2.8784292146514563], [4.1308489641910562, 1.16604186053305]]], [[[-3.3422205606170308, -3.706877411518442], [-1.4461266570992262, 3.3926682876206797]], [[4.4697736741439886, 3.8316559603471809], [2.7116514216198571, 0.80722558653992227]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-9.3749484305133191, -2.0339497985965753], [-4.2479600857353592, 2.7202450685496058]], [[-0.48028744131380313, -0.39314476360272738], [10.788745359910747, -0.56213629313030833]]], [[[3.5389529828257666, 2.1994331318616576], [58.960131741930979, 1.6067666382433332]], [[-63.534073232389176, -3.798513842595205], [-1.5135308461453216, 1.6943133166246307]]], [[[-0.18901261537940869, -1.4096994099365781], [-1.6030308502005222, -7.0742914809377613]], [[-1.3582104580208143, -1.1116549760395971], [0.42765761472213132, -0.13329943908497885]]], [[[-3.1220413101080138, -1.0245839487165711], [-1.5331731627426584, 1.4628287261852426]], [[-0.34467144697686086, 0.21879119034520669], [-1.9749778210275419, 1.9045526143085394]]], [[[-0.55198985663142852, -2.2260936814099748], [0.95547697711007118, -0.65119858750975745]], [[0.03744027982520344, 0.14128087987280691], [0.035763186522909392, -0.26815088991503366]]], [[[9.3339065984180536, 1.152290405704705], [-1.6976860033538843, -0.3626712944211139]], [[-1.0297544483983585, 0.56874344925904463], [1.0548389007954395, 1.245861308683551]]]])+(1.-msk_ref)*numpy.array([[[[1.0215429995428875, 4.4413944567912802], [-3.2117180178963012, 2.7454454941662316]], [[0.28257482283215535, -266.26742942456281], [-1.9768215821980935, -2.6408624783921435]]], [[[-2.0895020359487986, -4.4407198253067959], [0.040241120732323041, -0.025412069864471865]], [[-1.077354209275567, 3.3716198648912798], [-0.65363751982797968, 0.50102516598690727]]], [[[-13.871984772542943, 0.85743332475167744], [-0.98403754307086411, 1.7920858825222574]], [[-0.7523424362197938, -1.0205348080197614], [0.95498681563700794, 0.81124848970546615]]], [[[-1.2887252112661087, 0.53728321238807952], [0.37425164962583107, -0.90050396228191132]], [[-0.38197302782723774, -0.10582789516520635], [0.81172570160029645, -0.90453035971865636]]], [[[2.0512742385046745, 0.59078677735773788], [-0.26753607759062864, -0.13165784597740893]], [[-1.0136084253617428, -1.286375463018256], [0.9250683278374664, 3.2771701198672702]]], [[[-0.8534410952258723, -0.76948548848045351], [2.0259910338711542, -0.86357975279093091]], [[0.12563405313895359, 0.14655694277040987], [-0.31073386932164404, -1.0438246167384129]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.1574321857534802, -0.80083202021917632, -2.9693911999328559, -4.1217180562632247], [-4.6874401426960164, -3.0976981261312075, -0.24971578618963708, -2.0447990040836639], [3.7509669490595847, -3.7255448666419766, -0.47108426481032772, 3.3394971226104335]], [[-3.3031166363619837, -1.1510760338116413, 1.6248644606346758, -3.0904630511446074], [-0.074711690894617711, -3.3487123275588306, -1.5909762958149241, -1.1464682841593445], [1.1144226415171801, -3.5689384676410718, -2.5156935328790775, -1.5599808558509265]]], [[[3.2132189629065078, -1.3276127556881678, -2.8186158168199893, 3.0851531842754305], [4.8720057444690656, 0.8229779906711201, 2.1361629763628267, -2.4371243733211356], [-2.3255595537742204, 4.8493640268884146, -0.98702710210383326, -1.955847632178449]], [[1.5990684724544248, 1.795354109172016, -3.3718335926263321, 3.0953510439256799], [-3.2175932770363156, 2.7259280343458343, 4.7478537869267416, -3.9433387277741994], [3.724696472670102, 4.3669172984880138, -0.94810441061681416, -4.926645116616033]]], [[[2.7839200688086594, -1.3724365175860287, -4.5857259488252398, -2.2375075314081991], [1.3141934572978764, -3.4794675651457654, 2.5681969716585504, -2.5949024172253421], [-3.6058659416057082, 0.34810890556900542, -0.81881753125530832, -3.7947546874220048]], [[-2.9396860476958988, 2.7683251417763524, -1.9955368224764634, 4.3753948015635054], [2.6655896128664676, -0.52823997756894236, 2.0990072365712074, 2.1099151941203154], [0.65966287468625318, -1.5113299500438027, -4.3450044589463506, 4.8575549741296857]]]])+(1.-msk_arg0)*numpy.array([[[[1.6758094114799791, 4.6385088227078199, 4.2899895576899176, -3.2620879666997471], [-0.87138665717023756, -3.3263425743083541, 4.946848800157408, -0.51082518715821834], [-0.53048939821795837, -1.3935783580802674, 3.8631404478525084, -0.18876418500964576]], [[2.0400295904960224, -3.1722577351651582, 4.2822404331435475, 3.4262287664072986], [-0.43969196692109325, -3.5197771509292144, 4.3699836677484214, -4.7554950071467683], [1.5330594824283494, -4.3067211540339745, 3.0965791179793563, 4.6693270799677418]]], [[[-3.142159009026162, -2.4165670199286882, -0.29080432570117942, -0.27163852501282726], [-0.50632931932208436, -4.8232298281067028, -2.501161449539234, -0.59324903846702615], [4.3633352374804133, -2.2815655539231772, 1.4428653068396802, 0.8351619392496632]], [[4.8862950830607748, 3.351509749014312, 0.22997082196580276, -0.80797260650168923], [-0.61420011964701349, 3.9880414583226678, 1.7881041409901268, -4.5567506441822712], [-4.8286811426787981, -2.7263768804138468, 4.2107950207907709, 4.274196150691175]]], [[[-1.9984608020447006, 4.8113093531133693, 0.47477544716580056, -4.2298630808278261], [-4.2305847385433459, -2.5694222180347315, -2.1743403024023102, 0.060894744661648126], [-0.15950618589788412, 3.1757111462247742, 3.8340808525794863, 2.9661476123873429]], [[-3.7750707957826704, -4.2400451244425117, 4.8486925740886715, 3.6003211361381098], [2.7950705803456755, -4.8902072140994832, 0.45196484773593593, -4.285550029231846], [-4.1674438600856023, -0.40043221088785241, 1.9381577423622263, 0.12686412706925854]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.8359077477849866, 3.1152374465029276, 3.6098255435917626, -3.5638400647626467], [-1.0211343434942366, -4.7287655443722789, -1.2968614352040095, 0.3866654677580792], [-3.097403863869872, -4.8504644625873503, 0.56098100863286859, 1.437951573347382]], [[-0.45304420753577102, 2.3036484252139662, 2.0924016879254577, 2.8692612094837768], [1.5637495638563781, -0.0085056761042974216, 0.78704525707372497, 0.61018697377786957], [-1.6280334210845728, -3.1900931472178193, -2.7893602226009948, 4.0422674042925859]]], [[[-0.34811143186297766, 4.1245520269299192, 0.300394147603539, 3.327781665043549], [-4.0011637098002044, -3.975781365726224, 2.0598977266073142, -0.15264442482024521], [-4.9310471477842688, -1.4310538200072633, 3.3482738925413145, 2.1200717984667534]], [[-2.3380985159594969, 4.9045116667770436, 1.7468884425740319, -3.5907163036451539], [1.9614895987858008, -2.6510728764868032, -2.8344082479621804, -1.1651800915146762], [-4.3788112187688899, 3.2585434823431765, 1.5873242779683183, -4.7315204117604264]]], [[[4.7102253008852344, 2.7631872678510074, -4.1354076538378735, -1.375720741421139], [-1.6930901977119373, -3.4350287472176024, -2.6169194761527637, -0.36713737896124243], [-1.73585808908053, -3.8737236057069699, -0.93010067793456308, -0.42554118102135607]], [[3.3940226645550524, 3.3079223340161885, 2.6908717905851471, 0.50265031495123846], [-4.7501901632916077, 2.4752328172452023, 0.99036402141482771, -1.6870782681767551], [4.117798861865543, -3.0440731451234084, 3.9021411357946292, 1.6039285823941256]]]])+(1.-msk_arg1)*numpy.array([[[[-3.3894775300782753, -4.3247736030290396, 3.2026875575170628, -2.7678102826646578], [3.6152301167868632, -0.42217225425142146, 4.387572995370947, -1.4085323879702338], [-2.1392480584716389, 0.080827068722689965, 1.9962577460351261, 0.75254440390696598]], [[-3.8628528278946472, 4.7002295788687061, -2.9327254177737938, -3.696544848352926], [1.3450416700337717, -3.7024089373288991, -2.6186309667365193, 1.0055944799329577], [0.062611942723957981, 3.1642568862326996, 1.556535015402809, -0.547582558240018]]], [[[-2.4757260805316985, -4.7347856678665963, 3.769266441754441, -1.9113226177027185], [1.2927826401791229, 2.2410378952348706, -1.0659318288459696, -4.7479903520771103], [4.9406494971026103, -0.56260545391294592, 2.1318095746006351, -1.1406683833736286]], [[4.8370592147917222, 4.5681823680202847, 3.5091031499088068, 0.40087350031301927], [-0.42201213743623178, -3.3766444440738965, -2.7734206985807441, -3.5643813905015564], [1.4695737485473872, -1.789902102959692, -3.4048925559025731, 2.2982337685851491]]], [[[-3.4949824350187098, 3.8219793898789565, 2.955722166976436, -3.3518713307879908], [-4.8291862365619664, 4.2371542246096698, -4.3158298309820502, 4.2200483391752144], [-0.44636029675457412, 3.9965240672480018, 1.1976940390145963, 2.9511364613363478]], [[3.4856812522978871, 4.4741381536170906, 0.079884556525188444, -2.228599884187771], [2.1310476074944589, 2.0738725602045847, 2.9850266290595435, -3.0703970674252812], [2.2856932053285783, -2.7739273672601774, -3.2499507063984625, -3.5673195592665099]]]])
      res=quotient(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.4659969771586128, -0.25706933547494637, -0.82258579094055695, 1.1565384476751857], [4.5904245338140202, 0.65507543079986053, 0.19255394555730176, -5.2882896834299444], [-1.2110035093625653, 0.7680800251971508, -0.83975082500275267, 2.3223988794257355]], [[7.290936693194956, -0.49967522006085963, 0.77655474568349803, -1.0770936577435652], [-0.047777273689765566, 393.70325021745441, -2.0214546514520095, -1.8788802996910601], [-0.68452073961403537, 1.1187568208638847, 0.9018890828425411, -0.38591728350141891]]], [[[-9.2304321800361997, -0.32188047259919444, -9.3830583561834437, 0.92709002416931596], [-1.2176471891254723, -0.20699779866310464, 1.0370238040318258, 15.966022841588252], [0.47161576113081666, -3.3886664212697477, -0.29478684653085152, -0.92253839402652682]], [[-0.68391834712670663, 0.36606174705091832, -1.9301940012024759, -0.86204277424629772], [-1.6403825332686275, -1.0282357978624219, -1.6750776075889025, 3.3843169450724608], [-0.85061818986508086, 1.3401439392018855, -0.59729724025284359, 1.0412393243344391]]], [[[0.59103755998369178, -0.49668603121981064, 1.1088933262889928, 1.6264256720421488], [-0.776209949755715, 1.0129369566307584, -0.98138173339370682, 7.0679330570131844], [2.0772815268071287, -0.089864156817010224, 0.88035365491144812, 8.9174793337605607]], [[-0.86613624546354762, 0.83687730915232683, -0.7415949096714568, 8.7046494778143284], [-0.56115429514075865, -0.21341021898572124, 2.1194300188455748, -1.250632667090499], [0.16019793506557944, 0.49648279722349853, -1.1134923898803413, 3.0285357013084653]]]])+(1.-msk_ref)*numpy.array([[[[-0.49441525916865381, -1.0725437325688083, 1.3394967447326658, 1.1785807673057824], [-0.24103214152926641, 7.8791122363227979, 1.1274681481941196, 0.36266484996794657], [0.2479793757985039, -17.241480856636073, 1.9351912124199881, -0.25083461391732298]], [[-0.52811475906212313, -0.6749154869853583, -1.4601573018704761, -0.92687331196155442], [-0.32689839780952906, 0.95067217330902287, -1.668804701104766, -4.7290384961777177], [24.485096863824605, -1.3610529450917841, 1.9894053698355163, -8.527165465195603]]], [[[1.2691868594571405, 0.51038572586909658, -0.077151437871243136, 0.14212070871599822], [-0.39165850745948239, -2.1522303743110989, 2.3464553565748334, 0.12494739763055682], [0.88315012834633244, 4.0553562679756965, 0.67682654399841546, -0.73216892080378104]], [[1.0101788847485036, 0.73366373734916313, 0.065535497858414102, -2.0155301008193094], [1.4554086604673124, -1.1810664475857948, -0.64472877912289395, 1.2784127580525482], [-3.2857698686110508, 1.5231988810481016, -1.2366895435484802, 1.8597743228368271]]], [[[0.57180853958540767, 1.2588527729516996, 0.16062925415330001, 1.261940767825783], [0.87604505838135127, -0.60640280759934551, 0.50380584674432072, 0.014429868989024347], [0.3573485076016667, 0.79461829649672611, 3.2012189488176626, 1.0050865662254729]], [[-1.083022377130441, -0.94767863192035595, 60.69624449326232, -1.6155080872447736], [1.3115946215917389, -2.3580075786417036, 0.15141065856364941, 1.3957641096972342], [-1.8232735042350157, 0.14435569424565048, -0.59636527364750658, -0.035562871495410285]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.05854289426)+(1.-msk_arg0)*(-2.54658711679)
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(0.471712400856)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(4.36397875172)+(1.-msk_ref)*(-5.39860116497)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.3510734722404103, -0.19122437003608539])+(1.-msk_arg0)*numpy.array([1.3497191909188633, 3.5669747863754857])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(1.87747213966)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.2522547858747128, -0.10185204136788401])+(1.-msk_ref)*numpy.array([0.71890238070886636, 1.8998816073053724])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.522295851777)+(1.-msk_arg0)*(3.68415697016)
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([3.908117874806404, -4.4952395388116244])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.13364383278821218, -0.11618865852815621])+(1.-msk_ref)*numpy.array([0.94269341104321602, -0.81956855432317377])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-4.9034603951892901, 4.3717900134448335, 2.8197206771606815, -2.2206682575874845, 4.685003882217103], [2.3422292904949895, -0.58269546089568269, 3.0319474154393582, -2.368125469490395, 1.0818919144494963], [-1.0098893004875054, 0.24885695186814871, 3.0075652803034743, -3.5652661750818826, -4.3788384651740824], [1.7369422855476788, -2.9125314530841484, 4.0430769740889883, 4.0527140934713159, 1.3967334737847139]])+(1.-msk_arg0)*numpy.array([[0.2848029818185509, -0.81139460287321885, 3.3713743557747762, 1.0638245419325854, -1.5106748810301793], [-2.3663286652040307, 1.5152374849870043, -1.6117791710811904, -0.37595491664821257, -4.592633053230597], [-1.9586020440965202, 4.7815450246309403, 1.8370458177990407, 2.7180890272768181, -2.5851310917355175], [1.8719649482006941, 3.8248554801561436, -1.421680359981071, 2.0955960924646675, 1.1119537434640216]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(0.266967461129)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-18.36725859568757, 16.375741054575386, 10.562038778946478, -8.3181232956139191, 17.548969684940197], [8.7734635546606814, -2.1826459989978577, 11.356992356364591, -8.8704648104912511, 4.0525235168153717], [-3.7828179367513766, 0.93216211000321814, 11.26566236793537, -13.354684349940374, -16.402142967750283], [6.5061947182724742, -10.909687048630714, 15.144456020939426, 15.180554500288089, 5.2318491095443038]])+(1.-msk_ref)*numpy.array([[1.0668078447245812, -3.039301491807465, 12.628409250768399, 3.9848472073486336, -5.6586479664673082], [-8.8637343861962226, 5.675738453590836, -6.0373618727368896, -1.4082424691698403, -17.202969357425019], [-7.3364822657192752, 17.910591067582075, 6.8811600111514304, 10.181349501481387, -9.6833190112521379], [7.011959211376019, 14.327047438597425, -5.3252945282888335, 7.8496311258419151, 4.1651283596954816]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.25095352362)+(1.-msk_arg0)*(-1.71226254951)
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-3.9111523327561413, -3.7771398392743794, 4.2010557153278878, -3.4690668594562304, 3.3267555968702762], [1.208843305895563, 1.921778641258217, 4.5756941758993452, -3.5178635908063161, -4.9368347684111331], [4.313789353860809, 0.60111724648543507, 2.342682064294797, 3.7191656563975073, -3.1869093282232308], [-1.2739511611116283, 2.3194489223913433, -0.9133471236938675, 1.4546039220596194, 3.8534030955565388]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0868800706180215, 1.1254424523611173, -1.0118774450226617, 1.2253881795435222, -1.2778075815437175], [-3.516546357076896, -2.2119891606458233, -0.92902920523273735, 1.2083906649289631, 0.86106862454137945], [-0.98543372773157378, -7.0717543848192248, -1.8145669821839321, -1.1429857974493753, 1.3338796576286123], [3.3368261306928937, -1.8327428910316839, 4.6542584011559809, -2.9224130769596548, -1.1031686585101157]])+(1.-msk_ref)*numpy.array([[0.43778978772337079, 0.45332251978259236, -0.40757910999936292, 0.4935801524965337, -0.51469442213366345], [-1.4164470623779977, -0.89097803084648386, -0.37420825861346713, 0.48673363969710276, 0.34683408091094581], [-0.39692771460400944, -2.8484668498902654, -0.73089839018616864, -0.46038889033236158, 0.53727997039241149], [1.3440566654195645, -0.73821955421451624, 1.8747117115627854, -1.1771331862536374, -0.44435074842949412]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.9700313032471648, -3.8576489161829941], [-3.9035743253010358, -0.75124686664183127]], [[0.38791693308550723, 3.3931520744384649], [2.9388806192325978, -1.8258934057078156]], [[-1.7664068390808674, 2.8088714485649193], [1.8250803333823074, -2.8030902762523038]], [[-0.33162735451937664, -4.5469058867315617], [1.1039227157566245, -0.75094979639637849]], [[4.2093936730055486, 1.3406924777690161], [0.83367620726081348, 1.1908671321658071]], [[0.35902220532964346, -1.6313424517279662], [-4.5724077842813893, 4.4961524449877661]]])+(1.-msk_arg0)*numpy.array([[[-2.2310186334346973, -2.731359108862339], [4.6355483164107145, -3.560007376823735]], [[3.1927796952471983, 2.9053164241760427], [-2.3437471186986389, 3.7395247775177989]], [[-3.0775804488977587, -3.8087249897152011], [3.5389183467595764, -0.74449898026009365]], [[-4.8689930721573198, 3.627017931676896], [4.2139205987864887, -4.1562083418027598]], [[1.534164831216426, 3.2993167655985634], [2.8095395116108302, 0.65739354661261551]], [[3.1931446279460012, -3.2247079738567095], [-0.71459829316434931, 0.98203038634161643]]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(2.93464014503)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.33054523052474621, -1.3145219602859988], [-1.3301713778801012, -0.25599284052366811]], [[0.13218551983010587, 1.1562412789115952], [1.0014449724633536, -0.62218647448073261]], [[-0.60191599371067017, 0.95714340080806337], [0.621909414164876, -0.95517342424299789]], [[-0.1130044360228119, -1.5493912922942596], [0.3761697043588102, -0.25589161167421087]], [[1.4343815476421782, 0.45685072496473361], [0.28408123860494305, 0.40579664739519389]], [[0.12233943093066531, -0.55589182015688865], [-1.5580812495937908, 1.5320966874243254]]])+(1.-msk_ref)*numpy.array([[[-0.76023584602379857, -0.93073050659554168], [1.5795968457176253, -1.2130984382698455]], [[1.0879629315542902, 0.99000772857729757], [-0.79864889828669106, 1.2742702998345059]], [[-1.0487079494589855, -1.2978507760692231], [1.2059121977013549, -0.25369344909976277]], [[-1.6591448462251854, 1.2359327728190321], [1.4359241305679251, -1.4162582587296333]], [[0.52277783830258473, 1.1242662140987199], [0.95737104815591323, 0.22401163826687323]], [[1.0880872850289132, -1.0988427249979769], [-0.24350457222968017, 0.33463400546867245]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-2.78949035858)+(1.-msk_arg0)*(-2.29292660902)
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-0.34738455475512708, -1.9140978384170717], [1.1578473301360965, 3.4661343552239856]], [[-1.2402125784075646, 3.4826273743542959], [-4.5680443012000973, 0.79075220238565169]], [[1.6150217082659228, 1.8951234984608076], [-3.6309656414561409, -4.6103752620963121]], [[-0.79066131792239513, -1.6177590017645791], [-2.64332765168106, -4.8262209319535012]], [[1.2002268926879225, -4.4137909846196477], [-1.374836367664968, -3.4383645043636948]], [[1.1091659144100845, -0.4157590470156558], [0.72843454295160015, -1.0949940846481376]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.0299780758736095, 1.4573394852624932], [-2.4092039476853153, -0.80478425609111337]], [[2.2492034084692945, -0.80097296056486023], [0.61065308798514562, -3.5276415926064599]], [[-1.7272153954982505, -1.4719306477104899], [0.76825027665701229, 0.60504627063967464]], [[3.5280470858378075, 1.7242928987186195], [1.0552949638334741, 0.57798646143861654]], [[-2.3241358576241273, 0.63199421275295542], [2.0289617180541106, 0.81128407271555714]], [[-2.5149441777287134, 6.7093918427125541], [-3.829431739024669, 2.5474935414625124]]])+(1.-msk_ref)*numpy.array([[[6.6005427634465308, 1.197915050632032], [-1.9803358779197016, -0.66152271494215564]], [[1.8488174115813045, -0.65838987711028785], [0.50194929335944949, -2.8996778031150949]], [[-1.4197497143762883, -1.2099088058822192], [0.63149223524530729, 0.49734055877687239]], [[2.9000111135411637, 1.4173474581325325], [0.86743942150487152, 0.47509772995285265]], [[-1.9104109589537797, 0.51949143423692024], [1.6677814632706962, 0.66686548389853717]], [[-2.0672530405350167, 5.5150371963780911], [-3.1477455746828849, 2.0940082153583979]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.8654699916764272, -1.5888464079360753, 1.3462046070682536, -1.1723410085328103], [-4.5732936298038043, 3.9702516318019416, -0.28889419834939467, -0.26683581394214251], [4.6896625182496052, 0.52536435163525397, 1.9637112584730705, 1.2857153795436336]], [[0.39546328700686306, 4.453612983208302, 0.9576585869969545, 2.8636594762573999], [4.23343789158967, -1.7816475462796388, 1.0203215456690353, -0.92534099358640898], [-2.6833589102802415, -0.98812914183931611, 4.4665390776777123, -1.8168675914920938]]], [[[4.3006565232021536, -4.0132643337208576, 2.0955923070946394, -2.6593683055400277], [-4.0475706492050945, -4.7884493391516791, 1.3202988950904437, 1.0219377458104582], [4.7799996432172041, -0.32320179053058862, -1.6617686668694609, 3.7872487821778851]], [[0.87964650675908551, -0.91662536867166011, -2.6380129748823822, -4.6399623754262826], [-3.5993715542771532, -3.5727964097396958, 3.5292729572562891, -1.5192842589815658], [-0.83834663045617841, 4.1278220763760185, -2.5119911454339183, 2.7973970314560006]]], [[[-0.90235571404587667, -1.5971187090735564, 3.8359151363861628, 3.3316607107908496], [-1.4121358049771326, -3.3827829631241304, 1.3974877733076996, -0.90279883047745368], [1.7017730725880318, -4.5591097985247364, -0.94766062256463712, -1.9017219521953677]], [[-3.5908031509772096, 2.1975271487424308, -2.3182671151807863, 4.3146397040872859], [2.280325715508738, 0.85376401643385247, -4.5919054189676158, 2.3006194337556218], [-0.312090857663474, 1.5235778227315215, 1.2644970907170503, 0.3930271744253222]]]])+(1.-msk_arg0)*numpy.array([[[[-0.27043620337590646, -3.7947417934108927, 1.1701438426649888, 3.6947707314867486], [-0.47795776062863116, -1.5776094092549164, -2.6695526627528468, 3.7686779826744932], [0.27874790838915953, 2.7721797182583092, 4.3535888635659052, -2.2631309891313842]], [[1.2348449066856579, 2.7512293423420777, -4.4368573276137955, 3.2392830040844753], [-3.1357859147595137, 4.1573489511640815, -0.049830854996063323, 1.0969976472580978], [1.1315709751657996, 0.058029648889096563, 3.0382141227011843, 4.1445457221160016]]], [[[4.6474890464049619, 3.258980241116145, 3.5541002228291578, -4.7514792382829967], [1.8267738542845935, -4.2991813354401049, 2.9093334634994585, -2.3323700530701732], [-0.10900791745353899, 2.7824120050413184, 2.6699894965292064, -3.1257305475387511]], [[0.47967323745046286, 2.5980615141403813, 1.5927484212546652, -1.2147973698247139], [1.2276170368301695, 1.5719432384286609, 0.77361232076484043, 4.3721951784274928], [-2.8527614643785282, -1.0253544956787417, 4.5801664661935142, 1.2325895839848968]]], [[[-4.0198789633543877, -0.50620264570054196, 2.1466322772612152, -3.7127284547553718], [-2.6324137555540936, -3.6263533548577698, -0.059166100643862407, 4.6017543562432763], [-4.2873627535129577, 4.3994452964122051, -1.5804906889667256, 2.6423773767814733]], [[-3.2255262213127187, -1.3955436718787553, 0.69768782625043269, -3.2305898867777385], [-3.8563314810548874, 0.64111114861700003, -0.67826411457765268, 2.4368019698978491], [-2.842982368130802, -4.2459575975563002, -2.5362200226278677, 4.7740205065110963]]]])
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s1=numpy.array(-0.882321759614)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-5.5143941976484783, 1.8007562327721776, -1.5257524734022694, 1.3287000980744361], [5.1832492851617626, -4.4997775341472162, 0.32742499570186567, 0.30242460988241893], [-5.3151386862535617, -0.59543397395625441, -2.2256180776175754, -1.4571955928023188]], [[-0.44820756452805771, -5.0476064255231066, -1.0853847551209905, -3.2455954361921533], [-4.7980658364812312, 2.0192719117097928, -1.1564052847512118, 1.0487568548585389], [3.0412475732810504, 1.1199192710278401, -5.0622565169762064, 2.0591893735979014]]], [[[-4.8742496445762091, 4.5485269857505806, -2.3750885482078177, 3.0140572603608531], [4.5874088506829738, 5.4271010399251418, -1.4963916289081558, -1.1582370429775184], [-5.4175243794354717, 0.36630830760861144, 1.8834043802748051, -4.2923669748700179]], [[-0.99696793961372476, 1.0388787975401315, 2.9898536969499503, 5.2588098671104806], [4.0794319249820399, 4.0493123634412793, -3.9999840407413725, 1.7219163444927439], [0.95015975897794602, -4.6783636824055872, 2.847023909431682, -3.1704953447806474]]], [[[1.0227059507636671, 1.810131838720167, -4.3475241255103336, -3.776015579902539], [1.600477138401686, 3.8339561801158046, -1.5838754491541769, 1.0232081671339974], [-1.9287443090282972, 5.1671737082823403, 1.0740533283218971, 2.1553610476823772]], [[4.0697207247236573, -2.4906187848103292, 2.6274622493658533, -4.8900978096393075], [-2.5844604767600958, -0.96763341392261459, 5.2043433916608999, -2.6074608369186776], [0.353715472006358, -1.7267825553768916, -1.433147348955474, -0.44544653936349005]]]])+(1.-msk_ref)*numpy.array([[[[0.30650519544492122, 4.3008593543805063, -1.3262098887558063, -4.1875548134522189], [0.54170460540102017, 1.7880205175314356, 3.0255999397766598, -4.2713193249610901], [-0.3159254606970055, -3.1419147131426914, -4.9342417503890443, 2.5649724315090427]], [[-1.3995403527454378, -3.1181701146581333, 5.0286160114117857, -3.6713171456864884], [3.5540162991447488, -4.7118286564542471, 0.056476964840856979, -1.2433079376140128], [-1.2824924273208209, -0.065769259634330862, -3.4434310268275667, -4.6973178173997105]]], [[[-5.2673403956808844, -3.6936414699106566, -4.0281226027821866, 5.3852001115327139], [-2.0704168681989974, 4.8725777060277435, -3.2973611177526099, 2.6434461438303294], [0.12354667247601762, -3.1535117146577409, -3.0260950355527751, 3.5426198135529652]], [[-0.54364888117472598, -2.944573774623811, -1.8051786708182429, 1.3768190080177429], [-1.3913484774156855, -1.781598630317941, -0.87679161522998772, -4.955329652459735], [3.2332439195712386, 1.1621094963439376, -5.1910387750103562, -1.3969842300201556]]], [[[4.5560238309337047, 0.57371660642461042, -2.4329358920032194, 4.2079076190732341], [2.9835076907823863, 4.1100123796592651, 0.067057283807352, -5.2155059150468865], [4.8591828398144106, -4.9862142109416911, 1.7912860832736526, -2.9948001939070226]], [[3.6557255742196695, 1.5816720563354461, -0.79074081382212913, 3.6614645978919782], [4.3706634671918172, -0.72661831313920278, 0.76872649596004938, -2.7618064989838964], [3.2221605521475443, 4.8122553380210116, 2.8744842740094967, -5.4107477850231103]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-4.64723049514)+(1.-msk_arg0)*(-4.01841662368)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[3.5555933640135819, -3.4543039443260182, -2.5727658359255412, 1.5925093351209663], [3.5886774493387605, -3.5684594712816233, 2.9522627452710228, -0.20774099491900522], [-2.7736856436489452, 2.3431961505149523, -1.0341093731551387, 3.03111074942721]], [[1.234238905215987, 4.3381535928175339, -3.7670962368722738, 4.4617325550799993], [1.7758957041896393, -0.74217879655038121, 4.439039452663792, -2.1783880351790854], [-1.225875027647485, -2.9224262205083495, -4.1667913905112126, -0.81686264682698351]]], [[[3.0857120606613098, -0.21660697232854265, 2.9255954012646939, 4.8807124685269851], [-4.8972417554131766, -2.5985760728455887, 4.6619285579275314, -1.85221978332725], [1.9184798119957769, -4.3702984065434913, -0.57026356997946426, -3.1164814891977874]], [[0.74292777192717629, 1.5967546915165052, -0.26241376191395549, -4.9823897787286917], [-1.375478885941507, -0.87838004923389246, -3.5234500220560996, 0.37973455506708653], [-1.545173539330885, 1.2767593585031323, -2.8801288441907369, 3.2322816539618326]]], [[[-0.53047062191832062, 4.401955840977422, -3.4303709582883544, -3.5618139608763544], [-3.2822689071122682, -4.367750053486942, 2.9541695262538692, 0.74684401343833251], [3.5210511253075811, 4.1669147210141464, -0.9069543885295106, 1.617175227511809]], [[4.1212590144447354, -3.9967852343884349, -2.8815387013897729, -3.0830901107473965], [4.8899596517556869, -2.9038024311517843, -2.9649641996559097, -1.5196763388771783], [2.1390278062032619, -3.0094851920319421, -1.474688236986772, 4.6116848209788444]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.3070196783966608, 1.3453449870199743, 1.8063169334143898, -2.9181810069511029], [-1.2949702392447162, 1.3023072091871806, -1.5741249665485681, 22.370310188194733], [1.6754712293310319, -1.9832870133902509, 4.4939448532057238, -1.5331773990839899]], [[-3.7652600930847222, -1.0712461870498367, 1.2336373171609794, -1.0415753158156311], [-2.616837511447732, 6.2616050428043533, -1.046900020758682, 2.1333345667041748], [3.7909496403239396, 1.5901960030778663, 1.1153019336953598, 5.6891210697343153]]], [[[-1.5060480057061347, 21.454667156759662, -1.5884734072023083, -0.95216231751199265], [0.948948556604441, 1.7883757738342256, -0.99684721406547416, 2.5090059705518546], [-2.4223504808778058, 1.0633668603004776, 8.1492677067021564, 1.4911785971616904]], [[-6.2552924668400554, -2.9104223208692517, 17.70955326903222, 0.93273121966144568], [3.3786272858425823, 5.290683115122623, 1.3189432136263008, -12.238102730264622], [3.0075783572858419, -3.639864054406249, 1.6135495134239799, -1.4377554287220566]]], [[[8.7605803283445383, -1.0557194717588192, 1.3547311797034198, 1.3047370093408588], [1.4158591592152499, 1.0639872790871516, -1.5731089410550534, -6.2224914594244893], [-1.3198418113671595, -1.115268923480941, 5.1239958193235209, -2.8736715824497119]], [[-1.1276239806462305, 1.1627421096228667, 1.6127600482689901, 1.5073287929348782], [-0.95036172608782066, 1.6003948633995129, 1.5673816552933939, 3.0580396471632678], [-2.1725900344376954, 1.5441945045772494, 3.1513308227358015, -1.0077077414314588]]]])+(1.-msk_ref)*numpy.array([[[[-1.1301676576262418, 1.1633071925472058, 1.56190531122823, -2.523323747658536], [-1.1197486206009017, 1.1260928296981365, -1.3611310951626503, 19.34339741294372], [1.4487642580837596, -1.7149296796153974, 3.8858719667322754, -1.3257241176153003]], [[-3.2557850888482829, -0.92629653093236097, 1.0667146181046414, -0.9006404068534738], [-2.2627548533388042, 5.4143511541370986, -0.90524462927835803, 1.8446743916991288], [3.2779985994087673, 1.3750275697224086, 0.96439112186646592, 4.9193296318392736]]], [[[-1.3022655856027729, 18.551649471299712, -1.3735380572247053, -0.82332582580743807], [0.82054691689191661, 1.5463917588059082, -0.86196443676627976, 2.1695139312571881], [-2.0945837420610216, 0.91948335099086331, 7.0465953555885514, 1.2894081474916828]], [[-5.4088927289051929, -2.5166148845707665, 15.313284617276528, 0.80652393773655751], [2.9214673265799043, 4.574804069358624, 1.140477826710794, -10.5821726520746], [2.6006247980526229, -3.1473563102680648, 1.3952211310906615, -1.2432136347874374]]], [[[7.5751916461405404, -0.91287072584226103, 1.1714233453293055, 1.1281938551023989], [1.2242801359057545, 0.92001982129691329, -1.3602525474475136, -5.3805300054272225], [-1.1412548357495409, -0.96436257824365423, 4.4306711280088678, -2.4848368657374182]], [[-0.9750458802988019, 1.0054121970587981, 1.3945384879753819, 1.3033730703072131], [-0.82176887129035325, 1.383846428589409, 1.3553002171645059, 2.6442582021430101], [-1.8786182264789073, 1.3352505054080321, 2.7249262067007898, -0.87135543292087303]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.12990384376321629, -4.1712450338431948])+(1.-msk_arg0)*numpy.array([4.9014301175201922, 2.4196515668966256])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-1.2746094155008736, 3.4729790548595663])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.10191658886512223, -1.2010567780437891])+(1.-msk_ref)*numpy.array([-3.8454369298646003, 0.69670779140200345])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.1294265106576145, -1.8775625595659182], [-1.8752655212147742, 2.649691609465207]])+(1.-msk_arg0)*numpy.array([[2.7294063474283261, 4.5759667945949403], [-3.9795520059266276, -2.2251537084117832]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-3.9758504165671082, 0.54391769203931961])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.28407168085382895, 0.47224175027869209], [-3.447700908907394, 4.8714936988548292]])+(1.-msk_ref)*numpy.array([[-0.68649623639135637, -1.1509403813400993], [-7.3164599426910115, -4.0909750518851071]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-3.8361065947432604, -4.019756096005497])+(1.-msk_arg0)*numpy.array([3.0095804856418074, -4.0602336385581843])
      arg1=Symbol(shape=(2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[2.6723914831686679, 2.5272275800138448], [0.47278454047550866, 4.370927932982692]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-1.435458322219606, -1.5179110203926491], [-8.5023002062685453, -0.91965737199021769]])+(1.-msk_ref)*numpy.array([[1.1261750026509338, 1.1908624729496344], [-8.5879154053441678, -0.92891800112282064]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.1798115319638374, -2.7441926846840401, -3.6830582599376385, -1.7436674941860177, -4.4855074112212723], [0.43916740853270486, 2.453187544038089, 1.8963247868074316, -2.3555001793515631, -2.6471972973552385], [-3.5780283202349317, -0.79904906590157054, 0.45465055573202662, 4.2730874968620327, -3.2502089752054442], [-3.5141311173370058, -2.3058490206358995, -1.1758558035120545, -0.18094630868466322, -4.1193482981851739]], [[2.5771248087523375, 2.7996062242339352, -1.1981490515821323, -4.4297258491865179, 3.1314535702586817], [-3.8307669681934167, 2.8421438004729751, -4.780631671273941, -1.250592704407103, -0.52358411495150925], [4.4021777664421631, 0.8226677733790293, 3.3663383809683296, 2.1315382340123312, -0.0062995411442159366], [3.1548059977237877, -0.67015702806213717, 2.3375762808441012, -2.1726892603250256, 0.79347832134796548]]])+(1.-msk_arg0)*numpy.array([[[-4.4429307803249767, 1.2649100272574572, 4.731018360733362, 3.5282147934734738, -4.7313630369654582], [4.5916275322801638, -2.5826335508008338, 3.2335389829535579, 2.0918600938915608, -0.12576764071637214], [1.2081489314218894, -1.3551021176308877, -3.5375020940557187, 3.9875847346418229, 0.81213070505502216], [1.3839288634582712, 3.8904411979132654, 0.20703686659552467, 1.4574451899631482, 2.0190931167516224]], [[-1.946559417626923, 3.6942365398275605, 0.11521961561775296, -2.8941168196875511, -2.2933337888441976], [3.260765530227097, 0.099371765221226482, 3.5594938952197115, 3.4893620275498254, 4.4791351439113516], [0.074816139858112862, -0.16664594611472072, 0.18850425252290748, 2.5617956396496266, 3.7442304452186939], [-2.820664956470591, 2.8370389287496431, 0.78858694028967147, -3.6933937064677846, -2.8459692601501074]]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([0.45440328921346129, -2.1554685184113085])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.9977740202274008, -6.0391127217279523, -8.1052632042183088, -3.8372686456653473, -9.8712036591666319], [0.96647057571451866, 5.3987011147836901, 4.173219762757066, -5.1837216747016965, -5.8256561081178404], [-7.8741250452395182, -1.7584579268443803, 1.0005441565332709, 9.4037336399092375, -7.1526968496009724], [-7.7335072187080964, -5.0744549508590806, -2.5876921039620435, -0.39820642363277781, -9.0654015848244924]], [[-1.1956216417634398, -1.2988388372739443, 0.55586478825737151, 2.0551104371737492, -1.4527948534208817], [1.77723169485996, -1.3185735612449501, 2.21790837140016, 0.58019530033723454, 0.24290965536226794], [-2.0423298827332421, -0.38166540886682859, -1.5617664337076445, -0.98889787338828072, 0.0029225855494558663], [-1.4636288912486843, 0.31091014428551128, -1.0844863939683125, 1.0079893265740711, -0.36812336370043575]]])+(1.-msk_ref)*numpy.array([[[-9.7775057658921511, 2.7836726918216712, 10.411496732170242, 7.7645009999389627, -10.412255257119947], [10.10474096749593, -5.6835714267631792, 7.1160113927664925, 4.6035320244103355, -0.2767753748747433], [2.6587592125776784, -2.9821573694514183, -7.7849394536269196, 8.7754310527638104, 1.7872465370150836], [3.0455960515905383, 8.5616484085036735, 0.45562360904977223, 3.2073825708565593, 4.4433945895209614]], [[0.90307949339090221, -1.7138902787364314, -0.053454557389070921, 1.3426857293284267, 1.0639606977579532], [-1.5127873603231512, -0.046102164968973243, -1.6513782803208104, -1.6188415640241711, -2.0780332005093283], [-0.034709920009992172, 0.077313096754271962, -0.087453957649005626, -1.1885098844022097, -1.7370842641572817], [1.3086087467190506, -1.3162052261569039, -0.36585407467277731, 1.713499257780859, 1.3203483306950516]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.1431678071793794, 4.6596793828145273])+(1.-msk_arg0)*numpy.array([-4.3812707425969668, 1.6019119773160302])
      arg1=Symbol(shape=(2, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[0.31991231180455237, -0.98112854082606837, -0.53614003039701874, -4.5619375063515744, -1.9154608880954385], [-3.5633374514635552, 0.52482489974264279, 1.7468538352696639, 3.34552821574208, 0.54182527356937715], [-4.0636854925919197, -4.0125335907654556, -1.2775051177744579, 4.7788664600810229, 0.55465665654412533], [3.1745358840571782, -0.22805535598208415, 4.900004291008262, -0.83023686455463164, -4.6654238190780726]], [[-1.8666524510836249, 0.026194181998961952, 4.1475901670454771, -2.6497482373427683, 3.0827937972120836], [4.2593695522934922, -0.6218437791287732, -2.139054045692558, -1.558164244526786, 2.8512645042339821], [-1.1901376646736797, 4.4008908453140556, -3.6929653650712213, -2.5959746100862846, 1.0809497242994572], [0.88757861020434703, -2.1707268077926289, 4.2182207653108232, -2.3321184299682427, 3.7871878613664958]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[12.95094828895054, -4.2228593245193018, -7.7277717989296733, -0.90820354321181673, -2.1630135248018409], [-1.1627211465693974, 7.8943811721034107, 2.3717884825434141, 1.2384196276343085, 7.6466861353392996], [-1.0195591698059201, -1.0325565415114701, -3.2431712010650835, 0.86697710467287981, 7.4697883065066444], [1.3051255233833592, -18.167377781317548, 0.84554370998863959, -4.9903443030103469, -0.88805818460413843]], [[-2.4962758225879171, 177.88986054228323, 1.1234666867131271, -1.7585366478014404, 1.5115118588302909], [1.0939833526080134, -7.493327969514973, -2.1783831933549256, -2.9904930749002463, 1.6342501286342048], [-3.9152440269102406, 1.0588036710285651, -1.2617717530975714, -1.7949633885901717, 4.3107272041115285], [5.2498779592511022, -2.1465987180362265, 1.1046551714727939, -1.9980457780087866, 1.2303797839944539]]])+(1.-msk_ref)*numpy.array([[[-13.695223912712887, 4.4655420368345657, 8.1718776703776026, 0.96039692268842669, 2.2873193443032434], [1.2295413505665778, -8.3480618864413643, -2.5080923510241053, -1.3095901334746765, -8.086132109959566], [1.0781520249497663, 1.0918963401777202, 3.4295523999383897, -0.916801249667622, -7.8990681729038581], [-1.3801295378641414, 19.211435415448676, -0.89413610323501236, 5.277133465938344, 0.93909383423663817]], [[-0.85817366611877421, 61.155258728045496, 0.38622716150789488, -0.60455252115667646, 0.51962994695419296], [0.37609133409273388, -2.5760681879946921, -0.74888803326022635, -1.0280764578849197, 0.56182510424314291], [-1.3459888085764042, 0.36399720729763174, -0.43377389684377293, -0.61707536394694784, 1.4819486432213105], [1.8048113810980893, -0.73796111586468327, 0.37976010892781992, -0.68689135025524584, 0.42298191585828204]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.11150327774122726, 2.028867715536923], [3.0447217654588279, -2.8456032582976833]], [[2.6411506155154632, 0.99245739933543931], [-1.6296887678847316, -3.3619857185563831]], [[-1.403468436060531, -2.3776526554950084], [-1.5866114170149492, -3.2441651632848059]], [[-2.0892976159550205, 3.3489470483135424], [0.38604392313235891, 0.1697614298534198]], [[-0.19104806329309465, -2.1833375621227713], [0.4763863555254213, 4.2180320651239338]], [[0.72078419023658569, -0.21513136032860203], [-3.459091548831168, 4.6389626371262125]]], [[[-0.86510266315851148, -4.4255887922127526], [-4.1000557787250278, -2.8024340253632429]], [[-4.0606203419218527, 0.2125174638444518], [-1.6312496263340135, 0.91434734153787556]], [[0.17919056694061908, -4.6147669960753692], [3.4252589056355305, 2.4362311130330312]], [[1.9328471978014559, 4.988124985501619], [2.5263171283069443, 3.4503650303783946]], [[-3.3526883350022785, -0.95214126038675495], [0.32989598853599844, -0.86844319582130503]], [[-0.40154020153154057, 0.0061105209458425236], [-4.0393914081383482, -0.61626747955194983]]]])+(1.-msk_arg0)*numpy.array([[[[-4.7774276248783494, -4.1816851875308121], [-0.8003992060577314, -0.039657302502584812]], [[1.6720357431120085, -2.4145837552040117], [-4.5823064142439147, 3.9843767339115406]], [[1.845919612979646, -0.34175803091117807], [1.0904154570398896, -3.7880103796314533]], [[2.1357011945215678, -0.51181700224712667], [-0.04525421977881372, 3.1591336198579771]], [[2.4453859410984773, -1.048871137080293], [-1.5764375350841151, 0.8252030625485629]], [[2.9472762074828545, 4.5334368043407309], [-1.4857638893425493, -2.1862232516731961]]], [[[1.5504976295280537, -4.4267503197980762], [4.7360973351126372, -4.8110586499651369]], [[3.3589359198655799, -4.0552133944627533], [0.41451662480171603, -1.7102354877947921]], [[2.3590304133762254, 4.845263445102562], [3.1617202411599319, 0.51681574296086019]], [[0.76477636168260332, -4.569453676470471], [-3.4119269139110964, -2.4257333187818499]], [[-2.2574885007569567, -3.1570960309885354], [-4.0101745138842055, 4.3868776701941226]], [[0.15955097182048661, -2.1195802741723124], [-3.9651310961654662, 4.2341459265510792]]]])
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s1=numpy.array([-4.3802627070554063, -1.7396239833168603])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.025455842536938698, -0.46318402598752162], [-0.69510026431853345, 0.64964214445727053]], [[-0.60296625845324103, -0.22657485765336896], [0.37205274589118781, 0.76753061252265598]], [[0.3204073659326247, 0.54281051491849097], [0.36221832413369903, 0.74063255568195541]], [[0.47697997944044151, -0.76455392570845204], [-0.088132595908128442, -0.038755992780976474]], [[0.043615663276398563, 0.49844899909907481], [-0.10875748496958711, -0.96296326207326255]], [[-0.16455273083863195, 0.049113803147487978], [0.78969956374066708, -1.0590603686062279]]], [[[0.49729290436032064, 2.5439915951116561], [2.3568632175946678, 1.610942394585738]], [[2.3341942746613875, -0.12216287305907028], [0.93770242418926963, -0.52560056098705421]], [[-0.1030053440623213, 2.6527382010890692], [-1.9689650973337056, -1.4004354598445938]], [[-1.1110718272095708, -2.8673581379298949], [-1.4522202226081826, -1.9833970234186722]], [[1.9272488578881646, 0.54732589888267202], [-0.18963637642371489, 0.49921316568968238]], [[0.23082011134724786, -0.0035125527150941418], [2.3219911008795293, 0.35425326706346116]]]])+(1.-msk_ref)*numpy.array([[[[1.0906714835124429, 0.95466538589004257], [0.18272858492448568, 0.0090536356275407251]], [[-0.38172042521988825, 0.55124176714670037], [1.04612593369413, -0.90962049547708601]], [[-0.42141755790271984, 0.078022268016185253], [-0.24893836967438696, 0.86479068333732667]], [[-0.4875737683681704, 0.11684618856826312], [0.010331393983726487, -0.72122012562613569]], [[-0.55827380790645931, 0.23945393398228107], [0.35989565935050083, -0.18839122622015028]], [[-0.6728537543503037, -1.0349691576805662], [0.33919515533837497, 0.49910779281612216]]], [[[-0.89128319935655975, 2.5446592839895188], [-2.7224833530304293, 2.76557388039231]], [[-1.9308401999961238, 2.3310861619249845], [-0.23827943784229533, 0.98310640931379056]], [[-1.3560576515382201, -2.7852360576590369], [-1.8174733571628661, -0.2970847424024769]], [[-0.43962164756112393, 2.6266904344225619], [1.961301376982475, 1.3944009406888129]], [[1.2976876166381128, 1.8148151906764627], [2.3051961529285152, -2.5217390150197092]], [[-0.091715780738017977, 1.218412883760666], [2.2793035358165934, -2.4339431780412855]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank1_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-2.2474735989941017, 0.55283620558228197])+(1.-msk_arg0)*numpy.array([3.9805943311320764, 2.0795485397289042])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[0.27160783866563065, -1.2956702794116182], [-3.1860324995002243, -2.0427101428403027]], [[3.9277110845177674, -3.01360635798705], [2.2464688820170302, -0.47195329315413304]], [[3.088692141277452, -0.91078713206466944], [1.7498478890391382, 0.76142292924280408]], [[2.4048738234931379, 4.1854810888030052], [3.4750412161955531, 4.8876499563754123]], [[4.4975740670161777, 0.39462013746156632], [-2.4903358379316218, 2.9589786191287413]], [[-1.5887643673693876, -2.8880208575326405], [3.2385460532925396, 3.9031279251225932]]], [[[-2.5053343552464922, -0.1307594576223341], [-4.1121847868070347, -2.6198648679247993]], [[3.731028620529532, -1.1514789746078238], [2.8467823000917338, 0.80360846136984954]], [[-2.1836882395718304, 3.0714381297651379], [2.3378510536992767, 0.14933853679595543]], [[-2.15556738686724, -1.2637793006165552], [3.6284805436451677, 0.42593704482114347]], [[3.6979138780717147, -3.247515466759614], [4.636729566676669, 0.5515401984919297]], [[-4.0498632128384351, 1.9585520504419485], [4.3625748402329467, 4.5825130923618609]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-8.2747007966913202, 1.7346030349748476], [0.70541452397194659, 1.1002410728078551]], [[-0.57220950080905453, 0.7457754371394778], [-1.0004472427751456, 4.762067839327714]], [[-0.72764571417097246, 2.4676167678163048], [-1.2843822672085032, -2.9516757542737762]], [[-0.93454948739455757, -0.53696900100840039], [-0.64674732159137316, -0.45982703733979857]], [[-0.4997079682303357, -5.6952836047628015], [0.90247811751396867, -0.75954371027387169]], [[1.4146047363306482, 0.77820545967739807], [-0.69397611212264776, -0.57581346092403851]]], [[[-0.22066364292836679, -4.2278869585021583], [-0.13443856106756807, -0.21101706899111355]], [[0.1481725984465431, -0.48010968308871604], [0.19419686765808103, 0.68794224072741172]], [[-0.25316626960022465, 0.17999262307281294], [0.23647195346662772, 3.7018991711271041]], [[-0.25646899695663783, -0.43744679574398149], [0.15236025078059348, 1.2979293825321654]], [[0.14949948100753488, -0.17023358664213062], [0.11922977124985143, 1.0023497962503838]], [[-0.13650737729356904, 0.28226781384622079], [0.12672245768344512, 0.1206403984974424]]]])+(1.-msk_ref)*numpy.array([[[[14.655668078977952, -3.0722278610417146], [-1.2493891169523508, -1.948682902997303]], [[1.013464138648783, -1.3208740154739154], [1.7719338838818155, -8.4342971833699494]], [[1.2887637061444857, -4.3704990891872182], [2.2748230609449527, 5.2278361712728731]], [[1.6552196178634295, 0.95104821803662154], [1.1454811852533935, 0.81441886523395979]], [[0.8850536471037862, 10.087154590583364], [-1.5984166755750528, 1.3452595788962292]], [[-2.5054655132547974, -1.3783121824587057], [1.2291300681319992, 1.0198472628864836]]], [[[-0.83004830687531272, -15.903618579813619], [-0.50570405940915897, -0.79376175664209703]], [[0.55736601115479012, -1.8059804699752913], [0.7304908912992375, 2.5877633694698257]], [[-0.95231017965121911, 0.67706020823799562], [0.88951284404468367, 13.925063043641828]], [[-0.96473371809135755, -1.6454997630633472], [0.5731182831808137, 4.8822908573311201]], [[0.56235721228134428, -0.64035061911618463], [0.44849467923991965, 3.7704387557153423]], [[-0.51348611803394895, 1.0617785415810894], [0.47667916675049254, 0.45380089435971249]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.2990461800176325, 1.7031035845710036, -0.0068108454520015016, 4.8976439426737066, -3.2432871984462186], [1.2763483208047095, -1.1952335595025518, 4.1954200574907254, -2.9475362293262339, -0.46770358879205354], [4.496018393924059, 3.340601187567211, 4.383278068634084, 1.4534794929121766, -2.3676215490098009], [-1.5325457927437904, 1.6347590611999063, -2.6985829129311911, -2.3326882674369118, 3.5213105293839337]])+(1.-msk_arg0)*numpy.array([[-1.8202096751486074, 0.082981760552115702, 0.63528057642213298, -3.1228200807300732, 2.8917687073309803], [3.4624233682959762, -3.8766925272617545, 1.1316636839968837, -3.9862383999925157, 1.561627560733017], [0.090285798158447506, -4.7759308940714442, -4.0469172259307475, -3.8975052882445205, 3.42856483956181], [1.1407595199339307, 0.85772292797520677, -0.7186085084011431, 1.6060920597145216, 2.0584634696135575]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-0.19835629960435064, -1.758400184845895, 2.5685057460572889, -3.0920556687587064, -4.9272993528704916], [1.1535850620483048, -3.1779427118423911, 3.0818494866541144, 0.66734346194315286, -2.3340804745376618], [4.0335306164897418, -3.0360407826704447, -4.108814431820984, -3.6542029172109922, 3.3104419991274519], [4.293012333881439, -1.5232470247015737, -3.4717636969600809, 2.1297653127228999, -0.29200361487257442]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-6.5490543159393555, -0.96855289213943208, -0.0026516761593608639, -1.583944297044253, 0.65822816236175707], [1.1064189046782786, 0.37610292817695984, 1.3613319130797612, -4.4168204191941536, 0.20038023277012201], [1.1146607826759987, -1.1003149913648, -1.0667987424030392, -0.39775555048309136, -0.71519801574347042], [-0.35698611453980394, -1.0732067975121629, 0.77729452476679317, -1.0952794908912171, -12.059133346416193]])+(1.-msk_ref)*numpy.array([[9.1764651729199933, -0.047191624106538731, 0.24733469154090942, 1.0099495013243784, -0.58688715668276292], [3.0014460850837472, 1.2198748935326993, 0.3672027751184897, -5.9732935546944494, -0.66905472102128205], [0.022383813770829999, 1.5730786362726736, 0.98493550708669886, 1.0665815162829613, 1.0356818939783548], [0.26572472455548163, -0.5630885300059898, 0.20698658408991533, 0.75411692082689463, -7.0494451601629562]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.54720585161075697, -2.3608807796425122], [4.7763754091084429, -3.1158806133009485], [2.8972835546304552, 0.064419778436919373], [-4.400026127959932, -1.9559387940447159], [0.71891828090461907, -0.92841963403549688]], [[-2.6404779717270985, 2.9416087419139121], [-1.5182882966906908, -0.14085933821354679], [4.6279047540044314, 2.2092308851898164], [4.3115285751437344, 1.6216651706422134], [-3.4609661649047139, 1.5398740766989096]], [[1.0167197899179321, -3.3249629804371619], [-3.1866724974334462, 0.13812673554928789], [-0.60989763200794656, -0.65573703853501808], [-3.2086202259930441, -2.3126370454133935], [2.5041475475970376, -3.6904606939800222]], [[-4.7646193800512702, -0.31893906820622497], [0.57030028827590673, -2.6379288983144189], [-3.3672093766819211, -1.6977579911825389], [2.3318095282767706, -4.810000896747689], [1.0546112133823282, -1.9153597348102247]]])+(1.-msk_arg0)*numpy.array([[[1.9047389648794528, -0.60590202698205076], [-1.1515739910404674, 1.0162138578523852], [-1.5532340268008085, 0.062591642771005596], [1.5545305353943251, -1.2391625251756935], [-1.4161444492987449, 0.43828203023116785]], [[4.2931568777290288, -2.9550626681004664], [2.3005084529916324, -3.6588842781231836], [4.9532683202276306, 1.938348167838873], [-4.5807915304369757, 1.6352315759994713], [3.1762207191058707, -4.7214907229492109]], [[4.2070667911903463, 1.468570398089895], [4.1153972443463296, 2.9362789031780956], [3.3906727238734788, 0.88917766629498818], [-1.7078776710718024, -0.26185523026278013], [0.63294583053108688, 1.7683661666414761]], [[-4.9432052989072517, -2.4283921672549278], [0.77745759028259798, 0.24699923503435617], [-4.1407475959034095, -1.463314788317398], [2.1045688048838276, 1.2365574086301612], [1.9499490215573632, 1.564570548056615]]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[1.6602882856095622, 1.386285414303396, 4.2728596407635209, -0.60385403502304769, -4.9085270065445972], [0.97844174221031643, -1.8290384825625527, 1.8855746059642806, -2.7410169282581478, -0.74425288637616305], [4.3390610302999484, -4.2819153864324235, 0.94716939862324168, 3.0617540041534035, 1.7624559296821198], [3.3695398865687043, 2.5287525971240177, 1.9826517450512684, 4.8418113830626908, -1.4620964314594467]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.32958484159264817, -1.4219703891819804], [3.4454487941853995, -2.2476472601904049], [0.6780666341084719, 0.015076502355085118], [7.2865723714042803, 3.2390920331766306], [-0.14646314056051374, 0.18914424486156925]], [[-2.6986562999266712, 3.0064219615863568], [0.83010188750294134, 0.077012779969614129], [2.4543737168319182, 1.1716486201085734], [-1.5729667813046342, -0.59162902422231434], [4.6502556164161915, -2.0690199593268637]], [[0.23431792796139786, -0.76628629033302986], [0.7442165969768253, -0.032258165583316525], [-0.6439161071868067, -0.69231231444783248], [-1.0479680018840214, -0.75533078172714074], [1.420828461820711, -2.0939307654890644]], [[-1.4140267040741916, -0.094653596319647509], [0.22552632824757807, -1.0431739749133897], [-1.6983362736731409, -0.85630670914353513], [0.48159858858479182, -0.99343004429576109], [-0.72130072318802407, 1.3100091714869559]]])+(1.-msk_ref)*numpy.array([[[1.1472338758206333, -0.36493784376705307], [-0.8306904041251344, 0.73304807752235446], [-0.363511595836848, 0.014648654070888469], [-2.5743481789187554, 2.0520895006164817], [0.28850700982404348, -0.0892899294730988]], [[4.3877491040301644, -3.0201723215783187], [-1.2577693006046173, 2.0004413865568029], [2.6269277834777243, 1.0279880529296819], [1.6712014738807037, -0.59657843012251033], [-4.2676632865626987, 6.3439333718120885]], [[0.96957999940819506, 0.33845350130702734], [-0.96111129551655328, -0.68573959039030052], [3.5797954714351965, 0.93877364237849381], [-0.55781021883371151, -0.085524581631170243], [0.35912718149227496, 1.0033534097845058]], [[-1.4670267945517792, -0.72068954486478176], [0.30744707535518118, 0.097676314921147894], [-2.0884896231719887, -0.73805941561338539], [0.43466559070142491, 0.25539148694552738], [-1.3336664939472891, -1.0700871121714455]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.5971423043880701, -4.8258699828176832, -4.679047568789735, 1.5616292509000882, -1.5267297313077743], [-1.3726881486438289, 2.8928776703943591, -4.387866512491355, -1.5503789678627089, 2.8906902825663847], [2.2360827707522386, -4.1177288937496783, 2.7842492350935908, -2.4961533731582151, 3.3502730273345129], [1.4940877285781315, -1.7802361208477691, -3.4148052434380061, 3.120455511372267, -3.2698574350124963]])+(1.-msk_arg0)*numpy.array([[-0.018836435024791598, 2.9869867103356142, 4.5449678990407918, 4.5064112020163911, 1.9122426188799233], [1.1333596711443326, -2.1085434901685352, 3.7156042391400703, -2.1669426623041286, -2.5077506154566898], [1.5455931748405858, -4.8768198277542254, 0.83147419077256934, -4.1315043611531745, 4.5347095322376312], [-1.8289709074422102, -2.7499904854947168, 3.0439587729905355, -4.8335463137361359, -4.5756673053530061]])
      arg1=Symbol(shape=(4, 5, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[3.1036802826050742, 0.22002677521776004], [4.3999888870875843, -0.60368773781185769], [4.9449174368346789, -0.46542951349067696], [3.5610056812771678, -0.99247114631628541], [0.44463195203538586, 2.2911943440729887]], [[3.0924653325393852, -3.8012744542937291], [-3.5890632250945864, 2.4882383428998489], [-0.61915507467247188, -0.99747420506346973], [-4.2992201843760034, 2.0328406937857944], [-2.0441548640350171, 0.66453896947175384]], [[2.375476446573531, -2.9533887657770874], [-3.7266745726163366, 0.29808440635096556], [-0.35149137528664909, -0.14758784192969632], [-2.0351259991611146, -0.24898193378890898], [-2.9349849726441546, 0.51790387595677956]], [[2.0086669322947239, 0.54754368866187164], [0.95545045752602853, 1.5919265494504273], [-4.9921094499585283, 4.9542149949339702], [2.9462643653578873, 3.4782440888256332], [1.7275574555314162, 2.5313570632718738]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.83679440789827697, 11.803755710266097], [-1.096791402582836, 7.9939837776226126], [-0.94623370937106455, 10.053181917273196], [0.43853601781955148, -1.5734757193660729], [-3.4336932474575517, -0.66634667428244077]], [[-0.44388149939797156, 0.36111261240116699], [-0.80602583152268648, 1.1626208070658273], [7.0868619058198004, 4.398977427403401], [0.36061864742285438, -0.76266623971178549], [-1.4141248950484928, 4.3499183875765963]], [[0.94131969777163704, -0.75712442488548803], [1.104933852826006, -13.813969486553587], [-7.9212448180925437, -18.865031148161052], [1.2265350519757192, 10.025439738428956], [-1.1414958027250892, 6.4689089672194342]], [[0.74382054314563184, 2.728709616267325], [-1.8632427320798803, -1.1182903642522646], [0.68404053990971181, -0.68927271968008696], [1.059122714194461, 0.89713528771519679], [-1.8927633489368672, -1.2917408936320043]]])+(1.-msk_ref)*numpy.array([[[-0.0060690642429771261, -0.085609740024363931], [0.6788623305620991, -4.9479002524754341], [0.91911906661683285, -9.7651046341130492], [1.2654883494597937, -4.540596690133162], [4.3007314479453749, 0.83460515858318063]], [[0.36649066336134856, -0.29815254982816258], [0.58749131958046419, -0.84740414686769561], [-6.0010882428858316, -3.7250128577546975], [0.50403156139318384, -1.0659677705824522], [1.2267909147091562, -3.7736697630390528]], [[0.65064554820991916, -0.52332872419317733], [1.3086250845698131, -16.360533204183252], [-2.3655607199308477, -5.6337580379327132], [2.0300975776714534, 16.59359094164612], [-1.5450537479761848, 8.7558903162486938]], [[-0.91053966092465766, -3.3403195860260695], [-2.8782135837951608, -1.7274606585612144], [-0.60975401350942393, 0.61441798066963083], [-1.6405677543973545, -1.3896512695197583], [-2.6486339372981833, -1.8075945791064278]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.7021716224362553, -3.7762894696012319, -1.1323711636387488, 2.1182575599066569, 0.97941251257626849], [3.7144294163491853, -3.0017400950707298, -1.8409683498953591, -0.98631277414415131, -4.0958600553456073], [2.5070723136172894, -2.651263498381724, 4.502745156082776, -1.4171779757071548, 4.598286708700563], [-0.40836836427409473, 0.055066727801794002, 2.1827821973487005, -0.47289590575174323, 4.5379167483020524]], [[-1.6167700610229296, -4.9385238682622408, 1.2440325036662365, 2.6317281485742541, -2.5858437587979379], [-3.991369167009541, -1.6199562351806751, -0.51595267358065389, -0.55931907106788525, -2.2431748312488553], [-0.034091901699873972, -1.3672334231074181, 3.1310748266555173, 0.056359484550171146, 3.2587871316451285], [-2.1675476295998308, -2.4831218376611677, -4.1201351323569879, 4.5729289535052793, 2.8766263586714782]], [[0.66426063377272193, -1.0335940621227979, -3.6099133211196479, -3.4008006489140796, 0.87882752716531165], [1.5839932847745537, -4.8331936047326547, -4.1963280427812952, 1.7200984253334637, -3.6952488922084847], [-3.3474309095732133, 4.3797075234749858, 1.5018073750747369, -1.1339897432892529, 1.4575317711147804], [-4.2374769723471051, 0.95912644982251383, 2.8791149045387474, 3.3870509151801187, -0.52985894911762532]], [[-1.3385398495243015, -2.5606316257287274, 3.6170515542223978, -3.5636700696630363, 1.5724349395212736], [-4.2457210519993671, 3.6318987830079017, -3.4111834956725042, -2.3373248079052242, 0.27553204572924539], [-0.96740212350912813, -3.1295017604827313, -1.5009392136982469, -3.1061803062791249, 3.3480058162551227], [-4.0700952968568371, 2.5480241500807823, -0.80342208482257149, 4.4917843420917496, 2.8146774511290742]], [[2.5237248555662264, 3.2511522237597905, -4.8210740859330379, 1.7677426409409849, -1.6318561195956049], [1.9030879774464928, -4.5247407030821023, -3.7495889616360523, -4.127004043750695, 4.4727344226443702], [-4.9383275400598929, -1.048257586031609, -0.57226772096890066, -2.5313983305113972, -0.71261828162303953], [4.8371412675808951, 3.5866258849650414, 0.89814989823404634, -1.8219651263055758, -2.9151474498789876]]], [[[0.08653314098863607, -3.3677090161886469, 3.4833962066918556, 0.15831020696313569, 1.8013647145744729], [2.0127754954479462, -1.1494190171747167, -3.2392820512113496, 3.9260623054758774, -4.8298784150336296], [0.98325201650285976, -2.6878714728216724, 0.49156415356673921, -3.4770195044538923, -3.813737082816786], [-0.46600037780811565, -2.163145207140742, -3.6609359905407555, 4.0159135094612513, -3.3999636633461017]], [[0.4568840077659182, 3.2099163856092989, -1.2050303140836052, 1.2799151157361512, 1.0686655447026823], [-2.979953132743268, 1.4193852895150227, -1.1097811412655121, -3.4425725677531718, -4.8120702042146659], [-1.7566877581298757, -1.3902819085977125, -4.9271283051463932, 3.9307835302377612, 3.3128663111850614], [-4.2120067361868321, -4.8888621274553561, -4.6626218470252203, 3.0070695361849005, -3.3789693750297034]], [[-0.57027306025545954, -1.65054781883451, 2.4633962841198898, 1.5809500919314594, -1.3268162948430762], [0.22572797733836136, 4.7402438339969315, -3.3389812628450857, -2.7844823322013292, 0.9179438441697032], [1.3706582664164522, -3.0381758541528079, -3.8469543378609639, 0.59860264141534447, -0.83958165496100534], [2.6335691390501452, 3.7011392125121958, 4.066140088857253, -1.0666114445079202, 0.033335959264717552]], [[1.5861202736516811, -1.9876547322832128, 4.1938454084629075, -1.2213035633517499, 1.2441103279717343], [2.506686682428894, -2.5361766305311462, 4.947126916088445, -1.4810550853028959, 3.8408928068921249], [-1.8641186602111781, -0.28638530580061516, 1.7540270689092319, 1.6176241026034184, 3.779353311145746], [1.2324464536012396, 0.25627825482875721, 0.78085079045647987, 2.1434025301020725, -0.96991460053484957]], [[3.4484656559746423, 3.67304805229249, -1.7843427260466838, 2.7121943143610139, -4.5590948559278575], [-3.2935080168161814, 0.35769462188224299, 3.6802496159607792, -2.3179802897229287, 0.60614656285373059], [-1.2415554564936757, 2.4408589440898201, -2.449759325578329, -4.2875397364993244, -0.63541628220199975], [-2.0024450228898161, -1.6568568504114101, -0.3483800179816301, 2.264802199215791, -2.4819619332804299]]], [[[3.0039804469428262, 4.2217960421348462, -2.1995815733041124, 0.14050094916005307, -1.01382924180648], [1.5227755532297866, 4.510562986575529, -2.7284305111379936, 4.508314373845776, -1.0928176665546321], [2.3696476638363908, -1.4587234889308243, -0.18776366152546498, -1.2877318006632397, -0.43582942788889945], [-4.5854752338406186, -3.4903063602546092, -4.7452100122213121, 1.2569688066000477, -4.0262772753837739]], [[0.69581459707983484, 1.8265999264207746, 4.2867926178016518, 2.6192780851334838, 1.1510756119827272], [4.9277144921520417, -2.3217454290052384, -4.757462513246451, -4.6971129563910994, -0.18149349323902264], [1.6780115498479651, 2.7709805814122523, -1.217586333751032, -3.9602030036340441, -4.8278732103160422], [-2.4691091535540535, 0.17950989743085444, 0.011836858759907365, -0.10537586717586933, 3.7811029743184381]], [[2.6692810689909585, -3.4961843801999102, 4.3308161708244981, -0.86561138698126872, -1.5929089115439252], [-0.75234347236414756, -0.048952133253310492, -3.9003506240000929, -0.41034522320494204, -2.8467292170575762], [-1.3605949338435765, 0.40733417266027594, 2.1518825949883302, 4.3962450290805393, 1.3023546938326147], [1.2623346155399053, 1.6156339846399401, -4.2527326801955976, -1.8208808795282092, 2.5407768771191721]], [[-1.6892062478504242, -3.035667944418555, 4.0639912257251449, -2.488392261023471, -1.0133601064878883], [0.84345733903736608, -2.2467253448269964, -0.56509775835796283, -4.8292672527211735, -2.1067317491369431], [-4.50995575231416, -3.5800832227991597, 3.2207954080744869, -2.8176405104297606, -0.16155598750914546], [-1.3658564200783854, 4.6337544397732202, -1.2670054850845416, -3.0045817596660362, -2.9157150016429734]], [[3.3270205733114153, -4.5144113431200159, -4.4745424915213983, -4.7597538639841588, -2.8914760732985312], [-4.9050538983598999, -2.0087183195198488, -3.9910178885343592, -2.0162237174515996, 0.71604600152992681], [0.94272749910977893, -1.5214586603934266, -2.8791650554067427, 3.2115553198855782, -1.7761544325239988], [0.082188612937112993, 4.3402230262853472, 4.4302682187827394, -4.8726328262738292, -1.3399091365773641]]], [[[3.0860795482072536, -0.47563739217334522, -1.5924546254753738, -2.5291122108707675, -3.4771118064767528], [1.4409748696886995, 1.4940137292069577, 0.84026121777299423, -2.9504071152950031, 1.809039687768963], [-1.7534233464393409, 0.66261174156732494, 1.8173944865338942, -0.034948068604593985, -3.0684158165357447], [-2.1423403700099652, -0.43941722751811962, -2.7616905387896051, 0.50777370747998063, -0.51697648362686]], [[1.8628972954677554, 2.91033801577567, -2.6931160742612246, -3.6793089164577966, -0.95018749702785499], [-3.431871677003909, 3.1922011850436931, 1.0845710213837334, 0.17596262118057204, 2.5532033515771708], [0.24502277469587597, 0.83676983325696508, -4.0304455680737536, 3.6618474228273286, -1.2623748934501613], [2.599729287342325, 0.93753626084314501, -2.3422797706627216, -4.6437083811811224, 4.7094548792678665]], [[1.2349530138762237, 4.1748318875188399, -3.7431583715136627, 4.8827963597912589, -4.782680984230697], [2.8286123329856654, 0.48275479844280333, -4.9612973004438761, -4.548710202686598, 4.1680999211302456], [3.707909505041739, 4.9871492246258686, 4.6775671774668428, -0.64224843061358339, 2.4761484908726814], [-0.3421871303122872, -1.3558571164228206, -2.7067548789679243, -1.2106900003195809, 0.75910680864606039]], [[3.7071761191705868, -0.50601752008656131, -3.4022188870420909, 4.002234881051713, -3.0933088955970431], [-1.736047361301666, -0.5856712058504252, 3.922107445664091, -1.0895849448025108, 0.54437372352818159], [-0.64197733325899797, 0.37742488089862114, -1.6837901558786283, 0.93442623963358873, 3.8220389610913212], [1.6978480766480253, -3.4791584283749022, 1.4163780341607568, 4.2538588378147963, -0.46479413553153659]], [[3.1786997405639958, 3.0070010094023516, -2.5869362416410127, -0.30629833760214709, 4.9464208453684826], [4.2862564366221196, -0.48110503774032587, 2.0009133995175894, 0.85742654350912684, 4.7366174612036875], [3.4136097114380277, 0.16043559160274778, 3.9149337500793617, -0.98652269990865449, 2.8683220085980023], [4.7900738937437701, -4.3356877620588481, 2.9326629834944438, -4.7921400835959735, 1.2699907678570819]]]])+(1.-msk_arg0)*numpy.array([[[[-2.1750140655051009, -3.8240883558339664, 2.8336930436395038, -1.6293866730355444, 1.7546142095561166], [-1.1536863100951535, 1.0466363958624303, -4.1813414634180148, 4.8461344884009865, -3.6239071109884025], [-1.0281192899057756, -3.6934035801788259, -2.2850874960114109, 3.6534681302807428, 1.2497542738077874], [2.1160509016058935, -1.6943977242187191, -3.5470825552410634, -1.3726741488056526, 3.3220917258515854]], [[2.9823223006533981, 3.6037440742283966, -1.9280091302852407, 4.373692181065012, -1.6504482882251392], [-0.23493115244745422, 0.48598442030900202, -2.7368801829479539, 2.3813519091051658, 0.9094148117674532], [-0.058267837958442037, 1.1032600832758863, 3.01332557350554, 2.071641073555174, -1.2711395418674396], [1.4217963155317213, -0.5290325119109216, 1.8611128727272614, 1.1890832224533181, 1.8337729733716692]], [[3.6405978485411783, 1.6845697186792572, 2.9166124887926497, 3.6087990300050787, 3.0611870860001886], [3.943135899437074, -2.4844627388286886, -4.4523867608368883, 4.7717057030049936, 2.2114987870183462], [-2.7849660113501762, -2.6910939474401419, 3.2522072779054092, -1.55415459158605, -4.0167629352103118], [1.6337349040296525, -0.12620915150365875, 0.95746934830185015, 2.7973383212475165, 4.0599067015302897]], [[2.8065001579014384, 3.9001743026617195, -4.318579563335458, 1.5218952460993682, -3.0092444987222944], [-2.4410584140256097, -1.4945005707930372, -2.2303903019220064, 4.3727019791592436, 0.9259107543810412], [2.1309129557777231, 2.6995596236358672, -1.9266922773573283, -2.2249806954547982, 1.5256378926744922], [1.5179932844390001, -0.073703987984668373, -1.0724985617720106, 4.8830658377267984, -2.1965680724826697]], [[2.6519615533388539, -1.0619697941700545, -0.50865757354428087, 0.43141756786093133, 1.6413344246816859], [-3.6832916938497107, -0.30401019627133596, 3.3946234412394833, 0.79156817713364003, 1.2794392561797094], [-4.4026509675911694, -4.0760768117644419, 2.0297896995447058, -3.9765342689820993, -3.1981407443974974], [1.49912785403858, 0.73054929946724112, 0.76348046278169868, 0.35406891866243217, 1.22173911782847]]], [[[-2.2150124233156321, -3.2641039974226804, 3.2901128243941571, -3.8002859256388275, -2.1088332566072698], [-1.2705930256428246, -3.512144099092549, 4.2425074871741959, 2.5717857681450678, 3.0071302807125484], [2.6493808751747601, -2.4430979145369269, -1.7105991831594034, 1.3096334754290666, 4.6698534831710745], [4.4802639987433786, 3.2517764686548905, -2.994209839064208, 2.2337768906094695, 4.4366201280333843]], [[0.95323452797225094, -2.2718799563935219, 1.1988858949478951, -0.41231721766792084, 0.86856335897914061], [-0.26817092919407415, -4.1510456543442125, 0.86199286658411811, -1.1672482071226495, 1.8362849613309198], [0.85598681621738493, -1.0248963070175297, -3.506776055326819, -1.4900872348952454, -4.4191655935569809], [0.97881389459646684, -0.57792929225417478, -4.4406342439192761, 2.6065205134483405, 3.740919578309617]], [[3.0930617659916493, 4.8171890642468593, -1.8605923467612029, -2.9679453457716365, -1.2180818321312694], [2.6678397023177167, 4.0927634087446094, -3.010276955149358, 0.48947217812017474, -1.0802330271802649], [1.1023799389592419, -1.411663775735347, -4.3658115333943144, -0.26858764472189289, -1.1968997376297361], [3.7237217571151913, 0.84804333287388101, 1.8004624956764017, -1.0943799866232307, -0.11309696782280465]], [[3.9434293079503533, -0.41707050035036453, -1.5078714586854947, -3.6066111113007104, -3.9863199069585207], [-3.3674862602241937, -4.2477655023219674, -2.4520716128264497, 2.0300316454844314, -1.7079355862466405], [-0.78649209489923777, 4.8640406984735947, -1.8749382854838781, -0.20924987822377172, 4.2643102811518041], [-2.9995687843399299, 4.8674355943236041, -1.0933410253071862, 4.7669092826225032, -3.4742382126029927]], [[-0.59585212031907275, 1.4442554088271331, 0.72153583750138495, 0.51455663608509195, -4.2661437144239684], [2.20398701392552, -4.5930613828431488, 2.3485332572905921, -3.7935427203238001, 0.36317730387689817], [4.906118979386612, 1.9007022586232072, 1.9466336845040475, -0.87404987813651402, 0.10750896782015129], [2.2992806546279256, -2.0215587406356605, -3.3882637775678317, -4.4452818823421492, 2.9029148485150245]]], [[[-3.879716385296005, -4.9385128956006241, 0.20569593743397441, 4.5641676427303821, 1.1491955731993677], [-3.5720276496433758, -1.4684904382427577, -3.764904469733501, 3.6537338281636256, -2.3066406337874499], [0.63495979300139194, -2.7611561890483562, 2.6711556979075919, 2.9848545659047456, -1.8170901142443974], [0.44456994530778093, 2.3482695494772541, -3.9620738630797261, -3.1839789102978342, -4.4705387042103588]], [[-4.982005412151512, -3.7246641631914503, -2.7536147267987374, -0.42987880878525697, 0.92950837653585161], [3.674720326188309, -1.7301595962118723, -4.9986679139099532, 2.7496648070385445, 3.9365576592645937], [4.9652909860519188, 0.48263564443350315, 4.2092557915301416, 1.2413385590288613, 0.12308447811370904], [-4.1099043762741267, 3.3778758627631653, -0.31488703469009849, 3.2836765904681844, 4.1887502763628266]], [[3.4993814976239008, 4.7379089992785044, 1.1285393397497048, -1.9956058056120938, -3.7160671567115222], [1.741526768593169, -0.73748792551085174, 1.8520278561217047, -3.8767208522604957, -0.38764175956516667], [2.2612785252565306, 4.7109112914831215, -1.978938403211469, -4.5874525054875628, -4.0075787900438895], [-1.0528946919281013, 0.22539831963468693, 0.12946678049540061, -3.4556515938177745, 0.35187402023976766]], [[2.1917568174323847, -3.4079077854620419, -2.2061505345773424, 1.6186224408440939, -2.5690737507886405], [2.1374938182262104, -1.415316876626008, 4.2767433614667087, -3.6186283685393192, -0.051510888629166196], [3.0384899657103794, 4.9871718410434607, 3.8234427841538334, 0.11965407710069265, -0.94528481860510905], [1.726772206322015, 2.553254731393011, 2.2320663380986074, 4.1154944117038976, 2.730222747701978]], [[-4.7213983523229164, 0.15046119197002028, -3.9627670833001059, -4.6026607472533918, -3.748496947977241], [-2.3054558141436186, 1.0375676603055242, 4.8226131418668885, 4.0221609939276455, 2.4072629820881817], [1.4656644487309558, 0.80987982534120384, 4.4894035048400465, -0.78083428663149324, 0.13009194371924337], [2.1199775571472221, 1.8548728603944031, -2.6472777300640828, -4.3568908268275361, -4.8848683875696519]]], [[[-2.2577857508406329, 1.9333975687261686, 3.939060310450742, 1.2893341955539395, -3.3060973176568664], [0.51965484888247282, 4.5505009063751949, 4.5511158977766382, -1.3933069490671315, -0.98242788434686545], [1.9063523539762741, 1.1379452730309492, -3.9606080836883093, -0.0023400273930009874, 0.95261297313364857], [2.1824930288562978, -3.3006203560452283, 4.7941717970761353, 0.63897241320786335, 1.7047778196007588]], [[-3.4135233337765083, -0.085789300758712272, 3.087068271600085, 1.6528216506702078, -4.0223437058494582], [-1.0826645990694717, 4.8103526634689882, 0.36365797191358329, -2.3400561691267874, -0.16686426255298059], [-3.3178855292751108, -3.9797011767674384, 2.2361485050980265, -2.5046567610952364, -3.7075557052965546], [-3.2220616692517021, -4.2195203057242869, -4.7802973834844167, 0.069285238075917022, 0.87817053512230814]], [[-3.0473165154299231, 1.602932056041845, 4.1536533468153731, -3.3344331104157021, -0.061797229048822899], [2.799743972702351, 0.83608501388008793, -4.8736947923372655, 2.3479975756401572, -2.3447556653524226], [-3.5139873578100844, 4.1601300660767127, 1.1150774117905105, -0.97684116589813641, 3.0390228818753169], [-2.1130139170534115, -1.3819008483767203, -2.3770661206672203, -3.5719489787489467, 0.40707535400693118]], [[-1.1223423857444703, 3.0940833438482453, 1.634475456051856, 1.3528758414833906, -4.4876625236300232], [1.5532081397363084, 4.1826639085913229, -0.75101808157671002, -1.0924762364728604, -2.4698562528895565], [0.29186328560850683, 4.4346562350365684, 2.5896132839206043, -3.2785669550029106, 4.9080585971519639], [-0.1461630942378882, -1.5219384145671766, 2.9955270820372899, -3.4152099625184675, -0.8641681457705408]], [[-0.057502789534878751, -4.034306639579393, 2.5018325053149226, -4.0396665014738966, -0.90822921785515209], [3.8001388442209798, -1.0894363316691136, -2.8828025670039348, 2.0681597051929801, -4.9656792762870818], [1.9905991497509588, 0.91438462613015226, -3.1800212422008345, -3.3957552432778426, 1.234494591858847], [-3.552249643660276, 3.5374444632485886, 0.9533829888865295, 3.507701447585319, -1.8392045755033348]]]])
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[-4.093062582437403, 3.0513905963779404, -0.87074417855256314, -1.1364762008833273, 3.5188122859724391], [2.0975054505706314, -3.2034620250066004, 3.8810737734748599, 0.94617909860657257, 0.31336647849653509], [-4.3476526823996675, 0.76868281435698815, -3.8703544665498733, 2.1149451035061837, -3.9989507256289425], [2.4224137923441926, 1.0646543442124576, 4.8840808611276358, 2.2493171250162973, -3.8609395028482254]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.17155164581385776, 0.92260731262810702, 0.27665620567287447, -0.51752386318150134, -0.23928598521282129], [-0.90749392210300805, 0.73337263592075475, 0.44977771847286768, 0.2409718283752226, 1.0006834669277249], [-0.61251746415378228, 0.64774565376982507, -1.1000919398113402, 0.34623901960063136, -1.123434253957168], [0.099770857652245554, -0.013453673549501893, -0.53328825381625666, 0.11553595778888276, -1.1086849167108852]], [[-0.52984696975276357, -1.6184502482652874, 0.40769362832242034, 0.8624684600189062, -0.84743125375931372], [-1.3080492453989252, -0.53089114094524459, -0.16908771829902722, -0.18329972955012963, -0.73513198667897417], [-0.011172578738474753, -0.448068963943964, 1.0261140708672838, 0.018470098392867482, 1.0679678752085595], [-0.71034748293868111, -0.81376728387663033, -1.3502483547165898, 1.4986376896269635, 0.94272636288716671]], [[-0.76286543181594557, 1.1870237982422576, 4.1457794493905222, 3.9056254783893314, -1.009283264604979], [-1.819125896893877, 5.5506470485589299, 4.819243293428439, -1.9754348839779565, 4.2437824831066822], [3.8443333783036517, -5.0298441624443218, -1.7247400695473949, 1.3023225089764969, -1.6738920649893214], [4.8665004908686926, -1.1015019950140448, -3.3064991710018621, -3.8898346938252391, 0.60851276662958476]], [[1.1777983986676712, 2.2531326425828131, -3.1826901006911017, 3.135719047080062, -1.383605691257852], [3.7358644630651976, -3.1957543679181355, 3.0015441529010096, 2.0566421066173985, -0.24244418450213723], [0.85122954863217881, 2.7536887776887213, 1.3206956841961497, 2.7331679307185164, -2.9459533016642863], [3.5813291063141941, -2.2420391628969685, 0.7069414073062954, -3.9523787111428339, -2.4766708259630628]], [[0.71720928838032183, 0.92393454368689643, -1.3700856124528147, 0.50236912266903189, -0.46375196713417022], [0.54083248061655731, -1.2858715769294498, -1.0655836847516966, -1.1728400688501572, 1.271092078561477], [-1.4034074962584042, -0.29790096795172422, -0.16263093182043725, -0.7193899886625621, -0.20251670839727795], [1.3746516933750361, 1.0192717296296077, 0.25524234464409307, -0.51777843722120231, -0.8284464225329865]]], [[[0.04125526394465126, -1.6055781954094344, 1.6607328508939938, 0.075475468690709244, 0.85881288846444104], [0.95960441719012735, -0.54799333983233012, -1.5443497657324776, 1.8717769264474535, -2.302677408404183], [0.46877209126458463, -1.2814610193696663, 0.23435655599035893, -1.6576927146997213, -1.8182251120154418], [-0.22216885190040347, -1.0312942007145931, -1.7453761512489938, 1.9146140995098306, -1.6209558179794639]], [[-0.14262195218779808, -1.0020148079022992, 0.37616500669494352, -0.39954121689128314, -0.33359706978280174], [0.93022895526196481, -0.44307854391128554, 0.34643180802595142, 1.074641291477797, 1.5021467920178486], [0.54837165055086179, 0.43399356625582225, 1.538063590791666, -1.2270423371819625, -1.0341518910867176], [1.3148296134954662, 1.5261183336316537, 1.4554946525441061, -0.93869367350427846, 1.0547867740129497]], [[-0.14693692868014571, -0.42528122761158371, 0.63472029337755298, 0.40734863190090304, -0.34186835197804849], [0.058161217877664637, 1.2213743182090628, -0.86032409011786959, -0.71745153396254191, 0.2365180096403672], [0.35316470297065605, -0.78281837230644125, -0.99120876396447688, 0.15423634704047245, -0.21632715685517587], [0.67856714217834091, 0.95363794365558718, 1.047684307535514, -0.27482380051563565, 0.0085893650083519821]], [[1.6763425402099272, -2.1007172270137966, 4.4324012384538367, -1.2907741939663961, 1.3148782612128314], [2.649272940102422, -2.680440346088965, 5.2285311769981222, -1.5653009958516619, 4.0593718594593398], [-1.970154131449791, -0.30267557825190983, 1.8538002704692673, 1.7096383813441614, 3.9943318518783153], [1.3025509181255956, 0.27085596712733917, 0.82526742728351343, 2.2653243273484245, -1.025085633325902]], [[11.004577364240228, 11.721253881126609, -5.6941084911429458, 8.6550237516582449, -14.548763728020345], [-10.510084016062356, 1.1414578342852266, 11.7442351639455, -7.397026959756829, 1.9343056914124679], [-3.9619919222068409, 7.7891513980708336, -7.8175538664242161, -13.682190121515285, -2.0277098088174279], [-6.3901060269659862, -5.2872817104134828, -1.1117335193383875, 7.2273276008391978, -7.9203172757607927]]], [[[-0.69094305971211867, -0.97105181819736452, 0.50592393964875393, -0.03231650718762203, 0.23319002594450611], [-0.35025234637402025, -1.0374708644127351, 0.6275640467287853, -1.0369536628573173, 0.25135808823428285], [-0.54504070056683418, 0.33551978400576571, 0.043187364594595365, 0.29619012711762471, 0.10024476648128786], [1.0547013684887281, 0.80280248106850849, 1.0914418328379936, -0.2891143562797821, 0.92608070825967825]], [[0.90520379028102982, 2.3762726215607488, 5.5768029904344907, 3.4074888058015702, 1.4974650017974123], [6.4105953718688644, -3.0204206281721087, -6.1891100261245233, -6.1105996760449068, -0.23610973193259693], [2.18297003459306, 3.6048426342537772, -1.5839905758392123, -5.1519338401585033, -6.2807091821801855], [-3.2121300326188393, 0.23352921917607394, 0.01539888565065557, -0.13708627955214198, 4.9189378293586197]], [[-0.68967354077271881, 0.90332407804406933, -1.1189714555228043, 0.22365170799275536, 0.41156667310730388], [0.19438619352991834, 0.012647971568595782, 1.0077502352069987, 0.10602264644011627, 0.73552157603673418], [0.35154271930458181, -0.10524466846143511, -0.555991089081453, -1.1358765888436719, -0.33649493995664043], [-0.32615478154515926, -0.41743824722084305, 1.0987966908329678, 0.47046876333045179, -0.656471364335805]], [[-0.79869980788155492, -1.4353412480475189, 1.9215587293437579, -1.1765753432078128, -0.4791425105114675], [0.39880814761530758, -1.0623090599856921, -0.26719263654698949, -2.2834007581166769, -0.99611651652063005], [-2.13242213466321, -1.6927546804236435, 1.5228742357118443, -1.3322523150878192, -0.076387792402420204], [-0.64581176022680242, 2.1909573123630119, -0.5990725163428986, -1.4206429068466133, -1.3786244365441271]], [[-0.83197338541553345, 1.1288989669683922, 1.118929138797442, 1.1902506908823087, 0.72305869006269607], [1.2265852307015981, 0.50231134548523948, 0.99801627035718588, 0.50418818729868053, -0.179058470748551], [-0.23574371473694758, 0.38046446800219985, 0.71998012802568767, -0.8030994979015339, 0.44415511827659515], [-0.020552544548842026, -1.0853404615538818, -1.1078576663597075, 1.2184778359597008, 0.33506517797031055]]], [[[1.2739687818656389, -0.1963485320619259, -0.6573834043168737, -1.0440463222525338, -1.4353913511662757], [0.59485083607217015, 0.61674588128941699, 0.34686939961643198, -1.2179616565177604, 0.7467921845087987], [-0.72383312544738143, 0.27353367276121282, 0.75024114058365909, -0.01442696070962113, -1.2666769922765382], [-0.88438250177596711, -0.18139643561593638, -1.140057304626346, 0.20961476898981954, -0.21341377978473983]], [[1.7497672419171606, 2.7335989672108041, -2.5295684828613241, -3.455871792059837, -0.89248449714515032], [-3.2234609248154817, 2.9983451459121384, 1.0187071769157232, 0.16527676060978691, 2.3981523819975745], [0.23014302813663279, 0.78595446287868909, -3.7856846120842542, 3.4394707002638096, -1.1857133729012874], [2.4418528900714604, 0.88060154541205204, -2.20003777131567, -4.3617051923233827, 4.4234590361358341]], [[0.25285269613474376, 0.85478353168685162, -0.7663997542107529, 0.99973700244264996, -0.97923869817472109], [0.57914936574833775, 0.09884250735589685, -1.0158098200074461, -0.93133392587533703, 0.85340518301081436], [0.75918266107200716, 1.0211029191425869, 0.95771698103845204, -0.13149832053871877, 0.50698351671044783], [-0.07006172502911491, -0.27760742603876148, -0.55419944016712885, -0.2478849213892943, 0.15542470123453236]], [[1.6481340394115065, -0.22496495245546799, -1.5125563439692571, 1.7793110791448383, -1.3752213332633703], [-0.77181084961022894, -0.26037733823156739, 1.7436880740574425, -0.48440699298664536, 0.24201732938135051], [-0.28540988112307486, 0.16779531738811018, -0.74857837392156457, 0.41542663292834636, 1.6991996898008006], [0.75482823554091938, -1.5467620771124899, 0.62969246017299341, 1.8911779003967595, -0.20663788594423693]], [[-0.82329695614734721, -0.7788262435047425, 0.67002765511674633, 0.079332591815072462, -1.2811443540411589], [-1.1101589220603267, 0.12460828184057621, -0.51824520898126225, -0.22207717652053366, -1.2268043717622283], [-0.88413965277616979, -0.041553510870707508, -1.0139847431412237, 0.25551363837244639, -0.74290778358014509], [-1.2406498185765718, 1.1229618487573814, -0.75957237385641774, 1.2411849706686156, -0.3289330917825069]]]])+(1.-msk_ref)*numpy.array([[[[0.53139037620330942, 0.93428533740051845, -0.69231608033514369, 0.39808496455122633, -0.4286800346236706], [0.28186383346431459, -0.25570984434818056, 1.0215679284649595, -1.1839873910540435, 0.88537788953873742], [0.25118582215606744, 0.90235697739549792, 0.55828305822058799, -0.89260011463228506, -0.30533475817600703], [-0.51698474161755748, 0.41396819376500027, 0.86660843410041133, 0.33536602999806331, -0.81163961189014922]], [[0.97736497719874749, 1.1810169692815173, -0.63184606145598821, 1.4333439272758686, -0.54088397931888932], [-0.076991504373881872, 0.15926653929060242, -0.89692882523682271, 0.78041530046395124, 0.29803290763461948], [-0.019095502892224642, 0.36155977035043541, 0.98752535223855498, 0.67891704064836933, -0.41657713154661574], [0.46595028418171736, -0.17337423551704373, 0.60992285777390753, 0.38968568096945139, 0.60096304142393087]], [[-4.1810188781197928, -1.9346321918333294, -3.3495630067156243, -4.1444997496325273, -3.5155986814506135], [-4.5284665652221108, 2.8532636795328425, 5.1133121191095237, -5.4800317022354301, -2.5397801575825838], [3.1983745397868999, 3.0905678312010578, -3.7349744712753048, 1.7848578605136574, 4.6130230142765596], [-1.8762513080999379, 0.14494400836931928, -1.0995989084801578, -3.2125834316774049, -4.6625711678934989]], [[-2.469475520666498, -3.4318134419623614, 3.7999736025961983, -1.3391351661534783, 2.6478728691224296], [2.1479186384442492, 1.3150302396402451, 1.9625490619059449, -3.8475966111393767, -0.81472076024238438], [-1.8750176678767834, -2.3753771715920067, 1.6953212710128067, 1.9577890797232969, -1.3424283689255336], [-1.3357017799925226, 0.064853085288879675, 0.94370525395816474, -4.2966723226860628, 1.9327884479898345]], [[0.75365246504076377, -0.30179779649046401, -0.14455376763688635, 0.12260317766331409, 0.46644557631698047], [-1.0467428764338922, -0.086395684556194341, 0.96470716973791748, 0.22495322648758079, 0.36359974678959917], [-1.2511752858037091, -1.1583672218076277, 0.57683943745347144, -1.1300785452052515, -0.90886938105414661], [0.42603234620237479, 0.20761246696208766, 0.21697106885333825, 0.10062171263693416, 0.3472021291669547]]], [[[-1.0560222490544817, -1.55618379753658, 1.5685836828214554, -1.8118121812770263, -1.0054006086294349], [-0.60576387312707902, -1.6744386042653963, 2.0226443206714966, 1.2261163695404973, 1.4336698290316487], [1.2631103649594759, -1.1647635594330763, -0.81553980357668709, 0.62437667328720292, 2.2263844329465887], [2.135996355825684, 1.5503065642905656, -1.4275099205342356, 1.0649683365551041, 2.1151888434074824]], [[-0.29756386076413283, 0.70919522025201498, -0.37424695082671533, 0.12870988151235266, -0.27113271585523196], [0.083712847881666899, 1.2957998633792638, -0.26908165598820921, 0.3643708581562613, -0.57321889474470555], [-0.26720679362997013, 0.31993396488457454, 1.0946831983499457, 0.46514902416930487, 1.3794967941122622], [-0.30554877409369324, 0.18040772381342152, 1.3861984968934122, -0.81365737851784592, -1.1677739736283932]], [[0.79696031215153218, 1.2412000764246909, -0.47940143768391963, -0.76472273370736066, -0.3138517593909787], [0.68739731786368685, 1.0545440895034099, -0.77562992379146467, 0.12611772068479216, -0.27833354639200658], [0.28403993412684936, -0.36373020924862126, -1.1248978474030531, -0.069204467731984659, -0.30839396710516798], [0.95945657682801888, 0.21850739830554636, 0.4639083410322255, -0.28197866118978571, -0.02914063849952149]], [[4.1677408788228334, -0.44079445526177824, -1.5936427478752386, -3.8117636678004474, -4.2130711963825132], [-3.5590368305360509, -4.4893884345760799, -2.5915512363754263, 2.1455046391048338, -1.8050869954344777], [-0.83122962244409748, 5.1407188191292885, -1.9815892025569777, -0.22115250541037282, 4.5068743194938534], [-3.1701913398397421, 5.1443068246718004, -1.1555328445928867, 5.0380623389828392, -3.6718610860454057]], [[-1.9014545626508712, 4.6088382387176816, 2.3025303822002869, 1.6420283323022422, -13.613912167287349], [7.0332571132042432, -14.657156071318376, 7.4945261170191184, -12.105770657169208, 1.1589539047678128], [15.656170382119731, 6.0654294222610137, 6.2120035743566984, -2.7892258365668758, 0.34307743551880909], [7.3373535856782741, -6.4511008016385931, -10.812464031966636, -14.185569253194071, 9.2636419263559553]]], [[[0.89237035906801387, 1.1359032692728421, -0.047311952554692442, -1.0498004270688914, -0.26432552394343384], [0.82159912729546991, 0.33776627194429681, 0.86596256526532811, -0.84039229788405345, 0.53054850566267175], [-0.14604657717297895, 0.63509125285609258, -0.61439031427718815, -0.68654393162272298, 0.41794739529227126], [-0.10225516566847803, -0.54012353815280778, 0.91131333446186802, 0.73234435749372029, 1.0282649123073155]], [[-6.481223879473637, -4.8455150728290626, -3.5822509302515981, -0.55924082177491563, 1.2092222685027709], [4.7805418015781376, -2.2508108206623176, -6.5029005729644096, 3.5771123741574349, 5.1211729802461745], [6.4594796362208786, 0.62787359808119725, 5.475933262604852, 1.6148904799793853, 0.16012388440955402], [-5.3466843534314057, 4.3943689122135465, -0.40964495212957902, 4.2718225634002458, 5.4492570903471593]], [[-0.90415013091638952, -1.2241537668517453, -0.29158552517689984, 0.51561318810962153, 0.96013612934634218], [-0.44996570304983141, 0.19054790249438475, -0.47851634059054204, 1.0016449102441767, 0.10015665565400263], [-0.58425618242462651, -1.2171782538777489, 0.51130676022435284, 1.1852796804880066, 1.0354552340567249], [0.27204089471078263, -0.058237125716191156, -0.033450884567379276, 0.89285144905557579, -0.090915192208076118]], [[1.0363185379132827, -1.6113457412262699, -1.0431242545822854, 0.76532598324217516, -1.2147236098608878], [1.0106616075673289, -0.66919792588454285, 2.0221533667122933, -1.7109798086675203, -0.024355662255143535], [1.4366755717078099, 2.3580620758315014, 1.8078212894581898, 0.056575500187843436, -0.44695477770936159], [0.81646195140448297, 1.2072439739264116, 1.0553779076337531, 1.9459107495892811, 1.2909189667267387]], [[1.1806592969660434, -0.037625167773567954, 0.99095171588463471, 1.1509671068851441, 0.93737012660682106], [0.57651518418772807, -0.25945997625223022, -1.2059696337239572, -1.0058040895952907, -0.6019736544039499], [-0.36651225516174391, -0.20252308190514862, -1.1226453669628467, 0.1952597919317163, -0.032531519552240276], [-0.5301334531482127, -0.46383988892553174, 0.66199308561065784, 1.0895085050447322, 1.2215375288980024]]], [[[-0.93203966967829732, 0.79812853395918026, 1.6260889542900416, 0.53225183890083394, -1.3647946226633414], [0.21451943946356, 1.8784985953913482, 1.8787524708454044, -0.57517297559588909, -0.40555741857635347], [0.78696396131871382, 0.46975676766179109, -1.6349841204691917, -0.00096598995613235872, 0.39324948369444185], [0.9009579766074064, -1.3625336705382556, 1.9790887140040472, 0.26377508880905259, 0.70375169799170822]], [[-3.2062268400375031, -0.080579486877660783, 2.899596745536825, 1.5524490738755472, -3.7780747598647637], [-1.0169165278429748, 4.5182294982577513, 0.34157374540427682, -2.1979492046855502, -0.15673092723480392], [-3.1163969294930229, -3.7380218268975258, 2.1003516467611303, -2.3525539295553921, -3.4824032095027939], [-3.0263922622089279, -3.9632772163678491, -4.4899994157450998, 0.065077683149049145, 0.82484098233019032]], [[-0.62392835050776763, 0.32819523296585229, 0.85044729293371657, -0.68271455883427956, -0.012652785817013514], [0.57323866092912035, 0.17118574357244626, -0.99787348549590305, 0.48074502499085403, -0.4800812541852687], [-0.71947771908894553, 0.85177338057343766, 0.22830854842420883, -0.20000511737485924, 0.62223025545356503], [-0.43263286934310896, -0.2829397972042722, -0.48669671699874428, -0.73134517636226348, 0.083347382154714308]], [[-0.49897027558367896, 1.3755656369823028, 0.72665407552970707, 0.60146069508699818, -1.995122196741159], [0.6905243028926189, 1.8595261033105848, -0.33388714878133002, -0.48569240162831417, -1.0980471474744413], [0.12975639689151974, 1.9715566941253058, 1.1512886533960129, -1.457583245394602, 2.1820216200578666], [-0.064981096979301739, -0.67662242804297745, 1.3317495557748826, -1.518331908175786, -0.38419133352051438]], [[0.014893470745257414, 1.0449028368880875, -0.64798541999151094, 1.0462910642587959, 0.23523528850559533], [-0.9842523669219907, 0.28216871330551374, 0.74665831072392708, -0.53566229247241326, 1.2861323707931416], [-0.51557377376218627, -0.23682956582345002, 0.82363923077658285, 0.87951526843991856, -0.3197394289519836], [0.92004799376933311, -0.91621338812457598, -0.24693031014425798, -0.90850981865882074, 0.47636192541907191]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank2_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.1612648395677176, 4.8164570954472765, 2.1207136307822267, 4.1980162825965994, 0.42455819800470884], [-3.5821115586025352, 3.1363624706319762, -0.045544223962404651, -4.3303885146188561, 4.1114943523937093], [4.6790875857057497, -0.80180395586374864, -2.0149586188960402, -4.5530723019272958, 0.81679404525573851], [-2.554862516894445, -1.9856505757990019, 2.9104571292418484, 2.4406022096017201, -4.6463793319695998]])+(1.-msk_arg0)*numpy.array([[2.3656309860279112, -1.8917945872147222, 1.9656431422527545, -4.2914770863552976, 4.8097106904028983], [0.40664603295705071, -2.8909929294405612, 4.1268464537840259, -4.6622028459403131, 4.661298649626044], [-3.1919970829403201, 4.68387313440887, 2.1822537995819227, -2.9339237014858588, -1.5302491956021602], [0.8191280232969147, -3.7650026638769174, -1.697977300574991, 2.7298032753800943, -2.9159584523495976]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[0.8032783337903453, -0.33365977693299165, -0.14497708469277804, -3.197324895305381, -2.9146887263278884], [4.3019746633289575, -2.3309567056939127, -1.4913477554910939, 3.3301258072795967, 1.9144251873249605], [-1.5646812968823154, -4.9507028379904696, 0.97754739335913499, 4.5477578588620222, -2.8440233977440519], [2.4118130629473544, 4.4203539407547172, 0.73398956283840544, 0.66831534954551586, 0.91704124930513498]], [[-4.2052922516547584, -0.23722072701483121, 0.80278058040632239, -3.2579452357350291, -4.4484143285119071], [-4.1811166690150028, -4.0377493942397189, 4.3036350320618979, 0.52874167425302065, 3.0696116531475308], [1.0731070775036464, 3.2865970829455708, 1.0081547639410049, -4.5683356934052988, 1.2327057813805666], [4.957186393778036, 1.7702194088091243, -3.5817352313573636, -2.2289298377419198, 3.7182063358898887]], [[1.5677375020795212, 4.8162612404763774, -0.48002532400498676, 4.2989622458952006, 3.2908353185154979], [-1.8395170489692503, -1.0978029223474097, 1.0807323653436889, 4.1166009604709952, -2.2277078308183329], [3.4822454174419644, -0.3408676631412213, -1.8037375057975535, -0.25814619185105592, -2.3317434148647576], [-0.53686529563005259, -1.3078129664854621, -4.0864277402856954, -3.2420011690433723, 4.1557780758502929]], [[0.68542902212319756, -3.9286795345239724, 0.24503678026949949, 4.2089376264665717, 2.9120263147696459], [2.0701170709241543, 1.481357429354011, 1.7806010864830002, -4.4545471704939033, 1.0155559103306357], [-1.8439312201146296, -3.9936532486951126, -2.9388410587229377, 0.23716086396366087, 3.1701623359677598], [-4.9028284764883532, -3.8140201018316833, 2.8929240469275062, 2.9560936646407301, -1.9130144395557345]], [[-3.2117677326732141, 1.7632630547085117, -4.978718949862186, -4.5512158810069057, 0.088085318737397778], [1.2900259741525666, -3.6219606086309621, 0.4463091904444525, -0.80795531021689904, 2.3053256643820284], [-1.9722492129013802, -2.3882739757746032, 4.590369481856607, 2.4914265566372045, 1.749870479036467], [-3.486672684082742, 1.5198282064889295, 2.3407269359269236, -3.714016097970485, 3.22853917190678]]], [[[2.2428330766371642, 4.5537763295905105, -1.5944547269729648, -0.45079290319667642, -1.5722229193143953], [4.6938910016609814, -3.5302382421115732, 4.9973896944218978, -4.7573882659551883, -3.4715737495329471], [-1.4435492102473537, 4.0512974966696813, -1.7933529428223616, 3.7892649005247794, -0.13673779335000624], [-4.4070355102323227, -2.3858454023867415, 4.2011864763750957, 2.6266538923662894, 4.0603016955357933]], [[-4.998438662090253, -3.1322435498061263, -0.05175300738141253, 3.7333543632734987, -3.4266212841021537], [-3.1670869521257727, -4.7104761139704987, 3.1126409897583471, 2.9193954450837998, -3.4367555775250311], [4.637007348044321, 2.4282645610235338, -3.9618207024344709, 3.625365480848922, 0.93880111481744599], [-0.81134033306565634, -3.5881509092307464, 0.67089896443956221, 2.7515260649364155, 0.3673797860616137]], [[-3.4873849911123611, 3.9951554129745208, 4.5813854607813216, -3.9001872861658526, -0.38669877706785805], [1.1824706301464829, -4.3959233444901455, -2.0265139917934571, -2.9406351827018495, 4.8979621368210093], [1.5223430097196644, 0.53661269829977698, -4.2695951818768796, -2.1725069317000267, 2.7273106967280825], [-2.4119670313203025, 2.329742063096532, -1.2082285374501711, 3.315211059669446, -0.38565215144626031]], [[2.7153554360101664, -4.5918623826778262, 4.5534561728110461, -1.9183049634127514, -3.9543169486105603], [2.9460348430236607, -4.4986010143185577, 0.45655588125624114, 4.4964630450094738, 3.7849510862627831], [-0.29862749232644958, 0.74558414150115748, -2.5394803706234761, 2.891768347446467, -1.1751409842917004], [2.3966507662634173, -3.8015261569709144, 0.48493679686782087, 0.70598664415557089, 3.687205094220225]], [[-0.70284879528510391, 4.6232663571880313, -2.5713713355155212, -2.9412273901514254, -1.162532483411538], [3.840018746610367, -4.9011339749805192, 3.6638144036181668, -2.8604730416908186, 4.5032047777728206], [-1.3471478538161117, -3.2110842416931309, 4.6743452885481176, -3.1736720881687464, -1.418413555960135], [-0.30894341373277978, -1.61471716027597, 0.4148110057631369, -0.64317403109220184, 4.203039966272879]]], [[[-1.720304239997156, 4.7597999282762746, 2.9064476541429363, 0.44466355195328866, -2.0528088628367502], [-4.6499956454324831, 1.636109121759592, -1.6770198571794195, 0.35716808978065817, -3.1386538379575981], [3.1547020546046518, -0.059411310593548095, -2.4157622965000192, 3.238105638683086, 3.6943976815211208], [-0.34229371524768659, 0.15449320186190274, 4.9179389310986661, 4.2935636902693517, -0.59028055533989043]], [[1.5341547448871129, 1.087617195825489, 1.539026790905119, -3.7571691606534321, -1.7494746682493134], [4.7492945244862845, 2.3774940930757449, 1.5531866304817417, -3.11744599270112, 2.8490410115703106], [4.1912207793559482, -3.053643755917177, -3.5460552637391904, -2.1742935755070336, 1.4373817957669752], [3.8832158507588819, 0.069464082655652959, -2.2985135797335476, 1.831854594196038, 3.9037113055880805]], [[-4.4523181339895599, 2.3673528041538683, 2.7930058055162723, 0.55950565217550352, -3.6719674135086358], [1.2562743388576756, 0.69239532646342106, -1.4900468053632689, -3.9796382110004735, -2.4910969413500297], [-4.2862426159245528, -4.6295713034839805, 0.94164953298185328, -1.9028794542508445, -2.4422145341065429], [-2.1195806711780674, 0.60482659701142971, 4.0082678295286485, -1.7834446942894475, -0.75513872816817162]], [[-4.9868660214948077, 0.41036871013897436, 2.7822220829229458, -3.7765470718547256, 2.94437855946616], [-4.4283771297893022, -4.2735928975642521, 3.0687219216546673, 2.0163816060710964, 0.33047936115512844], [2.3951476133344336, -3.4011531487879143, 1.9830421739302544, -4.0763027612256542, -2.0252791103577339], [2.2076089749650327, -1.8689671891846746, 2.9975336184432617, -3.5948286744574331, -2.2087122247066238]], [[-1.1512553903690947, 0.35191584741817827, -2.2093629803650283, -4.4423989615324917, 0.63668476874347668], [-3.4027188123127328, 4.8871043136928165, 3.4093571748338611, 4.1104145717539247, 0.19713619513909375], [-3.1190848338473676, -1.1708559633214555, -1.9252734414823838, 4.8127378315193443, 3.4356312802976756], [0.59149891313894454, 1.2655980133366374, -1.2135831891518247, -3.9835630197417427, 4.2922181922989324]]], [[[-2.2474879755293511, 4.0059889719873425, -1.6229463383127185, -2.1395468213610691, 3.5057781383675142], [4.3084858461213766, 3.1314849639199185, 0.718418595724021, -4.1911849295310102, -4.9277529313361503], [-3.3739718782608086, -4.3955166223882713, 4.2146271781512841, -0.79133677207469688, -0.55734597362701255], [-4.6063417572606644, 3.7997056057478904, -2.284428307505654, 0.75649178497147052, -3.1249370691618363]], [[-4.2671489257563069, -0.12314196929905297, 2.0278484700227226, -1.7201541963376119, 0.81093706181871639], [-0.85456599855420379, 3.7594572604829821, -4.2596737282935528, -0.67063261733703161, -0.25537205849422051], [-0.15037778301778193, 3.1375024401789062, 3.8866820152742303, 3.0237526668030306, 4.1774016425413389], [-0.67283802646844659, -2.5952371871734892, -1.9144939645668879, 4.6113372568608852, 0.87000345892352371]], [[1.3511479814845213, 4.7615064071452515, -2.7750005517581711, 1.6182523463515448, -4.2374646191809546], [4.5389009310452924, -2.3685181375418063, 0.24552768327281616, 3.7165447031162895, -0.64916253641090194], [-3.3334636657078729, -2.9592596922163117, -1.307266168984178, 1.7695904558730957, -4.0049418081041921], [0.3809787502205273, -1.5908415336500017, -3.0101287148877853, -0.020735864098512025, 2.4927302089760666]], [[-1.1066554073960013, -4.4719345146238716, -1.142351985661235, -4.489236246845528, 0.72345005592211553], [4.2285831056397321, -2.7847814458174036, -2.2420721597509106, -1.8280675743034145, -0.049866208763851283], [-1.6856913348113425, -2.2859590293629473, 4.5639238149087884, 0.19276484204196009, -4.9633604913435434], [-0.79498464284815906, 3.1813024330281667, -2.4673375891760099, 0.51572413855953769, 4.2651754695245589]], [[-1.0498643230312696, 2.269131453413471, 2.3616287573972716, -1.1676113825333911, -0.28323823557364491], [4.5711028596523313, 2.3257980195102776, 0.64982375105532597, 0.45323875576619077, -3.8810896975326612], [-1.5777961015653306, 3.9447145069262461, 4.1304480167033422, 3.9241744358845416, -4.6548175390752498], [1.1611377237203637, 0.79728739540576044, 4.309522137722368, -3.5310811822470889, 3.2680080195405488]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.1803523940853644, -12.471580715596467, -28.702914314947659, -1.3014832636113038, -1.4276875612753153], [0.96729180556066885, -1.7852175586971841, -2.7902712993975256, 1.2495818717933316, 2.1736366963403158], [-2.659496760048957, -0.84054021736776441, 4.2568420394109081, 0.9150146003175692, -1.4631612534793257], [1.72536789998245, 0.94138724983122879, 5.6693787626566818, 6.2264989759663045, 4.5377073743638157]], [[-1.1453323115776384, -20.30369418413488, 5.999717996428684, -1.4783726388699194, -1.0827357210357893], [-1.1519547232777765, -1.1928568678181135, 1.1191602121380824, 9.1092821504030717, 1.5690770167974044], [4.4883285148502905, 1.4654845038475444, 4.7774977292366643, -1.05431330328902, 3.9072235793792531], [0.97161105370026135, 2.7208249279604502, -1.3447272856128991, -2.1608832247168102, 1.2953711172391245]], [[1.3527223964274706, 0.44032362965686356, -4.4179203153044391, 0.49330827057324317, 0.64443019036846994], [-1.1528643520703119, -1.9317799102297413, 1.9622930697627514, 0.51516133119193275, -0.95197117029624012], [0.60900751571383782, -6.2215160312922269, -1.1757329566890149, -8.2151652735045069, -0.90949699579412313], [-3.9501782813012838, -1.621572568194751, -0.51896516115416758, -0.65413721963832372, 0.51030483150819317]], [[6.1246549928579723, -1.0685565584328736, 17.132188392205787, 0.9974052017779268, 1.4416134432935854], [2.0279124990368276, 2.8338982877530552, 2.3576399646528459, -0.94241145551302785, 4.1337126197511331], [-2.2766664161885743, -1.0511719523892717, -1.4284597903436234, 17.701134210911977, 1.3242275434816386], [-0.85624375862388424, -1.1006801669924346, 1.4511325615531439, 1.4201228915075013, -2.1944509125459182]], [[-0.13218832535294858, 0.24077984102882105, -0.085274586149607992, -0.093284565950051149, 4.8198519809006166], [0.32910825557881201, -0.1172177844764536, 0.95126474447437848, -0.52547237778625944, 0.18416408777478172], [-0.21526599946268471, -0.17776779478033267, 0.092488894343422945, 0.17040767140965013, 0.24262264155601035], [-0.12176600342868137, 0.27934617622705726, 0.18137878087714002, -0.11431242805778564, 0.13150164064881523]]], [[[-1.5971369407363318, -0.7866243968387111, 2.2466060014153495, 7.9462465651099565, 2.2783738327416057], [-0.76314331912159195, 1.0146939987993373, -0.7167965233131407, 0.75295758057773721, 1.0318408356107831], [2.481461340683174, -0.88418872263692438, 1.9974381356104072, -0.94533152277277965, 26.196938467724451], [0.81281658618033226, 1.5014013711949141, -0.8526428376236429, -1.3637546876705164, -0.88222792964891816]], [[-0.62746843217646053, -1.0013150065633003, -60.602516246397379, 0.84009235809105864, -0.91529299872827019], [-0.990298819717225, -0.66582706179743489, 1.0076210141007849, 1.0743191628641966, -0.91259398577614814], [0.676376428852275, 1.2916065740834981, -0.791646746836558, 0.86511621716483056, 3.3408167301141898], [-3.8656558078176686, -0.87408878555344038, 4.6748655712294136, 1.1398628966665645, 8.5371122463062594]], [[0.013059706364073541, -0.011399862897572619, -0.0099411464833691204, 0.011677445368829377, 0.11777700541942115], [-0.038516156597278611, 0.010360559180243627, 0.022474171975540216, 0.015487886505036885, -0.0092986067858754073], [-0.029917189274440523, -0.084873548663139384, 0.010667105901684988, 0.020963902714347362, -0.016699316296102031], [0.01888260634204187, -0.019549041365493663, 0.03769504075654475, -0.013737956088667787, 0.1180966417316905]], [[-1.5947777801722172, 0.9430571201250838, -0.9510113527557067, 2.2574035918224906, 1.0951040523295528], [-1.4699040389401385, 0.96260781981680532, -9.4849035844276717, -0.96306551866918155, -1.1441068631866076], [14.500970693900548, -5.8080480439136775, 1.705226220573498, -1.4974880399540791, 3.6849948835959765], [-1.8068500323767658, 1.1391184318640446, -8.9297998060542909, -6.1338108170564958, -1.1744365729497486]], [[-5.849756562115072, 0.88930510049488198, -1.598950060462355, -1.3978838787374526, -3.536670511200017], [1.0706964272044186, -0.83888634209597412, 1.122189581528322, -1.437347701750552, 0.91301518702579587], [-3.051999333812498, -1.2804068790876109, 0.87958721459166456, -1.2955006812837113, -2.8986569785076592], [-13.308244065529557, -2.546262871010188, 9.9117291857522023, -6.3925067767611852, 0.97821919024949233]]], [[[-2.7199186498043422, 0.98304291277222777, 1.6098991423554594, 10.522759432725627, -2.2793586243776143], [-1.0062563371004105, 2.859887230916184, -2.7901205615868552, 13.100519670106145, -1.4907944065442245], [1.4832106185356178, -78.75752174054692, -1.9368990038816563, 1.4450077013573592, 1.2665359793586692], [-13.669802795882251, 30.286689183180101, 0.95143263290999081, 1.0897911206744466, -7.9268875509739205]], [[-0.52263564580817268, -0.73721154735438754, -0.52098115549515467, 0.21340640295373395, 0.4583112693286997], [-0.16882590703310343, -0.33724750702806638, -0.51623155912374707, 0.25719898844791961, -0.2814294187438941], [-0.19130558805517264, 0.26257285392575952, 0.22611152286957723, 0.36876526927913689, -0.55782253415552174], [-0.2064793683068262, -11.542712797899421, 0.34883585763139008, -0.43770065506517092, -0.20539530029179748]], [[0.45256393596710692, -0.85114420434524984, -0.7214301577592267, -3.6013195059984775, 0.54874087702502472], [-1.6039160847050595, -2.9101274111538786, 1.3522787416095963, 0.50631703488179225, 0.80886399298617817], [0.47009905865102525, 0.43523654498628944, -2.1398179984387791, 1.0588997712886237, 0.82505389709065446], [0.95064021213975403, -3.3314649667398184, -0.50270059402018274, 1.1298127860919351, 2.6683290682017611]], [[0.91301275837414964, -11.095076669918047, -1.6364877303913608, 1.2056177813484013, -1.5463610435856472], [1.0281582097647421, 1.0653968244196432, -1.4837031240263898, -2.2580409820336147, -13.777175936230595], [-1.9009568665326149, 1.3386848820818011, -2.2960037672337634, 1.1169612682445325, 2.2481209027643931], [-2.0624450949242106, 2.4361435172725239, -1.5189395287889673, 1.2665616957710759, 2.0614149054805297]], [[-0.70948119078415151, 2.3209925078626821, -0.36969662862767294, -0.18386328025206669, 1.2828861084076424], [-0.24004159329891453, 0.16713251709549634, 0.23957420808969396, 0.19871330032464593, 4.1432982141074177], [-0.26186977551624629, -0.69760420653167043, -0.42424833151328345, 0.16971505073607612, 0.237742056296847], [1.3808884971930144, 0.64538189586939476, -0.67304330890294983, -0.20504107534081181, 0.19029648742489946]]], [[[1.1367635977196711, -0.63776074641238867, 1.5742125642617271, 1.1941138615836289, -0.72875761558720065], [-0.59298384818750327, -0.81586293606095706, -3.556231049837538, 0.6095800018016222, 0.51846400428231276], [0.75722697434319097, 0.58124282908667046, -0.60618944663454599, 3.228540119772525, 4.5839794988888034], [0.55464024415196467, -0.67238433236239503, 1.1183815699097495, -3.3772508408545323, 0.81757246957286867]], [[0.46533425721650118, 16.124888915628805, -0.97919080500958344, 1.1543445233146306, -2.4485877749201834], [2.3235777917193312, -0.52817479710991655, 0.46615086094738617, 2.9608619152520252, 7.7755201078270684], [13.204414481653862, -0.63287618532840928, -0.5108857807239221, -0.65668419166650993, -0.47533149687541737], [2.9511568872246565, 0.76511333361464473, 1.0371673207380476, -0.43060189814672345, -2.2823479095771591]], [[2.1540624484700017, 0.61124713071357706, -1.0488131713695896, 1.7985187142188692, -0.68683927555822311], [0.64122508366173592, -1.2288093061691727, 11.853885844749803, 0.78310833360929466, -4.4834027935949923], [-0.87310300069636504, -0.98350852305972747, -2.2263691957265621, 1.6447066153540379, -0.72671645898884185], [7.6394211686534943, -1.8295078847760162, -0.96688793234888215, -140.35861324200607, 1.1675780711292341]], [[-2.2053858800948185, -0.54575982756916464, -2.1364712805125565, -0.54365644296770288, 3.3735600538324766], [0.57716784762883055, -0.87640709229350022, -1.0885475737198689, -1.335072206251299, -48.943007100450494], [-1.4478345822871925, -1.0676491477985364, 0.53475962977933633, 12.661033950737043, -0.49172374520414253], [-3.0699991899942547, 0.76717076134085094, -0.98916427987334388, 4.7323792452657614, 0.57221613203026678]], [[4.4256950446264565, -2.0476466116495886, -1.9674469653267306, 3.9793885204238522, 16.404491867276487], [-1.0164679016483549, -1.9977570248976075, -7.1502146919434582, -10.251504914038058, 1.1971842173406759], [2.9448541084364006, -1.1778746785886152, -1.1249092866390897, -1.1840399574190354, 0.99818720990998799], [-4.001574694414618, -5.8277345894888199, -1.0781657881041224, 1.3158517440295054, -1.421777212352997]]]])+(1.-msk_ref)*numpy.array([[[[2.9449704872101505, -7.0899495521241596, -16.317275182080923, -0.73987819927257237, -0.8116238844510455], [0.54989421629864665, -1.0148755574263986, -1.5862369975867365, 0.71037285764300051, 1.2356873497540135], [-1.5118931828107853, -0.47783740277736808, 2.4199655199313872, 0.52017522907867819, -0.83179026863997918], [0.98085171789267678, 0.53516777564287332, 3.2229763280009021, 3.5396927328343506, 2.5796342180031799]], [[0.44986043157174432, 7.9748283846058943, -2.3565525043682571, 0.58067108263958034, 0.42527391728988728], [0.45246156397266846, 0.46852699424921451, -0.43958062733501629, -3.5779184417179777, -0.616297695271943], [-1.7629131583174129, -0.57560891690417537, -1.8764922359930702, 0.41411023930348539, -1.5346683821795744], [-0.38162668032599895, -1.068678028158885, 0.52817823345858883, 0.84874568736145506, -0.5087922552748686]], [[1.2538088421342428, 0.40812635447040918, -4.0948738409316388, 0.45723666080799369, 0.5973082673548854], [-1.0685647862596204, -1.7905246034959168, 1.8188065845771639, 0.47749178536553083, -0.88236128412346126], [0.56447576394449062, -5.7665873146740516, -1.0897611963685434, -7.6144572505911023, -0.84299289952825407], [-3.6613339663647313, -1.503000193930716, -0.4810174722715958, -0.60630550075734957, 0.47299040188775576]], [[-6.2610087227732771, 1.092345926574865, -17.513603801173808, -1.019610521042104, -1.4737082094997387], [-2.0730600924127782, -2.8969896132540551, -2.4101283094416832, 0.96339244419303682, -4.2257418254383614], [2.3273520397840621, 1.0745742855260396, 1.4602617156234174, -18.09521611041551, -1.3537089371308906], [0.87530638832975538, 1.1251847058420892, -1.4834392527218803, -1.4517392116791616, 2.2433061651912682]], [[-1.497527558258295, 2.7277329253620644, -0.96605386623320721, -1.0567968683873559, 54.602864124744009], [3.7283828285414597, -1.3279301489203343, 10.776633762825266, -5.9529414926572004, 2.0863476101074867], [-2.4386931727189398, -2.0138856509722407, 1.0477829092872013, 1.930504705262029, 2.7486095388335681], [-1.3794557522878621, 3.1646410231549633, 2.0547935842410689, -1.2950161128895357, 1.48974828376088]]], [[[0.18130909392809713, 0.089298639969349919, -0.25503767907481373, -0.90206840008666933, -0.25864400522438524], [0.086633037029013843, -0.11518940226363301, 0.081371687585410898, -0.085476738543097314, -0.11713593381438588], [-0.28169876722621146, 0.10037427103078188, -0.2267518140166403, 0.10731528241816345, -2.9739110380124627], [-0.092272011880297697, -0.17044106569111811, 0.09679314051965543, 0.15481523246700504, 0.10015167922229733]], [[0.5783791949607725, 0.92297833277348518, 55.861351363300336, -0.77436874406576983, 0.84368615313672213], [0.91282398404000398, 0.61373688338347598, -0.92879099740474924, -0.99027109681524372, 0.84119829421285197], [-0.62346093340996123, -1.1905592890677379, 0.72971321687124702, -0.79743489165776504, -3.0794519561288842], [3.5632308805811737, 0.80570550196294644, -4.309133092574621, -1.0506870955290655, -7.869221549809791]], [[-1.1833641723816957, 1.0329626828487899, 0.90078568789106483, -1.0581149444853954, -10.671992513335116], [3.4900202580700017, -0.93878944885529436, -2.0364263313730109, -1.4033860704857268, 0.84256397630352631], [2.7108519088243943, 7.6905493792071544, -0.96656621482552296, -1.8995780375046687, 1.5131559666945711], [-1.7109879198991391, 1.7713748312115321, -3.4156174315277026, 1.2448216356383375, -10.700955351364328]], [[-1.7169770057030787, 1.0153185042147248, -1.02388222681896, 2.430376261783759, 1.1790159733094954], [-1.5825348627429214, 1.0363672686466368, -10.211680622998394, -1.036860038495099, -1.2317736054400952], [15.612101918746815, -6.253087460462134, 1.8358885147813451, -1.6122324770783254, 3.9673561796080064], [-1.945299211536391, 1.2264029375126522, -9.6140422340668223, -6.6038116790676176, -1.2644273173869331]], [[-6.6320077389265961, 1.0082262819183849, -1.8127676019580128, -1.5848141035386125, -4.0096072291650007], [1.2138739306261541, -0.95106533986240838, 1.2722529408211345, -1.6295551755561246, 1.0351069692929693], [-3.4601240215925997, -1.4516276431191504, 0.99720888421442955, -1.468739844611886, -3.2862761569356094], [-15.087871896365554, -2.8867586003912815, 11.237162430274845, -7.2473365283584128, 1.1090302940325201]]], [[[1.8554840526031589, -0.67061580970616086, -1.0982468851246481, -7.1784545167210725, 1.5549412031129584], [0.68645162841726515, -1.9509683311999457, 1.9033746495458566, -8.9369604235937459, 1.0169955808243683], [-1.0118220445830159, 53.727094235941031, 1.3213208466598381, -0.98576063881550269, -0.8640101467436111], [9.3253160684839465, -20.661084400293284, -0.64905179337540675, -0.74343769260357095, 5.4075931420480741]], [[3.0530643339720736, 4.3065456783752518, 3.043399349568328, -1.2466495209904973, -2.6773026322787445], [0.9862250298986267, 1.9700882319961439, 3.015653780741149, -1.5024713003449708, 1.6440174484632117], [1.1175438806467806, -1.5338636425197691, -1.3208686233135269, -2.1542045596655988, 3.2586144809977875], [1.2061840789750378, 67.428704955735824, -2.037783537894887, 2.556902250456468, 1.1998513126992778]], [[-0.49013878476524869, 0.92181182109943127, 0.78132805713181985, 3.9003248512267072, -0.59430097106900437], [1.7370837977684364, 3.1517454208252951, -1.4645538594674519, -0.54835482118695111, -0.87602122717844455], [-0.50912978921777752, -0.47137275927463718, 2.3174798299655475, -1.1468166281931225, -0.89355532411499472], [-1.029568644994777, 3.6080652047460862, 0.5444381195052389, -1.223617310124308, -2.8898713814820107]], [[0.58833016344129063, -7.1494819877769533, -1.0545253448651872, 0.77687995029940393, -0.99644921406363018], [0.66252796803361957, 0.68652390899424653, -0.9560734978240959, -1.45504387297134, -8.8777819323750826], [-1.2249448364484565, 0.86262616622583888, -1.4795064573291561, 0.71975117486211759, 1.448651539672291], [-1.3290051520706154, 1.5698101702715097, -0.97877924819057138, 0.816151190272976, 1.3283413152093919]], [[1.3292004609955046, -4.3483384076869367, 0.69262009420893544, 0.34446460321390654, -2.4034644312634796], [0.4497136789748703, -0.31311981438879216, -0.44883804105292363, -0.37228585313943136, -7.7623959137613436], [0.49060839224260838, 1.3069491410892138, 0.79482174460575816, -0.31795814548225088, -0.44540553707776637], [-2.5870701730988657, -1.2091115658184335, 1.2609347338369559, 0.38414082770086749, -0.35651710305587969]]], [[[-0.36446380679922674, 0.20447585578113839, -0.50471664032250985, -0.38285117909961314, 0.23365084468190175], [0.19011969693118927, 0.26157814351168068, 1.1401821002021792, -0.19544067777237747, -0.1662274945011924], [-0.24277855680265867, -0.1863553465193922, 0.19435361389574185, -1.0351193729432739, -1.4696939819378549], [-0.1778261506554043, 0.21557670732643167, -0.35857024735931109, 1.0827983060355459, -0.26212624611881202]], [[0.88232277086734456, 30.574488010123712, -1.8566489160971329, 2.1887588170252421, -4.6427803600849327], [4.4057482631496354, -1.0014750542457884, 0.88387113756367364, 5.6141060940744341, 14.743205212335811], [25.036960835043779, -1.1999999157489833, -0.96869325792047656, -1.2451424037461369, -0.90127859038865676], [5.5957043385886589, 1.4507354790093143, 1.966578497273386, -0.8164665592990914, -4.3275720633748351]], [[-1.2566923267053283, -0.35660506473895698, 0.61188359025701633, -1.0492660828845335, 0.40070595348196381], [-0.37409437358747394, 0.71689436262339801, -6.9156246576411382, -0.45686987140266394, 2.6156427787142329], [0.50937327382400599, 0.57378448570808116, 1.2988764957441057, -0.95953122652734268, 0.42397052989360601], [-4.456881911634512, 1.0673453418576384, 0.56408793822568815, 81.88601605933728, -0.68117171062506021]], [[-2.4667148031232382, -0.61043006476352535, -2.3896341142174182, -0.60807743796024494, 3.7733126883246473], [0.64555980270065161, -0.9802576354708612, -1.2175358689986899, -1.4932726304826485, -54.742546968177919], [-1.6193968723732008, -1.1941610677689345, 0.59812638994164502, 14.161313061361492, -0.54999093459785298], [-3.4337811427402412, 0.85807726013074881, -1.1063760740952102, 5.2931462215530027, 0.64002132969323922]], [[2.7774621809514977, -1.2850548821061469, -1.2347234692226763, 2.4973706971088112, 10.295073496852856], [-0.63791136228585754, -1.2537453501502185, -4.4873066698686008, -6.4336035152603621, 0.75132467415101745], [1.8481212176000923, -0.73920646151443192, -0.70596662651547615, -0.74307564558921457, 0.62643882985129795], [-2.5112942184038856, -3.6573492433874364, -0.67663150557354257, 0.82579762453775052, -0.89227395860538739]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-4.9305315522288051, 3.7090005378955073], [1.2880146839942217, 4.4704234016184241]], [[-0.19189222229114566, 2.2965549807515462], [2.5272474202371313, -2.2683685953082708]], [[3.1598790459958881, 2.9281717119169457], [4.9452166568602589, 0.0078010701944597471]], [[1.430261165428302, 0.25797512797104716], [3.2766625640357869, -1.9204433292635139]], [[-3.6809605421141054, -4.3217555772760274], [-1.7282205245148652, 1.3625895153745047]], [[-3.6205485466033558, 1.7876642821335755], [4.2762267725232075, -2.1867060997370737]]])+(1.-msk_arg0)*numpy.array([[[4.0217538285988788, 2.2999558397502007], [-2.5715583581530996, -4.1390474623700992]], [[4.0471010171585835, 2.4173477973189534], [-3.4895152721335201, 3.5508552524093915]], [[4.4172280737006879, -3.3631850282926159], [-1.9057349431073756, 1.2045943202364517]], [[4.8155967395188117, -4.6420424548539749], [-3.8374149698450943, -0.3933209719062809]], [[-2.4711528784989345, -1.7836158745302475], [-4.8551895693177602, -1.9283719407278643]], [[-2.1411473263178573, -1.5280817633203458], [4.6103581725765981, -0.97401092652330679]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[-3.8926936518458155, -0.42943654735655024], [3.5109626463015342, 2.221103145283382]], [[2.7929223584267158, -4.9084339281846381], [4.7113571000705541, -2.411046614946458]], [[-2.7983262955189092, -1.3602374358034708], [-4.0847100328341543, -4.7449098943127481]], [[-0.71046006033721998, -1.2393518939233914], [3.2193997066888596, 4.3920426147098262]], [[2.4387242223756589, 1.8632865687946287], [-2.6023448793121275, 4.2318228217344718]], [[1.7669354553926588, -1.5364044250280848], [2.7723054125741786, 0.62861976125649122]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.2666117586445247, -8.6369000513037815], [0.36685513739402009, 2.0127040975614214]], [[-0.068706608227820806, -0.46787937137434721], [0.53641601911247294, 0.94082320152845511]], [[-1.129203213740996, -2.152691607246731], [-1.2106652900962585, -0.001644092378616107]], [[-2.0131478815986199, -0.20815325270886581], [1.0177868120034781, -0.43725516752309423]], [[-1.5093795798396321, -2.3194261417725976], [0.66410126430732042, 0.32198642825410834]], [[-2.0490553492225758, -1.1635375770939334], [1.5424804039005888, -3.4785831348449254]]])+(1.-msk_ref)*numpy.array([[[-1.0331544653383824, -5.3557524479643455], [-0.73243683206427512, -1.8635097929421076]], [[1.4490560415859037, -0.49248860893050639], [-0.74066032313306573, -1.4727443386606796]], [[-1.5785250207505115, 2.4724985063405756], [0.46655329944806179, -0.25387085256988318]], [[-6.7781385729594543, 3.7455402921592786], [-1.1919659934963034, -0.089553086436131232]], [[-1.0132973855041656, -0.95724184588744143], [1.8656979741290565, -0.45568352503413506]], [[-1.2117858180858332, 0.99458302672645138], [1.6630051478692334, -1.5494436964190315]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-1.5933180417282564, 4.5136158010252903], [2.0624945157793553, 4.1062650325969372]], [[2.1397063270284802, 2.8801019859897634], [4.9530414838453982, 0.38203951306407191]]], [[[-2.8657743195913121, -2.408329480395055], [4.6776937800848479, -0.87300513763654575]], [[4.5560051613872083, -0.76635452301989027], [-2.444394418342327, -2.9801473765540711]]], [[[-2.3273948603545227, 1.1318734802271297], [2.7934112648380802, 3.6287370287518623]], [[-4.3102437461424632, -1.0155254974836705], [1.1087061289005407, -2.6087004179814102]]], [[[0.74597805379863136, 3.0877470949377699], [-2.0432564473471881, -3.0213154431647382]], [[3.8574693058864113, -2.4628548890525606], [0.74051073063011152, 1.7443843155533036]]], [[[0.67796115301052851, -2.4005925587261068], [-4.4989461873045453, -1.1135182897787721]], [[2.8118283362635621, -0.26233681370823803], [3.4385812408035363, -3.5955728510674723]]], [[[0.23877936967585534, -3.954753066751203], [-0.03417927042183333, -2.9080567302839375]], [[-1.4347396445029901, -4.7687993143142196], [3.5784533007732549, -3.7652709231222747]]]])+(1.-msk_arg0)*numpy.array([[[[1.2863477326082169, -4.3288252624726908], [4.466261896248799, 0.57663412035678441]], [[4.1294659913680611, 0.90020480897097688], [1.7671087029904262, 4.8246166552483025]]], [[[-2.6056947536125783, 0.61017543895534576], [-2.817411574880444, -3.5519395582428368]], [[2.3393627555891303, -1.0047622834588976], [-2.1890786449167643, 0.20952135144067352]]], [[[0.15146911791440143, -0.67984715727056688], [2.625416950312184, -4.6928424636604626]], [[4.0194464265430341, 3.3953865705897606], [-0.14116383637324859, -4.0821929142796201]]], [[[3.2088132961603186, 0.080597599948218246], [0.85093228041363744, -1.9188037904367228]], [[-3.2414439537138362, 0.32454741842625534], [0.9896368762128871, 2.2418420807009296]]], [[[-1.8015059334248749, 1.1962579644776739], [-3.9432293543026944, 0.33394301746358401]], [[-3.9144429200480859, -1.6951398452058184], [-3.6079197223014434, 3.0812386268355745]]], [[[3.0004384523320873, -2.4607633328573497], [0.0075455679341711601, 2.9952276076181858]], [[-3.3116497760737529, -4.8792437620118818], [1.1966495889580218, -2.5775661968279886]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[2.017224037694759, -2.6705571904868108], [1.0153906349888544, 2.1303327855517162]], [[2.5131650861911394, -3.7546724459043324], [0.70395178484633902, 3.5116829964315226]], [[-1.2238159930451111, -2.5384878762110876], [1.8866243670385883, 2.421952958626429]], [[-3.7139488063821755, -2.2566398952712707], [2.1086003003430598, -2.8581552031809321]], [[-3.404339236446706, 2.9787647747961898], [-4.1655846947934094, -0.50104045806207953]], [[0.1474792366322939, 4.3678397293020499], [-3.1176528448980312, 4.1922575920326359]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.78985675956403267, 2.2375381795387264], [-0.77230868641438344, -1.5376061022862475]], [[2.1072740414352622, 2.8364472615225345], [2.3250083355228717, 0.17933325518676221]]], [[[-1.1403048432184668, -0.95828542805559602], [-1.2458327184272397, 0.23251166385734559]], [[6.4720414941226529, -1.0886463242467277], [-0.69607490790776227, -0.84863792648209313]]], [[[1.9017522843147978, -0.92487227382181114], [-1.1004233232768035, -1.4294876342557388]], [[-2.2846327130334911, -0.53827646627809045], [0.45777360165134057, -1.0771061463806844]]], [[[-0.20085846431612558, -0.83139193777568521], [0.90544195891811452, 1.3388558136793667]], [[1.8293980633782603, -1.1680046183488948], [-0.25908695574193225, -0.61031826179765281]]], [[[-0.19914617960287453, 0.70515668151559885], [-1.5103395291131601, -0.37381880543251694]], [[-0.6750140838039097, 0.062977188781237467], [-6.8628814010414541, 7.1762126056135296]]], [[[1.6190710986062238, -26.815660001085337], [-0.0078252116698647595, -0.66578833256517511]], [[0.4601986545265645, 1.529612035579347], [0.85358621750106367, -0.89814875170794672]]]])+(1.-msk_ref)*numpy.array([[[[0.63768213573254262, -2.1459318259064477], [-1.6724082570329275, -0.21592277537095952]], [[4.0668742147827555, 0.8865600862872427], [0.82949890034799334, 2.2647244073647474]]], [[[-1.0368179822049308, 0.24279162650636105], [0.75037479712876942, 0.94600517339864354]], [[3.3231860561299298, -1.4273169059131237], [-0.62337023220525511, 0.059664084615149902]]], [[[-0.12376788567496529, 0.5555141958710349], [-1.0342444314647843, 1.8486763350884841]], [[2.1304964023401811, 1.7997152108871881], [-0.058285127244299302, -1.6854963676068957]]], [[[-0.86398964106510712, -0.021701322271786998], [-0.37707933915231384, 0.85029241681737766]], [[-1.5372491188521922, 0.1539160448632455], [-0.34625022291003954, -0.78436681052376445]]], [[[0.52917932329952067, -0.35139211500152179], [-1.3237800405278708, 0.1121078845463495]], [[0.93971031844359632, 0.40693923408269295], [7.2008550691816939, -6.1496802848080687]]], [[[20.344819520682776, -16.685490032693256], [0.001727528572889484, 0.68574576752998262]], [[1.0622253152698491, 1.565037547396162], [0.28544276268525298, -0.61483965148674069]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank3_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-3.3796517723435748, -1.8139157211780885], [4.7742661930659533, 1.7111753401723639]], [[-3.0591908720328362, -2.7480642368045372], [2.2640651389770827, -2.1511554043834238]], [[-1.7864874026639233, -4.6960505685757337], [-1.3573124989182785, 2.5246628333395922]], [[0.25376287201643422, -2.6597377156677213], [3.5295501423476789, -4.8339507551949588]], [[1.2057213289388127, 4.0315641894021876], [-4.2705380020942796, -1.1126784981175897]], [[-3.1586269244259793, -0.35531999397484171], [2.399404198291343, 4.3714492713674069]]])+(1.-msk_arg0)*numpy.array([[[3.0597816372859619, 2.369422136304471], [2.7977775990750873, 2.9917853865487176]], [[3.8704643643157084, 3.3746904046147979], [4.6988015461443986, 4.8901866307221162]], [[4.873112761390999, 4.8324438003711858], [0.34713495944618789, -2.142926524410349]], [[-3.7129979766227739, -2.6304273903923701], [1.5609152833583853, -4.3761520274149301]], [[4.4926367365040214, 1.1690343859974019], [3.7136378046157574, -1.7261917461427068]], [[1.7393507198298721, 3.9440904651518771], [-1.4570178729206509, -0.74715623479105009]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[-0.37226741077410619, 1.6736988517046347], [-1.8142305570216308, 2.5598612911402405]], [[-1.8696363602405053, -4.0849382899380675], [2.8671634979505214, -4.9469044200921255]]], [[[0.75059797339008583, 3.2295559281026875], [2.6810853596596509, -3.0624998116427857]], [[3.5277861047218835, -2.4137957363413221], [2.4644774337905195, -4.6137785368688462]]], [[[1.8324177185246802, -1.5848263387543318], [0.88450100004965826, 0.57063382686834085]], [[0.96143688227378732, 4.6919233145652388], [3.3207725382197495, 1.0937391146057731]]], [[[-2.5307676328464934, -3.1048942455889139], [2.7329754408975715, 2.3494887789715992]], [[4.8933641632581573, -1.9049747909129322], [-3.7582598682394686, 2.895365235903621]]], [[[0.84746238193462986, -3.9770340704679965], [1.3485829469793389, 1.2549125490391617]], [[-3.047760313978316, 2.3490179880925899], [-0.17900065657195441, 1.2031814049123604]]], [[[1.8134623064266862, 0.84717011135631282], [0.44179377800292219, -1.3792546543273945]], [[-3.3152287138342871, -3.8388042926685251], [2.0835898419740548, -3.3731194983766697]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[9.0785593219557033, -2.0192711304673807], [0.99982646315688828, -0.70859922272199172]], [[-2.5535800942873212, -1.1687486699189125], [0.59681819379868994, -0.34590830848122534]]], [[[-4.0756716384617446, -0.94724814808519575], [-1.0249820010032762, 0.89732715292165877]], [[0.64178072926435903, -0.93796882018227801], [-0.87286471967195622, 0.466245916918958]]], [[[-0.97493458211169426, 1.12724489679298], [-5.3092654144111595, -8.2295341556385289]], [[-1.4117541400203515, -0.28928701684120539], [0.76026370499108376, 2.3082861348060875]]], [[[-0.10027110696489079, -0.081729956624755282], [-0.97320220147832825, -1.1320495502991599]], [[0.72129316858314352, -1.8528067453618071], [1.2862204649673121, -1.6695478329476869]]], [[[1.4227431855869888, -0.30317098309316964], [2.9894818100973315, 3.2126256068512515]], [[1.401205331832621, -1.8180099189287051], [6.2160581945704587, -0.92478033119090386]]], [[[-1.7417659651552702, -3.7284447150396298], [-0.8042666322305958, 0.25761739709199427]], [[-0.72375223714694159, -0.62503946941859068], [2.0980373311984311, -1.2959663224119953]]]])+(1.-msk_ref)*numpy.array([[[[-8.2193110348374105, 1.8281554260311672], [-1.3060204102141693, 0.92560567422348805]], [[-1.4964287486980561, -0.68490082358563731], [1.0434652187387561, -0.60477929882724557]]], [[[5.156507879757128, 1.1984509482049888], [1.2587030817411933, -1.1019397917299942]], [[1.3319406014596888, -1.9466442314901686], [1.9842691856994223, -1.0599092677822493]]], [[[2.6593896752507118, -3.0748559903549113], [5.4634690069314553, 8.4685547418242137]], [[0.36105850092334468, 0.073985642171211397], [-0.64530963796730323, -1.9592666073597858]]], [[[1.4671429839833068, 1.1958532828928992], [-0.96247750749215577, -1.1195743575943959]], [[0.31898612718802399, -0.8193889445698852], [1.1644091097577318, -1.5114335052272487]]], [[[5.3012816052648892, -1.1296450211137747], [0.8668613144010876, 0.93156641623552705]], [[-1.2184809243638504, 1.5809320420024724], [9.6434939357265144, -1.4346895148935936]]], [[[0.95913254643663026, 2.0531304120788505], [8.9274468349932015, -2.8595810445716765]], [[0.43949241475877265, 0.37954992279843797], [-0.35859084150802545, 0.2215030434440946]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.87681125077570599, -0.99684084650519278, -4.3334077209581734, 3.5254841167022644], [0.25091163966084018, 0.7811887016317236, 2.9245662967758435, -3.7215182752660092], [-1.2870757381166831, 0.15979524651792421, -4.3136248673177437, 3.8419666032112669]], [[-1.155196686840112, 3.5530185022409384, -1.6401541392375241, -1.9577856684645267], [-3.3592523653662045, 4.1315598758275467, -2.4725701872940653, -1.1746796977858951], [4.531613822469243, -1.4813094493769774, -3.3925467538239786, 0.2293559793869635]]], [[[-4.0294177318922832, 3.0704498039964783, 1.3232394029158767, 0.1100696743999201], [1.1184941677190032, -2.3163357484974991, -2.1499889174300524, 4.3986690585228505], [-1.1203869273059941, -0.63684876116220313, 4.9011240069077733, 1.8822088299766024]], [[-4.1139032062052072, -1.0635481088307772, 4.4525172695456234, 0.399171769744747], [-0.39996915986232295, 2.0713156037310299, -2.0461410702461134, -3.39812175309841], [1.1775164685952468, -0.11382995767524129, 3.3401815607972338, -3.2313573238551907]]], [[[4.0094067157502025, -3.0166219202166378, -2.1127557920038518, -0.22766515038126744], [-1.4019351896712937, -0.097266182155763659, -1.5556722825664702, -4.3891045042347869], [-1.6225766927807905, -4.5449781249471393, 2.3524288372887163, 3.2315024803369035]], [[-1.1481040625956842, 4.8878169151850877, -2.5228625963942095, 4.7394501286175732], [2.5139191179828293, 0.5671262480306396, 2.4701123948994876, 1.9855974090925486], [-3.3697632315269144, 3.2223989140059714, -1.7659018148539216, 4.4500479581671861]]]])+(1.-msk_arg0)*numpy.array([[[[3.0898257696037703, -4.2635806997272727, -0.65345397467145361, -0.48484402242642055], [4.8196882199445756, 2.593196615981471, 2.7976491971972095, -0.046691440180470778], [-3.0069920803255457, -4.0124137601153578, -2.0748938404074293, -2.053466981515768]], [[-0.74386074543932512, -2.9753644774807695, 2.8243477307537397, 3.5658378482429267], [3.4174991356930455, -1.5801005209650731, -0.67695689213850585, -1.6518901585525256], [-4.913091905767307, 1.2977429214527181, -3.379924047364411, -1.9417418208982262]]], [[[2.0640816772447392, -0.60732339275620273, -2.4043598360126062, -3.591005296664771], [4.2273116231489052, 3.8133709609421729, 2.9904669782345259, 4.2736259165529482], [3.6262978898627516, -4.8176842058255156, -1.1194705729524088, -2.8478406637237419]], [[1.0760229147732367, -2.6883394724704965, -1.2542314619830752, -1.0756417759156536], [-0.35187077526104371, 3.6225902614587362, -1.1691966784278951, 4.7273434937161447], [-0.93345390328212208, 3.3290392867050134, 1.0188775488916333, 2.0013358698685932]]], [[[-3.350894554873368, 1.2374217178156126, -1.4096816962575689, -1.1134804446368172], [3.2684615018540271, 0.79564185667999787, 3.2405008287788899, 2.6315612946821165], [0.095294947976698907, 0.64813252945018007, -4.8396293457262276, 1.0775853048836641]], [[4.6384989495973663, 0.73655268832794629, 4.7358029723429045, -3.8678143783422136], [2.7687849415977928, 4.9596678240247716, -2.8623255247807955, 2.8455022812869188], [-0.75903829391266342, -0.51694141766477575, 4.8220710726643645, 3.4395066359176223]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s1=numpy.array([[[[2.6776697838776222, 3.7251885097867365, -1.1583699515815473, -3.0010234426109239], [0.21874214964072003, -4.5172687035620251, 3.1127403308098103, -0.84803672204851921], [-2.4523438307019561, -2.6242367293721336, 1.1703578942444173, -1.2640448505776201]], [[0.601633832680041, -4.9505250756065191, -4.2897272487731044, -1.0855586990648272], [3.0639554298038547, 4.6783094944361139, -2.970961623252053, 2.4593000112117647], [-1.974791889113007, 3.685167130615266, 3.233387329547142, -0.44227396536899732]]], [[[3.8912421029242559, -2.4018156009847491, -0.96795683944897704, -0.048967246967174916], [-3.4591127308672762, -4.5688935135474935, 3.3667524419321619, 4.3314763590932355], [-4.3352936492893548, -1.3789197623655314, -4.9588656286066479, -0.33656839309267461]], [[1.700796288244649, 1.034551073706699, -4.8663927607781776, -4.2331102833096415], [-2.8210232911993884, 0.27168801398996223, -1.1640448982864338, -2.9396598791387341], [-3.188962162403739, -1.7846974182658437, -3.4079316127442039, 1.5678564652520475]]], [[[-3.4456128734508984, -4.5122521310695509, 2.3913761436239858, 1.6429127909218169], [-3.2132901488290178, 0.47672629403431444, 3.0596388721921528, 2.0185922338533491], [-0.86627844232635809, 2.8417914975703171, 0.69987032067108768, -4.5447991584989573]], [[1.5803678555938525, -3.2096733722147084, -0.38484527523040768, 4.1259319395988374], [-3.7090101199702499, -1.6079631791043623, -0.69167280630762917, 3.7643892970684121], [-3.3948176405491362, 3.014022635968546, 0.64575721190846291, -3.4346051602373273]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.32745309225768932, -0.26759473886658719, 3.7409531514881569, -1.174760605546971], [1.1470658036092172, -0.17293385735848055, 0.93954714687523855, 4.3883928354851216], [0.52483494443284162, -0.060892085203058756, -3.6857314232947682, -3.0394226925220535]], [[-1.9200992764887692, -0.7177053843739265, 0.38234462102610861, 1.8034820872893325], [-1.0963776864016765, 0.883130943077021, 0.83224575098602516, -0.4776479861873778], [-2.2947298130258438, -0.40196533749329894, -1.0492237421797308, -0.51858349653389069]]], [[[-1.0355093888566298, -1.2783869847200542, -1.3670438071072974, -2.2478223959314083], [-0.32334712822110656, 0.50697958742728333, -0.63859429955479141, 1.0155126552378739], [0.25843391888566741, 0.46184613386763823, -0.98835588095677052, -5.5923517139600607]], [[-2.4188100801014025, -1.0280286163351846, -0.91495230418549856, -0.094297512474127196], [0.1417815872382506, 7.6238755376509051, 1.7577853511133419, 1.155957455218934], [-0.36924755096738809, 0.063781096173629079, -0.98011989099381747, -2.0610032840829726]]], [[[-1.1636265776238075, 0.66854019513789886, -0.88348953285203313, -0.13857409330505457], [0.436292748161004, -0.20402940507569844, -0.50844963982689595, -2.1743393393802464], [1.8730429080324589, -1.5993355349373863, 3.3612353143265636, -0.71103306606934735]], [[-0.72647900204491489, -1.5228393510372809, 6.5555244114242184, 1.1486980876079085], [-0.67778707435907282, -0.35269852904624949, -3.5712151357890418, 0.52746866819509586], [0.99261980710746833, 1.0691356048726106, -2.7346219016818987, -1.2956505189259346]]]])+(1.-msk_ref)*numpy.array([[[[1.1539233807722518, -1.1445275020381072, 0.56411509447329744, 0.16155955849668432], [22.033651163531239, -0.574062954000643, 0.89877371700625386, 0.055058276329924535], [1.2261706709637155, 1.5289831573523305, -1.7728712307673886, 1.624520665210109]], [[-1.2364011214690493, 0.60101997910115412, -0.65839797426783375, -3.284795056513091], [1.1153880054684162, -0.33775031832423175, 0.22785783796072737, -0.67169119303122105], [2.4879036281509443, 0.35215307079873209, -1.0453198775408674, 4.3903597610096607]]], [[[0.53044288241371274, 0.25286012486020948, 2.48395355869516, 73.334841533402795], [-1.2220797505171288, -0.83463774098365906, 0.88823488801520323, 0.98664417446978803], [-0.83645957649424219, 3.4938103994976455, 0.22575134250349913, 8.4614025623599911]], [[0.63265831552570839, -2.5985565534608455, 0.25773329931193489, 0.25410199685954488], [0.12473160939817766, 13.333640333476678, 1.0044257572444544, -1.608126003713453], [0.29271401030939609, -1.8653242015331524, -0.29897241631301164, 1.2764790108173965]]], [[[0.97251045835492633, -0.27423594291090808, -0.58948555626271393, -0.67774774826121953], [-1.0171697389496914, 1.6689699448856667, 1.0591121907328738, 1.3036616561526411], [-0.11000498606520545, 0.22807180963287499, -6.9150372615967362, -0.23710295379467677]], [[2.9350754845961888, -0.22947901637097645, -12.305732399878286, -0.93744017956783832], [-0.74650239606787627, -3.0844411666113607, 4.1382652298574598, 0.75590010934918617], [0.22358735410302732, -0.17151212187185794, 7.4673127666871499, -1.0014270856333181]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=-4.0828553409
      res=quotient(arg0,arg1)
      s0=numpy.array(0.779963905734)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-0.191033931063)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=-0.990432344457
      res=quotient(arg0,arg1)
      s0=numpy.array([0.12622088399376885, 1.9040788493513627])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.12744018781306302, -1.922472403094545])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=1.4321797839
      res=quotient(arg0,arg1)
      s0=numpy.array([[-4.7587285458827786, 0.47686930818900386, -1.0085154439103441, 2.6970688635595792, 4.2554075377932943], [3.1003833805251766, 4.1473647288693485, -2.8093345619113097, -1.4436140693649135, 4.6286897627858856], [-4.2338574316602839, -3.4292906959864156, 4.9395730507921858, 3.852545540545405, -2.5665076548634147], [-3.5930735859268648, 1.4144425261146516, -1.996800371891406, 4.9498189921388462, 0.7746280219472288]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-3.3227173008426658, 0.33296749021991962, -0.70418215313987498, 1.8831915475126224, 2.9712802719532068], [2.1648004080058394, 2.8958408542638252, -1.9615795401471718, -1.0079838338685603, 3.2319194941994591], [-2.9562332042768835, -2.3944554549200654, 3.4489895097806405, 2.6899873771807705, -1.7920289643203089], [-2.5088146239165785, 0.98761520167713446, -1.3942386244647287, 3.4561435985771998, 0.54087345084402538]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-2.42041925821
      res=quotient(arg0,arg1)
      s0=numpy.array([[[1.7732304810651156, -4.3834344438689721], [1.5870800293344889, -0.32649837049473884]], [[-4.1357102873221141, 1.4729368164934034], [-1.080372428216676, -2.0465062175192816]], [[-3.3444796081810679, -4.7577844888280207], [2.3030602294717442, 3.9771272400633499]], [[4.1249217576342776, 0.94269039243065045], [4.9892297942993871, 0.819871845831182]], [[-3.9891784755456738, 3.2359940509237983], [0.57674761085908433, 1.2354573136809197]], [[-2.3814082269838144, -4.2041851466442095], [3.8471400153061488, 4.7010538025387518]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.73261294507189034, 1.8110227924366959], [-0.6557045949588689, 0.13489331213483699]], [[1.708675169930099, -0.6085461481509582], [0.44635755749859773, 0.84551724275806595]], [[1.3817769780331441, 1.9656860986756621], [-0.95151293382785007, -1.6431563360665322]], [[-1.7042178720255119, -0.38947400919634539], [-2.061308088414183, -0.33873133468533712]], [[1.6481353228439064, -1.336956000473434], [-0.23828417696770593, -0.51043112034861737]], [[0.98388253146939608, 1.7369656650952909], [-1.5894519109689591, -1.9422477269568508]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=1.00514133563
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[0.19898189866181148, -3.1505350387372144, 3.1374119453812703, -2.6079020680020424], [-3.3801689378724955, 4.679014638657474, 2.5724132588708892, -1.8836513080038464], [-3.9833321437837155, -1.9264831368183799, 0.69546827602643546, 3.2878646582655939]], [[1.0773414420981808, -4.4827249703062391, -2.2316657445955457, -4.1426341328328427], [-2.7424764178234415, 3.2143817685743397, -3.7643451009809636, -4.2409452490245725], [3.4838574999068719, 1.7184236175449508, 1.0463790467448044, 0.888756761937052]]], [[[-4.0646289742202359, -2.5010708717921784, 2.3198665788256934, -2.096181287710861], [4.0789437319305808, 1.5631457560905542, 3.3186437865790417, 4.1479938797007936], [4.5824030950828956, 2.3228114644226761, 2.8051022378581845, -1.5031788824053072]], [[-4.9778300723551103, 4.1368161922875775, -1.3795626113565862, 4.1668327554847657], [-0.26623252807888242, -0.47547808459652607, -1.3615218402345288, 0.3213821443916185], [2.429365765527411, 2.6211271205158155, 0.79834853173596088, -4.0402745310012156]]], [[[4.5327021619624901, -4.5539303036442735, 3.6119639048330914, 0.15241792538588328], [-0.65168366061055671, -3.0060502550825854, 0.14087370722639392, 3.6787456205473283], [2.0011731604583529, 1.4452140767883881, 3.4686672463635055, -3.7914443112896903]], [[-1.4891872019687025, 1.5251231257809827, 1.6946894548108871, 1.1926385160648412], [0.16354406055567505, 2.6210408034432584, -4.4000135826741502, 2.6008238371451267], [-4.1515077174543507, -4.3653614145622219, -4.1209246997356797, -1.0037728397852264]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.19796409878799806, -3.1344199338636729, 3.1213639656223826, -2.5945625511234667], [-3.3628792469932169, 4.6550813033097667, 2.5592552685819099, -1.874016360923292], [-3.9629572504865589, -1.916629103327173, 0.6919109297133218, 3.2710471072380716]], [[1.0718308002197197, -4.4597956639733427, -2.220250690620694, -4.1214444038876055], [-2.7284485480987635, 3.1979400852833209, -3.7450903346201283, -4.2192526550331815], [3.4660374381442907, 1.7096338162974118, 1.041026778679718, 0.88421073776985992]]], [[[-4.0438382445866274, -2.4882778004885671, 2.3080003742760735, -2.0854592418138709], [4.0580797817744569, 1.5551502069275374, 3.3016687991559266, 4.1267767354493277], [4.5589639314036683, 2.3109301966727895, 2.7907540347154542, -1.4954900660500496]], [[-4.952368284857954, 4.1156562223264119, -1.3725060968635625, 4.1455192496771742], [-0.26487073871358818, -0.47304599636258127, -1.354557604962354, 0.31973826267068056], [2.4169394685314072, 2.6077199569976703, 0.79426494905670919, -4.0196083752580059]]], [[[4.5095172204200429, -4.5306367793610329, 3.5934885740051055, 0.15163830198160411], [-0.64835027424729252, -2.9906741952958615, 0.14015313293092826, 3.6599286987390367], [1.9909370846952252, 1.4378217525876396, 3.4509248833174588, -3.7720509314518593]], [[-1.48156995358359, 1.5173220637979123, 1.6860210546958041, 1.1865381253292049], [0.16270752655275347, 2.6076340814401764, -4.3775073481908091, 2.5875205256828298], [-4.1302725997909997, -4.3430324272182883, -4.099846015354875, -0.99863850406654309]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(4.25991348996)
      res=quotient(arg0,arg1)
      s0=numpy.array(-4.43282428706)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-1.04059021328)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(0.0917406250642)
      res=quotient(arg0,arg1)
      s0=numpy.array([-1.9512586922023378, -0.22473973961584015])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-21.269297989160297, -2.449729762126243])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([-4.3135274828405903, -1.0172619544385206])
      res=quotient(arg0,arg1)
      s0=numpy.array(-4.51021976132)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([1.0455989394428085, 4.4336856810937659])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-0.943237730597)
      res=quotient(arg0,arg1)
      s0=numpy.array([[-2.433848765862856, -1.5984014408053584, 2.6937885319126531, 4.403443429978628, -4.9814809970324525], [0.47414232696094416, 3.2633708002235498, -4.9195630369378041, 4.5808686501840601, 4.3910379677996474], [-0.65026279403872778, 2.8902014583927489, 2.5115120906868516, -0.99293647634315274, -1.7572073963383805], [-3.6183714804068079, 4.57942065581652, -2.5706231406790248, 3.0464050890212704, 4.3885826921445155]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[2.5803131987961514, 1.6945902278454283, -2.855895650194888, -4.6684343587379242, 5.2812571374532737], [-0.5026753188304649, -3.4597543062202201, 5.2156130711843849, -4.8565366943952037, -4.6552823592190711], [0.68939438377525675, -3.0641283365142842, -2.6626501561767815, 1.0526895226236705, 1.8629528265642601], [3.8361182584535976, -4.8550015624548992, 2.7253183977819373, -3.2297320073190972, -4.6526793297011819]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-1.3280325716055152, 4.1119316992821311, 3.4651213596900359, -4.0645505357877143, -1.6905016889460169], [-1.7288216210334708, -4.9345720113387355, 3.2105914053535525, -1.3909794174481735, -0.46094533721180397], [-4.7403742377195286, 4.642302846382087, 1.0201989560936315, -0.027381229275199814, -3.4321061901487102], [-2.3240550096887738, 2.5845808374073211, 3.8609054484024945, -3.1085327733685753, 1.160721508702764]])
      res=quotient(arg0,arg1)
      s0=numpy.array(-0.544744542305)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.41018914291121111, -0.13247898606879593, -0.15720792600284761, 0.13402331635654416, 0.32223838986204839], [0.31509586395581718, 0.11039347304149488, -0.16967108969291289, 0.39162660171090169, 1.1817985742086556], [0.11491593595511151, -0.1173436030201211, -0.53395912537577861, 19.894816877284359, 0.15872018874842406], [0.23439399671438391, -0.21076707465319183, -0.1410924327427997, 0.17524169182708119, -0.46931545441408318]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(0.120762390932)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[2.9177811792495101, 4.9926390136577314], [2.8212576099947242, 1.0496330226196502]], [[4.4887713739618267, -1.1110025414470814], [0.63895828672982358, -1.5559611097413351]], [[1.6131730259759784, -2.7990258291451307], [-0.68480419907631429, 2.2160021353148265]], [[1.7667980053444765, -1.2319392488576231], [-2.0662887540586263, 1.8003223772693993]], [[3.3991006555548395, -2.241747298922534], [4.4998340036374742, -4.5535698028071785]], [[-4.0030197716447766, -1.9715411743814517], [0.51984049117265752, 4.5866272801160388]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[24.161339939779833, 41.342664509412863], [23.362054926376047, 8.6917211105100876]], [[37.170275773092825, -9.1999051432522787], [5.2910370670734981, -12.884484132290943]], [[13.35824020645514, -23.177959690450272], [-5.6706744027759051, 18.350101535849607]], [[14.630366223361817, -10.201348609857735], [-17.110366382374075, 14.907972286525885]], [[28.147013563749496, -18.563290123861222], [37.261882353504717, -37.706853662494005]], [[-33.147900937947121, -16.325787848064515], [4.3046555070710601, 37.980593500317063]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[-4.6637794313652003, 1.6199946245508787], [4.1205840662652164, -3.9107130073923413]], [[1.2780548498313768, -4.2270047773737023], [1.3682530836445643, 4.3634898718690494]], [[0.095094313455357238, 4.8671395292555495], [-4.2709438429468518, -3.342037281781872]], [[0.4125016124804457, 4.330556777639508], [4.391171781647218, -2.7447805015170701]], [[4.5365606029450429, 1.7852502888925503], [-1.6963363654112529, -2.0436982533774239]], [[-4.64194878194788, 1.9279771573930411], [1.6685392206615282, -4.7193537932285166]]])
      res=quotient(arg0,arg1)
      s0=numpy.array(-1.75514503828)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.37633534435053134, -1.0834264581367594], [-0.42594569363283558, 0.44880435740491131]], [[-1.3732939853945367, 0.41522191970841449], [-1.2827634443202818, -0.40223424135643798]], [[-18.456887425784029, -0.36061120247896511], [0.41095015594181605, 0.52517218998285931]], [[-4.2548804299788019, -0.40529315937855898], [-0.39969856010036509, 0.63944823176485699]], [[-0.38688892134238068, -0.98313667792011517], [1.0346680493713507, 0.85880830762427285]], [[0.37810521415129539, -0.91035572260151032], [-1.0519051734258202, 0.37190367901556509]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(1.09744499826)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[-4.9420008922629064, 2.5441648186946768, -4.457337337293799, 1.3071387865484123], [1.728327308224662, 1.8983772293153205, -2.1632564914723709, -2.6084592542685323], [2.3042302240907464, -0.24871647789291718, 2.4320317737352859, -3.001684439734956]], [[-3.6797317245070813, -3.3939758349642246, -0.56810149300502122, -2.7860062636755112], [2.93146328628875, 4.109512083266786, -1.5882947168074169, 2.9261790755529704], [3.6370933019522464, 4.2152598565163828, 4.3728924529904578, -0.62082063257723163]]], [[[4.3463232484640528, -2.9099807085231957, -4.9664131685007327, 2.807863958697963], [4.9233231365152506, 2.4758727496123489, 3.2025306463658847, -4.7619518097362699], [1.850973840527816, -4.6052453371285615, -4.9420488762684611, 3.2330999011305614]], [[3.3502772019340714, 0.14351414614128544, 0.4799263776907452, -0.50129337016207831], [-0.45010641932629092, 3.9371009685461331, 0.80408987132139398, -2.4524383287663376], [-0.65706914019904339, 4.1853420105393351, 4.0487532461315059, 1.3226251786224932]]], [[[3.7527610334891381, 2.0996472830462896, -4.8684136136305796, -4.9172766955133698], [0.62330532062699717, -2.3586493511719677, -0.062264153141745382, -4.7877835904823653], [-4.9319185360737539, -4.4309087125440394, -2.0373824560734044, 4.8165196141255748]], [[-2.442376480822511, -2.8636159244711203, -4.7926121717236256, -1.0630367333545054], [-4.5500749975956021, 1.1899678336835029, -3.3000153288440681, -0.25584337364900911], [4.0784444734953347, 0.3672611099728007, -4.5055460796855762, -3.9784097794554163]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-4.5031877680407888, 2.3182618014825107, -4.0615587517917282, 1.191074530952017], [1.5748646273522604, 1.7298153732759658, -1.9711753162133949, -2.3768473667507313], [2.0996316241302955, -0.22663229436300736, 2.216085341489384, -2.7351570643572707]], [[-3.352998765624343, -3.0926158853926511, -0.51765828256144031, -2.5386295150027589], [2.6711710299249951, 3.7446178075271637, -1.4472658942592667, 2.6663560180162831], [3.3141463196043328, 3.8409759607077243, 3.9846119485922853, -0.56569635249276873]]], [[[3.9604018929006326, -2.65159594616261, -4.5254324147215135, 2.5585464083842977], [4.486168458847831, 2.2560335629916595, 2.9181696134593511, -4.3391257122547859], [1.6866210547792269, -4.1963336152876147, -4.5032314914215785, 2.9460245445157036]], [[3.0527973677453621, 0.13077115150991644, 0.43731246527304662, -0.45678222686037578], [-0.41014029863897544, 3.5875155245025216, 0.73269263844304544, -2.2346799453769615], [-0.59872626076086144, 3.8137145981570022, 3.6892539057083011, 1.2051858459597227]]], [[[3.4195436121522613, 1.913214134990717, -4.4361344954403528, -4.4806588970825798], [0.56796041862246505, -2.1492187352543088, -0.056735556898481053, -4.362663821943241], [-4.4940006505040229, -4.0374767934323961, -1.856477964092317, 4.3888482992416611]], [[-2.225511515105032, -2.6093480119831494, -4.3670636608863456, -0.96864693450674177], [-4.1460619938210064, 1.0843074920109657, -3.0069983772085389, -0.23312637449220772], [3.7163087717024044, 0.33465103996557866, -4.105486914449723, -3.6251564185562288]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[3.2443070682388271, -0.12185265918424637, 0.7306892425517244, 3.7132604038608399], [-3.4563595959096038, 3.4691018951267907, -4.8155302824712329, 1.1026541396172451], [-3.7202207116026234, 2.2087992230554256, 2.5797751466923273, -2.2922227596483422]], [[-4.7338336202727147, -2.9497202678665033, -3.7798747679642708, -1.2009881171885537], [-2.259731944552076, -3.9362093172376831, -4.6588191331466273, 1.4958099465042238], [-0.86896051982482714, -4.4856821327260796, 0.0033061610448381629, -4.3260632485114563]]], [[[1.3002602599373114, 1.2734765812316375, -2.7599514646606318, 2.5686797515203175], [-1.9793334499327253, -0.52326241998106404, -1.2228808758624576, -3.1502257097658006], [3.3330111230527937, -2.3250262334822658, -1.1457759629682238, 2.7744197963623849]], [[4.5870694903637013, -0.37157918447487948, -2.5114597402265284, 1.7570310453042257], [0.62815522514120126, -2.1378476638621202, 3.8720481763973815, -3.5725078041562011], [-4.1815272537104118, -0.25514488933936619, -0.46601051781987923, -1.8663356193085292]]], [[[2.3119147733982892, 0.88471861765160664, -1.7875997926931642, 4.4963369225639429], [4.1345646840714885, 3.2041689525716794, 4.0523268024056396, 2.8861939928314779], [-2.1751506584732896, -1.9879750231235271, -4.6698496442826629, 4.9767528593953312]], [[0.88881120483778187, -2.2589805109613548, 3.0059798589097397, -0.0043535912631966767], [-1.370045325718884, -2.2473828957383404, 3.9350383632351509, 4.4891497438637025], [-3.5822566001168834, -2.4693286282092188, 1.465576056886122, -2.7497397754428112]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array(0.523258413612)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.16128510729899054, -4.2941895327912682, 0.71611621348692833, 0.14091616442189669], [-0.15139003888108687, 0.15083397070199009, -0.10866060078917143, 0.47454446032679548], [-0.14065251880885304, 0.23689722820888662, 0.20283101582813184, -0.22827555106033739]], [[-0.11053586914650115, -0.17739255457950223, -0.13843273804903969, -0.43568991742956742], [-0.23155773624977138, -0.13293460063729032, -0.11231567456415871, 0.3498161078783113], [-0.6021659231621429, -0.11665080095494267, 158.26767254085357, -0.12095486902366007]]], [[[0.40242590636200798, 0.41088970250690077, -0.18958971572933561, 0.20370714305746876], [-0.26436092090979285, -0.99999234347979438, -0.42788993101452805, -0.16610188025247041], [0.15699269948205463, -0.22505484285571772, -0.45668475384684282, 0.18860102364387341]], [[0.11407248455926146, -1.4082016309693133, -0.20834831840249837, 0.29780829144152754], [0.83300813663398376, -0.24475944776461864, 0.13513737168906328, -0.14646809532592811], [-0.12513571761309641, -2.0508285114650708, -1.1228467890804572, -0.2803667294340329]]], [[[0.22633118644016179, 0.59144049098990092, -0.2927156378908734, 0.11637437821572218], [0.1265570751928585, 0.16330550022707235, 0.12912542327562115, 0.18129703509584055], [-0.24056191766460949, -0.26321176449673073, -0.11205037709349001, 0.10514052604077828]], [[0.58871716598945578, -0.2316347622623311, 0.17407249488409976, -120.190064243092], [-0.38192781201396442, -0.23283011301902107, 0.13297415814304314, 0.11656069489039071], [-0.14606949529933158, -0.2119031090613665, 0.35703258875798538, -0.19029379371999242]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.1983567439406988, -4.2833372915672445])
      res=quotient(arg0,arg1)
      s0=numpy.array([0.024047013471215628, 0.78201928339541027])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.12123113635301848, -0.18257242662981477])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=numpy.array([0.04771912528749489, 0.48865757280829403])
      res=quotient(arg0,arg1)
      s0=numpy.array([[0.28233119274339824, 3.4504356084143382], [-2.6193510032276537, 0.62500402618820061]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[5.916520704066321, 72.307184752997713], [-5.3602996228511435, 1.2790224913456869]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_array_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[0.6212685035907537, 3.2517047538772985], [-2.2541932049896407, -4.8552582069546588]])
      res=quotient(arg0,arg1)
      s0=numpy.array([0.85265271657907249, -0.30658168630965488])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[1.3724383445337796, 0.26221713873695895], [0.13600506186915942, 0.063144259942861145]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_array_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=numpy.array([2.7941505646436839, 2.5619560092254225])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.6520434156288242, 1.8278489464615717, -0.30175330451418425, 2.0983615054017211, 3.7968546538813559], [-2.9919709283757765, 2.2914163309949398, -3.9013649975196527, 0.80698383107204474, 4.5131892818839621], [-2.9360703992027792, -0.16394559474543335, -4.4511830614357297, -2.8817606054348319, 3.4756772567601217], [-1.0590225910416331, 0.38089209290513892, -3.7776009902416483, 4.1887685937105985, -3.6395326695231289]], [[2.3147904679396314, -0.99934200862554334, -4.9645072391393601, -4.9425764477475331, -1.9970809902390263], [-2.6380077126176573, -0.45343682048289047, 1.6190208990532104, 4.3814643798568262, -0.41777645756230264], [-0.00031460858805232306, -3.7339157390907185, 1.7333442072793934, 2.8595841304522196, 2.1136638592077137], [-3.2419433101483062, 1.2770797009970911, 1.1504286802556702, 3.3913152360752843, -0.75620559966390566]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.94914119846903044, 0.65416981088657367, -0.10799464722212079, 0.75098369141367605, 1.3588582884278246], [-1.0707980329461304, 0.82007618343471267, -1.3962615497125737, 0.28881186335602965, 1.6152276613123364], [-1.0507917634628945, -0.058674574240898163, -1.593036222800454, -1.0313548031017863, 1.2439119425918794], [-0.37901414635352049, 0.13631766939292017, -1.3519675847258346, 1.4991205723535372, -1.3025542415561453]], [[0.90352467396170522, -0.39006993290555475, -1.9377800482375656, -1.929219873389576, -0.77951416146400598], [-1.0296850153235957, -0.17698852706685694, 0.63194718926602589, 1.7102028153799216, -0.1630693329853905], [-0.00012280015227405925, -1.4574472495410349, 0.67657063627858693, 1.1161722215975056, 0.82501957551049254], [-1.2654172431042132, 0.49847838776248199, 0.44904310460954749, 1.32372110366587, -0.29516728505129003]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_array_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[3.9576981257543213, -3.5465111910443117, -4.1396824642320791, 4.550109472492208, 0.27390961522626522], [3.6402466184128173, 0.83994461270622978, 1.2497664407915829, 0.076140590805492181, 1.334436577986061], [-3.0631879247655913, -0.76502732466948853, -4.5406233135574512, 2.5257135248552469, -0.77460903908343326], [-3.4184028691274371, -2.8527983589382, -2.7465586921890486, 4.1584953782465206, 0.41923414116447155]], [[-0.65850302024985563, -1.1524691143386567, 2.6056494528703293, -1.9280049102217642, -4.8706245396976469], [-1.0079690655863094, 0.77766055741288653, -1.9050107170761734, 4.3706887417108149, -1.8188503897008212], [4.4988292786274435, -0.75972188909336413, 2.4496669086794221, -2.3520520235233255, -3.365219601313536], [-3.1807598193312305, -4.839166934443746, -0.35496821375479914, 2.8623554988033053, 4.8035466363254535]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([3.5496527301947456, 2.6250647817942045])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.89689830234795787, -1.0008858111482537, -0.85746980858185573, 0.78012468747274177, 12.95921184534733], [0.97511325530533111, 4.2260557142667627, 2.8402528779268947, 46.619716141455285, 2.6600385426723698], [-1.1588099775061567, -4.639903197873732, -0.78175450484873898, 1.4054059160958021, -4.5825087897178696], [-1.0383950827600408, -1.2442704613430555, -1.2924001006385264, 0.85359063972111449, 8.4669934570099006]], [[-3.9864126679300225, -2.2777745183224241, 1.0074512436438783, -1.3615446557614124, -0.53895855868150333], [-2.6043108577615648, 3.3755920327594398, -1.3779790099150604, 0.60060666337147539, -1.4432549244613768], [0.58349953270400401, -3.4552970231342166, 1.0716007031377734, -1.1160742855772006, -0.78005749781368527], [-0.82529487635005916, -0.54246212568319907, -7.3952108388147577, 0.91709949476635333, 0.54648470818267891]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_array_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=numpy.array([-2.3996087712674194, -4.4344873520663661])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[1.5397373777787626, 4.7059732451735687], [0.33384105295121635, 2.9852620258916707]], [[2.6379715212323607, -3.2329101106815994], [3.8872213655081129, 1.8425529762679247]], [[3.1073989065615724, -3.8812247706614302], [-0.20461340700219921, 3.1696581333305502]], [[2.3854828918886231, -1.2251671420244312], [-4.1156898879708725, -3.293815817922888]], [[-1.4446637723767033, -4.5456161859781332], [2.3368867453994877, 2.7483071654435509]], [[-2.4987110672848445, 0.77221328073026374], [3.6184111623877655, -0.81391446002113366]]], [[[-0.092486598103647566, 2.7766227340108909], [4.2780243441656864, -4.8697096440611105]], [[4.11756873760792, 2.7515665051299001], [-3.9154057540321805, -2.7837255709869604]], [[4.079380277853808, -2.88226612678395], [-1.199039033163606, -3.5499086975225422]], [[-1.3943203985273058, 4.071811904368424], [-1.7687036716242623, 1.7265984638867504]], [[1.049290615818947, -2.3704262272226186], [2.8335362576208407, 1.4833101197535745]], [[2.1280888091358197, -2.9891787298674144], [1.3063672271650315, -0.45946130943514518]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.64166183930287435, -1.9611418750932383], [-0.13912311746338926, -1.2440619744504944]], [[-1.0993340051174438, 1.3472654998564824], [-1.6199396385165612, -0.76785557642995683]], [[-1.2949606384878791, 1.6174406499653919], [0.085269486198005104, -1.3209062124141211]], [[-0.99411325731596845, 0.51056953812405237], [1.7151503766995557, 1.3726470153645789]], [[0.60204137844256356, 1.8943155402691918], [-0.97386156167665472, -1.1453146856068361]], [[1.0412993556299937, -0.32180799219299322], [-1.5079171262057864, 0.33918631643909281]]], [[[0.020856209694802941, -0.62614289174081206], [-0.96471677660152266, 1.0981448941988619]], [[-0.92853320140584694, -0.6204925815940675], [0.8829443954119508, 0.62774461848218155]], [[-0.91992150478294032, 0.6499660271758092], [0.27038954854722547, 0.80052290505877055]], [[0.31442651378351222, -0.91821479713344001], [0.39885189227117501, -0.38935694857314146]], [[-0.23662050029977025, 0.53454346332007374], [-0.63897718781416302, -0.3344941595249758]], [[-0.47989511304935412, 0.67407537614793911], [-0.29459261543643717, 0.10361091890837101]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_array_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[[-3.5669275870918016, 2.0547650653569267], [2.5172651756011764, -4.5270915157496159]], [[-3.3203373393730029, 1.4222113783499388], [2.9101595696029756, -4.7056396539354406]], [[-4.3812760948309037, 3.2706514248061147], [-2.4400732412389092, 3.4052509560675635]], [[2.3409396694201066, 1.850236556508027], [-0.067475891633186791, -2.3168113504526069]], [[4.4962105652396041, -3.969113981512856], [1.9784300151598968, 0.10999382415785064]], [[-4.7298718776834479, 1.5761091070208844], [-2.9902782701943043, -1.0969249443179221]]], [[[1.7480610764366737, -2.2013538991695878], [-3.8847144391294588, 2.712792208384859]], [[4.7713521439698052, -4.5236390150357568], [2.9227086866001386, 2.4859472817780048]], [[-3.0411659363012, 2.0062094174450342], [-4.4251501135818918, -2.8061425281440733]], [[-3.7800277917709457, -4.4778428357705646], [-3.6005052865098652, 3.2821335510610101]], [[-2.9172560353518371, 3.446385115446235], [-3.098363027436517, 4.3312946782393151]], [[-2.4596599911661854, -0.85794615201851165], [-0.17901688666218796, 3.4894890242142456]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([-2.6995719603451529, 1.1952859604024546])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.7568339682909504, -1.3138105206573671], [-1.072422558620761, 0.59631486373832354]], [[0.81304147272425265, -1.8981510072554884], [-0.92763709198030242, 0.5736886287260512]], [[0.61616111423111386, -0.82539274588247646], [1.1063487418002613, -0.79276740398089796]], [[-1.1532001424940124, -1.4590415213933976], [40.007947950070772, 1.1652100892106605]], [[-0.60041048371169692, 0.68014473076839987], [-1.3645021252505478, -24.542941215236176]], [[0.57074948966002936, -1.7128077925060692], [0.9027828571184241, 2.4610361669036265]]], [[[0.68377814512006596, -0.5429776470077573], [-0.30768953011390743, 0.44061095306451925]], [[0.25051304626783771, -0.26423106627861787], [0.40896513767606424, 0.48081709904465852]], [[-0.39303542964716159, 0.59579321580729383], [-0.27011195772406077, -0.42595340344062743]], [[-0.31621089215390724, -0.26693343295885574], [-0.33197728243334979, 0.36417956241179045]], [[-0.40972953553536706, 0.34682309735071204], [-0.38577982948350459, 0.27596505183718928]], [[-0.48595576815303648, -1.3931946166904241], [-6.6769453021379324, 0.34253896547836432]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[-1.3012819930354178, 1.9885839707205522, -4.2867215891323767, 2.5321889725424764, 1.6465101246929894], [4.8977994219209311, 1.6600096771903576, 3.9476755430578034, -0.50888176650157213, -0.66014654237845605], [0.84273129779502298, -2.8988796050135335, 3.088143788813797, -4.1268082318144215, -4.9259979173940511], [-3.1327854371500594, 2.5345495378066385, -0.28401869926918533, -0.027815068306703772, -4.9527938817851638]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[1.1045476437177149, 3.1016602847431489, -1.6614896359400175, -4.7140577701353106, -4.5631275760831436], [-4.2575163823501203, 4.9676822739268616, -4.4558520339238559, 3.1070778587023575, 1.0680867158685858], [-4.5357811651946012, -2.8368074215129258, 1.6177052454449043, -4.2754879883036976, -4.8929974119755526], [-1.4957562854855446, -0.49502758138027048, 4.461510188591534, 1.5789602969237997, 4.1704193224403809]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.84881497602314993, 1.5597331218652435, 0.38758981692494276, -1.8616532262211463, -2.7713935721676726], [-0.86927128197510184, 2.9925622375496577, -1.1287280287661199, -6.105696967809827, -1.6179539652216512], [-5.3822388904533556, 0.97858752623142553, 0.52384388683737082, 1.0360277842190662, 0.99330074718424644], [0.47745251486046741, -0.19531185877260857, -15.708508630141404, -56.766364170432432, -0.84203369289763641]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=numpy.array([[2.5751144984612679, -0.3692426068645327, 4.4302851582235476, -2.6235048277887718, 0.4915284540660565], [0.63768736845198504, 4.5581961163186158, 2.4650196927053081, 0.046722304018893546, 4.1085595170078051], [-3.380648108085146, 4.5438537821289504, 1.4555188409286037, -2.1614014233449739, -4.9167838392524699], [4.9339457613438427, -3.28229443378277, 1.2675519197268414, -0.12692998060530947, 2.0985757770918534]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[4.4946803296276254, 4.6325746234547971], [0.1765440358714141, -3.5854036389234789], [-4.047851885161009, 3.475598522309614], [0.33360391164742875, -1.570780837364627], [-4.4414771570466769, -0.88442802389905406]], [[3.9933053437272257, -0.37554191231879308], [-2.2221830191748406, 0.51329082449350771], [-2.1712667739535596, -3.6058460741671974], [1.1020344552028387, -2.9987684454744756], [-0.11382723091627511, 2.3342421080531821]], [[-2.8365431298039634, 0.32645828185508208], [2.3723632785248938, 0.43018638665628384], [2.5091572812809897, 0.42591104384957035], [-4.1660589308382248, 4.9370830763563731], [-2.1046505481815361, -2.497000815479824]], [[-2.9502505176461558, 3.6136024431530505], [-2.3558357320842549, -4.5879431484907371], [3.9980154220519246, -4.3159561928239132], [-2.2387343209468935, 1.8663012757506756], [2.4115844438896001, -4.7105018812428323]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1.7454293128765239, 1.7989781138753025], [-0.47812476834826478, 9.7101568786152779], [-0.91367750395193825, 0.78450898716037887], [-0.12715963321805956, 0.59873373234405824], [-9.0360529900264677, -1.7993424726133878]], [[6.2621678604378745, -0.58891226468926006], [-0.48751369236161035, 0.11260832386212953], [-0.88083141095341078, -1.4628061937346455], [23.586903050782738, -64.182803233801039], [-0.027704900086046112, 0.56814124229924057]], [[0.8390530570218464, -0.096566774008310888], [0.522103789487117, 0.094674346333109088], [1.7238919969460356, 0.292618021748069], [1.9274804235072878, -2.2842046012516124], [0.42805431700685048, 0.50785246964598285]], [[-0.59794952363696141, 0.73239606147774627], [0.71774052560214952, 1.3977853727166203], [3.1541235982771423, -3.4049541684683065], [17.637553478466753, -14.703392113120739], [1.1491529017987168, -2.2446184372577251]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_array_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[-0.21405220621448695, 0.59473645637279215], [1.3489925224703363, 0.99233093859926846], [4.8908854792134431, -4.7730319852855221], [-0.65496660976616639, 1.770355974596697], [1.4229315359944046, 1.3236869321775204]], [[-4.8873079624154405, 1.5356327943027974], [-0.46124772356053079, -0.98356146077842865], [1.9524472629104617, 3.4101083152069052], [-1.8543184182549544, 1.9094941868713677], [0.27504224990356541, 0.46721939475244589]], [[2.5012257837096152, 4.4615670722725902], [-2.9988031494096723, 1.1669295333591716], [-0.54618655136156935, -0.22321798921683378], [4.1507001442764384, 1.5044800129572877], [2.1707314978264076, 0.62148159830964467]], [[-1.3706673579331063, -3.8711447679588531], [-1.3199149836277559, 3.6480678419839414], [3.0438808056111117, 4.9566059147848236], [-0.79744880632400506, 3.6074264583149791], [1.6933697201372997, -1.691004450030662]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[-3.8576684069104097, -2.4931077058919762, -3.2904366566908063, -4.8303074962497403, -0.57057322490438445], [1.6211962532641202, 3.3858713522685076, 0.10390862348653496, 0.41892676682865204, -1.1313070389379098], [-1.3026662420897428, -4.4941439082117629, 3.5221456054279585, -1.48002780697992, -2.7401740192275739], [-4.6324044596969642, -4.0198290657094375, -0.47334418132605727, 2.9203578312830114, 4.6045089605640559]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[18.022091316568382, -6.4863493158595773], [-1.8481256673880477, -2.5123752660691396], [-0.67276910708201199, 0.68938081010868668], [7.3748912146441139, -2.7284385544834437], [-0.40098431334972401, -0.43104846851193701]], [[-0.33171559184146049, 1.0557186973857053], [-7.3406787271095775, -3.442460372114212], [0.053219682528910468, 0.030470769219607736], [-0.22591954149001656, 0.21939148582329404], [-4.1132118404883826, -2.4213614666774008]], [[-0.52081113611332353, -0.29197504396727653], [1.4986458544624561, -3.85125560690433], [-6.4486128350244529, -15.778950512839488], [-0.35657304925309719, -0.9837470715683998], [-1.26232747899561, -4.4090992021011051]], [[3.3796708099056101, 1.1966497605666921], [3.0455212006617502, -1.1019063350322236], [-0.155506805803135, -0.095497642835421198], [-3.6621257792647119, 0.80954050346659157], [2.7191397754476907, -2.7229431362410459]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=numpy.array([[1.6169577457585333, 2.9326596860551488, 0.46422804015435659, 3.4770811963516763, -4.142304035410314], [-2.1766472812859194, 0.069187631309707065, -1.5799532020820575, -4.5982262380483343, 2.9436528396462061], [-0.97541361365884249, -0.6332984940016928, -0.17278467962978628, 4.2060667419577822, -0.042545152223406824], [-0.47028624712939671, -4.5738278622260928, -2.5487987344471899, -1.726807934940461, -0.80305159940812221]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[1.8400223525382229, 2.039875790583924, -0.87195171841007202, -1.2067221198267006, 1.3338599697672917], [-2.2151503856123833, 1.9667767655662347, -1.0399511604601654, 0.52740736682804013, 0.16411281319770676], [-4.5983512868926741, -1.7297846090407565, -1.0945994435310693, -1.8795392901365315, -1.076336218481222], [-0.7569330148529998, -0.81093000301440199, 0.11134080749144815, -0.87158745005469296, -3.5816115095961285]], [[3.0848737454827013, -4.0679461731598101, 3.4099742806431816, -2.9033434972231342, 0.8491638524589904], [-2.1314980375780914, 1.9943698471076718, 2.3027299958185168, -0.29397032520773614, -0.93289014262587777], [-3.3659176846566039, 1.3166040741164853, 4.4343033106299394, -1.2900756722202047, -3.9214931624269322], [1.0530645486169012, 1.3960968063551817, -0.67875773653465199, 1.940191362732814, -3.5293718767265658]], [[-3.2417789860944248, 1.4474038180085955, -2.0061644889302119, -2.3927616799183804, 0.89814712716610146], [4.7697256741411707, 2.6914877077432635, -1.8368130563708429, 4.0314028274228164, -3.848120858718608], [-0.59670512268754194, 3.3811623069949022, 0.0031767252670933388, 0.91465063134186231, 3.5761639661048292], [2.8884519144201359, -0.26282790725772998, 4.2586528045209775, 1.163741219509741, -0.61725318579947341]], [[3.7268630431831067, -4.1666398549414376, 4.2691467895315487, -2.820577257557094, 0.41497440089923732], [0.026185384294588943, -0.61409680985153869, 0.47988222378212075, -1.0734368278448549, 1.9914054993074082], [-1.2890087148137122, 3.5921539918796501, -2.1191311297634172, 4.4227970069191116, -2.0495822103243935], [-3.8934452792567189, 2.2268329640074471, -4.2611645563440659, 4.6130714916551945, 0.85624041674340212]], [[-4.4952547014676858, 1.5923629440886291, -1.0904974553600355, 3.411923131775616, -0.86726172929343015], [-0.47802275353987955, 4.9808873441182335, 0.70659579257490002, -2.1298485948749657, -1.0438278410135182], [0.76818036666325717, 0.15569616082480309, 2.0213697283387955, 0.50091906386255758, -1.5740221351664152], [-2.6972528433100496, -0.88034102711814999, 2.7214480190254919, -2.2555797775468633, -4.7252013776799178]]], [[[-3.4795903358024716, 2.6232692343611888, -0.95025155771885661, 2.2164291393453723, 3.8715241578038722], [-3.4165198591411849, -0.03605943620854557, 2.0410729757833028, -1.4086472418971017, 1.4541358420872674], [0.57082282053031452, -3.6412677078252944, 3.6024700084347074, -3.672402670377708, -2.6117469554040174], [0.68913628721000375, 0.65154568484046571, 1.9898988437651148, 4.8659507973278409, -1.4538238770078493]], [[1.1313079221294782, 1.1756103877879216, 2.2012711873102555, -1.8007613721035973, 1.9638311943175859], [-4.1230235329775002, -0.94718106776525524, 4.0520060954270907, 2.2512037584714042, 1.9165277215113399], [0.52163893868005573, -1.0291889776031802, 3.3307642696099489, -3.2858450420558603, 4.4944592710833913], [3.9268721660983665, 2.7180515349019263, 2.9848565866631507, -2.5299456975050107, 2.4927786714759499]], [[1.5717096225199985, 0.90038226805589172, 0.4074158379630628, 3.2597437293213751, 4.3932852403241416], [2.6312651498035891, 0.12199696732750098, 2.3519321317468318, 3.5345983840062321, 4.220263627179957], [0.39409512821691717, 2.0861752536890696, -3.0433598483115354, -4.2676337476297324, 3.9601380743640018], [-0.92635896890434033, -3.4470619568912464, -3.4734578177517528, 4.5808062448830515, 2.6715168427725269]], [[1.0667495761555177, -4.8504807999089543, -3.9513953018103285, -1.0113140201676494, 0.45217515405008868], [-4.7541132691192018, -0.87409477602298935, 2.1003408770196339, 4.8710397848928171, -3.2064880710127075], [-1.0755356332139652, -3.1682417713858504, 0.0533350337238927, -3.4616054241840089, 1.5703359018037943], [4.5867712916296366, -1.1775339358097279, 0.81367914844447053, -1.7285583440905352, 4.2923653814727682]], [[0.65724678583018914, -1.0199617609023184, -1.0165027025095217, -2.2669378568133789, 3.4734471794086534], [-2.4070137981655715, 4.0959536220415842, -1.0990332448262108, -2.5786955381434149, -1.3498362241371762], [1.4253350069569457, 1.6449920670073981, 1.1438899852293227, 3.150111378386466, 2.9963278187691404], [-4.4568041386003001, -3.9328296279910702, -0.24155467529255858, -2.4308752383821353, 4.1500143931346773]]], [[[-4.0803494539292693, -4.0029500671242237, 1.9884120887484418, 0.24118916627254272, -1.2422014127904122], [3.9085175833108892, -2.5388150128021705, -0.63166791451714488, -0.12964790830840833, -3.5655375984648341], [-3.8312126781926361, -4.5356185456075071, -3.832293189271466, -3.1221707325936245, 3.9721592773256287], [-3.744166354456425, -3.3076180592228352, 1.4371610138479269, 3.3819422429381234, -4.2088171575576876]], [[3.9814809443554608, 3.2179719106301174, -2.1243207327548417, 0.92417075695384021, 0.70880901005096497], [-3.5110389056310543, -2.3750606646334549, 4.717888459207833, 1.5672666774243691, -3.940751795638139], [4.8823817581803741, -1.9746257167822314, -2.923958306535972, -2.2114340577340261, -2.1526245886945006], [2.3682918227575929, -1.1213218566290086, -3.1744364000066394, 4.4458249727362915, 4.5867679666320509]], [[-1.2061589128025698, 2.2719617725154064, -3.4782290274795935, -4.5554540774108379, -4.9287078157348478], [0.158006353335165, 3.9233162934266748, 3.1350251786338728, 1.7636997037176316, 1.8746847749190039], [-3.2566012642667963, -4.8596445618773378, 1.5830406536682995, -4.5824763879261576, -2.2228221787293911], [3.408513703345351, -0.56509317554856864, 4.4412245683647367, -4.8968826088338933, -3.3831237224044042]], [[0.38572480634359163, -2.9563917215211744, -3.5941115503901209, 1.0272759795595965, -0.34532295648803757], [2.6184987244720475, -4.8424908944503775, 2.7673038902577591, 3.0270453528387762, 2.3468297375685392], [-2.9406277521955126, 4.5829235512236526, 4.1259520669411121, 4.0588492147526978, -1.9157167828518853], [2.9699013306354445, -3.218351593608161, 4.5352426332149598, 3.8286857033894002, -3.4551886316767888]], [[-1.8941754987208173, 1.8405454437490665, 2.0105323977799028, 3.2723082325580002, -2.1533787604782093], [-3.8532927493422009, 1.4388646365835402, -4.1861724476847204, 4.1937009411417829, 0.036392427020000895], [3.7321259306148331, -1.0183291460108923, 0.8298184014227461, -4.5907596159137087, -1.0495869090357255], [0.34942011629719882, -3.3281565862320575, -2.2449206229361818, 2.7325310123739559, 4.9496698289194949]]], [[[1.7896658004291481, 0.79091881794536523, -1.2945280245186153, -1.9363670922320519, -0.11074067396089493], [2.7514647120109226, -1.8319244587758488, -1.2603920385739489, 3.8683263411769495, 3.4334681661266462], [-0.20152529950231823, 3.5101724169459274, 0.93001124149066339, -0.40457988782183385, -2.7156973863912368], [-0.9060657729300301, -0.76324694539672411, -2.8589646955446324, -2.5253438410181128, 4.0830038735064313]], [[1.5198266657597879, -4.0570565609919731, -0.74803464192103419, -4.6124584712911156, -3.2594291959694788], [1.6938242095838465, 0.13015135520039944, 3.9716743258762151, 1.726521773141342, 0.30326322901156555], [-0.094579894470891546, -2.6883706488668988, -3.903017845017879, 3.3102806577258495, 4.7230428964938262], [3.9950698958054733, -0.40152375713275212, -4.1360098955577111, -3.8180804608511365, 4.1916773849915874]], [[0.74434334596304375, -1.0133039214518336, -4.2378366407199781, -3.8506970194992327, -4.5411371315434295], [0.76708947253611903, 4.0574160556330199, -4.621217199826968, -0.42067741183921115, -3.1550620798997842], [1.3586280577813312, 1.9182321004295204, 3.06333036073665, 4.6766089882963264, 0.98483809123641297], [-1.7505578962226021, 2.7717290796403526, -0.47111402452452733, -2.2021513959862773, 1.5993779804922497]], [[3.8350520149712004, 2.4802055896053048, -3.9233970413709973, -3.957351726934025, 0.63607380879735498], [-0.95529885360371214, 4.8871085537139827, 1.3578542063485273, -2.8581621110469326, -0.0056970665184294944], [4.2815055511604072, -2.4367802380439709, 0.46429072062304932, -2.941943136815075, 1.2009790601565928], [2.8566953315269927, 1.4036216081208517, 1.1081963826248966, -1.6177384391213581, -4.5161368018547421]], [[2.3012082429914136, 2.8637182551259786, -2.0694604060134472, -1.9986333101442275, 2.9658671109179657], [-2.5158646673008214, 0.94748663573980174, 1.2894057649050872, 2.713500429486734, 4.8632926998378849], [-0.14508649330598899, -3.2803738650919581, -0.94999577371858379, -2.6719318192358243, 0.86878438693113402], [-2.3407956593157198, -4.6729636786074833, 3.8145623358633767, 2.8222519607538121, -4.6987690352309039]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[1.1379532689489342, 1.2615516985120689, -0.53925448620862348, -0.74629168448716232, 0.82491949666969355], [-1.3699494568877748, 1.216343946355628, -0.64315296004986977, 0.32617263389318024, 0.10149480629793425], [-2.8438289738582716, -1.0697772490209969, -0.67694993663398428, -1.1623923352770225, -0.66565513001473042], [-0.46812170376036255, -0.50151589003581876, 0.068858204726442565, -0.53902920613786431, -2.2150309858072101]], [[1.0519030762932819, -1.3871183869383037, 1.1627582623574335, -0.99000354900658483, 0.28955417380911269], [-0.7268139728975046, 0.68005498782928597, 0.78520191305115983, -0.10024017672612015, -0.31810378376386039], [-1.1477355182606441, 0.44894539941915595, 1.5120415545367005, -0.43989954864334874, -1.3371797556578777], [0.3590817419505723, 0.47605141946528878, -0.23144783547922645, 0.66158080733283164, -1.2034713381538249]], [[-6.9831606574573302, 3.1178724523562416, -4.3215064912131522, -5.1542808123412431, 1.9347110675767583], [10.27453161285826, 5.7977706535097262, -3.9567042433716404, 8.6841002238519849, -8.2892900166889998], [-1.2853707037798416, 7.2834081841989988, 0.0068430275474895318, 1.9702614926873858, 7.7034639374987961], [6.2220539574897735, -0.56616120639834522, 9.1736225220367302, 2.5068309512772968, -1.3296335688689458]], [[1.0718366447966512, -1.1983153742033059, 1.2277961164700284, -0.81119108190990241, 0.11934561704646091], [0.0075308521187437121, -0.17661273210872358, 0.13801294726324961, -0.30871779151178791, 0.57272332363043121], [-0.37071573599322416, 1.0330946529660319, -0.60945690080142889, 1.2719855410796062, -0.58945480263012417], [-1.1197452861733319, 0.6404316834314796, -1.2255004458380461, 1.3267080148992363, 0.24625263788542281]], [[1.0852063641490792, -0.38441479197962791, 0.26325867102896439, -0.82367762062101979, 0.20936699041878126], [0.11540020950985777, -1.2024436887151027, -0.1705803790679282, 0.51417003113920257, 0.25199208751709179], [-0.18544760599330692, -0.037586850094498359, -0.48798198081531446, -0.12092764306542247, 0.37998710903665023], [0.65114796505826122, 0.21252448386033262, -0.65698895970969451, 0.54452298968524115, 1.1407181455747164]]], [[[1.5986009151408305, -1.2051880233031664, 0.43656662514354055, -1.0182766672402479, -1.7786639990272808], [1.5696249403912488, 0.016566504145422396, -0.93771415944685255, 0.64716376144549304, -0.66806223249372454], [-0.26224865435849759, 1.6728790829509623, -1.6550545600141704, 1.6871831747623007, 1.1998944329928594], [-0.31660448302072897, -0.29933452720715764, -0.91420362907375718, -2.2355256357626923, 0.66791890882245253]], [[16.351302981675499, 16.991626473314209, 31.815964004557788, -26.027215240868486, 28.384136834036394], [-59.591916285173326, -13.690034617970296, 58.56546927136371, 32.537661947035886, 27.700438434325324], [7.5394825463098272, -14.8753318782686, 48.14103628870209, -47.49179845957314, 64.960444316480249], [56.756852225802739, 39.285223145377174, 43.141476737394441, -36.566444747618611, 36.029252979010593]], [[-0.99478239004092295, -0.5698790741835712, -0.25786576300245567, -2.06319005210135, -2.78064263836086], [-1.665407017331979, -0.077215557503053728, -1.4886087313519558, -2.2371538469293579, -2.6711320446824036], [-0.24943468432962435, -1.3204031935502358, 1.9262341721900402, 2.7011140216089045, -2.5064907423494214], [0.58632051106550964, 2.1817494039372298, 2.1984561398239144, -2.899330333864623, -1.6908835269627038]], [[-0.23199153780834578, 1.0548591019235467, 0.85933033679688064, 0.21993568124148896, -0.098336865269597826], [1.0339015574703501, 0.19009390377320562, -0.45677197429744126, -1.0593301705312079, 0.69733151546141969], [0.23390228699805432, 0.68901389522116585, -0.01159904514540158, 0.75281320339149915, -0.34150905599423176], [-0.99750883366200793, 0.2560843844668067, -0.17695500532610323, 0.37591850739910582, -0.93348286040284412]], [[0.22327591656806337, -0.34649526165758948, -0.34532017118964814, -0.77011046488954837, 1.1799785398016305], [-0.81769621938668124, 1.3914526763739816, -0.37335694957775734, -0.87601890529093274, -0.45855822600990248], [0.4842062174452117, 0.55882678991626866, 0.38859541105628659, 1.0701368503648256, 1.0178944264124792], [-1.5140386388552387, -1.3360371763348806, -0.082059498334589864, -0.82580228403370393, 1.4098178756817881]]], [[[4.1831992057437075, 4.1038488812032128, -2.0385322297171693, -0.24726860779380128, 1.2735124827003701], [-4.0070361214764834, 2.6028086723937589, 0.64758980772035335, 0.1329158282116755, 3.6554109441739913], [3.9277826601389108, 4.6499438618598878, 3.9288904067027266, 3.2008685227204805, -4.0722819752594903], [3.8385422368791877, 3.3909902557292964, -1.4733862576072101, -3.4671878632616466, 4.3149050809021716]], [[-6.2868946982602782, -5.0812877988961649, 3.3543751530682835, -1.4592972598342699, -1.1192336895863049], [5.5440506157617193, 3.7503020883973655, -7.4497073716319653, -2.4747677315970056, 6.2225819782663265], [-7.7094479213577971, 3.1180015987483989, 4.6170302538697596, 3.4919300751220717, 3.3990679104453179], [-3.7396138553761702, 1.7706055947545192, 5.0125437373898984, -7.0201098136898583, -7.242663625566415]], [[6.9807052071220825, -13.149092716920164, 20.130424959736899, 26.364918968345414, 28.52514370079134], [-0.91446969530929612, -22.706389836372598, -18.144115469907828, -10.207500500024588, -10.849832166461521], [18.847743163598125, 28.125436654972887, -9.1619271862538429, 26.521311945854873, 12.864694853108954], [-19.726944024484904, 3.270505097786184, -25.703809955145559, 28.340953719543325, 19.579998236262544]], [[0.091706772623405819, -0.7028874962989945, -0.85450654278424099, 0.24423672817931419, -0.082101159509252439], [0.62255282312834259, -1.1513109970757054, 0.65793152130764154, 0.719685525348984, 0.55796302853629209], [-0.69913958398737863, 1.0895983902267932, 0.98095259064306251, 0.96499876577408761, -0.45546514127832971], [0.70609943037971279, -0.76516893122578622, 1.0782621654510307, 0.91027697330529667, -0.82147736677818739]], [[44.521535350829218, -43.260990913471552, -47.256439164266986, -76.913774226848176, 50.613963000313284], [90.569490246699758, -33.819708272002096, 98.393641317885283, -98.570594344578666, -0.85538363639886283], [-87.721531962495845, 23.935256845799788, -19.504417261579562, 107.9032363501108, 24.669953077715874], [-8.2129243412357642, 78.226458534117654, 52.76560326187073, -64.226612659070824, -116.3392201049962]]], [[[-3.8054818982123697, -1.6817817292618982, 2.7526384886233615, 4.1174223232159948, 0.23547504235314207], [-5.8506169993397066, 3.8953392108695977, 2.6800529385397036, -8.2254719647640488, -7.3008049609878256], [0.42851625096931578, -7.4639061600713204, -1.9775429266056677, 0.86028432745241623, 5.7745626264168157], [1.9266261313415167, 1.6229412406923327, 6.0792011524802332, 5.3698015972882107, -8.6819546572515751]], [[-0.33228768365149725, 0.88701557714885104, 0.16354674125338944, 1.0084460128865063, 0.71262611846155199], [-0.37032968021657425, -0.028455674135723696, -0.86834801079356572, -0.3774785202128354, -0.066304032015749415], [0.020678498911600771, 0.58777259001576476, 0.85333728390868457, -0.72374404053648145, -1.0326236663823876], [-0.87346310708358477, 0.087787247187157927, 0.90427755922250763, 0.83476697765203334, -0.9164484347147005]], [[-0.29203692543596804, 0.39756137185606755, 1.6626799846709452, 1.5107889718622338, 1.7816774114682534], [-0.30096117915033938, -1.5918934676154548, 1.8130961607014711, 0.16504928622010323, 1.2378623848399262], [-0.53304642670265789, -0.75260242187995541, -1.2018722072227712, -1.8348286685377055, -0.38639303995496332], [0.68681684142560651, -1.0874648681279711, 0.18483767202070095, 0.86399579779448643, -0.6275026579684565]], [[-2.2208908920165635, -1.4362949922921338, 2.2720517794622439, 2.2917150465089051, -0.36835237777575208], [0.55321662257513893, -2.8301401996293736, -0.78633771531478691, 1.6551708231208004, 0.003299189448435054], [-2.4794335632399269, 1.4111472322647101, -0.26887224179859798, 1.7036886831982914, -0.69549081623718711], [-1.6543214064078813, -0.81284176411272246, -0.64176007082287723, 0.93683750600621163, 2.6153092712134516]], [[-2.8655795526557526, -3.5660451423503066, 2.5769955598603049, 2.4887981190963218, -3.6932460044957458], [3.1328804639142787, -1.1798577282432827, -1.6056325220638692, -3.3789863957517561, -6.056015209262152], [0.1806689550371644, 4.0848855385005285, 1.1829822322983539, 3.3272230840522998, -1.0818537533222763], [2.9148757826283767, 5.8190079965616466, -4.7500837289594413, -3.5144092394983248, 5.8511421167631879]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_array_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[[-2.644231509019054, 3.0956890251012315, 3.780662785459203, -2.2674515207822665, -4.5367367116234991], [3.8951005955093869, 3.2419591054163348, 0.15881268402545867, -0.16506025878522035, 2.5247545135576477], [0.90778386541018463, 0.8791825512977578, -3.6874370485186803, -2.5108845844588989, -2.1527961452094555], [2.0106883773081456, 4.7296598294294405, -3.7927776958997281, 3.1961899851943691, -1.8387438488223715]], [[3.7931095170081583, -0.069476245109831858, -3.9097115555365858, -0.23507395689975397, -4.6743206463289582], [-2.1755562462923317, 1.5235880671324615, -2.955350915418312, 4.1809350408504962, -2.3534756954078526], [-3.2793851488797312, -4.0794318487412875, 3.1740859363311564, -1.9501150022283142, 0.54890019629401543], [2.0522532116570336, -0.98997765847608132, -4.922214962523956, -1.7483332169469259, -3.7496003564398519]], [[3.2465384264425055, -2.1840288834929655, 3.1569033845145302, 0.78614306209122553, 2.8649328651684538], [4.9928738333684777, 1.6326665895615546, -3.8756704393469112, -1.9498232846764751, 0.68874915577997697], [1.4425727370608232, 2.0005995553857403, -4.3614178511123658, -3.587118624482033, -3.611022075642778], [-2.3187369733115935, -2.1537410604212459, 4.1862932828982622, -3.1089946716265238, -3.4201892802833269]], [[4.0608957548508222, -1.8166320046961282, 4.0571284379001877, -4.9567485886939, 0.62452340864676792], [3.3106376116930427, -0.18380869455542381, 1.3377447343153044, -1.2136592038074068, -4.9530268690729473], [-0.14305948720686157, 0.1439489764626023, -1.1435379577096616, -0.21099270763884803, 2.259957477382077], [-4.6260806375170871, 0.52027192991167759, -0.5372395685889666, 3.6484194264798848, -2.9413997821541771]], [[-1.3768950223705723, 0.55692472451116615, -1.1671916735290342, 4.0760708210852705, 2.0479677886784931], [-2.1645769735498255, -1.4018876158021754, -0.83509075643652153, 2.3985434959329579, 3.0205719404693241], [-1.5449614020918068, -0.83410901983458352, 0.14168990655150004, -0.64535800088220796, -1.22005909937146], [2.6813616732074106, -2.5652872821442929, -3.4826852531988752, 3.0015173960130745, -0.94664347913952707]]], [[[-0.48454482386897091, 2.9936199955419838, 4.6274376969020885, 0.59026133944995784, -1.0859007114684771], [-2.2659727158687391, 0.93466875105297387, -2.4999283577718243, 0.42206794628683042, -1.4981415178008337], [-3.7748892452419747, -2.7275459258975534, -0.39269308951579962, -2.0568503960088105, -1.9119485782284351], [2.7438071929334136, -3.8098373825226828, 2.1202627292238354, -2.5393320767242522, 2.8068331248052738]], [[0.12733798726697287, -3.6070110850827031, 0.4646336549914114, 1.2641905911213147, 1.6038189739494957], [-1.4764167252432703, -0.25730883150260642, -4.8166843606645795, 4.6088268626671827, 1.4864543107254846], [-4.0631181527524616, 4.2849056811841297, 1.8944724912692381, 1.2527921868356406, 3.4605801818213759], [3.8892046917289367, 1.3974415654450159, 3.1388201595846574, 4.2874985642463823, -4.6876059954390072]], [[0.69403942091805071, 0.17758282586761531, -1.4876803009554829, 1.4864430337345684, -3.7635829201000881], [-3.6810602658855016, -2.1350452832829125, -2.6590734915633139, 2.6290700046037463, 3.1510541147942241], [-1.9410655092233164, 2.186437492035159, -4.8493706929367209, -1.1792547027299194, -3.0811949823373128], [-2.9571999504704483, 1.5231352607648798, 0.28086456282379046, 3.7780250030540845, 2.7622088128783915]], [[-2.2317555147229031, -3.2529116832708138, 4.5102394880961754, 2.2884118254277572, -1.2836665647493293], [-0.073907990120475553, -4.0459101964144972, 4.4255933686566049, -0.52200010870981028, -0.22100923980650045], [-0.42076754538991956, 1.1557008431783808, -3.5038800098047962, 4.9570035993382824, 1.5248450711938508], [-1.3864162562345808, 4.8699092542168678, 1.8122595271766908, 2.0709990523043906, -1.2207805306253694]], [[3.4301371795198037, -4.7096700125959181, 2.1176599463308197, -4.0075845497459195, 4.457880956965651], [0.6538116082447214, -4.4884060996678867, 2.2744131747474219, -1.0735832296402661, 0.98081038136903942], [3.843769959947446, 2.4465606760018686, 1.7629022974770212, -0.42357315670206752, -1.3086717310420424], [-1.7568075813212491, 4.7290984063713921, -1.9520043286634694, -4.8673170475349483, -2.5177891661969012]]], [[[2.6928772239822996, -0.74713982266855528, -2.643884989678277, -2.9107089475907353, -0.57301492832766954], [1.4282106133005463, 4.3825718895054688, -1.9559858235855074, -0.27422052474226444, -3.4987810454619761], [-0.61592674702885564, -2.3752498629365801, -2.0196546860985998, 2.2517683346697375, -1.3349720319666822], [-1.7015744846070433, -0.10157617164824106, -3.7806732717129465, 0.43909403086282772, 4.1947973104089282]], [[-4.7523466268436572, -2.0240182037784171, 4.5510148990197461, 2.6893841141371411, 4.9663536482166233], [-0.53508523408987596, -0.94083844514171489, 4.6365580626394891, 3.407716046685632, -2.6416528574307252], [-4.7860961900217056, -4.1609453600041988, 0.27218944475370854, 1.8679884375336862, -0.08204806854213853], [-3.1109293163667564, 1.4169507688016347, 4.1806324984542815, 0.78256981592966124, -2.6873808557138599]], [[2.8116163908009923, -0.78401817155448228, 4.3869033277101561, -0.36533964224909887, -0.44057869780885461], [-4.3821486941821677, 1.937202058301029, 0.64757239684147816, 1.3820530485461147, -2.2554981493764026], [0.18706658281623412, 0.36155615480645498, -3.495526800078157, 3.3943413487954572, -1.2471455740430546], [-1.5464226410446438, 1.3566091617742924, -3.151959892878101, -0.50366850586515177, 3.2065634314403972]], [[1.323370546369298, 2.9077310173212325, -4.4827228475575875, 1.1560170131465188, -3.7009435308215699], [-2.0674488090067209, -2.8172544287685319, 4.5512783946070421, 2.9704416589794791, -2.8212663761149361], [-3.975937010428956, -4.2127208898169517, -0.57709453504440056, 1.1634939356298268, 3.9040784443707786], [1.0361666622984087, -1.2041413622982522, -0.41775616584391795, -2.2151437478769989, -2.2061341406968635]], [[-2.9615853864176289, -2.6175415555642676, -1.9063562747969485, 0.013431530435260797, -0.3386977107559952], [1.0309627412955527, 0.42066637490950498, -2.0091238261507738, -0.97963880674479853, -0.51650756730228409], [4.4002329899464563, 0.79130640359650339, 4.422375393012862, -2.4823312909370854, -2.24059325580102], [-2.628992799611872, 4.2085391890049841, 3.3475727531586017, -2.5154394977614372, -2.2037410165774327]]], [[[-2.3096123459285014, 0.33695664013240556, -4.6017754772204436, 0.95655621146746661, 2.7440293052102867], [-3.5979119528081305, -0.6722248318297277, 1.9201281332475073, 0.26287488271874437, 1.7914069106480301], [-3.6923838484676521, -4.7919266707695396, 1.4686657861934949, 4.0491501450867524, 4.7384017706834243], [1.4835450184136807, 4.38253706356954, -2.9802975660775077, 3.0210769200931331, 1.5907406277308116]], [[1.8803479939129675, 3.1221264579561723, 4.1907435627360208, 0.11628174774146238, -0.26155278122514947], [3.0493474974856447, 0.81705242544781242, -0.728530714536757, -2.5426305212056608, -1.1832862418489496], [2.8370657991926604, -4.3085932727447753, 3.0613609148554275, -3.433618758275033, -0.60274549568768432], [3.4892316058414572, -3.9339974772222752, -1.6851851278414687, -3.0786849401516116, -3.5831207361462134]], [[0.70218642491658123, -2.1959080172877132, -2.8424240400909282, -0.56464499910041255, 0.35983005924648737], [2.6521820106203808, -3.2526867196360776, -3.5249466894281642, -1.5585702325446702, -0.53683890778023091], [2.6758989917024598, 2.8834791559160386, -1.8788478260465302, -2.8201794857403009, -3.9711611835865899], [3.5412668819952682, -3.8757160907193153, -3.5687746567505552, 4.6413257663822591, -0.21140368457001824]], [[0.41598172048605697, -1.1549196863637592, 1.8694864117077206, -4.502763541169128, 2.9393599486333848], [4.3729065698719332, 3.0295147441924239, -0.065385579221032053, -2.6223531556231903, 2.4365198060593665], [-1.9568331807331441, 2.1492529338573219, -3.1889799797233254, -0.65222225142866375, 0.96118313406917366], [2.5071243181494607, -1.3728069843420974, -4.4852931581058861, 4.9093829555736761, -3.9351225509214407]], [[-0.30523504463535645, -0.08678221239463646, -0.13922789704460037, 4.2400508920015394, -1.9037852942927893], [3.5382716187425114, 1.8334650056325374, 4.9405038026938719, 0.81794458988117302, 1.0183556254358166], [0.9027975055084525, 1.5761819917362558, -1.7683242379819974, -3.9284843357153152, -1.8344904327690958], [-3.5642426149950159, -2.8621064429048237, 4.4540663351953675, 2.2666239332704912, 0.74685591190788614]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[-2.7722511358874491, -3.674644138826364, 1.6131685604809363, 2.060779480130023, -0.4329245355052187], [-2.9587312884495063, 0.25912606740310018, -0.53379194242495576, 4.6444018160737635, 1.9995312381935593], [-3.5093060611627336, -0.5742546215106259, 1.31981124346445, 1.9835286748619838, 2.2105056337153162], [-3.7924078247984694, -3.8050503633270569, -3.7341343263352433, 0.49621333437410264, -4.7643434889128846]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[1.0484146817068554, -0.89551990313264562, -0.73327119957638032, 1.2226286253436756, 0.6110672300609179], [-0.71172773793918009, -0.85511601033333651, -17.456106562892924, 16.795388279953908, -1.0980279948013847], [-3.0538669407114871, -3.1532144624518996, 0.75180975279323603, 1.1040934151439203, 1.2877443793535426], [-1.3787572292026986, -0.58614175984446604, 0.73092898085866109, -0.86736118588984967, 1.5076875105050358]], [[-0.9687682684481953, 52.890655403401333, 0.9398760206805179, 15.631864062225304, 0.78613437478070658], [1.6890595888241624, -2.4118357304690603, 1.2433867395088141, -0.87890486288896241, 1.5613690619352478], [1.1205283832189266, 0.90077350843847992, -1.1577015281047462, 1.8843217628844982, -6.694557888002759], [-1.79054130258097, 3.7118455223352362, 0.74654279969563186, 2.1017985034016058, 0.98000954488796321]], [[0.49688879310404943, -0.73862052497261865, 0.51099712724626511, 2.0520038123719284, 0.56307377394202374], [0.32309419671287803, 0.98805755614448232, -0.41622954937127499, -0.82734090476748079, 2.3421713797298183], [1.1182580392914498, 0.8063425567291489, -0.36987250833338586, -0.44971151761502509, -0.44673461604185433], [-0.69571002621182487, -0.74900766397861207, 0.38534532854423054, -0.51887144587384559, -0.4716606094816197]], [[0.50746919018257985, -1.1343956700106326, 0.5079404094972656, -0.41575227051671731, 3.299763390127151], [0.62247207995566489, -11.211545161747702, 1.5404878279597851, -1.6979885899312506, -0.41606466805130354], [-14.405052893487387, 14.316041216627964, -1.8021085056568313, -9.767064953057135, 0.91186648454873553], [-0.44546985701401132, 3.9609661057052703, -3.835866902995547, 0.5648417134206335, -0.70061182863785387]], [[0.31442087339371833, -0.77734838561030473, 0.37091126104100808, -0.10621123982088976, -0.21139225816855989], [0.20000422290145617, 0.30881543614856355, 0.51841615078172276, -0.18049476119123897, -0.14332535163454926], [0.28021705585593193, 0.51902632055348619, -3.055436664769509, 0.67082849350811247, 0.3548389874951543], [-0.16145697159434666, 0.1687625937720871, 0.12430768330487917, -0.14423522451686396, 0.4573258518600215]]], [[[6.1062076049533802, -0.98834564602573716, -0.63938868165211082, -5.0125784812649865, 2.7246793902993001], [1.305722380384079, -3.1655399681612071, 1.1835264315680674, -7.0100828894473812, 1.9749344459745803], [0.7837928734409394, 1.084759475672578, 7.5344623255216838, 1.4384766603301529, 1.5474952214410465], [-1.0783306115931262, 0.77660303876024839, -1.3954550290720853, 1.1651612310061787, -1.0541172762647835]], [[2.0349470960289682, -0.071839553938370784, 0.55769973745851453, 0.20497389335358043, 0.16156815177525147], [-0.17551011375897532, -1.0070624699893962, -0.053797601835663442, 0.056223866750581483, 0.17432494596933162], [-0.063775174056299955, 0.060474158985803141, 0.13678006336713483, 0.20683882780081234, 0.07487937102694632], [0.066627006789891099, 0.18542891081143803, 0.082555245037482142, 0.060437586979961357, -0.055278977724498857]], [[-0.76910896749765989, -3.0058759331991243, 0.35880823459322586, -0.3591068949906856, 0.14183079096627429], [0.14501037849662829, 0.2500143423675682, 0.20074358385300983, -0.20303451086895227, -0.16940107119036715], [0.27499944741099608, -0.24413775576455954, 0.11007447692180812, 0.45265195143106268, 0.17324185761851246], [0.18050586749808281, -0.35045603379761492, -1.9005314770160147, -0.14128862090469183, -0.19324822219675411]], [[-2.0810531375119807, -1.4277675720368164, 1.0297461649944935, 2.0295305960524028, -3.6180749297468138], [-62.840320897686986, -1.1479250874598383, 1.0494416068513719, -8.8973196338081113, -21.014514235423199], [-11.037927870054377, 4.0186886108872608, -1.3255025295037162, 0.93693734995345801, 3.0458188204244974], [-3.3499331785734152, 0.95369370836882905, 2.5627686026345531, 2.2425900248027442, -3.804452724761735]], [[0.58293040002367491, -0.42455867031996997, 0.94421733841548217, -0.49893675688522388, 0.44853850012957314], [3.0582681815051771, -0.44548804047421459, 0.87914160030119026, -1.862483674287235, 2.0386521963629343], [0.52020054764695078, 0.81728250511291722, 1.1342269172007942, -4.7206278456403403, -1.5279089406182966], [-1.1381617767665619, 0.42281447040722231, -1.024347748020944, -0.41080768289097203, -0.79416150686430742]]], [[[-1.3031808616855829, 4.6969870360123007, 1.3273293183565318, 1.2056533732331678, 6.1242838321936217], [-2.4571348430557074, -0.80074124273149694, 1.7941367564361193, 12.797386572216194, 1.0030081950153493], [5.6976029667344932, 1.4774471165842282, 1.7375772627457013, -1.5584667423957876, 2.6287487506332399], [2.0623875668734906, 34.548516686723367, 0.92822251724829374, -7.9921515996627956, -0.83658537027636026]], [[0.12083601357420887, 0.2837200873186877, -0.12618166150902208, -0.21352644216643227, -0.1156290232607249], [1.0732021459859062, 0.61036474909794758, -0.12385364612121681, -0.16851598362168405, 0.21738458930941693], [0.11998392817675935, 0.13801061341263238, -2.1097608029225454, -0.30741872378440249, 6.9990023130855086], [0.18459262911871488, -0.40527492849754637, -0.13736070360715677, -0.73380624938675232, 0.21368561150893675]], [[0.46941369661330407, -1.6833936907962788, 0.30085259347471316, -3.6125596317427977, -2.9956310870868545], [-0.30117901868931535, 0.68129766732849484, 2.0380906442303655, 0.95496424312573136, -0.58515288244831853], [7.0553020405626041, 3.6503630927565309, -0.37757148462856588, 0.38882690567724093, -1.0582655873811302], [-0.85346088994977942, 0.97287507754870617, -0.41872716922781367, -2.6203966062904991, 0.41159679877955418]], [[1.4988460188296069, 0.68215686493908345, -0.44248300470833474, 1.7158300027636206, -0.53595215877872682], [-0.95940884544316463, -0.70406444466182516, 0.43581791815071813, 0.66775547294991888, -0.7030632384289176], [-0.49888332477580793, -0.47084265175426104, -3.4370948855188428, 1.704803621334092, 0.5080657838015521], [1.9142950135667862, -1.6472556603123198, -4.7480536184427491, -0.89544016128208581, -0.89909703960042786]], [[-0.74639267328002712, -0.84449686348486408, -1.1595448673154043, 164.57585711247322, -6.5264853098101092], [2.1441178668954599, 5.2547713950059514, -1.1002336465992564, -2.2564496409248163, -4.2797158718521278], [0.50236104287337169, 2.7934888731703977, 0.49984577003747976, -0.8904958180988185, -0.98657158232186704], [-0.84081844348971257, 0.52524297254742713, 0.6603308715634616, -0.87877511491829141, -1.0030696062227809]]], [[[1.6420105441001442, -11.254883783587882, 0.82411839594772085, -3.964647115699016, -1.3820580624257732], [1.0540579854486258, 5.641576516112802, -1.9750805996391401, -14.426664828438744, -2.1169996622523857], [1.0270892681897976, 0.79141608070338931, -2.5822129584891305, -0.93659352923728578, -0.80035590233444609], [-2.5563146232350946, -0.86534529424141016, 1.2724930114242965, -1.2553165394681702, -2.3840516541080188]], [[-2.0235883866415714, -1.218736785510586, -0.90796544965467763, -32.72267950243662, 14.54792545314821], [-1.2478244498091906, -4.6570455515649121, 5.2229100124440651, 1.4965014899305085, 3.2156634876287051], [-1.3411921445071364, 0.88313055386243544, -1.2429277269670604, 1.1081749696750323, 6.3128640372265235], [-1.09051240879421, 0.96722236995783095, 2.2579420506759962, 1.2359336656058333, 1.0619375241649092]], [[-5.3178674406570208, 1.7004966951882976, 1.3137147285792692, 6.6132425369647008, -10.377494126407383], [-1.4079479882535586, 1.1480153633587658, 1.0593449079767543, 2.3958717088023298, 6.9557818411028975], [-1.3954690883004943, -1.295009994670471, 1.9874596944834033, 1.3240768345476523, 0.9403129597884331], [-1.0544628379523058, 0.96346952122651608, 1.046335138945212, -0.80454045121807127, 17.663525278332955]], [[1.1928729315179967, -0.42965181062626101, 0.2654276229378027, -0.11020195260914421, 0.16881679788989784], [0.11347448806541388, 0.16379300854215778, -7.5890332438087462, -0.18922445030336876, 0.20365659788197604], [-0.25357978352972949, 0.23087712318881673, -0.1556025241704885, -0.76080405611303425, 0.51625264404440907], [0.19792131199156643, -0.36145892323814727, -0.11063119329833297, 0.10107448102225275, -0.12609857201471661]], [[15.608769610987892, 54.900000327801543, 34.219747550928467, -1.1236524301867166, 2.5025634472519256], [-1.3465171706083194, -2.5985461812886945, -0.96434365384256282, -5.8247753550213286, -4.6784672956207523], [-5.2773113127174875, -3.0227115357819079, 2.6942703077745116, 1.2127688649789086, 2.5970936690693356], [1.3367057194336214, 1.6646283371898052, -1.0696615475314666, -2.1019558732173347, -6.379200342328545]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[-1.9825122640056092, -3.3673459830659693], [-1.1240474125804889, 2.5431130609711268]], [[-0.18246468831013107, -1.7495112712431196], [-0.8764942318729787, -4.3098393658692942]], [[-3.2323118547389207, -0.76270201773482071], [-1.7590474637054099, -1.4975085191364537]], [[-1.2613365461190993, -0.057443064035384239], [0.87828918300855907, -2.5947793717682579]], [[2.9833923952218653, 0.40907404450509777], [0.54549885372759022, -0.092392910481114754]], [[-2.4513509900114006, 2.0638669822088413], [-4.0372095229255436, 4.1063763542244587]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-0.56905630772730476, 3.180929932591086], [3.410880212046866, 0.21200486400044838]], [[1.7533885196085564, -1.4271842549219462], [-3.0338115260359655, -2.5501826813338946]], [[2.0505912180629862, 0.73142533549805133], [0.39937829230406408, -0.59940169993881653]], [[-4.302853343535185, 2.8282762097251979], [4.3745163581686057, -4.1671281602496641]], [[4.1399799398820285, 3.2519742313171811], [1.4135720665020326, -0.57491723040757758]], [[-2.4472247956423141, 4.4373041292353861], [-3.5207389020058577, -3.4361750590095519]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.28703797603629588, -0.94464006626810848], [-3.0344629362354638, 0.083364309378951118]], [[-9.6094676501371143, 0.81576168063659116], [3.4613023288847207, 0.59171177040365708]], [[-0.63440389115196183, -0.95899226498749901], [-0.22704236272441397, 0.40026596996220409]], [[3.4113443844739724, -49.236165535720964], [4.980724393284456, 1.6059662742770695]], [[1.3876753009468443, 7.9495980617676558], [2.5913382894255146, 6.2225253800733062]], [[0.9983167672088169, 2.1499952116518615], [0.87207237623242595, -0.83679009486662537]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=numpy.array([[[3.6343942763954775, -1.7145061274144435], [3.1669969208536255, -2.4835194872348634]], [[0.16724152684681837, -4.3443983728078086], [0.73845502903874038, -0.76117551896795899]], [[-0.69015096659747321, 1.3957608964069497], [-3.5333182789913198, -4.54023257108922]], [[4.5848455001868729, 0.22020273667576795], [-4.4054320725715854, 4.2313513580918478]], [[3.0492580129880782, 1.3530949125822653], [-4.5937656070873532, 1.234318758525732]], [[-1.8192571494120804, -2.1518970963457962], [-3.4224548486221873, 4.6260216837411043]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[3.899832118407252, -1.265382208878302], [2.2722808514140311, -2.1995747153678891]], [[4.8539636989175392, -4.0177092224919679], [-2.64249087343266, 1.4948573402221657]]], [[[-4.103294545746162, -4.8829139834584137], [0.018054781204480896, -3.1647935569541783]], [[0.32277608115780421, -4.3584921890396666], [0.041048891958239686, -2.1447949489804885]]], [[[-1.6339723164912723, -3.8472277811214006], [1.4616569813180602, 1.4044323953942612]], [[0.56388406454150442, -2.6253353108177957], [-2.8846299613056416, -4.6057608810664252]]], [[[-0.26500301309316399, 3.579882988268503], [-2.9238785208326923, -3.3166871196410774]], [[-3.6705056900914377, -1.4000525677958873], [2.2629203118709862, -4.3171654101260559]]], [[[4.9695755341626544, 2.9412372344176072], [-4.409773413550008, -3.7943062746027714]], [[1.0351843324487442, 4.7518223920939686], [-0.63299510895752586, 0.87727767690412151]]], [[[3.641530234720836, 3.0070054903992531], [2.6960337452721026, 1.3758620171622109]], [[-1.7603001290090736, 3.3930780581867879], [-2.4165115999546427, 0.0096068867438940941]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[1.0730349603882356, -0.3481686665358949], [-1.3253267603311154, 1.2829203000195466]], [[1.5326707983060535, -1.2686179756085911], [1.064010524988791, -0.60191085590656324]]], [[[-24.535141618890474, -29.196779505193252], [-0.0041558760627221194, 0.72847683047739353]], [[0.43709646283805176, -5.9021768660891789], [-0.053928287149717966, 2.8177403181443776]]], [[[2.3675578178886734, 5.5744727854091023], [1.0472115855091972, 1.0062127395957532]], [[-0.15959050954857093, 0.74302259335869059], [0.63534850167677803, 1.0144328091019983]]], [[[-0.057799769497655432, 0.78080776944884867], [-13.278120721714211, -15.061970480978404]], [[0.83317722975327846, 0.31780141986813876], [0.53479848879566061, -1.0202805309159924]]], [[[1.629765507869499, 0.96457473322678344], [-3.2590274137786346, -2.8041686058531283]], [[-0.22534548363800738, -1.034406802289338], [-0.51282953012362387, 0.71073834926720247]]], [[[-2.0016577842762087, -1.6528754559909307], [-1.2528636940169313, -0.63937165931335904]], [[0.5143384520376465, -0.99141645639321552], [-0.52237359985748888, 0.0020767059475010762]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_array_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[[-3.1605833182940812, -4.6912878923647412], [-3.2541764724912046, 1.2291695489690611]], [[2.6620419412399414, 2.3081045232020037], [2.4751636738231078, -2.5211160936837205]]], [[[-2.6980359901786146, -3.0437832963800684], [-0.068884489056912024, -0.40558505064352168]], [[-0.30771692589211241, 3.9059114177227592], [-3.5944576137285411, 3.8366501104488133]]], [[[-3.0507161650858974, 4.3797806307689306], [4.9250156791895829, -4.1246141897905355]], [[-1.3813675591889085, 4.7099089135243464], [-4.3140696627807333, 3.896002337440315]]], [[[-1.016697690505064, 2.9449107527384228], [1.4185908529943649, -1.4956204766445778]], [[-4.1078583969859075, -0.22881276895826552], [1.5735499243722986, -2.8917152290138084]]], [[[-2.4135040741992917, -0.64600628589456033], [1.3290237372879776, 1.4355400828710208]], [[-1.1585729870339256, 1.9247552077284267], [-0.35694198305167291, -3.8380579722817076]]], [[[-0.80289608485333552, 4.4423107462223275], [0.51534746502825968, 0.038458318328423502]], [[-2.4636717287695644, -2.963180983751128], [-1.2943545163663361, -4.8037210000330557]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[4.1125839006998319, -4.8911290143146999], [1.7990392579258918, 1.9064023189369816]], [[3.3168301785650929, 0.65106327522828433], [3.6191485719107437, -3.4491904575952734]], [[1.1047924235565576, 2.0251917297770028], [3.4295643229729258, 0.70512531892254682]], [[3.5032968195935119, -4.9686591573776608], [4.3721826110584345, -2.8537574907097039]], [[3.3768632277345336, -1.0610801958700797], [-2.7475642540163561, 1.4828706302220631]], [[2.4767157550180166, -3.6251056543527049], [-2.1338574551457468, -3.6366161461906845]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-1.3012104053373259, -0.87664283136262577], [1.503031275550444, -3.9792142739112166]], [[0.6758117631640036, 0.7794444488285599], [0.77021262840060023, -0.75617395157374456]]], [[[-1.2293498643602279, -1.0897064132357108], [-9.4515221661930173, -1.6052447549417181]], [[-11.761291847753741, 0.92658234784566484], [0.95958579242152153, -0.89901094921365798]]], [[[-0.36214199019902937, 0.25224834682247455], [0.41120513348502691, -0.49100149410091859]], [[-2.4827311892184931, 0.72815937334266623], [-0.16344782862593879, 0.18098688292518228]]], [[[-3.4457605759418835, 1.1896105226061113], [-3.5025315064521974, 3.3221390285621388]], [[-1.0643459896929437, -19.10812334016159], [-1.8135792493829455, 0.98687362506402487]]], [[[-1.3991537299786194, -5.2272915937008353], [-0.79839070296466874, -0.73915051800432019]], [[2.3715072634745464, -1.4274876321851853], [-4.1543743819213175, -0.38635962273923219]]], [[[-3.0847276524837413, 0.55752870443027369], [-7.0342941420191485, -94.260638840089044]], [[0.86612896930528271, 0.72012390294313711], [2.8095982207408059, 0.75704149890588146]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[-0.14638077289340412, 2.0501256121643383, 2.6431891692052911, 2.8092306608080486], [-0.47484705143690054, -3.0962380222609189, -4.1418148300817563, -4.4802191859424134], [1.4547336939680653, -1.9733837909951712, -2.5135592801501505, 3.8746248278274127]], [[-1.4450906707449516, -1.2572357883702212, 0.63732278449078716, -4.2049225129656698], [-4.7151846032000622, 0.73066162741343987, -2.1027997664862164, -4.965147882116411], [4.8980075070382547, -4.3012430017650249, -3.292885574494572, 3.594855423405912]]], [[[2.3904320934113699, 1.2169817700079921, -2.2971961103205372, 1.8040342026699374], [3.383529055615309, -3.9241798123549465, -1.3276465381881386, -3.5286509166306725], [-4.3104533957407929, 4.7218045791476762, -1.5382630463081446, 3.6990755275739762]], [[-1.8345009858111094, 2.516288057890681, 4.9681746853677105, -1.6390666831330156], [1.5920739969949143, 1.8155587048903516, -3.0933913943798466, 0.10726162630244751], [4.3668621234979614, -1.6158790677823918, -2.5185548667509297, 0.7775767028176066]]], [[[1.5786297398321683, -0.65419972370490687, -0.60942583485398583, -3.3307655917757053], [-1.1197409173565798, -4.9873205114787931, 3.9967686754634606, -2.0975130520110721], [-4.5948097109444355, 3.6110620325491922, 1.406782384472038, -0.70514816089656573]], [[-1.1782861632560779, -0.22111982967081634, -4.0600526483645272, 2.3835807092084549], [-3.0356662620277985, 4.3545554090525691, -4.7620614532956393, 4.1549157598914981], [2.1670149557814273, -2.821850194281085, -0.73310137881508375, -4.6912360669513049]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[4.4731853487963456, 3.5398988412061883, -1.065825268168167, 2.20305155997361], [-2.656145921140447, -4.0092249457175431, 2.3070448596353907, -3.2531267619196003], [3.1494479498931014, -1.6100288722840506, 3.2900121154981665, 0.23749367833570734]], [[-0.36906597153656318, 3.4889154849999073, 3.1045306928258292, -2.8548117932859798], [-3.3028115046004025, -0.046826809078337917, -1.413098420348029, 0.38948551826567623], [1.9507083643491354, 2.0905305645980219, -0.48350371520122248, 1.7968393111381484]]], [[[3.0882122678784505, 0.68006757598217149, 2.5098390603063878, -0.72554486464929457], [1.6893777347960706, 1.5290832127031804, 2.6040781530114074, 1.5664531876866237], [-3.1458746253482084, 4.5027407824899406, -0.55844884038356923, 4.6790444863887757]], [[2.7684740336996718, -2.0194087475859259, -0.91531249985508722, 4.2121148328362459], [3.0743662902638835, 4.8395886569707773, 0.22214790646959504, 3.7711110672894463], [1.8450345020182493, -0.6496537487691576, 3.083329634514854, 4.2849590543186995]]], [[[-0.52837838980804186, 0.28883706759433636, -3.1057121608218408, 3.9292143151367771], [-0.11059525802685855, 4.2900777670650285, -4.7809789257763571, -1.4011692467532946], [0.4020453913961699, -0.76223645584937927, -0.90849331313811543, 4.1456023326213138]], [[0.20769398628733615, -0.40384121432266351, -4.980704613781981, -1.4317679458492472], [3.2221538355918149, 4.4524994161743869, 2.9115009801286167, 3.2341061673795952], [2.5350335640323083, -3.3570667866441237, 0.15665390927267797, 4.5126199599996752]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-30.558558070012118, 1.7266741219183548, -0.40323457760256376, 0.78421882215251171], [5.5936873001588081, 1.2948697473813553, -0.55701303758909271, 0.72610884131002751], [2.1649652874282288, 0.81587214794752017, -1.3089057184685271, 0.061294625644794432]], [[0.25539295146532764, -2.7750685410591558, 4.8712061899784578, 0.67892137952205045], [0.70046282013197914, -0.064088228150294371, 0.6720080736499795, -0.078443890799010138], [0.3982656950905118, -0.48602940213797913, 0.14683283225698965, 0.4998363214940505]]], [[[1.2919054577581759, 0.55881492454706649, -1.0925663024721819, -0.40217910701221826], [0.49929458474499494, -0.38965676544407873, -1.9614242783062097, -0.44392410150402445], [0.72982453039781903, 0.95360591634284064, 0.36303858545120432, 1.2649226682477359]], [[-1.5091155878968439, -0.80253480568465907, -0.18423516841122145, -2.5698251792812625], [1.9310448484598322, 2.6656194833771889, -0.071813708046514615, 35.158063487271555], [0.42250807326619516, 0.40204354504123424, -1.2242455684487485, 5.5106577123412182]]], [[[-0.33470697813169054, -0.4415120598929258, 5.0961281639232565, -1.1796730231748387], [0.098768613625324589, -0.86019692481985188, -1.1962110679878066, 0.66801455438375901], [-0.087499900254526947, -0.21108373353289814, -0.6457952012805952, -5.8790514710417137]], [[-0.17626786494156416, 1.8263455381811149, 1.2267586273267446, -0.60067944849440913], [-1.0614321725338294, 1.0224923092994074, -0.61139508775420759, 0.77838068309333208], [1.1698274427082453, 1.1896686767595734, -0.21368655659313948, -0.9619255768836833]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(-0.260966845369,self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array(-0.321389461343)
      sub=res.substitute({arg0:s0})
      ref=Data(1.23153368731,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(2.88264225963,self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([3.5731081962401028, 2.1571295215774322])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([1.239525364032606, 0.7483167619475789]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-3.1548858884906981, -2.3718021535963985]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array(-2.55526652711)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.80993944549154762, 1.0773523091876263]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-1.11863358899,self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[-3.5670328800700277, -1.1684394252799546, -3.8606569391570558, 0.68520585670000678, 1.3584922231312433], [2.2304556151744572, 2.4538876187026037, -4.8648380156372664, 3.6144860804263548, -4.756372423589255], [0.1478796653528569, -1.0287460044952432, 2.9685219632016144, 2.8173754140185885, 1.3760926706793608], [4.9161137434654343, -2.9081204277035555, 3.9754089977057276, -2.9399747134279854, -4.8506938480241635]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[3.188741081252001, 1.0445238161840333, 3.4512256534817896, -0.61253824616578489, -1.2144210906110045], [-1.9939108186399355, -2.1936473594768273, 4.3489110853933228, -3.2311617637914147, 4.2519485114853905], [-0.13219669676361256, 0.91964519447979887, -2.6537035830390203, -2.5185864627673595, -1.2301549714111168], [-4.3947489078340194, 2.5997077652013165, -3.5538080000861192, 2.6281838328235008, 4.3362669383264389]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[1.117020997490286, -2.2278718538731757, -3.927169254050332, 4.9793632315189704, -4.3513328103546485], [-1.7376458981762188, 1.8908624017305247, 3.2981892801783665, -3.7327791792909748, -4.0899933732572515], [-1.1535025680405631, 2.6547573650680647, 1.5366495362282517, -4.8627953973842288, 0.46016902304526042], [-3.5761659999978859, 0.17540646590419939, -0.12894650156538923, 4.0528786870439824, -2.7509890562708792]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array(-3.54124885807)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-3.1702616746058778, 1.5895208927376092, 0.90173064336892894, -0.71118508399982971, 0.81383084503362968], [2.0379577115166394, -1.8728220809893505, -1.0736948541297699, 0.94868961917702488, 0.86583241949197787], [3.069996509924505, -1.3339256177118428, -2.3045260318534142, 0.7282331598772126, -7.6955394229680234], [0.99023615181050761, -20.188815958515367, 27.462930867323319, -0.87376137593112524, 1.2872638842387272]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(-4.27939966281,self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.1061830331793461, -2.8497534577773678], [0.77186069625006226, -1.8645806953503694]], [[-0.80009259335848437, 3.4149901671640812], [2.9206814522081732, -0.81458183024049546]], [[3.4866897571224325, 1.0380451157439605], [1.8899311213543886, -3.2200818187685822]], [[1.3379765365491361, -3.1137635692550534], [2.1904743801198876, 0.38436370350390803]], [[-4.7800283784050466, 4.2498938691628787], [-1.0002692424157189, 2.1873013374399006]], [[4.510766882232609, 3.9168216452266851], [-0.39328364638332314, 3.383750648993205]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.49216787379875376, 0.66592365339100279], [-0.18036658341548362, 0.43571081045673304]], [[0.18696374641331798, -0.79800683185623078], [-0.68249793951044035, 0.19034955704650935]], [[-0.81476142259508766, -0.24256792950785427], [-0.44163463809646902, 0.7524611096163959]], [[-0.31265519511472362, 0.7276169123245344], [-0.51186487655199298, -0.089817201895019086]], [[1.1169857351599839, -0.99310515586996639], [0.23374055270158189, -0.51112340743716278]], [[-1.0540653450612116, -0.91527362570677562], [0.09190159306720562, -0.79070685507622196]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[4.5299635732121075, -4.7324401268062513], [-4.213064275598394, 0.83426035534067466]], [[0.28249619317474828, 1.6788041297391034], [3.3361821335865933, 1.5834602170652614]], [[0.84050452215366178, 3.6873771830985635], [-1.9842125713918755, 2.7816052815798651]], [[-4.0923615734800585, 2.3026574423900623], [0.31332718199570131, -3.0383428962587788]], [[-1.9962344847893885, 0.2450239334538642], [-0.11053416776938896, -0.24750725809430918]], [[-1.7044376621715562, 0.64373764747369933], [-1.044142023909834, -1.5755296156269303]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array(1.57829984719)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.34841336396773886, -0.33350656424665326], [-0.37462040546959807, 1.8918552668724642]], [[5.586977401206374, 0.94013340760570241], [0.47308563621416649, 0.99674108018913377]], [[1.8778005419292709, 0.42802777389534036], [-0.79542881138335453, 0.56740611532694674]], [[-0.38566969678878982, 0.68542537771312595], [5.0372260623586831, -0.51946073931864845]], [[-0.79063850425400628, 6.4414109468670686], [-14.278841366834257, -6.3767820763977756]], [[-0.92599446857053125, 2.451774963586594], [-1.5115758307324307, -1.0017582859374663]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(2.26647963246,self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[4.9977814306147685, -3.1090760068807013, -3.1311627768307781, 0.97585901484228188], [-1.6735809445103058, 4.6718683926067666, 1.7883100000278684, 4.6515523315316774], [-1.4261571694195929, 3.0473710784547379, 4.160603087559215, -2.2819222620371726]], [[-0.15574431950537448, 1.8986680086339689, 1.4922612662884678, 2.5794427434071023], [-2.9357923757684459, -4.1999017916498351, 2.5798628309388514, -4.5445047175163165], [-1.9544922729401302, 0.077597372656242669, -2.7325572355422167, -1.9713422731638253]]], [[[2.2397471498045887, -1.4668620086097937, -0.21807905875983824, 4.8227337166258302], [0.33831020587129679, -2.2892650731643305, 1.9361589780843911, -4.9636656792443254], [0.11774500909425534, 2.2016070324092043, -3.5067144162895847, -4.5377092432779529]], [[-4.2476130301061534, -2.6790878870586585, 2.4665403820980227, 3.0705861170463162], [-3.7244707449935586, -4.5650613540619762, 0.061767378880595025, 3.4955644399478878], [-2.3742701500944907, 2.0187464344848909, 2.6080876085134852, 3.8393205826962493]]], [[[1.6701016838764868, 4.1898491932487598, -3.7616836829489966, -0.40257796690711078], [-1.1087384969139435, -0.50496908288299558, 3.1679142495345189, -3.6822513298061144], [-0.87964878281210712, -2.2262453631754333, -0.51185136766735795, 0.10910227625597013]], [[-0.93731726386227976, -4.6907751210124378, -2.0067793435273384, -0.38437354775392141], [-1.3996194854517832, 3.8067171643444784, 2.6897082560746881, -3.4057079513713062], [0.74033165085761077, 3.7694131841676448, 0.056837709998002417, -0.42137024792553746]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[2.2050855251621755, -1.3717643707676774, -1.381509338087809, 0.43056156378632199], [-0.73840546393661888, 2.0612884959102034, 0.78902540063244708, 2.0523247881476814], [-0.62923890821423301, 1.3445393617527228, 1.8357116596019971, -1.0068134870284915]], [[-0.068716399333471789, 0.83771677514361709, 0.65840488699565081, 1.1380833546720259], [-1.2953094012941175, -1.8530507539083279, 1.1382687027006304, -2.005094002358919], [-0.86234715942178708, 0.034236960061266246, -1.2056394403046398, -0.86978159650313802]]], [[[0.98820528440953037, -0.64719840743373469, -0.096219288996229321, 2.1278522196067828], [0.149266819355417, -1.0100532298534401, 0.85425827364793527, -2.1900332163384695], [0.051950614251230577, 0.97137737347261632, -1.5472075575114768, -2.0020957516175324]], [[-1.8741015667076077, -1.1820480752123805, 1.088269378983886, 1.3547821357265486], [-1.6432844538511642, -2.0141638551170216, 0.027252562959731899, 1.5422880443678313], [-1.0475585644316545, 0.89069692291601721, 1.1507218380257365, 1.6939576812024204]]], [[[0.73687036934146433, 1.8486154180433592, -1.6597032812795978, -0.17762258312022469], [-0.48918970240646675, -0.22279886201080679, 1.3977245611047633, -1.6246567041980158], [-0.3881123704857386, -0.98224812228207092, -0.22583541468297408, 0.048137329227839268]], [[-0.41355644693991489, -2.069630387950566, -0.88541688828161902, -0.16959055896591008], [-0.61753014031368336, 1.6795726331813321, 1.1867339187840245, -1.5026422044980898], [0.32664385783768152, 1.6631136367524397, 0.025077529567863417, -0.18591397949956082]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[-4.245621698442676, 4.7291786536930047, 2.1896219998076134, 1.2017880076784131], [-1.5842862898235732, 4.550510674113827, 4.9375750750645224, 2.3548469536090311], [-1.667798714354757, 1.3876007030011568, -3.0393568392994021, 2.2313272102248982]], [[-1.6600363284556394, -0.64591073483344363, -1.4393406060646097, 0.13391271881998534], [-2.8506022917076859, -3.1198537030529017, -2.6458421658625895, 2.8695464891644251], [-4.4475011460195812, 3.9731925239203179, 3.1734812127727032, -1.3021445780765251]]], [[[2.2270327191229642, 2.5017836706449303, -4.9770968299337071, 0.87126820457032217], [2.7915377436448265, 3.8641267316298471, -4.0288618902678639, 4.1046524113235225], [-0.62916133169498067, -1.8066201903638577, -0.20296359434424538, -4.3955812545951378]], [[2.7487746848432604, -1.5118616077617197, -4.3240312737751214, -1.863763446029373], [2.7632748669576905, -3.4751766644021376, 2.4523402269876389, -4.0355016699635726], [-3.1335249894178685, -1.5671905465064553, -1.2689315645077546, 3.0941143337653578]]], [[[2.5795945948148304, 2.7339856485021574, 0.62492130945042224, -1.9853013431326429], [-3.3493469870362178, -0.77734081411390932, -3.5839790418101591, 3.8119219847967951], [1.1423573012114847, 3.5420843051227315, -1.8699883445138212, 4.1127130117539537]], [[3.8231562337381249, -0.58899448675895716, 3.2338104335564619, 2.3099276098467323], [0.88558285165438555, 1.2097864974562285, -0.28655790399372449, -0.044782597057592355], [4.5877074178678594, -1.4971784770954208, 4.0247986777225933, -2.5809680500909851]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array(-1.37197362232)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.323150228581502, -0.29010822444843876, -0.62658012316412659, -1.141610345216111], [0.86598844611293024, -0.30149882520372634, -0.27786385046599155, -0.58261689585374243], [0.82262542266866345, -0.98873805652806146, 0.45140261406047577, -0.61486886192010026]], [[0.82647204690918208, 2.1240916868739093, 0.95319594023928333, -10.245282407914042], [0.4812925416896508, 0.43975575552783275, 0.51853948055705312, -0.47811514032025199], [0.30848190416999055, -0.34530761196757209, -0.43232448227529541, 1.0536261836216061]]], [[[-0.61605454223530109, -0.54839818423175535, 0.27565741017351469, -1.5746857455896064], [-0.49147593488418717, -0.3550539921716418, 0.34053627542718229, -0.33424842954727807], [2.1806388174336697, 0.75941452976124257, 6.7597030233675257, 0.31212564228862227]], [[-0.49912189234257581, 0.90747302218588277, 0.31729040227882666, 0.7361307709115319], [-0.49650276877191396, 0.39479248245883558, -0.55945484530407741, 0.33997597684917324], [0.43783714090539749, 0.87543510607609731, 1.0812037943547832, -0.44341400295079697]]], [[[-0.53185629442707805, -0.50182180841877699, -2.1954342115952126, 0.6910656798113447], [0.40962421260998477, 1.764957657454598, 0.38280737870313919, -0.35991650086077831], [-1.2010021915799372, -0.38733511236254931, 0.73368030680374929, -0.33359332839449168]], [[-0.3588588952277465, 2.3293488363055057, -0.42425913655472813, -0.59394658796834265], [-1.549232372509594, -1.1340626012995945, 4.7877709991638593, 30.636312149514648], [-0.29905429822723117, 0.9163727927641685, -0.34088006188142012, 0.53157326851604003]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([1.7158106303113243, -1.4900869249284554]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([1.0455457086724795, -0.33082901630453954])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.60935961708243469, 0.22201994445419609]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Data(numpy.array([3.7767095027826603, 1.7715995358649126]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[-2.0218429836477192, 2.9284588012750152], [-3.5513249040642569, 3.9891772805787244]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.53534511514799743, 0.77539953737965328], [-2.0045867207401726, 2.2517375963474544]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_constData_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[-2.1161691103609859, -2.6392997732273726], [-3.0428516616637333, 0.13647888008007847]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([1.6726359748992188, 4.9941892654225342])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.79040751833575951, -0.63374232509174055], [-1.6412858136804054, 36.593129006423652]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_constData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Data(numpy.array([-0.31217417736842279, 1.2520109737858141]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-1.2906161350208869, -1.5717561129091031, -3.1138238010692412, -1.5333100333898964, 1.3478107860622366], [-2.5687611978767135, -1.6391699547307104, 1.8865535713757255, -3.5322441507242264, -0.8837206796448811], [3.5226972395762886, 2.6373643642355802, -2.8191021179693134, 2.400922865253742, -3.0885753690794293], [-0.73759561858786782, -1.6468912180064308, 4.0094224539352439, -4.9545658228842617, -0.26179709243282012]], [[3.4721317155994686, 0.98561145151478158, 0.79772645613402471, -4.0856901534721359, -1.4406349379702998], [-0.4707649507831233, -0.88108462977525814, -0.050450063512609056, 4.8681040722438844, 3.8358270612800887], [1.6386517920812373, -1.7840332014865421, 4.7592323871899413, -0.6884682640866755, 4.2884324887926617], [-2.7625447736707445, -3.1909410696326446, 2.5595280232339324, -2.1756999048064851, -4.0748617187369316]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[4.1342821686936748, 5.034869079046671, 9.974636042347468, 4.9117132182919452, -4.3174960767865587], [8.2286152542498865, 5.250818528773407, -6.0432723400732993, 11.314978645897192, 2.8308577189007167], [-11.284396644437569, -8.4483745147280604, 9.0305423136977012, -7.6909720255952259, 9.8937567325895248], [2.36276947954401, 5.2755523595495761, -12.843542946870297, 15.871158417555355, 0.83862507347573323]], [[2.7732438359549541, 0.78722269385107935, 0.63715612150097223, -3.2633021906492408, -1.1506567978506828], [-0.37600704837245197, -0.70373554882753642, -0.040295224697638896, 3.8882279581973442, 3.0637327799782503], [1.3088158381921398, -1.424934157000243, 3.8012705054805056, -0.54988995983389377, 3.4252355439228754], [-2.2064860704194937, -2.5486526367927267, 2.0443335376641834, -1.7377642451708173, -3.2546533569233977]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_constData_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[4.4985900239299923, 1.7552556672762911, 0.52348971261890398, -1.9751847503553877, 4.9600173393042883], [-2.92909545071115, 3.0757875450406598, 3.0866441855384839, 0.10249111843226277, -3.809338018125926], [-1.9099009862172514, -1.0835733276706536, 4.653030801378705, -3.4947119745536916, 4.5295579708076659], [1.6616507447031079, 2.1824471901166751, 3.5992084565745106, -4.8336627487518804, 1.6840891003555836]], [[3.619513608256689, -4.9705601331349305, -3.7807928961917323, -1.3887195533026695, -3.4381364600679531], [2.35309601780855, 4.9498578024926729, -4.3312776065661058, -1.4322718647528743, 0.47436568395702317], [3.9843506575301841, -2.0623400180947504, -1.6020159149058557, 4.4981946071310528, -0.26355527676969182], [3.1842748673446248, -4.3182530579082972, 1.3309400837995833, -4.9923927181882366, -2.7269287961469346]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([2.4182168075135158, 2.5822140436673759])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.53754994223744546, 1.3777006122794471, 4.6194161016378112, -1.224299047002269, 0.48754200682949717], [-0.82558484289967438, 0.78621061178708573, 0.78344527653797036, 23.594403539578263, -0.6348128719496523], [-1.2661477348640122, -2.2317057330230998, 0.51970788734022388, -0.6919645524785617, 0.53387478934999111], [1.4553099170942723, 1.1080299301007308, 0.6718746181806361, -0.50028662180412431, 1.4359197544850366]], [[0.7134146526696109, -0.51950162044187453, -0.68298214542995861, -1.8594208150424061, -0.7510504814623733], [1.0973687533890795, 0.52167438878082761, -0.59617837465619972, -1.802879821361927, 5.4435093662915985], [0.64808905280140094, -1.2520796866720845, -1.6118529283269465, 0.57405565325558716, -9.7976184552884042], [0.8109268675730531, -0.59797654492211838, 1.940142967439707, -0.51722975122928105, -0.94693123169037785]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_constData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Data(numpy.array([3.6519586774675119, 4.4161741019590011]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[2.8595800564966991, 1.4955791208771423], [3.3585434066588213, 3.2351316857059267]], [[-3.5991854751258434, -1.944825930097005], [2.7204235973042943, 1.2428625153575021]], [[1.5626697466046018, -2.3401187415714473], [-4.0682640542044615, -3.5280027077203604]], [[-3.0452796595213485, 1.7489934281057948], [2.2507799715134205, 0.28027181292068093]], [[2.7228544692508008, 3.222327694953691], [-4.1663512374714138, -0.35114030432288068]], [[-1.0333889217155989, 4.3520500225195367], [-0.12022556365024961, 2.3018312514645789]]], [[[2.120380446713483, -0.77576575116529156], [-1.1475746931473498, -4.1126067502786556]], [[0.50756160543077655, -2.8427038581472486], [0.31093176061445327, 4.3439734172365938]], [[-4.4591729890302716, -4.2452330083826908], [-0.5715438649607556, -0.72158496080866907]], [[1.8083827848472112, -3.2334901275581398], [-3.1096450434206711, 0.61991980645777112]], [[-2.3816406734402404, 1.1088227087113527], [-1.0239803506480971, -1.8374762066618509]], [[0.53886116366128167, 3.6616629895521662], [-3.3742770392999111, 1.9345484312789329]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.78302640009051361, 0.4095279418425038], [0.9196553694270817, 0.88586207332153111]], [[-0.98554934296839725, -0.53254324647661799], [0.74492179062409325, 0.34032765020752587]], [[0.42789907680123346, -0.640784561996804], [-1.1139950950993895, -0.96605767460843506]], [[-0.83387571669708183, 0.47891928210936169], [0.61632131420343639, 0.076745614524597605]], [[0.74558742574244896, 0.88235601208616277], [-1.1408538829252615, -0.096151226050121275]], [[-0.28296840489778297, 1.1917029755488664], [-0.032920844475058865, 0.63030046469770229]]], [[[0.48013968601756185, -0.17566466657670138], [-0.25985721274853935, -0.93126010327679698]], [[0.11493242651045478, -0.6437028505932848], [0.070407496044262594, 0.98365085183340506]], [[-1.0097366829473948, -0.96129203930151186], [-0.12942059161735053, -0.16339594955927489]], [[0.40949082692301875, -0.73219262939017138], [-0.70414910545334752, 0.14037485663954613]], [[-0.53929954264795676, 0.25108219991134006], [-0.23187046683550419, -0.41607875148009954]], [[0.12201990936504169, 0.82914824121808606], [-0.7640724666636427, 0.43805981979305875]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_constData_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[[-1.496485663230068, 4.8095980236517661], [1.924115244839526, -2.0410651134213098]], [[-2.2750382218687148, -1.7843543178724621], [3.236507969890587, 1.5236211408142335]], [[-3.1742360660163094, -3.7431598241466482], [4.7139688779276536, 1.5979158386296524]], [[3.9935232087417951, -0.3565165336834788], [-4.2067408639730619, -0.045617119367387993]], [[0.17876959673458437, -2.7036760468420171], [-2.3159847582038919, -1.7371018033979144]], [[-1.066951236351855, -3.2258235275781133], [-0.039192340085377531, 0.68005632269132654]]], [[[0.63575878704426891, -3.8617812099321869], [2.2891172819954591, -0.53680206065794156]], [[-2.3648769895519752, 3.4710238144001799], [4.0374385117179887, 3.6965927779287284]], [[-0.88697729757401333, 2.0151886106428556], [1.4271175167819647, -3.6909485368852533]], [[3.9084398611794651, -1.4104254095002213], [3.6196794454006973, 2.8082060751903839]], [[-0.029731138786011968, -3.9870291077448385], [0.30258642860421503, 0.27466318589536431]], [[4.8848889452787905, -4.1936066801411371], [1.2983236030821557, 0.093737083229358475]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([-0.47496336177435694, -0.95246747070391624])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.31738584167200046, -0.098753234561114775], [-0.24684766832351021, 0.23270367939325834]], [[0.20877159654233077, 0.2661822021652458], [-0.14675179736709051, -0.31173324460471408]], [[0.14963076214127943, 0.12688834676799762], [-0.10075657563169137, -0.29723928525652393]], [[-0.11893341717275246, 1.3322337588865856], [0.11290530534979927, 10.411954291745824]], [[-2.6568464126454652, 0.17567317738718358], [0.2050805214032167, 0.27342287069490656]], [[0.44515939022514561, 0.14723786273917791], [12.118780372381067, -0.69841768383931335]]], [[[-1.498158562828632, 0.24663941816647911], [-0.41608504649164835, 1.7743364649839581]], [[0.40275560839397434, -0.27440534022049345], [-0.23590884862754913, -0.25766091314975786]], [[1.0738352304044605, -0.47264433000148515], [-0.66740647459198299, 0.25805493118787615]], [[-0.24369505596447541, 0.67530509893566038], [-0.26313586191013622, -0.33917292577587749]], [[32.03602383209207, 0.23889152673948127], [-3.1477534372493281, -3.4677653199099212]], [[-0.19498242055727164, 0.22712370123176659], [-0.73361330599151542, -10.161052999413185]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[0.91702308068503946, 1.7902309961862342, 1.9431168495614903, 0.51751997838723529, -2.6448874208326534], [-4.9191246099635357, -3.9226857882846264, -2.1187595308106646, 3.3904720964837516, 4.3325371431857285], [-1.857209916706136, -4.5236748767306825, -1.3295614082323128, -1.3386120465357965, -4.8799072235358532], [3.9988078221056114, -4.3019379441696861, 4.4575080134185256, 3.9458332575700368, 2.2077731506116782]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[-1.4878270799156312, -4.0590098841906368, -0.09232035063154953, -2.7432432755029379, -3.8661353929975997], [-3.8247188514338548, -3.040531962342321, 4.8063060152838073, -0.70127191606768857, 3.600540811750875], [-0.93863311226005486, -1.6260832259128222, -1.8331731330667491, -2.1439521318705532, 1.5199434469651605], [2.9631742099610392, 0.45164458094710547, 0.14285115098324308, 1.1169437187888462, -1.4832154021259258]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-1.6224532525442945, -2.267310695009543, -0.047511476549845048, -5.3007485509096632, 1.4617391131833042], [0.77752022050569813, 0.77511483877273091, -2.2684528118416782, -0.20683606769540311, 0.83104672683853453], [0.50539958020726727, 0.35946067527470349, 1.3787803419354669, 1.6016232166884363, -0.31146974262839555], [0.74101440773934235, -0.1049863077544409, 0.032047312209695505, 0.28306916331195758, -0.67181512815978905]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Data(numpy.array([[2.1279734708328952, 2.8312547298688013, -0.71325347849995602, 3.4649306615744351, 1.1102515512851472], [3.331909694726404, -3.0448577706574631, 0.78848719381260857, -4.3564123823420529, 2.6739921657714474], [3.2628089177852324, -3.6529834092288938, 0.008303880400003294, -1.3921712713931922, 0.20076433255455228], [-2.8225228366257404, 4.8723787717631595, 0.13355489266203779, -1.7484181376671035, 1.7432344777726563]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[0.96171822519896821, 3.7917938475604078], [0.06718105645993333, -3.4841784371207742], [4.2709684289954915, -1.43406090989465], [-2.1917834646921652, -2.1690677127402704], [-1.6799548871706573, 0.93636843404449266]], [[4.4658878835913445, 2.7093789333743077], [-1.0502462657768796, 1.9450523973335132], [2.5585355915584751, -2.3422762539537709], [-4.2353492427219033, -0.78169908428128565], [2.0431634993191938, -1.125084638178099]], [[-0.31131709437047128, 0.47925146129171292], [-1.4551200695031907, -1.8276558216369088], [3.827664800939079, 1.7494782052811795], [-3.8173147970369294, 1.9217035801440208], [4.8074103917164397, 0.92919749613002622]], [[1.0381376905821948, 2.0393068379607389], [1.0732001018128488, 1.116410776294364], [-3.2628111377643023, 0.90899475227475524], [0.98547408829449701, -0.70415713852342066], [-3.4035138803494447, 3.8035376134952799]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.45194089042028746, 1.7818802252625303], [0.023728368822202889, -1.2306128446740745], [-5.9880092530046527, 2.010590839193136], [-0.63256199871435159, -0.62600609495448445], [-1.5131299616074056, 0.84338403577155108]], [[1.3403388125013562, 0.81316097421925637], [0.34492457279871713, -0.63879909796690648], [3.2448663867158953, -2.9705951755894149], [0.97221035820418256, 0.17943642972133783], [0.76408731688626341, -0.42075091041020751]], [[-0.095413829683226051, 0.14688309164516591], [0.39833744271243521, 0.50031867569382349], [460.94893189183705, 210.68200901357943], [2.7419864749951457, -1.3803643413937912], [23.945540179106047, 4.628299680061656]], [[-0.36780488615045681, -0.72251207731544242], [0.22026204285109213, 0.22913053943266615], [-24.430487515128956, 6.8061508953848442], [-0.56363753444551024, 0.40273955260094152], [-1.9524131284382007, 2.1818852609862835]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_constData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[4.4254649001289721, 2.936303721661826], [-0.10866321776083687, -2.4774555926668453], [3.5405682229655131, 3.8137227697530012], [3.6766645026987366, -1.2591468984919274], [-0.91390628193190615, 3.9564823815810151]], [[3.0827171791614809, 4.9662179137261635], [-2.5805953609109631, 2.2545047537754206], [4.251385567178124, 1.8793874026662616], [-1.0088146723163236, 0.73018735309904326], [0.42223599831307723, -2.9436013688555382]], [[-1.3461947302636821, -0.50223446673645444], [-0.13736663755099077, 1.2695546810220879], [-0.16388271738930538, 2.4308639929065015], [-4.2064247290183676, 4.3018269448093633], [4.3529473650190784, -1.4514339801958487]], [[-3.2014891144845956, 1.8361910687777989], [4.008477743912163, 3.7750433131043089], [2.6722526445029136, -2.8680213896297246], [-4.3680108659319803, 0.50766726068793933], [-2.1099677982618994, -0.78120652338941454]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[3.7300092393838717, -1.1249697179536033, -3.5725434636503897, 1.6650565409157796, 2.9240625238395008], [-1.714407555681714, -4.4861526857108149, -3.2415138133318666, -1.7904113634477303, -2.2088734860220449], [3.0830024846717929, -4.1910096972328876, 3.2098022459509519, 2.9363052115217396, 0.52271188764701826], [0.73568079718682, 4.2466314988551552, 0.1935728786362203, -2.947379941604523, 2.1315070371830123]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.84285138930266224, 1.2703077041610809], [10.352810648674273, 0.45408269729776873], [-1.0090311042384306, -0.93676013683652426], [0.45287149254265618, -1.3223687743741479], [-3.1995211999837951, 0.73905612153163236]], [[-0.55613520671657724, -0.34521392042488735], [1.7384177130843095, -1.9898617105145819], [-0.76246055835472848, -1.7247714913557335], [1.7747673706378544, -2.4519890078195825], [-5.2313717798742054, 0.75039830779833039]], [[-2.2901608625877761, -6.1385720990144357], [30.509662112659463, -3.3011651722309483], [-19.585971584337521, 1.3204367892722375], [-0.69805247940499116, 0.68257167226699367], [0.12008228995544661, -0.3601348010169132]], [[-0.22979331519771745, 0.4006559065100464], [1.0594125177081715, 1.1249225893948878], [0.07243809040077806, -0.067493526839146584], [0.67476479158795666, -5.805731765350659], [-1.0102083258990284, -2.7284808477213676]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Data(numpy.array([[2.0290938764607294, 4.9247943211093297, -0.79425232349631436, -3.037750642146996, 3.2868540067853207], [2.4602655455843214, -3.3852212321135964, 1.0195082068710022, -4.593148780674337, -0.10233486810346992], [3.107060339812918, 0.17110699362467408, 2.99459477923401, -0.81833756923404266, -4.2206727158516264], [2.6335689825511235, -2.5204179831415106, -2.3823351030176454, -3.0248866818939102, -2.4807377019465635]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[-0.20007205076928969, 0.1510191055475909, -2.8813840193787232, -2.4070317900790164, -2.9011091549177359], [3.9399301211823463, 3.8377787496821831, 2.3084317415572233, 3.1261173134914273, 3.6447610822659264], [3.6998466810365365, 0.0093669039605934756, 2.1226698942624367, -2.4625032219835563, -4.962563350481684], [2.3977459353930346, 2.6477699960669057, 4.7509540997423123, 1.524682660364113, -0.60057910006695536]], [[2.8994896109900097, -3.5192927573931518, 2.4584157089476406, 3.2502646469164755, -1.3410113131793802], [4.7595777842817792, -0.29060437089352664, 4.5132842536006113, 2.1700360426778236, 4.0431101158761091], [-3.4138421039057554, 3.5533668082377918, 3.6984281377615833, -0.27011366975174944, 0.18667113680903658], [-3.5011051044820318, -4.4200172253729093, 1.0597199334934064, 3.0995825728999353, 3.4925871436010851]], [[2.4149707775940366, -4.3233633686119797, -1.3662762561714379, -1.1343752367892268, 1.9480221227699808], [2.9216015154921156, 1.2624991478234087, -0.47078333749491463, -3.5719648211848067, 1.3369250138699922], [2.8220474568777494, 1.777568334616384, 3.6477876540120739, -3.4392157479271979, -0.9703894032731375], [1.8125913104300837, 1.5014942502350888, -1.0583763470227678, -1.1017230053174396, 4.4084425486085941]], [[1.8173470168837316, -3.4867326132276935, 1.4200713610412823, 2.2045446214712214, 0.19952658908478771], [1.5960079412812505, -4.7960452854150883, -3.5652377024662596, 0.89818896604421461, -2.7682125988944115], [4.0984493763865348, -1.1960883102607656, -3.4201267111399556, -1.1801974583521933, 3.4947156569180837], [0.23762743920023777, -4.9043987820428541, 4.5132889249483341, -0.21672181907562393, 0.32171207210292341]], [[-4.9225210920943177, 0.61151291128581242, -4.2262326973671263, -4.5944620651142465, -3.8413825428081907], [4.2596697342981962, 2.0032225323442301, -3.4181032021343958, 3.0237458410121434, 0.38038214369681178], [-4.959973517356131, 4.6889423412262143, -4.1400150341822268, -0.97691888899623436, 4.8949832954197365], [3.2325023455673652, -0.95801158209668813, -2.150916877105733, 0.32304462085408048, -1.0148234228578037]]], [[[1.7646651396860662, -3.162098246215812, 0.70235827411889051, 0.87656536738416335, 2.75303760572295], [-2.8562455608613959, -3.3408288127088772, -1.1016692155452237, -3.4246997996322373, -3.564952851222646], [-2.4353980926365013, 0.78088030470272773, 4.0143780357397354, -4.8608748071351515, 2.4364962647292412], [3.3218423356959175, 2.1943952727301159, 2.8461735859356896, -4.7684745210270174, -4.5711222492690027]], [[0.60467954000527602, 3.3085048778241042, 0.82351479606088063, 0.17199429798002708, 4.5401361119931156], [3.7188149157815626, 1.9566733696100158, 3.0446765826288971, 3.8801964354865373, 2.9467555632117657], [1.8402362280124729, -2.0334317219815725, -2.1138916353486268, -0.67917834561851631, 3.7977130246404798], [-3.9275858897400906, 2.8257819262806905, -4.9059048819870155, -1.5834399295859249, 2.0974544090090959]], [[-2.7251702926060939, -2.8369810172616172, 4.3345789078036869, -2.8830746888848227, -3.6294955319583035], [-3.9085882036450581, -1.1888128722359061, 2.3232357896728884, 0.24447048337571964, 1.1945418730091042], [-1.3210299749062071, -0.0059324381643657276, -3.7999640834365875, -1.4330600237054369, -3.7159944866548655], [4.678695723477551, -1.878244402342204, -1.1480169378092508, -0.65279952897732763, -1.0078519981462475]], [[3.3802451998212018, 3.4285814973156441, -2.2601513281008745, 1.8594787208099106, 0.434661543440237], [-4.1937477786129147, -4.7591089423906574, 1.6295559943942237, -3.428077196622362, -4.8551517107017981], [-2.1654399148981063, -2.2260203439129214, 4.2167880248245257, -4.8875332856463958, -1.2876170460756917], [4.6126287773649643, -3.4968714336668336, -1.5096002689154133, 3.2901033652638478, -0.37491875800910091]], [[-0.039089123001980752, -4.6085704632788378, 4.1398217366651568, 2.8933663666039493, 0.43758403321942829], [1.9651126900914795, 1.1375444903845384, 3.6503136266001199, -1.8948619067322947, 1.9628526274764067], [-1.6455821408168401, -0.23774524034778199, 3.2306679660026099, -4.9619076626691703, 3.5411748525212872], [-1.122766706319994, 0.10557826413426774, -0.66056262857740666, 2.3437457128746022, 0.20398948747284606]]], [[[3.8272728987028319, -3.574821529419161, -1.1999969073156991, -4.6402354899247378, -1.9649875153174765], [1.311215726489011, 3.5239847578950609, -2.4634530415949598, -0.83343545108372119, -1.4442255572137874], [4.4843326551676306, 3.8070208620945536, -4.0360830437138473, 1.6868311976708208, 0.36285289308920188], [-4.4531875795589171, -2.8746247454387142, 3.4310673668170431, 0.16931178828400473, -2.5672541273029257]], [[4.3280617794114349, 4.2573223830673239, -1.6499789133150067, 0.21470408816302999, -1.0768300121483811], [-2.0457528351802159, 2.2421849599365622, 2.9546676903787885, 1.3469768791186079, 0.69617366992030494], [2.1841939071947722, -3.4228440233695379, 3.9323413210395781, 2.7543609366751545, 3.4518034562924722], [0.80995524628626114, -2.4333961669795734, -1.9386883472571093, 3.5939833783119344, 1.1644899010372463]], [[3.2466937254579875, 4.0125468976779946, -4.4230274733163597, 1.1390065734027317, 4.0884247433494654], [-2.4716533567279999, -1.5890143694170367, 0.92979271475137359, -3.8253127270526734, 3.5558758225105045], [-0.3954232769453867, 2.7514109299587801, -1.38464558730516, -3.7361559440872383, -3.9632554290758981], [-4.7829731616225599, -0.68603371817595971, 2.3011693055936568, 1.1343733183910931, 3.3269777110037211]], [[0.82561416277441957, -0.31275094448408503, 3.7844172531310267, -3.1322554557713578, -3.1544243215450898], [-0.7416574477298159, -3.2306144756427422, 0.35155858114923966, 2.1185592262383288, -2.9721107273819225], [1.6292009757241086, 0.93431461991440834, 4.115005694264406, 0.39376556717588862, -0.67506798595079509], [-1.5533056551420712, -0.87450109563193834, 3.3087670871077375, -3.4164021584270143, -3.4358691794038965]], [[4.8341974918664938, 0.41065244937078749, -1.3558802147068096, -3.4888444772488447, 4.2827589104487362], [0.69479625151875002, 1.1025705477853123, 4.905086576158995, 2.7692694349755973, 4.2973362595447142], [2.6591021183818668, -0.82754996224903454, 1.6381069558694383, -0.044324360441114763, -2.9381136559495769], [0.48956220520557014, -3.9425682952825838, -4.8341635187658323, 2.2324744877149811, -0.73949060042459624]]], [[[1.1169544109555707, 4.9022741098704277, -2.2327598150993233, 4.6179972559757783, 2.2240017315466645], [0.41061223596311436, 4.548422579148081, 3.8677377803060011, -3.7806096620761065, 2.5421148419385879], [-1.8325467075435276, 1.8336438900955434, -1.0338436582648782, 3.0078422236024895, -3.628078864347045], [-2.9777319962505064, 2.3626486997151277, -1.6785419725054385, -3.7739986692849747, 1.4336801112328903]], [[-0.91420812515484329, -0.38956902757274747, -2.5942987332312417, -1.4183151097507221, 1.6635511810183576], [3.527820042729191, 4.4310346870705537, 1.0831231184407821, -1.9645812037783306, 1.5511616128934644], [4.1464259491541746, 0.8291073673157312, 1.7666830779914715, 2.4983801789083504, -4.1723436618007899], [-4.3711218871851614, 1.2331031845927587, 0.45917461613982269, 2.4742441797740895, 1.887553195531579]], [[-4.6603804123972505, -4.9341252355970022, 3.2357296831000966, 2.3105308213383671, 4.0584286667138318], [-4.6997737366997372, -1.1316973189047861, -1.8775015563685891, 1.9545094702764807, 0.85054968964103228], [1.9981401847620823, -2.8236227262170202, -1.4181245553311452, -4.9988054432865461, 0.92016162156430692], [2.7137891855027743, 3.7254762417791429, -2.1033929120675654, -1.1210393769025462, 0.4412783766605175]], [[-1.288888869460302, -3.7626832944673394, 1.3944548117333433, 3.6221510836979434, 3.1519338123448897], [3.3100194013557562, -0.94475662065394062, 4.1318500434412329, -1.5552491479953656, 0.030249045489025406], [-3.6204330086581993, 4.9800981539391103, 1.9646052498196553, 4.8395660106012901, 0.18439745731175172], [4.4469815487046134, -1.1243864340420453, 2.1631579168906683, 4.6215619329621198, 0.69006420628723042]], [[-0.44418405071911238, -0.86336314889578958, -1.5364432216600941, 2.8463671682520886, -2.2486940411690606], [4.4289673526247597, 2.7983233111529184, 2.0354187557068997, -4.8989013631928922, -2.9168564647462003], [-3.4855176990290468, -1.6262529326646966, 1.5116001795206522, -2.7696057317713132, 1.5468250218272228], [-3.0933269892108726, -4.5294940307036944, 4.2742009508494103, -3.2758657710289816, -3.875073651087042]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.09860167293899072, 0.074426869697624701, -1.4200348504351168, -1.1862594520651304, -1.4297560051672076], [1.9417189943200732, 1.8913756500888335, 1.1376663092511679, 1.5406469605754238, 1.7962505946858129], [1.8233984755254582, 0.0046162989644085899, 1.0461171456319844, -1.2135974833647452, -2.4457041677823663], [1.1816830966812295, 1.3049026596469304, 2.3414166071158911, 0.75141060650360769, -0.29598389065887992]], [[0.58875344266903074, -0.71460705319372952, 0.49919154966737223, 0.65997977478668401, -0.27229793281545855], [0.96645209402565779, -0.059008427955640354, 0.91644116674175657, 0.44063485725207185, 0.82097034967449811], [-0.69319485877265508, 0.72152593114536034, 0.75098123832479113, -0.054847705739496801, 0.037904351864787758], [-0.71091397451363891, -0.89750290817775369, 0.21518054651563606, 0.62938315202608131, 0.7091843670771546]], [[-3.0405586564270757, 5.4433122078641825, 1.7202042924559073, 1.428230303180839, -2.452648944349006], [-3.6784299259348314, -1.589544166853504, 0.59273775293790409, 4.497267071830457, -1.6832497360345413], [-3.5530868130860997, -2.2380398294480188, -4.5927314860779269, 4.3301299173891019, 1.2217646389770247], [-2.2821353577551036, -1.8904499311068823, 1.3325442251950543, 1.387119650424985, -5.5504307865320976]], [[-0.59825417915121637, 1.1478007986736429, -0.46747463117565702, -0.72571612392553453, -0.065682346113757381], [-0.52539136002066233, 1.5788146725648882, 1.1736439630694806, -0.29567567317158072, 0.91127051723307917], [-1.3491724170914394, 0.39374143936166012, 1.1258747389228456, 0.38851031482889037, -1.1504287443580681], [-0.078224800911337947, 1.614483662351089, -1.485733839482779, 0.071342860098105712, -0.10590470055025532]], [[-1.497639104728216, 0.18604809036951944, -1.2857987268806492, -1.3978296740985525, -1.1687110333705457], [1.2959716876699157, 0.60946501676338971, -1.0399315561561684, 0.919951368320582, 0.1157283356399639], [-1.5090337164707812, 1.4265745699524373, -1.2595676673304188, -0.29722004292843585, 1.4892609423219356], [0.98346392595906207, -0.29146764052160112, -0.65439988288662043, 0.098283836211524186, -0.30875220522810598]]], [[[0.71726612716797289, -1.2852670525306256, 0.2854806772299362, 0.35628892537938467, 1.1190001870587081], [-1.160950112067284, -1.3579139124656641, -0.44778467817122297, -1.392004129708226, -1.4490114116425419], [-0.98989237036121891, 0.31739675666484451, 1.631684857329621, -1.9757520954839358, 0.9903387335981918], [1.3501966654201014, 0.8919343185001356, 1.1568562552298449, -1.9381950576780071, -1.8579792158912447]], [[-0.17862334498822058, -0.97733786094636022, -0.24326764474022602, -0.050807402585219145, -1.341163782420931], [-1.098544130736083, -0.57800457797210125, -0.89940254236439465, -1.146216500912083, -0.87047650985336922], [-0.54360885207596998, 0.60067912332925411, 0.62444711598030411, 0.20063041646305182, -1.1218507637296544], [1.1602154247649756, -0.83474069566684883, 1.4492124873398495, 0.46775079707192091, -0.61959153189510441]], [[-2.673024380029251, -2.7826956155347347, 4.2516370918749642, -2.8279072884889649, -3.5600454292542461], [-3.8337976852986819, -1.1660650343213235, 2.2787808612185563, 0.23979256049937062, 1.1716844111292661], [-1.2957521734529365, -0.005818921441126129, -3.7272520788225445, -1.4056385363524211, -3.6448892334665124], [4.5891692601838407, -1.8423043480020336, -1.1260497267919578, -0.64030826292301346, -0.98856683188404237]], [[-0.73593200682799031, -0.74645557133733464, 0.49207013228277213, -0.40483746762866968, -0.094632585225428459], [0.91304418360191142, 1.0361321110290609, -0.3547797104353721, 0.74634577722498108, 1.0570421169742721], [0.47144998307243818, 0.48463928564188841, -0.91806040391433685, 1.0640920899864339, 0.28033427776024533], [-1.0042410985624044, 0.76132335368275295, 0.32866348141541879, -0.71630672603224832, 0.08162565070536594]], [[0.38197267193873813, 45.034215108570336, -40.453677357354074, -28.273514396661859, -4.2760013407843624], [-19.202767605119405, -11.115903225031538, -35.670282223936795, 18.516288161102782, -19.180682633916945], [16.080366069881535, 2.3232085481108919, -31.569571797719117, 48.48696983370543, -34.603795540546699], [10.971497077465074, -1.03169394841569, 6.4549125905895277, -22.902709079615569, -1.9933527179279111]]], [[[1.2317987036367886, -1.1505478292817473, -0.38621615806404108, -1.4934487851639648, -0.63242657058787344], [0.42201167118883881, 1.1341861349584368, -0.79285651779240596, -0.26823922290929952, -0.46482056969023877], [1.4432718276200716, 1.2252806336949933, -1.299003753482582, 0.54290261957783159, 0.11678334290445416], [-1.4332478589157531, -0.92519115531943641, 1.1042808930526384, 0.054492597428667675, -0.82626465099725255]], [[25.294476208874944, 24.88105420404866, -9.6429659499147178, 1.2547943460101161, -6.2933138461331684], [-11.95598608708892, 13.103993661736766, 17.26795397305559, 7.8721322289913127, 4.0686453263703122], [12.765076756510821, -20.004115266484085, 22.981768528206576, 16.097301918102126, 20.173362778286307], [4.7336185922529319, -14.221488645386797, -11.330269477527336, 21.004304395619236, 6.8056242259248236]], [[1.084184660967205, 1.3399298380879323, -1.4770036680714878, 0.38035415719721494, 1.3652681062895753], [-0.82537155740324453, -0.53062750941664705, 0.31049032783968389, -1.2774057957955678, 1.1874313837613999], [-0.13204567098274725, 0.91879240190973888, -0.46238162068102573, -1.2476332257023812, -1.3234696916454463], [-1.5972021305820885, -0.22909066793719612, 0.7684409662205699, 0.37880695119666757, 1.1109942934765724]], [[-1.0088919216396086, 0.3821784019726937, -4.6245185305047292, 3.8275835957319222, 3.8546737191811999], [0.90629768889139628, 3.9477772952139678, -0.42960093043057479, -2.5888573443124248, 3.6318884029286278], [-1.9908666508480448, -1.1417227499269271, -5.0284941678970183, -0.4811774284595659, 0.82492605903777982], [1.8981233583054884, 1.0686312452335096, -4.0432789737427264, 4.1748079116357069, 4.1985964088387657]], [[-1.1453618456865053, -0.097295496954429947, 0.3212474185962147, 0.82660862666412238, -1.0147100234434032], [-0.16461741961400991, -0.26123099847196779, -1.1621575294708135, -0.65612039156104718, -1.018163820995919], [-0.63001855329721446, 0.19607063090701066, -0.38811513380726781, 0.01050172885347051, 0.69612449335739102], [-0.11599151087145801, 0.93410898231351536, 1.1453537964718543, -0.52893807172739371, 0.17520680948496276]]], [[[0.4241219494746567, 1.8614565034562423, -0.8478076062911637, 1.7535129273516694, 0.84448204937175653], [0.155914744851436, 1.7270945281038546, 1.4686297590577426, -1.4355460924413876, 0.96527368706934558], [-0.69584154418782296, 0.69625815850827, -0.39256372820103602, 1.1421163613070844, -1.3776281875983158], [-1.1306831208825956, 0.89712808563929969, -0.63736396639948434, -1.4330358134872638, 0.54438676971510058]], [[0.36272083887266665, 0.15456524678782807, 1.0293128959497599, 0.56273011827304109, -0.66002988081558867], [-1.3996964258809286, -1.7580554958378785, -0.42973948197701362, 0.77946642855231041, -0.61543824209667741], [-1.6451342503063591, -0.32895629727348302, -0.70094844974460724, -0.99125628987708947, 1.6554173512919785], [1.7342845180531874, -0.48924551119722959, -0.18218193141420783, -0.98168010081015655, -0.74890482775356437]], [[1.9562237094580275, 2.0711298042609818, -1.3582176911222428, -0.9698597054678304, -1.7035507144117215], [1.9727593027306063, 0.47503699940083699, 0.78809297398607103, -0.8204175255616859, -0.35702353063750852], [-0.83873178976000784, 1.185233228793215, 0.59526661615943188, 2.0982797243572837, -0.38624357270258108], [-1.1391299158818102, -1.5637918599529401, 0.88291227770738434, 0.47056326185285735, -0.18522934750092923]], [[0.42609492685303385, 1.243908843590626, -0.46099406634970602, -1.1974501740442325, -1.0420006247544567], [-1.0942622813504279, 0.31232793820309973, -1.3659520100945541, 0.51415121012784826, -0.010000059066704011], [1.1968821940765768, -1.6463751134045868, -0.64948061082063324, -1.5999164661504583, -0.060960120726340307], [-1.4701316169372389, 0.37171191925049446, -0.71512031503153517, -1.5278463026814995, -0.22812894460402552]], [[0.17905321081328909, 0.34802677776789276, 0.61934932518439623, -1.1473873944910122, 0.90646183165780692], [-1.7853428635963715, -1.1280206323131843, -0.82048930610832627, 1.9747760351079702, 1.1758020456807776], [1.4050327434029246, 0.65555214942257811, -0.60933494836416724, 1.116444406677128, -0.62353429006761729], [1.2469383549835309, 1.8258657604750115, -1.7229556141689497, 1.3205208146183709, 1.5620650454283753]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_constData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[[-2.2335072401269604, 4.537518349120873, 1.5794196135461691, -2.3967851694463169, -3.2072279562952541], [1.1456196593269006, -2.1227211525179115, 4.6151592595889781, 4.976412675715121, 0.83904010121652561], [-3.1178240681553304, 1.6660499205430179, -3.0965629549527218, 2.6837585866013836, -3.4209990763238798], [0.31679141857311333, 3.3375999262576492, -3.7957036218119269, -1.8467312134764557, -3.4725349517798865]], [[2.6385228575422381, 2.2955484223346678, 4.7611397588710567, -2.9105117955137096, -1.9257855769899281], [1.7801996351350526, 1.3972785002052461, 1.2032262081224001, -4.0979327577788505, -0.3048658121658212], [1.6629322274378113, -4.7538275337670495, -0.93805222855062187, 4.3390892780371129, 4.9359111187047926], [3.1186811496326357, 1.3015930182692372, 2.4332419290695597, -4.2322380808371278, 4.5486602911268061]], [[3.8860475996380703, 4.5261927568709428, 1.309176223937988, -4.0596920929196276, -1.8016328491642799], [4.0909148470083103, 4.4492406367866817, 4.6266510511779817, -1.5030239844562079, 3.218043805066003], [4.2177322962306523, -0.60320387181745616, -3.243670681991635, -4.3879458004273779, -4.7898943042033268], [-1.312040713259075, 0.2162261045160454, 4.7166787245228257, 0.52779288093195476, -0.054887274940284669]], [[-2.7369145912795192, 1.0906501284641585, 1.0471801459478547, -0.97134236186252831, -1.2759016453858396], [0.59588873620400662, 0.89551447503121029, -4.848401992674912, -0.30688297964620226, -4.8397528067020401], [-0.58510858085951156, -3.6097299198198685, 1.2252610422068102, -0.16478324455274329, 1.9054372023276009], [-3.8912940980849942, 4.9782320324773561, 2.921677799818835, 4.1295734959159009, -4.6889490080240046]], [[1.0763522710595552, -3.4832608128199616, 1.0728535209521652, 3.4983657463660318, -0.091172527259502978], [3.9041987057730925, 4.7994415486913731, -0.77998453943915003, 1.6759570572121918, 3.2216871151334612], [3.8104681490530066, 4.8521845118779492, 3.0040116286723375, 1.4887964078545322, -2.212431150356772], [-3.263438349766453, -4.1561701373164173, -3.8048115544543428, -0.053884823597889486, 3.2830028654211745]]], [[[2.2115692607762636, 2.1106425589993378, -0.22863449599345476, -2.7570697386289647, 2.1634840576129317], [3.6114486496402627, 2.5274611137434713, -2.4108622279051839, 4.1285446278689779, 1.384505088241152], [-4.1094688167953617, 2.8085753464456449, -2.0838133191707673, -4.4352590297545147, 0.5047164394595427], [0.93388921667130642, -4.3517898427272295, -2.4033674550802298, 0.73328740865764885, 0.61215738409958753]], [[3.9897435221094355, 3.1277838332743997, -1.0884192316875763, -3.5530032570314107, 1.1220397554268189], [-4.21612344129848, -4.0904022253914585, 2.5489941958929307, 2.9774914441738476, 3.4388445603043465], [-1.0344460468375227, 1.1844294988876598, 1.3044532949105818, -0.2557806483969669, 0.85930123608618647], [3.2375666032345212, 3.5979743940380686, 0.8850710093913019, -1.3719910395401316, 1.1005688933888464]], [[3.1264124695998134, 1.6673757877174804, 4.6597727703347047, -0.1932815866035984, 4.3124070598220587], [1.7058430201227957, 4.2642618262367655, 0.3832431902329958, 0.88080041891528094, 4.9560787841803773], [-1.8915395139660998, -2.9420690798467772, 0.90375298328869835, 2.0744202660969773, 3.0840337374236668], [3.6899705392098809, -4.6268793043382326, -2.1587743187847117, -4.1176045429359771, 1.4452508968190703]], [[3.2890484873900512, -0.87164310966547554, -2.8699219093948267, -1.4205402770251574, 2.1607235318898201], [-4.0828599962879322, 3.8803794121386233, -4.5584465342435854, -0.30469350567187004, 4.6375684054107502], [-0.47549349983751732, 1.700331551379862, -0.95659126140009842, -1.5839671487500597, 4.7607305515876206], [-2.5840456858416938, 0.01151107358073844, 4.0139807388849782, 1.3987079791084547, -3.1170756527906396]], [[-0.45325981372989155, 1.2724409038699447, 1.5971702241557715, -2.4292118416238759, -3.076952094202956], [-4.4733877927409162, 3.7180150887244281, -0.83080972466628911, -3.9642773466382475, -0.12915998255440453], [-3.5533574331260298, 2.8848481550193874, -4.1586422100885754, -1.3739774787317884, -2.8163100631290181], [-1.4264220876843545, -4.2726294800851106, -0.68133941199542392, -3.9010895195670203, 4.7678272355632831]]], [[[-1.6207861772980459, -3.4277745868773914, 3.9841858583710312, -2.4159391468028768, -2.7235265565212274], [1.6334010494219076, -2.6890479209878295, 4.4467505557394205, -1.1357165523162163, 4.5677502805480934], [-0.17389462729184935, 2.4649086545738772, -3.0892202714064454, -3.6056707503063814, -0.92519020541313601], [-1.6677503152700499, 4.8640184684995003, 1.6018600820792468, -0.96683883726106057, 4.4743459746455017]], [[-2.1735489939466546, -3.4847391020610807, 1.3840071186908132, -4.3088702545199933, -1.5623880597303308], [-3.3972811874222799, 3.5476059793951151, -0.53157067492401477, -1.935893177696749, -1.4606955736888572], [-3.1012174980815885, -2.3039684329931989, -4.1264439733605931, -2.0939740700001486, 0.4599324219612857], [3.3524550935819164, 1.3193624085176907, -2.3697151197467003, -2.7689218637602542, -2.687782302641458]], [[-3.1238825252724101, 2.0088400277605798, 0.038650819897522837, -0.5207998912452636, -2.8381442225136757], [0.74327320967865695, 1.8294426245999631, -1.2252735080485766, -1.6403879649887765, -3.0623924242641309], [-1.7662769414787416, -2.2453497917260057, -4.830456169986995, 0.19460164487428777, -4.7260521745819295], [-1.0588170791073734, -2.9710679201442813, 2.455614779791774, 0.59070319182594755, 3.1295333485313606]], [[-2.0437125073521667, 0.11147040986875378, 1.1843535272627195, -3.0976072658120115, 1.0570649578715088], [-2.0135127842710854, 0.17012081649583255, 0.248839619109682, 3.5626768933796118, 2.0597888126383861], [2.6495525990587376, -1.5003525138195761, 1.5060957201435334, -1.0818140303738986, -1.7170939826845864], [-1.2035020266192129, -4.0324629270103216, -2.1226353294199738, 2.051321956583763, -2.6160114573461457]], [[-1.8465745606885928, -1.4941552811574677, 4.6800692281858716, 2.1882987456239675, 3.1057130147421237], [2.3019128107497959, -1.8748132577112631, -1.9903087957917687, 1.5195761130925529, -2.888751102684318], [-2.0192421963722182, 4.9285563920835447, 4.5122211300678554, 4.4629347806852664, -2.7693748375733964], [2.5998358400649604, 2.2125126656745353, -2.6546837572842144, -3.2856303498585557, 3.1204368012544474]]], [[[0.85426483374658524, -0.89862805678354807, -3.0150359231141852, -4.7259047008056232, 1.6211627954630963], [0.95068807095675201, -4.4570780358178048, -2.948946282030386, 3.7828594719044233, -4.1381101778919458], [-1.9960078846423168, 4.0944722169189927, -3.0776211440596857, 2.912222038156993, 3.8072223786368902], [1.2309333492473673, -4.1545251592965959, -3.778067979361881, -3.4516639252777237, -3.5360526744017671]], [[-3.46215655608068, -1.3306923472829633, 3.8215539778673087, 1.3939915492813206, 4.122932428490488], [2.0929211375755159, -1.7830121095793725, 4.9031921318944747, -0.10797974446375669, 4.6402078945708869], [-4.5119123541567774, -3.2668377560009754, -4.5281640796549052, 1.2121462825124549, -4.603402194906363], [-2.0763288246591047, -1.749102157481317, 1.7743585842357499, -4.6813986649254824, 3.6874949029757804]], [[2.7431306755881044, 0.53486971920880855, 2.0068282291904493, -2.7518584216294304, -3.2796182899655433], [-3.7347190053408417, -0.93435098309759912, 4.6504706618974687, 0.89143195428144839, 0.71807016251176314], [0.47869673292915227, -4.6754927588742481, 2.5536517242490646, 4.3592293125848549, -2.5748425926246474], [0.038469440189023985, 0.44167896804969597, -3.965509013689962, -2.0799645953093613, 3.7325111492745293]], [[3.5836012029034432, -2.8065550253429929, 4.0307803352579832, -2.5729118051712794, -0.064849378101124344], [-1.7604188233544384, -1.2544394690866154, 0.75235812134655511, 1.974240306473023, 4.4255590081995564], [-3.6333886581480046, 0.83281537231977332, 0.28431773080478173, 3.3869829070217765, 0.10339876704431372], [-2.7348389280510519, 4.5981607242504055, 4.8808511032540007, 3.6541854708247516, 3.1848044184493283]], [[0.12760906817959849, -1.8519063754458562, 0.031150051467574436, -4.5378081034804199, 1.5784015961163975], [2.2211127841390805, -1.3563028982682344, 3.5416192528013042, 1.2504416181490186, -4.4443322536242347], [-0.0088566358566053438, 0.80661245543317506, 3.1973621961822936, 4.7599082675986715, -2.2644969517134084], [-3.14187066574112, 0.60339672582511383, -0.33814138563308394, -0.24840065424780988, 3.0435106443287712]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[1.6225850633259231, -0.26742482006053869, -4.2205675726714293, 3.8242634323534386, 3.1588699153319304], [3.9287563452628014, -0.78778141429160087, -4.0897754046868338, 1.5041801074808774, -3.3356325322500835], [-1.0804712189821029, 4.0589187040134114, -1.4714255821026567, -1.549824455195572, -1.8948378729395889], [-1.8212012892028806, -2.0170454829561422, 1.8836292178379486, 2.9980874996338933, 4.7804474834823107]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.72647405576965562, 0.35759305824962512, 1.0273299441196868, -0.67698393832299864, -0.50591510345906621], [1.4163383546326882, -0.76438917160704734, 0.35157726354830693, 0.32605516645436849, 1.9338587762054928], [-0.52042226496953381, 0.97391143165570437, -0.52399550305628995, 0.60459426992675491, -0.47430152044048857], [5.1219350278942022, 0.48615325358820916, -0.42747938853860296, -0.87862546075203907, -0.46726241372869382]], [[-0.10135399028138141, -0.11649713744158643, -0.056168235675557954, 0.091882403800166629, 0.13886531463099505], [-0.15022181489226674, -0.19138977664170506, -0.2222564786698317, 0.065258469542454747, 0.87718861672519133], [-0.16081522484688246, 0.056254632327526762, 0.28508521372390422, -0.061631555131659903, -0.054179423743495665], [-0.085749330319337055, -0.20545963008939658, -0.10990473937903844, 0.063187565291138489, -0.058791996531860485]], [[-1.0860823148600947, -0.9324763215761056, -3.2238345728400164, 1.0396275077197059, 2.3426346686724857], [-1.0316928439021247, -0.94860402419582235, -0.91222949947713039, 2.8080507139734201, -1.3115320450353112], [-1.000672227690532, 6.9969172445044805, 1.3011701823194866, 0.96185499197833157, 0.88114002201837938], [3.2167961939135634, -19.519232343004337, -0.89481769252757692, -7.9966360387789361, 76.895192506154686]], [[-1.3972900157492965, 3.5064071717836076, 3.6519632721759714, -3.9370911663118395, -2.9973026887953895], [6.4177474753343473, 4.270465234210933, -0.78876781218455738, -12.46163419281948, -0.7901774295285573], [-6.5359893145571037, -1.0594320121723333, 3.1211825893571064, -23.207841566254515, 2.0070267483398987], [-0.98277419695300261, 0.76819710439457778, 1.3089271625333123, 0.92606741014189231, -0.81559074876035864]], [[2.9347918894827587, -0.9068714876893148, 2.9443627239330961, 0.90295587835927205, -34.6471685087865], [0.80909557975646751, 0.65817447369334781, -4.0499134990589996, 1.8848155456837508, 0.9805017689314226], [0.8289978532209975, 0.65102015547824821, 1.0515504950718297, 2.1217608389343847, -1.427782245256553], [-0.96795758852254521, -0.76004345610633972, -0.83023032024642585, -58.622627010987458, 0.9621892044637862]]], [[[1.7764563900132175, 1.861402978212207, -17.183567721011233, -1.424975324424145, 1.8159395866302677], [1.0878616107843997, 1.5544280083675925, -1.6296063291333438, 0.95160806031802525, 2.8376611820573485], [-0.95602534546703699, 1.3988431359816593, -1.8853686696014644, -0.88580087857467316, 7.7840863465231438], [4.2068762280671823, -0.90279091758729857, -1.6346881692843971, 5.3577305417731873, 6.4178860654299648]], [[-0.19745164317607297, -0.25186568391041653, 0.7237849087526308, 0.2217226828409396, -0.70209759545635031], [0.18684970334952533, 0.19259265247837792, -0.30905578975460768, -0.26457890108570348, -0.22908317037216716], [0.76154905971170028, -0.66511465227051048, -0.60391691857821705, 3.0799101465603389, -0.9167697906262382], [-0.24332516078729022, -0.21895136763534889, -0.89007707396651603, 0.57418845428877729, -0.71579473036520458]], [[-1.3081368643627282, -2.4528216343392195, -0.87767700406409965, 21.159674217050807, -0.94837415576802919], [-2.397509827365258, -0.9590816819745055, -10.671488780271403, -4.6432487052213869, -0.82520387241244986], [2.1621411419059209, 1.3901017595752132, -4.5253243754774752, -1.9715269232216615, -1.3261124076104762], [-1.1083490670802378, 0.88391659597694672, 1.8944895578474292, 0.99324142521241243, -2.8298030561255754]], [[0.45732986705662249, -1.7256834715967189, -0.52411882795725973, -1.0588788870040879, 0.69614649226561887], [-0.36841334477509702, 0.38763737967877399, -0.32997647250687268, -4.9366989433006037, 0.32434672138224813], [-3.163408349419869, 0.88463929653025453, -1.5724376420492383, -0.94962834845902966, 0.31595573225190399], [-0.5821027529514925, 130.67244309842937, 0.37473525792221796, 1.075406825404436, -0.4825613090700005]], [[7.3592064224733305, -2.621443968128689, -2.0884640107870935, 1.3731336539263224, 1.0840703495301363], [0.74566138389855263, -0.89715411386199295, 4.0149175355282525, 0.84142259498537308, 25.825588284243182], [0.93872699131074888, -1.1562593082919703, 0.80209654106767636, 2.4277199472941478, 1.1843981868047868], [2.3384610775798702, 0.78069782268684762, 4.8956987861322876, 0.85505152227839765, -0.69961270982504542]]], [[[0.66663402866830801, 0.31521069766912019, -0.27118996386977362, 0.44722617306480589, 0.39671771013027812], [-0.66148556679604353, 0.40180437490499932, -0.2429799480403817, 0.95135640735230609, -0.23654340815944408], [6.2133674617142463, -0.43834128172546422, 0.34975531818914002, 0.29965886898860439, 1.1678368541521982], [0.64786149886420374, -0.2221355091432079, -0.67451035896944844, 1.117529806770019, -0.24148137517857179]], [[-1.8674153264166216, -1.1647697532399848, 2.9327296436545085, -0.94199139548368083, -2.5978940883060662], [-1.1947550055734879, 1.1441289499420333, -7.635708468292786, -2.0966646046258379, -2.7787574475651842], [-1.3088145886330953, -1.7617076023651375, -0.98363596603198544, -1.9383805951394246, 8.8250327878713151], [1.2107302232873989, 3.0764243984892854, -1.7128298124068477, -1.4658841613180498, -1.5101367026728498]], [[0.47102462086801578, -0.73247524032213585, -38.06971200103731, 2.8253185279750932, 0.5184463743704506], [-1.9796564210067595, -0.80430266700733921, 1.2008956142748182, 0.89699852321991658, 0.48048237399105664], [0.8330661786654856, 0.65532131676088134, 0.30461420833192621, -7.5612186271765296, 0.3113434908773135], [1.3896881823469729, 0.49525141183282045, -0.59920863574026362, -2.4909727972761937, -0.47017411806561288]], [[0.75833780417751817, -13.903460631573425, -1.3085826313850137, 0.50032955187729344, -1.4661581993184949], [0.76971175316208684, -9.1101400000248525, -6.2282061865415805, -0.43501684311466815, -0.75241910514622123], [-0.58493817248472524, 1.0329735451637634, -1.0290344992467484, 1.4326163385585957, 0.90258568885816182], [1.2877622313185644, 0.38433693830500154, 0.73014164690223704, -0.75552472405483506, 0.59243794626488988]], [[1.0261366712606494, 1.2681666335721995, -0.40487389834489312, -0.86589542526073071, -0.61011364023179804], [-0.82315796849073031, 1.010680858558026, 0.95203210524214144, -1.2469516048678373, 0.65593670260440451], [0.93839058848109713, -0.38446103122268371, -0.41993462162416101, -0.42457216294983452, 0.68421141379326567], [-0.72882981445945594, -0.8564189947187959, 0.71377160000332374, 0.57670451973429104, -0.60723481795171907]]], [[[-2.1318930819328727, 2.0266463699358455, 0.60403966508027185, 0.38536563991491851, -1.1233919840127111], [-1.9156664997069572, 0.40860879584503806, 0.61757696310051502, -0.48143508970636556, 0.44010459144677605], [0.91242189132396068, -0.44479512687310346, 0.59175616619287208, -0.62536484695907057, -0.47835432451280402], [-1.4795287578457619, 0.43836569027090183, 0.48204566438491747, 0.52762995721153394, 0.51503794114463897]], [[0.58259799933470657, 1.5157864904495693, -0.52780766532095225, -1.446956750918641, -0.48922593759185973], [-0.9637465295480413, 1.1312573100986847, -0.41137394348379425, 18.679850493933721, -0.43468860205942839], [0.44704890623548116, 0.61743056546072927, 0.44544443343357443, -1.664028106224396, 0.43816407899096693], [0.9714479994695947, 1.1531890657894219, -1.1367744383105753, 0.43086385657954834, -0.546996141290501]], [[0.68667134037794764, 3.5216598550844416, 0.93861008652334987, -0.68449350556436472, -0.57434403985402172], [-0.50435634251044359, -2.0159760645761433, 0.40504055498533065, 2.1130375782370008, 2.6231826862839349], [3.9349113713644837, -0.40287287671716626, 0.73762181426359341, 0.43210142958068637, -0.7315512114151741], [48.964300197313023, 4.2647020892921716, -0.47500313612582235, -0.905606384880695, 0.50465467951892407]], [[0.83661304087207999, -1.0682446887951156, 0.74379828476611987, -1.1652507845811333, -46.231553600387002], [-1.7030535346816533, -2.3899817994541146, 3.9849207638883168, 1.5186031253662182, 0.67744831649044057], [-0.82514913259019906, 3.5999425554343967, 10.544848860278927, 0.88517940064544209, 28.995389261740421], [-1.0962574317934117, 0.65201885697952477, 0.61425506253102191, 0.82045301848272723, 0.94137256349752652]], [[37.461659674171848, -2.5813656386011377, 153.46515521679009, -1.053470612786775, 3.0286636146620962], [2.1522758851416213, -3.5246164330888923, 1.3497914773591584, 3.8230073392459749, -1.0756278357865758], [-539.75883855685652, 5.9265728557527257, 1.4951222883632791, 1.0043150444775275, -2.1110416950949014], [-1.5215290481585355, 7.9225611921332453, -14.137422056552278, -19.244906974815102, 1.5707017461529567]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[-2.6874585590491771, 0.79360146339773596], [-0.018392451933292975, -2.5625386716545662]], [[0.25047035900046755, 3.5123881221912097], [3.2241912402616464, 3.8317764356120847]], [[4.8033677244302488, -3.8286321516148911], [0.97802976751120507, -2.5697623812222581]], [[-3.831487925923013, -4.1360745409593216], [-3.0261278762164281, -1.3356968965928906]], [[-1.1257434050622894, -0.71315814750367768], [4.8144493908849402, 3.053117912764483]], [[-2.6677907387020525, 3.7710924968629307], [-4.1717330814826816, 2.7779502924669766]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.8379451202889951, -1.9547461584905843], [-0.27684983484890235, -4.5242239951391738]], [[3.9305433773751659, 3.6376115894161192], [-0.90187355864675212, 1.9427935907567289]], [[2.4530359176561722, -4.3194026749588152], [-0.080174637873151156, 1.9295731942586034]], [[-2.9135920913638733, -3.5148981881701093], [2.4474456280454158, 2.9092695783381162]], [[4.9108280170361702, 3.9862746847136652], [-1.0492951598752076, -2.7168658633758791]], [[1.1961159840761599, -0.13340410403145775], [-3.4508638670667349, 1.5523815285115496]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[1.0559958629810686, -2.4631332585017018], [15.052361471597186, 1.7655241831796424]], [[15.692648795092872, 1.0356519447363319], [-0.27972086375793398, 0.50702164476524025]], [[0.5106908440883815, 1.1281842976575642], [-0.08197566223078441, -0.75087611537874521]], [[0.76043358290421426, 0.84981500051855052], [-0.80877138315300157, -2.1780911416048889]], [[-4.3622978335497731, -5.5896082778653362], [-0.21794707445919118, -0.88986601271349508]], [[-0.44835450049508024, -0.035375452642021639], [0.82720150107021195, 0.5588226444228227]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Data(numpy.array([[[-4.4169019301271586, -2.7610931413717066], [0.31828761198979283, 1.6967770470529464]], [[0.77700885317913304, 1.198886146508638], [2.5355370110054416, -2.9579567335368298]], [[2.5215720034606193, 4.3272829517623332], [-2.6738315325706918, 3.8868230473866863]], [[0.96560904612645349, -3.3102968683038183], [4.3480884953339896, -1.7627183077869857]], [[-2.3470127973383192, -0.26181462832621261], [-4.8313989278291416, 0.22269636825275185]], [[-3.7626845751363347, -4.5195155695769893], [-1.4890473590196662, -0.94990964799467026]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[4.7629968296212244, -2.5222021277008477], [-3.4310524739376644, 2.1919310879025664]], [[-1.3742589180550091, -2.3047410780538002], [4.5862140628166657, -1.3567040270088504]]], [[[0.7910758377129401, 0.35897898214342661], [1.7272734535487206, 3.4465366979151142]], [[2.2331681747298706, -3.1103463372655571], [-1.9100288779694941, -2.974514800976614]]], [[[0.12976170432897227, 1.566580213904869], [-1.927871817915654, -0.0045344827355329187]], [[-3.661414219148277, -4.8195564825060195], [2.7723672128455021, -0.15582447947156552]]], [[[1.7438303361494842, 1.7312698627030034], [3.7526077755946901, -4.3423738169112136]], [[2.1998330827473236, -0.54406679067850572], [-1.6739342573642526, -2.2468350872158016]]], [[[0.35265689190881844, 2.0392675039040071], [4.2014194302091532, 2.5045311673441777]], [[2.2774763033579983, -0.40923945772010217], [2.5653192334575365, 2.7158300175631203]]], [[[-2.2320008926286894, 2.437050849977398], [0.0014259021542875061, 3.7781945368879768]], [[4.2320773376379801, -1.2014611538064699], [4.8721244612266048, 1.9565920212174195]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-1.0783569354649678, 0.57103421529402987], [1.2426427861224281, -0.79386350828194752]], [[-4.31766385585588, -7.2410643431756023], [2.7028972785683592, -0.79957707429225711]]], [[[1.0181040209210641, 0.46200114795946468], [1.4407318481232243, 2.8747823201994782]], [[0.88074761481960395, -1.2267012170460019], [0.6457257661391389, 1.005597805827263]]], [[[0.051460638106263314, 0.62127125926005111], [-0.4455155439120308, -0.0010478821898360496]], [[1.3693511257338256, 1.8024907043684877], [0.71327332864034265, -0.040090448567329157]]], [[[1.8059382761013583, 1.7929304511468724], [-1.1336166890426085, 1.3117777618344626]], [[0.50593107410486315, -0.12512780990137468], [0.94963230935395604, 1.2746421690239338]]], [[[-0.15025776267975899, -0.86887788009365896], [-16.047305901388825, -9.5660474869403114]], [[-0.47139065462791763, 0.084704133074762031], [11.519358189739302, 12.195214672206765]]], [[[0.5931937285888006, -0.64768938275648458], [-0.00031549889193566058, -0.83597334243537136]], [[-2.8421374995246782, 0.80686564233757319], [-5.1290398739627721, -2.059766447627867]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_constData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[[4.1279709395152029, -1.9657087231458403], [-3.7377710265335695, 1.22782196811161]], [[1.8215787962906349, -0.70044545137567837], [3.7467062615418492, 2.6507538824540964]]], [[[1.1133419896487542, -0.3708951569418506], [-0.0073124291445791911, 4.0564625456474097]], [[1.9759977509192419, -2.1752679389556007], [-0.72043138159888542, 1.6311023589572962]]], [[[0.80809580068305209, -4.0903068748335105], [-3.4462953575889732, -4.1389239925683476]], [[1.6336176556607809, -3.9101654453433268], [-1.4628901569065391, -0.31101603669531741]]], [[[0.78002740373697854, -3.8031338232085621], [-4.0594616774741121, 1.7198796521104986]], [[-0.024272491469055524, -3.492818224029528], [3.3271896452110443, 3.1399617164339944]]], [[[0.73703077160725172, 1.9972443248334883], [4.6495049184247765, -1.0939320291845211]], [[2.6197764433817214, 3.1864460032183217], [-4.9956513730717749, -4.1254024521779229]]], [[[-2.3890779287582342, 1.0866794929512658], [-2.8261197963574993, 2.1583365696655061]], [[-4.474631259817281, 3.8880890489916951], [4.9481336262902751, 4.2374325970284534]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[3.1647296869923487, -0.72079540558470789], [3.3930992747229389, -4.6723611169594834]], [[-3.7327328649729119, 2.5529611229955336], [4.739006026542425, 3.4937002806448909]], [[-2.0855356321199503, 2.0858449835390651], [-4.3504418913289591, 2.4785504340200433]], [[-3.3849171058379635, -3.5867226805012131], [-4.7386008527706025, 1.8937456791119178]], [[-3.8850783723534588, -0.64221067382870345], [-0.54932931066898671, -0.37744984906066925]], [[-1.7155848558854583, 0.98073005700899785], [0.91764938734486989, 3.3028504180010554]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.76665503061027385, -1.6099687861829519], [0.19284097406394038, -0.58705205176715569]], [[1.8627244023878977, -4.8442020260947869], [-1.2470582935521366, -1.7626536917994671]]], [[[-3.3527280024267689, 10.064118646763927], [-349.12627151923658, 0.62935651303741602]], [[2.3982851318215421, -2.1785849649481515], [-4.8494559924515874, 2.1419258340588052]]], [[[-2.5808024622292649, 0.50987265648737878], [-0.60524266411057737, -0.50395827207368582]], [[-2.6630722778086358, 1.1125979072087511], [-1.6942833488340934, -7.9692046119413487]]], [[[-4.3394848560722377, 0.89003365729114292], [0.88354638261616802, -2.0854498023160364]], [[195.22515267177013, 1.3566697574384114], [0.56917274969211962, 0.60311107272435638]]], [[[-5.2712566720670599, -1.945219382549684], [-0.13812452833070246, 0.5870663411395346]], [[-0.20968556765854743, -0.17239561257719796], [0.075555682507240138, 0.091494067169471491]]], [[[0.71809497515100484, -1.5787404354398697], [-0.34702352613397042, 0.4543916230641401]], [[-0.20507821406100435, 0.2360155273662895], [0.66749418416116524, 0.77944612507045319]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[2.4210516786926952, -4.2048187677730473, -1.9030895126345628, -1.0009182271013559], [-2.5669502930860357, -3.5375645148590862, -1.2448881660248157, 0.70912796340880124], [4.8769385990353058, 4.3109956988621718, 0.15322219623423194, 3.6939288405661124]], [[-3.8948081835020085, -0.73719661368693856, -1.7425966674325286, 2.1098795375443462], [3.1601876559926225, -1.2337624004438164, -1.7466033464968298, -3.2064382663710189], [-1.0536820496435917, 4.0532216694530039, -4.9473401005620108, 4.8458193781430126]]], [[[-1.7816450104627277, -2.6176290111561196, 2.3584252689768217, -0.67559218649427599], [-1.4176104731822505, 3.809260629113286, -0.32504285103155084, 1.829844243948977], [0.87988563738322423, 3.9034626698350685, 4.8069244732780305, -4.3815646945155411]], [[-4.763821121847486, 4.3684253740515882, 1.8660539508343534, 2.2051532740107129], [2.5852209611239809, -0.89532293541255115, -2.9949929278515364, 3.1035083735839848], [4.5424484325660792, -1.4805350406696705, 2.5202092381185377, 3.5916214642017668]]], [[[-2.9908806085512705, 3.75833381822242, -4.9773636072777911, 2.9703828096032403], [2.5163819004474943, -2.1450771048048987, 3.2720866474431602, 4.4977324635802773], [-2.9843640892837131, 0.99244939794968534, 1.2111864070838223, 4.9739835032563384]], [[1.177598383765873, 0.26044159403746381, 2.5213175859180437, -4.0233301801667309], [1.7914480869935643, -0.75840859871456079, 2.3233213048552521, -2.7570956285841905], [2.4588566540767296, -4.5825953524375205, -4.3004842902067626, -0.325896757332794]]]]),self.functionspace)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[3.9046484668741162, 1.2930040722230194, -3.0076147672729525, 2.4536477774283814], [-1.3424954099613604, 4.594582862516182, -2.3460658583836711, -2.4712136205260071], [0.9785316892547975, 1.6897558715370655, -4.1823746907171433, -4.8287859089465623]], [[-3.7776797806169027, 2.6538585545163684, 1.6603886428482797, 3.6211695516533933], [-2.3287567374966711, 4.1016336799850208, 4.5321236545835024, 3.4043910648597162], [0.38774697630695343, 3.5250345318024614, -2.0857503811747078, -2.4023228738046845]]], [[[2.1787187654523041, -2.8295097689253677, -2.8885774395954797, -1.6320335780927708], [0.49899243674356875, 2.80365722986744, 3.0485336523512139, -0.56124845039428806], [-2.8493615288907046, -3.0707731075809939, -1.0123386477202159, 4.350320282228969]], [[-2.1793638976413297, 2.2230102033759067, 2.3966660666277768, 1.2053137737977417], [4.4306663119524075, -4.4097239634956349, -3.0565958592358813, -2.9273206387788031], [3.7606676004526438, 1.3562339841310864, -1.7863039526980651, 2.9337081886706153]]], [[[3.8687016882494767, -1.5920702986448698, 1.7940823162608055, -3.2432807099415273], [-4.687153820064224, 4.6943751999071122, 2.9889920025997938, -1.0735736023607254], [-0.56178770750408002, -3.9206110274265238, 1.628072970905996, -2.8404808962120343]], [[-1.6247112837087685, -2.7251846858701501, 0.24658513397380588, 2.4762197305715432], [-4.2131249748556154, -1.1196018611052869, -3.1074157584086959, -4.2060905634829027], [-3.0540245760563112, 3.8452782027608006, 2.5025769263979551, 3.1605491398824483]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.6127902189111984, -0.30750530370844475, 1.5803853404190789, -2.4513968384151705], [0.5229923670814004, -1.2987983238799534, 1.8845595310582337, -3.4848627441608744], [0.20064466045325202, 0.39196417476896428, -27.29614111733208, -1.3072222333894574]], [[0.96992704200909063, -3.5999331864041477, -0.95282441076547508, 1.7162920855036172], [-0.73690457371437423, -3.3244923645829672, -2.5948213506367104, -1.0617360391948965], [-0.36799239053005506, 0.86968708333146183, 0.42159025633547398, -0.49575163379805731]]], [[[-1.2228691757660797, 1.08094376890928, -1.2247907438884671, 2.4157081901162551], [-0.35199545021943229, 0.73601087004647181, -9.3788669483929148, -0.30671924796345551], [-3.2383316738351278, -0.78667925565450381, -0.21060007357050534, -0.99286911994573968]], [[0.45748231134172762, 0.50888135037869009, 1.2843498257679931, 0.5465895672664639], [1.7138443400312209, 4.9252887299974075, 1.0205686400163008, -0.94322949591345329], [0.82789439578254087, -0.91604315120947044, -0.70879192317842243, 0.81681998448648552]]], [[[-1.2934992046116502, -0.42361066782457119, -0.36044831316674114, -1.0918729732262147], [-1.8626559900270687, -2.1884412403600195, 0.91348192289938923, -0.23869218790886934], [0.18824368967625413, -3.9504392219151598, 1.3441968646477076, -0.57106761499157466]], [[-1.3796819918460328, -10.463707596099798, 0.097800108701507002, -0.61546520411827754], [-2.3517985284888421, 1.4762515390818598, -1.337488599581492, 1.5255512068120729], [-1.2420506787139487, -0.83910489734064453, -0.58192909391552128, -9.6980073252309467]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.62374931794)+(1.-msk_arg1)*(-0.399933415651)
      res=quotient(arg0,arg1)
      s0=numpy.array(3.68791942738)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.27123693703)+(1.-msk_ref)*(-9.22133355968)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.996552233537)+(1.-msk_arg1)*(0.0873556404986)
      res=quotient(arg0,arg1)
      s0=numpy.array([2.1807332936376591, -1.5921808487091518])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-2.188277965017587, 1.5976893083249937])+(1.-msk_ref)*numpy.array([24.963852147270945, -18.226422926111301])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.6781059319348959, 3.2791183368745305])+(1.-msk_arg1)*numpy.array([-4.2542801139362147, -0.933793751286216])
      res=quotient(arg0,arg1)
      s0=numpy.array(3.65497932132)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.99371235873994845, 1.1146225740673148])+(1.-msk_ref)*numpy.array([-0.85912991703235586, -3.9141184188521501])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.58292347204)+(1.-msk_arg1)*(-3.63383717699)
      res=quotient(arg0,arg1)
      s0=numpy.array([[3.7169817252496333, 2.6259016972472491, -0.40044244817140218, 0.18887777297220776, -0.84886391974168163], [-0.27340895294269352, -1.2402472205948047, -1.9491901334671233, 1.6124520174841992, -4.5102702074782481], [3.1954209095565336, 3.5806980280244574, -0.10185845096161827, 4.5074386499329222, -2.6461425783792203], [-0.8327798666143762, 0.92002688206027194, -1.3462010811902725, 4.2137295033166602, -0.33026695758887747]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-2.3481752535070028, -1.6588936506552001, 0.25297650533557459, -0.11932211272872624, 0.5362633979047885], [0.17272405000704594, 0.78351685504613811, 1.2313862090565204, -1.0186544365292911, 2.8493292866859683], [-2.0186831302904942, -2.2620790526280201, 0.064348310427209621, -2.8475404714998058, 1.6716806751004385], [0.52610241829246773, -0.58122006420950179, 0.85045240971236358, -2.6619919267953929, 0.20864366687440855]])+(1.-msk_ref)*numpy.array([[-1.0228806477042267, -0.72262502950764518, 0.11019823637317477, -0.051977500304166396, 0.23359987759418044], [0.075239736847340685, 0.34130511637911798, 0.53639996470136964, -0.44373259971447782, 1.2411866541626824], [-0.87935170287534858, -0.98537657402470047, 0.028030548976339654, -1.2404074344543099, 0.72819514180110412], [0.22917368777225916, -0.2531832983289441, 0.37046268603205368, -1.159581263024231, 0.090886559166800052]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-1.9251141038583039, 1.0010948216567703, 4.8534309664026321, 0.27098298973139556, 3.2564704543350196], [-4.8713313028269845, -2.9801900066040146, -3.4492150477424954, 3.622586087008294, 0.076514770429919388], [-3.752809005922134, 3.207144711338632, -1.0619140502310942, -1.7582773387578179, 1.8951955639045659], [1.9289600467550425, 2.8723737763732728, 4.7900700844197388, -1.8170286869787611, 3.4312664497502769]])+(1.-msk_arg1)*numpy.array([[-1.0396122054306201, -0.79726850481593647, 1.5169447137725323, 3.4466270357444841, -1.9008286437629298], [-1.4329919383046796, -1.7943661991567095, 3.1167471482521716, 2.3411069889277627, 2.9650448836817134], [-0.018553407313651959, 1.1306137779796952, 4.2140710934925067, 2.2939366534192906, 1.9725810831914767], [4.1673182073398749, 3.5425700964067079, 3.3560526198693594, 4.6040633382800653, -0.30340085519444315]])
      res=quotient(arg0,arg1)
      s0=numpy.array(1.89396762653)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.98382097078478137, 1.8918963374467499, 0.39023273219298754, 6.9892491348135701, 0.58160135431540538], [-0.38879877158636755, -0.63551908513630795, -0.54910105641831441, 0.52282197884039261, 24.752967510554765], [-0.50467999398332519, 0.59054635727329774, -1.7835413573416652, -1.0771722894793123, 0.99935207880463528], [0.98185943753238603, 0.65937366581890999, 0.39539455439069698, -1.0423432717941945, 0.55197334694517575]])+(1.-msk_ref)*numpy.array([[-1.8218020302530467, -2.3755706077547951, 1.2485409714235332, 0.5495133668039156, -0.99639051249775124], [-1.3216875656467648, -1.0555078597777037, 0.60767445559113187, 0.80900515674287765, 0.63876524667565926], [-102.08192999329738, 1.6751676508964009, 0.44943893553530029, 0.82564077072758102, 0.96014690735301211], [0.45448116325590787, 0.53463095294866181, 0.56434384112936742, 0.41136871658174851, -6.2424597495466765]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.10603989922)+(1.-msk_arg1)*(0.482799377247)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[0.50323036439747426, -4.6669622867631091], [-1.2033974383793189, -4.7262497807545003]], [[1.4556118351268443, -0.25133892570718785], [4.1966197594696837, -4.7539392769106872]], [[-3.6766474133335345, -2.072255960853131], [-3.4841937567746939, 2.7321617467868711]], [[2.5081336298318728, -4.5482182269382108], [3.5000419772481308, -0.38658746752230755]], [[0.48543170811448633, -3.1427569374012752], [0.1603939953613942, 1.5918963688400254]], [[-4.9521604456535693, -4.1097817924779321], [-3.8192190327100306, -2.9538921514833874]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.45498391581738018, -4.2195243499383954], [-1.0880235326300096, -4.2731277452889529]], [[1.316057256300686, -0.22724218708962288], [3.7942751996816755, -4.2981625529624452]], [[-3.3241544142536434, -1.8735815609492212], [-3.1501519603688197, 2.4702198796925048]], [[2.2676701189533639, -4.1121646969018135], [3.1644807567211126, -0.34952397991687362]], [[0.43889167873304319, -2.8414498786318587], [0.14501646412075711, 1.4392757168727368]], [[-4.4773795675408818, -3.7157626911790667], [-3.4530571956786287, -2.6706922178539414]]])+(1.-msk_ref)*numpy.array([[[1.0423177578793705, -9.6664629382426011], [-2.4925414055869419, -9.7892623799682266]], [[3.0149414098822676, -0.52058668165740074], [8.6922642348897536, -9.8466143515347326]], [[-7.6152695852667653, -4.2921678413716897], [-7.2166492356376715, 5.6590001469493378]], [[5.1949810791690094, -9.420513864111566], [7.2494749210472333, -0.80072072529758898]], [[1.0054522250684441, -6.5094469577050331], [0.33221665751940793, 3.2972212555820382]], [[-10.257180682156864, -8.5124007738245275], [-7.9105715804547287, -6.1182600696959994]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.5337930195921867, -0.54829047010045429], [-3.9135910661816666, -1.1141501823762101]], [[0.27340994586052414, 4.2186998017531749], [0.12372495760515356, 2.6414810026451381]], [[1.5405139095973297, -4.4326139059922571], [3.2261015993294926, -1.6672921754445813]], [[-0.11743595241858884, 4.3454444829454353], [-0.84962258675786106, -0.60417896950434535]], [[0.84348170837809011, -1.9823211401316732], [-4.8696894265795905, 1.8667386460897406]], [[-4.5137432877200387, -2.8720480270056656], [2.1211719392595496, 4.1938267593977976]]])+(1.-msk_arg1)*numpy.array([[[-4.1404134151519658, 2.1496156276052343], [-3.7919294493309783, -0.73104392265455509]], [[-1.0703917764981838, 0.11005113748932693], [4.7944869933959371, -1.0821278772261578]], [[1.7620133471125943, 3.0626882159861584], [4.4053208452283972, -4.2857836054639398]], [[-0.085561937786897246, -1.8391851771276424], [-0.25824647286356051, 2.3083825959149671]], [[-0.66389122950745705, 2.6039435290223807], [1.083968608617905, 2.7279206502755429]], [[-0.53856321883635694, -3.3896546602091639], [-4.2337921184005367, -4.8940034707325628]]])
      res=quotient(arg0,arg1)
      s0=numpy.array(-4.31228402269)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-1.2202989815150873, 7.8649625661073426], [1.101873943845848, 3.870469251727287]], [[-15.77222806990831, -1.0221831903994221], [-34.853792687934202, -1.6325250942085958]], [[-2.7992502994151121, 0.97285351581464874], [-1.3366857459138075, 2.5863997241776806]], [[36.720305271785598, -0.99236891407033045], [5.0755289347344688, 7.1374282130858839]], [[-5.112480780389955, 2.1753710513358255], [0.88553573851266432, -2.3100630780459901]], [[0.95536758468859917, 1.5014665430888923], [-2.0329724068473136, -1.0282456262720501]]])+(1.-msk_ref)*numpy.array([[[1.0415104943174367, -2.0060721402074764], [1.1372268604454916, 5.8988029160201112]], [[4.028696891526228, -39.1843657509827], [-0.89942553366675793, 3.9850040955854893]], [[-2.4473617238825098, -1.4080062084625069], [-0.9788808066872815, 1.0061833306739392]], [[50.399560064122852, 2.3446709316286487], [16.698326892432647, -1.8680976153280269]], [[6.4954676775783993, -1.6560589638871568], [-3.9782369972796277, -1.5807952559976015]], [[8.0070154660965862, 1.2721897818426653], [1.0185393855196145, 0.88113628208113681]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-1.88183350501)+(1.-msk_arg1)*(-2.60219222179)
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[-4.1025438277408863, 1.9950416703182086, 3.4144371460377947, 4.1582163123685891], [2.6418772130004999, 0.066407835609095578, -4.3050445895196212, 2.5016990258829788], [-0.89178406520666353, 3.8039263215216899, 1.8736926673923051, -1.6477437197815181]], [[-2.6647927534258842, -1.2463013957625235, 3.6036615811849213, 3.961354656712091], [4.3659407243385449, -4.0146483039293814, 0.97447781262190958, 2.6827974765327713], [-3.751334194260656, -1.8488451494685023, 4.8352507593557483, -1.8955042570215488]]], [[[-2.8936307038765086, 0.30967273479322976, -3.576890388215801, 4.9216272272266934], [-4.2728168411138157, -2.8923292013966826, 2.3288824486958291, -1.6682482959661495], [-4.0769809197091442, -3.7003924744202701, 4.6787879375401982, -2.2398389036558397]], [[3.9387104018759906, -1.5506809663295398, -2.1870301174230198, 1.70513404433256], [-2.4697787309158103, -0.050147891856210158, -0.099030765909376761, 1.2038802429483813], [-0.29925478459842392, -3.4679847229739025, 4.363305623381228, -0.3464994375285313]]], [[[2.3249529619382177, 2.8069801532026339, 1.1315412312065343, 0.81144926593488975], [-3.5054057716753286, -3.7661894814195485, -3.2017785186511216, 2.0957805380864203], [2.8487655566398207, -4.6475742769545558, -1.2947941950598474, -0.55671434898044492]], [[-4.1535917161713298, 4.0823942365227754, 0.45509563675886433, 3.9604426729704194], [4.0845260048911882, 0.24169759084649911, -2.9893454630399696, -2.3729477092951679], [1.4481101795229154, 1.0510382102164044, -2.4502499256054824, 0.67679876300592312]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.1800780020224368, -1.0601584385690386, -1.8144204239895345, -2.2096621732466653], [-1.403884672032442, -0.035288900655823865, 2.2876862262567421, -1.3293944545162102], [0.47389105509733537, -2.0213936628265397, -0.99567398625014747, 0.87560547487055185]], [[1.4160619131987713, 0.66228037307474741, -1.9149736528715604, -2.1050505510535205], [-2.3200462276366745, 2.1333706160727739, -0.5178342345524225, -1.4256295625465771], [1.993446383154337, 0.98247009873424751, -2.5694360029636534, 1.0072645916736218]]], [[[1.5376656309779482, -0.16455905050516081, 1.9007475309017641, -2.6153361676918236], [2.2705605090665615, 1.5369740169343247, -1.237560306209748, 0.88650153774223683], [2.1664939586074974, 1.9663761244386926, -2.486292185299122, 1.190242865637597]], [[-2.0930174701375863, 0.82402665389963536, 1.1621804541159455, -0.9061025004566382], [1.3124321170500164, 0.026648421192790935, 0.052624616176581041, -0.63973791504049171], [0.15902298678482071, 1.8428754264079477, -2.318645943844138, 0.18412863656963446]]], [[[-1.2354721901531733, -1.4916198195694526, -0.60129720732152381, -0.43120141275761592], [1.8627608459213116, 2.0013404328241635, 1.7014143441102452, -1.1136907343327285], [-1.5138244425214531, 2.4697053509683782, 0.68804928364426587, 0.29583613401415809]], [[2.207204678370021, -2.1693705769689093, -0.24183629186497424, -2.1045659259582377], [-2.1705033915153278, -0.12843728746620767, 1.5885281323143545, 1.2609764375954557], [-0.76952088251574025, -0.55851817252603808, 1.3020545755415156, -0.35964858804145833]]]])+(1.-msk_ref)*numpy.array([[[[1.5765721661098684, -0.7666772860254949, -1.3121387103715778, -1.5979666212010522], [-1.0152505994287127, -0.025519957769848459, 1.6543914601970204, -0.96138133260678538], [0.34270491539376025, -1.4618160371357209, -0.72004391209186047, 0.63321368267268008]], [[1.0240568437305366, 0.47894286414608772, -1.3848560267798948, -1.5223143869010356], [-1.6777933189486567, 1.5427946753174853, -0.37448340843620703, -1.0309759033434756], [1.4416053367805921, 0.71049522552069111, -1.8581451127506026, 0.72842591763572628]]], [[[1.1119972919935237, -0.11900455784940972, 1.3745680885004119, -1.8913388434626044], [1.642006614782471, 1.1114971358294525, -0.89496941432493493, 0.6410934142362964], [1.56674856129786, 1.4220288737453772, -1.7980178014376997, 0.8607507488880074]], [[-1.5136123953085283, 0.59591330469167525, 0.84045678836094673, -0.65526828881225874], [0.94911463889373704, 0.019271401795876723, 0.038056668173911935, -0.46264078144100224], [0.11500102955216439, 1.3327165817861977, -1.6767806724057719, 0.13315674169921637]]], [[[-0.89345934649646785, -1.0786982336271564, -0.43484152390110348, -0.31183294575251325], [1.3470971676584023, 1.44731409535519, 1.2304158362479929, -0.80539036299370426], [-1.0947560033365946, 1.7860226612154386, 0.49757822816388358, 0.21394051689140911]], [[1.5961894288174401, -1.5688288522034675, -0.17488932329753257, -1.5219639194244077], [-1.5696480723788881, -0.092882297019659985, 1.1487796474098502, 0.91190331345442965], [-0.55649623705626117, -0.40390490810614521, 0.94160988765099507, -0.26008792023077443]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.0141571662016409, -4.1813641260779368, -2.4591123275348616, -2.8592337851391916], [4.0207593340027632, -3.2261822122874939, -0.11810710493155341, 3.1027234301297568], [-0.18600667292264461, 3.667116414745843, -0.50499642362666108, -2.5467195320055058]], [[3.8025114906874098, 0.028954755959841094, -0.68774322008968181, 3.1357764849746168], [1.1266999885181326, -2.8475680018298744, -1.1677090288276757, 3.5539799954073956], [1.7257329577351204, 4.6847153303148215, -3.4655344223219364, 3.2671521648027699]]], [[[1.6191416798971421, -2.0798874915349153, 2.8244281036680716, 3.1188587480389671], [4.4311521728556169, 0.64258912149269776, 3.6323053052190719, -4.398955163929771], [3.656507392198268, 3.8283082285915846, 1.1584482142949293, 0.95562188355199584]], [[-3.7656980963841624, -4.2887002549248976, -1.90608927410296, -4.5347898560095103], [-2.8374564042579067, -3.6057893378700023, -3.5921621035239202, -3.1756484925419115], [4.2040292051635682, -0.20162572081404928, -4.6281909220253636, 3.0974098527194833]]], [[[1.4905304995486439, -4.1382319801988929, 3.7930804712525195, 2.8436411717839025], [-1.8548025770940333, -3.1150277777348814, 1.4590265101302116, 2.7294241322468906], [-2.0949167856384743, -2.7384130691345518, 0.15602635562511846, 4.7263500136314835]], [[1.6251790524982512, -3.96646319936682, 4.0336387546841621, -3.2011313333751845], [2.0085898771332893, -2.5741267104814902, 3.5029406910659819, 4.5487100222330952], [-0.29992977950664557, -1.7001843471596043, 0.85816537088468436, -0.14400909578909982]]]])+(1.-msk_arg1)*numpy.array([[[[-0.26546940855063639, 1.0068682028941929, -0.30579147316068944, 4.9107435758052347], [-4.8793567397208948, -3.5958879909177446, -1.3779695462405748, -1.1487322062533911], [-3.9943399892822917, -0.20354206045623968, 2.17797707889188, 2.4935487966754302]], [[-4.8450634637049115, 3.2248397497898402, 1.8009001058260257, 1.2664316310154025], [-4.0578341209855449, 0.46087296247377019, 1.7743402842695719, -2.4154646540260858], [3.4842261029379671, -0.98540088748727417, -0.49567727792924199, 4.4104695375844809]]], [[[2.9578986546291546, 4.163050552347654, 1.491341618907752, 1.9562509025134487], [4.0956618681531936, -4.1461819614979909, 3.5350235620036159, -4.9004901580969999], [0.49563106807316881, 2.2606186783896138, -0.38420364469005719, -4.0038750195810744]], [[0.12466656669298803, 4.8988574198956378, 4.5409461236193653, -1.2557434301404502], [-2.9399406098579384, -2.6163600284860102, 2.0135366292342605, 4.146487706454117], [-1.3842466586020041, -1.0534354187933639, -0.74422141627651506, 2.6281359119485836]]], [[[4.9603467482270673, -0.26732583547349531, -0.20397570368971607, -4.5287120551391169], [-3.2934782127498652, 4.2825979621795511, 3.8908813360313097, -0.56969620675744181], [-2.580483974601461, 4.256987598437064, -1.0291925285429149, 2.9519935203260488]], [[0.3275900728273653, -2.6644199415598093, -4.4310387042545045, -2.8203685612262808], [0.57003496909474283, -0.79523212078311367, 2.0066387865963815, 1.7627640119152623], [-3.0724699828301372, 0.28149944968731511, -0.67528639584407735, 2.7422732963264238]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array(-3.5555118358)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.1796039953276285, 0.85032341804966904, 1.4458517392579147, 1.2435191044106144], [-0.88428864809902974, 1.1020802923824493, 30.104131651160053, -1.1459325704863632], [19.114969263901109, -0.96956612053541269, 7.0406673581225467, 1.3961144095820175]], [[-0.93504302209330092, -122.79543439178657, 5.1698246263092482, -1.1338537210268158], [-3.1556864045708988, 1.2486134952745827, 3.0448611323716692, -1.000431021106321], [-2.0602908577832166, -0.75896006162618246, 1.0259635030301146, -1.0882602512673285]]], [[[-2.1959238527061613, 1.7094731567297239, -1.2588431021413971, -1.1400041242754082], [-0.80238991961892847, -5.5331030620898281, -0.97885820079279329, 0.80826280407472273], [-0.97237922816267253, -0.92874231213741987, -3.0692022240811716, -3.7206262194218591]], [[0.94418398522464519, 0.82904181324258142, 1.8653438136942178, 0.78405217191819565], [1.2530630710172086, 0.98605645051276247, 0.98979715651164302, 1.1196175660332262], [-0.84573909035404105, 17.63421760597674, 0.76822929211418844, -1.1478984070109959]]], [[[-2.3854002564010086, 0.85918620628560616, -0.93736788943546645, -1.2503377258271928], [1.9169219838843279, 1.1414061412904251, -2.4369069452203065, -1.3026600716942895], [1.6972091016556698, 1.2983840443474595, -22.787892606679982, -0.7522743397214009]], [[-2.1877662220241327, 0.89639350148627039, -0.8814651117847212, 1.1107047682570477], [-1.770153218571118, 1.3812497346456578, -1.0150077176199288, -0.78165278033075258], [11.854480877641825, 2.0912507762683723, -4.1431546371202028, 24.689494898321406]]]])+(1.-msk_ref)*numpy.array([[[[13.393301530329323, -3.5312584363839434, 11.627243229010782, -0.72402718262760168], [0.72868454295478258, 0.98877157597153598, 2.5802542919015385, 3.0951616194284468], [0.89013750590512886, 17.468192214558385, -1.6324835877546728, -1.4258842018801918]], [[0.73384215963974975, -1.102539075322601, -1.9742970886028341, -2.8075039731486431], [0.8762092608490647, -7.7147329639655622, -2.0038500322166368, 1.4719784161902543], [-1.0204595599575992, 3.6081881810186371, 7.1730377689501603, -0.80615267954988201]]], [[[-1.2020397758498016, -0.8540640549732964, -2.3841028713467343, -1.8175131989606406], [-0.86811654630075741, 0.85753878358784652, -1.0057957955396568, 0.72554208274903775], [-7.1737065426898683, -1.5728047679096775, 9.2542376547862855, 0.88801768746739551]], [[-28.52017128660399, -0.72578389837534873, -0.7829892139224357, 2.8313999105686642], [1.2093821976793828, 1.3589535832552451, -1.7658043981792397, -0.85747555220352178], [2.5685536704763918, 3.3751588112249413, 4.777491964133171, -1.3528645225812119]]], [[[-0.71678695386923774, 13.300292616683429, 17.43105561829773, 0.78510441655527985], [1.079561365255916, -0.83022311858275499, -0.91380628930294561, 6.2410663676942475], [1.3778468964706454, -0.83521780451092942, 3.4546615304627157, -1.2044443225621142]], [[-10.853539623804645, 1.334441234408186, 0.80241046695958662, 1.2606550380248678], [-6.2373574053593082, 4.4710364972374714, -1.7718743699894504, -2.0170095439682569], [1.1572161341416192, -12.630617359097013, 5.2651909733093767, -1.2965563427102094]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-2.474103218291325, -2.6780794095651972])+(1.-msk_arg1)*numpy.array([4.8154186815540143, -3.1767897538303478])
      res=quotient(arg0,arg1)
      s0=numpy.array([4.0237071297221689, 1.4571869711126268])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-1.6263295322420046, -0.54411641637960628])+(1.-msk_ref)*numpy.array([0.83558822104823771, -0.45869795738155289])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.9758679142790463, 3.6756653375311181])+(1.-msk_arg1)*numpy.array([2.1616757182582607, -2.8947346041118505])
      res=quotient(arg0,arg1)
      s0=numpy.array([[1.2257378731576107, 0.42501824134675203], [-4.232594044906218, 0.48724566225693877]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.62035415641882996, 0.21510458177657715], [-1.1515177950746678, 0.13255985447908417]])+(1.-msk_ref)*numpy.array([[0.56703133721890142, 0.19661517116415797], [1.4621699823168568, -0.16832135891311989]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_expandedData_rank2(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-4.7092871546777673, 4.1852783450755968], [0.26913846140799969, 0.45677937540863844]])+(1.-msk_arg1)*numpy.array([[3.4478716653613191, 3.9275647704673773], [3.2357632920769106, 0.037556772687022288]])
      res=quotient(arg0,arg1)
      s0=numpy.array([-4.9370874602703481, -1.2043823773166427])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0483725664013299, -1.1796318077815138], [-4.4749545308979926, -2.6366829199308586]])+(1.-msk_ref)*numpy.array([[-1.4319232092859717, -1.2570352747315325], [-0.372209666963493, -32.06831394575115]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.9566466811493868, -2.8415703141266757])+(1.-msk_arg1)*numpy.array([-3.5166570146980214, -2.2814389442004823])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[1.1645858700410514, 1.6511398870703475, 1.3202878187307077, 4.4483109974532269, -0.051384686889477393], [2.7226944149582746, 1.9821035559131168, -0.47454464600014923, -2.9263392082769757, 3.2864081269017777], [-0.082182457078530113, -4.6456657561222698, -3.204753226424506, -3.0665876822208937, -2.264675720249294], [3.2757637419557923, -1.8816261786861324, 1.469136092269661, -3.7136148677144831, 0.083810593936415323]], [[-2.6154953226607445, -1.0609854423462153, -0.014479386967733099, 3.4374639616404519, 2.8855012262780626], [-3.1977967070316318, 3.617833815184623, -4.7508035793552228, 3.1284501207601796, 4.1676565754434378], [4.2524453940280118, 0.77883150023151604, -4.5450330798992669, 0.57878354127524734, -0.54557072408306162], [2.0545346432643443, 3.2389363593019223, 1.6812879161322645, 0.58429198637870972, -1.1294666204599957]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.23495438447732933, 0.33311631699507538, 0.2663671436884722, 0.89744363147177497, -0.010366824628614017], [0.54930169328246625, 0.39988800562510357, -0.095739050314981908, -0.59038688785426863, 0.66303054026431019], [-0.0165802532165704, -0.93725981595383667, -0.64655672122293817, -0.61868191934749506, -0.45689674207808229], [0.66088304304881018, -0.37961676506863828, 0.2963971787331402, -0.74921920132772923, 0.01690872868852146]], [[0.92044012061147507, 0.37337997130375322, 0.0050955582185490183, -1.2097057547903465, -1.0154600827341795], [1.1253625121060706, -1.2731811692988224, 1.6718937257110698, -1.1009581938575654, -1.4666737454020453], [-1.4965124645648449, -0.27408489466532204, 1.5994793643866354, -0.20368439886842282, 0.19199620764997208], [-0.72302790926916838, -1.1398402999917925, -0.59167563363604081, -0.20562292035285609, 0.39747973676559345]]])+(1.-msk_ref)*numpy.array([[[-0.33116276770057868, -0.46951973996023394, -0.37543832486719836, -1.2649260302785621, 0.014611799409129992], [-0.77422802496196086, -0.56363288988059645, 0.13494197586422821, 0.83213665593380681, -0.9345262029154654], [0.023369483215179912, 1.3210460208958414, 0.91130673620717062, 0.87201784803122873, 0.64398538463773491], [-0.93149935528673822, 0.53506104542518462, -0.41776496431962018, 1.0560071261408968, -0.023832461791447188]], [[1.1464235452408, 0.46505099119276749, 0.0063466028773377144, -1.5067087244997881, -1.2647724952767783], [1.4016578068681482, -1.5857684135625523, 2.0823715626631052, -1.3712618208402363, -1.8267666491964658], [-1.8639312723392902, -0.34137731461599696, 1.9921782660250189, -0.25369232113203838, 0.23913448373007146], [-0.90054333844307399, -1.4196901335165861, -0.73694188503539482, -0.25610678202193643, 0.49506765163764266]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_expandedData_rank3(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-2.3266123634488469, 2.0448357476497687, -3.3451105656467881, -2.5817939023624992, 1.4392205799828126], [4.5160774763504854, -0.59256146426819001, 2.7557800690832446, -1.9828393432455838, -2.0278978887515442], [-1.3167380604859149, 2.906871797385028, -3.2321794842555338, -0.78111135285510436, 4.9156646277821174], [-0.56741518109179268, -2.6686468475514777, -1.8764925106581956, -1.9624233929765325, -3.4498783750216466]], [[-4.9302512054621292, 4.1284983559710522, -2.9533613529570868, -0.066832751370423438, 4.0577869606662169], [-2.1222929957634995, 3.3275825814034459, 4.6798455674508599, -3.499576200351866, -1.2980902345786216], [-0.34495427220489283, -0.37907271548423527, -2.0576000050069343, -2.4740013729018107, 4.6041072396225573], [3.8633721814726165, 0.72562061375566955, 4.5672995744614173, 4.6897571836676537, -1.7019525688829731]]])+(1.-msk_arg1)*numpy.array([[[-3.420778156572839, 0.55883146220079372, 1.4654652606365612, 2.2845095373357314, -2.3486264149223999], [3.500758633801432, 2.036509186438189, 1.4365962658181521, -3.7482420466445743, 4.7284636469766586], [0.75692091336523504, -4.0071259452811692, -2.773641985074268, -1.5401382907461345, 4.0856184108171369], [4.0924067210425292, -1.5508898547708294, 4.5992893128709653, 1.0632608033066706, -0.28254375048600266]], [[-1.7587986016209856, 3.4757378952629399, -2.9980347018980922, -3.1756270469320902, -2.9634886212479215], [1.5152006612641244, 3.2203197283738039, 3.9582373318237263, 1.5290081334829964, 3.5284520021271817], [3.5892825150501828, 3.0552565721288349, 1.1100733008428962, -1.1388018264427968, -4.7237659440015625], [-3.7400514288575151, -2.8564363732806641, -2.9533861626969427, -1.944569309376285, -1.3152939571508115]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([0.88151483244535811, -0.36359201179814526])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.37888341276526494, 0.43109322274834394, -0.26352337692458733, -0.34143501215907213, 0.61249459930310701], [0.19519479837571882, -1.4876344237707455, 0.31987851365025965, -0.44457198988318564, -0.434693895257248], [-0.66946863533363143, 0.30325205027561025, -0.27273078018697866, -1.1285392655263051, 0.17932770015742222], [-1.5535622976269161, -0.33032277510010766, -0.46976730652453247, -0.44919706705509072, -0.25552055366004806]], [[0.073747157425838414, -0.0880688280455003, 0.12311125133201009, 5.4403268508716733, -0.089603524118587505], [0.17132036553102897, -0.10926611223117901, -0.077693164562307557, 0.10389601225473753, 0.28009764045114505], [1.054029594920286, 0.95916165143588705, 0.17670684822773411, 0.14696516169337456, -0.078971230007221199], [-0.094112602855558558, -0.50107729150122204, -0.07960765565526115, -0.077528963133608519, 0.21363228238303861]]])+(1.-msk_ref)*numpy.array([[[-0.25769424151390097, 1.5774252025356099, 0.60152557424831088, 0.3858661205124182, -0.375332077866664], [0.25180680094135244, 0.4328558094977753, 0.61361347890134532, -0.23518087185284367, 0.18642732571476817], [1.1646062579063701, -0.21998680462824949, -0.31781853504851471, -0.57236083132398452, 0.21576044158001831], [0.21540254733546979, -0.56839293244046463, 0.19166327066628899, 0.8290673649436765, -3.1199233072013346]], [[0.20672748514983069, -0.10460858176149658, 0.12127678561157106, 0.11449455695668175, 0.12269053749396112], [-0.23996294424449513, -0.11290556294599724, -0.091857051843483861, -0.23779599587211003, -0.10304575819054594], [-0.10129935725972292, -0.11900539388899913, -0.32753874138046929, 0.3192759296267329, 0.07697079324174598], [0.097215778636822875, 0.12728867871842489, 0.1231102171434717, 0.18697817045912668, 0.27643403196784838]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.5925008652336654, -0.5860601221733388])+(1.-msk_arg1)*numpy.array([-0.91584734698007253, -4.360990883154809])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[3.8723284105152587, -3.4010300952138186], [-4.7312413414210797, 0.35678174781570604]], [[-2.1285240756231252, 0.81050078698193673], [3.2817482285783122, 1.4737347171058879]], [[-2.4680510274823009, 3.5118700252773536], [-0.52301203719946709, 1.6535167042297516]], [[-4.0692927537337713, -1.7677439110822482], [-0.62199113317150889, 2.8395701279277175]], [[2.4200134032709535, -2.1734817677195659], [1.6734563720278057, 2.9761820301322537]], [[-2.1900935703739668, -2.3745777706417837], [1.2564443397018277, 4.367856767201344]]], [[[-1.5513137235972376, -4.6108851095432897], [-1.8641272104945719, 4.6811274370407787]], [[3.6870866758780583, -2.2326047151766151], [1.6817089083468497, -1.6986688518556923]], [[1.5062342189334563, -3.0443758373123297], [1.4895534206459491, -0.42104223353101133]], [[-4.0337869097404102, 0.98722842599825888], [2.6060171250569386, 1.2953201228811864]], [[-3.2090937326077009, 0.25909746950079438], [1.4805185513956172, 0.51631073864388455]], [[2.113979529798435, 1.6043860517651662], [4.7864067473982477, -3.7863239834337858]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.4936652336145857, -1.3118723086355766], [-1.8249719430641913, 0.13762068611057102]], [[-0.82103119199201446, 0.31263279324263032], [1.2658619607760571, 0.56846064619270953]], [[-0.9519962213242511, 1.3546263657508266], [-0.20174035203352858, 0.6378075804733504]], [[-1.5696398825953721, -0.68186820486284361], [-0.23991935413122728, 1.0953015160023036]], [[0.93346676783185356, -0.8383726296360049], [0.64549886731743678, 1.1479965426603653]], [[-0.84478026593698774, -0.91594097517408546], [0.48464567805981534, 1.6848043623729567]]], [[[2.6470214657232831, 7.8675974274522122], [3.1807781146781724, -7.9874525836723684]], [[-6.2913113115509498, 3.809514810352987], [-2.8695160184426456, 2.8984549325014091]], [[-2.5701018751246103, 5.1946476515457158], [-2.5416392692307843, 0.71842839599736463]], [[6.8828892414340697, -1.6845173193788241], [-4.4466719820362695, -2.2102171328047979]], [[5.4757073740269746, -0.4421004939560812], [-2.5262229852890838, -0.88098595879413122]], [[-3.607103520299209, -2.7375792876257798], [-8.1670916793457824, 6.4606408799708541]]]])+(1.-msk_ref)*numpy.array([[[[-4.2281373891445195, 3.713533818084882], [5.1659715530343995, -0.38956464632688298]], [[2.3241035557309297, -0.8849736690885146], [-3.5832917345882951, -1.6091488630342172]], [[2.694827948806628, -3.8345582774874454], [0.57106900939774963, -1.8054501218812065]], [[4.4431998052425579, 1.9301730980727638], [0.67914280171523222, -3.1004840897240729]], [[-2.6423763864695777, 2.3731921863249745], [-1.8272219464803643, -3.2496485794777445]], [[2.3913303648207433, 2.5927658997667553], [-1.3718927546657687, -4.7691973794584595]]], [[[0.35572505542020144, 1.0573021666598172], [0.42745496618558237, -1.0734091316545882]], [[-0.84546993439499285, 0.51194895265672147], [-0.38562541252786919, 0.3895144239849565]], [[-0.34538806874180461, 0.69809268555727422], [-0.34156306687079863, 0.096547377605711207]], [[0.92497026887208389, -0.22637709008097776], [-0.59757454094279261, -0.29702426755456357]], [[0.73586343530399501, -0.059412522622234704], [-0.33949131999207183, -0.11839298739152082]], [[-0.48474752331267179, -0.36789484196411076], [-1.0975502759904159, 0.86822561314200686]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_expandedData_rank4(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-3.5642046714767126, 2.9338144581257577], [4.6152503988289517, 2.2423100072713087]], [[-4.3523817626222971, -3.3531233672057938], [0.57235479944568368, -2.7744711811519229]], [[3.8074695556661808, -2.3313000942894013], [2.2228652190902896, 3.883001128454687]], [[3.472726239942892, 4.3203249234202588], [3.1146413749584365, -0.63166829548000702]], [[-0.10596091083488712, -0.62971253410553452], [0.66146133628545378, 1.9313061067486581]], [[2.3165333698933832, 3.2977092843334894], [-4.2067458173257952, -2.776648739807408]]], [[[2.7201198656379262, -2.5811899519891766], [3.2863829387051702, 2.866130232793962]], [[-4.3137635582535658, -0.94867462369344757], [1.0775756951976447, 3.5446060334039]], [[-0.74508855420794973, -4.779278729224635], [0.50027545188155109, 1.7067292781085435]], [[-4.4828101797401354, 2.3091034479314967], [3.7970766460608534, 4.5203535622108397]], [[3.8157371047545858, -4.7209716201505518], [-3.1894387021023385, 4.4453843905482522]], [[4.3583641400659072, -3.63239831935253], [-2.6260910017233674, 3.2799281905932389]]]])+(1.-msk_arg1)*numpy.array([[[[-3.6100115878847139, 2.038086098052255], [4.6800877361426778, -1.1686177858692668]], [[4.2073151962478779, -4.6964772162899502], [-0.21246710513302958, -4.8813062157823763]], [[4.9892537231863248, -0.61142944207593342], [1.8761692847168892, -1.56516598758318]], [[3.6965794780646419, 1.2453461167024171], [2.320588111068437, -2.8885926125971739]], [[-3.7903622898580567, 2.6615135440948743], [-3.5273222821809105, 2.6761506290383537]], [[4.5771732921003885, -0.19810398336258928], [-2.0179543702938165, 1.9110100704797093]]], [[[2.2679866069347021, -0.88391840003125921], [3.4825607482061169, 2.8549449650280403]], [[-1.7340802526280283, -2.5779035316687771], [2.0066417392079376, -2.3845991610394091]], [[0.37259701466198614, 3.0832841138252949], [2.7623103874761501, 0.062169589360795641]], [[2.1390089975551447, 4.8511148680524467], [2.285259693330012, 0.62377397390867539]], [[3.4513435456307171, -0.58199500147995131], [4.9869017560312798, -0.71042596993805684]], [[-4.7937960157075485, 3.2206061736058462], [1.6914132137129378, 4.6845087029346395]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([1.1534005809343952, -1.966493943523334])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.3236067193797042, 0.39314026070729619], [0.249910726669793, 0.51438051705347421]], [[-0.26500446050933613, -0.34397797355590309], [2.0151846058623861, -0.41571907063583874]], [[0.30293100550677637, -0.49474565018878913], [0.51888012418783802, 0.29703843567859395]], [[0.33213115611248489, 0.2669707953403852], [0.37031569355229116, -1.8259592719592201]], [[-10.885151626637807, -1.8316303368055478], [1.7437157966201142, 0.59721272402340098]], [[0.49789940258338677, 0.34975811434133519], [-0.27417881446129433, -0.41539304716461783]]], [[[-0.72294385566062147, 0.7618555705316723], [-0.5983763852845857, -0.68611465069623012]], [[0.45586502759540959, 2.0728855757385389], [-1.8249241814633239, -0.5547849112119525]], [[2.6392754692276448, 0.41146249359730636], [-3.9308223822042252, -1.1522002749625708]], [[0.43867437269836168, -0.85162661087571734], [-0.51789682611843124, -0.43503100287614455]], [[-0.51536410647185071, 0.41654432641148187], [0.61656426951460364, -0.44236758191360925]], [[-0.45120000998668197, 0.54137618472245597], [0.74882932169251781, -0.59955396254198334]]]])+(1.-msk_ref)*numpy.array([[[[-0.31950052038758975, 0.56592338372587381], [0.24644849540470934, -0.98697845855259481]], [[0.27414170964966172, -0.24558845445555907], [-5.4286077847779302, -0.23628933116410278]], [[0.23117697453914818, -1.8864001331345022], [0.61476359853553486, -0.73691901695065321]], [[0.31201833689188319, 0.92616868954352483], [0.49702942777007963, -0.39929499781464739]], [[-0.30429824189116983, 0.4333626569338549], [-0.32699041614685076, 0.43099239946327589]], [[0.25198971228050643, -5.8221978243785673], [-0.57156920786393139, 0.60355546982798691]]], [[[-0.86706594188452879, 2.2247460211867862], [-0.56466895646724435, -0.6888027501798164]], [[1.1340270673996078, 0.76282681619600645], [-0.97999254430915461, 0.82466436106023389]], [[-5.2778038098542064, -0.63779200064816322], [-0.71190187476363487, -31.631123250806155]], [[-0.91934814008309784, -0.40536948660480032], [-0.86051224255297565, -3.1525745314459717]], [[-0.56977635449036879, 3.3788845926902282], [-0.39433179952763425, 2.7680490673712219]], [[0.41021644164245608, -0.61059745821750488], [-1.1626336649023497, -0.41978659198378937]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[1.6320119160503053, -4.670907174525718, 3.5037629273555009, 4.6567582610045086, 2.2035077383914494], [-3.9409201004832872, -2.4726443226866888, 4.2387261261827938, 2.3654160417238366, 4.3642191245760475], [3.1446388427691065, 2.9330741224279953, 0.71401812591388492, 3.2081484068577204, 0.28300874035208334], [-3.0795880389969255, 0.068567371887350959, 4.6971982609258376, 3.9152909109652239, -0.44555470038927503]])+(1.-msk_arg1)*numpy.array([[1.993717662808927, 4.4745195565339753, -0.69368697363530707, -2.4550260640824417, -4.937969064396305], [0.46862757903152197, 3.645110715123355, -0.05990748738878704, -4.2407808435334147, -2.2793574720110046], [-4.7401259192566219, -0.47180251276696694, 0.88468689367360298, -0.16010648866057231, -2.4376270413275334], [4.1741403533968597, 3.7375098484469191, 0.42398181321725303, 0.95580834268102954, 3.3884759032809146]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[-0.038568430512374796, 2.7067429432668435, -0.21270492324406476, -1.1958621983865947, -0.63757568723925129], [2.8603187341019467, 1.8604372405153971, -1.5777523034844521, 2.4953392703729369, -1.3290041722942081], [4.6922413415723412, 0.075770701609364721, 3.4671328573784059, 2.3587081616580896, 4.4243620955927749], [0.11598972856804135, 4.8425574746657833, 1.7206995212638301, -4.730716547038929, -3.1387692697449676]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.023632444183199187, -0.57948977406978441, -0.060707567165397762, -0.25680143382161208, -0.28934578995609911], [-0.72579972726449771, -0.75240794781754583, -0.37222322379797274, 1.0549261636673508, -0.30452278732070109], [1.4921399805137707, 0.025833203815061389, 4.8558051001026881, 0.73522414256651225, 15.633305494694435], [-0.037664040481798076, 70.624808000831649, 0.36632465262913405, -1.2082669345948245, 7.0446328296001992]])+(1.-msk_ref)*numpy.array([[-0.019344981103310364, 0.60492370389001582, 0.30662954809338888, 0.48710774027303227, 0.1291169869483981], [6.1036073463989382, 0.51039251916177741, 26.336479332627828, -0.58841504959587176, 0.58306087948621332], [-0.98989803678215571, -0.16059834265187445, 3.9190507762371944, -14.732120986417854, -1.8150283126098175], [0.027787692494252249, 1.2956641376284412, 4.0584276674672468, -4.9494405267161961, -0.92630709479321749]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.80992856044952877, 0.80262901505860462, -4.3094568836440663, -2.992524875894361, -4.8945376686688054], [2.5445282576965358, -3.958429631429695, -3.9866623171482796, -0.85999448651918353, -4.0895235405182522], [-2.1471857436845165, 0.77276530828386569, -0.87622034614052957, -4.7623897520548875, -4.9207512686508519], [-0.64723992205695335, -4.688391452731909, -4.7432741552511626, 1.6885295495971135, 3.7260001022377338]])+(1.-msk_arg1)*numpy.array([[4.1368493903405046, -3.8069333014373052, -4.5620525867698367, -4.3558553511903488, 0.10902683177766104], [-0.46031382773435237, -0.76759482681649249, -0.22246407734941887, 3.7053885944793734, -0.64395347435087036], [2.48345168398754, -2.0604362081353811, -3.2778772259981204, 4.2237016707551742, 2.5949549816810356], [-4.5396702123840456, 1.9817134473067712, 0.76345708875892626, -0.89336220272798084, 0.45949350116671361]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.6732635420059014, 4.5405528299440707], [-2.4688208067354314, 1.430618236693145], [-2.6372604360611449, -0.69467245220949536], [0.024975853349125998, 2.8372086670150516], [-0.85269033676333628, -2.8050473538466303]], [[1.4522917251239473, -2.8707841800695411], [-4.7299402782781828, 4.704676191535258], [-0.065845058996291783, 4.0982470385366696], [0.28886286736065525, 3.9778182591954145], [-2.8454109793347859, 0.34688819383840741]], [[1.0013076884454266, -4.9638468660693853], [-0.77013572857593715, 4.2009852904044855], [-3.2147726910976626, 1.3316004210368462], [-2.419541216407981, -0.22144828095012148], [3.791776164573621, -3.9776508570058287]], [[-2.3036188608093324, -4.9241527843443542], [-2.0152564312678631, 3.0694147889712333], [-1.8891951900589388, -0.048292690747466338], [-1.2977866394447601, 3.0792577071338467], [-0.50456388219849568, -2.5505066757723172]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-3.3006164648918905, 5.606115220117637], [-3.0759177159265398, 1.7824153000358294], [0.61197048891949557, 0.16119721602182083], [-0.0083460804454170458, -0.94809860725622508], [0.17421264161917202, 0.57309751068062265]], [[0.57075087326349894, -1.1282186281037225], [1.1949032112943829, -1.188520860439304], [0.016516337165820395, -1.0279895091461391], [-0.33588920846437509, -4.6253997223814567], [0.69578056004396938, -0.08482362074738109]], [[-0.46633491834162794, 2.3117920192370267], [-0.99659718199078029, 5.4363016110724605], [3.6689089739329934, -1.5197095421284386], [0.50805191140939099, 0.046499403131499355], [-0.77056854889827264, 0.80834219001205521]], [[3.5591421083673938, 7.607925000508633], [0.4298396265724741, -0.65468398275120487], [0.39828926775557705, 0.010181298648740908], [-0.76858982998219638, 1.8236327033000777], [-0.13541703391137011, -0.68451599725951495]]])+(1.-msk_ref)*numpy.array([[[-0.64620760626382501, 1.0975871735979097], [0.64850645158488318, -0.37579282940234754], [0.5780863736006292, 0.15227190809331695], [-0.0057338573794239308, -0.65135511587631389], [-7.8209219039055631, -25.728046097560437]], [[-3.1550034728960314, 6.2365803656158549], [6.1620273001253052, -6.1291139897950311], [0.29598063552917159, -18.422062057685178], [0.077957509717342346, 1.0735225625517204], [4.418659255162912, -0.53868518092566231]], [[0.40319193439579332, -1.9987692525184193], [0.37377314839214637, -2.0388815115058683], [0.98074835311098563, -0.40623865057403763], [-0.57284851180679641, -0.052429905853300422], [1.4612107691044698, -1.5328400242338964]], [[0.50744189622522551, 1.0846939433863401], [-1.0169262533928294, 1.5488691329933155], [-2.4745270138627036, -0.063255278467544002], [1.4526992920473065, -3.4468188801037147], [-1.0980870913676526, -5.5506915098826202]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.7774615745717774, 4.5225126955209607], [0.71963904690590663, -1.9600503744006215], [-3.3273300007615489, 0.43305843225775131], [2.5895340383249845, -2.8523759104670754], [3.3523026379200243, 0.11518941254945148]], [[-4.3582204679343803, -0.1857663334198012], [-1.0164025614012995, -4.9935255158552225], [-3.8191279446597912, -2.4055429327888236], [-1.5255470809554517, -0.5162969302874032], [3.4054781946198496, 4.6094651832633833]], [[-1.9098402803568701, -4.2857799260537117], [-3.8393163923009146, -1.4187322531315552], [-2.7331814054283674, -0.79747276400559208], [-4.974315338976643, 2.81783199667235], [-3.6719044806308321, 3.1925418540486188]], [[1.6652838253463003, -4.4249197975163401], [1.0942284153829647, 1.2600746508329141], [3.4656655183517771, -2.9286091846312958], [2.892713250109022, -0.1423480731087885], [-0.64169575210458074, 1.7981878515586711]]])+(1.-msk_arg1)*numpy.array([[[-2.1203162355300154, -4.5503342286066575], [0.6179128180781106, -0.86988560348176414], [2.8799916158109564, 1.364834922511859], [3.5183897064852125, -3.0704612734661429], [-0.70603977180095168, -4.1655763181346348]], [[1.7833577363971767, 3.2116507716932503], [-4.3629237918841079, -2.5909440025604678], [0.73009770979801747, -0.884851467624026], [0.82224650365791963, 2.4808316185154045], [-4.7400749658926387, 4.8746686614998254]], [[0.48097848615686534, 4.7486349159955008], [-0.39592894972336179, -0.13015121963926823], [4.4107402541924721, 4.2883667616837595], [1.423329923062763, 2.3241857153054006], [3.7281870752162032, 3.145435132718335]], [[-2.9623986215744891, 2.171311848209255], [-3.5401080761788464, 2.9912548961377272], [1.4432149987476208, 3.2808459642080425], [1.95212102944883, -3.2179246491749836], [4.6252739302815211, 1.8699805751564478]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[-3.5336856970569563, -0.93572336297994063, -1.6973795404762559, 1.4358390263810978, -4.5629003376015529], [-0.96222079744227429, -1.5114699464170744, 0.83340409464016041, 0.48700282711074649, 0.77547327562230617], [-2.5706532878693054, 1.2564304577685963, 3.0677373115562325, 1.3693235918393976, 4.3380248925255724], [-3.9307248801449477, 1.3248741508608592, 2.4315738379574956, -4.9725047936418498, 4.1388351872339868]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.73965758633520651, -0.78135451130002886], [-1.3002676369536785, 0.47739760936811765], [0.5101326108584856, -3.9195161992966239], [0.55447775743849914, -0.50338352007256282], [-1.3611242272662645, -39.612150427824027]], [[0.22078295591556613, 5.1797372523244798], [1.4870780572741107, 0.30268593634255431], [-0.21821842753539911, -0.34645155706032987], [-0.31923159448198485, -0.94326113238684228], [0.22771347555460461, 0.1682349784174508]], [[1.3460043304715286, 0.59980991376669412], [-0.32725369034137181, -0.88560082777796056], [-1.1224053059425196, -3.8468239293181941], [-0.27527880693649509, 0.48594933745392449], [-1.1814100599317063, 1.3587996934243196]], [[-2.3603933577674323, 0.88831550853220465], [1.2107839023693894, 1.0514251278566016], [0.70161815243899206, -0.8302828013781649], [-1.7189760490274808, 34.932013374298705], [-6.4498404012489985, 2.3016700861628223]]])+(1.-msk_ref)*numpy.array([[[1.6665842754223128, 0.77657717423078043], [-1.5143291021058822, 1.0756855375404044], [-0.58936961175781166, -1.2436518970018571], [0.40809550566115849, -0.46762974631503046], [6.4626675717751718, 1.095382724771405]], [[-0.53955568072741145, -0.29960318410801906], [0.34643510144016365, 0.58336650461120865], [1.1414966564827642, -0.94185761693766479], [0.59228324467689675, 0.19630628031183428], [-0.16359936946192813, 0.15908225347642616]], [[-5.3446325809901563, -0.54134574111187383], [-3.1733735526196623, -9.6536203137470693], [0.69551529556525216, 0.71536262685510921], [0.96205635085142238, 0.58916272603433795], [1.1635748971298614, 1.3791493734530085]], [[1.3268723700849556, -1.8102995584843018], [-0.37424680895361639, 0.4429158319378001], [1.68483132455493, 0.74114233477719782], [-2.5472318153581939, 1.5452520912559926], [0.89483028456696689, 2.2133038397405387]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.63755645237865366, 0.036148267958827418, 3.4393894437881833, 4.4088383173136094, 2.4246118923438278], [0.78822349580426021, -3.3696250946692752, -0.68939368297029713, -1.1189190816056636, 3.5147980036042252], [0.86865695535192167, 2.7220518422722249, 0.27647286391506842, 2.1755950080784263, 0.23307727361069208], [0.97606610896199086, -1.9451005528978871, 4.8164771242765667, 3.7945649243250372, 4.5747490938117767]])+(1.-msk_arg1)*numpy.array([[4.9927206397146868, 1.8940829983224852, 0.10129289888443971, -3.3447199599141908, 4.994347690909466], [3.036430405847776, -2.4616155726910094, 4.5367147949111128, 1.5047396309809127, -2.5761571995614219], [-0.94895196318102482, 4.191612490108799, -4.6484312296430623, 2.8864792980042955, -0.073606595369603234], [2.7973434090686631, 2.5057042086520989, 3.1893130307792799, 4.2238388271995451, 0.041076617703086349]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[-4.0686060510927744, 3.5703056421418253, -0.084643648845466934, 3.7158459504727901, -0.079157171473593024], [2.4829656290146218, 4.0299151648552218, -4.6547115025654673, -4.5600023875534959, 4.9833089790873704], [-3.7493612656270314, -1.3055299440641024, 0.23568117697877167, 3.7754064887877199, -1.0055478941113662], [3.9839684144287872, -4.3132237768537358, 1.7517662754660037, 2.1873989556239088, -4.381765105365595]], [[-4.4820754372225826, 4.4502810274411537, -2.3347110399397994, 0.90061216584118498, -3.73836588920609], [3.5928781133515795, -2.7837893941415115, 1.5051495097868353, -4.0848484585186648, 4.6363386971608449], [-2.9042601516400559, -3.7078728083410062, 4.31265874310456, 0.47541656227523887, -3.7420392274644287], [-0.58838067739431921, 4.0789191316410989, 2.6245578848194793, 1.6525765291776935, -4.4575826183991261]], [[4.6768480735944227, 2.5723842071759906, -0.68438240207233214, 0.91628451733056338, 4.647270578837432], [2.000981497625272, -2.664781398830649, -3.2674317637353845, -1.2649512681579864, 0.41531270054429292], [-3.3098849831760857, 0.13053295106396057, -4.4515739234938962, 4.907547431337294, -4.0105294195934329], [0.7909334234406451, 1.3363201082687395, -2.2036368117338432, -3.3644568064272273, 0.77892219378616456]], [[-3.5420291894270628, 4.1084036146878926, -2.9590075230847579, 1.2169943463281676, 4.53712686025845], [4.0696158815553769, -1.4897672448921218, -0.53041144252864658, -0.22379506211301337, 1.2713466379228482], [-0.32357484797796587, 4.9559335957229322, 3.0265194721415423, -0.75426088070496355, -3.9352090142727105], [3.7372505355092418, 0.029487329772179827, -3.1327594520396396, 3.3221048732617326, 3.4991697461353386]], [[1.5344645416180036, -0.64690522585663324, -1.652495208962661, -0.45319968107276143, -3.1571344456076966], [-1.1964162540893564, -2.747407689271558, 3.2531321456799951, -0.4844127710781132, 4.9643355089408754], [2.4509170001604161, -3.6638010891000783, 4.202587165664486, 1.7512442264556523, 4.5430518644470137], [1.3669404213392813, -1.4926684568001356, 0.95749155622670123, -4.7628201270755994, -4.1981861931741946]]], [[[3.6614807115882417, 2.7457928185897043, -0.41711913996066574, -2.2724464641823525, 2.9146103674234549], [1.3235649001432819, -4.027898110069513, 4.2585928729908211, -4.3060253955142223, -1.0765751634820386], [-4.8382367827357928, -1.1046398324644846, -0.7121920972327338, -4.4569760712813178, -4.1434000030456062], [-3.2502019117758971, -1.5512144205328262, 3.7624921479641547, -2.8917499246188596, -3.2308454682532526]], [[1.3741752379946295, -0.94540655281256214, -2.9171268457860169, -1.1235291138112413, -4.5259853359120781], [2.6089669974159433, 0.45464496921935638, -3.1586618972655014, -0.86000820503222997, 2.4919840376481108], [-3.5546280769458694, 3.562041406153293, 2.0863218088255264, -4.932426041651004, -2.5373881040714141], [-0.66141721368585671, 4.0595240471064677, 0.63391294033380152, -1.7466187963074207, -0.27771004854082548]], [[-4.8380535280032593, 3.3369112418171039, 1.69545097015952, -3.9763020281548087, 1.7498198615458875], [3.3472061994336659, -4.7907550051913486, 1.3254802434627724, -3.1766421527125477, -2.1927900761672978], [-2.0269230597585022, 3.9813990783764091, 3.9097980854450736, 2.1591601118555541, -4.144180184105057], [3.7141299682155893, -0.25402115174910556, -2.7465826253020662, -3.7258243060015617, 1.4174478323887252]], [[0.014730195153830117, -3.8874544981819126, -0.56466577577947596, 0.39321419170348726, -1.1363751256129149], [-3.6533105184012413, -1.4195122079984102, -3.2562286081578637, 4.3555078613126561, -4.7434949394506285], [4.4767094615933267, -4.7188429338495848, -4.2795446547360827, 0.88033957296444321, 0.23526756216394773], [-2.3728289753857412, -2.9335689856797496, -4.9345841290663479, -2.6379983065625909, -4.7225296866263813]], [[0.83891510695048588, -4.3660419337417142, 0.79117143993188055, 4.0333468313822127, -3.8408029469956162], [-4.9462828119814262, 2.9453071172214083, -0.023185495921649313, -2.3177952880928143, 1.0296756826936351], [4.6216127865088605, -3.3424622997648545, -3.746226412582081, 4.8302687926004246, -0.16182204590367277], [-4.6729787651866435, 1.0623881174405785, 0.22905606591630168, 4.8884502074482032, -2.3507764747582991]]], [[[-3.7866156083865299, -3.750453829500231, -4.4018773796946276, 1.5867681899860724, -1.0455759920253369], [-2.9457850815914965, 3.8077557971589897, 3.120968121994057, -1.3111133371378845, 1.5367179427195854], [-1.1568877693885993, 2.4945430692248287, -0.78730177000706725, -1.6842307440661433, -1.6446946896750347], [2.8726994146272844, 2.1464044286384869, 2.3497976823727571, 0.74782455644197832, -0.047507378201149919]], [[-3.5897570184713268, -0.1485639856849712, -2.3438700387163545, 1.6763043558549784, 4.5176011316764821], [-1.175821674688394, -1.5629949007034361, -4.4126514424409891, 4.0654462550418433, -3.4380153192336782], [4.9261546168275832, -0.58380292390969579, 2.0117215617850137, 3.4312845080702274, -1.2472650132855865], [4.3122463454306725, -4.0100668850589685, -2.7344491094726187, -1.4179009483878557, -0.19215319004703524]], [[0.13469908523158214, -4.856361893792764, 4.6020942649744079, 1.5005794276770601, -3.2645724496545556], [1.7206327618725723, 3.0310278528501584, -2.2648780140808213, 0.83176366237136889, -1.5533680279339301], [1.7660007996621694, -4.9416414125886092, 3.6215810433574465, -3.7342155656381202, -0.62409776450273569], [-1.3648155228171968, -2.976135475003189, -1.4711836017783284, -2.2075471775346243, -4.4218413874094695]], [[2.7074325784178965, 0.053281079557956801, -4.4434372385701355, 0.054904814374763689, 3.0910434501304707], [-4.6194195369597848, 2.3573980179401328, -1.8058405312697712, 2.5020987821476321, -0.52017121141071598], [2.1614994089090223, 2.3987508885741518, 4.5238843300165872, 4.1439971834651139, 1.9443848819040692], [-1.6234751237737557, -1.5835951781160373, -3.0419369527640825, -1.7071310888023108, 0.41866600877541593]], [[-2.1038253723285694, -4.0041988714392298, -4.1395158595249582, -3.2582230341220244, -4.4709213090053925], [-1.6648048523009482, 0.51447298641689798, -1.1264654766021831, 1.6123317298561695, 4.4350997634134846], [-4.8036136422118929, 2.1440303184868101, -4.9801007953175978, -3.6663665649993273, 3.8081764990447109], [2.2784506530706068, 4.2128462260953317, 0.10004235444517651, -3.7275831117038449, -0.62655691061776508]]], [[[2.1622704501431684, -0.74700065124816106, -1.1664131877664898, 1.1203848102771286, 1.7607157579309245], [-4.9901594244634984, 3.4913536140560879, -2.8062418026653124, 2.1698994005555541, -3.9641566375033954], [1.7443354173552441, 0.38801726361150379, 4.1442024423282149, -0.58202600295691909, 3.4813544655686037], [-4.9025318986548712, 0.97205380382763718, 4.8521428517136638, -0.047292093967597815, 1.4791460316358016]], [[2.8952574613892565, -3.4423290049651389, 3.7616305793021088, 4.9672121508654588, -0.0491291764777122], [-2.9680085493794648, 0.57782212043503023, 4.6122019493548443, 4.0662239110608311, 1.4760013414225481], [4.4451343343928453, -1.9898342231292698, -3.4871266362971864, -1.6836246313403969, 4.1657827537163534], [1.2869897164332276, 2.6241368212260987, 0.26235278047427446, -2.6486600733927821, 4.3767750458574373]], [[2.4563076493750078, -4.6132703622489499, 1.0663547029352918, -0.39332027081632326, -3.5641294547990485], [3.1043112513673972, 0.28657225710867174, 2.6033707628070832, 2.1004728312449714, -0.75747018527161991], [-3.3317885532607483, -2.6299181004079788, -3.103141358338469, 1.7114827303768827, 0.29653262430373051], [2.9398469210141611, 2.1605751566818396, -1.6160585411827744, 2.8490202175713648, -3.3637496551569059]], [[-2.7295557091935652, -0.94761014592475412, 2.8354159192075601, 2.6368508415318628, 1.2435851565962439], [-4.4557283847941767, 3.9741138257302673, -3.8440872292216008, -4.6826791071906158, -3.3784882328335906], [1.5490497321435148, 3.6651015589917062, -2.2406437971402515, 3.7477501528129924, -1.4458153008627326], [-1.4674133200630912, -4.2232660934627697, 1.7339946046200083, 2.2763744017328547, 0.68626682703677222]], [[0.13797153806481166, -0.80821413833936706, -0.88846621750220045, -0.91191778147144742, -0.65260757107881417], [-1.1712266050058009, -4.7823735630480027, -0.3685301783667283, 3.7759581093517305, 1.2112244370009151], [-3.8885787888663748, -2.3164537714575206, 4.2436624822504356, 4.3679735526525008, -2.3781879294518102], [-0.958949118008948, 0.85993853515398833, -0.23084403866501457, 4.3980271041966894, 0.66065067515051012]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.3815620340963513, -5.5999835447063608, 0.13276259463718185, -5.8282618529063175, 0.12415711766113612], [-3.8945031765437359, -6.3208758217724057, 7.3008617279289476, 7.1523115647886923, -7.8162631097139519], [5.8808302412101279, 2.0477087780906498, -0.36966322919244388, -5.9216818757022844, 1.5771903654331731], [-6.2488088694970232, 6.7652421377927681, -2.747625357613988, -3.4309102314986566, 6.8727484272454724]], [[-123.99143002723201, 123.1118744751474, -64.58707904342792, 24.914393322163455, -103.41756604947319], [99.392815098190553, -77.0103120103078, 41.63821933325238, -113.00260535778018, 128.25894459014182], [-80.342996100061683, -102.57401025587846, 119.30471324425953, 13.151849012979943, -103.51918470136884], [-16.276870528471239, 112.83857739150751, 72.605356577771005, 45.716617212751785, -123.31386453913301]], [[1.3597902040553129, 0.74791885281322978, -0.19898368976749126, 0.26640906251121044, 1.3511905687885331], [0.58178392715579419, -0.77478326964207578, -0.95000342855521391, -0.36778366882604446, 0.12075186812426299], [-0.96234667148671016, 0.037952361370333816, -1.2942919074005417, 1.4268658759189725, -1.1660585360104465], [0.22996332237663145, 0.38853410761094304, -0.64070581356054057, -0.97821338973511984, 0.2264710660181159]], [[-0.8033928519259671, 0.93185626666192256, -0.67115355794851161, 0.27603515001877088, 1.0290980375581134], [0.92305854482662741, -0.33790471268628097, -0.12030639464498134, -0.050760550967397698, 0.28836318014435702], [-0.073392314412456458, 1.1240905742133629, 0.68646642365094923, -0.17107927903433512, -0.89257276657641405], [0.84767239497828106, 0.0066882311506825743, -0.71056346968706496, 0.75351025239817715, 0.79367159652782426]], [[0.63287017046454597, -0.26680774267393442, -0.68155040160477987, -0.18691638134079333, -1.3021195085188471], [-0.49344650080586827, -1.133132976022686, 1.3417125256014693, -0.19978981898411802, 2.047476350593143], [1.0108492034950631, -1.5110876510460192, 1.7333030407608543, 0.72227816418187929, 1.8737233281716394], [0.56377700103495121, -0.61563191268405448, 0.39490508120089757, -1.9643639223725282, -1.7314879162437355]]], [[[4.645231626662266, 3.4835206425660368, -0.52918891936234425, -2.8829976222209313, 3.6976953655124754], [1.6791746340836853, -5.1100964783594351, 5.402773319571736, -5.4629498085699524, -1.3658247555581429], [-6.1381534659774637, -1.4014297192922045, -0.90354081174153766, -5.6544572637151127, -5.2566309239715157], [-4.1234522049606861, -1.9679880500771569, 4.7733823820173154, -3.6686928770986151, -4.0988951553095667]], [[-0.40781250120927864, 0.28056728159705041, 0.86571258339715373, 0.33342852164433873, 1.3431717798731251], [-0.77426031802271178, -0.13492449647843666, 0.93739267975611995, 0.25522370616029577, -0.73954341139921276], [1.0549031352387741, -1.0571031809409359, -0.61915546988478742, 1.4637907491412234, 0.75301792715324489], [0.19628807214553762, -1.2047405669932267, -0.18812565864868697, 0.51834217375415448, 0.08241571116625436]], [[7.0178384970952932, -4.8403565687457517, -2.4593363879613173, 5.7678248675309227, -2.5382011828229638], [-4.8552899194144228, 6.9492296252992514, -1.9226753540181212, 4.6078782431335608, 3.1807516232517834], [2.9401532242439088, -5.7752183936793484, -5.6713575740924353, -3.1319696788526894, 6.0113405249807181], [-5.387531188584469, 0.36847037915206743, 3.9840553999105968, 5.4044944101440082, -2.0560789392231733]], [[-0.013164665252372042, 3.4742945777663961, 0.50465291464077378, -0.35142326033015697, 1.0156008100087111], [3.2650354958275383, 1.2686459917739463, 2.9101555793338809, -3.8926030781979741, 4.2393547642816589], [-4.000923333230844, 4.2173227818029373, 3.8247132657661713, -0.78677679864136763, -0.21026324962332013], [2.1206439450301451, 2.6217883258099768, 4.4101349330687567, 2.357630993992013, 4.2206177053031295]], [[0.23868088751906261, -1.2421885779110455, 0.2250972713426431, 1.1475330380995565, -1.0927521135089673], [-1.407273705888445, 0.8379733669477345, -0.0065965372399420701, -0.65943911590823923, 0.29295444052197633], [1.3149013917071934, -0.95096853256925429, -1.0658440140060792, 1.3742663981393124, -0.046040212193626341], [-1.3295155967411982, 0.3022615001918062, 0.065169055428339756, 1.3908196722643451, -0.66882263855496438]]], [[[-4.3591610992770402, -4.3175315714599876, -5.0674519470246819, 1.8266914001088268, -1.2036696253721235], [-3.3911949515192239, 4.3834977360151823, 3.5928660937615469, -1.5093568629824752, 1.7690734337089491], [-1.3318120142374337, 2.8717240492412874, -0.90634371273537462, -1.9388905294424377, -1.8933765274563588], [3.30705855392991, 2.4709459993547251, 2.7050928078055581, 0.86089744845133931, -0.054690609346359415]], [[-1.3187687915138264, -0.05457794130803837, -0.86106737657127641, 0.61582381710103218, 1.6596308202217882], [-0.43196152858972747, -0.5741973302752128, -1.6210754600314816, 1.4935227139716134, -1.2630234537942544], [1.8097210862506903, -0.21447164041606495, 0.73904601321102503, 1.2605507561553173, -0.45820766302688626], [1.5841896463775786, -1.4731779985907898, -1.0045543832075017, -0.52089417488987533, -0.070591304347324968]], [[0.48720544694383194, -17.56541971252782, 16.645735859227603, 5.4275830417050743, -11.80793081615931], [6.2235140820226942, 10.963201993600647, -8.1920445356133929, 3.0084821005322389, -5.6185189603675525], [6.3876098892825848, -17.873875007518517, 13.099227866608944, -13.50662597680928, -2.2573563121712246], [-4.9365261512119458, -10.764656729267463, -5.3212585891621949, -7.9846793868810781, -15.993762732417187]], [[1.244456145727789, 0.024490348323154505, -2.0424009165634001, 0.025236688892413778, 1.420780723734334], [-2.1232901894915832, 1.0835647301941009, -0.83004443591951549, 1.1500756220053967, -0.23909376951096808], [0.99352103717968454, 1.1025723444239863, 2.0793779693456207, 1.9047649806501716, 0.8937255668836217], [-0.7462212028182923, -0.72789061026331958, -1.3982091986186549, -0.78467319628119492, 0.19243747444759893]], [[-9.0262999036215756, -17.179705294336955, -17.760272356880115, -13.979153710044763, -19.182141783901042], [-7.142716346861274, 2.2073065230556108, -4.8330129280802954, 6.9175844769372059, 19.028452215471731], [-20.609532485931457, 9.1987961128633007, -21.366736954525379, -15.730261935032082, 16.338686479598568], [9.7755161529660448, 18.074890618173395, 0.42922397750488878, -15.99290678991728, -2.6881939234637704]]], [[[2.2152909831513981, -0.76531768124043131, -1.1950145354467085, 1.147857506771355, 1.8038898613162366], [-5.1125219681793306, 3.5769642875614331, -2.875053008089425, 2.2231070012902707, -4.0613608044634653], [1.7871078622023651, 0.39753174508245692, 4.2458214707766224, -0.59629772780030399, 3.5667199522693105], [-5.0227457481015572, 0.99588931006054437, 4.9711211230084951, -0.048451732452724085, 1.515415828963488]], [[-1.4884872954643853, 1.7697434715324214, -1.9339003187767769, -2.5537045596255226, 0.025257910910835756], [1.5258895201883569, -0.29706542398240965, -2.3711894701193752, -2.0904954784999732, -0.75883035415497846], [-2.2852979645552565, 1.0229981273537436, 1.7927744820708258, 0.86557202856791482, -2.1416798979929377], [-0.66165716446680345, -1.3491008561570541, -0.13487877533293124, 1.3617085602318628, -2.2501536176814851]], [[0.5099801340266813, -0.95781008467716156, 0.22139723192300179, -0.081661401200031611, -0.73998679176419413], [0.64451904810690108, 0.059498311673537696, 0.54051346983156467, 0.43610148601307053, -0.15726643472544088], [-0.69174802813190606, -0.54602524470683367, -0.64427615418282713, 0.3553391174122823, 0.061566289354746929], [0.61037286073599384, 0.44857996849852316, -0.33552708743021503, 0.59151536362778567, -0.69838381214405543]], [[-0.71933298378840838, -0.24972827315461243, 0.74723083561731729, 0.69490202279274371, 0.32772799554021287], [-1.1742395962790766, 1.0473173881554201, -1.0130508519116648, -1.2340490149930832, -0.89034930228122089], [0.40822854873646786, 0.96588189478498399, -0.59048766902803984, 0.98766267742266345, -0.38102268104423292], [-0.38671451123585904, -1.1129776872150865, 0.4569679631791897, 0.59990392762558076, 0.18085520757266862]], [[0.030159367264850561, -0.17666851706307374, -0.19421091720724551, -0.19933722325995773, -0.14265428719611983], [-0.25601985616874789, -1.045384886685278, -0.080557462455205661, 0.82539130166929509, 0.26476303118772737], [-0.85000919375587647, -0.50635646326286343, 0.92762737261171346, 0.95480068154142494, -0.51985100836869158], [-0.20961786063985677, 0.18797501622924406, -0.050460480767628393, 0.961370124133335, 0.14441243915303825]]]])+(1.-msk_ref)*numpy.array([[[[-0.81490761143913681, 0.71510222577681681, -0.016953411767557651, 0.7442527268429614, -0.015854516442185024], [0.49731715595377535, 0.80715815197012797, -0.93229960946332147, -0.91333016938317646, 0.99811492344425379], [-0.75096556290425487, -0.26148667996347336, 0.04720495977764938, 0.75618220229591471, -0.20140279552449167], [0.79795540386102082, -0.86390248686139559, 0.35086406828604572, 0.438117634346333, -0.87763073914265599]], [[-2.3663564063413167, 2.3495702307568318, -1.2326339669420829, 0.47548717064607082, -1.9737075368487091], [1.8968958153014679, -1.4697293606494564, 0.79465868767096637, -2.1566364632048618, 2.4478012321883824], [-1.5333330979752444, -1.9576084108378162, 2.2769111738630841, 0.25100091321040136, -1.9756469123996181], [-0.31064144386250486, 2.1535060159737651, 1.3856614980145785, 0.87249425217443777, -2.3534251784884992]], [[46.17152954552143, 25.395503885328647, -6.7564696993528814, 9.0458909501238498, 45.879529858645704], [19.754410424249951, -26.307682257872514, -32.257263833104858, -12.488054760888122, 4.100116643103517], [-32.67637731399293, 1.2886683321491219, -43.947541955260725, 48.449076740671458, -39.593391676635264], [7.8083797793464651, 13.192633669150728, -21.755096714606506, -33.215130018794085, 7.6898005917946941]], [[1.0589912554347103, -1.2283251404979489, 0.88468020000115988, -0.36385537830180248, -1.356504255852514], [-1.2167284347655172, 0.44540866283177322, 0.15858171951180461, 0.066909955032156077, -0.38010555537075957], [0.096741984936241782, -1.4817185459825692, -0.90486483425033526, 0.22550793182826409, 1.1765436453381493], [-1.1173582782114655, -0.0088160832971308986, 0.93662832451898625, -0.99323857096453649, -1.0461771951231189]], [[0.30724023167449505, -0.12952747103172393, -0.33087308117744263, -0.090742517165486772, -0.63214150095200627], [-0.23955405753328521, -0.55010340875391828, 0.6513627698770813, -0.096992200194596798, 0.99399077040166472], [0.49073816078553922, -0.73358951275434803, 0.84146868134829411, 0.35064523634251671, 0.90963868469071851], [0.27369748882868883, -0.29887155424061435, 0.19171503777550236, -0.95364208137625561, -0.8405874906978511]]], [[[1.2058503644729346, 0.90428313894553913, -0.13737154625949854, -0.74839405500811473, 0.95988051028941379], [0.43589502252192752, -1.3265240995849261, 1.4024997460140423, -1.4181208919596409, -0.3545528859837172], [-1.5933962370479391, -0.36379553779236623, -0.23454912579624518, -1.4678340931831511, -1.3645628087065484], [-1.0704022412357697, -0.51086776681770341, 1.2391168724690929, -0.95235178749683169, -1.0640275048066499]], [[-0.55824120274490996, 0.38405938088011637, 1.1850456578794908, 0.45641940450637158, 1.838623945234602], [-1.0598596411071006, -0.18469373295455058, 1.2831661987791576, 0.34936738886977348, -1.0123368024211445], [1.4440224202270509, -1.4470339908758827, -0.84754168440069777, 2.0037353095954513, 1.0307816265955658], [0.26869232589506375, -1.6491299828220713, -0.25751906486389964, 0.70954165861001495, 0.11281617309449993]], [[-1.0664222343071161, 0.73553471899096612, 0.37371777746781165, -0.87647167783504354, 0.38570197613230645], [0.73780397286341803, -1.0559965132842812, 0.29216741703701088, -0.70020759433143676, -0.48334316246350256], [-0.44678212129008549, 0.87759518910961565, 0.86181262481625265, 0.47593031730306473, -0.91347602206637135], [0.8186827111948437, -0.055992312330068468, -0.60541223097888819, -0.82126042178822067, 0.31243926419590962]], [[0.0097891986431086209, -2.5834731924005689, -0.37525812715611173, 0.26131709673065368, -0.75519717977530265], [-2.4278688772354018, -0.94336068431523645, -2.1639814231749761, 2.8945259177319462, -3.1523692483319721], [2.9750724772730552, -3.1359863438789448, -2.844043292689995, 0.5850444521026974, 0.15635101071311655], [-1.5769033569209157, -1.9495525506744371, -3.2793607794124364, -1.7531260905535713, -3.1384364373707956]], [[-0.32564592995074487, 1.6947886310994575, -0.3071130286872919, -1.5656446866165117, 1.4909039509116502], [1.9200236743407997, -1.1432947949460663, 0.0090000314909340663, 0.89971034705778341, -0.39969442969898433], [-1.7939948646362371, 1.2974605355348239, 1.4541916980919711, -1.8749899243038253, 0.062815283916378306], [1.8139338569797663, -0.41239258133061324, -0.088913854307996948, -1.8975744990563612, 0.9125128214840722]]], [[[3.9903132669574171, 3.9522061969587639, 4.6386725044952701, -1.6721269901449962, 1.1018218335524743], [3.1042510009851259, -4.0125906736046355, -3.288857859076574, 1.3816435267627691, -1.6193843338163121], [1.2191215301463136, -2.6287348211628734, 0.82965397675970587, 1.7748324566613016, 1.733169594972731], [-3.0272337548020647, -2.2618683683876131, -2.4762029834428012, -0.78805312118767512, 0.050062995856922289]], [[-0.85641433384939403, -0.035443158458838123, -0.55918099400823096, 0.39991873290068081, 1.0777716552608188], [-0.28051774286460196, -0.37288630673558909, -1.0527336324275656, 0.96990031035438562, -0.82021306295526375], [1.1752409433009676, -0.13927883965593929, 0.47993977652566749, 0.81860728208231004, -0.29756209960458724], [1.0287798205598777, -0.95668836146513192, -0.65236209595358907, -0.33827099993946536, -0.045842307823175624]], [[-0.028977321289084711, 1.044731362878669, -0.99003169835596072, -0.32281416106747152, 0.70229552474313606], [-0.37015342959149122, -0.65205393026388836, 0.48723491909220606, -0.17893427293647135, 0.33417037946653849], [-0.37991328954172238, 1.063077233686011, -0.77909747707196597, 0.80332812967631195, 0.13425986825896488], [0.29360776902834734, 0.64024513389041071, 0.31649034461273418, 0.47490154602204032, 0.95125455642139467]], [[0.93797055128363771, 0.018458846940213706, -1.5393968845168289, 0.01902137819340147, 1.0708697797582025], [-1.6003646865417118, 0.81670359443424101, -0.625620468685961, 0.86683413384519226, -0.18020957634110213], [0.74883592977904867, 0.83102999915247688, 1.5672672009615276, 1.4356580302967228, 0.6736181628769734], [-0.56244128440353702, -0.54862516395351635, -1.0538571868044473, -0.59142329203005783, 0.14504382867560511]], [[28.582022599531488, 54.400001132137866, 56.238382426725082, 44.265368038847626, 60.740770396394609], [22.617604359248087, -6.9894957623506118, 15.303866059091916, -21.904718208471873, -60.254108224179795], [65.260641632605726, -29.128236508167777, 67.658350047449588, 49.810299560647785, -51.736892324968821], [-30.954436102223546, -57.234629654329581, -1.3591493254487661, 50.641971592170627, 8.5122387127351047]]], [[[0.77297282955440449, -0.26703930909107249, -0.41697175398097835, 0.40051743616638946, 0.62942424309539113], [-1.7838923202228156, 1.2480961768002898, -1.0031810158051391, 0.77570004223328171, -1.4171147613310755], [0.62356856569712205, 0.13870919900416831, 1.4814779011018779, -0.20806383694974961, 1.2445216609024319], [-1.7525670544279373, 0.34749176689438682, 1.7345538756462933, -0.016906073746356033, 0.52876812580128052]], [[1.1554665755806472, -1.3737970320195461, 1.5012269071159099, 1.9823617383543792, -0.019606933774573659], [-1.184500763949331, 0.23060268583970644, 1.8406809285106722, 1.6227868784433208, 0.58905649610435784], [1.7740060135765308, -0.79412175477793823, -1.3916752920222084, -0.67191675119788952, 1.6625197576521875], [0.51362395928031024, 1.0472652008026553, 0.10470221487771005, -1.0570521709015224, 1.7467245458360983]], [[0.77016825431363545, -1.4464777579771588, 0.33435247423008135, -0.12332444856321269, -1.1175226202014374], [0.97334793462054325, 0.08985391347385252, 0.81627947388123678, 0.65859726247433914, -0.2375026151279164], [-1.0446727935158675, -0.82460331583236968, -0.97298111799964782, 0.53663052634212494, 0.092976958185655253], [0.92178061314220894, 0.67744217511127236, -0.50671054411611172, 0.89330215945445546, -1.0546941058134403]], [[-0.64622629339370241, -0.22434808350701932, 0.67128885244124581, 0.62427828082638415, 0.29442059876625443], [-1.0549001908172659, 0.94087724184427546, -0.91009325556351206, -1.1086311052013524, -0.79986201440209026], [0.36673978234405147, 0.86771813720499169, -0.5304756854621332, 0.88728531228020247, -0.34229888023954863], [-0.34741224277158406, -0.99986440445286706, 0.41052575052198836, 0.53893495818876158, 0.16247467176482566]], [[3.3588826388314095, -19.67577136417054, -21.629488190198394, -22.200410658517516, -15.887568343529404], [-28.513219210787142, -116.42569009981248, -8.9717751600331557, 91.924757209696423, 29.486956442129557], [-94.666479527943238, -56.393488582763005, 103.31090336903718, 106.3372253340203, -57.896391242386095], [-23.345376801481297, 20.934988887592262, -5.6198404730794032, 107.06887154115022, 16.083375703566539]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.3680600484673997, -2.2508018046099441, -0.86753728489042459, -4.6536347415896637, 3.4519597055174565], [4.7864411134085358, 3.0978772361737192, -4.9888805511053471, -2.3930904899668293, 0.95403594679238868], [3.6927455298312069, 4.1939947048314874, -3.0004550273006703, 4.7942167009138963, -4.2125816384833392], [0.079636077370763836, 2.302832074387668, -3.8771561059378765, -2.0574547720962686, -2.1058355442212537]], [[2.4252341637858512, -4.8885003731848418, -4.4151739965765957, 4.2460794351071272, 2.5110708851599011], [4.4996317092324585, 4.6960032615648739, 4.8422541313219618, 2.5047480197099397, 4.9245918675005562], [-2.9663244851940851, -4.2955587344029311, -4.6496126393601722, 2.066721105281597, 1.9354386450062036], [-4.8989076602264898, 4.3000772425448925, -2.9549438549949225, -1.8442845401945651, -0.67392594371965675]], [[0.060595018310912963, 3.7368777368862105, -2.1802749682645017, -0.74370690588818178, 4.4489741381288468], [-1.2057281809389311, 1.9013735351103902, -3.4973595611598585, -4.1799068892504874, -1.8608072756312213], [0.26735244321281826, 1.9952177740569574, 2.6512539658888388, -4.168255512668618, -2.6518019325360278], [2.4069645340238841, -2.1932774299761872, 0.45684373712867554, 2.4658041713491485, -2.6397419084707998]], [[2.6825168182108943, 0.90458959976128916, 0.75734377279126086, -3.6301680925555879, 2.4617465685117565], [2.1135355907349638, -2.7183969442671629, -2.162412321682484, 1.4014666647746328, 4.758745031201034], [1.6273394291911645, -3.0984403397814764, 1.5800492929651213, -0.86492147689886867, 0.94088643741533939], [-3.4005526242534554, 1.7654176897474061, -2.317520856080689, 1.0667074412956428, 3.362861067951382]], [[1.7281700563935978, 1.4815094403842766, 1.742437280541532, 0.92008143559150302, -1.83741835512342], [-4.9138203676353278, -3.9882721562983647, 2.466740451144819, -3.6897636734114281, -3.051078403479667], [3.3212657241438581, -3.8938560298431879, 3.0801514848593463, -0.30181602422203024, 4.3438958226643916], [-3.8269051936851928, -1.2762552205317013, 1.8734916737402161, 3.1638427142563792, -3.6529045494753296]]], [[[-0.37140835828691632, -4.8454323680175051, -3.9260897336462754, 2.7652957150422601, 2.1756321277269439], [1.8681552359434983, -4.8137881052090634, -3.5332521541221231, -1.6696766888631478, 3.3786643156314522], [2.894522029926943, -4.1762223355456172, 1.1413975208909122, -3.0125650565968645, 4.7947744392435148], [-1.9386922805514062, -4.675748165434257, -4.0199083967929292, -2.1014752839582682, 0.25017961182409376]], [[-4.0836414402987975, 2.844529023137695, -3.679679906047201, -2.0859327271445749, -2.5266541945578744], [3.9689914975488954, 1.2443861380620511, -2.321949692887062, 0.14830064348730776, 0.65440399486653256], [1.8512913914856384, 3.5648856307050174, -1.4441339477392354, -0.70812370350041043, -4.4243960240335838], [-3.0416423741944354, 3.8344454667848993, 1.1754191030015964, -0.46559239233832894, 0.39859537096131437]], [[2.93617367595761, 4.2035084328067658, 0.5938995857874616, 4.105391129567602, -2.5411989084111255], [1.3490175102169886, -4.3848201731827885, -0.2919738413608286, -3.6803433812475705, -1.1373906909548337], [1.1706125340553104, 0.93515456684699849, -2.8367951834070504, -1.8634666789732024, -4.6206072544354964], [0.27178081685705102, -1.1287174508694009, 2.1124484209213907, 4.0181783624596576, -4.9196342237831292]], [[4.7561240109926732, 0.89475696842165942, -3.661850429361583, 2.6321930967552012, -2.6682811604633594], [2.2599299626536853, 0.64006013482343782, 4.8338341704353098, 0.65367040906907192, 2.4784696835888678], [2.7099548460465304, -1.0084618503383922, 1.042456131464685, -3.6583618648874494, 3.3598807817462966], [2.443629260056726, -2.9067467483241991, 2.8409244810003305, 3.1118241942472658, 0.6968064489049608]], [[0.36891015154106732, 4.316350965463295, 4.0657873705924175, -2.7212393419234573, 4.6410918083378725], [-0.40482130831149732, -0.54273169942042898, -4.5016595766884793, -2.4426368177560533, -0.070551085829224647], [0.35120906449149025, -3.8985470802793176, -3.0149363736244861, -1.3488564237328751, 1.4008724662577539], [3.740873293258101, 4.1655314302777349, 1.6919877398127756, 3.1565277615503078, -0.10399685682158832]]], [[[-0.27638850779794488, -2.7242091847354368, -3.0340337181737045, -4.7371606494805754, -0.32476475987838604], [4.758710325399532, -3.4853792415132414, 4.9659573375914015, 2.4141588944537897, 1.4181096174508303], [-1.1349485185345842, 4.9257304556942589, 0.21706030697095891, -0.22459667037545117, -0.016789728143809413], [-0.72162522833780152, -2.9294806627655881, -1.0171190373808248, -3.4173064052692572, 2.0526609379878948]], [[-2.2407689402057862, -3.7730329039073496, -4.7068808635239714, -1.138787746558946, -1.780732266983176], [-1.8948660819032446, -1.2009280130401487, -1.5170101041674044, 2.8718532447985421, -2.704937080651276], [-4.1751031007665631, 1.3606012846747078, 4.1495310603265576, 0.91009337087517661, 1.9172913438754549], [-1.9235478353832636, -4.5428359967342029, -2.1025261694230579, 1.5721916316148139, 2.8879178562214749]], [[-0.8730228845196919, 0.42343053748807513, 2.9427963981798042, -2.7774606343263599, -0.22796745517575356], [4.2516676247061014, 0.78989912553444253, -0.24709137949763438, 4.8778269507423939, -4.1276917172619676], [-0.73276335294801953, 1.1086701275706234, -4.9588033838341561, 2.263100466549103, -2.1027505539595062], [-4.172520489151812, -4.4186216000478593, -4.4472901759420278, 2.226846141323664, -4.5418956599640339]], [[-1.3442540781761281, 2.8755989232392656, 0.38690313984300229, -0.011717507899410506, -2.7964617827594065], [0.35777580149448784, 2.2793142993335014, 2.6762481590500089, 3.5670595691827618, 4.884272917012817], [-0.76594479271904525, 2.1648515565442352, -4.9825930795470743, -2.5891136262034111, 3.2186427449534367], [2.0711235876962961, -0.31117479486740862, -0.012308434259880485, 2.0763802931922539, -0.36178517067303417]], [[-4.4853589979940809, -2.4029102818018035, 1.5036537202364295, -2.5944804663786702, 2.2122129477253836], [-1.3367412967528756, 3.2687984274535342, -0.69557855901267818, 4.6206372700900662, 2.7311629096936185], [2.3576603179226927, 0.72829604004674664, 0.71343188115116973, 2.7296106196306393, 0.34196552647052325], [-1.9966168595349689, -3.800065382081641, -3.6811068406639191, -4.3932917407856662, 2.2658542980115399]]], [[[2.5060706092828475, -3.1847359302032316, -0.14406473289059551, 2.0281668051182624, -2.6341208925026818], [4.750613193214603, 2.1027646807692557, 0.91598888362344688, 4.291911505461405, -2.2993316656090657], [1.2920724905771008, 2.9678850330117541, -1.3502746761300077, -4.6523452614889029, 0.22961987380721816], [2.2042413062648283, 0.45559887131705334, -1.1263410039096633, 4.9492846294078916, -3.3332084933957375]], [[-0.098085426897393191, 1.8366609242914942, 1.0137187091791073, 4.2225110842259035, 2.9673893469768684], [2.7513901567570169, -3.5994886891904523, -3.9946295035898749, 1.3346422950946391, -2.0140900459489695], [3.3435089039454926, 2.638920491324952, 2.4793556282130558, 3.4066429521787267, -1.285932258836592], [4.6151809016431748, 0.33206682189405967, -4.4789831213414653, -1.3554419049194255, -0.55380115016802645]], [[1.8402924920433072, 0.63262471920093954, 3.223149970007702, -4.9674630833200766, 2.697747998554755], [3.6262126694489094, 0.55261650192946554, 4.0361233937462515, 0.055111636988044665, -2.230972882345136], [-3.1058309687257846, 4.7401356991913808, 0.43681011327767116, -3.420714016138505, -0.32379158457571311], [0.77050903514938796, -1.3523999382537157, -2.6958892615620575, 3.7213941215087516, 4.8544754153429874]], [[-2.623879226562952, -2.3586348661138121, -3.4991481381742409, 1.6778900578181375, 4.6393177307148559], [1.3612234316498544, -4.723650750038221, -4.8873822703336138, 3.5111861458050946, -1.5823537776925733], [3.3935793066200102, -4.1874759128921522, 3.0674648149478685, 2.868009654421396, 4.900775792076292], [-3.7015418625867036, 3.8977934423302418, 0.021199435865000282, -1.0997031588346564, 1.3799029761990074]], [[-2.3298039768915468, 2.7020244674537119, -2.8999094418452511, -2.0274609131681509, -3.0259609615864216], [-2.0147022876376166, 2.8335012779454782, 1.2152503694451973, -3.6835778719587853, -3.4517941167267296], [-0.36411705819261808, -1.9232336814235138, -0.76165130832459305, 0.46592063642971482, 4.6978699147324665], [2.0022872087419739, -4.4647901631822755, -0.10807838087692367, -4.4214461012490229, -4.9082030434733763]]]])+(1.-msk_arg1)*numpy.array([[[[-1.4415111547064443, 3.8337193749227136, 0.040418262769388313, 1.3259223173787547, 4.7003624353749185], [1.9442746686459822, -1.2482903390885625, 4.8311607581821754, 0.11988840485073915, -4.5654475486890513], [-1.5814283476241977, -4.2914328953617105, 0.27957262338464606, -2.5849004325552505, 4.6172420412194377], [-2.1946021180395672, 0.091721098706304183, 0.28286395114723639, 4.6849431493818514, 0.82710909027620438]], [[1.1417888263357661, 2.902673357968566, -4.6754290262055633, -2.6066847178769614, 0.48711071429262276], [3.8678741766446674, 0.51138595134972853, -1.9574163762101051, 3.2935922454780755, 0.044407314425804145], [-3.3398324241995248, -3.8430091981541157, 3.9201288141430606, -4.4861929167098946, 4.6557609286175161], [3.7201846044777227, -1.5636662708907831, 4.2128784881883483, -3.2971376773169672, 0.89984241151805389]], [[-1.7772225131126138, -1.3777929629841337, -0.85427300787577387, 2.2019172668087448, -2.3747702526213832], [-2.2120232680556851, -3.7922595017267566, 2.1749133865085257, 4.1511242985383632, 4.8382880213100012], [1.6537647064937566, -4.0619326433145408, -2.3213989882533417, -0.36059231105120482, 0.86054565732322352], [-1.7209176366063392, -4.0494646069658273, 1.4659386709895017, -4.0758606061917657, 0.50797344720677273]], [[3.0709160425369664, 4.164469686400075, 1.4606984722234397, -1.4050678184594325, 0.64648971236188757], [-1.5475404035861438, -4.8746103505958924, 0.30024055558273588, 3.5546814232678141, 1.6643200122772015], [3.3451534287133438, 1.0282371357549316, -0.40498132927676789, 1.4082794016529494, -0.017951435218152234], [1.8028786250261373, 1.1361055465796701, -3.7901133648396068, -1.9126743477350661, -2.6278442502748787]], [[1.5289566564058523, -1.2000073927538155, 4.2389955708593856, -4.3411876407738115, 4.0100967816374418], [-1.1805458296030826, 2.8060292172764125, -0.062212377557820275, 2.8003692374889511, 4.2402571004472058], [-2.6903503938012499, -3.9853868581998197, -0.13197884651109693, -1.6025663968341664, 2.5930856449980482], [2.1621202331207234, -2.9995490288227469, 4.0096414664653448, -3.521934007513905, 4.8746657192578766]]], [[[2.2701282792559745, -4.0370351955971282, -3.4436993116181003, 3.6169672191988784, 1.7737050238562375], [-3.2582510587684577, 0.28994476729916929, -2.400052947490825, 3.8560749161836636, 0.45434228177212255], [-2.332569707495912, -2.482919149391897, 1.5810959197968106, 3.3973397275997286, -4.073409086008998], [1.4977704633541045, -2.5132717821855719, -2.5831203907096767, -4.0584712028945544, 2.3937433229752338]], [[3.1845820121664978, -3.334244137073866, 0.91003624227140811, -0.036658407333394649, -2.5566003489357247], [0.26282841843137916, -2.5484306328600836, 2.855438575907999, -2.095655301271393, 4.9794554824955792], [2.0433993314745909, 2.0216444465184127, -1.1636258724388715, 2.2882641320745956, 0.017416928383112484], [-3.8283586564211047, -3.9508639181071912, 4.2143009252696757, 3.1357000328805391, -2.2562990008850594]], [[-1.855424261699862, -0.97690882989448369, -2.1141121326692067, 4.6033134194382406, 3.5303906559015488], [-0.14544895468714891, -3.3843132224831374, 1.9799985667467066, -0.094711016003926396, -4.6156852161382576], [0.1504955658841105, 3.6478552391080132, 3.8927029574184164, -3.982123923274731, 0.58865660136387277], [3.8707243352537706, -1.5317384473267239, 3.4777866633504999, 0.47435411663482085, 1.1028797733065119]], [[2.3084323503457869, -3.0880022975360397, 2.8087972937410335, 1.9923203482768379, 1.9794847369552429], [-4.4754908668082116, -2.0085306124393263, -2.1375194240371744, 0.36686076094253472, -1.6904603565434915], [2.1402939233033935, -3.0718832289841682, -1.639522437056864, 4.6467288308952206, 0.30685151286300894], [-4.2070833294076628, -4.8645868085784016, 3.0282769481082568, -3.5934215429124028, 0.19313990684310589]], [[0.60359458631841179, -0.37102679056150656, -4.2673659009846627, -4.6009176613920655, -1.4779294815618793], [1.2533945463817808, 2.6416824827557859, -1.5524596836149196, -2.9202137872334868, 1.5544432364970806], [-2.8874594310418589, 4.2547593659912533, -1.8482655271171411, 0.77663738148275119, -1.927864292992727], [-2.1465669537530587, -2.5577792570904134, 0.83798237505124185, 0.57551747976299694, 1.2743267330585359]]], [[[3.8799694310716326, -0.59320938746390084, 0.80894106160034784, 0.71456210746162796, -3.4503537080141466], [4.117277451180529, -3.8267971764883568, 2.6939020379751222, 1.3564585016325106, 1.042873152882505], [0.79927766458507143, -2.1352718230517418, -4.3807285296931848, 3.1269623749020976, -4.1350295965566417], [1.804367726220435, 2.0605050821873139, -2.1934138259186109, 3.3127637605803013, 3.9902568386333002]], [[0.62129570329307082, -3.7868231364741245, 3.8420900408885785, -4.2979289855177552, 0.5318841494518276], [-3.7203305620970784, 4.3035935018649933, -0.083187960716035292, -1.1154869007470003, -3.8003351338375699], [3.2664392041581749, -1.5000907021467222, -0.76751610965762129, 0.14891677023573191, -1.7037980649689222], [-3.4944752855516281, 3.8720027836248647, 1.8767336755468644, -4.8093352231436288, 3.944299315273943]], [[-4.711888770978792, 3.5221916885691087, -2.5615877159850395, 0.44892063625470335, 4.4279753562473161], [4.6747646836805465, -3.6241954241879224, 4.5077005547594418, -3.4908819611191055, -3.9573411136646328], [-0.78360177959439881, 3.6466625891807354, 3.2107024543984082, -1.9022855805134031, -2.9076674408184111], [2.1363429297591017, -4.1374244747351288, -1.393496376660508, -1.3141451073140562, -2.9419784760656742]], [[4.1100897534502341, -1.1502366667203443, -3.206233383896314, 0.16535659377769374, -0.41817826099159205], [-3.5639263510403749, 2.9367432902295079, -4.8739432410780461, 2.617164886055634, -1.9924844985675727], [-1.965616027611734, 3.8013438855776425, 3.4227730638023957, -0.67202502124958663, -1.3415494628265101], [0.89748437060990582, 1.8856861846981836, 1.4683914805240317, 3.1647279058211897, 3.4689140172080446]], [[-0.77279094175063978, 0.18365251591327691, -2.5844554651156604, 1.7418710479628832, 3.9274410822685688], [0.48748366323933467, 0.079612864165509833, -1.5838023664518852, -0.98168694457158878, -0.72181415290834749], [-2.2909684724835189, 4.1472987752882435, 4.0496488471862264, 0.17609781035076377, -3.8913575396282338], [-0.33423519620471787, -2.1516682796074593, 0.15760337672398261, 1.188920228970753, 0.93147037879999317]]], [[[-2.9129336094914891, 1.5970309330606032, -0.93820755801653277, 3.1524626617778289, 2.4278512546787514], [3.3260618947879053, 1.0393388904413383, 0.62486838440298165, -2.0709829384408973, 0.65879451726444671], [4.9312960623405395, -4.6325208870775958, 2.873435672775706, -0.31354740762742672, -3.5662655081553538], [-4.9668817101406013, -4.5077370819720972, 2.2592431743956887, -2.8854871350262137, 3.6272221664728157]], [[-0.66498588569494199, -0.89949687010800794, -1.6334807306789312, 4.4630624685691664, -0.55529158420064029], [-0.81813448002757916, -2.359044526627716, -4.2691074074863931, -1.6826005231012475, 3.03362193262228], [1.5369793626767567, -1.0905129343106901, -2.9573204241323103, 3.0991099156180031, -1.0141842268925458], [-3.6235402489138413, -4.5767181134913058, 4.6821512927560089, 0.61988245167294842, 2.1997160831281448]], [[-3.7925455051882064, 3.4378482224608202, 0.17988342316641504, 3.5648625615942784, -4.861228261344972], [-3.585545091630451, -4.8438323743810985, 3.1217558820660543, -2.6281111916873612, 1.8345358342633347], [-0.77553047873936265, -2.8975022408332807, -3.2455230568176399, -0.32982781145975615, -0.91970380198164925], [3.1197673377355351, 4.1071739642347467, -0.75895084612602837, -1.197205799127703, -3.3401820122594819]], [[-2.7563165282746729, 1.0965391342506887, 2.9625757703531832, 4.4535858799838728, -3.6070317143015007], [-0.91393873739433662, 4.1593158666031869, -3.1960065812636653, -2.4982368439332858, 4.3419239235942264], [-2.5208980603095443, -0.020549187459154794, 0.41791510837894918, -4.2230658761430995, -4.1456463031502029], [1.2035860135728811, -4.8576674622646054, 3.6099675404472915, -4.6122083905981492, -1.0460055960883152]], [[0.90411476169143778, 4.4913808895282941, 1.882747258180256, -0.3296421471846287, -3.6447861352968101], [0.85194634533563462, 4.7342391761560894, 0.16461861702240643, 1.451025761286707, -0.73776697416841319], [2.9359522861728156, -2.2294848830363567, -2.7252773610524583, -1.7019626085405681, -4.8418968220981533], [-1.7682151459372175, -0.16345081282332075, 3.4197052305114326, -1.2717938328418099, 2.1885043911922715]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[-3.8107961235708512, 3.399672096633493, 0.50516209829005376, -4.1294630691493319, 0.058948405855085539], [2.499420141148553, 3.1894609109521888, 4.940841851201597, 0.7742589407021585, -3.1156357592936521], [2.256314869827996, 2.6690957017091357, -3.0127200494342321, -3.1243276173042354, -3.5577182070266975], [-0.33457593221653337, -0.59063469296034299, -2.1569113130327588, -1.8038943306353508, -0.32547291320668492]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-1.6092480957301676, 1.6930838227363376, 4.3926597622281776, 0.81888595370703676, -1.1039515083214462], [-0.79616484007197885, -1.230131420016398, 0.76385796062535971, 1.5924162247720406, -3.9943946938093022], [-1.0319682449781586, -0.90863160107970786, 1.2700727352674892, -0.79487356565347145, 0.9046224977002123], [-47.852634752824713, -1.654830226639151, 0.98288436664559498, 1.8521894990129792, 1.8096361484771588]], [[1.4017912774766952, -0.69544274053488886, -0.76999730911386621, 0.80066144512619564, 1.3538734078456758], [0.75554452371245395, 0.72395011401687193, 0.70208460862118449, 1.3572910607699329, 0.69034595923965858], [-1.1460890787917473, -0.79143885739605335, -0.73117318803170617, 1.644959297094069, 1.756538294512868], [-0.69396533521848769, 0.79060721584189098, -1.1505031105368784, -1.8433555248881717, -5.0445781592401593]], [[8.3366935495929333, 0.13518293448663507, -0.23169650876291198, -0.67924890073026456, 0.11354574843685543], [-0.41896847587710129, 0.26568272302197843, -0.14444099597312285, -0.12085486870274186, -0.27147470074174829], [1.8894987164487362, 0.2531864465415658, 0.19053704578644431, -0.1211926900245702, -0.19049767333374948], [0.20987517312751608, -0.23032293652679334, 1.1057656201331896, 0.20486707912966895, -0.1913679881616509]], [[-1.5393987620564031, -4.5650127640634492, -5.4525609340257883, 1.1375404564922633, -1.6774525542025187], [-1.95381761596612, 1.5190802350841262, 1.9096557246475394, -2.9465296413692315, -0.86776304300277229], [-2.5375548549215674, 1.3327553918435526, -2.6135026847168668, 4.7743791539959313, -4.3889069976321124], [1.2143505851658152, -2.3390855847491654, 1.7818450514973732, -3.8712236451014244, -1.2279612466015302]], [[0.034110303923504504, 0.03978942303587036, 0.033831005863674456, 0.064068683026072271, -0.032082190585891887], [-0.011996451120465604, -0.014780437127890822, 0.023897287543051995, -0.015976200936626349, -0.019320514932640399], [0.017748777349117711, -0.015138825216776054, 0.019138151530809333, -0.19531237947697663, 0.013570400456548859], [-0.015403675521504108, -0.046188571773698221, 0.03146446108159192, 0.018631901513138465, -0.016137406564196252]]], [[[-6.7295742957344231, -0.51583015741713545, -0.6366181902896213, 0.90385275164336487, 1.1488247986850131], [1.3379081636576304, -0.51922105554332509, -0.70739931149056712, -1.496948575625358, 0.73976574991038324], [0.86350012724264458, -0.59848828446582392, 2.1897893550685508, -0.8296651173309485, 0.52128002533168039], [-1.2892299444436142, -0.53454977742934562, -0.62176047174173987, -1.1893645194059714, 9.9905029147856581]], [[-0.78103353528482611, 1.1212615111355102, -0.86677672851668852, -1.5290334484172121, -1.2623258528301675], [0.80359479553480617, 2.5630797494412034, -1.3736132702282977, 21.506723342202875, 4.8738408322257998], [1.7228303040898851, 0.89468814468570146, -2.2085630740453337, -4.5041013246498958, -0.72088052100825661], [-1.0485982632316866, 0.83179196016223012, 2.7134669691920577, -6.8503286639497416, 8.0017510069421789]], [[1.6827485007644107, 1.1754090494122071, 8.3193219349537859, 1.2035008834157024, -1.9442955979745948], [3.6625483463197344, -1.1268060390296946, -16.922207236694128, -1.3424947999082466, -4.3440146736683642], [4.2207320590402517, 5.2834494172020365, -1.7416984772469695, -2.651424845398402, -1.0693057382141049], [18.179509166021599, -4.3773947566735023, 2.3389171552158139, 1.229622332687379, -1.0043108138641574]], [[0.16279200014815409, 0.86532876303600292, -0.21143925882225204, 0.29414974974921682, -0.29017142277753999], [0.34260306889909004, 1.2096659338356384, 0.16017490741359727, 1.1844791043927219, 0.31239395253809116], [0.28570916664227858, -0.76776225143504806, 0.74272568152512985, -0.21164088444432186, 0.23044238501216635], [0.3168479578134471, -0.26636615011216064, 0.27253767070553414, 0.24881191621734522, 1.1111535232185568]], [[-8.4455137552559805, -0.72182169249511796, -0.76630563167883503, 1.1449326456861464, -0.67131526114099083], [7.6963235267652168, 5.740655581055556, 0.69210825612575155, 1.2755214924484166, 44.161414706434613], [-8.871171260356645, 0.79917869276326081, 1.0334001694198633, 2.3098349864927261, -2.2240680963747272], [-0.83286321536437269, -0.74795636797917964, -1.8414056355032502, -0.98704525816159083, 29.958941592231746]]], [[[-8.1635625439154857, -0.82824581991383284, -0.74366835685206367, -0.47630110878240928, -6.9475360278403153], [0.47414419360324483, -0.64736567055709426, 0.45435647478243507, 0.93461738372382219, 1.5910722570825724], [-1.9880327900170227, 0.45806706033206579, 10.394875513236396, -10.046074441157947, -134.38662320806714], [-3.126712843764087, -0.77020985272451425, -2.2183390408641004, -0.66026121226616086, 1.0992145989974025]], [[-1.1911516862885529, -0.70741384177832711, -0.56706251530463081, -2.3438043742341743, -1.4988753509987096], [-1.4085933181242223, -2.2225276392315316, -1.7594449070423572, 0.92939836203098547, -0.98674964412351107], [-0.63928857259095728, 1.9617030586203381, 0.64322827396768167, 2.9327712816350293, 1.3921179534008876], [-1.3875899796260189, -0.5875395245674554, -1.2694708586868846, 1.697691075335187, 0.92422840073483115]], [[3.4509061593405197, -7.1150278090631991, -1.0237609544777468, 1.0847030601263343, 13.215570823964976], [-0.70859726473620754, -3.8140566966647014, 12.192736369676043, -0.61763569717775724, 0.72988010147053028], [4.1114502210210109, -2.7174178996198477, 0.60754980914463907, -1.3312356627402357, 1.4327519941731757], [0.72203840754456217, 0.68182350111211165, 0.67742826086136276, -1.3529089385777839, 0.66331775870388199]], [[2.3242091417295865, -1.0864963093618025, -8.0752190808583926, 266.63755161295148, 1.1172430950303591], [-8.732640956301152, -1.3707313722455152, -1.16742821727462, -0.87588321885525466, -0.63967097465451417], [4.0790506665801747, -1.44320639808284, 0.62704852020310708, 1.2067170732424146, -0.9706972363437727], [-1.508518195565246, 10.040426373979003, 253.83631673510459, -1.5046991283570956, 8.6358642381389039]], [[0.79318471690176007, 1.4805872004338714, -2.3660488842253478, 1.3712642099759176, -1.6082168810578448], [2.6614859701491032, -1.0883871508094913, 5.114761173887552, -0.76996266944740066, -1.3026385919343793], [-1.5090037271193342, -4.8849890860292753, -4.9867665029015562, -1.3033793836529382, -10.403733510060015], [1.7818732672904125, 0.93622552490868249, 0.96648056169568619, 0.80980695499872712, -1.5701442983994456]]], [[[-0.13350618732657243, 0.10505609869989524, 2.3223999760622491, -0.16496470180470399, 0.12701616435631863], [-0.070427946584751405, -0.15911239867988236, -0.36526200066208875, -0.077954993198436073, 0.14551007895936063], [-0.25894517115451926, -0.1127321067005793, 0.24778360886945908, 0.071915542250503159, -1.4570861252952048], [-0.15178734345718489, -0.73436514723869994, 0.29704674788113067, -0.06760086704824661, 0.10037653896521816]], [[6.0216355440671512, -0.32158069306569737, -0.58264160226324446, -0.13987759444061276, -0.19904185932394505], [-0.21466773496656935, 0.16408849810642978, 0.14785718986693364, -0.44254156722828963, 0.2932513837443928], [-0.17665115001290027, -0.22381678224181606, -0.23822104672657657, -0.17337733987725398, 0.45930467091221561], [-0.12797649876520664, -1.7786621668236855, 0.13186803275638301, 0.43575065136816277, 1.0665104122321543]], [[-1.1720480968967626, -3.4094641697800347, -0.66919359418687074, 0.43420781933444297, -0.7995229036174859], [-0.59481103554815873, -3.9030888609042318, -0.53440172725511137, -39.137130212638326, 0.9668030167921513], [0.69447157129663928, -0.45503155392802491, -4.937869448232429, 0.63054417962352849, 6.6614186896151466], [-2.7993329275036629, 1.5948768201053507, 0.80007415133327731, -0.5795976568475607, -0.44431398420839768]], [[0.68749137245859127, 0.76480440298397023, 0.51552385306458426, -1.0750968588377383, -0.38882750338321731], [-1.3252007632934752, 0.38188562747166577, 0.36909212966314919, -0.51375639334602363, 1.1400069668780601], [-0.53156097667038804, 0.43078321360168975, -0.58807335681403994, -0.62897080135501693, -0.36808342335348954], [0.48733592583895757, -0.4627988520492039, -85.091619518684141, 1.6403465936634374, -1.307261714591154]], [[0.13969969853040379, -0.12045520576406867, 0.11223554381049297, 0.16053227516879448, 0.10756018248036125], [0.16154888749757929, -0.11486598426476786, -0.26782375170581046, 0.088357820716739977, 0.094290940363304362], [0.89386889705812578, 0.16923212002286725, 0.4273253517054002, -0.69855869810948634, -0.069280954797408409], [-0.16255056306891047, 0.072897695370011381, 3.011452527007445, 0.07361232179551877, 0.066312031169834876]]]])+(1.-msk_ref)*numpy.array([[[[2.6436119561953015, -0.99402062354855469, -94.284015750846265, -2.8740719374152319, -0.81074516613672321], [-1.9600091412108653, 3.0528123179686699, -0.78879513937034507, -31.786194238844747, 0.83470373559873767], [2.4097178536706165, 0.88800086509325504, -13.630791446728391, 1.4742525768405581, -0.82534034160453063], [1.7364405566941794, -41.54765018431825, -13.472187276303918, -0.81341352542851275, -4.607368203749612]], [[2.9774963795570994, 1.1712210357050823, -0.72713585803110004, -1.3042130002597274, 6.9792595335753642], [0.87895105718838717, 6.6479575507707143, -1.7368160080564172, 1.0322079490261915, 76.556579486779682], [-1.017916968528237, -0.88463803268189745, 0.86723479197115638, -0.75780782497573929, 0.73020761777881782], [0.91384499912761008, -2.1741673142932099, 0.80697131573226166, -1.031098009653016, 3.7780749752594698]], [[-0.28424245954735111, -0.36664586905418173, -0.5913356662715874, 0.22941920021463338, -0.21272040852475393], [-0.22837105991840628, -0.13320873691793367, 0.23226768542769899, 0.12169283836379569, 0.10440926543957123], [0.30546189328291901, -0.12436496186648753, -0.21761106162544938, -1.400923100155399, 0.58702532979062305], [-0.29354228671061605, -0.12474787344012876, 0.34459974914849045, -0.12393998399323236, 0.99446555930791669]], [[-1.3447007381347591, -0.99159397957318207, -2.8270468872768544, 2.9389777595767765, -6.3875155167786639], [2.6684040426860918, 0.84713705755877067, -13.753848347151074, -1.1616970910864697, -2.4811713124203831], [-1.2344614849961186, -4.0160610092315725, 10.196675181356866, -2.9322754165845422, 230.03526007623492], [-2.2904831261668903, -3.6347530223590461, 1.0895355024094606, 2.1589995568452744, 1.571426110477202]], [[0.038554661185528104, -0.0491233689150938, 0.013906220204692201, -0.013578866138248302, 0.014699995801850736], [-0.049933178684731697, 0.02100776623855044, -0.94753501102411342, 0.021050226186580912, 0.013902082930034702], [-0.021911051434380692, -0.014791137711964115, -0.44665040961794655, -0.036783752593051236, 0.022732918971957023], [0.027264166419645228, -0.019652422843783757, 0.014701665061103545, -0.016737510052522699, 0.012092809897138936]]], [[[1.1010039229887607, -0.61912270268896119, -0.72579511594296076, 0.69102648425499125, 1.4091520898523069], [-0.76710483509925576, 8.6203319495316659, -1.0414020839671947, 0.64817727753645815, 5.5011832299643837], [-1.0715307384454376, -1.0066458030905667, 1.5808149966447056, 0.73569920630646812, -0.61359418815393485], [1.6687604691785389, -0.99448860201463241, -0.96759723245491935, -0.61585262434927079, 1.0441470967914686]], [[1.0015320374124614, -0.9565768971408497, 3.504762516920692, -87.004895819537992, -1.2475398872099484], [12.135144783762843, -1.2515392296052736, 1.1169775942170195, -1.521939657260049, 0.64052403363463961], [1.5608603085186283, 1.5776567024161634, -2.7409676825656351, 1.3938342459008639, 183.12419048841593], [-0.8331144485647064, -0.80728189506466708, 0.75681850145717655, 1.0171447770857927, -1.4135807841518726]], [[-2.6629175618707306, -5.0576284091272461, -2.3370765319640148, 1.0733229308997487, 1.399517031618662], [-33.969593400165863, -1.4599245183270608, 2.4953764786404813, -52.167551987793757, -1.0704460161031917], [32.830481231628475, 1.3544511849680059, 1.2692573528595901, -1.2407554225832471, 8.3934195926012567], [1.2764644090516635, -3.2256432942743145, 1.4206857203948184, 10.415935432906297, 4.4799460202163308]], [[0.33540464834768929, -0.25073133569879485, 0.27565497247789073, 0.38862171004367679, 0.39114165734517853], [-0.17299978119591991, -0.38548525768389169, -0.36222311338804147, 2.1104981048203157, -0.45801662115596536], [0.36175355743062809, -0.25204699625193627, -0.47224662694585906, 0.16662451562790953, 2.5232365109694581], [-0.18403698716639647, -0.15916232378396458, 0.25567639749258486, -0.21546565897042955, 4.0087983542992767]], [[-5.1618020272469334, 8.3973336657940365, 0.73010747884889426, 0.67717703045156807, 2.1081085384405762], [-2.485758190258343, -1.1794134153637728, 2.0069028472538877, 1.0669204333307738, -2.0043419316582445], [1.0790232152870325, -0.73227073291083444, 1.6857078777816681, -4.0116994540557656, 1.6161074047681456], [1.4514505377278231, 1.218101894702917, -3.7180206315235851, -5.4136249008052681, -2.4449269394323663]]], [[[0.58152903261529321, -3.8035724273923459, 2.7892203485928544, 3.1576189756873725, -0.65393726579024258], [0.54801137319056614, -0.58960921255264775, 0.83756381561816573, 1.6633865813900681, 2.1635563861161193], [2.8229424764412947, -1.056687418187001, -0.51505471168422812, 0.72156764275062291, -0.54565869896237129], [1.2504739677173475, 1.0950299949917239, -1.0286772350780828, 0.68109742586436473, 0.5654560498418455]], [[4.2960150658729068, -0.70483769785834405, 0.69469889391031581, -0.6210190328185704, 5.0181899657283804], [-0.71743509270440153, 0.62020162930175993, -32.08511999494948, -2.3927629270426585, -0.70233166489553478], [0.81712701044959868, -1.7792895442185563, -3.4775761291835083, 17.923405788911598, -1.566556364035911], [-0.76380442945035731, 0.68933207202149793, 1.422202700621005, -0.55498225386012445, 0.67669704765388961]], [[0.63938692016458731, -0.85535380121748861, 1.1761143413649269, -6.7110304274916652, -0.68038320158752985], [-0.64446453528485426, 0.83127969019752723, -0.66834964142710429, 0.86302547121026563, 0.76129905482025795], [3.8447080237536602, -0.82615815852353758, -0.93833673229577663, 1.5837369952734104, 1.0361295130045038], [-1.4102230533624827, 0.72816315266446086, 2.1619862813380024, 2.2925322574094156, 1.0240455781522784]], [[-0.76016043559182711, 2.7162476277274243, 0.97445420941486671, -18.894484616105466, 7.4712817684395443], [0.87665324969248803, -1.0638749487225585, 0.6410266724019501, -1.1937832552892582, 1.5680561728587408], [1.5894903040144421, -0.82190080964734147, -0.91280594975624418, 4.6491239440679637, 2.3288948368117492], [-3.4812056004730509, -1.6568650938089704, -2.1277211552529862, -0.98723419841476978, -0.90066447360919322]], [[4.6037265899717079, -19.372009086478826, 1.3765832899997297, -2.0424693384664989, -0.90586163675094222], [-7.2981280713811376, -44.68773036013927, 2.2463144912435502, 3.6240863003218369, 4.9288562612576534], [1.5529319804082513, -0.85783986150827307, -0.87852511199796202, -20.203080321897183, 0.91426145523667013], [10.644355374374181, 1.6534696545676417, -22.573870439701796, -2.9923943762876424, -3.8194646743464684]]], [[[0.11485875652172529, -0.20949871745773949, 0.35661184922008227, -0.10613160824174506, -0.13780742604052315], [-0.10059221469715567, -0.32191226104746368, -0.53543424594316358, 0.16155417121321766, -0.50786083285242523], [-0.067847464031136201, 0.072223297071327186, -0.11643759259567375, 1.0670664916295332, 0.093816888128890982], [0.067361365086156297, 0.074222592429938089, -0.14809204073661839, 0.1159512819014852, -0.092240264549850223]], [[0.88819132205054474, 0.65662784673105312, 0.36158044711972498, -0.13233843288545707, 1.0636478379382972], [0.72192861611263814, 0.25037030301613794, 0.13835086274114211, 0.35102490748768334, -0.19469621003491197], [-0.3842827739285396, 0.54161181805118286, 0.19971954616099388, -0.19058204098661752, 0.5823741656582887], [0.16299934660236937, 0.12905201463451788, -0.12614600768542947, -0.95281725005495621, -0.2685049663865805]], [[0.56872391117841614, -0.6274015527913086, -11.990606333065731, -0.60504753711126091, 0.44369677724945983], [0.60155743629260816, 0.44529024671468936, -0.69092888570302369, 0.82070778430342151, -1.1757259099268973], [2.7812076664463956, 0.74440367383890671, 0.66458049296611477, 6.5395070945857272, 2.345223873583373], [-0.69136928480004545, -0.52515703785988455, 2.8419644355658189, 1.8016211703988634, 0.64574664048732666]], [[0.65445833674425824, -1.6450797552866434, -0.60889390532627619, -0.40504312238431178, 0.50010492657525019], [1.9737584772676349, -0.43369976902200214, 0.56442134418950762, 0.722066978963954, -0.4154596815556581], [0.71557607149487368, 87.784216977966409, -4.3164132965483732, 0.42715278035938115, 0.43512982023203561], [-1.4987664448512794, 0.37134990088316788, -0.49969821346699422, 0.39111292852953827, 1.7245551432815147]], [[-0.35999070803554079, -0.072466112585892847, -0.17287127190999943, 0.98735224238298436, 0.089298219737707685], [-0.38203452011811956, -0.068748726267553906, -1.9771330794401245, -0.22430539959405654, 0.44115950510464003], [-0.11085769845086882, 0.14598570085993148, 0.11942744538889521, 0.19123387997682142, 0.067220125741061704], [0.18406861515382653, 1.9912590680016935, -0.095175721668270499, 0.25591641097945816, -0.14871933294562462]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-1.2477855299909546, -1.0793898590883524], [1.8718797269672063, -4.4935879144820579]], [[-2.8993410557032808, 2.4975873134172684], [-1.3761214367044183, 4.6630001250449116]], [[2.754292339885871, -4.3388044209495202], [-1.2551182444555686, -0.070885237562180237]], [[-0.79492404806016825, 0.88101409850532253], [-1.2013801152443482, 4.6977340999061283]], [[0.76118919290234199, 1.1580831492685482], [-1.3180568649485536, -3.5358468659080824]], [[-1.2982997521166562, -3.0310356076973166], [-4.4569354211812806, -0.21344573014686929]]])+(1.-msk_arg1)*numpy.array([[[2.9212851410545309, -2.1839739346997411], [3.3727498220093146, -1.0296833961271235]], [[4.7139232786441241, -3.4361755073785396], [4.2446311529042458, 1.8451270534574107]], [[-1.0594893543577246, -1.5672288175450433], [-3.6760751212409968, 0.8761927062503112]], [[2.8318058139673656, -4.564843877221703], [-1.7900377303663371, -3.1043591931284986]], [[-0.95526576670442065, 4.3299548098263365], [1.5432334782353054, -3.7784569354070663]], [[-0.53980310281280985, -3.3471957813139799], [3.8212421364699285, 1.5153445897731688]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-4.2289057956957157, -3.6919465309396413], [-0.45333144068629316, 1.607657679646878]], [[-3.776206602115102, -1.5082074099718135], [-4.3643373212036902, 2.6659831723019813]], [[-3.0101010739647305, 0.4659284064581275], [4.416260793147595, -1.4197795662060466]], [[3.6410665210440492, -2.5169589413752913], [0.99844132781205985, 0.39923523218880774]], [[4.2082492194450083, 0.049082302679659406], [-4.007637906890487, -0.31758904256299925]], [[-4.3376735311727179, -3.5799163119124868], [-0.36905622941208271, 1.3792489302551658]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.3891287357101918, 3.420401349756836], [-0.24217979080353336, -0.35776704723316416]], [[1.3024361499958561, -0.60386573949570643], [3.1714768804528992, 0.57173131048893211]], [[-1.0928763916503719, -0.10738635837292754], [-3.5186013848944024, 20.029270057261527]], [[-4.5803954854922875, -2.8568883808391012], [-0.83107861961656271, 0.084984638061312456]], [[5.5285193992302419, 0.042382364954243627], [3.0405652544034307, 0.089819795541805927]], [[3.3410416385745139, 1.181086854546112], [0.082804930863069781, -6.4618248831031764]]])+(1.-msk_ref)*numpy.array([[[-1.4476182883568693, 1.6904718835150478], [-0.13441004065229514, -1.5613126187075066]], [[-0.80107510854552144, 0.43892036560217085], [-1.0282017833793495, 1.4448778295816787]], [[2.8410866627248845, -0.2972944354022104], [-1.2013521616110829, -1.6203964676697995]], [[1.2857754945925857, 0.5513789757268075], [-0.55777669424192844, -0.12860471593381179]], [[-4.405317730544331, 0.011335523079426313], [-2.5969096467977351, 0.084052577015485894]], [[8.035658758851028, 1.0695270147918117], [-0.096580173731941943, 0.91018830935452444]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.4818539886244526, -3.3233594046541395], [-2.1836227054603627, -2.5133134324831219]], [[-4.5798054992179074, -3.8440859393988358], [-0.60029234852218316, 3.4836095275576557]], [[2.9041203641785778, -2.1251742118587016], [0.27170467723916403, 1.1712639903460964]], [[0.49760903655143451, 2.6057568201509991], [0.12319589138576337, 1.4801125815852494]], [[-3.2300033878814824, 1.1458333264585754], [-3.1004042216583763, -0.31229036855368975]], [[2.1359375104477873, 1.352928996239128], [4.2668355595205902, -2.9506914626828085]]])+(1.-msk_arg1)*numpy.array([[[1.4983459962585668, 0.25142184408940871], [-2.4336834160426681, 2.5451846932146012]], [[4.3155799365556415, 4.2197266654668368], [3.1143608849968736, -2.542964439519555]], [[3.9300400624586977, 2.659369253693681], [4.859506402961161, 2.8059003308940467]], [[-0.31445246937457316, -0.96817926207313576], [-4.2711345785469099, -2.5444207224338058]], [[1.9601772003532716, -2.6622339518595197], [-4.6604655440079537, 1.671848077301795]], [[3.8259130278027378, -1.5204871314697899], [2.4731993542680009, -4.3217355960157171]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[1.8207140442807104, 1.6725422474360023], [-1.0486716289803999, -0.3152326546100479]], [[-1.4535711435975562, 4.4942683079669195], [3.8973967923323727, -2.4715323811682754]]], [[[-3.7271152807335062, 3.4183945082340461], [1.7451292680520165, -3.5269599471480131]], [[-4.1159974280778755, 2.1538329605565556], [2.0683446973277606, -1.597012649460626]]], [[[2.9242467741233753, -4.8675151098723477], [-3.5760426963408012, 4.2479333585418342]], [[1.4347405170323357, -3.0510582572906131], [0.59066184622151763, 0.45102773363770243]]], [[[3.1640324931651449, -3.0332878115523609], [-2.0662122482626213, 1.7525989202684054]], [[1.2329737355581099, -4.3395732247772179], [-4.3910330198835013, 1.791107259988661]]], [[[3.1174970014732235, -4.5693323115973214], [0.19213006859921933, -3.4076896340721174]], [[-0.68716860159403659, -3.4093421922357727], [-3.7730430453373867, -0.54849517942649406]]], [[[-4.4054650309872727, 2.093201229737339], [3.4864806341921568, 4.0832802720072134]], [[0.35833049520278593, -1.6135832218592485], [-4.0679993565336856, 1.024623635590034]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.40624126731971349, 0.37318088712419889], [0.31554565765947745, 0.094853615341327802]], [[0.66566954994686534, -2.0581707163643981], [-1.5507006575307218, 0.983376107900093]]], [[[0.8138151896123067, -0.74640604471473837], [-0.45397769341362115, 0.9175028869670856]], [[6.856654825287638, -3.5879733697404657], [0.59373608923898791, -0.45843618144548048]]], [[[1.0069302946920005, -1.6760720973936321], [1.6827056701451095, -1.9988635919059752]], [[5.2805146072970492, -11.229318126919708], [0.50429437862849613, 0.38507777696164669]]], [[[6.3584707285316755, -6.0957249341247248], [-0.79294131834715409, 0.67258729084583013]], [[10.008237463839588, -35.224983365627914], [-2.9666885306660662, 1.2101155562574342]]], [[[-0.96516833795581547, 1.4146524826385052], [0.16767715178352799, -2.973983698488031]], [[0.22163839050202194, 1.0996444168212842], [12.081842494251358, 1.7563627785472204]]], [[[-2.0625439693053997, 0.9799917925962689], [2.5769871470593619, 3.0181038941126372]], [[0.083980385511516389, -0.37816859809815268], [1.3786596829866467, -0.34724865291691065]]]])+(1.-msk_ref)*numpy.array([[[[1.2151492704803231, 1.1162590293646533], [-4.1709646700685221, -1.2537997871734139]], [[0.59727207491973633, -1.8466938954923315], [1.5312825048503298, -0.9710620953195731]]], [[[-0.86364181304174803, 0.79210547794008457], [0.4135645283221086, -0.83582663683213199]], [[-1.3216186498829623, 0.69158104667074183], [-0.81335966212666944, 0.62801218319920804]]], [[[0.74407556351828086, -1.2385408373743525], [-1.3446958113747927, 1.5973461950203969]], [[0.29524408408188751, -0.62785353167380076], [0.21050706602729347, 0.16074260681026795]]], [[[-10.062037354828897, 9.6462521588251029], [2.1341215714931732, -1.8102008470162987]], [[-0.28867592741074011, 1.0160235284025145], [1.7257495905328746, -0.70393518029338309]]], [[[1.5904159077614894, -2.3310812465188437], [-0.072168739514804903, 1.2800113347258273]], [[0.1474463430971058, 0.73154541323006383], [-2.256809752370994, -0.32807716614521193]]], [[[-1.1514807051213545, 0.5471115559935994], [-2.2930023951086813, -2.6855079451149848]], [[0.14488540706773795, -0.65242747984495775], [0.94128834727511901, -0.23708614579167042]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_expandedData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.2214033882620896, 1.8437102723927801], [-2.7572900819092752, -2.9961412391535172]], [[-2.8059849115265258, -1.269044719528444], [0.67341404132531402, -4.2733638424926133]]], [[[4.9355679245496127, 1.7080688550501684], [-2.3435999978819275, -0.81218495138861702]], [[2.4330407279031334, 2.2352671587230546], [-0.18327930257443192, 3.5261370343927876]]], [[[-0.1021063968779341, 3.9043368886982215], [-3.2581027236628914, -3.5497963957383449]], [[-4.1959527792213294, -4.1523319829495247], [-0.86234407697389681, 0.85284263592114939]]], [[[-0.3989564006510582, 0.39928096017302739], [1.6711794230025081, 1.4260231309474589]], [[-0.071723973416448317, -4.5227673393318604], [2.5808887044681592, 3.3106742772906514]]], [[[2.9026566089355574, -3.6455911450678813], [-4.6193540683463752, 1.0243348756405615]], [[0.13414649728744088, 2.8613534247012842], [-1.0982414122146658, 3.1900109174801017]]], [[[-1.1773335990582199, -4.8075828519502695], [-2.261147525231022, -4.770772357356603]], [[-2.1789003925285035, 2.07818708464404], [3.8126956435740826, -4.9293172055276919]]]])+(1.-msk_arg1)*numpy.array([[[[0.25556167317476941, -1.4517795016158583], [1.3625088850430167, 0.52044192315976456]], [[3.7766394771087359, 4.4031716844831834], [-2.190736165530689, -2.1574648945684762]]], [[[3.5333181416580715, 2.9859457596611065], [-2.8443121709518291, -1.5953091054085142]], [[4.4029477083124444, -4.0671633589023184], [-3.2541246982381979, 2.9465765231742891]]], [[[-4.847105712056468, -3.6998194793280437], [-0.82795752958551372, 2.5755050969606073]], [[3.6645373787211568, -2.868486093542387], [2.847225913617323, 4.845847700822004]]], [[[-3.1998210014968134, 4.3050951758916387], [4.0345506990393893, 2.0928948979668771]], [[1.9624714094012603, 0.26733452377211808], [-2.4594013518720392, 1.3342630786399763]]], [[[-2.3302434127138563, -1.9732506968892016], [-0.69212595228453022, 4.4420450009855035]], [[3.9087295814781733, 3.7327999798824933], [0.099039645643490104, -3.0091721284037911]]], [[[-0.93274793893162311, -1.9576017411961546], [-0.1389657314472279, 4.1390342764130992]], [[-1.0668756454296338, -0.60240995969476518], [-3.1271745501467283, -3.1255583780334395]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-1.2868080233216128, 4.707648038537485], [-2.4405544297914328, 0.86117832123992955]], [[-3.9889040598126311, -3.281921572106774], [-3.2822854148237512, -4.8326774944718736]], [[0.7070647704230959, -3.0363380655705052], [-2.3742868732574185, -2.3739202796679368]], [[-3.1878798619808291, 3.1856052784409368], [-4.2196600700023836, -4.7407176413398897]], [[1.7366830800164319, -4.4484197535908159], [3.0180944679915385, 2.2524728679385033]], [[-4.3041597810419887, 2.4641602620362306], [1.5948894686139212, -2.0325726394567791]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.30482943821471159, -0.69794481410117892], [-1.7073459442750005, -1.5712370221463625]], [[0.86976748155916139, 1.9231429690659778], [1.2788244206269974, -0.20152234936719365]]], [[[-0.80819555536288812, -2.3353297778475515], [1.4003761627721762, 4.0408549388850092]], [[-1.3490466382996071, -1.468408553319787], [26.367830009115544, -1.3705302565769615]]], [[[-6.9247842646761484, 0.18109727479455404], [0.93193441800291998, 0.85535555481878778]], [[0.56585166663816233, 0.57179601318170425], [2.7528690032850953, -2.7835384626425039]]], [[[7.9905469790145442, -7.9840517829835145], [1.9062018324265568, 2.2339085596209087]], [[58.831933996488509, 0.9329819009937721], [-1.8368547365612977, -1.4319492780846865]]], [[[0.59830814112500086, -0.47637900436695696], [0.96299605697539659, -4.342739722504346]], [[22.498496263562892, 1.054778637947047], [-2.0509815445733963, 0.70610193074756256]]], [[[3.6558540285310803, 0.89528561724026046], [-1.0897830568505107, -0.5165118092957125]], [[-0.73196988448064515, 0.76744268136335769], [-0.53310644999489454, 0.41234364815830282]]]])+(1.-msk_ref)*numpy.array([[[[-5.0352152078829571, 0.88636602313875545], [3.4551319923237465, 9.0454819818432224]], [[-0.64622383062622557, -0.55427192139519987], [-0.39309997013324322, -0.39916214785602688]]], [[[-1.1289399651798036, -1.3358930070669992], [1.1538542096834969, 2.0572323952644682]], [[-0.74547453939256092, 0.80702079685080641], [1.4850929028897735, -1.6400990968548561]]], [[[-0.14587360219199996, -0.19110791063555135], [3.6672630625033817, -1.1789291619549633]], [[-0.64790903404183386, 0.82771427011707566], [-0.83376604164575607, -0.48988751323431962]]], [[[0.99626818515460758, -0.7404899849445441], [0.78958117423073082, 1.5221047562090015]], [[-2.1501765833570943, -15.784194313785362], [1.9275900770451173, -3.5530606499073154]]], [[[-0.74527968646582288, -0.88011274125192773], [6.4271824209274673, -1.0014350941073082]], [[0.77214204899029593, 0.80853367023607481], [22.74314344829806, -0.74853573402373719]]], [[[4.6144940142907283, 2.1986902087714815], [-17.732143287224684, 0.59534666723554652]], [[-1.4949159964858461, -2.6475151065265172], [0.64997095840441976, 0.65030704713173437]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-2.3075406196765091, -2.7498523545941298, 2.0807793940167771, 3.9243688484158046], [-0.0048404526608134546, 0.67080471710025691, -2.0528671662913442, -3.8831934078069139], [-3.7365710905051976, -1.8322835097630596, 3.4512304770908173, -3.1245182761256252]], [[4.9492651278786077, 1.657103547354132, -2.7642633047911214, 1.6211044229926719], [-2.2724275558667806, 0.22701254037533847, 2.090769349531552, -0.64862538528681135], [-4.3939842024252886, -3.7111108751187363, -2.8858329610622793, -4.4203794961108747]]], [[[-0.77671565086876004, -1.8287900382867406, 4.0623774906682382, 1.9609572501702521], [-2.5349395566684874, -2.6583426494272788, 0.83853560175252273, 4.3619700111459281], [4.5430278376409969, 0.71539606300053737, -0.99906391245707837, 4.1896787917603486]], [[-0.25045744446598572, -1.3830707833505596, -2.3338415734165832, -4.8894017243233066], [3.9224587818950312, 0.10571905025881989, 3.2089804861596463, -0.43681929034910549], [-4.8901204120806181, -0.50040568517328321, -3.7732247371046346, 3.4259903537444316]]], [[[3.3581914231138015, -2.1822140881618379, -1.0523849972019272, 4.2201650240201385], [-0.84887323050464847, -4.0100544481991278, 2.415152368525697, 4.9526967105578183], [-1.8854363443765632, -0.30912825723373416, 2.6429311897690742, 2.0777838329226173]], [[2.9100598633368406, 2.5082880916851602, -3.6542139040531652, -1.467162468384374], [-1.2714873687462482, -0.41088990511171275, 0.25806726619125531, 2.4763910681151806], [-4.0330319358646385, 2.8640560697501103, -2.3518165459293847, -1.2598973085844123]]]])+(1.-msk_arg1)*numpy.array([[[[-1.3586383064015406, -3.3291069831604361, -0.94282227738494129, 3.4109435200118234], [4.7182877317650878, -3.8968569368497796, -2.6461987880842575, -1.727206922574922], [4.022393907849013, 4.2101006647574017, 3.5771183704662342, -1.7223239335314808]], [[0.59532462734066272, 0.81208949787724727, 3.6714703792636243, -4.6480394056205281], [1.7504079128761614, -1.1210354507778444, -4.4907328415891214, 1.3397133497131533], [2.1149397751262553, -2.9189791136445855, -2.0382463321315756, 1.4108888582557384]]], [[[-3.9481513735041451, 3.1406085350496369, 1.9285491213321757, 3.1607719933024807], [1.2208436780632166, 2.4756690654490123, -4.2675391927501138, 0.27530189566167795], [-0.19412996319143705, -4.464212452229348, -4.9119191598018883, 0.86907714621151566]], [[-3.4733643391181781, 2.8923825549945388, 4.2443820674372326, -4.8753098053040587], [-1.1280668348712783, 1.0616346840650088, 1.551212022828496, -1.145181267495917], [0.59569309793852554, -0.23379112156658977, -4.0743867419364586, -2.6504598725519624]]], [[[0.21501677502668759, -4.7111557367391033, -4.0701881885515832, 1.8090607564017969], [1.303492746933915, 3.1367847102285609, -1.1171550829443264, 4.1192981621915603], [-4.4391600628676144, 2.0599389165407604, 4.4900018722686923, 1.0009683869339279]], [[-4.5821026958966602, -4.5215403521877597, -1.9251568117075579, -4.2925372125549028], [0.14316540721141635, 4.0332268972901666, 1.5194786642733398, -1.4755967526955303], [-1.1482178219083314, -3.7256184071086018, 4.6483627439091606, -0.39539788818635913]]]])
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[1.9312282413059201, 2.696586743023099, -3.4232746843718251, -0.62574265025102704], [4.7612386519643337, -2.6438606049644542, -0.37216662522203681, -1.1728018225940859], [-2.3239869207654595, 4.5035478610189656, -3.3284957547189311, 2.7750855154167819]], [[-0.25900602592640354, 4.0146012189040388, -4.1130406991673638, -0.5409328988732458], [-0.81463661082151617, 2.8283009939669599, 1.6228248238548639, -0.9611763640705675], [-3.7973075061174413, -0.33879786345606533, 1.1322785419110843, 2.9441587101114672]]], [[[-0.85818007963295617, -0.47152316453047494, -4.4411354430474077, 1.7628625887013003], [0.73363702347242388, 0.53309713324177999, -4.9380026809567426, 2.6778700996085743], [3.1431388215796616, 4.4556518734884225, -2.9540460068340204, 4.5947912986964337]], [[0.26142771490005767, 1.6807019014718891, -4.7632318637158377, -4.7278198840742043], [4.1572099358929044, 2.1253208666369225, 0.39999497489050206, 4.4588577458764309], [2.6363989387290276, 2.6537976855124654, -1.5937150029564959, -3.4279326802402634]]], [[[-4.184428163001308, 4.9040076582265932, -1.1305812890448275, 1.656593405943231], [3.5203727196195338, 2.8663084033968502, 3.8332490595645936, -3.4127832213550633], [2.617476934981747, -0.72648224433209396, -0.24403185719512521, 0.68471498148643395]], [[-4.2189757207177774, -1.2672367374123072, -4.0600654727491845, -2.8776594533452804], [4.5594594582963577, -4.9427768298243402, 4.0764142931402407, 4.8206613607617541], [4.7525431608357991, 3.5513536628572258, 1.9880512551977727, 1.6304094840133789]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.83692058325571583, -0.98062964672192643, -1.6451886702719931, -0.15945051915892866], [-983.63499978206403, -3.9413267938742131, 0.1812911382349123, 0.30201993550881145], [0.62195710036691643, -2.4578881144879912, -0.96443740191024729, -0.8881642769130681]], [[-0.052332218871737973, 2.4226616528062364, 1.4879337623295481, -0.33368171180154199], [0.35848738443535827, 12.458787471787671, 0.77618548608361149, 1.4818667074609657], [0.86420599874289317, 0.091292843263602458, -0.39235761639311617, -0.66604207007606209]]], [[[1.1048832074814996, 0.25783340605475435, -1.0932355383637344, 0.89898063231528735], [-0.2894100656335164, -0.20053740376795745, -5.8888408203973874, 0.61391300095276768], [0.69185990795331809, 6.228230911420428, 2.9568138434396025, 1.0966929750635781]], [[-1.0438009357536258, -1.2151958682839812, 2.0409405325412875, 0.96695263564757195], [1.0598479594180616, 20.103480512109613, 0.12464861553869928, -10.207556864791655], [-0.53912761170789025, -5.3032924368025389, 0.42237478920469107, -1.0005669386937148]]], [[[-1.246036224796679, -2.2472623950280814, 1.0743038831328913, 0.39254232868011318], [-4.1471124228133567, -0.71478042017211973, 1.5871665529345289, -0.68907575424110401], [-1.388260570444896, 2.3500997638750163, -0.09233379141302861, 0.32954100933748809]], [[-1.4497900108075643, -0.50521977184882738, 1.1110639878650395, 1.9613774993263922], [-3.5859258773386151, 12.029443333441103, 15.795937056655546, 1.9466478549491917], [-1.1784045443758444, 1.2399735118199282, -0.84532582213471064, -1.2940812500387546]]]])+(1.-msk_ref)*numpy.array([[[[-1.4214439797600942, -0.81000302984049377, 3.6308801419783907, -0.18345148390169122], [1.0091030735387514, 0.67845975559517258, 0.14064197553785088, 0.67901639766801747], [-0.57776214214888233, 1.0697007553092495, -0.93049639682041108, -1.6112448195077345]], [[-0.43506687617374928, 4.9435452981450485, -1.1202707020047658, 0.11637872480580434], [-0.46539815366976717, -2.5229362657572585, -0.36137193663041423, -0.71744926948467413], [-1.7954683867490997, 0.11606724483651694, -0.55551604536776467, 2.0867403501586144]]], [[[0.21736250676510571, -0.15013751611135534, -2.3028376067391134, 0.55773165303815608], [0.60092625833660207, 0.21533456982671959, 1.1571077517801458, 9.7270310949817915], [-16.190900002799278, -0.99808239889285499, 0.60140362875051179, 5.2869774780364036]], [[-0.075266424531331852, 0.58107870225177127, -1.1222438951147222, 0.96974757971905834], [-3.6852514473287181, 2.0019323958963451, 0.25785964072219292, -3.8935825029921114], [4.4257671405840249, -11.351148271713111, 0.39115457218453475, 1.293335060734091]]], [[[-19.460938163926709, -1.0409351616172586, 0.27777125692243632, 0.91572016035446935], [2.7007229061306091, 0.91377275400835445, -3.4312595610824643, -0.82848657392146263], [-0.58963337611460354, -0.35267174113690225, -0.05435005689024839, 0.6840525539310871]], [[0.92075101775783674, 0.28026659914671587, 2.1089531242642123, 0.67038660606800138], [31.847494077694876, -1.225514198852854, 2.682771656481076, -3.2669232647440185], [-4.1390606121555402, -0.9532252836418047, 0.4276884926424373, -4.1234653313194567]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s0=numpy.array(-1.87942408008)
      s1=numpy.array(4.05176801604)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.463852834773)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s0=numpy.array([-4.0400497241741196, 4.2735161533712009])
      s1=numpy.array(-2.24199315761)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([1.8019902114618309, -1.9061236377414021])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s0=numpy.array(-2.28333335733)
      s1=numpy.array([0.25997898664577335, 0.86627839318278355])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-8.7827612022976282, -2.6357962697603279])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s0=numpy.array([[-2.0043626432923292, 2.6342663379560989, -2.4419469386658177, 3.7754943995518904, 0.84031802206669415], [0.74904636799525193, -2.0885448881474602, 2.2678725393104848, 4.3140678696918364, 3.9351104577172009], [2.6977956071873344, 2.4963806745218413, 0.46014849138114844, -4.0548318194719535, -3.4569822807558879], [-1.9377471595714049, 2.9193830580394327, -0.18019341841094949, -2.3667929488077233, 0.80972712726695484]])
      s1=numpy.array(0.785499635644)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-2.5517041031468857, 3.3536187904116397, -3.1087817585870638, 4.806487779536794, 1.0697879208793097], [0.95359225390531954, -2.6588744200193419, 2.8871719812474601, 5.4921322352450632, 5.009691003220679], [3.4344963189914934, 3.1780799904206392, 0.58580357074779021, -5.1621052836635313, -4.4009979430774777], [-2.4668975918538063, 3.7165937774715618, -0.22939974792387063, -3.0131050880351924, 1.030843415482217]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array(1.26689591398)
      s1=numpy.array([[3.3412046764235104, -2.8767517256623121, -4.7935211161051638, 2.6558755028881595, 4.9260846935446985], [4.1413785533413776, -1.9671581276596828, -4.4533645575218124, -2.3596676553038165, 2.8484041531415656], [2.6921283715934408, 3.6098531533587099, -2.212738430275599, 3.0956248857301567, -2.7058935180288755], [-4.5511406914604144, 0.18570059585722021, 0.99249305051969028, -2.1265627439700974, 4.026322177558928]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.37917339303393083, -0.44039111984495738, -0.2642933833594372, 0.47701630313719151, 0.2571811068617072], [0.3059116421410385, -0.64402342453657169, -0.28448062080175524, -0.53689591037650664, 0.44477393160063078], [0.47059268322724068, 0.35095497244855606, -0.57254662216109564, 0.40925369214478324, -0.46819873196755257], [-0.27836887494106566, 6.8222501286663544, 1.2764783726364288, -0.59574819392122003, 0.31465338790857289]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.2936238628809913, -3.4347597256202511], [-4.5853159076719274, -1.2578156775286398]], [[-3.5417474134268403, 4.8452233039316379], [-4.1081382394079231, 0.9060356607401765]], [[-4.6884165134925295, 2.1842317081919225], [-3.3592517007242995, -0.4334719652344079]], [[-1.3173382919932406, -1.3547189520397374], [1.2891941981727744, 0.34463130395779995]], [[-1.2277429851793786, 1.3257908449066305], [3.1556162147735698, -4.3854987749183119]], [[-1.2266058141385661, 0.89007994962406656], [2.3706284921503471, -1.9789003312695796]]])
      s1=numpy.array(-4.73822664232)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.48406799337056577, 0.72490405903023158], [0.96772827764743763, 0.26546127327369867]], [[0.74748374883466195, -1.0225815837215968], [0.86702020598146579, -0.1912183036261986]], [[0.98948760104067268, -0.46098084221740926], [0.70896813392660685, 0.091484008249657314]], [[0.2780234867256714, 0.2859126534684896], [-0.27208369195749454, -0.072734237927742446]], [[0.25911444889837498, -0.27980739314287428], [-0.66599098206728102, 0.92555698702805611]], [[0.25887444960609618, -0.18785085999789847], [-0.5003197759640029, 0.41764577354666138]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array(3.91173474399)
      s1=numpy.array([[[-0.015371878903528646, 2.1621113709482467], [4.7058208807934641, 1.3493845469750108]], [[3.7818907155164272, -3.2378411955538877], [-2.0063441730374376, 3.0001171927091974]], [[-3.9534646132736539, -0.36335541255796056], [-0.42331829566519019, 1.0318392525384423]], [[-2.0650459629635796, 3.7177199206449565], [0.59209896366757864, -1.1595184944001282]], [[3.7890153867562546, 4.3446227052761142], [-3.8538631330357829, -3.3605521989090859]], [[-3.3827825093376207, 2.1948925604751484], [-4.4150741132988767, 0.44438560142914163]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-254.47342960089549, 1.8092198193634219], [0.83125449163526532, 2.8989028759516606]], [[1.0343330990346795, -1.2081305128126949], [-1.9496828094396923, 1.3038606470096494]], [[-0.98944473433682456, -10.76558820591838], [-9.2406465396063258, 3.7910311459534745]], [[-1.8942603768377126, 1.0521865087975677], [6.6065556334713458, -3.373585469211775]], [[1.0323881918408866, 0.90036235810308063], [-1.0150165195174479, -1.1640154690233433]], [[-1.1563660191552014, 1.7821987346586927], [-0.88599526159885866, 8.8025686057569601]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[-4.5262661765098597, 2.004539963383408, -2.8991660197400404, 3.1690225427108647], [-4.9516959316987403, 0.0059522024901532689, -3.3989070372101473, -2.5126832913394703], [-3.0332355485844174, -0.79932503940666333, -1.3997153785705416, -4.7721507582330336]], [[3.5718438204926777, 0.65692778602772606, -4.9631813064637758, 3.6239347936408297], [-3.7837183833048114, 1.1469963156886891, -1.18069765381812, -0.37103743081033791], [-1.1525000740157889, -0.017595101950995229, -1.5291735814187071, -2.4208695105457356]]], [[[2.9464938037555646, -2.3289351122921453, -0.16461910719440809, -0.25587233374678142], [-0.63394064548931972, -1.1368929382894599, 4.6720692735363336, -0.86218157381442229], [1.2806185371587917, -2.6709262776212417, 4.0325548988050652, 2.8082887237860561]], [[0.16756717630542184, 3.3994732269648402, -1.8601414281832396, -0.561225426041152], [0.45609763647695178, -2.8712440396990733, -1.4609084771476724, 4.6760446559514488], [2.8095301225761045, 2.8790171075548106, 0.98413303389088025, 1.5553197724973664]]], [[[4.7300811001721161, 4.4970441467889408, -0.6408797402447135, 0.74514259363029112], [-0.78288541924968946, -1.0005263522457533, -2.3455571104845974, -0.49479272814354225], [4.2539637100377199, 3.7500113134769411, -2.3827272464571401, -0.74682383141824538]], [[-2.7302252569174645, -0.38216249095727761, -4.5701460836230119, -4.4474381480908072], [3.2284433461436919, 4.9456959343959142, -0.071891085436716295, -2.9956339828033873], [-3.4462136036103699, -3.5238905124700048, -2.1906711085468524, -2.9098438393018333]]]])
      s1=numpy.array(-4.01350432831)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.12775913671822, -0.49944881066751379, 0.72235278265184533, -0.78958991531601941], [1.2337587122482674, -0.0014830437451301903, 0.8468676645581823, 0.62605720233455731], [0.75575738817293292, 0.19915888311579771, 0.34875143118629442, 1.1890234488029918]], [[-0.88995638930744525, -0.16367935158159516, 1.236620394664808, -0.90293531467715704], [0.9427468052334057, -0.28578424784498246, 0.29418123346468422, 0.092447248204803281], [0.28715555777189045, 0.0043839748289006136, 0.3810070841664106, 0.60318098910959428]]], [[[-0.73414491744080956, 0.58027472298073746, 0.041016302395199045, 0.063752848587188674], [0.15795190278422835, 0.28326690226048595, -1.1640872642346201, 0.21482014301886715], [-0.31907740278879787, 0.66548483797096869, -1.0047466176529494, -0.69970990288391743]], [[-0.041750839814336457, -0.84700873572900404, 0.46347064211749572, 0.13983426455589626], [-0.11364074862452959, 0.71539577507021312, 0.36399823138169773, -1.1650777658237361], [-0.70001920834081999, -0.71733250348007394, -0.2452054248326295, -0.38752163826673425]]], [[[-1.1785414224690047, -1.1204782102932016, 0.15968083944101663, -0.18565884889528259], [0.19506280676650528, 0.24928996467957554, 0.58441624042602958, 0.12328197197980201], [-1.0599125756589991, -0.93434839151048776, 0.59367750762097327, 0.18607774411761865]], [[0.68025970164258476, 0.095219155056495627, 1.1386922025688373, 1.1081184382236344], [-0.80439513254568318, -1.2322637600040394, 0.017912297971031697, 0.74638862643626314], [0.858654512791168, 0.87800839969536892, 0.54582502704543767, 0.72501325556736573]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s0=numpy.array(0.520872651785)
      s1=numpy.array([[[[-2.0994093411354897, -4.5596403352713928, -2.3383778267081157, -1.906984108580204], [-3.516562144469507, 4.1760522298429841, 3.6083087745766935, -0.47052232091443891], [2.9553534562778436, -1.1647520194948333, 4.8131400467208927, 1.5181849993070458]], [[2.0365688392055947, 3.902331050226838, -1.1037862057152417, -2.8958326567042092], [-0.61808791518390294, -0.89562620667733839, 1.580019629591658, -4.0855327033172024], [4.1922098245188959, 0.66337787047425412, -1.6541245712605401, -1.9680385041531876]]], [[[2.2247695661282982, -3.5614946472298179, 1.7877279339075685, 4.6910578586182279], [0.91803532997688375, -3.2331736589512214, -0.91183921329794604, -0.97221248601018573], [-3.5420088381637891, 2.2462932313868471, -3.8781346586010432, 0.74488633893432432]], [[-4.13755923680176, 2.4768608506540826, 0.49788255459020192, 2.5624095716287698], [-3.7894109331503012, -0.34922769780871121, -4.9535418145058419, 1.9647256281344792], [2.0023693727543943, 2.3755929107018714, 3.7562909519165384, -0.69421977975561688]]], [[[-1.9082081405563525, 1.1967863378292707, -0.46027506819657837, 4.2300904385897393], [-0.93628064975083269, 2.0764962237288191, -2.7457522035414983, -3.9263346834057278], [-2.8396203460293501, 1.4737921071653082, -0.94334379857289008, 1.9483423663786592]], [[-2.6789821841152528, -1.1558330321740651, -0.8448255074661013, 4.312688223238478], [3.3795139943202415, -1.1692078688486651, -0.77994394415275181, -0.58595727322387692], [-2.0654448872855125, -2.2488066260743098, -2.2483034729838014, -3.2910830888265763]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.24810437944553571, -0.11423546891524262, -0.22274956845526006, -0.27313948209688227], [-0.14811984841608941, 0.12472848113879217, 0.14435368044290708, -1.1070094417036138], [0.17624715943134636, -0.44719617829938624, 0.10821888553594923, 0.34308905174423843]], [[0.25575990448118546, 0.13347731012065311, -0.47189632293607231, -0.17986973473025014], [-0.84271612336831347, -0.5815737055271688, 0.32966213965278174, -0.12749197953100827], [0.12424775323462196, 0.78518243518163433, -0.31489324373414396, -0.2646658846792756]]], [[[0.23412431548631851, -0.14625113986615815, 0.29136013478632172, 0.11103522222128528], [0.56737756682817131, -0.16110259043541217, -0.57123300269209309, -0.53576009286011628], [-0.14705571769679912, 0.23188096928155644, -0.13431009947768954, 0.69926460529504531]], [[-0.12588886876872957, 0.21029548415973026, 1.0461757436215102, 0.20327454968625261], [-0.13745478148804455, -1.4914986842478015, -0.10515156049743432, 0.26511215832164509], [0.26012815561001423, 0.21926006321962346, 0.13866674825037753, -0.7502993532796518]]], [[[-0.27296427507783155, 0.43522610120143895, -1.1316551509631687, 0.12313511007538511], [-0.5563210688196949, 0.25084208958937421, -0.18970125968131973, -0.13266129705804228], [-0.18343038445717558, 0.35342342332557891, -0.55215569612326754, 0.26734143894469437]], [[-0.19442930784424101, -0.45064696827780204, -0.61654465588621177, 0.12077679276185856], [0.1541264964903187, -0.4454919143654858, -0.66783344583872928, -0.8889259944138822], [-0.2521842412697502, -0.23162180587036826, -0.23167364105581062, -0.15826785217090855]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s0=numpy.array([3.8261166095028045, -2.9021005456270399])
      s1=numpy.array([3.0300471183293407, 1.0979256859871143])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([1.2627251194735176, -2.6432577201414524])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s0=numpy.array([[3.959321607112388, 2.2779837060429076], [1.3418035229685152, 1.6583709718651471]])
      s1=numpy.array([0.29310680995555227, -1.8052572597825045])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[13.508118790255311, 7.7718552714225533], [-0.74327551693666882, -0.9186341519351906]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_Symbol_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([0.76692967982204596, -1.0466286345923406])
      s1=numpy.array([[-2.0431644688427051, 1.298038762938897], [-1.540439297451881, 4.6613422695484896]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.37536365354691803, 0.59083727059555302], [0.67943516912586055, -0.2245337445889207]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-0.90541433347594946, -2.1249820948240217, -2.0520653897796013, -3.4971597732341433, -4.106728368849792], [-2.3344562359359164, 3.980157982133754, -4.2208413705115602, -0.11698988105400332, -1.0665092301020165], [0.27509241485592639, -3.3001771874651107, 1.3765435265721209, 0.89008242289919792, -4.8582919226393209], [4.8761752803727791, 3.0724287207523773, 0.52887310539760435, -1.0378001998226671, -0.39751807227150238]], [[4.3755131932985822, -3.2638360525061239, 2.4875357018418844, 4.90017624532104, -0.78831537214095615], [1.8721483216467707, -0.53777290954624579, -4.8291711463265283, -4.823758364713103, 0.81162641846183714], [4.1710874456546492, 0.63578497723473504, 3.7920025973936973, 1.7712504217594915, -0.99286549981118544], [1.7481644905093709, -3.92772488226265, 2.762314232137256, -1.3108290137224579, 0.20358052189310349]]])
      s1=numpy.array([-4.3859376002177122, 3.2652283346875492])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.20643575353899379, 0.48449893466759314, 0.46787382239039138, 0.79735739356176638, 0.93633989882709212], [0.53225933625230715, -0.90748167095131138, 0.9623578252235151, 0.026673858982443364, 0.24316561869213013], [-0.062721461163120787, 0.75244508433072421, -0.31385387847373641, -0.20294005615926122, 1.1076974561603798], [-1.111774887114384, -0.7005181105631475, -0.12058381892422541, 0.23661991902738244, 0.090634684873713234]], [[1.3400328383825801, -0.99957360342410517, 0.76182595728942104, 1.500714725909035, -0.24142733412130282], [0.57335908235217403, -0.16469687704021024, -1.4789688963018977, -1.4773111924421329, 0.24856651213015457], [1.277425961714185, 0.19471378784772597, 1.1613284612013375, 0.54245836437928097, -0.304072303080205], [0.53538812950324755, -1.2028944011471392, 0.845978886925708, -0.40145094901851375, 0.062348020115592984]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_Symbol_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array([2.2852248165104418, -2.8308975192358754])
      s1=numpy.array([[[3.4156372584338701, 1.4832550878049533, -4.1849935898974477, -4.3060732700623605, -3.5366240726243436], [4.4000112635993229, 3.7058503750236031, 0.64848240647483735, -3.5008379014441382, 0.29423908182932657], [-0.19363132959224494, -0.096493832030109417, 2.9793667868487548, 0.69924192466815693, 1.4824669221807687], [-0.85538764392447231, 1.4394611313762375, -0.2197525868900847, 3.0227879555350583, -1.1919071599583688]], [[1.5884286611927942, 2.4109857220090261, 2.7408145269633071, -3.7354342403618332, -2.0405335825832784], [1.6260463843296264, 4.1730185039625063, 2.1425148944463634, 2.6246381781448589, -0.99068890032593693], [4.1276557455018708, 3.5523170638961794, 2.6916868557197029, -2.6212378916490122, -4.4673158857679427], [-2.9843767161730361, 2.1525191819398239, -3.9439514824634738, -1.473488176877594, -2.7939579856510743]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.66904786533399563, 1.5406822705677088, -0.54605216649004251, -0.53069807994171614, -0.64615994507289998], [0.51936794694499688, 0.616653287437674, 3.5239580807333959, -0.65276510391062637, 7.7665577336051737], [-11.801937327615017, -23.682599897135109, 0.76701694688873823, 3.2681461678587902, 1.5415013868564325], [-2.6715663158588003, 1.587555764236293, -10.39908038786119, 0.75599904794047601, -1.9172842426672398]], [[-1.7821999743508012, -1.1741660240430396, -1.0328672339504767, 0.75784964667499011, 1.3873319916901394], [-1.7409697204935366, -0.67838125245497605, -1.3212965410760393, -1.0785858191077615, 2.8575040240225862], [-0.68583663313512933, -0.79691577872019914, -1.0517187440360518, 1.0799849675051687, 0.63369092126540705], [0.9485724452595341, -1.315155536353783, 0.71778203454664158, 1.9212217401260114, 1.0132212201380664]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[3.0113145926370386, 1.0076619105295492], [3.6822947483719162, 3.2707213947074791]], [[0.26473144922365321, -4.3551253699687189], [-4.4635605700262388, 1.6228635259225079]], [[-0.86950334293509179, 0.28221210055788593], [-3.9868684749965433, 1.8857926501821893]], [[1.878162302746242, 4.2274513568759851], [-4.3385368488629581, -4.6685865632944168]], [[-2.7988493639795422, -3.9139265728569619], [0.90777089657565746, 3.6117114795285534]], [[-3.9903565838372557, -4.4474608632894483], [-1.6321221216092918, 4.8069765103791511]]], [[[-2.5324400733687567, 1.7943708600223029], [-3.2045248126304884, 1.748258304989136]], [[-3.1069271298700474, 4.2458778587866561], [-1.906533242421673, -2.1820294253722028]], [[0.99814107038224265, 1.5205530435298016], [-3.3781204895359842, -3.8850703385800922]], [[-2.6337094758589972, -2.7709432747576623], [-2.5096025706381031, -1.568921419778595]], [[1.3050214709334726, 2.1035486829653465], [4.7315019974103159, -4.4119551001904869]], [[-0.72962807990541911, 0.75845044566269237], [-2.4597944085087287, -3.9551793380346889]]]])
      s1=numpy.array([-4.1911574070297313, -0.87838002358517286])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.71849236384828452, -0.24042568977233383], [-0.87858660287864365, -0.78038619814697818]], [[-0.063164282205107655, 1.039122358579033], [1.0649947345188258, -0.38721130425703326]], [[0.20746139037314465, -0.067335123248899717], [0.95125715591341453, -0.44994555609369213]], [[-0.44812497368770826, -1.0086596484745189], [1.0351643776456667, 1.1139134396298036]], [[0.66779867520248626, 0.93385339483843377], [-0.21659193592993534, -0.86174560599196615]], [[0.95208941022933735, 1.0611533835092486], [0.38942038274958873, -1.146932945614622]]], [[[2.88308022196636, -2.0428183836631963], [3.648221415089774, -1.9903211116453796]], [[3.5371104151354618, -4.8337595855798181], [2.1705107029187856, 2.4841519237494594]], [[-1.1363430902131133, -1.731087914913584], [3.845853046324911, 4.4229948704011859]], [[2.9983713257837055, 3.1546064349777136], [2.8570806521703158, 1.7861533478128595]], [[-1.4857139687750767, -2.3948047843569547], [-5.3866229540357047, 5.0228317831987646]], [[0.83065195053888896, -0.86346504394193868], [2.8003760814924914, 4.5028111202840577]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank1_Symbol_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([0.42808285422211068, -3.2666118931231991])
      s1=numpy.array([[[[-1.5472231398808614, -4.2964005973337285], [2.9979687713211645, -3.1399907559959992]], [[-3.0501548358403898, 0.28801219566185399], [-1.3688840267416991, -0.03552454646933878]], [[3.2089900721170093, -3.0717028844176353], [-3.0633689677335041, -1.6989839664333584]], [[-0.043336848153297858, 0.41522127411560472], [0.66648317280072611, -3.3442117770141966]], [[-1.414476493683654, 2.0748506205282355], [3.051527544887735, 4.0173316103323593]], [[-0.97903945847718088, -0.45619644561805561], [1.4539170580156471, -3.9857439426183836]]], [[[1.0689047957011439, 4.2970476759714575], [1.3195823374282014, -1.5422302915778543]], [[3.961482764708272, 4.3609593123978811], [4.9873342614700693, 3.1269472755820296]], [[-0.025336978659206366, -4.7259128217109048], [4.1579541989570963, 4.5254926528463333]], [[-0.78320670983594631, 2.0615433720008465], [3.3737319336061624, -4.7676067735182217]], [[2.8682951376156431, -3.6954191830842733], [1.5804106069710251, 0.21786216096011746]], [[0.086668682960421073, 4.9171624979689827], [-1.1540197079261527, 0.63107281473988408]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.27667816179059584, -0.099637555792113863], [0.14279096510850589, -0.13633252053518341]], [[-0.14034790929037008, 1.4863358589325475], [-0.31272397504780552, -12.050339744424008]], [[0.13340111518004769, -0.13936336629226787], [-0.13974250530416402, -0.25196403419908436]], [[-9.8780338779559891, 1.030975243582835], [0.64230106879248172, -0.12800710085541134]], [[-0.30264402139852803, 0.20631984297410538], [0.14028477473168596, 0.1065590037728289]], [[-0.43724780499445853, -0.93837393590856111], [0.29443416449516863, -0.10740350117446007]]], [[[-3.0560363338818006, -0.76019912727281946], [-2.4754892517655693, 2.1181090210471303]], [[-0.82459323620552372, -0.74905809917475397], [-0.65498154361932237, -1.044664845688891]], [[128.92665447844362, 0.69121289713943557], [-0.78562959975425783, -0.72182459318957148]], [[4.17081704242222, -1.5845467708752421], [-0.96824879907738748, 0.6851680619441326]], [[-1.1388688180250059, 0.88396247659157767], [-2.0669387301720943, -14.993938730467267]], [[-37.690798816164779, -0.66432864369897515], [2.830637874454943, -5.1762836503575782]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array([[-0.78355092726010955, -3.600471143181545, 4.2162797154899714, -0.97179156569234237, -2.6289624013939985], [-3.4475078762003051, 3.6782434960630752, 1.2723721258680545, 3.8619866042789344, 2.3316933109617191], [3.396658114135807, 2.2040312595779117, 2.873850062138871, -0.85428121096031973, -4.0347041290704713], [4.2868177450496887, 1.8567619191159856, -4.9899080461574989, 4.6401734604797813, 1.3227222249890938]])
      s1=numpy.array([[-2.5871296435213766, 4.9427866878541131, -3.2076099795024571, -4.0099956660612666, -3.9678022634605403], [3.1642305250479161, -4.4825562236285679, 4.3443082762121765, 1.1032930920066661, -2.3549972134004129], [-1.7430250147130111, 2.2890693369448778, 4.2730642757047193, 1.8182682718927836, -0.73012025413222936], [-0.6782242163601051, 4.7630008168236451, 2.8006624472671868, 4.7188890505508656, 2.5102747104437659]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.30286496435238858, -0.72842940036821058, -1.314461465836932, 0.24234229825162482, 0.6625739456837586], [-1.0895248778209985, -0.82056829018099575, 0.29288255919477285, 3.5004176426544689, -0.99010448831697551], [-1.9487144966161409, 0.96285037067489498, 0.67255015995866618, -0.46983232571672651, 5.5260816368747951], [-6.3206497816550842, 0.38983027518232194, -1.7816884898165863, 0.98331904199741771, 0.52692329627750711]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.1673681475481512, 2.3446871985321041], [-2.8976038377834512, 3.8730394639540844], [3.9430761239584786, -0.22221644398315199], [-3.850038805535573, -1.5290073955158356], [-2.7455624367947982, 4.4408898298212929]], [[-3.3963507436093563, -3.637600192726842], [1.2754379516519219, -1.2611744934607163], [1.3683267763109033, -1.7430919413163668], [-4.8088193695118084, 1.8877983901943622], [-4.2227758090018526, 1.6785700931045708]], [[-2.9272887964302887, 1.2840461190455521], [-4.699810304411467, -0.00085115389445800105], [4.9469141184996985, -0.99527206156143322], [3.3205455590527677, -2.4238134093396324], [-4.8470353870798331, 1.8014817477160001]], [[0.027479301207148588, -4.807300448134864], [1.5141551311985468, -2.8410590886772216], [4.595568801936432, -4.9419168839984993], [-3.2746584412304571, -0.032210834378823883], [-2.2209537351569439, 0.85945495424303786]]])
      s1=numpy.array([[4.0880937046576644, 1.2199396408395344, -1.0079621108225858, -3.1725670708169629, -3.4456705109411159], [3.5496651089424507, -1.0674514313879691, -4.4889122747084205, -0.63142244177021745, -3.3050536986985666], [-2.5611572300912036, 1.8220028640127586, -3.0285707156692054, -3.0649140725377122, -2.1239681609080954], [-2.5924538770425167, -0.22193838194338333, 3.8279175106051575, -4.5911348991559997, 3.0131159767364064]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.53016596588253739, 0.57354047336555547], [-2.3752026254261129, 3.1747795827740686], [-3.9119289124276522, 0.22046110820753353], [1.2135405555174459, 0.48194643687142075], [0.79681514180672564, -1.288831829892044]], [[-0.95680878037005268, -1.0247727830895546], [-1.1948440127093327, 1.1814818514233065], [-0.30482368390676196, 0.38831053819816297], [7.6158512136979102, -2.9897549806779846], [1.2776723750856629, -0.50787982469559956]], [[1.1429555210579738, -0.50135388173721251], [-2.5794747073342452, -0.00046715288503083321], [-1.63341542362058, 0.32862764485309826], [-1.0834057596607907, 0.79082589331215536], [2.2820659350220671, -0.8481679626241585]], [[-0.010599726170826653, 1.8543436744259667], [-6.8224122296467362, 12.801116525225359], [1.2005401864602658, -1.2910196915965488], [0.7132568554743286, 0.0070158762672700567], [-0.7370953366230939, 0.28523792674384163]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([[-2.7479300798046391, 0.78447559537483258, 1.6692949536695432, 4.2387346485952477, 4.5166987056717023], [-0.9434100249189914, 4.3635951163259925, -3.5485064597928462, -4.6700369823879839, -0.16384762363422212], [0.7254295370864261, -1.3543677586975491, -2.733383353040304, 2.7175555774921003, -3.1301474117222816], [0.95836687342147986, 4.538456658040106, -0.51924020497471446, 4.1015455525526985, -1.9650239321867691]])
      s1=numpy.array([[[-3.2600768439374104, -2.5161765500761768], [2.6277154597994201, 0.087259884108244812], [2.3973041556617405, -4.4292052393222994], [-0.67702377915757772, 3.2339803900412338], [-4.4067159317642322, 2.6457599964489891]], [[1.0081096943596304, 2.6366956763663474], [-0.70838239482593046, 2.8592732504196299], [-0.71039777415001915, -1.7604687138135611], [2.7440009066014683, 4.758904263631262], [-0.67900215544757092, 1.0454799437622677]], [[-4.8319964432937255, 2.0392175121425113], [4.3651180402575918, -3.6593237769251106], [2.509283450839856, 1.113154509340375], [4.2652922917687448, 4.0162853762587787], [0.71016021449043087, -3.0909313780742509]], [[-2.2974751012786521, 0.037588795575256739], [-1.1549596172812269, 0.47210347275695419], [2.1475679898173521, -4.5945650409426761], [-0.28605981072719633, -2.5235508857496738], [4.923931277447485, -1.3869659547741842]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.84290346864516918, 1.0921054326340678], [0.29853901892205387, 8.990105858972953], [0.69632172026530315, -0.3768836311421318], [-6.2608357033921544, 1.3106865649674526], [-1.0249579903970434, 1.7071460418684221]], [[-0.93582080422137259, -0.35780011829772967], [-6.1599429181159291, 1.5261203579215756], [4.9950979421896191, 2.0156600523198187], [-1.7019079589780355, -0.98132610443071444], [0.24130648528828358, -0.15672000654990997]], [[-0.15013039550003016, 0.35573916601189387], [-0.3102705920451182, 0.37011421816180734], [-1.0893083251019098, -2.455529156199558], [0.63713232097516481, 0.67663408421030535], [-4.4076637184868082, 1.0126874488143647]], [[-0.41713917721593763, 25.496078252965784], [-3.9295370938800667, 9.6132668364771181], [-0.24178056640659612, 0.11301183035776133], [-14.338069867717897, -1.6253072508717208], [-0.39907623024450845, 1.4167787791926731]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Symbol(shape=(4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[0.82971257323461067, 4.0071062598493654, 4.6199406796876641, 2.1611630259184036, 3.8633476982972184], [2.2170313638358135, -4.4807415548816714, 0.46753281960899784, 4.8879701320461706, -1.3621992046530251], [-2.5244860570422012, 1.2272577473280633, -3.3352248111724592, 3.6431950680095877, 1.9807064182937619], [-0.31505853963339803, 0.72832641547721266, -1.0741625859960235, -1.1287680900232711, -0.13494511459235969]], [[-1.2262094048473537, 4.821908136361623, 4.6015932600032787, 1.9584413518665738, -2.3597093712444686], [-3.6273221498899675, 3.8479599796443047, 2.5051417044968547, 0.58515784310034036, 3.4163088634231116], [4.0253635250937982, -0.58208425440922618, -3.1354267901591593, -0.17935449615796273, -3.2146299497137298], [4.8404578619629888, 3.5980137617592725, 1.3591837614513711, 3.2415585715038269, 1.350704132027996]], [[4.9335205045852089, 0.90270012496390173, -2.7196410900099757, 3.7243316926026608, -0.79514675295428283], [4.9547498698419261, -3.9523371982104925, 2.0322577907894814, 0.64674678343510195, 0.14544053796583434], [-3.6767541711986542, 3.6233742813454874, -2.5760016593517934, 1.5300085962351169, 0.23350566920612525], [-2.7493197719824449, 3.3519457506418568, -4.6330782129335804, 3.0325368929233942, -4.1130609613843347]], [[-1.446546184171662, -0.43178776430476518, 1.3609927198160854, -4.1372996473500177, 0.13649627234400352], [2.6217510475813457, 1.0256406745685451, -1.8535919312099249, -1.6588175341611722, -2.2322716107330742], [3.3236349661316016, 0.98218558299499659, 1.0778754479406869, 2.1039002080294997, 3.2663163835054849], [4.3378230626170442, 0.69642450771053799, 1.620251122949127, -3.9462690662160558, 4.3163548883623299]], [[0.21299720008263545, -2.852661980879847, 1.8920413730719154, -3.5680913309816464, 1.4637967705814949], [3.5537157061616096, 3.3525116010354719, -1.20110692752628, 3.9587827751034421, -2.1441490186808752], [-2.8996858221816146, -3.2597063656628955, -0.65242061178157051, 0.23914247402747613, -3.2810785007109153], [1.3996659903396651, -4.3767359875822809, 4.9092782265006125, 0.043584505794669326, 4.3418098067867898]]], [[[-3.1715124385713822, 1.2065262357785187, -4.9990307985783291, -0.10264907494347053, 3.3443016445088212], [-3.3097092015419882, -1.0685910484712169, 4.8455073032921234, 0.025767302018352645, 1.8162711678699104], [4.8451463297952255, -4.2988968894221236, 4.6922797510939365, 4.689016832264679, -2.5975494336240743], [0.65071777007174347, 0.29850595260484969, 0.032039517117727456, -3.5516478821996476, -0.47526458799540805]], [[2.4058185547618827, 3.5002975563496097, 4.4782273549369425, -1.2574273942354908, 0.59880561453248049], [4.2553298543543487, -4.6464705730852875, -3.2636245610486303, 2.8338415724185353, 3.10427610526121], [4.0446615769382248, -2.585758723069429, -0.54980800903734384, -1.9875054817772106, -4.0830396169197938], [0.30197946030173561, 1.0219143629919811, -0.76854609330538448, 1.1574511723564571, 4.1745755596787468]], [[2.1640852543418108, 2.8096849718767078, -0.98336557258417123, 2.0806622737989047, -2.3187646937574344], [-3.477892326084592, 4.824911258708509, 2.8845778231368326, 2.3786623779369007, 0.90862259894572972], [-2.3024551577835641, -0.57317853092616478, 3.6656693813850723, -2.8165652104885766, -1.7233932030810473], [0.031586590833540207, -3.1461124201245303, -1.6133207907041247, -2.8350032028607841, 1.8980308663422454]], [[-2.6012956633200068, 1.4874822589572947, 0.14633999713949564, 4.9592722428133307, -1.5156243651690282], [-0.13655382677178451, -4.2118053438746852, 1.4964131427776017, -3.9986698660785667, 2.3490723825651525], [2.4453746906674976, 3.1644274955170388, 0.19614217125422684, -4.7970538288820022, -3.3388866055890318], [-3.0807105167661875, 1.2502879272242948, 4.0809875917519864, -2.6512022074970365, 0.1604964894086649]], [[-0.34775091634706623, 3.4663539392646729, 3.780568303271739, 4.1612328242972936, 3.0795302251417116], [2.3991857089950539, 1.9333018602812748, -3.7686809734305182, 0.77447236430365773, -3.8938826704976748], [2.7178503967660372, 2.976225246741425, 4.5861452703500305, 0.17943104023541157, -3.5249959854303592], [-3.6878200371925187, 1.0133092649494477, -3.2254369484493548, 0.16308650841265582, 2.7936889500465654]]], [[[2.8816581006205331, 4.2925598590576168, -3.2843863474897681, 3.3647430224838235, 1.2261927615409132], [0.48087989093722694, -1.3686808107163118, -1.6456427777499405, 0.034660680323501225, 0.54856860643335814], [1.8939499008551932, 0.55494025698171523, 3.9028975651197637, 1.5545640453071794, -3.8360548141801565], [2.6707883859805595, 2.0861356183200099, -3.521459867054213, -3.213974701779585, 4.6309940679609749]], [[-3.8354723536563426, 2.4493928164442202, 2.4018651635981412, 1.1848797710102774, -2.1389879620494936], [2.383072973664814, 2.731544730707796, 2.5376695507073421, 1.8904032862212077, -0.2837559348689247], [0.71288504008317854, 1.4950485311831718, -3.238695986800705, -0.77521279079036098, 2.0745928918534684], [2.4786647917017666, -1.9352951090838291, 0.14563173222726888, 1.4567928389628113, 2.3865813509617428]], [[-3.3819627175655906, 1.4475971534085366, -2.035387968849911, -3.585282827937788, -1.4582556247075895], [3.7343507917421661, -0.38699697623065266, 3.7553796892736546, -0.53843614200014578, 0.092615495661394931], [-2.0259830895983955, 1.9444416982061057, 0.14953909200572024, -3.6196332540718812, -3.6982912620495343], [-4.7983318826314285, -2.150652644200127, -0.51368088163712855, 0.62996377007532356, -4.9104888711834018]], [[4.5203249062171942, 3.0201024762848512, 3.0450949051936824, -3.1720456404364104, 0.73588965970358711], [3.8761214689329719, -0.57130832152787825, -2.5941059300755054, -2.9573221114721635, 4.1237035633456873], [-1.0646060312860248, 0.39554424611030292, 1.4066983023126873, 3.88711131220537, -4.7275501420108457], [2.4404126836714104, -0.19673851964498201, -3.4083705990893822, 1.8146611596432898, -0.37960611651859733]], [[-0.6241830319634083, 1.5467312641493978, -1.4294678437609001, -1.3342789005605438, 3.7328150120949015], [-1.7621255797143154, -2.9076927822630094, -3.4106490031682855, -2.7021289202121346, -1.8521627374163083], [1.4279184372128748, -0.32089355042032697, 4.2138902709484345, -4.8215644536540019, 3.478546419108282], [-0.68057950457666649, -0.31100671907127175, 4.9895802905194095, -2.0366109738669778, 2.0484842926390314]]], [[[4.049988356008102, 0.43430212654552847, 2.7881135718227945, -4.4886235853828449, -3.7171479328930515], [4.5693533708801795, -1.9013647262459612, 2.5388615685577332, -2.9119375634900022, 4.2346306911823604], [1.150193213935645, 0.4877433572069334, -2.4669611091800636, 3.8382132451409099, -3.613735495649558], [-4.1555154315100431, 0.54110449032169328, -4.4643778002221897, 1.8657036655065777, -4.9379661721832688]], [[0.75187249262754996, 1.8154433491930968, 2.6077280113054719, 2.6130318609859371, 2.4299395616502064], [0.89144311411126242, -0.048032079789155802, -4.5167942630933311, -1.0620446524161844, -3.1740376940596029], [-0.90501470785970373, -1.8544823491807882, 0.90092908167728059, 2.6786198193143029, -3.3761799714868568], [2.303349509662973, -4.221403925163866, -3.0348043930887214, 3.8435377793183321, 4.9957142054939609]], [[-1.9500522066158164, 2.7228728601497396, 2.960961037119862, 1.0065839730842807, 4.3493394512040009], [-0.44946209780916035, -0.064617890945420697, 1.6003421145896848, -0.71674602858423064, 3.3096709226584746], [-4.2330838536508182, 1.2294224536833918, 3.6948610873112493, 2.0178620067737816, 1.511175636928864], [1.4735063809756443, 2.2812849610768442, -1.6014065704259228, -2.8362623561796143, 4.4524889493843336]], [[-3.641688649605431, -2.9271461246349464, 0.36127140393865709, -1.6084013904934968, 4.7635410992800864], [1.6567865819730017, 0.72304143125577891, -0.055382983768308414, -0.51122028574008294, 1.7335750255358828], [-4.5497419198713036, -1.9564016807892868, -4.0171105451267639, 0.31195688506382169, -4.4856957548169829], [-4.6147042236632103, 3.9256870027866526, -3.8886926574081491, 1.5313491804711052, 1.3081440996898008]], [[-2.4208230176671397, -4.974589472259753, 3.8578314916321865, 4.3206613134828658, -2.3199594228273455], [2.7945737435292104, -0.10404815162869951, 2.7445378585373517, 2.6375053330488605, 1.8206313891381152], [0.40123825957396964, 2.0797524210306388, -1.3299750510639594, -0.19664406182373995, 3.9503504636413815], [-0.14968144916518433, 2.6085882834366201, -1.0365063067078637, 3.8242039272658364, 2.8908708481831003]]]])
      s1=numpy.array([[-3.6808756837502599, -3.999853362471705, -4.9884559267815227, 3.5116244725634989, 1.6965525248647193], [-2.0234125545533543, -2.8370933224181205, 3.485839355777566, -2.560180942013015, 0.91325587818823273], [4.6336532462987989, 1.8504304187191334, 0.064629313111883135, 2.2416846669305386, -2.8551909666107145], [0.51919083188705084, 1.5803902486667134, -1.0996236363258105, -0.21382457852630044, 2.504419786674184]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.22541173473950582, -1.0886285232449702, -1.2551199976905054, -0.58713284870205207, -1.0495729902948112], [-0.60231085054657196, 1.2173031473631482, -0.12701673725982837, -1.3279367607074581, 0.37007476527029803], [0.68583844550548057, -0.33341461455652094, 0.90609547774087429, -0.98976313818284645, -0.53810739304178823], [0.08559336600914505, -0.19786770270251489, 0.29182256568404752, 0.30665748778378349, 0.036661144300008219]], [[0.30656358964360109, -1.205521227753692, -1.1504404894382751, -0.48962828743710674, 0.58994896997581148], [0.90686378253838484, -0.96202526216272644, -0.62630838620263951, -0.14629482385292814, -0.85410852694659967], [-1.006377774460794, 0.14552639851015134, 0.78388543429542767, 0.044840267856002307, 0.8036869500954037], [-1.2101588291656356, -0.89953641688901409, -0.33980839752871966, -0.81041935234863438, -0.3376884124555331]], [[-0.98898748971572903, -0.18095782306456307, 0.5451869536240741, -0.74659007662227539, 0.15939736957189068], [-0.99324318838648273, 0.79229670587878309, -0.40739215112213367, -0.12964869148445565, -0.02915542205855879], [0.73705255196488051, -0.72635186809863916, 0.51639258663628029, -0.30670985545265816, -0.04680920762525001], [0.55113642624808457, -0.67194053627822314, 0.92875997722260584, -0.60791093224711357, 0.82451584653731125]], [[-0.41193077320015314, -0.12295954982611182, 0.38756784230477687, -1.1781725750218883, 0.038869837424376059], [0.74659208809632704, 0.29207014661787672, -0.52784457611915314, -0.47237896509766297, -0.6356806168125112], [0.94646651203718712, 0.27969550578909069, 0.30694496417888151, 0.59912448625055992, 0.93014398578930679], [1.2352753252828361, 0.19831975575741034, 0.46139646639560455, -1.1237730848068914, 1.2291618657080878]], [[0.12554707087516759, -1.6814463089536908, 1.1152271122420947, -2.1031422715699066, 0.86280663234886645], [2.0946688381752154, 1.9760729785261417, -0.70796919631006094, 2.3334277701889068, -1.2638270771203188], [-1.7091636007042172, -1.9213707314619215, -0.38455668316758634, 0.14095789580493243, -1.9339681221909377], [0.82500598703908234, -2.5797821897269437, 2.893678889719062, 0.025690042103556264, 2.5591956294621645]]], [[[1.5674077100265191, -0.59628286533234731, 2.4705939415710549, 0.050730670180174485, -1.6528026560787246], [1.6357065661641945, 0.52811328370308364, -2.3947203907518082, -0.012734576525369285, -0.89762770512750523], [-2.3945419923841174, 2.1245775508055287, -2.3189930993235852, -2.3173805172419359, 1.2837468205773064], [-0.32159421399625648, -0.14752599608671577, -0.015834396720346446, 1.7552761913072314, 0.23488269207676107]], [[-0.84798710558852808, -1.2337618677154494, -1.5784561330961244, 0.44320974015890185, -0.21106306577962849], [-1.4998906876730556, 1.6377573964063308, 1.150340926489851, -0.9988538445408579, -1.0941748305323151], [-1.425635718422849, 0.91141123298176419, 0.19379271196082112, 0.70054286408993183, 1.4391629576145646], [-0.10643973460991107, -0.36019765543735438, 0.27089207367008106, -0.40797077882864091, -1.4714269448565969]], [[0.62082185478655849, 0.80602824316038157, -0.28210295203486724, 0.59688989119660207, -0.66519551163888935], [-0.99772019623342589, 1.3841461886967088, 0.82751312631657015, 0.68237865694941247, 0.26066106501429653], [-0.66051671427926972, -0.1644305638973742, 1.0515887300742786, -0.80800201128611726, -0.49439834346486111], [0.0090614017485307688, -0.90254085143368434, -0.46282132538039766, -0.81329140947414558, 0.54449751483710085]], [[1.0160593029314031, -0.58100669157693186, -0.057160021285226681, -1.9370788062010813, 0.5919989248796319], [0.053337568658102415, 1.645120184576889, -0.58449507150889279, 1.5618700227236668, -0.91754154716819647], [-0.95515697759422902, -1.236017128160058, -0.076612620629854578, 1.8737167167216635, 1.3041604016330726], [1.2033174945611036, -0.48835920411204231, -1.5940231117192811, 1.0355526689502084, -0.062689510250970762]], [[-0.38078147061801959, 3.7955999211758886, 4.1396594246640257, 4.5564807450816263, 3.3720343867384166], [2.6270684550694496, 2.1169333879532926, -4.1266429961633921, 0.8480343601402256, -4.2637367724613755], [2.9760009890742323, 3.2589171532581038, 5.021752807601171, 0.19647400528248091, -3.859811986563328], [-4.0381016156267391, 1.1095567947065466, -3.5317998224639453, 0.17857701473128845, 3.0590429437901228]]], [[[0.62189765773308592, 0.92638780480312466, -0.7088114221997992, 0.72615339207167973, 0.26462764828601565], [0.10377986124044501, -0.29537834144356107, -0.35515017854743935, 0.007480205893953539, 0.11838792789933854], [0.40873794394693097, 0.11976300933285895, 0.84229383548224346, 0.33549425532627219, -0.82786833849603736], [0.57638935069513286, 0.45021401201877642, -0.75997483624115225, -0.69361571333521688, 0.99942611624209299]], [[-2.07274605673163, 1.3236881493440258, 1.2980035019423808, 0.64032657430612838, -1.1559407694616799], [1.2878479242220713, 1.4761672220015543, 1.3713942037679618, 1.0216019295282357, -0.1533459091454735], [0.38525363227472076, 0.80794636537484255, -1.7502392708408501, -0.41893647172475834, 1.1211407199463896], [1.3395071582413223, -1.0458621353746653, 0.078701544653635269, 0.78727242279728749, 1.2897439032664264]], [[-52.328619239862576, 22.398461065221699, -31.493263209007747, -55.474561856028394, -22.563378047715407], [57.781068866961945, -5.9879481553626022, 58.106445952357923, -8.3311444308255478, 1.4330261486929852], [-31.347742874678424, 30.086064737218891, 2.3137967093483636, -56.006061023822852, -57.223124987375797], [-74.24389416494013, -33.276736834213622, -7.9481098731139088, 9.747338161938389, -75.979283002476009]], [[2.0164856248076672, 1.3472467920388522, 1.3583957414328152, -1.4150275849367266, 0.32827527910569843], [1.7291109343404714, -0.25485668433025016, -1.1572126839889239, -1.3192409062250146, 1.8395555914615467], [-0.47491337519105803, 0.17644954794284604, 0.62751836735308131, 1.734013427288976, -2.1089273668825674], [1.0886511915223933, -0.087763690650732459, -1.5204505117824383, 0.80950777172778843, -0.16933965874798032]], [[0.21861340949265876, -0.54172602891969146, 0.50065577415922036, 0.46731686817586637, -1.3073784050689892], [0.61716557677613615, 1.0183881975903761, 1.1945432172675068, 0.94639166059695334, 0.64870012516708764], [-0.5001131111408289, 0.1123895228630704, -1.4758698525691187, 1.6887012147483405, -1.2183235586646342], [0.2383656688941391, 0.10892676626826669, -1.7475469588089743, 0.71330114086363861, -0.71745964336343748]]], [[[7.80057756661075, 0.83649806558989892, 5.3701132619948577, -8.6454215092136639, -7.1595022573544043], [8.8009130559420878, -3.6621693016713364, 4.8900354409764661, -5.6086074419039234, 8.1562123810837974], [2.2153573277770584, 0.93942983437165395, -4.7515498303651613, 7.3926830163594017, -6.9603222432011673], [-8.0038305306863915, 1.0422073293455416, -8.5987223310472629, 3.5934834571818071, -9.5108886153396401]], [[0.47575115909621857, 1.148731049640862, 1.6500532153406198, 1.6534092533096845, 1.5375566659565321], [0.56406518254799598, -0.030392543759161873, -2.8580246346773506, -0.67201417707567601, -2.0083885589254682], [-0.57265267779474971, -1.1734331762330925, 0.57006747696484705, 1.6949103688624403, -2.1362951171934594], [1.4574561641380537, -2.6711148899616584, -1.9202879767506891, 2.432018156629927, 3.1610636738037106]], [[1.7733814936277252, -2.4761861878921745, -2.6927040664689303, -0.91538953859485539, -3.955298256171099], [0.40874175759894998, 0.058763643132780825, -1.4553544155679781, 0.65181031482653939, -3.0098215546885925], [3.8495751762802106, -1.118039311878817, -3.3601142838807521, -1.8350478655734386, -1.3742662371083425], [-1.3400097381492218, -2.0746052428441217, 1.4563224338981358, 2.5793028291538564, -4.0491026222949369]], [[17.031197604617297, 13.68947454408244, -1.6895691151530585, 7.5220603804237749, -22.277799550037091], [-7.7483448974469358, -3.3814701576359929, 0.25901130800777566, 2.3908396745755907, -8.1074637793458955], [21.277918334873206, 9.1495640691682656, 18.786944760106984, -1.4589383840429315, 20.978391659802767], [21.581729544228246, -18.359381460461023, 18.186368864652479, -7.1617079337899838, -6.1178378496319548]], [[-0.96662030484990735, -1.9863241373228016, 1.5404092844815382, 1.7252144933819629, -0.92634606832754751], [1.1158567578801735, -0.041545811202391605, 1.0958777251085527, 1.0531402710850688, 0.72696733943149161], [0.16021206257390477, 0.83043283402280799, -0.53105116727660817, -0.078518810173145562, 1.5773515624899941], [-0.059766916857001078, 1.0415938643021063, -0.41387083436372379, 1.5269819970334517, 1.1543076218951756]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank2_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=quotient(arg0,arg1)
      s0=numpy.array([[3.5027467965016044, 1.8291121337854834, -2.6522719832846109, 0.96054519707649089, -3.4104529002883224], [-0.86303546233468431, -3.7340294602793644, -3.1316902478523367, 3.0778071115663312, -4.4184069822814012], [2.5759867470173994, 0.12121701817763508, 2.5743273801486595, 3.3959679726307517, 3.9600888556371689], [1.6327064536849853, -3.6514741097797696, 1.854818922417051, -2.1965156387506046, 0.85038053595820706]])
      s1=numpy.array([[[[0.26695237473442912, 1.9202034816110602, -3.7475563395660263, -3.3038837112498411, 1.2139689957962023], [-1.6815003891503166, 2.4600747880217906, 2.5993499113335039, 1.4814118380772143, -1.9805031647942304], [-4.7416974493747617, 3.0785130850994218, 1.8643644186042305, 3.2914229335765004, -0.17350634402886911], [-2.1154798539937647, 2.8119991817155423, 2.2501183111283183, -3.1702197173766722, -4.4443483744440933]], [[-0.24528010138690171, -1.6488266149715147, 3.175177932583809, 3.3536604456843548, 4.5327429146842988], [-3.6634615457093656, -2.4424613006754967, 0.27087226970900069, 4.7856421199160053, 2.2615032604516934], [-0.40454214104811648, 0.070756494714244056, 1.075218723504805, -2.8535833356680964, -0.41577023641410094], [-0.74972844416367845, -0.49773995730537202, -2.3145218353563504, 1.0711816251409934, 4.8870968897292251]], [[-3.9188237303752995, 2.3917243311692262, -2.3107386430684986, -0.53878136360645534, 1.3677831389926087], [0.36627307297757383, -1.6044148308183037, -1.2631234838729046, -4.8288859214586459, -4.374810607456042], [-4.9169120428864446, -3.7318173146775102, -0.72105782875227042, 2.4469025715363903, -3.0040116677251492], [4.0552347619425575, 0.47421545774153273, 1.6151969686009426, 4.3995968897466753, -3.6443390256562056]], [[-0.15862138046634922, -4.2138477062195978, -2.593372396543093, 0.57063204721860039, -2.8273187829233573], [1.5604861116186077, -0.17498586650196568, 2.2659979358484694, -4.7834849261513455, 2.91212730185614], [-1.4565046728579292, 3.5441273914509548, 4.8794169561838707, 4.0596342529960179, 1.5186567800235942], [2.9168041280081507, -0.96400740789844175, -3.1166204133058892, -0.58980363779025513, -4.1518901661408485]], [[-2.8396987697416174, -3.7623192069025349, 4.5245770607850506, -1.8199589514137027, -0.59684556257812282], [1.9447285540692594, -1.6254035309228421, -1.1001356086211067, 1.6324471386756532, 0.080272896530604321], [-3.9231008512534316, 4.3124708145107178, 3.3742088201485423, -3.0639662388455644, -2.7765020203316979], [-3.1700508678346742, 2.3912335583531696, 0.056655897391175714, -4.6959009301975305, 0.38867451625623328]]], [[[-0.17474097012400325, 0.12012669167076773, 0.29566339780747164, 2.2120797156737346, -3.1798633227706388], [-1.9988661389958806, -1.5991454913672265, 3.5671133752846416, -1.7626007769793572, -3.6475072482940862], [1.4418896659594242, 3.7278213675322664, -3.5106214985769348, -0.79221213234814059, -4.0175168338312117], [0.44579910447863824, 1.9863447591315984, 4.6902646581273526, -2.6998600704155704, 0.85498742726381849]], [[0.55591044610200768, 3.9455486025935684, -4.4456495330738957, 0.5194360317228579, 0.98464029726397229], [-2.3880004245315503, 2.785392856190974, -0.34899686353118575, 2.0089052427354579, -3.6502350873698388], [-4.1650024353762376, -2.2208813980906186, 2.1054525877720494, -0.88491913606342809, 2.2640621291262351], [-3.2429926204961146, 0.62014200700678934, -4.9815674526570799, 4.4288821660269111, 0.19991289177508786]], [[-0.19414597255789978, 1.8615411850990427, 3.8605290614310555, -0.35641250541315816, -4.0278631851266731], [-3.9507065620463653, 0.51466043825639041, -1.9425390071978854, 3.5747492868407491, -0.73729392497866009], [1.4823561909279164, 4.1931661869800134, 0.5278995741842003, 1.9582586333016172, 4.1345348165320441], [-4.1758179813955545, -0.71728375488475127, 3.7910372025567902, -3.7224590174326022, 3.3190925341098279]], [[-4.5122957368677499, -0.93417158100093722, 2.9470980062793393, -1.9999257041203045, -3.4026262159291054], [4.8032897376115642, -1.4211073851950031, -2.9176168553689674, 4.1242590576304234, 2.3245138090534798], [0.087343390570948642, -2.8054849939841264, 0.34334117482590898, 0.12416451977415122, -3.7745049583819954], [4.7723550354855711, 0.84001327715681029, -3.1602931112074852, -4.7759633171513949, -0.2829877566866088]], [[-1.8775224970335791, -4.9227312652546829, 0.062784060149981613, -3.0117842126719419, 3.2924976729239912], [4.2289876101489465, -3.0317072828090943, -4.7662104336993272, -3.1338201714857949, 3.1544286763110332], [0.41721482470607452, 1.1850949276298213, -0.34038901084463191, 1.8758178074411198, -3.2570134736366843], [-1.472657287575132, -0.76356121043608027, 3.4250840678850967, -1.3745724706201621, 1.9491600115821086]]], [[[1.4661925487133178, -4.8195912896752935, 2.6730623774166995, -0.55406348345150747, -3.6748140807688081], [-1.8435051215881715, 1.1743657698391825, 0.22285789764274622, -2.053893481190463, -3.3091497081096763], [1.3928219341066725, 3.6955043339323606, 3.7592551931393672, 3.5756593606284195, -3.4594106762287091], [4.0716939181720431, -0.59429437769294413, 3.7298305009541206, 3.9701303788183182, 1.0285464267117481]], [[-1.3608042170028667, -4.8074259786106577, -2.6815296473087615, 1.8974012552376731, -0.44332486309543739], [2.9688185824153726, -2.3962679797788313, 2.4946755628083199, 1.2306120737548714, 3.4011205482549176], [0.44844922587452096, 0.60847485170913895, 4.8446631135326825, -2.9013501037424918, 2.9779228293890032], [-2.7667447888387606, -3.8437585074713345, -4.5534879485233359, -4.9306609240547683, 0.79155952663472195]], [[0.25933126786660488, -2.272350876794996, 2.4396810814242915, 4.1071457535946116, -2.9511597269910483], [1.3308340715762279, 0.67089562033213035, -4.0250505187001515, 4.7828103311194745, -1.0837913089779283], [1.1289279938385635, 0.33607084277339805, -2.7869992700255919, -1.7531642451009333, -0.3598929112837066], [4.7669762726869465, -3.3698639803210662, 3.5101558190527644, -3.2615162542076606, -4.7955085354114591]], [[0.71398038291907184, -2.5133723512652093, 4.6838398623083179, -3.4896975874589211, 2.0563218194921031], [3.5136246742439265, 4.6093014690117329, -4.7644755366458345, 1.6451364220263498, 3.2897861985080965], [2.9538937161433001, 1.8324608521132362, 1.8312768195427047, -0.9631736475358661, 3.3310039135130847], [-1.6029863040836623, 0.5800818858054253, -2.6740883189349063, 3.4284515329331562, 0.72123832926945575]], [[1.4698464788904371, 4.0690975699369876, -3.523124826554731, 1.7896830102083818, 3.7715630740847281], [4.3294418185579495, 1.7963874159815827, -0.23502861953967447, -4.1765668944205006, 3.0292195045706052], [-4.2983428397080168, -2.8719527142067545, -0.39394676984062471, 3.9864362559378037, -4.1173738114722767], [2.1568771304457215, 0.70438621747458363, -4.3519061526548128, 0.46221700756065776, -3.0082940772058997]]], [[[-2.357152253288394, -2.9151593213759353, 4.9931372399211877, 3.4506410780809507, -0.70578553304727443], [4.509645331828489, 1.889000168394813, 0.88904729157359697, 3.0604949675036881, 3.4495430043203523], [0.94259710455157908, 0.89772760263820661, -1.0867960764629627, 3.8552473697877758, 3.9996620791776696], [-2.1249145022938309, -3.6471853698054835, -4.504343538553651, 1.8572447644527639, 1.2060432928939937]], [[-2.1126608914573786, 3.3911162495529705, -3.7575346079650607, 2.6160873373096214, 0.56727271035090343], [-3.6341181436680015, 0.49935423010762836, 1.2788916700002542, -3.9374339256045445, 2.8848025916668689], [-2.2177889572748577, -3.3864479780856707, 4.2036253389501184, -1.6202578058194117, 3.9682399512231097], [-0.94588568949814711, -3.2087031311821557, -2.8166804874680262, -2.3389252363909563, 2.7337772070349997]], [[3.3943818535505077, 3.0171353390357911, 1.8681547979599564, 2.5348480147783636, -3.5600076743619358], [-0.9443324696887192, 3.0148276667723355, 4.9254813693834407, -0.93595492998966456, 1.4417603437831294], [-1.6013871078915463, 3.7810453444168175, -1.4999452400825444, -1.4836228286916811, 1.1474915815137177], [2.6825031496152523, 4.9350263847130442, -1.025977750505831, -1.2689691330016717, 2.7974180511664679]], [[1.7885856469497021, 4.4479909805967868, 4.5479422822060211, 3.1320371231034176, 2.8719232584905905], [0.15907440082558377, 3.8598542471741144, -2.9153551725067315, 1.2503538250988022, 1.0337834232155707], [3.8135047662292134, -3.081351096253667, 0.83557840065781708, 3.182731347282262, 3.1536071180898748], [0.96517121132694417, -4.3472798972373825, -4.5380361144307688, -4.5308992054248538, -2.8344140936592122]], [[-2.6160540454566972, 1.653123389013162, 0.99694767484170477, 2.389845485981219, -1.1546795468484672], [-3.5243321255210027, 3.6000871066950744, -3.1031036292598193, -3.6939092166074152, 3.535054782607288], [-0.62960001231127016, -2.6312206833984355, 3.060495355017764, 3.1389281453018008, 1.9842247219074194], [-3.9042197512956234, 3.8105559232847561, -2.0978465608678256, 3.851252343215009, 1.4455338424701791]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[13.121242318920086, 1.8241539659967612, -0.93467488654412834, -1.0601907036178748, 2.8853675906313141], [-2.0831079309304155, 1.423837524597475, 1.3475472391112764, 2.3644652394893537, -1.7686145918709155], [-0.73871157616002581, 1.1378047452374176, 1.8787886968600056, 1.0642044086067897, -20.188004168417006], [-1.6557693943002345, 1.2456428932403356, 1.5566944987640039, -1.1048908620756719, -0.788135065343462]], [[-7.4572381674788417, -1.109341708325762, 0.57606602610047708, 0.54540767123256895, 0.40353317366839442], [-0.49928520088541223, -0.74888070213420244, 6.7526740029553665, 0.38220829889753372, 0.80880366868016462], [-4.5214378137379949, 25.850801981818119, 1.7011535362993606, -0.64098781028143403, -4.3993339916802396], [-2.4396995312427512, -3.6748348348157465, -0.79027646481627079, 1.7075648898894509, 0.3742737611013125]], [[0.67680308321257587, -1.108937158317076, 1.1478026696098265, 4.9227240629316249, -1.9391027039842339], [-7.2412420648978593, 1.6531086177580807, 2.0997725219646712, 0.54925132347782768, 0.60625984100073083], [0.53941822838213926, 0.71071860159205313, 3.6783068951267657, -1.0839303591966356, 0.88291001389255575], [-0.6540366067522333, -5.5929682172659376, -1.6420734033335673, -0.60284431727501431, 0.72777860803085914]], [[-6.0555846522863046, -0.22794967071513658, -0.3703845997423571, 1.6833004766529012, -0.33973713996386279], [0.61554229154924645, -5.4892730268915786, 0.42389500090909349, -0.2008044787232805, 0.32984313442075691], [-0.65948651932006863, 0.27102445566530459, 0.19685655185895837, 0.23660880197953957, 0.6324965651959672], [0.32931426140446229, -0.9964085225968351, -0.30820089381934479, -1.628584728088907, -0.23135130233209195]], [[1.2009910827966579, 0.90647622190890631, -0.75376170070061344, 1.8739174845889575, 5.7141296075932848], [-1.7536909678999153, 2.098219202435224, 3.1000295541409959, -2.0891659028267848, -42.485733637231782], [0.86932582913301404, -0.79083501013218194, -1.010741504770942, 1.1130843600852818, 1.2283271812209555], [1.075835386394874, -1.4262316152158236, -60.195902939126476, 0.72626168034274596, -8.7745729592417749]]], [[[4.9389416902186101, -7.184377179886158, -2.9189797206371506, -0.39014663722090548, 0.27140646459694845], [0.43176251050419284, 0.53968539260102733, -0.24194225737661548, 0.48963751384116844, 0.236609663418303], [-0.59854473106333417, -0.23151202196847601, 0.24583552020191418, 1.0893994513522802, 0.21481813220224161], [-1.9359291072242142, -0.43448422453712976, -0.18400570655201834, 0.31965933041924033, -1.0094130449340308]], [[-6.7169622130003699, -0.9463904354960514, 0.83992888609406657, -7.1886223369880398, -3.7922777187315422], [1.5636636501067047, -1.3405755141433269, 10.699320969529854, -1.8587384715044417, 1.0229558839099056], [0.89652515651940023, 1.6813277212775344, -1.7735044151389059, 4.2196278824867921, -1.6492610393692821], [1.15141472622535, -6.0212490334306352, 0.74956918595726318, -0.84310878463246874, -18.678282461544985]], [[16.130595997392522, -1.6823104817236243, -0.81120753088993813, 8.7867013650995194, 0.77750660931494564], [0.79269118034172625, -6.0849640171724459, 1.6121633780573619, -0.87605870973402611, 4.2475465235156777], [-2.1126435515420758, -0.74685574294107127, -5.9323598672947488, -1.5992219794646421, -0.75744681973173678], [0.7499585139498175, 4.3660409517507022, -0.82607742433659848, 0.84129609840870145, -0.94353809532829069]], [[-0.68209339348462528, -3.2946914401619369, 1.044351801334225, -1.5389607250036061, -0.90453870517949897], [0.64077065505042197, -2.1657808154617375, -1.0549044868254662, 0.74626910399141444, 1.3240648859898945], [35.238008181812482, -1.0970677505551276, 8.9642819948027856, 24.788136878108993, -0.81542007375867498], [0.64492417028507498, 3.6639981715333989, -0.97389925657571752, -0.64443692448669765, -10.876114032646329]], [[2.3533177308193816, 0.8975519369637599, -70.374661525974844, 1.4670396915194519, -1.341962066857747], [-1.0447907134269858, 1.4573989406350043, 0.9270272565057569, 1.4099108246490617, -1.4006995990946087], [-10.5902444511509, -3.7283148204154193, 12.980463062887042, -2.3554563586901502, 1.3565823470014509], [3.00029546559113, 5.7865786290505623, -1.2900141703703221, 3.214386346823868, -2.2668262000178405]]], [[[1.7569225469587881, -0.53448240570435368, 0.9636837392125821, -4.6492628082443446, -0.70098423767835272], [-1.3973309413961368, 2.1935131397521528, 11.558875742186402, -1.2541968561701284, -0.77844370132438334], [1.8494731335987931, 0.69705959302077447, 0.6852385950596197, 0.72042286113201559, -0.74463166941070491], [0.63265726716851778, -4.3345298958024845, 0.69064445324216239, 0.64884185183463017, 2.5044924372084996]], [[-0.089077485697841366, -0.025214536576737206, -0.045204429605800178, 0.063885811102433965, -0.27342706955630397], [0.040830052363460788, -0.050585752178194637, 0.0485902936577364, 0.098501404921028421, 0.035640318082765507], [0.27030265899389111, 0.19921450794087842, 0.025020732161754954, -0.041779521203344459, 0.040705224790027833], [-0.043812142943806344, -0.031536065010852946, -0.026620695947365998, -0.024584334644928554, 0.15313695824366302]], [[9.9267913249583355, -1.1328916702246186, 1.0551901229015397, 0.62679231139912295, -0.87231041973231294], [1.9343714104791812, 3.8371503735174564, -0.63957641480236926, 0.53824575969461597, -2.3752980475331404], [2.2803291212537582, 7.6600735693231412, -0.92369144399704861, -1.4683891639601914, -7.1530371936648001], [0.54003360471891293, -0.76392619855932242, 0.73339404654786988, -0.78930386344925818, -0.53682051885406135]], [[4.75638834605863, -1.3511599150524798, 0.72503929947705992, -0.97314104948090352, 1.6514768945405305], [0.96651415204485591, 0.73676412694240012, -0.71276847714103175, 2.064247029707035, 1.0322761929546693], [1.1496581458132618, 1.8532281160137434, 1.854426341441253, -3.5258107209627481, 1.0195028468306877], [-2.1185258813374808, 5.8542906712481768, -1.2699535570999283, 0.9905252969189231, 4.7085239855049537]], [[2.6942193708737356, 0.97321059217031591, -1.1240274048165775, 2.2127319939054884, 1.0499861139398263], [0.91468808719462025, 2.2044737234330354, -16.849389931291658, -0.94816842534653856, 1.3072967639558744], [-0.92130595518206149, -1.3788837246684831, -10.052345034430068, 0.99339073834144709, -0.9617996900361917], [1.8360289511803629, 5.622041938632993, -0.90996651047297472, 8.5675965852846243, -1.3163902045491827]]], [[[-0.69266058287378107, -0.56007451864221236, 0.32699010166016512, 0.47316032491939247, -2.3133181076065279], [0.36204764090017366, 0.8643230853030498, 1.8364674963410841, 0.53347790831909536, 0.47331094340326102], [1.7321360799869117, 1.8187103180150102, -1.5023116930995166, 0.42350238443321025, 0.40821109917882609], [-0.76836336328943577, -0.4476620429556239, -0.36247378551620174, 0.87910138983003749, 1.3537710157710678]], [[1.7283768183264236, -1.0767764479501758, 0.97177391315026918, -1.3957768373035044, -6.4368936547662265], [1.0047758398119249, -7.3123924653501957, -2.8551864050995373, 0.92737406614871143, -1.265762212058299], [1.646447962418649, 1.0782608011134769, -0.86864880082099516, 2.253637721518714, -0.9201747259901194], [3.8603756778655889, 1.1379906337531722, 1.2963749797060429, 1.5611760705160984, -1.3356882559351226]], [[0.54643790900452727, 0.6147615913742237, 0.99286147188794616, 0.731727863604962, -0.52101542807755108], [-1.9641587914778107, 0.61523215501163753, 0.37657617262477489, -1.9817395720513307, 1.2864960049810152], [-1.1582576837771499, 0.49055717492410433, -1.2365910920287846, -1.2501957280157963, 1.6164117909869631], [0.69145079016331634, 0.3758478228531098, -1.8078549183962147, -1.461673790306929, 0.66304674113460738]], [[-1.2280740609188028, -0.49382196329361672, -0.48296911052379593, -0.70130574843702997, -0.76482393192673159], [-13.808102544160841, -0.5690669901224179, 0.75342985975254539, -1.7567152550415377, -2.1247348230041934], [-0.57598345181105293, 0.71284172758539177, -2.6287367373562751, -0.69013542114580673, -0.69650896782634864], [-2.2757782380711231, 0.50526206977067445, 0.48402339323959431, 0.48478580943065619, 0.77494521483800405]], [[-0.32506229656648855, 0.51440838694191171, 0.8529841208499036, 0.35583076016693155, -0.73646453535892209], [-0.24128842165591732, 0.23621110010831575, -0.27404193915401004, -0.23021154178207423, 0.24055653681581898], [-1.3506679150726961, -0.32318860266021909, 0.2778571562162267, 0.27091430469060479, 0.428570678799296], [-0.21781062289744488, 0.22316442878108095, -0.40535878639590606, 0.22080623656260162, 0.58828130547607715]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-2.737732068651372, 1.7491891127401002], [3.0103918391518167, -4.8046346954755368]], [[3.7725947079627353, -1.6934825250506211], [-3.3817459734667765, -0.35224656651476849]], [[-2.4787129406568842, -0.19358666103347666], [3.4507186158183227, -2.7270937470711107]], [[-0.48643047542224682, -2.465158246389596], [-1.1168614667404109, -3.7947435069959745]], [[-3.8896164741945984, -2.6974963997832626], [0.51985886862379171, -3.9512133823763094]], [[-3.2770942391409443, 3.349373809840797], [3.5963606214204695, -2.4925095327084614]]])
      s1=numpy.array([[[4.5686394256783771, 2.9992435562152391], [3.4079919572056099, 2.725951992659386]], [[3.1170630790690357, 0.0081436875040008516], [-3.1608586642930661, 4.6368828587090363]], [[-2.1114329647415699, -0.33594545760619798], [-0.85988427992900007, -0.41995706191964555]], [[3.1755993806732032, -4.7129625937003699], [-1.7226361115929123, -4.5692339809429408]], [[3.7603149915447265, -4.44121870720143], [0.5712815526900954, 3.8215679553960129]], [[0.53213421331268496, -0.73482265369819721], [4.2198185507005821, -3.4539858235409673]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.5992445044500001, 0.58321009279666869], [0.8833330233620017, -1.7625529387215026]], [[1.2103042550840792, -207.95033260039051], [1.0698820582106325, -0.075966242246809354]], [[1.1739482058149393, 0.57624431779161855], [-4.0130034893802753, 6.4937442285299918]], [[-0.15317753189608169, 0.52305915809403525], [0.64834439451501746, 0.83049883696541693]], [[-1.0343858115452065, 0.60737751901505688], [0.90998714412506321, -1.0339246687468264]], [[-6.1583979326195024, -4.5580709753355473], [0.85225480153012623, 0.72163282076044633]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[4.1505068014301099, 4.932100871434864], [-2.6037750679428004, -3.6097552857859414]], [[-3.262580114431918, -0.30164402771077725], [-1.2183845410107796, -0.48160607395709132]]], [[[-0.82266648894098271, 2.8590433482372513], [1.5219763083051649, 3.6417812908764837]], [[-3.4346672431783798, -2.7183528984327934], [-1.5294944239018893, -0.90696555212535124]]], [[[0.078704634184497912, 0.69983173422449862], [0.77681719053803988, -0.27036218000069567]], [[0.36692809640958579, 2.3202463028031435], [2.7604504623227575, 2.9148073009952649]]], [[[1.3959314588941059, 2.5425077632008097], [3.4260432133839576, -1.370988311169655]], [[3.3666501507494964, -2.0680704451543619], [2.4125311911270657, -4.885880719437429]]], [[[1.2602069223153114, 2.5551547568091042], [4.024833514800644, -1.911719916495497]], [[-2.968393305171404, -3.4324888240967089], [-1.4535598880276543, -0.4789331338016245]]], [[[1.8684842812969809, -4.905291306652507], [3.1902572378742953, -3.0329302584313855]], [[-0.39259710304581752, -0.18854520499168714], [0.15234425882326619, 0.45203430865193983]]]])
      s1=numpy.array([[[3.8939251222890086, -1.5083900105121262], [3.3913405731707762, 0.80546789566943566]], [[1.8983928628092395, -1.681907876828268], [-1.4301124140773789, 4.503760464858761]], [[2.4793270209778679, -4.35335529314714], [1.0753347573875445, -2.6782820780506036]], [[-2.4809101801712599, 1.3754517576368688], [0.51715442768771691, -1.6214953092075679]], [[1.037079683368658, 4.1078770253288148], [1.8301847468578067, 3.195925628992704]], [[0.6618087124718155, -2.3365621899441305], [4.8081468584099287, -1.4163566316043195]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.0658928128002296, 1.2666142045729871], [1.7261948500035285, 2.393118000403863]], [[-0.96203257798479613, -0.088945365763943726], [-1.5126419656964269, -0.59792088120013986]]], [[[-0.43334891584221502, 1.5060335530373004], [-0.90491062517365628, -2.1652679918142326]], [[2.4016764062524532, 1.9007966588322418], [-0.33960385678501098, -0.20137961581262601]]], [[[0.031744353818019586, 0.28226681204340642], [-0.17844102726025401, 0.062104322251456914]], [[0.34122220442405637, 2.1576967422126576], [-1.0306795109243909, -1.0883122897633011]]], [[[-0.56266908413336558, -1.0248286227860526], [2.490849420462526, -0.99675492328798043]], [[6.5099513230551782, -3.9989417752856671], [-1.4878434599395052, 3.0131944827057078]]], [[[1.21514956133543, 2.463797910406857], [0.97978432411288563, -0.46537905217415154]], [[-1.6219091052243528, -1.8754876140181223], [-0.45481655606791738, -0.14985740890114996]]], [[[2.8232996122373564, -7.411947310774349], [-1.3653637175180762, 1.298031043849043]], [[-0.081652477473545967, -0.039213695118713518], [-0.10756066334133976, -0.31915288746162512]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank3_Symbol_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[-1.2276922433567661, -3.0836618159809257], [-1.0839325610777584, -2.9431284679272252]], [[2.4922330291860124, 4.7252661113157437], [-4.8152233446823365, -1.6072007882423023]], [[-2.0321000852926128, -0.86768004503741381], [2.5423860660486799, -3.8037273127983773]], [[-2.7343681909284046, -0.55684496186163646], [-4.1874295518820714, -0.97078991967338446]], [[0.0098000139893805738, -2.0229845480236519], [-4.8761306939927653, 3.0824292066058785]], [[0.86199098848755629, -2.9058050774433539], [0.93218121123682351, -3.334239527086813]]])
      s1=numpy.array([[[[-0.19819215032382242, -1.5934395470735274], [-2.9810165344265762, 2.9583511976698675]], [[1.1795727862067054, -2.8715897262506749], [-3.7188144107794265, 4.9409520157501792]]], [[[3.3162391769907806, -1.2454143794833383], [-3.8423119413537532, 3.3548864353694263]], [[-1.1320618812618921, -0.41682544624776607], [-3.5374328612834627, -0.6731248711006268]]], [[[4.4141788256153784, 0.21189903977962299], [-1.3307446228140698, -0.90857988025482683]], [[0.79133860844213721, 3.278475446582938], [-4.221405690443345, -0.33108699219326887]]], [[[-0.86501682168598748, -1.8410771713910123], [-1.8803805004316034, 2.9116159583454593]], [[1.1995891710062976, 2.709735121250624], [-4.2197892931271603, 1.272576567634224]]], [[[3.0044110890052362, 3.5472555901552365], [-2.9601609264623918, 1.4660538744126566]], [[-0.85499262802248133, 1.0432009292476945], [-4.5566039090800539, -1.2690352306468111]]], [[[-0.1306910070591254, -2.7049078305415919], [0.97310731656877447, 3.5286093692263893]], [[2.2675081213796089, 0.84955135596605302], [3.0167756900945228, -1.8447943948873604]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[6.1944544289512118, 0.77046678401544377], [1.0344329796124712, -1.042358263079028]], [[-0.91891960695659247, 0.3774677667805309], [0.7914157962269418, -0.59566020041187817]]], [[[0.75152390891404663, -2.0011275525981307], [-1.2297976279486826, 1.4084727463495845]], [[4.2534983505626744, 11.552133844103437], [0.45434100130431099, 2.387671080425787]]], [[[-0.46035744485483521, -9.5899447557951003], [0.65202596363122423, 0.95498487683224731]], [[3.2127663669206399, 0.77547814753303612], [0.90105703922498337, 11.488603909204587]]], [[[3.1610578226662698, 1.4852002042165744], [0.2961341929113942, -0.1912494538524464]], [[-3.4907197006199788, -1.5453279986825603], [0.23005649150646593, -0.76285383871095924]]], [[[0.003261875189199015, 0.0027627030926609102], [0.68340357104884364, -1.379884179791222]], [[5.7031260085491073, -4.6742008727975275], [-0.6764751266756911, -2.4289547935046718]]], [[[-6.595641183617067, -0.3186766583151795], [-2.9861095769882495, -0.82349865722893023]], [[0.41110380264907764, 1.0972629314172648], [-1.1052328278945869, 1.8073773079142488]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_quotient_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=quotient(arg0,arg1)
      s0=numpy.array([[[[2.7241688917539157, 4.4478825135710469, -2.1605115054899349, -4.9151193024880815], [-3.1211617799882276, -1.5832365676046201, 2.6106452323418328, -3.6095487012242469], [-2.1410238762197906, 1.2606857379092906, 0.43261448274176217, -2.4524524085653256]], [[-4.3134182503123997, -4.3112101436303787, 1.7299810547534076, 2.3483305437433089], [-3.7721504119342386, 0.36410642966979712, -1.9791964190189217, -4.4179888465084911], [0.28517030090644013, 3.3144344525858589, -1.2105082918800325, -1.063828423154245]]], [[[-1.7814514582684495, 0.7385296487772175, -2.6485772752835999, 1.4255726389588581], [3.7948891173696069, -1.9603699938817929, 4.6172244434626801, -0.041282546452857005], [1.9345287635962514, 0.66010431270737246, 3.4309566341723325, 0.64875791113251857]], [[-1.7482604522386125, -1.632489706748943, -1.5053529190587378, -0.098684206841513245], [-1.3156226114344074, 4.1819593939689241, 3.4489328068329179, 3.938399598982274], [0.33439716882553316, 2.6197437811420308, 3.7832926926708161, 3.2728423092043926]]], [[[-4.0021793708733249, -2.3556275270139415, -1.3059674962160353, 1.1891239766883555], [3.5280237954408076, 0.53353400890376346, 4.8469585508859083, -0.96324448977167432], [4.0394512305649588, -2.4395116183988295, 4.9914796479684362, -1.1222584832277471]], [[-0.083198058146894738, 2.1455294362207287, 1.875409090989665, 0.99651483606748492], [2.3667840562713778, -4.0607636199798938, -1.6543116489102272, 3.6140413626681465], [4.0588121593418656, -2.7350000512281802, -4.0306732668076464, 1.7919489710014656]]]])
      s1=numpy.array([[[[0.13691947586699271, 0.81282156025427987, 0.54958054827600655, -4.8395047927832824], [-1.1867826892555522, 2.017352515271206, -0.43501574399841481, 4.6151666273402405], [0.41822727718839658, -2.0341447776642974, 4.5318287102865522, 4.2268144392609202]], [[-1.5716687471428372, -2.3984129980545612, -4.5900281442957507, 4.2907354098304236], [1.9403505453834686, 3.560687397417416, 4.0469419821527133, 3.2707054507083608], [-3.2876393588445154, -2.6732940204125177, 4.7051119235821481, 1.3272121568989084]]], [[[-4.3042448559547601, 0.16035525791268768, -4.1391672177826671, 3.2146687177816062], [-4.0492768390664118, 4.7295475408493601, -0.64574128447097756, -1.0684221102153337], [4.906073133001625, -3.1726866980255131, 0.21068695181940456, 2.9516812400257315]], [[-3.2764812066668481, 4.9847150231598505, -1.4541963174475736, 3.5269970598870675], [-2.360274251258081, 1.446889644192682, 0.89010807207190989, -2.0544608628745564], [0.81979116996534618, -2.2293118754086083, 2.1204981091383495, -1.2881673022267059]]], [[[0.15407501769049503, 4.7764959949689505, -0.36131985801291222, -3.9652432518986389], [4.6520506261487267, 3.7031616860084888, 2.5050701199715375, 0.86072973667468666], [1.0125940032365675, 3.0985321845199074, -4.1586132312019464, -3.8030805430283419]], [[-4.8789143601056892, -2.4667927866048025, -4.4682206185602444, -4.0304983546593469], [0.10244078634959219, 4.0976788873899377, -2.8352119171015278, -1.4951212333181765], [4.6475351738921606, -4.9688644247071254, -0.89115714927060985, -0.15656900312583488]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[19.89613876699504, 5.4721512458153656, -3.931200826279786, 1.0156244312057623], [2.6299353775930765, -0.7848090780464192, -6.0012660883173599, -0.78210582470441814], [-5.1192832055651287, -0.61976205024937825, 0.095461349137003798, -0.58021293430476439]], [[2.744483058630411, 1.7975261754866054, -0.37689987955811088, 0.54730257623508849], [-1.9440561505285912, 0.10225734220136407, -0.48905974628430832, -1.3507755171140998], [-0.08674014080627962, -1.2398316186987941, -0.25727513214147624, -0.80155114434750852]]], [[[0.41388246205461077, 4.6055842408319521, 0.63988168052375294, 0.44345864663237339], [-0.93717699929959453, -0.41449419356713729, -7.1502698596161984, 0.038638798334617738], [0.39431307099424984, -0.20805846134072462, 16.284618504107748, 0.21979267352285742]], [[0.53357866014348709, -0.32749910459557119, 1.0351786075905864, -0.027979668019534176], [0.55740243352362551, 2.8903098524160931, 3.874734894612085, -1.9169990872796436], [0.40790530695722016, -1.1751356147339684, 1.7841528253982422, -2.5406966187909044]]], [[[-25.975524331354475, -0.49317062748406099, 3.6144359831154502, -0.29988676637151551], [0.75838035287280126, 0.14407526706694826, 1.934859432573879, -1.1191021394160723], [3.9892110931465208, -0.78731201521368488, -1.2002750365235986, 0.29509195783008785]], [[0.017052576045850591, -0.86976476008499759, -0.41972168589875081, -0.2472435784313102], [23.103923160004129, -0.99099117612079224, 0.58348782993316795, -2.4172229529824643], [0.873325753862502, 0.55042758615604337, 4.5229657531296841, -11.445106855290328]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_float_rank0(self):
      arg0=3.42824474308
      arg1=3.51913263797
      res=power(arg0,arg1)
      ref=76.3815911088
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_array_rank0(self):
      arg0=3.58106943462
      arg1=numpy.array(3.94140059405)
      res=power(arg0,arg1)
      ref=numpy.array(152.611109443)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_array_rank1(self):
      arg0=0.92071149837
      arg1=numpy.array([0.20790983882697253, 3.3272017472264568])
      res=power(arg0,arg1)
      ref=numpy.array([0.98297152348190819, 0.75968205740072103])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_array_rank2(self):
      arg0=4.9029509735
      arg1=numpy.array([[1.3223707114167753, 3.4539720654926076, 2.2737047343486583, 0.86627469468020635, 3.3080148951526542], [0.70024571363620292, 4.9563893324522983, 4.9774534021685071, 1.226228306451413, 3.297561567091893], [0.28601780224475726, 3.100520935527308, 2.8904156603441429, 3.8285004485980951, 4.9201424225362507], [3.0268675914884247, 4.9467740792631023, 3.4245527629951309, 0.21147777472794668, 4.8385854925437899]])
      res=power(arg0,arg1)
      ref=numpy.array([[8.1854176267676291, 242.56106575701259, 37.144678321252648, 3.9639293360560126, 192.32859624752308], [3.0443174723572981, 2643.4824912150621, 2733.5075907656947, 7.0251983869072756, 189.15868339837419], [1.5757348895348264, 138.28596054848651, 99.016898088970905, 439.96389902179982, 2495.4535226040844], [123.00523163962876, 2603.3796999897359, 231.47724914370048, 1.3996402602379747, 2191.9865626446012]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_array_rank3(self):
      arg0=2.55584198527
      arg1=numpy.array([[[4.4706774435325842, 3.0745426708198615], [1.0476877992086231, 2.2067347987307109]], [[1.2980017484160911, 1.0224295408650392], [0.92167202150723626, 0.086266827939902688]], [[0.84136276580587332, 0.56857095578494077], [1.9695688771874242, 3.4231293089264527]], [[1.2285572574279928, 0.42962743484909793], [1.6130156048480968, 3.069131667280474]], [[3.2800206794935534, 3.1367477127113954], [1.3309137595065936, 0.68055666663970726]], [[3.9461073250526622, 1.85751979204157], [4.3804712489641169, 1.0396524227498338]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[66.367133729940093, 17.905261705716622], [2.6728119211970958, 7.9308449551092783]], [[3.380491471902217, 2.6102061019891605], [2.3747215080261861, 1.0843179955445184]], [[2.2023378029240344, 1.7049513689851274], [6.348429235670543, 24.83370856153515]], [[3.1672245381543673, 1.4965381315637294], [4.5431670056228564, 17.81457661126521]], [[21.713005776686298, 18.981535549977536], [3.4865235914676265, 1.8938691977091644]], [[40.567001020009954, 5.7148188835522662], [60.980498513996217, 2.6527340414735496]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_array_rank4(self):
      arg0=1.48995243945
      arg1=numpy.array([[[[3.9381451465550694, 4.2813759106334075, 4.4882503877432587, 1.795480730364083], [0.76692598614142227, 0.60640085608664629, 4.3160640480664085, 3.2538786070429531], [2.1415141156498687, 1.3387877091563187, 2.4331428880816253, 1.9125730743077822]], [[1.515011551700328, 4.4287604053215741, 2.0934708868132152, 0.050548798609006171], [1.3642652402837687, 1.5024677046637349, 0.11854560149891435, 0.38947520409293035], [2.1472512398904851, 1.07799877284641, 2.0477066651559306, 3.2840278634198388]]], [[[2.9248811189905757, 0.87004888091930477, 0.48444717003521881, 2.7202006917683819], [3.6053323537243078, 0.14907425091588744, 2.4693819471627845, 1.3118830573303941], [1.0472232858186117, 3.7768952183493942, 3.3091433764859532, 3.3246310081514814]], [[3.9422188797266675, 0.75464019862361686, 4.0437063307596102, 1.7903839681979576], [0.34099911191067789, 3.2309740655281978, 3.2334432747888697, 0.056432428120259039], [1.2769994032429888, 2.0468539276485513, 3.9519471411268055, 3.9485902151485743]]], [[[4.9773479045896245, 1.0082134105536371, 1.5507704922046086, 4.450876001456348], [2.9937345367909405, 4.0790534341133107, 4.427932099682887, 2.1527178350718663], [1.2852505127079303, 3.9119620883233468, 0.71505053544282471, 1.3489363492457234]], [[0.060670735925217496, 2.165534379045007, 4.9826768735684022, 2.0839094579077386], [2.0370466481506253, 1.6580726377392354, 2.8741147156713271, 1.4368917987272416], [2.9085882184172545, 2.6029505600916756, 2.4228636400695205, 3.2837472530552465]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[4.808150671674678, 5.5133576462168206, 5.987439202102685, 2.0461040606308707], [1.3577206329695757, 1.2735379617807969, 5.5901465559654255, 3.6600075886806032], [2.3488276515951068, 1.7054582339592144, 2.6384763580957604, 2.1439017567059828]], [[1.8296086080487661, 5.8470807705981356, 2.3042595149484462, 1.0203605449126498], [1.7228723196769249, 1.8204801452172406, 1.0484043807704273, 1.1680094539718866], [2.3542070862320861, 1.5370204319562191, 2.2625922183456404, 3.70427316207169]]], [[[3.210027104164344, 1.4147133692163987, 1.2130896065414605, 2.9584463466283522], [4.2106008774478285, 1.0612447300809318, 2.676879440099619, 1.6872597439760344], [1.5182740722420445, 4.5087279416622641, 3.7415565963830772, 3.7647344606026936]], [[4.8159672701468788, 1.3510855791331999, 5.0148546821351774, 2.0419499770575631], [1.1456491481250277, 3.6267327269563427, 3.6303053043352347, 1.0227571854126098], [1.6639530135639309, 2.2618230131656203, 4.8346851106593798, 4.8282179491704769]]], [[[7.2767810898365735, 1.4948401072278534, 1.8558832693299332, 5.898871071573045], [3.2993790422226614, 5.0860367708281213, 5.8451499035469112, 2.3593443205266507], [1.6694365712193511, 4.7582130678472172, 1.329924606044018, 1.7123737142241253]], [[1.0244871070969046, 2.3714326654936118, 7.2922599286453451, 2.2954911030350593], [2.2529952097942725, 1.9370125177476389, 3.1457004121159695, 1.7734951672019741], [3.1892401188779496, 2.823314184801859, 2.6276839292705674, 3.7038587076331693]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_constData_rank1(self):
      arg0=2.07890162999
      arg1=Data(numpy.array([3.4039479690055297, 1.6365293576089985]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([12.075084921032165, 3.3124137080873428]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_constData_rank2(self):
      arg0=2.04128325558
      arg1=Data(numpy.array([[0.093286927326640789, 3.4230322921669685, 4.3396831626859207, 3.4979135754014528, 0.93720955592473398], [3.4279642814590354, 1.9517239522494159, 3.3882315292458052, 2.5247462986534477, 4.3537308702118027], [2.830422531621033, 1.2671150156016149, 4.3585614005666935, 4.5146695435289699, 0.89492685081389656], [1.7316573503783941, 1.0895627765379317, 2.4350503893993847, 1.2704367697570658, 1.639302289414591]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[1.0688331723386533, 11.502941505139352, 22.124935658263272, 12.134303860016077, 1.9518404270096741], [11.543495844603262, 4.0257392418180622, 11.220805669327445, 6.0593653866073982, 22.347834523979934], [7.5362831435551971, 2.4699211598110691, 22.424999607029392, 25.067491050855718, 1.8938289899421992], [3.4406958275085726, 2.176000900306267, 5.6836861167122983, 2.4757826392041156, 3.2212552925826086]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_constData_rank3(self):
      arg0=1.287637079
      arg1=Data(numpy.array([[[1.0181641989150163, 4.3702178846738127], [2.4011631888211911, 3.5879800386802971]], [[1.8055651054447299, 4.3885955939119761], [3.2044848626847435, 4.4030367314215217]], [[1.637376639217561, 4.9023557241906435], [3.2052151672325246, 4.8726673780846985]], [[3.0432424113438072, 1.474091033784203], [0.10976759358817229, 4.69061342710026]], [[3.3365244210915939, 2.6046684841981054], [2.1884352104239042, 0.075913296130813671]], [[3.5236881399905351, 1.0977526043567034], [0.71970610972905769, 3.6373289557728006]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[1.2935635952092714, 3.0187100775878304], [1.8349830147969921, 2.477061525826771]], [[1.5784806624397163, 3.0327677773614501], [2.2481823094952018, 3.0438601841887691]], [[1.5127712764426395, 3.4533982671390642], [2.2485974239340183, 3.4275758963141061]], [[2.1583812380294258, 1.4515953753032913], [1.0281388391798263, 3.2733974843382603]], [[2.3244949551067471, 1.9318597851885995], [1.7389051945404239, 1.0193768921628645]], [[2.4371258878327544, 1.3198545449157557], [1.1995518757787387, 2.5081585297391937]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_constData_rank4(self):
      arg0=1.02235748291
      arg1=Data(numpy.array([[[[0.19894150668838331, 0.88571485950235207, 1.04083931472546, 2.1244673031808672], [2.7905487792556944, 2.2624625504986429, 0.63675026156544212, 0.34225009193031652], [4.5821298531763803, 3.2826310087733495, 3.081625923955662, 1.3173597846851326]], [[2.0139529664159821, 4.1543964089025947, 3.4422267277815464, 2.9413738572829295], [4.3061255503874101, 3.4419194615886721, 1.1864079337348854, 3.5168993356667451], [4.4871348830673465, 3.6689468054968923, 2.0673449144423044, 2.8503690791235408]]], [[[1.6779350003112237, 1.7464763286360121, 0.15276907716440305, 0.47851391132473309], [0.049048141027994298, 2.8672028310891196, 3.7165093844077206, 3.0011962704667687], [1.8464132659613657, 0.11914176136693869, 3.7912249112954379, 3.1585078796567894]], [[4.9756020393239284, 1.2294200006798195, 2.5090284134502712, 0.087989425714954272], [4.4126212864045495, 1.2106033036186603, 4.6686715342866005, 2.8212179050004504], [3.3433084130471138, 3.8144995177625565, 0.057904241437857315, 2.9376287547560569]]], [[[0.23815354921721071, 1.7297290907950356, 0.66865271187321118, 4.4935110239505676], [3.4683500361559951, 1.4918529153994098, 2.4264215292768085, 2.3455256742515167], [2.9127447018241925, 1.8197231527862217, 4.9636554656236793, 2.3608544905334612]], [[2.3902641243523388, 3.9196865389666797, 3.4585421502717875, 0.25439964728529268], [4.4676188731981545, 1.5101262183995887, 4.3913089432906673, 2.2759577668887712], [3.3686952526023433, 1.5215265977195453, 2.1525838338243775, 2.1634378347727594]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.0044085281580768, 1.0197772636944014, 1.0232810958230047, 1.0480953453034072], [1.0636457917040103, 1.0512982228084395, 1.0141789044484792, 1.0075962728602219], [1.1066268038711851, 1.0752820190881456, 1.0705135677889648, 1.0295568116052407]], [[1.0455373392379763, 1.0962099884716126, 1.079083237273148, 1.0671988933063201], [1.0998938571511381, 1.0790759059743424, 1.0265800364739386, 1.0808663857921086], [1.1043048240678828, 1.0845063249339935, 1.0467723887405576, 1.0650536065299703]]], [[[1.0377980271119558, 1.039372035937403, 1.0033836219682701, 1.0106366971998382], [1.0010851024467922, 1.0654501089930242, 1.0856474639143034, 1.0686114608622885], [1.0416713042752945, 1.0026378424828855, 1.0874424915686822, 1.072334938972324]], [[1.1162966258085767, 1.0275568290090866, 1.0570454179789974, 1.0019474472084211], [1.1024868844815157, 1.0271293925434384, 1.1087464080617782, 1.0643673282871158], [1.0767256408366603, 1.088002266017702, 1.0012811532938624, 1.0671105235277913]]], [[[1.0052797541284548, 1.0389872258225978, 1.0148945606995472, 1.1044605246222376], [1.0797067154085074, 1.0335367779564113, 1.0551164456741318, 1.0532308395604923], [1.0665235429693249, 1.0410567423606201, 1.1160017913159548, 1.0535878808632055]], [[1.0542732337851357, 1.0905357010317824, 1.0794725909497267, 1.0056409366147145], [1.1038283942129918, 1.0339544576884487, 1.1019674688963381, 1.0516119724589499], [1.0773302131600835, 1.034215125929141, 1.0487471390646399, 1.048998863531672]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_expandedData_rank1(self):
      arg0=0.0865810990086
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.1716054657432631, 4.7512027432972621])+(1.-msk_arg1)*numpy.array([1.4649935475640603, 0.20644989975348804])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.056895653409470823, 8.942968275597907e-06])+(1.-msk_ref)*numpy.array([0.027754390163784875, 0.60343584494639479])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_expandedData_rank2(self):
      arg0=0.675951046523
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.2820097421969652, 1.4750955453653107, 2.704761008352083, 3.1356136693162946, 0.7103309776106761], [3.0311210130463508, 0.99544863771055925, 0.10847781634729216, 0.46717090511294235, 4.7733157430035327], [1.4252144443841523, 3.7137582041432418, 2.7287484083564664, 0.37428133553174192, 4.7836937627258935], [1.3894303062505644, 4.9187478042351627, 1.4696400566047974, 2.1531297246246024, 0.4682810758736714]])+(1.-msk_arg1)*numpy.array([[2.5347565181105489, 3.9103057159804782, 2.4150569058756211, 1.7636075723931635, 3.7415987746395349], [0.34748169746431112, 2.6972267304824, 4.1336912883264967, 0.82899478345782895, 2.4502756808218011], [0.86261317070676891, 4.9540946505419843, 0.45140907149596182, 0.61607563732486981, 4.2190343130575316], [0.39944577996772707, 2.0447376744027528, 0.055549024852331631, 0.74149781674627147, 4.502130511787402]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.40913339676125721, 0.56118846437677605, 0.34670602269154321, 0.29287339159313647, 0.75715279442280958], [0.3051072478094044, 0.67715698414964187, 0.95840611751175242, 0.83280121661567763, 0.15421682589685298], [0.57225915609000433, 0.23353197104480941, 0.34346421466834515, 0.86365531621389313, 0.15359130034112953], [0.58033543134167243, 0.14567867530823395, 0.56238875802807375, 0.43031398215043182, 0.83243920891772583]])+(1.-msk_ref)*numpy.array([[0.37057537609068164, 0.2162303291498934, 0.38836102378294896, 0.50123020877698732, 0.23099952572884472], [0.87276769062713322, 0.34773055341788833, 0.1981171760793829, 0.72277084402744352, 0.38304117466569876], [0.71331712418584114, 0.14367593231474241, 0.83795789779336505, 0.7856243324727703, 0.19160488687176797], [0.85518556626590592, 0.44897410473633237, 0.97848001023044562, 0.74796715145718162, 0.17149688411940794]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_expandedData_rank3(self):
      arg0=3.50594827227
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.0585495063317065, 2.2072841657342335], [4.5929940605562152, 3.53867746632198]], [[2.3658472707893115, 1.0157804474502903], [2.5912186128381465, 2.303386734873869]], [[3.5699890195849346, 0.17800820021886163], [2.8535478365500655, 0.41648996954758838]], [[1.6868350492905462, 1.4655410391348529], [0.64339070470030235, 1.3756173185234977]], [[3.3298336975673166, 1.4920443565468071], [3.8262245890902937, 4.9800191799027473]], [[0.17072953948743816, 0.90029900749775571], [2.587973782054517, 2.6768136738584647]]])+(1.-msk_arg1)*numpy.array([[[0.64079186677948063, 4.920953374054772], [1.8517659736479612, 1.9703802352680586]], [[0.30145056688507804, 0.13081356452402204], [1.7605286210159314, 0.78065262400123547]], [[2.1201723232360425, 4.7700679823615211], [4.1414121353713158, 4.5571393008552041]], [[2.0316266229070692, 2.4902030100138104], [0.77416243467399715, 0.7637117618261261]], [[2.2862924868768153, 4.0129553521789605], [1.213522608807772, 2.1104782688990769]], [[2.3598928207056105, 2.3788902964909306], [0.52275306968126289, 3.2709841136196864]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[162.59983843543819, 15.941917694697498], [317.89847496769789, 84.70147259280121]], [[19.450313917529186, 3.5760433822187019], [25.805374290220975, 17.984475279828455]], [[88.094676980710119, 1.2502010152808862], [35.861509618318735, 1.6861881302863162]], [[8.2984507213901288, 6.2868749872424727], [2.2414177003496962, 5.6162170893532686]], [[65.179605292818465, 6.499410651546361], [121.49214955319928, 516.58508121530383]], [[1.2388376418046798, 3.0937687946432946], [25.700546654623391, 28.730476108204169]]])+(1.-msk_ref)*numpy.array([[[2.234122261369671, 479.69200882280364], [10.205923172551532, 11.843334549111432]], [[1.4595935248659127, 1.1783327542007143], [9.1021855168001018, 2.662587239736216]], [[14.291622998678642, 396.97146655408687], [180.41146208162297, 303.91670642152963]], [[12.789140733908948, 22.734023584089332], [2.6409972894860103, 2.6065998827801602]], [[17.602920632394333, 153.56072843844825], [4.5828305195571817, 14.118877728420378]], [[19.305568624364842, 19.771177139988808], [1.9266319126697893, 60.54107788298279]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_expandedData_rank4(self):
      arg0=1.46840052023
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.83942261933999351, 2.2124559234215084, 1.9731687887653759, 2.3325129202722463], [4.3387549502887968, 2.3426981118500443, 0.036629603358071368, 0.48453556543231902], [1.2034078237050632, 3.2462724624406505, 3.7024025294427703, 0.94483089507011542]], [[0.87980871764446866, 3.8139301162317767, 1.7506070802909894, 1.5536134907892172], [4.1774200647263369, 3.2878628101346865, 4.8910058452601337, 1.7664631940403468], [1.841728941725596, 3.7561223491835327, 4.2362625790177155, 1.5478533398707828]]], [[[4.8138540352183421, 4.6779325121485282, 1.0966018830255695, 0.51515122334152574], [3.8333743309248085, 3.4042771860805154, 4.0264585937272797, 2.0192469235777057], [3.2121050065663703, 2.1505060864940564, 3.7780273637415647, 4.1333257475249372]], [[4.6502524863720431, 0.44757367023501482, 4.7294695786168885, 0.12024964727621361], [2.6295475865782985, 3.9959021146183407, 2.5443630355132258, 4.1187851046426163], [2.6739922811460168, 3.8945909655595079, 2.4698856788476129, 1.1468991333212106]]], [[[4.1264941117437788, 1.1938335949831023, 2.5935561860094154, 1.6849325681080858], [1.1424984235805251, 3.63957805155255, 4.4130017257743672, 2.2551791811458912], [1.7956190205303808, 2.2423178703974052, 2.8002125277272882, 0.67278433311857433]], [[0.55968608029367439, 3.7776811975088931, 1.2326255339887497, 2.8308445093674659], [2.9382913971723488, 3.5877685084144573, 0.23281680314525005, 0.30308988745222781], [2.7505845823868662, 0.4158728506943431, 2.8953003328586089, 0.80113417926709984]]]])+(1.-msk_arg1)*numpy.array([[[[2.6964138009999084, 1.524812785823817, 0.8096109067892715, 3.4817990844709725], [2.9093050247133863, 0.015374867370276879, 0.012732310277273388, 0.31930475512690021], [2.2565795086565434, 1.6247419566309309, 1.9858531691197754, 2.1805933406154732]], [[3.9033299563424442, 1.2530713533146187, 3.2498640479210201, 4.3049203501036768], [1.8344740871334784, 1.9798564298379193, 1.692876732534758, 0.55559414444312061], [4.0166587569763763, 0.51355859047180463, 4.0126202737811454, 4.4523551138676858]]], [[[0.75482772951140786, 3.47469469890952, 2.4674313205608693, 1.8555969554693286], [1.1020890172024866, 0.80035927991708933, 1.3377965955295603, 3.8463225797006708], [0.68012404082574496, 1.5883102865304199, 2.6939491060201228, 2.949857913584967]], [[2.9617776205338382, 4.6717326478668291, 1.3328617636841809, 2.7017336598363229], [3.8614618605702264, 2.7163770705648003, 3.7263804478456399, 2.6432914128170206], [0.093713863627373162, 0.53144071904312096, 2.6270545927868167, 3.4682644458204535]]], [[[4.1274746800907511, 4.3653612747933783, 1.3234554771559257, 3.2414120153729797], [4.2426335706341627, 4.5603112199277893, 3.0381387863951415, 3.3885001109636796], [2.5548610087131651, 2.7431879467691416, 0.94754974857032803, 1.444947699677483]], [[1.8613570593364883, 2.0761210057267259, 3.6544842048614976, 4.5039637152728629], [0.50732790877181366, 2.7359811962424625, 0.078024660905333373, 3.8853838912842478], [2.6632751489932143, 0.56997183648450001, 3.9876321403138797, 2.589370398854427]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.3805529621676973, 2.3395706690036202, 2.1340884658356698, 2.4500052849444036], [5.2953848567031816, 2.4596106490706937, 1.0141716097537248, 1.2045979188997362], [1.5877496013776295, 3.4803490500665424, 4.1469195366924536, 1.437605965740173]], [[1.4021396537587043, 4.3284596815034, 1.9592019979058661, 1.8164017523018652], [4.9771380405413348, 3.5364044163575379, 6.5469280784013781, 1.9711729047959889], [2.0290016448683001, 4.2333918251285416, 5.0909313141209873, 1.8123866834323517]]], [[[6.3557266781312665, 6.0323627559376281, 1.5239194969589691, 1.2188497247270749], [4.3609141525430122, 3.6981542877975562, 4.6966975098082004, 2.1722024705370657], [3.4349636783741979, 2.2845474175637319, 4.269167540046074, 4.8935360315856569]], [[5.9685548463767164, 1.1876137537451097, 6.152988842299254, 1.0472804485139264], [2.7461586934772564, 4.6418853437179264, 2.6577434348575992, 4.8662762977650624], [2.7934505289252014, 4.4646888527916664, 2.5827770249177586, 1.5536523508269309]]], [[[4.880709614152825, 1.5819203200897478, 2.7084489990047382, 1.9103888816686976], [1.5510279076964011, 4.048029682080668, 5.4486034051680106, 2.3782872622207685], [1.9933759352180327, 2.3665651810521187, 2.9322442426234119, 1.2949422048146613]], [[1.2398825871763033, 4.2685998298640548, 1.6056719755097386, 2.9669547363611883], [3.0919883311513026, 3.9682547926187164, 1.0935640129793784, 1.1234891684017629], [2.8768683677754168, 1.1732379710631935, 3.0413403395955476, 1.3603944621801889]]]])+(1.-msk_ref)*numpy.array([[[[2.8176165689285462, 1.7964149974097783, 1.3648318570728808, 3.8099487235966927], [3.0577475620918073, 1.0059240985750251, 1.0049034016111194, 1.1305095924067898], [2.379567051345524, 1.8667205463365228, 2.1445132704764358, 2.3111070925413619]], [[4.4797033008009786, 1.6183337845680486, 3.4851545250736042, 5.2269989298788868], [2.0233544380087336, 2.1395784495142216, 1.9162281785463737, 1.2379350053001585], [4.6790484398650438, 1.2181042024163575, 4.6717946223428326, 5.5316041656612533]]], [[[1.3364074177487835, 3.7995643384970172, 2.580342872299437, 2.0398404582295218], [1.5271353270593468, 1.3599895385045062, 1.6718759243227426, 4.3826610272897426], [1.2985987357368456, 1.8407757431020637, 2.8149499120415595, 3.1057583123657344]], [[3.1200129323424717, 6.0180118212042952, 1.6687093298204425, 2.8233809612616518], [4.4082253537024751, 2.8393089722166769, 4.185296143740536, 2.7606968048836986], [1.0366583388778634, 1.226501198237663, 2.7435298390754403, 3.7901897283861592]]], [[[4.882548565773499, 5.3497889433996511, 1.6626900837033924, 3.4738564107927408], [5.1034069739083137, 5.7658452314926585, 3.2128972510007618, 3.6758070584799309], [2.6684838788207519, 2.8687050824131872, 1.439108347031846, 1.742134251639984]], [[2.0443593796182831, 2.2201863733300202, 4.0712774376733911, 5.6423718981281032], [1.2151919568425515, 2.8607736434326858, 1.0304287984377782, 4.4489246266284725], [2.7819728653554869, 1.2447916974865152, 4.6271609808178207, 2.7040971249043588]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_Symbol_rank0(self):
      arg0=3.79893155089
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(0.837095621287)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(3.05655403002)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_Symbol_rank1(self):
      arg0=3.37179185402
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([4.9785299218449088, 1.2295354781040375])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([424.59079426233438, 4.4568039294396389])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_Symbol_rank2(self):
      arg0=1.59325898848
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[2.7730234585827391, 2.9461092676041103, 4.2274406561101046, 0.068855396208130359, 0.54373103776876863], [3.2301184624753718, 4.3398223824586157, 4.6803261661102153, 2.7040945544227708, 4.4134766722703631], [2.7169806418465217, 3.0750700517194494, 2.0994094988931078, 4.8106479030105787, 4.7658181915844073], [3.3968787622908954, 0.73590036901179368, 4.6132011354578211, 0.6996787743144639, 4.7071770005681524]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[3.638688427104106, 3.9441894832299478, 7.1639682991336526, 1.0325914118297153, 1.2882180553691152], [4.5020359837705062, 7.5489569327556989, 8.8463929103012902, 3.5237208093815564, 7.8124312250817978], [3.5449341410359594, 4.1883674961310424, 2.658777175732808, 9.4000147825841314, 9.2057695023487245], [4.8656660562380614, 1.4088430687472548, 8.5740842231692245, 1.3852733665656678, 8.9577261755335762]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_Symbol_rank3(self):
      arg0=3.1888699337
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[2.0703687952090286, 2.275341624979268], [3.1384119238454771, 3.3097628721343817]], [[4.3730937241758312, 3.3789204532637624], [4.3772155060581968, 1.4451006497409109]], [[4.4130700497128137, 2.9304824324160625], [4.2586982029890601, 0.76289636370209746]], [[0.74294538919860864, 0.34990382554630711], [0.25502203438128246, 1.2436818346063017]], [[3.5720944046207084, 4.1964999189946326], [2.4707981767559435, 3.5515437059082759]], [[1.1369844860779532, 4.4927939454357499], [4.9349641284896402, 0.61744426043616651]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[11.033515941052148, 13.994126263620617], [38.073232993447121, 46.442800608033657]], [[159.38628737871892, 50.32093523727967], [160.1499604277187, 5.3432546855619911]], [[166.94926197421276, 29.91567535833963], [139.58448859208565, 2.4222707658849734]], [[2.3668712952545978, 1.5004600785975324], [1.3441213602927933, 4.2302296662486345]], [[62.956157890801521, 129.87088937755308], [17.554355997076811, 61.473526499351514]], [[3.7378961808817901, 183.1202580692547], [305.79442984444177, 2.0462922294571388]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_Symbol_rank4(self):
      arg0=4.20121411181
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[0.030620161997227259, 3.7968509523598986, 0.86504280799727218, 1.4961108151497033], [0.087502892392850085, 0.3214242451453862, 3.1140595044956174, 0.26016290018886629], [4.2770994554391457, 0.05678169627323406, 3.0518119955361924, 1.8994230609869798]], [[1.7928419609360078, 2.6123183979092732, 3.966896125947041, 2.0931727659559121], [4.9266597528425224, 3.45415217723566, 0.45140092308015811, 2.5625668942081741], [0.47283771938580932, 2.4525671736742178, 0.95961464052129541, 1.2846254814610383]]], [[[2.7987413020281071, 2.4874402075087976, 1.1487657852774535, 1.2003330696288617], [2.2247595836632015, 2.1044724040772054, 3.2261093278320625, 4.0022674814954593], [4.1823348550333863, 1.8273733602996713, 1.0911159183987078, 4.6009284293110024]], [[4.5033448129684848, 3.3384799887333378, 1.1352550398709591, 0.44443337050411369], [1.3723823684924896, 3.8448315329037683, 3.3647142346119105, 3.8954477121950353], [4.5386949728493136, 1.8521814231584852, 1.1425016758785782, 4.0052548993711676]]], [[[4.5410022216094807, 2.0085822090762693, 0.4834444913912595, 1.6630516099535486], [2.7502352663303387, 3.9766868466385756, 2.0415730927257028, 1.3535032715096351], [2.2982996728430551, 0.96568410671546179, 4.750699188413841, 2.3625781897853084]], [[3.0122587763573425, 0.38422968953347741, 1.8087742303812187, 3.5995990480925117], [2.6380115255948575, 3.0410743330342633, 4.5307904997144188, 1.6940055206124278], [4.098966507097038, 2.4539562011726814, 3.992763846246822, 3.9190451360125418]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[1.0449315395360363, 232.73476552202908, 3.4613530686418454, 8.5632338372736054], [1.1338277955155389, 1.5862359177427137, 87.342630892960045, 1.4527102470094735], [463.69891996700562, 1.0849164130868381, 79.877182928800096, 15.277507317105638]], [[13.110296894886334, 42.50631923668508, 297.07297198712894, 20.175819644440814], [1178.0280050224087, 142.30870825444148, 1.9115777514265457, 39.576707862778669], [1.9713109130019508, 33.796257439395248, 3.9646020352920228, 6.3212529981515546]]], [[[55.547541625050549, 35.531012275673511, 5.2012970763494906, 5.6008946765270213], [24.370200105769293, 20.505723490603518, 102.58297913470625, 312.54514226965989], [404.72703839050195, 13.776488420718763, 4.7882209713985695, 738.07910600334787]], [[641.61091335074047, 120.53810161502068, 5.1014002738285384, 1.8925552653585238], [7.1698258302939877, 249.3280542674045, 125.16362005098507, 268.11678263761763], [675.00679045118477, 14.275891978789266, 5.1547400800063086, 313.8882323555419]]], [[[677.24595929875068, 17.868971831176978, 2.0015531207861033, 10.881893988591759], [51.811647484349322, 301.27731326734221, 18.735498381603751, 6.9781427957436044], [27.083334569884414, 3.9992923458243919, 915.09510146677758, 29.70104593186603]], [[75.468594037642504, 1.7358776514189538, 13.413567600814194, 175.34742510423766], [44.103184419083505, 78.655510042711811, 667.39153255404153, 11.376282787206796], [359.08105337261213, 33.863706733506078, 308.31057868705631, 277.35374796260459]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_float_rank0(self):
      arg0=numpy.array(1.32499305251)
      arg1=1.96628444854
      res=power(arg0,arg1)
      ref=numpy.array(1.73902851501)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_float_rank0(self):
      arg0=numpy.array([2.3295916197829931, 4.4525744985320586])
      arg1=4.43777643486
      res=power(arg0,arg1)
      ref=numpy.array([42.648682627399502, 755.77317784220827])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_float_rank0(self):
      arg0=numpy.array([[4.5170938283855699, 1.6531169690030318, 4.1463979398672981, 1.7678501270672962, 4.6111172578095836], [4.7685467360434748, 2.5429716007750067, 0.51997690402818142, 0.26149276284130241, 2.9400236394350543], [2.0801442044690415, 0.73744031298228607, 4.3815680406520281, 0.60194705977793261, 1.8047285219280025], [4.5045797603997659, 3.50220549855019, 1.7474482810839982, 4.2625187693567268, 0.55998394775411375]])
      arg1=2.10023433617
      res=power(arg0,arg1)
      ref=numpy.array([[23.733270648898962, 2.874012697730878, 19.826853643701753, 3.3089744818574665, 24.782693330248819], [26.59314255600837, 7.1008797382769604, 0.25322113899627607, 0.059776268985928756, 9.6304477333920548], [4.6566197031727707, 0.52746717896418982, 22.262423154146216, 0.34436639739302549, 3.4556125192634877], [23.595390284796373, 13.907368832537356, 3.2292814579107065, 21.011003385286479, 0.29587597496020612]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_float_rank0(self):
      arg0=numpy.array([[[4.7157733600969038, 2.8514790448739493], [4.9558536684677845, 3.2672730650895674]], [[3.0254091246099462, 1.0216532069123858], [1.3472346200549545, 1.5473441883692787]], [[2.1935097636269756, 3.7437973081376259], [3.2898268832527471, 2.0503416302984769]], [[2.829714061424832, 3.2879954331873273], [2.4111746024887477, 0.65967220735937016]], [[0.89895644215625148, 1.8712268669274044], [3.3070493852829523, 3.9524311274321144]], [[2.4938433966832032, 1.1714373129812179], [1.2472826729097646, 4.6631177995143176]]])
      arg1=1.68260128928
      res=power(arg0,arg1)
      ref=numpy.array([[[13.593178489557847, 5.8304446780756978], [14.777713460918804, 7.3310987869115927]], [[6.4412175473346958, 1.0367023557689299], [1.651206361465418, 2.0844870896821082]], [[3.7497462841058939, 9.2184108859060814], [7.4164493073090965, 3.3471811430648946]], [[5.75575900904468, 7.4095035955395172], [4.3968174725372107, 0.49659389353637962]], [[0.8359120604204705, 2.8699894892002615], [7.481894130412714, 10.099146195292427]], [[4.6534236280682757, 1.3050488476196478], [1.4503427433136618, 13.338768965703595]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[1.8880799117875959, 4.9566108062522511, 3.2495081774299304, 0.14406965829827617], [0.23640187163284623, 2.4146795176906939, 0.30268367285856423, 0.24439531659311692], [0.18829981211761554, 1.4517329755655604, 1.5929593694581154, 0.39404680719099677]], [[3.1803171816568221, 1.6760797158774599, 4.91443529529377, 1.8170774741413507], [2.6631109870308065, 2.1749641884102018, 4.9030117972204978, 4.9538412167209795], [2.1807646338371129, 0.98466560833415528, 3.8020850748087063, 4.531066020216052]]], [[[1.0696669589099996, 4.22433181131177, 2.1828499532415231, 4.0260127003593631], [1.979645993937029, 4.7522773403158736, 3.3834078310473892, 4.8629600946932587], [4.5416082533710975, 4.0243064741431178, 3.0359295880298602, 3.5055825654434263]], [[1.0897610585592166, 2.8666067682419691, 3.9885338471022256, 0.78806048185066946], [1.0407041289264276, 3.9807066103897966, 3.1885993204879144, 2.8577177127451292], [3.1074584702429515, 2.3271412638435724, 0.12389820216168429, 0.022609522911882958]]], [[[0.051742311143962133, 0.30587654064363601, 1.6827153738746943, 2.6245753867011441], [0.012487400726081826, 0.42208643551122255, 1.0783640177769089, 1.9570769018682297], [0.50577028854312855, 4.3325250782220994, 1.5097939829605014, 0.35302546951139069]], [[0.3662293262767185, 3.1466130353923587, 3.5128405565001235, 4.7597992580495934], [2.1109737479317277, 2.6079653416788093, 2.3007367661622813, 3.3838224797836274], [4.4408575108090522, 1.5818217370423342, 2.2348924363905533, 1.4829526916953519]]]])
      arg1=3.96526007223
      res=power(arg0,arg1)
      ref=numpy.array([[[[12.430613364497557, 570.93755006981416, 107.02618109131431, 0.00046080929869353655], [0.0032836961558373803, 32.971407106090005, 0.0087495634637054266, 0.0037465292347736829], [0.0013322668118424469, 4.384529872947267, 6.3356943572661093, 0.024902439376465958]], [[98.271131235870172, 7.7515169363406002, 551.91569190846485, 10.677833966481126], [48.615983170043044, 21.781379415529468, 546.84610136573485, 569.67359665527454], [22.012630148224634, 0.94056372581756342, 199.49747572645003, 399.94954441619933]]], [[[1.3061054165608783, 302.89583820057277, 22.096214132365525, 250.31521228871631], [14.998461479634443, 483.16068991899016, 125.61099875583656, 529.34637359778094], [403.65214562348592, 249.89482701284936, 81.735740312013505, 144.58254704613688]], [[1.4061389976220291, 65.100384287005298, 241.20200716237309, 0.38889442136286406], [1.1714052389782457, 239.33053246675627, 99.289830883945854, 64.3035912498475], [89.642757441810446, 28.480498860178844, 0.00025337681601635598, 2.980831254177893e-07]]], [[[7.9444565838933983e-06, 0.0091213005939157401, 7.8739210582247594, 45.885780742423158], [2.8315003398666314e-08, 0.03270537331258308, 1.3487246201903436, 14.331812838121699], [0.067003562323811189, 334.84514137610347, 5.1221839771572268, 0.016103984456907004]], [[0.018628061253308036, 94.205944538282537, 145.7731766336826, 486.20023854403775], [19.349013145005518, 44.745046868164927, 27.220525145666791, 125.67205136837454], [369.29582968106962, 6.161854489675461, 24.260142802744863, 4.7705036955765827]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_array_rank0(self):
      arg0=numpy.array(2.6150248651)
      arg1=numpy.array(3.32321317072)
      res=power(arg0,arg1)
      ref=numpy.array(24.3983890724)
      if not isinstance(res,float):
         self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
         self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_array_rank0(self):
      arg0=numpy.array([4.758774893678118, 4.9217138552458337])
      arg1=numpy.array(3.96831766993)
      res=power(arg0,arg1)
      ref=numpy.array([488.10810748054388, 557.87587780350123])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_array_rank1(self):
      arg0=numpy.array(2.61519145127)
      arg1=numpy.array([0.67798947584442082, 2.0299674233981282])
      res=power(arg0,arg1)
      ref=numpy.array([1.9189469600247715, 7.0391217941546147])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_array_rank0(self):
      arg0=numpy.array([[0.16627370723842755, 0.67410750698295652, 0.24915443960760317, 0.22833019542517125, 0.97480738714236026], [1.3755228634133763, 2.8085691837841473, 1.4832512862120673, 1.9290257029100939, 3.7203057558117596], [4.1449611739152719, 1.7919109199405847, 1.7977333446548822, 3.1992378278408662, 2.8333558588013354], [2.5122915313905052, 2.3129974478614512, 1.4112933485326833, 4.0026312625414198, 1.4950952804184983]])
      arg1=numpy.array(1.09175595024)
      res=power(arg0,arg1)
      ref=numpy.array([[0.14103584571128178, 0.65015071605282426, 0.21932646341943451, 0.19939199509043212, 0.97252784937651759], [1.4163580630944899, 3.087708585041204, 1.5378879858381378, 2.0488939261130303, 4.1969386312666179], [4.722605233653101, 1.8904260992501489, 1.8971332398405851, 3.559488139670925, 3.1174711500878978], [2.733876177002307, 2.4979882478650723, 1.4566176623987741, 4.5458423934925669, 1.5512999508849699]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_array_rank2(self):
      arg0=numpy.array(3.76936482359)
      arg1=numpy.array([[2.3339004954476783, 3.2342100711631718, 4.5388835170669735, 0.43531083904299167, 1.1706553940518678], [0.27814215709166135, 0.20395534876562818, 0.3076715156579144, 4.4302526008936853, 3.7840710740547023], [1.3271407238169035, 4.1908729238075937, 3.4530453966884163, 2.8706231854735358, 2.1806548789346771], [2.5724171432316729, 1.2277028110277661, 4.2603537213348757, 4.0359499370517238, 3.3394036374287546]])
      res=power(arg0,arg1)
      ref=numpy.array([[22.12853053806915, 73.075589896904461, 412.68065702383473, 1.7817869838541818, 4.7272812942151612], [1.4463868769183594, 1.3107895677224324, 1.5041851690537897, 357.28400472572071, 151.57924011654424], [5.8182042155032505, 260.05588394155751, 97.69676046491341, 45.107502243140218, 18.05685055705327], [30.367016690464329, 5.0990122613313433, 285.17158975113534, 211.73346162681321, 84.021810240457341]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_array_rank0(self):
      arg0=numpy.array([[[2.6431462238279542, 2.9698724788072295], [1.8430424486040931, 2.3780270374457579]], [[3.0710451670020595, 1.9783123422713726], [0.59676885758208476, 1.5704597059552459]], [[2.6340745273225679, 2.1849214921077449], [4.897332749639796, 1.5120070895272133]], [[2.2729108558803359, 4.4387474513528486], [4.2873059858891409, 1.4209437448522793]], [[1.1829806807375773, 1.3708632357383665], [3.786827308074467, 3.2015704619556486]], [[4.2914997007415359, 2.2825933986460396], [0.35519103992226797, 3.5281725739330438]]])
      arg1=numpy.array(4.78828804486)
      res=power(arg0,arg1)
      ref=numpy.array([[[105.01173804517548, 183.48677395368517], [18.683562280550067, 63.304465867046787]], [[215.41041835238528, 26.226789959582618], [0.084429705592954207, 8.6822597216728248]], [[103.29714158912029, 42.200399143814742], [2012.4512432659715, 7.24028139749917]], [[50.982280362817583, 1256.8093626870129], [1064.342463160531, 5.3775204698637493]], [[2.2358260900731883, 4.5286353538550141], [587.42126571201413, 262.92017935860673]], [[1069.336833230255, 52.0306407381361], [0.0070385399022464346, 418.62605547549003]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_array_rank3(self):
      arg0=numpy.array(1.45904471864)
      arg1=numpy.array([[[4.6305772409569448, 1.6598669865992302], [3.3039540326093944, 3.1381477737691159]], [[0.73358325857962214, 3.442632268997516], [2.9515146164184682, 0.89941434456366176]], [[2.6618654673429458, 0.86313176412384318], [1.627821256294822, 3.8047898075550051]], [[1.0516541237411043, 2.2844976260460839], [0.1405159666150988, 1.1235902620673577]], [[3.3587040482880672, 0.2932962654720026], [4.9931887959217889, 0.25776319283180421]], [[2.5065136007743987, 3.1460954328229436], [4.6896984439574174, 4.9752407419191638]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[5.7508539534090461, 1.8721129933531742], [3.4839757397037507, 3.2724387070636891]], [[1.3193437994596007, 3.6713675640941275], [3.0496561804544253, 1.4046420158183306]], [[2.7335596066388215, 1.3855200748115735], [1.8495852752751802, 4.2096554181268253]], [[1.4877961235428092, 2.3703601312956515], [1.0545186338326824, 1.5287831400790113]], [[3.5567871862802471, 1.1171737137059439], [6.5951626375644272, 1.1022772572703159]], [[2.5777463740414515, 3.2822789101446674], [5.8807439672983142, 6.550595727760518]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[3.7231898249342699, 3.1890751557213184, 2.7568496443610275, 0.2194993917889374], [1.9035402813678688, 4.2836496150973842, 0.7725006112302677, 2.1267700005418786], [1.2101456875072989, 0.4100461979107124, 0.27430301248698374, 4.2957181791431349]], [[3.6396993817959409, 4.259425485744992, 0.86202576651833596, 0.18836216584051224], [1.4327317457052582, 0.50043841707757297, 0.57282571820771466, 4.6036424000394156], [0.20672266696620545, 2.3885502227833042, 2.2861234284160972, 4.2381604387829954]]], [[[3.0089568291571762, 1.9132673887868807, 1.0384092782714129, 4.6532841626494577], [3.9347865366946544, 1.3304256178628955, 0.17126473297284345, 0.65897971331699956], [2.3918126865362273, 2.5626833202224839, 0.10601285194872299, 3.1452391835754514]], [[2.1864637136691831, 3.7171642967339595, 2.8008252572493264, 2.3070583564961735], [3.8334301785358025, 1.1162764501632998, 0.93668926093901972, 2.6193377073215234], [1.56925683616486, 2.4246515161773488, 2.6866442943605779, 3.9108666424883904]]], [[[0.99355442783134229, 2.5647537624929448, 4.1734815373486072, 4.1482865162597555], [4.2891661511453272, 3.6765873931874049, 0.4695434295702664, 1.4298899909741414], [0.54899728586031327, 2.8571246405656163, 4.1257352257253821, 2.9018182751694148]], [[0.33134154413049427, 0.47227510207405299, 3.0825806107561577, 1.1924043281866055], [1.1012063599996282, 1.1697050308369377, 3.5091786452889786, 3.2676038461861103], [3.1155132650217507, 2.998118739814684, 1.1849882463817596, 3.4150332514643913]]]])
      arg1=numpy.array(4.81590559496)
      res=power(arg0,arg1)
      ref=numpy.array([[[[561.6601415940745, 266.44152322403022, 132.12585779947466, 0.00067360460404236871], [22.199569070553729, 1103.4592234939923, 0.28849039378999025, 37.867890551579571], [2.5057534567311874, 0.01365964032256446, 0.0019704775609283788, 1118.511781459662]], [[503.54530659534259, 1073.730087959078, 0.48918327307035758, 0.00032243188715948181], [5.6503498405172721, 0.035653409992834402, 0.068337524696083438, 1561.1139891285122], [0.00050463075680355065, 66.230718082036319, 53.627818508527376, 1048.1588777831039]]], [[[201.37524551786629, 22.75123736205321, 1.1990283157855257, 1643.8684892880681], [732.96666503852805, 3.9548227020307052, 0.0002038999658925216, 0.13418527740493214], [66.66751594581946, 92.947188754859468, 2.0240378695409154e-05, 249.2603094202392]], [[43.267989866884861, 557.29608401491612, 142.58939229737445, 56.034544623295105], [646.40183895674591, 1.6984951023111265, 0.72980495472371887, 103.26918757904966], [8.758787105752381, 71.192599730829158, 116.69054337346617, 711.75559317720047]]], [[[0.96933817090099628, 93.309391595430967, 973.33464388481093, 945.36070263123611], [1110.3196932082442, 528.60251753626278, 0.026231401383390158, 5.5965809039940435], [0.055692236213457848, 156.93214865027284, 920.86594045958179, 169.11263642805898]], [[0.00489433828466711, 0.026974544582351684, 226.23812801004911, 2.3337190535553773], [1.5908739098157132, 2.12740077128363, 422.33600856777394, 299.55752776948975], [238.11786753741271, 197.90599196973699, 2.2646436332587365, 370.49270389505483]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_array_rank4(self):
      arg0=numpy.array(2.53795666536)
      arg1=numpy.array([[[[1.1474049025553823, 2.2305274617734914, 2.3510158829827184, 0.67551062473122236], [4.8957015995379063, 4.0549804621966681, 4.7097422151853943, 3.2868012959927198], [4.3733508504298566, 3.692415063341326, 3.4337503960759577, 2.3692739038525525]], [[4.4953112742753305, 0.040715550088528528, 2.0387381130750204, 1.6647169375496436], [4.3743717105966713, 4.6820346751535729, 1.9700672164120869, 4.5464393749117047], [0.82343925213138369, 3.5068015877877605, 4.6143784582512497, 1.782006609711662]]], [[[0.88409042460555021, 1.2945262583967765, 1.8449677124274912, 4.8793228803544357], [0.96110853558332532, 2.1628972056053799, 3.2105516193362682, 4.1523639623244328], [3.8496974919447768, 4.7658498341695292, 3.8456820733037298, 2.4956125505552782]], [[3.4139302718786961, 2.9852081280914851, 2.7525087907575929, 1.2521176499055748], [1.5014892656442393, 4.6085246772419133, 0.087030580511979649, 0.83158894614339407], [4.3095784549320824, 4.043624285500603, 0.75502444320188766, 4.2096912902416648]]], [[[1.6820660011448327, 0.52893059099953654, 4.2022780979514831, 4.6231906279075412], [3.9735196732913933, 4.465981945181575, 4.1065154391868877, 3.175633863748712], [0.081523442180662847, 1.7888932958670585, 4.7847574610278887, 0.14542248064563681]], [[3.4028521636682756, 2.609694324458852, 3.3406835996082807, 3.0682874213405325], [3.0750554585426824, 2.6702901542565121, 0.90217409615294908, 3.2427847014373032], [1.6371812158732475, 2.8660380959978484, 3.3785709258261587, 2.3629270421320414]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[2.9114353905155754, 7.9838638329400311, 8.93200143719986, 1.8760023422028833], [95.550718414718091, 43.669230378836836, 80.355632421424957, 21.353014780965928], [58.742351895703493, 31.154699155549405, 24.484869307221448, 9.0851868847009314]], [[65.808561141406599, 1.0386489748737844, 6.6778607637640244, 4.7135889029868538], [58.798229955652694, 78.308528146463757, 6.2641343938993614, 69.018084893026895], [2.1531196731135203, 26.208721940770406, 73.526380993613841, 5.2576724003981994]]], [[[2.2782459675524356, 3.3389923275987727, 5.5751979732349897, 94.104203583103143], [2.4476719980424861, 7.4964862906028351, 19.88920440195718, 47.815161327117316], [36.069640977158905, 84.666375618203972, 35.934999747337905, 10.219645989935106]], [[24.037033188169492, 16.123878938424799, 12.982141033723998, 3.2096805384142506], [4.048822036506766, 73.126607890094746, 1.0844324256181743, 2.1695246634564271], [55.354942510554743, 43.209788838086489, 2.0202050839982673, 50.437508345086663]]], [[[4.7903707279751249, 1.6366057332330182, 50.090469774819269, 74.132316921324389], [40.478642157036909, 64.035255260642458, 45.816364170468766, 19.252792515318013], [1.0788844769474797, 5.2915033812331425, 86.170538179006982, 1.1450411543510066]], [[23.790301354875478, 11.365279678089241, 22.451933193361114, 17.421028465640678], [17.531188301923404, 12.025141162760573, 2.3169420399192528, 20.495342655150342], [4.5942427462033724, 14.430026655502814, 23.258332035945148, 9.0316408661067697]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_array_rank1(self):
      arg0=numpy.array([2.1478434307786176, 1.0919322874389026])
      arg1=numpy.array([3.096142960126715, 2.8545299361228902])
      res=power(arg0,arg1)
      ref=numpy.array([10.664182634329707, 1.285377790207427])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_array_rank1(self):
      arg0=numpy.array([[1.2818013777128099, 3.7915228582844933], [3.4399290960652156, 1.4318818471162238]])
      arg1=numpy.array([4.1920085212502896, 1.2153325859916642])
      res=power(arg0,arg1)
      ref=numpy.array([[2.8312970461702918, 266.92661123069331], [4.4883434339977972, 1.5469599858363507]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_array_rank2(self):
      arg0=numpy.array([1.4111024201319529, 3.0895974972159226])
      arg1=numpy.array([[4.1678530715777864, 2.553786062303073], [0.52787296758623758, 0.15783833099429992]])
      res=power(arg0,arg1)
      ref=numpy.array([[4.2008580492602556, 2.4095753213363724], [1.8138692242104899, 1.1948827706973038]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_array_rank1(self):
      arg0=numpy.array([[[2.3655796628498509, 0.51226305551897366, 1.8606677324064922, 3.44366857279261, 0.61261523162441933], [4.0471180410273346, 4.5400586608911286, 1.1260882024288561, 3.2167696149784319, 0.25799805025600425], [3.4890602286523604, 0.37078319878943572, 0.25680791303796408, 3.1716497090702811, 2.2515602855208581], [2.7240195069147779, 2.9198131189171854, 1.9786781433717866, 3.3655488391923627, 3.4073823333541422]], [[4.143988722091585, 0.12146550030902395, 3.3337969432577115, 1.3118192326492275, 0.057929449174447416], [1.0188650267710058, 0.52318072989635755, 3.94132252076529, 2.2301142778976986, 4.242288318547816], [2.079536873860222, 3.7415475494294523, 0.93379135450326212, 2.9297460428348967, 3.6886182803187784], [1.4313899331013547, 0.63988403955011541, 1.9467759577649377, 2.6456620045867369, 1.2421168603064936]]])
      arg1=numpy.array([2.2217867788334247, 4.1808754848695724])
      res=power(arg0,arg1)
      ref=numpy.array([[[6.7734443431178537, 0.22623274621871359, 3.9732567757292143, 15.600802151628125, 0.3366484713429756], [22.333082494644533, 28.830387142547117, 1.3019157936955079, 13.40846163354434, 0.049287540822928022], [16.06136661360943, 0.11032568098169791, 0.048783814117597128, 12.994179600447906, 6.0693664141833192], [9.2671095679748241, 10.812340398726425, 4.5549373128977164, 14.825379666361076, 15.237918408914899]], [[381.37317418711416, 0.00014866603392113843, 153.58310639117011, 3.1104017750665434, 6.727232304485089e-06], [1.0812714204039493, 0.066637386315990588, 309.24659793954896, 28.59646228662303, 420.64740259362452], [21.349065510826428, 248.80322270295068, 0.75096262297313154, 89.486599191279126, 234.41565190022862], [4.4792430687854177, 0.15464418206926142, 16.202948953376737, 58.420268471711296, 2.4756058912622398]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_array_rank3(self):
      arg0=numpy.array([1.6474605572393768, 2.1721718772090122])
      arg1=numpy.array([[[1.4444563684924194, 2.4590685383131992, 3.0029768485748081, 0.7830274913011972, 4.0240523140479052], [0.57912532230413816, 3.10009437443109, 2.7378845800576004, 3.5201883328402421, 0.074967465144234524], [1.5617381287847374, 2.5715828045511784, 4.7604356520530766, 4.5668656906264156, 2.2473119983603822], [0.61678024480248239, 1.2779888631527341, 2.1567556688654825, 4.442861310895708, 3.716783491235288]], [[2.0294972662234092, 4.1615404303660499, 4.5136340447850687, 0.52306921831580699, 4.1474077208145923], [2.6161790973808521, 0.53964908916058041, 3.0735696833247488, 4.2567574498869121, 4.9952383169820278], [2.8730794455890463, 1.364180841927749, 3.2339946130035595, 3.6764001390645058, 2.9421534906576774], [4.1675749659310624, 4.6247836426320204, 3.7779592392409542, 0.77186232798320109, 2.1303269631742738]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[2.056742132310029, 3.4132099920965748, 4.4780661286813972, 1.4783325034676429, 7.4554696381769912], [1.3352514911519515, 4.7005323019045857, 3.9229657421485196, 5.7973527139795591, 1.0381355728733364], [2.1807621085087181, 3.6104205451385383, 10.767973504838388, 9.7760879971205821, 3.0707963002994969], [1.3605898432098147, 1.8927235604523021, 2.935060351727711, 9.1892293141438053, 6.3952055556938605]], [[4.8275394427611946, 25.234736748986137, 33.160194585777042, 1.5004411934667281, 24.959595959638225], [7.6098400611992654, 1.519863694447219, 10.850949906765203, 27.169199254459556, 48.179994925097866], [9.2880332830653991, 2.8812742316331001, 12.288927576817514, 17.320396697194962, 9.7992854166544081], [25.353141391757891, 36.146206363700955, 18.740125936940977, 1.8198514171051545, 5.2202916798365928]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_array_rank1(self):
      arg0=numpy.array([[[[2.0317318334790482, 4.7278350727191718], [3.5974549949033268, 0.39799928860592815]], [[2.3722185279944625, 0.40071922236407559], [0.29433176607204692, 1.5968636347792051]], [[3.5909109950270643, 4.0587455560070271], [0.6584246883539927, 1.9026346227212241]], [[2.6012911182034411, 2.6273344471404543], [0.338034160760254, 0.10949038250943099]], [[0.060177963628417513, 1.3328967886352434], [2.053177716521875, 3.8259583363593368]], [[2.2689318209188913, 4.9700463636068548], [2.2360561878086367, 2.7712153846290857]]], [[[2.2217797962372234, 1.318607314231194], [2.8363194542491512, 2.2813854279651511]], [[1.3311502534077182, 3.0146877245462305], [4.8578372725765586, 3.243806822931115]], [[2.2805270520177792, 3.0847390505765917], [4.2585332312310769, 3.0492288622829977]], [[0.96782261599611596, 0.82243189401190897], [1.5046939745716685, 4.1546936486408894]], [[4.8083228310878452, 0.41342895818026559], [2.6445511280750229, 4.0483091949101917]], [[2.8657083888469046, 3.7452826560826185], [1.4762239953543037, 2.9234394672457857]]]])
      arg1=numpy.array([4.4427347738375174, 3.5743424937637589])
      res=power(arg0,arg1)
      ref=numpy.array([[[[23.322155969338539, 993.90698120751676], [295.21604805001095, 0.016687188121287298]], [[46.42081947096905, 0.017199833335267582], [0.0043670108907075365, 7.9995408513184181]], [[292.83768443848822, 504.57243600817145], [0.15619630994694086, 17.422157262656302]], [[69.91565358254482, 73.07948491804396], [0.0080778695974932386, 5.3976190591169603e-05]], [[3.7788920174198121e-06, 3.5845764416147055], [24.435895716554313, 388.11489669681691]], [[38.090819059755077, 1240.9227380236885], [35.699242337661417, 92.611869763337154]]], [[[17.347151208587217, 2.6874096098667919], [41.524264594178945, 19.068858810991902]], [[2.7799061092844783, 51.638811013212752], [284.17263304074879, 67.094287021317044]], [[19.043226403103272, 56.057564839258809], [177.49695322836132, 53.784974907636638]], [[0.88967052888548337, 0.49720801632565076], [4.3078500420244046, 162.50631915905885]], [[273.95470980881396, 0.042548613382642597], [32.331852277983607, 148.1167252530818]], [[43.082782980212635, 112.15803019130747], [4.0235380425775542, 46.266335182248341]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_array_rank4(self):
      arg0=numpy.array([4.4504550934987748, 1.1565597599940085])
      arg1=numpy.array([[[[0.10067621193521845, 1.0638695847074033], [2.0235904239179856, 0.49524404503427177]], [[1.2096351959413296, 4.5165536261800003], [0.11633837351450922, 4.4690815920558826]], [[2.2877903140673044, 3.9440003434323323], [0.5488539007099078, 1.3137787790289315]], [[3.882024079619121, 2.155271401350928], [4.915186701851149, 0.7243593079562366]], [[4.3867069797916596, 4.2608144955607221], [2.8172365977969669, 3.6617492499556281]], [[0.50282727540213112, 0.40544975480641854], [1.7691872007945859, 4.9349262925247102]]], [[[4.7966989710534547, 0.86289644281953359], [0.75812688549085328, 4.1115910817093999]], [[3.9179493484684071, 0.34780725972311527], [4.5076762365872725, 4.7416970036351431]], [[2.9918872512680741, 0.66944899400974112], [2.9055271375464682, 3.8197953188938798]], [[1.4281542260731608, 3.2785510485290512], [3.685044257773229, 0.93906782759165586]], [[1.3533666894973877, 3.0600535454983158], [1.9911229837366946, 0.55367374815910042]], [[0.66356974903394683, 4.9312617553651998], [4.0249207705824688, 2.0710835422449323]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[1.1621947282444631, 4.8957332245492955], [20.516580645889068, 2.0946835850279748]], [[6.0860121009189942, 848.30758258035542], [1.1896913829985503, 790.26413407066832]], [[30.437807008294303, 360.8338464495227], [2.2692340780451601, 7.1098423573544061]], [[328.94370846936664, 24.973965641305195], [1538.2566401135186, 2.949020416752032]], [[698.81176670414993, 579.06986604357712], [67.097860333640853, 236.75241101136726]], [[2.1185339569576724, 1.831873222641017], [14.032944465158257, 1584.2657609541182]]], [[[2.0090847364359057, 1.1337244420123773], [1.1165789036683884, 1.8185334996041067]], [[1.7680287776523596, 1.0518899749296993], [1.9263768193252271, 1.9930761198424092]], [[1.5452251458744026, 1.102269539657059], [1.5259368625264627, 1.7429668540578953]], [[1.2308744514784831, 1.6110156821279675], [1.7091381281409126, 1.1463549475248962]], [[1.2175577585552209, 1.560621913253653], [1.3359044967628741, 1.0838632882183694]], [[1.101327352811077, 2.0487941559002865], [1.7957526199734644, 1.3515321046773623]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_array_rank2(self):
      arg0=numpy.array([[0.023743923465572275, 0.51983609216690441, 0.35445778752120738, 2.9899867846171784, 3.8041905832221032], [4.6523834897630278, 1.7290073132581489, 2.6188828906778259, 2.8811869359465199, 3.4719881042644296], [0.74502276343427087, 1.487305541519206, 3.8877407334040224, 0.40939228795203991, 1.9878297870064665], [2.7507874110864061, 1.7832915197913253, 4.2892139464592205, 2.938845638679374, 2.9737295180281187]])
      arg1=numpy.array([[2.7044133744078089, 3.7935278989618313, 0.082523856775443313, 3.219433617894476, 3.3304017655477569], [0.16675383365812188, 2.2159328155824505, 0.59437766153530291, 2.4995161422805099, 0.24473166839410346], [2.4004364512640599, 2.9780796219916748, 1.2725667159143048, 1.9138967905115361, 3.72956340916978], [2.2289382434203717, 1.3681038185155354, 0.23275684862690829, 1.6737190691112938, 3.6657939843012892]])
      res=power(arg0,arg1)
      ref=numpy.array([[4.0441048883140199e-05, 0.083585582288554214, 0.9179696593052632, 33.992697927432054, 85.606447651099685], [1.2922229510605248, 3.3646647598315971, 1.7722249928942131, 14.083358189734209, 1.3561152873896061], [0.49334543046313983, 3.2615310000873134, 5.628972797953363, 0.18099864966908955, 12.966540602578302], [9.539433313165457, 2.206471125884474, 1.4034283420145923, 6.0757010556371442, 54.328242843777808]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_array_rank2(self):
      arg0=numpy.array([[[2.6135044941964076, 4.4211972961807957], [4.7706802723083968, 2.7553584348818849], [2.6823493031387935, 3.256569338779455], [1.7471378722329267, 0.086563866906200854], [0.99974662703897266, 4.3641616613676231]], [[4.0753074565682237, 1.1182896301149028], [1.3773732036819912, 4.2103426859683069], [1.7825245898384479, 3.5215475279606361], [0.3788548959873354, 3.7011250573926149], [3.9745866085148789, 3.6645647023166488]], [[2.6999367875731259, 3.360320131447347], [2.4113631673671478, 1.4372431484961803], [1.8060657588020277, 1.0137627641288824], [0.84462967732303795, 2.778121883002032], [4.910218443929768, 2.3637941894316681]], [[2.7579482104029811, 0.41014901347096633], [0.73992887365150417, 0.75426062208342171], [0.36746335621612664, 3.5062459002816193], [3.2605854914462213, 4.7368993905339796], [2.0131670335740912, 2.9316323601929835]]])
      arg1=numpy.array([[3.9864367313232103, 3.3771410862224558, 0.36007870193678893, 0.97614029798589963, 2.584063959500345], [0.010565754556016672, 4.2573952210447086, 3.3972910419659947, 0.83987908936953315, 4.1724183207311141], [3.2597220666884006, 3.2699200339892225, 2.1600247357455951, 4.53862711445315, 3.6698520949959694], [1.3779259989710531, 4.6670785147753513, 4.6864051014921033, 1.947694352078801, 4.4763161840333101]])
      res=power(arg0,arg1)
      ref=numpy.array([[[46.050472851744111, 374.45872304962757], [195.73182703621518, 30.657981364465829], [1.4265902272719369, 1.5297979474388448], [1.7240320042941442, 0.091768044959885692], [0.9993453994478857, 45.034430163238227]], [[1.0149550407071861, 1.0011819535651387], [3.9083886524469786, 454.95255394491846], [7.1259169426074607, 72.0133485821992], [0.44255597289468013, 3.001453646095734], [316.58969467355814, 225.59910397691408]], [[25.473747313249781, 51.982197221681403], [17.781492421988663, 3.2742460374466709], [3.5855091716309788, 1.0299653898934882], [0.46469269825745063, 103.28129180458687], [343.74609044531763, 23.501316765123615]], [[4.0466444815439422, 0.29286148873424533], [0.24518807376540028, 0.26815316086480556], [0.0091710032321757479, 357.56258307364089], [9.9940823807964456, 20.685030304909983], [22.922506875253394, 123.29056576623749]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_array_rank3(self):
      arg0=numpy.array([[1.2396826965042214, 4.514800287345051, 0.48181912829833867, 3.1107191272411847, 4.5201021218926405], [2.7831273924483497, 0.90107344627609243, 4.8887444001954297, 3.6772156262111606, 4.898286766913583], [1.7043241830443319, 0.49210756500671116, 2.8101568720909187, 1.3375248089620706, 0.98655392585491475], [4.0559082015976546, 4.1285999295165121, 3.5755816827045854, 2.2857705157583492, 0.55045474522625959]])
      arg1=numpy.array([[[3.3373348210485467, 4.1126951983629603], [1.0135924618396415, 1.3234810190493531], [0.48794052796729437, 2.6321356173112407], [2.8532210479457194, 2.8861986210997848], [4.0351564854314539, 1.3691979225302708]], [[0.061073737895324463, 3.5482618054651227], [3.0250662549560308, 2.8376137148381511], [0.49002590875095281, 0.12274568544480204], [4.6619619623414579, 0.71280037843710931], [0.70815298987278996, 2.1823936050248114]], [[4.5045232111930034, 4.2841088360945063], [3.6728511233375274, 2.4307149969752868], [4.0469338116052782, 4.5560577889285483], [2.7294903867216123, 1.3900656189976377], [0.48442962103731324, 4.4666674963205262]], [[3.7761185348655872, 3.7408357458436416], [4.2563202448950586, 2.4184480535048962], [4.6804119048432593, 3.098534755239859], [3.7762573338483363, 2.9247045865772394], [1.1684073968474744, 1.8967093840075258]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[2.0483704978936554, 2.4196792889602405], [4.6082570210385745, 7.3519354847190597], [0.70027120700448453, 0.14632137730956038], [25.482428078391095, 26.454171225629469], [440.17532873498192, 7.8891028604023958]], [[1.0645089009536692, 37.785029848890396], [0.72970375718677616, 0.74409245247033262], [2.1763289976111606, 1.2150551733249211], [432.94080372484399, 2.5298934249013332], [3.0807529168335113, 32.058859233019163]], [[11.041607068388004, 9.8173630848354847], [0.073957526354869962, 0.17843733335931586], [65.461038826370157, 110.77515915230258], [2.2117685335718598, 1.4981933121585811], [0.99346359102402226, 0.94132522892034864]], [[197.79386149657134, 188.25987433983789], [417.88225343227106, 30.852267271476034], [388.94706606379185, 51.827945490435781], [22.688176322612524, 11.221847673842895], [0.49780267142355272, 0.32227325346865454]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[1.1426869208372863, 1.815346936138472, 2.8378764034692261, 4.7355341349709743, 2.1594285584920692], [1.2775773218851536, 2.2732331898965072, 4.260464208265442, 1.1961842565715533, 0.26535084029487538], [3.6691122851256774, 3.1480080538339417, 4.2932026225144586, 3.951506070969645, 3.2166473624867336], [3.8824249281479135, 4.291820006792836, 4.3002687431072646, 0.56426328143435345, 4.2888012674455398]], [[1.3115275628221608, 2.7210394683318224, 2.9957178765371872, 1.1370837268451257, 3.6692340509217458], [1.2015048841152447, 0.85419908288336333, 2.2856738209205174, 2.2596873860777, 4.8701167477152731], [1.4043497826229223, 1.2223678858977125, 2.2285657177579634, 3.067419089494372, 0.030713547781912959], [3.7606305909520192, 4.7613744256288024, 1.6401543878414715, 4.9816191651580004, 4.2149425518956987]], [[1.9012619935001278, 2.8952281562753011, 2.1301049835859214, 2.987460669491234, 1.1726771558192175], [2.1971150304713927, 3.261511485101046, 3.5734348878966786, 3.1360717609224182, 2.2594826938536308], [4.8233646388367077, 1.0883298856614592, 0.27422629151722622, 1.0710275582950066, 0.37444777604284224], [2.3580487654391975, 1.8027219842451385, 1.7916565187510165, 2.0076178259752182, 4.8435962093069671]], [[0.55732770573726909, 4.2472971478292036, 0.034233864703871873, 3.7689785168307064, 4.3027789481255958], [3.0088394580246476, 2.709761437488293, 4.5664854989064256, 1.0818533674707296, 3.8792589474069712], [2.8140967457362107, 0.13780519683486633, 0.41705344654929305, 0.42695215193583269, 2.0296909018151483], [4.5090000018863536, 4.2901304585710003, 4.1361753729891468, 4.1172862035550848, 4.6341063445841568]], [[1.2849942678977304, 3.9888443449408677, 4.1333898660737818, 3.8026271729170964, 1.039860405239678], [4.7859890321168326, 2.4455112818921272, 2.5276361581980367, 3.7867118261349311, 0.041581613069539627], [1.8953537689388089, 4.7729805460978243, 3.0616498681626547, 3.5036041900422186, 3.2537261690806329], [1.8668769813858264, 3.8045773184375355, 2.4854628582193743, 1.2497794168106062, 1.0900211046430488]]], [[[4.7050447987242761, 3.6589859273263592, 4.2519501604384775, 2.6701393452502749, 0.75299797291459913], [0.3472312404644386, 4.3796157326892891, 0.14299154569450986, 3.2744935690981207, 4.1724523429264124], [2.8512107264981865, 2.0573785131509248, 0.049413727281215875, 2.0155135279828436, 1.8237389184652961], [4.8620137647615707, 2.7352755237103263, 3.9405507801623192, 0.36070041043921547, 0.9324683223282646]], [[4.0581755401768467, 0.860200105419898, 2.5910964864497985, 2.8572829789329957, 0.60015461089663347], [2.2719412122474685, 4.5930964739788323, 3.2820133640536762, 4.2555660944043741, 2.4013939925082952], [1.375178793112185, 0.13714032055558389, 1.5953660699003696, 1.9988966634100775, 0.1659929678181282], [0.77142104257579946, 1.4545647411972085, 3.9047050249856969, 4.5404138075786422, 1.5971212772018406]], [[2.903067027139969, 0.4141071514463941, 1.3072734901543581, 2.3461807406570112, 2.5628868863469583], [4.3339087865157815, 0.2110798828078988, 1.8378613617382551, 3.142679719048644, 4.15997748198909], [4.4923516420732668, 2.6724554296003897, 1.0210644891793335, 1.8124409889906392, 4.9044713303238341], [2.7187916319444594, 4.559175235738171, 3.7135484811829516, 0.13942078369100785, 3.4356619383242393]], [[2.6027001134883743, 3.2793999216734422, 4.1487829935465061, 0.60729949292042262, 0.73251546878520979], [3.7209779075106129, 3.2316795290352145, 3.6862900529057718, 3.1695342788299889, 2.9074803679167802], [1.4258944001558356, 4.0726417459220343, 1.1333699315059294, 3.5852735977654464, 0.54615550089042875], [4.3397373796041903, 4.0086981064148413, 4.9340611901291727, 3.5024796005871197, 2.2799844292897786]], [[0.72205732909893927, 0.35220279085679401, 0.11132899708322014, 2.7055514274636066, 0.20048606227804791], [0.56010241720713883, 0.26762703974251861, 1.8043015933720257, 1.284878614595292, 1.9885063179653732], [4.3323607098348447, 4.5635117506968399, 4.1933012065759714, 3.7275600601940271, 0.1712753292443483], [2.4455875400507194, 2.7545507935522489, 1.8437625033108749, 0.095214212496199116, 2.7663157461501813]]], [[[1.140397116216131, 0.29399015125418959, 1.3738352226238326, 4.3414169236840179, 2.5297132530275377], [0.31448224232723154, 3.4352124682046559, 4.8663870964016187, 0.74864559721006863, 1.9451787194906021], [4.3690555338695525, 3.7958428513209195, 2.7394317114271658, 4.370335792464676, 0.09354465322815779], [0.45022010001109763, 3.9222744231208662, 2.6429778921259439, 4.1136410463369648, 3.9186773085395861]], [[2.3731867438150802, 3.6043647056570665, 1.3047235383505087, 0.3231560657589006, 0.36542157844316048], [0.95724335870004906, 1.9759829964423599, 2.6118260412799601, 3.8335712190717417, 0.66531901355443523], [1.4774925908952741, 2.7323845346780309, 4.472510043905614, 1.5885575771302927, 4.3884359314195596], [2.4290986384985906, 0.45146295415714532, 3.9655977087592165, 3.9528471037262856, 1.2028420658989896]], [[4.7031767107073614, 3.7534300751802649, 3.7888005117560417, 2.7883834121250439, 1.9102171318704095], [4.1519580356261567, 2.0579656778454254, 2.382516619019194, 4.5414283390432031, 0.75642937530040322], [4.3962447764020114, 1.7003889006999306, 1.5307063453990897, 0.81813552637740916, 3.898594983251594], [0.8374177453920908, 2.8088999245214405, 3.5787074775278369, 0.94145614938386679, 1.0300704406382395]], [[4.9284823622311293, 2.4554904754127485, 0.71342816169747481, 2.306705080580445, 4.2916638269586533], [0.136479847252509, 1.2213288038304502, 4.4376498568002489, 0.2531024649193595, 2.8116109858764133], [4.9325324737285818, 1.4291175281981729, 4.1557262771092569, 3.7948005528697446, 4.3037201272547412], [4.2884918851131593, 2.8660710895795711, 1.6106909486205718, 0.59062216923564825, 0.25822631387331629]], [[0.1206295142253106, 4.2940873442544714, 4.0778410890146066, 3.988640766451776, 2.5484312394297097], [0.31659496524672714, 2.0488754947458716, 4.330959476025745, 4.7048124015236397, 4.9017537228413843], [4.3447815990379315, 2.3806360144547849, 0.046439338447475248, 0.97549742354337932, 4.8933996456312165], [3.9824217189045941, 0.14300366722973204, 2.8215530550392631, 0.95187607750227821, 3.4506972492044667]]], [[[4.9941384449929673, 3.1255371170854174, 2.4318285024447031, 2.8393365607492145, 0.33334416020039581], [2.3935623200344271, 4.0054530680669638, 2.815450506749746, 0.40116341636111508, 3.6293957486610768], [1.9642964969412697, 1.4637551022130786, 3.4015120626248603, 1.8188109870564002, 1.0507855698946305], [0.86093991435004547, 4.9870951111236579, 4.0906304037667693, 2.4975868862949557, 4.7327346225686187]], [[1.712109945951519, 1.277167656771965, 3.352527909279627, 4.7014451148267957, 0.75440514411938431], [4.6878491223709604, 3.1572564161546861, 4.1509784525998281, 2.1164946296398219, 2.2084881854691072], [0.10065027478737555, 0.078510761319632083, 4.0845457429589604, 2.3082677818750015, 0.95237763932281161], [1.3945383572624332, 2.1694670104167342, 0.34804115430596616, 1.8653135867884407, 3.2286943099480623]], [[4.4708494449991134, 1.0611952338218353, 4.6748710797085726, 2.019056981403426, 0.39934777598512611], [4.7199949762625604, 3.006261320267273, 3.4267279113095519, 0.71520548118099292, 0.51617064008116542], [4.1047059161127093, 1.1245182607862081, 1.4804031886072144, 2.2668342547218123, 3.2206317614015427], [3.3869532643203808, 4.0647384853273572, 3.2729147965027159, 1.6263687702975151, 3.1532585583794721]], [[2.6080902955591636, 2.0298979122364083, 1.5746728961302026, 3.5857405140897791, 2.4721046290847073], [1.2229900998741099, 3.8158874165741512, 1.6175075793764291, 2.7082511718200042, 1.3534345622491752], [1.1481547274042696, 1.7414205709959931, 3.4972522177167753, 2.7858374490667144, 0.64399626745816629], [3.3140867397909917, 3.6352968336661995, 2.4422951209358041, 3.138605809257887, 3.8660803436274764]], [[0.37871837828989108, 2.3788776949653947, 3.9493177518916607, 2.6009809486224422, 1.475891050359367], [4.8816337804552949, 3.6644326751062382, 0.25723881559224121, 3.1073345603705551, 0.39631392944201876], [3.1954803160269067, 1.1529949424863382, 1.9243720084037945, 2.6765988227307203, 3.2041231697325148], [3.9856291443056628, 1.737279048694981, 1.119052895859892, 1.1762849978836838, 4.9647380912960344]]]])
      arg1=numpy.array([[3.9941894208026612, 2.4129945334808083, 4.0951339876906134, 4.1287828353531975, 3.1710803403233232], [0.54090266805140752, 0.53500135943671334, 3.6157050354794156, 4.8093374425808655, 1.8525989193783643], [2.9734105319860675, 3.8184282471939088, 2.5261993697734297, 3.5784792512648318, 1.1493966659372867], [1.8223107488089394, 1.8067080470185737, 2.3016625483631885, 0.10797052583024033, 3.5556897771323368]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[1.7036188432374673, 10.822655656977542, 64.467637210293105, 498.36966656090283, 21.647744835493349], [2.6602999430388929, 26.576858543769628, 326.71582857498134, 2.045221157973498, 0.004996084853863025], [179.87200841740761, 97.554984099809289, 336.85943629261845, 241.8705974611031, 106.33237181021515], [225.41833479825229, 336.4263363911175, 339.07940579194076, 0.10171176608752799, 335.48217678839563]], [[1.923961050011459, 11.194708652792368, 14.11867994921001, 1.3634112179745139, 23.031243687914106], [1.5573179647788065, 0.68367938433363562, 7.3502046735719055, 7.1501764684854097, 45.60671031964273], [2.2691179529549284, 1.6233710028771535, 6.9148616823222024, 14.947923547435384, 0.00022383888002143392], [24.439980941580817, 43.188131926964253, 3.3000132159922257, 48.167184464835614, 32.182431941295171]], [[13.890383672017308, 77.741497116614667, 22.123107583070183, 88.394690387821939, 1.9199728316504352], [25.114989447619269, 126.62493552183261, 184.06056774667903, 107.83683830312141, 28.165299048141055], [628.65185317857231, 1.4142930221151777, 0.0050001316518757076, 1.3244563107038623, 0.017905235235287208], [33.546990119654176, 11.170240995425958, 10.892113314324769, 17.358789127878637, 639.52051407447391]], [[0.089484113502180732, 392.04932592782575, 8.893788534909843e-07, 239.3879936752393, 413.63010298306352], [94.450800902111766, 61.302519109631163, 528.77532668840013, 1.3838018553207718, 269.65955776312063], [71.651163880277267, 0.00027939238562773764, 0.027030499453972408, 0.029779404823460566, 18.591366600743346], [501.82841560396412, 408.63290914864234, 351.40111747235983, 344.82248921295002, 561.86110975143947]], [[2.2147995358344796, 80.414504928036635, 90.023732355580321, 69.102246543251638, 1.1319551549959701], [143.2995698460004, 17.043211266667612, 18.925188854354779, 68.189274768479322, 4.172776929022583e-05], [7.595881795142744, 142.068095823177, 34.753936675325697, 53.296774809512918, 42.150362765748582], [7.2398516791663594, 69.21468747651501, 17.941892258963172, 2.0279927071649304, 1.3143441505298596]]], [[[2.3109552394711725, 2.017080945882304, 2.1877868907072169, 1.7010353490324395, 0.85774351160963336], [0.56431225114017247, 2.2230767041260613, 0.34922483470893873, 1.8995154956867253, 2.1655656563365171], [1.7624896331589073, 1.4773128880846946, 0.19656160493681826, 1.4609758822995464, 1.3840615450266072], [2.3523433196027881, 1.7233561951821499, 2.0996085448865256, 0.57604900939790793, 0.9628862756721388]], [[2.1157181925128907, 0.92259402787994294, 1.6642335783873528, 1.7536204878326507, 0.7609751628449245], [1.5512183281074643, 2.2606190274264617, 1.8885817398521261, 2.170166384028458, 1.5978957690992184], [1.1858297511510305, 0.3454476063674477, 1.2838981223195398, 1.448515819058865, 0.38260172505612833], [0.87036373908127673, 1.2219752399846568, 2.0725284838022251, 2.2467096536544142, 1.2846536362397127]], [[47.15791700490535, 0.041266018843249756, 2.6347946618281086, 21.83322123606343, 30.050042490089954], [200.80262206910828, 0.0036091011464832231, 9.0297750642761638, 62.818933424805117, 173.16111477706463], [228.6402047577792, 34.960878683414826, 1.0782850930028864, 8.5862987379054569, 314.03474082503112], [37.202764897380753, 241.17842709348102, 114.86542165425675, 0.00080563941213446374, 86.707285015794383]], [[99.521022140733308, 302.43303923290841, 937.1071497109682, 0.090847199897652525, 0.22379912073681987], [555.24226482293943, 281.84642861563742, 530.78680476932402, 256.71792336527648, 169.51520450784923], [5.508814219755461, 857.23616633253755, 1.8259622217545735, 464.39215474207964, 0.054533767832102467], [1163.5279837585069, 794.41345937830897, 2157.0216637180133, 415.03624550041133, 52.652039395717409]], [[0.54700342829766568, 0.14467344331344065, 0.01712961337222774, 6.3211575639878168, 0.050938003295293972], [0.34169651092279218, 0.086984661690407361, 2.9842693751113583, 1.5910279546132238, 3.5731459147543019], [15.121515597689935, 16.650103431350381, 14.23464636412589, 11.445128909467762, 0.038049051502020183], [5.2422571511046572, 6.5348798786772644, 3.1063097223959097, 0.01282165305026902, 6.5866820872557845]]], [[[1.4779210663178426, 0.026250349712021337, 2.5712027932714392, 78.69374295779248, 15.794156488155792], [0.03207355017456872, 39.229257564677667, 110.49623908862259, 0.4228358011024112, 7.2309481833648963], [80.192754326885463, 52.786294358497749, 20.014478434337384, 80.262646042869434, 0.0008718002041265832], [0.093215876680305107, 58.187817600190755, 17.991084722270234, 67.042040681979799, 58.029288066065398]], [[27.113025446167068, 133.72436530671436, 2.7612078153771651, 0.013388320959430615, 0.021407254996506465], [0.84632125997980323, 13.47184066955154, 39.090542078840485, 169.21865021563329, 0.21098544911526637], [4.4393587151855209, 46.441219041408054, 304.8481430114291, 5.8548447260480643, 283.53961139112232], [29.634298471167121, 0.047995369940189291, 192.57445111189674, 190.22082752295285, 2.02428107072793]], [[49.956783676828572, 28.256637756640217, 28.934150053741089, 13.336724526460241, 5.1294527052220573], [36.461138574354322, 6.191661424610964, 8.9633003778081708, 45.729788483018361, 0.49402027491835671], [42.1262696001818, 3.823057876477864, 2.9313910471437628, 0.60225332414262167, 31.099379165694444], [0.63875826441333405, 13.586013243099206, 25.050868045453175, 0.85864534594512532, 1.0777162143735535]], [[301.20337727451397, 24.894399037510059, 0.29868740335421817, 19.904881600709022, 183.58530934151102], [0.00080326838218672486, 2.0451664329391437, 206.93011449158485, 0.0073233275239931121, 40.418201151932386], [302.09006804720633, 3.5884543595078902, 163.61202452043514, 118.19950524050039, 185.43754902697847], [183.10021960289902, 43.290429277185019, 5.5054006193996496, 0.15192759574726783, 0.0078678482429503632]], [[0.08794796726773664, 5.3385052301341025, 5.0306783314945331, 4.9044031660672021, 2.9306822898613318], [0.26661278263149008, 2.2806306437784998, 5.3912275821925482, 5.9294974980979509, 6.2156664059707412], [5.4110087378636145, 2.7100024127911917, 0.029357781989117954, 0.97188872965891415, 6.2034919571406029], [4.895614874772253, 0.10694459960237651, 3.2945011174131555, 0.94488813877041911, 4.152105202536454]]], [[[18.741839654190326, 7.9782295748356793, 5.049981100609485, 6.697334374086636, 0.13507112416303352], [4.906110245506814, 12.537720239295295, 6.5950176959367406, 0.18929022163609482, 10.475918477706962], [3.4222705854776549, 2.0023264479431004, 9.3083269348453559, 2.9744979875929545, 1.09447381566808], [0.76120268658580525, 18.693700281314833, 13.027825532609043, 5.3015891913750872, 16.992768199429726]], [[2.641943414470358, 1.5558185931844077, 8.8959953315190976, 16.387995140231578, 0.60099010296457023], [16.302471460795427, 7.9819156143820091, 13.086317250974668, 3.8752092918655157, 4.1848447442999621], [0.015789841579403888, 0.01007998005463886, 12.710374782094323, 4.5326489289534724, 0.91561815072051189], [1.8236591998568239, 4.0522083113013849, 0.14854577952412518, 3.0843918908441568, 8.3111868775896873]], [[31.403525682045469, 1.1464947085384112, 34.800356539832826, 5.0390544168910383, 0.12090524697421164], [35.578365643116818, 12.596683554855794, 17.025990778076466, 0.46232662279961045, 0.21824640098758297], [25.796947502431177, 1.3101098516722003, 2.4669326461062568, 6.5774319880503533, 14.760767553427058], [16.574559786871191, 25.222466153198393, 15.318135706382758, 3.0630558985156915, 14.059706447633344]], [[1.1090485862231514, 1.0794391135834174, 1.0502452984135289, 1.1478315845533094, 1.1026549571553601], [1.0219722442574923, 1.1555671240951726, 1.0532931200788567, 1.1135703332763656, 1.0332165412251215], [1.0150285938657717, 1.0617212164096219, 1.1447390162129008, 1.1169715428016171, 0.95359734849656153], [1.1381092701488724, 1.149533905730838, 1.1012115790888939, 1.1314436534568986, 1.1571987215999291]], [[0.031668235534278166, 21.790263353781466, 132.14400106770159, 29.929467878913329, 3.9912105929860973], [280.75288441506262, 101.25885788332717, 0.0080045979933801043, 56.335095210550662, 0.03721796127251583], [62.226305796260178, 1.6589720237546997, 10.25282634424167, 33.140076517770083, 62.826814653863025], [136.51508330859701, 7.1269189709694114, 1.4917554524127585, 1.7812332037197938, 298.12027412359288]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_array_rank4(self):
      arg0=numpy.array([[2.3412347795104345, 4.1923150063044581, 2.5722415752483925, 4.8294543253881832, 0.46667028289920942], [0.21742185116437446, 1.3712445007974432, 3.0668624967393168, 0.62500560857037613, 0.47409333932179937], [1.5605147938396644, 2.3416486293337049, 3.6077163586897556, 3.2802947433258338, 3.8948806234108821], [1.2076705010463236, 3.5551634143485371, 2.9629568080300883, 3.569812666905003, 2.4093024557065461]])
      arg1=numpy.array([[[[2.9694480859772545, 3.0271436901285265, 1.1390867944644718, 4.3562903527691947, 2.2900061858174094], [0.21550404896030523, 3.028530387093666, 4.2536284839436096, 0.058747812009427955, 1.5153246083402525], [1.0722965685549184, 0.052956855558658672, 1.4859660336855842, 0.83494994843376302, 0.40492064804339661], [3.1160831097985899, 3.1698116097667035, 4.3166634860124242, 4.9223031569745892, 0.60837411381307316]], [[2.0334703092182207, 0.20126963941661688, 3.4447751919710479, 0.74160636389881573, 3.4360003375569592], [0.49524963895025303, 3.0942883212091856, 3.2894119523109304, 1.7327900086221775, 4.1072943513407498], [0.73072459846262683, 1.6975810121180863, 3.7176603770399783, 1.1654207353328334, 3.9392303297491673], [1.2948773352773968, 1.7830193983918508, 1.1017589692912271, 2.318714218445169, 3.0244580139861932]], [[1.5300089004323527, 4.9928706919495021, 4.2216958230449606, 2.6328547380304785, 1.5952825257724161], [1.2958477672755817, 2.7315282178527109, 0.53150163428581176, 3.6808003813588215, 2.3078962610896796], [2.2190370366406151, 3.7519995880663308, 4.7039659821238518, 3.9618463332519522, 1.9695183363391568], [3.6257687472456226, 3.5448747976779913, 0.92308115608510555, 1.5874148609176157, 2.4528109153631412]], [[4.824868554764147, 0.19538857042653729, 2.7949796630740411, 4.0567077161018563, 0.23559296371358918], [2.5835777729870637, 3.545674184199775, 3.0137406565976228, 2.1048567649282428, 1.762153014051838], [1.8786449567906696, 0.90084758176640778, 0.615927826553058, 3.5283874006698053, 0.37393472818510015], [1.2360873246000603, 0.66880085324145155, 1.5671750653495793, 3.7760199819463862, 1.0791856661119794]], [[3.3919456272947635, 3.9074511641722953, 4.6880626821150448, 2.671533658171747, 0.25118635197195244], [4.6510952934227427, 3.8821085128364747, 3.4465589098726497, 3.4890279271731428, 4.9425406844396633], [4.123510270348766, 0.33650032837074451, 4.4304452516514576, 0.013015853571489162, 4.6037506041465237], [2.3442134380211814, 0.85670663806710368, 4.0769117357680065, 0.2902297325878978, 4.5864589216021061]]], [[[1.0440204558638519, 4.8170597608451908, 1.6911321673651796, 4.4450721115284839, 2.4410714136054366], [4.3964424741280732, 3.1427311903709376, 2.6232593070597248, 4.3713688335430358, 0.60112943701489985], [3.2279281161118658, 3.0992260278131081, 2.3369510060793601, 1.9255091066557042, 2.9563087443614333], [4.6304408021340073, 4.2917197933338374, 2.6932071734871124, 0.71733134418707445, 1.2860486785680394]], [[2.4955173223274492, 4.810788078437449, 1.8795435562446987, 4.9945989129232506, 0.38079756678203663], [2.9788215600520305, 0.18426219072722055, 2.4747807491602001, 0.10119349712774753, 2.8826068287054949], [1.4951778487187495, 3.8845882547228605, 0.34559127217397922, 0.86752228233055795, 4.5988960499136571], [0.24570937703234477, 0.79640781298646357, 1.3970212427369371, 4.2869899127382016, 2.2676582931607374]], [[4.197275379813612, 3.1981419889812686, 0.5532667954868189, 1.5557438014455729, 2.9389709055745152], [4.6918457394164541, 1.0290359923846508, 3.9195477411989161, 4.6508247006622359, 3.5140565246803419], [2.8791214361564763, 3.8511827084346733, 0.81783282094183896, 1.6864915014380599, 0.69255845477849953], [2.1405207566528643, 2.67510586163182, 1.5485298062897033, 1.9474723580735169, 1.4747181914076537]], [[1.2300887464108701, 1.4702017107556107, 3.4660770099011509, 3.8570428662670189, 3.2842215528334577], [4.9171809664305179, 0.69809106586048308, 3.3917233433362606, 2.5466263056669041, 3.3308624054783795], [3.2960347371380605, 2.7062943789595457, 4.5676048695335254, 2.3872637050279604, 4.0367790344531302], [1.3864823372255279, 0.38811043704589099, 3.9162285967950319, 1.7741489978592853, 3.3317988284361468]], [[3.1844024515056368, 1.8463116995297044, 0.52938900386663501, 3.3806115603749762, 0.62666423827758799], [1.6962041505622514, 3.6876349713293193, 4.1427371244403117, 1.4777786686900944, 1.5768050968245368], [1.7924151102888255, 0.24961564858492816, 1.2629932550044687, 2.6823030511031645, 0.807728489370656], [0.44340349701448384, 0.22735403898322329, 4.7426320167508358, 1.6550482091961409, 2.2221823744341251]]], [[[1.0760489002644931, 4.9682956106126088, 1.9007798348230871, 0.82955994762391894, 4.5795160440291856], [0.40756200125895675, 2.094172717723759, 3.1320512440506674, 0.26997350994302516, 4.9648698968969089], [3.7432393478409995, 1.6422167941712638, 4.8373294853433375, 4.603916784750286, 2.4934933176717466], [3.7600597984245545, 4.3418905753298853, 2.3187108855088674, 0.71569866951691175, 0.71162437655603983]], [[4.9622513310184519, 1.9367376165352701, 2.9684304238304704, 4.2910423807240221, 3.8811245561454495], [3.0488688872760981, 3.8726397501872678, 4.9871305446493732, 4.1702687699853183, 0.66332916618800652], [3.6608289254710806, 2.0234750724387891, 4.4769589082123531, 3.8614083123962053, 4.9717762172684274], [0.42439306983770841, 2.4692792089016091, 1.4941645503004908, 1.3998063204425131, 1.1880908476931427]], [[1.3704733934144704, 1.5166925083603924, 2.712715298742387, 0.13176219987576662, 0.23376484512719756], [2.4816728128935384, 0.43992338605487952, 4.8675373484252518, 4.2347331289708334, 1.5785817852208475], [4.3929399534565938, 3.3058416095963632, 0.51024801362270178, 3.7151041744075339, 4.089704832319411], [1.6204948412995155, 0.69868510511332549, 4.6457733115918458, 2.0533679142577417, 3.0208071643190335]], [[4.7747716813010577, 0.46661321601504896, 1.7601891334029596, 2.4538118831907454, 4.8041379325304003], [3.8223244584926381, 4.7308115909177593, 2.7981054044132181, 3.8767690117111093, 4.8040560173137798], [1.5665096458414347, 2.5385914912243277, 2.3069216253149656, 4.5736744857757623, 4.030201324455204], [4.0944691524235362, 2.4833388287852576, 1.8816712136043146, 1.2599367151686285, 0.5367511594413048]], [[1.3420244555785439, 2.9307910777467896, 4.4416238950575995, 0.083859243779708784, 2.5340028295230543], [2.7063854526369693, 2.4039209056117388, 1.2986234929426748, 0.49807077234414182, 1.7163940024208604], [1.7754742518793747, 0.95926006861701618, 3.9592964931194889, 3.0057670298531494, 1.8357274852418604], [0.23056825086973934, 1.8025417032807802, 4.1286795434739103, 4.0664491705248604, 3.6132434329990324]]], [[[4.317275096367446, 3.8677532713014164, 2.2568820925150663, 2.1922845752739994, 4.7670063885401976], [4.2912982481931321, 2.3364988520832184, 1.1046969444590944, 1.251761209586842, 2.0811969708217806], [1.3093455911006162, 1.8392064045906491, 2.9974233291629275, 1.7512111935379056, 2.4430153627965443], [4.760782799772838, 0.62297224403714091, 3.1156203230048569, 0.38172692112022938, 2.3248245727141592]], [[3.3841359973852656, 1.8241173478075534, 2.3232547664498018, 2.2620275153869014, 1.6070480511483052], [2.5410837790668745, 3.8574992276003632, 4.835110773006658, 4.3281445297385854, 4.5212612173556002], [1.4065628324415487, 4.5687523063527271, 3.4773121097106441, 1.7066084627258751, 1.929107035965568], [4.1437397214614755, 2.5865432924693477, 3.258155933791318, 2.2064845849603767, 4.5867659207932485]], [[2.3094176034456093, 4.5977869664761446, 4.2012077516994148, 0.29042102335717174, 1.2279441309250874], [2.5733343884523263, 1.0841311291944806, 0.51873473294297945, 4.4287675769564059, 3.0502564125981597], [1.1164401484757784, 3.709657030799804, 1.7079243076824668, 1.3512365618252313, 0.18495276679502248], [4.122846599769006, 2.0706790549634642, 0.33494732911368291, 0.39314050203512685, 1.0297501343769142]], [[4.7051566629817652, 0.82832808732544849, 4.5042341966688184, 2.2047741726354082, 4.3200660568625171], [2.0591855399714722, 2.2017887659157029, 4.4172730003728695, 1.7560437809510949, 0.32129332968207469], [1.8853021344240077, 2.7259762974956483, 0.045239159689823713, 3.8843302368848227, 0.42632671290077689], [4.1359949997543515, 0.95044408262307667, 3.3343518919179109, 2.1851006200115748, 1.7758288168447169]], [[4.8722484533609842, 3.8873571543926095, 2.6613650221554122, 1.9483024664808382, 1.7052942773508259], [1.7739303989096162, 1.4003717334254282, 3.6059467102596532, 4.54989273362931, 2.5997748074203759], [3.4158034830876178, 3.9179989473683232, 4.8777647134393947, 3.234170890480383, 4.0392750950860883], [2.3298114632013043, 4.4423328262142245, 0.025739869560743575, 2.9420795993606998, 3.2833059701541467]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[12.503962144512125, 13.132971468884431, 2.6352989149763579, 40.682679163335443, 7.0150578982853897], [1.2012043223198474, 13.148472699889821, 37.280491979291135, 1.051245339557342, 3.6293516555977487], [2.4897432889147169, 1.0460793854357846, 3.5398321740898409, 2.0345485423809957, 1.4112238108069834], [14.165151433495287, 14.827602866508775, 39.334134567573749, 65.844594509532726, 1.6778794476814614]], [[18.439177291957172, 1.3343847609175128, 139.38427459987696, 2.8947814378595407, 137.64227329374188], [2.0336212579703656, 84.343546215875733, 111.55969637002809, 11.98345120978334, 360.24786765425074], [2.849983736662459, 11.393730285539579, 206.0972652183294, 5.3139894855375527, 283.13223152541917], [6.3973698210886418, 12.877969492975538, 4.8505856288768872, 27.751900260360539, 76.310747786603358]], [[4.2440513973698257, 111.8493664521435, 53.977332173870671, 12.030709610357292, 4.5140167932959399], [3.4017435470388624, 13.206207985414711, 1.6522713316448412, 32.379911592159331, 8.8502789983106585], [8.1376119737560337, 34.632953517369593, 85.131940022913213, 42.227183547008252, 6.4286017484884468], [30.739410339327392, 28.477626786020029, 2.3919442202028178, 4.4805876250865291, 10.148857336162683]], [[1993.9683439443688, 1.3602723328857327, 81.56074383968199, 594.80491634899533, 1.4491774633645123], [58.46606092000534, 265.99881805501525, 115.10426014173291, 27.511068902201647, 16.037365723106895], [19.266447667520829, 4.1313112562578302, 2.6377391638348078, 258.85544759340439, 1.8019105843757763], [7.0041869127851397, 2.8667617954812985, 11.797445824993813, 382.30744175124897, 5.4708270647657082]], [[0.075387686026403333, 0.0508947881469035, 0.028073645969654344, 0.13054176051496408, 0.8257714547491406], [0.028875841506512141, 0.051887347668440081, 0.072314264055816688, 0.070011141214319422, 0.023124331258833779], [0.043167802297146972, 0.7737876707496214, 0.034163910419029844, 0.99012923649615936, 0.029936792088344901], [0.1675286686472649, 0.52052164692890301, 0.044728422454372183, 0.80156159959298434, 0.030333926676272525]]], [[[0.20329700161125591, 0.00064231765364434785, 0.075734211095125972, 0.0011330912539764895, 0.024116248981215258], [0.0012203701967391379, 0.0082665074703252902, 0.01826309688277427, 0.0012679666025041353, 0.39960732378313629], [0.0072587582318724315, 0.0088339064339752991, 0.028268972345493488, 0.052962838853615135, 0.010986607841924668], [0.00085392877292107773, 0.0014318284239757621, 0.016414217920867218, 0.33467772319701722, 0.14052048563055217]], [[2.1987335605971339, 4.5669969442116747, 1.8101452487306684, 4.8398725459423408, 1.1277504780655896], [2.5611842180973361, 1.0599004868787603, 2.1843856368697279, 1.03246451953112, 2.4845534486901903], [1.6032865879853495, 3.4090622047990107, 1.1152846177215852, 1.3150741166248274, 4.2714680962533569], [1.0806633338030049, 1.2858768473285465, 1.5543629165317858, 3.8708842817383502, 2.0461135792077436]], [[110.35449376875493, 36.017825680749965, 1.8589675052665902, 5.7170454894334686, 26.938933709891327], [192.08550785487566, 3.1682975162696829, 80.839117328880874, 183.45514423403026, 51.318186078746621], [25.191382703201867, 74.877043967168532, 2.5005444337027911, 6.6192046519566619, 2.1730226627099207], [11.009795732670804, 20.042818213493778, 5.6710129297839105, 8.8679595883132087, 5.2207986588645463]], [[0.56094347467324757, 0.50108128443812083, 0.19611821369613336, 0.16319825425716378, 0.21361785649851997], [0.099157203440424169, 0.7202916568342046, 0.20309288468760145, 0.30212913507794736, 0.20898610154249952], [0.21243510807059798, 0.28028624497717824, 0.11686336145660391, 0.32562750200368473, 0.14997831351215599], [0.52119069415534236, 0.83325977458849976, 0.15872113087580772, 0.43437783017011422, 0.20889414411558133]], [[0.092858157373484435, 0.25208304783910213, 0.67360590937074438, 0.080208722097169907, 0.62643421295138446], [0.28196741543631793, 0.063782892745365263, 0.045413937076955423, 0.33189332455804932, 0.308248145645456], [0.2624299959982776, 0.83002380034949497, 0.38959935901117132, 0.13507302500905105, 0.54725028990057034], [0.71825209146390079, 0.84392983004662792, 0.029022948779923865, 0.29076294016257409, 0.19041903900812265]]], [[[1.6142310645040197, 9.1245616626895618, 2.3300203192615996, 1.4465295197629902, 7.6749119871529006], [1.1988604911148735, 2.5394305667334165, 4.0301845692745095, 1.1276574946614673, 9.1106618971993605], [5.2899061992707246, 2.0767670039348105, 8.6079644299426334, 7.7587057863970701, 3.0332778424494524], [5.3296516504549762, 6.9047511471960226, 2.80628878571504, 1.3750597564413156, 1.3725688606185888]], [[68.180422705704444, 5.1959699632932761, 12.499698605328787, 38.515310014308291, 27.174393178141088], [13.385151817337661, 26.978918403062416, 69.63909193654122, 34.754010220054084, 1.7583873069425404], [22.529721982299506, 5.5939426099121841, 45.116312328896505, 26.722327011229194, 68.735222346917311], [1.4349027263426617, 8.1743329226847337, 3.5655478360445136, 3.2904769371890579, 2.7480547673198115]], [[5.8032617379242009, 7.0008405242593756, 32.479760452631012, 1.1841921149774504, 1.3497748827830267], [24.14729356404348, 1.7584886029107416, 515.64338291002002, 228.94462087172431, 7.5794360692721412], [280.47128892150818, 69.521959881319191, 1.9245390070410779, 117.53747644544345, 190.07105283963304], [7.9981990171038762, 2.4509246964048543, 387.94963326429058, 13.938084769197822, 48.227148322791031]], [[290.64720579511658, 1.7407313966086142, 8.0929009310925064, 18.448162956285248, 300.96540589977116], [93.753259089883656, 275.85863049436171, 27.770172926942823, 100.01727534392964, 300.93612043764199], [6.4295782194780386, 20.402905446130891, 15.49424974738232, 228.88509968949103, 120.01422469833794], [129.53565436546862, 19.106733994381496, 9.3492848619210562, 4.4670165778502238, 1.8919812584737146]], [[6.2009408953088423, 53.77928079127485, 419.52025317613374, 1.1207748915203528, 31.355508634715029], [39.637365450642015, 26.272489965871152, 5.8456065001935968, 1.9683751698906755, 10.316220582081003], [11.179108247062771, 3.684999727248472, 217.74160622057343, 59.550834421520868, 12.133506952753129], [1.3681987950208228, 11.598193048585859, 274.13287626600044, 251.89204459519325, 136.01885487078277]]], [[[2.258364497148722, 2.0747053044842119, 1.5309041764388072, 1.5123570699601565, 2.4583789229970194], [2.2473218543737374, 1.5540768050241125, 1.2317659975779929, 1.2664262422331285, 1.4809857609803694], [1.2802619919033593, 1.4148816990434314, 1.7604926669728738, 1.3915828163266846, 1.5856281318693382], [2.4554936214664305, 1.1247386386423244, 1.8001980289042561, 1.0746868445919864, 1.550657161945203]], [[73.144839899337384, 10.111893670729675, 19.045269260349038, 17.622174603280989, 7.6781712690806065], [25.106161192126628, 133.33383117250725, 460.75179070105071, 242.21432581437315, 309.44252851293209], [5.9541338615128199, 328.65551313691492, 82.32100175023831, 8.7116737044990771, 11.55226302592491], [191.69848161181829, 26.59635497666347, 62.342809533623665, 16.423402830780049, 336.25123937207837]], [[12.286007032158562, 147.53414058094552, 95.899440609879122, 1.370878452950036, 3.7953544310782132], [16.364660112009766, 3.2464753117725835, 1.7567109495021573, 122.78966243719825, 27.47155129674244], [3.3624287241823905, 56.226254294871978, 6.3925059618145745, 4.3392283015262842, 1.2224944699262827], [88.074692255881331, 9.4796377384641879, 1.4388089685053667, 1.532690019503649, 3.0602662474645834]], [[398.3669582075936, 2.8692720569814174, 308.49237032845457, 16.537062294945596, 244.04209656273943], [13.740402995438288, 16.474357686153589, 276.17586319672461, 9.3426389095769959, 1.5050859067063969], [11.012948330808216, 32.09950481856626, 1.0592566878350722, 140.1707313793992, 1.7203131455840119], [193.08063548616846, 3.3516493404660075, 69.616842617133543, 16.128198632737003, 9.5808418948911651]], [[72.555344618278824, 30.517430751943113, 10.38370186040058, 5.5467655183730065, 4.4795778461761984], [4.758265023076186, 3.4260193725453814, 23.827570373032557, 54.646777499440155, 9.8362926507859143], [20.158848092982041, 31.350884831136902, 72.90814037640402, 17.183111218573259, 34.879006922754499], [7.75773218541321, 49.71510505514285, 1.022892119712101, 13.290908599660602, 17.941802085226566]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_array_rank3(self):
      arg0=numpy.array([[[0.3496580366623756, 3.4606761187258748], [0.051300186147572292, 4.8474987046905635]], [[4.0545731686044482, 1.8748110864441245], [2.0714493340858526, 0.43122482727947992]], [[0.78388820681148452, 1.640542084358996], [0.20039251286290044, 1.2314297812983934]], [[4.9299483023499979, 0.81025708966245391], [3.2603559721552222, 4.4934678854546233]], [[2.6105560541433968, 2.4421457630272236], [3.5749638445885914, 1.9452742346580603]], [[0.62355634023067263, 2.8673007809830255], [1.7635932210946708, 3.7148714712005733]]])
      arg1=numpy.array([[[4.6415896059669945, 4.7929301059795133], [4.958600522674546, 2.6049827862075445]], [[2.3831072182810242, 2.71733365222524], [2.5768041229585741, 2.9174975979603901]], [[4.3898550624906338, 0.092998652689522857], [0.50922776187279428, 2.520523551393492]], [[1.2695758291933139, 4.0803148325429257], [2.421229473720389, 1.7831780604694243]], [[3.3785839953181318, 0.79647381036831555], [4.6860771616650592, 0.17108847942912839]], [[3.3599894231008971, 2.9026392542900994], [2.7016016807332268, 2.5395844858856802]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[0.0076169390611739432, 383.84932213444557], [4.0178628372806274e-07, 61.060704756473356]], [[28.105962107525514, 5.5171734257126657], [6.5309590338952823, 0.085950634733738582]], [[0.34339262259091263, 1.0471129634872616], [0.44106101672161568, 1.6899725808191945]], [[7.5790700340877697, 0.42379167162316184], [17.487687894442391, 14.577037363833718]], [[25.583945507194411, 2.0363382048522087], [391.44762426564705, 1.1205759351441442]], [[0.20454295208670276, 21.275502283807], [4.6309531544569209, 28.016921488667204]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[2.7010321866075975, 0.35520090447806413], [4.24621711957891, 0.040731959393781852]], [[3.2081672971378072, 4.9053701113456905], [4.0008027962684354, 0.30708581141930469]]], [[[3.9789999095002764, 1.4875966166728263], [0.090876723895219641, 4.5509299294251022]], [[3.1457398852858276, 4.6624837526768097], [0.94590337862638618, 1.1846516050492408]]], [[[4.7699168321290815, 3.7848015409719378], [0.41034197214565898, 3.0190893547458701]], [[1.5071603730674241, 2.6205226508010591], [4.1976666828509011, 4.6691583098743248]]], [[[3.0133501964034357, 1.3842375003414644], [0.94741747591272119, 1.9382367257675293]], [[0.28231519471651068, 2.2760931225447756], [1.4658272306233515, 0.75940257984696558]]], [[[4.8727946029247331, 4.3228275775244231], [1.4112101378731992, 0.88424142422143159]], [[3.7227779527032738, 2.9288199657521146], [1.721319993291172, 3.7048203640515256]]], [[[1.0599128745423434, 1.0982520568771135], [3.4873806738001059, 4.1439207998232783]], [[2.1728462302123255, 2.365414093735136], [0.25408082881112304, 2.5111899861960563]]]])
      arg1=numpy.array([[[4.2228980469202133, 1.6657152684585812], [4.7711615568199068, 3.5431179557415735]], [[4.1762417904839477, 1.4239041526492475], [3.935759529967735, 2.4207250769264053]], [[0.44660125010957202, 2.1130743057748518], [2.025670386551349, 4.711216270968043]], [[3.5040001178242988, 4.7814053293360939], [4.5137149527601776, 1.2802762843783462]], [[4.3086974053144989, 4.9096489531042122], [0.68687163768891057, 1.7000515622526216]], [[3.5180550095083598, 4.6836154219210702], [2.3002885973553249, 2.8743403859044054]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[66.421117541226039, 0.012638565937076027], [11.119182235887264, 0.0048366872800432786]], [[260.27590900643338, 1973.8159216485217], [135.98104104622109, 0.0152510876265549]]], [[[319.7429284581591, 5.252182702887608], [0.03288034916076444, 8.6511040335568161]], [[90.97399953974994, 428.07220930706586], [0.87404076140028564, 1.5071022737606443]]], [[[2.0092027991081052, 1.8119857435036137], [0.15224695967253526, 10.327936891054135]], [[2.2955795977911335, 7.0390818542552989], [861.24198344667877, 1422.0922752724091]]], [[[47.707853428325016, 3.1246544459522299], [0.77238687231155245, 23.670557834042906]], [[0.0033171948774326034, 40.950005429262504], [1.6316682064999724, 0.70302573575888927]]], [[[919.2375117108121, 548.69570260844762], [5.425536291902878, 0.54661514648689957]], [[2.4666778994514584, 2.0919820942322773], [2.5175430395009708, 9.2668789639101536]]], [[[1.2271618324654152, 1.3905671449215871], [347.42282899505426, 779.32349964007938]], [[5.9602427664550852, 7.2459348171397906], [0.019484384623554198, 14.105549198806548]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_array_rank4(self):
      arg0=numpy.array([[[2.9245154844018786, 0.46979891623203768], [1.2386887494231966, 1.1239383257524596]], [[3.1723504867645165, 0.98464003405595912], [0.85567108490973709, 2.9201582410968143]], [[0.75785335867657555, 3.4153784889632282], [1.0055015750636762, 3.3434731514791509]], [[3.3629735009410915, 3.9436123257061242], [1.1907608908908009, 4.8029299381233379]], [[4.155535356525248, 3.166757796161809], [0.81735352961627172, 0.51202972528692181]], [[3.6701244565994924, 4.3514053530525345], [0.63264324143579276, 3.0479626300303408]]])
      arg1=numpy.array([[[[1.2792289694356389, 4.995190001507944], [0.83935565652357003, 1.096317671693412]], [[3.7433807969010169, 2.9664164162314339], [4.3177001032970619, 2.9618287662653255]]], [[[4.5573612097359275, 2.6958098515855595], [4.1795809646469291, 2.3064021736695102]], [[2.5650425309133471, 3.3711073273996681], [2.0103920173733156, 4.4069919652927192]]], [[[2.6382649326825405, 4.7516027669656422], [1.7673870095056157, 1.6963362036998135]], [[4.9058104513747054, 0.1473454028173882], [4.5517522898395892, 4.2213356930472576]]], [[[1.855227512726092, 4.8314498784326503], [4.1826035908526622, 0.34902940298422852]], [[1.1767059488120137, 3.3079987349025588], [0.87366290441318117, 3.7157559608992141]]], [[[4.0247209483713124, 0.15211828104238664], [2.5485496584497853, 4.7967056154975563]], [[3.2283661648846671, 4.5346839854490977], [3.2073690201256064, 2.4463396916304592]]], [[[3.0393577851517213, 2.0469217994801694], [4.5375437664041698, 2.45094963151936]], [[3.9420326872651299, 3.6240805555490354], [4.9729198531177827, 1.9219860833777511]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[3.946296228078356, 212.82758014505723], [0.53041706762940244, 0.43682887621598526]], [[2.2283986586267255, 1.8869681752111511], [1.656116233293379, 1.4134828353824731]]], [[[192.74142391643855, 22.471304169625931], [0.93735202823916719, 0.96492861890867887]], [[0.67044662313666215, 0.59128811228759792], [8.6228192696285966, 112.47142675817138]]], [[[0.48118690743001274, 0.26781605380239937], [8.7658265709891285, 8.0332561407680938]], [[1.0272812123979016, 1.0008087369144962], [243.2308844667547, 163.23566494119223]]], [[[9.4883747080931222, 350.62100114134267], [310.73436429299909, 1.6143013212330772]], [[1.2280702580562302, 1.7816711852377634], [3.9391882067889394, 340.65197906228701]]], [[[308.88795450997407, 1.2419510330916987], [18.873037416358724, 251.94285968756367]], [[0.52146747006698335, 0.40068839888892716], [0.11684290542014132, 0.19446311849819073]]], [[[52.031573597142149, 14.317177932745059], [790.3318712668563, 36.749280433255429]], [[0.16449843746437195, 0.19027587917965771], [255.23553450036658, 8.5164732843877786]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[0.15074731836058505, 0.85807308000714233, 3.0273021514110008, 2.984687743760964], [4.2207203639158299, 3.3011772075938599, 4.9511060263969622, 1.0051949106121869], [4.6298154958183968, 1.9727696848518779, 3.8230277903404875, 1.9744931544967208]], [[1.5014928642522458, 0.051708526097382516, 0.10975720663170042, 1.2583690007263653], [1.4489452958239024, 3.2772628257795207, 1.5265174192449664, 1.8999480272345159], [1.8396391550921647, 2.5310313394742057, 4.6496185621430985, 2.4196432911533465]]], [[[4.4138735365558723, 0.13542405533432211, 4.9717681532929667, 3.9971005634025674], [0.62770048853337412, 0.27678880740296008, 4.0525945880153005, 0.2967049554965191], [3.4580192870758055, 1.3952051725609691, 0.58442969925093002, 2.7147512480836786]], [[4.6555357646326758, 4.7167068084381825, 4.7165514257312591, 4.8148832049423502], [2.9857013140628297, 4.1840306823742592, 1.3393295711794542, 2.2849281672986361], [3.1793681556555362, 2.1691512109752398, 0.6798915144059684, 1.36725883660759]]], [[[2.0792140056978448, 2.6583124529725075, 0.72895834115807689, 3.0054962399499137], [1.6174100720329252, 1.4607633428656259, 4.7018536980215551, 4.3829986856840399], [1.1682927106327075, 0.64142481726868017, 0.80949988466444689, 0.61338087069166525]], [[2.5076081801258807, 1.3804690467769953, 4.0496896098711641, 1.2063815372142286], [0.3244554360076462, 4.0742061121057471, 0.5237301138824545, 2.2148959198642371], [4.1908662031024049, 4.6525452998914592, 0.20190228405857391, 1.2389719402109123]]]])
      arg1=numpy.array([[[[4.8479854340733732, 2.3675353622991264, 1.1886079616579366, 1.2176045595613718], [2.8437113374076413, 2.2052043208183343, 0.70383368377111566, 1.1263907683990766], [1.9801043485982242, 1.47972896573732, 2.1811191611860909, 1.0165411914755564]], [[3.4932463487002154, 1.0714721785066579, 0.41982568748035087, 0.5888635647344701], [3.6572746991187954, 4.2755452516688317, 1.7353626171666932, 0.86772371396343806], [2.6438352301850401, 4.5186170830011019, 3.6568575177032936, 2.0125685615722504]]], [[[4.6226260183092514, 0.15309788808271782, 0.20751728640835199, 4.6430874187739137], [1.2998112199082639, 3.1662599819603248, 2.8427397257572875, 0.2714777116699893], [2.4660494534540809, 1.1075817234227205, 3.1848502481865966, 3.0007491281670249]], [[4.3768142218733503, 0.46246439886593355, 4.2304985855040034, 1.5461142899058429], [3.1012530316415141, 0.98703350135402412, 2.8310895187634837, 3.8086085403370804], [2.0776487589239743, 0.68742204327206857, 0.3578678734925661, 0.4621006648068911]]], [[[0.54030607115898421, 4.1263198277612201, 3.4906705242760832, 2.2064556666949722], [1.1699470409706592, 0.96182691677747312, 0.53094762832662534, 3.7564794458578263], [0.54561039486811913, 3.4264104961843467, 2.9768856175531546, 1.2781538501701653]], [[3.0820917906514724, 4.2364006121468059, 1.6814049675136276, 2.4576603965735102], [2.0946482839875191, 4.8231182091629741, 2.3597359096847494, 1.5229005160402973], [3.3292032510213758, 0.49593725641395459, 4.6705207941878628, 3.636549243040188]]]])
      res=power(arg0,arg1)
      ref=numpy.array([[[[0.00010379250824760492, 0.69601143625110473, 3.7306689937862547, 3.786509656067031], [60.036989528330501, 13.924216346758987, 3.0828672726220931, 1.0058534174715221], [20.791486724405434, 2.7329587383948839, 18.633741215050769, 1.9968379440700501]], [[4.1365595175367869, 0.041842501827563675, 0.39550241858831764, 1.1449151474806807], [3.8816052395122114, 159.98960649762969, 2.0834792268490365, 1.7453028643329187], [5.0108279625455889, 66.427372648110605, 275.83544183519456, 5.9200570868932898]]], [[[956.66785525113369, 0.7363163820155888, 1.3948838476904775, 622.23578847719898], [0.54590413882430266, 0.017127627022897544, 53.410568048538082, 0.71903154534061231], [21.319403187432687, 1.4461005502157092, 0.18074955483580313, 20.0223497697359]], [[838.64688791113019, 2.0489624538875035, 707.56738738948775, 11.359405233789484], [29.733015814636406, 4.1070969218146658, 2.2868082574988153, 23.270465142919555], [11.058285069458385, 1.7028368521312052, 0.87103483397718551, 1.1555180045317741]]], [[[1.4851245025439652, 56.501537135195548, 0.33169612345097088, 11.337071441141939], [1.7551268441864754, 1.4397839937639594, 2.2747815144478616, 257.51400259643117], [1.088571329255966, 0.21837464142774507, 0.53305483366776651, 0.53541011692366347]], [[17.004151886402333, 3.9193052040555449, 10.50320706216363, 1.5858484276145293], [0.094632818361268661, 875.6075188818711, 0.21735417650646249, 3.356902704672371], [117.97115196553986, 2.1435452183510466, 0.00056839163144127721, 2.179829978432049]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_constData_rank0(self):
      arg0=numpy.array(3.71603110941)
      arg1=Data(2.44051309701,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(24.6198894393,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_constData_rank0(self):
      arg0=numpy.array([2.9044035624782869, 2.6659762640226172])
      arg1=Data(1.75429778596,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([6.4914213726611658, 5.5857144017993567]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_constData_rank1(self):
      arg0=numpy.array(3.89743403053)
      arg1=Data(numpy.array([3.5670912365550653, 4.860831474436873]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([128.04475205562667, 744.17667504881706]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_constData_rank0(self):
      arg0=numpy.array([[2.2111792727812754, 1.231404546784129, 0.32066035199946535, 0.84802367624138231, 0.49760234163835726], [1.9826017786648478, 0.9905889745002997, 3.4667232977370355, 0.03734734663437294, 4.8114644027565214], [3.6155290905450093, 1.219523442921943, 4.6894368874492685, 1.8151339573326639, 0.83709436015278804], [0.65886534379028938, 2.3742666285986833, 2.3176190662770444, 1.8874069808839031, 4.8300145553553007]])
      arg1=Data(3.72536535779,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[19.224282613349867, 2.1715799985210116, 0.014449011669251083, 0.54111996767342962, 0.074263643225396997], [12.802972215547621, 0.96538767386115198, 102.65880530690609, 4.7989468736235823e-06, 348.12253096134719], [120.05883138224655, 2.0945455570297336, 316.35142070848508, 9.2157234392187846, 0.51559237298366323], [0.21132523985652221, 25.060200809399824, 22.904194649890609, 10.658595714923287, 353.14887172383459]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_constData_rank2(self):
      arg0=numpy.array(4.70470825688)
      arg1=Data(numpy.array([[2.1208739120036282, 3.5794846458506169, 1.523229614483012, 4.1301123214618478, 2.824184898743801], [2.6177284136919559, 3.3113189606106843, 3.6040521624934923, 4.0137734204121065, 0.017971370393516338], [0.96137378626279857, 1.3401997227402149, 3.3115212778453276, 1.1918202788236505, 2.0424676627024625], [2.2876777427308013, 0.052227352438751658, 3.7521479670173035, 4.6110524824120409, 3.8495737927632692]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[26.69052112431045, 255.45856304568022, 10.578442304115754, 599.28799920697418, 79.315248366190104], [57.611336676466273, 168.64375874866761, 265.36456105710124, 500.48822703035432, 1.0282206797189497], [4.4315464509731468, 7.9676112266851664, 168.69660330796845, 6.3319591907622153, 23.638846850492115], [34.55716316691823, 1.0842379451011355, 333.76607694452338, 1262.070352674647, 388.11841354445318]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[4.8471062667810925, 3.5800996038882009], [0.23039139706763781, 2.3750506511173333]], [[3.5661181342975343, 3.3505519737926188], [3.4859447790468931, 0.64666015636485119]], [[3.6843576424512356, 4.7011538883400181], [3.4406987152020636, 3.8248118184454638]], [[4.3704973706962651, 0.071051774532058354], [0.7411758681234567, 3.2646256199241082]], [[4.3454976275074522, 1.0546924742686388], [1.4197058171283945, 0.93388515783154302]], [[0.83543770812263896, 3.159388737475846], [2.3188941078758369, 2.6237653818958591]]])
      arg1=Data(3.55541867878,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[273.63864482029908, 93.18135438337697], [0.0054112877223820221, 21.660793411896186]], [[91.893966586437458, 73.622152648253447], [84.757169884313356, 0.21226299122689576]], [[103.19371661517209, 245.45298634024053], [80.910243197050065, 117.87527927376438]], [[189.3892811838862, 8.2578701836982731e-05], [0.34475903483373366, 67.126316013240398]], [[185.56566276235876, 1.2084315873472082], [3.4763932540990834, 0.78411698238991034]], [[0.52768146376474767, 59.744519356483153], [19.894205307798444, 30.864110253017827]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_constData_rank3(self):
      arg0=numpy.array(0.416158708974)
      arg1=Data(numpy.array([[[3.4255203546842807, 0.1554404624374591], [0.42008129757854495, 1.7809740034603809]], [[2.5003730182252575, 3.3612968870943751], [2.8699427451349462, 0.91467196212621371]], [[0.31894667242454694, 0.83632495663159179], [3.6261649787530623, 1.3411256154220004]], [[0.15113744450340977, 0.39882927579026484], [2.4524548065566916, 1.8416858701626051]], [[0.07737647453130693, 0.97548411758247011], [1.3856349018596359, 4.6571977289807585]], [[1.3695191711267094, 0.13933641443402286], [2.5904979228553029, 3.5695750225504534]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[0.04963224409664807, 0.87260448524375311], [0.6919230866109769, 0.20985057078110195]], [[0.11168772521148446, 0.05250690504868244], [0.080778371516402506, 0.44848391451251651]], [[0.75607333375095698, 0.48037100898878615], [0.041626536255099433, 0.30858760326517032]], [[0.87590252108812505, 0.70493542841582923], [0.11647960092077701, 0.19897323327952768]], [[0.93441457242077264, 0.42519993499768738], [0.29677817823865582, 0.016858301690712143]], [[0.3010009624402803, 0.88501149686934832], [0.10320273294696586, 0.043743788385669488]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[4.9483304751762134, 4.4768048114207479, 0.24751034372854303, 1.7440987330331605], [4.253731200823843, 0.53832489301511388, 1.9269547454526097, 4.6156487122298442], [3.340998693524563, 4.1226576323172086, 3.964961669538194, 0.29155244969071192]], [[1.0445111944167218, 4.790264583581747, 1.9974063657503256, 0.032574509940280029], [1.7806324425001534, 1.0648700520001797, 0.091495800584689563, 3.3074812199560415], [4.3570959392440756, 1.5871661814172247, 3.2555498456501111, 1.8100822145298459]]], [[[4.5365446079014546, 0.39229349102418165, 4.551000397327515, 4.367379362207263], [4.1119153580330501, 2.4129044493761405, 2.1442105775019638, 1.081351754571819], [4.1385708841799032, 3.825643052207262, 3.0287631467396903, 4.269077337693175]], [[0.44782756726650208, 2.1311001858053085, 1.1621891194459275, 1.9616020699349954], [2.666008574533945, 0.40624196033586202, 1.8992293264512534, 4.2079373549408752], [0.34070070196488583, 1.5984194606044793, 1.3134970508103276, 2.8338533429072119]]], [[[2.2085117323593897, 4.4597746792937905, 1.0899969122786313, 4.4184801552425634], [1.4058820401345726, 0.62488970320593917, 2.1111316277271253, 4.1187237271473762], [1.8385853403541663, 4.6382962589843704, 0.21379383032072216, 2.4427543703805354]], [[1.4323107979545362, 3.3238491510086154, 3.7268308794172289, 0.81545648787097502], [0.78835651060031842, 4.1341114827403977, 0.10269588207215868, 3.501449803534137], [2.8262530397440533, 3.0536891337819725, 4.8730165964788412, 0.55553680781472126]]]])
      arg1=Data(4.63133061589,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1645.3860917344505, 1034.7817660330275, 0.0015542838392035203, 13.146055960916442], [816.66125055160433, 0.056803878008730692, 20.860954253791235, 1192.0142917972116], [266.83597068836934, 706.46284418353093, 589.71552995192189, 0.0033183794140255749]], [[1.2234685603075142, 1415.6952997413678, 24.635386183572709, 1.2961218918939533e-07], [14.470793391748963, 1.3378876375926019, 1.5484792920798901e-05, 254.66200434795013], [912.71083136732079, 8.4947121836418678, 236.66435008032312, 15.612988205075695]]], [[[1100.3008756001395, 0.013118333159428987, 1116.6331361333748, 922.73020311663583], [697.97769936380303, 59.111201817833937, 34.214406388594291, 1.4365222506082906], [719.18088191800734, 499.68572367654849, 169.39429317745982, 830.39601518473728]], [[0.024220264348105727, 33.256240291478278, 2.0059424379536295, 22.655720133922934], [93.822196693587387, 0.015422435786984047, 19.506712232581485, 776.73181930178498], [0.0068275500030539945, 8.7772657068255366, 3.5357725994881188, 124.48274831751728]]], [[[39.231916591232697, 1016.6765204584598, 1.4904890438971659, 973.80539902235068], [4.8439560174613847, 0.11331773597022507, 31.837407812032318, 703.34618528345209], [16.784634479960712, 1219.3445764880751, 0.00078883488698834484, 62.574817004544627]], [[5.2803190842290073, 260.55136288523852, 442.65255615474524, 0.3887476504275143], [0.33242102685959485, 715.59892465039457, 2.6434754610412424e-05, 331.58279176702939], [122.94404990888125, 175.94788443949966, 1532.566851594416, 0.065717457856719202]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_constData_rank4(self):
      arg0=numpy.array(3.65477830729)
      arg1=Data(numpy.array([[[[4.7295335245694892, 1.8400642348124787, 2.6420485274777725, 1.411013013645974], [2.2354897184384863, 4.1754443409980917, 2.7370428756806695, 3.2104246783314316], [1.5268587936746159, 0.6179467961122268, 2.2473159529524134, 2.5064187023197908]], [[1.3144971562190693, 2.4351377966848324, 1.325599043957056, 0.73589056967823463], [1.0333917342529115, 1.396202415731137, 1.2531377191542774, 4.776199784523647], [1.6110894401368578, 2.4767586597305211, 4.1962144571800062, 4.7000652731601003]]], [[[1.6507308596749952, 2.3535355332501959, 3.2830909905971186, 3.1440279336080104], [4.1967922866584324, 4.2733391576165012, 4.1650273299434257, 0.78671395201555461], [0.76278975091182122, 1.5343263172123569, 3.6658759085706949, 1.5453940277267499]], [[2.9609146172527461, 0.4393633474333854, 1.1820651367385098, 3.0673922607696595], [3.6776491652263652, 1.2897335515914947, 4.9420469918218473, 4.878275635327876], [1.5712858304666124, 2.7176767477087225, 3.8217063919865377, 3.6327453362264168]]], [[[1.9165717809208263, 2.2148951722139465, 2.8778107880900934, 2.0714311537238146], [2.9891541605467182, 0.066088393662098704, 4.0829006579993381, 4.0856793800142084], [1.6489563527438285, 4.0663835873687866, 0.081040269855538902, 4.1085846767574097]], [[0.11507683672329533, 0.32640067436067777, 3.8450486113677003, 2.2704668969523465], [0.5193839318714053, 0.72541436369698786, 4.3303500373079329, 4.0106071978366877], [1.6340821502715943, 1.3125518983751367, 0.067527617698647277, 3.0700962491342194]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[459.2721928333516, 10.856765718369735, 30.697724064874759, 6.2259302401881609], [18.124668184733807, 223.97262397611729, 34.719606587045234, 64.124348913546726], [7.2345134147861021, 2.2275023731963617, 18.404608818876877, 25.749300391261372]], [[5.4938808268572865, 23.477085910044188, 5.5735006298368841, 2.5953989839829799], [3.8164180810462849, 6.1075628066445127, 5.0739057763896751, 487.90668758618381], [8.068995540100536, 24.778268310653232, 230.08358054612614, 442.06244435574371]]], [[[8.49438783417712, 21.120950291250381, 70.456976829433216, 58.837041614862457], [230.25595179432696, 254.27058147403042, 220.96913133004136, 2.7721111013478006], [2.6874762278139355, 7.3048702161379531, 115.71208548845865, 7.4104074450454931]], [[46.406998104047474, 1.7672602532635271, 4.6274145783473433, 53.274035214449718], [117.49122406243201, 5.3203568958628553, 604.902949640938, 556.91804546007302], [7.6632955332450043, 33.859017867052778, 141.6085998418157, 110.84875319655585]]], [[[11.988465091158718, 17.647297081851825, 41.668471410395433, 14.653045238278768], [48.136932883542052, 1.0894281223248692, 198.65783791463858, 199.37455859295696], [8.4748746853167063, 194.45042155502381, 1.1107451112672413, 205.38193347295618]], [[1.1608397415405385, 1.5265752695233006, 145.95805067095637, 18.965196880738716], [1.9603832326060242, 2.5603980166451201, 273.76966160374411, 180.88998904804714], [8.3130649972563369, 5.4800475220591496, 1.0914621130411233, 53.461059711436569]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_constData_rank1(self):
      arg0=numpy.array([1.4783936102507462, 4.0106593927683702])
      arg1=Data(numpy.array([1.6273546778227346, 3.3233326098871907]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([1.8893337201057752, 101.08499067812811]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_constData_rank1(self):
      arg0=numpy.array([[3.1375322635746721, 3.0078888480715835], [2.2626144309753267, 1.019158029000037]])
      arg1=Data(numpy.array([1.5601255119776016, 4.6465592330671077]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[5.9530553482581761, 5.5737618654454799], [44.434360885170527, 1.0921813552756787]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_constData_rank2(self):
      arg0=numpy.array([1.0552436385164441, 2.2716309177249743])
      arg1=Data(numpy.array([[0.70374617286164209, 3.2673815518282026], [2.8772444794294318, 4.2611615146544199]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[1.0385667234193299, 1.192071543137762], [10.599146239933456, 32.992315432739602]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_constData_rank1(self):
      arg0=numpy.array([[[1.8415894226949345, 0.21536299740352913, 2.2301855777068624, 3.0252398054536838, 3.4872283515784304], [3.2500532952716523, 1.5884656235909438, 3.3346262666533266, 0.32970244049688457, 4.711896173399337], [4.1129600648150015, 4.8249422710367345, 2.7146322008284414, 1.4095554534267916, 2.348520248451611], [2.3976112505241178, 2.1557244472020214, 1.0122800368870457, 2.9213102290506168, 0.19331539047547944]], [[1.3039991557716293, 3.5983768779167882, 0.80853122702269364, 1.9141732201568538, 0.14875380224154097], [2.9893246976899266, 0.53043535255198671, 0.30412029953490027, 2.3284185359224274, 1.4372705706550266], [1.7250613691584245, 3.5904450977945581, 4.6399486411115118, 4.9571601386174011, 1.0372373035072742], [4.86066549009265, 4.034566747259074, 3.0964887234352725, 2.2353045522355974, 2.334046799570431]]])
      arg1=Data(numpy.array([2.1046889080872893, 4.4992587765371121]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[3.6153335357082494, 0.03949413290721162, 5.4094043406479049, 10.276608961947034, 13.859656064628556], [11.950063722797461, 2.6484744629574553, 12.613963509038676, 0.0967825496451548, 26.113648164230376], [19.615715257521657, 27.449743266214924, 8.1813829876069661, 2.0595464036848199, 6.0312408134874174], [6.2996462118604466, 5.0362816140485993, 1.0260210385357602, 9.5476431306008624, 0.031463923250803084]], [[3.3011286499982817, 317.73694405844606, 0.38432975372952694, 18.565495769010809, 0.00018911227767102125], [137.95149147979728, 0.057683311526270743, 0.0047215818411767366, 44.823177617939329, 5.114540568421746], [11.626391652273755, 314.59789840823248, 997.27571906116316, 1342.8668304043986, 1.1787991238504885], [1229.1978961263053, 531.66373822836761, 161.64002854222983, 37.304070652761972, 45.312723651272528]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_constData_rank3(self):
      arg0=numpy.array([0.14690778598746662, 0.08937990074352356])
      arg1=Data(numpy.array([[[1.4356508647276427, 3.6705383672314276, 1.4338140007203501, 2.9116966541155014, 3.0888502453686275], [3.6263368423366344, 0.33714410643026849, 3.5222373486862235, 1.9335411335466854, 2.711526805844251], [2.4080415266830846, 3.4470415852827725, 4.4398408949041279, 0.15693943926523665, 4.4201090446796467], [3.3676865396880178, 2.7786046486004712, 4.2840213050689604, 1.0214530725299806, 3.1527931365167694]], [[2.5950336745284779, 1.3646554423488861, 0.37919873304003399, 1.9838751333833007, 4.4455765596423387], [1.6177554319903018, 3.2370292906030746, 1.7130822604473011, 3.2617288227013499, 0.62232896714562536], [0.65938313197091503, 1.6714766098444982, 0.24327154862987641, 4.6536767024457024, 2.0305284315027343], [2.96013112022891, 3.922121803633154, 2.1963390695298011, 1.0250383383240833, 3.016512668041845]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[0.063704068017549992, 0.00087620561162373323, 0.06392889412091983, 0.0037556678016244177, 0.0026737830647727899], [0.00095372665302357023, 0.52381034104193058, 0.0011644856278417882, 0.024515846009332449, 0.005513402313063998], [0.0098675496746535099, 0.001345144170744914, 0.00020036022003841225, 0.74007627173641799, 0.00020808810103552683], [0.0015662750530248365, 0.0048478240300503187, 0.00027014829783401532, 0.1409857976029284, 0.0023651822190904172]], [[0.0018985941617234282, 0.037051327961728865, 0.40023170287779797, 0.0083059794699476466, 2.175988223757003e-05], [0.020107709379747429, 0.00040284013610608665, 0.015973041590503841, 0.00037951491593791049, 0.22249804035299259], [0.20345354408437294, 0.017661271632769055, 0.55573339943493671, 1.3164652732819399e-05, 0.0074210051713575528], [0.00078619901564706265, 7.7025587526180568e-05, 0.0049724124778739998, 0.0841357667731122, 0.00068612268142506433]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_constData_rank1(self):
      arg0=numpy.array([[[[2.6293407838510308, 1.8869509453682936], [0.71784689445505834, 2.0169771404679451]], [[2.4173461182879303, 3.7311559652861863], [2.4320829068949728, 1.5416079627578103]], [[3.6797136799610524, 0.50570483974133584], [0.53481735203077274, 0.69016539302124058]], [[2.7406419627137195, 3.9398414102076589], [0.061401364579965825, 1.09814273589411]], [[1.846914744807078, 4.704058760890379], [2.1951663738029046, 1.3114009021743191]], [[4.0584681692836702, 4.8533603523590259], [2.8889959785521229, 4.6511202716780282]]], [[[3.6329282855902121, 2.3228159300343716], [2.8283599489901374, 1.6609435617406927]], [[0.52142153766116106, 1.5451351847924411], [4.5309754888004576, 4.2943383934704631]], [[0.43748767987399079, 1.2878422452187284], [4.5055124141418652, 2.8256466357577272]], [[0.037391108296888596, 4.596048163364836], [1.0073305715693948, 1.4335903855637979]], [[0.27062879285024, 4.8183774746017063], [4.2693298196214462, 0.71601345936317251]], [[1.0983636803545425, 3.6614079207905803], [0.25818090839222457, 1.9632547889768466]]]])
      arg1=Data(numpy.array([0.064193500998962413, 3.0994724162866394]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.0640240416673239, 1.0416025609208861], [0.97894474375230456, 1.0460677724038556]], [[1.0582977238350286, 1.0881997707547244], [1.0587107021293241, 1.0281742087899679]], [[1.0872303903560143, 0.95717670119745191], [0.96062206692873453, 0.97647660060974062]], [[1.066859606734305, 1.0920081134703881], [0.83600505740534314, 1.0060279119855233]], [[1.040169597850207, 1.1045067628556804], [1.0517680902272999, 1.0175549060242322]], [[1.0940896153474673, 1.1067243678615148], [1.0704760593886362, 1.1037046125355601]]], [[[54.513061730069374, 13.62864764760436], [25.091075768497298, 4.8192988532945353]], [[0.13287246782111772, 3.8520897648139871], [108.10558059879838, 91.547083273268271]], [[0.077122923728177459, 2.1903640686121704], [106.23364581737182, 25.016545120945636]], [[3.7699434205199566e-05, 112.99070209077976], [1.0228962153264056, 3.0537622334926544]], [[0.017404392600070522, 130.80738217806467], [89.904726343809045, 0.35508491129369285]], [[1.3374932848537511, 55.848535634456937], [0.01504094229987092, 8.0923208441911569]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_constData_rank4(self):
      arg0=numpy.array([2.1215924978785567, 3.8765582541808823])
      arg1=Data(numpy.array([[[[2.3371167203909566, 1.9576055960795882], [0.98460242131893372, 2.2370258849301061]], [[4.5125582581933879, 0.029187548715045551], [4.6282769719839765, 3.9874403879873861]], [[1.0763396867950579, 3.9773198106763386], [4.2173904609276116, 1.9070618131891761]], [[3.3578328945553024, 3.8540068555589935], [3.5365030769268695, 3.7136006256040464]], [[1.1820585553378922, 3.3226917700444676], [1.0431725574595814, 4.2547284953361348]], [[4.14518308940045, 1.6871573384498169], [0.46077131753567668, 4.3328995696712376]]], [[[3.3331339734729712, 3.1827038148290936], [0.35466568793411746, 0.40216291133128945]], [[2.6268838467429196, 0.59498626341329408], [0.47640348890014239, 3.853584126302505]], [[2.2134924331340917, 0.081504057870064386], [0.93742252795003667, 1.391653546519305]], [[1.0794234565349137, 1.8656009316275233], [0.13092815477462239, 3.850239882782978]], [[1.9968278788177301, 0.59672601976569384], [4.4066753405484329, 4.3011291118837587]], [[1.0981136044777244, 2.1351542198443938], [0.16109252921826278, 1.056030799747774]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[5.8002558985483761, 4.3598876945382266], [2.097162906915222, 5.3796161018284012]], [[29.790732029288023, 1.0221966708662387], [32.499903999276768, 20.069896668540114]], [[2.2469801900403348, 19.917697340534062], [23.859485837363671, 4.1972474194369855]], [[12.499030036462145, 18.153379041459978], [14.296873208184774, 16.333983026999213]], [[2.4329522060922955, 12.172983762645837], [2.1916177227289868, 24.539064179367383]], [[22.59819117022349, 3.5573790377767409], [1.4142183457825299, 26.025166322305889]]], [[[91.489320665004456, 74.619155630992822], [1.6169690862081694, 1.7244526858849887]], [[35.138249203497956, 2.2393267806307509], [1.9069437265870663, 185.1941983386584]], [[20.068802763508174, 1.1167623462827996], [3.5614170360571049, 6.5904246756244627]], [[4.3170070204332571, 12.525814990680514], [1.1941096008438457, 184.35693085530335]], [[14.963252557043248, 2.2446117249735025], [391.82474885292743, 339.61242791894421]], [[4.4277276914605928, 18.047773495569054], [1.2439253119610616, 4.1823216709084283]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_constData_rank2(self):
      arg0=numpy.array([[2.8658806984687888, 1.6844280099114612, 2.5533523653685664, 3.9318685453986002, 1.9946570012127918], [0.13308894845644795, 4.2822472517311585, 3.5742647567971111, 0.93317090298590344, 0.93116841356645097], [0.6348652801140845, 1.4874131935311599, 2.8406007869888588, 2.0247106914858248, 1.15828844526908], [3.517932680116346, 0.11829099052055744, 1.199310580902426, 4.3232367284870961, 4.1028453429809728]])
      arg1=Data(numpy.array([[1.9920996883773883, 3.2833312849388925, 4.8928421951957777, 2.5196206268598544, 2.0231936370086507], [4.7090258137349412, 4.2360588834112525, 3.5297094967677145, 3.8248808115766471, 3.6697231363409388], [0.98123317278886091, 3.4127770598339415, 2.8813406391488057, 4.4508048240631863, 4.8450347354960783], [4.9597430356580201, 1.2549719256962855, 2.2337005300771571, 4.9953807870488278, 0.36643637305845161]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[8.1452371523888214, 5.5401105210590336, 98.158670098032559, 31.489363254762221, 4.0428859139223468], [7.5086530782607081e-05, 474.0222012621943, 89.657869592567366, 0.76754726232281201, 0.76973671143028299], [0.64030164493174924, 3.8767760092574499, 20.250196612295408, 23.097331749065379, 2.0379533774714926], [512.20727772092926, 0.068640489204950542, 1.5007546667752765, 1500.0583317614467, 1.6774774759596718]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_constData_rank2(self):
      arg0=numpy.array([[[3.1761649693633531, 3.1113741950801255], [4.2261462931698794, 2.8608044180293133], [3.1062117614788201, 0.22433789191812928], [0.82024843821860349, 3.8383007451081679], [2.7129582067939442, 3.9289635478259375]], [[0.50438532566847405, 2.2549402597756063], [3.2389864047902757, 3.9647034987593486], [2.2860701329930082, 1.8022571945289347], [1.0717829042370373, 0.63658556345315653], [2.5466827682144584, 3.6439190693304897]], [[0.525697878971222, 0.52040082036041901], [4.6229726884369686, 4.3706429315716839], [0.29424921335381504, 0.96670937848272098], [0.053237332975485066, 1.7223663569056336], [2.046651810850634, 2.4764057623576665]], [[2.7370178259314835, 4.3582212255470312], [1.4967283246415726, 0.11845589926624719], [1.1516257968886059, 3.5689521257975638], [4.6678271070832924, 3.0131284035131878], [1.837822068355061, 4.6325767671915798]]])
      arg1=Data(numpy.array([[0.12954829133110202, 2.2125471411129669, 3.222696458714108, 4.6903297177958496, 3.4085384999599193], [2.5441221987524485, 3.5619833542900659, 2.4338062290705675, 1.8386727916261982, 2.5762053642909795], [0.12498086330650017, 1.7717798790700416, 1.801319003727138, 3.0449105138910153, 1.7269873876306607], [2.4878873541389384, 0.67964414313034316, 1.6301865339792843, 0.86854645347657333, 2.7889054256836867]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[1.1615039277089947, 1.1584068610332583], [24.262289073726333, 10.232967401882213], [38.575475960741045, 0.008093869893297672], [0.39479860481584678, 549.29356656822222], [30.01978004195556, 106.0765633457502]], [[0.17530380102342563, 7.9144127536027895], [65.77620418016933, 135.15017296100729], [7.4809091900236666, 4.1938070282046747], [1.135943159862874, 0.43586982899646515], [11.114107307319163, 27.971538622770407]], [[0.92277829820824653, 0.92161105354606732], [15.069339927969748, 13.642856863811621], [0.11040450435059418, 0.94083460459358736], [0.0001322642856450934, 5.2357767524626651], [3.444839584282521, 4.7876819461196618]], [[12.243271374368675, 38.95197222854312], [1.3153305247399245, 0.23461092573890288], [1.2587775642854706, 7.9569099854878349], [3.8120382812543783, 2.6064480413928468], [5.4590536154565017, 71.931038573810739]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_constData_rank3(self):
      arg0=numpy.array([[1.7620795768028656, 1.495282417866391, 0.41452354592718071, 3.5252701066630601, 0.65107007730307676], [2.4325946823596936, 3.6352789950085231, 3.8650902534790008, 1.7205421473181264, 4.4925096135451366], [4.5327823678782018, 4.9784449032735187, 3.8818314273950669, 1.3149467470797787, 2.5874623727973156], [4.7376694783676774, 4.1576787588252619, 1.6625890645886052, 4.5727676031230331, 4.8061865131201804]])
      arg1=Data(numpy.array([[[0.97031136938926599, 4.5907749129771549], [3.9703933455102915, 3.2388741272422426], [1.6242562277346313, 2.4686406828068814], [4.3381990327308024, 2.1030156294894025], [2.7690409561040297, 0.091263041383039117]], [[3.0121155768696757, 3.5491821735752338], [2.8765366902953033, 3.8344108658328868], [0.48364956259856529, 1.8430311402958957], [4.4027011431858165, 2.5402196584876084], [3.0679515365480743, 3.8613044739153133]], [[2.160570553693633, 0.41585712452069529], [3.8230563590078224, 4.926600343769663], [0.8483759628842531, 3.0169449872471579], [2.1083452837584744, 2.1637683085016044], [3.5747073918730785, 0.71480918916239189]], [[3.685352049585338, 1.6433332828029052], [2.0231714041911721, 2.2731517750868027], [0.35010212149062014, 3.0581193305649856], [3.6593101772828756, 3.3348535703861408], [1.9843575044015747, 2.2526197102037733]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[1.7326919467031943, 13.472487025838021], [4.939920238482352, 3.680497596787629], [0.23922243886628269, 0.11372773954397891], [236.49912214811241, 14.149925913229417], [0.30473866999362148, 0.96159257304128398]], [[14.550794823388943, 23.45480816574641], [40.964475226760939, 141.03666438638649], [1.9230009679693174, 12.08240808580978], [10.903450474389272, 3.9686396589575135], [100.41640670611625, 330.7187008823866]], [[26.189222530113938, 1.8747959495421727], [462.41138919588406, 2718.3294318245216], [3.16027265684628, 59.85373192810583], [1.7811457911875919, 1.8083800995753998], [29.91602539623014, 1.9729957959675557]], [[308.81185876081872, 12.887778200941725], [17.866583216395398, 25.511858161466805], [1.1948056531390077, 4.7335463145317842], [260.49549089395521, 159.07455962708627], [22.539079614596819, 34.342943546747257]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[0.29900744250377398, 1.9115364621619251, 3.3601990025465311, 1.0699950798252094, 1.3913785442019935], [3.6909596408449139, 2.9359088906149871, 2.3583487555352591, 0.31729576423981448, 2.8372397615706428], [2.1048048576332676, 3.7581009168648225, 1.0028388647998661, 3.8051473829562981, 2.5948990536745411], [2.216810548126257, 1.1169488249030195, 1.0371892477692801, 1.959538859879606, 1.2222589220495279]], [[0.56450618876765668, 1.5020360130694586, 2.0505908557474557, 4.351067720744922, 0.65576846275380729], [1.1892632121176023, 0.57932068806499315, 3.3053661506925192, 0.86244124189361893, 0.78995432129266796], [2.7727950643144261, 3.9442274839294504, 2.5393032317496531, 0.14166875858556324, 1.3184557464859246], [4.2640037505296391, 3.6526910937641324, 2.5771861806627068, 0.22550534756654542, 4.1316806025324979]], [[4.3648996190500169, 2.2343535194378039, 1.4230720041569309, 3.7167437125754561, 3.6357522931495638], [3.486459887227606, 0.61854323831653979, 1.1027399172732724, 2.124587190042369, 4.2787229555363533], [2.8445946998540621, 2.8908855858758602, 1.3946026281917112, 0.97453627406936327, 3.083471878805172], [3.9388218619402702, 2.1437914590989227, 2.4307292335713608, 3.6788281827662219, 4.3498733717252156]], [[2.2531560813810758, 0.026061210096764695, 1.8517004215279269, 1.2533726614436276, 1.1714357052048472], [3.3018222950100764, 1.2871685450572699, 1.8501519727724507, 1.813743743080853, 0.77730017536016272], [1.5737213098440541, 0.81094538821840689, 1.9475337919474405, 4.4525575350348454, 3.0260793471539951], [3.2522946142294886, 4.0196821576731212, 1.2909094907018492, 4.5719384959317182, 0.62183374077901599]], [[4.9904375707590827, 1.7736400490138162, 1.905926187330315, 0.88196612506335825, 0.16384845740317408], [3.4793459041851542, 3.9298715455650091, 0.011672799260725883, 0.35762771251972275, 2.7483544873091716], [4.8902406552604329, 3.5492567709378107, 4.2256114495112653, 1.8970172749600822, 2.8921019101425962], [1.1830733582449617, 2.15447865833272, 1.8641729802111719, 3.5931790633984368, 3.3348352489238491]]], [[[2.0903423124994682, 0.89253872853896099, 0.43260477076029391, 4.0406017621463537, 4.0665872370859821], [4.192517357012977, 3.1154539937234027, 1.5434236463469602, 3.5246113776608654, 1.9488880796925818], [0.52689389471359049, 4.285056733641361, 2.3954584143254962, 1.5973589917862687, 2.7210641653714034], [3.890931165789246, 1.8323550681789602, 3.6137777961060382, 4.7869970582099706, 0.97333790633933592]], [[4.7340821127927102, 2.7381225411670624, 0.32766150658083237, 0.5944527053112717, 4.6972811191268455], [3.476075925639968, 2.3957638832336463, 2.3952923378248516, 3.4909795822581384, 4.2631571841625657], [2.0811918850483266, 2.66009236623111, 2.4885296968425421, 0.045761009717298289, 3.6085884769505303], [2.9323242986996547, 4.046503797509212, 0.45082545085213366, 2.211033200024342, 0.79388517536710856]], [[4.6086603484288524, 4.64268635216456, 2.1062798475284836, 3.9825569805768124, 2.8472304323976778], [3.6215428011781454, 4.5202682828513616, 4.6951957821899555, 0.42033761140220627, 4.1297484260942952], [2.7374637346624979, 3.1294342231355352, 4.7932265965472487, 2.6559353897473641, 0.18904418114308622], [4.6719253295513861, 4.5042401616926595, 3.011462789048446, 0.12934183321922066, 0.4946146303031298]], [[2.228289987852266, 0.17217397925708963, 0.018377198017532001, 4.6087869652041755, 2.5780379780201197], [1.7903851944521525, 2.9777453086737014, 0.45633935305754891, 2.508863132809636, 3.9496735726385279], [4.0953700896076111, 2.4425203963484985, 1.5733071312419826, 2.3203119312313194, 3.7838332380110784], [3.9130086863147433, 0.85490639158291637, 4.2714969525464817, 3.8291123475289113, 1.6010400163880285]], [[1.2629218040518702, 1.5616919922923445, 0.40323091777106046, 2.9492641626001981, 1.6364377766189604], [4.6219533698900133, 0.3885481596077669, 1.0156550643650226, 4.0410058534086533, 1.2328321740163972], [2.3786860563432217, 2.8062427075797514, 0.18608767135742113, 4.0035363003053863, 1.9923659956118827], [4.8205002653924929, 2.5324525361358869, 2.0449716128478865, 1.8118605669561982, 1.0438892394278865]]], [[[2.1886439565161124, 4.6917205847707404, 2.094404743735014, 1.4950577058366985, 3.0489185270948296], [1.6617517538180244, 0.48914779753292986, 2.6858330864612898, 2.5577176914897772, 0.37105756156965181], [0.23949560941530407, 0.85304755483496542, 0.97586861196983055, 2.3070281108710553, 4.4928543775067027], [1.7187871239395287, 0.76267014883255391, 2.7169460847954214, 0.51799582506990693, 1.9213862556417236]], [[2.0991289018527737, 1.1802748788489885, 0.78799810662577507, 1.9515728770325635, 2.6704519835731477], [4.9249148405730176, 2.4553940362054867, 4.994037219849444, 2.5176198098803488, 4.2839270695190637], [2.7089155891438894, 0.72964286716793902, 1.0439770141644247, 2.892916478807483, 0.72583748605526466], [2.8100736363086996, 2.1047680012052159, 4.4282053003301387, 0.15548469335280657, 4.8954763672571646]], [[0.72003175896463545, 0.1959627147888332, 4.1771114258272206, 3.7743907394076905, 3.5972304673221331], [1.5546974932930691, 2.9222090526903566, 0.78767411806172971, 1.570766345761929, 0.1127389253251221], [0.62591080180116487, 2.5109668266709386, 0.093838531154416968, 2.6701708943329137, 0.98539184501580046], [0.97772905109579977, 4.3487307691099684, 2.2445737240833172, 1.1451615342108004, 2.6441068990986301]], [[2.4385117305470643, 2.3080393636379917, 4.4938118229535053, 0.09461485937076089, 2.0100631850505835], [4.523943112941561, 0.59359613062791017, 4.6145340847520497, 4.6736353726475919, 4.593770669157613], [1.4324443031743499, 1.5096541694737122, 0.90883608227099311, 0.040505518388953402, 3.4029165102918739], [0.45558264424680001, 1.5834034691882606, 4.7790375779945204, 4.9851935464407644, 1.8444165467588467]], [[4.4238038044705741, 2.0825528094339933, 1.6663678550465781, 4.3782019536700618, 0.43561238355489934], [1.3729438165543444, 2.5812675721783993, 1.2667899382179715, 2.9539738190039238, 1.92038545715957], [3.701015332083363, 1.2471179975398186, 3.7383587763124244, 4.9349735316413321, 0.14405812977899768], [2.9964512954672977, 3.1105122873211419, 4.2120521737130945, 1.7265588460676298, 4.9800979415918949]]], [[[3.1143825224964132, 0.39682907740400297, 3.0043737597171849, 1.5463170308365057, 1.2381970220291443], [0.01129215818699632, 3.4021925320447499, 2.8612130241484608, 1.3160494683542729, 4.9141891851943509], [1.3181747902552201, 1.6250223464348328, 2.6821101932285538, 0.66008658192040759, 2.496940162619127], [0.5001395592654243, 1.3121438178615685, 1.6170713417489222, 4.5754608070817069, 1.9645698972710832]], [[3.9076466621870272, 4.5139704807458321, 2.0673512000315846, 4.9514969459084304, 4.3998915335144275], [0.61494689976151062, 0.53122434627605375, 4.0248982053483449, 3.9221780275812517, 2.3427937836151771], [3.0524350434293392, 4.2342752083395929, 0.3628080442278227, 2.3214198165462925, 3.4275377151385138], [0.48535032826036567, 0.4338545448151459, 2.6770023191550019, 4.4705157128924977, 3.9429447597157239]], [[2.7014124016055514, 4.7853004316025247, 4.6412897711727039, 1.8941632287329757, 0.52984645060922086], [1.5804156502651019, 0.36469296455698419, 2.1895009228010411, 2.9370239393215409, 2.2633381700628057], [4.4502825616614379, 0.85980498718786891, 2.705328270468117, 4.4764590180283985, 0.54545389535474176], [2.2698814628184065, 2.6185727492147532, 1.2871231175597715, 0.65498172224210038, 3.2009238569194922]], [[4.9311508411769065, 1.3471363823264177, 3.2483178465929381, 4.8320961606339301, 3.5508296272684765], [1.8773849088226784, 0.39217657878606843, 0.8331314779032033, 2.1832818594514212, 3.8628539282889314], [3.6367579780180019, 0.75976846670810427, 3.2115769355207271, 3.2458554357739429, 4.5112835672533587], [0.98733907773260754, 3.6212163668243864, 3.4769114347291059, 4.1028604671959821, 0.79408005684353544]], [[4.2410441980303011, 4.1060342173891895, 2.6550757927092121, 0.55143106498083494, 4.4708034417994202], [1.0357638935136255, 0.9651039595169657, 2.8625447957639634, 2.1761982016221029, 3.896355745716725], [4.4897952138825747, 2.3245774358609537, 1.3501292670171141, 3.3738684505144061, 4.3510556967608602], [3.6120124738316073, 3.9950118254164768, 1.6400986342936312, 0.21768884721107948, 4.3040638287181689]]]])
      arg1=Data(numpy.array([[1.3196522386389473, 2.5247237743712945, 0.59686530569352569, 4.461970667305855, 2.3209231146231275], [3.0693049500477887, 2.8569031538213987, 1.0347491296607296, 3.078707161659008, 3.3987913250969752], [1.2744470851148644, 4.674078586763164, 3.2273302441942171, 4.9954983623000002, 3.6147884463244937], [0.75155415734024067, 1.8068698851389573, 2.7581333511925052, 2.645936732893214, 1.3756659854300066]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.20327434735471098, 2.3514034994907438, 4.950154439857311, 1.0933865635959037, 1.5463148047506075], [5.6030776620870189, 4.1424524788740804, 3.1025087876539201, 0.21983974609971813, 3.959727226655589], [2.6700981431355926, 5.7379714956382388, 1.003748012990795, 5.8329536636742967, 3.5196137219301207], [2.8591799756601923, 1.1571434986166658, 1.049366155334494, 2.4296377256146986, 1.3032421969522492]], [[0.23606526115825538, 2.7929879936039224, 6.1292645599432012, 40.952281938003757, 0.34462414225253724], [1.549016765156918, 0.25202060995955861, 20.459075539780397, 0.6882318733308127, 0.5514082788658845], [13.129404537937665, 31.962395151600948, 10.514593446685035, 0.0071978288000948635, 2.0097052513621052], [38.914854656050551, 26.329479865718042, 10.915146288098359, 0.023275536940155096, 35.937662981319789]], [[2.4097857542234484, 1.6158336514769507, 1.2343996944381441, 2.1893252391141376, 2.1607239227408011], [2.1073205077279868, 0.75071661749592744, 1.0601094774983872, 1.5679739007122544, 2.3812748400376984], [1.8663325167403932, 1.884401109516874, 1.2196001791706814, 0.98472263369259416, 1.9583531204352738], [2.2664884611419551, 1.5764179298639931, 1.6991541643046246, 2.1759673599854388, 2.4048308783631205]], [[37.509763960584969, 8.5548868037498961e-08, 15.627615894416682, 2.7392461135763329, 2.025908096185578], [206.37817791106104, 3.0845372565929305, 15.569389903936314, 14.248140947516358, 0.32494531651983988], [7.562862101319805, 0.39257553026390879, 19.573729721279275, 783.5680912356986, 139.85366259161034], [192.91954121297672, 496.46164944274994, 3.1247392218883543, 881.75621694476172, 0.12005512487731276]], [[41.718249459225113, 3.7809214982654762, 4.4679126033061589, 0.74713325127917263, 0.015023845485069517], [18.062352449329524, 23.961084067039756, 3.2663432412250729e-05, 0.09194922396400583, 10.448503436327622], [39.799944708058817, 18.915884482962824, 28.355776749916522, 4.4195909401151017, 11.760919764819439], [1.477251647819761, 5.9382928166703053, 4.2440226638777423, 19.463624928624075, 16.368746649703098]]], [[[9.6126888829766823, 0.70543892733175062, 0.076392942697735958, 72.672125047187151, 74.116165268153892], [81.388903490255942, 32.71654988913788, 3.7889458875980617, 47.780559785654141, 7.7525462242586629], [0.13992120506344496, 87.029692264424639, 14.603582817117937, 4.2102161678324093, 21.594636529738246], [64.722349609672108, 6.4158879521576662, 51.588609895768464, 122.27036964266269, 0.92040193590708241]], [[84.934326640867795, 17.772990715832858, 0.041268374689615681, 0.22629503442411553, 83.061646184061559], [35.143061776123126, 12.134860801919034, 12.128038507496097, 35.575243286075796, 62.962427984901986], [8.1168386035897662, 16.363973611608376, 13.526028973066971, 0.00014899292667191363, 39.107392104069234], [21.616204181248484, 54.246113754500684, 0.10269175597581168, 9.6488442470424101, 0.5171510785097575]], [[4.8599672849866282, 4.8971003017172086, 2.1615136103097745, 4.178468238029911, 2.9526597737504194], [3.7871690309822736, 4.7635485682264997, 4.9544230565755374, 0.40786704726077871, 4.3383681716135314], [2.8349529124658126, 3.2559882872598109, 5.0614994000152276, 2.7476328178636527, 0.17841218784409396], [4.9290167331181856, 4.7460719545796017, 3.1290651285750717, 0.12046828171052852, 0.4826618811365404]], [[11.78427835249259, 0.0044439352516221324, 4.5313148016693177e-06, 110.40504580266013, 18.460325172947417], [6.0082519039829023, 28.771399755774777, 0.089340275330071259, 16.97745576193962, 68.64947992965736], [76.748751360550713, 15.632924873331341, 4.0358142212425232, 13.347816794942286, 60.156609355447884], [66.706365721606176, 0.61715918309198947, 87.371838130939395, 62.400536523877008, 4.2588709095224582]], [[2.2108406298803898, 4.5497790828946671, 0.045641484317502042, 39.487420008733508, 5.3333459227717679], [181.80396257392914, 0.040235438508197802, 1.0542146727081678, 115.16782942894051, 2.0368713069835147], [19.014782486990544, 33.349101958584143, 0.0032955091242641002, 111.57853668718897, 10.411017368943718], [209.74303185141193, 23.526250605087949, 11.375257718316956, 7.5389739016212634, 1.1571841692845508]]], [[[2.713533494577673, 7.1709598142641484, 2.5655162050288589, 1.6695226777634755, 4.1401705160478439], [1.9102925483355251, 0.40198314506171146, 3.5223986479005012, 3.3096842765423888, 0.28266738571502115], [0.16178847432373616, 0.81663699455275918, 0.96934826270363961, 2.9019618099648294, 6.7858647172913535], [1.9942431498963546, 0.70801818405221317, 3.5744834539037993, 0.43243808360035374, 2.2985391318061428]], [[32.006346309032836, 2.1699761657684431, 0.32836021506776797, 22.765712786562627, 98.603142267418164], [1723.1617877659282, 66.597378101747054, 1839.1556442205795, 74.861661262461567, 898.00650686775009], [105.41928110691433, 0.22917486137423757, 1.2228210517791573, 143.32443213276468, 0.22364149662200544], [125.12433874131605, 32.410220259374739, 1048.38132552564, 0.0001666803921564912, 1675.5440278186695]], [[0.34643891847911373, 0.0051952919648390241, 100.8723381353563, 72.723590227684866, 62.272053914927596], [4.1543645516897616, 31.842276183348883, 0.46288826170819097, 4.2945420400353953, 0.00087242760311709061], [0.22043394062156357, 19.517261089712388, 0.00048254161244297731, 23.800298343446261, 0.95361706797676404], [0.92989075963549561, 114.8700133457277, 13.590172622823612, 1.5487540254479411, 23.058646867254662]], [[85.87792646600667, 65.250052867176436, 1820.2732075591623, 7.6631371965939761e-06, 32.710230671213047], [1882.0656798623791, 0.073871168329648379, 2078.0195448919221, 2214.4179322956365, 2031.7286564261742], [6.0212424864344287, 7.8267617242187066, 0.62031843430524403, 1.1062147368424811e-07, 453.79767634282217], [0.019695765355853097, 9.9325205351426327, 2475.3925673592403, 3056.8167820235049, 21.286205361467836]], [[215.98092925523872, 14.179379402725447, 6.3336609638666124, 208.04084731943252, 0.049593371672366347], [3.1447463821348491, 30.809827190508493, 2.3510151100280741, 50.167272978551956, 10.577648830511414], [113.33388234769058, 2.2217003924963556, 117.52238224364636, 320.68451181890146, 0.00090843124108629569], [52.824364775133063, 60.462087150181887, 180.88925227802781, 7.2004647718957093, 331.41140294320059]]], [[[2.3485280022294499, 0.49926231039055646, 2.2859039335476337, 1.3876121825395646, 1.1741849951519874], [0.034399840113927375, 2.5098374901307174, 2.2035463793059744, 1.2292480508926056, 3.3087467587124535], [1.2307396950760208, 1.4403638458658969, 2.0990528943774174, 0.73184702186323691, 1.9891784058236031], [0.59408795286962102, 1.2265053323235597, 1.4350640445334231, 3.1358302795423807, 1.6611406180092729]], [[11.735832613510356, 15.23008859185382, 3.7146040564106211, 18.001076932582894, 14.541724721065512], [0.41539096691316563, 0.31886954242245913, 12.379794012205114, 11.814806376232015, 4.6565186203702877], [7.5109183488236955, 13.567762957789014, 0.16010078629189231, 4.5800405509199935, 9.260689064757349], [0.27085872263120442, 0.22117094226934975, 5.9252358680104589, 14.966202198499241, 11.928077894991009]], [[15.501910522856214, 75.037829760823101, 68.972884158240674, 5.8230991790671407, 0.17344788006211381], [3.5337661930574575, 0.061906651088104385, 8.6839489366896867, 19.523210800181975, 9.5158214128160026], [61.424188398932763, 0.65927459624220675, 15.563967553496353, 62.42585010804428, 0.18790727376882879], [9.5918910869373608, 14.225843855194004, 2.0060734391719524, 0.31126878804565844, 24.752369300577108]], [[68.154938156507129, 2.1999601516906653, 22.584790585240476, 64.592103881311871, 28.584982146154246], [5.2942541343449623, 0.084019128911473118, 0.61689756002615181, 7.8933814909782667, 35.720883534496174], [30.451930379795161, 0.48338166732513482, 21.915159880198534, 22.539518912870452, 53.85632798681695], [0.96684810262266974, 30.108809414359349, 27.037348611278208, 41.897561344476983, 0.54330943108295893]], [[7.2978172832127335, 6.9801467982067891, 3.8316708790748399, 0.44093875969422691, 7.8471745882283184], [1.0495272492786305, 0.95231167903253233, 4.2495074150269359, 2.9144745478134038, 6.4945482526430069], [7.8930682076475041, 3.1912953865049145, 1.5113078446138721, 5.3275547949682851, 7.5594975102147144], [5.8516273333421971, 6.7218365431515794, 1.9751009816899012, 0.12276759704602085, 7.4474118408560468]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_constData_rank4(self):
      arg0=numpy.array([[3.2333899338922323, 1.6476287738891151, 2.6922690317476303, 3.8562235104048552, 3.4145406840646726], [1.377385187479931, 3.5232012437214908, 4.5798830297588662, 4.432547636764725, 4.6027572968536257], [2.4577258608032007, 1.6649266380053638, 0.37600332819012872, 4.6920979539780756, 3.6917869820089431], [4.8685086870569876, 3.9468674042175196, 2.4063025814835899, 0.19385030191233321, 1.0965928119421409]])
      arg1=Data(numpy.array([[[[2.5249585695288932, 1.4083168115039644, 3.5563533334597612, 3.7282079489210691, 0.67786047117036496], [1.6197163923909896, 1.546482679911227, 4.3911869299446007, 4.5697004273707647, 1.8833325660767066], [3.5573147260550879, 2.7999811982889886, 4.912934993488685, 2.1148383924457983, 4.0052962478167649], [0.61238748510527918, 1.3029880597536965, 3.1872192335712088, 4.5749608008043596, 2.7199616899900767]], [[3.7740885950634446, 2.5185389750438212, 1.5489403022102139, 2.747665429648499, 4.6579057552943564], [3.3919966435329383, 4.7040062385023589, 2.9950111353914917, 2.6806219919690775, 2.5927941722415673], [0.72640925991886995, 0.85021437575023306, 2.6605413708064853, 1.6667144147835979, 3.6135481678941708], [2.4471654613178884, 1.3985174201245376, 0.023483352983229924, 4.0458759917013492, 0.010864727575269632]], [[2.050245808745311, 0.99008604351498342, 1.3017946471286321, 3.7932463859498009, 0.011496134653381441], [3.3321083705564005, 2.320872032804814, 4.0376277696038319, 3.1592776047016553, 0.093391679782407436], [4.6892736957791366, 2.975503331015767, 2.6901467459070099, 4.5350335501681176, 4.3905753594191621], [4.2888293946942211, 1.8072023946733458, 1.4775879562603189, 2.1984324443097893, 0.79472808619208135]], [[1.9690448583931444, 1.2559161554595391, 1.2244043846563719, 1.4085861525983618, 2.5395563423143668], [3.4723529290207069, 0.34891208770500132, 2.4809847372323697, 2.124007553496488, 0.5382741938553145], [0.56107228508220319, 2.8055865567113827, 1.1941186216762565, 1.2345917988582331, 0.22576061808153258], [1.7648579635170283, 3.910617348776166, 1.0085026596369797, 1.2059971367112059, 2.198309626871318]], [[0.46837157387512951, 4.1744598539964466, 4.0435732909348765, 0.073856410335365816, 4.6273081917848868], [3.3676217436430225, 3.178378360857157, 2.6555958208848662, 1.1331618169258357, 3.5732049610266432], [4.78389666275985, 4.3264049426486801, 2.030771143429694, 3.4617921136527396, 2.5417968879812007], [1.9067846843480953, 2.8241924093937896, 2.073714554714233, 0.098881372728088154, 2.5385878547972403]]], [[[4.9008693338909923, 4.5920637969485032, 4.7176953152474521, 4.1452009849504909, 4.5857393902291337], [0.12411945061675667, 0.68397650411907251, 2.0634057652290192, 1.6634660895953557, 2.5327316977550471], [1.3359027823893022, 4.1243970657016682, 4.9449105255662884, 4.9137783426868564, 4.722847338870209], [1.8373417752827266, 2.4382118663672765, 3.3761019728582471, 3.0634888350687541, 2.1702756383283202]], [[2.7074151759638734, 1.5379753043156217, 1.2118953149603202, 3.114733572866164, 3.6271213395258584], [2.5087025602023707, 2.6128206391429694, 4.1512941672485484, 0.50883430696936649, 2.1378682502290363], [0.48005092250520803, 2.3958263789999021, 2.7459563583246482, 0.063999381197708691, 1.8526243706012402], [0.14274974537797319, 3.503042923802747, 3.5506334727628999, 4.1363091211508642, 1.2550806443382718]], [[4.1181320271195441, 1.453333491407836, 2.2016307311360817, 3.074447958068331, 3.6579993274704385], [0.24746513203886511, 2.0758863560731813, 2.3426147576625094, 3.8606826070477522, 1.6088638692844763], [0.89287965750756604, 2.5423082819470482, 0.87088446381028894, 3.7162337893008575, 1.8644029575851817], [3.7873475587846368, 4.2161791539242515, 2.272913530975524, 0.9542334374852407, 0.068965315702559635]], [[4.5749005822217246, 0.20441954277956917, 4.8673122706142316, 0.95639294170179134, 0.70374197911932579], [0.33827872626799083, 1.8903269490080867, 3.9551246249021803, 1.0725248952160871, 1.4886461692810482], [4.7255447448467125, 0.26291274056797209, 0.97360267182623728, 3.9784976181318013, 0.88071435997670677], [0.91635025599768949, 3.8709764696562492, 0.54637262158500044, 0.23687985346407575, 1.2529169658229398]], [[0.72099831467769071, 4.0160443901526994, 2.4533032223796685, 1.4321719395278318, 1.6490096161990082], [0.55720909924097617, 0.86408494440544259, 1.4534778127278676, 1.2640297145508097, 4.4355252032143779], [3.6775876096870266, 0.69060949336657318, 1.9896152252799235, 0.51276265519791742, 3.9231573198316543], [4.8086257616535883, 0.36193973879214097, 0.93113505850811717, 0.75398736056361237, 0.70643091938326841]]], [[[1.8641940577904528, 1.6483091949018509, 0.30287484664657965, 4.4998646505512658, 3.1187479648041583], [0.93767397787227091, 4.64170447678387, 0.95576364783693946, 3.4244434198787874, 1.8684246323516744], [4.688106644861314, 2.9008438111933597, 0.24903973428007053, 4.1976488534781353, 3.1292618788014774], [3.7489618031459422, 2.8512772271609483, 1.1692148214911751, 0.44155544389065349, 4.0549039361480421]], [[1.6759544024937816, 2.9877865379350461, 0.19835717442089768, 4.3510176473854481, 3.0791534337968045], [3.9946682881233602, 4.0365154352129835, 0.7165064286867926, 4.604326024892039, 0.4730555013531681], [3.6303823256813672, 3.559235435278564, 3.4099075780194146, 2.9751688080500855, 3.9126414701793264], [4.5869967621521344, 3.0257712938755521, 1.3301322854232063, 2.5534934386193231, 1.8663361696768448]], [[0.085451681886634628, 2.4665787951852529, 3.8518573697658902, 3.1736052143868365, 2.6612660948151761], [0.96059522967444355, 4.3708518952471049, 0.23574946563810822, 2.2614997875300937, 2.9664567147357266], [4.7744480238232914, 2.7896965031312466, 1.6036360489603811, 3.9091823551908362, 3.7964352596896371], [2.1732560553895044, 1.8971203499007536, 0.37133703090686843, 2.4451574313349083, 4.9442861920761239]], [[4.5236785825123462, 1.0889535095531215, 3.2899249830935595, 3.927654740406318, 2.8410314680770559], [2.2859151445760304, 3.2695235264008722, 1.1040330759460004, 0.20507634970854019, 1.0317870750504616], [0.91497492448354045, 1.3060631979991248, 4.8886693023519294, 4.5020492510878212, 1.553586559562812], [3.2956002803413553, 3.0241558597631841, 3.185262620359933, 1.2103543311793543, 4.9518643855035043]], [[0.30776930831341726, 3.1584493163902625, 2.1583682948841751, 4.6233346197304561, 0.70011706036267074], [4.776736584945434, 1.5951574667702739, 2.8368743790184126, 3.4767337755726855, 1.306136139884138], [3.2894002274787844, 1.6023057147484474, 4.875806095466622, 3.8291758246004921, 4.5142673484910549], [3.116723121367742, 4.4010548073659104, 2.441938021910679, 2.3338555151857689, 1.5720699762664361]]], [[[3.123993751384516, 0.24208524916638688, 0.61099818500575886, 3.1096278216621003, 2.9883514278352221], [2.227691497793971, 0.93075512596216958, 3.1471509537142945, 2.0889878373156985, 2.9708417003777514], [1.8798786972737911, 2.6797306716069396, 0.30300675454848996, 0.38719123214672696, 3.8295121860311276], [4.5835656248865506, 4.7039454649221124, 3.9457238981610239, 0.46875164958816562, 1.5441097635415408]], [[1.3544215777761461, 4.9147228902978544, 1.2114055954885008, 2.1521188872619694, 1.0348225431948808], [0.15792727449263491, 3.8036497010315937, 3.1151713687953131, 3.5570058345850861, 3.6647909075843876], [1.82370940399126, 1.817308204956376, 3.2576987396675756, 0.95921226496498802, 3.9527986750822222], [3.7518005656524132, 0.32730177926647397, 2.2299940019262263, 3.9905022389188152, 3.8406560526592957]], [[4.9195146682279747, 2.3952169031459425, 2.8882651565506383, 1.3402999880755813, 1.0487421793910057], [0.88825575933757961, 4.3976930678936474, 0.37431898017728099, 3.8349703190950066, 1.5714983576023873], [3.3075748105595113, 0.85007397471871993, 3.2373096911060175, 1.0383451092322007, 3.3675174729001656], [3.7196794131248181, 2.6080710627381216, 0.37110612101754115, 4.3977473101090743, 4.2209127344293842]], [[3.2247717958903679, 3.8033558928526525, 1.8444447867061917, 1.2592112877406716, 4.8279083856967606], [1.1361723787845472, 0.772442525029578, 4.1724347140063252, 3.6619802725432447, 4.3537034990793044], [1.699809776286435, 4.876497492926493, 4.9943915042460993, 0.89494161178811027, 3.6907427527606242], [3.4803254089781279, 4.3339488729089846, 4.9143771992345604, 4.6771050915936039, 2.3099531256351078]], [[1.8364481481987311, 0.98625159108595895, 3.748206056193506, 3.4575574290078759, 2.4211837596687431], [0.5720408978355499, 4.5176629034758502, 0.081032365962317943, 0.53304598692488936, 3.917698707002442], [0.71530173497611105, 1.7850007230180203, 2.9974958014977986, 1.8575503227849, 4.0568335904784885], [0.35240385480542941, 0.074948064268606396, 3.5620107434930097, 1.9283638175024516, 0.15338794971944508]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[19.358225365518663, 5.2210763595775447, 64.941784569021223, 79.45318520303185, 2.2155271047875456], [6.6911620788221189, 6.1401275993985349, 172.98319257871759, 213.29700378225769, 9.117077851552537], [65.015094803650285, 26.732098149443477, 319.09281514714678, 11.963120540786431, 109.98453000664506], [2.0516734151998919, 4.6140085540143669, 42.110659336759731, 214.61780396503161, 24.336070671288699]], [[6.5833427539728158, 3.5169708593168716, 2.1672160816429664, 3.9432742971111749, 10.235533813116147], [5.4398415221295533, 10.473885429053874, 4.4616574262239395, 3.8134494200100821, 3.6498221710681227], [1.4372378729392006, 1.5288925062816254, 3.7754030266852419, 2.2984900829016004, 6.0761959539073809], [3.3938352396427112, 2.0103959633564443, 1.0117951308434718, 7.5402563267974863, 1.0054399049168325]], [[7.6181331856481274, 2.6659639965501509, 3.6301611382512879, 42.81000490241118, 1.0114506543213508], [27.114481657739177, 9.9597834200598978, 54.532853200984071, 22.848810953876985, 1.0969061844582313], [103.97840718692419, 19.04666299600931, 14.357597160801689, 89.248644146469374, 77.351277938303383], [69.936625909520629, 5.9884041562567454, 4.3205382058737607, 8.8223923507379176, 2.1969856341457841]], [[14.26197472053031, 5.4471655470178888, 5.2203493276575301, 6.693603073581345, 30.802922826242916], [108.48308460069596, 1.6014709909781717, 28.461610237589298, 17.579759779079669, 2.067835489269326], [2.1324524891625765, 44.109159563059457, 5.0112630019275199, 5.2926240089220133, 1.3562268563477866], [10.826623223718824, 195.99999533706858, 3.9007322033043659, 5.0922525290093219, 19.434170248976777]], [[1.7774511953386483, 168.41246807306112, 143.40629307729895, 1.094939210900306, 293.69251765775243], [62.52611168863141, 49.560059034414444, 26.080445457356948, 4.0211640263300863, 80.483265782562242], [355.96440324383616, 202.9604713454813, 12.108095224013105, 70.191661276686062, 22.678919488954591], [10.398000774446874, 32.079900297198023, 12.763769002047825, 1.129110975882806, 22.589721579420811]]], [[[4.8027765991090838, 4.3506215420514156, 4.5291953001487402, 3.7706186838280851, 4.3418204812122863], [1.0405416796486682, 1.2448316793088929, 1.9360997596077467, 1.7033894620468149, 2.2500389707841273], [1.533785072728745, 3.7455855222924326, 4.8709823040512967, 4.8226689768991324, 4.536672874230292], [1.8009112482412031, 2.1829638631176498, 2.9475775363052881, 2.6668260613309003, 2.0034963929141627]], [[30.25427527226659, 6.9370734369011897, 4.6007743565692136, 50.531761727865153, 96.340411826716931], [23.556099635651709, 26.856502812053431, 186.42246599974305, 1.8980189258864779, 14.766589789419061], [1.8304498203946289, 20.434648503685725, 31.758967151362214, 1.0839360447995039, 10.310260780894533], [1.1969477200062228, 82.403444560076579, 87.493223931099422, 182.93734239472383, 4.857922788915797]], [[526.60576997834335, 9.1293910998771928, 28.507381361138851, 107.58764924445985, 261.45978546990114], [1.4572646095449093, 23.542826592881159, 35.32866591944493, 355.91724808378297, 11.567106785066551], [3.8910197757200189, 47.873544063220322, 3.7629445426008203, 285.68629053520237, 17.064708430701618], [318.33562734716605, 611.33672650656661, 31.773486794640014, 4.2717846681118985, 1.110646957863223]], [[908.60488953319793, 1.3557780385569103, 1404.3135473763614, 4.1538877709005844, 2.8515247914505584], [1.6548136418163084, 16.687311416252278, 361.072848647954, 4.9380090620886969, 9.17567714611517], [1137.0763618794451, 1.4791540966013694, 4.2617060331826977, 373.86004861993376, 3.7112220990518243], [3.9134610825594018, 318.55185661910912, 2.2558681873847095, 1.4229156807838528, 6.4595613972359338]], [[3.0063179837405514, 459.94936262218681, 42.323773379896906, 8.903399947694048, 12.3971878418955], [2.3412064097137688, 3.7402784637079236, 9.1977597563584705, 6.887705162814, 872.63651654476007], [274.35132263731458, 2.8700309292473785, 20.852150798708873, 2.1876150810697159, 399.1390458463801], [1542.4311831155853, 1.7376911486656768, 4.1434239707361655, 3.1616023703067642, 2.9401972592267835]]], [[[5.346014308884226, 4.4027077573578968, 1.3130545094219341, 57.193624296302033, 16.518681094398261], [2.3237694801359674, 64.974180912680055, 2.3618791073509908, 21.74501273319683, 5.3663908158615463], [67.74268587287979, 13.57927271233654, 1.251002979186983, 43.583514044540067, 16.675597495470537], [29.113535364555403, 12.987306853258341, 2.8616563920407319, 1.487449281112049, 38.333244574434261]], [[2.3498930437729384, 4.5864990184474097, 1.1064079987534996, 9.1895447132432917, 4.8051780894176295], [7.6630205911814286, 7.8282510682914239, 1.4408906186453803, 10.456232447568052, 1.2727179547064837], [6.3642739384542129, 6.1375817775109685, 5.6877020204711011, 4.5570920520836458, 7.3491930698101955], [10.364267544389406, 4.6761770252152433, 1.9700841311475172, 3.675622826544076, 2.5893913112781957]], [[0.91981272901959088, 0.089572939849567546, 0.023104627810462779, 0.044856576985154668, 0.074040960734934491], [0.3907789512473705, 0.013906754537967686, 0.7940569800460987, 0.10947024628463635, 0.054931888979295858], [0.0093707596646947552, 0.065300084095995622, 0.20833492409752341, 0.021844738322987559, 0.024391738191366689], [0.11933910211037302, 0.15634626423035725, 0.69542923369007081, 0.091469604592095272, 0.0079364472517035444]], [[1089.0525486228778, 5.3837827559535247, 161.71387972336296, 433.40912943013478, 80.7932303025012], [34.252174898721577, 156.69330119558373, 5.5107594163946425, 1.3730346840654664, 4.9284217494125464], [4.1141870628448869, 7.5309163297751551, 1914.6596540319222, 1053.2406558484727, 11.041468976307605], [163.1388904403278, 107.23058094652659, 137.55628324138306, 6.4951927109560135, 2111.1480170819054]], [[1.4947825749048087, 61.885393832065191, 16.761217691874023, 419.29990870758684, 2.4953533096177609], [512.3184334305754, 8.0321489263461459, 40.661095378379436, 93.784434572972089, 5.5066600650816282], [73.429118245008993, 8.1074914691943327, 583.09032841759426, 148.61026410184468, 363.62841758773487], [58.602957321499154, 313.64582812433048, 24.274858337300611, 21.078919574634668, 7.7935563859579409]]], [[[140.41731042939924, 1.4669267211983532, 2.6302539252079296, 137.2604974031195, 113.2871539748068], [33.986474133406624, 4.3631214636314013, 145.6595008356901, 27.287376141783785, 110.19060073846416], [19.598417343535569, 69.508194675315565, 1.6154205960284764, 1.8456684336445357, 428.93468331170459], [1414.8957555002273, 1711.8789755566129, 515.55455057238566, 2.0999936675651205, 11.519000488305736]], [[6.4206249362770311, 851.95369180477917, 5.2759862976020147, 19.195805694768183, 4.1401451380945558], [1.2421228124840362, 185.32575665054802, 72.015972551622127, 132.09115905327693, 153.15841882756118], [12.229011625121153, 12.122009685749436, 87.581249416849076, 3.731923614442469, 227.43968019891636], [172.59197697841296, 1.5673085979921064, 21.361885350753251, 239.52292227396919, 194.98486986769754]], [[75.172342619531804, 8.192501576240673, 12.631083376645334, 3.2443202558320272, 2.5115283142393361], [2.1814062058274764, 47.540231647660754, 1.3891419667641753, 29.004597291133656, 3.9745802582895062], [18.253428029593728, 2.1094824262742207, 17.161246041467205, 2.4887034707538978, 19.239934371268866], [26.212028147888756, 9.8761691111384202, 1.3852284647573192, 47.54249602481606, 40.704841830079921]], [[0.0050378125814024973, 0.0019497612930250981, 0.048503297694183017, 0.12669799753672725, 0.00036303914195417856], [0.15503833566701233, 0.28158454206330652, 0.0010641440314220023, 0.0024587609813977234, 0.00079038798240733846], [0.06149340759814733, 0.00033522166942266137, 0.00027626676985337652, 0.23031599753982823, 0.0023454278696425301], [0.0033124664757846517, 0.00081642474940775889, 0.00031502246404892118, 0.00046494957363797201, 0.022598570717233264]], [[1.1845169811641336, 1.0952035285223685, 1.4128575974180313, 1.3754957482227592, 1.2501359938400531], [1.0541625988233498, 1.5167415864539187, 1.0074998104060382, 1.0503790159141069, 1.4351119852105756], [1.0681802414158386, 1.1789111050656957, 1.3183657222992344, 1.1868240436757773, 1.4536421346793951], [1.0330281388112386, 1.0069347405195335, 1.3888077345171717, 1.1945988446578006, 1.014244078916358]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[1.0313210599206903, 2.0342066501354577], [0.79730351683943668, 2.0882094015954107]], [[4.0969031708241239, 0.30413980435144311], [4.2323093727801533, 4.735795011017899]], [[3.8997863098228156, 3.319313671860761], [4.5375000326966592, 4.7635718287861799]], [[4.947797183653412, 4.0373204008140613], [3.5747195263236051, 1.6727900698520675]], [[4.9585464359064826, 1.64740545084967], [1.5963469407615993, 3.7266467570972623]], [[1.1979641753613735, 1.7008718418093949], [2.2900122277839152, 4.8934097174502424]]])
      arg1=Data(numpy.array([[[1.9152794918274498, 0.16221381077000488], [3.6489313069112441, 2.5485828752893807]], [[4.4866774774815044, 0.65222435149429392], [2.3328805351944464, 0.9210759165849296]], [[4.4406413641367743, 4.5870669087826714], [4.2089169128206461, 3.2773393641891357]], [[1.6891712599234756, 2.3736159334244613], [2.3168225256389752, 2.032013458524748]], [[1.723794929444789, 4.2626731329313516], [3.1041551440928212, 4.5670513165539361]], [[2.5289998565262719, 3.4217297292463944], [4.3673430067523116, 1.0344731940759058]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[1.0608476916230409, 1.1220854720839075], [0.43755390602610478, 6.5308642920494702]], [[559.61737956776346, 0.46009529665154902], [28.955444395757809, 4.1887855861390504]], [[421.30854768798349, 245.51802999340603], [581.41558592079946, 166.6539392696568]], [[14.892945681975192, 27.455699654117279], [19.132136213164351, 2.8446970131994953]], [[15.799682599335863, 8.3974931969268081], [4.2710888352199357, 406.66768943482464]], [[1.5790083875679615, 6.1559607476482974], [37.28508222010344, 5.1687400862147301]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[2.3302494491120718, 1.0132674889860909], [3.0852569345524268, 0.17088254580133155]], [[3.6230347251293828, 3.293473167103024], [2.0100518919032608, 2.471623563795275]]], [[[0.76546887325291602, 1.7747202367792367], [3.4940241440626814, 0.88475954091475839]], [[4.2720401189901036, 0.49080799173091316], [1.2661405274859057, 2.1406717371587485]]], [[[4.7305887946610747, 2.6616276172918951], [3.0023717479164631, 0.49542242700029071]], [[2.8699129004418862, 1.2388173415101169], [4.2216641771459074, 2.077034126765561]]], [[[0.27814397646369449, 2.6361660365660113], [1.1563752072897118, 1.8622056259818975]], [[3.1891437597851811, 4.8696897219004533], [0.81876611375006669, 4.5871316008677274]]], [[[2.238850363868798, 0.99056135246678567], [0.64240922804323153, 2.8717381386368892]], [[3.7374803744994538, 1.8164702342423966], [2.4523661043542408, 0.92798845748147984]]], [[[4.9150381299713164, 1.8193848382475666], [0.22566086872986879, 2.8874912686558414]], [[3.5740900859234421, 4.3921545456404889], [4.6450457884797975, 0.21547054946598179]]]])
      arg1=Data(numpy.array([[[2.4572704899643156, 0.85476804031574094], [0.83429368446053331, 2.1303137047969725]], [[2.5877423491014229, 1.9854664698119278], [2.9869042396559382, 1.0883606164966695]], [[2.5384536057128098, 4.8768519647370097], [1.1420743929268007, 1.3895799396109989]], [[3.5821751632313141, 1.6045335345400935], [3.0321425357251695, 1.3727197306169356]], [[0.1740511653775034, 2.8457185449714384], [4.8676469883713906, 2.9601797680313839]], [[4.633537127603482, 2.3050739835206473], [3.6630061662250952, 2.9429502914828753]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[7.9947909063139768, 1.0329176122818962], [2.6195735542096212, 0.22086887617236009]], [[2.9270534501464627, 2.7031837680830719], [4.4251357835571019, 6.8734635978848138]]], [[[0.50076546203024763, 4.4124926659418424], [11.98823810051735, 0.78419365847052325]], [[76.497513428577861, 0.11933906316111055], [1.2928176570162746, 2.2895898103759058]]], [[[51.670292470118135, 12.000979166749985], [213.07059222654593, 0.032541791675802574]], [[3.3336510368513217, 1.277089189036928], [7.3987708846320537, 2.7612981591609742]]], [[[0.010215898651791198, 32.210551950349974], [1.2625369445414045, 2.7118670518572916]], [[33.66756078599991, 121.50717010905251], [0.75996366188292008, 8.0930208198335869]]], [[[1.1505944398230379, 0.99835075183040956], [0.28384836185231005, 20.125712106776852]], [[612.50841507780717, 18.273922719504725], [14.231233267493201, 0.80153074466203544]]], [[[1600.3520328459538, 16.009219201755418], [0.03233469999359459, 11.522168911383769]], [[106.23035149321953, 226.01229212879724], [91.816009413548073, 0.010919274461495123]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_constData_rank4(self):
      arg0=numpy.array([[[3.0895114577862808, 1.289689912925106], [3.7268252153406509, 3.513961416301322]], [[2.1610599528898464, 4.9993960010649738], [4.8123403742682056, 3.3684142439047005]], [[2.6117081735410101, 2.4756604580054122], [2.2519355111298038, 2.4276095104616662]], [[2.6959455533298016, 4.3897687183838698], [2.0395202524104636, 1.101816204760925]], [[1.8040355595652711, 3.8761772908802103], [4.5296019725692354, 2.9808438524964607]], [[0.42648354390290943, 1.4767813664841576], [2.5350350856385986, 0.70681507491321183]]])
      arg1=Data(numpy.array([[[[4.5355176532358383, 0.90455633744321273], [0.67923149452020226, 4.1919321823208788]], [[4.3498512646368406, 3.6603443221231906], [3.7553091527048337, 4.582702977474443]]], [[[2.069212315816225, 4.8372444114351074], [2.3030788702161868, 4.3662075620460179]], [[4.6874061864955818, 2.9121047488202412], [0.40169204027682653, 4.1627651821818166]]], [[[1.1209439453253882, 2.6614409069036897], [2.7152981216668755, 0.96550100384490667]], [[2.9789285603919078, 3.1315338204837073], [0.10289988933400737, 4.1710110267418594]]], [[[2.4009371667407344, 2.2334803103053944], [1.4578143620492721, 3.996595057053987]], [[2.9903817817656568, 2.3203107972164045], [4.0723847158810393, 0.58371264138581136]]], [[[0.18946226360655913, 3.7023239787348001], [4.2427937757539915, 2.9762198390895214]], [[1.9620060942962203, 0.95636843797834303], [0.55777559330401105, 2.5397153354509538]]], [[[1.2470548322492774, 1.185460468617201], [0.49135986655978142, 0.68472024799517184]], [[3.5174872210908026, 0.17466638524464012], [3.8408751993326722, 0.037911030308305581]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[166.68782592766098, 2.7741691153463082], [1.1886256490233889, 2.9050050902730717]], [[305.66082530106502, 123.39499848199972], [112.10795402328118, 317.1213522288009]]], [[[4.9260259588279895, 41.578151990847488], [40.706354920371602, 1126.2091248355032]], [[1579.360660725476, 97.071810517178093], [1.6287807325243011, 156.87310313857228]]], [[[2.9332505452537858, 12.871237792649577], [11.721647317882372, 2.3994357926362397]], [[11.226360571158803, 12.706937073752037], [1.0955566995647792, 40.418961249321406]]], [[[10.817087158461661, 9.1618842223947343], [8.6409332178776062, 369.46991477020657]], [[8.425718755125688, 5.2263827525778073], [1.4841735268049039, 1.0582289693827895]]], [[[1.1182754107377002, 8.8859190987816472], [313.67165190927619, 56.392138666817296]], [[19.372862629900258, 4.2406762383555892], [1.8389708433016947, 16.020888882033269]]], [[[0.34551574900260246, 0.36413613770054715], [1.2111423416573686, 1.305973104730195]], [[26.363827887238664, 1.1764200576893171], [0.26375603052666302, 0.98693153893287711]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[0.43977910102966156, 2.1619567767617616, 2.2392040324835922, 4.4442831697254279], [0.75444492751892078, 4.140831822740676, 2.9939728988191701, 0.093671394881980868], [1.1917868670487388, 0.72163323004159818, 2.8904114823134761, 3.6338074376344642]], [[0.29678393841936279, 2.0219904834587124, 1.9268411532344771, 1.7916372791303026], [1.0523469720158316, 1.7906509357188427, 2.1243015401968597, 4.739072438286775], [1.999819689346106, 2.5262351773914329, 1.1154399797930388, 1.691989614224586]]], [[[0.30528092474318025, 4.9840185030989117, 2.7479544355446293, 0.57342453223239143], [4.8246246147024259, 4.209215660022573, 4.7279849026887071, 3.8740087545645561], [2.8651376309797132, 1.0062053229112888, 4.5284537420899502, 0.91468308065787018]], [[1.0857867930786536, 4.8761921267697206, 1.5454326540800478, 0.9222059874251235], [3.5152606142360217, 2.7199623621575109, 0.36979788870636876, 4.5288016450642434], [0.6114084287783067, 2.5971371492355968, 4.1850394428231041, 2.2813032448830177]]], [[[1.550830370574334, 0.12839380849276405, 2.9293603091985121, 4.3435991057583001], [0.8180797927723471, 3.1163110687065023, 3.85140410907381, 0.39907055360850746], [2.8854121881189587, 4.2089077047967596, 3.5351263869462555, 0.53389806009645446]], [[1.043947437361981, 3.2915886658291176, 1.7540860243407435, 2.1266054643658685], [4.0499484443909504, 2.2291000709767714, 0.81192572278320263, 1.0748585565315512], [1.4402364778157744, 4.6363362665601331, 0.81719581854612156, 0.98694274187743125]]]])
      arg1=Data(numpy.array([[[[4.5782301611250338, 1.0887668196913494, 4.8712995267265802, 1.3647631598671353], [0.26260714487166831, 4.1323141051269996, 4.5574138820265633, 4.2245085493520946], [0.60033969637259477, 2.450383570417622, 1.2009400579659579, 3.4378585253487328]], [[0.12070082604328239, 3.5952504462134107, 0.48118940437662572, 0.017507177154898492], [0.051910123010556392, 4.7983030619229474, 0.82086016658407068, 0.34279047946540203], [3.0034410412163735, 1.9648273110517844, 4.0601688225625718, 0.89050113778203688]]], [[[1.7447676456115147, 4.2377373235400473, 1.6601525303224991, 2.9236795489832157], [1.2260501437415852, 3.4813973767931579, 3.1329070617552395, 2.5933617217362244], [3.7350494567620887, 2.789971536767998, 2.7832362065293612, 4.9516056128360875]], [[1.2296111214892274, 2.2641304713370398, 3.3639554362468673, 3.4098042954518459], [3.1038689602587985, 4.3584442826460803, 3.8167737183516888, 3.2780393210155188], [2.6790665011750976, 0.47637383863550631, 4.7390171598762763, 3.0514647008752158]]], [[[0.43206863471476437, 2.5586380711785894, 0.2168416509953105, 4.0518954138049104], [3.5913977397699695, 3.2512013734975111, 4.3459049747655021, 3.1324705985636419], [4.0273772744909717, 0.80438404579898048, 1.7986456279686485, 3.5123016662673701]], [[1.8014087925058129, 2.3576515028820717, 0.85425347463893153, 1.7909329269529413], [4.1387001973573039, 4.2432043805049151, 0.056986537363837624, 1.5790683300720869], [1.3881489195969052, 0.73455782945443682, 1.3308725428280885, 1.8724149929759428]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.023261934396486716, 2.3151029635250335, 50.747086918587627, 7.6576572831056318], [0.92867577829703207, 354.813194478068, 148.06712044142031, 4.5242326169084808e-05], [1.1110792618468059, 0.44959465254069714, 3.5775436175481503, 84.419926135445081]], [[0.86362085194823646, 12.570497666445327, 1.3710864389933923, 1.0102612473649686], [1.0026521146921838, 16.369037336754037, 1.8560886493405073, 1.7045951420987853], [8.016932771488527, 6.177197281226686, 1.5582588723894655, 1.5973070386508166]]], [[[0.12615970184568656, 903.98099863559617, 5.3557805976327337, 0.19672611117494446], [6.8858975522906727, 148.96777868515608, 129.92641605670465, 33.520707821819336], [50.98718197145979, 1.01740898033463, 66.936594162422892, 0.64302448814014812]], [[1.1065012328545774, 36.133058667693, 4.3247007252447212, 0.75870011585323627], [49.497199257207498, 78.346240764868696, 0.022439748273434701, 141.3640731610478], [0.26765025969647371, 1.575631008551976, 883.57793415803985, 12.387476298154949]]], [[[1.2087510262094325, 0.0052370527430253269, 1.2624546405170505, 384.14962725718397], [0.48619954683034616, 40.264810013037774, 350.7881500094673, 0.056272973382131872], [71.356030873000421, 3.1774001314467317, 9.6913919252678706, 0.11034470133298796]], [[1.0805574273996734, 16.590596154477836, 1.6161487079313464, 3.8624716480212311], [326.62644466973478, 30.004396654650797, 0.98819726434024246, 1.120742583088375], [1.6593183545440757, 3.0856311384531194, 0.76439398572007855, 0.97569072302288895]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(0.203077902528)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.00222517787)+(1.-msk_arg1)*(4.06891216441)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.0410946006738)+(1.-msk_ref)*(0.00152384295922)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([0.8791655451395447, 3.4078414357548916])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.98572572892)+(1.-msk_arg1)*(2.50758571018)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.52620097494540763, 451.64389162452164])+(1.-msk_ref)*numpy.array([0.72402303755962083, 21.639024677512047])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(2.04156358958)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.5675076850044962, 3.206504089939171])+(1.-msk_arg1)*numpy.array([2.4896061859326197, 0.66387045137834699])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([12.758366374816521, 9.8604638204640001])+(1.-msk_ref)*numpy.array([5.9113351415794266, 1.6061097483271392])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[3.2633636972440403, 4.3392554483417172, 4.1526498774796643, 1.8157292344373801, 2.7514516863823202], [1.2164169300427956, 3.6080955446523935, 3.6650007763362873, 4.1711902875971507, 0.78789193984293238], [3.4397862861054933, 3.280874792330517, 2.94117379891002, 0.66320190380348032, 1.3265506370173141], [1.8354110170370972, 3.0974624427495252, 3.590890001254877, 2.5370169508744644, 4.4107773342480723]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.8627052121)+(1.-msk_arg1)*(3.44110283662)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[29.544241258673043, 66.793140729198385, 58.895634481955348, 5.5155254676524006, 18.127425012496822], [1.7521286468033448, 39.384262681853251, 41.188671361427048, 59.651523809213231, 0.5053759389732444], [34.350369180049917, 30.000347123851764, 21.939999675930313, 0.30862023647074915, 2.2455470286073993], [5.6884086805790064, 25.445182856089204, 38.849009489450012, 14.370061130168523, 69.993358678047898]])+(1.-msk_ref)*numpy.array([[58.556583987676639, 156.10302277729923, 134.19026134903422, 7.7879190210087055, 32.551999563122656], [1.9623545684036543, 82.727710969502454, 87.304551510662023, 136.26315942589906, 0.44028219400822305], [70.18752549160574, 59.644921532529942, 40.947545113653931, 0.2433687062536927, 2.6442671401613538], [8.0822731622668655, 48.933052529148718, 81.378096489348664, 24.621714533539926, 165.13641666578212]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(2.76927009189)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.0688042114191942, 1.2559540038106969, 4.7127391695110585, 1.8545075671913149, 1.9692399350137069], [4.8496509984932699, 1.614360740249331, 1.3798772276090336, 3.4367731975694458, 4.4240804603233475], [2.205277465622141, 1.0906240816892439, 2.5364345130383947, 3.9441675041665003, 0.37033144089233433], [1.679052149698091, 1.2314877393049288, 4.9363400796946175, 1.7619854291517771, 0.095666565414163823]])+(1.-msk_arg1)*numpy.array([[2.3441408647312527, 3.40093738979686, 2.1701883561594979, 2.5086641663662586, 3.733598330015965], [2.7344592235519869, 0.7880243001017565, 4.7028726806633676, 0.97234403016533011, 4.1628974760494541], [2.5415900795960189, 0.31015207569412878, 0.83629978223164536, 2.5598828819323387, 1.8131814416566647], [3.8811568647161101, 2.2255441671439486, 3.9257659919920025, 1.1578034010274563, 1.8491289711483234]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[63.080896667664625, 3.5941027513313051, 121.54926829137163, 6.6125630307734911, 7.4323036530594369], [139.73892347466864, 5.1777037489674163, 4.0776488638580428, 33.136700253168001, 90.585721526147267], [9.4523166396689309, 3.0370662511492514, 13.244350125770714, 55.560085028235981, 1.4582157498134884], [5.5303731281410737, 3.5056411825536964, 152.63904351497234, 6.0178483593150567, 1.1023501622981631]])+(1.-msk_ref)*numpy.array([[10.888463830392636, 31.948961122386557, 9.1204455835454663, 12.874963752261417, 44.834713879767826], [16.204286632951966, 2.2314883199723043, 120.33383461298149, 2.6923484929222026, 69.425889812035123], [13.314084133973696, 1.3715148705278906, 2.3439590326382267, 13.564487768448272, 6.3399901068398323], [52.106173390097062, 9.6494719338828503, 54.528394523605996, 3.2521612182801367, 6.5764348244707227]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[0.46090222808819004, 4.2045732390840156], [3.168687878026804, 2.8174936316251884]], [[0.91274053948125511, 0.33286697242129948], [4.5866915233484846, 0.16126982603132675]], [[3.4930792741070187, 0.39250408817063348], [4.1897134864040737, 1.4814580336815248]], [[0.25538472163345283, 4.8906665956712159], [2.5517006323978983, 3.2782894420235902]], [[2.5310628728821531, 3.9359337468635944], [1.9696612522747803, 2.9907032242150753]], [[4.8766675503158048, 2.6629491303567168], [4.7269789798692559, 1.2575618105814521]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.745788420236)+(1.-msk_arg1)*(0.629902716352)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.5612073419704654, 2.9185328553538388], [2.3634700486185203, 2.1652227104614146]], [[0.93417341855094738, 0.44026587485951668], [3.1141440055544964, 0.25644975668313957]], [[2.5416665053889682, 0.4978444798897988], [2.9108368525218822, 1.3405966914643599]], [[0.36132068921385846, 3.2668012599468894], [2.0109852960098489, 2.4241739585906301]], [[1.9988428801501028, 2.7783038667353792], [1.6578827110919661, 2.2637383688417385]], [[3.2598249320877244, 2.0760160322246461], [3.1849065233404859, 1.1863909618113329]]])+(1.-msk_ref)*numpy.array([[[0.6139122143344341, 2.4710647915971786], [2.0677847780580016, 1.9203035723434714]], [[0.94411018300550675, 0.50012322149251287], [2.6102394706385139, 0.31683675461178185]], [[2.1987126399822974, 0.55483180514393238], [2.4655601168020902, 1.2809072605731082]], [[0.42324366346733799, 2.7179083748297317], [1.8041105514232338, 2.1125530896310121]], [[1.7949056029589481, 2.3704031336028186], [1.5326325033047934, 1.9938427613983509]], [[2.7130052953302695, 1.8532637348594145], [2.6602477638966655, 1.1552974155868148]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(4.60548445055)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.6438158183495908, 1.9561026194748328], [4.634587795976536, 0.87543260901294861]], [[4.5033581827409765, 3.1366606618176336], [1.0048270882914627, 3.660857363834221]], [[4.6932098053650142, 0.039239825616938263], [4.1791639483125431, 2.0414830145740903]], [[1.7719248736452657, 3.8181326385413095], [1.5506689656235093, 0.12991402765823531]], [[3.4511577319304045, 0.036787266044364893], [0.51521149266792687, 4.1990232679518344]], [[2.0393166876926165, 0.55455746151774565], [3.9673963687403249, 0.69937321333195024]]])+(1.-msk_arg1)*numpy.array([[[2.3532749323379116, 4.1414224553210977], [0.15324310180704637, 0.36913916912471056]], [[3.9366432347540963, 4.3099556427682275], [3.6243188651264138, 2.6098607721243945]], [[4.2933568257665264, 3.3806647562236591], [3.9815786666802211, 0.93507284071971375]], [[4.8394923319100887, 4.5787654294767526], [0.16996295592094879, 1.7825628691721573]], [[1.449828816745909, 4.4529349374437954], [4.3805450371472814, 2.3177016489913962]], [[3.1757882894173406, 4.3288841231688622], [2.132106827855405, 0.50476195523796208]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1202.6200068313053, 19.835109006813926], [1185.7897856090103, 3.8076133792195708]], [[970.43475357917555, 120.35651535614046], [4.6395622793923179, 268.01329663220065]], [[1296.8514652824567, 1.0617610950179888], [591.47488237046457, 22.59775524207668]], [[14.971800730900135, 340.77925482791665], [10.678750873929586, 1.2194633944786772]], [[194.5663582568192, 1.0577915311619319], [2.1964790627333177, 609.68920159670301]], [[22.523113704253451, 2.332514101109374], [428.03186100480747, 2.9098928145378773]]])+(1.-msk_ref)*numpy.array([[[36.380543647244771, 558.34586695918426], [1.2636952918456699, 1.7572797345375486]], [[408.39301229214232, 722.24943304515307], [253.46690175342542, 53.833949114989551]], [[704.17017058656359, 174.70747987631151], [437.40411613721091, 4.1707171513305097]], [[1621.4957948583531, 1088.8852301463749], [1.2963797365210665, 15.217032146292159]], [[9.1545220033743266, 898.5077906522248], [804.46547948305226, 34.456749348737972]], [[127.76797008516299, 743.43328854791253], [25.952179671691301, 2.1617036086810866]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[4.0207501319312904, 4.874461896399735, 1.6397347198618097, 1.2794087640320917], [1.6559954149930562, 4.9389177246915779, 4.0315954527312252, 1.3827773789354505], [4.348329947255257, 1.8620561392413073, 3.2996677778506616, 0.63094983859082343]], [[2.2728063302743542, 1.761661813827353, 0.61531237433062957, 0.87539981552332236], [3.4325013358464544, 4.5327566854262153, 0.81214527908626977, 0.72725243636153092], [3.2378171792624375, 3.9556627933950237, 4.5989936630324308, 2.2769052782318107]]], [[[0.62283209282003038, 4.8636526846679589, 4.0304257687106047, 4.4119889421037524], [2.9728618608819364, 2.6955721015901655, 2.8100239854149422, 0.61097446400436339], [1.6486222364086534, 0.041376475859494194, 2.0999380923682649, 4.7935524555592774]], [[4.7931681805434296, 1.7081483297780984, 3.9572806649706558, 3.9841045750157416], [1.5029795421924754, 4.9192052484507931, 4.2306736208943461, 4.7161904171479252], [3.8926611369791448, 0.05145568916947528, 4.3588824479268977, 4.2924920859911522]]], [[[1.8783594326049271, 4.1343666317566212, 4.0587705923803234, 1.4804734135114219], [1.0072721456549045, 0.86335283142342356, 4.1695273045337755, 3.2778539385002357], [2.7030849153175103, 0.66193949014796727, 1.8703289947061958, 0.39684247655008514]], [[2.8188988238712054, 0.45037769556219198, 3.2443538041648172, 1.3785013790916569], [0.57199731820375976, 1.7589328124124295, 2.0222439469385276, 4.5882448416301953], [0.36318387766299248, 1.4743603289700342, 0.53583824177734141, 3.7457367556284877]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.78604156399)+(1.-msk_arg1)*(4.35143079413)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[48.264149310962189, 82.526290266718419, 3.966141054520532, 1.9867005727277978], [4.0766914372898757, 85.602608977958184, 48.62772360020498, 2.4668438740554772], [60.033406836794917, 5.6521161812673579, 27.827844690841893, 0.27718983079598902]], [[9.8491273067163263, 4.8434009159547937, 0.25847093414558592, 0.69021559816577716], [31.062307445190026, 67.398921524052383, 0.56006160349917988, 0.41176476033642412], [26.398799653340177, 46.11876700808115, 70.178831823150873, 9.8986945233201027]]], [[[0.26736774327676022, 82.017443666509038, 48.58842739500728, 62.514141949752421], [20.810593363489716, 15.842052724385104, 17.787904336934783, 0.2534261049996745], [4.0263224941485429, 0.0001400289241106599, 7.9009748221039686, 78.766226832277468]], [[78.748636209850432, 4.4445309249863012, 46.171338302971591, 47.048563063821369], [3.1117063472799256, 84.654114297692928, 55.616411789522438, 75.27546495663816], [44.101316951986306, 0.00025703951460871678, 60.440181834112522, 57.910187842760784]]], [[[5.7910703105440771, 52.160417357874714, 49.546431246829187, 2.9836178002649101], [1.0203923259380079, 0.66407632991792609, 53.405705914671771, 27.318323808405179], [15.965372034933088, 0.3168052073596952, 5.7223559411908882, 0.076161148816950769]], [[17.944863610599853, 0.10835545333488578, 26.547549214934033, 2.4456497888663966], [0.21090631310246885, 4.8225263111017371, 7.1131991127350762, 69.722810741172907], [0.059496897073493302, 2.9494208065976668, 0.17582338629511987, 39.618554330884201]]]])+(1.-msk_ref)*numpy.array([[[[426.18725507012675, 985.06602134482591, 8.601464957938969, 2.9217540594988769], [8.9788485054695144, 1043.0154396787798, 431.21219144686728, 4.0970734108436231], [599.26117552257983, 14.957356859126138, 180.33800576524885, 0.13480045555498951]], [[35.608750881790463, 11.75208478609779, 0.1208549138068218, 0.56042225092178799], [214.12775247722789, 717.98536256809018, 0.40436858144187504, 0.2501109201996301], [166.08395228966938, 396.97055020643461, 764.77091441437403, 35.889043171492709]]], [[[0.12741470731756999, 975.59600424862344, 430.66806031293754, 638.38407642111622], [114.5478886506765, 74.807961065930655, 89.645940676459105, 0.11719097026601777], [8.8061823127779455, 9.5697055465192825e-07, 25.238250680922327, 915.8702179530336]], [[915.55077571336778, 10.275878770219435, 397.67753943715252, 409.5412222580598], [5.888423463407169, 1025.0214719553851, 531.83594479030978, 853.26918653422831], [370.18364566878262, 2.471107355029005e-06, 605.61517239430339, 566.48917536784063]]], [[[15.535637046042604, 481.12845971352488, 444.00369494164255, 5.5142502735583649], [1.0320320601225694, 0.52762805574059368, 499.18886939215844, 175.2074026565995], [75.719467164186653, 0.16607459200359856, 15.248684792630435, 0.017923163358030503]], [[90.884483975574184, 0.031085921333806124, 167.54791511474693, 4.0422279848567682], [0.087966385757282217, 11.673071385256209, 21.419705817930542, 757.02343643782808], [0.012187683651813762, 5.4158553529959912, 0.066207763933241293, 313.11773192710092]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(2.870993294)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.4130572400832984, 3.3637343922902825, 4.8670399558846205, 2.2781215766994944], [1.2652082973116738, 0.82975878253333135, 2.0720321202162539, 4.6021574810568078], [1.794237257577195, 1.4122795406324684, 0.032797174032186599, 3.6844368246762422]], [[3.9194779066825056, 0.23269859968528178, 3.7240017623004711, 4.6415495661402408], [0.94363490263541205, 3.2564298919793431, 1.8621164848251013, 0.45249396833144101], [4.6286031918440864, 2.696271253089872, 4.530247386856086, 4.1345165897257239]]], [[[0.90815310115009251, 1.4005162018138755, 3.4947252078814879, 0.31642609851110215], [1.0596983696732294, 1.0428900861275543, 2.0317834158244357, 2.56430756985363], [0.66988817453735405, 3.1782899197354637, 4.7979341331503802, 2.4593555683573292]], [[1.9242606235414104, 4.6804685114046993, 3.509395208588177, 3.134418370384672], [1.307211520663583, 3.4128803565090573, 4.457117371849499, 3.9792312984704599], [2.1287838680684725, 2.2135403766168729, 3.8955674088880197, 0.29996865632154068]]], [[[4.434986561015041, 4.4358289714052361, 2.440226914282853, 1.923100917243955], [4.0820042701059887, 1.119888471357668, 2.2487151402685477, 4.3066923956150669], [0.94447852519596842, 3.4247966175499966, 2.2576164318297454, 2.7389457346634711]], [[3.0803458937310566, 0.14009937133476433, 0.21472331905347664, 0.6390187735074867], [3.2940914359121027, 3.6407917055217487, 1.7142547576523874, 4.6505746440371754], [3.6343869113243912, 3.9862560655278632, 3.1077497254857716, 3.8748848386410533]]]])+(1.-msk_arg1)*numpy.array([[[[1.9086884985971901, 3.8603985356741664, 4.6098749835726984, 4.2059495204109245], [0.1836851644171289, 4.9956466198373519, 3.5951258532777191, 2.2641818156315074], [1.4980700997780487, 1.7894408139557016, 2.3592590838739542, 0.76709371753730116]], [[2.0092405791789489, 3.8130180412076973, 4.5216004169781989, 0.60562948322021648], [0.37357236967708707, 1.4085028743689403, 1.8704108495672485, 3.4470006861550289], [2.3007548316988577, 3.7066590014814902, 4.3303909277477075, 2.3572706276186741]]], [[[0.022985609592688864, 3.3291661149716734, 4.6704488024736497, 3.9322086998622794], [3.681039703080824, 3.4804737382676509, 0.55845083070169477, 2.3626488951690248], [2.6091214718874176, 3.2725539239212096, 3.210182345243846, 4.9556691222295477]], [[0.1614956594322928, 1.9750520885740359, 0.089323339115444558, 3.7423594932024336], [0.67262758973494452, 4.8841532135642165, 2.0942664876479831, 3.9597978137063823], [1.4932982997703534, 1.6378293275063041, 3.7428562558163794, 3.7576303217734117]]], [[[4.9245623521507689, 3.2618915080671154, 3.3098041099684261, 1.6556285152197254], [4.7174734175398747, 0.26581235706901496, 1.4879013736012703, 4.8085851238533861], [2.4316190995298297, 1.3117402684742345, 4.5738684960399194, 1.5303573237994839]], [[2.8486132472978642, 3.376856725358, 0.80787341924861689, 4.2091146774276265], [2.5859146794904944, 0.87320110568069265, 3.2893864357708948, 3.3768090489053746], [4.1954172385242181, 4.7807902976926053, 3.480660741673109, 4.7177764822843979]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[12.742594514110339, 34.729504747323375, 169.53560163720761, 11.052308218991024], [3.7975728353719465, 2.3991434847572637, 8.8931861934628937, 128.21422598760628], [6.6346558322107887, 4.4347533867746094, 1.0351949889709333, 48.70691508154006]], [[62.408968206033762, 1.2781547738954222, 50.782331757574603, 133.65310648069504], [2.7052982387001139, 31.013421111313168, 7.127041200226083, 1.6115982899229553], [131.84060929826907, 17.178253001792033, 118.84997038687439, 78.296372033120107]]], [[[2.6059335843581963, 4.3800743884830675, 39.874623805806408, 1.396154032731808], [3.0575667883003526, 3.0038429150964823, 8.523582780321787, 14.946358037395168], [2.0268907450053169, 28.560055990306047, 157.61887423266828, 13.380242595862713]], [[7.6098007317671881, 139.25320089691084, 40.496354644657352, 27.268702461386212], [3.9695834843122513, 36.577079245757872, 110.02798961771965, 66.468515397666636], [9.4417285818205112, 10.324587755356582, 60.854854047165951, 1.3721301176524141]]], [[[107.4896266202492, 107.58516874319787, 13.113011695720939, 7.6004989228454694], [74.078009940125995, 3.2579540574602119, 10.714795671880649, 93.886618688317739], [2.7077063038403337, 37.039665350769077, 10.815857868171006, 17.969056759004442]], [[25.757134094065083, 1.1592310900523093, 1.2541520417320937, 1.9619645710252931], [32.270067601447039, 46.515720770330049, 6.0979497385444619, 134.93134005973715], [46.202572053045806, 66.962791214408924, 26.512420283017921, 59.541797649417838]]]])+(1.-msk_ref)*numpy.array([[[[7.4858436178278476, 58.639026208205316, 129.26206196036998, 84.422890916260314], [1.2137625011029589, 194.16319160926568, 44.328529527418432, 10.891009292345652], [4.8547213866737557, 6.6011784553208903, 12.039725264908441, 2.2457096553820115]], [[8.3233247308755995, 55.780828246503432, 117.77103382094487, 1.8940777016873258], [1.4828873697173268, 4.4171244901611812, 7.1896600382891629, 37.917284933516576], [11.3193045986859, 49.861931283300216, 96.262777550521903, 12.0145027151537]]], [[[1.024538183670229, 33.486149115409219, 137.78940907411103, 63.252561195958961], [48.532720103257809, 39.279772799856758, 1.8021396847815019, 12.082845421579243], [15.669732789175953, 31.545324674784965, 29.537029152011435, 186.14694878756129]], [[1.1856874106121262, 8.0285554267167374, 1.0987856110712817, 51.775114718829116], [2.0327551965176611, 172.62326813627743, 9.1041926867029996, 65.120063800842658], [4.8303508068619303, 5.625725457205121, 51.802247569417254, 52.615730055140567]]], [[[180.1391131240554, 31.192578169205898, 32.809286468358465, 5.7323293521677776], [144.79535315259429, 1.3235812701216665, 4.8029349747949999, 159.3994130114699], [12.994506551780491, 3.9885886790618308, 124.44542909139204, 5.0228811053204776]], [[20.172351644418303, 35.213487646136542, 2.3444016687062321, 84.705178795371552], [15.290867779936836, 2.5116215710525034, 32.11033479613203, 35.211717073554397], [83.490314254541985, 154.79459527096117, 39.287520503286032, 144.84164144173505]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([3.9542068484170843, 3.4445569013941868])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.7658018484184983, 1.4938202258511422])+(1.-msk_arg1)*numpy.array([2.6333941322346055, 2.1178078600638446])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([177.17699502898986, 6.3442557527861085])+(1.-msk_ref)*numpy.array([37.350178205219549, 13.726039957296285])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[2.497373515532022, 1.7270984987081095], [2.5920572605765333, 3.962159032568946]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.96428248200709799, 4.6872206599835513])+(1.-msk_arg1)*numpy.array([2.5693053781834694, 0.07119967998778233])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.4170541359210147, 1.6937165399224603], [86.864729945653593, 634.80270684583377]])+(1.-msk_ref)*numpy.array([[10.501624822486045, 4.0713657464906881], [1.0701665261854463, 1.1029925019740909]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_expandedData_rank2(self):
      arg0=numpy.array([0.78754816786025739, 2.7666586312074624])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.2569317922082486, 4.7621995794561975], [1.717333315690909, 4.7086537189389004]])+(1.-msk_arg1)*numpy.array([[0.28149661563734818, 0.4177866393782842], [3.9472111210254246, 3.173173001075408]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.58331688371224655, 0.32066456483591965], [5.7409667984100583, 120.5079692426865]])+(1.-msk_ref)*numpy.array([[0.93498008786661324, 0.90503623792901544], [55.525434508404238, 25.258147925185668]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_expandedData_rank1(self):
      arg0=numpy.array([[[3.519784683436141, 1.2520542407199786, 0.87455122999672197, 0.49961347172433451, 3.2111196952430414], [1.9182510089265004, 2.1962492710785262, 4.6036076376580635, 1.858560839902403, 2.7879630387567729], [4.3337791770701433, 1.4884408038093206, 0.54067110098725168, 0.58154791546683482, 4.9077258694799726], [0.015694199414061926, 0.61917930104900787, 3.0543056041641123, 0.99675049553621364, 2.4291908598685237]], [[2.1245695749118525, 2.127805220425345, 0.95542359195459459, 2.799360658038871, 3.1045748330459886], [0.86644788608130463, 2.0633090436546224, 0.67677818532085021, 2.7100311803971819, 4.1789593574232766], [4.1476438621048777, 0.70116256566009005, 1.9644309355628324, 3.2087218636524351, 1.6365357787858492], [1.2926478122049951, 0.40257997085393232, 2.1574375519024334, 4.9092881621888997, 1.3788307888601001]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.5444763909294945, 4.7238329844614224])+(1.-msk_arg1)*numpy.array([4.8170083746592214, 1.7155220861088776])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[304.52959192133466, 2.7774488582492944, 0.54380658903497991, 0.042702172006967305, 200.67278437892614], [19.30443394650467, 35.707882942608741, 1031.4176950376982, 16.721203487734495, 105.58356274257586], [783.83750421865125, 6.0950114944505032, 0.061139380991146956, 0.085146024994086023, 1379.3967418028476], [6.3179944391363258e-09, 0.11321779159087692, 159.83554199344715, 0.98531751291115865, 56.457457356209808]], [[35.153886440602996, 35.407509837105728, 0.8062115713762753, 129.36934049179368, 210.92880994149198], [0.50804913372202531, 30.616047976519784, 0.15814551614094027, 110.99367258184927, 858.66191443885259], [828.68772880039353, 0.1869275832415726, 24.277362407623283, 246.50637195853599, 10.245858516003567], [3.3621254879838078, 0.013595263173574285, 37.797967139953855, 1837.6216557809607, 4.5606477878492724]]])+(1.-msk_ref)*numpy.array([[[429.11393363272629, 2.9529187665901291, 0.52429906769807766, 0.035344197880650879, 275.78368868068475], [23.054613775191527, 44.246943413213678, 1563.687892716797, 19.798250479853458, 139.62116746992461], [1168.9410384172102, 6.7928090367991594, 0.051705504607249063, 0.073452468930403386, 2128.0222941701836], [2.0363876532331954e-09, 0.099352316579658997, 216.68433754387084, 0.98444388843661068, 71.907127490328151]], [[3.6428438553473064, 3.6523666271628543, 0.92475296754402769, 5.847084082665913, 6.9829756056660601], [0.78198026338635351, 3.4645112252096033, 0.5118314539267147, 5.5306599037216557, 11.626690823595668], [11.477625416414028, 0.54387439285799455, 3.1845945411958461, 7.3896497588589609, 2.3280587069822234], [1.5532693472899002, 0.209949652529443, 3.7400586726937237, 15.327013543966917, 1.7351385334712566]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_expandedData_rank3(self):
      arg0=numpy.array([3.5283370657191298, 3.3164096419225157])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.6693077288957978, 3.9995391992746887, 1.8119311704479442, 3.6435117612564993, 4.0260330528261781], [4.6140523340441604, 0.096423834228759456, 4.8240790797404802, 4.5847651283309112, 4.9396328839311074], [0.022294782575039924, 2.5155663434755895, 0.12324477092613059, 3.4194385851878697, 2.9347898887103718], [1.9834553837039579, 4.4072374447331573, 1.1096556447775063, 4.2125633266607982, 3.5160592677881515]], [[1.7645513811186877, 1.3264576069298044, 3.5122183892913834, 1.9662987249270567, 1.0913572297640663], [4.5712774724351242, 1.401743673321671, 1.6102091037242205, 3.0427057640393977, 0.27805612432871102], [3.7814770452507642, 4.8895735889429517, 0.31399500845652512, 3.5495101925617258, 0.50443051053204613], [3.4298545435647765, 0.44271308069259468, 4.2996486302269661, 4.4935297826601603, 1.9237945030545209]]])+(1.-msk_arg1)*numpy.array([[[3.2528039234154078, 1.4542365261737222, 0.61584897933335037, 4.5471575313958645, 3.6354003894416107], [4.5462606686540372, 0.38779591984691558, 4.896299532459544, 0.68279729170399461, 4.5252404208659351], [3.4913793095634089, 0.6999834133506535, 0.75303380278580656, 2.7534786071948223, 3.4007231077037998], [2.3009521703519864, 2.6961547222473459, 0.7935795937568676, 4.7565337484448404, 1.2083239111268023]], [[3.6045402426169009, 1.0009125563871737, 4.7579537176789941, 3.1755048921868596, 0.86835229324662067], [0.79997460323645897, 3.9686422635274599, 2.2076082495257578, 0.70384711009638967, 3.1258991868575823], [2.6213410779394075, 3.0270858545052315, 0.048551263084322602, 4.8333987876074058, 4.0147459148543048], [1.2129736092333245, 1.3308980141129714, 1.4240342130659636, 4.2437071837095246, 2.5368191259552657]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[8.2047045165877126, 154.89162907922145, 9.821080537690646, 98.873060944681484, 160.15304069282121], [336.13748467248297, 1.12927263788198, 438.04759988994351, 323.95161450139233, 506.75157764914036], [1.0285086666354106, 23.847828034307842, 1.1681137806294213, 74.538768863257175, 40.457877596908162], [12.192164044757181, 258.98302729492536, 4.0514848178216836, 202.61592808653873, 84.195486164665809]], [[8.2936506330029101, 4.9050592032448916, 67.406273514634663, 10.563045659009118, 3.7002832574507258], [239.94884958901463, 5.3683807693166177, 6.8926246033554541, 38.391938331185663, 1.3956450293685101], [93.088220802725502, 351.43525642429915, 1.4570927797549431, 70.488289711642679, 1.8308000278625669], [61.068335873469969, 1.7002258277728275, 173.25675663100714, 218.59384276162456, 10.038262809472238]]])+(1.-msk_ref)*numpy.array([[[60.414065546901178, 6.2559949883164636, 2.1738044171632165, 308.94939872928751, 97.867035888877368], [308.60023976180804, 1.630592490236821, 479.80753425897427, 2.3652625678902939, 300.5288715599703], [81.615905366160831, 2.4170740991391839, 2.5842752594145795, 32.190110355312889, 72.800469490946483], [18.194181977379344, 29.945641568074524, 2.7198216805918225, 402.28654155174036, 4.5881964550816052]], [[75.295560170388413, 3.320039939168546, 300.13389954554793, 45.01780526875023, 2.8321966103372045], [2.6092837644909084, 116.50530501905462, 14.106892127805494, 2.3252560667453483, 42.418587856178718], [23.165883057725644, 37.679683799357413, 1.0599346682641142, 328.54653958694979, 123.1261797994926], [4.2811135630102841, 4.931241049546367, 5.5137781301822706, 162.01799358327179, 20.933457724046363]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_expandedData_rank1(self):
      arg0=numpy.array([[[[2.1455590287376474, 2.4174975797375251], [0.67064569136059449, 0.61632152509385818]], [[1.0055742550174787, 3.1599557456640328], [0.26956824941109991, 1.0432937304822754]], [[4.1240904400756406, 2.5417673516952428], [4.9918503188982895, 1.2120003512959534]], [[2.20492635903793, 4.9975703110417715], [4.7082655577257322, 4.5580219534023154]], [[1.5621746084636698, 4.0106334270484574], [2.1428649828479842, 0.62406632666591977]], [[4.044205717075374, 4.0313212495286672], [4.1508933180202785, 0.95492668691828253]]], [[[0.29567379533555088, 4.1141946907884277], [4.4119988676548738, 3.1646765436612072]], [[2.2244792897000818, 4.4034297951018493], [1.3968289549007205, 1.8519152731154302]], [[2.3275720893068739, 4.864499135802987], [2.702800444629279, 3.7512304695779548]], [[3.9107282013009708, 4.3712600215844466], [0.46478264192564261, 1.2746368635436547]], [[0.30205473241694408, 4.5985222473946106], [4.5223275875242956, 1.2375320150833979]], [[3.8206464927924455, 3.6955187124158222], [1.946173383306514, 1.7761957489540661]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.4885469700822807, 0.20158251200321486])+(1.-msk_arg1)*numpy.array([2.7161809900663956, 3.0008409556035476])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[30.770528833765258, 52.572243773568815], [0.16642030119700846, 0.11390436148454065]], [[1.0252647057431521, 174.92087483561158], [0.0027831069151293005, 1.2095362192172481]], [[578.00213277231637, 65.837154778525957], [1362.0083961682788, 2.3703146255372287]], [[34.780874645262912, 1369.0275929883271], [1047.5336410388807, 905.62597574099914]], [[7.4056689586792945, 509.97600802343254], [30.597485838472068, 0.12047130128431992]], [[529.41896578011244, 521.89018301326553], [595.05550715687025, 0.81300748001371148]]], [[[0.78221316701648003, 1.3299309133314619], [1.3487989578009274, 1.2614139884366793]], [[1.1748844786853061, 1.3482704693577328], [1.0696909873719598, 1.1322640965637174]], [[1.1856629709784063, 1.3756085681098416], [1.2219294927290809, 1.305399332570558]], [[1.31640275174569, 1.3462790756837855], [0.85688669399292405, 1.0501324294340189]], [[0.78558712679401699, 1.3601044444877284], [1.3555312120330747, 1.0438972663004151]], [[1.3102332301271593, 1.3014678201120016], [1.143652118957484, 1.1227756554558939]]]])+(1.-msk_ref)*numpy.array([[[[7.9528705767636687, 10.997439252781287], [0.33785003001919778, 0.26858309939038238]], [[1.0152132030046497, 22.762713624552461], [0.028417982989984471, 1.1220072169541246]], [[46.918347439883853, 12.601497285743537], [78.814252414644415, 1.6858097858444701]], [[8.5648645172911682, 79.059793557827248], [67.237587384854777, 61.5681307908254]], [[3.3589581083560454, 43.494625900741461], [7.9257762041544462, 0.27784952242856331]], [[44.490666040766293, 44.106717645691489], [47.751210590663497, 0.88225672558873536]]], [[[0.025822214547933736, 69.72220356584198], [85.990066867206778, 31.725519261733623]], [[11.014812376069097, 85.489864699356559], [2.726162539902798, 6.3546026235003756]], [[12.618797515310343, 115.263594856022], [19.760825426455007, 52.845023651193358]], [[59.878506829689655, 83.62933525134558], [0.10033902407933776, 2.0713240548331253]], [[0.027530855583311767, 97.36707157879043], [92.605586336059389, 1.8956020288234756]], [[55.834177124684395, 50.52468410718803], [7.3754372912129327, 5.6063768859571201]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_expandedData_rank4(self):
      arg0=numpy.array([1.0347030033441573, 4.2920070607543614])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.3212620875791066, 1.1532505999953218], [4.4857116209930243, 4.162977940137873]], [[4.7284149502453925, 3.7591763881211904], [0.68231641540126775, 1.793674644546019]], [[1.0729349945538684, 1.4423041725182177], [3.805186469993604, 4.9646217881701524]], [[4.5341466338489367, 0.078508394210729096], [2.0216151412982297, 2.4359465381503074]], [[4.1293196461146531, 1.34942700537581], [0.19690237752601972, 1.2121191998353493]], [[3.229863616468224, 2.4793768936055516], [2.3700695620298862, 0.92302312156700117]]], [[[2.0221865810311321, 2.4749595682896586], [3.8010968681477753, 3.254900813026556]], [[4.7930155507148298, 1.7489559107739903], [0.75165729613538834, 1.3786244480137289]], [[1.041367401739334, 3.2486298156674329], [1.1878730082992008, 0.70073052325431351]], [[3.9396396633615716, 0.90192091539410035], [1.0648896754523776, 2.2559680455713291]], [[2.3190944309766532, 0.32226560621286521], [1.1632542104197556, 2.6347016662949927]], [[4.4173919776061261, 3.6898379410071835], [3.258681687599112, 4.046747968484218]]]])+(1.-msk_arg1)*numpy.array([[[[0.8579198516406128, 0.32184726412670883], [3.3961018587054879, 1.1694809776316697]], [[2.8038267852284884, 0.1527106297464724], [1.6286597721209639, 3.6799252547048784]], [[3.0705731861594208, 2.8312388602886855], [0.074971429524355204, 4.2893682991886815]], [[4.5243180977373454, 1.0117021741767613], [2.4808034649512525, 4.8240271971988591]], [[1.2906587326520418, 3.648860536247716], [3.2940742221919832, 3.3905825147329463]], [[2.1004314526880461, 3.0366186790229137], [4.3289889437166265, 2.3577450995406828]]], [[[2.6712120998408806, 0.62451185445275048], [1.2020590910797488, 2.5614179625970124]], [[1.5643603646170112, 4.7858513749339835], [0.27364698402226706, 1.0079882219240019]], [[4.679541553094273, 3.0179428418677263], [0.72419695611809043, 3.5394203662208223]], [[1.1848020715189023, 4.5924860930511668], [4.3068987544304678, 2.9297468521251595]], [[2.3094102153909692, 2.6656031826005706], [0.31943089246404693, 4.1982538728897563]], [[3.7861204213694801, 4.9301478066201296], [4.201230655394256, 0.82916671034705425]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.15883756403575, 1.0401266550686981], [1.165357031853403, 1.1525969673601306]], [[1.1750458774346209, 1.1368282730210735], [1.0235498569283956, 1.0631010884271519]], [[1.0372806905521035, 1.0504339746528795], [1.138614048686349, 1.1845527220971002]], [[1.167284181400436, 1.002681859062458], [1.0714000517529063, 1.0866514821537399]], [[1.1512742763297505, 1.0471109923426285], [1.0067398238909833, 1.0422176079363252]], [[1.1164845602685365, 1.0882626603422427], [1.0842121318886604, 1.0319894145045672]]], [[[19.026435244346242, 36.796709073951533], [253.9823694661774, 114.61644674517439]], [[1077.3362385356129, 12.778950698558226], [2.9891232768201097, 7.4507695838891239]], [[4.5586048248756725, 113.57415878266119], [5.6431324737067943, 2.7753928976656703]], [[310.78075298836751, 3.7205716467627119], [4.7175179093921358, 26.746176652697169]], [[29.322383852270889, 1.5991334077077863], [5.4443358280025027, 46.43778357863129]], [[623.31718131599746, 215.9804541697132], [115.24947358484624, 363.25976757895154]]]])+(1.-msk_ref)*numpy.array([[[[1.0296999495160486, 1.0110401341075361], [1.1228342697931104, 1.0407027223859902]], [[1.1003749208830427, 1.0052232301837811], [1.0571332924702026, 1.1337588867425805]], [[1.1104339234051903, 1.1014044148937614], [1.0025608812218851, 1.1575773898536612]], [[1.1668928625406729, 1.0351161527251311], [1.0883156237524263, 1.1788848496572333]], [[1.0450137987655683, 1.1325580162560618], [1.1189329096158493, 1.1226228720327427]], [[1.0742846803691553, 1.1091484094550741], [1.1591430707641852, 1.0837563793429896]]], [[[48.974520173042059, 2.4837274490755306], [5.7609647613405235, 41.735707869144491]], [[9.7658240963324143, 1066.151145862272], [1.4897919318298349, 4.3422443590114801]], [[913.1891875267894, 81.158312678916573], [2.8719095383090956, 173.48071478268972]], [[5.6179437354882111, 804.42251220547075], [530.64540541812289, 71.373155296000661]], [[28.911621838418856, 48.575988796576695], [1.5925434311651954, 452.96963618823531]], [[248.50124293433569, 1315.5562439603377], [454.93817753676791, 3.3464239951606118]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[0.33736877441101115, 1.5210954027200598, 0.11294994487990541, 4.079064175559151, 4.0708333595631183], [3.2973082517430048, 1.8146802714049572, 2.6868997078819765, 2.9584820696295098, 0.62977396229267357], [2.6277436162132166, 0.85564310894736595, 2.5007955100757759, 0.36363843421523123, 2.3882884939685258], [4.9731776559948448, 4.863195664195529, 0.85274349206287059, 3.258965796274365, 0.052525196088163056]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.9237436370778025, 1.1300356576154427, 3.9030209524650319, 0.94470247244102978, 4.3981048795265512], [4.1533663843724584, 1.8307804882452772, 2.5276139019352692, 2.639750611406448, 3.4629800680903453], [3.0031217928096927, 4.0259063808226436, 3.9980400237780094, 4.5107084257127319, 3.0715287675100109], [4.2799269558342061, 3.3860922825886526, 4.9045314797401813, 4.832494739343975, 2.8689187424243685]])+(1.-msk_arg1)*numpy.array([[4.9391860351511649, 3.0401964979338167, 0.65999102251018693, 3.6014276153731939, 0.75864713882807744], [3.3252985914046156, 4.05672980074179, 3.1347487096637585, 0.33194398550915583, 0.34888860298499286], [2.0056099453011087, 2.8939591344036542, 2.9489914636822845, 4.9988099311956891, 2.5806420327750068], [1.7723423028853897, 3.4262431097248851, 3.0061635280398278, 1.0929334502021071, 1.6800414197417639]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.041715711531217149, 1.6063615045719262, 0.00020109177364466775, 3.7739665792511516, 480.23212177839241], [141.94054041053795, 2.9771876304712581, 12.16135449108579, 17.518806927852353, 0.20164182622675467], [18.199473330687372, 0.53384671660166283, 39.042039814750048, 0.010430604316246844, 14.497878001687162], [958.39054945467308, 211.82655993300529, 0.45782207770818878, 301.61674320083523, 0.00021322536210855768]])+(1.-msk_ref)*numpy.array([[0.0046689823292115394, 3.5792448830720272, 0.23709002262158221, 158.08460657000188, 2.9009117020927859], [52.848810871175296, 11.217140253464182, 22.16131098652712, 1.4334036363916178, 0.85101620674557255], [6.942562801507342, 0.63688015467176884, 14.925509234342325, 0.0063660746737406251, 9.4559823495219693], [17.166187872701567, 225.71524333714868, 0.61948220900769968, 3.6371572570121735, 0.0070821432081704687]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_expandedData_rank2(self):
      arg0=numpy.array([[[2.6502860555291394, 0.34696466219128308], [2.6655470075644039, 3.7839752483991194], [3.5782074989034558, 4.8665923865853697], [0.86491805987399173, 3.6037767557953471], [1.8572768790277681, 1.2335845001953192]], [[4.5574853398344022, 4.723102098024321], [3.3028729390936911, 1.6191219782715605], [1.5984372106112787, 2.9704574273234226], [4.9790273095101609, 4.3717022371711707], [4.3060452904203377, 3.8418930184962825]], [[0.34376792702396758, 4.4141156506490695], [1.4101360756768497, 2.5571583354761929], [2.3399017272567151, 0.26588384391830588], [0.30759216097321296, 0.17296320346599711], [3.0147716443392283, 1.3653414335692471]], [[4.3523711944173318, 1.4526482349019312], [1.3394330035175588, 0.38123084559670806], [2.4409816423578987, 0.34015053489332342], [3.837041074486979, 3.133302870811868], [1.1837435658498363, 4.413789131721825]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.7495994791447438, 1.5060286666979066, 0.6521275317466545, 4.5544527458873203, 4.429611263156553], [3.2585132426626195, 0.65598896111542737, 4.0102620451858639, 0.6612207034438935, 0.14091603493547517], [3.0292690371475159, 4.5566916394591388, 3.701772388252996, 0.052247451250636012, 0.69456597600736791], [2.6358052905492531, 3.399558072915478, 0.5046068931880292, 4.2554748013426655, 2.91989002024164]])+(1.-msk_arg1)*numpy.array([[2.0445551409555986, 1.2922845084793237, 4.6763659696624638, 4.7954734233076373, 2.0937285002261121], [2.44569851933531, 4.0451882441914595, 3.0151675261367075, 2.0324154267907941, 1.9179026083900759], [1.0448860447697219, 1.3431945486212542, 2.7147315443004572, 1.1704765614486803, 1.1058906744241082], [1.665787289069516, 4.219692715051397, 1.1123635200233137, 4.7832634785348782, 4.0629368049579062]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[38.65256157423655, 0.018890949778565213], [4.3777044983375877, 7.4200475436561799], [2.2964686970759307, 2.8064565641524508], [0.51636340790024338, 343.34241245106642], [15.524550964921989, 2.5342111859281311]], [[140.10868249152077, 157.39056537543632], [2.1897137053965317, 1.3717819573444046], [6.559529018449612, 78.730944978598728], [2.8904561298023994, 2.6522314654423447], [1.2284340370120599, 1.2088482922866652]], [[0.039375220543645277, 89.826623889853934], [4.7878050353255155, 72.115021406442423], [23.264062082668847, 0.0074189174944359572], [0.94026011156475098, 0.91239946962372109], [2.1521563324506143, 1.2414635831496836]], [[48.256650972230865, 2.6756136340356287], [2.7006934777194092, 0.037689848738233743], [1.5688005212326213, 0.58033403055539312], [305.62028016443298, 129.04037458760135], [1.6364556013036562, 76.344701283024179]]])+(1.-msk_ref)*numpy.array([[[7.3357643403054373, 0.11483858534521971], [3.5500714885211218, 5.5830770049296436], [388.27783867136498, 1635.7445209351179], [0.49861475028017682, 467.64642178535121], [3.6555666785057674, 1.5519686079194865]], [[40.836023280937845, 44.561191195033629], [125.60741546297172, 7.0238503238188619], [4.113166607520566, 26.64658496235764], [26.114833031686153, 20.047864092945524], [16.447541396463944, 13.216026259669691]], [[0.32768015565904557, 4.7183286846561376], [1.5866699360192082, 3.5293750561031518], [10.052461528016051, 0.027427933366593455], [0.25158633393310753, 0.12824566410935861], [3.3884649350563398, 1.4111138650503292]], [[11.587280783237313, 1.8626224547750403], [3.4321605963370607, 0.017090005060157083], [2.6984391318789638, 0.30133396146455149], [621.45384931602632, 235.7816775548724], [1.9844544144214975, 416.70499325219129]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_expandedData_rank3(self):
      arg0=numpy.array([[4.030290016143069, 4.6374811350236769, 1.8068715515841749, 4.1674724219708299, 4.2156716112557575], [4.568658018828085, 0.50381752160422066, 3.2022890485028488, 4.6519217163705333, 4.5629132189848658], [0.95276601503865199, 0.025219891710767696, 3.1518991288705829, 2.3088175807562124, 3.2561552740150144], [3.3236965362383382, 0.93447614199160001, 1.4956969018327433, 0.27667793872901852, 0.42351983939233939]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.0272780744330587, 4.4370663201639902], [1.6715354578308541, 1.7512854672855267], [0.32771182100060076, 1.3368929173568185], [2.1207680175383419, 4.1652850985593686], [0.62019629732059289, 1.5258927146805774]], [[0.93796822743199737, 0.49655520068412601], [4.8851323849589301, 1.6173227288319132], [3.5921954855643317, 3.3137750244348192], [2.4640313968905621, 0.076988664796253672], [3.0064309090944645, 1.8587058349750789]], [[4.4444035825849744, 1.4085349438718391], [0.94126808807487905, 1.7856374124581782], [4.3615834868391063, 4.0753422234880068], [0.64176497636248464, 3.8449919981164129], [1.8930239384426573, 4.2913130921924907]], [[2.7380423542155854, 3.4719451139874535], [1.8563650842864181, 2.075493186677972], [4.5532271520880299, 4.1091782973042559], [4.4106597627184003, 1.0391478872525513], [2.7829596639929286, 3.6927273179660682]]])+(1.-msk_arg1)*numpy.array([[[1.2503252338343933, 1.7989068365445176], [2.1870597593766612, 4.8799256412656078], [4.4508153065502185, 2.697409451242204], [2.863279106097175, 3.9602157186221572], [2.9034053499332506, 2.8680778245542933]], [[1.7227524742017897, 1.3509306990694998], [2.4538624425553333, 0.33862232400096604], [1.7391364904518838, 4.8197894463862054], [2.4368198679106285, 1.9685666714410164], [4.0803182127753646, 0.45523939462962998]], [[0.7431918749090437, 0.5441278331097994], [0.7235430313774559, 4.0194480682940386], [3.0840250863591279, 2.9447389785857347], [1.7529281671649095, 0.72084771762550548], [2.5712635913685662, 3.7768194079401902]], [[1.702579779110805, 1.7720876991883068], [0.73900129988500107, 1.8276945396821682], [1.8538669331740554, 1.1538428681491297], [4.0566575080681266, 1.3102935631118013], [3.2323360353114743, 2.9500401992507963]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[274.06754116494966, 485.19610229311178], [12.993143907857846, 14.684198844282122], [1.2139424736028774, 2.2053842116136564], [20.635085394256635, 381.89736722424408], [2.4408441997244128, 8.984205080474295]], [[4.1577731580696025, 2.126285051000159], [0.035120888607369241, 0.32997405063172697], [65.420375778212488, 47.313217239153246], [44.163876783509146, 1.125641579419725], [95.932586523392942, 16.801085453222324]], [[0.80650385537542124, 0.93411728733977828], [0.031304869237758007, 0.0013998867224633779], [149.47368972526763, 107.61003859963857], [1.7108452618763841, 24.959172579542361], [9.3446500102370038, 158.55434082425614]], [[26.805344236836945, 64.720333694328673], [0.88178735648376039, 0.8687894467006132], [6.2532040409230651, 5.2295401559345533], [0.0034573195301696035, 0.26310494665712253], [0.091538696208966139, 0.041893489649035297]]])+(1.-msk_ref)*numpy.array([[[5.7130397953868322, 12.272798565665388], [28.654899634699316, 1784.0514595941158], [13.916687371310287, 4.9321704148856149], [59.548201515924333, 284.99008250260283], [65.199089724032831, 61.967865296542278]], [[13.697853921880682, 7.7862571579610762], [0.18596000628355458, 0.79283548892591371], [7.5694591993892617, 273.03186058350178], [42.354532377130035, 20.619535940398102], [489.68634345709717, 1.9957816451835622]], [[0.96467885082528715, 0.97401543222246767], [0.069757746536739496, 3.7660827891575975e-07], [34.483354907015837, 29.387680897041029], [4.3350678766419231, 1.8278837784854809], [20.81133661122357, 86.3763000890917]], [[7.7286205884027774, 8.4015381473999113], [0.95115188907746584, 0.883502316656264], [2.1092922526617883, 1.5912632541345528], [0.0054485606993094349, 0.18570466798353599], [0.062220085899274372, 0.079298065085393596]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[2.3355707568606783, 3.9747737186340752, 0.028397526792414238, 0.8459378092414036, 2.7592739324921762], [4.7496160178787532, 0.70845742054957062, 0.90847168138135825, 4.1042001049492258, 1.8090298865338104], [1.5566571032161198, 2.5546516886079051, 0.38504794004727266, 2.7265014534133214, 4.0322538313855079], [3.3249462546725592, 1.593175410764865, 0.80429933126821029, 4.3267597107250619, 0.071983884552361557]], [[4.5802123677278184, 3.9797286526538231, 1.0853511863281557, 4.1413124443928178, 1.0123969260762871], [2.6838861989936844, 0.45701638081946688, 3.2111848325647703, 3.2909482840085689, 4.3089937235321916], [4.1485725277193568, 4.8379724932200761, 4.3307908617046742, 2.0727091085184108, 1.0811379484476538], [4.8059119676921549, 1.0646422341915533, 3.479021077005422, 2.9211258736784722, 0.96457194715026484]], [[4.9953560601336342, 1.6240882519772188, 3.3444827946152031, 3.4956182012170847, 1.1920616922239438], [1.1463636612988901, 2.6558484765653425, 4.0262461921490988, 3.4540306717258469, 2.7938899422788239], [4.6528028086921784, 2.658199972725515, 1.1110771615037438, 3.6651893267519573, 2.5913499334981296], [3.8693768953322119, 1.2961795086986891, 1.5683690757147435, 1.4072827379787314, 1.3386391039540912]], [[2.2609054559667139, 4.2392436813384293, 1.3793506822098345, 3.5795486603373412, 0.08970449593481182], [1.5288787927740171, 3.4107264081791051, 3.9202320920283147, 2.9620615597151825, 0.28417967186118215], [2.097005622395566, 0.70988166811066189, 4.7038223206636687, 1.0677461582674688, 1.4448182664190881], [4.7562686822103704, 2.1840796143525045, 3.5211099018657546, 2.0507466347508143, 3.2627576457150069]], [[2.5656649667369016, 3.7292896134287261, 4.2164645830927183, 4.6589717798730517, 4.5103708708333681], [1.4115708308851074, 2.8083641299949993, 1.296547465161328, 1.95005885991914, 3.7850167210669765], [0.32856105374358341, 0.25854177195976036, 3.0569883043821577, 2.2173224740297766, 3.1425139612476589], [2.4120671838376477, 1.1243313925581311, 1.517178802935093, 1.011195061020721, 2.1652098804237387]]], [[[0.41202008102698595, 1.0351047462333103, 1.7634013234618102, 3.8929483987681262, 0.41279379856855836], [3.8494501237563226, 4.9109502198935555, 4.3637821600048223, 1.177825839072095, 2.8980137295170652], [2.0107584677883859, 2.0584657806352991, 3.4243815650407239, 1.0218977791197856, 0.27531798283179298], [4.2494697782641957, 1.9032897209954736, 4.5942823073021088, 3.6009095278578482, 2.9983724280268746]], [[3.7258470070913705, 3.5229262150117546, 1.1456386680478881, 4.3303120774714845, 0.10394659053644913], [1.3830800026631473, 1.029354303582164, 2.167911873744512, 4.5943966459484287, 3.1782332459833302], [1.9722472753002744, 4.4648638540855838, 3.1601684840160078, 3.0758445424660987, 3.6148014069916798], [3.3296591333063619, 3.1931942939590425, 4.7025971978482568, 3.9826845321986584, 1.2424776736674725]], [[2.3642714671917107, 0.438857274171912, 1.3444938712761945, 0.81181819076190231, 0.48069111828791972], [1.9232100652423354, 1.887700842715939, 1.6608188806421835, 2.7192827893037395, 0.58715873916340255], [2.0608484745534632, 3.1170731712092357, 1.9891822024129786, 0.043878451864835522, 0.44643976731241741], [2.0244342631805048, 3.635178191485398, 0.67840233824710561, 1.9973211344782926, 4.1037735553607675]], [[3.8995970983191923, 2.7279244484214367, 3.7120553773487153, 4.2285255004152473, 3.8431525183374409], [0.20200349627152622, 2.3787788507988799, 3.3122468285805371, 1.0362586821700468, 1.8118402372416589], [0.95727901262064485, 3.6082104880550743, 3.9220847802632095, 4.5038615917293532, 1.069771304122336], [3.0035067788814409, 3.9881023252028931, 3.1332283608060618, 3.9464569006564902, 0.22821379448930793]], [[2.4532669234836071, 2.7427813827268692, 0.94028758082274755, 1.078143408280275, 2.4077010013952713], [1.4994619818379669, 4.8303932955726543, 1.3934041659382188, 3.9857478998758316, 4.2090596569393508], [4.2223403087960767, 2.6345764172533412, 3.9680691260423226, 3.3050168455073305, 3.9113029560864425], [1.9050219534209447, 1.3814705340899729, 2.3129325738328963, 3.8903933304216376, 3.27927681871983]]], [[[0.42968449020600563, 3.2722736919984317, 3.663091498044464, 0.012982514528004705, 3.118459359727038], [3.400559354016945, 3.1245535193550902, 3.6592288636892287, 0.17046441271924281, 0.29544698052275792], [1.1313971929116979, 4.4808348487547489, 0.60020248715739866, 1.3113195236780844, 1.8844277660697797], [0.48049281189539161, 0.9428849251652659, 1.4938233242934875, 0.46574975292198767, 4.9780171304317289]], [[2.4653445630223101, 0.77174963718073919, 1.8593755745339005, 0.92125905628117644, 0.64263213436163258], [0.49578297299872837, 3.2215163169638181, 3.3058137610990563, 2.1705013388832879, 4.1610226350148292], [4.1464874974692414, 4.7965993603374457, 2.6269763774158279, 2.1295055627274819, 2.3693132070574507], [3.8741501360136947, 2.7686242477806879, 4.8476297478525998, 0.63744954979903179, 4.8214363988224136]], [[4.102462585282094, 0.022186417188353035, 2.4196633082179795, 3.0190497435562302, 1.0470967539830742], [2.6525555456823571, 4.0597800017473515, 4.2691076331345625, 3.8974215658440809, 1.8102136645504008], [4.7852951073683965, 0.97456512934562223, 1.543618085938486, 2.944597126290216, 2.0986123271080519], [3.9122021440861396, 2.6295053593775459, 3.7286625584538409, 0.65740229869046285, 0.38348880084308279]], [[4.3947375882776747, 2.33204120080753, 1.202270384983801, 0.74808612592268209, 0.51489454734620621], [1.1795312080499094, 0.72177475293946625, 3.1529387948408703, 2.540898822119519, 1.9237679011651756], [2.3817028606804924, 2.1816238085041033, 0.76500917681184011, 1.7692990860672022, 2.4455584863869611], [3.7776980024253062, 2.6975445930117696, 1.5681986357848545, 1.783627134880259, 3.0007563269585096]], [[3.9920432597591904, 0.96954241245489392, 2.791075952078415, 3.7046957766445887, 4.955495299847251], [3.3805431594793371, 4.6027994638363685, 1.6570591172307627, 3.0295271260950463, 4.1373788223584151], [3.2389782276615575, 4.0594937780896867, 2.1422335284182834, 4.1760467101049672, 2.5304371187280119], [4.0519528585359783, 1.8567502611409232, 2.3830176677495047, 2.6667969953200048, 0.20532016906775669]]], [[[1.091963260531307, 2.5003686683462165, 1.8508408757118482, 1.8527235378792568, 3.6611689367934859], [2.9820264518735438, 0.76132005605908937, 2.0039663207206666, 4.9111402610721822, 0.95741971423894545], [0.94055336529444489, 2.8103718592185789, 0.73267163528360235, 3.4652154590340998, 1.4209888500878092], [3.9825161240002123, 2.8350269694022403, 0.90798704704881794, 3.1273332698421652, 3.1325028985279091]], [[0.6584419667197029, 4.334217018329813, 1.9661930913992642, 0.033658830706255194, 1.0908947263590261], [3.8190138060599774, 1.4673991805634128, 0.62534980604558466, 3.0463915883838206, 4.5096389551374445], [4.9512766727521464, 2.386163451444395, 2.5953056946867683, 2.5352620071495888, 0.64440028172605057], [3.7746356781379102, 3.909760006422569, 3.3903217975840056, 2.0869439573634572, 3.9893590844515168]], [[3.4440356529172806, 4.5726660029392372, 4.507013062571791, 3.1341283797836077, 0.97713177412273522], [4.8099770463522633, 2.0479553112711635, 0.10304430813719355, 3.8557901332877509, 0.90807749043100039], [1.0386489744903582, 4.112589105431276, 3.9099468226640948, 2.1404854265773401, 4.9718909459028131], [2.0219420115340254, 0.74894703986955613, 3.9517737630460794, 1.4851196794169832, 2.084425884506699]], [[1.7960818501514302, 1.4176506977756573, 1.3839653293363485, 3.9036924620981424, 1.4015573861688935], [2.6764469451533888, 3.1093771520552473, 0.63041655466955648, 1.9620787323622439, 3.8621711324073877], [0.93771302303176618, 4.4136881074026721, 4.8978274931697312, 3.1745113453093388, 1.1021266281275273], [0.75941502922493287, 4.6001369849739024, 0.93801046666788845, 1.7338266836791525, 2.6304674825153311]], [[1.1036451704633632, 2.0306892189468635, 2.6569334048670545, 1.0827821406742963, 0.5160047156400146], [1.6053804624895434, 1.2468901052047288, 2.8763689871190774, 2.3030995168485462, 2.7763645169516464], [3.5395858380062633, 2.956665644394108, 1.746887161841677, 4.2643345407978916, 1.9133246400842747], [4.3325866703578857, 2.6964197233107838, 1.1346528015131396, 4.085511539680561, 4.8956652538875876]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.1462986433325009, 1.9551991941352791, 0.88621832524449407, 2.0356927819666359, 2.9340517095225076], [3.4111348052277801, 4.5377602485361779, 1.1563121902099338, 3.9934585539389604, 4.0996155679888222], [0.61822297340682653, 0.21059536837616732, 1.9890618032183176, 3.3154662065236722, 4.9221423310775485], [4.9512499309976921, 2.349321503202586, 1.147799407691819, 4.6885931359191266, 1.3631765716645374]])+(1.-msk_arg1)*numpy.array([[3.3572474051568246, 0.20641367570918268, 3.5424406446602417, 0.1197759798197837, 3.600230802213511], [0.68364188364790957, 2.9290132570725813, 3.2690302525865582, 0.46124215529604384, 4.6767224388305557], [3.0005826303326653, 2.0466471191596161, 0.91589423602816433, 1.2629128122698963, 1.5341399078319591], [3.4680059423829128, 3.7303827840513955, 1.7551603709370478, 0.67970481100498192, 4.3099503542298878]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[33.687401757042871, 305.44163148394716, 3.8622386894053551e-07, 0.49971623203155074, 67.246077409691679], [639.1874257920407, 0.23952790690360759, 0.67165552275461637, 348.84187340591734, 11.68010992088092], [6.2645430765152348, 48.856106606468536, 0.019117037649532181, 63.995798452399342, 324.17706341051098], [145.70514988811837, 6.8967624714922859, 0.4053531659706831, 434.23103468340247, 1.8270742164524491e-05]], [[19.595801074711474, 14.887884991304968, 1.1736726736688963, 16.092655532745479, 1.0243819436624453], [6.8915865884006617, 0.21632109089776771, 9.7865944938625393, 10.267523721192363, 17.39126501839295], [16.147861470883154, 21.809882095434492, 17.563687418808364, 4.1581058282571188, 1.1647811186192507], [21.528189962055592, 1.1302867431137176, 11.446070250555248, 8.1328641058775677, 0.93190378468912805]], [[4.1598951928796346, 1.5369017638176055, 2.9152106254105719, 3.0316629895013443, 1.1684693752822022], [1.1286846211014587, 2.3764940566907291, 3.4361629817935553, 2.9996772843945401, 2.4856435731759756], [3.906078402931676, 2.3783587034448228, 1.0978407995049937, 3.1616411727741052, 2.3252752850477973], [3.3172503842971794, 1.2584788517530792, 1.490080869292074, 1.3536247323433868, 1.2949455911580785]], [[5.2627183524975569, 18.921973523656089, 1.9245746736811635, 13.409859045193468, 0.0073833158296420457], [2.3731593486691831, 12.153813149223314, 16.136171728271119, 9.1205423628164031, 0.077211720745526985], [4.5152102084759118, 0.49780623894960291, 23.383173973139737, 1.1427523887195716, 2.1150986394182492], [23.916976742641616, 4.9050830603962279, 12.967960418952558, 4.3147637597572013, 11.104548440644056]], [[15.871347315573415, 47.553290083098261, 68.175932659878384, 91.368743234621505, 83.079252720942648], [2.7493839346930535, 20.691192243988162, 2.1425307207710031, 7.0960229132465003, 49.668474784342159], [0.038170389452000471, 0.018894471888431495, 26.53854059791054, 10.3437914220541, 28.776439985675747], [13.241899064383876, 1.4103490681313477, 3.3975920665536004, 1.0332037247305348, 9.6466017434850997]]], [[[0.048577447829649699, 1.1248988352060258, 6.9236805901322809, 103.16206767605965, 0.048889322847372386], [99.282766490002516, 227.85512537650646, 152.28571369722141, 1.7477010125273156, 37.695272722204272], [10.83431691200756, 11.736528659789073, 66.608826042451227, 1.0766885589967949, 0.012280052841289645], [139.10236625238991, 8.9831551368710976, 181.51552247466574, 79.067326706037221, 42.337088165205977]], [[390.914858569194, 303.19160829773335, 1.8532932375779545, 773.34183800706217, 3.4555774481635681e-05], [4.3564355018248015, 1.1402929356627682, 33.487006262665929, 1011.6588852847166, 190.01969105462169], [21.800374578181408, 888.53371715173603, 185.16770843847749, 163.78169996988228, 340.76375808611891], [234.70640237915532, 194.11259913834564, 1124.365872163226, 528.99841966083602, 2.6782988306734374]], [[2.7046482467007049, 0.3858460628848438, 1.4081669226691991, 0.78578924771141634, 0.42868460834884153], [2.130215608027838, 2.0848023495261327, 1.7978826048252783, 3.1795443617055432, 0.54026794330616501], [2.3074658844425966, 3.723279779024927, 2.2149352775371263, 0.026916502197464234, 0.39356506464119068], [2.2603863602833045, 4.4477739277771065, 0.63847919842074463, 2.2254178458541212, 5.1171886392272468]], [[229.19905912878585, 55.014653077276762, 188.2485691377392, 316.70843821741454, 216.23489407780031], [0.001682594645208497, 31.838630195892321, 119.42312110266015, 1.1528466575454577, 10.734729948708107], [0.83999787147340665, 168.08229276378981, 234.52300457969724, 407.44102120755241, 1.3090980354814585], [80.796041595077355, 250.68900735318718, 95.658527331819187, 240.39719647577496, 0.0027388291400249451]], [[39.609910739502311, 62.577100495756788, 0.77692509514872277, 1.3613231163057526, 36.679530552030172], [5.26341660527632, 636.89124086561708, 3.8963749371857404, 289.64059511373608, 362.17533990964733], [366.88315647056083, 53.058204027237032, 284.40991119337519, 134.40409173577675, 268.09595604737751], [14.043725603728721, 3.7613757065891411, 31.112018708123394, 262.26877447143966, 130.16428654873744]]], [[[0.59320486347538881, 2.0810947969347104, 2.2314319285971962, 0.068176559074503401, 2.0200639126572422], [2.1311628831126188, 2.0225035321456444, 2.2299769652057102, 0.33494871638168749, 0.47058523480794945], [1.0793097110231753, 2.5274644688094412, 0.7293543894285387, 1.1824155043084301, 1.4795249232666134], [0.63564044905486627, 0.96429469340110574, 1.281609718533018, 0.62351130787886677, 2.697343597919847]], [[1.2092820682144678, 0.94689769237900678, 1.1395344737492883, 0.98287653235689199, 0.91108265552752876], [0.8626404600618659, 1.2793670269740869, 1.286345472829737, 1.1772751836432858, 1.3502079742556437], [1.3492133290899675, 1.3912379946853919, 1.2255626756564135, 1.1725570720103506, 1.1992059045414096], [1.3300477582585488, 1.2391924359611783, 1.3943420516244229, 0.90953034916605724, 1.3927520119687562]], [[16.572333145095421, 0.00051317462561331173, 5.7984549954281484, 9.0051639086302746, 1.0958598279552403], [6.9613721017464218, 16.231142094482308, 17.938224605660313, 14.965551596569604, 3.2556715023728184], [22.510258172620411, 0.95004488664002285, 2.3714688841675842, 8.5688288874483529, 4.3686080442310269], [15.078653203195238, 6.8415649700277932, 13.704222572480434, 0.43416522445122741, 0.14861353555031606]], [[135.40118432477354, 16.565975199642914, 1.8418082880556879, 0.38202412579313999, 0.11071634533813958], [1.7288217255691267, 0.33926228083675081, 45.027156356299173, 22.015165043763705, 8.7518242627648544], [17.76470620667315, 13.280462266155206, 0.41143448724223025, 6.6309641639065893, 19.393415937229509], [81.993116910608791, 26.844909107851578, 4.4447327180724061, 6.8106749633721879, 38.215747566614048]], [[910.26644665226854, 0.8587760745612053, 156.36931456276869, 630.20377960470864, 2638.2593376891991], [401.55660165307557, 1834.3783565704273, 12.011932065028333, 234.09681451872879, 1085.4495487929648], [325.31265955484349, 988.519403294065, 42.518069985649738, 1136.3062505549674, 96.513014862047669], [979.51386474469859, 21.030177962660961, 71.824739844757033, 124.96350579030394, 0.00041275328446913523]]], [[[1.5458860894288402, 93.458222362077535, 21.07711428107919, 21.183480093995314, 617.47921310120682], [223.5759010047332, 0.25918505643865358, 31.241718370674917, 2643.7263621281236, 0.80618198652464801], [0.73826922891911517, 166.70233192762441, 0.21435448255685693, 470.26139674462632, 5.6952729837087972], [936.54789569355569, 174.06993968339953, 0.62007166386653023, 282.96391177225922, 285.28744771301467]], [[0.37466167328911826, 31.35507853272177, 4.8957841863545957, 0.00034648310816217667, 1.2267725332645898], [23.291128672602024, 2.4619285077119568, 0.33191541732597718, 13.695194756995235, 34.41829354621138], [42.866107111470072, 7.7150576688360104, 9.398574659076111, 8.8956884655573987, 0.35616023813478997], [22.660262551401654, 24.612229872966093, 17.607852830955768, 5.6316228413728284, 25.80564049286254]], [[4.1347182171407226, 5.7245603606357989, 5.6303215279209109, 3.7105864005978719, 0.97379651330219563], [6.0668510961902706, 2.2768449538490776, 0.07364606392289462, 4.7069609921615996, 0.89522767900587097], [1.0444866056781237, 5.0685204298367195, 4.7829225876342338, 2.3953103652886822, 6.3018356580669783], [2.2436810927770927, 0.71762086399751002, 4.8416967910925814, 1.5745182324685907, 2.3234453781819369]], [[15.57513143908956, 5.1362538457949052, 4.5885840694675677, 593.18262256671028, 4.8685404934167513], [101.07610532410406, 204.14800282966942, 0.11495698995452971, 23.573726796047136, 564.17547825940687], [0.73968591307019282, 1054.9030450289274, 1718.484432483672, 224.98770856161264, 1.577634152587192], [0.27517915983386959, 1280.7443748482071, 0.74078663772711262, 13.200804590954883, 93.188763133666455]], [[1.1438896387537256, 2.6264694967173066, 3.7888358112995908, 1.1145141748405187, 0.40578521174131965], [1.9065099529532294, 1.3509234110852706, 4.2216886941868337, 3.1181441608273133, 4.0228770064315391], [5.6016905942131849, 4.383152179944271, 2.1391920303383811, 7.2210190772433052, 2.4217410619751223], [7.3790243904370421, 3.8658003477529981, 1.1879220461484297, 6.8114069570527951, 8.7163601904016197]]]])+(1.-msk_ref)*numpy.array([[[[17.249898357320049, 102.81110619384454, 6.4162316087769104e-06, 0.57023931131257799, 30.189593074666753], [186.94333892023084, 0.31438820024780906, 0.7245044147737395, 114.48797701343882, 7.31659555033497], [4.4181383229884661, 23.308443676673207, 0.040594823015984813, 29.002552296456429, 107.88821121839882], [56.462498745347482, 4.7758307659580144, 0.48135317892159507, 136.69600821665824, 0.00014569884423945849]], [[1.3690390281320746, 1.3298968518373742, 1.0170497404999952, 1.3408670616041225, 1.0025464003264304], [1.2260347318097999, 0.85075648056678432, 1.2722798462389455, 1.2787396777390221, 1.3518977752624024], [1.3413519322674816, 1.3845985993479497, 1.3533065319243125, 1.162352477283757, 1.0162335409587226], [1.3826996446458262, 1.0130134549989145, 1.2934931762479651, 1.2476591121293943, 0.99258213161454023]], [[298.28415454330985, 5.5727825243370601, 72.01189518818812, 84.217332331652329, 1.863304053598849], [1.6223558508920943, 31.821085409685505, 138.93878754134218, 80.721379409378699, 38.077632674317414], [231.92069542130406, 31.921004234115617, 1.4522660553549929, 99.604294027603771, 29.166984817599356], [120.69338939514761, 2.506741849818229, 4.9245199073104358, 3.3545337257419741, 2.8099458877669141]], [[1.1026419769942906, 1.1888692125397893, 1.0392730687719034, 1.1650252115887094, 0.74915605835792676], [1.0521640611932068, 1.1583032038659344, 1.1777808800019967, 1.1389001313052278, 0.86010914883277545], [1.0927477138900139, 0.95978874462740704, 1.2037698633328902, 1.0078822218315751, 1.0450613479189008], [1.2053696280530366, 1.0980856416946787, 1.1627305472943139, 1.089832026779644, 1.1521661839941537]], [[29.731323539320556, 114.28413206047595, 177.81102430599589, 254.68216341252057, 226.62880413760064], [3.4591084889216108, 41.165728600693086, 2.5472076207347349, 11.072322738968225, 120.55285722377913], [0.018184707993081726, 0.0076731732899983501, 55.868833630278395, 17.581815163913785, 61.703919188049731], [23.806201038778521, 1.5248648693749101, 4.4851207799987405, 1.0408949437288975, 16.139030225508485]]], [[[0.54543383878004248, 1.0238678238507566, 1.4737233773869232, 2.5324402426819219, 0.54613385142485471], [2.5130612167042328, 2.968313082211461, 2.7380236135943603, 1.1183918710930716, 2.0697181793482007], [1.6120909419563829, 1.6381421036191623, 2.3198651898689584, 1.0149188775026678, 0.41404491603108395], [2.6887842071078989, 1.5526774933415779, 2.836088251288146, 2.4009697660832612, 2.1184535853827637]], [[47.111360867888052, 39.98413825045224, 1.48919438397826, 73.176757920046512, 0.0013189374051101853], [2.5854944810250395, 1.0884355385873008, 9.6442924591080583, 87.031409150078545, 29.573911368252812], [7.310477548431173, 80.038122518620767, 29.084252782674728, 26.869163333597758, 43.115828033901693], [33.893457111515211, 29.983527040936277, 93.172183426991197, 57.269473339252983, 1.8887415766520388]], [[16.658193910227538, 0.067724226152173897, 2.6318594591684135, 0.50584563626932433, 0.091203320277038896], [8.4818766202941909, 7.9805694150726785, 5.2509777370373047, 26.317491536496988, 0.17541019416823569], [10.632271904574292, 41.121966113825565, 9.4705715217305357, 3.6431614011605995e-05, 0.071624934299072962], [10.030347822237159, 67.979111585176341, 0.2812723255125239, 9.5978347048657753, 101.04505103864751]], [[1.8732822499443536, 1.5886354407680254, 1.8311764524157512, 1.9445752002135603, 1.8607266511740341], [0.47819192216902306, 1.4913870295999645, 1.7374107803373309, 1.0165636534177938, 1.3153937654207974], [0.98006340725137731, 1.8073676339324212, 1.8782571457353081, 2.0019856978005439, 1.0315973507919316], [1.6607430838330619, 1.8927739042713401, 1.6934502467253001, 1.8836316090146021, 0.50587152175049865]], [[66.485703375291124, 112.02077796974949, 0.74980393610463758, 1.4217363599698551, 60.904477838956872], [6.649688605130299, 1580.5064491178944, 4.7185507254270673, 643.30626024033484, 830.11904056385993], [842.43973502683093, 92.799927889422577, 630.07016094494281, 267.93607233127227, 589.01051147794851], [20.371102768677765, 4.5325106369263617, 50.476489675102265, 574.42839066462852, 258.31576710594379]]], [[[0.079293081388786338, 35.062980433046022, 49.189432839076787, 2.1826155118339816e-06, 30.346460955053953], [39.351453285377666, 30.524754679308426, 49.033959805676666, 0.0049482713400963623, 0.025770933651767098], [1.4483600207831404, 90.044318070963286, 0.21615446044116995, 2.2552472525158702, 6.6942019824349517], [0.11088561929181708, 0.83822613152753722, 3.3342584500329018, 0.10098678707022389, 123.47393431611877]], [[6.3392110998010507, 0.58844241308670797, 3.5587660877020029, 0.84547749594364974, 0.40454503708852546], [0.23788630802509059, 10.960243978274812, 11.555256944462649, 4.8844950438707615, 18.504785545379832], [18.372731565461979, 24.753150020545451, 7.2190285074163265, 4.6975435719620195, 5.8441319077329599], [15.987834019210117, 8.0381920581258086, 25.295126599338055, 0.39789602292076481, 25.016186001330052]], [[3.6432086304885596, 0.030562633730876598, 2.2463578242720494, 2.7511255687043215, 1.0430516323484362], [2.4436098194485458, 3.6084769234850853, 3.7785234398951788, 3.4760787016730585, 1.7220798055078665], [4.194927726948789, 0.97667919921375101, 1.4882728856161576, 2.6889213758423911, 1.9717682276765538], [3.4881507174625814, 2.4241541438288148, 3.3379680326854868, 0.68100870429127969, 0.41568225368624862]], [[6.4858702716139875, 2.9135252245520284, 1.2619313788761262, 0.69312547500131438, 0.43243948190462245], [1.2318640051857168, 0.66248137952081221, 4.264167398653341, 3.2468613301988833, 2.2848624095837597], [2.9921003257311161, 2.6782396262849142, 0.7129861855711227, 2.0556604367611677, 3.0937671223907612], [5.3578144917204398, 3.5016747773501433, 1.7651211428057652, 2.0767065408571161, 4.0059064584813671]], [[8.3621433514949679, 0.95365570273274469, 4.8292099361420293, 7.4566882256143039, 11.650935856695007], [6.479466924433118, 10.403239398615689, 2.170177133134628, 5.4764151229528739, 8.8337046579399736], [6.0678847872046875, 8.5798756597872625, 3.2180736613413861, 8.9606784538418385, 4.154879025389751], [8.5554366944210116, 2.5840770561582742, 3.7893610197008814, 4.503272147291006, 0.088140172969393638]]], [[[1.356768223082748, 24.003822367789844, 8.4574135309438088, 8.4872856324405372, 90.081727880594457], [44.218975094850308, 0.38839585975101476, 11.141861736915144, 249.4735196708078, 0.85992958823343546], [0.80852530621969054, 36.001076233801129, 0.34002035937864916, 74.436799308572148, 3.3820914852590072], [120.60072672263775, 37.108298408960401, 0.71551705145366951, 52.151517138044468, 52.451100493267901]], [[0.21037824084341566, 237.64054251246316, 12.454835441016836, 3.2026948328113561e-06, 1.3833894660023329], [148.21796408904399, 4.1810831775045152, 0.17356446814004595, 63.783147893638073, 275.54923499043804], [390.44607729425456, 25.6428495335925, 35.081788273170943, 32.148438057170296, 0.19412335542072254], [141.89422486144048, 161.78808628651328, 95.06083098288758, 15.555985800937034, 174.42096619049548]], [[8.7627381837209786, 14.411305035913445, 14.050110446786926, 7.4261636436212228, 0.96020981764257474], [15.749630442448629, 3.5189845610225525, 0.018522467913873096, 10.683732138919066, 0.84430428221343257], [1.0688219870556972, 11.96384765223635, 10.948504725492722, 3.8027874126961851, 16.69195094223193], [3.4409082930306454, 0.60206247725441775, 11.154902973387449, 2.0020222303062236, 3.6297138556761932]], [[1.488903977848874, 1.2677170720161148, 1.247163765259103, 2.5236481691401953, 1.2579173971639328], [1.9526017032485814, 2.162083598661126, 0.73081242415075232, 1.5811051962196421, 2.5053719046461329], [0.95722891526901122, 2.7433093091485743, 2.9444127399784104, 2.1927655086608757, 1.0683288352197364], [0.82939445770876508, 2.821554720091485, 0.95743528634458375, 1.4536281084922029, 1.9297382640085119]], [[1.5296528392931148, 21.180102273670212, 67.46206257787685, 1.4088681685609656, 0.057749946419434275], [7.6918509289022348, 2.5883004759169497, 94.972404990744565, 36.437600327196193, 81.538296693090857], [232.25111154908103, 106.93852094174578, 11.070047633138483, 518.35548240830315, 16.386837768733759], [555.07168698869032, 71.890700843628807, 1.7236804749267554, 430.96481735633682, 939.84069209824179]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_expandedData_rank4(self):
      arg0=numpy.array([[0.97018672891663804, 1.7102236955929297, 0.78195394699329279, 1.3867699322022029, 2.9493979942665529], [1.372807105007642, 3.4015488477938107, 4.2169228252015545, 0.1280194371964424, 3.6476223327495818], [2.5560679415923113, 3.5816644538808773, 2.5695002685836399, 2.9263535806723158, 1.9782752198200309], [0.16708945623228597, 3.8328882503126556, 1.1554873589521828, 1.6131222775063589, 2.1255251151953622]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.3049680106453474, 0.56397158339569031, 0.84365607394774655, 2.3814501378537929, 4.7575171898864435], [2.9755924480311786, 0.52040562831442549, 0.19408139074685915, 1.3233757039269669, 3.7725693499451554], [4.033920155682595, 0.67932054312397994, 4.3693565244042212, 0.4644337076296513, 0.041260655713720625], [3.8675521691380119, 3.8647764943072618, 1.5494916515230539, 1.8898420413758299, 2.8009632225186603]], [[3.3439872864475282, 3.1516147730497948, 3.8398523718179147, 0.050747002170552787, 2.7197690330263113], [2.0462447131788801, 4.8466913760435002, 1.4541181829147807, 2.3017502869146824, 0.030114768360685253], [2.2828729169671105, 2.0016024317614587, 2.8684871694225214, 0.99534619481478126, 1.7190047066652565], [1.0703968121769549, 4.5943301924652218, 1.4250755425640504, 1.902957867341174, 2.9290850854750032]], [[0.20353716254772072, 0.54843085311392725, 4.7226235441241418, 2.6134176052138276, 1.8369493849765466], [1.1722384197914899, 1.9597234289951004, 4.737825503460952, 0.61108295646750199, 0.82694621510822675], [4.2701400271003527, 4.6645132650836914, 4.5579421384540559, 1.3368874285476753, 2.270714376901029], [2.9333467932957404, 0.24629564889049846, 0.13416698671942165, 2.1562189623493233, 2.0937784311622081]], [[4.5653127545307699, 0.99581963169149679, 2.8318270390425129, 0.96335652623287082, 4.8446522163481909], [2.4719257298585475, 3.331218731549662, 4.1456033346835071, 2.2475367966584376, 0.86888811882078087], [3.2345741989494829, 1.5117388468973691, 4.6570016606211029, 4.9102600958972733, 4.1935979063880167], [1.1441262867531012, 4.3250659775922715, 4.6430376234566673, 0.61416099364374088, 2.8857066208097919]], [[2.8904633880398225, 0.8847982535001514, 3.0297535612168192, 4.7887654912984186, 3.2229121162230712], [1.7938163102651115, 4.6325115485265274, 4.2341527197758637, 1.4903190943621225, 1.4914038984408058], [0.39902210618578998, 0.35068079610336717, 4.3029779233889132, 4.3201536570912076, 1.3922282630732457], [0.80355834287853323, 0.11182418923786336, 0.9482546845062455, 4.5705596231750985, 1.4598326978817073]]], [[[3.0939758009280309, 2.8934099631747054, 3.4853583528280838, 0.83371034812621347, 2.8355467911016334], [2.4258782749919079, 4.6521714649883359, 3.3444455930684298, 0.85767674276598505, 3.0255268650091556], [4.3630413293221455, 0.45215032896048574, 2.5044028868900505, 2.4399632627483769, 4.572219734170349], [3.3323754208332406, 2.9009467307526347, 3.7880863525674022, 1.3837441327288269, 1.0882443778043511]], [[1.5327317216717715, 0.28365545177565626, 0.30437406478247431, 2.9930273378995902, 0.577174450808981], [3.5985674553157776, 2.9465673403185204, 4.8943396029624289, 1.2101098658707272, 1.9582034901726955], [0.068897200349343449, 1.7899864531854439, 4.1009165836196422, 4.4581240543317655, 3.2322836999058246], [2.1720934951511546, 1.8028717272921959, 0.44734766187568109, 4.978462010347795, 1.9227714313662334]], [[1.7688676701859629, 0.97145138031164746, 4.2629045681104962, 1.1392364592704896, 1.0046183145839542], [0.50153248502120584, 4.3342868378816553, 2.3531477745273746, 1.5973188679310857, 0.89127504550624259], [1.3398464518679021, 3.2909452689343337, 3.7362504714194733, 0.013533948844510094, 4.9877423478698857], [3.5909618294918926, 0.30321179640052393, 2.2091363643339825, 1.9616498736590589, 4.7063274805937745]], [[1.7505419351121856, 4.5956269735754045, 1.0154746870701838, 2.7174440140911265, 1.2223292506556618], [4.8661723733938222, 1.9302256384633156, 3.7430007057335524, 3.5619330122813428, 4.4654492816826652], [2.142153837320699, 3.0552330879085177, 1.105452045048726, 4.3110936615098545, 0.38216339328174603], [2.3106693866510049, 1.760671766878021, 2.8442393347026513, 3.5602672997311089, 2.7843557092916695]], [[4.7187575040058469, 4.4710831616411371, 0.93556356552160413, 2.954363656299591, 1.9401475124297332], [3.9919214664778799, 3.359100746917647, 3.9116758090696546, 3.0114900342990247, 3.6263358406580948], [2.1423827266175888, 3.2455282946142683, 3.5484688989558686, 2.7864670537614078, 2.7338281660624286], [0.15978804462210003, 0.51169109406205115, 2.0532691616188639, 0.6499890925567503, 3.0168388262443702]]], [[[4.7610219367064719, 4.8699541182933963, 1.160348196419184, 2.4107692143602089, 0.70674648186303957], [1.8821116243626808, 0.4984196014822953, 3.2347388717536871, 0.88420355160826325, 3.3335019378305524], [4.3490795182628696, 4.6200513343752849, 2.5016406283832024, 4.3974418612755528, 3.3929954828569131], [1.6760101629323831, 4.4726833017126699, 2.6858310845274866, 1.9609370099234928, 1.6551700244709782]], [[4.2053046626341741, 4.8026358506599562, 1.0964488775492984, 3.5489641770039237, 1.9605415520611196], [2.263543024703397, 0.027902013299403221, 1.3918600052776007, 4.6082623279956456, 0.22838206618538645], [2.3624002341274446, 3.9272777645435477, 4.1980522135580838, 3.2932641571609804, 4.6720878907690322], [2.0373531744063054, 0.93196393629339325, 0.61907364567575451, 0.3888078753989489, 0.84276423314472892]], [[0.87622969791361949, 3.5723443214738571, 1.1587716714551783, 2.3747083633516683, 0.98853452612476367], [1.1844099501338687, 1.2757738112060839, 1.7928786531778265, 2.237261394631326, 0.68726245090157811], [4.0482707268703466, 2.3202997116446249, 3.1056640975099237, 1.1734231718805808, 2.8374443045441922], [0.47513796950868931, 4.6400198836661666, 4.0243771885158521, 1.4356498989229445, 2.1111544744502839]], [[0.059079909424089389, 2.6143238323424005, 4.08834170239935, 2.3060977803623359, 3.8571869831120522], [0.13576826814637113, 0.019988069664584163, 1.5110602008086291, 0.4497088021540453, 2.0627079585581374], [2.2298145099420692, 3.5031142981649137, 2.5314572379585418, 4.6644410971863524, 0.59836490588460711], [0.9266408655729691, 1.1152982455139848, 1.6901097479005576, 1.2196576814482558, 0.013884846249335248]], [[3.3695950506345915, 0.85194172640825883, 3.5966446489522248, 3.213206198377212, 4.2082193604187319], [3.6428658451253448, 4.9226773170040552, 1.009550219333655, 1.2668426283226217, 2.1309416230342704], [2.6140717754091281, 3.9688684042824476, 1.4377977465093692, 2.677289877084386, 1.5653779408380799], [2.5503467988172486, 3.7352570279875588, 0.7766312831818879, 2.1190339118936325, 0.83050098722053967]]], [[[0.39012462810075688, 1.7437424567365387, 3.7731366765254442, 4.3454068288628607, 0.2255890499807594], [4.9432175646055416, 2.3764494877183413, 4.3913489731048161, 0.23600656197706799, 1.4909587309998324], [4.1557059334838957, 1.211515570264434, 0.6072141869664669, 3.103216272290076, 2.7598370577127298], [1.8305267697714236, 3.8164334343397726, 0.33084203838849557, 3.1313813188936992, 3.3139570250185622]], [[2.4615198855870379, 2.6453889620676074, 1.9496798286456516, 3.6829328889083799, 2.1481973396986551], [3.8473376493678977, 0.42560888489530985, 0.88708001815696924, 2.4457778751014376, 4.1972306979662015], [3.9471911551048153, 3.2415447475149914, 1.8866060422013706, 3.0086208335621496, 3.2664693349454232], [4.8003644056962989, 1.7528776583159666, 2.1498517986396375, 2.9398314417517546, 1.1525499701337245]], [[3.4330699551788446, 4.4709949334122312, 4.8584506837379138, 4.7644401348529133, 0.25905960271691858], [3.1563166919644696, 3.3477260039171708, 3.8758703682916806, 2.0049942519530979, 3.8472059911140715], [1.4996464905225317, 0.33656460598772092, 4.3193407410671343, 4.7983770535837369, 2.4874908630363186], [1.9158462290790779, 2.811255049025156, 2.4776996534687128, 0.94329547588417761, 0.92728245010237331]], [[0.94459147887866735, 1.4691132061854646, 4.5136331113575237, 1.6091203779961383, 4.683535661661324], [2.547244506680864, 0.62352972434427378, 1.0605770204209637, 3.3205279213058936, 1.1485225591587207], [1.4888013195671708, 2.6677110203203807, 4.0422417450674484, 1.4002770189664675, 4.8239575327574249], [0.62298983852478218, 2.5807147898565113, 0.17216341679122377, 1.0197126329659461, 2.6278621466925807]], [[0.083937644803915551, 4.179289836393238, 2.0485014564597268, 1.9260176548575341, 2.5078848288198574], [4.7903392511175715, 2.0052725059855017, 3.1205728903813137, 2.3455496352120817, 4.6906865701890199], [0.67555326993815079, 4.8660561656048591, 3.8667481868273095, 1.334053663432339, 1.3191298991256686], [3.6122720729554207, 0.97111850438363556, 1.1125394666926667, 3.9065915120921075, 2.6917314000117205]]]])+(1.-msk_arg1)*numpy.array([[[[0.3512351241739825, 1.2353952530504433, 0.79295619053207367, 3.2339234410649302, 4.1966821900532087], [4.1568145009709383, 3.6667262092209509, 4.682489286295719, 3.1601070987890885, 3.7245604363718514], [1.9405737239099943, 4.7810311696582675, 3.8718245218750877, 0.69659683163058206, 1.4171625994025019], [0.63404759162778801, 3.1673181551459084, 4.2315890264612825, 1.2861757804860694, 0.70733758731338048]], [[4.3237597902851999, 0.49813048561684781, 2.3203357202238402, 2.7999482246423124, 2.2268699076786418], [4.9265652666588045, 3.6132821749023223, 1.6960638799061862, 0.26387978016260383, 3.4113046660525934], [3.3820065316884054, 1.2793899193050169, 0.93430424540281698, 0.55713463918879103, 0.34431766892299909], [2.1696151314166325, 0.79974149242377934, 4.1881792077293962, 4.438673887939097, 4.3659509637763625]], [[0.37466125312004916, 2.5160661616417528, 1.6416576571056725, 1.7625589727134612, 2.5513266572935724], [4.3600810364189506, 0.62495634432308467, 0.82371252249544691, 3.9497305689053808, 4.3316287369696491], [3.2828893705579283, 0.49819462912862789, 1.9298571067408405, 0.53963428000476543, 4.5138949652340425], [2.5685273309912038, 1.8537918796908821, 1.6949798102025619, 0.86194510838931493, 4.6355438807258746]], [[0.06006092580988704, 1.2045604743790372, 3.2969363769718965, 1.1112813449835468, 2.3433634786018667], [1.4491253896754757, 1.1573972877969738, 4.7581046144662436, 0.45434862762320399, 2.3782841162712889], [2.1324900881125921, 2.1384363312597396, 2.2007175413911946, 4.4676647622102337, 2.2106085411005409], [3.2720173180023866, 1.8168277264557551, 1.1360395319277221, 4.5369604675497026, 3.0503076512366225]], [[4.5657126303593847, 3.8533277642841952, 3.1560047028144513, 3.7551624284300842, 0.60629162719578589], [3.558491710401753, 4.0360410238267352, 4.3350272488035939, 0.37786086781437145, 2.9739096292866893], [1.1130983684734501, 1.3768787058941041, 0.79849546003552097, 1.1581501279616115, 2.878929979133126], [3.151539471765191, 0.60783899958066301, 2.4639519256223759, 2.8232935405985655, 0.20797408486998611]]], [[[2.5594135345693774, 1.7732054606414409, 4.2739900798141939, 0.32646692721048931, 3.46839212107357], [1.2085025374980238, 2.4773552455111663, 2.8870218826330158, 4.1657545332509667, 3.8043783747998856], [4.7117173682039528, 0.13024107090327519, 3.5043867175387056, 0.81339737289185443, 4.4233212639416868], [2.6758398795475209, 0.43939728491458591, 1.045742791237503, 0.68687134025032126, 3.652766415307946]], [[4.2067806168139255, 0.96420424594722631, 4.1287935867373413, 4.1547792008363569, 0.56119189706610895], [0.58959588129344209, 4.0514510743020713, 0.47647131285621314, 3.2609575366073402, 1.8538070751724471], [0.12623022092687194, 2.2185639501111396, 4.297568527038421, 2.1097104214563123, 0.65033179713957845], [0.44830028401087874, 0.22051532644911043, 4.0181327016795088, 1.551107910385997, 3.1514623500078676]], [[4.3326494370044912, 1.3339576691114781, 4.1731099165173626, 0.91884453546166511, 2.1417454222117716], [3.814370025272888, 4.7616501970246334, 1.393531992493197, 3.89122547682175, 0.36286252317818896], [1.953941789227531, 2.4272966632687858, 0.25925004294261211, 0.90559544626165389, 0.67584887336169353], [0.29641953812742278, 0.3173362926139171, 3.6411801690196568, 3.0522729140792166, 2.6730737802130249]], [[1.9980154953761575, 1.0693153090194167, 0.3729341643363121, 1.1778150383420416, 0.48637605218949254], [4.1466594264697862, 1.5736867028855099, 3.1312990744267397, 0.76389844264925144, 4.4172861295259258], [3.5403586105375862, 0.6144287270552713, 2.7212168148251279, 2.0951213787649947, 0.52469829309579541], [4.398304374196039, 2.2596506469505631, 1.6363382675336582, 3.6513796295986114, 2.1038415701761499]], [[3.6332738494358199, 1.2286446808356557, 2.3554502945796121, 2.0196743494702925, 1.2256368565837978], [4.2946299802398737, 3.4794473697265715, 4.0016926662227243, 1.4477732922725115, 2.5683812726989723], [4.252679911721148, 1.6662877833282963, 1.7966508712215659, 1.1965233209111277, 2.7610685672841044], [2.5941898507778003, 3.9481077829961522, 1.2226365125656649, 2.3213811587390758, 1.6121566799905969]]], [[[2.6257466068654645, 4.1766648624522089, 4.4769552184727077, 0.090304365044183654, 0.52337934262339714], [4.6861760707386342, 2.4893175752144914, 3.4417767215220567, 1.5704620121589075, 2.3381165665025274], [1.8556782343756628, 4.805625494353448, 2.684321535689222, 1.7678680032333771, 2.6809138064158926], [0.66282285822757492, 3.3161727313251901, 2.3930776389187556, 0.31992806719889649, 1.9093187853218725]], [[3.0679352483485829, 3.3275338627402209, 1.8443775940734188, 2.9638917449250362, 1.746925703003779], [3.080590402172692, 1.6778281272329711, 2.9261669055974995, 3.8527963328170918, 3.9237991795696088], [0.49938343937140739, 3.5779042399655507, 0.68377899023850752, 4.5103147200660674, 2.4447984047737483], [4.8064533402357021, 0.4587946228980147, 1.7519659103152216, 4.3969217651642785, 3.4711020151419185]], [[4.1559540045057748, 3.2899947823146958, 1.1330026163155553, 0.2139258865831839, 0.66931896460498508], [3.1403766933685384, 2.9491038707139112, 2.2580969717105011, 4.4812005223667155, 1.0623743599274724], [4.4492430538520527, 3.1314525102479851, 2.6629440902850017, 0.41193876188449469, 1.6967909858209889], [2.0375846339594696, 0.79511333985497945, 1.6018903691914943, 0.93022546993260558, 0.12059784240267317]], [[3.8039000496829991, 3.6408724200256093, 3.9050651236007874, 4.303827639579155, 4.4438212816255787], [4.9213278774096834, 1.048254603145476, 0.090203152353548158, 0.99987084075984634, 4.936163311889886], [0.076761191376270105, 1.9915059701748907, 3.9226916508313643, 2.1842180852490856, 4.0094786334563306], [1.5304416694588505, 1.6303438466176001, 1.0983684069105637, 0.65694089904620212, 4.1488735843382933]], [[4.2522667565639072, 0.60058684708883892, 2.1480297636547494, 1.2237091552290877, 1.5073729156768507], [0.049588838024949129, 3.3668072154876549, 1.1563922063434662, 2.8087441474573582, 4.2705537320743421], [4.3053600302261588, 1.4154002182980965, 1.7052467107908027, 4.6259418965106844, 2.0547343132669811], [4.5077953478836807, 3.4937434881954283, 2.6115863495208083, 1.7377420568307305, 4.3461523982358505]]], [[[3.7031953850662247, 0.71687709398720267, 1.3374746311524242, 4.5700134297580766, 1.9165124495362207], [3.8675185181045455, 3.0654539434069368, 3.3095246783612935, 2.7997853037761642, 3.6404216064763291], [3.7180345266898827, 1.4091258647135392, 0.78096079518007289, 1.0937856078702268, 2.8231029555194924], [3.5636968269142271, 1.4724724675171679, 3.2172942642219353, 0.73123173660436958, 2.3655083072068877]], [[2.6945310079944518, 0.67267158057644483, 2.6652666837842496, 4.6283432551372767, 2.6240892814589851], [4.2241902633935906, 1.2798136940719522, 3.752213646236024, 0.53873570615900379, 3.9292242963864132], [0.80354502307071973, 2.6458679876295053, 0.87423417847681884, 0.86747230568485922, 1.920030563921105], [4.1890132008743874, 1.5631941474535196, 2.4389889996172065, 1.408061731184572, 1.197981862125866]], [[3.5822552967123378, 0.47932225970174175, 2.8739152479449821, 2.5254731578072049, 0.52397019584696025], [0.37198925609195416, 1.8770832958875145, 0.8327921406791543, 4.7840004234216966, 4.7036994986787626], [0.84136686663140925, 2.4144865219645579, 0.83401876588460611, 4.8250428780450472, 0.28505898208485142], [0.84226754713907082, 3.541791499871568, 4.4211424089950482, 3.567123820630997, 3.2312040716079298]], [[3.4462003137694976, 1.1323861523244363, 2.9745074078427201, 2.2311974790283804, 1.3891871948666581], [1.0188183414488852, 2.6479322070744575, 1.3894739762430899, 1.8460997070284777, 4.797211445054324], [4.8427578958896742, 2.4602320421983008, 2.4928684791410749, 0.076627658960639444, 4.3537850917036724], [3.2264484304670544, 1.485087112147337, 4.8271591249875794, 0.68352001257817863, 4.6332489559263799]], [[3.6867504585720856, 0.038118591384325995, 4.7289262515647819, 1.4407528197778776, 4.4644190965487818], [1.5461301758821522, 0.53029326399087062, 0.96542903731595164, 1.2158471858298763, 0.25754222308414476], [0.72210677891933006, 2.2411319724154573, 2.0207822680584151, 1.7325526621743415, 1.082643198017279], [4.3695657520749211, 3.1000635690924798, 0.26650368894915127, 4.6338846516654462, 3.1116530940625431]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.87783443838921127, 0.98307528861122484, 0.97478854948126725, 0.93045767594531748, 0.86589255368870532], [0.91387504481438764, 0.98437242537303671, 0.99414301192956223, 0.96073732302580817, 0.89209441317166438], [0.88506557576009104, 0.97964912610925636, 0.8761253548609873, 0.98604145080817107, 0.99875195466225208], [0.88953348644050534, 0.88960821980235194, 0.95418468796408462, 0.94440580820129083, 0.91871807706186226]], [[6.016251729763491, 5.4261653491447426, 7.8503461238565473, 1.0276062499613026, 4.303780497709937], [2.9983567463803387, 13.475211011459319, 2.1821594824233146, 3.4389797940278588, 1.0162915969192665], [3.4043186022537939, 2.9273812727486033, 4.6613241944376478, 1.7059580067179365, 2.5154695548528325], [1.7760660062667233, 11.768525807516518, 2.148414206777443, 2.7764501895149274, 4.8153940338966823]], [[0.95117055969269682, 0.87381050455877129, 0.31299320372220413, 0.5258208527360172, 0.63647188933801324], [0.74951938692563702, 0.61753936689133349, 0.31182508689601302, 0.86044841179056708, 0.81595569491793141], [0.34983935575455039, 0.31749886646994924, 0.32593123445816008, 0.71977251240918261, 0.57206445109811921], [0.48603030083687837, 0.94121965655628725, 0.96753891198848796, 0.58840347164119311, 0.59750982453100598]], [[4.4493295608310612, 1.3848756716160069, 2.5242475711110903, 1.3702533788040574, 4.8748584799153107], [2.2440078484709605, 2.9719929376341443, 3.8787696045260494, 2.0852597090116984, 1.3285746162173826], [2.8795446866846799, 1.6393572030916119, 4.5847409790782718, 4.9805651000712352, 3.9401197818478062], [1.4536872657490951, 4.113187161739873, 4.5638551248965484, 1.2224006543214621, 2.5691122836823226]], [[22.790147641611313, 2.603869860625132, 26.495760082357766, 177.60001699182703, 32.651971115230872], [6.9600987197746447, 149.98413067654064, 97.481649856486698, 5.012479694683547, 5.0183644154095202], [1.5396925304029976, 1.4612565667577739, 105.01524900532191, 106.98438191034754, 4.5079188408254334], [2.3848341218364908, 1.1285675388531702, 2.7888610611403597, 140.26341965707758, 4.8498929005100697]]], [[[2.6653884984134102, 2.5012710181606335, 3.0172999373234077, 1.302346296417876, 2.4558295898759739], [2.1568698550056364, 4.3669988137869815, 2.8855429751880877, 1.3122738763898638, 2.608202424269467], [3.9847035367556667, 1.1540382112181293, 2.2112083958021249, 2.1665173386758876, 4.2577578154056708], [2.8745282172746887, 2.5072513971258439, 3.3210601229358674, 1.550302200381751, 1.4117337081086112]], [[6.5300688781422531, 1.4151842871649136, 1.4515386424435077, 39.023204871268071, 2.027076851862224], [81.898229391819086, 36.865593941396199, 400.13485863665659, 4.3993370342569333, 10.993378808623563], [1.0880053651418116, 8.9473435243760306, 151.48228426228158, 234.57446739816078, 52.303405853184984], [14.284055457986536, 9.0896029344591227, 1.7291964449218065, 443.53951016937299, 10.52671324403738]], [[12.750716935060094, 4.047183443561762, 461.63297571252178, 5.1524980341273006, 4.2450428606119743], [2.0580485844730476, 511.57631869686475, 29.560183785252992, 9.9613304857279381, 3.6061437021503822], [6.8769870744080874, 113.9794316992224, 216.34173238331604, 1.0196676925225256, 1310.1387306828942], [175.5236063423267, 1.5470560719505211, 24.027059537163129, 16.827617963409683, 873.84462015391648]], [[0.027368377418659603, 7.895381298203838e-05, 0.12401130786148956, 0.0037503541035777037, 0.08105825244284337], [4.5274192566043529e-05, 0.018916512876183074, 0.00045554691991388701, 0.00066096217356275387, 0.00010317770645002457], [0.012236231154908905, 0.0018729226115701797, 0.10307096283778604, 0.00014170424146311882, 0.45586214939855951], [0.0086538583625793818, 0.026804388595114161, 0.0028898748698910947, 0.00066322918299965459, 0.0032684255229351949]], [[448.73327255076344, 325.68131933497182, 3.3557991068607209, 45.74899127462303, 12.31351499373943], [175.18594031818202, 77.240977419088878, 157.9066417737433, 49.259172369081384, 109.15321199789088], [15.997062043147519, 66.683582239614864, 98.690313948127994, 36.814760168776402, 34.390492979107989], [1.2297093001050809, 1.9389894371709011, 14.254681774871109, 2.3189975127369591, 49.601314336378216]]], [[[87.188881389848405, 96.573696678567416, 2.971158987259944, 9.6064511499792982, 1.9411109305527006], [5.8492036106051559, 1.5964012894367035, 20.815663759725552, 2.2928569372901517, 22.837226709316056], [59.233139019007027, 76.384524600607364, 10.461637280504904, 61.983472673172678, 24.148562851312054], [4.8205375866183777, 66.518467984587701, 12.435710505252418, 6.2983064882795112, 4.7271742183906857]], [[213.84337495306346, 458.21361535268562, 4.050659736938333, 92.560874637271908, 12.198500927189199], [17.95539663903174, 1.036239359688901, 5.9048464323851322, 357.57616018629722, 1.3382678771142786], [20.369027284447398, 149.98412536897624, 211.87383496826797, 66.7956115866792, 387.91210909442918], [13.454470889742163, 3.283878962358064, 2.2030246525590229, 1.6422246683634072, 2.9306431054012072]], [[2.2862387110661557, 29.11525242367582, 2.984839789080068, 9.4031000749046481, 2.5418478923930237], [3.0579391543030372, 3.3332984199396583, 5.4301137445248209, 8.2592117602325903, 1.9128177786459777], [45.622424668532851, 8.9324736721939004, 18.743569108460481, 3.0263971779274748, 14.552002715150754], [1.5657941409445053, 79.745459734840665, 44.605214061115753, 3.8761377085972413, 7.3325299924808709]], [[1.0654928630891223, 16.562628653684882, 80.631232821574031, 11.895842253324439, 62.908547767343727], [1.1569441166171048, 1.0216943048835976, 5.0657979839298797, 1.6207322212499626, 9.1600111005238887], [10.960295497400418, 43.012637884693341, 15.152571816428209, 149.67659224831479, 1.9012253280055946], [2.7046898350699076, 3.3120238917337965, 6.1396479469178562, 3.7047513306264874, 1.0150206455661721]], [[9.9624518389600247, 1.7882120293807429, 11.631561294994151, 8.9542722809566406, 17.653851676237792], [12.004186658006153, 28.742455756293641, 1.9912065585578949, 2.3732781218226973, 4.2792699575844084], [5.9499693509232872, 14.994188484838737, 2.6668609082075472, 6.2121997536608813, 2.90938155763856], [5.6968379374154994, 12.785185086218853, 1.6986563562031736, 4.244647094100702, 1.7622454826848895]]], [[[0.49756896645019077, 0.044159409757084489, 0.001169716307727203, 0.00042014333332039439, 0.66789134164119068], [0.00014416773454470387, 0.014235661663973413, 0.00038698852695200758, 0.65555762894956859, 0.069414323462076558], [0.00058993578173911038, 0.11444337843517639, 0.33741441809584788, 0.0038783128050388421, 0.0071691221568966484], [0.037808324054493178, 0.0010825220660314872, 0.55324675886427888, 0.0036877133694954586, 0.0026600299783262112]], [[27.312486530770446, 34.96662015601774, 13.730589347465557, 140.9573338155771, 17.927896608233777], [175.80132931990519, 1.7715520511340908, 3.2933235086716302, 26.740859690005941, 281.31565487861195], [201.04315318577466, 77.89797183959304, 12.614904709405625, 56.965111792748182, 80.550881828341531], [632.61312144404337, 10.540227003162812, 17.967793967310399, 51.936005547902013, 4.7048204624674028]], [[1.6423938787144519, 1.90819557836081, 2.018095073931105, 1.9908614239103273, 1.0381495676460426], [1.5779994379404061, 1.6222608506932827, 1.7509339903154029, 1.3361150703271627, 1.7436955061659016], [1.2420114794675652, 1.0498434532665895, 1.866827886298364, 2.0006498643007364, 1.432609603635854], [1.3190111572075589, 1.5012360389419785, 1.4305838269746241, 1.1460567619826829, 1.1434075806034911]], [[1.5709441725133233, 2.0187696020373029, 8.6563329300734448, 2.1585477806543554, 9.3889527490289613], [3.3804877065598355, 1.3473695474156568, 1.6605316377203512, 4.8928852566034422, 1.7318511651476551], [2.0378646402902114, 3.5809339883806515, 6.9094166845424922, 1.9534024688211538, 10.041028105655393], [1.3470217579904507, 3.4350261207944648, 1.0858071821882311, 1.6283994910413704, 3.5133466936496198]], [[1.0653363483361447, 23.3655674465681, 4.6861379756476289, 4.2727330642729475, 6.6259418117953821], [37.040290785758764, 4.5358538180638437, 10.516767529129771, 5.8625737894410062, 34.359077785485049], [1.6642589067212712, 39.216526163710121, 18.459906350381164, 2.7343716233505178, 2.7037747590773558], [15.236918937141331, 2.0797375165083327, 2.3137643063245599, 19.022904928068716, 7.6111654600195324]]]])+(1.-msk_ref)*numpy.array([[[[0.98942557071828707, 0.96329906036599056, 0.97628552957856984, 0.90675747087546144, 0.88071622274328587], [0.88177959296990815, 0.89495684233266959, 0.86786109895534447, 0.90878559386187485, 0.89339163322586745], [0.94295680308175056, 0.86527652491196572, 0.88941846820200265, 0.97913700479640564, 0.95801402007235437], [0.98099242400466868, 0.90858726847120064, 0.87978622375750015, 0.96181964533265618, 0.9788187513615183]], [[10.178056137685276, 1.306443895456237, 3.4734496958139354, 4.4929967860211804, 3.3035321149495243], [14.065345934487933, 6.9516189669394928, 2.4846924406514042, 1.1521206297838473, 6.237556771331267], [6.1402562473208429, 1.9868521733758291, 1.6509819758706248, 1.3484717549456591, 1.2029407526225562], [3.203576803494752, 1.535967723349654, 9.4638400444833817, 10.82544873866383, 10.411123975419475]], [[0.91196700452019985, 0.53856332038686894, 0.66779022844769675, 0.64822462259693814, 0.53391274398114508], [0.34218523937853856, 0.85751731549231025, 0.81660492930441386, 0.37852487616371344, 0.34458828839731992], [0.44599051089570624, 0.88467435003823969, 0.62209245437214888, 0.87570312777371151, 0.32948151082808325], [0.53165870786222191, 0.63384071339168746, 0.65908928340409612, 0.80896184287282502, 0.3197692178726092]], [[1.0198326615610163, 1.4826987028578953, 2.9388642594920742, 1.438158853680825, 2.15163184821062], [1.6061355904924346, 1.4600089794225335, 4.7388380908036023, 1.1601643385434179, 2.1763405240829075], [2.0082742103073867, 2.0121826685922075, 2.0535799284442389, 4.3095123421172179, 2.0602322282421994], [2.9150157852457443, 1.8113299228572528, 1.4498485252673332, 4.4082724209289816, 2.7111725514494847]], [[139.53001103508228, 64.570875832306996, 30.372515205348009, 58.066432171493638, 1.9266171223465289], [46.939919542985287, 78.679790389898869, 108.71938936137957, 1.5048522530292527, 24.94276664718798], [3.3331853130317821, 4.4336957663666228, 2.3718104151433463, 3.4996271314827747, 22.507616748048115], [30.22618179491808, 1.9298442849383814, 14.368137239989364, 21.193135711744144, 1.2522538337785103]]], [[[2.2500888283811267, 1.7539206814814012, 3.8738403257254546, 1.1089831731883781, 3.0011228173132478], [1.466565458339429, 2.1923387581800693, 2.496213285836093, 3.7432382871714798, 3.338248597901035], [4.4501756902834622, 1.042131230565063, 3.0355470441019623, 1.2939908953430939, 4.0615434912254882], [2.3346459913094555, 1.149384268463685, 1.3928494007503316, 1.2431399811517774, 3.1816721506263539]], [[172.44368168052978, 3.2557042255240956, 156.74128350874932, 161.80775958564129, 1.9877998918537945], [2.0581375656305472, 142.58117390670907, 1.7919616085395236, 54.172038826367732, 9.6744447876158635], [1.167115048592694, 15.120243540807003, 192.71570321857376, 13.233779682096754, 2.2170029531628588], [1.7312142610899319, 1.3099142410824285, 136.88239118351291, 6.6786387624661439, 47.376081840067677]], [[510.37226306672579, 6.8189537440296926, 405.67248948455347, 3.7520951260575326, 21.806279717919988], [242.08321847436295, 946.2604183229389, 7.4293605881738953, 270.3952186263769, 1.6857279964762015], [16.641984930968192, 32.888938976195931, 1.4522124305175599, 3.681232243137039, 2.6448595090593519], [1.5320076106918974, 1.5788243048206132, 188.67822299307346, 80.845770230256463, 46.84459544708016]], [[0.016455968307605574, 0.11101878439244862, 0.46459302870921604, 0.088825190167559451, 0.36795980765360597], [0.00019869056828057501, 0.039366968787762954, 0.0016018206737603588, 0.2079933551919797, 0.00011391535703447719], [0.0006909340470785709, 0.28280343225157434, 0.0037213815924972268, 0.013478286424767909, 0.34008634577036179], [0.00011844800376730906, 0.0096107092889603958, 0.034609985319525437, 0.00054995284881239953, 0.013238840697281358]], [[110.13763571122512, 4.9035480179320228, 21.075908503388334, 13.64824816316554, 4.8844987812334324], [259.19368720399592, 90.257593542508246, 177.41517307903854, 6.5112275649102225, 27.762311000872113], [245.49806165443513, 8.6391332752205834, 10.226699984183439, 4.7038981670791395, 35.624417807430163], [28.705181056491085, 165.52954687699852, 4.8655706451857226, 20.166898571569909, 8.0546738517893814]]], [[[11.753895603361231, 50.384075543930322, 66.785680205650834, 1.0884426880330851, 1.6342367456246536], [81.274816398185067, 10.341347264924481, 25.279773354762067, 4.3659319379231736, 8.9732912709726982], [5.70588769357109, 90.915992587861552, 12.41810572407736, 5.2545338400487767, 12.378455410775915], [1.862723092977455, 22.468829932645473, 9.4482718587704753, 1.350186769659663, 6.0004749508861561]], [[50.10679047412615, 69.780850263804197, 10.51820833979227, 43.878083634596777, 9.2884906413747199], [50.922370811604182, 8.5047044271112373, 41.816231162211089, 136.38803062995245, 149.31995950069566], [1.8910404622557371, 96.042333094647702, 2.3926083344947351, 315.57113089391902, 22.626942426043584], [460.45076938151942, 1.7956065100218825, 9.3484121375318949, 273.06619733733356, 83.807946948388079]], [[50.502422469593668, 22.304884430002954, 2.9131283015419833, 1.2237064229183945, 1.8806998318115025], [19.367751444546915, 16.1691165485902, 8.4232178837957168, 68.645814518428153, 2.7252899564837834], [66.606457584067613, 19.205324006802751, 12.3425270247802, 1.4751376668072738, 4.9593815774418815], [6.8407126355388428, 2.1177570822117553, 4.5345336927096795, 2.40575650765369, 1.120538712025873]], [[59.410134362945335, 49.869657078887968, 66.227219163333913, 101.62256848691047, 118.10627012253919], [197.21823910256447, 3.0819757258087237, 1.1017020240333775, 2.9259477655478099, 200.3850131858359], [1.0859149212239578, 8.4857963436200485, 67.492612905203089, 10.436609364572655, 74.084497165772177], [5.1723268978471726, 5.7580181243801833, 3.2523600927014762, 2.0246462186487424, 86.046050050428263]], [[18.192405662717064, 1.5064194055604805, 4.3294494552693203, 2.3044578064868859, 2.7964983242434873], [1.0344095316212181, 9.9435219419184424, 2.2010182492881993, 6.7950642178954919, 18.420792876824517], [18.863442951956234, 2.6264205960349845, 3.200677665770757, 23.47497344947028, 4.0624728507587697], [21.657077837909203, 10.843007724385485, 5.9398890090665351, 3.2724263316723095, 19.395777772384278]]], [[[0.0013256488273610428, 0.27730006411047781, 0.091351158943167865, 0.00028110379135010165, 0.032416951564797694], [0.00098796382902019203, 0.0041494075023902153, 0.002681209134616011, 0.0066745833045648555, 0.0014832245887413814], [0.0012909152248374274, 0.080359532598583194, 0.24725988835755974, 0.14127745068713657, 0.0064018449906163661], [0.0017014765960916731, 0.071748671471389122, 0.0031622661675518417, 0.27026865554503043, 0.014517088099383738]], [[37.353330031728575, 2.4690021185914142, 35.913094340005792, 502.06431542736379, 33.980106447257747], [291.69266773770977, 5.5821919620988112, 154.7087262997812, 2.0623690208474663, 196.24796189529428], [2.9436728316075182, 34.989132884401812, 3.2369687806344198, 3.2076928874876351, 13.194351725553943], [278.22668181655013, 8.168920586021498, 26.498047778252776, 6.6319440146944082, 5.0009642806349701]], [[1.6781893679636251, 1.0717284431995027, 1.514892660387936, 1.4404952297703202, 1.078666257374824], [1.0552320651188474, 1.3116425775199176, 1.1278993961517423, 1.996497344727836, 1.97346138739891], [1.1292979987620253, 1.4175739577644928, 1.1280993617746833, 2.0083748522906615, 1.0420577398220052], [1.129445007173538, 1.6684040948195642, 1.8944968241323743, 1.674523455597505, 1.5951707055038959]], [[5.1959274235988993, 1.7185396724466229, 4.1467503431257837, 2.9063420904453126, 1.9430713065293257], [1.6277032958858795, 3.5472263567346607, 1.9433377795489331, 2.417544746631497, 9.9134288781737698], [10.131701896970446, 3.2427220023430143, 3.2937242606442219, 1.0373207328499983, 8.0193422098521356], [4.6776508112461057, 2.0342485466650291, 10.056411793446026, 1.3865794134423677, 9.1658817397358714]], [[16.117078233244065, 1.0291591789742229, 35.364189033497482, 2.9634520874495429, 28.969869549681118], [3.2085250677644273, 1.4916026345443381, 2.0708346242811531, 2.5012028076872963, 1.2143290473533954], [1.7237155220372486, 5.418699438787816, 4.5892104423145792, 3.6927721137067913, 2.262190104658965], [26.970268586043929, 10.355382941019398, 1.2225621925915426, 32.918556881780546, 10.44627229014273]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[4.6144565897282224, 4.4889538992470222], [1.7207391852326275, 3.8492198814534961]], [[4.6126364571466443, 1.5326470921349336], [4.4874876308665623, 0.37380334023893425]], [[3.1647622720980793, 1.3982204398397657], [0.7111539017039803, 4.5610103051924193]], [[1.7218262466908043, 0.02045197761858495], [0.010388305882066023, 2.7286870446600457]], [[4.1142185099468502, 1.7054179025617422], [0.9618503767032669, 2.3648288199971703]], [[0.45071774751959848, 3.485669452178263], [3.0123504584703147, 4.0325931972767943]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.6108566040435921, 0.61112781996723176], [0.74856055598219418, 2.6050803596781433]], [[3.8464241946348836, 4.3993702356157103], [3.3109563290510478, 3.4980392005698184]], [[4.8501887512410926, 4.3072786319702772], [2.2609584554636677, 1.5017602948836273]], [[0.65141054640804008, 0.24647564438051248], [4.1042888456079751, 1.8374713845413715]], [[3.7191044641405862, 3.0768343114502432], [4.2124580086691497, 4.9264231892044474]], [[4.7254931071567601, 4.5513542230085431], [0.14928660669790972, 3.0892965191490336]]])+(1.-msk_arg1)*numpy.array([[[1.0913199378678862, 2.4048867202292148], [3.1232432682913593, 4.6204848836912173]], [[2.5068000649296591, 2.5733167959489611], [4.927049566929953, 0.80670026894986646]], [[4.924120151224229, 4.5551945121111865], [4.2554509880137843, 3.0530297658200385]], [[3.2834105608496884, 3.0297823583975432], [0.23592730095712799, 4.2502916383241915]], [[1.1489177669036477, 2.3336122133412021], [0.80168288692775858, 4.7282371623075772]], [[2.900510837987647, 3.1824273971701977], [0.36407739910551135, 4.7006619228384885]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1153.8907497309124, 2.5034795084381236], [1.5012291472900936, 33.492146075255427]], [[357.95659777281554, 6.5437808069587495], [144.12998293468075, 0.031995513414190054]], [[267.1460389736273, 4.236769839088887], [0.46269610454497567, 9.7667871253136109]], [[1.4247082340666799, 0.38338701244614665], [7.2331364004655726e-09, 6.3248830833616108]], [[192.57452690336768, 5.1677918684164403], [0.84886990120067729, 69.421749926619114]], [[0.023148819726395931, 293.85737047523196], [1.1789467145309498, 74.272803931645214]]])+(1.-msk_ref)*numpy.array([[[5.3060093806262252, 37.011373281286851], [5.4474774246896294, 506.64738496334741]], [[46.172986502134059, 3.0005539467481976], [1630.9941452720007, 0.45211731171255087]], [[290.89727745222444, 4.6038947465228572], [0.23444357921520426, 102.83312026534645]], [[5.9545622212684766, 7.6189590596480514e-06], [0.34044640404446913, 71.273894324547456]], [[5.0788531370706753, 3.4753874421938131], [0.96929860430190495, 58.534737829611061]], [[0.099116672381565171, 53.184669050920185], [1.4940277619426325, 702.50204367007495]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[1.1618590951546295, 2.7284462745723577], [4.6538595939914416, 1.9837376740923796]], [[0.84224335563814889, 1.5280494713881612], [4.7866545194654133, 4.3546325118352245]]], [[[4.7070542594079479, 1.0169188180529418], [4.0151555566141717, 3.4893426128995992]], [[2.8730678909325009, 3.3793139622343693], [4.1444037188162648, 3.5846802964432594]]], [[[0.091941482258780197, 3.4098375956701403], [1.8328532142830909, 4.2029424336795378]], [[0.45831861935789969, 4.9076946678188627], [1.7469670922599843, 0.73644539791092956]]], [[[1.2669847815040249, 1.6259501690926781], [2.4398539555318579, 3.9921144194324838]], [[4.071286440561809, 3.3159767174690646], [2.7927809119232769, 2.1520636443242425]]], [[[1.7118310396095238, 2.1590009172823992], [3.2273974388577487, 2.8967018995883231]], [[2.0634730850465872, 0.076428553337053909], [1.6132751021964764, 2.6623769845387755]]], [[[3.0443878215549023, 3.9366882683245117], [4.4112764698445082, 4.3556365990005732]], [[2.629500930954404, 4.145165796724406], [0.68529243359031211, 1.4229598895597997]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.7666825234023307, 3.133862815211172], [4.2951721789067552, 4.4316725721084049]], [[0.65306495312130519, 4.5738589699056744], [2.0539147254754115, 0.010273224597918627]], [[0.92171031235410028, 3.2273799521882243], [4.3060934894577594, 3.1279416403595013]], [[4.1630188636197509, 0.29344242615550048], [1.476032360057733, 3.2361671706165511]], [[1.4191932023061957, 1.5048150832576059], [1.8193213997492041, 1.6791744423871566]], [[0.66684575348727226, 3.6671453170527069], [3.0113108541298157, 1.2348787545091724]]])+(1.-msk_arg1)*numpy.array([[[2.60305935746953, 0.50515794916016543], [4.5668566555901942, 4.2286204816662014]], [[0.83147772458937308, 3.8582116714373029], [2.9071225712327613, 0.75315351144484521]], [[4.0420898542717971, 3.2709491032019535], [4.1908212044612716, 1.0070241025061883]], [[3.3893825806000759, 1.1750221928685989], [1.6695307076937029, 0.64621326885876629]], [[0.26687568810744872, 3.2158001680536952], [3.3213916605557126, 3.3695366159750679]], [[2.1693564694842404, 0.93824536785115897], [0.29292908565017556, 2.9845983857608336]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.0443999838409623, 119.63833220571827], [123.83265800969841, 8.5560793162434123]], [[0.47834542683420167, 6.1787681883213095], [1031.999604113628, 678.61567176635413]]], [[[2.7501027967889877, 1.0110169034423679], [577.09794190576861, 303.69157802435711]], [[8.737825550694641, 12.194632899254474], [1.0147132383030937, 1.0132018959301314]]], [[[0.11083013491144618, 3.0976057390742446], [7.0666408078017318, 102.90675336236825]], [[0.03475006185090114, 944.02448057686559], [5.7260149518871426, 0.3840815115364723]]], [[[2.6781769626241272, 7.5656001928941334], [1.2991789713716946, 1.5011305417929264]], [[7.9429834807774551, 5.8673146705004191], [27.761943594276353, 11.944687230015907]]], [[[2.1445003324696974, 2.9810505448091766], [5.8308093557296976, 4.955412493993169]], [[3.7355654043356945, 0.0092957211272584207], [2.2324343145662366, 5.1772812711166782]]], [[[2.1009702645102495, 2.4937941248749405], [231.05260602252287, 220.54398779468102]], [[18.380999054018247, 72.378677651211518], [0.62708538796013458, 1.5458749478403273]]]])+(1.-msk_ref)*numpy.array([[[[1.4777410205037036, 13.63683790139817], [2.1744588091051016, 1.4134372390261849]], [[0.45654561717774073, 6.9331273752563911], [750.92678342787599, 503.36549848864018]]], [[[3.6255673554905177, 1.0140476965156822], [213.40842044627453, 124.17087656244163]], [[21.501446507595634, 34.46429450351809], [2.9177206877126296, 2.6156900919320467]]], [[[6.4627946141082801e-05, 142.34999719105093], [7.2556657365365345, 109.54979539947078]], [[0.038020128321142578, 785.8582639385105], [1.7538262135285465, 0.73486461221202037]]], [[[2.2301372272236297, 5.1942538838417986], [2.852076027925333, 5.0865913012688182]], [[10.422351785546068, 7.3990649601651874], [1.9419361311506067, 1.6409501549223475]]], [[[1.1542636989333015, 1.2280157959575289], [43.288027249240912, 30.576680196835952]], [[11.089344081985914, 0.00019536852293845498], [5.010501135632377, 27.099502704312314]]], [[[11.191384085158653, 19.545705075210076], [4.0249403411722122, 3.9772898283620091]], [[1.3273734000304171, 1.5166870752938388], [0.3237096049199909, 2.8656197442294902]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_expandedData_rank4(self):
      arg0=numpy.array([[[3.1303808419882282, 0.47887019820795823], [4.2825163808603213, 2.4501016785088008]], [[2.1302920679804243, 4.3923264074390183], [2.8276004993418193, 3.899054550601281]], [[1.7695131494154901, 1.8985514773651697], [0.03818358468877818, 0.4192972298427991]], [[2.3963333110003422, 4.2524100501376525], [0.99154600516042946, 2.0684263382870087]], [[2.8902636747162549, 2.4389072883591192], [1.2727888775049805, 4.9426368644115719]], [[2.1957704399234235, 4.3387257609800107], [4.85853258370107, 4.3923611798862572]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.5739240860532384, 4.1807963782933841], [4.9413550078585011, 1.5993430372051207]], [[4.7340862283426191, 3.1519331352277069], [1.6846673603591056, 0.18645155449631085]]], [[[3.9392066567881243, 4.1175186762420646], [0.29172761417360182, 3.5661437353461829]], [[3.2700748584814758, 0.42638819508427644], [0.53266079153417811, 3.6214465857906446]]], [[[3.2288664285228181, 1.6914710376195627], [1.3622186435354002, 0.51706182738166562]], [[1.2175340042979232, 4.871258274984136], [1.2711636656233167, 2.4018259580840216]]], [[[1.6377742274659643, 4.2537042432725443], [3.6373760996268971, 3.8104858774367325]], [[0.51719739946176502, 3.8865753276423867], [4.7001722353689033, 4.2404921705447594]]], [[[4.1503603748421733, 1.0377667486582989], [1.2980533003419645, 0.55472923681398267]], [[1.7362856423491826, 2.0074293322026904], [1.8056635178146927, 0.09568629526805969]]], [[[3.8937260237737101, 4.5779639310270488], [2.978434775119899, 3.8374281411129321]], [[1.945283494267309, 2.992497847097205], [1.116297180756578, 3.0741891290535683]]]])+(1.-msk_arg1)*numpy.array([[[[3.2711486504156193, 1.9502249775825242], [2.8456962354261974, 1.3310538005745529]], [[1.219283617175142, 3.9959286582589546], [1.3173003185825671, 1.4068483123392388]]], [[[4.1776895689228626, 2.6196029409866215], [1.6140711179128755, 0.54408260313277212]], [[1.6885637069863557, 0.87350148256392668], [3.9488654430597574, 1.3311582731333713]]], [[[4.3934648250072534, 0.07383024365741335], [3.3249132341063565, 3.3799345669782577]], [[0.41720309537878264, 3.585349738393635], [3.4290855440784491, 4.4038344683548818]]], [[[1.33938213856801, 2.5518067161151108], [4.4029934709076972, 3.7987326828139372]], [[2.3298124445653161, 4.5405346706912493], [0.88010018036911974, 4.0225573429108552]]], [[[1.660051245779349, 4.0936040531105284], [2.2257937501546392, 0.5136891010625787]], [[2.808497679382298, 3.6701606588312115], [3.4385110295413912, 2.0716045602920623]]], [[[0.91602237312160661, 1.1837869811464468], [2.5163355397502811, 4.6195311831644918]], [[0.97739386127220673, 3.7274958149546404], [2.7514625324209674, 2.5820894925352689]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[6.0260444920406826, 118.02958570834681], [0.026293153387755064, 0.30800557774766069]], [[978.40281111281604, 97.965489558669361], [4.5252832036554711, 1.181854414941784]]], [[[19.669335457340175, 22.508903298051393], [1.5398973236389026, 195.85801308069591]], [[29.934417734179991, 1.5576838867775258], [2.0643382256532066, 138.07898321875217]]], [[[6.3137414803793419, 2.6256541144757461], [2.3948160100525091, 1.3930335722467109]], [[0.01876664579642744, 1.235819741966881e-07], [0.33125593883636006, 0.12398339507593276]]], [[[4.1842021949159838, 41.160594690218907], [193.45609570397556, 248.54515826556676]], [[0.99561865518631221, 0.96754169214407693], [30.448271447738303, 21.800637815640016]]], [[[81.857144097318567, 3.0084692302809501], [3.1812670409268873, 1.639791189217064]], [[1.5201518899122244, 1.6228972071419414], [17.908364064537221, 1.1652049943642155]]], [[[21.381882322943913, 36.62458092822439], [79.130105017062562, 279.14615102075965]], [[21.649460486125886, 113.33527235915126], [5.2172544146402675, 94.574780461354692]]]])+(1.-msk_ref)*numpy.array([[[[41.799591423426818, 9.2581879562386771], [0.12302603470050529, 0.37527765045251144]], [[5.8914240287617217, 334.36766218324817], [3.2559096824226486, 3.527954742819233]]], [[[23.556828069766638, 7.2507038004603857], [10.898200707394516, 2.2370668099115782]], [[5.7842643858544713, 2.4792158266221622], [215.5851303437901, 6.1187171335251449]]], [[[12.272609815432885, 1.0430355384557712], [8.4281109919469994, 8.7307070589238105]], [[0.25606756320192947, 8.2325047330784459e-06], [0.050768650987148341, 0.02175952158318898]]], [[[3.2237308380873322, 9.3010324043972723], [585.97137213716667, 244.35252788744521]], [[0.98041438927775382, 0.96218471716947707], [1.8958102763272169, 18.607166249174934]]], [[[5.8234427782571219, 77.071794943400761], [7.2747296778422328, 1.5808767096690204]], [[1.9688291293422693, 2.4236656816557427], [243.32383784621689, 27.39100807114804]]], [[[2.0554232941394011, 2.5372669685042042], [40.162168753061778, 879.65942417773681]], [[4.6879815056223908, 362.19709313981463], [58.662367899995466, 45.656631624384474]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[3.4671418894782806, 1.0257750551519367, 3.5559218523862492, 0.68409863771840596], [4.7032428406214537, 4.4707035823743304, 4.6256272211675569, 3.1272685388974759], [3.3246520379018749, 2.8331737125363654, 3.5059930952963163, 3.2507473798225255]], [[1.6007836875820975, 4.9204331986074541, 3.829117582341413, 2.7537360461105815], [0.40927288666103823, 1.4168505024411322, 0.57565416164844618, 1.3768877031457045], [4.1607188049367654, 1.3259530303513103, 2.5751450606598807, 3.8089618329370376]]], [[[1.8197462205920003, 3.7559581915663656, 2.0188100392863007, 1.0685183671609229], [1.3454489002557317, 0.91475852367024046, 0.60418549417128475, 1.2363575720879596], [4.1995136077062316, 4.116533162518782, 2.3008443007326806, 4.1006796083617374]], [[3.0547776916055347, 3.137370850347915, 4.8690744532880865, 1.7519133657421608], [2.582602063004924, 3.9981074002594554, 0.82950569028536392, 2.4537120352355375], [0.025667025141619282, 1.3695493968646055, 1.963809324708399, 4.4317086724999308]]], [[[1.9348127644222235, 4.4562499304022438, 1.744574456800029, 3.7308397256574435], [2.6960905710715917, 0.96160019124770801, 2.6812540463442818, 4.4916948456973405], [3.4236522264363782, 3.4872823921881264, 1.7538689494136332, 2.8635788080459919]], [[1.8610203333348216, 3.0699057164170411, 0.23266398898973975, 0.86276350677339919], [4.4225378923856891, 2.6251484566860959, 4.7074613058449417, 4.504756596798587], [3.7759974598874328, 4.2061456268142416, 4.4552003164739498, 2.1120596726468115]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.9719627776223283, 0.94303977083091095, 1.9055735941335854, 2.2999368780972147], [2.4407826038415839, 1.8657651242853095, 1.6515752016539278, 2.9291495810174748], [1.8053850790806711, 3.8691622350982264, 4.4208801616102056, 0.039303808640057163]], [[0.75046822642875699, 4.8805463094186363, 1.0072801220145013, 1.4581295350333117], [1.1222309620901421, 2.1868459598865275, 0.11342625018985387, 0.26493810570053178], [4.3130433672247142, 0.14830752094844563, 1.999077888363499, 3.2113567039510889]]], [[[0.15876946988390439, 1.0241716045774498, 2.4452417846888261, 3.0196224826619149], [2.0569856742415222, 3.2692980621604608, 3.7190553348226771, 0.14751272819858818], [0.25480785728266314, 4.6733175327231011, 1.473982171364145, 0.89024683933677839]], [[4.1554209283480219, 4.5882087626060803, 2.9798086714003, 2.4968082521512884], [2.3364254892270742, 1.9779384977970658, 0.050784706857876234, 4.0389564350533744], [1.1780416163320697, 4.1240304699523609, 4.2759683706270852, 3.1465929148443266]]], [[[1.2929809405762001, 4.3509545744393057, 3.5754373844270155, 2.5402714056688267], [0.76349062009093582, 0.91109580660250611, 3.0510276178845821, 2.8687634569573981], [4.3693321975656882, 3.2075771982911085, 4.6278434300185802, 2.7935012079818158]], [[2.2009227293102769, 4.8451180563229332, 2.2844282068485851, 2.988308823185081], [0.77418638430738684, 0.31462981850746108, 4.9561769271848526, 0.13692620774025929], [0.16123664626596249, 3.8543716186878236, 2.585728022694247, 4.989896549267705]]]])+(1.-msk_arg1)*numpy.array([[[[2.5693728987960971, 1.710855017803256, 0.56217571349171858, 4.1596042585002833], [0.43015921416422326, 1.1585790013461195, 3.5191765886879165, 2.4304109043820423], [2.5790214987016098, 4.8671389697626326, 3.6439337258249638, 2.1925978845014598]], [[0.62500003347000932, 4.1010167434192386, 0.26669788235289554, 2.9336653437482956], [0.97282336017568283, 3.846736947374048, 3.0705069779395449, 0.54989618486178637], [3.6092709971541943, 2.9570817379279313, 0.51240665994448553, 2.9551634591045373]]], [[[0.24383666186997577, 0.22973098857002547, 1.2431857728038567, 2.9458638328515545], [1.3309390691515504, 3.1804703267283454, 2.2370421357397432, 3.8776112233183802], [3.9276030566282905, 0.78775925156464166, 2.2986207984451958, 0.77712874342979077]], [[1.3735627151765781, 1.1291719375396336, 0.33347661226810033, 0.83766189469100327], [1.8403194866499577, 2.984683479701109, 3.4191156694528901, 1.4682286837715017], [4.2321401039122364, 2.0460094600980141, 0.34232222770745763, 4.9463508745267246]]], [[[3.3683963153186536, 1.6051774611855731, 4.5655290903289911, 1.4863989391609591], [1.5206955516303395, 4.6654860172007231, 1.6615812867479092, 0.24689396637151911], [1.7653349116690833, 0.89832176094283156, 3.5723010289544717, 3.6395702524765277]], [[1.6569706854571884, 0.9287931693634619, 3.538383018032822, 4.9726125855597809], [4.6558019432395685, 1.9663684400366501, 0.31874119925078997, 1.6189131410465516], [1.6352572261263967, 0.81971853704759068, 1.8194873970770127, 0.090651720832823066]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[139.55556398090226, 1.0242892188976613, 11.217084183385808, 0.4176221034607861], [43.769933248765348, 16.347397665228488, 12.548138903367747, 28.210635338176235], [8.7488868812560163, 56.22356752126349, 256.17951040899277, 1.0474248917035034]], [[1.4234596517778819, 2384.2640330468998, 3.8667289709915789, 4.3798951347994608], [0.36693483816153061, 2.1425074483118527, 0.93928207651770212, 1.0884275127130856], [468.27396542193412, 1.042729942548015, 6.6255905182147821, 73.312366015599906]]], [[[1.0997191330254195, 3.8780434628206377, 5.5722700651373867, 1.2215485979571821], [1.8411026402852568, 0.74730743089782459, 0.15351832206296559, 1.0317926421794252], [1.4414381096530018, 744.56289019660255, 3.415193585995961, 3.5123110763462311]], [[103.58441622132476, 189.82255601935654, 111.80437283507669, 4.0551258222517212], [9.1778642784737627, 15.503548034699463, 0.99055196873561724, 37.538810752842032], [0.013371440170787898, 3.6580581119998592, 17.917747517299613, 108.2670659758559]]], [[[2.3475707583781324, 666.24964485293344, 7.3138442474633898, 28.349404220011689], [2.1323659223069975, 0.96495352655136113, 20.270803363488579, 74.406464000329251], [216.45259921887117, 54.962857921121348, 13.464200134639537, 18.896276443848887]], [[3.9237503490510997, 229.18084432111655, 0.035755121056109763, 0.64331666474992832], [3.1613321828096179, 1.3548097255606353, 2159.9823949815095, 1.2288665882452852], [1.2389068524243472, 253.91125861465054, 47.620526688201025, 41.710967972776523]]]])+(1.-msk_ref)*numpy.array([[[[24.399921772989504, 1.0445003704248348, 2.0404781596515718, 0.20613851771288802], [1.9464261852136515, 5.6690701762402025, 219.20623805850769, 15.975514091486952], [22.161270874813724, 158.95585670093783, 96.66191232883385, 13.260910002007561]], [[1.3418621816524894, 688.52114782834894, 1.4305768378186186, 19.524720845395521], [0.41933118198982455, 3.8203550938018616, 0.18347405891998933, 1.1922841400781323], [171.69056302115141, 2.3031685721080977, 1.6236691016084541, 52.044938887472412]]], [[[1.1571780094087072, 1.3552866671563442, 2.394916245410478, 1.2155919873064078], [1.4842751002268899, 0.75324517739253194, 0.32394309517082387, 2.276664152015083], [280.33561070576906, 3.0486177070543574, 6.7895581974453272, 2.9941109375864414]], [[4.6360692067388998, 3.6367063179480339, 1.6953028875210761, 1.5994889629834259], [5.7321485287682599, 62.566959837292018, 0.5277570389668631, 3.7355123879814589], [1.8546177658351319e-07, 1.9030021031932292, 1.2598951712239741, 1578.2217980306673]]], [[[9.236630325686221, 11.008072451077, 12.689382430326827, 7.0783610702618542], [4.518725958090231, 0.83303120264788211, 5.1489461160157877, 1.4490259154239344], [8.7811870096585185, 3.0713406105478396, 7.4409987588247919, 46.020667457600503]], [[2.7987860156269591, 2.8342516457784468, 0.0057444076035051504, 0.47997055329188054], [1014.1890471802593, 6.6713071867167661, 1.6384940113061921, 11.435024671806501], [8.7820366753062284, 3.246456064047174, 15.156738500667617, 1.0701266369016877]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(0.421424419096)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(2.48771302465)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.116522636314)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([2.8044231738848495, 0.28683471867959537])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(2.59099652656)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([14.466420182057547, 0.039330155537770611])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(3.29361422974)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([1.9584220519472992, 2.1728625457493935])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([10.323374696538368, 13.330047183754633])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[0.25550657320718656, 0.21576616323548928, 1.6007565718699253, 2.0602761138560388, 2.5272617509701587], [3.7042390304618067, 0.42709866525505824, 3.2502128567956508, 4.9788691002231227, 1.4628378573699032], [1.1193837887348634, 3.9014355756107673, 0.63318523186864273, 4.0886672226897094, 1.3350026503909793], [4.7957252225829077, 2.228768196109427, 1.8202119249130493, 3.0845908676539797, 1.8628602148186784]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(2.39282877523)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.038195714506104941, 0.025488024080060838, 3.0825877041505878, 5.6385806443048878, 9.1933341068786891], [22.950879877253428, 0.13059221235427232, 16.784843440785881, 46.570877281752104, 2.4847700266982975], [1.309780112119054, 25.983571894947978, 0.3350403643213487, 29.067684984197548, 1.9964445224120344], [42.57628269623828, 6.8054948994438957, 4.1920581627738542, 14.810373757296736, 4.4309323427873757]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(3.20499738764)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[2.0984222352831177, 4.9671864988199976, 4.2530983033755678, 3.3679707546568483, 2.5460830815214894], [1.5127062145520227, 2.2244514474298929, 2.5623205863447733, 1.8885515183930697, 4.3534068881008938], [3.54396120107715, 3.2290210334673985, 1.7982176907430092, 3.2992376499703044, 1.6485449015411362], [1.2575887414979274, 3.3715903373108942, 3.2988821727609396, 2.9784452182836438, 3.2169640809160116]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[11.519670330767186, 325.49207741245675, 141.68857974190519, 50.537392749584193, 19.40346528106824], [5.8232934366516531, 13.341022438820303, 19.773915650797708, 9.0215598132640604, 159.24789696006306], [62.034555096673017, 42.986117116296505, 8.1206030594460223, 46.649363859885611, 6.8214962182524568], [4.326365386620715, 50.75089643444992, 46.63005370017666, 32.105544097665444, 42.38668742963597]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[4.974826281145547, 0.36379328716806247], [0.71068324616236833, 3.0910483512011573]], [[0.1159133926556223, 3.7991056693207264], [1.9960782533366048, 4.4413836654953958]], [[0.68851230284691112, 4.807725641165626], [4.0960158985732615, 1.8616187867657379]], [[2.8451932448764063, 3.5419255151200502], [1.7361114847079329, 0.79332171085314085]], [[2.8457664221535515, 4.7984405903146357], [3.0708790836941682, 1.0223864608847488]], [[4.3010952685677335, 4.9638498467355028], [0.95721806919420604, 4.7434398236706388]]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(3.06562116589)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[136.79087871376078, 0.04505539643735354], [0.3509902426221943, 31.803777714506719]], [[0.0013520358547255843, 59.852664297250321], [8.322057097795394, 96.615271393696133]], [[0.3184921202223725, 123.1880852945756], [75.382196069489638, 6.720209388039148]], [[24.668040368604057, 48.279193122087563], [5.4256802020244761, 0.49175611357920346]], [[24.683278113228901, 122.46019610561018], [31.171872891295678, 1.070227786166414]], [[87.561538430112876, 135.86773572707563], [0.8745538915657084, 118.2078240658609]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(3.0351914275)
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[1.6176608024667893, 0.84985827332440134], [1.1444604578521083, 1.5972602878282174]], [[1.2594362289656651, 4.6220908198302766], [2.258328449656513, 4.0834653627893109]], [[2.2709280542988934, 2.7739894703117871], [3.9110587625610345, 1.3283336101718131]], [[2.0122032297089945, 4.2908852131962387], [4.4754311229283319, 1.9784866566997721]], [[4.6077341368231526, 0.38562023177508581], [3.1982430587319057, 4.7485703424553858]], [[0.58832152057045661, 3.8248297788598968], [4.4303716834870759, 0.51330430790259718]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[6.0257836173356472, 2.5691522121516073], [3.5632222355844587, 5.8908326876282509]], [[4.0483926433207182, 169.31985422062809], [12.272542879187233, 93.108683156740611]], [[12.445430165633892, 21.756003895119996], [76.887921440176385, 4.3702268972709879]], [[9.338054439116215, 117.22092671118945], [143.87675048077614, 8.9949496990009372]], [[166.64231610254015, 1.5344074929080356], [34.845628773084783, 194.84737170826347]], [[1.9216772638926132, 69.868224053459542], [136.85591913013349, 1.7681055836044475]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[3.9601104285380049, 0.85836285561754011, 1.4545920094292963, 4.481517127524274], [4.073885793197543, 2.8326448764179601, 2.1437749200623082, 2.9499583817926736], [1.0441358627128716, 2.2185683158498986, 0.2332813033133192, 1.4591140802774936]], [[3.9990047846789651, 3.0333959827994121, 3.1952698992841082, 1.4480197090650426], [3.3040075013052981, 1.8526813220681386, 3.7360962321229212, 1.1341359511603677], [3.3417985007611666, 4.4641557627577875, 4.2728168071596579, 3.8509952113175938]]], [[[4.0829227567880633, 3.5284468809026595, 2.4142380473023164, 0.81390936793558255], [0.71175992094010121, 0.13794865789704197, 2.1624607083017091, 1.3539502735357785], [0.63378427519773561, 4.6607344710884835, 1.2204188651597621, 1.9032755076771286]], [[2.4528807885991575, 1.5423676651582725, 2.5872633127010642, 1.396287960048133], [1.0475068655115234, 2.862509249866469, 4.3546427108131054, 3.9141893587172656], [3.51515868143556, 1.1480494480676979, 4.9183579242758322, 1.8900484744553521]]], [[[2.0014960624865639, 1.1818212193246445, 1.3806154105201325, 4.3157892231538577], [3.228623736889682, 3.21318114733316, 2.5756891731338722, 4.4780711932622008], [3.89222754595608, 4.7078725481047137, 4.44460432422464, 4.4344847459586543]], [[4.6903681051841177, 3.6079514844200786, 1.2417203653239037, 2.8213567933772921], [4.6049007835741378, 0.95038516942302143, 3.1511983003766799, 4.1302909360486701], [1.2048162595773617, 4.461401467456759, 4.8379309029887336, 0.58923222309063283]]]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(2.90095275539)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[54.190184378418238, 0.64207012165617572, 2.9655450465739466, 77.580633991888021], [58.831084441046571, 20.501611949575775, 9.1355599442599331, 23.062888407752627], [1.1334782930948912, 10.091154634498411, 0.014663902011381727, 2.9923690706012125]], [[55.748616211678282, 25.006538066981175, 29.077198554494188, 2.9268411390126996], [32.041548071433084, 5.982424766887795, 45.767556988714404, 1.4407244385603215], [33.116312991152007, 76.711968508052536, 67.55730384142359, 49.971173196925264]]], [[[59.210466254341121, 38.771634144054715, 12.895166575418635, 0.55028203869419157], [0.37292933349941088, 0.0031941922525893672, 9.3684766044154522, 2.4086475560134724], [0.26634315004510739, 86.926965865419646, 1.7822075024547621, 6.4687610285602748]], [[13.503083319437183, 3.5149911963612244, 15.762720104293672, 2.6336957683884559], [1.1441267799012835, 21.134947548857191, 71.379104688469809, 52.387292692679601], [38.349566820765425, 1.4925979841505979, 101.6102485895283, 6.3392071700769828]]], [[[7.4854146621711868, 1.6235635623333526, 2.5488503258096533, 69.547210136155698], [29.966466199289997, 29.552559115012333, 15.559029097128814, 77.407708720952556], [51.539138880280014, 89.501983259314841, 75.741382749698829, 75.242196081596873]], [[88.540014158608514, 41.36060962130788, 1.8739526504270052, 20.265503989613705], [83.940333652927947, 0.86275588533718006, 27.928946413959704, 61.225275389889511], [1.7169097839068417, 76.574747616618879, 96.864653893274422, 0.21558173212724008]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(3.43795393556)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[2.0376751831356974, 0.71472690884350598, 3.2728771990285135, 0.76182498058141124], [2.1697479106042956, 3.4367968091578605, 1.8041471546187271, 4.205272172353502], [3.9290449723009817, 4.2453447163159828, 3.8733604005700588, 3.4809301771549532]], [[1.9751241973642057, 2.8836618173854101, 0.35536584420846762, 3.1276175433418518], [2.2985000756946272, 4.473712654500611, 2.8456011381839454, 0.050042392285355342], [3.0779800674529585, 3.9856108893806006, 1.6525408409365046, 0.30374568794664386]]], [[[2.6576989054381777, 3.1652382454400851, 3.1242365111452202, 2.3033185157715765], [0.97651929121861869, 3.2255594362801467, 1.9839077872483093, 4.6481236163606416], [4.8542782573850047, 2.8727671989493908, 4.8470555940461839, 1.9877056050704147]], [[3.864304651299602, 1.7122596424407606, 2.9906600805238321, 1.1203071962636502], [2.820959923352842, 4.0501686863924737, 2.406262168824203, 4.088525375838179], [1.3049641441371111, 2.7899461429358472, 4.702778351410136, 3.2272067361564938]]], [[[0.67884237507570477, 4.1995150156467647, 2.1464744430024818, 1.2384208186116432], [0.061504484102775904, 3.2940743676645838, 3.2538195521218145, 1.2395796718312813], [3.9038917840546277, 4.3885616221848629, 4.1337112753024465, 4.6703568462088834]], [[3.2095940967472223, 2.854633323328025, 0.20754874416836924, 3.5553437247840018], [1.5070102213140744, 3.1380153463680567, 1.9301188512946035, 2.6587878422581253], [2.4097223713851639, 3.7970096346206876, 1.3945820236834241, 3.4105903544063438]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[12.382413689692868, 2.417174919368974, 56.91729034604797, 2.5619271593989743], [14.575918128656777, 69.687408125674011, 9.2803470031072859, 180.00629391282069], [127.98145059289712, 189.13791531782195, 119.47674700108321, 73.590717926120021]], [[11.461968549268065, 35.197189513274537, 1.5508961619401962, 47.570909577418405], [17.087782726123798, 250.75657848434059, 33.581186931596271, 1.0637455043303825], [44.742564664945107, 137.24082898074516, 7.6958703238188022, 1.4551200626863738]]], [[[26.627111987099934, 49.833046701006374, 47.37270760033212, 17.189761176901516], [3.3396991650392196, 53.686822151313436, 11.58696919025188, 311.02044865278083], [401.18962048603777, 34.726834883795107, 397.62728127193333, 11.641437800210278]], [[118.14811327779094, 8.2848554910266099, 40.169013004943025, 3.9886029777260834], [32.574739957049047, 148.6297278793165, 19.519942298979988, 155.83908056792606], [5.0101685924142583, 31.350773857072618, 332.73637497010679, 53.796143678725038]]], [[[2.3124012846851345, 178.73109949448164, 14.162970753477602, 4.6149303522545537], [1.0789091048709369, 58.426821290563026, 55.593446578623151, 4.6215392326956479], [124.06731173584437, 225.72820714064446, 164.78192709618853, 319.67790901029122]], [[52.63874002891535, 33.957835771637278, 1.2921365239780367, 80.673524191232971], [6.4299805926986435, 48.18565842712529, 10.842335530042403, 26.6629416157982], [19.603527956943239, 108.72676580949542, 5.5964730111415619, 67.468303373546775]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([1.8886382967507336, 2.2655142538524])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([3.497853548921976, 0.49037721444803473])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([9.2454495157077972, 1.4933640956044392])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[3.956455096567721, 3.9581250921474997], [2.173416369025527, 2.4065224794968305]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([2.8158228293865903, 0.45911747228493033])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[48.073810791322884, 48.130970416897007], [1.4281973124029557, 1.4965897874190874]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_Symbol_rank2(self):
      arg0=numpy.array([0.53263949441311731, 3.6721494801184997])
      arg1=Symbol(shape=(2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[1.5718810921683852, 4.5103309975164345], [1.7319570446348358, 3.9287588198951493]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.37152306412260128, 0.058361161804177747], [9.5151969596164214, 165.74323582542874]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_Symbol_rank1(self):
      arg0=numpy.array([[[3.4932528490642349, 3.3093949541685692, 2.7121848982489585, 3.790019102052915, 1.6116148240129442], [1.2737132709390222, 2.2345872704843841, 2.1805572502594885, 1.5364433086022455, 1.128232351604914], [2.3815588558729801, 2.7166068008323245, 0.44005271972593168, 0.87975271588612702, 1.0008727895824068], [2.5932611850696659, 4.9491821270566279, 0.42415997594945681, 4.8593861826583629, 4.0711854037089532]], [[2.6067515234175329, 2.8550667387882296, 3.804210045893162, 4.6680929834998022, 2.053371302170961], [1.0442621872951767, 3.3769474262508417, 1.968392726107417, 3.618110597365892, 0.40071785635209023], [3.2960353512324074, 3.6870845482410219, 2.8140191564496178, 2.6483788980492813, 1.0804282446478048], [2.0846069460762817, 2.044935104669237, 1.1770907132112747, 0.84360892684111666, 2.3476616896963796]]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([3.9917994439820812, 1.7575663399854506])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[147.38907711574302, 118.77695135908914, 53.669026116578841, 204.08938370460896, 6.7196299387780591], [2.6267881535315873, 24.769980778835404, 22.464333210513352, 5.5531144212293846, 1.6186930346062427], [31.941462904036907, 54.019165817658475, 0.037752202815591641, 0.59965124370599321, 1.0034885523377093], [44.873636904500366, 592.15879394756701, 0.032596667816793615, 550.42139731733869, 271.57105096444144]], [[5.3866990718904511, 6.3208547499132264, 10.467749775046615, 14.998818808094068, 3.5414599871902803], [1.0790934001121733, 8.4901725849241974, 3.2879175218085472, 9.5844871655928117, 0.2004299691038533], [8.1358898346961848, 9.907934704681697, 6.1620057320189652, 5.5387991810647783, 1.1456371125388569], [3.6366889867895877, 3.5159272891644675, 1.3318433767556563, 0.74163155622362176, 4.4814288122528643]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_Symbol_rank3(self):
      arg0=numpy.array([2.9727781081409548, 0.10284126662361193])
      arg1=Symbol(shape=(2, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[3.9114549098975049, 2.956365714570933, 0.83764684768045594, 4.1941958898593468, 0.77365443853110083], [0.75705929136351313, 2.3283856705960377, 2.7258902317583438, 2.3061913275610602, 2.0227748108838148], [2.6829508841693976, 4.4994498316514671, 3.7834090179071636, 3.1620228135348465, 2.910469480512567], [4.4343567895929326, 4.5060731303834896, 1.4022120552138231, 3.1453151338814984, 3.0084705882308591]], [[3.9844800065270189, 1.7538271715170806, 4.7188235325150485, 4.2492237995012241, 0.78035369762646722], [4.5588733645833903, 3.4783929741186759, 3.5630367372322942, 4.96293609226211, 4.3636452142243529], [3.517005466282058, 0.86533130106807488, 1.8392807013500749, 4.5909818985187965, 4.5010498514397872], [1.266523063764895, 4.6028520929776375, 0.28470758848718702, 4.7515243041373871, 3.6136319962111236]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[70.917556092889299, 25.051940703368821, 2.4908241720381956, 96.501971993267802, 2.3230805234549434], [2.2814558166205408, 12.638768852390683, 19.488976000237287, 12.336820348799776, 9.059436277526796], [18.598235043132433, 134.57701915744838, 61.683249894366334, 31.343724469995017, 23.830052464767686], [125.36356056880686, 135.55164773133723, 4.6075934619089978, 30.778337215533703, 26.515232955082929]], [[0.00011587795516903278, 0.018514516490104667, 2.1806821099880963e-05, 6.3456816112560083e-05, 0.16948887270529833], [3.1375911277162539e-05, 0.00036637858696621678, 0.00030221513923276621, 1.251555232447237e-05, 4.8915790501678895e-05], [0.0003355733052008361, 0.13970036655042453, 0.015243987842771407, 2.9166108847222236e-05, 3.5786258821205739e-05], [0.056090233812172548, 2.8389172086781232e-05, 0.52330702811443686, 2.0243680437035251e-05, 0.00026936215060126235]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_Symbol_rank1(self):
      arg0=numpy.array([[[[0.036081177871003879, 1.0180896808467192], [4.0952183610432993, 3.8287438150350428]], [[2.4237485035781821, 4.0063114023311526], [1.4338436138419151, 2.8487130965278329]], [[2.7312333587341056, 4.2522966071531219], [3.1599635558895205, 3.4898970083813823]], [[0.11888068336472656, 0.76200871893066069], [2.8404180390201557, 3.8922001837889373]], [[3.7416675150913106, 3.4366916739576232], [3.7827797514714145, 2.1405561130874569]], [[1.910413131968782, 3.0834799199963938], [4.961555113430105, 1.428957351601214]]], [[[4.2845960127378504, 2.6510541343968974], [3.981041804645828, 4.0710341228871814]], [[2.8409141234521034, 4.8220981832584924], [1.2563454589807592, 3.0422616915339553]], [[2.3538591497097894, 4.821643504807235], [1.1617381439238892, 4.4165664661543387]], [[0.17994678376038156, 0.97044527990046447], [0.59937479398037907, 4.6808668155560342]], [[2.282245728131044, 0.96593815496765167], [1.098140069071792, 2.022435999803502]], [[4.9449859490516648, 4.8678440605535753], [3.2970878153451881, 0.94366244429162194]]]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([3.84759735998496, 4.4907522888232485])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[2.8118817673778146e-06, 1.0714145251808738], [226.87968678629224, 175.13216333340594]], [[30.154580947354088, 208.50617836850628], [4.0008830813598433, 56.14423707405134]], [[47.745560760227306, 262.23479192769054], [83.671090324631791, 122.6095694050277]], [[0.0002763125763420641, 0.35142235033168789], [55.517818760388899, 186.56637508356837]], [[160.2965315616149, 115.57211061612794], [167.17998902006988, 18.695338763787827]], [[12.068815193466758, 76.143588409706481], [474.74159811516245, 3.9486780254755653]]], [[[688.25715796304496, 79.701781621426562], [494.80801400372667, 547.05748974009077]], [[108.73466649130623, 1170.1562326700116], [2.7865978808043677, 147.8822662042848]], [[46.727657004988558, 1169.6608283378691], [1.960582032139998, 788.70739030138679]], [[0.00045189403833953808, 0.87395731567012547], [0.10039195667359763, 1023.9277087123174]], [[40.673990307967031, 0.85587655482023695], [1.5225930107359216, 23.637846100330812]], [[1310.1569970165287, 1220.8397663621151], [212.22463406312218, 0.77074001929297886]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank1_Symbol_rank4(self):
      arg0=numpy.array([1.988727885764211, 1.2978827506864123])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[0.4900910633465243, 1.9497213112124872], [3.7984548704207626, 4.0167029001441223]], [[4.3775200092862443, 1.8502091870890922], [2.2857317361614675, 1.1271163446625141]], [[1.2415310451987045, 1.3528713433199879], [3.0961621291256756, 3.2658774574880272]], [[3.6298187290719546, 0.6331077058259047], [1.1402270504307679, 3.4127624431455938]], [[0.40812128814010079, 2.0608267526372295], [0.83248386576032951, 0.97883165076280565]], [[2.4793150959532788, 4.0135411414373321], [0.50230814946769486, 1.0176991579908552]]], [[[2.7899140853211, 0.64603913440753458], [3.2777708667048424, 1.5524556304598922]], [[0.84138226222523083, 3.6655959547234285], [2.9270151094016241, 0.66010965614354777]], [[2.2897038040297271, 1.9646496872466399], [4.1864445148159701, 2.9048842009280809]], [[1.5542025996754667, 4.9153700345001488], [1.0191996507456456, 2.7199114227116779]], [[1.2547067679441326, 1.5381372752997233], [4.1474452797640486, 4.2513036134169209]], [[4.1537867585621742, 1.209486226365923], [0.60892807523053927, 0.28784479911703231]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[1.4006483607996048, 3.8206631446902821], [13.618367301195827, 15.82298907245125]], [[20.277743613304317, 3.5680167166209626], [4.8135244685332914, 2.1703469841866094]], [[2.3479594001395143, 2.5347437039841085], [8.4030651927688567, 9.4430121558803304]], [[12.127588322391555, 1.5453624785379261], [2.1899979381651948, 10.446403258360924]], [[1.3238993286169785, 4.1239377076144894], [1.7723895173112412, 1.9599952364424178]], [[5.4987301320837476, 15.788632092339123], [1.4124622134021696, 2.013074724167923]]], [[[2.0697467357820472, 1.1834626029609097], [2.3504980414873606, 1.4989703010241626]], [[1.2453009171704539, 2.6006113049456663], [2.1450721479072437, 1.1878123075657749]], [[1.8166684118545575, 1.6690448695957059], [2.9788866126297719, 2.1327301097093616]], [[1.4996532296750262, 3.6024187103393834], [1.3043962507664149, 2.0323121922897274]], [[1.3870026751778537, 1.4933846489816216], [2.9487494734562891, 3.0296909007995168]], [[2.9536290894961135, 1.3707451876811594], [1.1720664943851313, 1.0779391324013432]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[3.782050707962183, 0.19207336170422443, 4.8014151607572391, 4.2776981143783477, 3.2622451848992582], [0.95799460882590159, 3.9628783094575351, 1.0794152532952797, 2.3145442202066944, 4.5619050208453897], [2.7146914844550873, 0.69733261220901632, 4.7289835515819263, 1.5615278153110663, 3.2766956818327793], [4.1769464412931683, 3.3705874198223515, 4.7768739713105282, 0.33310333049944746, 3.581284283519488]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[2.4224909344730867, 4.8350697723636671, 4.6267789016423837, 0.61078759597470189, 1.7895953152471822], [3.4287751521206138, 0.89598670900361432, 2.8361042119003099, 3.8045977529536024, 3.4780257340814331], [1.1405620260539402, 2.2384077901882113, 1.0610127282121966, 1.0236104782935245, 1.6296754630429107], [4.5402342245518721, 1.0454100515309008, 2.6715217514739629, 1.6878292861576185, 2.7111064218614964]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[25.092230108717501, 0.00034317339868174403, 1420.8391454564589, 2.4296030229127901, 8.2982573538483422], [0.86317358315275294, 3.4340741756926745, 1.2420131349660246, 24.358164322932229, 196.12257276480699], [3.1238135912162623, 0.44622595066395171, 5.1992084290662568, 1.5780455186003277, 6.9182074281335684], [658.93845952769573, 3.5617934735947041, 65.214983651565888, 0.15638489833613195, 31.772993946588851]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_Symbol_rank2(self):
      arg0=numpy.array([[[3.8917460115324554, 3.712911837988444], [2.1128574494722221, 4.6511508616735773], [2.9621010785733044, 3.2702162758050775], [2.7473068655546493, 0.41815157014898163], [1.8213353908558534, 4.7142197682148135]], [[1.3370319619085467, 0.19844415818828376], [3.6424303011040635, 1.7789323964183965], [3.7650219300086425, 4.8537421551892797], [4.3900562328300934, 1.0883470871409491], [1.233549618108857, 0.054440220793508423]], [[3.3191105800160305, 1.0466439244165571], [2.8804204652112539, 1.1589581578960986], [0.7379284598999053, 3.5360842380223705], [2.0476311953066806, 3.7873855447662628], [0.93693350685794485, 1.5965710036073597]], [[4.1426263523047782, 0.61820191395959834], [2.0215924674217582, 1.3786949310735688], [4.024762293746833, 3.7632001648628952], [0.79977331701287391, 3.351027529632296], [3.4721577841461668, 2.461749154721339]]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[0.75737996683301267, 1.1993521512141578, 4.0129121320523966, 0.25579115771401151, 0.73843313663763821], [3.1213008295792317, 2.7950693683275585, 2.5286139246643051, 0.27266933688703232, 0.71521035425005663], [1.5645864209487184, 1.8451500365503926, 2.1812412997179425, 4.125640166838175, 0.81437757251632681], [3.7340359187891683, 2.5685335541569221, 2.6849252816728466, 1.7328343904525874, 1.2106405929371238]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[2.7987468224009859, 2.7007877367925497], [2.4526397897530998, 6.3188789143260395], [78.070840863730155, 116.13153918584607], [1.2949964369939211, 0.8000930849320077], [1.5569710957559186, 3.1424482112029928]], [[2.4758622119402758, 0.0064227039106183421], [37.078955097472083, 5.002798369898005], [28.568914780480167, 54.302909727512137], [1.4968624419472027, 1.0233527190584868], [1.1619733317491381, 0.12471445927277239]], [[6.5340659070165206, 1.0739329784274692], [7.0431193581302276, 1.3128485349117205], [0.51535590124568875, 15.720275548783517], [19.235907035719119, 243.23201775530885], [0.94833167651265116, 1.4637669719076978]], [[201.80404762844231, 0.16598666800166043], [6.0979561406523013, 2.2815432041021486], [42.041901218434198, 35.101719955940233], [0.67898116278227538, 8.1291753589228737], [4.5130655519720433, 2.9761555287862205]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_Symbol_rank3(self):
      arg0=numpy.array([[2.2630327887726769, 1.9406215572813823, 2.4734186870538677, 3.0896926771094946, 0.050947347996613018], [3.0977417332944559, 3.0281123806552821, 4.5479725985369219, 3.0724591213492598, 1.7167857108683697], [3.2762374526436719, 3.9165198393743479, 3.3535143334853732, 2.0715996170811635, 4.8958759805721348], [1.0209498659300833, 1.337507031254866, 3.912261690027667, 0.55371626932175677, 1.9603689971985807]])
      arg1=Symbol(shape=(4, 5, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[0.062651121269178128, 3.3797332278941372], [0.8386651025846017, 3.4819972786199256], [3.5115330139787924, 1.8210026913593118], [4.0981773846861209, 0.41403186654587931], [1.3588596916986995, 0.041666699256048031]], [[0.42177296687225302, 1.0617524824938338], [0.706541664192184, 3.6171779784711027], [4.406971525604412, 2.8077425986216054], [3.4671636794516933, 3.8619770143018255], [4.4213554115808957, 1.2007797269170399]], [[4.6212403666033168, 0.98131708253379435], [3.5037112396901864, 4.9142581406933834], [4.9916141756868111, 4.6139596318351552], [1.6247480747748098, 2.5419947949867034], [0.60758911666481763, 0.72142404968133322]], [[3.7699767954204932, 1.3467023924697474], [3.693357118202075, 2.6763397904915092], [1.1366685107280585, 1.0849440356540887], [1.7153693786797517, 2.489497718316084], [4.6814536784796692, 4.5658766224036427]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.052499211706502, 15.803760493842965], [1.7437569183500121, 10.060261299884839], [24.047926664628339, 5.2023079498843563], [101.80280289788436, 1.5952932929110184], [0.017504899908592698, 0.88334432603963176]], [[1.6110521309579777, 3.3217616529567713], [2.1876007206496442, 55.015456627970437], [792.47302337968245, 70.304507432790061], [48.999781847614379, 76.323831595518698], [10.908455250150286, 1.9135610495767568]], [[240.80941740685907, 3.2043994991145022], [119.4955006476643, 819.71952061540253], [419.84992753694871, 265.84993393776881], [3.2652562954201736, 6.3686554430713702], [2.6250250264551331, 3.1452867580292838]], [[1.0813005926896473, 1.0283152326881677], [2.9272276792316174, 2.1777659722309473], [4.7140494649313256, 4.3928979387501137], [0.36277960878917465, 0.22956954827570111], [23.365010962556401, 21.616154902212717]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[1.0308016947022023, 3.5339379116284824, 4.1332729707776252, 2.9401907186530538, 1.4820850896083406], [3.2194902546879813, 3.1299116837026979, 4.0366015173577727, 3.9431699585269371, 1.3681378074937303], [4.7930941084367715, 4.7113925604243807, 2.0006798001748027, 0.7900277565754803, 3.0458201910927887], [0.84084756207353273, 4.5071030174310556, 3.3501684636872611, 2.774757408208258, 3.8474780138205102]], [[3.0650591545784569, 4.5697342234365612, 3.1902307916309178, 3.4277191987898825, 0.8767741332122202], [0.28719054162582164, 3.2843544812627128, 1.8235210729314089, 0.068121919241194012, 4.505481755392287], [1.2217040285499872, 1.6403047390958954, 4.1928000291539913, 1.9396153292000566, 2.0803988023071502], [2.5720449467090623, 4.5758201495217499, 1.0287442926022536, 3.3415023096393388, 4.9180317332636996]], [[3.393752836590207, 1.3532831285807969, 4.5325794533442139, 4.8347080787133327, 2.161082839981916], [3.1425751887115889, 2.2521390032983035, 2.968272179362442, 2.786429843013051, 4.181968185299346], [1.3974127583961948, 1.5862481434610187, 4.6050427714944497, 1.5985811977776818, 3.0035896282618775], [3.7666616867694866, 1.733520837256401, 3.2382131571486314, 1.7562836934256012, 4.5276678040786615]], [[3.3795040585584464, 3.7673817297552956, 2.3232477542509895, 2.3706216316294881, 3.3109389761238117], [4.2947265823387353, 0.72447157735226553, 4.5688582222466136, 4.0390474295407435, 3.4026073338035143], [3.7702682933119278, 2.8826461815583637, 0.098869888725446983, 1.3064414374823976, 0.36635589465140111], [2.2227551670258046, 1.6918908466570994, 3.8301482803523963, 2.4095819814815314, 2.4915426335860209]], [[0.013743816497684254, 1.0065246407703308, 3.682646619927016, 4.9076175079733657, 2.1201493712113315], [3.2944511560020531, 0.73695708539159788, 0.57837064186777154, 0.10963159181646319, 1.4608123357863634], [4.83095244277518, 3.6536319623632556, 1.7552309131422643, 4.1918337089639177, 4.1646181065381542], [3.6944739752821563, 1.805260966955498, 3.4479009964175513, 2.5977746686227459, 3.7095608834888125]]], [[[2.0277522394714707, 3.7747700391060621, 4.1004319364946031, 2.3288285639803834, 2.6143048512422844], [2.4414071756783855, 4.3166526252584374, 1.7821954397002162, 3.5762583419078471, 3.1659949653589057], [3.5189130269489852, 4.3439992213700886, 1.60346162458995, 2.0527552131556481, 3.907901577434814], [2.5374020881214419, 4.9278555627693539, 3.4369211229103693, 1.0384470268256614, 0.87962541105615166]], [[0.17786160494099998, 3.1972774768857706, 2.7552324419562, 4.5256555963906679, 4.5401365573701176], [4.6406614236548247, 2.6018159474262799, 4.4468222803551569, 3.5260929820982398, 3.2263125114650517], [4.3438370692567565, 2.7723633170945896, 4.0824942364602652, 2.4464310960400666, 1.17847530453511], [0.83245910947991419, 4.789391162758724, 0.50466086458933268, 0.4257860433985543, 3.2250788851235082]], [[0.69359888963149396, 3.0304145349506224, 0.21780169104464214, 4.1675805089776778, 3.0137996969588823], [4.860569039162506, 4.3953825386375831, 3.5268553037766339, 4.5792282525753771, 0.29849893310483366], [2.6484117553176203, 4.7056787147242263, 1.0883397771248202, 0.34903578696449433, 3.5450885755076338], [3.5562393070804923, 1.2545045971307398, 0.98994442000972827, 3.7689666920249811, 4.0737023334429887]], [[2.8300462511641884, 0.9606433669208827, 2.129608297849225, 4.8915071000276127, 0.8195763139225497], [2.0605690797531957, 3.4098441250918645, 0.90424543546027214, 3.8622846444081729, 2.1880083606103948], [2.58024054841795, 4.136642873204579, 4.1843173267541243, 1.8096354677611892, 0.77131506635048452], [1.2259163361260452, 0.66136215492586869, 3.719932577396035, 0.16456344371843218, 0.43678328418331619]], [[1.6276113767253932, 3.4183231826994649, 0.25292479725747569, 1.4692625970871585, 1.5302487840143595], [4.0244798147690846, 4.5639335974959616, 3.0425234575898581, 1.4576106698413207, 0.71576378311158906], [1.1764924532183263, 2.1699194226503717, 2.2326806558669734, 3.6574633741051152, 2.1067849111843753], [4.6564306137994116, 1.6916997905975819, 0.58190005226936636, 3.6161749026650352, 2.4514115553498534]]], [[[0.045712347147976236, 0.68402811361771609, 1.2799146790182232, 3.732469771209467, 1.9808529519362363], [1.3786152765356048, 4.8777081289602915, 0.24004339984631703, 3.1140097464153658, 1.4024834655853529], [0.8958978274951821, 2.0664181869738889, 0.72920974809280703, 0.9813270146507076, 2.6706376675657291], [4.2119374847489892, 1.5375418384756612, 1.8863917789130789, 3.4975937502487282, 2.1717096887529781]], [[2.6993754514573181, 4.7160662390329504, 0.65800397455042037, 4.5393712061653329, 2.610059143805918], [1.8636509509608576, 0.4697885668343591, 1.7874169324895257, 1.3872341025850434, 4.2951407721264596], [0.43648187623886192, 0.68477480704253924, 0.74644661076601093, 4.4433885068782732, 3.8503212664290452], [1.1854790331492371, 3.651641337816351, 2.4882485200677693, 2.0687406342262693, 2.2561203123026936]], [[0.78956250887451473, 2.345553583889247, 2.2799820773323898, 4.738973866466913, 2.7853281149359987], [2.7706925877835569, 1.0172649821856865, 4.9066192582579538, 4.0178303809220912, 1.8536049750126984], [3.046836623982089, 2.2871263504409947, 2.5132159507350971, 3.9675946503250517, 3.5561915401823092], [4.8710208700051956, 1.0884738556173563, 4.4716428278858764, 4.4157937248780605, 0.6485351539979568]], [[2.3068163646830135, 2.1563733559891465, 4.6707938815826262, 4.1049792919402135, 0.98190725148249991], [1.88227075156594, 3.7744072294233808, 0.50444039251693673, 4.403087472637087, 3.5438461157804304], [1.5434416108271407, 1.6509936911125509, 4.5190936723146011, 4.4553399197398376, 3.6823702206641893], [3.5830918413271649, 2.1356331388591769, 0.091490810229737471, 1.0451018639968395, 0.10437234632157692]], [[3.7847412332165473, 4.2320935840107845, 1.9658955075874731, 1.1576922802519849, 1.1213725923818016], [0.88312246737447841, 1.5809788606625446, 4.6554832134627704, 3.5767039778275795, 1.6467363103684045], [4.6777951653195649, 1.7542043599828325, 2.3667655383650716, 1.5883390572611549, 3.728391740257698], [3.1667579776596972, 4.0042227702518192, 0.30043881478975837, 3.3248534938628551, 3.5386226401913592]]], [[[3.6930973132188827, 0.50681342629242598, 4.1147012971012229, 0.92681301543167394, 2.1515474865430884], [0.87033380039138919, 0.59203197503214255, 4.2086517931075251, 1.6520720750268696, 1.5396170926770516], [3.8331410864264073, 1.6909932492298088, 0.073167107076226548, 4.9323676075472074, 1.4149609689687355], [4.791583572438018, 4.0340651460905814, 2.6180595092609158, 2.7728003746159278, 1.658867731964925]], [[4.8346033356015656, 3.8212104232189232, 1.725220378083814, 0.83091611047493563, 0.2635930239047799], [2.0542732946765305, 2.3360997042671467, 2.9524376333455984, 2.4229630462339142, 4.3613524253371914], [0.92935860074503585, 4.8505110363150221, 3.7622132271113267, 1.609076379655163, 3.3306743209159095], [0.85357676133546756, 1.4058362317492701, 0.77783045634185433, 2.3793297395093607, 3.4404381023658206]], [[1.2024500267297173, 0.79449613501644167, 3.0642047566442985, 2.3235852780573119, 0.087658158614806456], [1.1123442151980087, 0.39843855290997338, 1.710046663371918, 1.6930614671005233, 4.9889589255212856], [1.1259591964129825, 2.51011307070904, 0.10379246710826476, 1.9595984375810205, 1.4179620798601582], [3.6910216647914686, 2.6023538364251508, 2.3434042652619769, 0.44244010491172447, 3.390423201760675]], [[3.8241885670774618, 3.6039601395550398, 1.6642768341027672, 3.2186422004231132, 2.6080649291400348], [3.7636250782620633, 0.63603519746366455, 2.2322391696192412, 0.67373996994908014, 2.2147422037692608], [2.1936654705299721, 4.510795663339235, 2.0235687134654912, 0.73241685889702179, 4.7632821254015933], [2.2682463877959749, 1.9403332488394553, 1.3057751925283725, 0.77119030342259931, 4.1650231960231974]], [[0.84423531068708213, 0.13793565324863322, 1.5484335008503516, 3.6122517573592714, 2.9248592778785536], [0.64602013016650406, 0.9791172461544253, 1.2489764184705261, 3.6173173522867423, 4.1091734050564686], [0.23285827954528804, 2.3636953197832358, 2.9906510774901891, 2.9688616157704968, 1.7128130113850619], [4.444465282396183, 3.0021670551120989, 1.5993643502008592, 0.17216379734657775, 1.556419518723432]]]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[2.0933273823518066, 4.4323014162175944, 4.7843412091598525, 4.6279996492503415, 1.1122041880622815], [4.942099681744371, 4.9026186640513139, 2.5158118889709939, 0.9809674174327373, 3.8567138295979984], [4.8595643286238177, 4.6754801430001312, 0.99106807168394206, 3.5918642448090403, 3.0795701804270732], [3.9701201202893448, 4.3589871558943321, 4.4011718378296161, 4.611162074691487, 1.2186094632136806]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[1.0655647560138428, 14.050293598916792, 19.503173760208256, 9.5601182514188601, 2.278732880420109], [11.560182009175316, 10.89709855615196, 18.560498394666876, 17.672569299907764, 1.9273666102943334], [26.592062188129375, 25.652036251358208, 4.270348220272048, 0.61056503246313654, 10.29322284505429], [0.69567852696328014, 23.378762011960657, 12.56424475346321, 8.4686672708337412, 16.786697001281709]], [[143.23318560580745, 841.07661366386367, 171.03753083756726, 235.12806488700247, 0.55829173546520028], [0.0039668585232058851, 194.56386055230723, 14.336239522406194, 6.7417984398291677e-06, 789.91107594821312], [2.429179186469248, 8.9662396274268001, 574.28308456393336, 18.847002832818898, 25.71115603084365], [65.838240254364464, 846.05275967977343, 1.1338360950271567, 210.02347013336592, 1164.7364772043807]], [[345.9036918146615, 4.2521464740379225, 1380.9579922805578, 1880.4457980974232, 39.919237903581227], [239.4331939030744, 48.633520302424586, 182.2296439922797, 134.66726073726872, 939.50134697113026], [4.9577311310228422, 9.0916736705639654, 1489.8281310026846, 9.4348787579964668, 192.8393178600239], [569.60391527852414, 13.903309098130235, 276.36008176361082, 14.79872686681475, 1373.8131272040876]], [[280.24080413032175, 463.35216126606525, 49.464083428007598, 54.307814882703184, 254.87888687087295], [849.6212498081743, 0.22499810749953314, 1131.3310811281419, 639.52684882936262, 289.21777450646482], [464.99747836926645, 134.24995851897225, 2.2343785550233826e-05, 3.4455950368498898, 0.009588354606279435], [40.310142640545358, 11.40005937569641, 500.17448622857762, 58.563373468112729, 68.368375290442671]], [[0.0084956222704646485, 1.0072593839752695, 4.2627079123357845, 5.8666380395006357, 2.3066735887626271], [3.7660016184551988, 0.71214537515318699, 0.54390717046387949, 0.085548517311765382, 1.5242724904434957], [5.7647981164890094, 4.2253712882307735, 1.8696038179010663, 4.9231195988861227, 4.8875826370787356], [4.2779370695431433, 1.9289672467385834, 3.9616005685946543, 2.8914887023490845, 4.2973712360755147]]], [[[32.907749956520455, 709.66004289347109, 1068.2305834234032, 65.227422439298607, 115.50906955330558], [82.367634088798923, 1377.0835351127139, 17.387864490103844, 543.37632311085065, 297.55784069930758], [501.65533705571869, 1420.7402188222802, 10.313828269918769, 34.962412902844704, 842.25648281737608], [99.662596101532984, 2649.626954416658, 446.48146822383598, 1.2049612403007497, 0.53053496562155134]], [[0.00021059062373352365, 298.36333907273684, 143.85601301513631, 1638.9147514575757, 1664.7856724872286], [1853.4760584642281, 108.62822150888159, 1503.6297583796315, 482.13937898642985, 311.88439788184934], [1340.4501250312605, 148.29461481519721, 988.86077982433676, 80.32099506841233, 2.2369544785542725], [0.4069768599653848, 2163.4902249756337, 0.034988105714108327, 0.015207819312090889, 311.30017988276092]], [[0.3983442902808807, 16.269288395896439, 0.021611584850733334, 36.266991671698904, 16.04581074892484], [53.404224255727598, 41.462805882304657, 23.830016235188179, 45.96519461668386, 0.047758954567110753], [11.591829031698264, 49.225610217502783, 1.2373500261891623, 0.070785907652819516, 24.14117305273723], [24.332663650289454, 1.7690400908280988, 0.97489451887529777, 28.162221604690998, 34.246667124079451]], [[2.7745639264589088, 0.96137776838728539, 2.0991880171279611, 4.7459243725777682, 0.82268582712724858], [2.0324093853654466, 3.3311578497452965, 0.90597937165505171, 3.7642210178467175, 2.1556436390708877], [2.5341086959349304, 4.0263509269725812, 4.071866126794756, 1.7893218929514652, 0.77513631584755927], [1.2211730020010507, 0.66658700480510968, 3.6280754714613108, 0.17031330115829771, 0.44372375171534928]], [[6.5447125289642898, 114.48917969756432, 0.0049832045756018689, 4.4101631423819843, 5.1591045224101366], [214.87825532337456, 349.04631914542131, 73.062579514929837, 4.2767964946883357, 0.27535208525239419], [1.8717254018213205, 19.841144158854057, 22.147484657111963, 148.60178963694116, 17.705593866231002], [377.12885967597651, 7.5958715426175392, 0.12390463639977459, 142.23559317910477, 31.759009965819999]]], [[[3.0785534475785579e-07, 0.15795422916884935, 3.3178218356509483, 602.07807342670924, 27.705917827907147], [4.7602816548132427, 2210.1868899676001, 0.00097381905570940563, 249.64341606294937, 5.1743888851657838], [0.58613415501629196, 34.026963541856787, 0.21553744702273231, 0.9124696197092923, 118.34853627172784], [1083.2034778370644, 8.0890504585250422, 21.849974026065599, 439.01787720657364, 43.322136052880005]], [[103.83959069992299, 1410.2932585191318, 0.14129638155516974, 1179.6966402416192, 88.723860675397418], [18.368944542548125, 0.029240607302080595, 15.110437409469144, 4.6197488313002246, 910.90539731527406], [0.020733336183393177, 0.17025774767445409, 0.25480513719455594, 1067.5182022029157, 546.36069391923638], [2.2155897938962181, 426.48328175749464, 70.956696429993897, 29.927975811998561, 44.888822076615867]], [[0.79123056326654206, 2.3277608187446077, 2.2632598292686366, 4.6735742832425391, 2.7599598172642374], [2.7455867830176865, 1.0171094608746383, 4.8374037905207237, 3.9682295672382186, 1.8434156751704784], [3.0166676165640887, 2.2702882612182327, 2.4926137331738194, 3.919054410750936, 3.5161207787033515], [4.8026199198298842, 1.0876499552058199, 4.412220229231484, 4.3576024437079255, 0.65104846365417091]], [[20.132304946198893, 15.801239256538949, 253.72340474980143, 159.56210104746646, 0.93652245558555247], [9.6966292679187038, 118.0218503169685, 0.085611743985299138, 205.25269873120914, 94.110717849868252], [4.7536940426338949, 6.054969585271893, 225.34892950719626, 214.1370931922539, 108.00725901788705], [97.908247584859609, 15.262124651553316, 0.00018595543825511944, 1.1716966022153255, 0.00029846413840511015]], [[60.270253632289624, 85.019961364305189, 8.0175180643770911, 1.5697826668373658, 1.4230115438445576], [0.68197377931134506, 4.0983285524002904, 114.036629306037, 50.639538033237081, 4.6463187573582045], [115.72812085767018, 5.6449755175530427, 14.19833146941779, 4.1573705368600322, 57.54938453203107], [34.807965166203026, 71.696371546015129, 0.024644106334991516, 40.442227978563295, 48.997463252860364]]], [[[178.89962840611011, 0.067330454948102317, 274.78745326673373, 0.73952824804623563, 20.94407503870805], [0.57616328504069936, 0.12479091001557997, 300.55410515425501, 7.3383983276585552, 5.5469088087079248], [207.38754527191972, 8.0491615418654909, 3.0988292077012596e-05, 564.30426758998567, 3.9671042614056762], [503.01812786352184, 254.02245222280197, 45.648748794200429, 57.337672744810682, 7.458973920230946]], [[961.87833640834572, 344.98997834201651, 10.774632823768158, 0.44601634935084378, 0.0029912889129202389], [23.060736156123422, 40.387704330999568, 112.07578092887758, 47.354660071627613, 613.90407539981402], [0.7266269664804007, 975.75074466198487, 322.36687971788871, 7.9517919055736188, 189.5444235646622], [0.50151889731204913, 4.4141348483827025, 0.334479013067571, 43.748272612061321, 218.31945502632468]], [[2.2510688872251174, 0.36331789948766297, 138.15548384273265, 40.881515758254359, 2.2235527417524493e-05], [1.5977420578218693, 0.017422948239740423, 10.604936638584995, 10.149110007980006, 1180.4881700974386], [1.6856214410516663, 57.427357191081562, 4.6771013435353907e-05, 19.314236470290787, 4.6505256088421847], [313.40766869436538, 67.312956482096112, 42.438610064258789, 0.027627700545773608, 215.64459720206131]], [[485.49316662403959, 369.31991158806068, 10.473864457842343, 219.26085626608918, 83.121163426992922], [451.03909612198635, 0.12411370220008355, 40.560192066019802, 0.16186399588596559, 39.114804001109469], [37.42760230466083, 1039.6025403074625, 25.796424186544325, 0.2378914051086215, 1336.3980851266983], [43.666188918023757, 21.254181996241417, 3.4220632630160002, 0.3017752833497423, 719.70806335883492]], [[0.81355654044442161, 0.089454395516740726, 1.7037460783982528, 4.7831451761750667, 3.6982812404322085], [0.58717024839751708, 0.97461048447872145, 1.3111787707189098, 4.7913203518967675, 5.5966352510243391], [0.16932923146440235, 2.8527396049270397, 3.7999041398270945, 3.7661931672155595, 1.9266426094873814], [6.157990225245789, 3.8177424957144508, 1.7722796284739313, 0.11719564874175531, 1.7144600765114955]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank2_Symbol_rank4(self):
      arg0=numpy.array([[3.1040240365980809, 4.0509709799810274, 3.6147645039393641, 1.1391849390542035, 3.4628442497321177], [3.3168653803949071, 4.7543990937751994, 2.5635175832667909, 4.73330341475683, 1.4259104516176682], [2.2236407855629756, 0.99835690784825226, 0.83327380045092503, 3.341633827801592, 2.9047792826442693], [4.2099838263676901, 0.1782906644520425, 3.3772118116529684, 1.9071233627809057, 2.7207640749873017]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[[4.8051427316990623, 2.5459383274628098, 3.3215550607486741, 0.18841832971591208, 4.9251601686076407], [1.8967426030348744, 2.8980863173640432, 4.3881164276809503, 2.1375493773879755, 2.5115078817958714], [3.0158484180264997, 2.6199836810396064, 1.4650400882253654, 2.9267262569314889, 0.60465024815822732], [1.8350911930339708, 4.6886179943808024, 0.75915342721668611, 2.6781133046338303, 1.3332119699477976]], [[1.5989693070870432, 2.5855534520859802, 4.441212648600751, 0.044362264369233427, 1.8287253543397084], [4.4734408536916641, 0.5711743510104299, 2.4528659918023039, 3.0115195405128197, 4.5517292432810157], [3.494275565643377, 0.64366288351127854, 0.20483619365952566, 2.3399312075060514, 3.8003819051350116], [0.03379242379355589, 4.5685392075161095, 4.4382056063192215, 1.4332956237812942, 4.7848571583050781]], [[2.7891463305479349, 0.60412355781074234, 4.70796912435015, 3.3353693906979767, 4.6282945824886488], [0.17826951323777832, 2.1220963542319278, 3.1126288948087018, 2.5584806245654672, 3.8758816069428867], [4.355416440290556, 1.4427800041206074, 2.9553245214815225, 0.85753739700881382, 2.1528696914887449], [2.3265508313318222, 2.1353355537269652, 4.4614108749619863, 2.3431879137025469, 3.9393115622517421]], [[2.6891780237467349, 2.042477290254852, 3.0132213958923035, 4.5796343588938733, 2.3642962655218773], [3.0361543876016217, 3.9348069379318651, 1.6893572696543115, 3.8319401381335596, 0.5398255636692515], [2.6667335302544419, 1.4145719685827991, 2.0083347043767081, 0.092734793498450194, 3.0182046249890826], [0.84536706267213502, 0.63205931115713776, 4.3273497890586876, 3.2277182451818702, 2.5102658363256158]], [[3.5799021284751946, 3.8899599163884813, 1.9732069093010181, 1.2224627482804706, 4.5923684295139671], [3.2900544709253183, 2.5917213659333007, 3.1273089111036829, 4.1437277937910615, 0.25009953701954002], [0.80268128455902654, 2.0000897316829316, 0.36323108963530931, 0.88158427318065691, 4.7376147663586616], [1.9066685413763784, 1.441713580861548, 3.0748364447525001, 3.1097339051946653, 1.2014250894268261]]], [[[4.1936678058722858, 4.4832275666591999, 3.2743656711356479, 3.380918836338052, 4.7066161765456682], [0.28487098621385004, 3.7508588221170265, 2.7053575641058969, 3.2580109815905471, 4.2279094300973243], [1.0244049464570546, 1.3397676958246092, 2.8204504390484519, 0.69596504752501154, 2.2929978217334894], [0.9138524427570065, 3.9519625903971649, 2.8666285555858853, 4.8437581672294421, 3.6431253571370035]], [[3.2176122146522177, 2.6374309430160441, 4.4721352893748545, 3.0086996834788207, 0.47183448827975188], [1.0754085032445935, 3.3329686833116687, 1.2622948455264709, 0.10512067478622578, 3.0617684088337422], [2.7090652914422861, 2.2777584485726869, 4.4776700913201415, 3.3735989936808677, 1.4282355919923071], [3.0983293072589513, 3.1851415178110698, 2.6304274314680551, 2.8655036811066612, 2.640065142780887]], [[2.8884540586606469, 4.4114797715825649, 0.54922790475115013, 2.5795073881923516, 1.5172487022771355], [2.9508125940473744, 1.3850587237183416, 3.6456815050229556, 4.8114960291976256, 4.1883268556887128], [0.94771832034951442, 1.8071772490013656, 2.9066564364215526, 0.1667590415115974, 2.7028786827452325], [2.0593251702148043, 2.4584602221866714, 0.18871104825405846, 4.5108133924364449, 3.151933598934193]], [[4.8748115418703764, 0.86197637315318554, 1.9568020246635578, 0.68459965123517719, 0.38377228156762128], [0.49067473646871623, 1.7406766022615359, 4.3604300784222447, 4.4255271323011875, 3.3610794774687065], [4.4288379285434738, 4.1240121041731461, 1.9975550224193779, 2.9163046928601286, 2.1312972508778936], [3.0270670335799656, 0.70996380507978685, 0.12970975772521859, 3.1014125447530954, 3.2548791094263252]], [[1.9118353652202431, 0.75904328690323308, 4.094464367474357, 2.4971423604686982, 2.3599309213033974], [3.8890220947501941, 4.8105650335963395, 4.2227688052045957, 1.7598716164682353, 0.60316572971777405], [0.77181246902848366, 2.2343221934780959, 0.016238857140827935, 1.6962861111491461, 0.048545021315639307], [2.742685844746882, 3.3819491830523272, 0.41420176649130275, 2.0022522572811678, 3.4999257253426865]]], [[[0.17081291055378409, 2.366108576151869, 1.6221209771407692, 1.2156488923090114, 1.8059195997586888], [3.6445988305600818, 4.4790919709274677, 3.7981640612766472, 0.16569070590019727, 2.4502212081697681], [2.1204234847091432, 3.7870340573070189, 4.6871240445727542, 2.2366351249563801, 2.0285881325967154], [3.1961274715391585, 3.5895246644700167, 0.99967788554185477, 3.648838539042889, 1.2616818250737332]], [[1.1817888357664124, 2.5820346903159641, 4.6888376232579612, 0.78263731067608133, 4.0178560734532862], [0.13514173751715086, 2.8899114236011783, 3.5912984591352055, 2.4950992107634287, 2.1633561395035668], [0.11194332837845875, 0.37468933451089664, 3.959305244651651, 4.8282053037950581, 0.54647525858074331], [4.9656654663993454, 0.9072625081310931, 4.6219677095680245, 3.4436762814094504, 4.8517634835241834]], [[4.3825916032450074, 2.927779744224102, 2.0994696388936109, 2.4666699075322982, 2.996916706891827], [1.6282758233202967, 0.32890165397818111, 3.8054861588166453, 2.8783190278971595, 1.7874841546096973], [4.8548613601666064, 1.2665200649028487, 1.183656031145194, 4.216775985694456, 2.6724817963581553], [3.9174996190251354, 3.1640801713551339, 2.6802063247134371, 2.0968165369674412, 1.2343793232103606]], [[2.2475457254727491, 3.4506356460028438, 2.8177142207432113, 4.3887501859792222, 4.5189445075733907], [3.727108626703969, 1.7020700337050971, 4.9959550788908995, 3.9823973218468023, 3.2030910126634677], [4.2340408138420971, 4.1460629108747034, 0.048806946130594921, 0.91500111111296079, 1.8807983002290984], [0.68133606987303297, 2.2699832009785403, 3.5453200616892691, 3.9714507993677763, 3.2849034213031296]], [[2.7710078428497198, 2.4623121381598745, 1.8864849372730854, 0.9901386368946461, 3.314948477074517], [4.8446972192216968, 1.1014700855757684, 2.1074410241125445, 2.5099079748743787, 3.8338526649530098], [4.0338328704431934, 3.5948298246116401, 3.4884953953775342, 0.65015316975285142, 2.035670036362534], [1.8697527017688222, 4.7279172627227641, 1.5531592308136957, 2.6264963097755838, 4.4093053501842707]]], [[[2.4461492402813763, 0.92400785712860989, 0.48086338280876167, 0.99596543416688765, 3.1199971769368706], [4.4168350126095, 1.2069126692019843, 3.156837715723277, 3.4979906727437973, 4.689910919948705], [0.83668529420460624, 3.8272706055524095, 3.5042341922073987, 3.0340226738355325, 0.028362692015294969], [4.6742187274482214, 4.5737834775822916, 3.3007403415133383, 0.12816774467175851, 2.2111171625255874]], [[1.2493593750227676, 0.51149993283969963, 3.0221070468803615, 2.0603044605596419, 4.9897807711915174], [4.4652850944618496, 3.4063383047323414, 2.6586926407819282, 1.4367684278117856, 3.3032800199211207], [3.0451238404590497, 2.0613096541449032, 4.9824723538772044, 3.2995138850680119, 1.6916677891038558], [0.21073201203938222, 0.29349632264013858, 2.955031951257634, 2.8763498402799206, 4.8931576081842953]], [[0.53581663111433031, 0.5882482270761632, 4.2948692521613365, 0.876943291348386, 0.78622670040076803], [1.2579798181625326, 2.0513825041664906, 3.0121773940159029, 3.3926945898764655, 1.6302708862664659], [2.0097233986964138, 0.2763469642098127, 2.4572557880356998, 2.3425698426504531, 2.9778397920833615], [2.968222076815171, 3.5140311454483699, 3.060882217788043, 1.5372258171015287, 3.7992733248157426]], [[3.0554744038676951, 0.072883957715162415, 3.0181891765197122, 3.4077681507546358, 2.7690058077552888], [3.2948111319354698, 0.37631262649088221, 4.3580167505878178, 1.1544130419882472, 4.5108068782179913], [2.8795884388670179, 3.2538557604306733, 3.9222829166011968, 4.5963291297046336, 2.1437020632434565], [2.4041959079281616, 1.6475319794511889, 2.0778758462274132, 1.970516103466263, 0.52676645151789692]], [[2.693082081825084, 1.4667767123152085, 4.2369341835969827, 3.4355549130378891, 1.5612299270615628], [1.6822302921112895, 4.553457732682717, 0.92890904986681444, 2.8813979047862306, 3.7592253829085718], [4.8044439280389861, 1.2084256699757334, 4.1102551412864043, 0.85923342880549436, 2.6119497146922437], [0.336318488475568, 0.37087454562708938, 2.8920643007580873, 1.6794430652358725, 2.3141360940629707]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[231.08417309132133, 17.881786459136052, 43.048321697183695, 1.2379060945626323, 264.73404072740266], [8.5714687963606142, 26.646571054114428, 144.08690262254552, 11.259371249976176, 17.197832101848512], [30.448889712745274, 19.446241302293515, 5.256418755228478, 27.525171609461669, 1.9835475643046903], [7.9933232085262489, 202.51106683053163, 2.3629068815528331, 20.769741433651603, 4.5273117310761295]], [[9.364150916690587, 37.228591985278747, 499.22842128761351, 1.0640271234445047, 12.913940735209598], [522.25167585178372, 2.223423507199398, 30.921541268093957, 67.557908839347277, 582.69953066467303], [132.73303649610514, 2.4607260271829454, 1.3318340072179069, 26.402592537783278, 203.68391393264451], [1.0484093746739764, 596.56492515116781, 497.13271739260483, 7.4269730007355994, 807.39055006042008]], [[36.021916001315489, 2.1734481324057282, 424.05623306288578, 72.6780019865448, 382.78832388242347], [1.2574439959465724, 15.286200525847798, 54.587846485042995, 26.781622854471532, 145.56335002292298], [269.56955769920705, 6.3853789707458226, 44.597194118314704, 3.0100553263018259, 15.902797788914398], [19.879319274612261, 15.548485213609231, 308.90449013735082, 20.308896761157332, 157.92505324616863]], [[1.4196849289072129, 1.3049456948587947, 1.4809178184541778, 1.8162720564290764, 1.3608352097650429], [1.485350113385963, 1.6698881658861846, 1.2462577775882075, 1.6476528699865924, 1.0728796667272749], [1.4155386863887944, 1.2024211809657663, 1.2991525958587919, 1.0121578661745649, 1.4818798087929101], [1.1164593158849576, 1.085852693427577, 1.7575312232907392, 1.5228959917305358, 1.3869684230898169]], [[85.333035784760725, 125.42167200408393, 11.598794148428574, 4.5649570622946944, 300.10614514966039], [59.533829155037388, 25.006903833877658, 48.637752453051952, 171.89503904231634, 1.3643064526055868], [2.7101463269375645, 11.992626859869697, 1.5701416042049472, 2.9892043985216965, 359.43929153403667], [10.678736052137012, 5.9938769398596428, 45.56885541607069, 47.587507753159286, 4.4472166847371799]]], [[[152.67207246060352, 216.04361288904772, 50.705302860481012, 57.61539998165005, 282.40091587859172], [1.4071495459298566, 89.779386415975281, 25.630378248738804, 49.720677622762686, 159.07069120668129], [3.4153575846859576, 4.9848758395630117, 29.423008705575395, 2.3036069476809011, 15.632477608677798], [2.9913581507332654, 114.26072224458849, 31.098061020783934, 332.87390759337279, 78.899966325784703]], [[150.87982926906815, 61.065086656096248, 1066.7515466363297, 108.9375041793743, 2.0867819531197735], [5.3475444451554308, 180.60864953882682, 7.156395423670137, 1.178085335368197, 118.33413033054775], [68.280435484581261, 34.854615960938979, 1075.9965316411185, 192.41953069125682, 9.2694098345635982], [125.27524134645276, 143.43178951208054, 60.401945987036356, 87.140497092215483, 61.316390710078622]], [[15.167181082431846, 63.616983150794759, 1.6770434114913688, 11.339561429200005, 4.1716354230635444], [16.084194077114617, 3.683513660178718, 30.937622776511493, 92.707364929532517, 51.563223244208459], [2.4404035727645557, 5.4807458169175964, 15.429315679194286, 1.169976529575401, 12.736054877637217], [6.9490731666866052, 10.118305375243084, 1.1944058638396575, 69.852870932557622, 19.436804767091054]], [[1955.6851564312533, 3.8192291201359683, 20.948983582347136, 2.8987934732020699, 1.8159728905608037], [2.1443026723899492, 14.970723085054939, 879.03651484078762, 972.65343660205815, 185.90069884953022], [977.67262979307998, 608.67565147312007, 22.319164208063391, 93.107560194988807, 27.477417427477498], [110.60320952005142, 3.0153813687607269, 1.2234195145072082, 124.15470089189996, 157.6080226895703]], [[1.9706024457989169, 1.3090694557696025, 4.2748928772116104, 2.4254386320256547, 2.3101866559493778], [3.9743688827149697, 5.5115125532458888, 4.4739997571248491, 1.867164497975327, 1.2386344813282157], [1.315013834208685, 2.2094884679734865, 1.0057783480156273, 1.8255113917789147, 1.0173734777463883], [2.6462242146864097, 3.3199547681955406, 1.158311273197, 2.0348460619921425, 3.4618754420073596]]], [[[1.1462599533147748, 6.6251024768679248, 3.6557864486645832, 2.6418535222181325, 4.2341946573453502], [18.403962892137933, 35.85373489132482, 20.806949307381966, 1.1415774561003624, 7.0857384888682136], [5.4440736845189939, 20.622702576484436, 42.338452267673574, 5.9738861932014853, 5.0588428912578891], [12.860646534103001, 17.611528657201397, 2.2230684575542008, 18.466423948586851, 2.740849173050536]], [[0.99805850217094871, 0.99576299137224855, 0.99231912128099997, 0.99871382498822125, 0.99341464180298555], [0.99977779173897618, 0.99525897821274334, 0.99411171685687849, 0.99590535630845423, 0.99644880350680409], [0.99981593246287792, 0.99938403434108281, 0.99351029587665041, 0.99209172538332602, 0.99910175597158446], [0.99186749288529885, 0.99850917045043686, 0.99242824645767957, 0.99435307306486753, 0.99205329239937634]], [[0.44961931314025694, 0.58625142566369381, 0.68186160746381375, 0.63768979953736382, 0.57890515371439144], [0.74305433309449698, 0.9417745549719313, 0.49952679813306333, 0.59156409637113516, 0.72178743465276385], [0.41251086990812591, 0.79373597939621998, 0.80582347583501468, 0.4634251399767344, 0.61419551751504997], [0.48942477632744963, 0.56152101952348699, 0.61333078681985775, 0.68219164506363261, 0.79840272060209605]], [[15.052945636798356, 64.267456885071056, 29.947896755631987, 199.30791162039139, 233.20669099083645], [89.712215564584326, 7.7949740240823102, 414.64355683712711, 122.07095877594175, 47.674693681522989], [165.37246217352092, 148.71897751598382, 1.060651796246094, 3.0159416707447599, 9.6707672443496531], [2.2750558867143207, 15.465992927876323, 72.04466947436039, 120.46942243637905, 52.620412414807376]], [[19.199509725338412, 13.814306965485255, 7.4757704918191861, 2.8743934307227748, 34.292266727792374], [175.24390983100184, 3.23672102011118, 9.4620148822744188, 14.533538301031989, 59.63586734884484], [73.810984942629972, 46.218188796477655, 41.263688949849467, 2.0002969600019651, 8.7648712180628472], [7.3435666666729276, 154.72503938235744, 5.2394831245071503, 16.457541131166753, 110.15576709641807]]], [[[33.65759600094102, 3.7743313116133335, 1.9961519536734358, 4.1856385953580739, 88.665194499476996], [571.93101638866153, 5.6682886363710931, 93.48716680775685, 152.66065176230896, 846.87554794733887], [3.3290954950206602, 245.06998793172809, 154.03691750860736, 78.357572204889763, 1.041612716895165], [827.98651729767062, 716.67706127013878, 114.97081000788079, 1.2022993558329391, 24.008065497840512]], [[0.11598197506439578, 0.41395423589882818, 0.0054554486897124372, 0.028648172392911161, 0.0001833563291549344], [0.00045297654564447097, 0.0028124879914337101, 0.010208951233078863, 0.083954740005917752, 0.0033594492014688672], [0.0052431690096320016, 0.028598559650333172, 0.00018568164302022573, 0.0033813367849287092, 0.05409515635081464], [0.69532656804903659, 0.60284963041603523, 0.0061243654243180231, 0.007014292597753183, 0.00021659789353365593]], [[1.9195981707281271, 2.0460841098805571, 186.24678669458166, 2.9074727621966177, 2.6035543480300891], [4.6229125011447003, 12.141581184009922, 39.094112759074008, 62.120777579432719, 7.272658707671523], [11.541333274072191, 1.3997983636215312, 19.897702848183155, 17.305523146404454, 37.494013962278515], [37.057695936582761, 72.006272012526637, 41.481523659313822, 6.4940176384330686, 101.89138087312365]], [[7.1893595735889795, 1.0481781821513669, 7.0183693293378386, 9.0253883582770005, 5.975448227524744], [8.3906411304190645, 1.2749996851527103, 16.668458867345638, 2.1070407151497492, 18.396472612461196], [6.4176423690704167, 8.1716946346638579, 12.581282789767817, 19.440757514308896, 3.9906954229536091], [4.7215601959891318, 2.8968967528881175, 3.8246553699551598, 3.5685428309788816, 1.4050576323296939]], [[14.813518775173025, 4.3410467857650783, 69.463500960440285, 31.146152378997403, 4.7714724209196753], [5.3857994007456815, 95.35568043016255, 2.5338931445612665, 17.886172047206014, 43.062675266553001], [122.5880059530587, 3.3519043984049843, 61.191407533410576, 2.3632025097067251, 13.658113953235681], [1.4002145339805114, 1.4494918716312382, 18.07815014444612, 5.3707951930846596, 10.137569991069599]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[2.9634784704518098, 4.5764324864280521], [0.97578127921744007, 1.3168779302307507]], [[1.6424353684407733, 3.0557158145661627], [1.2489838252167824, 1.4257986508504101]], [[3.9566360809440493, 1.4737364382238602], [1.4445356169388326, 2.739739309179138]], [[3.6319647208651267, 1.3338462329717422], [1.6337670849316062, 2.2793522100540708]], [[3.7994100768184724, 4.424616121186447], [4.0461725862082929, 3.0861509731073933]], [[4.5618156889869308, 3.8457418281755356], [0.87813008563655204, 1.1660218998239817]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[4.3920320009649521, 0.93514438992910942], [0.74956731367546647, 0.23265536520688618]], [[0.80899196868996348, 0.84308896354283891], [3.1764150848488932, 2.1324076097360889]], [[0.018481752259819389, 4.0972248312195934], [1.1764244090945102, 3.4422860409727112]], [[4.8418361656192603, 3.2803791946763843], [1.3718608062037392, 4.0043822740233539]], [[4.3910504731760058, 1.8852667092452031], [0.26288763981263968, 2.186804364829952]], [[1.9776591525489831, 0.48635715217586306], [0.59832291844059804, 4.7730333118722941]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[118.07786727213049, 4.146562105064084], [0.98179082297709896, 1.0661367318535373]], [[1.4939232795154831, 2.5644449709177124], [2.0263036232183267, 2.1306635524122322]], [[1.0257455301342342, 4.8983958125976939], [1.5413746683606084, 32.115923153304152]], [[515.36385976868974, 2.5727308478796331], [1.9609530708885861, 27.090297939483268]], [[351.20683845384735, 16.506225047483625], [1.4440574467182918, 11.756000574609638]], [[20.116377338302968, 1.9253481327178352], [0.92518795586071056, 2.0815810996396622]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[2.3775630299106059, 2.7022741245721802], [2.1846398899179849, 3.1861272407310226]], [[2.2856469505710164, 1.6853261399351782], [4.3446612979087877, 3.3410712084456407]]], [[[3.4475395924317325, 4.3940307128975462], [3.207296733049493, 0.58835828452484396]], [[4.902020085380637, 4.4984142131241693], [1.7043330075063101, 2.1138193963997365]]], [[[3.6639739559058708, 0.057880473832362077], [3.1411945603003342, 4.3341513095598936]], [[1.5222609952080928, 3.5605567715169633], [1.7762323324966611, 0.46234997807258277]]], [[[0.80825267235443465, 1.7214361275089327], [0.84154191297291303, 1.6127093333049445]], [[0.91553418561648015, 2.3082984680251886], [3.011116199842184, 3.4944378500499984]]], [[[4.8170764413418414, 0.25444108532388421], [1.6735895023745933, 2.8326678402087269]], [[4.754613239440074, 3.6657710986739751], [2.9210988382300322, 2.4470825209682463]]], [[[0.413841780898909, 4.6219414814571218], [0.48243498080326164, 1.6610414134763767]], [[0.26042184049873079, 3.6493345201175402], [0.58378921656118421, 4.5719309460054216]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[4.9964015875797685, 0.46851990360749673], [2.3080648696885424, 3.7758156150589164]], [[4.2278886763569909, 3.2800967366630167], [3.3899745396463006, 0.65890243029307971]], [[0.59763632813877143, 1.4250953484736681], [4.511445840575651, 1.6689562879820801]], [[1.0411533582352044, 3.3879024487698493], [0.40729559862097847, 3.5825772138263834]], [[4.009466255132951, 1.3828919987729928], [1.7695772454922001, 4.3712865954758771]], [[0.31877532376766499, 3.6952812977057548], [3.7047007573212167, 3.8957941578992767]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[75.736759097499771, 143.57984264918451], [1.4421360023093139, 1.7210313699843893]], [[6.7393250483666733, 3.3358097782712832], [256.33266005778898, 95.081478638010609]]], [[[187.29635433377516, 522.34034426313087], [45.728330674109664, 0.17555089964825452]], [[218.9545455331079, 163.62696822730706], [1.4209270866114545, 1.637525287319654]]], [[[2.1728934381855685, 0.18215630778398753], [5.1098427987619939, 8.0844229452396892]], [[6.657172027586201, 307.71112831535868], [2.6085803109577874, 0.27596307721621233]]], [[[0.80120269954452317, 1.7603483455936433], [0.55739613492542139, 5.0487029313859573]], [[0.96469542546685894, 1.4059413939361287], [51.889356903499824, 88.448649861371223]]], [[[546.50929854326876, 0.0041373484097621946], [2.0383667849415859, 4.2202458256209843]], [[15.783657280077302, 9.9617159365324621], [108.40169947595631, 49.991305800628602]]], [[[0.75484232018288777, 1.6290320429155678], [0.067642131258107294, 6.5218047709140894]], [[0.0068432063468134904, 121.0133903623813], [0.12285186614621731, 372.91719402005714]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank3_Symbol_rank4(self):
      arg0=numpy.array([[[3.5433781635646482, 4.3647597345070013], [0.028153687848100392, 4.3746582478118672]], [[2.0492594107400355, 3.6975385935145244], [2.5594409585523228, 1.8390249161883732]], [[4.428581946945334, 0.88244655590292909], [3.5573595660085378, 4.7850953574258188]], [[2.0154498916188097, 3.2754656887762628], [2.3614994436976611, 3.9212547946500682]], [[1.3677055820423809, 4.9805525804859716], [3.983096927670946, 2.8367354426015727]], [[4.5015877138447378, 3.6316831939643071], [4.9260192954829813, 2.6038662473603345]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[3.9815781142580819, 4.4816363883430688], [4.5194046318042815, 1.473740319737268]], [[3.7377537322468242, 1.291746779234006], [0.36588252843252073, 2.9697371574231286]]], [[[0.56743814952945337, 1.1313410192150053], [2.7081101131628578, 0.33850790702062777]], [[0.46710585910083019, 1.3281222256915646], [3.7238782589277353, 2.0121403762427517]]], [[[0.86350883084854435, 0.80237903022574564], [3.1408401819334006, 1.1059300826088736]], [[2.3214803268129049, 3.0204158037089925], [3.5292400122316412, 4.6682643972653954]]], [[[2.5888174910132835, 4.9254647379753251], [0.19852176252081985, 1.4740939659842389]], [[0.26533035648045322, 2.8711217092813945], [1.9796669941426788, 3.3925869179828405]]], [[[3.9329721922432532, 4.3186737832137805], [4.7425749387706402, 2.2456636892191009]], [[4.762280137814809, 0.70896302669985878], [0.67081134445729862, 2.5411727822479397]]], [[[0.31367224273388311, 3.0956729851911016], [2.3098877186988322, 3.9024878882178502]], [[0.45198159323568343, 4.7069502894241246], [0.031963109215458883, 4.7229826973353415]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[154.00992570221152, 289.92754651442425], [780.26070331790891, 8.7727420428971055]], [[1.6023033753972552e-06, 0.009935556696278108], [1.7159722459501676, 80.063690030740901]]], [[[1.5024913831580082, 2.2517615424522019], [34.512003258793762, 1.5568362838085488]], [[1.5511256332944239, 3.4839073693686018], [9.6670076206013729, 3.4071199218450134]]], [[[3.6145585402666853, 3.3002681751861953], [0.67517442531565108, 0.87083360820980771]], [[19.029714062877677, 46.19925896853691], [250.89794155982375, 1492.4818802013406]]], [[[6.1371052396850381, 31.56265865534138], [1.2655896512997911, 5.7485842466519497]], [[1.256082209811417, 11.788749268122471], [14.95491710865001, 103.09701818758376]]], [[[3.4265351443314942, 3.8664087074567908], [2027.1735740778747, 36.80025508342711]], [[721.8051082325577, 2.6640011514920459], [2.0125932482355253, 14.147864717404913]]], [[[1.6030338375202822, 105.34322238824826], [19.669190626080894, 153.39616972284358]], [[2.0558692676854822, 1817.7943912578658], [1.0310612494682172, 91.824758955436579]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[4.562075634821225, 2.6341139490649548, 4.4805057779104489, 0.2417104595609976], [1.0064708753468325, 3.7735868613327326, 0.39106008880809628, 1.5964019563982403], [0.20867913330034496, 4.2602134516721311, 3.5970912299346942, 3.3082170675908578]], [[2.8784149622680322, 0.88359415187288914, 3.1057003796615521, 4.9282778641458131], [0.8654591632592239, 1.4511011006369707, 2.9078650331733553, 3.7817305476673009], [3.6790360998617282, 4.9573598324432693, 4.0298619787681815, 0.89556843754350512]]], [[[1.265920108729067, 1.8064097981130482, 2.9153842150042335, 1.5578941973089053], [0.75740206195743309, 2.9266243679530453, 0.019482786862339514, 2.8891168568328105], [2.8335590998467328, 2.2243072882778145, 3.9699965375045712, 1.2287579923753418]], [[1.1301691835919523, 1.051299848089333, 0.43579977133106407, 2.868855050452888], [0.63144529317555997, 1.0415963137161719, 0.74728454354930696, 4.9601896354296784], [0.11780075352039057, 2.9195498261996815, 0.83392293613406288, 2.5455753315738159]]], [[[1.7869323752184745, 2.4209982060659749, 1.4809284887412919, 4.2512025535986515], [2.9860641202435994, 1.1673688585628921, 1.3161618744039452, 3.5765468182381448], [1.6857345051800658, 1.3936612018126546, 0.25495598427343985, 4.4720174467047542]], [[1.7052458132490449, 2.8781764346952934, 1.2655661844601789, 0.50148796404721452], [1.6583901430689896, 2.115904526336077, 4.6896778497395193, 2.6937787462622964], [1.2457656553497902, 3.6814528566994453, 2.6125893570082357, 4.6436535358740922]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[2.7110769447776706, 3.8988585456579754, 3.7947949396773528, 2.8700139366998054], [0.56087776262844413, 4.1779931721398063, 2.0767767214052464, 2.9362466890567926], [4.4155043765697393, 3.5738369517098896, 3.5006538700152801, 4.5015731186510344]], [[1.6032389089870944, 3.4664344639556486, 1.2752978043859355, 0.027716688227616429], [4.9291227652387972, 2.1927752516869612, 2.334271040100004, 1.2143541304490371], [2.3522361027583019, 4.1383754440306655, 4.9317705374005705, 3.8980578561046215]]], [[[0.014888984995799624, 0.22418337003330405, 2.3359741506595619, 1.4423257868042305], [2.1351654099203419, 4.3532268859486365, 3.7585687354724313, 2.4372473488906907], [0.83547034793063224, 0.77850306933802838, 3.3526299550863703, 1.7374340339904284]], [[4.2885192130272749, 0.69332362827563054, 3.9114666078603073, 2.1017062317765633], [0.18256685828559857, 1.1700558518304869, 1.9078545881216433, 1.7590166898438877], [3.8852705345113727, 3.0615650904329539, 0.73625760506765425, 1.7883774482027384]]], [[[2.7954183079049875, 4.7907222510621876, 1.3561772061726916, 0.59726052437298005], [4.1688556161621397, 1.2589878091046758, 1.340554305768435, 4.316807291242192], [1.8071697749215714, 3.4576002297096906, 2.8516772370767987, 4.4154020223389114]], [[3.4915353408462875, 0.21508900634751202, 1.0483567138085086, 3.526907251489686], [0.55076343929867877, 0.77553701114624363, 1.8607688392247257, 1.2707536611509276], [4.5186791747406776, 0.72531814514962789, 1.4722009190752119, 4.7734422818787428]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[61.240708380099782, 43.651044656923169, 296.24604561836003, 0.016984365598040712], [1.00362422959304, 256.84773546349635, 0.14229209757871039, 3.9488966074797247], [0.00098890973802535963, 177.61721270913173, 88.347365623255513, 218.26815308053494]], [[5.4466892942443517, 0.65116187837130102, 4.2427749894892006, 1.0451995558591047], [0.49054707589683849, 2.2623857275930046, 12.081140507910963, 5.0294551994873098], [21.416083688433893, 753.71535495487592, 966.3895759140338, 0.65054638904077344]]], [[[1.0035169811114306, 1.1417576737549084, 12.176356981706245, 1.895406930228239], [0.55251246557004752, 107.20160682314656, 3.7285160808311809e-07, 13.273899906403653], [2.3873176194653234, 1.863340423463907, 101.7470250540819, 1.4303485869574886]], [[1.6900780226428473, 1.0352936882116224, 0.038822467742965661, 9.1615515476966412], [0.91949192917261002, 1.0488402046617646, 0.57362713077676186, 16.726038157390256], [0.00024612683069046004, 26.58244288459348, 0.87483944796843904, 5.3173891908269937]]], [[[5.0669619944504785, 69.121131743771429, 1.7032344893817175, 2.373478412374189], [95.63561892935995, 1.2151060497783019, 1.4452421006907106, 245.0174758469912], [2.5694877110954013, 3.1509245354497102, 0.020296929826057436, 745.13407086431937]], [[6.4460322475473504, 1.2553103459021513, 1.2800620489844283, 0.087669117325362436], [1.3212816597761106, 1.7882747621487409, 17.735441359599125, 3.5227675570433248], [2.6992695994960552, 2.5736176585970783, 4.1116158847137534, 1524.8137885901854]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_float_rank0(self):
      arg0=Data(0.641571292305,self.functionspace)
      arg1=2.53337057042
      res=power(arg0,arg1)
      ref=Data(0.324847822623,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([4.8048221088445215, 2.7661241890065624]),self.functionspace)
      arg1=4.95332496784
      res=power(arg0,arg1)
      ref=Data(numpy.array([2379.9577950413568, 154.43082719990113]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[4.6485225748370711, 1.7719750989765677, 3.2106882314676803, 1.6265318607321895, 4.7521772053332478], [1.6466829081196548, 0.8161628960528271, 4.2136635319122417, 2.3843522711311125, 4.746137269334854], [3.6323819919231326, 3.3389171192295444, 1.8434923978387685, 3.9437469125530611, 1.5215459007147896], [2.6585328979621892, 3.9033311096581231, 1.0811112196997539, 4.6207214488430344, 0.98528410355034768]]),self.functionspace)
      arg1=1.49412501594
      res=power(arg0,arg1)
      ref=Data(numpy.array([[9.9323485575638699, 2.3508598422552205, 5.7137462371072125, 2.0684865333958968, 10.26507689110452], [2.1068925449601319, 0.73821621528988979, 8.576693961165498, 3.6630140495085803, 10.245589557074659], [6.8706235661926662, 6.0580420495325749, 2.4940283612048502, 7.7689548730030902, 1.8722190748146197], [4.3099109110012321, 7.6502991180336499, 1.1235866585789018, 9.8437261596566383, 0.97809275102755888]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[0.10651701995803627, 0.83788314499831862], [4.5733353890015893, 0.9995981169987509]], [[3.9343559601050053, 1.8128928633468124], [2.1503309825296362, 3.6281798727310699]], [[4.7498365769304733, 3.0237188774486907], [1.9058290523236803, 0.4707623417632088]], [[0.64159183267467823, 3.6204752227405139], [0.99484430019982306, 3.0774106249691529]], [[0.34219212057078618, 1.2911335443774723], [3.6727414676779775, 3.1485115889316511]], [[4.7136563548166368, 1.2197577866941449], [1.3798978618276272, 4.4768561840304413]]]),self.functionspace)
      arg1=0.945120844873
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[0.12044629094821103, 0.84605594465052825], [4.2072664755508304, 0.99962016780917418]], [[3.6494505786153399, 1.7546596718534033], [2.06185301322706, 3.3804425590389657]], [[4.3605685077021761, 2.8455728244127738], [1.8395566762038456, 0.4906344963035888]], [[0.65740999037031844, 3.3736575515512408], [0.99512655004555672, 2.8933051755512715]], [[0.3629350183059083, 1.273154650470018], [3.4196697364226178, 2.9564442805363913]], [[4.3291696373259168, 1.2065323670559502], [1.3557270448227543, 4.1233318348767281]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[2.4272714916519509, 0.75680130810662549, 1.6931814570943409, 0.15219561470113735], [2.9322989938230144, 1.5093660253733585, 0.99413141149261741, 0.32956552344733003], [3.0317146871922307, 2.217358946619302, 0.7149820960005584, 1.2874666287321892]], [[3.6308298639297294, 2.1579105126885825, 0.85469286916577414, 1.0681065862768901], [0.1538787005323026, 4.9090837142763473, 3.1973022746683828, 4.9610037676052841], [3.8099304978461004, 2.918552319917926, 0.32832066658265935, 3.0192836527254485]]], [[[0.26949969792247341, 3.218774532431496, 3.5701684687464543, 0.46123058839650227], [1.6333129863160198, 2.5504449672029121, 3.2867627071685988, 3.4904167518783695], [4.7572862330341472, 1.6618665893664222, 4.4739580671713233, 1.5224373819846062]], [[3.1526000903367675, 3.5927985066328878, 0.51877776403478237, 2.969901989727501], [3.4210053387866819, 1.3049944832213207, 4.6775643277792627, 3.53619555568747], [3.153428443204469, 3.2008384812556394, 2.8849158593676036, 0.28447662918039684]]], [[[2.7185539641724179, 2.4998346197126731, 4.6520574343353749, 0.64957835876186953], [0.35560037315841919, 3.0721859389182335, 2.5190756343641909, 3.3656079938261101], [4.1218653904299565, 3.8955951125834107, 0.84669738135586525, 4.6563569555690352]], [[2.8215263955393497, 1.9871360608498041, 1.5192387586783014, 0.4219292924191389], [1.0416055457863116, 3.6251265456723476, 3.6654155148938461, 0.71163453359781548], [2.7649113442477686, 1.6811896078730393, 1.4052160819161357, 0.34609133725925018]]]]),self.functionspace)
      arg1=4.90724251117
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[77.601329903517453, 0.25476203409199571, 13.252675237856899, 9.7240911176447523e-05], [196.20234449013572, 7.5402946215950255, 0.97152971626833851, 0.0043094651725249233], [231.07951045092364, 49.785300562476415, 0.19274851815425811, 3.4554218129696972]], [[559.8668501808454, 43.569547715917388, 0.46278100899213925, 1.3817158755504195], [0.00010263319078057317, 2459.8392408857353, 299.98256561136759, 2590.1713713296776], [709.08918056752964, 191.72981279353905, 0.0042301723923906746, 226.46698380298042]]], [[[0.0016055040201878954, 309.99927711874113, 515.43930570972861, 0.022426661133366863], [11.106655251951917, 98.937995976328821, 343.4848755458446, 461.3499607811093], [2108.4643704813743, 12.092576350600872, 1559.922416147997, 7.866205637137301]], [[279.95559080053187, 531.67195228926005, 0.039934143931949939, 208.86237636345578], [418.04400428231548, 3.6924940841256579, 1940.6601713642301, 491.81358713160091], [280.31674784737385, 301.61421123237693, 181.12773662717086, 0.0020935061133064187]]], [[[135.3323718033644, 89.669993890876782, 1889.2797059210827, 0.12037535061885529], [0.0062583905022960172, 246.61703936302592, 93.108195406069342, 385.8589365966057], [1043.3126350032192, 790.84052853229764, 0.44192119587397061, 1897.8637701000098]], [[162.41824441644948, 29.072018039344233, 7.7854367926396248, 0.014486391331848147], [1.2214460953609376, 555.56444583216489, 586.52890042659374, 0.18836028690280079], [147.04044622624218, 12.798404899713319, 5.3089822952524823, 0.0054789477088401433]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_array_rank0(self):
      arg0=Data(2.94255090833,self.functionspace)
      arg1=numpy.array(3.73216930338)
      res=power(arg0,arg1)
      ref=Data(56.1509708122,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([1.6468328380281947, 1.6709692616984935]),self.functionspace)
      arg1=numpy.array(1.38244610437)
      res=power(arg0,arg1)
      ref=Data(numpy.array([1.9929953369583022, 2.0334892468159063]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_array_rank1(self):
      arg0=Data(2.5587739468,self.functionspace)
      arg1=numpy.array([3.9400997403663456, 1.5766012911524778])
      res=power(arg0,arg1)
      ref=Data(numpy.array([40.521588555449497, 4.398490437123888]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[1.4152627658425718, 1.4154873681658067, 2.3527278277810608, 4.7676660942703926, 2.9063119344311956], [4.3643835902270904, 0.092835841550109988, 0.17280289596952159, 4.2549252374783446, 0.31124594260178601], [2.5546829157174753, 2.6421129871508007, 0.098316806743365584, 0.60302179644890985, 4.7100364019344036], [2.1780285763902527, 1.158879827658063, 4.0240180246678641, 3.0892183641957489, 3.0675326807953636]]),self.functionspace)
      arg1=numpy.array(2.21215917735)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[2.1561334287340244, 2.1568904549873387, 6.6370596756649967, 31.660696951892007, 10.592214230752859], [26.03822331775503, 0.0052050243410297244, 0.020575077692074468, 24.615522709393648, 0.075624879398151865], [7.9633222574030507, 8.578742466762316, 0.0059092486455797188, 0.32663419788746845, 30.820295713842711], [5.5956486413786095, 1.3856806151892216, 21.757249168546071, 12.12336256350226, 11.935900573449432]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_array_rank2(self):
      arg0=Data(2.67791164073,self.functionspace)
      arg1=numpy.array([[2.5672839351073846, 1.7422705075617984, 4.6592797811033844, 4.0680051253765495, 0.74091191241379573], [3.0883911314629686, 2.2134845940719425, 3.4793864671407517, 4.6316954775774271, 4.5380188476854553], [1.3725799997330843, 2.0908096614202099, 1.9730320952100548, 3.9603710487256212, 0.68842767188704201], [2.7563979853025771, 4.5240967505866925, 0.76772203172618547, 0.4240321760619139, 3.598180293530123]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[12.539332612841502, 5.5633550548394997, 98.451511006914728, 54.989188365808765, 2.0747192313726264], [20.950872953717024, 8.849533824096012, 30.794166879811065, 95.812445362756833, 87.366998189925638], [3.8653121088696549, 7.8422470264409494, 6.9832198964241057, 49.457465414961547, 1.9701839268438885], [15.106946897809371, 86.177044060708042, 2.1302403220383539, 1.5184460700024145, 34.616887095861451]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[0.19716498096281096, 3.2373701601002414], [2.3656659064126062, 0.93956724218579413]], [[4.5950019740294614, 1.2535496958973327], [3.9024618606192378, 4.5801492813710283]], [[2.1851446799870216, 3.5949519615884649], [1.6336227134957435, 1.3875817278085616]], [[0.30686933510023662, 2.5780748047946096], [4.1463599105172007, 2.2814396928060803]], [[0.69184548515869482, 1.0632034892076772], [0.20284362883903218, 0.76144316310907789]], [[3.1655916110032698, 3.6802107435176219], [2.3160747948405538, 0.3269547434869981]]]),self.functionspace)
      arg1=numpy.array(3.78687060799)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[0.0021360510346718257, 85.51306051007208], [26.068328231510939, 0.78973490660022283]], [[322.09965534902403, 2.3531487074935371], [173.50940426536789, 318.1747111881275]], [[19.300456482414514, 127.15628610835334], [6.4147265347755802, 3.4571227974554426]], [[0.011406680038252088, 36.101304516901223], [218.28555425538394, 22.724331068288674]], [[0.24781934543138728, 1.2612257735034], [0.0023785361767510562, 0.35626773744153944]], [[78.55211806314955, 138.95899137235216], [24.058626253675111, 0.014501987406934371]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_array_rank3(self):
      arg0=Data(2.03629482103,self.functionspace)
      arg1=numpy.array([[[4.0882513077155034, 2.5201292410972598], [0.085133297158048918, 0.35941575984475954]], [[4.787933074865669, 1.6648758172529259], [0.47309681081498378, 2.0418512705295031]], [[0.58294623590419092, 4.010006806076138], [2.3575390426025815, 0.22302159958214585]], [[3.6203939608814713, 4.785144874257905], [4.6165182253080319, 3.8304475232032265]], [[1.5344876881863649, 3.5530655818717203], [2.1718986749911471, 1.9954564195670603]], [[0.98600188935818023, 4.6196681845786438], [3.4678955638653215, 4.5432372095828164]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[18.307043718322959, 6.0023092844695292], [1.0624111584044622, 1.2912258124665177]], [[30.109836352042844, 3.2672422820532758], [1.3999467921478415, 4.2717584510242119]], [[1.5136919396139354, 17.316221617535803], [5.346922706693519, 1.1718664944987072]], [[13.125771767427752, 30.050194367632713], [26.654376037093527, 15.240454845579835]], [[2.9779142320536049, 12.512125955269285], [4.6856582832246598, 4.1331205366154933]], [[2.016125074761896, 26.714149734848977], [11.776795760124045, 25.300922911207685]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[1.0937417600616277, 3.5975499723848445, 3.7859938718002843, 0.57493966135228824], [2.7665116896864519, 2.0981057660582532, 4.9970190550311937, 0.12896170923700515], [0.87902247433072001, 1.4733099578551694, 2.9109893025427747, 0.92137668708967568]], [[3.067593774396792, 0.42727977576830256, 3.8329131883844356, 0.55170394796294853], [4.6911065573187081, 0.96986566698858945, 2.1507630619804567, 4.1013150283285578], [1.3171157022619611, 1.0083373627812322, 4.8958537495563554, 4.5433820210226363]]], [[[0.70651329682014041, 1.8676265109703185, 0.391277780101489, 2.1296453609390711], [1.6261781415489329, 4.2739173824898957, 4.3388384901938384, 2.3940484415205061], [4.8660479546569153, 2.36562215275924, 2.9523738949738232, 4.5796334356559951]], [[0.22991685042728657, 3.7430234765948094, 4.7746809836684889, 4.8076395789519957], [1.4193359011801934, 0.58522600046644502, 0.67580417556357053, 4.094589779623826], [3.5332561156501727, 3.98280416872245, 4.2559341174273975, 0.9311474503372611]]], [[[0.48410644434915823, 1.6155309191224609, 3.3826014840901371, 2.2040183398639175], [2.4367392390269238, 0.40957399773830017, 3.5512423259897381, 1.6268565208868844], [2.6794574143770435, 2.4148278576975342, 0.48898223798793156, 2.1153409718601766]], [[4.6834403018547359, 1.8911030881528847, 1.9132859272256215, 2.687827498825238], [1.5334015333457669, 1.1358711981020582, 1.9490443355528577, 3.0230315796108846], [4.6172316640138744, 3.260300704250533, 1.1551621822164229, 4.0144433014707541]]]]),self.functionspace)
      arg1=numpy.array(1.88616525744)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.1841309506853008, 11.187183857677969, 12.31807524029243, 0.3520528584050418], [6.8164420326418806, 4.0459416762994467, 20.791433026215785, 0.020998198127813159], [0.78410587158808176, 2.0769714657516287, 7.5033857754137543, 0.85688535192202164]], [[8.2828765341083201, 0.20112332496276042, 12.60759050593942, 0.32569793426235233], [18.455938960516278, 0.94392144853935023, 4.2395967325926236, 14.324329819778155], [1.6812430252732018, 1.0157837184211775, 20.004625683334258, 17.375043956162287]]], [[[0.51929725501269708, 3.2486118899123624, 0.1703568949462172, 4.1614224092572671], [2.5020614394531213, 15.482543759578128, 15.929117720858075, 5.1892831406138127], [19.775534122496509, 5.073676594592035, 7.7058558020745167, 17.637455878435716]], [[0.062490935550946237, 12.055700786808655, 19.081005789553213, 19.330196160196646], [1.9357880207034734, 0.36402723934027076, 0.47754471048276426, 14.280058384801119], [10.813066280928654, 13.553627672258667, 15.359897609988955, 0.87410517749402383]]], [[[0.25453435225678928, 2.4712519948394513, 9.9598898994876315, 4.4397712402603249], [5.3651986207278002, 0.18569268606747868, 10.917123311599482, 2.5040305154061882], [6.4175182747267874, 5.2745644716162934, 0.25939130231708762, 4.1088583605942057]], [[18.399091734966497, 3.3260640155134058, 3.4000352101170552, 6.4553826204124007], [2.2396373193779082, 1.2716271811597131, 3.5208835048857283, 8.0573883955544225], [17.911568354438053, 9.291562707527433, 1.312668245625183, 13.757424024195053]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_array_rank4(self):
      arg0=Data(3.55857618206,self.functionspace)
      arg1=numpy.array([[[[2.6929921778959858, 3.5295479243146666, 4.011942979601967, 2.6815650838046619], [4.1104220793985249, 2.2417214814137769, 0.15062451614445987, 4.7693857807478386], [0.76125292679050771, 2.4091739027024364, 2.2645892349545051, 1.8554750135018399]], [[2.0267387509537373, 4.7592425705215646, 4.2330013149405792, 1.7964530853854155], [4.1541571200529708, 0.66708813227045771, 2.5579513706499748, 4.8675374011396606], [0.21522435626701367, 2.5963678190512138, 1.7269666917355724, 0.99080034630952274]]], [[[0.88760906471427325, 3.6790931857486977, 2.1292038298509199, 4.4601424098356102], [4.027133213881843, 2.5746769867439556, 3.2859202542687265, 0.11077182110109153], [1.2916151952767339, 2.7662415268717537, 4.4045961016362964, 3.2478285943415894]], [[2.8729781819939824, 1.6626802007601182, 2.5875142186774367, 3.9237150685209832], [4.8749720356476862, 0.028539427936146243, 3.3825551592136374, 0.69568079369955726], [1.0569210416368764, 4.0529750706755152, 2.9658940579334336, 4.9946819011070236]]], [[[2.1049808523552067, 2.933990428960588, 2.4293604026208655, 3.5379251421848052], [3.6628133025908078, 3.9226785771652204, 2.8664284269893594, 4.3312063804437928], [4.2309511111657079, 4.2832663211998456, 3.7360461434051229, 3.0360268514742415]], [[1.985074588072314, 0.49586622769499772, 3.95941528932679, 1.2590785519481051], [2.5385384280768122, 0.091591309917943092, 1.2474453121079438, 4.5576070796805785], [2.0091679494360242, 4.9823678721042342, 1.4902399839595231, 4.8031717480277791]]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[30.519869805453748, 88.258391090482732, 162.81295242750465, 30.080371171350837], [184.49225636947631, 17.211069224476191, 1.2106977103318215, 425.84268570099096], [2.6282136843726756, 21.287291006539981, 17.717983838193724, 10.540939972333634]], [[13.10065392260627, 420.39494098150033, 215.55255044334524, 9.7800697453057897], [195.02407753410151, 2.3321144277770913, 25.712119889374296, 482.34507791661485], [1.3141595028232447, 26.997028233721867, 8.9543847717265805, 3.5172619747581306]]], [[[3.0854450218697429, 106.70794585899974, 14.920368190802515, 287.58793398266687], [165.98275449790685, 26.26384578453689, 64.781329024943332, 1.1509749617083014], [5.1527230924859229, 33.493713749153727, 268.00895131054654, 61.723539923330478]], [[38.353472356718598, 8.2527014694422309, 26.695323060516767, 145.56300021021158], [486.91862695428341, 1.0368910105197486, 73.235623492025738, 2.4183118915105233], [3.8252118018894565, 171.51771008343314, 43.154591156739613, 566.82578287434376]]], [[[14.468582592310915, 41.441865128205222, 21.839803950966811, 89.201912868475972], [104.52545014447773, 145.37161163788281, 38.035923452501549, 244.1694291302652], [214.99231550682256, 229.75399456660671, 114.70796461725143, 47.172571471669755]], [[12.425804314182598, 1.8765462958213881, 152.31114493525743, 4.9442466140371746], [25.086264090717659, 1.1232905763990528, 4.8717724568848819, 325.46224834330485], [12.811695504522321, 558.03464767419121, 6.6303115564036492, 444.50289529663843]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([2.4970552655744407, 4.0314135106825564]),self.functionspace)
      arg1=numpy.array([3.9637521035633396, 0.013773822001239708])
      res=power(arg0,arg1)
      ref=Data(numpy.array([37.610290074531157, 1.0193878705685611]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[0.16549905132924309, 2.2004831313024971], [4.3195440401432323, 4.6734240567118848]]),self.functionspace)
      arg1=numpy.array([4.4934983844153278, 1.5290609859659308])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[0.0003087869090805081, 34.602244917616865], [9.3674915806558605, 10.566059294542912]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_array_rank2(self):
      arg0=Data(numpy.array([3.1867237146473304, 1.8263977206983601]),self.functionspace)
      arg1=numpy.array([[4.3828090344166997, 0.5498961729275732], [3.1513895216067471, 4.0389162076405158]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[160.71722957763765, 1.8914164679084773], [6.6740409241866629, 11.390996677454744]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_array_rank1(self):
      arg0=Data(numpy.array([[[3.1447332639303265, 1.8782503062881659, 1.8045254479717723, 3.2624972714526828, 3.121309803949964], [2.8803948918722737, 2.33419371674391, 2.0040893546804797, 3.4691202637324081, 2.0548245169179702], [4.7616809565263987, 2.2978742928712914, 2.1093363074336549, 1.2890641985403462, 4.9016470284788722], [0.42228447216982234, 3.3168331790411778, 3.6034695907498469, 3.5665809649394751, 2.3454327446469883]], [[2.4183442286059171, 1.0846808439805047, 0.30453491567013552, 1.0343994447074518, 4.3562070068360059], [1.4210630998777287, 4.6063357345561053, 2.1409347516809665, 0.52254769003248891, 3.8875518475499744], [2.6264128402327769, 0.83521571552404283, 1.7458969748957824, 3.2939559731382464, 3.7889585303674731], [1.9898979388077229, 4.253022733015186, 3.77314509160087, 2.2749254449086855, 1.633122443121465]]]),self.functionspace)
      arg1=numpy.array([4.5692724824690387, 2.3561789121857131])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[187.7569541452111, 17.817804514799356, 14.838583730286198, 222.10104469524862, 181.45121756970522], [125.70759773758033, 48.096977700558035, 23.963012313582986, 294.04204652003557, 26.862906923325671], [1249.8890346768785, 44.771141453188513, 30.276844664787902, 3.1906153649610576, 1426.7926391201447], [0.019466494074683195, 239.51248103798667, 349.7915489817716, 333.72617150909309, 49.164281134361829]], [[8.0100883009100912, 1.2110937366680288, 0.060723473612899106, 1.0829495186681033, 32.051915717032848], [2.2886763736198379, 36.558170073622982, 6.0111952432806008, 0.21669768196613179, 24.512218189702438], [9.7295832973738126, 0.65424987123538014, 3.7173987714524328, 16.58959330999479, 23.072586113877733], [5.0593978792075935, 30.291743625209751, 22.846340168422884, 6.9355117888279016, 3.1762178336616378]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_array_rank3(self):
      arg0=Data(numpy.array([4.0810455765535485, 0.97749478294117786]),self.functionspace)
      arg1=numpy.array([[[3.9836400514043451, 0.96645198925917497, 3.0105503433613179, 3.7881290037876512, 4.26142944451618], [0.55521671919512217, 1.9841868674676439, 1.3659924981050615, 2.5454633194727192, 1.9238326260627747], [2.1923315526417246, 0.62321268947067787, 4.1633878253722107, 1.73635554798437, 3.5801041975839127], [4.2811555894271924, 0.56011966007088854, 1.8816041779309134, 2.1446167207886351, 4.6946587954395635]], [[0.87595396667243886, 3.9830797401698614, 4.3227412560540586, 2.1823125736174327, 2.8503947960978011], [0.64057215022006897, 0.57666443307289528, 1.2422146964818177, 4.4638792285682989, 0.90315999861271878], [4.8153156231345182, 2.7428980075181491, 1.92858005601273, 4.4615875811868282, 1.7138076469956154], [4.1005295371935402, 1.8759323326797237, 0.84365267157809021, 1.022236341392176, 2.6691190813863623]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[271.07757402547043, 3.8929719807932481, 68.985558283243634, 205.91170053591435, 400.64471602329797], [2.1832854364878909, 16.288634566843434, 6.8282468312876459, 35.86710380295132, 14.96311565797167], [21.827978140872933, 2.4023734238484602, 349.04257020316942, 11.495196167379518, 153.68285283650019], [411.91497929768951, 2.1983917932023243, 14.100356807772746, 20.411299412374593, 736.82361425760246]], [[0.98025871403451892, 0.91332439694338385, 0.90629026971306115, 0.95153909390545077, 0.93717840872546976], [0.98552487569432889, 0.98695955033298743, 0.97212032235820944, 0.90338336784343676, 0.97965185467407678], [0.89618557210168914, 0.93947437662427424, 0.95705064854740185, 0.90343049245862805, 0.96174085565985945], [0.91088594872909834, 0.95819825075636011, 0.98097971678432394, 0.97700014835191096, 0.94105343650376005]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_array_rank1(self):
      arg0=Data(numpy.array([[[[1.3239379509756473, 1.9357423308686339], [4.2022965930557001, 4.5877427035998295]], [[3.2045432032535754, 0.64988229264451725], [1.3388829696322802, 2.7042448278528894]], [[1.5095586488578219, 1.1730583282903495], [2.0737581069210513, 1.4169443222643814]], [[1.2161776915638201, 1.2673800514098981], [4.3173503100583401, 3.083611781080843]], [[3.3729060807983178, 1.0249698623747918], [0.69138594467044101, 0.38296485065681585]], [[3.864327066408511, 0.50043832109631392], [3.1138306298996605, 1.5228239002700494]]], [[[0.84381837940058468, 3.6678957564904517], [0.73716154417961666, 1.9088823867755191]], [[0.77173694764457079, 0.5374111365808909], [0.67605286399715636, 3.7333892239723756]], [[0.76002861298019875, 1.0114500455214404], [4.2790272725573955, 0.20937941905135968]], [[2.2588146396641533, 4.0924348289798527], [2.4484639958319865, 4.6074290818006309]], [[4.0038389806304435, 2.4319091209232004], [4.9979818556031397, 1.2182276488235639]], [[2.7652909707048119, 1.9741598168126702], [2.9193817628093739, 4.7508566843969362]]]]),self.functionspace)
      arg1=numpy.array([0.32757382911013994, 3.9860313530053233])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.0962778684250258, 1.2415486712258161], [1.6004345786774565, 1.6471097393813163]], [[1.4644540458239008, 0.86833948027244212], [1.1003163453390779, 1.3852461924415884]], [[1.1444229970144051, 1.0536764878946767], [1.2698770156407526, 1.1209318554460004]], [[1.0662101277907321, 1.0807110610349699], [1.6146580405743161, 1.4461160870180076]], [[1.4892251892756942, 1.0081117453689925], [0.88612837949956746, 0.7302200774901112]], [[1.5570768140468505, 0.79710422521364221], [1.4507431464537972, 1.1477075972909689]]], [[[0.50819001115002749, 177.73943511671075], [0.29655186166257758, 13.158142271559086]], [[0.35599924241110015, 0.084138332120992781], [0.21003785189196231, 190.73112472087053]], [[0.33495341870059758, 1.0464263956320294], [328.51952161921741, 0.0019643623211414284]], [[25.738257534558574, 275.02902617942567], [35.492988603479596, 441.13042981780222]], [[252.05228032289918, 34.546034633519461], [610.12312284363168, 2.1964245916190968]], [[57.649138349100554, 15.045379265002332], [71.558969623292128, 498.46438093376645]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_array_rank4(self):
      arg0=Data(numpy.array([3.717452308167259, 3.2500331453072939]),self.functionspace)
      arg1=numpy.array([[[[0.83598480275611542, 4.1605655349411057], [4.7497626785333642, 3.0481444056349996]], [[1.3403885513634946, 3.5719240966410868], [2.423621215060491, 1.3786128181067934]], [[3.6652554386282734, 0.21046030362833348], [1.1411386402477302, 1.5462908124629224]], [[4.7576802126723168, 4.5583725989568995], [3.1502264435710545, 1.6418843462240091]], [[1.5209307061100004, 4.7752785843930408], [3.4420266263320793, 4.8958859907673782]], [[1.7458240066138848, 4.1368374692370473], [1.9753650484973051, 1.1305576784701001]]], [[[4.8005427167400239, 3.626236006839628], [1.5316755131126349, 4.9567857918673788]], [[4.9714859062295389, 0.83737681398981068], [1.3681822351145803, 2.6001548978531512]], [[1.0012136009834534, 4.8349529917454266], [4.6579995301978254, 1.8571474675821114]], [[3.8328670035660046, 4.9478798407725924], [4.8012753637161483, 4.3530655198675285]], [[4.4483671835335183, 1.6956872358945356], [0.48256720758377242, 1.4911683445468702]], [[1.0907183899998854, 4.4576723988840579], [4.2845937496134336, 0.4266472463198801]]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[2.9972053070404674, 235.79977207151288], [511.12879145018792, 54.725576198262026]], [[5.8123319988920326, 108.86124710031901], [24.102320811254668, 6.1114973194852844]], [[123.05381263391817, 1.3182992991806952], [4.4743408673360499, 7.6166716093911093]], [[516.47021722532554, 397.5493351829881], [62.575186983995962, 8.6352915476036376]], [[7.3672218393223892, 528.54342321119634], [91.790951042545501, 619.23642192769569]], [[9.8980152532809225, 228.56649076081899], [13.379591171334624, 4.4126077346159445]]], [[[286.64158902045796, 71.816781154338472], [6.081994087375425, 344.60226302529696]], [[350.62504490170215, 2.6831377331323671], [5.0159798227049439, 21.42829145039104]], [[3.2546854143256545, 298.50623192504901], [242.31097547651055, 8.9258814235891872]], [[91.621589173429584, 341.00384626775821], [286.88922397878861, 169.15353931880298]], [[189.26261976353143, 7.3790693497678213], [1.7661202149540407, 5.7984360078664903]], [[3.6168077640944416, 191.34982677425418], [156.03830574005815, 1.6534669407050551]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[2.2424385067183676, 3.6039381390448311, 4.1309416296257071, 1.6566078326765861, 3.0896146258769077], [2.2614306709678309, 1.5649934410510165, 1.74666069260594, 4.9778873561920118, 3.9388338633368716], [2.5715540951907014, 2.280036739368398, 3.746108852647219, 3.4651974020126617, 1.1994884096922604], [0.24052607922166327, 1.5811463983796059, 0.417231948602997, 0.20443974100343928, 3.1866615655770336]]),self.functionspace)
      arg1=numpy.array([[1.4484835614968037, 0.31580264238259498, 3.4631921192553352, 1.9061003669874592, 2.5863430019223905], [3.8286381930689086, 4.8159898033102539, 4.1878431307384574, 1.1679325969788799, 3.6184703779336651], [2.2934812990641258, 0.77899589292096594, 1.7660558827217669, 2.5759486594154493, 0.46346940669013631], [1.2270537150559935, 2.881761312546979, 2.4953929552306007, 3.8047933086702326, 1.6837224437760514]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[3.2211647688691167, 1.4991039598146805, 135.98664030383489, 2.6173076256529519, 18.495266344388092], [22.740779821753637, 8.6451059736344504, 10.335497807165522, 6.5178093472738459, 142.66592370732994], [8.7252139619784739, 1.9003571412119129, 10.303268719859469, 24.564705781906103, 1.0879583135087278], [0.17404147051971036, 3.7444673788378622, 0.11289971041833263, 0.0023814614225233796, 7.0384660948319997]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_array_rank2(self):
      arg0=Data(numpy.array([[[3.500442867065797, 0.27585047953663733], [3.8187854782818493, 1.1427595630185445], [0.64833633129340507, 2.0923546505271609], [2.5741040779106301, 3.0791329754886503], [2.653847185018896, 2.3674973521797167]], [[2.6958131447073197, 0.034206055262766731], [4.1548670842087017, 4.4189635131293388], [2.3525159005313721, 4.4394935068100745], [1.4772824364888952, 1.8644277835123524], [0.30743666219224636, 1.2890939056297686]], [[3.602233730088793, 1.4890048779845393], [4.7013459598592684, 3.0705972732990876], [1.0373112237598023, 2.8608924597182175], [4.5696714762880912, 0.17571209779824132], [3.371303025057828, 3.666353025970349]], [[4.6440532526231975, 4.9908661649578656], [0.8036891650246466, 0.18595980573104423], [1.5851373577982735, 0.083692420135069917], [1.8454739253142576, 3.696019945280459], [1.8951105584430459, 2.4541066909536737]]]),self.functionspace)
      arg1=numpy.array([[3.6673413491413358, 4.3492870360092182, 1.6399454960266349, 2.8451829182351585, 2.9392961247458413], [3.1996231061817113, 0.62250647618756139, 1.0983662138569095, 1.2551315110970929, 0.28981893222524474], [0.88964592730642145, 3.5010238302569561, 1.8433001626096257, 2.9385112604711656, 1.0690544715278136], [2.7201379181582817, 0.99227647096357363, 0.350582091641639, 0.11415914638379417, 1.6942885335591613]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[98.965712031104204, 0.0088870923065785621], [339.59399314524012, 1.7867440313423819], [0.49131819384423464, 3.3560190224522581], [14.733515113444579, 24.528346227923716], [17.615575011648932, 12.593535844996007]], [[23.880586722015739, 2.0402437926512686e-05], [2.4269220219586485, 2.5218319227397341], [2.5590502830578457, 5.1405449759918209], [1.6319205906376903, 2.1855979025650107], [0.71046369680145616, 1.0763723971301911]], [[3.1271730439790186, 1.4250048073013808], [225.66592146748312, 50.790073727835974], [1.0698556984606129, 6.9417560933701505], [86.911939346274124, 0.0060372781482334204], [3.6664400457520991, 4.010487619804441]], [[65.170692674812869, 79.274735991784794], [0.80504687425552668, 0.18839169202987741], [1.1752760138357814, 0.41909554943411664], [1.0724539478181261, 1.1609461182667877], [2.9538812813280253, 4.57711668282652]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_array_rank3(self):
      arg0=Data(numpy.array([[1.0077997150395439, 0.78015180855820832, 1.7652037759444517, 0.49288003688965515, 2.86248835548141], [1.3770418380737617, 3.9305807674432791, 4.8733715903612245, 4.2657405933255435, 0.92727111149733554], [0.46802267612672516, 4.6573768054689868, 0.83955763984342902, 0.98353083474252501, 1.4549660307435985], [3.6804750234760091, 2.059688855325188, 2.2729130340396533, 3.6235422138290825, 0.44084406649593572]]),self.functionspace)
      arg1=numpy.array([[[4.379575628529107, 3.0198291351629858], [3.0335987646724218, 0.092692672678198543], [3.0124640041452011, 4.2708354782052069], [1.9540206326251275, 2.3749402866365901], [0.95606770992625212, 0.734485660430238]], [[1.3887165889948316, 2.7748636430731684], [3.7686479836105895, 0.31583138536918298], [2.0477071179268185, 0.45417367695796707], [1.7266022782481327, 4.037787160761285], [4.1184280526739583, 1.9972683891950023]], [[2.0423727983761042, 1.4381556885577018], [0.28877443144143972, 1.2758717806962172], [2.2468863212708992, 3.9048573220635632], [2.8287135869213849, 1.8706488389456342], [1.4222999864858841, 0.028678437277337522]], [[3.9320996462762139, 1.1269228300212448], [2.5098155129575828, 4.9665029170538464], [3.9799355935669252, 1.9186358554446663], [0.69930939550596993, 3.3967620878365774], [3.4292172377751564, 1.4981336627095516]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[1.0346124515157771, 1.0237398330935408], [0.47088484193672708, 0.9772502594992597], [5.5393728148535279, 11.324530133094292], [0.25096318680445479, 0.18632827555560516], [2.7332410248022638, 2.1650688876483954]], [[1.5594023773563077, 2.4297374882944083], [173.89942148771487, 1.5408064611420618], [25.6137648909841, 2.0530241851394919], [12.239110010219424, 349.77073390591522], [0.73272882425148833, 0.86000908312342061]], [[0.21211045557171243, 0.33557721088041698], [1.5593447627174792, 7.119716931613806], [0.67507208186172396, 0.50515901653550166], [0.95411178723787071, 0.96941300868332347], [1.7046122766959142, 1.0108119448122965]], [[167.95402888075347, 4.3424017017455467], [6.1317462140030292, 36.182340525407433], [26.252861889621887, 4.8322854657965149], [2.4604083238665408, 79.294466403110164], [0.060280381144870873, 0.2931510026337118]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[0.93829829538380427, 3.6434622552849021, 4.1240721176441326, 2.3609821065867886, 0.095081531164207589], [0.66427266494058679, 1.7111055739262504, 4.5523366431234482, 1.2351689349929742, 4.6884822214833886], [0.6408612977014515, 1.5863183043025928, 2.6515451987155747, 2.9741591460064511, 2.2514741410766392], [1.8223064485485352, 0.19342737784573188, 2.9435828153255392, 2.9686434921637459, 0.20062464198954466]], [[3.6515265497891849, 2.5273445130429493, 2.4002206951791121, 2.9534249025256534, 0.58854056675732902], [3.5677729947135099, 0.75553994690511694, 0.16765074294539437, 1.4815173779298292, 0.92163099135093163], [1.4024424443912362, 0.80937673556701695, 3.3207587612345626, 0.084927333956087134, 0.79727279366041326], [1.2502961441485116, 2.0767362168808128, 3.6942573166645771, 3.7356043263254066, 1.4714581918679754]], [[3.5069836903069, 2.6949550786289787, 4.8632385462496792, 3.2982681339005735, 0.71881932360374223], [0.073046508440611779, 2.3192094847135674, 1.1538665118754265, 4.0615080220844746, 0.93177734527151623], [0.43178225810017229, 1.9013607574641422, 1.7739546643018889, 3.5637887755066155, 1.2539554833195998], [1.2110298484892261, 1.9488037573800345, 0.3984340418569966, 1.3135764747843897, 1.59387984235277]], [[0.4767572571497431, 2.7122205144204079, 4.15288026978721, 2.7721687130909833, 3.0118267319135072], [1.8829350632106325, 1.4994266020797569, 2.0131256423501687, 0.38829069050683745, 3.269980717929966], [0.56034156069370145, 0.20980403501750516, 1.1823179527937453, 4.647004718477338, 3.9944513569394178], [4.4489534638021109, 0.66351421029799118, 4.270350593075654, 0.1137164993650982, 0.19377375086397031]], [[4.4428961176297186, 2.466503763524416, 1.56798280014558, 4.7525476956662063, 4.6107596925308911], [2.7192987416277425, 4.3766718266307407, 2.7331856987400944, 0.064907262116195963, 3.5525594357507861], [0.32468870468383643, 0.40548808755373084, 0.92682467133418522, 2.0994779337198946, 4.0425921998720069], [0.34293292856909913, 4.2753216108332683, 0.10947350537096087, 0.51200734860065333, 2.9079004470645637]]], [[[2.5356894857185313, 2.1486265805661389, 0.52318114227611667, 2.0084893023510371, 3.0830553449035776], [4.8049437777722233, 2.692383236071159, 0.68814039718617193, 0.46758500473933168, 2.0929859442442176], [2.1743175725785782, 2.0155858174729726, 0.57073696250852102, 0.4948800507939109, 4.3470245289263509], [3.8057571166561837, 0.13852841751898995, 2.4087878019155213, 1.9116333420902083, 0.085797325087652559]], [[2.9954520348771005, 2.9645735494018073, 1.3485209052473863, 3.6712093141777613, 4.6040421886977843], [0.79336388280126069, 4.5403184862486912, 2.0372123824139856, 1.9624635269308082, 1.3623053526836553], [0.19107676309762298, 0.56085764258912285, 2.2176224918087803, 1.4638005182298106, 4.8061955439188324], [1.4922325723461067, 3.6393567506684099, 2.450552917759508, 3.7916535728369554, 1.1823033966175953]], [[2.0042890498712866, 3.3683104602783032, 2.5514948049929331, 3.092950738625563, 0.36720388169562573], [4.8613974382081837, 2.2440699419135464, 3.8155213546037934, 1.4621973305065403, 4.7907532602524245], [1.5991057204535293, 3.6077709808297, 3.567065674895312, 0.81464522071212797, 4.3891250574486786], [4.6797404181941538, 2.5845431833088903, 4.3011364106260546, 1.8796436060077693, 0.83109111657379076]], [[4.8267389143242072, 3.2883471389444061, 3.95832532372811, 0.45428566603826254, 1.9180434077203998], [1.7076888422479775, 3.6250310273300799, 2.7530164195117184, 3.0120793225814912, 1.8120300115419627], [3.7219587636740945, 3.4015775788885616, 1.7436147870204579, 3.1358356953890687, 3.9638890486064531], [0.2796518487006121, 0.42437144936463589, 3.8143862086020008, 4.4645845160710884, 1.6333988498515846]], [[2.7543576826795735, 3.8392222016040121, 0.78485616765416777, 4.5782314430000541, 0.23675800228886065], [4.4722202351485931, 1.8314215104870253, 2.1943977611861949, 4.9540070319116198, 3.056434349385992], [3.5874254547429749, 0.25451417635275736, 0.029249406819877734, 0.041078821575485577, 1.2683452311146945], [3.4529545803645609, 0.71147669535865654, 3.2715035319210051, 0.61449022556560662, 0.3538238684021906]]], [[[4.1299752232634752, 4.0326761755391809, 3.5850974455747493, 2.4857599474684768, 3.8379566708395041], [1.8323465829582413, 1.3178711508744418, 0.26401047885098361, 1.1272884401894931, 1.9804305646292164], [3.9708450572869802, 0.66343275643365918, 2.9084414463949915, 2.9532527091376153, 0.54929514190261997], [3.7739394199944272, 3.4530807538420203, 1.6687113919946832, 0.87469263872741709, 1.0298102232968933]], [[3.7927492683198927, 2.1627946458745964, 2.2469044922467467, 1.2635950024484004, 2.9046467178483706], [3.1905691295136314, 0.97484236967899884, 0.95077945682540266, 1.344454187266954, 2.6262919082377869], [0.098791828659572548, 0.38092838904409082, 2.0382547969227156, 1.922344323847893, 0.91534299637854433], [2.579514903921825, 2.8249063421408027, 0.14870824240512964, 0.50937625010593868, 0.89476825480988942]], [[3.6130960369656919, 4.2922553543846504, 1.1605290037577893, 3.6811672829325843, 3.7867118118335892], [1.4955288922398229, 0.035964255197560614, 0.73911083108638354, 3.6352509018365158, 2.7415854533631783], [3.9814997159596364, 4.558434463822195, 2.0950170903038581, 0.48772563080992298, 3.6448379844483676], [3.1110617378518968, 1.069647509811525, 3.9529885578791046, 1.8371374917820391, 4.3140551428972067]], [[1.287774961201279, 4.6920744773805776, 4.9674284541549003, 4.0396241624246665, 2.3545557079499706], [4.0139204625469853, 3.8719684461044706, 3.3767103428205378, 4.2990989985210097, 4.1869317806767787], [4.7220004878221964, 4.9362586146029495, 0.43347830383920155, 3.2482749697153293, 3.1716859221670002], [2.2139832862403637, 2.7716970462169486, 1.7745348156311891, 0.75374132293320062, 4.3303383351123399]], [[1.1507936337399161, 0.35963358853122102, 2.1994848583879629, 4.2430660506889266, 3.6419843674286652], [3.9055749882091773, 2.9854838453653021, 2.8559040808086031, 0.24372483380571197, 0.8046357336163299], [2.2416123101487897, 2.8320890515239294, 3.2980411449493343, 3.9200803873266747, 0.27966342874918809], [2.3856764809852726, 3.7196934350230824, 4.6199080309402145, 2.7354967836774446, 1.1347374966593389]]], [[[0.8691630362573457, 2.137813581607853, 2.0384126922237882, 2.5375809021904572, 2.0629086204337761], [1.0881329702206328, 1.9886625199925223, 0.54335876020851082, 0.46538578509538614, 1.1510983733636788], [2.4812463768402737, 3.142807851793652, 1.2910322747604885, 0.49646713675840082, 1.0819444453725973], [1.2438295002891435, 0.62932830866103628, 4.1885676002184189, 1.3990782819251435, 2.9252948396120688]], [[0.44355136845307774, 0.013686119309205363, 4.3386805187656972, 0.89093416396756986, 1.3818410447690093], [0.7422235489103981, 1.7637568256815881, 3.2835665697623004, 2.291152176659363, 3.6854465230015614], [4.4404009412904024, 2.7520998257576004, 2.5513610183328002, 1.1487816492405796, 3.664239304650061], [2.8863991576826256, 1.4118779727962396, 0.31603142805900664, 0.17711535801358461, 2.3861664342293212]], [[3.6833162039011711, 0.041722631446837244, 4.3264188450393251, 1.9722487178255081, 0.13705265983141032], [2.4430663469103191, 0.37720081281033346, 4.340216631891713, 0.86703214732395406, 4.2371678007403402], [2.853047273959509, 0.13694037219527727, 3.3609776590060725, 3.2580850617161188, 2.0889516368543357], [4.7697939076428923, 3.6575622670438208, 0.91190026754848785, 1.146493735800924, 2.5514160486690547]], [[3.9007976710818766, 4.1811170935333948, 2.6192059300907271, 1.3190823370685456, 0.54219596558249683], [4.6020979500120003, 2.6421089434831249, 2.8025877949874012, 2.4703375693536618, 3.6959699726783892], [4.3107045225549943, 3.2621251960109037, 1.7228876966665847, 4.5154705933623678, 3.9075913482047708], [3.631896654354295, 3.7251593823358471, 0.6931367639938526, 3.0569225530878725, 3.5480196424598476]], [[1.800657209791771, 0.27910813186904471, 3.826737266417767, 0.87515346360901192, 3.6867313113494133], [0.43592782335363894, 1.2893965106428984, 2.2347686737954771, 2.1689206554493503, 3.5877105922389818], [2.5168444979295796, 2.9829984355314467, 4.4091479989493347, 2.3692721895974356, 4.9836889042494539], [3.8772719151163253, 1.0938770386186725, 2.0432271285457189, 4.4669935329165815, 0.28375764248934937]]]]),self.functionspace)
      arg1=numpy.array([[4.6227116492406441, 3.1943477920021079, 1.5187135587067428, 4.229750770890476, 3.0947549893954149], [3.9167887873522211, 0.22220700459425829, 3.2622148430029454, 1.6699845702243439, 0.39559974520829716], [4.7359285570587426, 2.0375679248641116, 3.6766972171201258, 4.9813872242112351, 1.6328278140271895], [1.1979994864290162, 0.38212469549793232, 0.4675125054415159, 2.0291171566027417, 4.1882068954914597]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.74497222752443282, 394.20177841054664, 698.99783310900955, 53.051778838080651, 1.8881358842553082e-05], [0.1509240837390626, 11.977644215665091, 1103.6406598322631, 2.654755412843727, 1264.7034632450429], [0.12785740895506334, 8.4400834063948142, 90.722272557519716, 154.25037157558933, 42.594462012330837], [16.024247246783062, 0.00050324351864008076, 147.05498250181319, 152.93242789171933, 0.0005958317754609253]], [[62.623748228274401, 19.330820384152428, 16.392851491022029, 31.796887019621959, 0.18390170753427013], [58.149879690190879, 0.40842460196743835, 0.0033302847223020508, 3.5099160574236645, 0.77051829580162212], [2.9457909642113158, 0.50886366994242227, 46.239518709433931, 0.00037931954174648646, 0.48495155920463695], [2.0412345792915025, 10.323485980602324, 64.994869095114751, 67.347221086117187, 3.4343552817151335]], [[6.7235442716327496, 4.5069677145696563, 11.046970161919784, 6.1253077422969344, 0.60568450076240887], [0.018798928436626658, 3.5879532759494914, 1.2427864390621919, 8.4027545724677086, 0.89824320721248962], [0.27930048905606375, 2.65349992485974, 2.3882092461890925, 6.88963429958911, 1.4101405018682449], [1.3374833806136661, 2.7547028284866455, 0.24720423255735857, 1.5132119568870346, 2.0298877917504123]], [[0.043579055132864003, 68.054482842939905, 412.53793932196669, 74.647761468810742, 106.00625508648359], [14.537341842622279, 5.5477872668787347, 19.288483863762725, 0.018290956288445709, 150.10595487155166], [0.086301468688386673, 0.0013534479288033129, 2.0307099389717975, 663.70637778110074, 349.95627743648947], [552.03828235804372, 0.17638874348064415, 464.19836837573581, 0.00010147723959584061, 0.00096701953667573267]], [[101.01086181621083, 16.345456438840181, 4.0228463522457636, 124.42900090207036, 113.29564564414423], [22.10740289847266, 96.423644309565859, 22.458668178484068, 0.00021102734468522426, 50.558082911039371], [0.030768804307281704, 0.061205192621764305, 0.79043400228889571, 9.9278589293175408, 75.416062966084155], [0.036440666896688498, 89.679630711855822, 0.001063890204874788, 0.12597403830470041, 27.206023980471667]]], [[[38.261232464670741, 19.998814924673191, 0.079071436436666365, 15.355923908256822, 82.269063792380223], [467.76651600996286, 48.389964104955965, 0.23132112957580722, 0.050922955567238568, 18.045706654879179], [20.951872823335336, 15.56953257449627, 0.11117605366225608, 0.063594684750459735, 315.98293257561852], [187.70034355502548, 0.00043409929818164232, 31.291346608852454, 12.653258169520482, 6.6472488078282255e-05]], [[1.2760664074710586, 1.2731316454918353, 1.0686987141671715, 1.3350715724683009, 1.4039582598980966], [0.94986540126748376, 1.3996169150215201, 1.1713050942704077, 1.1616159459860882, 1.0711165413771504], [0.69227612667151539, 0.87941384355657681, 1.1935995639093393, 1.0883566533850833, 1.4174281544354055], [1.0930189497231069, 1.3324889078286726, 1.2203859704936395, 1.3446825859886571, 1.0379128272457199]], [[9.6618334770354597, 52.545027861513724, 21.234970921165221, 39.783417660732823, 0.038074483163310552], [173.92513322214882, 13.968704090449313, 78.913939985723886, 3.4537029294648387, 165.81483689800365], [4.6247807911892185, 65.740531300753787, 63.351585146443298, 0.51234224245772786, 124.61631372036803], [153.60571909601873, 22.14545104532559, 116.64995428153827, 7.8359778982504888, 0.54686055824451274]], [[13.857704951563226, 7.3003673201778874, 9.9502721690341591, 0.26775942939824737, 2.9673530183346877], [2.4441002258373423, 8.590965123103695, 5.425926575794338, 6.3052092952511654, 2.6985608272217543], [8.9780019134114042, 7.7249918748948394, 2.5305719048485233, 6.7437638099840891, 9.9736393321963437], [0.11908583020014624, 0.23896914383954046, 9.3534152768022558, 12.16534207774483, 2.2691372328264268]], [[1.493041685533705, 1.7026543026372678, 0.90861307208241138, 1.8254580584317142, 0.56555531167819217], [1.8086178183983586, 1.2704541282607902, 1.3646593844548711, 1.8833223540849688, 1.5557899394176213], [1.6575703441293292, 0.58196894394589749, 0.2472841615897485, 0.2828444261393257, 1.0986028489344919], [1.632706679061221, 0.87400678146317701, 1.5982103459049581, 0.8247770722908021, 0.66297980894315411]]], [[[826.19602320940623, 737.98381603687358, 422.74351409487633, 74.621925706180363, 583.78937880172043], [17.602944135586537, 3.6958108169336406, 0.0018232203417275679, 1.7637389100102374, 25.435036222329622], [685.90950872723863, 0.1432329379501725, 156.98645168908482, 168.77576850533731, 0.058578555630392735], [539.08786334946728, 353.92288493480595, 11.302684930288489, 0.53043438297722434, 1.1492570660410597]], [[15.123711071528046, 4.8152229546991752, 5.2044822219835618, 1.610768014544296, 8.7818111070573774], [10.633239571285131, 0.94940842579351348, 0.90226909550838708, 1.8277686454679438, 7.1522030797769895], [0.0089469663464751218, 0.13993934416517578, 4.2671228394119014, 3.7872616287852248, 0.83507313315687426], [6.8950387808247946, 8.2975789571466443, 0.020586211312097667, 0.25297140652450606, 0.79727289806117008]], [[112.5003891961654, 211.9303741762547, 1.7287035771248744, 120.49180477735162, 133.68886041551221], [4.3920531063968529, 4.9020211926192391e-06, 0.32906737648284762, 115.05757991066795, 40.77556164091142], [160.76443993707409, 264.40271313622651, 15.167340801266532, 0.071370053031617198, 116.17716628644695], [64.904646181162576, 1.2808820604620772, 156.57215720518084, 9.3576954555016698, 215.91482223517434]], [[3.5249702533279024, 2209.6756047365134, 2935.6289484213385, 1048.1395715006749, 71.223382073314937], [1015.3358541720349, 848.6253568210883, 429.17291268934986, 1429.2174756403235, 1252.8648224532867], [2280.7767724152682, 2845.0078141881163, 0.015545112585030279, 353.78600514414421, 314.13799758195165], [52.413967778119925, 160.50481577436577, 17.409506791958719, 0.24456643965263791, 1481.7048769206274]], [[1.2577616077009901, 0.18827656110359436, 3.6220230674870204, 10.589928387518395, 8.2521749327466587], [9.2495306065250666, 5.9650935068601054, 5.5481839788005107, 0.099750001930937901, 0.70122911027345403], [3.735983415734919, 5.4728397218242746, 7.0181545038707762, 9.3056889933345435, 0.12486805402371239], [4.1359439090761896, 8.5416139642682047, 12.168352650608773, 5.1713612510828737, 1.2292345145299473]]], [[[0.84536317752566559, 2.4848700928492149, 2.3471010876559317, 3.0513696979253555, 2.3809313164144292], [1.1064835983519914, 2.278641648075638, 0.48154177653615449, 0.39998184659198832, 1.1836210275370993], [2.9703958226689946, 3.94263437687012, 1.358008812216744, 0.43219225742545275, 1.0989489703534077], [1.2987436976875038, 0.57418926682448101, 5.5620409792872643, 1.4952666915543225, 3.6180202672850865]], [[0.7329739740304746, 0.19401114797447216, 1.7520645650905946, 0.95682997035376105, 1.1315467569175834], [0.89233540114967647, 1.2421393383948718, 1.5751013049659477, 1.3727311870815022, 1.6461517281040896], [1.7676488934088519, 1.4723346807463253, 1.4303344354317431, 1.0544311620499673, 1.642525606760219], [1.4993863230155051, 1.1408832267405689, 0.64392395692121918, 0.51610746109682681, 1.3942118745217931]], [[1.8396017551174384, 0.22646782698652945, 1.9833440499036696, 1.3737201706663715, 0.39489723955716832], [1.5183244511953236, 0.63393160553018213, 1.986298684935065, 0.93547175931095106, 1.9641095823845491], [1.6325355089260518, 0.39474594734791213, 1.7625005600978936, 1.7370661028257064, 1.4111413598536118], [2.0759029914335856, 1.8335770985906212, 0.95780010879337851, 1.0659995117090368, 1.549442045502466]], [[15.831408534191262, 18.225310106125633, 7.0552953948469215, 1.7540654163495932, 0.28878321713643901], [22.141911227914104, 7.1810416809582014, 8.0937559459471018, 6.2653964013714152, 14.190166985741241], [19.389774181282721, 11.014199217288628, 3.0157342650470205, 21.304391138422677, 15.887405847098691], [13.695454791810235, 14.418491713244471, 0.47533847539879437, 9.6538150669888143, 13.061289630926]], [[11.743513758062411, 0.0047728800570242258, 276.06220531085347, 0.57205358793087546, 236.16296074289869], [0.030888277850934325, 2.8994889705675448, 29.017385438307951, 25.601065170976597, 210.71238954490033], [47.73857710472106, 97.26242081483791, 499.67854271289599, 37.065080266355572, 834.61770341918668], [291.65520920325849, 1.4561568800053957, 19.93748242057432, 527.71405472513345, 0.005114830298086283]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_array_rank4(self):
      arg0=Data(numpy.array([[0.54357016197252805, 1.791701850316187, 4.1328837918970693, 0.72226729959937974, 1.8852296666143138], [1.6507104854366337, 3.105802848536523, 2.4117480471915349, 4.9345380147952591, 3.5030543089686064], [0.69644163885865351, 2.976542065131472, 4.8621247072285954, 1.2628638671131518, 1.4669207783274834], [0.53079307236758022, 0.19781385888753128, 0.23991713068067833, 3.2639984915314697, 2.7066276641171139]]),self.functionspace)
      arg1=numpy.array([[[[0.47621978694862582, 4.9590250317317333, 1.3408333592473518, 2.4056597551465964, 0.45448728513076281], [4.2132264118733023, 1.9810751088079823, 3.6907580467188557, 0.62190446642484465, 1.0278032951872329], [3.8292827274317776, 3.6247308332116774, 3.5100802701536908, 3.6665439760147831, 2.2040018926773315], [0.50917257347120515, 2.3524606736549321, 3.6714289533233968, 0.51184567714435736, 3.893840068331369]], [[0.46581660472588937, 0.76549595820332861, 4.9294885988000328, 4.3806313052957639, 1.556850393157027], [4.5338750524972911, 4.9342342144157998, 1.0293651101612398, 4.1614041806407522, 2.0526734259933939], [4.8425511243615169, 2.2854698348876337, 4.2180356792923943, 0.60495791825438705, 1.2245966959249872], [1.8061460754082772, 2.2766654858434836, 0.099875495808300477, 2.578688129299922, 4.5637363971227654]], [[1.4826237491030798, 3.7842241759819055, 1.5761679520078389, 2.2899854172888356, 3.7306319651718867], [2.285655096426821, 1.8346705370746255, 3.8899488686128301, 4.7267060861061818, 2.5803143331628906], [4.0696366528140917, 2.9373637052885182, 2.1507565873744041, 3.1891941329035416, 3.5039118676555225], [1.3707994952252285, 4.0874807978530736, 4.0836980114052723, 4.2863716431772945, 3.2187958551094931]], [[3.061450757386412, 0.17194454654434002, 2.7273806321232645, 1.8338701274332598, 3.343639392716824], [4.0418117454891123, 3.9417306258461009, 0.44687391903495671, 3.6123959626016395, 2.1469783676113763], [4.7068371110062746, 4.1094111936200379, 0.45453747301844666, 4.4686411930956478, 3.5624823833279122], [0.77757057482478376, 0.44918020142882559, 0.10046368484892956, 4.5223007558858361, 4.9662171705779752]], [[3.3304253199958609, 1.1204108978941352, 1.671262232841052, 4.2721254431869058, 0.29792214375470139], [3.9006395204171227, 1.4116619795642631, 3.2033023911981102, 4.5495850006885146, 1.1579283196819949], [2.1751230409280535, 2.9093373575391972, 1.8376778760318804, 3.5591646938478081, 2.4984715985706147], [0.70973821575043772, 2.2708995969054402, 0.20559057902621583, 0.091717599650712911, 0.69143910191137326]]], [[[1.0865253872020202, 4.4642461435464256, 1.6166023476362676, 1.3194862349483265, 4.8589089292557635], [4.3863156320294863, 2.0727829981768391, 3.4984823404588803, 0.42520329634114362, 2.0310683390859525], [4.9765338328312154, 0.40925977320850088, 3.0421381506586376, 0.55335032573923759, 2.7556985242226104], [4.6045562782219669, 4.9473429045874768, 4.6015958891194586, 0.31506394399709092, 2.5973296974169711]], [[2.7785661887152742, 0.50372728355420471, 0.49344996452261436, 2.0693145957761074, 2.1644705883184656], [1.0998990329373743, 0.41816526650440733, 3.0033773934375287, 4.2709063143402552, 3.4680278771751527], [0.124201010044333, 2.8141436183338882, 2.1914102207146833, 3.5746131613311141, 4.4768167215324048], [4.0813422809983146, 1.1214826494561294, 2.2685972281623394, 1.4434708535387688, 2.2201088158583246]], [[3.4570900018138979, 0.98520514284327643, 4.019079692331422, 1.9381238285280156, 1.0346163106500363], [3.5572126515327365, 4.8559613033489173, 3.5278138842167994, 0.67168067575632839, 0.57214551971465977], [3.3099705426172195, 3.268102815882703, 1.0810825024095683, 3.8036965119259576, 3.3512188416903781], [0.22339436196677082, 1.9673345866247081, 3.7474290427817096, 0.86787612027439609, 0.99262462242519778]], [[4.8752068195287617, 4.2284880606943771, 2.9058819873474033, 1.4633184749248365, 1.9724159641639138], [1.0393565111840539, 4.6878174937097858, 1.2890035118842995, 4.3326980551889962, 1.9964900188543839], [3.5103443482161016, 2.0937698425255071, 2.7128660805951412, 3.0913400561372639, 4.6469938363601733], [2.4086861311182362, 4.197315704513203, 1.9294962063459811, 3.1566028535871506, 3.3985795911910825]], [[3.2609312149659111, 1.5498253657934749, 2.4339870352468593, 4.9238099164364213, 0.74439336407928069], [0.51362814521356348, 0.2993013135710989, 1.0941436429997204, 2.0731527505715786, 3.4623825976841953], [1.7773699988761456, 4.6342075621313636, 4.1797150732280173, 3.6833982651397323, 0.56592969598987186], [1.6751454493766995, 2.3966033002103848, 0.024896683356317832, 1.187546121645263, 0.94572250802053714]]], [[[1.1627733332230896, 1.9574518892656263, 2.0976225881750645, 2.4260956476926938, 0.36017174867958013], [1.525344793013808, 1.0794751920260151, 3.0799685176132408, 0.057269782015166672, 2.8434826270292337], [3.9687622081557099, 4.7434773800676009, 1.0450541904419512, 2.4555678910138377, 4.1100995721200286], [1.2680489093393219, 4.2824429037949399, 4.3820241722637308, 0.84691217671636898, 0.61902588933428193]], [[1.8856646601200033, 1.9195818076426494, 0.23620034867417314, 4.3796357803438388, 2.5800830367153922], [4.4498128532004504, 3.2872812840200019, 2.1992448468964896, 0.11651705198284144, 2.8265196276732949], [3.5260755564377626, 4.7226642924419462, 3.9558898635020565, 0.20311600146501324, 0.50542995936841029], [1.5453938517380168, 3.432554250894317, 0.58917225408852958, 3.6996580371785077, 4.5227066711638173]], [[1.579457466970986, 0.70473304830421102, 1.247004548528307, 1.9897239724976621, 4.7790858453525118], [0.91809682939706527, 0.40049542847154213, 1.7461490929034447, 2.7522223889320094, 0.96468794504814981], [2.1394287966309147, 1.952499567558392, 2.4739469871844055, 3.4543440403597838, 3.8502560627418947], [1.4685107851117829, 1.1305964987882768, 3.1038560206885979, 1.4302805370615397, 4.3745734476068616]], [[4.6101712384055329, 3.2445943734123586, 4.4288976810793486, 0.15032010781566527, 4.7373323838574688], [0.34913884747742718, 2.2577670242869927, 3.8016219035896448, 2.2235528134762506, 2.0034269950294434], [4.5747154115433597, 4.5051408200161243, 1.8485387257790302, 1.5148675901403763, 0.96614895024810354], [1.9306322594118326, 2.4974900586543853, 2.4264429923090218, 0.53558365864036139, 0.21684283050117129]], [[3.6826627527954718, 0.66478442415619909, 4.0086934289556648, 1.5268726779656323, 1.4863094185107888], [4.8564464885839005, 2.6794634078993731, 1.7597766082192174, 0.10818066844441199, 1.7195165775328456], [1.1241512954536947, 0.51508206290461478, 3.4044653436524164, 0.60746852809059027, 4.8258666864961146], [3.3321183544939283, 1.5859476259776024, 3.0466594121945842, 2.7559159904178014, 0.76803067676898207]]], [[[4.0510460839110571, 3.2787805330637263, 0.70515955221039128, 0.44417612441787363, 4.7983993225543182], [3.3238203358465559, 2.1463547912294549, 2.4759261931956522, 1.3913023708841574, 1.0534270216694028], [3.8752767006210229, 4.2372089579849117, 0.11404097429207473, 4.123026992603954, 2.1947808850608683], [2.5865731436625001, 4.4759308343105042, 1.6121641794348764, 2.9787705443412875, 4.8296770683487491]], [[2.1360028619369884, 4.6595335655950159, 1.1994328307576625, 2.1507614986961965, 4.4186983081856113], [1.8764614464853318, 3.7668476573526393, 2.5170783511700341, 3.1097554174922593, 3.7163684587856536], [0.466210910177565, 4.4736498042851665, 0.65473730647641137, 2.2991117284752822, 1.7339519433107728], [4.8186771274604165, 1.6762413503912452, 4.7641040382152866, 2.7437583589399885, 1.2241035246795711]], [[3.721121738652104, 3.7529886459216009, 0.7746267461614218, 2.1004041171420353, 2.5645256403346628], [3.4765931036760009, 0.80986063643697559, 2.944433736692218, 1.1295924750777153, 3.5405501855780979], [0.667073059513415, 2.7497916001607297, 2.9936299415095444, 2.8111801147045812, 3.8888638844670789], [4.4936898700592218, 1.3159632154065577, 2.7174469586568417, 0.97787207471214876, 3.3871989018375999]], [[1.102202972216074, 0.46700028530509657, 3.1720900182561338, 0.7868824892884001, 2.6024021718407631], [2.1021023901231812, 0.25190193596594374, 1.4796441387713006, 2.0315184989761113, 1.4875668861182554], [1.3220888506197206, 2.6343855425307301, 1.3806421394787456, 3.3608791494533055, 0.93201359611817747], [4.8141711860968339, 4.7788135367117075, 2.3544032177903977, 3.5354463976273496, 0.58718763804436735]], [[0.49587076593671181, 3.0295219946362817, 2.4926944056222751, 2.0923506876085054, 3.8647538577884024], [0.083584949518620402, 2.1404751386025294, 0.79167806872479196, 1.4164125978375051, 0.77477654515670935], [4.4052731462242187, 4.5169810731231372, 4.8245552767184865, 1.6796421217086455, 1.6232179829946329], [2.0575210066288689, 3.4554291729309203, 1.66875785597477, 3.7814741497288509, 4.5259032851640617]]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.74803769373301565, 0.048654827665330726, 0.441592703671492, 0.23073584803136529, 0.75801367484872806], [0.076660551622769385, 0.2988969455931918, 0.10541259604782764, 0.68446978390718971, 0.53443493902505379], [0.09687659743738064, 0.10974199310391901, 0.11768630755839159, 0.10698011205278717, 0.26091729139885655], [0.73316110513843546, 0.23834126052814894, 0.10666201358247894, 0.73196738126485039, 0.093138166219967441]], [[1.3121255375185978, 1.5626938464559417, 17.720280353628738, 12.866601158149821, 2.4791161258398451], [14.069388763766289, 17.769388820328803, 1.8226484836572026, 11.322468428510192, 3.3103343359896327], [16.844274282835517, 3.7916759506779445, 11.702642467372833, 1.4230334741750148, 2.0424356911939703], [2.867047751440007, 3.7722578762853298, 1.0599735825089505, 4.4987655722313589, 14.316540487121546]], [[8.197314938512168, 214.80294692519939, 9.3609200213985968, 25.775721318714645, 199.07368890677677], [25.617824938826946, 13.508949506911188, 249.57059519695233, 818.17785883833574, 38.915888403323997], [322.05243391979542, 64.589179162862308, 21.154882450890522, 92.331750782264876, 144.31015060268822], [6.9945465891306684, 330.31104585162757, 328.54279149122681, 438.01624296966861, 96.292669457585646]], [[0.36932672972247482, 0.94559220526875654, 0.41173286390050928, 0.55064334738242249, 0.33692784842070495], [0.26846255916647638, 0.27734822638584911, 0.86468076952884587, 0.30871736240829573, 0.49731042267183251], [0.2162291694941583, 0.26262241184676366, 0.86252744821690097, 0.23365326338284648, 0.31377182666521797], [0.77647537279793832, 0.86403218065287701, 0.96784157890798961, 0.22960939011871626, 0.19872996077864516]], [[8.2619035660763327, 2.0347971645796403, 2.8853991329124042, 15.01034534167907, 1.2079170521652471], [11.860328989092677, 2.4474919042004366, 7.6221064468426061, 17.89753133064066, 2.0837810349100865], [3.9714688286977475, 6.3259760609392393, 3.206496854921824, 9.5513942388588848, 4.875170357327109], [1.5683267223756412, 4.2201180447675464, 1.1392323235721173, 1.0598776873667832, 1.5502352944995159]]], [[[1.7238718175833585, 9.3699337533133242, 2.2484710699092036, 1.937375348728968, 11.419395854379083], [9.0110067501995683, 2.8260807143968409, 5.7745458572564177, 1.2375261605826104, 2.7676074664483461], [12.112858610445684, 1.2276765127901996, 4.5939364117911499, 1.319618115792937, 3.9795631924458563], [10.052589654558087, 11.936929583653992, 10.037685042450951, 1.1710629887454558, 3.675895179608339]], [[23.309755216953281, 1.7697886827352907, 1.7492955267194357, 10.43428276350889, 11.622402086220038], [3.4780961125506171, 1.6062384705059209, 30.073495967356632, 126.48194885834401, 50.918165686430996], [1.1511409234347099, 24.268782785763516, 11.98270533109396, 57.455451684478774, 159.72477076859764], [102.03051578951519, 3.5642197416001808, 13.078089612742531, 5.1337965801862113, 12.378829427363703]], [[20.977605395530485, 2.3805394841426875, 34.405075710860544, 5.5081611112182509, 2.4863762950063868], [22.910575482161214, 71.876857965038027, 22.325228396358256, 1.8063627665644066, 1.6548152098778603], [18.429240779011096, 17.762335756772881, 2.5901944330062325, 28.462605253693877, 19.110761218170687], [1.2173369342526723, 5.6516445565443876, 27.08705560317857, 2.1469244249901651, 2.3961394705996977]], [[2397.2871627027021, 853.85582268878102, 103.39332796626879, 10.338092405726318, 23.300780130289041], [5.2544862830768659, 1777.5141461250735, 7.8270331261201473, 1008.3882074461916, 24.213619637125461], [271.35256418106917, 28.281250798339357, 75.977484874362148, 139.01468222718694, 1665.3756413741703], [46.753403078022842, 812.40842311622271, 21.757875630886357, 154.27795628122951, 227.01439846024613]], [[59.62162305785364, 6.979070252592555, 21.143487526771331, 479.46057217899602, 2.542626900331785], [1.9038960647297256, 1.4553038004116219, 3.9418771224080045, 13.449976541078197, 76.750881007210239], [9.2828970362931855, 333.48768723374349, 188.63956691375282, 101.25457562553866, 2.0329123388103167], [8.1663445094888942, 20.175447035198665, 1.0317035415030475, 4.431551180192808, 3.2726205995835622]]], [[[0.65661462612628618, 0.49255464113975622, 0.46820005268832893, 0.41574144807134256, 0.87783222175277453], [0.57589729574611959, 0.67670278286786922, 0.32816322507438128, 0.97949459237035674, 0.35747481985080104], [0.23792870971631824, 0.17977415873155625, 0.68518212955665758, 0.4113322724866551, 0.22606878029009803], [0.63207723901410384, 0.21240406394071656, 0.20488826454189393, 0.73610058638105746, 0.79935846710675185]], [[7.8209952770248528, 8.1157560511457731, 1.2938708998285839, 118.76435978450374, 16.680780156393567], [128.21234846228776, 36.076522818775132, 11.010542782234802, 1.1355219366479619, 21.825104662229499], [46.810619671713226, 172.65647217274886, 74.808784067928414, 1.2480112270512858, 1.7355145111301276], [5.3959963858480915, 42.270970927228397, 1.9015072308870122, 56.568218097694462, 138.82271484474907]], [[12.156613305523887, 3.0480988472684296, 7.1858022748042263, 23.259177001038552, 1916.0236386985764], [4.2714257260984168, 1.883955805309578, 15.823434751986117, 77.678020994088769, 4.5980411428620167], [29.472401385919653, 21.929445531227451, 50.023185613385799, 235.79447442772226, 441.01801632289369], [10.200266677025056, 5.9775494404570715, 135.45944947123124, 9.6018295952261496, 1010.5811490047298]], [[2.9327226530783292, 2.132361590518228, 2.8112387707058102, 1.0357046488936705, 3.0210617842356551], [1.0848944929387154, 1.693711545384927, 2.4283944567932512, 1.6802411399885246, 1.5961011968534027], [2.9085552233029386, 2.8617090322994945, 1.5394354548025104, 1.4241040897091959, 1.252926261669391], [1.5692140968537547, 1.7911705689957993, 1.7617158433601514, 1.1331434823945259, 1.0519096481003447]], [[4.1003375407208411, 1.2901035069269386, 4.6459366682945324, 1.7950716287835891, 1.7673875297312558], [6.4290239065850265, 2.7917759193198268, 1.9626305201158889, 1.0423221904200084, 1.9325868087949878], [1.5383893427846766, 1.218184546354945, 3.6857485866678381, 1.2620797801857651, 6.354133834754947], [3.5849795845964523, 1.8361672952561965, 3.2135452353045766, 2.8747678270966897, 1.3421634870870478]]], [[[0.076852764663298365, 0.12534016197561895, 0.6397766855532373, 0.75477648598899905, 0.047871990644445202], [0.12181504751028428, 0.25679820510787593, 0.20841799402564889, 0.41427415738709511, 0.51313167174815666], [0.0859031748667516, 0.068304731488859224, 0.93031539343331326, 0.073427605249796024, 0.24904119296964103], [0.19431173470364108, 0.058719780981140272, 0.36019222580652299, 0.15157075901951031, 0.046932940903316886]], [[0.031390796059589714, 0.00052587744287338099, 0.14318819011231756, 0.030648980856396269, 0.0007769100467454812], [0.047802747382228797, 0.0022341187628117671, 0.016928676770073717, 0.006479336081521252, 0.0024245472544313956], [0.46979371523915503, 0.00071072074361414083, 0.34612503860855803, 0.024099917721017165, 0.06022030677418768], [0.00040633883346519498, 0.066123587171852169, 0.00044390882049171111, 0.011724611997527071, 0.1375768509494919]], [[0.0049332426036214454, 0.0047138628552637497, 0.3309625622077883, 0.049874449553864129, 0.025712908401350244], [0.0069939938740786354, 0.31472845984153708, 0.014949664821279234, 0.19939848896155909, 0.0063837485870062039], [0.38588292976251892, 0.019737750587400855, 0.013935829440155178, 0.018081753247630349, 0.003882778522759795], [0.0016375235652006015, 0.15282113496321728, 0.02067042247621783, 0.24761629261964799, 0.0079459197860005919]], [[3.6834664723434565, 1.7374862361817471, 42.624675552375372, 2.5366516964026387, 21.726210551589581], [12.021398571520585, 1.3471458167598567, 5.7566138917113241, 11.058405881505585, 5.8108198633922061], [4.7777462475554886, 22.563966127401383, 5.1204110868848574, 53.290708334277113, 3.011770934914265], [297.35839441351811, 285.17747049965266, 16.202220337437002, 65.514194811665973, 2.0029385220509308]], [[1.6384328982677507, 20.419810219789714, 11.964984589740464, 8.0314156692741427, 46.906148794312678], [1.0867872024632277, 8.4256320324813796, 2.1996023383843735, 4.0972946653902076, 2.1628951756733579], [80.346272034639668, 89.798984192402415, 121.97625086305433, 5.3250679109507963, 5.0341452739470176], [7.757660263185052, 31.205138227757796, 5.2676690738997394, 43.173499871257953, 90.60030148715947]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[2.849243303068957, 1.9892325346862623], [0.31423724189065921, 4.0732489434328381]], [[3.7554459843332877, 4.0258892139369911], [4.1935340427457914, 1.1083536268316869]], [[1.8017450341819086, 1.4378351239989688], [4.5403318726499435, 2.0060972507777968]], [[0.23134834317401881, 1.8808580643265231], [3.2281732733681414, 2.9403294327339435]], [[4.8131091827106305, 3.3522462608917909], [0.27709794864974568, 2.0930177778830403]], [[4.7612636048956247, 4.6664827629643568], [2.2708597771004393, 0.48540238419224824]]]),self.functionspace)
      arg1=numpy.array([[[3.8598343386061651, 2.7919334928973889], [3.9480276842915214, 1.6813713005554454]], [[2.7506083011982851, 3.1088827436997186], [3.9112159805990041, 2.1188827372240584]], [[4.42368935721138, 3.117723115919977], [1.5648872193273866, 1.7152333553080619]], [[0.11921018662921293, 1.951235203079845], [3.5354059615391882, 4.9939742352812386]], [[2.8163913946452976, 2.8741437359250903], [0.54988824263645286, 0.021020867593693673]], [[1.7433075432750091, 3.3814803000169533], [1.4840964172979563, 4.2508681298152666]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[56.908963860017934, 6.8219762190753999], [0.01035521930546148, 10.605648133200322]], [[38.07752804637888, 75.935432934182046], [272.29867092576984, 1.2435641045350931]], [[13.524084614377733, 3.1023677052009386], [10.67254475945831, 3.3006859600815086]], [[0.83987388615504521, 3.4303081191768325], [63.004132733466257, 218.35216988960477]], [[83.556051141379413, 32.351239846012085], [0.49375398155147798, 1.015647315658101]], [[15.187219503623009, 182.88385869703643], [3.3776976728877681, 0.046308487082536785]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[4.9175256468015354, 1.5684656989904728], [2.0026144705382878, 1.2956707247324684]], [[1.0066826442974905, 4.422578690493344], [4.7134060040729535, 3.0827937563800414]]], [[[4.9553271416217575, 3.7911593935663301], [0.21278395786678053, 3.279117898016696]], [[4.8866774145213006, 4.5896093474479471], [3.9163361647306498, 3.8299500836607114]]], [[[2.6268532105398337, 3.2913820314323101], [3.9592261099038697, 4.8021038682943322]], [[1.7136461891773151, 2.3610883189429801], [0.48861131546838366, 0.92125183796895249]]], [[[0.69149994729552855, 1.8981494485552992], [2.215265168082075, 0.75327394688362381]], [[2.7148102458839305, 2.0485677866214167], [4.1614953403485577, 4.0990810908533764]]], [[[4.9990618008898364, 2.5784297838364023], [0.73349265343387038, 0.96239390564101412]], [[1.3344474239751336, 1.2560527038185976], [1.2356810251659511, 0.55227522161941689]]], [[[3.4178596376943653, 0.5156316055846264], [0.22956027812055568, 4.4065510267410621]], [[3.4264632175444585, 4.2089755206519648], [1.7839172113828115, 4.5177702001625555]]]]),self.functionspace)
      arg1=numpy.array([[[3.7892183843287843, 0.45448830899214909], [3.3767514335421405, 1.6672435353272606]], [[2.8466891573924538, 4.7621480170533692], [0.32762090202685229, 1.9774055194910045]], [[2.9568358675093327, 0.80882250837200831], [3.0687574494966854, 1.7005463168042829]], [[4.3511501508242185, 1.5906320404087193], [4.127318885117039, 1.9810950771136029]], [[0.035018235663849109, 3.6697447289500751], [2.0863152787394181, 3.9131927554497934]], [[0.29893381784668716, 1.7037717820032052], [1.3401843772594808, 4.3303692311351103]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[418.00267532180209, 5.5042425117126514], [1.371110540715295, 1.1249351873767852]], [[1.022745383403187, 151.45579789233381], [13.262126831102675, 6.5341561371729009]]], [[[95.203928286003958, 44.420567481214903], [0.00063029735235329144, 285.83231423495698]], [[1.6816487171813088, 1.6474474057838793], [14.87181969238279, 14.230143581331918]]], [[[17.386126378739203, 33.869025138151144], [3.0434114191895447, 3.55760048094682]], [[5.2221254530837813, 13.96339241860672], [0.29584760653867748, 0.86980873303646533]]], [[[0.20086838839635537, 16.257635241257535], [3.5436042676116406, 0.63720241771427955]], [[61.685223879103567, 19.29544801942847], [16.85745321267672, 16.360261029745327]]], [[[1.0579711923596602, 1.0337248038987181], [0.32065412445762848, 0.86877889271766318]], [[1.8256535518329273, 1.6090206459205141], [2.2890083229300586, 0.097950126203607144]]], [[[1.4439667330079473, 0.82036770865963204], [0.081491798683657191, 12.514035026892902]], [[5.2094557421446535, 6.8629598758671735], [12.261546442332401, 685.58819207863564]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_array_rank4(self):
      arg0=Data(numpy.array([[[3.0338960613775465, 3.5573810786193047], [1.0557848100409124, 3.1567368745511777]], [[2.8825419605960172, 4.9137017599314534], [1.0638997106844128, 2.8584741264998805]], [[4.0302117093556156, 0.990390866554127], [1.459179736773913, 3.1197815657639043]], [[4.5244762030603694, 3.8969093651530962], [3.1196137993144144, 2.7451917133475185]], [[4.3926994777409876, 4.7506616311671452], [4.0907434227508652, 1.7082865997990251]], [[3.4240082207114253, 3.1411050557772171], [3.7975366681398226, 0.2467589784824154]]]),self.functionspace)
      arg1=numpy.array([[[[2.6662528920862223, 2.3405702166218876], [2.4234158288765975, 3.1678380085956244]], [[3.0744756361235637, 1.8979304825486289], [1.1567114474990641, 4.4439801776171572]]], [[[1.8891260400748402, 1.8358589267378473], [0.58501320317584027, 0.012412405801217691]], [[4.2019618586778842, 2.5128735601526802], [0.63049831323613437, 4.8030594832603386]]], [[[3.1025917108191781, 4.0321598728686601], [2.1264174317342617, 0.68869875519853574]], [[0.33942171038304736, 0.75848263455050535], [0.62789694475310476, 0.76386164779289201]]], [[[2.6652840966385383, 3.6245630611857171], [0.66049325042630369, 3.4076441344805271]], [[1.7856697551908409, 3.716693745362559], [0.055000053413959291, 2.5868934719246508]]], [[[3.5474359102341717, 1.1394294601058281], [0.067763597309808424, 3.740577433097366]], [[4.223846526389563, 2.4579956030120784], [4.1984143973626313, 2.0300208317218771]]], [[[4.3047402979680731, 1.4299209022925692], [4.1608749068813875, 0.22542931791431703]], [[2.3764623289540996, 0.88559183399034314], [4.8110112777205583, 1.235728722645751]]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[19.281275294385296, 13.432488533096498], [21.657988199480773, 55.704694855135479]], [[1.1816313960007812, 1.1085224375115574], [3.7798516319191711, 165.42665174434893]]], [[[7.3888037260954267, 6.9836619566305993], [2.5379512328702614, 1.0199574312676252]], [[1.2972858367706455, 1.1684174505698937], [1.9390628461271535, 155.18152297388423]]], [[[75.524038030269423, 275.91715856908303], [0.97967750874913073, 0.99337226192529748]], [[1.1368471731628473, 1.3319062825269243], [2.0429562566357125, 2.384748125758517]]], [[[55.882494812914459, 237.76541031223238], [2.4556517347739439, 103.03020941467213]], [[7.6260722066371383, 68.615380275439108], [1.0571132607551457, 13.631410640527548]]], [[[190.567473812331, 5.3994210761553862], [1.1113715938354016, 339.97823042607507]], [[383.84754700691883, 31.901214537905403], [9.470792786504644, 2.9655355530807102]]], [[[200.00160858727557, 5.8122258501681356], [117.03019861394021, 1.2943655941032859]], [[23.832226378740074, 3.2598845356356208], [0.0011918411463106131, 0.17742476788108635]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[4.744791861006056, 1.3653271551208723, 4.6585899814536642, 0.64955682029208772], [3.4159773335322576, 3.7685832200756124, 1.3979093561264471, 3.5302740985981034], [4.5964394085344038, 4.2511513708170838, 3.4983174331038547, 0.11992942885504757]], [[3.2088416231193717, 2.8631085935288549, 0.76446657167721555, 1.140538484950044], [4.9295159009518779, 0.79643251310372465, 4.1537338038156157, 4.8740881937344556], [1.0108227786146331, 2.939019793082525, 2.0613247829700407, 2.829966556517276]]], [[[0.031145423748416792, 2.2871632630886727, 4.1047450859290082, 2.7523053891576987], [4.6660615821252023, 2.5629078424975824, 0.83175792048870145, 1.8939670821710153], [0.13942391054009107, 3.1475278175836583, 2.0240200427020354, 0.25057564501456331]], [[3.2003416743221518, 4.0164271617915324, 3.1412634287880468, 1.3772730063842202], [1.2207376598702127, 3.4554159977131191, 0.039127539877066318, 3.3819541696707756], [2.050677416002046, 3.5074222169763241, 0.64957769572462642, 1.4752243128377898]]], [[[3.1431551830490609, 4.7772238218502734, 2.5269255795589527, 0.72011563010296087], [4.2623284899253386, 2.7741360095188439, 3.3028956022939404, 4.4178927648640354], [0.21418438267765616, 1.698638829656782, 3.8285264475912011, 3.6368348887961495]], [[0.1768281483912188, 1.2306450710888925, 0.89083054946710249, 4.082322586927865], [4.3562041714101865, 1.9602053938465576, 4.3392201927255254, 0.38276768060984767], [1.2981830680656774, 3.3923331330430684, 1.5181064218229952, 0.69903470213169761]]]]),self.functionspace)
      arg1=numpy.array([[[[2.7129900064423782, 3.13343631656626, 4.3721846859233473, 0.85811760038739338], [4.5725791902541424, 2.8162588698608433, 4.7415607341589299, 2.8388443668241319], [2.7794282679782554, 4.8971981227758405, 1.0585163335552918, 0.11965301026035934]], [[3.3756035224739915, 4.5976126113959204, 1.9532249060876348, 3.8404828876933959], [1.6896143189305204, 4.6780699853020611, 1.6969262611884628, 0.90077114590758567], [0.20342900117336757, 3.5276401532578534, 0.23515770902236743, 0.51131991841331381]]], [[[3.6010050649845491, 2.3568353747557289, 3.3700079886525511, 2.2658489451745663], [3.5716489005504291, 3.3830799047480538, 4.5862060263051756, 0.7047959036895346], [3.4698451999985842, 4.740067622338092, 3.4540809766558453, 0.93685547734729901]], [[3.2354138095683869, 3.4419758111883603, 1.0648846786451458, 2.8905543893960446], [4.504698393115266, 0.20850034422675889, 4.9500781710620281, 3.2509729097929241], [4.7711832612992966, 2.5429881940971462, 4.5509262463902029, 0.41718970358050828]]], [[[0.42043903697305368, 0.90786289445326152, 4.9899699887188751, 1.0359537190584409], [1.8929590513219308, 3.8214952172292143, 0.75501936152766269, 0.3297909296946499], [3.3868472104760845, 1.4184739697569548, 4.2054310891973508, 4.173013516166316]], [[1.3884636715112906, 1.8275291732609351, 3.9296804319546963, 3.9053290181407356], [0.7190152644513601, 1.666599927500183, 1.5672952726528675, 3.2369619199742981], [1.6196973592386423, 3.0354538267318603, 0.052796894861964351, 4.9747996599031055]]]])
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[68.323548438257305, 2.6531125561310631, 835.08761033941653, 0.69056327130315021], [275.13098094843627, 41.943697549578289, 4.895505795719, 35.904002473101983], [69.367038299425445, 1196.5233843672663, 3.7642968256659932, 0.77587310051650915]], [[51.195412392558474, 125.99790523083369, 0.59179721374620808, 1.6570276803499091], [14.810578348457092, 0.34480060207222285, 11.205849995962042, 4.1651870654570207], [1.0021922376834025, 44.838386604813252, 1.1854246492393894, 1.7021772395574031]]], [[[3.7558223485996102e-06, 7.0275553113308362, 116.62169924725086, 9.9148494415404596], [245.05084101392521, 24.142173403690773, 0.4296249630643949, 1.5685232119085084], [0.0010739467483089631, 229.30285871505814, 11.420647987019818, 0.2734592232845936]], [[43.104165069473815, 119.78496596844101, 3.3834426128018729, 2.5225801523620248], [2.4558821832891709, 1.2950230730098853, 1.07814844130968e-07, 52.5182100589466], [30.769427005999592, 24.316338111652417, 0.14037765226527171, 1.1761043181847743]]], [[[1.6184974065097917, 4.1361693904672503, 102.07614301600678, 0.71166452466838181], [15.555924641036095, 49.364026431271697, 2.4647657808510943, 1.6322478709237536], [0.0054135297821679817, 2.1202757502693865, 283.07364260512281, 218.72936776686166]], [[0.090209536898369719, 1.4612359616594954, 0.63490799319574776, 243.10510777527134], [2.8809025668209065, 3.0700807027823651, 9.9772838218883493, 0.044666026598874475], [1.5260540767322526, 40.766524159653564, 1.0222854841435096, 0.16842725257784424]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_constData_rank0(self):
      arg0=Data(0.293675555215,self.functionspace)
      arg1=Data(3.98207492869,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(0.00760343261918,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([0.55948154356542468, 1.1076361941239388]),self.functionspace)
      arg1=Data(3.71804786872,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([0.11541317773074694, 1.4624151817254398]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_constData_rank1(self):
      arg0=Data(2.11276520868,self.functionspace)
      arg1=Data(numpy.array([3.0794310942707512, 2.1704549701433837]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([10.008223588443558, 5.0707824800081234]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[1.8195259564382267, 4.4491017423412691, 3.9739223389200613, 4.7590784426489376, 3.5606947363234691], [4.7736924087811525, 4.0437831091524234, 1.0816965965809766, 4.4004542276087379, 4.2863088080409888], [3.7755013440969409, 2.5459680423465292, 3.2162879374505966, 4.766042196546552, 0.82408270069493406], [1.6046787350017109, 1.5047268402511718, 2.86496532646939, 4.3359225722592649, 4.8304386942605717]]),self.functionspace)
      arg1=Data(0.186997528605,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[1.1184370653640694, 1.3219813396815554, 1.2943524268457991, 1.3387364895754845, 1.26804782596236], [1.3395042659650112, 1.2985773664382869, 1.0147934078069534, 1.3192662237635033, 1.312798416716487], [1.2820141826927041, 1.1909499163456281, 1.2441539322670165, 1.339102584194465, 0.96446561043436951], [1.0924638291152822, 1.0794043036425027, 1.2175314618875381, 1.3156266719652572, 1.342467557044102]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_constData_rank2(self):
      arg0=Data(0.609667424527,self.functionspace)
      arg1=Data(numpy.array([[2.5335143673575042, 2.7778246768746735, 3.5863597617804643, 0.88309772931924346, 2.4357239561370787], [0.23061171162181829, 0.75040282954964987, 3.1309299965845745, 2.2168161894229375, 0.71897411107163445], [1.3036115221093505, 4.223212286656933, 3.8414830842977001, 3.7111733567261336, 4.0791974938639264], [4.2327772873727403, 1.9016363797854938, 0.55149881443804327, 2.4366706210354403, 4.5554758453634676]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[0.28544997156434659, 0.25294494585248983, 0.16953764610099401, 0.64597559108348301, 0.29960278693820058], [0.89215420487098773, 0.68981585488275776, 0.21239348835720445, 0.33388019799942137, 0.70062789421601346], [0.5246198831633182, 0.12370922251992898, 0.1494301934743589, 0.15938330661896821, 0.13284703500867504], [0.1231250703877672, 0.39023391653438638, 0.76116540105545927, 0.29946247110682017, 0.10495335106116277]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[1.8935888770525993, 0.18438811243311626], [1.8547039515616122, 2.5848316147999353]], [[0.94126945069257772, 3.015758529806619], [2.562484848296493, 0.75489810244914246]], [[3.8221117277459573, 3.2200898300801351], [1.0910671676078743, 0.57967996760507545]], [[1.5447406003957114, 2.1582595428064288], [4.0179920217874621, 1.4826814281266669]], [[2.2770935697513841, 2.9939917848257815], [0.63632214564112588, 0.35164472545501235]], [[4.570405846008696, 2.9370373825018459], [4.9804331307989234, 4.5362509955373191]]]),self.functionspace)
      arg1=Data(2.97581799576,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[6.6857750133095006, 0.0065306258820274389], [6.2854499761263796, 16.87809130598729]], [[0.83517510460818822, 26.705267024777505], [16.447567180735266, 0.43312964149142835]], [[54.054128753955894, 32.458070259666322], [1.2961018685334218, 0.19737468912276102]], [[3.6475376028496886, 9.8680584482476146], [62.722167845941676, 3.2285481163556717]], [[11.574452175959907, 26.135759126955307], [0.26048252727494969, 0.044595232758332298]], [[92.024882313144971, 24.683880420651732], [118.83381071224272, 89.993465908643046]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_constData_rank3(self):
      arg0=Data(2.88929313553,self.functionspace)
      arg1=Data(numpy.array([[[0.7138195935681525, 2.1888794766872186], [3.8551975335390543, 0.7011836940397701]], [[3.0722691140942717, 2.4307714360526762], [1.5908905420744268, 4.026147883097063]], [[3.6980740049163114, 4.2045543797791689], [3.5283369358563621, 0.39172115743668307]], [[3.8647742381741987, 2.8069235335206617], [4.568533402602954, 2.5946111029795316]], [[1.4150014256872985, 2.5016058471797757], [4.2558660695179489, 0.12902099051892016]], [[2.1211617242175387, 4.2527438382214724], [3.7387687961087397, 2.9051956349622898]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[2.1326622260846451, 10.200402027366691], [59.764413519985311, 2.1042607780576676]], [[26.042088979275626, 13.184967793115796], [5.40840253262422, 71.649827330285433]], [[50.587295176735878, 86.581152840605796], [42.250129299529092, 1.5153111579186722]], [[60.374775261717431, 19.652002118823809], [127.39190809932262, 15.688268606570597]], [[4.4876686097076446, 14.214087197425428], [91.425503362952156, 1.1467052197452003]], [[9.4932193766525366, 91.123136962367781], [52.819377332034179, 21.811712353970428]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[0.92698356931466819, 3.3560057717442713, 1.9437512441816884, 2.7074641611594377], [4.7918184020046182, 2.0883646899219737, 4.0076570013831647, 2.9825392298921667], [0.43003583975643794, 0.21510300237283203, 2.1239913984545034, 1.3530404565580079]], [[0.57266189942725387, 2.8734248564703924, 0.29026428094187029, 4.0150254515816188], [1.4412530556648915, 2.9820044900851612, 3.5227651571825076, 2.0302216418533376], [1.6323401566599611, 4.1165248635630922, 0.12221566582228714, 0.45976208558995946]]], [[[2.3904665196451016, 2.8785492581250667, 0.65238629407251425, 3.1365824615863516], [4.0934613915076152, 4.1540461566434956, 3.1856757201568211, 4.6681794039943227], [2.6795108137471937, 0.86351908518764076, 3.091560533229472, 2.0176767172632708]], [[2.133737848573054, 1.5346263877690642, 0.61971927891527145, 0.19311621796081924], [0.22943675010315151, 2.7694264398998545, 1.3234574620259671, 2.6276569157200558], [3.5075449769651614, 4.0719863242411876, 4.4211979890548232, 4.0620292327269691]]], [[[2.9878940297012972, 3.6824099219922011, 4.9621333295186858, 4.7471726063255231], [0.50550293619256015, 2.1471592402871011, 2.2165357981439384, 1.9279731502851958], [0.71715073832282306, 1.9157230529373692, 0.027976238861695785, 2.5303672968534978]], [[3.0037613984531659, 3.791591111663148, 1.5950553817293485, 4.9985907639749723], [2.3457064933262428, 4.1451191861227752, 4.6189776391770216, 0.67031831933065489], [2.3464808129811892, 0.01210521163062484, 4.7298243083107083, 4.3248243370403534]]]]),self.functionspace)
      arg1=Data(0.104546026231,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.99210471184759286, 1.1349394017276393, 1.0719542156577564, 1.1097437636040819], [1.1779954352241178, 1.0800266753636962, 1.1561916024797769, 1.1210270642700921], [0.91555485028055339, 0.85159056392863486, 1.0819383635800162, 1.0321148352785599]], [[0.9433855682863036, 1.116667514728773, 0.87869307799414365, 1.1564136601148935], [1.038952425743453, 1.1210060499742227, 1.140708077437127, 1.076843140410874], [1.0525639932689463, 1.159435909920504, 0.80271752133708751, 0.92197507912771026]]], [[[1.095390218944011, 1.1168755458037083, 0.95632876058760863, 1.1269446241715677], [1.1587550776521913, 1.1605362718494643, 1.1287758887075507, 1.1747804647127484], [1.1085403430713532, 0.98477606526551642, 1.1252425244306163, 1.0761455682929753]], [[1.0824563437898946, 1.0457932614575232, 0.95120650963147346, 0.84204478022828166], [0.85735336232498816, 1.1123721298305156, 1.0297322048168354, 1.1062778988179736], [1.1401918278239414, 1.158118039707664, 1.1681231690296892, 1.1578216504286913]]], [[[1.1212373122606498, 1.146005929571982, 1.1823045742987421, 1.1768431766650038], [0.93116252292972823, 1.0831661739693086, 1.0867732033758917, 1.0710411936415394], [0.96583879614397983, 1.0703276986005761, 0.68804682680236706, 1.1019229788192335]], [[1.1218583448826991, 1.1495119418651203, 1.050024422967949, 1.1832097444463778], [1.0932277342077832, 1.1602752858214322, 1.1734798289992812, 0.95904365879895281], [1.0932654567198554, 0.63035090519902626, 1.176392816855202, 1.1654347853981735]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_constData_rank4(self):
      arg0=Data(2.39950411076,self.functionspace)
      arg1=Data(numpy.array([[[[3.3180113581161255, 2.4107304700869183, 4.6032194705565397, 4.3885683594399474], [2.9075960915132018, 0.65966877009958635, 4.7286894955142129, 4.5009941347070059], [2.3470433958092451, 4.6010293194580552, 4.7431970099632377, 0.63842281087055497]], [[1.8873764944919416, 0.052813475443539891, 3.3686080408465027, 0.69036003290433501], [0.37692837830475734, 1.6743164741100409, 1.6898052023245, 1.4984645690095573], [3.5590803105525244, 2.125020882531989, 4.5388994730755972, 4.28267885293916]]], [[[3.5879409971526788, 1.9042748955395441, 0.53714749845124743, 1.4312398171116982], [2.1310531584825543, 3.0755149870502434, 1.2421686046125124, 4.6114333844478956], [1.9702236497233767, 1.273156737566872, 0.75289964046730729, 3.7036609234130631]], [[0.48082426813480666, 3.6338894558937938, 2.609339974351486, 2.5171513566988559], [2.9853902393145848, 4.2327851044882516, 4.1051110701371627, 2.1587792706343438], [0.91814564043404989, 1.4392429955265686, 0.35216441992590847, 4.9270058777481527]]], [[[1.3278646810904076, 1.166074441036397, 4.2501477550307181, 3.2427551173234681], [2.7932213453081118, 0.95563862219683093, 4.5381896673625883, 0.62454123201764444], [3.0237649269565439, 1.4247682182440276, 4.142869481080754, 1.6179208869055126]], [[0.62541428228243023, 0.89275078705877353, 0.85962274700155206, 2.0224633154195839], [4.2831640521203038, 1.5721598261855847, 4.1554870186445996, 1.0219434165234964], [2.4971252241726187, 1.9338569621863486, 2.3120377750598009, 2.3465942448851926]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[18.249352029366023, 8.2484145234759332, 56.206000394788525, 46.578894856956403], [12.742064018472469, 1.7813706036327885, 62.730180326901291, 51.395445424833895], [7.8012043125136259, 56.098359156346667, 63.531799129940985, 1.7485507242462999]], [[5.2171386775479238, 1.0473106923484743, 19.075693494508169, 1.8298719084866715], [1.3908445078314207, 4.3295592441318762, 4.3886534215836521, 3.7119199098685227], [22.536316859525211, 6.4234170738654885, 53.129198994303806, 42.455924496333914]]], [[[23.112850363706862, 5.2948763970907793, 1.6002259612900906, 3.4998144803322226], [6.4574212648415887, 14.759422498228204, 2.9660262554652927, 56.611539769178748], [5.6095027555787267, 3.0475739077515747, 1.9328285829395451, 25.576501525884851]], [[1.5232515938423892, 24.061323081127078, 9.8144557347018591, 9.0536425742493982], [13.639894485416685, 40.641770196449293, 36.34469014285753, 6.6160443960908228], [2.2336076916909873, 3.5244163221357478, 1.3610223811095492, 74.62096418967316]]], [[[3.1970535604868919, 2.7749167833217965, 41.264114757948938, 17.086020920346879], [11.528252571553827, 2.3081221295451644, 53.096201878516631, 1.7274343403458263], [14.105810803772854, 3.480046362604297, 37.565896218550286, 4.1210369797540478]], [[1.7287548599757718, 2.1845086401959146, 2.122076747078558, 5.8719423853325239], [42.47395835672301, 3.95923927250452, 37.983060184344531, 2.4460349754857145], [8.8963319951045747, 5.4337619204464769, 7.5658070818344525, 7.7981380685402497]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([3.2510009615350728, 0.71913105661466536]),self.functionspace)
      arg1=Data(numpy.array([4.8137680004675483, 4.0510208749758965]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([291.56240356518686, 0.26298222779742175]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[3.4076839962427434, 3.5971236339032049], [1.7162398254538058, 2.8550103037862931]]),self.functionspace)
      arg1=Data(numpy.array([2.9519784653147991, 4.0114060733787316]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[37.308555159056048, 43.769165329278742], [8.7294627464744661, 67.239955055467348]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_constData_rank2(self):
      arg0=Data(numpy.array([2.352169893984148, 3.012365479907726]),self.functionspace)
      arg1=Data(numpy.array([[2.0154551665379214, 1.2049763329716183], [2.3178550382363534, 3.2795911964255482]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[5.6063277647097989, 2.8029217674728075], [12.883638710839234, 37.206717153983874]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_constData_rank1(self):
      arg0=Data(numpy.array([[[2.5087915717556104, 4.2491056751078835, 0.46366947846480216, 0.57951561196621504, 3.6044773423310836], [0.42859653892233429, 1.7119729808113417, 0.64955546353616311, 2.6797571464405987, 1.7165563658015104], [3.2371907251789374, 3.9230756156823658, 0.72955791501813128, 0.15651643242166899, 1.0227558915290935], [0.023942756794907021, 1.1649773167314061, 3.2248100610398995, 4.9320543051214552, 4.4023265505763742]], [[2.1428824844848671, 0.80086833181876071, 4.4508465846648066, 3.3414602902081856, 0.53900750674622244], [1.9680119777084619, 2.7170143010578638, 2.4676020725018182, 2.0280227638240924, 3.2219032795656259], [1.562260571789714, 1.0658368903620834, 4.1758619915017352, 3.7302312293375985, 0.49151097283981021], [2.4474308285807567, 3.2352180605672767, 0.013327310613502483, 3.3767419152381759, 0.30863771179641614]]]),self.functionspace)
      arg1=Data(numpy.array([4.4704788258852863, 2.8195329172003509]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[61.065786575850602, 643.85906364384107, 0.032195280036442959, 0.087254478303517444, 308.56858648917932], [0.022650684138087546, 11.062237412312884, 0.14531313382123404, 81.995796715365017, 11.195253386662666], [190.85191702390156, 450.60447373606155, 0.24423709011265113, 0.00025078252289496291, 1.1058226317709443], [5.677176225525421e-08, 1.9791141427708479, 187.61043532526242, 1253.6212013369216, 754.34130208482657]], [[8.5755257928631856, 0.53467195670321088, 67.344995139780863, 30.009285592001035, 0.1750744384024214], [6.7456046448058533, 16.746978770094113, 12.765347193168045, 7.3417896852928077, 27.079454800554061], [3.5179875058481769, 1.196947149880472, 56.261870534269136, 40.928683679911053, 0.13497948091265055], [12.473313619280413, 27.396170942908164, 5.1600012422310128e-06, 30.911290568626097, 0.036348343435984788]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_constData_rank3(self):
      arg0=Data(numpy.array([2.4213573380709961, 2.7822673365970148]),self.functionspace)
      arg1=Data(numpy.array([[[2.4415682966470955, 2.5998160356226943, 0.49261948712494003, 0.12476109750987099, 0.94766127570507197], [0.75638733386921775, 1.2245865285188027, 0.99606093730532486, 0.30566850745934365, 4.6657535054367925], [2.6492189254189626, 3.0862329196319553, 0.99780063834692501, 3.159815283189336, 3.3496611747708132], [0.62594686293449386, 2.0909268126240188, 0.070139560201646772, 4.5212012954509531, 2.5570460142488707]], [[2.720299101930475, 0.11709782434825737, 2.2531522629488085, 3.0804452708833936, 0.19620433558867736], [2.5381794176696819, 4.1891358463618076, 3.6402799588344736, 0.44726497714378893, 4.2668222884497782], [4.7391625948897955, 1.111477489673625, 2.283832975808382, 4.0235246940220879, 2.9998776891914356], [1.0098922880550221, 0.024993474240958859, 0.21177736534877045, 0.28202934483781061, 2.162397059440559]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[8.6637517752093203, 9.9651181274170799, 1.5459480350128849, 1.1166462406866227, 2.3118397892675948], [1.9520784948581285, 2.9533392574687856, 2.4129373962645366, 1.3103723078556424, 61.933282012504705], [10.410128013488114, 15.321284533404945, 2.4166524773859304, 16.351407375160143, 19.340475185705195], [1.7394039623651705, 6.3538802129669305, 1.0639904291098334, 54.501384433667795, 9.5952488928222124]], [[16.176993545527736, 1.1272964499198506, 10.029938978598038, 23.385483750942385, 1.2223426980804368], [13.42653760789339, 72.718880256834424, 41.470211824776534, 1.5803891693051566, 78.73556512355043], [127.66693774045078, 3.1184538854298278, 10.349819231847377, 61.383235466141251, 21.534868140821132], [2.81057370459714, 1.0259048226644274, 1.2419771883776256, 1.3345459097306041, 9.1404332629874698]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_constData_rank1(self):
      arg0=Data(numpy.array([[[[2.1009332652920798, 4.0299967857694066], [3.8986272286918728, 2.563942077602182]], [[4.176152339805693, 4.9549564913304591], [2.3860650239494858, 0.41627214317441247]], [[4.8543005272150141, 2.5771793413108379], [0.55069599448462936, 2.7242539381762185]], [[0.85820331387739024, 2.4248111257190974], [4.1723003888642189, 4.3641387346818838]], [[1.2495160307196007, 3.8624494374962226], [3.0732503730587895, 2.5751068758705871]], [[2.9527031124645924, 4.4086215477331132], [3.2991610745240898, 2.1572604701135627]]], [[[4.5371936122527643, 3.1385542214898829], [0.17108997465875941, 4.9929468902355261]], [[2.850134289597797, 1.9086796353960038], [3.4761509584382435, 1.0860052175697383]], [[1.1532618563260899, 3.7851839842134347], [1.934419194391318, 4.1221217001085675]], [[2.0501345003090048, 1.9311847431150848], [4.7693151932055455, 1.4510566704825552]], [[3.6544657745240428, 3.6319927401982084], [3.590539035590921, 1.6076051377238998]], [[0.18215479952641309, 0.26701457167139159], [0.35258390593499456, 0.9207378463941176]]]]),self.functionspace)
      arg1=Data(numpy.array([1.0656199923665313, 1.0547579731653816]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[2.2058142978123567, 4.4159565292064071], [4.2627251730471878, 2.7273493026167794]], [[4.5868197191482949, 5.503618916722484], [2.526188668154802, 0.39300754992522013]], [[5.384560809547879, 2.7423567412649787], [0.52955437409673944, 2.9094341406807152]], [[0.84963496337384281, 2.5699251701900643], [4.5823114971034515, 4.8071612041768699]], [[1.267914681266731, 4.2205858651583705], [3.3082179695231826, 2.7400067973163602]], [[3.1701192573793149, 4.8593923433303674], [3.5679696751142269, 2.2688891097907176]]], [[[4.9289186913808924, 3.3414082762831221], [0.15532361620284646, 5.4525236432562707]], [[3.0183724364979265, 1.9774495490684685], [3.721586283689474, 1.0909227400719894]], [[1.162301975571947, 4.07138212839173], [2.0055870767341522, 4.4545473367907551]], [[2.1323324884669024, 2.0020501630000238], [5.1952553914530268, 1.480941429029542]], [[3.9232230025709547, 3.8977804688228859], [3.8508718825949217, 1.6499445564337121]], [[0.16593723379146144, 0.24838946922334426], [0.33302090436511161, 0.91658375272299719]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_constData_rank4(self):
      arg0=Data(numpy.array([4.0856596286253302, 3.71530783382893]),self.functionspace)
      arg1=Data(numpy.array([[[[0.28893516855833851, 2.0594811694116069], [1.5183162755287314, 2.2440513219058222]], [[2.0500697037834645, 1.4078008263067776], [1.6582049590608137, 0.31837114786991572]], [[0.51279259947605971, 4.761054267309107], [1.558352804790593, 2.7590552021698329]], [[3.4853399377383942, 1.1529379154936876], [3.278546732738016, 3.0651180253587063]], [[1.917062758588983, 2.7078919224382672], [0.12088189986194935, 1.941095297869738]], [[2.0517024557637624, 3.1193027367276289], [0.6806976510717625, 3.4073043911576244]]], [[[0.37042869058190625, 1.5087328778304654], [1.2646692750012243, 0.49901092943358633]], [[2.1295384847203334, 3.8309693550465438], [2.1218202111332722, 3.9612724264642183]], [[0.64023428006807848, 2.2005915543343102], [3.7045139037830235, 2.639162001085547]], [[4.9785303263853171, 0.88987630124539474], [0.99802473994738183, 3.6443586753995909]], [[4.0250119297981035, 1.7486657330921609], [2.6092714111181685, 4.8574494602823846]], [[1.114130808012564, 1.4848455877673725], [3.6100587943296287, 3.7755888629733558]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.5018105192022895, 18.150264403229293], [8.4740168152554034, 23.534431416139032]], [[17.911422693083487, 7.253296095426224], [10.318058445522993, 1.5653384133044814]], [[2.0580253688274972, 813.30309692675098], [8.9652444918409273, 48.585436246766847]], [[135.03815626966806, 5.0669585009990303], [100.93728844079766, 74.746478391653227]], [[14.853471499576612, 45.209731578380996], [1.1854699072198434, 15.364490020109908]], [[17.952631746088631, 80.669952474058363], [2.6066620690228577, 120.99190680123668]]], [[[1.6260819484266147, 7.2438575260492124], [5.2584156841215624, 1.9250128662703498]], [[16.361603716569583, 152.62744141374495], [16.196698348829461, 181.09426519336483]], [[2.31702229404452, 17.9608030301257], [129.28664745682968, 31.938066855513021]], [[688.2344600717181, 3.2153249259504926], [3.7056885476257917, 119.471827328701]], [[196.89554198751554, 9.924996917548162], [30.709386807257808, 587.11198996870098]], [[4.3156753862691026, 7.0202773270638916], [114.21279307513286, 141.92732952654069]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[2.1997628558828182, 0.22138893616203748, 3.6771897783152601, 1.6993491870022208, 4.3222497378853131], [0.44107187889280175, 0.68824235738961859, 4.6183244236262819, 0.31073425546569861, 0.42318019298085569], [1.9019360027745045, 1.0181138515570025, 3.9845844723592876, 4.1968811530246342, 4.9140773833312679], [0.68828915469374985, 1.0742564710834419, 2.2361464641278705, 1.3950879018728963, 4.6938796288344502]]),self.functionspace)
      arg1=Data(numpy.array([[4.0784385818286299, 3.2388581315045331, 4.0448835462534269, 1.203161371842699, 2.4332198159143617], [0.94161179924046634, 1.946849124783566, 1.6742780303554172, 1.8910579811663364, 4.9380834915022271], [3.7936884667489172, 2.3346806912856928, 2.3611613916291936, 2.7075519130135577, 3.3627715560405513], [1.279069920219353, 3.1830306171526543, 0.051582117907059827, 0.35486711106403351, 0.85998861856834841]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[24.909151565684201, 0.007569250969335182, 193.84140205130143, 1.89263628815669, 35.222633214748249], [0.46266412751334046, 0.48317781066951171, 12.957812616756275, 0.10966761987426654, 0.014313666982357242], [11.459937891193867, 1.0428022960554577, 26.157802116098637, 48.596646376529314, 211.42749315630724], [0.62015131918053135, 1.2560810044131645, 1.042384543166095, 1.1254192576000197, 3.7801569553321763]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_constData_rank2(self):
      arg0=Data(numpy.array([[[1.3920019893150863, 3.660761086592605], [3.9456359739492703, 1.4999780169367145], [0.56020878923921169, 0.17923965675863904], [1.6475859439128346, 4.2806427488722276], [4.6165023592404619, 1.8198445287680154]], [[1.619676571266814, 1.2878416239437882], [3.5687292944150188, 4.1331035696611815], [4.811827556794996, 4.3879866096798468], [2.3557229293808253, 1.3610569657825498], [2.2426951153613062, 1.5138964694249266]], [[4.0177486635849116, 1.040730934764128], [1.8836220817199625, 1.297155233199349], [3.9164656664140849, 1.44793916218878], [4.1207295964962247, 4.4256474266348649], [4.7095005696897179, 0.94620480979485433]], [[0.77752718824851919, 2.5136468603623574], [0.12560935379525914, 3.3699411818660585], [0.53418505360684732, 2.0693541388337144], [2.0718623460519909, 1.7971315199466675], [4.8967233074244101, 4.1644043088149143]]]),self.functionspace)
      arg1=Data(numpy.array([[4.5250361502617276, 3.1375580353708843, 0.038506492838833846, 0.67728891958917337, 2.3843425513805832], [3.622827217002444, 1.4786820182256479, 4.3209020118428274, 3.0790449619709355, 2.6830924808451471], [3.1386438110802652, 4.2192737189984628, 2.5001656767754348, 1.6881530226534827, 4.221671867655667], [2.2814779410129304, 3.2241411319660251, 3.4058237138733918, 4.1688934024890418, 2.9539631938502633]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[4.4665843579620272, 354.96121012628623], [74.191086138713061, 3.5684250128646227], [0.97793465830342496, 0.93594939016741885], [1.4023899816420047, 2.6774048001494215], [38.366298445756385, 4.1687976573918197]], [[5.737497981355224, 2.5004184904651181], [6.5613307973371482, 8.1522345362957864], [887.55421183521264, 595.88904229519665], [13.989016145801557, 2.5835159967709194], [8.732708947668927, 3.0423886770459334]], [[78.647834341675733, 1.1334940949259686], [14.463519966924999, 2.9973949683754544], [30.362256019075989, 2.5229138390007009], [10.918686946070631, 12.317078479129135], [693.55196701752254, 0.7918030841308199]], [[0.56320955749485679, 8.1900272760043968], [0.001244857703321806, 50.249162391296096], [0.11818600284076916, 11.903583414096122], [20.838903278345484, 11.516388427083516], [109.13288723413959, 67.629538730364217]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_constData_rank3(self):
      arg0=Data(numpy.array([[0.31548925021775232, 0.017341193347881999, 2.6888695746987534, 4.5473996367652232, 0.18984959288803568], [1.2128410505832299, 2.9785343495715062, 2.0985467249445224, 2.86884980169821, 0.10758377195262532], [2.2704874371559045, 0.51157606057816651, 4.502732473902463, 4.1891555926246618, 4.4533443166698996], [1.9970386864216696, 2.3689682875645448, 4.1743483218336674, 3.1319336348192137, 4.0529673920426488]]),self.functionspace)
      arg1=Data(numpy.array([[[1.3782901639450291, 4.253103518231887], [3.2155747904371501, 4.160452845826228], [1.5575637471304784, 1.8560967403746678], [4.6045512479713455, 1.0134069484691763], [0.56191137176998318, 1.7462798201665728]], [[3.2897553616582087, 0.98035292258703421], [3.655697211183774, 3.9763503315336699], [2.9437947874766128, 0.13982851537319285], [0.39836528976041197, 0.751925169025213], [2.0497871328932162, 4.5828295003488817]], [[4.4456198089816885, 1.6585669022689122], [1.9380097008552413, 0.6873341227305112], [2.3951112409749018, 2.2248653232125641], [2.688806063837021, 2.9662370238190907], [0.53431528911272475, 3.8320242904169519]], [[4.0038222808373849, 2.482261805603827], [0.56785050983908614, 2.7930124679534978], [3.7772744892574344, 1.497725810874172], [1.0200113471135031, 0.18276534474458564], [0.23503531227287916, 2.482142092786614]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[0.20391790247245462, 0.0073982583807140822], [2.1758390243615968e-06, 4.7181433640203072e-08], [4.6674776736635231, 6.2708005624764445], [1068.3265159956102, 4.6406810317361344], [0.39312457014166591, 0.054941608335244987]], [[1.8866628142919437, 1.2082516210099594], [54.051783692107684, 76.700897712844963], [8.864666966638806, 1.1092090542428976], [1.5217205477248017, 2.2088286331811391], [0.010358264134737916, 3.6530912395153188e-05]], [[38.297474859991723, 3.8962548681445859], [0.27281306795831972, 0.63084702135915971], [36.74075354637003, 28.437827362037407], [47.073607493180575, 70.044595474664476], [2.2212784042279776, 306.04237306855526]], [[15.947553718172312, 5.5672135614209282], [1.6319004395340695, 11.121123073405133], [220.86851393026885, 8.5010327998944764], [3.2043092826713777, 1.23201884761976], [1.3894666990283073, 32.253675952699844]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[1.0368694860996337, 2.7233614111110658, 4.5296548876917866, 1.6244568882927268, 4.1823622730645091], [2.3103376356496175, 4.6664404470918912, 1.8732102885962247, 1.3308197843185945, 2.2159584683239917], [4.4781553084710621, 3.4972170003304299, 4.824974876455169, 0.93206982156942619, 4.3966738047573548], [4.855454453718969, 0.53271097349148089, 3.7023751709080917, 1.0120364597856879, 4.976794221372475]], [[4.8173953974207731, 2.7729189371987379, 0.6534115153883342, 1.7171781261634076, 3.6939368342237722], [2.0357384055553771, 1.9076222932969591, 4.9953013134372206, 2.6947996233918472, 3.6182174336257971], [1.9043126829687551, 2.4991273519756017, 3.3940667514330785, 2.6208295323982256, 4.1309339645606649], [1.5427129473236356, 1.0524188365634453, 2.6898947178495778, 3.6372610740479896, 3.3604289090339226]], [[4.2143026530125214, 3.8428105631826472, 3.2132710374747435, 4.0095153378708748, 4.4114437557106978], [0.73510078152859271, 2.1589637163119937, 3.6227024915842394, 3.5775087622449542, 1.910324117226428], [0.93613676940487001, 3.1636313429751266, 1.4300749251932678, 0.51873363704756437, 2.4671607492050591], [1.8153473962002633, 0.75121750381786478, 0.48794695519157671, 2.7379308446144419, 3.9710167988982148]], [[4.1168398626424096, 0.73539156914124348, 0.051028225410150667, 1.0582737828885838, 2.7078488878227391], [0.68227601243433678, 1.953163458910782, 2.968740837930798, 1.6184573124858312, 1.7796284079817144], [0.93499266322741348, 4.410359720410244, 0.08771914702499542, 2.6245986026225525, 0.59855276821151138], [4.9761319088954137, 2.78726645786025, 0.87279445257016108, 0.68734758695128495, 2.9976182375351414]], [[3.5184686653615862, 2.3271940518433949, 3.4679364998726294, 1.8250203612305431, 1.6199774892808718], [4.516674785535395, 0.7513942634462063, 1.8755165099881956, 0.7024247296635201, 0.71959080316548862], [1.2862456351505438, 4.1839156834417279, 0.17171225370070622, 2.0209681421727725, 1.5185687078507601], [1.296706794397285, 0.2004921643793276, 4.684012302433314, 2.7928319815382951, 0.86916223819592675]]], [[[1.5573164527808838, 4.5208696614212052, 0.70369444075752885, 4.9099273460521804, 2.9771466390282937], [3.4522424359495698, 1.935471321311359, 3.9385583409996241, 0.90764925079140257, 4.7335950102289228], [2.8131380941045054, 4.7906464615728925, 3.7837181505370845, 2.3140410922917312, 2.9049441296742331], [3.5594404183325401, 4.9629842784112492, 4.3770129448310691, 2.5072440478243307, 3.9366934302263039]], [[2.2175754635670741, 2.1389796758072817, 2.0105412408504724, 0.20915741724565823, 3.5544792881318417], [3.5406065536903206, 1.600647780584697, 2.427624828071631, 1.0146239802559132, 4.9861785279049338], [0.69549631140255097, 2.4029093884788622, 0.68731909135437763, 0.63974988131250543, 2.9324332626037237], [1.6944885249928054, 1.0965319471859498, 3.2657682870706282, 4.6088768329965433, 0.53617251240881281]], [[4.1354010969377475, 0.21694961148217529, 0.028757553170426109, 2.2106197375079559, 4.7695946347107148], [2.1660859979574143, 1.3088950266131203, 3.4836826469527473, 3.7322018176427387, 1.9945535326271424], [2.7557939871011317, 2.995566801365197, 1.9436407549325017, 3.2349919993034875, 3.4042229147215703], [3.714756527367062, 2.9836887027267807, 1.3025535051549124, 0.11123851818177362, 0.21510824733782735]], [[4.3213651179961863, 3.7758150178532519, 4.5950711869725618, 1.4725797356219894, 2.6646857126433368], [0.76260837772479517, 3.4419541519880257, 3.132342154303974, 0.89730680720796208, 3.5467590081064482], [3.1952259210517004, 3.2059945554917451, 0.65530068993116797, 3.7074796769738052, 3.7496714097392481], [4.9113736710287936, 1.3015933456957953, 3.9168429524922739, 2.2491791595277375, 1.2315025962442332]], [[0.23961587078433641, 3.7847774442813065, 0.29112663627121232, 2.4673491173093769, 2.6857838341558398], [2.4567211020466875, 1.7312812481941624, 4.317329352167639, 1.3467676949084348, 1.1170796944564354], [0.71388222784115019, 0.49182329991346851, 0.4831263714250012, 2.8767433001899896, 3.9763901695620514], [1.9013544445033184, 0.28561851824716727, 2.3631321296887458, 3.9747504416409631, 0.19488055021847089]]], [[[3.5299330150168218, 2.245592563290256, 4.9764077658289061, 2.506708589455275, 3.3632836909338466], [4.1256914800439608, 2.663722279038363, 2.5472407907308345, 3.9606981359938178, 3.3162201811864676], [4.7000167224015916, 2.7277190077360087, 2.2362118623990579, 2.8227641360599613, 0.4079918488899465], [0.010816456433592929, 3.7678363494630913, 1.1922549233625799, 3.143530633575879, 1.3013143014574176]], [[3.3196525380368205, 2.2796789157241579, 0.47417777503248604, 2.6322268682739689, 0.57271650399808671], [4.2373490209960671, 2.7657499777619257, 3.1776602715866415, 0.88537583681302712, 1.7284189253665196], [3.2035913112407921, 1.2519528521947942, 1.503082350730605, 3.7267050235711583, 1.7046638268512024], [2.5193745537229084, 0.56006620298299548, 4.2745196298042751, 0.45240145177673219, 4.370166974883281]], [[1.8074441703332511, 1.3239846458763831, 3.1372792244509489, 0.76635926542028199, 0.67103665329746598], [1.4581027749866684, 4.2311565259202064, 3.0088142088595742, 2.4820768182001731, 2.8830747862553414], [3.7765961590936592, 4.8630572643079146, 3.1398452849079566, 0.22628637030656501, 1.2625151634030825], [3.830935821070709, 2.1436293524096479, 1.1102619871577539, 1.5018027084957637, 4.1704947464501023]], [[3.6887206366247365, 2.0550333766576308, 4.7225791756762581, 4.4990867571425799, 3.9207401424511588], [0.12850618102364414, 1.0550014089546045, 0.60631601080975084, 3.2740684683025441, 4.6207537910302579], [0.45977197206031428, 0.71459743015442423, 1.4965720136229494, 2.6550629396571535, 3.4078359035340697], [4.4925161456886373, 3.6744729901065352, 3.9930501255718407, 4.7390847054911962, 4.2533438599542155]], [[4.8078051749971449, 0.16244356633732188, 0.92118575959702109, 1.3391781122550344, 4.0285723588153211], [4.948117361049774, 3.624110747508555, 0.55889807812560954, 4.5017401603480485, 3.9611808627641212], [4.4527027905665317, 2.8543248478331038, 4.3088607130975909, 4.9187544697527379, 3.1778432860618535], [1.2016462871502653, 4.6024476191131036, 1.4752227076679851, 3.3378448549436199, 0.71232786552636995]]], [[[4.1275624354383611, 0.61017090634083393, 4.2040914546896211, 3.3835464498879602, 0.35604109333131606], [4.7439755119231872, 0.95623298071273177, 0.11991676605855918, 4.4805762340233688, 1.2271540444711662], [0.98136926958558079, 2.4417093643322345, 2.3299549892221139, 1.0537254015625672, 2.7486628622957929], [0.71902583141259846, 3.0720924831361103, 4.4501133021422419, 0.48950711409444142, 4.9214121954520804]], [[3.5900708017741914, 1.7440038036508838, 2.2054144199706895, 3.0741785473772407, 0.33605716205384822], [0.44825467016597531, 1.2520639761278614, 4.6852438272692627, 4.485303212916615, 3.8168984514769484], [3.9271127040893252, 4.6412921873258757, 0.51192312577151422, 2.2598697532097454, 4.1533386275764759], [3.4126270180553031, 1.6628530716575081, 4.5033837370528014, 0.65131126479409163, 2.9366055466814798]], [[3.6049414545038232, 4.3949436649049778, 4.143603555875961, 3.8922733094094277, 3.8185459560753929], [3.9634222445796938, 0.85447142601771398, 0.81863222753093789, 0.78812940493963635, 4.3585886232608129], [0.52465775199160847, 2.9086035415491689, 3.9489633082162587, 2.5125970138396627, 2.416685484057592], [3.1451460128945095, 3.8318576481116873, 4.7034899778694168, 3.7008988586290816, 1.3934474432626613]], [[0.30425157171685258, 2.0535115736758849, 4.4045071514423979, 4.7831945879625524, 4.3145583963131591], [4.7388919495111024, 3.0016201471963484, 1.1613268701194992, 4.6306047124528673, 3.160700127726332], [4.29328427223773, 3.2766659710812416, 3.4584138325667433, 2.8883164035135995, 1.5337183493333189], [4.8955731145331258, 3.8118362780681743, 2.3229022234098293, 0.19574996080825879, 1.0289909412093292]], [[0.83161569803310287, 0.33299656405630318, 1.9692590948622133, 3.2061215549323046, 0.76516617158722988], [0.91363118059550019, 3.5600956690367722, 2.1986971190789668, 3.6012316070399795, 1.7736578506578167], [1.9575799876492384, 3.0939930848814305, 2.5781242661031172, 4.563805866405712, 1.354024574781596], [2.0395151058719048, 1.1723058941201603, 2.4351162729719107, 1.8900828537347603, 1.0881450607066434]]]]),self.functionspace)
      arg1=Data(numpy.array([[1.2294152447177682, 1.3349076562906259, 3.3863415946391999, 2.3388794364279146, 0.69719920158979298], [3.1397486674606467, 2.1814972765596305, 2.8699756593097967, 0.17507992815815757, 4.2453302298516649], [2.1908471538547025, 3.6979733155572796, 1.2075802569117076, 0.028659140931619996, 1.7873816924100863], [3.2275483636558087, 1.2152183468410733, 4.0978795114881752, 2.6458651275873626, 0.45676771495832397]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[1.0455178231260904, 3.4270864790667419, 6.4058454042509521, 1.8157157509570563, 5.8074469047779154], [2.7996783950811346, 6.6444838810354998, 2.1633263541492505, 1.4210000603394686, 2.6597371501046077], [6.3164232373106017, 4.6608038907095191, 6.923078274880786, 0.91714806598913357, 6.1754235859764837], [6.9768836658638893, 0.46104642245691735, 4.9991768501200911, 1.0148181752637542, 7.1918490996634628]], [[8.1562655729921989, 3.9019367427686822, 0.5666193779313109, 2.0580541880887333, 5.7219759938432846], [2.5829459680414106, 2.3682712225813867, 8.5608196519076962, 3.75589196528674, 5.5659435848580934], [2.3627879366068365, 3.3963369058983219, 5.1104918235209436, 3.6189045031177547, 6.6430501187938242], [1.7837880662017431, 1.0705815358262019, 3.7467689995438223, 5.6050841856999689, 5.0429926075938054]], [[130.47722037049144, 95.459937372563672, 52.083112516185871, 110.22373309187617, 152.3247917262342], [0.35269954194140168, 13.547804154071764, 78.176384706064184, 74.922691624302985, 8.9520917556769728], [0.79973313815347535, 49.40833456959394, 3.3581333584132866, 0.10831893980527552, 21.287003347077754], [7.5322748925615386, 0.37957728755267256, 0.088048754115870126, 30.287394859606373, 106.68069100432184]], [[27.377326877835799, 0.48730762555350826, 0.00095000189001342614, 1.1416470099953779, 10.276763704027296], [0.40893350814222695, 4.786328596350427, 12.74352582803742, 3.0836324827503456, 3.850260568441644], [0.85452329483586198, 32.162293925020428, 0.0033730690045523297, 9.5529517371147126, 0.3010713910151569], [42.652657712185032, 10.995596953731411, 0.72744512640968884, 0.41607848463778313, 13.035339182977012]], [[2.4039052991708165, 1.8020025225328167, 2.3797819450225215, 1.5210925585123587, 1.3998112947561647], [2.8611403311569146, 0.81932306621987394, 1.5503139950507641, 0.78171689469005357, 0.79498731189754079], [1.1918471040253464, 2.7124835820882378, 0.29275483011350317, 1.6331866331843603, 1.3381227401078652], [1.1985970470721794, 0.32615411887840845, 2.9346367581082551, 2.0463602198111466, 0.90686190983279991]]], [[[4.0180471352614537, 114.08600130355599, 0.33176036058738517, 147.8434256137167, 30.733613769026153], [48.921841692313379, 7.9513026917512502, 73.996203973148198, 0.73768892698464172, 131.804534519458], [25.724503788605134, 136.85684955262633, 65.240865534117603, 13.932644996899798, 28.453535697390539], [53.851777738308144, 152.91772964236475, 103.07117822181901, 17.921609953085508, 73.886251481179457]], [[5.6824131200759167, 5.2522443812356254, 4.5885603994461936, 0.032931710578165158, 15.904364400069248], [15.769264558946652, 2.79042709753256, 6.9226527452641635, 1.0321780221633456, 33.27963911070885], [0.45286275409100279, 6.7698272813930247, 0.44132802732510218, 0.37740861558668687, 10.453384102432992], [3.1597094734803894, 1.2226617715732582, 13.220798288895217, 28.030504108292131, 0.25673117194722411]], [[58.801799425254195, 0.012455642356401904, 3.7727285472950115e-05, 9.7442163626244387, 88.557491197061054], [9.1913874603834174, 2.165279582634601, 35.944922035570052, 43.805230762978205, 7.2535373549163173], [18.344144022957583, 23.306827267863746, 6.734740805754643, 29.061879965841197, 33.641758101574062], [43.220147520403273, 23.042575417986367, 2.1353078525445821, 0.0018313680124762066, 0.01215463749422762]], [[1.292065327471507, 1.2618943433603773, 1.3060327832580365, 1.0701070560128481, 1.1871950302806911], [0.95365956205710856, 1.2416059782843349, 1.2212842126063479, 0.98120760832319032, 1.2481434187489528], [1.2255417161260791, 1.2262638548478417, 0.92867218460976086, 1.2578656912858224, 1.260360228349795], [1.3213436065872204, 1.0472306167757663, 1.2700219297003006, 1.1524772085547523, 1.0371305123617185]], [[0.0023218744576244052, 284.42787620765415, 0.0053070217771253063, 46.253908983374863, 66.305311928768262], [45.413975031643702, 10.278962574510073, 497.39132725147363, 3.5390782645167307, 1.6000484287924037], [0.23910929228220984, 0.049161808767018957, 0.045575718709134551, 88.753759057040526, 350.77508450215657], [15.300828503896362, 0.0048936554676968476, 38.510567462201571, 350.16141801224484, 0.00096567887429779245]]], [[[15.851581539184886, 5.8845388821404923, 33.638478619142703, 7.4881671836491863, 14.257990389850653], [22.307931336513679, 8.5542362424801883, 7.7559911607581, 20.399828856604302, 13.824518654342604], [29.680216494466784, 9.0109458342852022, 5.8308174527181942, 9.7131287386715144, 0.14028037478477562], [4.9315192112454205e-05, 18.286457784903906, 1.4699855954602554, 12.296064864691468, 1.7807129754410065]], [[84.525283871885549, 21.057525166712733, 0.063334330583038154, 35.838053485904794, 0.12731144980254891], [208.43841210755755, 43.034065807437607, 71.908462731951658, 0.63749901352720395, 7.5651877122146791], [74.102443586555566, 2.2955022534109646, 4.5131272666555784, 129.64168708168165, 7.1877654959868114], [30.476833255810426, 0.11721842829606928, 215.28038381575976, 0.053227602740067465, 233.63861679824882]], [[2.0437439649369833, 1.4034063668221064, 3.9776613817008535, 0.72517506567753243, 0.61770651710140212], [1.5768388806211386, 5.708205911450186, 3.7818195747037606, 2.9975819355976503, 3.5918063636251274], [4.976170209501027, 6.7530227048237252, 3.98159049346387, 0.16622571909779493, 1.325108143453307], [5.0627613095374979, 2.5112493303003731, 1.1346316706017916, 1.6340834478869242, 5.6095274210090995]], [[1.0381166878902259, 1.0208574919484634, 1.0454936468248708, 1.0440420321479023, 1.0399331413324764], [0.94289325242669919, 1.0015356485618023, 0.98576260940240024, 1.0345749831865474, 1.0448407412946326], [0.97797726245185379, 0.99041572370262532, 1.0116217248281296, 1.0283799957055164, 1.0357629725085804], [1.0439983030219655, 1.0380015567766152, 1.0404779426228783, 1.0455981907179215, 1.042362732881704]], [[16.553862129471579, 0.038835073053132892, 0.86352490185608943, 1.6854215064097791, 12.068022740643574], [17.427268719719489, 9.9886421167678652, 0.35349798611552291, 14.717708872966496, 11.709568038238897], [14.432386159328312, 6.5186615770970286, 13.609677659666769, 17.242856695588323, 7.8977320548592544], [1.3886452781244791, 15.311373150607459, 2.0036089487437123, 8.6225119895544111, 0.54535940955939088]]], [[[97.09146113286765, 0.20301789825353736, 103.0224902227757, 51.118040022601001, 0.035681653010442585], [152.15366615604819, 0.86550276887219901, 0.0010642419010461694, 126.5352622138718, 1.9360924442859717], [0.94110667054127939, 17.836146653210598, 15.333169036175089, 1.1840061067202463, 26.138809098169727], [0.34485451996307354, 37.429819356821667, 123.77957326527945, 0.099697023740571447, 171.29832307142894]], [[4.7268450559828654, 1.9657707375759765, 2.6146623000156195, 3.9146896311118886, 0.26575936521791876], [0.37716082015837937, 1.3141276061562142, 6.5326000789356256, 6.1954001588182566, 5.0921993908090837], [5.271434964239492, 6.458205109830355, 0.44322094537781542, 2.6933243374731464, 5.642710829421639], [4.4444632592083861, 1.855178417854058, 6.2257622050892794, 0.59389884093241485, 3.7028368281318151]], [[191.47046385819368, 431.26598586375087, 338.79848895188161, 262.16947600745243, 242.40857627800852], [282.37107227204797, 0.52493407624885946, 0.44040123572136114, 0.37693732296432669, 416.83328041634229], [0.071135433966108066, 79.455383741563296, 278.17357267110572, 43.61693913148487, 37.186896356199462], [109.46419530208706, 245.89024021593238, 569.50436202203207, 213.23304004287229, 3.8946271679065023]], [[0.042924353903490101, 6.7115885522742023, 50.54351757333221, 62.869829490166744, 47.858154836995375], [61.340837327114251, 18.323940971023529, 1.4854593758045511, 57.701566399725046, 21.006765572961147], [47.236320001171897, 23.108087676560668, 26.656047608316076, 16.55022584269356, 3.1006863780616922], [66.853959539419819, 34.482823359126769, 9.2996900132700517, 0.013364052872567062, 1.0785475363400325]], [[0.91922804296095295, 0.60515398420241207, 1.3627871969396097, 1.7026105537264682, 0.88491872463501264], [0.95958051232717922, 1.7860346752939353, 1.4331449287988387, 1.7954316473341825, 1.299199098870907], [1.3590894992584424, 1.6751488266892511, 1.5412401645892342, 2.000595839602425, 1.1484782024856448], [1.3847835850234866, 1.075314931435879, 1.5015843092439625, 1.3374806961970849, 1.0393392861112034]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_constData_rank4(self):
      arg0=Data(numpy.array([[0.95939404685667384, 0.74651197110172096, 3.7371234657575605, 3.0756768534854406, 1.0493437008575417], [0.97658482597740881, 2.0955116627536601, 0.24815133464646152, 4.2318918329637496, 0.56968090821919093], [1.0082734735276895, 3.3374287863685783, 0.4398608027265839, 4.5520456497820767, 1.3324240404870815], [3.4607178543566084, 0.65654620225290805, 3.5220283259916827, 3.8850263909045806, 4.9563862531113445]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.5375701481764126, 4.1703385854940471, 4.1354793724485717, 3.0100655696446132, 0.16503602915047819], [3.7567360449873544, 4.233157017894813, 3.7497956855206551, 3.889782467336778, 3.8358430210774483], [2.9391941185444423, 2.3138083165190166, 1.6538836360993989, 3.499581218546874, 1.7227136501534659], [2.9928832285038567, 3.5636727919096804, 2.5356785353728748, 1.6643633219290346, 3.1029535098854621]], [[0.053051857513636842, 3.2047329165230281, 3.6212156249084284, 3.3220303718342983, 2.5269085308113679], [1.8147125665470987, 1.591680326840764, 1.618926344693876, 2.5182941546029318, 4.6334548525076054], [3.3213102299140482, 4.605200285124865, 4.5424599581290197, 2.0680748420633428, 1.4161151955677007], [4.0871274775061082, 0.52572740793618056, 2.3229819774954579, 3.6920032822748317, 2.8469804498519]], [[0.18392896346024337, 2.80128484899666, 3.8635174342116718, 4.4670675243690168, 3.9949090637920084], [3.2740836311608712, 2.3476763866750905, 4.6692863083518876, 3.5511666282775005, 0.43062627849343255], [3.0893957432128469, 3.4481746809272775, 4.2339155661321968, 2.6763339151576337, 2.8333309003494622], [3.936486429184598, 1.2260250441647227, 3.8633086225786863, 0.23608565626873054, 1.7236550883367925]], [[4.6004567961511844, 2.9688005261212731, 1.028620399612858, 3.3260880183177401, 1.9434893500734602], [4.0882820363841841, 1.1837025637393932, 3.0671199445916559, 0.12131444305377924, 3.3521230753516238], [2.5723120164848887, 4.8648541972880173, 1.8369610226208677, 2.6451926263632668, 3.1578816357504826], [1.8008383107265962, 0.04427954465973636, 4.5939753081349082, 2.4108315806983942, 1.8272522949149461]], [[2.6987958734556114, 2.6976830555077496, 2.4591057762102397, 4.5781574636812321, 4.7692017824952266], [0.55413175370199896, 4.1692571205250895, 1.5166214036452794, 1.4576280223490783, 1.599326941611438], [3.6898919749784356, 3.7165544482115767, 3.6641707394920875, 0.73560937700501527, 0.022445672286671874], [2.0454506316150112, 0.4289741150904498, 0.11611378748321567, 2.0980896917977971, 4.2548976289594975]]], [[[3.0580209991445413, 1.1374932105358262, 3.03193337442704, 4.0337021616796438, 2.9334024831295511], [2.7115105122200154, 1.3053217921424265, 4.8953582436242478, 0.81232624033060996, 0.090547891716975748], [2.1480619722267873, 2.648428680775099, 4.7502623348943098, 4.9153201148306005, 1.0881078252162946], [1.5633337121930468, 4.9018076243720339, 2.8568676463717289, 4.2843236322383653, 0.6625358545506832]], [[2.1254122347918281, 2.8506255330980266, 1.8411946721553938, 4.3565296497178245, 2.8344441609693631], [1.3625427478145524, 3.6898206336625039, 2.0095255666467478, 3.8138137343837251, 0.074585897068667584], [1.7455263237179999, 0.18949423143594624, 1.1185748337921433, 0.29555565151253782, 0.43722470152790871], [1.1048415648629641, 0.64599183528903381, 1.8817107305834526, 4.9700396532573343, 1.9367392500043641]], [[1.9351319100970923, 3.7659864408752739, 1.9195626309557989, 1.8116049512069843, 4.0887910621851873], [1.2413338551824848, 2.6858884621619956, 1.6723739421068495, 0.87384360395634808, 2.7279623962130404], [1.1503649864121877, 4.0369391481884751, 1.9124388123686864, 1.8253683017745608, 0.63190277899319902], [2.1273280764522315, 1.8941179382842845, 0.72933479167333448, 3.1479891338333545, 3.7256251976808201]], [[0.80045537204637163, 1.8840600416142195, 2.7778721266377997, 1.471962756779968, 4.9720383674631758], [2.5815718748478211, 3.641385159634551, 3.7846442968435481, 3.9267181296330227, 4.4404917791055682], [3.1569244931133493, 1.447793701807125, 2.8462818375975911, 3.7846671556607667, 2.7591484272641291], [1.8936628976792533, 0.26561203909874342, 4.2273940986713967, 4.6334718386935796, 4.4948642370938856]], [[2.8949289351415528, 4.4726121529205303, 1.3326307939271125, 2.6251109311754646, 0.51227147174914323], [2.5783817847891162, 1.8369418845327989, 3.9599976477308942, 3.3357873589595468, 2.4842125122302892], [0.097033510075575674, 3.8115903741155845, 1.0336185821132704, 1.6377904061072244, 0.17494435670101935], [3.5938343487799962, 4.9429067676512961, 2.6625088436114352, 3.1865798397349616, 3.3427675798554515]]], [[[2.5855387334160382, 2.0467604922148972, 1.3550487582444839, 2.7324310407352956, 1.151003621265138], [3.7440909506647135, 2.5392971502996908, 2.9496319851793653, 0.85874835389932358, 0.18883260404791266], [4.938679584348475, 3.4691710381867282, 1.7828952782349472, 2.1566154019654933, 0.75303894375013536], [0.95197104228274043, 0.25048068515982558, 1.3982907352583491, 1.7539703396426165, 0.76453608816700791]], [[3.3513926716974805, 4.5055005715665839, 4.1783810959459018, 4.2519385430973307, 3.4698687552253094], [0.99859924425091806, 3.0734550849779754, 0.54644426721889661, 2.2832217257751681, 4.8211515563713991], [4.1282591712735623, 4.8477914552356083, 3.6813914603120783, 1.4181394483889203, 4.952386525665931], [2.6570576894347635, 1.4561243347449015, 2.1788553410001703, 0.16496522422052273, 2.5403020705886203]], [[3.2466743925817099, 1.5952295922682329, 2.8553213619516042, 4.2844667724034879, 0.88365985952775883], [0.79525586038659624, 3.7901676175670729, 3.1845121442560753, 2.543750301797215, 2.2256659445942004], [2.0124982336906241, 0.34054654958130393, 2.2254604825922639, 4.6225724888602144, 4.2945703671746003], [4.6700362971637475, 0.17018111371432984, 1.4821915479009304, 0.92677060465230199, 1.4461594892616472]], [[0.8293560772113906, 3.4379368239107349, 3.8438673585740872, 4.7245935077493817, 0.18360402652760621], [4.7060641354024044, 0.16745130807458733, 1.0079657166514637, 2.8107431186945653, 0.90321086949050011], [3.2755943123865419, 3.9490419496232279, 4.4451736150014245, 0.63440714106230556, 3.9671393086255891], [3.3110864432573388, 3.0644706556046772, 1.0602303086971878, 3.778009423728848, 3.2315604768194666]], [[2.9937172616513896, 3.8974768724539826, 0.30032208814850569, 2.9856617060411805, 4.0929504276339754], [1.2621434911738385, 3.1407068380742809, 3.117665349128659, 0.52961129059490419, 0.44853645381246693], [3.9344639322244874, 2.5182697825305489, 3.2967317947277355, 4.7449891703944527, 0.70665421172103682], [3.9669377040787639, 3.3144157248150248, 4.9232023051172735, 0.52992356791762352, 3.7781265468014111]]], [[[0.93578722365149181, 2.2782642200191532, 1.4224158734571553, 3.2478024409125834, 2.0411610555614796], [0.13046736610275356, 4.6487646359199317, 2.4667232459796913, 3.8922436605413706, 4.1691204475093784], [4.7315719578075486, 1.8666414332909098, 4.8323684974516619, 0.96244817086180923, 3.906941441156631], [2.313598918504796, 2.6692758374600154, 1.8525567551343256, 4.8410492585427036, 4.2030363458493598]], [[4.9141005184702902, 4.0599681610518248, 1.204776816180712, 0.40355064018355286, 4.3066346511406559], [3.4676728155220564, 3.494876433732883, 4.8237853379541269, 4.8693346872500358, 4.7829286014201795], [3.2824746964995657, 4.5080459361097391, 4.6348598911166921, 0.3380124425150921, 1.5508698303512591], [4.909580543990975, 0.76464276046496749, 2.3019935410340504, 3.9904346275536944, 1.0930982446266508]], [[1.4606531968613359, 3.6858809719267018, 2.53545232969251, 1.8887021964606185, 2.1248341398498618], [4.1667518340302845, 2.7344586812287965, 2.0644621905723879, 3.2725429826536936, 4.9913442637873544], [4.9280861410396275, 4.2921653222763503, 3.815678809086203, 3.2343084477017841, 0.19408323955945375], [3.6900660014568651, 4.3867540721224039, 2.5464507577042212, 3.6530440870381344, 0.1375721022019932]], [[2.4351492206055871, 1.4486106891228432, 4.6882427008627978, 2.7581158559632066, 4.6066198435531751], [1.8001175712906179, 1.9519384151033561, 3.9565242026611873, 0.20671665842109904, 1.8133631390982847], [2.26862438108195, 0.23369227884946417, 4.5418722559642424, 1.0416424287397754, 0.90630915277080593], [2.677428151696164, 1.5941720213942663, 1.5355488283149987, 3.7456041757558762, 1.0201020488724537]], [[4.9801293775145972, 2.0690569066202196, 4.6887636018985557, 3.4933168889831294, 0.40441409648635551], [1.4202422726044595, 1.6532559454547204, 0.85000544690845314, 0.78561344608825456, 4.7495863822108886], [1.9800066678006956, 3.2190208993172282, 1.1608607006437133, 0.98824707746727936, 3.4588193416335584], [1.1608619877276181, 4.1532069695173766, 2.7640566820344916, 1.7223296622300188, 2.9514334188264639]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.93825125563314471, 0.84124302214983682, 0.84245952458014983, 0.88269333556413343, 0.99318204473723215], [0.85579068193448504, 0.83905524371332996, 0.8560369295864203, 0.85108379470530982, 0.85298892435305518], [0.88529038001056681, 0.90854104790307566, 0.93373828233236367, 0.86496215660581099, 0.93107790216010222], [0.88332227230459837, 0.86266716367198459, 0.90022325624452804, 0.93333273719043086, 0.87930104203910919]], [[0.98461027856447858, 0.39184722761684182, 0.34692755165665151, 0.37863810278231402, 0.47772233612950865], [0.58829917966009637, 0.62793564247219025, 0.62295387643277433, 0.47892692814485405, 0.25806056031940661], [0.37871782543728644, 0.26020097936692221, 0.26501755382762843, 0.5462991837034521, 0.66100607347456952], [0.30275067530279537, 0.85753519536028977, 0.50706860243067153, 0.3398218986548307, 0.43504901008986374]], [[1.2744013387395805, 40.164325070979125, 162.93275649056125, 361.04674351236332, 193.74702071432827], [74.909255483489972, 22.08679207874907, 471.34755469195295, 107.93837417864503, 1.7642075232217631], [58.721176599295923, 94.234325359179593, 265.50652918011036, 34.064450279719857, 41.897493069333564], [179.38490171120884, 5.0343713555330982, 162.887910573955, 1.3651107250194801, 9.7019064426675374]], [[175.69124857623157, 28.093032372932264, 3.1761847941147456, 41.969493710443857, 8.8778450808262726], [98.8187181912987, 3.7807386985866422, 31.374200378528514, 1.1460254304437507, 43.215276325539101], [17.994332329315856, 236.46146903883366, 7.8764052893167644, 19.529771099230175, 34.742333866542921], [7.5631430310155157, 1.0510074444321462, 174.41649200987209, 15.00869248708862, 7.7909563600961214]], [[1.1388139118545508, 1.1387528744430799, 1.1257422971533728, 1.2467081491015102, 1.2582328437703392], [1.027049072996026, 1.2223948712408985, 1.0757821210712244, 1.0727297198853343, 1.0800760521405333], [1.1944948292267881, 1.1960297800171873, 1.1930159320737779, 1.0360657097490804, 1.00108167864649], [1.103535338274267, 1.02087643134226, 1.0056082793569363, 1.1063367432426985, 1.2274475027785454]]], [[[0.93010692590305977, 0.97340856442369816, 0.93068201335268064, 0.90885184213238124, 0.93285728259298184], [0.93777463322373833, 0.96954551612449857, 0.89048503758160669, 0.98093705472665504, 0.99785688833030861], [0.95037801696764779, 0.93917731649354297, 0.89355166329552138, 0.89006396455600245, 0.97454823665891877], [0.96363652388799614, 0.8903489734334985, 0.93455045212164567, 0.90347094365677827, 0.98442466723246858]], [[4.8180785022444752, 8.239060882379226, 3.9044295204716359, 25.102083484285242, 8.1410195505711158], [2.7401319181278909, 15.328610007776232, 4.4222229555533232, 16.801208776755459, 1.0567292019546166], [3.6376448529082603, 1.1504893882008909, 2.2876364112085157, 1.2443974130512774, 1.3818979110528089], [2.2645120566420478, 1.6126895430223258, 4.0232312453444408, 39.520683694867238, 4.1903964483799498]], [[0.067405728639416282, 0.0052542529948743314, 0.068884360934096098, 0.080069224725673127, 0.0033506009452116052], [0.17727244824786959, 0.023674256068767136, 0.097216468300694858, 0.29585368686107855, 0.022325936623031655], [0.20123484679472231, 0.0036017022625058556, 0.069571691526680995, 0.078547957421576775, 0.41449543050554238], [0.051566025801824997, 0.071371016684398408, 0.36186432430699383, 0.012432958447067351, 0.0055582870344197087]], [[3.1733133092893939, 15.150561426972125, 55.008890651987429, 8.3605632371290799, 1303.6285525426908], [41.442313825884042, 191.1849244373461, 235.07716472706068, 288.55234638172038, 605.5103303491652], [95.043472193413479, 8.0740751613010922, 60.714711374727834, 235.08491705399402, 53.542894107469031], [15.361911109167204, 1.4669493589585463, 445.25468281033284, 799.88993721724853, 654.91928771827247]], [[0.19614219746694297, 0.080730048932077081, 0.47244062122669417, 0.2283000533232471, 0.74957841744706366], [0.23438247536112583, 0.35572105398736659, 0.10772139496393851, 0.15305206644848798, 0.24713663266710745], [0.94686504178960884, 0.11710299620732563, 0.55900584408439435, 0.39790178335173909, 0.90625233562618757], [0.13236729211335932, 0.061959813780592399, 0.22354612563257445, 0.16645651225463101, 0.15245211460925842]]], [[[1.0215319178077711, 1.0170071545453927, 1.0112273968636676, 1.0227690363539113, 1.0095287326482789], [1.0313299651266068, 1.0211427836219107, 1.0246010361915412, 1.0071006932801463, 1.0015570851476143], [1.0415311959163864, 1.0289964558601739, 1.0147984805596713, 1.0179281076710336, 1.0062239046521428], [1.0078745468714123, 1.0020659507050329, 1.0115877507588444, 1.0145566573475036, 1.0063192287425913]], [[56.775264530400442, 228.1566330140509, 153.82071925065861, 168.07986432229742, 65.489393904566924], [3.3317993196390545, 40.614704346290885, 1.932037229934864, 15.669879874420682, 333.77101146713943], [144.80395390308644, 344.66109049716289, 84.505406427006577, 5.5242231774773529, 390.96562419917518], [24.588718747717412, 5.7829969238948058, 13.817804639432204, 1.219957718783891, 21.361148862145424]], [[0.069496158083610579, 0.26977764024708745, 0.095840804242301861, 0.029634396940621848, 0.4839628381397772], [0.52040848890440516, 0.04447388390924964, 0.073136322942243731, 0.12378929547474384, 0.16074553805517719], [0.19150168327506281, 0.7560181873428925, 0.16077266540077423, 0.022449100544013095, 0.029389506645088589], [0.021590829558907912, 0.86955887924647091, 0.29602261907667959, 0.46712710737615687, 0.30491373099803992]], [[3.5146950600265661, 183.1777574039337, 338.89011277862971, 1287.5338929031516, 1.320837479090293], [1251.8794628084809, 1.2888950554123819, 4.6073339923701386, 70.802679883680725, 3.9309666864267516], [143.22466786640697, 397.4527527494003, 843.029263351228, 2.6155978568919718, 408.5049379023136], [151.13984722989622, 104.00514831578523, 4.9871289292756416, 306.69787745615662, 133.97829979621906]], [[2.3612625750737157, 3.060490987073281, 1.0900160275506019, 2.3558097753249334, 3.2370947367204468], [1.4365365143772395, 2.4630057526659201, 2.4467718669709, 1.1641583407632248, 1.1373828801206887], [3.0931520062326388, 2.060076464165093, 2.575803687507134, 3.9032567776945815, 1.2248393187414464], [3.1221148416038398, 2.5889098576171197, 4.1080909123732576, 1.1642626814433616, 2.9574334911496218]]], [[[3.1955442468188169, 16.918546423863617, 5.8468042251817725, 56.377430656217307, 12.604482996766334], [1.1758274458679141, 320.96430278358446, 21.378330316492313, 125.47759867732978, 176.94928092627066], [355.71619815091628, 10.149156661810508, 403.1342792544088, 3.303083227959025, 127.78819777723952], [17.677234464441963, 27.490541391571327, 9.9732331978803117, 407.50233490524363, 184.55894881703412]], [[0.12648041566422141, 0.18117693560730927, 0.60234521146893216, 0.84383452869899445, 0.16331586713323942], [0.23245335884315066, 0.22980780823433528, 0.13137931773012226, 0.12888534999143247, 0.13365738069959154], [0.25129158744043645, 0.15004569832481288, 0.14224933286106514, 0.86742791762402793, 0.52071759989373267], [0.12672118936002347, 0.72489187107932429, 0.37961720386170839, 0.18655594772004877, 0.6313250373859598]], [[6.2903486340229513, 103.61278351706827, 24.342612812802113, 10.782732874165859, 14.515904534543491], [189.82354842395489, 31.27392860554513, 13.453434030086612, 61.574810809539194, 536.08212656352816], [495.04221913584001, 222.2927433092263, 122.00731495335528, 58.680893234458942, 1.2768013306639912], [104.1601712412167, 250.4065801843492, 24.682040074052949, 99.416476597835555, 1.1891138597772566]], [[27.243460707907648, 7.1417132691629419, 579.72495196375576, 42.229589826528354, 518.93620689428644], [11.507432808709696, 14.140368849417296, 214.75907279604445, 1.3238462594183078, 11.716160314598822], [21.732692638168395, 1.3732095679947105, 475.28284077799702, 4.1109081157714025, 3.4211581641363802], [37.849488653184757, 8.70152519104305, 8.036061007277354, 161.30080230442064, 3.9924731098647674]], [[2897.4260115628645, 27.436975909506788, 1817.4606779977166, 268.18355124120382, 1.9104451947313725], [9.7118714125913286, 14.102168741871434, 3.8984696603816942, 3.5166656891171706, 2003.3041937938788], [23.792036633026711, 172.88253667308561, 6.4119465859063336, 4.8640151803568807, 253.77605673327099], [6.4119597958473786, 771.19591424852831, 83.459515791653047, 15.750833739030888, 112.65063312193547]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[1.7890280015401587, 0.32291500571268078], [2.281206349946832, 1.7792977808243036]], [[0.71164532667640634, 1.2375540584510141], [1.8548738449364408, 3.9399632266823237]], [[3.8423075241308569, 1.3765868313104146], [4.9183745160313679, 0.96859844954291707]], [[0.51889263944620656, 2.4320477274659047], [2.8687053823238533, 0.95831382109822938]], [[4.1726536308903333, 0.028346811499851889], [4.4202727469158338, 3.9978169409688338]], [[3.4406120895268399, 3.2341286824707995], [3.9923033911676185, 0.45627749403452722]]]),self.functionspace)
      arg1=Data(numpy.array([[[4.6821775491907287, 0.99791352303592995], [4.4096526573892714, 1.7553960866091203]], [[4.1625950578472777, 4.103613941819261], [3.8399901548024111, 2.6646534572617577]], [[2.254756366249405, 3.2630570582134033], [1.4720237542289163, 0.093350642212098586]], [[2.6065105984352401, 0.17255498423489657], [1.6762437262533303, 3.2352942223972589]], [[3.1106036945968989, 3.5811161599024843], [3.4025634552575941, 3.3344370933909562]], [[0.25831737761022933, 0.091929275749060818], [2.9710062960513941, 2.5391053358588054]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[15.233450743473824, 0.32367749403375934], [37.964820939798194, 2.7497010419338466]], [[0.24267955988737472, 2.397991582957669], [10.72320398599115, 38.617154107983438]], [[20.802369539202328, 2.8374231097682161], [10.432246069486668, 0.99702606471505206]], [[0.18086165918912961, 1.1657392211516326], [5.8505029051960955, 0.87130887155930159]], [[85.085764166776499, 2.8723531842486768e-06], [157.10189009510776, 101.56419128595809]], [[1.3760123430328328, 1.1139395380756119], [61.127821106872382, 0.13637873462869562]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[2.3903464773455445, 0.14240819509379035], [1.7757076842634916, 2.1866217842490445]], [[2.5982989154114366, 2.8993252256105668], [1.8960512316704026, 2.1106631231582806]]], [[[0.62734020816482372, 2.1429383884778961], [3.8615755433351597, 2.5357090353132379]], [[0.16393980612379058, 3.9730152153271141], [3.8460933276385343, 1.0833542226511019]]], [[[4.2959740735436132, 4.8257599830957991], [1.103309537893459, 1.915767929918107]], [[4.4163284033187393, 4.4762329715073825], [2.7472744532129876, 4.040544367953756]]], [[[2.9187271738228016, 4.9540061816303416], [2.3574465279043904, 1.3567868183631413]], [[1.1780691044401306, 3.8458970269781472], [2.2742869891360797, 2.5336072967815642]]], [[[0.37539754016593191, 0.87654201166650958], [0.67017841516247711, 2.0784824701574935]], [[0.16319226014266874, 2.9678433068819214], [4.6885968405344336, 3.6039588089979659]]], [[[3.6828119965152464, 4.3651069954253323], [0.87823943259061832, 2.8463379097068993]], [[0.23537335468914677, 1.3421101291545394], [1.4830381487187474, 1.2225148873543095]]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.81506969939160845, 4.1604891589952482], [3.5565810183723103, 0.74145068258409053]], [[2.5897154781947411, 4.4121765749407391], [1.3350103332782, 1.5652870289976781]], [[4.0079204341141228, 3.6162544908182501], [3.9097590307738197, 2.382674670058861]], [[2.9834655744192458, 4.7863571014837989], [1.5473818457354547, 4.9877993786820074]], [[0.72352799364686715, 3.0103386658367737], [2.335414020523539, 1.9415491213673162]], [[3.5239896976425809, 0.79634331771389111], [3.0200437966206759, 1.6895211289860428]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[2.0345669059715839, 0.20420765226483048], [10.902028536157079, 25.919383794685491]], [[29.845263968489967, 44.075424200403262], [1.6069861082818169, 1.7399654769021429]]], [[[0.29894459465182516, 7.1981501943576349], [388.06880885198092, 60.667763715575049]], [[0.089453138385179529, 6.3071287367985969], [8.2361440440171076, 1.1335111122029775]]], [[[344.55672429333674, 549.13232358192045], [1.4269362316291494, 10.495998886866357]], [[332.68596512890491, 350.68059618883666], [11.111195567566895, 27.857936296737748]]], [[[24.428049823822658, 118.40742189021634], [60.623317309614933, 4.3077244223929183]], [[1.2886303391680685, 8.0392354145545148], [60.238285179771601, 103.2216115779913]]], [[[0.49218982651823312, 0.90906402739053316], [0.29976046199449574, 9.0474079165059731]], [[0.014498648050662512, 12.68660037856324], [20.084579083792487, 12.050785816752819]]], [[[98.903326372728557, 180.02607414114806], [0.90177156296585947, 2.3002126336952382]], [[0.01266716777335593, 2.4317886212786335], [1.9461014016736105, 1.4041636981279122]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_constData_rank4(self):
      arg0=Data(numpy.array([[[2.5922685756068446, 4.2735670226053974], [4.7457518045051676, 1.1549378663959236]], [[3.7754633248466467, 1.4796830705297941], [0.26625422806508264, 1.8299729422717539]], [[4.5419761601205675, 0.2230890913327041], [0.27686719577351315, 3.7389250352772714]], [[2.6075255463356433, 1.5302943392889459], [3.6629825096980646, 1.7164180192037748]], [[0.26931779175726206, 3.5073108791497374], [2.8688801098471801, 0.37567722956579769]], [[4.8487251258945347, 2.331811168556035], [2.5199888954206351, 1.4772334357465156]]]),self.functionspace)
      arg1=Data(numpy.array([[[[3.1440749415975997, 1.5696270627315831], [1.0510328687898598, 4.7469022805237984]], [[0.93272209843928877, 0.29606847072174591], [0.16444251043942337, 2.1244939055630687]]], [[[1.7653612549639979, 2.1019208617724927], [3.223794845844997, 2.2421524171990339]], [[4.6622975722337268, 3.8753957444913034], [0.092422433608501151, 3.6117627471240215]]], [[[3.3958809815361448, 4.1276288788033106], [3.0124633884425975, 3.9977513761414882]], [[0.8653151392732541, 0.9597793022921024], [3.9366794999616079, 4.1037630257279574]]], [[[3.0267567237413284, 2.1041001697315922], [4.6517095766366996, 3.8588849645727823]], [[0.65855669020073315, 2.0237237943381192], [4.0387328775365532, 3.3868020549766169]]], [[[3.9368997604602938, 2.783673713127031], [0.28706902249134247, 4.3634840489452955]], [[2.1454157850723439, 4.8733417015364102], [4.7982627605131496, 0.51211800867379365]]], [[[4.8079144926509381, 3.0742418101870599], [1.2285001356130401, 3.2510090940323546]], [[2.9766095143765403, 1.3952712491321737], [1.7371316505312604, 1.8347811646888823]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[19.982089819753146, 4.4598814613173188], [4.6023704143141355, 986.96318935235684]], [[4.2737059639530326, 1.585742235114556], [1.0239701500405975, 1.3580176303551177]]], [[[10.436645568457294, 16.320964396876253], [3.5366248481960678, 2.4073773667265304]], [[0.0020919983028293756, 0.0059264763026050382], [1.0574400983945467, 8.8692700079223563]]], [[[170.5788913990153, 516.25328104364678], [0.010897197851219832, 0.0024852973995408549]], [[0.32914638308283112, 0.29154374712064945], [179.77132516528425, 224.08715736409718]]], [[[18.189578222774855, 7.5125367400067233], [7.2363490244618749, 5.164465259411962]], [[2.3513496229740207, 13.83712906562452], [8.8629824049886334, 6.2319201386386096]]], [[[0.0057149257406957244, 0.025944408986880121], [1.4336568984346112, 238.77370638710099]], [[9.5936397644978548, 170.05481132653702], [0.00911695973121502, 0.60569646526312171]]], [[[1978.9676948628648, 128.16956947909657], [2.8295108580198618, 15.681027483133011]], [[15.660548622704859, 3.6312818899417185], [1.9694975505204728, 2.0459832321146361]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[0.43894448162140937, 1.0175200651025451, 4.2454918626425489, 4.040617675258674], [0.24902273792048915, 3.8761264247595442, 2.6882216878664362, 3.100469639720445], [2.7193884759210802, 4.2017965795472465, 1.225294635386704, 2.441669988306856]], [[4.1058712058669684, 2.340215625064022, 0.80518901197208159, 0.40674864951457163], [1.9233140518832035, 0.84146140409092252, 1.7823846787340654, 0.25451427575254898], [4.1571413689674479, 1.0566402029253852, 1.9900968118368068, 0.83805678999015021]]], [[[1.827835718599647, 1.7220610702581773, 3.1572936315439049, 3.8437916925820437], [0.875461636950846, 3.5654278054191573, 3.6296434362888892, 2.0299553141111129], [0.5472163185469654, 3.8829939450600257, 3.0846806775441777, 2.6905313067071597]], [[1.0685446387860231, 0.194557256764119, 4.6800073703324623, 1.0814348005918073], [2.6992826396069187, 3.8371432622434072, 4.8766273127945405, 1.224148370607437], [4.0540309260592133, 1.3252621881207038, 2.0702959144968642, 1.6709871069666911]]], [[[0.61961256440892698, 4.8036559956694731, 1.940006666337962, 2.0233977789775963], [1.7350044769470352, 0.17095661839800683, 2.0408394035517445, 1.3170514190165814], [4.6965495731264708, 4.632416057399559, 2.1436211736077246, 1.3508423460265986]], [[3.3001048109105842, 3.7893790036390111, 2.0084983542655523, 3.9141378563705116], [0.29551717680653244, 4.8978980255737481, 3.9709435145136256, 0.35156300110359617], [1.898026469625913, 4.0722633123294285, 2.9488729441319816, 0.54130313027135424]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.1712649410532938, 4.6468470473651529, 1.4718085960626268, 2.5816444198836552], [1.7440734728283638, 4.4910486930062135, 0.89662386141934025, 4.2646666450269741], [0.14110208339996441, 1.2258194356370284, 2.7187090121381634, 0.79370841567908013]], [[2.8691531492027202, 3.9519319095097352, 1.7379905536657421, 2.3197735940788924], [4.0656166323730103, 3.1438920324179787, 0.7091674500490498, 3.6420401582386179], [2.3735911605149642, 0.92845397282162845, 1.1525430536813841, 3.1148696867106547]]], [[[0.78528930609985281, 4.2675074489312506, 4.2822752286716064, 3.9653052449545001], [3.9182331222598532, 3.7304650838884612, 0.24491647480682679, 4.5082092776337435], [2.7921746345169325, 3.9539969909915396, 3.1397871409835907, 4.0506298368304323]], [[1.4096729982538763, 4.6362916386946269, 2.3669193770845021, 1.3791421489064801], [0.58878346541211946, 0.99153418630360279, 3.7247506394927297, 2.0468471696061052], [0.096316273498492908, 3.8049287526568008, 1.3073210379890663, 3.5734356403660503]]], [[[0.68975228626851393, 0.19788957930177278, 0.40514936933869578, 0.012933413454958905], [1.6770139965608712, 4.5236620320416137, 4.5376592694827202, 4.7046251067871587], [4.7951184330186463, 2.6212364306164413, 1.1912011252008163, 2.4175366943816181]], [[4.412110939483826, 1.9921935432532698, 0.32769158248746166, 0.082082626758738786], [2.8047771032889721, 3.6263613822816136, 0.94820670134964402, 2.9474062951886379], [3.014801164533861, 2.3496506541180899, 2.3034043285290959, 3.9587832272815167]]]]),self.functionspace)
      res=power(arg0,arg1)
      ref=Data(numpy.array([[[[0.38121229005218271, 1.084054418963128, 8.3982702723642806, 36.781757238665278], [0.088510840020624079, 439.05989680596542, 2.4269936434074326, 124.67356992079502], [1.1516083334976002, 5.8105704119044246, 1.7374020507363559, 2.0310022609253906]], [[57.537874133007939, 28.792148527416604, 0.68620098042724642, 0.12408664672912224], [14.283651196199811, 0.5811866378318461, 1.5066141827169777, 0.0068482526443579108], [29.42834589623611, 1.0524833630148958, 2.2103720875513351, 0.57677548423290947]]], [[[1.6058184761096195, 10.170421242595655, 137.46880036226136, 208.32974186833857], [0.59384290208584223, 114.71843713577495, 1.3712603176563263, 24.333993736087347], [0.18573524082302223, 213.58139152554011, 34.357055478274113, 55.095335189285329]], [[1.0979643748230621, 0.00050561105766347891, 38.585089851375919, 1.1140156794366243], [1.7943696078234928, 3.793708093776945, 365.65691945506433, 1.5128048032379049], [1.1443250791131596, 2.9197768578718422, 2.5891418109732109, 6.2629654903828378]]], [[[0.71881055148766615, 1.3641935012246145, 1.3079865976722993, 1.0091568572635536], [2.5194773312938978, 0.00033871556648452824, 25.456981854282365, 3.6533056553575354], [1664.4066333001288, 55.621030098300153, 2.4800698667639334, 2.0689098793678862]], [[194.00006892608079, 14.210832562259018, 1.2567486710098459, 1.1185235418508188], [0.032741800696658531, 317.85179648069459, 3.6972177437903446, 0.045907840606081753], [6.902811148583039, 27.095751947775732, 12.072794366630305, 0.088053956541452441]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_expandedData_rank0(self):
      arg0=Data(3.67909205267,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(2.661610404)+(1.-msk_arg1)*(2.71023762049)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(32.0465447789)+(1.-msk_ref)*(34.1422080564)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([2.4243783801409169, 3.8887226747362464]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.42204732755)+(1.-msk_arg1)*(1.9475394273)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([50.202018779975091, 405.65267687068911])+(1.-msk_ref)*numpy.array([5.6107955972073187, 14.082259185679174])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_expandedData_rank1(self):
      arg0=Data(2.41270712949,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.6177366683484862, 1.6594817580478016])+(1.-msk_arg1)*numpy.array([0.86335635056151605, 0.98672524964917896])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([4.1571120392775098, 4.3128009907308824])+(1.-msk_ref)*numpy.array([2.1391329728660877, 2.3846626763033756])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[0.3209085450238634, 0.93250732154240479, 2.1081903994708098, 1.5073688869339239, 0.14691048567796366], [3.2250114362066609, 2.8756152444177618, 0.15828635034752236, 2.0570480259072084, 4.7391641890227634], [3.5501087201406083, 3.4843440791717351, 3.6080468691882683, 4.7560219673357205, 3.5400129663270321], [2.2472521055470582, 4.4058823711305548, 2.9000291444436432, 2.8676095660809984, 4.0141021949341669]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.20155126246)+(1.-msk_arg1)*(0.990215968062)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.026281755365967305, 0.79953986729518367, 10.889650193071754, 3.7203090219390562, 0.0021541543717889361], [42.470648684805091, 29.420503373313604, 0.002735125235241687, 10.066259094028148, 145.64440554083021], [57.759956801152889, 54.403680845359773, 60.832465838705254, 147.30955209671981, 57.235723825427648], [13.360762727220157, 115.31992148426265, 30.227683569928626, 29.159078776040619, 85.589262466199258]])+(1.-msk_ref)*numpy.array([[0.3244971320322485, 0.9331450866626817, 2.0928624555437261, 1.5013288878695397, 0.14969330482841464], [3.188274993905678, 2.846050098901292, 0.16116700531904884, 2.0425826459657501, 4.6675681490217436], [3.5063726601331773, 3.4420478667339105, 3.5630326476403975, 4.6840085205539204, 3.4964987051519216], [2.2295192353233362, 4.3424183346436092, 2.8699755901695343, 2.8382041451181239, 3.9598879442213071]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_expandedData_rank2(self):
      arg0=Data(2.58578657945,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.0479430275357182, 2.2626109943690453, 4.1603620073613392, 0.90488858535026517, 2.7600899792457736], [2.2276569393883383, 1.4758025879429515, 3.8963573270930736, 0.27733855880676295, 3.1695706378367103], [4.5330125424583532, 3.9084440081002718, 0.461965220895729, 4.2519298897409366, 3.7763798273210791], [2.6385363053371589, 1.2891706073672102, 2.1610401679825406, 0.22123448421857148, 2.3572803924821706]])+(1.-msk_arg1)*numpy.array([[0.96754044561744079, 1.7303799430516735, 4.0883955256184326, 2.2498875416545263, 2.5748942852740533], [2.3989910673331836, 0.2799310100328255, 0.6009331885141993, 3.6815492910270171, 2.6493265948500451], [3.9041468951302991, 1.0629901095022976, 1.9114645622421795, 0.25667878955195572, 2.723070226874317], [1.9822489192210173, 3.4543813162674746, 1.7744982543923371, 2.7485593886465449, 0.37547754116728588]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.9978768633573969, 8.5809767915638115, 52.063684351534739, 2.3623835176307741, 13.765529248260352], [8.3007042463022476, 4.0635474989801095, 40.514315002879364, 1.3014511108599711, 20.311483047343806], [74.180162462189614, 40.982210201973736, 1.5509702491165782, 56.795658077883566, 36.149863572910739], [12.264243307483895, 3.4033166005083699, 7.791648427294815, 1.233899329139738, 9.388509018099878]])+(1.-msk_ref)*numpy.array([[2.5072642173561199, 5.1753782112152837, 48.623034028076333, 8.4778772452776643, 11.544696664453035], [9.7680111520896489, 1.304660412810781, 1.7698670052510617, 33.03545386195028, 12.390611844675872], [40.815246073420077, 2.7452502198226787, 6.1469018819763308, 1.2761560653168236, 13.289811920748443], [6.5744796921894642, 26.622720636160299, 5.396907632918774, 13.615558995585591, 1.4286284151002238]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[2.2501997752173879, 4.8823450369336046], [2.8594805210173586, 1.969970538309249]], [[2.6324066210115409, 4.6671152924714159], [1.421081787717462, 2.3850770861793578]], [[2.225841308895431, 3.8787974668517449], [4.1246039828490435, 0.97102009707051418]], [[4.9555141460461067, 3.0820157635941268], [3.7197963220439321, 4.5754155793745719]], [[0.43197455841072135, 1.014027311869979], [3.7326221932224004, 3.3662199056772355]], [[3.161772284707328, 2.3548917111361298], [0.59581145030950522, 3.2390409727938896]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.57058366622)+(1.-msk_arg1)*(4.92989966627)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.5743137382022701, 12.06559167473273], [5.2075941809086483, 2.9005051714370795]], [[4.5729822507264375, 11.240786321187031], [1.7366049085890867, 3.9165090548249846]], [[3.5137326298544336, 8.4061677975632083], [9.2578325449288847, 0.95486252390011872]], [[12.350797196680972, 5.8580900024449187], [7.8713291800644418, 10.895857862921105]], [[0.26758179029216472, 1.0221190563931477], [7.913997314435063, 6.7285483085533428]], [[6.0979353559259746, 3.8389414131771549], [0.44339380979033322, 6.333615960597724]]])+(1.-msk_ref)*numpy.array([[[54.502267355151965, 2482.3856616518533], [177.60280936656133, 28.291562294861833]], [[118.11307834039455, 1987.6601103763371], [5.654532704747754, 72.618797782989716]], [[51.654900743739269, 798.39471441921842], [1080.8683063347951, 0.86504041596190295]], [[2671.2691575587928, 256.98441844208213], [649.53449442988938, 1802.4213847547744]], [[0.015953109667757617, 1.0710855978226141], [660.65054107508627, 396.97170716815856]], [[291.47924775536899, 68.199221648683974], [0.077859062696213927, 328.32340695930975]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_expandedData_rank3(self):
      arg0=Data(0.351995248727,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.2720439333293956, 0.41557208661792383], [3.5185384040900765, 4.5133050356926878]], [[1.0849399606033614, 0.83158659303602211], [3.3039141381694397, 2.7887445476932924]], [[0.3498808081393105, 3.7823259142804835], [1.1673755464423223, 3.9710736566798595]], [[0.47396651882066521, 1.1952901082040712], [2.5057144799718438, 3.8276833297097768]], [[0.33600848208001005, 0.45140429565751095], [4.3482114324858996, 4.9304432734574926]], [[0.53217409266997362, 2.4695199661095293], [1.0655302230983912, 1.5207421955353058]]])+(1.-msk_arg1)*numpy.array([[[2.3690900125007208, 1.4525065829211952], [3.8959600225662689, 1.2183959310644477]], [[1.9497253542272135, 3.2986772046448953], [3.9875041276247796, 0.26012544201343951]], [[0.89593623675650835, 3.5700763958416113], [2.4104778336185801, 2.5930416396772298]], [[1.6721810743408616, 0.92729181111340353], [3.3054361361217373, 0.38251646005237749]], [[0.48579760569929631, 2.0413694190810281], [1.2950271659090946, 3.9139843704991959]], [[1.2364513569543132, 0.46304703284947668], [0.67040440089037956, 3.7303020616757565]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.09326349589394349, 0.64796769210577432], [0.025378872314996904, 0.0089821905094849587]], [[0.32212145724722585, 0.41966822069034104], [0.031753860086892981, 0.054375964979887094]], [[0.69397197038164671, 0.019268787736754483], [0.29555489101010357, 0.015822104100780662]], [[0.60964024132868866, 0.28706478799099783], [0.073071949989753249, 0.018377501798699736]], [[0.70409704062079448, 0.62417269222896987], [0.010672007836118234, 0.0058106594415198417]], [[0.5736917885307431, 0.07588633471672622], [0.32871631277861862, 0.20436164022243031]]])+(1.-msk_ref)*numpy.array([[[0.084276184466874668, 0.21945309938534902], [0.017112974625452689, 0.28022203580835309]], [[0.13057838845063491, 0.031927968293957373], [0.015552980364168686, 0.7621539153153063]], [[0.39239718965206161, 0.024049261138235307], [0.080711795994957453, 0.066703871917394783]], [[0.17447267565833588, 0.37975831283481143], [0.031703437712648018, 0.67072248007935664]], [[0.60215550878843527, 0.11866266386624347], [0.25867403584706494, 0.016793921851834807]], [[0.27498867771443852, 0.61663079704755019], [0.49658806326376342, 0.020344419164947537]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[1.0972374399115281, 0.86547419460803021, 4.0954949877671387, 2.3982969983068041], [1.9773067688589629, 1.2983009755715267, 3.6405309907004324, 4.1747329641224082], [1.0707335114554777, 1.0159905239300915, 2.5107968455656717, 0.31286017495843355]], [[2.1231325072127474, 3.2236061592005267, 1.4132819145395297, 3.3529613548025621], [3.0406062353814383, 0.51231774763032856, 1.4781488105447143, 0.30091286199999212], [0.65668238967557591, 4.4340508301240131, 3.7458719565042173, 0.14297281937039452]]], [[[0.65758753240022672, 1.9754347736788336, 2.2997125315057567, 4.4875118403156788], [1.067348814736121, 2.602465855812869, 3.1517640741569393, 2.4787187181938268], [2.748572452361711, 1.130102281550196, 2.8100658642703396, 0.77138332867698856]], [[2.1730309819610305, 1.5319732616744943, 0.35462609733790967, 1.971931312454364], [4.222619561922742, 3.0399117641087341, 1.4668518220797719, 0.48970242968596178], [0.89495925699988066, 4.8376451011616695, 1.9595281424021134, 2.6587023153856539]]], [[[2.744999435421851, 0.36972823727905352, 3.3841152971814221, 2.4587982496873479], [1.5666376609076567, 2.5259674881097589, 4.4114733150194105, 1.1457837013059271], [2.6440508199169206, 3.6482504721490274, 2.2879402644451954, 4.636893297158732]], [[1.481880319591163, 4.5821099701914791, 4.3975361419207859, 0.060426330260365792], [0.9572100809919819, 4.64568557862186, 1.2209236130862779, 0.20970175590321302], [0.71203921010443405, 0.50051842635591126, 1.620821469641947, 3.4818148248241774]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.13337628295)+(1.-msk_arg1)*(2.81450835099)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.1109020413808688, 0.84895625296848531, 4.9428058300546036, 2.6950883646589281], [2.1655255381253093, 1.3443023896894857, 4.3252458544301371, 5.0513312655104752], [1.0805383604041836, 1.0181425182349801, 2.8388141155511848, 0.26794273155888343]], [[2.3474053498832439, 3.7682763114828748, 1.48001378524014, 3.9401091649880735], [3.5267576384554515, 0.46859628999409259, 1.5572363593135483, 0.2563758524293156], [0.62086175481526451, 5.408396663590028, 4.4673634389652284, 0.11030223964043259]]], [[[0.62183175234026156, 2.163202048834493, 2.5698763384763019, 5.4823617889057346], [1.0766679142593309, 2.956565827691942, 3.6732369159462896, 2.7977429571320642], [3.1453840066731331, 1.1486888354135119, 3.2252593141616361, 0.74513467409858125]], [[2.4100303824031815, 1.6216581158870578, 0.30883092425753739, 2.1588544002599463], [5.1170510673862806, 3.5258447089903355, 1.5437544535645127, 0.44522195877830151], [0.88180988535652793, 5.9696372715536183, 2.1434708756974374, 3.0290788714808077]]], [[[3.1407501988877109, 0.32377880089082361, 3.9816270421434328, 2.7722734306982666], [1.6633082615232631, 2.8582622582598511, 5.3771955268706622, 1.1667707430947523], [3.0101668890958027, 4.3356419361542207, 2.5549715987535939, 5.6896558317046093]], [[1.5616925939578172, 5.6135289697052677, 5.3579455863212297, 0.041559537274506196], [0.95164305660236337, 5.7018847904778625, 1.2538647183176537, 0.17026283965344255], [0.68050517070735417, 0.45638339711827292, 1.7286572008462771, 4.112157085252786]]]])+(1.-msk_ref)*numpy.array([[[[1.2984535748430035, 0.66588802673991065, 52.886091576194346, 11.728434976888099], [6.8124432092376219, 2.0849515315468725, 37.96669781558731, 55.81676500118067], [1.2121002335544084, 1.0456612110461316, 13.343600005011645, 0.037989113534254798]], [[8.3229977666156056, 26.960797787568126, 2.6474039722948119, 30.117785503162047], [22.871543004183724, 0.15222858768597156, 3.0038149447695499, 0.034046081858696668], [0.30615768362573925, 66.134165789540219, 41.140494316261893, 0.0041923146321386469]]], [[[0.30734687666291499, 6.7943063110476452, 10.421571963220737, 68.403012257849909], [1.2013471455106799, 14.760622208700246, 25.303672295137911, 12.869328215621923], [17.213566105213733, 1.410913392329801, 18.319614668740918, 0.48163847557896261]], [[8.8853561846495683, 3.3219340662694825, 0.054053753231582714, 6.7604465984762214], [57.637566650934694, 22.85684351937628, 2.9396489809783355, 0.13406387033112507], [0.73172835654396529, 84.510004165131818, 6.6414491359513583, 15.676043352030597]]], [[[17.150660556365747, 0.06078578449935209, 30.912049423584929, 12.580353704057709], [3.5378600345054116, 13.571763439055239, 65.190766017363202, 1.466712098131526], [15.434120034143048, 38.193717712074871, 10.272119363398014, 75.006990848404783]], [[3.0252062000253552, 72.539472453045178, 64.612757639589717, 0.00037132228834398394], [0.88418834384612943, 75.407973535816012, 1.7538218971366544, 0.012320926085508253], [0.38447758204286459, 0.14256577463548992, 3.8931525167047463, 33.490110698947859]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_expandedData_rank4(self):
      arg0=Data(2.89488331207,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.1469486426840088, 2.175422614496727, 2.0455802947978157, 2.4939546013699236], [3.4960040046389045, 4.2722588065000036, 0.8536865966470476, 4.44864890136013], [0.76198747174925174, 1.5066884445895092, 2.1904397823190123, 3.0263015347179345]], [[3.9612427227044327, 2.9811654323074559, 0.39509391693209012, 0.96036025931555336], [2.7082804699747696, 2.9469909216432901, 1.0772272048068834, 0.062671456318210819], [1.2945913616154845, 1.9918367240224342, 3.8180201901260182, 4.7311853653039915]]], [[[1.7621572119560784, 0.92197668604008365, 4.1282716804272646, 2.4772930867355827], [2.8630077151384636, 3.3923226980743149, 3.3905252800960102, 4.2137648600213238], [4.4311152454700569, 1.1947662742056064, 4.0620240682455844, 0.95792873612054841]], [[3.8962530748230373, 4.5561249651854583, 2.1836404694418095, 2.1395638527213499], [2.4740584920822308, 1.8706096529019729, 0.077574753977072128, 0.63218753671436179], [0.44772574791498582, 2.8325193473571773, 0.98354439391341464, 1.2867979072045623]]], [[[4.910260619560221, 3.2795591147160383, 2.7909441121626077, 3.6947277452195117], [2.2771349956379905, 4.5216776255017237, 1.6660149640611819, 2.2330357300866623], [0.90659023981371245, 3.0165916447401719, 2.5345214994703391, 2.2613048818397381]], [[3.1305411000871839, 4.3235237391685262, 2.959291181552421, 4.9578509122529475], [0.3090873873631127, 3.9333280342284502, 2.7643896690898768, 4.4307508779804934], [4.6083095283676352, 1.8863620214453791, 3.4508901508906522, 4.6981498906363415]]]])+(1.-msk_arg1)*numpy.array([[[[2.5557827252966598, 1.1802401073258892, 2.7155253319612607, 0.81373623282138263], [0.98246845871081179, 2.757185485070091, 3.0939378585728474, 4.6361641710943129], [4.0684268022929944, 4.0905469416740807, 2.5482649369856243, 4.1416101943356374]], [[4.0699070794359775, 4.0896009382972931, 2.6921522274932261, 2.1152683624756849], [1.5704669010898324, 4.4446140805289431, 0.56515329456074737, 2.2090044571174317], [3.9978701659101112, 4.6605397577951031, 1.2850874909562606, 3.7806524279796343]]], [[[0.89623598815528049, 1.9005052380163692, 0.063718029362882248, 2.9136829602720886], [2.438261989150559, 3.0918144704896986, 0.28356264149181004, 4.0053444917969996], [0.65469044723981618, 2.6232380848149774, 2.6267369723137146, 2.1541455090219648]], [[2.6021857286111585, 1.697226355139696, 2.052550052851343, 1.3892519253187079], [2.2190119683157135, 1.4798907919063136, 4.951038651544966, 4.7966701231927429], [3.8411069291675619, 0.17695036013894641, 0.27249412068679468, 2.5599709486737106]]], [[[2.0740194523018931, 1.1913522135942791, 1.5142831722124812, 3.8107216213882888], [2.3363223092629046, 1.6901027819052215, 0.54386984927593285, 4.6780805150235913], [3.0670415315669506, 2.7618774907539794, 4.9807021868117074, 0.79213023687906592]], [[2.6803637267534861, 3.3485364798572288, 3.3792681199789296, 4.93509921880813], [0.96408561264106152, 3.0262343647296337, 0.73256728218050338, 4.1178555633159224], [4.1149633734857831, 1.781912567386386, 3.8945106160301703, 4.6773869391587528]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[28.361503260667632, 10.098169436409798, 8.7963683121478553, 14.167294213389104], [41.10210306896343, 93.800979424756932, 2.4779255335480346, 113.14477940518229], [2.2477969954300314, 4.9605997594637286, 10.260654036933685, 24.947945900212119]], [[67.395777494759443, 23.779272661521567, 1.5219052834976696, 2.7754418149766331], [17.792066433998986, 22.930976592514899, 3.1425452462246937, 1.0688852669436146], [3.9593471892924872, 8.3079467393345965, 57.878426784991447, 152.77846003774815]]], [[[6.508284158799186, 2.6644837589145434, 80.489354235369959, 13.918596355584267], [20.972649271359746, 36.81302724378525, 36.742761035174667, 88.146434607040248], [111.05559412383187, 3.5607478839683941, 75.016446415968716, 2.7682777396339344]], [[62.897219586244944, 126.83783183851989, 10.186764602569063, 9.7205118739188308], [13.870823672114863, 7.3035082137011385, 1.0859527167100775, 1.9581114385923968], [1.6094745465763309, 20.303874321087047, 2.8446880200277898, 3.9266833906324954]]], [[[184.81143026970628, 32.654749713820493, 19.426139205437334, 50.769290655533318], [11.251132727679428, 122.27758354084021, 5.8760334362644819, 10.735904934228477], [2.6212606975577999, 24.691780452883606, 14.791555701470383, 11.063399007933254]], [[27.871158106011048, 99.054191708734521, 23.232756094003111, 194.40077209849125], [1.388942730562605, 65.425402105701082, 18.885484924872127, 111.01259033957662], [134.07222752386755, 7.4268269494427699, 39.177623445806397, 147.5067610456357]]]])+(1.-msk_ref)*numpy.array([[[[15.129643525283283, 3.5061903896221871, 17.929610076407684, 2.3749032506752124], [2.841436524544354, 18.741418799831159, 26.80759302264056, 138.1011812341624], [75.528731076563872, 77.325641534444742, 15.0092244093694, 81.638668973488791]], [[75.647665521609596, 77.247925864068208, 17.48964900406849, 9.4726956091046191], [5.3085547952267715, 112.66056409058605, 1.8234435204585, 10.465140783541152], [70.071441743527572, 141.72612768960548, 3.9195508594137354, 55.624561214096332]]], [[[2.592569286006519, 7.5393213156260863, 1.0700750093367863, 22.133319917443071], [13.352957701007654, 26.747155327775257, 1.3517653924123487, 70.630362359063781], [2.0055128403916354, 16.254296772669615, 16.314861075804988, 9.8723485631728138]], [[15.894605866305097, 6.074246414661947, 8.8617779195512885, 4.3784645990641406], [10.57705720358225, 4.8212933744137985, 192.99818922343121, 163.79171359112627], [59.316330150369645, 1.2069402835495879, 1.3359547563973275, 15.197148582679176]]], [[[9.0663358713168467, 3.5478494975065114, 5.0008076477232901, 57.431144601207713], [11.981714044305068, 6.0284261262234038, 1.7826546506386751, 144.39337902928867], [26.052034321467545, 18.835122155974588, 199.18051164640431, 2.3209828053886343]], [[17.271861852813359, 35.138923602497819, 36.3057266496849, 189.75581651956307], [2.786453922986142, 24.946164730606547, 2.1785915408249821, 79.603110808840071], [79.358767665623731, 6.6463956738542063, 62.780833101155245, 144.28696671545177]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([0.4128146823408807, 4.9141646393647571]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.3292069727929821, 0.34931175272752346])+(1.-msk_arg1)*numpy.array([4.8022023358173742, 2.7998432196896879])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.30850275479044403, 1.7439398735512845])+(1.-msk_ref)*numpy.array([0.014281650890452522, 86.288071236665175])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[4.0813804187804097, 3.8062637662049781], [2.160989453416617, 1.3470978962746738]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.3752044659429163, 4.5550475108195787])+(1.-msk_arg1)*numpy.array([3.6918561992608354, 1.2591866006753785])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[115.23881202090389, 91.054709811079348], [33.447118454259844, 3.8852530687717071]])+(1.-msk_ref)*numpy.array([[179.8920173371049, 139.0331592630335], [2.6387006825789725, 1.4552502607723752]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_expandedData_rank2(self):
      arg0=Data(numpy.array([3.5859389719882331, 0.83117531757622531]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.7901592456290283, 0.60461008744538625], [1.5068958884705301, 2.4786435933521065]])+(1.-msk_arg1)*numpy.array([[3.9918708063823294, 3.6655508391142182], [3.5313249339307133, 4.2881471653303391]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[453.56149814006091, 2.164305207210814], [0.75680704537852128, 0.63233420045048061]])+(1.-msk_ref)*numpy.array([[163.64513636805876, 107.87606693295417], [0.52048547011714952, 0.45251211748618431]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_expandedData_rank1(self):
      arg0=Data(numpy.array([[[3.8261924506423619, 3.6181960290058863, 2.8385978731813322, 4.3935059078924379, 4.8843030644701368], [3.7188264262523059, 1.9361808847463897, 3.1007666534619154, 3.1119662443958598, 4.8330230194647541], [0.54310572572022808, 4.4296649206119199, 1.9031749003709573, 3.0871113016186897, 0.70054653275921464], [0.55308500799101101, 0.57905373457168829, 1.508232047127156, 1.1552556878543518, 1.8590449554174859]], [[4.8207077104954301, 4.0770737051076518, 2.1537855622046638, 1.3983944090557086, 4.9279607575328717], [1.4378935790126173, 0.62215782126066776, 1.825016775783429, 4.4080321973818792, 4.5771807296836089], [1.9649918528417882, 1.822644357817313, 1.9202791482614663, 1.4004603296632974, 1.6848193399359963], [0.19108337833033795, 1.6456733935953154, 3.4997162386713634, 2.5400547243140528, 1.7241229665073463]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([4.2414147611586994, 1.1684568042289354])+(1.-msk_arg1)*numpy.array([3.3418187510703028, 2.9159650823070273])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[296.3164433641403, 233.77421415768609, 83.522109646039098, 532.63410936555783, 834.63680138455777], [262.6201468551879, 16.483775664873104, 121.48544006844068, 123.35745685234336, 798.09756950291262], [0.075081804498888099, 551.47645169916723, 15.32445682191657, 119.23240368062314, 0.22102131533116171], [0.081109871063966679, 0.098535403552971107, 5.7142272420298905, 1.8443523098792221, 13.872991589532189]], [[6.2832662402696045, 5.16613714308816, 2.4509400196838027, 1.4796601896653772, 6.4469121247859373], [1.5286106732624134, 0.57435672824419604, 2.0196636139320847, 5.659423204418208, 5.9139860948811585], [2.2018076138460607, 2.016596227621462, 2.1433792480444538, 1.4822147291273604, 1.8395762358496324], [0.14459058657664098, 1.7897328407223756, 4.3219413096050427, 2.9719509933706494, 1.8898170643072858]]])+(1.-msk_ref)*numpy.array([[[88.613737578265884, 73.515573163408462, 32.673150941779738, 140.65609977413436, 200.37957731656488], [80.573672782179983, 9.0974836405369235, 43.893539536462036, 44.425589433310464, 193.43516507957546], [0.130026434725134, 144.56205456737595, 8.5894857806750462, 43.250886604918065, 0.30442367754841232], [0.13818379115068688, 0.16108290760479205, 3.9482828606972058, 1.6197907010198112, 7.9417828954505447]], [[98.158347961646285, 60.221989220189919, 9.3671342086023479, 2.658588113631815, 104.66307442845223], [2.8835380003111446, 0.25062318470162087, 5.7788981929011562, 75.612782297221969, 84.388035886962612], [7.1685235342769618, 5.7570200352679519, 6.7031756071905049, 2.6700572790387316, 4.577429098978822], [0.008018093316589165, 4.2741575098811548, 38.581645263172803, 15.153336444561093, 4.8958118442126208]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_expandedData_rank3(self):
      arg0=Data(numpy.array([2.894940595988106, 3.6123537243453434]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.4892097204116364, 3.7053573653092355, 2.8099733876937822, 2.1590196115237275, 3.2169969989335088], [3.7040254861275947, 1.4765275262790263, 0.33606405276716733, 0.59536749100378372, 0.17259703383217961], [3.1220192145848191, 2.8743583878897607, 0.52772109937725209, 2.461778872849004, 0.49956404311148311], [4.8714124757723694, 2.6883790997270891, 0.12400283576828951, 0.60091327315086596, 0.42321781340168507]], [[3.5934880443461039, 2.8841835966530085, 2.0380791937857454, 4.7525550401655901, 2.1136651975948166], [1.8768258400543936, 0.8133772656479995, 4.9585225235529267, 1.3050165873997137, 1.1085838138739461], [4.9191869234841503, 1.9801485265532199, 3.5143065612255491, 1.1142609587899972, 4.0432463176367461], [3.0309256520102807, 4.5745405751732191, 0.83164118637325168, 3.4661999277915676, 1.8968254441501764]]])+(1.-msk_arg1)*numpy.array([[[4.3463559792173108, 1.2283482731102804, 4.6222544782916115, 1.6350817146278247, 3.7933790657505244], [4.9291523024272639, 2.9409748652608481, 2.9461252684525152, 4.2425778431253258, 3.4627101090752341], [1.373494503361447, 2.4176345272006232, 1.4868662882032828, 2.3273489576691957, 4.7843558123861829], [4.4921372622179536, 3.2597110112651366, 4.8208977028415116, 4.2621410907561206, 4.7049065825515548]], [[1.9834793183209261, 3.9747530540328926, 4.1850016246899662, 0.92524639269813491, 0.17168443278908971], [2.7215288982621986, 4.3914917442854176, 1.6119636645082758, 2.6649437760050425, 1.3158523437496106], [2.9018251212758384, 3.6331591492200999, 4.9196872810601215, 4.4672597598830075, 2.5798810009282338], [2.385562976514402, 1.4254107372559968, 2.9283902657884604, 2.1969677551136511, 0.89149961221847362]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[40.809152588266535, 51.349935352406661, 19.824176210568403, 9.9240529261164028, 30.555781051338432], [51.277288610477186, 4.8042285523144939, 1.4293562791741543, 1.8829779917554932, 1.2013723586173641], [27.62153931824669, 21.22842719762815, 1.7523344014640869, 13.691617351819016, 1.7006642140683674], [177.35244569354927, 17.420571726119658, 1.1408922407900597, 1.8941108723874573, 1.5681013401709298]], [[101.02123349926924, 40.62277127795285, 13.703160488136344, 447.64007497255739, 15.100171561875678], [11.139743534216366, 2.8424571390163647, 583.19745218041157, 5.3447273420864185, 4.1529553802741237], [554.46559498880652, 12.720599565492265, 91.252759445544598, 4.1833473017550711, 180.00453875732609], [49.047947501306858, 356.15104526067296, 2.9099220943182504, 85.785244647708311, 11.429593294719076]]])+(1.-msk_ref)*numpy.array([[[101.49631648819218, 3.6902371345499145, 136.08678978383699, 5.6861532584877512, 56.386379338550356], [188.57849938769641, 22.786132919824194, 22.911222176076659, 90.895457101224522, 39.675673268640786], [4.3058562873568915, 13.063994009317387, 4.8573169440892636, 11.868520199590403, 161.67704898251748], [118.50828206415309, 31.97509705580925, 168.08058825312017, 92.805421792399784, 148.58379907191232]], [[12.775134018323801, 164.8460631641471, 215.94998929597477, 3.2816588805971629, 1.2467055881880069], [32.96418474261872, 281.53433429322524, 7.9275255044523831, 30.653474561145419, 5.41962995250364], [41.553711320939499, 106.30185015776199, 554.82203074212975, 310.30875462214198, 27.48099926399389], [21.411338485039813, 6.2385008734110254, 42.995954783355842, 16.805337456279513, 3.1424603707380241]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_expandedData_rank1(self):
      arg0=Data(numpy.array([[[[1.6130764817774108, 2.2006173161154434], [4.8292098043241252, 3.6944284237484535]], [[0.7819957340312218, 1.4763846521865889], [2.0874160384161931, 3.3869844174352171]], [[3.1611809869576373, 4.3964170418659574], [4.4997668843096594, 4.8997120288493683]], [[0.70620169093169116, 1.2903085549201025], [0.8545699074358295, 1.0503850237129453]], [[1.7091273243803042, 4.8959935560395902], [2.2958886222904002, 3.9940909607964583]], [[0.29924635948104433, 1.0088844343293746], [0.98836951312102339, 0.19873904257939623]]], [[[1.839243788735123, 2.4034070773473024], [2.0014256500640455, 2.7805844902232755]], [[3.0021400904888624, 3.690200054479825], [0.69169745940457417, 0.37402294792059276]], [[0.4605513143833041, 1.2653757057613844], [2.9297101405360726, 1.1702207949998666]], [[4.0868854641610355, 1.4801616719673576], [2.066250563579521, 1.3963123741136751]], [[1.5797872297542774, 2.9155913706205223], [4.3510767088620952, 2.2151953762255339]], [[4.9797799418947557, 4.5406005068432718], [3.4130357288376802, 3.2316385017517044]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.56099850685115249, 1.2571096338426169])+(1.-msk_arg1)*numpy.array([2.4284455078580911, 2.8089167638721828])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.3076578405821775, 1.556563948699732], [2.4190979654262832, 2.0815802741336449]], [[0.87113972558434327, 1.2442873304998492], [1.5111242698612457, 1.9825512063471764]], [[1.9072812760107229, 2.2949726429459378], [2.3250838696316882, 2.4388475361256932]], [[0.82271469783038409, 1.1537160416720875], [0.91560994899946901, 1.0279606454490318]], [[1.3507845997577843, 2.4378090211173378], [1.5940167061578028, 2.1746760553213575]], [[0.50822143907712614, 1.0049744757928927], [0.9934585637639668, 0.40395958670830362]]], [[[2.1511991177353775, 3.0112146354042348], [2.3923060574406319, 3.6168272407821527]], [[3.9827500513124638, 5.1623079227798678], [0.62915418287094982, 0.29045996052862344]], [[0.37731455859018981, 1.344315197190423], [3.8623334931234443, 1.2184848206211778]], [[5.8693142115643822, 1.6371831961757919], [2.490115931734707, 1.5214546313455888]], [[1.7768886443937417, 3.8389491234103152], [6.3501811412945415, 2.7178206769764737]], [[7.5243605600994368, 6.6998248839881471], [4.6796834774280969, 4.3691834154236293]]]])+(1.-msk_ref)*numpy.array([[[[3.1935870107451696, 6.7897020292951256], [45.788432783928734, 23.892279494917393]], [[0.55036735493705335, 2.5756794839626522], [5.9724576536033087, 19.347394274597416]], [[16.36277414225621, 36.45277480952614], [38.568820557030811, 47.428739891088838]], [[0.42966671264774214, 1.8570060520385183], [0.68273592429964702, 1.1267919091792242]], [[3.6751916266301907, 47.341376685330758], [7.5257430681400939, 28.874250635751547]], [[0.053402879729046948, 1.0217124438519694], [0.97199022214822528, 0.019766008279593697]]], [[[5.5379593135512497, 11.741194634038278], [7.0216222999220443, 17.682404673564832]], [[21.931271461895367, 39.155790912757055], [0.35508953716047026, 0.063140313128432615]], [[0.11328593742119797, 1.9369840515665144], [20.477248336586065, 1.5551009964651263]], [[52.161557307493823, 3.0087370791203694], [7.6793200916178126, 2.5541359999499158]], [[3.6128250680272722, 20.2012617142253], [62.196377105911438, 9.3375654517865243]], [[90.866810177167196, 70.10944760190975], [31.444755915768621, 26.97280849127322]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_expandedData_rank4(self):
      arg0=Data(numpy.array([4.6159960064141421, 3.4709016215321506]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[1.0548638671893895, 4.598786643640409], [0.14650738064629162, 2.6689598147606266]], [[3.722010632219527, 1.1352937366824203], [4.5793834338877186, 2.6778612797752519]], [[4.186822825551209, 2.5594510559890189], [0.39599310323006365, 2.7794864908340573]], [[2.0072444880725175, 4.1554301681806516], [2.7180288059792308, 3.4068786982665391]], [[2.7011722626772614, 4.6787640303007025], [1.750541636160694, 0.82487501048843404]], [[3.4618359286373619, 3.6761681913233804], [0.35732322572761582, 0.87119491268266736]]], [[[0.48875331018126811, 4.9567358345421795], [4.5353305633496257, 0.20970650991092399]], [[0.30699447239585254, 1.8935921009464434], [3.3353492451975031, 4.8734116685266375]], [[4.2468735015150507, 0.42110685914486029], [4.0744855887724274, 4.0159703478559354]], [[3.7299387284230354, 2.4459309198040087], [0.26683246546667289, 1.3676863128293342]], [[3.5191743665376798, 0.52877863684098547], [1.3872737068698417, 3.9763532773286059]], [[4.726934572897127, 4.2418032568388329], [3.2153869818191869, 1.401524939435401]]]])+(1.-msk_arg1)*numpy.array([[[[2.5479787633787847, 3.7161191322512148], [4.1081208244302463, 0.38055577882208097]], [[2.9013145489122647, 4.5111613981593353], [3.8283487350503935, 1.7753675990991189]], [[4.468281398028279, 3.3018197565563834], [1.5649370685554724, 0.45228142267368104]], [[0.064878274080837517, 0.47476353072692995], [4.2115503801301477, 4.0953936779999749]], [[4.3111841632484218, 2.5558567054963772], [0.71692036800773729, 1.1552982008688946]], [[0.91142847548920991, 1.4712283183764314], [2.124775483679056, 3.0378774807455247]]], [[[4.8998525939516648, 4.7740102090039551], [0.41649981586304557, 3.6969984671311478]], [[3.0646167870834393, 3.6039182332731032], [0.5076032721256627, 1.966894308483212]], [[4.4485606299054155, 2.2005916876395579], [1.24528623340519, 4.6967679417689432]], [[1.9306036040662951, 3.3117683387823655], [2.9817934979742762, 4.7154267021783198]], [[4.3341470447476622, 4.1544888485093576], [1.2697528896716428, 4.0562296548136096]], [[3.6049871362542594, 2.5477434055731019], [3.7015435045095679, 3.807047381211516]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.0200679359329623, 1134.5273284640239], [1.2511799820564304, 59.278570771087068]], [[296.75836679288398, 5.6772285202153725], [1101.3518363541818, 60.091169936183476]], [[604.17345637214828, 50.136595718441171], [1.832502293159467, 70.196712508141488]], [[21.544831991422988, 575.84895662970325], [63.898778820673222, 183.2617057009893]], [[62.272362084662589, 1282.1569012703158], [14.548703157382173, 3.5313108510521922]], [[199.33240762136285, 276.66321781256801], [1.7272589996333076, 3.7905711984192663]]], [[[1.8371430120276153, 477.3420965098436], [282.54296635799761, 1.298178073232896]], [[1.4652536100884328, 10.55303256820663], [63.469318003480076, 430.32624304722646]], [[197.32863023627104, 1.6888223246827392], [159.22981129960513, 148.04722411866871]], [[103.70905602866175, 20.983819528983062], [1.3938227618572681, 5.4847336535177149]], [[79.783062692303389, 1.9309647464703261], [5.6200657805225509, 140.92548168119956]], [[358.62074865360324, 196.08750765402385], [54.667743703559744, 5.7206235259624103]]]])+(1.-msk_ref)*numpy.array([[[[49.264513077473993, 294.09622675087547], [535.65186700709421, 1.7897404179295975]], [[84.575181835381571, 992.22128845927568], [349.17223059057403, 15.111769188177197]], [[929.23345934796657, 156.05752032787672], [10.953012568592914, 1.9972614113853597]], [[1.1043237039778278, 2.0671357514298214], [627.46181397016016, 525.32542392246921]], [[730.75295112873687, 49.861718120103262], [2.9938181193866482, 5.8536216692558343]], [[4.0311635930822289, 9.4904346563154309], [25.787835336947762, 104.22141711972239]]], [[[444.72098875101307, 380.25687789988075], [1.6791678876671541, 99.543835531686753]], [[45.315685934236768, 88.656355543738556], [1.8807465856217538, 11.560933215550243]], [[253.62400581145724, 15.462967478675594], [4.7098314078685384, 345.40775914095843]], [[11.050447369235794, 61.633908929826347], [40.877784422742415, 353.52169650211226]], [[219.96643489211314, 175.89825314352379], [4.8554353690955629, 155.65321714973678]], [[88.774360992485796, 23.818165952504348], [100.10844167544609, 114.15355981753133]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[1.7820765876179827, 3.8353852200292451, 0.97124555153006109, 3.9805081866932426, 0.24677642866184571], [4.6664409686757491, 3.6707002133791491, 4.2051996485844674, 3.5802367243435049, 3.9570699113089782], [0.28778141756913606, 0.52878818734997601, 4.359077632401287, 1.1951737555909725, 0.25022482720254491], [1.1356121603810467, 3.2334671797772145, 0.04717483875775802, 1.7055464618420202, 2.9604302044451423]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.9135329531446712, 2.2424729757561059, 3.0684184167133246, 4.5675933930067831, 0.028729870966215376], [0.57754917213139667, 3.434758508226893, 0.48391820785691486, 4.9322483033931208, 2.9735405809476063], [2.1217757278900544, 4.3876681400683513, 1.6169062821166114, 1.3959293199648721, 1.7232493003296676], [1.8459415332051214, 2.4817498108048128, 3.8418795130873575, 4.9301719636098769, 2.8089903038839075]])+(1.-msk_arg1)*numpy.array([[0.20878668719935178, 3.7465817686345857, 1.9017881456335066, 0.70066998392011748, 1.0644652069719456], [3.8298418081325547, 0.61986581929126683, 1.0876911346796816, 0.89629502111111969, 0.71447662627207553], [0.58356768862661501, 3.6543461958484449, 3.9780063158779853, 3.1147490982545092, 1.9512950856655822], [1.5731112475210354, 0.84985994271501608, 3.9913695312613866, 0.7993801327846104, 0.94671235004785381]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.5941962270414773, 20.378672212433848, 0.91436627874403897, 549.88924657287998, 0.96059641831167009], [2.4342892503002393, 87.051477741848387, 2.0038338235458406, 539.54820844011067, 59.74684928337598], [0.071162708194531146, 0.061073404205340999, 10.81038475492295, 1.2825912425384447, 0.091869907551171637], [1.2645948661253039, 18.402193672651922, 8.0270367482938741e-06, 13.903597834622714, 21.087793369087439]])+(1.-msk_ref)*numpy.array([[1.1282103625992959, 153.91754209914131, 0.94602480434616165, 2.6324426239569365, 0.22549058092472299], [364.84608895459081, 2.2390745320038192, 4.7696572979709222, 3.136666440668554, 2.6718345554442338], [0.4834220969550217, 0.097448620811111192, 349.55520349412524, 1.7425220688636285, 0.066983072158411303], [1.2214705157984336, 2.7111079204990136, 5.0849676982377291e-06, 1.5323117054845081, 2.7940709857910857]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_expandedData_rank2(self):
      arg0=Data(numpy.array([[[0.92677450891339386, 4.5659203479455126], [0.64964972401730314, 2.124191523511552], [4.6028129023358195, 0.52618757698416274], [3.5753485545391603, 2.77077620765417], [3.9332970376239005, 0.32468600980096701]], [[4.4841675203501259, 0.88302139917555122], [4.575560013484548, 4.8036303119098491], [3.6772124498906509, 1.3575230673907848], [4.9574656632768921, 2.1779128115771704], [1.8500155527704767, 0.5974732600603333]], [[3.857405023612019, 0.95182321664899039], [1.3551471830835788, 0.37490043048088217], [4.7705949350866863, 0.93146174350407718], [3.3770532514519682, 1.5264729126040393], [2.0938725563913669, 1.5267751864448265]], [[2.8071601625013445, 0.052099313957997606], [0.45248153892968157, 2.3318084429512678], [0.13986781231429501, 4.1188225524139188], [4.2281799005143501, 3.6780774842945889], [3.795159495203869, 0.11881256402295316]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.71279991553043098, 4.5084796823906954, 2.7868009254588979, 3.187128937338449, 3.8699861080044471], [4.9044219025010483, 1.3056472263946834, 3.8189603891907034, 2.6090826949096599, 2.6068972851167507], [0.099857838001435356, 3.0795743690902788, 2.5859402545143197, 2.1008816117923739, 3.6557760045474779], [3.671648047122706, 2.685000743390356, 1.813106995717755, 0.10237541988609916, 3.2502195862522876]])+(1.-msk_arg1)*numpy.array([[4.6432386058435604, 1.257375192003771, 3.9161709154119011, 3.025435343311198, 0.66766198791634068], [2.0465584747239824, 2.3638506918539144, 3.3765473478258241, 2.3564750755876007, 0.078865300803989666], [2.9402055806651481, 3.8850300447764883, 0.97533283876229493, 4.9777459861458029, 4.0971379767152731], [1.1691804388360094, 4.9939520827169286, 3.6201058562869344, 1.2868000951525354, 4.7628600341729426]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.94723803230249182, 2.9519686341471632], [0.14304353753948748, 29.863865437421488], [70.422916470226582, 0.16706071633102809], [58.00922217264467, 25.741146611854095], [200.30894374360463, 0.012863845807866488]], [[1570.8062986107402, 0.5432754863634891], [7.2829480315428663, 7.7604950698814523], [144.44203486115157, 3.2133376840831747], [65.161352569121703, 7.6203618075969253], [4.9716296722792643, 0.26114711659616469]], [[1.1443165399330739, 0.99508155890079542], [2.5495418185719538, 0.048735165696952083], [56.852301943356899, 0.83226683812361513], [12.894234789717267, 2.4316947486428515], [14.904664464811903, 4.6972164750725529]], [[44.246544566186657, 1.9438320361365666e-05], [0.11892908186083978, 9.7108138821540564], [0.028254975457013352, 13.021151017427195], [1.1590514726826173, 1.1426301370985361], [76.317616054579844, 0.00098423668724647163]]])+(1.-msk_ref)*numpy.array([[[0.70251042433418287, 1154.3763166180361], [0.58138998508786999, 2.5787260382230128], [394.9232015876554, 0.080898205972228229], [47.20945682052573, 21.830418883104723], [2.4951489178320041, 0.47187047026187651]], [[21.562790793498689, 0.77522355401155563], [36.407563144051821, 40.844042113323468], [81.189663298803595, 2.8069021085786221], [43.487274434066379, 6.2601391731722194], [1.0497137033221746, 0.9601946660516173]], [[52.9454375430279, 0.86487052274331966], [3.2566522655198402, 0.022113150505862524], [4.5902261675935838, 0.93309450925124482], [427.49147118681014, 8.210274751095838], [20.652728536002808, 5.6617659596229837]], [[3.3427602738628832, 0.031604220268765637], [0.019058434406302015, 68.586805588195219], [0.00080799363577159868, 168.09068017817694], [6.3934360943295054, 5.3436867863658977], [573.84099547386165, 3.9237284867864422e-05]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_expandedData_rank3(self):
      arg0=Data(numpy.array([[1.6955458510822479, 4.2831821020195724, 2.7671673398492524, 4.2035947812676193, 0.18961403969827006], [1.4127620559227727, 1.608151968203652, 0.16417610392128873, 3.6249174801977793, 4.4039318093651127], [1.2423629835966568, 3.3122265956291486, 3.1411190838057981, 4.4735879848252234, 2.1214800374249214], [0.018466793134949581, 4.3481017816868013, 0.29044522887871005, 2.8695841351536617, 0.7204903051199274]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.566306631330395, 1.5955580546193973], [0.14214496612208835, 1.400864663584565], [1.4744009112354102, 3.7264562495666329], [4.277681646784516, 4.496456934674355], [0.85306909388263363, 0.40288448494781887]], [[3.2329044912994331, 1.9872005948664107], [2.2457441410122421, 0.30446986103565588], [4.6515288393559446, 2.3185846913497721], [2.6453759515739672, 1.7770005773833253], [4.6368839152457682, 3.1111149081860203]], [[2.664674613185273, 3.199871607075008], [0.2223848233305846, 0.010422701564148819], [4.5481660410509788, 3.7567116501384752], [0.58622325755174143, 4.9869205565471155], [0.99891129970773396, 1.3408610004592454]], [[4.2941172048878471, 4.458474437268217], [3.5224875497531767, 1.0967619520940366], [2.1042779332915877, 4.876463963684234], [3.1813331651312349, 2.5196167925305537], [0.34095754367964931, 4.2088749126250766]]])+(1.-msk_arg1)*numpy.array([[[4.8792851032910578, 0.46077500763566415], [2.4614121174698895, 3.8481424174376677], [1.16006369613799, 2.2944516173082952], [2.5345448541015956, 1.4203463471892948], [4.8876072491776181, 1.667147633263119]], [[1.8866755487504776, 0.50932711145214948], [2.6046995989149089, 0.43717598703106691], [4.7375930905055226, 3.2993008460042543], [2.6852210733419732, 4.4930430253432485], [3.0802842128995365, 4.1049391403386704]], [[1.225854132657668, 3.2093737247030041], [1.8840628562014023, 4.0259491261940816], [2.6760910870207306, 1.2466215900565343], [4.8128919650819277, 2.2809745677729536], [4.6055975564595562, 3.0900402441730082]], [[2.1011619891587894, 1.5965902012823368], [4.5783316897084241, 0.075281292028366059], [2.9197864413235144, 0.73817417295783982], [2.4194332689621989, 4.0332868029386493], [4.9059874036872717, 3.6243435794946124]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.8768463518002632, 2.322077343898385], [1.2297092314184721, 7.6739580402075802], [4.4847416580784607, 44.383920880295769], [465.21256886685768, 636.919095195419], [0.24208820706662335, 0.5117586884179518]], [[3.0560384597765595, 1.9870886829957317], [2.9064164696599613, 1.1556341825717031], [0.00022386777351662552, 0.01515757240342607], [30.168506320357064, 9.8599131637786517], [966.97235009618851, 100.70755326817411]], [[1.7829599725790364, 2.0025488032434327], [1.3051691599165993, 1.0125606734818688], [182.31499665882058, 73.689016757164509], [2.4067427380480866, 1756.9916533465628], [2.1197436239230907, 2.7414316892767219]], [[3.594858876751826e-08, 1.8653088394202943e-08], [177.17512670055424, 5.0125974262669493], [0.074154573745596114, 0.0024079759690064637], [28.607153704756467, 14.240601912566394], [0.89424648301061727, 0.25163709092859149]]])+(1.-msk_ref)*numpy.array([[[13.148209495168116, 1.2754402092855059], [35.895318789432373, 269.85322765646862], [3.2567861826995168, 10.333058582733562], [38.071062862751418, 7.6870125140713519], [0.00029547080574480745, 0.062532169727549464]], [[1.9192500233904113, 1.1924336455561231], [3.4468317100972001, 1.2308389972078171], [0.00019162749306137728, 0.0025767449777152861], [31.756973755502226, 325.79959287940028], [96.208178399979573, 439.46715322366776]], [[1.3047728671757275, 2.0066825276400291], [9.5485812559873242, 124.15860394204689], [21.391593440939431, 4.1655864005432743], [1353.7452757955598, 30.488057180113415], [31.942400288414603, 10.21709659531296]], [[0.00022772341218856411, 0.0017066231438588643], [836.27237644202285, 1.1169970502542452], [0.027055917612671718, 0.40146518972971623], [12.813342333719977, 70.228789068797909], [0.20022829756490468, 0.30478631853414856]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[3.3822022388694379, 0.44415750370886542, 2.7128676406464338, 1.4454501150558801, 4.7769046218742792], [2.4859273708158383, 1.2702535015287415, 3.0107143953668003, 4.8096594071984118, 4.9211996207502446], [3.1026472893828227, 4.1512669605860255, 1.7074936825306426, 0.58638658161640966, 1.8061222003405857], [2.4746106269959891, 3.3028128400875181, 2.5791983724431349, 0.56958042961688604, 4.1078651449746681]], [[0.46555034277203944, 3.1129784302399699, 0.10864629357265747, 2.1133881467906677, 1.8700889860311434], [4.2021917944547056, 4.1412926457013421, 2.2228282526397352, 1.8600159822930302, 3.9022034240869448], [3.7208398664720983, 1.0306679207102576, 0.71714970028366243, 4.2821221668331546, 4.6359134383054279], [1.1775705712368458, 2.373589074690631, 3.2947672139786977, 3.1687924336744455, 3.3506416782361153]], [[1.3621626537268603, 0.078036167315166091, 0.72536367942717861, 1.5382951765805966, 1.3905493843383938], [4.3003452066623442, 1.078691329280504, 1.0225623423048915, 2.9568809764389163, 2.3826907657241798], [3.0521802000779852, 4.7652859535325698, 4.6121638649807792, 3.7693281815819057, 1.6774658597555745], [3.9025987941704394, 3.0400428965578685, 0.16728269645991778, 3.5768130000768825, 2.132588060053477]], [[3.2743483086623471, 4.9783107476765682, 1.9884664170817816, 1.0064994073784068, 1.1695793403743877], [4.0785971872126305, 4.9798724112570447, 3.1210380106318882, 1.3879579172149485, 1.4628238734244257], [3.3450340576337694, 3.3875211884392114, 2.3673447049579055, 1.5357570779371363, 4.5871318390219287], [4.4107510907179073, 2.7190560250727658, 3.8405044351791489, 3.7352936645545078, 4.7901088226622335]], [[4.5091374477887598, 4.0128613153881192, 1.9415017582113949, 0.75484736681875242, 0.69978592531822248], [1.0715856542126008, 0.019448986300631917, 0.6917078712586362, 3.6054077335244568, 2.7970644891969281], [3.9645535588423457, 2.3293568431422109, 0.4759079891430914, 2.3184287142388174, 1.499887308141822], [2.6592447412628766, 4.9010599157874468, 4.877737169117137, 0.78221927502554667, 2.7197037752874422]]], [[[1.7125459327037373, 2.8846927478411968, 4.1955107834009588, 2.4950667626974239, 1.6727601611793708], [4.6149017498325762, 3.026961587284795, 0.78777504786223917, 0.95522757149669735, 3.569517304723945], [2.5359853001739556, 2.060732542251746, 4.3881478491811148, 1.343437263218761, 0.40574804829826483], [2.7518206623716748, 2.5341460332741295, 3.343694588384015, 1.4404565982911917, 4.0551380392158149]], [[4.7940914686847922, 2.9695779126925888, 1.019322919072349, 3.0116586872159834, 1.8515152972676838], [4.6839718585110415, 0.95424105435492423, 1.2204052736658162, 0.55267075692303813, 3.6776091177922519], [3.4278827936019827, 3.2844011234907051, 4.1346919729056024, 1.1314082190654271, 4.5211085174325705], [2.5097516805403886, 4.5165958533945512, 1.0348479457298529, 0.82909663675478029, 3.5834171583350085]], [[2.6086542731637925, 1.1644179816153977, 3.2801622304744504, 3.6193288620835706, 0.75355468453896768], [0.80550202992136055, 0.64305600105671867, 3.0576838582312305, 3.390968165737001, 2.2171303267287605], [4.39816035874001, 3.8982565616298253, 0.7192001529216846, 0.37458518126217932, 0.87793376407054369], [0.51486562453605378, 0.56616525833216869, 2.1166952355006643, 3.5997300075815302, 4.0256083996238088]], [[3.8433623260678296, 4.3295136926202389, 3.0138047245863331, 4.4784445814294243, 4.5889709132662624], [0.054330961245888071, 2.3696827500170685, 1.8395269162901944, 2.488492767156246, 2.829014397173637], [1.9399824283074218, 3.0699652313328212, 0.61238142969393405, 4.1107289159308653, 2.6050740558153755], [3.2749453710702658, 4.6897806712718344, 4.3219095562102048, 2.1503689793842025, 1.0557883171020561]], [[4.9951544142298676, 1.0383779422635022, 0.41261540659366513, 4.5580360940637474, 2.2281666667096172], [3.7150592918136498, 3.8177851316262719, 4.5217327385746442, 4.2463733824262393, 3.4656315681249126], [0.50783141132903276, 1.3658622378803773, 3.3545935333876362, 2.9053744754267621, 3.8519981836594357], [1.4162670785907245, 0.11067531279686484, 4.6963951312188597, 2.6936750091478907, 3.5519136788718351]]], [[[3.8803688441405777, 4.3364800344769714, 4.8567903110138957, 3.6399173432881882, 2.4748083761379021], [2.5583726284605652, 3.4088006267717641, 4.9752501261383122, 3.383357152868042, 3.2285050206833836], [0.70266067738072457, 4.3422206728505834, 2.5732098566581687, 4.8147286047773878, 2.0687021001184238], [1.5070970414545399, 3.3517880329575473, 3.8593267463968779, 0.39067465335152657, 4.9736220492558454]], [[3.7286280540141195, 2.3635299721875196, 2.3738071169101764, 3.0061958431485278, 0.36837246591359363], [3.6738144720381194, 1.7825872329258559, 4.9904980859495218, 4.8837781783250067, 2.9139559779829018], [2.0918793085935401, 3.3710230051409567, 0.82058726769170431, 0.39571565951657739, 3.4978864326954993], [4.203264889858648, 1.3831664696877244, 1.1691711569846361, 2.3089889965241261, 1.6820271414142192]], [[0.52441424147613491, 0.98220324474886933, 1.8677192333499175, 1.3795137253028156, 4.792651268117968], [2.3417421207599913, 3.6610020025190688, 3.6939101089786464, 4.7090952459908335, 4.3421166528552329], [3.2443044771161422, 0.68507729715575738, 3.1989329587425388, 1.678042491395273, 2.2243028547470778], [3.0017790051530913, 1.289980248941454, 0.48090558620419022, 2.3207404735216288, 4.211799024702346]], [[2.7906104079026695, 0.54251146463418176, 2.3506276631427658, 2.684500362728321, 4.9100849452606319], [1.0801870135194829, 0.052678260725852952, 1.5388947354462685, 1.0163202043299546, 0.57571303210770464], [0.66470903084376309, 0.972885683188215, 3.3016342996057397, 2.5185977007929443, 2.6960811595909271], [3.1513105264439085, 2.8986860537404175, 2.9406313540826594, 1.2148817965035628, 4.3286737130216997]], [[4.0462384521659445, 0.93747062572926332, 1.8174670809998861, 2.7856404329111437, 4.0648538445893818], [3.1583289391304916, 1.4741262897781398, 0.19416132432584465, 0.52164740242513663, 3.1138272325835619], [4.8769037594962832, 1.7145545738852574, 0.3681831222358638, 3.8934849240425691, 1.6800302623596104], [0.78430826590252356, 3.2788738319280393, 3.5315963295317223, 2.9758914970154531, 2.1601717186328044]]], [[[2.6337291308628208, 1.9847852660519143, 4.3421510765900679, 1.3289154873840485, 3.7414303850077344], [0.98993755435252406, 1.324454708594722, 3.5431676330505311, 3.5667855309349994, 1.3712155187342265], [2.4708652871493859, 1.6310596144598157, 4.4377560057102539, 3.6076900919227808, 4.7386762596229079], [4.9823979513489443, 4.8587840828669835, 2.8512253913040708, 3.4585043645736739, 0.80810330887720672]], [[4.4814847923342063, 4.0240577755256712, 3.1165509621877177, 2.251777339100804, 0.22267965958345362], [2.9434281968636489, 1.5233774798192601, 1.969682969359531, 3.2525509398133114, 4.4483809912073573], [1.2921358923808719, 4.2213288743466029, 4.4643320093158367, 3.0892368541804665, 2.307860090813203], [3.7951904390635929, 4.7029037611886837, 3.8107543642358976, 1.741627993583676, 3.5935837259763899]], [[1.3498036962350428, 3.448726086055145, 2.349651863039242, 2.3801742897557179, 1.2418946301213958], [1.3881519896008934, 0.31989414253355625, 0.862511331237929, 1.4263477580425188, 3.0213469646304385], [2.9896528816856569, 2.2300061367821962, 3.9307347887617832, 1.6311576440694111, 4.1589447531351782], [1.93637750333237, 4.7343480307274275, 0.1070860255164655, 1.4035357587228274, 1.0976302570969729]], [[0.36693777382809117, 4.3019934831844004, 1.0634058007108109, 4.3164806361615096, 4.6362429921278956], [1.2240208000091759, 4.1550698867389455, 4.2090987191305791, 2.0772801618401426, 2.5290918178220121], [0.93318179689165082, 3.9987269683712117, 2.4234609774846838, 1.2267100264683115, 4.1683926825233337], [4.326709916649027, 4.5745453522116764, 2.7025255494932696, 1.2098323180337285, 2.6554414521520049]], [[3.8035701860286735, 2.0757999296682219, 0.39681525419495811, 3.4266849020113956, 3.9197981386350298], [2.038861423188052, 3.5254634601206414, 1.1206035714392897, 2.7104671385339194, 1.9613963846114939], [4.3778521931040792, 0.83524093346317108, 0.13682058278899675, 4.547867437718895, 3.3943250172444901], [3.4551121825629116, 4.4651747494581597, 1.0295505518699606, 0.25636262924342834, 3.1182017592704749]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.3449820906785479, 2.0332256824231068, 0.69062340208394535, 0.6986533536523678, 1.8383593558086577], [1.1846015269997057, 3.8953736666339913, 3.2006636903367802, 3.8357192486360647, 1.3131618873910178], [3.7599164789941186, 2.5436493938744382, 3.5383276348197055, 4.7808503314916893, 4.2898938808984246], [0.97073244978903506, 2.0753568536234219, 1.7048288941743901, 3.008355044890743, 0.79997278751044343]])+(1.-msk_arg1)*numpy.array([[0.54109295882365804, 3.9954393897300631, 0.73068895222427988, 1.9203723999961537, 0.27418917481854843], [2.66241232067072, 2.2521809551417946, 4.8410304917309315, 0.21209750748119061, 2.184341974639187], [2.6717551610140222, 1.4556503666556537, 3.6715777102138922, 0.82589577169009964, 2.770814983351169], [2.1571691991982958, 1.1699050577530858, 0.6670777323590823, 2.7956155846581754, 0.74930496703139993]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.5225247982563683, 0.75579946230122197, 1.4109938031545437, 1.1355289390316776, 1.715115625608753], [1.3691039628539641, 1.0860262626885731, 1.4626230900707557, 1.7191636715307068, 1.7328145829815058], [1.4778789862720585, 1.634034634744401, 1.2027066991249922, 0.83181670881304537, 1.2262334586543309], [1.3669506126116375, 1.5100998861549255, 1.3866117242615734, 0.82351378464203584, 1.6281206773607477]], [[0.21130086108267904, 10.063251711577109, 0.010964801349958333, 4.5788476378488339, 3.5707328122279964], [18.521116010814733, 17.97945836595342, 5.0738544151511533, 3.5317359599911256, 15.931860185236165], [14.462450829434657, 1.0633430518055087, 0.50865366758455322, 19.244445842445153, 22.615356437490746], [1.3942237514211762, 5.7980796045510914, 11.294176011362406, 10.433502462499728, 11.687017877191625]], [[1.2379459042441432, 0.17178832592540527, 0.80111815918382867, 1.3463996154998954, 1.2557057719506501], [2.7384966741342005, 1.0537062784126408, 1.0155282267753942, 2.1143110064360906, 1.8214269793307518], [2.1611411102281708, 2.9397074237314431, 2.8741414799804761, 2.5002381103089575, 1.4293911725503516], [2.560960301923235, 2.1552022379056246, 0.29086877493135832, 2.4113344625294491, 1.6871389541110267]], [[2.2903071130245465, 3.0691539856354422, 1.6164444481100717, 1.0045363984720324, 1.1156530828215088], [2.6701485820189474, 3.0698265988292861, 2.2148468146610143, 1.2573948534990989, 1.304403550808745], [2.3247389851929916, 2.3453294153086119, 1.8259086737963852, 1.3495061166479225, 2.8985980359946639], [2.8202710841261629, 2.0114428683502696, 2.5602644465074942, 2.5110572063566403, 2.9876215357944704]], [[15.938918756021309, 12.86369054257783, 3.3861079440893693, 0.59629506542174227, 0.51878848875138972], [1.1355341920417477, 0.00071511804470751638, 0.50783242905464276, 10.565311504120666, 6.6252105173887452], [12.580446880519501, 4.7327305801627135, 0.2553708850363906, 4.6919928481568034, 2.1069732894066622], [6.0375176234346828, 18.578091668652853, 18.41589056889541, 0.6366481400084506, 6.2922628484817222]]], [[[1.8913545164131722, 3.5077985526960056, 5.4670352538899794, 2.9538208112861701, 1.8394156388936658], [6.1202316947078019, 3.7136546972258855, 0.75383780322265637, 0.94718441461918179, 4.5146287999940515], [3.0112917593124622, 2.354997777699142, 5.7656377372479142, 1.4186867620016479, 0.34351047109590166], [3.3172230471379791, 3.0087047733772079, 4.1783001412603467, 1.5408471238722889, 5.2510289475828005]], [[448.33790447191166, 69.394105418471526, 1.0774013738736097, 73.303939623545205, 11.018407174414769], [409.53704815960333, 0.83322192237910675, 2.1725295806133125, 0.099268333256806465, 159.62070267467612], [121.37410806114625, 102.7518974275003, 251.92800996625988, 1.6175857913876206, 356.79918234034619], [36.033887173083706, 355.41391549645914, 1.142745946677711, 0.48187751055736927, 144.27642557263471]], [[21.518364184311174, 1.6277674765910795, 44.79271947132402, 61.373445255860069, 0.40428322281851498], [0.50043871792928041, 0.24337103107211153, 35.774898331593725, 49.81817928390543, 12.786786389494202], [114.52342409729677, 77.83545964805478, 0.34819638665659741, 0.04315974456493215, 0.6592347339296647], [0.11946181907695784, 0.16190278618096871, 11.023583585827479, 60.316059613486409, 86.270497553247765]], [[174.89947203535473, 276.1860720778314, 68.825765002749222, 314.44243535967519, 345.26625360799147], [1.4060375547415779e-05, 27.36569232091421, 10.359492083204266, 33.014202038802267, 53.994020551886216], [12.703153525024049, 73.87664722033287, 0.15243202997402835, 226.37074693762963, 39.35220398677513], [94.662534876320024, 375.2776386183308, 274.3300733317169, 18.854996772081773, 1.2314993443228495]], [[8.2662362942475198, 1.050696677449418, 0.31271515181343984, 7.3296265337722168, 2.8635951808817541], [5.6034732014924744, 5.8078135302731182, 7.2530622582581241, 6.6786631116036679, 5.114717639044426], [0.41073540684797966, 1.5059521679548993, 4.9006116750644422, 4.0575078240808642, 5.876254948417106], [1.579348832082917, 0.055549977738016974, 7.6231718159670674, 3.6737780180214217, 5.2825820056557262]]], [[[163.72437088959632, 248.6470014084432, 380.72925102533577, 128.72332768760234, 30.177514096181298], [34.190843445177663, 100.58599967383317, 416.83665122756366, 97.792080511203949, 81.997951020513653], [0.26532394380160329, 249.88687531140155, 34.942380803073483, 368.47922220815138, 15.381473608797757], [4.67515590880065, 94.405205407031033, 160.41110823345946, 0.029191699340957125, 416.32401656897889]], [[28.432852188481021, 8.9167892314671544, 9.0157432264797137, 16.440247030446692, 0.078847259062527902], [27.381679052650409, 4.3509610374350798, 59.680615184498158, 56.487662531360492, 15.187345292797859], [6.5363122338513699, 22.000916351707392, 0.60473281238767995, 0.094598470424945805, 24.168575390107979], [38.564356095886545, 2.2821028605235494, 1.4881889309205265, 8.4026795686362306, 3.7535306967336712]], [[0.10188645603016303, 0.93843862070439954, 9.119883705437168, 3.1217380533342061, 255.91766467010302], [20.302536276909741, 98.673682466637359, 101.84800771379983, 240.4768434991652, 180.46658220577086], [64.345132988833697, 0.26229668457110195, 61.217224763499694, 6.2434768241779919, 16.923330620892944], [48.878998877774329, 2.4619492712888245, 0.074993475584161945, 19.665573641636318, 162.02092472556109]], [[135.15160706813379, 0.053733953954878573, 59.507868946398915, 112.28766433321033, 2013.692262355376], [1.4459506616612585, 7.7323443440046038e-07, 7.8526781575385485, 1.0804683562340811, 0.0713807430739717], [0.14191494285613682, 0.87685001156938591, 301.97122229325174, 82.771379656372474, 114.62248560506977], [241.66392203024736, 162.07449218528163, 173.59784075789904, 2.5359723859578125, 1102.34439769185]], [[401.96329648396585, 0.75805614788664211, 12.974301315709313, 81.036613118856408, 409.95684187230438], [138.87340087353144, 5.284403139659215, 0.00088367716408122639, 0.061316524774659285, 130.67157050100869], [895.49987559643341, 10.103789404737601, 0.01375498363780102, 340.79209134327954, 9.2594772474846341], [0.35266270563520757, 163.0813355223687, 224.2527014053845, 107.58880083891546, 27.221948503271246]]], [[[2.5601300153113171, 1.9453608955441204, 4.1594975109317112, 1.3179013338037633, 3.5996996031875388], [0.99023061414979319, 1.3136067891134677, 3.4143839073505533, 3.4364751003586815, 1.3586042574379136], [2.406308683433223, 1.6078715953626628, 4.2483719477788684, 3.4747254036634185, 4.527747640653355], [4.7536379167470155, 4.639109451953364, 2.7651194126492915, 3.3351577991883534, 0.81315830632260921]], [[22.487074608564885, 17.984319406644985, 10.581572277625481, 5.3903392220284578, 0.044279668006342747], [9.3980599781826211, 2.3954706965776391, 4.0829817697615161, 11.562395243955848, 22.143711467458782], [1.7021750308849131, 19.86233304486014, 22.308818747160139, 10.390012266022273, 5.6726929597895479], [15.926354052344751, 24.854187840129772, 16.06220146556922, 3.1627759939141464, 14.220614396475815]], [[1.6675893206068246, 8.2530806754339014, 4.2903973937649678, 4.3858471825621619, 1.4467663497482168], [1.7491648726510081, 0.14325850584766664, 0.77712340839945471, 1.8320105012318062, 6.5865783225501469], [6.4692215701031737, 3.9246617439413387, 10.315161049774618, 2.3028688805066779, 11.356914633407502], [3.0850974924064771, 14.16455929951079, 0.022174837324517156, 1.7823411984149247, 1.1721162243336323]], [[0.048993607156460055, 80.594161951986578, 1.2031510281735318, 81.413406889702131, 100.94025977318668], [1.8369607338658724, 72.594421428027331, 75.471409885834589, 9.0185746896476715, 16.302739044866236], [0.81217168161873543, 64.683623158233644, 14.339039913868337, 1.8491289059051115, 73.296922464670345], [81.995206467711114, 96.952954777970305, 19.902923853263111, 1.7736450748830954, 18.877908323420915]], [[2.911637868858711, 1.7936585122205184, 0.47739911592080608, 2.6784576352939737, 2.9825986550987378], [1.7680793332901343, 2.740047446457182, 1.0953685214969588, 2.2203607053181718, 1.7141322856489067], [3.2583026783263587, 0.8658677090996495, 0.20367825064500786, 3.3591418005261713, 2.6582039370594814], [2.6962184076725038, 3.3101912350776437, 1.023570584089786, 0.33658895457462124, 2.4837617200978745]]]])+(1.-msk_ref)*numpy.array([[[[1.9335088426107476, 0.64459176726306089, 1.7160311477156458, 1.2206089642939166, 2.3306734400372835], [1.6368014762452368, 1.138188948284103, 1.8155354278484961, 2.3393071993598178, 2.3685073419099765], [1.8453253308277662, 2.160196231064297, 1.3357582830333188, 0.74914521083429941, 1.3769686883849575], [1.6327654372898979, 1.9088177267126047, 1.6697501792640468, 0.7374499720767328, 2.1479461702513976]], [[0.047139055622386641, 93.423312480378883, 0.00014075246867831213, 19.880850769679355, 12.195770180865443], [309.78476174731458, 292.2329405650334, 24.324366022400458, 11.935415521798953, 230.43205274057954], [190.52915781490535, 1.1282756260074911, 0.26490965821673229, 334.00691840675773, 458.67311976819445], [1.9214276266345018, 31.616228298169805, 117.20262695511788, 100.29765709915318, 125.34789387332106]], [[1.2533709504899977, 0.15510038455602732, 0.79087831488537907, 1.3698336376727107, 1.2724031427080189], [2.9033131211100289, 1.0569090379247206, 1.0164364393609135, 2.2081729604663929, 1.8859022960124012], [2.2599524306939354, 3.1294795404794344, 3.0556798527286011, 2.6367545429513966, 1.459324821483442], [2.7045546779132685, 2.253382258336571, 0.27075983433372625, 2.5376608770945661, 1.7391165649630391]], [[9.7550970329594939, 21.810068872410675, 3.7433999364681725, 1.0125186068541967, 1.350959518779612], [14.873326776640118, 21.823209335222032, 8.8968909648482803, 1.8767892799645611, 2.0760137864496362], [10.163524328577719, 10.412878647324424, 5.2326471936729053, 2.279337499661334, 18.638265973274034], [17.286382274053079, 6.8272316229061678, 13.250830738563163, 12.562516815958174, 20.254265103212767]], [[1.5112814434210553, 1.4637287246435564, 1.1995110756681091, 0.92578534050099859, 0.90675745953381082], [1.0191381249805245, 0.33949553923540654, 0.90387534866833319, 1.4213821706557752, 1.3258090654903463], [1.4588760631674571, 1.2609333826317308, 0.81579360435366688, 1.2593086084287672, 1.1175664886149224], [1.3075675383812821, 1.5462155932678385, 1.5441946158695463, 0.93487131229624165, 1.3156522720813781]]], [[[4.1884366218787781, 16.787063936325744, 45.510335337912863, 11.407657252618691, 3.934346208590104], [58.650850593378976, 19.082651664792127, 0.52988301184386888, 0.8851894528823876, 29.598814660889349], [11.912563761240483, 6.8557513040913944, 51.288207157966326, 2.1946615730818171, 0.090576830154497856], [14.806419278546683, 11.889574938796171, 24.871808477292142, 2.6423651014486649, 41.568247330411694]], [[34.125089625508515, 11.603634903910017, 1.044046047243846, 11.977252209561053, 4.0042381617830261], [32.385059793587956, 0.89988363539857008, 1.5661120522620104, 0.26301948539488107, 18.782612551681552], [16.031541618346921, 14.559712515360197, 24.453529225085024, 1.3205669755835348, 29.904059982215813], [7.9440281163990152, 29.836878428159057, 1.0802012221696093, 0.65566813336641261, 17.716502002900643]], [[103.72539502503052, 2.0894655607301038, 314.38729868034147, 506.21546009640332, 0.25416097860916581], [0.35096655777383251, 0.11795777500006002, 223.76918650878983, 369.24384395396544, 47.204766270151325], [1300.4534623550142, 725.13975080597834, 0.20277078236808457, 0.0086207572328261331, 0.53247180456876231], [0.040206971598924016, 0.063678170338025031, 37.715677740421782, 493.08260858826623, 847.24274341824832]], [[1.3305028691502789, 1.3645427631212779, 1.2636276504788657, 1.3743661720229297, 1.3814913399957027], [0.53914639306306678, 1.2008004970686117, 1.1380032804396181, 1.213324926761538, 1.2467824810276313], [1.1509095539336052, 1.2685856429720324, 0.90121393769947367, 1.3496173582007664, 1.2251644911667523], [1.2860963241809595, 1.3878731670565203, 1.364034095372411, 1.1763191727699402, 1.0115808321003437]], [[33.563691682600037, 1.0857401668594813, 0.14461713958126221, 27.478682936627955, 5.7548857604385786], [17.579240615840483, 18.658436207127089, 27.002873135808191, 23.540000569330893, 15.103212525842157], [0.22760976109200398, 1.9759455436877931, 14.066216871541757, 10.275236497094717, 19.025613707936813], [2.138714024314055, 0.008163566695658547, 29.333485918132101, 8.7100586138650762, 15.936690251140199]]], [[[37.439022468694702, 50.381953918966296, 68.195768435032988, 31.557251332309036, 11.257624019999168], [12.302103829492289, 26.48384737239466, 72.730882485112033, 25.958992742688736, 22.904841670485336], [0.38953088674430009, 50.560345740856562, 12.49364756989733, 66.629224937710333, 6.9737703463919711], [2.9919182073544364, 25.316888148076714, 36.899056268684838, 0.081176486233419748, 72.667311827370867]], [[6.7916547849860347, 3.4976340851527028, 3.5197942241542881, 4.9639341797785272, 0.233703785622452], [6.6468073186867747, 2.3197553074856181, 10.381339987014618, 10.059764687843607, 4.7437831654290452], [2.9281225861924933, 5.8645748808114959, 0.74988677763014022, 0.25937658085298804, 6.188579492047527], [8.0858242228890873, 1.6034814799034744, 1.255471246960923, 3.3807664396458916, 2.1317383012954076]], [[0.093489547806251877, 0.93619583300368769, 9.9115503229266935, 3.258479265744048, 315.34579524096034], [22.740049399291504, 117.30082706188375, 121.21884267559861, 295.62573098329165, 219.46767591899206], [75.270135065172582, 0.24940496058859196, 71.476883783896426, 6.6893129534267297, 18.825627234605854], [56.589134557261339, 2.5469141203774406, 0.068023336316161892, 22.000187799892171, 196.23725787765298]], [[2.3340012951139841, 0.60346133657217371, 2.0256255499638085, 2.2604584262328098, 3.7219238865185011], [1.0657777333898133, 0.087942903743500125, 1.427627977894991, 1.0134597595612298, 0.63380432619786309], [0.71369420634321257, 0.97755297415553255, 2.6817362405634801, 2.144447205853333, 2.2685091308665464], [2.5804875877084434, 2.4084076669890684, 2.4371547001960012, 1.1744006788241255, 3.3539978552091525]], [[48.086959396589435, 0.83618021625556915, 5.2352064638652376, 17.092477457103453, 48.702452426393833], [24.204935837972315, 2.9307478058008463, 0.010656893219697721, 0.16477984616903826, 23.271684825875937], [80.671516193832687, 4.4544077263114614, 0.062754270667242734, 43.223382870301016, 4.2102899925465991], [0.51008484748659577, 26.852068223695426, 32.985753906542541, 20.526047815526272, 8.4489964310463073]]], [[[8.0768689970022702, 4.3875133539572815, 23.748568286854336, 1.846735540079512, 17.224205937223999], [0.97841990954739322, 1.8333893252679803, 15.315488499885268, 15.536562113736068, 1.9758785002396186], [7.0378808431329434, 2.8729851614760396, 24.890922772470038, 15.923470153795062, 28.675206515200291], [31.951603732211286, 30.266081229316569, 9.5847335488078489, 14.536950216813645, 0.63152480657212129]], [[5.7823158033358366, 5.0979986318623798, 3.7805238478607981, 2.584766404105308, 0.17252346605050309], [3.5360143412066454, 1.6363170484431424, 2.2101194942941582, 3.9742352719858376, 5.7323773385937375], [1.3496464440648788, 5.391581430548154, 5.7564322613000103, 3.7417899564002894, 2.6602386826715789], [4.7604530576773447, 6.1179298254306573, 4.7833003938171563, 1.9137928917315712, 4.465959139990666]], [[1.22152012111633, 2.2838109333783048, 1.7680241243618997, 1.7833119402124338, 1.155478426017537], [1.2445620791491723, 0.46752056277272558, 0.9060457648330652, 1.2673026587306, 2.0908897246247284], [2.0762326800592237, 1.7074469355911415, 2.4920697645354926, 1.3859619484974814, 2.587675875208947], [1.5539772530504905, 2.8213114408792195, 0.22529749163205395, 1.25374584413924, 1.0641120069092784]], [[0.060641200277992656, 59.087691590311671, 1.1875180249567718, 59.645647666436965, 72.835747624087773], [1.7596396185377248, 53.61758237194794, 55.589505203092031, 7.7195889496728478, 13.382403061824702], [0.82420871925576866, 48.16613845508725, 11.877782266379448, 1.7704688109948579, 54.099586988100469], [60.041647609075689, 70.158288436317747, 16.108770916816951, 1.7032084646515986, 15.336393870668292]], [[2.7210738189395238, 1.7284971400504396, 0.50028820441682376, 2.5164243162198123, 2.7831424721884508], [1.7053979905202126, 2.5705841773462246, 1.0890670810229328, 2.110970332378137, 1.6566115511112416], [3.0234314294602398, 0.87380227472307159, 0.22527550930158141, 3.1109902489091796, 2.4985967897151666], [2.5320505149368993, 3.0685074776433137, 1.0220613519158321, 0.36062173893728272, 2.344687021303947]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_expandedData_rank4(self):
      arg0=Data(numpy.array([[3.3733291722957905, 3.5617909777691708, 4.2271259802882239, 2.2436547139466807, 2.4436215576843097], [2.1452113396163135, 0.41066069124641857, 3.4207364429482641, 0.46395554030507469, 0.7247694526811127], [1.4613617296428862, 4.6606635355943258, 2.6035242731287402, 0.40657532209463365, 0.18774321966856561], [0.51640048222687918, 4.0267358171228969, 4.4445855205788423, 0.57104561445267399, 2.5456320040207805]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.5444582649590748, 2.2608032106180507, 1.2706929849152471, 0.65950678062166834, 4.1901677828895707], [2.3095597634476608, 2.2060328595646355, 1.021729540923775, 0.2766861532809981, 1.1859677164490985], [4.0569126834726976, 2.9421684819130518, 3.6697769751288627, 1.8880018392781313, 1.7009516558170239], [0.30084876908227176, 1.0354179329374738, 0.90556740181581896, 3.7326786690524396, 2.4232640326334174]], [[0.85923221558946838, 2.0917528716205562, 3.1180859559059648, 3.3880289620931894, 4.5777615705975405], [3.6420694064894708, 2.4800028142775825, 2.7346911680735464, 4.3998668683446143, 0.034940900839474154], [2.5152473833665918, 3.7563171713588677, 2.5733738426243549, 3.4596782011423111, 1.8306827981941254], [4.6985984802417358, 2.0753683236596725, 0.42515148355630078, 1.4166358600443698, 0.62629789758127286]], [[3.9483816591877718, 0.87583376643295852, 4.4993332903824603, 4.0201894655295938, 1.9694715156093143], [2.0056752820811576, 2.0725012412295931, 3.3971363647063839, 1.6292703974328973, 1.990179296212079], [2.4288341857268927, 4.8716271049756825, 1.6041720595218201, 3.9424772795690228, 0.17022196698941969], [2.1088541940769452, 1.1005404410946815, 3.5352970276041793, 0.38592889841191363, 2.2732669813319584]], [[0.071155962483268484, 1.3095943394677678, 0.63388062791893873, 4.5248317431860245, 3.8943083503152942], [4.0235936355124684, 2.3498000543711917, 4.1050539010083584, 1.2101432529343128, 2.3525118738379334], [0.92322146361571145, 0.12479973767315741, 3.1356022918280448, 3.8683967873797167, 1.631867030930116], [3.6100172910384711, 1.3407014894395284, 1.7410672847575548, 2.7482784432654404, 4.0345129375193389]], [[3.5064810401583788, 0.019597498678497013, 4.3359212139751051, 1.994909647876921, 3.2457316571270636], [1.2003401188503415, 0.74416886695108586, 4.2712873687968669, 1.5052748454078679, 2.2345821678256961], [1.2689755815073118, 0.14900918514198533, 3.8807007887547291, 4.951207579561431, 1.7428425054178205], [4.3458882479967471, 2.7806734993917135, 4.8643969445967743, 0.44907558569769135, 3.7298817364800727]]], [[[0.090733132772010575, 2.0070166245714876, 4.9374300221905081, 1.4086190267088017, 1.790793530916565], [1.91837149348142, 3.9963460222108562, 0.14763567642729039, 4.9532852049742582, 0.62644477703042833], [3.2470452827675214, 1.6464624407651764, 1.7499688259507056, 1.5838368907119191, 1.5332526587068189], [4.7985630168846187, 4.9108819302764086, 4.8042891833867865, 2.6073139068944013, 4.3591866697814954]], [[4.8011892001735488, 0.79062571588185226, 1.3448515415383138, 3.0501117424369095, 4.6278128125904434], [1.1491898136269247, 0.89082280374054157, 2.4353499572519195, 3.5858782631472343, 4.5928899403913324], [3.0876517356212441, 2.5263524552665944, 3.1917531910387713, 3.1792808535499093, 1.3332024595994276], [4.6523342183939409, 2.631525221828062, 3.7357715490444825, 2.1295117971366437, 0.014911019907380477]], [[0.11019466209474743, 0.43035151765497848, 4.5040115936905876, 1.2517098565370561, 3.9044681125684324], [2.3972437738212951, 4.6518910010733654, 0.95960362555274259, 0.77391294981958303, 4.4791916200312389], [0.97990416148624104, 4.578746281350349, 4.1896464810754388, 0.56492418171191827, 3.8339266881585679], [3.3641551939832732, 4.0984640936097527, 1.4774348215537414, 4.2620339593225749, 1.9370485056932369]], [[4.9838637865450073, 3.6803287843553649, 3.5289074782398431, 2.5505157032234407, 0.83303453757825729], [0.34526961207277784, 3.6468870309215284, 3.0174430071927554, 0.055503374865066075, 2.6918956801820619], [2.0091906889453575, 4.6803658486392337, 1.7112668147042256, 0.19635812395134139, 3.081177529473591], [3.7579704666826901, 4.8434418028992523, 0.73160008269476506, 3.2297268506318648, 1.4074296784780638]], [[2.2303115329858829, 4.794024051557507, 0.73922022099136986, 1.9230077649880766, 2.2199078425867063], [2.0461777066012816, 3.7945683703151829, 2.8415592568495138, 4.1288811386333473, 3.8715178355834983], [4.3521893070297608, 0.050733733384003853, 1.4295219979365994, 0.061020563311187992, 1.8202915590065618], [3.6042200312997354, 0.048947311389430391, 0.84050802026246352, 0.98796956508236267, 0.18067895951561966]]], [[[4.6862740885398848, 1.6184744761954679, 1.0025794558915915, 3.5426007309788536, 3.2086892838363745], [2.3471943589384585, 2.9389031173519662, 0.51756057029562164, 4.4393009653111397, 3.9118121475259877], [2.1486000315894587, 3.5850382616522638, 2.697310629976402, 1.1176686241044906, 0.63518930048383138], [3.8953546067962996, 3.2222795812727991, 3.584757512150337, 3.8307368143180796, 2.9561050362119481]], [[4.2173230428465533, 1.3589640376372565, 4.8065303609039063, 2.7517095107553771, 2.3311545935335549], [0.70779586746894707, 1.9237211932732821, 3.1555854494731279, 0.96888337347373232, 2.1105988485898535], [2.7369479698508963, 4.2525609627828604, 2.5412347401832034, 2.9285020055362994, 0.8365049502410169], [4.5901065005045547, 0.17560997842279788, 0.79486284846672839, 1.4402872973847269, 4.0385878570517724]], [[3.8032613330922107, 4.7441598314159235, 1.0146133493011924, 2.6844434792023528, 0.55516619353935737], [3.8495210980550376, 2.7824341378559629, 4.9130448417396453, 3.1851008056147436, 3.6666826091339337], [4.1872221158880096, 0.60789729699146233, 1.5221664088008808, 1.5840760275215269, 2.1073783747711023], [0.51060373660367564, 3.6243992602329218, 1.8413483043419931, 3.6113543388827707, 3.5370275447988657]], [[2.3975087860264845, 2.5518624278394477, 0.98042409208426029, 0.57513659008528739, 0.32140780597027807], [4.8243182500690374, 2.8557763087176742, 3.3195131274841274, 3.4555683295789446, 3.3540155765969502], [2.2193173115131346, 2.6687650768455859, 2.8648943734490082, 2.8946991054103184, 2.9120004541657232], [0.31174811076243397, 3.9913626675708258, 3.0481148606612192, 0.95185348650192447, 2.7485273970652306]], [[1.3785457653291464, 4.6560954548001661, 1.8058535280490862, 1.5128377553123113, 3.4125606697440372], [0.25929378070051118, 2.0022010031812894, 2.2939079027511413, 1.7336414457118094, 0.40958251017844999], [0.79579447472356357, 1.6140322101872295, 4.1209031163764891, 3.7990572661490125, 4.6239788349313313], [2.3300784709278632, 1.4489154240306621, 4.5942340930489269, 3.1396076469384151, 0.078648182839603981]]], [[[4.9090384478178732, 0.53374762083152094, 0.44111275062497418, 4.7180500412500814, 1.8466333259286469], [1.7911181732258461, 1.2859950445467996, 2.1178637084485055, 4.5492280721558274, 4.9729344228693586], [1.5500169591631447, 3.3018159058054191, 3.1097682256316617, 4.7213347145111424, 3.1342490233037914], [3.6558599341334261, 3.5751731124787258, 3.3792478745775076, 3.1388910729568154, 3.3874957891368016]], [[3.4623984158035408, 3.0258757626821722, 3.1585101862845311, 4.3949619022247219, 1.9756773000819121], [4.7281713988585459, 1.968686889843354, 1.0527138847198436, 1.7435357231906974, 3.2298342119544667], [4.2707823133223641, 4.289247409539076, 0.044667966049266816, 2.4458926343293648, 4.8800582977191453], [1.5801537980158884, 2.0245432632996083, 4.6994696108307439, 2.5742371461798954, 0.52400923681852274]], [[4.6520218424960982, 0.18948590339572224, 3.4362541332033536, 3.2170373245040564, 4.4350982416899267], [4.1585069792252041, 1.9147966598968094, 2.7541709032346997, 4.230036913293822, 0.41602079546960885], [4.0910144906348949, 3.1079377439995359, 4.364479140867811, 0.36832250369102493, 0.90737029396767255], [0.74587073849139129, 2.0807667443255111, 2.3440729162506591, 2.7978693735250202, 3.829889929371368]], [[1.8460499196186457, 1.6278900719688707, 0.61889915656414263, 0.60460978877011318, 2.4082128586332976], [0.24436353518203852, 4.9080681824991563, 3.735751144597776, 3.7930018565606978, 1.4657878083569658], [3.9333327088274004, 4.0862835232244574, 4.0886359841106712, 2.3060187737913611, 0.42844307607723103], [3.6045250539060012, 4.130127028284476, 2.9982226899735842, 0.64485114339896366, 4.5935127283762807]], [[2.5759182456634657, 4.0397244905718379, 1.3177204766054806, 3.9876562451814981, 3.2555560645635788], [3.8410563581929602, 1.5018567762136288, 1.5754746513567435, 4.0847082616915849, 1.2494699341764282], [1.7588680634915557, 1.2264666028905418, 0.39223282700126277, 3.0972272659836206, 1.1939904103687939], [0.93538260746365554, 1.2476984840284704, 3.9568802841591966, 2.9437683224289035, 4.8351283642909353]]]])+(1.-msk_arg1)*numpy.array([[[[3.5873891306949792, 1.2721680557315325, 0.70237734709464783, 4.9541704997722258, 2.9333272540462501], [4.196118342996237, 4.6929698519385497, 1.5454527688748838, 0.22053260219954232, 2.0887539268282937], [3.8373505810662731, 2.5158305456019794, 1.0728917017302304, 4.6499360142044059, 1.6388376694914337], [3.1762833862829183, 2.2778049883843217, 4.4739353726722833, 4.5980147275587822, 4.8653944000206701]], [[1.764501326763356, 0.12582419739675471, 4.9401980424120868, 4.0710302962627196, 4.2083113057427681], [0.66019450696263438, 2.2479935695062361, 3.6816454793457947, 1.8726892959658572, 0.57009163943143804], [4.330179697967913, 1.1422834839104221, 4.5459097264651316, 2.3291675968693091, 4.1189844242808551], [2.413973880500988, 2.0617313895749976, 2.9290987558777402, 4.3501828704346366, 3.7578485239671209]], [[3.2918792480123948, 3.0994609982651031, 2.3710124555990895, 2.6198714111090635, 1.1431134024125533], [4.9998313155615373, 4.7884566959966071, 1.3709727815863051, 4.1868995614814803, 2.9449297164321013], [0.026384205040532585, 3.9819201528910937, 2.2931681092232377, 2.8405715155217242, 2.0532151386522459], [2.0648799207712636, 1.2147538321553402, 2.6438274703694646, 3.6377753580350771, 3.7104581408407764]], [[2.4018740623069448, 1.4188402217072904, 4.5310766746646536, 4.3790884282110811, 2.7239191791122228], [0.81439079637153966, 1.7717806850153581, 4.1840558322163188, 3.9834769725263994, 3.3497418445558775], [3.5445222306765478, 2.5585055850981386, 0.89031456386426333, 3.9799767669185626, 3.5976240227416714], [1.0104662789727679, 4.1489479294895242, 3.1909836096281174, 4.9236598070393738, 4.7877268190885296]], [[0.83267683692908079, 1.8955774885040317, 4.1843074906156623, 2.9563666251003258, 2.7971794734553201], [1.0525672502572088, 4.5387748912015455, 0.60688121785473514, 4.8966152402810232, 1.8777444879996084], [3.2401143686389804, 3.8828975199849474, 1.5991345689464673, 2.1790125092846981, 2.9163441895799265], [0.50046742503316022, 0.70128258890021755, 3.6076022347757868, 4.230187992126587, 3.8378140430997782]]], [[[1.7624989353747442, 1.5177069803435632, 1.8034718734404167, 1.8773789757653876, 2.1763992715805758], [3.0525023821259416, 1.9007042693988314, 0.28268313035967685, 3.1784263022940062, 3.5253021849093509], [4.3848777875954283, 3.1838845160115015, 3.9049255279070914, 3.1221823302073601, 4.2234386364546772], [2.2754232831129571, 0.91976616363769237, 0.48716905141353234, 4.964337375623443, 1.4706627515421899]], [[2.5756748704285153, 0.80353301089727891, 0.29870855213707587, 1.5668848109493931, 3.4125928472774145], [2.7717730648110717, 2.3402352339607844, 4.6233850231669864, 3.965502183633216, 0.45577192173084424], [2.1735225584014679, 0.70676504265821105, 1.5481510823926696, 2.1724159523888993, 3.4118832935071559], [1.919355607570322, 3.0710885085710893, 1.074521501401029, 0.060106614842385657, 2.8657185531483376]], [[0.76435246235785459, 0.30989821878569734, 0.42579469318927149, 4.3488559810828944, 3.2674921564039856], [2.7063302466426746, 2.0583491025498888, 0.92458058431835843, 1.5256691384885568, 0.7643251751250022], [1.7958879648550021, 3.4979058460611325, 1.6734017777478225, 3.9270880750387249, 4.3295379356202934], [0.6320506917959291, 3.0168576468674231, 3.6357172564316795, 3.1362777071695169, 1.5833626076554264]], [[4.0662819395244458, 1.6498490390208422, 0.2971876207130793, 0.60003269054146524, 0.045232252653029451], [0.25711734599749686, 2.1304150146656995, 0.016747446516304082, 2.5209365246799256, 3.0727742341918414], [4.650302416012047, 2.780159343153922, 3.2406836412472715, 3.5809822221878291, 1.2264895444100863], [2.1257210578541508, 1.5968886434758458, 4.2076685868206463, 3.1565885882798046, 1.1234013151625712]], [[1.127939997458733, 3.6677789493313018, 4.1396747935387879, 3.7652447598056455, 3.5969416925373836], [1.171044770071521, 0.51089052194646478, 2.9586108047257729, 3.1292963920811414, 4.2919984195917236], [2.0152982795912813, 0.59269887845738523, 1.6209023093187844, 0.55185206093898398, 1.4314560748530589], [2.3700419819567662, 4.6723068573163786, 4.4533955092005275, 0.31424151153690605, 1.7590935857739602]]], [[[4.0687336533628287, 1.3178939970812744, 4.8778432623438706, 1.1884155571929402, 0.33750807356565093], [0.69859121789941026, 4.7501648251412396, 1.1013314119275333, 3.8611309740599769, 1.3329088557307609], [1.7940475939741116, 3.0182838151899709, 0.11183188329963958, 2.6975582846525978, 1.1934482452666391], [3.7230290678664311, 0.86908833001100461, 0.14940814165645905, 1.8256529382370117, 1.7526619873054321]], [[2.0807814975132271, 1.7394887826454457, 0.79119146629356318, 3.4746326809925492, 4.0808831192742234], [4.3171544945282969, 1.2985022324380744, 0.90840292655033505, 4.4367652864559526, 3.0906300795994959], [4.1017751615135634, 1.2593213972971053, 4.5404506880749738, 4.2263167392175616, 1.715089361066064], [1.0384397639445306, 0.66415742456294513, 3.1691173682410678, 3.2890906518956844, 2.9210798229356696]], [[2.6592848606019075, 2.4779804852453946, 4.9704106644757262, 3.3176083539391747, 3.675091993667313], [3.933554729846267, 4.4019777106698657, 3.5364051604036368, 4.3655090962550123, 1.6304794327412522], [4.7283543653648632, 4.6716709285176217, 4.6397083552155163, 0.14061716607387001, 2.9388130576421121], [4.7409170649358527, 1.5787339663496138, 2.2049997320887877, 0.54663956979477601, 2.7712452482199272]], [[4.879056863477655, 2.7843369096011998, 3.2286623878363203, 1.5353756061271835, 1.1356753355902831], [4.544352769332014, 0.99241049749727384, 1.1948462307379935, 4.397974426074116, 4.2495171923275468], [0.35554776187549636, 2.2138403793082335, 0.082638936556330367, 3.3531479634738854, 0.039961997294651795], [0.70057062478870646, 3.0543473802217043, 2.7341720932405558, 1.1347128992065254, 2.746938234954555]], [[1.1035269635481735, 4.5735720135619102, 1.8515939351301751, 3.1695797191860025, 1.0156308634486149], [0.37927414577235086, 4.2424696958481976, 2.2739571377578107, 0.88071153015297998, 3.4356767664205217], [0.2306492737958773, 0.96717616609789647, 2.9747440731761241, 3.1396605136792508, 2.684673749494193], [3.417260367138149, 3.4611937215724855, 0.81753808361240576, 3.8654598959965649, 4.415105627452717]]], [[[3.5201870391613364, 0.18392904106906741, 2.7025198796974266, 4.8558356649229584, 1.6882810482656079], [4.4591449977981714, 3.6669668899008974, 0.67969063359450088, 3.4030894289301723, 0.88568302164407897], [4.2665947109323241, 2.3158069934367242, 2.0211801297039829, 4.1887631953426121, 3.3040151702509171], [3.1757508700954964, 2.5633262286314578, 4.9391283946855182, 4.030666375456617, 1.9913055076822486]], [[4.2136910545371187, 4.6692859903376229, 3.0722995667292294, 0.44023238327516939, 3.1991655697957957], [3.0165652453817211, 3.9518085212850242, 2.3692641178798892, 0.93436389059120717, 4.1609875775602108], [2.6978836582731569, 4.9258518690270163, 3.1230399688071659, 1.5438802551678801, 2.6537903515069319], [0.10366739387391205, 4.7342988763320832, 0.47433540999258067, 3.9804736827237788, 2.1292147117971583]], [[0.47108758403422996, 1.9075598610805893, 1.6363949682806478, 2.3722352247920431, 3.3950776961136397], [0.709249333770957, 2.8402068158094846, 3.1599898182683388, 2.5236827837692837, 4.6674383274893056], [0.51999648573966939, 3.0623836884508835, 1.8989051676856443, 4.0388070517375869, 4.3256320068873899], [0.76317534850322766, 4.2660051208130563, 2.708556378139197, 2.5246375668138716, 3.112628400118294]], [[3.1722935714548401, 0.60906565289114256, 0.4304500916430386, 3.5592944921868348, 3.6681065402962547], [1.8568412539780299, 0.32832261822319497, 0.76273213192062828, 0.65097864437091091, 3.0108423823845674], [4.8023818941729575, 2.1045635152078104, 0.36249609733696636, 0.77921004492409296, 4.7161358628303613], [0.91510767593107922, 4.4662698194339994, 1.222908568120231, 0.054258931769785329, 3.5880713266037172]], [[0.36621784189997852, 4.3660814215817627, 2.4313732443535576, 0.37095059731194796, 0.36818867311511805], [2.0453068935485845, 4.6425512327586684, 0.96052820863486343, 4.4104410686630935, 2.333446607223181], [1.5263656986222118, 3.5194232060581148, 0.7154871762948869, 3.734894596468008, 4.1678804140791357], [0.9296199721236954, 3.3246779410062479, 4.9220883986036208, 1.6800894134915165, 4.3938334607201037]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[251.03878477949311, 15.625605456754432, 4.6881393674848324, 2.2297609624445052, 163.1753844999121], [16.579948500780862, 14.618906642907335, 3.4636435826756724, 1.399930696611567, 4.2292231108803922], [138.76761598520835, 35.779785174939818, 86.667793483703974, 9.9306079909625389, 7.9104712722326864], [1.4416698516521493, 3.521773899380062, 3.0074108689549162, 93.55640557202986, 19.038192261564383]], [[2.9785938394832199, 14.254566735332405, 52.498824868761652, 73.972213857268855, 335.2791418131539], [102.14443477046095, 23.34206411708216, 32.2583969551414, 267.46542256172597, 1.0453838630863588], [24.410831545543839, 118.09774212913182, 26.281436916100823, 81.020559834844633, 10.231278444480681], [390.90249677151428, 13.960956815219404, 1.7161002367493943, 6.0466284072532543, 2.2156888772571124]], [[296.39123123449451, 3.5343659762217818, 655.82198319458701, 328.71557549816686, 17.099293688865782], [18.015377442050237, 19.837157243957549, 133.89392878942127, 10.471210676698343, 17.617414726729208], [33.155834431274805, 1121.6535171825669, 10.099135606849146, 293.87926323444896, 1.2781053238598636], [20.904415416804426, 4.8863891482863018, 163.40137384818931, 1.7442508006616739, 26.495240257593636]], [[1.0591869311323909, 2.8814429150052807, 1.6690303103206343, 38.727275919760537, 23.266495230640924], [25.828811173538082, 6.6784623207053349, 27.586295757500771, 2.6589309689943934, 6.6931138039711238], [2.1086773467277942, 1.106112294363033, 12.602552275283472, 22.784376516316588, 3.7386392573647256], [18.490868435715413, 2.9547943621513197, 4.0835516086849957, 9.2156266155549638, 26.057731938946347]], [[22.9421270776105, 1.0176641949967171, 48.137621317381864, 5.94418977386667, 18.174148340645431], [2.9226224437851163, 1.9442977934380243, 45.43646129077684, 3.8379361105603707, 7.3636602134314062], [3.1074604120860676, 1.1424063866538641, 32.051152268095827, 83.413553628980111, 4.745500875188382], [48.568218502724548, 11.994874699189639, 77.188242082341844, 1.4936765828741554, 28.010491867673103]]], [[[1.0717051535841384, 4.6266427788914433, 43.312210000295188, 2.9303201203766061, 3.9227847843063537], [4.3239708754506809, 21.118795831233655, 1.1192750168757133, 43.839527317094792, 1.6130495296055121], [11.920607061408997, 3.5136058661843417, 3.802439508019015, 3.349612334986845, 3.2227557174282455], [38.956513895968634, 42.443427300668695, 39.127143136347811, 7.315542870253922, 27.857360957536962]], [[0.013939891597982279, 0.49477736798783278, 0.30212884937088563, 0.06623390607932611, 0.016265682640231129], [0.35959923124246307, 0.45256603966889891, 0.11447118180944392, 0.041114742185779457, 0.01677917384320184], [0.064057579960362862, 0.10556553819990724, 0.058389339031920603, 0.059041084718201825, 0.30527747672049599], [0.015914549801624663, 0.096132695107331176, 0.035980010702451909, 0.15028227945079306, 0.98681703816318744]], [[1.1451361647882976, 1.6976931122962688, 254.49598540254485, 4.661898202065931, 121.74552389708654], [19.072820441820927, 305.25741903193017, 3.2549409825259912, 2.5903716978090476, 246.84486720497409], [3.337229184753598, 278.9961423814994, 172.89099078594657, 2.0032589241742951, 111.62859137492238], [62.64137399931343, 154.55037439913022, 6.1535667689864937, 188.98874240568361, 10.829679424445542]], [[0.021765248729131714, 0.059227495250421161, 0.066531291034730489, 0.14104014053905184, 0.52742813516487463], [0.76708664899590828, 0.060768285259923692, 0.098539746442377757, 0.95827092786438528, 0.12652873562251413], [0.21374079402366955, 0.027478142407270721, 0.26869035172951444, 0.86002272045693073, 0.093832755686492755], [0.05579920745546265, 0.02424363210171197, 0.57015650182499567, 0.083716249059426942, 0.33930396750200864]], [[0.4877555068057165, 0.21369524814314203, 0.78823719591459307, 0.53847219768717158, 0.4893917213682939], [0.51754022731599636, 0.29479416877666581, 0.40063572819174847, 0.26471705566248804, 0.28758176459581769], [0.24635615378824874, 0.98380135869610053, 0.63117916416812692, 0.98054903804169968, 0.55657413836955094], [0.31342209495539125, 0.98436725855193119, 0.76295140721086718, 0.72758164902444078, 0.94349816507425355]]], [[[5.9169769846863369, 1.8478051807536995, 1.4627924667252399, 3.8341654904577531, 3.3779744627094357], [2.4362307130610086, 3.0493481096067176, 1.2169482334179782, 5.3877712550885253, 4.410636719987207], [2.2594277761706625, 3.8963930964492168, 2.7822924415382975, 1.5280745589114628, 1.272484049333406], [4.3831848036240419, 3.3954353715872325, 3.8959781231822506, 4.2770418616964001, 3.0693128300580819]], [[659.26512847066067, 8.0983483702115837, 1632.7301456603877, 69.083452671490448, 36.162216838333009], [2.9725184301058909, 19.315551193347478, 128.63051184160688, 4.4427094567295864, 25.752835531088046], [67.531549694903987, 696.008931781451, 49.966927778969861, 90.68815788206885, 3.6237635555583272], [1170.163018142546, 1.3103463255163992, 3.3987896233080868, 9.1781849551715773, 500.70850864299081]], [[38.061814019009574, 93.646337633214898, 2.6401851154382463, 13.048233222984226, 1.7010056998659298], [39.784441543041297, 14.330880301530023, 110.07100398207214, 21.0671450673515, 33.398933997767983], [54.960269291648196, 1.7890348155253659, 4.2909549557042368, 4.5528277290415291, 7.511827297707641], [1.6299989385634843, 32.074601409509256, 5.8236290620727402, 31.676726824183358, 29.502100324873052]], [[0.11558784015962237, 0.10059621511370127, 0.4138018559827758, 0.5959402187116587, 0.74881557872665772], [0.013012833234665958, 0.076523384485115656, 0.050412526973770466, 0.044602589069300348, 0.048871185614954848], [0.13569376341408274, 0.090550153347218137, 0.075897993078273704, 0.07388917967198666, 0.072747564352401081], [0.7553538545885945, 0.027538484591555439, 0.064360130876232907, 0.42457997988952645, 0.084277835005800661]], [[0.099672439732655813, 0.00041461581102410856, 0.048771201935430941, 0.079619709357886786, 0.0033188984900272031], [0.64809658182669228, 0.035117988762436644, 0.0215587371987584, 0.05503270955762412, 0.50403956341569267], [0.26418396440791952, 0.067221673378000349, 0.0010149127329937598, 0.0017387175151985842, 0.00043749845129864383], [0.020293068674779015, 0.088604556038632612, 0.00045981608936278496, 0.0052393426288735636, 0.8767325874093207]]], [[[0.038997824143600944, 0.70276043259184839, 0.74712754278554105, 0.044244225893730969, 0.2951151893433055], [0.30614357784929053, 0.42746665249344429, 0.24668617962876072, 0.049466452021355342, 0.037385343729099733], [0.35902477869699212, 0.11280665460191017, 0.12807222957157199, 0.044148286870681203, 0.12601686003183921], [0.089272840509573251, 0.094162406535044402, 0.10717924918056274, 0.12563085741967603, 0.10659662356676004]], [[124.33367931389871, 67.688211166662398, 81.423665171277619, 455.77027017228704, 15.674445341486589], [724.97348536489767, 15.522558710888063, 4.3335375858156295, 11.343769843662075, 89.928705262747968], [383.37444551490711, 393.36315514854664, 1.064196989986155, 30.175218505480416, 895.7907618943193], [9.0348094647781494, 16.778525779909419, 696.56056958166437, 36.082185819545501, 2.0749184479861058]], [[1032.1081437176081, 1.3266455142541937, 168.31114296299825, 121.36609155463002, 746.78453151946189], [494.32303152453437, 17.396630786876983, 60.846839637724543, 549.98414960769321, 1.8599928685035443], [446.9789811077535, 103.13812853493245, 672.11881352573778, 1.7322510540530949, 3.8709971116681241], [3.0422744185110036, 22.283620648707959, 33.003783345711746, 64.945230338951859, 302.7766962099804]], [[0.35546934253645945, 0.40168610778015312, 0.70697452323641474, 0.71265737548132368, 0.25942480146071029], [0.87204564309434052, 0.063932796050567703, 0.12330556135781642, 0.11941307852048062, 0.43987676864836445], [0.11038378986494193, 0.10131830172392731, 0.10118484696992834, 0.27471242200785762, 0.78658795048896413], [0.13271310319608276, 0.098859747444357257, 0.18639955581994061, 0.69676910800516867, 0.076254280074521036]], [[11.099319362680394, 43.581534110724412, 3.425509693950699, 41.511979929225831, 20.945483950663242], [36.197939958130803, 4.0686178973746996, 4.3583360811888356, 45.452390701487218, 3.2138783165729676], [5.1729819191590893, 3.1455368578858511, 1.4426678849868231, 18.065137269534148, 3.0515190228591313], [2.3964819777939845, 3.2085630868785557, 40.335244834710352, 15.65194446988367, 91.637645842873724]]]])+(1.-msk_ref)*numpy.array([[[[78.406509424072254, 4.6965552722781974, 2.349072844017225, 413.13590746213964, 35.397210719200075], [164.36028660224392, 300.72093482632425, 6.5477156993971075, 1.3075375327221384, 12.676072934608069], [106.25428274700575, 21.306212925868287, 3.6859532356270419, 285.39034035449697, 7.3350406575213523], [47.562442153544907, 15.951987156720993, 230.40961356762816, 267.93026962546395, 370.86337759363653]], [[9.4063184088374854, 1.1733112568988873, 531.31404376484875, 176.14036417210329, 209.69713108099663], [2.3131748767382931, 17.383894672865136, 107.4107080615849, 10.79204040911511, 2.0630110747863939], [244.80677537538938, 4.2673841085024691, 321.98443598575813, 19.272064041697039, 187.20337252047631], [21.464124535888452, 13.721200728332153, 41.294414279617975, 251.10684537746732, 118.32769195493131]], [[115.04431784905273, 87.177147272696416, 30.504291762787506, 43.667456727045078, 5.1956591047761682], [1349.3367600732129, 994.92487208527609, 7.2159068323772946, 418.01160269725244, 69.768452058486858], [1.0387659484686327, 311.07274642805038, 27.266341353913816, 60.024166429958775, 19.293253208210331], [19.620413001287648, 5.7609060820696838, 45.20177344577445, 189.41386660506086, 210.33641927779001]], [[6.9654979180221854, 3.1473897081459699, 38.923209702902689, 34.424485305094557, 9.0359920451204463], [1.9311470917690459, 4.1861723173603984, 29.404892198295716, 25.004906378395351, 14.983458633682456], [17.537650973951262, 7.9053834999187549, 2.0533418456071284, 24.934278995695625, 18.306605232372135], [2.2627117174009772, 28.58237163175642, 13.179378009877027, 53.454966360682874, 47.894049623706564]], [[2.10429624543865, 5.4393676783645439, 42.03909464973291, 14.033649088056391, 12.173083299758016], [2.5611311921967714, 57.703046036132491, 1.7198497786003675, 79.442507327323, 5.3533865899733142], [18.083161870949965, 32.114122059630091, 4.1736737693636821, 7.0069791685132774, 13.540682318778497], [1.5638617254849836, 1.8712053543707523, 25.111470043583459, 43.798228665334577, 30.846232960427237]]], [[[3.8389784138074461, 3.1847435024798458, 3.9609282306422178, 4.1907810049239309, 5.2651560729495213], [10.275742694619769, 4.2660565080810446, 1.2407977605210736, 11.312360413411158, 14.741178711484043], [28.408990955510809, 11.359585061278585, 19.695453841299607, 10.837024100700821, 25.115600104831554], [5.6785146002327336, 2.0177853637690473, 1.4503805475540397, 44.210897038153469, 3.0724208649696538]], [[0.10103183031795171, 0.48912621395159961, 0.76655638694672901, 0.24795484185296851, 0.047970617308501405], [0.084852171543899194, 0.12458323714551429, 0.016329906940323751, 0.029326924643824654, 0.66655572842493649], [0.14450965766263671, 0.53311805459199402, 0.25212359746660873, 0.14465205042479509, 0.048000920047979213], [0.18119102655028962, 0.065008851571884768, 0.38430790403557069, 0.94791147263925446, 0.078046143042242694]], [[2.5600924126538551, 1.4639381114187275, 1.6882054476574173, 210.28547697450207, 55.620085325422806], [27.893567831712058, 12.572007430820422, 3.1177165273258263, 6.5296486320461833, 2.5600064990241505], [9.1037220031142549, 73.841433958552315, 7.8306301523599444, 125.17994849206698, 205.34830994689099], [2.1756582195648844, 40.866067044822152, 87.479823720232218, 47.331191755191533, 7.0097902905102929]], [[0.044035087880939211, 0.28166724868777232, 0.79594105158390349, 0.63077564456185764, 0.96585954199489943], [0.82081500723689815, 0.19474048203778771, 0.98722087665203562, 0.14428064900026233, 0.094440258790925866], [0.028119930002760227, 0.11823638705816729, 0.083014778361124031, 0.063923094824068452, 0.3898851994372538], [0.19544374968993944, 0.29335932336770709, 0.039504182495780943, 0.088552963635007725, 0.42200647060222912]], [[0.69552677562950949, 0.30707472022570098, 0.2637988934294852, 0.29758999965164223, 0.31415727542519661], [0.68594264110411518, 0.84835464878417544, 0.38582099450626456, 0.36519430021953464, 0.25117598170279593], [0.5227103022558931, 0.8263054667355878, 0.59346848778695882, 0.83724201832135359, 0.63078632530856094], [0.46630261167604192, 0.22223422520794278, 0.23845958068384013, 0.90379305373023333, 0.56764720868105667]]], [[[4.6811797733896077, 1.6486680551536046, 6.3630067240785806, 1.5696421385701445, 1.1365984618624589], [1.3034617853083614, 6.0621457356879604, 1.5186331058897593, 4.3266441112183394, 1.658085941673604], [1.9750732576264816, 3.1425745513278107, 1.0433383407935006, 2.7825538569562611, 1.5726418315299475], [4.1058002532135527, 1.3905577191735023, 1.0583179110768526, 1.9988970796313363, 1.9443059719684739]], [[24.597655399952206, 14.546461611870223, 3.3796377814026939, 210.18920242713668, 534.38855027390719], [768.76254160501799, 7.3787177376525594, 4.0478078987919774, 924.15704556571973, 116.39242772516312], [551.85169789930626, 6.9468921446360055, 1084.0618281852437, 668.45460678082736, 14.010306188992681], [4.9447316228761888, 2.7794222956397121, 131.33769148228959, 157.97386731328683, 89.658039679803451]], [[12.737868293233824, 10.709115736204799, 116.28185804283063, 23.915019465742553, 33.668767846957273], [43.11559852605302, 67.49833144580569, 29.484535919629796, 65.183560686882132, 4.7595368369900122], [92.240713189535811, 87.370976721137083, 84.739280389422021, 1.1440239037220152, 16.644009214104297], [93.356213408445356, 4.5296147061141721, 8.2473283438364771, 1.6871839339123409, 14.178268462029216]], [[0.01238730369744408, 0.081605024753112806, 0.054707679665997794, 0.25112193468154009, 0.35984136931730909], [0.016741674266770697, 0.40936191911898112, 0.34118003914860801, 0.019099068157754064, 0.021829210173224957], [0.72615772625577779, 0.13636427121570011, 0.92832453584268426, 0.048909361080732038, 0.96467382398568169], [0.53232354594538345, 0.064000133853662114, 0.085373732712884995, 0.36015319150509645, 0.084398457406430913]], [[0.15789143622637727, 0.00047598562445131808, 0.045178935325232879, 0.0049831506372526573, 0.18289820515662036], [0.53025125588322597, 0.0008281679001680796, 0.022290318149170152, 0.2292026682516474, 0.0031930197936969359], [0.67990482696092747, 0.19833925037088215, 0.0069030273201501581, 0.005238879338766462, 0.011213909374566783], [0.0032929106096041723, 0.0030596037363150529, 0.25474819670706195, 0.0015559362018386519, 0.00062045384853855613]]], [[[0.09764708589529135, 0.88554351659392527, 0.16762573168710648, 0.040393388453270514, 0.3276730942715691], [0.052500786277275738, 0.088619951660260377, 0.63814549388211594, 0.10550374255634826, 0.55692554601407762], [0.059625263679851409, 0.2164374715123224, 0.26296278950589042, 0.062772438156587493, 0.11264281668967348], [0.12260750914933183, 0.18377698577970888, 0.038229987829925358, 0.069685897391369406, 0.26820613870053323]], [[354.0671414454099, 667.88118746468285, 72.209986339069644, 1.8463752143944991, 86.16784852923044], [66.816021170738154, 245.84363263772013, 27.120254590591095, 3.674907121771199, 329.00499457371046], [42.864048159290157, 954.79369834087788, 77.498416407145314, 8.5896447878123325, 40.310557072028189], [1.1553509218450475, 731.18783960005987, 1.9362021346522331, 255.85861454076661, 19.412216252911598]], [[2.019227565974326, 17.20984353350795, 11.484387182114887, 34.419784866927664, 158.28417092574011], [2.8805398955121406, 69.179079713875382, 111.46543314567383, 43.14402069233067, 1056.1181002201965], [2.1720508195808974, 96.362465954754626, 16.989090933109072, 413.49045608756251, 634.27802326527092], [3.121827148277458, 580.2985733713457, 56.844376918689285, 43.205511785412952, 103.86231549156813]], [[0.16907848388383317, 0.71088040329247604, 0.78570392703752479, 0.1361193042660658, 0.12806857349155887], [0.35332657551338092, 0.83197355129747452, 0.65223635300652749, 0.69438109380131441, 0.18508624024834391], [0.067832880294430464, 0.30753761688349879, 0.81619534744349687, 0.64624239237348158, 0.071191220096133587], [0.59886313823189352, 0.081889107982700618, 0.50400011685196078, 0.97005691882149847, 0.1339422149999808]], [[1.408022507267322, 59.120313408828949, 9.6970719691638205, 1.4142628339220635, 1.4106177740870653], [6.7604652339883557, 76.54678709955715, 2.4534552565709387, 61.622267977390493, 8.8491675516320569], [4.1628668353422009, 26.801958487548312, 1.9513787170045662, 32.779617168741964, 49.125572713796117], [2.3836128340907106, 22.342910927757142, 99.394401952539141, 4.8058800732184714, 60.673407147928067]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[0.83085015440632881, 3.9123264788494581], [3.0201526093096041, 4.4830702854237341]], [[1.3940752901367395, 3.6846997155096011], [1.2289551355465522, 1.9677738036482435]], [[1.1965482136339496, 4.7587150560255589], [2.4923433706707439, 2.1253089937168284]], [[0.47113956178333921, 4.520433899416755], [4.7101195859775196, 1.8821702580714446]], [[4.3135011452357928, 2.3529100316388534], [2.7768332795491966, 0.18020114328576636]], [[1.3650211379568389, 3.4375224097799331], [2.6324261267690314, 2.9642972726069159]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.45969031995358967, 0.44821046807326564], [0.46508755091726239, 0.96080681236266408]], [[0.27968564891777575, 3.3284738564198477], [0.98311574279286551, 1.8927126651248012]], [[2.1137148338028853, 1.499876926114575], [3.2638391474203274, 1.7643464337062447]], [[4.8130292527842276, 4.7761399680159879], [4.733065190923293, 1.9885691187709016]], [[4.4377769230693085, 3.2703615882313972], [2.4750166210469664, 4.51854852458988]], [[3.0458110831742093, 4.5387343082677436], [0.46467591979341116, 4.1589292311144606]]])+(1.-msk_arg1)*numpy.array([[[2.3189074255006812, 1.3766786511032176], [3.7371940871684965, 4.0149406758696768]], [[1.1995680745238997, 2.8914349615060142], [2.7334375739872296, 4.8695630870099835]], [[4.0287633281423751, 2.1618424890819963], [3.9954069802754471, 1.8645485491232709]], [[2.7798832908576454, 0.045483136920189356], [0.66094500011163515, 1.2942907590239925]], [[3.6231051133163064, 2.5682404466691051], [0.24181403753707353, 3.6051024203697555]], [[0.4344793974615222, 1.4368297976933846], [1.9829781622362093, 2.2492761853501939]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.91834394416798448, 1.8430435969999053], [1.6720738702627114, 4.2270581770447579]], [[1.0973743068392663, 76.781145097840167], [1.2246846638354161, 3.6008956509198899]], [[1.4612422658694233, 10.378896230074517], [19.699913010617472, 3.7816904710038619]], [[0.026721407910359828, 1346.5794181864285], [1532.8653862582746, 3.5170473847138974]], [[656.49822884378932, 16.416620882807234], [12.52544433108211, 0.00043361504225077183]], [[2.5799364669567906, 271.56639139049611], [1.5679398583935984, 91.767558912446788]]])+(1.-msk_ref)*numpy.array([[[0.6506996945027399, 6.5402257493310447], [62.224458223680024, 413.08285518533751]], [[1.4896394131223554, 43.4225012745624], [1.7568761417807823, 27.010377809949052]], [[2.060451246679694, 29.149158449543446], [38.424645726309357, 4.0784426926014188]], [[0.1234223926109199, 1.0710250960309473], [2.78507626051733, 2.2671929607418297]], [[199.55174084717837, 9.0026800632909314], [1.2801373510519634, 0.002074577738985861]], [[1.1447621850981597, 5.8951324617743301], [6.8164326830541428, 11.5207927368845]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[2.1053117329856699, 0.17146037506187578], [3.5847559370818414, 3.4845823421534785]], [[4.6955029723949249, 3.4992453707863334], [3.4687862491453227, 3.192205426994287]]], [[[2.6815059414558955, 4.0064265752961656], [2.7405832446792995, 4.9774701410979336]], [[3.6728458539256184, 4.7031526686331837], [2.4540099476063881, 4.5288181338798319]]], [[[0.9935309215390804, 0.51357850296807384], [3.1260899504101634, 2.2151740877270973]], [[3.8888610250130631, 1.3255227459417196], [1.1457438783747516, 1.9272799576971187]]], [[[4.8108890455648678, 4.2843750400428213], [4.8115201066067081, 4.0043736076067766]], [[0.74076873791540232, 1.943890930106702], [0.70502161008805297, 0.57619114306878838]]], [[[1.6456677206297672, 4.8006744998622768], [1.6465266374236469, 4.0256863563600307]], [[0.93048402996075286, 4.6840147412432813], [1.0436555617049474, 2.5578866668256026]]], [[[4.0234725906129274, 2.1140493117740671], [1.1790464913145902, 3.1072974811208023]], [[2.7091995820676185, 2.415346105767088], [2.0591097157094507, 2.6650287978120129]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.6319448741458547, 1.6731175103025691], [0.33871991722920552, 0.20332942425391093]], [[1.1213773407040832, 2.8293309093523797], [4.2247542422737858, 1.4868252002544911]], [[2.3710330290816328, 0.65482134364831046], [4.7749236680854867, 2.6796661971140638]], [[4.5430847036204289, 0.35948728145832332], [0.8721414784626047, 2.4822029226241455]], [[0.30924026392504778, 0.78998862894870314], [1.7425622868934008, 3.6561547883348333]], [[1.2187033437600356, 4.8120135246398412], [3.0779335840078601, 1.7759308645947529]]])+(1.-msk_arg1)*numpy.array([[[4.5576869980851829, 0.33994653215249587], [0.038180449657958491, 4.3808211696354995]], [[3.1734587098028757, 2.1063769994715713], [1.0873179208119146, 3.4477829212136615]], [[2.9402689621303906, 4.0315953235368394], [3.1426422214229324, 1.5583179472650819]], [[2.3927525638692773, 4.9875349661277824], [1.6287583456655348, 3.6210205162237323]], [[2.1947490377108472, 4.563284552525535], [2.4674104262587866, 4.0349699158663688]], [[0.41368598342457952, 4.6031625658796349], [4.3132357526632221, 2.1666371347031426]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.3700361486241399, 0.056259709978123011], [8.4659569027371102, 8.0738724121075478]], [[1.6885429505491096, 1.528463114579556], [1.2877572036500966, 1.2661830590410417]]], [[[3.0225551841160065, 4.7415272683649077], [17.330166223622765, 93.770915390219542]], [[243.77971288743322, 692.91283140318762], [3.7990755994792402, 9.4478905133223705]]], [[[0.98472956740900874, 0.20598673421597219], [2.1092959942287259, 1.6833733237163544]], [[655.17459425579716, 3.8405149894982262], [1.4399085098667339, 5.8017226313657506]]], [[[1257.2090835968161, 742.53742425568237], [1.7590241864133789, 1.6466584300498293]], [[0.76974137286191979, 1.7855118297453478], [0.41995970862615534, 0.25449406617592929]]], [[[1.1665455565649965, 1.6243742295216734], [1.4828146175858667, 3.0048076174255045]], [[0.88200967787963669, 14.743278754119514], [1.169090054008515, 30.993966638843045]]], [[[5.4554341470251693, 2.4901119686762487], [2.2090614842739362, 234.07382480150133]], [[21.490961702778687, 15.093330073885145], [3.6063932702212456, 5.7018679404759007]]]])+(1.-msk_ref)*numpy.array([[[[29.75603095378354, 0.00032326461527185465], [1.5434288566471912, 1.5286295865930357]], [[1.0608283692641565, 1.0489847803022438], [232.49905809895651, 161.55923440322672]]], [[[22.879278159325224, 81.813514265603629], [8.3610808082574941, 29.387525375544371]], [[4.1146941861202979, 5.3839459674005772], [22.090671297709005, 182.68103721096151]]], [[[0.98109829970573414, 0.14096336715171232], [99.002374914945975, 24.691343617872946]], [[71.383995110650829, 2.4244829467650155], [1.2361671050512804, 2.7799347587271095]]], [[[42.893988730026607, 32.505016371436383], [2528.7555728751099, 1011.9576492972686]], [[0.61340105962833591, 2.9524105977228139], [0.28205764873673983, 0.13583343002533699]]], [[[2.9841230478149625, 31.281493559818927], [9.7334069643639438, 575.51511669051399]], [[0.83712839262520733, 45.153401267328796], [1.1881676144915239, 44.237315791835428]]], [[[1.7787502689085497, 1.3629990884456207], [2.1343640647613036, 184.72183105900811]], [[73.611433659478024, 44.862174251399942], [4.7822208282283407, 8.3626045133113536]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_expandedData_rank4(self):
      arg0=Data(numpy.array([[[4.1638691590527106, 3.8969515960594308], [4.2164250480424599, 0.9669118160391631]], [[0.3483612556942618, 1.784295099715087], [2.1969731620187103, 0.49305087370211442]], [[4.1431280604063252, 4.2137348853349792], [4.3573111911548184, 1.3345029126298951]], [[4.531326750709832, 1.8019507072762111], [3.809164791492925, 4.1207538346135371]], [[2.2354706552153503, 0.53510135958677341], [2.3437122897034648, 2.9299921705860941]], [[2.6513735238693252, 1.2179255523098536], [0.13239596929905342, 4.6598892399551204]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.1877917514651371, 0.51011818802115894], [3.8688354905170761, 4.8133435334089523]], [[2.5184787800832562, 4.8549125005642093], [4.809620100564783, 3.0297659720860821]]], [[[4.5594988584345444, 1.1308741296458473], [2.5425229801744091, 1.4362029105262963]], [[2.0847934299685562, 1.8389879993421121], [1.7968342727070594, 0.074755511381705206]]], [[[4.3606977295490026, 4.0495581672470173], [1.5073477226274807, 1.228092371770114]], [[4.9020751919855314, 2.7586164275411518], [3.6335734265976947, 2.7750044310516802]]], [[[0.53414048664617952, 1.0553845321488486], [2.9998958358139687, 0.55273297844447034]], [[1.2999775495909709, 2.4748238357936989], [2.2351666078717569, 0.1567980930932607]]], [[[4.995708536857423, 1.4160653411490363], [1.9320950554439793, 0.77578194756373053]], [[1.452015085632822, 3.2118809766960799], [1.7128768791593607, 0.27940701809527457]]], [[[3.3818098518294919, 1.629869726004769], [4.5837503821590486, 2.4760774517710096]], [[4.0643267306398609, 3.7470501054830718], [3.4855967396048317, 2.8352947046336965]]]])+(1.-msk_arg1)*numpy.array([[[[2.0640643679665409, 4.8953965352481097], [2.7397393699755761, 3.6755092109500693]], [[1.6306926662034484, 4.5502246024407142], [0.45895060745521737, 1.1329069202751556]]], [[[2.6064282019030132, 2.7537086164123932], [1.5124791016410093, 0.79513678098962126]], [[3.2354799796341975, 3.9453190694321361], [1.523201913510936, 1.2859563192086065]]], [[[4.9809367151926072, 4.6003722309187394], [4.4810844117197437, 3.4328388346576699]], [[2.4939663134327907, 2.1822328893195007], [1.4529266273445178, 0.95525905487075979]]], [[[4.0585472835091823, 3.0752744056695138], [3.5192551928887106, 0.025952228545260608]], [[3.2504159528972503, 1.8947734937264089], [2.9297230594215917, 4.9779566730351288]]], [[[3.8187296883138666, 3.5808269764893734], [1.7652846824945281, 1.0805482172649861]], [[3.6190129911013522, 1.3245319254781858], [4.3356390463582475, 4.8032312138409585]]], [[[3.6050843445244287, 4.94461903196681], [4.7588035611832797, 4.0617600741394604]], [[2.2709270160447614, 3.6097095812106188], [1.7111605557052219, 2.1048069198495605]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.307183148700779, 2.0702210729965493], [192.93805206133931, 697.20808388134458]], [[37.489458839845845, 1081.5592229845395], [0.85058238448221046, 0.9030787609420553]]], [[[0.0081636732103465004, 0.30345437427244498], [4.3587285150608199, 2.2969798302940885]], [[5.1598134049619855, 4.2521797622838688], [0.2806579443384728, 0.94851011024753762]]], [[[492.01955681152151, 316.16012209186499], [8.7415952272798609, 5.849907954133772]], [[1359.8704332193483, 57.990769132071101], [2.8533674290405844, 2.2272154978790462]]], [[[2.2413854264251163, 4.9268587968083191], [5.850622547360933, 1.3847058866505284]], [[5.6893892785360523, 27.381128169899476], [23.690559665796957, 1.2486110365795267]]], [[[55.634675905265745, 3.1241314178970061], [0.29875325347969095, 0.61563824776923992]], [[3.4443447958627442, 15.420149199802953], [6.3049952646394134, 1.3503481861248963]]], [[[27.045692934430118, 4.9000715830739061], [2.4686760830273737, 1.6293094972998949]], [[0.00026978147731319095, 0.00051241370873112324], [213.64256908750031, 78.531118453655978]]]])+(1.-msk_ref)*numpy.array([[[[18.996860650668168, 1078.1615280641518], [41.536755560021888, 148.32529500407745]], [[10.449420948211339, 697.64954304864534], [0.98467586667318274, 0.96259739285233148]]], [[[0.064022654735740453, 0.05481300911263777], [2.4007011606918587, 1.5847122265618685]], [[12.76342234924785, 22.315555959597546], [0.34057447798338603, 0.40278397449125269]]], [[[1188.1560384366501, 691.73576650424013], [629.77655225657736, 139.43822617858334]], [[39.281638928434852, 24.826974799552737], [1.5208279177452153, 1.3173846899520834]]], [[[460.59766634949546, 104.24929228620424], [7.9437314193614998, 1.0153998580622989]], [[77.257085862103196, 12.604941555660817], [63.344853389710664, 1151.6678025765204]]], [[[21.58468278577995, 17.825012552446417], [0.33159832380559401, 0.50881752688950554]], [[21.811646853348215, 3.0899400338160738], [105.72177663565793, 174.77026818024723]]], [[[33.623921779599947, 124.13731471949077], [2.5553611785988264, 2.2272599715826868]], [[0.010135418295065566, 0.00067643009321342537], [13.921882150104999, 25.515352678620943]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[4.1503484720240937, 1.4589046431326274, 4.1283134609195979, 0.15652211862220608], [4.0129809178819418, 1.5745053554566852, 2.4000764922597817, 2.7172162868538319], [3.0620658085652792, 0.86504589609446936, 1.1868981863484163, 1.0461586824744549]], [[1.009915350101547, 4.1706827548215637, 3.8278299376971345, 2.2945627554729509], [4.7608554582160449, 0.89564831698268688, 3.7796175850174722, 2.9555065451127884], [2.1172078262865104, 2.7324909336986112, 1.9893299467857504, 2.1774717618727712]]], [[[4.739971070092226, 4.2808512720703114, 3.916776552581839, 2.2263817845378471], [1.433317003008004, 1.4230853718893044, 4.491016197788527, 4.3629863127432964], [3.2093850505328581, 2.4567012031615727, 2.2095762945018218, 3.592549347795805]], [[1.8131557143742341, 3.9451139143271008, 0.25874007750702427, 2.8837448937287822], [0.21271615167693833, 2.009677802454009, 1.0497143521235053, 0.68730933267957772], [4.7830668566597581, 0.044114366674847569, 4.9165622698784892, 0.2606076933101602]]], [[[4.6036618255362285, 2.2160388858922735, 2.2154920392234771, 1.5868230798326368], [4.8177110862754358, 2.9627786950305621, 2.4255702128653436, 0.73857566630357008], [4.2787762236145879, 4.8376573347379077, 0.10116395945696734, 4.6514529818083581]], [[0.74948845366901296, 2.9081001628588488, 2.9540463369269525, 1.5523341115364664], [3.7647721224677184, 3.7332162380932852, 1.246812524443254, 0.36794831767471109], [1.5620057355679855, 0.12508229866776416, 4.259348694261571, 0.40108441537481337]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.4682283308719972, 3.8354761933056221, 0.78789901457656897, 4.8716429559374621], [0.94424270581466063, 0.68444001780060804, 4.5813702954218058, 1.6356732817853874], [3.3098782337091346, 1.743590158875189, 0.24022759925781223, 1.3468995142697961]], [[0.53740704481106227, 4.7898506624421175, 1.3899004865639393, 4.935920135739134], [2.2652703108870593, 4.5553529152757157, 1.5311559709593086, 2.4738713085919866], [4.404586695060301, 3.8483281402702616, 0.36047598931427971, 1.3922585649570907]]], [[[0.52350196984306208, 4.8463655838180246, 4.5125011186706114, 2.22685358174299], [1.7594174362400776, 4.2921076193002259, 4.6557022236556831, 2.3931703836649669], [4.8167500369441631, 3.7901097076704469, 4.4629013258304999, 0.5744637130776078]], [[3.9802294290740372, 2.5315387488570567, 3.9879517252478984, 2.6362019952443836], [4.4194933314373772, 1.3823143097322579, 1.9404518230397845, 2.749402736959774], [2.5721078161420601, 1.8549060151126782, 0.56184322240424067, 3.3282793786273568]]], [[[4.6514872015665523, 4.0350547092937372, 4.464858984399144, 3.4020435475656483], [3.4455573643391224, 4.1309581860535518, 3.6687160982034346, 3.4560880112474495], [2.5956754069738253, 1.4446384415158868, 3.328135397070354, 4.402688779115727]], [[1.4254160510225147, 0.90911384769520687, 3.8313237390003589, 1.1750328319040106], [4.8018034239975815, 3.0063765089450558, 2.7718466458879321, 2.5340597992074638], [0.80903561452792438, 3.6680144033492983, 3.7126165671597549, 0.30374101492690497]]]])+(1.-msk_arg1)*numpy.array([[[[1.6623238051528897, 2.6816323951063481, 0.67707223496479896, 3.3276288289894849], [2.18291699928132, 0.51970367047828947, 4.7099628286231123, 0.013876384250646065], [2.4346270448162426, 1.2410814383501587, 4.6577315016253449, 3.0407919643107122]], [[2.4941725746899697, 3.5048974661775634, 4.2192210318990968, 2.8635300810232707], [1.9328977853191573, 4.4348717542126774, 0.68376949405740139, 0.31617757412452618], [1.4238748062014248, 1.0037830039200015, 3.9066545472923009, 3.5205706386690214]]], [[[1.2567823038604655, 2.4031110283371762, 2.174474388936853, 4.9572342913686542], [4.8133011452482029, 3.3912341914449482, 0.40134995162177006, 4.7591705126411217], [2.253146800044517, 0.15259375595519994, 0.086409481907702729, 0.70231103284734708]], [[1.4583598571769305, 3.0066633191272052, 3.2146696937225054, 1.5272445600288178], [3.4173075435041409, 1.1900433284829506, 2.5627842537480183, 2.8444075678217309], [3.550056652964662, 2.3527745895104086, 1.0026968865802466, 4.1545125247187347]]], [[[4.3711954324573554, 0.67948596089850155, 2.4425469488933662, 3.1015498696165262], [0.74895396761766031, 1.207493554822886, 2.2374786646575746, 3.6037954006364901], [0.43572741811353316, 2.7165252150150221, 0.98006012484336891, 4.359517112535972]], [[3.650822403175066, 0.80086718596944118, 0.80602917668163776, 2.6580046947141658], [4.0437318708849022, 4.3726074878414671, 3.0588777528566182, 2.5541644801418304], [2.0170697114055032, 1.6897835637014056, 2.4069159352636742, 2.1407492109355233]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[577.75398963602515, 4.2571729586889129, 3.056093776475004, 0.00011919566711347496], [3.7138076142246859, 1.3643737029020178, 55.20163000700984, 5.1296220762699152], [40.611486051063416, 0.77664416150429672, 1.0420202727350092, 1.0626639534085036]], [[1.0053164172264235, 934.75900642048248, 6.4602027854791872, 60.309551218588545], [34.287806454998595, 0.60529834373817382, 7.6588354737912914, 14.597645653570641], [27.21783248654447, 47.865350583997326, 1.2813761509217674, 2.9547266327041011]]], [[[2.2582390102871495, 1149.8096085682939, 473.79650902425107, 5.9436537784558947], [1.8839582992471708, 4.546568590234191, 1089.2367975332083, 33.971268436789863], [274.9839498290865, 30.163253751500285, 34.404646597851361, 2.0847727494524468]], [[10.681469448230724, 32.281064093234214, 0.0045554238901022925, 16.313199051966009], [0.001069593237324592, 2.6243152829974234, 1.0987212458684041, 0.35666946129835664], [56.011638382682214, 0.0030607184347728965, 2.4468387556604441, 0.011382622900089654]]], [[[1214.5545990502246, 24.798397781392843, 34.871907085274671, 4.8106827619943449], [225.30300407126686, 88.832058363146146, 25.80909455247583, 0.35088234742397789], [43.521182985002476, 9.7510231586729184, 0.0004881926185035757, 869.33065415786984]], [[0.6629609448958711, 2.6392084301173138, 63.433836944157591, 1.6765398530244346], [581.54594523040464, 52.468340928301124, 1.8430881779598738, 0.079373922046640483], [1.4344854281175616, 0.00048809166147149803, 217.02510418238231, 0.75768106724549966]]]])+(1.-msk_ref)*numpy.array([[[[10.652626051869847, 2.753340139046998, 2.6116957056846655, 0.0020885599159312799], [20.764361413161126, 1.2660665488241303, 61.77973036492822, 1.0139675916579196], [15.249798135308872, 0.83533457572443381, 2.2212525824486096, 1.1470757920054762]], [[1.0249140880262366, 149.19782446388797, 288.14163567263989, 10.786365399659044], [20.412482643890872, 0.61338904235138425, 2.4822254935839667, 1.408650521860084], [2.909688028319199, 2.7429016329373614, 14.687380912606425, 15.480527677597145]]], [[[7.0681039797773675, 32.933435954418883, 19.467423733567689, 52.860693384283913], [5.6561685736804437, 3.3085831381312292, 1.8273363029335781, 1108.7673685551813], [13.837024668349237, 1.1470050486198071, 1.0709065194627398, 2.4550790121743096]], [[2.3817237093703683, 61.965559965592902, 0.012958380941022848, 5.0404182978036127], [0.0050453054778345505, 2.2947405623058872, 1.1324023793930709, 0.34418711024576354], [258.81915192209976, 0.00064714616284330018, 4.9377247491126672, 0.0037472411421793335]]], [[[791.68900569844845, 1.7171766565286393, 6.9795459039587966, 4.1874455169973448], [3.2465066037459867, 3.7117076290934121, 7.2612732604123185, 0.33552314817213075], [1.8840101921427748, 72.414997460992637, 0.10589256783188808, 813.51221800744281]], [[0.34897077794960807, 2.3511997349826714, 2.3942531733375798, 3.2183965994788908], [212.87904148425267, 317.31824864762746, 1.9635593169651071, 0.077794361902056408], [2.4585064667770498, 0.029816481454927112, 32.71720276638144, 0.14145793848278276]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_Symbol_rank0(self):
      arg0=Data(2.60589436021,self.functionspace)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(3.1993779747)
      sub=res.substitute({arg1:s1})
      ref=Data(21.4191900069,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([2.9375880561661027, 4.9652781622954949]),self.functionspace)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(1.3816287043)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([4.4318969344779156, 9.1524134638762948]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_Symbol_rank1(self):
      arg0=Data(3.22425111181,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([4.8720311479848313, 4.520879242439146])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([299.97235594250395, 198.85910093580921]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[2.8879716996312488, 2.4017553293148244, 0.89266982589248378, 2.2888394272191577, 0.10947665634596537], [4.2180490192152122, 2.0698749388347104, 0.97440423422990241, 4.5287341306361002, 1.444870521160172], [4.8455458170688281, 4.2666702165353065, 3.8243428937087174, 2.5004139096226559, 3.0638091967292107], [1.630373944343819, 4.1967488400445365, 3.925217225273617, 4.9627711163669046, 4.1379239194209214]]),self.functionspace)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(0.965403261048)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[2.783927490946088, 2.3300417817639869, 0.89618318407455511, 2.2241998565363494, 0.11818377746888521], [4.013144512595983, 2.0184289684966465, 0.97527872557568829, 4.2981559231091655, 1.4265906415644107], [4.5880916198012418, 4.0577944982579348, 3.6509195081723886, 2.4223784383326987, 2.9473970590270571], [1.6030341719993055, 3.9935784609341809, 3.7438457643460552, 4.6952038846181186, 3.9395247946978142]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_Symbol_rank2(self):
      arg0=Data(4.57010996853,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[4.5924901187777794, 4.6769448901488717, 3.7072986632981721, 0.18356702982531134, 0.90934174432560611], [3.9273553079712507, 3.366093776107872, 1.4963118856771167, 4.2439507480162471, 4.5403469921951309], [4.340511593282093, 4.9727971090119247, 1.559494115113139, 3.479054802545976, 1.3639542875680386], [2.7174963785137312, 2.9859114842425543, 0.48483334548430757, 4.4878396620543501, 1.9339814939402469]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[1073.2641457014647, 1220.2268460206951, 279.60516459786032, 1.3217239972257511, 3.9819779556431278], [390.6307553844548, 166.48495409547047, 9.7152972023031765, 631.96962047009231, 991.50745135553109], [731.84528117244963, 1912.8518681294756, 10.69428435967831, 197.66099740920393, 7.9452905657314377], [62.136714009849328, 93.429185827495814, 2.0890769905483992, 915.47185253653947, 18.892347675531823]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[3.1895609326769883, 1.9710777945373965], [3.7237618273481092, 0.76226638799850344]], [[3.1455839713749918, 1.0291788880511012], [2.2522182417500116, 3.4483060961927241]], [[1.6104507450177825, 0.95764723177033495], [1.6448969244102776, 1.4065098897905872]], [[0.023715375333342066, 3.9165673919995747], [4.0151545683594616, 3.4932482215081637]], [[1.2040995380957624, 4.8426224451387201], [2.3292864043203578, 2.1384441032459414]], [[3.654570681599961, 2.1368323843356638], [2.2643716078655323, 3.6641822360499225]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(4.43236019129)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[170.89006013784712, 20.241059327208589], [339.46689192552691, 0.30023045246734748]], [[160.69092645762782, 1.1359625907362179], [36.550777347838419, 241.46967109045502]], [[8.2654332730180098, 0.82546043445485384], [9.0782997647991568, 4.5354690329123315]], [[6.2740455990690212e-08, 424.59140207007249], [474.05191238838427, 255.73406025375496]], [[2.2778481314840575, 1087.7373124532105], [42.429130012678037, 29.047770407185897]], [[312.38739537387403, 28.950858423422467], [37.433122798666744, 316.04541292844937]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_Symbol_rank3(self):
      arg0=Data(2.89790450323,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[4.8341066886408628, 1.2033152388676123], [0.45497376335340728, 1.4671393889196029]], [[0.18507679430178847, 4.7163549132309086], [4.9966312573847524, 1.5198424359212457]], [[2.5201944478240108, 2.5271648484150191], [1.72131240003187, 3.2606990254563999]], [[4.8066284095335083, 1.8438361613714103], [0.84845678072610409, 1.5647047162561127]], [[2.8670711214815365, 0.098727590924669192], [2.571461849392271, 0.75315243002396259]], [[2.0561906400761858, 3.2360041441358587], [4.081116891502294, 2.3290468246520253]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[171.30249650585017, 3.5977650846474609], [1.6226921238139678, 4.763670782285149]], [[1.2176459772771082, 151.130623016192], [203.64028981189873, 5.0384270877201143]], [[14.606350132169574, 14.715079679754142], [6.2429669975918465, 32.115676593732829]], [[166.36670577446736, 7.1122549693247104], [2.4663707558021524, 5.2847582236944159]], [[21.126516445206772, 1.1107605503854139], [15.425226739600978, 2.2285364407264372]], [[8.9152379508851975, 31.282824960694825], [76.881013935554748, 11.918330800565474]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[1.4570910868547737, 1.0335310047180302, 1.5135102670609959, 4.4904721333658779], [2.2197946026005195, 4.2756905616310164, 3.9917560127807863, 2.3907889389693375], [4.9517308279700014, 3.1197596472305866, 3.4297327388671737, 1.3858485510519367]], [[3.0059217475688698, 4.7929869942836127, 3.172862776768286, 4.955993886065106], [4.3570080806028724, 1.5935416420447939, 4.4120618556707214, 3.6027913926823589], [0.63833288604529537, 4.0441938411632101, 3.4005072509180394, 2.2071294708465299]]], [[[4.2429004005198321, 0.34551842355545426, 2.7410659382742186, 3.8576324118083063], [1.2902279851138809, 4.3914848722097677, 2.5263585613522235, 0.5721650275622101], [0.80959840551364848, 0.087162586569382, 4.0152918581314108, 1.935978367420468]], [[4.9955081476395593, 2.0087370274402563, 4.2057355947219177, 2.8340766205447805], [3.930479873233133, 4.0947669603515369, 2.923105273974298, 1.1150578467079946], [3.3929095089493906, 4.6875946900929364, 2.3253804121017883, 3.3094556995744919]]], [[[0.81743541040898959, 4.3086420988936887, 2.8246655473162443, 0.012343482447535747], [4.7479456788073549, 1.5977228677973385, 2.3375390172028236, 4.1118894241690054], [0.41034141789990664, 1.9670414993053418, 4.4564775296921839, 2.4420839049381358]], [[2.7961978389483253, 4.6846832006791139, 4.0097349203067774, 1.8857339045838566], [2.1833772517854797, 2.104265842595189, 2.5855718711957438, 0.4852527779561599], [0.35118899507897117, 4.3957436586370662, 0.36827379148147549, 3.3167387716747485]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(4.27033201946)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[4.9904820350890216, 1.1512407166907808, 5.869448474325762, 610.24388699258168], [30.121084226180056, 495.00121726030937, 369.12191262784427, 41.352023776368149], [926.49522561132505, 128.84314288799206, 193.08080838107043, 4.0287785785862855]], [[109.93180301615698, 806.14778367964379, 138.47248899711568, 929.90621522955155], [536.47136180634379, 7.3140677433688523, 566.02241949998358, 238.25051195765423], [0.14705740503177714, 390.27790409413552, 186.15217042098467, 29.394014572409137]]], [[[478.99249703211024, 0.010693474256700718, 74.141537836717646, 318.99557978287106], [2.9688107715959533, 554.83519143895046, 52.334398865597691, 0.092158517813117236], [0.4057709243646816, 2.9844048476874571e-05, 378.50580146395231, 16.794241437975653]], [[961.98242988985339, 19.659938259925681, 461.33066433021435, 85.496372334414588], [345.52540137635901, 411.54934524233647, 97.568695552168549, 1.5921192522030621], [184.38253543486519, 733.12747818005982, 36.732552266572931, 165.78038034744381]]], [[[0.42281185065476046, 511.49831698014066, 84.290564059247529, 7.0762849917064322e-09], [774.29085424615971, 7.3963722920512556, 37.559759634327854, 418.94862356364621], [0.022284465490423431, 17.975512969031268, 590.75870103368845, 45.275830597299588]], [[80.722236126414529, 731.18495946548524, 376.27392721455925, 15.010418942197814], [28.066773694950484, 23.974351116420902, 57.776831349319217, 0.04560151010338874], [0.011463275935538534, 557.13657367566702, 0.014041235772960245, 167.34394341974229]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank0_Symbol_rank4(self):
      arg0=Data(4.05232998096,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[3.0001772438865317, 4.8987993242195573, 1.5328126079427633, 0.39573034273262936], [2.6573254703968421, 1.274563489856309, 3.8471577883042181, 1.6473640962747704], [4.9397547023867068, 1.0424552355969519, 1.3582970321595982, 3.3360161709947156]], [[4.3855585753199273, 0.51026018248051297, 0.097228309381004005, 4.2376129876625219], [3.5131160259331793, 0.17730472355191559, 4.0722138567187223, 1.7002548583850146], [3.0533775385363615, 3.3016267525114285, 2.1609922619397133, 4.8729444183001318]]], [[[1.3369833489067648, 3.2227576829424449, 4.1379102186312302, 2.3854589590915647], [3.7489093190931295, 1.89330341133535, 4.9966968423000786, 2.9540790785398872], [2.1185071603145387, 2.1186619631305588, 3.7538277062535608, 2.9369272032947369]], [[2.4323007021029022, 4.4025071478019822, 1.7809901776053021, 3.5546654058295073], [4.6827765114253195, 4.2511903354304534, 3.5705364134553488, 1.7265297735797795], [1.9362921689651806, 4.3694024901219821, 3.3029322700543404, 0.14978737865561781]]], [[[0.6238785315282227, 4.3858841068175831, 1.8238523322642652, 0.96805461691970696], [2.3466613669393634, 2.8920122405465545, 0.35409664046592004, 3.0162621015068991], [1.6208779332743151, 3.2690542646306548, 0.36480928725165818, 0.43002270396785947]], [[3.9685505873367282, 4.756633507583536, 1.7473855490751862, 4.3514530352056315], [3.688359088526739, 2.0982975399232626, 3.161281631518984, 0.68238866367103257], [4.8386433839032907, 3.7325272871351687, 4.3958625366927881, 0.41220534547728527]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[66.561349740793517, 948.47060720416448, 8.5407809511691593, 1.7397515410205977], [41.197366083620352, 5.9505670639136907, 217.73902039055525, 10.025616157039034], [1004.4137671312707, 4.3003623732557079, 6.6902697680249057, 106.49103523940529]], [[462.51413381837597, 2.0421496463891708, 1.1457400927339618, 376.02590449243223], [136.43867129840262, 1.2815894747469458, 298.33480183869943, 10.795755576255953], [71.705454785354789, 101.48793928132189, 20.570525014053676, 914.76960582681227]]], [[[6.4936847683132797, 90.883558202523318, 327.06036192415991, 28.16144051481232], [189.77115380157662, 14.143912991139167, 1087.7188846729171, 62.403369978044033], [19.38326690044472, 19.387466048891234, 191.08171283515247, 60.923488975890479]], [[30.069126924451396, 473.61422466715027, 12.08693772426764, 144.60630833138327], [701.044805947334, 383.23819323470462, 147.8536760211021, 11.200062222149777], [15.02083413454818, 452.1753445408483, 101.67350692304929, 1.2331801044086883]]], [[[2.3940542085640248, 462.724863300354, 12.834052801804932, 3.8751760732374496], [26.673338902356537, 57.212342260069285, 1.6412934223856563, 68.07646017064981], [9.6608492345087331, 96.966110799351441, 1.6660819368874582, 1.8252689351880602]], [[258.05201325940556, 777.3722388251465, 11.531733317195226, 440.95970790831802], [174.35464910594237, 18.84280234355947, 83.392314997617589, 2.5983094223420311], [871.90029167521152, 185.47046405799813, 469.23108563912558, 1.7803244987862838]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([2.7878196001216557, 2.5143644054709942]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([1.6556528086817253, 3.4922657586127062])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([5.4601389743331525, 25.026594759439774]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[0.35890178551627616, 0.77692583467868148], [3.5172276507260039, 3.3093141085091133]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([4.4831572581214996, 1.1297235022110259])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.010113132934405227, 0.32251828547441802], [4.1405260154899359, 3.8650954480101758]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_Symbol_rank2(self):
      arg0=Data(numpy.array([1.6102343379374504, 3.0895377009814329]),self.functionspace)
      arg1=Symbol(shape=(2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[1.711641461920993, 3.8674778765701268], [1.1789100541417574, 1.6881075024821472]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[2.2600598160917915, 6.3115925362404788], [3.7804227816162701, 6.7141786295677317]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_Symbol_rank1(self):
      arg0=Data(numpy.array([[[1.2896801093956465, 1.0068183372363844, 0.80993989564220126, 0.76534424543882196, 3.0624444608938122], [4.2139717949571853, 4.4050756330214211, 2.9603966222715434, 4.7500604387990046, 4.1903378637626876], [1.7960048056579558, 4.4643112223306041, 1.4880984578819545, 0.8578043011844253, 4.6534930502202512], [4.5091824311904771, 4.8903281789048121, 4.2885201059988578, 2.2600362610683091, 2.6587675527407773]], [[0.34914711668753018, 3.8825146922686327, 2.0544423596650407, 1.5523718833825779, 1.0779201637332612], [3.6581086491593404, 4.42794330273182, 3.0781489794948254, 4.0897041077750238, 4.4260245137193746], [1.0638135599985163, 3.4952706617437412, 2.8982333342125592, 4.9030701007045918, 4.0127204654082584], [0.50207694322356655, 2.4631551238483369, 3.2747292360186884, 4.8105950504009103, 2.9007932075266658]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([1.1427371823165398, 4.7575616728488459])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[1.337370968789146, 1.0077953516122209, 0.78593313892512728, 0.73667998574627247, 3.5929256445783979], [5.1743803234600252, 5.4433900979083161, 3.4564402723886904, 5.9332037619343412, 5.1412309824632141], [1.9525700774248143, 5.5271160635166643, 1.5749712276752594, 0.83922853422969879, 5.7955675548779126], [5.590644589918508, 6.1338361267330832, 5.2791163568862602, 2.5389891364584982, 3.0570183770719441]], [[0.006696294787005324, 634.95353143548368, 30.736952377675365, 8.1035281736364588, 1.4290016823034764], [478.33043344767663, 1186.7141271562878, 210.41081320405186, 813.1335501623189, 1184.269559117133], [1.3421911683213399, 385.16204105763194, 157.9893820513733, 1927.2789629899423, 742.84433451003838], [0.037704756667655236, 72.869570471453557, 282.47415883084733, 1760.3655649965967, 158.65437666134227]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_Symbol_rank3(self):
      arg0=Data(numpy.array([2.421157002882687, 4.1164254452975833]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[4.343269959123667, 1.8466672882823674, 0.83089224005614648, 2.3132372048615792, 0.87848717022282707], [1.9551740499636092, 0.77569484350660955, 4.1943243563373542, 1.3226698617897175, 4.5194239662687705], [0.82672760089153918, 3.3330997436356324, 0.14616778821819434, 2.5022332908798148, 2.976106022650356], [3.8843358298887081, 4.6587102193016188, 1.1650074752347277, 3.9905476811882585, 0.030811476531331095]], [[1.7260216400400168, 1.9773919012806114, 4.1405339521588092, 0.47522570982882861, 4.3838674407685074], [1.6182888818699268, 4.0724221996904761, 2.9593168241569785, 1.4431558042352122, 4.9684698385956843], [4.9040109934516058, 1.9374169599591229, 2.2534300323673166, 3.6962803034698495, 2.3986642905708568], [3.6435640014936053, 3.8161499023186609, 0.63297484727306741, 2.6557924523954832, 1.6698326518770892]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[46.549535761835848, 5.1187347439551116, 2.0848830156822622, 7.732789386848359, 2.1744991239118705], [5.6341928963880852, 1.9855673563308331, 40.805323378311414, 3.2205910803380777, 54.395445260891719], [2.0772194209808883, 19.054028269859494, 1.1379725484868486, 9.1393438419568938, 13.896102932556575], [31.022308668722495, 61.525004514348879, 2.8014934445974151, 34.077043093101594, 1.0276194465144688]], [[11.499402009128403, 16.411465858415212, 350.30148334958665, 1.9590062607623557, 494.28261266038129], [9.8735037368423928, 318.11630172888101, 65.850659180166957, 7.706345421730445, 1130.3822976882568], [1031.844091665833, 15.508934525491993, 24.253722041806085, 186.82677249779397, 29.787070226222387], [173.39791696700104, 221.36115267805701, 2.4489271383345477, 42.858626860570809, 10.62052499854563]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_Symbol_rank1(self):
      arg0=Data(numpy.array([[[[0.9665358375258053, 2.6291018325178954], [1.0592641147619906, 1.6011149998069567]], [[3.672103405676757, 3.9662907501367126], [0.34560638760364543, 0.11232753426398201]], [[2.7155015866012708, 2.1508566280238259], [0.16945162338406555, 2.078555195336897]], [[2.8618924469861038, 3.0863386774114261], [3.2825240560897098, 3.7280369756287204]], [[2.3440113576087009, 0.55918798593290187], [4.2030729552727459, 2.5131875260940846]], [[2.3411000463431013, 2.040639157877588], [4.9992874676593955, 1.7965530193044463]]], [[[4.3953974885339191, 2.7217730492210714], [4.2851103803049106, 1.6770985189734318]], [[3.949970994126565, 1.8685764045758964], [4.7152661318160298, 0.86864845486418385]], [[2.9569405697921232, 2.8109329345813228], [2.4046012691786776, 4.5610437443990275]], [[2.3061994645490449, 1.4325381802625128], [1.5689243574571663, 3.9804902929531933]], [[0.87106643604588574, 1.7498422856254856], [0.9188859266104672, 4.3444917000003223]], [[3.5343636821835611, 1.9668206992651012], [0.3388835153314752, 3.737052755125879]]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([4.0584231893792611, 0.43103043912981692])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.87098010005073401, 50.554069273782417], [1.2632167260262552, 6.7551208127950302]], [[196.18406982551332, 268.22405397652631], [0.013408170300414106, 0.0001401106924959124]], [[57.643075975285036, 22.38092701077937], [0.0007432609707311949, 19.480984017503907]], [[71.333310007749205, 96.910130975947808], [124.44864804421618, 208.59723198760082]], [[31.728758050906379, 0.094511107374872547], [339.38956271164625, 42.099990398138758]], [[31.569128095733365, 18.07850004442524], [686.22238592200506, 10.780163618760362]]], [[[1.8930050712227144, 1.5396939719433553], [1.8723837535625272, 1.2496591728294362]], [[1.8077989653174744, 1.309270689336534], [1.9511995055777906, 0.94110899561353478]], [[1.5956862064034949, 1.5612347930196762], [1.4596245611809529, 1.9234316041427855]], [[1.4335722839029221, 1.1675796753239718], [1.2142564478430342, 1.8138063702942206]], [[0.942237265972105, 1.2727406806102817], [0.96419445893545008, 1.8835238126007916]], [[1.7232124861075844, 1.3385098082283704], [0.62724532960881429, 1.7651333678658134]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank1_Symbol_rank4(self):
      arg0=Data(numpy.array([3.0239637367549048, 1.0747994386975934]),self.functionspace)
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[2.3102839251788727, 3.3540586561134949], [0.89823027200930461, 1.6450872448318261]], [[4.4611133292220249, 1.7701937946867181], [0.94138772067730836, 2.1968879055021042]], [[1.8932854175032032, 3.0880513285030089], [3.7247842117188243, 1.7837892288019375]], [[2.5010327486206112, 0.45460993505306646], [3.8716805865960877, 2.8755382356805654]], [[4.2551359206019157, 3.9411148193394649], [2.2472161957091905, 4.3071892160315031]], [[0.72133605180052152, 3.4171698963288391], [3.9035266020081609, 2.9983075968946897]]], [[[3.9525004900887835, 1.5577731684851706], [1.7895593942134065, 0.59497328528604387]], [[1.6710360257885388, 0.38009176472418821], [2.6149928209833773, 2.8126954712629475]], [[2.2256793124008918, 2.4229041437653867], [1.9067071123474806, 0.20310446148649786]], [[1.7704220728883147, 4.8292057716758636], [4.2274405317136639, 4.3338779731586214]], [[3.1454582179844501, 1.4534836706438257], [1.043477535273782, 3.3194138866224936]], [[3.7708165519026826, 0.39843417284153138], [4.931844099289143, 3.8644449118195925]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[12.890457901607462, 40.91495087901739], [2.7018947728257734, 6.1743379961122775]], [[139.2857161454487, 7.0911011066893961], [2.8340590190014252, 11.370324876820099]], [[8.1258467726981554, 30.482118190519145], [61.665755026696615, 7.198588112829527]], [[15.919805661287095, 1.6537691541893564], [72.550266251164928, 24.094411740525789]], [[110.89671028478362, 78.344313004378023], [12.021522433240124, 117.47197158352434]], [[2.2215557275348599, 43.874463408984766], [75.152504750649868, 27.600465523815277]]], [[[1.3299082685571735, 1.11892513922097], [1.1377904870709361, 1.0438521366759239]], [[1.1281043288126793, 1.0277968883250002], [1.2075941682514655, 1.2249391723498355]], [[1.1741533322971749, 1.1909769664152228], [1.1474459447104381, 1.0147586008813623]], [[1.1362209044172731, 1.4167284340679687], [1.3565470190780839, 1.3670023557107975]], [[1.2546977672106168, 1.110539241891185], [1.0781755286931447, 1.2705410736446936]], [[1.3125927319833013, 1.0291576794334432], [1.4272564209972374, 1.3214877181026294]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[3.281568481447636, 4.8244767400172872, 4.4340968530628748, 0.20749718332260808, 4.9605086146686803], [4.9991377507037704, 2.772894229024045, 3.9307136874427222, 2.2997366899245617, 2.2904035845298281], [2.6859180022786404, 2.4515188610928078, 1.9655777755669033, 1.6923455513894774, 4.0350162653141552], [2.8651352056927575, 3.6220058381972584, 2.6162171862593921, 3.9865162738866022, 0.40097689321176927]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[3.8796477561750811, 3.6148041984732364, 4.94930150076096, 0.98445713280647096, 4.8274062392452146], [0.24519433749060007, 3.2398490932529573, 2.1115365289390433, 4.8791027255396395, 0.49692169421322174], [3.0066375512630867, 4.2785123727394714, 1.8956215788536837, 2.0870878047516013, 4.6127079329943763], [2.7037849617187533, 1.4646440075503417, 1.6460871650803306, 0.64043505813788393, 1.6228309813653894]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[100.5111689635881, 295.48722774541727, 1589.4022510145383, 0.21263159371692933, 2278.1774929103744], [1.4837649983467549, 27.229336753152751, 17.998988513360409, 58.165393663608079, 1.5095520355153325], [19.504120674859831, 46.3664117954104, 3.6003640690767527, 2.9983111342652862, 623.14250249597865], [17.219562134541651, 6.5866034488553815, 4.8699832998854511, 2.4246059107291469, 0.22694978713372824]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_Symbol_rank2(self):
      arg0=Data(numpy.array([[[4.4207528647343972, 2.6338553083573104], [1.4358830268817462, 3.2293318660915347], [0.53291175569357974, 4.3973193292006254], [0.04805635049018947, 0.94503059336246031], [4.4287595728574294, 0.62326515225858325]], [[4.7719471841862333, 3.3016592494389863], [4.2438290049101584, 3.1614591869517015], [0.19230891897641117, 4.8303432716563082], [1.1107298331325688, 4.6485614474664292], [3.5335187519932916, 0.22782220280847815]], [[1.453421201516887, 3.342177953267909], [2.7575010968520011, 1.7222451171301765], [2.6340106048285739, 2.8583069225600863], [4.2199053657703676, 0.84663280667491192], [3.4212923069337466, 2.6595892153929412]], [[1.910295171654705, 2.2053927357846024], [3.821497110878227, 0.66828353867181467], [0.041866456431668142, 4.1470773646081227], [3.7291135786628904, 0.26517922396657923], [0.25832581401110832, 3.171388853365563]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[4.9309030636387785, 3.1146891782719588, 3.1615555455785547, 2.962176104766896, 1.4186362469776173], [0.54119844854601085, 1.9673785484856969, 2.5931320690473521, 4.3701384161190031, 3.0041807473195874], [2.2258749999764627, 2.0438007918965071, 0.86793732755379971, 1.2476910173355014, 4.0779046896752442], [4.0837950357936847, 2.1026934404520974, 0.4684693454498845, 0.90355899582981725, 0.082423907117400302]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[1523.6295000456594, 118.54901592408423], [3.0858658458618362, 38.523678428161752], [0.13671165782454792, 108.01264558191785], [0.0001244840414018799, 0.84579738331183518], [8.2573219634683177, 0.51134669829701984]], [[2.3297483941265642, 1.9086974295711927], [17.180562560882866, 9.6264930324703446], [0.013909641972678209, 59.380694693524752], [1.5823960186702997, 824.65926989612433], [44.352092856891275, 0.011751746418201502]], [[2.2985987184770225, 14.669804265495964], [7.949253082569288, 3.0376034021882123], [2.3177663080012429, 2.4881348029704933], [6.0281573023792552, 0.81242974815710212], [150.7914243928787, 53.994969427890389]], [[14.059074638520942, 25.277037665576305], [16.759406545174045, 0.42849538202311566], [0.22614477739920807, 1.947122407709311], [3.284576579759888, 0.30139353011753556], [0.89443456482897166, 1.0998031059912288]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_Symbol_rank3(self):
      arg0=Data(numpy.array([[1.1231558649733067, 3.440650859897735, 1.248862408058997, 3.1647599951825049, 1.3493001746912479], [4.144786185879485, 2.3047650309488725, 4.495952362561165, 2.233770946259737, 0.57110262063888917], [0.81013927712369627, 4.574157161211283, 0.97496374407913211, 4.2425889961006611, 1.854463472282962], [3.2956752568846865, 1.5764357272756497, 2.980803858088021, 3.1765733809912859, 1.046966403445456]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[4.5411944825652597, 4.5439548136184875], [2.4675428958348413, 4.2332058978748668], [0.88658522897795033, 1.3109952257338799], [3.9948857338152979, 2.4786976954864253], [1.2331125767886761, 0.74838087755143368]], [[2.2093182237666587, 0.6231086753804852], [3.9798099404169145, 4.448084455988238], [3.3108698200753088, 0.25970956386192157], [4.8885976780913039, 3.3273845118263159], [0.79430800409358981, 2.0110806489443624]], [[2.3050155318702954, 3.654593053867405], [3.1291144246941531, 3.0033706246537628], [2.8263879867772572, 0.11011401972590912], [3.8440672608264195, 4.7044686083129044], [3.1242984747461753, 4.1054262301185211]], [[2.6102249045532053, 3.6229113015940957], [3.1994302598750348, 2.3062003008558336], [0.26586410983077591, 3.4919337008844069], [0.21136585579472666, 4.8957342738670162], [1.1130601143398082, 1.801996005198834]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[1.6945640266433959, 1.6951073767918012], [21.095203148706752, 186.94341561316313], [1.2177788094931232, 1.3382281797669402], [99.725045862556172, 17.385754338910797], [1.4469000183152982, 1.251326871047131]], [[23.134425179014464, 2.4253305395841318], [27.74502817425871, 41.019842812991804], [145.01324387457137, 1.4775563042451685], [50.852168227724576, 14.500592830449479], [0.64084958065867759, 0.32413993522791518]], [[0.61550085546612321, 0.46325793971049828], [116.46325722390095, 96.196419486485638], [0.93084448999127167, 0.99721195350538072], [258.61614271052815, 896.7474587396124], [6.8864274050870984, 12.622645223071141]], [[22.488029478376326, 75.243096826877292], [4.289942155007922, 2.8568081142511632], [1.3369286403632863, 45.325310066087148], [1.2767238485020052, 286.7195808019801], [1.0524133393642559, 1.0862223336653765]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[3.225496358240282, 4.5157244474002241, 2.1512306455464203, 4.5621403785824404, 3.4804458603117689], [2.8725988658785062, 0.67024460241436723, 0.77073832493511163, 0.92921445423913973, 1.6829555782074763], [3.2099479193658285, 2.0533201979483007, 1.411644800563838, 0.99225198553221527, 4.7011346632446527], [1.3447253290770582, 1.6889774635568702, 1.698984125826829, 4.9953580438096932, 1.5401354314797253]], [[0.33689572094661091, 2.4900425413186049, 3.6406593951236683, 4.8721488900676233, 4.6017305533404187], [4.1833157572755466, 4.1014454948754357, 3.6593497389457328, 0.98273011006683308, 3.6153678141816297], [1.4189068640249831, 2.6398086286018985, 0.20324616271496057, 1.7466014813761428, 2.7696914631747056], [1.565851894199404, 1.4549488954076251, 0.89326391049783616, 0.74273771097099095, 2.3856753170369989]], [[3.0900645104385052, 4.2056897813053338, 1.0273565552211166, 2.1119270969259998, 4.3032788593932532], [4.4669613824435341, 3.6052484937705582, 0.88864267976176514, 3.3573388415699243, 2.6836866610365822], [1.4871250764339907, 1.3596302929446122, 3.7813105040110999, 1.9361516536508623, 1.9899163628825041], [1.7187741208557703, 4.2271812113737637, 0.36638248886663122, 1.4185864260271186, 0.36735242532698381]], [[2.3708656902865046, 2.9022872501912294, 4.7684897191726323, 4.4661615520067288, 4.7003288475364737], [2.6467615353067924, 1.4418721860464885, 3.6786584691019297, 0.48442123163497292, 1.773830236065755], [0.63418536752935617, 1.8161797333289664, 2.1999082380161252, 1.6198759391775239, 0.82041831616221761], [4.5623478587810267, 3.5164484365848798, 4.7922451772136787, 1.161200553234889, 3.2737919531386215]], [[0.7584633476293392, 3.0277711284635256, 1.2922772556036435, 1.7158045968706135, 0.081710601475805861], [3.7083141711415024, 3.4551856936927665, 4.8883241620676525, 4.7569926217293146, 2.4294245319665628], [2.2079547749320616, 1.4189326151585251, 2.1118043852778414, 0.064777808483089588, 1.5534955323429405], [3.1480647849992196, 3.505183182830701, 4.9266119624998641, 0.016459312525816942, 3.4376172452233282]]], [[[4.8407487350092637, 1.2883066518266593, 4.0101402419355567, 4.141706865767472, 3.8732544271516303], [2.0134785301830336, 2.7900779790262313, 4.5814132440858257, 0.56558562860828965, 4.0453919768188822], [4.6909218505458412, 4.7371391922045651, 0.15158762290105932, 4.218948369318241, 0.17815873514769506], [2.3438272723498979, 0.14114358182957448, 1.1762608372232382, 3.8711885519114859, 4.10093018694658]], [[0.42151835104471835, 1.9243716245568026, 1.7017008949102685, 3.0352137558472054, 2.9014430472758428], [2.5507248747313458, 3.6802313246278562, 1.378101272264094, 3.4613347087313082, 2.4941286100890765], [0.30646598032513789, 3.898942849417129, 0.28301110808782293, 2.1160575664254933, 1.7497825191958867], [0.057124281091512824, 4.4849983584557398, 0.4227961424551594, 4.5340369597498329, 4.1996386297178647]], [[0.35997884324464963, 4.3021465410208011, 1.4291776673171823, 4.9884116104774581, 3.813449662632602], [3.4442797425470362, 3.7066046400112977, 4.9780088154210977, 2.9830187245536894, 0.8949489467693027], [2.4295786792446483, 0.63779429397374954, 3.8675279842120673, 1.8057928261310032, 3.674923453563026], [2.7981451653550802, 2.6352215131467025, 3.0866921816005544, 2.4165110257510078, 2.9089905570740755]], [[3.7502421278332636, 4.186252416432354, 0.76627011437107351, 2.1017112325378973, 2.6353636800606703], [4.9833950417234707, 2.0119017345456891, 3.115701860474601, 2.2398447538368678, 2.0418434971290531], [0.038466628741340519, 3.7925304613233699, 2.2595143287991877, 3.3195589078370462, 3.8856619242989678], [1.072567158012347, 4.2588157209479585, 1.5754710014387974, 2.6244517784000583, 4.8585269298571374]], [[2.1585454297814439, 3.6099745591289572, 2.0615540581777396, 3.2332621983124041, 1.0966468757425709], [4.0264148234645223, 0.10863523766795379, 0.71929244536263648, 3.8637215749133023, 1.3066439549928837], [1.2025282246469349, 0.22826690119093179, 1.1865539024630765, 4.2726312757647342, 3.7543138661719495], [0.97066512043903408, 4.1212100435531962, 3.8512422238352477, 4.3636177478281182, 1.9504520399561411]]], [[[4.3774949912111083, 4.1621357467263067, 2.3730907062000126, 3.2362938212316998, 3.9515998485812913], [2.6568623380002094, 1.2108410475493161, 3.4032582611283799, 0.67829527511754295, 4.1796645446711667], [1.6260625244621323, 4.0013604039683202, 3.6315978081247331, 4.3076007255443773, 1.9520107164698217], [1.4136089932471572, 4.3263410759122669, 0.46145270938776695, 0.40168854588205366, 3.0631338997598521]], [[0.087208247389492372, 3.6396926841143031, 0.30891419479027515, 3.1479038433537831, 2.5616268365224912], [3.5851169726313707, 3.4668028760507918, 1.0511112862347043, 1.6470416350999619, 2.0907590193825576], [4.1254819456806819, 2.4376836443562939, 2.9827970072504151, 1.3234011121067175, 4.1257218300080085], [2.9844740592152172, 3.9361945941858281, 1.266746562679699, 0.79391882257868263, 1.7236175243335166]], [[3.6484239294281546, 4.8526875859903562, 2.0323675410173916, 3.1840261657270501, 4.1060410500159072], [3.1813680921823742, 1.9065570210403366, 2.7500797877524903, 3.6048447577730816, 3.2400573099377943], [0.32139025531872012, 2.7744313360293456, 0.17543834442983158, 4.5007551229479343, 0.15232252752759079], [0.86574685249502115, 4.9904688536064707, 4.2581881806977124, 1.0988436500336873, 4.5993623650581181]], [[1.3596611200394064, 2.882409219531243, 0.98563079806238341, 0.19279185566255025, 2.4259972034356827], [2.2345409146253967, 1.964822891114151, 0.087881617018680594, 3.1703139586682778, 4.3837922472775279], [3.0176564445872982, 1.1476312504820825, 1.6584425387402812, 0.83069577735933242, 2.6297341504546257], [0.2751311477658358, 4.3105488710817674, 0.89628491153430467, 4.2426176167869523, 2.9739812180496847]], [[1.4342911248248695, 3.2639192898870788, 1.9390904967163334, 3.6451122773524696, 3.475102733198288], [3.8187660392141871, 1.705623575141721, 0.70110406727325858, 2.9710431066215475, 1.5670993699855045], [4.4690826512178159, 1.5253527940490619, 1.8576473284158812, 2.7295647669675565, 1.8034322935640192], [2.9738085589089303, 0.43209428469777023, 1.9073337791895313, 4.1203182121236175, 2.0592186544322866]]], [[[2.2125720335393688, 4.3208221934045783, 3.112049860274146, 0.82687539763414419, 4.0074169878759447], [2.7576629349606305, 3.96247019387837, 0.049564024338460889, 1.5384245330632744, 0.92138622098957346], [3.7927402617846262, 3.1801141267665427, 2.7783715455096623, 3.1068821041148209, 3.1093456866072784], [3.5378361851430102, 4.5845606029715436, 4.0978959930088203, 4.2794821481504606, 1.8572293373508957]], [[1.5932542359172688, 4.2592738932357888, 4.8468101521035614, 1.8416386833415312, 2.8512540891839433], [2.8139174289547846, 1.7821981790912058, 0.9061085419926086, 4.1257111697178841, 0.97862208677157103], [1.9512557683256806, 3.890610806881718, 3.1031042937538453, 2.8803653845691244, 4.6314143405568631], [1.4990818200651983, 2.4946223440994073, 3.8024269640690864, 3.9070777600641908, 1.5036422054917951]], [[1.4528761104334789, 3.5646345822174719, 4.2069539264947942, 0.23711180733627019, 1.5628834752873793], [0.27904004691302942, 3.6574259653610692, 3.3547584937469281, 0.67919450166976592, 0.62374384148076856], [2.8018907294111091, 0.33828598631433904, 0.77813935754487951, 4.9866634855593555, 0.44315392036921486], [3.1050635917755338, 0.21021576433817968, 0.84366355539773008, 4.9637413927316558, 4.1418707730571098]], [[1.2529420740932, 4.5310926325318786, 3.9123995319974543, 4.435548713002861, 1.3283159264819075], [4.8384714339443917, 1.4357450445687583, 3.0332949288276621, 2.7723837859611464, 3.3656636377243121], [1.8303392471515418, 1.9277849143204111, 4.2405890765460859, 1.5757213251452955, 3.672490822162418], [3.8122389632673346, 3.4286856866006103, 1.2545208522291043, 0.10130576598202913, 2.3439580046020692]], [[3.1055368781700019, 1.3472132935110332, 0.22474117946010519, 0.051223927199863843, 1.710396853163116], [2.9432725515644771, 0.47800533147206381, 1.0870765163473486, 1.8004172051147442, 2.625479381790087], [1.3225118987282127, 2.2900407321108212, 2.5365808808682009, 3.1783210775376718, 4.2479474298164996], [2.0130682866084237, 4.5485889192352182, 3.1362630013678254, 3.7507347791626096, 3.98718995874719]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[1.4431664878494019, 0.51731691425383286, 4.5298466419556798, 4.1657794424863379, 3.1197966215173931], [3.623376259253245, 3.6838445413358145, 1.7693422157750147, 0.68412905645203281, 1.3985014514631], [2.882874486833217, 0.90980499888417077, 1.6079363535867508, 0.57971353448648422, 2.205199682477756], [2.3403967098783212, 3.0365880980800775, 3.9147080627278976, 3.1416913909035027, 2.5632846088912862]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[5.4198779313322003, 8.8080686354556601, 3.0208050230022807, 8.9390238833475504, 6.0488011986776709], [4.5852930825239699, 0.56133970007741274, 0.68673298394289395, 0.89946862149540097, 2.1196320440923495], [5.3822134817561311, 2.8244052386202041, 1.6446698350565041, 0.98883754990071393, 9.334700699862351], [1.5333442871906207, 2.1305862452550284, 2.1488273060216736, 10.189384204472146, 1.8649995718866703]], [[0.56959402827235839, 1.6031142038696242, 1.9512276063694016, 2.2686603124027567, 2.2026238739485251], [2.0966365041882513, 2.075308423063766, 1.9564032550774495, 0.99102843614112168, 1.9442035082418783], [1.1984179830893686, 1.6522910708979108, 0.43855923424559318, 1.3344151431728064, 1.6938589480638861], [1.2610950524550957, 1.2140704534216458, 0.94328084459407113, 0.85739511823835513, 1.5679952104380599]], [[165.75927695713406, 669.71034142566793, 1.1300436571844974, 29.562810966891288, 743.04393103214943], [879.94508955965159, 333.29673858933143, 0.58578997204275307, 241.3667689182582, 87.516378690627974], [6.0354232305641329, 4.0213873626654237, 413.64827650067201, 19.943039442687603, 22.577503607296869], [11.628002553857382, 685.3531470423253, 0.01058500588337279, 4.8739771107120111, 0.010712535818423406]], [[36.456894619998216, 84.659250721383771, 669.86147552882517, 509.89820283217767, 630.8673029569344], [57.668238299512012, 4.5925495169531674, 227.26761901080673, 0.048832521576863977, 10.887112615453868], [0.14999486254612962, 12.011534856533377, 26.692048306888822, 7.4585565556230886, 0.43841985919788173], [557.22868583599757, 188.34316871229248, 683.87306158660772, 1.8637567157140655, 139.82640372256586]], [[0.42210487331444285, 31.696115837725191, 2.2253951583176139, 5.3887983760625087, 0.00040413815101046744], [59.66436001876356, 47.854329120238276, 141.26645040344641, 129.75962325257987, 15.947486539656884], [11.835303012236453, 2.9791325111391851, 10.300385458054858, 0.00019583559597909946, 3.9522855864784985], [35.792738851334001, 50.047992503035921, 144.74715103259098, 2.7262596841372409e-06, 47.099293992129667]]], [[[303.17671253200336, 2.5040392699228482, 153.27595605752131, 172.29514236058256, 135.1514504940715], [12.627412126976207, 41.175195718504305, 248.34177219681436, 0.12682604628636621, 158.21463738802581], [270.53347911739377, 280.31682512014089, 0.0010745743181342783, 184.22564845962162, 0.0019292668749982518], [21.896749435735941, 0.00082966418363872795, 1.8007822744612818, 134.89043976908528, 166.22772114731163]], [[0.041484156338104809, 11.150036171346951, 7.0882318806305484, 59.746208989103671, 50.605560974341785], [31.483684181369025, 121.50550725754341, 3.2590426498086522, 96.937001633151198, 28.98593523283024], [0.012820711769853031, 150.29910653969299, 0.0095615708781469598, 15.81949530934782, 7.8544459013925749], [2.6322121948737956e-05, 251.76244550484085, 0.041949306434413795, 262.05285165300194, 197.61314793248576]], [[0.16402200765056679, 13.219224395353645, 1.8810511200522366, 17.176478473703384, 10.679489921051116], [8.9188919766710537, 10.155790316580498, 17.113152022947251, 6.9155840252488465, 0.82170253877737831], [4.8099018771299544, 0.45124582028570237, 10.948908546511644, 2.8453349093726539, 10.002709934697259], [6.1754183418144786, 5.5535349382263224, 7.3465121092793613, 4.764223060063892, 6.6148324608559941]], [[2.4701916696818418, 2.6632309662870863, 0.83349363942575116, 1.6621935514981387, 1.9404803652939944], [3.0005255977300478, 1.6132668494696893, 2.1759851530931491, 1.7361780199841923, 1.6296538503905327], [0.10765022406410125, 2.4892138209934704, 1.7465942101923586, 2.2724075766953433, 2.530871773865019], [1.0490937044968411, 2.6947270767275797, 1.3647534478235699, 1.9349799991077725, 2.9488844409165567]], [[2.9330867909133578, 6.0210238953288258, 2.7504375104055252, 5.1609966000748519, 1.1377150050143945], [7.0142206408413648, 0.044854255341685689, 0.63078612611936025, 6.6210748191421906, 1.4536039744040508], [1.2942356645825961, 0.12670143241439527, 1.2702556562835097, 7.6212894067303605, 6.3603621796712062], [0.95921634574690062, 7.2462448534767834, 6.5911867518241127, 7.8492206530816233, 2.5453915870865051]]], [[[70.562358644640611, 61.011399762448249, 12.077697026697738, 29.539572443396303, 52.531710371375446], [16.726409098207661, 1.7359203007553063, 34.149650100803079, 0.32658898730803237, 61.755092008595213], [4.0614598545600957, 54.461441000674533, 41.180489066956113, 67.362961628413231, 6.8773910058222585], [2.7125671749128495, 68.211295035425877, 0.10757735814916544, 0.072121208566536046, 25.208993422114222]], [[0.10867107689805902, 3.2393619982379374, 0.34344084119580404, 2.8385883229034694, 2.3532592560882759], [3.1951400692540237, 3.0990618088593482, 1.0463960578988381, 1.5745588892171896, 1.9562040839080346], [3.6304624603103135, 2.2494374513004418, 2.7028068668802261, 1.2903737828879693, 3.6306545200403324], [2.704189398725799, 3.4785930867627828, 1.240016984661886, 0.81061716058145994, 1.64102468392108]], [[8.013634661479287, 12.676983654969197, 3.1278651049776633, 6.4380508891410857, 9.6904869971525347], [6.4294110950342525, 2.8224342869140644, 5.0867368406302722, 7.8602825061978248, 6.6211927976474501], [0.16119069382873172, 5.1593567057592793, 0.060897622470704467, 11.231592003719312, 0.048521782545480702], [0.79310116222738924, 13.260712386449951, 10.274330011449564, 1.1636498726920637, 11.62988997903908]], [[1.1949550542093503, 1.8472537890086009, 0.99164465328989715, 0.38508467348777126, 1.6715759491242348], [1.5937829994747665, 1.4792567180725398, 0.2442099752455715, 1.9520717197497797, 2.3555325579767699], [1.8970163411029703, 1.0830992143852836, 1.3407978722984593, 0.89804776991118951, 1.7515743653446814], [0.47325325891418418, 2.332636789137815, 0.93849584038213518, 2.3112549872710275, 1.8810510122961679]], [[2.2152183926959923, 13.57994421761863, 4.3073601928862457, 17.325457859725965, 15.593431097847924], [19.198023934201569, 3.2460016587607221, 0.45700356543027987, 11.037193918620241, 2.6929408807637563], [27.155723599378213, 2.5372784539258824, 3.9184806968468062, 9.1553096030928689, 3.6707202757035708], [11.059861613316063, 0.15717297451180529, 4.1533341250849114, 22.701006447532812, 4.9178646703503626]]], [[[6.41500182055176, 30.723981838942805, 14.253598301906241, 0.6408804332858834, 25.759781440482769], [10.740911364008195, 25.088672289396278, 0.00088342994013138867, 2.7405170968282611, 0.82561866509182802], [22.645396800284662, 14.993922307518906, 10.93063546084683, 14.198265080649534, 14.224628264056003], [19.242608922099709, 35.29384739042461, 27.141608939202847, 30.04041525581896, 4.2584545281442967]], [[4.1139259651099529, 81.476599780658105, 120.62784196448699, 6.3872896606758802, 24.085555231449018], [23.140540358800898, 5.7816259288516347, 0.74126582757412729, 73.963319439116219, 0.936486801345637], [7.6131545150871016, 61.892923229567444, 31.144618156058591, 24.840084128690354, 105.07468675144396], [3.4190789302803442, 16.052403755923041, 57.730586204572681, 62.691823960225562, 3.4507611717380078]], [[4.3159612904301703, 144.86972448982326, 277.10962099235928, 0.0035737500753609385, 5.743359771294168], [0.0067599941496288944, 160.20196618601858, 114.23794372357496, 0.21994099697483274, 0.15758301117252146], [56.446991239728334, 0.0143642947465502, 0.37456005076808457, 539.16622024777382, 0.041339424451277515], [84.394151226380984, 0.0022306545323816426, 0.51401325661267461, 529.52891262763126, 260.70195617518107]], [[2.0308084332922927, 115.23572275429443, 72.656369908913419, 107.77274136350673, 2.4399160484799323], [141.62561168647551, 3.1152182813996077, 32.66081913173899, 24.62116269340634, 45.278654636217844], [6.6802581113685866, 7.8625981271917302, 93.579228521837621, 4.1727212206666842, 59.556919652148181], [66.971274875040635, 47.996106441055268, 2.0388586668575175, 0.00075163918960895041, 14.530085996171573]], [[18.259395901261957, 2.1467537402757255, 0.021786031628606685, 0.00049205276251394672, 3.9581629338434214], [15.912776181649335, 0.15076303715008252, 1.2386396898682568, 4.5143380386346097, 11.87274070798029], [2.0473009865302454, 8.3633554337255216, 10.86937340084555, 19.376521266643717, 40.756971394571217], [6.010018856344379, 48.56520111831459, 18.726061960645758, 29.622630258183545, 34.648168985493932]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank2_Symbol_rank4(self):
      arg0=Data(numpy.array([[0.26614650258051065, 0.81123103879959235, 3.5618785596277727, 1.4315651091782433, 0.40922021101369049], [2.5780175465191135, 4.0537342117277566, 0.64448627850113971, 4.355591705631384, 4.8512185338144169], [4.7079947711833032, 3.8629254545168874, 1.4158015510061115, 4.0045697096090604, 4.7549663613415722], [2.6550857606128417, 3.5394087437327126, 0.32307366654515424, 1.2553482333345822, 4.7852453488488953]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[[0.056369421651721451, 4.5658112674282831, 0.75256343488550759, 0.2384162110839351, 2.9962058181656559], [2.411118031325429, 4.4286855842131327, 0.287420709128904, 0.58299837598449045, 2.1945919140928738], [1.941909864437616, 0.34938299049933552, 2.9238020267842519, 4.9979048085668056, 1.1275680272544586], [0.50922416049676478, 3.0794351707783387, 1.112353094091429, 1.715576419828414, 1.695761684109564]], [[3.6446939026861527, 2.4373646140958525, 1.0114306893469984, 3.286519939794426, 1.2728686651583572], [3.1168041643732836, 3.1371014737587366, 3.1567389258962173, 0.14296725383724776, 4.204060430133076], [1.2909249091660984, 4.7967585569969406, 2.1321258484873802, 4.3021415778064549, 3.4318162277513893], [2.7288501935148601, 3.2544451176360876, 0.5386812125287983, 4.5711244842952468, 3.8219929396955443]], [[4.9337622562393912, 2.6393768626506913, 0.81742278768662824, 3.3879342341553271, 3.1915551295769506], [4.338780090787524, 2.6477226458240937, 1.6265130575984972, 3.1598755598519888, 1.898186874947323], [4.3048497835921253, 1.511145130476403, 4.3624208858782589, 1.9944054631432657, 4.4729064874114952], [3.1567926205280687, 2.1350611531533841, 3.9829043564194317, 4.2321910140948802, 3.6288685703469459]], [[4.9043191306280365, 3.3334269695155547, 0.52784434710275696, 3.7706631139210338, 0.87869391288299015], [2.437689045119857, 3.8680932288011172, 3.0937400324844653, 0.62461294639312925, 3.338819532076744], [1.4555841665081017, 3.763691401306148, 1.4327917238348262, 3.2824961440267946, 4.970652609459866], [0.44171058159123522, 1.1575016654697592, 4.4397088759516539, 2.6879308102296733, 4.9772485499428276]], [[2.1514686389449817, 0.14270976941200283, 3.5393972682360855, 4.6174459579919365, 2.719142470851668], [2.2675601019376641, 2.0294928181371881, 0.91965956689021766, 1.3621126116840467, 4.1885832316820668], [1.9697529205471569, 1.2767668755462314, 1.2949423638169331, 0.92518705981188898, 4.5835824917720487], [4.4876046598809696, 0.20420518620394654, 3.6923356508482268, 1.5820667998010329, 1.3828337425120703]]], [[[1.7886728335652868, 4.1949707712587552, 1.4521673846879559, 0.82590148257421681, 2.1474407897936363], [3.9592718028990137, 2.4980715088009617, 3.1217366660615551, 0.027325928176603495, 4.660548084731964], [1.4858934087587847, 4.244314507240424, 1.3463482043139654, 1.346035887264027, 3.1500531705265038], [3.9143878606018885, 1.2902248020100322, 0.63513970693437982, 3.6899373293585072, 1.9816844570597181]], [[1.6939598021158044, 1.0867468526349433, 4.9476144174828907, 2.9087316397771481, 2.5630487550672321], [3.686741164217636, 2.3176771123361828, 2.9139908591933033, 4.0453572719360862, 1.5032749465018691], [2.6025487758236294, 4.0049251830595729, 1.4486640050445418, 1.2217194888162561, 3.9454058648191186], [1.3168750641109823, 2.7597573161236868, 0.098257248299581032, 3.0130910697141564, 1.3242505244378355]], [[4.3846971723787247, 0.15145795610333951, 3.8014815559758972, 3.2870401821797497, 2.9030039300023591], [4.6286214527633076, 3.3999343920596248, 0.96826105439666021, 3.7355003837772998, 0.90566253009888897], [4.1656925869852692, 0.48545611102756231, 3.697882724639018, 3.1136531322427534, 2.1730555460805068], [2.3820978501372823, 4.4932927588023128, 4.3346460391466861, 3.5825481424360541, 1.9914912577838404]], [[1.6214694917712444, 2.820303471664849, 4.751649720690847, 0.47647685060127792, 0.14096799608708532], [2.9530514679305329, 1.8604421520810173, 1.8948563718885807, 1.8600522733122682, 4.7316452587470037], [3.5677024286053802, 4.6739294261199023, 3.8243818383195598, 2.8816768683782943, 1.8404931627098631], [3.8279203768512398, 1.7049931468712252, 3.4261367861221697, 1.5827291752448931, 4.7489803782422895]], [[4.3457550975676353, 4.8226995915509931, 3.3862077416494669, 1.5458066466593066, 1.5379082361434333], [0.60912436549684756, 3.3616187951613767, 3.0788515758860746, 0.92140790638782333, 0.5632185580811736], [3.8189177195471751, 2.4022930259536306, 1.3565531897478782, 0.35337476335771401, 4.510737711605632], [4.3697090417174032, 4.1159217604835243, 1.3012131042876627, 3.4668990483225119, 0.85317726555944406]]], [[[1.7298861256078493, 1.7338416769779779, 2.3921612833193588, 1.1850488216069064, 1.6076554747058067], [3.67250875673566, 3.5660176389605942, 4.950672552968868, 2.8197683882648001, 0.60239240868930177], [2.6670695330725089, 1.9379286945744842, 0.4301092937730393, 0.99943028553629998, 1.28060457644443], [2.1436446395213862, 3.9270197916935343, 2.5305782106049235, 4.640681715650274, 3.5000440621007267]], [[2.247925993157486, 3.4422673629314176, 4.6852203548785436, 4.6423415043132632, 0.80360945659815775], [2.0931817234433261, 4.2264938163302777, 1.0481721458461359, 3.8741188864935219, 0.72212566694915215], [1.279335166544463, 4.5715094014822757, 2.2335720684761911, 2.248569473151941, 3.004452376624982], [0.55355219699578617, 2.1531727276359445, 1.5661312032751846, 2.5245272484744157, 4.0626833446284722]], [[0.37479126017206149, 1.0432950276827728, 4.5439255171931441, 0.56093056541311404, 1.4604060303789426], [3.733447899170351, 3.1164326954471289, 0.058069342445048105, 1.4054860479723954, 0.97067941774299504], [1.0565773590149663, 1.1570862895170295, 3.6919354304138694, 1.1706157378179733, 0.6712417637254221], [4.6580300710244504, 0.22378407498252187, 4.8151842423094537, 1.7705798985970636, 2.7296235935473754]], [[0.32650936566310718, 2.5786866482172517, 4.0478785049610675, 3.1557343497493511, 4.4129606889132775], [1.9554301150514111, 1.6881647699403803, 0.018384251402512847, 0.7041442089048684, 2.9940747393619773], [1.7520598149428839, 4.4938961389194887, 1.7799146317290826, 2.337582847774835, 1.3822898142877051], [1.3846465226319447, 0.28549776588272419, 1.2493826702613979, 3.2867181373964525, 1.6331049702387268]], [[1.3879394491315122, 1.0475880248307157, 1.3118989583068843, 4.3708097849974967, 0.79848251709280305], [2.9224409621691882, 4.2451138776617867, 1.6920510909889375, 2.6277941491342256, 4.1862300200621885], [2.6237941739085717, 2.3755306964577798, 3.7435485320158999, 0.15550559399007882, 4.2153140782085439], [0.92152898475676104, 3.8206758800267164, 2.1062043157603076, 2.9709587231238812, 2.2810225491089624]]], [[[0.74760658852492612, 2.0759303491610268, 4.8354713746940607, 2.4780652056457519, 4.7774944384392182], [0.32808252227912321, 4.1943698516784087, 4.4870090143955892, 0.75381435848365996, 3.2393713286390833], [2.2695283117280245, 0.29333219390495896, 0.47072106646564604, 2.1552909410401297, 0.48309683635284051], [3.7143099710433889, 1.009628521198854, 0.11030054594516608, 1.7150887029481436, 1.777619383692804]], [[2.5492479688086993, 4.2038608964901041, 3.0490271205926653, 0.33194626748701966, 1.4577598961288887], [1.2444810525812351, 4.2836905563274987, 3.5328901438141114, 2.262274020302562, 2.9947478144229049], [4.1948558678446517, 2.0908538919063138, 4.7917272432490767, 0.48895639157219384, 4.2159704336022923], [3.0692127456781533, 2.9939744611324803, 4.2383765337169539, 2.6900763007226542, 2.4094527030225659]], [[0.40768575491923126, 4.2367514128728239, 3.8909149884210823, 1.731202851036387, 2.088783633015693], [4.5085678621754504, 4.8619462417810375, 1.4641815584686984, 2.9247640151976801, 4.6514270119310721], [3.5685623492921237, 0.99511440643814375, 0.71961190118728302, 2.7144125866936277, 1.5242142219748669], [2.4531212896457482, 0.2444226920214167, 4.5928756515702887, 1.7865541879856854, 2.7100599765547573]], [[0.70353505361957369, 4.7609903765785724, 3.5967956412047748, 0.53475492875989816, 4.3930499778128258], [1.6201326477698594, 2.4277595815653248, 3.4474720641350749, 3.5551927382525501, 3.3941290321208535], [4.5199281354258805, 3.4155262288197581, 0.10575955037820928, 0.83270424894852979, 4.57551805103079], [3.1339261374788774, 2.362720651429989, 4.4729961726680605, 3.01148688353052, 0.83964010750900908]], [[2.7825974881720774, 2.2716791082982444, 1.1534674742137254, 2.4206940378952768, 2.2641690073614571], [4.7631515108632057, 1.6294921592224114, 2.2079942040819871, 1.3163886576260639, 2.93817633825872], [4.7699854772533543, 0.48741414759315699, 3.7086603163101488, 3.6070466283735199, 1.9777736917290394], [2.5402866099292303, 3.0567068534393536, 1.9385514241928223, 1.5936792036084801, 4.9327005952273]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.92809918204625086, 0.0023725201225367294, 0.36928945543899533, 0.72935584986485857, 0.018947132266571378], [0.041105345891968896, 0.0028447271170953686, 0.6835460880723776, 0.46221745154501132, 0.054748808997929048], [0.076495579634465558, 0.62971925528633232, 0.020852922211677578, 0.0013390854758169701, 0.22479391621974687], [0.5096330721980179, 0.016970579015773021, 0.22936719450003862, 0.10321688246410307, 0.10595996782626149]], [[0.46650867935709767, 0.60055413714080375, 0.80929343738090886, 0.50280753984898319, 0.76621906894228053], [0.52098039795343087, 0.51877287743002853, 0.51664602551884498, 0.97053376042756745, 0.41499057713648918], [0.76333020555513031, 0.36659656046713479, 0.64015444552199385, 0.40656222989889046, 0.48775398151320903], [0.56502688425342718, 0.50619278846208182, 0.89342454753524148, 0.38431600315183923, 0.44952220145949157]], [[527.05211721953799, 28.581766976546156, 2.824587771874548, 73.969475188903587, 57.638714738169874], [247.52229993315288, 28.886389863052308, 7.8942935805025813, 55.365258416471164, 11.147822436876041], [237.08042120069189, 6.8181601516856505, 255.06828641066909, 12.597136275557171, 293.50064679801926], [55.14885983171412, 15.06151411790605, 157.50165583600554, 216.17793548028891, 100.45486565133932]], [[5.8096154033252727, 3.3066249591306702, 1.2084926433241934, 3.8682201810765813, 1.3705985547316237], [2.3978335020398953, 4.0058241479271857, 3.0341640130222247, 1.2511853002781159, 3.3130284147864861], [1.6857616799629962, 3.8585569639666044, 1.6720330658802667, 3.2467537541726004, 5.9495329715661844], [1.1717188422539699, 1.5147871542352844, 4.9176310907182659, 2.6230675192251831, 5.9636287044725105]], [[0.1462641246103851, 0.88028334213223969, 0.042321585680269745, 0.016152240198373417, 0.088075679425496212], [0.13185279652796414, 0.16310589983520682, 0.43967584978134194, 0.29610233895264826, 0.023694628890104492], [0.17204866818936532, 0.31956532333293181, 0.31441754865109267, 0.43750972109328662, 0.016648428405196349], [0.018139142868986317, 0.8332198778929365, 0.036916054109942212, 0.24327090536544677, 0.29067062854129611]]], [[[5.4407135423296005, 53.12906815552229, 3.9559967363387747, 2.1861583142776224, 7.6420882962657313], [42.500353639263473, 10.651763398362053, 19.227648635064543, 1.0262159682045449, 82.569103397161655], [4.0843877196853322, 55.670686920969686, 3.578771672729983, 3.5777133333955167, 19.75023899804911], [40.731688384075035, 3.3935263739110804, 1.8248366144249795, 32.932073860716088, 6.5318892697662703]], [[10.707359718879697, 4.5770409896567559, 1017.2635857918722, 58.625675105991071, 36.13788184383759], [174.18271136791179, 25.633756077513734, 59.058811248840932, 287.73436734735691, 8.1992399996515211], [38.192051684163566, 271.90355095802619, 7.5958792150643095, 5.528771356919747, 250.17029926241324], [6.3163974267410117, 47.591898437995944, 1.1474299622827475, 67.845849768496933, 6.3819391700709307]], [[0.14570006381631284, 0.93562947145496134, 0.1882475383966048, 0.23598135190998515, 0.27934863262732595], [0.13089477925979781, 0.22456341982687361, 0.65353527243375553, 0.19378386393527786, 0.67175665025322295], [0.16041420448609917, 0.80794465616341593, 0.19701284663144011, 0.25465814006384363, 0.38495566865793029], [0.35117881280987212, 0.13891245580605821, 0.14893913036921141, 0.20725203673284201, 0.41691805443935614]], [[10.869147368931111, 63.431784566174265, 1087.7479396123215, 2.0160030238689788, 1.2305101428242204], [77.115052774592144, 15.449352328789711, 16.251841834642736, 15.440491713246519, 1056.1958937078164], [190.51541111174646, 970.20014601430125, 277.94595662642149, 69.426844202385851, 15.002441173166648], [279.39694531224882, 12.29054190606338, 154.69026739136834, 10.266882407451936, 1083.4838284030709]], [[956.18027180934894, 2030.731755939262, 210.1035642839974, 11.486634051581392, 11.344246474494593], [2.616787502757195, 202.10128294835724, 129.31024540563382, 4.2849759180204963, 2.4337945849962939], [416.1097780937755, 44.42369583862974, 8.5190887585586541, 1.7472794850946707, 1240.775596437489], [993.04427420031811, 665.132841363892, 7.8061716913259334, 238.65779313229251, 3.8472671407757599]]], [[[14.585778446743923, 14.675437248507615, 40.694166891460789, 6.2710963813079781, 12.069476506304964], [295.79852115100755, 250.81005515498208, 2142.8441137063573, 78.929909320771216, 2.54279460734567], [62.30176082525638, 20.132983451231478, 1.947121286653235, 4.7038411540726992, 7.271714589157642], [27.689889605562353, 438.77328555612621, 50.427106559509632, 1325.6155568064103, 226.44125228030811]], [[20.861448883938429, 104.79074270040981, 562.12425309102173, 530.47634916853053, 2.9624532865005326], [16.924760416005324, 302.4114484097588, 4.1227722697886753, 187.8380748812767, 2.6535504216930628], [5.6345948942771766, 482.05157745050263, 20.460673033910894, 20.879598199240849, 57.991207876066241], [2.1129480648702725, 18.354063480878047, 8.3021002630948111, 30.317002598894494, 242.35678785790313]], [[1.1391853039030935, 1.4372755448438677, 4.8544897085915846, 1.2153513633401321, 1.6615935276849683], [3.6623473540203584, 2.9552128453974071, 1.0203956749311374, 1.6301656722058202, 1.4014412809286343], [1.4439285376756732, 1.4952809070802076, 3.6098657481881622, 1.5023314775425483, 1.2628713372589884], [5.0509566869237297, 1.0809159585359114, 5.3346299495143477, 1.8508097783239643, 2.58332870320623]], [[1.5730413212609136, 35.793382870900828, 274.83554861017842, 79.70879044455333, 456.09500336408689], [15.074949669314689, 10.404316895145779, 1.0258350611756615, 2.656355633233952, 63.693816688902565], [11.368781118435358, 510.29750388059909, 11.81674838563251, 25.616762309965349, 6.8062332000748418], [6.8285245250869551, 1.48603264921333, 5.6600836144287987, 95.594139334401049, 9.6391106710174128]], [[8.7064208700503194, 5.1211978974131247, 7.7330182092883497, 911.34474887870806, 3.4728849758502487], [95.262821887731903, 749.14870401295059, 13.988401196691642, 60.173338070819163, 683.43147895881793], [59.799221660600502, 40.605470856922317, 342.71596682675982, 1.2743837242773748, 715.13687826946477], [4.2073728594756643, 386.51016956454629, 26.681539188176174, 102.74888557020921, 35.041976982065066]]], [[[2.0751233664786608, 7.5920234314581609, 112.3620635331304, 11.2433131817274, 106.17760652454135], [1.3776305388225938, 60.081712092930687, 79.954754239470489, 2.087740435830248, 23.645467639936264], [9.1718783045453804, 1.3316677814975038, 1.5835169229227912, 8.2037546760960964, 1.6027692718947029], [37.597520088580055, 2.6801666750980258, 1.1137201981372316, 5.3374317705975312, 5.6734895208885803]], [[25.081905170079768, 203.0618723438254, 47.174211614546749, 1.5213022354163639, 6.3126133369785427], [4.820963900792572, 224.62047376264394, 86.95847769167851, 17.451457563126805, 44.046263472313335], [200.76372703500223, 14.051863781961591, 426.90012597469047, 1.8552531762919475, 206.19383835878313], [48.393291349808848, 44.003229841188862, 212.11680642832954, 29.96853540944316, 21.019510872660195]], [[0.63088359754314105, 0.0083374394541827403, 0.012323500496777304, 0.14141694874073152, 0.094414159397040981], [0.0061327150427742515, 0.0041138700236968209, 0.19121783384350374, 0.036713252550444726, 0.0052185761337247899], [0.017738301154465355, 0.32486199994901144, 0.44349381043750336, 0.046563109992179268, 0.17867774375573403], [0.062554306146088173, 0.75868616827272306, 0.0055754905611232752, 0.13284360354694905, 0.046792666950496005]], [[1.1735026855303501, 2.9526738692795114, 2.2658684933111428, 1.1293138415436708, 2.7156646289118016], [1.4454768157236304, 1.7369038229168758, 2.190215633885833, 2.2445321090037229, 2.1638068530965202], [2.7951631196112197, 2.1743615981188245, 1.0243426581872814, 1.2084853279849326, 2.8307235099469068], [2.0394814384831861, 1.7114028548721072, 2.7654891501791252, 1.9834768692743279, 1.2103929813841054]], [[77.965011342530431, 35.03678023261682, 6.0848151810607121, 44.242541508877927, 34.6272529790646], [1731.7562519302819, 12.82031154056477, 31.712067308135527, 7.8526852424001499, 99.466872212184398], [1750.3834835264358, 2.1448401505422452, 332.30261722331261, 283.43024200591861, 22.115494727260216], [53.352105132801405, 119.74788671027885, 20.798368767419536, 12.121299987235357, 2258.2077857731547]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[1.717199239046866, 0.70126422929029963], [0.50329506258277368, 3.9826978624360496]], [[1.7367160639025869, 2.6729603393800474], [0.20872108221056671, 4.5445425693241628]], [[4.292058151184639, 3.7383508539606147], [2.3203863544512378, 4.2002753525669903]], [[3.411654107021187, 4.9745876774987785], [0.5615093454997887, 1.2213051169823661]], [[2.2163315547569771, 4.0716941949721424], [0.86187309291718417, 4.0279984449771256]], [[4.0522013050863999, 3.1127533632058468], [0.93881428582733595, 4.6471216896030638]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[2.5426312924526946, 3.8144722591480003], [4.9287745147844957, 4.2948658440516025]], [[0.074948883412065168, 4.0932814928225412], [1.3132738767681427, 2.64674373804448]], [[3.8869677361246895, 4.2567606497225521], [2.0969559850306263, 1.2052551779543008]], [[2.3681719064444771, 1.7373388883978884], [0.16952376200086711, 3.2154486425883655]], [[1.9115891437046608, 3.0508845394669741], [2.4157189807461044, 0.3926914886210674]], [[0.1570061249032941, 4.7650982520117005], [4.9731245705706026, 4.0983534420773333]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[3.954234562424213, 0.25829736088096511], [0.033911825948535687, 378.16492735633375]], [[1.0422392094837252, 55.950060268510413], [0.1277627409741138, 54.980414009590284]], [[287.83974737244188, 274.00717706017184], [5.8420311042850166, 5.6390864137363321]], [[18.287432017020269, 16.236902685374194], [0.90679695651048353, 1.9018605985028194]], [[4.5783774831999082, 72.502600884687538], [0.69831152373135752, 1.7282772599614011]], [[1.2456935362286041, 223.81181055461741], [0.73052550060722743, 542.44539616449083]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[0.49936532579711013, 3.6528852341723925], [3.1578566999102313, 2.9230973857371034]], [[4.3719381401095809, 2.4436798382868901], [4.1710266190675247, 3.8147573269173591]]], [[[1.3903308765528453, 0.61700613662372539], [1.1531759162293731, 1.9242475253741509]], [[4.7449012173838252, 0.48927788931868543], [2.6854246903216312, 4.4160601455437165]]], [[[4.3429204879551637, 2.7173280177417554], [1.6874749241121163, 2.8346508605912271]], [[0.066361885111118471, 2.3353977480704837], [0.015828566512852631, 0.90783974358813491]]], [[[3.2138580956233627, 2.4467566145529624], [4.8911644667547121, 1.0640453714949798]], [[3.9781969867144449, 1.9882339349842095], [3.4490836537571745, 4.7268149680759013]]], [[[3.9041834788595935, 1.6159738537374004], [0.34508244400122784, 0.24415303832512766]], [[2.1606975950908343, 3.8311930421561069], [1.8985663514186204, 2.3620963333436737]]], [[[2.6172376995287063, 4.1559138158644178], [0.29675984053324528, 4.9557662712763522]], [[3.5084428365151434, 1.2554517271746375], [3.5004749124578676, 1.0479299806671052]]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[2.900731588810328, 4.1819717460947023], [1.0281860677374688, 4.8754770367951217]], [[1.8607979372427557, 3.0863641074205925], [2.0606895069125812, 4.1611055945104951]], [[3.4654244681584965, 2.0623544135833751], [2.6174563380152844, 4.6640086216272234]], [[0.21544654342253763, 2.087305079538917], [1.664351920411405, 3.8601322340666719]], [[0.18625458046655582, 0.31824247804702654], [2.0995306693960947, 3.1032154619103967]], [[2.2669787814350921, 3.2316104535984214], [1.6600379113799244, 1.5637130811828388]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.13341131883826479, 42.860388066476197], [122.58715994580614, 88.744910141937098]], [[4.5575567943230855, 2.5060039359083537], [1056.7696108158839, 683.79921171448734]]], [[[1.8463498321157736, 0.40716562302298753], [1.552502278279815, 7.5393299866588839]], [[24.74538668187817, 0.22922947636876417], [60.977315547139163, 483.12405617272839]]], [[[162.25008239305589, 31.951153426337722], [2.9420083851272052, 8.574612367140265]], [[0.00082494248791121756, 9.2080667759625818], [4.0011057594695375e-09, 0.63702211096217209]]], [[[1.2859886906720079, 1.2126083052640044], [27.479809988086839, 1.1383453860552339]], [[9.9560946936388746, 3.1387369993861407], [119.01658705542908, 401.72050973697691]]], [[[1.2887693911232183, 1.0935077078839144], [0.71276674272180485, 0.63845174727766107]], [[5.0406940785940897, 16.777517390058115], [7.3116464644205976, 14.402003136700325]]], [[[8.85608129319494, 25.264104115229234], [0.019725085743417619, 176.33047522957494]], [[8.0336069212191834, 1.4588538181214499], [7.0934654858514907, 1.0759543097082795]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank3_Symbol_rank4(self):
      arg0=Data(numpy.array([[[0.16615253565179416, 3.0072670627985696], [2.3789994852602221, 1.1260412698910285]], [[2.3129421276920321, 0.54260787649307185], [3.6758400563525573, 4.0832052081106207]], [[0.095413437354560898, 2.3598181930140685], [3.4290080758242993, 2.3012053281723244]], [[1.7243031767273453, 0.21415178527041082], [2.8220403069847486, 2.8186001952578392]], [[4.8684835585844697, 2.3611327294180984], [3.8172362668751272, 2.3353333050208263]], [[2.0516072663686007, 4.0966131066587579], [0.21734498967734711, 2.1446092598031927]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[4.4200133694907242, 0.3821141391468269], [0.62852235011304369, 4.9195169847592908]], [[1.4136192530190239, 4.3554498715578456], [3.6889151995021785, 2.2045179617475394]]], [[[0.97650281980439657, 3.4941339407592906], [2.2476929962526726, 2.9295130961742739]], [[2.3384023902827642, 3.9732519095074519], [3.635859191351138, 4.0372403245238706]]], [[[4.5269542793475219, 0.052286622806772015], [1.13912345006645, 1.1275698027370009]], [[4.2792423558646133, 3.0162124599982141], [0.38116674276366042, 4.705224363583266]]], [[[0.19917966120448333, 2.3089579129617595], [0.81625740538505231, 3.1716425926856089]], [[1.7540520268006243, 4.1053091972129963], [3.0318131198675355, 0.36671950994651464]]], [[[4.2722490232437229, 0.45040834601888763], [0.048908027435105132, 4.9103147668907905]], [[0.59233698475915963, 2.4339810178753987], [3.9250626471032497, 1.4339836340831205]]], [[[4.7153554676139429, 4.3465687182064707], [3.3286607516677376, 3.1984170291643501]], [[2.8091783827426497, 4.2781133060945313], [4.8711639715960127, 1.055569618123718]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.00035861661022517369, 0.50366838720790774], [1.9977529826654314, 225.09988040572046]], [[3.4046956602047778, 43.587977190087926], [1.5494567916554658, 1.2991293597208236]]], [[[2.2678165547116307, 18.725803865908048], [0.25304994798480412, 0.16679135399038586]], [[20.990908664576935, 176.32107074578522], [166.5383866578992, 292.92653190067836]]], [[[2.4029185465055463e-05, 0.88439693541575715], [2.6592226095887326, 2.6329741022038808]], [[195.03643351079293, 41.1321983919944], [1.3739323512629027, 50.475569743741815]]], [[[1.1146245962190726, 3.518284130517995], [0.28424748497501057, 0.0075385606796811375]], [[6.1703715510501125, 70.74590393485208], [23.142879922375617, 1.4622985737067371]]], [[[864.40465350892953, 2.0398952324250894], [1.0429142050570612, 67.942045283326578]], [[2.2110139007984304, 26.059502703290583], [27.912092938228529, 3.3744753148235258]]], [[[29.623422121778788, 22.726837728873939], [109.28357530565839, 90.947421284438718]], [[0.013738365880983868, 0.0014596577096225502], [41.119829095648406, 2.2374898220681616]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[0.71864340965774398, 3.269595777037245, 4.8462567346422833, 1.8220782253456289], [1.1936677454962479, 4.1378728214829561, 2.468671340927572, 3.4848933487063247], [1.9941637489311261, 4.1148698927172216, 4.9797190122874726, 4.9844305870098315]], [[1.6629597562763527, 1.3759236029767814, 4.2757045134769491, 1.8714755534619463], [4.0638953614665105, 0.15848685094697812, 1.6408588208290884, 3.8703688036764397], [0.97315692351707039, 0.42008950593593808, 3.9983772545564831, 4.209180147419346]]], [[[0.42876451274929694, 0.71520706844169102, 4.4520028348391749, 3.3868657262437165], [1.6984660593063317, 3.4209942033657241, 4.0886649427443453, 0.41159521498088625], [4.0001330329989111, 4.6806080323641561, 0.37275138256901719, 2.3114218518963385]], [[1.2420550626022127, 1.8713331078021223, 4.2798986397372492, 3.4505959557744803], [2.9555801017120538, 1.2741380601412096, 1.9020895535357079, 1.4212340316955865], [2.3680826625487228, 2.2291061310292015, 2.033704984724511, 1.9386578980576012]]], [[[1.6174363453766694, 1.3278301427841366, 2.6451941885151418, 2.3973955536434781], [1.6368705154062069, 3.1248058969206607, 1.0266095212087307, 4.5519211892023002], [1.0693549245820502, 0.24065005206204515, 0.38555801849214866, 0.78021655867795203]], [[0.061507040918411297, 0.41965986717223192, 2.2850438455912023, 1.6820411775919777], [2.8082413856658119, 0.41507895986286703, 0.91827525042282931, 3.3871272646752941], [0.51659549173762886, 1.8165210307433481, 3.9614701038341456, 4.4509447340747847]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[0.34447886147253659, 3.2490438401046835, 2.2964236762367443, 3.0807222049062677], [1.3351112059747896, 1.9242631642949506, 4.026638512323137, 2.5699863574293698], [1.1205121446684363, 1.7827991241121091, 0.68129513386222995, 4.341602413671743]], [[4.8378942340371855, 4.2421655768418329, 4.1555041442728715, 4.8126250161519266], [3.2758037149292711, 1.4428490874355402, 2.9847224972226316, 2.4825593471398029], [1.645691242681147, 4.6157696915088282, 4.6522413626253902, 0.83369504384820436]]], [[[2.6605053478439458, 3.3149764015870549, 0.37425164859110366, 1.9367010766825563], [4.5529331072930272, 3.8824618179478825, 4.0153041747899341, 4.1644467128602081], [3.7505573891244195, 1.076351726573674, 4.1218965881305527, 0.64624020394940018]], [[2.1609010044126058, 1.900386081432283, 2.1167193616924971, 4.775292729723108], [0.33351782514183709, 0.48599247118057809, 3.5532620384009221, 1.2199313775939826], [3.0681062357056237, 0.16364821460529594, 4.923930975267119, 2.4000703942801165]]], [[[4.7131387599185857, 0.66465749142325836, 4.2198374019285225, 3.8403057758143428], [2.7652159032403163, 2.7273498254930901, 3.930451095090862, 4.3929084373711342], [4.2385953544605846, 3.130041164700943, 1.2387175730531614, 3.2682627664360426]], [[4.4602255421653272, 0.19031816742988023, 3.5961219498895178, 1.3690007219282496], [3.9540976690670075, 4.7920601969149637, 1.6101971855356725, 0.35947931766613728], [3.9619531957939191, 0.21167169165173608, 1.7025415516509297, 3.2599765479769451]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.89242538579614483, 46.947640321862714, 37.495877794652117, 6.349427938892406], [1.2666247426041928, 15.375934772835326, 38.045889218669586, 24.741133711024734], [2.1671333527600702, 12.452986894325473, 2.9853981769448805, 1068.487304003391]], [[11.711229545223995, 3.8720419739655991, 418.9431047558258, 20.413632396439258], [98.804371415934611, 0.070098862554202279, 4.3845789755339295, 28.782604592493321], [0.95620864059871369, 0.01825713587002048, 631.11508673812295, 3.3142889836904335]]], [[[0.10507907448708084, 0.32918864436866713, 1.7487307213504288, 10.618430203879704], [11.154074795046752, 118.52939247847527, 285.5524170519418, 0.024801741677464559], [181.18186294919542, 5.2660018760394411, 0.017117270835453838, 1.7185152007142723]], [[1.5974565555367259, 3.2899717199223928, 21.705384361201919, 370.34102815281813], [1.4353828107027446, 1.1249531119255631, 9.8215667255892694, 1.5354707704020572], [14.082810496897867, 1.1401735845714116, 32.960106144188835, 4.8980483469239635]]], [[[9.6434306265411713, 1.2073900723562452, 60.63204438071979, 28.72873782561404], [3.9065718612552041, 22.364312229317221, 1.1087354301193426, 778.73411579740241], [1.3287272413301714, 0.011580107462756476, 0.30710160543196013, 0.44435577732069609]], [[3.9658073206813911e-06, 0.84767725766491386, 19.52668520490386, 2.037841961056754], [59.313500505519755, 0.0147931077919415, 0.87172406304557326, 1.550470370879093], [0.073032497043656344, 1.134681202920321, 10.420182698728908, 129.99868835239116]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.140262283959)+(1.-msk_arg0)*(4.06864528811)
      arg1=4.46275133456
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.000155958622914)+(1.-msk_ref)*(524.593659717)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.6329306374090051, 4.390954056092423])+(1.-msk_arg0)*numpy.array([2.8910285998649545, 1.8497241438393446])
      arg1=3.82334138938
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([138.6928850548791, 286.23452127637711])+(1.-msk_ref)*numpy.array([57.910948447545643, 10.50125285579084])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.7251912258283142, 0.11503252017618351, 2.1674596792312628, 1.1168719961221016, 3.6189149626408592], [1.0646676157417034, 2.7709883279276926, 1.3834020489505785, 1.4059337042359208, 1.7682236508965676], [0.16888483498843071, 4.8108319261364203, 2.8359719498240543, 1.8644393852123309, 1.5082888433287411], [0.14389400641242409, 2.0482117192304323, 1.6890335188554204, 2.9607241107073086, 2.4631715914210841]])+(1.-msk_arg0)*numpy.array([[0.80030073730059548, 4.0337864787613782, 2.5670263127849324, 2.6489792903149438, 2.7733893942734622], [3.8796579839958323, 3.6921317268226095, 3.7236205012320438, 0.87240893972644706, 3.462037564234659], [4.7625769041163997, 2.4055316616514948, 3.2726484870987114, 2.4980834954595883, 3.2103693037963441], [3.1930041035964591, 0.3007384190484601, 1.1941774306278088, 0.56887479837135502, 0.27471657433460384]])
      arg1=2.74194194917
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[4.4606070390670371, 0.0026596512941015333, 8.339849513494169, 1.3540120865659386, 34.008657390126423], [1.1874609304155961, 16.356058014121196, 2.4348508920871126, 2.5451360835686754, 4.7723536538303533], [0.0076225171800397493, 74.234902458409508, 17.429399120874969, 5.5185838656855681, 3.0859876868775227], [0.0049136245921858378, 7.1412091719250155, 4.2089223821890052, 19.61307944567379, 11.84283423550135]])+(1.-msk_ref)*numpy.array([[0.54290761999069481, 45.796219240506311, 13.262767934190091, 14.456290262829173, 16.39494766729619], [41.156405421317146, 35.928667988643824, 36.775113072509093, 0.68779335437172129, 30.117326739445577], [72.211009921644575, 11.098355828988485, 25.81214857676737, 12.308784710708597, 24.487496569871901], [24.126020687199421, 0.037087220745398247, 1.6267407180155635, 0.21294625074596829, 0.028937090701989035]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.71375101746233249, 2.8861162938346814], [3.6593730636868451, 3.6908720452807251]], [[1.4228281478043099, 4.0544509623915452], [1.366343855649373, 3.6254668636600118]], [[4.6474390140372668, 0.041208060480864436], [2.3340559479594689, 3.6381463804660226]], [[2.7374091624609327, 1.9957302986560055], [2.8457086218311085, 0.67879201650381471]], [[0.99824670502094193, 1.1961484018150987], [3.2316970925193651, 0.50535471462699355]], [[3.4914118479971079, 1.7761945368389078], [0.37592022813308967, 0.32490977157156997]]])+(1.-msk_arg0)*numpy.array([[[3.1187624322621219, 0.18408594725360741], [0.3231149712637344, 1.9912829665208835]], [[4.1996047867753532, 1.5803320592229437], [0.76824881541168477, 2.9574011881839839]], [[0.65490220116523801, 1.7862723366151794], [4.1007003944855427, 1.6550460911699598]], [[3.1575724728574905, 1.0341361436353429], [1.7440568211687584, 2.607028559091614]], [[0.39921629605298781, 3.6908052540250598], [1.6486316174456812, 4.2936932598862487]], [[2.3050437056813196, 2.7936944295395696], [4.3977464815345053, 2.2206144040841198]]])
      arg1=1.62545779912
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.57802471319767279, 5.6004213119304449], [8.2374690028949136, 8.3530338029271736]], [[1.773955990892333, 9.7312335422151346], [1.66091371953793, 8.113766007498791]], [[12.148641360502227, 0.0056067600897713636], [3.9659690596230397, 8.159941493158648]], [[5.1389838535073622, 3.0747010331550455], [5.4735284051768156, 0.53271527350190961]], [[0.99715165596551947, 1.3379390806726863], [6.7306656790959876, 0.32976786268419522]], [[7.6317712146009491, 2.5441125931668571], [0.20386186101807618, 0.16083886186648344]]])+(1.-msk_ref)*numpy.array([[[6.3525394788171825, 0.063873698147897215], [0.15939718265911607, 3.0635715893770108]], [[10.303837244589488, 2.1040548460654529], [0.65146062138489369, 5.8269960971868455]], [[0.50257696248898254, 2.567617394927117], [9.9123101157839546, 2.2681228403386946]], [[6.4815333559175761, 1.0560767077364046], [2.4697136958910137, 4.7470894786058855]], [[0.22479200320615053, 8.3527881012626981], [2.2538514511780821, 10.68169343191194]], [[3.8861510435657709, 5.3118398874496586], [11.105638398605292, 3.6574423387669315]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.6603153885129767, 2.3889385828826528, 2.8102061383506061, 1.7624681578571555], [3.384680802595609, 1.4156411559487772, 4.7540497050665174, 1.4816782060165268], [1.5836779989705299, 1.3065013917373134, 3.0702564325817328, 4.4130262994219818]], [[0.038064300741688178, 3.8719961365344906, 1.9996215712403662, 4.3898094231771374], [3.1068100144851036, 4.6443811379468443, 4.3478039391964387, 2.2142515211852936], [0.29126096577475591, 4.6865181996464313, 0.10412497796973892, 3.3985603633033943]]], [[[2.894538764530977, 1.4195406262162247, 1.8740655457721607, 4.1084768022604479], [2.3015578387205244, 2.2801381892669537, 4.2945721441341549, 3.9809399336269879], [1.5467801049846828, 2.0624677221154708, 1.3925608030782186, 0.93754071226523394]], [[1.5467633712554656, 4.0131893322413772, 2.499485404918961, 1.2959764887320608], [4.7533298692011217, 1.6695842027652124, 4.6225771840597831, 3.3008165840241928], [3.3483314380130227, 0.06309259218814732, 0.27943162811725769, 4.8844745196614676]]], [[[1.6332800440467741, 1.6142623888870495, 4.1193111579995758, 1.5516341533319187], [0.94411035368629681, 1.990525845480178, 3.8017761867103719, 3.130053245574965], [0.39782387070288788, 0.17566156101254901, 4.8715359591474856, 2.5731123739307145]], [[4.81531089298492, 2.0513828772301999, 3.813825393897003, 3.4944245016032784], [0.54299117814131115, 4.6889142999032565, 4.0923459353138991, 0.42777605077353947], [1.9850259010384057, 0.6067404914626453, 4.807610638315289, 1.6680817422764302]]]])+(1.-msk_arg0)*numpy.array([[[[4.8441503687604843, 4.0546715596547598, 0.31838802466938027, 3.7726290506930371], [3.0009825516015942, 0.68903257646565819, 2.1834680459433682, 4.9931396203035678], [0.45405025305272773, 0.32784952709186982, 2.6258985047005328, 1.7646834639845645]], [[1.4511941112768729, 1.2868940906237187, 4.5853094187186052, 0.41030668402274634], [3.3561330804997893, 2.7789174508902437, 2.4443925850430963, 3.1020474480712936], [1.1159578200121756, 1.434595400913429, 2.5580845392190201, 2.9613758906943555]]], [[[2.7932901972730448, 0.093683479613450346, 2.4004262781433097, 1.4207459512615097], [4.3132836135404427, 3.7844023905310267, 1.5956204526030879, 4.6910068611782894], [3.1051743312912756, 3.8682092980977063, 3.579997820721998, 4.4301278280715817]], [[4.8157837620926829, 1.1225295196651213, 2.4573165661368783, 3.8979292973243207], [1.06751152999829, 2.0462111080122938, 0.70177323013015025, 3.6788747954922001], [2.491931069804461, 2.9230009154711833, 3.0286616869165113, 3.5894042753609718]]], [[[1.2756872559512371, 1.0241813427530861, 4.7039240080480162, 2.0120058126999285], [0.55521752474933261, 0.78469195699991112, 1.7865097827384311, 2.020244174317757], [4.1990485242704265, 1.0020633325494983, 3.9835195290870962, 3.7157091068446912]], [[0.98736268364181357, 0.26448762440261026, 2.0300150919196085, 3.3914701452001057], [3.3428401570069819, 0.61186981825653797, 3.1169676509952726, 1.0895388263102375], [2.4496408997304724, 0.88204100668820495, 0.49444184913803618, 3.8336951815761271]]]])
      arg1=4.52674825053
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[355.55469586708796, 51.527462101056607, 107.4792834919469, 13.005560420945566], [249.45568414201134, 4.8231086335013913, 1161.173193823146, 5.9287056094918498], [8.0138593338912738, 3.3542867076876965, 160.44160369412754, 829.01023840604]], [[3.7528541642353878e-07, 458.59925961989893, 23.031116204053035, 809.44956972593161], [169.27183046478498, 1044.755076368127, 774.97445464362522, 36.538902596677772], [0.003757861519177614, 1088.3547015112983, 3.5704130756532774e-05, 254.11988933189909]]], [[[122.87219207404058, 4.8835417646585668, 17.172281893783673, 599.76161188575554], [43.529328952072795, 41.725360083326962, 732.94107831047654, 519.97669266509831], [7.2027025745092956, 26.494243827896181, 4.4772415739963494, 0.74680358901665755]], [[7.2023498483264339, 539.31899117661476, 63.236748426987717, 3.2336933119248998], [1160.3775161919925, 10.178689214038352, 1022.7353321179854, 222.6734724666928], [237.55612091300844, 3.696614733640779e-06, 0.0031147906713780244, 1312.5171602123148]]], [[[9.2145037400657586, 8.7386937720800724, 606.95455426144065, 7.30558953167301], [0.77078689888968221, 22.560673499783128, 422.13667269062194, 175.08055156275549], [0.015413497775556237, 0.00038092698925180733, 1296.8521517690854, 72.117961670073797]], [[1230.4628334575032, 25.855739988761421, 428.22696612251212, 288.22047294306367], [0.063019311280293797, 1090.8758766539859, 589.17554557737526, 0.021409660768646317], [22.279863455293512, 0.1041619531651492, 1221.5808311920698, 10.137290796560066]]]])+(1.-msk_ref)*numpy.array([[[[1264.1762974926778, 565.01804046952577, 0.0056235456158454863, 407.68308427059293], [144.69435207551751, 0.1852473887567524, 34.295123202827781, 1449.9798621883717], [0.028041147318505742, 0.0064206747243056245, 79.06161158809617, 13.079723956356938]], [[5.3962325562524001, 3.1323674922825409, 985.93745963313518, 0.017727193892722867], [240.07202674850259, 102.16762648502255, 57.167895288880842, 168.10038157258504], [1.6431965105545416, 5.1224141933003464, 70.230866661556561, 136.24875448372111]]], [[[104.58155181763951, 2.2129792576279517e-05, 52.658650456158, 4.9023404655798197], [747.50839332489397, 413.47409324757598, 8.2910815461583915, 1093.0813881905001], [168.86878658902569, 456.57245192961142, 321.5791071790735, 843.65262749946521]], [[1231.009908191648, 1.6874568276220148, 58.548953876779102, 472.66841549744612], [1.3441121342432241, 25.561971772591018, 0.20126655940734489, 363.78888090165657], [62.376177540646971, 128.43707033221889, 150.83462662953923, 325.42174421624173]]], [[[3.0107690931940705, 1.114226372428583, 1106.7727898357787, 23.683893010548928], [0.069702616674645848, 0.33367970515928158, 13.828179159237965, 24.126059965251425], [661.98357877614592, 1.0093742297040245, 521.50366943320751, 380.5706564028423]], [[0.94405534251162004, 0.0024287032560438909, 24.658790296564732, 251.72882212635636], [235.79762339451304, 0.10820794340950914, 171.79155183290922, 1.4743085279784476], [57.725635045108326, 0.5665541345430477, 0.041241705444297061, 438.41949225814261]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.36787301516)+(1.-msk_arg0)*(0.362264675499)
      arg1=numpy.array(3.82031677506)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(3.30931405154)+(1.-msk_ref)*(0.0206699575363)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.17705460196386788, 2.4383553739235095])+(1.-msk_arg0)*numpy.array([3.213009111859924, 3.6651953585554389])
      arg1=numpy.array(3.59830217632)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0019699834196596532, 24.711094559854754])+(1.-msk_ref)*numpy.array([66.684116064200495, 107.10060473710207])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.39630351038)+(1.-msk_arg0)*(3.7306103215)
      arg1=numpy.array([0.45550879795113641, 4.1280997527774632])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.7453323150783506, 155.61368122821091])+(1.-msk_ref)*numpy.array([1.821590569453339, 229.27907031129882])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.3536615648918842, 2.174984555496231, 2.0045703524810414, 2.917904305953944, 1.8733722501657992], [0.21549353574156185, 0.15975894327145304, 4.0141948321149981, 0.36330837399075183, 1.1017066855016364], [3.8887277369496065, 0.38260398557838554, 2.046004752275441, 2.5099037118802432, 0.66754856085498282], [4.5638388122858498, 0.81976926195036304, 1.9122223195494843, 2.3048063510664778, 3.4135675000194068]])+(1.-msk_arg0)*numpy.array([[4.7256579216406784, 2.0415952729063807, 3.335438705907896, 3.2922799687009618, 2.3564920852824258], [2.8290010246985986, 1.2719672714995789, 1.9141179026112844, 0.47797879518919006, 3.9219608454828467], [3.4711309005142037, 3.9410091469904853, 2.708628767992082, 1.0695426417043621, 0.076212122308649932], [2.6848378023836728, 4.862536151620005, 1.2864601985759225, 3.5680284014007753, 3.9162077203516192]])
      arg1=numpy.array(0.125837530787)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.1137285402773764, 1.1027184782620525, 1.0914544477841746, 1.1442565099986659, 1.0821970392301872], [0.82436778431494329, 0.79390040769207182, 1.191119509914063, 0.8803718411861523, 1.0122632721428815], [1.1863693666092978, 0.88612344260287845, 1.094268061577468, 1.1227727346248206, 0.95041516415671068], [1.2105103208400836, 0.97530211909114783, 1.0849958946756111, 1.110792714847751, 1.167071974932675]])+(1.-msk_ref)*numpy.array([[1.2158294761032626, 1.0939710153475266, 1.1636765354766849, 1.1617709544059702, 1.1138969951742768], [1.1398098210994416, 1.0307349301285007, 1.0851311812265065, 0.91129203337459219, 1.1876404584252671], [1.1695304538794455, 1.1883647743859753, 1.1335903151710902, 1.0084960866744341, 0.72329700651085171], [1.1323325424876038, 1.220205910099051, 1.0322054958682219, 1.1735894953741868, 1.1874210898805175]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.829690572291)+(1.-msk_arg0)*(4.367815865)
      arg1=numpy.array([[1.9279984638428758, 0.25889931251637494, 1.3212308573346148, 2.3235617415428131, 4.9067816751926303], [1.0918774936002886, 0.67691430886378123, 0.18171619216610641, 0.18939044337387639, 1.7432094070686379], [1.7838128580386303, 1.4858995049615895, 0.19667573133743232, 2.699265572555817, 4.9172874648571225], [1.0849922514786459, 0.28648881719300218, 4.1633398400258246, 4.1067395713235575, 0.37676497297619332]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.69770280968523346, 0.95281250503840609, 0.781393010733056, 0.64803234100837404, 0.40007303100953295], [0.81557963860336125, 0.88127852078057067, 0.96664220354238184, 0.9652581888707028, 0.72219400518371457], [0.71673992214479343, 0.75773536176885414, 0.96394615540284867, 0.6041338489046717, 0.39928907418546172], [0.81662873352491783, 0.94791715983191382, 0.45964273360105029, 0.46452572116559715, 0.93207411554072006]])+(1.-msk_ref)*numpy.array([[17.156497375823268, 1.4647516390302655, 7.0135351568549549, 30.739279353293448, 1385.5979740800583], [5.0013840243239382, 2.7127144490686823, 1.3072131091999659, 1.3220867225278035, 13.065161670519817], [13.871126396015462, 8.9406365032444253, 1.3363630334904752, 53.4864089020327, 1407.2255832830606], [4.9508734755683941, 1.5255574536220098, 463.06073584047022, 425.98943173131801, 1.7427262375992467]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.2899385382856461, 2.5260413142309917], [1.7956076656763977, 4.2775279422810755]], [[1.3621371181242403, 3.131606889481076], [2.837623975525104, 1.5763140603039199]], [[3.9673396989399863, 0.03145194868584774], [0.86994090395493262, 1.7190643999611968]], [[1.1400394270648841, 0.80840626935289239], [2.9566255345400876, 4.963531053050378]], [[4.5358963287030996, 3.5849836939725224], [1.3390629805226055, 4.6024332609305514]], [[0.87702000431480931, 2.3565772932328102], [2.0294951171934925, 4.4801121658998371]]])+(1.-msk_arg0)*numpy.array([[[3.4847376071510259, 4.8726288419447723], [0.99574570276489671, 2.0897308253367859]], [[0.49345862081913494, 3.6258834541315759], [0.67869520038871178, 4.1705360895404313]], [[2.285421927766452, 4.2469841915013715], [1.3176975251508694, 0.27907324938957573]], [[3.0273264519571632, 3.1130549616367702], [1.6110120017054934, 1.0877833885699917]], [[2.6929800898543053, 4.729232731039235], [0.5893170526956617, 4.740304211526305]], [[0.38088415631455419, 3.8029299703341497], [0.75276755453448163, 2.028978696115165]]])
      arg1=numpy.array(4.40827919201)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.0719850733712448, 59.438776152295354], [13.201857749317202, 606.00237977867198]], [[3.905556108153903, 153.27820555879714], [99.254776941625138, 7.4347166161292053]], [[434.86294102516075, 2.3834771501501282e-07], [0.54107059286590431, 10.895156527604177]], [[1.7820450709494604, 0.39156675235688798], [118.9601863259625, 1167.4542098427628]], [[784.79163896869011, 278.18539049260573], [3.6222150151140515, 836.82375300715387]], [[0.560750878222817, 43.764464817813362], [22.649163755577852, 743.12735780591163]]])+(1.-msk_ref)*numpy.array([[[245.49185764031267, 1076.1003768488758], [0.98138137224336042, 25.766036933454529]], [[0.044438932977698409, 292.45051674993573], [0.18112376037450681, 541.97465294790504]], [[38.23174030231511, 587.15780155842958], [3.3742805656167834, 0.003602227312256844]], [[132.02114859744066, 149.31555631384737], [8.1836910243587919, 1.4490694501069552]], [[78.811176354470263, 943.33455457542505], [0.09719284869156912, 953.10877819904681]], [[0.014191166567939213, 360.84737858476234], [0.28594781138565822, 22.623768716405923]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.3343599468)+(1.-msk_arg0)*(0.074604206878)
      arg1=numpy.array([[[2.6206791462707595, 1.2273643741406222], [0.086910295394941883, 1.4957526136700254]], [[1.4391090263152231, 4.8022787303773384], [1.6271346881310622, 1.7200608929883479]], [[1.6820845795288455, 1.0015320876340745], [0.59943766736182236, 0.84835839124257073]], [[4.3408823332860766, 1.0958242430212417], [3.5301860261808859, 4.2173083580278643]], [[1.4498634203163312, 0.025939669311294627], [3.2141533008010286, 3.9793596266216946]], [[4.2271409839234453, 0.20289524468443751], [4.8889211651288598, 2.3312041942178521]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[23.477286552071824, 4.3845687655770211], [1.1103379127914363, 6.0575538644871836]], [[5.6581177534710267, 324.82669295745529], [7.0959696493282838, 7.9362129789288005]], [[7.5814317252205816, 3.3405177320776995], [2.0583223096647263, 2.7778067347724722]], [[186.35267726794908, 3.7422237913265022], [70.199068611143645, 160.58497379504675]], [[5.7318743795935969, 1.0317316912713195], [47.978016605379089, 120.57429990676918]], [[162.49780211210827, 1.2767819817852883], [360.55106716428133, 16.567191339339264]]])+(1.-msk_ref)*numpy.array([[[0.0011114100864196164, 0.041349461204697797], [0.79805261860993637, 0.02060311023796443]], [[0.023866213190192404, 3.8609595935114523e-06], [0.014649880850487696, 0.011510236991055554]], [[0.012702598149102445, 0.07430812317738375], [0.2110045754074806, 0.11058578411331153]], [[1.2787869641378608e-05, 0.058176382942725702], [0.00010486844203593981, 1.7623628048764176e-05]], [[0.023209231922778854, 0.93488857671953507], [0.00023817078981626695, 3.2682845195518082e-05]], [[1.7179543591154514e-05, 0.59059400950828311], [3.0833918907664661e-06, 0.0023560316396703555]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.46949024694415914, 2.7589002427526168, 1.4293035075954446, 1.5975322310801285], [1.5912152996546207, 1.2360059309643836, 3.5975871576571157, 1.8386351886091892], [0.29249455355170889, 2.4246014434792746, 2.4817487925618109, 3.4042023417967244]], [[4.5144213583380237, 1.8862854769946003, 1.6899161167303489, 3.5252253473573512], [2.2410066716638726, 4.2818340104828003, 4.3640922381478662, 3.0954626533755598], [3.2303654693453958, 2.7316761734199098, 0.27164721127471159, 4.4524347319522688]]], [[[0.49659897551844062, 4.5290844088118494, 4.5154220764311557, 3.0766157561979139], [0.50941728388666596, 3.3326771411614051, 4.0028468012266085, 3.149367013149023], [3.7158528893754648, 3.2777898603596047, 2.4124102516513326, 3.945272168261452]], [[1.2083730104696533, 2.2315450752024351, 2.6728944998538942, 2.7157874520600251], [4.8493313712654453, 4.6643284304097721, 0.14610186992836419, 3.4804752685642879], [1.8477243645684607, 4.5813214228532022, 3.6487766087640217, 2.4982195184973013]]], [[[2.5225480832319316, 1.1826833628972642, 2.2060163937397204, 0.21137033800455479], [0.9951789484388458, 3.7248419064172946, 0.32846209315433494, 3.5454861334404892], [3.5054592599805607, 2.5664832385247154, 0.86460089827436926, 0.77850640258201709]], [[4.1898845871328243, 3.1758893396086125, 3.1352566284128232, 0.62713039833979778], [3.7855300287938114, 0.66315424161112968, 0.95710266138657896, 3.3460723270981321], [1.9404648762199614, 0.76413602239843015, 3.6715441285371639, 4.5786517890275462]]]])+(1.-msk_arg0)*numpy.array([[[[1.6972811226063222, 3.1000779854841936, 2.7585830033440217, 4.7739876262809027], [2.4711261956201684, 1.3371717986381646, 0.94072835981064384, 2.1942052562449903], [2.3701626594278946, 0.58757009623719458, 3.1764879678409517, 2.2181822834609308]], [[0.73624953254768155, 3.6757291743768632, 0.5344941952423562, 2.8709476017269702], [4.276806344791761, 2.9604212849504763, 0.27544550579150145, 3.6309243888828462], [4.0460674647282495, 2.1136237955379049, 2.9736206741021252, 2.8268358568547063]]], [[[2.2046993490948923, 0.22859460496946629, 1.8652051520149904, 0.27041969471210692], [3.3796961672428494, 0.58745796197752675, 3.8867368487656657, 3.9124840237710656], [2.3070139556301275, 3.4307611199261334, 3.1194149888125264, 1.9436304412439191]], [[2.7541902894037222, 1.0880274965259635, 2.5247670281519183, 3.2189755755109046], [1.8548259216200582, 4.8408189405377771, 4.3166950249214295, 2.4086945351781015], [2.5587750023351061, 2.3939940969564297, 0.89937029849809247, 2.9704467846557971]]], [[[3.0270331402605835, 1.3593653666062027, 2.9614280218114342, 1.4077826577235173], [3.9357257958011216, 4.0810286097438455, 2.9468834645281747, 4.9699458490472281], [3.8039858239364124, 0.15212271211417566, 2.4019171669886736, 1.5247050855113444]], [[0.20992909468227883, 2.9558456492095173, 0.4124018740022144, 4.9423664063985182], [0.86153514616011262, 1.1250117852942012, 2.2134775411864775, 3.5628841546409271], [1.3268965469751046, 4.6395869225029234, 4.2414594838112301, 3.5473988966217296]]]])
      arg1=numpy.array(4.59697388928)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.030936839083246091, 106.18223736045881, 5.1653896725576871, 8.6149474470736873], [8.4594610875543452, 2.648598247215102, 359.72410419226122, 16.43914469182204], [0.003513641042473141, 58.638342907461379, 65.266659898014609, 279.03632713880853]], [[1021.3832340750839, 18.490988056297187, 11.155528110323164, 327.64520687658506], [40.829870998970016, 800.91388784191747, 874.12923906507945, 180.24060057680592], [219.28886552882824, 101.45038607741493, 0.0025011337528564287, 958.48662416097329]]], [[[0.040044851611486132, 1036.7230334689066, 1022.4244566172239, 175.25080906200236], [0.045022057347034398, 253.08527982135556, 587.5900347899186, 195.12790560309057], [417.39197493200624, 234.48382435817069, 57.295171150366507, 549.73104235962603]], [[2.3871292085756322, 40.043417328320587, 91.796107412714079, 98.766021381395888], [1419.2588653648661, 1186.8781519441854, 0.00014452547534084321, 308.95717221407739], [16.816057355510299, 1092.8417477051826, 383.86308174331128, 67.281780021743415]]], [[[70.346982645482967, 2.1625917132577515, 37.980474424407547, 0.00078930541840317053], [0.9780291113912899, 422.0538342289737, 0.0059881869073713642, 336.3916906496757], [319.28476239341535, 76.15844797269456, 0.51232239657119805, 0.31632616806558195]], [[724.84746557587891, 202.79720388412031, 191.14126602067518, 0.11707369861570126], [454.60407493836607, 0.15134453810194964, 0.81746060316582625, 257.79541530483107], [21.061794998915595, 0.29036123576652767, 394.99806318306605, 1089.9173590456896]]]])+(1.-msk_ref)*numpy.array([[[[11.380782723094086, 181.4793018155591, 106.12612155135186, 1320.6859557670871], [63.992298464513965, 3.8025882610684878, 0.75511968026872422, 37.054641189140106], [52.825714272700424, 0.086770759868804018, 202.97298534035514, 38.952940055549611]], [[0.24474709429338831, 397.07206317075628, 0.056151458653994603, 127.50587302985072], [796.59991640025601, 146.82493428070376, 0.0026659910767939591, 375.30510315919634], [617.32718536445896, 31.199224751257393, 149.85850513341336, 118.74546573113682]]], [[[37.876348743358889, 0.0011314699622954317, 17.559944569581795, 0.0024495989008178343], [269.92107195890856, 0.086694661559342412, 513.22462226716095, 529.0406311911072], [46.658673695744461, 289.18519524305992, 186.74173571413587, 21.220206605715294]], [[105.35148610706712, 1.47377956375306, 70.631895225813679, 215.75701479849266], [17.115224709116333, 1407.8423328069591, 831.33152324038292, 56.890615426140322], [75.112619222674155, 55.311938271291112, 0.61412485645874959, 149.12462360965409]]], [[[162.63851374965856, 4.1015030062233286, 147.05460197858216, 4.8174179318994286], [543.6427767541395, 642.23221226019723, 143.76371283721915, 1588.9505591991128], [464.88231595755599, 0.00017400729087847696, 56.158475479618545, 6.9518527415959221]], [[0.00076486642013412324, 145.78462496667893, 0.01704664750637493, 1548.8194563181992], [0.50402452261311159, 1.7185719705863045, 38.574589722904477, 344.04718980371626], [3.6701074173180186, 1158.2118704645518, 766.78137257598132, 337.22676296149803]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.72736355587)+(1.-msk_arg0)*(3.63176811662)
      arg1=numpy.array([[[[2.4230442914827521, 3.3633533566120124, 4.3216792142332032, 0.20604426147165311], [1.1332614496079298, 2.8987612834391157, 1.8503929428950814, 3.3394589682505229], [0.56719130025909048, 3.0122765532520481, 0.43716356327367201, 4.6038852854141963]], [[4.7221791339331647, 1.5944328753788479, 0.87606184479629701, 2.3058638551054385], [1.9869284123467061, 4.6771345274692067, 4.0583455998726743, 1.540780351857715], [0.81512502978039347, 3.4558222581035007, 2.2483651018852151, 1.8688839193765736]]], [[[2.6608003254461887, 4.6406870095544601, 2.9163540492687412, 0.96579270529611805], [2.0813023369333812, 1.4764678509018478, 4.8593145245656757, 3.186555303589989], [1.8733616532678281, 1.7010830923009856, 2.6160206413276992, 1.544239420440874]], [[0.74132980809094351, 2.8524892789376151, 2.638186607371245, 1.2912242638173426], [1.9128485148301406, 4.673237769121215, 0.13874269750232499, 0.82469676763517474], [4.8092450423318844, 3.269310879900615, 2.4589687744781958, 4.9282347076275368]]], [[[3.6479426607678116, 0.84856814278839954, 0.82018013279983659, 4.3068804137573178], [3.9192442034498951, 2.9641098816461025, 3.7630781747058228, 1.8916110522965071], [3.1493495514357241, 2.5150671244784171, 0.17595423041099323, 0.49371327061423426]], [[4.1465071974206458, 0.10618424681505306, 0.73323920038175139, 2.3788105245377915], [1.1525239703578165, 1.9007791584780322, 3.944944563006235, 3.0926853431068531], [3.6833522917585131, 1.3988949051943902, 1.0760051113893669, 0.16876875501291017]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.7600303159954001, 6.2864496421849987, 10.614423936857156, 1.1192099435977112], [1.8578808382398651, 4.8766190542690051, 2.7494968122026768, 6.2048787794597366], [1.3634587371818843, 5.1887830418418508, 1.2699175559534328, 12.384759157055854]], [[13.212001157469254, 2.3905204273612033, 1.6142204621055243, 3.5267495111282487], [2.9625420445256392, 12.890677560148546, 9.191477061045207, 2.3214333634395872], [1.5613398285940709, 6.6123533883316981, 3.4176323744734223, 2.7774271587015398]]], [[[4.2818432647402922, 12.636409657543648, 4.9237395037485641, 1.6953661065690351], [3.1193733604937961, 2.2412457497231899, 14.240404206628961, 5.7073756193710485], [2.78423327266347, 2.5340167884822433, 4.178311330568806, 2.325826682071773]], [[1.4996145714738924, 4.7548257848491131, 4.22924297707777, 2.0254226444900389], [2.8449796124730056, 12.863250234188218, 1.0787859017192611, 1.5695299718219731], [13.855960452658184, 5.9714704146006712, 3.8345926466091069, 14.787094501824614]]], [[[7.3445013996736295, 1.5901433964731213, 1.5656599332354446, 10.528910510070798], [8.5185346878352721, 5.0539568156574068, 7.8215642703524777, 2.8121451290538615], [5.5924796781270523, 3.9539946007877345, 1.1009527370716927, 1.3097834811948603]], [[9.6452488832371266, 1.0597572954614582, 1.4929974738836, 3.670210462788055], [1.8775454766644266, 2.8262728685473495, 8.639045154975749, 5.4219218341943227], [7.4880371264706245, 2.1482010376707619, 1.8006368391611272, 1.0966371658989789]]]])+(1.-msk_ref)*numpy.array([[[[22.760981874833284, 76.537651281230083, 263.42124653886378, 1.3043949917287707], [4.3128093776452427, 42.038643727550017, 10.875233764726634, 74.214958881249458], [2.0782324419680207, 48.666560619987742, 1.7573700859139691, 379.07010209594301]], [[441.54813961467715, 7.8175511460367133, 3.0952669190860376, 19.568435638661363], [12.969241886778393, 416.62733373285596, 187.56546313311387, 7.2948939236286847], [2.8613190903334811, 86.232035427518724, 18.169794485397084, 11.137705540081468]]], [[[30.928756094279148, 397.49607482407185, 43.003392983349059, 3.4750254239369003], [14.647892184715918, 6.714232633545751, 526.97507620468582, 60.932267392368836], [11.20221211011679, 8.9703099491905292, 29.193121958501145, 7.3275108146300383]], [[2.6015504495743405, 39.603257390778815, 30.039735241791536, 5.2873504449087871], [11.787483503750364, 414.53873181191585, 1.1959480023530671, 2.8968605943962973], [494.02081917858442, 67.795401393194311, 23.840367540505955, 575.96158490580183]]], [[[110.47875067865496, 2.987434456301242, 2.8800348777788769, 258.44118540612561], [156.76159089191023, 45.73531025674113, 128.16464565438008, 11.469001526775154], [58.077474609243367, 25.629176290334243, 1.2547440683115991, 1.8903305139360531]], [[210.15212280066865, 1.1467684070181938, 2.5745454096827163, 21.498831651388105], [4.42129557810458, 11.605419216667963, 162.0447253723369, 53.98449887165426], [115.6411308936699, 6.0750012644983995, 4.0058063540362747, 1.2431697569990439]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.1778528324239228, 0.31756847934611787])+(1.-msk_arg0)*numpy.array([3.7533559055373704, 4.3430317948873141])
      arg1=numpy.array([0.10722876037143872, 0.47340929400677101])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.1319915445236797, 0.58098536175684934])+(1.-msk_ref)*numpy.array([1.1523762988163209, 2.0041819109326924])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.2323416968233674, 1.3760728175478043], [4.1928751407361515, 2.6619263653683718]])+(1.-msk_arg0)*numpy.array([[4.9707474106910778, 1.4890812095887704], [4.5452194975819973, 0.61874237321666192]])
      arg1=numpy.array([4.9748789032414615, 3.646087376878083])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[54.330300356755608, 4.8946794546296699], [186.09422204228827, 35.50601286154356]])+(1.-msk_ref)*numpy.array([[2914.8324650828936, 7.2484979014502322], [249.74826409393575, 0.17371076295897556]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.630038991275367, 0.62961752472780619])+(1.-msk_arg0)*numpy.array([4.8988311373855593, 2.5678088091670208])
      arg1=numpy.array([[0.8121334990703899, 2.2872204711421267], [4.066219412883922, 3.688303421281053]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.849198552724042, 19.082732585162354], [0.15240605258710968, 0.18152410418361739]])+(1.-msk_ref)*numpy.array([[3.6345212467893266, 37.878574218819196], [46.277696994430379, 32.403464781986827]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.4300029182997842, 3.9924782303949682, 0.29545865857442349, 1.0791529126677835, 0.88486674350631811], [3.3092000282896641, 4.972234120376954, 3.0775010595657313, 2.1580206224287797, 4.0363380718950674], [1.9941996028031721, 0.23662227368755706, 3.6035489899042163, 1.011783553757919, 2.7028924489983153], [4.7024480861255649, 1.8251194601009211, 2.2357404219211783, 3.8600082659524828, 4.8143087301250924]], [[3.7281310853890353, 2.7914757280476614, 0.6055805435007714, 0.59034680699322051, 3.7322962105272959], [2.2467382103459128, 0.33488292726255947, 2.346196758790136, 2.1655923958213461, 3.6719400167366274], [2.1415227942999442, 2.6862415322236868, 4.5865630602395653, 0.51482563443751439, 4.6777358483105402], [3.1153868074022601, 2.7154449924701414, 2.2158096763013062, 2.0153142071263224, 3.8638006445520201]]])+(1.-msk_arg0)*numpy.array([[[3.6950418577467792, 1.6631608880398039, 0.36836112685644878, 3.8059850924134175, 2.6586817838000631], [2.9592924095066713, 0.56295314542970742, 3.1156896245449595, 1.8934273722984412, 2.533908839084519], [3.1330658060962837, 0.013423955561978763, 1.7532092721021708, 2.4726929139221152, 0.66296144725584649], [3.909599319447532, 0.78614707401546036, 4.0359883019688487, 2.3735595021944129, 4.7038489517548179]], [[4.3603588119731302, 1.3371838473304083, 3.6492762808682655, 4.301612976166898, 3.1133063285724529], [3.6872040735544016, 2.1289814238480487, 2.9640758786591652, 0.34770729468193234, 3.2503569575004549], [4.0581143060351383, 1.0728231329096287, 4.0794031598980522, 0.41459163981187075, 3.2562707337096093], [0.319786706176306, 0.47661474047433428, 4.8582658453484608, 2.8378462970522662, 1.1984403558760059]]])
      arg1=numpy.array([4.089616144241158, 3.5607161795879847])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[154.57846358982991, 287.6413157894367, 0.0068317952936932519, 1.3655157658079879, 0.60638841007121935], [133.49581483220254, 705.71369226244008, 99.207221343994831, 23.235880605032467, 300.78515872816837], [16.824362225045761, 0.0027550391267912042, 189.15322796845641, 1.0490746570555292, 58.346376333907955], [561.75522996622715, 11.710652699312538, 26.853401928574744, 250.56517759845431, 618.44603308317676]], [[108.37219521242827, 38.680121043588898, 0.16763936523517856, 0.15310090924893485, 108.8039262338476], [17.855769582254293, 0.020336623285804736, 20.833518373604015, 15.66366724676951, 102.66746169396724], [15.052535971895331, 33.733682783325619, 226.65681265994354, 0.094038063766567981, 243.11230163057687], [57.181570543962678, 35.057803480477993, 16.995854386608148, 12.124901322288894, 123.08149923840695]]])+(1.-msk_ref)*numpy.array([[[209.57781518546355, 8.0082261683162539, 0.016835510931070982, 236.53071975039316, 54.540912749264869], [84.523517418818813, 0.095395367481501167, 104.33912145128707, 13.609449546579835, 44.80744054475295], [106.73944563200722, 2.2067335236186398e-08, 9.9354256233654148, 40.542930136283985, 0.18618966178780769], [263.99372914677298, 0.37380978293815237, 300.67857872134459, 34.295915879161853, 562.4399326010099]], [[189.30284751303475, 2.8140432583447632, 100.42888728101927, 180.37707548653677, 57.045716241754157], [104.19522088238753, 14.740998248192437, 47.892105058336305, 0.023248372836313005, 66.502994622489581], [146.57843860230315, 1.2844050504103472, 149.33489206229811, 0.043496596045871153, 66.934835611175131], [0.017256329932475511, 0.071457151963254223, 278.20503505700083, 41.017085411384976, 1.9051570232170953]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.6426097628029241, 3.7404177549001574])+(1.-msk_arg0)*numpy.array([0.098878036946074929, 3.3576843244589072])
      arg1=numpy.array([[[4.9791629649745888, 1.7537722483839455, 0.99615935445834281, 0.92173727045771348, 0.9501846569242518], [2.6722288230340521, 2.8025161401169427, 1.0267764337062029, 3.3280499936028418, 4.8012013798155149], [3.6377056451227676, 2.5189355515453413, 3.5192986783655198, 0.14447046113744266, 4.9423488104022999], [2.7107729775399561, 3.2535985488075867, 2.1099630387640573, 1.4928854145652422, 3.0533891637467145]], [[1.1773351505607317, 0.34056680387859845, 1.3478350357584108, 1.1609498936367781, 3.39885308670888], [4.8603304587417648, 4.1552875706206844, 1.6343080695530701, 1.3670913863115048, 3.6547667847836562], [2.6611251949248853, 0.56985345197591708, 4.9546593140194632, 2.7755224757232542, 4.1775486932751766], [2.9324636499171213, 3.8966200571189447, 3.0757994702158462, 4.1914575655002659, 2.8956668266404439]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[11.835344417322261, 2.3878016545541834, 1.6394818324081886, 1.5800327515727901, 1.6024979390419472], [3.7666670700622804, 4.0182655493140178, 1.6645837179304455, 5.2156598182473646, 10.834878792231898], [6.0820600333961812, 3.490738066633611, 5.7349538584500737, 1.0743316099396774, 11.621071788786724], [3.839412959609338, 5.026462075122998, 2.8495062703775389, 2.0978197108564869, 4.5510373580138683]], [[4.7262749580907943, 1.5671752728866077, 5.9183656958576583, 4.6252109011914477, 88.566867341548672], [608.94919432098231, 240.24137857967523, 8.6363047455943782, 6.0706353592940641, 124.13361062457355], [33.466642072662182, 2.1207061815607307, 689.64240599788025, 38.918190684188204, 247.40111702383501], [47.870437471525243, 170.7858914968642, 57.834551266478073, 251.98247003688721, 45.602195944019137]]])+(1.-msk_ref)*numpy.array([[[9.9183246168228179e-06, 0.01728359095191867, 0.099760657081227527, 0.11850756247932054, 0.11095816914832075], [0.0020638437597374853, 0.001526690735769808, 0.092937758328371534, 0.00045252441382663045, 1.4971728549768179e-05], [0.00022103904695756977, 0.0029425323780448991, 0.00029070746211821709, 0.71584930864348784, 1.0800226616353243e-05], [0.0018877470026703003, 0.00053759879128859262, 0.0075804990139752395, 0.03160815901664428, 0.00085437538487413527]], [[4.1622218186551514, 1.5106077724112958, 5.1169945895162048, 4.080429917982519, 61.366621285968847], [360.35252491282398, 153.40761038577844, 7.2395436879855124, 5.2377475909661184, 83.666773120485104], [25.110648764032479, 1.9941858571544955, 403.9692495478148, 28.842670149934911, 157.60033740897137], [34.881309214160282, 112.1443715761982, 41.494748093139748, 160.27794451453676, 33.360778849285275]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.4256641726888644, 1.2251738329836805], [4.2500626107858324, 1.3611480194712662]], [[4.4481856843549616, 1.7477175387868757], [0.83978248240628051, 2.0106250376468613]], [[3.2479049273039373, 4.8790305925840265], [1.4059991042308104, 2.6727819225829252]], [[0.39566560096955572, 1.3923291655844354], [3.3133614142624404, 2.3280058259908296]], [[1.030881753967227, 1.6428103571551809], [2.2574560544274975, 0.51666303326598917]], [[2.3410627466628875, 2.4286300558732603], [2.1090837584494997, 0.37116836458421981]]], [[[2.2207800510829765, 1.1948978288150571], [3.5678736896320737, 2.9849556629592731]], [[4.5976434023448292, 2.1027683608450034], [4.9452521623020953, 1.875362491620987]], [[0.33236046882586018, 4.6493162630081715], [1.0458674639273013, 2.5366170661365359]], [[2.1513255936428961, 0.345679434237115], [1.4552723204009206, 0.6152307832845243]], [[4.6218825739377118, 0.71754969851929962], [0.87605931717934504, 3.6288544681216917]], [[3.8742389781543278, 0.76298250849542726], [4.5993428606076696, 2.1894981648738416]]]])+(1.-msk_arg0)*numpy.array([[[[0.2667184165844117, 4.5113213322621952], [0.83675284550568163, 4.7739778577159964]], [[2.4558625865456221, 0.23432798915568076], [3.4569782779685485, 2.0731550256713156]], [[3.0112369357951003, 3.2508868801154711], [3.2343456091301048, 0.62772220506916554]], [[4.3604130599906474, 1.0096500878672905], [2.3668729910718391, 1.7628916717363088]], [[1.5324586320895874, 0.80173833169258457], [3.1765426316131928, 1.2223419724921634]], [[0.5139713318109943, 0.93379776057577302], [0.40575380659830806, 4.1876899010924946]]], [[[1.6920069845538908, 1.3159373645820325], [0.69133672212380226, 3.123626428867158]], [[0.20308314073337916, 1.094389293356373], [0.80099502963309899, 0.21416602544468516]], [[2.1614227565767981, 3.5582455395881292], [1.7168901972920994, 3.136786289002333]], [[1.9813422228953859, 4.3445763466953764], [0.22576821273125544, 2.0439662016215796]], [[2.3578164712490151, 2.7425556909834392], [0.99316041125277499, 1.460316491126703]], [[2.9581278432493718, 2.6668411731786574], [3.7270574191390651, 3.6394971854030076]]]])
      arg1=numpy.array([2.4228354292984062, 1.3933030920048479])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[36.736644809149205, 1.6356430891081546], [33.304158464685955, 2.1107200698996289]], [[37.191227655563949, 3.8678348466341577], [0.65504127052247008, 5.4315189975294285]], [[17.359236524113989, 46.528482756943689], [2.283197118081802, 10.825858742927357]], [[0.10577736045449823, 2.2297851447090355], [18.219049110467683, 7.7471504383461998]], [[1.0764723443407862, 3.3291462091464408], [7.190537904109461, 0.20190575684883169]], [[7.8528451825504106, 8.5835545034590695], [6.0985418329295076, 0.090602724185071357]]], [[[3.0393957430120269, 1.2815784726557307], [5.8840157157951545, 4.5891517974844236]], [[8.3774618672135794, 2.8167370272311083], [9.2728856019584853, 2.4015436749922325]], [[0.21550474957812321, 8.5089364852629465], [1.0644784663222415, 3.6580498211225478]], [[2.9077729962420933, 0.22763153151462073], [1.6866724602492005, 0.50823675927338241]], [[8.4390631986205644, 0.62973542806818117], [0.83163309869197088, 6.024605958772538]], [[6.599665716740148, 0.68597328652394507], [8.3817767086926924, 2.9799101081153156]]]])+(1.-msk_ref)*numpy.array([[[[0.040683741306094014, 38.483125437901485], [0.64933041924688828, 44.138288220546833]], [[8.8186118663269646, 0.02972950389178005], [20.191720726251507, 5.8498764225134048]], [[14.451741183166332, 17.397876389466866], [17.18417203547045, 0.32361147568864229]], [[35.438081630988016, 1.0235413061351606], [8.0642565372543764, 3.9497005499829485]], [[2.8129787761303775, 0.58544607618266398], [16.449534126746261, 1.626498333441408]], [[0.19936665176170659, 0.8470859879687993], [0.11243065874488828, 32.132305707297057]]], [[[2.0808147793520244, 1.4659892932752383], [0.5979144653663957, 4.8888874339535011]], [[0.10848754066191353, 1.1339090973774286], [0.73405251653964965, 0.11682417145788972]], [[2.9268056610728981, 5.8619040151538924], [2.1235743956802824, 4.9176089380413694]], [[2.5927138161226306, 7.7420171946699243], [0.1257350543139745, 2.707596726495975]], [[3.303842465326158, 4.0783392642494238], [0.99048321511702619, 1.6948235857394156]], [[4.5317856286218712, 3.9223211381381593], [6.2529669162999344, 6.0492384012037617]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.0191463906297804, 2.6175757777751225])+(1.-msk_arg0)*numpy.array([4.87195460765078, 0.093972736113348956])
      arg1=numpy.array([[[[4.7708292117166184, 1.0757687204783415], [2.3888485974837836, 1.5599211653153964]], [[2.3962125126588791, 0.45603292708300702], [4.8359016333248412, 1.9220678859679237]], [[2.3132517130932717, 2.9575459544156817], [3.5388676060754771, 0.25660810654628768]], [[4.9186894128458105, 1.3570980505823009], [3.2077321905817149, 2.6047488266672132]], [[4.5711816300203978, 3.949752612080736], [4.8441796090318539, 2.3444161512220933]], [[1.9695102118598291, 0.56355787961172443], [0.79954178397028508, 1.4943167829598349]]], [[[1.7929057433974394, 3.1160946784887509], [3.8047804417167463, 2.9417302038008533]], [[2.086388773283355, 1.7991254296285528], [2.7022849570871137, 2.7183553235393587]], [[3.0839424464892335, 2.5874520386972377], [2.3118442783890094, 3.8316607804070473]], [[0.31727270278710323, 1.9957493652480405], [1.903434597208759, 4.8317995924507233]], [[0.14683803990135874, 1.8952965276238185], [2.791112225646736, 4.7743669798553734]], [[4.9815366315159713, 1.2353893119904105], [2.5040313379364827, 1.4115696889474536]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0947003886387885, 1.0206119408996461], [1.0463474504779928, 1.0300265067874039]], [[1.0464935931844477, 1.0086863585071451], [1.0960522195618529, 1.0371253462720067]], [[1.0448483503323409, 1.0576939895896951], [1.0694195849881092, 1.0048785381699359]], [[1.0977744869960784, 1.0260719942723247], [1.0627245494006907, 1.0506406452033215]], [[1.0905632554634781, 1.0777856991850925], [1.0962243079655694, 1.0454660865067309]], [[1.0380589331468704, 1.0107454252390151], [1.0152791849549443, 1.0287457306610854]]], [[[5.6137619056972348, 20.054584576409884], [38.905853146257186, 16.956916619340536]], [[7.4456108036832029, 5.6474603621529038], [13.467373115615828, 13.677246898689702]], [[19.443626774782715, 12.058528120601395], [9.2494949696833419, 39.925302645898014]], [[1.3570255628576076, 6.8237355749775386], [6.2437271427707852, 104.5214653033327]], [[1.1517640232310749, 6.1950244166026476], [14.669109233635082, 98.901856313850104]], [[120.72035053823544, 3.2829803494178016], [11.128403165586681, 3.8895014071550373]]]])+(1.-msk_ref)*numpy.array([[[[1909.4780674924521, 5.4930003582556921], [43.935836998552624, 11.823956657085276]], [[44.451159363379006, 2.0588054936741278], [2116.7286552701257, 20.980329268003146]], [[38.978999043319362, 108.12196139665501], [271.45053059106851, 1.5013094717230386]], [[2413.2294286113397, 8.5759199250211822], [160.68221098784429, 61.843518474791061]], [[1391.9267684325268, 520.30457187897935], [2144.6576687834181, 40.950816687539707]], [[22.617184494055827, 2.4409595124858585], [3.5468879592130369, 10.6572826257975]]], [[[0.014410835402952517, 0.0006306328627832576], [0.00012373643770610435, 0.00095246403898473426]], [[0.0071991734223795199, 0.014200431841841417], [0.0016778703811674944, 0.0016153036388008213]], [[0.00068045103737144805, 0.0022013626817611637], [0.0042241589693809041, 0.000116115862193309]], [[0.4722386494307656, 0.0089200879795232561], [0.011096249973297534, 1.0908144026507551e-05]], [[0.70663900823387782, 0.011311859806862069], [0.0013599814013508958, 1.249493637952921e-05]], [[7.6554584681597736e-06, 0.053858901233999862], [0.0026814178950497172, 0.035507538929632119]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.716625229451493, 0.058138874267548032, 2.271061679070848, 4.8435868144125429, 3.6171641450959409], [4.0191863450753411, 1.4487579054355113, 1.6725864424068753, 3.2740419023779483, 0.92025107568724585], [2.4752931677038341, 0.36963437344770123, 0.7987128550187198, 0.7377402550063149, 2.5445129364837635], [1.5788907824979461, 4.607922556450613, 4.4817779051685198, 0.1540154976855502, 3.4686787832083152]])+(1.-msk_arg0)*numpy.array([[0.90775004181143404, 0.63533187006717662, 0.5206109802684783, 3.6901552228739014, 0.73561998493294878], [0.55491531268552741, 3.0534873777739682, 0.56889823133059636, 4.8012709823850281, 0.88972715624620669], [0.51870780701550789, 1.7472341634146695, 2.0752239239763739, 0.6039430149817776, 0.43940262279123499], [1.5850574566094455, 3.4183088049510237, 1.2816424084830049, 3.0260813892910234, 1.173737898110186]])
      arg1=numpy.array([[0.21268565214542171, 1.7325407594437627, 2.1551712616357732, 2.5897786213076088, 0.085459882787470104], [4.3478493167723524, 3.8131652430906087, 1.776358216921434, 2.1854546812446318, 1.2049261630682322], [1.8260260338615233, 3.4010472351165784, 1.067710618400499, 0.38463527744979531, 3.5614715975375115], [3.774110839494019, 2.6794991495398208, 0.99427096394272907, 2.6750122166896739, 4.3107896180556029]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.390822561867924, 0.0072341591120227249, 5.8577974533255253, 59.48823410542871, 1.116138483258196], [423.35199332370587, 4.1105852592129306, 2.4935502569288821, 13.356517855205054, 0.90471084685375447], [5.2332622404276403, 0.033882336305095948, 0.78664989043631472, 0.88959230779626974, 27.832325048396541], [5.6053504540274499, 59.959988742910838, 4.44342801451858, 0.0067100368893851497, 213.07560236999402]])+(1.-msk_ref)*numpy.array([[0.97962538472913774, 0.45571212253367077, 0.24492756338529112, 29.411696264212107, 0.97410152981032394], [0.077256721136796491, 70.567990308661521, 0.36715856996871665, 30.837072120042841, 0.86867684735285144], [0.30160711534572837, 6.6718701007180075, 2.1803872328237754, 0.82368910458666755, 0.053464493070354231], [5.6884250323198495, 26.936856173842749, 1.2798216987361093, 19.335770917229635, 1.9948327704786577]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.9025123215436894, 2.1544285081528489], [2.8712370628893327, 1.2703624604745005], [4.2401749376917577, 2.8821686203070356], [3.8430208373520141, 0.39003573558286259], [2.5940415617478139, 1.626902358033564]], [[1.2902944022506453, 3.7964387875837362], [0.52656116679068277, 2.0751661289992107], [1.0662621255980178, 0.39138541063802817], [4.7826959266820337, 1.134218825843017], [2.3809578143900185, 4.3801830997916182]], [[2.5012771901483353, 4.5532100620382536], [1.2356559857225686, 1.9058211334520112], [4.5157821642526663, 1.7018038646087628], [0.17374766914046905, 2.9620533620687444], [3.4627096330765319, 1.2833304212491903]], [[3.7425203201546569, 4.057443503818023], [1.2251892164695308, 4.5435624312287635], [4.1172187353412246, 3.3665187920926591], [0.98016737429390455, 1.607917701330537], [4.4321891488357572, 1.8761838022500983]]])+(1.-msk_arg0)*numpy.array([[[0.18452084634564936, 2.2297852028321765], [1.7364870169808853, 2.5192110070576779], [2.9561819238206231, 1.7438483180044948], [3.1754345086941238, 1.3944954299286954], [4.1552489427090862, 1.3588451886843576]], [[0.40306047950263907, 2.9250901450259437], [2.7172351684483087, 2.226919605864123], [2.431379088909853, 2.482618871138178], [2.63941095849255, 4.2408710376378123], [3.0552911496764894, 2.1288874456751881]], [[3.5773132513114327, 4.7260932919049461], [4.4432333695547603, 2.8566629912135317], [0.65269978633254089, 3.2770907785322243], [0.030539407038648762, 4.1702219842088173], [1.6287279723346166, 1.0419991004194908]], [[4.3119418813578312, 0.32329777356022465], [4.6723284586813145, 4.3601754251550568], [4.5483218377598291, 3.2323347511697516], [3.8444052201022219, 1.4335682682126369], [2.0933166160507595, 1.8138699328514734]]])
      arg1=numpy.array([[2.0924573768559793, 2.1113341847549578, 3.4853307474184438, 2.6116902530030486, 1.9713312870843298], [2.3915845412903369, 1.7577042436206558, 0.66720130424449187, 2.1080655438791931, 4.0907704964913547], [4.5606213580458794, 0.81104051564569457, 4.4797220573187371, 0.40331073596710715, 0.91182650248153374], [3.9608493461286964, 4.5148198551182741, 2.339051952836348, 1.3671829434325553, 0.93246941323146482]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.8413231377955652, 4.9829116249890646], [9.2712191553421395, 1.6573949220368271], [153.68800053341576, 40.019725863384686], [33.649924516434965, 0.085524657888367148], [6.5476532855776872, 2.6101382763304684]], [[1.839592602280818, 24.301195891284912], [0.32388394589159752, 3.6081549138365356], [1.0437365350491201, 0.53479246504528677], [27.089110709187658, 1.3040809413919747], [34.77000408001777, 420.91812990573698]], [[65.443318349700959, 1005.3972739314405], [1.1872239756760592, 1.6871665717019533], [857.0808462831443, 10.824563819345224], [0.49368640433162064, 1.5495245834205462], [3.1035151743810614, 1.2554109310203321]], [[186.30174221081953, 256.56442846627925], [2.5016203653464881, 929.02365107180549], [27.389906323217154, 17.104242620021196], [0.97298431663984852, 1.9142575625840994], [4.0082220593843765, 1.7981293725364438]]])+(1.-msk_ref)*numpy.array([[[0.029122681616397281, 5.3545816372711998], [3.2064665729673156, 7.034019231172226], [43.717344660976416, 6.946047062905496], [20.44346100605064, 2.3832671134344974], [16.575235752156679, 1.8302994534738595]], [[0.11381762313816698, 13.026010847937016], [5.7951845261367829, 4.0847148422874504], [1.8090118406673013, 1.8343597056947252], [7.7368542563634826, 21.024051175633812], [96.435930132313928, 21.998718738051345]], [[334.62913140196537, 1191.6522873520062], [3.3520480202935476, 2.3427185979398191], [0.14789988863688752, 203.81919311045417], [0.24486573318317936, 1.7787579439160395], [1.5601599569458748, 1.0382260411677022]], [[326.47034283309864, 0.011418530849242513], [1053.9561256143431, 771.33881930059897], [34.57387879144548, 15.551961409897745], [6.3033068584985905, 1.6362608536898862], [1.9914469891037136, 1.742377753371245]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.36698277679150265, 2.5224349932395151, 4.4307653023201761, 2.4494768100342359, 2.1700756446085903], [3.9481491312817987, 2.0896371361415742, 4.9953895209879473, 1.3159597156877079, 2.1321246514322891], [1.3607478062748242, 3.2718261770398436, 1.293144725444453, 0.67118595528285985, 1.8051255317688621], [1.912487920759256, 4.6749136946853334, 0.40360733443379276, 4.4790206978675089, 4.6271006600966622]])+(1.-msk_arg0)*numpy.array([[1.5632050241366113, 4.2789534362581927, 0.27280147612267214, 1.3749015874667749, 4.6057615100788736], [1.7674115383306679, 1.6311233619314924, 4.5771524363452043, 2.163834859224512, 1.2144080529322592], [2.7911899569660505, 0.10825700462687558, 2.7075982982394691, 0.71671611130568846, 1.9031006786744844], [1.3871967995096601, 3.8768574968808158, 0.042250892777092972, 2.0798852187811101, 1.8022954495790178]])
      arg1=numpy.array([[[4.913855296924079, 2.7543924809022129], [1.4898394296586071, 0.92258594136965344], [1.2908153368202377, 2.513036619268092], [2.5976214740682213, 1.4849701326032994], [4.2761551919279377, 0.29353024890264406]], [[0.62833499957959016, 2.4227635590235823], [4.4363916651054289, 3.7530535109330589], [4.0996924764424802, 3.5336010416306682], [3.9110648757518449, 3.9922467129967258], [4.4069924529549747, 2.3864084257067018]], [[2.6504551287091584, 0.2793142312178305], [2.8897246973134765, 2.1549843012378673], [1.8461200599018111, 2.950891914060672], [2.3722637320709761, 2.8288249717253477], [3.7881809861911879, 0.26685820549838285]], [[3.1274427168329173, 0.584428281946891], [2.9929564245984808, 4.7284150866204291], [0.91704174811635819, 0.79524878932361098], [2.7528857428720044, 3.3614517226620566], [4.0656869180324442, 0.68376410266632359]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.0072565782153845813, 0.063221288261697053], [3.9686907029237188, 2.3480831226488315], [6.8310062130243336, 42.133230806973998], [10.248616510907524, 3.7823549110024102], [27.467477292619289, 1.2553520992348095]], [[2.3699268436877579, 27.856115632987283], [26.300243798420556, 15.894363153340899], [731.00364620646565, 294.07994788806315], [2.9266145286802647, 2.9925806066328118], [28.1238393851763, 6.0908953635131002]], [[2.2624117078967134, 1.0898481704270693], [30.732792282973875, 12.863668761097973], [1.6073631639734201, 2.1352986288774751], [0.38835204580062999, 0.32371942395912467], [9.3690701206521876, 1.1707147950686185]], [[7.5977291253610701, 1.4607444500547986], [101.0655612216494, 1468.8166387263366], [0.4351590281135847, 0.48600351618438015], [62.03440022290215, 154.49714210976339], [506.91820000685453, 2.8504562223646963]]])+(1.-msk_ref)*numpy.array([[[8.9818596652659952, 3.4229145095357372], [8.7215081518807374, 3.8235181630061628], [0.18697446043709301, 0.038217488036447637], [2.2865321501649918, 1.604461188508862], [686.0881566980969, 1.5656657045022524]], [[1.4302466024390328, 3.9741177629420776], [8.763441742817049, 6.272986613615644], [510.78471674388982, 215.91385414653504], [20.468369753023694, 21.792013692365419], [2.3539342332282138, 1.5897487200612235]], [[15.189545709059601, 1.3320340353532059], [0.0016212264304185076, 0.0083036273676942284], [6.2893017096404655, 18.902067674887487], [0.45377997942193654, 0.38976465586654996], [11.445947281075432, 1.1873442813611144]], [[2.7830992692030816, 1.2107919163771204], [57.715772279304673, 606.14558063865911], [0.054933104376238023, 0.080760250124788255], [7.5080307600708052, 11.723950689566896], [10.96751824106779, 1.4959759990091266]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.2325922765537447, 4.3447737465099356, 3.1179813328568176, 3.752621339238257, 1.7659760299891885], [0.60978737661071591, 2.6692270044943158, 2.4044389146935741, 1.2476941179760417, 1.7443638639557828], [1.8945618268766293, 1.5445747838536972, 1.9519032193352384, 0.63530861727506405, 2.518534454758504], [0.026942436297970399, 0.30025907786090261, 0.69555316168147352, 3.5468043713970037, 2.7825015758691576]], [[3.6075402951777096, 0.93569224890039682, 0.10185061450998867, 2.4497684003872888, 0.2204227130796326], [1.8181416512979509, 1.6936200522545763, 2.1337931990926933, 0.057970883054462644, 4.8817791196487583], [1.2709985926884038, 0.94313542214717883, 3.2807220484056847, 2.1530045933057544, 3.1210317482524119], [3.6639467963676813, 0.45858216465886492, 0.43056751560367429, 1.8011636285996449, 2.0144185538553656]], [[3.4858812206661556, 1.5000674778965544, 4.6660447723108422, 2.8037475335184201, 2.5890929098764293], [1.6363778125505002, 2.9459273831440931, 2.8831128020540597, 2.9239956087303951, 1.4177298255026589], [3.7626956128227995, 2.5196299159064539, 2.6171426687931953, 3.2863992846201668, 1.2282127870217903], [3.4297077884859593, 4.0911867209389872, 3.5272950631830633, 0.81464029736071331, 2.8166631312806225]], [[2.3598961663737228, 1.7658756850402653, 2.9358838755342873, 4.4631980034144449, 1.3475270720550936], [3.9799660078018224, 2.3062219689724275, 2.6990160153375222, 4.7599139570127624, 0.64568299709613475], [3.0117701839506728, 4.6772278527491258, 1.9252598921565469, 4.646308032534864, 2.1906815398585109], [1.5813406504997596, 3.2831890959606871, 0.52586388716874355, 0.072917868918089615, 1.7333144232561959]], [[1.3122088445247675, 2.7753413956542019, 2.4045970769802607, 2.0495483685160982, 4.9911888473708563], [2.2754167419101008, 1.7682945872192362, 2.3656347392146411, 4.0028993155691319, 4.8692024568534915], [1.0787273366503534, 0.98788754281816205, 2.6274663121940325, 3.8955711877583732, 3.4008042065481519], [4.2993226803162701, 4.6569654896421717, 0.82493229326354356, 0.22556186105186141, 4.2705729287155849]]], [[[1.874335468524599, 3.6795100409513548, 1.3213443055308358, 1.0838796519970866, 0.88616736589081035], [1.783756050659576, 2.9282017494172581, 0.7674804219704916, 0.12363608270722364, 2.8172171799284893], [2.0028545316027118, 1.6648693187106467, 3.7416692944981418, 0.85067890796140877, 3.6955490989128856], [4.6330819533297394, 0.92705483063757033, 0.16516402423344373, 0.24213551499847719, 4.4442590278814471]], [[4.4283372181428611, 3.1468289921008199, 1.3212318194758756, 2.9402920282919744, 3.303166748663465], [4.8473128083860848, 0.73180912790911512, 3.0609078971300581, 2.9629624638875711, 0.010347919246793865], [3.479557285693279, 2.252362397408501, 4.362892967545382, 3.4550346008128181, 1.7063922500974709], [2.1390905924439796, 3.6695114709280592, 3.713951675134477, 4.4323108243189955, 1.0843740740566934]], [[2.1733878653051226, 3.9736787115954368, 3.9194283214160843, 2.2267027238419344, 0.49176247553687918], [1.6930779219645746, 0.081078419211249295, 0.8814513484629326, 1.572070844393582, 4.8418183159457087], [1.0326251201913144, 1.631380493581553, 2.8334260050530253, 2.6421086930289048, 2.5795140915741195], [1.389545258256045, 1.6335183468519554, 3.7238411146791668, 4.6738199504128133, 4.4273638841217977]], [[1.307969493798564, 3.3777307202104283, 3.5669179184452151, 0.33642776695586607, 1.7844844168567047], [1.6249759958447176, 2.3002948789964472, 1.5834967645983571, 4.011805110886483, 2.1576786613030143], [0.82051445431386894, 1.4676297802958596, 2.0925985639313618, 3.2931324736370677, 0.48532956785129372], [0.26743468957578803, 0.68981840950464424, 2.5332906670521695, 3.115233879894673, 3.4980062490668247]], [[2.8162810957333999, 2.8461892165501572, 1.7269038158965162, 2.9565895780719251, 0.48105842771945084], [1.9422970958785979, 0.7085230461476546, 1.988521347459175, 3.445985980461074, 1.4055473299637664], [4.3645520294751057, 0.89771407272438108, 1.949074558333828, 3.7719699205339903, 2.026479958468824], [2.9205291861852434, 0.34924077590474595, 2.5218652258731589, 4.3641128443443806, 1.9192394630602754]]], [[[4.3689974750289968, 4.7659432708015723, 3.2501839100628525, 2.3372332962303366, 3.2646356975013497], [1.9428830166432753, 2.6307725998420213, 2.7601853543640962, 0.26736801916720165, 2.6299094034262898], [0.50375395012715041, 0.42908419875691756, 4.9000120472929067, 0.13348068551363967, 3.2623311875423795], [4.2069331527758296, 1.3648807567369747, 3.8944495978582161, 4.0667101210311225, 2.9480055827906053]], [[4.5645919365900092, 0.92379989877611435, 3.1972927383978877, 1.7983803021374056, 2.1562475272559558], [0.96797021351388024, 4.2914667312251815, 1.4786144740829013, 2.3416498291240937, 1.7185262090192677], [3.118696009507592, 2.0776245786543726, 4.0731925804099083, 1.5689230145167241, 3.1716920590081408], [3.142029650326279, 3.9318748068231306, 2.7880962873348327, 1.0895930307181783, 1.8442283646601845]], [[0.75318779978376227, 1.5775061844340117, 2.2009332055648203, 4.4191084185055685, 3.9709214927249619], [3.1675859036766867, 3.5196012015642615, 0.63746095365896505, 2.2289351629253975, 1.0121471354805678], [2.5409293815164209, 4.4576463966127351, 1.9929807193360214, 4.9744177905899649, 4.2914110085118242], [3.3738232622791493, 4.537830923730283, 3.7006854986700923, 4.6347718120835619, 4.5614760728773494]], [[0.39354814504278796, 3.4171575108995538, 0.12602615019352656, 4.4173393236719098, 4.7658849833922057], [3.6196453304821614, 2.3127222075846423, 3.9046209543840757, 2.0202526124102076, 0.35581257531919741], [4.9487670837218536, 0.78168192584203278, 0.44642759070519406, 4.5681057737570381, 2.4914423172007645], [2.8170170245598047, 4.5173194404839139, 0.81353508456010104, 2.2071974004564225, 4.1688761044567215]], [[0.39683900754284185, 3.8775494792796215, 1.1146985789475237, 2.444662525949135, 2.3870003284600991], [3.7402349184567245, 0.2162515395719837, 1.7857021515347566, 0.25706989130845037, 4.8015234264149846], [0.93237792518213514, 2.4270105138556008, 3.4309372867786014, 4.3513283406383083, 0.55357513969609951], [1.7376933454803833, 0.9140921745606656, 1.3872288839771978, 3.1097284268509187, 3.5276800284343999]]], [[[1.5308268805253762, 0.70469561561798189, 2.0580731425615273, 4.3173731916596676, 3.6053403478527062], [1.9312338012020061, 4.4830730053715353, 1.1043264769396519, 4.612564277590133, 1.0824295075683359], [2.6221278292199113, 0.34453260542348446, 4.5747097968194899, 4.4116236848625903, 0.62971973517351942], [0.65432461190208435, 0.55994248910081901, 3.8136420952551489, 0.19680084252961272, 3.5776831286743329]], [[1.2034542294225135, 0.26920737597448602, 0.65457140937136948, 1.1639764522913632, 3.1622245795374613], [3.483676255928065, 1.6043137432373247, 3.5656466749211453, 0.92835160101278069, 1.5807482822861783], [2.3811140104657489, 0.3098056624971291, 4.6707494029138745, 3.8306983254610181, 1.6512570370055071], [0.72224075035598656, 4.2382729399431982, 3.0548235585668677, 1.6889365132763867, 0.01138176652121823]], [[4.7682828701253603, 4.2477224842478458, 4.201985457732861, 4.2831531970027745, 4.6104403547682331], [1.0995402320756635, 0.88328183260275783, 3.9823196101706988, 2.8762000362673281, 4.0500902563594092], [4.274825681957676, 2.1205837781098493, 3.4712963066011806, 4.1373622933894332, 3.3571383299713933], [2.8809825330626606, 1.7933110147697671, 3.5828595061373063, 4.5394264260848738, 4.4230905253769119]], [[2.4514103390002746, 3.320132172655172, 0.25996664323217306, 2.9162644892109992, 2.238760666605685], [0.72459965615424615, 4.8521655001074215, 2.8293839588117309, 2.8197805309144051, 2.2870201768960081], [2.6242709242692484, 0.34948415804521593, 0.31833704299926768, 1.175390636105085, 4.8318157324594422], [2.6779361719365276, 2.8309124164187538, 4.3952371650795534, 0.6098195722933698, 4.7353263533801533]], [[3.7282124664834768, 4.0325020057590333, 0.82769231314730041, 4.8430029850927419, 3.4364947950549971], [2.7162200656243254, 3.0429675175979907, 2.5473171623496258, 3.4518958274413829, 4.8496227760885997], [4.8287130572901136, 4.9559005330769441, 1.6477336124406676, 1.9084101237548865, 3.898453025984264], [0.21375836975416729, 4.343711063450824, 3.1251547427283679, 3.9032378304463919, 4.7503468734726013]]]])+(1.-msk_arg0)*numpy.array([[[[4.5031225013836229, 0.47198449704916157, 2.7614798592017507, 3.2406397366477719, 2.7503809001598549], [4.8042376680273167, 3.9657944082423686, 4.2276618427981099, 0.51397374954194575, 4.9965087277593589], [4.3508288049139017, 3.8337728408831975, 3.5964356850585943, 4.951915285345919, 2.389412362580575], [0.99602843722470857, 0.1185561764048437, 0.84860211514447881, 0.66193944233766433, 1.5841599673727547]], [[1.8857474182314311, 1.1170120964573256, 3.4852748289182309, 0.8236510902011086, 4.2787890645363307], [2.4455947069444779, 0.77969099875205727, 3.1002642392858393, 2.9703395684154472, 3.8328466179517164], [0.11786954241979335, 3.6059759733165619, 0.75217450465879365, 4.6183331199078266, 0.011433417390695887], [1.6563225863422293, 0.63026835516181046, 2.5771287792836386, 3.0815643165782727, 3.8213612739363341]], [[4.9215292658114604, 4.4822776828403219, 1.5708919959563779, 4.5955500343272959, 3.3832897115057348], [2.208997976984425, 4.3675139839127786, 2.6307068423257709, 2.3621189602320651, 3.2023444344412892], [2.515223449913365, 4.3811423471964961, 0.22369656802809623, 2.2074051174999236, 4.2438305333397501], [1.3329525091975372, 4.6572020211648741, 3.3969679041106979, 1.3141994125675709, 3.563352985510003]], [[0.11658855083911367, 2.6971488628347022, 2.8888289807343663, 1.1802643627174347, 4.9578041974955678], [1.6754809890326707, 3.7282552761880616, 1.1466218902705188, 1.8939111575558258, 2.5346008909364652], [4.9279587732609311, 1.9493785831981181, 2.3980557468642352, 1.5574876821141581, 1.2909903449310147], [0.64392729085082789, 3.1665389571126319, 2.4204101095341333, 4.9836996020856326, 4.1603486640419316]], [[4.2903849815054835, 3.3669885946459899, 2.6273937152234024, 3.4187434153862841, 1.892540993203669], [1.5620658233834903, 3.3730349339944978, 1.6744986052537325, 4.6943927147411477, 0.15400411927376259], [4.2175701340093168, 2.056943819070963, 1.8401989265209173, 2.9516679104680117, 2.9440079451872578], [2.2297690776100172, 4.3937979003827863, 0.93403965295505731, 4.0097181933949191, 1.7035262822182973]]], [[[0.33022189601035196, 3.7620975277265782, 1.1435484827221447, 2.6206927832113665, 1.2050094131147704], [2.3326082593029822, 2.9263346374955908, 0.23839604038065521, 3.484243224984283, 1.1652268139465214], [0.74481068530828232, 2.3685351569468214, 2.8314241534791922, 4.0641617633811871, 0.3278532324249], [1.5274800969790345, 2.0141814610521624, 1.887643349742093, 0.14874702928667274, 4.090971928027197]], [[0.48215375171386943, 0.02024550958730497, 0.7691578600542972, 1.240310428715824, 1.7841646965984634], [2.7574273342664806, 4.5097157029158312, 3.9541101808067523, 3.4453266661914164, 2.0854067489682606], [1.552894744444387, 0.19529778035721326, 0.27617664423311611, 2.4705321060746717, 0.8423784079070068], [0.15747764854114427, 1.2746715552928594, 2.5336928995954775, 0.44950878321935928, 0.37084602208101203]], [[2.2468280247795627, 2.400949377282211, 2.2732018899289916, 4.3174310201502299, 4.3531727748434488], [0.26903264773723723, 1.5669793132425089, 3.0175150532109769, 2.0400959581383242, 0.35182246538657158], [0.99823231784870403, 3.8711892558989032, 1.2731410397743486, 3.9043031465924085, 1.6269067778715935], [3.2466439097711586, 2.3349721777155668, 0.35730971016442298, 1.4636877493052332, 0.46277255224239167]], [[0.34446783020024152, 0.71206245975062987, 4.3206773509058118, 4.7683989281916705, 3.4753899769392826], [1.1989150076055344, 2.5413671711395334, 0.54056376229202463, 4.5497266100940497, 1.8913048116348008], [2.0449190002293127, 1.3770480139987735, 2.0523895209812548, 4.3921403172799529, 0.47758298679617822], [0.31674832198242836, 0.80129761120245824, 4.6316177212348162, 3.3417924378720576, 1.1941657187994326]], [[3.8639442703631461, 2.3835951309344856, 2.7165347075699962, 4.2796548704405284, 2.8860894042454928], [2.2844111293946381, 1.1285782635544299, 2.2578886307743353, 0.28908145103772792, 1.5052978916880235], [1.6393486380388242, 4.7392674470273759, 3.8510769084073218, 1.1186734221048644, 2.4005198478733849], [2.9428342810207959, 0.87566104238567544, 4.1753925233611353, 1.2192443145475718, 2.8443247628446602]]], [[[2.1206153344256293, 0.76164275437950579, 3.7914286836190865, 0.83184726601133974, 2.9255560034889676], [0.056048672242224831, 2.6605092153974947, 2.8752917090112664, 1.8800360123797693, 3.2445549484619036], [0.98917989801353079, 1.9704763794186375, 0.55792967182107711, 1.9945009235471309, 1.8689713077232992], [4.5455260225272109, 1.6647799077523244, 0.095711699031395661, 3.0055626566430766, 4.6560910979847971]], [[1.3274290359425254, 2.5551661573706741, 2.7060005682928234, 1.4648625830299644, 1.5346434735355836], [1.8853348091664308, 4.7237846297263939, 3.8114390120302928, 2.1356489076256246, 1.255951865317696], [0.11509338319455166, 3.6903457176415002, 0.53492427748732985, 1.9170186305845311, 3.8840613185908879], [4.6373939015870151, 2.2018244131462263, 1.7210813004934336, 2.4873498013345352, 1.5135606515626561]], [[4.9847152144764602, 4.163070751038866, 4.2105030082026875, 4.2009800707180709, 2.5695512530649385], [2.1305878223509129, 2.8384947558853817, 2.359265172698902, 4.1081013593350271, 4.1837681512202813], [3.6586591793951091, 4.4458324312776387, 3.8812342221670013, 3.2955260776715689, 1.4685274618097872], [3.6616812335931379, 3.3615877869204276, 4.2924713654062225, 0.28619930701757573, 4.4660518078553366]], [[4.3543652264925479, 3.8250936874071599, 4.7346274528622683, 4.9540090955944907, 3.7807098079039094], [3.9451020370410896, 1.4254013168584432, 1.3381755888325451, 0.88549741489760292, 1.0689263333238703], [1.6247048917342881, 2.9522415693632982, 1.3050684468109615, 1.9001545629008412, 3.7403661015226652], [0.67190177082529989, 3.1030968574603222, 1.0838934565918024, 1.7896566424891356, 4.9506525859838133]], [[3.3013574673521386, 2.3912525520863759, 3.1003113758211986, 3.9905449532279009, 0.10410194359468884], [4.5731503440796049, 4.3303721128957671, 0.18816308935468271, 2.2738498233608886, 1.0585377051487666], [0.10200790342417153, 4.3497693150284569, 1.6450593275508631, 0.99653204757782032, 0.60157720313248986], [0.95475091677400226, 1.3565637710427962, 1.7274320945356187, 2.8671847545164857, 0.44971317425780577]]], [[[4.4418500734642477, 4.696764369019526, 0.50664429949109269, 2.8995452660128516, 1.5065568013787458], [1.3609457390292721, 3.4467690277883656, 1.4059175972981819, 1.3564453931685141, 4.9312835043085652], [4.7424704850219648, 1.7202591181203262, 4.0365388341456745, 0.48913433920822225, 1.450947934780257], [1.9833291019461197, 0.30888263782088649, 2.106903432016153, 3.4066631931834421, 0.95582160456171472]], [[1.3792925523768409, 4.8350567321051674, 3.8506459618979716, 1.9942661253232647, 3.0212514441172544], [0.88410608139314473, 1.1461533152092342, 3.2207442997054936, 4.4003865437751548, 4.8118930964513833], [4.0582620996419916, 4.0254391973614387, 2.6679237383352632, 3.6548883517878092, 0.96756751189296908], [1.582287913770791, 0.80394447667037638, 0.040249505806420484, 1.5145099955677008, 2.5224946454834809]], [[4.7638948566034749, 2.2645236256109782, 4.0705963316624354, 0.62321129387289909, 1.8245570679687875], [2.6312439082562116, 3.7147327468706903, 2.6404582190025447, 2.1793018655034539, 0.046657080183546286], [1.4047274338433116, 0.2303785328246156, 4.2777619689176953, 1.8244428783257711, 3.3506416523756948], [3.5492829881355319, 1.1747768273106745, 3.2132184613232089, 0.98040405340685033, 4.7695370534855419]], [[3.8245913678014682, 1.776632339298885, 4.6466702487773901, 0.38207668458900529, 1.1804269484139058], [3.7199633835477171, 4.7820194931239497, 3.6816612947705027, 3.412520498003464, 3.8566403022351716], [0.53946246232378214, 2.6972158088472753, 4.8306593028029443, 0.64665094714744131, 3.9578813129990706], [2.3623601159545835, 1.8099433555055684, 1.951721695796357, 3.4886139963782412, 2.5491671098910218]], [[0.98399124829105178, 3.8768944866199564, 1.6780386744351281, 2.4731957092831016, 0.30708981497816001], [4.4249761094747688, 4.5552820226856348, 1.0123048250825575, 2.5125326002757919, 3.2346499839470235], [2.9139801570577299, 0.54527976000665734, 3.5322983338667, 3.0693816807296566, 0.84527671189511122], [3.1542671425458595, 4.9592679619335538, 4.5889786155421932, 0.71225267584056928, 1.8564812580018322]]]])
      arg1=numpy.array([[3.8004722880401887, 0.27089595070535805, 4.6669343670715442, 1.191704597597032, 2.563677693480269], [2.7371389619139412, 1.5111355874731676, 1.7434267987152365, 4.7930757152842647, 2.8790554372239758], [1.7062437504442416, 0.55371964371283067, 0.44211872389597145, 3.4165956275427316, 4.9581488716874791], [1.4125422431493018, 2.6299459118155797, 2.6753548017254434, 3.0957420962763282, 4.0048093758995469]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.2138937434523638, 265.81319363713635, 75.32733302984758, 152.31521156792348, 8.6827821561409095], [0.15260774499767996, 41.731722859786068, 28.056415940676896, 2.3187625414916715, 8.285811161346242], [11.341341762625438, 5.2187097807644927, 12.702191190543534, 0.17834009238099857, 33.461966832279067], [1.0837242711109625e-06, 0.010333289201547991, 0.25164064226214938, 122.92527110144736, 48.872493060364313]], [[1.4156185837101309, 0.98215505272124459, 0.53859382613736417, 1.2747129400470056, 0.66388186114222381], [1.1757963209969333, 1.1534142810500445, 1.2279085755768153, 0.46233691878473138, 1.5365016322142184], [1.0671180788146433, 0.98426539149951753, 1.3796662205940347, 1.2308936467449867, 1.3611418187996525], [1.4215807812888428, 0.8096163679134295, 0.79590871385544892, 1.1728117774997737, 1.208907092735503]], [[339.57619620369343, 6.635864719248767, 1324.1604708578964, 122.90458397107824, 84.748671378514857], [9.9581953339239782, 154.82049502523378, 140.00516775605649, 149.51430638783401, 5.0989036161711647], [485.08346097006734, 74.646989952151245, 89.119562027431115, 257.92880974254018, 2.6099614925378822], [314.78196858529503, 716.90321927397042, 358.81856597163653, 0.3841347962800068, 125.56925090421979]], [[2.7821348544549251, 1.9692620805278294, 3.6091635802150472, 5.9454715964580549, 1.4268240897150752], [5.1865550842649597, 2.7068919165233982, 3.264897347503561, 6.4194521417297912, 0.59374393467265485], [3.7206102285894129, 6.2867821040417473, 2.1828670356753368, 6.2372861516762592, 2.546067166750448], [1.7265514564758766, 4.1235588197241348, 0.46490421615998845, 0.04414017096224241, 1.9260663211402582]], [[2.0068807073311508, 13.693687108797338, 9.4813391753043312, 6.2949328746415771, 61.655244731311647], [8.2297982848767095, 4.3117170275376111, 9.0924589543337664, 35.018254572535412, 57.865579975383397], [1.2144368917706965, 0.96924095982174807, 11.900303830075133, 32.661345445453826, 23.057061869271276], [42.0565417252635, 51.61795662336538, 0.61055314565786056, 0.021977668705046954, 41.339313490923004]]], [[[5.5824011282400399, 35.370877361792893, 2.144063292454788, 1.2466600173058702, 0.71836195148715309], [4.8746122790664925, 18.930050178530145, 0.48463329148800427, 0.0032739749695419864, 17.030239124573445], [6.6935474409877367, 4.0359728283153338, 37.030504934862833, 0.64233097422515073, 35.794495434602553], [66.462805925206709, 0.81276120744293368, 0.0072331242088582316, 0.020610214193655482, 59.308491337217738]], [[9.4745217521837564, 5.6539742209713246, 1.5234061658194165, 5.1027172344387308, 6.0837917981987077], [10.861380463221543, 0.62385940020596853, 5.4223259534486612, 5.1622873164334671, 0.0010004001971308449], [6.5813679829552818, 3.4110197714122581, 9.2637345887278428, 6.5114031508299055, 2.2423460689200576], [3.1551576390485843, 7.1318076105243788, 7.2627289145943301, 9.4873717908932367, 1.1302133391366951]], [[3.8705625964704717, 11.082779619402517, 10.820327328762975, 4.0376034868190329, 0.29013363555385008], [2.5042721095295875, 0.012524379319481065, 0.80252270563444794, 2.2005677686095901, 15.640915041537898], [1.0575673924715869, 2.3473321412378279, 6.1457182355997491, 5.4405163927378819, 5.2177853824698683], [1.774549376356541, 2.3526976680324894, 9.8964915893418581, 14.706999338331233, 13.381564413706434]], [[3.6212724156776352, 341.77458435365145, 443.79532160622267, 0.0053995396401777772, 16.051650722043689], [10.247204314569627, 54.207049398105163, 9.0527444226275353, 779.56704973336571, 39.88630997776896], [0.38744389712881283, 6.2893472641719939, 34.440695216571505, 302.64959276239557, 0.031271619599639658], [0.0017972614500148681, 0.16867223213719496, 86.078141373423009, 231.91993769730641, 404.17851158308383]], [[19.707980901763779, 20.316578650207287, 4.8206783486042033, 22.669059812980699, 0.12162698111635191], [6.7620249816473796, 0.37081800723278957, 7.2357767937106043, 35.233466307093508, 2.6647429211816478], [69.56996757556783, 0.73296257473204252, 6.8301805224435226, 45.705858529548216, 7.6406116104943207], [21.882130733118391, 0.048376159454048107, 14.341003548048464, 69.549814621167258, 6.5334799361303268]]], [[[12.377913215631525, 14.357779155898383, 7.4720490927620604, 4.2569210290446158, 7.5288264915313059], [3.1057056868169273, 5.2091219196271634, 5.6539000736988578, 0.10531957403622365, 5.2062059594377699], [0.31039314138048313, 0.23606302830613687, 15.053745516129199, 0.032192200147830648, 7.5197607371368376], [11.604793708145456, 1.7002175938285984, 10.172904016618759, 10.952605137426106, 6.3260079056318581]], [[2.3180554349640148, 0.95706142657616466, 1.9033029967984221, 1.3839898780570639, 1.5302961226179954], [0.98213572339308974, 2.2401970992790687, 1.2418010540762823, 1.6018120746475177, 1.3496171398779735], [1.8772520110475357, 1.4991432280847536, 2.1763711480965457, 1.2832417088703334, 1.8948493283804266], [1.8850162504612744, 2.1342314905339865, 1.7643130997453733, 1.0486582652275571, 1.4034172795727213]], [[0.88221946109879079, 1.2232823710129335, 1.4173362959557472, 1.928921415971198, 1.8398442409734246], [1.6648741183321984, 1.7442751353103123, 0.81949304090829889, 1.4252806661474535, 1.0053523929951762], [1.5102707431275231, 1.9363405760309713, 1.3564879093202993, 2.0325573013275089, 1.9040763279387776], [1.7119566758049658, 1.951663485356316, 1.7833976559825384, 1.9699881205122185, 1.9561530848921302]], [[0.041330579304833019, 66.575969220741683, 0.00084457429149437503, 160.04917379897734, 207.46379231460861], [81.046800701117249, 17.541335608480363, 104.99915810603338, 11.0521736729083, 0.029289005948224842], [235.94754415034282, 0.4310493611379711, 0.063582982023087264, 179.49470850107411, 22.621020594022539], [34.415076725916364, 172.76783330683409, 0.49407401595105321, 14.954305839355882, 131.32704277388152]], [[0.010229872391313894, 828.23761893158189, 1.7132220091943906, 84.109911670918351, 74.722060840516832], [692.65653754275422, 0.00050423007728407903, 17.721750493079433, 0.0011883547104149812, 2389.8947675292538], [0.7066958212143829, 81.141420272896198, 451.49845644279998, 1466.8382673613194, 0.053288223424822354], [15.481838876493056, 0.64059355947389729, 5.0674742996407698, 277.3264722929016, 518.24234012783575]]], [[[1.8248026346447752, 0.60995219144200452, 2.7718940674717709, 7.8935889135921622, 6.1194181728986088], [2.5336919165094716, 8.3248872073637212, 1.150474554974241, 8.6665596626940804, 1.1183839359518621], [3.9027098294627036, 0.22198227091533584, 8.5662631019116819, 8.138091231153977, 0.52033984403339106], [0.54928810327141231, 0.44080036607496392, 6.6247152020432125, 0.10064292473611978, 6.0532140502144376]], [[1.6275166441058797, 0.031707318630369311, 0.32807787927796955, 1.4908352903122486, 20.651603762886424], [26.639745150969844, 3.4665683331291137, 28.320041204234709, 0.82240472527168218, 3.3342502598667756], [9.7929045483222819, 0.045876736907520844, 57.603711310464256, 34.197114688531606, 3.7397350174062742], [0.42495336606775203, 44.614182514739767, 18.857637932561573, 3.9683569823909077, 7.7256774954084187e-06]], [[65.291863519838543, 47.922617714621502, 46.554547610713925, 48.999517421732641, 59.668681064273784], [1.2890043428858735, 0.71745802537532566, 40.325246420304815, 16.885108227728107, 42.187482134980421], [48.745058651054876, 7.4710925293637072, 27.925890323372041, 44.663660788137271, 25.536085436825694], [16.960326881799489, 4.7711027570327982, 30.392141284147556, 57.241463742665687, 53.400545619633164]], [[16.052088510598516, 41.054675817091507, 0.015443149171252423, 27.477943829667932, 12.120861033568017], [0.3688925393247694, 132.88615317739115, 25.021975183914261, 24.759990872053674, 12.948135957757092], [19.821785599792509, 0.038598397441726807, 0.028911252710557001, 1.6491724573456408, 131.16840903246825], [21.103726721465616, 25.063844290331975, 97.837966547082644, 0.21629117748178628, 123.22793278093872]], [[194.42458416859921, 266.20168746580623, 0.46890049187823668, 554.31154145596417, 140.29466090971479], [54.694910153866417, 86.201262067761647, 42.294616599776383, 142.82967415773285, 557.35212596484087], [547.79037402227641, 607.90180246316572, 7.3890923644662729, 13.305668610005352, 232.49362319516038], [0.002072382245497573, 358.51838293854064, 95.910494199220395, 233.63851787424505, 513.04554344157634]]]])+(1.-msk_ref)*numpy.array([[[[304.55163860163788, 0.057646293526300518, 47.484021202666064, 87.224407133770995, 46.762780362897757], [389.48878979396829, 187.90456423034902, 239.59437619194449, 0.079696463021355995, 452.12991537844562], [267.223822125818, 165.21743301281697, 129.59172330165313, 436.98478651368919, 27.395855479844862], [0.98498992441753386, 0.00030232532314380494, 0.53584840887168572, 0.20846097352547613, 5.7455378861990738]], [[1.1874829167603553, 1.0304304503281434, 1.4024578435890007, 0.94880113399528865, 1.4825941058626815], [1.2741242586063193, 0.9348076298760517, 1.3586823100437742, 1.3430162209733081, 1.4390424168253533], [0.56033332588738394, 1.4154522683513475, 0.9257531786703902, 1.5135834311852341, 0.29782953071792634], [1.1464773326005173, 0.88245439491398348, 1.2923350117226378, 1.356457367643995, 1.4378729883586134]], [[1698.1873887588833, 1097.7462051058394, 8.230041665490953, 1233.3480062191479, 295.38691196627485], [40.395869378245528, 972.59375615898944, 91.295752786209505, 55.229974061240064, 228.55260427872756], [74.039687489913874, 986.83857757075577, 0.00092236483168324653, 40.26010788070019, 850.56114386289971], [3.8238592954338784, 1312.4896416948402, 301.00167320187177, 3.5791867010443927, 376.26083296882035]], [[0.077219956008650947, 3.2622059138950883, 3.5403347324115177, 1.2183667987744971, 6.7387530264323088], [1.849728882789164, 4.7980624919478227, 1.1770945792692471, 2.1405763044776078, 3.0292903913064011], [6.6904376393078797, 2.2154942280261323, 2.835829031273851, 1.6955605562747329, 1.3557744407027759], [0.59182045759965751, 3.9495648564829247, 2.8673600923500517, 6.7807191210595512, 5.4678896886282971]], [[41.832764190954897, 22.473858649823075, 11.899460896341079, 23.370158391479567, 5.1316241884760236], [3.1374921862061478, 22.577468601500431, 3.7494575616207357, 52.688177176265036, 0.0082621823681525985], [40.036703279594668, 6.3533291704422545, 4.7755996355423802, 16.036203112063134, 15.929729339193822], [7.813149014496882, 44.466686221878376, 0.83951066177960454, 35.171389542716327, 3.9183556855522457]]], [[[0.048184215144014914, 37.586510621805282, 1.4436133383840311, 13.972143729259843, 1.6660271996657381], [10.158606719224251, 18.897030152200177, 0.019750629154929544, 30.466754921114493, 1.519758863023398], [0.44644898876524053, 10.592618914553835, 17.266340919998733, 46.434226054209162, 0.04724408116853597], [3.1883550196742747, 6.7976705208695902, 5.6915587693517402, 0.0054309524638065654, 47.277461481053912]], [[0.33208588034334935, 0.0027582462990913786, 0.67259547181859058, 1.3846407504254652, 2.3985693973234765], [4.63085717097628, 9.7388590823454599, 7.9840204162564943, 6.4837757300796452, 3.0362710509301181], [1.9446502729452042, 0.084751512771688242, 0.14307300893938024, 3.9224706827140472, 0.77166947068694747], [0.061219382259114552, 1.4430158908262174, 4.0749942403818702, 0.29870349820562525, 0.22335370205056976]], [[4.1014390977352297, 4.6043668843277583, 4.185740125370188, 12.807636664737341, 12.993056677038462], [0.10136936636365072, 2.1881572063758319, 6.8585719177003641, 3.466196235818396, 0.16182601877152938], [0.99692020085002486, 10.589213069447855, 1.5235072729864363, 10.747633395965217, 2.3361210379312376], [7.7919990087889808, 4.3860361100537499, 0.16625180111762283, 1.9428856492828424, 0.260971386441057]], [[0.006046701470016326, 0.19638487633484503, 1112.366938818247, 1784.39561446472, 391.80588052769087], [2.3858259077726549, 87.401481107790403, 0.052422182878893504, 1424.8638466803222, 21.209916369345805], [30.83856201647815, 4.6343995829795261, 31.382302874542802, 1203.3604031972104, 0.028950550352760094], [0.0040446965608582726, 0.34584112032008019, 1552.0555833783628, 324.69326640711955, 2.3408655000054885]], [[48.988525956457877, 12.191952171731678, 17.764540921686876, 65.744705811084827, 21.147419284781698], [10.787753272115092, 1.4165816873662695, 10.431077066087855, 0.028070319189960939, 3.2462718823459671], [4.1500243334019853, 88.1881140148881, 48.520312951355891, 1.3810822970120031, 12.442855138553574], [22.366742811507759, 0.68231077509924398, 61.238136003654972, 1.7695384774923033, 20.278285498152808]]], [[[3.6059821484369379, 0.62840397004826987, 9.7180429944317641, 0.73042374197871507, 6.244033098024369], [0.0073239844462168399, 5.3099872564344039, 6.0621006935814528, 2.9362586641597703, 7.4499824932398733], [0.981608882459304, 3.1813417561968973, 0.36949008178986631, 3.2478076553654156, 2.9068344408603926], [13.243380087843965, 2.3861003284809557, 0.018250633487932353, 6.5381952488813653, 13.79772409529857]], [[1.1698058273487841, 1.6811094317827702, 1.7353554401669917, 1.2353925915789845, 1.2676402045432424], [1.4206530052296205, 2.3624774274362017, 2.0977821232121423, 1.5221840099608006, 1.1344968429501392], [0.30205438014808356, 2.0606117257553676, 0.70721280251236174, 1.4338236936668034, 2.1198214234646784], [2.3384548786570516, 1.5481231500193882, 1.3507278655383381, 1.6562552811336004, 1.2579675577030123]], [[2.0344164643004916, 1.8786870215798517, 1.8881206558161605, 1.8862314475932951, 1.5177686555614567], [1.3971265746014947, 1.5860567804811716, 1.4615429553732684, 1.8676790288411289, 1.8828107893749468], [1.7744149334423132, 1.9340700205682961, 1.821354987796755, 1.6942762798451145, 1.185172835765657], [1.7750627832784938, 1.7092089706007978, 1.9042843192942114, 0.57515249089021181, 1.9379539897330433]], [[152.38697898115288, 97.870627261932484, 202.851653658044, 236.80254353625276, 94.044751126422724], [108.76520377778317, 3.3569201661004469, 2.7054786748817494, 0.66002494881461093, 1.2557490131037901], [5.2496602123171678, 40.394189201232287, 2.483545905875169, 8.9641483559163451, 90.660028517475411], [0.25702401442527611, 47.892309369178534, 1.3168461723617668, 7.3048601996139855, 236.25482748742238]], [[373.03956024654929, 75.384373618415268, 273.18742357876408, 955.00833354811834, 1.3440475879223399e-05], [1876.9237653933953, 1432.1443298781485, 0.00025294944277384697, 58.732435230224091, 1.3258600142274819], [1.2152311716376118e-05, 1464.2343617387758, 11.799412459939674, 0.98292298522752153, 0.080481072712525692], [0.79486256344268202, 4.5358459863782281, 15.033822631130402, 185.41008688128622, 0.01901967483795448]]], [[[8.2169634982062956, 8.890867649536851, 0.38271963167152667, 4.4984252088186105, 1.7840706644163613], [1.5454529870093578, 5.7427169191480774, 1.6180784065749338, 1.538239158701916, 9.5243474950425497], [9.0133265546522559, 2.1517226499026845, 7.1781660909610912, 0.36416998374008963, 1.6917650032974703], [2.6307688805339016, 0.19024434770041126, 2.8652446643222342, 5.6485565344020809, 0.93816982729251841]], [[2.3296396810765194, 63.086890542287222, 34.667431980456698, 6.1434647514243288, 18.317470395136677], [0.72328491489464808, 1.4315454334040374, 21.671923285469266, 49.243177898099191, 62.295130011863257], [39.801692342604561, 38.960649654139218, 13.207268915752827, 30.222367146086647, 0.91694341192479656], [3.3427978510252712, 0.56331284316294772, 0.00021408967292550852, 2.9792434145543201, 11.397033667312467]], [[65.131239108055667, 8.9061301253642142, 42.761364643005251, 0.28221273264455177, 4.9967640497873695], [13.30699632802753, 33.47792040799056, 13.43203267770174, 8.0374642732900217, 0.00027471488690727029], [2.4823392682270047, 0.019692740818154038, 48.834686282962906, 4.995927451978079, 25.404091644828121], [29.636121689941426, 1.5387052612994476, 22.711459361444554, 0.9484306666268596, 65.337818772350076]], [[63.611118862382476, 5.925018126102124, 116.2248839243269, 0.050868136296219363, 1.6711464964316436], [58.376837373076718, 127.02857928988502, 56.536083336503758, 44.695474260711102, 65.275814194072467], [0.14798615872627141, 21.577635949790377, 131.07124758898809, 0.25934799516586143, 70.727879507444129], [14.314732729557742, 6.2757312248229198, 7.9260879519445409, 47.853475432892267, 18.1177534387413]], [[0.93741356852035496, 227.3872798498916, 7.9485703254034528, 37.577273469030466, 0.008842920439718082], [386.144763331207, 433.73865328591296, 1.05019699584602, 40.028665304496819, 110.0933594360785], [72.473669467812854, 0.08814766530848514, 156.62652762002966, 89.237212923984302, 0.51008731804315355], [99.539001131430553, 609.5577037870404, 446.73150017759451, 0.25693757444173593, 11.913915367961559]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.4078049392484955, 4.3857505797487182, 2.6949848036197701, 1.2934424349434184, 4.2800664185901489], [4.7355278959922051, 1.4924421002812311, 4.2088650002091166, 1.175241873101448, 1.0546624678508656], [3.0095891631279907, 3.1358330380323505, 1.433752619228525, 4.2310903443093686, 0.058028750108887271], [2.4930821136859427, 1.4722486700167836, 3.8062105127050865, 3.7957622532426916, 1.6961041990415739]])+(1.-msk_arg0)*numpy.array([[0.025818316994839975, 1.4520886610155888, 2.1014183025056408, 2.4713547579119068, 4.0577553934571498], [2.6705524386200077, 4.8246752776097193, 3.4606472191804216, 4.5855899877762036, 0.22784489494219429], [2.6954219898894736, 0.57772690637465574, 3.0639690492883327, 3.8275166682372879, 4.010932809144319], [3.0688618721588208, 0.93595170589856602, 2.1273474643833548, 3.1615025855009113, 3.1665192666101283]])
      arg1=numpy.array([[[[2.4931062451439643, 3.8896624526903483, 4.4560505508180634, 4.8320792402535693, 3.2243767771275818], [4.2816175719505569, 0.61372944276041641, 1.662179868939206, 0.96565318594481175, 3.1060016292773756], [1.1678253132256935, 1.3834859919736564, 2.6497205100540109, 0.4503361558399479, 0.43709203594804968], [0.75393438550599146, 0.70604391620995677, 0.3574035951977198, 1.4565954164138535, 2.9265745406596002]], [[1.5942310003919338, 0.96714421353455282, 4.7704388030265816, 2.0355912543821701, 4.2960555583854623], [4.3857847521556907, 3.3183283279453666, 1.680060110985198, 0.44243125485478807, 4.980759916693855], [4.929557079833577, 4.0324011993487536, 4.1174436174806806, 4.1877882276797775, 3.4564871331502838], [2.3887678564854053, 4.5655087099450684, 3.7171808727242013, 4.9108917220278965, 1.3430921167840439]], [[2.2112282651207655, 1.3924586490291868, 3.3320457642334151, 3.6628749812950998, 0.50746641352369626], [2.2347865092390915, 4.5796058005274221, 2.0270242674170373, 4.3439855222844592, 1.6294206708082279], [3.4254731067125408, 4.1875662458607064, 1.0619375757951237, 0.83770323256533574, 3.4559088158505205], [0.78717622417010924, 3.3939652757103622, 3.8683511195664053, 2.7708177101126714, 4.8366733958861419]], [[1.3494151631578266, 1.1737382371752805, 3.0912257752252099, 4.5332988694021443, 3.9234258925762537], [4.3636015274262974, 4.9727173244209331, 3.9731207979725944, 0.91041764745705167, 0.67439663083373158], [2.8754056322802599, 4.7204216545355129, 0.78656383103933269, 0.13413455692538495, 3.7719870846136097], [0.57414490864683676, 2.2977898465685964, 0.82066179575841158, 3.4180544469718295, 3.8089788583301973]], [[3.6938558208247887, 1.975149024792481, 1.5915489489032426, 1.8377976280392596, 0.13443426058467831], [0.91553139706131714, 2.4740876450379981, 3.8777720383976719, 0.88782387140085417, 3.8623022650145296], [2.3048037723697878, 3.372457679352018, 1.1275872440001877, 0.79149812958418753, 3.7363523886334984], [0.3589358766943746, 1.6047358093122455, 4.517125723591481, 0.21369953504194117, 4.5675346106568755]]], [[[1.9402523070836435, 0.74434420759621656, 3.9852936311061753, 3.5737119836491855, 3.8193535601375084], [4.062081138163415, 0.21850966569154059, 4.5071721470362949, 3.9963676786629514, 3.5488189244538573], [4.8506453067309083, 2.3674434002196816, 0.099804707688254904, 0.39140639537486011, 4.2878974687158449], [3.8632381783013177, 3.6005786104157393, 1.3566351836403205, 2.5340165481503232, 4.7880417272550737]], [[1.8773223980046458, 4.9944949347690857, 0.34888489981829057, 2.2273160276219581, 3.9170193998115206], [2.71177070642094, 1.39950890969001, 0.42446933924184721, 2.2934673620430348, 4.5169666359236365], [0.4550798885601246, 2.9309660567133413, 2.3765339897289892, 1.8382954388614625, 2.6621982644693167], [0.37473053579470211, 2.6255005510106479, 0.21671459994162193, 3.4395332634931726, 3.1342821806049197]], [[4.4993610377318758, 4.4701877054186197, 1.1256692171633738, 0.50506406755673161, 1.8947488991052139], [2.2762110092897974, 4.8906855144545656, 0.82559405672602482, 1.217704381657464, 3.0348911458279377], [4.2015521861870821, 0.96435999434291508, 0.14469731014568157, 0.083867331346465665, 3.1308196475658763], [4.1108348800288015, 2.9623736108948582, 1.5786138772701148, 4.5278024658724849, 0.17824761662044625]], [[3.590084997544531, 4.4937219322228357, 4.840631297467656, 3.7901994680381854, 0.90705926575011564], [3.3716889762065865, 4.9830769381099334, 2.7098773297374099, 4.1727805474934696, 3.9168451471648176], [4.1678489605744584, 1.5398083586816469, 0.32796576643336683, 2.1298924279432478, 0.94116189217000745], [3.0850494962872035, 3.2711674481720956, 0.64754827374967372, 2.1899270753130216, 0.72189024669919566]], [[2.2760396397664246, 2.4324741638459377, 0.20034795256488311, 2.1157154512779797, 0.85861745833355618], [4.8395096726636515, 0.66716013431630683, 3.0443299861041635, 0.40967246395378815, 3.2547702870318673], [4.4632415143450004, 3.9641154665616112, 4.4323667688688335, 1.4316317203649449, 3.0337325152761383], [4.5108009370894759, 0.99013827904742546, 3.6723531353695966, 1.3442864565301826, 4.7608398680945081]]], [[[0.44052165098709234, 1.1702123863716816, 0.59091929361922413, 3.0017297737104163, 3.2620259254521997], [3.774053394463726, 0.70080540722870677, 3.7307420198091763, 0.46483338910586214, 4.1585024709696121], [2.6780160242826128, 3.7521283824458602, 2.771453285048612, 2.0972080494308001, 2.6864561850275228], [2.068830822078783, 4.71799543456126, 0.97919942231081047, 2.8643969634017314, 1.3457228485821668]], [[3.5903817536811942, 0.40626727079972175, 0.94623282954815768, 1.5088847719970135, 2.1283052946410068], [4.5824215145346647, 0.69743125859154564, 4.7449299649502974, 0.50840000947265129, 1.6502760341788927], [2.9927939397839212, 4.4878895092659468, 4.059578333028858, 3.0856608142336208, 3.1987644356446689], [3.3336024828515476, 0.10263707737534904, 1.4511789964393278, 1.0902596260565263, 4.4516096720265574]], [[4.2446631127414021, 4.8269679685734337, 2.9873179073328768, 3.2205246460028021, 3.6981559999656195], [1.976871864424983, 4.5771178641014139, 2.4865514298509837, 4.3878717483254732, 0.87237041171905794], [4.670240237388593, 1.3771152419053125, 1.5193203116225584, 4.1087578743658737, 2.1113476820489017], [0.17213491936673211, 1.2939086947838587, 1.3376101704430632, 2.7612739347448088, 0.55228658704041833]], [[3.7974761528862224, 2.0102094570063467, 3.4093991215747526, 1.4198366766310293, 1.254988172909677], [0.096312931645255465, 0.042132341318747879, 4.5766049479443147, 4.7538123181831562, 0.043634726096178619], [0.85306698076434562, 2.9389059548527805, 2.6578588648034471, 2.5498438420166272, 3.3103529903472766], [0.63964707214736694, 2.6808087402601242, 2.2107399020706131, 1.9734104935033123, 2.3774736388226918]], [[1.5535305147738179, 3.8137678088221949, 0.24452119147864795, 2.1296995396598533, 3.338838692289853], [3.2266542462541783, 3.7728975517029406, 1.3275925867593805, 3.0959347906994306, 0.84206979702985252], [1.30613764284503, 2.2884061482846434, 3.3831372550947578, 0.86898837560730124, 0.26816037769568868], [1.1126339535059795, 1.5200002945732518, 2.5090539268417604, 0.78313131979721617, 1.8058383570817502]]], [[[3.6283401173347181, 4.1374467955171754, 1.6258624557423647, 4.5992784739800863, 4.6063610433012059], [2.9724426595125473, 1.4040958553784579, 3.3396187968058766, 3.0192384774722085, 0.49877852693977531], [4.6895060223366052, 0.33132110664057901, 3.1262492101940991, 4.6657997044815582, 0.98493691343737655], [0.050679857109377092, 3.2045855964222305, 1.4169669847239919, 0.90359730428436857, 1.3056888091256436]], [[0.7708826076197306, 0.66689021850747687, 4.9218868070544595, 4.2007565249718679, 4.2017419292003737], [1.2997413913379694, 1.3886772230995452, 4.4847505667574845, 0.60953758381147527, 4.6198862760400319], [1.7644179305290772, 0.83412846417105679, 0.015132262776255083, 0.041299041540919137, 1.5858255882029968], [3.6135170784868271, 3.1197554395276632, 4.0669491238558688, 0.33406157270923259, 1.1002816111465434]], [[0.61327124128548627, 1.668439983450817, 4.3601232548667177, 2.4215833319723705, 1.7065900983585403], [1.2317965111311875, 2.9887712122148935, 3.829468799303049, 2.6510419076898923, 3.1199107744192589], [4.3825014357459038, 3.4625709138825127, 4.4797865552457896, 2.9416187862846401, 1.7130559076845984], [3.763187695279957, 3.0791568765293515, 3.0472759365379316, 1.1234956795336135, 4.481737239893528]], [[4.0188539106726902, 2.2339977722990021, 3.174677654436322, 2.2096554721141071, 0.36977162067934288], [2.6753090538612354, 4.610829699251763, 1.3886270142499195, 1.7572882180600626, 3.1040907151065262], [4.2263330684837666, 4.0327531752387982, 3.031529303638592, 4.6846683688369817, 1.3275031371866515], [1.8143092481969483, 1.8268227808760349, 2.2954638607471485, 1.2516974616568992, 4.4132369089519266]], [[1.1448491634308651, 0.31538926066657724, 3.5180159732768166, 3.2970762809564089, 3.947866250884724], [0.46605954523240362, 2.4817330071057406, 2.0154999181843958, 0.29591363939106868, 4.1112548972041703], [2.2913769777331079, 3.1734917151904889, 3.7864679968070991, 1.9808321306192438, 3.3593719102384667], [0.47153579555569342, 2.4819796110486942, 3.4633025999438853, 3.0435640695222248, 4.6063173032599973]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.9417484649697503, 30.505492834863723, 50.179208714544046, 69.827157974209612, 17.001641746996057], [43.048345855577338, 1.7147950785299066, 4.3084716945081603, 2.3362204712676307, 15.322032623436169], [2.7904035475161968, 3.3726224313469788, 10.261002380270108, 1.4854494239300253, 1.4682622448765761], [1.9396234276093303, 1.8596936059599303, 1.3689665137028364, 3.5963987484144364, 13.087092851731548]], [[10.557627525170673, 4.1778137144111609, 1155.6631738036017, 20.273979719892157, 573.13428904087436], [654.43493691280935, 135.0555882592592, 11.985948170184598, 1.9233587669330565, 1577.1270064992982], [1462.1509070908985, 388.13133958953324, 440.12869031141798, 488.36441294004868, 165.65958958893151], [34.173849393955351, 853.60539663308282, 243.5529610811148, 1422.3557027502675, 7.2832319345420666]], [[8.9548498026586802, 3.976782807432468, 27.204037477159797, 37.76365096252546, 1.6538376398583261], [9.1664559731600388, 93.708335439356972, 7.4601594578668626, 74.187446552209806, 5.0298770440853691], [29.844141971117189, 63.530623161454592, 2.8656552076486546, 2.294449067491398, 30.758374771791178], [2.1823464615907091, 28.926321012109014, 46.295952632325886, 15.595276477122079, 120.90946555460046]], [[1.4151198993051286, 1.3525765389367159, 2.2153152212210951, 3.2105765188976165, 2.7442993671272475], [3.073405317780705, 3.594899596958077, 2.7796156440066411, 1.263969331505004, 1.1894929467214983], [2.0956476047015502, 3.3689413826780314, 1.2243236283627787, 1.0351163022454259, 2.6394208135888881], [1.1592017588956121, 1.8062229802114853, 1.2351126788479476, 2.4096706582372072, 2.6646634239605453]], [[215.02409891889667, 17.668873281897174, 10.115431025469194, 14.470323712806836, 1.2158740294226835], [3.7854086153898856, 36.497565910889385, 280.94488237166905, 3.6359410644178962, 274.69624905688175], [28.534450531720442, 134.75351937178127, 5.1524707925313384, 3.1607627348333933, 228.72920249690816], [1.6851953102667034, 10.311248247477421, 711.77271388496672, 1.3643967620120876, 765.89996996051207]]], [[[20.435491845428285, 3.1820467505925452, 491.52021088719067, 259.16248880036585, 379.72631979975614], [553.86157466039015, 1.4046706986169355, 1106.6279584758338, 500.05807201914354, 249.32174415833475], [1887.8720481264309, 39.709686610794158, 1.1678980832428278, 1.8379915512358582, 786.88351993623951], [406.545337386276, 270.21970320639997, 8.2457190321698413, 51.451104229119366, 1712.7425103942173]], [[2.1206141543472494, 7.3880553546767525, 1.1499268350356799, 2.4396339596165295, 4.7991005354148859], [2.9618981706650693, 1.7513425077738634, 1.1852614333538694, 2.5051181944146412, 6.1022383565112559], [1.1998784435558225, 3.2336100434952324, 2.5898421774857576, 2.0877330459008521, 2.9036856626778338], [1.1618891607099051, 2.8613300762892764, 1.0906518121972224, 3.9639320997571281, 3.5078725747654538]], [[643.19648711644732, 616.7862606847051, 5.0420055843387228, 2.0665376006191361, 15.227757036544112], [26.347003877018349, 1128.7420493019274, 3.2757210464685813, 5.7550413769808673, 78.39219404655995], [419.23912239375306, 3.9987085560634417, 1.2311614146939731, 1.1280985799323733, 89.980423438755722], [367.99272766913947, 70.633367617027744, 9.6675299074835195, 670.03240584155355, 1.2919803930435783]], [[1.7855121596119887, 2.0660098674630687, 2.1850440341527455, 1.8441502041075586, 1.1577361066233487], [1.7236429140239855, 2.235885273605339, 1.5489459487290913, 1.9616686545249657, 1.8822512968572194], [1.9601071545723749, 1.2822790182249844, 1.0543852927892903, 1.4104688946349284, 1.1641289822421133], [1.6456825264968225, 1.6958912495061869, 1.1102244413772846, 1.4242085155003057, 1.1236322163448453]], [[1.1287745836604881, 1.1382115018737458, 1.0107197232132863, 1.1191841995978997, 1.0467564532138227], [1.2937726431693095, 1.0361446760326105, 1.1758856596892187, 1.0220425123187646, 1.1891293653086776], [1.2681221636550319, 1.2348794047592593, 1.2660401242152575, 1.0791703558543706, 1.1752226405077548], [1.2713360362136894, 1.0541090750922393, 1.2158525263155711, 1.074165384827662, 1.2883671114386956]]], [[[1.6247733539989175, 3.630404938092274, 1.9176049634218653, 27.311738930820169, 36.38337133359601], [63.960489206242485, 2.1644145425084869, 60.979937032549842, 1.6688839200435104, 97.695246607367366], [19.118331508416883, 62.433905167609751, 21.191436214385313, 10.081593488541239, 19.296949741250106], [9.7712578475375498, 180.96511077097432, 2.9413993216162053, 23.476568188403672, 4.4049206721224214]], [[60.54754075390553, 1.5909332215480667, 2.9489366208714971, 5.6096966849810057, 11.38650842591333], [188.14745646300179, 2.2190753521923878, 226.54782512155538, 1.7879107474635534, 6.5935506570472668], [30.583138043757774, 168.87956965271925, 103.51030916608561, 34.007663160333024, 38.700457974039935], [45.148503989907219, 1.1244605260502383, 5.2516642003215166, 3.476590678156755, 162.0203291221614]], [[4.6150920533702902, 5.6924086323642875, 2.9338523409264856, 3.1910168377604142, 3.7902345961891966], [2.0385881317807044, 5.2023667738993282, 2.4495212204935575, 4.8594691358926507, 1.3693153716095028], [5.3798754131097093, 1.6424155161976211, 1.728759052284822, 4.3945529454129861, 2.1397916613452588], [1.0639829730358823, 1.593908399737163, 1.6192037979521279, 2.7043829984129002, 1.2201651462172722]], [[239.29722657726222, 18.167716120794225, 136.71806934342769, 7.7528244438860465, 6.1120968284463801], [1.1490408200713049, 1.0626589444973431, 736.2489524525846, 950.68502776324942, 1.0649643612445752], [3.4229911086551548, 69.356056397056264, 46.240422700261, 39.569042036164504, 118.51634248169752], [2.5159869183430508, 47.796797377949851, 24.261804197549562, 17.228502884097416, 30.858429976323446]], [[0.01200278451759516, 1.9267363823449646e-05, 0.49852227260216758, 0.0023277242280185743, 7.4473906155782249e-05], [0.00010249534321658059, 2.1644747199966991e-05, 0.022836098068777202, 0.00014870309910359382, 0.090970649222069599], [0.024274364270924397, 0.0014815405229531683, 6.565008678770339e-05, 0.084259781167955156, 0.46607745723494415], [0.042110215489827608, 0.013204967133669707, 0.00079052235714979606, 0.10758959687239605, 0.0058524740539832153]]], [[[27.510295784900276, 43.800241362820231, 4.416102194633571, 66.788551662141757, 67.222079022496658], [15.110427229928799, 3.6062529998660278, 21.132406189026156, 15.770387207028834, 1.5771888146534365], [72.526813163113516, 1.3534655426099187, 17.389913791319586, 70.973043849343838, 2.4590111922106717], [1.0473854905862845, 18.679977088901218, 3.6489056980194126, 2.2829169196305612, 3.296207297246688]], [[1.3473913746212476, 1.294270342330899, 6.7109547070160156, 5.0774773966678923, 5.0794130228139753], [1.6532254432312397, 1.7110851634473576, 5.6670057114139363, 1.2658750405295913, 5.9710939568683976], [1.9787411683868268, 1.3807589783558765, 1.0058701845372946, 1.016102363011685, 1.8466680646168048], [4.0457852717292058, 3.3424134550754219, 4.8213745917270661, 1.1379313281492025, 1.5304762447710332]], [[2.2698624619671253, 9.3007378817128306, 339.64105084937285, 25.451401352662401, 9.7873083771437805], [5.1885859014623996, 54.320052723888239, 167.10101305296735, 34.58682939482577, 64.727041203492703], [349.95366720444525, 102.3287102619411, 398.55096108277024, 51.00215641632041, 9.8722609867975866], [152.93376448056733, 61.295474389167282, 58.738351564655062, 4.4893261469172145, 399.59147946832519]], [[212.87176560625383, 19.685782454273369, 69.037904211318363, 19.05685419733631, 1.6376018907081964], [35.465318687300972, 468.866329697487, 6.3742669052379108, 10.423061784203131, 62.834281801815244], [280.74403110530307, 216.85522586259481, 57.03768654236513, 517.39677083952745, 5.8751809012875222], [11.246762858584672, 11.436064996919635, 21.367805662837778, 5.3101536933395357, 360.23320321669843]], [[1.831000809287169, 1.1813181146010263, 6.4153102386340022, 5.7085063971460013, 8.0509642233677905], [1.2792002672602831, 3.7105639050891321, 2.9004244273954822, 1.1692251084724239, 8.7768329989625933], [3.3555386322192198, 5.3476835619064031, 7.3928970611354528, 2.847783323995944, 5.8995159792142138], [1.2829067227802899, 3.7110473831396349, 6.2325179993627389, 4.9929069674363999, 11.400690508397235]]]])+(1.-msk_ref)*numpy.array([[[[0.00010984178993552858, 6.6517465371120164e-07, 8.3843726390132607e-08, 2.1198733363879353e-08, 7.5763022645624658e-06], [1.5866557652389617e-07, 0.10601160571263699, 0.0022926542922465159, 0.029273398500925179, 1.168007305654207e-05], [0.013976847798946559, 0.0063522054364246201, 6.1951647657129157e-05, 0.19267870920626107, 0.20223966868754634], [0.063488908214561449, 0.075639974815470509, 0.27065579631219905, 0.0048620732789663073, 2.251068482868842e-05]], [[1.812401764404896, 1.434401472283044, 5.9262276840135568, 2.1367406248642564, 4.9651400654773399], [5.1341318505967299, 3.4478344598027704, 1.8713637085434105, 1.1794263481881264, 6.4098653911470462], [6.2886062253928445, 4.5000911276213698, 4.6451270421348809, 4.7686224618594286, 3.630171842294486], [2.4376087031739351, 5.4901102377237203, 4.0009007660328582, 6.2449755370479725, 1.6503334466812167]], [[5.165933115929052, 2.8124533957135323, 11.874803844780294, 15.181790906571271, 1.4576869541687241], [5.2571044697846538, 29.990255359276784, 4.505476048279677, 25.176170798529775, 3.3535790749935206], [12.727934485069039, 22.415194037855873, 2.2003316892580256, 1.8628151981847809, 13.01888612094022], [1.7942137485912455, 12.433581960427894, 17.684452991843127, 7.8275053970961554, 36.298416470468062]], [[3.3902566087666632, 2.8920309884366548, 16.392722392733365, 60.435512333703436, 34.805807498932666], [51.833694825457151, 89.940467192115051, 36.406471230574027, 2.2789512863173647, 1.8407470034178646], [13.484901438869953, 71.584552918304439, 2.0373635464919393, 1.1290318003743833, 30.349119240372037], [1.6811314872480119, 7.9961745282890249, 2.1011972231380782, 22.03299512932545, 31.382060691280696]], [[176.57136657829619, 15.902127471160878, 9.2921610213287202, 13.119168721686929, 1.2071867508348715], [3.604996161100146, 31.985455511031482, 228.45144716340769, 3.4677733955342753, 223.55473303036754], [25.233619689947666, 112.56887665059965, 4.8517161061766316, 3.0301011298243701, 187.400324992771], [1.6532389162421344, 9.4653811719142702, 559.37586224805705, 1.3489330837249782, 600.29777756916519]]], [[[6.7253329251607195, 2.0774858062320765, 50.133807284903206, 33.461815442161189, 42.593219536416029], [54.061528813980843, 1.2394141954195099, 83.70755203815979, 50.682133466505825, 32.653526648947782], [117.29801770134225, 10.231845712372515, 1.1030032677929316, 1.4688397020156507, 67.487263773475206], [44.469443598682233, 34.356651716543077, 3.7908977920643885, 12.050755430382651, 110.30213437630343]], [[19.190661771215186, 2591.6592910853633, 1.7316164212196108, 33.288898754858621, 475.50792837941685], [71.352425647676839, 9.047314337819353, 1.9503441319927344, 36.941264256544152, 1222.3696358575435], [2.0465981708235859, 100.74459812625294, 42.10029545365844, 18.047467567980895, 65.997511373393479], [1.8035008987221524, 62.293946199022749, 1.4064280304597419, 224.29067894284549, 138.73366243565386]], [[266.60204752118022, 257.11917419082198, 4.0449467507172381, 1.8720135752144687, 10.509153997108392], [16.874691761395916, 433.36011741405775, 2.7869237504226891, 4.5345504934597018, 43.279656457125455], [184.20340567939007, 3.3108671816750461, 1.1967808058326159, 1.1097309189774698, 48.753338409916523], [164.58367480172615, 39.5535478200973, 7.0977588847242155, 276.18359915934224, 1.2476808487005551]], [[236.84545681348979, 937.83450665958117, 1590.6345234312453, 321.23274792009965, 3.980381170009875], [169.83178482610114, 1975.9833013388736, 61.986899483489864, 575.25063958343299, 389.56744738607949], [570.94645686206695, 10.433304650426555, 1.647839808757116, 25.627171306079877, 4.1925666672106923], [109.75820395419593, 145.72515153316098, 2.6809291279413108, 28.080659333206263, 3.0023046498687731]], [[0.034511321457782761, 0.027382569520458918, 0.74354002107319672, 0.043746926508698658, 0.28083930551463204], [0.00077855480629778775, 0.37277137072185768, 0.01107750784002043, 0.54555998682626572, 0.0081145321922758817], [0.0013582792520050963, 0.0028418949684033664, 0.0014217452627239579, 0.12033059086554253, 0.011252511483682505], [0.0012660147420381072, 0.23119268259077211, 0.0043754696699979485, 0.13692514637354775, 0.00087462841101475751]]], [[[1.5477485806266476, 3.190985103595104, 1.7966590322004499, 19.616665572269564, 25.393097403499148], [42.189891859762042, 2.0034825626633501, 40.416376535737996, 1.5855126069561396, 61.767848629226648], [14.230671149460202, 41.282587983657962, 15.612123149733137, 8.0004409170298612, 14.35026569088012], [7.7784654788861802, 107.57097903585483, 2.6403985257789673, 17.119300059026138, 3.7975568123253227]], [[0.13947396598700701, 0.80019473345107139, 0.59502346988801325, 0.43698532348974728, 0.31108060572544943], [0.080930549627508439, 0.68205263183038589, 0.074027042316128111, 0.75658855032612748, 0.40436774666963171], [0.19359084895039783, 0.085238800739828743, 0.10781872869704566, 0.18397412515459147, 0.17290466426714865], [0.16057496974710969, 0.94524394373301834, 0.45104184348940657, 0.54981396283540007, 0.086952484633177715]], [[115.90798193323334, 222.47391463378068, 28.358680412289896, 36.820595771832387, 62.857264253108681], [9.1479102895527244, 168.18253530178876, 16.187176619674535, 136.06720569662872, 2.6559512513847046], [186.66584046010891, 4.6737946086594357, 5.4805194903911403, 99.546214762233305, 10.634458621574643], [1.2125691687468818, 4.2580188489462731, 4.4715585768385422, 22.017315546980765, 1.8559588257883044]], [[163.53614307012916, 14.85201736367685, 97.13975792042605, 6.7242997492449277, 5.3895692259794492], [1.138000502708616, 1.0581802750368294, 465.35163201950439, 590.3076970637087, 1.060316275144332], [3.1424423755244804, 51.658109547908047, 35.425122185313029, 30.644148202326715, 85.047457364991288], [2.3597240038925817, 36.533327617902536, 19.43918585111723, 14.13626583461981, 24.314761649382294]], [[8.6528687505362392, 199.81948572544701, 1.4044499944604907, 19.263332568887382, 103.30920946993196], [88.402447450618766, 188.79176333029611, 6.3221399224902113, 73.723913389101568, 3.2208813033755161], [6.1365108655908918, 24.01432876642231, 109.86565315554968, 3.34359162500879, 1.4513311303240053], [4.6901993226351699, 8.2591083880427227, 32.626854524343074, 2.9677024545994075, 12.284674625911194]]], [[[58.468159686790656, 103.47676457782809, 6.1909625420514169, 173.67783795088965, 175.06263250795664], [28.02284496636511, 4.827951146550606, 42.297888039740513, 29.532534986756755, 1.7494190083473717], [192.1689316613955, 1.4499261135856865, 33.297569132193388, 187.12798283594813, 3.0174630427774365], [1.0584733845043224, 36.354700769799024, 4.8981357270022698, 2.7544279271669714, 4.323553776409141]], [[0.95025415450523065, 0.95681769953919416, 0.72195925234135683, 0.75725596201669487, 0.75720657139111636], [0.91756514168882453, 0.91217949855684322, 0.74315405113254795, 0.96045692320254061, 0.73653631194724345], [0.88977265457999011, 0.94628439049667412, 0.99899887579973279, 0.99727009161297742, 0.90035331903150828], [0.78727020447810991, 0.81342546388854609, 0.76399269098501266, 0.97813067513399288, 0.92975963768337044]], [[1.5887438444205395, 3.5235362567057065, 26.879176705706541, 6.2213993231814957, 3.6264844405733774], [2.5341378422302316, 9.5462776558388125, 18.007235270691176, 7.3979844959342458, 10.539661184444801], [27.337096766615421, 13.650979425430517, 29.420238623491631, 9.2124614700491758, 3.6442281559802034], [17.128429549641474, 10.220354410471904, 9.9773259108458046, 2.3352052570171629, 29.463592566340427]], [[102.09373996051133, 13.084618288604085, 38.636773789657504, 12.723087063578797, 1.530546205282963], [21.745515977284942, 201.80131261568772, 4.9449929381300644, 7.558872683515844, 35.621721192812544], [129.63330825076224, 103.74024421563153, 32.767395208182471, 219.70273372132328, 4.6090375288568222], [8.0716342619252064, 8.1887367419833197, 14.043894500085408, 4.2239179827805762, 160.74912850576223]], [[3.7418901882372988, 1.4383952141668428, 57.684076781642347, 44.715437409143483, 94.674128468747], [1.7112012236041689, 17.470736225929031, 10.207590799436881, 1.4064654927370335, 114.29329498337465], [14.028859934450313, 38.778701049280848, 78.60283918005247, 9.807745247317742, 48.044265966595333], [1.7220366694580229, 17.475702880797623, 54.158597470853806, 33.385185940393285, 202.22747035873948]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.422452713603541, 3.6522830249438947], [1.3134522384001326, 0.64265985314619889]], [[1.9001136780092214, 1.7655624082226455], [3.8066518758397603, 0.014971559443750396]], [[2.428351908326583, 0.40443635583334042], [4.5230006607896991, 0.66428642635134016]], [[0.7640158425895911, 1.1937658774036279], [2.794084658239735, 0.67086623221578556]], [[0.47059413014459622, 2.2460092935596179], [2.4567148644427821, 0.4426406534640781]], [[4.6816659244714884, 1.1587038158484289], [1.0543543511675311, 2.9660005440550621]]])+(1.-msk_arg0)*numpy.array([[[2.2320907556407512, 1.9376834811223149], [3.5307974110857789, 4.7656825641988592]], [[3.2164505129984318, 2.0151636115354026], [4.9044214281846648, 3.3693782862041948]], [[2.4381396386429603, 2.0793392795793135], [0.75701678839087572, 1.7768329544493946]], [[1.7213791537086345, 4.9997486771542921], [3.1454834285618212, 3.7457399362625319]], [[4.7704810433635858, 0.49329370712626464], [4.4629568048125385, 1.2487329748986156]], [[4.4983896369894314, 3.7139191194713463], [3.9709776997883321, 1.4497064042349761]]])
      arg1=numpy.array([[[2.1289126348034881, 3.4278968287783584], [0.61461096675190363, 3.7873780720694463]], [[0.93199040824852919, 3.2804178366740397], [3.9783402697788017, 4.858641718652672]], [[4.7505298168534464, 0.28799955134933231], [3.1212398359009654, 3.7718297123724662]], [[1.1992964820959378, 3.5320663623058612], [2.0030099692631138, 2.2748438663224366]], [[0.84519362917292096, 4.3983399405246821], [4.0626428402612325, 4.4039407100289027]], [[1.7057013228707718, 1.3712194387583636], [0.37686461063074539, 3.5066479505675487]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[6.5772747059655261, 84.803310768100445], [1.1824389256427945, 0.18739245270818572]], [[1.8189463870144817, 6.4547531411387213], [203.98500273973193, 1.36231114706623e-09]], [[67.675949207342526, 0.77050068771012958], [111.1076016835957, 0.21377392837103881]], [[0.72411088607782526, 1.8693196907335192], [7.8310913268729845, 0.40329590673941695]], [[0.52883831428373673, 35.125899250993456], [38.536471295263858, 0.02762045976826151]], [[13.915630331186657, 1.2238276173332716], [1.0201471815524088, 45.262471379998559]]])+(1.-msk_ref)*numpy.array([[[5.5255696914777239, 9.6555257310051168], [2.1713439898407465, 370.0975211333967]], [[2.9707792364064494, 9.9601293045097616], [558.97602374708185, 365.74242032136453]], [[68.981606165484962, 1.234702593445419], [0.41942982082471308, 8.7422750230415449]], [[1.9181647412334333, 294.26007068199334], [9.9282527974310284, 20.170122696532736]], [[3.745557140152179, 0.044686258570963311], [435.69737558628907, 2.6597836454338535]], [[12.999310225940976, 6.0445687386938447], [1.6815247098264492, 3.6774974727298324]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.42578413707994, 4.3795602993716907], [2.7614058724862449, 3.6946240486311521]], [[0.42000741223387278, 0.35473458414220044], [0.89223884286444533, 3.2256160513196961]]], [[[0.21247233880943192, 0.82131419541780859], [2.1501262908874192, 2.7464410416123806]], [[4.0394065776445549, 1.6170644287627258], [1.9727354454008261, 1.8708633316270786]]], [[[1.687163764663693, 0.27959681455309598], [3.7022266754323589, 3.1128788290340901]], [[2.8309287985325793, 4.8852943822731572], [1.7922307072283974, 1.3299218377796376]]], [[[3.5650057527410755, 0.081534236240412172], [3.9221104186733027, 1.9769725625533436]], [[2.900453618825416, 0.77335175877621487], [3.5262409330923297, 2.3377179751549133]]], [[[1.8770787898544339, 4.3543920418104687], [3.5725305468197281, 3.8986601144819981]], [[3.7419858856351538, 1.5540085025666077], [2.5831779174864997, 1.9741055720297771]]], [[[3.9926903330134751, 1.3751353826188362], [1.7431769292588148, 4.9456658291309594]], [[1.5286410003264141, 2.4844308566059348], [3.6363199939750595, 3.3308001502546425]]]])+(1.-msk_arg0)*numpy.array([[[[2.9204463345701419, 4.4457524881314701], [2.9976155851682589, 1.9306580714929646]], [[4.7505727904908239, 4.8446653386077836], [1.9514078391375158, 3.074988589032611]]], [[[4.351414962508664, 4.0979740752372598], [1.6022847864652052, 3.1064988896314532]], [[1.8386078788810578, 0.62805833484665752], [3.9662014597094015, 4.2312833049366576]]], [[[3.3513475702039139, 0.66432855683713321], [4.3338981486554395, 0.051638487310947008]], [[3.1244087661171807, 0.79067991527307246], [2.749406082663989, 4.2575670252711699]]], [[[1.632187602397563, 4.0678564422397567], [0.68009775052568222, 0.25073636897634971]], [[1.8654944074415818, 1.3249792230103348], [0.5630251714175184, 4.6145491005529351]]], [[[1.0013602694174699, 2.7003168034921878], [1.1406872891939854, 1.4756771573162442]], [[0.57902099168780574, 4.5187214920219354], [3.6208781913956307, 4.8189613728431677]]], [[[4.066948121022846, 0.67327604995879531], [4.4919716161314787, 3.7383222875574491]], [[2.3355008927126191, 1.8161430577307345], [1.8155405551145705, 3.0722354349699978]]]])
      arg1=numpy.array([[[2.3430388936348105, 2.1207420251476718], [0.93723461646841633, 4.178716683978271]], [[3.3611260193321613, 0.063668203770499809], [2.1517862619834505, 0.68451148963778208]], [[1.910630129081428, 3.153460981237727], [0.52056610194911934, 4.0324820797702543]], [[2.7505833929927435, 3.9030678647844708], [0.25596201395428109, 3.5589210899516841]], [[1.5858006495334316, 4.9123760482336181], [2.8203001758388102, 3.9308882932304439]], [[2.411497987418048, 4.9297305247166046], [1.8328565292140391, 3.826232999502404]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.2959054282326834, 31.8343973734626], [8.6203211789945993, 15.983433823549468]], [[0.44350995921877934, 0.3785768111210906], [0.62097577030233264, 133.45911984616714]]], [[[0.0054824993431979059, 0.51600639237868062], [1.049947016384672, 1.0664382550823523]], [[20.168138446215853, 2.8127853276734487], [1.5921281337025444, 1.5353801366726836]]], [[[2.7165234961412925, 0.087604775798626189], [62.033446073935181, 35.906064354688496]], [[1.7189327655955886, 2.2835638886654843], [10.514930766761493, 3.1573781500241198]]], [[[32.998007302482698, 0.0010128611273696271], [207.27557161330679, 14.299162239049343]], [[1.3133295304477508, 0.93632964117872086], [88.682955293312588, 20.535346736091604]]], [[[2.7144930242159426, 10.308884841771048], [520.50609152168863, 799.46330519140724]], [[41.335203538185056, 3.4670174804585283], [41.699667227348975, 14.489985358478611]]], [[[28.180640033664854, 2.1558562065880786], [15.47919633493313, 2644.4845368227434]], [[2.1767349279107457, 5.3014446419719814], [139.70881377097757, 99.860239593480102]]]])+(1.-msk_ref)*numpy.array([[[[12.31872185193029, 32.973192079421302], [10.259324542062139, 4.0355931334284048]], [[4.3079410934229365, 4.3878617736693215], [16.341135118628035, 109.28481164973924]]], [[[140.12563471471447, 114.53080124940377], [1.0304701347853853, 1.0748355525525533]], [[3.7078661489349378, 0.36756922229410338], [2.5679963682240299, 2.6842771435820327]]], [[[10.080919479642944, 0.4577617052013882], [101.94645591057981, 8.7380029523866459e-05]], [[1.8095032597543961, 0.88491717793624136], [59.050414332152769, 344.41530982694957]]], [[[3.8480599921398984, 47.436315506155154], [0.22208261941641766, 0.0045196465835917448]], [[1.1730403878814153, 1.074684237810547], [0.1294637453640281, 230.9849427840357]]], [[[1.0021579754098735, 4.8321203403777675], [1.9090787666792919, 6.7631509868393236]], [[0.21415425977643046, 70.362383180456405], [157.26608554843241, 483.74189591790662]]], [[[29.461176166120207, 0.38520167348124468], [1645.6543079847181, 665.49072314173338]], [[4.7335585921266627, 2.985278617339405], [9.7952862107791372, 73.301771523511746]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.13653709528080227, 4.9376478085623434], [0.72148090038732104, 4.6107580782495274]], [[3.7150031548936098, 3.8074284223299362], [0.12545697134250791, 3.9841773401805942]], [[3.5871974105932543, 2.7072576331875671], [0.90337277246942516, 1.6938697248487862]], [[4.1430482467689327, 3.6920113127013625], [1.3549204960137655, 4.4964719883836617]], [[2.8790799142099424, 3.6517378019990185], [2.2739459794533725, 4.0300799622424135]], [[1.0028811480708504, 4.3282185722473967], [4.9535738678343328, 2.8121641331795053]]])+(1.-msk_arg0)*numpy.array([[[3.2751039218126996, 3.222446766638575], [1.6872976874207835, 4.469210333982347]], [[1.6990289261136184, 1.5954122079841448], [1.4993690083784226, 1.4646256037425969]], [[1.5002115154083144, 0.56956386556150684], [0.87187067055944834, 1.8755003952535438]], [[0.50401571858731498, 1.7970389564395179], [0.22686492240402872, 2.3135397111806215]], [[1.8436183189984645, 3.7808232786139695], [3.4918328828151077, 2.9252694231828822]], [[4.5197564075161942, 1.1184611020411137], [0.22190755675476914, 2.9162575292651023]]])
      arg1=numpy.array([[[[0.084143092171946779, 0.6829722727885027], [2.8011650280535476, 0.83323303919369696]], [[0.48464222652636091, 2.6079226975823988], [4.4644518279095706, 2.9158425818939806]]], [[[1.8298135108424052, 0.48724029811627229], [2.1562159528103022, 2.6961101935840777]], [[1.1265274175512852, 2.3271027197875616], [1.7769468788097615, 1.7459888892234319]]], [[[0.82658165851691956, 4.5125950713786631], [1.2960835034909948, 4.7991716053410993]], [[0.2386150318954916, 3.1679998296339362], [1.757459458646641, 0.58319177931568955]]], [[[0.80429246970887613, 0.67100445125597585], [0.58638529650529403, 4.6174686552347097]], [[2.5200860252299999, 3.3260673751819994], [3.7028390865058238, 4.5653893699065913]]], [[[0.89807957674487504, 0.44367324002515829], [3.6326971301886819, 1.8998478276408035]], [[4.9639467493287031, 0.3635075649231217], [0.55935558618677883, 0.6876925574355166]]], [[[4.3614810006104259, 4.0942112492970066], [0.2758962471706311, 1.2239973822012697]], [[2.0333781649555931, 3.926949283448967], [4.3338005934584771, 4.8094579982948042]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.84574086681331784, 0.2566848150274289], [87.632313412206742, 3.7832397885510369]], [[0.85366950969806077, 0.42683629215623675], [919.13566449409757, 86.189754066426758]]], [[[11.038727710462561, 1.8954270932655493], [17.863536299788624, 36.76606692067849]], [[0.096478307036443778, 0.0079818726596443228], [11.661916032040955, 11.173381619878057]]], [[[2.874422060276479, 318.70283792234119], [3.6357537634799386, 119.06488912561814]], [[0.9760435639323829, 0.72474737916134113], [2.5249183061414429, 1.3598184764370449]]], [[[3.1369371680576426, 2.5955204897535129], [2.1509716975454043, 416.21643814711098]], [[2.1499786896680124, 2.7463395289563084], [261.50473748596107, 956.34457087048816]]], [[[2.5849157111641587, 1.5986691186401234], [110.50721560559052, 11.712877976964414]], [[59.025222421087555, 1.3480074148411609], [2.1806480575164748, 2.6077724994077474]]], [[[1.0126270614433048, 1.0118487147726873], [1.4981500598690629, 6.0095337117511765]], [[25.884059906721799, 535.68596686405033], [88.318706516850241, 144.42494713152141]]]])+(1.-msk_ref)*numpy.array([[[[1.1049754569052259, 2.2484562799214132], [26.516253086237306, 2.6511674462434325]], [[1.2885660857486037, 3.9128970407079797], [799.69440618647718, 78.699258063510641]]], [[[2.6376944427017111, 1.2946819497573998], [2.7380275073281779, 3.5234468372940033]], [[1.5782136011399777, 2.5665942573122305], [1.9700960652376658, 1.9469591548686973]]], [[[1.3983130836099533, 6.2359823812608095], [0.4821282447916993, 0.067112924798705634]], [[0.96781192363278501, 0.64766758667589908], [3.0198984290168434, 1.4430443625004452]]], [[[0.57633900834757057, 0.63144954371623241], [1.410161230176626, 14.976531345873996]], [[0.02379459428430868, 0.0071984468768864635], [22.328469731763679, 46.032668729373071]]], [[[1.7321827627143955, 1.3118105518367891], [125.37079468709315, 12.511985315335069]], [[496.236162882548, 1.5754478700566699], [1.822856513382614, 2.0920824701094465]]], [[[719.89930240263027, 481.03792980228059], [1.0313695842530008, 1.1468637760361462]], [[0.046829617205681566, 0.0027067675687327984], [103.38621705240224, 172.01270010316983]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.7575509319089877, 3.2907956025384051, 4.5865235198041665, 1.931449054498861], [2.5435455349200149, 4.1783180632835411, 2.6207660107636777, 2.2883402373787951], [3.6755192398883363, 1.2912153915724656, 3.2049727386713749, 3.5050765560323427]], [[3.1306226807338713, 0.42732515710184649, 4.3106750021564952, 4.9027952870537819], [2.9809340168199356, 2.2806131175185795, 0.78876387354885891, 3.5711162661305469], [1.8900822622288018, 3.4895050470710709, 0.34826251881061204, 0.24706992174721576]]], [[[4.4677510820106345, 4.2419983104272525, 1.8578171722810839, 1.349536092806404], [2.9407949089073808, 2.2909028371458291, 3.1142737494038557, 4.2072634042035038], [4.2868615805595525, 1.3789487475349815, 4.0469208777363486, 4.6950323849203235]], [[3.4187177304029044, 1.125576015229824, 4.5570804418297017, 3.9342649299120969], [3.8044387080217903, 3.3134067972564614, 1.7937871822455453, 1.3394686279228833], [0.066377351094144513, 4.4460446540171912, 1.3609179127119138, 3.2024850101938247]]], [[[2.8322598455174819, 4.5227922753480563, 4.1012569300975414, 3.2723052288739867], [3.7683343073725486, 1.0981585333013848, 0.49518036833348306, 0.8948447458915767], [2.1065254056643439, 0.54383741206093972, 3.9387459239670433, 2.4061534902103943]], [[4.549753982055635, 3.0128665004166617, 2.813362735268218, 2.8831437356020033], [2.2419589705748231, 3.4023117497661968, 0.51369660276778673, 1.9610505044683648], [4.2214713673270854, 4.1931863265887772, 2.1585567840654352, 4.3926934877248218]]]])+(1.-msk_arg0)*numpy.array([[[[1.1231495429949123, 1.9444993133657358, 2.7618611468461949, 2.9683592046650853], [3.2541054909830187, 2.3294577954653346, 0.68775250074614636, 2.6722225617231672], [3.0788726205772949, 0.82765196234020844, 1.1764324947241827, 2.743293227823516]], [[3.3810211040866576, 2.8855292665571142, 3.2108717073378976, 4.6338897010395952], [1.6976124251516886, 0.66759331822846435, 3.1436844444787435, 3.0002344974193718], [4.3262943341786766, 0.84716101902005059, 0.012919323492598569, 1.675520578699244]]], [[[0.79216029279182132, 4.7244948922952341, 2.0022870754762265, 2.5285319926844263], [0.61973551152065787, 0.97241266215923849, 4.8099486704547196, 2.2989277683813514], [0.023717130563797879, 0.89523554348735612, 1.5744955314930282, 4.1637682786612196]], [[3.0303847542262008, 1.7747108546461756, 3.0679130547086793, 2.4293389286800902], [0.75938824203665134, 1.577740322283973, 4.8926021652737504, 4.5018695845991497], [3.7264851393815679, 1.6161556863500572, 4.9130020610607268, 3.1529083613911557]]], [[[2.2501038568909859, 3.0857914506633914, 0.34517941737150304, 3.4984385334756234], [1.5941637583484403, 4.2499568811213662, 1.8952553072235354, 4.7046109182783491], [3.8808672980560774, 1.8375394467470834, 4.1997600153987937, 1.682461609146328]], [[2.1613500482702057, 4.0147465496309387, 4.238993044695925, 1.8121437328230758], [1.1737406025714425, 2.0707503534091174, 0.016311477031344696, 3.1620315591820636], [2.5626670341707816, 2.7325355445715265, 4.8790287413334559, 4.1381852983030045]]]])
      arg1=numpy.array([[[[4.2407767621571324, 0.16924849893251487, 1.9440938350359895, 4.2235228107145888], [0.5195843246089934, 3.654667857276988, 3.9312522311073783, 1.0622697172742692], [2.5018117844810908, 3.5044746156366151, 0.47009907471187745, 1.7309992116748105]], [[2.8711495684051722, 4.6199207800320119, 2.3281036734548706, 2.7417832455274374], [3.5463794428311215, 4.0465743866069142, 1.1769274547403288, 2.7743220516224651], [4.9948663841819121, 4.4449967587707446, 2.1306255073951577, 0.48719525800645053]]], [[[3.2818680813719339, 3.563572438512804, 2.485713311402538, 4.8206377757413481], [2.9329756775351945, 3.0754864233160522, 0.87513740267230411, 2.8282253452187009], [1.6659185975049133, 1.6279967096262509, 1.3019354139714303, 1.2537737650675866]], [[2.7229494536436993, 3.1143564571679101, 0.95497375962153874, 0.14089255855592953], [4.6670565607867944, 3.6679997036332552, 1.4942065287414636, 3.8314508246210348], [0.94588902457513802, 1.4082968455039735, 2.333435429665152, 3.3094925190077937]]], [[[4.6256845167287164, 0.61474302382697166, 2.9549732446889569, 0.48525401371549831], [4.6066575136897212, 3.860295329287835, 0.47867487283809412, 3.5544924790412802], [3.0579577823433941, 2.8570842793659894, 0.2430480591374338, 3.021551893257493]], [[4.2908260292979232, 4.2080177068659195, 3.1594158172148581, 2.4461034958856183], [2.2693373034845963, 3.7782402885175723, 3.7509987891353904, 1.2864485330575837], [0.46672960271400488, 0.2691993596886989, 4.3664440896819308, 0.72804554649925479]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.929487880843947, 1.2233547208365574, 19.319070938231242, 16.12257695281685], [1.6242767114474692, 186.01767343068929, 44.151654460039516, 2.4093941030745651], [25.960974271754967, 2.44901872955269, 1.7289703836384456, 8.7674027444521734]], [[26.486827843278228, 0.019684905804192108, 30.011465262895985, 78.171263750315376], [48.10979084187359, 28.111402035792977, 0.75633484309923438, 34.170890710683167], [24.042752535244666, 258.57317534797011, 0.10567562097304344, 0.50603985619925229]]], [[[135.9897690830947, 172.342705219843, 4.6629877060109566, 4.2420163510272006], [23.658961738610621, 12.799577259063408, 2.7024332485340778, 58.185098615591883], [11.300375732013732, 1.6872672926002126, 6.1721598506927418, 6.9515691395205463]], [[28.423832127479386, 1.4454382202843299, 4.2562635257866992, 1.2128632996399815], [510.79779433928877, 80.978114653544083, 2.3943418696113441, 3.0643319497006343], [0.076870829668221008, 8.175973723168962, 2.0525236408952861, 47.087517490798604]]], [[[123.43123029378307, 2.5287544543783262, 64.737104229110685, 1.7776034152467481], [450.94447893691006, 1.435420242476432, 0.71431688845027974, 0.67373255910398733], [9.760078536084638, 0.17547403742033271, 1.3954060394360481, 14.196726743050876]], [[665.75075462090354, 103.64678907373536, 26.259767642574374, 13.331547050578573], [6.2473009763450245, 102.13400159744508, 0.082197982473423942, 2.3783280615884612], [1.958495117840205, 1.4709180912265336, 28.781017469919032, 2.9372260380795239]]]])+(1.-msk_ref)*numpy.array([[[[1.6364190902398394, 1.1191293501308208, 7.2067221456659141, 99.011131366895242], [1.8460839273290222, 21.988443229754896, 0.22956458379900216, 2.8408863770834509], [16.667258246851102, 0.5153467268943609, 1.0793778135555903, 5.7365249527470619]], [[33.035097805829686, 133.722742461905, 15.117179114131885, 66.969759810482017], [6.5327297448696013, 0.19492820262825403, 3.8499009197437757, 21.075709046435104], [1504.2317063640025, 0.47841957818570174, 9.4571283386688695e-05, 1.2858926180813062]]], [[[0.46549802484464381, 252.9963522562779, 5.6170468621939245, 87.515077784710499], [0.24577990592729507, 0.91756048360366138, 3.9533622823823662, 10.531132819579453], [0.0019632961121637412, 0.83513004748034936, 1.8057823856996316, 5.9799284406297906]], [[20.468880771730959, 5.9685957742412779, 2.9169053388992134, 1.1332152388646457], [0.27676942603008986, 5.3259151938897293, 10.722968375599988, 318.74457559315704], [3.4704502754748954, 1.96610281632066, 41.040577070361472, 44.717907801375638]]], [[[42.577283317116162, 1.9991007675439987, 0.04314545106932216, 1.8361880715189494], [8.5703755210994164, 266.53229717274706, 1.3580399978836757, 245.74261577904673], [63.229447232217531, 5.687827786260657, 1.4173382214295536, 4.816205790678989]], [[27.305316516470477, 346.89985219243437, 95.892294060280491, 4.2812040023144853], [1.4384097749103422, 15.646117206739492, 1.9727156553321953e-07, 4.3972462254428137], [1.5514891394730668, 1.3107601795294452, 1012.9061391377238, 2.8123264089767743]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.84169109959)+(1.-msk_arg0)*(4.18836072471)
      arg1=Data(3.91095759188,self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(477.523250838)+(1.-msk_ref)*(270.886904928)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.7778403443627484, 0.077489406761098695])+(1.-msk_arg0)*numpy.array([0.20546201310005199, 3.433939600872046])
      arg1=Data(4.08490071315,self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([595.10525244536655, 2.901784966752161e-05])+(1.-msk_ref)*numpy.array([0.0015580327181124228, 154.40444079468159])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.03369991777)+(1.-msk_arg0)*(3.60340540097)
      arg1=Data(numpy.array([0.4421734331831142, 4.9655567331291355]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.8527893011083765, 1017.7837697882678])+(1.-msk_ref)*numpy.array([1.7626405373755476, 581.28699544120457])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.7015024336949289, 4.1610666062010226, 1.2403123731996408, 1.2434432639280237, 0.3577249695805278], [1.9789728044969657, 4.7525031010949821, 3.0263531488301849, 1.1826514933702974, 3.9204086401669791], [2.4790613812432829, 3.5776109893309207, 1.2277666289701807, 3.2673656595897631, 3.8779709764934585], [0.46446625844209793, 4.3430874208655643, 0.25578955362251438, 1.7202278035450314, 3.0876339229669645]])+(1.-msk_arg0)*numpy.array([[4.6355521201408756, 2.5272695875145321, 4.5001699116066529, 0.23135264509901338, 4.4277052788922653], [3.8993964451423953, 1.0470804931753779, 0.85682842518473412, 2.0120593077132032, 0.46873818410588153], [1.3323025400781345, 1.261298901316187, 3.8080470368885875, 3.8066927355788214, 0.35659133349431005], [3.9485089052235685, 2.105856932856407, 0.79754454876460978, 0.28593383396166316, 2.9710362993361947]])
      arg1=Data(2.04353246529,self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[14.504373536873059, 18.423190418231254, 1.5528653189273971, 1.5608862320559751, 0.12236674542260885], [4.0344506602405641, 24.17201856414178, 9.6111382867772157, 1.4089163353673062, 16.31142303276506], [6.3935027642789697, 13.529617069769044, 1.5209364968594834, 11.240348481924046, 15.952638217802214], [0.20864598574693743, 20.107685627866907, 0.061657979183033264, 3.0298948942354564, 10.013046334229159]])+(1.-msk_ref)*numpy.array([[22.972061800663443, 6.6501517864526329, 21.621931550634088, 0.050219714437433817, 20.916410787184631], [16.133268300127458, 1.0985755255084186, 0.72923320558650884, 4.1734939416676085, 0.21258638780783623], [1.7973389305681542, 1.6070333579150022, 15.370358996316011, 15.359190416516521, 0.12157561024313426], [16.551236633254664, 4.5807583453704774, 0.62984408150017468, 0.077421395298146345, 9.255561377035864]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.38730396501)+(1.-msk_arg0)*(0.639189938143)
      arg1=Data(numpy.array([[1.5943316686297482, 4.0430038904933623, 2.6104819227467284, 1.1435220127070593, 1.3040936913851267], [2.0857789447618167, 3.524016085516517, 1.6190395793860493, 3.3794996034875866, 3.6274530043946345], [1.6228389726171746, 3.172250458943052, 3.931459808890172, 1.4549775071463353, 0.032974430750406701], [0.96557805420351495, 2.6850390325174605, 2.3246358831240004, 2.0275315613868892, 0.93269856057778744]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.9946064456550436, 138.74024998010313, 24.1643286484138, 4.0355237490572025, 4.9088468299811927], [12.739684494805685, 73.657100643932395, 7.2086654945606563, 61.750710519503059, 83.564362165326372], [7.2421580368724454, 47.954584839620281, 121.08779640617983, 5.9010038790576482, 1.0410501226304545], [3.2479965476433286, 26.465440643905957, 17.049750086658459, 11.865773487689165, 3.1202847039271915]])+(1.-msk_ref)*numpy.array([[0.48990248430046346, 0.1637423617996086, 0.31088548498657559, 0.59942319778109021, 0.55785742488904611], [0.39317600019066429, 0.20655549183357358, 0.4845149292676848, 0.22035678383158261, 0.19721125878771059], [0.48369174455553371, 0.24177392215814278, 0.17212417880364883, 0.5214298866889624, 0.98535053694542285], [0.64911332720773141, 0.30068293097198157, 0.35331360444406407, 0.40356040410439542, 0.65873587417398838]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.4375260364988858, 4.3357591962349709], [1.8374748441210027, 3.1751002646792892]], [[4.6794977904094432, 2.511016783099699], [2.4157391655827318, 3.6789997249025594]], [[4.37628616298288, 1.7648192133490712], [1.5152948560962392, 0.64212395816600976]], [[1.7216030651506262, 1.5534671222192911], [0.70059361770716033, 0.6437776584315783]], [[4.6710443188989172, 0.62728996826133232], [1.6648232353091148, 4.7059732230630589]], [[1.3728938907282384, 0.16637592580310723], [1.9519686007559722, 1.955745977935927]]])+(1.-msk_arg0)*numpy.array([[[2.7408798193077382, 3.1838334079769561], [0.85551536960846153, 4.4449690098425609]], [[1.5022669631767576, 1.3969625676115685], [3.7621262276662542, 3.0326141134242683]], [[4.3669130149771807, 2.217197604473768], [1.5106987622660764, 4.1038714658780062]], [[2.0099609139829129, 3.9272096870370077], [4.6825137551654405, 1.8934871470899028]], [[3.9541579417582691, 1.510180812990634], [0.50161405355264721, 2.9961179381030849]], [[3.1551503987121921, 0.22406440068020725], [3.8808250734158367, 4.4881768769252481]]])
      arg1=Data(4.23466746897,self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[550.08091488088871, 498.60781695621955], [13.148910706054384, 133.28335805654632]], [[688.76480492417818, 49.343523599946259], [41.887992317929431, 248.70091966425591]], [[518.64417552043585, 11.083920698241883], [5.8122649725515032, 0.15322509683892699]], [[9.9792502865259038, 6.4580386014145352], [0.22161576193459959, 0.15490311157852146]], [[683.51120396037015, 0.13878603790089775], [8.6580564398908582, 705.41834748618624]], [[3.8268973199616942, 0.00050301069153131336], [16.984619915821597, 17.124241241949356]]])+(1.-msk_ref)*numpy.array([[[71.502052620970446, 134.84269509972361], [0.51642509479613963, 553.99859915143645]], [[5.6035759812073227, 4.1191652849212081], [273.38138718103966, 109.73253663786009]], [[513.95643057986626, 29.131754825767175], [5.7379757910410554, 395.06779259899287]], [[19.226432967836054, 327.90540185147069], [690.64658972834786, 14.931859767227349]], [[337.5399670778807, 5.729649596478855], [0.053847469223591952, 104.24818226762659]], [[129.77290883465605, 0.0017743736893920856], [311.81542160452278, 577.16430450006203]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.329788562521)+(1.-msk_arg0)*(2.54473671788)
      arg1=Data(numpy.array([[[0.73600019297647123, 4.1310456979934118], [2.1608366330974511, 4.6685918646389268]], [[0.88908289636614712, 4.9103127351545881], [4.7856769314647316, 2.8123076236909803]], [[3.4754483860595169, 2.3052862334078599], [0.18747160482668596, 1.257616310450014]], [[3.4686657120267768, 2.0702612117253443], [1.4635518254531843, 1.4326880640301423]], [[3.0789259504481006, 4.105508758140072], [3.4832787126364311, 3.8099763674317497]], [[1.2961885835050324, 3.7452017055563398], [4.2862001292288836, 2.2541213457220346]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.4419990145176298, 0.010228435370247216], [0.090988355062931375, 0.0056342821740940109]], [[0.37296794075787093, 0.0043090951181856893], [0.0049480169102871516, 0.044170292588074471]], [[0.021166675788970878, 0.077516675242813277], [0.81223715661554385, 0.24781364738830661]], [[0.021326535438371371, 0.10060552417285343], [0.19720268232359009, 0.2040712758454015]], [[0.032861172494581009, 0.010522331845391265], [0.020983613826089986, 0.014604576975526718]], [[0.23743376075079439, 0.015692607371392425], [0.0086111533290584349, 0.082043571767405607]]])+(1.-msk_ref)*numpy.array([[[1.9886264911215248, 47.394634270110231], [7.5253714905070446, 78.303457481496139]], [[2.2942995795895906, 98.137114197542957], [87.352564521010237, 13.829054786536656]], [[25.691579791248362, 8.612374219027247], [1.1913696087064805, 3.2370027790631424]], [[25.52933291972132, 6.9149122156477523], [3.9235513671715316, 3.8120595683513128]], [[17.739620171057528, 46.277543645100387], [25.880170082957545, 35.114759313920146]], [[3.3557503122632677, 33.053261316679155], [54.785605126354596, 8.2104738277922014]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.7276341280999221, 4.1694482101546253, 0.13473900596526292, 2.2960656046104591], [0.1994377945949336, 3.79578857243696, 0.92365847442951665, 0.36154085813267278], [0.57961696023338838, 0.26965602002330563, 2.4675739844242877, 1.0175001691656598]], [[3.739262922364138, 0.336379587044976, 3.5910634944607822, 3.8864729750746352], [0.33112040863660608, 4.0381184873396645, 0.06965134995626987, 2.1199963644343232], [1.7258586786584715, 1.9830081064664644, 0.10947697017678747, 2.6147386699736566]]], [[[2.4639062799177851, 2.9649460272414503, 4.3632977069860539, 1.3154012147976808], [3.023722692570916, 3.2988131547512327, 3.6781331851568573, 2.8734158946219206], [3.3367175649580201, 0.31638161029385081, 2.4973533119737028, 4.9182747336544583]], [[0.68056177945136909, 1.1676291166600714, 3.2441549636693634, 4.7693083376950831], [0.86975269987359227, 0.50558723193523059, 1.7730188516393497, 1.9005535005120906], [1.1939757289867565, 4.7488973423502605, 1.6495880188624039, 0.48439403556679828]]], [[[4.9807324487093911, 1.5593030565099255, 1.258120500114815, 2.8978266415423941], [1.4357863614791131, 0.25933273197019702, 1.4868443282746866, 4.6173172906203694], [4.8309604335090794, 1.6769101806090254, 1.8728552437287698, 2.6719033084174129]], [[1.9971494775774978, 2.8193412343579065, 4.5489706588175132, 4.1040022413178008], [4.0829611633812322, 1.3708979596152164, 4.0506786218375952, 4.5092356458458598], [3.890702155333964, 0.43181200123213642, 4.3220413707173568, 2.6126142417362876]]]])+(1.-msk_arg0)*numpy.array([[[[4.5896985166831215, 3.2653178110763572, 2.9552159299651155, 1.97346643439011], [4.2677964399897634, 4.8470621029468681, 2.8434553126942679, 2.625834715231568], [0.18910565404031976, 4.2674535140665055, 0.98150532467667695, 3.097250064992318]], [[0.80529518396682609, 0.1161286718019789, 4.9741077544486298, 3.124531655673334], [0.075725025850836652, 1.13606127636742, 0.20194327923725422, 1.9084641267356135], [2.8422854974342182, 1.8615495033187606, 0.50325083976391904, 4.704219231594629]]], [[[4.0588602607066324, 2.8236641090051484, 2.6924015568281603, 3.2503595161910921], [2.6417664974921373, 0.40335289417014808, 2.9405102716768097, 2.0482101988059851], [2.2950528131694004, 0.34466541634309872, 1.3768820976206428, 0.49318190122383243]], [[2.4477268362682154, 4.4737062135660164, 1.7505416867925561, 4.0683157120229811], [0.5653545723389013, 1.4328236132226899, 2.3259010492181487, 2.497673293072125], [0.079323739107013014, 1.3470910284504469, 3.0312285267333028, 4.0835302216921674]]], [[[3.4869728511462723, 4.2369603238035669, 3.9638806726934512, 1.1947792838975222], [0.24483080159896559, 1.7496825066468267, 4.0892568349702039, 2.6755277777210265], [4.5269827247084677, 1.1954825702416998, 2.0280951616989245, 4.0303513185553959]], [[3.8661644909491688, 1.4692155551551997, 3.6403434208277039, 4.5938594642304578], [4.6044445985008711, 2.0866429099329715, 2.9401614327473888, 1.9493374842127846], [0.72957214708379126, 0.13816177062988177, 4.2402383234604146, 2.2131290119560174]]]])
      arg1=Data(4.51370397827,self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[11.797419422137576, 629.29049222281537, 0.00011770383719643043, 42.596727795596529], [0.00069109565162857041, 411.90570313932756, 0.69876115744180312, 0.01013102130534424], [0.085288204706234924, 0.0026967791109947107, 58.964617390766328, 1.0814550125782139]], [[384.93416721118655, 0.0073155845225416323, 320.71010452854011, 458.22596290770196], [0.0068133181666378802, 544.64412765949623, 5.9886158210970872e-06, 29.71529951379291], [11.742794291658061, 21.980356475549364, 4.6109176233856916e-05, 76.585708143484595]]], [[[58.570056017552133, 135.06538847379602, 772.56461902350929, 3.446621915968751], [147.57886066025793, 218.63178106173643, 357.33444006150285, 117.2398048744697], [230.20198928682362, 0.0055475528751852852, 62.24536308334347, 1326.2888493961811]], [[0.1760409159843884, 2.0127714718033767, 202.75019686571977, 1154.3748092151056], [0.53266052910856632, 0.0460281233335849, 13.262287920345218, 18.146054650372559], [2.2260498634778472, 1132.2427002522913, 9.5756580824314259, 0.037938531135661889]]], [[[1404.0260014753055, 7.4273057383654306, 2.8191401478057121, 121.80301370884543], [5.1174895204190722, 0.0022611299474287732, 5.9917747025836974, 997.37509341642203], [1223.2765273124937, 10.312661992412474, 16.982567232117695, 84.438879546326675]], [[22.696787402611456, 107.60518730846498, 932.44910739432578, 585.91893774633354], [572.48144089637685, 4.1533697980032427, 552.33249333070921, 896.24553276239055], [460.48094732287126, 0.022585425368782074, 740.13613101022952, 76.305245744634334]]]])+(1.-msk_ref)*numpy.array([[[[970.72852406006837, 208.78888782255643, 133.07621211406305, 21.506991849698611], [699.12194314016062, 1241.7878672629236, 111.82227774933838, 78.063652356356059], [0.00054357351380360122, 698.86841783244938, 0.91919121955742844, 164.48319995781461]], [[0.37627829628171661, 6.0174642081342925e-05, 1395.6165829191998, 171.12470105316348], [8.7341079977452086e-06, 1.7785543045362227, 0.00073115793549840191, 18.489471254412383], [111.61477761691904, 16.524716185823713, 0.045075810926939124, 1084.9500483734344]]], [[[557.38593389489154, 108.35191300347485, 87.402504102960904, 204.50634992228572], [80.224415031018395, 0.016602734265702041, 130.11321500777299, 25.436211600879549], [42.511983888847091, 0.0081648873431669333, 4.2358329816103613, 0.04114575763482492]], [[56.853979841587318, 864.80921011381849, 12.520119907175099, 563.2709075561844], [0.076216657923596637, 5.0699975270722524, 45.152761241461484, 62.281369688359234], [1.0770397791946937e-05, 3.8375981833067048, 149.23962505272911, 572.84167294450947]]], [[[280.83565854957817, 676.60924414235717, 500.8863608870974, 2.2328200636192697], [0.0017439028158988152, 12.492407206744399, 576.47663342115163, 84.957123415492362], [912.27748675992405, 2.2387586135342428, 24.327966673218754, 539.93151348514493]], [[447.51701426001472, 5.6777277774453436, 341.0597613870147, 974.70712734475796], [984.88562765274492, 27.662683851178699, 130.04355777620731, 20.345307521863432], [0.24095203313614216, 0.00013181517207310094, 678.97524986174233, 36.079312719847891]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.22780382383)+(1.-msk_arg0)*(2.91418044188)
      arg1=Data(numpy.array([[[[3.8560170811792935, 3.151650718779083, 1.77481612474843, 2.4342179148037992], [1.9626570623601642, 0.94179967260981179, 3.8492440296320196, 4.2444403983815429], [2.9491987385203573, 1.1140676060498775, 2.3137964505656767, 1.1888905789655002]], [[1.2550493690284277, 0.18505617632363974, 2.8555740932287264, 4.7872673764772209], [4.6615923315370793, 4.3257203356375733, 2.5378760018133324, 0.82575401642803803], [4.6915000568958991, 2.5489609867467076, 1.6790490903606055, 1.4517456521162957]]], [[[3.6747975245887741, 1.7714570531017282, 3.7880140668348172, 4.4262262498164473], [3.6754326098136914, 2.9908656001106531, 0.88832008158743614, 4.0291445571892419], [1.0038096863482238, 3.4241402914189094, 0.49063162893516998, 1.4483190590581643]], [[1.1459748196999253, 3.290406423055257, 3.555377744529582, 0.70375029880808504], [1.6207448188173064, 0.068573085778185716, 0.091189580266861373, 4.6046083900615162], [2.5221035871212818, 3.5027122867476308, 3.5989494059135292, 4.6480865154865283]]], [[[1.204480425845383, 3.4285701013711822, 1.8863083247327788, 1.448245995441019], [4.1636591045881453, 1.1357683342270311, 2.7308093981873509, 2.7817636680381592], [2.3056818472655451, 3.176201799816154, 3.901011209995056, 0.38913982359230492]], [[4.0963748019706445, 2.6112563141037635, 0.018032864430356559, 0.67809926258972875], [1.9379757984441355, 2.294293775834674, 2.5639391855090148, 0.50875894449295689], [3.0723184467534446, 1.4950710325789516, 4.2654129811196917, 3.580234576736737]]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.2063930227286832, 1.909428580838805, 1.4394200696976109, 1.6480081827124318], [1.4959932197512593, 1.2132258234918276, 2.2033282372118888, 2.3894758753806182], [1.8317199692870278, 1.256885485855465, 1.6077788868427878, 1.276334787794444]], [[1.2937824989222813, 1.0387089375953478, 1.7968607027994876, 2.6710638268242994], [2.6030527985351868, 2.4296686397364025, 1.6834425633102514, 1.1846733382851109], [2.6190791439276184, 1.6872766518163307, 1.4114059224976894, 1.347077590754765]]], [[[2.125841891512386, 1.4384281151630429, 2.1758143467430391, 2.480304709902418], [2.1261189847362627, 1.8474504751406182, 1.1999829190561624, 2.2861964765560487], [1.2287641585314317, 2.019250211391773, 1.1059347322179527, 1.3461306188818272]], [[1.2651428593541181, 1.9645838678665366, 2.0743746551867455, 1.1553792070482694], [1.3946182212206986, 1.0141725446701817, 1.0188907849694484, 2.5727883259543907], [1.678002193480894, 2.0520747611877539, 2.0930070700801764, 2.5958477535021465]]], [[[1.2804249075871648, 2.0210867805701889, 1.4727354472080039, 1.3461104343003618], [2.3501884938962565, 1.2624956050939706, 1.7514359532161183, 1.7698471578970298], [1.6051036226640567, 1.9190736031689948, 2.2268612409779664, 1.083137609967306]], [[2.317958810167779, 1.7089864366612006, 1.0037076883631784, 1.1493129407610558], [1.488434779047197, 1.601356650336982, 1.6924712048675745, 1.11005671060541], [1.8785926304268543, 1.3591085937177989, 2.3997826829367064, 2.0849836885246296]]]])+(1.-msk_ref)*numpy.array([[[[61.827733157693984, 29.106740191214648, 6.6746925565420527, 13.512454292852681], [8.1599306900332031, 2.7383025466980566, 61.38144811210897, 93.672549376932665], [23.439659383800066, 3.2923251330766425, 11.879454217956201, 3.5666395269521236]], [[3.8281686631943765, 1.218881922756248, 21.206114868189871, 167.40344650971207], [146.34782441376245, 102.18054320272763, 15.096807708981762, 2.418668315146745], [151.1050228062542, 15.2768664171974, 6.0248453640970094, 4.7245420736864112]]], [[[50.933539888178323, 6.6507545835820627, 57.490330564665399, 113.77709905431215], [50.96814977338925, 24.507907841862792, 2.5860640745549626, 74.405301794495415], [2.9260793625522052, 38.955560312502229, 1.6900768663921746, 4.7072581071347273]], [[3.4066235362029218, 33.763532247818681, 44.826146872186229, 2.1227731356413191], [5.6606018898019794, 1.0761017201781364, 1.1024503993433574, 137.69440479237963], [14.844260854067507, 42.370875598984533, 46.964657851637853, 144.2489289098433]]], [[[3.6266108660579173, 39.140572608510439, 7.52005862285157, 4.7068902586355916], [85.918779897532858, 3.3696365659778658, 18.556918384269323, 19.596339446020892], [11.776795168632082, 29.881194204136662, 64.875965829417382, 1.5162186880030044]], [[79.952755603553811, 16.329447272346375, 1.0194749570009416, 2.0653243000185308], [7.9473367763424676, 11.63421740691761, 15.52358059158238, 1.7231651126776035], [26.738831523955486, 4.9486310109477882, 95.797560431808932, 46.033904980104552]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.5916235985906129, 4.504935516706805])+(1.-msk_arg0)*numpy.array([1.9130691617101301, 4.7083912672813462])
      arg1=Data(numpy.array([4.4678046900882222, 2.9449467966801763]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([70.430146217866294, 84.154643870781427])+(1.-msk_ref)*numpy.array([18.143355631338757, 95.845967921420723])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.979581456121942, 0.93684336502414112], [1.3268298414524893, 2.391387496736658]])+(1.-msk_arg0)*numpy.array([[2.1066450822592442, 4.9590171283770568], [0.036908756602782263, 0.011528301135626801]])
      arg1=Data(numpy.array([0.76910485266523076, 3.7361325184859568]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.3156617042389365, 0.95106224373463266], [2.8764311069297706, 25.982827323802177]])+(1.-msk_ref)*numpy.array([[1.7736817437526022, 3.426353258583966], [4.4320598015843084e-06, 5.7345369442841937e-08]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.21704527252671296, 4.6452992882816933])+(1.-msk_arg0)*numpy.array([3.0137046016971931, 4.6327955934832383])
      arg1=Data(numpy.array([[0.15850666637550914, 0.51191736852894865], [0.90325384635282824, 0.81502997808203903]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.78494423589897722, 0.45747629083114721], [4.0038950951240526, 3.4965211641074516]])+(1.-msk_ref)*numpy.array([[1.1910792309456006, 1.7589761869154379], [3.9941592473765359, 3.4888485629309081]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.0670664791505668, 4.8503566777028704, 3.5696552886566142, 4.198670124219384, 4.2499995875398469], [1.1592921411961203, 4.5673711154464796, 1.4880543846460101, 2.361837837636378, 3.458138374255646], [3.3651944446353168, 3.9279533153821258, 4.8893621165714114, 1.7504970862547022, 2.8500036214291646], [1.0673938884675878, 3.6032348497610638, 4.6571907656209799, 2.1242796546510219, 1.1875289756247367]], [[2.4777624926204855, 2.4601977035595888, 2.6952284064560978, 2.791655155242339, 0.81432795668269853], [2.9816785610730929, 4.3553363430290011, 3.2692579385588751, 1.2037455977817622, 4.5798492698140381], [2.0576584064643804, 2.5457009677617459, 1.0723983672056099, 1.4145300755808954, 0.65450381091046173], [1.7399842058142052, 4.5192856391866494, 2.0121538939775401, 0.75646849730767829, 0.1539162311748076]]])+(1.-msk_arg0)*numpy.array([[[3.4886306900852673, 3.2337518263670733, 2.0230752848152278, 1.0449822144078205, 3.1864580868686216], [1.5591262983241359, 4.687641376085721, 1.6212848776716777, 2.9503611664622937, 3.8406728807646853], [3.7672920676421393, 4.4643548338906704, 1.8712601343149049, 3.9448081355100468, 0.47988597076099904], [2.9236455770903786, 1.6042221886463046, 3.9590079309529105, 4.5248003327612407, 4.5206980680834468]], [[2.0455275632580232, 4.6702499681708627, 1.0842385248732407, 1.8493203054089822, 1.4618504086389914], [1.0117450370468095, 0.85376072333859143, 2.6177289051359525, 1.4896548209721843, 3.4847410585052367], [1.5544430074610742, 0.41383049962160617, 3.6015740705933732, 2.2690527620259413, 0.42325990139153613], [0.43440729520109239, 3.3598288057123731, 2.3102342453680693, 3.7602055942104018, 1.6214398901513447]]])
      arg1=Data(numpy.array([4.140346842687868, 2.1514650377693396]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[103.56257036714585, 690.78178161749952, 194.11740269761401, 380.10080288637658, 399.71270070520865], [1.8440845824328684, 538.57635993811971, 5.1844384109123549, 35.106276843731266, 170.21361433714026], [152.05645654777638, 288.4394352178511, 714.07393932738762, 10.157164883123841, 76.421888382740775], [1.3100085533983949, 201.79031649725141, 583.80150259536106, 22.63444688522285, 2.037290408993413]], [[7.0437772323417613, 6.9367861275610636, 8.4413386874007497, 9.1044981124545945, 0.64281782202586002], [10.490250808469121, 23.70461162770853, 12.788495043836852, 1.4902791208492161, 26.411813350865163], [4.7229315699779022, 7.4658697103124059, 1.1622784422328709, 2.1088070850618865, 0.40173668404814794], [3.2924926650459816, 25.666092424867443, 4.5010769851079049, 0.54855824196336556, 0.017843122014008533]]])+(1.-msk_ref)*numpy.array([[[176.51430360979003, 128.93252034603387, 18.492515962706463, 1.1998237901583342, 121.3027256897786], [6.2892009166335008, 599.76876481686611, 7.3941868718450152, 88.195460218815157, 262.81430332065315], [242.63925111910828, 490.03421660977824, 13.388435555781081, 293.5985401907941, 0.047841076949577857], [84.935638911719622, 7.0772787763210117, 297.999028026556, 518.09450666369753, 516.15249564072258]], [[4.6632297591009184, 27.546211634236862, 1.19006272718543, 3.753766723324282, 2.2635124938732609], [1.025440006864798, 0.71165937299418514, 7.9277566895759488, 2.3571526955855453, 14.67104002614597], [2.5832503208921769, 0.14983283449936521, 15.749756252521514, 5.8289080461382055, 0.15727449057780327], [0.1663214612726627, 13.562912043767048, 6.0588919393777276, 17.280154764578231, 2.8287485674969055]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.7827969031335402, 1.4081770725012335])+(1.-msk_arg0)*numpy.array([2.2189553310370416, 3.6503039411574676])
      arg1=Data(numpy.array([[[1.1641940006068805, 4.2215170109341029, 1.7571301868177003, 4.6685243083816657, 4.4885296546460065], [4.5905507447131688, 0.31687213055275837, 3.4663522978209191, 0.44571335132378692, 4.259220085401993], [3.9646737801740057, 3.4759197064879706, 4.7952861371191764, 2.9960952449414977, 0.019184323012488527], [4.9681080779716886, 3.1752712601167321, 3.272602627347406, 3.6217185053915184, 3.1712355466722753]], [[1.7442626877802336, 1.4927696657986944, 2.5598499106998189, 3.2724986494485568, 3.070638293279242], [3.1888405318980491, 3.176076454255369, 1.6338636216810767, 1.60957371061563, 2.1604948982409002], [1.0490291485895713, 0.38269193061458984, 4.4179342118519109, 3.0018221856141176, 2.1486790800081721], [3.6511708088841455, 4.4557541204620517, 3.8691785827580594, 0.58890744805338358, 4.5163002425283523]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.7063742254966305, 274.94606812305432, 10.35840072806173, 498.34964367480319, 392.22104888125472], [449.24164763683581, 1.5243786032964846, 100.67079686268343, 1.8094183057637803, 289.08984809927603], [195.36202549448907, 101.96043499515031, 589.90053190498156, 53.849646431014357, 1.0258525721493033], [742.39930418163226, 68.345998298669102, 77.795132198717013, 123.78730976716048, 67.980006792226192]], [[1.8167590303687045, 1.6669058154842464, 2.4018191144409711, 3.0653545371451987, 2.8607025476479184], [2.9788204366558868, 2.9658340834800314, 1.7493862307216808, 1.7349015381392821, 2.0949481143302919], [1.4320092006603335, 1.139960852040065, 4.5368877529103768, 2.7941047788579714, 2.0864921926117037], [3.4895793726752098, 4.5960023158643963, 3.7599452527070563, 1.2233349202928794, 4.6922469426111606]]])+(1.-msk_ref)*numpy.array([[[2.5292052610169256, 28.924911116635965, 4.0572155690975737, 41.305032219894265, 35.784757221253912], [38.816156349893376, 1.2873150054815941, 15.844292646884812, 1.4265369809220416, 29.807317802488416], [23.570354732785304, 15.965576314774811, 45.696337117561889, 10.891659449598986, 1.0154081053583324], [52.444923423632112, 12.563642596791581, 13.577089657886962, 17.932987438698863, 12.523295171922346]], [[9.5686251312767343, 6.9091992020011501, 27.509196807916378, 69.218308549625817, 53.297770782494041], [62.112297605790161, 61.09420013183864, 8.2940937725133104, 8.0372970390785081, 16.402460277022069], [3.8895516305729676, 1.6413408601035357, 305.02304210606206, 48.754168036876159, 16.153424906779133], [113.02104787912128, 320.33167198811424, 149.88267276336143, 2.1436789947214256, 346.4549115196794]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.5854089122660433, 0.65602353100268052], [3.0366120185356156, 4.4736306365869858]], [[1.5511098354519113, 0.680149436298081], [4.3182119960485492, 2.2954727648237321]], [[1.5427565019455118, 4.8968805131250051], [4.2377845798564548, 1.0732689234334383]], [[3.5145829300058162, 4.4818656957028882], [4.3146096962019564, 3.1591153301417334]], [[3.2895576690610455, 0.43922344817971093], [4.8028421626172975, 4.7529226333075343]], [[1.727012055308091, 4.7026303315824975], [2.5142351567388159, 0.58452407227143]]], [[[1.8643474122768213, 4.0255541566706015], [0.15221840771205353, 1.659156937891483]], [[1.3625627661938366, 4.6012230130928105], [1.8068078021992153, 3.3775519252783899]], [[3.2333833797209635, 4.9176918090199875], [2.868502057448604, 2.3743593135346495]], [[3.5880067406560823, 0.87180030206458581], [3.8615725170447002, 1.9934589393342339]], [[3.7917047005719846, 3.3063534919874269], [0.45752152806069846, 3.0576085620134301]], [[3.2011046926101248, 3.9606900836296615], [1.9272172664796576, 4.0276841743991767]]]])+(1.-msk_arg0)*numpy.array([[[[1.6759498425850772, 2.4415635850784425], [2.6487046586362708, 3.9129959483891916]], [[1.5758385963907506, 2.0417975116635292], [2.4363050419660661, 2.7478418335915098]], [[3.7527776923374359, 4.9047686298756714], [3.6106973699676206, 4.6396414342422956]], [[4.5410691509479797, 4.8680193391162341], [3.2472406537327694, 4.6132171275261262]], [[1.8954740939456978, 4.5031113364310889], [2.1840324446560282, 0.8798872316854115]], [[2.5907041759670477, 3.1159103348875723], [0.69048173938170399, 3.8332310996778856]]], [[[3.6773341931297665, 0.20333879710157812], [2.8068161064318216, 0.1897789887934864]], [[1.7554189770018807, 3.0417912917760996], [0.65594813718654765, 1.2690355183108213]], [[3.1375325201718844, 3.9819457601364681], [4.1877232229672687, 4.4688579347947819]], [[3.4649941535446893, 1.237317226888603], [0.45516916429830301, 4.7749505676833914]], [[0.86774553236121121, 4.6877492306795832], [2.3333175743566001, 4.8793609581525006]], [[3.5838667812821794, 2.7921392560130065], [3.1612788740209914, 0.96996694515093107]]]])
      arg1=Data(numpy.array([3.791652563633729, 2.7416873241370623]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[321.89620564000927, 0.20221903786466996], [67.460980792866152, 293.14137920956119]], [[5.2826315376629447, 0.23189630351502435], [256.36095863191412, 23.350801458647464]], [[5.1755709216636365, 412.98575106890968], [238.72215610586281, 1.3074833571330016]], [[117.42586706048517, 295.19267337398105], [255.55102285859141, 78.375213490530129]], [[91.370445890015787, 0.044176125685497905], [383.71151408376573, 368.80766999627423]], [[7.9385574995923616, 354.22798790599506], [32.976156669679341, 0.13055566672607558]]], [[[5.5169623450704401, 45.524258720785021], [0.0057356232392854097, 4.0073998921859921]], [[2.3354122820444552, 65.673668766492526], [5.0625841683588018, 28.135944414804907]], [[24.964374520792422, 78.812156254845021], [17.978247987220886, 10.706090712149626]], [[33.207342615836481, 0.68650244088142875], [40.618506938932995, 6.6287176688679725]], [[38.635205848337407, 26.539537312551971], [0.11720728537579977, 21.417331745384548]], [[24.287022240623809, 43.541239937046392], [6.0421406141187459, 45.590330872093034]]]])+(1.-msk_ref)*numpy.array([[[[7.0846738219441354, 29.505468430116377], [40.178728506410174, 176.43704630875789]], [[5.6091347047659568, 14.978224501866322], [29.265241020983414, 46.185337578647868]], [[150.57276032190734, 415.51384823096311], [130.07435602213354, 336.57151144500926]], [[310.25247360274471, 403.8323358150783], [86.99317312932321, 329.36092496635501]], [[11.298209146440069, 300.53360884401354], [19.335374225094778, 0.61558290544557293]], [[36.943391013573006, 74.387959755312068], [0.24553930248076705, 163.18331059350021]]], [[[35.523433170257526, 0.012686920793029894], [16.93801575922765, 0.010499824546673737]], [[4.6775206283222177, 21.114936779833172], [0.314712097792756, 1.921731655706103]], [[22.987390407696875, 44.184890900168028], [50.730497017263616, 60.622750339492399]], [[30.178352008681351, 1.7928920797689945], [0.11556246282007576, 72.697649898466992]], [[0.67778380349809764, 69.115349981749432], [10.206321534384204, 77.139348377295676]], [[33.102398426053142, 16.696291746576147], [23.467537872927082, 0.91979629348181491]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([3.7144303704226664, 2.301404821282881])+(1.-msk_arg0)*numpy.array([3.9994954778271365, 0.013065605167019089])
      arg1=Data(numpy.array([[[[2.7310930696775242, 1.3579285858664039], [2.8892601641246647, 0.81523682058966884]], [[0.65588040533951286, 0.17008695966248324], [4.377298448077668, 4.020875812013978]], [[2.403478824721371, 2.5366522403205143], [0.12175134438863353, 1.7312639492950785]], [[4.9733141822253621, 1.211137281696278], [2.5521890722872218, 2.5539660589332032]], [[1.7784490465976126, 2.6019961393907316], [2.2092010538873859, 2.6612772751896028]], [[4.7405711968432422, 1.3133868631207264], [1.4427359183676587, 4.9756720475933802]]], [[[4.5955747364877668, 1.7069230774017248], [3.342221109480163, 3.4117351971817618]], [[0.088347097783672376, 0.38243939492310364], [2.9809047689525547, 4.1675303785924349]], [[4.8389604298702364, 3.6949304090270205], [0.52110351588237325, 4.2454964546334271]], [[2.1781937231900796, 2.704403076172559], [4.2205417176025746, 0.43370866660239893]], [[3.8658245536746025, 2.9205753325851798], [4.9187926202436882, 1.0133509222650914]], [[4.3992482420698833, 1.9721157750255374], [3.1385366300292667, 2.860104464899774]]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[36.010462304026582, 5.9411831215903534], [44.316646951199814, 2.914721891540327]], [[2.3647277144861114, 1.2500610841841933], [312.31175961398134, 195.64369177857353]], [[23.427388900835577, 27.900910801982448], [1.1732353613032351, 9.6969210176818503]], [[682.73641611074424, 4.9002441191179607], [28.475587978139028, 28.542065026229388]], [[10.316307486537937, 30.398864426256772], [18.155428211510102, 32.858007066437203]], [[503.05405149091723, 5.6038816827323945], [6.6405488784212441, 684.85210851857869]]], [[[46.085676923429034, 4.1485300213536975], [16.212900155100641, 17.180046010956204]], [[1.0764182010807499, 1.3754360121008298], [11.996835818475553, 32.256474700959288]], [[56.450772938832891, 21.753994389535354], [1.5439592534398472, 34.422315192882976]], [[6.1445618611854886, 9.5274139061526384], [33.713716956810302, 1.4354878117754402]], [[25.084271889643507, 11.408482501756794], [60.334898761560254, 2.3271584767003684]], [[39.12885598884337, 5.1747829955953186], [13.681330951613765, 10.847704563176228]]]])+(1.-msk_ref)*numpy.array([[[[44.068904757545972, 6.568709922039087], [54.87185877056838, 3.0958279256498704]], [[2.4822380077584603, 1.2658820330895579], [431.67457668130942, 263.38327157900665]], [[27.983805576135218, 33.657190806890846], [1.1838452710094622, 11.021226238916222]], [[986.19103225183039, 5.3593356050585559], [34.389916283917678, 34.474730022775717]], [[11.766182815679798, 36.848111307992255], [21.377186420728179, 40.003941647744739]], [[714.24722087881526, 6.1754091144187377], [7.3881301875993612, 989.41957029465129]]], [[[2.2005561834326867e-09, 0.00060866456641342991], [5.0546180932853012e-07, 3.7388049632550169e-07]], [[0.68165639178070514, 0.1903416007418838], [2.4230452905526599e-06, 1.4090051708419818e-08]], [[7.6564127997998437e-10, 1.0945351515127807e-07], [0.10430580390350931, 1.0046958273939586e-08]], [[7.8807067446954115e-05, 8.039978840874019e-06], [1.1195566933919465e-08, 0.15238739688371439]], [[5.215395898220247e-08, 3.1478525089200333e-06], [5.4154161355261087e-10, 0.012330427153605552]], [[5.1568695624944987e-09, 0.00019265901164491864], [1.2229354198774389e-06, 4.0919814609424385e-06]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.0220017736305547, 4.7400950842066871, 4.2172198298049386, 4.7901692066481232, 4.2932055519569223], [2.6949455124227333, 4.1115473541623926, 4.1807935899783057, 1.0815048397313682, 4.328398772184836], [1.7511382412927314, 2.7535584426769839, 0.099188651989289667, 0.32632971343175016, 3.3338247034087694], [1.506034208349291, 0.87471070131263684, 0.91037361520195836, 2.4397368553842185, 2.2392788512643129]])+(1.-msk_arg0)*numpy.array([[1.6447413752503068, 0.91556391966544526, 3.1360334669288026, 4.795139247162977, 4.5239893858471447], [4.7096810312150517, 3.7887507489612209, 4.6673650988874087, 3.7217508257930723, 4.3495909238995223], [0.75417514082722248, 2.5183663650460115, 3.7336551462152521, 0.094781334985577823, 1.3423733944248419], [2.5958359941575222, 4.7063378748544196, 3.1623413985117179, 3.3993330655622414, 0.69930835193257401]])
      arg1=Data(numpy.array([[1.634134409036945, 1.589095216372604, 4.1601674741713319, 1.117395744535169, 4.7034766174399314], [3.8651274089999892, 4.0050608743663236, 0.20985250149812407, 0.13625343006970886, 3.2325923343327156], [4.778670615263434, 2.1296195226146475, 1.0769329649371624, 2.9638474714358223, 1.9803030231059069], [1.3015093497789452, 3.2347470609049536, 4.3170986346243714, 2.9847058616084308, 2.7366613490130018]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[9.7216479900742225, 11.854683786297445, 398.30323517816834, 5.7573303518883732, 946.83400444807569], [46.145628229642114, 287.82505071818741, 1.3501210168918927, 1.0107331161811306, 114.02147098682755], [14.546154619926565, 8.645845810935926, 0.083034167041967921, 0.036186985108139752, 10.853882836940519], [1.7039376635035162, 0.64855403761034391, 0.6667257982012359, 14.325338180711157, 9.0808712857623224]])+(1.-msk_ref)*numpy.array([[2.2549273427892502, 0.86919986137624727, 116.15202775006613, 5.7640055425945596, 1211.240576190447], [399.20674763992236, 207.44923453282527, 1.3816765461280174, 1.1960966566305147, 115.83597225728006], [0.25970485288643752, 7.1487590993668446, 4.1319036229665489, 0.00092717672643228936, 1.7915459590798128], [3.4608734118731759, 149.9545642068409, 144.0744953146563, 38.552624384751212, 0.37576028480274964]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.5999976314665494, 3.9425738004235691], [2.6227241995695354, 2.3421521295019607], [2.3610034662371238, 2.558226426688528], [3.6101452078445799, 0.64527299031482743], [2.5839695472861592, 0.96110215198399307]], [[3.7368033855368203, 0.86241416884632738], [0.37481617726890354, 1.8606516531583992], [0.91208801839134379, 0.4800777172801145], [3.7265702535146548, 3.8016506921969806], [2.0848997585577549, 0.12101017899002886]], [[3.5189776017199992, 1.5567495778534042], [3.3272222496175496, 1.2466112103376743], [2.1907184048672423, 0.92744342308447947], [4.0600258623966896, 4.9450987504415949], [2.4298615296328707, 4.6840601832102831]], [[3.6573181417168592, 2.2444589199587868], [4.3883971642775395, 3.433672843677563], [2.9593747707258804, 1.0364903102373204], [1.6056193187311594, 4.6932404113070145], [0.74907692806670578, 3.6285541432445032]]])+(1.-msk_arg0)*numpy.array([[[1.2037159843662459, 1.3356614206821678], [1.0233313600356582, 2.2727008885500948], [2.4474703556669328, 0.54244827767044412], [0.33158768693260332, 2.1002640857722343], [4.1462079313761446, 0.11859497960644819]], [[2.0921699802235509, 3.2119293456444571], [1.4699649393845979, 3.8581446860730626], [2.5822291685066503, 4.3442271281699769], [0.92904281508882303, 1.389954975212512], [0.92942988534975179, 1.0765835706730225]], [[1.906091225321668, 0.44571022806387089], [4.279000009327957, 0.86156627746630632], [4.869857747971615, 1.1177591992450266], [0.85982056645102378, 3.8690767845832097], [2.4184458866640202, 4.1421444630139739]], [[1.1204198269115662, 2.5464336816521795], [4.0152977158611254, 3.834948856341768], [1.3620224342011154, 0.1034426713568648], [0.56039135552598995, 0.9734348621988802], [0.3600885834172558, 4.3656295529836706]]])
      arg1=Data(numpy.array([[2.5080461081751215, 0.34463577575571397, 0.55137678131214096, 3.6738629934820004, 4.7273745014204867], [2.8389694955114382, 4.6271127965831846, 4.1055186108330179, 4.5893829577268068, 2.4651160547019035], [2.1236066551706712, 4.8610737154460999, 0.30841447138922895, 0.27600046264729722, 3.3313018860994474], [3.6226951782353987, 0.43824956143148863, 3.0289440404390233, 1.0012923282910173, 2.5251515120748143]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[10.984272492094002, 31.206372476683406], [1.394174500568375, 1.3408573804955177], [1.6058937801822588, 1.6785260367522521], [111.75558981363004, 0.19999707185010498], [88.927667213958671, 0.82898220580482507]], [[42.199833702521289, 0.65690013773461842], [0.010666190284487936, 17.691746617517897], [0.68537796254660599, 0.049160764986281649], [418.75231851249907, 458.89632161455557], [6.1176129664581955, 0.0054833952165128036]], [[14.4668414338411, 2.559746930073552], [345.04693300134022, 2.9198166725308425], [1.2736257262932027, 0.97703690580169045], [1.4721573457710571, 1.5545067511078672], [19.252558961926781, 171.41384133030203]], [[109.68997656820351, 18.705551677021738], [1.9120105763674524, 1.7171020589391899], [26.74473702990819, 1.1146698653077038], [1.606602145700692, 4.7026273443891835], [0.48212561043348778, 25.906704175833617]]])+(1.-msk_ref)*numpy.array([[[1.5920546594685676, 2.0665804737512543], [1.0079801264853088, 1.3270192591294081], [1.6380598705470988, 0.71372573013781671], [0.017327830267218524, 15.275315085187799], [831.51092037220246, 4.195315813584861e-05]], [[8.131394049542978, 27.459534330521208], [5.9449506158674534, 516.70124574612794], [49.141934340085051, 415.87396825985729], [0.71335173088102655, 4.5319501504994699], [0.83493047182340585, 1.1995032194287889]], [[3.9347304631905264, 0.17977353833889101], [1172.199141239508, 0.48465596379491033], [1.6294459855729908, 1.0349307735597473], [0.95917208401141196, 1.452713258042166], [18.952890233402378, 113.80559428032775]], [[1.5097030535334646, 29.550799802471161], [1.8389886122908754, 1.8023219769046785], [2.5493919943323995, 0.0010365269812504004], [0.55997210824498622, 0.97340099200370012], [0.075834426238672412, 41.325236594608413]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.1399977331269051, 3.3517143002139203, 1.1393715976381773, 4.7100262560999111, 1.424698762438614], [2.8216965630368134, 2.1229188783917778, 1.2484550941665222, 3.4747183521132454, 4.2823368607259136], [0.82608361049672396, 1.4843549377809715, 2.9604425611069396, 4.9161344427559088, 2.7928354116502088], [3.9482221432738944, 1.9860067655398437, 1.0408494265295922, 0.65440091147710333, 4.9104916628815243]])+(1.-msk_arg0)*numpy.array([[3.521957876859712, 4.5204586168051373, 3.3804680044327706, 1.132419006619966, 0.56687241227747753], [3.5481402892355636, 0.77765783346337036, 4.0379219228915275, 3.1348102225491594, 1.0623690394851049], [4.520018405093996, 3.2577272164219337, 3.2913113745186475, 3.6211616784967093, 2.4991955396360916], [2.7351726614412337, 3.1887578448293552, 4.2522745710497674, 0.12410731765750825, 4.5364354869870382]])
      arg1=Data(numpy.array([[[1.740511239659863, 1.7328393694485935], [1.2759264907873382, 0.6660760455967466], [0.80658776195762982, 2.518009590291951], [2.9483267519442364, 3.8787172054817161], [2.8933735682001878, 4.0211164115643383]], [[3.8290916808211737, 2.1914286726239927], [0.70760519461808724, 0.48046992575746922], [0.96995138343361809, 3.8164388900807671], [2.3405001388761231, 2.1976230405903814], [2.3204439046318344, 3.6617580710331739]], [[0.79011399386577619, 2.5614807328372335], [4.2349721440001487, 4.1670502128821694], [1.7122438631973953, 2.7261075192252684], [1.1845475038268156, 2.7121727106843023], [0.62072384089801447, 1.4969472322389075]], [[3.9581624965012669, 1.7159480874993109], [4.8765609912346015, 1.8238189601868511], [0.22879005881334186, 3.0592326743167821], [2.018934943298317, 0.67595890008837745], [0.34740088099079064, 4.6236614867804802]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[3.759136410378686, 3.7372590091523197], [4.6795282156091993, 2.2380395713187986], [1.1109783848117898, 1.3889416244560999], [96.447879336907064, 407.81872492314585], [2.7846986300530316, 4.1508617354637067]], [[53.09397940074264, 9.7109386145917398], [1.7034907178856329, 1.4357593923176397], [1.2401580957283576, 2.3323911339793368], [18.451106590251488, 15.443203479211379], [29.226709640839182, 205.61858543759988]], [[0.85988328290934413, 0.61299733958364366], [5.3266926859281023, 5.1856889791611156], [6.4132402999282947, 19.27386792016026], [6.5957044665801936, 75.128213875526797], [1.8917815176957875, 4.652713821744439]], [[229.43219831682569, 10.553442551928852], [28.38710949954357, 3.4951264568055094], [1.0092021808138321, 1.1302998085613649], [0.42481594166767273, 0.75078924404113367], [1.7381907171813948, 1568.6568252311024]]])+(1.-msk_ref)*numpy.array([[[8.947134616476534, 8.8611301333906827], [6.8543251962092837, 2.7315010215344229], [2.6709529465967781, 21.476758339113371], [1.4428816993453213, 1.6198636574726777], [0.19352672453755701, 0.10203185493289424]], [[127.64488681665173, 16.043070271771381], [0.83699220123214313, 0.8861906901270068], [3.8720745505320848, 205.76175177929738], [14.500484374354979, 12.316442552148231], [1.1507224996718397, 1.2479988310294274]], [[3.2933314575700954, 47.657253240201392], [148.65500474275134, 137.19597711167967], [7.6888580837513185, 25.727846808452131], [4.5917887914467324, 32.786274185079016], [1.7657291676758047, 3.9399069394491217]], [[53.66073746683152, 5.6213825450204249], [285.7203388811285, 8.2892361725313517], [1.3925868954229199, 83.771987386089052], [0.014805936535883758, 0.24403059565710208], [1.6909987202767509, 1087.4934430002986]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.65819128962902962, 3.946635400060607, 1.1874374145549591, 4.7872867947134843, 1.5974763771726421], [3.876183944072253, 2.115845877429225, 0.45844739853526018, 0.52243077089733336, 4.6293332248155146], [1.9731677478554577, 3.2845663178081703, 3.5045757244728697, 1.8525975120444018, 1.1809128201661203], [0.17535032777945125, 2.740389606028014, 2.8159299836920919, 3.8331666734530545, 4.4485591840430665]], [[4.3606223620086562, 2.8065931476582358, 4.5003469650508245, 0.59974591361098906, 4.5262986506564244], [3.2304100149412194, 4.9330333557323653, 1.3266852794257653, 2.6941938839485031, 1.1300844782294137], [0.73201230993286093, 0.84898176017703386, 1.9700206002027469, 0.96782891044775965, 0.14563436418993925], [4.1131873633694447, 1.01371213623404, 4.460132768765444, 4.8640953564472866, 3.3426716150408198]], [[2.0799521391835212, 4.0470348888969543, 3.1369014756229872, 4.7614705242284243, 2.7442517075826767], [0.42997939986738998, 0.6846987683132153, 0.18230858707330969, 2.2428224904478595, 3.7406584922924559], [4.8411330401150341, 4.9486329936461759, 1.9513495619089336, 1.9911501229456117, 2.0972669498193097], [0.76399946649811845, 4.5453967911323776, 3.0708830407862435, 0.58536097008041788, 4.9071892953727252]], [[1.2640186962508815, 1.4353356472684238, 0.83242243018195183, 0.42113749644026471, 0.80213264215807989], [3.3110056288430383, 2.9133580451557743, 2.1406764045655855, 2.2875906596365341, 4.1983433555503975], [4.2835316956603799, 3.1913439600029947, 3.8135235878907574, 4.3950934973579967, 4.7491250838299646], [0.88691062660540798, 1.6791595546642215, 3.5985172621624208, 3.9252481725861079, 2.6384163685727069]], [[1.4380722138576609, 2.259417192160476, 2.5551140142188347, 4.7073025064497491, 3.0540888424943993], [0.88572154158169136, 0.26989698038328747, 0.48313768817437885, 4.9263854358597809, 1.6488602175188929], [4.1715865063892252, 2.2631771759343979, 2.6117249960389897, 4.803509348741426, 1.8720605815761804], [1.9628895811251217, 3.7072617989476964, 1.5266970916357263, 2.9087577203290955, 4.4637245137695718]]], [[[1.5353172093574274, 4.7847150918223376, 4.9033726955507557, 1.894859815052625, 0.13232043560216553], [4.6632933231358971, 4.3019470462905955, 0.73164011723992484, 3.3103466632195953, 0.57958366014272678], [0.56251029817972986, 3.600325751946603, 0.17436887169310425, 3.8929366168838579, 2.7787025416051954], [1.6896637440860798, 2.7037976089588929, 3.1531045893524361, 4.3611303771274628, 2.248318584122583]], [[2.0290456713423688, 0.92167409903756781, 3.0783846714872851, 0.84140199422183415, 2.3367321521695126], [1.8823417127840769, 0.32615316566634461, 3.0733433750778398, 2.3216976589695371, 2.3794719506628055], [4.6536202121879819, 3.4358182027223565, 0.17490465886782849, 4.7499082964795205, 3.4757279584678433], [3.2224929674137948, 0.055805532152862013, 2.9293153554384181, 2.1336348477884122, 4.6090287968066432]], [[0.23264016469757004, 4.2131941703455045, 4.6755674985230531, 4.8053383259188633, 2.4870374387030827], [3.8402232570987311, 4.0867962627708616, 4.5679716972092859, 2.0152353029421008, 4.4801858569606123], [3.6197489074648934, 4.5287988328938527, 0.93427669519722267, 0.58479395896429553, 3.8418092153722756], [4.3722986783216786, 0.98626026879648798, 1.5516579518050377, 2.0084248990409881, 3.7977616885186993]], [[1.8337992096308535, 1.9096498171498153, 3.0785582813016892, 0.5933548377626181, 1.9377657325472397], [1.095528587284982, 0.012556245207298254, 3.6413175399786701, 3.3071804422664632, 4.2348725273679397], [1.7535560119974531, 0.56754601924377757, 0.35292161172447917, 0.96464087883893435, 0.76028667575677433], [3.0722115441888356, 0.034976641716845232, 0.59868214575710066, 2.0601575547859845, 3.6242239989674223]], [[4.1942401779393927, 4.1204345555104194, 1.0981547847710278, 2.119523904935126, 2.7943755370726646], [4.6106435843440066, 4.8794023246577627, 2.6669015620667382, 2.6416788457965881, 0.66937921164116088], [1.0680574271358856, 3.6923166339506022, 2.7640855863058111, 2.5106811100234427, 2.3784077083571251], [0.66562467661145153, 0.8495159631849728, 2.7536598759398405, 0.75661338702837933, 4.0002845383905097]]], [[[1.7807901067361476, 4.1853730258007005, 3.6908656940144149, 4.4186825840361443, 1.751974791090728], [4.6233149928072486, 2.2015866057477531, 2.0315479441213835, 2.7204099514570368, 0.032561831446835998], [1.96559988118165, 4.4477228411118421, 4.9305064633986468, 4.880836151378519, 1.8853777773240545], [2.4021887910650284, 3.2967950797765213, 4.9074657440291256, 1.6032404856381741, 4.0820546577239032]], [[2.3988985611009381, 1.5634578806778139, 4.6043449580554423, 4.8253296414046991, 2.1710945665381329], [2.8995331989803335, 3.7521918173475952, 2.2386531449875995, 4.1903060507284211, 2.1171797570669844], [3.8767101082755566, 4.141973830655302, 3.6856260324821228, 3.9903698313609248, 1.2742760336588725], [2.6046453982112303, 2.7155822293560821, 0.67889687962144585, 1.1086828316654502, 4.1409586191998642]], [[4.9866413065797177, 1.3872063651374771, 0.16709710018800503, 2.4584407307800697, 4.714174824728337], [1.6561060599103701, 1.8943317059648528, 2.9173307070203962, 0.46726037668781811, 1.8487811322319958], [0.88570158279570543, 4.320119321571477, 3.6671406896318519, 0.26523076623322911, 1.4256638205471333], [0.1280952257098221, 4.2971314148710373, 3.1853143940282878, 1.7408751881336062, 3.1535412222215018]], [[0.61136516508966687, 0.44356307963081776, 2.9429614363861836, 1.4082962294123333, 0.29364106491299924], [0.67286288927572513, 0.034036767244404617, 1.1572582414331571, 0.68230909010251584, 0.21330021101915964], [0.58779511890170411, 3.3241345341814252, 3.1380269492061692, 1.9565169568571068, 4.9396053655666288], [0.020418566010952374, 1.2511787037456994, 4.1892288815673657, 4.7686110576292764, 2.2006683797361859]], [[3.055438957254963, 3.4607692377698984, 4.0553866994998176, 0.21225199904170322, 4.3110854201443711], [2.0796818113193711, 2.7981571907082858, 2.7287863796883292, 0.48023570370736474, 2.2374331422997398], [4.4572108495324692, 1.1179203430688793, 2.6708454844489538, 1.6358418007080657, 1.0195816991242843], [3.9201321042185517, 2.7430149973462319, 2.9877333649250004, 3.2513819203301999, 3.407112755018542]]], [[[4.8425650656100734, 3.8803429103812452, 4.160237920708167, 3.3462238185499968, 0.45172118180029258], [3.8343322947383132, 1.5144973458216104, 3.5816610863237113, 2.3532664982404734, 3.5233131095826868], [2.6087798288443018, 1.1136597638896524, 2.6210854181772958, 1.4655633750007642, 0.42791684388145007], [0.61601986456788238, 0.21169252788025555, 3.7170974696967889, 0.92852484243146372, 1.1803659280529435]], [[4.8408065264059701, 1.8935136399368513, 0.91339664686148381, 4.1114912105816259, 1.4585740823967794], [0.70258767679717227, 1.0032204663017275, 4.0699791224400483, 2.6752866343649688, 1.4231886274142567], [1.9268076958843592, 2.9506186440901492, 1.1730711411460431, 2.5374957246219263, 2.1099943387436966], [3.0060849639643443, 1.6653092240199192, 4.9454795260311259, 3.7767145304139511, 2.8952954711847565]], [[4.115480862105338, 2.0633019634032381, 3.8238917812170978, 2.4398747577832682, 0.058184377247131137], [4.9846388494182667, 2.1765255675654922, 3.1709714361130077, 3.536107452481668, 0.91886295015625585], [2.3384727616454342, 0.98848779492478756, 3.4680639726643085, 4.2012041387627113, 1.2815344706084895], [4.2040447187795342, 1.9293355134162062, 2.006068510920568, 4.9695666948034916, 3.1331791241408657]], [[3.6272603429785764, 3.8415034341419565, 4.075365007055062, 1.1943153233545003, 2.3603352038294223], [4.9139649119947899, 2.1941827143555943, 3.4160781710180492, 4.1322540589340155, 1.5972270155611759], [2.78964378786746, 3.8805530588896349, 0.96120497258176629, 4.6583445205235234, 4.3315034623915727], [1.4007761474525748, 4.7408505309020548, 1.1506642952871873, 1.4791560424415309, 4.6829905639637754]], [[2.5533224596984843, 1.0222954988745685, 4.1241784910873793, 4.0789614896238975, 1.990111757820161], [2.8761900968428593, 2.114933966659283, 0.057135676978918769, 1.6316099971653286, 2.8694095572264269], [3.159866801845002, 1.592642246722008, 3.3336930932192619, 2.5597769863290329, 4.269313116636952], [2.5070345264515441, 1.9085255530110925, 1.0110814598554472, 3.7255233331332125, 2.8418631255921967]]]])+(1.-msk_arg0)*numpy.array([[[[4.1142892088016536, 2.9364619984189448, 4.6008779330981318, 2.0517792510737376, 1.7724376577140282], [3.2100837676070508, 3.0105877236639147, 1.6367964590363528, 3.6705731073881065, 3.5932886252896745], [4.1777041336371195, 0.28593345678024351, 0.043629089582010019, 0.89487241217035884, 2.1650514177180211], [4.2839581355515914, 3.4164725736870478, 2.5863353464822034, 1.5340312612931675, 3.0209698025676817]], [[0.13643199371092291, 1.7085472606246719, 2.2246200903200624, 1.5851843013775164, 3.1577929138042298], [0.61080128307774484, 0.84931364219070116, 2.3739922619936045, 2.1886471444115934, 3.9267623566467935], [1.4177239687909318, 2.4119506278773302, 1.1547109812097296, 1.6559044064676076, 2.9656299295137969], [0.99961550288150347, 3.568097859120682, 4.6386914714560108, 2.303091325536923, 2.061091940577604]], [[0.86379476104033381, 4.9816961350335562, 0.709742057293129, 2.9228166076878468, 0.99597617345213507], [4.5356098722523104, 0.62663269457104276, 4.9730909485172452, 2.3055729215115224, 4.8717731714798767], [3.2848191271568208, 4.7564285555024526, 4.2041902338334305, 0.91645664952739303, 1.0903817416077481], [2.3574786335289453, 1.597263797500091, 3.0820658754521291, 1.7287750555201846, 1.959146970936696]], [[3.8938324820399006, 1.8921156851121119, 3.1647684291179585, 0.73209483727865088, 2.739695177386186], [4.9562395844732485, 0.72544436262165723, 4.1240740966617491, 4.7332218222841753, 3.7634988014810453], [1.5340064190070086, 3.6294816522587139, 1.5664801491258991, 2.980643480176556, 0.65961257472927859], [3.8482436106760023, 0.40929630483578633, 3.9306655650033084, 3.4270549493025335, 1.8027577081740385]], [[3.4452296161177984, 0.6906434721403828, 4.0041342063019938, 0.075422249041262684, 1.6746927054522607], [2.819614765490893, 4.366588622224131, 4.0239972977835317, 3.6172117777222743, 1.5714478216002197], [2.7647827288333326, 3.8984185698980673, 2.323318771956965, 4.551303634011358, 3.8228962888460423], [3.2882925165170187, 0.88244823865569755, 1.7967310741469977, 0.24700777970805216, 4.6870050202973852]]], [[[0.75051120011781336, 0.52803845958206919, 2.4764989900982277, 0.43396806286576078, 3.3741556607861174], [2.7385610508573444, 1.895415939430795, 3.8522838374785762, 3.4368723118312845, 2.7898633147462419], [2.9807090637019953, 0.88877183402668303, 3.4573365859913863, 0.091601816475525649, 4.5754020884438749], [2.1540359195845706, 1.5203296991994983, 4.2490736929344406, 1.6133913196592853, 0.90965899998589095]], [[3.0103201812413984, 3.0058782645327495, 4.5228781907555025, 2.931725514204512, 0.33382675370904846], [1.974993653203476, 4.430772318279268, 3.5955790748948231, 3.4993872649265922, 3.0809269520017946], [0.88247272348868022, 2.8960930376334235, 0.52631747805161333, 3.8849497415037977, 3.1511016565769303], [4.2928403006035412, 3.8105091331220171, 1.9912428930644248, 3.7092985787700914, 2.4611961019283619]], [[2.5288119355194114, 0.4942956881768153, 0.14456877515708819, 4.4312121580185639, 1.923184088589315], [1.4011258515114342, 3.3601980946456877, 1.3027421394371956, 3.5196352127397854, 1.0641418222343997], [2.9482204101939251, 1.9535396968865057, 0.36686571032085041, 3.0424148833782905, 1.3386993884044915], [1.0343482539231379, 0.040298748964706574, 4.5356107764869051, 3.3383822587852738, 0.90892788124729118]], [[1.0405803661401396, 1.8482940321533177, 0.043712414710217237, 4.8982989335778901, 3.1842777875800374], [2.7527579574542798, 1.9021895145302443, 1.2985520787301748, 3.6728975699604307, 3.1150852653199474], [0.87788694157641267, 1.8266369328767249, 4.1122528046836058, 2.251783911074217, 4.3930940404860959], [4.5499588291952486, 1.8869297124201168, 2.7607173667621887, 2.4808644467573329, 4.1906583129115127]], [[3.8142815553325455, 0.76681433472476868, 4.4702463195962752, 0.98407249959675336, 3.7300778655963209], [1.181585439516708, 4.051478086828932, 3.16756856725867, 2.3882135781321039, 0.37208735489695766], [4.8981535956532829, 3.3768578110241267, 0.61875123225365947, 4.2687731081958518, 3.9085464300105963], [3.6205156955005822, 4.0918832876945688, 2.7102262533928085, 2.0568204267048444, 4.741444757060469]]], [[[3.6367634821353647, 3.6072238630981888, 4.717766515914132, 2.8957662751531617, 2.6238509451373511], [2.9026312741328422, 0.36157472059774265, 2.0773147860517609, 0.31847230636364271, 3.6646598173334306], [2.2967929472635422, 0.48372270999378714, 2.2280397574014681, 3.6861562034408095, 4.7739462001219506], [1.8050323792674061, 0.92879692607392461, 2.4054323191163687, 2.202870632065276, 0.91775635171169456]], [[0.24687897810078499, 4.1317421772383476, 3.5215579717665166, 2.9532603629661009, 1.511155362830515], [3.0128827168640924, 2.6832388580024751, 1.4760740103640935, 3.4156708201162291, 2.8715766060246217], [2.8371637640439995, 1.6466660203857879, 0.64512659293427754, 4.7677196347541297, 1.6816597646256213], [0.2479096868444649, 1.8628443929383329, 0.13786445099165995, 0.97446602730405063, 2.5896206825374124]], [[4.0909198386562515, 2.4573963627170876, 0.84501980860042858, 0.76528840162277378, 4.6432401913338266], [4.2321586766489272, 3.1537401986379465, 4.8108206435672551, 1.0113364868028223, 1.9822837646125249], [4.9025162618516172, 4.4384438613735071, 4.5496110878044611, 4.4260132318661718, 1.8460956153392392], [1.9651262828716241, 2.4957242079876929, 4.3565277355579157, 3.2054500280756528, 0.34062502143041995]], [[4.2247246064879187, 3.4099573130486469, 4.991381140836606, 4.0455759933383568, 3.6788562516535808], [4.5199494113123162, 1.8233351078205438, 2.1821257332091708, 2.5163934907035821, 4.7698221020330749], [3.5652493261249529, 1.6230534452008687, 2.6055590020432553, 3.8185193251272271, 4.8474774127635074], [3.0829379359575828, 2.3347134095366235, 1.7011247356092396, 4.7678974120693649, 2.5993217631538572]], [[0.64808206749526864, 0.25809750750276711, 1.2492782054275455, 2.7754373593121069, 2.9413704881108367], [1.8683928236518299, 2.7175350106054124, 1.5312297470138012, 0.88531073897886758, 1.0623484792194189], [2.9643708079318252, 3.134247988268462, 0.84090258202644663, 3.7589259406397355, 1.8581542693956836], [3.5882073837327577, 0.29669397836736394, 3.9254025019126009, 3.3635273581131004, 1.5993940145049255]]], [[[0.53031840123456553, 3.0054188413126841, 0.50663899247332433, 4.9861410154968171, 3.4636217501342164], [4.1397592981008451, 1.5523520836870293, 0.91648718531531126, 3.3857163994696711, 4.9732178172408519], [0.76525415747246051, 4.3406933563762982, 0.33262365396480897, 4.9883769361067785, 1.251544354252446], [4.2407395969732606, 4.3168094258977252, 0.86215422957358312, 2.1654117737946854, 3.4149634368254991]], [[0.35444391658481628, 0.71790853528433707, 4.416255879614603, 3.7341171934813673, 3.2755038313751923], [3.8024019176521651, 3.537414640866098, 3.3656851912680961, 1.7287585335045295, 1.5814980059019823], [0.93124162691233914, 0.49797976286682644, 3.5673262320254095, 2.372136619873932, 3.8483863927353519], [2.0398650859976364, 0.8617211987588248, 3.4339515845417599, 0.68527232996817811, 1.8765978409859767]], [[1.1454593067800214, 3.4846005321315121, 0.37715264456215447, 0.15360944631803325, 3.9056943429196509], [0.88870034537629405, 3.4537062265783813, 0.5669128844991973, 2.9499782020876464, 0.58237838173723266], [0.11049121063252167, 3.8381532690505984, 2.8867026160081801, 0.68949307452299424, 1.1550800391231455], [1.2273700289380938, 3.4055803019162441, 3.8850392606203337, 1.0016887760212549, 0.24282079234777967]], [[0.93000496045803194, 3.2404990395111386, 0.77748328411175571, 3.1941984114214099, 2.4920709887874999], [2.0565570867153058, 0.31140230963046411, 0.1869027924494841, 2.8762866523347839, 1.6164963141195769], [2.2618619286213746, 1.3342919235519071, 2.6449395839275636, 2.3957714537365491, 2.8536163713855225], [1.6034027351343529, 1.0739279808643531, 1.1189883445016029, 4.3188266951998688, 4.6443957468522949]], [[0.89829980388116726, 1.3420666950830913, 4.189520066919318, 1.8960795153951771, 4.498654014935747], [3.4648638113390877, 3.9274107607806563, 0.26816070473568376, 4.3535722531111292, 1.7328822837237727], [4.4227526483489878, 0.70295400154913257, 4.2469995270963405, 3.1816955551746235, 4.3105180031260701], [3.2227180685525592, 4.0590441699353059, 4.2926529305528742, 2.7554877711909258, 0.13896230844324511]]]])
      arg1=Data(numpy.array([[2.2425521931617092, 4.3680340077226694, 4.9281556931626183, 3.4803358100640276, 1.6719846435696744], [1.1060820503126851, 4.2084797258653968, 0.50917790431228538, 1.6425751652025926, 1.2494102925002815], [4.9616456874820072, 1.4933270979401352, 3.3416150234947648, 1.8383117389258465, 1.1378312229305125], [4.4878482361507688, 4.8442120234267563, 0.17253242929623416, 4.2797707331116239, 0.6458139969178337]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.39142194636737493, 21.730523172192612, 1.4700037870750831, 33.507041086939239, 2.8589881819822764], [20.870244116995714, 5.3692660552308205, 0.17394984807478908, 0.23316574749946906, 31.078485281555224], [4.5911416668974327, 14.395572800243379, 16.648435166361935, 3.9857774002729842, 1.4519520346049275], [0.020156774967012774, 9.5899636192801037, 10.192963537073098, 20.354414209208642, 28.422729127877396]], [[621.68372722395475, 90.711662235241548, 713.51110178758415, 0.107189835630929, 731.65883876867213], [167.66975323744799, 1065.4811003943801, 3.4375926129753425, 75.88008991906473, 1.7060441293731246], [0.25598333680790269, 0.48913148357200453, 19.331178315147447, 0.86689752372354478, 0.00022136456532555188], [481.6732672371877, 1.0612931828290812, 686.07756218675263, 1001.9555384072349, 194.65213149257212]], [[36.933153137556971, 981.89581449008369, 279.79092952482989, 2187.8304442595977, 144.75095952994556], [0.015616113096401398, 0.15463797509065588, 0.00022758436655848243, 53.551531503929866, 666.15799836582357], [2374.2450704516605, 2645.6440964481508, 26.965995268451792, 29.787297624787346, 38.473312218092055], [0.2653776689319941, 1740.2683221168825, 251.94705581625564, 0.071421355338564405, 2538.2339684170352]], [[2.2601452058139713, 3.5176350931028209, 0.52816495863598256, 0.049302516617968709, 0.46424216832657078], [64.511200843191062, 41.32820917450892, 14.139321165672566, 17.813826484794721, 147.40781720553491], [158.08223684007257, 56.754123131745679, 105.4899814551033, 172.88008478886522, 226.38235348741685], [0.65857373297210053, 6.0728929595956176, 86.19793193003467, 116.64254972169196, 29.269566406926003]], [[1.8357219425670481, 3.9072985232048918, 4.7993645403872494, 13.331058487548527, 6.4671762979919905], [0.81636004295592701, 0.11193652843602669, 0.29632713665075028, 14.38456819037583, 2.3074273975100592], [10.892665597077887, 3.9181763249120598, 4.9784744942334589, 13.789724201097245, 2.8530848583126818], [3.0882851739405228, 8.9422808477861349, 2.028766152864836, 5.9609008899884319, 12.197871341012213]]], [[[1.6067577230417616, 5.6490654925636532, 5.8042220608451638, 2.0277902568224015, 0.10676952358231935], [5.4907166175881308, 5.0221022309021688, 0.70778590924739071, 3.7585706680627267, 0.54699959765166872], [0.52920483876054081, 4.1243895264104466, 0.14487799568263518, 4.4967129236486123, 3.0968896731478011], [1.7863469246506056, 3.0046845669200049, 3.561603473615687, 5.0985778035616605, 2.4500991094644222]], [[19.644066934494937, 0.70945464232197375, 113.52642792834055, 0.48348043702394761, 35.586332036144732], [14.323954284750602, 0.0089586931360169653, 112.74605810305215, 34.632649554510905, 38.407048058467709], [646.22628649703802, 180.24869497999026, 0.00065064627817552011, 704.39465269526193, 189.22572797520107], [137.63080095478125, 5.3139398338745023e-06, 92.124636471839068, 24.271423658522398, 620.56431258416353]], [[0.47591549077626338, 2.0798803967984685, 2.1931323493552815, 2.2239180638722802, 1.5902767002964233], [1.9839987971983053, 2.0478715298218408, 2.1672877709940952, 1.4287490479074814, 2.1459793965881624], [1.9251595151330256, 2.1578043509150029, 0.96597699796472736, 0.76096207618382727, 1.9844159567842101], [2.1195090936911547, 0.99298028042708308, 1.2506883459181131, 1.4262884919213561, 1.9727983880183912]], [[2.7075413348449309, 2.8939273794826978, 6.3408470574080171, 0.42427945963094116, 2.9642438917786489], [1.1616757667178819, 0.0007537621102050578, 8.3542864192630173, 7.1326125126799935, 10.706126703262065], [2.5156855043710125, 0.39439217207308447, 0.18072886999104584, 0.94258261416735434, 0.63752379540489612], [6.319389126972192, 0.0040555156882074953, 0.4305545532961404, 3.2779706387219405, 8.2899655033030601]], [[5.9972124913892726, 5.8656499191618776, 1.1241013467219965, 2.5562628805549164, 3.6107114724992502], [6.7501047752047292, 7.2452329074394513, 3.4061009283761496, 3.3659002183712263, 0.60561005652948652], [1.0857413874402946, 5.1143345137388119, 3.5618774383124947, 3.1586658402468548, 2.9521334167841835], [0.60136896360527492, 0.81565442790648168, 3.5450996890190685, 0.70577123730641533, 5.6527339685197848]]], [[[17.516671768612241, 1215.6951059440196, 651.46246227756217, 1591.1615067550883, 16.154700878752109], [1991.8818939064543, 50.180325810977742, 33.676650078718417, 143.38432375892387, 4.1743431771327598e-08], [28.590304620369103, 1643.7270633923135, 2740.8311652445395, 2606.5401609957376, 23.250334390166685], [77.345981652334643, 372.03920367905584, 2677.8671507271397, 10.402333504251965, 1073.9009846620374]], [[3.6938737150305836, 1.9491001762298314, 9.7797223734005385, 10.488886175375145, 3.1825225271013107], [4.9023809780022418, 7.2043599002646959, 3.3315381181437722, 8.4960377055832037, 3.0652284650358346], [7.5642910684176838, 8.3501150968926563, 7.0143369755880149, 7.8978561752265328, 1.4361270403321695], [4.1768478849245669, 4.445281948963431, 0.56082591750370536, 1.1665731829531005, 8.3470589775603283]], [[214.68718982112026, 2.9852502582615879, 0.0025319856884625873, 20.203865322949863, 177.93547885192109], [5.3964510395923586, 8.4557324386050112, 35.793410251727394, 0.078667117803584749, 7.7952239630673024], [0.66658395131576853, 132.91748630555497, 76.871524260713997, 0.01185694935485449, 3.2708900869044335], [0.0010416424617887175, 130.56874833556569, 48.01106044756721, 6.3760651806585438, 46.42934987793582]], [[0.40471942451652915, 0.22438506613743331, 7.2739813030684033, 1.8764887110526303, 0.1051190333453432], [0.48269789501006899, 0.0020010748577811931, 1.3079906290839916, 0.49522847996347419, 0.058408600773189941], [0.37650036487297162, 9.099297825943216, 8.1848322857771691, 3.43429623229575, 18.846148687444824], [0.00078216712332014355, 1.5097439290141395, 13.92119122220582, 17.664274140404991, 4.2630669048740817]], [[3.5639524472503306, 4.1066478797221793, 4.918561478895553, 0.171423732945961, 5.2729357443993221], [2.3005241372185812, 3.2245197189433963, 3.1337168540422176, 0.43405915606029771, 2.5000951999067609], [5.4767680810538364, 1.1352287653926161, 3.0581183203748408, 1.7506588618216334, 1.022310568403344], [4.732341356525489, 3.152315724566304, 3.4742317408782561, 3.8251365501358285, 4.0342795576073813]]], [[[1187.1749271851661, 439.29844735136368, 600.4935950920144, 226.00766564157129, 0.028256019608007697], [416.40027100334515, 6.4419558505095322, 306.6529127713215, 46.558989389036022, 284.86176220649509], [73.944857436763101, 1.6211315389406347, 75.523133380506891, 5.5590869380564278, 0.022161382599752782], [0.11369281548735539, 0.00094160498180021531, 362.23364679565492, 0.71690604091822574, 2.1047468213414278]], [[2079.1541079518784, 22.036727465179119, 0.64480122697724107, 942.63402531927659, 6.2244985939715534], [0.18087762841151442, 1.0156974859242096, 897.41593076467109, 117.56348234101009, 5.5263180292144884], [23.978249104299561, 188.9550933959591, 2.1668151010853891, 90.996588248585255, 37.229510345183478], [206.79470235792465, 11.829549273164828, 2306.1774327926428, 624.68826650432209, 172.4003939164696]], [[1.2764607829320203, 1.1331105485459672, 1.2603788448282771, 1.1663622678859364, 0.61219396763448941], [1.3193634132789147, 1.1436027849836436, 1.220314945540933, 1.2434789249046485, 0.98550665415165184], [1.1578513028407529, 0.99800424402018406, 1.2393173776312296, 1.2810090367006093, 1.0437271219278796], [1.281158431237793, 1.1200620789226563, 1.1276243581200591, 1.3186742521278125, 1.2177931746328297]], [[248.23822767875598, 317.34469267480677, 408.6700215649106, 2.1382164095675105, 39.467703442806247], [910.27089367237625, 28.878145221429772, 192.03335284011533, 433.64980218506196, 7.419294752943407], [80.69497776171869, 331.38260314155679, 0.84422159605563518, 724.23259222339288, 530.47818603225255], [4.2308208977158275, 780.74615619683652, 1.823249881981809, 5.3409794514412656, 740.77432453901383]], [[1.8319538841359955, 1.0143424583322322, 2.4968562513408776, 2.4791424442115049, 1.5596216334258459], [1.9783850405097454, 1.6221134841513747, 0.15746838919640285, 1.3718626104544354, 1.9753717124638248], [2.1022922507549242, 1.3506125567612544, 2.1762695307816946, 1.8349433009880562, 2.5532540203269916], [1.8104365903113515, 1.518024160608183, 1.0071425872902178, 2.3381950958810558, 1.9631038183041951]]]])+(1.-msk_ref)*numpy.array([[[[23.855481579837559, 11.197487862023401, 30.651722867067122, 5.0115173358497476, 3.6093863434349691], [13.673805131969525, 11.841330206863574, 3.0192163305393769, 18.469077239899296, 17.608405585494769], [24.687957198319847, 0.060345630118264652, 0.0008904858508670154, 0.77951018278768835, 5.6533383808577469], [26.118358552446871, 15.724483047978124, 8.4230093396624568, 2.610615517697553, 11.933101282699457]], [[0.00016645032749469795, 10.378214237577909, 32.871943659718909, 7.4809233990064747, 151.818518073299], [0.11609236533369076, 0.4899672460717604, 43.662273669055935, 30.612531433441838, 393.33574711731978], [4.5936470303614456, 46.794868832037849, 1.8745001574658195, 9.05218607339628, 115.40454669078174], [0.99832159065397885, 258.85856612430314, 814.40072730547843, 38.246127319008735, 23.549840629989273]], [[0.48598332418948048, 2733.906039077614, 0.18458664814181047, 197.48887744560875, 0.98032606096331032], [1721.8801657175552, 0.099919144958569198, 2710.7119337120516, 61.352281590505932, 2449.2262034909536], [351.11460641027566, 2176.4370300817982, 1184.687397079188, 0.65055148974660992, 1.5317681918282156], [68.46688586678566, 10.052446893059155, 256.50100811453467, 14.846096549666003, 27.501205596206308]], [[113.42561223345868, 9.2017609949501402, 55.126188077381805, 0.33779162540714847, 33.369584438500212], [262.64153942983904, 0.32723179368017213, 138.52964209836242, 223.7549186042622, 100.75177152322804], [4.4334414594947553, 88.807005837237952, 4.7687449452422292, 44.746409312202964, 0.23499833169633866], [108.87050448033781, 0.044643819674917248, 117.20378445539339, 72.72855322947332, 7.7758515935577943]], [[7.9108023452440017, 0.53856113324977095, 10.171503879371366, 0.013279947737054277, 2.3681875643459889], [5.6586182049475591, 11.757312454903461, 10.256008125659287, 8.5820817423169498, 2.1291709271923671], [5.4758358755331162, 9.7264966843033562, 4.0938157630830476, 12.600650545755213, 9.413504697462109], [7.3175659246284992, 0.81132198456575033, 2.6637398016615617, 0.096520931022876955, 13.235088129088439]]], [[[0.72800575350471874, 0.49345249470751873, 2.726574157914079, 0.39719022005571153, 3.8387864426286402], [3.0474437859828187, 2.0284485388287581, 4.4448025629838588, 3.9177859987178016, 3.1106509173916916], [3.3468510336233375, 0.87772375255507895, 3.9435965867360911, 0.07108545423102168, 5.3763676186020382], [2.3367115023350253, 1.5894179451796919, 4.9538745775280333, 1.6973725366077521, 0.90056772920233186]], [[103.33127851842784, 102.69112133986779, 573.1922149467589, 92.44405033015282, 0.0098798050350431422], [17.53405135909578, 525.64869447154001, 218.24379934108305, 194.70599894360214, 113.9215198421925], [0.59086053349054379, 87.80688763625264, 0.067123869709387329, 302.28495114461128, 125.24747047208223], [460.14359174613855, 278.64753504250848, 18.149233762225386, 248.80148575337628, 44.272182363798954]], [[1.6038221078160171, 0.69852966017477292, 0.37353258634046754, 2.1340028441376422, 1.395137925817495], [1.1873613999173336, 1.853588674032808, 1.1441511215735893, 1.8978615628841096, 1.0321612604137955], [1.7341615408979518, 1.4063073903886623, 0.60014563103685881, 1.7621551807597546, 1.1601234823285695], [1.017344410170556, 0.19491500863641359, 2.1594563484876725, 1.8474512864984525, 0.95254192519790148]], [[1.0675212193539807, 2.7427834483514153, 0.0058491290236752707, 13.59725521689127, 6.702445480019624], [5.2765740728074002, 2.8753805411177122, 1.5359092161633325, 8.47362910363384, 6.4648948540171665], [0.80740908352183438, 2.6901931479499384, 10.201692712976284, 3.7936007443886286, 11.371005257327381], [12.045552468213414, 2.8375890689592187, 5.3016578669841135, 4.4480037617269925, 10.523140591487692]], [[5.326268691794346, 0.71767989098146623, 6.4942770584757676, 0.98013970184939547, 5.179767018040601], [1.2317956895360207, 5.74326100754927, 4.222914115574028, 2.9673481338355736, 0.29077651877653371], [7.2800369259851543, 4.574349040084984, 0.54893135774140001, 6.130658789921962, 5.4912343475171612], [4.9903790254450477, 5.8149126014336847, 3.4753743240303159, 2.4621288479199293, 6.9902028540341812]]], [[[605.43768117181719, 581.42738845518693, 2202.1227331326922, 195.48206030850059, 119.84714073123476], [197.79225738848282, 0.0064259198762503479, 37.61265400151467, 0.0034230852060503953, 628.83283046417671], [61.909692854335354, 0.027231995817965964, 53.243958447710654, 647.34847304790389, 2335.3378679683824], [18.732152227028159, 0.6931611663915711, 77.865541495796748, 50.325703949035834, 0.65323065838793037]], [[0.12381693826637462, 8.319331379506373, 6.5532080358826468, 5.03865157148332, 1.8525381610044589], [5.1913120808806088, 4.366451093724379, 1.7886845607101427, 6.261151049599972, 4.8319631913794279], [4.7457466888387616, 2.1060219736603241, 0.51968195533657668, 10.302432133818568, 2.1732058809546597], [0.12458967854743706, 2.5319901191540164, 0.051870542967502931, 0.96211063021817866, 4.1409191210019287]], [[110.78224037150159, 20.175199216977951, 0.5696620647389754, 0.40906122420447133, 169.14515904505654], [124.0877185436685, 46.439139911766048, 190.42054949134592, 1.0383875303520445, 9.8404233301644801], [202.82180637224374, 145.47753543500102, 158.0144187427222, 144.12050334550617, 7.757450339568897], [9.558681325244093, 21.246045038021091, 136.69777354199763, 49.03275759447078, 0.027355728350701138]], [[14.138800047723715, 9.5358329556474164, 19.210884956149869, 13.056270148494134, 10.963684136353903], [16.008098200861827, 3.0168515382772463, 4.1972676189643181, 5.4545122176546448, 17.672521773822194], [10.349356710518444, 2.4358878750587301, 5.8150777914076901, 11.740981296109439, 18.205043942677406], [7.9226359149312486, 4.7525654729823366, 2.6556142522036472, 17.659414788182023, 5.789513714447672]], [[0.61047338949427443, 0.21414564845884348, 1.2881956080978831, 3.1947460588052561, 3.4129546143447578], [2.0365026671971833, 3.1190191376700169, 1.6238454631167543, 0.87057035119313753, 1.0712415601687506], [3.4433372856558995, 3.6687322968428093, 0.82105696938670825, 4.5115472670165264, 2.0238095299818157], [4.279144821904997, 0.25094375904474858, 4.7395813273682856, 3.9756098566320475, 1.7063452103931405]]], [[[0.058044690562492826, 139.56134058311306, 0.047285957098584137, 1353.5098967633724, 263.83403231007486], [587.3413411785873, 7.1967292136502339, 0.67612814873856164, 238.22723404486587, 1337.8372680322068], [0.30097997751880839, 726.55502743522493, 0.0071548115285338075, 1356.2359293420179, 2.7373098535226568], [654.42924989544372, 708.78512405312176, 0.51394315982435768, 32.051976323031269, 247.60287339005467]], [[0.0065752567160133855, 0.20080211588793989, 1332.8443955392265, 591.28888841153878, 313.41257991725945], [645.54130564618549, 454.9369042174755, 357.48329035819484, 14.178671255332015, 9.211508162936374], [0.70816036086507816, 0.034137306213038912, 473.87714743031472, 65.653250885982104, 684.24868607629116], [31.606400643104756, 0.4862985377590584, 394.00392606631573, 0.16028265266557451, 21.09930322962305]], [[1.0237075467657253, 1.2403349286756344, 0.84515327046615074, 0.72382021490997339, 1.264990129922869], [0.97984783159393751, 1.2384306350370893, 0.90672078899638098, 1.205199603397644, 0.91094108635774695], [0.6838219066593354, 1.2611886153257503, 1.2006993717982741, 0.93786682266912247, 1.0251858755360233], [1.0359794998361955, 1.2354359299193738, 1.2638333817539431, 1.0002911652583251, 0.78332480301489138]], [[0.73303406866006093, 153.21520021262529, 0.34055150172794046, 144.06326490540641, 49.795217553430341], [21.886216950378646, 0.0067847460076985618, 0.00076327770299749196, 91.980650157392148, 7.8100169548934844], [32.887707458745894, 3.4359351091375672, 64.245481873410824, 42.066778744418563, 88.917800056151478], [7.5428488409903771, 1.3569585268536186, 1.6179392660659451, 523.86557186488312, 714.99688735071572]], [[0.93307988405150388, 1.2092561410288796, 2.5223328564542515, 1.5116235410586958, 2.6410089928246583], [2.2311916806456553, 2.4192579719131921, 0.42741482292448801, 2.5856844964056624, 1.426265581801192], [2.6121454266680155, 0.79642201105343313, 2.544627897300475, 2.1116599021220415, 2.5691414076495636], [2.1292030711772294, 2.4713177668387583, 2.5622597981915174, 1.9243601060497151, 0.27955733950250283]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.6132431887082088, 0.23135953616439864, 4.8556818861322428, 2.2771345368364981, 3.6095303683158515], [1.0846171509881994, 2.8570277601221097, 3.375801865944533, 2.8244423164431107, 4.7097699111905662], [4.472638422789565, 0.47277391896437454, 2.5622017936890322, 2.5098301129128586, 2.9774069751383965], [2.0288137803516419, 0.15836482591704956, 4.7130600865917742, 2.4532779549269832, 2.1687643391613611]])+(1.-msk_arg0)*numpy.array([[4.5324774736577949, 2.8931543604608794, 2.1338424154929179, 2.6940549526996622, 2.5406656698080634], [0.95082482101905474, 2.1497459033738, 3.190691339475813, 4.8619475632255824, 4.6006776188740615], [4.2110291785410006, 0.34749221154903887, 1.4320430010223908, 3.8397498844157809, 1.520161151175335], [1.8999312925379821, 0.17015070101580471, 3.8988587075801706, 1.3277734922277962, 0.40875503938954422]])
      arg1=Data(numpy.array([[[[0.01807451182005932, 2.6572981047291067, 2.0834242581420952, 4.3639944987473323, 3.3090461541588905], [1.9704699256760729, 3.429766913435278, 2.5196911652528913, 2.7486633366652313, 0.62431283459992493], [3.1496124669904484, 4.3339919969299956, 0.18912970563592993, 2.0095199748763699, 2.7645753771802068], [2.3056954278387809, 0.49393271539951539, 0.52112978465332149, 3.9895432168708154, 4.9254379335569061]], [[0.074796705046112105, 2.9095301317656408, 2.7049888971356046, 1.867441772011688, 3.8752759725379655], [0.57430886574764817, 3.405159775564107, 1.9912254907491509, 4.2810383550622797, 2.7954952991610065], [4.8181226939494293, 4.4556256105032208, 3.9771934731415195, 1.3299656809764286, 0.23597135708879879], [1.87852358748908, 3.5513772854444801, 1.7991832488899049, 2.6098598216965438, 0.71888843198562846]], [[4.7055556650923238, 3.9955633781821578, 1.0709881144798343, 3.7323212627444988, 4.6942375545166239], [0.068297601428743832, 2.3412349480527297, 1.4849528092021445, 4.7294719991740637, 3.8997905769993926], [1.3868232946286498, 1.4473285235392972, 4.8564353320397924, 1.8641927398760705, 1.5081687694306289], [2.5383288241953741, 3.1936732671417123, 4.5182860720248108, 3.669605216924321, 2.1541879397186476]], [[2.835589565112365, 3.6169633010260021, 3.8723393955928529, 1.9265761433231499, 4.3861517073375964], [3.1455764295549158, 2.6387942326064957, 3.7657848790490775, 3.4908056610135501, 0.53056041157268119], [2.4683804098372444, 0.013996031344766267, 0.79656790403505862, 2.2268430485958968, 0.79596880571597795], [0.55134771160626916, 1.7404508804835899, 3.3973256168124806, 3.0829345797925831, 3.7172440114527796]], [[4.2810041807495081, 0.33116898541123313, 0.2463052893070414, 2.7909372555979757, 4.3209022269966244], [4.6865862189944103, 2.0142062825483147, 1.4072728495775662, 4.7393460601516999, 3.1195164236424184], [0.93670681285731605, 2.068598609870905, 0.87147066079625091, 4.2582838092323838, 4.023630916997444], [3.5261239922861329, 3.4247415320961982, 3.0702287271723221, 0.52669449229281184, 4.1355215004710235]]], [[[4.6892373463194588, 2.0136573914816558, 3.7689966455924355, 0.5426127896726417, 4.4893689654797546], [3.5432216082172281, 4.6885152521877176, 1.5583519353511084, 0.49779402173875731, 3.6250535799168309], [4.8007799532105979, 3.0008451318640406, 2.0342593535800026, 3.5178089087839224, 0.45291700031592569], [1.4278275902515329, 4.08677665918675, 2.482378200797362, 4.2881534301179398, 3.233817787867256]], [[3.2590062105575694, 4.1832303875378951, 0.60228836380560669, 3.1863817192109645, 1.3609492532688598], [2.2866286002053475, 4.3776460540559867, 0.85104035558370361, 2.9866056584394456, 1.6671096456100667], [3.6937353519143183, 3.2387810481472679, 1.0582263217786789, 0.78356847455402645, 1.2796552762203957], [4.6468688018582007, 3.2230967194291242, 2.2302318228959512, 1.0398236502557527, 1.2857758420070355]], [[4.2390032837092058, 1.6889524039293564, 0.7412099215121144, 0.092319609519862705, 0.095443805421891764], [1.6997463663114578, 1.2192153255153799, 1.5257894285520157, 2.5578666560289403, 3.9233513930281751], [4.911481032350884, 2.5518866278707346, 0.58584690346890722, 0.67177413838706868, 1.0337927435042995], [3.5010862183976577, 1.9590646183626661, 0.67339573104083805, 4.1522214628623724, 3.7622210941388614]], [[3.0284482553958814, 0.29638280593877958, 0.47456472572220637, 2.0849381174055579, 2.9174570984907864], [1.1364833210508263, 1.1063059822720787, 1.2109994512183271, 3.1432585000641704, 4.5506628940465212], [0.660605941393407, 3.9627573428897986, 0.65229803305172029, 2.7387222012564472, 4.2808354416776861], [2.4289721072886103, 4.8549862777975719, 1.648142037762153, 0.71725806349460786, 2.617904907671496]], [[3.7953095087778159, 3.1851217501506985, 0.19048315285230955, 1.4389307744783482, 0.31353563163898024], [2.4381075362160849, 4.2839798173881798, 0.19813720609133534, 4.118465969469459, 1.5978124503969751], [1.733713121579372, 2.1117629485070548, 3.7753987436879424, 0.020499788857963323, 1.1000557846265233], [0.72110956559399275, 3.0930399385379883, 0.40400715197041559, 3.833604320092364, 2.7537418738045747]]], [[[4.7858563406304278, 0.96406537293201755, 3.7972757955143268, 2.3223037375772355, 4.0133063469444634], [1.8528641752608259, 4.5019592023659323, 4.098857027106761, 2.9288968819699388, 1.7229347226910485], [2.5461398125303583, 1.7239519565814301, 2.1892924169867318, 3.7464731434461696, 1.7774813245870231], [0.83427651221209975, 2.7674880860306685, 2.1057690286549002, 4.1204337320464717, 1.9258713179572309]], [[4.7922295513190525, 4.2216711361285579, 1.1913734719505347, 1.1817789754399579, 2.584779257080525], [2.3350067026980383, 0.64917918698899102, 3.0134674481553159, 2.9051373790141208, 1.1952266280827037], [3.7772848512119586, 3.2544731335283452, 4.9736491014845052, 2.2519272843079676, 1.7814736732906089], [4.8736442129750914, 1.5669531236556271, 2.7484461467295658, 2.586377891243361, 0.59193755312539043]], [[4.8865982787486564, 2.3122109707804115, 0.26759953753118892, 1.1339028290463413, 2.865020856161526], [4.2310297776350483, 3.4915922437364655, 2.1474791550937788, 1.7589091214930836, 3.6069039429657392], [0.63196807948059308, 1.612208992675545, 3.6597376431447044, 4.5519310007719014, 4.0846433756977021], [1.0346425592738624, 0.38614687175647938, 2.1289198152909967, 1.9526719043041538, 0.78044763051021826]], [[2.7660816562986263, 0.30629696898348036, 0.75515361122982239, 0.52463005382545225, 1.6274797173631046], [2.0845578345310445, 3.4215746130057871, 4.1777444041725893, 3.3162331677889925, 1.740934683501036], [4.6934640614560701, 3.6953862948385354, 1.0476262563875607, 3.4019236679432456, 1.4321482092491435], [4.6000492122585985, 0.40988572616265379, 0.43141150476560064, 0.29034507172260426, 0.33390177085145789]], [[1.5349610246130179, 2.011099820314977, 4.8461376625915964, 4.2983265799773429, 3.2125943835874318], [2.3186268289928975, 3.336645954373104, 4.6007291025394013, 1.5914946608648119, 2.5415276369890467], [1.6722826539181779, 2.4375815185940746, 1.3638605860395163, 3.2222596119319831, 4.1282336362069509], [2.088689636387175, 2.5123689441799049, 4.4215046081116895, 0.42305139200833441, 4.3704252530170393]]], [[[1.0316380302820023, 4.9319039748251807, 4.467072779536684, 1.5191708113804272, 0.011952051112909068], [2.2043286002768614, 3.2838895947180595, 2.8498228411045488, 1.6371075689296295, 0.30020870052695614], [1.1531835591443502, 2.0585334141789509, 1.3755609514100158, 4.8563093984464256, 0.1244683117390533], [3.6254607768998568, 2.7856532982452165, 1.4289361869838255, 0.72524064818540079, 4.8551481833287493]], [[4.0900410208608511, 3.6422388932600978, 3.1431583571504569, 4.7955176244249937, 2.9593724623605628], [1.685403791956019, 0.10238232907137634, 1.5234788667810335, 2.2966261750553039, 3.143957826506607], [0.13493173699189917, 1.2933583491031433, 0.7419999618323746, 1.5483484425438807, 1.378113380853444], [1.5784615220480029, 0.78628087851109851, 3.5183286458242353, 3.1570481603856764, 1.7373388630423197]], [[0.83860912136141086, 0.36136158949495079, 0.77139955768586177, 1.6068567329146513, 0.47277252497023281], [2.4371597888475574, 0.36040066166999818, 1.6754937758140183, 4.8157248497989764, 4.0552819163793377], [2.6772539621711284, 0.24395711518236174, 1.6717483908278186, 4.4773313092601601, 4.6357835854523257], [1.7319432890710325, 2.8734262039418343, 3.418497842804824, 3.4095347190012255, 2.3702173947743685]], [[0.45435264911490431, 3.7413044133446141, 1.0635378936432114, 1.6381721277311492, 2.6187305465288264], [3.8359219311674915, 4.0071104171853724, 1.2951043751086491, 1.9378207382963213, 1.9524109591618506], [3.2070897052203264, 4.241656146815826, 4.0148521581079182, 3.8852828387851246, 1.3026484569585881], [3.8191348516452774, 3.228830067231574, 4.7847460266159958, 4.0322416595206994, 3.5386082782237041]], [[3.6907306376067495, 0.46285128970430128, 2.9075548240267102, 0.14335665232958647, 3.6865175899436933], [2.6243183982503226, 0.12825630373970937, 3.1184494035757599, 2.7528622439496981, 4.6184978203264953], [0.58405082989270696, 4.1127041375228801, 2.1013325802180089, 3.8889574266622762, 2.7354225848030684], [4.9162065112259778, 3.171573912314857, 3.5823018495094638, 4.7245630725595378, 1.8826261027921065]]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0175138320349315, 12.840181536180449, 7.3988218730839179, 66.156187660529952, 24.014154612493797], [6.6380466919010779, 26.966809546488236, 11.25030572990581, 14.018027110131182, 1.8215903646279388], [20.604153866365685, 64.276768108785163, 1.1992261716651023, 6.8917766717433739, 14.233938264204374], [9.1598765357808833, 1.6071587045139897, 1.6496994646204193, 46.169689030326339, 113.44708611463365]], [[0.89629462608852117, 0.014137579154584627, 0.019072362323677219, 0.064989582193404991, 0.0034390481379630543], [0.43142431735006903, 0.0068437957927111005, 0.054219171360242548, 0.0018988471090197879, 0.016705844278510223], [0.00086508487482994494, 0.001470627492167878, 0.0029624294475418009, 0.14273289206592407, 0.7079295425636617], [0.063943866800600241, 0.0055251583559200318, 0.071818535988286247, 0.021921952882947428, 0.34913514574422727]], [[1695.0739205900588, 552.02185767412311, 5.4320759860846621, 364.17180008346179, 1665.0281723251999], [1.1139590970365831, 40.42713958228066, 10.448385841989987, 1760.3589466195117, 474.49557613280956], [8.9476255380121703, 9.8453129946851252, 2151.4430165596282, 19.024037372218796, 10.838798935064757], [55.198684938342481, 155.47444089908356, 1260.8831512163144, 329.81272832472803, 30.082364008012462]], [[10.313537229493615, 19.618318306304218, 24.206405522141875, 4.8813115165588936, 36.945454511062344], [13.310471506910108, 8.7715272013632113, 22.174249280246499, 17.683753925342188, 1.5474488071750234], [7.6237955576091991, 1.0115841670619645, 1.9261245197183878, 6.2495467393123603, 1.9251751575890959], [1.5741475453722766, 4.1881040278898238, 16.374413463561886, 12.641713726834436, 21.305953184620112]], [[243.47186219077884, 1.5297144979329413, 1.3718387179108213, 35.959192190796841, 256.26539878848308], [409.77057447111684, 13.268465582315892, 6.0881484733621303, 438.48200910571092, 54.824744216182744], [3.3278816159524789, 14.227930765015964, 3.0605672726552999, 236.47394165440565, 174.97494730849741], [92.393259348426852, 81.119369143294847, 51.463717362103949, 1.966102762652842, 201.99906326577212]]], [[[1.4635909973385248, 1.1777001212934681, 1.3581786152820183, 1.0450605708065732, 1.4400218404392802], [1.3334979192253602, 1.4635051549904816, 1.1349406506469768, 1.0412629476074751, 1.342391165281283], [1.4769117940152396, 1.2760250967896496, 1.1796725781662027, 1.3307481530595033, 1.0374742154191383], [1.122971446641285, 1.393692761787251, 1.2234029805886355, 1.4166772040197448, 1.3004021190008406]], [[30.607514031848563, 80.759798003285511, 1.881880205932027, 28.360736280856386, 4.1732696231783875], [11.028261207242247, 99.044904520567513, 2.44343957895676, 22.995174341224384, 5.7551863995630166], [48.308923406352207, 29.964505380712918, 3.0371111224126226, 2.2763560818951856, 3.8318927437529591], [131.3937420595243, 29.475175592724128, 10.39429258979081, 2.9790009259600283, 3.8565929138864399]], [[173.69672155369969, 7.8055501436158599, 2.4639860052101556, 1.1188698048034014, 1.1231307210863022], [7.9087309055982358, 4.4076314550499376, 6.4001766779152502, 22.465587450385861, 118.30645428838167], [393.65281593081062, 22.302732238056777, 2.0396140707000003, 2.264383437231408, 3.5174850121878993], [70.777179838095876, 10.842379039565888, 2.2688552103997273, 156.29248907518368, 97.245811054857626]], [[23.207394113071043, 1.3603438631664799, 1.6368043013004627, 8.7129815533366362, 20.681252379872458], [3.254449248629089, 3.1540571204355845, 3.5162461027990868, 26.145567470676493, 112.73084157082255], [1.9855865718451089, 61.226143635091418, 1.9685321836556726, 17.178269939838081, 85.186097874747404], [12.453831166774053, 154.62229575739676, 5.5360646715382096, 2.105887283226604, 15.153011520470347]], [[358.29613718507994, 139.18292306138693, 1.3433683220123123, 9.2982231855770667, 1.6255838769593502], [43.736660354873791, 764.04175352864775, 1.3593969101111805, 591.18897201140317, 11.893982224988552], [14.682107339083368, 26.376417683983401, 347.40989766496108, 1.0322772390255506, 5.4996772423892919], [3.057077964827577, 120.67433726388781, 1.8702371882857485, 380.20210954211376, 71.329212570764213]]], [[[1298.6880399362556, 4.2382446566061462, 295.37255145874582, 32.419530555164414, 408.23645951579113], [16.047461441474287, 848.81225782107663, 464.0534049511399, 80.432998087957102, 13.209273843623409], [45.334242893846941, 13.229417405653379, 26.562831842654884, 273.72834310526474, 14.333920379954669], [3.4893946428590343, 63.157879303426405, 23.438892158621492, 479.29730447543557, 17.90202438321483]], [[0.027597194690976289, 0.042314977569578506, 0.40962912040649696, 0.41258397782068212, 0.14422851831114816], [0.173905679540098, 0.61488207158899688, 0.10461138302817034, 0.11345502909213519, 0.40844841217882216], [0.059030115488194895, 0.087331031545950177, 0.024090217640572297, 0.18507312700507522, 0.26327146724446554], [0.025964320600138011, 0.3091699684347235, 0.12758590364095776, 0.14405589397693858, 0.6418228799632838]], [[99.249699241262377, 8.8064368767066465, 1.2863073119616655, 2.9062154810669694, 14.814456227404083], [53.561800865389316, 26.712290975084102, 7.5420361650318286, 5.2325539506455758, 29.773441284210733], [1.8123020511078267, 4.5579555608639497, 31.290866415957748, 72.44020792381059, 46.670207868097549], [2.6470902135196828, 1.4380856725252109, 7.4114812956281275, 6.2789607885927117, 2.0840172330356537]], [[12.748199118231097, 1.3255919120481192, 2.003516187691722, 1.620561152994854, 4.4710861100677315], [6.8089745368108812, 23.303056003342729, 46.731728530593593, 21.150165401648632, 4.963118762973239], [75.113202677935291, 29.980589212151767, 2.6222729122610278, 22.88545177918262, 3.7355100169169417], [68.926077721306555, 1.4581711606943144, 1.4873430892521768, 1.3062754366267395, 1.3596964011154997]], [[5.3373191324833886, 8.9729639267446935, 197.82681276402991, 108.82065506916292, 33.285106027876516], [12.550295932645533, 38.10924263533753, 151.35654421061398, 5.6768963874885232, 16.005634101979535], [6.1999959392177741, 14.289572711539408, 4.428424523536564, 33.63796382046754, 90.38883758883199], [9.7656451776627851, 15.504450609463357, 124.47375503063047, 1.5865651508553569, 117.72655660001543]]], [[[2.0747354312622446, 32.755856417654279, 23.576167380656592, 2.9292292429476654, 1.0084913424858908], [4.7562287961893732, 10.20820582280991, 7.5090638925895332, 3.1841132671834611, 1.2366214559296094], [2.2610316291967933, 4.2901087996851412, 2.6462468483961992, 31.050108053849758, 1.0920484736871792], [12.998537596008292, 7.17579807054104, 2.7480806734930701, 1.6704155421797124, 31.024610767735474]], [[0.00053280877122171546, 0.0012160904843115427, 0.0030507080037199619, 0.00014519421840424217, 0.0042804761177742462], [0.04478182462559066, 0.82805502402701914, 0.060352739127081625, 0.014518283915760913, 0.0030462166890834305], [0.77984548471827742, 0.092229941540399055, 0.25476913145767677, 0.057649140611927156, 0.078893093096227343], [0.05453710709544865, 0.2348047949095135, 0.0015280458133135871, 0.0029736103428613407, 0.040694526906828371]], [[3.6697629998859109, 1.7510793793949826, 3.306630127090965, 12.075397093929308, 2.0812255474925818], [43.746865436077123, 1.7484726294773187, 13.43120403350218, 1747.5993553498131, 537.56797130668519], [63.475051489086034, 1.4596779642018771, 13.35344025481821, 1034.1903896837928, 1322.1687461055531], [14.659615239179713, 86.037229300913197, 200.30176355257669, 197.5376464597648, 39.434326425848006]], [[1.5034274353071857, 28.718467397081916, 2.5972302008480779, 4.3498298883077142, 10.486769834402288], [31.263538419951693, 36.455099441407896, 3.1971479152911053, 5.6919273341890895, 5.7669453821834207], [17.780861935533569, 44.995773769615838, 36.709257948950778, 32.679573785006838, 3.2188668384702477], [30.796078664002696, 18.131178993827863, 73.255332444259338, 37.286627786166576, 23.942037034554414]], [[17.412826426130735, 1.4309232032967232, 9.4963317821262745, 1.1173732726553343, 17.356125916171582], [7.6265529773241632, 1.1043871759896675, 11.180500480513688, 8.4245424230709407, 35.710511286023632], [1.5716835583261022, 24.140261966756302, 5.0873778341109057, 20.300912914204542, 8.3115666643993436], [44.966533714255235, 11.649904392867679, 16.010842000373671, 38.76649326313418, 4.2949901454953929]]]])+(1.-msk_ref)*numpy.array([[[[1.027691930755545, 55.472723719943801, 23.303739866258397, 731.55197116554632, 148.54053124896271], [19.646702460187502, 178.2700192376123, 45.057091055073236, 63.6862606005175, 2.5689684627964016], [116.73550312114462, 699.12281206089824, 1.330860603362692, 20.841050580461296, 65.236310200664704], [32.606945480907655, 2.1095297377949072, 2.1980423622570386, 415.41240165207614, 1708.9939646899747]], [[1.0827023412183747, 21.997563907155079, 17.701296239997841, 7.2708403465830962, 61.36804626148583], [1.8406440167288223, 37.242946449045654, 8.2926798949061826, 94.438599720477171, 19.487770402197121], [167.08790075542535, 113.68394643833884, 68.385517540709557, 4.1077915903502387, 1.284903416441004], [7.3569438851916278, 43.501540815621958, 6.7622631030749822, 15.999824391313524, 2.1462223345838969]], [[35.390910067679229, 20.662792072897414, 2.2517955560096907, 16.925426213070502, 35.088615682060478], [1.053127609195232, 5.8972071356291078, 3.081701336941324, 36.038282647563705, 19.216053791378062], [2.8608165252416056, 2.9950635978661486, 39.67853558840423, 4.1079206983184866, 3.1364066546960951], [6.8473458578861983, 11.252187936277693, 30.707865689227191, 16.139714689676051, 5.1177321327949921]], [[16.613305016849459, 36.038303182529546, 46.417297655777283, 6.7485551092870146, 77.237460481637569], [22.587927473271641, 13.669544975235336, 41.765540074491206, 31.80279374101805, 1.6918296166498963], [11.545343261905233, 1.013967376462686, 2.202155405963738, 9.087566700795044, 2.200848297571012], [1.7270448314291502, 5.6117883748944841, 28.988841426976752, 21.228285919546622, 39.803915827726982]], [[54.147950493914237, 1.3617772016647198, 1.2581740277998377, 13.495317863632859, 56.200300539544521], [79.035365028692127, 6.5410555516549156, 3.7142544250835687, 83.020707526113469, 18.333291222561314], [2.3950638744471102, 6.881353612400841, 2.2537196030771121, 53.012888124903526, 42.595072176290451], [26.785216649984481, 24.369161589179079, 17.509812953213448, 1.6341188062257672, 47.279096943863173]]], [[[0.78941994574879493, 0.90344544120938075, 0.82691505133093568, 0.97300944688160462, 0.79741630354247961], [0.83638311189289882, 0.78944869056264744, 0.92442756695332284, 0.97521094084981352, 0.83293896058709471], [0.78499224992495953, 0.85957350992021697, 0.90250737254062796, 0.83745557912114521, 0.97742028696064431], [0.9305319824453071, 0.81377000274715627, 0.88234247108427666, 0.80554837945121227, 0.84953453976507265]], [[12.113000106264762, 24.572647050850815, 1.5855973112249149, 11.458091203044173, 2.8337531532290541], [5.755007816581486, 28.514988983371062, 1.9181159703844188, 9.8335262919899442, 3.5820017239129864], [16.894684761131376, 11.926942824786085, 2.2477127547061793, 1.8215792748908097, 2.6628146931021086], [35.039619754296353, 11.784627740453102, 5.5118876273007391, 2.216276792593507, 2.6753175712617505]], [[136.76321273730926, 7.096389315327488, 2.3631125587944353, 1.1130596708602447, 1.1171016214650169], [7.1858202087196625, 4.1147557667238432, 5.8724873667109154, 19.447783531425358, 94.823745678721991], [298.41497009272831, 19.313316898498801, 1.9733283078774913, 2.1802021973399222, 3.3182755198629659], [58.095715697035985, 9.7082929037120156, 2.1843079623960002, 123.66342040675185, 78.654939114195528]], [[120.2179632476558, 1.5979336968602844, 2.1180486323787564, 27.036901112887612, 100.86476924541923], [6.0332096534672166, 5.7520454774936365, 6.7877659791767675, 144.15251406298958, 1334.8793467225985], [2.8425685411793946, 526.82022929145683, 2.805465863221404, 76.02957033345136, 871.20778611788091], [46.584666455454297, 2160.0034223021048, 13.55067343013118, 3.1089962963609343, 62.806411174306717]], [[327.8037281774902, 129.1719270717399, 1.337384811493614, 8.9898964470768412, 1.6136831064478945], [41.307682583352864, 691.0591525169408, 1.3530992708964071, 536.79570988078888, 11.45684210944613], [14.097524427105439, 25.102820658959608, 317.99229667491198, 1.031781429571057, 5.3597058771159825], [3.0058488755387698, 112.23654942949939, 1.8526131764206384, 347.53338019490474, 66.871359362326203]]], [[[973.26805401704667, 3.9989971236166424, 234.94942762548669, 28.185073278185499, 320.5245723777283], [14.351808180300601, 647.09855651882253, 362.47509218784643, 67.416903399976945, 11.906392565099173], [38.884792300909957, 11.923818217328888, 23.279222029452438, 218.40059253021985, 12.877704488741536], [3.3182755882051631, 53.454841339150718, 20.645121359539896, 373.89564789018982, 15.940114008404205]], [[0.0063110761873832796, 0.011535092287319212, 0.28385342593332125, 0.28674677432558593, 0.065079521656092307], [0.084742968563108187, 0.5034902872236553, 0.041366895267259708, 0.046385514548268253, 0.28269968850369609], [0.018450952301678616, 0.032064028063656609, 0.0052098100001671194, 0.092521292158642604, 0.15212680233942991], [0.0057906807421707314, 0.19084573175057593, 0.05474094121678539, 0.064969644628392414, 0.53489452297244544]], [[5.7822199757707713, 2.2940522785447928, 1.100864252065795, 1.5025848876026089, 2.7978044836894722], [4.5693532060958137, 3.503764968650783, 2.162282263353231, 1.8806697313696199, 3.6518970366212828], [1.2547559134723412, 1.7841598455080372, 3.7218448067465042, 5.1274474721268843, 4.3353571770688362], [1.4499691964888222, 1.1487405304418612, 2.147919221262049, 2.0161879500806421, 1.3234743744064168]], [[41.326595838364256, 1.5099766039927576, 2.7620941125603418, 2.0255495928325389, 8.9318449845658012], [16.520122484350356, 99.824258312698447, 276.10121716569881, 86.633283781804437, 10.40478826730086], [552.59230262954009, 144.28628835939534, 4.0938421902489104, 97.219635217773813, 6.8676539468405897], [487.32900478976211, 1.7357910039130513, 1.7867961534030272, 1.4779150709820486, 1.5671110937548118]], [[1.901925460580794, 2.3216577681080537, 7.6113586154792392, 6.0509146852098992, 3.8400563960249081], [2.6407918753031638, 4.0448395319783783, 6.8679150150147077, 1.9474951034942627, 2.8991956246528687], [2.0145167027167736, 2.7756889699218505, 1.7704028369907525, 3.8556322779987258, 5.6348568442682661], [2.3983412900523304, 2.8640055599588035, 6.3712674425528304, 1.1938469686027551, 6.2364153321894591]]], [[[1.9389053713797957, 23.69782762651981, 17.585003816342024, 2.6512487411058898, 1.0077005360306097], [4.1155692688846965, 8.2289509876356188, 6.228071963984509, 2.8597226749703792, 1.2124938416997324], [2.0962156485692063, 3.747928285199241, 2.4178032951104322, 22.575504429864932, 1.0831635458822755], [10.245975385690098, 5.9767781919381511, 2.5020654501553539, 1.5927662778862652, 22.558685435585616]], [[0.00071462541239696725, 0.0015794749493487101, 0.0038228622597051144, 0.00020485645689328954, 0.005293593144695255], [0.05054105152449985, 0.83416308584779231, 0.067327340850919001, 0.017120350063763515, 0.0038174532350808035], [0.78743562809883139, 0.10120276991670123, 0.268706805848834, 0.064426215876907902, 0.087096697666767281], [0.061080229307879944, 0.24843871798855313, 0.0019670723037699838, 0.0037299677978961448, 0.046099643731815881]], [[3.1301530643350026, 1.635091696898451, 2.8565966576884203, 8.9033182100977726, 1.9027380913350809], [27.555551352168045, 1.6329551805219986, 9.7748949312300457, 701.12005256903103, 249.12547791203789], [38.202321480364262, 1.3936810886732769, 9.7252060033911132, 442.40798470158063, 548.85591393882567], [10.555291512200291, 49.890204997935776, 104.74144434057008, 103.47177813839403, 25.156506739732556]], [[1.1374746139462357, 2.8883135824716919, 1.3519076741889007, 1.5911030029630342, 2.1010121523090701], [2.9668393411969318, 3.1143788357193398, 1.4436383930597667, 1.7321768463009459, 1.7393566239095344], [2.4823881760198887, 3.3285083167307308, 3.1212218136807555, 3.0086491180626882, 1.4467293126760843], [2.9527531193190937, 2.497735544386062, 3.8825498592394814, 3.1366473800992432, 2.7270153642455162]], [[0.03681417165478959, 0.66094482109819841, 0.074183568410653086, 0.87963125760177419, 0.036953191903723891], [0.095577467146825273, 0.89159514694370601, 0.061428144635110399, 0.08519448707812291, 0.016052521727068492], [0.59302733578250277, 0.025238455017443939, 0.15260009196562152, 0.030831639179645154, 0.086534132309738734], [0.012299075300992751, 0.058576924684524405, 0.040564265447292649, 0.01459933265055449, 0.18557963992324589]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.6718522589835594, 1.6099314033435603], [0.28678495710137991, 4.5430194925665806]], [[0.37538461111727095, 2.9578757082105427], [1.6348454267377122, 4.1990160986135887]], [[4.3564665585221913, 1.1151414297180122], [1.0462342299848775, 0.85631098718122112]], [[2.6015642251867872, 2.1335030004432576], [3.4416908368287848, 2.7177790046345169]], [[3.7254999288718866, 0.029441866798020959], [0.53924253690559298, 0.14305705570776092]], [[0.53526988815203214, 4.7797653516475211], [0.89712815348195962, 0.62983428775133454]]])+(1.-msk_arg0)*numpy.array([[[3.5408340030367271, 4.9984979594106518], [2.184036944671941, 0.38271982288104223]], [[4.7740357085733089, 4.9702343736659875], [4.3521942197838941, 0.13835592156069285]], [[0.30617805850925722, 2.1492430129880238], [0.70553868465959069, 0.31035390708163274]], [[1.1300856508308821, 4.5779772121638755], [1.5520688336371586, 4.5755829964110548]], [[0.56209552783827066, 3.7513692455508063], [2.3434187284580403, 4.3616478080473682]], [[2.4028336440821594, 1.255484181219253], [1.4043174975655766, 2.0376385197048208]]])
      arg1=Data(numpy.array([[[4.4212694122863283, 4.0702155053213476], [4.2543612246583038, 2.5688407645734572]], [[3.6900407503480674, 1.5175172965189383], [2.7295912192990501, 0.43604332050903333]], [[4.1094758082815659, 1.2348121577340003], [4.012291417246078, 1.9580097620721517]], [[3.7794806562510992, 0.38368470300289864], [0.40374117932025877, 4.3400573594669742]], [[3.0695395945446662, 3.3962496872298615], [3.1402501416628708, 3.6492142135124643]], [[3.206180783312234, 4.5177998300743747], [0.7656423486995293, 1.1909736000549684]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[314.41962823829368, 6.9462525846141947], [0.0049232154572154321, 48.821779558301252]], [[0.026903006386513693, 5.1846587409865208], [3.8256401525738117, 1.8694735387867245]], [[423.16261456800248, 1.1440462948932182], [1.1988282456167469, 0.73806031032765718]], [[37.099747731736592, 1.3374207252839794], [1.6470883750054393, 76.650372702428001]], [[56.659615237471492, 6.3128290194096915e-06], [0.14379213023108683, 0.00082846176633170237]], [[0.13482032440862693, 1173.3374080210654], [0.92024485451161619, 0.57661219509505024]]])+(1.-msk_ref)*numpy.array([[[267.75873545022029, 698.91996512110882], [27.754570191014846, 0.084817916216020015]], [[319.97478921781646, 11.396301829533627], [55.387591379611969, 0.42212182511549101]], [[0.0077200987336165849, 2.572232399988414], [0.24672970001351416, 0.1011699349222452]], [[1.5875717409476808, 1.7926287783632815], [1.1942042802986172, 735.14908537450776]], [[0.17062092329384312, 89.143970759672428], [14.501775905582008, 215.88478779019147]], [[16.621454469156511, 2.7951777219351288], [1.2968975926314581, 2.3343300735329966]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.3211443985401443, 2.3973626870146627], [3.9850658213870673, 3.7848650860896775]], [[1.1941308413100598, 2.0848655309018382], [4.6945005825085344, 0.94502151386508726]]], [[[0.58703898862091808, 1.054227704282118], [2.4018474738467681, 2.0467156480134592]], [[0.42904431361847578, 3.4987013409609165], [4.2986212895491134, 4.3732591699673833]]], [[[1.6015826327753195, 0.79130140760930268], [1.6889573177080104, 0.60560136475433801]], [[4.6978010714802023, 1.3140919014684251], [3.8367750917047379, 4.2647075302335313]]], [[[1.6363559331139701, 1.0848410642187694], [1.3348657236363155, 0.42458501172962776]], [[1.2400335554353186, 4.3616426699137065], [1.7681631432116605, 2.1670674093232152]]], [[[2.6322333871466577, 2.4030496071655101], [4.8215089542421365, 3.0106552330950156]], [[1.1141435708970056, 0.65221855784369387], [1.4083738896489024, 0.16776512624754664]]], [[[1.5582290965833301, 4.0957467874896354], [2.4934173565612032, 0.95885462187628179]], [[1.2108640794943137, 0.58073361437469506], [1.8419845213380481, 2.9791851183659759]]]])+(1.-msk_arg0)*numpy.array([[[[0.96540477956870352, 1.5256735491255962], [3.1863303675214163, 2.2409232621325836]], [[0.96350328366841831, 0.34983193691887721], [3.9136656356239263, 0.77173627357003272]]], [[[0.73126890833173452, 2.6010023160505602], [4.3592974494133658, 4.5838747334539756]], [[0.82191535422551254, 1.732432398531434], [2.1304175940340442, 1.7863684702472398]]], [[[1.9178126791874577, 3.089857559559726], [0.39289458356080487, 3.6624919240018836]], [[2.6847121270678951, 0.78423855584758639], [4.1854158821736585, 1.5773547668549039]]], [[[1.2115820887875717, 3.6947472932489265], [1.4029110586563001, 4.4910070071583519]], [[0.50787611547789246, 3.6161669656186728], [2.050946294631347, 2.3900908996165855]]], [[[0.76228550635170844, 2.9168730141179364], [1.8276105767096065, 3.3740926619621012]], [[2.1998534438906643, 0.523678667576846], [2.6374485975932433, 1.916563751193642]]], [[[4.0904819730172743, 3.8122195983675233], [3.813326783264765, 0.23887117254511592]], [[2.7030249786630862, 3.4405036156449444], [0.4602450738416195, 3.0249954258274281]]]])
      arg1=Data(numpy.array([[[4.1486373366190143, 4.5510027818349803], [4.1161388970254018, 2.6549511214870214]], [[3.7999807622770185, 4.8224022941815141], [4.144496808284841, 1.9900968579811358]], [[2.2198310613016252, 3.1220942125901479], [1.8000534006861721, 1.0793829045799257]], [[3.9147866044137709, 2.3079117438899583], [0.46933087133302798, 3.8459070781049105]], [[3.4830521808274, 2.7481197357521867], [3.1791862757623446, 2.7661548314367654]], [[3.6958815908251248, 3.2166305065887375], [0.41138891608466366, 4.2915806185456997]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[145.42325394305692, 37.616430181902572], [540.23613099576744, 427.27648669193945]], [[2.0756608081002241, 20.57640832043321], [60.678805591747782, 0.86059516547778681]]], [[[0.13211095921348698, 1.2222227791165636], [68.413707869197538, 31.626013324390769]], [[0.029985157558472721, 179.56442235604004], [18.213206921222596, 18.847964097935751]]], [[[2.8448868798743319, 0.59475250097330745], [5.1362593669673116, 0.20891364669426379]], [[16.197433304612662, 1.6350561239072006], [4.2689718044206355, 4.7851074183632951]]], [[[6.8752081385048944, 1.3754689703956808], [1.9475959710251216, 0.13847624152046534]], [[1.1062446835994757, 1.9962159932688806], [8.9525734908394003, 19.576458158459218]]], [[[29.107961718150499, 21.194173184233957], [75.416890825899557, 20.673680928026986]], [[1.410050565366765, 0.25699297715608316], [2.5785580753641217, 0.0071681127595139551]]], [[[5.1516044655883153, 183.2781232658588], [18.894823172394798, 0.87358544669356153]], [[1.0818935331550756, 0.7996551515383401], [13.756126301974504, 108.29978614210975]]]])+(1.-msk_ref)*numpy.array([[[[0.86410204117027967, 5.7692015765994222], [195.19911437643682, 39.336361184462561]], [[0.85809925460649616, 0.013257515963752542], [37.435256149356881, 0.50261502391527424]]], [[[0.30443573429096282, 37.803048328588872], [1212.058587438798, 1544.2950755469212]], [[0.44360983007557714, 9.7523720471420834], [4.5048117504354783, 3.1728298626462546]]], [[[4.2440642990679249, 12.234341808324851], [0.05411174979489379, 57.565714103139982]], [[5.9161449721553012, 0.64565587220220833], [4.6891490267482947, 1.6354662709504328]]], [[[2.1198669239654708, 166.71440840134275], [2.1844020912744493, 32.029618191822919]], [[0.7276173509677033, 1.8281138669687931], [15.839709236868398, 28.532950946765592]]], [[[0.38851667834285503, 41.62286695572088], [5.2443016240545068, 28.277348871798839]], [[12.26121497157113, 0.12789540206503369], [14.62379762110848, 6.0464867845304031]]], [[[182.40891059694673, 140.59333647552933], [74.103762036190005, 0.0099949936010935263]], [[1.5054222400155728, 1.6624950105216358], [0.035784154705522001, 115.62956744246495]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.2810335633277168, 3.470343827116658], [2.1981944516896093, 2.2423302273514083]], [[4.3362125600729744, 4.0646437684138395], [0.32584680211849176, 1.3122523736223322]], [[4.7852024985099977, 4.0618231817127395], [4.5674528620361219, 1.9386569618982223]], [[2.7739406587054631, 3.215278388851774], [2.1581406352451027, 0.72223609110622]], [[1.1619052697465124, 0.51008311071866252], [1.3846620400461866, 1.8134030279519295]], [[1.8687267716589675, 3.8709565488627873], [3.8461155510529639, 0.79828481142195606]]])+(1.-msk_arg0)*numpy.array([[[4.9881209047718027, 2.1480651586848363], [4.1213592130583852, 1.170314109243678]], [[0.044477613167608211, 0.64194994625098212], [1.5868447701245671, 4.1419562776167016]], [[0.36861089722307921, 2.8315407779782111], [1.0884559243412857, 3.6382102564225929]], [[0.49381003498890919, 0.053071959990323388], [1.8952660387110523, 0.069774954162345693]], [[0.90975688583739822, 1.756753227592085], [0.89586066330711334, 3.4603358297526259]], [[4.7090349466805703, 4.7909265718289067], [4.1212797764148652, 2.3956609485059475]]])
      arg1=Data(numpy.array([[[[2.9438714486040398, 0.16213360977658373], [2.9396480456747804, 2.9681465461203489]], [[0.083034921493962952, 0.88720219878576245], [2.9253432621014239, 1.0458701452946495]]], [[[3.4791008612258798, 3.6255904888331281], [0.05494409982150638, 2.6401494599564028]], [[1.7938705912350026, 4.2664003777437136], [3.0596069185896071, 4.7744926169274358]]], [[[1.7484819389862787, 3.4091591759585702], [3.9027634625698404, 2.8187604902827328]], [[0.25501973833970032, 1.2605918938897742], [4.3869369232900404, 2.4762688578528143]]], [[[3.7622750799974383, 3.4286733371001907], [1.659098738572629, 1.9835635825083262]], [[0.57899697824502727, 4.894153211386234], [1.5461524512111329, 2.0927457741151203]]], [[[2.401359815343052, 2.9665159993987911], [4.4977412002570105, 1.5716586700226325]], [[0.11004086638345241, 2.515236085228147], [3.4207547899479382, 2.307769487577688]]], [[[4.0557019340328422, 0.054963920449582711], [0.39259823007953076, 4.340328064510989]], [[3.3045819647058479, 3.3429416960624239], [4.9435543293830353, 1.107211302619137]]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[33.042210034500876, 1.2124467482807757], [38.770819272946767, 40.170273101152489]], [[1.067587382556024, 2.0113229197747873], [10.614917623796631, 2.3269455000575499]]], [[[164.65385706422887, 204.12772611737319], [1.0800955889565811, 40.542487959444422]], [[0.13378553916869096, 0.0083621981676875491], [2.2966073615803668, 3.6599322608069107]]], [[[15.445196262687093, 207.91624228609794], [237.51668973011178, 51.980295435323072]], [[1.4730921314555296, 6.7854677424116039], [18.249401109663715, 5.1514524606617762]]], [[[46.457242299015611, 33.054868942632886], [6.9426569855002676, 10.141455902045044]], [[1.5611015723543424, 43.155628166747213], [0.60463944848323115, 0.50611762989982667]]], [[[1.4338322737082236, 1.5607379400930641], [0.048422212377031378, 0.34714535197409413]], [[1.0364624978489312, 2.2673209027829988], [7.6602993253394214, 3.9495293236328348]]], [[[12.627252249380915, 1.0349639488147102], [1.7012824532521644, 355.89693579608826]], [[85.753969731486393, 90.30161820048454], [0.32833115622203884, 0.77923434081696652]]]])+(1.-msk_ref)*numpy.array([[[[113.40618047052415, 1.297654390885056], [9.4646081964798299, 9.6730959292299197]], [[1.1247858314339831, 3.5128957059873578], [1.5841929828287826, 1.178787396555788]]], [[[1.9803768262397174e-05, 1.255204777473189e-05], [0.97594046399910706, 0.31029466563563912]], [[2.2894588557484195, 7.1706781713077801], [77.34037066102492, 884.7960017995091]]], [[[0.1746434385634229, 0.033293687582215785], [58.094918639077839, 18.799403391673085]], [[1.0218508080655986, 1.112764996767525], [288.78815265246612, 24.485499987913954]]], [[[0.070321494262294684, 0.088985047827622274], [0.0076634678595157426, 0.0029558943179616788]], [[1.4480054467521908, 22.853904373017794], [0.016299836200548458, 0.0038032698471232285]]], [[[0.79682873923334296, 0.75535550335952206], [12.607984549083183, 2.4243861135582043]], [[0.98797168875584596, 0.75835665134917063], [69.854008837616433, 17.545293870347017]]], [[[536.05735952275654, 1.0888974384388872], [1.8498286316787913, 897.93296405773754]], [[107.7516622555075, 113.76703632098781], [75.112090464551699, 2.630898176441153]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.1405820186494597, 4.9762310014130025, 3.2719331328842616, 2.9693922898043672], [0.41796028014751246, 3.5587103582810107, 0.46110092726293039, 4.0228177028177612], [3.7439517125434771, 0.21278195371742484, 3.8779266152472482, 1.5517124104122046]], [[1.8335136807249528, 0.64013531650410294, 4.1561356391309303, 0.0522530949482784], [2.4367354090581834, 3.4843960474305797, 4.015277937118725, 4.9265904747045477], [1.8704714560066926, 2.1131938415681737, 3.662224105257073, 4.8745195357120306]]], [[[1.3830321097933393, 1.5177257517964595, 0.90478870509170384, 0.52423190123907448], [4.641745908128116, 0.52625252789747556, 3.0789779332980713, 4.6186884513314279], [0.45899646743642286, 2.5115819240673516, 0.87309382460271812, 2.363930711298333]], [[1.8255752701582613, 2.9736332459778509, 1.8463911676067866, 3.574293249487086], [0.12765926194675367, 4.954403029057433, 1.897436305084198, 3.9831719825539595], [3.0063644731792101, 1.7910477387581811, 2.2252734730429378, 2.9922191389430499]]], [[[2.4754878862554159, 3.8056301559494465, 1.7655710234310134, 0.66242281073684861], [0.54497848874462163, 1.5166529529562911, 1.0860293140856654, 1.3840229002618707], [1.2390018786672334, 0.8959759455468026, 4.5215678637867134, 4.0923022554901749]], [[1.0291776004413213, 4.2972106285201948, 2.7719393530773786, 4.7614641748447912], [4.9695550673927515, 3.2678317859244359, 4.2339494559745035, 4.9597103630979724], [3.1916598108600267, 0.68701248959658123, 1.2253076519741746, 2.3902559820548515]]]])+(1.-msk_arg0)*numpy.array([[[[1.491676395000672, 2.8637021904985969, 4.4147409632783834, 1.5816116326972409], [2.0370001745891515, 4.8956336521316972, 4.1854134386317048, 1.664921318881168], [4.9491235746330409, 0.99712012370888659, 4.3862311525595725, 4.5986781558298766]], [[0.78984459640222116, 0.62581402622548687, 3.0598682109103708, 0.19621670833836913], [0.81133369623683826, 2.0263575279560881, 4.1733709241712074, 3.8559781017927044], [0.68459350454020584, 0.95355506377834054, 4.9974502138594143, 1.8381668323712235]]], [[[4.8927236656403448, 4.2751760869856748, 0.6722622493809407, 2.0616461766851368], [1.8251811432107254, 1.5963017584587131, 1.624235206913563, 2.5402087618300326], [4.6813912989360009, 4.5086668725974306, 0.57055042185970606, 4.054364995307858]], [[1.3216150090698187, 0.62554901830575382, 3.8364452368320747, 1.4752577263655127], [2.1101672587118188, 3.4431708970084531, 1.4183604730028148, 0.77968925648366605], [2.9519532046719457, 4.3150465330631462, 3.478638214760486, 0.49921325486770801]]], [[[4.1906322242885894, 0.72352604090995687, 3.5289044139609467, 1.9841174011394236], [1.9334925455221403, 2.3687267465851778, 4.5805052812473281, 0.18974389070993244], [2.416936588389448, 0.75582034558710065, 0.054146340201278703, 4.579458775697141]], [[1.6168576706057873, 3.4306629484138806, 4.0565240393636222, 2.9208716853703551], [0.016371799719774302, 1.189766669672659, 1.0478369360717998, 0.39672450590597041], [2.2770334121175861, 0.7858926652541478, 3.6623599343966293, 4.6720598437355578]]]])
      arg1=Data(numpy.array([[[[0.060301238434190184, 2.696453023283103, 0.79196841084113467, 0.83930520207447834], [1.149337134607431, 3.9247297214349137, 0.55043243663015839, 3.3255692897036746], [4.3298825486217503, 0.38143890410487852, 2.1088112685140428, 1.4907094716192302]], [[1.5777129060141515, 3.3125980521516665, 0.58702503548479279, 4.0005355529643403], [2.693356216941039, 4.6375263565277312, 0.69861320767027302, 1.1743042693764274], [0.47006223202604835, 2.1154919167361075, 1.8665806853190825, 0.7798823077430902]]], [[[0.49889660001435676, 3.6165555671342617, 4.7813426715215925, 4.7583674815143695], [3.3087339677012477, 0.39775553657370277, 0.21882353517637354, 3.4923211650565995], [2.4963849026159357, 0.2067862851783053, 0.80644796537338448, 2.9077029175269478]], [[1.2372436144715431, 3.1200819317482376, 2.3166960300981576, 2.948030815886312], [3.8653217389223435, 2.8076006753326914, 0.91548554235737889, 4.0519754268889638], [2.8964987566301605, 4.075167132648561, 0.97223985484533171, 4.1078399559971448]]], [[[2.0225357352278897, 0.07790113783713018, 3.2278221180446582, 2.1199131885857456], [1.098163469828179, 1.7681489978532967, 4.2556417207759667, 2.8862808682747172], [0.22531017542129586, 1.4360789994514136, 3.1892817617665274, 3.7339548361898074]], [[0.51947997621150865, 1.6338275166633804, 3.5733731429004978, 2.6386221196421076], [1.8829652978415734, 0.72815716670887787, 1.5218025436856415, 1.7378111395472429], [0.6516445952735892, 2.7249673009807518, 3.3954886079232556, 2.7531152002428119]]]]),self.functionspace)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.0079634862072822, 75.711098566255018, 2.5568711216035775, 2.4929430410846636], [0.36690733992002378, 145.77217002860135, 0.65304395040345564, 102.42537799294475], [303.7054571329295, 0.55417659975180722, 17.427915290595561, 1.9250593045520643]], [[2.6024764543174626, 0.22816882316641002, 2.3077371986420574, 7.4432320068078208e-06], [11.010644466348236, 326.68607830020011, 2.6409714148675656, 6.5051942288883051], [1.3422516115693561, 4.8686274066726796, 11.279124054579691, 3.4395834582167919]]], [[[1.1756031397405418, 4.5216365294632315, 0.61977939234261004, 0.046279588053169285], [160.64650153422227, 0.77464629388943151, 1.2790126905859569, 209.27271569670864], [0.14313503545148684, 1.2097722727507829, 0.89633157593274149, 12.201657549230848]], [[2.1057905097921088, 29.970602703230153, 4.1399118215514044, 42.738738620160674], [0.00035043270808554264, 89.383660157169061, 1.7974551200377142, 270.46644669140539], [24.246384546333921, 10.75112686220395, 2.1764063155609881, 90.22038847591142]]], [[[6.2545064770909349, 1.1097263313303858, 6.2647174048993479, 0.41765945668784726], [0.513453884150188, 2.088496634808874, 1.4207863842366568, 2.5549307743256078], [1.049470078100381, 0.85407063563670216, 122.99895651869258, 192.77901824699842]], [[1.015052427033331, 10.827239850981794, 38.214789180416005, 61.418772180613679], [20.47112464500368, 2.368437238877517, 8.9904856799180859, 16.164851967598587], [2.1302947157271586, 0.35952902288831318, 1.9935878896738162, 11.01288408165761]]]])+(1.-msk_ref)*numpy.array([[[[1.0244076063747511, 17.064143206877887, 3.2414891120972995, 1.4692832928691859], [2.2653478925948369, 509.69871274760254, 2.1989986506748203, 5.4482890141794353], [1016.7775681003135, 0.99890052320061173, 22.596949884044434, 9.7228438986607806]], [[0.68920773195292906, 0.21169230559975571, 1.9280560609046051, 0.0014810344686633137], [0.56943209930369665, 26.448672430220878, 2.7131912237767817, 4.8786573294881546], [0.83684140996117229, 0.90428672982580949, 20.149784679743043, 1.6076411934510346]]], [[[2.2080784225862922, 191.37280723639213, 0.14976265937276589, 31.271327115115959], [7.32136315959304, 1.2044537071121031, 1.1119747901446371, 25.937852039632311], [47.153495029809761, 1.3653631797510983, 0.63600981383712218, 58.567815529432259]], [[1.4120053461909794, 0.23137621354051086, 22.531287214702971, 3.1465002215784024], [17.930363231411885, 32.178460951032349, 1.3770777331251178, 0.36481171530103784], [22.997016211037728, 386.96525480857156, 3.3603124720174873, 0.057624559586043261]]], [[[18.137717091420896, 0.9751048570080143, 58.571205902939376, 4.2738274451737626], [2.0627703276016311, 4.5940768746966647, 649.54701109223333, 0.0082525760574437331], [1.2199823907899761, 0.66895895779221448, 9.1410282761057645e-05, 293.39330920767503]], [[1.2835146119960597, 7.4940294140794999, 148.99145566562893, 16.916539351216411], [0.00043371609563549703, 1.1348750527377172, 1.0737001385022715, 0.20056251184253737], [1.709534383817451, 0.51864266275177284, 82.081282851180291, 69.70015507359345]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.0926632274566)+(1.-msk_arg0)*(1.03398420003)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.67195400481)+(1.-msk_arg1)*(1.85765872144)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(1.49086732197e-05)+(1.-msk_ref)*(1.06404961167)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.4953224449613813, 1.6528671478066483])+(1.-msk_arg0)*numpy.array([3.8783948240354191, 4.4478205351514397])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.716840405805)+(1.-msk_arg1)*(4.02146939202)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([2.9371447599996743, 1.4336455104983894])+(1.-msk_ref)*numpy.array([232.94107489800635, 404.1144221423487])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.72724211257)+(1.-msk_arg0)*(1.61821407244)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.818081866334945, 0.12126894551138694])+(1.-msk_arg1)*numpy.array([4.3869855949006702, 1.8887184818399656])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([376.45059100979336, 1.2072827131725989])+(1.-msk_ref)*numpy.array([8.2610985867087923, 2.4820476039391512])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.5906266942449361, 2.3405488852825105, 1.1400590984472114, 2.0059585131379216, 0.19929124759332326], [2.5615130201293552, 2.3935035084161398, 2.2694066389013807, 1.2436607780454065, 4.5889175217209335], [4.9301967334904058, 4.8767787009014443, 3.2913816233036921, 2.9932087975441801, 1.3257162279884926], [4.0760164548899622, 0.74520672095605955, 4.9949516971700909, 1.1142805584534612, 3.9267910433026487]])+(1.-msk_arg0)*numpy.array([[4.497366078460435, 4.2031378578919361, 1.775938828131898, 0.73273008818628926, 1.197056886809855], [4.448178570844858, 4.6199230627407504, 3.5015959741399807, 4.7334653920046534, 0.62760172790550506], [3.7709063179737323, 2.3972024464174249, 4.6261502461668584, 1.5353936010980271, 1.4288604469006851], [0.57870858178265239, 2.3201727440127851, 2.7677150949674156, 3.2050523664585304, 4.315687731243778]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.94724683025)+(1.-msk_arg1)*(1.50892776252)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.3826533356151103, 5.2378463266823276, 1.2907782263944252, 3.878782723778619, 0.043244478918042159], [6.2437232752283016, 5.4710777707660156, 4.9322960803855285, 1.529001982620267, 19.431807111060628], [22.344863904601088, 21.875848025629381, 10.173330132610031, 8.4558309002480936, 1.7315757262878904], [15.426946650375978, 0.56401587998916236, 22.919906717937405, 1.2345537264723996, 14.346243801301707]])+(1.-msk_ref)*numpy.array([[9.666445705389993, 8.7282578802362174, 2.3788597476335713, 0.62547548193994806, 1.3118058841030877], [9.5073643398521881, 10.066669625327563, 6.6261021884729256, 10.442311610125648, 0.49513082859211555], [7.4099450178134232, 3.7406492008936634, 10.087151046156514, 1.9098185836263075, 1.7134381563482153], [0.43809589374570373, 3.5607686661084363, 4.646535165233928, 5.7978726318708045, 9.0833196001784007]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.56693954252)+(1.-msk_arg0)*(2.02602044822)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[2.0175279123532635, 2.3465149206594753, 3.4841326264743278, 1.8606529976385953, 4.1716157047528313], [1.8859088379487512, 4.1993920593128857, 3.6890994900160603, 0.53762218966670972, 2.269273224718658], [1.3032663771615791, 1.847885908903377, 0.1171444059030816, 1.6496961413032467, 1.1219480177988124], [3.9845273860243546, 4.7290549577007006, 0.25659407891793529, 2.3585174736165935, 0.0287886446266961]])+(1.-msk_arg1)*numpy.array([[3.5859464560752197, 3.4631188948780878, 1.3987798103786273, 1.8275610505772819, 0.49775489369583209], [2.9547164677869304, 1.0596776638543355, 1.4651817814474362, 0.053596759920808396, 1.6632991752003008], [4.9939100339089002, 3.3298330847413253, 1.7332682662375358, 0.5304898937828455, 1.461944254133936], [1.6465506373814756, 1.7444440653886593, 3.9106443938252342, 1.4092844043571313, 2.2099909582917499]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[6.6989614981711343, 9.1348181962198467, 26.696749691778876, 5.7780385904662204, 51.041651053887854], [5.9172589351609401, 52.395837690096513, 32.387321260779906, 1.6600108740393189, 8.4932899370772716], [3.4164702086994869, 5.7089125738766535, 1.1167623191541574, 4.735996815727554, 2.8796722803146078], [42.788575697477775, 86.327479839597089, 1.2736603499001653, 9.2387654428635422, 1.0275110981005953]])+(1.-msk_ref)*numpy.array([[12.577905718120634, 11.533045938383783, 2.684894284595444, 3.6342124318843094, 1.4211288765632302], [8.0546292963728554, 2.1132146430085545, 2.8137722663849516, 1.038568426382561, 3.2362377537293057], [33.990039633113966, 10.497182587525618, 3.4001338173495204, 1.4543584205724407, 2.8073475197182312], [3.1981923810806112, 3.427070191183534, 15.818851618809296, 2.7048822206960392, 4.7608015549797003]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.8001645101809025, 4.8155109540911347], [2.3444459777130811, 3.3913721675618631]], [[4.0630435804079106, 1.5938430850083494], [3.6601168996922646, 4.5460530023339434]], [[1.0023217417989683, 1.6209194667521654], [3.7101555535772355, 2.6649748973116796]], [[1.0416299201937502, 2.1286191991481744], [1.3818259854258486, 0.69803618413201496]], [[1.1257164215092905, 2.5895931742974794], [1.8112477559216857, 1.7089212843669739]], [[4.1255638551967788, 3.9305843509396059], [2.5482788868268114, 3.791434821017388]]])+(1.-msk_arg0)*numpy.array([[[2.053254403090679, 4.9071503575370476], [4.2043310760375041, 4.0900096489347355]], [[4.6942129497019538, 4.8671880487677415], [4.5435010982311432, 0.75296997092472495]], [[4.8354352418463371, 0.86200561648240037], [2.1498807283789438, 2.1352205768312493]], [[2.0030350996643844, 1.0427526623030912], [3.7667710986048224, 1.3444425311481751]], [[3.9933352716867243, 1.1078559781719715], [2.7270861710489749, 1.6942245275063605]], [[1.6041108879394808, 0.81617842914610395], [0.3383205312402906, 1.22708823533043]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.68912729956)+(1.-msk_arg1)*(1.25535952213)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1564.9363491999861, 1588.5357642363501], [54.345769680857536, 306.90079215345327]], [[716.11389263021624, 8.8980125527223795], [438.83330782419682, 1212.6399251854007]], [[1.0109336644070019, 9.6293746801791329], [467.68358449007042, 99.112685158903943]], [[1.2107670803823944, 34.553589647845392], [4.556209267435201, 0.1853206293732316]], [[1.7424372010466498, 86.635383683512501], [16.206566959384364, 12.338528243166994]], [[769.27143747964135, 613.03477820583851], [80.342194807945589, 517.70664036777464]]])+(1.-msk_ref)*numpy.array([[[2.4673343070289646, 7.3661273300586103], [6.0668583094185458, 5.8604904937123141]], [[6.9671111575326501, 7.290899969521603], [6.6874661439856915, 0.70034439583432839]], [[7.2312389367081407, 0.82993099513976998], [2.6139636292991262, 2.5916066653314762]], [[2.3918151147229345, 1.0539598749977079], [5.2850421676615857, 1.4499957474192331]], [[5.6871225923548909, 1.1372148998751017], [3.523373747021358, 1.9383887515800404]], [[1.8098514976624707, 0.77492308918095154], [0.25652982654369544, 1.292918269310533]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.61804675502)+(1.-msk_arg0)*(0.332300365683)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.4663602324458926, 0.20099251812009017], [1.7778656741484036, 0.097232021290753562]], [[2.5666156875171779, 2.9105194368922582], [3.4407109931757538, 3.9017986237829239]], [[2.3485177594635545, 4.8451425915060371], [4.7179518786670549, 0.68084991817519225]], [[2.2105649056989165, 2.7540543056180895], [1.5869777494988133, 3.5639190760463095]], [[3.7804094050216732, 1.4983161895578418], [3.4383747455075482, 0.17393768957511158]], [[0.3922122091779347, 2.8746528301273711], [3.3457160429352202, 2.5533807584584522]]])+(1.-msk_arg1)*numpy.array([[[2.2380572571583346, 1.7717588920644172], [2.1741166212433285, 3.7751452813386703]], [[3.2225218323985341, 3.1337493265528855], [2.5690366741327404, 0.6632504003256291]], [[1.2802755255362082, 2.0756834117828009], [3.4306113666489289, 4.4403046228630911]], [[3.8740508382910206, 1.3490975126798421], [4.6503177886258094, 0.59575903800748142]], [[4.7168770557483048, 4.8245651635086118], [1.0167033076267384, 4.299994322553335]], [[4.1191850777311307, 2.2182293936489539], [0.12539760462880975, 2.2283124597114159]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[23.845064177206751, 1.2949384629432412], [9.8376658107280903, 1.1331869719156467]], [[27.126176185495389, 42.213361881228934], [83.473424869326152, 151.02671174967875]], [[20.492102418139844, 508.02725321156203], [431.37431465375005, 2.400142881625265]], [[17.161032413959781, 34.519794826718162], [7.6963726768523166, 97.804033367531659]], [[129.19955653256974, 6.8670608790584717], [83.223025232994715, 1.2506612892025453]], [[1.6559239591092472, 40.310609727130341], [73.874746953834688, 26.668415450083337]]])+(1.-msk_ref)*numpy.array([[[0.084949060004740937, 0.14199333982377182], [0.091149061478416268, 0.01562101789268432]], [[0.028715912329566402, 0.031666314949837994], [0.058992272596205785, 0.48156523206878055]], [[0.2440212334988959, 0.1015896486639415], [0.022832748330702157, 0.0075067349961130611]], [[0.014008320484205371, 0.22620303014028245], [0.0059561586499802107, 0.51873751966658099]], [[0.0055350264327891276, 0.004915800602959786], [0.32624119654507333, 0.0087616366555563572]], [[0.010692934100194292, 0.08682516138061587], [0.87096581771135562, 0.085865986305921432]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.1559098990494756, 2.8320101047093047, 0.12096003182303475, 3.3248924327015854], [3.4936793993015534, 3.1505438802546917, 0.67943521766522819, 2.2985626993449708], [1.9863446972459955, 2.5910452376785016, 0.58465076014997619, 2.5371712155841091]], [[0.9890035796971578, 2.0458336743614693, 2.7433805667077746, 1.3062907835118907], [0.463286285137614, 0.21995598054384191, 3.9311547041783679, 1.3450829156748163], [0.83150173566446584, 4.0079872695516077, 4.676141313826129, 0.37813382172058607]]], [[[3.0895475858082264, 3.9850221106316512, 0.32299198292619258, 2.6870523515842324], [1.877895997477828, 0.26258787868052075, 3.5571886598495541, 3.9814208729839184], [3.2617867895812949, 0.7691523762443343, 3.3465633081945239, 4.7985414545729839]], [[4.5212515047440665, 3.8424000104020677, 2.7712940837017568, 0.16291178103543372], [0.094104178750517686, 2.6340635191004504, 3.9399624139454019, 3.8765167501772879], [2.6641085155030706, 0.39000361969600711, 4.286433727249447, 4.9799815910340754]]], [[[4.4864028679293089, 0.80160432130711456, 1.4013457186375917, 0.13760722444037787], [3.6961476392397477, 3.4341393884491649, 2.9315764910830264, 0.58716115017092396], [0.35038390226402011, 1.8857667650007044, 0.2726806736686529, 0.75075880296172992]], [[3.0506349134025323, 2.2525610873731474, 0.095957802477346185, 0.34491493941146639], [3.1923682001911904, 4.1806243924749111, 3.6584816833438021, 1.5782623262523598], [2.8112913170213867, 3.5073241321025219, 1.7752531596846495, 0.66609554985876129]]]])+(1.-msk_arg0)*numpy.array([[[[2.3300508660197154, 1.9298567507449289, 0.26393231864130817, 4.469783991499793], [3.2195585901471522, 0.286261328015817, 4.1337321630765045, 1.1474678857386031], [4.9108260300239284, 0.84777535328342923, 2.1078468696026609, 1.6953471451483937]], [[1.2827907070717492, 2.6978964009925934, 4.6821172341776878, 3.6840673658048435], [4.2572923647806267, 0.76573504477192789, 3.4197505475689867, 1.2953464602877385], [3.0377036216234741, 2.9826683903454132, 3.1137351527100758, 1.0784138245237802]]], [[[2.525614800623698, 2.2035089650078801, 3.5347308448842578, 1.2142172782388954], [0.36505416804594804, 4.2851693090800556, 3.478497474428393, 4.7091747849808021], [4.0803213487810943, 1.2208987018651209, 0.47464973433771629, 3.2190649097080217]], [[2.5503149534115366, 0.028155124699867723, 1.9337712844431438, 4.2239865392429268], [4.1641177651874495, 4.3237697154171588, 3.1114900473999714, 1.9176763492376632], [1.2186699206480782, 2.8033480195562386, 4.7265502916163129, 0.17755649915046121]]], [[[0.71150186589601405, 4.3567348639405088, 3.8397423812986626, 0.83810335415160586], [3.1346016332034092, 0.09739779416684495, 3.7928534314338807, 2.4039784077427258], [3.5743587788616065, 3.9342653821590066, 3.819765064309077, 2.6879146973404389]], [[0.47532429872684673, 0.77966106793086887, 4.3171860699602025, 2.6837391288764523], [3.3875813791697311, 4.107084600362537, 2.7226738805737614, 4.7372448810063696], [3.4583378132331672, 2.5018831448536352, 2.6790197941141924, 3.9220864556544668]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(1.42421946106)+(1.-msk_arg1)*(0.268962724382)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.1388147570942184, 4.4043539217249981, 0.04937200114758953, 5.5351043347011482], [5.9395602391192552, 5.1263750325507953, 0.57668943111652116, 3.2718484166876189], [2.6576340757485957, 3.880423582484259, 0.46559626253669589, 3.7660217124700361]], [[0.9843752908344966, 2.7717085633359853, 4.2093547987451512, 1.4630737915133212], [0.33426913299305494, 0.11570225190023417, 7.0263074034851751, 1.5253408879644286], [0.76889591407327329, 7.2226973581248384, 8.996363508779833, 0.25030788802697723]]], [[[4.9856046227840514, 7.1638278063683716, 0.19997758912180602, 4.0868002890849562], [2.4533998135249009, 0.14890856904569799, 6.0939259255842737, 7.154609315463337], [5.3860876234033714, 0.6881080236876479, 5.5865561647400916, 9.3335971106397579]], [[8.574960824150704, 6.8014633788252716, 4.2704849415148303, 0.075447942757859626], [0.034529781760223144, 3.9725016099353123, 7.0487386379204207, 6.8876341521639732], [4.0371911534169822, 0.26157244731616863, 7.947738806299065, 9.8402314234708985]]], [[[8.4809831852132156, 0.72982382003840307, 1.6170098126667511, 0.059324772868375633], [6.4357566907052153, 5.7959190012148802, 4.6265223288653834, 0.46844613953833275], [0.22455922862572211, 2.4680578704099765, 0.15712596367262605, 0.66479124912942822]], [[4.8964125372965288, 3.1789876819642546, 0.035502500681872912, 0.21958387339202279], [5.2235711079026323, 7.6697946454162897, 6.3425526772313754, 1.9153610827946173], [4.3585341137984415, 5.9726255451970598, 2.2646514098685753, 0.56063124091286287]]]])+(1.-msk_ref)*numpy.array([[[[1.2554736419794974, 1.1934262925755792, 0.69888073062213885, 1.4959009510353336], [1.3695512389182716, 0.71431440929329559, 1.4647824006951606, 1.0376908285800339], [1.5342453316943958, 0.95655557002691305, 1.2220828093897345, 1.1525559795203297]], [[1.0692761498096859, 1.3059595650230553, 1.5146907845553645, 1.4201067909713869], [1.4764320142965024, 0.93072513784568156, 1.3919530793168771, 1.0720810741710305], [1.3483005918023554, 1.3416864884049646, 1.3572954141800397, 1.0205118758536891]]], [[[1.2829855208565992, 1.236759050681246, 1.404388995411376, 1.0535923089889594], [0.76258881332568285, 1.4790260769367778, 1.398344519199116, 1.5170401306506875], [1.4596677771729261, 1.0551485099124489, 0.81838274782965392, 1.3694947524645813]], [[1.2863483116305432, 0.38281482597162508, 1.1940769028567295, 1.4733164407607258], [1.467670600450192, 1.4825977107784443, 1.3570321231790736, 1.1913956726482218], [1.0546300878328116, 1.319497069247185, 1.5185436082654928, 0.62820229653025739]]], [[[0.91251679381700457, 1.4856295156228747, 1.4360034433074149, 0.95360804834992607], [1.3597358815612937, 0.53451194625995135, 1.4312657834343105, 1.2660653466167271], [1.4086064725962322, 1.4454269515292451, 1.4339901351560682, 1.3046582291396476]], [[0.81869540849877254, 0.93524781557677017, 1.481990189970728, 1.3041128032713261], [1.3884191249347742, 1.4622367061488966, 1.3091747135596239, 1.5194669879890079], [1.3961601808003721, 1.2797318625606113, 1.3034956016162593, 1.4442221219890676]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.81747582602)+(1.-msk_arg0)*(4.36892346688)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[2.8873721448218403, 0.83446374038527293, 3.2494030511713614, 0.21744760860339554], [4.7958342156790019, 1.8479359715269446, 2.4938634921649458, 3.6240944378638984], [2.3000306419881249, 3.8732790365446403, 1.868994059724743, 3.3918260071570483]], [[4.4733461361205, 3.9569395354273733, 1.8001835370495582, 1.3109351273792849], [3.4129753289539901, 1.8960139591391232, 0.43720124707410646, 0.17392465379051519], [0.95845247603653627, 0.60755216698350512, 1.5991951052099926, 3.1664871913598085]]], [[[3.5183268881637186, 2.1333376859033182, 3.807110912809105, 4.4341137439278624], [4.1567913430175736, 4.3077774242433629, 2.7037161010461146, 0.50181383816233083], [3.8217371704994929, 0.52712689942857971, 4.0258075574541863, 4.6964332295131399]], [[0.65395095321290986, 0.2466426119864577, 2.0461501131694821, 4.426390004903392], [4.5751689575657402, 2.3710756840392282, 3.9604307206095322, 0.95963506393199971], [4.2312276685370067, 2.2570107014334782, 0.50145019623290943, 1.5826489046082635]]], [[[3.7850149257692673, 1.8616766804184031, 4.0440953970807492, 4.5534151950503299], [2.4537840527373986, 2.0286723557389199, 3.4300877036174136, 3.89814127197236], [1.1679456457614352, 2.0030618481226132, 1.6251878538495292, 3.0397111878744068]], [[1.2550544695913177, 0.097066592597493076, 4.0749889536555264, 0.78702912419822146], [3.8480143173049637, 2.0027584133380896, 0.65428053374037165, 0.065328993463125484], [4.9841948653846213, 4.3596423945629397, 1.9925774661066034, 0.91398781851756039]]]])+(1.-msk_arg1)*numpy.array([[[[1.7388265069846918, 1.5085909733431107, 1.3015745064624955, 4.9402803304591414], [0.75674651200552845, 0.54727497625281896, 2.5760454934261983, 0.36734106430446678], [1.5555277081197996, 2.5492511777224363, 3.7579959306394812, 1.9320930320497018]], [[2.0232599123266763, 2.9881622327203234, 2.8796761104721873, 0.8909347568062802], [3.0547336764379294, 1.8718116962964331, 1.7405208886403634, 0.9509429948719299], [0.26192960472534599, 2.9036366158485971, 2.3970559183825686, 3.2955253819716699]]], [[[2.218720053537151, 3.1301531159774743, 3.4561578757448084, 3.7597972561201556], [0.60547446109667902, 3.1820813831908388, 0.43086533946490252, 2.966282048959378], [2.3764083456736511, 3.5302899897379376, 4.7165989083182938, 4.3759281860025929]], [[3.1004025957972723, 1.3331064864872073, 3.015102586937553, 0.9045740879179599], [3.6505490607638769, 2.840644796704193, 4.63910497747076, 4.2444411309496264], [3.2795305378132737, 2.3767381946193455, 0.75279606613113781, 4.3705018419136445]]], [[[0.52448275955167944, 0.91755110114864913, 3.5816055791933628, 2.0937602056764315], [0.32128585193939524, 0.91320280031641099, 2.9303642007567947, 0.90880818006276598], [3.44080127464728, 1.1293732064166828, 0.39842589331383188, 0.49435966019542987]], [[0.55109947572353801, 1.6233027514261338, 3.4973380968545902, 2.750872506866374], [1.518831513240408, 0.86268483140137298, 0.055757563414975155, 4.7917531928740296], [3.8076712780318052, 4.549106724487439, 2.5269175306800733, 0.93350462151985725]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.6128383214724913, 1.6463310701287612, 6.968161384861042, 1.138730192972542], [17.553696951613485, 3.0163479571263392, 4.4368973004347909, 8.7164344039180008], [3.9517169495189588, 10.115665407429173, 3.0545367358384237, 7.5870420599439683]], [[14.477482611217264, 10.6341242910513, 2.9315086477248902, 2.1885000389095937], [7.6835173711757063, 3.1042462792233545, 1.2984942006913021, 1.109501755985947], [1.7729168328462306, 1.4376088457596872, 2.5998062529876416, 6.6313833553710539]]], [[[8.1826787948279467, 3.5771259726093665, 9.723573528067357, 14.142086081075153], [11.982764092694586, 13.113939113933398, 5.0295548418240719, 1.3495996460076052], [9.8089146999268486, 1.3701651002648021, 11.080792549309336, 16.541586389000074]], [[1.478018147493348, 1.1587667471286556, 3.3955630062031328, 14.076977236541378], [15.385544875479821, 4.1230612869055507, 10.656328136321379, 1.7741699041759587], [12.527687218643711, 3.8514432597038972, 1.3493064673834563, 2.5742324687448503]]], [[[9.5960540800596537, 3.0412121867717605, 11.202525603659675, 15.186876527377978], [4.3319159195314896, 3.3602908037612784, 7.7624747787898194, 10.267044074787043], [2.0093022561858862, 3.3092664757259587, 2.6404946214657432, 6.1476585377959285]], [[2.1166413700506923, 1.0597068388327187, 11.411214115671926, 1.6003293819155773], [9.9641223924685551, 3.3086666041157256, 1.4783092089100465, 1.0398024234170962], [19.644558764104858, 13.52665758644045, 3.2886024148917619, 1.7264387653297113]]]])+(1.-msk_ref)*numpy.array([[[[12.986736995989231, 9.2483252570939189, 6.8154579362126153, 1457.5693761542454], [3.0521166042596555, 2.2410982929111061, 44.630698353910411, 1.7188414858033321], [9.9110617112745967, 42.901777643676155, 254.99080982916365, 17.26883291816587]], [[19.753494109014575, 81.94882055023939, 69.834651130907403, 3.7199049518841871], [90.401025719503906, 15.800119117884725, 13.019223546112343, 4.0640550766316785], [1.4714076957328819, 72.34602315634092, 34.27783106935577, 128.93481783602476]]], [[[26.351880056065237, 101.0345797294685, 163.39364221850494, 255.66898718610037], [2.4419140132166026, 109.07458622451341, 1.8876225164499558, 79.347125275096502], [33.249962235453495, 182.26675620729657, 1048.0677808592216, 634.20921550973969]], [[96.698239352054486, 7.139820865912534, 85.269677714270003, 3.7954748871407418], [217.62976817083668, 65.928973257751579, 934.89801570640213, 522.43420020359201], [125.92951237531668, 33.266137878331072, 3.0343897103792181, 629.1550057060947]]], [[[2.1670321365625727, 3.8687999163600257, 196.59327440724593, 21.91747464741945], [1.605991515443155, 3.8440739085716449, 75.25412755247406, 3.8192450693426716], [159.73540197122054, 5.2871512099202338, 1.7994581764637512, 2.0728853827392753]], [[2.2537721941042408, 10.952709916170726, 173.62246374613227, 57.754846345416318], [9.3890332086989385, 3.5681352932592412, 1.0856897015196805, 1170.8904207996156], [274.36919613827246, 818.7129453261191, 41.511974507366574, 3.9608873995356526]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.038556365881090439, 4.7886684643606392])+(1.-msk_arg0)*numpy.array([4.0865885737429748, 1.4716922747753571])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.8619595488162497, 0.88311876373182197])+(1.-msk_arg1)*numpy.array([3.7789219196657471, 0.8559498925598108])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0023300654433039592, 3.9875893472491075])+(1.-msk_ref)*numpy.array([204.30790366802535, 1.3920119123574608])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.50564350356244236, 3.0777360990451879], [4.4842647497293884, 2.1801023923881768]])+(1.-msk_arg0)*numpy.array([[2.4911152466742528, 4.1509467106410529], [1.1684495169478035, 3.1593518450983189]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.5636541681845579, 2.1588117738462884])+(1.-msk_arg1)*numpy.array([4.2411170925948003, 2.158542845402974])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.088024582756551603, 54.939890162828981], [25.519886191566805, 5.3790787018850068]])+(1.-msk_ref)*numpy.array([[47.990204939552619, 418.4412372064258], [1.3993906927516704, 11.978534174761263]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.2861711509356246, 1.8034305781241404])+(1.-msk_arg0)*numpy.array([2.9032697923727051, 4.9046614755877558])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.5369434251436238, 0.93777440143146029], [0.18617665413673279, 3.3451689464684335]])+(1.-msk_arg1)*numpy.array([[2.3475828726848889, 2.9695781012443088], [0.53391274501574182, 0.15354316454901426]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.435474265090674, 1.266186220818059], [1.1160399342193996, 7.1894531406750879]])+(1.-msk_ref)*numpy.array([[12.208618459119734, 23.69082980609523], [2.3373565518218062, 1.2765513730010647]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[3.6194798116747249, 0.52011445885250696, 2.7071650118830015, 4.0766022773431363, 0.58982054532234507], [3.2435514783300063, 1.7187800577221795, 3.8914326996220661, 3.8585607901345798, 2.2359058037478814], [0.046950321601889188, 2.5628269198269766, 3.2304117794319733, 4.7973783466758473, 4.1412240940218759], [0.075454038276931742, 2.8706704990373857, 1.9909459830765037, 3.3451729328286728, 2.6002494572223598]], [[4.0531522419285757, 0.91942172911744757, 1.6082216713589568, 1.4888648084975364, 1.875150584953674], [0.1813594250236798, 1.8393640369279791, 1.331378498396405, 2.5295263068339202, 1.9528249324783893], [1.6331248568407368, 3.7471344444002472, 2.4341206317727946, 1.0450864082512443, 3.7866608566010131], [4.7357179640110152, 4.9335161630695819, 2.3703091094566084, 2.0092391577454087, 1.8144346123915238]]])+(1.-msk_arg0)*numpy.array([[[4.517752798780907, 4.9510355458696793, 2.6060482785159884, 0.91758664014717595, 1.6046630977979053], [4.9384721343757256, 3.5321441153811395, 3.3894973918700679, 3.3760117040054647, 3.7634881393889135], [3.2653553347946072, 3.2131677186024739, 3.8352281645221868, 4.6979230674393611, 1.7370395571561525], [2.9953672600724195, 4.7455373836917305, 1.6338679716676177, 3.8924295428026467, 2.1557972131099432]], [[1.1381810845404134, 1.3359632309369429, 0.60717254473210513, 0.59118534234845677, 4.2185075471074462], [1.6007673475989752, 2.9467072072771314, 2.3297450506815109, 3.7727701374669844, 1.143504513475275], [1.188057439586149, 2.1217962639922177, 2.7560419868744868, 1.142865973334203, 1.0777011067346174], [0.127111370115636, 4.6611913836220333, 0.10884870573877785, 0.91730802040035719, 1.4124333811964525]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.1232877675502742, 2.4376299038009446])+(1.-msk_arg1)*numpy.array([3.7822098566574867, 3.0708673368506645])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[4.2415090341459374, 0.4798410486909051, 3.0608250272310018, 4.8477550000710705, 0.55265291389835836], [3.7499321341460261, 1.8374693033860861, 4.6011120508237484, 4.5574762971321521, 2.4690895734976759], [0.032200888599453935, 2.878123094788454, 3.7328724706176621, 5.8205420326100041, 4.9341591767146014], [0.054867410956683682, 3.2692421571752082, 2.1673540044078288, 3.8821558775405256, 2.9253732714848306]], [[30.30918015806223, 0.81482153080927067, 3.1841605763299023, 2.6385003895430073, 4.629787769876236], [0.015580997181970677, 4.4173506808248693, 2.009098124468788, 9.6041825726798464, 5.1112789035811428], [3.3056923497948381, 25.030326222096459, 8.7449846181744473, 1.1134891113641256, 25.678823237975983], [44.293571091883727, 48.939457110329997, 8.1966396194678559, 5.4787183089000697, 4.2728297002616147]]])+(1.-msk_ref)*numpy.array([[[299.95368192295706, 424.11753278878041, 37.439750771404817, 0.72230966202618274, 5.9814280060015257], [420.06142071339104, 118.24867821663764, 101.17727360131857, 99.663148650259885, 150.31567036585452], [87.860325095206562, 82.666296943325861, 161.44360755403073, 347.7676012772074, 8.0725481427810042], [63.391960512835354, 361.28780446563087, 6.403704539014166, 170.74137577401339, 18.271538229685998]], [[1.4880504002105515, 2.4338748211929282, 0.21606294099474685, 0.19906441725306165, 83.134221198941063], [4.2409665551090185, 27.623049281078963, 13.426269530061493, 58.999376295536479, 1.5095267486294113], [1.6975236926963253, 10.075431643548509, 22.493620972185912, 1.5069397210389601, 1.2583401577244746], [0.0017744719309234476, 112.94458198244885, 0.0011020772528306799, 0.7671656052013156, 2.8875648729855339]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.1509043294186494, 4.4470658985303357])+(1.-msk_arg0)*numpy.array([4.0800398779638822, 1.0142611514429811])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.6898381422118565, 1.8169568769540765, 4.966311990879869, 3.0179414778435287, 2.5810147978559659], [1.4557849099101778, 0.48724762823973661, 0.1614228264696016, 4.5650598294329807, 4.3269617278617503], [3.6227817073081061, 4.3606779968478655, 0.7980996822301164, 4.2332967593544666, 4.9706169647319216], [0.37492520264102319, 4.5452632647216156, 0.052556927020817872, 4.8052020982033827, 2.6679797866417894]], [[0.25157620955602411, 2.8901462300717888, 1.3905925713062439, 4.9453925836215458, 0.23200440755475432], [3.8561699717874731, 2.2499535504566177, 0.57460468055895664, 2.6968491035199178, 2.5268854785552923], [3.5774857661167969, 4.8560528089718931, 1.6491004392696571, 0.93509719702419292, 4.0583738227118555], [4.8815683559372838, 3.6594060855818658, 0.71121082246920719, 1.5198766672193547, 0.62988417304089872]]])+(1.-msk_arg1)*numpy.array([[[3.4677326333027532, 2.5885603009516029, 2.1254031963654905, 2.3030043652842607, 3.4596086620464805], [1.2407333528951634, 3.3142535086562357, 3.1863573214797638, 0.23227203197938343, 3.4360734362534866], [3.7365471471060649, 2.420319101522745, 3.9711799760897266, 3.2295447234548162, 4.8366232401238998], [4.1481398242212348, 2.423134136031285, 0.97569231584601024, 4.5059764090133694, 3.1041649178828141]], [[4.399842118820855, 3.074726250087604, 0.94124300941166494, 3.7969985722741573, 1.3568164283996469], [3.07405114824215, 3.092185407433234, 2.7283593511371009, 2.108737175584253, 2.9577496449980889], [3.7688414152714804, 0.12023138875381124, 2.5289033251684705, 4.4064393300140825, 0.82182016359369869], [3.2285420960370179, 0.87236595521333404, 1.9915455963323632, 1.2024379969332657, 0.61477482887184198]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.9331434390565148, 1.2909387506651211, 2.0097397804598951, 1.5283147522265688, 1.43728581631049], [1.2270435612614206, 1.0708810093842067, 1.0229469788086456, 1.8995367862440677, 1.8370221174569492], [1.6639174767397376, 1.8457479726144972, 1.1187045001554208, 1.812997261916923, 2.0109561525048192], [1.0541080824836646, 1.894258922308667, 1.00741412082923, 1.9647432534219875, 1.4549612108669061]], [[1.4555927742000172, 74.649580388621033, 7.9653760741391402, 1603.1649240581748, 1.4136956560377367], [315.55990822980095, 28.71675868273055, 2.3571439406367922, 55.944399035603865, 43.411788709839932], [208.19675412500561, 1403.0703787763268, 11.714895011484854, 4.0365633634066072, 426.70211119581154], [1457.5229690294645, 235.26880570485852, 2.8901270397649856, 9.6603283954529608, 2.5598309260437806]]])+(1.-msk_ref)*numpy.array([[[131.10549517689455, 38.083919931162129, 19.85674617726978, 25.489586897793334, 129.61637612033155], [5.7236319364302881, 105.65660615217149, 88.26631728088033, 1.3862458623206551, 125.39717433698823], [191.32703930540308, 30.060952058608397, 266.10820692209245, 93.792479327768305, 898.57362502303181], [341.2882602391557, 30.18017629654517, 3.9429436486903979, 564.46811582049622, 78.632671623198462]], [[1.0642854156164026, 1.0445011555757031, 1.0134176132637225, 1.0552387938666126, 1.019398846647283], [1.0444911704804598, 1.044759418436598, 1.0393907335407129, 1.0303108980477216, 1.0427724367387354], [1.0548181359742774, 1.0017039768202336, 1.0364592402271227, 1.0643848450239266, 1.0117052938885991], [1.0467786594494026, 1.0124296806839508, 1.0286025338317595, 1.0171728104682347, 1.0087434712244925]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.4739605763785182, 4.5742633813993292], [3.290946687146429, 4.6642024236537871]], [[2.3386633239515113, 0.41755344416994006], [3.1144033603049301, 2.3565387897423768]], [[4.6584277824451075, 2.8465194189834868], [2.7527969058331756, 0.41418496148458217]], [[4.6045147733019869, 4.9603123620166363], [1.8028562602027827, 4.6516565914133787]], [[0.99055118369882755, 0.63845707767696924], [0.61808100663678089, 3.4116670138274126]], [[2.6963603359570314, 3.8406233827219047], [4.5108766996040295, 1.5861192860775999]]], [[[2.3897367690972318, 2.683077967399897], [3.99672692140421, 0.23492450191789824]], [[3.57260121731714, 0.82576676549978878], [2.6549285266535336, 2.6375691929339027]], [[4.1433312240136972, 0.16130600127124978], [3.1688863286653675, 1.8285548308115798]], [[0.14571252972217816, 0.87354534188846156], [4.2990321203673618, 2.1116361859080697]], [[4.220645344816174, 3.7129939518892123], [0.10988329988884649, 2.8147910297817065]], [[2.7857194508194341, 0.11988888127721486], [1.6757521090457217, 2.8016012826687211]]]])+(1.-msk_arg0)*numpy.array([[[[3.1959329930629368, 2.3649880482583758], [4.2850325864988585, 0.45978222070304753]], [[0.50358945680602585, 2.0813700759094584], [4.0724704524230679, 0.96458694633875219]], [[2.0875778154150977, 3.4423301168435048], [1.595935139465972, 1.589244287515164]], [[0.88401871617664163, 0.18629721402662366], [2.8339218214844148, 4.2595476905643119]], [[4.5020981103568545, 1.7375651486516961], [4.4799948891283785, 1.2274667288523071]], [[4.5780106295317893, 1.8511799458704949], [4.1145733038847085, 1.2638790959005386]]], [[[4.9458282489926928, 2.9425429074618745], [1.326941108250369, 4.3291973183599133]], [[1.8355745936980041, 3.4824141052645139], [3.0195141631232518, 3.4275000316985658]], [[2.598470501124619, 4.477310170267164], [2.0726099028994169, 0.035057483329073352]], [[0.01705140480364049, 0.58884499936479195], [3.7658689443494655, 1.7064776834394986]], [[2.5880211922401712, 0.21056305182156254], [2.4755164997887853, 1.0745497169486617]], [[1.9085850359412542, 2.2823556194582051], [0.25087379470318194, 0.54121108180936306]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.6352855916112947, 1.8977685754136455])+(1.-msk_arg1)*numpy.array([3.9159810118321334, 1.0461869133495456])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[51.851300675253057, 54.971154466458103], [23.082783272028873, 57.865461429775657]], [[9.3828608668593176, 0.10010781532447743], [19.961062660726594, 9.5730399356143536]], [[57.676855658454691, 15.748802473861165], [14.418847136485892, 0.097993602416965836]], [[55.934389085390379, 68.055764138394224], [4.7264014277429576, 57.456188009579364]], [[0.97529165917463601, 0.30652534639136464], [0.28141351343129856, 25.381617668347843]], [[13.65283595385662, 34.678909860530318], [52.986407009330527, 3.3724239750137599]]], [[[5.224212877496071, 6.5079887543793573], [13.864232921044835, 0.063998165335161339]], [[11.205640524738071, 0.69536781615140819], [6.3790226256999798, 6.3001001357012223]], [[14.845224550897029, 0.031354802255203602], [8.9249320891918522, 3.143549491315289]], [[0.025852985339636848, 0.77370138857968207], [15.92175702384996, 4.1309688362822472]], [[15.375326580095651, 12.056042290660116], [0.015132486865637297, 7.1276242318708771]], [[6.9885676143425055, 0.017854001214028931], [2.6637794527794711, 7.0643737632178691]]]])+(1.-msk_ref)*numpy.array([[[[94.622599614863546, 29.100943592770015], [298.34610293639207, 0.047704650577639104]], [[0.068129897662004932, 17.646144810352038], [244.45117029339639, 0.86832267634619553]], [[17.853141123892527, 126.56227148892995], [6.2374031302397261, 6.1356248905542241]], [[0.61708372636160846, 0.0013872092128280182], [59.093850703290975, 291.45764868899164]], [[362.04333478004082, 8.7017134992547547], [355.13248153587102, 2.2313125559725098]], [[386.54298037822673, 11.151255935754868], [254.49793772804833, 2.5019402134171851]]], [[[5.3248059890399899, 3.0929412013797237], [1.3443915901190124, 4.632346885084381]], [[1.8877950988588561, 3.6889961070592285], [3.1776340675736141, 3.6281599535304614]], [[2.7156408040621436, 4.7982807566192545], [2.143564413179559, 0.030030920905158041]], [[0.014128323311318251, 0.57461643973718513], [4.0037102352630392, 1.7491241014938972]], [[2.7042169933038833, 0.19594368766951656], [2.5813567821774548, 1.0781241401919484]], [[1.966421947820064, 2.3710239065974816], [0.23535205400547746, 0.52607990195933374]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.9169745528398314, 4.7900973802263076])+(1.-msk_arg0)*numpy.array([3.7040647130193727, 4.7161046767201622])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.8133638638582417, 3.2774740376530835], [0.31067455695986712, 4.5389611120303517]], [[1.3591405855917289, 2.0724511239101173], [4.9757034764313035, 4.8484776096465803]], [[3.590778438058412, 3.6247513195899055], [2.7865460068367613, 1.0135973901381989]], [[3.112614260414635, 4.4830815154142751], [0.3721550193816654, 0.87100228784756328]], [[1.8392456306125371, 4.2826049031653808], [0.15934319798370594, 4.3929026758435485]], [[1.439132547889272, 0.077554186956850596], [2.8228297376211477, 2.3511283709663013]]], [[[4.1769140605064132, 2.5219374827209333], [0.47767537426107437, 0.15907488814858256]], [[3.0258368328710241, 1.674932762286427], [2.5168552243273665, 3.544532093762442]], [[1.8276009695880429, 2.1098923113073917], [1.3891585185841477, 4.9979426303521457]], [[3.9403466293747171, 3.1877183991701825], [3.3631451661193577, 1.0072988684898518]], [[4.4092191496508857, 1.7974697091322969], [1.32364920352538, 1.782539189145351]], [[1.9038090321636425, 3.614761920515666], [3.9588992806237475, 0.023906637783880039]]]])+(1.-msk_arg1)*numpy.array([[[[3.1521478211263441, 4.6641583758629981], [2.5961353933160072, 2.2457055140736286]], [[3.5385624787250198, 2.351411346800913], [2.8218688330028017, 0.7034042931794281]], [[4.3159686057992399, 0.52525031205627182], [0.32565219352381847, 0.72482281454245989]], [[3.8277748178005688, 2.7498033487260756], [1.1192883655797463, 0.36450658723900553]], [[0.13693713354481873, 2.7049318343126023], [0.24336564019393536, 2.3913794457129045]], [[4.4029276229646301, 2.1481153426941804], [3.1639015566752553, 2.1799328093582129]]], [[[4.7263929369659126, 4.7272161497610687], [4.5642591876746295, 1.9214086138264015]], [[4.8689964748401664, 1.9916559100257827], [4.2000710433336232, 2.8480328622539663]], [[1.6742185360983539, 3.6760933741680528], [4.9985391520626647, 0.76788268254172021]], [[4.2772030919932682, 1.9631929294254722], [0.10246686208793201, 3.262099138640318]], [[4.4594779811918084, 0.19066572171875662], [0.8874051755785517, 2.4460914657737627]], [[0.51237698929907993, 4.717434174807285], [1.9313112700676234, 1.8662890712095725]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2134.9807744781938, 184.93627592671746], [1.6401854594608278, 1379.0821736590835]], [[8.7119379052879715, 27.133774786175564], [2764.9287699927168, 2257.7823564725909]], [[304.6029624448355, 321.53858458602389], [84.615161991819605, 5.0246202893511773]], [[142.22925534533186, 1261.6484012332933], [1.8089179485122566, 4.0037827695694554]], [[18.715543857282189, 916.7883329765458], [1.288894484058754, 1092.8531848904001]], [[9.8956664557260776, 1.1314726821077579], [89.649025445839413, 42.29321493374669]]], [[[694.60811072453328, 51.973970846138094], [2.1134099045512822, 1.2829971752076548]], [[114.44872808250032, 13.788944667912622], [51.56181694887006, 257.93029613443326]], [[17.514509529512139, 27.255446261760774], [8.8126422232562938, 2513.7494735025366]], [[479.50433382807233, 147.48443144640677], [194.13184710371064, 4.8451819022372709]], [[999.51036756107624, 16.706995183396391], [7.9531147211583137, 16.320762786723037]], [[19.735389595774684, 287.92728497938378], [493.64498356028452, 1.0381610855952625]]]])+(1.-msk_ref)*numpy.array([[[[62.02387580336913, 449.16602084481752], [29.947912216671643, 18.927122387840011]], [[102.87376499491181, 21.736881196150996], [40.247260076854914, 2.5119497331671421]], [[284.70745271991876, 1.9892922652593226], [1.5317624618699697, 2.5833972828042961]], [[150.23582031191174, 36.623025882917865], [4.3302732500015901, 1.6117106777436643]], [[1.1963912075403624, 34.533194559811058], [1.3752980371892276, 22.904785703370067]], [[319.04398769512937, 16.656630499609889], [62.985850612679251, 17.365250919142259]]], [[[1526.2300289035238, 1528.1799474700997], [1186.8870360152225, 19.689242863929874]], [[1904.0369633800747, 21.955656679849284], [674.68069651884025, 82.868172433295385]], [[13.419176776626436, 299.33369442939761], [2327.7330708140817, 3.2902815008701896]], [[760.41934188633945, 21.007493548365403], [1.1722492971959564, 157.50558070159039]], [[1008.8590660759792, 1.3440928534518573], [3.96042166178011, 44.426969957989471]], [[2.2137504014496421, 1505.169884464713], [19.993981281569379, 18.075961392137181]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.483155877914438, 4.2651723284161527, 0.38176287064065206, 1.7326105038094273, 0.43101442603876733], [0.49722742079316606, 0.7012040610422936, 1.2522663872719766, 4.0512017322525882, 2.0525754172576356], [1.8164830052781162, 3.3599436646988554, 3.7487606685407813, 4.714907741265467, 0.13761673965348981], [0.42100836536229302, 1.7178511670781313, 4.3323592963736006, 4.0716327090671616, 3.3975527310711433]])+(1.-msk_arg0)*numpy.array([[0.34509894884834019, 0.60911189818543243, 0.9472742778570783, 4.3794618513354955, 4.2532171788676658], [4.1121954686000484, 3.9062494990749381, 2.2858864606998881, 1.5986503743480618, 3.2519673280300792], [4.3173541312667307, 1.3684297311185805, 4.4918584537059241, 2.91213008305489, 3.5364884175006011], [1.8541100250428806, 0.092116640660692639, 2.1125731976151982, 2.4830018305417005, 0.44151975272565125]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.81103909022431286, 2.5775929915304649, 0.3917879981321587, 1.2973725796606064, 4.3994403394589146], [0.46366290938499, 4.4981114927765518, 2.0422741341977715, 1.6259021795619317, 1.057122019963334], [1.1411179626698431, 3.5885007528003312, 3.1217011756494051, 0.3123467302945595, 0.90325053239978437], [3.0633033861438554, 3.8040519890208988, 1.1649207469097878, 0.30089021592516374, 3.4568422729221875]])+(1.-msk_arg1)*numpy.array([[3.9039921098077341, 0.83794246506453207, 2.855389347875009, 1.1706178573441788, 1.7270269517189261], [0.32422938305105742, 1.9641134586259625, 0.41265419095669897, 4.3541172927554683, 0.55393081517070053], [1.9156682134790344, 0.59608362106945834, 1.6858872706537293, 1.5085211128131113, 4.0719495003173591], [3.6115928046321852, 1.2991094075502709, 1.3313392897922305, 3.3903689258065208, 3.8822833812287163]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.0910464050213524, 42.045558072702278, 0.68572784670715536, 2.0402531550260963, 0.024658605232632219], [0.72327564507337783, 0.20257741541261787, 1.5831551996175091, 9.7246042856764188, 2.1386429281995323], [1.976121018526829, 77.400321203670259, 61.87368097486624, 1.6231443773749408, 0.16672639549196913], [0.070646151208769145, 7.8324420271121831, 5.5173881975051851, 1.5257149581407043, 68.573733343659853]])+(1.-msk_ref)*numpy.array([[0.015708532049479518, 0.66006779363119372, 0.85670064128928802, 5.6345414911243354, 12.184625065284763], [1.5816149198965435, 14.530606403577099, 1.406583142781155, 7.7119927107283575, 1.9217352104953278], [16.476550928909496, 1.2055910128415004, 12.586842378732234, 5.0150093661751107, 171.30043220964049], [9.2981436209168056, 0.04514003294871799, 2.7066659858178417, 21.83322457442884, 0.041840399305717833]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.3034366019523094, 3.4786914483996663], [2.562507064909147, 4.9421008520516105], [2.402805906842163, 1.1788565823058432], [4.0282435810973078, 2.4473016138769239], [2.3339205554459466, 1.9686111959100889]], [[1.5078280544908687, 1.2321401743560694], [3.836883382793852, 0.89810453117543454], [2.4416075851922683, 1.6657953697483068], [1.9559807689120241, 3.9825541765274775], [4.2885432914895159, 0.014640138377699938]], [[3.9451378690469605, 2.2701400195863104], [3.3558953511195577, 4.0798123628091973], [0.011965360505098291, 2.798495875440238], [1.4381490124461265, 0.39754338623479291], [4.3788686390661038, 0.58703738148474227]], [[2.7749886986164221, 3.6634826836197338], [3.0848718024324957, 3.9292479578427435], [1.8549583151585847, 1.8958270787569587], [0.81014172387446903, 3.8197141003578818], [1.6476519435083465, 0.77731838568451006]]])+(1.-msk_arg0)*numpy.array([[[4.0267893261206202, 2.8519219630701786], [0.74666315730590305, 2.7726374785187575], [1.1349608576391568, 4.7023901143508056], [3.7796893840517507, 0.44152187786724983], [3.8610157269518521, 1.6663974990358581]], [[3.9355877759807307, 4.325959954138324], [1.1928488875908387, 1.7926231797317531], [4.3584230746490116, 2.7870190138416273], [4.5065818254882943, 4.8811410208513903], [4.8509432842932636, 4.7453365632725397]], [[3.8186062674047361, 2.8633136054923698], [2.4478659785596668, 2.120651485500836], [0.92055812946396209, 3.8511273168152305], [2.6119520565726462, 3.0710662104302213], [1.9032184578729299, 4.4850462861321638]], [[4.9794171240614533, 1.0139542216071096], [4.1662447825490672, 3.3041251802897067], [1.2455139457376641, 0.039180025355142083], [4.2232984186218214, 2.0318681542280901], [0.39554409260537182, 4.2111348703489631]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[3.5143964662391869, 3.2747664398096616, 4.1569672618931186, 0.74572539875383914, 4.3562846815617915], [3.2127325496386394, 3.313132361199945, 1.3594723234821309, 0.34209923777157109, 0.75482535174824461], [0.53159552187124959, 3.8911227346364781, 3.4604034710556428, 4.2429828166708665, 0.82772415559239443], [3.1522913301775985, 1.5587306519500193, 2.4078249858260858, 2.6235513207107291, 3.5671646429244279]])+(1.-msk_arg1)*numpy.array([[2.7705782868446263, 3.6371561748741139, 4.2334813194161995, 3.5028005782889906, 0.84644219857170677], [2.3168561271916914, 3.9078126385718344, 4.1103806509169596, 0.80434432423018221, 4.4102267030481359], [3.4736152963879117, 0.63323324743579379, 2.3092354158425081, 2.3844014189844938, 3.4900511235148128], [4.6771543682792238, 1.3042469709649021, 1.4899775297372906, 0.32413214126429502, 4.2477971703012658]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[18.773007778752564, 79.937425924787988], [21.791275984754847, 187.23991354575571], [38.250325399339076, 1.9818050208109612], [2.8265076444467949, 1.9491898373150396], [40.131713553648687, 19.118089358070552]], [[3.7410774193937626, 1.9555400392235887], [86.059416366436622, 0.70043171224510969], [3.365383875663658, 2.0011908119939812], [1.2579854003746651, 1.6044081429536265], [3.0011207255809613, 0.041238892785510968]], [[2.0742638006647769, 1.5462367515085578], [111.16934741724722, 237.72558489234666], [2.2327892127343897e-07, 35.199758460779023], [4.6726062182327732, 0.01996160493232043], [3.3952421301044033, 0.64345630278110677]], [[24.962524084718861, 59.918180697489667], [5.7888046803204016, 8.4405031216082591], [4.4269200821403931, 4.6654195428459246], [0.5755791985760067, 33.650213640414393], [5.9373801280270095, 0.4071443450364477]]])+(1.-msk_ref)*numpy.array([[[47.433592363823571, 18.238724225946374], [0.34556901943646728, 40.819843468854735], [1.7090727312318659, 701.85673173250802], [105.36917969465225, 0.057060879272647531], [3.1376822965540736, 1.5407166296615522]], [[23.908462273330997, 29.765393307064972], [1.9919641170794007, 9.7856020077347079], [424.50922700603769, 67.560765515609631], [3.3567448650056342, 3.5793836008899467], [1058.3979932623101, 960.48562837404972]], [[105.03007987327508, 38.635528568736568], [1.7627602860008322, 1.6096491170663383], [0.82601093857255037, 22.504028203140912], [9.8676200101395271, 14.517539337536631], [9.4499529305977461, 188.23479444673688]], [[1823.1042626157996, 1.0669614821406761], [6.4313220234066559, 4.7531099249744342], [1.386970625966506, 0.0080112085239114929], [1.5951217904364103, 1.2583424141984365], [0.019452062119782763, 449.07713782063649]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[2.5583509132111142, 2.6608676868071313, 3.6917351518291435, 2.4818296803642514, 4.9807498051319223], [3.974684410902698, 0.40868798105861159, 4.9438761225263281, 4.5173901509259151, 3.9527467339924143], [1.6420036694042213, 1.1278894011861704, 4.4505288107346592, 3.1561004420831038, 1.4716007764567995], [2.7122546532302629, 2.0920521244292738, 2.3961593708023714, 4.9817943884075611, 1.205693724081937]])+(1.-msk_arg0)*numpy.array([[1.1176880458591691, 3.1030756043318628, 1.3059189422745454, 1.7523372917993592, 0.74568286320958466], [2.5868653845777123, 0.63672290597719328, 1.7911904910791274, 0.10542450194866845, 2.509426752458531], [4.4458238341726242, 0.3311331790465214, 1.5871090225474569, 1.4171946079246898, 0.6599380324464782], [2.3955788795136237, 0.65938715424187355, 4.2435411711859157, 3.5031862415442232, 1.3649313180814147]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[2.4148095486427827, 3.8554954703997395], [0.31352247598856964, 4.197891605818203], [2.7206820405538501, 4.3725107607451523], [2.22261121052945, 4.8006623958951407], [1.3180799007778468, 3.5891936428646178]], [[1.4970605488626623, 2.4083180018306649], [0.42147185110461449, 4.4570275741450818], [1.9302134548437195, 1.7125256898674663], [0.55607333276986393, 3.4557118456106681], [4.0900658442539077, 3.8239811489829671]], [[2.6427143716849506, 4.3759653976377253], [0.99303505548581816, 3.1248807448630505], [2.955305310089924, 1.7061309877508424], [3.2778870307548327, 0.93684509152784112], [2.3298276078925011, 2.1400861662367805]], [[0.6704125410923214, 3.3613616298875963], [1.3375817795748111, 2.6575868641309746], [1.124116670034337, 4.1555487219920906], [3.7923313060101536, 3.019443405242471], [0.79953258905951152, 3.2518546106292407]]])+(1.-msk_arg1)*numpy.array([[[4.0662437097256081, 2.530504057439698], [0.52707996604009089, 3.3866468183225207], [3.9980489828487098, 2.3856277107331505], [4.466687753541704, 4.414626179107616], [1.6399585025866328, 2.6017877385278489]], [[4.4210792556790359, 4.7188734284225919], [0.61336083530550434, 4.1033457730258931], [1.5698674662022116, 4.2053142006223956], [2.418847811776188, 1.8888479571570425], [1.6194669654410987, 3.5991577233476266]], [[0.14166227666430919, 1.3201599612686588], [1.1667173551218055, 4.8741469138791187], [1.8325018576747514, 0.48000815435494565], [0.99664356434472989, 1.260579617946459], [4.9656436577891299, 1.4079489160092471]], [[2.3114636065175946, 2.1758082506532173], [0.21214872437928778, 1.2263950176654819], [0.43736920561720394, 4.1229462241415717], [2.1177334006180639, 1.0658774484463085], [4.2484954805353627, 4.8913059211674161]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[9.6637564931402107, 37.40144406364962], [1.3591091958463353, 60.841744429087001], [34.93445194449184, 302.14932340121186], [7.5409248890682887, 78.553758749672696], [8.3002307114567735, 318.21877535846829]], [[7.8920959445196921, 27.753091280670297], [0.6858238917469478, 0.018533729952814451], [21.862437548066229, 15.438622979755737], [2.3129461981312254, 183.2748358129752], [276.28427374619173, 191.65955694944728]], [[3.7082780540285971, 8.7593062575431428], [1.126944380294373, 1.4565541273681639], [82.462081739061091, 12.772477675149833], [43.267319080327695, 2.9351276385866072], [2.4599245891250492, 2.2860465819171711]], [[1.9521364778479169, 28.614119810313174], [2.6840582474420938, 7.1113043760956991], [2.6706681365026745, 37.765564627347764], [441.28381422897434, 127.5607171019136], [1.1613192613235019, 1.8372612443769738]]])+(1.-msk_ref)*numpy.array([[[1.5721115564721415, 1.3251818819670218], [1.8164100612435741, 46.294246643538557], [2.9069578241360432, 1.8903096427746], [12.250793293705707, 11.898195235859829], [0.61800802596353399, 0.46602904292592517]], [[66.819898983460632, 88.680361704966614], [0.75814259744378365, 0.15687040051237267], [2.4968878551879934, 11.602209449047058], [0.004331559910134823, 0.014272010540019607], [4.4370849259285476, 27.424058879392561]], [[1.2353510561829621, 7.1680432483063203], [0.27540943738164425, 0.0045753322251340309], [2.3313756780593087, 1.2482250124139325], [1.4155370074501645, 1.5519900416000594], [0.12697464268608347, 0.55701856026581109]], [[7.5334429897262716, 6.6915148018020414], [0.91544210595053255, 0.60006015318032424], [1.8816943480356041, 387.33836892214958], [14.224203607791559, 3.8047933569171852], [3.7498821369243864, 4.5800292289624371]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.583212066254813, 4.1361615822502138, 3.1132666301649823, 0.34838428580793063, 3.0520407806110712], [4.947685122964419, 4.6464829746133836, 3.5792445539994269, 0.052020576589853811, 4.1605426684832354], [4.0098759949594065, 3.8743996214562979, 3.7835659269987216, 0.89613517409697241, 2.8684472482714947], [4.7945862318993768, 1.9316404642105673, 4.7816644170670637, 2.3044194580587027, 3.7283757581018868]], [[0.62409122392406313, 0.62726688534823816, 2.3007857898218318, 1.5473087130304102, 1.9657117085418114], [1.2858579095467784, 4.3261687171115089, 1.9820987855977596, 2.4170177846750041, 4.4227097068453123], [1.589667741453062, 4.959599750231094, 0.60527870749037482, 1.4827824599009733, 3.7444675341425886], [4.114034148813845, 1.7121434774135555, 2.1141963100189067, 3.4762963777125115, 0.40312478032614391]], [[0.60671395743002121, 2.3500728898290295, 2.9707496565872629, 3.5490379394459626, 2.9570859217048908], [2.9896330615758604, 4.9590429084958938, 0.84075597261396395, 0.41215594864311061, 4.9690004976833473], [3.611948400307162, 1.4913830658659155, 0.97348662402871267, 4.4398779274978697, 3.7206157715833781], [1.8056626445969652, 2.1561096097661885, 2.310883022829608, 4.6350079795900854, 4.9478197678413975]], [[0.80984124709403882, 2.6304824313207291, 0.063539952605008235, 3.0597029396792914, 4.4313625662079099], [4.9036482609302068, 2.6575283219278383, 4.3995837447190249, 2.4290648927586678, 4.0035466564003785], [3.7302058283176596, 2.4217951161696876, 2.3598266853794407, 1.199131367479185, 1.3550742737352826], [3.8406262330543859, 2.3602751893373366, 2.9300963169403498, 1.7914360884368572, 4.9206589741976812]], [[3.7401706365092604, 0.21791772035560353, 3.1149100736906852, 4.8437702288036197, 2.5033099008448847], [4.2510870799636304, 1.2535840068357655, 0.94294756455034756, 3.3307017901404343, 2.3293177376362615], [4.0779889432052725, 3.2071885456796987, 1.013791367316403, 0.30741049718242114, 2.376180223099531], [0.26212792475097707, 4.7686006065162925, 3.7149330846495272, 3.9963208536543191, 0.86715039721234444]]], [[[0.30337020221849886, 1.3027200953054849, 0.091850634016613933, 0.65691416173980832, 1.4179936431976743], [0.32283730695828677, 3.9107085891324114, 4.2059343468383883, 4.6371663090721009, 2.6141951321270347], [1.8611675933126595, 0.95643551946417837, 0.57570668864710528, 4.235168800684936, 1.2965245820881799], [3.9199229995406064, 4.0131974015435254, 0.76970897313164532, 2.2114968726771083, 3.1101947956236451]], [[1.1386200816617629, 2.3253358216263691, 3.8264232943429377, 2.9146523045781652, 4.8147438343578584], [3.2284536900998804, 4.8079967877716756, 0.35048847997721477, 2.7274270010215815, 0.69933252139649138], [3.8563962486788599, 4.6464735887098083, 1.0042145321436651, 0.049331072609358317, 0.075592715174484154], [4.0632938673025301, 4.9829810563769179, 2.1311864203280986, 2.7228068188487078, 4.1863376293184418]], [[3.2181379662903407, 2.4104535973682792, 1.4539853210083564, 0.51307923742968331, 2.9875272657454768], [4.0949134300035057, 0.51651699202368784, 0.088396712056840188, 4.1138832411303135, 0.57706752124901717], [0.61960303374089598, 3.7284488933724504, 3.7654921805567496, 0.81634827298575041, 1.7715297816664082], [3.714582075236244, 2.5596827824253516, 2.8435572803322957, 1.9548933317411137, 2.7407424470828481]], [[2.0345260172515434, 3.9583937303928933, 1.4696813435310994, 2.4415714644152788, 3.111022188144803], [3.6537712617659817, 1.6922475489980835, 1.1329816657680525, 3.2661229220548806, 3.2668084963145674], [4.2222977410778224, 2.8196649879520446, 0.48421163265755246, 4.3267926445358844, 1.8112962117155522], [4.9986640847849522, 4.2049948185548534, 1.9545229949157654, 4.1539348965708429, 0.93411594552190513]], [[4.6631582766206678, 4.1319729633110338, 2.3955004822664954, 1.594749895134923, 0.75405543412756293], [2.8880782417168138, 3.7638460349023952, 1.4558347625819279, 0.75845283461027346, 3.1766864919247424], [4.609996738092569, 1.8922046476636005, 3.2978874591744671, 0.55094857709851341, 3.2531098863473229], [2.1991026760956887, 2.8039375852028328, 3.4277524466130509, 2.2948359532568738, 0.32925237153829234]]], [[[1.7568821910369057, 1.9280541708385013, 3.1461844579628866, 0.70634156978140894, 1.535741536974045], [4.1993275862252695, 2.8591535820896916, 4.2863562119741312, 4.0219856323690113, 1.8201884575691349], [4.4350300020735993, 1.9231990619979673, 1.7814716121284226, 0.27757623744187215, 1.6463565403969882], [4.7478838323465116, 2.467491249920482, 1.2461250299580844, 4.5096521993184027, 0.69546084849396339]], [[0.17762503059989715, 2.0507244947217691, 4.8909938161395843, 2.2094636456095289, 3.1411010176919074], [3.2506856765912566, 3.7006718462327379, 2.5720044980147718, 3.340045117984054, 1.9813784739677627], [4.0639837079800589, 2.2211251365431366, 1.0246234235886422, 4.2265597352353055, 1.1947662816622324], [3.5139773181237168, 0.3390621073967619, 0.012961552060619145, 0.29540974764841371, 1.4503961985662779]], [[1.3519695765837474, 1.9133028847202669, 2.9341532466878024, 1.2020743225200086, 0.46443991938146167], [4.2010906730285624, 2.121815916513158, 3.8094479623711992, 2.7201020082184373, 1.1179950445731992], [2.4146831057510956, 1.4513445812508374, 3.4627283875530037, 1.641866281048695, 0.15710931183378227], [1.2765282676685883, 4.2350173232559989, 2.6060295162247598, 3.4440014866719388, 1.9277787505378832]], [[0.36334170117252962, 4.4986191413426093, 1.3465591034399369, 1.4752659932648009, 2.100399109607618], [3.1307516484936118, 2.0343714359654608, 0.027779394721010535, 3.8173642397952339, 2.4081431422440467], [1.3527771174756607, 1.1999059738012314, 2.9220283565982061, 4.2505520115961577, 2.5867692292102893], [3.5324961666305095, 3.5560510081528607, 3.4896595960664527, 3.5542976899774801, 1.523541404101183]], [[0.035965824643930042, 4.0956227690343709, 0.31280252764192523, 2.5578146830603945, 4.9660353801157244], [1.0087871217169961, 3.3136938942933476, 3.4864986614105367, 1.6581028628283474, 3.3844242392026489], [0.38932946693161963, 0.068265884316856679, 2.0759544698794241, 0.34807611211804068, 2.8040798946213767], [3.0509732825683344, 3.9968283563657754, 4.7927945242313328, 2.3431380415776482, 4.7996026609588931]]], [[[1.3436165248425269, 2.6239584588851206, 4.6806910924429506, 3.5998854883505742, 2.6317860963467954], [2.6999929385452535, 3.2576589492725305, 1.6286626082733808, 3.6069487900400512, 3.8168401348328667], [1.547335941627038, 1.9922021877531677, 1.4991059921708674, 1.2861910761375792, 4.0575733450293736], [1.0955342320538584, 2.2177434060918868, 2.0867885991096835, 4.7672421525184756, 2.0890101185064727]], [[3.3581537530062544, 1.2743081635663274, 3.2962580705260565, 4.4768060067360835, 2.4927768152577716], [2.6369950096228609, 0.55129230313659616, 2.6164417243774447, 4.7970846095607831, 4.8434770505887457], [4.7467388607900167, 2.6172314060865927, 1.1649274885782686, 3.3471295918306496, 2.8347391745965691], [0.95110988770101434, 4.7000249713897304, 2.3384334898760137, 3.9789012406840505, 2.6735982241518137]], [[3.845894824825864, 3.646273466386726, 2.1195641582290765, 0.5754000912936329, 3.8466261908484447], [4.3724398151284865, 4.8356402607827675, 4.084511063012271, 3.5236256506009695, 3.6260188845796848], [4.5425125827607689, 3.4491840752705518, 3.5880145446044134, 0.045694040807440099, 0.16197827838037057], [3.3824068040186579, 4.2555350802274052, 1.2107411831481973, 4.6434854087240298, 1.2498452100851996]], [[3.1815127276023021, 0.072377423371070448, 3.7122134655422232, 0.062913217031057317, 0.10199544501822026], [4.930175077675413, 3.5029197418193641, 3.1619878618551294, 2.926311082297961, 1.1821459991715726], [3.4664593633021186, 1.5168720733614172, 2.3939926180857727, 2.488846656327544, 0.46346121664765189], [0.8404019975518251, 1.756547885687225, 0.91933226921271638, 1.0942624114721353, 4.4741633003294812]], [[4.31185060353432, 1.4755119525613749, 3.1693667462983641, 0.34794743040850873, 3.3308137069829296], [0.4006027599271203, 2.5478457951930911, 3.6008899468304785, 3.4190801107612239, 1.8904415258230589], [2.8818823414175405, 3.0025743779339891, 1.5686042514029361, 1.0870624194148444, 0.17818168749255373], [0.51587133711747635, 1.7488906663538204, 0.048051106215481983, 3.8651857755389836, 2.5536320517607134]]]])+(1.-msk_arg0)*numpy.array([[[[1.6963870279590449, 4.9345721588831379, 1.1411725344872305, 2.0937980596615624, 0.13171265447332614], [0.84536654761656793, 0.55652159335989027, 1.3678295721349181, 2.5750044662440792, 1.5831210280610786], [4.2139749551089363, 3.7735074272472664, 3.6198692194394728, 2.1382082863435703, 3.8137049339852207], [3.5726563744446431, 3.9405781995301008, 4.5582913644894454, 3.6263640732191891, 3.8403007489539918]], [[4.6156663681451642, 1.7369170096972804, 4.2098292478932082, 1.570152503809318, 1.5695280453939953], [3.3078612747422471, 1.662817565895703, 3.0565388724912017, 0.94419492891996171, 3.5070880363316319], [0.1792359966989244, 2.3547399953437451, 2.2161667098123665, 0.62391083812943871, 2.5145819670949483], [1.7224460613826367, 2.1822613552232846, 3.3692236909110505, 2.1442852794466667, 2.092991299429626]], [[0.71554527548033586, 1.9449053997176946, 1.4928886758248827, 2.7899674348790433, 3.405073310401884], [3.2834667044408374, 1.8287845388347561, 0.3098457837142588, 4.6016965863526087, 4.2426546443498188], [1.0886440757776379, 2.2966888916708976, 3.6091296950085443, 3.0827415146622732, 0.26031190671872456], [0.27794764116216497, 1.1266031224063739, 0.26686817125247381, 0.54100550940745074, 1.4425869963648341]], [[4.9619536289428456, 3.1049993276867442, 3.8680342010177657, 2.5797498087711359, 2.0569689741243296], [2.5309103306164391, 3.3692409467545339, 4.7822850931719278, 4.7852167596062118, 0.058145019223503285], [4.9109307804820839, 2.8866280482661226, 1.3815126568930194, 2.0284221005960723, 1.0320741762532539], [1.9945306074332754, 1.5800527169113261, 1.933994545220882, 1.158537178855223, 1.6966235863504751]], [[1.9944095599278924, 4.447941405204042, 3.0481107900764619, 0.7165279634354691, 4.6397393160384217], [2.6153059622442427, 2.3599043881422133, 1.3520150956203427, 4.2653690364695604, 0.81219199186928293], [4.1759514944073066, 4.7680760852843713, 3.1664232660827185, 3.325193923307141, 3.2960242716424499], [2.6974309820753244, 2.7072175177013023, 1.8140780859047576, 4.6601310575077912, 2.5229862163941714]]], [[[4.4829762293829054, 0.10962853832287738, 3.7732646300940997, 2.6650356960799937, 3.7894831851173145], [1.726904933796181, 0.6561870647264425, 0.024698876629848085, 2.8948040164617037, 0.80773312955078513], [1.1618839561934233, 4.367899776226249, 0.12628310820153715, 3.4553649311969878, 0.46868617148469244], [3.4173125280568519, 3.0958117184141987, 1.6055636232846602, 0.73323794583781321, 0.33601915374707991]], [[2.9965158664541285, 3.800548031712947, 2.1002184188222124, 3.1140579915000961, 3.4494037992109692], [2.4347666622965001, 1.9440348438432595, 3.1167134384746968, 0.533882613900175, 1.0909561734890485], [1.3005991673707673, 3.7279925602172366, 2.3867577629542742, 0.70504066114599395, 2.2932373285918795], [2.9402288426263232, 0.37333927294957886, 1.5813234564166023, 0.19735441295519984, 3.1238029310803856]], [[0.68945410747719216, 2.68500542668572, 0.39654883425381932, 4.3621961304670869, 4.8258602364039636], [4.6366035408625663, 3.0003041852561707, 2.3550419699507334, 2.9957455542888058, 3.881385683564071], [0.1806956170217052, 3.5096813782137741, 0.99627194423908505, 0.16040246885428197, 3.8316522925980987], [2.0554365962818735, 4.1140696452633874, 1.8305941520594522, 0.86193292329233484, 3.6268585189313223]], [[4.1275773906334496, 4.9407334301282315, 0.28720835845415865, 2.3333615111282011, 1.6174142013478527], [3.7741863637106201, 3.8834287645217862, 4.8707736146926317, 0.12377528244567282, 4.2790928027096342], [4.575160432515422, 3.4113224090338687, 2.2886333214090664, 1.4661975409489552, 3.4393519415464784], [4.8795361797371397, 3.8026142747125689, 2.8367947838946135, 0.91416404445061294, 0.16992295406390212]], [[2.7124958572313527, 4.7067391643389831, 2.4580707189763107, 1.1004649524050356, 0.80067786060688928], [2.5808735506152636, 3.8050845718505522, 0.34597207418465176, 3.1538085251029044, 4.2341022728098432], [2.1848984684267578, 4.1997784325865597, 2.3888146043759262, 2.7359051016104314, 4.7738716951938178], [4.5834005831893432, 0.78667595412508473, 3.6333627271346027, 1.6448228195120103, 4.734859950651316]]], [[[0.50845716930364293, 2.8398772047192091, 1.9298303267131516, 2.9202445446609571, 0.93700205841894724], [0.44523426184353193, 4.9472656128531005, 2.0463568045453204, 0.094376781729368947, 3.7053246057296447], [2.4155066556511882, 3.8586027675723447, 4.8944042750845869, 0.010199506343444091, 3.5891673513326068], [0.77261779941411124, 2.001263359736781, 4.5316707480313889, 1.2939372180487949, 3.1868646901951081]], [[0.35768813554039242, 4.0070762599048662, 0.13417397884651158, 3.8215591633974393, 0.83118632290912464], [4.5369746716907473, 3.1406228377696737, 0.093482751600530586, 1.8440929077828006, 1.3946858816782091], [1.7110020430818724, 3.2326465959804129, 4.7205959175542143, 4.0061312822187265, 0.15158174404612401], [4.0059656033670148, 0.89353161068499198, 0.56260689177173429, 2.4358158015701092, 3.225384301858031]], [[3.3861064921574715, 2.5263325147576969, 1.9319213021343513, 1.2538028844521463, 2.4068632466245443], [1.7015517253603272, 3.7465526195636372, 4.5919239594287955, 1.2974527497631394, 0.16275372406045435], [1.5481122701350338, 1.1120585864179851, 3.762065245587396, 4.8109006738894866, 4.017322377637508], [3.3123509705235237, 0.76290329086719288, 1.1761304386721225, 1.5828600691170116, 2.203769825084319]], [[1.8724037386949477, 0.76409596544222291, 2.7563416068544009, 4.5338832017629942, 1.3003641517394391], [2.0227158677538326, 3.9085319239904295, 2.8821567608455867, 0.39841227554996977, 4.1801673017200907], [1.81417302554718, 0.76168655714640199, 1.7313993276358086, 4.1155624086222451, 1.2258932197091679], [2.7829750443809163, 2.1295394459981094, 0.36605711992770434, 3.5971526698756087, 2.3224772989350964]], [[2.7556078069310743, 0.25728303911196398, 0.88973619011897309, 1.7275581971200982, 4.5598218117936229], [0.64185609974674862, 0.86440167659887235, 3.185354116289532, 0.37860506133313943, 0.60395426158406718], [1.8967853739123783, 3.1109370465442727, 1.4412782564400803, 1.5377608175260047, 4.9944782271909292], [0.89398633361652491, 2.1041992840864183, 0.22692990305482222, 1.6438409393791107, 2.9769129632934694]]], [[[0.91333094600097786, 4.0536019725945271, 4.9127098263520157, 0.53891288380623259, 4.1322738166636732], [2.2042634040950873, 0.48875645525422484, 0.36700903242333083, 3.5043186109108952, 0.77157844757504079], [3.1978442066884019, 1.129614725465427, 4.9594043412811635, 4.5906938457749327, 1.372632460945107], [1.4048540358928367, 1.736248019606355, 4.9608402943586976, 3.0419581266850777, 4.5151286903061933]], [[1.4507195765247307, 4.1379643377470279, 1.8971438690526692, 4.9384931289069502, 3.7963431013853381], [2.7922047632905511, 3.5786865409890845, 1.6101599673725999, 3.7760416716843186, 3.1691813265301367], [2.1287421837409988, 1.0235321868343321, 4.4720372452400561, 1.49558591140317, 3.1571706973319968], [4.9322406413251674, 0.24703253447389559, 1.7411524008688866, 0.31657235147671609, 1.7302448059585227]], [[1.9318355676222365, 4.9638791271182736, 4.040514035276467, 4.9971277499187607, 0.64439393129856926], [0.084399782084029179, 2.3737136005936756, 0.036674826722701004, 4.774436421972708, 2.584205435629638], [2.7383472913215083, 0.61956050962340559, 0.68337971760047744, 0.93608717798877772, 1.6916724161200503], [4.4230409139907314, 4.6328400263239082, 4.2237152061434093, 4.0421680929390629, 1.0486717361443065]], [[0.15284003846183067, 3.0331936503351047, 4.6786792918307505, 0.78609558289576686, 2.3472113181577772], [0.98753939525099854, 4.7624280341029932, 1.1652189244553048, 1.76861751580854, 1.4034228287289128], [0.44999949022368452, 4.0407891008521775, 4.8426508503992141, 4.5371421428797296, 4.1896815059356234], [1.0402607154011296, 0.63272469580194857, 2.0496870326303309, 2.3548523263596457, 0.9494119786209736]], [[4.1055029998640382, 3.1889654123994227, 3.147086743519294, 2.2122267489369447, 3.0972357830172741], [2.6867719033360959, 4.2486456462993942, 0.98436105358348414, 1.4967535032464963, 0.59703111204535053], [1.2422086032512758, 2.0969315051663657, 4.7988190126279795, 1.3569724633733684, 2.2618689754006338], [1.3548119586621661, 1.0771281637472137, 4.8502256630192528, 2.2853326881045009, 4.3845732380169844]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.320501848820677, 3.4959677080920275, 2.5851775708692966, 2.8399186797631981, 0.86474766726181929], [4.4133761264222633, 2.5564847496443619, 2.175079758191536, 1.9465236997784909, 0.34941121642579975], [2.703014208401628, 3.0853355785668342, 2.8181372590190299, 3.80731477004622, 0.14494177901461133], [3.665527784049277, 2.0131496375365643, 1.2885044268563552, 0.83275070311977117, 1.6032790127470113]])+(1.-msk_arg1)*numpy.array([[2.6573762985212377, 3.8376307180845153, 0.29315842670408548, 1.7921499358495072, 3.7396843732877163], [4.6608084142145261, 3.334508524504749, 0.89380585239972998, 2.0777634035749957, 2.1276651737968022], [2.9747925573320888, 4.6037960090713588, 3.7927654781070155, 1.9355540363322297, 3.3285986748498386], [3.9206557824946318, 1.4116721939167565, 1.4895583515310997, 0.9060654155946879, 2.22433121714166]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[7.2796250729449792, 461.32885913762578, 135.18920308267164, 0.010506623048834728, 124.07189814324123], [1000.3808115023293, 762.62666874897172, 246.9770791480012, 2.8394381423024979e-06, 473.19335199872768], [403.4858894410682, 347.80695039465894, 313.92337871971108, 0.62262954192802711, 94.899546488441629], [873.34639670223089, 17.192739538763501, 863.22245688715248, 36.850872637589944, 294.61293778718959]], [[0.19239504539395597, 0.19583936590603565, 18.4122753924485, 4.5999696804069528, 10.620276606021207], [2.4084361334488791, 167.41606471942234, 10.93302744034934, 21.874289964087048, 180.84475463051342], [5.0554583725846651, 269.93485088425308, 0.17287138340121744, 3.963521386950084, 101.05392045961231], [140.43026502770238, 6.5531308951927132, 13.699272544049913, 77.933978026279988, 0.041747379203691509]], [[0.27477364067064053, 9.105653875858426, 16.689476753479656, 26.432223216747172, 16.491756205427897], [16.96511060408567, 62.766363294645217, 0.63864346592796783, 0.10112669426137978, 63.092698990420764], [27.660556468794603, 2.8103403948222416, 0.93289109652406665, 47.159486283382414, 29.863495570745595], [4.6073501285460967, 7.287832111005919, 8.7182762910337672, 52.70584468294377, 62.399795328439041]], [[0.54936772838326842, 15.590727984509545, 0.00039879572516439269, 23.949122947531468, 68.566710079620393], [91.415257947758107, 16.050284019282184, 67.179472553707271, 12.43413988247298, 51.391880346017651], [42.041155888883168, 12.32874798555401, 11.453789142853713, 1.6748472141999751, 2.37008801060963], [45.672447078864202, 11.459972398346707, 21.179057002023843, 5.2368623060506154, 92.318726247814141]], [[3.1290068640827351, 0.26778732912801628, 2.6711966199712789, 3.9130091398284539, 2.211131570426959], [3.4953760802926297, 1.2158442932073639, 0.95046945027671659, 2.8304896506411978, 2.0775916488903574], [3.371955173179995, 2.739491605346775, 1.0119149874207201, 0.36058357072281805, 2.1136876718616242], [0.31416711456952662, 3.8604416383201063, 3.1107405734563152, 3.3134802285688605, 0.88403060498966224]]], [[[0.005173109329091043, 3.2127812596947631, 2.6527208632079378e-05, 0.15652976022980555, 4.6708472373991565], [0.0068070737181416428, 411.00403616927565, 566.68251944134181, 871.81359324858227, 69.481561975908349], [15.511893181865625, 0.82153545645451242, 0.087433534027401325, 584.27363636600262, 3.1458926113893622], [415.29520251282088, 460.71320900085857, 0.31500417322736507, 33.207059321097482, 149.57430257030558]], [[1.3935791577080414, 8.6479875721257766, 30.895942398040752, 15.406676089862524, 55.588882013986989], [20.009527489121464, 55.38995310376049, 0.068543358537436913, 13.001583390119439, 0.40080793544197263], [31.518421068992545, 50.756420251558779, 1.0108097539753178, 0.00045601868520077474, 0.001357847767237783], [36.023666742006938, 60.690483197528387, 6.9201530607952382, 12.945352794996353, 38.878525042485393]], [[12.708099863365439, 6.7779024211032652, 2.2572570256760582, 0.23422219311018772, 10.810387055803199], [21.462539529357169, 0.237649086850486, 0.0051099248407665944, 21.679387494227619, 0.30244672080807888], [0.35304523892809919, 17.503278663738538, 17.883735289413654, 0.64316460594744429, 3.4687875635970675], [17.361994534777214, 7.7239139421682781, 9.7092572500265089, 4.2975000091244366, 8.9618632985106874]], [[3.9850247459817489, 14.557435523970272, 2.115942614949903, 5.6833938590831217, 9.1085150971344788], [12.456309355336222, 2.7842638182174499, 1.2751055174540398, 10.013283067682025, 10.017374744349191], [16.506145591832574, 7.5217713265308177, 0.2437325816214366, 17.310609403050233, 3.1782103292245716], [22.926395581601497, 16.374734260214392, 3.6856813424718364, 15.989925324740518, 0.8757586341139536]], [[1.7125542854661244, 1.6416944557158362, 1.3569545751227821, 1.1771262964047147, 0.90607329159881256], [1.4485781667376472, 1.5890304694514996, 1.1402319135088859, 0.90791606065975072, 1.4975987553345875], [1.7057070336300559, 1.249613655502817, 1.5173208029462522, 0.81197246603007089, 1.5100903199748819], [1.3169958878392303, 1.4336900827296777, 1.5379361399863141, 1.3367514727004799, 0.67829589722128347]]], [[[4.5871488698407932, 5.8976817628592144, 22.15734474156303, 0.39073720696760866, 3.1887506095705631], [48.357487915800291, 17.10867604861479, 51.11443290957655, 43.033958801873766, 5.0477542857416866], [56.049352351241858, 5.8576248263131134, 4.7627627324053057, 0.031293519264864934, 3.8482843984075692], [67.389050949250034, 11.488743082753926, 1.8126101816319098, 58.635138491468645, 0.37468027421394179]], [[0.0048358018941410483, 9.1693541742894364, 133.97428339123064, 11.540921810485615, 34.171534563544704], [37.985217182675534, 56.667790562658801, 18.442761337260169, 41.300157565875644, 8.2460166930911143], [75.652126905845549, 11.729894093284294, 1.077939406193215, 85.384688407408362, 1.7315851737585366], [48.302869480601984, 0.035542983471976188, 1.5028426773092709e-06, 0.023231800024449571, 3.1494920425572457]], [[2.339282939437799, 6.2245107034236735, 20.769721637753481, 1.6797994390390616, 0.11517593051032521], [57.111009039062317, 8.3311921345460735, 43.345994600296443, 16.777247803993948, 1.3693366327603007], [11.993679400487704, 2.85687720803279, 33.124945182397049, 4.0443751379408406, 0.0054298358413709625], [1.9897949450925918, 58.420325200301612, 14.869174205991103, 32.62257073236664, 6.358142904317992]], [[0.021182720653821218, 306.53367098502162, 3.1045728228778278, 4.3948343198935378, 16.869588954868725], [77.106378137551346, 14.937951466732548, 1.1878531379168041e-06, 164.04279113529586, 28.391258992317777], [3.1595092705450813, 2.0014205863813821, 59.293514639864497, 246.99559342162598, 37.281934149485743], [122.101092185168, 125.23004459683014, 116.5590450919811, 124.99512486723866, 4.9680245060627746]], [[0.61757275850502336, 1.2267349615288738, 0.84497444131382238, 1.1458222980382455, 1.2614815410493747], [1.0012688622135371, 1.1896384791309107, 1.1984361464351814, 1.0760460981241535, 1.1932857885212953], [0.87220758079582572, 0.67768428263841385, 1.1116757082353361, 0.85816232846123064, 1.1611906623305261], [1.1754803244730567, 1.2224010512690111, 1.2550058579296581, 1.1313556807726082, 1.2552640929880725]]], [[[2.9525469339750776, 34.331941176841191, 286.44408378477397, 109.41872127842434, 34.708848631563669], [38.121650825223881, 75.869947429549796, 5.9768571604377305, 110.20773154555498, 135.59761944319354], [4.9536768030274141, 12.508779611989416, 4.4108097714419632, 2.5157200901671359, 169.67478673802739], [1.3971725023685213, 18.53307437882405, 14.827132018905637, 306.34254250883578, 14.885072411513075]], [[11.458273511229516, 1.6290456394829016, 11.037080327605988, 20.440736881971841, 6.2890208762498121], [7.0429734396497494, 0.30155263265031479, 6.9328991305940324, 23.49142503312336, 23.951022682310523], [22.997733306264859, 6.9371121974080472, 1.3597829555339389, 11.382674275811414, 8.1466037776714515], [0.90401395533137874, 22.54437523200107, 5.5296962339261944, 16.121779898772619, 7.2411650676849399]], [[5.6724639404822401, 5.2959664999289879, 2.6325107045132743, 0.49059169014589854, 5.6738539153527965], [6.692301594906195, 7.6194213305383727, 6.1299466423528672, 5.067558032528984, 5.2580911558209911], [7.0295736396847888, 4.9300341330394541, 5.1871891688346556, 0.018759580269435035, 0.09580354063203228], [4.8073953237332674, 6.4626440831706757, 1.2794166174906447, 7.2315527054200519, 1.3329064861639071]], [[2.621612734162305, 0.11228809005825162, 2.9810017621948766, 0.099919779480178411, 0.14941525663871918], [3.7755619657783521, 2.8403680641965376, 2.6082079046492477, 2.4452806861025045, 1.1495210370312723], [2.815726969121191, 1.4147684625489507, 2.0687822628265682, 2.1368189629688303, 0.52707500534556662], [0.86520009699114775, 1.598603582062694, 0.93235585485519012, 1.0778999232282065, 3.4824166847662896]], [[10.412211913729211, 1.8657881756179495, 6.3562166300730034, 0.18404263013042532, 6.8832572101436362], [0.23069574966209563, 4.4792867955126825, 7.7997419099983221, 7.1780344438779409, 2.7759270703150092], [5.4574438705629857, 5.8284858576668963, 2.0580812273467903, 1.143210093652677, 0.062939672489726331], [0.3460378912471771, 2.4502831743571729, 0.0076984397120640757, 8.7377162106728612, 4.4956075353052203]]]])+(1.-msk_ref)*numpy.array([[[[4.0731850673898382, 69.538417183732605, 1.4203782502153859, 7.1259785756011871, 0.0045763474609585627], [0.63992822634542457, 0.21069292484607582, 2.2987311606226672, 12.347859683129712, 3.3898781457898117], [45.713195630719134, 34.090160743258267, 30.52510865568286, 7.5347191548551224, 35.063718794223696], [29.478530668327895, 38.249599878624309, 56.322903797698075, 30.670866614801028, 35.717276060136818]], [[354.06775190751307, 8.3211532006042734, 248.71189053927253, 5.648750711704972, 5.6401341855974536], [98.589492524624958, 7.0391470062430113, 72.800315175730304, 0.80222441302121983, 123.39684966958237], [0.0013643372898136834, 26.75344602947613, 21.197980554931316, 0.16358968926699569, 34.422296663204712], [8.0582312571084902, 19.980161778593644, 105.79490208336905, 18.678420340957818, 17.021072273576319]], [[0.90653723378540385, 1.2153266588186997, 1.1246505649551966, 1.3509246962765125, 1.4321791858851334], [1.4169915407642306, 1.193589963140725, 0.70929196805316508, 1.5643718485100531, 1.5275564741247718], [1.0252113755449124, 1.2760278229572806, 1.4568245003773976, 1.3910285196181256, 0.67398021901876459], [0.68705746988450556, 1.0355643482697472, 0.67891290897313161, 0.83519104004128297, 1.1134066789931476]], [[17.648749708734702, 7.618123422054536, 11.294596314603313, 5.4652338505432398, 3.6420933773877988], [5.2811975956631159, 8.8189436080078902, 16.519947050757121, 16.538100849449183, 0.006106905352716495], [17.324838017513155, 6.6848149782493964, 1.7845838888795116, 3.5520067965943616, 1.0582103608719422], [3.44635095059733, 2.2701222952497373, 3.2611498014408102, 1.301775993433776, 2.5790017660191276]], [[13.219378246035276, 265.40573224819235, 64.583596428368324, 0.28748662918171441, 310.79614795490323], [36.425236757487191, 24.803181417027798, 3.0890833081413831, 226.90229970631631, 0.45935882432607572], [209.61841806131795, 344.18297049042172, 74.468044094619771, 89.419203891444255, 86.520816387304677], [40.890107979900094, 41.447665182612518, 9.274525839129435, 315.93522069968429, 31.844525950940945]]], [[[1088.4947221979462, 3.3517216684672312e-05, 487.49333176676714, 96.40959672139256, 497.33664381824281], [12.760353221597539, 0.14034631089585703, 3.2253464664650753e-08, 141.74850772761505, 0.36965231432736723], [2.0123852059314302, 964.24683714451464, 6.4795140896739295e-05, 323.45733135414497, 0.029244534202375889], [307.18650611360169, 193.82335407476168, 9.0863712054261505, 0.23546829846814393, 0.0062011680836009109]], [[38.840253369575386, 85.802942572738999, 11.873905786581156, 44.157255510011588, 62.102928003807179], [19.437719766424269, 9.1766907869272138, 44.282938662965961, 0.12335751831562224, 1.3368072067819288], [2.4022177917872654, 80.461547749388075, 18.188845877202368, 0.3117947779489656, 15.919164531262028], [36.460354130548446, 0.037426228487437825, 4.6093136781011834, 0.0044667802443011566, 44.619713228411896]], [[0.71722455312066291, 2.4176515169391983, 0.43747699410798874, 3.7305446386141412, 4.0830350039142527], [3.9396117618971287, 2.6698885128574226, 2.150277801880863, 2.66626240661591, 3.3607781772110839], [0.2166982642639356, 3.0715914759843015, 0.99666718143746047, 0.19481077933001945, 3.3222621849741039], [1.9040375201537971, 3.5402960966993184, 1.7167471280705608, 0.87564043826433879, 3.1630913736024753]], [[19.022561901357346, 27.639770205669929, 0.074862049047160401, 5.815400036911341, 2.7156960170379931], [15.794374400906536, 16.75907624303737, 26.832790880062912, 0.013022908160584564, 20.502152903588602], [23.559610112677191, 12.802282978817361, 5.5861720047076346, 2.2146681859915383, 13.02181391393556], [26.933186806653566, 16.042561776395555, 8.7270864821437879, 0.82988396972403611, 0.025156248641406323]], [[8.3572675611992491, 26.997477968034023, 6.7772584141844492, 1.2259147380821849, 0.62314708239956129], [7.5179896682438496, 17.172002328145965, 0.10452825142841664, 11.517402481708757, 21.554516424341358], [5.2746794409470201, 21.18444359841202, 6.3774287996675323, 8.5114712662758585, 27.823363931137482], [25.514454570867045, 0.60018966586605249, 15.56501086343637, 2.8828970281745674, 27.341823706246824]]], [[[0.13371117398701562, 22.308592084452378, 7.069035680907982, 24.239610419644368, 0.82401284835387911], [0.090079074228967881, 116.30342692660651, 8.4159865941362089, 0.00089214720403877311, 49.219831882357695], [13.783834729450065, 55.52746051749449, 112.64552612666343, 1.1910667035321338e-06, 44.770418452519721], [0.46421401766060139, 7.8762160921250892, 89.58441258922538, 2.1523781211778004, 31.434196471677527]], [[0.008798850925699717, 596.06735110180989, 9.6374645810558374e-05, 479.2000922967573, 0.42688191945523163], [1055.889139600263, 194.15978608321132, 1.8258190241357366e-05, 16.734386018605896, 4.625299874674889], [11.853261086021831, 221.76966447718402, 1267.4881269297205, 595.42047545195555, 0.00016899173560970968], [595.30711818346697, 0.5955524979873893, 0.070793762767075469, 60.260418834284046, 219.48524611720268]], [[102.10104722847015, 33.616470827064369, 12.153205509054017, 2.3580910166820663, 27.974076559060443], [7.5082986262519595, 149.84802621300531, 324.18326966154859, 2.6849099834583958, 0.0010221667472632956], [5.2465786869711124, 1.4960683168499012, 152.21487453800486, 386.83405675332108, 195.24962377285314], [93.919372661071804, 0.35828970752931832, 1.8502105351294105, 5.707405338256291, 20.023917111971937]], [[3.3670064705796232, 0.59405473091135075, 7.1168595380992761, 18.64806580622291, 1.6625662064936018], [3.9097905680345248, 13.991827298649886, 7.7590446892071663, 0.16843113132391793, 15.93506734605597], [3.1672811205312761, 0.59043435939468125, 2.8935483850263268, 15.461830289863425, 1.4832175999868047], [7.2505640419723481, 4.3193131396108013, 0.14296351954065262, 11.914846064443648, 5.1087995120102283]], [[29.194905636704966, 0.010901728421862643, 0.6778148566063491, 6.1704683060150023, 156.08925669400878], [0.22857950758863152, 0.6156751600075856, 47.294443522545478, 0.039441358169032624, 0.18665961735931838], [8.4219175609109129, 43.715663150079159, 3.3760419348860484, 4.1886857702503848, 211.34564436738947], [0.68865234538835185, 11.896700336390237, 0.0071783074931785085, 5.2301075345635351, 37.755248812769928]]], [[[0.70086862437319519, 241.62158774257463, 513.37193009292105, 0.088588370824112334, 260.53455020753637], [22.172669756937697, 0.060400250161296465, 0.019644790128351568, 136.52217392908742, 0.36178961037679269], [95.361198336050705, 1.6125814650121997, 532.76999821012998, 393.54745114814455, 3.4618075848532421], [3.7915006215429465, 8.6983172941433224, 533.37505039294876, 78.393040900623575, 368.75357800737351]], [[1.6908414815594983, 7.4250724447333871, 2.4693713982138981, 9.5307703345075385, 6.5746721886622046], [4.2611944433203819, 6.0488984548588256, 1.9589856291187839, 6.5250940380020888, 5.0953389022640918], [2.9053750917851797, 1.033379906962433, 8.2851505033794197, 1.7651285671851287, 5.0681001984884873], [9.5137406333348764, 0.13892100756491918, 2.1876739433994588, 0.19716580614295806, 2.1683521424325525]], [[2.6666697642326374, 10.875926806070192, 8.0042884617955963, 10.984616138246389, 0.51966077435130809], [0.025160702021216397, 3.6242853764189804, 0.0072701349071887713, 10.263467364486495, 4.1132506926037466], [4.4839905508217157, 0.49011340514401347, 0.56717819722317941, 0.90630411104379116, 2.1882170803240726], [9.1588098767711443, 9.8133754013803305, 8.5508375838456931, 8.0091697875521568, 1.0733561116478687]], [[0.18233308650163824, 2.7329585754268853, 4.047394388698879, 0.80406995584334995, 2.166429319649644], [0.98870323931862369, 4.1129828112609363, 1.1486019936543106, 1.6763802371009549, 1.3594475071184937], [0.48505100236723953, 3.5440352685864234, 4.1757083337072558, 3.9362965081845926, 3.6621547754375583], [1.0364108544967154, 0.66052246379322233, 1.9160609184735529, 2.1728183687486258, 0.9540529738502046]], [[23.138221424109872, 13.191190506470253, 12.808960751714823, 5.8481072839581616, 12.362018562551066], [9.0105533724016933, 24.971074089876122, 0.96554644264580047, 2.4524052821258042, 0.31749950522903275], [1.6200179987162113, 5.1917034136919753, 32.739228731094187, 1.9718863971255505, 6.1440253719198257], [1.964909825046107, 1.1797047935695222, 33.524453136149205, 6.2866955072359909, 26.782986133047729]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[4.284103559789787, 2.5253212787480899, 3.8014753084063004, 4.4256128746505201, 0.76090131544176676], [4.7446130317888784, 1.227552500070102, 0.17555838966896081, 3.4102809066574564, 1.9305734688678147], [3.0373964980442238, 1.6846617783413855, 0.9891004757561237, 3.1148964493196494, 4.6927810091042428], [3.414785344265451, 4.2654126214900456, 3.6163550904474535, 1.6745636132664226, 0.67073274952569073]])+(1.-msk_arg0)*numpy.array([[1.7056127457394283, 4.241994621570953, 2.2584193346976003, 2.7062860553421295, 3.5281437730486194], [2.8020603660987282, 1.2180829026821975, 4.6891882691142595, 3.9478794916989166, 0.84956949584254138], [3.6825195085052491, 3.6909700467794586, 0.024609550254892046, 3.2058498776982369, 2.0952531239721659], [1.1602042961860637, 4.6698150965563086, 0.51049507596031962, 3.2494985870955371, 4.0337357873382915]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.5164308510723461, 1.9929802821626341, 3.2293756853203139, 2.0182324720474569, 4.7918247007835175], [0.3197339115805985, 2.6848935709898312, 4.7958567184893033, 3.3236941372033706, 1.5367155292234973], [2.0897109469612181, 4.961166426470454, 2.9010244331692365, 1.0841852536662451, 1.63881375552439], [2.4680815467312422, 2.8941836237563554, 1.3232786730817432, 0.5675362542675475, 2.9293517060021288]], [[3.8352566809049469, 1.2471070948314937, 1.2689146385115884, 4.1231617533467322, 0.34980622416420243], [3.95735131865588, 4.3483845233216378, 1.3109980303430979, 1.8192206162677009, 4.8531794588954194], [1.8651225667701317, 2.58771877320993, 4.1537060905726113, 0.46388864005327712, 0.25085831276978621], [1.4099229332883525, 4.5233632123844103, 2.3353132771892016, 1.8592452671468671, 2.3760961807452761]], [[2.9793107360998006, 1.2807124409668502, 2.6043241805167368, 2.0008393352149834, 1.455052218697001], [4.2664817550802532, 1.2115960239645067, 3.6202175365507152, 0.13744564775495449, 1.5149858433799692], [1.2485753543422879, 0.79848531834474212, 3.2278798734950778, 3.4170706729916902, 1.5987537727301686], [2.4854416986361501, 0.46446937327258192, 4.7203149193585858, 1.0768361370077726, 1.9303745914831167]], [[2.6571446433411325, 0.13100787865587799, 1.5958510830799231, 4.1475277636906096, 3.0358581122684924], [3.9462372587087859, 4.4816499742195521, 4.4352793133103434, 0.46096454632251649, 0.50957996281487994], [2.1339224921095008, 2.4481362250404719, 0.45560403782192788, 1.4913398416067971, 1.4081905273230408], [4.1181422872099995, 1.5350294568543632, 1.8464345469420764, 0.69260408708601118, 3.9937749808172578]], [[2.0695211206379267, 4.2675289163216572, 0.55440564445698004, 3.9743834581873241, 2.6660206727752662], [3.387022165848161, 1.1392434501963478, 3.2772046142474638, 1.8876335239467721, 2.1440310409804115], [0.086517857582330396, 1.4695372298064904, 0.079424290236978987, 3.3527972489968603, 2.8690805303502915], [0.60770966808449911, 4.267692052415283, 4.0586438293407001, 4.1355894151869954, 2.6448606196546902]]], [[[3.2799401393239256, 3.1601872445258565, 2.040265597818951, 4.9942017967125736, 3.1939824734037812], [2.5112350703684774, 3.4487555693404031, 3.1477482837336197, 3.8688362545607942, 1.3309823835655317], [2.6470434414362543, 2.1782870979891373, 1.1189931258090735, 3.6432270804272568, 4.7918075795731099], [2.1475125631715986, 3.5072852616911345, 1.9756140254742476, 1.4154223660528304, 3.0256922161739905]], [[0.23089418812022872, 2.7367360127033189, 3.6744530437534313, 2.9080156264190027, 0.033692145018140406], [3.1512490037481018, 4.857223381331889, 2.273902148493872, 0.72812950064742976, 2.1523949242431697], [4.0887815174887256, 3.4161884396083928, 0.61281312485283201, 4.1038342797965592, 3.9216281919269043], [0.7424242763143768, 1.0235143694483431, 0.20168997596986557, 3.6231507079889598, 4.014496180398706]], [[2.7195636841409865, 3.1187616588168359, 0.78397107339123573, 0.38484531272210315, 1.8795660593717942], [2.8163931263821387, 4.676497353870011, 4.0084489224029261, 4.6585475149811639, 0.39676506635824405], [1.0025037933262932, 4.4398075749426535, 0.38803086337711501, 1.4743302755920562, 4.1350577239224853], [2.5856834642667943, 4.9516120964666666, 1.0181335136968532, 1.7070120352979195, 3.5940914585487849]], [[1.7791766794173518, 0.53884805346843401, 0.1652836328048217, 1.8068983947330353, 1.9023593862359018], [4.5921643940245458, 3.6721996088902973, 3.227439546800432, 4.1167300913839444, 2.7373386774190935], [4.6976266543047931, 4.8070696088323208, 2.2219633783856492, 3.5224307319084343, 0.077165770070035417], [3.8885424574834229, 1.334623973558255, 0.70804832038593724, 1.4557221976682138, 3.0795705114077316]], [[1.879814848585055, 4.5383387044427019, 4.0171161042715076, 0.39917803654673395, 1.71138900862458], [1.5102276504868302, 4.7538925595001746, 4.5298351431474986, 2.8750557548291233, 4.2487351613606794], [2.8665968355884521, 2.9731729940053229, 0.73701760311287456, 3.7712396517943034, 1.7798761053048169], [1.8717590618220827, 2.3155024892400409, 4.3582525606868519, 1.2080724626094017, 2.777425175290571]]], [[[1.7768455441452826, 0.49743646959692206, 0.37495669249240332, 4.9203979188207345, 3.8441306429328916], [3.8235268874526525, 0.98716466561586214, 2.8389823376608363, 1.2229491986134082, 0.92550713839002807], [0.17660207360725572, 1.5493731594557067, 4.5754104030986689, 2.6411403989979116, 1.7905514426917768], [2.6813799319441798, 1.1612447566337643, 4.2974622014578188, 0.75543439497621589, 1.2437033249306761]], [[3.3990250659847665, 2.8362095632431159, 0.89477448686137295, 3.4431215085684972, 4.3518250907694949], [2.2362490162452229, 1.7811537813328633, 3.5598947827594536, 0.017165350927452829, 2.8105447598343374], [2.4683604050376755, 0.91219667536982885, 1.6116335166973046, 1.0800723980262394, 4.566365986755355], [2.9712244314785266, 1.6860181682183069, 0.075061139700259627, 4.61060619457374, 2.1970799649475934]], [[1.762633839788726, 2.4225732122030674, 2.2104998186925684, 2.8497045343758471, 4.0188240305744918], [2.3945270488029062, 3.4947471704468001, 1.8833180718709244, 3.8433056414931532, 3.3840617964960056], [1.5702623500154165, 3.6343846869414289, 2.6543102701825321, 3.5058079343820387, 1.0405967792180608], [0.086490077815708208, 1.4079298942147147, 1.4525542612350992, 1.8149040880532665, 4.8859508157090064]], [[2.5738147907600655, 2.9455770379916633, 0.3006418943685647, 3.5294326542283119, 1.9576894426672515], [1.6470161406266308, 1.5712058695547433, 3.0519294991609605, 2.7134619968968487, 4.0180062067577964], [0.22842925415037207, 3.372760189088567, 3.5063529264035549, 2.9060166964832628, 0.62674789257718888], [3.0615317749664013, 0.24455013162925451, 0.13741710589537118, 3.216822851446457, 4.0880216275835393]], [[3.4090493068773657, 0.49853871609025718, 3.619922460450181, 4.7979984044061785, 3.4323134960773634], [2.9522982793021995, 4.63840528023169, 1.8330099871379737, 1.7568550426629115, 2.3501425425601807], [1.6586981774514431, 3.4402079445337601, 3.9374946857395465, 1.7779657676877414, 1.7023448465069801], [4.8668908751474165, 0.82431254351685801, 2.876356633655401, 3.6876267259028443, 2.3459738259873566]]], [[[3.0528739982746083, 4.5631027247296823, 0.062741427749684803, 3.0203331453795026, 4.9245696383348765], [0.51668279771971148, 3.3866592735497116, 4.2199459575453382, 2.8700774122079658, 0.32052649642636821], [3.8575117651527591, 0.03399549004611107, 4.2055574146455612, 1.0943960990881936, 4.8771442996244909], [0.7729827258855626, 2.1751805084535598, 0.35066965043368664, 0.63620134916113136, 1.3382564718632164]], [[0.56513283451074781, 0.21820684521424596, 3.8759163452765328, 1.8263314480396369, 4.0534098952001925], [2.4370065604944862, 3.7333874218189012, 0.3330328007221745, 4.5615318186903249, 1.4310270554098417], [1.3821053166686741, 1.5374023754670405, 3.1742198167506497, 0.59722568858278524, 2.9978631475657904], [3.6203726426353202, 0.096279910763649432, 3.3581505152997808, 1.4653321594660231, 1.2490421771585105]], [[1.3676567202232066, 4.5140344405024058, 3.5021058019398734, 4.670341913989243, 2.9115737606731158], [3.6023034966993372, 3.9685713632636213, 1.3059614170630525, 3.4019581372693617, 4.3356071686843922], [4.8066936136573934, 4.0139212784119174, 1.0816030809897319, 4.5486528003853754, 1.4663095426220516], [3.5514407788457159, 3.4764387358083852, 1.3035739936609947, 0.7306041147406358, 3.4564366895434739]], [[1.2558106984593123, 2.8680077305626215, 3.4462148603453113, 0.28312966756901081, 0.40719759137091222], [2.6418675756957524, 1.5449500781253112, 0.41477251263629994, 2.646258610998486, 2.041569523562226], [2.3598417818997199, 2.3982546146123505, 2.085337799083212, 0.18416452116882634, 4.6143654846928568], [4.6647569242514892, 1.2114201474662407, 4.389394840948456, 0.82771292066967139, 0.76038973732464032]], [[4.5760362452900463, 0.69597471907332342, 2.2051596436580985, 2.0702842428083952, 4.9689886862353632], [2.39049397413187, 0.53603793490551011, 2.7186029975514479, 1.2435972771642763, 3.5268271366096067], [2.423943525656886, 0.21518248246080326, 2.1711258728543981, 2.680469258010401, 4.6241526465260376], [0.74591317097051402, 3.1682443195292946, 1.6495469423948093, 3.1609548971044048, 2.4752657988567455]]]])+(1.-msk_arg1)*numpy.array([[[[3.9494183790045629, 3.426606106891438, 2.6588657194444574, 1.7312542175493821, 4.3594266803745843], [4.9572545891096649, 1.0273021419431179, 0.31979020735075109, 2.8792536121785655, 2.0955790780693193], [2.1791993326545116, 1.6334642429316633, 3.3983765443819594, 2.0361539439140706, 4.3527634628955738], [2.0864743076184302, 1.9351334061782288, 0.83932198463616914, 2.234062049319304, 3.7262383473651028]], [[1.3269776962810143, 2.5085857553855462, 1.3605359959479981, 3.5239634341205108, 3.6784750424871775], [0.80196221123240374, 4.4932568244178759, 4.1944878978700508, 4.8556074229352308, 1.5182053175803811], [0.30197416869689159, 2.6959432104754559, 4.2930072235660859, 0.11620090006538603, 3.9593026985916278], [4.3983221500950318, 4.367993020895871, 0.23782112979578779, 2.2920128337816283, 1.1440586288193575]], [[4.2910588740686517, 4.9063694298466203, 0.15302555016939884, 3.9764600064647166, 3.1185093358719991], [1.0994887694719278, 1.6415661922508282, 3.658739244789206, 3.4633640373585401, 1.782209240151311], [0.15899627375522399, 4.7822818975111314, 3.7449197130138234, 1.5673698952497681, 2.3225357749379412], [2.8068134347085336, 3.8916523182226994, 1.5818476988568779, 3.2079563637376314, 0.86026442751007193]], [[0.72859927213643527, 4.5101967582305136, 0.31265438595071821, 2.6559088891964548, 1.1685667439081584], [3.9069389135684003, 3.2734596383075534, 3.4496653900094834, 4.5893986267510014, 4.5165836169831994], [2.5547465274799235, 0.34780061735109929, 0.85576459583826381, 2.1550087039848767, 4.7689937376508693], [2.3690636294816301, 1.4887734091756704, 3.907733944587545, 0.7328498498315863, 3.1638887968627531]], [[3.7486975693617253, 3.8659308831440935, 2.0083058401988696, 0.29940589155422348, 3.6617123885856095], [2.07819694356116, 4.5711685747518223, 4.9223735583129145, 3.3728432440910634, 2.9946776689282153], [0.32508814941398556, 4.2983370364523354, 1.3250679522974504, 2.0018917867630508, 1.0056121039733676], [3.6648359285998615, 2.58996888853531, 3.5532985057174216, 3.3163334247346694, 2.5888714228005933]]], [[[4.736731057735005, 3.1694207298650023, 2.8710640469257296, 1.5550873011261002, 2.9613423304862838], [4.9178315232636249, 4.3586024355628847, 1.1907682269606785, 1.4943182471092398, 4.1593746030868735], [4.9540320402921472, 1.6016822786173737, 1.7430631804769219, 1.6216464758185205, 0.97904352193409871], [4.5101004970423357, 1.5945874729382807, 4.6840981702547628, 2.2610640484996849, 2.847098662139532]], [[4.8187997939411913, 1.8890196005827051, 3.8137115713491516, 4.9520697450858018, 1.8642285685689211], [3.4611112011039533, 4.6535732694463885, 4.1065175972605168, 3.0117473980849314, 4.2196130399260019], [1.0155670325022859, 1.5259854448128856, 3.8795641539301924, 1.84326457584961, 1.7489871899663862], [1.3923992788414843, 3.708786515584054, 0.77169671568782816, 0.84088793254956296, 4.6292483080007303]], [[2.7198508770240748, 3.6357151038610462, 2.6709303071587467, 4.7596298969848627, 3.4252433230864745], [3.92220614242185, 4.9233479864132939, 0.49533485597965687, 1.9182258129610261, 3.7865325097799136], [4.791566770414768, 2.414376239019834, 0.90682479671563565, 4.5573354005035283, 1.7841234340031824], [4.2531916083460963, 3.6826808235199069, 4.7111455984125294, 3.7210052907944577, 0.80907186647010088]], [[1.828659145623851, 1.0345438094373363, 4.7425592412109365, 1.5823749310433524, 4.2263849340876609], [1.673033972273557, 1.6525868970425597, 3.953164257259695, 1.3867804339930476, 3.5789734742430017], [0.94166609732008844, 3.8212189687046512, 2.8804059492708869, 0.5257231437745431, 3.1834247651093963], [0.8513350392834641, 4.4743162109101648, 3.6688054557138194, 3.4889993478920407, 0.64288652344926411]], [[0.86226169164842636, 4.7746602554869106, 3.6114555545729243, 4.138985503311253, 4.792842832577648], [2.4276832600664995, 2.8502437323820828, 4.1798130899317902, 2.8413680949168549, 3.3854694331586064], [4.2076612651576291, 2.9595201794040387, 4.1494932220384397, 4.5902153913759065, 2.9946010382423371], [4.2161279131736658, 4.3609795746909041, 3.0058178084230769, 4.459308293091083, 2.5175129208757911]]], [[[2.6775652276053452, 1.3821978946399349, 0.92826610229125761, 4.586871252688657, 3.3787853921862978], [4.502333628005684, 4.4172890509105978, 1.6487734795096824, 4.5516863905281362, 0.025484692878733128], [2.6912912471339516, 3.3149503836162815, 0.284632504954975, 3.7617867598229973, 1.3758624493717728], [3.0434821503573946, 2.33422669018077, 2.3566588247056668, 3.4154760430097384, 0.79805820078329237]], [[0.028818616946144952, 0.75478586336321551, 0.33930356555273028, 4.2095581299320406, 1.4498622350054855], [1.5141249526163594, 1.8432874832772834, 3.1350166098177459, 1.4189489256508021, 3.9325874637098015], [2.2913819068757717, 3.3406727411979027, 0.87812298815864787, 1.0221902351277827, 4.2081554988585106], [3.5596321120382268, 4.46601410574588, 3.7193665897716128, 2.2854763781769027, 4.1992765879128253]], [[4.0018610022963736, 3.773877206098748, 1.8571910230374782, 2.0760582582178624, 3.9268884606970667], [4.2378395647486, 4.0706781791667677, 1.4846908708874982, 4.7669911021697269, 4.273861255757005], [0.63349051544872015, 4.8248264907904472, 3.7148593788273838, 0.098732705094510667, 0.61393184787464494], [4.5780253280364027, 2.4147715315088742, 2.9099962533485959, 4.920511068243675, 4.4828060960199965]], [[4.2705988554663596, 1.2545236977136076, 3.5515556062168225, 2.8761780578328948, 1.7016894010932808], [2.7974632171759213, 3.8526889555941044, 1.0745844613646856, 1.92293308200373, 0.51150108654045789], [4.8262577112496716, 4.4569579178996204, 3.5338796907058572, 4.8683224547578456, 3.3784899810523257], [0.3029519238503055, 3.7703951826384827, 2.0585791806394842, 3.4650490427770939, 3.1947506694014041]], [[1.6769718050186329, 3.2929471076444399, 3.6611000374303009, 0.014904701488797308, 4.5079103977712069], [4.9008749996034133, 3.232214126940943, 0.84602620286055019, 1.9738243660081436, 0.92768758337527946], [3.4953432998060792, 3.3073379314192262, 0.98747841886356158, 4.3036193390358433, 0.078857788481432267], [2.4423411001955797, 4.5963765594401398, 4.0964150829278241, 3.8078361869788258, 2.4959756438756715]]], [[[0.089627027651745669, 4.7042789798374178, 0.41612064988216968, 1.1435801719947869, 2.2462655782025256], [1.0424362583591253, 2.3628885078642954, 0.65067789347884031, 0.8063437670062632, 2.4617945613407928], [1.4428720106873596, 3.206339306420086, 4.7316933849420444, 2.8045867016495962, 0.51039367197277641], [0.65775383486674388, 1.3665317200021829, 3.0183560977524397, 2.6855865678962232, 2.321680187317829]], [[4.8545877617835931, 4.8434444190181818, 2.5263640245752246, 0.34665483222008214, 2.7078728301500523], [0.13098260431362696, 2.2246288543051298, 0.54535586516567147, 1.0812697917536147, 0.7645219740310728], [0.7047514861113412, 3.7630941330901586, 4.9856353961645903, 4.9557089888601817, 2.6463162958070496], [1.9378103329293572, 3.475297471196785, 1.5541864214511176, 4.178702146031025, 0.48956813883820055]], [[2.4613144539970042, 2.8027021810105857, 1.8011926215599356, 1.2681124056478741, 0.26141445482292519], [2.229779775571449, 3.8383143897137977, 2.0025681986939947, 4.5027333258359654, 2.0575080475293692], [3.0350352720912444, 0.42440702966675536, 3.4498072778406481, 0.30451335812071828, 0.36284129716703567], [3.5943759855159771, 0.060195584170132731, 2.4889583435658276, 0.9036878888618165, 4.9304745650384882]], [[4.7196737231930133, 0.067501339274848166, 0.29918910647500901, 3.3949491705126249, 3.7595926943533629], [1.9458056253405289, 2.7065006893815218, 3.4982723209331108, 3.508394675731739, 3.7226854513232901], [0.010130163910267621, 4.7140751506284593, 3.8746055158114769, 4.4922004093099002, 4.3583491330360253], [0.63631100313796218, 2.0772931678271669, 0.45692272483373619, 0.41738617811056822, 0.82779501152237289]], [[0.15405632212861745, 3.5469748309557567, 0.72011916633350626, 4.3383194023662197, 0.44915073764465557], [0.61000347424347745, 2.826756938218514, 2.933996212265225, 3.3541108901501739, 3.854047808587421], [2.0190819132204094, 4.1300465298511355, 3.1334606596320866, 2.3388556909431149, 2.0808197527725794], [1.6003672856743869, 4.9000225699911297, 3.6600047578195025, 2.6351355741529137, 0.074428990573695716]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[166.68321119836162, 18.167051292499291, 109.77746379086835, 18.846915789421306, 1066.012178306514], [1.5923079254653809, 49.713775379224607, 1072.2840270989327, 125.92421328079554, 9.3538201137497339], [20.912438481931986, 1363.83685518577, 68.083391850871791, 4.8423173862244129, 10.85177535701516], [36.264517925399346, 67.409134367579185, 6.8568841211383988, 2.2835127204036847, 70.947982994039251]], [[34.913032638883458, 3.1749137656534114, 3.239705039864647, 45.58443489675161, 1.3827156046963451], [39.093835268676109, 56.160044460926485, 3.368498204753426, 5.393897911496893, 89.642682573109639], [5.6282041343294944, 10.992141678348055, 46.892681749778774, 1.5368451756682269, 1.2616081127507748], [3.6917769559473355, 66.042618008028697, 8.7003232134445128, 5.5976443965542924, 9.0353093698218103]], [[53.43892810575629, 5.5303416839564594, 32.387950751368464, 14.467421089025502, 6.9800847782231674], [298.09546501673327, 5.0427535997605837, 125.76304110017652, 1.2014671533477104, 7.5616980860081657], [5.2980248515633317, 2.9045796899213778, 74.475932553813038, 95.882106200290238, 8.4566961406390977], [27.633590428856955, 1.8593879955634884, 546.45813969588482, 4.2122434018267718, 13.168155333929043]], [[52.0528865073112, 1.215143610637766, 10.736862338719922, 477.74120618943249, 91.428933328146869], [354.13135848736596, 785.28403704101265, 732.94697178301737, 1.9850472345591275, 2.133905153598215], [23.903264336082749, 38.144412864922948, 1.9692828525595465, 9.1910663361819935, 8.1218254859354566], [457.30975435484294, 9.8081755610340782, 15.586465497079443, 2.8015851847267217, 380.07778849045394]], [[0.56807605365585045, 0.31157688969216291, 0.8594245571569461, 0.33756180253896984, 0.48263508274777234], [0.39632953694717948, 0.73249401883668885, 0.40840276298548839, 0.59702344568705545, 0.55662699713286357], [0.97663611898415237, 0.66927957279298467, 0.97853099663466414, 0.40005340181526533, 0.4565847860421155], [0.84699740897517573, 0.31156300077648236, 0.32987847542775034, 0.32301502054400333, 0.48543377097086227]]], [[[165.15743810224936, 137.06328444380983, 23.967858060175402, 2382.7761100748144, 144.46861034944988], [49.899840445558915, 214.80840279479276, 134.43424095058913, 413.15228614767955, 7.9434899251197875], [61.649972296996346, 29.713887351411884, 5.7103673034537437, 290.77231678260097, 1738.6978062332587], [28.32368185774267, 235.30388114458435, 21.672638116423066, 9.0596147992604621, 111.16689848578098]], [[1.0484768259080748, 1.7525853626200603, 2.1240932317620751, 1.8152227666896035, 1.0069315555303253], [1.9080398817562221, 2.7069955062770363, 1.5939269663040392, 1.1610012984434139, 1.5547100493769948], [2.3124130601113149, 2.0145482583506191, 1.1338743387504489, 2.3195605433542243, 2.2345087974990165], [1.1644088922131663, 1.2334847836068894, 1.0422178106087361, 2.1018688371175651, 2.2774614868002381]], [[0.0088136620053545986, 0.0044007938011604722, 0.25565152150249054, 0.51194014836046908, 0.038004976006256012], [0.0074472195196177218, 0.00029277875407959455, 0.00093605750772483539, 0.00030206615610479796, 0.50143296463110409], [0.17479531026339643, 0.00044195417966101327, 0.50911073744987356, 0.076917980364780092, 0.00075099951025957669], [0.01112534264730978, 0.00018141316411855098, 0.17010626036279788, 0.051311954570738942, 0.0019247843782036734]], [[8.8700857065604577, 1.9368363942052438, 1.2247896059161407, 9.1769353704609014, 10.317116853583185], [279.67907368522987, 90.471500527730001, 52.426155541082281, 156.08215672081704, 28.736051559396966], [318.30922587312722, 364.04859135609627, 15.270041224329891, 75.286379139163628, 1.0992922405710075], [117.9715085479794, 5.1413185347356771, 2.3836427914335276, 5.9647803166386799, 43.72852672366907]], [[3.4437959571001882, 19.794354547782476, 14.048668710541037, 1.300288464412926, 3.0826232606486736], [2.7005448870689905, 22.809834564201203, 19.6839382360603, 6.6277197676576014, 16.360838982180258], [6.5909427564911827, 7.0696008088487803, 1.6238871933482779, 11.950652798902523, 3.2246774357755847], [3.4255947445228756, 4.5867747860388954, 17.583008852592826, 2.2137562101595516, 6.2154476802167293]]], [[[7.1999688716257797, 1.7378561963266883, 1.5167610306046206, 236.64608108969304, 71.581163765945959], [69.961226120075892, 2.9943903875311135, 23.432207365565073, 3.8911216444021077, 2.7961378045835881], [1.2167763594228447, 5.5920990434996742, 161.3029563611978, 18.808500468301336, 7.3104436734623839], [19.668435086126827, 3.6333089016942379, 118.44913849695037, 2.3147186966532094, 3.9818849236483445]], [[5.887384798874594, 4.3897273867459781, 1.5946956340180602, 6.0243587679673434, 9.6770670454472576], [3.2102499899023189, 2.5319433968675189, 6.4026750316524348, 1.0089930397117888, 4.331358646335592], [3.6233896205065799, 1.6092523520805779, 2.3176907946196423, 1.7565080301514522, 10.822808958381762], [4.7099916336525647, 2.4093759212365637, 1.0399256806791608, 11.075439328995779, 3.1453325173404583]], [[0.98086804668041594, 0.9737994986119255, 0.97606542441131194, 0.9692517123932235, 0.95691210479819488], [0.97409885940127872, 0.96242398776930005, 0.9795715867184297, 0.95875456281356186, 0.96359215568415602], [0.98293816106534249, 0.96095228044320635, 0.97132948794546514, 0.96230733087686593, 0.98866050830845809], [0.99905257325913488, 0.98468842318307803, 0.98420697464610163, 0.98030631886944908, 0.94786149454117152]], [[18.622263662532379, 28.410323438835189, 1.4071805470611529, 55.153871525441104, 9.247180886379935], [6.4969510731710267, 5.9607581617291672, 32.059380113231946, 21.824308564997306, 96.085864619335979], [1.2963341339196004, 46.160124955004491, 53.726365077556274, 27.16160193342948, 2.0382842327326784], [32.411065046436001, 1.3202971951870492, 1.1689813792265271, 38.665238410241749, 104.04187366939084]], [[194.50866734568376, 2.1613942600230032, 269.47940551349831, 1665.4110853194532, 201.63189863286664], [95.998066126118218, 1301.2651347493984, 17.011336045116316, 15.121876521513363, 37.840501588107664], [12.992723468009483, 204.1078985830527, 440.30456820231882, 15.623562449138465, 13.899714846747512], [1852.5841435617058, 3.5765838288229177, 85.363647404771186, 299.21562946424416, 37.59740597553251]]], [[[42.490445292405497, 271.51387258093695, 1.0801000390510396, 40.825844984013877, 423.23858067334129], [1.8861650717691942, 64.02065817694124, 178.14135894321305, 33.946442822182284, 1.4823716641332649], [114.14466679199138, 1.0426341618434052, 175.02111195419721, 3.8345184700892481, 399.29172418358974], [2.5839297466817186, 14.459780180485916, 1.5382763394800896, 2.1843715942055129, 5.1734116141015463]], [[2.2699258657060741, 1.3723402882784104, 276.48802701262463, 14.142222838221086, 357.67645496609657], [34.294084292670156, 224.84780858831598, 1.6210547859745981, 747.45899527618985, 7.9706053430185335], [7.4245906839701474, 9.3004425469824152, 99.915955925011602, 2.3780934771379534, 77.363661371409293], [190.85069696286322, 1.1498801941984327, 130.46800562842512, 8.3772636795260471, 6.1213599963304866]], [[5.8012713128285007, 331.17292593161659, 90.182930572770729, 404.87108838469334, 42.213114038278718], [102.57979291143603, 164.26255388560361, 5.3589594913904133, 79.289339544715489, 263.29586678844601], [482.43407390187093, 174.12299006095989, 4.0163144379634996, 346.24320074415584, 6.5856431362939256], [96.087442500278726, 87.255972564079116, 5.3425383066817167, 2.5578503909535137, 85.041047274662716]], [[1.9106365774138434, 4.3868386740771816, 5.9103499218251532, 1.1571594323824979, 1.2335938985161417], [3.9040782138336279, 2.217771876496474, 1.2384208353259505, 3.9129263152982507, 2.8649086931817478], [3.3757670440917433, 3.4432865463781219, 2.930289806604268, 1.0996000228328651, 10.793543506997288], [11.077627897396802, 1.867406946743307, 9.6115349184791707, 1.5322382117214088, 1.4799686536216035]], [[0.16079934665983925, 0.75732596218038495, 0.41448992969296972, 0.43742962024469484, 0.13744422195626443], [0.38491749737104552, 0.80727956392355171, 0.33764240357078928, 0.60855165144206325, 0.24449507303838353], [0.37980949045378504, 0.91764878388324955, 0.42016237345230567, 0.34282406400495269, 0.15773877601527689], [0.74237099690365038, 0.28214117876644362, 0.51747012015918159, 0.28296376822584451, 0.37210365772452297]]]])+(1.-msk_ref)*numpy.array([[[[8.2374509744757987, 6.2310739716842898, 4.1356038759851588, 2.520252052987654, 10.253349124037408], [14.10880852114949, 1.7306580438505152, 1.1861868131658124, 4.6520307164598913, 3.0614261217318495], [3.2012064578618848, 2.3920397678422138, 6.1378606434607379, 2.9658163681085918, 10.216936060522949], [3.0465798503870154, 2.8100857437925257, 1.565389242656247, 3.2963648524275233, 7.3120939119513597]], [[6.8041134062128901, 37.524373262641149, 7.1421954291150422, 162.75492244058341, 203.47028812131617], [3.1863062538159408, 660.44049686194205, 428.87945891629676, 1114.8953438380161, 8.9697488585949703], [1.5470699955329243, 49.19187615829938, 494.49628214401275, 1.182835123273414, 305.30930743908453], [575.77861554181004, 551.08916962108538, 1.4100984442200517, 27.44092547458796, 5.22368528222575]], [[32.975872595792978, 54.437226606778403, 1.1327684408883043, 25.520535047040216, 12.686531316416593], [2.4490867338594513, 3.8088480078627485, 19.700566450674557, 16.801732644877688, 4.2712382533629194], [1.1382918080431828, 49.203189231300819, 21.133418738785998, 3.5854415956630938, 6.6332108411845523], [9.841548961990437, 23.816855759906492, 3.6279806813114694, 13.645506613790838, 2.015419690334193]], [[2.0655093389937558, 89.143659572418827, 1.3651599336533269, 14.071654151957175, 3.2007925422572852], [48.894232439506141, 26.023029547033101, 31.013057732402689, 96.457336155750369, 89.712295394073635], [12.723459391364521, 1.413773509074602, 2.3442833574880728, 8.5461219620017985, 115.34199702224771], [10.575984519460158, 4.4025719984485905, 48.932948267403411, 2.0742686374155883, 23.333629877778556]], [[112.87178370055182, 130.85070064022483, 12.578833852726046, 1.4586079639748248, 101.14792469243936], [13.737540761510571, 318.36573766817685, 495.70957007044876, 70.272880004849782, 43.623911221396561], [1.5066098591024193, 225.70357117208815, 5.3154008399453385, 12.477523305920817, 3.5531959718397239], [101.54703779856557, 26.18953671186657, 88.225749984673229, 65.440409995785643, 26.153324514356143]]], [[[131.69840387504769, 26.196569564582305, 19.263558914746156, 4.9643976113271737, 21.141413714072591], [158.71533598303131, 89.202384915409652, 3.4106878500327862, 4.6630897502985018, 72.648473949871047], [164.74711162074564, 5.2085490867637461, 6.0253396347756674, 5.3167995872920102, 2.7422051056874794], [104.27217624315848, 5.1706125457721006, 124.7465315373903, 10.274844523483072, 18.793711480401807]], [[2.5873761731088947, 1.4515942659721122, 2.1220068632498679, 2.6563036786687126, 1.444512241217383], [1.9794160897046742, 2.5043987983718221, 2.248192431350458, 1.811494509858175, 2.2989162090673951], [1.221829429826905, 1.351268404129212, 2.1497542978799786, 1.4385504517142802, 1.4120423052914135], [1.3161227022789821, 2.0785340156748267, 1.1644384613385095, 1.1804419287840822, 2.492409516044066]], [[66.878355559616836, 275.37209866006469, 62.009053940577672, 1563.7831642811664, 198.91741774271597], [428.72953492829225, 2013.9438663969961, 2.1498990492354637, 19.378353467559641, 347.64272389914294], [1642.8930580767271, 41.714041429649335, 4.0603924169879484, 1143.9766708975292, 15.751469597108832], [715.00154093279377, 296.10007640331418, 1450.9045191214605, 314.1651301632931, 3.4911317949342098]], [[12.318172368404861, 4.1396589988005887, 673.42783849359284, 8.7835740154781039, 331.48459405281432], [9.9480290204091713, 9.672598171264557, 227.78462364296209, 6.7146869608927435, 136.26067442075572], [3.6439772293329091, 190.03648938473899, 52.211971143281538, 2.0583645692896879, 79.155221458686171], [3.2188850924116039, 465.92993000075228, 154.14984196804548, 120.42404596346991, 2.4176548847487584]], [[0.86886225843628495, 0.45914393491879973, 0.55501537234616793, 0.50927847039830254, 0.45778494446885326], [0.67315857157754855, 0.6283471337715445, 0.50589999984361378, 0.62925698222850313, 0.57584430436850165], [0.50360843799018018, 0.61725234262809392, 0.50840680971064067, 0.47315970155882692, 0.61373230501746301], [0.50291379694192406, 0.49117685587207943, 0.61261104737593364, 0.48336601729251116, 0.66337231424624676]]], [[[32.801244117188119, 6.06072396816114, 3.3537699159000538, 395.21596812910877, 81.824365421168679], [353.97608777840338, 316.83004435944116, 8.5791636911685014, 377.49808038748864, 1.0337797787161174], [33.39344573490429, 75.290949025348723, 1.449249921520243, 134.8084069774726, 6.010875375702792], [52.850898277999534, 20.965774036914468, 21.58791715912739, 85.833115494776592, 2.8301972521004362]], [[1.038351047651878, 2.679598016440635, 1.5575170189220102, 244.01189075138637, 6.6416444565268495], [7.2230665151493838, 11.102046020189706, 59.978410574010347, 6.3788653183755342, 169.9533393255752], [19.931295499389439, 78.456851948452169, 3.1478869620769308, 3.7994917189831545, 243.56534791292796], [104.42671342207558, 341.08103000019088, 128.64813798496093, 19.778177190941534, 240.75754768238974]], [[3.6426760306348158e-07, 8.4766224382707642e-07, 0.0010279504756410388, 0.00045691719488952123, 4.8088761759062273e-07], [1.5196917527067221e-07, 2.8229347846105712e-07, 0.004085886674641475, 2.1399594484468345e-08, 1.3298429043606629e-07], [0.095670807369308442, 1.7272468112678567e-08, 1.0548152615523445e-06, 0.69366425216944927, 0.10286018648234291], [4.3095495936360842e-08, 0.00013028156352281085, 2.0802605245214825e-05, 1.2117355090068293e-08, 6.1323898358226129e-08]], [[144.77059919895427, 4.3123868287650957, 62.644845539333659, 28.52217742972736, 7.2603599016645219], [26.023004707253232, 88.969614253944329, 3.4968635099138266, 9.394958831209749, 1.8146400305073505], [276.57452886834204, 179.873617454813, 61.368051447187582, 290.46549659062038, 51.206367617160026], [1.4232344916624737, 80.836171067924624, 11.00332903765816, 56.639307761899538, 41.339223794240127]], [[3.4570411410507074, 11.423910531174823, 14.999574888545746, 1.0110856207974437, 28.061142182774567], [37.526660339085439, 10.92207471770087, 1.8697110453737464, 4.3059048102497135, 1.9861274302473562], [13.268805770825251, 11.546162115840653, 2.0759367058937226, 24.125682063936903, 1.0600637893375966], [6.0893242325380506, 29.958764850214443, 20.697506654028665, 16.719217875142657, 6.3357564161873148]]], [[[1.0134073100175627, 2.0118154888872706, 1.0637856444288476, 1.1852237337475025, 1.3962447624490213], [1.1675435009738728, 1.4206521596976218, 1.1015168691943893, 1.1272934071440486, 1.441685749220299], [1.2391246014175781, 1.6103467664571391, 2.0200276846319012, 1.5170242195226937, 1.0787926646336017], [1.102675676307072, 1.2251475597760588, 1.5659864825608278, 1.4904345310768312, 1.4119795240220847]], [[1774.9006926610004, 1744.6801626768183, 49.078880225681132, 1.7061451264912009, 64.919988130605432], [1.2236764888469898, 30.827860922144183, 2.3174293804814763, 5.2928980742103562, 3.2485856102919919], [2.9627154859790181, 330.09406600239703, 2172.1219248514321, 2074.2183131494771, 59.044393064725774], [19.814168187519339, 211.84367514289337, 10.970248920005263, 626.33117570265529, 2.1265118199202879]], [[0.19110647396166197, 0.15191032786465589, 0.29787670950569434, 0.42628541255963204, 0.83881226527079389], [0.22329833273379321, 0.075714627991170846, 0.2601556002015204, 0.048435507447965945, 0.25072075066395544], [0.12994036339038487, 0.75174329679984364, 0.098316667517836617, 0.8148534873436537, 0.78351493265595318], [0.089209656343636007, 0.96033416842222785, 0.18758717814953854, 0.54464749056211625, 0.036329531809930905]], [[260.37914096057972, 1.0828001051048666, 1.4227542027710578, 54.650042153069712, 83.9887310216603], [9.9059273324305508, 24.279028828918172, 61.726688583429258, 62.467448287518131, 80.413930287805769], [1.0120099524921058, 258.66683481728825, 96.180374188572245, 199.15060406492566, 170.08826693692333], [2.1167734353707801, 11.566250985136383, 1.713406942941581, 1.6354040587533634, 2.6526443316625494]], [[1.2396898966327183, 140.74373555550895, 2.73011878140345, 424.37752406960999, 1.8709145270380925], [2.3414433125863705, 51.54514097278458, 59.860916078742576, 107.54988448204469, 215.98593129891395], [16.709865467389939, 317.39490032774103, 79.060664489276746, 26.10131045413662, 18.212435328292464], [9.3186797062565674, 928.92396446765156, 164.77531185411863, 39.456744017470534, 1.1093847559675023]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[1.8330182743955314, 0.51315847573514439], [3.5081313841494084, 4.9678592431396824]], [[1.0258222707908158, 4.3721981758909285], [1.6634020126353788, 1.1841933714382604]], [[3.7812524116647661, 4.6636773739427699], [4.4324520321811223, 2.9550409820078141]], [[2.6457849618280407, 1.7622308946360954], [2.5568270711833447, 1.0982574892177273]], [[2.5069279846847907, 1.8253817113244526], [2.1112035031434506, 0.11089467367952881]], [[4.0726172614925567, 2.5551269593936849], [2.526995844755652, 1.4980030787443543]]])+(1.-msk_arg0)*numpy.array([[[1.7777657981247783, 0.77365394115174146], [2.4807313382338383, 0.085764537010346364]], [[4.3763603423854756, 4.5766418463589984], [1.4106659592123203, 3.0761970130687932]], [[2.3896835332690451, 1.6030744986612715], [2.4686271706486531, 3.1140530144859015]], [[2.3897733108834349, 2.1352715160353566], [3.2063551498505682, 4.3999464806354176]], [[2.5893825833705568, 0.65882432197369412], [2.4264211829317768, 0.14732891095987533]], [[3.5056210198166098, 3.8883675903421158], [2.1511534166653146, 0.89281798644805965]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.5994709737091073, 4.1891720273323818], [0.81315542040855027, 4.7447955983529688]], [[3.657618783805638, 4.7304165816569475], [2.2300397852721354, 3.4319499918560932]], [[4.3305083934575617, 0.52175413406175941], [0.15896518356994319, 3.09716992780139]], [[2.9154018514376454, 3.0267958585623806], [3.9951833072070984, 4.5214437046485045]], [[0.037211149689526483, 0.63091217864605798], [0.056602724025506021, 0.83655237314944653]], [[3.4905197296035877, 4.7199782956501117], [3.8861275928188541, 1.5467459697285659]]])+(1.-msk_arg1)*numpy.array([[[0.90822788822037237, 3.665520758176406], [1.8390274295540725, 4.1256668773801]], [[0.98629587963012932, 1.528244910921652], [3.5586402081432023, 3.884784503702686]], [[0.51865831072432889, 0.58136509908097944], [0.66264233275879736, 2.5881980638631608]], [[3.2607068846622367, 1.6855053918902172], [2.2640400527749027, 0.49575795238899856]], [[3.6714632286576805, 4.2865661226764953], [1.6454332783112764, 3.0329224853088594]], [[0.11130350793473628, 4.2873506203234042], [3.082454394813487, 2.3365490377953977]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[16.234093147394255, 0.061121459472582115], [2.7747980580152056, 2009.9300633777395]], [[1.0977352408593835, 1073.4384339524947], [3.1105171406376773, 1.7864167408177338]], [[317.28980143718451, 2.2331189600166979], [1.2670503967235798, 28.669171786629317]], [[17.057536095298108, 5.5562515153644032], [42.544326345401878, 1.5277124888311355]], [[1.0347907250720865, 1.461811011805187], [1.0432041055513612, 0.15886117443176465]], [[134.51658249690027, 83.748701692222539], [36.692099661270063, 1.868415969364027]]])+(1.-msk_ref)*numpy.array([[[1.6863318080080287, 0.39036041424984319], [5.3167097518473554, 3.9735904697791059e-05]], [[4.2887149638649662, 10.220625608980217], [3.4020987718981499, 78.673739327640462]], [[1.571192466319701, 1.3156880535494775], [1.8199423500078613, 18.91583002371646]], [[17.128144494823434, 3.5916580958909012], [13.983973305018635, 2.0844628648289989]], [[32.887793529170985, 0.167164482714033], [4.2996804279085197, 0.0030024923462590837]], [[1.1498316308289962, 337.7063774307515], [10.603381119860041, 0.76728228874778692]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.9163562747145964, 0.73737978055861297], [0.81853408806473771, 1.4859537861456147]], [[0.18715357292716223, 4.6314846420768134], [3.4616310242677164, 1.5806571336013557]]], [[[1.1982360745471909, 4.8072275310805734], [1.4945594088414826, 3.4639701835925649]], [[2.0365967291346885, 2.4561896016788594], [1.9064907840934913, 0.44241446862023559]]], [[[3.9538701176862507, 1.4114362337477548], [0.31455111867899088, 1.3059559869286812]], [[1.115028829863002, 1.0704282681970807], [1.4736634418506247, 1.907484376321757]]], [[[0.54178811941300364, 2.1401292294088328], [3.4899260636687273, 0.72294665306699679]], [[3.684608788823867, 0.43359515888746752], [1.9738177736434623, 0.30685792053239136]]], [[[3.8083332701815489, 4.6368246301973706], [1.6787196016146968, 0.16658798774682618]], [[1.6296330777877763, 4.9753866615606306], [3.4141637729122545, 1.4543642278316573]]], [[[0.051533107064767984, 0.99260763561215615], [0.1147988504587486, 0.22635582031478885]], [[4.884806962538824, 2.1432847898252758], [1.1018212023101239, 1.6563569812523118]]]])+(1.-msk_arg0)*numpy.array([[[[0.093528335988408193, 3.0487088016518378], [1.0741768772530245, 1.2938893134654335]], [[2.4868652419160275, 3.6012055571049957], [3.5942386764579157, 1.0779659403759938]]], [[[3.2496084128785436, 4.6107456958612696], [0.20239244084112301, 1.6356901829322286]], [[4.6420870337252502, 2.669695964442893], [2.3982613207442807, 3.1402488678207519]]], [[[4.3399219488730907, 4.7945146082332215], [0.067671905305766611, 2.5860316356193254]], [[4.5077891573037689, 1.1201735260585042], [3.1866031077046828, 2.6644468615887447]]], [[[1.640942204011185, 0.46498239409413539], [4.4402498999010316, 2.5739300943523746]], [[0.39019216958710001, 3.1623557252309573], [2.9623737984365666, 2.1173567150616934]]], [[[2.9893444281027697, 2.4949157750849187], [0.27857400258927434, 4.4647345960296825]], [[3.183561261270683, 0.17935793012061857], [1.0543825748398792, 2.5568705389897519]]], [[[1.1876523988941305, 2.967412117525305], [4.3692815614384255, 4.1754246064680949]], [[1.4428935261591573, 1.1840438281543302], [1.5312254250687858, 4.719410361570894]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[3.6477579555518078, 2.312226283009498], [3.3347917898048935, 0.51898434012544203]], [[1.8704070829179804, 4.6280710640462335], [4.5104008903511152, 1.8774958142166707]], [[4.812949795694915, 4.5666403262938546], [2.5118362052727141, 1.9422816352469956]], [[4.8355630908314664, 4.9982490602289404], [2.623602920494867, 0.22246109550153376]], [[0.12490301736140329, 4.9553462526144063], [3.0194182171439605, 3.741320402315095]], [[0.32796118048565104, 0.91887577765465556], [1.7757093141409039, 1.3045091818964916]]])+(1.-msk_arg1)*numpy.array([[[3.0753770580171929, 0.3294909262329872], [1.4016950539595068, 4.5063938160514132]], [[3.8035854025932347, 1.832531407669431], [1.4830879519509503, 4.039125902950965]], [[0.29912660193420071, 4.6479666338496832], [1.0995788493206762, 3.67741937885148]], [[1.0177353854725613, 4.959868993669196], [1.2449281099711933, 4.6007883898688515]], [[2.3658945970830323, 2.4313317051384558], [0.34143208642749667, 4.8585732351767019]], [[1.2429070039040917, 0.63062044392602701], [1.3067964939728522, 1.0316414107279646]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[333.38612647692798, 0.32913162397467144], [0.62939216883672533, 2.4987061292816035]], [[0.0037405225272975416, 165.97334682787101], [1.9049267344458809, 1.2682172027895631]]], [[[1.4025108944816362, 18.85474381769297], [6.4218389856043281, 314.18647664704855]], [[24.733499949823571, 57.575480235565188], [3.3584547056833922, 0.21629477768753]]], [[[747.20338861123412, 5.2518479203211559], [0.0050831922250794796, 3.3837942556964182]], [[1.314543400674361, 1.1864345175373339], [2.1236206735730763, 3.505372935588889]]], [[[0.051631607308085768, 39.615346666695373], [516.57179841158938, 0.1975959112902978]], [[30.618653522596016, 0.11164869124038772], [1.1633069722721361, 0.7688887703695606]]], [[[1.1817770374224941, 1.2111919396978452], [13.027008291194734, 0.00013898724223874789]], [[4.3690589062650229, 127.06081815008753], [98.898024859242867, 4.0608084291258209]]], [[[0.3781066466255476, 0.9975695443362681], [0.13683568649213004, 0.25534872305664896]], [[16.718324663533167, 3.8716993900705812], [1.1348392538181995, 1.9314716156799059]]]])+(1.-msk_ref)*numpy.array([[[[0.0006843249107158423, 30.820456666743276], [1.0238567409215009, 1.0886018992178321]], [[3.5857813597267612, 6.0251836030868011], [318.99476630332919, 1.4025913176194875]]], [[[88.469390617334852, 334.74209589820208], [0.053527781234795992, 2.4638478239637429]], [[9.7452824535883629, 4.290227921089774], [34.233382015278586, 101.6951962374397]]], [[[1.5512714519196358, 1.5981915598615952], [3.6624970846704356e-06, 82.776734503264194]], [[5.2370236705859972, 1.1329039169177217], [70.949289458263209, 36.739704793108203]]], [[[1.655419426052801, 0.45871016854845215], [1625.7582059628962, 108.76923591664706]], [[0.30986403889694986, 4.1925418059692339], [147.88195456674649, 31.543464654942323]]], [[[13.340207551189582, 8.697458848419652], [0.044716360034470187, 38.007452358191848]], [[1.4849461580494792, 0.55615553232754833], [1.2934169144131367, 95.693572362928762]]], [[[1.2383172324090252, 3.864753132886996], [2.5342861998920529, 2.4627850981143502]], [[1.6146812611200967, 1.2470295809455281], [1.5520082691218218, 4.9569039142419511]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.7329115542387408, 3.9623918792500188], [0.99507946623400578, 1.0914737178823863]], [[1.1482380910362158, 2.6481946378859953], [4.4490876000995394, 4.2843332126552962]], [[1.8910893641104893, 0.57386375317439065], [2.9175662878966935, 2.3908344956360286]], [[4.6462404615391888, 3.5278502482383729], [3.898905256790965, 4.977367719693186]], [[0.96262868889055986, 0.71880755803178786], [1.5400487280306434, 2.499713431513054]], [[4.9928206459259519, 1.5371410128195642], [2.1455755141624819, 3.6742500318662601]]])+(1.-msk_arg0)*numpy.array([[[0.13023871901737807, 3.5613810955424161], [1.2384717401796073, 2.7079170402290038]], [[1.3445283285743859, 1.1153406373722745], [3.0323601880361282, 2.8973171387002963]], [[0.033208779504839685, 2.1120081404158584], [3.850373115679973, 2.747331968213389]], [[3.0114830541389455, 0.66204115853158885], [3.8987277821327222, 4.0084578765986132]], [[1.0799146962828572, 4.272479249516449], [4.4625907655800932, 0.015792840333685877]], [[2.7266997166651676, 4.1677356427019543], [2.9451445565065177, 0.097670071931825683]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.6151598833078626, 1.291332855618182], [3.8500397302722438, 0.61704943856991035]], [[2.3118409767382024, 1.2246854024523992], [1.2664833720397028, 2.9643785712810922]]], [[[4.3569930917729254, 4.7916358397700964], [4.0557754677754696, 0.98962315375330923]], [[1.0395617492682478, 4.5372424159463831], [0.43519746296373818, 3.1360815614979418]]], [[[3.0420145800379341, 1.7523956235213736], [1.302507098608132, 1.0496024610016748]], [[4.0577009066503793, 0.56305862889046732], [1.4012419461338901, 0.14789720344032387]]], [[[0.43391556126030695, 0.12954853271319774], [0.16713853696802367, 4.9871100273509779]], [[3.0309838415241663, 0.9860799157365765], [3.556462612476055, 0.47174815497263284]]], [[[3.8392331193876124, 0.5851882897075289], [2.3832700872262715, 0.77465585072892662]], [[4.921836254498781, 1.8757653844737578], [4.744456336210332, 3.3234881664136333]]], [[[1.653805931277436, 2.4788728216271014], [0.6235697134382554, 0.27768520321694468]], [[2.1818700314685175, 2.1443114444101736], [3.3466308293455524, 2.390721388895146]]]])+(1.-msk_arg1)*numpy.array([[[[3.4529145166779487, 4.2239473600966102], [3.6707466931251247, 0.097943874119835114]], [[1.5066644895939791, 4.683743366460976], [4.1218395231625315, 4.436473598189032]]], [[[4.0479955506704446, 3.8409557222829513], [0.22168987355686706, 4.2940196162326254]], [[4.2780114042286153, 1.9563087258916829], [0.47889375587192429, 1.8164529705860444]]], [[[1.43079728924623, 2.7867019325672926], [0.63982239999435808, 3.0803397822769516]], [[1.0291993380624167, 2.7053696547782189], [4.571727764473458, 0.56733694977890137]]], [[[0.73744636617675852, 3.1996124735302116], [0.13164867203424149, 2.0975164140130795]], [[1.6351428564043007, 2.4390668200903423], [1.968205160933121, 0.72143194980474823]]], [[[3.6182533744224257, 1.2819982612271179], [0.59551671683252672, 4.4190848307833548]], [[1.2669670138581501, 1.0383436264186168], [2.1849211513867353, 1.1929139337085954]]], [[[1.9076918692558404, 3.5683642184694753], [4.044372420424053, 2.2988065606557493]], [[3.018742139774524, 2.1624100149692742], [3.8009982623638732, 4.5760506574727016]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[103.53730248912865, 3.6629328142062669], [200.52099128688326, 2.3386718776609108]], [[0.98866120377726419, 0.99397722951714795], [1.1172315471054224, 1.2962410187714597]]], [[[1.8262439519207554, 1.9393278859631322], [51.926593884681751, 2.621567327181523]], [[4.7197358152325668, 873.70029522138736], [1.8836229932027604, 95.859990105837028]]], [[[6.9464369296005968, 3.0542809894673213], [0.48511727772579, 0.55827106230118684]], [[77.075268502292829, 1.8274013265176592], [3.391872015328925, 1.1375916990846462]]], [[[1.9474476899334274, 1.220174779715598], [1.2345538667504297, 537.64216080764186]], [[61.821241214552899, 3.8257510846699572], [301.19896111665349, 2.1321037557152471]]], [[[0.86396167877229579, 0.97795818047594762], [0.45527015567983914, 0.77432652892521658]], [[8.3755611916292718, 2.2478672135781763], [77.227645310834504, 21.00793912293047]]], [[[14.286593891695741, 53.840716880532931], [1.3074618174758699, 1.1268020659290885]], [[5.2891452122174005, 5.1396456022016368], [77.877323523993809, 22.447134832647539]]]])+(1.-msk_ref)*numpy.array([[[[0.00087755287927905536, 0.000182267337655396], [105.88918306599378, 1.1324724560925838]], [[1.3802195994953372, 2.7230440082126894], [60.709059664501595, 83.05682604367189]]], [[[3.3147487386797203, 3.1176798220776023], [1.0244948236714799, 1.5979708272939821]], [[115.09691877912694, 8.7601583353388364], [1.6643590299379827, 6.9054831671457011]]], [[[0.0076597161976214344, 7.5713380869311321e-05], [1.6134202752050959, 10.003978188879675]], [[4.0049685588725383, 38.37086167711746], [101.52716127673378, 1.7742324056806402]]], [[[2.2546279350939717, 34.033907625245249], [0.94715214423708094, 0.42102058127869407]], [[9.2521652211838479, 27.624884438096078], [15.373865640812241, 2.722745861667673]]], [[[1.3207222526221523, 1.1035835509913854], [2.3745453067701643, 612.39023043756936]], [[6.6528139868629417, 4.7260094539688451], [0.00011581843267145031, 0.0070944400593615179]]], [[[6.7773788024553738, 35.85193496871895], [321.44533474917307, 26.609123191629024]], [[26.068251969158563, 10.337150631170688], [0.00014457235173937023, 2.38285426896735e-05]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.3340140161497915, 0.1968200465346491, 2.2702055493308659, 3.2621942329902538], [2.9429292737836321, 3.2355744378207372, 1.3429554352950956, 2.7247029389272135], [0.1927655391760538, 3.4410473269725959, 4.2574092934703485, 3.5386618775907075]], [[2.722197863867962, 3.8122748993252138, 2.5363419760089814, 1.9777612467957739], [4.737552374300801, 0.29882587202724581, 4.0014058928439553, 4.3646795941486234], [4.1980125346571571, 4.505957516559346, 4.2460984113834357, 3.2858682504111618]]], [[[1.0639626456194158, 0.79467829490074804, 0.11759575644554687, 1.9583652105739031], [0.091529146050895446, 2.0382313415733728, 2.9230075206902466, 0.20590222935205593], [3.0445340343314946, 0.78859942092412116, 0.18613655254125896, 4.5574543999144534]], [[2.960421089088777, 2.368769171985988, 2.0352254986904192, 2.7413391992121632], [3.3165076653188135, 4.6332322299357607, 0.80559462621301459, 0.43788895216009233], [2.2571730858165595, 1.9235921035080004, 4.3217179318312722, 1.8866138800777399]]], [[[3.7042298150166406, 3.1504008894837114, 3.629484335009896, 0.25850202125938654], [3.5333576323871672, 1.3738686272295937, 2.9333640571620454, 3.5445215586538246], [2.3331010173213627, 3.1769645235380004, 1.5085474198859545, 1.4232755710229192]], [[1.1660119493366545, 2.6803616228099592, 3.9685224080344552, 2.8104317623070729], [2.6217138430408484, 0.066922879279693814, 2.4621193260782506, 4.0316928958020037], [1.8251904124505618, 3.2381157750123997, 1.3147223622029334, 1.4734063070415806]]]])+(1.-msk_arg0)*numpy.array([[[[4.9791259194266999, 0.10124633363424591, 3.4183556410114857, 4.0357321911895481], [1.6488476441264783, 3.9136741008087728, 1.1542782818077006, 0.63829071373986013], [4.9919611864130395, 4.7320038572967862, 4.978730886862146, 2.0590140741685587]], [[1.6229524417176349, 3.902217701903119, 2.8358895215370996, 1.7044877914403282], [1.7168673262338505, 0.32837101736884128, 0.22139873648201294, 0.76030551910585731], [3.8206325016038285, 4.4889288233999691, 3.8072758282289545, 3.9357155658793896]]], [[[2.04454875785876, 1.2591157998857039, 2.1257350501746899, 2.3142739384462514], [4.1866634985237292, 0.61724258336034177, 2.8325682410214199, 3.9555172997937031], [1.592270802557824, 2.4305454326690619, 1.7581000978342975, 0.43777594171989592]], [[4.163482993293032, 0.818168406333457, 1.0460802172860322, 4.7369298923601999], [0.35825567112493989, 3.4631979504624733, 4.916294314537998, 1.3519943601045525], [0.50956722154608503, 0.81728178212134783, 4.4548891384700333, 1.5010237473989125]]], [[[1.1101128348649423, 4.2308802231914147, 1.8828273655662933, 2.228469020958618], [0.55012793187400177, 3.8863986566589785, 2.0938505397199907, 4.013954831181473], [0.21986475989401752, 3.1614262964356805, 4.4475463546309095, 2.5595200895143853]], [[3.3856464191307007, 2.6972801272692473, 4.612428121108584, 0.17956434019113354], [4.6878818776195086, 4.7020396919515068, 3.5088587843261281, 1.7076890529693554], [1.521487834579744, 2.2533928625675723, 0.11467296937519797, 0.10504928474215175]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.2107148436420152, 1.1207865950859661, 1.6099309779255231, 1.0415281480811693], [0.66596593221544187, 0.4493907510106192, 2.9105766274468348, 1.1272218006882904], [3.5334454662119943, 4.7967479765531369, 0.11553286178217662, 3.6437278590437026]], [[4.0096039380703026, 4.5351416497999644, 3.5912299536672241, 4.8261625694811618], [3.6692801354425884, 3.2882655083017518, 3.0688991244861339, 1.1823780777939985], [4.7043728039592043, 4.1916276753815476, 1.8376291229230761, 0.7692506202383449]]], [[[0.79564048010314881, 1.5864078276673668, 3.0019384852845925, 1.1028049998535776], [4.3172019467866818, 0.62849316410275946, 0.091914232945836893, 2.1387334695044986], [0.12499876300667512, 2.2169093567026628, 1.6658079132793515, 3.790637656470047]], [[4.6153342364668779, 0.34865800078124165, 1.0150485394861921, 2.1868651976164299], [3.8329597983208754, 0.55370932849918164, 0.6589007871916267, 1.78055708245564], [4.3922398440972135, 2.0410637626442192, 2.7123463542262476, 3.4936128717758295]]], [[[0.52942963524699405, 0.23232928198021488, 1.923176762201924, 4.6895560819434463], [3.8003902777692771, 0.04581386157312925, 0.38152316447006146, 3.7166295036352328], [3.5834667754139944, 2.2970000216393718, 0.38968434181447387, 4.6132605463823158]], [[4.8471466374391099, 3.9214018858015729, 3.3832308701436111, 0.87242169445574647], [1.3547934718169747, 0.89294503256500057, 1.036117520681433, 0.78557078898006005], [0.3009596683929544, 2.6332473024393499, 4.8062768545972832, 3.1211003323538105]]]])+(1.-msk_arg1)*numpy.array([[[[4.1532428938406785, 4.6634078331890958, 0.23378065772583095, 3.4742268073882587], [0.87124945598201686, 3.6113194964878783, 1.4116487701805778, 3.00388757815014], [3.9309165073276606, 1.5601005565180579, 0.51458108348580245, 3.6494318084278934]], [[2.9562334214408379, 1.6332841217098197, 4.505329232690598, 4.8403664270223281], [0.47095406297637027, 4.4199891925005668, 0.077558567612983284, 3.8333512022142893], [0.91165977152274491, 2.6814178209250139, 3.7652168907391461, 1.1508859249351295]]], [[[0.92479110940287379, 1.3285017187411405, 3.3821370441913197, 3.730659921585934], [3.0842318942099984, 4.6543016094290799, 0.31356903946534737, 0.13105202827785853], [0.435594015109104, 4.5174887391360077, 2.0401238025696533, 1.5857535668510503]], [[0.8531310636785544, 2.559616728411791, 2.450476429492765, 1.957647463156972], [4.3972334916708489, 1.9649136670258331, 0.19351351965088237, 4.4737017973100928], [0.27667676829354443, 3.7162006064822171, 2.9378570663815013, 2.5751745147443597]]], [[[0.27748891271679915, 1.5499737528747646, 3.4995351955074785, 3.8700023924169242], [3.3600662337288307, 2.8588851031082139, 0.092709035650630461, 2.1138197162959376], [3.7850748221592441, 2.6044487985161364, 3.9171117293583873, 2.3052681513537454]], [[3.8161613104493455, 0.089976351616552902, 1.5954472824152997, 0.22626728046823461], [2.5248125027810757, 0.6711696400084497, 2.778668940613656, 1.5451584174225244], [0.44273418110881646, 3.8308543267775219, 1.2133744339125305, 2.9517012903604325]]]])
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[35.479598934932852, 0.1617343875028093, 3.7431770110928237, 3.426375273384108], [2.0520663232457368, 1.6949911722651378, 2.3590280423959973, 3.0952893085565747], [0.0029763936254641409, 375.29239966078154, 1.182189149613621, 99.958608077021296]], [[55.444145834757563, 432.26763035073839, 28.288061799232665, 26.877149747534215], [301.15977317713811, 0.018838041359482621, 70.490362436065922, 5.71038843384635], [853.15901418000215, 550.08787428808671, 14.256496604404592, 2.4970856867821563]]], [[[1.0505669121239711, 0.69448485354911638, 0.0016194700405647031, 2.098465414201176], [3.2873221451076899e-05, 1.5644572321386778, 1.1036119666317721, 0.03404896490039392], [1.1493161411969346, 0.59066346316595075, 0.060768681177480244, 314.03472719702859]], [[149.78002431375489, 1.3507667156054612, 2.0571061523237306, 9.0733059028665775], [99.026028434280576, 2.3372556621651879, 0.86724165159508337, 0.22984132102797011], [35.722486393385623, 3.8009549249675909, 52.981026471973948, 9.1860703198930018]]], [[[2.0002557796011331, 1.3055243408917887, 11.931107334543142, 0.0017567773022965712], [121.15094745313039, 1.0146582773383808, 1.5076861399002797, 110.28134967083562], [20.819851477375842, 14.22729752511369, 1.1737662918014304, 5.0952098315020597]], [[2.1053253876364448, 47.76594468962017, 105.99846121807519, 2.4633115340448146], [3.6905969161667009, 0.089392830487525124, 2.5435613168684932, 2.9898764925673418], [1.1985143073994335, 22.066121900311021, 3.7251978511901891, 3.3523664630476979]]]])+(1.-msk_ref)*numpy.array([[[[786.0428070651908, 2.2997364121986167e-05, 1.3328956371938627, 127.38304710054193], [1.5460317798681831, 138.04240905376292, 1.2245050344262494, 0.2595957466786698], [555.70689641740444, 11.301555458910423, 2.2841471112703777, 13.953343377492091]], [[4.1851691550031873, 9.2422917837792991, 109.52528004611331, 13.212935458123429], [1.2898827040699861, 0.0072834597521575783, 0.88963676391303659, 0.34977257570939108], [3.3939801928261204, 56.061809230878879, 153.51079679608534, 4.8395435957386281]]], [[[1.937482465047351, 1.3581177317430453, 12.813883541162243, 22.882715310670946], [82.791412557346206, 0.10585672806865586, 1.3860846942815144, 1.1974701662676976], [1.2246087728919306, 55.260182876306338, 3.1616899383288701, 0.26984470864164439]], [[3.376586147521063, 0.59828860329208244, 1.1167180725522752, 21.008009467881294], [0.010956777786944174, 11.482232878940694, 1.3609472091414956, 3.85427373719889], [0.82983061408822978, 0.47245026598346346, 80.573100214410829, 2.8459597749230285]]], [[[1.0294111640090495, 9.3530055138664512, 9.1560682926104136, 22.22216556335048], [0.13425820266407859, 48.467222107602666, 1.0709139216230288, 18.8731073743736], [0.0032360097580606318, 20.041020793515674, 345.74853361839592, 8.7280030445681263]], [[105.00205147458416, 1.0933851069086826, 11.462113221127364, 0.67803858496337466], [49.441343136252975, 2.8262992796742488, 32.72167497927088, 2.2861693867153323], [1.2041940807660156, 22.473298491236246, 0.072239394007081861, 0.0012925422551324313]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(4.3728658799)+(1.-msk_arg0)*(0.880314263488)
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(2.00334433199)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(19.2165424393)+(1.-msk_ref)*(0.774622892442)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.2369640523083776, 4.4969500263530993])+(1.-msk_arg0)*numpy.array([2.8344065970488579, 3.885482772225048])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(3.65259157167)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([195.15289668123245, 242.57330643309206])+(1.-msk_ref)*numpy.array([44.942728597909763, 142.23346625420265])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.48167311822)+(1.-msk_arg0)*(2.86902788909)
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([2.9203641564254461, 1.7449628462713425])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([38.21362514286389, 8.8186056178003369])+(1.-msk_ref)*numpy.array([21.714619076606997, 6.2911518365130226])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.3693900364742695, 3.6616272813570006, 2.1536710009451165, 0.69350497098043817, 4.0779248787915394], [0.63379757067457365, 4.7499160983952322, 1.6051648383943482, 3.2408169070533734, 3.8058376850043878], [4.427905277074589, 3.1232461450281854, 2.9330008226098312, 0.71798613111765963, 1.8219366932053933], [4.715664126399866, 1.1310289775424207, 1.8350355728644023, 3.6608354010908619, 0.94110860296808885]])+(1.-msk_arg0)*numpy.array([[3.3226881070837018, 4.1416134178195048, 2.6501753055790371, 0.41015139287718777, 4.4243722129925995], [3.5024921726948608, 0.033487010074861052, 2.8815685995687823, 3.9695836779536742, 4.6147614364473757], [1.96310006659926, 1.0845418101858311, 0.58343607687990562, 4.108062406263052, 0.79844437337373553], [1.0066044213666345, 2.2604037102787315, 4.1763431897176995, 0.56703370431742239, 2.4425751652311303]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(4.36661178749)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[201.19244552687411, 289.29846081140397, 28.501316586427183, 0.2022670849148519, 462.96813309350125], [0.13651960574843847, 901.20749523311508, 7.8963151484501974, 169.75787921315563, 342.45298548555502], [663.28046845991366, 144.46189854125154, 109.79205009311097, 0.23535025390891259, 13.729278104141326], [873.17282903620105, 1.7119813454964277, 14.165539759843725, 289.02536314516635, 0.76717597343812849]])+(1.-msk_ref)*numpy.array([[189.29647811787473, 495.38159957832914, 70.514084270980618, 0.020411623120732299, 660.97259683000175], [238.27791005803468, 3.6199934228783353e-07, 101.62984136818267, 411.61084160322952, 794.47816263488141], [19.01804636736431, 1.4253020356232882, 0.095100653088068685, 478.09554733928769, 0.3742315521757808], [1.0291612284307825, 35.204064881147332, 513.77848635586952, 0.083966413793912967, 49.383443958074423]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(2.00767445645)+(1.-msk_arg0)*(2.01762376719)
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[1.1419153054921136, 1.3138001283991125, 0.87880618619939366, 0.81181999887901923, 3.7739461454305849], [4.5132793952901231, 4.8208484134044074, 4.7923093622554127, 1.3456126382811608, 3.5078675505948635], [2.9162156483122432, 4.0837420988946835, 4.9428391386321957, 4.8122460361240416, 2.7427278272494178], [1.2867022042605338, 2.9832431791426259, 2.790718045655288, 2.3502542815776137, 1.9248719023230132]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[2.2164100193723768, 2.4984950185392285, 1.8450525449165132, 1.7608909815221845, 13.878680829178769], [23.234824942188418, 28.789789122948736, 28.22278740984504, 2.55451184782806, 11.529433855848801], [7.6334157948087151, 17.223497478081523, 31.344710150489938, 28.617691779424387, 6.7640267936301237], [2.4517496839317063, 7.9984844212331723, 6.9940967639937224, 5.145246774740647, 3.8251266190893789]])+(1.-msk_ref)*numpy.array([[2.2289569117629404, 2.5147746533360769, 1.8530854262302001, 1.7679719027796972, 14.140033609965581], [23.759042069630993, 29.484130377659703, 28.899376520859128, 2.5715608527197014, 11.731106935357994], [7.7442563082986204, 17.574731451839291, 32.120034141316331, 29.306636164301516, 6.8563607281381058], [2.4673941924414642, 8.1173150674008845, 7.0912534157141822, 5.2053741420460877, 3.8616979989297517]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.4082030590272625, 4.0628998722294245], [0.48304662393532133, 0.60500246798392565]], [[4.230115502581044, 2.8177063332352219], [0.60233421185492864, 2.0011683404960929]], [[0.35930027093200961, 4.39532071079321], [1.1110395348869087, 4.0584830407603434]], [[3.6789760982756885, 1.0321970150615891], [3.8234207151546453, 1.6144155970847629]], [[4.247877328110456, 2.5757232323109718], [1.6169865727554422, 0.16925376433548753]], [[0.86209944645620706, 4.7068737503568174], [3.828178325814473, 0.53742899057515947]]])+(1.-msk_arg0)*numpy.array([[[4.1856662073085209, 2.0160910304647826], [1.8348680046935748, 2.8295953260355353]], [[0.14009593876393212, 3.9818709561294705], [0.98887308011013964, 4.1705555452737135]], [[1.7956061772348519, 3.569729690255464], [0.37058176591429576, 0.88827650768346522]], [[2.0735575704141915, 4.3603806353696166], [3.094977924313441, 2.4871959341916021]], [[3.4662477768277915, 3.5218836784694414], [1.3945355794011611, 3.9091017434876099]], [[0.15530007580966373, 0.44522622125153605], [2.5094002470585175, 4.9013177141133726]]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(3.23619597261)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[17.188330075085936, 93.392714381756264], [0.094913205258578756, 0.19666331511861634]], [[106.41394673068761, 28.572664613380795], [0.1938702210049818, 9.440888076105054]], [[0.036422814036028942, 120.46017544610731], [1.40601406727069, 93.064547650564222]], [[67.733470741948366, 1.1079966999791544], [76.723623248849123, 4.7117085724900969]], [[107.86674861925177, 21.367333114244595], [4.7360344756113006, 0.0031871233460731628]], [[0.61865853405041205, 150.34686191323757], [77.033012231612176, 0.13404980830678279]]])+(1.-msk_ref)*numpy.array([[[102.83763208536419, 9.6706236104968664], [7.1297923578026126, 28.964661381916013]], [[0.0017284856913354131, 87.49832102285896], [0.96443704015603449, 101.6410242365085]], [[6.6477841272253704, 61.437881755485975], [0.040255413086370387, 0.68154073112143909]], [[10.591448520752436, 117.38870049667385], [38.713639850980663, 19.080735239667824]], [[55.85878388889796, 58.812694696043351], [2.9336107399063978, 82.428447288233599]], [[0.002412531588940807, 0.072901822608479078], [19.637519034283777, 171.39086959890889]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(3.76232040276)+(1.-msk_arg0)*(4.63894603594)
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[2.3613724720500224, 0.88297140571254895], [4.3995082153731566, 4.1079584410658772]], [[3.9713862392601347, 2.5318698406704243], [0.48292820778199752, 2.5525905112253047]], [[3.0801564303738505, 4.6434532000897022], [3.2261503889824645, 1.2375481526604619]], [[4.4269570467397568, 0.73268113054420669], [2.0637926135678071, 3.7882000896118275]], [[4.4768896381607206, 4.8301298738541814], [2.2225124753509715, 0.99439791737153205]], [[3.2582140799324422, 0.22058386724616522], [0.81640936725471347, 1.8651688583109938]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[22.848907125560807, 3.221892137204291], [340.19068808258328, 231.17902062240512]], [[192.91107312570131, 28.640403236932265], [1.8962859505022114, 29.437637826153786]], [[59.223465133010272, 470.00445292147685], [71.863254457575508, 5.1541165455889617]], [[352.79141662580889, 2.6401316884157855], [15.403569892265056, 151.33578321414598]], [[376.92245269871376, 601.90249427131971], [19.008928843763947, 3.7344962433296023]], [[74.982189422678942, 1.3394800845367723], [2.9499013142453316, 11.839195604481679]]])+(1.-msk_ref)*numpy.array([[[37.468321511944616, 3.8764055276753338], [854.90385474855418, 546.54115825247391]], [[443.2089470587195, 48.672872638239248], [2.0981314228572607, 50.24532201973782]], [[112.89527226662238, 1243.0452153983124], [141.24365584045924, 6.679223852077329]], [[891.68139851300145, 3.0780300905600471], [23.732925237338385, 334.60292224571521]], [[962.68842599691595, 1655.3577572923721], [30.277865370344884, 4.5992390640888736]], [[148.36682844334609, 1.4028180681659295], [3.5000236244127367, 17.497844919291111]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[3.6321568029753233, 2.4950342882967744, 1.7831722333662352, 1.5099216602251215], [0.87321886056383424, 3.8391245721032239, 1.9148705380435052, 0.17298027320540652], [4.9933217568074619, 3.4292611542500797, 1.9476076571006642, 4.3467256996072923]], [[4.3086446222161836, 1.737970415303171, 0.41958683819668091, 0.20298924613914746], [0.57897588320868487, 0.76897067546831499, 0.62642487013058379, 3.4020208218625156], [4.731881465550253, 1.6645732561402853, 1.6464201048635694, 3.731772608720028]]], [[[0.15161900962681907, 2.9157465441288375, 2.5000421098827297, 3.3555316210204635], [2.8417976637680118, 2.8887149636500236, 4.6924049160482637, 0.2302599324281871], [0.95369397489192653, 3.5990261856548487, 4.6779081019045163, 2.4190564206278675]], [[0.085721434282177891, 0.82327118659545595, 2.8358617342414636, 3.4741753882800999], [0.36110036810625462, 0.66026314973462841, 4.550472902520255, 4.9374458871762448], [4.968433245886386, 0.60163070784170058, 2.0115208810000236, 1.9110820296708382]]], [[[0.67651565859394946, 4.8631398224808491, 4.7118119603161688, 2.2975406866221997], [4.9037598539814473, 2.4301457195033414, 4.7987927366077994, 0.43733910627037847], [0.35537288855639215, 0.25218848696217655, 4.4224640582235395, 2.2897449698617307]], [[2.1197144754992361, 0.55454549004194753, 4.1221231198412607, 3.969470557686511], [4.6402091760274873, 3.3644314904332195, 1.3238105302320835, 1.1449200486514439], [1.1835694596221549, 4.499054028971651, 2.8653531923243367, 4.6494742215191298]]]])+(1.-msk_arg0)*numpy.array([[[[4.9351868819257012, 0.13553530566495448, 2.4763155195034616, 1.3581329205804795], [2.8275626947417605, 0.5528161105259185, 1.4070818909956362, 3.0356817418568913], [1.294478806753504, 0.65641386035283567, 4.6938765082914058, 2.1506415457458918]], [[4.7374681968369883, 1.9878867674472502, 2.0110161510875164, 3.9371062371121006], [0.89250550092640502, 1.0301212505166739, 3.0990220546543537, 1.3603703970477132], [2.8528563177599153, 3.8850230878956626, 0.73277314683211259, 0.030698406797289718]]], [[[0.56149541164583572, 3.771406387287243, 2.3055239852248577, 2.1929563801988072], [0.26118616465788402, 1.0971842966437451, 4.5034589362870223, 2.5613243344956715], [0.81523332014933036, 0.65319001935591259, 2.9084940610949541, 3.856959052777488]], [[2.3035958574575588, 4.9970881160508984, 1.4003731840018923, 4.8643790422505138], [0.98824027123115643, 1.0572035159115489, 2.1881707495775911, 3.5768375605233045], [0.19157509779453066, 2.2509010746844562, 0.97123119504607913, 3.5875042440803817]]], [[[4.0324386750712629, 0.70847859695172888, 4.2644257644637502, 2.2440834633508593], [0.52428943536330919, 4.9633713241576558, 4.2692667852893518, 1.6738584453135963], [4.8455407501762187, 2.3287442060494969, 1.8463051922136413, 3.3655973707543292]], [[3.3567332324368353, 4.4779817501936563, 1.4720682669952236, 0.68537584699220777], [4.6456858849481435, 0.93617031087822733, 2.600726918404086, 3.9254209169320098], [1.9715119235623306, 3.2153585691442848, 0.96584126755570743, 2.1426248843580384]]]])
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s1=numpy.array(3.17462305311)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[60.022132057638203, 18.220834732329152, 6.272548362335483, 3.6992425858115769], [0.65026152695497086, 71.567694062917809, 7.8647749175174138, 0.0038100122513998725], [164.86314148629816, 50.010366304656479, 8.2996169913184055, 106.15111063722614]], [[103.22681555535591, 5.781547143440477, 0.0634748335584783, 0.0063312274857967744], [0.1764153899683791, 0.43431672351587819, 0.22653516029757045, 48.760082837888653], [138.98823627474809, 5.0414252076870536, 4.8689461743946447, 65.405623715954761]]], [[[0.0025072732579297548, 29.881702136699758, 18.337188449261923, 46.676045762055914], [27.541480355598392, 29.011069069203081, 135.34043626729817, 0.0094467787432179449], [0.86026342790067445, 58.301233501183752, 134.01750501440605, 16.517009879233257]], [[0.00041016736780463437, 0.53936230846610855, 27.359263579281251, 52.119519116467899], [0.039412631444641674, 0.26771309842040925, 122.76694209020394, 159.07742203209867], [162.26854614770295, 0.19927642111709859, 9.1955141796081552, 7.8154833353894597]]], [[[0.28919858264713466, 151.6008967701429, 137.12542477296992, 14.024084430441267], [155.65744502253776, 16.758580918871857, 145.32399447704478, 0.072399161523280242], [0.037462089437994672, 0.012609655038831585, 112.13490245120101, 13.873577518833631]], [[10.859445412812203, 0.15384984432215978, 89.696847068885504, 79.570166558853799], [130.61874273727324, 47.070194795867586, 2.4364140354933532, 1.5366996027408824], [1.7075072915800553, 118.4168884765577, 28.272768212730899, 131.44849842537013]]]])+(1.-msk_ref)*numpy.array([[[[158.84648183249129, 0.0017562796359265245, 17.790392952968386, 2.6426619741220776], [27.105892629733287, 0.15233185387396181, 2.9570461540901354, 33.961101866083148], [2.2691290463102014, 0.2627896445138988, 135.47522693699685, 11.370462716972773]], [[139.50985305932494, 8.8568846403485804, 9.1881912612813554, 77.528797776776699], [0.69696060301208829, 1.0987923865195321, 36.262116766439675, 2.6565080829612926], [27.883164683425992, 74.319456129903131, 0.37267409652835476, 1.574573938116684e-05]]], [[[0.16005480083646204, 67.636444370628709, 14.179368099982021, 12.095997222591468], [0.014094074492948111, 1.3423707899361295, 118.78534278582266, 19.802547249969425], [0.52282162636934137, 0.25871419634800458, 29.646381808454557, 72.628484383970729]], [[14.141756635752596, 165.25823879054542, 2.912519848688933, 151.72356888062208], [0.96314245136695886, 1.193148348987034, 12.012396085483857, 57.167786314525287], [0.0052686217769844336, 13.140103239261345, 0.91149461452618286, 57.710763159309323]]], [[[83.646814659700624, 0.33484498731766293, 99.901003217952209, 13.014171309808928], [0.1287490313386106, 161.74429268978793, 100.26147757551765, 5.1312435766870585], [149.86606592315877, 14.637715081435621, 7.0050816502248097, 47.121996502966439]], [[46.729129075365961, 116.66509534825502, 3.4127772570407302, 0.30139481182433192], [131.10878891465657, 0.81107775631234436, 20.785925741974243, 76.800655013762125], [8.6273416474295086, 40.762628679894341, 0.89553274435240182, 11.236453519003103]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(1.60552275559)+(1.-msk_arg0)*(4.48113996499)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[1.1361181958301001, 3.0712558047232457, 4.0855832076299716, 1.1888440710462549], [0.29584977564606663, 4.9759153102342069, 4.4441886699192228, 3.9937867822505004], [2.1798589491883442, 1.0763870233121027, 0.068297453462055221, 2.9476391422707691]], [[2.3495700601145324, 0.66330973937153448, 2.8176837069420797, 1.7698562987893989], [4.0872866829231596, 2.8147386784043666, 2.5361760454294711, 4.858793018548198], [0.47461728802510239, 2.8302370290447252, 3.1592737325682565, 2.933315198436043]]], [[[4.5430248564742595, 2.2878245188172088, 2.1283500032701972, 0.23434469368400851], [4.9072089789539453, 3.836106989898568, 2.241467950590927, 2.3982652541878409], [0.75123252093714632, 1.2146503586436819, 0.59756540637810063, 0.13255812608716622]], [[1.3171998027049625, 4.1760826430040039, 0.23122073970369292, 3.3047292509747397], [3.763078804964155, 1.0251831165169889, 1.2647414401356962, 0.60844669084567404], [3.1975101498081244, 2.260291382370661, 2.6009174969016891, 1.7499045501606536]]], [[[2.4734644445704284, 4.605234540130505, 0.32381342241035743, 2.8483067479156636], [2.7786723154515887, 2.0787197703798221, 2.1011160077776263, 0.74443313155766722], [4.252994667444379, 4.6545086394721968, 1.6029363727777248, 2.3422456489296528]], [[4.5850536514167572, 0.13568494694598707, 1.2129746076488122, 1.0823004857211485], [4.6718882422071619, 2.0599031737785611, 1.7077372621918132, 3.3849069264407832], [3.7952043746962905, 2.5069634820817495, 3.7579427278650717, 2.0090611148900868]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.7123975887887171, 4.2805617135602452, 6.9193162079112494, 1.7556822390978151], [1.1503542071029413, 10.547028460495834, 8.1997075965483308, 6.6250372043794288], [2.8068221304525407, 1.6646498465325295, 1.0328638583013416, 4.0372267442475005]], [[3.0416573687409816, 1.3689494261032256, 3.7963143299696536, 2.3115942011067037], [6.9248989522255355, 3.791024733870648, 3.3226114239295779, 9.9780998666888188], [1.2519561824564154, 3.8189443864866224, 4.462710327898427, 4.0099402625456886]]], [[[8.5925231907201542, 2.9540265266324748, 2.7392003890604744, 1.1173394368209926], [10.2094645372221, 6.14846577535417, 2.8898993490205203, 3.1125964982014578], [1.4271373607693101, 1.7772646932299034, 1.3269950005668332, 1.0647708016876263]], [[1.865683486710247, 7.2222306173488722, 1.1156880751340319, 4.7808679397447156], [5.9395147157498203, 1.6247798660759607, 1.8199173465363083, 1.3338489704931995], [4.5442345036397054, 2.9157690697448579, 3.4260323730540172, 2.2898613577731792]]], [[[3.2254106546445369, 8.8493637748157816, 1.1656854383869026, 3.8517559819100362], [3.7268402723797376, 2.6755865962359198, 2.7041080858433117, 1.422550554258535], [7.4900674842211545, 9.0582356434190689, 2.1359452242768442, 3.0311279646366245]], [[8.7652140298654544, 1.0663482466452721, 1.7758552002814534, 1.669316939024414], [9.1330775820768739, 2.6518564399232569, 2.2445996950749674, 4.9658385741946871], [6.0305442536665108, 3.2769738059801337, 5.9250893062360364, 2.5887853669443945]]]])+(1.-msk_ref)*numpy.array([[[[5.4960963504725049, 100.13380962014836, 458.46053190991825, 5.9483890686484395], [1.5585227432433335, 1742.8260983443258, 785.04297443984069, 399.49083221493009], [26.2987478246237, 5.0251165406024585, 1.1078684033616748, 83.187536984076814]], [[33.922116255854924, 2.7044088104942063, 68.455254381589356, 14.218836343299268], [459.63339820195011, 68.153541917291392, 44.878251985924805, 1462.0490394832811], [2.0377938517109513, 69.756370936516973, 114.26536365228102, 81.419384224538916]]], [[[910.48791875817437, 30.921645871472283, 24.34348407604503, 1.421181156760988], [1572.1701274655848, 315.35230697630698, 28.844729212946795, 36.492400123336317], [3.0856327241052375, 6.1831429335903207, 2.4504578635901555, 1.2199635080382119]], [[7.211221167128012, 525.11236660047666, 1.4145377201840059, 142.12244144122761], [282.63522804293171, 4.6536369777288868, 6.6655814697895508, 2.4907789269282139], [121.01003009228056, 29.670702800026465, 49.454722613441966, 13.799638695882662]]], [[[40.849459691260414, 999.53236363886469, 1.6252805660811136, 71.672785892085457], [64.564711758636605, 22.597176190225017, 23.369145012115386, 3.0543245827539858], [589.32091588758226, 1076.2011275061918, 11.069661324073465, 33.55149689539914]], [[969.7309607049466, 1.225698389001362, 6.1676216052731645, 5.0698848984804536], [1104.6235965516303, 21.968340787759228, 12.953896684284038, 160.28374306881298], [296.58727849296343, 42.954355255616619, 280.46631624506153, 20.355385144367176]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([4.6046141462871093, 0.24667423188963111])+(1.-msk_arg0)*numpy.array([0.78798480347939248, 2.0456138955706202])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([2.7252996240995273, 1.6105707034139205])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([64.180086331364933, 0.10494756731346012])+(1.-msk_ref)*numpy.array([0.52237226931798453, 3.1666749674702608])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[1.6680567725254649, 4.9622892702440975], [0.13660241159284808, 4.898980186657667]])+(1.-msk_arg0)*numpy.array([[0.084504137716898672, 0.45018522151619594], [4.3130381599639795, 4.7055402309057071]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([0.87473824510801912, 2.6160588912908627])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.5645026733043559, 4.0601279444227831], [0.0054740425990445936, 63.878852787965528]])+(1.-msk_ref)*numpy.array([[0.11515932819648127, 0.49751715042410205], [45.775226048970787, 57.489170045845754]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([2.9292476849680136, 0.43293362276832703])+(1.-msk_arg0)*numpy.array([1.7969733165063142, 2.7550389351698978])
      arg1=Symbol(shape=(2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[1.2461775142640326, 4.2392628657534264], [3.9078836063919886, 2.3346421016545031]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[3.8164620717214648, 95.214213081927568], [0.037946945147315302, 0.14163607922170265]])+(1.-msk_ref)*numpy.array([[2.0758886461667894, 11.996895405103958], [52.476877203028714, 10.654688417347336]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.3280643658022449, 4.6888935515622778, 1.4305510702086406, 3.0662903344369568, 3.993810173754782], [4.5418170122430404, 2.5458311444439707, 3.1596373075182917, 0.50755423879762751, 0.34530773498260509], [1.5372137452355028, 3.7400683302059239, 2.8248854926092317, 0.42044451369619551, 3.9042950263811766], [2.4952613992175054, 4.3824856814666715, 2.9550268886299071, 3.870491613046358, 2.2088668351363889]], [[1.1149216739032479, 1.1777722854428889, 0.59620410238167054, 1.4425428628161241, 1.8669402562207789], [3.7348289382632234, 3.917333495941298, 4.5187980087338691, 1.648790290888938, 3.9344428467859278], [1.3453084654869378, 4.9365291790509289, 1.0278673193318935, 2.7224065928858177, 2.4840082639152277], [0.11988411364485824, 1.940212894740819, 0.19827148490466451, 1.5657154904998321, 2.8318595377065794]]])+(1.-msk_arg0)*numpy.array([[[0.48670271193453413, 1.1566532716984976, 3.7983871575652777, 1.9003382880010458, 4.8997014399934082], [0.988531024342859, 0.42153500927954052, 4.7087998131436528, 2.7960750448800771, 4.9359653427114729], [2.1384431095768015, 0.51266177336692076, 4.0504952707328377, 3.3164109703371896, 4.4635346985558151], [1.1154434461017932, 1.7441792048516687, 3.0410813486773964, 3.7597649233566393, 2.2290785152600874]], [[1.676261225690084, 0.2332486432141955, 2.6161791735612718, 3.4820679314712089, 2.4106101423950643], [2.8427883319885536, 0.96875845575645647, 2.0660224333842798, 1.4697233450482792, 1.4341079420512974], [4.027211265104107, 0.5323027280017375, 3.3888280863263018, 1.4124582678836137, 2.4639118458726341], [2.5521454416224434, 4.118754438745464, 4.3845888757239164, 4.2927827617591987, 4.4270971648297586]]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([4.3190853179583426, 1.5982402872609229])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[38.466497542240191, 791.42578746626771, 4.6949644431911262, 126.39335657329939, 395.77063857496955], [689.65336113834439, 56.599506680529657, 143.87185325600396, 0.053450726589337548, 0.010126832801225298], [6.4050304230984114, 298.06816646648593, 88.697451616267259, 0.023700948518398619, 358.85836774755444], [51.901282985226523, 591.07911579085328, 107.74447113691294, 345.63050476577052, 30.654736233466199]], [[1.1898927997901587, 1.2988921179633774, 0.4375494843609381, 1.7960810938126084, 2.7122615101657876], [8.2153507886650505, 8.8662787421152434, 11.139970319501321, 2.2237324083003727, 8.9282504004600494], [1.6065280419923409, 12.830822895618613, 1.0449085602368224, 4.9563205682801907, 4.2810421352741104], [0.033700763000047172, 2.8843800716051438, 0.075310067126189983, 2.0473774641689966, 5.278605632350664]]])+(1.-msk_ref)*numpy.array([[[0.044592880223928759, 1.8749080453817448, 318.66804786490081, 16.006431733695369, 956.97691477061278], [0.9513990097839734, 0.023967598644351634, 806.0401663820536, 84.855982636709655, 987.94612987485493], [26.651406508613199, 0.055812959749886891, 420.60985333670823, 177.3417525955295, 639.76232922530812], [1.6029871571861045, 11.052298132927479, 121.96614733318074, 304.90749466582406, 31.88476034439277]], [[2.2832419855695565, 0.097638940604344068, 4.6508571422291567, 7.3449036868322066, 4.0806630304657059], [5.3112015058591275, 0.95053708786016855, 3.189051300006291, 1.850472611140229, 1.7793255148525788], [9.2670696447290002, 0.36503607750094713, 7.0330956785989587, 1.7365891616649605, 4.2258211953073168], [4.4702584816388766, 9.6060231420720843, 10.615894498666853, 10.262869804755315, 10.780862381229511]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.18951324810247053, 2.398134909111608])+(1.-msk_arg0)*numpy.array([3.8173287898953103, 4.311988367063881])
      arg1=Symbol(shape=(2, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[0.64423089085053264, 1.7941688802313245, 0.6425233810333546, 1.4687154215106066, 0.51106573825491208], [4.7582338969683731, 2.5855961013469617, 2.5649148572298515, 0.55541431205496528, 3.2444192788942852], [1.2325587036712848, 3.0670263272012628, 1.0220341532642641, 2.5620363456933446, 0.41332363859420401], [1.6695723400787195, 0.45436884076031125, 0.86296165245712264, 0.60139224005909797, 4.3793601423574273]], [[3.1634751530021976, 3.2596681474241254, 0.90981935285499227, 2.4565524574602642, 2.1142313789401368], [3.1001955237497212, 1.227709142707236, 2.7233603872899796, 1.4860084119156234, 3.5038102125450856], [1.18819140014304, 2.71404339935909, 0.87327378064211703, 4.0454264516384271, 4.7834087837932087], [4.4565857792043717, 0.47199354192586446, 4.2684319071999974, 4.8517715553977974, 4.1967691787546286]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.34247833225209479, 0.05057820418973296, 0.34345238574367565, 0.086907669497367473, 0.42739194831072391], [0.00036546024460350142, 0.013560250166417046, 0.014034825393774056, 0.39700015979159869, 0.0045327306358925968], [0.12872069356607077, 0.006088377315986734, 0.18269344902758264, 0.01410218269807992, 0.50284178665224921], [0.062225302288049775, 0.469658223613378, 0.23802942861007856, 0.36777147871808991, 0.00068631508856181452]], [[15.911851970485309, 17.308598742995812, 2.2162378258998081, 8.5739205094127193, 6.3553680819524834], [15.055059234028516, 2.9266737032852368, 10.827613490588838, 3.6685579308924856, 21.429154591507295], [2.8272392263521988, 10.739732558569468, 2.1465136990493301, 34.415236189785595, 65.628241545015058], [49.310491676740433, 1.5111163032939878, 41.827763339608893, 69.672273240117207, 39.286366396898529]]])+(1.-msk_ref)*numpy.array([[[2.3702135937442983, 11.060503873678847, 2.3647984083257643, 7.1521933864550142, 1.9829756575762587], [586.33807255449722, 31.929731772219679, 31.057304392670616, 2.1043481384413987, 77.174168192073196], [5.212581023616087, 60.851587178313729, 3.931679854218058, 30.937780721400141, 1.7396223605827175], [9.3602981603088953, 1.8379488481332584, 3.1771388701391774, 2.2380288962551012, 352.96745482026762]], [[101.80913702098962, 117.1758987151411, 3.7795661049565639, 36.234207759853341, 21.971274816866263], [92.816373166406038, 6.0144991562346641, 53.512566622064107, 8.7727594986564768, 167.41323492570913], [5.6769941024111281, 52.788886116655917, 3.5830039772127562, 369.43792181419167, 1086.2313016962812], [673.74426244689869, 1.9932589335116815, 511.77378457025765, 1200.3575601048503, 460.8879021092298]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[2.8036151519289603, 1.6196966668865218], [2.063176669634093, 1.8186432395263508]], [[2.9033943052901767, 4.8530853208719122], [3.5156407241687568, 1.4401474544405783]], [[0.69326887121894776, 4.4294135889151613], [3.9150695192425617, 4.6830399849409083]], [[3.6697725903811391, 0.28987823652478872], [3.9849780812481104, 3.5261296193440974]], [[3.354445515143643, 2.3008425673325243], [4.7444277905355854, 3.1570464555616891]], [[3.7796279135712658, 2.8760290852897588], [2.4113735276654649, 4.8247628584104003]]], [[[1.3397497174815336, 4.6603077310944858], [2.2390427201805574, 3.9538413654430635]], [[2.9849970448778054, 3.4170672271967204], [4.9629142591708248, 2.5703875076219784]], [[1.0765427544991728, 4.502462766626711], [2.5882813218860083, 3.2270198174765539]], [[0.18948219034263092, 4.0604154732764046], [0.026793895364622955, 0.25915447025891891]], [[2.8091235236653542, 3.1026510834489018], [0.019048927750599, 3.949978261496462]], [[1.5359950871644859, 3.1361158025711995], [0.67287657577358873, 2.0462174296251017]]]])+(1.-msk_arg0)*numpy.array([[[[0.35223722773847133, 2.1746972737279218], [4.3430585015687146, 1.9675094971344662]], [[2.5692582293130557, 2.2674912865611248], [4.1432431929484705, 0.3163932563169069]], [[1.4346203845726677, 4.3809162961661974], [4.2395909103295653, 1.1296460891857552]], [[1.4231817479478379, 4.8732509518385685], [4.1680012539344871, 2.2971092840263472]], [[2.9387551057030956, 0.47863965423102617], [2.0726545876511482, 4.8828200701606281]], [[0.028359479921546388, 1.0959985099625835], [1.3049140998984421, 4.0647043206183859]]], [[[1.3901694293865479, 4.5304480250016166], [4.264839910398039, 2.754479892670012]], [[0.13215596331816573, 1.0128066809468068], [3.728505399896783, 2.1343683906074746]], [[4.9685927389756017, 3.0541506172313735], [0.59909807396095693, 0.34352635467210624]], [[2.1912905166861729, 2.1055708756111895], [4.6610190427372418, 3.5049087613090397]], [[2.9996896590987263, 1.0030646941216004], [3.261183225388625, 2.6738254374991763]], [[4.0139374774930943, 3.0261194707967145], [4.9732111473458627, 3.7307262231203455]]]])
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s1=numpy.array([3.7350822117316134, 4.2876755803153106])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[47.018094643590281, 6.0569228845561529], [14.956135131180488, 9.336375571073269]], [[53.578684153634839, 365.03403125758297], [109.489291459637, 3.9053775251171259]], [[0.2545392560412526, 259.51132796463105], [163.65552854708127, 319.50429780210965]], [[128.52086944123496, 0.0098023700481783063], [174.83977407781452, 110.71438275735183]], [[91.883321111633393, 22.473842664862882], [335.43025847498308, 73.258071055962134]], [[143.48934934735209, 51.716675057338612], [26.778687830563978, 357.1403714693115]]], [[[3.5045826429247895, 734.42074714099897], [31.692311628939358, 362.93163017777209]], [[108.74402048634022, 194.14811559883722], [961.81948004577828, 57.271821128159623]], [[1.3719546219980312, 633.55185335241845], [59.00098062393193, 151.90649203694809]], [[0.00079881765466964007, 406.77360155743008], [1.819398075841448e-07, 0.0030586513701901345]], [[83.815863174932687, 128.34865236369617], [4.2134990100485189e-08, 361.41364921809071]], [[6.2976585195427024, 134.39039986936217], [0.18291227807762614, 21.540726782863864]]]])+(1.-msk_ref)*numpy.array([[[[0.020295157202900028, 18.205905896026543], [241.11226169818895, 12.525744565201077]], [[33.936391439638967, 21.281005613697154], [202.21680389034123, 0.013592757626711947]], [[3.8496883349254283, 249.05648395771127], [220.34669224531166, 1.5766824306952802]], [[3.7362855855925843, 370.7316603539071], [206.7671069800584, 22.337943211236034]], [[56.056861002664945, 0.063797858084481185], [15.214374890305052, 373.45799373589563]], [[1.6622670750235507e-06, 1.4082945134232365], [2.7021408785219494, 188.26657925540388]]], [[[4.1060858841004881, 650.6094938193761], [502.13277468571698, 77.045463973530801]], [[0.00017041396604438694, 1.0560782189034172], [282.19915181285006, 25.81072966438677]], [[966.54693704515432, 119.96455427702071], [0.11116889260389266, 0.010241115987134971]], [[28.894214691899464, 24.350342415282185], [734.90149918720977, 216.46967184507199]], [[111.05765849840053, 1.0132067685749151], [158.92283639906634, 67.827834338872464]], [[387.18180268891234, 115.31437749723601], [970.40498263091149, 282.92056163215761]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank1_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([1.5318382259518091, 3.188946622421799])+(1.-msk_arg0)*numpy.array([3.7707722896715228, 2.1467727093650311])
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[3.8902454306812717, 4.0781000506416731], [0.6993065529102287, 0.50880488578523309]], [[2.1347053532261788, 1.5812153359074212], [3.1287805118340732, 3.0411423912345725]], [[1.497247409797559, 1.515861680427105], [1.4681277652908828, 0.83271781296243885]], [[0.21133885290137788, 4.6289684679491616], [3.4200465222030187, 2.0780351650301849]], [[1.9280956277780645, 4.6922575390233074], [0.35613882730890539, 0.6850398529468904]], [[3.1612912381211222, 4.9795438532084857], [4.1023739371768295, 0.65785889208902726]]], [[[3.6342484664885442, 1.8655652603495607], [3.9938625956998886, 3.6667519712400392]], [[2.4366923970082408, 2.7125138126759807], [1.3102646897930601, 2.2626368404169339]], [[2.413839276651137, 0.35710871119187698], [2.2798857551939591, 3.3939213034863442]], [[2.7711710695673877, 3.4147420056397135], [1.7336302664763703, 1.5884208864521938]], [[0.80585116557078307, 1.3155758152372716], [3.8274276989913978, 4.3276500721340518]], [[4.6674791750555427, 3.8900768058669999], [0.41462736183061805, 4.360257008849918]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[5.2544062655148647, 5.6926796951395975], [1.3474746285036787, 1.2423307354744373]], [[2.4852779986649289, 1.9627341656456088], [3.7974365046643115, 3.6581272638615143]], [[1.8936928483452231, 1.9087855654530002], [1.8703212368310114, 1.4263630614598803]], [[1.0943158320279169, 7.2002021071376454], [4.2996860336066618, 2.4259336863320105]], [[2.2756642565081258, 7.3971877139070816], [1.1640228508971, 1.3393010820899687]], [[3.8504538894639979, 8.3613378476917486], [5.7519166671650757, 1.3238657758204633]]], [[[67.66736197722291, 8.7013535074308468], [102.68284972235986, 70.266698173907116]], [[16.874597580448846, 23.235398335731322], [4.5699474667149582, 13.790208176098231]], [[16.43325193941558, 1.5130623346837215], [14.068836767443356, 51.208155271400763]], [[24.87096797709545, 52.459653826933803], [7.4668618490045633, 6.3096288157288507]], [[2.546037074495946, 4.5981818350392087], [84.65905862276017, 151.21993866209775]], [[224.26580048917683, 91.038791781048332], [1.6174316253484264, 157.04765340076989]]]])+(1.-msk_ref)*numpy.array([[[[174.76536469291784, 224.25413712649541], [2.5298891409170161, 1.9646742652327636]], [[17.002346471807964, 8.1556849598160994], [63.609782515224111, 56.624794897494162]], [[7.2955625270492668, 7.4780542886375647], [7.0189692267007384, 3.0199780367752815]], [[1.3237992168933095, 465.8839132014121], [93.630742298450386, 15.770390524440215]], [[12.924470347432813, 506.70997444795336], [1.6043134095202913, 2.4824341249919311]], [[66.414679109112441, 741.92497647888626], [231.59686041307177, 2.3944722212650893]]], [[[16.06175512967512, 4.1588050061762036], [21.140144798947212, 16.465586717748348]], [[6.4336967063195756, 7.942812052361174], [2.7209938894550394, 5.6326319649239078]], [[6.32234566970887, 1.3136621723116013], [5.7073476185885212, 13.367672487769097]], [[8.3068419342199675, 13.582002795400889], [3.7600564540184931, 3.365239025474918]], [[1.8508455869730047, 2.732056798293129], [18.616034626962382, 27.280604614162169]], [[35.367515484501659, 19.528697524608553], [1.3726745480428062, 27.968715321577559]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.7596476374131319, 3.3801524498308724, 3.9197380751568431, 0.40610327446856059, 0.84601224376381812], [4.3263454662238008, 0.34159777184957107, 4.6178812962488553, 3.4089334665444873, 3.6087319026629716], [1.6855854257312737, 1.3674524542038562, 2.6047490472571471, 2.4883020920655072, 2.484082080041857], [2.9793749396785709, 4.2566550371346388, 0.28694610987415281, 3.451994097104174, 1.5982799284869009]])+(1.-msk_arg0)*numpy.array([[3.3195839876105993, 0.64966424506771325, 2.765215218364808, 2.3010988484605166, 2.4785622446660702], [2.5457015397443024, 0.35652857596273113, 2.2945417382935176, 0.67972536567758346, 4.9254895249677046], [1.1415010952469777, 1.9379473486647254, 3.8829088357850967, 3.9692681147723747, 4.9182211517983374], [1.9585960970802885, 1.5460060413917556, 4.7517903439300069, 2.7387068050815038, 3.9585409596556218]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[2.6276184900669164, 3.8566890961631204, 3.3200306222151426, 0.13183677384887113, 3.6778329212651841], [1.7437130398758796, 3.7037387497544763, 1.0174228482144398, 3.5660617117950544, 1.6149925533214036], [1.9729972996868259, 1.5854549272299534, 1.2136241065579925, 2.4338226746678235, 0.59169641776760373], [0.14489957606738479, 4.5360546268967861, 0.57958386049091093, 2.4297370453654614, 3.9951797950762775]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.485617564839539, 109.63338278005978, 93.246290690309252, 0.88798145854641342, 0.54063411424354491], [12.859166625153334, 0.018718090280399228, 4.7426300696145987, 79.314123474343091, 7.94554257652646], [2.8014227554989404, 1.6424143190154088, 3.1958287852453977, 9.1951316842088033, 1.7132402629862058], [1.1713873636337073, 713.65593281517511, 0.48500903474214802, 20.294049712183373, 6.5107307220718438]])+(1.-msk_ref)*numpy.array([[23.399633897921589, 0.1894957944612855, 29.278822179957334, 1.1161341055454872, 28.170925298248015], [5.1004888639387786, 0.021931809633678068, 2.3279857255127077, 0.2524008610083136, 13.131085350913589], [1.2983765074718632, 2.8547478069585464, 5.188184040058947, 28.652005998990653, 2.5664945530886971], [1.1023073165191881, 7.2155971585914438, 2.4677181194018569, 11.564333557639953, 243.92725593982325]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[4.7920175006120163, 1.6085019314856679], [3.412856890770831, 3.3985729374076068], [4.1273996297468205, 3.0905151475346191], [3.7564124659708704, 0.75170649926119448], [0.2570826292493551, 3.671108433152011]], [[4.938584129065946, 4.4835334121067838], [0.87268492254472718, 4.1642279358753695], [2.6426549154606302, 3.4316270483828104], [3.4384503989896484, 2.1504642635854774], [3.1200624220017716, 0.44776091354039088]], [[1.1838504039077244, 0.44217744924630215], [3.0714430812533324, 0.65757229231169945], [2.501797631903635, 3.3859505457709314], [4.8040089960231889, 1.2232903304790628], [0.48414689742909911, 4.2804279472003319]], [[1.4217991749400511, 0.61389066600831266], [1.861791847800071, 0.5518861939565356], [0.46716380119939971, 2.7211536650792856], [2.0536683976731318, 4.6474512180929217], [3.5157620749766725, 0.28140012752676985]]])+(1.-msk_arg0)*numpy.array([[[0.85368046227601713, 3.0834998141889547], [1.668184939095241, 3.1172026694716246], [4.4021591116465073, 0.36171186650966736], [4.9927024687763586, 1.2328716888918534], [2.2955152664316674, 1.4258001225552313]], [[0.10289262712453565, 3.8579989866265785], [0.1068634477744513, 4.0266050921127885], [1.2348213182049281, 2.4790754731314144], [4.1461821358792816, 4.0221121728281579], [2.1503716684957856, 1.4582412058034269]], [[4.5967516208049881, 1.3676734069392611], [1.8613508651394586, 0.11683690443369088], [2.0269075129205718, 2.2641656702448287], [0.30735256100401193, 2.8350272744623251], [0.88109943518505696, 1.4495613180372258]], [[0.71248527749456692, 2.6043462857479165], [2.6570920025419285, 1.9349749753731738], [0.49872361587399106, 0.083832144470343428], [1.9339140764003928, 2.6621369642893775], [3.556886338994826, 3.1341870430100913]]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[3.8490561363169706, 1.0941580553944328, 2.1844918736482377, 3.5972546321550225, 2.7011604778893847], [2.4219553535111209, 3.7729068123176774, 2.645233486432184, 2.8286464942179412, 3.5286659902975748], [1.5505615959782353, 2.9891818399345391, 0.22362176897741487, 3.2009164072410736, 4.1447913208056377], [4.140936232943325, 3.4092723396778193, 3.5489906776366285, 1.5563484549037452, 4.7964613546361576]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[416.24929011430015, 6.23057710798256], [3.8310289968323596, 3.8134885700458856], [22.127907975523726, 11.761655849672255], [116.84383941266741, 0.35819060590913482], [0.025498332572249965, 33.543463878510011]], [[47.849145061905944, 37.86130174620893], [0.59821976212469308, 217.49331375609884], [13.073610612476948, 26.092924024664569], [32.898845032441777, 8.7220008801783369], [55.429016139026849, 0.05870276031252894]], [[1.2991259100571422, 0.2821471599549098], [28.625643993362445, 0.28562750897982708], [1.227602760462682, 1.3135545561470698], [151.96963663365864, 1.9062275846847492], [0.049464775040771347, 414.36458330948142]], [[4.2943108651224486, 0.13258624425332244], [8.3227857670995817, 0.13179372792081037], [0.067135034073856903, 34.908772907028343], [3.0648278178019068, 10.924933074049013], [415.87356993141856, 0.0022840480585571954]]])+(1.-msk_ref)*numpy.array([[[0.54394114961534057, 76.270663872590191], [1.7505328992363394, 3.4694209403226135], [25.473145066047628, 0.10845436374754279], [325.15563186188166, 2.1235122178399024], [9.4361599539591374, 2.6069765847980539]], [[0.0040554472962007794, 26.311332904887344], [0.00021670204231961606, 191.59255060673428], [1.7470859939540939, 11.040492485984611], [55.861026723993078, 51.261014014309403], [14.904903990883627, 3.7852896046097659]], [[10.645624381862609, 1.6249840074806821], [6.4056866649582735, 0.0016324022854085414], [1.1711559974508683, 1.2005083906270773], [0.022907006685385648, 28.092985768736892], [0.59175205847742052, 4.658992634445406]], [[0.24567126831809949, 52.648006451513126], [27.984506696980439, 9.4919261496613068], [0.0846657424864736, 0.0001510755450633561], [2.7912327620019757, 4.5899364164206489], [439.72980019432663, 239.68671433253499]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.3508759483212587, 2.5429734388790446, 3.4479995383899711, 0.54881779682369114, 0.37937012480892734], [1.5383195752276198, 2.64803522526788, 4.2291165159048552, 3.3622118202726949, 0.059241164886007745], [0.21913019784177531, 2.2010641680817966, 2.6894548331040968, 4.4145579876705776, 1.5722253158477602], [0.32417862174292583, 4.8730335690178839, 2.8109444642777826, 3.6972091208341435, 2.590029021549094]])+(1.-msk_arg0)*numpy.array([[3.6334553030947618, 1.3417648566540437, 3.1469678054525878, 0.47881105768041787, 2.3983979260918424], [2.425581303297871, 4.3121281724711276, 4.4574817635944406, 4.1330002146578364, 3.9689365543740949], [2.3638690957184085, 0.19488952798860595, 0.86129494146208585, 4.4051470445234582, 3.5607287797325227], [4.1870837294633878, 2.731820776541912, 3.497709505474166, 0.45958804538041398, 3.9131432942741093]])
      arg1=Symbol(shape=(4, 5, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[3.8893495711247419, 1.262314654314229], [1.0263670067451838, 2.4168190433735814], [1.9425239771132263, 0.49217444020247353], [3.6287230680064289, 3.1537769923277361], [2.0410693306646408, 3.0547296178918337]], [[1.3200447009369527, 0.39853879361466904], [4.8931341158433455, 4.3613049225156519], [4.82014192419037, 3.640064162318895], [4.8770112803931722, 0.40014597494837173], [1.1062279464285678, 1.9461381493066936]], [[3.5975697612223487, 4.7011227312781534], [1.7899203383580964, 3.0782496576410505], [3.1459368351235306, 2.9033988897821001], [4.7365448639247347, 3.5700174764230157], [4.4123663674146414, 1.3982610258882568]], [[4.3352003581299918, 4.746415810097508], [3.0445271424071061, 0.30990565907764978], [1.1051309040366075, 1.0154129493525652], [1.6002256946823119, 4.0963168956490827], [4.9145684598348662, 2.5708701401429788]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[110.28702058058356, 4.6016658731768798], [2.6063304300968766, 9.541965443173023], [11.072282894502127, 1.8389792767246098], [0.11335927190162395, 0.15073517458106619], [0.13830524785116069, 0.051778762673368013]], [[1.7656711824494207, 1.1872585695289493], [117.3336736875311, 69.903587964393878], [1043.7877495301125, 190.36579176795985], [370.13180918783928, 1.6245268822106838], [0.043877503960966582, 0.0040865417881465705]], [[0.0042474696272982448, 0.00079535363578534662], [4.1047361403992255, 11.342505622435008], [22.474839908573216, 17.680181589772719], [1133.8021536860479, 200.56662509541206], [7.3636854358990504, 1.8826906459261252]], [[0.007570982322654473, 0.0047640817627386742], [124.17207866583583, 1.6336270928157968], [3.1335780021735107, 2.8560802271484422], [8.1045068066997867, 211.92974975342418], [107.45185095009992, 11.549216779059275]]])+(1.-msk_ref)*numpy.array([[[151.10528689045498, 5.0968273498986614], [1.3522059965158348, 2.0350298580732575], [9.2718784732723538, 1.7581254635339205], [0.069088275267822408, 0.098018741612496685], [5.9627361314287173, 14.472936751602125]], [[3.2208767440403578, 1.4235215273287407], [1275.3559731661035, 586.25136422602816], [1344.9429827969402, 230.5301327413886], [1012.8227392414784, 1.7643962677926659], [4.5948367162687305, 14.625226982344408]], [[22.086979510247655, 57.075479178222366], [0.053552284022476998, 0.0065131593678913659], [0.62516112297010951, 0.64821651019121362], [1122.3992599193427, 199.04437877146407], [271.38864310844997, 5.9046660576572068]], [[496.71993354561283, 895.05818247118873], [21.320165934146861, 1.3653971997551038], [3.9898099877638185, 3.5658662420203577], [0.28821312606762561, 0.041395706343518565], [816.59810910824422, 33.366201008096944]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[1.4442626183096905, 3.46012506723345, 3.376428902219784, 3.122737102138573, 0.43846948313366485], [4.6077061847255418, 1.4932889184182963, 1.2852407005463558, 4.8460005664567056, 3.0945373821074895], [3.0565758820825812, 3.6078390411020256, 2.859794776641452, 0.3189762846500207, 1.5900827620083746], [4.3403818327786752, 3.8043721423773125, 3.4084359290135224, 3.5268913046272492, 2.8981811506818711]], [[2.2725267301295, 3.147902367773499, 1.8667909449264808, 0.20160360141911834, 2.0020907467257389], [1.6529541732455566, 0.52098137524124077, 2.7038842114797235, 4.22787974954164, 4.6731373738668198], [4.4927521098392598, 4.1784210642765762, 3.2028614925115289, 0.86547143909412227, 2.2623143904781005], [2.9650621801889572, 1.633609885354844, 2.5737374909948536, 2.8124284136278463, 2.5770968819895606]], [[1.1034626061375687, 1.8228924385999441, 2.9948258152971041, 4.7995150897569756, 3.7386215212511482], [1.7573028680465288, 3.2384337087456112, 3.1696668034501703, 4.496626657825157, 3.1115115330209249], [1.8569836023005875, 1.912100489210907, 3.9878321527753822, 1.0864314364916916, 4.3655880244978365], [0.80623307330632543, 3.3703354593966233, 0.18529533990931987, 2.6512399789926056, 3.3107890953866601]], [[4.0985014630049106, 4.5926014968285855, 2.6116462077581635, 0.23047387319425908, 0.12475546469152643], [2.846521821211101, 2.4069781326622564, 0.45926598829205345, 4.6301760510635654, 0.16097394604039103], [0.074416121153824497, 0.38274328225558035, 0.1795267647687071, 3.7974140598835593, 2.9379420273284342], [3.8489304530158459, 1.8157277429071712, 3.9402796272810026, 3.5178558545169487, 1.8775396487155225]], [[4.0547440318510537, 2.5065317101959694, 2.0936349398390921, 4.3544026829079767, 0.64031710748520598], [0.55565430440107677, 3.2687677840032503, 0.79486109977610542, 0.93141462631740268, 1.3687911857312696], [0.34927507870776303, 2.7173163055314182, 4.9732029335245915, 2.6382971883491551, 4.6027800595158839], [4.6869636488477662, 3.4396382449274339, 0.4785379087322002, 1.9670995421773863, 3.7453906192943984]]], [[[1.7819917902957951, 3.5075815112743847, 0.86047786332233211, 3.4828058622986888, 0.32127558959586106], [4.6231262024867537, 3.4996665140107055, 3.7498294464799784, 2.4223022614254481, 4.1326473223791442], [4.9664575595283766, 1.4036512419911369, 4.8661357468388289, 0.31481558197878445, 1.2455404693862728], [1.9495087253720758, 4.8248106751649456, 2.2962893206156654, 2.5025130711901955, 4.5307512060878068]], [[4.5386259126855908, 2.5734992055131336, 4.4830829145637265, 1.6011119576830768, 3.4366353481997476], [4.7144597059128444, 3.6975657210934174, 3.7079059440698683, 1.7324493093489, 3.8270486089767459], [3.4592798648804912, 2.3403417633886923, 0.82712269488712986, 0.93337008608617589, 4.5488328153200124], [2.5926317777002135, 0.57722165752476784, 4.0213215099120312, 3.0248666226974201, 2.2148481014715728]], [[1.8331450792434227, 2.1885203638291495, 4.9915104767563161, 4.766695198511063, 1.0092138260548262], [1.7208855348157743, 0.34424829233630239, 2.3441560506125172, 2.7450356662874635, 4.6290009689576612], [4.6170374858489023, 4.9623765416253747, 2.6749429455774796, 1.9312253065583882, 2.7783701883165062], [0.16820182092187619, 0.19521080156823511, 3.5695600991327763, 3.9687622682163357, 0.51844029591840135]], [[1.6191930625019328, 4.6595156256770087, 0.17521458999822662, 3.5503832740003229, 1.6571373970891863], [4.3976725200626081, 2.61238955584072, 1.0649058389102599, 3.356516274729656, 2.8753551577327179], [1.817675573725033, 1.1081803802264845, 3.8887788054068837, 1.6440963035324101, 1.9394308238383737], [2.1873290153428422, 0.81821800221596752, 2.9234906043657598, 1.9772456749422882, 3.458109788766381]], [[4.7235699071140287, 3.4891003145843213, 2.5125439623876997, 4.2401427647378513, 4.3946196027529094], [0.52929142082653735, 0.81223477974365654, 1.5968353509962745, 4.252523198309226, 4.9306705858770608], [0.74017139680230937, 0.52126162665220188, 1.0415070232359267, 2.4334941025794881, 0.083252922048513658], [4.1094698509060592, 1.4714265577787697, 3.9626797500873612, 1.9560739021303468, 0.57072314319845963]]], [[[3.0821016329857835, 3.7991205020685483, 2.7909010911094416, 4.6369763721786832, 3.465682636760874], [0.33624138850940499, 1.5360370393479714, 3.6850149106547487, 0.73683601714018576, 2.5319140787084216], [1.775412331048269, 1.6454430145299619, 0.34930507594317112, 3.7584304906105821, 4.9278655434577328], [3.3646664738592191, 0.70991347878143363, 1.2941400835047483, 3.878150751517067, 1.0976105385221611]], [[2.7639964151047196, 0.27657366126164684, 4.658584761850447, 3.8015306761700782, 4.3442407346868865], [0.26354380267021121, 0.33209535676671154, 4.2546134592867277, 4.2702072329830303, 0.86491439779293522], [3.7130902149328695, 2.3451254685982788, 3.7489695823125624, 4.2117814687396535, 3.1980225232745716], [4.0457387676388725, 4.1844007180435163, 2.992708006905981, 1.3152357033104982, 3.693032358445806]], [[3.9228662494282536, 3.6609919995553803, 0.94523440783635937, 2.6570931108093316, 2.7157375490787734], [2.0503745741831967, 4.6489226078357637, 2.2532243878124349, 2.118988865128828, 4.7791824823182507], [2.9720823356870874, 3.2271039778960025, 2.1452571444778328, 4.1717032632767754, 4.1528683981397867], [1.0895602236693487, 0.42097051450766054, 0.46792543783938062, 1.0219268664822463, 1.1008564694591072]], [[1.4703970717090129, 3.2296798044438764, 1.9165524669900618, 2.5965159920327876, 2.7768224503903327], [0.76579707347704118, 1.9408112809305611, 4.7571242825112989, 3.5654239048915581, 0.46714476250524289], [1.1163710927772039, 3.9134456322154798, 0.77256774001692652, 3.5114226519733927, 3.6339268581562294], [1.217713035643561, 3.6815658356799168, 4.5024038038381473, 4.4087748439349204, 0.78943145327687891]], [[2.0413490682721025, 0.25741758378191376, 1.6315407069662049, 1.1953211668786505, 2.7124226150051025], [4.4035934433629631, 0.21762386590040564, 1.8635278858356408, 3.4920891446031495, 3.4450140528264055], [1.7418745092844523, 4.8658284741224316, 3.1796433918430997, 4.6582766729031322, 4.9227377524488514], [3.142824408471637, 3.0035068646520569, 1.6229932558978415, 1.0453749108152905, 4.4171743265715433]]], [[[2.0598070880743098, 2.9064177896420915, 4.2655081630920737, 4.9295474480418422, 2.8751380105268285], [4.0552060744581908, 4.9220269240825694, 3.2132780306104873, 2.297288616396024, 4.1094795299700229], [3.5479104680632032, 0.037603332645456698, 0.22952314718097619, 3.8491439237094185, 3.8689408051252259], [3.6096087556760486, 2.9405867125356377, 4.4705553742519024, 4.1186971597922195, 0.047400328975945809]], [[4.2217206814305914, 0.25751089484540857, 2.9477563570024619, 3.4697001579952316, 1.2709097600504873], [4.5652611953608204, 3.4576600607793511, 4.4469860751079171, 0.96170503725998924, 2.4677817151387154], [3.4700576191842369, 0.89377106136141971, 4.6628367310261121, 0.21911666362283944, 4.9929708982256216], [4.4125172490837645, 2.1433323616952715, 2.4021262586733516, 2.7374265250177316, 1.5180124235526076]], [[4.7560525792621915, 3.6498899320553355, 2.2860496283265732, 4.3788401469548459, 0.28698038615824617], [3.1204833609916305, 2.8863865143921763, 1.3386803396814566, 4.6767355212489168, 1.9640949272887009], [1.5136821574086066, 2.408947475435034, 1.5875269530251011, 3.9115029034278757, 3.9922621086694883], [3.9801135699560484, 0.922569831982338, 0.36278140001547421, 4.8385375805441404, 3.6247897626838004]], [[2.2468707186036245, 1.9570514161877082, 1.0728926462079846, 2.0673914035259942, 2.7715527226434045], [3.20601602587399, 2.5768621292867455, 4.7102767648319759, 2.4688110169988273, 3.7092740902455601], [4.6330810074669575, 1.8363462331243199, 1.2536935009826689, 0.99761762158557366, 1.720238993176709], [4.679028150663278, 4.1839560207651578, 0.015196163423849733, 2.8488844470496564, 3.9508551199826716]], [[4.9033473272397776, 0.23292273013197193, 3.1179333987585007, 3.7447357637178635, 1.5745238948298788], [2.7987614193131862, 2.9838514875538076, 2.072189103682132, 3.2086575850825105, 3.0107643904653378], [4.4001821863162149, 1.978514614308106, 2.3081369234714542, 1.4927221429973172, 3.4739537135067784], [2.1280927676633228, 0.44090616363490642, 0.077448245651711656, 4.6826870521537138, 1.4316831447680112]]]])+(1.-msk_arg0)*numpy.array([[[[4.9636719224708674, 1.6498986963329725, 1.4880580295564476, 1.7258071070589054, 3.8085543083267863], [0.056750655583862213, 2.7683704687053301, 0.98311119695092686, 3.8278821731990877, 2.7598699606652439], [4.3296790604587043, 3.2990904701071084, 4.9073339522892176, 1.8685330192645606, 1.4626061928522354], [0.25698699193540225, 0.67309478884399077, 1.1058328409148876, 1.0039539720959088, 3.2116612133068858]], [[4.2205857353245912, 3.006234891037487, 1.7942940176175284, 1.7635618798501427, 3.5448358687473394], [3.1726838292166271, 4.1591913674240351, 0.86595817388343899, 0.56290224460021809, 0.77527775735421889], [4.2010940530306264, 2.5575657599790484, 3.6404291661979018, 1.7610460372139316, 2.0175500230550725], [0.60942533274684696, 2.63590581436616, 0.33259572626509837, 1.5824837657226793, 2.5364920037180743]], [[3.2433706307572789, 3.5631135575653445, 1.8786841720932852, 0.67953596247218573, 1.3996204480287209], [3.4111878809022627, 4.4703892327845978, 3.3705465233937772, 3.1763280380227323, 1.2992429692692526], [1.5309047670890537, 3.3234333329771939, 0.47411337444491825, 1.7577033839343499, 0.47622014708220978], [3.2045372834791048, 3.3646770839161277, 1.7367696839797295, 2.1913349740978849, 3.0540541637127485]], [[0.52684552449066413, 3.5537635829898995, 3.228278856619998, 1.8166639558409787, 1.122071113245845], [2.0359713181356489, 2.2146883729636824, 4.175732345324108, 2.6595348211345775, 3.2619792588621976], [0.47435141009254156, 1.0763828427298825, 2.4247349383330441, 0.38851198229862416, 4.601494727891752], [4.2240574606800596, 1.4417007985976844, 2.7079506602594683, 2.8092307796974656, 4.6564102342395]], [[3.4090015041272048, 4.5834752679033093, 3.523522885062949, 1.6647562779256113, 4.7167520821986058], [2.6396428666823231, 1.182930527108716, 0.35388079995465244, 1.0218934359157172, 2.8699089688727408], [1.2829491829174648, 3.9411603529213957, 0.30159523772704616, 3.1733543029625291, 0.61577912137731783], [1.0756880491772145, 4.9222735315676864, 2.791545413147329, 2.5858612303294017, 4.0357126693019181]]], [[[0.29736451580245271, 3.1581472222359013, 2.6877259261821305, 2.5698432868957002, 1.0009301092395322], [2.015423130222691, 2.4760786571535629, 3.7462714827034462, 3.7459754675988117, 3.4290383467021335], [0.44996140347133201, 2.8567207662926188, 3.3182684523698676, 2.6472462387832141, 4.6820708539959384], [0.71215752887847272, 0.88406306286791536, 1.9835938850670145, 2.4533451538180011, 1.1662962345000065]], [[2.9203203602955727, 3.4405663003716582, 2.5374062235393113, 1.1686428021823982, 2.9109791507353959], [1.5968525763362853, 1.9520314010131172, 3.2453772857335337, 4.186243407047356, 1.2653370480800552], [4.1369201809638598, 2.7490149667188253, 2.9223944164497713, 1.2725096026608724, 2.0470644739631032], [4.6928156014642708, 3.5027442003400537, 1.8568624350815057, 0.1699689466609601, 2.5229998663469551]], [[0.33692644286516049, 1.9215687859552908, 3.0718523208884396, 4.6076851748546437, 3.1079965504321176], [3.6264863568803598, 1.6214576392962472, 2.7846422417374144, 2.8463665396949658, 2.9093951111594936], [1.6686083780416441, 1.9617183330997523, 4.7761775745857449, 2.0963225504326477, 2.5991305897124404], [0.19036226719154972, 1.8882855931280038, 2.837555303688521, 4.9874956645419166, 1.7304355386919303]], [[2.187546156831591, 4.9293053413003305, 2.9693412431562338, 1.4413230346027668, 0.36398769281249282], [0.35405461052810572, 1.8005723402268485, 1.7419341843138261, 3.9776856312646536, 2.9342257835319558], [4.025457709342442, 0.19910878143760444, 4.7013015608232633, 4.7126589441946365, 4.7149329104683], [4.2243771528390726, 2.8233723277325877, 4.6167202361152935, 0.027825809078640108, 4.7110194812641204]], [[2.5470284391968625, 3.6822631652567148, 2.7546851540011312, 3.5754564732840604, 4.3253771029493562], [3.4308708525353744, 3.1985971238620157, 1.3764814851102973, 2.696959618032027, 1.12899181862795], [3.5554385223771336, 4.5879768801100704, 3.5713903354312468, 1.9187622294010194, 4.5302137204354249], [0.3295000906653705, 1.2833755475446076, 4.6956559042343882, 4.9082667035389678, 0.87659656668629815]]], [[[1.5151725348616312, 4.4620179890988458, 3.0668905044042143, 4.583984390587835, 1.5755477170445344], [2.8998772637699517, 2.2187922412362773, 3.4487713763951544, 2.7517744044208263, 4.1072048591422137], [4.6543609819363132, 2.9851077254728211, 0.91138337421849902, 1.7188514982860348, 4.248971942646425], [1.3130408449211963, 4.0389698860352912, 1.1033532429128208, 2.925290908553241, 1.8176181064273977]], [[3.9039310183658253, 1.4227803679795552, 4.8380855403811927, 1.6981808459185639, 3.4534460256318567], [0.22562167195859459, 1.1358583552895205, 2.8213820379694852, 3.9162473928310773, 4.3077815078889383], [3.5668560240255851, 1.3520510806383161, 1.3456534175466666, 0.09231350670980816, 2.7367664848564517], [3.3726109191712972, 2.6244846096483894, 0.98225825254257404, 4.4984449766625421, 3.016669609557574]], [[0.43326123249404541, 1.0976159723897116, 2.4202732300895042, 0.66017348868116466, 4.0501181872870253], [0.95198702132713175, 3.4015964899016633, 4.7598121798699555, 4.5879103851489891, 3.3314705154550492], [4.5952640867341819, 1.4527333661925663, 0.39477322417346455, 1.8592425644820223, 3.8200298261065679], [3.429251884033341, 2.987602021626746, 0.76176870670185404, 4.1931305208089436, 4.8882039293718034]], [[3.1702179250130129, 0.44085856882889257, 3.5060128786611173, 1.1794303273619426, 3.8087311331098768], [1.0165615021845413, 2.0853493506316712, 1.4865790035647031, 0.49473877042203651, 0.89947745507116172], [1.5797953163486611, 4.1054362039765406, 1.5078058537009054, 3.9989185098255646, 4.0565845886026448], [1.6957583143219113, 2.036118264154692, 0.80231902758039442, 0.22002971482583525, 4.9569206034266928]], [[1.368606533946982, 0.61906879082639099, 0.23277609838141045, 2.9372405070621359, 2.7326675080082694], [1.8434661726047745, 4.7676803592251336, 4.4955104705304834, 3.8964599013181513, 0.56934742843339914], [4.846761495643209, 1.260107613448491, 4.264783355402475, 1.7006749462739856, 3.331782287391738], [0.56696579009795034, 4.173858385123391, 3.9558906165257932, 4.2424123380164813, 4.9343038793885956]]], [[[1.9357980024216477, 4.4578539919977809, 2.5393303790235775, 2.1369976333457643, 0.31750023537196859], [1.1270122542725376, 0.74152896406965019, 0.95061430890075682, 0.34808295401682537, 1.2616817769176205], [2.7937016372214281, 2.799812549967835, 2.4293235548580272, 4.3065744664182173, 2.8158815115424884], [4.5216127563646884, 2.1865838050083899, 4.3786346446565485, 0.85995090135093677, 1.3060141194518451]], [[1.9577173724734485, 3.8961908526137026, 3.5501875747529286, 2.7026793678226122, 4.0183138719590934], [1.4330027850837539, 1.5589525730772686, 2.5806455526086065, 0.40128469797241284, 2.3711358230421595], [3.9693236816426603, 0.36190512492753707, 4.7467858180500277, 2.6418695356676594, 4.6761644473744202], [1.0099519966440333, 2.1786580184212454, 3.1709670976138673, 0.78690093195037147, 2.8896721608058011]], [[0.91716546741437754, 0.6324580932475653, 3.4447473104949911, 0.51238114995049544, 0.32531068852319023], [1.7941319853799806, 4.9577705686704219, 4.6813223693743975, 2.8355721706883945, 4.8147235255635135], [3.2616376037817756, 2.7636164470920508, 0.12504259315113275, 1.3939307173172608, 1.3634390783697237], [1.3638895680289838, 4.5743939901488222, 3.2804383462088804, 1.6562951413586415, 1.0408281135939441]], [[3.0675653038574016, 0.97927973938338331, 1.4096210263713127, 4.1497768189992765, 3.4760811448440574], [1.3641406570668233, 2.6599303807384698, 0.77665931801713228, 4.3874820215659645, 3.0943050951679418], [1.5435852438053943, 0.20019817205455503, 0.4892504866374861, 3.9723870460664013, 0.76029607480755412], [1.377926039143677, 1.4024715640187702, 2.845692893687461, 3.7173301835719554, 0.63070747020975948]], [[0.085659627559389148, 3.7904033158712163, 3.7487753262682069, 0.10987439583243662, 3.0536017625489893], [0.36691572494491137, 2.9822946094023499, 1.085464738482423, 4.9401925529659341, 2.8430269235795183], [3.3398247781262689, 4.4750765528609584, 2.5593653253850075, 2.9150283391491638, 3.4837412074287797], [2.7562567980706523, 1.5007228991799144, 2.9867976797740732, 1.2660713094756244, 0.52171407012548832]]]])
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[1.5148435611441973, 3.9432482467952688, 1.0620242919532705, 1.7902479233458746, 0.4744941827512304], [3.5754476679106255, 0.94029753801066718, 3.6312380614993374, 4.7729497711139315, 4.9696111596079442], [2.4723378349866643, 1.750717088851848, 1.1388159565492231, 1.4199308459105053, 2.9237707680705376], [3.9009989089293096, 1.2990948605449659, 4.6190790186375352, 4.3019539943611225, 4.5811498515933433]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[1.7451749621899246, 6.556011960611035, 6.3172869834713357, 5.6123377854415661, 0.28680991703894165], [10.117556630968362, 1.8356956472684296, 1.4624951109052851, 10.920655876382789, 5.5357413205485839], [5.4331962350493264, 6.9846116331871269, 4.9121988309437663, 0.17712162997774189, 2.0189212548566067], [9.2417686043403098, 7.5689904174094726, 6.4082244799869068, 6.7485949210377676, 5.0124249125994806]], [[25.45678121554382, 92.007120215640015, 11.721888528384087, 0.0018091062987190254, 15.446329743896268], [7.2553199558309869, 0.076446855818448678, 50.516851846490368, 294.41297732660649, 436.95099510133389], [374.12661219262367, 281.06403551876792, 98.505948104092369, 0.56568150437694331, 25.008654700633954], [72.668546804481835, 6.9262293500725622, 41.58691976392285, 58.998333772548243, 41.801377292233589]], [[1.1102214954295919, 1.8920585107621601, 3.2056653082530389, 5.289908322099615, 4.0572664245358707], [1.8198393688076087, 3.4832783393720108, 3.4047766345959727, 4.9360741404500992, 3.3384710768589234], [1.9296595757587132, 1.9905414027768549, 4.3450737245479329, 1.0920319435290966, 4.7834467950829556], [0.79553427451085734, 3.6341402344247817, 0.16689922152124986, 2.8165221232797095, 3.5659881878329167]], [[12.49541841109694, 15.319667013896687, 5.5767353938405702, 0.072266134108675384, 0.024083662751521479], [6.5063230264779479, 4.8186980678164293, 0.24831976385566143, 15.544778921864511, 0.038009822610980036], [0.0095500191452002649, 0.17918479562486228, 0.046206849414131036, 10.900015268952474, 6.8851499957611129], [11.166159669455903, 2.9091427354523827, 11.645041817015398, 9.5054597425927145, 3.0888182061058349]], [[1.9430103640660641, 1.5465284490189257, 1.4199257580841644, 2.0098696797928488, 0.80934856982483117], [0.75667834729922423, 1.7541722985473407, 0.89678590755895771, 0.96684879077621144, 1.1606230950128751], [0.60706568316972687, 1.6069301413785833, 2.1406704568283219, 1.5845854055742512, 2.0634751296007821], [2.0812974611152977, 1.7970997672664861, 0.7048913299021673, 1.3785384623820449, 1.8712036093733642]]], [[[7.8904213634488718, 88.847818337369404, 0.58433946715160578, 86.624299000390877, 0.017253099506899721], [238.4750634980638, 88.133062221415599, 112.81013870912321, 23.647667100734925, 159.69532295447704], [308.09074509789349, 3.3613800197855395, 286.41200439663982, 0.016044503485571151, 2.1925372331574242], [10.879584771489519, 277.8100548340949, 19.535824116762875, 26.568914297299774, 221.87188035316848]], [[4.1467168838950785, 2.4322865748903721, 4.098982213364752, 1.5567440775335202, 3.1924563337378062], [4.2976038902152238, 3.4198725744355452, 3.428864506256057, 1.6765322601662493, 3.5323649263129315], [3.2122321598214154, 2.2244999500698679, 0.83654867370200692, 0.93722040438659993, 4.1554850775800061], [2.4492859604167161, 0.59647335880364416, 3.7007222544950817, 2.8314368790459019, 2.1121563488361521]], [[9.0308862798438128, 17.185721765361809, 343.12042166465784, 290.24769037627038, 1.0338651998346664], [7.1791260326217099, 0.02080995941231913, 22.055105075281517, 39.12660436744239, 260.94123313454878], [258.50066637504023, 335.90384307167966, 35.6189211588418, 10.912580602872652, 40.879684950634513], [0.0015445676930073405, 0.0026524704635945088, 101.54897078395348, 149.23102804277318, 0.092045644166678584]], [[9.9763544134948319, 1548.6630518063332, 0.00024524427488991919, 423.09322948366912, 11.142616656198763], [1175.0883629904406, 97.836551076858044, 1.3500660783796592, 323.62392716542712, 154.63630661070917], [17.324416021274327, 1.6327642026835045, 653.35463143978677, 10.730252267714381, 23.607769389962574], [41.917270245463357, 0.38382058149609449, 167.38842536213511, 25.887048441678512, 373.12162350671616]], [[2243.1661449774142, 497.82287341547817, 97.366454665003801, 1311.7075913271692, 1566.9993294100934], [0.042351701817471332, 0.35575628538793413, 10.235848581084419, 1330.8515454926353, 2776.3443692877522], [0.22419830811931091, 0.039253385756343982, 1.2239798848542256, 83.06429809898934, 4.3132560014621876e-06], [1122.733688838227, 6.8170357343794272, 937.07140707564292, 28.058951200776011, 0.061592717566150319]]], [[[16.165711494615316, 27.112675958205436, 12.648262120323599, 44.376976358397023, 21.604302505830542], [0.067564999748561005, 2.8896679844222226, 25.143700082455446, 0.46999790865666546, 9.9417339625617611], [4.1338234382956243, 3.4255023035505472, 0.074241747058981136, 26.400392364854383, 51.580634559272781], [20.08069292056862, 0.42867636078406107, 1.8917144325449726, 28.528509637708542, 1.2589305826547805]], [[5.929351594462986, 0.10538253995264046, 14.788485526111199, 10.359606408764293, 13.085994611209495], [0.096844980074809517, 0.14516656785654455, 12.61700108569096, 12.698071074791025, 0.7756356745171441], [9.9413571871787934, 4.4468869377864104, 10.110145368512779, 12.395470010813264, 7.6542862840974095], [11.552646544562741, 12.254736716152744, 6.8148136591689719, 1.615628918504818, 9.8475299831843657]], [[4.7424759685298445, 4.3836440616557768, 0.93787296589100899, 3.0431349059068347, 3.1197396076052883], [2.2652734386624571, 5.7542863063779581, 2.522198885431544, 2.3518008256802827, 5.9382535229823885], [3.4572376113966721, 3.7970325159424783, 2.3850307405946229, 5.0865440322913882, 5.0603990010913957], [1.1026109641457091, 0.37332907603882942, 0.4211064682772066, 1.0250084314765051, 1.1156387384259432]], [[1.7288033466809709, 5.2841025056788773, 2.5186039191338971, 3.8762034672085859, 4.2639062343888749], [0.68461982530470988, 2.563990324169878, 9.1576142511983445, 6.0807854929283245, 0.33934696244369877], [1.1691894308880255, 6.9405395735933135, 0.69323052577913058, 5.9504291524391109, 6.2473440793534962], [1.322720000936207, 6.363955110296672, 8.4692688660592719, 8.2202852691181452, 0.71481488257066028]], [[8.0561411889481747, 0.018916519186423585, 4.1839604062405575, 1.6847956712381269, 18.494286094133685], [76.268414754772806, 0.011577257313334638, 6.1716343854046229, 38.713014306430942, 37.2068870840234], [5.0661501909213014, 102.11486129338942, 29.433324903961413, 89.895414309747139, 105.64615997326047], [28.44788748581988, 24.915902522123655, 4.1201958837075141, 1.1385369321179397, 76.958172338071904]]], [[[16.758572495512237, 64.203576379487842, 286.75784988482894, 504.24261997389368, 61.549860597080396], [235.42781486850993, 501.24832038839378, 94.974011852740048, 25.650835032360682, 247.9600532575374], [139.77925283912063, 2.7666835854201366e-06, 0.0032105899051928701, 192.09006833473856, 195.97292859055034], [149.50350153664394, 67.198640323467075, 344.39704659550046, 250.13677379560039, 6.8269152889334076e-06]], [[6.4948659151540893, 0.1716198833996031, 4.0729971354870624, 5.0337366413254916, 1.3653842568605559], [7.1896609573353665, 5.0110566027900383, 6.9486255757632982, 0.95053870918577343, 3.2332826538119059], [5.0344103544306087, 0.86424787939984749, 7.3899253255375985, 0.13914730814738654, 8.0767126660484241], [6.8787386595660793, 2.69225812026523, 3.1219797919693386, 3.6995538786172295, 1.7198629632686158]], [[1343.5560179144129, 395.56210402408624, 45.566430810593339, 917.25126980334232, 0.0031316739824466138], [191.80056215766396, 133.7869493772414, 3.8470740224303448, 1243.1363885431961, 22.601675359553941], [6.7856970582052289, 58.035083320431113, 8.4556433409028404, 544.60566885308856, 598.5194727215727], [590.15291107716246, 0.68917381123715959, 0.0092462475431901465, 1454.6168654798419, 383.1524010451526]], [[32.544135916658867, 17.966331252463764, 1.3534794356970781, 22.747515539215208, 80.273943761509457], [150.1900343034568, 58.680709558082228, 785.98591005060212, 48.805038540640737, 281.2255461583066], [732.05152390474927, 13.662192809971737, 2.6449336131470025, 0.98979135546666508, 10.315543775286178], [763.79851569465097, 472.10560257297493, 1.5062710416073364e-08, 90.362912328186184, 368.92465424297649]], [[1456.2982756095944, 0.0012621261382355778, 183.01132858392697, 423.5730755307988, 8.0014792643875996], [111.58804483995966, 149.63235660993564, 28.15855945917388, 208.70999468469466, 155.91576057077714], [886.82006683443251, 22.781093607491165, 46.148297172621383, 6.2664814197550447, 300.32806232151501], [31.810748338978573, 0.023479914429190975, 8.1357123629760899e-06, 1179.3390685733073, 5.1755684638711887]]]])+(1.-msk_ref)*numpy.array([[[[11.324858869612932, 2.1350780684669077, 1.8259635128586662, 2.2856313733684579, 7.5815984368329179], [0.012955697968669402, 4.6762798536119741, 0.97452763830660694, 7.6399589107247134, 4.6545455682038135], [9.2072689016463727, 6.0993865011245489, 11.130714139478455, 2.5779898524113678, 1.7788617250319068], [0.12767553646866989, 0.54898740556026726, 1.1646159472339481, 1.0059957417736007, 5.8562049041979103]], [[292.41517616391764, 76.729612650133163, 10.026873344488044, 9.3665511005108577, 146.95825347635616], [94.89652961741939, 275.99789617795477, 0.56693702879542018, 0.10372782884082966, 0.36652431247533263], [287.12612428346807, 40.566015280191728, 163.21645753087006, 9.3139718083537986, 15.92201143685868], [0.14186921021474272, 45.691039978988762, 0.013025629621993324, 6.110042075444218, 39.263863432338901]], [[3.4889181485129681, 3.8552851710545593, 1.9536167178456083, 0.66344594067865548, 1.429112648235169], [3.6809399716287712, 4.9054917261820705, 3.6343819351785851, 3.412376265106067, 1.320510747974436], [1.5718802419380258, 3.5804534681938409, 0.45266720324984966, 1.8202798665699886, 0.45480373181789774], [3.444570399424209, 3.6276608739583756, 1.7972648131499089, 2.3005993377464193, 3.2730366712020396]], [[0.31750026859888142, 9.6798583466078405, 8.1505018536329423, 2.9118286400536237, 1.2289915162538909], [3.5709010428761268, 4.1514060216430302, 12.920084204392065, 5.7611275171288892, 8.3034513125380389], [0.26311102616618659, 1.1408497131159425, 4.8825242741083441, 0.18404842756031803, 15.372816055356273], [13.188989001794054, 1.9249785197366991, 5.9502361077351322, 6.3545191645732375, 15.702808569709182]], [[1.7894868221732239, 2.0593640641650923, 1.817763800140088, 1.2735898848758382, 2.0875635652301145], [1.5849688530621788, 1.0829757567482006, 0.6108509745256524, 1.0103292048123695, 1.6491332957578329], [1.125498075350601, 1.9169914374924948, 0.56622617790121266, 1.7296874762258834, 0.79448073068791258], [1.0352255586604364, 2.130240410490567, 1.6276113710532318, 1.5695630790715172, 1.9386777600624152]]], [[[0.013084974963266825, 61.051214514431578, 34.296004501762184, 29.214586294209969, 1.0033295419559576], [12.253088141182921, 25.579033870397062, 112.42789717978809, 112.39613758525228, 81.937121246701707], [0.057536879177283236, 42.651217525201645, 72.860453931771332, 32.484713190783964, 249.52606497092739], [0.29709370732810458, 0.64365573834896528, 11.575154057845573, 24.749228405912856, 1.7332915940807605]], [[2.7393224062554067, 3.1958898550578976, 2.4001971851337021, 1.1578199250890771, 2.7310824941120275], [1.5528496658079751, 1.8756164043809798, 3.0251116041245849, 3.8432622878131979, 1.2476830088264186], [3.8006685570486498, 2.587957559385599, 2.7411517244664774, 1.2543321239196321, 1.9613549846002449], [4.2790489744362752, 3.2501686691942622, 1.7895050106427355, 0.18893760840761326, 2.3873812545562543]], [[0.019247195653672621, 10.715741516108476, 58.866286667642591, 256.6043347094606, 61.420597677644047], [107.55411528093578, 5.7838527807693767, 41.2157871873966, 44.631156063329797, 48.325686748414689], [6.4183268928876283, 11.551365998958804, 292.34982074764946, 14.699105606939746, 32.08777382351105], [0.0024209545132158781, 10.056977208422053, 44.131501892052988, 342.11932749829157, 7.3248552492960375]], [[41.93713530568737, 2025.9627139584404, 180.29470254115355, 5.7247937038724679, 0.0080369055181174001], [0.0070426316298989912, 16.560057580103283, 14.13944613403166, 727.78994334429831, 170.34254526428802], [770.46496819857884, 0.00045143346881527745, 1616.0815492027612, 1634.8008423999527, 1638.5693122550749], [969.91542169580714, 141.74043347075693, 1481.9402832408684, 3.7620753327028926e-08, 1632.0881391460796]], [[104.19096841826151, 650.68462497052872, 153.8107059484604, 562.13726272655617, 1448.0776594842073], [457.87985186141611, 323.18602798872246, 4.8936713533867238, 138.44537246501596, 1.8274805427219822], [546.66952389819721, 1940.886780070236, 558.96743910150883, 25.497941438576298, 1822.4464210614897], [0.0040172559553544856, 3.455222797119561, 2178.0570538125899, 2714.2150904897676, 0.51968128313040951]]], [[[2.7935937562855173, 40.351608349116162, 15.969177541183988, 43.133669730967206, 3.0769293006436924], [13.904603881797856, 7.1732713698245378, 21.344601544344084, 12.214379920694443, 32.877129479789147], [44.78944723450298, 14.93693870701998, 0.79500069657218797, 3.8158271204530467, 35.754438850140723], [1.9607569262234239, 31.543202212157791, 1.2752779829194505, 14.207818747193473, 4.3810498612988784]], [[10.853077297359848, 1.8539630550568891, 15.800457842146184, 2.5271836930661915, 8.7564516158108976], [0.073782393516266143, 1.2498473741790372, 6.1465489013343122, 10.91309280330398, 12.894328628746464], [9.2660748022339448, 1.6956329917775104, 1.681611192625927, 0.015433775853860611, 5.8274638497292273], [8.400778263664316, 5.4153626453028476, 0.96914637596939723, 13.910012684651873, 6.9106265330456704]], [[0.38576685560661217, 1.1118996197576643, 2.7362192926667377, 0.62319468932425715, 4.9180609423322377], [0.94550683621741527, 4.0317059577210408, 5.9108521241369418, 5.6683627473610496, 3.9371883070474194], [5.6787106073775657, 1.5300298505917074, 0.34698788110239931, 2.0263954199806986, 4.6011554592889325], [4.0690553795521591, 3.4778041878770076, 0.73353069597665055, 5.1163075394138184, 6.0927623304898351]], [[5.1464990162873852, 0.31255725560145275, 5.9374163482701228, 1.2640658441253081, 6.678330153630788], [1.0235977014895652, 2.839303733108824, 1.7558808596397011, 0.36815862352890472, 0.86033854934256981], [1.9142534261949082, 7.4289562514708587, 1.7915881044895698, 7.1567657114680525, 7.3037500936726678], [2.1168054976795667, 2.7445990230998825, 0.73144131128320111, 0.11651154655741286, 9.7085173332252932]], [[2.5029225637745998, 0.24608906387062005, 0.014095255561679211, 23.342526324770553, 18.900778953391956], [5.9793827346385084, 96.208747759302128, 81.017007500003089, 53.331561205651411, 0.19265464562782877], [100.94934244136863, 1.96593389135666, 69.450337584899032, 4.7237168379243553, 33.743226584483125], [0.1903078647858891, 65.209371796779138, 55.744927017184295, 68.390563781637283, 106.37353667719982]]], [[[13.153480726977174, 340.5957255971598, 37.914836729632647, 19.344842798350456, 0.011384241205435963], [1.5943129039999602, 0.31143680455643502, 0.82071993447001801, 0.016296897387027314, 2.476310950558537], [55.023445289553599, 55.494452250425347, 31.899066442570106, 297.67892492862626, 56.747298512342091], [359.99685230737447, 21.155694715846259, 317.58618615558453, 0.55511351848080182, 2.8334302281254589]], [[2.3933759279726514, 5.851910526884156, 5.1859537040623325, 3.6386648032962232, 6.0913039331266701], [1.5958046250874964, 1.7803616609376967, 3.4266850918431553, 0.30538472458552118, 3.0697568248510505], [5.9950049754462524, 0.26703776621878222, 7.5632297106826671, 3.5326685203312103, 7.4173771948140166], [1.0129477847007129, 2.7500442125760816, 4.4781187718500313, 0.73247074640925292, 3.9690449183396233]], [[0.67072259027090642, 0.12048970784896609, 302.81148027464008, 0.045560426909332548, 0.0055881423666241416], [14.878990996139914, 1627.7321356350963, 1248.7781726801904, 123.24893783778862, 1421.8409535064379], [235.2881715020701, 109.45124305667115, 6.7489998954224046e-05, 4.637260011404245, 4.1869050283308775], [4.1932988018094228, 1122.3619285765694, 241.61846525153018, 10.28524120573022, 1.2030260749744441]], [[124.21230051859355, 0.9138635173884585, 4.37957061046097, 455.73669696869598, 212.68843824822133], [3.8032762505978037, 67.262399233741874, 0.33711409152080585, 579.13643669312103, 128.93773210608887], [6.4721719366841342, 0.00098835626687892686, 0.04617190473879839, 377.65239079248221, 0.30760490679430869], [3.9713983196570246, 4.284809391362348, 89.928222026014254, 283.86256452679862, 0.13767946096324296]], [[1.2908856219175234e-05, 447.7593269432993, 425.6703478867197, 4.0383424430858267e-05, 166.34059223253368], [0.010120758998087356, 149.27502479858674, 1.4559985747889512, 1507.1089803023067, 119.90537034683756], [250.75908124778991, 958.1080118970425, 74.082416822450568, 134.46157939499429, 304.22396931778201], [104.03283350093201, 6.4218336636605233, 150.31039094118771, 2.9469734149034088, 0.050759363260239651]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank2_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[3.1510292056366627, 3.083331396751972, 3.5190758984088166, 1.7650586877641974, 0.56751097522939042], [0.30584133352416482, 1.0645167967353846, 4.0878110449442264, 1.3284422724284752, 3.1366894438872688], [4.1842207922841146, 1.5876444925086732, 2.4193128441179121, 4.1200240981643006, 1.9862744147572942], [1.4237716297764542, 1.6538805524024014, 1.149616341030284, 2.2672889062654455, 4.5663746546739423]])+(1.-msk_arg0)*numpy.array([[3.0426741189302393, 0.4444880946365104, 3.170482355640921, 1.4816469450536165, 0.23138419245904912], [3.4423751286293505, 0.86032295628839028, 1.7292554484112104, 1.8245266132068585, 1.5428761264430768], [4.4655128052369006, 1.3556318438418411, 4.2043489189005863, 2.5038180332154867, 2.564349021405401], [0.21612164897812475, 3.7752624510272184, 0.86239215046358775, 4.2154153533012373, 3.2220409266775709]])
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=power(arg0,arg1)
      s1=numpy.array([[[[2.0122574683554433, 0.48366096451688301, 4.3589921057588628, 3.165919879139897, 0.46215315160221904], [2.1157152098011265, 2.4478923755839088, 3.7760394228346525, 2.0494427590383779, 3.7033718869599803], [1.0734678781800928, 1.3500992499344893, 2.1052752132035324, 4.4835325139295072, 2.7472955276396736], [3.4027904743817645, 4.5576263172055365, 2.9747831514066654, 0.20222801321506353, 3.6021492080135791]], [[2.9421929019144222, 4.5232562233066709, 0.31680750869296659, 2.3491436429947985, 3.1734734438613041], [1.4615830182287379, 1.0990850788780064, 2.0385235708136622, 1.908562909371651, 3.4947875206500125], [2.7053353369282118, 1.717276646435409, 0.53538326716198115, 2.0933108704481755, 0.72529007726994166], [3.4793150547507827, 2.9131104738952405, 3.0737387149111406, 1.6572750933783051, 1.4478106937399773]], [[1.5205881402478447, 2.3888168297566006, 3.2579621070795426, 2.1194838234248898, 0.57867754572725927], [2.2409586452799206, 1.2606474173501065, 0.67450198901108083, 4.3882301601128431, 1.8788319762140784], [3.5356768061812685, 0.59865719880911961, 2.6794645073346444, 2.6857737293907249, 4.4156858579967873], [4.0313642551682491, 4.1053313227215531, 3.2467863169563973, 3.8737537629179095, 3.4867297309867862]], [[3.4449296507767246, 0.20173945521363273, 3.4590353320153744, 1.8198831995587632, 1.5764623221717402], [1.0935581546784647, 3.2581908001869744, 1.0432161695018143, 4.9239939070652481, 1.445236247747486], [1.6803206229065171, 0.96517472950748251, 0.76962167234781342, 4.6620058510874234, 3.0767524643954198], [0.94735077245330102, 0.85088021647087964, 4.478181938487074, 2.1498265152013634, 0.76859153684647918]], [[1.8635470964049703, 1.5899976108245519, 1.9756840936357234, 1.7047189554853279, 3.1173625935150504], [0.74125392413428925, 3.3669341151670547, 2.6586265375239582, 1.9838353627873007, 2.4771116895213652], [4.4496098895908256, 3.7012043024334913, 4.1577245435464762, 0.18830034234049584, 4.8445035153733151], [0.70048872519005523, 0.97858941275067968, 2.4913047620101136, 4.945128316519904, 4.3397514678362805]]], [[[2.5603681206338673, 4.3622873974788394, 3.1370399394862858, 1.0899664529438282, 2.7100662981208483], [4.6216053926643079, 1.306931487525274, 1.7503911688366052, 4.1299265294643952, 4.047532671046608], [3.1141856774561716, 1.0939111265214323, 4.5341821120402939, 1.8990041644629567, 4.0512001874321353], [3.0365710815399916, 1.3376019586623271, 0.51069586641956333, 3.336810972535726, 0.016540680617881564]], [[2.1332452000171735, 3.3222439851594143, 3.0020178332998082, 3.4800041170166724, 1.6001089741283272], [0.6589180420860502, 0.57308031221418898, 4.4425553450349673, 1.48138567609105, 1.2129029763734271], [4.9695185902151175, 0.86755351333496278, 2.7551772605255587, 0.6040540482449871, 4.7537800745765466], [1.4175289395535242, 2.6606653366062387, 0.20804570329383204, 4.428614876320613, 3.5259524085734681]], [[0.51451939870535657, 1.3030651363932828, 2.2332765617729686, 0.051122398460683123, 3.090150146299985], [1.6092388922708549, 2.3067340422261648, 1.8435800172414547, 4.5571673943806541, 0.27635046675589242], [0.48438459281699348, 0.43227398086611352, 2.5593753916856246, 2.9874789107774489, 4.2349444853596951], [3.5868273409555367, 0.91694691684871132, 2.6954708965618592, 4.9608257120590586, 4.1555060892199256]], [[3.0698710129086981, 4.484023456716189, 1.8925775911514742, 4.2129879581770497, 4.2082479163368234], [2.739716767912435, 3.1642234477155453, 4.0391249945889927, 0.11894944663766566, 1.2690752666828369], [2.3571795349508919, 1.9000478970361434, 3.6669692227941844, 0.27234896546023679, 2.1612844450385222], [0.55540752939212912, 1.8030676256932214, 1.466290871826839, 2.7560516421810779, 2.2179645285023351]], [[3.687779079455936, 3.3859515850412278, 2.3132031041363703, 0.27523367605264798, 2.09000535015418], [4.6197324192556941, 4.6668890366912743, 1.3953929587362721, 1.3974901051870474, 1.9238317261342557], [3.2044245991698377, 0.80167834911654667, 1.3238452049496001, 0.20795104020008134, 1.1969392837394459], [2.4645061632415173, 2.6699869117276167, 2.2758562752599829, 3.7918025428347342, 1.9648630755666892]]], [[[1.2415935219035683, 1.9233791837726744, 0.64228499677401962, 4.8327845797883091, 3.2640054938623453], [2.2496078061499434, 0.14481337738801156, 3.688235702353782, 0.54714796766200458, 1.6123178458267264], [0.39441398808521633, 3.7912956493952947, 3.7760990967920227, 0.58382368385135064, 3.019739931192948], [1.6743253751868838, 2.2164394919184112, 2.516845837194845, 0.66664630436573435, 4.4097321666175571]], [[2.0388206782241793, 1.8906978295588099, 3.9194239191619413, 4.3962233012424781, 2.3439143914355309], [3.0381569681412346, 0.34958773655717285, 1.7616952121774871, 4.7662757773609732, 4.4198276083837209], [4.4520737204848171, 4.9032135674110497, 4.4472969989955518, 3.2218241819776572, 3.7422720637714484], [4.9383606484091676, 3.4134519583318417, 2.9754535058603846, 0.72644981878939008, 0.41792556374010253]], [[3.8896283733329065, 0.13642687767093428, 0.63977597271885445, 4.0923668063860861, 4.0880615805827567], [3.1498442275458984, 2.8826393324012121, 4.3693352661823788, 1.7684657004353306, 1.5261107244814074], [2.6048699129594604, 4.8982732694552045, 3.4300862998413515, 2.2021388443827292, 2.5513542593562994], [4.0441387195308645, 2.6122013509280344, 3.4272193080449491, 2.0825608003861698, 3.8742686685384173]], [[3.1004927593981662, 1.5901872444480658, 3.8540035760421989, 1.9162460535779622, 3.1019181366041657], [3.87205933765292, 0.49735208456750374, 0.5639914345041126, 0.31120252497700329, 2.8622808895915721], [3.7018994553342384, 0.95785873389593212, 1.2343541615312776, 4.1865617129367028, 0.29087597914680813], [3.3615720117246659, 2.2388809286860321, 2.9449102022777889, 2.0336046478291911, 4.6322227231148405]], [[0.085859893056578748, 3.5209416306500216, 1.3478447788375709, 3.3804928473040672, 4.330657690362786], [3.989383657082592, 3.8429149796391249, 4.8413365296244946, 0.62364229492395074, 2.2665693817311925], [3.9823383356001147, 2.5918999076507605, 1.5760588280635732, 4.0179091383385339, 3.4895044653387282], [0.20239947547994175, 3.9262589923755438, 2.9073188374261481, 2.0450993527802224, 3.9061493779681764]]], [[[2.7324242147883377, 2.9663707345869792, 3.6359528560770342, 0.4552776446648833, 0.43423808560368088], [3.9086089610837162, 1.8237665501028486, 4.5438562334297243, 2.5382696653646297, 2.6213633082593693], [2.0594709627597503, 3.6507674189052217, 3.2024031146165157, 4.2810470348763703, 3.0339566113782706], [3.2427678928966772, 0.66461347354838807, 3.2092819673069788, 0.67820468215091667, 1.5877214487239981]], [[0.34775181033062291, 2.6083936588437027, 0.94443378408368728, 3.7131086354808955, 1.9301737243295163], [4.0355007253219037, 3.2745221196143373, 0.8924465100189396, 4.6083585347143083, 1.9690075356017989], [3.5167554559039615, 0.68237233257762542, 1.9505327089012461, 1.7878850920148857, 2.0650349844743432], [3.8726035368673561, 4.6636351982922921, 4.2514134813073037, 1.3152177023945222, 1.0001880020976355]], [[0.42591571618058049, 2.5588079091892526, 1.7274838705608055, 4.1029471135293791, 1.3751861823494707], [0.95727124346885306, 3.0840837462245756, 4.9906030418983258, 2.6821901663370116, 3.3921298658847987], [2.19405302079941, 1.6413212257191787, 3.1309128968472719, 0.82350024122954701, 0.85100075561911237], [0.55371963046323491, 3.3949093107053385, 4.9703227677192441, 3.9587708488878182, 2.5863231398937936]], [[3.0292071119775135, 2.6468557759429658, 4.0551836371131227, 3.0681545536904946, 1.5872905606037471], [1.2895707784774888, 1.6835338276719338, 3.7880521556227249, 1.1246479914188801, 2.0071585711972149], [3.0526687510827615, 3.4267119014311187, 2.0508878759921885, 2.8659675348853448, 4.3539505321184917], [3.8433796777407232, 0.63592129719030144, 4.2532983457990978, 3.5464489954115965, 1.2181731851345043]], [[4.2040161364012922, 3.4834805405123874, 2.2916389048253856, 4.603135360527479, 3.8323302910379318], [3.7610792515723537, 4.1823704978858132, 2.4180307340350451, 0.047074770752668769, 3.9181177392871258], [4.883662214152011, 3.168245398039188, 4.2690510944312114, 4.0030078510779212, 0.36138982324759183], [4.812132782192549, 2.8458204570423962, 1.8111757646805382, 1.0504995371841912, 4.6114652085707055]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[10.069655708753057, 1.7421357075676287, 148.85046678679154, 37.849526618628978, 1.6996572877053455], [11.339225297830861, 16.601904040829673, 76.238794474699148, 10.508717572775984, 70.138215270804565], [3.4282514691204331, 4.7093422069745614, 11.204166206446299, 171.72268478612105, 23.409715059668891], [49.674004877151006, 186.96482440698827, 30.394001884935221, 1.2612496080925071, 62.44533798105293]], [[27.465769863591582, 162.91598056521053, 1.4286481163815787, 14.085708821505161, 35.636302340931451], [5.1849387160030291, 3.4472661180000732, 9.9283988363674052, 8.5768107169826138, 51.170741546476464], [21.036023229820668, 6.914853414764992, 1.8273142255436212, 10.560179772202078, 2.2629840994297945], [50.286957707968782, 26.580912257125082, 31.850799428891424, 6.4631019456578267, 5.1051520632477585]], [[6.7747441694304396, 20.198444083597263, 60.289791608759622, 14.392845044113828, 2.0711234248096377], [16.769648924050827, 4.8848793962699464, 2.3365076121882087, 249.95084577179895, 10.632798645340227], [85.505695980571886, 2.1238480163402249, 29.116296261212153, 29.348348621608846, 258.73619680820099], [159.53385577659702, 175.09372633598971, 59.447967518439491, 130.83681874750525, 80.398688221491184]], [[7.0805649829918957, 1.1214529469718391, 7.1375410475205943, 2.8123724289928518, 2.4490999128903579], [1.8614247361783263, 6.3677850188615599, 1.8089357233311727, 16.407429688860176, 2.273135205705425], [2.597972136334179, 1.7304764604680767, 1.5484999290851573, 14.138150545669932, 5.7440316380566951], [1.7130398718521991, 1.6216699870001914, 12.735980027878639, 3.392261815783046, 1.5475938472846456]], [[0.34795209496536178, 0.40627459770863028, 0.32653584966386878, 0.38071086025608286, 0.17102065926821858], [0.65710149972577447, 0.14847302575737559, 0.22177339846292043, 0.32503149580706803, 0.24579145224282822], [0.080404703703798458, 0.12285942845189478, 0.094862106145649452, 0.89882112242714451, 0.064287636772049683], [0.67245270735737228, 0.57443622610447331, 0.24382313696886335, 0.060725509961706843, 0.085567620446857312]]], [[[0.048159372301620179, 0.0056962161925940428, 0.024320973991442719, 0.27492111384167189, 0.040333000654569511], [0.0041895402249217265, 0.21260758549283965, 0.12572367136034368, 0.0075013083596147416, 0.0082704468229676496], [0.024988464395956221, 0.27363934686077873, 0.004646714358957419, 0.10542778944780164, 0.0082345907935191462], [0.02739508010330519, 0.205021157344039, 0.54606585979912903, 0.019195436989549612, 0.98059518546811841]], [[1.1426756679613295, 1.2308561248940943, 1.2064583806823672, 1.2430565204302688, 1.1052155545290119], [1.0420565416821506, 1.0364791585386379, 1.3201599884327517, 1.0970422453857254, 1.0787812026775194], [1.3643787452484637, 1.055738265069355, 1.1879823857508414, 1.0384882538865707, 1.3460993085728392], [1.092671160599763, 1.1809833223930546, 1.0130921840190041, 1.3190098754588737, 1.2466326233195064]], [[2.0635922363081067, 6.2634318922772669, 23.207486145762072, 1.0746347424077953, 77.552968233508238], [9.6390625419539919, 25.736333418541058, 13.407019564059143, 611.87962781176282, 1.475658186553763], [1.9778653781145068, 1.8379410083205654, 36.73115107454462, 67.114430176566174, 388.71351915676939], [156.06731756697417, 3.6366760416404351, 44.48931195825007, 1080.1879957252074, 347.57921328952511]], [[2.3913664889068493, 3.5733166342268343, 1.7117313516459047, 3.3085770615153023, 3.3041260344910808], [2.1773288513988667, 2.4563136510534487, 3.1491730731123306, 1.0343595876413094, 1.4339405731103751], [1.9531736526601224, 1.71536684930179, 2.8333083175352169, 1.0804190999481826, 1.8474752563573651], [1.1708613142710074, 1.6687650886405032, 1.5165483324180624, 2.187453424021685, 1.8774557259481566]], [[67.745007273508492, 47.976497362399726, 14.074710369194005, 1.3697637925213795, 10.905061971799693], [196.59188511088087, 207.48060455875139, 4.9291594437650783, 4.9409907405357645, 9.0183620376502223], [38.985719978041836, 2.50040564342601, 4.5420451216801743, 1.2683577576025002, 3.928676493827338], [16.732357180273272, 21.162767796449543, 13.486455684755143, 76.299550770759055, 9.4514526376185728]]], [[[5.9127926864846625, 15.689191680723887, 2.5075738201245952, 1009.5536906914602, 106.89402919957988], [25.025861767887406, 1.2303200701280645, 196.18289361647149, 2.1883418270783839, 10.051718136877794], [1.7586260047684348, 227.36557152475734, 222.47350862048037, 2.3062868447960225, 75.355405125575317], [10.984621250640004, 23.865532731277401, 36.686653792121028, 2.5965522803118124, 551.0028777510795]], [[2.5662555410070684, 2.3964245231599861, 6.1212079509096711, 7.6305556594625079, 2.9549385781292687], [4.0730516681797351, 1.17538698688198, 2.257699095732725, 9.0541024831071901, 7.7142694861164403], [7.8301182061865537, 9.6457524933404457, 7.8128480146893251, 4.4339593930725929, 5.6399248992860365], [9.8037449659195186, 4.8446411362917825, 3.9566897213183405, 1.3990626743990402, 1.2131092656338931]], [[31.075591999131326, 1.1280956012379886, 1.7598552270505408, 37.171366927633443, 37.030250603226527], [16.164792321493881, 12.765732519357895, 47.476546778351668, 4.7703055386952888, 3.8508493035319398], [9.9877537841625568, 75.757985845248257, 20.706027156818561, 6.9975035261960175, 9.5265203136906162], [35.620810697482085, 10.052656478539987, 20.653646397127268, 6.2959603370030397, 30.656743096543753]], [[80.629179805111846, 9.5017970272073189, 234.32890303875263, 15.076454843071655, 80.792064533050109], [240.39661210370383, 2.0221886829962328, 2.2222770212285394, 1.553668485936794, 57.54609009482553], [188.92832434905321, 3.8813883857411211, 5.7412184660189229, 375.2457453542379, 1.509591941017923], [116.68897113232629, 23.806049774940597, 64.688125348942037, 17.80176248334606, 705.26547345762515]], [[1.0606927930089247, 11.204146983200847, 2.5218000265026617, 10.174651615144882, 19.530167295061528], [15.452292024896122, 13.974603434560432, 27.727345940458786, 1.5341572646158208, 4.7372536072893938], [15.37776163344158, 5.9222658729193167, 2.9493573021539721, 15.757765022441829, 10.965016471851953], [1.1490078282385958, 14.797190992336395, 7.3535173493499713, 4.0693015530843937, 14.594385883899447]]], [[[2.6258147682321296, 2.8520748496416304, 3.6132817196744336, 1.1745133412225579, 1.1658149956838473], [3.9786731943350508, 1.9047559996892107, 4.9797876310068672, 2.4517317289723821, 2.5247760619221595], [2.0701695401629929, 3.6322436681393375, 3.1001150135514881, 4.5382173383609734, 2.9209983959979375], [3.1446433096512161, 1.2646737781476867, 3.1076585830454757, 1.2707612218192783, 1.7523486196252738]], [[1.1912014418180097, 3.7148802129397942, 1.6082838178837002, 6.4763395559121895, 2.6408938826183626], [7.6168186812740144, 5.1939365650112217, 1.5667628221216086, 10.161209850639274, 2.6929996633940281], [5.8671288782970281, 1.409617166531477, 2.6680838435294159, 2.458443859445854, 2.8263028360373541], [7.0174593757040684, 10.447769929386194, 8.4908554704957453, 1.9381193323790993, 1.6540369977796168]], [[1.0611833905289769, 1.4287074588447883, 1.2723430066732908, 1.7719255184690699, 1.2113553184163692], [1.142787751799488, 1.5372705317868012, 2.0053751479654496, 1.4534981021222044, 1.6047352658040768], [1.3578641909282863, 1.257149151602833, 1.5473406835316517, 1.1216706370704599, 1.12597977000651], [1.0802626113776985, 1.6053572744870146, 1.9997126713719324, 1.7366614645767984, 1.4341990808964709]], [[11.93723979087593, 8.7292190918294974, 27.64684730576635, 12.323952073609805, 3.6668461441951488], [2.8737650727442512, 3.9674168173694686, 22.216648244292909, 2.5108460168286455, 5.1708108007253371], [12.168714212248886, 16.527973149609572, 5.3592586212331526, 10.444102560613612, 35.30687626317377], [23.245979620967649, 1.6829622924255754, 32.514475488434378, 18.230012116953322, 2.7106218056510456]], [[592.71975348057083, 198.42876430991274, 32.471309539217906, 1086.6685119134647, 337.05129133718486], [302.48290654298484, 573.55167708939859, 39.342708696196475, 1.0741110367417903, 383.95378642498036], [1663.8900011282562, 122.93753451789829, 654.25132164776983, 436.78735460759134, 1.7312605885139136], [1492.6082380775251, 75.339404659731713, 15.653086673040541, 4.9303709798966953, 1100.5029562498839]]]])+(1.-msk_ref)*numpy.array([[[[9.3850019899171375, 1.7128991442864083, 127.79287764543986, 33.880359148827736, 1.6723917820327356], [10.530060961475202, 15.239029437206314, 66.802327491625306, 9.781475310681822, 61.613320142489385], [3.3018640836957158, 4.4920316503763731, 10.408441372121889, 146.7883130786243, 21.264026757023032], [44.097808711835881, 159.40344866289971, 27.389250110973052, 1.2523559618739608, 55.05011136267504]], [[0.092031517516777142, 0.025537663239114449, 0.77346282687945445, 0.14885823671457235, 0.076294544404159406], [0.30571612805696935, 0.41017418764690605, 0.19149373994159219, 0.21277417299097706, 0.058795757192266464], [0.11151764116798721, 0.24847255760811021, 0.64784365193528148, 0.18317313936522703, 0.55538794478324771], [0.059538029896557994, 0.094227499298848133, 0.0827206728153808, 0.26085992750460563, 0.30914920573714988]], [[5.7810303538675383, 15.74333175914688, 42.918780222254782, 11.537907500672002, 1.9497996235178514], [13.273980165782305, 4.2829437081711887, 2.1777601119923613, 158.14422365906864, 8.7403799881053388], [59.131275039598968, 1.9952728480438606, 22.016463715273243, 22.177330819615197, 163.23454745600139], [104.76561950746822, 114.09997712646081, 42.368871720079213, 87.344622833953238, 55.884146695853936]], [[3.8743922093723313, 1.0825449758937706, 3.8959381489034479, 2.0451984992149845, 1.8585430849164193], [1.5371607681753652, 3.6001360084197245, 1.5070361257896652, 6.9302038541604247, 1.7650884485962186], [1.9360023980234415, 1.461498919110781, 1.3533460028343567, 6.251913057507668, 3.3522721459364155], [1.4512931711839379, 1.3972795815106753, 5.8160209943748171, 2.3284748297643536, 1.3527980057428539]], [[0.065374124518890617, 0.097564557496791826, 0.055478431512241071, 0.082483595696842882, 0.010432721405581841], [0.33791685158956269, 0.0072402473090362393, 0.020417410854851617, 0.054820461461193029, 0.026630758882804143], [0.0014843382600243041, 0.0044388360346492928, 0.0022754880755484811, 0.75910843187497457, 0.00083274552255795423], [0.35869301036647289, 0.23875016197216128, 0.026083236812728065, 0.00071870143141310129, 0.0017432761153619032]]], [[[23.689429856507342, 219.75043301140389, 48.322077822843795, 3.8473121394981002, 28.505016839544162], [302.79362715935849, 5.0307980018442278, 8.703858939326155, 164.88642550905487, 148.91933394217364], [46.976007427902957, 3.8661184267435242, 271.77707844884873, 10.459139333254083, 149.59601382249039], [42.678394220514441, 5.2251958534226093, 1.880058059220741, 61.85777815721228, 1.0206574265440904]], [[0.72546586565033389, 0.60663798372410693, 0.63657956376116198, 0.59240919528928127, 0.78605203823821423], [0.90562268171012084, 0.91739379551388767, 0.51254281356994791, 0.80021833699715583, 0.83320286152382539], [0.47347730465070514, 0.87763789018588745, 0.66066436467994116, 0.91312875983883479, 0.48909722096360714], [0.80794314710904303, 0.67012550645041358, 0.96918483090517671, 0.51361890160182733, 0.58832810972560101]], [[1.3255104264003224, 2.0414839470378103, 3.3978583553800021, 1.0283949382472302, 5.4327596951397465], [2.4141941843649146, 3.5373481425293707, 2.7448105911793483, 12.132882036118092, 1.1634091841481866], [1.3038130055745043, 1.2671275188758098, 4.0622894235724329, 5.135694746732625, 10.169984874310352], [7.1311523067951956, 1.6523582635016643, 4.3766561073536776, 15.134838830075671, 9.7369997174799519]], [[6.3342826219033377, 14.825304897695347, 3.120664073738237, 12.595710159493001, 12.559859916394267], [5.19370871356851, 6.7040553954360051, 11.345360765500525, 1.0741468818813498, 2.1449603220117814], [4.126462596974168, 3.1347137806015355, 9.0704692464096404, 1.1779422182937995, 3.6679201326817794], [1.3965123759317633, 2.9571372067820838, 2.415027932044278, 5.2449752661420721, 3.7950883066982626]], [[4.9490670727298589, 4.3419003115476889, 2.7267667553763189, 1.1267694142444242, 2.475214959225259], [7.4137698383822608, 7.5669376392436583, 1.8314532351610651, 1.8331195602171522, 2.3031237109495697], [4.0132173256306496, 1.4157316216334883, 1.7755020149419212, 1.0943686403394164, 1.6804316125318943], [2.9116760609809633, 3.1830353572499956, 2.6829613885050483, 5.1774294379020116, 2.3444703460971494]]], [[[6.4102659014538368, 17.780706190869882, 2.6145840707200767, 1382.5733660314961, 132.18544002200281], [28.970476429711123, 1.2419670200696684, 249.38979299255772, 2.2676487370555272, 11.163468660008967], [1.8043398736385261, 290.97413146161415, 284.4320795473426, 2.39557779680205, 91.715435533909115], [12.248871360185557, 27.567697879698855, 43.214149940781141, 2.711654328598196, 734.10561872919891]], [[1.859573534116578, 1.7776249711310617, 3.2954868440426424, 3.8099975169501863, 2.0404641901760496], [2.5203880663690374, 1.1122313685657936, 1.7092024600724218, 4.2640675774074674, 3.837459520708602], [3.8752958694347712, 4.4454863356180514, 3.869667598397176, 2.6652473724411077, 3.1225573529404493], [4.4932820097818533, 2.8252673841594613, 2.4727583021205302, 1.2473673121473927, 1.1356000953698202]], [[266.65906875506477, 1.2164360244885679, 2.506267803769719, 356.78292823167232, 354.58380952482088], [92.162428589391084, 62.791166476399802, 531.06655047222216, 12.676222351091385, 8.9502126239530835], [42.136078175195131, 1135.1334457064236, 137.82897958126202, 23.630409440853288, 39.019029641700456], [332.90802787967863, 42.582065122921875, 137.26265711412225, 19.90171808351748, 260.84140414546783]], [[17.213333402494467, 4.3038093959216486, 34.372923705682041, 5.8052505940927652, 17.235867226300581], [34.947293757760015, 1.5785048299739459, 1.6780640914392175, 1.3305959165614571, 13.832894704914409], [29.894228606060807, 2.4088243791146047, 3.1046809674676057, 46.6417746507936, 1.3060023818210369], [21.87418556123329, 7.8059205069199216, 14.922766497977779, 6.4654749218441045, 70.212878224294968]], [[1.084213306472378, 27.541311631788705, 3.5582601395194198, 24.12927529259813, 59.039133288346918], [42.81211635026127, 37.296180415129399, 95.498366204154323, 1.7991005603179491, 8.4523028803225788], [42.529014741396693, 11.482271126774114, 4.4113551509302935, 43.977746000889901, 26.737914950911197], [1.2099760039226581, 40.34132445589524, 15.453502530693807, 6.8611809533924841, 39.58455618406073]]], [[[0.015209447902351042, 0.010628413095590986, 0.0038106080773096188, 0.4978551414182818, 0.51416279944287457], [0.0025095491510032345, 0.06118499989604094, 0.000948341155771347, 0.020477871183755481, 0.018030279645151644], [0.042641297467756592, 0.0037251016606761026, 0.0074034760475182254, 0.001418443404113405, 0.0095830427435141339], [0.0069595461477889881, 0.36126967460775627, 0.0073258692472498478, 0.3538256012707639, 0.08783875616552643]], [[1.5872100889819103, 31.981975320476071, 3.5066173618230523, 138.76100593515423, 12.989972345226915], [212.94656380748415, 77.486527692563797, 3.2726116283573998, 455.80524272284458, 13.67770659088451], [106.90106514987248, 2.475670100299773, 13.346097233239192, 10.752642475901906, 15.538748802130151], [171.50925802928319, 490.53616291888511, 283.68794766899606, 5.7386680392394105, 3.776205459941731]], [[0.93889204989304997, 0.68466940364183193, 0.77433888754678604, 0.54475368131024715, 0.81579703461293995], [0.86786475007520525, 0.63344396515766987, 0.47767020619168793, 0.67227665902013967, 0.60520473126277241], [0.72265816010831152, 0.78427959363601274, 0.62906759363579778, 0.88522339497137104, 0.88162669509907765], [0.921294509107912, 0.60495575078473185, 0.47910651682634825, 0.55650621171828085, 0.68188607745560148]], [[78.121559025506158, 45.067384070471199, 341.8554092992382, 82.624117843042697, 9.8130322448277401], [6.3940479118072853, 11.270414917798284, 232.77049050718776, 5.0434238797643678, 17.953689509649056], [80.803594583959665, 138.40377716041462, 19.11954466890862, 61.76923510053799, 525.44114125923227], [252.05701646277336, 2.4966019220045927, 454.60323800590527, 164.42373641494513, 5.7698402282954024]], [[136.83309314623904, 58.893129623301981, 14.603363272042083, 218.27098879733427, 88.578031499316523], [81.493160084533002, 133.4112100663896, 16.930761893217475, 1.0566232259493469, 97.930346457423752], [303.06823606058055, 40.727135817488325, 147.6513513398167, 108.15649202532931, 1.5262771108392956], [278.73666255018725, 27.928637186177276, 8.323666048770983, 3.4181525062841436, 220.40866816485564]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.1252505349367889, 3.4543164761099789], [1.4787065428321, 0.51366400993111405]], [[4.1929421377280578, 1.6551125048188251], [1.6292541807861305, 4.1490481772724577]], [[0.63614114292882906, 0.90959544359884914], [0.56333843596667377, 0.3195402443314454]], [[3.8953046175122328, 4.9023704827424748], [4.0182431088014301, 1.9422991130475789]], [[1.703445724261452, 0.28220581029137493], [3.019272642781778, 0.40234666631244098]], [[3.9270316238946177, 3.1327559788810957], [1.2944708676635688, 3.1892016042483222]]])+(1.-msk_arg0)*numpy.array([[[0.81012360454915744, 1.6273684282055101], [3.1526761624134032, 0.32650005387865927]], [[2.9954499466309321, 4.1134140651911792], [2.9838349481306352, 2.9652374628783824]], [[0.24955670617497938, 4.4793493268560614], [0.34636309294859163, 1.1075602629140471]], [[0.74774372130133349, 4.8025178950060043], [0.83450042121335288, 0.59959994925426585]], [[0.48535002592122278, 1.1018468033122122], [4.4607022679698378, 2.1623748597114725]], [[0.46225353366033534, 3.7165431027041107], [4.8476054080409625, 1.7238237110906607]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[0.64098325957722091, 3.9117804845214525], [0.12429886370725896, 0.47231843334129642]], [[4.5015332976770077, 0.9433177217694857], [1.8315960235887312, 1.8151273170154569]], [[0.57842961252228486, 2.0967729238867605], [0.31257540503040859, 3.4133845991468998]], [[2.5636989806824282, 2.3678892575819788], [3.9849023500862226, 1.29558649184994]], [[3.1258956332176813, 4.7276189710063417], [2.0664320426044682, 0.5532026202129886]], [[2.5016832661557773, 0.40607320290732851], [0.26645760006697777, 3.2528046291719326]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.6213038538173941, 127.63049409309244], [1.0498231344586892, 0.73004297062592305]], [[634.2922691814299, 1.6085104159556258], [2.4449947104533098, 13.232904017686652]], [[0.76978513142763327, 0.81981182771421635], [0.83578897720994849, 0.020358941162009259]], [[32.656561225604612, 43.132562032832354], [255.28500572690271, 2.3634066864911509]], [[5.2857683280043837, 0.0025263014262146624], [9.8103738142290648, 0.60431574452740433]], [[30.630985167522233, 1.5899464218551698], [1.0711932816285539, 43.48896343007668]]])+(1.-msk_ref)*numpy.array([[[0.87374138255639577, 6.7187172150462224], [1.1534141584453403, 0.58938367398149161]], [[139.57571450674865, 3.7965404882725862], [7.4062032963451427, 7.1919731016918593]], [[0.44802767332620924, 23.197962309550618], [0.71790818730505412, 1.4172383081312174]], [[0.47461353361780179, 41.081198486187184], [0.48628683824302121, 0.51546653259439135]], [[0.10438566783863823, 1.5817325791797097], [21.975949958271439, 1.5320914807341679]], [[0.14508967359503899, 1.7041908941602666], [1.5228739186165599, 5.8784799004413895]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[4.9862739962112093, 1.5738862384411307], [1.2097174212686321, 1.3883505442154342]], [[0.58352762233958178, 0.81239645494893242], [0.46613422886560973, 1.4971305116235019]]], [[[2.7725084705119478, 2.8693361428576014], [0.94888547414332003, 1.0632643644814386]], [[4.3644275876187795, 1.6467165121965996], [3.2745931647357849, 1.7772249390117398]]], [[[3.4143429672365846, 1.1658764559735983], [1.2771897314161327, 2.2792483465085658]], [[0.80690728074981366, 0.38967548494828552], [2.5678009688111558, 2.3043924017257318]]], [[[2.4208713432756293, 0.95127436874696392], [1.6147767935211617, 0.68445640583635192]], [[4.4606297255062675, 2.8495861687362778], [0.21612939096824166, 4.1504004649680946]]], [[[2.19778499486632, 4.164814822045269], [4.0905903287375516, 2.7551169296819378]], [[3.677932454439433, 2.7445374877133797], [0.28434386523267108, 0.56143324676998607]]], [[[1.7562896504974523, 2.7174262886909788], [4.9124227624288945, 3.1776559072367001]], [[0.47966863599073839, 0.22970217182444427], [0.46305228274203147, 2.1411451613820569]]]])+(1.-msk_arg0)*numpy.array([[[[2.0262442774072134, 0.81810121069611363], [1.8882220665926168, 3.4987078490544832]], [[4.8566076718101119, 4.5405006984121199], [3.4351694908266581, 3.2000922796736995]]], [[[0.43640131999621856, 0.4192611321794229], [2.259093720986364, 0.17151976461856175]], [[0.64064763691305926, 4.7977664054538476], [3.2048497253265995, 1.4606076390299534]]], [[[4.1049946843374938, 2.0735651943442575], [4.6994322080559616, 0.60895628165459148]], [[4.4987165811021042, 0.10603046247524611], [4.8365779714163333, 0.64486201801822052]]], [[[4.0839331144985378, 0.070783287659321223], [3.3140152414410831, 2.3167646019362911]], [[4.5945114584842264, 4.643208065433587], [0.20602366943451669, 2.836241615527662]]], [[[4.71421287354634, 2.868245278801695], [4.8458209975642159, 4.654808721951297]], [[0.18274684594842808, 2.864789114605705], [1.4195223073673944, 3.9587325239508186]]], [[[1.6898045963174166, 4.2985648798491178], [3.3615170234049345, 4.3726310481613195]], [[0.97738538562945365, 0.99248735295956869], [0.94065107640051149, 1.4030746199246507]]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[2.5989675510748014, 1.1336959232214372], [4.5711360658552485, 1.0405312487099154]], [[1.4633893008829939, 2.0914251682055487], [4.0435990890334175, 3.8321464022412237]], [[4.833790684397238, 4.0513896940912], [4.6273518303231489, 3.4519873139705664]], [[4.7045451750000273, 2.8533640997027678], [1.9914573096779222, 2.8249438625044787]], [[4.5554248285032317, 3.0703173544756641], [0.94864517808950288, 0.35505530690911247]], [[3.8823617509286392, 1.3850846466693008], [0.0775138700805931, 4.5778033774342255]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[65.087493014258598, 3.2503289350750943], [1.2409047301721465, 1.4506100709111061]], [[0.085238326615848611, 0.38684531371473119], [0.45193432896447344, 1.5218195338100786]]], [[[4.4472898641410774, 4.6764085418251886], [0.8960750071981054, 1.1368893517938861]], [[386.90952035202338, 7.5148401231148654], [94.223317035080797, 9.0583570545588241]]], [[[378.35118777548274, 2.0998312220788722], [2.6945243095322478, 28.154861610975274]], [[0.37054433061459124, 0.012765539364531633], [25.92988635837407, 17.845972223545495]]], [[[64.034239137885777, 0.79056691189291306], [3.9248289517812336, 0.3389858695279061]], [[19.644670914694583, 8.0478250778084117], [0.013200911402301046, 55.72750000929576]]], [[[36.131657857558388, 664.54075243803732], [75.574822147279747, 22.45792303767109]], [[3.4399907067352649, 2.6058634903274256], [0.63985912098302811, 0.8146798401871298]]], [[[8.9045239547185702, 48.479432777717449], [9.0678245586608526, 4.9597583080117431]], [[0.9446447757672074, 0.89223947709630547], [0.029465920976833707, 32.631188140489918]]]])+(1.-msk_ref)*numpy.array([[[[6.2673224545246944, 0.59345571195305058], [2.0557023793808802, 4.1364322801465176]], [[1371.8973842352405, 1008.5806599247956], [3.6113602919643153, 3.3545739417954401]]], [[[0.29717557150556462, 0.28025153937566127], [5.498281805767153, 0.025039484777990964]], [[0.16521355973738788, 567.34741556215135], [86.761898030296365, 4.2708736112422585]]], [[[921.77458155147713, 33.95838602920508], [528.10194028754108, 0.13405234186679629]], [[1052.1335843022682, 3.0926071868240561e-05], [230.68383351804854, 0.21992864769914022]]], [[[749.63003120284611, 3.8855022667268659e-06], [30.532393685024395, 10.993625526571146]], [[20.836336971255442, 21.27844413011103], [0.011530665493225331, 19.009653240004397]]], [[[1168.5974696065475, 121.5168764323159], [127.14383959928061, 112.37523388176855]], [[0.19941483228405665, 2.7140556338885853], [1.1324496534952544, 1.6299169850967949]]], [[[7.6655560385143193, 287.60132223898245], [5.3616193847268896, 7.7176141556979259]], [[0.99822849958715221, 0.99941563800543221], [0.75572042055030153, 4.7131020146328977]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank3_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[2.994838456883377, 2.1137024083951301], [0.9983147309639393, 0.68785551376776743]], [[4.7503961316641927, 3.7620783531603772], [4.8123659591792638, 1.5635733552199096]], [[4.6924699889259367, 0.75246198504003003], [4.3722636734210871, 3.5177773562360821]], [[1.031024874530692, 4.3645289686668063], [0.63970225439951911, 2.0779393961814772]], [[0.38581598426035618, 4.1004338297409495], [4.8693166871085216, 3.7618785522193474]], [[3.9796337571892271, 1.8856559408072799], [0.45517178578009565, 3.443813234602711]]])+(1.-msk_arg0)*numpy.array([[[0.40456924161936203, 3.0045131685321067], [1.1034735993267555, 2.7127896457133289]], [[1.9557924187662601, 4.7127905007574205], [3.4134780508977576, 4.8653847647357038]], [[4.9738206141380994, 3.8040999003286227], [2.0693797213224112, 1.8546986644497201]], [[2.2909018120719504, 0.045656112746852759], [3.1580755049923002, 4.1360350104702732]], [[4.3926043906012158, 3.0894675347495366], [3.9892131451461506, 1.1159296452671825]], [[1.888589183857067, 1.1241976054870646], [3.7527216904101848, 3.2936972102560413]]])
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=power(arg0,arg1)
      s1=numpy.array([[[[4.1159378587392883, 0.44351072359228066], [4.0853806195599427, 4.7810984858576253]], [[1.6831272823713954, 4.821651863499218], [1.1849952302437852, 3.8887739164286086]]], [[[1.3663034089346853, 2.0787236930628423], [1.5769889570033233, 0.72658965926962393]], [[2.6009572483102503, 3.3483198924818569], [4.68384230777668, 3.0387526908314677]]], [[[2.359505374582576, 1.4128645264943898], [1.2772531999151402, 1.6954545120689437]], [[0.22890763380152823, 4.1866884920998926], [0.19387128757346087, 1.41705129758306]]], [[[1.56111814453161, 3.617324888262428], [2.6331779142407954, 2.0940119404186026]], [[3.462943357644086, 0.36010071606512667], [4.3000989305925579, 3.3084262215605178]]], [[[1.4502105889280303, 1.7269044769391053], [4.7033966530228462, 4.5983061790252711]], [[0.63754379751351686, 4.4207716014648639], [3.4497982094632884, 0.46385749167338908]]], [[[1.1888318684486576, 3.8041554431671174], [4.484696313737893, 1.5915790776697292]], [[2.7683660252553381, 3.6716277167055904], [2.4677137312081392, 4.6890330234369966]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[91.353105862030631, 1.6265848085743462], [21.277853461790624, 35.815137432816293]], [[0.99716511077583847, 0.99190034505852331], [0.64185212624042465, 0.23337989439984455]]], [[[8.4065590709389557, 25.511401887035742], [8.0805950579651533, 2.6187854275246898]], [[59.53682442408855, 192.64379653944027], [8.1137070598720538, 3.8893532354056823]]], [[[38.386229595010924, 8.8838121691309162], [0.69540773022659708, 0.61742640204304877]], [[1.4017242065907538, 481.32440179970979], [1.2761618225278688, 5.9441570155862928]]], [[[1.0488531823030189, 1.1168601668581775], [48.424927202185067, 21.879455794553149]], [[0.21286898657827277, 0.85139774602639628], [23.219496331656917, 11.242552888106255]]], [[[0.25128363743162951, 0.19307148810652439], [762.74903596841136, 657.62617749520621]], [[2.743411612697888, 1094.3068134104892], [96.611838761208276, 1.8488676077696538]]], [[[5.1655027918478957, 191.37994134445159], [17.193545833773399, 2.7442285482802959]], [[0.11316293643573255, 0.0555836526009355], [21.147545700883111, 329.75878605867632]]]])+(1.-msk_ref)*numpy.array([[[[0.024121651539801631, 0.66941758942790341], [89.513615117223992, 192.4354512791015]], [[1.1802494312227167, 1.6076193389476605], [3.262841924039209, 48.468245508121264]]], [[[2.5005484734643377, 4.0325477635057174], [11.527952620254437, 3.0845863345861861]], [[24.36816911857635, 60.997846893238567], [1653.2882111884162, 122.45575849364579]]], [[[44.039672198279462, 9.6455890217717695], [5.509708384433706, 9.6336469165967209]], [[1.18113144098032, 21.005086987663859], [1.1272247130118285, 2.399700885759469]]], [[[3.6476456433698554, 20.056741846546615], [0.00029527195109553176, 0.0015594618596340036]], [[53.637911663667381, 1.513012106777714], [448.09808313186466, 109.62847072188582]]], [[[8.552289283747049, 12.880112518139299], [201.42670720720341, 178.91013760130679]], [[2.4159722062926883, 453.29951319807475], [1.4599480390121695, 1.0521961140037925]]], [[[2.1295167372523545, 11.23233290118019], [1.6904949260680817, 1.2048142829887996]], [[38.904598942400071, 128.46535014269364], [18.945021640996039, 267.56902338761307]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.41362222670722781, 3.9101224737117497, 4.6552621697496557, 1.5862102227617363], [2.6965336241166526, 3.7953592739048228, 1.6294016615232136, 3.7508844810196198], [1.8070806134971205, 1.1008681235214699, 3.0676377330135054, 0.88196296244008332]], [[2.0828209446548431, 3.7230409904576862, 4.4382898802437163, 0.80681259116692838], [2.4787546875479118, 1.6740517046392531, 0.72084573356699555, 2.3830547570996936], [2.3012199407183331, 0.66480330335553706, 0.050104311228036887, 2.9321488808629992]]], [[[2.5155491075234488, 1.5568021126355689, 2.5370320146091871, 3.1214839569594539], [0.83655662005838571, 1.1995920298057741, 4.8883828988722904, 0.72970961791194044], [3.6013623221995439, 2.0506196958019443, 0.46236934054008322, 1.2484034465403304]], [[1.0048706626734358, 0.1444060254000254, 0.2877106636045364, 3.42415153277924], [1.0255489142928842, 4.0674027145629159, 0.69079162440229336, 3.0508128194836264], [2.5345792920711587, 0.87527083809153705, 1.0888622902164946, 2.4218583648283767]]], [[[0.25581083439965074, 4.4868103421726575, 0.4113181362949942, 0.035320493362683489], [4.3237137321216856, 0.031966950725394239, 4.7711931420216338, 2.6016384352173598], [3.5655015832050094, 0.72097552293745626, 3.5994547866796816, 1.9487488235989376]], [[1.0086320593886742, 2.093578963247444, 1.1940745612112476, 3.2669937638043431], [2.7945802024214079, 0.3426833680611171, 3.6901602274620058, 4.6704947649872679], [3.6456755116166195, 1.3832263027357257, 4.1290768584473163, 2.9303819495372032]]]])+(1.-msk_arg0)*numpy.array([[[[0.89322142582123154, 4.5190929419754582, 2.4193574227367627, 2.4710101802656013], [3.0115273695413189, 0.10313731786384348, 1.5391737214278798, 4.0612335501381267], [1.2007381964505008, 2.0466905143854661, 3.1739619801948238, 2.7351563085717547]], [[2.2461470641491021, 2.5337563125284479, 3.0342229299833412, 4.035083544141731], [4.0488742274848315, 4.151619149274433, 1.8392913983960122, 3.6807979899267957], [0.81155339933055115, 1.7198543300171278, 3.3951919926070762, 0.56461136332189077]]], [[[0.29954208641047564, 1.761103223169197, 0.70316160731001554, 1.4664327913292485], [3.4070676417860439, 4.1734204020061192, 1.721530463325583, 4.2386463278032105], [0.81781892511826182, 0.61970588755101497, 1.4397640518717076, 3.3475596270219907]], [[4.2913004570513369, 1.216441924680894, 2.1566307545346448, 0.47553536282117809], [1.2808838973575187, 3.6946053062125044, 1.8194345342309663, 2.3873652499706308], [2.165041400565539, 4.2849814268138982, 4.6815308240317952, 3.8384345974722778]]], [[[2.3400143181107769, 0.35000109235259708, 1.5416244485422252, 2.16856118583474], [1.969250227859916, 1.4583626665739273, 1.3826582476954072, 3.9521504429992151], [3.4439704451761046, 4.3337412206514863, 3.9669547380069661, 1.7452068753446572]], [[3.5310183708788521, 2.2481115061682702, 0.4770169587383713, 2.9890904379201126], [3.7699820023933239, 2.8324585238090143, 1.532856699348278, 4.4308848805763823], [3.5122800224493722, 1.8695156943376272, 1.9252616463199059, 2.7113264942949629]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s1=numpy.array([[[[0.52247247013722231, 3.3469047089433772, 1.2495052091827141, 4.1448205811240841], [3.5975986496059171, 2.6506535577431678, 2.5920262511508305, 0.039920241096636604], [1.143135100208353, 1.3511859867815368, 0.77461914194020987, 4.8534036749873053]], [[3.7574743299579185, 3.8964210958478578, 2.2813389750845054, 0.088206566808563036], [1.2632874585944536, 0.095960063386179809, 2.9656880929676372, 2.2327455174696138], [4.4117505053275714, 1.4839981427298778, 1.2878579281480831, 2.2383251946354568]]], [[[3.5795996984597425, 0.96904868965425606, 4.4535884074193168, 0.075820116849265914], [0.89096234621277204, 0.9925998132492424, 4.6605846437538396, 0.98000447997126838], [3.9776794265995017, 1.153265361609217, 2.2252769259509178, 3.2340717427463348]], [[2.6230403573471865, 1.571043591970503, 4.7303870859817394, 0.070057184638583833], [2.6599922605705792, 4.1811417757463367, 1.225379102041479, 2.9969057151358549], [4.2538681651756338, 3.367490004759361, 2.910082490940844, 3.2028269381114067]]], [[[4.9439435152688631, 2.0923733248948859, 2.3502730487855308, 3.6992161759543039], [2.2245251433734281, 0.5570866222032721, 4.0814714628888424, 4.2761905043005362], [1.0927126112940606, 4.0687345993708313, 3.0758680500764219, 1.3602502475521072]], [[3.7203828614829302, 2.71847680458704, 1.7708023508410022, 1.8502921028726214], [1.8012798801662162, 4.2905147083411208, 4.5397769039897344, 1.7090253284996437], [0.02824370738209029, 1.8159467658066295, 1.9797776760530104, 1.6097728163964122]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.63050143016438576, 95.941102923115068, 6.8328153971751702, 6.7679842187619341], [35.470434494012025, 34.308263890016434, 3.5447249597890576, 1.0541916095564297], [1.9667992355977622, 1.1386549747051129, 2.3828008801865739, 0.54356128459815578]], [[15.75166513868194, 167.67113371810078, 29.958379630993555, 0.98124337382206861], [3.1479679173760156, 1.0506858287644703, 0.37879540119400945, 6.9509521057254329], [39.524734106202729, 0.54560330417086966, 0.021165271083959363, 11.110004494324727]]], [[[27.170964158913318, 1.5356192097905408, 63.197711642487121, 1.0901406064795418], [0.8529945700788637, 1.1979776298023255, 1628.9690139801226, 0.73432185848408749], [163.47321657310943, 2.2892135651730188, 0.17968334144435127, 2.0493628069158776]], [[1.0128264945411196, 0.047826862779475343, 0.0027583683067710857, 1.0900571715845788], [1.0694091208939203, 352.89173866395288, 0.63553467184394374, 28.297480422038554], [52.259153946296756, 0.63850648009809163, 1.281133407244196, 16.996595403076928]]], [[[0.0011824550887261317, 23.125816679283961, 0.12394032229211686, 4.2543881287486954e-06], [25.970465440368024, 0.14688892375416332, 588.56810601518021, 59.658869811693968], [4.0115220093686759, 0.26418995855162647, 51.393803963145338, 2.4782236003757556]], [[1.032493500487293, 7.452984933732437, 1.3690126651141521, 8.9397330259654861], [6.3670912244099123, 0.010103031106387885, 375.1965274021004, 13.930225037176447], [1.037209996415821, 1.802414286512662, 16.567308357055577, 5.6447138845956486]]]])+(1.-msk_ref)*numpy.array([[[[0.94270865979021223, 155.73785591742029, 3.0160267598286286, 42.500540870412244], [52.781629922117524, 0.0024260914550419178, 3.0581196922294973, 1.0575423614609958], [1.2325943430816397, 2.6320209538368058, 2.4465285833417472, 132.08417098949491]], [[20.917990882432285, 37.431553755192056, 12.580958151759107, 1.1309415804996148], [5.8511035886193508, 1.1463683604427406, 6.0935580782883134, 18.348663067685546], [0.39804271094736271, 2.2359865777302512, 4.8270060862869002, 0.27818576787538746]]], [[[0.013363760365558907, 1.7305232954662719, 0.20837546880988791, 1.0294517984540479], [2.9807888108693716, 4.1295276381975832, 12.574782579077187, 4.117991510360155], [0.44934319363872754, 0.57588381955097789, 2.2503083673814284, 49.7747442230025]], [[45.635863699739446, 1.3604490032754009, 37.92177929334872, 0.94925815563598737], [1.9318577537834762, 236.09265870173175, 2.0821915213275983, 13.570230220387646], [26.731849782236953, 134.30177477497688, 89.306702215999096, 74.292441680294544]]], [[[66.895280925545649, 0.11117905392601868, 2.7656741586507887, 17.521517529840736], [4.5152287932090509, 1.2339208396764474, 3.7525253860665284, 356.5936531339828], [3.8623497368748589, 390.14533725303738, 69.306747800255863, 2.1329089977844387]], [[109.24402287970932, 9.0450018564269374, 0.26961678847760401, 7.5837875541540392], [10.918132076821342, 87.099247123233042, 6.9524148883341317, 12.731177765748349], [1.03611857561133, 3.1149107546479664, 3.6578549041185471, 4.9810891536931505]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=0.232915043717
      res=power(arg0,arg1)
      s0=numpy.array(0.850960157603)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.963107587848)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=1.63341252038
      res=power(arg0,arg1)
      s0=numpy.array([3.498520465355571, 0.54749821740302218])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([7.7337053066828707, 0.37382732420812992])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=2.82470279054
      res=power(arg0,arg1)
      s0=numpy.array([[4.6788817029196395, 0.51459663183246329, 2.3719143248358048, 3.815231555395274, 1.8437414461669188], [4.5807023217386735, 0.094351524300472014, 2.8486676521890519, 2.0103815004312007, 3.9312850248916877], [2.70139385357496, 4.8567975793975302, 1.7438073881701819, 2.1207027057791827, 4.1190999939306403], [1.8261419682378914, 3.3971477360578661, 0.92257202905584124, 4.065271804848078, 0.90107358925905912]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[78.154171039601593, 0.15310164546565058, 11.469465681935132, 43.916094715610136, 5.6301969036899688], [73.609973658192345, 0.00127048755602764, 19.241012245313978, 7.1890510329897035, 47.795097486600575], [16.561801165201231, 86.842990430304283, 4.8101821744281539, 8.360040382287206, 54.529795747564606], [5.4797070068165485, 31.640287698779868, 0.79640907969513819, 52.540845667556539, 0.74509419835911961]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=2.79168062948
      res=power(arg0,arg1)
      s0=numpy.array([[[1.8565796112468258, 0.49274686834763626], [3.9217976447796676, 1.1985384986463934]], [[0.98843945014079093, 1.5036184128869019], [2.8504553888438955, 4.4723235848259044]], [[1.4007441685978452, 1.1464051425320632], [1.2613571316451542, 4.9230667984774428]], [[0.69183238950054926, 2.7907870631016118], [0.70352094251761221, 3.8677978192336515]], [[1.5043061245823031, 0.34565157804537672], [2.5148011258739604, 2.6747590303812934]], [[2.8686985124799058, 4.3520283161258178], [1.511685334298783, 2.2586959264031448]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[5.6255175920489746, 0.13864493859234156], [45.375464589616371, 1.6579492527843593]], [[0.96805985267560135, 3.1225665657618702], [18.619812162093901, 65.475825811955545]], [[2.5620469943087114, 1.464377662881327], [1.9120871303917788, 85.605382941191394]], [[0.35754736789024488, 17.551999486437392], [0.37466769048797327, 43.652710076372102]], [[3.1265551993526723, 0.051525968062166083], [13.124364120451059, 15.589863598249504]], [[18.954402236628916, 60.67688564668228], [3.1695595332534987, 9.7242894672411708]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=2.8006579144
      res=power(arg0,arg1)
      s0=numpy.array([[[[4.3465092458111458, 4.916675308269201, 3.0829422959074169, 2.8548423988647063], [3.0998853972169425, 4.3400544728799444, 4.8791730741608301, 3.4625221001952791], [4.8059842669734429, 1.0586584609453384, 4.8785685416937596, 1.9376998934219059]], [[0.81251556129850455, 3.0586875926517756, 1.8292726039378033, 4.9167106364883724], [1.7633201619159689, 4.2761409322102271, 0.60358216020923672, 2.3270030678100944], [0.5647925650569392, 1.0193341393875948, 0.42861084593633014, 3.8323265289047175]]], [[[2.7852985290131445, 4.2338263044900284, 1.157657284742756, 0.54273044768117107], [1.7256946477001329, 2.5218311354021066, 1.8232862894706594, 0.77395056100282589], [2.8968330056486544, 0.82624837461025058, 0.31612269229430795, 4.4656072047386735]], [[0.14359182277192473, 0.70575077927878715, 4.8035372134967718, 3.9306439635300485], [1.4316542944968795, 2.1879379459484847, 0.70421395576538137, 1.3760532616877821], [0.468897108363936, 3.8986699305125825, 1.9135051041337274, 0.88026501726690076]]], [[[0.10593754485004742, 0.55280176940620496, 3.0267705088251726, 3.0881066291436756], [1.9458146814459332, 2.1105274602999482, 0.14380224535610428, 1.10263110941984], [1.2729221982801067, 1.0879038760534159, 1.3124624055627085, 4.9880302915432626]], [[2.9234044777634951, 1.9214939882063009, 0.74372701977846423, 3.4591449696215415], [2.7601339723978331, 3.942593362900134, 3.1250135116143403, 0.94359606181834277], [3.2556686584233119, 3.8162118979306259, 2.2097404649629437, 1.7066917026785247]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[61.26515599953084, 86.523751558252144, 23.41126097378411, 18.876858513008365], [23.773386470661571, 61.010688016871164, 84.688082403948272, 32.408133968370286], [81.178144379517747, 1.1730937773919701, 84.6586986469555, 6.3766305129661296]], [[0.55907413627327263, 22.899066614358784, 5.426904278056357, 86.525492757350335], [4.8965404895501932, 58.52761226275134, 0.24317428605039942, 10.648116580567105], [0.20189514848731188, 1.0550957337442628, 0.093225509925309008, 43.060481709771629]]], [[[17.617066227111312, 56.919993238109569, 1.5068355417185026, 0.18057644613034904], [4.6095121913872124, 13.33733628067734, 5.3773121233469254, 0.48789211242209896], [19.664802737857475, 0.58594276579517324, 0.039743565794833527, 66.083497669528384]], [[0.0043592185941315319, 0.37681158910957779, 81.062436963965723, 46.226337610208439], [2.7318029969747926, 8.9602827638240345, 0.37451805523455756, 2.4449495015377161], [0.1198949110412218, 45.180900665126899, 6.1561383274672838, 0.69965058298550886]]], [[[0.0018599372689110918, 0.19011878107259297, 22.236120862917765, 23.521259919327754], [6.4517025511238284, 8.100433137469583, 0.0043771330735153449, 1.3147179469256682], [1.965686065657257, 1.2661277892572171, 2.1415140759720575, 90.086685355361709]], [[20.174159818249244, 6.2283913698096693, 0.43638857164992162, 32.319685980779823], [17.174913140746135, 46.620994734407077, 24.317050048073298, 0.84993271565410322], [27.272806407212428, 42.555295620649915, 9.2125975416766046, 4.4687592085315746]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(2.58885134663)
      res=power(arg0,arg1)
      s0=numpy.array(4.19588734813)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(40.9636158046)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(1.47720828675)
      res=power(arg0,arg1)
      s0=numpy.array([1.9548308290217016, 4.8030832291251562])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([2.6917138698989511, 10.156572137123385])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([2.3772086932096697, 0.42430540934466643])
      res=power(arg0,arg1)
      s0=numpy.array(4.9475132601)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([44.740695817664388, 1.9707527725212322])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(1.53978759706)
      res=power(arg0,arg1)
      s0=numpy.array([[2.0062580590954107, 4.8013959482524591, 1.2712098531772134, 1.1565788563811814, 4.3978538585207119], [0.37127894984579884, 1.8632092597299281, 4.6957509059469107, 4.1000317614613122, 2.6929131266088442], [0.93550847404789494, 3.8887320619281756, 2.9955781083410402, 3.6447541541471193, 1.0270011421557541], [2.8320090176226693, 0.1100576762083859, 2.5283562396827923, 3.5902528933700939, 3.4679485133526509]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[2.9215372996998092, 11.198536216550725, 1.4470129892607941, 1.2510551766465057, 9.7825977434913955], [0.21748529626704521, 2.607026896684649, 10.821391424774884, 8.7813676995677898, 4.5967515182262462], [0.90244272296995953, 8.0942958698956033, 5.4160041966089203, 7.3257111235700858, 1.0418777638380621], [4.9674065642649898, 0.033442511441626098, 4.1714338800040807, 7.1577191323441687, 6.7857390463952774]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[3.362707261447401, 0.089169687259389591, 1.8035727227086706, 4.7384941485418519, 1.5941380151720494], [0.70347652836507357, 3.0961838772315797, 3.4629278889522519, 1.8382975122559564, 4.7922668653038203], [3.9509936672315482, 1.0558145551532356, 2.2646222832751968, 0.56867147859300582, 3.1830805507474587], [1.7347549620045706, 2.5294048401321096, 4.8783553403611641, 2.2383734560300983, 4.3959201649221926]])
      res=power(arg0,arg1)
      s0=numpy.array(1.54138679069)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[4.2844177743107581, 1.0393361223229631, 2.1822891595217833, 7.769949081860215, 1.9932284503534652], [1.3557868681967498, 3.8177630296275988, 4.4742931396989931, 2.2153251972698622, 7.9528481791348984], [5.5263408999305739, 1.5790643803047799, 2.6640898694440098, 1.2789689059082818, 3.9640385000011382], [2.1182666929283904, 2.9874771600806738, 8.2546700327034994, 2.6340038844233993, 6.6995330844341821]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(2.79929747217)
      res=power(arg0,arg1)
      s0=numpy.array([[[0.81351753457912068, 4.0856853959362338], [1.6244463449702764, 3.4629297833588595]], [[3.8281498201908963, 4.1405384084733514], [3.7056583902858717, 4.2236410437586898]], [[2.379537264042173, 4.6338995550353088], [1.2064183957615335, 2.3069692768018806]], [[1.9080590783589502, 1.2272997258288145], [0.92953448268736405, 0.24786729559683601]], [[1.2814299061540475, 2.8239848545565911], [2.4836196102062531, 2.8387710923511271]], [[4.8788597341535516, 3.0298134664451872], [0.39985537568557461, 4.2297128724368225]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.56116469565203886, 51.417665149212446], [3.8889021264010464, 32.36408592084058]], [[42.850848643174174, 53.373488550050887], [39.122213873952234, 56.426620691081531]], [[11.321775329694306, 73.144668846283736], [1.6909743022598307, 10.381545777257926]], [[6.101828053363926, 1.7741865961555392], [0.81501520158760632, 0.020148346788231835]], [[2.0020270287162494, 18.285122474584167], [12.763232088098752, 18.554391274930051]], [[84.490478186219704, 22.265183987636064], [0.076843635567732632, 56.653987535477945]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[4.2016796060814903, 0.19077414610061313], [3.8908780933547407, 2.6001363506315394]], [[2.6622947907771524, 1.087760729953769], [1.135802493473812, 0.47336596260120123]], [[4.2449763117179433, 0.86733372446997148], [4.1807979910458757, 1.3954131391903337]], [[1.2463944414230512, 4.838588164059245], [0.12858687708673727, 2.6829042127106972]], [[1.4744493791147011, 2.8928529315675831], [2.2690237463324903, 2.2103064072138059]], [[1.8990055049089438, 1.0668881507381875], [0.29147345389351259, 3.8051425449966176]]])
      res=power(arg0,arg1)
      s0=numpy.array(0.559971027327)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.087472665010894152, 0.89527508656093802], [0.10474731127359693, 0.22140942872261826]], [[0.21357109544716332, 0.53218714856841598], [0.51756608727924314, 0.75995897207717378]], [[0.085303874360751653, 0.60474946791390771], [0.088538277414726335, 0.44523255275352952]], [[0.48541696497769177, 0.060461211696590557], [0.92814834685926206, 0.21103394275586465]], [[0.42528773918632312, 0.18684436060091114], [0.26827590557711312, 0.27756756652117826]], [[0.33247960285560235, 0.53866755410702127], [0.84449473283943099, 0.11008649586952581]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(3.53172759236)
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.2283340727803482, 4.2727482062823485, 3.0390041253381148, 4.6025538064362941], [2.908563229745154, 0.43375743852374354, 1.4517124653406479, 1.826480291508672], [2.4036254523870126, 2.0280603936882358, 1.4593521507965419, 2.8432108141303947]], [[1.8558521367065015, 0.17504395973797315, 3.4330226238533963, 0.35746787420316356], [0.74858262468213965, 0.0178495463670541, 1.296397491023572, 4.0610014566008852], [2.7955410272019825, 3.3401929843424276, 3.1102304547565458, 2.4638265120630156]]], [[[3.7959114774656304, 0.90726749910624194, 4.0128734827196517, 3.5402339790637765], [3.5813101505416993, 2.0145544269676998, 2.3613271194538856, 0.90451443556626776], [4.5817716150195356, 3.7323266611259904, 1.4530222634768013, 4.0072208431292458]], [[1.9895357326596996, 1.9330206925753786, 2.1654518213597811, 4.3210613953703687], [2.9258995210572594, 1.5970181189532275, 2.5276302928199295, 3.8310245039772544], [4.9659132229299496, 0.12872291220261003, 2.1153499581975086, 2.6040498070018763]]], [[[4.5925065551054409, 4.6495049643986661, 3.7182862103548313, 2.4573667077189096], [2.5189016621454159, 1.8656103055566835, 2.9935276314397914, 1.3188637599463839], [1.4315191886267045, 4.6909373439219486, 1.0276954891133769, 3.7517311188496776]], [[1.9920352995438479, 2.330453863376134, 2.496675999838017, 4.6392571397746849], [1.8734087436163296, 4.1632600386811349, 0.052323541913635301, 4.1537656877422924], [3.9186696612853376, 2.8441156827108944, 3.9220702406129804, 0.90951594942249447]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[62.744159462070385, 168.84428914393376, 50.684529023109953, 219.54897721589754], [43.409640155556922, 0.052342564059688089, 3.7300804849442843, 8.3936891190043212], [22.136949076732012, 12.148594328535699, 3.7998702160294027, 40.061756505226676]], [[8.8801854194039453, 0.0021232534211133342, 77.958605145101629, 0.026433510779704329], [0.35962429919572936, 6.6868792606958162e-07, 2.5012698462660361, 141.09982423119385], [37.739473169716284, 70.764993125790426, 55.005879779269634, 24.157961282699656]]], [[[111.16973489213248, 0.70914130421052612, 135.28209668558642, 86.902314595911193], [90.515964696381545, 11.865263451317382, 20.791503164533644, 0.70157067410775331], [216.06779016432998, 104.73125360851321, 3.7419798865927008, 134.61028304775132]], [[11.352977474800815, 10.254378788314884, 15.313201125365365, 175.68402944010771], [44.330355685792092, 5.2243827150109761, 26.440798225932653, 114.84429675122867], [287.13049404815524, 0.00071704600757257792, 14.098125961143136, 29.373794640748841]]], [[[217.86099999325882, 227.56146099028899, 103.34642556954623, 23.935007486911868], [26.119731680538948, 9.0461911121969276, 48.056218984254862, 2.6577457126505157], [3.5500394375550548, 234.80436417279248, 1.10129078778784, 106.66697008031805]], [[11.403432052878548, 19.847224672837754, 25.314829002839264, 225.79502115186926], [9.1804479818306568, 154.05312677655914, 2.9839209720129856e-05, 152.81593968019402], [124.39535173342006, 40.106803786833964, 124.77701699203134, 0.71536760294980628]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[2.5696535587815625, 2.2313837165928447, 4.1663604218325805, 0.18716510339330428], [1.9783717805442529, 4.3205896530646566, 1.7702179439603221, 0.085279544464223297], [3.8126164358201424, 4.9739155567474667, 1.3809900651228815, 3.6872352893004723]], [[4.8181471923673849, 3.7092714998975356, 4.6369925589895873, 1.8664600858698648], [0.47613719370369773, 4.2221594863469019, 1.3479464033863122, 3.7672360225610526], [2.3519768200073261, 4.2357134863685619, 0.82114930621679127, 2.2090697436998612]]], [[[0.68428464510101328, 4.0820403946868291, 1.7933726744186695, 0.18968276553431554], [4.529454353734387, 4.0847289754157199, 4.4539540792210408, 0.36188326368476292], [1.2083810511516562, 4.2572099193804167, 3.9658118789807015, 4.5791329541308974]], [[4.1202818063256927, 4.8772002654554765, 2.6559989611936281, 1.8244994302491708], [2.5482412476206706, 0.39359032332017702, 3.8157349594277679, 3.7844750162312373], [4.383009464403484, 4.8134939327154651, 0.65451057255867939, 2.7604823417370672]]], [[[3.7402901007947138, 1.5581387712908941, 0.87853700179726668, 3.6405966324606913], [2.7245246566297276, 4.4909656278867169, 2.0803510884659406, 3.8361830303039852], [4.0327106161258355, 4.8681939899875522, 3.0092808711405361, 2.3394522543597405]], [[0.31563661916601476, 3.1802390018695705, 2.9635420277056999, 4.5193247339305236], [0.29357530208991717, 4.7682863697565923, 3.6438154472835733, 3.478429843020888], [3.4605242997128616, 2.9731680567727916, 4.6145992895339454, 0.90660664283614245]]]])
      res=power(arg0,arg1)
      s0=numpy.array(0.965629371924)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.9140462336779468, 0.9249245869110283, 0.86440034694784451, 0.99347524396303699], [0.93314569610975584, 0.85975015460092419, 0.93996399154729882, 0.99702177535054304], [0.87516136771986675, 0.84032744439872586, 0.9528475165249114, 0.87900757834106202]], [[0.84491806570211525, 0.87833037002337377, 0.85028839226306852, 0.93680530984736277], [0.98348490505854491, 0.8627150430016457, 0.95394936698237287, 0.87655151682098253], [0.9210316860789578, 0.8623061666276105, 0.97168864091975637, 0.92564671261686016]]], [[[0.97635113637171378, 0.86695332051153862, 0.93920307819880111, 0.99338776664146178], [0.85349249267009519, 0.86687180158053923, 0.85574923444563578, 0.98742282575721763], [0.95861728732986728, 0.8616580924124263, 0.87048475807821868, 0.85201082121633753]], [[0.86579454495192576, 0.8431747791452241, 0.91129002641752499, 0.93818115775740063], [0.91473101899563702, 0.98632841980719765, 0.87506591823490221, 0.8760231706852083], [0.85787524282920069, 0.84505558616422383, 0.9773683929902075, 0.90796595269753233]]], [[[0.87737800234899621, 0.94696210572414374, 0.96974027628650539, 0.88044258200826209], [0.90910855345152997, 0.8546421957226602, 0.92982333114770177, 0.87444031653325838], [0.86845038214842074, 0.84344041783857571, 0.90009931271384558, 0.92143523154166262]], [[0.98902125997810386, 0.89473341302373643, 0.90154037625310046, 0.85379492616222463], [0.9897846820716435, 0.8463927969104178, 0.88034346853748568, 0.88545047750525585], [0.88600516436083054, 0.90123690375178811, 0.85095460654435418, 0.96878870960602004]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.19592932373873312, 3.9285069733102418])
      res=power(arg0,arg1)
      s0=numpy.array([0.69032309520056367, 0.31321826048607598])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.92996294942218383, 0.010457592430965978])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=numpy.array([1.1979667651285812, 1.2184754081075873])
      res=power(arg0,arg1)
      s0=numpy.array([[2.274607734130373, 4.4489210427673287], [2.1107106537756652, 0.025123330475006915]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[2.6764685160499977, 5.9784359907729625], [2.48489702063892, 0.011233974323302974]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_array_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[0.80166887409645204, 1.2815164621714576], [4.4380943712318217, 3.7969136384746602]])
      res=power(arg0,arg1)
      s0=numpy.array([4.0447772473860679, 3.2977508451784288])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[3.0656919827170199, 5.9943960388490947], [199.48014727361416, 92.817183458633153]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_array_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=numpy.array([4.202507228699103, 0.23920935169720894])
      res=power(arg0,arg1)
      s0=numpy.array([[[1.8171820116486408, 2.132549306424754, 2.5880070244444831, 1.1361004743578067, 0.23095695208630052], [3.100694507303321, 0.30457847476505528, 3.2309157723438355, 0.1598152351445484, 4.1178961539264636], [3.1246969372655924, 3.7813064263840017, 3.3296931047429079, 1.2179660020397962, 3.5587855425677879], [0.63572275793456268, 2.7619240618578109, 3.5880801700741665, 3.6606055129212507, 3.3193353442097693]], [[2.1493074283616336, 3.3941386933280002, 3.6316947955361143, 4.207013680003211, 1.4734104602448821], [2.1570117322621103, 2.3979923375257521, 1.834389768312777, 4.3739111154048258, 1.9024401081176578], [3.8718560913910229, 3.3163985217291643, 2.2906533028806662, 2.3251907024504406, 4.0388189753245856], [2.6493774333308915, 4.1943392071789942, 2.700706523422002, 2.2835537088213322, 2.6864124530428861]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[12.306194455091035, 24.110197413048922, 54.38639744194883, 1.7095793185875419, 0.0021146309921680007], [116.24115478970512, 0.0067646010311644663, 138.17983469981817, 0.00044998665753841411, 382.98147809744114], [120.06979905749206, 267.63576308025267, 156.82216862145842, 2.2902474214651711, 207.41963879512414], [0.14901546223224985, 71.481803280027663, 214.69016331361081, 233.52601032490242, 154.7822450717031]], [[1.2008504309111305, 1.339539362420938, 1.3613926230573414, 1.4101340448669695, 1.0971464818155352], [1.2018787090376388, 1.232716314800703, 1.1561911997196601, 1.4233184730590389, 1.1663094691812601], [1.3824065166207762, 1.3321353309790966, 1.2192861642850892, 1.2236587464664479, 1.3964381953527427], [1.2624667881722109, 1.4091166422810348, 1.2682749875988981, 1.2183811183683471, 1.2666660236195442]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_array_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[0.56468542198072635, 1.4854124900775914, 1.8587946387745795, 2.7222638288049756, 4.5359810659289082], [4.9095070907122595, 1.7203862710299578, 0.076183088914358918, 2.6377637608636411, 2.8462690386916063], [4.7012644485453565, 3.7781096122809465, 0.59430618532291379, 1.072920272798868, 4.7687475568152484], [0.88692733875165009, 3.0543937524027163, 4.610949946563073, 0.41432820349358435, 0.58613356076367595]], [[3.3558718424856582, 4.526453422505119, 1.0471838289929487, 4.2265219014521787, 3.9206502466008755], [3.2375148820846413, 0.39809145976068466, 2.7649646421190885, 2.46280012771062, 0.34100960479956999], [3.1364968826460351, 4.7727597640579855, 2.0696085256371806, 1.970466426719417, 0.49267584498724298], [1.0499902565967947, 0.96383535225705752, 3.4815230990443631, 0.764465449674821, 2.0333894461048669]]])
      res=power(arg0,arg1)
      s0=numpy.array([2.0186366054914111, 2.079305357736394])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1.4868319570792852, 2.8388155656768004, 3.6901196088892725, 6.7678192621035764, 24.195687511493052], [31.454669254006614, 3.3482497375497191, 1.0549703932460803, 6.3778061978002247, 7.3837596484313126], [27.174340252008495, 14.208332362449411, 1.5180914772476262, 2.1247266612443929, 28.493465990333803], [1.8645086742256021, 8.546094490754367, 25.503972683054503, 1.3378092452543093, 1.5094016361126541]], [[11.665221222059854, 27.481615127467542, 2.1523796337970511, 22.064218115755761, 17.637878797066627], [10.697077416368382, 1.3383217906398541, 7.5689002695037795, 6.0669324906718707, 1.283551162856821], [9.9345836516792367, 32.9114896405192, 4.5495282642168853, 4.2310414757607608, 1.4342691381531203], [2.1568060279127055, 2.0249806158078552, 12.789094891132134, 1.7499979310279083, 4.4304889757708921]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_array_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=numpy.array([1.6246279925135538, 3.9452682155879168])
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.1353004477559128, 4.411892598356153], [0.7080347656817727, 4.1988227189456548]], [[4.3217422438580178, 1.5992276741567011], [2.6120940003414619, 3.3347804317306]], [[1.1587026968363168, 3.9735424742125125], [1.5394483230651532, 4.5680686732532667]], [[3.7532817570208499, 3.4240194245250031], [3.8849876399530823, 0.22277560643946526]], [[1.5590609465057297, 3.1690421128687434], [1.5998588378700993, 3.9607688497261107]], [[3.2354634778692049, 4.4957567702902947], [2.2421208402365083, 1.8689490571105551]]], [[[4.0651592705124742, 4.1438360595603561], [4.7316529364843092, 3.7267286831305766]], [[2.6802289288348935, 2.7985997507960878], [1.7333593561334966, 0.57878063568818494]], [[0.85804964437585074, 1.8603700291100813], [4.7420303336141751, 0.39952849141933078]], [[0.64127875568379777, 3.0471309161038018], [4.1776299690548049, 0.61439359881436029]], [[1.7190983917431899, 4.8186154867353306], [0.7459607491082878, 0.44890523544236449]], [[4.7046153778946778, 4.4592777723828112], [3.7518619179027763, 4.5739907832832616]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[6.4013124051735915, 11.150021594976145], [0.57068239850259406, 10.288461619605039]], [[10.782244597719178, 2.1442645993250298], [4.7582984649659581, 7.0760303543601637]], [[1.2703714670702964, 9.4067829494615243], [2.0155733638924191, 11.798317763201059]], [[8.57441427244264, 7.3862245342046817], [9.0685720947984159, 0.087202212558480388]], [[2.0574570568703865, 6.5136087446464153], [2.1456396463761318, 9.3577040382127574]], [[6.7368536866347872, 11.496395970522222], [3.7127050799806876, 2.7621346267085318]]], [[[252.91486791348143, 272.78388927540351], [460.36984423900446, 179.4904371396199]], [[48.893705240345831, 57.983082213065273], [8.7595064376117691, 0.11562560091636476]], [[0.54662393818527333, 11.578216994710635], [464.36617433254611, 0.026791623042223524]], [[0.17327985718982167, 81.111088577518842], [281.66655022865075, 0.14634079823572138]], [[8.4786070337676254, 494.66539959022663], [0.31465192088581401, 0.042428421776386095]], [[450.07827456201045, 364.35329138495371], [184.31379062336774, 402.75598372481517]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_array_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[[2.8012225037433787, 1.1312970159127413], [2.4438543321944146, 1.9063612203078846]], [[4.2794119026226758, 0.12039014193428088], [2.2863990797528788, 4.6037495443918175]], [[4.7496015582534836, 4.6045308805943561], [1.6319381749351218, 3.6510139806715736]], [[0.69889284314688593, 2.0010379887176559], [0.97408368728442452, 1.6497904983961076]], [[0.95666368582691097, 0.69468744270600957], [3.2873293958728516, 2.2840772013703545]], [[0.65894316994282376, 2.1241526275566796], [3.4943410672003696, 4.9221805278213067]]], [[[3.6248449189118301, 1.9514886607712725], [4.6351684342377784, 2.1481383051363254]], [[1.8127309795709905, 2.9462092819921244], [3.6698205299846034, 4.9506823030069258]], [[1.3892284900909231, 2.4136499293229843], [1.6289951474179605, 0.72984820429712394]], [[1.6583494618559669, 3.6917960239178869], [2.7246323371196079, 1.7368210104655275]], [[2.1764222211342568, 4.8382184057794779], [2.7793244012341187, 0.93016110211876624]], [[3.7225892121755462, 3.7734933549416927], [0.89677489406337352, 4.3952328181628033]]]])
      res=power(arg0,arg1)
      s0=numpy.array([1.9979399458596603, 4.0744524805710443])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[6.9502154463746466, 2.188003468649665], [5.4272480823018565, 3.7412668699350671]], [[19.333746898669965, 1.0868939251683027], [4.8668884411273119, 24.199490668348492]], [[26.76990180478662, 24.212580705705218], [3.0940828765437351, 12.514994251322337]], [[1.6220898588527537, 3.9946327777902493], [1.9624221710398608, 3.1325501831694349]], [[1.938903950103773, 1.6173754237316371], [9.7300138438823112, 4.8590735806587979]], [[1.5778537244027233, 4.3499369144580831], [11.228908149606056, 30.166226264856888]]], [[[162.70707835340633, 15.507551692151598], [672.62617414115971, 20.441517825814628]], [[12.761197849257949, 62.717928821114398], [173.31840527517895, 1047.7533900458743]], [[7.0392370878329933, 29.68203133701854], [9.8582507252968981, 2.7877764933734741]], [[10.273253959184899, 178.75214321536319], [45.942479027462589, 11.47046709909047]], [[21.270039066170472, 894.63999291985533], [49.611273196349622, 3.6937091270690634]], [[186.65397155652943, 200.48980234217927], [3.5244779137960749, 480.17154494797086]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[0.33857554847030868, 1.5582290431503356, 3.4711111958431635, 0.28127239107459501, 3.5108076946815774], [0.61104419800687271, 3.9433586477257903, 0.66324200127116528, 0.19709136165006261, 4.5111861918507978], [0.19154557670773642, 3.2352251815708755, 4.2244233341695212, 3.5605009819070546, 1.3890961304327976], [0.96777749971608251, 1.2082637461350236, 3.9679775321815112, 0.70592627325996082, 2.333035673276787]])
      res=power(arg0,arg1)
      s0=numpy.array([[3.4290335518718402, 4.718263155010554, 4.8779200523535096, 2.8392660940384009, 0.27754499951645772], [4.8453059138691268, 2.5908844788999286, 0.57168333138926231, 1.7658488409264161, 0.58130416924203376], [2.1300992785685859, 2.8780095239661736, 3.7452336988645492, 0.1158205711674191, 3.6943952250122702], [3.8779188036321948, 1.1188913607820976, 4.9848786220573524, 1.2292011211963005, 3.1336401585676801]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[1.5177353990351679, 11.217780286640279, 244.87198200574301, 1.3411407616186184, 0.011108377813227968], [2.6227780729715517, 42.694697477451371, 0.69013705692423566, 1.1185937963617583, 0.086532657569135127], [1.1558554836973329, 30.567979451553835, 264.61700004620434, 0.00046409626559622784, 6.1428834753013231], [3.7122104535485647, 1.145377574308418, 586.51311873690577, 1.1568237425977916, 14.364823772518509]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=numpy.array([[4.7361764254900365, 4.3313316201657992, 4.9771909675066786, 0.66143078894602791, 2.8870885293301316], [3.7916685705833597, 4.4129642368700459, 2.8834241243147005, 4.0777847243758032, 3.8095161684771339], [3.5025105090718029, 0.83423661722495424, 4.8130977485202679, 1.7584033176353049, 4.0219141105852456], [3.9416691071993095, 2.4236908962881274, 3.6544066259290675, 4.3641972937774431, 3.0159602920517958]])
      res=power(arg0,arg1)
      s0=numpy.array([[[4.8024407921206205, 2.2701428978079576], [3.9862120508659973, 3.3162645315669699], [2.2071449235074123, 1.9398886558858703], [0.15087694288253739, 4.4628757103717582], [0.2200496543578028, 1.1939370703615251]], [[2.2222779157252721, 0.060505600411500736], [1.3058290885364199, 2.5347471627830123], [2.8376346719449423, 4.5734621497945938], [0.9081702694018372, 0.92743673855377595], [3.1704167681319753, 1.4168964800642179]], [[0.34012615759206966, 3.4843027291578688], [1.1831632037980466, 4.3368792240069078], [3.0378832875422508, 0.15553517250291216], [3.9150177352818267, 1.0028187034479694], [4.5625888815362972, 2.5855572889976219]], [[3.5923447393769017, 3.5904275327730666], [0.50091670920737164, 0.057929531424805052], [0.098108158816348559, 2.4353439908055337], [2.5592301341099533, 1.1610680353864891], [4.9722521899208116, 3.1080644890874134]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[1688.5905348559411, 48.565822737292415], [399.23282381329386, 179.93025516851267], [51.441284285015193, 27.059521114142804], [0.28623048116133487, 2.6895227635914831], [0.012641512194677278, 1.6682157724878197]], [[20.651161421851274, 2.4042109389319263e-05], [3.2464017825107869, 60.610678416356748], [20.233249986862052, 80.124737208437196], [0.67517316840450425, 0.7355164671314236], [81.098133998681504, 3.7715947312363434]], [[0.022885704873537673, 79.207548266620805], [1.1506322058898704, 3.4006086762683223], [210.21407989044371, 0.00012888195861791415], [11.022115659838882, 1.0049617139944793], [448.01378951311261, 45.630730102752615]], [[154.56684180552705, 154.24194422250261], [0.18720800757553829, 0.0010038111840126713], [0.00020667147047963226, 25.861193972233906], [60.404553801562095, 1.9188969973723562], [126.11787215921503, 30.572457413430222]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_array_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[0.79712969498950426, 1.543520118108509], [4.9303272791040849, 0.96971751262345118], [3.2994696701460038, 0.55257967740152047], [2.7092120040954275, 2.6460600865428034], [2.5941074373134176, 2.5077366563067995]], [[4.323840535428265, 1.2059716572425809], [2.8176875234493561, 1.047400156113679], [2.4535769560600942, 4.1528596803212938], [1.3828083182682218, 1.9951137986609149], [1.8175240987577443, 1.6982849719468371]], [[1.7385579104824407, 4.5694733072245066], [4.3710111080690268, 3.3444094387644743], [2.2877338736765025, 4.7814995087818017], [1.3607818644549039, 1.4944491605582884], [4.0223800021174334, 1.2173780498702607]], [[1.6153297087771612, 0.067610372537573848], [1.7056541302426738, 0.17424885770932447], [3.7390155737726514, 1.4111469337576374], [3.4978513248064838, 3.799201840715952], [0.13898432716201925, 4.2088647605696412]]])
      res=power(arg0,arg1)
      s0=numpy.array([[0.56548268156214809, 1.6747710261626778, 3.5721338471426591, 2.7326597907377894, 0.66929241283051677], [1.1696909074564397, 2.9654316410790509, 1.0784502099852928, 2.6526631892051475, 4.8611785339676805], [2.5529749704901534, 2.909525239544243, 1.3240062187467407, 4.2007031525445315, 4.9201004938532513], [2.3339521037597355, 2.694557754224626, 2.9175800198449275, 4.105441762640436, 3.7996672182689526]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.63481339066957998, 0.41481478019024753], [12.710821203595648, 1.6488210038624813], [66.737241750423266, 2.0208618394148776], [15.233610864414979, 14.296562982046533], [0.3528815187961849, 0.36533443136492721]], [[1.9693760033998924, 1.2080692613922279], [21.389283370353166, 3.1222304730129222], [1.2035872207304037, 1.3684036063504763], [3.8536400518449994, 7.0031596244535192], [17.707993576701181, 14.665027794973435]], [[5.1012195048096034, 72.441657213849254], [106.50541228214929, 35.580422925952327], [1.9004302687600045, 3.8266354847286408], [7.05026211308168, 8.5412813694812808], [607.27062318333014, 6.9565502456299839]], [[3.9317857588590308, 1.0589777447227404], [5.4232893555247834, 1.1885349429886136], [54.793293075569885, 4.5312232986770669], [139.77893408339875, 213.93313006457501], [1.2038587865080139, 275.46768922955164]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=numpy.array([[0.56885633416238868, 4.379518637976771, 1.1859021192585408, 4.408791331967862, 1.6394220181901871], [3.9152547618043125, 4.0160038414132933, 1.0521466542573792, 1.6260438621071469, 2.203221542277086], [2.7586369227102661, 1.7134747917955866, 3.225429615516493, 1.6778579023366926, 1.5877985426115782], [1.2134549070885001, 3.8447840568436598, 1.4989735460440765, 2.1942062566548017, 0.48853968649227075]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.6314586172495775, 4.0574858222681032, 4.1242850612343611, 4.4246325838069422, 0.77608685005331968], [2.6555217748853908, 2.0231463889948365, 1.5926360996407509, 4.9440097626519188, 0.4919267520787895], [4.6871736672954523, 0.64903464612670336, 1.4167054766294167, 4.342060269745871, 4.0770908600475311], [4.4265622150102821, 4.5267761903151396, 3.4908970801600354, 0.99709353946436774, 2.7766463908203196]], [[1.5690097142337809, 0.5185162745624956, 2.4670028934399002, 4.8896188923694437, 1.1186017604284404], [3.5619398370652022, 3.5141733302504474, 4.8584133336305468, 0.14290679055925559, 1.9720122356377083], [4.6204582862665395, 2.8552264095098794, 1.0280018048422821, 1.349197753625667, 2.443004658540147], [3.7871297387756444, 1.1545198442668929, 4.5149287728866083, 0.4597204074474795, 4.2126698086519871]], [[4.6043814063097646, 0.23599084285132876, 4.6248784884212419, 4.4468857282782999, 3.4008772926365514], [2.1159557719535944, 0.14941294251093618, 3.4220936689971087, 2.2029678332192471, 2.7936185847489359], [4.5328887800055897, 3.3280232112463266, 3.806089860522984, 2.0579569140301408, 3.6903555852542604], [1.0234937123901144, 4.6939505151497274, 4.0231796941582498, 0.80285394291365508, 3.3986907349197715]], [[1.2885735157069735, 3.8910525544460968, 1.9285653571452805, 4.2137843283099894, 1.1916810842354508], [1.2480318898278373, 3.2730623856508116, 4.3814190622323146, 4.9403361329646476, 2.2809189975854651], [2.0448093449524598, 0.89175149091585415, 3.6306623422975139, 3.541874597609961, 4.0962419172122582], [3.8911380636187838, 0.28280058350326664, 0.93969194994770955, 4.9495744034910985, 4.3313734488545306]], [[4.3645357878253259, 4.6292903099355112, 1.5696809166405057, 1.0403871522284887, 0.5695879636261314], [4.8483427975038778, 2.3331786391980773, 4.1287843422266075, 3.2106838907831254, 2.8048000600008494], [3.3636738940367374, 3.240272892580669, 1.9433280101589097, 3.3607296220911698, 3.6077105768218964], [1.1369608301148519, 0.98031505949705888, 4.1917637432935377, 2.0590588098167117, 0.26120540029054157]]], [[[0.025582225898582306, 0.32971103919519523, 0.029308264401063218, 3.1009882419909744, 2.7122300482485926], [4.7470291151822908, 1.9874521248647818, 4.8491516726300912, 3.9386450963255539, 4.0514648151660158], [3.7819916424134061, 0.98954626667802992, 0.11215150876908732, 2.2304678842363055, 3.5822942288137152], [3.8173472355434184, 1.2037184373609793, 1.9652990105370376, 1.329947663642441, 2.2066721148095749]], [[3.4016942062830284, 2.0150502876802774, 0.35375294156471293, 3.717217384780422, 0.91380222499359154], [1.857506361766514, 4.5458026596043632, 0.37564123715554865, 4.9426925462093632, 1.9677846384923185], [2.1066980447223878, 0.15600041693044259, 0.18412724638003597, 3.1402643391445721, 2.8345873479828971], [1.6709546753255076, 3.7101487633870587, 4.5079273156309423, 2.6666673563098815, 1.5266249330817563]], [[4.0173267908021213, 3.484749755510042, 4.0101700136860936, 2.789866247290071, 0.5409988668044361], [0.28104849314850783, 2.6755938175357188, 3.1130658618146145, 0.91706551108902434, 2.3594616216776001], [4.0595376244047916, 2.3073920240121839, 1.3291134767974271, 0.84114154737403268, 1.2431908033533396], [2.8111627324579165, 4.2139947166529339, 4.5633620960621046, 2.3910947904771236, 0.64615627861004621]], [[4.6608782780144811, 4.097447851093877, 4.2106712548799772, 4.596428886204655, 1.7565276516057595], [4.8504541313399745, 3.7666336449294824, 3.7260947367552593, 1.3987553989597814, 0.44760205851350643], [0.6124042920746402, 2.6575605076574615, 4.2317558632635661, 3.6487834459680433, 3.1761207584149576], [3.7523576013300777, 2.7651291690431377, 0.14957049809451606, 2.8678441069795024, 1.6105189363457044]], [[0.9951214292430739, 2.021720195709134, 3.0317272792452563, 3.0797455077422948, 3.6352835384574704], [0.34090941581115897, 3.8946450285902321, 0.40904890571186914, 3.902323434329007, 2.7843779319743733], [2.2347933777815463, 3.5471054827602035, 2.3765838161063306, 3.3960146144993062, 2.6209812762961575], [1.268067515185757, 1.8759334259654841, 1.1216613528441555, 1.3484040707660938, 4.4302226316900359]]], [[[3.6664996289812555, 3.5406691854562289, 4.4119891666109421, 4.2555226059123497, 4.0365348324804726], [0.66927651579794079, 2.7735907740038304, 2.5751790024675216, 1.1699220417589282, 2.0274579377223465], [0.017866470635792268, 0.96601986683160901, 0.81889967177811762, 3.6528369801422378, 4.4428933658704235], [1.100602472888788, 4.7055739793248215, 4.6591599088783839, 2.9041024078732822, 1.2879388862953816]], [[2.2387629477706001, 2.4774533237621568, 2.2795039085915145, 0.9151145810328295, 1.4448871971555546], [0.7928027475629843, 4.5139740981655825, 0.72363320571170708, 2.1773965874924941, 4.9174102128838717], [4.2304016144162855, 2.3559245876475772, 2.5628613671764486, 0.95594784612335781, 4.1633346543643253], [2.5231837390725183, 3.416265706045519, 0.5564520349440476, 1.5468131137008676, 4.3531136052253547]], [[3.212801631417701, 2.4595830857578411, 2.3470027705182259, 3.5671322853990564, 1.7638949118914593], [0.16082536390395169, 1.8795954727868955, 4.4469126731328901, 4.8268050867381032, 0.73816676107904133], [2.6906862024550895, 2.8202395458354532, 2.8083472069853554, 4.5758834379074402, 4.9396950023005646], [2.8504884771922199, 0.75747558524664538, 1.0879202284566909, 2.777628535283585, 3.4702713457066592]], [[4.5001718081162716, 2.2003420662307129, 4.5527951853565236, 3.2520417581392707, 4.2919824892545559], [2.2340927213130612, 2.5774078152723958, 0.28542450581540652, 1.6126513224137506, 2.4487174607097644], [2.4734903475057486, 0.71058190719796244, 3.2266268386056911, 4.6123826836517372, 2.2816477586037474], [2.5823773222535111, 2.9501118710383643, 1.080135522311567, 3.6544226622109046, 0.20840622500946576]], [[4.3479990727791931, 1.1337269059200576, 4.1676547940865243, 4.9118226036831762, 3.1472554307471103], [1.8797509284058882, 4.255067006364019, 3.7310889654550556, 3.3760458995912304, 4.4375175675353633], [4.5409505607951282, 1.6348085214916637, 1.4737442476030713, 3.7282993968098097, 1.7241482702250266], [3.5171186490543378, 2.2139265604614895, 1.0950338024424586, 3.8058647996700778, 2.3493372841560634]]], [[[4.1277438071635499, 4.582060461773513, 1.5574307976016639, 1.0484479458415592, 4.6126185075778006], [2.2297829097245345, 1.1961770003702521, 4.6109333564929766, 3.1151661503044652, 4.3703326416806858], [0.87938461699660431, 0.6555186745498538, 0.43298787857619364, 2.1071152461244589, 0.24796632241129721], [1.3123436590477957, 1.2477778705000098, 2.6026229230427771, 3.967835021199281, 2.0902098782731304]], [[2.376172278115634, 2.8072983292146647, 3.0410547530396035, 4.1991690687139336, 0.57734069447628711], [4.1383104175284329, 2.9481447256013404, 1.1533216434686149, 1.9070059629159055, 3.4143309933041674], [2.2546902047124067, 3.7191527281039147, 4.3820572468881052, 3.2911258119389304, 4.2652448290023939], [0.74047376662305142, 2.5193336283848491, 1.6831173542901734, 2.5232863158482672, 0.30832642578084718]], [[2.3115736366255173, 4.9624467806245889, 1.0039242651411378, 3.0664084944973706, 1.4404602165141045], [2.9056557839360999, 1.5164807901654782, 2.5536280028516938, 4.0290955814294627, 3.0018934543951037], [1.5633784324106563, 4.925446452867277, 1.8040913728558106, 0.69868442863816116, 1.9023276343203706], [2.7094517865119081, 4.4496096287267086, 4.482030674135741, 2.7818416366398147, 4.3343543378784268]], [[2.4559732123577724, 0.25208037717009291, 0.18263882167367368, 2.0054127459894771, 2.4832504238079278], [1.1881762121776023, 3.5324563018865853, 0.40105406816730521, 2.8316935346386525, 1.969447738175528], [1.2019813043188259, 1.4925159160766812, 2.6213733385658409, 0.36295019704244824, 3.9358304453361281], [1.2861203140022202, 4.7033578947020409, 3.2528013656252832, 3.3996883721239826, 2.7571853845047452]], [[4.2760995653299245, 0.040936991441413967, 2.6827698947455545, 2.449684627626215, 0.022059099193226201], [1.2677495334427056, 4.9966747787608616, 2.9517969078978314, 3.3195218862404023, 2.1795073425319234], [1.5514032713959269, 0.95886914042362259, 0.23311150858159796, 3.6049103939168199, 0.43301221814119589], [2.1441710004237104, 2.2798539797259965, 2.8950500254680036, 3.2231153792228606, 1.2734773719702843]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[2.0825991225522058, 2.2182518556567379, 2.2389530750823794, 2.3302973636663369, 0.8657146979730701], [1.7429315780783394, 1.4930885763927153, 1.3030926129087579, 2.4821694002086665, 0.66793717338994274], [2.4079751330012251, 0.78200115534436754, 1.2191477548315217, 2.3054585692509471, 2.2243426307956611], [2.3308754200736148, 2.3607485933032324, 2.0363535267139827, 0.9983456041620008, 1.7877199580618148]], [[7.1902849768784263, 0.05633737140038049, 52.18123219434375, 1043.9813864145319, 1.6337110735908316], [260.68837103210626, 245.72128804837214, 1015.115087065842, 0.00019931421696853612, 19.568706193095583], [814.70537458951344, 98.966062052534141, 1.1285671595712172, 3.7125178961869216, 49.994436836986729], [340.97318517270065, 1.8762387602843249, 736.30174956172084, 0.033257165524402395, 543.57673704444107]], [[6.1158417589652547, 0.18043260986713749, 6.1481419644755784, 5.868551871915896, 4.2698719284110762], [2.432310137541311, 0.10493134482739708, 4.3014798021047254, 2.5513738437857487, 3.3815088059334997], [6.0033902910160659, 4.1616149509315203, 4.8796797545837576, 2.3534487556056733, 4.7042175481293347], [1.0279217019183062, 6.2571841773745964, 5.2114691235092767, 0.77074066988108114, 4.2666165032362731]], [[3.0580775237335773, 399.46864912097556, 18.094174714659758, 567.6097376700526, 2.1665751101541697], [2.6560665683809308, 186.34853380275518, 674.13323666089957, 1144.5312045955732, 37.916798214761016], [23.420893839921842, 0.60344194036697707, 294.34780975028798, 263.90917317971815, 501.04913162092544], [399.50735385788704, 0.0038167270824231163, 0.76014901099947274, 1153.9971805993519, 640.83992252092435]], [[11.197692678401438, 12.332723323832436, 2.0941967685997516, 1.067062430875436, 0.39743032226207914], [13.303832921370256, 4.0107171876787264, 10.223332184197579, 6.7690414927650355, 5.4237573295074242], [7.3058421044957358, 6.8716130658418129, 2.9720002846523736, 7.295361085318893, 8.1947913667291719], [1.2342138193805225, 0.96793166200679293, 10.480234804217339, 3.2676494269627572, 0.11071021704620403]]], [[[5.8434988612321468e-07, 0.012982844189951895, 9.9512054427754114e-07, 84.01332698491882, 49.726023749240568], [445.0041355032663, 14.719988835704209, 483.67781087062718, 214.25678605240225, 239.30736060151804], [182.77769336951289, 0.95969048069629082, 0.00019043421209457643, 23.123788986210869, 147.80274034183009], [189.55931650543559, 2.0666909978785051, 14.087951972045129, 3.0538226713384407, 22.172827870464918]], [[136.54954209329733, 16.672982665474567, 0.015402020840038135, 194.98334483456702, 0.69627744931247193], [12.023340803403277, 437.4884672866329, 0.019601431424874725, 612.29465690177165, 15.157062242912978], [19.933705993723208, 0.00057489687622042725, 0.0011186939755757242, 99.041725468521236, 65.644894256943644], [7.8600800747588346, 193.49856240355979, 423.03252807986894, 51.367983355610647, 5.4685202882578601]], [[4.319470012732971, 3.7191528815335166, 4.311374080046102, 2.9431951914488224, 0.52394227078166999], [0.26304923804992852, 2.8164932601432322, 3.3029836902065584, 0.91293459667793531, 2.4674812654921108], [4.3672352653373299, 2.4102214794829595, 1.3489796859145831, 0.83358762599558422, 1.2573830977971776], [2.9668383837806633, 4.5422359278158106, 4.93928895952007, 2.502299846822047, 0.63160754224378535]], [[12.216812263383618, 9.9077196979255309, 10.356730061486031, 11.943314313384056, 2.4993078101993045], [13.035036918009919, 8.6402331277754598, 8.4895346644379828, 1.725768430062437, 0.27060461666032132], [0.45052076638807137, 4.9003628538072839, 10.441189635732107, 8.2049785410499059, 6.5479679713032626], [8.5870472407613079, 5.2269537013402472, 0.045526399528322256, 5.5463257999028928, 2.1703787399240859]], [[0.98928296444414754, 4.7159718278718481, 11.515162590447375, 11.920827349343067, 17.17868971642941], [0.093389955617063536, 19.995467144780157, 0.13952590074271926, 20.082424791401078, 9.5462919774328991], [5.8809475982743384, 16.27400595416319, 6.7345417052655732, 14.785756578388298, 8.3554283583329205], [1.6875067053721986, 3.9990654277091777, 1.2878239449672197, 1.9320677071849097, 26.55942693788954]]], [[[36.021800999461256, 32.713508459915893, 60.022262330520896, 54.331625411716466, 46.963233122024441], [0.33030042413148702, 16.679886030973751, 13.591535503906877, 1.5417721809902636, 7.0269799616205182], [1.5066404088166292e-05, 0.90903788955745723, 0.57628192353894658, 35.652722107420189, 61.189236929998678], [1.3026969340598149, 71.695841846751989, 69.76186236941794, 18.935718870307923, 2.0098382655801768]], [[3.978598132190919, 4.7328033468618473, 4.1034618879796758, 0.85899215593100886, 1.8787605486302594], [0.6717721893314299, 13.230323203216702, 0.57449793148691797, 3.7935643245691262, 15.320500402258686], [11.838292881274269, 4.3419933222057114, 5.0157996467975678, 0.9257090152352806, 11.518530836697748], [4.8834783710216723, 8.207809868197705, 0.36626619872387367, 2.1115286212646596, 12.432764266351723]], [[43.143798633757569, 18.226204993351921, 15.669875491062571, 60.459806661486105, 6.23706643597545], [0.0027551956113942896, 7.6555324384999937, 123.10265065786194, 160.36014580845, 0.37561382844003516], [24.349666515821738, 28.337777990393189, 27.954162949604964, 134.99431328983309, 172.77502255246611], [29.329870796557113, 0.4082366925279079, 1.3123244348645597, 26.979866219087324, 55.322809102813913]], [[12.474570405310306, 3.7553454323163282, 12.720293567920427, 7.2331118741722111, 11.521529244258414], [3.8524959998373132, 4.8967381671767418, 0.12200895403296103, 2.2295818969258878, 4.4934905678887711], [4.570025891420479, 0.56367610011687663, 7.1385186234133187, 13.000868529021293, 3.9910782470735624], [4.9125898684652753, 6.1421754119632697, 1.1380772566807495, 8.7969055672333702, 0.071982829521252498]], [[10.315138107608284, 1.2205299564280467, 9.6441332039251702, 12.518552336705966, 6.1746975618336828], [2.7240562395274992, 9.9672802825897904, 8.090208971575592, 6.9024933380431239, 10.654377598843148], [11.051384505623075, 2.1824405450795608, 1.8510595960016742, 8.0806069840641594, 2.3748316169778669], [7.3660549335428742, 3.5322354319737368, 1.1550558063779315, 8.3491640244716372, 3.8813831799596135]]], [[[5.5864978896137449, 6.3411421732531874, 1.7119037848373098, 1.0590896015899895, 6.3924950012243533], [2.646067627934634, 1.2428000663603131, 6.389661206347772, 3.9702440360275322, 5.9873611597127772], [0.85558580172985721, 0.59901024640729106, 0.36214267153145557, 2.4704788671722446, 0.18412927846519112], [1.3907382897500686, 1.3081521311584663, 3.1921465432988736, 5.3249779128171815, 2.4464480631672525]], [[27.872084674426905, 52.914261104525494, 71.965425111437241, 248.84479097967187, 0.12099243354019522], [235.26183680173608, 63.872473947601748, 1.7305576533351292, 11.964459834142902, 112.31685819069638], [22.779427211488965, 156.03931930383817, 293.16594781509059, 97.516437364250692, 264.23997148379192], [0.31498718601931158, 34.90252115679754, 7.4022037003026977, 35.113531911574427, 0.010848174749639722]], [[3.5114627404009666, 11.036457458660825, 1.0058881250125118, 5.3634657380474877, 1.7281809280463987], [4.9475563377755041, 1.8666794619534528, 4.0767916765302257, 8.0758852154933933, 5.1952073596372577], [1.9538759022887358, 10.913338838982044, 2.4217246202190141, 0.58422674561571863, 2.6220520081732901], [4.4553087747829485, 9.3716787779595538, 9.4742213638943866, 4.6349229382885264, 9.0101670945619983]], [[7.1817638525212075, 0.048624283754982164, 0.023976343531443359, 4.603598129096004, 7.3579442458434086], [1.4598358912872458, 15.943862471971347, 0.13469310907649518, 9.8148283420762201, 4.4243803758092977], [1.4973111582869614, 2.4077649857632979, 8.2858808269408062, 0.10819660000978976, 20.213082557375806], [1.7369461957369565, 29.881465960046306, 13.304504576293832, 14.658415231039342, 9.2570622062927157]], [[2.0337234610748278, 0.20987637175862464, 1.6194964922263329, 1.5491581774037391, 0.15515897081091853], [1.1228866898472771, 2.1944891469741545, 1.696898627408171, 1.7970744895356925, 1.4631925311098004], [1.239300390546175, 0.9796900890580128, 0.49094153432430154, 1.870962365672221, 0.66437900159247276], [1.451554585960001, 1.4957251261918962, 1.6808823346733421, 1.7713848483370849, 1.1253623563970916]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_array_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[[[3.2043025682946764, 1.5580627322192382, 0.53089357952615579, 3.6424535189241518, 4.0386160761954049], [3.4808120883735101, 3.1918662005862153, 2.7561750445997832, 0.02419393204643927, 1.287918216322081], [1.4691009565840163, 3.7496669683953425, 2.8438998468630197, 3.281062933286103, 3.2572467790698956], [1.1945563533260743, 1.3945516918315197, 4.8716604545689126, 2.5916112411820769, 2.7747990753694185]], [[4.2542355955569722, 1.9255876983377678, 2.7541424014553, 1.19809499552218, 0.7775943170048657], [1.2713856509725405, 2.8876408074042534, 3.1637232560062087, 4.5583337053570094, 0.73173355347356395], [4.4828920432980581, 2.4404674592950202, 3.2522668196646123, 1.7881307905260992, 1.0927935295446538], [3.5309973221445445, 1.0645366584447646, 0.50154603072926485, 2.517651472080606, 3.0216920387784811]], [[1.146447882209765, 1.2353048762923069, 3.9351361970919001, 2.6238265493722919, 4.7934298104613609], [4.0547491840172114, 3.0968776294978881, 1.1837065920621959, 4.802382994598914, 0.60981468391371252], [1.5464975276020083, 0.55863017361278133, 2.1828334767925783, 2.9906449241529969, 1.7158755001554034], [2.0316585491162495, 1.4006347053254411, 0.33100955637906843, 1.8754970349482387, 1.1552064830079696]], [[1.1496177212615712, 0.094397111872452644, 4.928890981295905, 2.7694281849321465, 3.8097965593480652], [0.83100007987517222, 4.5459335356513026, 0.12215981939342874, 3.6687510163148844, 4.5662154829060437], [0.45718777108515585, 4.4512071641888298, 3.7843582411172538, 2.9665900636483085, 0.51968247751101837], [3.3435193615004333, 1.5351271464348004, 1.0293108677201761, 3.6981454395049798, 2.0498896350162439]], [[2.1760000636758003, 1.9641047402695195, 4.8608869359795381, 1.443668378200057, 2.3565777410926705], [4.2216806565872798, 2.9877047436285111, 4.6564418130870129, 2.491195098993789, 4.6991129882044484], [0.60265732132496597, 2.3033374167483327, 0.10276662449747931, 3.9482030204886596, 4.9268073359816968], [0.065406115338691007, 1.0192636654244258, 3.9349073134693691, 0.91335379108780912, 3.3631246959685663]]], [[[1.3638188424374964, 4.2311315413393569, 4.4023622748701419, 3.5896853513782339, 4.256041087669856], [2.0659063673032603, 2.9012056485687485, 3.79029991336968, 2.4525666792180649, 2.8016353936246436], [0.13431672559564817, 3.0847024177296083, 0.74210626863926521, 3.490113529244518, 4.5510101031307046], [0.70451347657171948, 3.581187934068474, 4.8845827797145134, 0.1120200363142551, 0.13165431690241769]], [[2.3566660635224292, 1.8724782146276868, 1.3008088190167553, 0.54671362398016798, 0.17871583271012903], [4.5995854847496958, 0.74258953591987553, 4.1039428135777216, 2.3337847838912626, 4.5148485519422765], [4.4840019630516812, 3.8735269054946624, 1.4827020183782815, 3.9436863728958471, 3.8953825347050963], [1.6798374907954703, 2.271971796174332, 1.4772289014867261, 4.0920429194989643, 2.8023637978639058]], [[4.7655087551860058, 2.1623879579096452, 0.65146122358532976, 1.0133057620059962, 3.6207763109544135], [4.7148740925723853, 1.3232211397536811, 2.2996918722699311, 2.3001521375067435, 4.992250849326239], [3.4310429008998748, 4.2574247113716366, 3.8895392346408095, 0.57567810588523893, 1.5383444595771119], [0.57175405791522349, 2.388482952546195, 4.5061341952087615, 2.0311244371662505, 1.6927741228680206]], [[4.0486953201896858, 3.4470861776629516, 4.2069507837930864, 2.9592939737080552, 0.59909673412313424], [1.7175216027592088, 3.699716110615952, 1.4852296597315122, 3.6201954587570744, 4.6149714567718547], [0.69848485484243472, 0.092076583926633856, 0.96197711038294842, 0.36201067963405875, 3.3713052650340649], [1.8721084882978394, 0.64768306750596205, 2.8306673460137368, 2.6567661710364203, 0.70160125814917107]], [[1.187038857011065, 2.9547593447124014, 1.4762476722481961, 2.294229630084581, 4.1702129074897956], [0.17350363321143306, 3.6667764899982349, 3.7182257377560863, 0.86488687456925073, 2.21725056217144], [1.604617155899428, 1.108246295904276, 1.4120251991786761, 2.2813138723483481, 4.5556502918949091], [3.1041762186517103, 4.7648458031588206, 1.1658320728725571, 1.8647816293008033, 3.1926566678398642]]], [[[1.6109384716421988, 3.5948487966998139, 2.2717733673124845, 2.9497845914065408, 3.625867621779129], [0.38991009541707039, 0.63730455061325098, 0.3127668540996032, 0.026126924900049864, 4.2704611790573628], [1.1568559895991795, 1.5644330088419056, 1.0030014535009499, 1.3038456949137207, 3.8127659017367659], [3.8086237876363014, 0.77584042636146811, 0.44000215493293632, 1.62487876894494, 2.696212664043165]], [[0.61980216648467001, 0.46626960220634134, 0.61361520156543903, 1.6159120553072137, 1.6034342360059988], [1.6455080424312318, 4.5975135284176565, 2.2731334251531803, 2.2965452453266035, 1.5768064248471247], [3.3612500868437452, 4.737839146181952, 0.044114804300228308, 0.011037364013977756, 3.9545278002191155], [0.028580786422913798, 0.25855196586322232, 2.9858723976231278, 2.6669662945718189, 2.7178389622517174]], [[3.91097626964349, 0.41463881715962941, 2.5910381251349377, 2.6124439265277615, 4.7830820176053459], [2.3514523697033747, 0.091574258832147648, 0.70435729084526166, 3.6874232416013459, 4.6956440612809054], [2.88668618616469, 4.1899526365131825, 1.1157336969260452, 4.2555626584953323, 1.1193725435159438], [0.42345745711365568, 3.954999419012537, 1.6174022749343484, 4.9430835933146291, 4.0784452022033895]], [[4.4850090892585364, 4.5640413206990242, 4.7424172322381342, 3.0095767241859761, 1.4851947321382406], [4.5111063214991569, 0.18272290125139093, 4.9082488896308476, 0.99363215389754223, 4.666569672236065], [2.9288598793602416, 0.73095751701447131, 3.0636711960452501, 2.0790893627661444, 1.4578236437485237], [0.90561049616309586, 2.1701743275952121, 0.23456244180079328, 2.3205115404123156, 4.20699649114803]], [[3.7338551262966879, 3.8515002934368425, 3.1358020594059379, 1.0590099797901922, 0.80511403046201302], [0.28286018664226309, 3.9453807431624264, 3.3494773973501126, 1.920751210795923, 3.8348002371940142], [3.6043961896293335, 4.5290459436890513, 0.20378155707573989, 0.96349903452073005, 4.5816288393746953], [1.8413544657545489, 4.8787738161719378, 4.253484875236258, 1.5388758440904204, 1.376315477194743]]], [[[0.86581781362082366, 0.095047548610343022, 2.0442104462775688, 2.2414041402701108, 3.8934373312107549], [0.33441716320186182, 3.3302774366859129, 2.766277206908434, 4.6655197675131745, 1.2814182850459046], [1.7029317437383062, 4.9802248172435339, 2.3862797852066997, 3.4873534373407851, 4.8948649759828751], [1.9995552236572405, 3.0472627443904323, 4.6078689044120855, 1.0296770125304131, 0.8460501743858615]], [[3.2025922864090703, 0.53347959834357972, 0.56385642895526433, 1.2995632769627785, 3.883985971596478], [2.2567575344152657, 2.3965077250649758, 2.0699406387372328, 4.3109360261771315, 3.5960065747514469], [3.6489592126619974, 1.6662776612279409, 2.1818606498894164, 1.9980284853472035, 1.5667768573087444], [1.1015240396156649, 1.298563404704721, 3.6284481332080492, 3.0448898406725551, 3.9537170114038718]], [[3.1644945568021172, 0.13881462396288574, 2.2457723349319414, 1.8804847148109471, 0.57438158491247582], [1.0250001211283772, 0.89672712768653096, 0.71384438012566798, 0.68067141569318435, 1.7203191520454084], [4.7373886796874976, 1.946093509066996, 1.0807658346617224, 0.81748338505365759, 0.6833664649141078], [0.10733379672396112, 3.2611140502517317, 4.5493153382594631, 4.381303271571074, 1.6590638079942446]], [[1.8672090884711763, 0.20604483973691634, 4.9410613405681278, 3.4243452436036632, 1.839096947341569], [1.2491421496379054, 4.2844962017548127, 1.1787566753087586, 3.5720490286814197, 1.8581766423710042], [0.8902035273057578, 4.459714604145355, 2.1745000495239228, 0.38835809996930565, 4.6462975514870024], [1.505654589851888, 1.698043646905425, 4.9178989836071096, 0.41678269673525498, 4.8403161078358554]], [[3.2123219502581133, 1.1942649524774118, 1.0473827409075025, 3.2119093188022836, 1.8749361913506506], [2.7161321446876538, 3.9078134029188774, 3.8550069833120495, 4.5394707574460051, 0.7090160200501131], [3.0409609114038254, 1.1892964026517427, 4.9939893152020414, 2.1999584236330985, 2.4757436967477315], [3.493792258397574, 0.19553922677303837, 1.1495823312922084, 3.5528625180599582, 2.210299333829906]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[3.2606247657173939, 1.1693101895546716, 0.42167117796423109, 1.5433265895668391, 4.0912926710423969], [0.27501520403964064, 2.9580272119062823, 2.9162723510094186, 3.1603400299838813, 1.1714025094644873], [0.59082808546511434, 2.3842820909942923, 3.8283884404181214, 3.9275108476777136, 3.2027683190647633], [0.17282699731379714, 4.7415774445861691, 1.206811744686354, 2.2530206047130936, 1.6935397503590677]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[44.133781952515989, 6.3060133019674289, 1.8728720619110948, 74.075465359118354, 118.3109536803818], [61.193276744915032, 43.489813696943003, 25.986447822577347, 1.0290080332643301, 4.5823629007529911], [5.6766318539505658, 84.082795129952927, 28.825452366250431, 48.325049050637354, 46.983725657992487], [4.1036164219431601, 5.1978594371824016, 316.68487372249865, 21.393221247295468, 26.564806000751833]], [[1.9453112270930724, 1.3514645570113564, 1.5384669157074653, 1.2061082124452023, 1.1293323668680935], [1.2200142106735015, 1.5709294178194888, 1.640253147836034, 2.0400762363679767, 1.1212603523069533], [2.0161445444968487, 1.4648066598711524, 1.6631277907079214, 1.3227179564990956, 1.1864055618959164], [1.7372395565373242, 1.1811734929159139, 1.0816079943667249, 1.4825979501565667, 1.6042156066021733]], [[0.37157983843712983, 0.34413468832315308, 0.033436536621518513, 0.103752031034983, 0.015934512999894089], [0.030155275228220667, 0.068958866398889582, 0.35981490825176393, 0.015811792895320247, 0.59061379669860115], [0.26304078389316804, 0.61730403286878288, 0.15183801297005972, 0.075584048269879092, 0.22724897474940722], [0.17301153235351854, 0.29834959523453114, 0.75138545287157832, 0.19798837611907891, 0.36878006609096065]], [[1.6468516185218438, 1.0418132478238664, 8.4896178416751553, 3.3259862531738165, 5.2237937935675207], [1.4341959075825357, 7.189799980627618, 1.0544402502652941, 4.913657563424489, 7.2533577870260659], [1.2194405125740546, 6.9002508825568318, 5.1664469530886867, 3.6230735575193447, 1.2529629711537007], [4.2668996798503382, 1.9467345828386919, 1.5630817821416929, 4.9767347730218274, 2.4339842855130605]], [[21.4490761563282, 15.913224968610013, 942.29078491377356, 7.6440536215668118, 27.662851658823048], [382.89512735567786, 67.306756335559015, 706.46888514217642, 33.439825171968749, 750.24281971954781], [2.3374495107830726, 25.663819601600633, 1.1557897620276407, 260.46517402588205, 1033.9964222286701], [1.0965272315389509, 4.2038501303959137, 255.63160383077806, 3.6211386265478778, 114.22556611999663]]], [[[0.17194284535638368, 0.0042446752366060696, 0.0034028680161694537, 0.0097155406603081848, 0.00411035277757565], [0.069464584988358793, 0.023629716074730453, 0.0074988344251045716, 0.042168187448360823, 0.026870880605424863], [0.84080682497688941, 0.018645853597161356, 0.38365697278492261, 0.011048192895927513, 0.0028087086806511475], [0.40273487814945558, 0.0098227025664790763, 0.0018259606398453687, 0.8653598026246041, 0.84370163333182113]], [[12.882341800185953, 7.619760004660467, 4.0990552291799256, 1.8092696599785694, 1.2138794077428832], [146.69480054656134, 2.2374909070113866, 85.697065317839332, 12.566596935715884, 133.81458211207655], [129.41202471981381, 66.748199958717336, 4.9929344054791409, 72.02525745291986, 68.349226956178683], [6.1831117370710942, 11.751779512334315, 4.9633855346991771, 84.598190518891201, 20.889084930193075]], [[164.11273626622346, 10.119024407594582, 2.0082485866107125, 2.9581008222467635, 48.199052618566], [155.45542182488629, 4.1216427155388562, 11.720910733996448, 11.726686167049872, 209.18877606062924], [39.340971942960344, 95.273156881315614, 64.263896704984319, 1.8517883556683379, 5.1887853131956678], [1.8440272782304994, 12.889436930018174, 124.33050789557331, 8.7927286978110057, 6.1213744430108488]], [[105.50424802822964, 52.798978002603427, 126.57721984482615, 30.120297606788327, 1.9924567122466565], [7.2161178609978469, 70.611142042065808, 5.5235651357222206, 64.436816682832045, 202.42179834844242], [2.2338628166268286, 1.1117670085365852, 3.0250495206820927, 1.5167324447732491, 48.389956521062246], [8.6209655881267739, 2.1070222820948747, 25.976472998864502, 21.26551410836127, 2.2418877881147283]], [[1.2065820769609139, 1.5959164047757912, 1.2630696337284644, 1.4375656444117808, 1.9342796994182485], [1.0278287623937594, 1.7862000442806061, 1.8007978999743004, 1.1466293322687966, 1.4201648189096718], [1.2889826364326007, 1.1916352352840438, 1.2503016814644732, 1.4346312718206116, 2.0558962417083197], [1.6340900934544758, 2.1250747487756394, 1.2025408366776638, 1.3431421137611832, 1.657124537747088]]], [[[0.42838863902311947, 0.15081249474388109, 0.30255998187840732, 0.21176762579499617, 0.14837075851553946], [0.81449868604009812, 0.71507424989814672, 0.84824364986482781, 0.98634530607077664, 0.1056899359742389], [0.54401821519979876, 0.43900174906062517, 0.58989563540080825, 0.50352446928175265, 0.13447290764908471], [0.13476633866537452, 0.66479892259783613, 0.79330904403191027, 0.42525756189415276, 0.24199734926516719]], [[1.7135117259742065, 1.4995135521913989, 1.7043248735488756, 4.0717092094045153, 4.0278023771405227], [4.1777748009908455, 54.313381164728376, 7.2074934633702439, 7.3556132479219309, 3.9356815058121435], [18.552113349448319, 61.356387727341769, 1.0390753885768806, 1.0096364789945289, 31.064993344412731], [1.0251447171956416, 1.2518911197808824, 13.388802997679695, 10.148458919216004, 10.607115251281039]], [[190.61704209842529, 1.744781561521715, 32.405346814923568, 33.350058375054061, 614.62939608545889], [23.492776440550124, 1.1308090036536751, 2.5742550158740678, 141.19741138609427, 546.55717552386852], [48.193028940224018, 277.21033045098562, 4.4718846523565965, 302.73393283767456, 4.4937830170136515], [1.7655599601715504, 202.22177886278877, 8.7694306632987367, 761.89795102631444, 238.67052732247652]], [[461.97865347775786, 514.7262678484617, 656.98062744091897, 61.382119176234241, 7.6274614734847699], [478.76977823234165, 1.2839817585264204, 824.28241645562093, 3.8934459054467347, 592.22973508662653], [54.965046110952933, 2.7181572297905148, 66.096786387347038, 17.187913226936015, 7.3471411433429274], [3.4517469299722965, 19.468761028315694, 1.3783440202809301, 23.914222593236417, 315.82738803990975]], [[77.189649517067508, 88.518022616056555, 38.479372307778647, 3.4304932954670142, 2.5527347598158436], [1.3899303689880025, 98.739441705138319, 49.345359371039613, 9.3538146986414805, 86.813926448281805], [66.391773845249688, 194.78177699611058, 1.2677008409014128, 3.0695404033736886, 207.07626328180038], [8.5280881555256549, 292.64741316850007, 141.33351824755508, 5.9970962261586926, 4.9632094461968155]]], [[[0.21873109598217363, 0.8463243766588181, 0.027638701603881651, 0.019551471501109259, 0.0010756933470721446], [0.55596087384668391, 0.0028909132671869203, 0.0077807650900663982, 0.00027737146314932634, 0.10545363249573018], [0.050315821339026491, 0.00015963727039723907, 0.015161003076288275, 0.0021942299517462524, 0.0001854435136457469], [0.029892501626034969, 0.004751187667765255, 0.00030691218254154639, 0.16405375451481446, 0.22645460063587408]], [[146.11853195982187, 2.2939875134410275, 2.4050463112343419, 7.5579586534825927, 421.9634755266253], [33.526944894697444, 41.672957925190168, 25.068028073548486, 820.08743473245204, 269.53986504701157], [292.69473097518579, 13.374425713971327, 29.837983201646953, 22.413676780724082, 11.455664237058038], [5.5532249515981515, 7.5462063236364401, 283.49865132561547, 114.31694606821578, 470.33539617223335]], [[1.8127914417892568, 1.0264380912502677, 1.5252596929342104, 1.4240390040368205, 1.1140180857286328], [1.212496590812016, 1.183609328951357, 1.1436099316277686, 1.1365006623100142, 1.3818027215253481], [2.4364537742467101, 1.4417108128113152, 1.2252739850752887, 1.1661084912540951, 1.1370765828724172], [1.020381745446038, 1.8460174927715185, 2.3518193391868465, 2.2787019891744693, 1.3659826598553715]], [[4.5570765026684379, 1.1821849919008778, 55.33938288349092, 16.143203066895815, 4.4541961209801642], [2.7583783450724035, 32.465418325371544, 2.6050993547748886, 18.200966894372474, 4.523764489035746], [2.0607855140158917, 37.431044973846745, 5.8490840963163535, 1.3708792681907787, 43.556393786921973], [3.3973671847055593, 3.9720128495357758, 54.307953710820833, 1.4028990339696505, 50.991161343141478]], [[5.4320588790574451, 1.8760409818388157, 1.7363463068863336, 5.4308781708967953, 2.6852007885054503], [4.1825248143682554, 7.8359148883307421, 7.6209277536141657, 10.929744353626413, 1.4528460607918079], [4.9631551563206084, 1.8711367987967016, 13.88678644282372, 3.1866939509992505, 3.6850107269352907], [6.3003340814144257, 1.1085070884109074, 1.8323951908499061, 6.4994795606061739, 3.2041018197379119]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[2.6228528962108562, 4.3538421169222667], [3.4419577339543626, 3.8820382139112639]], [[2.3279274068436431, 0.81128413704667568], [3.5338795020346785, 2.0358589667291671]], [[1.5149934461444536, 1.7363149375416465], [4.7146457189848903, 2.0478938989912789]], [[4.6014457666242121, 4.8713547064514167], [4.4936909810112846, 3.0065627489029776]], [[0.91191451256540823, 0.25369457730527811], [2.3522337890336056, 3.1357900331136883]], [[4.5271373654648164, 4.5187610131699785], [1.6652318155893706, 3.2480218888355536]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[3.8980125340975555, 4.8278037779989544], [0.075772920266793567, 4.2679786901440373]], [[3.5439986232693808, 3.0784006099999734], [1.0270807799343713, 0.89944557318124629]], [[3.3278047783381486, 4.134805592480153], [1.2783642017193695, 2.7979733831313225]], [[4.8058765603559737, 1.3850855355186549], [1.2331720778760329, 1.6751964875814627]], [[4.2133910390767007, 2.7970077552424053], [0.47904204742373252, 3.4366325809356084]], [[3.0741367410862379, 4.4226288129300491], [4.815309312880335, 0.28828262913169139]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[35.456419765862393, 948.28275014829228], [0.00013910240318950516, 279.60621981874613]], [[19.01871113156604, 2.4898342507049809], [1.0990292875366712, 0.80593378956936979]], [[6.1810991055650826, 11.758723811008288], [3.1830147834154849, 8.2240972793989435]], [[1371.3228034942588, 4.8885793821448411], [2.5646747836318129, 4.7170203351970388]], [[3.7120239838639932, 1.2981466649247613], [0.17707777810560033, 47.995535448751212]], [[161.43207867855446, 827.32439343398551], [13.700219277658688, 0.017598618634030155]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=numpy.array([[[1.8123117268843179, 0.26374800703139772], [3.0436481932566819, 4.5631996439410782]], [[3.4833202585664673, 3.4365574311992386], [2.2390121024875724, 3.7339347339557909]], [[1.8089163213771855, 3.4555819072141341], [0.77207836279027187, 3.4391373890097214]], [[2.7075169388746674, 1.8278169004812703], [3.2298255000996168, 4.4504989379195834]], [[1.5526580372323751, 4.2953386505768085], [4.9471331702238279, 2.4684927441062667]], [[4.2417530109196777, 1.5911410104450281], [3.4465720979752241, 1.8661417173565911]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.7169514611360621, 3.7651050144119669], [1.9474075315847874, 1.5503670397333305]], [[0.48765895359960792, 2.7107113667949991], [0.3691276999278037, 2.3240785227874796]]], [[[4.4660557652234552, 2.5464313242754875], [1.7341350541764049, 1.9113363757006863]], [[2.1953063453692518, 3.7638587950728595], [0.25444290004485992, 1.8929938406714375]]], [[[1.3825011784400016, 1.1806987289988216], [2.7820963779808463, 3.1285053765379742]], [[1.1719963442795782, 2.2941908811317986], [0.81497413605662328, 4.1550020934970915]]], [[[1.7991326602082567, 3.8733304643016586], [0.96683943238338699, 3.5355795139329094]], [[0.075328160106421266, 3.3028076899381853], [3.7820207361531164, 3.6754551203169896]]], [[[0.4569882421907524, 0.25190415171889291], [3.0668793754317174, 4.8314267002792652]], [[0.12010141116803998, 4.9556974350144367], [3.0790111682850716, 3.1760081471461388]]], [[[1.6255467812464968, 0.59541723806142544], [1.904141230015387, 1.3562447639701207]], [[0.87452899797953487, 4.5550165803475604], [2.637147548495689, 4.059709047939565]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[10.79832481225694, 11.053188569106627], [1.1921850351644094, 1.1226043659884253]], [[0.1123920112667094, 20.804301224125989], [0.010591088134955485, 46.911088251040809]]], [[[183.60898459926733, 25.94125888111903], [6.6316588357989756, 9.2646963137344311]], [[5.8158304243106365, 19.44688180819637], [0.0060327192216268989, 10.835701375193423]]], [[[1.7966030689189583, 1.3504968877235446], [34.321355652037973, 51.484588893591777]], [[1.130359195548073, 1.8986034772085549], [0.49477917321124038, 134.07614148769747]]], [[[4.904422776589545, 39.106494271792364], [0.94022207173785477, 10.057396455473651]], [[0.00023592189247829018, 47.413549859980712], [372.52860714258765, 328.03226101664137]]], [[[0.29644819177128817, 0.11757722756351394], [123.17629157576252, 867.63184614085105]], [[2.7951294427244497e-05, 2746.4767652884925], [16.05608511768423, 17.333693273834054]]], [[[7.8524946227899894, 0.11087867099548039], [2.7863829181294886, 1.6239346093818197]], [[0.62997110859961614, 186.00864553438578], [6.1079648210762425, 13.662737852158862]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_array_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[[2.6506726050247624, 1.7734446887902187], [4.5417158030016989, 2.0319244797134499]], [[2.6602608385153581, 3.9146037597720196], [1.6407989812809403, 3.5252164640034183]]], [[[2.055040014391794, 3.8725403034480355], [2.2635787539533503, 2.4744887827251767]], [[3.2257283468141433, 2.9389002307719641], [3.7764457713317232, 2.2640523165983271]]], [[[4.6887917629088784, 2.0592441348758657], [0.50543014499384065, 0.50055513025720766]], [[0.61527807927361089, 2.7899284326008709], [4.579532023430108, 1.8647908391275974]]], [[[1.8855447008444715, 2.3032442680585277], [3.4374065861272949, 1.9687598369479546]], [[2.2763980559225101, 1.0485047897323516], [3.6130215760146269, 0.054991053558274765]]], [[[4.1505608079758067, 3.705669899904847], [2.9911572250159351, 1.8568914081124488]], [[2.8304677546697765, 4.409185435448391], [1.0693693858761411, 3.0668746498410511]]], [[[1.4045082809997176, 3.1695747962125624], [3.9327357620923027, 1.3884965722835334]], [[4.7221199531500506, 2.4780435456781542], [3.4308676795967261, 0.75885004610090356]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[2.0614260079796365, 2.9646360044887401], [2.4699950226539769, 4.7061901029272644]], [[0.93621354458441597, 0.58673737707177909], [4.5318139186077522, 3.5085872849938373]], [[4.6175180142815373, 0.34655632437051864], [4.6786080119187847, 4.8064930675846531]], [[3.620100856714751, 3.0472120549310042], [0.26232653491696178, 4.2631193581677298]], [[1.9818360249984031, 2.1557948790828614], [2.1602485096096666, 1.9137455974474198]], [[1.0743468369251907, 0.84471173713709868], [1.632355884419836, 4.9392865937550985]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[6.8038675553810268, 3.6071078651414821], [139.17456387022787, 9.0993463931161074]], [[11.08342528590814, 34.454784964816035], [12.697416772720194, 235.12861030711014]]], [[[0.8733218278812015, 0.7747261651585764], [0.29912602762871787, 0.26731090074585223]], [[130.9047531273992, 84.862952000480732], [114.4623118101144, 17.147784346139844]]], [[[1303.9879060121291, 23.344227634494381], [0.58531252761890129, 0.5883441355904665]], [[2.5840927141925376, 74.058945407334377], [1325.7286051806368, 18.683885858784599]]], [[[11.310786348734942, 19.358432172514675], [46.064912603509306, 8.9678463138017985]], [[0.047539385699603989, 0.24584041868292891], [188.45992591982346, 1.0830022982494603]]], [[[17.100059834379131, 12.613479420069384], [9.951127538044485, 4.1636362549863586]], [[8.8471211986124931, 29.846190364454404], [2.0018813738802321, 7.3198725824967479]]], [[[1.1059684080434302, 1.2552055545621281], [0.51494875167389775, 0.79110666476881453]], [[10.114332939499477, 3.3679409105898319], [239.81159290049706, 3.3603702406355711]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[3.2169685569295692, 1.37056392894033, 3.2362590055492366, 3.4558904652086988], [1.4108433914987986, 4.7801461857252905, 2.3229559260779147, 0.24614508654157452], [3.0506343074479951, 0.31613189052536084, 2.0342210473657008, 2.6273691394940926]], [[3.8182893293748954, 2.9300632589545432, 2.273667450594242, 4.2334365310541946], [3.9357921273616103, 1.0705245308679752, 3.8942383936052853, 2.7017451597454154], [3.5706656461813333, 0.11663707358109748, 0.36891754509691249, 0.34312704591897802]]], [[[2.2354360005054055, 2.1847191513977062, 0.80275987283083605, 2.2093990751176942], [1.2053528303832999, 2.9561374988453055, 2.5511066297908442, 2.9666234165443273], [4.5662215241605466, 2.3068713541333175, 2.9091661165526643, 4.9957342026048135]], [[4.711227367871893, 1.2650193423376337, 1.312246392279149, 1.4198813614613415], [3.5339681744023075, 4.9417376680767413, 0.35561303337992289, 3.0242473333852971], [4.6926569059098551, 1.6965079417228255, 0.36839799852899707, 1.2073943816862833]]], [[[4.2479827356552606, 0.92566531741679381, 4.1192300133871704, 2.764896574189458], [1.7987764069784402, 0.59020466247793535, 4.6863178600607558, 3.5058880616529549], [1.2067715119707558, 4.1269117791136978, 3.7920711574426322, 0.94647098659272622]], [[4.4905335669727231, 1.2697249047530681, 3.1446823588083279, 0.2276155020446099], [2.3876571837943246, 2.4221620018720991, 2.3217352502780231, 4.8970857563391768], [0.046699040528079327, 4.4847088773046799, 3.5592796459382066, 2.5276279249980513]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[0.78950549430439243, 4.6006547850118009, 0.81625974482738739, 3.4584168411526695], [2.4025225994041732, 1.9543017364038495, 2.9376064903652797, 3.6854274630252308], [3.4539363096534976, 0.59762369687496053, 1.8267002675333743, 4.3614458789875821]], [[0.79352863847115895, 4.4168626427654925, 0.93481818191771471, 3.4255084908252442], [1.4817713825685079, 3.9846906874279648, 1.7829217724119342, 4.4184515405738409], [3.0659049563758458, 3.0835087859110426, 4.8243366749497794, 4.273535899647138]]], [[[4.107201264150012, 3.8368129568650522, 0.79228417189429312, 0.14539418953747776], [4.3488028310176601, 1.0716918894347667, 0.25886840962075214, 3.9835423056447561], [3.824177518072291, 0.052364241246456764, 0.68173364932392966, 1.5287192942555303]], [[3.6552119930473594, 3.2415969032163585, 3.9933637252785217, 4.3146639084877458], [0.88250631225660514, 0.79844967418941404, 0.78545819791625904, 2.9398539459967266], [1.9691437299317476, 1.6481600124795055, 1.8633891305573522, 4.9238635402004958]]], [[[3.2546133779320066, 1.6931433634907991, 4.3909576568984408, 4.7467337835160599], [3.741081541219601, 3.505021990675564, 0.74124661801824421, 4.5622212814332075], [1.1059574716636467, 4.2585197428703516, 2.6376728805880729, 0.2248061398757063]], [[1.1341860156384855, 4.4497953329872413, 0.38426545563496317, 3.3190361071325944], [2.3231741901653304, 4.7968358187567537, 4.1194856591404099, 2.5460254127465194], [0.47622574084669328, 3.7185842226430945, 1.6465364345433005, 1.5516432275199843]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.46751415705995203, 8.099098843080748, 0.51838663707223975, 72.828447509363684], [3.4439925293918052, 24.60268308533864, 12.221613652231301, 1.3785997812359698], [43.873284451186493, 0.84980973299124929, 3.4063487780183412, 47.923155238067316]], [[0.41352423479824252, 77.664851740364355, 0.85791305250778116, 183.53739093818339], [4.7006651368678609, 4.3927568370610253, 9.5053694090122942, 55.380307036044059], [54.618326638397981, 1.1403569134638609, 1.787031157506203, 1.6460393086112699]]], [[[23.525712363741398, 18.871734366446628, 0.82951779715546548, 0.014116668907489948], [5.8811295495530063, 1.2271308912958951, 0.031820140107573558, 60.363353414303496], [457.08582692625947, 0.00110911890807776, 0.32806327197905844, 8.3339795375985073]], [[448.75479854194793, 4.4271070682721305, 6.1532528404735709, 7.9716710804402853], [0.64293712659534041, 0.32880092138004124, 0.91770774980238534, 26.081521046759992], [24.040469929673353, 2.3342166353607232, 1.2577064545097709, 6.8530892048836272]]], [[[150.3448740370298, 1.6281479121649149, 443.45478389189435, 74.158659987175625], [10.732324571449537, 2.0964234962994546, 0.24581251776640334, 204.64374842165628], [1.1292296885990609, 395.2710403125468, 39.563960103359804, 0.24350351668746831]], [[1.7601984397878461, 6.6560027191766276, 0.049408027528523253, 1.3139849855642411], [7.4830350238188723, 44.604941349741807, 26.761019569481231, 97.172777095054101], [0.9659489394053461, 361.39121090429245, 5.8998093923189971, 3.0356441390198579]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(3.5718097013,self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array(2.1122305293)
      sub=res.substitute({arg0:s0})
      ref=Data(14.4515417175,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(0.174217145129,self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([2.3882814546896514, 4.9509111254564973])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([1.1637748767307226, 1.3213749219050812]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([2.1819865247559118, 3.2236768081946718]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array(3.84895873235)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([18.932636389769002, 77.082798797239619]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(4.00685828624,self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[3.8054227432273877, 1.8097420725664204, 1.8911854734556581, 2.989569268072866, 4.4572294187364738], [1.1879324622813543, 0.55295057251076807, 2.7899259714979006, 0.77203123374562521, 0.53023260358586433], [3.225448687631526, 1.8578333249710053, 0.59159436052330361, 3.4937880504893872, 0.49765387730655702], [3.1125965537360623, 0.46587932265375426, 4.5898684925763158, 3.0237695616233995, 4.913098375723373]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[211.63729740701635, 10.770442353438034, 12.847967078399291, 80.481551837213061, 398.7598524905639], [1.9937924383372867, 0.093106610635912218, 61.013609927281401, 0.35462459337610452, 0.078700234511982159], [109.10599288381358, 11.963876477110611, 0.12204853673453961, 150.28384701258065, 0.061042316962753404], [94.596116310417756, 0.046861767583751629, 448.47659726416788, 84.234597360379425, 589.06518301124675]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[1.8527853843704487, 0.16759407327031869, 2.8208996963347421, 1.6642849046043653, 3.9321543826748089], [1.9671092367147975, 2.2945249738369431, 2.4179145331169072, 3.6999245840202413, 2.7733553259605834], [3.9465566014456086, 0.34489628667877709, 4.4395100265057827, 2.2176276671745456, 1.297765365174439], [0.99542904286288514, 0.11179190714476474, 0.44747079597759187, 0.14629833584216231, 3.5254089319811146]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array(3.96942488127)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[12.862154545939907, 1.2599206533211345, 48.859663329361581, 9.9186664550663846, 226.09414074513222], [15.057836177100297, 23.648053668148489, 28.033165145523899, 164.15228049886716, 45.759823843662559], [230.62814899773235, 1.6087883834720103, 455.04791458130381, 21.269381614272508, 5.9842251688488579], [3.9444897284437546, 1.1666293518571713, 1.8531608223228484, 1.223468661737519, 129.05046841129129]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(4.52422475753,self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[0.69052212976819316, 2.605658225578122], [3.8440977571252861, 4.3005229258378082]], [[2.8771950415038527, 0.28728421161124029], [2.606919674107643, 0.22665469517432724]], [[0.17159005347142828, 4.1743207219428617], [4.0961283511846585, 2.5113445188359798]], [[2.6316674619704581, 1.489845832767944], [3.2460498614261013, 2.2497228816332067]], [[3.0782764785905141, 2.2316675917067559], [1.4151665816669121, 1.5685334981645074]], [[4.6906194407088249, 4.6673650156345987], [3.8976278184319737, 0.25101859662689041]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.18724202204896032, 76.155982586048339], [442.32492097747922, 734.83968363217605]], [[119.2560126089556, 0.0035422617455414607], [76.32292656477378, 0.0012120627555744889]], [[0.00034408857290721786, 642.19911520749633], [589.54316147942222, 64.456387682492789]], [[79.656192428818954, 6.0719794650889485], [205.81915698658213, 39.184188317428323]], [[161.88754650475656, 37.781417259082687], [4.811575629581907, 7.6640420909741547]], [[1088.4194280233125, 1064.2191048334253], [470.88368561689236, 0.0019236876571273583]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[0.36111698224820998, 2.2794198810365303], [0.99287197366396596, 2.6266291879044905]], [[4.2742242009494369, 2.1085284837120946], [3.8169626893419126, 3.0479651850763685]], [[2.1638509029074666, 1.9909913247371145], [4.0426670356547998, 2.3776326130794354]], [[4.7105926323852634, 3.8143219218562381], [4.0761680185379454, 2.7560487320221849]], [[3.6146662098368676, 4.8119970069519455], [2.6572313757740442, 0.87626418293556241]], [[0.63549977759308374, 4.0474643117307991], [2.0558829774584302, 1.5966202744724884]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array(3.27415651223)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[1.5346563963317006, 14.932340852962755], [3.2465925931309156, 22.541105563361583]], [[159.09278139351372, 12.192754584899808], [92.494376236079276, 37.153963261430313]], [[13.019622115458406, 10.606168150491662], [120.88586136962167, 16.777112535774226]], [[266.94611667487902, 92.205126944746624], [125.78586677414711, 26.280834239980134]], [[72.763323167886924, 301.06269405647987], [23.374287558899084, 2.8272433177975529]], [[2.1249346430162008, 121.57564535539055], [11.454711668855968, 6.6438252775329047]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(2.89315549177,self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[[2.3288984838436222, 4.0590533940750877, 0.55056060148542185, 2.0082737658387928], [2.2804720474473363, 4.7587652488050454, 4.5950879453706204, 3.1355476740668689], [2.5556781903903869, 1.4235249256960312, 3.8199647978646607, 0.23834252326835895]], [[1.2979917194949737, 2.8226110701634903, 2.2559770945497681, 0.13882902484999901], [1.0752213900707912, 4.5358548411859525, 3.9443764375197898, 3.8368498196243364], [2.941725325926448, 3.5923702410108085, 1.2523830047330669, 0.44499873017465275]]], [[[3.5868639469867265, 3.3475302938312246, 0.52353810261158362, 2.1072381344585511], [1.7214217098927074, 1.2574883642333099, 0.78547277738269972, 3.7498752299506712], [1.904386490877928, 4.0394944570151727, 4.5068876794802835, 4.9827376432208563]], [[0.36975302990753778, 3.0037687417828889, 4.2617436204117745, 1.1821002583857938], [2.2722381712395929, 0.012411962310012573, 4.5526378949433024, 2.0399678122131526], [1.8298272866344742, 4.9533451531336086, 4.4929476932550694, 0.88056140901023794]]], [[[4.3297828006478847, 4.5059980419620791, 4.2325892042198916, 1.8436298090642513], [1.8636179679131775, 2.4184207694025766, 2.1641465988273434, 0.52003553245899636], [3.643989988492681, 3.5290703947329298, 4.9610622831436819, 3.8564945911280413]], [[2.0631271529747268, 2.6994450148017641, 3.1378142159449114, 4.6035287204055271], [2.5736745998323149, 4.9843840628418112, 1.5824987375388733, 0.070225375449437605], [2.2763383915136637, 3.2599859357126557, 1.5747792657002944, 2.5566758386570512]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[11.540474178656178, 57.579441475840902, 0.17787255265154397, 7.5181983579652778], [10.859786349465082, 91.221309230688675, 82.436401344921876, 27.284240120957225], [15.100061093650243, 2.7778514139923511, 48.304571748611821, 0.015781460561312393]], [[2.1267345325097149, 20.128134816946254, 10.525729060287732, 0.0033041677875489612], [1.233469199430886, 79.399363155057912, 52.997825765737495, 48.92489488909947], [22.685000612268077, 40.439320610323833, 1.9176476695383682, 0.096083153680347699]]], [[[40.260250273391691, 32.969291533537742, 0.15377074390124612, 8.6407975311418586], [4.8134733033923727, 1.940351774496456, 0.49727666027222056, 45.784651025882575], [6.4472569790737992, 56.780385619990021, 77.941196056448007, 104.203770740941]], [[0.056221377461187279, 24.097025603284614, 66.297020600101405, 1.622557971866518], [10.746731967393279, 3.0562158545383705e-06, 80.25230756219861, 7.8666240221891757], [5.7437194746803408, 102.43530735385139, 77.245767961828236, 0.69211948090505926]]], [[[69.405745622707926, 77.89669256892843, 64.993352125514221, 5.8699633046035649], [6.0559811984827094, 12.871149014262926, 9.3333271580518442, 0.15081320468011597], [42.143450268312201, 38.411959861419085, 102.89770857185523, 49.653138655110403]], [[8.127792532158999, 17.690664522647985, 27.341339462541075, 82.875269629739719], [15.409746979295493, 104.30341758529211, 3.7733872016444918, 0.00045996838586555907], [10.802932846280594, 30.53605276515399, 3.7203793358128299, 15.117121219513963]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[3.363160348794731, 1.4747273090645547, 3.8109598648117493, 0.097077877108141891], [1.399670968540023, 0.31903522080112051, 4.9811609279762905, 4.0968961989875972], [1.3694038415813077, 2.5291882529985394, 0.25796886416578041, 1.4409794924466945]], [[3.8332653506076269, 0.30844802774952051, 3.1310836062414462, 4.9186072060625863], [1.4396942493728822, 4.9233494197018599, 2.877984731676321, 4.5907812399967058], [4.6979732237573959, 1.7362277362396417, 3.6582600810410506, 3.6340540427818278]]], [[[1.6700525749979165, 2.2228603133183031, 2.43293787570083, 2.8702405809229559], [2.3331534161034795, 3.7066646454895795, 1.0966440087471487, 1.908962341065916], [0.12194089951562262, 3.2863646606649546, 1.5287400782053076, 3.9848455825060265]], [[0.16847133235214878, 3.9680014373284296, 4.0276476190904686, 2.2217051359360545], [3.2812773462847749, 2.5746845803589187, 0.45331383549165188, 0.49856364484344085], [0.99898081458672161, 4.936995751896931, 0.92264915898809019, 3.1494883288087285]]], [[[2.3856311599393383, 3.1040108715817962, 3.2687441960370713, 0.53479076453786467], [0.43690258103987056, 4.1906738334484865, 0.24466473178265893, 2.3452754194038277], [1.6500280521182431, 2.7318639086175973, 2.5018514108619847, 3.1280747679610545]], [[1.3891452862852116, 3.9471405345793027, 3.1589373882841056, 2.9668546717211757], [2.6194581323848189, 2.0076093741913592, 1.8800761418341025, 3.3967868026439478], [0.90094569401184488, 4.0774519884774971, 3.8540850721879107, 1.0198889585281983]]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array(2.00849238724)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[10.437608757016037, 2.796732570855792, 14.263501043607951, 1.070044884344566], [2.6541081289361332, 1.2491835539886058, 32.258573344623144, 17.411160945841779], [2.5986728054311996, 5.8346623235407371, 1.1971017864238225, 2.7316794316336193]], [[14.487111575767798, 1.2399943684015171, 8.8779334798183491, 30.881579059582251], [2.7292321012216174, 30.983878007745933, 7.4414194044604054, 24.57031850503601], [26.477443060302715, 3.3562292986644335, 12.822654061032608, 12.608012720195397]]], [[[3.2048605752550285, 4.712356359297333, 5.4558743090152948, 7.4013392310989188], [5.0891191426307136, 13.26289176086039, 2.1485268196086005, 3.7858882182922375], [1.0887602669331347, 9.8933161844787758, 2.9040881714239197, 16.102412154782947]], [[1.1246695694008009, 15.914366269274741, 16.590306117931707, 4.7085616011817333], [9.8582787326169132, 6.0227551543607536, 1.3718140529079674, 1.4157939945510962], [2.0070653303874577, 31.280151027729012, 1.9030181310598322, 8.9926176516018206]]], [[[5.2788164648950202, 8.7118895818118745, 9.7724886706492704, 1.4520186028277782], [1.356203209200491, 18.587893694200702, 1.1860463294065218, 5.1323233682001286], [3.1604164206443546, 6.7204776478672414, 5.7244822368755992, 8.8593242933736196]], [[2.6346971202602827, 15.684518960368685, 9.0520713170100979, 7.9172040480346126], [6.2137787017322639, 4.0555058532697092, 3.7103853645188334, 10.685269105192523], [1.8744315841156032, 17.176657594081007, 14.698989442831733, 2.0365447740381124]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([2.8357414444877005, 3.6891343488251347]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([1.6663064470718421, 2.1585946872152251])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([4.2544114472866665, 17.092362603596417]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Data(numpy.array([2.1826925247474129, 4.7623840489478839]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[1.9596090510170103, 1.6531514644083005], [2.1422376737044955, 1.1058073486744655]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[4.3422622141831333, 2.9957763009314511], [37.645999409814237, 1.6144270269364216]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_constData_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[0.34317639105920478, 4.0267662317463717], [0.021983922914570804, 2.9950336783640799]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([4.7576271370607008, 3.2466945465991546])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[1.7079076889935045, 534.18636579622114], [1.0262271234948366, 34.0239171333671]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_constData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Data(numpy.array([0.14556722800772615, 4.8672115186639315]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[0.095580431475943556, 3.8888041767379606, 1.5278903437430531, 2.8506438392057816, 1.7877227926794346], [1.1653877399272885, 0.25644363373951395, 0.3017159511496661, 1.3227021278783166, 0.016434072341888761], [4.0462025280810767, 1.3640292333534116, 0.16564074253790054, 2.3947049034061796, 2.3350554051383665], [1.4073339246759307, 2.7856894284068856, 4.4057269668376362, 2.1767685957610095, 2.3998020090068262]], [[0.44318567928189484, 0.51719743514215488, 3.1153397125509845, 2.4404648227538495, 0.20077145696089502], [2.2230671075006168, 1.5681756314561104, 1.7022108584873161, 1.9485490291512897, 2.9579182408016926], [1.6578889585719427, 2.2620349918359768, 2.4211732850343819, 0.29065075006546631, 1.1671061999073591], [4.7663064519039011, 0.49559351029077009, 0.59162260665272004, 3.8237284798701694, 3.5985021642414425]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.71051809022118317, 1.2185907887647907, 1.0636476605989249, 1.1647287228694665, 1.0882448922076124], [1.0225296698662185, 0.82029221968289789, 0.83993671012890059, 1.0415518490443116, 0.54988386161886138], [1.2256493490258211, 1.0462269648333007, 0.76972630607552928, 1.1355510491091061, 1.1313891277230235], [1.0509976930539708, 1.1608273161824265, 1.2409316283553498, 1.1198874938085581, 1.1359025669135907]], [[0.019048375265866422, 0.040392892076813035, 252.34530112869473, 76.897260563307086, 0.00040374113478464535], [48.830543623527468, 8.933617190962778, 13.316523814347423, 25.709069662967124, 196.06013180381848], [11.711776138356932, 53.140197960494561, 73.983547874348574, 0.0024440795133155902, 2.1214871018810673], [1999.1945425025394, 0.032817981118402346, 0.077712992552959839, 684.04970295137264, 509.05215002230352]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_constData_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[0.90901830218666801, 2.1031729462777378, 3.809707265498671, 3.1046566073462261, 1.5703351538625598], [3.5654085846261792, 2.8934813171240541, 0.77100436933517813, 4.0608551284106973, 4.5426236668979003], [2.0272072849820204, 1.4832931221354035, 1.4665823265023095, 4.2020677640304767, 3.0565813553135173], [0.67173683351983537, 2.3937174070412652, 4.0490072670645638, 3.3279345819411286, 3.2874639727662562]], [[1.3923277890987722, 0.9803787650347221, 4.4553190867780357, 2.1016816280202586, 4.3276817471863804], [0.53852243844535541, 2.043759174858772, 2.080057262645536, 4.116441166720195, 1.0972775733365638], [0.7040166318132911, 2.3877467989876302, 4.293371537003428, 3.8342757271858128, 4.3555569043482611], [2.9400651629937036, 0.77541708672365461, 2.8743788812669071, 2.2788474408283403, 2.3473711533154371]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([0.56507687513412042, 0.29438879690369207])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.5951976560020642, 0.30105044515450208, 0.11365859272361049, 0.16997264011344085, 0.40806161833568899], [0.13066584360900352, 0.19174664925273907, 0.64398214490726269, 0.098479222606798922, 0.074802794215587998], [0.31439135013592129, 0.42884742964825895, 0.43295752446587282, 0.090852956241032873, 0.17470145133939016], [0.6815245432898368, 0.25504422319014558, 0.099147463294035967, 0.14963393351596363, 0.15313076484932742]], [[0.18220669815076321, 0.30153775637089958, 0.0043040255072707163, 0.076531743703850977, 0.0050310694394879859], [0.51760944446256563, 0.082149134235392732, 0.078582501363619522, 0.0065139690223586129, 0.26137210133907268], [0.42277738424105282, 0.053940936625366552, 0.0052466456556731576, 0.0091980920133449603, 0.0048624641092330596], [0.027453261870646756, 0.38742932072563124, 0.029749422477760207, 0.061624387409868378, 0.056671033061228837]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_constData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Data(numpy.array([1.021338993290795, 1.3724181111753326]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[[1.8202792798322445, 3.8813697468539989], [1.553958150908852, 2.607428848850601]], [[2.296601270395767, 4.1532684582978305], [4.2626183887747997, 3.7941784777490808]], [[3.7102990847392028, 2.3710773751799583], [0.80248643907413653, 0.58621601141659052]], [[0.6615003322924804, 3.5212403942097099], [3.4302775013211426, 1.1344866723209992]], [[3.4618729283995795, 1.3638117446919293], [3.4165222200185075, 1.6245870395623254]], [[2.3755391981622145, 2.7261971435179886], [0.60071735247613645, 3.388034417554826]]], [[[3.8066291175548153, 0.46425745067916679], [4.1022974551150018, 0.91499572470049861]], [[2.2349096788746521, 3.096510160395372], [0.67109786943566141, 4.7174783753378362]], [[4.6261219985504427, 1.3368341307894001], [2.3992986934840426, 1.1679456807228887]], [[3.0727322333969429, 3.1619342672199724], [2.36599217446676, 2.8004153207887383]], [[4.6486077419229606, 0.79814622665732049], [4.1341107371770329, 4.034564707788828]], [[4.5179777992174008, 0.27465386847567747], [0.88752956694649765, 3.6869509079366769]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.843695132611765, 3.9953365083347609], [1.5686441750189897, 2.6613011480826869]], [[2.3377109113767442, 4.2814001901737004], [4.3965611278185737, 3.9036920130957391]], [[3.8155709386863355, 2.4151643575871886], [0.7987272454181048, 0.57957315388743702]], [[0.65569271570509391, 3.6171091197012344], [3.5217023102278153, 1.1375454620038292]], [[3.5548352583601543, 1.3728716912468304], [3.5072796898989167, 1.641496750641547]], [[2.4198062164087744, 2.7851693794048025], [0.59421994069883299, 3.4774137443660136]]], [[[6.2624516526470586, 0.34886215135231696], [6.9395220494145589, 0.88521920169665891]], [[3.0153013145668917, 4.7171786270858442], [0.57846591918918, 8.4064317856227806]], [[8.1838181544093889, 1.4894676624066356], [3.3237978390688929, 1.2374625342123164]], [[4.6675367947524542, 4.8544980229681567], [3.260638348828075, 4.109378867118691]], [[8.2384599645601657, 0.7338650435476739], [7.0134864950750568, 6.7827583157812867]], [[7.9224067192670509, 0.16973810857027372], [0.84895599080959583, 5.9938316544277779]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_constData_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[[2.5034617118254849, 0.9612441914654597], [2.3799311810361652, 4.7372690518099168]], [[3.8093754142087501, 4.773000957995909], [3.4428452974513726, 4.0207892389362483]], [[1.1328822220808037, 2.593766660254138], [3.7223056789373774, 4.6979322972876192]], [[0.77535817276681873, 0.62700895899768272], [1.2454342417203854, 2.3737569534498517]], [[3.6351810968259537, 2.5563058101628982], [1.2402537615373495, 1.1405938505415727]], [[2.5090891331871519, 4.4979265801259078], [0.48100453120541853, 0.79374962620868617]]], [[[3.1815113004636002, 3.6134363502481248], [4.0587498709522931, 4.2622483472821191]], [[4.6951738871420483, 4.5881382734827207], [2.2619532131346118, 0.46867960700371869]], [[0.28723835042584817, 2.5476404493018623], [4.0561419209528653, 2.8902759632401689]], [[3.1814308815422874, 2.4418992880684511], [1.7824795047409978, 2.18369022905188]], [[0.61336857959573987, 4.4268939178321123], [1.6543023668237955, 3.4080507373194213]], [[3.570362444580526, 3.4060300989261023], [0.47354477769687492, 0.68942720279922509]]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([0.62211854287621215, 1.883524798970301])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.30476796385333554, 0.63366797525283081], [0.32317091897451417, 0.10556540893370035]], [[0.16397794167050711, 0.10379019525889638], [0.19513587963116327, 0.14832261171809061]], [[0.58409374002579906, 0.29198130051869686], [0.17089637127627713, 0.10755484923632719]], [[0.69211435862585746, 0.7426028573161727], [0.55371024365383703, 0.32411934106918072]], [[0.1781113343492369, 0.29721910755209957], [0.55507337236582011, 0.58195978939302795]], [[0.30395504132451617, 0.11826508413200046], [0.79588816970932241, 0.68609915868931437]]], [[[7.4959112130616239, 9.8535209582700407], [13.062908996770489, 14.859224716042588]], [[19.545116685155744, 18.264447576102267], [4.1876686615166898, 1.3454682714875716]], [[1.1994504587005681, 5.0179706271617812], [13.041357185339319, 6.2336592979053016]], [[7.4955295547332597, 4.6930191434066568], [3.0912261781811172, 3.9852193847119386]], [[1.4745475498641163, 16.491834358986388], [2.8502683011048893, 8.6519955405610638]], [[9.5884267532621177, 8.6409336277467368], [1.3496191854208557, 1.5472935800365364]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[2.8406661352258418, 3.9417947049231095, 3.936848123145936, 3.6043346547156192, 2.8540630075528126], [0.89311606049374714, 1.5279569374471036, 4.1252090619638073, 0.43000531476263487, 1.6922224232642229], [1.7846279087749577, 2.8795718985536163, 1.6375948184394007, 2.7549640650954026, 4.4002814556363568], [0.029381925609190118, 3.7938400018957528, 3.3544978268173851, 1.895007842020024, 3.9848908005033925]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[3.4037126479497681, 2.4059463842479789, 3.4729951150908307, 4.0238895448023406, 1.981206504592554], [4.423612218384652, 0.47874826602616438, 1.7869114942762023, 1.4532014354584601, 1.5968361063568817], [3.7186079052705487, 1.317760052255303, 2.4805201245671116, 2.3482846701885745, 3.1190355589744145], [3.3174928746131811, 0.7987546759787284, 4.1969611569169141, 3.1232819905904265, 4.8115962693308738]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[32.441472307915006, 31.838376861442899, 134.48387258653111, 151.1286194301473, 7.0381124518329417], [3.7735825409564265, 0.32450216481855593, 10.96422035356515, 1.1743592273629857, 2.2078100799535552], [10.421176691049389, 2.2134889120401691, 4.4269148744704676, 10.505215700503934, 149.22049601840175], [1.0358631904844013, 0.42635573069391491, 122.92322204549846, 8.6554987309182465, 523.4170610698867]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Data(numpy.array([[2.848343854801727, 1.2530913257373271, 4.1159241820034689, 1.991532592045911, 4.6999003049871648], [2.3359372828801801, 1.7427395080320498, 2.2849761697748545, 2.3345859107422751, 1.6463558447268609], [2.9676298254820126, 4.6779726319951411, 0.69528815027670277, 3.5450781083435943, 4.8692786718277921], [2.1556168937292384, 3.2785616918675005, 4.7500619170915508, 4.856980296985185, 0.38556908978138227]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[2.9628469937343702, 1.2368869881338076], [4.9395152124440367, 2.0188575671876281], [4.9202426506459043, 2.8895179032034433], [0.92297118443777004, 4.4144990453362114], [0.25092354501725589, 1.8677467937233196]], [[4.0844267799293057, 2.3262487452088449], [3.1806580186789488, 1.036771818755396], [4.8596737987313059, 4.5652775175068285], [4.6533167053515045, 1.8344072883081217], [0.38574499245805416, 1.8985395494464814]], [[0.6112721318259392, 1.7600403053338625], [4.5745716982463662, 0.69185351664640804], [1.5259400998852533, 2.8409491994753346], [4.5340958204762964, 2.9487752672485779], [0.051857979174159137, 3.701599515431123]], [[1.6278964252394903, 0.32430056806252677], [3.140144417957023, 3.5961648171804499], [2.1723087045107521, 2.66617183905189], [0.55636918621323461, 4.207435157155702], [4.9187521824401781, 3.1716463399633152]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[22.05921586146555, 1.8322623069669899], [7.4003083054585312, 2.4117110935250157], [704.95932021814042, 78.835597767535731], [0.85245419233778541, 19.24431160223287], [0.0015062758432555904, 18.843759328893626]], [[26.764797155428273, 7.1859762113269365], [7.5120105420843455, 1.0649560964286147], [37.057908445655862, 32.126750288048527], [36.220061629760828, 4.1224320481267656], [0.20840257270783646, 2.8732791124468355]], [[0.23207235699047482, 5.3532833379154283], [1227.7255108347645, 0.17848042242428916], [1.3415641338892457, 2.0667491888195553], [212.47644307782363, 46.22920465473819], [5.5217782026336032e-07, 585.66129982221423]], [[2.8588162330724964, 0.088265425770892067], [42.587288900550938, 66.428489842643913], [39.847234696463367, 105.43748066563971], [0.057974023733165125, 1073.5947278787396], [1.8482379372434499, 1.5605579718543487]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_constData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[0.1959445421242709, 0.89112690496968738], [1.3607561136346975, 3.5633732783378322], [2.2173744887967386, 0.54854325322957742], [4.2778440716726323, 2.5272140561822836], [2.0745485921683162, 0.49061992026316781]], [[4.9694165145485805, 1.305404220530314], [4.2482847523396856, 2.7215725419205459], [4.7494074869385505, 4.0444985920230723], [2.810057694028206, 0.21426275353763505], [1.9715165057388484, 1.1833507041575788]], [[3.7042252593980907, 4.6398922947628893], [3.2217167109260014, 1.3782581092324719], [2.5778042985607255, 4.7185245406825214], [2.1221275648071236, 1.3769586996474055], [4.106568486861887, 2.3942742662189267]], [[0.61957057459082454, 0.15688057554210758], [4.8741380008706354, 3.3679314689950512], [2.257129134563995, 4.8182805170454781], [0.010257391365706109, 4.7933381556683488], [4.07715528122037, 3.5059660926677854]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[0.37998260196314099, 4.0111482143078296, 4.4592028657215037, 2.6044597210400262, 3.4131123325085801], [4.6739840277470677, 1.9458889366563012, 1.0326348687937514, 4.6784051597168732, 1.3244581447732282], [1.5059462726055837, 0.16350760521980578, 3.7372832969117384, 2.5426052826331267, 0.60141991281890417], [2.0526365406179665, 0.60349901379384896, 1.2859017982349439, 1.1837323329600513, 2.6634052099379453]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.82728851087064892, 0.42219796685221489], [6.6206645640792869, 141.14698308904454], [27.519872629029926, 2.2706263498348829], [60.030956663193251, 11.235889943096035], [12.765763352814169, 1.826309154477993]], [[2127.9173240956511, 7.4853571820153171], [16.914368919036257, 6.1214702508467527], [1.1647667603685332, 1.1386958952951645], [76.385963799945898, 1.3918005576838], [1.7402049564783775, 1.394485604438914]], [[4.5566572186256549, 6.6836981252284167], [0.0029257828916663736, 0.082423739210337937], [29.918331450888381, 503.05981957196821], [7.2452540871471696, 3.614534455876298], [0.12393059250773131, 0.29599958158601297]], [[1.5613467367247138, 1.1194267871354466], [0.085307518634042401, 0.1825297674434096], [1.7639899859205526, 3.3588749279508257], [1.0017316367920022, 2.2445477340825599], [54.271735547919263, 31.014769318758173]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Data(numpy.array([[4.0232222243944076, 4.2095901519879009, 4.2560862419285082, 2.5755953783169927, 4.237324774279208], [1.0076075067011614, 0.094198676501741518, 0.073938664130737575, 3.3888949667756214, 4.9878694562438639], [0.48549753607702245, 3.4587916717435236, 3.6903110976063975, 4.5391215516355823, 4.7110978232066376], [3.6317197958177538, 3.5028904864556916, 3.371795426666595, 2.3984026669297189, 3.7814208737962658]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.6491034839695136, 0.49256002965282158, 0.31811019531862472, 2.6459287146958963, 4.6288393003299557], [3.8496595399424782, 3.743494301462587, 3.5299405112404436, 3.7691356147005899, 1.8536873129587641], [3.3199325074393045, 3.8426446382493022, 0.16346843105920447, 4.9984600085953037, 2.0953764679296483], [3.3092891508758244, 1.070122120656557, 0.24615622804640408, 2.5272445112402795, 3.6589181342312891]], [[0.57115890175572714, 1.7708891400496523, 2.3872796550379403, 2.2752613835832571, 3.2190214661766903], [0.92644570474647503, 0.3101700379282834, 3.0713771378619668, 0.45886717109301695, 0.75488895002741685], [1.929332144510072, 2.8276184079284588, 1.9707856564360411, 0.11732156394965058, 4.6056635654437637], [2.5033595504848241, 3.7489642126153724, 0.96031306523091109, 1.1078330364170415, 1.0160671751546175]], [[0.12911181882186978, 4.8905334394768172, 3.7542748622766462, 0.035645668437630104, 2.085962250458667], [2.866552388539827, 3.4907069140589684, 0.66723198007854567, 1.8246655695227876, 0.056486450062847984], [4.4188888954163454, 3.841161138928368, 1.1930497248851741, 0.44298426781424444, 4.1781961116436408], [2.0605039330269554, 2.7105202633482768, 1.4553693339724385, 4.2651759678736951, 1.5980592880321984]], [[4.9309228436867434, 3.4958409337601974, 0.75302827269679584, 3.1111309735148209, 0.19654393952489496], [0.26049826032263151, 2.5686505914852358, 2.7717653032726473, 4.5317962613564289, 4.4324897416482614], [2.4695471211703559, 0.97599431158389782, 0.79835563547036026, 2.8145992828132957, 2.5794252704614165], [3.3142331416134452, 4.7539796994136969, 4.460437626880652, 3.2721330431877882, 2.3467870241767264]], [[0.76920279126515312, 3.8898088229055299, 1.8193817999083377, 0.95809878773429713, 0.68618821648879291], [2.8135064629619198, 4.2894988243293675, 2.1122986692112313, 4.2550361493592774, 2.2715188013949592], [4.7551909774712779, 3.0440990305153126, 1.4176854032567945, 4.9987770148153752, 2.7142888297905192], [2.0524057975403145, 2.1734721307960934, 3.6967985107838115, 1.3362206684850093, 2.2699581255209851]]], [[[2.4111961728222493, 1.8708078859508557, 2.2106657258520994, 2.4906662172521781, 4.2203578000153374], [3.231203079705776, 3.1225430294624128, 4.2913108699411824, 0.44620278051669432, 0.073418529817787986], [4.9350792798463265, 0.85811478889177273, 1.430202485694825, 3.8399475911410472, 4.0978487184807886], [1.6271518116403481, 1.2982794414436103, 3.0898630417297386, 3.4893439683383169, 1.6646325976151812]], [[2.7056698191583104, 3.9633441152030064, 3.0340651575085125, 0.59082375373862595, 2.0693977356379505], [3.5527482451358043, 3.0408988801065657, 0.8601386685042377, 2.6716793987783314, 3.0609521614764783], [3.4961678535004994, 4.5972502221720069, 0.69839300615637756, 4.4027147875132222, 2.8127867474534924], [4.2584049826395312, 4.8573098925496137, 1.6576603191891757, 2.5175120423113335, 3.3889102545435437]], [[0.46288740379715415, 4.5138240042934052, 3.3939457660131103, 0.70638968708977157, 2.9106527161691496], [4.7588445797633954, 1.6133211594422809, 0.76788015947851973, 3.4951092301958453, 2.9694738013577444], [0.79177547354351085, 2.0898757641075592, 4.4209636432008494, 1.4965849472606629, 1.5932016441232455], [0.36349029912774838, 3.9275745394304638, 1.8344458288638212, 2.3109087318891728, 0.29792923392574788]], [[4.9559111233995505, 3.048749211394143, 0.61704659809839235, 3.4742192102612646, 4.5600300619738086], [0.88086305528580977, 0.42581635361330999, 2.8126530485755028, 1.403549176128166, 0.79405463374413898], [1.5122891150270181, 0.60015714440176549, 0.6937690988624392, 0.87247618145204575, 3.3089332001848666], [3.4183478133251528, 1.6139400168361748, 1.8231064026180341, 3.7919681168204891, 4.5375078992873945]], [[4.2554737866303975, 2.2034615190237021, 3.9636939964380051, 3.8459453283159846, 1.1949282949606483], [4.4386812250029823, 4.0516447893859322, 3.1244917313622818, 4.8126243643600715, 3.8259233304942546], [0.58809732102578005, 4.1823364876270013, 2.9924146918841985, 1.0561281875491637, 2.8323791134162475], [0.65496942835698746, 0.39217936266444564, 4.7866626313961866, 1.3719579355922602, 2.7225662995104107]]], [[[1.6726477712602112, 0.23823609981942195, 4.1234650152560768, 2.6669530437643729, 3.3031273803852099], [4.2572793303087844, 0.19623477798079961, 0.47487460820837668, 0.57421988926597656, 1.5278938712799055], [0.59199392569077414, 2.6182326160741654, 1.8760061103028971, 1.494931125425385, 0.59293543654071024], [1.7999346355630312, 1.4476216297107491, 3.1732949658350069, 0.45627849176843877, 4.0968239594910907]], [[1.8690601215111329, 4.2896196584540238, 2.7222704048588491, 3.6905835183185394, 0.84167175308967701], [4.8724127338058354, 0.88031211207174043, 0.88629162897564606, 1.2614737763218706, 3.2858743644457205], [3.0406839735532847, 4.9449169841996374, 1.578828089276336, 4.8860505404827848, 3.571448673538852], [3.4922926014055067, 3.3969035242499785, 0.42729376321447105, 3.924387520913363, 2.5764517788350005]], [[4.2894488811296219, 2.7473753323919943, 4.3832226326498738, 2.6676575052167113, 1.9376275277288038], [0.49649914738693868, 4.7461965350414568, 1.0206544136568003, 2.1123123862853856, 2.1526455048900952], [3.2952952619544504, 2.029884689572492, 1.9442069758333873, 1.943079462092975, 1.0308751417557294], [0.38333258919335966, 4.2087144913674637, 0.45586584744218989, 3.033866507937772, 4.9923517608115811]], [[0.7289391051866404, 4.8318565566069607, 3.6550039084837231, 3.06464094403611, 1.2661019730005014], [4.2084349117236357, 2.8180728416819161, 0.417764837632499, 0.55444142932196216, 4.2088527862867053], [3.0204258576028562, 4.739465705233032, 0.86621794105233263, 2.6440154989720259, 4.987652854243442], [3.7034091825060247, 1.7907753921101017, 2.878431721207209, 1.8397249352523515, 3.2968563716111787]], [[4.1686985531821561, 1.5356921213828787, 4.789099007023232, 0.1087112484781446, 3.4396587425565084], [0.71552367534828598, 4.0208632174444281, 0.29181217053147551, 0.81690364140933169, 1.342399421127765], [1.8638054265129174, 1.5794786711769062, 2.3963566853604177, 2.3286049802666859, 1.0990515414478741], [0.57544117404251538, 3.8054806520693623, 1.4958017002046291, 3.8814997646562377, 4.9165623640540828]]], [[[2.037622441284868, 3.1479066417245094, 2.949063436737061, 2.1758439001810803, 3.5090305342305634], [3.4365559369935377, 4.7697386635603136, 4.4917699106894977, 3.9345558704922428, 4.9159429515318385], [2.427516272710347, 2.7234572489637117, 3.7754329156035067, 1.5666325998871433, 0.56414390227297317], [3.6414434814307315, 2.3995198869090704, 1.2383095203417744, 4.22272728875393, 4.1921931827163039]], [[3.3377513143101676, 2.4536634114426721, 1.1005811925445699, 4.9809891183750032, 0.80385938536112667], [4.9070577056495894, 1.303335179712136, 4.9627191338941756, 2.6359688358854565, 3.9807457932738415], [0.61896372219450435, 1.3736046750442108, 1.6539254673722446, 4.6198272129630622, 3.3852814063428811], [4.1042778840992211, 3.9224857604700265, 2.9094280757174777, 2.3128607762398765, 2.4571417654860004]], [[2.5457845058597517, 2.6328458536558235, 3.4729192604787125, 3.0927218870968476, 4.6220410050607068], [4.5083183777062699, 0.9791091975155507, 2.2437425496288066, 4.0488453821854611, 0.74583788091057968], [2.0305202736158621, 0.64296293380226011, 3.6771128348500501, 2.983187718929543, 3.0950523043681515], [0.07767714127753933, 3.3946736834776927, 4.9448847615146461, 3.5284549181871743, 1.5851013715963429]], [[4.0523157434149688, 4.1696015947596363, 0.011243173535194651, 1.2315594977276922, 0.69762017502510443], [1.8784008204307359, 1.2973332631816499, 3.0477754163570245, 3.1008654609903554, 4.6389017948377331], [4.3853992121231098, 0.21336963117449939, 0.96970153562755945, 4.1194068744673915, 3.0645441179923743], [0.49288175666859452, 4.9184804275043357, 0.19593030185472718, 3.8426217832249501, 2.3501524308711166]], [[3.7760010590601154, 4.9675690378469364, 2.7841851135633813, 0.16057699776683737, 3.7327767488462027], [0.39072317493017472, 0.039600518930111919, 0.48882552944802116, 4.8620618117280934, 3.5859837156083021], [1.5893784897140526, 3.8208568921315855, 2.3283733361280929, 2.7531455204419579, 1.8918572055300027], [1.7948941197080235, 0.27708271832280479, 0.13067744841622442, 4.7787790272467641, 2.3696077143058529]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[182.72582523618212, 0.057902175944160458, 0.009971467966027249, 50.133238755308433, 475.70986426575234], [226.61262372074151, 202.49834432399265, 159.87844770792751, 208.13670293576581, 11.977614008850312], [124.91621017171401, 224.95585855193698, 0.0006846518134932904, 647.99763178467981, 19.611403206167076], [123.31282814621785, 1.3134600676756476, 0.0035539054949623675, 41.681123151673305, 184.71113326552785]], [[0.094633837500943702, 11.086232095541048, 38.977860506389824, 31.838707358045209, 137.18588045323713], [0.72497942516490632, 0.0072417529757393103, 112.5830000455225, 0.037656580131651407, 0.30615246941582358], [15.901807691840117, 79.487082197245627, 17.390460657714819, 0.00012090997616985149, 619.70978984394958], [47.601371600629697, 260.57415269533675, 0.84326728929692651, 1.5389291491477941, 1.069400928450535]], [[0.00016451087806760805, 858.93357271457194, 278.76130121181814, 6.8741009576263592e-07, 22.855765234662194], [88.422564553941413, 204.49664796708322, 0.17869283306849099, 12.930596293399164, 4.8771625401143161e-06], [557.83804585690973, 307.2719508638329, 2.1196559686823258, 0.031260658425485144, 439.52337365148537], [21.691923873597528, 69.680211652801773, 4.9388754949270837, 479.80636080049294, 7.353750449056788]], [[60.911975869606543, 25.117080179332156, 0.48163241454693584, 18.601933239716029, 0.015143907820450313], [0.0312859691831033, 11.35623541902198, 13.815336791476964, 49.010253561590368, 46.291678515013615], [10.261795316527932, 0.93933494627339964, 0.55988326639375041, 14.371935104308205, 11.479331795681926], [21.892542103036043, 55.440317863563472, 47.047181668151616, 21.183427198103423, 8.9989179803293844]], [[0.32894105192143647, 316.02221933780964, 12.629371890720531, 0.8341215931927326, 0.202748176183318], [80.095723555661237, 478.31274705874614, 23.773536212870471, 462.23980260428095, 32.346555254008493], [740.25370373455132, 111.83348330679966, 4.3882678609030927, 914.77174502058404, 68.79257470633685], [21.045571916960046, 26.830657680483, 254.71953159979975, 3.4149688940788017, 32.252489149185195]]], [[[2.4273946045681702, 1.8797437772761534, 2.2240474036482825, 2.5080171574171399, 4.266842581986948], [3.2601624881980147, 3.1497088302155389, 4.3391274766714236, 0.44347187970471058, 0.071974273986097015], [4.9953781924412866, 0.85711645503085576, 1.434100926068552, 3.8794535441594324, 4.1420557945764136], [1.6331892572120459, 1.3008602077661386, 3.1164950807159855, 3.5226760756716216, 1.671098602847523]], [[1.0982967887350383, 1.1385094198991592, 1.1102124875450776, 0.95163770537907799, 1.0709078052574799], [1.1268404826549545, 1.1104477977392815, 0.9859081045282827, 1.0969896249182258, 1.1111355512379555], [1.1251376897564513, 1.1545332587727997, 0.96675052186049082, 1.1498405599568127, 1.1023210296848556], [1.1462364926623307, 1.1605332213259782, 1.0487601802085655, 1.0908649551672061, 1.1218400925827314]], [[0.94463861605657018, 1.1178824934270726, 1.0945600961201305, 0.97462722997273854, 1.0821982558700276], [1.1222601899836131, 1.0359972453949122, 0.98066065342388786, 1.0969397189575445, 1.0838003592774863], [0.98288514342489663, 1.0560130371734588, 1.1161656692653454, 1.0302598172985911, 1.0350364130670806], [0.92790481128846236, 1.1064422907080049, 1.0458832305479062, 1.0638921729309847, 0.91435875284784562]], [[226.83003625948226, 43.715715987748155, 0.19471956820421452, 68.062563389128712, 171.06940360387904], [0.65057944622778641, 0.055395366858232042, 33.266530300523392, 3.1545614117370429, 0.45772377589264124], [4.0622211085386519, 0.1772406394144514, 0.2896637509430911, 0.62982530265958758, 57.698968194784626], [64.423928130814019, 5.0641729492741581, 7.6535782935792733, 91.561224195074288, 168.2229176024118]], [[1371.2294054563179, 51.447620887886899, 962.15560619513531, 827.78799849552422, 2.4309220130824714], [1692.0696609139259, 1073.4601292698835, 293.69391973588279, 2532.9823443556388, 806.51492004035674], [0.070801541258587217, 1257.6410487188066, 236.77426038181488, 1.31309296255789, 179.99982114936009], [0.12115308701588726, 0.0093832855576592902, 2465.5563201032764, 4.8421469097694967, 147.78004580875111]]], [[[1.2836964271337732, 0.49835487450016558, 1.9893368963785478, 1.6100131088656562, 1.7862282975997468], [2.0204220839072029, 0.45357016528286781, 0.69659423553910249, 0.76389395765842016, 1.2285046073756547], [0.77528358025518285, 1.5956659036401908, 1.3572334495300038, 1.2155649100138912, 0.77588196169791668], [1.3302293182089555, 1.1967340212160773, 1.7517901670206675, 0.68321438740654927, 1.9830864926658136]], [[8.699380145633917, 153.95891197293025, 31.940126066541318, 91.508954644190098, 0.55091471811640202], [239.2012583489811, 0.643442585025504, 0.65868616958008519, 2.2331442160730881, 61.233349662988395], [46.827077983626708, 251.73951166588324, 4.8528728005267059, 241.52496991890681, 81.690894561999031], [75.597337947944283, 68.692004986980479, 0.052815340012322541, 113.17050513119887, 26.402164627920811]], [[215.65436890701986, 41.662367579045743, 233.57038992881778, 37.372532557505302, 11.484623458142361], [0.075482224741476533, 313.27526693697763, 1.0783636284894982, 15.792780128840233, 16.934492000597778], [81.506305476499932, 13.635309531681274, 11.629194687288704, 11.60432598342938, 1.1187537309959485], [0.02905794794837609, 201.05075038139435, 0.055081100048851896, 60.078200167624615, 377.54032758722178]], [[0.23808875858827441, 1274.3148687637922, 358.93544865543333, 161.3377969708236, 2.9182185850962705], [680.70561497723747, 110.25233459242413, 0.019026711938885759, 0.068759034776483818, 681.012469674062], [151.03855433885093, 1167.394962165416, 0.52105331568433821, 82.548199961042201, 1471.7558348658815], [381.02390747305822, 14.079407916621212, 121.38487820621513, 15.912762111838006, 224.75998803260782]], [[833.45655429626936, 7.5456577812404442, 1602.2942322082656, 2.8826692564631517e-05, 336.95843432294424], [0.20659484566238231, 703.08419626011926, 0.0030202980100945682, 0.38568095837674271, 4.0037178408797116], [18.788129522148893, 8.614255908297384, 61.390908706196178, 53.632041388881078, 1.5604149113503141], [0.07401860904501481, 542.45787445216968, 6.6657439454124416, 595.4352817295138, 1813.3650187838462]]], [[[13.263273198685415, 64.368855785812514, 50.787998378896702, 16.833302521692087, 95.491992499117075], [88.521757493864342, 291.14022532616229, 234.09792037224832, 144.70866953685913, 324.8794425028421], [25.049612050797826, 38.039601619252785, 124.5608594052707, 5.1058269654799036, 0.12505981220682347], [109.24167447085387, 24.016246919565631, 2.1733500745508927, 187.05900015625204, 182.19327610365198]], [[68.171358908491172, 23.199531190995007, 1.3989352471976839, 277.08955592384137, 0.46543198771582145], [262.94861485355915, 2.5294667845206318, 273.54570961175335, 29.820002272168065, 126.36003746488427], [0.18630563699371175, 3.0402874620098981, 5.8268976316665055, 212.86838058193177, 71.632894803575738], [140.6375213213251, 119.99979274626239, 42.137359615146757, 18.861480877791944, 23.314938920957154]], [[23.35326170868683, 26.15703733890102, 66.5444549517469, 45.012081412847522, 174.45505028141798], [160.3996307894638, 0.93128894341234736, 15.25489549039081, 111.634012831475, 0.37203439785705117], [10.89404057878383, 0.22554968222193986, 80.681464058579948, 39.858946618733903, 45.126546008513166], [0.00018125742314094586, 61.622935428302291, 219.05472042324718, 70.200977413328857, 4.7266279473242934]], [[28.675906777367839, 30.706933380991938, 2.1146902991321548e-05, 1.647966835146538, 0.42163384883183347], [4.5358064594358138, 1.8669949668606109, 14.480582291972571, 15.092943798485205, 39.658103487832278], [34.657435189029513, 0.024603101272468259, 0.92886530949031365, 29.827795630769621, 14.672402044963926], [0.18326240301858329, 45.634068457081305, 0.020052780853957317, 25.244832403755982, 7.7631452448976273]], [[152.05519268785068, 428.95636676108057, 48.03864008716414, 0.00099164077987656525, 145.57734910530388], [0.028621050171084243, 4.9810032491376366e-06, 0.06676671220180995, 395.5098315129311, 125.08563906976659], [5.7666717350829124, 158.99918354126859, 24.433204421626005, 46.044651829964145, 11.143750996310521], [9.133300619209356, 0.0078031813485643876, 0.00045496990924390158, 370.4958015845404, 26.110150070946268]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_constData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[[[1.8533012765387555, 3.3263456896984942, 0.62352451493816152, 4.7321285317336992, 3.5976612620288138], [2.5875365728261288, 1.1789687904220709, 3.1033239263028771, 1.6056054416506054, 3.9995587910626007], [2.1334631319930328, 0.22649591502302446, 3.1844923408391028, 3.4739707497134966, 4.0020755247357558], [3.8421466415387462, 0.039577933086326927, 2.2133686091424845, 1.3239453269837249, 4.6019493691748243]], [[4.363420254564927, 0.89534437789298438, 1.4987448937704249, 1.9570449776372383, 0.90971164424482531], [0.87570877087026577, 0.17757420944601451, 1.5348507199957631, 4.1989686800187744, 2.3722028700560411], [1.5078885983977304, 2.8633084279976364, 3.2707648953948687, 1.611986731966401, 1.5286122676661005], [3.5100692456712093, 1.5899287605732848, 0.69610777068759688, 3.6096491884043385, 4.823006534027753]], [[3.7530085822237926, 3.638694762218742, 4.8712971060750316, 2.0515630273735077, 3.6576949379558319], [2.9550168181264107, 4.7654719048355636, 0.50897096252212937, 0.65190009830231477, 4.4213108551813116], [4.5342738596220373, 1.8764326959603104, 0.40312818449750776, 3.3373158658298863, 3.5358357832816867], [3.189502930840908, 0.67143879120694772, 3.9211081844630393, 0.276806031970594, 3.7669352973236272]], [[3.6967453290820931, 2.488537005765354, 4.6115880504233706, 3.9098776892233147, 0.12835091739116183], [4.2841181782928839, 0.97437820331074654, 4.1557328620862259, 4.5371374016016004, 4.9565917023426938], [1.8780067692620106, 3.8776855088852278, 1.4804080382355818, 4.8364708802715972, 1.0617578433568047], [3.605864679088413, 1.3487753043313828, 3.6459871127850993, 3.9919380972374912, 1.2009360875518402]], [[0.66003672620194531, 0.517568224114199, 4.2242040334646571, 4.7903168749401965, 2.7022085408808452], [4.118055169048608, 3.0833249900171635, 2.7957586298593728, 0.88054173119929857, 4.0659169195525937], [0.50208369443266876, 4.7469137370160484, 2.8611127656325417, 0.22857053516935935, 4.9338389441347363], [3.2251952298097235, 1.5206848636714334, 3.381007795732534, 3.6101303001541893, 1.6011012050849833]]], [[[4.2542455084171502, 2.8043528853012347, 3.3528864559324627, 1.4791638963655163, 4.9049411783538766], [1.1101933138044173, 4.5053391377623466, 2.9999455845042151, 1.5710867479329291, 4.4854812067697649], [3.5480634197529266, 2.5087026955038647, 1.8411394118436817, 2.091309390911297, 2.9810327513287991], [4.2802955812218109, 3.7213864123014022, 2.5007574527986405, 4.6506622011805838, 3.0942378246890154]], [[2.3690277587587554, 4.6067737117990823, 3.6327856295500438, 1.5400177603735259, 0.48821051642475533], [2.3569492770655209, 3.473749326260366, 3.4303692833882353, 0.3407721487205661, 2.1274528119977512], [2.5222447936210575, 2.4061069832326192, 4.4915704370845093, 1.4972800196726166, 3.2853482860845915], [0.28149853389073409, 1.1818507028793825, 1.1618271127212469, 4.5613680045562335, 4.8239274143311182]], [[4.4925818071893042, 4.0374165997546632, 4.5420976175082819, 0.26907220187794584, 4.3466271322521601], [1.9378523080535373, 4.1934279877642417, 2.9740906324082355, 4.8253726053193127, 1.8038469366624785], [0.5276638479348621, 4.6740214690830557, 4.9063379482105223, 3.1748975591390098, 0.45421985305583784], [4.0892581224698281, 0.95142542038006783, 4.4477940470568722, 1.0349016412494347, 2.8455632815517165]], [[4.978748123437855, 0.65014521296232874, 3.6757527562836954, 0.04383229407515897, 1.9184093666004138], [1.4487724840894236, 2.1834690367202008, 4.901087734790142, 2.0713928550442762, 4.5370800626865728], [0.73059324565627293, 3.207223650320838, 4.8448947777060773, 1.3871094294380104, 4.3961321065805263], [4.1523027866319149, 1.1497598826539595, 1.4746445036173439, 0.73020019733936514, 3.6268773301455193]], [[0.58306750780358962, 1.7661975484435626, 3.3750783265655424, 2.5334062754125841, 4.1585208550939727], [4.8672803983003714, 0.34354947315249973, 1.8968348332915805, 0.74951245676723122, 4.2843624000855893], [1.0507631882613111, 0.30116948792864351, 3.3935696195010041, 4.2833421612327021, 1.5562263294520566], [0.87872170068258781, 0.83083755335137555, 1.3360901672318146, 0.74493112996547184, 1.8703276655345111]]], [[[1.9622293952472716, 3.4014862119313922, 4.8402403458103409, 3.3978698995911594, 2.0513375641244362], [2.1118997530085299, 0.63112554805440346, 1.527427535470121, 0.55295857680664984, 4.7216068680815466], [2.1119793664674407, 3.4307889122305326, 4.0209550213700291, 4.0628615648005306, 2.0022919487880309], [4.5242723268563196, 1.5532249856802889, 1.8620124634477762, 2.8489433444932328, 1.7932299243445691]], [[3.4121990283292734, 2.7228716147476462, 1.1260458348618567, 2.4303796824868904, 0.7185680686614081], [1.0936832621723009, 2.0593620547362423, 1.7718804942553064, 4.8024201252939269, 1.1576852269389961], [3.8299752466272956, 4.6783875708580336, 0.11872934588132263, 4.0764509591535179, 1.7011728419322647], [1.108467186956384, 4.9029617479292007, 3.1179143969416789, 3.3344024167989219, 4.9149747198236433]], [[3.2578759183284851, 3.1424188168760625, 1.902347027212643, 4.2935923549009285, 1.8905050603644], [2.0003771761426368, 2.4064838506490722, 3.9042032070928876, 1.2513746274817685, 0.64565413603645028], [0.38376474409368178, 0.86976631694351136, 0.62534929671602646, 1.346970673446793, 4.5965689930049489], [0.092078054477530119, 1.0959446099191288, 4.8917917447728065, 0.48607344936422903, 3.1714679404326338]], [[3.2905801025548826, 2.7265982199358825, 1.3577308426151182, 1.3348838650659234, 1.273826497913976], [3.9864518864718641, 4.5407019535593314, 1.7308451814671133, 4.0735868688251227, 3.7118084716050253], [2.4106320807029582, 1.192652927146612, 1.8000488723290355, 4.8427598347333349, 0.47592154440453249], [3.2366074638834736, 1.0891336575392152, 4.2904010003167263, 4.7590830336347869, 3.1072329681031716]], [[1.0596091405129167, 4.9676836211185664, 1.5642380604798771, 0.25629227841288027, 3.0174696613541512], [1.2937305707425608, 1.6581985059256317, 1.3743007638255282, 0.54734057044351003, 0.87490046655526388], [3.877578879937424, 0.9791242155992127, 1.7194994782572253, 1.6086236428427845, 0.17854283822757303], [0.19389780668939471, 4.8547128093425176, 0.25761576941940462, 3.8533051291911815, 1.534043303328348]]], [[[2.9138631553606857, 0.64450875265245888, 4.4012423141922863, 3.2434742200309721, 0.84372252431217809], [0.12124907383729609, 0.74876904870662142, 2.8144058476301619, 3.1655232365615507, 1.3508660199633968], [0.26534696164320171, 4.4164371673244984, 3.7869493384521995, 0.061783389867352094, 0.2874221178718514], [1.2869154919501165, 1.9214355105245944, 2.7744792039628998, 0.31122491209307229, 2.9042513947420403]], [[3.6567814404568373, 1.5683224003939895, 3.6471399157341526, 4.2077888208078615, 0.84890990179451598], [3.5185798707283658, 3.4519324213328915, 0.53112699200364522, 0.92906652114194188, 1.9131344664959435], [2.1179210834079321, 1.83821323098681, 4.1235727577805399, 2.7266271018446653, 4.9093887927048119], [3.7846402557753445, 2.6738075843161639, 3.1704711375294501, 1.3449419947504124, 3.3187822579195756]], [[3.3067868207050966, 3.236054557974942, 3.5602737896700378, 0.23860310405031987, 0.21724474227957519], [3.1205849934642198, 4.6642318937224925, 4.0463864358079613, 0.3281907274332222, 0.75424528877031261], [4.7564906404742526, 3.8299113644754015, 3.1065517514167826, 2.8825741879857443, 3.001761120321897], [1.5250410390899574, 0.041706946844434045, 2.0651181745578486, 0.19899731394806638, 1.8023971981667726]], [[1.2307318024319696, 2.0989408938495924, 4.7883994963561642, 3.2538092636905351, 0.27381490327942404], [3.6465204426652913, 3.0875782745124063, 0.87959113864333294, 1.4166457978445075, 4.7945038059974143], [2.9884001808796952, 0.21313221261327661, 0.7272611733999278, 1.4648982705971272, 3.351809648136399], [0.81345873786643574, 0.94896723948594697, 4.4973838793390897, 4.9991616152643878, 2.8227121549736971]], [[1.5996714636236655, 1.1710728775738894, 2.5299567193411785, 1.3919821290311047, 4.6708525928441285], [1.3591707382518126, 0.41614807271231224, 3.3771904995112836, 2.9599875093475005, 4.8231599772156191], [1.7572985816817011, 2.1872857700995048, 4.2973058511543671, 4.421585929793844, 0.55885493083137183], [4.96674529624421, 1.3402237788624545, 3.9092902649921131, 0.53599309546905027, 1.5738416605022321]]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[1.7098900280913045, 0.40863436884547233, 0.63518359001010893, 3.3250175317362296, 4.3991969023537596], [1.2345050163490083, 1.0388396854703157, 4.8712890963567972, 2.8974670455538551, 4.5812383968202193], [3.2916205611506721, 4.4465830971040932, 4.3163072472872015, 2.9638175628292389, 4.528132401403111], [2.8042674171271997, 0.012169525405441065, 0.43703581265198843, 2.9301887329278196, 4.4240263055361293]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[2.7024659116905543, 5.9556984437550762, 1.3972089894132917, 12.660048500610605, 6.8887527098903574], [4.0069478014161666, 1.88218458765783, 5.2841571585298439, 2.3662192154334618, 8.5461385748467418], [3.1407185674257745, 1.1291882256682977, 5.5193175735049227, 6.4465097135699452, 8.5576840730397663], [7.8541268047519965, 1.0214577292263474, 3.2782680082204854, 2.0344036795461604, 11.806135597871165]], [[0.020141452688889089, 0.44875663173914732, 0.26151111493464468, 0.17352613758217331, 0.44302356191933656], [0.45671213908221153, 0.85306691838511983, 0.2531961381750551, 0.023334977137314764, 0.11967612332225351], [0.25937989692739999, 0.077113766725257701, 0.053551027102947943, 0.23630722536733106, 0.25461368717739236], [0.043227344137036758, 0.24101838095874836, 0.53634865679531285, 0.039541697033587543, 0.013349519654050464]], [[0.18208713683904151, 0.19178321930226486, 0.10961343362101947, 0.39412631133684006, 0.19013657037298642], [0.26155560228468316, 0.11500639934063242, 0.79374582049692877, 0.7438922162033883, 0.13444863609225211], [0.12772952884896158, 0.42673055934186799, 0.83280464800595411, 0.21989360794726001, 0.20094822962633455], [0.23515089853208102, 0.73732493940317823, 0.16871228703038435, 0.88194464761152447, 0.18093988141183084]], [[84.906181306851451, 19.883989627926283, 254.85824981761837, 109.68581904979128, 1.1667363558673149], [171.95978826689347, 3.2242198047826069, 147.37920916647087, 233.05090781987553, 385.76205897307403], [9.5484427959604261, 105.5243772790834, 5.9219937185193929, 333.91904840564081, 3.5811194546327201], [76.123427839588572, 5.0557172772317678, 79.882928758571907, 121.05120198519, 4.2329308348630565]], [[2.6585820808842988, 2.1527303243474134, 522.08415763912035, 1207.7094347766242, 54.76827958378373], [446.11534210833798, 96.322841307707833, 62.909573042371278, 3.6856757756175473, 412.95506402372018], [2.1039106596165684, 1132.4995408839804, 69.304885491169088, 1.4029952560512666, 1493.8284998864901], [118.85154275596224, 9.5141105271744379, 149.70988746538617, 210.21408356801018, 10.717806881443176]]], [[[2.4503779991263146, 1.805420058107821, 2.026586604892826, 1.3656296386179196, 2.8103932368204196], [1.2634985862843311, 2.5834874638029679, 1.8813673307317815, 1.3923333261785054, 2.5727020917676655], [2.1116525743592192, 1.696400461773196, 1.4738426114884258, 1.5536023705872644, 1.8738861840432106], [2.4638625708064823, 2.190182387972492, 1.6935633593024142, 2.6638047012474368, 1.9191135200245504]], [[1.0944701764646674, 1.1918877124696021, 1.1484636912642636, 1.0604373928460398, 1.0187770836477736], [1.0939665697578376, 1.1415250655332099, 1.1396397167492391, 1.0130695895228274, 1.0844417298485585], [1.1008786489681679, 1.0960176262966883, 1.1866670879628747, 1.0587118800576516, 1.1333595057018591], [1.0107840670080157, 1.0460631281481001, 1.0452653000330145, 1.1898273420451595, 1.2017909175406425]], [[1228.2774954723473, 597.45460704665732, 1328.4516112949232, 1.5311786308911632, 974.83620927283494], [21.505637808707203, 764.8657153366662, 110.9469313058263, 2080.3579101940104, 17.394195640716166], [2.3059231459391305, 1637.051613801721, 2364.9032768686184, 152.47510940978867, 2.0527757717094564], [648.56487056369576, 4.5106786775555916, 1144.1908344403528, 5.1480624342686498, 90.517853172109355]], [[199.65204628873704, 1.9970048492640999, 49.918990478617381, 1.0477347082248574, 7.6973392497228126], [4.6704604360012292, 10.204767172570888, 183.82019779547963, 9.0577813027009988, 124.80010436657962], [2.1754424524115925, 30.324711426589769, 173.15340772317364, 4.3739138535648925, 107.42228461514316], [82.878279461248866, 3.3978980351141836, 4.8007937599266972, 2.1745330045525444, 47.389752726811757]], [[2.4288329588213164, 14.703690866987955, 170.1645278711114, 47.264841580612732, 560.67493494824748], [1648.8800898547713, 1.6868649882493265, 17.938055930331025, 3.129067352061591, 679.03201312884767], [4.9492169430092403, 1.5814952036217864, 175.02152409543982, 677.97844923403477, 10.681665650447535], [3.8090792720011968, 3.5413545466930283, 7.6406928677918877, 3.1073253825716636, 17.228782539280576]]], [[[10.358017804326344, 57.538983681609501, 319.43877156544454, 57.291615857474234, 11.51813302687852], [12.379912012641693, 2.1210481249825381, 6.1702949492962098, 1.9324406658304385, 277.33552718657489], [12.381086301548409, 59.583181621032786, 120.35978371880985, 126.52147709458035, 10.86439156440729], [219.23136512461835, 6.362880649853814, 9.1922840060368749, 29.790041663564988, 8.4690496007934026]], [[162.62791569856176, 58.142369250189439, 5.3667015345775795, 37.579433357020704, 2.9218040575753963], [5.113704650270158, 21.603345180835188, 14.067773281194395, 1294.4792294542847, 5.6261409047308737], [303.33681333616482, 1075.7682316375428, 1.1938224788611369, 438.17529916890396, 12.659161319412814], [5.2277644945418338, 1504.0030376695834, 104.8312783917261, 144.80409271731591, 1531.2053452156449]], [[117.25118354881045, 99.035067982872448, 16.151173919277415, 533.22872489623171, 15.873880783252941], [18.640787349531884, 33.758847918237521, 301.72248741982412, 6.2339524051311646, 2.5707681848888209], [1.7528055069855253, 3.5677903850576405, 2.4955545202648532, 7.1693158128665955, 830.49637616712516], [1.1441419510056703, 4.9664716217275364, 1278.902268889143, 2.0356881633221753, 103.33286272378353]], [[35.699739690655086, 19.344137503211126, 4.3716691829637915, 4.2644881338802527, 3.9907718673415467], [76.034933710658592, 138.84712098257913, 6.5569494933843648, 83.58492188430165, 56.418595873252819], [13.723354566045286, 3.6538863380097126, 7.0689628856987037, 192.78060258345434, 1.6771205444560906], [33.666500897402713, 3.2651964399299369, 105.78692833145161, 176.02741530271996, 29.251789196023655]], [[4.954706025200343, 1813.0035497212862, 10.617289208676137, 1.4726753072844314, 95.327024188321943], [7.056390614078941, 12.236130491763243, 7.9694829077915896, 2.2856574509065357, 3.7485554675515194], [349.44442414525963, 4.3875925556449245, 13.423092392763646, 11.353427720209869, 1.3095125375275354], [1.3402359964243327, 1528.6178946068121, 1.4756219542100879, 336.86547119578438, 10.143978708699429]]], [[[20.178297717354461, 1.9436745117939707, 93.532341469244429, 28.345933454602875, 2.3869061116650037], [1.1331768422500641, 2.1642793988744895, 18.211495506919331, 26.156690798503824, 4.0266515166920751], [1.3147026067379375, 95.009352192194811, 49.644309988736133, 1.065780579148609, 1.3449718905405559], [3.7696910355725, 7.2519724788515116, 17.476951634379688, 1.37839142111093, 19.979296515110118]], [[9.9601112503355499e-08, 0.00099332590502943896, 1.0392620550534407e-07, 8.7747718746384365e-09, 0.023690260440493056], [1.8318182338778325e-07, 2.4575027117712572e-07, 0.096169453654496431, 0.016637644612486496, 0.0002172055959676062], [8.8056349408651533e-05, 0.00030222102516560471, 1.2719983566427497e-08, 6.0152712885831287e-06, 3.9798240797187177e-10], [5.6682571628769885e-08, 7.5925933677100077e-06, 8.4999299123586024e-07, 0.0026595252150656886, 4.420181812126814e-07]], [[0.064753976121258369, 0.068658369393900021, 0.052497931445482392, 0.82077901959100608, 0.83541874189003229], [0.075544495859848043, 0.021051742514543829, 0.035106939585230028, 0.76211589827231874, 0.53562601321805492], [0.019503951165154212, 0.041996401032819174, 0.076427127831493927, 0.091994880458736092, 0.083352376469359821], [0.28299170937923268, 0.96606658955646241, 0.18097774721064303, 0.84813278977804907, 0.22494114200017529]], [[3.7551151235443596, 9.5496236318771075, 172.06107326225433, 33.051350459226001, 1.3422795447279185], [50.413207321515515, 27.642460187164122, 2.574407116144807, 4.5859116270721811, 173.19394427381442], [24.846823662285114, 1.2575072309507829, 2.1855212705439615, 4.8300824951270123, 36.723592270564694], [2.397730130006305, 2.7737583527033935, 125.8371049387427, 215.81743781211958, 20.792746966165613]], [[10.791856116479739, 5.705577292819763, 43.041840244356351, 7.9244056310137507, 1038.7697201284607], [7.547037726402257, 1.8567600849195764, 151.72229488868629, 81.585377471631972, 1302.8130085582716], [13.642517808541974, 25.85757938634239, 596.04106737473069, 717.03186860492417, 2.2957172970641357], [1612.9170280615085, 7.3373669507025756, 334.72553940875275, 2.2189822511468735, 10.385198510141258]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[3.6373797737254314, 4.5585433886202225], [3.9759491302475904, 1.8953025434533137]], [[2.2691978200194192, 1.7000974956206705], [3.8210567314606276, 0.21089571413420283]], [[0.867895240123157, 4.0787011750808775], [0.9572960915506864, 3.842095560685109]], [[0.40752371687434763, 4.8586866592659979], [1.046603897333761, 4.352555930951687]], [[0.034184275926923149, 0.090332967755422586], [2.2421228630428049, 0.8089494033847775]], [[2.4512967561678973, 1.3392890237910013], [0.90307347135709937, 4.0551638357626834]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[1.6933089346809411, 0.24294535777903498], [1.1159696750893844, 4.8077175965167198]], [[2.4913154796964334, 4.2253102461139864], [1.4950781337770402, 3.8213790390220201]], [[1.5763974510964862, 3.5844524832591786], [0.10328266261078495, 1.6594723590359124]], [[4.8491518797875761, 3.159993070443488], [1.3910563690805127, 2.671708119575626]], [[2.532427190794325, 1.8308638038440306], [3.0439169819703209, 2.2799635139133403]], [[3.4457439388839037, 2.8086460405740548], [4.4307478104490388, 4.9901678936952623]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[6.7920430058568648, 0.0015805721077051516], [1.5469044619912484, 19.610136893397748]], [[7.9355165543127431, 11.58832437429991], [4.6494398302776672, 1.3267458180040312]], [[1.4844075034524558, 182.52612690734014], [0.11379739172703203, 7.000770629130284]], [[1.9029398898008079, 267.80478647175636], [1.4126193538345955, 72.048183900750232]], [[1.0322731210303313, 1.056152179221528], [12.131534611799207, 1.9478053698455169]], [[20.751050018987986, 3.9871917648998001], [3.8354493949465001, 677.59679097151854]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Data(numpy.array([[[3.0217842074693371, 4.2284795529632913], [3.1979814096813288, 1.5786798678387433]], [[2.8473770583138012, 0.73129293047856403], [0.81112431312067346, 0.69581745512219151]], [[1.6931654944735206, 2.7999502038968824], [4.3362553495021174, 3.3841059754575435]], [[4.1534069386260963, 2.2231739841724658], [2.5713775071666634, 1.2767616718214316]], [[1.5828273355545468, 0.76786821557503504], [1.2432093970756293, 4.9263917080230106]], [[3.269087076274011, 0.61248847803838735], [1.7002769930926775, 2.906506980788977]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[[2.3487843378752307, 0.59026460649924739], [2.3289594255460768, 3.7044062840908429]], [[4.2912928164339617, 1.0131231873925668], [3.0137924581554398, 0.83312578913940194]]], [[[2.5626279623092771, 3.4953308097574194], [1.9741190950958414, 0.84423064199540676]], [[3.7955021089561329, 4.2390379237076079], [4.0165177657822602, 1.7645979068281799]]], [[[2.5865747146330262, 0.012859597248622751], [3.3289686108194072, 4.3466697191659609]], [[3.9045901516092956, 0.8370066622090927], [0.79536487254229382, 0.27990391853565283]]], [[[1.098500566401089, 0.55947852378538787], [3.9953051952396117, 3.9226489844802548]], [[3.1105862285191725, 2.4885433292769608], [0.59388867537424217, 1.3779803427408095]]], [[[0.95521016307018303, 2.4470879848782081], [0.89868051771496416, 1.3218647180843526]], [[0.01866618299879326, 0.71324098993744456], [2.0232646685922036, 0.24101946264636778]]], [[[3.8669199648700348, 3.9554309135844239], [4.5087086639729828, 0.16543507211266759]], [[4.0228636456677398, 1.0264097531250957], [3.5647003063284948, 2.5875345898408915]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[13.201034036884598, 0.20330715503553196], [35.689189207582011, 253.98906734167144]], [[105.43993815464833, 1.0425761525090396], [5.706457498836996, 0.74959621670089116]]], [[[14.577461044526748, 35.27898441277037], [1.6443885429145793, 0.8835304679897964]], [[2.9502536887059092, 3.2269464567128585], [2.6312926714230778, 1.4846374958508786]]], [[[4.9981786574423364, 0.00062893971573017545], [29.002881722615179, 61.207806915896882]], [[367.47028771025765, 0.46230952725182006], [0.46079332744217627, 0.013446825826067186]]], [[[1.4772710517614507, 0.089627629009400273], [21.744539827459029, 20.875187492919522]], [[18.504760269924269, 10.426165129647547], [0.51413275535291925, 1.5058445931619033]]], [[[0.9300366209249995, 4.1225429271267462], [0.92124470584491402, 1.2389553457243949]], [[0.0070885919056726046, 0.65696461981900378], [32.191074424149484, 0.00090312249058339348]]], [[[83.204713615283808, 89.593921270007044], [2.5153557724323785, 0.33221435836527391]], [[10.662952714762817, 1.0453180388450471], [40.221547075110735, 15.850994607421347]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_constData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[[0.72890125636110303, 0.94546824439029198], [2.8545873252624259, 0.68864368150086763]], [[2.4456923048914314, 0.5673289249511817], [4.5765118596436691, 4.1665575162987434]]], [[[3.5050017504612723, 4.7546222609046538], [1.9137117160049637, 1.6777498400769577]], [[2.3255525213691843, 2.1454769408013705], [1.4104405188868891, 2.0093074335005454]]], [[[3.0381089365234959, 2.1898096346716769], [2.889369851059167, 2.2874499819781469]], [[0.45552569942386167, 2.8257260609893518], [0.15734281409661302, 1.7886411676023051]]], [[[4.7615859350451482, 1.0515095550268589], [2.6648608953861253, 4.3861280132499703]], [[3.6681196794784268, 3.132470653998384], [4.3312942602417603, 4.1942220885109984]]], [[[2.9002283255750982, 0.42316965700145298], [1.0299675378097557, 3.1706346054718852]], [[3.0318187243744306, 4.6992872746047176], [4.4540656429265146, 1.1713522888838079]]], [[[4.7437996242136977, 4.36579525569128], [1.5563121954750283, 4.8264781248638009]], [[0.99999854147618583, 4.9817730849187187], [0.54079469381144696, 0.53538964318514748]]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[2.1532928370884146, 2.4122694798532676], [1.9180433503618499, 4.9597851867653224]], [[4.0292455466121462, 4.054914404935297], [2.2292219013640926, 1.2428043406761446]], [[4.021191453733687, 3.8618229599084675], [1.3733287347642842, 2.7982834460843482]], [[4.9115790874970724, 1.6439786808210508], [3.3855099946510889, 3.3775278711110173]], [[4.5283839098758971, 2.7555006462099128], [4.2036293662929873, 3.3633375323410868]], [[1.3344488060816464, 1.2886748132134869], [4.6089601664426976, 2.3534172617746516]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[1.7490380797743847, 2.0650872204221278], [12.350026935054336, 1.8338133332788633]], [[4.9179562133095382, 1.4470174303309615], [1523.3302263008306, 790.10723113545566]]], [[[132.22398719262054, 754.41487207693615], [14.571413596990414, 10.472278541915578]], [[6.4513230158847215, 5.5841217629012485], [1.3587810449202724, 1.5476906904623344]]], [[[68.563908941125632, 21.058229585087098], [49.597542077711793, 21.991668171155922]], [[1.1554730717102071, 2.4508290261738166], [1.1757505706280071, 6.2998309828598842]]], [[[1955.7298557449951, 5.3312080968997915], [3.7612594421653776, 8.8501022572902919]], [[87.644483266385066, 45.607046850132072], [194.76798049480465, 164.83952572193857]]], [[[79.870271472250423, 1.8948519163429103], [2.8404833168261403, 24.872245355730975]], [[77.752826467116094, 852.29568262311398], [221.95768147724539, 4.1403187434997557]]], [[[3.9301324688851769, 3.5240536928964437], [1.4839436986662164, 3.4009827034312958]], [[4.6089498947978456, 2022.643479488135], [1.5885936107918328, 1.5812617291752864]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[2.3776825047805645, 0.55831637405362211, 0.91768135232215231, 1.3220482060065428], [1.1022514119832105, 4.2359057483183706, 1.3652152553609782, 4.0935604480993568], [3.3844637721671043, 0.072526166446539669, 0.48377105529457504, 0.84438543272263045]], [[3.5981681758049517, 1.7910791142718845, 2.0759881099982351, 3.2191377582891358], [3.0607945532576273, 4.8424979085507722, 3.7628683651364523, 3.6984140259894827], [1.0961285316741947, 3.0417702169721257, 3.9505061637260543, 0.018842332558307275]]], [[[2.8059901204217903, 2.7797035536850827, 2.4808145753822375, 0.37512893834566441], [1.3228224269341353, 2.0122620045540982, 0.19005070809711402, 3.2144487148652905], [3.6592660010440219, 1.5972117116702809, 0.58563655985677054, 0.63860070642300204]], [[4.8404442882250693, 1.9940170986636689, 3.8856275799123194, 1.1092589179771484], [3.2152823347402362, 2.1051603483016752, 0.82180719097371102, 3.8084904994626387], [4.8980684411300004, 0.99063450643943152, 2.4990751153938779, 4.3842791925622553]]], [[[0.20065771270469407, 2.7895560137870472, 1.7621740475034964, 2.010229305113949], [2.49114145027995, 3.97941955639378, 1.1739938746203664, 1.5051208543635408], [1.9385592223298109, 0.71177550376119858, 1.3888929317269152, 4.5979956602254575]], [[3.4710943072299738, 3.0265723296971911, 0.22552933677460174, 0.9725688300130545], [0.014820036060761344, 4.0806664856496013, 0.4102537353965402, 2.7876882354990222], [2.3727012659231592, 3.3898712730890317, 2.2053426948477668, 1.8190725439226165]]]]),self.functionspace)
      res=power(arg0,arg1)
      s0=numpy.array([[[[4.0928881271618422, 3.8075656220429721, 1.3006742066759851, 2.1588200165836033], [0.43496080819646327, 3.9488694242707099, 0.12040045424018646, 0.72579025732583202], [4.1716186121032255, 2.9136322149064342, 0.21662866334150682, 2.2040116880841549]], [[1.8720603629404955, 2.56266608726804, 4.9788538207373128, 0.15809427583591135], [1.0818460926908311, 1.4555958803576969, 1.8718697301099836, 0.50439328997318444], [3.5021631112881813, 0.85499109227249404, 3.0849194879211006, 0.074663404435456035]]], [[[3.5830514870943628, 4.0488435171323625, 3.4458525466055399, 0.78338812190096008], [3.3732339237339035, 2.8451012414542429, 2.0329913766455445, 2.2853402105287239], [2.9415141497971544, 0.73611802740989951, 3.5825462489932218, 0.48993206328425243]], [[2.8543494026123915, 3.2651170822996001, 4.6941928187441375, 0.38904147444264542], [4.7683412563431915, 3.9502259549564647, 3.9172719567622174, 2.5305521001126197], [3.1309214080349888, 4.5178733726157363, 0.17378257207513262, 2.2095837771611055]]], [[[1.4199107959923349, 3.0101484298732859, 4.4374018420594847, 1.5945747967273172], [1.8714968860908747, 2.9573040478898607, 0.99946722187793846, 3.9924705840271923], [4.0855099266440931, 3.5266682184619631, 1.2602042917971876, 1.7108336716004262]], [[3.7860686830415031, 2.2449096894428822, 3.2041170786376485, 0.36945241253973782], [0.57092542082891362, 0.10822128603288286, 2.1262840284012032, 0.49089429579244376], [3.9394491476032694, 2.4603577829854508, 4.3345623394537069, 1.6994167396648896]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[28.524150806662217, 2.1095263246584919, 1.2728297838503451, 2.7659876704474251], [0.3994672151433224, 336.20366950725503, 0.055572443305678412, 0.26929107739676628], [125.71828107721457, 1.0806465481351171, 0.47713204951664978, 1.9489684635052213]], [[9.546685396570691, 5.3951030489988741, 28.004788351894884, 0.0026375500936920127], [1.2722531000194985, 6.1592072675396725, 10.581258452069298, 0.079564220412680076], [3.9505960498460491, 0.62093021143918481, 85.65659016813548, 0.95228451461808916]]], [[[35.91108933488055, 48.775318850621005, 21.524519152405833, 0.91248912639702806], [4.9947263453564963, 8.1990512046707185, 1.1443565951979238, 14.25046822715789], [51.83531195962108, 0.61303732107835474, 2.1113244768296138, 0.63404567273441381]], [[160.27172319725915, 10.585781183393349, 406.85124627171018, 0.35091287150434292], [151.75481846391375, 18.029548337129217, 3.071272054379135, 34.32745252865562], [267.81579951148956, 4.4545134402920565, 0.012610097902246275, 32.326094708250444]]], [[[1.0728829812410283, 21.629565013563575, 13.815083209852519, 2.5548341195657698], [4.7649853678531642, 74.798461102645831, 0.99937455074337933, 8.0341781732386366], [15.308657049268311, 2.4524531338689757, 1.3788014502187904, 11.811047133883946]], [[101.61241423703471, 11.55922708298711, 1.3003254864660578, 0.37968276446138077], [0.99172782317020736, 0.00011464421912164931, 1.3627238368191192, 0.13758492228741773], [25.869650515550749, 21.156011338621266, 25.391161128576975, 2.6238089559229221]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.78872646591)+(1.-msk_arg1)*(0.848201053395)
      res=power(arg0,arg1)
      s0=numpy.array(1.71332262891)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(13.1762471001)+(1.-msk_ref)*(1.57885633543)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.3686857987)+(1.-msk_arg1)*(3.91406319067)
      res=power(arg0,arg1)
      s0=numpy.array([3.0747444377169217, 3.0380299756239242])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([43.98203637826483, 42.237771952207076])+(1.-msk_ref)*numpy.array([81.155012130531063, 77.427599894790191])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.13254081020840769, 4.3936816002942827])+(1.-msk_arg1)*numpy.array([0.99226663066351573, 4.286863576712312])
      res=power(arg0,arg1)
      s0=numpy.array(3.74097089106)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([1.1910878847147843, 329.23856733342274])+(1.-msk_ref)*numpy.array([3.7029958853431757, 285.96023786798821])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(4.93696541945)+(1.-msk_arg1)*(1.35887083451)
      res=power(arg0,arg1)
      s0=numpy.array([[1.5078080553722337, 4.0929813860623598, 2.5796960914662455, 2.5242517328319978, 3.6962524280853986], [3.7634027515962405, 1.0032280885036917, 4.8225674482448202, 2.682134562026476, 2.4753772097624211], [1.2301963107658813, 3.6526041106515019, 3.5183586857921805, 2.3754974316270991, 2.0375320732713109], [4.4657640048216187, 3.6291093934690548, 3.8106089601630475, 0.96373223594836521, 1.0254184928742891]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[7.5943101006507483, 1051.0398383658544, 107.62165781369784, 96.675193849111182, 635.35945673187246], [694.41965171850165, 1.0160385522623463, 2362.2257004932121, 130.43487310312739, 87.779665402533467], [2.780997520028802, 599.16936276235651, 498.03772653925978, 71.62873718091933, 33.576699150864464], [1616.2663721424037, 580.38146642644153, 738.49776321162255, 0.83328436338896239, 1.1319274137911888]])+(1.-msk_ref)*numpy.array([[1.7472267023176233, 6.787080377354358, 3.6246701650065041, 3.5192194354941644, 5.9090097994544006], [6.0553579904824009, 1.0043890944077314, 8.4817793022922974, 3.8216396974178934, 3.4269502653412629], [1.3251456509504245, 5.8143914816064788, 5.5259333748891013, 3.240424593150097, 2.6304680252492996], [7.6405461942848616, 5.76362841767956, 6.1588031589258723, 0.95104003812845161, 1.0346971398769065]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[4.1040031454298633, 4.0931921817270007, 1.9491441274256893, 1.4992980500897748, 4.5539305927979647], [3.561141568481919, 1.3019092130450294, 0.78639794954212372, 2.3255664849777675, 2.7568855273408852], [4.7641393723501269, 0.28638546863580455, 1.747061134404279, 1.7598157868274746, 1.7837065381189026], [1.2548593902494551, 3.6867048174984043, 1.9148515611913279, 2.3640509251777488, 4.8451130183064191]])+(1.-msk_arg1)*numpy.array([[4.5771597260658323, 2.7062351554709676, 0.76588399916552596, 4.9145331443912079, 4.639687001153721], [0.67379888438406843, 3.8655553099830438, 3.7720783794195869, 3.1222638345446194, 0.30554432571693513], [3.7616672291763145, 3.5212799356944267, 2.455216598293529, 2.0979096511538486, 0.25658687754652454], [0.93479599778421185, 1.8694818728106022, 0.97243232391362833, 3.7033500324934181, 1.9397358628119694]])
      res=power(arg0,arg1)
      s0=numpy.array(3.17942903313)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[115.25039778243337, 113.81815850049689, 9.5312720944483598, 5.6646238900482109, 193.93808249064907], [61.508677455553176, 4.5083016602478532, 2.4833980185396292, 14.73147773601775, 24.261578974139976], [247.32117211174852, 1.3927253887155797, 7.5445727538160883, 7.6567054359076288, 7.8712451328458668], [4.2695050356176507, 71.123476819299, 9.160602252414531, 15.402064543880712, 271.60536051878279]])+(1.-msk_ref)*numpy.array([[199.21967732947991, 22.88099421737887, 2.4251642917076071, 294.31437695172883, 214.16216815219596], [2.1801294783209415, 87.469847422937733, 78.505541528681903, 37.022567935005675, 1.4239342227069287], [77.565800942811805, 58.737023147144868, 17.114961877377013, 11.320955936474489, 1.3455387087804949], [2.9484513181581566, 8.6922578924004377, 3.0796441525952085, 72.506123764441867, 9.4281098539084311]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.53899318577)+(1.-msk_arg1)*(3.85022979605)
      res=power(arg0,arg1)
      s0=numpy.array([[[2.5650693629730785, 2.0917082361753758], [0.83012534278060945, 4.1091353760809302]], [[3.9787816626199337, 3.1135719982010346], [4.0492291464155699, 2.1919345113725317]], [[3.7995467033174046, 0.49322081894947206], [3.7895604196524015, 3.6820468537004518]], [[3.9342934879525053, 2.9910935147628117], [2.5398374332301197, 0.1337517314843856]], [[0.97893765740384231, 1.1257693552811521], [0.10819233918463116, 4.0074619678824384]], [[0.18238140478512208, 4.6192324928876713], [2.6845513699667278, 1.4638787129977295]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.6615049309262946, 1.4884968242812888], [0.90452170040865165, 2.1419400432689066]], [[2.1050442096328368, 1.8444343120691666], [2.125051992495246, 1.5265243639449786]], [[2.0533901899130851, 0.68320558239932971], [2.0504795382486964, 2.0189161444672372]], [[2.0923249163409126, 1.8049664043667568], [1.6526756648758976, 0.33812815684475056]], [[0.98859185255964055, 1.0659354065115636], [0.30160497461297409, 2.1132092528101372]], [[0.39964423803939353, 2.2813851234266549], [1.7027815426808135, 1.2280219192664967]]])+(1.-msk_ref)*numpy.array([[[37.594611084693604, 17.139687909968295], [0.48829756542185376, 230.71735680021573]], [[203.78665222897058, 79.279365048030783], [218.03345442811749, 20.524117052722001]], [[170.64776341108183, 0.06578681363800197], [168.92735222455724, 151.20770810984317]], [[195.152312090116, 67.928935668453477], [36.190602538198327, 0.0004325655877559781]], [[0.92130782690730351, 1.5779458643143465], [0.00019117655352615329, 209.50083747685022]], [[0.0014275959882414952, 362.03164049954603], [44.797592028889852, 4.3374317105571949]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.4791507786608229, 1.146500334416269], [4.0475245428960553, 3.1568556718576812]], [[0.90200297927015116, 0.2081451774485763], [2.3950803834524712, 4.9344838023201767]], [[2.317616996242684, 3.7444793083682395], [3.5240697105155343, 4.7691978462698437]], [[3.1794364939363704, 3.3541312216699866], [4.4262063309319144, 3.1603091654520239]], [[0.38271126101433395, 4.8191891256872257], [3.0016488356335267, 3.3272351285651833]], [[1.3331285288233794, 4.420799199080558], [3.3984653525910393, 4.082381346460374]]])+(1.-msk_arg1)*numpy.array([[[1.2594166889853475, 1.0159511028001531], [4.612291648265666, 4.3110691051635976]], [[4.6951026126035389, 3.8477939182911798], [0.83495207143294614, 3.3931007634545871]], [[2.9279496686323245, 0.3387202040236672], [3.0397657425269373, 3.0983124410559926]], [[1.0617147013084123, 4.6983535893932267], [1.0867449799766606, 0.5260874942461915]], [[1.6437897103055221, 1.2492783946699688], [2.3299559099049318, 2.7528601355226456]], [[0.22990932983058196, 1.3906283806053581], [4.9598572317949294, 4.9471427119742328]]])
      res=power(arg0,arg1)
      s0=numpy.array(3.84065123537)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[7.3185142085570138, 4.6775709138291326], [231.9492963084115, 69.965160122970019]], [[3.3661613949526412, 1.323247419161331], [25.101324785833825, 765.13200933710755]], [[22.616565896776702, 154.27371866459978], [114.67889652334952, 612.55128211901376]], [[72.123729322208348, 91.236921861416349], [386.09710110077373, 70.291056485552957]], [[1.6736256575997119, 655.17542701464538], [56.777753012191873, 87.993869659571871]], [[6.0129391132108365, 383.29802956667925], [96.845558214465129, 243.088013763905]]])+(1.-msk_ref)*numpy.array([[[5.4451424770983756, 3.9239798936906518], [495.95892132934938, 330.68141342964367]], [[554.42248837175111, 177.2839866650971], [3.0757430956573084, 96.148965247265366]], [[51.417155752268521, 1.5774286964666047], [59.76596103331503, 64.664940624906151]], [[4.1732196468139655, 556.85320600888838], [4.3161751508187916, 2.0297759121842103]], [[9.1335311436923998, 5.3713614677497992], [22.995220772118721, 40.624204805106871]], [[1.3625741086050118, 6.4966579200254664], [791.70744120023767, 778.27718525184048]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.522463994721)+(1.-msk_arg1)*(3.95798406927)
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.9778249935746528, 2.7864495641057689, 2.329250206871448, 0.33836036760584282], [4.2591563083630408, 4.4145364053130374, 1.5131983550651795, 2.3780363313386608], [2.0245863392065901, 1.5826845938744423, 2.8202683820177548, 3.2284678479574702]], [[0.7158968504642631, 3.2635067205830373, 0.68043653625035216, 3.8395488853700286], [4.0784138552535705, 1.4624246142518504, 0.45609468844581835, 4.823589827790987], [3.4262437456576591, 3.9745004371528552, 1.3544300946232177, 1.80594239208108]]], [[[4.8312628629798891, 2.4156065104032134, 0.88842870194784973, 3.7470180348019926], [1.2674690519727458, 3.3880348671556502, 2.0762107974565422, 4.21037502622021], [2.6878985242398503, 0.33072286408315749, 3.4678569531319172, 4.6917605271016471]], [[3.6579717757894796, 4.2908847926583906, 1.6029082129395789, 2.2859773270421275], [0.33074081060092103, 0.74235943250782843, 4.9666544599542304, 2.2681671051935797], [2.6643884590594991, 0.32784659163832769, 1.2023574721416899, 1.571922725707966]]], [[[0.14792836767860298, 4.1907474700440996, 0.98190928084177009, 0.36397390764400356], [2.9618391186125441, 4.5284953520944642, 1.9177207064971398, 3.061009284984257], [1.0123497561896682, 3.3082033117872198, 0.95644551739713557, 3.4158124686031499]], [[0.55957253998169654, 3.9196940605683572, 0.4786579321156002, 4.5467608146882297], [2.9387175139107078, 3.9243122386631297, 1.7654755227882744, 1.8831106179009411], [2.4909693824542845, 3.8508851803524604, 2.7431244065690592, 1.360782607534508]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[2.057279380558152, 1.7081390480995029, 1.5554541460214797, 0.56769847470874291], [2.1320574891492337, 2.1723472418748262, 1.2416211731335021, 1.5723912296552798], [1.4456051546806978, 1.2710902329760156, 1.7189393014667642, 1.8447279449630336]], [[0.83977818203050136, 1.8551612394008856, 0.81778201410724027, 2.0196000281294473], [2.0842975592197566, 1.2196773364938367, 0.6635421581028359, 2.2752881958422702], [1.9029320570546431, 2.0563808675506543, 1.1717584037858797, 1.3618163411084439]]], [[[2.2771784697760742, 1.5853216625275146, 0.94006340829889345, 1.9940230126307246], [1.1318296489280484, 1.8918150745731221, 1.4647479764951525, 2.1192643338463788], [1.6763039286471082, 0.56096705576352979, 1.9149723991472347, 2.2425844335488114]], [[1.9691228116256316, 2.1403409282752168, 1.2795503913921478, 1.5402887697886005], [0.5609829596793956, 0.85585579243547549, 2.310299850313775, 1.5340072289811066], [1.6686274851779692, 0.55841280049472031, 1.1010693609651538, 1.2665671668019989]]], [[[0.36845272515949562, 2.11409694449274, 0.99050705602306421, 0.58975958793303229], [1.7634935949933539, 2.2014676535884123, 1.4052227348592867, 1.7941005706652302], [1.0064333917218227, 1.8683929173067206, 0.97700248801439626, 1.8999029478086471]], [[0.73835300106849877, 2.041516522182389, 0.68049465642542872, 2.2061024194735683], [1.7562875224657715, 2.0427728577396338, 1.3457870666303404, 1.3919150329070984], [1.6109728162590951, 2.0227132268708119, 1.694210998653281, 1.1746265276527335]]]])+(1.-msk_ref)*numpy.array([[[[236.25881306200796, 57.743819396120607, 28.407671871317433, 0.013718022514627042], [309.63689154828745, 356.81652786880892, 5.1525824512539078, 30.836620205802191], [16.310760234800327, 6.1545989624538917, 60.567881344340641, 103.41889720170656]], [[0.26637925530613094, 107.93321478169095, 0.21785929009641705, 205.38642200929041], [260.80434002401944, 4.501510175849079, 0.044724467583760905, 506.72116015281978], [130.8585598242108, 235.47824060181568, 3.3226951987786055, 10.375998909416314]]], [[[509.91903538668026, 32.810402927700444, 0.62610762451572477, 186.48297608604409], [2.5551973382657192, 125.177199577994, 18.020022382568087, 295.83646564364574], [50.073655144706173, 0.012532769125458349, 137.26301257186552, 454.08436486321892]], [[169.54942971035047, 318.86758450192315, 6.4718015057550549, 26.375514111081053], [0.012535461105953811, 0.30753403603824336, 568.86604333692276, 25.571498139050643], [48.362450515226442, 0.012106880046410102, 2.0738232554316549, 5.9906166806249361]]], [[[0.00051889193462074815, 290.41551208886534, 0.93029050570038574, 0.018311463141930575], [73.52461605621464, 394.68915253579036, 13.160122751648277, 83.761516656348363], [1.0497801607983059, 113.9036441371109, 0.83840434913438056, 129.28877785909904]], [[0.10046611875608689, 222.8859235375904, 0.054143344294312064, 401.02778877641799], [71.278947375322801, 223.92711809679122, 9.4858156186658835, 12.244869603360574], [37.052692402397746, 207.79706373206378, 54.271112493950945, 3.3848056749251243]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.6234674363747663, 3.426662703042116, 4.1893115129216829, 2.1912681441676454], [0.04624312844937617, 1.0939487834509605, 1.8400250294237654, 0.47642764397128123], [3.3458410004880346, 2.0707690970416626, 2.2573681206425618, 4.7293686775350166]], [[0.36497027245723168, 4.2948445827353074, 3.3057983663558632, 4.7707688172935265], [0.13176065787574126, 0.13333939103187759, 0.92684406953928555, 3.2301798269794149], [4.0620462037765535, 1.8841799334226641, 2.1711000805270979, 0.75458891967531272]]], [[[3.9177408569244614, 2.388625987403513, 4.1751558350966924, 2.5302986645445578], [4.1129048700676627, 2.3146445799836846, 1.7235401672130799, 4.4835988658456341], [1.6657671927215945, 0.63343026930008717, 3.4998378462491915, 2.3298288292676159]], [[4.2778762307211347, 0.49319126287589421, 4.4879492014126425, 1.9967487325681967], [3.6828088395228731, 1.4041232904547396, 4.2850973066638893, 2.6686080450548459], [0.96618819660791244, 2.2870687759180686, 2.7053761942881023, 2.4692966904336537]]], [[[3.0574189560787945, 3.1560253498331572, 3.9400655198570766, 3.5094591758443299], [2.5526474286646326, 4.4442228481571915, 2.7778631374789362, 1.8402299650284082], [4.5354027975813631, 1.6916090260597927, 1.4157740345102912, 2.5740119143251192]], [[4.6240315851455778, 1.6656203559272198, 4.9904598246743532, 2.0890418941046156], [0.088936354500770531, 3.2286903529260833, 4.3145514906429323, 1.367138583501962], [3.3319632059511557, 2.0970714515157232, 2.9456461583491724, 3.4704303547886672]]]])+(1.-msk_arg1)*numpy.array([[[[3.1240313779549309, 4.0799827244698754, 1.5202614480757985, 2.1276003307677867], [4.8830817344076367, 2.1252140820018188, 2.0432360659796704, 4.5189145056884845], [0.73525474716149064, 3.3536384763110112, 3.9056864286464612, 4.4199798108400623]], [[0.36923852488327974, 4.2743813482808477, 4.0516743853061055, 0.76184525468049802], [4.763693314065975, 2.3009625486940393, 2.903105947799101, 4.2289363651622649], [4.6463945231832788, 0.24064207894692041, 1.4680891444248829, 3.934760756996472]]], [[[1.9487158694360718, 4.2711836161555299, 4.131781632392336, 1.3792103120912254], [0.44730295035905188, 4.0150784652902214, 4.5267746218612901, 2.0925946896888976], [3.618656566318899, 1.6088403335774302, 0.53258081093377585, 0.45296779880119581]], [[2.8825053663363032, 4.6570892741837326, 2.0446858283857359, 0.1162714012053019], [1.5965688961039533, 0.21715431615588529, 1.7203663204755153, 2.9240356064996322], [3.3341723823466176, 3.9698910292688847, 3.9749750879837489, 0.75758725585179287]]], [[[4.3832890768094988, 3.9555001661947764, 0.14497395112393421, 2.939217707346518], [3.2895187433585278, 1.4448144365229676, 1.6105013918990336, 1.1194304604958771], [3.5823766868405014, 2.484882284292889, 3.5873529228259389, 2.8877047681157442]], [[0.098257780432971328, 1.2117998787813304, 4.6305129696063032, 1.8932083926136984], [1.5938505795007765, 1.6061145003544708, 4.6880446981193238, 2.4756614434991269], [0.79037876309410748, 4.7994770184518929, 1.5875925235100781, 2.059452808584902]]]])
      res=power(arg0,arg1)
      s0=numpy.array(2.03226587123)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[26.542261483762164, 11.359137969204527, 19.508611250156118, 4.7300737378779507], [1.0333370065918632, 2.1722755771433913, 3.6871590564237655, 1.4019432546687234], [10.726396937028218, 4.3426672290179207, 4.9570741631158528, 28.612344346398917]], [[1.2953995317249434, 21.024635958142898, 10.426091039606337, 29.464825935098087], [1.0979428034571954, 1.0991727054941727, 1.9295228328385312, 9.8817184418500563], [17.82506482755829, 3.8044398583650008, 4.6629047085294184, 1.7076493251052807]]], [[[16.091179580668442, 5.4406436689891082, 19.313753379010674, 6.0156520498128447], [18.479686387926037, 5.1625640213659238, 3.3948204033621674, 24.035940912842154], [3.2585459740563025, 1.567054807808028, 11.964151216562566, 5.2184545274714962]], [[20.773159838027933, 1.4187089337221821, 24.110207404665182, 4.1205930033755056], [13.621765383375957, 2.7067156040687634, 20.879808612007999, 6.6355863308102254], [1.984116400653565, 5.0625886348414593, 6.8108792022063218, 5.7609656856013451]]], [[[8.7422956981866839, 9.3754978967599509, 16.347955356680785, 12.046061487319975], [6.1117515656332602, 23.3740560522103, 7.1701417980757487, 3.6876949515293966], [24.935365001391329, 3.318811865354359, 2.7291715093047313, 6.2050536085135093]], [[26.552882287608874, 3.2582066809167407, 34.432172973278988, 4.3993064662276185], [1.0651006876009836, 9.871286263205544, 21.320521755964222, 2.6366474116740566], [10.621351312550658, 4.4244283283185153, 8.0760992428038776, 11.717230108199004]]]])+(1.-msk_ref)*numpy.array([[[[9.1651762388482823, 18.053242549666486, 2.9390766825492656, 4.5212598148797829], [31.907594994224592, 4.5136153565747668, 4.2586986897561685, 24.645501337979507], [1.6843957705013375, 10.785873810643924, 15.954211774740314, 22.975644236315887]], [[1.2993264350421783, 20.721739371822292, 17.694439080536711, 1.7164592628960007], [29.31735343178935, 5.1127157874973701, 7.8361021867684828, 20.064578337092435], [26.977333421876832, 1.1860775168558184, 2.8323235528984481, 16.286571844503058]]], [[[3.9825985787958498, 20.67480241779683, 18.728727545762528, 2.6593158038040654], [1.3732847489012308, 17.241139639064745, 24.783259236060402, 4.4104043606860905], [13.015946210083227, 3.1296191376414435, 1.4588966168786912, 1.3788126524630961]], [[7.7224572517377821, 27.182712673811828, 4.2630793135572107, 1.0859487432753114], [3.102502392565238, 1.166485380193627, 3.3871881452845702, 7.9532753432179311], [10.638004192309758, 16.697410300081653, 16.757719247602022, 1.7112841187004737]]], [[[22.385544697501793, 16.527874767836988, 1.1082791256793043, 8.0393661594299761], [10.306416901978539, 2.7859589124133923, 3.1333078188574586, 2.211886203306217], [12.685343999855435, 5.8249921950121166, 12.730188447149956, 7.7509837679429996]], [[1.0721646464361345, 2.3616240427616342, 26.675207744982849, 3.8288760960706028], [3.0965274659178292, 3.1235753384351592, 27.786027567664789, 5.7870269971554738], [1.7515448629941885, 30.07083216109228, 3.0828158070529561, 4.3079569527818675]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.7895712453558188, 3.4833647305135838])+(1.-msk_arg1)*numpy.array([4.9990707854155083, 4.0501666158171643])
      res=power(arg0,arg1)
      s0=numpy.array([1.5660089716283152, 0.68417961380609771])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([5.4725113707287276, 0.26658606412690317])+(1.-msk_ref)*numpy.array([9.4143474668602405, 0.21498668418015912])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([3.0802915637249089, 3.2348243707551658])+(1.-msk_arg1)*numpy.array([1.9968341337166096, 0.5399565772240541])
      res=power(arg0,arg1)
      s0=numpy.array([[3.7964022654096632, 0.5913984144416472], [3.2632822541766782, 3.5179824799204709]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[60.902535482506096, 0.19830075535554545], [45.875683223188275, 58.501107063727005]])+(1.-msk_ref)*numpy.array([[14.351927531077042, 0.35033417814834028], [1.893874518943562, 1.9723085683495878]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_expandedData_rank2(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.063510513546382194, 3.5665236656714412], [2.3713845834118175, 4.9227510448330216]])+(1.-msk_arg1)*numpy.array([[1.6421602923886971, 4.0548022510521102], [3.8225685155579261, 3.5805150107085759]])
      res=power(arg0,arg1)
      s0=numpy.array([2.6941303581203631, 0.23454545290837145])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.0649668924255125, 34.284525503159578], [0.032104510967082549, 0.00079393398694076569]])+(1.-msk_ref)*numpy.array([[5.0911552813345091, 55.624014695010111], [0.0039142596072662406, 0.0055601740153695522]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([1.9742662115263474, 0.55547638280326794])+(1.-msk_arg1)*numpy.array([1.7475928837907562, 0.95477678604590965])
      res=power(arg0,arg1)
      s0=numpy.array([[[4.3831700803755886, 3.4308862868130738, 1.958736037053922, 4.6913407777497262, 0.19962888788593955], [3.2151425150690551, 4.3259796676046589, 1.7436141788057387, 2.900091467757683, 2.8837887265169067], [1.9828919697080243, 1.6769494248157348, 3.4274350946307823, 0.54980273208541108, 3.8830635905163757], [1.3084885778362925, 0.67095517954315975, 4.5331190790927174, 3.1465645523052719, 2.4896378649815376]], [[1.629197917328383, 4.466571680683189, 0.27164518442896513, 0.20977956458361816, 2.213462823465099], [0.64926308775603547, 1.331224965494155, 1.2711309426798296, 3.3418938670643374, 3.0271851652695769], [2.7278627499370374, 4.9437219174122502, 3.2800163473280337, 0.19023849511670393, 1.0787600251698994], [4.7173470998904872, 4.5706122323379628, 3.0890855041000882, 4.3803410567481338, 3.9791856620451926]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[18.495283851261135, 11.403406736684278, 3.7708406554551743, 21.150417654068104, 0.041538869999233861], [10.031093378526432, 18.021879622574026, 2.9970041936059908, 8.183211697923257, 8.0926410490771374], [3.8632025315123371, 2.7749948438872245, 11.380771226764313, 0.30697234233376702, 14.560869632233828], [1.7003367402554699, 0.45482764339031895, 19.765266311393638, 9.6130692012291732, 6.0544999011421829]], [[1.3114340798421611, 2.2963885752791136, 0.48484339557350092, 0.42000597078642515, 1.5548176975683865], [0.7866909173094877, 1.1722459691207336, 1.1425500819886403, 1.9546344267399738, 1.8501451270487326], [1.7461804209685128, 2.4295767571514024, 1.9344474614454636, 0.397802293922695, 1.0430112040719952], [2.3671361351985789, 2.325948957628778, 1.8710653552336838, 2.2716556665389906, 2.1536320929039272]]])+(1.-msk_ref)*numpy.array([[[13.230757393779871, 8.6232759071753762, 3.2378373521642061, 14.898887029297398, 0.059851545799268241], [7.6980284119215128, 12.930541130737218, 2.6421470620575231, 6.4284665525432478, 6.3654459388621802], [3.3079406660265076, 2.4681383089052851, 8.6081224395207325, 0.35155025397214429, 10.706244775013827], [1.5998022713256703, 0.49788702779504634, 14.031850504100822, 7.4133717627375439, 4.9236227550307881]], [[1.5936307739256172, 4.1742685024346482, 0.28813648819360754, 0.22513096271156446, 2.135339612715518], [0.66206960620677624, 1.3141120680025846, 1.2574145195730735, 3.1644338494243573, 2.8792863440202918], [2.6068330625063552, 4.5990346303706708, 3.1084682591864441, 0.20506467042485757, 1.0750678574410519], [4.3977552056401574, 4.2670547485257178, 2.9354740518836633, 4.0972916262120682, 3.7382606692236156]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_expandedData_rank3(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[1.653218684182798, 3.2106161110111788, 1.0761807142019333, 0.63398072473331779, 2.763063535405359], [4.3477618693816718, 4.4827400878540482, 4.8659833028629533, 0.93100118233470874, 4.2072270896509414], [0.97275929286806806, 2.127117830383821, 3.9372246517620271, 4.3476246589160423, 2.7488160882035513], [3.4236312741423838, 0.65363521058811125, 0.87327016381978151, 3.3703580202440211, 4.3951028595224448]], [[4.54791909338359, 1.9296636721043068, 4.5987611002971462, 4.1984388764976526, 2.1181861230560761], [2.4865039939864979, 4.840157805580346, 4.6421519904567035, 3.725393162768623, 2.4346420005824476], [4.9643545857916056, 4.0653776869205931, 2.7035060261927728, 0.14441842724889084, 3.8771948411292838], [0.5955255175344798, 2.4069324484208443, 1.3622838551339134, 2.513616600003485, 4.4353591314713263]]])+(1.-msk_arg1)*numpy.array([[[3.2216081186110666, 2.6784747611697681, 2.0694864514542197, 0.5871954354421719, 3.7625250869011286], [1.0641668613895048, 4.4287430119366169, 3.8270864888601093, 2.5941895123342538, 3.0552995966694803], [1.7634737239165093, 3.154968994138605, 1.1786739607842787, 3.853520821149544, 4.5330586866156999], [0.95574334405831796, 0.34719337025039942, 3.6926643939713815, 4.0968545049195697, 3.3841174864655614]], [[1.9165756301116736, 2.9998793374694626, 3.1764505046473368, 4.0377129713363891, 4.3434659429041957], [3.6724330947432948, 0.12254323658512813, 1.6276884628546826, 0.41270568218508258, 0.19420728064077539], [1.3251808798021514, 1.6362309105955746, 0.18540588818472634, 4.2934359078973481, 4.7954261808427985], [3.4196064274694207, 4.4975136981985493, 0.28378110834460618, 2.4329914465280473, 0.47462206684452229]]])
      res=power(arg0,arg1)
      s0=numpy.array([0.16300917726235101, 3.7066186093082165])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.049844156496765879, 0.0029560742148173787, 0.1419704004086168, 0.31663405878606571, 0.0066572038674704136], [0.00037573842758289235, 0.00029413817960609551, 0.0001467699058611842, 0.18474321288485299, 0.00048483863385271204], [0.17126631605648376, 0.021099988403393551, 0.00079122843125617741, 0.0003758319577948298, 0.0068314960876974676], [0.0020086513254797193, 0.30554421184994102, 0.20513902763505409, 0.0022124455076757629, 0.00034481881279894104]], [[386.95945558023413, 12.529564823639529, 413.61229662638345, 244.80424413386078, 16.039879301736637], [25.987579166742847, 567.46990291461691, 437.80619181308009, 131.72444735345198, 24.280492809268676], [667.7410198105988, 205.64117011729755, 34.53306978909783, 1.2082892001473791, 160.70825505708734], [2.1819302304399217, 23.414849695508053, 5.9581222452125173, 26.927268387301847, 333.90377757036867]]])+(1.-msk_ref)*numpy.array([[[0.0028977169563822371, 0.0077612178420787782, 0.023425211578186597, 0.34467875443722773, 0.0010862457631412436], [0.14509825036265642, 0.00032440657344668397, 0.00096620081918988483, 0.0090433382744372279, 0.0039180665715819758], [0.040809066434873004, 0.0032700429811207865, 0.11788399983445075, 0.00092096420968709638, 0.00026847942856621015], [0.17663506215071062, 0.5327031299010565, 0.0012330012225202627, 0.00059230774362352249, 0.0021579085772512148]], [[12.316553018731444, 50.91726306137587, 64.169719861396999, 198.32134291274556, 296.03028194098454], [122.89473481687254, 1.174152193127314, 8.4356541583835618, 1.7171981547020307, 1.289732521631632], [5.6754282357745147, 8.5305931651025126, 1.2749461835122264, 277.24910600980843, 535.16971220782239], [88.243045863120699, 362.23124401242148, 1.45032448707207, 24.22804481828657, 1.8622992473510789]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([2.8652662085414566, 4.7722169933077954])+(1.-msk_arg1)*numpy.array([2.8841070565743547, 0.76917029959186756])
      res=power(arg0,arg1)
      s0=numpy.array([[[[4.8514455512324428, 1.4648009472781063], [0.037001279317308455, 4.0714457700696016]], [[0.408816712491613, 3.0801427035112767], [4.1505948835974831, 2.7999441064643382]], [[4.4190822702570953, 3.1259533599768798], [2.0833536716842489, 3.9084795608943739]], [[4.8250375922764164, 4.9009630272064548], [0.44941598657368526, 3.7265435718747866]], [[1.0647001260634861, 3.624070570871389], [0.74669071343457449, 2.6301459931278091]], [[2.4532876269681059, 0.64410930864973992], [4.8709363834377983, 0.45919035900533167]]], [[[1.0812844665392114, 3.6540606988246371], [2.5312793954999422, 2.6050639134792428]], [[0.022538527464795934, 1.2679839421844652], [0.024289150009014254, 2.4589155534482017]], [[3.728590107858671, 1.4567213852805825], [3.5585696507604787, 4.3535179984646435]], [[3.4142028104256754, 4.3257148423555432], [4.9729009854540207, 2.5803490054908629]], [[3.4338015077572099, 3.1711358250701767], [4.7360090884322146, 3.6038626951826531]], [[3.4877640898966482, 2.5680073531630137], [3.8430085930970845, 2.25661952597385]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[92.300374397167516, 2.9853816255463088], [7.8987761700451446e-05, 55.858878966452288]], [[0.077077237978219804, 25.112243412163004], [59.026996947091412, 19.107415900880135]], [[70.639598466058885, 26.197304840811828], [8.1910741837440177, 49.688979111534785]], [[90.868103285527809, 95.025474068602833], [0.10109869120231336, 43.345534195202397]], [[1.1967776728303119, 40.017251228711537], [0.43302650312488628, 15.971808305140733]], [[13.08377649474073, 0.28354260662485903], [93.366856936198758, 0.10752743359104561]]], [[[1.4520075457086699, 484.94002704681844], [84.105888478486634, 96.466284812103723]], [[1.3797590934964191e-08, 3.1051401546968536], [1.9716835921339482e-08, 73.234094285061417]], [[533.99237775445022, 6.0209925526563914], [427.37210170663224, 1118.625581773957]], [[350.72953042479611, 1084.9413350201794], [2110.4408438556798, 92.176220601063491]], [[360.44205072823974, 246.5505614050065], [1671.9186390998243, 453.96130785479318]], [[388.2866859233481, 90.09117401500329], [616.84859306841975, 48.616076253482667]]]])+(1.-msk_ref)*numpy.array([[[[95.088031542856399, 3.0069296336967866], [7.423074109947069e-05, 57.356203548319023]], [[0.07578914605279137, 25.650190873691884], [60.631234618643177, 19.481690068703539]], [[72.645183599099667, 26.765940220935871], [8.3051336267414442, 50.981663703188069]], [[93.60287663889882, 97.914166738018267], [0.099586653094216285, 44.433267678056914]], [[1.1981921332388876, 40.999918442175016], [0.43064989141460192, 16.265479317215089]], [[13.306882077878534, 0.28120236177460867], [96.193990417983855, 0.10596219287516129]]], [[[1.0619537731855817, 2.7093829771385307], [2.0428534541113619, 2.0885032245955966]], [[0.054090281659084652, 1.2003614425958058], [0.057293731466489937, 1.9977833194652024]], [[2.7517892156871899, 1.335562905586976], [2.6547566591043092, 3.1001093931528918]], [[2.5715223458145759, 3.0848697740398858], [3.4340892490635069, 2.0732460045581678]], [[2.5828688884856379, 2.4295149269225216], [3.307556380103398, 2.6807084981456564]], [[2.6140333069148274, 2.0656145320114794], [2.8165135291935783, 1.8701207912275732]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_expandedData_rank4(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.7624947458007609, 4.1608794472429782], [2.9900166907263941, 2.3922625395291099]], [[3.8003127880314209, 3.163225159871184], [3.4926951926001384, 4.8467326266943713]], [[4.1482125253047926, 4.7220071339216307], [4.7126269880162486, 2.2321681291430759]], [[0.50264099616202829, 2.3440459563960721], [4.4482588823136791, 1.936927299957891]], [[2.0384757044893385, 4.1741006615212211], [0.034550981694949863, 2.2404340217668719]], [[0.087226724959822935, 3.6668425714944508], [3.7856606922838649, 0.94684033175322169]]], [[[2.0030542439735095, 2.1267322762484904], [4.3435875043528682, 2.5813463423596037]], [[0.3530706933070234, 4.5739380188714769], [4.3762130222662714, 0.70465523164854071]], [[4.9134178406667601, 0.4076370825919205], [4.0492404495040768, 2.5596738873251472]], [[3.6189799984253539, 0.3682020372705247], [0.73254171987175432, 4.1853300456371656]], [[0.21003101579343389, 2.3056933541439033], [2.816993974355853, 2.5571549201415924]], [[2.4105585742599378, 4.4689188507882553], [2.8258611042679993, 3.0576861849783215]]]])+(1.-msk_arg1)*numpy.array([[[[1.3333511995069474, 1.3167899039014908], [2.4948695990281404, 2.2579917493087462]], [[4.2472640347505095, 3.6310722171509444], [4.8103559575177339, 1.1310730341140001]], [[1.5495029932515521, 4.9877399228938994], [1.4272631996488843, 3.171934113629912]], [[2.0885268183562431, 1.7218471678422396], [3.597180931618595, 3.8554973339113063]], [[4.1983440891640038, 1.6362049603598041], [1.1089202541391232, 3.8490299270101946]], [[1.2189871190570589, 3.1034723600591789], [0.24374405015725653, 3.6874784096497226]]], [[[1.2192712076337342, 1.4308219331494723], [3.3207404794525646, 4.1312919419303293]], [[4.8780781189990821, 3.5381072109337803], [2.3744545593325128, 4.4865150616394063]], [[0.96493047895262529, 2.774735502490028], [4.1355186870174414, 4.8423231772247233]], [[0.96266241020306664, 0.77967857163251386], [2.9093623273050238, 4.9032933597126469]], [[3.1973281014607111, 1.6086444293588347], [0.57038891726194196, 3.6137992721255441]], [[1.8988863591323759, 3.9381171017182237], [3.1356385522072712, 0.20859191915578965]]]])
      res=power(arg0,arg1)
      s0=numpy.array([1.3117223646163834, 0.21697960321684032])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[3.6410072213647946, 3.0926162707876546], [2.2508643223136398, 1.9138540634102512]], [[2.8043755230241669, 2.3591770594251948], [2.5797992894640043, 3.7251888483027416]], [[3.0820050217385719, 3.6012262093835345], [3.5920719558465013, 1.8324957890888436]], [[1.1461255171270306, 1.8889779292763389], [3.3434234315809555, 1.6914191301810952]], [[1.7386729500626179, 3.103730827098047], [1.0094191837564077, 1.8366104625986268]], [[1.0239505061835075, 2.7046295011851136], [2.7932482589007357, 1.2929373627739769]]], [[[0.04686094936708262, 0.038791866761992276], [0.0013112257589111785, 0.01936722429570966]], [[0.58305424915583848, 0.00092219173790723742], [0.001247463351521714, 0.34072508911221505]], [[0.00054897028461704713, 0.5364135511541851], [0.0020558934912902191, 0.02001929657961481]], [[0.0039674668839738067, 0.56972869146097438], [0.32651201535343932, 0.001669914090217125]], [[0.72548324979404111, 0.029511106434106676], [0.013511266801397338, 0.020096496535205708]], [[0.025141947514299091, 0.001082704547631117], [0.013329443255606985, 0.0093535671965274316]]]])+(1.-msk_ref)*numpy.array([[[[1.4359015678599458, 1.4294634481565824], [1.9678873400228172, 1.8453811937512947]], [[3.1659624703486062, 2.6785054446629291], [3.6886003024289371, 1.3592139453014882]], [[1.5226371187827676, 3.8704803341948923], [1.4729616524287141, 2.3647586152480331]], [[1.7624468019295794, 1.5955329660855846], [2.6539865600628638, 2.8466837411987767]], [[3.1242152065323392, 1.558883065921129], [1.3510682725796357, 2.8416925534452475]], [[1.3920274087104798, 2.3212352209279525], [1.068373894669493, 2.7198161466801882]]], [[[0.15520820771203664, 0.11233979395293915], [0.0062577673832918176, 0.001813647129642202]], [[0.00057942817346151327, 0.0044893034577981441], [0.026567877307318695, 0.0010539826167806823]], [[0.22892350235543485, 0.01441245594202548], [0.0018019718605267246, 0.00061196405154058657]], [[0.22971821299470008, 0.30382257773489402], [0.011732813777519922, 0.00055752872898595489]], [[0.00755637557465739, 0.085611868796965848], [0.4183123638054001, 0.0039989975902562334]], [[0.054945853176721952, 0.0024363516026187303], [0.0083032775249303331, 0.72708024864668652]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.11116210647036527, 0.97043081798542452, 1.5441347724164989, 0.59720052578335425, 4.6654781695343841], [1.7806911893038184, 4.6987684222883335, 1.5125497104602434, 3.5569266324579845, 0.02911951890381502], [1.5531096176639849, 4.7925481968016195, 0.76508159203361192, 2.827745910274547, 1.9521806557222121], [4.3333028715469615, 0.85811673970290692, 1.0072648654549694, 4.8684916904307682, 4.9830301003868911]])+(1.-msk_arg1)*numpy.array([[0.36456886487344775, 3.3838431782546863, 0.38905172322017717, 1.8851653691861583, 4.5792433044190641], [4.4163494663324192, 0.079842848519039972, 4.5655662963210641, 3.1842933936340421, 2.5645758488705854], [2.4567215622096117, 2.850200132993713, 2.5228212673957611, 0.43895394951977418, 4.6718174632010516], [3.2384447695133307, 0.39342653314811454, 0.78656026773449172, 2.4884300991944106, 3.2657265628269654]])
      res=power(arg0,arg1)
      s0=numpy.array([[4.0655616446325578, 0.09606363297350784, 3.3633661076038743, 3.7071499171614706, 1.4432173365801413], [3.9652756948560119, 0.13198414438856382, 2.5188639024357595, 4.2574787861189458, 2.4498013779335288], [2.8386310503461867, 0.19664307965065755, 4.3703401695277693, 1.1607120744958814, 4.1967410414759074], [4.8064297749786382, 4.6159637588414171, 2.1423236673002473, 0.91923602182065767, 1.0157802191727316]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[1.1687217422485319, 0.10295416125544922, 6.5074414938805836, 2.186910835187752, 5.538073833904968], [11.623586472265595, 7.371152974136224e-05, 4.0442878322631923, 172.92160467650677, 1.0264346487998359], [5.0550780624317522, 0.00041202253499112872, 3.0906257671322015, 1.5241385458073073, 16.44512172650786], [900.62466458833569, 3.7154778514989339, 2.1542143835171221, 0.66365874546805226, 1.0811435595665406]])+(1.-msk_ref)*numpy.array([[1.6675020928620774, 0.00036069426434817887, 1.603032724862721, 11.823193110349873, 5.3656061438966001], [438.71689346730227, 0.85070689388525811, 67.877629694177685, 100.78737309831484, 9.9530375606127031], [12.976648021756162, 0.0097015883517895512, 41.295761085647285, 1.0676061721479673, 813.07753785848581], [161.45252340077684, 1.8253176336254406, 1.8207950396821546, 0.81094375189223622, 1.0524612834024929]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.21116270903244103, 2.9778270912133982, 0.86150030315960757, 2.5051365945351582, 3.3497988760140838], [0.57454166401219819, 2.9524381985321022, 3.6969485701372458, 1.2645673442182443, 2.4312124380136475], [3.8210739434392034, 1.9971829052055612, 2.8669406104048361, 3.4385144712875548, 3.6872384383154695], [2.1907509996378778, 0.05713584104608592, 1.7737360711030672, 1.714450961421121, 4.08549921297125]])+(1.-msk_arg1)*numpy.array([[3.3721846801205921, 0.95167435743443207, 1.7448096666586621, 4.6815728126514138, 3.5466210908978955], [3.7120877611420449, 1.2814125986747866, 2.0862197238562508, 0.36407548871242645, 3.5788717576808082], [1.4026422429794168, 3.7553508637153068, 1.1018422467196214, 4.2904180864531609, 2.2143791179339121], [1.6976762748386571, 3.8478685906208128, 1.3025472018082678, 3.9098080111411595, 1.0159313254197468]])
      res=power(arg0,arg1)
      s0=numpy.array([[[1.3995405691438778, 2.3924842081328608], [3.0962693651713651, 4.1414557605319127], [2.9343012072322789, 2.1695222902981754], [2.3574724196717152, 4.9727563825612773], [0.35596559976354974, 1.0639025749094295]], [[1.2802597247162655, 3.0202799379361469], [2.0693862105958298, 4.6333426915016611], [3.0266974618555009, 2.1141761068657385], [0.8532355630826115, 3.0742631451271496], [4.1676340045420126, 0.88452294017418498]], [[2.5624508853904655, 4.6487490837342103], [4.7449848623063824, 3.8505311001542872], [1.4811182396122664, 2.2725464651399414], [4.9594037515945431, 0.47472147923787267], [1.8239815881863244, 0.89374286295176908]], [[4.2305987736171957, 1.2281551756185014], [2.5443780066189219, 3.857569908656505], [1.8734190775826423, 3.9320365177990775], [4.1377617666192945, 1.1679251042213357], [2.5262168926043485, 4.564180333119797]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[1.0735609154471257, 1.2022611071477391], [28.94895458744427, 68.829561357422406], [2.5278740645153186, 1.9488471952080733], [8.5709637583639466, 55.59952687896039], [0.031427303780929373, 1.23059691032647]], [[1.1525166796189892, 1.8871530176886988], [8.5605695366853709, 92.472400095222667], [59.995365473693361, 15.923192593006711], [0.81814832101819401, 4.1379133299063886], [32.142765490723228, 0.74205911346917874]], [[36.433682885073161, 354.76486776515407], [22.416336889377074, 14.770384597751891], [3.0836893809582966, 10.522064103054591], [246.17519088506842, 0.077166786355311698], [9.1715901002062719, 0.66085955636079707]], [[23.56624863211756, 1.568670143750158], [1.0548075937807451, 1.0801884578102543], [3.0449658382059526, 11.342129035157384], [11.413377298050905, 1.3049074197523673], [44.085265652686978, 494.11288732761972]]])+(1.-msk_ref)*numpy.array([[[3.1066336346896759, 18.947375546613486], [2.9316940022879825, 3.8665952222431108], [6.5419422550476716, 3.8627034019548296], [55.416101881591629, 1824.6116167727505], [0.025645682330830113, 1.2456920822435642]], [[2.5020725716543502, 60.530927426547521], [2.5393545511991138, 7.1332180957874183], [10.078758534703008, 4.7677766441839937], [0.9438519856096117, 1.5051318884883178], [165.38849118537539, 0.64458279028958365]], [[3.7428078981325204, 8.6304573801145192], [346.33713309934552, 158.06489389788391], [1.5415690115472449, 2.470704654276271], [963.12611692277346, 0.040906048439861543], [3.7844104591454308, 0.77976936840365541]], [[11.57250936396324, 1.4175001472023134], [36.36007967703555, 180.326052471742], [2.2652694927344195, 5.9500061720859767], [257.89007230678567, 1.8347617681967261], [2.5637904504922759, 4.6759226519175003]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.3639953241739091, 3.822293639309966], [3.766112660012451, 1.9043432963037339], [0.05428663375354232, 3.1105759440383558], [2.5053478596295577, 2.1384793787934213], [3.9833694454248811, 1.5834939399990495]], [[1.90996570232871, 4.7346992215492278], [4.9256810218074332, 3.1435106274735816], [2.3426540219023853, 4.3335412745506741], [0.73200717006506244, 2.0381842149190006], [1.410168742760741, 1.5164625394165339]], [[3.5127993175886738, 1.2226349155497236], [0.35690431560687041, 1.8137502307226905], [0.91083723570319708, 4.0614943100476903], [1.6589539822364541, 4.7650383682329149], [2.5281605074983453, 3.1421599888508709]], [[3.8675347349871942, 4.6469990949407087], [1.0789191875041844, 3.4701576806362198], [2.1648989670166707, 2.7010195816963027], [4.8159661056264422, 0.60408533873369097], [3.843205825608047, 2.3214406677704007]]])+(1.-msk_arg1)*numpy.array([[[0.82976913676015385, 2.7935420953110603], [1.75605429102879, 1.0762574040100463], [3.136639643914632, 0.23298278621944138], [0.53040242185053055, 4.1232105657688862], [2.3290035365518338, 3.5687641296447508]], [[0.65765694702813049, 4.1145753914354204], [4.0196585313393376, 0.6399026789709531], [2.4490060853353635, 3.6334097923635342], [0.20596363412526889, 1.6508375826217763], [4.9168575733004181, 3.5591784807102935]], [[0.11823775655772015, 0.98800854018634687], [4.3800087495267706, 4.4751281754698518], [3.2004679340792257, 4.6353416515037305], [3.707216512629028, 4.2485673252844833], [3.8687604394683075, 4.4880106751911866]], [[0.26302575472987988, 4.826498547343502], [1.8585748165258007, 4.3170343877481914], [3.0684129465162826, 3.8185410221764453], [1.5066211482755558, 2.9472298061000934], [3.5631279862328267, 4.762585048926935]]])
      res=power(arg0,arg1)
      s0=numpy.array([[0.50179516317418593, 4.5950977283283088, 2.111678216435882, 2.4364954233758032, 3.6823694504644249], [1.3799630717305995, 4.4134313025280942, 2.234112769764053, 0.59640084916706715, 3.8143309702881578], [0.72067817456852312, 2.0988436426045221, 1.8755525410995237, 2.6579255930498684, 4.1439409696355476], [4.3091850538561243, 4.1012345470531839, 3.5720026914821621, 3.6310564923938968, 4.1125730182131148]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.049328614656966342, 0.071667826601445522], [312.08769927578072, 18.248894979224922], [1.0414128861736491, 10.227729617525442], [9.3107007407433198, 6.7156841127253051], [179.92620658755811, 7.8787848678248622]], [[1.8498736252242243, 4.5944166400094684], [1499.5561547334319, 106.38034009371627], [6.5740415600960755, 32.573267109882345], [0.68500287126892256, 0.3487430895525313], [6.6053832028117112, 7.6155133565484228]], [[0.31642732587663824, 0.6699920226133671], [1.3029149949272074, 3.8369969643486392], [1.7732757590963586, 12.86212817660058], [5.0617069769576126, 105.42966374289492], [36.384849247068942, 87.098802819936921]], [[284.14842845347562, 887.22478184128431], [4.5844298309291389, 133.94001127412008], [15.739761084218619, 31.14766806529888], [497.8516763568648, 2.1792533787322532], [229.17384389399479, 26.645773779098313]]])+(1.-msk_ref)*numpy.array([[[0.5642948779067426, 0.14568273621581576], [14.555446819199606, 5.1617816678245205], [10.428941669269616, 1.1902348858725817], [1.6037675094486747, 39.329281117962417], [20.82152913973891, 104.80331293468201]], [[1.2359042048548918, 3.7626621810348198], [390.64371397673074, 2.5857952128871093], [7.1607826534795729, 18.554183861405843], [0.8990193573361368, 0.42603974141691053], [722.3560861992429, 117.31992201532215]], [[0.96201015994942418, 0.7235145320137425], [25.720274225803664, 27.599559818837001], [7.4841306088103599, 18.452164534810322], [37.486263086500223, 63.635395683681509], [244.69503465045605, 590.14488796890112]], [[1.468460475681874, 1153.2069340630542], [13.776777780942192, 442.56130117171176], [49.723496051645228, 129.21600325152758], [6.9784274890962257, 44.724548705789147], [154.22878341725584, 840.94647380854065]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.65735259627250886, 1.6833562309883234, 3.5884667170466487, 2.820994582781577, 3.2936356240380036], [2.1221463976304538, 2.6494544124186121, 0.83964148470392597, 1.3550872744898184, 1.2857884471395487], [4.537134055908755, 1.281589597808191, 4.8295390125962259, 1.9336515820309317, 2.9193761054833467], [2.9587653504119409, 2.7668096189515459, 3.4308202461286585, 4.1239548643025996, 3.6886973965128282]])+(1.-msk_arg1)*numpy.array([[2.2865850493024862, 0.34130727462042731, 1.9948011902582659, 1.4012003834304398, 3.5748950904618857], [1.2271445499050606, 2.8412714953876415, 2.3755133566650359, 2.9167959137441914, 2.3431102455096635], [1.2653374649344724, 4.483865664943715, 1.1749253431004893, 3.066815732076968, 3.7562363027792856], [0.59747046957582417, 1.6198138229942443, 4.126562346625855, 4.1300211321089613, 3.3057041190524434]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[0.8628294278388573, 2.0887303637104715, 3.666620852463204, 3.0028659392615649, 2.5023305827498965], [4.5401190361079244, 1.2662069183267124, 3.9325785129612556, 2.6534141243477576, 4.3654899860460237], [0.84656295955987293, 0.15197935008572092, 1.3437421411154684, 0.89088064516282728, 3.8952895122469902], [0.87163777444523227, 3.7464622348321113, 0.67792820645810725, 3.5825998295163588, 3.0992413273048434]], [[1.1725534410401868, 2.3897961311627003, 3.1799488899501678, 3.4795803929079767, 0.08104007946082166], [1.9344145286138485, 4.4859060544103038, 3.966195691115979, 2.1209041033198397, 2.7495732230710432], [4.1804544861517039, 4.9966241699364948, 3.202447208412007, 2.8776817572663722, 2.2008263006981603], [1.9894530723571093, 1.2820939049135809, 0.93650603977125468, 1.271971182007668, 0.36117790305567921]], [[2.6513779745552117, 2.4830186151571549, 2.5656283035939902, 4.2740009164222732, 2.8434569047160494], [4.7315088269172749, 0.66818137857073034, 2.7922754380983172, 0.68926201033202572, 1.9876455344181783], [2.4496509467573961, 4.1741891372562376, 1.3352924339101273, 1.8040361768660496, 2.4512829389732378], [3.5025762215934959, 4.4288896309211125, 2.8319091117250044, 3.0674880745131392, 2.8634205922147284]], [[1.4464409873108235, 3.1505151935188871, 0.49228280960176185, 3.5893441533350248, 1.4610916845748263], [2.1507549792254936, 0.0239848232226074, 1.1201294209010904, 2.7664639283532972, 0.32811705467203772], [1.9790022978461081, 2.3920935785774531, 3.8164636616007055, 3.1105946420353492, 4.2493252111062292], [4.8115795697200587, 0.40505295281299014, 4.7160860597850132, 1.0620816494797016, 0.36272641082372181]], [[3.7334624616279206, 0.99958416904651826, 4.6008081782957833, 1.8821844187579198, 3.6546904649221452], [2.41547156462696, 2.8956039076586717, 4.2772199522032217, 2.8031181915296099, 4.3155194207140015], [0.94532208906396165, 2.212235378101175, 4.9084747066876195, 3.2932016815482288, 0.82789523632914497], [1.2604375502563474, 0.58968098412071324, 0.096213686412993799, 4.6691634351592848, 0.1668986541736904]]], [[[1.6647145911340311, 4.05671817813049, 2.6104538276943239, 1.3774954760533624, 2.0629379694019998], [3.7976637444340464, 1.3265031260144611, 1.4792359740225405, 3.7242609328425704, 0.86942151411247171], [1.5896313758888296, 2.3934632148318293, 4.4228886379375218, 0.89207110549208202, 3.9966358123719194], [2.3634923160724228, 4.8290064615560722, 4.1895099155963997, 3.4234215200725715, 3.2216954530889987]], [[0.98713085223150376, 3.5842791036190147, 1.5266316085311886, 3.8684978368058722, 3.1792926210906818], [0.42306608391269779, 3.330102034049347, 1.3509320315894562, 4.9668712043600447, 0.56720394993904766], [3.2266519605902775, 2.7020682806049776, 3.0129291605571011, 1.1210500434797535, 0.26401200976589245], [3.4550194458893544, 3.970935141143983, 4.050471813960618, 4.0816596513318606, 0.83432874029967286]], [[1.3787957623997735, 4.6889835369388395, 4.3271792649813001, 0.64096903565799801, 0.40911502847142422], [1.9109729614900268, 4.1249504295791688, 2.5287091492468297, 3.3245307170342424, 1.2018983837708319], [0.59781151596970494, 2.2839326117115943, 4.4896255019327116, 2.6079524871873847, 4.0710881732273485], [0.1336393171083998, 0.64227335026681509, 0.16345753817735945, 3.2627824484272971, 3.9156729347990233]], [[4.5125386734900355, 2.7562858032963407, 1.0013371107072087, 0.092514069790010214, 1.3161718246790988], [2.8232812477531462, 1.3737536449110295, 3.0047680694754551, 3.6846979015839367, 1.7024134970416387], [1.42468778586117, 2.9162519413162014, 3.2716332478682864, 1.8294830377197129, 3.4307107274016682], [1.1330316960919999, 4.6995589139714671, 1.6124801374821225, 0.015782687990778484, 3.8065104356315218]], [[3.7359101995133654, 4.2733310089339316, 0.20977587157388686, 4.4680456066707794, 1.8704723586004193], [4.4285654745366161, 4.2398514739290647, 3.481221255271532, 1.6446846482828255, 0.96661899737192503], [1.0838519610365891, 1.5392987245114036, 2.3478477553499109, 1.8382006305706757, 1.9383969188926435], [2.3529359342562706, 3.0362095735056056, 2.1010518325537482, 2.7577867573448902, 2.1675060786867979]]], [[[1.6325441868626354, 4.2790886965962613, 0.68174543109022545, 4.5027035190233278, 0.10687072347931521], [4.5761396159718259, 1.5380252640643952, 4.0957627351721335, 3.8299452904165716, 0.98440320880263488], [4.5852977554125651, 4.6876390034991235, 4.8965575625660183, 1.1492868915092185, 1.2176918476604894], [1.2232430135104595, 2.7585037999503319, 4.471842401703416, 0.20721942759338077, 2.9866371436916803]], [[2.3227017236591472, 2.7925269433584679, 2.252609342867689, 2.2066913154639338, 0.90943243141751451], [2.5708454891103671, 2.5306160210926878, 1.8937460198266429, 2.1611120295983293, 3.858559876626523], [1.5484494817072734, 4.547911626729622, 0.66761976612146889, 4.5658457706319737, 1.8812178195519977], [2.1450204347544681, 2.4849902116220868, 2.9527465243282753, 1.0269287703951064, 2.9241671593198064]], [[3.5659615173043404, 4.070038330373662, 2.0087341631265199, 4.2498134294773076, 4.0823743302035052], [1.7424392483166702, 1.2937515897408169, 4.0215988173416779, 0.14868427391685893, 1.6957852764606431], [1.8663001012907405, 0.52169345605109252, 1.3472554378933193, 4.7803719687626645, 4.7869423887078106], [2.2234001288596463, 2.4661254214079951, 1.6279529498544845, 1.4802036682074275, 4.7055927893881151]], [[1.9148221006067618, 3.0751223026985013, 3.8945991583762445, 4.6856315950486458, 2.5958915137722749], [0.82371709612301725, 0.44921644521017373, 1.0573822797490395, 1.8866412308480274, 2.7864543341448869], [1.4649619670996348, 4.5634462549131563, 3.3093571792457781, 3.8708520946496536, 3.7454005315233325], [4.0172149390313363, 4.0463374875128109, 4.6142848618248475, 0.60135095194546606, 3.9240065239296231]], [[0.027406033601392939, 1.1622188076939652, 1.3545074241185251, 3.8591321495415962, 4.117883489255969], [2.8253565018903264, 0.95130836230240212, 0.86628902802425412, 4.9688666719976577, 2.5500451498419792], [1.9635209644369704, 1.7831145727031639, 0.68933040586317484, 2.6751534364123297, 0.46007270397677419], [1.8040475654592758, 1.9837938368605053, 0.084476513756675572, 2.3543180640122259, 2.836622155368357]]], [[[2.1626041362963995, 2.7512373491056623, 3.6197769165522224, 0.99182926361575985, 1.7954043066454777], [0.34338229117117064, 3.8117171015926199, 0.59330752466475678, 2.6738895466477279, 3.0271599265753357], [4.8740073780922657, 4.5696555272793828, 1.0291487838506166, 1.2896133938395256, 1.0567094569869491], [1.8188572588246938, 4.5779039403601001, 4.5626891645218688, 2.9293580437746738, 1.0074625800849757]], [[3.4624786372809946, 1.6671844633230699, 4.4299022618508506, 2.3596005921526007, 0.84263076875105347], [4.693065568066574, 1.1315744157084484, 0.42986118400091466, 0.76231909924345276, 1.4617230580598251], [2.246830357690897, 2.1693518989062137, 1.6938105846500837, 0.10822965688884442, 3.8472876743645403], [3.7643223408176767, 4.8321633167182645, 2.8125788706772337, 0.78267288362025089, 3.3259873555211197]], [[3.5552037103585579, 1.0904796807517976, 3.0006097311691953, 1.5406248171645964, 2.2465900667872516], [2.5861416332780292, 2.5479043290858723, 4.276382782766861, 4.1248482418655357, 4.1896680224255762], [3.1792960624717632, 2.3818535098338369, 3.9894805508888487, 4.5026276306543336, 0.98709717097386196], [3.7080625906654983, 2.2865185604279628, 0.26307853878248799, 3.9236403332117669, 4.8533054956001269]], [[0.77120348323998489, 0.44935164813117712, 2.2535293329877057, 4.6152910660489148, 0.93304487348198251], [0.57542494632660635, 4.6069511480099097, 2.6056857303014778, 2.8980388670338617, 1.4043907240887046], [3.6035074985215969, 2.4920515298877532, 1.6615360716014482, 0.67662547001618056, 2.6369780552982451], [0.61576428593669175, 2.1599021601419377, 1.2269475234084324, 2.2896533298143851, 1.5326798643140442]], [[4.7055934403077782, 2.9680477316889267, 1.1460705753750802, 2.4458996125838706, 1.2456376014207855], [3.4568668142897421, 4.2145504307840422, 2.6712881842097431, 3.9166488298816686, 1.5264691345512209], [3.1175007605590905, 1.3016087013913367, 3.3837275380854765, 4.3232263066432504, 1.330254625051505], [0.67021900700848203, 0.9925385398531843, 4.1744684473084481, 1.3259038581257898, 0.76105117099166764]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.90756993111272044, 1.6228392875381443, 2.3492093609118108, 2.0602005203176752, 1.8274811755369187], [2.7034905504011415, 1.1678356238502516, 2.4598732746797882, 1.8992815136176462, 2.6346768280255115], [0.89628603215247649, 0.28983042343201415, 1.214363805678325, 0.92685913292359312, 2.4445157081154534], [0.91364975701141227, 2.3827116580659209, 0.7745120330441525, 2.3136821282757896, 2.1034297352761682]], [[1.3072988212779988, 4.3342713852016264, 7.0105497064331272, 8.1579800556010422, 0.014553019336658221], [3.0364418090553928, 12.511116045838378, 10.168966913990044, 3.5452854991335432, 5.4883267898674122], [11.110675116534003, 15.001053163226626, 7.0942459249737535, 5.9256022489111988, 3.773061517690071], [3.1832828848506285, 1.5193839261961528, 0.89545169258777169, 1.4992445085579909, 0.18008899705465511]], [[33.083686806603154, 26.144027752237246, 29.402083707416043, 183.53927224814572, 42.52199185329372], [264.37098835878879, 0.23530939557733571, 39.838810172818199, 0.26305562428557067, 11.764659090720254], [24.905054226018127, 168.61740873209254, 2.8224528100088579, 8.3085937290279599, 24.964645782183524], [89.849727812721895, 208.54874764651566, 41.905551485009113, 55.821843398953369, 43.603076438051204]], [[2.8327448721953825, 25.464299446964691, 0.13543760115895187, 36.787018796260796, 2.9144341571992403], [8.674355795196508, 2.6903557082457963e-05, 1.3771629734821822, 17.64688536352477, 0.043124083590419235], [6.8591989877402284, 11.70930590796954, 43.738400933976223, 24.564538923530421, 59.222567489074386], [84.087187579693065, 0.07812552000804171, 79.463993617823064, 1.1851990887859634, 0.057223609528991103]], [[76.617463358299446, 0.99863105737690416, 152.45233907910531, 8.0285500755102248, 71.42081268990195], [18.258655657587298, 33.174205207545775, 119.89919788140683, 29.81039250799753, 123.47173746036448], [0.83093830402618263, 13.669378326169303, 188.68014261227768, 50.68111563718336, 0.53683497393874768], [2.1432885467538245, 0.17558862773158618, 0.00044787073355088372, 160.04041217424532, 0.0027481659657768911]]], [[[2.9492760885393716, 19.527023183648836, 7.6618278939389413, 1.9731937311231627, 4.6492798853710093], [16.975355216160768, 1.8213983114840802, 2.2953260377252835, 16.286609891508679, 0.74308411194504842], [2.6741174977442124, 6.3730931668154316, 23.4575511268988, 0.78476643472636654, 18.918381373117114], [6.2049271738324192, 28.264827424608718, 20.908430889998176, 13.620817135051459, 11.973735428585785]], [[0.96626465179872956, 29.434885384904227, 3.0675773303468419, 36.03020173752266, 21.424059471291159], [0.10237303642748988, 24.222972485287841, 2.2187455582909883, 69.861821824137621, 0.22260932880305953], [22.280019041880475, 13.923862249379223, 18.580581840360399, 1.3535651833609643, 0.02935068998747872], [26.705458744389141, 38.613498750197621, 40.696628396799284, 41.532131845393565, 0.61885158593144696]], [[1.3095736761930137, 3.6598688481696762, 3.4212437435180658, 0.68835484530811586, 0.47216014251707655], [1.7224752147377989, 3.2864808002017529, 2.1791675023692454, 2.7419936561007123, 1.1669715736711139], [0.64922329973256232, 2.0006235032674256, 3.5287644562762006, 2.2363639207414501, 3.2504106828675448], [0.18454323696482039, 0.68953077339040203, 0.21854565046697244, 2.6991678318805699, 3.1458995667356344]], [[7.7054407226120665, 3.9507339585579744, 1.0018123317176597, 0.039730252662761818, 1.4510386211624533], [4.0814192064408079, 1.5377245151114489, 4.4409465427544701, 5.8549607678385325, 2.0564270788079932], [1.6154875324565627, 4.2646019170025617, 4.9836894994733099, 2.2671374934404356, 5.3148643128243007], [1.1844117964915224, 8.1413424268188876, 1.9106141303353323, 0.00361724531537577, 6.1187792290017766]], [[5.4447847448124662, 6.471907814237305, 0.13425210424887696, 6.8535201829682899, 2.2370301266235222], [6.7757533600114801, 6.4067858719201345, 4.9722421048394478, 1.8959922615219948, 0.95728547386227136], [1.1090828263645185, 1.7412359812822145, 2.9964229601371448, 2.1875266375791353, 2.342019554071296], [3.0047751330546224, 4.1703891863299791, 2.5976789151870503, 3.6852574680676318, 2.703795889351909]]], [[[9.24265602267843, 732.02419004276635, 0.17584166250275141, 922.35094870938508, 3.9246592761680361e-05], [992.59872773487052, 7.0514482680250143, 600.12957629426455, 442.61168140205029, 0.93116164659687595], [1001.6435561624741, 1107.1563759408455, 1349.3584571524118, 1.8800612284949925, 2.4439630958177339], [2.4949224507981835, 99.860822109504525, 894.01417502780873, 0.00079168680115611009, 143.20775313123701]], [[2.9447836178401308, 3.7289416932168025, 2.8313825845460543, 2.7576277500283655, 0.88544302606263192], [3.3538922417255423, 3.286779515766439, 2.2667970999430995, 2.6848431663228176, 5.6436034795275649], [1.7513384092022433, 6.9669876128614492, 0.59582460045015573, 7.0022168691180422, 2.2475961274006298], [2.6592494498220622, 3.2110273204828372, 4.0053176481953718, 1.0346416463120114, 3.9557019927809054]], [[464.25723781955804, 879.18391594468198, 29.038612833331307, 1083.2701477800024, 892.12830620896591], [14.611010558217293, 3.4688719455337349, 829.78831672832871, 0.00010055917484487686, 12.816089091145551], [20.357086760627706, 0.043176416785075208, 4.2187477693069475, 1911.9949671794279, 1924.7202311786484], [47.417119209347817, 78.208239327660038, 10.522830295879473, 6.6462123586780928, 1771.8102245880457]], [[3.5118672231832004, 8.7771978521485412, 13.859554192677029, 19.816724607711052, 6.32536834645372], [0.68729652918866202, 0.21279935714371578, 1.1139258931146843, 3.4126134045133401, 7.2539635425791609], [2.0924273289431015, 18.829675539456453, 10.1158671641436, 13.696610890822559, 12.851263630714108], [14.715691178555197, 14.92267205902718, 19.237406752010106, 0.37403351035248367, 14.06262558280905]], [[2.7509676410048107e-05, 1.5509576269206387, 2.4250429813653454, 51.544682854446705, 62.296657822222436], [20.742088428697027, 0.86439403871651999, 0.65767950505176254, 107.80454035617009, 15.376806069902075], [7.169369452428513, 5.4111141879382947, 0.33752730781704426, 17.684450645475806, 0.1036725177079961], [5.5986618991515362, 7.3876154463188044, 0.00073576423487290484, 12.179071753645166, 20.984462628104428]]], [[[9.797571183547177, 19.973785894740114, 44.978852350593243, 0.97601766048292971, 5.6494542489812556], [0.042313185125594038, 52.408252054168621, 0.21339699263925085, 18.357658186832719, 26.501519287348202], [108.46604409258413, 89.627302504596202, 1.0887294301042869, 2.1223834229448904, 1.1772778740780891], [5.8706088901642826, 90.106820941786467, 89.223633986920746, 24.04751322971001, 1.022241784971573]], [[31.072873983771956, 4.1132632961847548, 61.439763638727754, 10.754076917235587, 0.62266234569459733], [72.076325876684308, 1.4077676208134016, 0.096713927772294983, 0.47194888748214636, 2.8585799186840495], [9.3913503213360663, 8.5223839592125152, 4.2975946222296439, 0.0021292173847801731, 41.592166697595111], [39.157575154577792, 78.142908346365502, 17.48184362156741, 0.50764126306387947, 27.800648024726726]], [[77.609886250712236, 1.3460436605952892, 43.373059713551775, 4.4050849535754555, 16.069970807673101], [26.045691103476027, 24.7480657075577, 146.25485152775394, 129.22726120091534, 136.32842261603398], [52.894104585588423, 19.639380158296696, 115.24868727319391, 174.55186487205759, 0.95642265654305181], [89.668776424506561, 17.071195627033255, 0.010242767653518656, 108.85312306885457, 225.77288779516431]], [[0.34252318704910789, 0.036921704533056467, 28.522787980190426, 548.43743344002166, 0.75141419735759851], [0.10237741832231785, 544.36196332224586, 51.909123749371794, 80.481652861625989, 4.0572676516013768], [197.65624730886566, 43.190004027921098, 8.116566584896205, 0.19969363333776063, 54.528589466001769], [0.13538011793632698, 23.943688071842629, 2.3244187319664027, 30.456089229173006, 5.8182639172509294]], [[302.74082673947169, 55.309802587159076, 1.653532740388707, 27.091469125709157, 2.2483874411529112], [97.059327774582911, 201.61290506976323, 37.50107301291029, 153.84474527605465, 4.7595900480609643], [66.298393065605325, 2.6441355360883088, 89.69730228500066, 221.464032612707, 2.8652194546625926], [0.22854206192237467, 0.97275185184244128, 194.63008272266899, 2.8308041398526456, 0.36523434397088256]]]])+(1.-msk_ref)*numpy.array([[[[0.71365275376512827, 5.3881325111215075, 19.509323426946608, 12.357384210037798, 8.14421065386035], [31.80095238543025, 1.7154800568431283, 22.897098714804862, 9.3124994583722032, 29.073002154630217], [0.68326118701313598, 0.013461109602347568, 1.9651934311853283, 0.76781761845766805, 22.40367913560846], [0.73042106282804609, 20.494344105069885, 0.41113758571357412, 18.50212237861, 13.283032664746615]], [[1.0558335910194581, 1.3462859448457816, 1.4841563297364879, 1.5304778966007779, 0.42416157479116801], [1.2525683807127488, 1.6690955774714831, 1.6004033300680613, 1.2925400346618297, 1.4122918043614143], [1.6294014541001725, 1.7316632289334335, 1.4877319049112359, 1.4344143939262981, 1.308961926773637], [1.2646198219650986, 1.0885135133260084, 0.97785926417084534, 1.0855725495561122, 0.70639402750903613]], [[6.9942594757526635, 6.1362991710909647, 6.5502843856986033, 18.129658884398779, 8.0414402357022485], [22.207011347246876, 0.447403190975453, 7.7552903166751292, 0.47600212752181176, 3.9366505714850164], [5.97290397416561, 17.294898226210851, 1.7803276162946204, 3.2445787118859895, 5.9808443840093712], [12.188352962259248, 19.463894923460987, 7.9764261268917034, 9.3548122222094783, 8.154456658567538]], [[1.6773103621793548, 4.9926653695285124, 0.37045126181797405, 5.9935947157318212, 1.7011638010864516], [2.9243311370414942, 0.0053699029068716838, 1.1722888300175747, 4.1612661729417537, 0.20982603559902427], [2.6024396498737055, 3.394252217087065, 6.5316634713721813, 4.9042476032225899, 7.5928038890013489], [9.0369472627636593, 0.28186774897365385, 8.786642698200211, 1.0880590531102821, 0.24148056100903487]], [[110.9786981183052, 0.99851424363351704, 234.18682836728991, 9.5915425325644712, 102.83284402391116], [23.398699314417559, 44.737224285961602, 180.44150803995521, 39.835573911921792, 186.28445836482337], [0.81790030194267205, 17.089755146596623, 295.16268742727061, 70.864906960838255, 0.50906190907581983], [2.2874584789538823, 0.15134925207343328, 0.00023183469178847567, 246.86498047496161, 0.0016609266595478498]]], [[[1.8690284501467009, 5.5759444439612533, 3.2461664728940947, 1.4814389462334019, 2.4317559587036048], [5.1422184820646955, 1.4144278347209582, 1.6168157917075248, 5.0205205392411578, 0.84222260851055941], [1.7661183885118223, 2.9182374020067332, 6.1997560970554542, 0.86922655013008543, 5.474774290779103], [2.8734589295864019, 6.9054558926749943, 5.8007507894188803, 4.5275175037241784, 4.2023585643247152]], [[0.9638669047602284, 37.601579557112657, 3.3268934158022598, 46.705427878125562, 26.745907410765845], [0.086800913859489778, 30.510106052996676, 2.3505269650809102, 95.008070431884605, 0.19966702212071802], [27.893493200808752, 16.848701912201008, 22.958185842809222, 1.3835603182168887, 0.022734000086454903], [33.875369897233291, 50.305679931588678, 53.221656231933117, 54.39427028342638, 0.59771979687650922]], [[2.144786022311755, 39.278639265506499, 32.457361124521285, 0.34764727687863006, 0.11965576897443976], [4.6571896292693911, 28.969127975919442, 9.0592380251372191, 17.353113803571365, 1.5478430078857368], [0.29459505680386217, 7.1130675999516066, 35.426956369201264, 9.748222612270645, 28.078600441157569], [0.008387743302750857, 0.34933014139042456, 0.013534211574953787, 16.597223136566445, 25.598796215454659]], [[81.06106675537319, 19.245799978853146, 1.0039050789787469, 0.00096524841205578211, 2.2284888928669035], [20.642288524384938, 2.5249454081854852, 24.755804401802767, 44.882672593229216, 4.7203000379842752], [2.8078182713807629, 22.688186314301099, 31.729506678950756, 5.8231552368191259, 36.442331690301877], [1.439504466505602, 91.254312636177374, 4.0291992213000931, 5.5521554736712809e-06, 49.349056475575303]], [[21.937540526550652, 30.057685006713147, 0.025751257591239321, 33.36546665509983, 4.3372241255447701], [32.67876388521514, 29.508811583393982, 18.592462937596171, 3.2085251256357554, 0.9235312679115838], [1.2076427811674124, 2.7473768122270013, 7.3878916116905824, 4.1639138738602179, 4.7153068594196954], [7.4254612865540883, 13.494489327320656, 5.6951463808872456, 10.771650341732553, 6.1262127877974129]]], [[[1.8592851030947986, 6.2932131141220085, 0.61585146940345159, 6.7121911259360276, 0.059044383438177847], [6.8510078790019477, 1.7241374385109112, 5.9540181096301108, 5.4693461255411329, 0.98030578901477394], [6.8683612581786138, 7.0629061389829859, 7.4635381546945521, 1.1925109783325292, 1.2830207345712101], [1.2904261450783538, 3.6107780929159126, 6.6540326219704626, 0.1364752389697865, 3.9926953297588543]], [[43.758802555541223, 99.95218747150787, 38.141354555002025, 34.776918238457426, 0.65332936801128871], [68.980380867931956, 64.270599642622727, 17.517551262195678, 31.669993061120937, 426.04160927314342], [7.1035071535414538, 890.30890496344193, 0.16338509809983762, 906.15946622795502, 17.003878196696128], [30.626264342968589, 59.235597006835178, 128.36037135204043, 1.1265364546043382, 122.88284098458034]], [[4.4541654248941427, 5.2027472212754855, 2.2694012099159351, 5.4737842888524053, 5.2212797153710406], [1.9201812683114114, 1.3533697120474275, 5.1300713128224213, 0.10653031565243731, 1.8599173111545286], [2.0815315979787123, 0.46556913300712499, 1.4193647695524916, 6.2851661165534249, 6.2953171592514678], [2.5569356825524845, 2.887942529111386, 1.7728148137760731, 1.5853129920119422, 6.1698078132317136]], [[7.3322271685367655, 31.346139509596597, 64.690498671256393, 114.0570888647337, 18.644059249818493], [0.55170496216648712, 0.085930138161247338, 1.1866295648403613, 7.0062938459220732, 23.168198236862022], [3.2252163919291066, 105.17922360416081, 39.260694830336902, 63.488411127540836, 57.38699842477692], [71.142136377457234, 72.735697273207904, 108.81426378389665, 0.21019688910057283, 66.200252848179446]], [[1.3557547441018254e-06, 1.7588822936734951, 3.1260940530981052, 159.58560576544238, 203.63901995143098], [49.469524313018546, 0.829029021439178, 0.58323932475614926, 412.3890358085585, 33.658121030072657], [12.609873883271764, 8.7798807015769125, 0.24722664031391933, 40.292358350008968, 0.05413729305354345], [9.1733503365915503, 13.105913312820945, 9.3017943998332026e-05, 24.935226601401027, 50.214529409726801]]], [[[1.5854001063748751, 1.8306488011917375, 2.1567305050763173, 0.99511016749252568, 1.4185820048233801], [0.52800860507179326, 2.2243464647771232, 0.73205111981392246, 1.7997227463585219, 1.9382260478308604], [2.576271803566323, 2.4789107025795665, 1.0173147355874017, 1.1641161391298167, 1.0335054236844765], [1.429624565316495, 2.4815831341629133, 2.4766521340275069, 1.900565139412342, 1.0044519977661466]], [[7.476643698291328, 2.2886121112267341, 11.143924732724029, 4.0172508410786181, 0.75778550242483167], [12.235866154705091, 1.2216777362407389, 0.25471821307974013, 0.64429321919422866, 1.8494847913781056], [3.7108927635040114, 3.5058392047066111, 2.3481099854512344, 0.027278517164991078, 8.868325168928159], [8.5606244417250217, 12.828681624440243, 5.3390749428724709, 0.67238788344481804, 7.0050976292153351]], [[187.57573834917025, 1.4296551053389772, 93.161143230987506, 5.9503555453565289, 28.221820073120053], [50.44701430045604, 47.439483855298036, 401.95443173314175, 346.35333781190167, 369.37114011480242], [118.27693868584987, 35.922242790978252, 301.79969212427903, 497.2425933703995, 0.94781986243849392], [223.16229140394998, 30.349875996984728, 0.0040452590113181294, 281.77045807086569, 677.60569768918083]], [[0.34198378379481081, 0.03674296831011762, 28.663718920541022, 553.54929165637077, 0.75109836592289503], [0.10203477255356443, 549.42980677201956, 52.211574768735986, 81.002819315790603, 4.0656347504494903], [199.19929230294207, 43.42990362500521, 8.141604979256801, 0.19922097739686129, 54.850274896099428], [0.13498248527480219, 24.055800463593638, 2.3273044980966513, 30.609525386477458, 5.8333551091992]], [[167.28777249529918, 36.462709480986028, 1.569405769042814, 19.233714863553139, 2.0669823456851972], [60.356694403595853, 116.20981150827143, 25.740235638734188, 91.201157037677518, 4.0477927500105437], [42.892208278387507, 2.3902232844513525, 56.237305948049652, 126.41337472851258, 2.5685717674437938], [0.26639370718375321, 0.97554610243143913, 112.59623952324556, 2.5409056357457094, 0.40549854819856468]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.8209511370877753, 1.4468151194658323, 2.1053379837350352, 1.237160453577191, 0.23539080866995332], [0.2756498506102032, 1.7637941049771446, 1.4683049400832362, 4.2715541630750247, 2.1400134881008319], [2.2320758260937357, 0.76841822790917713, 2.615842044887942, 4.4515135805708796, 1.8371380445768046], [0.046352984983620352, 1.8164223760593505, 3.3933866733351423, 1.3954815283404045, 1.1899942882993007]], [[3.4185979180784711, 0.43105113071927259, 3.4719189627647933, 0.68899143415519859, 2.8113595368297313], [4.7861675321871555, 4.0944320353875421, 0.59400161139638663, 4.6385142564337993, 3.1414331282821473], [1.9418383604838472, 0.45936247242341005, 4.1203876681304914, 3.674318806104258, 3.327944632317092], [0.7020844555323833, 0.73043309145192148, 3.9888333848541038, 0.15572358379073489, 2.6062269966597609]], [[1.3221153208047005, 2.3983037980427087, 3.9565452210210053, 0.6129412898816492, 0.64542479955339116], [3.8534873796112148, 0.35693364971716696, 0.021440600349234465, 0.80020495628658106, 2.2588051568397898], [1.2344426069272656, 4.0862538486174156, 4.9959476504535569, 1.241885368365538, 3.5491411519425116], [2.6896576806111891, 2.6896951354655014, 2.5685533886379925, 4.2802427441335045, 2.6023194706310777]], [[0.20417084720187192, 2.7804833492105168, 1.0773707554201311, 0.27988202612054131, 4.9744947598583851], [4.1602177081389593, 3.433759544530794, 1.2158019053670106, 3.6047997475747793, 1.7798531792509915], [1.9283838592058442, 1.7664890569831821, 2.7734838284630268, 4.5765656355949469, 4.4059659169718692], [3.4760388499793669, 2.2407887226952967, 1.4772826631520879, 1.2912450752827158, 3.2292386198531906]], [[1.3488891636980909, 2.2077608808255831, 4.5449595532216618, 1.0919908273904166, 0.97827340579310684], [1.9214204690910661, 3.5817582293789183, 1.6641567364030476, 1.0432605734849725, 0.3491633811497093], [4.7031290012165892, 4.1215669434154787, 2.3198328481311172, 2.4269880042343628, 1.1336433055751398], [4.8371606352268408, 2.9577913328511469, 2.2099866069374716, 4.0265637104586736, 4.966625009223856]]], [[[4.7222298838359063, 0.85795741290331917, 3.658606817115281, 3.4339270022600279, 0.78277021644025258], [1.3501889192708545, 3.8729550807961486, 4.5167922223215822, 4.1594836917939144, 1.6323344093813725], [4.2494796731057045, 0.26213607969541453, 2.6740653222186994, 0.33391915743032191, 4.3909291528466881], [4.2485313496502313, 2.367246978894034, 4.2982713043735821, 0.72355016098623315, 0.51649124311989592]], [[0.20287819120604514, 4.9286245639430364, 0.12903001242490969, 1.1975230064605944, 0.79845372429730144], [1.423257877382567, 4.1538649773029839, 0.36238416572925053, 0.45621393479377326, 1.3610706845936908], [2.8453365871820324, 3.1192017256763931, 4.2779888481444539, 2.4038774869386388, 0.9211967288390045], [1.5222532964362669, 2.2829195626505312, 4.0017955048433214, 3.4027932038606998, 1.3833291483889729]], [[3.1045045261150976, 4.8216595733741281, 1.361230385060298, 0.35309404285454704, 2.6493432257183875], [4.010362256502467, 2.4465937893021485, 1.5086855789439506, 2.0314674004854671, 3.0262041067620666], [0.32435801713962081, 3.8669394685941376, 2.4755150795962573, 0.88495222048287348, 4.9703776554801902], [2.4467646956175466, 3.2396761666188389, 3.5964612123105861, 0.6539847173201615, 4.0053541090488931]], [[3.7543864862946976, 0.89328810377774315, 0.96097041811487771, 1.4916537440376336, 1.978705807222934], [3.3338430303110922, 1.4442947609779306, 2.5419791358351809, 0.99297820205748033, 1.4971370482342548], [3.639054889237872, 0.87387563074952346, 0.18226601153833413, 2.4833480916245945, 0.30901985229774698], [2.2159321956854829, 3.2490843117176502, 0.21547384731664029, 1.5391657139685064, 2.0165230996785954]], [[2.375117126813854, 4.1145215066126974, 0.43217548858885418, 2.7946398580320131, 4.9617640501543123], [2.9454202947123163, 0.98891222572257531, 1.1543793977122556, 3.5095008947413913, 2.3580880120539276], [3.7312077832919788, 0.028369534838703921, 4.2535785210634662, 2.7553598477134833, 0.17852111835322299], [0.50064518863192042, 0.62481608097934849, 1.2244696900432825, 3.4574766636171068, 2.7320279720073799]]], [[[2.3412867345058008, 4.7925191552679403, 1.8569815521390169, 4.5981512405172049, 3.7108999765422688], [1.530733790154712, 1.3576169291632838, 0.71860695550761422, 3.4757046993719047, 4.6626258867699004], [4.5906852640186706, 1.4198796850945454, 2.7731764492276083, 0.55385833393012018, 2.7822647287726028], [4.7055614023585788, 0.92963932171711883, 0.70610879759010292, 2.556795213256938, 2.6852130847909352]], [[4.5576775609449509, 4.6887314112108323, 1.7938711183698137, 4.3467863278528007, 3.5113843532094378], [4.8943440516808243, 3.1963885032589197, 4.8124368692613499, 1.2530035240502739, 2.4336842185066181], [2.0865651490844077, 2.361216318387056, 1.2613276584059965, 4.4291457558451217, 4.5141033587016031], [1.3810434634469781, 3.5003949295417311, 0.95202395085950375, 0.54133226721722116, 3.8966142956233263]], [[4.6098429096947982, 2.6686313960327253, 2.2100861682521065, 0.9949747666105937, 4.8666567850751941], [0.035316258882655716, 0.76331872931348677, 4.782178086163646, 1.4692314292969844, 2.1502874389439821], [0.52011556551022631, 4.0033566086542001, 1.4174958299845557, 0.73088707320437329, 1.4676194318921723], [0.31307901621026707, 2.1519182594911461, 0.22565836182012936, 4.3962106911714187, 3.659925982670825]], [[2.2861508436348004, 3.6123367906914936, 4.7645283447615467, 0.031266374832193286, 3.4348180427316355], [3.7259844960050632, 4.0740910673104977, 1.7416725439669949, 0.80464903999512383, 2.9990448825833682], [3.2017203799044172, 1.237210112804523, 3.3656337827221159, 2.8440453502342229, 2.0736672101554667], [3.9927045015347207, 3.6201283854623254, 0.23147334676216869, 4.3117009573618938, 2.2594977647763104]], [[4.6146572393113079, 2.9054801175054568, 2.5798353468724686, 0.93421078302583538, 1.4035922156918463], [4.2161790300251587, 1.7946019909686943, 0.67513602798191386, 3.8299325517436698, 2.2730770566335345], [2.9202183332144656, 1.6544630812352161, 2.4973481589237294, 2.7922164197488604, 0.16127483885469476], [3.9579925210662381, 1.2657028420959262, 4.434798116886137, 4.1880063317068537, 2.9956661676484644]]], [[[4.5115219595181042, 4.7083085460369007, 4.7207368342787666, 1.4893031847159415, 4.3685125539795227], [1.170781082366702, 2.8426807169190922, 0.25722550193407751, 0.62428127656968635, 2.1867433643681253], [0.97528284362311246, 2.088413922833888, 3.3748608145988976, 2.6189928178260562, 3.7975192192862863], [2.9654024182684933, 4.6562484724172863, 3.8840936757001803, 0.34478533793181215, 3.5232975410203506]], [[2.4267560567940527, 4.0251370568230831, 0.15003912687507071, 0.87160772944137599, 1.6419170422519587], [4.0364153581346143, 1.8139543598353529, 2.8053971894681826, 1.4459968678369515, 2.8004047801379417], [4.6666310146268417, 3.7764247314385102, 1.5544411052946614, 1.384859455720173, 2.6707858160099187], [4.7024336943781275, 2.0577440351265244, 1.1942099443155616, 2.0281214729185368, 2.7921445790561359]], [[0.77188087144384665, 1.3829266910980702, 2.9992758786139069, 1.6816457393410273, 1.1351154121088796], [1.3377019640815484, 2.2680899417616356, 2.3041717804445496, 0.44607424575693672, 3.4384790881431431], [3.3620793414934811, 4.1068065932443156, 2.659166208388426, 4.2312860633344496, 2.3066070418846096], [2.9035740795406544, 0.78192243231685943, 3.5308059861760657, 3.7010038783685562, 1.4542979461901722]], [[0.77121313469925434, 3.7058016889396588, 3.8811212544949294, 2.2138480067846737, 1.4256601614007718], [3.3641381933869159, 0.53796131032901096, 2.6789718913566434, 2.3371423866786172, 2.7790068914399724], [4.9717420745947978, 0.027597533073954482, 3.4905893769310485, 2.9246627658167439, 4.8780150034812086], [4.7573255813430801, 2.3361181904064554, 2.8457842248261636, 0.43742362870311069, 2.777847636914307]], [[2.6084208753082447, 1.9797457653618238, 2.720398965671337, 4.8350915907864991, 4.8360613195937363], [0.96455057729157589, 0.51241604371915839, 3.4394986768175198, 2.525227646542084, 3.3480179083377442], [1.1348383418995729, 4.8322899231893937, 1.7673092305340665, 3.9468495838479836, 1.9566029030988887], [4.9732345159893665, 3.5894551069898375, 3.6092892189737622, 2.953040197278654, 2.1420700114475562]]]])+(1.-msk_arg1)*numpy.array([[[[3.4744331256796972, 1.0572752166941479, 0.30910664547831257, 3.6703918166477787, 4.4363733087140522], [3.5231892269210046, 4.6638162229424678, 1.7461532219672373, 3.1354607304053186, 1.1329817857376578], [3.2395128774821984, 2.3671027956035302, 2.6530754476695422, 2.8841739707307732, 0.99935649953398176], [2.6593866195244891, 3.9825097584520397, 0.45145662826502714, 1.9401809066636648, 0.34110086308547916]], [[4.9553313584830825, 3.10216065144966, 2.4113554236870138, 2.2965100326553181, 3.5127627887202779], [1.8598147740153783, 4.4867031112267837, 3.8829047865836102, 1.0195640862569424, 1.9517212852708412], [1.3118726343162066, 4.3209001446173492, 3.3870700751056373, 2.0251764047547041, 2.0637259831407451], [0.10737540921121758, 2.6345775298072214, 1.1353369251719589, 3.3371772901653243, 0.38278263492470871]], [[2.4438710046126202, 1.6278786252533826, 0.75880457396349743, 3.1744760781899721, 4.9876910414131688], [0.26063091846843978, 4.1995701844867863, 4.2769397326027967, 3.0895558189342101, 3.4660694140646062], [0.95015089105435702, 2.7001204848389149, 0.093934747885888153, 2.7190296019755014, 4.2068150800565096], [0.054548857993292656, 2.0350996494715048, 3.3022041708000658, 4.3489856335975459, 3.9244497283192015]], [[3.888886232418006, 1.4840680461111309, 2.261120494588408, 3.2272521563083401, 0.88779009030683609], [4.0945962364848043, 1.8260785925965097, 2.3402023996224894, 1.9196549887996179, 1.9128223313478998], [0.29260274739079462, 0.060771438986811671, 2.772746929589184, 4.6500420554978099, 2.7795973706739026], [1.4054713040626119, 0.40866328936448593, 0.46136437345870474, 3.3056007032468679, 0.31256868147544642]], [[4.4545222824796245, 2.0788012278799211, 2.9299840575041283, 4.4012208924141438, 2.4315540105826714], [1.1558802205124077, 3.1859041240218784, 1.902365147667392, 4.5383599701999975, 2.9884952612368036], [1.9542925523885815, 0.92941987630163303, 2.2187610059711536, 3.2751591136261964, 1.9630981254374067], [3.2618355218137842, 3.1088650540834073, 1.4315273410857603, 0.98032671735625831, 4.4253920296259217]]], [[[1.893514499336955, 1.9049461849913227, 4.2310662581115563, 2.4937575431431012, 0.91236108052937814], [0.61612375761232951, 1.1376327752701387, 1.7500714091469523, 2.2518415527282394, 3.3582642214511078], [0.33065485990338994, 4.5355760740109199, 1.2011521716974807, 1.9383935680226123, 4.7178990610916438], [1.9904588524072138, 2.0037389638937828, 2.5071842511830815, 3.1437844082617499, 1.4334798001798317]], [[4.196366762806651, 0.16184045888841642, 2.5078949148119749, 1.6904008917151729, 4.7591442739888192], [1.0248024260439643, 4.3910326956575094, 4.8055141830799171, 4.2364295270331898, 3.9528188592960953], [4.3676724540449561, 1.6379369152515499, 4.1291058481445253, 1.0314953176265389, 3.1251831096881624], [1.2381718998524889, 1.8021518576803586, 4.6852051033857007, 4.7488340668060083, 4.8730143796926235]], [[2.515138209406607, 0.74473005476341825, 2.9654232273702634, 3.3173743219041607, 3.2660578864106333], [2.0380286253642854, 2.4582307426638006, 1.0857776648446513, 4.3056079042993858, 1.5422844891426513], [4.9411952182621128, 3.8614580765494115, 0.60368920413386717, 1.654479642740194, 1.0814716221233696], [1.584804357087634, 0.53643466483101043, 2.4034751726748773, 2.8092501110091481, 2.7446697637689188]], [[0.84222672231979789, 2.0836143217433238, 3.8792053370489827, 3.4458140211733452, 1.0356422469203823], [2.4587285830231904, 3.0182267254491442, 4.2268706096216819, 1.4460576313867277, 2.2297079477860673], [0.1413146131704959, 2.610753446875965, 3.456652885612423, 1.1444095061442092, 4.139552766543483], [2.4456458098676039, 3.0945510104523062, 3.3519899918042451, 4.42356986899616, 1.8712590640011548]], [[1.4312844075205564, 3.7938143308442389, 2.7214634249993868, 3.3039635577865534, 4.8754101295000565], [0.3286210668118672, 3.8310271425106763, 3.6990793686496026, 2.3541877107199869, 0.71899777056158587], [1.9699484488688794, 3.3556253748009732, 0.38716024651776437, 2.5956369507648938, 2.7701132678112801], [1.1923424518993828, 4.8932388877650181, 3.3214064588120804, 1.6501797135119121, 1.2235496167721223]]], [[[2.5043816392541567, 1.9644999797078702, 4.1779850783898773, 1.5063557683038578, 3.0488986536681115], [1.546112296724119, 4.0938148150073639, 3.1417690118098642, 0.95307012067367858, 3.8908395905820452], [2.9049959321016288, 1.850455392062246, 3.4508259128513119, 0.70589532942461664, 4.2231424564385849], [0.85119462991503714, 0.28914343431640854, 4.0926568506246754, 2.8606648745768064, 4.7923716525007887]], [[2.5854060499166707, 0.95883171350775842, 2.9679012669855789, 4.916052331535643, 4.8142865644684205], [0.54327009564601458, 0.72877652066011533, 2.8028496254121773, 2.1496209114761755, 0.62545494897682752], [1.6129569854342147, 1.7192364873999326, 0.26769157636526536, 2.0516521731266328, 2.3022965789097425], [2.3139422164623227, 3.6440432446434312, 1.3360386420191532, 1.5434678304418332, 0.28115111056135994]], [[0.70052759641298734, 2.8634564837990943, 3.3804483261727243, 3.710729286239308, 4.3730744551994167], [3.3481242656495223, 2.242855497166206, 1.8021829469604977, 2.6799845966615461, 2.4580561210319818], [4.9334199170160264, 2.0583620641658826, 1.9819915354273505, 3.058859737178854, 2.0667108720446059], [0.16121434194484105, 0.15895823040557447, 1.2880405389269165, 0.43412561190023435, 2.6753136504610753]], [[1.5277019689743787, 1.3429788661713411, 4.7295134107220989, 2.385273171734295, 4.2848116280789883], [4.0540881627967487, 1.8359318318094715, 2.2871344173370978, 2.121514315586841, 3.3595162031389001], [3.0486760806432849, 4.5861383840907006, 2.4787774331391752, 2.7149079374371841, 1.0666977084698557], [2.6263787463576791, 0.47822359151229565, 1.5542157894953788, 0.06601300594570618, 3.6710291280479064]], [[3.5536594361085094, 0.81382435948765974, 4.8201349800005042, 4.1357696196726668, 3.6798269298791939], [0.067946615126193752, 0.79198578451191604, 4.8770644158525256, 1.4298032686976916, 2.1019418827707987], [3.0464509679485299, 3.0252179252620026, 2.4160729892877488, 2.9845588133986749, 3.6739406264667918], [0.19585371250152567, 0.75664679757114717, 0.74048538001930908, 0.045070504126316642, 1.6322861604816432]]], [[[0.87848806097351528, 0.55239785906949834, 2.3200953162817837, 0.60083798710725811, 2.326079994432857], [1.0625002275356363, 0.24077176299150033, 0.50790312149075101, 2.1315000521990872, 4.1496299336605702], [1.5816830730019407, 3.4196540463705962, 4.7629556538572713, 4.9957338787716941, 4.6427588738730465], [3.5153611973056345, 1.3142723586048544, 4.6460545741519255, 1.6338805459107366, 0.72046309126267383]], [[1.1195431401078377, 2.9021412523291823, 0.55564826385665256, 3.1074086190883832, 0.15870199610197794], [3.9455632214447238, 3.8596687552110205, 4.5445701178361437, 2.4833914836825959, 1.9783105434720272], [4.6389765679096708, 4.5265149797447517, 0.94550993911053982, 2.2598212793074328, 2.7790911393419684], [3.7077382445593901, 0.13101948553319856, 0.50200267367641704, 2.7669028343537847, 1.2962434130623803]], [[1.9742213181911181, 4.58644449259707, 4.5549201762265321, 2.9054664705913336, 4.82779369969853], [4.8330534841709936, 1.792387721578927, 2.1766392081907258, 0.43946366389969688, 3.5757411213374666], [4.7828867112447284, 1.0998958097858094, 2.3530623953168872, 3.3105003214775559, 4.8654625843034003], [3.7367914662925368, 2.5913868180818294, 1.2303513128580452, 4.7179056129338761, 3.3931669037766685]], [[0.62606153200726133, 2.4564622337868456, 2.4928364222365516, 3.2224399720745622, 0.51237303940276946], [4.1631927094105574, 3.3980921592539555, 1.2233222344725878, 0.28825589298535914, 3.1581496500580912], [1.8691872152864613, 4.9155743652252983, 3.2856870121668535, 4.6462484055194331, 0.35576068696998864], [0.81357080983742569, 4.6834810210041917, 4.6426775962725122, 4.2834500303711076, 1.7813551199544149]], [[2.4948977217270949, 3.6134203398498252, 1.7781386309257732, 4.6770001866183843, 4.8158860455265895], [3.5759635825991296, 1.5861276705377205, 0.64004891160703092, 3.6059271342140735, 2.3247891718592824], [2.0111275239066644, 1.4117689956206876, 4.5962774581562407, 2.2271304813352972, 0.90985584541392839], [1.349571400138226, 2.0642804759044684, 1.1462393375052575, 0.31758363773743337, 0.64225585384022699]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[0.087203687284069187, 0.22956538405653529, 3.6555538546991979, 0.20296227845988291, 4.2862996028905496], [4.1326196525841556, 4.2737066828398733, 1.326008849078014, 2.7770144228811144, 0.68243436770633703], [3.0448805504392067, 2.1377012164958904, 4.9687863339602565, 0.11937341914545618, 1.0165392725482856], [0.7139754502253417, 1.557017554150099, 4.1193973508874011, 4.558460967116738, 3.9595516965362831]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[8.9502553524288672e-05, 0.029319043697142156, 0.0058812247313614332, 0.048895806405537895, 0.56313386001880283], [0.51045632486448833, 0.013530741963771631, 0.027821598871267957, 2.9815165400932285e-05, 0.0054041859789171016], [0.0043171133091628308, 0.15342200923268717, 0.0016928033605569783, 1.9221022694878018e-05, 0.011314002331803763], [0.8930805429936467, 0.011900461202039437, 0.00025399494606550464, 0.033230429279364017, 0.054858322271889644]], [[0.0065342531483392353, 0.53029554894480446, 0.0060411388469388469, 0.36280226124082182, 0.015969014594929887], [0.00087335532053559387, 0.0024169891340990999, 0.41723177249418109, 0.0010853135711700666, 0.0098249465997642393], [0.057409469682000176, 0.50865624937349108, 0.0023264119459801475, 0.0044850405438384804, 0.007466754809692], [0.35587896519799389, 0.34133819480143451, 0.0028233330986768743, 0.79520308183192157, 0.021596183602434978]], [[5.5499495162909005, 22.394056271543253, 168.79117618364461, 2.2133864770319915, 2.3085747832638965], [147.68384786684413, 1.5883160901251825, 1.0281821366514501, 2.8214861966766493, 18.689666875889731], [4.953742171856911, 199.69611902139869, 649.35867848065755, 5.0017654846492299, 99.54061522121944], [32.670167547315543, 32.671753747466838, 27.923851174203026, 256.78892639420769, 29.173197901458785]], [[0.72209510520271203, 0.011865330950216784, 0.17940292952983256, 0.63996761758138609, 0.00035870810243007365], [0.001314307511553556, 0.0041863041457981835, 0.14386469792441103, 0.0031869184841842781, 0.058519549286924939], [0.046177551416418966, 0.059780116736481023, 0.011998518263975055, 0.00067661371684654714, 0.00088816887112103484], [0.0039133507849580633, 0.028058476166863369, 0.094810551696607751, 0.12755675287801296, 0.0058006790263105748]], [[7.1221196783543972, 24.859195508495922, 746.0870567683387, 4.9003637477967814, 4.1528813521372667], [16.386880993937059, 183.63978767439539, 11.268974347390367, 4.5648527874085048, 1.6622642750767227], [939.21636196266672, 402.87510935319278, 29.263468607306109, 34.202409578495462, 5.2066228726834476], [1141.5230971735712, 74.057357420600582, 24.939854330260978, 350.84925051663816, 1378.2146802694056]]], [[[812.7522343469675, 3.3782764162071071, 179.69110666257257, 130.63909596110958, 3.0364275165861661], [6.7923609471545658, 243.5641895560731, 607.2426396103242, 365.74605814669025, 10.136452373469066], [415.56356860132752, 1.4505574040965941, 44.445425756326827, 1.6060885227054833, 507.92809368929193], [415.00476768243306, 28.75794198236828, 445.35272228163348, 2.7917081465224132, 2.0810141910965139]], [[1.342692363706623, 1285.2854710729705, 1.206126167560196, 5.6938124161517312, 3.1891017183145438], [7.9031054252869533, 417.13593096767494, 1.6927531836546892, 1.9399114201626215, 7.2205420433545759], [62.352192066591307, 92.812644259650554, 499.54604278740595, 32.838041370867622, 3.8114939712362546], [9.1252453922075318, 27.547133713522175, 334.46560285785546, 140.11930893749815, 7.4577967801231608]], [[2.4012972459738338, 3.8983168079168387, 1.4682970074580346, 1.1047662840395471, 2.1118717162411316], [3.1006699959922974, 1.9944415617925744, 1.5306786339385885, 1.7739814000717946, 2.3488240578340362], [1.0958444553083477, 2.9776907871266776, 2.0107844425287587, 1.2836532613654086, 4.0653879849886536], [1.9945377465434624, 2.4946563249483336, 2.7588840895089488, 1.2026617186364745, 3.0962913266508969]], [[46.276846531475897, 2.4902507031100032, 2.6684893027874925, 4.5884392525254105, 7.5458928208552045], [30.117566288140843, 4.3717727322289104, 13.414249047273012, 2.7571692057086126, 4.6142089814610951], [41.13440144234265, 2.4413616758851124, 1.204617637213651, 12.634524906442012, 1.3711167926593779], [9.6147738008461339, 27.619941891010274, 1.2461762997070753, 4.8165964070818594, 7.8430603432886254]], [[0.40352939638844626, 0.2076060297192123, 0.84778393765831028, 0.34376364777801771, 0.15019286072438293], [0.32451858799111138, 0.6853316444726365, 0.64334408169766066, 0.26159928033891294, 0.40616357585069962], [0.24035142489491376, 0.98921885272599597, 0.19686339289933177, 0.34896192107507279, 0.93406342005417142], [0.82589223508758514, 0.78762333690408415, 0.62634352102412305, 0.26685134709997527, 0.35208677277978334]]], [[[13.557407036387426, 207.73981206274627, 7.9064413219849365, 167.31280926820159, 62.298970052563348], [5.4981595327167652, 4.5342260898968112, 2.2258554249889899, 47.945452907670138, 179.76585834539102], [165.92769000055836, 4.8597238174921351, 21.92934483552138, 1.852803739537916, 22.152384380277297], [188.56867575518112, 2.8154377445677614, 2.1950944661817546, 17.234160226067598, 19.883333173874]], [[31.900101006163784, 35.239755252650994, 3.9073507823568936, 27.177476417525885, 14.406906863050866], [41.197812648868293, 11.340672567463917, 38.712313738698512, 2.5907455403457749, 6.353116474342432], [4.8804056405539171, 6.0127929741162411, 2.6071815998279564, 28.93232761059808, 30.861347210381599], [2.8554276044689484, 14.28712423387576, 2.0611872658001831, 1.5087280333382918, 19.305284851916952]], [[1620.3342392366558, 72.116637332349399, 34.5758770946493, 4.9289169596269788, 2445.746003473444], [1.0582516556598054, 3.3998590365806685, 2135.9595859104084, 10.542724441998805, 31.415090940624083], [2.3021341637561181, 612.82761477655333, 9.7035785929962177, 3.2276048537134083, 10.515513897317106], [1.6518909192060882, 31.497332899654239, 1.4358686287573459, 1150.4377082503597, 353.36590449993446]], [[0.0077566037722804904, 0.0004628917947812837, 3.998529035420575e-05, 0.93570348728617969, 0.00067506514295639876], [0.00036355670049972806, 0.00017347529492584137, 0.024676034534020706, 0.18081506561182845, 0.0017045296553998667], [0.0011079402933420717, 0.072100917103847431, 0.00078200430887733819, 0.0023696458623691549, 0.012184665949530759], [0.00020623622570377595, 0.00045528896144017854, 0.61140462621659508, 0.00010468917051041384, 0.0082087090116712835]], [[1.078637622166835, 1.0488155314842678, 1.0432278297534314, 1.015442809481728, 1.0232916205656584], [1.0716099586961405, 1.0298762302956437, 1.0111364775694156, 1.064841720841464, 1.0379914312907843], [1.0490691295706747, 1.027511431844073, 1.0418171730943244, 1.0468686661692626, 1.0026490531443348], [1.0670809788990514, 1.0209796165196188, 1.0754598923227607, 1.0711148343408881, 1.0503683089506874]]], [[[0.21872020288868149, 0.20468957864558357, 0.20383429969873421, 0.60546639188424123, 0.22951631758895669], [0.67405471511501236, 0.38376757564623071, 0.9169879062623626, 0.8103212174446961, 0.47867735346797702], [0.71994581621078579, 0.49480049187363168, 0.32077567809621532, 0.41380673851242855, 0.27820148241662002], [0.36822395781579914, 0.20831137773297317, 0.27020423354745399, 0.89033232182420075, 0.3051284174922399]], [[2.9285283826177202, 5.9430273453541513, 1.0686895191096448, 1.4709723478617016, 2.0688546956785419], [5.9727793722547329, 2.2326037449235412, 3.4630572363174683, 1.8969496844165041, 3.4554106032031169], [7.895194849702043, 5.3233125126661864, 1.9902559665412078, 1.8462881577963064, 3.2626822191264382], [8.0213501664868847, 2.4870861039439629, 1.6968319014710675, 2.4546783633393141, 3.4427959113879973]], [[2.9824856368437556, 7.0838590955028158, 69.832218906719859, 10.812657520949273, 4.9877788824533367], [6.6445294171631346, 24.802650732861171, 26.102519417207787, 1.8804486238598968, 130.04473374905595], [116.71310741556661, 334.96777834690045, 43.146439195008796, 399.51878493381599, 26.192666196964375], [60.983894025846496, 3.0251871761060181, 148.20349886402545, 188.58270205316936, 7.8370274130362594]], [[3.2217310327254127, 276.34381445639428, 360.54001766925222, 28.742427750385005, 8.6946262686255338], [164.57187040433274, 2.2616152497452631, 58.204363933271203, 34.65389337218464, 67.742376817888925], [1885.7076603940025, 1.0427537447858912, 199.37190165274035, 84.493086916010427, 1635.7813834735182], [1362.1121552873863, 34.600093754048395, 74.964311317616378, 1.94170239087407, 67.623351672468999]], [[36.216989586941757, 15.247097323408722, 42.250865323998397, 775.66328246448029, 776.69907567280518], [3.7710284704717516, 2.0241533325971601, 113.658700577248, 32.299235750537989, 100.21427054817752], [4.7668514219337972, 772.67850040330825, 11.382176699717501, 228.46460992714799, 14.769164678680598], [938.06688022737114, 139.70873164203357, 143.57450509406027, 58.193263542063846, 19.063367125515303]]]])+(1.-msk_ref)*numpy.array([[[[0.00020842949838026066, 0.075832216948336431, 0.47044870904496638, 0.00012922536847106942, 1.9944221184183271e-05], [0.00018505626526605, 1.1451153607690557e-05, 0.014125749902015924, 0.00047652821854094379, 0.063044220759560399], [0.0003696996160968269, 0.0031055396527487149, 0.001545820348325737, 0.00087966677330043928, 0.087340689315604902], [0.0015222029286494402, 6.0348947012221753e-05, 0.33242787626994236, 0.0087992557869005714, 0.43512648908484647]], [[0.0006808939132911105, 0.010409478849856487, 0.028768564093441695, 0.034065529197883124, 0.005688736131943142], [0.064774532037770963, 0.0013569903668711246, 0.0032995932655075595, 0.22305045810199192, 0.056580583400917449], [0.14507433809921877, 0.0017319700557780198, 0.0068445546965467926, 0.050783511152181972, 0.047982838500609894], [0.85384111465151258, 0.020713732660809961, 0.18811067392882314, 0.0073659939505787255, 0.56933262135016527]], [[23.756637413443862, 8.2493448555655302, 2.6740618789190043, 61.246520871942728, 642.44590428702043], [1.401919272203253, 231.29242899957322, 255.69182955705074, 54.862461989507416, 89.378862060243719], [3.4268135203255503, 33.116271215435937, 1.1294860349498697, 33.938010403571603, 233.47476884262576], [1.0732686735970314, 13.985108252382499, 72.274710412907837, 280.72127015415543, 161.91291049301407]], [[0.0020258953642247075, 0.093790148069754595, 0.02716330484570751, 0.0058190840371013909, 0.24273376541894545], [0.0014593027984677305, 0.054360815083384285, 0.023944826385242656, 0.046824848659068496, 0.047337855125572051], [0.62711591525386134, 0.9076337424375287, 0.012012626713232624, 0.0006017999618097978, 0.01188210733830748], [0.10631440676861728, 0.52115399400433216, 0.47914405886439371, 0.005135605160759147, 0.60746284917259674]], [[654.07219513288248, 20.605062064409491, 71.119995147055249, 605.2500170244931, 34.430457717036575], [5.3778871809779085, 103.217733856907, 15.938658445424331, 738.95505564233554, 77.441833670956584], [17.189932412778699, 3.8678539008643225, 25.260390312086546, 117.53598169295802, 17.411653582747263], [115.27874225594162, 92.269734325846912, 8.032364626836225, 4.1653105453965642, 626.92121087577164]]], [[[14.683574773727869, 14.923692593668267, 404.84674547067544, 34.412548057942502, 3.6493885509465209], [2.3970170962305049, 5.0238683898848935, 11.979495211327711, 24.414174484139544, 117.3404734588658], [1.5986667245140669, 623.64485904915523, 5.4976934683149707, 15.649029990078276, 807.77314186832632], [16.848893122654143, 17.169392066533611, 35.074437103669936, 86.552560396011003, 7.6444582970092414]], [[443.69846574040639, 1.2649976655228394, 38.193717305142719, 11.649649191444109, 1004.8254651609875], [4.4304740702116971, 588.68655173303978, 1074.8329952478318, 470.28346288866476, 311.4990753635422], [569.04729624870231, 10.794892554231298, 402.40134632376328, 4.47375407203746, 93.622499426664291], [6.040107506657705, 13.702691602122648, 902.50466533044187, 989.88992832747897, 1185.5515209704545]], [[2.0333923682503023, 1.2338546547395151, 2.3088834311897823, 2.5499540507507255, 2.5132963996270914], [1.7772687989288993, 2.001001355224556, 1.3584953877350907, 3.3700538819409598, 1.5452595858033591], [4.0320489511363746, 2.9730887409854772, 1.1857140121566394, 1.5949628101307878, 1.3568457483772021], [1.5639112510652229, 1.1634243495266834, 1.970322349345895, 2.2093450618713808, 2.169449074691717]], [[2.3637054634053669, 8.3993514883689695, 52.569115514722917, 33.766633046214558, 2.8799719425245942], [12.320781057611082, 21.818223332193952, 74.980336186945621, 4.3796514370269284, 9.7510120656352459], [1.1552715345022797, 14.390409274498284, 34.142525426213076, 3.2183630598986395, 68.5827819051853], [12.157240495577103, 23.587133598740358, 30.680997143163147, 91.664014587064855, 6.7616020390599862]], [[0.57875391152610489, 0.23467013671822837, 0.35351087730186254, 0.28297184580319801, 0.15523111684362692], [0.88200066512796238, 0.23135706463902941, 0.24332014223837814, 0.40676931720909854, 0.75978401079711111], [0.47109501384023655, 0.27744091732460929, 0.86249179697365996, 0.37092176225143353, 0.34700031551612709], [0.63407956155661704, 0.15417724956717138, 0.28109218292967408, 0.53231738119866445, 0.62656375067443471]]], [[[16.257152537762092, 8.91197103106893, 104.79711597668619, 5.3509248377899139, 29.809638880184565], [5.5931172731546388, 95.421687984291054, 33.057199455841982, 2.8898570879879171, 76.119217089107892], [25.396252621766898, 7.8491963860515508, 46.635515764444307, 2.1945727792081531, 110.20114224672203], [2.5799566489829182, 1.3798159654501976, 95.298735411736345, 24.17310605429028, 207.70569595235983]], [[7.1293014960092451, 2.071875484196839, 9.53345086000229, 41.882899546016809, 38.766753268418746], [1.5109508615024216, 1.7396329696691664, 8.4099194787563469, 5.1198941258006796, 1.6082997216880857], [3.4055740313730363, 3.6919595049356766, 1.2255302449919558, 4.7526571776580386, 5.7495765348688561], [5.8006719162805886, 15.934599500626179, 2.7594463047419944, 3.2304467529681706, 1.2381263508259053]], [[3.0742739452852184, 98.555870496105328, 225.75544548836868, 383.35087557850096, 1108.5479841168369], [214.35449608530499, 36.440880461384616, 17.979203857575545, 73.441263126977461, 51.454381854457075], [2722.0443987835511, 27.110357805495262, 23.986243837107494, 134.81309407748995, 27.475658490168119], [1.2949277489684394, 1.290252529213153, 7.884944479678107, 2.0056728560125285, 72.893363453886252]], [[0.038885704704095879, 0.057584613995130117, 4.3074701082485572e-05, 0.0062830793231466791, 0.00011084678295777082], [0.00018100983103308866, 0.020195998988285759, 0.007740404873995561, 0.011006426289080617, 0.00079223903503845296], [0.0015338764561537281, 5.8421225780323753e-05, 0.0051506195590626594, 0.0031180940221543973, 0.10359516530237557], [0.0037636551795097771, 0.36187239363108259, 0.036754901947764776, 0.86908828956284012, 0.00040860251359676802]], [[1.0600267972209336, 1.0134394740170096, 1.0822794711560066, 1.0701973994286753, 1.0622229536473768], [1.0011152168793962, 1.0130764847551417, 1.0832906513574845, 1.0237316955776901, 1.0350815649038072], [1.0512437073031704, 1.0508776159560038, 1.0404291094681644, 1.0501769441214299, 1.0621203915691109], [1.0032179485455162, 1.0124893741709446, 1.012220986963313, 1.000739609403551, 1.0271377014944232]]], [[[0.74381084712960932, 0.83018514221461914, 0.45764767555545965, 0.81674663877561171, 0.45672586016789124], [0.69909862930559485, 0.92208522913920843, 0.84272384140024381, 0.48766984458730517, 0.24708123001594079], [0.58691244580699509, 0.31597115152630928, 0.20095552284519991, 0.18579781390348024, 0.20926025253909594], [0.30594536379747173, 0.64224388508384045, 0.20902803072319165, 0.57668141576508114, 0.78448420329139379]], [[1.6416511634722217, 3.61462231007375, 1.2789322958207181, 3.9585351627319119, 1.0727965388916703], [5.7372819947538467, 5.5231806505080225, 7.4798232329514782, 3.0028944165601481, 2.4011333296717123], [7.7991106521432636, 7.4202655661734145, 1.5199014310818206, 2.7198751136799277, 3.4229549771332097], [5.1638545048528686, 1.0597274808645905, 1.2489120492478551, 3.4045322689034845, 1.7752486633253415]], [[16.361298526390136, 660.54220237837683, 631.71088244015971, 61.14749325860334, 929.58732318303453], [936.53515717602045, 12.647956069282692, 21.790696208646008, 1.8629322630371306, 157.93780267850033], [872.32810182628054, 4.7451833104906909, 27.973191884265884, 108.49434770543792, 980.5060816304042], [198.38337237524451, 39.19872197110751, 5.7077038600910308, 795.65957824162695, 121.96446428291003]], [[2.5850012468138215, 41.530018682745791, 43.886010765868733, 132.74012803586911, 2.1755080981124779], [553.0791608877953, 173.27068690403996, 6.3965677180989964, 1.5484894121617234, 120.40557482295823], [17.039413106673134, 1731.6896205610515, 146.10683594368192, 1150.8883635527441, 1.7154640838445658], [3.4355422148415156, 1217.7628618343379, 1144.6710119665145, 663.76601380900411, 14.913831968487195]], [[30.978878233126196, 144.39304528394217, 11.553071774273496, 624.00854329218964, 755.43154526700687], [137.13881550766732, 8.8703934920958929, 2.4128075494746883, 142.9117677795814, 24.513255407195867], [15.919974048923006, 6.9781149286067441, 558.4019400096289, 21.430665750486632, 3.4976127430491806], [6.4056908159112709, 17.128092730727161, 4.8422295671073723, 1.5481127817358711, 2.4201464841046656]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.3097700907752605, 0.18295119362025031], [0.9662116552224439, 4.0109714488612047]], [[3.2029143702295402, 3.0511974656170011], [0.32492568179388737, 3.7086418293609746]], [[3.190162852643053, 0.87082202478720272], [2.0849758502846405, 2.4746467414907705]], [[1.525590598559998, 4.7476522832814529], [3.1361486676417374, 4.7508992550875391]], [[4.0639293175207198, 2.590194096052453], [4.2055643739581114, 0.41902378924596378]], [[3.9585442727489522, 1.7962307945993095], [2.8990176915702781, 0.47787983889563324]]])+(1.-msk_arg1)*numpy.array([[[2.6184401850102654, 1.2396922085927533], [0.47732942719126781, 0.18923253625848838]], [[1.2384062648743259, 3.1358082264589857], [1.1401724058538616, 4.1709008722647622]], [[2.0690482194452633, 2.2738263324449051], [1.3885789474359336, 2.1426893929400501]], [[2.2943481432524599, 3.290636821297328], [4.9394059797631193, 1.9994070926339349]], [[2.1004985226131203, 1.3261311104546796], [0.48920829679859373, 1.5458025767547732]], [[0.77064941361777028, 2.5961636560541383], [2.1088985693731481, 3.2677748434122926]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[1.9181250199557798, 3.2593750767528267], [0.016893479646777777, 1.5508337672326227]], [[0.57766652085388293, 0.65190732757652947], [2.1435110254280931, 4.4673444847379624]], [[2.6189879673142351, 2.079084043266318], [2.6259143305098691, 0.56118713407218845]], [[3.2242743076159575, 0.96989278129815293], [2.6617334985802921, 1.5779530259938046]], [[0.83393290189271663, 1.7084604883090591], [2.11831063291705, 4.9076541684498727]], [[1.5265100478957516, 2.6159603222314338], [4.0189210651089029, 1.0392638272285288]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[16.56282630256845, 1.2413051009280169], [0.019391060017991051, 5.8123501752416793]], [[0.17245368304543332, 0.2710468565741469], [1.2811242470213575, 257.51382833358588]], [[21.573170161861654, 1.8915158206352107], [7.4849659339785504, 0.23940333800570004]], [[5.9656676192920663, 0.86490592473407391], [21.546627360716109, 8.7322261027452868]], [[0.47805999293821383, 4.0039803776716205], [23.494729119337109, 1.9475679589280601]], [[5.3356000240937744, 5.6255104648808656], [56.405838507561377, 1.0185748017363203]]])+(1.-msk_ref)*numpy.array([[[5.5042261505491021, 4.3264196623264901], [0.142573292596227, 1.0865785947347484]], [[0.50682677426431899, 0.26141018198534571], [2.3852857340668332, 514.38913726517058]], [[7.3305807164366401, 5.2818471227266821], [3.8212385407764504, 0.29001184904094179]], [[14.673035738813164, 0.90430015891800053], [125.91005716683297, 2.4892624602842854]], [[0.6828668189483672, 2.0345325688333404], [1.4436996114091032, 11.693775024413734]], [[1.38537721584155, 12.140562799643099], [18.793410951055698, 1.1341126270173976]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[4.0882829695164, 3.0701446554184626], [1.5238147530514663, 2.6146092450258056]], [[1.8128755266961512, 0.80952904985632546], [2.3981846245676985, 3.2303766116903363]], [[3.3627874706088781, 3.7134051167614968], [2.8071726394785497, 3.0219967526289957]], [[4.4013636862423979, 2.7348014528575622], [1.4931402237522717, 4.3378201703893859]], [[2.3382761056674246, 4.4138114601728908], [3.3433336438262828, 4.8745659133671069]], [[0.39146965396010047, 2.9811907450852435], [4.074837787771318, 1.1099243794924092]]])+(1.-msk_arg1)*numpy.array([[[0.85021069557793594, 0.019820916376238909], [0.4096194233504204, 2.0317063082270868]], [[2.2995353960378098, 2.4923300888492932], [2.1031160700603895, 2.7217523171832569]], [[0.055711294753188999, 3.5576426235413932], [2.3118384735677058, 3.6554549541965891]], [[2.2943402990386459, 3.282371596376962], [0.75377353358259214, 2.8792081872818294]], [[0.72344715150963146, 4.5730288986600014], [2.8087663651892361, 3.6550069421240261]], [[0.52863262267963229, 1.5707974131707565], [3.2644924153405714, 2.6118153949543621]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.4255435195163821, 4.7884212242744253], [2.0784911192388313, 2.8318148196780042]], [[4.3618547817578905, 1.3945440143906869], [0.56735925240139062, 1.1262893063439412]]], [[[1.5287642369821706, 0.73933285785020597], [4.8912342113505023, 1.379281396759183]], [[2.1580297492020688, 4.6624362306039515], [2.4846613287516051, 3.8908085871236304]]], [[[4.1171526466950823, 3.9944342043000165], [1.6531807536948868, 1.1917136727649269]], [[2.1659920502133683, 0.78820583749952922], [2.2910646803039114, 2.466037697262109]]], [[[4.8600441695769838, 0.25016449853860923], [2.2292244201319629, 2.443126871796045]], [[0.64964547794709648, 1.6905430736936231], [1.6801079798034062, 0.10863673278594413]]], [[[2.1703865762921413, 1.0136766922810048], [2.2123828999035906, 3.4537559193002911]], [[4.4130480486088306, 1.5718743064259757], [0.46998317022627445, 4.378115411926764]]], [[[4.2011576702214368, 1.0189480594506573], [3.0578901688586497, 4.2984100207212714]], [[4.3935924467809571, 4.4052671735308868], [3.7376514355217183, 1.2099983606105502]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[153.50603867990378, 603.69668166848066], [9.4521989879328512, 24.428932177649735]], [[9.4349676548866377, 1.6599230540458652], [0.22721407025101537, 1.3647228549214288]]], [[[2.1586707525128745, 0.57839330866735383], [3.6149664422598629, 1.2973377702029474]], [[6.3260320204488494, 40.12889382253119], [18.917420627453311, 80.547299069461189]]], [[[116.61622727823602, 105.33343632830967], [6.4671341511392493, 1.9180379921727455]], [[8.7548104757123326, 0.51268385935753258], [12.247055982033975, 15.297549044334129]]], [[[1052.333300101222, 0.0022458065881670671], [8.956359056393449, 11.506820556361729]], [[0.52516965917634617, 2.1901565159403713], [9.4945028153660935, 6.5802177690823689e-05]]], [[[6.1223287641635338, 1.0322730000326588], [33.277317744823385, 237.63905653111868]], [[143.07917920915006, 4.5361801049319252], [0.025208301732978632, 1336.5855277615044]]], [[[1.754001818440462, 1.0073752552949777], [27.998529600629386, 77.270099493546226]], [[416.28070865701983, 420.80652998657689], [4.320573177678714, 1.2356197678969183]]]])+(1.-msk_ref)*numpy.array([[[[2.8486082719512602, 3.7871003100668958], [1.0146074805235059, 1.0208462542776728]], [[1.8281917683539086, 1.145940965748192], [0.31616372059709819, 1.273319971737096]]], [[[2.6539793420486846, 0.49933608898879361], [52.270706393315066, 2.2287485387140427]], [[5.041520571822411, 25.47830574508615], [11.907467898577821, 40.359183518441164]]], [[[1.0820317212559514, 1.0802091506148981], [5.9800628617435256, 1.8663473603876688]], [[5.9701389482690175, 0.57682962800728788], [20.706195749403218, 27.098002985843632]]], [[[37.617187269946442, 0.041622189478417462], [13.892170913161783, 18.766487706449926]], [[0.72243776935195858, 1.4855256165387076], [4.4544357119090279, 0.0016763944332465592]]], [[[1.7517285269285376, 1.0098757602634285], [37.762095483208419, 289.48311412707886]], [[64.702379308780436, 3.5619822634030984], [0.063307994148811414, 220.75430890051098]]], [[[2.1356652408160537, 1.0099722418075516], [5.7876123886834563, 9.8809412445304812]], [[125.45277148914427, 126.54428472373361], [31.298415728110765, 1.645199496203265]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_expandedData_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[3.9555250031350497, 4.8016386166257314], [2.5017225196638306, 3.4228144930072295]], [[2.5468880076285947, 3.6347697290034096], [0.90653857832999052, 4.4257851071449448]]], [[[0.95402687126642416, 0.43572105428442548], [3.4054135559703989, 0.18407529401184869]], [[3.8359672787196653, 0.3349465518424421], [1.5768441719913358, 2.2171402290220548]]], [[[3.5662597266062872, 3.3723984860899856], [4.0102565743375953, 3.3395755906231472]], [[1.0734792604085122, 3.1091226125143483], [3.6741330734457556, 2.2697201604722199]]], [[[0.6824954322359198, 1.7082403344235348], [4.1104186896930734, 4.8796064541792949]], [[3.739188496903914, 1.074755462923177], [1.6497983827423688, 0.60578941586690704]]], [[[1.1909395077727649, 1.72175177068672], [3.8960615868598896, 2.2873308393773666]], [[2.7359217817130514, 2.2453740602712005], [3.5594560180552435, 3.3627197234593607]]], [[[2.6091587442708528, 1.3666326075461932], [1.4310428976365277, 3.7864563207407835]], [[3.6194761814745995, 2.9179898577244749], [1.5450757083637763, 4.5040819805382135]]]])+(1.-msk_arg1)*numpy.array([[[[1.3003885309673278, 2.0954340108917719], [3.5622351558336294, 3.8533819929401512]], [[4.1190240208310751, 3.3152864071960595], [2.9582579517737035, 0.42914018317515124]]], [[[2.4421248244731113, 0.77120524826257975], [2.065526787146867, 2.0024379746247867]], [[1.9809082790510433, 4.0197167377195635], [1.8135866958618669, 3.4295389224375481]]], [[[4.2290839989358275, 1.931685593882001], [2.9969765088640119, 1.0335120578643882]], [[1.0535511434136606, 0.42062728639514474], [0.50175773075779484, 2.6327484342307645]]], [[[1.7588885023663821, 1.1976756894853495], [1.8463692735927493, 2.6812582109755052]], [[0.27323450355448309, 3.7883976347227089], [2.4444705811386518, 4.7910557378810426]]], [[[1.2577615781837521, 2.7623029276328421], [2.4928431958739776, 4.165499358946958]], [[0.91824657401055354, 1.6175978277113277], [1.3388943344498201, 3.4010088433953762]]], [[[2.846465501124908, 4.303040885675264], [3.7382438929910999, 3.4334110554130244]], [[2.2259688807083693, 1.945445569313029], [3.3054409367669702, 3.1391270471818662]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[2.9506592607612663, 1.9643485137520742], [4.1925459092390671, 4.0177988370188018]], [[4.9277838903582589, 2.0169467330674551], [0.45578770146218772, 4.6931935329940853]], [[0.11282730370137804, 0.71963356014008739], [4.6678537914784437, 3.945838103954828]], [[2.9756002793961889, 2.1431218408020656], [0.95896445020832799, 4.0596364819922233]], [[3.7044080866148685, 1.0557238480768447], [0.89286290664098988, 3.747617249171852]], [[0.37086215073802525, 2.9854169977046361], [3.6785917336694913, 3.7030126952170588]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[72.239809697110672, 180.46023041975411], [5.4144136413649084, 10.083996290268407]], [[38.492960353936674, 183.04739903151724], [3.5280813780550333, 471.1095743912947]]], [[[4.5793969421828793, 2.0035625652257316], [10.904620054082995, 1.1378544611174499]], [[0.049093558040161425, 0.76860589368541909], [11.449885229363026, 30.813408611438572]]], [[[0.00041750622817803414, 0.00063732817074496184], [0.26728838472788746, 0.33328268209259432]], [[5.2273729316641679, 120.32823739657599], [154.98689589796069, 22.545989595084659]]], [[[2.1048080450596962, 6.4413844034877386], [22.947808118297306, 41.24567408323324]], [[0.85498037513652536, 0.9559653382962644], [10.089766375980505, 2.3367651827606299]]], [[[4.7567433074784162, 9.5321889290920314], [1.2352462501579786, 1.1320546705810624]], [[0.73341706365653969, 0.77534220251874264], [110.2190094120629, 84.991995438346137]]], [[[0.075163511318028897, 0.25779338716440564], [4.7835744027335858, 62.890592906387198]], [[111.55037727101137, 44.735587382095567], [7.5589388126983028, 363.76359696509581]]]])+(1.-msk_ref)*numpy.array([[[[4.0839212286339972, 9.6534906821915367], [11.079327517474194, 13.485996792761073]], [[366.43852092711325, 115.79565891822608], [61.20020290981283, 1.816333537252153]]], [[[49.152068309681205, 3.4211852452379508], [4.2594596110742033, 4.0750382996163852]], [[0.21088224387171478, 0.04249348218620836], [16.510729877061234, 200.82937235454307]]], [[[9.8306016120189456e-05, 0.014776180236022174], [0.37304931584385714, 0.7117425299625173]], [[5.0693145266058677, 1.911828673322999], [1.9912119203344147, 37.109466394660004]]], [[[6.8071434501984154, 3.691380167501185], [4.0854015995351274, 7.720085632959659]], [[0.98861641503977482, 0.85321928546536141], [30.720545914057727, 822.80294103923097]]], [[[5.1917370117248129, 37.236913110845187], [1.1447411247115122, 1.2534265601662806]], [[0.90117325374372836, 0.83251021003090586], [5.8640494744565164, 89.401871143544881]]], [[[0.059398883086368585, 0.014005668041623639], [59.660178003038624, 42.745206789863992]], [[18.163085069955223, 12.603836014982177], [75.740058474860689, 60.920997983912073]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[4.6935413929789371, 1.2479327969399656, 4.4746465801566835, 4.2328316451395631], [3.9082773250601481, 2.7835686219041404, 4.1708403065683122, 2.3676831745842692], [3.0550451743472133, 0.028441010498803661, 3.5926122280344486, 0.23339074208968158]], [[2.0515529668351751, 1.560665118445741, 1.5550549595685144, 4.0236324829037864], [2.5865729353827422, 3.4403334584284573, 2.3323800115756139, 2.4962948439840313], [3.8391615843117757, 0.822464815634663, 0.81895807502781626, 1.8371091501425201]]], [[[0.74248853824268268, 2.9388477177939731, 1.0097615338369814, 1.7684348134211829], [4.1047462963492354, 3.3570044031749102, 0.26219501628108627, 4.4964217355981821], [3.3983309623957756, 1.8972257335370384, 3.7838687992171431, 4.0793124811633756]], [[3.2596605040520621, 2.8105623332530918, 1.4507424716578778, 3.1477847546492339], [0.73899772813994768, 0.54081597345735533, 4.0179811829343679, 0.93755083547265616], [0.81605516385927668, 2.0460588925903913, 4.7546492904965074, 0.77835188544169887]]], [[[1.742515006836552, 0.71533295671148645, 0.95104567618443914, 2.8786369339530049], [1.3301842618518136, 0.22136513264629115, 2.2410189455565028, 0.26526425668034248], [4.0735913835072521, 2.0034245198239309, 3.2497968481807291, 2.67537466482402]], [[1.4931906651742219, 3.3936352894000459, 3.8448075963818806, 4.2403656868295183], [1.7548725165251478, 4.6455513174661007, 1.055719528415562, 3.6306513431113672], [2.7272636516713264, 0.1596939180050139, 2.3928495601854021, 4.7656771379741825]]]])+(1.-msk_arg1)*numpy.array([[[[1.9238224003775126, 1.9820260218177657, 4.8165281079526245, 0.33729757176940472], [4.5309694877223237, 0.82806701559876306, 2.3339174513919581, 3.538143450721515], [2.5987248821273763, 3.1759774552235474, 1.7483799189965166, 3.0305724399227709]], [[4.9651270190056298, 3.8318317767389818, 0.097580438268945704, 3.1671331592584218], [0.89463331451284256, 0.65714323547967313, 4.9415133998072047, 1.1993823086786557], [2.935986243127044, 2.5214564669282673, 3.7217028682927227, 0.088995035766809255]]], [[[3.1895394107414794, 2.8708603081541328, 0.77109894110832244, 1.7902392097939339], [2.5842155753227685, 0.67010917495010847, 3.8497643059036268, 3.858009014940575], [2.9637758615631973, 3.8560232352174855, 3.0974340244631127, 1.1317965249721684]], [[3.5495512494765693, 1.5171676945995007, 3.1291560125085724, 1.5124816285926794], [3.2693688594300387, 0.80969909150080543, 0.88271774550953108, 4.1384885872427244], [1.0118699310932757, 1.2532042774645034, 0.9248918801729803, 1.1557221085553953]]], [[[4.6844822718389389, 1.3419628816653995, 0.64339388862236291, 4.673240066273749], [3.781392579592286, 2.3137572465331497, 3.1567794524610799, 0.34480522163326488], [0.49015533161044872, 2.1269093146489371, 2.717891022755258, 2.5439400426201435]], [[2.9944455216305732, 0.99271297410313386, 1.0460124057063085, 3.833566887831712], [4.364397904931022, 3.6100739408052887, 2.3126198815332177, 0.68041988882528126], [3.4561739003164464, 2.8874820764194582, 4.9308730467839341, 0.36457651652807566]]]])
      res=power(arg0,arg1)
      s0=numpy.array([[[[2.1907798196837152, 1.2836964223940179, 0.98203735120377822, 3.5793741886026642], [1.7154888036739024, 4.454346606648369, 3.7257325651003659, 2.5465975728547203], [0.60952167513437383, 4.5445362459094021, 2.4538166581534897, 3.5822147570729745]], [[0.83590296000204689, 2.9214091224033045, 3.263408961740347, 2.8536788490466614], [4.4881424033264183, 0.14413713391470476, 1.4042337678340826, 1.9836545537394605], [3.5248574366522067, 2.2205409380297909, 4.0765975300728732, 0.39678057784051624]]], [[[3.1676178620930822, 0.79271198024590317, 2.3202862252825671, 4.4334190843681665], [3.1640441097704914, 2.52660132356558, 3.1146953877532613, 2.8174396489960016], [0.38331458868218116, 0.57353947641776171, 1.333684397175573, 3.6359834342340713]], [[1.971243210454025, 3.4477733121903551, 1.7652414002815267, 3.3453873168440991], [2.4368915371998332, 4.0624575482928709, 0.15477539686994407, 3.4940111402692335], [1.7463744268956638, 4.0875605379184172, 2.1207435176750402, 3.707308724931325]]], [[[1.7595031833219439, 3.5240400229398978, 1.9822157553909832, 4.5711774030026575], [2.465448148295557, 3.5014690586728001, 3.2952671154127393, 0.51898628388711765], [2.0819881238359579, 2.8395044418785256, 4.9574391058236751, 0.39756965996493188]], [[0.86802405930172288, 4.7988950707493077, 3.411151781919405, 4.3417828760924095], [2.6810408852719396, 0.36308012478806967, 2.5915724777367743, 0.77248099800103087], [3.8758052288038112, 3.3125805482493242, 4.771625339918363, 2.6772517027976237]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[39.683890810926961, 1.3656949386661019, 0.92209487879169072, 220.88805275858522], [8.2423844535268831, 63.963880991555925, 241.23076239001171, 9.1450360359747638], [0.22035969007437092, 1.043998002703153, 25.150784316338921, 1.3468953622149706]], [[0.69230685201709774, 5.3288535943179793, 6.2919812028455802, 67.980247047210284], [48.597943560400878, 0.0012761699709172029, 2.2074188067914191, 5.5279352942055571], [126.05638406664588, 1.9273037220542109, 3.1608824254135142, 0.18301777733114435]]], [[[2.3539018612484446, 0.50526077907711375, 2.3394286709140477, 13.922440967828026], [113.07553978533767, 22.455231625408253, 1.3470108873616418, 105.37463910998726], [0.038439988291727636, 0.34828928135125231, 2.9729281361183708, 193.62074956815152]], [[9.1359354651907196, 32.417986205716112, 2.2806004016020731, 44.755292685589581], [1.9313948666104277, 2.1342374034068992, 0.00055492921222424286, 3.2314254246835565], [1.5761496342890005, 17.827553945675994, 35.672754267023294, 2.7728600680833777]]], [[[2.6766763191127905, 2.4621603647749222, 1.9169204971278755, 79.429384554457684], [3.3211900088787467, 1.3197085740925685, 14.474471017971261, 0.84031291093673566], [19.831257163435509, 8.0916528611441283, 181.73780113440176, 0.08477744350198535]], [[0.80949878648701457, 204.90143354113908, 111.91852620739435, 505.76169853175281], [5.6444019106017755, 0.0090358481293219117, 2.7327941194203391, 0.39170553576540107], [40.2363099235981, 1.2107860761279479, 42.067474895165638, 109.20151387493736]]]])+(1.-msk_ref)*numpy.array([[[[4.5211765850200072, 1.6404959483782251, 0.91639839682805446, 1.5374386359936958], [11.53464860301003, 3.4453761028979404, 21.535794741117346, 27.311518694837243], [0.27621417664589282, 122.51014570486102, 4.8038796367418657, 47.796558039920619]], [[0.41067094033489437, 60.823538288006525, 1.1223396095204494, 27.690358693676696], [3.8314267653807379, 0.28002400413793171, 5.352711443984612, 2.2739217207362916], [40.401741274298381, 7.4744758341598851, 186.78724239819181, 0.92102830895330778]]], [[[39.54639924365199, 0.51330378720520031, 1.913677326061952, 14.381928786582202], [19.621605317663825, 1.8609878977431493, 79.347580198305948, 54.393021854455334], [0.058311121490953098, 0.11722345968663182, 2.439740496877274, 4.3103239701191249]], [[11.122322470264908, 6.5393767030100394, 5.9195480105770519, 6.2117726802521425], [18.395441905176401, 3.1112561437519681, 0.1926351392466561, 177.2316799427536], [1.7579702325446793, 5.8383422218751759, 2.0043162025123418, 4.5464642683106078]]], [[[14.109946636163526, 5.4213630497114504, 1.553048749816675, 1214.7017311049335], [30.332775217079639, 18.166156836229359, 43.138643223278152, 0.7975983743504772], [1.4325302819389831, 9.2046187816829974, 77.559288106316998, 0.095704263686866825]], [[0.65454078420078254, 4.7443613591640945, 3.609284389561862, 278.3192972670368], [74.009355596144118, 0.02579742774467764, 9.0451306980000883, 0.83891243686624251], [108.01428515090421, 31.766640880409064, 2220.3300933956493, 1.4319420411435986]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s0=numpy.array(4.3468538692)
      s1=numpy.array(0.863357756624)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(3.55609513755)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s0=numpy.array([4.4942909841626237, 4.3405218347466414])
      s1=numpy.array(1.67728092587)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([12.436442346389851, 11.73105157700294])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s0=numpy.array(2.4556155036)
      s1=numpy.array([0.7226360676042195, 2.9513466841886795])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([1.9140089009127657, 14.174198302323934])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s0=numpy.array([[4.5761967742695342, 0.98523425410404364, 4.3511052196787174, 1.5298898326838017, 3.6650326067464345], [4.5659214947025459, 1.351707302155899, 1.0635281490646287, 2.4347374406772819, 3.6365811296177828], [1.8920527894335173, 0.97829328536335702, 2.05781721316202, 0.82221041263051575, 2.843958379285275], [2.5701230082184097, 0.074053707430552743, 3.3737877901382882, 2.5352855948083599, 3.5878435104363948]])
      s1=numpy.array(1.63134197966)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[11.953883792552702, 0.97602450053572776, 11.009670266055259, 2.0009851628522277, 8.3215430589889454], [11.910128058313553, 1.6349872773933409, 1.105698572466421, 4.2700758519882509, 8.2164172694339914], [2.8299113772166509, 0.96483222033067551, 3.2454405826019466, 0.72662134933123756, 5.5017902699003143], [4.6641769925337284, 0.014316743643068747, 7.270104871717999, 4.5614826218710638, 8.0375403793288509]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s0=numpy.array(0.319436128251)
      s1=numpy.array([[1.2762326789819458, 2.494537380845002, 4.0186716551342148, 1.1440598889826066, 3.2249171914724539], [2.8328686572261796, 2.9912097876785055, 2.6539041051270273, 3.6800654491798617, 0.30330864898294618], [1.8450805437323452, 3.6066886464826577, 2.2035544783521885, 0.095306074147251452, 0.84871732341220407], [3.6866368046717675, 0.093006940714914596, 2.2697409166246714, 1.5743430485870089, 4.8769711213695537]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.23306588432656386, 0.05803198748449942, 0.010192533860060596, 0.27101020965772182, 0.02521619467545564], [0.039444373452382345, 0.032923702561229513, 0.0483818812812482, 0.015000329707491532, 0.70741683476414463], [0.12177233190101132, 0.016310509215425796, 0.080887836042889363, 0.89694288242745235, 0.37963141468194744], [0.014888259718828976, 0.89929934100810727, 0.075003263657782621, 0.16585571775111738, 0.0038273241275505417]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s0=numpy.array([[[3.9990092711499501, 3.1418148946094955], [4.9357680621094877, 1.0635959628893392]], [[4.5728457961426106, 0.54784808273523677], [3.5251176451410129, 0.25969227533249428]], [[1.2763554397207766, 3.9012918103055041], [1.1457096365052986, 4.3932761118390662]], [[4.581182131645547, 1.7916916332665529], [3.8990352727738258, 2.0664445687953301]], [[0.060065502010894729, 3.8660892717565436], [1.1019578061763378, 2.059516245540808]], [[4.9984481176320132, 3.7875468114641984], [3.0245732381539043, 1.2596538953347387]]])
      s1=numpy.array(3.4718797969)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[123.00036902150124, 53.2293619754643], [255.41378085249119, 1.2386979390813317]], [[195.92432883772173, 0.12378255455756083], [79.381814342946953, 0.009269837932725726]], [[2.3330324203492245, 112.87678336196092], [1.6036138260102279, 170.48430492889341]], [[197.16718242668981, 7.5735520643301237], [112.65027046177003, 12.42850822583714]], [[5.7482175846675609e-05, 109.37986142514603], [1.400850187986129, 12.284433547790313]], [[266.85280532537831, 101.85667318171879], [46.645311292355338, 2.2287442409697924]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s0=numpy.array(4.83402282426)
      s1=numpy.array([[[4.0722208716372945, 3.6133498466078122], [2.984842837964369, 0.24174571241398266]], [[0.11968834684344114, 1.2619196757929159], [3.2762600125358992, 2.5303030144583967]], [[3.5447364349417532, 1.555186124940906], [3.2069008991974202, 3.2528949763497215]], [[1.9180796104596523, 1.4558499067609596], [4.5672804116255978, 3.8565098962379691]], [[2.8828204461866149, 3.3280541101656569], [4.702165024949581, 4.8243300253452635]], [[2.8391386248568034, 4.9759471053431961], [1.2067229135236022, 0.45272046394469068]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[611.8657689017615, 296.92497188361767], [110.29451418931849, 1.4636212064962699]], [[1.2075462585031276, 7.303694658540957], [174.5714499257264, 53.890021630792923]], [[266.49800957645323, 11.593842337458991], [156.49849829659556, 168.26133160384654]], [[20.538019330233052, 9.9140419490353349], [1334.8415163266088, 435.55525776828307]], [[93.915895121582864, 189.41587322006944], [1650.9442939972494, 2001.3866188084519]], [[87.66924206873459, 2541.4635079145555], [6.6953145331013824, 2.0408004935031143]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=power(arg0,arg1)
      s0=numpy.array([[[[0.81449677724632485, 2.7358770763932796, 3.3211337978647881, 3.2443589214399351], [4.8035616983248968, 0.20046423762981008, 3.9309381095018714, 4.3918219013564439], [4.3494623099848297, 0.86146060760791587, 2.9007746074968628, 3.5096167302486445]], [[0.20610998669176192, 4.4612671121805221, 0.62824140106394, 0.089208179024192161], [1.5682836229562038, 1.58139644146464, 1.8922869474656954, 3.1586229145566773], [1.4199733111471782, 0.13562222777997968, 0.019716280549113421, 1.6945750723967716]]], [[[0.88095758695766002, 0.43600010528855188, 1.2696386711911725, 2.0971143243902994], [3.5223014121609784, 0.44955132169188294, 0.75844747940749002, 3.7822892415991407], [3.2334221492190096, 4.0243690550273472, 3.3381255892499344, 1.5075758201340839]], [[1.9891463949284451, 0.027538840478672499, 1.4730625134994335, 4.5314715807044728], [1.582385335867363, 0.29628482589746508, 2.8849452891729523, 1.3290574625455631], [4.9263374397711051, 2.2084604358564301, 1.1970886644155296, 2.2039919804074071]]], [[[2.0847499108291916, 2.0207531210563952, 4.7775370994038591, 4.2708103440842162], [4.5834927849242559, 2.7469049300088599, 2.7359972519528157, 2.9081498334048885], [2.3480336803905848, 1.0285636858794853, 1.2307424745437394, 2.7025320251326606]], [[3.5489974823905586, 4.4362670651832836, 4.4251293482502181, 2.3536311028732335], [1.937909715845201, 1.5579008843322639, 3.4853322224672421, 1.5030891473567816], [0.71777964570281194, 3.9801743536093932, 3.3851288012483138, 4.5179448817460974]]]])
      s1=numpy.array(3.24375470263)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.51398089432008087, 26.171829876035059, 49.082324402606069, 45.496370646043964], [162.48916136138055, 0.0054447624152660322, 84.800783405115695, 121.50155414080945], [117.74118121948092, 0.61648086209335373, 31.643310600480763, 58.706837929073913]], [[0.0059580688850734871, 127.84483216104411, 0.22139716575341156, 0.00039388571136438077], [4.3043689401878833, 4.4222103037930189, 7.9154910731583339, 41.710776827297202], [3.1185991464411793, 0.0015328307075492533, 2.9432729564292941e-06, 5.5337188604589356]]], [[[0.66289924262176481, 0.067699107675356446, 2.1692666911949017, 11.047479926945385], [59.397899290929352, 0.074765484652629902, 0.40785690194605312, 74.833256801880864], [45.000758684170606, 91.514800072028109, 49.901575110297856, 3.7869875216135402]], [[9.3068275150024267, 8.7009438696589347e-06, 3.512919811117698, 134.48665375139188], [4.4311866833303997, 0.019335478040472795, 31.086615662481698, 2.5162028331391908], [176.35122837828075, 13.06599037419546, 1.7923491432036083, 12.980430125919105]]], [[[10.837591967203453, 9.7951269291987977, 159.65090703860912, 110.97386324537597], [139.55950649932237, 26.51557715391721, 26.175559139507691, 31.905026177032052], [15.939470428668217, 1.0956578912756487, 1.9610119632752272, 25.151200661003688]], [[60.87064770758699, 125.53552750651964, 124.51606919673735, 16.063055706449422], [8.5514452965642676, 4.2126168351600546, 57.39936838324553, 3.7505510379855744], [0.34109125038437094, 88.2948246116609, 52.217014620892918, 133.18880307750993]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s0=numpy.array(1.23824775417)
      s1=numpy.array([[[[4.5762604372347822, 2.2328232821491829, 3.5048022471333424, 0.54861407628447678], [4.4320217186088691, 3.8019472167907553, 0.15056954508522782, 0.7885320740021583], [0.81243717562704243, 2.0801573437814365, 4.2576342015144926, 2.9473054490224939]], [[0.22906021156878853, 0.035858690998116005, 4.679642736823789, 3.1203777778766937], [1.0564845691998839, 4.0081922606050089, 4.6348248746143401, 2.8814466353451618], [0.83596957048946041, 2.62603772041978, 2.3712529337381887, 0.51242646903535538]]], [[[2.3816773659218962, 0.59864299904541562, 4.0269517622027475, 2.3008702619665207], [2.6092487367535728, 3.5624502219943457, 1.8176275736667125, 2.2965069880043005], [4.8128525542655982, 0.67157465957412588, 2.0547463090591793, 4.8045829215315097]], [[4.8690924701819478, 0.78062170427911215, 1.682688920196332, 2.5830566264598347], [4.5040649868115619, 4.3233804704942997, 4.9187606474246408, 1.4166734023331089], [4.6711769715149387, 2.2347816334410369, 3.172903867569429, 4.1456398113659958]]], [[[3.2657909714616409, 3.3581604054038396, 3.8993112067498372, 4.7747489350314005], [3.4331571861558978, 2.0764657909371396, 1.9901049096146928, 2.0410622277706572], [2.5763479715917774, 1.2151832419988267, 3.4524377223088334, 2.8932658470435055]], [[0.99266488814588039, 0.61855237453159873, 0.38456872070746895, 2.9697396338057804], [1.1142126445629446, 4.0279054899368898, 2.8317651171603844, 0.98137105135052993], [0.87875777890766815, 4.6422752097594113, 3.55831251883474, 1.0380962748594598]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[2.6589582278076862, 1.6114723422336259, 2.1148136563133337, 1.1243862544616288], [2.5782502746392515, 2.2534576596643441, 1.032699556306516, 1.1835366990916567], [1.1895982125593259, 1.5597475227755895, 2.4839371029581017, 1.8772935793160637]], [[1.0501673620409171, 1.0076923398791484, 2.7183550015010005, 1.9480253916829979], [1.2532847159364982, 2.3549977661125627, 2.6924443553571584, 1.8510578841099639], [1.1955955415252013, 1.7527344561694378, 1.6598550831087857, 1.1157246922626225]]], [[[1.6635568187104866, 1.1364716055191844, 2.3644575581652996, 1.6350766987429719], [1.746457332521864, 2.141027637494807, 1.4746520539218606, 1.6335528314522814], [2.7968491891798055, 1.1543225986519152, 1.5513006294082603, 2.7919109675464493]], [[2.8306654093010759, 1.1815377097242015, 1.4327362009933415, 1.7367093976975099], [2.6182507211536517, 2.5190822322845356, 2.8608699740212913, 1.3535614786092394], [2.7134416412878504, 1.6121468754253825, 1.9700145377914826, 2.4251950313818207]]], [[[2.0095094622355729, 2.0495694369253155, 2.3008352150079987, 2.7741679218333499], [2.0826817127732902, 1.558517562470884, 1.530018770546552, 1.546770863408133], [1.7342213979820322, 1.2965168883589013, 2.0912804761682469, 1.8557390720186824]], [[1.2363083293531518, 1.1413171146515795, 1.0856526089930094, 1.8863151659943427], [1.2688414120501603, 2.3649395050574489, 1.8315095172837457, 1.2333281436742765], [1.2065778699883625, 2.6967344542893481, 2.1391353435249418, 1.2483695614188617]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s0=numpy.array([3.1959294935422, 3.50492440277328])
      s1=numpy.array([0.77302189994594395, 1.776150006487464])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([2.4550745157017904, 9.2775007238945992])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 2))
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s0=numpy.array([[4.083441367693772, 0.57548710004773529], [2.1507719608532248, 3.9998636805689656]])
      s1=numpy.array([2.6415954614047745, 4.5294651489803881])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[41.122959745611162, 0.23233311310796134], [32.097749969383308, 533.26454267806218]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_Symbol_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 2))
      res=power(arg0,arg1)
      s0=numpy.array([3.3148590766307029, 3.0646104653057011])
      s1=numpy.array([[0.1244916505865756, 3.0953689617927838], [1.9094715431770841, 4.8558270186149866]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[1.1608966438524775, 40.834893173799941], [8.4863268990280183, 230.01397712913831]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 4, 5))
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s0=numpy.array([[[2.6470147676914118, 0.12880661916417463, 4.3709797230291567, 3.5137117178572113, 0.45662505798566028], [0.21639049966532123, 3.7099768301332943, 4.755180074635617, 1.1688886658879127, 4.849086780151028], [0.35973031978192288, 1.6029685419952839, 0.22522540526788157, 2.1157614832579008, 4.511670313613414], [2.1492659495999629, 2.129209255009247, 1.1510744576198659, 3.1125607490749316, 1.3191287242679999]], [[0.24951764989230543, 2.8382231922816814, 3.3239447099247177, 2.9665821948402473, 3.8703919820086754], [4.2096681147701709, 1.6692823957690046, 0.34034818515175147, 0.61951302862791191, 4.3598968313749582], [2.0090462938333391, 3.1793154659422376, 0.32780176806447209, 3.9892855855082976, 2.3950112690620928], [2.5281427302529145, 4.9776802373065889, 0.6101314287153059, 0.79161271960411006, 1.8151652997251331]]])
      s1=numpy.array([4.5717652489549403, 3.2847284716811398])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[85.653054441460952, 8.5279826436545061e-05, 848.35155374091232, 312.6904470160045, 0.027770714373400021], [0.00091382925440024144, 400.89575138824983, 1246.9481975984456, 2.0409996049744068, 1363.5666202273217], [0.0093332151846842928, 8.6470703294994209, 0.0010972829263169194, 30.757949481234476, 980.56590239438151], [33.048564355942581, 31.661918369854259, 1.902613081826404, 179.64659434644565, 3.5475190559226806]], [[0.010462642362314358, 30.770523416893766, 51.700061561269749, 35.582298312865021, 85.23450100778409], [112.32714284706249, 5.3820783243975265, 0.029006517684525425, 0.20746372598323948, 126.03922457633143], [9.8909884852390118, 44.671398900000199, 0.025639799130299696, 94.140680788213686, 17.616610238324323], [21.04233101079917, 194.78002355622775, 0.19732136982422363, 0.46413250605783901, 7.0871128828263865]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_Symbol_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 4, 5))
      res=power(arg0,arg1)
      s0=numpy.array([2.7764847633617831, 1.3426236589201561])
      s1=numpy.array([[[2.7214580056299917, 4.6246709600978244, 3.8981213157813932, 3.7556107736280806, 1.3596395006822581], [0.58027522572406887, 4.1744542006032512, 3.6356273058890691, 2.0751144916020334, 3.379091572309036], [0.73659360932636431, 0.32154986756323745, 4.3681392223423678, 0.88264773320759149, 2.4458923897848557], [4.4038752947659381, 0.16642632985757075, 2.1749123990492261, 2.0700312683387661, 2.6960505990703858]], [[4.9536714617419797, 4.9698225894015282, 4.9762646906200763, 4.4867982240497204, 3.8492873256797568], [1.8854929239271319, 3.0022643137625673, 3.4182532466869855, 3.766352994202764, 2.7798291826558987], [1.325597184622177, 0.59897196325655411, 0.34749846963020314, 2.0733805153988349, 4.7398388528908848], [4.8303060415612871, 2.4980513614368611, 2.8891476439787089, 0.063260839542345473, 1.3884984412637533]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[16.104743778718923, 112.46562718047498, 53.554807303856528, 46.301568801761, 4.0086092177494068], [1.8086283419599674, 71.015115078427826, 40.962231038318329, 8.3234526370730801, 31.521814724970813], [2.1216600540977182, 1.3886917429094046, 86.546436531087494, 2.4629189280505956, 12.154630637472529], [89.763126495339222, 1.1852481719825116, 9.2164445592072006, 8.2803582510617311, 15.692269368070166]], [[4.3037153931447429, 4.3242435834837227, 4.3324588274667546, 3.750637634949777, 3.1083710041781463], [1.742837593867087, 2.4218799736234202, 2.7376642644972971, 3.0333396717406598, 2.2682508667459369], [1.4778006534862875, 1.193001626463476, 1.1078065330393001, 1.8420352795860313, 4.0409431685873809], [4.1500985333331544, 2.0875476760424725, 2.3424959690974774, 1.0188130428681177, 1.5054430819082973]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=power(arg0,arg1)
      s0=numpy.array([[[[3.3271765211904825, 1.458948729071486], [0.15244711207496031, 3.0328133253375666]], [[2.3416354004280562, 2.9629673258141622], [3.0512677297542323, 2.2856861770874355]], [[3.3274986575085213, 0.38611282727515184], [2.9992562709805939, 1.8265235048438466]], [[1.4331897034212908, 4.8983265838988386], [3.8078148758094423, 0.55394969447095976]], [[0.58471445610156902, 3.1639677092914167], [1.8483184685101941, 4.4657987646995236]], [[3.432019079531865, 2.0668296357452043], [0.16174747023138619, 0.25838144683878972]]], [[[3.4338679510143013, 1.9655644311481677], [1.662872261291531, 0.49350543878943665]], [[1.1220219785276022, 2.4650623836412993], [4.7313626554269392, 3.8472272892584773]], [[1.3062116546994071, 4.9925438855881978], [0.79557607564767574, 1.4304108758821652]], [[1.6697128394940584, 2.0222910817092932], [2.5849815712723223, 0.44339475432183284]], [[1.3521644380129991, 0.54597260776656065], [2.2827116070786211, 4.7558699459027549]], [[4.7551711030155799, 3.8749392964913141], [0.76229935279876504, 1.6632266597967003]]]])
      s1=numpy.array([3.0284201972055174, 1.5811979608379474])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[38.112288427252885, 3.1389336809072055], [0.0033584716870383876, 28.789305481966053]], [[13.154054468356025, 26.827929561316484], [29.323106247832996, 12.225119927448835]], [[38.123464459682054, 0.056026955855153139], [27.83540512819253, 6.1988515743408357]], [[2.9740839241917656, 122.95735282993142], [57.349600905500147, 0.16715537939826991]], [[0.19688289708373899, 32.727499200340382], [6.4255788669511693, 92.93251911891285]], [[41.866775008437116, 9.013118098785597], [0.0040181625017241098, 0.016598938977517338]]], [[[7.0336365077189313, 2.9111293032155681], [2.2347123270016813, 0.32736620671111027]], [[1.1996707080418467, 4.1644485353537597], [11.675805792201473, 8.4184973342703042]], [[1.5255990012241905, 12.711158971140692], [0.69655935571153338, 1.7612228265797396]], [[2.2492655893860785, 3.0450847338614051], [4.4892819368256003, 0.27637957150202574]], [[1.611326663575396, 0.38407416089871066], [3.6879248673521285, 11.771576990137271]], [[11.76884202416983, 8.5145806929665486], [0.65105422513976241, 2.2354654532058253]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank1_Symbol_rank4(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2, 6, 2, 2))
      res=power(arg0,arg1)
      s0=numpy.array([2.2481823301264514, 0.19581723055774236])
      s1=numpy.array([[[[4.652228060799354, 4.4551596250180552], [2.3793539277769047, 4.6269228645433653]], [[2.6888899727997235, 2.263231574825801], [0.76136875518711278, 4.7542566901806014]], [[4.05580459502454, 3.5779848543118082], [0.96232207351552534, 0.098826376577599212]], [[2.3760975124308388, 2.3908054283348306], [2.6891239202117365, 0.49325697470721341]], [[2.3309816239569452, 1.8392287981512727], [3.2593009651080957, 3.4476055124509948]], [[0.66947645108964948, 0.43479219565721933], [2.9552939671769818, 2.7031279159322428]]], [[[1.0077814223303401, 1.7588993333095329], [3.5350855327497519, 2.4395075280533818]], [[0.61730855425730058, 4.9934600696579716], [0.17877386672144635, 4.1533674443951032]], [[4.3719862410264465, 3.402155519131969], [1.1848762400021058, 2.9601160891650515]], [[3.2702427056555794, 3.850170982252032], [4.7432137926906295, 0.34329120951831416]], [[1.1989224567512429, 4.1950054714115508], [4.0642024198675717, 3.2367068542511128]], [[1.6339345628465674, 1.6008046223719521], [4.8610742338583801, 3.5742119167093622]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[43.331180103952015, 36.937340909149817], [6.8727687773800872, 42.451921362873023]], [[8.8315366061208618, 6.2557045305458487], [1.8529919564006867, 47.064928774491804]], [[26.727598358946629, 18.148798797447828], [2.1805961831398295, 1.0833536111121214]], [[6.8546616630965875, 6.9368248981430929], [8.833210570144832, 1.4912256188775932]], [[6.608651210233849, 4.4370945404271591], [14.019252104122126, 16.329629176696308]], [[1.7200573373046693, 1.4222423402510782], [10.958865539552262, 8.9339934702279518]]], [[[0.19334836185456916, 0.056811600869993054], [0.0031378514226116233, 0.018726859382710927]], [[0.36547149131752826, 0.00029099516026804179], [0.7471390707238349, 0.0011449765685098354]], [[0.0008016446451673744, 0.0038973251928933628], [0.14485415660036222, 0.0080130244435836651]], [[0.0048325970500232409, 0.0018771755963540538], [0.00043762028114764699, 0.57134527525857037]], [[0.14157420975355398, 0.0010698198897885658], [0.0013241547129468952, 0.0051042150920746834]], [[0.069651538930864496, 0.073517656355957861], [0.00036110597620980046, 0.0029439136174118099]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s0=numpy.array([[3.851513546439663, 1.7845191491159491, 2.659280371607653, 4.9385017137279918, 0.2825571473305169], [3.7293544322879475, 0.21472500210657586, 0.68628824939009747, 2.841615648121206, 1.3861912991185013], [4.2522910674150491, 4.2002948217441016, 2.0272506486067234, 2.7969071408912334, 2.1253916796424797], [2.4540583995372427, 3.4905165350227505, 3.6228000631720541, 3.3441299368066417, 0.51568500293129849]])
      s1=numpy.array([[0.52199196403445014, 2.9482365381878513, 0.6043103911010711, 0.048315485601639854, 4.516902552437946], [3.0524107541438066, 0.36945743740711928, 2.6532081242195833, 0.47525122196762914, 0.80878344194274043], [2.0382652373662196, 1.8508421559890993, 4.6809790342383595, 2.1860874815180309, 1.9732142405435495], [2.130007199426744, 3.4358232616728777, 2.5123033130987187, 3.780336408829819, 0.93594572844157331]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[2.0215984698011376, 5.5149811084074187, 1.8058820295439051, 1.0802179491064559, 0.0033166575584681959], [55.572601954809016, 0.56644671600203877, 0.36831370951237397, 1.6426970605632099, 1.3022795788037864], [19.111746487086631, 14.242705107083248, 27.329225852662763, 9.4727764510189996, 4.4269768460599153], [6.7679576122083605, 73.328543459904608, 25.379847866337158, 95.93269754432491, 0.53803125198947099]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2))
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s0=numpy.array([[[4.0242799906597675, 3.5569320351835603], [3.1606032282531427, 3.6774186139792282], [2.6364080348962844, 0.19386377381628056], [2.4101015469250693, 2.0183080077921955], [1.0954460392063756, 4.0655252292426294]], [[1.829823212623235, 2.5682889156996236], [1.3283799489257182, 4.7509842315932689], [0.064248894774213292, 2.6467992399324256], [0.88501344835612206, 4.2492545413788401], [0.64200054053842714, 1.1218974993237818]], [[1.8306941376730308, 1.5788762662636455], [0.026416594386278101, 4.9597032745201437], [1.4253945140410695, 4.9918653049437962], [2.842810129878508, 3.239078112468043], [4.990995803409195, 0.50510820264429934]], [[0.13872161894441745, 1.3985133043641447], [0.50211996677867676, 2.9646363344586719], [3.358172693085665, 2.111653067878517], [4.7645104945368582, 0.82413354847607811], [3.344308932667456, 4.0606016100070184]]])
      s1=numpy.array([[3.6621378883711313, 1.4685609530826345, 2.6260393161758402, 2.6151245318263054, 3.8833773664940896], [1.1463349584069034, 2.0747124179197716, 2.4817735885409715, 3.6985220597706423, 1.0938711014037417], [4.6151178343955195, 2.4853458803109141, 2.6242135515057221, 2.0511835669036298, 0.40795984904641597], [3.6978683586997043, 3.1460980993939995, 1.7784400377163951, 2.9114165062875941, 1.2981397976790578]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[163.85191768735007, 104.25927794761098], [5.4192935609803818, 6.7691551749101144], [12.752511091797727, 0.013456689236233031], [9.9786087211246386, 6.2745147546617162], [1.4247765292345183, 231.96894798477652]], [[1.9989814414281439, 2.9484182940203039], [1.8024296844152554, 25.35895285870679], [0.0010999994905796547, 11.196900431057074], [0.63649196336061042, 210.77959033103821], [0.61584090101989952, 1.1340764707687629]], [[16.293066115197846, 8.2300000406360656], [0.00011962393257434639, 53.511601103383789], [2.5348865719220268, 67.981517576682947], [8.5255055661169656, 11.142125140270434], [1.9267809625184815, 0.75682026644393707]], [[0.00067260794143508124, 3.4566540732890658], [0.11447509626265509, 30.539956171417838], [8.6226882598897365, 3.7785204022972114], [94.187579213040408, 0.56942169184333224], [4.7931717338356492, 6.1664525779815262]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 2))
      res=power(arg0,arg1)
      s0=numpy.array([[1.9091977402537306, 4.4353949115445372, 2.8908664434932834, 0.46105237267213878, 1.2404991973307902], [0.089061848426993648, 4.5407788026100784, 1.3276215314212247, 0.70207899441707711, 4.1162124535849944], [0.35069611924850835, 3.2270418498978062, 2.4583342370687813, 3.5955518897476701, 2.1718899476387903], [4.7780122854431006, 1.9330364778081626, 1.3989826017409703, 4.9900372857005353, 3.3335319011330506]])
      s1=numpy.array([[[2.2417697121015592, 2.7297255035035524], [3.7538013763733651, 3.2335959951597038], [4.7101002993550232, 2.5070059062189078], [2.6708443641457529, 4.1114411250581266], [4.7572833487120363, 2.6166970827697686]], [[0.37117311104895057, 1.6283685376241233], [1.6125211955654908, 3.5498084461213071], [3.4258849058641814, 1.1328903460946333], [3.2538687383963172, 1.2851195982310104], [3.241615124385604, 1.4672573840742986]], [[1.9454762415771993, 1.1102199750796913], [4.1412048178607739, 1.2195739886773511], [3.8801242050112599, 2.4632417976175982], [1.3628621571678596, 4.7303131797938445], [0.19521809111741839, 0.57296620329695924]], [[3.8228408060506567, 3.2962772558661042], [1.2081433852372185, 3.0344144059893847], [0.79500548272749649, 3.9814357641863025], [2.1396414529491321, 3.4072250791799736], [2.5153589308891302, 3.6110993213728864]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[4.26189815544552, 5.8431346751659508], [268.19787670707467, 123.57131682051535], [148.4185732847983, 14.315284680318257], [0.12645304176389807, 0.041450425795837208], [2.787821813544495, 1.7575741483664531]], [[0.4075245742438911, 0.019485475877513852], [11.471933261784518, 215.12300402482893], [2.6402092167992901, 1.3785726934946056], [0.31634439279104498, 0.63472793391748716], [98.166951595529085, 7.9730852657334728]], [[0.13021889164450065, 0.31244477172790147], [127.957000457278, 4.173734820154551], [32.789540173826722, 9.1673342326003517], [5.7204718256246823, 425.54467142998811], [1.1634744090733022, 1.5595405955438493]], [[395.05189640977375, 173.37533017365573], [2.2172722540466356, 7.3887492785970323], [1.3059350818745521, 3.8066448463525391], [31.166770931303585, 239.10945167447053], [20.667730735073537, 77.31442004167701]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 4, 5))
      arg1=Symbol(shape=(4, 5))
      res=power(arg0,arg1)
      s0=numpy.array([[[[2.8222688826038569, 3.2871350643262622, 1.7077952930069646, 1.9992644189432025, 2.2775152657900049], [0.58078299241665954, 2.9817063823955094, 4.5611160152513364, 3.4255285299668397, 4.873813265485814], [3.3225644355733106, 3.3357799829297958, 1.9485871577120686, 2.8570392883699789, 3.8081118996720225], [3.6845264639714101, 2.0645542905712087, 0.36978554173068562, 1.1792461083866603, 2.0608022924730136]], [[0.67618369136712297, 0.95737237100454364, 4.2479250117620326, 0.51077274812755358, 4.740551567497806], [3.1471387451112287, 1.3684944698652557, 2.2706154874460989, 2.1734931352650118, 4.280249786030029], [2.5802452543825143, 0.41840878198565318, 1.4560914429457255, 1.5291421189717613, 2.5042180802916207], [4.5111182162681249, 4.9878344966488388, 4.9169052644371396, 3.2176207675445712, 3.4116911951270565]], [[4.5681141806561749, 3.8225646689003638, 4.8797005606338955, 0.053380235382507406, 0.39980965246029704], [3.5665550348239625, 1.6973472969112389, 0.92718361931911153, 4.7978607388376444, 2.241624425993082], [4.2467145427654165, 3.1163391451689004, 3.76442201970757, 1.3057589911283292, 3.1872020755877224], [2.7498683624252922, 1.5025916525213279, 3.4930856156913075, 0.51645379916135115, 4.4318472340112756]], [[4.2934213261177661, 0.90283889635883419, 2.200213797352605, 3.8477742386159699, 4.474406750386696], [1.6066911357209142, 4.8916428481069065, 2.1207501844240091, 2.2477327622226517, 4.0663860120084045], [1.0057299811576361, 4.2020807130474829, 3.7027132087893677, 4.1160865051489859, 0.15301837633299453], [0.66396930824131528, 4.1155260739649826, 1.1344918395124903, 1.5146930107030021, 4.9072682875403029]], [[3.4412249317959493, 4.4756626642248181, 2.3704154105566171, 1.6694436974594242, 3.395948331588269], [3.4110007339755746, 4.661862504099533, 0.9358059275457099, 4.7188443045342678, 1.4907883456424782], [1.6754187241891649, 1.1645670598697231, 2.5140372154600392, 3.1571011807975697, 2.7967501552654146], [0.47858116679022583, 2.7649334180221197, 0.23728856918040239, 4.7077160995150296, 1.3146459333843858]]], [[[0.29863672364329874, 4.7400694038589029, 3.56199771182993, 0.25306256523558951, 1.8780388354742528], [1.2841579334701489, 1.0932030540894664, 1.7822377895716224, 1.5135823964909487, 3.6957450020357352], [1.5851650901842673, 0.093841238254039713, 0.70943124911474764, 3.7143048657946971, 2.5463395823270312], [0.39940128821329601, 4.9775045815294963, 0.78919218152888126, 1.9248640880195618, 2.7960071757488332]], [[4.4060667970051535, 4.0711545546769292, 0.94215642378888853, 0.57152392535674756, 2.3813167177197663], [0.93255703473734486, 3.5440859666200377, 4.4435253743164056, 2.0261943018863957, 2.0574341221680399], [4.7523647359184737, 4.3707131574240838, 4.4326857687394039, 3.8953920701142803, 0.89112426157083524], [2.5326666570678689, 3.6157907776940759, 2.2052643527503148, 4.745759734136044, 0.55017377561605618]], [[0.33583908878249713, 4.9622656693080032, 0.87414822344717946, 4.5759101377058595, 2.0300225165500758], [4.4157602273270538, 1.7403065403379745, 4.6517675481682268, 4.6851644609954208, 4.0897685782394859], [2.6352177136039554, 0.67043782722053158, 2.7078502686444752, 0.47129107326049208, 2.7153613704527406], [3.4895687200296712, 0.42004946048183084, 4.7776309152166796, 3.8992077036801605, 4.8430758621650218]], [[0.30926893086833684, 1.4915918132793859, 0.77467357760587086, 2.5289596351101062, 2.1787416749469792], [3.435029963908975, 0.97322387569204605, 2.4958286236293352, 4.0723333667853971, 1.1013616089402534], [3.6588530247354121, 3.9600151093731539, 1.8688750661842546, 0.65207430331595551, 2.8165190866144814], [2.4520695231106413, 3.0994194630780671, 0.70474215713209321, 0.75761874981044441, 2.6119623079369259]], [[0.2870834556630914, 3.4922217551754837, 3.9209719315089901, 0.094247902045567697, 0.96583958045665264], [3.7043038553002794, 3.3482597065460538, 2.3447848219999043, 2.0999271335392948, 2.8163806500352493], [3.6534544503500408, 3.9968494487482036, 4.6091698747817551, 3.6124895004964426, 2.9290955179361289], [1.7647268684668953, 0.33063616413608221, 2.8884703304341843, 0.58021963754660222, 4.835534609251722]]], [[[1.3162306716761423, 4.8773154978943856, 2.554042067089513, 3.0912435952826138, 4.0320253045362149], [4.1032053038450593, 1.0004540806958488, 2.4913049639943483, 1.9360500150700615, 3.0407681249976535], [4.7778286993249601, 4.2963459076969643, 3.708326023976221, 1.3988070555512064, 1.3731666173628199], [3.4438916153761707, 4.7240405233417171, 4.8167969959432408, 4.5374527660879469, 3.7371184519783451]], [[0.40266971786477918, 2.1954938661769812, 1.9437807941776069, 2.0579745036239312, 1.2607262839148545], [3.3457707695955894, 2.5065023543463636, 3.1117461101433936, 3.3088826314821032, 3.2921203125179903], [0.071210806801167495, 3.8355723996067099, 0.54690196237696753, 4.0191320095552987, 2.0353454044363906], [4.5765337865198674, 4.3835971602406332, 4.5120751073267886, 4.1399655394996158, 2.7217937655833624]], [[4.5099233521523949, 0.14089356722576504, 4.1455956173419946, 2.5960748132494, 4.6872095599609187], [1.5929355049235607, 1.0503614819344687, 0.50829741668544348, 0.86939575190889384, 2.6226514397174747], [1.3408718042810581, 1.0144429650874836, 4.5380563063772517, 3.0004033796361531, 0.37494929493519186], [2.1205793585315442, 0.74899933218302295, 1.155862142758282, 0.76051429167307771, 0.70817892964157925]], [[3.5877124441351822, 2.7858654126642777, 2.2995834089487097, 3.6034091927733845, 1.2979562187015781], [4.4582524156511401, 4.1078397909165174, 3.7941711647822851, 4.9688276910151421, 1.1657009960385745], [4.4979726391023558, 4.5495336531948869, 3.1114763069223002, 0.95798931852293934, 2.4669771064210524], [0.30924595578955277, 3.7290860732647646, 1.1019505105904295, 2.3550663095943336, 4.9717955492206451]], [[4.3701326493894301, 3.4390498541325956, 2.9467758952314353, 1.1057949963650262, 1.470040594926088], [2.1576135879979996, 0.19741818603079803, 2.3082790619541487, 1.6529549084892874, 4.6250400352946262], [3.3813185131826664, 0.51081065332119868, 2.6363321404898712, 2.0691823627282826, 2.677073520816625], [3.5938510933456369, 3.5827716848730384, 2.011695512734653, 1.5524309429111045, 0.47169432365278857]]], [[[2.9051799238232752, 2.5391963727363867, 1.8567286976899569, 2.674115584461314, 0.58394618417328403], [4.3275192243831935, 2.6944704975523899, 0.30136130082242801, 3.586794453742022, 3.7309758575781413], [0.71582826497216157, 0.14404105014681493, 0.97993908088418469, 2.9961703283547938, 4.2338526544012538], [2.0702657937904889, 1.2969813578278235, 4.0615850270209464, 2.2040635824609898, 4.2132584466889753]], [[0.70544000983951549, 2.4898060317258599, 3.7928807116247345, 3.4508947057455983, 4.1646389638428527], [0.20473675872897179, 0.96307692399391975, 0.25200093551028868, 0.22619709729099935, 2.5870931867293256], [2.8129287131906811, 0.80044259125696626, 1.0247848024385007, 2.3630744390569891, 2.6407706947367644], [4.0902846569509936, 1.6984929095939945, 2.9336365456712024, 0.28150192146703756, 3.8998722547911258]], [[1.4879074391224798, 0.990474295724316, 0.79282808259524062, 2.7957946885730602, 3.6638330761815343], [0.6397082984016238, 4.9072421776113933, 4.6908150327325249, 0.93754371707861461, 4.0873838504343949], [4.0986156027911118, 4.1715684671408058, 4.6100402505603286, 1.9530286648498756, 3.8645510854495617], [3.2684777103466316, 3.7776889805284597, 2.5949229409219732, 0.44268278285848361, 2.3693532466459595]], [[2.9280138455311771, 1.6481532262504928, 0.16734044619935623, 3.3565167148221948, 4.0550016615118762], [3.6749384851004669, 0.045070285161957893, 0.28484281341110068, 0.32301190119832129, 1.9325393469289947], [4.2628635311770733, 0.07794818688650082, 0.80156457391461755, 0.24009374244433887, 2.8405259433640659], [3.6325429719202509, 1.2019773695604123, 0.1270132444245167, 1.8152718571658362, 2.6579315208934857]], [[4.7017059775134031, 1.9776250868664715, 0.95522965441679331, 4.1122948883872521, 0.58949170237525617], [4.5707521239919169, 1.0770442208267228, 1.5598515647705367, 4.2787018420083935, 0.54165624331712725], [0.028998955243803547, 4.1716087505252499, 2.7296215923628515, 0.80533937453912041, 3.9676867750118716], [3.2435055987624333, 1.6280837288150234, 3.8178514200556184, 2.9854406823076198, 2.6350781873794049]]]])
      s1=numpy.array([[4.3702179193111661, 3.5628895859905425, 4.6641847220157509, 3.7131983012609773, 2.4192349735538401], [4.9601610234069797, 4.6337145673437519, 0.72873150528686281, 1.12011190567139, 4.691653655329806], [4.9126622186195892, 2.6101363102132846, 4.7551946899812725, 2.43071232438494, 2.1223962606524833], [1.111424023917005, 0.44045797049540353, 2.8640914457004434, 3.6201763388665205, 4.0175711789432365]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[93.156496530057098, 181.38664749787122, 10.370391491189402, 20.647548771324999, 36.490831882746271], [0.093044193022923161, 118.44449988568603, 759.07452106592973, 217.20813725897986, 1014.2311160302894], [190.08701464289371, 193.41343823411847, 18.456052579172272, 98.277296719162607, 345.00702894663743], [298.68359358599071, 23.76071864452727, 0.012937400639528791, 2.0555453274003677, 23.572583968463864]], [[0.24805001675160937, 0.85623563765610355, 173.03132244561806, 0.091295122180673441, 255.80183835188353], [59.431862158958147, 3.0578693961101684, 18.573780483794351, 15.894896311767646, 177.76846851608866], [29.288872062166234, 0.044854435998494473, 3.8143728313126917, 4.5411654670165875, 26.328422588743745], [214.35933029993615, 306.60802651874747, 291.35445014139748, 64.311798385993797, 79.234121304898736]], [[1194.3593277008401, 520.24816138785161, 1624.7738422248769, 1.1594913120156886e-06, 0.013898570289292181], [376.52257636433393, 11.794867039528718, 0.702837611377089, 1501.5229989069394, 43.160840201375585], [849.87593295204033, 200.65507008513828, 484.3543887651382, 3.4706158902662785, 222.84113573214921], [111.95200419949506, 6.68065368344012, 341.68679333084543, 0.045869689172115258, 1037.0289032811422]], [[223.73035682503553, 0.68418271377338624, 18.691322233719152, 148.93584308229833, 260.80133509614336], [5.8165816532981118, 363.1467579706254, 16.30498136129636, 20.234744849609331, 182.85731938616209], [1.0214424868290417, 206.56032771073544, 129.13018643693255, 191.29462937722641, 0.00093927179440876434], [0.21857489238305886, 191.19793337922647, 1.5976747716658128, 4.6728417985260267, 367.47278412488276]], [[19.88085268534104, 37.54713929230914, 8.0684190706239196, 3.455042761681681, 19.25393936104167], [19.461052239740749, 41.438347540011264, 0.85170983000328826, 42.674335452319959, 2.6274473698337109], [3.48503450491302, 1.4456643675604288, 9.3023617555508622, 16.139727678053944, 12.038161950150918], [0.16816569424573299, 11.709518020293507, 0.030806887906240882, 42.431279003523343, 1.9383241635509971]]], [[[0.0024924463488708213, 2249.059434004927, 545.11642855416403, 0.0010962618977762653, 22.783410415645747], [3.4575219697234685, 1.5558308541324322, 17.572357301342713, 7.8137422812303976, 654.47539614180778], [9.8265650530634598, 7.9966378990017188e-06, 0.18217599598662126, 670.94110248568006, 103.13600734469843], [0.010542090632391914, 2866.0918929359354, 0.30903666144981007, 25.743651182271776, 164.02187800886239]], [[964.60355244420828, 668.73996572075669, 0.75874078752939067, 0.074845696247876389, 55.726679683903626], [0.72357652895594016, 351.75950186343755, 1003.1944358301756, 26.367723312739678, 28.304987303180667], [1369.6406499661819, 929.25851368351186, 991.90492192931288, 545.06159931409627, 0.58617625568824083], [74.142189980897797, 385.9710479626865, 39.038997001283548, 1360.8422764950367, 0.062740870464113058]], [[0.45151991188532065, 3.2133937316904255, 0.90663234084596811, 3.0290800692122555, 1.6752728913880937], [2.9514526271837038, 1.4974473675922904, 3.0655914310942678, 3.0816145962834867, 2.7910276512422398], [2.0261063716079892, 0.74724320682027212, 2.0666513195163141, 0.57798365498214954, 2.0708272237098551], [2.4861932228363277, 0.53148028131871894, 3.1258174960499612, 2.695647494670407, 3.156962737186864]], [[0.26860841483097903, 1.5649750654520849, 0.75127784945570653, 2.8270924891540092, 2.3923696083732557], [3.9838425608121852, 0.97005635749473496, 2.7856399555289251, 4.8205075386535094, 1.1142078608135517], [4.2757214207307568, 4.6718334277663018, 2.0146531798492897, 0.619429600695939, 3.1895436345341346], [2.7309912203690874, 3.5504955745562858, 0.67573571782262387, 0.73277612307784468, 2.9312279638305503]], [[0.0028652467418642182, 353.22026214045394, 608.13203032323736, 1.5404160413280234e-05, 0.84953149303579911], [465.77311369833382, 289.91428853299215, 54.499819716357152, 32.48408444896284, 128.76447889071937], [436.52674539024179, 665.35200193755577, 1298.6374868721871, 414.03336799756386, 154.79327618719745], [14.365653438502195, 0.005558509224358037, 144.97539085772772, 0.077778135712161384, 1626.2096038625846]]], [[[3.8568952261543443, 2403.2561341428741, 100.13221812105103, 255.7755333541358, 943.47671881002361], [1028.1758835211745, 1.0022327275951846, 88.615852834949067, 25.67581309268714, 235.90319903450245], [2171.8527780240324, 1288.851012976733, 625.43014339115291, 5.2006653268278962, 4.7488422422623282], [434.85319540910149, 2054.3527458763629, 2260.2741480415079, 1685.3731572454024, 649.65115972940407]], [[0.093081412124010596, 7.7883537599409252, 5.6677249202381645, 6.5784124452230719, 1.8307709915324959], [23.388724098231307, 11.005814335160952, 19.355662771615389, 22.721614879756032, 22.422400732629963], [0.0010115524483833415, 33.409991443820424, 0.20697029640304715, 37.745708060921281, 6.3912759074106056], [52.977413883824156, 47.344054791175338, 51.051839357540452, 40.779843818265178, 13.646817579699073]], [[1290.3327505895338, 8.97039954592128e-05, 864.46242553694049, 93.359596745930588, 1549.988419324113], [9.1514891812218071, 1.2631940979371628, 0.040043627289604859, 0.51400511915873115, 97.992527381493574], [4.0341423196540873, 1.0705664171384526, 1329.0587102340712, 185.81557718898048, 0.0094222905128666101], [35.6742495716205, 0.25300829776539485, 1.9912714467740991, 0.2720460619881121, 0.19382098422847296]], [[22.315264748534329, 12.066187661384603, 7.5694632784882385, 22.553324326518901, 1.884961931685539], [37.838379092165702, 31.011151350745873, 25.566226525137878, 49.248501935855749, 1.4516243641817137], [38.663040683353401, 39.749183199739896, 15.785550899102361, 0.90093434456643851, 8.979260859050914], [0.057686703383620773, 24.513256919574342, 1.2661463662228047, 8.02107572395904, 49.320034095976638]], [[22.87617370347628, 13.75735696697792, 9.9115265429389101, 1.2379264751190715, 2.2653699733458419], [5.1147506381360976, 0.031954522851966503, 5.9025782784497878, 2.9056034598290128, 25.801124983512807], [13.27181510425423, 0.2403322214404344, 7.8258062154300925, 4.6800460257531107, 8.0847132006029501], [15.104927100262019, 15.006265156044487, 4.4083836024051299, 2.5433361900167726, 0.20294510648511455]]], [[[3.271761499783056, 2.8170151199189948, 1.9892688573104604, 2.9838590827374474, 0.54997271614953935], [5.0948489886971906, 3.0091131239746947, 0.26366064668354117, 4.135367580012689, 4.3205316998568266], [0.68965368299692553, 0.11607038741248726, 0.97772888101825883, 3.3858479861310724, 4.9724354931507699], [2.2451171478150869, 1.335110664208, 4.7480890643920102, 2.4069525635339106, 4.9455609920940073]], [[0.85753733599195336, 1.494494284897254, 1.7989189296486623, 1.725584807512905, 1.8745532665679498], [0.49729112702973027, 0.98356563647841067, 0.54493262171192225, 0.51961139461076211, 1.5199397309520006], [1.5770139714895592, 0.9066110568807888, 1.0108419269804756, 1.4604984444290123, 1.5337502433468888], [1.8597378053270794, 1.2627966096431786, 1.6064707905798719, 0.57216287982495828, 1.8210960461214389]], [[3.1208522056991668, 0.97295907168649998, 0.51432721771789336, 19.003427571699834, 41.225255341337345], [0.27817279966107678, 95.196668369680054, 83.659745226241938, 0.83134478005165535, 56.394351942050271], [56.839327693898298, 59.785261954687037, 79.599621595734575, 6.8016779471123874, 48.02920237616695], [29.725736638414304, 45.001668869658715, 15.349362372380206, 0.096911860918780637, 11.829708602884393]], [[48.873899466337157, 6.1033675247091104, 0.0015463166447087944, 80.132785982948818, 158.86739142758375], [111.25126112347697, 1.3392002935250955e-05, 0.010606568255146637, 0.016721857298770581, 10.860131013597442], [190.38389868261129, 9.7306615335502949e-05, 0.44899394490186662, 0.005713067854121406, 43.790858925639199], [106.67477615629015, 1.9464294286405528, 0.00056986946777532041, 8.6579390431676071, 34.428918018074768]], [[502.1508344904762, 15.480302716835642, 0.83192017383421279, 293.17516880191243, 0.11964038670215521], [448.27801525445989, 1.3474123392206767, 5.9665844570322015, 343.82780349091377, 0.085156190683549016], [6.6452550158059886e-07, 310.53727960478108, 56.503143992930752, 0.41904790262652175, 253.90223686828571], [112.98940914144904, 7.0864112696648824, 217.52007167001642, 80.980461017937614, 49.041964888003712]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank2_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5, 4, 5))
      res=power(arg0,arg1)
      s0=numpy.array([[4.0263810031987175, 3.9337028776133809, 2.7044516181272131, 2.0101097109017902, 2.0966529325951524], [4.6683099309913469, 2.3302469819110359, 2.8452094891434507, 3.6781444273081494, 0.74456761879071787], [2.384448652034846, 4.8237346295813577, 4.1544294567108357, 1.2273277601579948, 0.28965071143546162], [2.8305697389381548, 4.7668428866872485, 3.8008601048058628, 1.8851660151134266, 0.089539293740701575]])
      s1=numpy.array([[[[4.0554954983019718, 1.4361382073005602, 3.329328542143883, 1.2646429335400478, 3.8779415943504376], [2.1036688384590319, 2.3898282225687919, 0.23658199680831846, 4.9334202894209911, 0.5868793230727789], [2.1458648619408716, 4.306750334144108, 0.63774255170857674, 2.6569824113040528, 0.84397752571067197], [2.4302939331538886, 1.7795144943615659, 0.74216005591231282, 2.8626718084245812, 2.604608457630472]], [[2.4570952312026617, 0.14361667388985022, 1.4020745033143263, 1.8194479791564631, 0.028434398404663892], [1.4152006926540646, 3.6561113402422571, 2.2583789950877957, 4.8458080704007207, 3.6864894865168756], [4.1080367252062624, 3.942193538937083, 2.2392600309638726, 4.7360978281712853, 0.52161677298675857], [1.583047474784214, 3.7497221827694029, 1.0615220568992372, 3.8886680564593092, 0.027088098294211667]], [[3.8703438829988981, 3.4060804124327309, 2.7739702777381514, 0.22832437771733383, 1.3566349188236384], [3.2826639800213342, 1.4330477837693336, 1.352428804724245, 0.1907026467604119, 1.1095445636643619], [0.11305933530369509, 2.0383139480709094, 2.3846359957382735, 0.91308998834739863, 4.707685029612759], [2.87840200733531, 0.85021652022931526, 2.3756668916227301, 4.6612705362815454, 0.67731530543362917]], [[0.051974138514102208, 1.9174934145006086, 3.4393491439904738, 4.3802812628022938, 4.6462910402688076], [3.7990715147746754, 0.87221204194548663, 4.9594447626802127, 3.9083185004990262, 1.493963026405889], [1.8927756444707973, 2.1923459739354687, 0.20549995962125978, 4.525067974465931, 3.263133593802042], [4.730645470438132, 2.4040893767793148, 0.61808311476250466, 1.2378342455331339, 3.5824691716260411]], [[1.6173247859235633, 3.476786631098685, 0.040383283454241421, 3.4103940890693343, 1.238967983502645], [0.41619631548439839, 4.5378898825756524, 0.64951538573315826, 0.47245984344339514, 0.53362450069682144], [2.8738046273764968, 0.75007698480133467, 1.1449522879945695, 4.1651756008485101, 1.0483757873306139], [3.9774784410327282, 3.6429449824160658, 3.2047816796216382, 0.75989852727579399, 3.4785491663023178]]], [[[2.6214431332740604, 0.90665755494434352, 3.9560304185160353, 1.8925092180895287, 1.8889994571253881], [2.5330183387768921, 3.7011649546810053, 3.5311426278673705, 3.87912571646353, 4.348931631469438], [1.089092973480777, 4.3135564865946998, 4.9082718761720185, 3.4319327440735785, 3.8769799122172866], [2.3872544585205859, 3.9931483275881128, 4.4701138136570755, 4.9090080324311112, 2.6096505857539096]], [[3.8967915614367428, 4.4268316075385981, 0.76429610464236364, 0.96072632509582878, 0.020593870569997407], [1.0573189780666432, 2.0483697074418696, 1.6197744464503732, 2.1345370102744918, 4.4349265747959965], [4.1631328994682306, 4.1247737131519981, 1.4113013150609715, 3.4660343035974663, 1.340173855147857], [1.5486539287156298, 4.095156885657091, 1.0599907454342723, 0.73371094503727918, 3.1846490592573735]], [[3.6786071267027767, 0.30887280476113449, 1.8500506112641062, 3.309088406901036, 3.5669414978329326], [2.2118133580055659, 0.029993604685587497, 3.5463479810592862, 0.27774372729714936, 4.8226375940600708], [2.5235798188915024, 0.99310765506339571, 1.5411887054464135, 1.579907338246044, 4.4453427246091648], [0.8136724987967664, 4.8120280928945451, 4.9823609975939158, 2.2939108232471566, 4.1541091728100668]], [[0.048265859279350648, 4.9110724774636001, 3.0380625273187398, 1.8869513164273923, 0.90983580707309586], [0.44915671168892063, 3.7394153993741375, 2.1910084634857929, 0.58704475652545174, 0.5617084833534407], [3.7869551278920035, 2.6197606494264605, 1.9066730881864948, 0.30313384718504277, 0.64406714614583571], [3.9425969510868666, 4.8288727922185499, 4.6981386191786907, 4.6368189919625822, 0.89916899783142623]], [[2.8396380085729884, 3.8439441174872675, 0.69892698295545908, 0.047247868780212483, 0.081586225376185845], [3.6314470099217147, 4.0094665181352065, 2.9779345289309904, 3.5096926997025601, 0.091688053971924421], [3.459620104743518, 4.5884449096448252, 3.9190322006850842, 2.3098757883450403, 0.52562916883535815], [1.5431690124438815, 3.5849151033912312, 0.83708719423283451, 2.0542792743193692, 0.56546969164057004]]], [[[4.16244134540087, 4.3547129445394539, 4.570554475267472, 2.0041360892724494, 4.3923399577701607], [4.6637248046927766, 3.3396603449696567, 3.4111110038019867, 4.1601183201744387, 0.80185035841224717], [0.073423936633665518, 1.0797285004099149, 1.7388075969333301, 2.924718943322719, 4.2261428712950586], [0.68701769892922904, 2.9203279823561932, 1.2485206061813956, 2.0728743580474434, 4.3316089478092419]], [[4.7559810081987512, 4.3179209850197875, 4.8921224702644803, 3.797989075040952, 2.0148989761235012], [2.8295048144903099, 1.3329838934255214, 0.69445051580405504, 3.7395319077689542, 1.9657910918702433], [3.9715980729322009, 3.8052836968123516, 2.84623853375835, 3.3981935097679017, 0.99302803137076867], [0.99953562703401677, 1.0628096333638823, 2.4888897496152116, 1.4625231440522077, 0.90746136414284617]], [[4.8022726758318255, 3.1627635867878632, 0.16170018844338979, 4.9541392214670488, 1.9927055101022393], [1.3746127419975442, 4.4758773895602291, 0.74212392440445107, 2.4959216606358656, 3.8611922694932774], [1.1844515595008138, 2.753350077821604, 0.22625701224958045, 0.80614596881784295, 1.4551386334712668], [1.6423804600117242, 4.850655929986905, 4.5268872142994665, 3.7261162894608479, 4.018319650430878]], [[0.99016551813568876, 2.7719580651992013, 4.9656335846480308, 1.0232740893788441, 0.34883670115018883], [4.4017023507140358, 2.4383868638940713, 4.61854679231402, 4.5313278121448857, 2.7163518495729622], [4.4637905034725502, 1.8663576117542997, 1.8885949719747832, 4.3572218186042617, 1.8167577440613918], [3.1569613190131509, 4.6752757487589651, 2.6595731559339941, 3.409969985612999, 0.99713899884193868]], [[4.7079033287263954, 3.159755272615131, 0.76863531511346395, 4.8082109955474515, 2.6041229609892129], [1.0695376849068614, 2.9136707573011043, 4.9918206222947932, 2.5232896054377569, 2.7604078066954081], [0.82644244142955214, 4.842491045555211, 0.34105900746294893, 4.5529464087458154, 2.4792745076526694], [2.4017824247862323, 4.3298897666302159, 4.1367171798523117, 3.187944719758669, 1.5347259655086682]]], [[[1.9802723344479389, 3.2641548802245515, 1.5301400576908299, 3.2851544415060698, 3.2688144945359019], [2.6612623122772203, 3.403286479935812, 2.7556147528579342, 0.67542347732356323, 4.7021980951035127], [2.8384844122287429, 4.1382026841466395, 0.99455797726966177, 4.5606655595537475, 3.7032904520020047], [0.19408187990418224, 1.7442662234892183, 0.038062995463387768, 1.0517176349125448, 1.6883581017991889]], [[4.6301063465167438, 3.4990397612165958, 3.1348786068304384, 1.5839618872646726, 0.85908897621038294], [0.67563150388277016, 4.7299521407609504, 0.039338436326521779, 4.0647290068394657, 0.33458098157890936], [0.2796660165572179, 2.0127466325187293, 3.6010249258064055, 1.4521698853254275, 2.9850867059055952], [3.993738402003626, 2.7235796290938654, 1.5937101831030469, 3.405518183514078, 3.597369381161009]], [[1.3916274103605806, 0.90110628922898584, 1.6640009164454557, 1.9327533583932908, 3.7219537881579039], [4.7220468936240287, 0.81674903756383943, 3.1152027908691844, 3.2366710537307668, 4.9261414175333051], [0.79482476720975004, 3.5982735605966751, 2.0710609334596812, 0.67650961395221654, 3.4217490270815474], [1.7534867051582572, 4.0707919438438962, 0.60032012096418108, 0.39616587918934232, 1.8945624694204479]], [[0.70426267462787417, 0.63089080831831112, 0.41400210954114613, 4.3623223743163466, 0.710038869714266], [1.2650119816267875, 1.7922815952292912, 1.1343319504197191, 1.7866603543050905, 1.9617315087612353], [2.4528963945141333, 4.2370513484242052, 1.1215931760685476, 0.019183538279235392, 4.4017091004711624], [3.2691582457862474, 3.5938640281217626, 3.2621035247676984, 0.037544045241483963, 3.5158255997578149]], [[2.986642348244303, 4.5800515777815507, 0.94339039720554452, 0.91233032257538871, 1.8058156969787305], [1.4661459636625016, 4.5003288382006037, 2.056762265105875, 4.226454744387989, 2.2416358769351246], [4.8541340495844558, 0.029499459016085645, 0.38641999496283602, 1.4917737038793721, 4.1396385691272233], [3.7087606587660811, 1.2478857446076166, 4.698829013422313, 2.810484147168586, 2.3144827883529295]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[283.94192705516645, 7.3916493204158726, 103.26670901117151, 5.821049785276653, 221.73003678113986], [18.730119942330674, 27.902468379981904, 1.3903110274821759, 964.49345506745226, 2.2647070312896882], [19.863947580257943, 402.91859551067557, 2.4309719723216832, 40.480714145521745, 3.2399219926722123], [29.520311416109667, 11.92490083144475, 2.8115360243746252, 53.910423083471372, 37.632786989506002]], [[28.939036033085497, 1.2173723143245627, 6.8226971292597929, 12.083993541161137, 1.0397114442607533], [6.9464604994542949, 149.5067169619843, 22.043844287870613, 762.59537692450681, 155.85820616339248], [277.63014882434402, 221.21921203311712, 21.474118486231028, 656.20320257864375, 2.0429534303573811], [8.7417643924299924, 169.95759597056229, 4.2795190947681974, 205.58233806704948, 1.0377961206630593]], [[47.021356345630885, 29.627605669210524, 15.797010399033978, 1.2550303207243305, 3.8563261086823997], [26.204198483425849, 4.1609276420241414, 3.8402224151336788, 1.2089230277994834, 3.0158597812858252], [1.1190528277906624, 7.5982415111197934, 10.723872763757962, 2.480431321323513, 108.16652032634677], [17.526600356190258, 2.3300266473702123, 10.628605532724043, 103.28520621428818, 1.9617960982996734]], [[1.0369542260694646, 3.8143619033278173, 11.037701062009399, 21.290597053176416, 25.635822852114686], [14.189057846924792, 1.8385351082930947, 31.900806216499262, 15.313674237042871, 2.8379132310514086], [3.7490996138319872, 4.6212836265111585, 1.1542812704343823, 23.555375552354331, 9.7599274777877021], [27.191000764496749, 5.3575644952870061, 1.5396252138331785, 2.3732080068676069, 12.19762980254877]], [[3.3114092074501413, 13.118349472931321, 1.030348866456569, 12.489131305367238, 2.5024251502662826], [1.3608747737770808, 28.777433540575775, 1.6174707602858862, 1.4187581898779631, 1.4844803133684077], [8.3946863725167606, 1.7424874630024085, 2.3341703837725207, 21.83808358968253, 2.1731046730283898], [19.004870057537328, 14.835545405496054, 10.725619447123492, 1.7552037928638959, 13.135478508130003]]], [[[56.775903184546976, 4.0429528874797152, 443.82938660861947, 18.466766815046238, 18.36717122865339], [49.544305430595863, 299.68869568090145, 230.62069444402712, 394.23441004909637, 813.07330900260536], [5.355217411687355, 769.94212953374119, 1924.9419825701411, 197.92972503025672, 392.93312530607784], [39.57805363282791, 469.95238972055176, 979.98614341907842, 1927.1266203214607, 55.75360505424473]], [[27.020217129867298, 42.308398212103342, 1.9089910065826745, 2.2540976656566167, 1.0175745306755644], [2.4460259417491419, 5.6568548251020747, 3.9364822447068559, 6.0846145104493665, 42.599126126264238], [33.848791106917112, 32.767999072714133, 3.3000069996270791, 18.768422713521041, 3.1072950506738666], [3.7066235416760085, 31.957194240174989, 2.4515608158829965, 1.8602308181800835, 14.792660418874283]], [[46.82824811005262, 1.3812221732052514, 6.9204501228498376, 31.820294515310621, 41.667645208634625], [10.102199965527189, 1.0318593962101785, 40.779993121192881, 1.3369876141731807, 154.89180708555193], [13.995665457827025, 2.8247780925540584, 5.0104386718677985, 5.2174516545580714, 104.39831051809327], [2.3415329593682652, 153.18298246346612, 183.04636965070506, 11.007725462349292, 76.991036254323944]], [[1.0648797268357471, 599.57556592374272, 52.289633279010829, 11.676534531088912, 3.2706141857478759], [1.7949649067863367, 130.35298007150303, 17.349918563630695, 2.1480748408392518, 2.0783490307230368], [138.67903240716566, 30.325782499087214, 11.980340723865329, 1.4840933793683204, 2.3136779789095132], [169.84247002035391, 538.70303024668681, 454.36242408913074, 419.4867639157963, 3.2254912033762135]], [[0.43276700406819557, 0.32181530243228151, 0.81371130627441379, 0.98616081943632239, 0.97622324094546009], [0.34263102843218868, 0.30648128343726566, 0.41546929202052174, 0.35515907180753614, 0.97331886344664842], [0.36044333901352738, 0.25836829869704797, 0.31476629262094397, 0.50595802144831825, 0.85638505977813806], [0.6343463505941469, 0.34736594121479941, 0.78121864529409168, 0.54557610334825279, 0.84638055323036754]]], [[[37.226765923253296, 43.996314500414137, 53.072836093382655, 5.7060669035189777, 45.458623411231073], [57.548453574030404, 18.211581145438636, 19.3781481866436, 37.151694488989023, 2.00728335193633], [1.0658824480860649, 2.5555039220830884, 4.5311149206011425, 12.698539029774267, 39.34553892684972], [1.8166539934661337, 12.650178750957185, 2.959215974599871, 6.0572828720769172, 43.121824461734846]], [[1778.9285637949672, 892.88506164922046, 2203.9174564488094, 393.9877480525933, 23.820371398165587], [85.829511816832579, 8.1459000228238239, 2.9824691491539768, 359.36357785893006, 22.049005980854872], [517.75490897794873, 398.53616752227367, 88.119533314411385, 210.02406543566019, 4.771103963674479], [4.8202111497277205, 5.3248388515902398, 50.218756953723371, 9.9876669085542211, 4.1700766655972776]], [[933.81391842996766, 90.407674631949519, 1.2589642796957263, 1159.2885082936575, 17.080911904130922], [7.0829411974142786, 586.65279066171661, 2.8774785788518318, 34.974859053829121, 244.45076096795657], [5.4025249247364968, 50.463617619113165, 1.3802016393317034, 3.1521760346311081, 7.943644815231651], [10.371248003132742, 1000.4282262758318, 630.85754052122854, 201.6718868205196, 305.75704196888171]], [[1.2248578118855773, 1.7643914089270625, 2.7653208703847549, 1.2331929501973948, 1.0740702991270339], [2.4636436744033836, 1.6478597823414667, 2.5755411723193014, 2.5299354922365946, 1.7444084210183284], [2.495176612820114, 1.4656565506120223, 1.4723479681143481, 2.4412984845915844, 1.450840871359433], [1.9091719215618885, 2.6056443428372482, 1.7242375956148184, 2.0107257075899785, 1.2266087011637268]], [[0.0029279094131526092, 0.019936798916744065, 0.38581384937578539, 0.0025857083181233826, 0.039687641292822699], [0.2657385769568108, 0.027044549838645796, 0.0020595603798023378, 0.04386859957893581, 0.032700503794982232], [0.35914541816819667, 0.0024781783976649497, 0.65534121959191938, 0.0035476755143556138, 0.046327558456697407], [0.050996441323462094, 0.0046770965520814426, 0.0059419283555202235, 0.01925244853161601, 0.1493224046986123]]], [[[7.8493429126689458, 29.852881937682678, 4.9139450228905659, 30.512333053518326, 29.997966881300261], [15.94244296428611, 34.502947420204023, 17.586939654082855, 2.019324959969703, 133.290869466663], [19.170652996832192, 74.121742063214555, 2.8145874806790245, 115.03911919256703, 47.143318394216081], [1.2237720446063967, 6.1402819027989892, 1.040398392874486, 2.9870583525919829, 5.7932849655818375]], [[1381.2774040133302, 236.13299165571274, 133.71229546475016, 11.865657220786945, 3.825257155730061], [2.8723285900612843, 1614.3563415631211, 1.0633605412018641, 571.24782626222282, 1.6862545693227011], [1.5476691178024076, 23.179647926607029, 276.90203854997628, 9.6584269227934261, 105.82246118590105], [511.30088339777814, 70.342159039241594, 12.04767914922445, 204.04570204987058, 275.325762035948]], [[6.4117934893084945, 3.3306976738521685, 9.2240996980368664, 13.205919969509903, 143.97715220677563], [547.30504890150917, 2.975897520944685, 64.039755782726729, 75.31594199878559, 718.75382983493375], [2.890044196892386, 122.06008152152071, 15.884396470141256, 2.467718803158935, 96.429580269159743], [10.394776985540092, 229.39212604630595, 2.2290240968869846, 1.6971859071009061, 12.549385381565916]], [[1.5628541384140517, 1.4918169349645714, 1.3001555555240389, 15.891457440428868, 1.5685881150719603], [2.2300805273679707, 3.1153297593938634, 2.0527584614930028, 3.1042466226559604, 3.4686620593030768], [4.7359165991844803, 14.678118921876514, 2.0362459546898948, 1.0122369341034347, 16.293292289489685], [7.9462353092643205, 9.7626794411938889, 7.9107727142865265, 1.0240890863370236, 9.2914004738266289]], [[0.00074137788991844339, 1.5855083387831416e-05, 0.10264546220754077, 0.11063442638020922, 0.012809430600349359], [0.02907361694753538, 1.9218384242639118e-05, 0.0069910313676880423, 3.7216298822964072e-05, 0.0044750328917186315], [8.1834012335544152e-06, 0.9312900652542061, 0.39358371937848197, 0.027330120255474422, 4.5889713419761676e-05], [0.00012979845046427231, 0.049230335034922648, 1.1903958412694998e-05, 0.0011340991307220488, 0.0037536490368367858]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s0=numpy.array([[[4.9088480363487701, 4.8982441147824893], [2.6106169601037212, 3.9914973477443576]], [[1.123597764544952, 0.98164172066478195], [1.2552398059694336, 3.5364457076901652]], [[2.5710086249089694, 4.89643882915819], [0.84556342639436055, 3.6559505124211173]], [[2.7047584400193987, 1.9588571946647504], [3.2324063857638343, 1.7238437247692366]], [[4.1372481473964111, 3.4604256844038646], [0.89673410554080091, 2.4439041728937561]], [[3.501577700745083, 1.4884432455240455], [2.1671689259885074, 4.4273547624480427]]])
      s1=numpy.array([[[3.6411965775071979, 2.0690791698927584], [0.84941984484809496, 2.5706600567791971]], [[2.7737732352991626, 2.8605427273280735], [1.3071411242720992, 0.44399348715347892]], [[4.5991967989578759, 2.723911897610428], [2.0713326775741998, 0.75023239545592568]], [[4.8424321714350356, 2.8136237303942502], [3.0465583109944827, 0.35321280512458125]], [[0.42098353986776649, 0.74860947765387109], [4.3441203656501557, 4.8429604818438809]], [[0.84361456363313414, 1.9048681712993838], [2.7877412295640651, 1.2459651380836416]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[328.08963492133256, 26.776158520709441], [2.2593832572028929, 35.100710344505444]], [[1.3816023614336272, 0.94837748626212748], [1.3460144853879643, 1.7521051779536141]], [[76.939056949585407, 75.713486921562279], [0.70647292500886072, 2.6447283462484203]], [[123.75218517058723, 6.6311055382356976], [35.669788653189286, 1.2120878110919167]], [[1.8181320385569018, 2.5327800310693629], [0.62282403150129861, 75.766340798504075]], [[2.8783847232981739, 2.1332031758981906], [8.6373816461690591, 6.3837122243681153]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=power(arg0,arg1)
      s0=numpy.array([[[[0.10530844912183927, 0.38081924095895653], [4.7821341196655975, 4.7271370133982487]], [[3.0152260831054494, 4.0083047549704052], [2.5810781526852788, 1.0484050096696762]]], [[[2.1004893360464942, 4.0691497766933171], [3.67537942556259, 1.2480281138711038]], [[3.9577192979864844, 2.9719008786058096], [0.25684759272663321, 0.15688984763462452]]], [[[3.9544876733573928, 0.96153186008557712], [0.81104009657988441, 4.2551846360548353]], [[3.7397876155221823, 3.2509353462716173], [3.0865661524809913, 3.863501893861125]]], [[[1.6171925164741299, 1.9430628785697046], [1.9597327590837992, 4.0017173921348439]], [[2.9301221042016783, 0.41863149011463524], [1.1584086713231718, 0.93453127861199836]]], [[[3.6997969871966019, 4.0853950406084909], [4.1832507672719723, 0.28478144902751462]], [[2.9949315302525115, 0.51309503846844096], [3.4867604349338071, 2.0847248447379534]]], [[[1.7277140284744574, 1.4246939728969092], [3.6603524941914736, 2.5437028527417156]], [[2.184511710502679, 1.7775494715302997], [0.45769266806122227, 2.4893136483858935]]]])
      s1=numpy.array([[[2.6650862894088614, 3.5300154736499376], [0.63834825503494297, 1.282322112464507]], [[4.7457879969029815, 2.1105714034209324], [2.9505485608429738, 0.17651416027658898]], [[1.7243205480934991, 3.6965155633089557], [1.7846947217568314, 0.62128639373650341]], [[2.1976861895880133, 4.6427380808274918], [0.40765200453166039, 0.5324362731645661]], [[1.533680183679804, 3.4589007390059741], [4.3930216033177896, 1.159955203007025]], [[1.3694536136799493, 3.5085324762257377], [0.63391556587199505, 3.0340672308071617]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0024818762441677863, 0.076309682019868888], [250.65438195638387, 240.62572209682824]], [[2.0228934577085251, 2.4260454476710374], [3.3733525105382633, 1.0624901262986215]]], [[[33.858262674748225, 780.863688247401], [15.599497725275846, 1.5962040220179488]], [[57.914918713337975, 24.872001383242615], [0.78668227770626997, 0.72112590812211264]]], [[[10.704710976265261, 0.9345960080818253], [0.46107708727423674, 211.25484975307066]], [[10.528267444227, 8.1993338235497113], [2.0142089608904183, 2.3157058304925719]]], [[[2.8760256532674613, 4.3053015621792134], [22.729686916541638, 625.276870394708]], [[1.5499828721957138, 0.70119532732955525], [1.081439774849968, 0.96459075392373472]]], [[[7.4370924027884566, 8.6583983237608351], [141.17405388437453, 0.012978069643245976]], [[123.81633216675033, 0.05332031926670263], [4.2578008850822391, 2.3446728169684246]]], [[[2.1144951642224137, 1.6237328717773851], [94.872082049457617, 26.460128678067541]], [[1.6410507881076817, 1.4400111741711952], [0.093359509368616186, 15.912272100653357]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank3_Symbol_rank4(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2, 2))
      res=power(arg0,arg1)
      s0=numpy.array([[[0.60885663282977232, 0.81468269711477437], [1.5178892578677474, 3.8842004534281247]], [[3.4997107650373582, 1.1796091784705893], [3.3090387082939849, 1.2732443676178424]], [[3.437455144293327, 4.2766477047107783], [3.3250206467916179, 2.5772173929319764]], [[0.63775958149076561, 3.0917024976240914], [0.87008990988528112, 4.5348828068287688]], [[4.4352374116902915, 1.9194964239835217], [0.61228877024124118, 3.1781221404992417]], [[0.14485127977612541, 2.7675635187075152], [0.11385325481052272, 3.8604799744306]]])
      s1=numpy.array([[[[4.5545746767508941, 0.78369834510461955], [2.4583556574110559, 1.0108101084960959]], [[1.3805277345347962, 1.6901883507153261], [0.11701406296407972, 3.768902892558331]]], [[[2.5678448423725664, 2.5130453494633738], [3.937271187096036, 0.9576465821901039]], [[3.4200344802916955, 4.2708510798150963], [0.69002850547297656, 4.1665488195529541]]], [[[0.73894012003514042, 0.47667239865063349], [3.5494051198254728, 4.2357987117858711]], [[2.9940792208894109, 2.8727546372457411], [1.7507406081413182, 4.1205884892104097]]], [[[2.4884086609147786, 1.5177065679151827], [4.3441152654542625, 4.3214281634044598]], [[4.8516052965988958, 0.66960288276055502], [4.8339997188869335, 0.32719089324271944]]], [[[4.366698912781108, 4.5883456566471938], [1.2291791897942508, 2.0326335455844493]], [[4.7047925388757124, 4.2003040159430771], [4.523161793892176, 2.8939676781310975]]], [[[0.99481286139975211, 4.671502613310671], [3.2139342408441212, 0.34229605590788509]], [[4.1173807542943335, 2.0439432013546832], [2.4162892978464163, 2.638226362795757]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.10436566989811337, 0.67783625612581944], [0.60419629387840035, 0.81287968200165195]], [[1.7791270687580509, 2.0245564218709378], [1.1720781735332209, 166.34916885127043]]], [[[24.945382017704667, 23.290428191255568], [1.9162515709328871, 1.1713853482968768]], [[59.89605760063764, 165.79414765747296], [1.1813867822135993, 2.7360245198525868]]], [[[2.4902735882838805, 1.8013968375029523], [173.79685333447279, 471.22388005850394]], [[36.500062672289175, 31.549111164057315], [5.2458859647639082, 49.452049907456782]]], [[[0.32651772891527531, 0.50527363070578868], [134.73340882196496, 131.3270330056863]], [[0.50908350356923815, 0.91102851138197816], [1492.2451955662514, 1.6399191538110318]]], [[[668.17650620891493, 929.56169574690773], [2.2288864551033862, 3.7637088533484704]], [[0.099465750268876288, 0.12739479001071857], [186.80943153689736, 28.396540802092595]]], [[[0.1463102487670557, 0.00012029470255221828], [26.355581664322319, 1.4168643199427386]], [[0.00013020117986257005, 0.011782126991963045], [26.151409690347823, 35.293323234139635]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=power(arg0,arg1)
      s0=numpy.array([[[[4.9078912614143544, 0.20967868056482342, 3.4910460821049987, 1.6258153165972409], [2.7489880352392304, 1.3812615978623735, 3.3122050386737971, 0.61264072654454449], [3.2571764366850822, 1.039731426150369, 0.83395690185553129, 1.9778514097332243]], [[0.27167559889351062, 4.0673537341307524, 3.6158998187612688, 1.0088375526975863], [3.0704762241623622, 0.31615080864108608, 1.5878316584888734, 1.1104759480619302], [0.88746827096773551, 4.5391889045110769, 0.80577432261653859, 1.1669442533048939]]], [[[0.70469387911954517, 0.33806103177744501, 1.1053432726024999, 2.5508887626079382], [0.99835540814944135, 3.561169561779904, 3.2083889949105657, 4.2983642989601734], [4.3861916326520367, 3.9905446893165424, 4.369403174925182, 4.9339679526017148]], [[0.83309782648158948, 3.1523951979634779, 1.0053979636491457, 2.939113750200554], [2.0297729478724893, 2.863906475269927, 2.3921224436347388, 4.787520489784205], [0.65311248770042285, 2.2875251713210929, 1.1109614856235601, 2.5716150071861059]]], [[[2.2622967896686372, 0.30587659054674693, 1.3459537024800152, 4.9604957191591481], [1.8539226948089866, 4.9039414943158084, 0.29227602640783384, 3.4284261504864255], [2.5227980027411316, 4.336838433819846, 3.968926067743209, 1.978950709584929]], [[0.064633483168804337, 2.100208106944268, 0.17326292449486683, 0.020937542049739515], [1.174368569705885, 0.093727760600778906, 0.85723871569391441, 0.66979289503587824], [0.14090321825514501, 3.6338551312409484, 2.1232163602974334, 3.0090091883491255]]]])
      s1=numpy.array([[[[2.3096705712685197, 0.93857082155306915, 1.0836387937407845, 1.7384490010709901], [2.5878132567160184, 4.5480955665382563, 3.4737841291488549, 2.2389971287817869], [2.0016380998706942, 0.94701649622642492, 0.022363657541109125, 3.6697010777340999]], [[3.1866744653299812, 3.2676336729097888, 3.045513300001093, 4.4424297733895113], [1.2094652763643758, 4.4144444766072155, 0.91913471498991273, 4.8582929118065294], [4.5481322677596863, 4.1843735856934705, 3.4975497537621667, 0.71518180163080547]]], [[[4.8499039479403496, 4.0639931402769633, 3.4271148308462056, 0.7343997091653176], [1.8287905103339768, 2.3429832136106974, 2.9793046139944455, 0.15387298566642527], [3.1576774886127867, 1.2361375845696989, 1.9424272705974535, 4.7443938835463815]], [[1.5850882408595515, 4.9450862039040775, 2.0355321471886696, 0.081326668803160043], [2.3560366655664167, 0.19550375559354671, 0.41567189563684886, 0.1156503694903523], [2.9309653617609435, 0.95577531156756157, 4.9299490319978725, 3.9923297163068403]]], [[[1.939396338658643, 0.62892483553148926, 0.93905622222004215, 2.1030299546820199], [1.5640927174520824, 4.3491306625828186, 2.8473810647073199, 4.8482215674267968], [1.3895657924375924, 1.7612417921573766, 0.56624777113727853, 4.8354302116521142]], [[1.6693204344958845, 3.7239854943425406, 2.2000497064179059, 3.2342046979395551], [2.0376586332300142, 0.018492921996274884, 1.1526590358834625, 1.8252260246138079], [0.4359570951888439, 2.8070583546143544, 2.2377209358691847, 2.9840683954032734]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[39.422094281249862, 0.23079725466857942, 3.8758568761568166, 2.3277510720851442], [13.692959455403804, 4.3449858857407184, 64.087734795837221, 0.33385242915976215], [10.629740292300873, 1.0375872535113935, 0.99594758449744247, 12.216370559343659]], [[0.015721848387264974, 97.950853893842677, 50.1251215241899, 1.0398617176659084], [3.8838140058142159, 0.0061988502021321354, 1.5295593991730208, 1.663783498700133], [0.58102154353338309, 561.10283685205377, 0.46986898944496425, 1.1167424325269539]]], [[[0.18315425873518132, 0.012185388178316395, 1.4095153327381353, 1.9891802310345192], [0.99699443594498161, 19.605247352663923, 32.239125192402398, 1.2515501521767864], [106.53825196676669, 5.5329719139571782, 17.53773043409717, 1944.448303371541]], [[0.74867988400114382, 292.29476350684723, 1.0110184399075979, 1.0916375841617285], [5.3009961484918078, 1.2283925325635583, 1.4369763361340187, 1.1985469400477879], [0.28690373388391177, 2.2053272177078687, 1.6799399454985919, 43.418759378276675]]], [[[4.8709329993987636, 0.47473063041732805, 1.3218024071237049, 29.020846477288764], [2.6261537698804478, 1007.5617041525666, 0.030123793158183126, 392.87839356704922], [3.617779926332608, 13.250008124301866, 2.1827161258382324, 27.126305256349013]], [[0.010334096177777748, 15.852590615531923, 0.02114046657261829, 3.7113336536413737e-06], [1.3875146520780948, 0.95716506588148409, 0.83731555397120083, 0.48117398307397513], [0.42556512784174316, 37.409730543813616, 5.3916523760115753, 26.770007167994471]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_float_rank0_float_rank0(self):
      arg0=-0.63638833874
      arg1=0.675249449251
      res=inner(arg0,arg1)
      ref=-0.42972087524346775
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_float_rank0_array_rank0(self):
      arg0=-0.0120734848986
      arg1=numpy.array(0.186721047866)
      res=inner(arg0,arg1)
      ref=numpy.array(-0.0022543737516678149)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_float_rank0_Symbol_rank0(self):
      arg0=0.416882080966
      arg1=Symbol(shape=())
      res=inner(arg0,arg1)
      s1=numpy.array(0.220367887621)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.091867423569640894)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_float_rank0_constData_rank0(self):
      arg0=0.416970345798
      arg1=Data(-0.0686904741385,self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-0.028641890754520932,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_float_rank0_expandedData_rank0(self):
      arg0=-0.702135729267
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.388305302896)+(1.-msk_arg1)*(0.618444759567)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.27264302702700538)+(1.-msk_ref)*numpy.array(-0.43423216227036321)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank0_float_rank0(self):
      arg0=numpy.array(-0.172668075783)
      arg1=-0.72035494477
      res=inner(arg0,arg1)
      ref=numpy.array(0.12438230219442528)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank0_array_rank0(self):
      arg0=numpy.array(0.367217155332)
      arg1=numpy.array(0.353427284375)
      res=inner(arg0,arg1)
      ref=numpy.array(0.12978456198487195)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank1_array_rank1(self):
      arg0=numpy.array([0.17057698496056495, 0.65197508237616231])
      arg1=numpy.array([0.32087931119793067, 0.47355654012965243])
      res=inner(arg0,arg1)
      ref=numpy.array(0.363481689701)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank2_array_rank2(self):
      arg0=numpy.array([[-0.035529960222147716, 0.99948208956536089, 0.29174060409603397, 0.80304088757032743, -0.22356450624851254], [0.49066384343149982, 0.079758333795518332, -0.99232738609739557, 0.25810714747140073, -0.29886716922556156], [-0.69989595642288926, -0.64245939648716655, 0.16655367423389711, -0.95321218749485381, 0.21638802862009299], [-0.47773968220001661, -0.23536079683934341, 0.30508717152899334, -0.14545129929727274, 0.84846996077931758]])
      arg1=numpy.array([[0.71176134831396065, -0.11756983608048777, -0.38521912408149217, 0.27444535320199082, 0.57303910984221029], [-0.2450861922400549, -0.53694004187819711, 0.45031452258385873, 0.45834369692056742, -0.3311126283689303], [0.63516776015807475, -0.60784580998742488, -0.25877736622125513, 0.55939114578330851, -0.89510238302072809], [-0.73226548502549438, -0.45276749570816377, 0.0051676384025589606, -0.63694640946938041, 0.50513135858382707]])
      res=inner(arg0,arg1)
      ref=numpy.array(-0.400418822582)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank3_array_rank3(self):
      arg0=numpy.array([[[0.89352907932481784, -0.2813007089590529], [-0.69957301452894849, 0.17341921348802947]], [[0.054935566453697016, 0.26169631254615577], [-0.58304701755746646, -0.57114551586649376]], [[0.47352481574743033, -0.048416145171658709], [0.93223229353677217, -0.454772692829994]], [[-0.32462211909078809, 0.089935113935614819], [-0.286419458027239, 0.86148561956606584]], [[-0.46565312018771388, 0.68711061108884097], [-0.80180026607064825, -0.2937284863457541]], [[0.25592134854363269, -0.76990158619153037], [-0.5453961772646585, -0.3664968003943212]]])
      arg1=numpy.array([[[0.32367504239132705, 0.14792860294053156], [0.6280980890935608, 0.76580645778924739]], [[0.011893082020882684, -0.82206139884528118], [0.52472067780122855, -0.98039256498560623]], [[0.73565201035010919, 0.59105957526157393], [0.26717809906364232, -0.018751742395435533]], [[-0.54650947408049366, 0.40974338641807573], [-0.5411625415474377, -0.55103964175712683]], [[0.17755009800646215, -0.9728086543767589], [0.63120223112293616, -0.39282275192220806]], [[-0.51566477827268198, 0.54812090272268477], [-0.7413890093151918, -0.73686610466046076]]])
      res=inner(arg0,arg1)
      ref=numpy.array(-0.568960512795)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[0.21900292911625985, -0.60296685402518957, -0.036182884002972315, 0.43073749211345658], [0.17662974671291853, 0.47378352574048255, 0.13148163346860686, -0.9075698563504826], [-0.26566039804635011, -0.16850033546278254, 0.07451540452840133, -0.35668991292063268]], [[0.90587343082002314, -0.55800425115866381, 0.82036937796381881, -0.52043951259439081], [0.84970150242946874, -0.92369260371025752, -0.1994015294909357, -0.3565446913147281], [0.52982924316795788, 0.35685085558316776, 0.8621190418377902, -0.86631488594347084]]], [[[-0.055899301832907478, -0.62636120778693516, -0.98921303748822575, -0.70197234469007541], [-0.40991408807660368, -0.053106017924667848, -0.066485118669267873, -0.024378451385486777], [-0.9243594466057472, -0.96050256112690779, -0.64964483254915906, -0.99257146022549003]], [[-0.53042944297798789, -0.3730158086125106, 0.21336738115690945, 0.50992862268150629], [0.3421173202861485, -0.22668604447876262, 0.27710925912205209, -0.22342248055928193], [0.94642089014990094, -0.86328145577679227, 0.026956203656329336, -0.19213054671644891]]], [[[0.5092821302248649, 0.66099009757380012, 0.27398216648635487, 0.79901877176611036], [0.94090012810807888, -0.72354491228080819, -0.96828772385126149, -0.22141913324735873], [-0.48542816727608407, -0.76982959274647911, -0.3068381860033953, 0.2572561109009035]], [[0.52546472411875222, 0.47516779288045119, -0.63172905926969736, -0.65471479410742206], [0.17532321209404156, -0.87251359508673287, -0.041241206248513462, 0.14482583937594073], [-0.70481434929401998, 0.5951965426459036, 0.19848055508804241, -0.90212192321127316]]]])
      arg1=numpy.array([[[[0.40293190800871881, -0.52178068293538615, -0.0010302315027994613, -0.69940520456709776], [0.51617551422053642, -0.61106171553887001, -0.50747804751707726, -0.13235539501135385], [0.77055869833341051, 0.31268222395947043, 0.98450151992025936, -0.24145694940836915]], [[0.10755497690663995, 0.93179498417987583, 0.76190767263619685, 0.29396668842920293], [0.49604049008825579, -0.92333650195578332, -0.027861799684368416, 0.60296582674521226], [0.60520530056161781, 0.49193959262973541, -0.98823171213192729, 0.26461406021984279]]], [[[0.88914934413791347, -0.1774138189223089, 0.58911406224725549, -0.79660932378069171], [0.12292073845561458, -0.11657999170114852, -0.80012642473751705, 0.88884685738952696], [-0.66836072518944745, -0.14373852043843982, 0.99356869621344757, 0.82527812224016284]], [[0.94254195204021229, 0.61233093725048682, 0.8263349635204229, 0.8169097790734059], [0.67347372348904333, 0.13773189624189031, -0.26566944449497609, 0.73618620492089559], [-0.1682516688141209, -0.65535861723124555, 0.12215518357441701, -0.56159828576022308]]], [[[0.23636310358669732, -0.36815150321797252, 0.21433798217009592, 0.44508327099916811], [-0.075644838498539313, -0.22190339086905841, -0.062431398652471382, 0.87125454618723563], [0.6058326030522907, 0.99421458438737109, -0.95649568451053169, -0.70291984701882626]], [[0.36370399365467687, -0.061169008055809604, -0.033210650166190181, 0.67962607885974613], [0.94337960804017129, -0.31509433991294244, -0.044306808278911758, 0.95119341821522396], [0.38655537876329316, -0.29896841578308009, 0.68221891453643635, 0.23502327204928264]]]])
      res=inner(arg0,arg1)
      ref=numpy.array(-0.859513882924)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(0.512327913472)
      arg1=Symbol(shape=())
      res=inner(arg0,arg1)
      s1=numpy.array(0.182639204946)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.09357116278819369)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([0.58625725162039455, -0.28007626156554055])
      arg1=Symbol(shape=(2,))
      res=inner(arg0,arg1)
      s1=numpy.array([0.7864599154637979, -0.64706580404901626])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.642295599934)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-0.26075473705615426, -0.51450542272909772, -0.16856910628056898, -0.54718938779657567, -0.9568631809070558], [-0.11758930278718438, 0.20419170093188632, -0.083034850977817154, -0.33812209846191621, 0.92648020321085189], [-0.77802026073675545, 0.033673216378877324, 0.77538315561521731, -0.3910441632839885, -0.68936858819485569], [0.24776180216425892, 0.89734719440562949, -0.92107425463731385, 0.93393573450430356, -0.10325627604201504]])
      arg1=Symbol(shape=(4, 5))
      res=inner(arg0,arg1)
      s1=numpy.array([[0.23265342042754522, -0.28081742178632396, 0.4428345471223345, 0.01671345696113713, -0.87022530868228753], [-0.77897533524290874, -0.4939411820039783, -0.10652493119613227, 0.70303935662829242, -0.91961632036392893], [0.52865223434846653, -0.5142903456727872, 0.41068002106248835, -0.92525654803403823, 0.75220135716434777], [-0.13850931830090429, 0.13639684710500544, -0.88394196219489274, 0.093351125079879305, -0.51127182914072544]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.517892578812)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-0.60701972383443858, 0.63247284447185637], [-0.46514049007897684, -0.10351657000372971]], [[0.19919865855096641, -0.69257134222891215], [0.78785435913304291, -0.36616282433975167]], [[-0.15077393567545161, 0.81552282868731019], [0.19993984709848323, -0.088710282197768553]], [[-0.86178608361706299, 0.090968269122525314], [-0.63832971364696678, 0.038434452465043734]], [[0.93889015760497818, -0.051865010053750105], [-0.65702149659973563, -0.94850894232875693]], [[-0.65755291879070099, -0.48072839388188982], [0.83775132967348176, 0.09533525734039805]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=inner(arg0,arg1)
      s1=numpy.array([[[0.89895641777708768, -0.8712282875855526], [0.44422955286521848, -0.58067102296397644]], [[-0.98582540818324271, 0.57002371980436073], [0.16340834580687713, -0.87671545609296908]], [[0.12538570170606245, 0.66049692536130844], [0.76013747086247552, -0.14070088510666978]], [[0.83035526161070083, -0.47396747348278589], [-0.3970066793194873, 0.70526638804043529]], [[-0.11822968768894304, 0.03554365617771138], [-0.94156797461851593, 0.70980097134860909]], [[0.57147627890600949, 0.26333690444755931], [-0.79087682280997873, 0.49787188531400139]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-2.46352842652)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[-0.65019687663827663, 0.62930833831460187, 0.78572602752040477, -0.90978251154122325], [0.63049371648481589, 0.53082697058380135, 0.19169045634727655, 0.82205763161534007], [-0.4768138930049064, -0.35901467993910008, -0.34569595980049761, 0.76731149306291235]], [[0.99556802711118131, -0.68245716684943059, 0.32327797602635222, -0.62538819631468434], [0.46672688901476089, 0.12450635059310788, 0.92098820805792148, -0.4033040956353986], [-0.0061991138734938112, -0.9676385814273647, 0.78290970118147851, 0.30151977166102761]]], [[[0.97375856209559064, 0.10574849725714053, 0.84441958537348283, -0.19638642254445804], [0.41361106989578089, 0.79996385256545621, 0.0079277509781106037, 0.1048976021503023], [-0.23560849563109021, -0.40674746439775289, -0.39542450836664211, -0.35648421869119362]], [[-0.42952614087019247, 0.17735609117221585, -0.071001480584516052, -0.32802509238901489], [-0.90864119046339553, 0.23020637146589484, -0.24610668380516465, 0.32913261497032864], [-0.30317727860754995, -0.49221336969349849, 0.79822741353179549, 0.66823188243723486]]], [[[0.41926062587643331, 0.69250979988697403, -0.5429967032926788, -0.13415325250304422], [-0.094088968189156352, -0.84460092533303022, -0.37212022685061452, 0.23567602722548697], [-0.6180155699232508, -0.52544828473462024, -0.74491176472923715, -0.31595288219285478]], [[0.14168981260676028, 0.81668897040907096, 0.96673577849435421, -0.99681022807563391], [0.63462149811915136, 0.92885047025976486, 0.93462686104547688, 0.81781294905929247], [0.57662620972609746, 0.88592570308008223, -0.24829849630726386, -0.49957190371886084]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=inner(arg0,arg1)
      s1=numpy.array([[[[-0.23854040381072172, 0.22085429232296749, 0.22707153887648013, -0.33066930660012583], [-0.50047027356273155, 0.095246687594949808, -0.078058344555555381, 0.51414749567731755], [-0.12179656181226273, 0.45027168260978812, 0.75074186341977089, -0.032527261344430736]], [[0.16519254383081572, 0.73336425784372894, 0.81903227673262902, 0.19257304717391022], [-0.60292411572448978, -0.83809335389718242, 0.57037504328479804, -0.069253179846518131], [-0.16979516136030148, -0.25229797132748844, 0.27847465772874136, 0.29719439565863581]]], [[[-0.79211871042291415, 0.5074554533208564, 0.44315776402632823, 0.98386852323144236], [0.6974804701393702, -0.020378949131680146, -0.61579872410792058, 0.19492854979603291], [0.18571370888159455, 0.99619760931903123, -0.9518870064518965, 0.69701576633410278]], [[0.70130984915793326, 0.51663054163723254, 0.27356197741065125, 0.7600903693176202], [-0.82118767785971691, -0.68572604284120686, -0.55071785018561514, 0.6738545857941487], [-0.83061018396068076, -0.93535244116842176, 0.67729382568747587, 0.65955197200247562]]], [[[-0.32937585780916767, 0.91741417480604315, -0.030390309344732191, 0.11755904365803271], [0.95065599462429429, 0.31290419096179045, 0.53929723013537378, -0.83646560089194155], [-0.3484227540608178, -0.30745915232939081, -0.42719417971417517, 0.91450620742054056]], [[-0.65158176435155579, -0.93808477435808602, 0.54847650215495158, -0.4356434554024875], [-0.073117442553074286, -0.23884993713529745, -0.027729788345835171, -0.7892471432171404], [0.61312306461815425, 0.55316343115508504, -0.3878893552389282, -0.45791501970181869]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(3.13516156612)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank0_constData_rank0(self):
      arg0=numpy.array(-0.810401259044)
      arg1=Data(-0.325113918466,self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(0.26347272885752965,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank1_constData_rank1(self):
      arg0=numpy.array([-0.070866698348063917, 0.050603972040196954])
      arg1=Data(numpy.array([0.82981473915326354, -0.90417609429403045]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-0.104561132599,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-0.012332909244574752, -0.43045347177805371, -0.56902413378307704, -0.61290486950010425, -0.43154713220921792], [-0.19526684671168137, -0.47024244784236813, 0.77901986141687618, -0.71109341477698207, -0.32125703210104373], [-0.52994874156774552, 0.91638894385906422, 0.4049789497535381, -0.92738222662475445, 0.93783732431033617], [-0.6882772029887636, -0.4796143837430713, 0.34926097584118532, -0.35815861799291304, -0.83700114852955054]])
      arg1=Data(numpy.array([[0.82605893992853674, -0.43129145518369238, 0.42275995124864907, 0.79933162121258117, 0.4565204718350151], [-0.43642891929309124, 0.32815803774379337, 0.88547286455610918, 0.47079208955831264, 0.729559199963703], [-0.35223043377128738, 0.4159016971778966, -0.38353234059288832, -0.31502231691509142, 0.31931352402769808], [0.98254332028266433, 0.34217713422490603, 0.7072477440265148, -0.83027415700191431, 0.28019515667625261]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-0.226903257331,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[0.10241974677245125, -0.5612793405960681], [0.48956883482708147, 0.21824369883334827]], [[-0.94208184708149623, -0.051386571005039627], [-0.91342350452431909, -0.25978647540698407]], [[-0.5951270487657907, -0.27571085909871629], [-0.73723013197562359, 0.317886957344657]], [[-0.45037371104994639, -0.029486570998041861], [-0.82789139856903593, -0.94832948730039912]], [[0.27098807734732322, -0.89513169154431971], [-0.24930925111402669, 0.97881744824711525]], [[0.08702563256538931, 0.23178250733209138], [-0.67548328336784969, -0.36820646116643152]]])
      arg1=Data(numpy.array([[[-0.31373964435579982, 0.017883438358496484], [0.6888236499822904, 0.61142611013031667]], [[-0.03634678500533739, -0.63105984256155212], [-0.25953474206005089, -0.016239653369635842]], [[0.24128973568149914, -0.21787692878990894], [-0.57616711332523507, -0.97817129188864271]], [[0.009337648257052189, 0.49174072868617746], [-0.15474415928388274, -0.92936833360630899]], [[-0.70978657661432498, 0.87557995989779003], [-0.89720836523803316, 0.67329579802870576]], [[0.26372536245876654, -0.76542154521883288], [0.58655422134985846, 0.6979828223763247]]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(0.856438930143,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[0.016639529837428668, 0.84838979094454325, -0.50467114288267378, -0.54181350086674529], [-0.54620830647663876, -0.31499053555167711, -0.48094938871622994, 0.27646333695070058], [0.82724160718728901, 0.1737915495866984, -0.34238117387678146, -0.60400537575240221]], [[-0.36746819329091229, -0.64176561322584069, 0.52245880667087774, 0.73688903719870424], [0.3179495016926901, 0.85279408078258245, -0.60047774438704749, 0.15873638993823813], [0.5076892764693155, 0.84293811414455178, 0.48925585544262318, -0.82237799915319121]]], [[[0.25242600532246828, 0.98620415816453733, -0.67524390609349005, 0.51688560582684695], [-0.69177865939521888, 0.63232405584020346, 0.0023024450747841652, -0.79237123558851374], [-0.75432092404493756, 0.79156821576387371, 0.053607107332242165, -0.95012768912242951]], [[0.88260562556496636, 0.99069034151790114, -0.53215163846733593, 0.3971989742729396], [-0.85212374540238023, 0.50628972942019668, -0.29218975010767423, -0.90647005705651607], [0.13419889681450714, -0.68500671786361833, 0.18920196138340883, -0.16974207058474455]]], [[[0.61164050051299412, 0.36642578193074971, 0.37341536941942066, -0.82463585738792067], [-0.08223679232670289, 0.0040037760019404622, -0.016299163145779616, 0.84470568309483696], [0.75150985970177553, -0.1990527406518785, 0.35874622129606437, 0.22351265216298954]], [[-0.10960282976903146, -0.035365424124620626, 0.91643455408768437, -0.32048481948855656], [0.59775606736847942, -0.60614780477219266, 0.71467667645915967, -0.93549262644917519], [0.41148771993312305, 0.17822720261692204, -0.92517551904060591, 0.23574681120190943]]]])
      arg1=Data(numpy.array([[[[-0.52344546615220522, -0.45296337582365509, 0.63184857984224196, 0.42541233474254758], [-0.23897433488742448, 0.71055269293239243, -0.46634951228173249, 0.076642826379174744], [-0.97484132074029883, 0.16864792330825473, 0.95635345134512728, 0.21669053973675578]], [[-0.9069669442268995, -0.21691444531397885, 0.15933955144867706, -0.81348324723411847], [-0.31275687451151102, -0.34212629791189086, 0.30384255829756257, 0.55895935486113646], [0.85216976110340892, 0.46078915037373602, 0.91937027831781348, -0.3482307708329897]]], [[[-0.45307960753759979, -0.63289481149081328, -0.63520551311755002, -0.92739929533503007], [-0.82080462816910016, -0.063643912885194887, 0.7430471470832356, -0.6508836953472783], [0.10170220900992355, 0.77593531995384457, 0.91230159203931382, 0.78425954312077284]], [[0.6357266342050405, -0.12466997751467157, 0.32490859933816507, 0.1101144072176754], [-0.23463819036749256, 0.64140028824878947, -0.81732691350469833, -0.25015974677132657], [-0.23974953965755108, -0.82125675953865751, 0.97770789947702896, -0.03498443568673304]]], [[[-0.077955029958820754, -0.43754142290157572, -0.71454906310952526, 0.33099228123870361], [-0.57320856346297955, 0.57803655157871647, -0.46331816432125361, -0.27964923157478383], [0.25214848873585183, -0.14481715104089021, 0.11289539377069313, 0.089390090920334897]], [[-0.91410123483641836, -0.86128585400035518, -0.92255914908006731, 0.29388782909507194], [-0.27855479268337269, 0.030076820585910991, 0.98119952695768076, 0.75737484900128438], [-0.31289264565854547, 0.04614406591056075, 0.68094161393432828, 0.28036598839232596]]]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-1.21336440422,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(0.685017565314)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.074549723403)+(1.-msk_arg1)*(-0.879253747732)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.051067870020334076)+(1.-msk_ref)*numpy.array(-0.60230426156491856)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([-0.33806363855530708, -0.38292988645561055])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.11856789813520696, -0.13776690018680293])+(1.-msk_arg1)*numpy.array([-0.34564715796422996, -0.069488375244919798])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.0126715683864)+(1.-msk_ref)*(0.14345991152)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[0.8235093234178823, -0.039529146873249488, 0.063372460107542938, -0.02619523847089944, -0.95775695528662586], [-0.34622913082399309, 0.65703970311335413, 0.13991122080937823, -0.72993544844032421, -0.61110494663001846], [0.94702026611810308, 0.28367545790239079, 0.67686520536412309, 0.42751460713834732, 0.19113835646707988], [-0.75397238968468683, -0.20449122930723362, -0.21008054171449841, -0.13985634479509779, 0.64669654227550311]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.52739632868989683, -0.36965406751101404, -0.39081726084476509, 0.97579405276136999, 0.69376649986660421], [0.778547720225927, 0.25692204796805784, -0.57060516104094372, -0.41495750688346966, -0.25648389392393289], [-0.33884072596680825, 0.84308494012737856, -0.52829054733196146, 0.65321147965010384, 0.48804191576280909], [-0.24641018217509747, 0.087863580087031101, -0.56288330676632148, -0.6022514263107599, -0.63532666639301216]])+(1.-msk_arg1)*numpy.array([[-0.14157133127819188, -0.54776353090035768, 0.81467492509210548, -0.62049750034243933, -0.37900634079761653], [0.82176313795168232, 0.42864041831927735, 0.24199983631564392, 0.41189660175178577, 0.40908911745866505], [0.030519565802787252, -0.68547715815876176, 0.14694228902994233, -0.16803721604883659, -0.53884356711170156], [-0.36084080845702471, -0.2746282620691598, 0.63989515977938782, 0.26879023564652971, 0.44513874904187345]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.0941431468163)+(1.-msk_ref)*(0.0194138324902)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[0.6807678093224121, -0.57723267525409039], [0.70068737825030447, -0.098432327342198533]], [[0.098140634624181322, -0.54424338697813068], [0.71804851090573774, 0.29950212330472437]], [[0.42465927843511464, 0.58260173822051264], [-0.40034392645695749, 0.27714450388518275]], [[0.18874402904939913, 0.062894210730298328], [-0.83705822051765666, 0.096190962372082689]], [[-0.75297233787755746, 0.45002414314126327], [-0.72288520216827656, 0.70912440219389161]], [[0.14137295031492436, -0.34614151067367471], [0.93935035354863272, -0.83373700012705565]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.10440843510343933, -0.89208525680149098], [-0.62476879751924952, -0.77336672258946537]], [[-0.16279839471814728, -0.0072511576320637428], [0.39123164448829528, 0.29096390323811661]], [[0.43544714395872375, 0.19856771196494227], [-0.61262553243481022, 0.92741953379021758]], [[-0.044525429250246829, -0.89645569569903594], [0.60174376712715039, 0.51259492479278346]], [[-0.77538544226011674, -0.68714118832869286], [-0.40232551304269637, 0.50588918195801913]], [[-0.95544279028286105, -0.73188009487106576], [0.94088511797831398, 0.42873552842727891]]])+(1.-msk_arg1)*numpy.array([[[-0.22185702833951537, 0.60513168028384201], [-0.6423119626413778, -0.58036119662568053]], [[0.30081297970455267, -0.6070556981649895], [-0.90233647395001904, 0.62817784677024058]], [[-0.52044268719947984, 0.11509410080733606], [-0.75548193826004817, -0.60670270362377821]], [[0.22643631677427467, -0.45635249547105694], [-0.32459092263105682, 0.75218467015134483]], [[-0.045797508987332813, 0.52811472165708517], [0.87346978951632415, -0.44939888990809318]], [[-0.17980683491748972, -0.72256211726739084], [-0.71437002134646899, 0.82199165311469602]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(2.29079134711)+(1.-msk_ref)*(-2.46432742627)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[0.15001435572408761, 0.88232864246715459, -0.24610469990288886, 0.67560609162045515], [0.86401150897253132, 0.43421666202257314, -0.86140940673783661, 0.32139301112348884], [-0.45383172852577358, 0.52020632092451913, 0.81512898306376735, -0.9742734405822282]], [[0.98066673304755825, -0.94023788286751175, -0.40227847101427572, 0.35551124176897919], [0.023933521307637706, -0.53811922182304706, 0.6913422958258757, -0.84406449580488818], [-0.50650935800253394, 0.9426192897711656, 0.48842047540240663, 0.94440029359694688]]], [[[0.38346320727782701, -0.87406936246287126, 0.68998833790835068, 0.1571712550522113], [-0.33552688217408133, 0.84039147835532146, -0.87599849362515503, 0.47001164445580623], [-0.91299234258139927, -0.17750683498323405, 0.36632349854464397, 0.70016097477658779]], [[-0.066200692553295193, -0.058261135948909137, -0.22814373144827504, -0.64926256398498539], [0.26196442127305009, 0.64998457480300531, 0.50294142170873934, -0.22074066985171492], [-0.28710043984106881, -0.35115422025792342, -0.023340121762502664, -0.73626684924280084]]], [[[-0.63319886833240191, 0.4626606248706171, -0.45536290831045778, -0.6161109835018006], [-0.18669620353738425, -0.34302502854744232, 0.09082321884037392, -0.024033185577168048], [-0.64389952822168084, -0.62679870762489798, -0.23800700726467983, -0.93865957831833335]], [[0.32502225087125924, 0.28665512174603269, 0.29368673585583949, -0.94801857817809299], [-0.94387554673357887, -0.86999806926650147, 0.23024162709965412, -0.84247401709034264], [-0.26926348184769733, 0.09437686600404227, -0.99267356825664055, -0.69396226836589103]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.69982398293267623, 0.41309680911744429, 0.22295573078252606, 0.065367899473801172], [-0.42418776970603456, 0.72498482032837264, 0.58244790217178544, -0.46788783378843202], [-0.16876136986509649, 0.25776299059074637, 0.031694131873162279, -0.37723534572713935]], [[-0.33211559209104569, -0.23554524976887059, -0.60874446346290112, 0.18836992215618942], [-0.97274913731614254, 0.17533771183253855, 0.76831525607694795, 0.36098529286068604], [-0.78559144936195024, 0.85956422754002038, 0.58705783447497617, -0.75059133979061365]]], [[[0.023467803523911757, -0.22375429863866336, -0.83149004680439464, 0.79185129225030026], [-0.63331631885344386, 0.56090673973895022, 0.76276329432083534, -0.20867007167164942], [0.14833356982262336, 0.52730370024687101, -0.09927133106555841, -0.10049391484766845]], [[-0.1045637502068355, -0.59290766899230785, 0.81880317570587868, -0.7594690941697726], [0.50563811016995319, 0.7841853659235094, 0.52355731289255059, 0.012124560297168907], [-0.4414905530370985, 0.37628519305253927, -0.59792951426456353, 0.62279388927523338]]], [[[0.52498756740874608, 0.85236651782771622, 0.28393668963979235, -0.97371138479150177], [-0.94199870660473217, -0.7761222781471373, 0.28133263444008993, -0.062866327413718448], [-0.6517057507208186, 0.65491103442817566, 0.59299834678865948, 0.82740874618675231]], [[0.44012931494554919, 0.99719641511647206, 0.9149812057157205, -0.99082386522433707], [-0.56711688982673825, 0.35833134825795643, -0.20106138526283757, -0.80539330297231193], [0.97328132737404571, -0.58869057173184602, -0.60198505227481314, 0.15998823045583488]]]])+(1.-msk_arg1)*numpy.array([[[[-0.3906025329404188, -0.66797630599954649, 0.55626332612710394, -0.457068692078344], [-0.78410812521096473, -0.49243638570973625, 0.29841020790238093, -0.13552326940959891], [0.5302429043181327, -0.16736605349594891, -0.32299170682193212, -0.87075419197777704]], [[0.54629425419275934, 0.93422482754651637, -0.026459574386814344, -0.34680953003122483], [0.29185098278590171, -0.53003159945694334, -0.12574506159971754, 0.65975689963329986], [-0.71584634254643165, -0.19571211281268908, 0.46323808195948635, -0.49723992236554437]]], [[[0.027689417062008559, 0.022300669629664371, -0.84390270292480429, -0.505113378158903], [-0.54750825000226544, 0.94307048396077531, 0.3108339975708474, -0.61172090959047609], [0.43765690868955454, -0.7275628428159473, -0.3370827804832035, -0.059131771918396803]], [[-0.23280105822411001, -0.79969955562783657, -0.81404675045904007, 0.25135673576902229], [0.55826423982562923, 0.64622779798664243, -0.16536107142796919, -0.53329031234256563], [0.41394958803272464, 0.10216346250295305, 0.18870302903326386, -0.58477587277594778]]], [[[-0.63316090072805942, 0.13981387221419306, -0.71451383537974622, -0.0020177533913716683], [-0.88954622708813003, -0.95431668093249011, -0.31610740921578206, -0.78120894488186576], [-0.50298104954776512, 0.33848149773071357, -0.12181804287981279, 0.76299698123850668]], [[0.74324353622072326, -0.56716250169963001, 0.39532337596723721, -0.60851190458351456], [0.2290837754652173, 0.48925543744198086, 0.22910725686311295, 0.59838985543456413], [0.36495477921428865, -0.98765458011552831, -0.57313136561592271, -0.10846619791113143]]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(4.14398942825)+(1.-msk_ref)*(-1.80134112143)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=0.632900201456
      res=inner(arg0,arg1)
      s0=numpy.array(-0.415595086573)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-0.26303021401605353)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(0.999174947457)
      res=inner(arg0,arg1)
      s0=numpy.array(0.179761656912)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.17961334409989607)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([-0.89586680198608448, -0.5329978260854904])
      res=inner(arg0,arg1)
      s0=numpy.array([0.40951182464381075, 0.10116422779058509])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-0.420788362209)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[0.82915949739164163, -0.67570692643852426, 0.75218979601145652, -0.50092542241303839, -0.97701534673742074], [0.72468477729664316, 0.044770096307641838, -0.52743237074001836, 0.66073248431585263, 0.30257629217964599], [-0.60601180259608856, -0.58211315041099954, 0.68235941118894128, 0.14667286548249048, 0.23840475278172502], [0.075215330753106713, 0.26305457075387761, 0.26433874834934534, 0.56687137963963474, -0.1591025565710984]])
      res=inner(arg0,arg1)
      s0=numpy.array([[-0.16734958352282581, -0.10149615186247174, 0.96695224441165717, 0.3831490627081815, 0.066620776151524952], [-0.86867376532633922, 0.083238057363643758, -0.42880513340587889, -0.63921541496779399, 0.75985548113897661], [0.89634316071057807, 0.088681455440007939, 0.79584642939613115, -0.92347046094434537, 0.44857381260040063], [-0.22580773893185047, 0.63003986671384404, 0.86545403738101223, 0.90136746877644258, 0.94917607092307033]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.465273611817)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[-0.3742838621009954, -0.34024870084457293], [-0.8967489719697368, 0.010633330369861582]], [[0.10563522268305081, 0.14893846708090375], [0.35099571301070842, 0.49403607159770901]], [[0.69858426705700261, -0.41226551244410303], [-0.58305928721088862, -0.42922286160560374]], [[-0.5386541191810752, -0.99774294227342897], [-0.046478481229671464, 0.81376473186054876]], [[0.22175359329385302, -0.16041330028462775], [-0.14487588115345829, -0.10263747494444986]], [[0.57320856721576252, -0.85284752014658793], [-0.94322124174624244, -0.15115716592628381]]])
      res=inner(arg0,arg1)
      s0=numpy.array([[[-0.33793978529193724, 0.74123089467334857], [0.29545011266112686, 0.34624550290809775]], [[-0.45512929811837788, -0.91024661040613108], [-0.5200563320761562, 0.71020926586160282]], [[-0.21095700837536935, -0.15580525235259124], [-0.89515575124269331, 0.13859093927943289]], [[-0.71714883439749633, 0.43791667133688628], [-0.92832042339506282, -0.90546237863971779]], [[-0.5219715681100674, -0.87695138344919088], [0.70101819328786918, 0.64236500674875208]], [[0.75155326127522737, 0.98388216733025358], [0.59060723822120886, -0.47209909129118954]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-1.80389499717)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[0.47666934946418626, -0.88214710258603213, 0.3636510088395386, -0.19441886669958097], [0.70575107980593454, -0.65573730618414361, 0.26544004523743681, -0.85086612214343438], [0.55134232652814341, 0.97502824059544491, 0.2719855508001221, 0.5347895091858752]], [[0.90960465505298282, -0.35710565290405905, 0.41454246141397499, 0.26873843718307788], [-0.90985544910537253, 0.025433449771046224, -0.53688990806797143, 0.55844243141763172], [0.8579979326131435, -0.79223150833738365, -0.094695737606996788, 0.99699279840135735]]], [[[0.60099384398637579, 0.60985046540422272, 0.89502856413376986, 0.46537781986512705], [-0.060308132182928853, -0.4671280687428665, -0.80017301116860828, -0.5652427861665319], [-0.034866355344519651, -0.41404694851228308, 0.15047102117195066, -0.65018991024402673]], [[0.25108809110562391, -0.91493293278251264, -0.91464574407133248, 0.5102632657525874], [-0.3445737196653067, 0.86264270015762778, 0.62904776472261381, -0.55359153606907618], [0.43302789096157812, -0.91957919152987389, -0.074775957544543026, -0.38308705772296214]]], [[[0.60162928599114496, 0.79885806191599618, -0.79337594878821083, 0.7081564258191646], [0.4538694100093601, 0.42897242810134029, 0.85639986714763694, -0.48475717056198642], [-0.58458176147906338, -0.31525210564145234, 0.86730735788313429, 0.30413722759701955]], [[0.46777289889353502, -0.95870799607266388, -0.86207032987295507, -0.92878805941032638], [0.86537821294426176, 0.81112621864674694, 0.26005210872768125, -0.78135714141956969], [-0.32339691490194911, -0.13696718837735444, -0.41555966629715835, -0.064777040410046061]]]])
      res=inner(arg0,arg1)
      s0=numpy.array([[[[-0.68401529532315464, 0.90556502224450908, 0.48457224187655901, -0.77894679239637665], [0.98522777839839604, -0.60354824284671116, 0.49065541812686697, 0.52666518392913564], [-0.67522480580272082, 0.1478160106107278, 0.13476953370390632, -0.72042768467991625]], [[-0.96162085605835124, 0.75160429743818602, 0.69697687580575374, 0.50621367498927783], [-0.42421790872200082, -0.18211384248830376, 0.70846147315852592, -0.24759685176012969], [0.071826829821121319, 0.69584252433706562, 0.99192970878845244, 0.25286171321878248]]], [[[-0.83749448587272113, -0.98062422740868205, -0.70827061467238428, -0.12139047733490904], [-0.17202101827519689, -0.27455132136970928, -0.094150764948197185, -0.12057516398673851], [-0.57297916590968123, 0.70513719849205425, 0.33334212562777488, 0.61854220862466236]], [[0.6537213164119009, 0.62690553714607855, 0.63391724736003874, -0.14436430195570016], [0.43081222750530945, 0.89031578618300045, -0.50907976600363147, 0.9260276160691876], [-0.80842929040163991, 0.10502857604915294, -0.38104950466739163, 0.84540645189580199]]], [[[0.50178366499302829, -0.87089038463486212, -0.17378829682616792, 0.90081161719622549], [-0.32661850240335188, -0.79988436725446554, 0.0063366318941258104, -0.53110715255828311], [0.11242639060426063, 0.67787609101010182, -0.00823335147505877, -0.56791051717160079]], [[0.85498533626192219, -0.30165229053871001, -0.78535693924039318, -0.49028232989097198], [-0.5380742604888975, -0.17626719841525973, -0.12024049586203223, 0.63065460766273196], [0.60056889335149388, -0.42963647480414702, -0.54339793672977188, 0.77104222992950699]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-5.51600565928)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=inner(arg0,arg1)
      s0=numpy.array(-0.873400339604)
      s1=numpy.array(0.855330697719)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.74704612186077646)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=inner(arg0,arg1)
      s0=numpy.array([-0.37064115965122846, 0.42964982058714885])
      s1=numpy.array([0.39060232148933216, -0.12345536636629384])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.197815873409)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=inner(arg0,arg1)
      s0=numpy.array([[0.9348864493378346, 0.83569467575517264, 0.14607913752316226, 0.032948460405173963, 0.89420522049356199], [0.21113582348988169, -0.045072336294261994, 0.67731302934206283, 0.68096532942090215, -0.78467027199106276], [0.39016278298896334, -0.87047882217087835, 0.73558927863011925, 0.90684444163575284, 0.6094790511869248], [-0.85279584633671179, -0.61076625548870656, 0.61960595397028051, -0.5407467627922804, 0.03601230387135379]])
      s1=numpy.array([[-0.29463256857987941, -0.58118207813979428, 0.20385968783055386, -0.99026066196642715, -0.074650680885644194], [-0.51896049796030086, -0.85822896953027761, -0.81585062417638898, 0.47043371942002499, -0.75396574734798349], [-0.90177277931853794, 0.77808856057449272, -0.051352063785525592, -0.73363300121363983, 0.08184391481589337], [0.76752589555435158, -0.15324639774452198, 0.66434410198978511, 0.4423048434542427, -0.5166493348433443]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-2.63167610151)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=inner(arg0,arg1)
      s0=numpy.array([[[0.45476749798930705, -0.033912454220248467], [-0.83384311092554664, -0.063613100777223597]], [[-0.44997425194098328, 0.26052583479965619], [-0.071949218095757939, -0.49348872656756471]], [[-0.50556703845734541, 0.06251106243202309], [0.11068299468145804, 0.1847496512300395]], [[-0.45082954291927724, 0.161858965346243], [0.1527727063510349, 0.4363194729076032]], [[0.44530347029786355, 0.50656201028453807], [0.27127337437017873, 0.38638316385496663]], [[0.21592859530611497, 0.27749017090164085], [0.57988022129892558, 0.065956058905883186]]])
      s1=numpy.array([[[-0.17617577620932057, -0.94575560608910969], [0.53088690347224921, -0.38402042765434685]], [[-0.70180078704757576, 0.97047493736578661], [-0.32425969978341951, -0.85777490997105699]], [[-0.53136786934491065, -0.021922916569488748], [0.084927022056330514, -0.54874543853406821]], [[0.73926229239145935, -0.89711851362440354], [-0.015268866214127819, 0.48078033366936923]], [[-0.85914091799056336, -0.49277459361156772], [-0.52085627931046341, 0.84060963382393705]], [[0.19112806355965017, 0.63496830282036565], [-0.93800953393428022, 0.90879407694622261]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.262012493921)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=inner(arg0,arg1)
      s0=numpy.array([[[[0.65307563149725878, -0.06930447643128379, 0.94229289108358572, -0.2903387731204683], [0.38759539665166054, -0.59039123894913015, -0.22461720486431735, -0.18042318396225077], [0.80748948600701231, -0.85662395836275196, 0.93909220516936043, 0.89470348639530695]], [[0.019706795127593546, 0.70684634920783318, 0.025954512488624681, 0.22272698829445137], [-0.99047171333386763, 0.36683522843014038, 0.29366966924771232, 0.60243755421928813], [0.61961009894058927, -0.673174510107031, -0.18710522911259009, -0.54402271390273627]]], [[[-0.43979613106186299, -0.59204897544682766, 0.73276162836251468, -0.73385907684266427], [0.44175111893953622, -0.46522123773455126, 0.30862451466144036, -0.95524506792267072], [0.3509254436208602, -0.24766257484748833, -0.46358994494557804, 0.79944572050208818]], [[0.43958686152765303, 0.14290989188090197, -0.15060850245206447, -0.90115617309598339], [0.4771415365091165, -0.32233133341301556, -0.010671212430992227, -0.087580424907550469], [0.50084615210161498, 0.24320024705013821, -0.9855125225069552, -0.19729384047476306]]], [[[0.66497450062395846, 0.83225645006452242, 0.99451387986805462, 0.010368081065823098], [-0.38584109800059929, -0.50721988161009701, 0.65602368368740338, 0.31134297441922243], [0.33613665308772278, -0.99552836597211658, 0.8807919840673637, -0.06711885679162588]], [[-0.37798022536764986, -0.11570476004966479, -0.45480935395410893, -0.2464644455596654], [-0.18844950270214866, -0.5067378072008375, -0.14111064314790656, -0.56991693839863822], [0.55942265575348826, -0.093635370457385125, 0.59361699953708924, -0.27482141974277452]]]])
      s1=numpy.array([[[[-0.24582605859789419, -0.57770069368163046, 0.76247275911808199, -0.60476898916323685], [-0.15371226667885729, 0.3593075115734039, -0.83830787691424002, 0.92449957224060064], [0.62193577730931282, -0.26864932213443549, -0.056969365374245129, 0.25092017174033776]], [[-0.21656464786277341, 0.21385957822178292, -0.1234542114973014, -0.79955326189559406], [0.62564298598981805, 0.46520904061419688, -0.97050448660222965, 0.67879944027120809], [-0.43113917477274377, -0.96261427957364987, 0.84568905095285873, 0.56112871213219795]]], [[[-0.85048485851094235, -0.9483546466862256, 0.21037538529474942, 0.5006926743477258], [-0.15006492066882915, -0.76369379646688174, 0.10010391078623671, -0.080776976250223953], [-0.39769017868169332, 0.07806375915760011, -0.33362041098245898, 0.6489838379357511]], [[0.51784530877063628, 0.8985672766657411, 0.44072181425195955, -0.91271560952279462], [0.49861779427777564, 0.079242885867201185, -0.21454557347174097, -0.16822134400011257], [0.19482951914314528, 0.11091877020537666, 0.73416551137210373, 0.67802401198278539]]], [[[-0.31261217652417228, 0.46407867428866623, -0.39416379610904628, -0.79895605924811752], [0.49550848332229003, 0.54630671582562074, -0.39309234622871236, 0.0030272662784036619], [0.72992127580354293, -0.95482332518794921, -0.34314554153257926, 0.73620389433399636]], [[0.27703115875029449, 0.65062187417403461, -0.31559746571963765, -0.27532340212163486], [-0.80753568903972472, -0.31580507555133819, 0.79642798106375845, 0.56809090389616723], [-0.32118828443085068, -0.12863182118467198, 0.3549535632774401, -0.28220542778634816]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(3.15245280086)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(0.362364640086,self.functionspace)
      res=inner(arg0,arg1)
      s0=numpy.array(0.801138493302)
      sub=res.substitute({arg0:s0})
      ref=Data(0.29030426178454793,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-0.26160322063239949, 0.29380978673155278]),self.functionspace)
      res=inner(arg0,arg1)
      s0=numpy.array([0.70198003865731229, 0.71755868814259705])
      sub=res.substitute({arg0:s0})
      ref=Data(0.0271855261981,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[0.33753196386256268, -0.075592264596325176, 0.0034014916151035024, 0.73047030921646439, -0.13836394942740249], [-0.75032819668330597, -0.31244399022238567, 0.49983550796993192, 0.78748112229717826, -0.51361366209773962], [-0.92715175285905604, -0.28571732142479234, -0.75893540522917147, 0.020491391174954465, -0.049652508255737526], [-0.52286003500776435, 0.027564726995311428, 0.7246872830129214, -0.55823005787793734, 0.25352853665037056]]),self.functionspace)
      res=inner(arg0,arg1)
      s0=numpy.array([[0.7615759137740552, -0.91698934091417206, -0.68238655862914288, -0.74211709691563388, -0.67286456878033962], [-0.42247680604434379, -0.41168479223306953, -0.36749214654691209, 0.44506759869320733, -0.9755455559943389], [0.33431467509382329, 0.0013638106967346353, 0.027299592897490621, -0.58099122346857124, -0.022312876787589309], [-0.35804911162818764, -0.011836862110518043, 0.60311962348644998, -0.47039276042226108, 0.86501193620027483]])
      sub=res.substitute({arg0:s0})
      ref=Data(1.7525158532,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[-0.72617085074747956, 0.091642062127776702], [-0.96126170772411546, 0.49350424069568022]], [[0.39356458058441923, 0.50418782231660542], [-0.020966789765773486, -0.69618898046426758]], [[0.33967472406099675, 0.15989568286279798], [0.26907207171673986, -0.68003192972155047]], [[0.97835413211956701, -0.46207879257845996], [-0.47025739225504881, 0.16355886773062811]], [[-0.044820979118111115, 0.15347654316787374], [-0.54701414416571437, 0.48994732061716117]], [[-0.86694925948467794, 0.11712320805927789], [0.35840563438566186, 0.56544172176790219]]]),self.functionspace)
      res=inner(arg0,arg1)
      s0=numpy.array([[[-0.39143196418521464, 0.12748751080363441], [0.71713896374984865, -0.61198416714073933]], [[-0.93698374545980401, 0.90484517057834535], [0.58540710315754096, 0.93884768434675459]], [[-0.8539150523275858, 0.76865244328665083], [-0.18765121712574961, 0.16759904638309053]], [[-0.14289541487774948, -0.5539376817281505], [-0.14397946675807338, 0.76705372132464444]], [[0.54680816524455866, -0.96495833412679644], [-0.51299054633570984, 0.60334550323825997]], [[-0.066319636165013351, -0.12587489416965791], [0.025701671525855119, 0.63654158498969449]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-0.480668366973,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[-0.63826228295294785, 0.59278203369232174, -0.40294287173934573, 0.49916436326001845], [-0.41031835664182559, -0.32396177537462467, 0.73013655835247504, -0.80225526497679578], [0.80421918201541476, -0.22900114798374327, -0.68150671436557597, 0.1399711698065158]], [[0.91091344866606661, -0.72222551607688401, -0.49288803784377544, -0.54266739117950102], [0.38403387418160895, 0.56440196196844683, -0.70616037464977621, 0.022883763955821257], [0.82867284454440671, 0.55121098584649375, 0.1091244585578437, -0.0017222779972809743]]], [[[-0.61084340583425711, -0.4439480681993544, -0.8323890451698126, -0.64841483620669216], [-0.77629535944592054, 0.3914025633446756, 0.60823425333857761, -0.038114132418143321], [0.90865442827496845, -0.58039447193456906, -0.27955163652682313, -0.41261139599255547]], [[0.99415887957984417, 0.34578890816771457, -0.19844612458813438, 0.36505756757401664], [0.59585085936570459, -0.58930183443517836, 0.8338725036323249, 0.79048333880559718], [-0.11476885295688311, -0.29163168576125864, 0.49482369527891978, 0.95311043605169976]]], [[[-0.7143192101983078, 0.4482688324265256, -0.21654780241686855, 0.11135785890560523], [0.67960753322875567, 0.82171666644609664, 0.82911699164924046, -0.93783799663140854], [0.81064430281008315, -0.29549304948450761, -0.27461800284033044, -0.86621139483899667]], [[-0.89240786091737423, -0.1364271579578753, -0.96492498143118377, 0.44443398013784852], [-0.4995810639319298, 0.19606815164274316, -0.079628286298303541, -0.79779295597326438], [-0.96061780278675202, -0.10219884290790593, -0.7655814866922952, 0.27353015465521402]]]]),self.functionspace)
      res=inner(arg0,arg1)
      s0=numpy.array([[[[-0.20908371290983441, -0.74434733384973573, -0.13311657849960512, -0.73053577563972083], [-0.52302448520761469, -0.17943366612993428, 0.62023771057281429, -0.8745241002723747], [-0.52809146826066877, 0.6350855002676663, -0.27301213697715832, 0.69512418953084532]], [[-0.42381615505653136, -0.58620391643884706, 0.61769200542870317, -0.58561914480266819], [-0.92255547019551876, -0.66652937710823901, -0.52556789705674545, -0.76673997455359499], [-0.76927995031844931, 0.64847058811101599, -0.27113898264134417, 0.3363474505824513]]], [[[0.43609105376451729, -0.77423584073215435, -0.91561453854390296, -0.53498896514292449], [-0.89348193080194238, -0.77231183413474369, 0.36087361648250149, 0.5892759750005705], [-0.49988303142104695, 0.24714830902514429, -0.56785989577185503, -0.66819181973688857]], [[0.97055503831638368, -0.04595828760358045, -0.71158103015664964, -0.85873703164654369], [-0.97155386059927018, -0.71415139361441993, 0.44929069574789349, -0.30074810892662041], [0.172117624820876, -0.04072158743082599, -0.89725374175447681, 0.42413090171336632]]], [[[0.26588143015939525, -0.97954161168525555, -0.092362534381466466, -0.16729224088362327], [-0.47865620264672715, -0.79052182009125738, 0.29713246131729343, 0.93546449404270837], [-0.73518170338125044, -0.10275052706112686, -0.24539516894025404, 0.65126602759389396]], [[-0.95436865060926301, 0.96710060287042454, -0.40716838168706482, -0.59025210770459702], [0.34216597920763214, 0.17880973837263969, -0.6251265790469267, -0.44899622157774211], [-0.36015934094525193, 0.23340156370488918, 0.33570572704808344, -0.15011719964717485]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(0.0552601251612,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.0096536093987)+(1.-msk_arg1)*(-0.807479043031)
      res=inner(arg0,arg1)
      s0=numpy.array(0.787551832124)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.0076027177685556039)+(1.-msk_ref)*numpy.array(-0.63593159974110891)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.28302771665453319, -0.12400321816607685])+(1.-msk_arg1)*numpy.array([-0.56715181692414496, 0.81309255762289268])
      res=inner(arg0,arg1)
      s0=numpy.array([0.2452450322490336, 0.37138122917584049])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.023358673914)+(1.-msk_ref)*(0.162876147852)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.48901241595372991, 0.75048250811230766, -0.89579868180462641, -0.24069264467689688, 0.22983184326811346], [0.25335620221819855, -0.8787149262118763, -0.63780964560866682, 0.89853779792772981, 0.66595107386478225], [-0.86276901889328483, -0.84510973458994698, -0.24255772379639762, -0.028225975075878429, -0.99554896674325133], [-0.1690927278286023, 0.20173758805014641, 0.51501869278388979, 0.82590306860626383, 0.29565341435925796]])+(1.-msk_arg1)*numpy.array([[-0.35323157913709724, 0.44765664801559124, -0.074858955846049513, 0.67546374398615039, 0.6351209911772826], [-0.76719364803361811, 0.27279635052704987, 0.56656632351012504, 0.41410803682128661, 0.24064679268219935], [0.32469268630501968, 0.3282711805198768, -0.19265378774709441, 0.064888309574934189, -0.28014825158217227], [0.27440770377713952, 0.22894641107066027, -0.43167956346323377, 0.12381352247693411, -0.335824491798969]])
      res=inner(arg0,arg1)
      s0=numpy.array([[0.063545481895826539, 0.32387048132990093, 0.36704512969217351, 0.89698453094584307, 0.15215907967477782], [-0.75599379718429915, 0.12448898431699029, 0.92536221358410176, 0.47585166528332956, 0.18445183370839535], [-0.41635346292083941, -0.78474928996799509, -0.28992641368563454, 0.82933547125242169, -0.20905759132714485], [0.87375203401832557, -0.11316884379198, -0.97234834478057475, 0.01187194914800993, -0.53735002126232922]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.11927494379)+(1.-msk_ref)*(2.76821645165)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.0060263683351180486, -0.9494238108460038], [-0.24810313594963529, -0.74841895706578709]], [[0.9409094009506731, 0.30546531012053135], [-0.77183940627226866, -0.55642323152201545]], [[0.82847311205979746, 0.69908332067646661], [0.36677054841154444, 0.080328116067538868]], [[-0.66872818242149634, 0.25993409180599691], [-0.31396775451165015, -0.18560853288587209]], [[-0.59191940342132732, 0.52276303318531836], [0.75932071224341979, 0.9254361623214733]], [[-0.82617643655035522, 0.78901061108299642], [-0.726373397995284, -0.67362896762476443]]])+(1.-msk_arg1)*numpy.array([[[0.74143311186563254, -0.1911732596260638], [0.95373934558002293, -0.88430875451655933]], [[0.49959045086996734, -0.46820343802857356], [0.63723726563142491, -0.095927275910457022]], [[-0.81332847619979476, -0.32934965533279681], [-0.2502700893123968, -0.13026434790516972]], [[-0.90823945594400279, 0.1984343359080476], [-0.12167838649534013, -0.85669616152861905]], [[-0.24956253867832734, 0.17448521380075199], [-0.16370701063884141, -0.29476537604056263]], [[-0.98397008656811802, 0.85013057469763464], [-0.55379432633417203, -0.41850386098474979]]])
      res=inner(arg0,arg1)
      s0=numpy.array([[[0.7894922445362047, -0.031149169873611049], [-0.30233390667964932, -0.58807156303070718]], [[0.46842393267289273, -0.8007302092413684], [-0.83252410841201785, -0.18155812825981665]], [[0.86806072022991421, -0.60990831205843365], [0.0047320749275556739, 0.65481544748241816]], [[-0.23001555898515091, -0.085528817909554578], [0.59196550636234146, -0.48677791101692636]], [[0.75537850775303439, 0.57609803379890678], [-0.34101884053986153, -0.027607920142951548]], [[0.89015028360761428, -0.97342623957280927], [0.10554834240929156, -0.58692889338682552]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.247686640643)+(1.-msk_ref)*(-0.676143963147)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.29540242927318161, 0.58985024242657236, 0.40806256717382028, 0.46696865417528532], [-0.42457561574030711, -0.71866300793665561, -0.00064199152572053109, -0.089293974392212538], [0.77065659912126394, -0.46514024792288944, 0.42857842322284778, 0.050490399507414629]], [[-0.20211793444601756, 0.22942511757316697, 0.51133064504785386, -0.46886063393462263], [-0.49161131879921127, -0.87159025314005634, -0.82898998743049757, -0.6910577407328371], [-0.72790654027343837, -0.44329129201343664, 0.84324666219247923, -0.36326737559085598]]], [[[-0.18882374287622139, -0.15693072256620755, 0.60394668134929574, 0.41038468097394332], [-0.14794940334308904, -0.25397615800552042, 0.4833881031380487, -0.89305621557544579], [-0.45801948497025302, 0.39576178452021016, -0.87357016527108211, -0.31718981683755421]], [[-0.61368863076449154, 0.31615179928956683, -0.025789841742435149, 0.72023927453645098], [0.87633790324855032, 0.2862012095707247, 0.71884927958920253, -0.60609278893527962], [0.92741643963665754, 0.42593989643706442, -0.019184444114751509, -0.99304309496436782]]], [[[0.34639236576133148, -0.053274879560228472, -0.010571177825885236, 0.47745069851359023], [-0.46383515752514137, 0.66116314733067916, 0.024022903621741865, -0.16480991144957802], [0.63582497254431725, -0.70317039954890914, 0.32310157066048051, 0.64536261301881015]], [[-0.48217014673084235, 0.2819266533899929, -0.38005388538701879, -0.57654037176641504], [0.49772397527029999, 0.034289275635107197, -0.067983708991538938, 0.51267457985146292], [-0.23472654543514437, -0.28396690355522902, -0.2638716620913284, 0.7825854421238756]]]])+(1.-msk_arg1)*numpy.array([[[[-0.42031219506998752, -0.33218114313748903, 0.73707702159461763, 0.31026940039563344], [-0.78026206183073499, -0.87181022482966508, 0.6551622417360663, -0.20785366919066561], [-0.46753192785037445, -0.12036189975821365, 0.49049885185158137, -0.3367167600943044]], [[0.40860443607319885, 0.36262526284897123, 0.77000552145399026, 0.069777229413515496], [0.011885030028479626, -0.89064787609608853, 0.34111920580273392, -0.51631983380623314], [-0.49655315620976181, -0.17605924420529795, 0.97622675293819117, 0.49138405755962489]]], [[[0.5285465147779318, 0.56130287314622862, 0.73785457206872707, -0.1964144793804965], [-0.28888395234260544, 0.42327893078533085, 0.57497583381551953, -0.71716555322314401], [-0.62342373278960861, 0.075455273759849728, 0.038106562570339841, 0.89848621707166365]], [[0.35228923679777258, -0.034557402437558116, 0.59857720874669118, 0.81133676258937415], [-0.18699524821406843, 0.26657032574714057, 0.043414698121165607, 0.21055613847668408], [0.36658223716745653, 0.61257133830598032, 0.86260298460998919, 0.12849786168480248]]], [[[0.40047231210566636, -0.88392157125100645, 0.47528968389954707, 0.77766756603377596], [0.24099445713947643, 0.15299887916920207, 0.55980172063083056, 0.99538606514207539], [-0.45814780660269805, 0.88794655203821593, -0.055024816906658502, 0.96953005698964301]], [[0.40504834892563957, -0.79259302979257984, -0.2647984024393617, -0.56965048948370112], [0.44209813433167211, 0.60813528047947463, 0.83218804351318409, -0.18255129384851787], [-0.99275191246145877, -0.39351262506982421, -0.17034984831508915, -0.58038798964262273]]]])
      res=inner(arg0,arg1)
      s0=numpy.array([[[[-0.6381104351568363, -0.90437423360155611, 0.31754160295457656, 0.70941267152841525], [0.62966941358439521, 0.86770453299310013, -0.40764602540993722, -0.32367394431083274], [0.15507850164500092, -0.65029503986741477, -0.65964836312697162, -0.84199625341460771]], [[-0.018055728113845149, -0.35059324722713603, 0.19812962831054182, 0.59865753052965909], [0.51069660018283547, 0.68262805073202748, 0.023383428948237528, 0.28964973844390407], [-0.85753831028017768, 0.21678820792273101, 0.30546553024132006, -0.068439883823597336]]], [[[-0.63310251388538341, 0.14995507325652846, 0.05989133068782948, 0.022476041240253153], [-0.53505096055830914, 0.67071301908170256, -0.9845992384915212, -0.50340084645353689], [0.41893231315632762, -0.55600390072868144, 0.57703856018679267, -0.26313601371018547]], [[0.7909260479395106, 0.28072499149797836, 0.61025507411660485, -0.40592349251443571], [-0.01873593368731874, 0.76715863506097959, 0.055405929664817943, 0.69361295333109929], [-0.77114069443766131, 0.50401849781986896, 0.24287180213848414, -0.18411202876696686]]], [[[-0.67114007301043843, 0.34422623970073696, -0.29534016157374254, 0.35761329504728212], [0.71120050041399607, 0.16850426613688874, -0.59330605471184117, -0.12573833409623258], [0.86051138476136324, -0.99029456195528054, -0.90504138433385406, 0.89229660588054172]], [[-0.29692930631860515, -0.54729599559215991, -0.68413358350696774, -0.055178351082899368], [0.093439041337114226, -0.68405271066591133, -0.041692856870497508, 0.18076876018830657], [-0.24099067013426212, -0.22117215786368916, 0.05784968197960616, 0.32740953859681454]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.17590834413)+(1.-msk_ref)*(-0.956119935143)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank0_float_rank0(self):
      arg0=Data(0.240153729026,self.functionspace)
      arg1=0.494786577508
      res=inner(arg0,arg1)
      ref=Data(0.11882484166036042,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank0_array_rank0(self):
      arg0=Data(-0.68099713877,self.functionspace)
      arg1=numpy.array(0.599013007948)
      res=inner(arg0,arg1)
      ref=Data(-0.40792614449895914,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([0.033542800183077981, 0.33729160037543515]),self.functionspace)
      arg1=numpy.array([-0.79953029369586592, -0.40929278080095122])
      res=inner(arg0,arg1)
      ref=Data(-0.16486950194,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[0.11805480437705862, 0.31753327694424094, 0.73062236450582518, -0.90810070531761888, -0.74339066588611802], [-0.87396679950262013, 0.033682508122346633, -0.66916008311259567, -0.61758963219973451, -0.014600527977821409], [-0.62476994832958632, 0.94174223303834825, 0.19392565663742545, -0.2183301186062907, -0.82572189921234873], [-0.58500710929329358, -0.49464134340105126, 0.43417012225850549, -0.38613577382401987, -0.52016166685121545]]),self.functionspace)
      arg1=numpy.array([[-0.18028690878941611, 0.062613895915825779, 0.38680633581348545, 0.90119541158920313, 0.56476663391541426], [-0.64735554161587672, -0.85217678865091129, 0.14569414336970454, -0.91439705751775535, -0.58339013293759967], [-0.54453380416009844, 0.6384627434546204, 0.7632588493700363, -0.33123026273510781, -0.13210066089445971], [0.26324992126960245, 0.069723410191333324, -0.40664489631445289, 0.76527538981583887, 0.90980377502679532]])
      res=inner(arg0,arg1)
      ref=Data(0.192897864145,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[0.031191586406912997, 0.25749528378875186], [-0.43560426520028694, 0.82186118773863481]], [[-0.27636706429065239, -0.96873716307434288], [0.57212427498318874, -0.89488694319467221]], [[-0.53664653426082554, 0.17125910714144532], [0.55223023402118465, 0.7026196657323478]], [[0.47151493133476885, -0.9878569008612994], [0.923273401741177, -0.61291345033616373]], [[-0.21627815010371965, -0.68308350321451483], [0.42003476980191601, 0.35437944514165998]], [[0.57930415742194397, 0.15851258736510698], [-0.034124513547037383, 0.13910191031173214]]]),self.functionspace)
      arg1=numpy.array([[[0.4376882511962199, 0.87791312611900185], [0.3872095550010406, -0.80749936076640538]], [[0.99370860605609601, 0.1415181468078317], [0.88396994180208766, -0.080771360869665632]], [[0.10037555131668507, -0.30418314544151359], [0.18496158262509454, -0.64787225673616877]], [[-0.63902641553526074, 0.82753535127444788], [0.92366200098645801, 0.71478320656855199]], [[0.87210131386272871, 0.66318134839062681], [0.40332439860072489, -0.61714848609477224]], [[-0.026058266846441747, -0.6502882417343101], [-0.039896755710729703, 0.14651276739152008]]])
      res=inner(arg0,arg1)
      ref=Data(-2.37679561615,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[0.84900275444177553, -0.246217726373545, -0.73088633570089478, -0.047481367331499991], [0.032816845637419156, -0.76812807628518143, -0.81281363329523049, -0.58084890710073589], [-0.73993039657350201, 0.83123891495121938, 0.46715743977232393, -0.76686166407968437]], [[0.065941685788379445, 0.6522770195720593, 0.55982464930930576, -0.80087140104814658], [0.99239590675011136, 0.31694749481754458, 0.51372543692541339, -0.52022278210177308], [-0.32907326950409344, -0.55598341424051401, -0.067704116072573095, 0.31263074534856905]]], [[[-0.23768937860779404, -0.80388720531878, -0.95492614309454349, -0.76905056297799268], [-0.28127606377571679, -0.15077250683701848, 0.077788021738735891, -0.20476199518683469], [0.53944050035171465, -0.76150320944042771, -0.41896215933942926, 0.029055212182700796]], [[-0.58861237768203778, -0.10490589957463636, -0.20817583378648052, 0.45198229030492376], [-0.50700996207133375, 0.90434965349211405, -0.92835653637992666, -0.83045937562172556], [0.13469897080944304, 0.84359291895905919, 0.79200244141954079, -0.19219096641228584]]], [[[0.93312085161368596, -0.049735922253603126, -0.2192679149037029, -0.25989030793661816], [0.72644194255242556, 0.74091598098057099, -0.30432653743395388, 0.21345622350494531], [-0.97093017386630387, -0.70687128737696248, 0.59112558415095928, 0.49788697633828582]], [[0.80030801684525965, -0.029383632137730986, -0.021414167385272354, 0.39283038151162608], [-0.33822921166771125, -0.26639948236330291, -0.53349415516836562, 0.020369480646021065], [0.51135584516748556, -0.3710959845124091, 0.26001287550270291, -0.85402920503239654]]]]),self.functionspace)
      arg1=numpy.array([[[[-0.60896275596671168, -0.96198549401243993, 0.87828678808797989, 0.20466313727661678], [-0.60814598635215744, -0.42907263934755946, 0.34070813799556454, -0.44693550838732787], [-0.45080256697753263, 0.38604990495638059, 0.80729145776923339, 0.73718833304242581]], [[-0.035964177002679509, 0.94827643636590153, -0.26496089896423247, 0.17351685300419173], [-0.86296994839532148, -0.34374708342173599, 0.93025018200816256, -0.51672268415125755], [0.44209583101516747, -0.96811144437281405, 0.4995052724717195, 0.9123136046672522]]], [[[0.088471951920575664, -0.6656163426311541, -0.09894089828246333, 0.069975899888207627], [0.19541385901389585, 0.69922988340466619, -0.50164250295332447, -0.50835814069727259], [0.21174890510515954, 0.1461844479384613, 0.71251500577579563, 0.47855838895259528]], [[-0.71000611047816142, 0.18410545018704094, -0.16709456416117296, 0.7936780300267583], [-0.37262949205049356, -0.55397580023642123, 0.24386454933063217, 0.47508493827539522], [0.19879432084889914, -0.090159004691243494, -0.76971400164621384, -0.79794747780441577]]], [[[-0.074209069031669017, -0.087535757179307172, 0.14630099406390684, 0.37408318110918937], [0.90736322792861746, 0.56357314444323792, -0.031967808854433954, -0.48045831936901751], [-0.87344873813351165, 0.96387391665293354, 0.27247943246116457, -0.097155528382333989]], [[-0.21814477511318953, 0.32096345155185513, 0.91631079955393258, -0.53655097537736762], [-0.44317638848986629, 0.77227497773824183, 0.56920867302973188, 0.021731072527379158], [0.89544436460468901, -0.81538646463898146, 0.025217872921130091, -0.2345693190627598]]]])
      res=inner(arg0,arg1)
      ref=Data(1.37543330539,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank0_Symbol_rank0(self):
      arg0=Data(-0.106643138625,self.functionspace)
      arg1=Symbol(shape=())
      res=inner(arg0,arg1)
      s1=numpy.array(-0.371177673332)
      sub=res.substitute({arg1:s1})
      ref=Data(0.039583552071703984,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([-0.92024409156689857, 0.13502538802540598]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=inner(arg0,arg1)
      s1=numpy.array([-0.29036161883440736, 0.11296149068103323])
      sub=res.substitute({arg1:s1})
      ref=Data(0.282456233261,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[0.73595986730425511, -0.9833801614430997, -0.89118743733523642, 0.19372902320394902, -0.41180660331218566], [0.06356562403573407, 0.19335614079388197, 0.99359376812703726, 0.50721751034950335, -0.36969266690147706], [-0.59791965729879415, 0.45529603512837813, -0.018725343452399734, 0.72560266336715107, 0.12692647699253357], [0.63669972540937247, 0.70123453657942969, 0.49868769191857143, 0.49555373109496226, -0.27369218123971439]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=inner(arg0,arg1)
      s1=numpy.array([[0.98300135991939608, 0.68793232992386599, -0.25055339955295519, -0.054868300201233788, 0.71539213882944575], [0.24974682173540041, 0.50749389241263354, 0.32990112868184518, -0.46188117713439181, -0.75149828298226984], [-0.77268215188293188, -0.52042803109930436, -0.25675153521053651, -0.19376736760535507, -0.42493066975475724], [-0.54408610920185119, -0.58734555767376029, -0.27286358828023549, 0.71148194018596822, -0.74763772579781196]])
      sub=res.substitute({arg1:s1})
      ref=Data(0.148514728996,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[-0.49227750523656844, -0.81183479930462843], [0.27406916568990058, -0.84228302296758106]], [[0.22202966888524145, -0.66634978561081737], [-0.23404276488694853, -0.9171645500490313]], [[-0.47377038707866492, -0.13504041250926524], [-0.9670443599160139, 0.20139298091984736]], [[-0.23250385322648737, -0.90421198801558611], [0.19213418612130528, 0.065903549879422707]], [[0.6555462928245317, 0.73646833905623255], [-0.563490192613324, 0.53443488443282772]], [[-0.00018124583762912927, -0.031950402416796919], [0.37583005820425308, 0.85444592826547527]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=inner(arg0,arg1)
      s1=numpy.array([[[-0.49636740352371111, -0.28148408551591353], [-0.43884851631496558, -0.38324804076806829]], [[0.23564680055796106, -0.016505796314268295], [-0.22366608386054065, 0.80458566585743796]], [[-0.36030630269745245, -0.50051909551455886], [-0.29757066247376041, 0.87180747520694424]], [[0.23218405593215019, -0.93265720819471953], [-0.86955030678770262, 0.085666272405931165]], [[0.83659792765684871, -0.79925090721456105], [-0.26072116698218517, -0.86447992276016361]], [[-0.37565601292140038, 0.27386020442564174], [-0.6697299135824486, 0.91664436827310425]]])
      sub=res.substitute({arg1:s1})
      ref=Data(1.55021879151,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[-0.83968257823322712, -0.10396603372732893, 0.98697992513952637, -0.35718582289614176], [0.89429366566276514, -0.59356902279344115, -0.84351580760809086, -0.37303794899859577], [-0.1670634278350005, -0.87884178594433138, -0.95146352963379388, 0.55930566082024646]], [[-0.25385708460946765, 0.14676656988033709, -0.63140426344741507, -0.95389230996397067], [-0.8584570731987442, -0.12886076811019387, -0.63679302420252282, -0.33682096774487924], [-0.27210940785030169, -0.51241819304459835, -0.52435622838183504, 0.24876087655558754]]], [[[-0.32497127279305182, -0.58509362681837374, 0.02298872279207842, -0.010834757206011458], [0.38572519732066701, 0.68950075470476957, -0.82262108077500273, 0.78287327514621197], [0.24600980693060626, 0.69776952926394631, 0.24306320851998198, -0.67431730137111012]], [[0.1273559857119535, -0.14081164647951683, -0.050027567547235918, -0.28432666386476568], [-0.96331679335003373, 0.67932705857822362, -0.58180837439779687, 0.99295600171406395], [-0.65989470597792832, -0.11151145959979525, 0.21128225871466588, 0.65834117228317912]]], [[[-0.91752307061586458, 0.31556541316140496, 0.97369350529662557, -0.3860997673248574], [-0.89080655577413514, 0.069854425635371697, 0.70248676602123949, 0.12927124535519852], [-0.60224223753613426, -0.12772923336826736, -0.82134089084871365, -0.61602193311634257]], [[0.36664667645826166, 0.13617980798990859, 0.56759892028326764, 0.57544969899862974], [0.6701543451089258, -0.5717212108957066, 0.90843064276732965, -0.81375506798475628], [-0.019658982931119784, 0.22422528528620522, -0.04352929479518397, 0.84131658706173162]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=inner(arg0,arg1)
      s1=numpy.array([[[[0.19685797142393513, 0.24271378750647488, 0.44733622931992634, 0.71331624505949143], [-0.97390922016657511, 0.63590566708393315, -0.33638939464605633, -0.028943843140245118], [0.39286210673651722, 0.67909882257518994, -0.49769194733926558, -0.24304639300674524]], [[-0.65473784770544841, 0.91653597596526182, 0.31956499040034325, -0.19240175600454301], [0.53224502860495915, -0.88854214744352422, 0.31394998318098577, -0.56256146044378186], [0.49291895555914089, 0.49184457226810308, -0.65994583573790688, 0.75349940000159754]]], [[[-0.81824117975917177, 0.80863655492456599, 0.43533307508720198, 0.0069418423293134968], [0.77225173735695951, -0.67749370116804597, 0.53550238808214279, 0.52843547956595915], [-0.34272605451451565, 0.38461277527105242, -0.49784716014578767, -0.36616932315289685]], [[-0.98001203837196105, 0.84279460917846771, 0.46032419598609797, -0.44243794570275741], [0.66781151603250244, 0.83150368330823432, 0.83861536279022464, 0.5523655924524975], [0.27576899852024828, -0.24644967587869582, -0.095085600277724502, 0.73797764651815911]]], [[[0.29407051663058459, 0.13408438607502959, 0.3344642154085693, 0.7807057635449024], [-0.78692434697474178, -0.59759183761569723, 0.57827840688291876, -0.76157209499881962], [0.8263743021169041, -0.028440984004201031, 0.76083543356431704, -0.15600362792990552]], [[-0.4479268157381131, 0.20917295511417056, 0.71285675204059018, -0.88743818634664073], [0.97571322537341065, 0.86014957421297566, -0.52686267718310908, -0.24905615741326814], [0.04611946214750362, 0.84450444135455549, -0.011975913090262802, -0.13168813795717571]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(-1.67285839581,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank0_constData_rank0(self):
      arg0=Data(-0.109880112782,self.functionspace)
      arg1=Data(0.679037989257,self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-0.074612770843086082,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([-0.38855187589347273, 0.14448607308006922]),self.functionspace)
      arg1=Data(numpy.array([-0.60202560601945909, -0.80720775185258908]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(0.11728790033,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[0.86706829877617553, 0.49321667183194462, -0.46368761238578982, 0.29128551483742715, -0.92830470873637738], [0.29788055279544867, 0.43176703576888875, -0.85641119676627464, 0.46156140731641848, 0.72244217301749325], [0.44947710909496763, -0.075355916107305854, -0.41900674017982698, 0.75202312792256043, 0.22573770677908533], [0.0031182461358558733, 0.71442090495797195, 0.54936572245556348, -0.74234307901918961, -0.5861969161328362]]),self.functionspace)
      arg1=Data(numpy.array([[-0.76298566426626668, 0.42924119580309217, 0.088601689342662659, -0.92253185082545741, -0.10621318192598928], [0.6270532350670448, 0.85936605057441562, 0.46565958269064978, -0.87889242410910473, -0.08233008340904413], [0.8819854104670195, -0.80865701059438289, -0.050226129858086166, 0.033418509554747455, 0.90566607852023706], [-0.74726523489863084, 0.99737958054864184, 0.56971288996407088, -0.9525569131192575, -0.41645026842223332]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(1.71527222467,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[0.15940026922242234, 0.15456201881086518], [0.59670596059649061, -0.78199474933041069]], [[-0.40931306095332465, -0.30330934407899113], [-0.41127966248227921, 0.87766929968889151]], [[-0.50526082253226456, -0.55791104709617079], [-0.46341249236547877, 0.94360358235444308]], [[-0.085176359264879542, 0.99973232174249427], [0.14305711539257082, -0.16880638300304596]], [[0.34290503849199472, 0.17643564090355612], [-0.063107186944666438, 0.90573192192593233]], [[0.046518471261303596, 0.099777293593690031], [0.27897146863434386, -0.11021578865267534]]]),self.functionspace)
      arg1=Data(numpy.array([[[-0.020655884066142161, -0.63581685932798959], [-0.028189504194750636, -0.33312032709984463]], [[0.18371093040455544, 0.53810369487480392], [-0.6278500905042923, -0.23388196616213763]], [[-0.6265531075236217, -0.81242119009526403], [0.85751366598472889, 0.47472755352559814]], [[-0.25421798028957743, 0.76915921038357227], [0.23926613772390404, 0.90786986627874433]], [[-0.32449040895249714, 0.72903050951989146], [0.85899390598941605, 0.14162684860538022]], [[-0.47547023426125623, 0.52010210858651917], [-0.026594493728175239, -0.85487359950804409]]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(1.65664288426,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-0.77842547938200712, -0.27838514037726103, -0.89144217523800173, 0.0054425153781361146], [0.99706601886824098, 0.78178928147355919, -0.48010670862462801, 0.69424527709964767], [0.66013789019771019, 0.45824582488789067, -0.96353240437680876, 0.834555866938125]], [[0.3621919162667917, 0.25713435067190016, -0.72525415889975187, -0.37623226602641213], [-0.53778025315253108, -0.27139019205119497, -0.067900376115989092, 0.4568352783260623], [-0.41907694255774697, -0.2194008083746839, 0.66797370775622289, 0.26820893540122404]]], [[[-0.40605657137194817, -0.12790178125270946, -0.17410735990789283, -0.15888167658286534], [-0.16153499568694052, -0.85631531260351879, -0.049876455481712823, -0.46438180769276838], [-0.44000243900407199, -0.40478747773712298, 0.92325256219560048, -0.35254132751685119]], [[-0.98549823069498244, 0.51509155773569848, -0.35801892388132384, 0.41842044524973021], [-0.81537594633586319, 0.0061482420516145719, 0.73869381108433863, 0.10350459228602715], [0.6485592561496587, 0.29062672359343233, 0.67430545525394114, -0.87352827882424622]]], [[[0.90625102942488267, 0.57609384066425284, 0.94149558862395133, 0.13561300628620221], [0.051759965405846264, -0.73180643620986086, 0.15917838120678218, -0.19984359746002767], [-0.75895521708451175, 0.02467549025441973, -0.71190411029359635, 0.064476244560973006]], [[0.010518277032398826, -0.47261451925786702, -0.72252534248328493, -0.88093533929134749], [0.7554786003180507, -0.53706340160103583, -0.89826064239038872, -0.70850211109993722], [-0.065050104790129737, 0.59873154953592889, -0.27115597493835475, 0.60224768895299086]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-0.20249524630900306, -0.96774449919067917, -0.52843683450160439, -0.46516378685072324], [0.97628567894067331, -0.3008376220458906, 0.2085767180427871, -0.64493892545106712], [0.63542883246455828, -0.75580660556926271, -0.11642715040710971, -0.57722651873841335]], [[0.9635443621483939, -0.45469939476511745, -0.41046595433690669, -0.00087468895810327574], [-0.0064587635049175862, -0.85926878616987445, -0.89729357458235381, 0.73746038100106448], [0.011783382508033791, 0.032903862162879216, -0.37599153545615205, 0.59273806574501897]]], [[[0.45474791914542756, -0.53886378385785316, 0.72930018614416792, 0.3818111441831058], [0.819037183960172, 0.064783887997734713, -0.67285340441766239, -0.82697093043866676], [-0.41303793669871669, 0.65375901690488325, 0.48148908822783976, 0.61591587360434841]], [[0.54012711113369427, -0.63211866188134502, -0.073210952913265626, -0.81920966825262687], [-0.51175703045000764, 0.19787169073795052, -0.21697146593273309, 0.063276039413608931], [0.32218457663689404, -0.064958869530121088, -0.35773287769104711, -0.74493151067112429]]], [[[0.17263744713312823, -0.8114330555959357, 0.062894929516047027, -0.73765216130276956], [-0.10051063265176641, -0.094306413867799588, 0.90282523931118108, 0.6151602807402341], [0.88428575660803332, -0.031152847200344835, -0.52110285906370746, -0.52480281782828131]], [[-0.5571306490178789, 0.67789963231677008, 0.79860247005714258, 0.44194411259296795], [-0.27253617209725989, -0.24175511809765515, 0.32343391507911723, 0.48083522723160876], [0.9522975767175601, 0.15975334942746611, -0.76757362234650128, -0.9820616906911519]]]]),self.functionspace)
      res=inner(arg0,arg1)
      ref=Data(-1.34087782434,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank0_expandedData_rank0(self):
      arg0=Data(0.504999017422,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.949315936357)+(1.-msk_arg1)*(0.226218043381)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.47940361508382373)+(1.-msk_ref)*numpy.array(0.11423988963078481)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([-0.081888632374237824, -0.70704843929112626]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.63647001611409015, 0.28001115497616635])+(1.-msk_arg1)*numpy.array([-0.95518664215107396, -0.099716134713024651])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.145861790943)+(1.-msk_ref)*(0.148723065209)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-0.57859909800275866, -0.87214342898618158, 0.016011568175356761, -0.23815542580177862, 0.60391292640371552], [0.35788399426640716, 0.017142972407319101, -0.2967562302731197, 0.3534441869214251, -0.90980604318790004], [-0.07925842687212481, 0.47853292187338226, 0.64048368330147909, 0.14555091638444151, 0.28561330493337755], [-0.3552511008600614, -0.38420485695947626, 0.26739266799421046, -0.60565377378093421, 0.11194898284027333]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.93077040816398893, 0.90091471866981743, 0.065383612598433727, 0.6037065816632452, -0.74867109215247041], [-0.63866698099654573, 0.53705324255622111, 0.53082384464137666, -0.05830484773571265, -0.25706797972194462], [0.09395735902475999, -0.73147043052977256, 0.4501866674408872, 0.7474043741109182, -0.95363680556286257], [-0.16128566467235839, 0.82346724291291906, -0.76766336820905723, 0.84787123161224653, -0.58085210405634502]])+(1.-msk_arg1)*numpy.array([[-0.45783242037211935, 0.64404206556021504, -0.43367995912720581, 0.68924549824928949, 0.67655360249298435], [-0.58959025803189502, 0.4825575464599865, -0.36957742684587358, -0.68246492861111885, 0.61875212284828685], [0.12481128840438638, 0.31263132178344266, -0.49457193786999665, 0.55938196857186795, -0.72677278202984485], [0.41178913492830782, -0.43314660108339909, 0.93644188167580755, 0.17440462131539314, 0.40728030234091483]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.28127930167)+(1.-msk_ref)*(-1.04924200455)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[-0.78262529392534508, 0.19262299116314141], [0.40314342927705571, 0.56765433666196907]], [[-0.077381984119281988, -0.10070169049292366], [0.73590227931222296, 0.67072024003607411]], [[0.52384371186421408, -0.12399733883675723], [0.6575827271672896, -0.072890130875433945]], [[0.69043721918464129, 0.25684356518514662], [0.64215553103016898, 0.79678359995253389]], [[-0.34984047601112822, -0.52668353895434161], [0.64670167849951898, 0.073109332448158648]], [[0.19460045426646722, -0.27300730637033577], [-0.3615187862955016, 0.66093465992733225]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.56847726275050459, 0.22553822811057311], [-0.50483648978341789, 0.18596122471399257]], [[0.82907064642940376, 0.39924212625792066], [-0.61560545397453392, -0.77936842489418856]], [[0.22312691317620548, 0.33763279650280675], [0.25967515105910666, 0.83249352707317881]], [[-0.5125813274545159, -0.24474831588504764], [0.61761543752610981, -0.90901051596869187]], [[0.96045364618153561, -0.63252896492430533], [0.30994914138101914, 0.19728471412527138]], [[0.5793481344933038, 0.98228090549135483], [0.56156762048276998, -0.43886014095897963]]])+(1.-msk_arg1)*numpy.array([[[-0.072477101298686453, -0.88184779719335049], [0.71644251282215632, -0.4218857555860287]], [[0.49993394558426107, 0.51515328476076028], [-0.40601362900867066, 0.96411337080457771]], [[0.36949361406832071, -0.56377042221000795], [-0.26007700280810964, -0.32112177435121891]], [[0.033011707625483888, -0.19851785034552694], [-0.3443811451793517, 0.74419473885301524]], [[-0.81079826467032756, 0.029779666629704549], [-0.57319734205670558, 0.15998162467162769]], [[-0.61086054296645664, -0.92839666154121758], [-0.40381664895113789, 0.41316312614238138]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.68558433172)+(1.-msk_ref)*(1.11559168604)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[0.20663983007903708, 0.30744479922588486, -0.61313072201040253, 0.25088112795266682], [0.33976801897833919, -0.064270205661395075, -0.85843808732189264, 0.50003701608777096], [-0.22943788417321742, 0.74280772825811958, -0.0089561666890796143, 0.20113324496421714]], [[0.88307214121599276, -0.39885344149136404, -0.3459359788500429, 0.25014778756241052], [-0.88472844025773534, 0.74531568368805612, -0.72531133224660183, 0.10223747230426095], [0.53780460336790248, -0.85214383675588068, 0.44977767427703097, 0.41749848532253431]]], [[[0.810262475188956, -0.83945779116489216, -0.40109970785754068, 0.81761928138724738], [-0.64032533906144784, -0.20290569312629381, -0.7513065784748898, -0.11353011082332332], [-0.04329051434534259, -0.53099678691971941, 0.72579753561683891, 0.10912003488729338]], [[0.25838798321207412, -0.024144789607555728, 0.037189255130095189, 0.83617814730817952], [-0.057771627027428885, 0.57135552620608943, -0.29473564102969707, 0.22430933502957195], [0.0066994836377327616, -0.10368706774933312, -0.043730730137522489, -0.48065952960266367]]], [[[-0.43491153992500364, 0.88082656940550486, 0.68378859836467565, -0.012356816132639503], [0.62178344914568107, 0.25078866625698182, -0.17819520948670231, 0.77144460986479291], [-0.13233066669247773, -0.52312935440794983, -0.69513281910345093, 0.35433089338923129]], [[0.98606210920219017, -0.48210615134767609, 0.39736669460124818, 0.4619219074410863], [0.29477660552266371, -0.048973046813332921, -0.90547262460988409, -0.3696640293795237], [0.24863189129449159, 0.042312225378325463, -0.5264617262481659, 0.63154246835173189]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.089737235920831493, -0.56346650919450236, 0.28624099902385591, -0.075846454418770026], [-0.95811348785909334, 0.2752992871874782, 0.13840105973254935, 0.27184883084441624], [0.0074529560285576668, -0.90632485195229795, 0.24414236087203656, 0.69344832107210985]], [[-0.99378322357862214, 0.7208847594355694, 0.89881914525516038, -0.97829419124154593], [-0.45862576369309904, -0.92319150295659647, -0.013676613305693364, 0.49602557059441765], [0.8901810990644663, -0.49566489033100725, -0.37871731857378066, -0.36883668970072847]]], [[[0.18437957632400348, -0.12147921286972108, -0.393537207341218, -0.10680783054154741], [0.59524170515902464, -0.66587261729761993, 0.92947386439191648, -0.59547765733667379], [-0.41650352833637716, 0.47904744601647264, 0.25048486764121192, -0.41733337222442124]], [[-0.79428459272932228, 0.5722780947544559, 0.12114888559828185, 0.56254489926953277], [-0.34164943537144987, 0.81303102768839741, -0.038515748676847927, 0.41404092010551152], [0.20093724259501333, -0.88224230847965268, 0.51276578069438661, 0.72798848537859406]]], [[[-0.45399090015655497, 0.79734969926852561, 0.59881323493903804, 0.8863764997702912], [0.73518245422321593, -0.84664030571345616, 0.38783733192343139, 0.10097672989077644], [-0.20113660943856826, 0.47646146092328245, -0.45757028576837544, -0.88890427591403598]], [[0.83506841533390697, 0.6716787741303738, -0.10053183201377869, 0.11446135026109738], [-0.41932879449869054, -0.53813698492040141, 0.79149344313352699, -0.6411930529087273], [0.97161445556347514, 0.30396839332542291, 0.43157396752624089, 0.34620878385763731]]]])+(1.-msk_arg1)*numpy.array([[[[-0.2365951445271044, 0.75967395787052205, -0.030266120399357277, -0.062905760835206603], [-0.0034883077622471781, -0.11075118427509789, 0.13174467113806743, 0.14259302169144994], [0.85476603592412981, 0.14614550828496253, -0.52196009130408605, 0.81755710115293234]], [[-0.94646680517447002, -0.10727315772865564, 0.61196965517547364, 0.45137437283758808], [0.10990216419068144, -0.65862678223718785, -0.84446480541514646, -0.49352662713553697], [0.34069886888804035, 0.59232100391463494, -0.45038974037479473, 0.93140701293250738]]], [[[-0.24206999953430408, -0.67008991280004926, 0.29703689217222728, 0.34177624779386373], [0.42746834677094858, 0.89522807405287197, 0.81107598861107144, -0.38725554644468985], [0.010653314070785402, -0.53865696273892572, -0.18015869599227829, -0.98478472344271717]], [[-0.29749384360660525, 0.91543915007368271, 0.65063936887232066, -0.64142279626575327], [-0.6680021459857739, -0.59184516083841476, -0.99887826884567055, -0.77809132563010275], [0.97229142472558627, -0.84596239894234082, 0.41786543173372448, 0.48987897019575266]]], [[[-0.94580762978353428, 0.48552438448269752, 0.10135988934582585, 0.1698050051381188], [0.57086484517167979, -0.72706535861218913, 0.59049934683374494, 0.01124997633120417], [0.85945579027607155, -0.17921010647927127, -0.60581393629582103, -0.70955535208358023]], [[-0.45839119112204441, -0.71839988876461813, 0.56529846008009965, 0.27304931740440574], [-0.802780743166704, -0.2200444749049042, -0.60485461325660217, -0.54399770036370776], [-0.72600872797946381, -0.98033359034866319, 0.53661806935321499, 0.44150780904733433]]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.15490107051)+(1.-msk_ref)*(-0.542684806247)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.796068672144)+(1.-msk_arg0)*(0.141569698859)
      arg1=-0.603765866012
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.48063909124250953)+(1.-msk_ref)*numpy.array(-0.08547495183288302)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.62865123941)+(1.-msk_arg0)*(0.56747818964)
      arg1=numpy.array(0.933535236912)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.58686808371773458)+(1.-msk_ref)*numpy.array(0.52976088620826978)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.39152578782990766, -0.52109249508064992])+(1.-msk_arg0)*numpy.array([-0.11199372735313928, 0.6249548954816011])
      arg1=numpy.array([0.21561276669150931, 0.99522225948885312])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.603020808702)+(1.-msk_ref)*(0.597821745753)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.85357846934618076, 0.2964009731213777, -0.023080151467793186, -0.27661013218309982, 0.91448024743450684], [0.038642001670084358, 0.45125127830899081, 0.078079050913618886, 0.15283070827828116, 0.11079659530969765], [0.75962692381603292, 0.35160201211707931, 0.18704639739340623, -0.54900920331954639, -0.22652306109506082], [-0.45955692269483905, 0.044781495048763054, -0.2231804184681454, 0.2212845809191446, -0.53383944584935916]])+(1.-msk_arg0)*numpy.array([[-0.83645714719341768, 0.96709801321284594, -0.058716806047270031, 0.79069168020660108, -0.006189196069628089], [0.76088034415206862, 0.35707511806818282, 0.49950296883713152, -0.14173588322467801, -0.93983638713043782], [-0.41603986165671203, -0.43918722106447605, 0.59689715924249764, 0.70425582964428801, 0.21973881370350368], [-0.62574249966976314, 0.2359074345188692, 0.95673105277827286, 0.73034029598894068, -0.46841895831725866]])
      arg1=numpy.array([[-0.55385492909427314, -0.47641952194387671, -0.89304150409657335, -0.87332537255000009, -0.44614032725195774], [0.68166147218263906, -0.81572051421033853, 0.6817858041358531, 0.80364966651683156, 0.69869146527717207], [-0.28246872658949052, 0.7565359744867568, -0.58420487370735263, -0.053675921069090915, 0.012529594167807634], [0.84491978275008273, 0.20578097220399294, -0.83873899314782618, -0.62655075305381036, 0.037134445285017748]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.284110545531)+(1.-msk_ref)*(-3.19151566867)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.50531540411783182, -0.84947843142415125], [-0.95083916187690498, -0.21527209334022501]], [[0.2583894311796151, 0.54220175557198469], [0.65696482234713316, -0.55740417358634153]], [[0.29656481425005188, -0.31044204944929144], [0.50754888593660263, 0.38016247619963761]], [[0.69329849552127065, 0.58139227981628316], [-0.051956674604789699, -0.43018500683995664]], [[0.3632473698109191, -0.52776919844933068], [0.2062693834122924, 0.37147496020488502]], [[-0.3209875527716135, -0.33881385226990934], [-0.41746150569840856, 0.49408272714889878]]])+(1.-msk_arg0)*numpy.array([[[-0.41041698765894008, 0.89834680164904657], [0.55229868254055225, -0.045937486672473327]], [[-0.57946737794748171, 0.82419491486371754], [0.52384160056564988, 0.66720084983331551]], [[0.92072943418247588, 0.044221354735691332], [0.47640305718056308, 0.041941185364050781]], [[-0.59582446422571267, 0.54567956683968699], [-0.40584249131142558, -0.44707794880717588]], [[-0.47053963882295413, -0.63790818284295314], [0.83414330884494636, -0.48717768922188109]], [[0.082591022081887511, 0.19857141903282627], [-0.10712395103176831, 0.79089222821183669]]])
      arg1=numpy.array([[[0.49639001737089017, 0.99796022318233724], [-0.26032037322246437, 0.87839121298696399]], [[-0.70668657882973518, -0.052141122088173564], [-0.92298831309774809, 0.10209313818414234]], [[-0.015592425837284374, 0.24294783103686157], [-0.066724432625728358, 0.18394037896882964]], [[0.86297004238084862, 0.02978315080350713], [-0.42676542085722868, -0.21841822625410234]], [[0.71093267788754444, 0.70330080989264721], [0.22278712618694252, 0.71358709166414203]], [[0.70882998330176572, -0.18543709228630068], [0.80986692291703566, -0.66366121056967531]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.35746173436)+(1.-msk_ref)*(-1.32984776059)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.019006423843182851, 0.97637325818737142, -0.3778371247022172, -0.24522935505580201], [0.25482670730200452, 0.49070982279658582, -0.13970200142272571, -0.69060997302262028], [-0.0035452601352568003, 0.42044690617808511, 0.23511837122113199, -0.52221931154567769]], [[0.19381933190495082, 0.50012173987864328, -0.2386525690285064, -0.37667383268862786], [-0.90971892925469122, -0.35065845094282455, 0.8087148795715664, -0.44030179135579273], [-0.13870392554075761, -0.04716338746550508, -0.29904290935096611, -0.79131372436671921]]], [[[0.44896770989300028, -0.53803971979467313, -0.25044275134156724, -0.38914200762310536], [0.68803157043978946, 0.58126023831649132, -0.57827031016518915, 0.86926678146573488], [0.21238462391570478, -0.033692118582254516, 0.89777850769742762, -0.95875294470846728]], [[0.11907635360994329, 0.98460261094680579, 0.86174977469907055, -0.26058948337859], [0.086267828609031483, -0.91293579034927186, -0.38540499021322527, 0.12009708559161325], [0.73392875969958893, 0.18353973800235357, 0.52212765409565609, 0.37309668927918338]]], [[[0.51977741225129326, 0.59772951631814597, -0.87702243675428693, -0.46372790641150741], [0.36789568031225017, 0.89534194799267564, -0.23506346221068353, -0.55222847869683522], [0.16542389254588907, 0.14294883580010453, 0.089959855117651921, -0.68981061483411676]], [[-0.41115958809013176, -0.986489932594244, -0.96413374324085255, 0.32322608077112491], [0.18297301423492152, -0.22781262416705839, -0.024144967009124052, -0.24163855793719113], [-0.058032940521662413, -0.24375443215880965, -0.078100049154545825, -0.57894164618957267]]]])+(1.-msk_arg0)*numpy.array([[[[-0.40001705419647693, -0.67263334791472262, -0.78918590831661017, 0.78865053701091914], [-0.090034058366400105, -0.37881095982421065, 0.91867249206684143, 0.83980019508043813], [0.67448518939744662, 0.96143694144932024, 0.78309732697025192, -0.56731276091215888]], [[-0.59467823931614316, -0.021013646322696644, 0.72513102655611639, -0.62960315578871229], [0.86606083067435691, -0.72841084190946459, 0.15543842306958466, 0.31989412769972758], [0.47194268630299918, 0.19444174622614785, -0.66695252858363485, 0.8443318710904939]]], [[[0.18223491454007679, 0.82014337464069054, -0.96721600534140117, -0.97710964280180557], [-0.33988905556481974, -0.32222643536859397, 0.95343454998321575, 0.39976170447642656], [-0.32911640832985767, -0.0051494009105315097, 0.68395988105583672, -0.66658071774810845]], [[-0.51922775999439819, -0.4962702374477268, 0.66698724811873289, -0.13569420026129531], [-0.46821413546121105, 0.052814763063328218, 0.98772928696039708, -0.18783043849008596], [0.94396487905072735, -0.62988912286541021, 0.77657229350292489, 0.45494324806001996]]], [[[0.49245323985257272, -0.11950235129083819, 0.027258906056445653, 0.60373375693159259], [0.43120373969315473, -0.1588341022553017, 0.32757005714599741, -0.80325727791557533], [0.20663774771076104, 0.75387367922315862, 0.6484883492094573, -0.0062877488438248808]], [[-0.84891489459846925, 0.79397666107969944, -0.97762738566042362, 0.49200737923277527], [-0.43113202215326463, -0.43968935940027642, 0.22521681397809279, -0.24856511685500804], [0.37263947745544268, -0.1323935505978322, -0.24082796491432612, 0.56535324006815268]]]])
      arg1=numpy.array([[[[-0.81163163217881018, 0.14583559156390025, 0.90627553136559591, -0.030656591956136126], [-0.30432921218385545, 0.059587722748475569, -0.40268326742636673, -0.61586544340169969], [-0.8514338629390128, -0.8041774874930705, 0.85523791052074705, 0.38333292243623918]], [[0.16821600296096539, 0.83859885983438409, 0.43414495762074878, 0.024013057438022178], [0.98564703788576091, -0.79662656719344027, -0.4795278224208761, 0.95284714676502769], [0.23808354955330957, -0.46334260291805918, 0.095014259985530369, -0.10374068985654117]]], [[[-0.55773665932704719, 0.86766281454843952, -0.060782779594130254, -0.81799578780935178], [-0.39579207222264756, 0.14496469860413042, -0.39443931203825033, 0.46619641301760439], [0.59557714470818546, -0.1844163855468639, 0.31701568421598036, 0.99386589088114263]], [[0.28596875555199719, -0.5113185780054228, 0.11275973679314433, 0.95984183359351416], [0.56397724998403853, 0.21456711389176819, -0.8869207954469096, 0.082002513156396395], [0.80154311652059129, -0.14918064076265192, 0.87456914801889662, 0.32879945324200155]]], [[[-0.79827538301490764, -0.63675913901470316, 0.73918094313334581, -0.69235823201746327], [-0.13014921576975857, 0.21884429385731696, -0.61439214698250044, -0.98441367018944836], [0.83244430972608208, -0.9972055581758188, 0.64180363710022958, 0.40388529426265696]], [[0.75678188832694704, -0.95635607269517875, 0.8829297874583959, 0.64433269944660188], [-0.88188838313425677, 0.92295774091578653, -0.069016329600187332, -0.16861637091927673], [-0.87830132902116187, 0.45182237326637598, -0.94926829916048394, -0.98497920687933416]]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.17042066276)+(1.-msk_ref)*(-2.30685891421)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.227339785007)+(1.-msk_arg0)*(-0.0149592559564)
      arg1=Symbol(shape=())
      res=inner(arg0,arg1)
      s1=numpy.array(0.635876771666)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.14456008856153646)+(1.-msk_ref)*numpy.array(-0.0095122433840913612)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.033361279454394044, 0.91376878259700156])+(1.-msk_arg0)*numpy.array([0.50925860750659369, 0.48879591924969445])
      arg1=Symbol(shape=(2,))
      res=inner(arg0,arg1)
      s1=numpy.array([-0.46612066838983734, -0.38714800843906838])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.369314146234)+(1.-msk_ref)*(-0.426612329185)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.08003675403706767, 0.3319862096718158, 0.66827333708676617, 0.607456046276307, -0.69561213566796809], [-0.42473970768048064, 0.6367226794432892, -0.54569252502744092, 0.95203511751775638, 0.48653286550240304], [-0.65050600867588138, 0.55691773779535825, 0.68154389653661185, 0.60500644813274218, -0.18211902938470059], [-0.096881900189787151, 0.97898368934439595, -0.3484199958999632, -0.98555484306110941, 0.011922065145519856]])+(1.-msk_arg0)*numpy.array([[0.72170018389282453, -0.82220508857232821, 0.63232191976549523, -0.42356815103129319, -0.69721793304350754], [0.21318264513719209, 0.49220897902328264, 0.93896918069166802, 0.6313604662544634, 0.38110713666159479], [0.049806416394649888, 0.18369511877237943, 0.46404711190049142, 0.1490883441845956, 0.83668567266277205], [0.6817692022719557, -0.9163133905345624, -0.87078633764096547, 0.66661389710605423, 0.98192029962712479]])
      arg1=Symbol(shape=(4, 5))
      res=inner(arg0,arg1)
      s1=numpy.array([[0.57208604575293798, 0.39234153264650828, -0.29009133725107783, 0.12759653247178671, -0.98336748506173111], [-0.1974182175878838, 0.35986928511839644, -0.54640829310035754, 0.35801029947023211, 0.49334707473060635], [-0.42524363097892448, 0.095258194595194245, -0.22676220937963953, -0.75472190256512572, -0.10329993962857897], [0.23630617509426188, 0.5557554336335444, 0.09396275303176016, -0.9059894011937113, -0.17992459063539745]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(2.96071032352)+(1.-msk_ref)*(-0.943967895314)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.23417355119360739, -0.79746650137153896], [-0.68905854933431909, 0.97924559740085515]], [[-0.57490689997294653, 0.11755459726100459], [-0.18551827322372039, -0.28115639935127712]], [[0.42753682451019381, 0.59480593623279998], [-0.81919572772503813, -0.93207258510735103]], [[0.71794278874088091, 0.92979890479176408], [0.70868008342341748, -0.29350941559703214]], [[-0.60288740992332945, -0.93310780888144329], [0.64280139894490818, -0.35382392352693892]], [[0.067455129112901746, 0.74269303148847654], [-0.57212799883903886, 0.51288565062414815]]])+(1.-msk_arg0)*numpy.array([[[0.035303762215068035, -0.14051521447535187], [0.73755086943775838, -0.14467420041566514]], [[-0.18157757081297499, 0.60515644014893488], [-0.85621051744096821, 0.26156966436842044]], [[-0.33976608554885623, 0.51727713455536284], [0.097642795986651265, 0.57610067005438714]], [[0.85269706081186714, 0.92839106154294537], [0.34658773508857732, 0.026038378828081576]], [[-0.81720780340681221, -0.046230336220644652], [-0.50866941130344445, -0.071290157425963896]], [[-0.25242451536247112, 0.096811324881940886], [0.80639304326790806, 0.93085561303797748]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=inner(arg0,arg1)
      s1=numpy.array([[[-0.35721402584340045, 0.33542387573125221], [0.47700399724022646, -0.21707666735241982]], [[-0.081458753900366165, -0.38962235070010265], [-0.3671611702608697, 0.60348983065197248]], [[-0.31993521788188928, -0.93590274141857055], [-0.23173794675225401, -0.75610815094688211]], [[-0.74572591224255924, -0.17346317565433256], [-0.18648487012711201, -0.01549825671269911]], [[0.084385706823225659, -0.69800281778542606], [0.34217274697142686, -0.28217682024030544]], [[-0.85104867535945328, -0.77962424377577522], [-0.22628217577907894, -0.32677607984054569]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-1.20312537617)+(1.-msk_ref)*(-1.65882209349)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.6504287358463714, 0.62045327015963792, 0.80964237397933592, 0.81216332593550145], [-0.5568200525651803, -0.10796909287722523, -0.33648126007358914, 0.33166376822741994], [-0.94163507845985062, 0.61964853010327259, -0.46454757672287483, 0.055604987229706415]], [[-0.060056726265880656, 0.74728830315894723, 0.69729966533994747, 0.68350499535477316], [-0.68381082464483223, -0.25297584451747812, -0.64441401977897605, 0.35528099935921187], [0.18697482066110838, 0.80051452974383719, -0.67562366442940802, 0.85668770179874065]]], [[[0.31275516897416789, -0.25123451994873913, -0.65799558075378806, 0.77913870256557183], [-0.018113422244537603, 0.6929823252020606, -0.89103604580901608, 0.79014085976433135], [-0.085404306812976172, 0.77288156384954276, 0.33706167507451723, -0.028439137641345091]], [[-0.4761926037389399, -0.18952521186298066, -0.75710147168348718, -0.85342594668312444], [-0.78856624635670491, -0.20978714041715407, -0.37623935023851685, -0.37175957771646995], [0.10807335509914107, -0.63166539524803178, -0.5906019608482993, -0.64676981552591584]]], [[[-0.73140886567025221, 0.49637411246093421, -0.11063031597587347, 0.89696132684372265], [-0.48829865458866939, -0.050314729979547668, 0.46821033443170812, -0.22351744507542914], [0.45691291022912961, 0.356365872659403, 0.72908302324142338, 0.90067301473415395]], [[0.54351986200752611, -0.52061756865428044, 0.93177033296418288, 0.51829696827968319], [0.29329348511343589, 0.42792680739548605, 0.37468795047510084, -0.3812019147372252], [-0.93625603446672034, -0.24396948511864336, 0.95240938920520501, 0.91198666564296182]]]])+(1.-msk_arg0)*numpy.array([[[[-0.081334993884744966, -0.80300373203401176, -0.5497982501464358, -0.3623358836711752], [-0.2140368343143777, -0.20032583127800874, 0.79656389647002968, -0.32777889884602396], [-0.55775860783205533, 0.50129127783058669, -0.20479439403995747, -0.96609492321516255]], [[-0.068803797467580363, 0.52735861124689576, -0.44262812182552769, 0.89296146004043409], [-0.20361174369912671, -0.62442630136434341, -0.43255853863195215, -0.061442818745899963], [0.32744088359121837, 0.15862755589284694, 0.98670447554080387, -0.93034763442081458]]], [[[0.6305973208606106, -0.44906131745049871, -0.33809985338808879, 0.44226070064170808], [-0.63320475664709841, 0.92604677575070449, -0.051078240833776656, 0.55274951997769017], [0.98835085295526137, -0.057928566511622037, 0.99823635957831436, -0.10431553153258011]], [[0.47089987169254166, 0.59220916424122794, -0.33587503733499524, 0.89627349018749691], [0.0016749915312315178, 0.64839482562108919, -0.46260177953088033, -0.28731132607195575], [0.85677685750856702, 0.77078831326635977, -0.28210527627928039, -0.23226397942468702]]], [[[-0.89837662784618311, 0.58221029274502256, 0.051491715984914865, -0.30071516110450558], [0.21123965025042457, -0.79037942692110774, 0.11400685630646246, 0.012084261028387111], [-0.83220932976237894, 0.047446867956052019, -0.39335461550617734, -0.81802118085602427]], [[0.71776417868504372, 0.41834623633932178, -0.28182912447916375, 0.84875677719629228], [-0.13001359249654998, 0.081730000935394953, -0.067107866223648172, 0.95667811209204889], [0.28947081075616876, -0.48499942320090916, -0.23938769003950555, -0.39561319722857835]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=inner(arg0,arg1)
      s1=numpy.array([[[[0.75300980475018897, -0.19556258337598043, -0.72042063459656824, -0.4111971934581522], [-0.80901740603729144, 0.27750300889539381, -0.26889325610476589, -0.26744368502352467], [0.40578180840277045, -0.96642023676315092, -0.4402356157193752, -0.30939583932583758]], [[0.20593382402893301, 0.99050439798598977, 0.4282760300981201, 0.71410819676152393], [-0.97132046114828863, 0.78522932808220847, -0.67598855002659719, 0.12859450614590617], [0.434364455762142, 0.79505192775709976, -0.94326690391854995, -0.96128276005311997]]], [[[0.099202298951634793, -0.61755098941181408, -0.13076775768362792, 0.39459716393179578], [0.62570646082268189, -0.62468462236032329, 0.28557176297155129, -0.0854248752793767], [-0.47938969134675169, -0.2303185395767664, 0.27545950865220803, 0.030590915839713739]], [[0.011555267018080073, 0.1756893546139322, -0.6905305966182147, -0.62552721372417719], [0.70780446458441681, -0.18638849955576009, 0.36084623545346117, -0.39138646934264054], [0.22283175223732488, 0.91666780218862232, -0.6643203079928488, 0.69833257400454718]]], [[[-0.34719258446789114, 0.33548157623946429, 0.83739148115150353, -0.1807937630818357], [0.90459032193505262, -0.76367770330507523, 0.87159501375970172, -0.54603683348234933], [-0.62210384777624972, -0.50024740916816368, 0.52208617209826347, -0.46934948860040726]], [[-0.65016839979443053, -0.058997451547941004, -0.8743403010148918, -0.041669342828929201], [0.22847380449016019, 0.54735793289978041, -0.93868533747645944, -0.35384304832639679], [0.2294673102017184, 0.19407961588249711, -0.37913666514395161, -0.6371943704672729]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.755640149612)+(1.-msk_ref)*(3.29219222219)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.291531225166)+(1.-msk_arg0)*(0.0245199868034)
      arg1=Data(-0.700555766847,self.functionspace)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.20423388100607154)+(1.-msk_ref)*numpy.array(-0.017177618158112615)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.13795126128122193, -0.049083343324659578])+(1.-msk_arg0)*numpy.array([0.41150496249996782, -0.895867771687894])
      arg1=Data(numpy.array([-0.48311607076511565, -0.98030703413828491]),self.functionspace)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.114763218027)+(1.-msk_ref)*(0.67942081766)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.75796400257798324, -0.70348397505435734, 0.89005890441617064, 0.28168276991104246, -0.22815083366244493], [0.87001455744515122, -0.61199454394107877, -0.88968426082566543, -0.48988343936641043, 0.0091061395097682407], [0.00038701284385123103, -0.58154294262613138, 0.12466518547258865, 0.26742919042369873, 0.2060419497399868], [0.05209424073622726, 0.36005249160908215, 0.84151943818677521, 0.90383961792029832, -0.3365232541585006]])+(1.-msk_arg0)*numpy.array([[-0.75334843408531493, -0.089920064534031008, -0.41287254028214426, 0.93961825826695544, -0.099839208277126223], [-0.79338565350602552, 0.50638366051092198, -0.35393904632388651, 0.29272444192355396, -0.6584314991602549], [0.85093602922976408, 0.36590867303067798, -0.5094538711100558, 0.89305104272018543, -0.10276418819907129], [0.023578509730667108, -0.16387847175810877, -0.18004804284015163, -0.81549358690093499, -0.47628686906637374]])
      arg1=Data(numpy.array([[-0.72674958907045362, -0.93280779919345203, 0.43718221749563302, -0.0088212548455395279, 0.921486643264948], [0.49429499981575908, -0.81673962137327916, -0.54992531147908963, -0.8645863398574325, -0.086203025330993999], [0.040650358243210727, 0.49655770251078479, 0.6536740821874063, -0.066082623968604537, 0.37594522027749044], [0.12475948518736812, 0.71132898153986934, 0.33196050203008243, 0.095617436608621098, -0.64995797842956304]]),self.functionspace)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(3.92500564327)+(1.-msk_ref)*(-0.613040413195)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.93288754023006093, 0.60106570738315845], [0.30801084889880515, -0.10038465186627255]], [[-0.49486005038268965, 0.60921104657314995], [0.87331640600526983, 0.96637335220871035]], [[0.43202597147178401, 0.91322807330597522], [-0.62307024365185915, -0.17477126639044194]], [[-0.31822550805442673, -0.90911614050867762], [0.55699981820870859, -0.68063487208669682]], [[0.0034300727389189323, -0.86962091301762912], [-0.57714211960863127, -0.85825991865608864]], [[-0.17754376238064573, -0.24779798715479107], [-0.011140993027736412, 0.098200156176100561]]])+(1.-msk_arg0)*numpy.array([[[0.24010403647853407, -0.88838665532645722], [0.90501777439104525, 0.53632909560538189]], [[-0.54131241789007545, 0.15141641667204264], [-0.99744268542316106, 0.29568079322935215]], [[-0.07533138726578037, 0.51085817705272096], [-0.68401123133882247, -0.026930646609903741]], [[0.31995995928285237, 0.31660009563403202], [0.35070586390968561, 0.66058148339954159]], [[0.55381138010714537, -0.22547846639192781], [0.20473202038390381, -0.42712246976137891]], [[-0.83406606159475327, -0.36707521115120145], [0.99394483242876586, 0.64057999530308241]]])
      arg1=Data(numpy.array([[[0.56165585181397337, 0.76350367189248303], [-0.11994286131079535, -0.16253635268259248]], [[-0.95622051234148109, 0.83826617585901686], [0.49593782958986288, 0.49840249855170282]], [[0.69853814174312601, 0.087064491712804282], [-0.01647409369557451, 0.43352480342262467]], [[0.98625851181227997, -0.85890738533846744], [-0.75112351254266674, -0.69214254512586582]], [[0.21952135478796331, 0.74856199577848415], [-0.32886277865741609, -0.99776445778129808]], [[0.097666284406329407, 0.62425865896691324], [-0.0010713686346326412, -0.078146997352248615]]]),self.functionspace)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(3.91262521622)+(1.-msk_ref)*(-1.17757862828)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.44681514011031398, 0.33985438742685536, -0.4431136159656559, -0.13354837017362287], [0.73305422778738927, 0.61064286420775193, 0.99914094831721179, 0.20213178104609941], [0.77095705236519541, -0.9529046335984086, 0.9520486082598747, -0.042873934491836962]], [[-0.25981270183444449, 0.85055011439346706, 0.1307660257197174, -0.75354074853278541], [-0.89931998085420384, -0.31754146844196685, -0.96168179930810993, -0.32969850226885389], [-0.81759489072088565, -0.79455455035559308, 0.30554468587756012, -0.97694761502203065]]], [[[0.9524044737827746, -0.5491619770191285, 0.51931562514984453, 0.75704791120318649], [-0.63299678476462362, -0.60042020485151748, 0.63938597028752708, 0.97527046159828057], [0.7349335457464401, -0.47635296268781113, 0.47610605929806882, -0.91077052361951272]], [[0.92333547515269676, -0.13572130553211714, 0.79962527069075584, 0.62582507802038934], [-0.34955705672146808, -0.22067602283071563, 0.98794413498735012, -0.39365343011975495], [-0.9395621189187584, -0.10369246837169155, -0.48126680305363423, -0.51669696645207086]]], [[[0.65028702636009528, -0.53411372748691854, 0.32701223381980959, 0.92786702524886722], [0.40479639977686066, -0.081428651882120562, -0.83257147695365452, -0.83547803542192933], [-0.21493301104465234, -0.858545831833744, -0.046828862716373321, -0.61361609559691499]], [[0.97566639940685507, -0.0056602780003758291, 0.72342396028118405, -0.18050425435644346], [-0.7670123805919975, 0.64963751109942836, 0.74915475749297489, -0.95417005675424726], [0.86875515153318328, -0.50354690200796526, -0.20723844381933199, 0.588505435821433]]]])+(1.-msk_arg0)*numpy.array([[[[-0.45187484182260351, 0.26218751303890997, 0.85384327705187646, 0.75303646136734614], [-0.80410026271033752, 0.53147720985892377, 0.50441076578741106, 0.58596387599943167], [0.98671580330013953, 0.4923260155932212, -0.53541284591070015, 0.24215788450392051]], [[-0.085406833040039487, -0.13311435037468855, -0.73318155510231131, -0.38549084622077223], [-0.68928525519152739, -0.68740139381758092, 0.90616221111805806, 0.19007205825157003], [0.17561487416286736, -0.90180859526252921, -0.73315538314293272, -0.54323538049256781]]], [[[0.89111566972666645, 0.20443274795271837, -0.728969135510944, -0.67352997540305459], [0.60201785288645926, -0.035196174636839217, 0.34578764531428541, 0.34953478993313891], [-0.35854486353024217, -0.55920923257522803, -0.42049272504701074, 0.42935874870770507]], [[-0.081340864936127133, -0.75079067727008786, -0.31332393287911153, 0.89072681020132172], [0.69085093202892045, -0.97027053638239891, -0.12717831450461681, -0.83994461823002342], [-0.72897706097294757, -0.42745265769195995, -0.33355490737943816, -0.97263634639973628]]], [[[-0.76033244671178002, -0.69089710234761292, 0.87282786896240006, 0.92436829090199435], [0.58083081564855843, -0.81615646614553428, 0.90398073899985509, 0.42059572648475063], [0.019855973515656711, -0.45396080542761208, 0.53736366245784306, 0.59033308886792213]], [[-0.94138407203855867, -0.67937937246377555, -0.054229586011050657, -0.5411240345139039], [0.80177386348099788, 0.23753684966099531, -0.34285918400305304, -0.52316423985274541], [-0.15503198735542156, 0.62711500005635878, -0.59892199914183242, -0.11115015793655636]]]])
      arg1=Data(numpy.array([[[[-0.56067461849346856, 0.82436543003426865, -0.85784285960585782, -0.84913542810300635], [0.31644257130645426, -0.15652880988651585, -0.92651670576264755, 0.66860658189646704], [-0.80377814282619164, -0.38511769253035455, 0.4108650741538038, -0.50997661822690676]], [[-0.019249788314307237, -0.5845033284576342, 0.14061092565362476, 0.83198289487612698], [0.84011230181354235, -0.35733816393052509, 0.96442765761882199, -0.76747774313560524], [0.59919634836449132, -0.55144540917616425, 0.97222443025153171, -0.21855343547764483]]], [[[-0.080736106169028155, -0.59334688941447178, -0.34294452941442688, 0.7014605270562615], [-0.44551858269727185, -0.1899099392185104, 0.73618040445628052, 0.99607451421460791], [-0.38697159025089833, 0.53225190075118189, 0.80372671392552042, -0.26158470174624071]], [[0.78921994505062454, -0.78939118206556924, -0.7493958134408456, -0.43440374824874217], [-0.040305328457116429, 0.31872351340112015, -0.72449815921690686, 0.22002072913721205], [-0.061081502789878339, -0.5769041864612805, -0.53698267097296615, -0.060752999644593242]]], [[[-0.61527880545476776, -0.016981853524558055, -0.67572274024216328, 0.39259015365922023], [-0.95150026727961223, -0.26067466866724942, -0.535056872864345, 0.50070596980230753], [-0.39057009491319405, -0.92296854823798524, 0.16321388919501878, 0.56519023268513768]], [[-0.39144376514710566, 0.085591582644400699, 0.86408260436397399, -0.6950100726632451], [0.32560339228050483, -0.4547551945702446, -0.43659989630539719, 0.036767150522588832], [-0.17649832158303713, 0.45518147529791864, -0.64714458530939223, -0.87261235042399221]]]]),self.functionspace)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.755579480427)+(1.-msk_ref)*(-0.59812450412)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.403235728374)+(1.-msk_arg0)*(-0.295926359992)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.541361045232)+(1.-msk_arg1)*(-0.810551377619)
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.21829611538783306)+(1.-msk_ref)*numpy.array(0.23986351876555856)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.90850727965316946, 0.50515315395193428])+(1.-msk_arg0)*numpy.array([0.74762075729523225, 0.81043766812190188])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.61574683887358339, 0.93798419219177864])+(1.-msk_arg1)*numpy.array([-0.007714614761548555, -0.86522376090217512])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.0855848124973)+(1.-msk_ref)*(-0.706977533319)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.97936996315172942, 0.70300297214989427, -0.25061232251134102, 0.44815350126104292, -0.30428700430274702], [-0.12023987608121867, 0.12967971629631569, 0.25525677189486284, -0.64554781120983828, -0.31958253742431997], [0.076147714924661436, 0.29338897471920244, -0.28506107676730741, -0.85770078975051978, 0.91865830392579051], [0.61918931910660246, 0.84653161661238596, 0.85127368640798529, -0.28022811001072645, 0.8813404201048407]])+(1.-msk_arg0)*numpy.array([[-0.49148868927913258, -0.46133680864660875, 0.042472319927527868, -0.16381315384583361, -0.081512938414658453], [0.13382615090707151, 0.23827946149774837, -0.2677152798942255, 0.90613469033837268, 0.70139580236050714], [0.72992965875251414, -0.43511959013079471, -0.15960126320034851, 0.12536531981177279, 0.17187813577533584], [0.76790552207840812, 0.038936831804500249, 0.97968906398820965, 0.58515284859248307, -0.33790017411269457]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.77792892176813555, -0.48474074670279599, 0.93420142438233378, -0.44466398223378789, -0.51263103365865992], [0.63495992637336518, -0.66572912657297234, 0.82268523377741287, -0.66447875079745655, -0.43956314173978117], [-0.70214786437828991, -0.98050935373364578, 0.15802372275413235, 0.3060961579403354, -0.40748742528675685], [0.72454726587244744, 0.88591188793736531, 0.46813774668437813, -0.83119094161369311, 0.92495780926047999]])+(1.-msk_arg1)*numpy.array([[-0.46866746866959152, 0.77485146518085779, 0.078830102748751463, 0.86057905914440269, 0.63438949285317259], [-0.46737572651749382, 0.13973168612107201, 0.54911400282390876, -0.44226114996595678, -0.28915038149170735], [-0.79873536394315914, 0.17697126209294067, 0.31687102417355439, -0.0020160127671704497, -0.045717467295027348], [-0.49355269181235517, -0.94737060160283404, -0.076589075013161478, -0.54223770216012301, 0.1330028382259909]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(2.38259814517)+(1.-msk_ref)*(-2.66813933913)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.82515479906730982, 0.86187150324023643], [-0.64054582590394871, -0.4585412457900413]], [[0.18034602851741566, -0.48817089152477067], [0.84083037658278736, 0.27098186545704439]], [[0.16449913181804132, 0.4832607127000943], [-0.58311581170613991, 0.27572947234154332]], [[-0.91649111471063804, -0.31690517728986922], [-0.33677447363475643, 0.75657122713227598]], [[0.15263583122141222, 0.7147460693527834], [-0.77672397702369111, -0.88628760970188258]], [[0.26526071178948207, -0.10765846826010272], [0.26436608712533771, -0.10491818539219988]]])+(1.-msk_arg0)*numpy.array([[[0.74852211953177816, 0.3186875609801596], [0.18785460974395507, -0.0051836560942222221]], [[0.27784421485642063, 0.10151035726096147], [-0.46057699701818189, 0.1048205876836934]], [[0.8619190179244669, 0.7182321218553569], [-0.52469116547623029, -0.64351673986699343]], [[-0.95304560847909769, -0.50575897884598531], [-0.49654624538600833, 0.27045902883502704]], [[0.72623722391565204, 0.28200506732420449], [0.49675058588390431, -0.80454908927028956]], [[-0.7252012828553136, -0.15094050576566165], [-0.7527450566268663, 0.39277655861690786]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.77848925392078994, -0.52609071667174501], [0.72333872332332971, -0.52349654559260528]], [[-0.781853676722845, 0.26887365320155632], [-0.25467080531596542, -0.12846944357675794]], [[0.27015633772991388, 0.46747008919478983], [-0.45708381185132296, 0.033007137251819252]], [[0.61986039250907288, -0.25968752477805968], [0.40222628722983833, -0.75395899203083494]], [[-0.60867569571751678, -0.32633068042911528], [-0.64743474613387764, 0.066047300331868453]], [[0.78079227379994265, -0.46586476049812209], [-0.70304649915648398, -0.13207766237217911]]])+(1.-msk_arg1)*numpy.array([[[0.65793158785296946, 0.69958336185059955], [-0.83288605047716091, -0.74606078974779488]], [[0.91843015701277997, 0.75420640449017595], [-0.61909674393108149, -0.88241525774172858]], [[-0.2168876481400932, -0.88985122212027634], [-0.20849132046611341, -0.096336219615894558]], [[0.99177271903176045, -0.36477583556657356], [-0.87653688087840109, 0.86997207008395683]], [[0.88000911935290405, -0.11788561531713126], [-0.26746816228835169, -0.37355939266452176]], [[0.26514166517441073, 0.67136051090322457], [0.57373631491666877, -0.47031764453494174]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-2.28253572206)+(1.-msk_ref)*(0.205671973309)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_inner_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.85848399921195329, -0.57896707886291332, -0.43034381488523143, -0.44028867476623801], [0.1542009868569123, 0.89134745980511632, -0.31625638349919094, -0.46821880735750909], [-0.00014140379166804884, -0.45054390630032426, 0.063612421180540002, 0.2139474282001117]], [[0.14851515293243445, 0.030852297972091103, -0.32904205566508327, -0.60470261985184459], [0.2050582049845926, 0.66891227664110531, 0.12664539701770172, -0.1512184013561424], [0.43610165485599883, -0.031232254854981267, -0.94367086360711649, 0.99864791296239752]]], [[[-0.0739525337370186, 0.13690549306105271, 0.44836619794469867, 0.86486218754491251], [0.18292690193610994, 0.82337273414879975, -0.99426240224917328, -0.2611768138402506], [0.63533397617306964, -0.46590412506632273, 0.19710132665811919, 0.92162975601912445]], [[-0.50199990627637336, -0.41579217916435351, -0.033835742233214949, -0.99227315756588963], [0.41300955053414445, 0.2404521065620393, -0.70048488468070169, -0.02128933259204846], [-0.2371289244231205, 0.72390162758131571, 0.57245237356825229, 0.3193794886016077]]], [[[0.036010838035816839, -0.16697536583993622, -0.40253894630502418, 0.19689771264364264], [0.692038111953033, -0.29737990759908306, 0.3210505962031367, -0.39445483259245995], [0.24932266060009267, -0.87499616800979152, -0.024935835903332482, 0.61934652294663839]], [[0.1921206747357862, 0.91683813432396177, -0.35927087384140122, 0.41710362048575012], [0.32246985496822211, 0.83179661383912151, -0.56990023935119183, 0.67618438981012918], [0.50718837602815503, -0.84834818782953691, 0.37937837632206328, -0.25426188524343107]]]])+(1.-msk_arg0)*numpy.array([[[[-0.52660798398775155, -0.91198538123189898, 0.82532828731900576, 0.75474274557208942], [-0.14733998379424951, 0.45177887906173697, -0.36321745594766464, 0.34951620533919336], [-0.69560845662216808, -0.11401790466463391, -0.73834803054149334, 0.10722259179197136]], [[0.0177056943396805, -0.47868540405574977, 0.40133180486325815, 0.32115356551548468], [0.91979852980436405, 0.34980505610258494, 0.4856493865847229, -0.44919356645331976], [-0.85304974198046613, 0.71519478383129353, 0.67204216667633654, 0.7475580019235577]]], [[[0.24217862286310465, 0.95354035165723561, -0.31263155472319304, -0.44578837088375711], [-0.11528633107193453, -0.23498522545753464, 0.9180284456824146, 0.20783304264420299], [0.15122575609614941, -0.091557510721294477, 0.40550183981931331, 0.96902348433008778]], [[0.85700567482802037, 5.7867086591834038e-05, 0.72809005526213588, -0.2944154096127316], [0.048631557146810112, 0.90940535486443252, 0.87370296139337378, 0.29272107817043769], [-0.56122498183076353, 0.73490381573551034, 0.98989427623480863, -0.86335822872138035]]], [[[0.44987693855421917, 0.61924124200328845, 0.92027614760278231, -0.92836895964474153], [-0.26038483014295299, 0.7091524621105445, -0.61699126503434165, 0.7867106439773095], [-0.010275133721955321, -0.64211611345792963, -0.7266804879141564, -0.37259764086320635]], [[0.23436695670222973, -0.62268616476340166, 0.56504976209184354, 0.75852938727342356], [0.018278357410181023, -0.50994535892748472, -0.47740324751282759, -0.4048677745526883], [-0.75546010712197464, -0.54279502794620971, 0.65458780889854662, 0.63133081314691397]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.85032148694968157, -0.89307913993242494, 0.82344755515608292, 0.52020307096338647], [-0.71582198836216415, -0.61956713081684667, -0.025632675537781591, -0.96725308876812122], [-0.74840121493833189, -0.95704548609145279, 0.02925160124606685, 0.28204436866124238]], [[0.77582565278870219, 0.20399239563725824, 0.42072111286039204, -0.98584512646851774], [0.0051541472756353457, -0.31450623803480027, -0.18137342071886264, 0.84383008132128734], [0.60539191928307545, 0.93603849652654358, 0.52593897438047366, -0.91238023684979264]]], [[[0.11132156128689674, -0.23856418739373941, -0.45006040991635454, 0.48701817686331794], [-0.64125441277940864, 0.88477096108526365, -0.46055858667528038, -0.3010739289286819], [0.302230111668218, -0.12141629391555764, -0.69959171269062126, -0.088172608374930794]], [[-0.59067769345245158, -0.43595737005003898, 0.45225872072232964, -0.40597848146882853], [-0.37249427288094927, -0.16711051368782659, -0.56238851046142724, 0.40758376843097333], [-0.78419565596843221, 0.94913458725501987, -0.75346190090301124, -0.80854672867882327]]], [[[0.047814610347620023, 0.89226292666026841, 0.65891391291871959, -0.043438552217093696], [-0.85376353357903323, 0.58078752052539806, -0.24143839852724791, -0.58458556312805987], [-0.63293655693372997, 0.96460464839254167, 0.65159578698950993, 0.061326179501126621]], [[-0.17234535569232556, -0.049385685202222529, 0.83562180983328016, -0.9741260950898365], [-0.46358357818427764, -0.93135550143789358, -0.22307605504315453, 0.40839527256023089], [-0.82005299493558326, -0.98533424617530185, 0.94391858495261372, 0.67582394987210059]]]])+(1.-msk_arg1)*numpy.array([[[[0.41324938725852634, -0.96435227534002066, -0.21728111111808257, 0.70938235599301858], [-0.3679456961352745, -0.24296128506749803, -0.51762107537142632, -0.83438579427741177], [-0.96896461424687486, -0.16496310799157143, -0.86728612607361533, -0.39351263591764996]], [[0.076791725845727354, 0.85612310092343114, -0.31486157389407832, -0.88870188389020055], [-0.66810623930448432, -0.36426484354507327, -0.87988747852457183, 0.32315595765127236], [-0.75543936721157112, 0.49986454498032074, -0.30458644036080273, 0.059409472050175882]]], [[[0.0079330968533004587, 0.21116245050181348, -0.44143395119942364, -0.28063483186401439], [0.88751279455182308, -0.073463439487072346, 0.60096508702459972, 0.79177470299857999], [0.95243642490697522, -0.21215275520223864, 0.18355745101229259, -0.14013995159823978]], [[0.1627217916908017, 0.41334334239730808, 0.33573546545915911, 0.077565211932971945], [0.91790152497156097, 0.63377408643159061, 0.98034579788904486, -0.49206922858999569], [0.49897607647039011, -0.010403252244100214, 0.046849195786473175, -0.32202425604129448]]], [[[0.44597817005287776, -0.15805646513978289, 0.66962570699549295, -0.62314847800938744], [-0.61692082273961146, 0.22088164537570232, 0.68823694440407768, -0.54817719482377592], [0.13092507486506988, -0.38768295304299594, -0.26250766251874946, -0.020094679080584044]], [[-0.81049573082826165, 0.16068165668178569, 0.25337709546983245, 0.53604096791339551], [-0.43325951698250842, 0.57308382811271863, 0.8779157187028388, -0.4315436666110668], [-0.47430817479538279, -0.29432135991384145, -0.84855309577270099, 0.77697355338037299]]]])
      res=inner(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.113297048661)+(1.-msk_ref)*(5.16080189005)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_float_rank0(self):
      arg0=-0.081861603174
      arg1=0.765137400833
      res=outer(arg0,arg1)
      ref=-0.062635374280529704
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_array_rank0(self):
      arg0=0.470792845884
      arg1=numpy.array(0.986628916768)
      res=outer(arg0,arg1)
      ref=numpy.array(0.46449783555674595)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_Symbol_rank0(self):
      arg0=-0.101616871453
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.565708860786)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.057485564586096022)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_constData_rank0(self):
      arg0=0.806587192294
      arg1=Data(0.459316802107,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(0.37047904978481383,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_expandedData_rank0(self):
      arg0=0.62510111232
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.932536673263)+(1.-msk_arg1)*(-0.39927472313)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.58292971173616104)+(1.-msk_ref)*numpy.array(-0.24958707354962301)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_array_rank1(self):
      arg0=-0.147579432862
      arg1=numpy.array([0.4366735032511968, -0.4840678581242126])
      res=outer(arg0,arg1)
      ref=numpy.array([-0.064444027955627856, 0.071438459968642856])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_Symbol_rank1(self):
      arg0=0.726559338492
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.5301158194866995, -0.43139233925845621])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.38516059913063933, -0.31343213264233444])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_constData_rank1(self):
      arg0=0.307392721359
      arg1=Data(numpy.array([-0.55600277881165461, -0.97997933943118731]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([-0.1709112072620588, -0.30123851602330931]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_expandedData_rank1(self):
      arg0=-0.533630322003
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.37338763703449107, -0.059381069842770806])+(1.-msk_arg1)*numpy.array([0.66805487237042271, -0.44911334201438247])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.19925096498253622, 0.031687539421061568])+(1.-msk_ref)*numpy.array([-0.35649433665848967, 0.23966049731483582])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_array_rank2(self):
      arg0=0.507057355338
      arg1=numpy.array([[0.82727299573137292, -0.94610810308954418, -0.57533662315989043, -0.28952788602325241, 0.018136635315888716], [-0.92320697591850909, -0.40737177126670399, -0.067864270738446253, -0.44712910432163211, -0.26252030844042507], [-0.6037930078105489, 0.95214729595957825, -0.34240635693853072, -0.64429323264604199, 0.76924375662754518], [-0.56483829149320042, 0.816975372319108, 0.76638795986714259, -0.46644827039746395, -0.35838294865216191]])
      res=outer(arg0,arg1)
      ref=numpy.array([[0.41947485735778095, -0.47973107261607756, -0.29172866656833152, -0.14680724418344251, 0.009196314337997431], [-0.46811888763848197, -0.20656085297769719, -0.034411077642552856, -0.22672010113180602, -0.13311285332021847], [-0.30615768571176444, 0.48279328978113084, -0.17361966180004085, -0.32669362260742862, 0.39005070484553961], [-0.28640541027796246, 0.41425337166409515, 0.38860265209282813, -0.23651602638954558, -0.18172071014166363]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_Symbol_rank2(self):
      arg0=-0.98443044389
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.3374920159765622, -0.52576581720217441, 0.3160254440817325, -0.16216754278020518, 0.79824223591788046], [0.69357054284718744, -0.38580732649335014, -0.61784685731011724, 0.24621889158688592, -0.70762306603983793], [-0.31589471921938239, -0.98653928824618475, -0.12577286327427495, 0.32942863889014395, 0.0089694343303994284], [-0.71907440122444277, -0.011545381705613256, -0.53224388487095342, 0.5977797181914648, -0.87674886038971689]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.3322374150969743, 0.51757987681026996, -0.3111050681977609, 0.15964266612358924, -0.78581395863599768], [-0.68277195736374841, 0.3798004776756756, 0.60822725599754035, -0.24238537273886238, 0.69660568900805697], [0.3109763786634902, 0.97117930944263742, 0.12381463562234972, -0.32429958121254304, -0.0088297842193129604], [0.70787873198696516, 0.011365625237330739, 0.52395708384099238, -0.58847255332737269, 0.86309826981307503]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_constData_rank2(self):
      arg0=-0.461803900652
      arg1=Data(numpy.array([[-0.52799308286320312, -0.69180625252696748, -0.70146211143834436, -0.79159352615299805, 0.23646350770741109], [0.25829241615476461, 0.3018732714936927, 0.29415929156311038, -0.56921835144236077, 0.60564013367395853], [-0.5177543192396723, 0.47622606084243757, -0.6682456782013706, -0.51288857119335751, -0.48979664027252068], [-0.38711325402410046, 0.62799192622923905, -0.10406963163713212, 0.86589686023678203, 0.48939305358190266]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.24382926518345957, 0.31947882591234072, 0.32393793922175912, 0.36556097810826205, -0.10919977022111776], [-0.11928044528907927, -0.1394062542783433, -0.13584390825684978, 0.26286725501873759, -0.27968697612198423], [0.23910096420426005, -0.21992305248913621, 0.30859846078718056, 0.2368539427768824, 0.22618999900405529], [0.17877041070238711, -0.29000912111057531, 0.048059761829436061, -0.39987454761959623, -0.22600362109607669]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_expandedData_rank2(self):
      arg0=0.542639921047
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.74293955171720949, -0.6474141214322553, 0.89013588888960116, -0.14757583754509063, 0.081887518972254991], [0.21487307077342988, -0.050194366663281365, -0.93093206605908807, -0.4171126339311515, -0.67949074748359117], [-0.39271355958770982, 0.91153629318962737, -0.43013281164693429, 0.75192193609147018, -0.50864788267834871], [-0.17454337482838866, -0.48266661220717322, -0.6330500907575769, -0.85640884246823479, -0.72710868516954452]])+(1.-msk_arg1)*numpy.array([[0.13502712344398327, -0.30609572077649028, 0.076158340221367027, -0.76620762911605222, -0.75414774859793399], [-0.045647655526027675, 0.51826952770744628, 0.9282009241583189, 0.62578373001108756, 0.27753347187078115], [0.7635669473363762, -0.13422090527602437, -0.85991252821868414, -0.50396182803855938, 0.35889330854675716], [0.57190812057601392, 0.4595840391218784, 0.51334244288848341, -0.85329159837300739, 0.86017378063831629]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.40314865968646924, -0.35131274773866755, 0.48302326846809335, -0.080080540833902758, 0.044435436829833551], [0.11659870615960571, -0.02723746716316373, -0.50516090282636039, -0.22634196674407769, -0.36871880556661635], [-0.21310205496873427, 0.494635982167832, -0.23340723495178708, 0.40802286003413124, -0.27601264689726801], [-0.094714203136141789, -0.26191417234009035, -0.34351825126744434, -0.46472162666085692, -0.3945581995129398]])+(1.-msk_ref)*numpy.array([[0.073271107604837349, -0.16609975775495828, 0.041326555724787953, -0.41577484736909115, -0.40923067475690406], [-0.024770240190621184, 0.28123373559619913, 0.50367887620095897, 0.33957523384567084, 0.15060074126384146], [0.41434190801665766, -0.072833621441829532, -0.46662286641985401, -0.27346980657751113, 0.19474983661408432], [0.31034017739546715, 0.24938864670352595, 0.27856010267904557, -0.46303008557113867, 0.4667646324122165]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_array_rank3(self):
      arg0=0.329599447984
      arg1=numpy.array([[[0.32822728567176518, 0.76182410862170369], [-0.91107903577061466, -0.82291242821270738]], [[-0.44260136214831292, -0.61872702571531413], [-0.82643984233761358, 0.59956489999270834]], [[0.1872384874131392, -0.95740409540478066], [0.34436766116193884, -0.24392537634176192]], [[0.55128906940654687, -0.055866350184200497], [-0.14960910116281423, 0.39785898000498165]], [[-0.69334745701646527, -0.36482981315924334], [0.62969646397122481, 0.85157868691814409]], [[-0.4054544548193193, 0.40561559734997887], [0.023758608459736985, -0.58272572766033925]]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[0.10818353217063062, 0.25109680566245424], [-0.30029114725959566, -0.27123148207790621]], [[-0.14588116464095621, -0.20393208612841801], [-0.27239411582628553, 0.19761626006805122]], [[0.061713702092689962, -0.31555986134283281], [0.1135033910224429, -0.080397669391482268]], [[0.18170457295589357, -0.018413518181581431], [-0.04931107715661414, 0.13113410018503457]], [[-0.22852693909358954, -0.12024770502531482], [0.2075476069222584, 0.2806798651229786]], [[-0.13363756449101502, 0.13369067698016712], [0.0078308242331922451, -0.1920660781627985]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_Symbol_rank3(self):
      arg0=0.919807317132
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[-0.55754776587765598, 0.28594849631215613], [-0.58697580165350427, 0.085453966486368937]], [[-0.75286055329872736, 0.34273933589263694], [-0.15060145658769941, -0.94232463825279988]], [[-0.40393189781704875, -0.25007169909298854], [-0.93292559580244649, -0.34616235151020591]], [[-0.65743006449546648, -0.15834455396877423], [-0.77465977785317319, 0.50612786455318615]], [[-0.93203814231272553, 0.42336941766670089], [0.58156223174772181, -0.25735383631207576]], [[-0.9560608584980328, 0.067738346141000161], [-0.1371257626427338, -0.59757935921724936]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.51283651470511682, 0.26301751923094197], [-0.53990463734057748, 0.078601183652153114]], [[-0.69248664570455254, 0.31525414902316323], [-0.1385243217401706, -0.86675709737911211]], [[-0.37153951523531759, -0.23001777863347453], [-0.8581117893592386, -0.31840266383486182]], [[-0.60470898382578708, -0.1456464793685523], [-0.71253773195754511, 0.46554011322064104]], [[-0.85729550314577851, 0.38941828821993485], [0.53492519612943079, -0.23671594173195348]], [[-0.87939177327042028, 0.062306226430942448], [-0.1261292798461538, -0.54965786717534537]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_constData_rank3(self):
      arg0=0.676272532347
      arg1=Data(numpy.array([[[-0.78502641500144188, -0.2959496388941778], [0.9709955494182978, -0.72028076814454289]], [[-0.42229143063252117, -0.57707492065624755], [-0.37028192685951566, -0.77738667668782058]], [[-0.52303963001352782, -0.74377516733057392], [0.32553195575683924, -0.14808641834981451]], [[-0.51098172394733266, 0.48839251208215906], [-0.05041292302866851, 0.43615558206766214]], [[-0.18673861291251259, -0.77362964466616568], [-0.10240459910802069, 0.31449844481000633]], [[0.71163512256337746, 0.6064924623513579], [-0.012160009387012227, -0.56867060961205484]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.53089180163193517, -0.20014261174200376], [0.65665761910231268, -0.48710609907360664]], [[-0.28558409518208988, -0.39025991794586762], [-0.25041149635943355, -0.52572525645611778]], [[-0.35371733510683534, -0.50299471590710387], [0.22014832007939297, -0.10014677714355522]], [[-0.34556290443665305, 0.33028644092488008], [-0.034092875119587848, 0.29496003998196829]], [[-0.12628619464122146, -0.52318447889672626], [-0.069253417562711334, 0.21268665969070522]], [[0.48125928644266136, 0.41015419336342923], [-0.0082234803415122126, -0.38457631323338359]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_expandedData_rank3(self):
      arg0=0.305104655943
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.30293977445085996, 0.80288896011525268], [0.43540848013586975, 0.40694710798426192]], [[-0.59373430985932441, -0.10176913923597497], [-0.30052400560538772, 0.48490916950152085]], [[0.76780025804496965, -0.13092427930556072], [0.71353871888096498, -0.21625028363463095]], [[0.048762689975171636, 0.027421493530511487], [-0.80924187302600847, -0.71846315628216839]], [[-0.71726816616769762, 0.39535977450661641], [0.98564221665367024, -0.34445301274369311]], [[0.055019366867498087, -0.40840665672813459], [0.45115914313174343, -0.18779261237584977]]])+(1.-msk_arg1)*numpy.array([[[-0.61380336879006725, 0.58597512263121998], [-0.6415509561610433, -0.94320197837698028]], [[-0.67863445716181414, -0.49823099756251277], [-0.84557582756626659, -0.97939605205818969]], [[-0.44458557380422148, -0.40082933239532137], [0.63008051889144734, 0.76000112210178639]], [[0.15432738338689655, -0.14349590068753426], [0.79389836449546203, 0.6986714070934783]], [[-0.27877479322191001, -0.2681783738236474], [-0.4386996383193229, -0.32550724130405539]], [[-0.74293716545466415, 0.67250397682981777], [-0.47603169981880589, -0.37618199584110656]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.09242833565526147, 0.24496515993634904], [0.13284515452649295, 0.12416145736851268]], [[-0.18115110233114812, -0.031050238212201299], [-0.091691273332825993, 0.14794804532433831]], [[0.23425943356371101, -0.039945607192100482], [0.21770398532614302, -0.065978968385907263]], [[0.014877723747726992, 0.0083664253490682624], [-0.24690346324422066, -0.21920645410514972]], [[-0.2188418570574189, 0.12062610797451954], [0.30072402939495479, -0.10509421794167362]], [[0.016786664998306396, -0.12460677248584391], [0.1376507551407222, -0.057296400387559537]]])+(1.-msk_ref)*numpy.array([[[-0.187274265651311, 0.17878373818152046], [-0.19574018374937929, -0.28777531509740889]], [[-0.20705453256337916, -0.15201259709141823], [-0.25798912194327256, -0.2988182954950877]], [[-0.13564512853273156, -0.12229489555231286], [0.19224049993272416, 0.23187988087511388]], [[0.047086003210833269, -0.043781267408492426], [0.24222208735305073, 0.21316789927842547]], [[-0.085055487371535093, -0.081822470476801093], [-0.13384930221170924, -0.099313774865009377]], [[-0.22667358825326839, 0.20518409447092045], [-0.14523948799114966, -0.1147748784130293]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_array_rank4(self):
      arg0=-0.311141703693
      arg1=numpy.array([[[[-0.96242947025367509, 0.31613060708727825, -0.16474461524119355, 0.097607895254317345], [-0.4608076213705925, -0.045189493507184153, -0.23256772880073062, -0.025792894346069906], [0.47221318146065183, -0.53932971340006897, -0.9206012533737864, -0.43277979347555129]], [[0.52164453678492451, -0.4177205567007356, 0.36074511135280884, 0.44545791479432495], [-0.60244720292977383, 0.73113422773618209, -0.35506077006676295, 0.12888290818921266], [-0.018600374399256658, 0.34889421111020491, 0.0095060116754939195, -0.12269843664561075]]], [[[0.11970738714252849, 0.0046044121318216114, 0.7711293226528344, 0.083756858882147567], [-0.96727123111763413, -0.62793542034570082, 0.66371387282856142, -0.27404421582471739], [-0.025508922135425216, -0.5860353376499825, -0.93418277964334107, 0.098139531888658205]], [[0.57463111535317091, 0.58390453459973113, -0.19312942733947369, 0.68367140971995921], [0.020894590195250196, -0.71205314610397163, 0.15500610556452066, 0.99622189868944999], [0.089625133479853458, 0.87254489846494532, -0.27731691759675536, -0.94015242319703418]]], [[[0.13787664510865594, -0.92567081934687612, -0.23120204812863565, -0.27798371946865919], [-0.17735552235919139, 0.21911400717697926, -0.11484383008164167, -0.93977080140677649], [-0.041189774819279545, -0.96264056886361749, -0.67189498094862232, 0.15088102270862103]], [[0.095815805990953073, 0.7429753189232895, -0.37866380561605983, 0.91725934981023949], [-0.18496465764990355, -0.47182302550197419, 0.13864024675702846, 0.87186119166113274], [0.28701359062133958, -0.17167237546109382, 0.63761401351486469, -0.43423124188146778]]]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[[0.29945194505864181, -0.098361415678494227, 0.051258920260317739, -0.030369886823271755], [0.14337646838775525, 0.014060335998828468, 0.072361519362965046, 0.0080252450899979972], [-0.14692521378574402, 0.16780796587930935, 0.28643744239621199, 0.13465584226569072]], [[-0.16230536989716976, 0.12997028567926514, -0.11224284854506973, -0.1386005345324347], [0.1874464491043781, -0.22748634924576874, 0.11047421291295953, -0.040100847630841464], [0.0057873521799039112, -0.10855553925329554, -0.0029577166680344004, 0.038176600618327103]]], [[[-0.037245960380109341, -0.0014326246351975983, -0.23993049121748097, -0.026060251768527446], [0.30095841878272594, 0.19537689649525561, -0.20650906515625561, 0.085266584198790005], [0.0079368894925766695, 0.18234003338045129, 0.29066322161846631, -0.030535301151425942]], [[-0.17879170422573282, -0.18167705168916282, 0.06009061905556938, -0.21271868718615194], [-0.0065011783913076883, 0.22154942899842625, -0.048228863768091407, -0.30996617881405725], [-0.027886116724593339, -0.27148510625662159, 0.086284858203819523, 0.29252062668419959]]], [[[-0.042899174258519579, 0.2880147957900609, 0.071936599151949429, 0.086492328074269056], [0.055182699386120027, -0.06817550549594581, 0.035732704950179113, 0.29240188823021263], [0.012815856711982919, 0.29951762663978643, 0.20905454907482426, -0.046945378460433888]], [[-0.02981229311669948, -0.23117060653130411, 0.11781810160608347, -0.28539763682787384], [0.057550218704099279, 0.14680381999605527, -0.043136762576336647, -0.27127237655685715], [-0.08930189756883819, 0.053414435377910949, -0.19838831046325628, 0.13510744839552941]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_Symbol_rank4(self):
      arg0=-0.111922144748
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=outer(arg0,arg1)
      s1=numpy.array([[[[-0.82058593032213789, 0.57777086667213129, 0.11005843471741805, -0.03559890834505719], [0.72389998035039249, -0.50416194299085926, -0.46057311463524719, -0.15729602546149657], [-0.34008426955026017, 0.062821636548204607, -0.39542655418137529, -0.74127730536860703]], [[0.34004465549432261, -0.25081472805409, -0.35861429698781899, -0.68691535610963284], [-0.0094152650227661283, -0.87988743713452089, 0.34011246187301225, 0.64806304518974267], [-0.99945801544102686, -0.21680273713805787, -0.27058462116010928, 0.95480352893431575]]], [[[-0.60612016965312021, 0.25726804525611358, 0.48612124931942691, 0.082483131804683474], [-0.7472148132284302, 0.69600980898468734, -0.95410695279794222, -0.029308572438860647], [0.72990151707198692, 0.90557781923312364, -0.57276048695481663, 0.11775359268890351]], [[-0.71551556575308672, -0.22338854994744772, -0.60512218700540221, 0.5140540472063726], [0.84065980533819462, 0.92545325396365352, -0.32115613548273969, -0.80992720171403865], [0.73902640621023918, -0.083865619539678304, 0.53428912058641531, -0.55973480990477809]]], [[[0.94251761773170717, 0.11663035422671175, -0.73182088262176892, 0.35617553042270056], [0.046808918993267001, -0.114521429431653, -0.88441571362118054, -0.9659424474275764], [0.43132887243545825, 0.22700422485379113, -0.024998626366275367, -0.64025923943350027]], [[0.36981298877494084, 0.33604559918531107, -0.3139233631262297, 0.41468543351352882], [-0.89836021489246654, 0.56852421712720336, -0.40023788382921088, -0.83084371003221169], [-0.72253714998569851, 0.78486376410638692, -0.57995214413334373, -0.034605008782604862]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.091841737271952675, -0.064665354571043054, -0.012317976061216863, 0.0039843061726778208], [-0.081020438384085736, 0.056426885960019373, 0.051548330803392696, 0.017604908530037721], [0.038062960843232352, -0.0070311322990747761, 0.044256988034418986, 0.082965345870113019]], [[-0.038058527153129643, 0.028071722298281455, 0.040136881256289232, 0.07688103991634905], [0.0010537766547218655, 0.098478889101201936, -0.038066116188460203, -0.07253260594976621], [0.11186148467406354, 0.024265027327798616, 0.030284411136152243, -0.10686365877159733]]], [[[0.067838269362795398, -0.028793991400273188, -0.054407732831565143, -0.0092316890171388957], [0.083629884484244466, -0.077898910587437711, 0.10678569647643391, 0.0032802782868689095], [-0.081692143245752413, -0.10135421176508151, 0.064104582127077689, -0.013179234645562679]], [[0.080082036719896124, 0.025002125622336516, 0.067726573004441182, -0.057534031479893562], [-0.094088448417159548, -0.10357871304792775, 0.035944483482311641, 0.090648789505843874], [-0.082713420408696295, 0.0093864200095277687, -0.05979878429172767, 0.062646720414838361]]], [[[-0.10548859323961392, -0.013053519387810368, 0.081906962754640059, -0.039863929271780674], [-0.0052389546070770159, 0.012817484001634473, 0.098985703517602289, 0.10811035041953983], [-0.048275252494852865, -0.025406799712567166, 0.0027978998786755476, 0.071659187272328242]], [[-0.041390262859479353, -0.037610944194055751, 0.035134976087694648, -0.046412483114722784], [0.10054640200733035, -0.063630449722238608, 0.044795482367695941, 0.092989809977459967], [0.080867907486741053, -0.087843635814029694, 0.064909487822793127, 0.0038730668019837347]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_constData_rank4(self):
      arg0=-0.672892796531
      arg1=Data(numpy.array([[[[-0.15035387426968971, 0.7448082478008573, 0.19832246991022884, 0.66992547222564558], [-0.87242392963594284, -0.043981690167481657, 0.070163273395474457, 0.41278977409290363], [0.92244264078304616, -0.52871594235556008, -0.80680164879181415, 0.3638857320985347]], [[0.10604864161257521, -0.88216765288310506, -0.91687431340483028, 0.64065686176858061], [-0.57691631049459136, 0.20733489672099492, -0.23698932317186117, 0.12997631174408308], [0.74193649559644603, -0.46758879464846692, -0.11845259801302221, 0.92706200519521409]]], [[[-0.91657636595994929, 0.62692336839030194, 0.77538458911871211, -0.78481494192413681], [0.95702122108174481, 0.41867966052078098, 0.0038112330327841892, -0.4421255675128164], [0.79198271993779445, 0.41795257294183141, 0.56507785727994664, 0.49783786847345235]], [[0.91348936829492056, -0.053708934298375421, -0.11362449808283137, -0.12015710605876295], [0.72087868244402276, 0.25135052985676531, 0.19612782696734055, -0.8311278206221322], [0.9492318741345187, 0.96648002899763163, 0.84111363429424535, 0.73138033303662486]]], [[[0.82118391220944731, -0.64895386300413671, 0.52527934759865125, 0.83792978829018172], [-0.97163515948825285, 0.94417103202880415, -0.56367072296837883, -0.22800894175864372], [-0.74379926261052098, -0.041615774090976387, 0.87421839143037627, -0.73199242523809449]], [[0.8704284040109711, -0.91798362066932904, 0.27031454140249722, 0.23116809961765994], [0.34664387098973815, 0.69188536416441937, 0.98111032452439617, -0.47437466946671369], [0.55282380606543646, 0.12145535131127505, -0.43470110890054059, 0.1573076701434708]]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.10117203892652984, -0.50117610474171603, -0.13344976139273398, -0.45078802447294447], [0.58704777777287598, 0.029594962492935646, -0.047212361248816302, -0.27776326546857588], [-0.62070500819550267, 0.35576914902190249, 0.54289101770095904, -0.24485608788933896]], [[-0.071359367022948705, 0.59360425895727842, 0.6169581208139775, -0.43109338733192759], [0.38820282953277591, -0.13951415847295701, 0.15946840841698903, -0.087460123892198857], [-0.4992437233699471, 0.31463713165734236, 0.079705899933288138, -0.62381334523299992]]], [[[0.61675763412457241, -0.42185221856648425, -0.52175070455875916, 0.52809632103027082], [-0.64397268579274924, -0.28172652761827743, -0.0025645512536596514, 0.29750310954134268], [-0.53291946722279082, -0.28123727562395545, -0.38023681964257788, -0.33499151553589501]], [[-0.61468041563286802, 0.036140354998707852, 0.076457106269333219, 0.080852851118895397], [-0.48507407258899582, -0.169131960944747, -0.1319730019655079, 0.55925992349274367], [-0.63873129034228371, -0.65033744950311534, -0.56597930558020459, -0.49214055762443826]]], [[[-0.55256873915248883, 0.4366763796961381, -0.35345668916538403, -0.56383691853880769], [0.65380629967542914, -0.6353258861449701, 0.37928996910057294, 0.15342557445393845], [0.50049716587533277, 0.028002954607859499, -0.58825525818799951, 0.49255243005761967]], [[-0.58570500295454042, 0.61770456568139775, -0.18189270770719163, -0.15555134902037324], [-0.23325416375045002, -0.46556467757113401, -0.66018206997418782, 0.31920329794069852], [-0.37199115685201789, -0.081726430997440744, 0.29250724482300328, -0.10585119807854079]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_float_rank0_expandedData_rank4(self):
      arg0=0.184226502646
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.65730545940807961, -0.4190764869923187, -0.91233525335447063, 0.94305454000779299], [0.9595976861826756, -0.25922356850125716, -0.47777030754140215, 0.79357928766377217], [0.82177864020114644, 0.57356814342818963, -0.44599790358764069, -0.88625831217822171]], [[0.49549282141744944, -0.71265510458079873, -0.44671696505642289, -0.67951574493726463], [0.74664559903254291, 0.24573973378307845, 0.52339278211342077, 0.065647498013209127], [-0.29054338210583186, -0.24512034012153916, 0.52316907153411552, 0.4542799080891522]]], [[[0.68013291879022675, -0.59070959949748247, -0.4160977604455971, -0.43475815273698326], [-0.88030221968057565, 0.25403036852465499, -0.12823056233409758, -0.65367638618314627], [-0.52340105656477909, -0.48022748873278265, -0.28561993280419773, -0.24563030027621902]], [[-0.77189443540773661, 0.76873304337010162, 0.57399920199108667, 0.69732176176072125], [0.14482102250488738, -0.79852504962461524, -0.79507037231493638, -0.089040013221562742], [0.5770105324062218, -0.96386024653463465, -0.46049683884974102, 0.11283893421851321]]], [[[-0.55499542827162029, -0.68767713721104951, 0.14947438804422863, -0.25844280664845853], [0.91170284074015062, 0.9253661175288237, 0.83613688420269416, -0.93229398965659227], [-0.17563135055601586, 0.49891895617816684, -0.46929202386808755, -0.057064086850217244]], [[0.11910718268125309, 0.044264612600426467, -0.44246145558194883, -0.34283821975409845], [0.74769433079688574, 0.99312538575709097, 0.9078007214123327, 0.58344305988196599], [-0.87486562805907941, -0.74611549307091285, -0.00091221631963400718, -0.95022821552442194]]]])+(1.-msk_arg1)*numpy.array([[[[0.29776538578889933, 0.83374470873797013, 0.61415531489559494, -0.71595656682753428], [0.21585625924589302, -0.14697143751100739, -0.084575556424320242, 0.47798305756602555], [-0.1338160730259188, -0.49693412037852136, 0.45330650019972452, 0.39188345056255458]], [[-0.37922219968267679, -0.20117902721507952, -0.83581818808135022, 0.60961342345292491], [0.99750808819140802, -0.67257491810321568, -0.63236999997628573, -0.88694252684793451], [-0.69149659826752496, -0.53943479886510315, -0.92239128362138367, 0.14263274986737273]]], [[[0.10260430817581145, -0.55598580788903007, -0.33118324349604222, -0.13224437802571609], [-0.87170370823813759, -0.7395010730547209, 0.53349827266138816, 0.18763994168424558], [-0.079441770664999467, 0.15424036810085751, 0.54771635762931381, 0.70513379076545357]], [[-0.2022059207172906, 0.17229783384265307, 0.02013953811621394, 0.016855182454158291], [0.2794366381731519, 0.14338154424900629, 0.18037912383416299, 0.80754634988290008], [-0.019647626451636979, -0.56963585459384491, -0.85708942953768896, -0.94946928910942985]]], [[[0.15117646226304982, -0.46746072048210907, -0.95653799430305986, 0.94491270358315815], [0.29755076195009256, 0.5322007738011707, 0.96013909022432498, 0.37016824938675996], [0.0015505646618996227, -0.083969383227496142, -0.31112268986924185, 0.63143348905277041]], [[0.66576238925902853, -0.67268178702623427, 0.026781312366727317, 0.54021600142687909], [0.059925616543585969, -0.4159475667789827, -0.45691210248484726, 0.70846379196415565], [-0.69182695381065407, 0.80175306625454734, 0.31836086192875279, 0.57079550481579777]]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.12109308595670129, -0.077204995539657414, -0.16807633296590838, 0.17373563970982189], [0.17678332567237773, -0.047755851428334765, -0.08801795282633168, 0.14619833673839361], [0.15139340483322833, 0.105666453092785, -0.082164633965282549, -0.16327226929330937]], [[0.09128290957580644, -0.13128995750955394, -0.082297104144863603, -0.12518480918250632], [0.13755190742559817, 0.045271771715951509, 0.096422821758778823, 0.012094008966416618], [-0.053525791152222002, -0.045157662987925187, 0.096381608341148561, 0.083690398689492282]]], [[[0.12529850896296191, -0.1088243635946864, -0.076656235165616884, -0.080093973975456464], [-0.16217499920303352, 0.046799126359105415, -0.02362346803110723, -0.12042451448862652], [-0.09642434613201388, -0.088470630723586588, -0.052618761306428334, -0.045251611163710535]], [[-0.14220341224687463, 0.14162100004828904, 0.10574586550426303, 0.12846514938794293], [0.026679870485655267, -0.14710947716735806, -0.14647303404882633, -0.016403530231338868], [0.1063006323749541, -0.17756860225833554, -0.084835722100706279, 0.020787922213349297]]], [[[-0.10224486673485472, -0.12668835393782565, 0.027537143744500309, -0.047612014402794466], [0.16795982580174315, 0.17047696349920111, 0.15403857390976763, -0.17175326115207679], [-0.032355949467882575, 0.091914094400366408, -0.086456028276758454, -0.010512717147088228]], [[0.021942699705354392, 0.0081547147703450798, -0.081513126517405443, -0.06315988619858888], [0.13774511161075656, 0.18296001650672933, 0.16724095200507291, 0.10748567441498308], [-0.16117343494229214, -0.13745424785825541, -0.00016805442222254076, -0.1750572208613658]]]])+(1.-msk_ref)*numpy.array([[[[0.054856275632848166, 0.15359787179018658, 0.113143685744508, -0.13189817435288698], [0.039766443715062841, -0.027076033921469671, -0.015581058969369883, 0.088057147019305837], [-0.024652467131351836, -0.091548435042671669, 0.08351107115837525, 0.072195317541883797]], [[-0.069862779573163628, -0.037062508589506049, -0.15397986163792568, 0.11230694896862831], [0.18376742644834052, -0.12390612492939977, -0.1164993134737172, -0.16339831976896946], [-0.12739199989025174, -0.099378186400325627, -0.16992892025248146, 0.026276732670810578]]], [[[0.018902432851615351, -0.10242732090806175, -0.061012730684148059, -0.024362919258238698], [-0.16059092551203377, -0.1362356963916424, 0.098284520939950423, 0.034568250213138987], [-0.014635279573597717, 0.028415163582012386, 0.10090386900791133, 0.12990433217005182]], [[-0.037251689588008036, 0.03174182734226863, 0.0037102366720506495, 0.0031051713149854066], [0.051479634561722588, 0.026414680440939704, 0.033230615134270516, 0.14877143976325899], [-0.003619613506474992, -0.10494202127344078, -0.15789858805836002, -0.17491740650216633]]], [[[0.027850710925077213, -0.086118653658676339, -0.17621964933822257, 0.1740779626866551], [0.05481673623364039, 0.098045487262745729, 0.17688306664548908, 0.068194801975018524], [0.0002856551047878403, -0.015469385801321393, -0.057317045048345332, 0.11632678334158844]], [[0.12265107656626197, -0.12392581301732901, 0.0049338275135852533, 0.099522104616139501], [0.011039886754714482, -0.076628565511696972, -0.084175318657294898, 0.13051780664469484], [-0.12745286013659204, 0.14770416338157275, 0.058650508172417133, 0.10515565957812352]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_float_rank0(self):
      arg0=numpy.array(-0.447383148159)
      arg1=-0.833832446992
      res=outer(arg0,arg1)
      ref=numpy.array(0.37304258517261873)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_float_rank0(self):
      arg0=numpy.array([-0.7077557598550499, 0.34384980137545629])
      arg1=-0.117774156535
      res=outer(arg0,arg1)
      ref=numpy.array([0.083355337649769559, -0.040496620331747418])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_float_rank0(self):
      arg0=numpy.array([[-0.74034215690736938, -0.026692072802847244, 0.20164819911474718, 0.83226009454281358, 0.91414431655496875], [0.69310910714139085, 0.40494661103705387, -0.070740970341285792, 0.8162325138343971, -0.24023752347865712], [0.55012364496204058, -0.40587261759026627, 0.18284956859225754, -0.82971461005420166, 0.78296352261416358], [-0.32346840994150194, -0.29416615770821286, -0.93637076734800884, 0.61578785330116848, -0.12974392129838419]])
      arg1=0.964178232458
      res=outer(arg0,arg1)
      ref=numpy.array([[-0.71382179226108633, -0.025735915575689355, 0.19442480420079461, 0.80244706690161316, 0.88139805134749083], [0.66828071382412468, 0.39044070766956146, -0.068206903746024325, 0.78699362246359428, -0.23163179075773749], [0.53041724363284937, -0.39133354303128232, 0.17629957385098963, -0.79999276616663406, 0.75491638531320893], [-0.31188119975339523, -0.28362860598806422, -0.90282831138693886, 0.59372924396502325, -0.12509626470964516]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_float_rank0(self):
      arg0=numpy.array([[[-0.15791645491015305, 0.41702383789148212], [-0.90077914152118699, 0.0088427934648849238]], [[0.015777707296524834, 0.02048903309684591], [-0.15738286559336179, -0.84730278576106199]], [[-0.82290207937519555, -0.14283446280549805], [0.65481568668046997, -0.045837257239726314]], [[0.5564462185637844, 0.086215516976181572], [-0.58292048601627267, 0.26964649866065393]], [[0.38964230741750172, 0.75334221328525208], [-0.20459531306113932, 0.51971180708870413]], [[-0.96004303118597267, 0.076424350124239648], [-0.36806507639747976, 0.95496844011225734]]])
      arg1=-0.639352122152
      res=outer(arg0,arg1)
      ref=numpy.array([[[0.10096422056954935, -0.26662507574394978], [0.57591505572195523, -0.0056536587675272655]], [[-0.010087510642728482, -0.013099706791313899], [0.10062306910750114, 0.54172483418175643]], [[0.52612419077194139, 0.091321516911156322], [-0.41865779889767035, 0.029306147689852638]], [[-0.35576507070229002, -0.055122073741165685], [0.37269144978046176, -0.17239906114958373]], [[-0.24911863612763688, -0.48165094277071724], [0.13080844758802121, -0.33227884676968744]], [[0.6138055493461263, -0.048862070436030913], [0.23532318768481883, -0.61056109877409215]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[-0.87020281833627244, 0.38653786763242937, -0.39861176181522451, -0.65499761474764417], [0.05055951134247616, -0.025737969733530264, -0.12216547307173986, -0.87787279426303932], [-0.80700029009802332, 0.94697847812541758, 0.29159653307353262, 0.63930031280993882]], [[0.23785001278341023, 0.93992568249991604, 0.22199263237551592, -0.052778005340911482], [-0.16862506415237455, 0.25374353614534706, -0.11888781793537628, -0.65094094688477777], [-0.44952265798288749, -0.15066801692167542, 0.94480116118414803, 0.32746587459986021]]], [[[0.78917025548681541, -0.084561499269018459, -0.84479426524784618, 0.74962830861495], [0.97673242354717993, -0.71494837460876259, -0.80013195830983519, 0.12796340276698337], [0.019165508803590026, -0.21892235269927185, -0.39596112938741901, 0.64963194873588415]], [[0.95797829118030275, 0.53000085273002395, -0.93284698498085139, -0.86034783100269485], [-0.41924351261022297, 0.064151217188172405, -0.89760216010433047, -0.81302206425904222], [0.68233887569792473, 0.18494775692731746, 0.73925938459522, -0.36536865017631448]]], [[[-0.94205154403397806, 0.21091249594021089, -0.076130048892607549, 0.36845534393432722], [0.5111382941147895, -0.54496836419397821, -0.11635448135542692, -0.16379745454182637], [-0.8580412291711903, 0.74391783752126805, -0.6049896435642117, -0.90606576695648267]], [[0.91914643333426516, -0.92427239857047083, 0.1023644003817421, -0.1239878505251808], [0.084055849795886317, -0.41367241918571263, -0.71001744057354998, -0.19230922867658373], [-0.50023319705649949, 0.94827281200621139, -0.2159049077858417, -0.39167467063244432]]]])
      arg1=-0.568664125355
      res=outer(arg0,arg1)
      ref=numpy.array([[[[0.49485312457061714, -0.21981021841376658, 0.22667620888885412, 0.37247364570005387], [-0.028751380295943364, 0.014636260046930409, 0.069471121892915821, 0.49921476472250564], [0.4589121141297911, -0.53851268799316132, -0.16582048743679897, -0.3635471532232159]], [[-0.13525676948514392, -0.53450201613747816, -0.12623924612506784, 0.030012958245168815], [0.095891024619134027, -0.14429484604656836, 0.067607237001580431, 0.37016676421796135], [0.25562740912907533, 0.085679496061730795, -0.53727452595913372, -0.18621809516292637]]], [[[-0.4487728130925599, 0.048087091020520438, 0.4804041919520522, -0.42628672645983823], [-0.55543268934228696, 0.40656549212084198, 0.45500634024081343, -0.072768196511931019], [-0.0108987373007763, 0.1244932882183814, 0.2251688893176586, -0.36942238393052951]], [[-0.54476788706308565, -0.30139247135510183, 0.53047661480414698, 0.48924894681818404], [0.2384087454092334, -0.036480495812768111, 0.51043414729245151, 0.46233648106615199], [-0.38802163994444683, -0.10517315442943466, -0.42039029135128647, 0.20777204388463608]]], [[[0.53571091732737131, -0.11993837003027148, 0.04329242766674498, -0.20952733589077499], [-0.29066601095821287, 0.30990395817049171, 0.066166619371113802, 0.093145736222396441], [0.48793726510512936, -0.42303938640998484, 0.34403590650625121, 0.51524709688037884]], [[-0.52268560258516061, 0.52560055512280734, -0.058210962210568262, 0.070507442573543411], [-0.047799546305145553, 0.23524066443971348, 0.40376144683052478, 0.1093593593230564], [0.28446467347764942, -0.53924872923740019, 0.12277737554587888, 0.22273133399889083]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_array_rank0(self):
      arg0=numpy.array(-0.207683000649)
      arg1=numpy.array(0.928165743638)
      res=outer(arg0,arg1)
      ref=numpy.array(-0.19276424673857764)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_array_rank0(self):
      arg0=numpy.array([0.93891187273728738, -0.83684388514200214])
      arg1=numpy.array(0.325006863482)
      res=outer(arg0,arg1)
      ref=numpy.array([0.30515280284446328, -0.27198000633418834])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_array_rank0(self):
      arg0=numpy.array([[0.074246212290295599, -0.1783493867998498, 0.06354609168547376, 0.77790826354226272, -0.50614134071462624], [-0.41467785638114374, 0.87706967672955005, -0.25212998844060541, -0.93711898146715567, -0.34260493725944197], [-0.43608695675866183, 0.86498660822160822, 0.45142390637338137, -0.29020859837544655, 0.83090104827681044], [0.31739964223258488, 0.33895596714672016, -0.83141613296839489, 0.49329189927812389, 0.018359301817349793]])
      arg1=numpy.array(0.771453015092)
      res=outer(arg0,arg1)
      ref=numpy.array([[0.057277464330503514, -0.13758817218653971, 0.049022824028066493, 0.60011967537460065, -0.3904642633569666], [-0.31990448259708865, 0.67661804655870939, -0.19450643977759668, -0.7229432637527089, -0.26430361183417556], [-0.33642059763373061, 0.66729652692669539, 0.34825233365631891, -0.22388229822233913, 0.64100111893618472], [0.24485891098942519, 0.26148860283873598, -0.64139848257453524, 0.38055152301852974, 0.014163338741977116]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_array_rank0(self):
      arg0=numpy.array([[[0.22890218267521778, 0.14190434160962129], [-0.84482969323319335, 0.91841360418106577]], [[-0.45435795577157667, -0.55641507914234856], [-0.98245728870817817, -0.32379807029523966]], [[0.5404127425337717, -0.65902737365430708], [0.2880201704204961, 0.69027355927695666]], [[0.31775056241027944, 0.46458796273435676], [-0.98236559656202838, 0.53823105267672466]], [[-0.99671747177264791, -0.97760339687001663], [-0.020380770488373612, -0.12334078575248042]], [[-0.95617586546311739, 0.58649753976234509], [-0.44759240577104609, -0.45666066208325384]]])
      arg1=numpy.array(0.628984462801)
      res=outer(arg0,arg1)
      ref=numpy.array([[[0.14397591640387697, 0.089255626076413061], [-0.53138475075635072, 0.5776678874546719]], [[-0.28578409473020422, -0.34997643964855274], [-0.61795036996273456, -0.20366395530055093]], [[0.33991121855325124, -0.41451797858890305], [0.18116021216769854, 0.43417134386733591]], [[0.1998601668022463, 0.29221861016413575], [-0.61789269702744543, 0.33853896953051882]], [[-0.62691980354697951, -0.61489734741241586], [-0.012819187977093804, -0.077579437867938736]], [[-0.60141976308130241, 0.36889783998134418], [-0.28152866889756928, -0.28723246122264223]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[0.72014732698955375, -0.13486060745334227, 0.23474332003161913, 0.67651069382593376], [-0.78542889973998498, -0.13478196489366945, 0.96798211398773448, -0.68984751443494652], [0.22216169271582276, 0.24976428201837564, 0.02856353065993078, 0.22585321890236276]], [[0.55690843170751414, -0.37328922333704595, 0.015239158523985052, 0.84768206207577035], [-0.85714719370844827, -0.045370799462534528, -0.5453445151133498, -0.014684318738259261], [-0.34003935275514685, 0.52980156993147798, 0.79481595972951635, 0.51231503305548731]]], [[[0.89326872458915108, -0.50630791010133658, -0.43917997165505662, 0.80947422995533214], [0.48814414134967454, -0.11445907719210546, -0.12746204293519114, -0.28181681024183125], [-0.47017192506529093, 0.82642588873841349, -0.81703786624207453, -0.29160845380722367]], [[0.060809028860670633, -0.96985541156429012, 0.79244483057402282, -0.59923632202828148], [-0.39639332317995435, 0.10597687153371615, 0.39091924958652591, -0.90537659080526978], [-0.72195636813718789, -0.5289409025268379, -0.92922629277285518, 0.47252160039886726]]], [[[0.30055684213557465, -0.19201302940683496, -0.2904517457543192, 0.39597497120161962], [-0.052595824562017501, 0.018124718453008004, -0.72467427579590482, 0.43256723364373473], [0.21958817986407686, 0.15716805388254484, -0.17828449289135739, 0.062420281231962527]], [[-0.23906809121972139, 0.43514000120426499, 0.61688858358287857, 0.72030306154158108], [-0.39263270024044195, 0.61790128793808297, -0.23700797179115174, -0.87988848288027777], [0.57534289451001275, 0.30915542819777286, 0.18564180823835774, -0.69127657529842268]]]])
      arg1=numpy.array(0.114228590767)
      res=outer(arg0,arg1)
      ref=numpy.array([[[[0.08226141430682879, -0.015404937139412466, 0.026814398639240705, 0.077276863194720422], [-0.089718436365181162, -0.015395953910646709, 0.11057123276873601, -0.078800309418203748], [0.025377217081398371, 0.028530221958956544, 0.0032627718546214312, 0.025798894915467288]], [[0.063614865340356416, -0.042640301930397237, 0.0017407476026737514, 0.096829527369603638], [-0.097910716017431093, -0.0051826424845894574, -0.06229393544405485, -0.0016773690358486836], [-0.038842216070632991, 0.060518486719556792, 0.090790706999233098, 0.058521024254812586]]], [[[0.10203682758628999, -0.057834839065194266, -0.050166909255364066, 0.092465100550213786], [0.055760017357669445, -0.013074499088175694, -0.014559809540803381, -0.032191537088449837], [-0.053707076418539827, 0.094401464644172678, -0.093329084064324511, -0.033310022734219957]], [[0.0069461296726802913, -0.11078521691099369, 0.090519856257273906, -0.068449920601848985], [-0.045279450696398826, 0.012105588689219772, 0.044654154984065199, -0.1034198920813558], [-0.082468058527763025, -0.060420173894805439, -0.10614420992733231, 0.053975476520654854]]], [[[0.034332184522605724, -0.021933377758095979, -0.033177893603407532, 0.045231662939468953], [-0.006007946919961509, 0.0020703610469405293, -0.082778521289453816, 0.049411545511217646], [0.025083248335021995, 0.017953085308636513, -0.02036518637863605, 0.0071301807604233769]], [[-0.027308411157448493, 0.049705429124028748, 0.070466313563085761, 0.082279203645240634], [-0.044849880037611277, 0.070581993354444636, -0.027073086618310722, -0.10050842143176002], [0.065720608047837387, 0.035314388891081662, 0.021205602142554264, -0.078963549026759283]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(0.31150593716)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.0976110907104)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.030406434288916324)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([0.21753518916899894, 0.67794786956241904])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.030952868924)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.0067333381967076842, 0.020984431543877198])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[-0.67841415541111783, -0.79120161824747925, 0.99778157783813981, 0.37589907957802704, -0.54291662392065487], [0.1186812537163795, -0.88741396850060283, 0.42949930828715233, -0.48459973169141835, 0.45084695211038994], [-0.055487489729630468, 0.80663068796825454, 0.16697489753241723, -0.58970430504174653, -0.84847083385262478], [0.017364504202953324, -0.25913628441983616, 0.011459915479210325, -0.87705482432327342, -0.95333966566438555]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.985523645243)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.6685931914254617, 0.7797479029377028, -0.98333733774771714, -0.37045743114936192, 0.53505717026950339], [-0.11696318178461865, 0.87456744907659234, -0.42328172393265695, 0.47758449406048187, -0.44432033169069107], [0.054684233143749039, -0.79495361597163527, -0.16455770968028471, 0.58116753632044571, 0.83618806906111387], [-0.017113129479938243, 0.25538493563625764, -0.011294017677252192, 0.86435826754534906, 0.93953878246065348]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[-0.00096420892125248514, 0.40373827646162863], [-0.64259199021783608, -0.79227077852303807]], [[-0.8353946061109554, -0.40431254157555618], [-0.26610548626938391, -0.92724633290585579]], [[-0.33905417196105048, 0.14712833906266387], [-0.67685648202400062, -0.043853885816762528]], [[0.33001876601374724, -0.40895761857101887], [-0.38062653348994213, 0.29431808351701561]], [[0.68730166276521043, 0.63964290349700081], [0.28795765097750481, 0.44545935060097652]], [[0.26961800395213786, 0.15772111165456715], [-0.031087614964484311, -0.73402761759544544]]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.132163724346)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.00012743344208062684, 0.053359554278312432], [-0.084927350662270473, -0.10470945678032274]], [[-0.11040886244240827, -0.053435451294531228], [-0.035169492134336726, -0.1225483287432583]], [[-0.044810662121512759, 0.01944502924740231], [-0.089455873512205394, -0.0057958928765993586]], [[0.043616509220536739, -0.054049361970126754], [-0.050305020251040639, 0.03889817406006503]], [[0.090836347500433609, 0.084537588377824754], [0.038057555607190083, 0.058873566820295459]], [[0.035633719553121612, 0.020845009524301326], [-0.0041086549747490512, -0.097011823714432996]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[0.53115746995584812, -0.40532824329484862, 0.48851862059348239, 0.47661809041227632], [-0.87070744866115879, 0.8253731035585814, -0.082200471673910203, 0.076870483125222933], [0.73516434669144859, -0.082759271510864929, 0.2061834405864007, -0.49573617232584954]], [[0.33208421768398488, -0.78690472870595007, -0.43572726760813962, -0.76825542672134439], [0.32671779018671732, 0.57233895326777295, -0.47001380353020017, -0.91076771515899502], [-0.68978889873170623, 0.49337695377260782, -0.86821293507108166, 0.15593073379914335]]], [[[0.73387684934422315, 0.76908713068985657, 0.67676009495533229, 0.21451773522762707], [-0.71211994688902003, 0.46846097780399232, 0.13174425014625046, -0.63775164057668587], [0.99594041168765268, -0.87963468131938138, -0.30227966845566057, 0.1030070906679923]], [[0.46974676085532674, 0.42707515433171728, 0.059720931261722843, 0.17442389535816161], [0.3033761813886362, -0.70796228579719167, 0.38616676623909219, 0.60199130799665834], [-0.78915302036328061, 0.25338182844982704, 0.41607514813484525, 0.88750933112060393]]], [[[0.30778510374825352, 0.37932094464699784, -0.41132879707457937, -0.79175075664366878], [-0.68766300352371501, 0.44472209424333942, -0.095653083366781022, -0.92301933282022031], [-0.55928835763081941, -0.42302319028959645, 0.20246528908708639, 0.56128372761521517]], [[-0.23081299869419425, 0.45256463522818091, -0.88654997223223453, -0.062935698624705161], [0.65160296487900382, -0.91415512656785491, -0.02084975018800761, -0.93607783608669148], [0.095565757731735479, -0.25079927207559405, 0.85193259346335459, -0.26743385731274039]]]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.454250673521)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.24127863847320982, -0.18412062751381109, 0.22190991243219027, 0.2165040885821192], [-0.39551944499417391, 0.37492628819768992, -0.037339619621626179, 0.034918468733525214], [0.33394889963329327, -0.037593454823926349, 0.093658966755271147, -0.22518849016779147]], [[0.15084947954867922, -0.35745200301161983, -0.19792940478250795, -0.34898054502441422], [0.14841177624364324, 0.2599853550042498, -0.2135040868178095, -0.41371684803225095], [-0.31333707183625847, 0.22411681355099794, -0.39438631051577377, 0.07083164085090092]]], [[[0.33336405309615952, 0.34935834711228103, 0.3074187289456663, 0.09744482570937156], [-0.3234809655021505, 0.21279871468581868, 0.05984491436146714, -0.28969911227114892], [0.45240660279600231, -0.39957464644185109, -0.13731074298771997, 0.046791040313384977]], [[0.21338278250288936, 0.19399917649931223, 0.027128273248945185, 0.079232171944619567], [0.13780883472604877, -0.321592345150915, 0.17541651365557454, 0.27345495711133272], [-0.3584732910112341, 0.11509886623134288, 0.18900241627564679, 0.40315171141779926]]], [[[0.13981159067740717, 0.17230679458656037, -0.18684638310975338, -0.35965331446623061], [-0.31237138250619317, 0.20201531083975302, -0.043450477543730638, -0.41928215360658555], [-0.25405711314631207, -0.19215856910409554, 0.091969993932454114, 0.25496351130564765]], [[-0.10484696011426355, 0.20557779036423415, -0.40271592199660899, -0.028588583488793569], [0.29599108566463606, -0.41525558194621859, -0.0094710130656492312, -0.42521398751055767], [0.043410809815195538, -0.11392573825894133, 0.38699095437531011, -0.12148200980665927]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_constData_rank0(self):
      arg0=numpy.array(0.887539290361)
      arg1=Data(-0.160935502236,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(-0.14283658144885403,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_constData_rank0(self):
      arg0=numpy.array([-0.80443673361283907, 0.88962940157758075])
      arg1=Data(-0.952119275543,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.76591972002781683, -0.84703330133200139]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_constData_rank0(self):
      arg0=numpy.array([[-0.95383898291833091, -0.46383909201486806, -0.71519060195390449, 0.0048290787630054233, -0.044981813281804284], [-0.52949066543669643, 0.99055946009096418, 0.68357813283801949, -0.84851215078442399, -0.15535224483408228], [0.27022658310544156, -0.8170007898109839, 0.73088394412821422, -0.029016832652413393, -0.16030994451808289], [0.64656138098619609, 0.077804961279747342, 0.25404457715323114, 0.57257780429924776, -0.24032882851783999]])
      arg1=Data(0.148359356872,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[-0.14151093806492476, -0.068814869383281421, -0.10610521774656882, 0.00071643901956229322, -0.0066734728894116005], [-0.078554894593759389, 0.14695876444227876, 0.10141521215940949, -0.1258847169882043, -0.023047959132160099], [0.040090642079161652, -0.12120971174003259, 0.10843347189871702, -0.0043049186307659408, -0.023783480268841486], [0.095923430661193479, 0.011543094016891225, 0.037689890083197612, 0.084947274804849357, -0.035655030436637015]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[0.81268236173682173, -0.61774115797698559], [-0.72454518228764386, 0.24997841496722639]], [[-0.75470763598792434, 0.67603998942603849], [-0.35958699159539176, -0.52667774173871607]], [[-0.084304086883719265, 0.4423511992191278], [-0.55266737759737361, -0.80092220260002445]], [[0.51243243708803732, -0.60985056807272109], [0.59800145126702575, -0.47236649314802048]], [[-0.50307511490707713, -0.26267426698901741], [-0.21398082083038661, 0.15170633594316407]], [[-0.71895932140375174, -0.32112868090794633], [0.32074928270899106, -0.55325289435955582]]])
      arg1=Data(-0.635300701687,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.51629767466022392, 0.39245139112385014], [0.46030406271142948, -0.15881146243533611]], [[0.47946629071182573, -0.42948867965097798], [0.22844586807814618, 0.33459873888964281]], [[0.053558445552326576, -0.28102602725609183], [0.3511099727872431, 0.50882643730866328]], [[-0.32554868684931848, 0.38743849382094486], [-0.37991074159991289, 0.3000947645504653]], [[0.31960397350184117, 0.16687714613329696], [0.13594216562115033, -0.096379141675088126]], [[0.45675536137236566, 0.20401327631270727], [-0.20377224437069213, 0.35148195199710708]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[-0.18026562838092008, 0.27831868086485678, 0.98579853033629061, 0.88127902674675784], [-0.5364354806419025, -0.13897951092749539, 0.42877943629615789, -0.79530200142915541], [-0.68639139086178824, -0.68881420582826625, 0.78988873526541759, -0.55015997231733227]], [[0.67967080280078607, -0.84205267815405049, -0.76960487747871231, -0.66764363542422145], [0.13744122174468965, -0.027930655143734606, -0.72106750744828885, 0.030892117028429311], [-0.63383485403404816, 0.83772516805124941, -0.88145536554766912, -0.55175259156627643]]], [[[0.95237671798280288, -0.31426345768375819, -0.42409201981500755, -0.80473471628180415], [0.072934421982475017, -0.84154816359610995, 0.14914477114807712, -0.034838092348106864], [0.99012346249899785, 0.017641123115833768, -0.44204693967023934, -0.17622878218081217]], [[-0.91456939505022539, 0.7675074592143587, 0.70302295778984458, 0.7321972361192377], [0.73700357930770388, -0.54386864178108807, -0.8730357957009629, -0.58210683464430324], [-0.55715123434724068, -0.44056110733015719, 0.55416164304167537, 0.86092420650071788]]], [[[0.21182039332567482, -0.90059380223329022, 0.56588168450013643, 0.60138762580131289], [-0.98715507058350171, -0.90566299557741159, -0.090968976101634125, -0.91128877303181222], [-0.11950637352599403, 0.90900379164478329, 0.39364466601609749, -0.74427105029357277]], [[0.88229174877567651, 0.15812352807834085, 0.14579456797030232, 0.14467696011463982], [-0.30198142121146998, 0.8616472362664942, 0.31192655241301304, -0.40610229411455534], [-0.85333199406519222, 0.051210495043784832, -0.31613404527326283, 0.46299292449748264]]]])
      arg1=Data(0.338135219123,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.060954157752957767, 0.094109348140309079, 0.33333320206655742, 0.29799147681766613], [-0.18138772879229101, -0.046993867381099209, 0.1449854286475093, -0.26891961652234059], [-0.23209310335330599, -0.23291234242289091, 0.26708920058189306, -0.18602846279231652]], [[0.22982063583666248, -0.28472766684086931, -0.26023051388452234, -0.22575382696035681], [0.046473717631196391, -0.0094443381972803067, -0.24381831963362285, 0.010445712760586458], [-0.21432188725670331, 0.28326438326400094, -0.29805110317675221, -0.18656698345103803]]], [[[0.3220321102229175, -0.10626354312630162, -0.14340044804853397, -0.27210914962596733], [0.024661696758665703, -0.28455707270026925, 0.050431099873229621, -0.011779985989960235], [0.33479561395108726, 0.0059650850303512139, -0.14947163880812167, -0.059589157878517755]], [[-0.30924812279865005, 0.25952130290011216, 0.23771682188088594, 0.24758167287655555], [0.24920686678376869, -0.18390114236286731, -0.29520415008171336, -0.1968308220855445], [-0.18839245471074709, -0.1489692265642277, 0.18738156859955105, 0.29110879521355876]]], [[[0.071623935111932435, -0.30452248265911952, 0.19134452738624036, 0.20335033662828797], [-0.33379189610029752, -0.30623655546131173, -0.030759814667536179, -0.30813882895359362], [-0.040409313798827007, 0.30736619627159828, 0.13310512540001893, -0.25166425467804671]], [[0.29833391380282542, 0.053467133815298003, 0.049298278187605613, 0.048920375610437397], [-0.10211055403246573, 0.29135327704184211, 0.10547335315050814, -0.13731748820684589], [-0.28854160079804264, 0.017316071963036032, -0.10689605467076786, 0.15655421397743211]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(-0.219684990046)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.984785150581)+(1.-msk_arg1)*(0.959198032324)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.2163425160025613)+(1.-msk_ref)*numpy.array(-0.21072141018301316)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([0.3549348644413739, -0.33299431526460643])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.0403903428868)+(1.-msk_arg1)*(0.867826851273)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.01433594087726137, -0.013449754572887379])+(1.-msk_ref)*numpy.array([0.30802200581530792, -0.2889814081080247])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[-0.49543918245473906, -0.18156754427494026, 0.73371800638179052, -0.62302790811506226, -0.01418456574302196], [0.3452768699724631, -0.18619769930544483, 0.90490194291210213, -0.40087053280296869, -0.84660055652106725], [0.40670965695263761, -0.87788723802393731, -0.97163148178830383, -0.42472364241307847, -0.3871763715678489], [0.47028774699673925, -0.39379776090155527, 0.18107041954039649, 0.99960890315982254, 0.62771463148164619]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.779222341332)+(1.-msk_arg1)*(0.817056551009)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.38605727973990189, 0.14148148695978688, -0.57172946281013015, 0.4854772652764815, 0.011052930529052624], [-0.26904745102766325, 0.14508940720338592, -0.70511981063167628, 0.31236727514166129, 0.65969006782516337], [-0.31691725113289343, 0.6840693490383325, 0.7571169581507825, 0.33095415106009546, 0.30169647876145583], [-0.36645871931446328, 0.30685601326093603, -0.14109411626020171, -0.7789175899363241, -0.48912926483136554]])+(1.-msk_ref)*numpy.array([[-0.40480182965139777, -0.14835095150053357, 0.59948910370781616, -0.50904903378710897, -0.011589592363559949], [0.28211072852302999, -0.15213405000039656, 0.7393560604774887, -0.3275338949333037, -0.69172053079376206], [0.33230478957194803, -0.71728351887502739, -0.7938778673621274, -0.34702323440218974, -0.31634499078556977], [0.38425168454314829, -0.3217550403174585, 0.14794477247950574, 0.81673700277407779, 0.5128783518165454]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[0.54236255244194465, 0.75659586809114421], [0.3661458269609339, -0.51975633455419379]], [[-0.80861697268642407, -0.75737083498588476], [-0.27761315519041752, -0.65538128106110061]], [[0.45251432733535735, 0.95093655581500647], [-0.36525727052008605, -0.4908409940140579]], [[-0.27352299292749049, -0.23117084777931862], [-0.14429773179196048, -0.48471103153587536]], [[0.68495464141255469, -0.1919787494952232], [-0.77482644344771479, -0.9802309778068532]], [[-0.072370570945621493, -0.22381199096031423], [-0.58589816807084705, -0.20352533557704966]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.325076605672)+(1.-msk_arg1)*(0.972088798165)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.17630937759136275, -0.24595161666443635], [-0.11902544260938283, 0.16896062501333384]], [[0.26286246076957165, 0.24620354027208669], [0.090245542179160948, 0.213049122268229]], [[-0.14710182154807064, -0.30912722777364771], [0.11873659369764412, 0.15956092425870005]], [[0.088915926114081431, 0.075148234526391064], [0.046907816857081411, 0.1575682168633965]], [[-0.22266272986959096, 0.062407800247038729], [0.25187795022079568, 0.31865015903987687]], [[0.023525979553538398, 0.072756042330043777], [0.19046178774584188, 0.066161325257617018]]])+(1.-msk_ref)*numpy.array([[[0.52722456177304045, 0.73547836810939327], [0.35592625688361718, -0.50524931059547862]], [[-0.78604750115463928, -0.73623170474671928], [-0.26986463838387154, -0.63708880184658223]], [[0.43988410861191157, 0.92439477367345957], [-0.35506250112093157, -0.4771410319612836]], [[-0.26588863746540259, -0.22471859158860277], [-0.14027020867559534, -0.47118216410307007]], [[0.66583673416833034, -0.18662039187004836], [-0.75320010619762001, -0.9528715531404548]], [[-0.070350621333050561, -0.21756512930754782], [-0.56954504604711753, -0.19784469885724082]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[-0.822250636341469, 0.63638809184653833, 0.52959023053761745, -0.0036888432821744477], [-0.0095192693710410836, -0.61885383255801396, 0.73352016357184868, 0.6177409427823235], [-0.68277734563867454, 0.41285719119533026, -0.26534099849346537, 0.55786359386333229]], [[0.062308714666052589, 0.1809406597141674, -0.3972832420118888, 0.53326051657459539], [-0.8537018610811522, -0.36944105535194449, -0.74127572741761094, -0.071601359497779793], [0.62774540796583267, -0.31605339710016711, 0.16176277052083066, -0.62123826544507965]]], [[[0.92456868073754506, 0.61315530468207013, 0.33222920883688434, -0.54113394998908704], [-0.16602855481677925, -0.21715771822207275, 0.63670831412113249, -0.94084038285711369], [0.55803542236941817, 0.037332690808041802, -0.28711960314612517, 0.046507744312877186]], [[-0.049053830386043717, -0.079295986576197075, -0.61783181583030999, -0.88326371464697995], [0.45009667779065543, 0.95843215991445696, 0.72270648953330463, -0.50524740319664985], [0.031675253637922784, 0.056197570308277767, -0.32562149073832658, -0.02611660637834845]]], [[[-0.49213582326329286, -0.6446132567206968, -0.17581570927366008, 0.54391357448983135], [-0.96359375506215361, 0.52153541000680015, 0.67931642406343462, 0.74271670340750484], [-0.94227463976154291, 0.03116700024629071, 0.28594734622339146, -0.089203926776785236]], [[-0.83316454913290783, -0.6887417787132184, -0.79411226326946527, 0.39282591376873621], [-0.44894050979038158, 0.18717976723150787, -0.54009302765234679, 0.5825689649809529], [-0.25538648970381628, -0.4225818231655476, -0.49715341260075241, -0.71712430014819817]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0433066760346)+(1.-msk_arg1)*(-0.474220233298)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.03560894192731321, -0.027559852925898149, -0.022934792545000734, 0.00015975154096367178], [0.00041224791473810842, 0.026800502439382718, -0.031766320088679183, -0.026752306882404632], [0.029568817311362693, -0.017879472627665899, 0.011491036660463316, -0.024159217930957023]], [[-0.0026983833201773056, -0.0078359385317347419, 0.017205016655798717, -0.023093740433358587], [0.036970989928007195, 0.015999264098020646, 0.032102187779613538, 0.0031008168794098494], [-0.027185567015006656, 0.013687222077863069, -0.0070054078974107628, 0.026903764301949202]]], [[[-0.040039996328471571, -0.026553718138784876, -0.014387742716342342, 0.023434712663520253], [0.007190144835949046, 0.0094043789514640531, -0.02757372068820307, 0.040744669660695822], [-0.024166659252403602, -0.0016167547463250957, 0.012434195636642468, -0.0020140958160594523]], [[0.0021243583407863733, 0.0034340456015022032, 0.026756242292054064, 0.038251215543365844], [-0.019492191009345811, -0.041506511050591728, -0.031298015810347801, 0.021880585607578374], [-0.001371749947612444, -0.0024337299712742584, 0.014101584409319901, 0.0011310234115512418]]], [[[0.021312766663102251, 0.027916057476434795, 0.0076139939633141267, -0.023555088961271925], [0.041730042579474977, -0.022585965041755498, -0.029418936301922497, -0.032164591659981562], [0.040806782559806382, -0.001349739182637529, -0.01238342908586028, 0.0038631255579396201]], [[0.036081587212842318, 0.029827117082252233, 0.034390362520542221, -0.017011984585592475], [0.01944212121631634, -0.008106133539733466, 0.023389633777105828, -0.025229125434263269], [0.011059939973226046, 0.018300614113956164, 0.021530061779014453, 0.031056269743083013]]]])+(1.-msk_ref)*numpy.array([[[[0.38992788859509775, -0.30178810938339307, -0.25114240267777288, 0.0017493241218717072], [0.0045142301419594938, 0.29347300885288535, -0.34785010309766623, -0.29294525400382271], [0.32378683213920972, -0.19578523352731481, 0.12583007020903647, -0.2645502036302062]], [[-0.02954805320542013, -0.085805721862706297, 0.18839975171217543, -0.25288292657849815], [0.40484269572865111, 0.17519642345877637, 0.35152794839395946, 0.033954813405475195], [-0.29768957381716588, 0.14987891570739648, -0.076711178775283217, 0.29460375517287252]]], [[[-0.43844917547917728, -0.29077065163410137, -0.15754981292296344, 0.25661666800917293], [0.078734099999305959, 0.10298058379768445, -0.30193996526515826, 0.4461655458544711], [-0.26463168818444949, -0.017703917344623388, 0.13615792518832087, -0.022054913358206019]], [[0.023262318889819274, 0.037603761253741638, 0.2929883478418393, 0.4188615248233527], [-0.21344495154843937, -0.45450792247472693, -0.34272204007230178, 0.23959854141701173], [-0.015021046169942029, -0.026650024902359754, 0.15441629930469944, 0.012385023169686635]]], [[[0.23338076492211271, 0.30568864898892922, 0.083375366669169365, -0.25793482218839608], [0.45695565532985627, -0.24732264380647698, -0.32214559310237417, -0.3522112883640634], [0.44684569949829844, -0.014780022127987881, -0.13560201723693716, 0.042302306967164985]], [[0.39510348686524543, 0.32661528698330894, 0.37658410275227222, -0.18628599647282285], [0.21289667328961806, -0.088764432885149416, 0.25612304157579907, -0.27626599048531247], [0.12110944072844429, 0.20039685076896629, 0.23576020730831521, 0.34007485291978418]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_array_rank1(self):
      arg0=numpy.array(-0.0705991943231)
      arg1=numpy.array([0.31103878225265236, 0.89166364596776915])
      res=outer(arg0,arg1)
      ref=numpy.array([-0.021959087430276469, -0.062950735012525461])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_array_rank1(self):
      arg0=numpy.array([-0.55998676190775054, 0.049723485599342698])
      arg1=numpy.array([0.79135932201024461, -0.21409044108347763])
      res=outer(arg0,arg1)
      ref=numpy.array([[-0.44315074423802975, 0.11988781285773868], [0.039349143851881999, -0.010645322964171226]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_array_rank1(self):
      arg0=numpy.array([[0.39370173512983175, -0.95148127262943816, 0.19145825873523825, 0.22194728033627098, -0.55485368848311123], [0.21660110788510956, 0.89997912244714051, 0.90465802099620984, 0.82682412007740247, -0.37263197268419423], [-0.41711154749590573, -0.12043205783375033, -0.21821535243959223, 0.082355203560406265, -0.75205178168271147], [-0.45919525734405653, -0.76847783785798152, -0.84715014107714515, -0.47796513160077603, 0.35432747942008547]])
      arg1=numpy.array([-0.75239679614805688, 0.18991554154474533])
      res=outer(arg0,arg1)
      ref=numpy.array([[[-0.29621992414961629, 0.074770078234287884], [0.71589146112126512, -0.18070108116110323], [-0.14405258046847899, 0.036360898890916739], [-0.16699242263878492, 0.042151237939446307], [0.4174701375456249, -0.10537533872636949]], [[-0.16296997961487605, 0.041135916703192388], [-0.67714140832936831, 0.17092002241851337], [-0.68066179660718984, 0.17180861797029279], [-0.62209981892417388, 0.15702675052675744], [0.28036710238991797, -0.070768602889205501]], [[0.31383339197227755, -0.079215965427251692], [0.090612694467631247, -0.022871919482844782], [0.16418453204586825, -0.04144248683194262], [-0.06196379130497072, 0.015640533083202294], [0.56584135107551004, -0.14282632138796272]], [[0.3454970404320506, -0.087208315973275191], [0.57820026311513117, -0.14594588474193354], [0.63739305200281837, -0.16088697781237343], [0.35961943368690824, -0.090773006807466847], [-0.26659486030288887, 0.067292295138250136]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_array_rank1(self):
      arg0=numpy.array([[[-0.11650349333382004, -0.93704280994921563], [0.16842803438649012, -0.29159530904824105]], [[0.62977412573709302, -0.38013561791327355], [0.37907571072272361, -0.8115279466802332]], [[-0.88418803236921328, -0.51276945192377066], [-0.16107750386484887, 0.67441583586588005]], [[0.53090414717970158, -0.48588821848507924], [-0.017722197481937263, -0.62956203569481728]], [[-0.26077893723743117, -0.052624881493034126], [0.40460561788751304, -0.045268579762606898]], [[-0.42518607569172384, 0.93698214557284532], [0.36176722067350453, -0.50067180425631896]]])
      arg1=numpy.array([0.81982036515978107, 0.2126887982468002])
      res=outer(arg0,arg1)
      ref=numpy.array([[[[-0.095511936447322462, -0.024778987988724282], [-0.76820677862291331, -0.19929850915390346]], [[0.13808073265387649, 0.035822756224733326], [-0.23905577274280818, -0.062019055855874694]]], [[[0.5163016537299655, 0.13394590196995157], [-0.31164292108789893, -0.080850587744778962]], [[0.31077398758790681, 0.080625157358167762], [-0.66530713758475613, -0.17260290372311216]]], [[[-0.7248753555668368, -0.18805689002881085], [-0.42037883931892644, -0.10906031850733718]], [[-0.13205461803750645, -0.034259380721609022], [0.55289983682910471, 0.14344069364892528]]], [[[0.43524603180570509, 0.11291736504789307], [-0.39834105670527314, -0.10334298127187019]], [[-0.01452901841107556, -0.0037693128847257049], [-0.51612777799406018, -0.13390079279373981]]], [[[-0.21379188355197046, -0.055464758769106969], [-0.043142949562109442, -0.011192722802633705]], [[0.33170392540223981, 0.086055082632399194], [-0.03711210359124506, -0.0096281198280482819]]], [[[-0.34857620383444338, -0.090432315470145769], [0.76815704473172519, 0.19928560652059688]], [[0.2965841349553916, 0.076943835410132652], [-0.41046094139062184, -0.10648728436333367]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(0.654270043708)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([0.89432534309656209, 0.97754379508378975])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.58513028131735656, 0.63957762153638897])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([-0.36864048842372288, 0.33590128236215699])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.69595748477141672, -0.7498044849723482])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.25655810710828075, 0.2764082915625044], [-0.23377301160426023, -0.25186028802310845]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[0.69291416229716574, -0.93879232170841798, 0.57330438409422735, 0.89884450984499531, -0.39661039016943334], [-0.010318982832071777, -0.79751025910321327, -0.73558448117500341, 0.83233963091997976, -0.49273643825067115], [-0.93171664721640468, 0.15950365328559402, 0.76570967255785183, 0.65150761591250417, 0.75969286064158115], [-0.84120636770796553, 0.18737586465879708, -0.21932984875625117, 0.97862628646486183, -0.87747381940340796]])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.070686201510304691, 0.47540317309665325])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.048979470105481426, 0.32941359143968196], [0.066359663228608021, -0.44630484861895603], [-0.040524709220825676, 0.27255072334861813], [-0.063535904149334382, 0.42731353210081674], [0.028034881960597131, -0.1885498379696503]], [[0.00072940969984920017, -0.0049056771814968122], [0.056372970881505037, -0.37913890775480169], [0.05199567286418922, -0.34969919643125202], [-0.05883492687622232, 0.39569690163345561], [0.034829667165656747, -0.23424846624471221]], [[0.065859510675644245, -0.44294105051365384], [-0.011274707377775271, 0.075828542892479811], [-0.054125108212813737, 0.36402080800480213], [-0.046052598623889461, 0.30972878790144009], [-0.053699802633250623, 0.36116039652788129]], [[0.059461682819556716, -0.3999121764374769], [-0.013244888127439309, 0.089079080620521181], [0.01550359388640902, -0.10427010605353085], [-0.069175374888336388, 0.46524204186118967], [0.062025291218366002, -0.41715383805361983]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_Symbol_rank1(self):
      arg0=numpy.array([[[0.23559367313114099, -0.45352491249309046], [0.11081796130794208, 0.71016108875912942]], [[0.41496165495690951, -0.63594701659516328], [-0.30152010461577827, 0.74042628234976204]], [[-0.60520065904702447, -0.21278105423709337], [0.42339402644574631, 0.86474709042163367]], [[-0.81099508845128621, 0.053693738504768218], [-0.65224833082674705, -0.14165284262001743]], [[-0.0047545178395964616, -0.62316913459286183], [-0.16360096345568675, -0.6455317755933454]], [[0.63623467657702726, 0.75913128598714219], [0.85965956229310625, 0.85578947264836502]]])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.041189853763310413, -0.66502644884690421])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.0097040689438328519, -0.156676023813201], [0.018680624823608549, 0.3016060620188829]], [[-0.0045645756206223256, -0.073696875277074358], [-0.029251431394381848, -0.47227590696673499]]], [[[-0.017092209885056376, -0.27596047580362787], [0.026194564614768316, 0.42292158610106467]], [[0.012419569015821964, 0.20051884442857806], [-0.030498050292498285, -0.49240306118397748]]], [[[0.024928126643606024, 0.40247444512584873], [0.0087644205076288974, 0.14150502888119473]], [[-0.017439538033559472, -0.28156822587020691], [-0.035618806196715258, -0.57507968669379184]]], [[[0.033404769096071471, 0.5393331837050398], [-0.0022116372370168326, -0.035707756243140293]], [[0.026866013364117022, 0.43376239121603233], [0.0058346598726757423, 0.094202886896659593]]], [[[0.0001958378945280288, 0.0031618801148460898], [0.025668245523688682, 0.41442395660928938]], [[0.0067386997602764285, 0.10879896775486751], [0.026589359436260011, 0.42929570434067915]]], [[[-0.02620641328735485, -0.42311288759727905], [-0.031268506656964165, -0.50484238332861286]], [[-0.035409251657084487, -0.57169634592906848], [-0.035249843230566694, -0.56912263395590701]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_constData_rank1(self):
      arg0=numpy.array(0.45620442526)
      arg1=Data(numpy.array([0.31772359770243996, -0.5126401738892179]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.14494691128130036, -0.23386871589418687]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_constData_rank1(self):
      arg0=numpy.array([0.8539936228705296, 0.89072822837147902])
      arg1=Data(numpy.array([0.90381714995212459, 0.33301162500766246]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.77185408230013164, 0.2843898040982959], [0.80505544874861534, 0.29662285477018252]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_constData_rank1(self):
      arg0=numpy.array([[0.22004350247352922, -0.35970181275916224, 0.15646348648285158, 0.4212211618405588, 0.37626278163592164], [0.33038291515486939, -0.90764100376178236, -0.65433267556937502, 0.43104735731894261, -0.033957692220172087], [-0.21077756673600101, 0.57526900030575967, -0.70337369772725622, 0.37091995859980731, 0.46236022366388907], [0.44196297866078726, -0.73366403186605034, 0.48888423737293074, 0.51899052032344106, 0.62487345661284288]])
      arg1=Data(numpy.array([-0.28684372213436249, -0.13123413337250533]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.063118097280988925, -0.02887721835136434], [0.10317820683031562, 0.047205155669967837], [-0.044880568840860662, -0.020533350053017728], [-0.12082464590410658, -0.055278594132305549], [-0.10792861678507662, -0.049378520068318395]], [[-0.094768265112624012, -0.043357515551431247], [0.26035112388079856, 0.11911348054202836], [0.18769122017445578, 0.085870781615659628], [-0.12364322838954604, -0.056568126380260077], [0.0097405508315272452, 0.0044564083098445504]], [[0.060460221784978521, 0.0276612113049645], [-0.16501230127621783, -0.07549492871119387], [0.20175832950749614, 0.092306637658250992], [-0.10639606153869237, -0.048677359317411267], [-0.13262512752262628, -0.060677443258448219]], [[-0.12677430584465005, -0.058000628487279486], [0.2104469216965614, 0.096281763408519253], [-0.14023337434087066, -0.064158299211114753], [-0.14886917260202534, -0.068109271163192403], [-0.17924102815779291, -0.082004726546068246]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_constData_rank1(self):
      arg0=numpy.array([[[-0.66447070728135871, 0.10916086666511138], [0.26746102869120603, -0.70631701651855239]], [[-0.87747366027630269, 0.72562057325760065], [0.3023068504577977, -0.0090824437971253058]], [[-0.37222489903654399, 0.080330054398139561], [0.27150597071903082, -0.27524188286388296]], [[-0.54612661303996024, 0.17675616726836862], [-0.94091695872832548, -0.31531671754763124]], [[-0.43884398137503489, 0.69552266955434705], [-0.44929811099008043, 0.35789835388832625]], [[-0.24615638375578608, 0.35713986793045427], [-0.45039744128586578, -0.65163958989322479]]])
      arg1=Data(numpy.array([-0.95289915314530238, 0.048853322366110286]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.63317357425826681, -0.032461601665653525], [-0.1040192974017919, 0.005332871008954668]], [[-0.25486338773922163, 0.01306635985502296], [0.67304888689264508, -0.034505932900650085]]], [[[0.83614390778459757, -0.042867503593248958], [-0.69144322976197647, 0.035448975780835307]], [[-0.288067941791259, 0.014768694018898286], [0.0086546530027705078, -0.00044370755469304134]]], [[[0.35469279107151847, -0.018184422985325136], [-0.076546440808103258, 0.0039243900431994872]], [[-0.25871780957205776, 0.013263968711860512], [0.26227775709111256, -0.013446480432204441]]], [[[0.52040358707589041, -0.026680099479553146], [-0.16843080210323788, 0.0086351260197597233]], [[0.89659897315227477, -0.045966919504494969], [0.30046503312369433, -0.015404269249778172]]], [[[0.41817405821518361, -0.021438986490541878], [-0.66276296281169733, 0.033978593188676118]], [[0.42813578947223174, -0.021949705454682798], [-0.34104103833228383, 0.017484523656806621]]], [[[0.23456220962219862, -0.01202555716809737], [-0.340318277705355, 0.017447469097796535]], [[0.42918334038011258, -0.02200341139200963], [0.62094681336520607, -0.031834758951573609]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(0.0952653164761)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.03600206368683323, -0.21567891629700786])+(1.-msk_arg1)*numpy.array([0.98400030929841975, -0.057944651857548823])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0034297479909205435, -0.020546720218266729])+(1.-msk_ref)*numpy.array([0.09374110087793984, -0.0055201155973095091])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([-0.6816375542209947, -0.73859164214492656])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.65424365167993637, -0.84029348680087512])+(1.-msk_arg1)*numpy.array([-0.4070349497467638, 0.0094425568140983351])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.44595704259572422, 0.57277559717078019], [-0.48321889305717752, 0.6206337462999445]])+(1.-msk_ref)*numpy.array([[0.27745030762784956, -0.006436401332354777], [0.30063261194383994, -0.006974193543371655]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[0.11413458360117934, -0.39371474720074584, -0.52902000724565745, 0.4389579328234896, 0.59175704124133555], [-0.35473395573288258, -0.37884322851013308, 0.91011404456304401, 0.40604179011912978, -0.77022685281477199], [-0.55045292580057259, -0.69989776505223489, 0.78949225848162996, 0.0304730903118684, -0.47456543934448558], [-0.16824189615650686, 0.97460850007093991, -0.71636079682124976, -0.032193665122135284, 0.7901741074913109]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.57288820519795935, 0.13608562597774565])+(1.-msk_arg1)*numpy.array([-0.44236731972391885, 0.68478136995014638])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.065386356750296082, 0.015532076255075834], [0.22555453488380359, -0.053578917829483381], [0.30306932246477614, -0.071992018840776828], [-0.25147382229265536, 0.059735865066181809], [-0.33901062927000353, 0.080529627384065799]], [[0.20322289922258346, -0.04827419242147124], [0.21703481723257054, -0.051555117899231596], [-0.52139360151517788, 0.12385343946549975], [-0.23261655237671477, 0.055256451181486192], [0.44125387930432752, -0.10481680341016721]], [[0.31534798870785552, -0.074908730978852506], [0.40096317444283791, -0.095246025477558532], [-0.45229080297922442, 0.10743854820005679], [-0.01745767401560161, 0.0041469495705669874], [0.27187294279504337, -0.064581534880598201]], [[0.096383797928202675, -0.022895303754141116], [-0.55834171437631597, 0.13263020781538562], [0.41039465116510582, -0.097486407461336438], [0.018443371030564225, -0.0043810950706636978], [-0.45268142623459651, 0.10753133804936152]]])+(1.-msk_ref)*numpy.array([[[-0.050489409835459249, 0.0781572365171051], [0.17416653745497421, -0.2696085239577023], [0.23402116268558962, -0.36226304529271769], [-0.19418064421467912, 0.30059021458935353], [-0.26177397626168614, 0.40522419737888704]], [[0.15692270921261856, -0.24291520417459792], [0.1675878635915837, -0.25942478501550525], [-0.40260471053644903, 0.62322914224674986], [-0.17961961839090143, 0.27804985329478749], [0.34072318845906002, -0.52743699944288935]], [[0.24350238542058847, -0.37693990862278237], [0.30961189840691822, -0.47927695037751511], [-0.34924557432730197, 0.54062959032808566], [-0.013480299284966142, 0.020867404530375777], [0.20993224143642408, -0.32497357168530994]], [[0.074424716668023833, -0.11520891613306304], [-0.43113494995653046, 0.66739374384363559], [0.31689460564510707, -0.49055052782583386], [0.014241425352168394, -0.022045622106052046], [-0.34954720204617096, 0.54109650782703411]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank3_expandedData_rank1(self):
      arg0=numpy.array([[[0.87420183956214492, -0.94796363776977888], [-0.89619505429056767, 0.98269351205225663]], [[-0.78543686020072778, 0.49801945489413346], [-0.32439744666765336, -0.90769684324544953]], [[0.047647877322015475, -0.020522477822865426], [-0.13937854856216125, -0.72983275447960549]], [[-0.29260969381059909, 0.4335589962680908], [0.1678945182964815, -0.50664460296582137]], [[0.14014475688327788, 0.85900221376298114], [0.33192132101855942, -0.56956032203941187]], [[0.60043591489976333, -0.696389117775569], [-0.20269845166148004, -0.77081614130858345]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.72311213346341008, -0.73911599083212276])+(1.-msk_arg1)*numpy.array([-0.85972430731293925, -0.31600218264079616])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.63214595728342038, -0.64613655883523913], [-0.68548400855344005, 0.70065508340303362]], [[-0.64804951770740904, 0.66239209553082101], [0.71059760204075861, -0.72632448884480227]]], [[[-0.56795892368055045, 0.58052894316333237], [0.36012391053478138, -0.36809414285775111]], [[-0.2345757297499296, 0.2397673402171733], [-0.65636660085721954, 0.67089325167055047]]], [[[0.034454758225325448, -0.035217308057908903], [-0.014840052722447738, 0.015168491530377444]], [[-0.10078631960981793, 0.10301691402126495], [-0.52775092016322467, 0.539431059468931]]], [[[-0.21158961996345749, 0.21627250376790499], [0.31351177077367381, -0.32045038711087054]], [[0.12140656332218028, -0.12409352324598588], [-0.36636085975833743, 0.37446912772083052]]], [[[0.10134037414357799, -0.10358323084371088], [0.62115492344394152, -0.6349022723524127]], [[0.24001633458372393, -0.24532835606293965], [-0.41185597960602605, 0.42097114176282285]]], [[[0.43418249543122239, -0.44379178617233073], [-0.50356742067539362, 0.51471233278939754]], [[-0.14657370983066273, 0.14981766693991197], [-0.55738650444968318, 0.56972253603268719]]]])+(1.-msk_ref)*numpy.array([[[[-0.75157257096926233, -0.27624968937023692], [0.81498738183947717, 0.29955857859935919]], [[0.77048067226724026, 0.28319959322770621], [-0.8448454989500459, -0.31053329467546265]]], [[[0.67525916057412061, 0.24819976214996387], [-0.42815943088722647, -0.15737523474412571]], [[0.27889237013043444, 0.10251030119007973], [0.7803690398093357, 0.28683418364172264]]], [[[-0.040964038325601658, -0.015056833231957783], [0.017643673030608136, 0.0064851477852228088]], [[0.11982712611688695, 0.044043925558949155], [0.62745495929927331, 0.23062874337829964]]], [[[0.25156366632436855, 0.092465301906004382], [-0.3727412077458776, -0.13700558912426949]], [[-0.14434299844408216, -0.053055034235113242], [0.43557468033862989, 0.16010080036037913]]], [[[-0.12048585403501635, -0.044286049060779549], [-0.73850508320766028, -0.2714465744423778]], [[-0.28536082779507677, -0.10488786190688115], [0.48966485333826798, 0.17998230491004891]]], [[[-0.51620935102301002, -0.18973905964424856], [0.59870265189986993, 0.22006048118437826]], [[0.17426478594807124, 0.064053153142937608], [0.66268937315215459, 0.24357958306826874]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_array_rank2(self):
      arg0=numpy.array(0.218264810985)
      arg1=numpy.array([[0.62659455796785046, -0.37652765022503543, 0.94590766670555371, 0.094449960134427524, -0.22520111876829429], [0.12465652166384422, 0.040617779034448631, 0.23922827763552656, -0.30156812276149814, -0.20731745558322934], [0.12441485550270071, 0.0098506262715623016, 0.7491573134680285, 0.4197162216745971, 0.15950621707742796], [-0.13087565182032113, -0.03397043006609568, 0.67705732058749746, -0.2032399434958112, -0.89027035091146112]])
      res=outer(arg0,arg1)
      ref=numpy.array([[0.1367635427591869, -0.08218273640705627, 0.20645835808290763, 0.020615102696297342, -0.049153479621609805], [0.027208132139027282, 0.0088654318635911934, 0.052215114800425164, -0.0658217093136899, -0.04525010525679922], [0.027155384920043782, 0.0021500450812480623, 0.16351467942225445, 0.09160928179121422, 0.034814594321363766], [-0.028565549407122866, -0.0074145494974611941, 0.14777778810415348, -0.04436012785174917, -0.19431468986738801]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_array_rank2(self):
      arg0=numpy.array([0.16636827760669792, 0.40745449772685505])
      arg1=numpy.array([[0.7102545042272097, -0.16509784417725548, 0.20930882056968447, -0.72285238509450433, -0.53798767304345074], [-0.81804276796566344, 0.55893736573743813, -0.90691450200816082, 0.37895267866036297, -0.17452628062456754], [-0.82868114455863062, 0.10780418724989116, 0.61897790911348216, -0.076685948682186478, -0.70998524155215792], [0.73039426394358054, -0.6291725941235613, 0.85291511223351368, -0.003280370193414317, 0.98782742462344664]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[0.11816381853068003, -0.027467043972348996, 0.034822347966067792, -0.12025970627206621, -0.089504082537874244], [-0.13609636631506308, 0.092989446827762556, -0.15088180363563389, 0.063045704443169065, -0.029035636704612518], [-0.13786625470536643, 0.017935196951554336, 0.10297828861580524, -0.012758109198890991, -0.11811902176320789], [0.1215144356661054, -0.10467436080167493, 0.14189801816701311, -0.0005457495389906904, 0.16434314720726304]], [[0.28939639227813418, -0.067269859175030214, 0.085283820355021217, -0.29452945549934045, -0.21920549710315873], [-0.33331520514053564, 0.22774154361731933, -0.36952639289693606, 0.1544059733458045, -0.071111518012019329], [-0.33764985953185223, 0.043925300968756235, 0.25220533306185283, -0.031246034703007672, -0.28928667999011437], [0.29760242795770758, -0.25635920332211809, 0.3475240986587505, -0.0013366015895157769, 0.40249472714075923]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_array_rank2(self):
      arg0=numpy.array([[-0.81956487123327637, 0.26071542752343602, -0.47037712698936929, 0.094964802681870131, -0.46512807889738683], [0.076723197191971071, 0.081140132594234249, 0.093588154099443432, -0.74062097527274462, 0.030840140350320233], [-0.8940842367217956, -0.96021909382187043, 0.28300310355728842, -0.76699825705987146, -0.71884907337947257], [0.607311761899342, -0.50060925046611637, -0.49716368786464571, 0.7905725723857413, 0.65601088235599914]])
      arg1=numpy.array([[0.90239786983064629, -0.98569087469079975, -0.39644746029235201, 0.35389309790413259, -0.32994298913565023], [0.49986542917608778, 0.17646704298247662, -0.044357612957905967, 0.21383297018993774, -0.48466538221114286], [-0.90245331337920187, -0.60180396016839999, -0.82878400962931353, 0.56657680356113072, 0.41317536821994039], [-0.8755948255170094, -0.31647123327854643, 0.24493355541838535, 0.22973481239643379, 0.69502961990367562]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[[-0.73957359398893652, 0.80783761479178084, 0.32491441174526092, -0.29003835121414567, 0.27040968340528149], [-0.4096721460966668, -0.14462618935885049, 0.036353941352061714, -0.17524999067914535, 0.39721472156310195], [0.7396190335736692, 0.49321738512309049, 0.6792422601320468, -0.46434644505433942, -0.33862401745193699], [0.71760676042737093, 0.25936870555096808, -0.20073893780717755, -0.18828258193948416, -0.56962186093966893]], [[0.2352690464291349, -0.25698481780096144, -0.10335996910070097, 0.09226539031766913, -0.086021227470861453], [0.13032262907182957, 0.046007680554972952, -0.011564714026239559, 0.055749554241675769, -0.12635974232898764], [-0.23528350141759999, -0.15689957676060126, -0.21607677739509398, 0.14771531356530201, 0.10772119276761485], [-0.22828107927197547, -0.082508932883085279, 0.06385795661573955, 0.059895409830952602, 0.18120494449463803]], [[-0.42446731741226623, 0.46364644173669678, 0.18647981737454861, -0.16646321865351346, 0.15519763529991185], [-0.23512526445715623, -0.083006060686406896, 0.020864806543246229, -0.10058213817354636, 0.22797551003568195], [0.42449339678934589, 0.28307481779483684, 0.38984104134416625, -0.26650476907790493, -0.19434824264607031], [0.41185977843344895, 0.14886082948434515, -0.11521114210099156, -0.10806200102447626, -0.32692603578280427]], [[0.085696035649007254, -0.093605939420331769, -0.03764855484039175, 0.033607388212941701, -0.031332970859533446], [0.047469621849195502, 0.016758157916683988, -0.0042124119619863058, 0.020306605820965655, -0.04602615238841435], [-0.085701300834655811, -0.057150194330560136, -0.078705309940336921, 0.053804854354307471, 0.039237117316015674], [-0.083150689834489297, -0.030053628222785254, 0.023260066760475869, 0.021816721128383787, 0.066003350712207753]], [[-0.41973058759542264, 0.45847250293161657, 0.18439884558952974, -0.16460561676319402, 0.15346574868232638], [-0.23250144677989149, -0.082079776691141942, 0.020631971299584634, -0.099459718629367935, 0.22543147813593661], [0.41975637594664955, 0.27991591986596737, 0.38549071421975595, -0.26353078018821086, -0.19217946526786128], [0.40726373908521918, 0.14719965676113705, -0.11392547408926021, -0.10685611194580481, -0.3232777918825776]]], [[[0.069234849712631316, -0.075625355349228679, -0.030416716672266243, 0.027151809935376289, -0.025314281017562862], [0.038351273892126232, 0.013539115736628589, -0.0034032578859345508, 0.016405949138027464, -0.037185077691507543], [-0.069239103518940165, -0.046172323906909257, -0.063586959000342266, 0.043469583824017291, 0.031700135250803743], [-0.067178434458411013, -0.024280684836416194, 0.018792085471295353, 0.017625989313352068, 0.053324894582130404]], [[0.073220682810813176, -0.079979088269338222, -0.032167799494768862, 0.028714932888125641, -0.026771617887004651], [0.040559147202621569, 0.014318559266110586, -0.0035991825969682134, 0.017350435554230489, -0.039325813376447354], [-0.07322518150769447, -0.048830453123799245, -0.067247644433303611, 0.045972116965767557, 0.033525104162037522], [-0.071045880241275547, -0.025678517830482094, 0.019873941163425009, 0.018640713139358167, 0.056394795515904469]], [[0.08445375090072002, -0.092248989474977755, -0.037102786006173623, 0.033120201781381385, -0.030878755311258225], [0.046781482814716134, 0.016515224812117129, -0.0041513471169879728, 0.020012232965677588, -0.045358938477042088], [-0.08445893976008606, -0.056321721761895538, -0.077564365608342803, 0.053024877200849189, 0.038668320031062062], [-0.081945303459161165, -0.029617958548113513, 0.022922879328620416, 0.021500457024564171, 0.065046539171222786]], [[-0.66833479043802047, 0.73002333693094479, 0.29361730468612446, -0.26210065131205157, 0.24436269839804986], [-0.37021082166152319, -0.13069519347717917, 0.032852178569655249, -0.15836918292753943, 0.35895334805415413], [0.66837585309302427, 0.44570863590292037, 0.61381482150211797, -0.41961866482035887, -0.30600634416972761], [0.64848389361817615, 0.23438523343652531, -0.1814029286909854, -0.17014642081114781, -0.51475351493650523]], [[0.027830076957407138, -0.030398844917494179, -0.012226495316944143, 0.010914112808373068, -0.010175488092547638], [0.015415919992063605, 0.0054422683727855725, -0.0013679950092270034, 0.0065946388121835226, -0.014947148410333245], [-0.027831786844226113, -0.018559718594971984, -0.025559815177069185, 0.017473308141061086, 0.012742386345198203], [-0.027003467308958726, -0.0097600172511493073, 0.0075537852256059431, 0.0070850538576605069, 0.021434811025459081]]], [[[-0.80681971066690761, 0.88129067334156275, 0.35445742493578192, -0.31641024032072806, 0.29499682560305551], [-0.44692180070851523, -0.1577764014315399, 0.039659442524270186, -0.19118468793822496, 0.43333167831972702], [0.80686928186969908, 0.53806343438331783, 0.74100271865665412, -0.50656738895622833, -0.36941358372717226], [0.78285553124992913, 0.28295194105025451, -0.21899123094380268, -0.20540227438989039, -0.62141502721061759]], [[-0.86649966483556939, 0.94647919848408657, 0.38067642106990418, -0.33981490977932066, 0.31681755804071332], [-0.47998032943634339, -0.16944702410205875, 0.042593026918541728, -0.20532650086502105, 0.46538495411361408], [0.86655290278952168, 0.57786365329131406, 0.79581423070031576, -0.54403786489596084, -0.39673887766166882], [0.84076286991306148, 0.30388172083941561, -0.23518987663041085, -0.22059575337864104, -0.66738071180326641]], [[0.25538139780555885, -0.27895357668559462, -0.11219586166014048, 0.10015284503437284, -0.093374889922357707], [0.14146346781782879, 0.049940720839618299, -0.012553342133480382, 0.060515394206625515, -0.13716180735253283], [-0.25539708850187232, -0.17031238846072397, -0.23454844690374935, 0.16034299381136813, 0.11692991151966856], [-0.24779605308001609, -0.089562341204431253, 0.069316956348724146, 0.06501566490334218, 0.19669553949698271]], [[-0.69213759333464642, 0.75602318288766357, 0.30407451106004657, -0.27143538927798816, 0.25306569759616782], [-0.38339591294254394, -0.135349914396069, 0.034022211826050246, -0.16400951543761769, 0.371737503413203], [0.6921801184397538, 0.46158258854089107, 0.63567589086477516, -0.43456342082194044, -0.3169047872847649], [0.67157970506218845, 0.2427328843342331, -0.187863610101379, -0.17620620069404128, -0.53308650707110417]], [[-0.64868787254737004, 0.70856297191008322, 0.28498588947480247, -0.2543957255037767, 0.23717921200821557], [-0.35932780057766306, -0.12685317032996887, 0.031886428972115993, -0.15371363247901712, 0.34840126090158796], [0.64872772809087409, 0.43260621912315134, 0.59577061735375592, -0.40728321023822228, -0.29701073058812655], [0.62942052897876322, 0.22749505279354201, -0.17607025935204601, -0.1651446570141834, -0.49962139823904422]]], [[[0.54803684026106292, -0.59862166179657317, -0.24076720561066772, 0.2149234408121751, -0.2003782580583072], [0.30357415450550063, 0.10717051079085479, -0.026938900079114956, 0.12986327787822058, -0.29434298720226715], [-0.54807051178022215, -0.36548262336787241, -0.50333027712197964, 0.34408875682200768, 0.25092626084706138], [-0.5317590361946819, -0.19219670227285171, 0.14875102908940974, 0.13952065368609301, 0.42209966303593122]], [[-0.45174872123813992, 0.49344596997025192, 0.19846526594614977, -0.17716215848691977, 0.16517251248774784], [-0.25023725783376488, -0.088341034119429562, 0.022205831375323396, -0.10704676293172814, 0.24262797371559403], [0.45177647679142546, 0.30126862942744326, 0.41489694185883325, -0.28363358896222568, -0.20683941139564599], [0.43833086931408, 0.15842842688566058, -0.12261600359199887, -0.11500737223975258, -0.34793825707172882]], [[-0.4486394528862046, 0.49004971035580636, 0.19709928140351843, -0.17594279766386267, 0.16403567326376461], [-0.24851494020522769, -0.087733005875737027, 0.022052994443025126, -0.10630998804668029, 0.24095802880041983], [0.44866701740527282, 0.29919507620887009, 0.41204131467055755, -0.28168141311701467, -0.2054157897990585], [0.43531395252923738, 0.15733800543983473, -0.12177206969360403, -0.11421580656190354, -0.34554348900647436]], [[0.7134110052674274, -0.7792601703814569, -0.31342048849911874, 0.27977817675962907, -0.26084387766161171], [0.3951798981904423, 0.13951000411196171, -0.035067912181022814, 0.16905048130394262, -0.38316315796098172], [-0.71345483741623117, -0.47576970486225817, -0.65521390644481536, 0.44792008104541392, 0.32664511370006416], [-0.69222125357662645, -0.25019347697910849, 0.19363775097069844, 0.18162204160280435, 0.54947135449153284]], [[0.59198282282377634, -0.64662394043616811, -0.26007384823418078, 0.232157723415788, -0.21644619143005375], [0.32791716125306553, 0.11576430057368851, -0.029099076815721794, 0.14027675545110513, -0.31794576503173938], [-0.59201919439498518, -0.39478994691540664, -0.54369132943946885, 0.37168054882657897, 0.27104753787372793], [-0.57439973407376044, -0.20760857298335048, 0.16067907780860699, 0.15070853698807446, 0.45594699421656493]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(0.189833447168)
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[0.56112660968063799, -0.19337957476011125, 0.20089541710065451, 0.57427272106112981, -0.64845978807920446], [-0.32274979748125743, 0.26300963069344996, -0.80488460148091479, 0.29749052217391814, -0.11222549505076929], [0.66275599140909636, 0.28640922184854589, -0.29241701394419839, 0.5262033754300528, -0.031616965807117259], [0.83093281754561077, -0.11718098125542253, 0.86279986001008391, -0.54984669044667855, 0.059551925744476542]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.10652059861358146, -0.036709911288667331, 0.03813666954854672, 0.1090161702537997, -0.12309935692115243], [-0.061268706628763561, 0.049928024833020113, -0.15279401847186966, 0.056473651324196204, -0.021304152585665481], [0.12581325448068587, 0.054370049884322709, -0.055510529767711363, 0.099891000669524055, -0.0060019576081698648], [0.15773884112001768, -0.022244869614290157, 0.16378827164210974, -0.10437929266161804, 0.011304947349589364]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_Symbol_rank2(self):
      arg0=numpy.array([-0.55368373959577077, 0.99799075918041913])
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.23708906355627457, -0.70899243052866479, 0.87649982659607106, -0.29624662438542204, 0.93527459339650676], [-0.4197397079227887, 0.40079068158167797, -0.17929791312967125, 0.40132497300694303, -0.010919894400852614], [0.071935063306008296, 0.41235686379712022, 0.13492620627142649, -0.44801882381293168, 0.36795944155496696], [-0.10040933774504479, 0.079371292400747029, 0.70607528653311413, -0.19638599343111141, 0.17808760913072996]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.13127235932709747, 0.39255758028020582, -0.48530370174475723, 0.16402693883234412, -0.51784633442069183], [0.23240305113952622, -0.22191128337328125, 0.099274339043354032, -0.22220711184765599, 0.0060461679678549939], [-0.039829274859329182, -0.22831529039517343, -0.074706446457833753, 0.24806073777804277, -0.20373315961972557], [0.05559501761301118, -0.043946593992994998, -0.39094240508381001, 0.10873573124716823, -0.098604213399172497]], [[-0.23661269453190109, -0.70756789399647269, 0.87473872736611868, -0.29565139357504383, 0.93339540150593758], [-0.41889634976803125, 0.3999853965841364, -0.17893766044374546, 0.40051861448926029, -0.010897953703276909], [0.071790528440454734, 0.4115283395541447, 0.13465510703015474, -0.44711864610418611, 0.36722012242504454], [-0.10020759120498036, 0.079211816360152565, 0.70465661124571455, -0.19599140667671566, 0.17772978823700294]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[0.88354224569112616, 0.58252098883021342, 0.078414963095690737, -0.30580572563920594, -0.43413381208064394], [-0.99832879256670903, 0.050917466920893384, 0.5415130300087958, 0.60494062917191327, -0.93440166776717049], [-0.20266810314316119, 0.38085105974111655, 0.59471557232204608, 0.99947412724977536, 0.72967656769917144], [0.41320475526711542, -0.95845554998695559, -0.080530528739385598, -0.5409082635847382, 0.93566219766179692]])
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.96834309925510476, -0.23232226915428322, -0.55708023820274066, -0.84892462629867582, 0.96017236262703731], [0.57731850444002175, 0.22817007872538775, -0.034410544154563238, 0.45202093499230678, 0.074955254340899558], [-0.77909518865998728, -0.78685835355134182, 0.41368308949172938, 0.23438171061119073, 0.085070064646553201], [-0.8583530816823115, 0.59649161976306297, 0.26193639259035906, 0.81589930012403533, 0.020529937150608912]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.85557203651536029, -0.20526653941263365, -0.49220392469179697, -0.75006077074243205, 0.84835284552604684], [0.51008528789197916, 0.20159790375655015, -0.030403169457776456, 0.39937959200250528, 0.066226133746707924], [-0.68836351259579676, -0.69522259673757469, 0.36550648589396567, 0.20708614294233912, 0.075162995958904896], [-0.7583912093854881, 0.527025545261394, 0.23143186853751829, 0.72088149988940831, 0.01813906677394668]], [[-0.56408017970499713, -0.13533259795503205, -0.32451093121563135, -0.49451641275382402, 0.55932055412494408], [0.33630014607638142, 0.13291385988058052, -0.020044864207101897, 0.26331168202367616, 0.04366300887668096], [-0.45383929969107745, -0.45836150618004129, 0.24097908235305987, 0.13653226582894776, 0.049555098177760347], [-0.50000868590704106, 0.34746888817331512, 0.15258344642235494, 0.47527846709413213, 0.011959119289594838]], [[-0.075932588392055828, -0.018217542162040249, -0.043683426320006512, -0.066568393242233709, 0.075291880380901316], [0.045270409220123674, 0.017891948302792129, -0.0026983015499827128, 0.035445204935901357, 0.0058776135029697521], [-0.061092720466803113, -0.061701468755264444, 0.032438944195805286, 0.018379033187881387, 0.0066707659798074942], [-0.067307725223190867, 0.046773868350609378, 0.020539732558391366, 0.063978713509026133, 0.0016098542640218482]], [[0.29612486413542494, 0.071045480100872485, 0.17035832648285079, 0.25960601135825828, -0.2936262060918719], [-0.17654730417522199, -0.069775716493771942, 0.010522941424826147, -0.13823059002943472, -0.022921745944190029], [0.23825176951018145, 0.24062578978303895, -0.12650665736668687, -0.071675269090013552, -0.02601491284941336], [0.26248928699850788, -0.1824105526193488, -0.080101648607410675, -0.24950667752295089, -0.0062781723276692505]], [[0.42039048108160398, 0.10085895233917437, 0.24184736744574897, 0.36854688418418019, -0.41684328804175413], [-0.25063348311724282, -0.099056346079793217, 0.014938780709589858, -0.19623757164846709, -0.032540610302488962], [0.33823156422664874, 0.34160181659474315, -0.17959381663434265, -0.10175302550961855, -0.036931791458954956], [0.37264009546191024, -0.2589571807618965, -0.11371544463790471, -0.35420947343677689, -0.0089127398769698798]]], [[[0.96672479706965364, 0.23193401045115355, 0.55614924156771672, 0.8475058971529017, -0.95856771543737451], [-0.57635368546402521, -0.22778875919376729, 0.034353036997388543, -0.45126551424574451, -0.074829988562680833], [0.77779315898945744, 0.78554335002193976, -0.41299173923754401, -0.23399001015418985, -0.084927894922165334], [0.85691859563181583, -0.59549475853421918, -0.26149864254401262, -0.81453576314885112, -0.020495627367037819]], [[-0.049305577724397177, -0.011829261454650105, -0.028365114600971453, -0.0432250915778946, 0.048889544512418219], [0.029395595852644448, 0.011617842435837571, -0.0017520977437199148, 0.023015761005022079, 0.0038165316834499036], [-0.039669553496822094, -0.040064834188379077, 0.021063695024928109, 0.011934122996907709, 0.0043315522025991343], [-0.043705164643005993, 0.030371842317875875, 0.01333713760509775, 0.041543525624845631, 0.0010453323957541494]], [[-0.5243704057657399, -0.12580553590825491, -0.30166620774718783, -0.4597037466360806, 0.51994584541687128], [0.3126254926194626, 0.12355707068793019, -0.018633758029388995, 0.24477522613509295, 0.040589246893220465], [-0.42189019627654412, -0.42609405121931943, 0.22401478325406621, 0.12692075029171063, 0.046066548469799164], [-0.46480937807917588, 0.32300798439275075, 0.14184196962117882, 0.44182010219222223, 0.011117228472316375]], [[-0.5857900837176635, -0.14054117967283866, -0.33700046979760523, -0.51354899755265226, 0.58084727316108242], [0.34924341930853475, 0.13802935098234104, -0.020816336231009386, 0.27344582881312257, 0.045343478720724563], [-0.47130633361278318, -0.47600258746652452, 0.25025370843490768, 0.14178701948352301, 0.051462338430981226], [-0.51925265328454817, 0.360842015755241, 0.1584559661366331, 0.49357063595795764, 0.012419393096749191]], [[0.90482140691480062, 0.2170823157572157, 0.52053670365677351, 0.79323658662210461, -0.8971866569826481], [-0.53944737338160498, -0.2132025020955689, 0.032153269846799747, -0.42236911552248718, -0.070038314664048987], [0.7279878436332704, 0.73524175785490364, -0.38654616874814757, -0.21900666128921895, -0.079489610282800313], [0.80204655105704215, -0.55736276431574694, -0.24475380208534783, -0.76237766676596574, -0.01918320751268416]]], [[[0.19625225911780195, 0.047084313607413532, 0.11290239517508985, 0.1720499437234696, -0.19459631142410916], [-0.11700404620430589, -0.046242797049300094, 0.0069739197119293248, -0.091610225475888984, -0.015191039217883312], [0.15789774405368293, 0.15947108995660134, -0.083840367049691389, -0.047501696701019364, -0.017240988636183036], [0.17396079089164096, -0.12088982511817173, -0.05308615183045045, -0.16535676351197101, -0.0041607634199622235]], [[-0.36879449554430388, -0.088480182408869679, -0.21216459908034743, -0.32331384356618204, 0.3656826618406388], [0.21987236422413878, 0.086898816283777922, -0.013105292207533892, 0.17215265211699038, 0.028546788048896522], [-0.29671922824036129, -0.29967583781617868, 0.15755164302990429, 0.089264522870207696, 0.032399024272885081], [-0.32690468079076151, 0.22717446551345769, 0.099758752702803391, 0.31073611309427418, 0.0078188483202279221]], [[-0.57588872047760353, -0.13816567126324597, -0.3313042926920447, -0.50486869498749609, 0.57102945616754963], [0.34334030478015526, 0.1356962989559353, -0.020464486460794115, 0.26882388905549609, 0.04457705698389261], [-0.46334004101727683, -0.46795691606866913, 0.24602377532702605, 0.13939045316795448, 0.050592492183748346], [-0.51047594422708786, 0.35474285503269432, 0.15577765163134755, 0.48522801923042252, 0.012209473322260016]], [[-0.96783387400633847, -0.23220009720366463, -0.55678728488578122, -0.84847819997071072, 0.95966743414601297], [0.57701490837033631, 0.22805009029856946, -0.034392448587071953, 0.45178322950006328, 0.07491583741515552], [-0.77868548373043989, -0.78644456618492253, 0.41346554482773695, 0.23425845565642928, 0.085025328617695725], [-0.85790169718658338, 0.59617794107449218, 0.26179864737920366, 0.81547024091517273, 0.020519141016097583]], [[-0.70657726901964291, -0.16952011595658048, -0.40648839614481264, -0.61944040755291951, 0.70061527396130074], [0.42125578478901393, 0.16649035989599067, -0.025108567751362491, 0.32982908437335673, 0.054693092718486012], [-0.56848750317235797, -0.57415210268476424, 0.30185485685551428, 0.17102284213023411, 0.062073632785243568], [-0.62632013051595559, 0.43524595777003106, 0.19112884790083587, 0.59534260090266222, 0.014980214075136019]]], [[[-0.40012397334230559, -0.095996666368996522, -0.23018820349070981, -0.35077969244997176, 0.39674778611355294], [0.23855075133831627, 0.094280961539002298, -0.014218600475994572, 0.18677719981910881, 0.030971867525915794], [-0.32192583676003717, -0.3251336134090676, 0.17093581975157424, 0.096847637372184936, 0.035151355242836704], [-0.35467557504931385, 0.2464731737630817, 0.10823336299585037, 0.33713347063036275, 0.0084830676559666165]], [[0.92811381777262458, 0.22267056825648607, 0.53393664609347202, 0.81365651959656804, -0.92028252990397164], [-0.55333412469070775, -0.21869087829530848, 0.03298097702301233, -0.4332419738536693, -0.07184127952371902], [0.74672810753929908, 0.75416875601488165, -0.39649685305909843, -0.22464445135073227, -0.081535875598238008], [0.8226932749868181, -0.57171070348261654, -0.25105438922179169, -0.78200321243435444, -0.019677032202884498]], [[0.077981181784148937, 0.018709035172928282, 0.04486196613272958, 0.068364349015717696, -0.077323188043300392], [-0.046491764413586281, -0.018374657082262713, 0.002771099314976952, -0.036401484896201909, -0.0060361862638677686], [0.062740947481100151, 0.063366119254491968, -0.033314117927311537, -0.018874883082360853, -0.0068507372858806429], [0.069123627512957586, -0.048035785528132008, -0.021093876191388899, -0.065704802037083218, -0.001653286693724891]], [[0.52378478437234255, 0.12566503520030953, 0.30132930432361676, 0.4591903455255395, -0.51936516541064637], [-0.31227634977199015, -0.1234190810853425, 0.018612947687650764, -0.24450185905063848, -0.040543916472088389], [0.42141902566529776, 0.42561818570660231, -0.22376460161134121, -0.12677900410271978, -0.046015100951008518], [0.46429027495538805, -0.3226472462888863, -0.14168355928570139, -0.44132667369009515, -0.011104812655639675]], [[-0.90604203233966685, -0.21737516492267214, -0.52123891995073368, -0.7943066814918387, 0.89839698294973358], [0.5401751006151726, 0.21349011730086151, -0.032196645366396937, 0.422938901424042, 0.07013279800290502], [-0.72896991640933595, -0.7362336163323917, 0.38706762864935335, 0.21930210644219802, 0.0795968436424251], [-0.80312853077664748, 0.55811465983435249, 0.24508398073869858, 0.76340613222477693, 0.019209086112197304]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_constData_rank2(self):
      arg0=numpy.array(0.372750342838)
      arg1=Data(numpy.array([[0.76871462138898106, -0.69295903418971228, 0.054927886711077623, 0.35251075234789675, -0.21985084950520184], [0.084613974030040451, -0.55016187666396577, 0.54086086224118057, -0.068569600221233706, -0.0033024490285564756], [0.86799036985074474, 0.44034217230811157, -0.096308260724639938, 0.40820711124699538, -0.98327313126459037], [0.011621419115650511, -0.81993724395384415, 0.87297909269920226, -0.80204874976487361, -0.89363699263737328]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.28653863866720669, -0.25830071756679701, 0.020474388602912477, 0.13139850379170509, -0.081949479526255378], [0.031539887828570062, -0.20507302814280526, 0.20160607182797236, -0.025559341990718815, -0.0012309890075989335], [0.3235437079418127, 0.16413769569380987, -0.035898937203226067, 0.15215934066616374, -0.36651539678211714], [0.0043318879596210098, -0.30563188878931291, 0.32540325609389825, -0.29896394644752133, -0.33310349537816136]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_constData_rank2(self):
      arg0=numpy.array([-0.7248354981517402, -0.71175522139340708])
      arg1=Data(numpy.array([[-0.52973733182056404, 0.93852863726967595, -0.5026798935486676, 0.88752851790782139, 0.68830786230505958], [-0.84886692931572005, 0.14548667127138337, 0.38695425878446543, 0.84252686359018125, -0.74463622630528481], [-0.55814036535876954, -0.4005814259625835, 0.11271709200032509, -0.57891927518335184, 0.71421601651008437], [0.12439171343058342, 0.44403183920210854, -0.76908159470076143, -0.53483151543574237, -0.37397019025429445]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.3839724227997322, -0.68027887232503947, 0.36436023105121224, -0.64331217540159136, -0.49890997225564726], [0.61528888357509803, -0.10545390384543163, -0.28047818292797538, -0.61069337887661224, 0.53973877003582305], [0.40455994976341803, 0.29035563743792364, -0.081701349530271167, 0.41962124121716921, -0.51768912211503848], [-0.090163529570405437, -0.32185003936329376, 0.55745764081426119, 0.38766486791811644, 0.27106686914687256]], [[0.37704331189029833, -0.66800265800393088, 0.35778503892274621, -0.63170305675644389, -0.48990671492176041], [0.60418546920865202, -0.1035508979205533, -0.27541671413025892, -0.59967289432452231, 0.52999872211146914], [0.39725931931452813, 0.28511592152208531, -0.080226978771512414, 0.41204881687703737, -0.50834697895385239], [-0.088536451532290145, -0.3160419800170185, 0.54739784070583497, 0.38066912367713823, 0.26617523555897993]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_constData_rank2(self):
      arg0=numpy.array([[0.11795870957734778, -0.82634566459596437, -0.81327808756997855, 0.76035332192267013, 0.34229080331266304], [-0.18303348598622482, -0.68197677284763358, 0.049397167409949372, 0.40385722090682474, 0.27340856521325163], [-0.51879714918404063, -0.77485833329140297, 0.14980458961576404, 0.33593403206236161, -0.68622055619837052], [-0.42127435895109344, 0.88103478757968023, 0.16906790486401468, -0.86313143450198027, 0.30859617370497516]])
      arg1=Data(numpy.array([[0.71881854358742059, 0.10699636106889332, 0.86427296110378049, -0.48139841430720809, -0.57445230950683612], [0.54036999379716311, 0.018193274282380045, -0.74821022888831834, 0.67727540924340257, 0.79758060153710808], [0.92275615326190508, -0.32084725677028247, 0.45755998301402356, 0.78740423840981788, -0.17005371414933568], [0.88434579513768496, 0.46409423804178584, 0.72546943412878195, -0.23777811168984098, 0.73729617122869739]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.08479090782184065, 0.012621152681158628, 0.10194852321439524, -0.056785135744259696, -0.067761653143153577], [0.063741347162632786, 0.0021460551573362981, -0.088257913092238044, 0.079890533302821878, 0.094081578541242064], [0.10884712509333168, -0.037846728380054477, 0.053973185150567385, 0.092881187878556412, -0.020059316679890804], [0.10431628881459487, 0.05474395744169152, 0.085575438287639818, -0.028047999220672113, 0.086970504934456391]], [[-0.5939925871246502, -0.088415979096824415, -0.71418821443562552, 0.39780149260613329, 0.4746961754781131], [-0.44653240165203389, -0.015033933328050005, 0.61828027884821601, -0.55966359816574329, -0.65907727224603063], [-0.76251554672722455, 0.26513073962963107, -0.37810270825624148, -0.6506680786944401, 0.14052314943574493], [-0.73077531381569683, -0.38350226156979722, -0.59948852168920652, 0.19648691173071509, -0.60926149461803791]], [[-0.58459937043861465, -0.087017795907056467, -0.70289426094492502, 0.39151078174698639, 0.4671894756758771], [-0.439471075135558, -0.014796191315010117, 0.60850298405058745, -0.55081324958764899, -0.64865482630101234], [-0.7504573596182722, 0.2609380433882092, -0.37212350793419696, -0.64037861315843214, 0.13830095942754353], [-0.71921905702012845, -0.37743767436686998, -0.59000839397873028, 0.19337972794111463, -0.59962682010954249]], [[0.54655606747631091, 0.081355038572370489, 0.65715281702320216, -0.36603288348679153, -0.43678672181967271], [0.41087211985100563, 0.01383331653725795, -0.56890413303175424, 0.51496860727475702, 0.60644305987982161], [0.70162070645727403, -0.24395727751506019, 0.34790725306359332, 0.59870542837089513, -0.12930090645873554], [0.67241526306128385, 0.35287559558024229, 0.55161309419317905, -0.18079537710387028, 0.56060559303460589]], [[0.24604497672057668, 0.036623870381803238, 0.29583268613762698, -0.16477824994665644, -0.19662974248590948], [0.1849636792628897, 0.0062273904690034786, -0.25610548029293395, 0.23182514389383688, 0.27300450480673372], [0.31585094496172028, -0.10982306526056425, 0.15661857414959859, 0.26952122929709221, -0.05820782242247808], [0.30270343262385391, 0.15885518955210115, 0.24832151538672387, -0.081389260860483786, 0.2523696987292216]]], [[[-0.13156786382434668, -0.019583916954280335, -0.15819089291446184, 0.088112029918889223, 0.10514400874187398], [-0.098905803687049446, -0.0033299784134075525, 0.13694752644398009, -0.123964079126567, -0.14598395785432702], [-0.16889527544676564, 0.058725791875782175, -0.083748798738854546, -0.14412134263647744, 0.031125524105657915], [-0.1618648937013103, -0.084944786214908891, -0.13278519950504486, 0.043521356673813509, -0.13494988842428499]], [[-0.49021755061878514, -0.072969033028204039, -0.5894140848730246, 0.32830253704319784, 0.3917631321923421], [-0.36851978451348505, -0.01240739048262939, 0.51026199730884458, -0.461886097924876, -0.54393144472215127], [-0.62929826352685037, 0.2188103767492133, -0.31204528060012182, -0.53699140143727631, 0.11597268318631791], [-0.60310329144937291, -0.31650149075691858, -0.49475330348674562, 0.16215914926404193, -0.5028188634874633]], [[0.035507599934963802, 0.0052853171599755132, 0.042692636147536107, -0.023779718062417327, -0.028376316901741236], [0.026692747046911769, 0.00089869621546185359, -0.036959465934232799, 0.033455486773038327, 0.039398222497056649], [0.045581540181239222, -0.015848945655704657, 0.022602167081037312, 0.038895538984033459, -0.0084001717865184108], [0.043684177290701016, 0.022924940770542988, 0.035836135088460681, -0.011745565189564714, 0.036420342400898663]], [[0.29030005934950692, 0.043211253028426433, 0.34904287617628504, -0.19441622575106127, -0.23199671326093799], [0.21823232395636041, 0.0073474851908776114, -0.30217010369289549, 0.27352256456557295, 0.32210868518597002], [0.372661735631025, -0.1295764814548247, 0.1847889031382175, 0.31799888745444393, -0.068677420401234285], [0.35714943514494163, 0.18742780921442601, 0.29298606952009665, -0.096028407379531749, 0.29776238269766414]], [[0.19653114665091584, 0.029253721562885138, 0.23629963024799305, -0.13161844975166823, -0.15706018172570282], [0.14774178468837604, 0.0049741970180766787, -0.20456708515823371, 0.18517289789545652, 0.21806536790818287], [0.25228943590503677, -0.087722388126170669, 0.12510081845486398, 0.21528306306646142, -0.046494141994754296], [0.2417877150009666, 0.12688733974674193, 0.19834955709121985, -0.06501057235623571, 0.20158308831286206]]], [[[-0.37292101119377785, -0.055509407095608122, -0.44838234833749058, 0.24974812496429724, 0.29802422051433475], [-0.28034241228656592, -0.0094386188320220894, 0.3881693337375981, -0.35136855152793167, -0.41378254232194395], [-0.47872326170430801, 0.16645464213594244, -0.23738081476837347, -0.40850307414244419, 0.088223382108833098], [-0.45879607741032458, -0.24077076764881802, -0.37637147424617123, 0.1233586064830539, -0.38250715173775651]], [[-0.55698253862310243, -0.082907022006087835, -0.6696891061497009, 0.37301557295920751, 0.44511915909986421], [-0.41871019275435556, -0.014097210187558348, 0.57975693090798153, -0.52479249478559575, -0.61801197557259813], [-0.71500529495090615, 0.24861117062213989, -0.35454416581908893, -0.610126735800818, 0.13176753751576692], [-0.68524270887364702, -0.35960728777920181, -0.56213603658288525, 0.18424435131716724, -0.57130008238040131]], [[0.10768231693031474, 0.016028545960305677, 0.12947205625415303, -0.072115691896970854, -0.086055592479499454], [0.080949905161456975, 0.0027254359876389767, -0.11208532628493141, 0.10145896473855656, 0.11948123469876069], [0.13823310685482076, -0.048064391629815836, 0.068544585480011758, 0.11795676879669599, -0.025474826860777677], [0.13247905891902742, 0.069523446872890429, 0.10867865085844275, -0.035620252441307933, 0.11045035035618911]], [[0.24147561166851664, 0.035943718989873627, 0.29033870062606959, -0.16171811034664768, -0.19297808056016716], [0.18152867082179433, 0.006111739986096397, -0.25134927902075527, 0.22751985904382227, 0.26793446736908455], [0.30998519517562628, -0.10778351264298884, 0.15370997000428663, 0.26451588067200321, -0.057126829861366607], [0.29708184869799775, 0.15590504864228658, 0.24370987214488157, -0.079877759796142839, 0.24768287562499769]], [[-0.4932680607862624, -0.073423102403897655, -0.59308187207584895, 0.33034548761890592, 0.3942009833392196], [-0.3708129976963993, -0.012484598797124345, 0.51343723942105191, -0.46476030803048674, -0.54731620399982528], [-0.63321424072685328, 0.22017198299562465, -0.3139870660380002, -0.54033297443453954, 0.11669435430715584], [-0.60685626341107246, -0.31847100615749324, -0.49783203859276987, 0.16316822805560094, -0.50594778870348578]]], [[[-0.30281982115194922, -0.045074823419397766, -0.36409603764775844, 0.20280080838734199, 0.2420020284354675], [-0.22764402273330622, -0.0076643599605310678, 0.31520178453557718, -0.28531876386235389, -0.33600025662437272], [-0.38873350693358599, 0.13516472241711763, -0.19275828852590599, -0.33171321577146995, 0.071639269415513879], [-0.37255220793772326, -0.19551100262394949, -0.30562167080121511, 0.10016982157473925, -0.3106039718914651]], [[0.63330414285787817, 0.094267516246131194, 0.76145454469693052, -0.42412874969034597, -0.5061124684810121], [0.47608476269951672, 0.016028907542755561, -0.65919924007356345, 0.59670319631570223, 0.70269625585291962], [0.8129802714769454, -0.28267759471412895, 0.40312626243972233, 0.69373052592673379, -0.14982323792269561], [0.77913940976611373, 0.40888316843009836, 0.63916380879320223, -0.20949078812375652, 0.64958357560178692]], [[0.12152914514172762, 0.018089650593991421, 0.14612081876443422, -0.081389021311778581, -0.097121448412615288], [0.091359222702666984, 0.0030758987655383546, -0.12649833579597286, 0.1145055344567002, 0.13484528126205939], [0.15600844953236792, -0.054244973483518207, 0.07735870767779511, 0.13312478486899304, -0.028750625165572231], [0.14951449075922954, 0.07846344048518604, 0.12265359727103547, -0.040200647165923094, 0.12465311893389569]], [[-0.6204348806732346, -0.09235192261588572, -0.74598116071878029, 0.41551010390795912, 0.49582784595761104], [-0.46641032790797154, -0.015703186929638673, 0.64580376816942919, -0.58457769553317385, -0.68841688873567641], [-0.79645984226047728, 0.2769333529921591, -0.39493440450959588, -0.67963334983160539, 0.14677870623610581], [-0.76330665475298443, -0.40057432542511012, -0.62617547336691548, 0.20523376263602452, -0.63638350192544324]], [[0.22182465213926089, 0.033018667626216448, 0.26671132883329551, -0.14855770868284679, -0.17727378468979577], [0.1667561124707857, 0.0056143748307076096, -0.23089481376185872, 0.20900459983698519, 0.246130321855664], [0.28475901815934557, -0.099012235783046854, 0.14120125999864111, 0.24298993513234984, -0.052477925510804588], [0.27290572861157342, 0.14321770609822104, 0.22387709151205562, -0.073377415458279152, 0.22752677732850421]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(-0.921419557017)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.27411787614041705, -0.32088094729570171, 0.75322329571656921, -0.7137139228727496, 0.22883188436579371], [-0.54384414564382388, 0.65122352185543342, -0.89272014052793924, -0.92983942126507046, 0.029342402377037446], [-0.30097855865431367, 0.84358871207542063, -0.40336790135540901, -0.57151413265873607, 0.85041879066142778], [-0.87329972577643389, 0.2208570947409334, 0.67495300452707463, -0.081365038924114508, 0.61526159570110073]])+(1.-msk_arg1)*numpy.array([[0.073197150752966511, -0.26693292886439668, 0.57752280048264093, -0.51305472225006565, -0.8102674893157904], [0.71818111351797986, -0.13429691550499201, 0.12139067548095128, -0.17097142145570587, 0.55948298386490847], [-0.54227231534025067, 0.80440840691869786, -0.8401661844295587, -0.60447001903774922, 0.7220179464068277], [-0.13044206585017437, 0.15860167865578378, -0.8719997620994473, 0.94997502334238826, 0.31743055979649304]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.25257757200387571, 0.29566598031255498, -0.694034675474408, 0.65762996665061724, -0.21085017352380497], [0.50110863176568243, -0.60005008902739709, 0.82256979642583683, 0.85677222763945171, -0.027036663400078514], [0.2773275301870175, -0.77729913738548106, 0.37167107298197177, 0.52660429894364225, -0.78359250537059433], [0.80467544646840894, -0.2035020464003586, -0.62191489843895464, 0.074971338122167649, -0.56691406696077651]])+(1.-msk_ref)*numpy.array([[-0.067445286221740147, 0.24595722106761103, -0.53214080298820987, 0.472738654901382, 0.74659631107102176], [-0.66174612347605788, 0.12374380439342375, -0.11185174242771087, 0.15753641142036548, -0.51551856315162214], [0.49966031658365728, -0.74119763796416382, 0.77414555347815084, 0.55697049717211089, -0.66528145633685021], [0.12019187053211269, -0.14613868848924111, 0.80347763451302112, -0.87532556518581417, -0.29248672579149548]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_expandedData_rank2(self):
      arg0=numpy.array([0.45174073833470918, -0.32999422808025969])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.097138691944465005, -0.21553892514741024, -0.51682088548807403, 0.4592939442072681, 0.13793149312241337], [-0.86233231279442091, 0.11269037329575693, -0.42541177929681973, 0.41252622674889983, 0.34353528222730056], [-0.93182078713230698, 0.64040311928947813, -0.020493489734802806, -0.30033328959216421, -0.62520987906799363], [-0.22284178783794584, -0.19029020302641642, -0.12015018256454324, -0.67086908239846776, 0.19266225130706749]])+(1.-msk_arg1)*numpy.array([[0.44810843186804195, 0.067119478562676527, 0.97389928428461525, -0.89095081193287706, 0.40325892981648548], [-0.8596003320912915, 0.36057172930421189, -0.25785774808618123, -0.49573945621014581, 0.12835464234318317], [-0.75805229637792237, 0.54720140452346189, -0.018817847608054938, -0.49377906267057603, -0.47063696225714668], [-0.96006565494914842, 0.54119283890237635, 0.53704331265262861, 0.059162490255933786, -0.963426135668078]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.043881504419860488, -0.097367713185960714, -0.23346904839718075, 0.20748178546885201, 0.062309274542727881], [-0.38955063567162906, 0.050906832435839232, -0.19217583127582769, 0.18635490225397966, 0.15518888203738346], [-0.42094141037477822, 0.28929617793967971, -0.0092577441838546035, -0.13567278198685628, -0.28243277238432968], [-0.10066671376974026, -0.085961836813015058, -0.054276732182756866, -0.30305889460861268, 0.087033387654681951]], [[0.032055207664939869, 0.07112660122526851, 0.17054790916239329, -0.15156435058061529, -0.04551659660088845], [0.28456468590925998, -0.037187172747809617, 0.14038343172530382, -0.13613127375886538, -0.11336466027693221], [0.30749548135886562, -0.2113293330101218, 0.0067627333257069778, 0.099108252065771313, 0.20631565143119507], [0.073536503761607933, 0.06279466865893818, 0.039648866749088719, 0.22138292498899451, -0.063577430900280743]]])+(1.-msk_ref)*numpy.array([[[0.20242883386607799, 0.030320602802544179, 0.43994998174637695, -0.40247877760246653, 0.18216848669536381], [-0.38831648869168123, 0.16288493921850758, -0.11648484950577695, -0.2239457079700185, 0.057983020900797093], [-0.34244310406208445, 0.24719316649721854, -0.0085007883723327796, -0.22306011834502665, -0.21260588881764811], [-0.43370076781652445, 0.2444788526272168, 0.24260434257541652, 0.026726107029935567, -0.43521883385765325]], [[-0.14787319607055013, -0.022149040517439978, -0.321381142545419, 0.29400862544127038, -0.13307311926126275], [0.28366314804600062, -0.11898658947930775, 0.08509156853421343, 0.16359115918099476, -0.042356291120556541], [0.25015288238770073, -0.18057330509015374, 0.0062097810955520508, 0.16294424062817092, 0.15530748106608544], [0.31681612471131321, -0.17859051311615401, -0.17722119340446973, -0.019523280303312754, 0.31792506395213493]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[0.61580845642221882, 0.17822708313385949, 0.58972637660522143, -0.74843177869717703, 0.9976037022648363], [0.55187701135213363, 0.22487493386286173, 0.29391218079007464, 0.80458982942274737, -0.61481939252808426], [-0.22926550831669279, -0.77143931293288204, 0.84539028644557157, 0.11279761558864387, -0.99157575517310503], [-0.70875966152114378, 0.13774326141678772, 0.99325430842393869, 0.28694347396680775, 0.3742883562671242]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.25127802526427567, -0.23594707373025448, -0.18977891359341159, -0.037150166425794717, -0.93771552478023867], [-0.21845508307326078, 0.41466029902966173, -0.001028329177914955, 0.50999614420739126, -0.5183973685029688], [0.039164165095863623, -0.6280324353802238, 0.29056793167506711, -0.015424500823010723, -0.39665605959047312], [0.48049748658112046, 0.26031355791525357, 0.63368537749662179, 0.039647163322244383, -0.96028049287581796]])+(1.-msk_arg1)*numpy.array([[-0.34344273002027159, 0.47629490451433143, 0.37993140152802929, -0.72076603796295546, -0.30431457202317413], [-0.96759705784412997, 0.22558806065373616, 0.42407326960757774, -0.61572292288676489, -0.30944640576414106], [-0.064828948526520991, -0.23692625772967313, 0.4289947836296979, -0.29011540864147944, -0.49760550944029203], [0.010984444410195859, 0.62280811555934634, -0.29684745695028547, -0.10474252571079257, -0.35566145942346195]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.15473913287081689, -0.14529820327116746, -0.11686745984144443, -0.022877386642497183, -0.57745314987806962], [-0.13452648750493229, 0.25535131868503169, -0.00063325380374573768, 0.31405993834563695, -0.31923348331115337], [0.024117624054748717, -0.38674768461458248, 0.17893418949061982, -0.0094985380429014766, -0.24426415578692889], [0.2958944155262756, 0.16030329028556817, 0.39022881417352573, 0.024415058446990923, -0.59134884805022492]], [[0.044784549498488105, -0.042052158724912941, -0.033823742210066503, -0.0066211658000068308, -0.16712630279091828], [-0.038934612251912228, 0.07390369558747055, -0.00018327610988122208, 0.090895125191598508, -0.092392450892552616], [0.0069801149084086841, -0.11193238907127138, 0.051787074914685788, -0.0027490637904810161, -0.070694852508180375], [0.085637665486503894, 0.046394926127432644, 0.11293989645580155, 0.0070661982734553539, -0.17114799123560198]], [[0.14818527935961659, -0.13914421286154799, -0.11191763106951802, -0.021908433036564867, -0.55299557871511384], [-0.12882872459178671, 0.24453611566880001, -0.00060643284004921255, 0.30075817820605882, -0.30571260176893755], [0.023096141174752338, -0.37036729250733225, 0.17135557350441089, -0.0090962349812983689, -0.23391854078079449], [0.28336204172940016, 0.15351377129057595, 0.37370098157879467, 0.023380977968702612, -0.56630273558833222]], [[-0.18806445939605604, 0.17659028807032834, 0.1420365698599349, 0.027804365136953689, 0.70181609812323087], [0.16349872638996013, -0.31034494515787303, 0.00076963423571309558, -0.38169732133783985, 0.38798506458061288], [-0.029311705743887109, 0.47003943269114079, -0.2174702739359303, 0.011544186586481986, 0.29687000021031124], [-0.35961958854143095, -0.19482693916950383, -0.47427027421418871, -0.029673196965564842, 0.71870443733125022]], [[0.2506758883014385, -0.23538167429185616, -0.18932414681258586, -0.037061143566127629, -0.93546847919197995], [-0.21793159965245734, 0.41366664949423465, -0.0010258649950349147, 0.50877404160208484, -0.51715513406291036], [0.039070316095744829, -0.62652748267771285, 0.28987164439848295, -0.015387539126622512, -0.3957055535732375], [0.47934607154227427, 0.25968976912598885, 0.63216687866172028, 0.039552156914569626, -0.9579793749056178]]], [[[0.13867456560131439, -0.13021376588753436, -0.1047346196515868, -0.020502322818301964, -0.51750364131421567], [-0.12056033836115324, 0.22884148655487174, -0.0005675112333939019, 0.28145514786628684, -0.28609159042222909], [0.021613802385206764, -0.34659666346983992, 0.16035776172760699, -0.0085124274158016825, -0.21890536070150413], [0.26517551685660068, 0.14366106835671069, 0.3497163922703842, 0.021880358002870158, -0.5299567284680603]], [[0.056506129312494491, -0.053058582600226736, -0.042676520642884179, -0.0083541412179948937, -0.21086871661713485], [-0.049125072358105482, 0.093246707319849645, -0.00023124545587287649, 0.1146853491989516, -0.11657457395678666], [0.0088070390357265303, -0.14122875236985979, 0.065341444418099248, -0.0034685836024421924, -0.089198005166710989], [0.10805184051620076, 0.058537994119798868, 0.1424999573544154, 0.0089156532299397833, -0.21594301232524582]], [[0.07385367239004674, -0.069347718991095625, -0.055778334362210739, -0.010918886430919539, -0.27560601484886921], [-0.064206609870738993, 0.12187371277487236, -0.00030223847125104912, 0.14989407893852374, -0.15236330109254351], [0.0115108251721478, -0.1845863826895032, 0.085401454466280377, -0.0045334486744893826, -0.11658204749783375], [0.14122406414520675, 0.07650932549609557, 0.1862478512348138, 0.011652784234181108, -0.28223813383129936]], [[0.20217574348506837, -0.18984061580542186, -0.15269418371615734, -0.029890646067556849, -0.75447637412999424], [-0.17576673802644699, 0.33363145926466092, -0.00082738319784902775, 0.41033771067408381, -0.41709725029700478], [0.031511088913965229, -0.50530851005452693, 0.23378800258216276, -0.012410396486117223, -0.31914543132539791], [0.38660339076636258, 0.20944564115946235, 0.50985680978769621, 0.031899704374540414, -0.7726319179609461]], [[-0.15449060284863858, 0.14506483653961416, 0.11667975637014111, 0.022840642754224341, 0.57652568930954007], [0.13431042146977437, -0.25494119315493041, 0.00063223672048457694, -0.31355551957325356, 0.31872075519115273], [-0.024078888193108473, 0.38612652040840251, -0.17864679924060667, 0.0094832822260523877, 0.24387183759999828], [-0.29541917281107583, -0.16004582354428049, -0.38960205884640275, -0.024375844869244035, 0.59039906928647978]]], [[[-0.057609384191028935, 0.054094525794602985, 0.043509759092783228, 0.00851725178965956, 0.21498582644519576], [0.050084215665156487, -0.095067304235787228, 0.000235760411691559, -0.11692452524126092, 0.11885063619986905], [-0.0089789922185020514, 0.14398617553681753, -0.066617204556014317, 0.0035363060217187998, 0.090939553128906211], [-0.11016150050591385, -0.05968092017716746, -0.14528220018461835, -0.0090897270523892966, 0.22015919532577868]], [[-0.19384574716500419, 0.18201884844699157, 0.14640291471165023, 0.0286590988628573, 0.72339062016296429], [0.16852483919273198, -0.31988525618398567, 0.00079329355447954836, -0.39343107508576891, 0.39991210978414432], [-0.030212776613142995, 0.48448891044928449, -0.2241555255717424, 0.011899066317236065, 0.30599607808113888], [-0.37067465091411628, -0.2008161122652572, -0.48884981223160789, -0.030585380433049966, 0.74079812364697029]], [[0.21242800175564358, -0.19946736424681422, -0.1604372501240636, -0.031406389836203252, -0.79273559609842548], [-0.18467980525479508, 0.35054978897429206, -0.00086933949827786299, 0.43114578643762352, -0.43824809985135532], [0.033109004748793802, -0.53093252044319728, 0.24564330699068226, -0.013039723169044989, -0.33532917983756177], [0.40620790781719063, 0.2200665532916421, 0.53571146279823922, 0.033517326757746539, -0.81181180094038241]], [[0.028343562099633311, -0.026614267321890657, -0.021406608942340102, -0.0041904501915509366, -0.10577207529566482], [-0.024641212483882933, 0.046772693009819899, -0.00011599307930903727, 0.057526349025995904, -0.058473987094562439], [0.0044176244393334093, -0.070840561223218301, 0.032775369859471555, -0.0017398469144806845, -0.044741857730592402], [0.054198970782686796, 0.029362748638236957, 0.07147819961500862, 0.0044721054876027027, -0.10831734989267999]], [[-0.24916119765983072, 0.23395939781496136, 0.18818016956231853, 0.036837204328463928, 0.92981597962150964], [0.21661476396977195, -0.41116709915064237, 0.0010196662811575599, -0.50569981182781576, 0.51403026215308167], [-0.038834236580655136, 0.62274173638534969, -0.28812011627979184, 0.015294561051745037, 0.39331453183241155], [-0.47644965811545342, -0.25812061277161535, -0.62834705673336688, -0.039313165911725907, 0.9521908549013407]]], [[[-0.17809572813400945, 0.16722976811395951, 0.13450763856231679, 0.026330539381400425, 0.66461493794636373], [0.15483215073657766, -0.29389449318651933, 0.00072883824007131958, -0.36146469454551905, 0.36741914343361581], [-0.027757980397102495, 0.44512405632438701, -0.2059428289029194, 0.010932263982449683, 0.28113381453565439], [-0.34055723595099524, -0.18449974919737977, -0.4491306336654039, -0.028100310056547437, 0.68060807709602189]], [[0.034611854722271317, -0.032500119457352532, -0.026140766506491261, -0.0051171850856654121, -0.12916399466438469], [-0.030090715615586239, 0.057116661968406063, -0.00014164541477605007, 0.070248532213112461, -0.071405744247479266], [0.0053945998309697764, -0.086507235924800002, 0.040023774572054083, -0.0021246210490874235, -0.054636699308723467], [0.066185290904252722, 0.035856438458254883, 0.087285890608512981, 0.0054611295819299863, -0.13227216696363556]], [[0.24958298120600111, -0.23435544754259599, -0.18849872357467046, -0.036899562861086956, -0.93138998506398674], [-0.21698145245962572, 0.41186312854357027, -0.0010213923864420761, 0.50655586751358772, -0.51490041974120593], [0.038899975717292985, -0.62379592227138614, 0.28860785002609307, -0.015320451897743989, -0.39398034015069999], [0.47725619873357156, 0.25855756294049009, 0.62941073138376957, 0.039379715786606789, -0.95380273684436956]], [[0.07210258950085055, -0.067703473008461745, -0.054455820752140158, -0.010659997812662601, -0.2690713502730499], [-0.062684260442749024, 0.11898406671968646, -0.00029507234669234872, 0.1463400653285459, -0.14875074181349326], [0.011237901587616705, -0.18020980877183612, 0.083376571738193794, -0.0044259598503585823, -0.11381786770887546], [0.13787561803190629, 0.074695276628862661, 0.18183188362084865, 0.011376494776614206, -0.2755462206083456]], [[0.094050439042214648, -0.088312242392534906, -0.071032037623038613, -0.013904874726560809, -0.35097600241615928], [-0.08176519396168884, 0.15520252173304627, -0.00038489163770331165, 0.19088561851795571, -0.19403009895017886], [0.014658690978305074, -0.23506522792090287, 0.10875619353059893, -0.005773211059285588, -0.14846374454751265], [0.17984461444293215, 0.097432333706147095, 0.23718105833372266, 0.014839471590537065, -0.35942180723387379]]]])+(1.-msk_ref)*numpy.array([[[[-0.21149493744321629, 0.29330642995073852, 0.23396496992130594, -0.443853821279526, -0.18739948686437899], [-0.59585445062967402, 0.13891903541845912, 0.26114790556696588, -0.37916738272667555, -0.19055971347901932], [-0.039922214723592371, -0.14590119305840279, 0.26417861552018801, -0.17865552197981069, -0.30642968067561804], [0.0067643137568983817, 0.38353050428983193, -0.18280117425741635, -0.064501333079727732, -0.21901933433643572]], [[-0.061210795995042602, 0.084888651543109414, 0.067714065485299832, -0.12846002856808619, -0.054237098526819127], [-0.17245200126846361, 0.040205902040139567, 0.075581341877197367, -0.10973850056476241, -0.055151730285599582], [-0.011554274398516954, -0.042226675832980669, 0.076458488965962232, -0.051706423054358575, -0.08868677849888143], [0.0019577254870752356, 0.11100127378823799, -0.052906256387953307, -0.018667954837507841, -0.063388504496175141]], [[-0.20253723674626006, 0.28088366823476657, 0.22405556878166821, -0.42505474394799519, -0.17946232990739516], [-0.57061750693629165, 0.13303522961472675, 0.25008719270080598, -0.363108048306788, -0.18248870762479602], [-0.038231340913671634, -0.13972166349355497, 0.25298953933248269, -0.17108870873548282, -0.29345109406101871], [0.006477816601046283, 0.36728637330913932, -0.1750587751917663, -0.061769430163904945, -0.20974294376392319]], [[0.2570434533096862, -0.35647424257006316, -0.28435273462853433, 0.53944420781713176, 0.22775869642277441], [0.72418038706443744, -0.16883727348792241, -0.31738991147032691, 0.46082660236076622, 0.23159952387750446], [0.048520045256771838, 0.17732314049268505, -0.32107332896378538, 0.21713159131700083, 0.37242377651991265], [-0.0082211072679231507, -0.46612938571511858, 0.22217007020703583, 0.078392634822963284, 0.26618833869033548]], [[-0.34261973898416559, 0.47515356011337373, 0.37902097277103008, -0.71903886793860194, -0.30358534370345769], [-0.965278407205867, 0.22504748449491163, 0.42305706379207364, -0.61424746744116299, -0.3087048800428539], [-0.06467359906399385, -0.23635851187487469, 0.42796678440128899, -0.28942020574481575, -0.4964130984850153], [0.010958122410933675, 0.6213156818825899, -0.29613612206150641, -0.10449153143365647, -0.35480918867376049]]], [[[-0.18953814741420519, 0.26285620842561908, 0.20967540639411625, -0.39777420691511434, -0.16794421651905295], [-0.53399457247613602, 0.12449686471030776, 0.23403628862535761, -0.33980332650374806, -0.17077635758677381], [-0.035777606361917715, -0.13075415502669735, 0.23675235907521289, -0.16010802466826263, -0.27461704138226428], [0.0060620623524625409, 0.34371348146074632, -0.16382328736920471, -0.057804992050746219, -0.19628138327975833]], [[-0.077231661198989249, 0.10710678515187833, 0.085437048791039943, -0.1620822151175165, -0.068432719257216348], [-0.21758832438859832, 0.050729100219760161, 0.095363448456011579, -0.13846065156200915, -0.069586740030311497], [-0.014578405512300274, -0.053278776537335579, 0.096470173596240993, -0.065239683330849799, -0.11189900602518128], [0.0024701262102630752, 0.14005393379566156, -0.066753552249054132, -0.023553968541843572, -0.079979347165419878]], [[-0.10094200175675486, 0.13998887408500751, 0.11166646677373258, -0.21184191805711397, -0.089441759509529345], [-0.28438856139702823, 0.066303078866943227, 0.12464029948514045, -0.18096846702808803, -0.090950067955789027], [-0.019054017639757281, -0.069635513095759508, 0.12608679240417073, -0.085268452434620889, -0.1462523204527523], [0.0032284620113680104, 0.18305089145780429, -0.087247083434246198, -0.030785104153119507, -0.10453323516213034]], [[-0.27633052756349302, 0.38322203597810967, 0.30568894154778242, -0.57992102353832375, -0.24484840959498205], [-0.77851875172076079, 0.18150585924119797, 0.34120503965630772, -0.4954044014971376, -0.24897743082925253], [-0.052160712636609596, -0.19062845729248759, 0.34516483978386703, -0.23342390715175859, -0.40036833196038385], [0.0088379722543031371, 0.50110507546099725, -0.23884044475220653, -0.084274770894954318, -0.28616159296976862]], [[0.21115525063925022, -0.29283534385772314, -0.23358919348980664, 0.44314093761525841, 0.18709850030873187], [0.59489743531568962, -0.1386959144127187, -0.26072847000752947, 0.37855839341485725, 0.19025365121190829], [0.03985809475131008, 0.14566685785130995, -0.26375431226892782, 0.17836857930399133, 0.30593751703270822], [-0.0067534494395351287, -0.38291450726975823, 0.18250757315568117, 0.064397736029366728, 0.21866756242838475]]], [[[0.078739572075770259, -0.10919799339212885, -0.087105165896797149, 0.16524679207098567, 0.069768835043069841], [0.22183663131237086, -0.051719561395955745, -0.097225373720103217, 0.14116402889787399, 0.070945387514289374], [0.014863041837569547, 0.054319018911965271, -0.098353707134072321, 0.066513456632693826, 0.11408378006301542], [-0.0025183542312800086, -0.14278841919747509, 0.068056883110224772, 0.024013848399459123, 0.081540905283376805]], [[0.2649452236786316, -0.36743261389196852, -0.29309401935640983, 0.55602725711149792, 0.23476022435702151], [0.74644240949915375, -0.17402749851657953, -0.32714679173927064, 0.47499286858879181, 0.2387191226522388], [0.050011599509460526, 0.18277422947873798, -0.33094344113508456, 0.2238064315136252, 0.38387245231423561], [-0.0084738322487509302, -0.48045866475612514, 0.22899979823560151, 0.080802502069188556, 0.27437123189434159]], [[-0.29034314790948651, 0.4026550857599368, 0.32119031636744816, -0.60932860729374261, -0.25726458321223272], [-0.81799715389474137, 0.19070995521476297, 0.35850742286746023, -0.52052617815034674, -0.26160298560849976], [-0.054805763364800794, -0.2002951568885657, 0.3626680230163663, -0.24526074841369436, -0.42067086416262306], [0.0092861426063809357, 0.52651593121334239, -0.25095195666184134, -0.088548313813679572, -0.30067274305965053]], [[-0.038739521037540997, 0.053724929546237393, 0.042855356179613349, -0.081300690479495341, -0.034325958113092678], [-0.10914264097540498, 0.025445795347007808, 0.047834453646614886, -0.069452077564897519, -0.034904816722671095], [-0.0073125508149104951, -0.026724716942247633, 0.048389588693396116, -0.032724326340283931, -0.056128714968637361], [0.0012390191380361006, 0.070251270404350835, -0.033483685337544811, -0.011814707150909627, -0.040117764579743719]], [[0.34054948437856364, -0.47228247962890013, -0.37673076638413183, 0.71469412839624447, 0.30175095156405918], [0.9594457833350678, -0.22368765160076465, -0.42050077255986168, 0.61053592223883546, 0.30683955348118125], [0.064282813592263413, 0.23493033292863835, -0.42538082654294046, 0.28767140541102892, 0.49341355880155519], [-0.010891908761196951, -0.61756142751369736, 0.29434674129669508, 0.10386014903041751, 0.35266528021378796]]], [[[0.24341835308106527, -0.33757861530782302, -0.26928005154825979, 0.51084989310256024, 0.21568589306309663], [0.68579376320646013, -0.15988771751215328, -0.30056602702723156, 0.43639957041603278, 0.21932312980832713], [0.045948143614428671, 0.16792377423395441, -0.30405419763972097, 0.20562209883080329, 0.3526827124419577], [-0.0077853311021682369, -0.44142126917646374, 0.21039350311149663, 0.074237277069651039, 0.25207849559708889]], [[-0.047306921742877518, 0.065606413544001499, 0.052332990361121878, -0.099280664787473716, -0.041917281647125951], [-0.13327997448473866, 0.031073235211133744, 0.058413235235428483, -0.084811683527500278, -0.042624157163655446], [-0.0089297508042640566, -0.032634995454959591, 0.05909114062794376, -0.039961442573541499, -0.06854180576926798], [0.0015130331979117809, 0.085787621073987977, -0.040888736863611809, -0.014427577100436309, -0.048989969381242157]], [[-0.34112597128951433, 0.47308196598922819, 0.37736850147326051, -0.71590397257235772, -0.30226175977820469], [-0.96106994652200917, 0.22406631317332423, 0.4212126021251531, -0.61156944595265983, -0.30735897575153542], [-0.064391632434560725, -0.23532802626875834, 0.42610091713159282, -0.28815837957332102, -0.49424881614705896], [0.010910346736070288, 0.61860684410071509, -0.2948450155605607, -0.10403596493744989, -0.3532622769126994]], [[-0.098548650060661186, 0.13666971453403123, 0.10901883622353085, -0.20681911085038249, -0.08732108047505191], [-0.27764566117785688, 0.064731021809417991, 0.12168505719766103, -0.17667767449412519, -0.088793626676505033], [-0.018602243703815295, -0.067984443466907643, 0.12309725352834454, -0.083246723206886156, -0.14278465354382056], [0.0031519146386568826, 0.17871072429331988, -0.085178440535527325, -0.030055184199512499, -0.102054734723073]], [[-0.12854661489118116, 0.17827163690907599, 0.14220389977219083, -0.26977433560232222, -0.11390140095068722], [-0.36216031230938489, 0.084434984415575218, 0.15872568701824527, -0.2304579207232765, -0.1158221865662299], [-0.024264720582517545, -0.08867873956216038, 0.16056775241193022, -0.10858681942818438, -0.18624794819787185], [0.0041113496427998084, 0.23310982584253287, -0.11110654672399826, -0.039203907779559545, -0.13311994303517405]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_array_rank3(self):
      arg0=numpy.array(-0.409786441515)
      arg1=numpy.array([[[-0.24581316871434389, -0.33314609913795179], [-0.79698015548755907, -0.41895026934279977]], [[-0.95131583957856281, -0.020459344268369817], [-0.84875716041095783, 0.40273550306252726]], [[0.54396865218080359, -0.63003080302737158], [-0.61695250839228399, -0.46130757001642531]], [[0.21174308619500604, -0.7300390108976802], [0.83728873908140167, -0.61171328391097091]], [[0.53377663455002256, 0.27485489141149699], [0.14771677004966133, -0.8418463957002067]], [[0.76892157851423759, 0.10470902366721901], [0.11160616603087448, -0.73241482746681186]]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[0.1007309036848804, 0.13651875447021333], [0.32659166187500405, 0.17168014004557156]], [[0.38983633265737877, 0.0083839618834575125], [0.34780917647484783, -0.16503554867158765]], [[-0.22291097827266779, 0.25817808081717608], [0.25281877299758398, 0.18903758756078076]], [[-0.086769445807171969, 0.29916008844260283], [-0.34310957290841881, 0.2506718098410905]], [[-0.2187344276358959, -0.11263180788440072], [-0.06053232955068201, 0.34497723879588438]], [[-0.31509363746314295, -0.04290833820305831], [-0.045734693628880323, 0.30013366586015877]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_array_rank3(self):
      arg0=numpy.array([-0.49129863317869038, 0.54128675640520529])
      arg1=numpy.array([[[0.093841980947059422, 0.89307566094746704], [-0.78966131499119219, 0.20641795950763542]], [[-0.54236894611313802, -0.93859447771012605], [0.54563092327710838, -0.33381561458664288]], [[-0.87336998291594892, 0.440422743370255], [0.04740961922232878, 0.55570236388504091]], [[0.7408413781394021, 0.0022393702192720077], [-0.44183603347710343, 0.95376042666640082]], [[-0.29549285253968072, 0.33527228096563499], [-0.20900772183724192, -0.87406076379355291]], [[0.048859208273817734, -0.82158991022488981], [-0.26546800212877186, 0.55957256702419222]]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[[-0.046104436974070996, -0.43876685154864609], [0.38795952472925999, -0.10141286136963554]], [[0.2664651219039515, 0.46113018400805172], [-0.26806772682607022, 0.16400315518012215]], [[0.42908547886590187, -0.21637909183861539], [-0.023292281123452295, -0.27301581183088786]], [[-0.36397435648210558, -0.0011001995279094016], [0.217073439336395, -0.4685811940011273]], [[0.14517523456681744, -0.16471881338111832], [0.10268520806242887, 0.42942485856689466]], [[-0.024004462243119612, 0.40364599992689132], [0.13042406659854328, -0.27491723734327678]]], [[[0.050795421481472869, 0.4834100277386893], [-0.42743321185025152, 0.11173130776566899]], [[-0.29357712761649007, -0.50804876041955194], [0.29534279265504343, -0.18068997125701405]], [[-0.47274360519424358, 0.23839499820596746], [0.025662199011260219, 0.30079433007403888]], [[0.40100762658383915, 0.0012121414423801585], [-0.23915999342376301, 0.51625788773790071]], [[-0.15994636769212542, 0.18147844547646322], [-0.11313311181692208, -0.47311751573486854]], [[0.026446842367061171, -0.44471573760087441], [-0.14369431380165307, 0.30288921977785938]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(0.486339854454)
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[-0.49084347844775067, 0.49942444151472287], [-0.29698395872438788, 0.83921420942239666]], [[-0.37839827410279647, 0.37383080543226832], [-0.17244319503908656, 0.26101122354273798]], [[0.4738538426654797, -0.26162261310669099], [0.58392252414423051, -0.97096810104765696]], [[0.70466416226398265, -0.96918503456628025], [-0.9989524823032967, 0.69932771377714076]], [[0.7449338236721359, 0.90258755187316897], [0.48736045156462038, -0.83355029493585708]], [[0.98854092641895397, -0.85952093674841046], [0.6019169310344108, -0.13927335956251197]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.23871674586788863, 0.24289001019695355], [-0.1444351352611398, 0.40814331646607083]], [[-0.18403016155273291, 0.18180881950428585], [-0.083865998376862053, 0.12694016046859016]], [[0.23045400887431544, -0.12723750358013766], [0.28398479540461563, -0.47222048494282509]], [[0.34270626611429239, -0.47135330864979086], [-0.48583040484967327, 0.34011093853390134]], [[0.36229100738243847, 0.43896429860983194], [0.23702281108048828, -0.40538872911904827]], [[0.48076685027624416, -0.41801928727823795], [0.29273619263256884, -0.067734185418927409]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_Symbol_rank3(self):
      arg0=numpy.array([0.84078924197312488, 0.82053556099293945])
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[-0.16627127848334244, 0.095638884964547533], [0.79284093871174588, 0.14081558725965659]], [[0.41210333817235401, 0.33745202843857536], [-0.85166669143136464, -0.57899275275885254]], [[-0.23713850907702394, -0.53641718430044993], [0.82288012058380233, -0.15794404205779999]], [[0.82446350722896056, -0.17813496098985859], [-0.63119644382977236, 0.066569849044106677]], [[0.038454995745195619, 0.31627807111320605], [0.55094035937309749, 0.70276004505845435]], [[0.87919781557349719, 0.12828643084686009], [0.7798813646417786, -0.99158604320853461]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.13979910219791183, 0.080412145592496806], [0.66661213186470958, 0.1183962308700471]], [[0.34649205331652788, 0.28372603519316314], [-0.71607219190233629, -0.48681087770004855]], [[-0.19938350728950796, -0.45101379776933331], [0.69186875282040872, -0.13279765139594901]], [[0.6932000472775417, -0.14977395881957536], [-0.53070317954376645, 0.055971212916059804]], [[0.032332546722682765, 0.26592319966399458], [0.46322472712970764, 0.59087308557369689]], [[0.73922006490046799, 0.1078618509471692], [0.65571586140612725, -0.83371487762043406]]], [[[-0.13643149676734265, 0.078475106127124217], [0.65055418442401114, 0.11554419688865254]], [[0.33814544377431555, 0.27689138946305181], [-0.69882280643263539, -0.47508414319583137]], [[-0.1945805795785451, -0.44014937524622266], [0.67520240137316789, -0.12959870315538935]], [[0.67650162642232159, -0.14616607014826902], [-0.51791912813461061, 0.054622928430621366]], [[0.031553691506765187, 0.25951740451063932], [0.45206615685185619, 0.5766396078154622]], [[0.72141307282536649, 0.10526357850271027], [0.63992039304428094, -0.81363161023688402]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_constData_rank3(self):
      arg0=numpy.array(0.387355709924)
      arg1=Data(numpy.array([[[-0.10395844223183359, -0.034376748558501147], [0.12862393234867664, -0.91804162386511479]], [[0.80081261014870431, -0.74967915907766769], [0.3734576493144337, -0.18944617576320444]], [[-0.62224883551171062, -0.5896500922539134], [-0.36425230899400796, -0.41356142099185367]], [[0.55848582677774394, -0.40903018734005703], [0.43711358327212779, 0.4802731028544085]], [[0.21468754528887946, 0.58178173257469279], [0.45898096180989745, -0.24878056263588544]], [[0.42123126772180419, 0.61407672776484268], [0.55341950102533555, 0.22644467592489304]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.04026889619326831, -0.013316029842744909], [0.049823214628092742, -0.35560866495172894]], [[0.31019933711995984, -0.29039250287949242], [0.14466095287660874, -0.073383057905076002]], [[-0.2410316394288011, -0.22840433009155836], [-0.14109521174170145, -0.16019537782533158]], [[0.21633267391378774, -0.15844017859730997], [0.16931844236564411, 0.18603652871340198]], [[0.083160446517138942, 0.22535647604207926], [0.17778889630331099, -0.096366571455027653]], [[0.16316633675041714, 0.23786612683094052], [0.21437020370525914, 0.087714638201317044]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_constData_rank3(self):
      arg0=numpy.array([0.56605108848823593, -0.18851042340241464])
      arg1=Data(numpy.array([[[0.44970194057812041, 0.057775049092888997], [0.6868268383241618, -0.59156660892409763]], [[0.56945239876636711, -0.2197598623523993], [0.3022233880249896, -0.24972154753827924]], [[0.18469333360262574, 0.36337543839803699], [0.24891933100108354, 0.5292816133280307]], [[0.71451631627973322, -0.95542670983685118], [-0.2721393858590675, 0.048385636119469666]], [[-0.59563969298710884, -0.43356870481545284], [0.46921748373561711, -0.36221706760565597]], [[-0.027495688126619067, 0.58580762011459719], [0.53855251006612104, -0.7732493783904304]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.25455427295951705, 0.032703629426491083], [0.38877907943632545, -0.33485692289478003]], [[0.3223391501639391, -0.12439530929060053], [0.17107387775814786, -0.14135515380300973]], [[0.10454586252228719, 0.20568906243509877], [0.14090105825892682, 0.29960043334114139]], [[0.40445273857274766, -0.54082032907388355], [-0.1540447955860452, 0.027388741992621507]], [[-0.33716249656215164, -0.24542203729522175], [0.26560106740625722, -0.20503336538719852]], [[-0.015563964192805787, 0.33159704101057075], [0.30484823453099946, -0.43769865231075494]]], [[[-0.084773503223268987, -0.010891198966595796], [-0.12947401809662953, 0.11151647191901229]], [[-0.10734771279896853, 0.041427024698907154], [-0.056972258838703042, 0.047075114659147238]], [[-0.034816618517034395, -0.068500057746451987], [-0.046923888480060058, -0.099775101027580179]], [[-0.13469377330982613, 0.18010789360132076], [0.051301110852765903, -0.0091211967514763931]], [[0.11228429072028415, 0.081732220118797549], [-0.088452386526816792, 0.068281692777923259]], [[0.0051832238104897056, -0.11043084250016359], [-0.10152276169699764, 0.14576556771603397]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(-0.0528042862132)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.84057186546492013, 0.95475104878375916], [0.82290236437194797, -0.94329886724569456]], [[-0.54658057476605704, -0.1365510319173604], [-0.40460022411662555, -0.1163723229781406]], [[0.37751551926701143, 0.54195098813444509], [0.39841038225251446, -0.14876508383575748]], [[-0.16099764993361343, -0.94944586040930856], [0.80917016966538169, 0.028354118907784942]], [[-0.72728317469000103, -0.48372986468381352], [-0.37012289584931302, 0.11281353766308144]], [[-0.5644983995881232, 0.50017748063404488], [0.66734901930012702, -0.71438652176093376]]])+(1.-msk_arg1)*numpy.array([[[0.74423031805071438, 0.52445338133253472], [-0.60489290108393923, 0.60530980668442291]], [[-0.49209968010950811, -0.15600012165422306], [0.52935145057545641, 0.2918272989589823]], [[-0.56506933354856748, -0.53581802436899206], [0.80213522312628971, -0.32830786440359261]], [[0.47913514471837093, -0.69421694646042065], [-0.56527548892904078, 0.33046868249119821]], [[0.99709158711162793, 0.84770941063786132], [-0.202469867157685, 0.79247373048264391]], [[0.92657022957706991, -0.88882680191552499], [-0.027318369583105628, -0.53152810510410364]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.044385797366810691, -0.050414947642373205], [-0.043452771973852146, 0.049810223370671203]], [[0.028861797108546687, 0.0072104797720782153], [0.021364626036197258, 0.006144957449837893]], [[-0.019934437529316978, -0.02861733509100204], [-0.021037775854790015, 0.0078554340654006863]], [[0.0085013659867543053, 0.050134810957033538], [-0.042727653234230618, -0.0014972190101310502]], [[0.038403668914408086, 0.025543010224658234], [0.019544075326502097, -0.0059570383314900173]], [[0.029807935058769851, -0.026411514844819781], [-0.03523888861925209, 0.037722670361948735]]])+(1.-msk_ref)*numpy.array([[[-0.039298550722924083, -0.027693386453387144], [0.031940937877196263, -0.031962952279848106]], [[0.025984972353948643, 0.0082374750731305912], [-0.027952025503582674, -0.015409732219068238]], [[0.029838082819026012, 0.028293488316995598], [-0.042356177903685531, 0.017336062438026448]], [[-0.025300389316533296, 0.036657630334980845], [0.029848968706740923, -0.017450162894779127]], [[-0.052650709546660845, -0.04476269034498264], [0.010691276814952039, -0.041846009680883298]], [[-0.048926879599259482, 0.046933864842350362], [0.0014425270063455088, 0.028066962192300718]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank1_expandedData_rank3(self):
      arg0=numpy.array([0.20678794651781152, 0.2502637168038282])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.053314958879399743, 0.79556982667577225], [0.15870834897630259, 0.47094597084800416]], [[0.069161910646639901, -0.61982663962747275], [-0.62942126336988258, -0.62334021680981344]], [[0.45765903223210813, 0.41444013187142548], [0.25471122760688547, -0.7489874822821323]], [[-0.013047736550351852, -0.082011819940301489], [0.73058388881809155, 0.21765601080646824]], [[-0.39232638166275224, -0.47018056584844681], [-0.10867343405645791, 0.22042469754020755]], [[-0.53349238995623782, 0.96587951492430824], [-0.72626294178900785, 0.47479682180950289]]])+(1.-msk_arg1)*numpy.array([[[-0.68299337038412622, 0.84196499441255401], [-0.62088074728991893, -0.44309842738306671]], [[-0.26453631540983413, 0.09336790644058901], [-0.10107396285692016, -0.10155806106462872]], [[-0.031104927846619335, 0.60884094737193872], [0.090208804348483884, 0.8858826510976896]], [[-0.11796959941075391, 0.36030799184669116], [0.9595854250855973, 0.0077306962795962164]], [[-0.88641372669713081, -0.50131085906678696], [0.76027428371304806, 0.15483479187808658]], [[-0.59528068006432378, 0.92672986021064641], [0.69245958914554029, -0.69230260953398992]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.011024890865352634, 0.16451425076981416], [0.032818973580041828, 0.09738595023249591]], [[0.014301849479867032, -0.12817267800560067], [-0.13015673054690463, -0.12889924341606873]], [[0.094638371480606553, 0.085701223824263081], [0.052671211711858748, -0.15488158342866787]], [[-0.0026981146479526531, -0.016959055835643451], [0.15107594212769027, 0.045008639521928157]], [[-0.081128366828803714, -0.097227673704382969], [-0.022472356269573734, 0.045581170566149215]], [[-0.11031979580192995, 0.19973224147481758], [-0.1501824223645338, 0.098182259795170368]]], [[[0.013342799770401842, 0.19910226180085616], [0.039718941302608529, 0.11786068907820885]], [[0.017308716819682358, -0.15512011860719832], [-0.15752130480630805, -0.15599943949212802]], [[0.11453545043525039, 0.10371932779481163], [0.063744978532565014, -0.18744439115546785]], [[-0.0032653750449682141, -0.020524582880106161], [0.18283863945261036, 0.054471402249120934]], [[-0.098185058475117645, -0.11766913597815938], [-0.027197017524804881, 0.055164304081771989]], [[-0.13351378839700537, 0.24172459738963603], [-0.18175726318899943, 0.11882441735269111]]]])+(1.-msk_ref)*numpy.array([[[[-0.14123479654701251, 0.17410821223445269], [-0.1283906547645266, -0.091627413903815985]], [[-0.054702921442987697, 0.01930735764351655], [-0.020900877225600075, -0.021000982899885081]], [[-0.0064321241559871052, 0.12590096926300215], [0.018654093409050009, 0.18318985427624612]], [[-0.02439469121367863, 0.074507349747933632], [0.19843069956187193, 0.0015986148088105868]], [[-0.18329967430890029, -0.10366504311350089], [0.15721555791932124, 0.032017968661982241]], [[-0.12309686943222785, 0.19163656476969809], [0.14319229648597373, -0.14315983499445606]]], [[[-0.17092845942470511, 0.2107132889204002], [-0.1553839235087135, -0.11089145934681743]], [[-0.066203841524054896, 0.023366599296013896], [-0.025295145616664919, -0.025416297833424133]], [[-0.0077844348538098507, 0.15237079843166534], [0.022575990664680915, 0.22170428491573674]], [[-0.029523510418393975, 0.090172017233676349], [0.24014941507270302, 0.0019347127844132758]], [[-0.22183719386915671, -0.12545991886417421], [0.1902690680323956, 0.038749530505957135]], [[-0.14897715553440818, 0.23192685928940851], [0.17329751051601472, -0.1732582242149657]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_array_rank4(self):
      arg0=numpy.array(-0.730935877851)
      arg1=numpy.array([[[[-0.78191809977016868, -0.68664195999519873, -0.8091776474747967, 0.2261814966467186], [-0.66902295768628917, 0.73975108805058309, 0.86251664890996871, -0.8523231851989983], [0.9122388979308611, -0.30021501495916425, 0.46570071050475015, -0.12990023109379245]], [[0.40617071029300522, 0.036831899135205903, 0.8675573142267925, -0.97899632074763021], [-0.76466833566117609, 0.76346806464430306, 0.047664613252238253, -0.42508616770889129], [0.023624890751065886, -0.64092450309316806, 0.7730603866640835, -0.66041329891979572]]], [[[0.10268058043631068, -0.25518645921054062, 0.75164024165418319, 0.22733024998903351], [0.15083427638758629, 0.71619440821272562, -0.23384587894353004, 0.88967050676253456], [0.042075387601280712, 0.58956579004336107, -0.98905184748058761, 0.15987692907760476]], [[0.015268771894321409, -0.038912224301465237, 0.64817891334974553, 0.38179161125282923], [-0.20446335810878913, 0.23547727215007752, -0.5759362456810484, 0.81575606232508835], [0.64623346560637107, -0.31553003581827754, 0.53002340359386113, 0.49644058502816635]]], [[[0.99568301932228476, 0.22822554384281291, 0.41332850760217266, -0.48455525766166874], [0.41057334644739329, 0.3253517871792273, -0.97380219806987234, -0.72358766087548076], [0.25192451036393582, 0.41091345762120102, 0.72457229680370649, 0.84860644997230583]], [[0.53535627062493352, -0.38101327657706063, -0.80377189834711249, -0.22316933585178012], [0.68592233680235259, 0.23682060765030966, 0.43467238902828464, 0.20557286974186306], [-0.49884304184952444, -0.51082452482911989, -0.6428107713982214, 0.52471220548819542]]]])
      res=outer(arg0,arg1)
      ref=numpy.array([[[[0.57153199266316856, 0.50189124379848726, 0.59145697409447462, -0.16532417080514383], [0.48901288287896394, -0.54071061093555584, -0.63044436393219294, 0.62299359558627143], [-0.66678813966900963, 0.21943792550325641, -0.34039735764866835, 0.094948739447601321]], [[-0.29688474468542064, -0.026921756527314725, -0.63412876706049903, 0.71558353511866146], [0.55892352119143773, -0.5580462000420604, -0.034839775929957718, 0.31071073115665637], [-0.017268280260266566, 0.46847471430468179, -0.56505757235821896, 0.4827197743904787]]], [[[-0.075052920199474737, 0.18652493857876923, -0.54940081986170974, -0.16616383583784314], [-0.11025018422139511, -0.52349218847901424, 0.17092634280745009, -0.65029209285870193], [-0.030754410372267206, -0.43093478829631865, 0.72293348037847094, -0.11685978350347632]], [[-0.01116049318828395, 0.028442340828930216, -0.47377722303386527, -0.27906518652727086], [0.14944960414763064, -0.17211878663299818, 0.4209724653231412, -0.59626537352794118], [-0.4723552254797484, 0.23063222371922024, -0.38741312178750426, -0.36286623481847408]]], [[[-0.7277804417897632, -0.16681823823679007, -0.30211663554507717, 0.35417882262629546], [-0.30010278940778728, -0.23781129417227123, 0.71178696449952827, 0.52889618210424016], [-0.18414066313507077, -0.30035138886718138, -0.5296158878308016, -0.6202769004606089]], [[-0.3913111056323243, 0.27849627378777597, 0.58750571811038799, 0.16312247441026681], [-0.50136524538830218, -0.1731006787461089, -0.31771764425202204, -0.15026058600713754], [0.36462227670419278, 0.37337997248384136, 0.46985345548409868, -0.38353097653769841]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(0.52214955509)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=outer(arg0,arg1)
      s1=numpy.array([[[[-0.63356952377056008, 0.70861656867149492, -0.71179736004649685, -0.48904256497927845], [0.22159313397077818, -0.59103409207570023, 0.57273284734425256, 0.13169326304940521], [-0.25555408026064197, 0.40960211322711304, 0.27829292963600993, 0.33656739070816588]], [[-0.66126843081107523, -0.70125386645412546, 0.27311620511191048, -0.89495402288189552], [-0.60916259537984874, 0.36143517868145381, -0.85276663841872535, 0.43081844835940708], [-0.21434356273762534, -0.50044873749432739, 0.66471445539646878, -0.071095630217035133]]], [[[0.2818580997474065, -0.75623587954807059, 0.93975613079863374, 0.11439034521741265], [0.045424577040247938, -0.71290172745689895, -0.24760179948265559, 0.74747518271124602], [0.17385691844753604, 0.12566294845535753, 0.44444126981517407, -0.95254939659314708]], [[0.2056835014759657, -0.30850404770879458, 0.65766967371384255, 0.099524110082933825], [-0.47313122466214574, -0.49424767293869465, -0.21505357784214185, 0.79206194570994248], [0.22737291353174016, 0.00039044018773015665, 0.39407979039625873, -0.92243568668488995]]], [[[0.40550461861584242, -0.52455411109268568, 0.25310331679817311, -0.063416137458261135], [0.89947686223094903, -0.0031746229826121741, 0.37965781985945801, 0.1970716890739177], [0.92860507796657932, -0.7479862483669879, 0.41090968619146495, 0.91901702447818967]], [[-0.93956206276376664, -0.13367346718891016, 0.4178483469095664, -0.55325018441845653], [0.75086844923581419, -0.007614765310602678, -0.39102362434727933, -0.80502269729085829], [-0.18790167022429194, -0.54307117748189215, -0.54359404051528371, -0.44321438767876353]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.33081804495527417, 0.37000382606110388, -0.37166467486239468, -0.25535335772392009], [0.11570475631380318, -0.30860818822024921, 0.2990522014261337, 0.068763578709575029], [-0.1334374493094852, 0.21387356118539172, 0.14531052939408828, 0.1757385133160142]], [[-0.34528101694295377, -0.36615939437404549, 0.14260750498700714, -0.46729984487363635], [-0.31807397815495486, 0.18872321774233475, -0.44527172084578837, 0.22495166113535581], [-0.11191939591982039, -0.26130908562793076, 0.3470803571470456, -0.037122551686656055]]], [[[0.14717208138157356, -0.39486822804899224, 0.49069324558944782, 0.059728867861844212], [0.023718422691709221, -0.3722413198143919, -0.1292851694393102, 0.39029383409336738], [0.090779312616670008, 0.065614852627241316, 0.23206481129755277, -0.49737324363219892]], [[0.10739754878499412, -0.16108525125455914, 0.34340192752575732, 0.05196646980051528], [-0.24704525845644637, -0.2580712025291238, -0.11229012999075075, 0.41357479255603247], [0.11872266564007679, 0.00020386817031249147, 0.20576858722530042, -0.48164938340149821]]], [[[0.21173405619713379, -0.27389569572758771, 0.13215778425792668, -0.033112707959346628], [0.46966144342748739, -0.0016576279779488997, 0.19823816172599126, 0.10290089477074767], [0.48487072831440742, -0.39056068679813472, 0.21455630982697557, 0.47986433045126725]], [[-0.49059191305138478, -0.069797541420004591, 0.21817932843385154, -0.28887933764746415], [0.39206562669947187, -0.0039760463190446687, -0.20417281148254518, -0.42034224322763752], [-0.09811277350825022, -0.28356437370428073, -0.28383738640453904, -0.23142419533587832]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_constData_rank4(self):
      arg0=numpy.array(0.749217389397)
      arg1=Data(numpy.array([[[[-0.23764815269520856, -0.18455544660624201, -0.86854924762358765, -0.52190811115135682], [0.60231705734854768, -0.38772913109778928, 0.31513849762036106, -0.83855541759102703], [0.60463621224720465, 0.83322375980887253, 0.82300214150592166, 0.63478805277209704]], [[-0.92311897579065683, 0.50028758892228264, -0.40790848163492055, 0.61140861252818102], [0.2482285795009469, -0.68985582771863574, -0.57852022126144687, 0.088837349136333676], [-0.29717142897784332, 0.14598829284286263, -0.69389346371608074, 0.41237743422733653]]], [[[0.68613166005853499, -0.089223037792774296, -0.85328947134450228, -0.2366765680382874], [-0.082832060713332822, -0.13298655372469881, -0.096193670872633641, 0.9301320089812195], [0.8616956824865214, -0.45302600597414466, 0.5867348229104401, 0.49530471719458924]], [[-0.53236094581109183, 0.15096918538813942, 0.79579488271314314, 0.15431250414984166], [0.17451306913175046, 0.11950361241978413, -0.82875828286528663, 0.9324872081668687], [-0.433803042765466, 0.30160290882483398, 0.24618890795144455, 0.034820689896103874]]], [[[0.31785046073691126, 0.37491673301292194, 0.24333760131866189, -0.70222242846076033], [-0.53548006611759313, 0.4194970567389269, 0.60139635888872478, 0.18758102830458601], [0.082890811553601784, 0.57208279180229082, -0.69611056660004111, 0.64094977905094641]], [[-0.04569723508861423, -0.70575800156534818, 0.40407628063560219, -0.74110273778927382], [-0.75353144830274577, -0.86137928176577705, 0.06012369990444677, -0.083786106712061326], [-0.59306811754723965, -0.48642200425558246, -0.86997408610813132, -0.27730301946656444]]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.17805012855724847, -0.13827214990526759, -0.6507321998669976, -0.3910226325417735], [0.45126641329577116, -0.29049340739412999, 0.23610724248551976, -0.62826030083199469], [0.45300396447454949, 0.62426573010729247, 0.61660751592694618, 0.47559424771811448]], [[-0.69161678914441582, 0.37482416131991358, -0.30561212772328006, 0.45807796453301197], [0.18597716830734645, -0.51685198230344431, -0.43343740988669271, 0.066558486800845598], [-0.2226460022220616, 0.10937696764620802, -0.51987704940478408, 0.30896034471790745]]], [[[0.514061771131468, -0.06684745144914396, -0.63929931012040386, -0.17732220043701213], [-0.062059220285990778, -0.099635838606480592, -0.072069970967678337, 0.69687107556320149], [0.64559738968694469, -0.33941496152475487, 0.43959193228908511, 0.37109090717239257]], [[-0.3988540780373353, 0.11310873895584569, 0.59622336452158076, 0.11561361151040921], [0.13074822607049297, 0.089534184520623714, -0.62092011712920792, 0.69863563174858279], [-0.32501278321308014, 0.22596614398418796, 0.18444901091380161, 0.026088266380950405]]], [[[0.23813909241184159, 0.28089413594907464, 0.18231276240201874, -0.52611725462716996], [-0.40119097721058639, 0.31429448970953106, 0.45057660999928112, 0.14053896832670726], [0.062103237437161951, 0.42861437579287853, -0.5215381414395287, 0.48021072019493094]], [[-0.034237163175738057, -0.52876616747861038, 0.30274097609492739, -0.55524705848121436], [-0.56455886452568493, -0.64536033676494542, 0.045045721483279219, -0.062774008138522491], [-0.44433694676314811, -0.36443582417346981, -0.65179971363669942, -0.20776024431655701]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(-0.305801562699)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.076413284641204005, 0.38935210549404298, -0.87416483595564998, -0.17808529342420831], [-0.45136890149782793, -0.76051790558909405, 0.090218653691126249, -0.21900622726658181], [-0.021002797765071968, -0.74561261640475429, 0.0089512091020149676, -0.40272388162468853]], [[0.78254729019255809, 0.091319539172886266, -0.92162394941705261, 0.0035206880978480015], [-0.97199275524939011, -0.91028332618997254, 0.57471596823472937, -0.34679234179543195], [0.32401911995733079, -0.45404235298596274, 0.89010307011958845, -0.62562287427214014]]], [[[-0.89077603808072126, 0.35306075395242598, 0.41112703301475717, -0.78019271431447534], [-0.50972742129771409, 0.25265771402863657, -0.20898468571319029, -0.32322942473076854], [-0.67890323338268299, 0.27809696504865955, 0.50181490703872633, 0.2474557882082471]], [[-0.50803662282881779, 0.28745039939039341, 0.75142850079254853, 0.38461906474766083], [-0.79004617450693693, -0.80488457439045957, 0.28994104666897669, 0.84024531664112345], [0.09295599333181026, -0.77393733706441026, 0.082149420520204242, 0.65119514574683679]]], [[[-0.56328794490739398, -0.77708657567269745, -0.35569077221534151, 0.9655445481998306], [0.61573578226884251, 0.96366054275303337, 0.79633760080273963, 0.78496512183264811], [0.87001146332336354, -0.23162665449416164, 0.092499885172479646, -0.82048676789744923]], [[-0.27665579014160069, 0.78610284117250617, -0.46563337780422298, 0.59181244373631237], [0.97715866220645253, 0.36730209557650206, -0.57698674789558768, -0.73888924091888231], [0.12486787492493812, -0.48572943068852403, 0.37108312575493785, 0.17693016092234015]]]])+(1.-msk_arg1)*numpy.array([[[[0.24779411139647256, 0.030923197183163165, 0.26385000008947146, 0.26084330968092395], [-0.40869869895284539, 0.27562112292321661, -0.034057342292344694, -0.21741963202156644], [0.0085135581535666915, -0.79692161059190791, -0.53555664354946408, 0.36274753547938765]], [[-0.18566438430195764, -0.79711944059062279, -0.10551670269570645, 0.29702369829533692], [0.582560976676052, 0.93278036958236088, -0.79230495881995266, -0.23574276521562432], [-0.54366001095056271, -0.12021671908123022, -0.74952574526028304, 0.22443224791911232]]], [[[0.79516700008567032, 0.43491471551636662, 0.87961514755304537, -0.94994363819548955], [0.70962114713121704, 0.049025057757413171, -0.60055359926960605, 0.15148560426616231], [0.9200852427312125, 0.78218919602399173, -0.35575228880385246, 0.57639041676455971]], [[-0.91251891001791252, -0.60692661160038575, -0.25804212419225925, 0.75833778551217645], [-0.8441072639167031, -0.77616174870793708, 0.82337316848673225, 0.74305007885292218], [0.49959972075119308, -0.29782295278750781, -0.59591336226686775, -0.13699931234570717]]], [[[-0.1854359391396716, 0.98823418676405672, 0.7025739902289343, 0.90216845426126446], [-0.21513498062117953, 0.45844653154002812, 0.79638962266025293, -0.053347997254850288], [-0.45013562465321955, -0.33375799278898621, -0.07278996036103158, -0.59715058872691285]], [[0.32569967219899443, 0.81866841960423642, -0.13546178282279953, 0.70390122030123692], [0.01050667752783685, 0.4836265262041195, 0.37130052883088704, -0.30679879418041689], [0.85442902550949595, 0.17981560260205076, -0.95381997614648251, 0.84655270932562532]]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.023367301854272833, -0.1190644823003728, 0.26732097289208628, 0.054458761022900794], [0.138029315431939, 0.23256756399000567, -0.027589005283380732, 0.066972446539016614], [0.0064226883776180854, 0.22800950326494837, -0.0027372937314451046, 0.12315359233719049]], [[-0.23930418422705066, -0.027925657784056007, 0.28183404395291045, -0.0010766319220990319], [0.29723690348774096, 0.27836606364808447, -0.17574904119446835, 0.10604964005322114], [-0.099085553227430109, 0.13884686107481159, -0.27219490980608729, 0.19131645261289915]]], [[[0.27240070446024833, -0.10796653028647371, -0.12572328916387296, 0.2385841512440387], [0.15587544198356698, -0.077263123778010506, 0.063907843471332701, 0.098844063193091136], [0.20760966969008737, -0.085042486493835376, -0.15345578275828739, -0.07567236673308915]], [[0.15535839316956876, -0.087902781332143645, -0.22978800979921479, -0.11761711104380929], [0.24159735476888958, 0.24613496064122914, -0.088664425162067378, -0.25694833087969221], [-0.028426088023140866, 0.23667124710569462, -0.025121421169927098, -0.19913649319163421]]], [[[0.17225433380240421, 0.23763428919342208, 0.10877079398120119, -0.29526503169537649], [-0.18829296442773819, -0.29468889988561175, -0.24352128276175403, -0.24004356092093421], [-0.26605086505066022, 0.07083179290714435, -0.028286609435257626, 0.25090613579720472]], [[0.084601772955133644, -0.24039147727297611, 0.14239141457752291, -0.18097717011950412], [-0.29881664590797025, -0.11232155481005192, 0.17644344916330479, 0.225953484534754], [-0.038184791282996997, 0.14853681895363158, -0.1134777997472311, -0.054105519698704661]]]])+(1.-msk_ref)*numpy.array([[[[-0.075775826492745932, -0.009456362022272409, -0.080685742345592323, -0.079766291720105256], [0.12498070081298421, -0.084285370102877968, 0.010414788494386745, 0.066487263233719507], [-0.0026034593874927556, 0.24369987386791345, 0.16377405851146176, -0.11092876321494609]], [[0.056776458857157167, 0.24376037059066927, 0.03226717257523308, -0.090830311097463659], [-0.17814805703521466, -0.28524569467359262, 0.24228809454161065, 0.072090505998011312], [0.16625208092584495, 0.036762460557632756, 0.22920614418401311, -0.068631732133799569]]], [[[-0.24316331123317722, -0.13299759964586186, -0.26898768669576834, 0.29049424903651661], [-0.2170032557172539, -0.014991939273644384, 0.18365022914138368, -0.046324534511052534], [-0.28136350504384455, -0.23919467847070952, 0.10878960585009977, -0.17626109017155014]], [[0.27904970867621398, 0.18559910627123855, 0.078909684820260756, -0.23190087986361196], [0.25812932039162706, 0.23735147566237183, -0.25178880160798389, -0.22722587527712226], [-0.15277837532988947, 0.091074724370164006, 0.1822312374146508, 0.041894603804057927]]], [[[0.05670659996953898, -0.30220355862540099, -0.21484822412394811, -0.27588452313118017], [0.065788613265257867, -0.1401936657590519, -0.24353719112707614, 0.016313900927415532], [0.13765217744561678, 0.1020637157582809, 0.022259283627229495, 0.18260958319954562]], [[-0.099599468729128809, -0.2503500820476085, 0.04142442487325633, -0.21525409315411986], [-0.0032129584067909834, -0.14789374747609313, -0.11354428194759202, 0.093819550694657403], [-0.26128573121651361, -0.054987892273438095, 0.29167963923948115, -0.25887714141917151]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=-0.13947691156
      res=outer(arg0,arg1)
      s0=numpy.array(-0.22162336321)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.030911342229990056)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=-0.45033575378
      res=outer(arg0,arg1)
      s0=numpy.array([-0.44267195731964026, -0.60497311267532083])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.19935100957690399, 0.27244102271340437])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=-0.680487553446
      res=outer(arg0,arg1)
      s0=numpy.array([[0.39932278514188635, -0.54500945200125672, 0.89782984799698551, -0.97035528306165131, -0.75606329797636973], [0.75185922465721133, 0.93683842862929168, -0.24462593704433355, 0.1990535233524664, 0.41923729055376269], [0.053231265989874155, 0.55565189692394878, -0.79953334298706213, 0.62247129001417245, 0.64700772814381535], [0.56874479249578358, -0.47655857813739266, 0.50623865021162961, 0.33734149256001245, 0.66717526036068286]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.27173418509634184, 0.37087214859713963, -0.61096203667403082, 0.66031469254377329, 0.51449166389005863], [-0.51163084432259809, -0.63750689027189988, 0.16646490540867059, -0.13545344511087468, -0.28528575816215157], [-0.036223213960268986, -0.3781141999052634, 0.54407248846756096, -0.42358396523195896, -0.44028070598507252], [-0.38702375238046183, 0.32429218091029566, -0.34448910054218645, -0.22955668694789777, -0.45400446064236682]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-0.72183483493
      res=outer(arg0,arg1)
      s0=numpy.array([[[-0.16385350028424317, -0.34536923971536071], [0.13472883605258246, -0.38734810865141145]], [[-0.46099867794929517, -0.83091893257231586], [-0.42244279705699617, 0.21317079893174107]], [[0.48152910468130039, 0.17056053007672545], [-0.80176061420272982, -0.71988116581898809]], [[-0.23046047361734434, 0.5491603476096043], [0.44303996442250071, -0.58480077893604565]], [[0.9285604709322377, 0.19490947465360353], [-0.97220365753378868, -0.23256498438669082]], [[-0.037644332049410423, 0.40583503476846006], [0.9756752923755514, 0.7647728846118278]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.1182751643303671, 0.24929954813981112], [-0.097251967132316791, 0.27960135806881026]], [[0.33276490460044317, 0.59978623053348712], [0.30493392668097263, -0.15387410845877356]], [[-0.34758448179158108, -0.12311653207349363], [0.57873874060634278, 0.5196353024981113]], [[0.1663543979314481, -0.39640306886683901], [-0.31980167958627564, 0.42212957373019211]], [[-0.67026729425782527, -0.14069244846286233], [0.70177046665417175, 0.16787350711524757]], [[0.027172990210933459, -0.29294586533087175], [-0.70427641361711246, -0.5520397089226613]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=0.053669180045
      res=outer(arg0,arg1)
      s0=numpy.array([[[[0.89686499446541434, 0.96263187822646556, 0.9259583572609198, -0.89864742439127832], [0.85954636088937697, -0.078732275716914435, 0.37752716161919819, -0.78396673855359822], [-0.019061992951265205, 0.16486638031292911, -0.49946937172326611, 0.23686277432820235]], [[-0.19640772031026188, -0.60815063504169586, -0.81465441480796508, -0.16254098778630754], [0.7609345062214099, 0.097091806915676537, -0.31017255677785371, 0.0093272197007276958], [0.16881275013246366, 0.67787801558821958, -0.92757011792795097, 0.73370216626439877]]], [[[0.65783331490394237, -0.0089955993106864973, -0.64039184822739026, 0.48198194008476114], [-0.97595708718418495, 0.38697821605598826, -0.94118908369554877, 0.51502317676806619], [0.67828098224443756, -0.29997956947639093, 0.36860937320652609, -0.69470242420618988]], [[0.7205008278909848, 0.048951321680601456, 0.3562700393486613, 0.12259007684179957], [-0.80123945416681841, -0.29376382593730987, 0.56075103074310872, -0.75316131392791452], [0.56186805450528721, 0.54873959417865903, -0.11226664228972139, 0.063209298504477429]]], [[[0.59894505141413057, 0.045929030490961864, -0.095097350584195661, -0.94611160636672365], [0.5065004198015044, 0.046813448876324548, 0.15459076159399743, 0.048600963487507975], [-0.94157876904098159, 0.22113027321379697, -0.68435619992093999, 0.60829610267289103]], [[-0.38198061273703954, 0.89934409109388413, -0.49781425170980254, -0.56025326430750266], [0.27088315136633567, -0.073002232924548549, -0.68887464041086721, -0.76156793252495536], [-0.0057170250627616159, 0.98684528221805046, -0.28797012951961309, -0.15309102500176364]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.048134008864026419, 0.051663663589597168, 0.049695425790013042, -0.048229670416635222], [0.046131148399600516, -0.0042254966808040282, 0.020261573208820317, -0.042074852040728153], [-0.0010230415317180619, 0.0088482434483828018, -0.026806111637981325, 0.012712230881379597]], [[-0.010541041303560362, -0.03263894592653669, -0.043721834462786581, -0.0087234415381962412], [0.040838731016853556, 0.0052108376662522715, -0.016646706794730057, 0.00050058423343766914], [0.0090600418807515754, 0.036381157267154622, -0.049781927663441396, 0.039377193660653953]]], [[[0.035305374617181924, -0.00048278643901795336, -0.034369305401869096, 0.02586757552084969], [-0.05237881662828632, 0.020768803551003544, -0.050512846389249355, 0.027640871601315599], [0.036402784157179319, -0.016099657524051406, 0.019782962816897363, -0.037284109482423201]], [[0.038668688654656169, 0.0026271772967171392, 0.019120720886444193, 0.0065793089057534955], [-0.043001864524840228, -0.015766063664938889, 0.030095048029373839, -0.040421550160129514], [0.030154997778780744, 0.029450404077797237, -0.0060252586380951899, 0.0033923912219552422]]], [[[0.032144889801409539, 0.0024649734067119407, -0.0051037968303061249, -0.050777034144764262], [0.027183462223197379, 0.0025124394162710827, 0.008296759417282638, 0.0026083738597717627], [-0.050533760482214293, 0.011867880446512338, -0.036728836108472128, 0.032646753055026026]], [[-0.020500586278685371, 0.04826705994732873, -0.026717282703982655, -0.030068333312920938], [0.014538076621838116, -0.0039179699825149621, -0.036971337104648672, -0.040872726487183784], [-0.00030682804741515718, 0.052963177127923977, -0.015455120728771423, -0.0082162697840939616]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-0.305092568332)
      res=outer(arg0,arg1)
      s0=numpy.array(-0.669130797674)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.2041468336126448)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(0.163002133099)
      res=outer(arg0,arg1)
      s0=numpy.array([0.28519182757745032, 0.22359358779742511])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.046486876237472698, 0.036446231758196557])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-0.318192267251)
      res=outer(arg0,arg1)
      s0=numpy.array([[0.096208144208031365, 0.080103180248881189, 0.9999105158610111, 0.42213777519872386, -0.99735812647625588], [-0.54986587584089341, 0.79051809254813654, 0.053343304294660721, 0.64238108326346977, 0.48714458165341612], [0.48292525954601739, 0.66066482081242683, 0.61181434723672234, -0.29762861091220638, -0.42019823689805613], [0.093170315285040495, 0.54280441267986368, -0.11377693742869432, -0.14572799867332242, -0.58340368833008194]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.030612687533599317, -0.02548821253743588, -0.31816379409029227, -0.13432097578292695, 0.31735164352504869], [0.17496306971796879, -0.25153674417111216, -0.016973426936197314, -0.20440069332298824, -0.15500563891551583], [-0.15366308324789885, -0.21021843722751971, -0.19467459428416364, 0.094703122505027867, 0.13370382969361663], [-0.029646073861071095, -0.17271616674464879, 0.036202941681352387, 0.046369522299867669, 0.18563454231255466]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(0.621743700122)
      res=outer(arg0,arg1)
      s0=numpy.array([[[-0.94387832276191252, -0.31924198538292758], [0.41030987078239001, 0.32872075826381852]], [[0.34285130773490669, -0.054679621014722013], [0.11365734429599406, 0.99826759799256881]], [[0.46805344949717487, 0.71628075978882166], [0.064139280908565688, 0.98768746766416116]], [[0.40489664850047569, 0.12281805103356014], [0.0062495723241140499, 0.27896624486217569]], [[-0.80919091734824966, -0.023040434317922243], [0.37562122276914311, -0.71195543800268202]], [[-0.47966348723284646, -0.93883615831688627], [0.35965375258732135, -0.56441629024046036]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.58685040085896245, -0.19848669322628279], [0.25510757725683308, 0.20438006054986424]], [[0.21316564066277591, -0.033996709890963296], [0.070665737788634275, 0.62066659008782588]], [[0.29100928354525085, 0.44534304991731732], [0.039878193835257585, 0.61408846070966838]], [[0.2517419404056927, 0.076361349491381375], [0.0038856322209748725, 0.17344550528975594]], [[-0.50310935505723642, -0.014325244885243459], [0.23354012888884645, -0.44265380834578444]], [[-0.29822775136558366, -0.58371546688028808], [0.22361245489641249, -0.3509222727032506]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(0.575774222676)
      res=outer(arg0,arg1)
      s0=numpy.array([[[[0.47966315281125338, -0.7355413009795404, 0.35137750763722875, -0.94091515814024151], [0.59861050504397784, -0.64171979023533887, -0.12179496042060989, -0.52556968266104631], [0.41915793089990117, 0.32646904067421234, -0.74028690125000396, -0.63102125447483859]], [[0.74316309123261903, -0.60641010467784873, 0.4727462198477157, -0.58278230524956243], [-0.83935991499926144, 0.59890960883219102, -0.85967074166873059, -0.91946197687099218], [0.9376200568337334, 0.85861377860813204, 0.62817808703185096, -0.68722706974547876]]], [[[0.31447838718483845, 0.5089932963892092, 0.48685133305720707, 0.023359923575308361], [-0.31578236406869675, 0.42666165768720732, 0.35612584003046011, 0.18952129448940935], [-0.73817669150901066, -0.47126611650544281, 0.36413087852349579, 0.93039020613275758]], [[0.90114068575953898, 0.35961879552519016, -0.72431381975849307, 0.40062145384926318], [-0.99282871155349905, -0.12974369327622037, -0.43923130368689334, -0.47082027733924092], [-0.040772962143140079, -0.78183904488078926, -0.17278937279458639, 0.59510609858595731]]], [[[0.62917657952611239, 0.72165905153892695, 0.59424289918993001, -0.81513110907529751], [0.12816403169968549, -0.25636245562979521, -0.71223068471443018, -0.066020871203465115], [-0.10546207627241366, 0.56748622877135579, 0.70377775576841728, -0.40905184765879121]], [[-0.71083761658548372, -0.69188209970282033, 0.82966246172361369, -0.38857731368290716], [-0.031626311946679309, -0.99454533862063887, -0.99847344695912321, 0.28546074895671203], [0.43785675647944955, 0.11588840708523529, -0.67220561796405187, -0.17786509414992691]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.27617767895616135, -0.42350572081750049, 0.20231411132561353, -0.54175469378215046], [0.34466449822731238, -0.36948571339848113, -0.070126398662016254, -0.30260947549617295], [0.2413403318423209, 0.18797245812193492, -0.42623811512435711, -0.36332577228720897]], [[0.42789415117586543, -0.34915530664368749, 0.27219508725577929, -0.33555102879432436], [-0.48328180260399262, 0.34483671447847025, -0.49497625304151077, -0.52940250501292363], [0.5398574593887574, 0.49436768095689748, 0.36168874976278542, -0.39568763188452594]]], [[[0.18106854892971483, 0.29306521957573084, 0.28031644784972948, 0.013450041838341139], [-0.18181934520640566, 0.24566078430045427, 0.20504807871833303, 0.10912147601516625], [-0.42502311075105365, -0.27134288190440214, 0.20965717353415114, 0.53569469772134048]], [[0.51885357786480812, 0.20705923245315266, -0.41704122654484393, 0.23066750617734053], [-0.5716451796450116, -0.074703074143213602, -0.2528980624552345, -0.27108617920504374], [-0.023476020584159568, -0.45016276832388902, -0.099487666807455855, 0.34264675132300532]]], [[[0.36226365600251648, 0.41551267943683862, 0.34214974336174336, -0.46933148070675751], [0.073793545726893284, -0.14760689361352516, -0.41008406885736093, -0.038013115797559525], [-0.060722344987533365, 0.32674394225009412, 0.40521709026413594, -0.23552150961987306]], [[-0.40928197613828227, -0.39836787813974722, 0.47769825898227086, -0.22373280073525753], [-0.01820961517720418, -0.57263356926021836, -0.57489527278539587, 0.16436094083502545], [0.25210663360533725, 0.066725557506647312, -0.38703866716161167, -0.10241013632534642]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s0=numpy.array(0.326668222206)
      s1=numpy.array(0.602549535775)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(0.19683378564248266)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s0=numpy.array([0.25980579781038293, -0.13860900374942564])
      s1=numpy.array(0.054937744255)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.014273144476083001, -0.007614865999431588])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s0=numpy.array([[0.28753403164778435, -0.20580170444046186, -0.93428998829439314, -0.34563902987880457, 0.87384606079835847], [-0.44466335201064133, 0.77788321424900397, -0.71494268653741089, -0.91050314269119315, -0.32970230695513347], [-0.58072770093865045, -0.83547384753061604, 0.66031217772555784, -0.93680929926433576, 0.074388629508242321], [-0.28808608194738561, -0.90027352223351076, 0.38853521256382018, 0.14333986352807959, 0.8759160670965116]])
      s1=numpy.array(-0.635440634973)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.18271080764666137, 0.13077476574820612, 0.59368582341085729, 0.21963308461769532, -0.55527729574250528], [0.28255716275093662, -0.49429860359735134, 0.45430363470275076, 0.5785706951367513, 0.20950624328368639], [0.36901797903096062, 0.53089403217832543, -0.41958918949443991, 0.59528669597329253, -0.047269557969500803], [0.18306160283957712, 0.57207037861758725, -0.24689106218098639, -0.091083973897249287, -0.55659266185900214]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.9456243319028439, -0.39923472671213589], [-0.1949828088219463, -0.65891132614891701]], [[0.94925925762283025, -0.13075371695236693], [0.63120563016303088, 0.12895084440439386]], [[-0.86453779533546848, 0.72881194778835212], [-0.071249841409085191, -0.86156502188544848]], [[0.88386908158874578, -0.64056630144507753], [0.12463872622937577, -0.39045654192688151]], [[0.93910139509931678, -0.15033327790579509], [-0.38589914763558331, -0.83727058233064455]], [[0.077126427944703035, 0.15706960057620512], [-0.10307143099944338, 0.75719806142911095]]])
      s1=numpy.array(-0.904971642853)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.85576320516429005, 0.3612961065168202], [0.17645391282777223, 0.59629606531971502]], [[-0.85905270986475624, 0.11832840603957535], [-0.57122319610697014, -0.11669685750798083]], [[0.78238218895361789, -0.65955414572123039], [0.064479086033025892, 0.77969191328072229]], [[-0.79987645483271608, 0.5796943381752947], [-0.1127945128389566, 0.35335209821043756]], [[-0.84986013232897306, 0.13604735348194805], [0.34922778561151063, 0.7577061344046081]], [[-0.069797230204534294, -0.14214353447577976], [0.093276722242819812, -0.68524277361693198]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s0=numpy.array([[[[-0.45824045869862395, -0.1938286951652195, 0.43331126090350924, -0.23664365739996041], [-0.11403720838431974, 0.3562997310797591, -0.26378841792641805, 0.45086130060842389], [0.54147676113656873, 0.14952563271813557, -0.10452476680275713, -0.94901206613061695]], [[0.13588747220883812, 0.69984991042098255, 0.59536203349545547, 0.68704445571770933], [-0.83080884292616997, -0.2193352627257239, -0.20864429528932726, -0.90433608282716782], [-0.094343633381194891, 0.88913110402908635, -0.99538141218114884, -0.43704792980458107]]], [[[0.48115993450643435, -0.34226732199370358, -0.05899436381065426, -0.31949481147781378], [-0.91150760454508539, -0.47535421553499257, 0.56116889396452341, -0.43032740169297123], [-0.29384042951659817, 0.52661650389539116, -0.52617909826511933, 0.98480155135112035]], [[0.7089648167925009, -0.98369711528557979, -0.89084164801020638, -0.3658215877606974], [-0.12162027721428648, -0.40537371282140788, -0.56174932859496529, 0.25238222994021942], [0.18591672242181345, 0.2189727908279635, 0.30056660818705505, 0.31683884450830635]]], [[[0.57506285298971571, -0.60014390580869548, 0.57152131606400625, -0.69870314600448324], [-0.71064751134920257, -0.1672632144008992, 0.57831463084427326, -0.7231944258109928], [0.91201983783418061, -0.93653159589793389, 0.55138918006432758, -0.72284409845053221]], [[0.094534560703354087, -0.62201714267453623, 0.74276787544178569, -0.38140061509096257], [-0.79351650029747622, 0.56589065437995711, -0.9205640541943958, -0.086281210036292633], [0.93782803208939391, -0.34119904466842343, -0.047208474113730858, 0.46549811331101498]]]])
      s1=numpy.array(-0.550595161634)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.25230497942453606, 0.10672114174385586, -0.23857908373512188, 0.13029485279585867], [0.062788335182686741, -0.19617690802411089, 0.14524062660544748, -0.2482420506831374], [-0.29813448481919613, -0.08232808991491003, 0.057550830872549613, 0.52252145194407018]], [[-0.074818984724899132, -0.38533397454797813, -0.32780345506334468, -0.37828335314582895], [0.45743932915812474, 0.12076493443256749, 0.11487853948889785, 0.49792307169593719], [0.051945148070684372, -0.48955128393696173, 0.54805218952763879, 0.24063647555268131]]], [[[-0.26492433191151038, 0.18845073147525579, 0.032482011277838771, 0.17591229736694108], [0.50187167685537781, 0.26172773113602621, -0.30897687787652761, 0.23693618529080177], [0.16178711878437799, -0.28995249908156301, 0.28971166565786372, -0.54222696934386105]], [[-0.39035259789488563, 0.54161887218983995, 0.49049310117672906, 0.20141959624241057], [0.066963536190806153, 0.22319680493319177, 0.30929646237569386, -0.13896043468755545], [-0.10236484783235418, -0.12056535915943223, -0.16549052021661925, -0.17444993480406984]]], [[[-0.31662682449174345, 0.33043633082256751, -0.31467687139569867, 0.38470257160871923], [0.3912790813763139, 0.092094316568532536, -0.31841723764517232, 0.39818735177241477], [-0.50215371002598164, 0.51564976541903429, -0.30359221472091225, 0.39799446322275711]], [[-0.052050271730489012, 0.34247962921018005, -0.40896439843562166, 0.20999733331342282], [0.4369063457407581, -0.31157665631566167, 0.50685811421387261, 0.047506016785933913], [-0.51636357691341972, 0.18786254314867296, 0.025992757435157422, -0.25630100893893126]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(0.356750512388,self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array(0.14382921906)
      sub=res.substitute({arg0:s0})
      ref=Data(0.051311147595916123,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(0.6971073044,self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([0.84680734255015477, -0.85520146706647671])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.59031558391133865, -0.59616718942571045]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-0.672497339075,self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[0.436919678441541, 0.44096657176684584, -0.055481017880169903, -0.91014875323883104, 0.32338875711937654], [-0.26206266996339944, 0.42430477393906907, -0.98863970352503006, 0.93956736011842668, 0.56436529075499542], [0.51931312337515045, -0.53394040311921609, -0.086854052873552634, 0.16309248205384397, 0.8021988412561416], [-0.14402098737812752, -0.30831107162070293, 0.75820109462043961, -0.40582651249559887, 0.4771531254887309]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.2938273211414491, -0.29654884613423704, 0.037310836893587787, 0.61207261471555963, -0.2174780786495582], [0.17623644822128093, -0.28534383143085124, 0.66485756992449807, -0.63185654956138171, -0.37953415629903364], [-0.34923669361652554, 0.35907350032231561, 0.058409119445345124, -0.1096792602043503, -0.53947658615381855], [0.096853730782747602, 0.20733837527229021, -0.50988821861601208, 0.27291724977938503, -0.3208842072225]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(0.211945705953,self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[[-0.32351913444774905, -0.068339586770982397], [-0.23418439133724855, 0.84740298257866287]], [[-0.039379381340717057, -0.10584058987718614], [0.98039398389507126, -0.42263482488705528]], [[0.84610161745178813, 0.19843158914502146], [-0.61853641208671895, -0.029702079301567874]], [[0.51530962812238124, -0.18777173267520042], [0.18720536210868577, 0.6246059425116044]], [[0.44051507275515478, -0.025698251973472308], [0.98551660552646614, 0.46123484367008638]], [[0.62721023729420766, 0.93411985731927483], [0.44541588437270296, 0.012248862905623659]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.068568491339735041, -0.014484281962691746], [-0.049634376145076799, 0.1796034233690593]], [[-0.0083462907782389066, -0.022432458539970539], [0.20779029502842208, -0.089575636320883195]], [[0.17932760461854161, 0.042056723244657937], [-0.13109613651717056, -0.0062952281658339186]], [[0.10921766291662204, -0.03979741243980725], [0.039677372630256474, 0.13238254742787431]], [[0.093365278077896824, -0.0054466341562679861], [0.20887601268641665, 0.097756744551640254]], [[0.13293451652408211, 0.19798269260397025], [0.094403984055919279, 0.0025960938956502617]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(0.93558395951,self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[[[-0.20958818867387619, 0.60755452193036708, 0.42247591455422673, -0.80817866673466732], [-0.94247939314966578, 0.6783376834484347, 0.19420590433524088, 0.2495604302785428], [0.71890576537734763, 0.3154734148970495, 0.90459498187278231, -0.74249160793171498]], [[0.19653287145178977, -0.29904727710365808, 0.16041371867692966, 0.49270131383450688], [-0.13629553737064493, -0.30914726068581477, -0.77864738780527532, -0.19063340201388068], [0.66084701763685261, 0.74335940946787926, 0.23960186763475644, 0.41675654324592548]]], [[[0.0967011854608264, -0.62619413503873433, -0.52434139717478789, -0.56408454603172054], [0.91861711661226875, 0.37702596613619321, 0.97505968654814312, 0.22668801073418199], [-0.6963072463397546, -0.00086110947464534959, 0.057093779244521192, 0.45833438114390468]], [[-0.89235178475029953, 0.5410762848873496, -0.42647524632918654, -0.69022699230920725], [0.67443017622120816, 0.77031495256001681, -0.81496784990627158, 0.44226154962963893], [-0.42926967611154265, 0.63759382190223102, 0.085043921615083784, 0.5776606672737592]]], [[[0.57146277721118977, -0.99444796182308903, -0.90683295897766514, 0.03248602670140599], [-0.097941549250353921, 0.44255470278439946, -0.46041561064135883, 0.3813771608181693], [-0.31586331387305044, 0.40098342982956159, -0.76003258102418125, 0.24958748090325922]], [[-0.69970819452092159, -0.66997964894631545, 0.37563238406441823, -0.65960947421679705], [0.88987897733278265, -0.80624063465635909, -0.5856842461519578, 0.44528704824075049], [-0.1766735818038927, 0.086426238236088304, 0.79521466040340694, 0.76264190695457246]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.19608734742597528, 0.56841826524564765, 0.39526168893613345, -0.75611899701490626], [-0.88176860239928212, 0.63464185576533738, 0.1816959289381305, 0.23348473549694843], [0.67259670248610448, 0.29515186662943416, 0.84632455489316083, -0.69466323845149236]], [[0.18387300204668022, -0.27978383559324077, 0.15008050207944013, 0.46096344605292899], [-0.12751591851673297, -0.2892332182240181, -0.72849000614475978, -0.17835355307095466], [0.61827786939087614, 0.69547513964876551, 0.2241676640276492, 0.38991073688160671]]], [[[0.090472077982723706, -0.58585718828130318, -0.49056540050364661, -0.52774845307459983], [0.8594434392335083, 0.35273944623567716, 0.91225020229901799, 0.21208566665606784], [-0.65145389056585745, -0.00080564021186003076, 0.053416024048962993, 0.42881029509005136]], [[-0.83487001605225042, 0.50622229301171562, -0.39900339959354342, -0.64576530242513297], [0.63098605468187596, 0.72069431338564249, -0.76247084788843256, 0.41377281174140201], [-0.40161782327389201, 0.59652255245422436, 0.079565728916874329, 0.54045005434101046]]], [[[0.53465140781566578, -0.93038956164881625, -0.84841837037423917, 0.030393405490039896], [-0.091632542448162338, 0.41404708113067562, -0.43075746002392795, 0.35681035418483797], [-0.29551664985720993, 0.37515366497772912, -0.71107429151099533, 0.23351004362752781]], [[-0.65463576313128102, -0.6268222127521258, 0.35143563320306415, -0.61712004361787542], [0.83255649709746493, -0.75430580528942581, -0.54795678603731401, 0.41660341971147691], [-0.16529296920485032, 0.080859002174449829, 0.74399008064039662, 0.71351553499660214]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.943421221943)+(1.-msk_arg1)*(0.690295173994)
      res=outer(arg0,arg1)
      s0=numpy.array(-0.216259198802)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.20402351758993897)+(1.-msk_ref)*numpy.array(-0.14928268126468969)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.855064098466)+(1.-msk_arg1)*(0.875262020946)
      res=outer(arg0,arg1)
      s0=numpy.array([-0.87981858530793877, 0.61433340922003832])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.75230128545981712, -0.52529444271217363])+(1.-msk_ref)*numpy.array([-0.77007179304256668, 0.53770270128863928])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.229690330921)+(1.-msk_arg1)*(0.758485887141)
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.99443754833503362, 0.69199034408265669, 0.21249303888968685, 0.84753977889196563, -0.016736103979785089], [0.64466239296559658, -0.99537612926113717, -0.38556568478745379, -0.57210458700208133, 0.46520440333256574], [0.70470505071356238, -0.82855350585586396, -0.61625605179875498, 0.53231517113400484, -0.9327217699594208], [0.55231803359045806, 0.36204990285924343, 0.3572594963431388, 0.19876578225280461, 0.58807303395854782]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.22841268955750973, 0.15894349112659892, 0.048807596421016973, 0.19467169228254988, -0.0038441212614479282], [0.14807271837270047, -0.22862827252102275, -0.088560709730677339, -0.13140689191002669, 0.10685295334744194], [0.16186393630021723, -0.19031072894592801, -0.14154805646982901, 0.12226764781212821, -0.21423717199935771], [0.12686211190912158, 0.08315936199771666, 0.082059051939785674, 0.045654578301451684, 0.13507468977575462]])+(1.-msk_ref)*numpy.array([[-0.75426684605519456, 0.52486491002452274, 0.16117297111352591, 0.64284696108013872, -0.012694098674390902], [0.48896732703493462, -0.7549787464415838, -0.29244613047712958, -0.433933255209695, 0.35285097456358927], [0.53450883556320239, -0.62844614093285034, -0.46742151815457356, 0.40375354481617581, -0.70745629914337205], [0.41892543369181762, 0.27460974175949721, 0.27097628602336371, 0.15076104068528845, 0.44604509686573413]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0671093667629)+(1.-msk_arg1)*(0.470013039064)
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.16193013785241295, -0.61822118838681583], [-0.21561189549901205, -0.4050663451890697]], [[-0.64237686486527101, 0.91101114878986222], [0.14237722466477876, 0.86204930944628777]], [[-0.95720051416683805, -0.057435718640591382], [-0.38539843358753534, -0.72003408746831599]], [[-0.85471409970470225, 0.019098731514053835], [0.85156900609585917, -0.87795187611535797]], [[-0.75427706517899407, -0.21560990278350389], [-0.22260564889936241, 0.25405754804882696]], [[-0.064291257188151762, -0.95510218651483436], [0.71361279186126336, -0.8297259704278741]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.01086702901110481, 0.041488432472047761], [0.014469577773487632, 0.027183745922601418]], [[0.043109504624246411, -0.061137381309231263], [-0.0095548453887126898, -0.057851583275337057]], [[0.064237120370860407, 0.0038544747075422762], [0.025863844829473719, 0.048321031657702455]], [[0.057359321994506184, -0.0012817037778828275], [-0.057148256754006678, 0.058918794454403185]], [[0.050619056207942215, 0.014469444043611741], [0.014938924135481035, -0.017049641170892249]], [[0.0043145455582877159, 0.064096302930873358], [-0.04789010257571575, 0.055682384462148724]]])+(1.-msk_ref)*numpy.array([[[0.076109276208110102, -0.29057201956761686], [-0.10134040226190019, -0.19038646392497455]], [[-0.30192550247990907, 0.42818711866416231], [0.066919152058230152, 0.40517441575611196]], [[-0.44989672265744507, -0.026995536669105161], [-0.18114228902108981, -0.33842540968085738]], [[-0.40172677153329545, 0.0089766528411931428], [0.40024853652806147, -0.41264882944516396]], [[-0.35452005570126355, -0.1013394656596282], [-0.10462755755206499, 0.11941036025564801]], [[-0.03021772917626632, -0.44891048130077421], [0.33540731701785648, -0.38998202495136242]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.251610493849)+(1.-msk_arg1)*(-0.851978643832)
      res=outer(arg0,arg1)
      s0=numpy.array([[[[-0.240729135600114, -0.67547568254644208, -0.016732264647327133, 0.19576162584202295], [-0.5393251893654083, 0.31348531267591406, 0.21174769680895444, 0.84376347503208415], [0.39315758446894478, -0.40518657237469857, -0.59523718069355969, 0.76818182191391382]], [[-0.099574666208944285, 0.33951207207487877, 0.52119187919444498, -0.44768839895818702], [0.059484020247627667, -0.98791453218980174, 0.71879067840546607, 0.60737879885040313], [0.37894060071559621, 0.51552562402196389, -0.76546869384111327, 0.22339674986284819]]], [[[-0.86450090423662451, 0.71628561124951329, 0.26032290994672969, -0.0096185290731636286], [-0.2966479053686033, -0.24649420538862987, 0.64695956874968874, 0.38208409309881186], [-0.98785128579288206, 0.19218445087480474, -0.42491094617314706, -0.29666387999137589]], [[-0.52751272457757881, -0.44092526926623687, -0.41467747816209877, 0.088837557649594556], [0.94468183480934109, -0.99610181682479437, -0.8718474073220055, 0.30534621920924288], [-0.031391941678135193, -0.64741120154693133, -0.26201001793464895, 0.42968046485439104]]], [[[0.49272894643356691, -0.11316009681601136, -0.039099533076986992, 0.82471483658195055], [-0.43859758298422435, 0.31359967555003854, -0.39450170629503734, 0.1440295002408909], [-0.270940173818917, 0.86505449919297028, -0.79628028928319061, -0.79654036164025732]], [[0.5503371976992939, -0.47856288342352693, -0.12542696824916222, 0.25389863351890729], [0.49055128865111675, -0.39995446959807457, -0.96476714572480571, 0.85234622098318202], [0.97489220391842735, -0.34417928364914241, 0.053129162979288269, -0.65457043849450636]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.060569976692092374, 0.16995677006823354, 0.0042100133711195269, -0.049255679354717141], [0.13569987724122257, -0.078876194336670952, -0.05327794256540562, -0.21229974464423743], [-0.098922573988555654, 0.10194919357602127, 0.14976792099135763, -0.19328260757728066]], [[0.025054030939642428, -0.085424800122323272, -0.13113734611399655, 0.11264309915216054], [-0.01496680371060599, 0.24856966332448918, -0.18085527756737282, -0.1528228795319222], [-0.095345431685338045, -0.12971165685177638, 0.19259995608300881, -0.056208966557164418]]], [[[0.21751749944754228, -0.18022497638313961, -0.06549997593180247, 0.0024201228501958614], [0.074639725968948398, 0.062020528748652527, -0.16278181659318941, -0.096136367356288249], [0.24855374986732606, -0.048355624594632503, 0.10691205300830336, 0.074643745351673244]], [[0.13272773714238753, 0.1109414247504068, 0.10433720506825961, -0.022352461752518364], [-0.23769186298618419, 0.25062967005477876, 0.21936595671691531, -0.076828313010041865], [0.0078985419485021885, 0.16289545214434187, 0.065924470005818761, -0.10811211395911131]]], [[[-0.1239757735456524, 0.028472267843832524, 0.0098378528267505459, -0.20750690731665578], [0.11035575445546501, -0.078904969235907355, 0.099260769145011502, -0.036239333684378268], [0.068171390938004464, -0.21765678974790056, 0.20035247682845331, 0.20041791376265122]], [[-0.13847061409637645, 0.12041144343580579, 0.031558741423104948, -0.06388356056717813], [-0.12342785199557685, 0.10063274161252825, 0.24274553798472701, -0.21445925359157028], [-0.24529310887707001, 0.086599119531419674, -0.013367854934981721, 0.16469679128830039]]]])+(1.-msk_ref)*numpy.array([[[[0.20509608247947533, 0.57549085595752614, 0.01425553214247071, -0.16678472449926712], [0.4594935434200681, -0.26708279155492859, -0.18040451556187823, -0.71886846117295289], [-0.33496186562818286, 0.34521030643080053, 0.50712936596578251, -0.65447450685074071]], [[0.084835489076737214, -0.28925703473100467, -0.44404435041242268, 0.38142095500379103], [-0.05067911490025908, 0.84168208335715855, -0.61239430738709344, -0.51747376533697798], [-0.32284929909062093, -0.43921682201496526, 0.65216297967473313, -0.1903292599846636]]], [[[0.7365363079832028, -0.61026004366885622, -0.22178955977485851, 0.0081947813554142344], [0.25273768011159614, 0.21000779881949291, -0.55119573599760441, -0.32552748746816978], [0.84162819877768369, -0.16373704782194617, 0.36201505167004144, 0.25275129014904163]], [[0.44942957568981784, 0.37565891294078224, 0.3532963554722886, -0.075687701887663661], [-0.8048487484737471, 0.8486574750171475, 0.74279537171879451, -0.26014845774117079], [0.026745263898196151, 0.55158051749570935, 0.22322693975040439, -0.36607857972781987]]], [[[-0.41979453955932355, 0.09640998582122226, 0.033311967165402398, -0.70263942801935875], [0.3736757739389665, -0.26718022628132992, 0.33610702871872239, -0.12271005828705922], [0.23083524184989293, -0.7370079590633426, 0.67841380097377968, 0.6786353770678516]], [[-0.46887553934624049, 0.40772535640758867, 0.1068610983089017, -0.21631621345627941], [-0.41793922163510111, 0.34075266660278186, 0.8219610044284521, -0.72618077742872533], [-0.83058733777697569, 0.2932333993185236, -0.045264912223032278, 0.55768003448117742]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([0.25431329102082678, 0.52118225487969716])
      res=outer(arg0,arg1)
      s0=numpy.array(-0.77887569151)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.19807844040407516, -0.40593618917225704])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.72368546229391817, -0.11140163975108863])
      res=outer(arg0,arg1)
      s0=numpy.array([0.42996412295349207, 0.49846252449359763])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.31115878508939698, -0.047898708331157704], [0.36073008247434268, -0.055529542583053954]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([-0.43944644080185924, -0.68087771948603404])
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.24891746969874662, 0.93385694344000925, 0.22610401509353961, -0.77949548446521399, 0.77583568447590312], [-0.30002373676409277, 0.97600877806225861, -0.38932454480308398, -0.1745750122704437, -0.1612615099318484], [0.62377572788345437, 0.48829718400739619, 0.10962910960154559, 0.084927381218901532, -0.74268686230144554], [-0.78610208254502978, 0.76846230068610777, 0.61131324015860811, -0.50588653808467954, -0.62980206296112606]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.10938589611251885, 0.16948235910871659], [-0.41038011001281521, -0.63584238597563181], [-0.099360604683865844, -0.15394918616352507], [0.34254651626935922, 0.53074110781233619], [-0.34093823019000991, -0.52824923154183923]], [[0.13184436327705448, 0.20427947767961369], [-0.42890358371083132, -0.66454263100538136], [0.17108728553051925, 0.26508240820546208], [0.076716367795187382, 0.11886423623394611], [0.070865796577884457, 0.10979936912327137]], [[-0.27411602347697311, -0.42471499507202737], [-0.2145804595656208, -0.33247067307840827], [-0.048176122022676141, -0.074644018134784842], [-0.037321035403268941, -0.057825161646246716], [0.32637109826867078, 0.50567893709604639]], [[0.34544976228134272, 0.53523939324648195], [-0.3376980229269182, -0.52322885880214798], [-0.26863942760275256, -0.41622956485081131], [0.22231003861088663, 0.34444687236978133], [0.27676427497793532, 0.42881819235657115]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_array_rank1(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([-0.15149163105143937, -0.64565125044028027])
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.19406669211056204, 0.97684763423987886], [0.50264965562821495, -0.53165078443596836]], [[-0.89340948961014433, 0.24043534232003205], [0.0017607854038819148, 0.94674226842652942]], [[-0.84197751920023545, -0.85155643134596604], [-0.006618770156334941, -0.83961100875268402]], [[-0.17034729331881659, -0.14209414855377323], [-0.32763085170526196, 0.20943875210899798]], [[-0.91189198824902662, 0.70548838331044594], [-0.73522099550236386, -0.2033476102939078]], [[-0.44365771274855592, 0.32734237831353186], [-0.38643433643962966, 0.055662231228680747]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.029399479720586544, -0.12529940242999327], [-0.14798424139973912, -0.63070289653660727]], [[-0.076147216178562599, -0.32453637868973323], [0.080540644483982043, 0.34326099376863889]]], [[[0.13534406077787473, 0.57683095412200225], [-0.036423942170472824, -0.15523737941896554]], [[-0.00026674425276563868, -0.0011368532977733522], [-0.14342353042927458, -0.61126532945425616]]], [[[0.12755254769228827, 0.54362383811423709], [0.12900367271694343, 0.54980847471898564]], [[0.0010026882865377706, 0.0042734172278144642], [0.12719404116468844, 0.54209589768459554]]], [[[0.025806189310065487, 0.10998494294041114], [0.021526074327276629, 0.09174326469399062]], [[0.049633332107602388, 0.21153526908631642], [-0.031728218162370189, -0.13522439218982643]]], [[[0.13814400464258503, 0.58876420247945738], [-0.10687558587554251, -0.45549945685548116]], [[0.11137982779191608, 0.47469635509604891], [0.030805461153836554, 0.13129163886030437]]], [[[0.067210430532829699, 0.28644815700357978], [-0.049589630802974252, -0.21134901588022711]], [[0.058541567921520168, 0.24950181253530684], [-0.0084323621967952109, -0.035938389195093741]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s0=numpy.array(-0.870896710391)
      s1=numpy.array([-0.5298834156958725, 0.65002629094587738])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.46147372362041317, -0.56610575845258837])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s0=numpy.array([0.91942561355949515, 0.45548720570358969])
      s1=numpy.array([-0.20108239063231048, -0.84491913543288177])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.18488030038312214, -0.77684029450353553], [-0.091590456225308786, -0.38484985604381616]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.89144116307641319, 0.29620862577605633, -0.90737889555558771, -0.25298774967123716, -0.57862153354268053], [-0.63088505147536944, 0.27892757846095928, 0.88225952946559949, 0.44415941243854284, -0.73329457741353665], [0.24635545567455308, -0.43524150974073472, 0.10303448269089377, -0.94557095857164986, -0.79238441138013815], [-0.35590731609750659, 0.37515811633377671, -0.37113584668940902, -0.5343189409619844, -0.021910696823390197]])
      s1=numpy.array([0.40828859302138776, 0.36891245935990247])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.3639652582338182, -0.32886375184517147], [0.12093860305890482, 0.10927505261866194], [-0.37047245260369166, -0.33474337993068393], [-0.10329201236491647, -0.093330332919143458], [-0.23624457182201875, -0.21346069297782858]], [[-0.25758317002510439, -0.23274135591317721], [0.11388294856468781, 0.10289985895333464], [0.3602165019652212, 0.32547653280886463], [0.18134522158173894, 0.16385594119055211], [-0.29939581128238596, -0.27052150598890817]], [[0.10058412238050614, 0.090883597029628824], [-0.17770414363654921, -0.16056601577397139], [0.04206780397055155, 0.038010704408372925], [-0.38606583627710384, -0.34883290782596782], [-0.32352151645447713, -0.29232048196069543]], [[-0.14531289733546929, -0.13129864328571336], [0.1531727794784718, 0.13840050334552195], [-0.15153053266462027, -0.13691663795880959], [-0.21815632863004655, -0.19711691459286421], [-0.0089458875781401742, -0.0080831290514060804]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_Symbol_rank1(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.98222190577135327, 0.87056128816866685], [-0.51824444290580707, -0.41366005871775147]], [[-0.6789885814302099, 0.063695322916400343], [0.15645819966765107, -0.69894660922680374]], [[-0.58049831907832239, 0.93322399301310655], [-0.51466157168952797, 0.75135067949234435]], [[-0.93061847097994455, -0.32863307485677318], [0.11774458490419604, -0.37182345802723371]], [[-0.70843873081241227, 0.64036202420800792], [-0.38721283789403316, -0.21967292928053173]], [[0.60973439102329108, 0.53132559789452327], [-0.80530496428452758, 0.10826086410843661]]])
      s1=numpy.array([0.79450538715833074, -0.089754568845369143])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.78038059552026251, -0.088158903662984608], [0.69166563330150177, -0.078136853073047852]], [[-0.41174800175353166, 0.04651480652951924], [-0.32865514510348498, 0.037127880218761865]]], [[[-0.53946008576529469, 0.060942327377197306], [0.05060627719386955, -0.0057169462458280737]], [[0.12430688250104252, -0.014042838253492699], [-0.55531684636674417, 0.062733651557084477]]], [[[-0.46120904174408273, 0.05210237634433635], [0.74145148987432152, -0.083761117129045159]], [[-0.40890139127070341, 0.046193227468273625], [0.59695216250173988, -0.067437156289510511]]], [[[-0.7393813885826146, 0.083527259622341601], [-0.26110074837211328, 0.029496319942097598]], [[0.093548707015105217, -0.010568114451953076], [-0.29541574047447666, 0.033372854161828572]]], [[[-0.56285838810207212, 0.063585612837428598], [0.50877107796487575, -0.057475417387737586]], [[-0.30764268568367475, 0.034754121316570762], [-0.17453132572623348, 0.01971664905457339]]], [[[0.48443725840370888, -0.054726447376489205], [0.42214104986231976, -0.047688899955530911]], [[-0.63981913242940425, 0.072279799858393168], [0.086013839752568863, -0.0097169071808798271]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([0.43904425537440295, -0.78040379153059036]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array(-0.93082785122)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-0.40867462082069556, 0.72642158435445825]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-0.55635532856152148, 0.21040862926605297]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([-0.67800830418094105, -0.81183922291675081])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.37721353284002745, -0.14265879791371289], [0.45167107760497915, -0.17081797807833116]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([-0.84315546751003323, 0.059135405653836459]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[0.73224594278291644, 0.74680714650292956, -0.75857193386834254, 0.30417072867810924, -0.70304069858566809], [-0.053681997336618492, -0.78888464076560649, -0.56040766926353647, -0.78769568474447671, 0.94873986727368376], [0.32817755131061999, -0.85033232852042051, -0.69062906524921019, -0.98288252703343848, -0.68873238694249839], [-0.008976775962731276, 0.8964201514819865, 0.014835998892172242, 0.82919903026133746, 0.11328542493710869]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.61739717021945495, 0.043301660864843688], [-0.62967452874951146, 0.044162743553634816], [0.63959407354075237, -0.044858459026919643], [-0.25646321294145868, 0.017987259428403016], [0.59277260889457939, -0.041574596902020047]], [[0.045262269561228925, -0.0031745066888091028], [0.66515239809620952, -0.046651013245755192], [0.47251079037410515, -0.033139934853420247], [0.66414992332636502, -0.046580703849141106], [-0.79993520633654969, 0.056104116911196253]], [[-0.27670469670160369, 0.019406912623236242], [0.71696235199253033, -0.050284747187626398], [0.58230767238621506, -0.040840629929841936], [0.82872277658832172, -0.058123156946190267], [0.58070847770180334, -0.040728469088779702]], [[0.0075688177335895175, -0.00053084528801972228], [-0.7558215519082091, 0.053010169294160803], [-0.012509053581907821, 0.00087733281276847382], [-0.69914369601886417, 0.049035021022272002], [-0.095517225424920651, 0.0066991795583231633]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_constData_rank1(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([0.24689283882500135, 0.41387092927897373]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.75898551202820341, -0.81350462319250871], [-0.74900001844006225, 0.23645965596184659]], [[0.29933584043779349, 0.4094660825475458], [-0.084240989944021072, 0.90310891131985205]], [[-0.50545406215649358, -0.086490095743806616], [0.96346222523784042, -0.99384762928923531]], [[0.48213700229155432, 0.78896656194425319], [-0.85876311454649934, 0.64525323750920771]], [[-0.2860560778656962, 0.044068214241254955], [-0.58272069316677544, 0.9437692776686164]], [[0.35287528413502556, 0.99879579433295973], [0.87690843939505236, 0.93646651152091032]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.18738808769169035, 0.31412203917239023], [-0.20084846581726151, -0.33668591437342493]], [[-0.18492274083264532, -0.30998933366175702], [0.058380195728003459, 0.097863777549915876]]], [[[0.073903875407754463, 0.12388640244849219], [0.10109424352271593, 0.16946610809217375]], [[-0.020798497152707755, -0.034864896791512681], [0.22297112288391466, 0.37377052436806946]]], [[[-0.12479298830144538, -0.20919274241254016], [-0.021353785268434584, -0.035795736298916657]], [[0.23787192388962328, 0.39874900648437289], [-0.24537386255471685, -0.41132464189564083]]], [[[0.11903617319833802, 0.19954248917818426], [0.19479019421641794, 0.32653032416190508]], [[-0.21202246322858503, -0.35541708824786544], [0.15930840356967113, 0.26705155702820216]]], [[[-0.070625197127407363, -0.11839029477217415], [0.010880126515971788, 0.018238552779693093]], [[-0.14386956617801774, -0.24117115479102108], [0.23300987615942564, 0.39059866797365606]]], [[[0.087122380651275422, 0.14604482176454492], [0.24659552906933663, 0.41337254356051278]], [[0.21650241399184614, 0.36292691070500493], [0.23120687549394339, 0.3875762653617979]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.42301761793568682, -0.64579232017582444])+(1.-msk_arg1)*numpy.array([-0.59902956112796768, 0.91403172934818167])
      res=outer(arg0,arg1)
      s0=numpy.array(-0.632895615371)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.26772599561634597, 0.40871912787979603])+(1.-msk_ref)*numpy.array([0.37912318271573869, -0.57848667381479302])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.19045186856659369, 0.37237724045541643])+(1.-msk_arg1)*numpy.array([0.12373747109285826, -0.82813557105112912])
      res=outer(arg0,arg1)
      s0=numpy.array([-0.76618053384993812, 0.91512858383215612])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.14592051433107101, -0.28530819288569775], [-0.1742879487695348, 0.34077305670929153]])+(1.-msk_ref)*numpy.array([[-0.094805241659167427, 0.63450135392807749], [0.11323569668817973, -0.75785053235705369]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.28262772167167172, -0.24888395951884856])+(1.-msk_arg1)*numpy.array([0.43210488170185002, -0.43710308635708883])
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.061521544681513607, 0.82180939828878441, -0.15310783850642262, -0.76901640326374499, 0.20345268274410033], [0.73116631388945863, -0.19741656018012521, -0.34177803174475052, -0.98074505948982726, 0.48055773216974784], [-0.074238394529835494, 0.82794965027340717, 0.81508526068789866, 0.34883122834381952, 0.11831862599779397], [0.79095287039946682, -0.10634816706551886, -0.46467713211100814, 0.66511520994934137, 0.66016176381680314]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.017387694007058144, 0.015311725636050866], [-0.23226611788672658, -0.20453517701591512], [0.043272519567144473, 0.038106085080850893], [0.21734535398257576, 0.19139584737922444], [-0.057501368191954519, -0.050636109256083806]], [[-0.20664786945765207, -0.18197556726760974], [0.055795392623967259, 0.049133815172220617], [0.096595946429447133, 0.085063069817192222], [0.277185741704358, 0.2440917136843769], [-0.13581893697484126, -0.11960311115980519]], [[0.020981828306530101, 0.018476745578907886], [-0.23400152331563046, -0.20606338724229151], [-0.2303656901963814, -0.20286164702545709], [-0.09858937531474439, -0.086818497314033391], [-0.033440123697079133, -0.029447608123160739]], [[-0.22354520771067035, -0.19685548217781798], [0.030056940161685908, 0.026468352906838341], [0.13133063916146065, 0.1156506845376508], [-0.18797999643715796, -0.16553650698840233], [-0.18658001524229531, -0.1643036737016729]]])+(1.-msk_ref)*numpy.array([[[-0.026583759786720517, 0.02689125705774514], [0.35510785282904372, -0.35921542438928972], [-0.066158644445443698, 0.066923908756620062], [-0.3322957419590627, 0.33613944332581058], [0.087912897409063495, -0.08892979555507588]], [[0.31594053356758228, -0.31959505244141834], [-0.085304659382619163, 0.086291387752732693], [-0.14768395597535655, 0.14939223252468153], [-0.42378472791052568, 0.42868669243247021], [0.20765134201011823, -0.21005326790416007]], [[-0.032078772686049833, 0.032449831375186318], [0.35776108568647869, -0.36189934748277858], [0.35220232014646602, -0.35627628309085285], [0.15073167665741716, -0.15247520652681792], [0.051126055889902197, -0.051717436597165836]], [[0.3417745964957003, -0.34572794081460545], [-0.045953562149054612, 0.046485112052757599], [-0.2007892572003821, 0.20311180860528236], [0.28739952911326128, -0.29072391105190021], [0.28525912085814437, -0.28855874445926416]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank3_expandedData_rank1(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.31904802359565143, -0.62184504477613567])+(1.-msk_arg1)*numpy.array([0.42431340013249708, -0.7503095674680158])
      res=outer(arg0,arg1)
      s0=numpy.array([[[0.39867398825873623, 0.51940535456649073], [-0.94346029896180661, 0.14681505442278198]], [[-0.15247920442569418, -0.88744409941455116], [-0.91239854086306993, 0.35413746615956754]], [[-0.75815159447433245, -0.89855391396079165], [-0.47927457656921413, 0.91055292786308217]], [[-0.3781918637419972, 0.29554337545459397], [-0.2981305936724008, 0.041260535212849669]], [[-0.47802162093565403, -0.95606295118685347], [-0.11704018315034204, -0.50903409609204853]], [[-0.58567967770120188, 0.092681690218913149], [0.57160778628851605, -0.18687632027110856]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.12719614801294574, -0.24791344407983443], [-0.16571525181943744, -0.32298964596736407]], [[0.30100914372472681, 0.58668611185241093], [-0.046841052947676591, -0.091296214091345654]]], [[[0.048648188811455037, 0.094818437703525341], [0.28313728596983534, 0.55185271573675898]], [[0.29109895119391865, 0.56737051149667661], [-0.11298685865938192, -0.22021862850090351]]], [[[0.24188676780292756, 0.4714528122129899], [0.2866818503433276, 0.55876129886072046]], [[0.15291160641405047, 0.29803452052674645], [-0.29051011201395016, -0.56622282619805975]]], [[[0.12066136666684012, 0.23517673644261244], [-0.09429252982557576, -0.18378218354285225]], [[0.095117976684577693, 0.18539103237134999], [-0.013164092212158468, -0.025657659366921826]]], [[[0.1525118533955101, 0.29725537627469273], [0.30502999500919137, 0.59452300868959329]], [[0.037341439115389688, 0.072780857931731557], [0.16240632230096699, 0.31654033027693967]]], [[[0.18685994363070657, 0.36420200540457659], [-0.02956991008784866, -0.057633649804107984]], [[-0.18237033448723655, -0.35545146945897005], [0.059622520639325161, 0.11620811374658697]]]])+(1.-msk_ref)*numpy.array([[[[0.1691627155024476, -0.29912890769116118], [0.22039065204313291, -0.38971480692535504]], [[-0.40032284734250634, 0.70788728883727803], [0.062295594932768227, -0.11015673998175074]]], [[[-0.064698969679364393, 0.11440660592050976], [-0.37655442325010996, 0.66585779838377468]], [[-0.3871429271495383, 0.68458135455341873], [0.15026527238047321, -0.26571272905840421]]], [[[-0.32169388086727807, 0.56884839492522288], [-0.38126846643506673, 0.67419359853061422]], [[-0.20336262518114606, 0.35960430024406348], [0.38635980882218474, -0.68319657346168461]]], [[[-0.16047187560681286, 0.28376097370418069], [0.12540301452577396, -0.22174902220537379]], [[-0.12650080588465631, 0.22369023678732181], [0.017507397987450869, -0.030958174329052072]]], [[[-0.20283097931605501, 0.3586641956445904], [-0.40567032155880339, 0.71734317937720271]], [[-0.049661718064651826, 0.087816369195910488], [-0.21598998809618936, 0.3819331524652973]]], [[[-0.24851173543390201, 0.43944106565079566], [0.039326083106813833, -0.069539958900357357]], [[0.24254084334228998, -0.42888279089148651], [-0.079294126858483568, 0.14021509103262986]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[0.0058117566793332021, 0.22157369982776975, 0.45422362230813129, -0.76928920474940266, -0.16700266037852174], [-0.42506461744959068, 0.54526321019378932, -0.97561640875850397, 0.54855187986931631, 0.68788645431304962], [-0.31834527576136606, -0.76339241654135748, -0.6597815679078205, -0.98002930638668095, -0.89311935442242429], [0.7044269754298218, -0.53874539689884382, -0.92996167003831243, 0.60229525829647446, -0.49046787002613534]])
      res=outer(arg0,arg1)
      s0=numpy.array(-0.385205498982)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.0022387206316239867, -0.085351407603423704, -0.17496943708057375, 0.29633443197688936, 0.064330343122414702], [0.16373722806422372, -0.21003838695917598, 0.3758128055507573, -0.21130520060252403, -0.26497764487655423], [0.12262835079819034, 0.29406295673281868, 0.2541514880849981, 0.3775126779835753, 0.34403448657069013], [-0.27134914456676124, 0.20752768943662558, 0.35822634914115725, -0.23200744550653102, 0.18893092060801139]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_array_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[-0.98578657432706973, -0.58429086489392712, 0.51940246542354584, 0.76552239792220611, -0.96917957943110444], [-0.48804823981544332, 0.96284271070504035, -0.8886079944086458, -0.070686023578256707, -0.43511295701959996], [-0.87688277188728914, 0.03665044520374483, 0.96383841029356021, -0.39855722752821121, 0.59893624898063824], [-0.28987190212815372, 0.80708715112805307, -0.74469451674700293, -0.85236459917220153, 0.084796608224562364]])
      res=outer(arg0,arg1)
      s0=numpy.array([-0.24223204620459104, 0.99830023117823186])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.23878909902026027, 0.14153397178190621, -0.12581592200325487, -0.18543405686414116, 0.23476635266530141], [0.1182209237770438, -0.23323135998725703, 0.21524933275936406, 0.01712242012942709, 0.10539830190898797], [0.21240910811581168, -0.0088779123360123498, -0.23347255033598927, 0.096543332753787361, -0.14508155313668242], [0.070216263989719627, -0.1955023720831823, 0.1803888765889656, 0.20647002096983844, -0.020540455921444795]], [[-0.98411096504311091, -0.58329770549893645, 0.51851960130686936, 0.76422118681785278, -0.96753219819929304], [-0.48721867063388619, 0.96120610068511725, -0.88709756624497604, -0.070565873679243621, -0.43437336558131068], [-0.87539227389128949, 0.036588147919683581, 0.9622001078145207, -0.39787977237916838, 0.59791819581839423], [-0.28937918690660969, 0.80571528955211591, -0.74342870822569473, -0.85091577640174976, 0.084652473593710559]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[-0.47242916589245931, 0.29259677322082656, 0.41728722496973836, -0.84793337502290855, 0.54306729237452878], [-0.8766853585433112, 0.97843404501898834, 0.11128631696084179, 0.3716594622926348, 0.71564197275665342], [-0.43924791933959551, -0.2518254934601416, -0.73779002195279508, -0.037913530952564001, -0.75576175725003925], [0.72808754397431241, -0.8318337564356868, -0.27956296593612118, 0.79550663011065015, 0.21495967707306152]])
      res=outer(arg0,arg1)
      s0=numpy.array([[0.95861776962835754, 0.8679666483435744, -0.23882785994730305, -0.17886128258403855, -0.99999141776102585], [0.93274409014892545, 0.96562322341133222, -0.1919860445432191, 0.29748623106890704, -0.82446358862186586], [0.75546392719730315, -0.85194699705257504, -0.20445233503755289, -0.062140384316769826, -0.39942291976189082], [-0.76104055962675732, 0.74928105663634392, -0.25565934915631305, -0.65437519833641744, 0.18355150455982527]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.4528789933152147, 0.28048846614540307, 0.40001894889489725, -0.8128440007579063, 0.52059395657418195], [-0.84040616307262594, 0.9379442619645546, 0.10668104095515661, 0.35627936480424022, 0.68602711177642095], [-0.42107086075121974, -0.24140439287632148, -0.70725862529844541, -0.036344584480482597, -0.72448665010544078], [0.69795765749884409, -0.79741062029595644, -0.26799402687637297, 0.76258679148124231, 0.20606416619581022]], [[-0.41005275969942839, 0.25396424056862577, 0.36219139405357492, -0.73597788953728904, 0.47136429758733972], [-0.76093365230672261, 0.84924811868037731, 0.096592811539002524, 0.3225880178113133, 0.62115336450757608], [-0.38125254434107742, -0.21857612952606581, -0.64037713253569972, -0.032907680387767341, -0.6559759993865667], [0.63195570524408873, -0.72200395755252833, -0.24265133054456395, 0.69047322347223261, 0.18657783043812229]], [[0.11282924666678547, -0.069880261175816361, -0.099659814922871404, 0.20251011333461519, -0.12969959924518504], [0.20937688802803309, -0.23367730907146814, -0.026578272921175098, -0.088762634008515348, -0.17091524084193768], [0.10490464056218118, 0.060142943683259176, 0.17620481203345978, 0.0090548074604466944, 0.18049696311404001], [-0.17388758998167292, 0.19866507588146121, 0.066767424875044598, -0.18998914604321737, -0.051338359650322622]], [[0.084499286541632787, -0.052334234138228103, -0.074636528264021645, 0.15166245100240999, -0.097133712443549278], [0.15680506765170435, -0.17500396821598516, -0.019904813385670007, -0.066475488110154773, -0.12800064111822659], [0.078564446225450363, 0.045041830747639337, 0.13196206960418289, 0.0067812627734652421, 0.1351765172297088], [-0.1302266719487081, 0.14878285257278567, 0.050002990650332514, -0.14228533616569722, -0.038447963545138526]], [[0.47242511139245924, -0.29259426208539568, -0.41728364371105281, 0.84792609785605, -0.54306263164124657], [0.87667783462005899, -0.9784256478641935, -0.11128536187507508, -0.37165627262231238, -0.71563583094622329], [0.43924414960898284, 0.25182333223357695, 0.73778369006251399, 0.037913205569581013, 0.755755271122031], [-0.72808129535301591, 0.83182661743960229, 0.27956056665993917, -0.79549980288264499, -0.21495783223774306]]], [[[-0.4406555125001777, 0.27291791101837132, 0.38922219298516858, -0.79090484439265041, 0.50654280751552028], [-0.81772308710136532, 0.9126285730919691, 0.10380165445966531, 0.3466631670013825, 0.66751082075128676], [-0.4097059008742196, -0.23488874077378397, -0.68816928274731559, -0.035363621932682429, -0.7049323126355409], [0.67911935375308574, -0.77588802030176751, -0.2607607043014224, 0.74200410790999616, 0.20050236841021959]], [[-0.4561885740026036, 0.28253823931724908, 0.40294223526364853, -0.81878415882767097, 0.52439838939195682], [-0.84654774183411163, 0.9447986364466241, 0.10746065210530327, 0.35888300799033651, 0.6910405085417245], [-0.42414799174942108, -0.24316854473213129, -0.71242717919877552, -0.036610185969320169, -0.72978110416679565], [0.70305824113811566, -0.80323799323178491, -0.2699524923136698, 0.76815967641253236, 0.20757005627874872]], [[0.090699806886545539, -0.056174497136775789, -0.080113323760356472, 0.16279137470683022, -0.10426134138378168], [0.16831135429568417, -0.18784568214961755, -0.021365419805094973, -0.07135343008262264, -0.13739327165865606], [0.084329470607847901, 0.048346980404556879, 0.14164538801817192, 0.0072788688422496684, 0.14509571039146757], [-0.13978264764881534, 0.15970047261561504, 0.053672188030846606, -0.15272617132284941, -0.041269258137544784]], [[-0.14054117200837515, 0.087043511288387401, 0.12413720382945058, -0.25224850393310316, 0.16155504202529478], [-0.26080182314634309, 0.29107065640220414, 0.033106147002220608, 0.11056357267853249, 0.21289363327009428], [-0.13067020802919554, -0.074914616936525222, -0.21948237295098319, -0.011278753429592614, -0.22482871675032839], [0.21659601934513531, -0.24745908907794367, -0.083166133082781937, 0.23665226918194443, 0.063947544164254411]], [[0.38950064548133179, -0.24123538566882091, -0.34403812298461034, 0.69909019328363764, -0.44773920873426404], [0.72279515679686546, -0.80668324398616342, -0.091751516246046036, -0.30641969402705871, -0.590020749027382], [0.36214391587341077, 0.20762095004462056, 0.60828100914860661, 0.031258325786477102, 0.6230980505255348], [-0.60028166933594218, 0.68581664396777342, 0.23048948614146691, -0.65586625103351393, -0.17722642676865372]]], [[[-0.35690319298766354, 0.22104630738266434, 0.3152454457449031, -0.64058307749647014, 0.41026774942966754], [-0.6623041638815057, 0.73917162615358789, 0.084072798054561387, 0.28077531696363189, 0.54064169520596683], [-0.33183595815753508, -0.19024507625779735, -0.55737374743144308, -0.028642304987340508, -0.57095074515764954], [0.55004387531427323, -0.62842039641218894, -0.21119973614502799, 0.60097656289488421, 0.16239428183067914]], [[0.40248460920213353, -0.24927694229275649, -0.35550659822137093, 0.72239429255142185, -0.46266454893595255], [0.74688945857093403, -0.83357394646793115, -0.094810043547830208, -0.31663416282638496, -0.60968902965481164], [0.37421594584296008, 0.21454197293465052, 0.62855799365803722, 0.032300318842696754, 0.64386895957634815], [-0.62029199668030011, 0.70867827084234647, 0.23817282931638978, -0.67772948465818195, -0.18313425136978603]], [[0.096589246106556739, -0.059822093509451314, -0.085315347526403656, 0.17336195847970667, -0.11103137600849385], [0.17924036864741424, -0.20004312518437031, -0.022752747360373332, -0.075986644904530526, -0.14631467238097862], [0.089805262769366984, 0.051486310159909954, 0.15084289275565635, 0.0077515099327702466, 0.15451725600185473], [-0.14885919847730514, 0.17007035386633521, 0.057157301175663831, -0.16264318806397732, -0.043949007916505752]], [[0.02935692993100843, -0.018182075937788908, -0.025930388530097933, 0.052690905798939251, -0.033746410258020824], [0.054477565104766507, -0.060800267586091604, -0.0069153745051445687, -0.023095061821828348, -0.044470267220309764], [0.027295034518103979, 0.015648532944373406, 0.045846555509224736, 0.0023559613841980754, 0.046963326047434743], [-0.04524363979881682, 0.051690469312575885, 0.017372150144006601, -0.049433087721614262, -0.013357676945928779]], [[0.18869903682144079, -0.11686985747277037, -0.16667408177674989, 0.33868402441520445, -0.21691352354741872], [0.35016822562186944, -0.39080898305592165, -0.044450305650046658, -0.14844930758605857, -0.28584380626262201], [0.17544568644195674, 0.10058487386832871, 0.29469024473957495, 0.015143533231555935, 0.30186856772518805], [-0.29081485265648394, 0.33225346775204356, 0.11166385611149954, -0.31774358088873839, -0.085859821847595413]]], [[[0.35953775679479938, -0.22267801203696125, -0.31757250321606628, 0.64531169025363944, -0.4132962361036992], [0.66719311588238595, -0.74462799317912276, -0.084693400938679733, -0.2828479251737665, -0.54463256743912014], [0.3342854823490945, 0.19164941447119049, 0.56148813119399277, 0.028853734813565693, 0.57516535068207142], [-0.55410415192348195, 0.63305922751424282, 0.21275875604694175, -0.60541281096620503, -0.16359303293686978]], [[-0.35398222460572853, 0.21923721940728563, 0.31266541284617333, -0.63534041519438622, 0.40691003465502523], [-0.65688373178694426, 0.73312209510079973, 0.083384729161586624, 0.27847739461552085, 0.53621697352042297], [-0.32912014512808768, -0.1886880718277836, -0.55281208722454167, -0.028407890532951886, -0.56627796803764952], [0.54554220427283329, -0.6232772759679106, -0.20947123451300709, 0.59605804837052512, 0.1610652139715108]], [[0.120780933074526, -0.074805100606873842, -0.10668338034700729, 0.21678209478627272, -0.1388402305165532], [0.22413280818005191, -0.25014581114193324, -0.028451387364211975, -0.095018216237520284, -0.18296056098390592], [0.11229783717662568, 0.064381541758987165, 0.1886229168264735, 0.0096929486475502424, 0.1932175589757765], [-0.18614238762129118, 0.21266607677659871, 0.071472885919437262, -0.20337870730362068, -0.054956451135350137]], [[0.30914592913078631, -0.19146807150897413, -0.27306241060282577, 0.55486657045668364, -0.35536976715760349], [0.57368115537541253, -0.64026297226840367, -0.072823005733380269, -0.24320473435134915, -0.46829835786050011], [0.28743294433670652, 0.16478835722914636, 0.48279149194599008, 0.02480967433671797, 0.49455174979557381], [-0.47644243099446576, 0.54433137935052966, 0.1829390712819664, -0.52055980885659181, -0.14066428131901687]], [[-0.086715084197504191, 0.053706577954032705, 0.076593697976789762, -0.15563944675194544, 0.099680818592575285], [-0.16091691658619464, 0.17959304107579113, 0.020426770915084113, 0.068218653487708766, 0.13135716082564522], [-0.08062461646955553, -0.046222948211129433, -0.13542246857866205, -0.0069590856495186275, -0.13872120763202214], [0.13364156414775299, -0.15268433753742156, -0.051314203016782224, 0.14601643884412624, 0.039456172146454617]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s0=numpy.array(0.936613051668)
      s1=numpy.array([[-0.99225104930139985, -0.44743263332382432, -0.5091617176325276, 0.74277462115549664, -0.23276043614259012], [-0.030627838933931839, 0.4152724318442973, -0.23216302409029166, -0.46161781990058759, -0.87774869596377014], [-0.16923551793437808, -0.9569757304177664, -0.55943979910689978, 0.12985423156170239, 0.94164062722761321], [-0.80600947095617603, -0.63249175840722938, 0.5124107721052944, -0.82073637344243688, -0.33827106757663539]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.92935528330668027, -0.41907124411315061, -0.47688751014417907, 0.6956924046217835, -0.21800646240302055], [-0.028686433689897275, 0.38894957966316213, -0.21744691847761421, -0.43235727500128879, -0.82211088472398752], [-0.15850819490308482, -0.89631595923852847, -0.52397861746588903, 0.1216231680949727, 0.8819529014419597], [-0.7549189902653477, -0.59240003599647673, 0.47993061696895184, -0.76871239934461733, -0.31682909689384964]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_Symbol_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s0=numpy.array([-0.031722434829994928, 0.71938399302034339])
      s1=numpy.array([[0.099269950916488714, 0.72790045697058781, 0.050692331604960783, 0.86293132531332861, -0.12110286824728811], [0.33619024127189778, -0.28407717834471491, -0.63802425603424218, -0.97711564476601787, -0.63929112996209225], [0.21699906107219191, -0.96023189468253967, -0.73922407016413683, -0.35117749626275852, 0.61882256456667384], [-0.38863000450896523, 0.48936574903908681, 0.19081512150528557, -0.39807898894904437, -0.41449280151285439]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.0031490845485251085, -0.023090774808972998, -0.0016080841857188607, -0.02737428273001322, 0.003841677845700059], [-0.010664773019228049, 0.009011619776729065, 0.020239682882002244, 0.030996487362458476, 0.02027987120761629], [-0.0068837385730326972, 0.030460893700749416, 0.023449987390545428, 0.011140205238956145, -0.019630558475796638], [0.012328289991016284, -0.015523873081924084, -0.0060531202565289851, 0.012628034784126331, 0.013148720883493547]], [[0.071413213677237145, 0.52363993725683411, 0.03646725192548804, 0.62077898250623931, -0.087119464925950671], [0.24184987818065048, -0.20436057488357323, -0.45898443694974705, -0.70292135417442536, -0.45989580577461719], [0.15610565103577878, -0.69077545462221523, -0.53178596333142725, -0.25263146952038995, 0.44517104746906311], [-0.27957420445117348, 0.35204188659112956, 0.13726934403713434, -0.28637165260766467, -0.29817948663050581]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.2816878161179992, 0.78496502472757634, -0.98941695226353654, -0.64944696619417885, 0.42544477996743391], [0.75302328310643141, 0.49576132785600047, -0.47347547694861336, 0.11472844680189542, -0.63906388397222536], [0.32395112108511093, -0.11808328639283627, -0.2260509437874525, -0.69666934574255834, 0.72027400311628664], [0.43224217256966257, 0.73789805088831817, 0.72501716704316221, -0.73391441690659098, 0.86157830456633233]])
      s1=numpy.array([[-0.012085383038652697, -0.092068215810325871, 0.51852357289379092, 0.9431762497172862, -0.92487728621793952], [-0.43149363453549561, -0.63839756360525457, 0.12139646774146762, 0.71955945140511979, -0.84457064560713313], [0.003737666838816045, -0.7632837774951986, -0.92609124365165862, 0.41696137230397134, -0.091981659719451159], [0.0381392238267515, 0.19681243060329967, -0.29789086356741779, 0.16240683097319653, -0.34154457471849797]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0034043051551075876, 0.02593449464549134, -0.14606177285415414, -0.26568125799722703, 0.26052666293187304], [0.12154649958112183, 0.17982881550701565, -0.034195905882533149, -0.20269113043337378, 0.23790526071844198], [-0.0010528552092027574, 0.21500774036091932, 0.26086861995023763, -0.11745293836986968, 0.025910112849281131], [-0.01074335466819319, -0.055439663761518758, 0.083912226799810766, -0.04574802553948476, 0.096208945359404496]], [[-0.0094866029957782465, -0.072270329300176281, 0.4070228692184058, 0.74036036818179229, -0.72599632184603857], [-0.33870741150294709, -0.50111975930142316, 0.095291981302521547, 0.56482900256518109, -0.66295841771318842], [0.0029339377425546789, -0.59915106927567641, -0.72694923597301619, 0.32730009392103104, -0.072202385796162491], [0.02993795677425656, 0.15449087445521353, -0.23383390908631718, 0.12748368209080252, -0.26810054553947532]], [[0.01195748285304119, 0.091093853487394169, -0.51303601316937431, -0.93319457044262955, 0.91508926574752425], [0.42692711680322631, 0.63164137171477819, -0.12011172312832162, -0.71194431938167591, 0.83563251414785711], [-0.0036981109322378582, 0.75520590884149874, 0.91629037581177231, -0.41254865019661713, 0.091008213423761072], [-0.037735594600361327, -0.19472955525509555, 0.29473827033802746, -0.16068807172827942, 0.33792999218012199]], [[0.0078488153497475802, 0.059793423440927071, -0.33675356131603867, -0.61254295396529479, 0.60065874763614602], [0.28023223188117741, 0.41460536090918793, -0.078840567681385643, -0.46731570271140271, 0.54850384352621162], [-0.0024274163891136675, 0.49571233363948936, 0.60144714860856374, -0.27079429826297569, 0.059737209850302862], [-0.024769403207284501, -0.12781923596461534, 0.19346431760082353, -0.10547462366475328, 0.22181508787100954]], [[-0.0051416631277017549, -0.039169941817418306, 0.22060314737772654, 0.40126941203148031, -0.39348421353186869], [-0.18357671440230228, -0.2716029109797834, 0.051647493507092378, 0.30613281247653862, -0.35931817248728037], [0.0015901708458316666, -0.32473509876915657, -0.3940006853851471, 0.17739403929478237, -0.039133116980381279], [0.016226133689101006, 0.083732821232876689, -0.12673611290474893, 0.069095138468599826, -0.14530835644018217]]], [[[-0.0091005748133650335, -0.069329510139243045, 0.39046032322855945, 0.71023367611012223, -0.69645413053839944], [-0.32492475331744558, -0.48072822927317566, 0.091414366696203939, 0.54184502048734595, -0.63598136037040176], [0.0028145501541232952, -0.57477045607131338, -0.69736826875069013, 0.31398162150089959, -0.069264331387519709], [0.028719723541151449, 0.14820434264905341, -0.22431875609094698, 0.12229612505834772, -0.25719101698171321]], [[-0.0059914655428908474, -0.04564386092345997, 0.25706393502246344, 0.4675903099620845, -0.45851839151925988], [-0.21391785721872908, -0.3164928238329765, 0.060183674044538109, 0.35672974909993743, -0.41870546473439191], [0.001852990675094782, -0.37840657906196368, -0.45912022466856112, 0.20671332359807701, -0.045600949760913896], [0.018907952247747536, 0.097571991934458785, -0.1476827700783537, 0.08051502617615694, -0.16932459188445553]], [[0.0057221324983327678, 0.043592042392601907, -0.24550819598498674, -0.44657082468149656, 0.4379067142109781], [0.20430165441198445, 0.30226559091083066, -0.057478250463768338, -0.34069375444692168, 0.39988348924563566], [-0.0017696935891834428, 0.36139615059667846, 0.43848149328590358, -0.19742098462077118, 0.043551060206192198], [-0.018057987191821084, -0.09318585944931318, 0.14104401870621744, -0.076895651754747052, 0.16171298041405213]], [[-0.0013865372250305953, -0.010562843399740399, 0.059489404148274035, 0.1082091461905009, -0.10610973453013628], [-0.049504594495162112, -0.073242360914545099, 0.013927628191214981, 0.082553938241333336, -0.096896278384980439], [0.00042881671108031523, -0.087570362261107676, -0.10624900998099049, 0.047837330620821489, -0.0105529129538731], [0.0043756539118730417, 0.022579984474422402, -0.03417655609356518, 0.018632683467572801, -0.039184878571067193]], [[0.0077233318239734476, 0.058837471586139896, -0.33136968842466136, -0.60274987741468644, 0.59105567072812792], [0.27575199799554578, 0.40797682651597977, -0.077580098175371259, -0.45984445776387961, 0.53973459707062443], [-0.0023886078870079715, 0.48778709541907345, 0.59183146708069745, -0.26646495405096499, 0.058782156714524053], [-0.024373400510409853, -0.12577571631535875, 0.19037129227123428, -0.10378834018535169, 0.21826880246924524]]], [[[-0.0039150733841145256, -0.029825601728061001, 0.1679762927480008, 0.30554300347676538, -0.29961503373645654], [-0.13978284664886295, -0.20680960642792562, 0.039326521820620935, 0.23310209097007595, -0.27359960748000667], [0.0012108213626771003, -0.24726663542564795, -0.30000829660805944, 0.13507510400705783, -0.029797561785385392], [0.012355244315992123, 0.063757607537424529, -0.096502079213676817, 0.052611874965647135, -0.11064374788059485]], [[0.0014270817465203522, 0.010871717495208166, -0.061228967559454228, -0.11137335121428757, 0.10921254946670216], [0.050952186423540755, 0.075384082335688191, -0.01433489386739443, -0.084967944776942916, 0.099729677424209726], [-0.00044135598376892206, 0.090131056896971457, 0.10935589755001672, -0.049236169140519877, 0.010861496667540363], [-0.0045036048899347826, -0.02324025860859965, 0.035175932156440715, -0.019177532333960919, 0.040330705832403863]], [[0.0027319122419203127, 0.020812107076751019, -0.11721274302868336, -0.21320588140650251, 0.20906938343714307], [0.097539543325026887, 0.14431037176457803, -0.027441786105421785, -0.16265709310130888, 0.19091599153467051], [-0.00084490311647743104, 0.17254101828044155, 0.20934379966075306, -0.094254511732224072, 0.020792540990718238], [-0.0086214075413580723, -0.04448963568697839, 0.067338510855074044, -0.036712217419020345, 0.077206473460600555]], [[0.0084195158945863866, 0.064141103672264388, -0.36123947828001107, -0.65708198081046165, 0.6443336538816048], [0.30060838806392232, 0.44475201296051597, -0.084573197756905832, -0.50129501223327899, 0.58838647910849151], [-0.0026039179112016305, 0.53175640988348849, 0.6451793808127132, -0.290484206442927, 0.064080802697064676], [-0.026570428110511961, -0.13711318726240346, 0.20753143302419866, -0.11314386067821908, 0.23794363541105631]], [[-0.0087047872204440505, -0.06631434236147761, 0.37347904955837047, 0.67934533302807609, -0.66616506533552289], [-0.31079364746607741, -0.45982116871764095, 0.087438719784324034, 0.51827996654372477, -0.60832227982595644], [0.0026921442563090292, -0.54977346193018772, -0.66703944731592058, 0.30032643677424181, -0.066251998259409181], [0.027470691421442366, 0.14175887725368502, -0.2145630447934716, 0.11697741827849439, -0.2460056780751422]]], [[[-0.005223812220963792, -0.039795765626467812, 0.22412775567619597, 0.4076805512939064, -0.39977096755517583], [-0.18650974604160261, -0.27594234985591459, 0.052472672958854924, 0.31102394056838351, -0.36505905074578976], [0.0016155772347514301, -0.32992343827170356, -0.40029569115373365, 0.18022828944229652, -0.03975835243369899], [0.01648538096699571, 0.085070632592686199, -0.12876099405703362, 0.070199081460008431, -0.14762996900570502]], [[-0.0089177805884605651, -0.067936956995204495, 0.38261753377797508, 0.69596791631053911, -0.68246514681109471], [-0.3183983118944585, -0.47107231787616849, 0.089578216931155544, 0.53096151669010538, -0.62320703323099202], [0.0027580170752322612, -0.56322561168837981, -0.68336092363529743, 0.30767498391881881, -0.067873087424455539], [0.028142858924153236, 0.1452275089327672, -0.2198130876038355, 0.11983968402607026, -0.25202507597625923]], [[-0.0087621101733154627, -0.066751037001520935, 0.37593849186455491, 0.68381897259242097, -0.67055190991629843], [-0.3128402925080826, -0.46284919301233862, 0.088014523130965486, 0.52169295497687185, -0.61232821684589822], [0.0027098726228295808, -0.55339384200957231, -0.67143204989580441, 0.30230415291425455, -0.066688282349724626], [0.027651592012096445, 0.14269239087488328, -0.21597598999169038, 0.11774774050064464, -0.24762567998136703]], [[0.0088696368459055992, 0.067570190922065496, -0.38055192565266877, -0.69221064735140736, 0.67878077422478933], [0.31667939918902394, 0.46852917564793872, -0.089094617836998985, -0.52809505520761502, 0.61984257290718225], [-0.002743127578600779, 0.56018496849464883, 0.67967171508690671, -0.3060139624270411, 0.067506666159101461], [-0.027990926216080288, -0.14444348024618958, 0.21862639943688228, -0.11919271465534081, 0.25066448740213604]], [[-0.010412503828477101, -0.07932397728230775, 0.44674866081150943, 0.81262019413865116, -0.79685420419156283], [-0.37176555407425693, -0.55002949049029248, 0.10459256285703512, 0.61995681217630327, -0.72766374492868646], [0.0032202926580209312, -0.65762874291729889, -0.79790012357912221, 0.3592448722193069, -0.079249402432282026], [0.032859927802128423, 0.16956932027676988, -0.25665630517821642, 0.13992620207987758, -0.29426739561979248]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[-0.21517290237885955, 0.30055746826720453, -0.019017947321739781, 0.16856597210882129, 0.23154115085683058], [0.74464905838045903, 0.84417205163237563, -0.077634634862838059, -0.823466245160414, -0.93036920916374011], [0.15165768106384658, 0.096832496836942505, 0.5022209299931728, 0.11035285027102248, -0.72371862278947252], [0.37588393494568106, -0.80493498776596528, -0.57594565823435584, 0.55015419444458025, 0.043415148570128625]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array(-0.794656804172)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.17098861094882659, -0.23884003720328359, 0.015112741240607684, -0.1339520966881646, -0.18399575097422641], [-0.59174044096240264, -0.67082706472161524, 0.061692890833173246, 0.65437305472279961, 0.73932422245421114], [-0.12051580816235272, -0.076948602476452771, -0.39909327921673055, -0.087692643327656472, 0.57510792790573861], [-0.29869872648358031, 0.63964706494443579, 0.45767913614932793, -0.43718377395922359, -0.034500143215396704]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_constData_rank2(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[0.32170850381565574, 0.19644887253780952, -0.1653381402168459, -0.97604096864978773, 0.41084060629913055], [0.77913745970315063, 0.58072255401919892, -0.95466054710982884, -0.57722873105388572, -0.76399066939231042], [0.46397524647426347, -0.47864109524121456, -0.97214009578327132, 0.52080197918895932, -0.49527215961905324], [-0.58203655991408465, -0.5002019338615824, -0.10918317326753502, -0.2135490646958087, 0.59149461703262451]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([-0.51091576483256573, 0.061290234111242015])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.16436594628011614, -0.10036882596315017, 0.084473862364883814, 0.49867471800562463, -0.20990494259159531], [-0.39807361113393758, -0.29670030784224, 0.48775112358209383, 0.29491525860972739, 0.39033487717751614], [-0.23705226791577652, 0.24454528125546207, 0.49668170056151378, -0.26608594152364112, 0.25304235423204519], [0.29737165416902001, 0.25556105360961884, 0.055783404476829196, 0.10910558371833816, -0.30220392465556889]], [[0.019717589514438937, 0.012040397388731888, -0.010133613321407844, -0.059821779470708915, 0.025180516942478321], [0.047753517310044492, 0.03559262128951509, -0.058511368429127797, -0.035378484062027808, -0.046825166985859201], [0.028437151478228825, -0.029336024782595328, -0.059582694059481932, 0.031920075230089509, -0.030355346611832198], [-0.035673157018436187, -0.030657493629272382, -0.0066918622505755219, -0.013088472169442882, 0.036252843553468993]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[-0.4908727541030613, 0.29186039448744849, 0.83268303847262048, 0.39849977115359958, 0.81774312490612777], [-0.36533425390050245, 0.74727051395141708, 0.63412709175670789, 0.89205729404909451, 0.12008278800168659], [0.73519564323632558, -0.16095387650383564, 0.43753657479667751, -0.6617806270511799, 0.83299461768843286], [0.90772193640279286, 0.30807698601402089, 0.86657814277374823, -0.62019187429919165, -0.63430869100107934]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.82342171271984732, 0.81514700396005191, 0.14757068789888517, 0.33223708398470198, 0.11655422087356238], [0.86064758833060662, 0.37042513939977484, 0.26073452844607958, 0.42192222813132285, 0.99848971268944187], [-0.29835182160821572, -0.51782290887774329, -0.47786049228830474, 0.14037758803892575, -0.35609103226435446], [0.76588123463095359, 0.018894939017414236, -0.28281549656738214, -0.41295835941891124, 0.14130817602790735]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.4041952839110512, -0.24032418590394514, -0.68564929369189165, -0.32813336408176419, -0.67334744447508377], [0.30082415706197929, -0.61531876646291639, -0.52215401597636424, -0.73453934491013784, -0.098878774964523103], [-0.60537605573782505, 0.13253291665968714, -0.36027711579665578, 0.54492453737129709, -0.68590585478342381], [-0.74743795154616399, -0.25367727947323354, -0.71355925852834412, 0.51067945535037262, 0.5223035487371932]], [[-0.40013345483272972, 0.23790912614104248, 0.67875908405930918, 0.32483589453462303, 0.66658085827616054], [-0.29780112251097546, 0.60913532059518583, 0.51690679897538139, 0.72715783060483041, 0.097885124866744888], [0.59929252590857396, -0.13120107020785782, 0.35665662806845483, -0.53944849541957374, 0.67901306692357488], [0.73992681688755335, 0.25112803213837193, 0.70638857677928701, -0.50554754821535519, -0.51705482905535216]], [[-0.072438429993809061, 0.043070039184952766, 0.12287960878913846, 0.058806885356685004, 0.12067491546698125], [-0.053912627161123119, 0.11027522379036409, 0.093578571145856854, 0.13164150852804296, 0.017720699630224886], [0.10849332681264792, -0.023752074275663233, 0.064567573323667721, -0.097659422372098195, 0.1229255887483509], [0.13395315057586824, 0.045463132751904285, 0.12788153264726035, -0.091522141519630631, -0.093605369871270674]], [[-0.16308613243074074, 0.09696684639513467, 0.27664818458566481, 0.13239640193664298, 0.27168459126734978], [-0.12137758719562966, 0.24827097650296837, 0.21068053584094817, 0.29637451412215499, 0.039895955322433507], [0.24425925666709408, -0.053474846585668187, 0.14536587574710258, -0.2198680657690516, 0.27675170275575656], [0.30157888921941101, 0.10235459947609411, 0.28790939520002884, -0.20605073982817027, -0.21074086984435197]], [[-0.057213291402542076, 0.034017560883335116, 0.097052722783806847, 0.046446830345100706, 0.095311412798145922], [-0.042581249321797278, 0.087097532535393943, 0.073910189114521088, 0.10397304288247054, 0.013996155795861744], [0.085690155387047454, -0.018759853672484141, 0.050996734579113898, -0.07713332537516783, 0.09708903865654625], [0.10579882306726886, 0.035907673073939578, 0.10100334025705292, -0.072285980701056615, -0.073931355272960034]]], [[[-0.42246845199600258, 0.25118894464484204, 0.71664664890526253, 0.34296786699364412, 0.70378864832439292], [-0.31442404455402895, 0.64313656566286004, 0.54575995221551199, 0.76774695877608001, 0.10334896189366707], [0.63274435730251266, -0.13852456564548843, 0.37656479790519459, -0.56955990067551454, 0.7169148088059254], [0.78122869543985196, 0.26514571503312911, 0.74581838867824246, -0.53376664091783799, -0.54591624516722292]], [[-0.18183160836617787, 0.10811242731328638, 0.30844673060204852, 0.14761433328035051, 0.30291261103655986], [-0.13532899192860634, 0.27680778429979508, 0.23489661636115233, 0.33044044750072177, 0.044481683485038365], [0.27233494863192304, -0.059621362140867461, 0.16207454671155927, -0.24514018102750371, 0.3085621473764999], [0.33624302482823809, 0.11411946049010617, 0.32100232933776368, -0.22973466149188571, -0.23496388528656351]], [[-0.12798747606809008, 0.076098082328771646, 0.21710921938120745, 0.10390264991760441, 0.21321386806242279], [-0.095255254415947815, 0.19483922507678228, 0.16533882824406906, 0.2325901379107764, 0.031309729104110298], [0.1916908893548355, -0.041966233091796111, 0.11408089250752454, -0.17254905972894025, 0.21719045884111587], [0.23667445104814444, 0.080326307673455194, 0.22594684341779267, -0.16170543589149, -0.16538617743741643]], [[-0.20711012614012259, 0.12314238794543113, 0.35132748291952809, 0.16813591135494899, 0.34502400129946409], [-0.15414264241839443, 0.31529004026322066, 0.26755231547242603, 0.37637880112599259, 0.05066559747389289], [0.31019538390671159, -0.067910018200872099, 0.18460640652716137, -0.27921995669957783, 0.35145894511650305], [0.3829880619307453, 0.12998452837501809, 0.36562858084900346, -0.26167273747325626, -0.26762893623023815]], [[-0.49013139521144072, 0.29141960143719958, 0.83142544784589834, 0.39789792200596596, 0.81650809784128586], [-0.3647824942127243, 0.74614192077664199, 0.63316937765674663, 0.89071003123760129, 0.11990142849075121], [0.73408528658556815, -0.16071078990656676, 0.43687576885985702, -0.66078114816777134, 0.83173655648757483], [0.90635101548072849, 0.30761170125136894, 0.86526936080111005, -0.61925520638132636, -0.63335070263408366]]], [[[0.14645278036449008, -0.087077080350622699, -0.2484325013505703, -0.11889313263413354, -0.24397515092333791], [0.10899814014709329, -0.22294951907151286, -0.18919297295673396, -0.26614691865844309, -0.035826918544096385], [-0.21934695939798163, 0.048020882249823156, -0.13053983411080805, 0.19744345558574677, -0.24852546157718317], [-0.27082049323951018, -0.091915329972851928, -0.25854516746241224, 0.18503537544377738, 0.18924715342209486]], [[0.25418515741847642, -0.15113199845969624, -0.43118235315505016, -0.20635231068587195, -0.42344612365366685], [0.18917844606743822, -0.38695379125288903, -0.32836553525164214, -0.46192770289011054, -0.062181618589182724], [-0.38070114657487769, 0.083345604526365233, -0.22656646190161986, 0.34268516933857895, -0.43134369601092798], [-0.47003921356023209, -0.15952932105606812, -0.44873401466097462, 0.32114956041194714, 0.32845957150061256]], [[0.23456869592660484, -0.13946855178923095, -0.39790632668464782, -0.19042729682023588, -0.3907671322330189], [0.17457880641867463, -0.35709105566935867, -0.30302428424021144, -0.42627893768367331, -0.057382820189838091], [-0.35132095200512742, 0.076913498661833904, -0.20908144302647899, 0.31623881622953981, -0.3980552180821027], [-0.43376445139033182, -0.1472178201993572, -0.41410345791214814, 0.29636519436581815, 0.30311106334452592]], [[-0.068907533255012435, 0.040970658222237402, 0.11689003654171048, 0.055940436708606191, 0.11479280750973621], [-0.051284741390553036, 0.10490003236110834, 0.089017231650945203, 0.12522485133114264, 0.016856932144666416], [0.10320499113424195, -0.022594316969123569, 0.061420329048770615, -0.092899168236332499, 0.11693377528050929], [0.1274238160422472, 0.043247104226950113, 0.12164814952983061, -0.087061039435461146, -0.089042724114859764]], [[0.17479538571900574, -0.10392886915011725, -0.29651096271873462, -0.1419021948671943, -0.29119099347490196], [0.13009225159295768, -0.26609632869367478, -0.22580697069043912, -0.31765360267688886, -0.042760403936702218], [-0.26179657551627922, 0.057314232031200257, -0.15580285057275883, 0.23565414661920642, -0.29662191328332538], [-0.32323164134266924, -0.10970345196662379, -0.30858070539803112, 0.22084476472116391, 0.22587163655282577]]], [[[-0.37595023095914909, 0.22353039926992421, 0.63773631356166438, 0.30520349673127128, 0.6262941141140792], [-0.27980264943029509, 0.5723204638284185, 0.48566603994756341, 0.68320994172786809, 0.091969153932658781], [0.56307254693713515, -0.12327155365539567, 0.33510105210147795, -0.50684536370080435, 0.63797494623615614], [0.6952071973537709, 0.23595038240980135, 0.66369593789175707, -0.47499331839635006, -0.4858051234010507]], [[-0.0092750107540875166, 0.0055146843554388014, 0.015733495232775355, 0.0075296288744007931, 0.015451206477011037], [-0.0069029684484225228, 0.014119630790623819, 0.011981792728033238, 0.016855368171097201, 0.0022689569563329503], [0.013891476844819005, -0.0030412136811563966, 0.0082672268986715251, -0.012504304591038198, 0.015739382503057224], [0.017151350633099936, 0.0058210958634037037, 0.01637394116153406, -0.011718487643979062, -0.011985224034681245]], [[0.13882642170305576, -0.08254264239531979, -0.23549566700887073, -0.11270191066079341, -0.23127042793488933], [0.10332218842994467, -0.21133968147333287, -0.17934096834200325, -0.25228762658304987, -0.033961273317892671], [-0.20792472091605735, 0.045520250507877377, -0.12374212366751389, 0.18716181665815296, -0.23558378643951078], [-0.25671783018886152, -0.087128945780537759, -0.24508172776299739, 0.17539987289698133, 0.17939232742247641]], [[0.20271000721784282, -0.12052618968689298, -0.34386342148360749, -0.16456381172440204, -0.33769385928732831], [0.15086783413028346, -0.30859160548350384, -0.26186808347493551, -0.36838251665818733, -0.049589191127625415], [-0.30360518668280417, 0.066467248783138005, -0.18068438611380569, 0.27328784204227358, -0.34399209072539844], [-0.37485136166545463, -0.12722296671907293, -0.3578606881481341, 0.2561134189355338, 0.26194307640096282]], [[-0.069364333544099072, 0.041242259999806852, 0.11766492137594181, 0.056311275809253644, 0.11555378943984614], [-0.051624717059196398, 0.10559543332591165, 0.089607342706021831, 0.12605498913446817, 0.016968679744864209], [0.10388915536938927, -0.022744098713378066, 0.061827495330016559, -0.093515013339207032, 0.11770895006661646], [0.12826853117359877, 0.043533796969816414, 0.12245457674100985, -0.087638182544547957, -0.089633004164012015]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.1297264235237201, 0.17556296511730318, 0.051024536555732691, -0.81610512646164546, -0.045535530406377811], [-0.0023125351413044726, -0.18782565475146962, -0.32963944148114699, 0.8248232936167641, 0.39089275122812794], [0.30005042614748834, 0.1932357505868556, 0.41014178392250544, -0.15236086193428666, -0.21488727481200942], [0.31029054950830015, 0.94872371519956045, -0.16496439126215323, -0.13672438556647526, -0.8794933380832155]])+(1.-msk_arg1)*numpy.array([[0.37224239980959362, -0.8943263482537458, -0.29803349694730041, -0.022762896926632692, -0.072786946976018729], [0.67853583778722304, -0.50941459252154675, 0.54633527902441781, 0.089288197723190521, 0.73196991478331386], [0.98354184550084511, 0.14558190233747048, 0.70335454369757811, 0.027744531455062393, -0.54592953117078324], [-0.53098366168725453, -0.77649585642285857, 0.81662301877817955, -0.25877780107168125, -0.095150321392519777]])
      res=outer(arg0,arg1)
      s0=numpy.array(0.506775792187)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.065742211048790258, 0.08897106072598035, 0.025857999933994461, -0.41358232197028805, -0.023076304494337067], [-0.0011719368281943284, -0.0951854949796756, -0.16705328909261413, 0.41800047803674001, 0.19809498366370226], [0.15205829240687263, 0.097927200582459697, 0.20784992745622455, -0.077212796505007883, -0.10889966892371317], [0.15724773903513833, 0.48079021233663699, -0.083599960064486709, -0.069288608806700486, -0.44570593313011214]])+(1.-msk_ref)*numpy.array([[0.18864343704901279, -0.45322294360979709, -0.15103616151366278, -0.011535685122460174, -0.036886662714628635], [0.34386553672173659, -0.25815898367660972, 0.27686949382718173, 0.04524909713409922, 0.3709446334211996], [0.49843519790253282, 0.073777383885129222, 0.35644305607050736, 0.014060256906990124, -0.27666387063722803], [-0.2690896657897926, -0.39350930276844198, 0.41384477725926705, -0.13114232513845278, -0.048219879500520396]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_expandedData_rank2(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.37250917746423107, 0.25860771016318007, -0.85507601958162249, -0.55310035865902174, 0.70313144755048218], [0.61880138818901265, -0.4517851480888242, -0.94855999846811034, 0.4773890742586111, 0.26488998003455655], [0.93077121306968325, -0.98656779179070098, 0.23755995922111417, 0.22692502836939998, 0.99806814978107306], [0.91126729009359053, 0.58164684978731729, 0.25946684909951534, -0.91995087667248199, -0.15565405978553604]])+(1.-msk_arg1)*numpy.array([[-0.40795998945589962, -0.8652747811908863, -0.97193360665812523, -0.68342930941211311, 0.6433553760739934], [-0.58828119442140459, -0.78067084838994827, -0.94826988472059237, 0.1049183584423421, -0.1284596949627963], [-0.17953391762002036, 0.43457481897735017, -0.48605959716918345, 0.70158090737543999, -0.5694023925121392], [0.07118011971686844, -0.88118363176359882, 0.57408382930309054, -0.82300502164821965, 0.89741192882682141]])
      res=outer(arg0,arg1)
      s0=numpy.array([0.74081372798542433, 0.22544703391695342])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.27595991246606105, 0.19158014185175953, -0.63345205377719949, -0.40974433864826515, 0.52088942892366052], [0.4584165632668582, -0.33468863980412883, -0.70270626868300923, 0.35365637980103226, 0.19623413361538447], [0.6895280922556678, -0.73086296374681714, 0.17598767901065898, 0.16810917623953339, 0.73938258682283164], [0.67507931836540791, 0.43089197116192063, 0.19221660377004349, -0.68151223851120069, -0.11531066430578907]], [[-0.083981089166154918, 0.058302341204344119, -0.19277435238819157, -0.12469483531807958, 0.1585188993039901], [0.13950693755090621, -0.10185362160435697, -0.21385003814690537, 0.10762595081596409, 0.059718660313111786], [0.20983960924184475, -0.222418782417212, 0.053557188183832601, 0.051159574567401736, 0.22501150401512451], [0.20544250765713995, 0.13113055707169044, 0.05849603152926347, -0.20740019649511207, -0.035091746095781237]]])+(1.-msk_ref)*numpy.array([[[-0.3022223606577194, -0.64100743638579283, -0.72002175850272476, -0.50629381452009159, 0.47660649456883974], [-0.43580678474303897, -0.57833168152530157, -0.70249134843617056, 0.077724960251782466, -0.095164705521259566], [-0.13300119081191533, 0.32193899173520168, -0.36007962220199641, 0.51974076747619635, -0.42182110912073773], [0.052731209845902113, -0.65279293128652705, 0.42528918176217051, -0.60969341823794243, 0.66481507653278782]], [[-0.091973369579624167, -0.1950736329426262, -0.21911954878528123, -0.15407711069887273, 0.14504256129040791], [-0.13262625039142828, -0.17599992723494548, -0.2137846328630289, 0.023653532714261776, -0.028960857207239028], [-0.040475389214924253, 0.097973603953440555, -0.10958069448866162, 0.15816933462055777, -0.12837008049707868], [0.016047346864021643, -0.19866023611727227, 0.12942549653606836, -0.18554404102934915, 0.20231885755569901]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.53686752514784009, 0.53732300062463301, 0.08280398123420829, -0.41872292572720937, 0.68137112731530625], [0.64363523760684593, -0.32717631424467775, -0.3732868714777533, 0.30865646183312712, 0.68906586934622904], [0.42909559767020711, 0.18926706547476946, -0.0065041966691554709, -0.81293636614644149, 0.88676736033239956], [-0.24830434712797, 0.16824779243268506, -0.33776025998906012, -0.40742433553166513, -0.3198842345543913]])+(1.-msk_arg1)*numpy.array([[0.7353838671709152, -0.10440764524380275, -0.82399152219279315, -0.62387581418076654, 0.53780979165836174], [0.09864992792444971, -0.17394061623578772, 0.58441943794331719, -0.013076167157401786, 0.80831020344489102], [0.30329129976712244, -0.89369679667501667, -0.47265969924256068, 0.80814623095292082, -0.72444517882447901], [0.99678172081037553, -0.0068194542052721907, -0.46694314396070546, -0.67665219583056291, -0.20355491929801262]])
      res=outer(arg0,arg1)
      s0=numpy.array([[-0.40649243838561944, 0.38171053383068276, 0.99516460754031844, -0.47370749880913676, 0.52314270055507306], [-0.47349626793409283, 0.99605807374875877, -0.17397416338267768, 0.6991320269762904, 0.85574577930502294], [0.78418165052276723, 0.024808749802585517, -0.90177616141957118, -0.47989175917679661, 0.98684086677378446], [0.69219903347674672, -0.17029624311976854, 0.3490363898669695, -0.66634253480184413, 0.51446055388690715]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.21823258938739837, -0.21841773672458478, -0.033659192239930399, 0.17020770308681396, -0.27697221098795716], [-0.26163285716571433, 0.13299469775933873, 0.15173829060433128, -0.12546651779402573, -0.28010006543885529], [-0.1744241157974972, -0.076935630950929729, 0.0026439067637846316, 0.33045248572721175, -0.36046422658229632], [0.10093383952579781, -0.068391455398959716, 0.13729699167271381, 0.16561491160790734, 0.13003052250513195]], [[-0.20492798962053954, 0.20510184940793294, 0.031607151880215484, -0.15983095150647841, 0.26008653674433968], [0.24568235013914749, -0.12488664556709116, -0.14248753098375869, 0.11781742281661271, 0.26302370083265258], [0.16379030965109062, 0.072245232598941042, -0.0024827203827230836, -0.31030637429213359, 0.33848844249610566], [-0.09478038489469659, 0.064221954665314121, -0.12892664914721433, -0.15551816061140311, -0.12210318193577605]], [[-0.53427155996489228, 0.5347248330389992, 0.082403591487716782, -0.41669823604945222, 0.67807643050404109], [0.6405230086321364, -0.32559428836179261, -0.37148188295411166, 0.30716398670494721, 0.68573396543736842], [0.42702075205275003, 0.18835188493350671, -0.0064727463256251507, -0.80900549977137604, 0.88247949212475651], [-0.24710369816016126, 0.16743424832579798, -0.33612705657472891, -0.40545427897174452, -0.31833746873865598]], [[0.25431817252963468, -0.25453393467851515, -0.039224866841895509, 0.1983521898402803, -0.32277061248129563], [-0.30489483855216343, 0.15498587349043844, 0.17682879022601422, -0.14621288052624842, -0.32641566948274559], [-0.20326580232236546, -0.089657228192998165, 0.0030810867359083566, 0.38509405269821939, -0.42006834828864154], [0.11762363122142633, -0.079700240933446048, 0.15999956795654141, 0.1929999629386796, 0.15153156065923595]], [[-0.28085832694615964, 0.28109660561712574, 0.043318298359575319, -0.21905184214925358, 0.35645433162398382], [0.33671307637405151, -0.17115990059161593, -0.19528230202662633, 0.16147137498715597, 0.36047977975011541], [0.22447822976148527, 0.099013683758604726, -0.0034026230104433042, -0.42528172596527708, 0.46390587164838509], [-0.12989860671609052, 0.088017604495664245, -0.1766968145508605, -0.21314106716189149, -0.16734510232977667]]], [[[0.25420476953251503, -0.25442043547091198, -0.039207376084482284, 0.19826374263027799, -0.32262668586184312], [-0.30475888291771464, 0.15491676375128688, 0.17674994051350956, -0.14614768275172746, -0.32627011749620066], [-0.20317516408379208, -0.089617249145140923, 0.003079712848754473, 0.38492233543824328, -0.4198810356431581], [0.11757118167690528, -0.079664701805026286, 0.15992822256126887, 0.19291390233977104, 0.15146399123245827]], [[-0.53475123295702087, 0.53520491298307504, 0.082477574046873878, -0.41707235083428873, 0.67868521258170422], [0.6410980749674996, -0.3258866093427723, -0.37181540215983144, 0.30743976082361185, 0.68634962250701881], [0.42740413446945885, 0.18852098866087907, -0.0064785576055620912, -0.80973183094414014, 0.88327178879596135], [-0.24732554970372894, 0.16758457204298127, -0.3364288339535832, -0.40581829884803833, -0.31862327449284311]], [[0.093401078534924145, -0.093480319499940526, -0.01440575335997633, 0.072846970692538329, -0.11854097182779237], [-0.11197590198626199, 0.056920225549345858, 0.064942271167079263, -0.053698249720075676, -0.11987965813506768], [-0.074651547615864344, -0.032927579371867494, 0.0011315621739927219, 0.1414299241836813, -0.15427460962889469], [0.043198541055870565, -0.029270768929458794, 0.058761558655512434, 0.070881307915864805, 0.055651592085908461]], [[-0.37534128107435399, 0.37565971856768221, 0.057890915241978758, -0.29274260780510658, 0.47636837736307008], [0.44998600830144048, -0.22873943975651331, -0.26097680709987969, 0.21579161780072417, 0.48174801795620881], [0.29999447496577469, 0.13232266712522983, -0.0045472922011591012, -0.56834984946670142, 0.61996746208560505], [-0.17359752151460209, 0.11762742015774928, -0.23613901519819044, -0.28484340153972132, -0.2236413133017707]], [[-0.45942211869119742, 0.45981188990803989, 0.070859157450826063, -0.35832037638931002, 0.58308046634037869], [0.55078813799404402, -0.27997975000345682, -0.31943866473706395, 0.26413146446892044, 0.58966520935618194], [0.36719674662464596, 0.16196449244148139, -0.0055659388473995831, -0.69566686417338008, 0.75884742582990738], [-0.21248539703784963, 0.14397733825165782, -0.28903691690260541, -0.34865165551737592, -0.27373958358613831]]], [[[-0.42100166198250649, 0.42135883749367059, 0.064933362674097703, -0.32835483500848511, 0.53431873523667539], [0.50472694296114995, -0.25656566211634696, -0.29272471499390468, 0.24204273368481913, 0.54035281074283137], [0.3364888940130763, 0.14841975979360536, -0.0051004716793430225, -0.63748978137469714, 0.69538669225517857], [-0.19471571276278965, 0.13193683156667491, -0.26486539815922017, -0.31949468790036289, -0.25084734702907457]], [[-0.013319012108526053, 0.013330311885671022, 0.0020542632530974598, -0.010387992300972936, 0.016903965820271073], [0.01596778557391592, -0.0081168353214283069, -0.0092607805990814768, 0.0076573809365693361, 0.017094862750111676], [0.010645325323991065, 0.0046954792732331269, -0.00016136098783188816, -0.020167934913150116, 0.021999589575585697], [-0.0061601204227921518, 0.0041740173872998245, -0.0083794097833248277, -0.010107688403689733, -0.0079359279408514737]], [[0.48413433601864431, -0.48454507294572741, -0.074670656347642556, 0.37759435266065505, -0.61444423969252282], [-0.58041491392347511, 0.29503980076696884, 0.33662120206952917, -0.27833903934922377, -0.62138317462428216], [-0.38694818094907607, -0.17067652778698425, 0.0058653295054289815, 0.73308663574191302, -0.79966566627271696], [0.22391494101685352, -0.15172184842726352, 0.30458415073301098, 0.36740555336466441, 0.28846397713509675]], [[0.25763830108809005, -0.25785688001591012, -0.039736948221326673, 0.20094168143488564, -0.32698438893961934], [-0.30887524644332476, 0.15700921700385881, 0.1791372934310618, -0.14812169245038515, -0.33067703222925054], [-0.20591944122097464, -0.090827705004917061, 0.0031213103815928797, 0.39012146284880828, -0.42555234853047952], [0.11915920995448749, -0.080740729088133761, 0.16208836534616225, 0.19551958110972822, 0.15350980805322986]], [[-0.52980281385959105, 0.53025229567390353, 0.081714352613486294, -0.41321289496269425, 0.67240487387446746], [0.63516555576609057, -0.32287095753706985, -0.36837473980438035, 0.30459481033073271, 0.679998359769864], [0.42344907153368228, 0.18677647494485211, -0.0064186070786565463, -0.8022388281998849, 0.87509827049712607], [-0.24503687714346459, 0.16603379731704671, -0.33331562772934287, -0.40206298442080163, -0.31567483529492407]]], [[[-0.37161918201238792, 0.37193446169719635, 0.057316835778345648, -0.28983960448292995, 0.4716444357666163], [0.44552368938303499, -0.2264711284966503, -0.25838881164645938, 0.21365170455724294, 0.47697072876327395], [0.2970195579764443, 0.13101047979061556, -0.0045021986479320923, -0.56271376692466546, 0.61381950974081301], [-0.17187602909005545, 0.1164609593065009, -0.2337973255112821, -0.28201873126992438, -0.22142355798299859]], [[0.091426522585685024, -0.091504088348216051, -0.014101206919545486, 0.071306941159461637, -0.11603494315207816], [-0.10960866290394543, 0.055716897153641434, 0.063569351818593273, -0.052563035864821778, -0.11734532881172009], [-0.073073368222467974, -0.032231470196656488, 0.0011076402572692888, 0.13844000905017559, -0.1510131499858417], [0.042285297466200182, -0.028651966964480888, 0.057519303351293211, 0.069382833696610594, 0.054475083377855685]], [[-0.18738630281441657, 0.1875452803305093, 0.028901602676600351, -0.14614953835036038, 0.23782331843772175], [0.22465211972546262, -0.11419643957394347, -0.13029070200533044, 0.10773233714734674, 0.24050906341715267], [0.14976997831861869, 0.066061093254028869, -0.0022702013243867934, -0.28374437443132683, 0.30951407810228282], [-0.086667252909821468, 0.058724602073791626, -0.11789062178711057, -0.14220591921792125, -0.11165123840422364]], [[0.35773766755980457, -0.35804117024355081, -0.055175814747286688, 0.27901289570871302, -0.45402656411607123], [-0.42888153571473298, 0.21801149456092328, 0.24873692014873636, -0.20567092916085458, -0.45915389802560264], [-0.28592464822387809, -0.12611669616296448, 0.0043340228953747681, 0.54169407885061993, -0.59089081066343141], [0.16545574806756855, -0.11211066048440989, 0.22506402779644022, 0.27148416447812679, 0.21315247169612075]], [[-0.27619716435145086, 0.27643148851752364, 0.042599382049791863, -0.21541642829476643, 0.35053856756117879], [0.33112494084034905, -0.16831930784499372, -0.19204137065915569, 0.15879157431544358, 0.35449720880842417], [0.2207527588478482, 0.097370439336699421, -0.0033461526210031003, -0.41822369320250785, 0.45620682736543683], [-0.12774279195598232, 0.086556852485168548, -0.17376433043495762, -0.20960374932462555, -0.16456782048854146]]]])+(1.-msk_ref)*numpy.array([[[[-0.29892798131575177, 0.042440918301254099, 0.33494632306522676, 0.25360080095615339, -0.21861561359886944], [-0.04010044974857517, 0.070705545227982614, -0.23756208236953219, 0.0053153630725502055, -0.32857198557028988], [-0.12328561998348145, 0.3632809900578447, 0.192132593671722, -0.32850533199220072, 0.29448148721706857], [-0.40518423223042327, 0.0027720565683601595, 0.18980885717603449, 0.27505400102214916, 0.082743535490837128]], [[0.28070376850828194, -0.039853498002016492, -0.31452624380816796, -0.23813997007499224, 0.20528766267328152], [0.03765571665040008, -0.066394965478200452, 0.22307905563837119, -0.0049913107461110771, 0.30854051925773712], [0.11576948394030993, -0.34113348134159177, -0.18041918611812779, 0.30847792923029366, -0.27652835594015635], [0.38048208276319501, -0.0026030575051283426, -0.17823711674981824, -0.25828527088818787, -0.077699056909105946]], [[0.73182799756462547, -0.10390279330325776, -0.82000719979954062, -0.62085912977309921, 0.53520927024703402], [0.098172916806815697, -0.17309954509160883, 0.58159354059979473, -0.01301293875732735, 0.80440170638206987], [0.30182476730314145, -0.88937542192313268, -0.47037420409684783, 0.80423852676145102, -0.72094220206933846], [0.99196188999362067, -0.0067864794678288735, -0.46468529060329788, -0.67338031690501687, -0.20257065137610791]], [[-0.34835685238212472, 0.049458684484993465, 0.39033096301788134, 0.29553465150308472, -0.25476453124154547], [-0.046731210614792691, 0.08239697425837493, -0.27684387020357032, 0.0061942784381429795, -0.3829026047357838], [-0.1436713630232557, 0.42335087424665979, 0.22390244391607225, -0.38282492973673909, 0.34317511368528175], [-0.47218297582375024, 0.003230426594822939, 0.22119446881170046, 0.32053521925060613, 0.096425491690957238]], [[0.38471070221642573, -0.054620097491439, -0.4310651501544232, -0.32637607824152115, 0.28135126679311656], [0.051607989703959932, -0.090995763713803565, 0.30573476302254488, -0.006840701399632723, 0.4228615827163808], [0.15866462961503064, -0.46753095568998626, -0.24726847150530182, 0.42277580170411477, -0.37898820725434079], [0.52145908128867269, -0.0035675476892577432, -0.2442778973372797, -0.3539856570633208, -0.10648827019283227]]], [[[-0.34820151660436899, 0.049436630366727338, 0.39015691056761981, 0.29540286966893653, -0.25465092920864629], [-0.046710372704194189, 0.082360232629801758, -0.27672042277430087, 0.0061915163479121012, -0.38273186466420323], [-0.14360729853661267, 0.42316209789027415, 0.22380260359420326, -0.38265422430121149, 0.34302208849623733], [-0.47197242474863571, 0.0032289861155438372, 0.22109583600280588, 0.32039228941518044, 0.096382494607234429]], [[0.73248503820017485, -0.10399607800618592, -0.82074340838066118, -0.62141654173133287, 0.53568978512244902], [0.09826105718388127, -0.1732549551544908, 0.58211569961915277, -0.013024621870818405, 0.80512390423478541], [0.30209574783079735, -0.8901739098115532, -0.47079650956621266, 0.80496057811028587, -0.7215894693564856], [0.99285248077835575, -0.0067925724197212909, -0.46510248852368963, -0.67398488277685842, -0.2027525208180625]], [[-0.12793779305617814, 0.018164232732045989, 0.14335323570791028, 0.10853827282678574, -0.093565008562775664], [-0.017162538678417591, 0.030261173187888572, -0.10167388278076332, 0.0022749152414610224, -0.14062509139800691], [-0.052764850138230034, 0.15548015251931502, 0.082230575740432549, -0.14059656442089863, 0.12603474390260305], [-0.17341426595313089, 0.0011864088400887125, 0.081236042817840956, 0.11771999967067397, 0.035413296787300215]], [[0.51413041366086509, -0.072994728651121263, -0.57607886312192647, -0.43617156254968281, 0.37600004976980689], [0.068969324070885477, -0.12160745560243132, 0.4085863462536557, -0.0091419672498351083, 0.56511555096004429], [0.21204066117046202, -0.62481205296162212, -0.33045153360145524, 0.5650009125393648, -0.50648282630475916], [0.69688202492307261, -0.0047676988414039343, -0.32645490671992977, -0.47306922122897926, -0.14231176332981477]], [[0.62930164050051629, -0.08934640174456035, -0.70512726729960384, -0.53387909479567575, 0.46022845928055678], [0.084419259450092557, -0.14884894819349009, 0.50011446736380749, -0.011189874854433537, 0.69170804496714988], [0.25954024967564954, -0.76477726173306482, -0.40447654267440286, 0.69156772619922424, -0.61994090411692049], [0.85299175047187659, -0.0058357191533255663, -0.39958462461979138, -0.57904226063948006, -0.17419126304604884]]], [[[0.57667453472590369, -0.081874559574480782, -0.64615903188991186, -0.48923196568550875, 0.42174057008995969], [0.077359463303747003, -0.13640103953272725, 0.45829099944397839, -0.010254090344002934, 0.63386202947180836], [0.23783547204057745, -0.70082062908352449, -0.37065106308762596, 0.63373344525241493, -0.56809661604384121], [0.78165793503600445, -0.0053476908543547723, -0.36616824533139614, -0.53061823575626554, -0.15962403258714422]], [[0.018243954369501016, -0.00259022314833061, -0.020442199513532596, -0.01547757898189497, 0.01334238856263294], [0.0024473813799207674, -0.0043152492287012014, 0.014498715615703409, -0.00032440335938476676, 0.020053165600141299], [0.0075242779732235037, -0.022171500228082629, -0.011726096220274007, 0.020049097647613503, -0.017972579187145822], [0.024728908319375256, -0.00016918213316878745, -0.011584275630553812, -0.016786895029730534, -0.0050499430639498813]], [[-0.66315164090726775, 0.094152325550812793, 0.74305591192528642, 0.56259633691444133, -0.48498404949553675], [-0.088960153328027619, 0.15685550122406339, -0.52701551740750785, 0.011791775825282447, -0.72891487249880649], [-0.27350086409594815, 0.80591446677856349, 0.42623324924068534, -0.72876700601441913, 0.6532873925192535], [-0.8988739939655751, 0.0061496212362069083, 0.42107819596207119, 0.61018881977220896, 0.18356097376263242]], [[-0.35290465768688623, 0.0501043685475554, 0.39542674113186593, 0.29939286197506432, -0.25809048702143766], [-0.047341287454328362, 0.083472668317688237, -0.28045807217173319, 0.0062751448604553952, -0.38790140549172308], [-0.14554699538826155, 0.42887772792704165, 0.22682549456148804, -0.38782271644409494, 0.34765527129322821], [-0.47834733351496567, 0.0032725998751936751, 0.22408216679084714, 0.32471981260797111, 0.097684328311014132]], [[0.72570685289040349, -0.1030337311302041, -0.81314850797498606, -0.61566614922534812, 0.53073268095956616], [0.097351780380135311, -0.17165170849329095, 0.57672898469943112, -0.012904096131689271, 0.79767354178965033], [0.29930024914713482, -0.88193652146372803, -0.46643990728956486, 0.79751172703354734, -0.71491210820123818], [0.98366493734877547, -0.0067297160988549383, -0.46079857692025861, -0.66774803943781724, -0.20087631299611852]]], [[[0.5090320020900998, -0.072270871125343319, -0.57036613525488467, -0.43184623558544505, 0.37227141798024849], [0.06828538476185482, -0.12040152644076198, 0.4045345700893877, -0.0090513102679338955, 0.55951154157394611], [0.20993794456070841, -0.61861605887971116, -0.32717458697911028, 0.55939803997348747, -0.50146025258919336], [0.68997134373223035, -0.0047204196097283459, -0.32321759293819374, -0.468377995953834, -0.14090051839752152]], [[-0.12523310983009375, 0.017780229738001179, 0.14032266059197207, 0.10624370732827136, -0.091586987032444445], [-0.016799712109569733, 0.029621433470892064, -0.099524434687913624, 0.0022268221413116272, -0.13765219092204073], [-0.051649368921252485, 0.15219320696192698, 0.080492171055127792, -0.13762426702268321, 0.12337029230003767], [-0.16974818226446495, 0.001161327431285161, 0.079518663167041384, 0.11523132684868677, 0.034664638024999223]], [[0.25667573016374728, -0.036442067570408181, -0.28760302618716138, -0.21775536190697106, 0.18771518811554158], [0.03443241470338667, -0.060711604742175324, 0.20398365078781885, -0.0045640581779165523, 0.28212967530304039], [0.10585970034877726, -0.31193270354712288, -0.16497543505923096, 0.28207244293640565, -0.25285772987342731], [0.34791309331703896, -0.0023802376766713291, -0.16298014924117726, -0.23617623962825735, -0.071048074171440642]], [[-0.49001755010305026, 0.069571254984447231, 0.54906059955317577, 0.41571499142277629, -0.35836553981488445], [-0.06573464303119704, 0.11590403112754959, -0.38942352966661903, 0.0087132063691557314, -0.53861146986966302], [-0.20209589347017032, 0.59550818884071888, 0.31495326209196517, -0.53850220802372584, 0.48272863678287858], [-0.66419805848892977, 0.004544092401106167, 0.31114407815511891, 0.45088213934897109, 0.13563730089642254]], [[0.3783259916242448, -0.05371361500215447, -0.4239111349054201, -0.3209594969200823, 0.27668192330236291], [0.050751496560915862, -0.089485585772093298, 0.3006607477465939, -0.0067271721985147064, 0.41584371497669725], [0.15603141006727381, -0.45977174902438372, -0.24316477067234671, 0.41575935759765603, -0.37269846795874095], [0.51280487619245019, -0.0035083401876507296, -0.24022382847571835, -0.34811086345578335, -0.10472097652846026]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[-0.18206432415722151, 0.64911697883449326], [-0.55876780182523622, -0.24794717410280542]], [[0.17766684732565152, -0.13347008256188309], [-0.52358198401429745, -0.39344993855458998]], [[-0.75111355080242093, 0.68434282168061378], [-0.84345862542490102, 0.21949644390669554]], [[-0.90124657080766557, 0.61137562975055282], [0.21850007208304723, -0.37722926669482559]], [[-0.36390149007085193, 0.78221926150697452], [-0.31346828895115175, 0.48618581229297275]], [[-0.82106784245014608, -0.86675666184252598], [0.08665551591794407, 0.33267947705969103]]])
      res=outer(arg0,arg1)
      s0=numpy.array(-0.926072087526)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.16860468873632203, -0.60112911563794913], [0.51745926467871373, 0.22961695711760327]], [[-0.16453230818706199, 0.12360291798037523], [0.4848746609272217, 0.3643630059342981]], [[0.6955852939608026, -0.63375078545732533], [0.78110348998920409, -0.20326953001324755]], [[0.8346192932036699, -0.56617790570573001], [-0.20234681787856929, 0.34934149448404883]], [[0.33699901256380288, -0.72439142440695414], [0.29029423272225424, -0.4502431101157669]], [[0.76036801085842698, 0.8026791512097351], [-0.080249254521788965, -0.30808517779778744]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_array_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[0.46918330738413117, -0.24213524189525071], [0.86422191862181474, -0.80319559061249479]], [[-0.033107170044501588, -0.46958791462673366], [-0.077326737052856087, -0.47772863959503575]], [[0.59857737492122376, 0.39086217883066499], [-0.97416243828913873, 0.90790508228090383]], [[0.51347375977647181, 0.36886253842667638], [-0.19267516790666539, 0.28119832863319472]], [[-0.098112210547213863, 0.90858524300514043], [0.66608767640719924, 0.2118568680934676]], [[0.1391770891635955, -0.60161731228857418], [-0.049510161766364691, -0.90395108303532767]]])
      res=outer(arg0,arg1)
      s0=numpy.array([0.049155314751041512, 0.40248250800352303])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.023062853150401626, -0.011902234027680621], [0.04248110042460429, -0.039481332063205865]], [[-0.0016273933640537286, -0.023082741746762302], [-0.0038010200985041649, -0.023482901644880854]], [[0.029423259267104936, 0.019212953424699211], [-0.047885261272744666, 0.044628360083588073]], [[0.025239964278213151, 0.018131554176231423], [-0.0094710085231619091, 0.013822392351431494]], [[-0.0048227365903687524, 0.044661793598069217], [0.032741749385585765, 0.010413891033304284]], [[0.006841293623970306, -0.029572688345220498], [-0.0024336875850006377, -0.04443400000614639]]], [[[0.18883807426935298, -0.097455199434040227], [0.3478342052785246, -0.32327217572708783]], [[-0.013325056832410108, -0.18900092160711202], [-0.03112265906476247, -0.19227742100932116]], [[0.24091692309245924, 0.15731519001948757], [-0.39208334136543976, 0.3654159145455631]], [[0.20666420662883286, 0.14846071957451459], [-0.077548384809074597, 0.1131774085546871]], [[-0.039488448566812342, 0.36568966733969938], [0.2680886385506086, 0.085268683608030402]], [[0.056016343903193863, -0.24214044470824408], [-0.019926974079386598, -0.36382449901255959]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s0=numpy.array(-0.0960047603256)
      s1=numpy.array([[[-0.086736153263994309, 0.57926128569497526], [-0.38244717087730806, -0.0040451107243879569]], [[-0.019374467367295978, 0.27865402429565989], [-0.11512987096258964, -0.35611747237492963]], [[-0.35880697532969941, 0.50391757711607332], [0.43796387141458215, -0.50934947350913617]], [[0.42986805708912867, 0.4828346691923211], [0.43191182138686912, 0.84641104138665302]], [[-0.72125714319774459, -0.13622812349072277], [-0.43441465150652236, -0.49609905386556674]], [[-0.88490560287187803, -0.69943298079513494], [0.28282210011575648, -0.7937127457300126]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.0083270836056703931, -0.055611840899019035], [0.036716748977262598, 0.00038834988558519862]], [[0.00186004109603254, -0.026752112816256252], [0.011053015668075511, 0.034188972583097631]], [[0.034447177669665173, -0.048378486214863077], [-0.042046616506409215, 0.048899974126192326]], [[-0.041269379792453861, -0.046354426692677496], [-0.041465590894020361, -0.081259489165229168]], [[0.069244119165794094, 0.013078548345326971], [0.041705874499793252, 0.047627870764098412]], [[0.084955150314455552, 0.06714889568502555], [-0.027152267936383367, 0.076200201921148167]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_Symbol_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s0=numpy.array([-0.76345631500829447, -0.54584378720011051])
      s1=numpy.array([[[0.72563993820952466, -0.55805672019507435], [-0.1982209228112235, -0.76203197323001692]], [[0.88460492334798135, -0.77371127898326852], [0.019321342646404949, -0.90340836099931732]], [[0.68175919448024147, -0.19330696147538884], [-0.71047253737448313, 0.21044217124803111]], [[-0.23617545393012485, 0.8559830410780036], [-0.93918098731917743, 0.02109427492851812]], [[-0.36000969402840877, 0.15648769695022136], [-0.92619787569198464, -0.72082396243335278]], [[0.14886215907356659, 0.53688739460653112], [0.90869511791119684, 0.20678900862285077]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.55399439324829014, 0.42605192716574636], [0.15133301528700027, 0.58177812220068803]], [[-0.67535721501744461, 0.5906947619329207], [-0.014751001057836931, 0.68971281823622177]], [[-0.52049336234090837, 0.14758142047345071], [0.54241474529851563, -0.16066340458336628]], [[0.18030964175290434, -0.65350565825100615], [0.71702365570455096, -0.016104557404698298]], [[0.27485167437019253, -0.11947152045775072], [0.70711161714431303, 0.55031760612904479]], [[-0.11364975541048369, -0.40989007186070631], [-0.69374902618651002, -0.15787437450742009]]], [[[-0.3960860520159411, 0.30461179362375179], [0.10819765920957901, 0.41595041823544565]], [[-0.48285610153612563, 0.42232549471966857], [-0.010546434843904683, 0.49311984115611196]], [[-0.37213402067359169, 0.1055154039438721], [0.38780702050215993, -0.11486855174063951]], [[0.12891490421692456, -0.46723302492108526], [0.51264610698463875, -0.01151417891522267]], [[0.19650905481721964, -0.085417837153532003], [0.50555935616441006, 0.39345728155921145]], [[-0.081255484679500889, -0.29305664877202914], [-0.49600558457089866, -0.11287449561805317]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[0.86508727790541973, 0.30622891662585294], [-0.10736062151882364, 0.36048329169192517]], [[-0.16018313758658942, -0.042279753274256482], [0.75269064898465232, 0.23024909962569517]], [[0.26712090352715667, -0.52804503764078259], [-0.3673855394486556, -0.78605951935076335]], [[-0.29251922208656556, -0.87920695633633628], [0.089527013791669718, -0.93182006208067114]], [[0.21884151204774116, 0.99340076115625275], [-0.50000620157336662, -0.73858436626246138]], [[0.86087509783599936, -0.069625078351980285], [0.32668574910102799, -0.71552914918575938]]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array(-0.681497792714)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.58955507039735755, -0.20869433074566568], [0.073166026589463029, -0.24566856759826206]], [[0.10916445469523613, 0.02881355853289104], [-0.51295701587937936, -0.15691425316925758]], [[-0.1820423061414807, 0.35986152760568363], [0.25037243420923783, 0.53569782737923788]], [[0.19935120417836052, 0.59917760008185628], [-0.061012462287283367, 0.63503331551443976]], [[-0.14914000741469224, -0.67700042600822308], [0.34075312271547475, 0.50334361534080907]], [[-0.5866844789775344, 0.047449337214401907], [-0.22263561692341405, 0.48763153579249846]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_constData_rank3(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[0.91507306413736567, -0.74679810688069326], [-0.97536729464520788, -0.023232617588867077]], [[-0.95511206150404093, -0.60116534538267752], [-0.54403182827348706, 0.0073699663861639397]], [[0.38920449255000111, -0.18698496607748094], [-0.96969916200323181, 0.49626403683605491]], [[-0.80773863628731379, 0.1621518336988037], [-0.6579460749095134, 0.029587928217427173]], [[-0.25794372025455026, 0.11227401716501872], [0.57328088075678996, 0.74246631020041698]], [[-0.82291696726753027, -0.89699440729235969], [-0.065603803157668006, 0.36769651571599549]]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array([0.38101228359140871, 0.580846755705277])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.34865407781996527, -0.28453925208435382], [-0.37162692027314503, -0.0088519126813401735]], [[-0.36390942763935263, -0.2290513810602719], [-0.20728280923689041, 0.0028080477227842445]], [[0.14829169249051133, -0.071243568922443101], [-0.36946729211152673, 0.18908269393919624]], [[-0.30775834235683974, 0.061781840446115538], [-0.25068553648127778, 0.011273364096860606]], [[-0.098279725892249703, 0.042777779668024804], [0.21842705751643862, 0.2828887843391481]], [[-0.31354147290471823, -0.34176588749118414], [-0.024995854853384358, 0.14009688912155574]]], [[[0.53151722053747574, -0.43377525754849339], [-0.56653892871570199, -0.013494590553034796]], [[-0.55477374225960119, -0.34918494050797055], [-0.31599912245306533, 0.0042808210650602694]], [[0.2260681668035868, -0.10860961091176607], [-0.56324661225970296, 0.28825335576942657]], [[-0.46917236634529097, 0.094185366535611742], [-0.38216584304021201, 0.017186052113133194]], [[-0.14982577306440506, 0.06521399862029971], [0.33298833969544522, 0.43125914750038002]], [[-0.47798865065217061, -0.52101629136154493], [-0.038105756226059065, 0.21357532823777037]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.46851724905930325, 0.84577180219587578], [-0.36334251552324859, -0.82050761117352855]], [[-0.10283290824427804, -0.91856848874519792], [-0.48206246767896443, -0.065748257178073644]], [[-0.97712924484478436, -0.96396534529930733], [-0.85909823362639726, -0.51597579428558293]], [[0.1020372702907375, 0.95229473167400758], [0.37641323957730344, -0.87417925464018098]], [[-0.14941592738194065, 0.99892690256630345], [-0.53560333925481962, -0.40901319624557431]], [[0.30028519815201915, -0.60372030350790018], [-0.16826894577142859, 0.5585494373536346]]])+(1.-msk_arg1)*numpy.array([[[-0.23804804009527158, -0.94872855834093595], [0.57347718195803155, -0.32757423961852195]], [[0.56324876684842851, -0.21103457362415434], [-0.87869630570195301, -0.67405204377987071]], [[-0.53089923066425193, 0.47057840146109098], [0.82698833468735411, -0.57906987684928213]], [[-0.11722672516894539, -0.04462255867848719], [-0.22033833324241736, -0.63284044528153705]], [[-0.51069309161562892, 0.67338525851336506], [-0.32156355728740849, 0.27388592216639851]], [[0.19830301012148444, 0.60714940443935905], [0.054878169528460363, 0.57388293233868581]]])
      res=outer(arg0,arg1)
      s0=numpy.array(0.418229056068)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.19594752682575967, 0.35372634248151308], [-0.15196039729674712, -0.34316012371790916]], [[-0.043007710147756069, -0.38417203198193145], [-0.20161253082329847, -0.027497831537717451]], [[-0.40866384172810372, -0.40315831644702177], [-0.35929984331945819, -0.21579606939811835]], [[0.042674951237474744, 0.39827732672677557], [0.15742695388000288, -0.36560716450259817]], [[-0.06249008227050934, 0.41778025554147319], [-0.2240048790035391, -0.17106120298523927]], [[0.12558799497438258, -0.25249337266533883], [-0.070374962355581994, 0.23360160395185567]]])+(1.-msk_ref)*numpy.array([[[-0.099558607107939345, -0.3967858494199093], [0.23984482048698033, -0.13700106502792503]], [[0.23556700009061696, -0.088260790524593014], [-0.36749632650437514, -0.28190815001092151]], [[-0.22203748410806348, 0.19680956064917213], [0.34587055059573568, -0.2421838479922257]], [[-0.049027622613378709, -0.018662450595453229], [-0.092151893127624937, -0.26467226207190025]], [[-0.21358668964697433, 0.28162928103831064], [-0.13448722303025742, 0.11454705069803157]], [[0.082936080738598553, 0.25392752231106563], [0.022951645040640685, 0.2400145170856807]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank1_expandedData_rank3(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.37263302280859256, -0.61120704376135748], [-0.97057819659004774, 0.011515184698977388]], [[0.5137551947513026, -0.58525470079594633], [0.61862055757808565, -0.53325694196530593]], [[0.75752830825014894, 0.40314989046901695], [0.64469071765580876, 0.10422961189630242]], [[0.74210625677068931, -0.52410913313626972], [0.73013110243298396, 0.63433771817615447]], [[0.46821935277319993, 0.72124149705780671], [0.72490334911802101, 0.3250344383357151]], [[0.82776648366004624, 0.33509491283515969], [0.99823069141168985, -0.6295117915933468]]])+(1.-msk_arg1)*numpy.array([[[-0.18047919726300599, -0.46298101650946322], [0.36583152437234401, -0.64313604382112222]], [[-0.21953579236848375, -0.066687033685572494], [0.059594714133789584, 0.34182514986529799]], [[-0.70448303812928725, 0.089418128333621505], [0.11330071852217549, 0.2669748480393539]], [[0.26935680488016955, -0.13116557821823505], [0.71203726906106057, 0.2867360584047427]], [[0.20566966764921091, 0.28383714504875335], [-0.56559342786563183, 0.30606522845819861]], [[-0.62990605466839988, -0.24594700302788519], [0.41946149744086259, -0.76178433236048471]]])
      res=outer(arg0,arg1)
      s0=numpy.array([-0.79088772869978285, -0.19984658916272169])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.29471088504762216, 0.48339615060572882], [0.76761838542663419, -0.0091072182721327194]], [[-0.40632267908457231, 0.46287076102337699], [-0.48925940770992538, 0.42174637164433271]], [[-0.59911984313774924, -0.31884630119860707], [-0.50987797740063556, -0.08243392101592649]], [[-0.58692273187126831, 0.41451148189695647], [-0.57745172925629118, -0.50168991715694178]], [[-0.37030894044807844, -0.57042104945207983], [-0.57331716331081739, -0.25706574868454335]], [[-0.65467035415569985, -0.26502245451105116], [-0.78948840424900524, 0.49787315104299312]]], [[[0.074469438617691897, 0.12214764296793766], [0.1939667421042266, -0.0023012703856693929]], [[-0.10267222333567765, 0.11696115574551909], [-0.12362920841792149, 0.10656958099910982]], [[-0.1513894485979991, -0.080568130531557883], [-0.12883924098838062, -0.020829932427230278]], [[-0.14830740421193719, 0.10474142260631431], [-0.14591421046284961, -0.12677022935476828]], [[-0.093572040631701145, -0.14413765314961785], [-0.14486946179387014, -0.064957023861813651]], [[-0.16542630838268005, -0.066967575375886199], [-0.19949299887617158, 0.12580578438764445]]]])+(1.-msk_ref)*numpy.array([[[[0.14273878240089888, 0.36616600457828602], [-0.2893316633976224, 0.5086484049426514]], [[0.17362816419461724, 0.052741956605308338], [-0.047132728103785694, -0.27034531638942844]], [[0.55716698993359448, -0.070719700422363613], [-0.089608147932056792, -0.21114713118581427]], [[-0.21303099162150788, 0.10373724624061363], [-0.56314153847729831, -0.22677602996805524]], [[-0.16266161630952361, -0.22448331496823934], [0.44732090153217402, -0.24206323336928484]], [[0.49818496887093205, 0.19451646660524274], [-0.3317469509880136, 0.60248588037966422]]], [[[0.036068151987837763, 0.092525176996505962], [-0.073110182354012038, 0.12852854472525799]], [[0.043873479303976941, 0.013327176223441189], [-0.01190980035176529, -0.068312590290615982]], [[0.14078853229312968, -0.017869907956788781], [-0.022642762146342376, -0.053354012772900812]], [[-0.053830038723070635, 0.026212993422470455], [-0.14229821957859209, -0.057303223262150788]], [[-0.041102381573925366, -0.056723885315678055], [0.11303191741179838, -0.06116609196868017]], [[0.12588457651842663, 0.049151669669916437], [-0.083827949548644101, 0.15224000049984401]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[-0.49084496458913551, -0.63573427649552783, 0.39745491457353044, 0.62663243144012015], [-0.96745314643410629, -0.64202147024699863, -0.23227363854984162, -0.55552905804747077], [-0.12212669420115629, 0.28228519155481702, 0.70953411133363486, 0.40831664559220604]], [[-0.78008003113938273, -0.75033103100507192, 0.88001158558447901, 0.13155054771684216], [0.87903339267036684, 0.52863902256467044, -0.29683377872220085, 0.94767946484029797], [0.54613062140053592, 0.62464020685084964, 0.86415873976763047, 0.23611481654382604]]], [[[0.38033177922912498, 0.54524248880139292, -0.30181583422369296, 0.81963957788030384], [-0.85100828856260291, 0.18406304501855297, -0.12946783785231242, -0.6546649160849447], [0.87287883164421598, -0.31175588079069483, 0.83033861883162707, 0.86256894995670241]], [[-0.8562455476931139, 0.84366423843605931, -0.98152626870329018, -0.23984021709069303], [-0.68190429131978481, 0.046443352026970697, 0.39850819999950526, -0.072732809020822797], [-0.68773126368306703, 0.5091681804006456, -0.12274420814119313, 0.42525465394801198]]], [[[-0.64263434896929894, 0.22426699757657831, -0.94476764198013896, 0.98444425571442795], [0.2193524312219246, 0.90285876883445293, -0.94160259829942849, 0.090062357591226494], [-0.10697055477766715, 0.37936898054743917, -0.67251070806371205, -0.70648523910447469]], [[0.0083028974910694942, -0.99444162135925751, -0.12788485948274486, 0.13892368666107902], [0.98969735836207473, 0.78581854056237632, -0.057459139218135169, -0.56847478665863904], [-0.58503363696951616, -0.18530090447895642, 0.77913684534276717, 0.53764955391113078]]]])
      res=outer(arg0,arg1)
      s0=numpy.array(0.775735100818)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.38076566809159501, -0.49316139307076107, 0.308319728227335, 0.4861007723790739], [-0.7504873640858204, -0.49803858994942096, -0.18018281441784134, -0.43094338985182218], [-0.094737963438711575, 0.21897853153022417, 0.55041051538925689, 0.31674555423416612]], [[-0.60513546160207221, -0.58205811798364582, 0.68265587606444544, 0.10204837739579688], [0.68189705748559715, 0.41008384546557058, -0.23026438126327514, 0.73514822520110312], [0.42365269265197991, 0.48455533383646393, 0.67035826711645907, 0.183162551016265]]], [[[0.29503671110462121, 0.42296373702064394, -0.23412913659000634, 0.63582319058145775], [-0.66015700052512394, 0.14278416478434813, -0.10043274624906111, -0.50784655468120787], [0.67712274846748499, -0.24183997961579584, 0.64412281219248924, 0.66912501135719926]], [[-0.66421972626474124, 0.6544599630597967, -0.76140437900813085, -0.18605247498507654], [-0.52897709417522776, 0.036027738366971228, 0.30913679870344379, -0.056421392938549399], [-0.5334972811689227, 0.39497962975644807, -0.095216990677242624, 0.32988496185371452]]], [[[-0.49851402148685386, 0.17397178197523283, -0.73288942200111329, 0.76366796395640157], [0.17015938034862843, 0.70037923806627289, -0.7304341865223638, 0.069864532045943159], [-0.082980814095018507, 0.29428983437221612, -0.52169016192103523, -0.54804539818318798]], [[0.0064408490223168942, -0.77142327140280853, -0.099204774363951795, 0.10776798007805009], [0.7677429800683816, 0.60958702478786364, -0.0445730711542996, -0.44098584594117018], [-0.45383112735650977, -0.14374441581766281, 0.60440379927304444, 0.41707363090804139]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=outer(arg0,arg1)
      s0=numpy.array(0.824036379218)
      s1=numpy.array([[[[-0.71755665966259508, -0.92287371272965846, 0.13723511217235473, -0.054836537600281954], [-0.60703393301806718, 0.82908490038213589, 0.80735263199431495, -0.87496760256511252], [0.8910295108167352, -0.74772877342134958, -0.38572450362345889, 0.58583746220523514]], [[0.60375140267586969, 0.38473096619517122, -0.78527673735176573, -0.25866295899384628], [-0.72319230135210599, 0.32957418271642314, -0.21675780243836007, -0.035384796544362551], [-0.65644024333371176, 0.38572171859854265, -0.42148382727172984, 0.23278920653919877]]], [[[0.29473568253676352, -0.29175333161686279, -0.70179378618233446, -0.89162419665327919], [-0.38379077232884029, 0.13791261505296437, -0.73183322580274068, 0.13164106754781191], [0.90111041802609471, 0.96196596738063533, 0.33342843368356312, -0.83863874742539979]], [[-0.12717789529472467, -0.55079196853098233, 0.81589099822157873, 0.58117409511614193], [0.75672756693039789, 0.23912826694076506, 0.25685596939969635, -0.86726018558597562], [0.89054738275980894, -0.41840453236005226, 0.61114346650368301, 0.87428804755321909]]], [[[0.070395712868359883, 0.040400858319854294, -0.53796969032864017, -0.32818543392303301], [0.40857761482454169, -0.019578541069132926, -0.46747199858727106, -0.010209390798921802], [-0.87654648052185813, 0.03483492165028057, 0.027962856486791088, 0.88934394370801151]], [[-0.14174695509135971, 0.81140897084213703, -0.10890006867133262, 0.48199046700503567], [0.14376892833386101, 0.69433834787835358, -0.086376762998851664, -0.23467727750086431], [-0.42709311251570936, -0.63525709644607775, 0.11613407779398521, 0.022994640598969163]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.59129279171211813, -0.76048151271320819, 0.11308672493608146, -0.045187301892987329], [-0.50021804422666205, 0.68319611937520053, 0.66528793962070709, -0.72100513515079778], [0.73424073186979644, -0.61615571108723532, -0.31785102334153031, 0.48275138116585614]], [[0.49751311980880442, 0.31703231235650658, -0.64709659933146302, -0.21314768816709967], [-0.59593676548451258, 0.27158111620936853, -0.17861631468855393, -0.029158359623781646], [-0.54093064128968604, 0.31784872837968225, -0.34731800692393561, 0.19182677487758945]]], [[[0.24287292466393662, -0.24041535901034419, -0.57830361052337287, -0.73473077463331438], [-0.31625755840713227, 0.11364501195672878, -0.60305720158190979, 0.10847702865848935], [0.74254776614582951, 0.79269495269126677, 0.27475715922092797, -0.69106883690033416]], [[-0.10479921235522915, -0.45387261945061802, 0.67232386401105859, 0.47890859703479544], [0.62357104430776178, 0.19705039125854024, 0.21165866300465178, -0.7146539431701866], [0.73384344081144759, -0.34478055589437245, 0.50360444932042392, 0.72044515709931767]]], [[[0.058008628344512314, 0.03329177700719161, -0.44330659574743425, -0.27043673668201995], [0.33668281834953656, -0.016133430092978948, -0.3852139331016507, -0.0084129094279649513], [-0.72230618802550151, 0.028705242707039457, 0.023042411011967523, 0.73285176325259482]], [[-0.11680464763865862, 0.66863051039774757, -0.089737618284515042, 0.39717767924841613], [0.11847082714828505, 0.5721600581378774, -0.071177595030143898, -0.19338261403654694], [-0.3519402620263854, -0.52347495762795737, 0.095698704969175583, 0.018948420380593467]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[0.86039641635936004, -0.0120302311439906, 0.67096925535396901, -0.43943108320557478], [0.87197606618025492, -0.89797346607945472, -0.75653291559800473, 0.3601386420220245], [0.37575219719139197, -0.51117008929463936, 0.16803062051210227, -0.62842131436685977]], [[-0.85559385167504831, -0.97075623667855315, 0.97400736046539071, -0.59254620173889516], [-0.5660939635063742, 0.88260257439764467, 0.041365088141428696, -0.55248636934786566], [-0.4039930252892685, -0.7755803688643379, -0.49589957266436024, 0.088069170430581156]]], [[[0.42463573915298891, -0.21263980732647925, 0.48608355981173323, 0.0060029795066731673], [0.072355568601896403, 0.65357116754267341, -0.30560262804356841, 0.83173860463265914], [-0.13518049554554756, -0.66359886798738943, 0.23373572480916227, -0.80416018813487034]], [[-0.87368663698083004, -0.10870147674243347, 0.023941225481552308, -0.1287150025514483], [-0.53039447427543807, 0.65206896850207174, -0.0398101170267946, -0.85512369226477958], [0.57459649991482942, -0.65402734411186825, -0.75099321120586726, 0.97107089726608176]]], [[[-0.66617809137806172, -0.088721113354887748, 0.16836305012814079, 0.10013838021442423], [-0.78360049735170789, 0.9008880075932173, 0.99567379511283249, -0.8811748785620217], [-0.87917035586942172, -0.365550866352091, -0.62101037230290546, -0.269736020973528]], [[0.21941782005281185, 0.85166674683917321, -0.58546102439084047, 0.01712973004054108], [-0.19591591942740716, 0.08002664800535042, 0.82688266589565651, -0.21602947020826369], [-0.87960577210088675, -0.048983196694969422, 0.23952024186085286, -0.15746782528961778]]]]),self.functionspace)
      res=outer(arg0,arg1)
      s0=numpy.array(0.643126053917)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.55334335205780449, -0.0077369550833480015, 0.43151780949564522, -0.28260957851061774], [0.5607905265528611, -0.57751013176214416, -0.48654602866711477, 0.23161454370676934], [0.24165602783046511, -0.32874680240862891, 0.10806486990722815, -0.40415412010629836]], [[-0.55025469758370038, -0.62431862781071235, 0.62640951022253788, -0.38108190048803581], [-0.36406977689627357, 0.56762471084963095, 0.026602965906339444, -0.35531837856180354], [-0.25981844016441002, -0.49879594212346628, -0.3189259353069186, 0.056639578050792172]]], [[[0.27309430727372902, -0.13675420019161924, 0.31261300169580686, 0.0038606725218732945], [0.046533751313882057, 0.42032864593585956, -0.19654101224042472, 0.53491276668810561], [-0.086938098666796762, -0.42677772135273478, 0.15032153435602272, -0.51717636851259419]], [[-0.56189063920178117, -0.069908751792347304, 0.015397225869895685, -0.082779971670871574], [-0.3411105052603175, 0.41936254259467898, -0.025602923469429391, -0.54995232581746212], [0.36953797958493179, -0.42062202497269535, -0.48298330044153087, 0.62452099423269292]]], [[[-0.42843648711414911, -0.057058859531080794, 0.10827866405439625, 0.064401601312975551], [-0.50395389570945848, 0.57938454934486849, 0.64034375883980477, -0.56670652246067255], [-0.56541736169139301, -0.23509528618308037, -0.39938795018089751, -0.17347426276806632]], [[0.14111331676970695, 0.54772907414728433, -0.37652523833887697, 0.011016575685642273], [-0.12599863216093329, 0.051467222339911918, 0.53178978597011284, -0.1389341807048913], [-0.5656973892141467, -0.031502369998691923, 0.15404170798129399, -0.1012716610974549]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.66055677207503227, -0.39538926366614091, -0.57632468525210223, -0.77233011880294966], [-0.70899802155122194, 0.65412748295138168, -0.34356819556419582, 0.41155936649636238], [-0.5830906245099805, 0.17687271522883652, 0.66650984395097157, -0.17958050837208162]], [[0.22092518585805321, 0.89639997757050871, 0.83553624435412477, -0.15441071844060872], [-0.33701337430601108, 0.42738536479854439, 0.86472511796016271, -0.094920174006458913], [-0.19088743746592796, 0.76930522165853565, 0.66959429992389485, 0.55145410418430818]]], [[[0.62526663170664309, 0.61584008982360472, -0.90721608192836922, -0.58014031076496453], [-0.45092442093365959, -0.24798724062428734, 0.2130960069836898, 0.48555084781544267], [-0.97417099556511055, -0.89573136176831691, -0.44947150480363129, -0.56535014058258448]], [[-0.22407427064503582, 0.42499780514031849, -0.74704723132086825, -0.63094568072438362], [-0.6407784560489973, 0.9266923585895912, -0.3051621050536939, -0.043656146656939621], [-0.6757488224193724, 0.3091989176599097, 0.83302586171831061, -0.38415318169606327]]], [[[0.86941214333266559, 0.91505176052050086, -0.85830384056119891, -0.48162400968963026], [0.70209504881792717, 0.21701974208126718, -0.23438102444153075, -0.69701646828766628], [-0.20830521431426918, -0.96828565232147157, 0.89412386900179941, -0.81373620766143229]], [[-0.18326517123943686, 0.24559500155415126, 0.68774207336015047, -0.75698760304890667], [0.93895007796777596, -0.28938088101318904, 0.17689423244989255, -0.58596454395126729], [0.75732931948821092, 0.70449439332517283, 0.28483794184612043, 0.062490709531508593]]]])+(1.-msk_arg1)*numpy.array([[[[-0.37256803195482036, -0.16950885657059711, 0.72085956494261105, -0.82200949608587548], [0.67706150550007549, 0.8951969492406977, -0.62000205940970088, -0.30902647049495169], [-0.075931880374245919, -0.19959349914393121, 0.23750236188375329, 0.50612686473254631]], [[0.74313205633459933, 0.54931582282003877, -0.86540822259462558, 0.65607212029551776], [-0.2412662199764517, 0.40068128843399187, -0.27701423905858635, 0.80574766172024015], [-0.28737948354543374, 0.003257377707307052, 0.35901208704047227, -0.46570915932654211]]], [[[-0.14373906745932752, 0.15139835010070923, 0.88696945536060556, 0.26574345731353599], [-0.29590907549968004, 0.16495046111897338, -0.17517455731266129, -0.91055087647987509], [0.030704254378940199, 0.87390430650365825, 0.0068182999971446723, 0.080893159202821696]], [[-0.17930739798281881, 0.20084412540854601, -0.39175591543496568, 0.043701384416751532], [0.56023445893725232, 0.78794519990731615, -0.029196440790060896, 0.10899047006938356], [-0.96810799466177744, -0.20482686265058869, 0.86543186186537202, 0.42225762969100566]]], [[[-0.69981007190169642, -0.94616184994813257, -0.34993801748879005, -0.43972013911213126], [0.2391621718893413, -0.18990111737700688, -0.46804009258406043, 0.5441461842446178], [-0.30666942251665086, 0.67694842160939506, -0.40596762460223035, -0.65635128781816654]], [[-0.021485792751663624, 0.36655081149262059, -0.24531008723736125, 0.85607000033831704], [-0.30228352700408112, -0.15845337958341599, 0.84844967876578492, -0.21738785518874826], [0.88441923369913966, 0.21004666523979543, 0.96445398791814618, 0.9971158933069606]]]])
      res=outer(arg0,arg1)
      s0=numpy.array(0.25604156768)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.16912999146379323, -0.10123608691295641, -0.14756307590468451, -0.1977486143848598], [-0.1815329649200593, 0.16748382619750515, -0.08796773939727752, 0.10537630539115461], [-0.14929543759909994, 0.045286767287025985, 0.17065422531942098, -0.045980074888375862]], [[0.056566030927111884, 0.22951565552555336, 0.21393200985796745, -0.039535562416142922], [-0.086289432686469089, 0.10942841880654779, 0.2214055748148735, -0.024303510157080967], [-0.048875118739211937, 0.19697411497793299, 0.17144397426216851, 0.14119517333897166]]], [[[0.16009424860022031, 0.15768066203868511, -0.23228502784153146, -0.14854003464267793], [-0.1154553956410924, -0.063495041854102999, 0.054561435694472006, 0.12432120026306426], [-0.24942826889296796, -0.22934446208738449, -0.11508338871745226, -0.14475313628292596]], [[-0.057372327532728445, 0.10881710428872589, -0.19127514423846828, -0.1615483212136547], [-0.16406592042241491, 0.23727176425044194, -0.078134183774505053, -0.011177788228914878], [-0.17301978785023298, 0.079167775602631316, 0.21328924755241671, -0.098359182870755693]]], [[[0.22260564813900552, 0.23429128727209819, -0.21976146088313406, -0.12331576647330529], [0.17976551695977358, 0.055566074980017123, -0.060011284932475754, -0.17846518923921598], [-0.053334793628963255, -0.24792137638253117, 0.22893287711941093, -0.20835029428768692]], [[-0.046923501745306129, 0.062882529212319793, 0.17609055862269055, -0.1938202925990381], [0.24041024993621501, -0.074093534431263416, 0.045292276590037289, -0.1500312804382333], [0.19390778621185961, 0.18037984888881339, 0.072930353165051878, 0.016000219233888797]]]])+(1.-msk_ref)*numpy.array([[[[-0.095392902969199236, -0.043401313371995737, 0.18457001308509602, -0.21046860002575093], [0.17335588928408333, 0.22920763026602498, -0.15874629925614603, -0.079123621960173474], [-0.019441717687919222, -0.05110423241956747, 0.060810477064440989, 0.1295895158911316]], [[0.19027269669724209, 0.140647684426323, -0.22158047799637093, 0.16798173419166701], [-0.061774181191020891, 0.10259106523071884, -0.07092716003826853, 0.20630489446141964], [-0.07358109348606838, 0.00083402409470508508, 0.091922017581944557, -0.1192409032369461]]], [[[-0.03680317616916088, 0.038764270903965177, 0.22710104983488782, 0.068041371411285662], [-0.075765023581705115, 0.04223417465445619, -0.044851968272000116, -0.23313887386639004], [0.0078615654256322127, 0.22375582863958127, 0.0017457682201820963, 0.02071201129688582]], [[-0.045910147276159295, 0.051424444728941344, -0.10030579873591861, 0.011189370975855454], [0.14344330913470285, 0.20174672423027359, -0.0074755024705662086, 0.027906090818755235], [-0.24787588863683271, -0.052444191016051842, 0.22158653063231165, 0.10811550547096532]]], [[[-0.17918046788802902, -0.24225676333981688, -0.08959867858869365, -0.11258663375877871], [0.061235457420322846, -0.048622579797410213, -0.11983771904235876, 0.13932404206113272], [-0.078520119700712127, 0.17332693510743413, -0.10394458703051859, -0.16805321268181136]], [[-0.0055012560589855356, 0.093852244408980864, -0.062809579303994376, 0.21918950493052053], [-0.077397148137992686, -0.040570651712746687, 0.21723838584886299, -0.055660327237140171], [0.22644808708275435, 0.053780677453972942, 0.24694031102187974, 0.25530311648105064]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_float_rank0(self):
      arg0=Data(0.855868237407,self.functionspace)
      arg1=-0.331240406713
      res=outer(arg0,arg1)
      ref=Data(-0.28349814305116411,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([0.76708082216518658, 0.24900288446126861]),self.functionspace)
      arg1=0.0458016595743
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.035133574682782366, 0.011404745347113238]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[0.27130062800907284, 0.44990188647787566, 0.088311580580226368, -0.99526858922829242, 0.85698394752482043], [-0.62451037819636279, -0.939987131220962, 0.64573112346806516, -0.63920340736457271, -0.56916567178378163], [0.74097375520088504, -0.5438764905274347, 0.63856368133120744, -0.16653574269539528, 0.63460309111640489], [0.34795479077961011, -0.21677917281420012, -0.66518992587141823, -0.1291185912908297, -0.89585459262077771]]),self.functionspace)
      arg1=0.961957356457
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.26097963492462783, 0.4327864293811024, 0.084951974599460092, -0.95740594105835508, 0.82438201268673272], [-0.60075235248949543, -0.904227535852556, 0.62116580451310155, -0.61488641998648663, -0.54751310501498129], [0.71278515475677606, -0.52318599106667352, 0.61427103082257406, -0.16020028279880186, 0.61046111192953467], [0.3347176707047691, -0.20853232001520014, -0.63988434263284266, -0.12420657874752894, -0.86177391568700312]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[0.57447118168186195, -0.9350773708453195], [0.77222865652792017, -0.78557350460581077]], [[-0.23304322070648698, -0.23819108901843822], [0.30653294545689524, 0.79724898528053201]], [[-0.88280206793438487, 0.40332879737058502], [0.77270626667504283, 0.18441274328652613]], [[-0.76004623321329823, 0.1704480317704693], [-0.51375334514695004, -0.037200815454657032]], [[0.43916408941534923, 0.85295616833129495], [-0.30158632262856933, 0.53164939486293217]], [[0.072519001758186263, 0.1379034515343005], [-0.92697335821079752, 0.1044833237344418]]]),self.functionspace)
      arg1=0.835880092036
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.4801890242164556, -0.78161255880327773], [0.64549056049165143, -0.6566452533312096]], [[-0.19479618877258173, -0.19909918941096647], [0.25622478666067777, 0.66640455519216402]], [[-0.73791667379485848, 0.33713451226702801], [0.64588978530538588, 0.15414694083101407]], [[-0.63530751537019969, 0.14247411648371164], [-0.4294361934254064, -0.031095421046065351]], [[0.3670885194795544, 0.71296908048772056], [-0.25209000311566765, 0.44439514510908834]], [[0.060617189864015708, 0.11527074976061906], [-0.77483857597646988, 0.087335530259407157]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[-0.99003147993777652, 0.99248118182244327, -0.28393801264996088, -0.25782938592976867], [-0.25796348811729675, -0.9366938080443088, -0.41605694953379224, 0.5317307438090042], [0.25368074706940624, 0.054844991120822861, -0.26663260028699876, -0.42921257694890991]], [[0.17425190117495082, -0.020499236232457418, 0.1047698276591571, -0.64415325405356194], [0.78001969337194033, -0.6623207724498501, -0.83880153563372839, 0.2428790987632552], [0.43639348261735389, 0.74734546870049678, 0.53186214770861451, 0.032918407378838221]]], [[[0.87184649475290432, -0.070427396650536878, 0.08083953652836029, -0.93929653796714008], [-0.42554636590081141, 0.58060523059381164, -0.23351855337671146, -0.18988344428505166], [-0.84737202935955835, -0.27240603315182432, -0.29324807086273341, -0.1206112357169844]], [[-0.72707031300431013, -0.12170258962411951, 0.91097572803827753, -0.0017605112124874367], [0.75740702212793365, -0.20946431282305711, 0.65275147924456922, 0.046018209195791693], [-0.16409908608468871, 0.4434084280382713, -0.11952060793439201, -0.65385351680468551]]], [[[0.68236855318799572, 0.26826896833718883, -0.9998479914754328, 0.87396403269938672], [0.6901199070263444, 0.99922382186703151, -0.11227870700938802, 0.77890290565720632], [-0.54285727890099156, -0.72884280010118552, -0.92003470771768225, -0.69492614208004144]], [[-0.88632998203428381, 0.013555649733227204, -0.30928316046881354, 0.23624644768272107], [-0.94817677989959215, -0.62591771484007364, 0.72923780693280738, -0.51992672063312817], [-0.9931941841403773, 0.37974651536759785, -0.51815249801128216, -0.47193207425752437]]]]),self.functionspace)
      arg1=-0.874147612629
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.86543365461470378, -0.86757505566883686, 0.2482037358924544, 0.22538094217598878], [0.22549816728306471, 0.81880865606586484, 0.36369518915247584, -0.46481116026183678], [-0.22175441942054497, -0.047942618052900565, 0.23307625098981871, 0.37519514945003146]], [[-0.15232188340806688, 0.017919358413311041, -0.091584294723755344, 0.56308502919782411], [-0.68185235276432443, 0.57896612203132436, 0.73323635984337365, -0.21231218434126997], [-0.38147232099661249, -0.6532902572732936, -0.46492602666697119, -0.02877554722174502]]], [[[-0.7621225319668089, 0.061563940645709775, -0.070665687862263488, 0.8210838262142246], [0.37199033981494378, -0.50753467620322112, 0.20412968593872172, 0.16598615949946155], [0.74072823647285579, 0.23812308354527709, 0.25634210105258221, 0.10543202375817955]], [[0.63556677832580022, 0.10638602817063451, -0.79632725782720293, 0.0015389466734016624], [-0.66208554018122201, 0.18310272898515223, -0.57060114722138566, -0.040226707705941854], [0.14344682433545528, -0.38760441878902691, 0.10447865408576004, 0.57156449072358828]]], [[[-0.59649084170207478, -0.23450667821427396, 0.87401473473968649, -0.76397357270737676], [-0.60326666915450633, -0.87346911836662722, 0.098148163681275422, -0.68087611544967686], [0.47453739434932518, 0.63711619368994876, 0.80424614328680499, 0.60746802805242717]], [[0.77478323779636249, -0.011849638851929214, 0.27035913635002179, -0.20651426823382299], [0.82884646849904142, 0.54714447612935924, -0.63746148796878288, 0.45449270158323285], [0.86819832494285754, -0.33195450981259139, 0.45294176911407463, 0.41253829603504794]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_array_rank0(self):
      arg0=Data(0.72749091269,self.functionspace)
      arg1=numpy.array(0.544429243769)
      res=outer(arg0,arg1)
      ref=Data(0.39606732744475071,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([-0.71865788045453294, -0.23655700550904335]),self.functionspace)
      arg1=numpy.array(-0.0695063340636)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.049951274716309058, 0.016442210249995634]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[0.9481360650271351, 0.20080183704454591, -0.34235850408590363, -0.21548109592585418, -0.3378140780018144], [-0.44688833119774674, -0.64497216507845367, -0.6086273707942087, 0.14685104020157436, 0.57295381791887801], [0.064227911010919403, -0.91053622229373166, -0.75875775638735554, -0.13519195091034986, 0.1532033919276008], [-0.78483268225608693, -0.89080247571077642, 0.73468888286281442, -0.32300084310035859, -0.66646383399504394]]),self.functionspace)
      arg1=numpy.array(0.18612714645)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.17647386022947992, 0.037374672930944604, -0.063722211428272668, -0.040106881498517877, -0.062876370368989881], [-0.083178049867473028, -0.12004682862549122, -0.11328207577706595, 0.027332965065878692, 0.1066422591766607], [0.011954557798883093, -0.16947550879455675, -0.14122541604290076, -0.025162892045901657, 0.028515310165888364], [-0.14607866758873334, -0.16580252285431119, 0.13674554529552091, -0.060119225227093877, -0.12404701163337635]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[-0.26866259702851858, 0.40120836097358192], [-0.25200668304318952, -0.11455848141269542]], [[0.94398866644768331, 0.82628729899896092], [-0.24330240460710928, -0.094657592656990586]], [[-0.44847718163489403, -0.23055820282066608], [-0.26922667564184399, 0.77233417483910416]], [[-0.55642929541911657, 0.0075577282469232454], [0.084036871591784301, -0.80233885576629538]], [[-0.16845827560251547, -0.4472604377306062], [-0.13739162918347314, -0.99320600213394195]], [[-0.58138064046843674, -0.11018358249045401], [0.063922835096953401, -0.54131725355090565]]]),self.functionspace)
      arg1=numpy.array(-0.0182349338312)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.0048990446797230677, -0.0073160079148620613], [0.0045953251903030727, 0.0020889663283588564]], [[-0.017213570870039975, -0.015067294222775286], [0.0044366032489731432, 0.0017260749387172708]], [[0.0081779517318981317, 0.0042042135726663796], [0.0049093306159126299, -0.014083462573735854]], [[0.010146451383687458, -0.00013781467449654628], [-0.0015324067928540101, 0.014630595945068301]], [[0.0030718255089234531, 0.0081557644873139733], [0.002505327267116123, 0.018111045729624971]], [[0.010601437509660291, 0.0020091903359937642], [-0.0011656286682931991, 0.0098708843001668878]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[0.95462962342599722, -0.94724113177473135, 0.49513578884703269, 0.53035952876880899], [-0.61444141225101778, -0.25095323015371473, -0.30817817857191754, 0.40591894392835703], [-0.12576247497385351, -0.84457431957082418, 0.51631066561800831, -0.88990809350170674]], [[0.82039700166326313, -0.67660864343626925, -0.051274119820798436, 0.12256730192970489], [-0.093676451895046586, -0.014472530095783753, -0.29970090215351486, -0.054009917955756048], [-0.76514661163061826, -0.012158493375796775, 0.69391579183245367, 0.69139942245021779]]], [[[-0.30363471850512314, -0.69687459624795034, 0.24579045612403783, 0.5283226969362016], [0.072033061111246743, 0.24904040981223075, -0.43822789190142641, -0.78423278697539733], [-0.70651970260583874, -0.11576991731667685, -0.196095882982803, -0.49017897450254355]], [[0.22539170133650122, 0.85382114442168566, 0.41170067506447894, -0.29281346939592434], [-0.61023932590586205, -0.85662278469281516, 0.47503142183387981, -0.24738801228027674], [0.44543053294663659, -0.54165686048766837, 0.046479281819532092, 0.72957276124480908]]], [[[-0.37555395269063441, -0.74769587040326813, 0.16503520916337711, -0.17115445642731308], [0.90114286973352353, 0.46163510304640254, -0.87719837601530304, 0.29280571120417842], [0.14051749500984201, -0.67883102537652262, 0.30867803228715318, -0.95551022441636113]], [[0.15049183021264345, -0.53128173055033634, -0.69946290860329308, -0.75378481192125379], [-0.51780039536351685, 0.38122248695605632, -0.77441203786786561, 0.55202816275395561], [-0.45762426631314512, 0.96921999872840758, 0.52912481251169008, -0.73547276091528513]]]]),self.functionspace)
      arg1=numpy.array(0.0681689406737)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.065076090164669864, -0.064572424515626697, 0.033752882215332007, 0.036154047252364915], [-0.041885820179195378, -0.017107215858218408, -0.021008179971993609, 0.027671064406977381], [-0.0085730946954685194, -0.057573736685342103, 0.035196351133705256, -0.060664092030950717]], [[0.055925594535252726, -0.046123694473710092, -0.003495302432159468, 0.0083552831337797926], [-0.0063858244917548166, -0.00098657704549761542, -0.020430293018753081, -0.0036817988929165736], [-0.052159233974919353, -0.00082883161361609265, 0.047303504445960265, 0.047131966210829553]]], [[[-0.020698457112247038, -0.047505203008625298, 0.016755275021677719, 0.036015198584005634], [0.0049104174694365663, 0.016976820921840368, -0.029873531164582749, -0.053460318329685168], [-0.048162699691727581, -0.0078919126253580597, -0.013367648613408738, -0.033414981432352053]], [[0.015364713516749069, 0.058204082940020441, 0.028065198893786877, -0.019960784023706889], [-0.041599368404426747, -0.058395067789452075, 0.032382388813130387, -0.01686417873251525], [0.030364527574687536, -0.036924174388078795, 0.0031684434049112082, 0.049734202278434551]]], [[[-0.025601115120736075, -0.050969635431480281, 0.011250275382527588, -0.011667417986230454], [0.061429954825379572, 0.031469175952461063, -0.059797684053640755, 0.01996025515599403], [0.0095789287809408662, -0.046275191896349495, 0.021042254470253027, -0.065136119801339168]], [[0.010258868645640094, -0.036216912770899022, -0.047681645520021625, -0.05138471212458539], [-0.035297904432346656, 0.025987533096782377, -0.052790848266402686, 0.037631175076978191], [-0.031195761461139799, 0.066070700593066689, 0.036069877953084542, -0.050136399005945999]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_Symbol_rank0(self):
      arg0=Data(0.13526834139,self.functionspace)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.186437136791)
      sub=res.substitute({arg1:s1})
      ref=Data(0.025219042267186804,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-0.37878778164185922, -0.041532624459693412]),self.functionspace)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.841137822954)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.31861273001173479, -0.034934661319584476]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[0.48067065827537525, -0.0032158902833219205, 0.46763545396383388, 0.65530068408047781, -0.74820566585201309], [0.37708547444564489, -0.75639182538834082, 0.035169659524271202, -0.1012707877390715, 0.74831892537447309], [0.54590005868351144, -0.0047114367000242741, 0.91884100749636599, -0.13937112070758029, 0.35791907582899984], [-0.68701806072524962, 0.96221709753424522, 0.77287600630611686, 0.92836483164411421, 0.6352839934990282]]),self.functionspace)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.343855151627)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.16528108208383219, -0.0011058004409868068, 0.16079885992880083, 0.22532851608563748, -0.25727437267957365], [0.12966278299177048, -0.26008922580817878, 0.012093268608381152, -0.034822482073383752, 0.25731331754984232], [0.18771054745169702, -0.0016200517808669097, 0.31594821395358319, -0.047923477843301851, 0.12307231808930433], [-0.23623469944102976, 0.33086330597053581, 0.26575739633710427, 0.31922302994997415, 0.21844567391068639]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[-0.37643528314038566, -0.59304956435947886], [-0.088186629705936603, -0.067780905592197627]], [[0.43417687370262481, 0.5376674236540806], [-0.51663583121435663, 0.84820519919657467]], [[-0.66236171701786217, 0.20375262754694679], [0.96272678703502135, -0.0967213939270537]], [[0.37998426215660341, -0.44362745414313753], [0.1977757743822377, 0.49528879837658213]], [[0.045927458358524076, -0.72581328963738256], [0.2489277185551626, 0.99383152900538629]], [[0.33664416907072958, 0.16439585885306185], [-0.99788915854005578, -0.084499600074841696]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.557959416572)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.21003561095812501, -0.33089758892829324], [-0.049204560460175344, -0.037818994538944363]], [[0.24225307514017122, 0.29999660201180084], [-0.28826182696455238, 0.47326407807705734]], [[-0.36957095718691418, 0.11368569719110633], [0.53716247641229609, -0.053966612525569412]], [[0.21201579721944011, -0.24752611548902645], [0.11035085568638874, 0.27635104897684443]], [[0.025625657870356894, -0.40497435962627759], [0.13889156461363741, 0.55451766009470349]], [[0.18783378416706981, 0.091726217492507159], [-0.55678165270253299, -0.047147347558325954]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[0.18883207447106432, -0.43599063917090297, 0.12518088037274278, 0.20491911364547089], [0.25899270782736128, 0.99802102753401378, -0.096535168166835161, -0.9489574908051539], [-0.1664977816548685, -0.36690330037288454, -0.35520644081561903, 0.36039105437224817]], [[-0.37783943358668037, 0.66736445175844938, -0.38114439626018615, -0.027219852952713008], [0.39220951342919919, 0.59400619494213824, -0.72896307628054524, 0.85783661850033432], [-0.27571424669388889, -0.20383726369149535, 0.75409286551576549, -0.045683595120755927]]], [[[-0.6987821005206627, -0.92937088898496345, -0.17111860966440595, -0.96504251971824395], [-0.59237395589282182, 0.097964473906504201, 0.53223460624068819, -0.11247093659483332], [-0.42157667905055241, -0.94655372855454045, -0.92689998603794366, -0.69368060274724508]], [[-0.38946904544117467, -0.97505837833092412, 0.42583619578909548, 0.060867706966672985], [0.69630182388957262, 0.19011049546844228, 0.52182257803912391, 0.27315264335436185], [-0.36311249123763223, -0.57802550619147564, -0.43321660456243016, -0.5773879939234885]]], [[[-0.99169208833663935, 0.51039800836964155, 0.57534331567328234, 0.26422912452505054], [-0.47896721274669862, 0.61659399412722915, 0.06538053886589501, 0.3421646783944301], [-0.56674330148530982, 0.57872925362027283, 0.39773558206460113, 0.5183057761886547]], [[-0.69809218338147638, -0.90292170317526943, -0.73704911192921774, 0.91825867025217267], [-0.90363031912830971, -0.99204226562952913, -0.98136183906301144, -0.24372168415721984], [0.23561607701103471, -0.92018590632952413, -0.17878658423578808, 0.8845598381737021]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.881331923275)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.16642373536962454, 0.38425246855046558, -0.11032590605618922, -0.1806017565450074], [-0.22825854130373427, -0.87958779166562062, 0.085079525424169467, 0.83634653047769225], [0.14673981012693571, 0.32336359137364423, 0.31305477564375978, -0.31762414108106229]], [[0.33300195469215244, -0.58816959579375816, 0.33591472380154597, 0.023989725354082055], [-0.3456667647973774, -0.52351662222572437, 0.64245843001492242, -0.75603879683877384], [0.24299576731309167, 0.17964828764437496, -0.66460611549309889, 0.040262410749900307]]], [[[0.61585897260214362, 0.81908423302507871, 0.15081229336370516, 0.85052277994560277], [0.52207807784514548, -0.08633921820066022, -0.46907534915171217, 0.099124226861684914], [0.37154898535558528, 0.83422801807026281, 0.81690654737855539, 0.61136285975791338]], [[0.3432515028748177, 0.85935007587996837, -0.37530303343498828, -0.053644653246287782], [-0.61367302562861048, -0.16755044860599902, -0.45989889631163211, -0.24073814451519818], [0.32002263026770322, 0.50943233107384089, 0.38180762329374834, 0.50887047116058426]]], [[[0.8740098955105069, -0.449830058352236, -0.50706843094585108, -0.232873562502979], [0.42212909479579869, -0.543423970724074, -0.057621956063446701, -0.30156065408619542], [0.49948896390137149, -0.51005256614876271, -0.35053706549596686, -0.45679942657298039]], [[0.61525092660296399, 0.79577372122635859, 0.64958491136483854, -0.80929067991745374], [0.79639824708711504, 0.87431851793753679, 0.86490551705026841, 0.21479970064214776], [-0.20765597030668739, 0.81098921459611117, 0.15757032414032651, -0.77959082342960806]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_constData_rank0(self):
      arg0=Data(-0.761549634663,self.functionspace)
      arg1=Data(0.0997529748633,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(-0.075966841563735246,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([0.55868192127741945, 0.65764903734185398]),self.functionspace)
      arg1=Data(-0.995214717967,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([-0.55600847071728854, -0.65450200121934787]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[0.28355419590890651, -0.64569032712320906, 0.50141124723693098, -0.80757327909511289, 0.55012578249109922], [0.73848320842017556, 0.30326193928008305, 0.4209398421529722, -0.31301100573218643, -0.56419991074326048], [-0.64831653760568275, 0.50941318603805041, -0.53963934101870326, -0.050527384769094574, 0.97712479575240763], [-0.30360813532673103, 0.91177458447138848, -0.7012270814638073, 0.84659160172487069, -0.012813769261334373]]),self.functionspace)
      arg1=Data(-0.912549180814,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[-0.25875714919310722, 0.58922417907594948, -0.4575624229170972, 0.73695033428569789, -0.50201683215703385], [-0.67390224688888734, -0.27674143426217118, -0.38412830812876142, 0.28563793686674155, 0.51486016636421783], [0.59162072530037657, -0.46486458561498423, 0.49244743858174261, 0.046108723579722101, -0.89167443191712059], [0.27705735518094093, -0.83203915014659047, 0.63990419875454296, -0.77255647263822824, 0.011693194642573103]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-0.84881460202203995, 0.54149795397130895], [0.6989075063336776, -0.76126229191446915]], [[-0.64412555419430562, 0.73975769511094036], [0.18952045967857267, 0.44246537421386645]], [[-0.14357708781339995, -0.20689271372893048], [0.22560575090442936, 0.3430078217353314]], [[0.67664786945362243, 0.10124599613681817], [-0.17759393001224488, -0.95180121682446628]], [[0.001928327449682854, 0.37269203891262626], [-0.43934319126823351, 0.7467473182371025]], [[0.017458344573415996, -0.44215311866986218], [-0.65332164942710658, 0.92129804824183625]]]),self.functionspace)
      arg1=Data(-0.901088980022,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.7648574839634189, -0.4879378390277509], [-0.62977785201160286, 0.68596506215005937]], [[0.58041443863474806, -0.66658750695059343], [-0.17077479770497611, -0.39870067274521642]], [[0.12937573161223787, 0.18642874438788787], [-0.20329085596946309, -0.30908056822689572]], [[-0.60971993851970441, -0.091231651390189017], [0.16002793325274831, 0.85765758765160716]], [[-0.0017375946147822365, -0.33582868920592185], [0.39588730809929545, -0.67288577932408178]], [[-0.015731521904523791, 0.39841930271556358], [0.58870093870825346, -0.83017151858606031]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[0.36506979897896308, 0.61233242903390339, -0.24865941965977933, 0.57815860382927831], [-0.73624130099188112, 0.13597465903021155, -0.99478808372728134, 0.91700497409411619], [-0.49724567921575669, -0.54691125683738284, 0.9507305292564705, -0.43122965813968106]], [[0.08995318683066178, 0.76455311872055609, 0.071910852797488545, -0.35672973572366673], [-0.26623785199264605, 0.97237948150930698, -0.91926047313140868, 0.32892783516690316], [-0.087694443509900077, 0.60849775878089041, 0.72436347181268745, -0.83397913799482115]]], [[[0.53469221329579275, -0.24350908703846508, 0.90393177678133441, -0.3445113911551021], [-0.38976994345460647, -0.34763449585953965, -0.93882937967617619, 0.46110064961193653], [0.77347702173187827, 0.90339541842388771, 0.075137977932457023, -0.8468468433600711]], [[0.85846586709643158, 0.29065862494062333, 0.12332472946607553, 0.57985673762797574], [-0.83233995096314151, -0.15708811684665114, 0.963272973347304, 0.31822269071413434], [-0.19626619878721407, 0.87629225061940486, -0.20561672906490625, -0.98671240041738906]]], [[[-0.36810699494738608, 0.20825338962819551, -0.43615912691127967, -0.22491076709170787], [0.85985869915219282, 0.39753233629467633, 0.36508767523776808, 0.035424381439172814], [-0.10016681257513804, 0.77433650969734646, -0.94187407357980368, 0.93922055606159649]], [[0.46394552974372494, 0.28410240599782344, -0.36314219052725183, 0.52252472849970011], [0.80853947102134094, -0.10437974451611942, -0.84506815967908877, 0.64333966413214849], [-0.89726693506002442, 0.6859164881500488, 0.18387818447739512, -0.092837085709315481]]]]),self.functionspace)
      arg1=Data(-0.625847677008,self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.2284780856367617, -0.38322682826753468, 0.15562292016023038, -0.36183921914874262], [0.46077490794311682, -0.08509942448601282, 0.62258581131595925, -0.57390543284158413], [0.31120005323944666, 0.34228313962120194, -0.59501249319574878, 0.26988407980367352]], [[-0.056296993017436327, -0.47849379330048197, -0.045005240174972469, 0.22325847642233471], [0.16662434120119735, -0.6085614396728436, 0.57531703167456749, -0.20585872154247678], [0.054883363757180281, -0.38082690879759484, -0.45334119614342061, 0.52194390618719344]]], [[[-0.33463587960543817, 0.15239959645336235, -0.56572360267231248, 0.2156116538572152], [0.24393661367860511, 0.21756624168154021, 0.58756418637719687, -0.28857877042651042], [-0.48407879726996261, -0.56538792404026061, -0.04702492894410662, 0.52999712969845847]], [[-0.53726886871296053, -0.18190802522142877, -0.077182495453983466, -0.36290199224190617], [0.52091802479123506, 0.098313233014037937, -0.60286215269399968, -0.19915893175467633], [0.12283274458616834, -0.54842546943026704, 0.12868475223925496, 0.61753166367621082]]], [[[0.23037890767821731, -0.13033490012784815, 0.27296917638326201, 0.14075988111843279], [-0.53814056941952093, -0.24879468920558637, -0.22848927345180842, -0.022170266833151632], [0.062689166963445897, -0.48461670581656724, 0.58946970098398255, -0.58780900320931251]], [[-0.2903592320483564, -0.17780483082612158, 0.22727169636507724, -0.32702088751077341], [-0.50602254970798366, 0.065325820632101905, 0.52888394464858368, -0.40263263432421231], [0.56155242696340457, -0.42927924073019363, -0.11507973460762629, 0.058101874431367724]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_expandedData_rank0(self):
      arg0=Data(-0.642877126562,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.547089422365)+(1.-msk_arg1)*(0.222624469009)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.35171127582219308)+(1.-msk_ref)*numpy.array(-0.14312017893859127)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([-0.3829468436692347, -0.26589916801218894]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.230991317554)+(1.-msk_arg1)*(-0.606772663729)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.088457395972133118, 0.06142039915553054])+(1.-msk_ref)*numpy.array([0.23236167639977173, 0.161340346458065])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[-0.87174569179957362, -0.56872338095146957, 0.26797673469459404, -0.48582324665863297, -0.94500956212991372], [-0.35643086869262919, -0.44329490689890672, 0.81325997657563187, -0.71848389647741118, -0.73924415246545094], [0.75438515837096842, 0.27031972889573064, 0.84218967617015683, -0.039217444296754778, -0.94152621651398638], [-0.14387273018259394, -0.68139020051338228, 0.90325594873061155, -0.98700102624342323, -0.37376869960732795]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0527910607114)+(1.-msk_arg1)*(-0.405654453504)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.046020379740734634, 0.03002351053182907, -0.01414677607051794, 0.025647124509388729, 0.049888057167299285], [0.018816363628586782, 0.023402008343175927, -0.042932856797595058, 0.03792952699913682, 0.039025482933386632], [-0.039824792695377205, -0.014270465219636713, -0.044460086325253584, 0.0020703304828178165, 0.049704167657409905], [0.0075951940337911118, 0.035971311443487773, -0.04768383962741439, 0.05210483109867816, 0.019731646113009465]])+(1.-msk_ref)*numpy.array([[0.35362752220125299, 0.23070517229470502, -0.10870595586426984, 0.1970763636227523, 0.38334733748168076], [0.14458776925139519, 0.17982455319909638, -0.32990253135430569, 0.29145619237682907, 0.29987768267425641], [-0.30601969915035704, -0.10965640189649449, -0.34163799283335194, 0.015908730934015996, 0.38193430281948687], [0.058362613736320623, 0.27640896941210458, -0.36640979825637765, 0.40038136190847112, 0.1516209375760387]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[-0.77328627374203962, 0.079523600460909227], [-0.66474016911635925, -0.43502454702821791]], [[-0.0082597579492271755, 0.35076593869127404], [-0.44497667487629888, 0.1174660507470533]], [[-0.94247002006062308, -0.3077056658142272], [0.84316414408989693, -0.49022494198661248]], [[-0.41017795822495784, 0.50503730298836658], [0.83082960996619759, 0.8780217163336772]], [[-0.82118917871286246, 0.10197202621331503], [0.93307721066846172, 0.4777386015723819]], [[0.69942467243314588, -0.83580669402499019], [-0.72490505988022047, 0.23436457019346668]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.871700854773)+(1.-msk_arg1)*(0.012858077492)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.6740743058054266, -0.069320790496428114], [0.57945457362090469, 0.37921126949188244]], [[0.0072000380645622156, -0.30576296858255642], [0.38788654784386817, -0.10239525684305484]], [[0.82155192208509165, 0.26822729190886141], [-0.7349869051173944, 0.42732950096094091]], [[0.35755247679387892, -0.44024144870738124], [-0.72423488117853474, -0.76537228063762108]], [[0.71583130901461889, -0.088889102413116095], [-0.81336420210922156, -0.41644514734886523]], [[-0.60968908480953632, 0.72857340960686301], [0.63190036032710706, -0.20429579616623125]]])+(1.-msk_ref)*numpy.array([[[-0.009942974831298395, 0.0010225206171716164], [-0.008547280606563382, -0.0055935793366241457]], [[-0.00010620460777657516, 0.0045101756212571021], [-0.0057215445677053622, 0.0015103875831883589]], [[-0.012118352551854714, -0.0039565032957760695], [0.01084146990320918, -0.0063033502925898544]], [[-0.0052740999723792188, 0.0064938087781903336], [0.010682871507618564, 0.011289671268303747]], [[-0.010558914095506591, 0.0013111642150701356], [0.011997579080822433, 0.006142799959951809]], [[0.0089932566379832025, -0.010746867240130867], [-0.0093208854343046401, 0.0030134778049339369]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[-0.83504325403320423, 0.72986446282641149, 0.86742233864227591, -0.0059036297732719678], [0.29820188050100183, -0.17552230466048235, 0.76126162856103852, 0.46771314116943086], [-0.26052995472384133, -0.34668561183634683, 0.96178401739697628, -0.80594576244112615]], [[0.57742472247806353, -0.77451120347821623, 0.29491164193254282, -0.62602578123637809], [0.14785524502051084, 0.78988617504439018, -0.12588877364162165, 0.2155924612104001], [0.8585041497903676, 0.54583569168921997, 0.80556935761893178, -0.46444010468878583]]], [[[0.28107157933058624, 0.047773080942044244, -0.83081398509678506, 0.055612189618716279], [0.38529122516130831, 0.049472585315569528, -0.54818234037853664, -0.016806731189929369], [-0.28894913844719183, 0.99248029239537039, 0.058901579190173026, -0.54663411835204334]], [[0.45320846987436503, -0.98170747061466113, 0.24373236322199898, -0.46104918112498638], [0.68019717040089955, -0.47457064947511274, -0.72411938225628325, -0.72698055420509688], [-0.67048338311954292, -0.41460665211487036, -0.4987048196003625, -0.47176296727437927]]], [[[0.1963630166123862, -0.31000614317997255, 0.91699734932975696, 0.23826455365961907], [-0.4962649474436871, 0.36154518492720933, -0.41701367639334053, -0.6258638542675814], [0.5472178354039805, 0.61426522480721935, -0.16313628065434438, 0.5905860497489801]], [[0.054834609730624484, 0.78207162444878109, 0.66991726836949272, 0.68954181735556452], [-0.0052733385953620537, 0.25102504584418339, 0.58525398946693841, -0.11076231426807115], [-0.82109331075278047, -0.72719539932325872, -0.75213299670452183, 0.5381412754258712]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.767088501143)+(1.-msk_arg1)*(0.319392052905)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.64055207812595427, 0.5598706368270967, 0.66538970160711142, -0.0045286065140827374], [0.22874723355155543, -0.13464114159918497, 0.58395504163061207, 0.35877737242457119], [-0.19984953247198076, -0.26593854635140807, 0.73777346032839741, -0.61823172691356432]], [[0.44293586488864523, -0.59411863819461253, 0.22622332937967313, -0.48021717820552656], [0.11341805828892357, 0.60591260208842612, -0.096567830683489536, 0.16537849792762915], [0.65854866148779023, 0.41870428260826925, 0.61794299110268414, -0.35626666377644667]]], [[[0.21560677650261212, 0.036646181054818809, -0.63730785455658556, 0.04265947117990472], [0.29555246841256128, 0.037949851317392397, -0.42050436983406647, -0.01289225023759724], [-0.22164956151803497, 0.7613202199075908, 0.045182724095949089, -0.41931674652032708]], [[0.3476510058612664, -0.75305651219474512, 0.18696429318401911, -0.35366552530240108], [0.52177142792457676, -0.36403768819235283, -0.555463651583611, -0.55765842368533891], [-0.51432009339849838, -0.31803999533473809, -0.38255073258006228, -0.36188394746130614]]], [[[0.15062781209312515, -0.23780214771706604, 0.70341812224952238, 0.18276999934227742], [-0.38067913470441744, 0.27733715400130349, -0.31988639598072471, -0.48009296588973766], [0.41976450915878921, 0.47119579060167477, -0.12513996500919464, 0.4530317676979459]], [[0.042062998589029399, 0.59991815018493377, 0.51388583328340731, 0.52893959915074173], [-0.0040451173991361278, 0.19255842616598259, 0.44894160556819007, -0.084964497635031278], [-0.6298512370439433, -0.55782322890500768, -0.57695257310230985, 0.41280198436964627]]]])+(1.-msk_ref)*numpy.array([[[[-0.26670617917053724, 0.23311290912488289, 0.27704780147502889, -0.0018855724328792466], [0.095243310793489563, -0.056060429216212543, 0.24314091434427901, 0.14938386032897505], [-0.083211197082619412, -0.11072862927720316, 0.30718617176810004, -0.25741267159654352]], [[0.18442486751064571, -0.24737272327720133, 0.094192434742560657, -0.19994765944084364], [0.047223790239943692, 0.25228336700908505, -0.040207873851150808, 0.068858518776934732], [0.27419940282941913, 0.17433558211770256, 0.25729245088765929, -0.14833847848818729]]], [[[0.089772028735781384, 0.015258342395699169, -0.26535538428264493, 0.01776209140889061], [0.12305895537073776, 0.015801150586481249, -0.17508508306003137, -0.0053679363773820998], [-0.092288058513918331, 0.31699031805638828, 0.018812696296924055, -0.17459059324863618]], [[0.14475118358732478, -0.31354956439226267, 0.077846179848978347, -0.14725544444989982], [0.21724957063484182, -0.15157409398454316, -0.23127797604748224, -0.23219181162992941], [-0.21414706417356219, -0.13242206976723658, -0.15928235612601679, -0.15067734260254473]]], [[[0.06271678699054288, -0.099013498483561571, 0.29288166591131454, 0.07609980492795361], [-0.15850308034906932, 0.11547465883199276, -0.1331908541929305, -0.19989594125385893], [0.17477702783617033, 0.1961914311796239, -0.052104431581555574, 0.18862849084666464]], [[0.017513738572134958, 0.2497874616518197, 0.2139662516213636, 0.22023417660936923], [-0.0016842624396383862, 0.080175404722865939, 0.18692547316696756, -0.035376602938641172], [-0.26225067814828773, -0.23226043145327582, -0.24022530187540778, 0.17187804671144227]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_array_rank1(self):
      arg0=Data(0.094765405644,self.functionspace)
      arg1=numpy.array([0.43204953548936986, -0.029784398113217314])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.040943349488930969, -0.0028225305690599833]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([-0.77657105275802074, 0.22498494171069083]),self.functionspace)
      arg1=numpy.array([0.22091855299981256, -0.068451744610724008])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[-0.17155895327684303, 0.053157643375473115], [0.049703347769472994, -0.015400611771238836]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-0.19358546521433406, 0.53808130011597055, -0.33754340271808458, -0.26681469630310262, -0.027436007634214166], [0.48475778158319649, -0.51694798347645632, -0.36535005917852104, -0.3145900258061618, -0.45174258519432553], [-0.14077024477916988, 0.20624720031697841, -0.39314518635241624, -0.82591162366689486, -0.8368370323312504], [-0.16007342247871792, 0.84886675840320991, -0.98705708408575532, 0.56621771577631375, -0.89012344094023943]]),self.functionspace)
      arg1=numpy.array([0.093504543477614499, -0.20572310259722482])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.018101120548767929, 0.03982500252161994], [0.050313046321185106, -0.11069575450940593], [-0.031561841775035078, 0.069440476068388893], [-0.024948386370939966, 0.054889947142010558], [-0.0025653913686855419, 0.0056442206133916839]], [[0.045327055064157948, -0.099725874835443026], [-0.048336985196639454, 0.10634814304215551], [-0.034161890493007052, 0.075160947708285047], [-0.029415596745616123, 0.064718436154984627], [-0.042239984197992783, 0.092933886201467808]], [[-0.013162657473308326, 0.028959691509341617], [0.019285050309175173, -0.042429813951200129], [-0.03676086117030436, 0.080879047507583193], [-0.077226489323828348, 0.16990910169186516], [-0.078248064673295287, 0.17215671065943897]], [[-0.014967592291771834, 0.032930801115678199], [0.079372898717814624, -0.1746315032303572], [-0.092294322033783904, 0.2030604457786914], [0.0529439290226019, -0.11648406523501688], [-0.083230585983840435, 0.18311895596474367]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_array_rank1(self):
      arg0=Data(numpy.array([[[0.52793389561713155, 0.15497295848870052], [-0.93668753153367779, 0.10326774528137728]], [[-0.67071187831479984, -0.60665708959908526], [0.47550232291478522, 0.86657992322827249]], [[-0.48175286497054448, -0.85598675032131721], [0.16819608414084075, 0.72547085781731191]], [[0.078203982287778251, 0.094239263802861961], [-0.63867238064066001, 0.47206552186393114]], [[-0.26618052116044821, 0.49845726599722706], [-0.10756974178355838, 0.8587717008057989]], [[0.29948403844539584, 0.87726990965275875], [-0.26732263279729618, 0.59819481827892873]]]),self.functionspace)
      arg1=numpy.array([0.44042322360500719, -0.49121081330391614])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.23251434815804645, -0.25932683823679598], [0.068253689949198448, -0.076124392979348615]], [[-0.4125389421486792, 0.46011104417629545], [0.045481513271244951, -0.050726233147726979]]], [[[-0.29539708755757343, 0.32946092723961007], [-0.26718587102406083, 0.2979965223785534]], [[0.20942226588979879, -0.23357188276687302], [0.38166192329957543, -0.42567342888180493]]], [[[-0.21217514977127494, 0.23664221661367285], [-0.37699644393968895, 0.42046994780271041]], [[0.074077461575048104, -0.082619735285356294], [0.31951421383139034, -0.35635913009673148]]], [[[0.034442849977932184, -0.038414641743184604], [0.041505160354219132, -0.046291345417766132]], [[-0.28128614870924368, 0.31372277952924693], [0.20790861889209256, -0.23188368892751923]]], [[[-0.11723208319034543, 0.13075075028488403], [0.21953215591983727, -0.24484759902774436]], [[-0.047376212438673015, 0.052839420348393965], [0.37822300080964472, -0.42183794559520382]]], [[[0.13189972563036714, -0.14710979809630417], [0.3863700415809414, -0.43092446580758464]], [[-0.11773509567916281, 0.13131176787090398], [0.26345889021021729, -0.29383976320098087]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_Symbol_rank1(self):
      arg0=Data(0.160050694185,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.21517760971413735, 0.031654711132441227])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.034439325807763689, 0.0050663584909650698]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([0.55218278896308548, 0.78462062840604418]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.23472990213215428, -0.35573709390400765])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.12961381201236505, -0.19643190064953797], [-0.18417392331662014, -0.27911866216630243]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[0.22428640196734184, -0.400197568067443, -0.50810851247871436, 0.10146937527389244, -0.76342954810249708], [0.17564484454998208, 0.23135853180747268, 0.5576178174996882, -0.12701440531230945, -0.96396775645428501], [-0.53832441582431012, 0.38767488140957207, 0.47347205373899248, -0.4917227771122421, 0.6251184549408928], [0.11992205925875243, 0.37955996533464709, 0.62823121058933507, 0.684385363539463, -0.27322614822378566]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.17985168468961521, 0.42300246615168646])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.040338287246798658, 0.094873701156474061], [0.071976206825616576, -0.16928455824043578], [0.091384171974431147, -0.21493115385116113], [-0.018249438087412341, 0.042921795979727458], [0.13730409036806573, -0.32293258158042382]], [[-0.031590021199359858, 0.074298202411472031], [-0.041610221712889889, 0.097865229519794331], [-0.10028850389026532, 0.23587371197248913], [0.022843754775268468, -0.053727406683896764], [0.17337122498477187, -0.40776073827087084]], [[0.096818553095555135, -0.22771255548334912], [-0.069723980533358321, 0.16398743090131157], [-0.085154746518409818, 0.20027984638549765], [0.088437169863892912, -0.20799994738143446], [-0.11242860725168888, 0.26442664807692956]], [[-0.021568184389134493, 0.050727326812440965], [-0.068264499206168228, 0.16055480138900435], [-0.11298844159908834, 0.26574335139274824], [-0.12308786060948718, 0.28949669657531135], [0.049140183059302366, -0.11557533451578757]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_Symbol_rank1(self):
      arg0=Data(numpy.array([[[0.585681387533457, -0.042678497027160534], [0.72537053491754189, -0.42269212722581773]], [[-0.17301064618650686, -0.56365144219493746], [0.7659481649677089, 0.055324612840208287]], [[-0.48486920402571476, 0.34629412920590052], [0.78251610810736616, 0.79594065701895245]], [[-0.0049378100610935061, -0.28580197046661615], [0.80028366453072208, -0.96390382254518991]], [[0.7241590450107096, -0.13642710718524853], [0.77825241328937511, -0.7431327866109696]], [[0.53970093752904136, -0.50967836680432654], [0.20768939454873436, 0.33358440419719271]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([0.90237499100016239, 0.94341172058980005])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.5285042368044659, 0.55253868553036012], [-0.038512008370784445, -0.040263394312580185]], [[0.65455622991799989, 0.68432306441170188], [-0.38142680450123678, -0.39877270702587136]]], [[[-0.1561204802954814, -0.16322027139916556], [-0.50862496507788524, -0.53175537689404817]], [[0.69117246846932723, 0.72260447619478629], [0.049923547013770421, 0.052193888190545441]]], [[[-0.43753384361896025, -0.45743129003090638], [0.31248716172558355, 0.32669794026428517]], [[0.70612296601086666, 0.73823486793880433], [0.71823694321414056, 0.75089974472562582]]], [[[-0.0044557563094397637, -0.0046583878856818499], [-0.25790055052764144, -0.26962892870586558]], [[0.7221559645784873, 0.75499698891483891], [-0.86980270319423791, -0.90935816371044287]]], [[[0.65346301172422527, 0.68318013063422001], [-0.12310840961846684, -0.12870693192472438]], [[0.70227551443785452, 0.7342124482744935], [-0.67058444162999931, -0.70108018084334756]]], [[[0.4870126286455479, 0.50916019007820112], [-0.45992101165803162, -0.48083654497426892]], [[0.18741371553674335, 0.19593660905947533], [0.30101822373523629, 0.31470743672559687]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_constData_rank1(self):
      arg0=Data(-0.919108703748,self.functionspace)
      arg1=Data(numpy.array([-0.93688283577102016, 0.53170944224710404]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([0.86109716874912567, -0.4886987762342358]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([-0.29779080932637259, -0.053805000864227903]),self.functionspace)
      arg1=Data(numpy.array([0.5272408344369186, 0.62962043017047886]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[-0.15700747479688201, -0.18749517746888575], [-0.028368193552534646, -0.033876727789458161]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[-0.61169447974141988, 0.55875640915843894, 0.63173687966141756, -0.037640820319297719, -0.99090884499283916], [0.41618393390700192, 0.12956118624983004, -0.71693208691742694, 0.71158379483885348, -0.61635924994323665], [0.4413481405572679, 0.08995759548061022, 0.33046412157090854, 0.4466056588564935, 0.23129456001205373], [-0.64922128479527008, -0.03976505531208363, 0.086830616590950704, -0.25200669262905184, 0.37855163655036073]]),self.functionspace)
      arg1=Data(numpy.array([0.7021467630050735, 0.76928149433839033]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.42949929889851046, -0.47056524345402373], [0.39232900399893633, 0.42984096540855693], [0.44357200512519002, 0.48598349081460718], [-0.026429380144050489, -0.028956386503352194], [-0.6957634379448181, -0.76228783702921965]], [[0.29222220200751886, 0.32016259859560831], [0.090970967536415606, 0.09966902296652376], [-0.50339154412354337, -0.55152258716297897], [0.49963625815296731, 0.5474082450406158], [-0.43277465219587863, -0.47415376484562249]], [[0.30989116825059387, 0.33952095709136498], [0.063163434474430291, 0.069202713478412253], [0.23203431325032853, 0.25421993326729203], [0.31358271770583507, 0.34356546862510468], [0.16240272661314625, 0.1779306247584132]], [[-0.45584862359299383, -0.49943392012359505], [-0.027920904868097222, -0.030590521172928441], [0.060967836369070667, 0.066797186485410387], [-0.17694568348510326, -0.19386408508895242], [0.26579880623410884, 0.29121276864970469]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_constData_rank1(self):
      arg0=Data(numpy.array([[[0.38252460007169198, -0.051755825978095027], [0.011862285227803859, 0.6098177254103021]], [[-0.83769475721431519, -0.45494251353843351], [0.037872621083969049, 0.33679832205419591]], [[0.80112411616460832, -0.0098861829681524682], [0.75418188513559348, -0.019244760229890856]], [[-0.091164744830903643, 0.137855274995063], [0.9289097954794594, 0.12542265606508685]], [[-0.22835257905635764, -0.75524782434503912], [-0.40829474194126947, -0.792401786421711]], [[-0.84502099168268718, 0.21593138093987796], [0.75161492138409214, 0.14572990444308442]]]),self.functionspace)
      arg1=Data(numpy.array([-0.55350465181593345, -0.6950619290155835]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.21172914557371106, -0.26587828642174482], [0.02864709043745153, 0.035973504242129577]], [[-0.0065658300547568656, -0.0082450228529704107], [-0.33753694777441379, -0.42386108457158]]], [[[0.46366794491994245, 0.58224973387562284], [0.25181279755235625, 0.31621322105122179]], [[-0.020962671946439068, -0.026323817067499786], [-0.18641943798079832, -0.23409569141620115]]], [[[-0.44342592497903893, -0.55683087356227712], [0.0054720482615758436, 0.0068715094044450612]], [[-0.41744318173786099, -0.52420311591095481], [0.010652064310326861, 0.013376300168830322]]], [[[0.050460110345517742, 0.063365143400381338], [-0.076303535987132104, -0.095817953363042219]], [[-0.51415589291526809, -0.64564983432742418], [-0.069422023575135466, -0.087176513266857339]]], [[[0.12639421476185966, 0.15871918409459548], [0.41803318404884215, 0.5249440096740855]], [[0.22599303897647877, 0.28379013094061861], [0.4385980748916728, 0.55076831422566885]]], [[[0.46772304977848056, 0.5873419205376299], [-0.11951902382326084, -0.15008568217107038]], [[-0.41602235536036208, -0.52241891713412325], [-0.080662180017938692, -0.10129130849746691]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_expandedData_rank1(self):
      arg0=Data(0.94577337941,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.21258989853496657, 0.94855972467563565])+(1.-msk_arg1)*numpy.array([0.092694441816000728, 0.70626041960124941])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.20106186676583832, 0.89712253637866823])+(1.-msk_ref)*numpy.array([0.087667935488839999, 0.66796230378977828])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([0.80743269872209877, -0.67335052968308773]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.8696787996472739, -0.85444435201596103])+(1.-msk_arg1)*numpy.array([0.50146272797380598, 0.012381584608076812])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.70220710022059385, -0.6899063090561024], [0.58559868039664387, 0.57534055701467002]])+(1.-msk_ref)*numpy.array([[0.40489740375643585, 0.009997296274555461], [-0.33766019349748838, -0.008337146554164487]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[-0.55391791827152037, -0.3590000067754664, 0.024151024496113394, -0.54258909914201925, -0.59450570915289913], [-0.99285035958068391, -0.46489595794231819, -0.99762597209037041, 0.57921432631499536, -0.24793237890852571], [0.056552525067596671, -0.013315620681917606, -0.91314148739809764, -0.24033483034116565, -0.71124375711070598], [-0.63975316839291207, 0.60169935268198982, 0.20660245627981144, -0.72877339290849275, 0.80719704598441488]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.54261649421559821, 0.91431185496179834])+(1.-msk_arg1)*numpy.array([0.25664152255339001, -0.95718588355394463])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.30056499889569466, -0.50645371935141159], [-0.19479932509987957, -0.32823796212617484], [0.013104744243796084, 0.022081568006269269], [-0.29441779477604213, -0.49609564571859072], [-0.32258860369170422, -0.54356361772096662]], [[-0.53873698139636672, -0.90777485396770363], [-0.2522602148736629, -0.42505988567048314], [-0.54132830751410499, -0.91214125310001382], [0.3142912471444923, 0.52958252511351178], [-0.13453219824587753, -0.2266875132649456]], [[0.030686332891219043, 0.051706644097327913], [-0.0072252754127268446, -0.012174629845651772], [-0.49548563261477258, -0.83489608718553021], [-0.1304096430776239, -0.21974098454116026], [-0.38593259401614172, -0.6502985988938883]], [[-0.34714062139668322, -0.58493390611101115], [0.32649199332409612, 0.55014085127998347], [0.11210590052288269, 0.18889907504085823], [-0.39544446353761303, -0.66632615271696749], [0.43799843123325022, 0.73802982843369436]]])+(1.-msk_ref)*numpy.array([[[-0.14215833791480725, 0.53020241201708695], [-0.09213430833553303, 0.34362973868124691], [0.0061981556979067605, -0.023117019721045257], [-0.13925089252468009, 0.51935862626899254], [-0.15257485036368287, 0.5690524724933822]], [[-0.25480662795046749, 0.95034234867208855], [-0.11931160647523331, 0.44499184826367533], [-0.25603224841607841, 0.95491349755168409], [0.14865044659021648, -0.55441577670092168], [-0.063629743213368034, 0.23731737316718857]], [[0.014513726137586764, -0.054131278674034118], [-0.0034173411655507436, 0.012745524147490483], [-0.23435002163251498, 0.87404614142491133], [-0.061679896781367427, 0.23004510692889601], [-0.1825346807314851, 0.68079248407223825]], [[-0.16418722719471227, 0.61236270174460505], [0.15442103799169507, -0.57593812653074694], [0.053022768942921003, -0.19775695465860654], [-0.18703351315243549, 0.69757160400172169], [0.207160278882039, -0.77263761766272621]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank3_expandedData_rank1(self):
      arg0=Data(numpy.array([[[-0.33561040691589383, 0.39295988096864809], [-0.74025647782746185, 0.24467768682694735]], [[-0.36609502502031499, -0.85371902689107326], [-0.87097730147411467, -0.71741878239906942]], [[-0.3037948221187714, -0.87597603983988059], [-0.01638418537184827, -0.62994381943706013]], [[-0.36032580215061172, -0.83457235834116617], [0.33584754764220115, -0.67422681043058752]], [[0.3773462048543037, 0.10333686426053723], [-0.22292593411219541, -0.46206698428249293]], [[0.26067951497054764, -0.11270196798769594], [0.27159358928923694, -0.66048112318849106]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.23855386295122938, 0.34303848989880215])+(1.-msk_arg1)*numpy.array([-0.87981738521624453, -0.73300661027850178])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.080061159016420455, -0.11512728718275073], [0.093742097589926285, 0.13480036415829807]], [[-0.17659104236041209, -0.25393646429173861], [0.05836880737053942, 0.08393386420104805]]], [[[-0.087333382425823114, -0.12558468454243304], [-0.20365797173983, -0.29285848578258861]], [[-0.20777499980948755, -0.29877873823381401], [-0.17114302189506544, -0.2461022557392141]]], [[[-0.0724714283610145, -0.10421331701869856], [-0.20896746815652353, -0.3004934978942056]], [[-0.003908510711763429, -0.005620406208180875], [-0.15027553156896242, -0.21609497654077278]]], [[[-0.085957112024028823, -0.12360561904132039], [-0.19909045999460284, -0.28629044151663563]], [[0.080117729852744138, 0.11520863557939669], [-0.16083941013350289, -0.23128574689939468]]], [[[0.090017394837980089, 0.12944427228226438], [0.024651408154617992, 0.035448521866812188]], [[-0.053179842734475458, -0.076472175797127381], [-0.11022786404281368, -0.15850676052035992]]], [[[0.06218610528847697, 0.089423107163048854], [-0.026885489825670659, -0.038661112907122354]], [[0.064789699877737111, 0.093167054735975321], [-0.15756032334298134, -0.22657044710524468]]]])+(1.-msk_ref)*numpy.array([[[[0.29527587066410155, 0.246004646747608], [-0.34573293496872265, -0.28804219032427225]], [[0.65129051871154442, 0.54261289154901071], [-0.21527168264484398, -0.1793503618318055]]], [[[0.32209676765404915, 0.26835007332996441], [0.7511168419486608, 0.62578169003168671]], [[0.76630097196565627, 0.63843211938305755], [0.63119751723537121, 0.52587270983647194]]], [[[0.26728396603877158, 0.22268361278144105], [0.77069894888400459, 0.64209622764821672]], [[0.014415091132757789, 0.012009716181593114], [0.55423552405024834, 0.46175298375145202]]], [[[0.31702090507409708, 0.264121194830302], [0.73427127008947946, 0.61174705541979335]], [[-0.29548451119784952, -0.2461784724675575], [0.59319646939572812, 0.49421270887261098]]], [[[-0.33199575127618686, -0.2765972625217103], [-0.090917569710151858, -0.075746604588426053]], [[0.19613411244748058, 0.163406183306749], [0.40653456590617848, 0.33869815387051988]]], [[[-0.22935036924082611, -0.1910798076376051], [0.099157150783659539, 0.082611287526377225]], [[-0.23895276156995107, -0.19907989625827519], [0.58110277478838646, 0.4841370292613334]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_array_rank2(self):
      arg0=Data(-0.53334445403,self.functionspace)
      arg1=numpy.array([[-0.14622035743864048, 0.50649767048635241, -0.024778745805814406, -0.014030242656037251, 0.47192766812552156], [0.93282861324086097, 0.32755079828488132, -0.33403983401182868, -0.2189428902004753, 0.20544718376209503], [0.009457548649367542, 0.38281810872422306, -0.0070076677237429497, 0.017701352393783898, 0.67621360244218298], [0.6183268723103903, 0.30250428096384541, 0.31053406436293884, 0.97314794751525824, 0.69542049607892209]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[0.077985816706134276, -0.27013772353284116, 0.013215606653341953, 0.007482952109287914, -0.25170000449789953], [-0.49751896743219715, -0.17469740167823117, 0.17815829289519891, 0.11677197623764954, -0.10957411605552697], [-0.0050441311208559341, -0.20417391519018996, 0.0037375007161409937, -0.0094409181280493883, -0.36065477460195949], [-0.32978120812425593, -0.16133898057229873, -0.16562162101526468, -0.51902306075761517, -0.37089866480225198]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_array_rank2(self):
      arg0=Data(numpy.array([0.6082991034824301, -0.89790957751207823]),self.functionspace)
      arg1=numpy.array([[0.76657659534322575, 0.68288841558950208, -0.52316353807759675, -0.14719334499260706, -0.41525172281721101], [-0.64498670026315219, 0.900497810651681, 0.80284887559478979, 0.30499901681381969, -0.83593848020492501], [0.89752625807910302, 0.44866872380829048, -0.5104140244105988, 0.36221640249806852, -0.96865683502897593], [-0.15265985147484296, 0.59080359638445579, 0.25838669533340086, 0.84017981147952447, 0.59099373775915232]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.46630785569789784, 0.41540041098163127, -0.31823991118729827, -0.089537579797582909, -0.25259725070924405], [-0.39234483152816635, 0.54777201090730865, 0.48837225125618766, 0.18553062849086915, -0.50850062807512098], [0.54596441814145857, 0.27292478245318913, -0.31048439345382645, 0.22033591290620613, -0.58923308433025423], [-0.0928628507899079, 0.35938529801485997, 0.15717639512309556, 0.51108062608703186, 0.35950096084262273]], [[-0.68831646685528314, -0.61317204872986231, 0.46975355144497893, 0.13216631421490138, 0.37285849899596457], [0.5791397355341964, -0.80856560871280225, -0.72088569469136476, -0.27386153832889609, 0.75059716758689299], [-0.80589742319780389, -0.40286394423758543, 0.45830564101476035, -0.32523757693498556, 0.86976624949505466], [0.13707474274083284, -0.53048820762218307, -0.23200788844155604, -0.75440549955975733, -0.53065893738360437]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[0.16196797258458528, -0.87553179470356857, -0.60222680248837368, 0.14709037205386077, 0.3556166421447251], [-0.6419686001157352, -0.42480335802533564, -0.19594124821840864, 0.70726884481089414, 0.75204164044023791], [0.039321557679237396, 0.43748758848062752, 0.75704401346022876, 0.76454523005635644, 0.30534874386382027], [0.15112591468115033, 0.089900406644022102, 0.35917646346824705, -0.3421665984079667, -0.20565912906789818]]),self.functionspace)
      arg1=numpy.array([[0.21860412872011858, -0.98840740295497787, -0.069274004080670748, -0.04734481193948481, -0.038090044095202558], [-0.86828963553517347, -0.69761872791299773, -0.64672359733995943, -0.59855991393624142, -0.36198596308426945], [-0.28082063984983763, -0.61440801497291919, -0.65080269815869363, -0.08863549331887266, -0.87236881392923471], [-0.71107952303930211, -0.83721058185920727, -0.12005709440444634, 0.53721957267291631, 0.44292111553365765]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.03540686752741732, -0.160090343144213, -0.011220169993762529, -0.0076683432022368221, -0.0061693672177574122], [-0.14063511188384054, -0.11299189099710567, -0.10474850988376293, -0.096947535730656884, -0.058630132544837656], [-0.045483949696384196, -0.09951442052488324, -0.10540919357334143, -0.014356111151892359, -0.14129580813813747], [-0.11517210869308966, -0.13560130057009678, -0.019445404175084332, 0.087012365018589527, 0.071739035097889398]], [[-0.19139486514793533, 0.86538210740746502, 0.060651593119051994, 0.041451888167280078, 0.03334904466701076], [0.76021518292261792, 0.61078737686848739, 0.56622707185620269, 0.52405823568621102, 0.31693021991667014], [0.2458673987975328, 0.537933752029497, 0.56979845431680587, 0.077603192539908744, 0.76378663330288632], [0.6225727309835577, 0.73300448328001067, 0.10511380333082067, -0.47035281661220257, -0.38779151919528992]], [[-0.13164926544987388, 0.5952454298374138, 0.041718661973068893, 0.028512314708729315, 0.022938845462094994], [0.52290729084214282, 0.42012469586705137, 0.38947428411982227, 0.3604688230675388, 0.21799764909391406], [0.1691177160095069, 0.37001297428036994, 0.39193082796291628, 0.053378669728404292, 0.52536388140317802], [0.42823114747491675, 0.50419065172250122, 0.072301600079234543, -0.32352802548498089, -0.26673896716241818]], [[0.032154562625952315, -0.14538521264143797, -0.01018953903388653, -0.0069639660029988905, -0.005602678757511307], [-0.12771704554137983, -0.10261299824046391, -0.095126814548745872, -0.08804240043740863, -0.053244649988340303], [-0.041306012395915859, -0.090373503515240747, -0.095726811005818702, -0.01303742768945047, -0.12831705340903637], [-0.10459295160373282, -0.12314561597310006, -0.017659242683655499, 0.079019826819075351, 0.065149431674356759]], [[0.077739266214421829, -0.35149412170983746, -0.024634988719088115, -0.016836603044893077, -0.013545453580280448], [-0.30877824459808556, -0.24808482951769487, -0.22998567408179366, -0.21285786671644183, -0.12872823269555231], [-0.099864492988332443, -0.21849371519147551, -0.23143627021792171, -0.031520256508898711, -0.31022886832129093], [-0.25287171228110933, -0.29772601588880282, -0.042694300777761471, 0.19104422052836662, 0.15751011984107519]]], [[[-0.14033698649397452, 0.63452651681903649, 0.044471735424079929, 0.03039388264353381, 0.024452612286143812], [0.55741468181951725, 0.44784931817282714, 0.41517624244614615, 0.38425667003504382, 0.23238362198275467], [0.1802780330480053, 0.39443065327205262, 0.41779489708847989, 0.056901203566484281, 0.5600333862627751], [0.45649072597650547, 0.53746290523823537, 0.077072884828785077, -0.34487809702360556, -0.28434144850084203]], [[-0.092863767958509091, 0.41987878387237565, 0.029427829557329737, 0.020112235096971152, 0.016180778638975154], [0.36885235291393648, 0.2963507782388044, 0.27473035586423977, 0.25427026141947123, 0.15377285267623284], [0.1192935508110344, 0.26100258795817677, 0.27646317158976197, 0.037652655202089311, 0.37058520159371811], [0.30206896921014958, 0.35564986654813641, 0.051000656857773538, -0.22821267846839069, -0.18815437721902542]], [[-0.042833565847117706, 0.19366978028331397, 0.01357363482865376, 0.0092768015480884699, 0.0074634107847082144], [0.170133755001869, 0.13669228432781116, 0.12671982891509115, 0.11728257667017039, 0.070927981444274549], [0.055024346697669368, 0.12038787336918849, 0.12751909302112266, 0.017367349197354328, 0.17093303430810691], [0.13932980932687153, 0.16404408643115326, 0.02352413693508254, -0.10526347363689131, -0.08678651623995487]], [[0.15461188959077027, -0.6990697620905032, -0.048995344841561164, -0.03348551044822845, -0.026939901486009932], [-0.61411420748623446, -0.49340399180947137, -0.45740745160257895, -0.42334277887979366, -0.2560213939483702], [-0.19861568954565081, -0.43455164699245108, -0.46029247252651229, -0.062689122968882793, -0.61699928327677966], [-0.50292439282868884, -0.59213296109501801, -0.084912642470785227, 0.37995866657417571, 0.31326430572584263]], [[0.16439940756968691, -0.74332352474153685, -0.052096935668691363, -0.035605270037304716, -0.028645299245797128], [-0.65298996188512815, -0.52463833254152281, -0.48636307505495496, -0.45014197957837859, -0.27222851749423338], [-0.21118881466214914, -0.46206041147986443, -0.48943072872619697, -0.066657581796754739, -0.65605767389624636], [-0.53476141098993868, -0.62961721937532433, -0.090287934222408328, 0.40401148870954356, 0.33309512231155203]]], [[[0.0085958548563875789, -0.038865718705879404, -0.0027239617471298216, -0.0018616717534910992, -0.0014977598658942031], [-0.034142500985980338, -0.027431455047747157, -0.025430179235327116, -0.02353630818032329, -0.014233851926492399], [-0.011042304987375741, -0.024159480199343396, -0.025590575833450396, -0.0034852856629657121, -0.034302900634486315], [-0.027960754479714533, -0.03292042418424472, -0.0047208319624260859, 0.021124310413273343, 0.01741634819180889]], [[0.095636593105673365, -0.43241597115517316, -0.030306516989649795, -0.020712767602474032, -0.016663921536330933], [-0.379865938753006, -0.30519953495358043, -0.28293354701377521, -0.26186253330913822, -0.15836436605357448], [-0.12285554452349227, -0.26879588081367173, -0.28471810299413258, -0.038776928225864371, -0.38165052867160615], [-0.31108846575241911, -0.3662692385080476, -0.052523488710992272, 0.23502689533326737, 0.19377249072196931]], [[0.16549294696525504, -0.74826790726683801, -0.052443470077691248, -0.035842106447187334, -0.028835839854709232], [-0.65733347053146696, -0.52812808164427516, -0.48959822772967981, -0.45313619954270129, -0.27403930630958157], [-0.21259358425439054, -0.46513390955723105, -0.49268628658480323, -0.067100969597146648, -0.66042158811452734], [-0.53831849601105852, -0.63380525900206763, -0.090888504592315636, 0.40669886140569361, 0.33531077894988187]], [[0.16713274388359242, -0.75568216528161936, -0.052963109386781393, -0.036197250136248346, -0.0291215615256234], [-0.66384669915578909, -0.53336107082386552, -0.49444944151115372, -0.45762612710289663, -0.27675464142343453], [-0.21470008069856733, -0.46974271715593979, -0.49756809858503592, -0.067765843630636147, -0.6669654155395176], [-0.54365245753044744, -0.64008535691316371, -0.09178907886134513, 0.41072866177999229, 0.3386332261724983]], [[0.066750496108133092, -0.3018089589180033, -0.021152730128449971, -0.014456678854190486, -0.011630747118187625], [-0.26513114952063954, -0.21301700226411008, -0.19747623807484771, -0.18276951784766768, -0.1105319591241169], [-0.085748229629182188, -0.18760871559184417, -0.19872178638594207, -0.0270647365466678, -0.26637672151926256], [-0.21712723914733534, -0.25564119952020703, -0.036659282968337772, 0.16403932169473331, 0.13524540625896436]]], [[[0.033036748905903843, -0.14937397284919135, -0.010469097230317108, -0.0071550280097616888, -0.005756392754132836], [-0.13122106537841574, -0.10542826835455232, -0.097736695193885337, -0.090457914485085111, -0.05470545977284734], [-0.042439276058652609, -0.092852973250212315, -0.098353153036193158, -0.013395120001029619, -0.13183753494436584], [-0.10746254333035064, -0.12652421496421079, -0.018143738205833169, 0.081187799304811195, 0.066936858716619474]], [[0.019652600066000812, -0.088858227455614322, -0.0062277611367119465, -0.0042563178458444369, -0.0034243104532474429], [-0.078059591319401836, -0.062716207321863907, -0.058140714387147165, -0.053810779663678976, -0.032542685280703795], [-0.025245889716534881, -0.055235530391411852, -0.058507427209493333, -0.0079683668924601572, -0.07842631111580145], [-0.063926338277470546, -0.07526557175582109, -0.010793181607459476, 0.048296258040422958, 0.039818788397699714]], [[0.07851745785324965, -0.35501267545920356, -0.024881591795980235, -0.017005142115993393, -0.013681047331464441], [-0.31186920055765677, -0.25056822754100783, -0.23228789453402926, -0.2149886330614775, -0.13001683804575534], [-0.10086416429015496, -0.22068089794451889, -0.23375301154023265, -0.031835783028036121, -0.31333434542709176], [-0.25540302832994444, -0.30070633597038343, -0.04312168258246251, 0.19295662621858101, 0.15908683987279001]], [[-0.074799031122100276, 0.33819999891035724, 0.023703250334382715, 0.016199813253598407, 0.013033140821264918], [0.29709971102396349, 0.23870182711568327, 0.22128721341197746, 0.20480720969492902, 0.12385950565997629], [0.096087443100167647, 0.21022990051787482, 0.22268294546368689, 0.030328105247130716, 0.29849546961935869], [0.24330766159591738, 0.2864654969459195, 0.041079527607113538, -0.18381859377967324, -0.15155281146521365]], [[-0.044957934723226296, 0.20327500565598383, 0.014246831346276771, 0.0097368927893578729, 0.0078335652947771964], [0.17857169022284652, 0.14347166000404216, 0.13300461177659414, 0.1230993105950835, 0.074445717902715203], [0.05775332821580751, 0.12635861725166669, 0.13384351609835515, 0.018228698360462859, 0.17941061049868173], [0.1462399954062793, 0.17217999911159285, 0.024690837473640866, -0.1104841094341404, -0.09109077086643394]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_Symbol_rank2(self):
      arg0=Data(0.854050122987,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.4967368660277891, -0.41710125019255129, -0.18771339357458627, 0.95253849714906269, 0.49647539817772102], [0.23699146911735891, -0.27107962949291453, 0.86796931502658436, -0.47648713727359548, 0.28523892885897117], [-0.83180920937989367, 0.3891535502317307, -0.42868769761014502, -0.38770354285751729, 0.7158231453426418], [0.082618386623673157, 0.34954154495987244, 0.51191197994961679, 0.68285837745301192, 0.69016466065876414]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.42423818152330406, -0.35622537402505866, -0.16031664686871799, 0.81351562064018901, 0.42401487487379619], [0.20240259334659494, -0.23151559090774523, 0.74128930024756046, -0.40694389819032778, 0.24360834227273834], [-0.71040675757277449, 0.33235663743631078, -0.3661207808670392, -0.33111825846003151, 0.61134884531695954], [0.070560243256951177, 0.29852599946211106, 0.43719848943458556, 0.58319528124657705, 0.58943521331702897]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_Symbol_rank2(self):
      arg0=Data(numpy.array([0.033106738601949903, -0.65276288347814493]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.81688427365923943, -0.62897163955820878, -0.23101135561054575, 0.90577886031471344, 0.35711439869619044], [0.38456547762232729, 0.15709951517941345, 0.52488859456119408, 0.95456630302244516, -0.72980807545647042], [0.71014882673413582, 0.69151619286488808, 0.37138261915220738, -0.11107998213629666, 0.69346813224027781], [-0.67272852873578004, 0.82729204147322166, 0.78450057419712405, 0.11038342355381436, -0.34003597662094909]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.027044374116080152, -0.020823199658893472, -0.0076480325642804313, 0.029987383959611313, 0.011822893048627296], [0.012731708742976404, 0.0052010525835379023, 0.017377349495282315, 0.031602577072393792, -0.024161565183729496], [0.023510711575168448, 0.022893845836193423, 0.012295267293579644, -0.0036774959324956382, 0.022958468182861305], [-0.022271847550929813, 0.027388941364527446, 0.025972255443023791, 0.0036544351495844522, -0.01125748219324851]], [[0.53323173394175516, 0.41056934106399279, 0.15079563860453499, -0.59125882065258017, -0.23311102462448915], [-0.25103007005890038, -0.10254873252153253, -0.34262779249055597, -0.62310545243200399, 0.47639162372060118], [-0.46355879583759602, -0.45139610402631336, -0.24242478935146061, 0.072508889435989834, -0.45267025760136725], [0.43913221421557785, -0.54002553847058121, -0.51209285690317508, -0.072054201847177243, 0.2219628645853978]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-0.44460493383173771, 0.46098117955248052, 0.93393668572896216, -0.13843632447883225, 0.83556264652213086], [0.70988908851878896, -0.094022016357883098, 0.64928852367640544, -0.60708102991252066, -0.94310897139965832], [0.56818039346526783, -0.24277868494849697, -0.92334751329488118, -0.9402236612859769, -0.26194740321349563], [0.95738652362024945, 0.11877751496958133, 0.40376863256613849, -0.60736316928414658, 0.53705186154868612]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.93496728787021621, 0.71224269159820963, 0.52116028987472451, 0.99782305173693375, -0.23156029267358869], [0.68485552342804557, -0.25897498996362378, -0.14321409811634012, 0.46437529206707406, 0.14677906425866039], [0.65005603229491982, -0.071554740422746343, -0.5531293412550875, -0.5624715131770226, -0.89836830805324852], [-0.5314546212916087, 0.045854615865973392, -0.0099757668464044436, -0.33778347195634506, -0.40580972953031758]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.41569106915837672, -0.31666661477016078, -0.23171043619548115, -0.44363705189328201, 0.10295284860219872], [-0.30449014467802632, 0.11514155827685189, 0.063673694616787388, -0.20646354600257533, -0.065258696152606085], [-0.28901811922540477, 0.031813590631002309, 0.24592403416911085, 0.25007760989830752, 0.39941898215854471], [0.2362873467339269, -0.020387188452970849, 0.0044352751586664903, 0.15018019819860542, 0.18042500794610222]], [[-0.43100232320539589, 0.32833047610057631, 0.24024508516236318, 0.45997764737434754, -0.10674493685418852], [0.31570550701289191, -0.11938259634802309, -0.066019003878215146, 0.21406826989210745, 0.067662386175566594], [0.29966359654251751, -0.032985388642649167, -0.25498221617685679, -0.25928878160901248, -0.41413088231895268], [-0.24499057820162262, 0.021138114909822304, -0.0045986407677960492, -0.15571182333576816, -0.18707064779275889]], [[-0.87320025009850621, 0.66518957882590724, 0.4867307138591454, 0.9319035538831506, -0.21626265228599989], [0.63961169775356252, -0.24186624381331803, -0.13375290014443708, 0.43369712120754195, 0.13708235280813164], [0.60711117633963663, -0.066827597118615911, -0.5165877837512205, -0.52531278083350275, -0.83901912018718627], [-0.49634496762442576, 0.04282530796724187, -0.0093167346261358273, -0.31546837629293073, -0.37900059383411133]], [[0.129433434840695, -0.098600260361766598, -0.072147514994579637, -0.13813495576271279, 0.032056355812974287], [-0.094808881462405417, 0.035851545742506548, 0.019826033356776979, -0.064286408612549958, -0.020319554166411277], [-0.089991367816201775, 0.0099057752631619266, 0.076573192964752027, 0.077866488908274065, 0.12436680659515904], [0.073572624398900055, -0.0063479444808741022, 0.0013810084960740228, 0.046761502327335121, 0.056178807393926201]], [[-0.78122374146445683, 0.59512338835784595, 0.43546207106996571, 0.83374366987010151, -0.19348313097578296], [0.57223969364083693, -0.21638982799704776, -0.11966435084136927, 0.38801464801905189, 0.12264310338600817], [0.54316253873201892, -0.05978846827883403, -0.46217421624814375, -0.46998018614350062, -0.75064300102858128], [-0.44406362987283338, 0.038314404188228415, -0.0083353781472694285, -0.28223925177927767, -0.33907945159078229]]], [[[-0.66372307578107193, 0.50561331514282193, 0.36996600315135603, 0.70834369670056829, -0.16438212510319788], [0.48617146329339339, -0.1838435195744394, -0.10166612557484909, 0.3296549528161416, 0.10419685614022117], [0.46146768425198109, -0.050795929457901942, -0.39266048389657221, -0.39929238980702059, -0.63774185935808725], [-0.37727383669779829, 0.032551691461475046, -0.0070816880338700041, -0.23978880102380171, -0.28807989900833342]], [[0.087907509634219061, -0.066966494000229554, -0.049000541299680449, -0.093817335292682824, 0.021771765625592354], [-0.064391497226538294, 0.024349350742642446, 0.013465278275774005, -0.043661501306927177, -0.013800463580722542], [-0.061119578901973531, 0.0067277209745115352, 0.052006335971510942, 0.052884705812773276, 0.0844663997551863], [0.049968435096552202, -0.0043113434430349963, 0.00093794171361506653, 0.031759083125702019, 0.038155049028087633]], [[-0.60706353002698543, 0.4624510057271109, 0.33838339521152738, 0.64787505615255925, -0.15034944057211075], [0.44466883173822758, -0.16814948890259321, -0.092987270335606345, 0.30151354781803008, 0.09530196193910985], [0.42207392151571022, -0.046459671771133387, -0.35914053338561841, -0.36520629840074281, -0.58330023245356399], [-0.34506738645943175, 0.029772875839366542, -0.0064771509282419721, -0.21931893182882578, -0.26348760018026129]], [[0.56760090405476704, -0.43238902676310692, -0.31638652552665558, -0.60575944591891218, 0.14057586096312694], [-0.41576279650397635, 0.15721880362870141, 0.086942562182460548, -0.28191343057400692, -0.089106785499743602], [-0.39463668558644671, 0.043439525510963924, 0.33579433016397264, 0.3414657855159608, 0.54538235769373478], [0.32263601884547843, -0.027837467426158138, 0.0060560988112823878, 0.20506193804268499, 0.24635938855178666]], [[0.88177603715560782, -0.6717224722601115, -0.49151094491809921, -0.94105587196248763, 0.21838658944039208], [-0.64589338825759868, 0.24424163640283006, 0.13506650076443127, -0.43795650404479414, -0.13842865231598955], [-0.61307367596980489, 0.067483917638865853, 0.52166124408205616, 0.5304719302339912, 0.84725921094615064], [0.50121962123192398, -0.043245899603284621, 0.0094082352094353085, 0.31856662279255393, 0.38272279660131137]]], [[[-0.53123008149925377, 0.40468233275503218, 0.29611305855949399, 0.56694349414460532, -0.1315680182022122], [0.3891214807682089, -0.14714451169519555, -0.081371442617515596, 0.26384893616221883, 0.083396986482949492], [0.36934909220379841, -0.04065600056770112, -0.31427724675150004, -0.31958528566992528, -0.51043525874642171], [-0.30196209583440115, 0.026053693684927475, -0.005668035131907851, -0.1919219460022204, -0.23057313179656977]], [[0.2269901286189939, -0.17291734403039122, -0.12652660982316311, -0.24225016831198884, 0.056217903341582942], [-0.16626832335757546, 0.062873607497918788, 0.03476933040677007, -0.11274042273061843, -0.035634828198688503], [-0.15781974866339832, 0.017371965781665416, 0.13428801407633856, 0.13655609429010873, 0.2181046764285739], [0.12902585406697825, -0.011132523338759505, 0.0024219035563228858, 0.082006627118898959, 0.098521952474675739]], [[0.86329972026702351, -0.65764751814965983, -0.48121205768386632, -0.9213374335296074, 0.21381062041799301], [-0.63235964452355009, 0.23912391298847882, 0.13223638136449178, -0.42877977116571703, -0.13552808398698363], [-0.60022762092185122, 0.066069891633803557, 0.51073060177832075, 0.5193566729912128, 0.82950614326389682], [0.49071730299867972, -0.042339745532938539, 0.0092110995108370623, 0.31189152886300242, 0.37470340463268703]], [[0.87907836658395466, -0.66966743121864758, -0.49000723586287454, -0.93817684301964666, 0.21771846618601393], [-0.64391736768944108, 0.24349441324509746, 0.13465328367871443, -0.43661663731804923, -0.13800514919740733], [-0.6111980627253647, 0.06727746002264226, 0.52006529439955895, 0.52884902548836377, 0.84466713978111374], [0.4996862098381486, -0.043113594816367548, 0.0093794520284616496, 0.31759201272468385, 0.38155190968446723]], [[0.24491225314716797, -0.18657012352194163, -0.13651658459067673, -0.2613771572690553, 0.060656617353203594], [-0.17939612593839585, 0.067837826118212347, 0.037514561105138067, -0.12164190187347865, -0.038448394728662898], [-0.1702804896029225, 0.018743578441354151, 0.14489079458296164, 0.14733795225828655, 0.23532524542385014], [0.13921315797314865, -0.012011497551444086, 0.0026131262204789267, 0.088481503327403219, 0.10630080484923771]]], [[[-0.89512508143271929, 0.68189155448313932, 0.49895183817208399, 0.95530234269057135, -0.22169270361125459], [0.65567144875690286, -0.24793916534586277, -0.13711124752901219, 0.44458664652723406, 0.14052429807083208], [0.62235488491720592, -0.068505544181882455, -0.52955857713656684, -0.53850264663597103, -0.86008571137770495], [-0.50880749234028944, 0.043900591275866201, -0.0095506647415252895, -0.32338934395266322, -0.38851676620630443]], [[-0.11105309103107347, 0.084598416963281242, 0.061902124132146435, 0.11851894246467698, -0.027504156129397814], [0.081345437185975145, -0.030760405747151498, -0.017010614682868679, 0.05515734320500059, 0.017434052502204173], [0.077212040106976479, -0.0084990942517072596, -0.065699328611040816, -0.066808968576346869, -0.10670595515799218], [-0.063124859236117228, 0.0054464973224450566, -0.0011848967959318571, -0.040121081396771931, -0.048201071224089051]], [[-0.37751046331742838, 0.28758125764183501, 0.21042817759048987, 0.40288964914279302, -0.093496782729429725], [0.27652317819990896, -0.10456597756644181, -0.05782536056062744, 0.18750017667542368, 0.059264782065056677], [0.26247223525108931, -0.028891559694117285, -0.22333627775077566, -0.22710835373289315, -0.3627329432834156], [-0.21458470570986782, 0.018514655545049635, -0.0040279017383713419, -0.13638637057525604, -0.16385323957449083]], [[0.56786469513785753, -0.43258997846855962, -0.31653356536335719, -0.60604097108772303, 0.14064119323859536], [-0.41595602121101083, 0.1572918706696366, 0.086982968518111062, -0.2820444491271093, -0.089148197652701378], [-0.39482009198692003, 0.043459713920463654, 0.33595038972874219, 0.34162448087524605, 0.54563582276365752], [0.32278596311837732, -0.027850404818664711, 0.0060589133678719187, 0.20515724005920838, 0.246473883453876]], [[-0.50212592243782594, 0.38251126339726521, 0.27989010384247365, 0.53588272743151122, -0.12435988624110939], [0.36780293374893169, -0.13908300045451646, -0.076913397993396637, 0.24939361506183694, 0.078827969696487782], [0.34911380225493949, -0.038428606546668943, -0.29705914239824321, -0.30207637321982633, -0.48247037219634065], [-0.28541869369331047, 0.024626306811420928, -0.005357504155237172, -0.18140724241453351, -0.21794087067882587]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_constData_rank2(self):
      arg0=Data(0.401591456378,self.functionspace)
      arg1=Data(numpy.array([[-0.20636599075220774, -0.41037145249758167, -0.65372099878190437, -0.71522947074190424, 0.6239040269837941], [0.27537558542853469, -0.082977678406752631, -0.81947272137821492, -0.57172533214450616, 0.87255906268418171], [0.065289537431470146, 0.58518655847804424, 0.65745809407314848, -0.17463204844297064, -0.81657024710887893], [0.45031562839620509, 0.52969267736231962, -0.92934239922061845, -0.94765722904536687, -0.30658080708702773]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[-0.082874818773104889, -0.16480166926453246, -0.26252876796582264, -0.28723004479983538, 0.25055452683663249], [0.11058848240323885, -0.033323126718247949, -0.32909324364046688, -0.22960000878421025, 0.35041226475931919], [0.026219720423361716, 0.2350059222721321, 0.2640295535064574, -0.070130738664517259, -0.32792763477154407], [0.18084290903748679, 0.21272005373479075, -0.37321596757699904, -0.38057104675963827, -0.12312023281567695]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_constData_rank2(self):
      arg0=Data(numpy.array([-0.6037699296970036, -0.64977695945749225]),self.functionspace)
      arg1=Data(numpy.array([[0.11246978049741929, 0.57470907517187886, -0.10635824940352423, 0.81226307149406152, 0.18553308629923349], [0.86526504782660396, 0.38800189087870085, -0.90036808440952543, -0.22258074352878832, 0.061709096654924744], [0.49362869521577335, -0.50310428478554359, -0.8247507935097822, 0.54201170525013609, -0.7474481860057034], [-0.6772962728394869, -0.39473202374126393, -0.12896798728005354, -0.71086115611699374, 0.27274538987649644]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.067905871463964276, -0.34699205791275528, 0.064215912765062194, -0.49042001757144171, -0.1120192984713563], [-0.52242101709554312, -0.23426387437813767, 0.54361517502536494, 0.13438755987228332, -0.037258096949009514], [-0.29803816260685112, 0.30375923865522891, 0.49795972861494914, -0.32725036917382772, 0.4512867387168164], [0.40893112303633961, 0.23832732622341887, 0.077866992613241981, 0.42919659025308804, -0.16467546487091408]], [[-0.073080272002464669, -0.37343271543781081, 0.069109139910643616, -0.52778982887501491, -0.12055512469428045], [-0.5622292919016123, -0.25211468891891992, 0.58503843628018815, 0.14462783876392396, -0.040097149195305506], [-0.32074855267827435, 0.32690557245798679, 0.53590406291694026, -0.35218671782780392, 0.48567460965480408], [0.44009151281753389, 0.25648777418710111, 0.083800426642185732, 0.46190120061813794, -0.17722367013999815]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-0.92127095180878116, 0.69094113165284088, 0.47472214054501838, -0.75831139586704532, -0.96280360258309083], [-0.41455067519827504, 0.38490685299598559, 0.2447115140349736, 0.00037197105984443368, 0.57403409938038674], [0.60014207779041695, 0.059576141602880606, -0.21413099513921741, 0.54640485285359564, 0.27570424665373161], [0.26235808321486132, -0.54732464402956138, -0.69381431133305127, -0.43435017056214087, 0.19733412935079175]]),self.functionspace)
      arg1=Data(numpy.array([[-0.77313075143792598, 0.0084607718125822018, -0.64464610044251924, -0.57638099822004474, -0.93184039847213129], [0.17235025217443223, -0.13940553724244564, -0.24589043482940109, 0.11872633083126116, 0.72206642689275036], [0.27659434094381496, 0.35400790191399789, 0.79422056366365057, 0.77235596300316445, 0.72206224814221165], [-0.90875871953557441, -0.28709729641379167, -0.14124077404126978, -0.11279523096346988, -0.41136629847249173]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.71226290324985631, -0.0077946633008145116, 0.59389372653449879, 0.53100307083467602, 0.85847749083429425], [-0.15878128086522264, 0.12843027198276238, 0.22653171493595742, -0.10937911980968021, -0.66521882437264979], [-0.25481833174623092, -0.32613719674413849, -0.73169233463251804, -0.71154911317111302, -0.66521497461116363], [0.83721301051106789, 0.26449439952886161, 0.13012102233520961, 0.10391496978920721, 0.3789798213358076]], [[-0.5341878363141318, 0.0058458952508420045, -0.44541250615534517, -0.39824533917335175, -0.64384685944016862], [0.11908387827805471, -0.096321019660967649, -0.16989581530363551, 0.082032905381541163, 0.49890539412580021], [0.19111040694049122, 0.2445986203625056, 0.54875965503971991, 0.53365250311622614, 0.49890250685517412], [-0.62789877807529637, -0.1983673308786163, -0.097589060251598136, -0.07793486452694344, -0.28422989579042374]], [[-0.36702228524379077, 0.0040165157055319775, -0.30602777669607167, -0.27362082124449405, -0.44236526860901304], [0.08181848063572017, -0.066178895043562072, -0.11672963356175863, 0.056362017911272308, 0.34278091979021946], [0.13130545759548645, 0.16805538896646399, 0.37703408604727923, 0.36665447601957124, 0.34277893604481896], [-0.4314078845768779, -0.13629144309824281, -0.067050122585106856, -0.053546393486248157, -0.1952846897589422]], [[0.58627385931063136, -0.0064158996833117606, 0.48884248426681426, 0.43707627931148307, 0.70662519329070561], [-0.13069516030443096, 0.10571280753791433, 0.18646151886583789, -0.09003152965882627, -0.54755120008577141], [-0.20974464077002977, -0.26844822624836778, -0.60226650425809436, -0.58568632841116564, -0.54754803129161733], [0.68912209311737016, 0.21770915159319723, 0.10710448851657722, 0.085533909039054626, 0.31194375200733482]], [[0.74437307275220732, -0.0081460615817876107, 0.62066758789719856, 0.55494170154669709, 0.89717929268143093], [-0.16593944369964753, 0.1342201534770579, 0.23674419649447009, -0.11431013904581014, -0.69520815711664008], [-0.26630602791480074, -0.34084008330567861, -0.76467841994093577, -0.74362710365597917, -0.69520413380056711], [0.87495616904764772, 0.27641831127906408, 0.13598712607855884, 0.1085996547258206, 0.39606495415058607]]], [[[0.32050187502514199, -0.0035074186676044851, 0.26723847620238139, 0.2389391319835753, 0.38629506636365168], [-0.071447913409503849, 0.05779065959023412, 0.10193404578332567, -0.049218080609913097, -0.29933312480639557], [-0.11466237079428039, -0.14675421476397255, -0.32924467092312093, -0.32018068595637578, -0.29933139250253826], [0.37672654077579221, 0.11901637807593664, 0.058551458244335386, 0.046759339155051823, 0.17053217678558658]], [[-0.29758332449039365, 0.003256609052298156, -0.24812870181746413, -0.2218529961515622, -0.35867175527043327], [0.066338793177525238, -0.05365814663020442, -0.094644913451999269, 0.045698578368020992, 0.27792831602934442], [0.1064630573291825, 0.13626006746142846, 0.30570093774447354, 0.29728510311223189, 0.27792670759962512], [-0.34978745886909945, -0.11050571686628821, -0.054364541850942243, -0.043415657383104544, -0.1583377073736541]], [[-0.18919399673137172, 0.0020704482801614183, -0.15775232325603056, -0.14104706673541656, -0.22803207474906834], [0.04217609115391481, -0.03411414008345777, -0.060172220593820748, 0.029053700173535084, 0.17669796855874853], [0.067685819945866643, 0.086629809657718854, 0.19435491661184207, 0.18900439708044442, 0.17669694597037741], [-0.22238372215003435, -0.070256014080766552, -0.034563243659110725, -0.027602291744995251, -0.1006660697221663]], [[-0.00028758226501068873, 3.1471622582281117e-06, -0.00023978969320618513, -0.00021439705078210269, -0.00034661766062553807], [6.4109305965778964e-05, -5.1854825436255177e-05, -9.1464125649100977e-05, 4.4162759110745077e-05, 0.00026858781408937966], [0.00010288509014784348, 0.00013168069446825413, 0.0002954270648162116, 0.0002872940661354553, 0.00026858625971511293], [-0.0003380319440485181, -0.00010679188562550962, -5.2537480413379293e-05, -4.1956561606879575e-05, -0.00015301635802709439]], [[-0.44380341460495148, 0.0048567715274985867, -0.37004884368659985, -0.33086234721321167, -0.53490816390321061], [0.098934921784932739, -0.080023532019606242, -0.14114949430354692, 0.068152962391460847, 0.41449075105419381], [0.15877458339739445, 0.20321260714874206, 0.45590968597204679, 0.44335865962359283, 0.41448835230889181], [-0.52165849312267687, -0.16480363798143483, -0.081077020522569002, -0.064748308820518144, -0.23613828265910014]]], [[[-0.4639882955716233, 0.0050776651753136745, -0.38687925015906333, -0.34591048987069228, -0.55923663290811498], [0.10343463844766608, -0.083663128776170687, -0.14756919646730587, 0.07125266687350551, 0.43334244573811742], [0.1659959024790921, 0.21245503780889283, 0.47664517930097938, 0.46352331243053752, 0.43333993789408654], [-0.54538434615223841, -0.17229916799778416, -0.084764531601854431, -0.067693164275266796, -0.24687822509823401]], [[-0.046060147125207368, 0.00050406013957605804, -0.038405527363708322, -0.034338555967167061, -0.055515455530660382], [0.010267963028836155, -0.0083052440269815869, -0.014649203364190285, 0.0070732566975936642, 0.043017931695248535], [0.016478423622624156, 0.021090424892967006, 0.047316596764745297, 0.046013988219705738, 0.043017682741414716], [-0.054140338157903846, -0.01710414918495224, -0.0084145803543831509, -0.0067199046520093047, -0.024507616848450015]], [[0.16555125717813401, -0.0018117134878740671, 0.13803871100037254, 0.12342103672819367, 0.19953591183576236], [-0.036905531010606243, 0.029851046417642118, 0.05265276350523454, -0.025422987370125903, -0.15461680254716362], [-0.059227421476175081, -0.075804064323990838, -0.17006723965732767, -0.16538535095957618, -0.15461590774715231], [0.19459340895559352, 0.061476429782864085, 0.030244027499690442, 0.024152955053165674, 0.08808627485865099]], [[-0.42244239447602977, 0.0046230067772818273, -0.35223775765493898, -0.31493737452003212, -0.50916211581020088], [0.094173014178650746, -0.076171862063934961, -0.13435572686106556, 0.064872643327702567, 0.39454059973685485], [0.15113249016354247, 0.19343163555432813, 0.43396597022193678, 0.42201904631534121, 0.3945383164472816], [-0.49655017442725752, -0.15687135600164295, -0.077174644356947958, -0.061631861577182097, -0.22477254178579015]], [[-0.21315543139002682, 0.0023326707186971034, -0.17773166748077057, -0.15891068889978324, -0.25691235506227206], [0.047517696436332503, -0.038434698624787204, -0.067793037093998518, 0.032733353599794568, 0.1990767802604175], [0.076258234398599897, 0.097601481906666898, 0.21896998218178887, 0.21294181892830485, 0.19907562816114829], [-0.2505486381595653, -0.079153943824087511, -0.038940681203838215, -0.031098124178917125, -0.11341543541909244]]], [[[-0.20283710202171965, 0.0022197518752673944, -0.16912811526403432, -0.15121821389447934, -0.24447586080532094], [0.045217481802082023, -0.036574169540466002, -0.064511343162710441, 0.031148812584023172, 0.18943996371338578], [0.072566761118097123, 0.092876834589071117, 0.20837018473262209, 0.20263383001307855, 0.18943886738440421], [-0.23842019576214504, -0.075322296383291257, -0.037055658749250882, -0.029592740591353535, -0.10792527356643546]], [[0.42315351331907014, -0.0046307889205369004, 0.35283069744974671, 0.31546752467618921, 0.51001921438612385], [-0.094331540419776261, 0.076300086046971319, 0.13458189471327603, -0.064981846759155956, -0.3952047500647719], [-0.15138689919766465, -0.19375724889873078, -0.43469648728816512, -0.42272945251481608, -0.39520246293162081], [0.49738604267856829, 0.15713542556152799, 0.077304556374597697, 0.061735609635313313, 0.22515091287721484]], [[0.53640917987930903, -0.0058702045684928122, 0.4472646902320635, 0.39990138534549696, 0.64652420433825786], [-0.11957907152048142, 0.096721556817881457, 0.17060230270454543, -0.082374027462791474, -0.50098002071131065], [-0.19190511218055217, -0.2456157486729188, -0.55104159342484349, -0.53587162057501614, -0.50097712143438333], [0.63050980516248001, 0.19919221299691575, 0.097994870373590698, 0.078258945492572313, 0.28541182508031826]], [[0.3358094737538993, -0.003674937679882433, 0.28000214367942722, 0.25035118488565344, 0.40474503601306355], [-0.074860361428392627, 0.060550818878563145, 0.10680255230774935, -0.051568802046775448, -0.31362967567806171], [-0.12013879916546898, -0.15376339257669061, -0.34496983729126629, -0.33547294426511104, -0.3136278606370525], [0.39471950483010948, 0.12470075966525992, 0.061347954295154328, 0.048992627807579216, 0.17867702190504334]], [[-0.1525650837093265, 0.001669599039271629, -0.12721067697020758, -0.11373964245809277, -0.1838839137263929], [0.034010586956230986, -0.027509470318417387, -0.048522574872747463, 0.023428757125600986, 0.14248834968431801], [0.05458150345350378, 0.069857841107499263, 0.15672682344306155, 0.1524121915081218, 0.14248752507421869], [-0.17932911070949292, -0.056654095026781766, -0.02787162517426588, -0.022258348697097797, -0.081176610353327086]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_expandedData_rank2(self):
      arg0=Data(0.0596557925971,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.31928510411621724, 0.064704454835101899, -0.23987836773877858, -0.90618834188613073, 0.86835655239726695], [-0.51412650198807031, -0.46973335346415701, -0.8729202258712625, -0.97687278919286924, -0.90832968295672534], [-0.049162725267693608, -0.20127692395668162, -0.062009945166549674, -0.30360297570745054, -0.522325587313895], [-0.6617808520544175, -0.89148228241511585, 0.0438043718299026, -0.63153844785264757, -0.46117950601537605]])+(1.-msk_arg1)*numpy.array([[-0.024236153865825116, -0.12283916168511944, -0.38330902066210149, -0.45060899935199306, 0.17935121229739748], [0.82995869977987646, -0.049795990724521344, 0.50232359921601777, 0.41922814719074886, -0.47855235126117446], [0.026221206389239082, 0.75310141641819639, 0.88473844001614332, -0.53523113043558856, -0.61177775227866893], [0.34495492330617306, -0.92331067160852265, 0.71145038846183661, -0.17562305144957313, -0.76964110985703527]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.019047205950485778, 0.0038599955377482727, -0.014310134154344371, -0.054059383777427077, 0.051802498390104021], [-0.030670623971249079, -0.028022315510176303, -0.052074747948349376, -0.058276120505795247, -0.054187127176214024], [-0.002932841342075465, -0.012007334430132764, -0.0036992524278103649, -0.018111676150652026, -0.031159746904932027], [-0.039479061254859865, -0.053182082143704276, 0.0026131845207288946, -0.037674926662162808, -0.027512028960864989]])+(1.-msk_ref)*numpy.array([[-0.0014458269683699464, -0.0073280675522834401, -0.022866603437198128, -0.026881437007708439, 0.010699338722843706], [0.049511844058188738, -0.002970619294826859, 0.029966512451436323, 0.02500938739965845, -0.028548419813669051], [0.001564246850000988, 0.044926861902391355, 0.05277977288024395, -0.031929637308752111, -0.036496086705428019], [0.020578559360085652, -0.055080829928124457, 0.042442136817172668, -0.01047693233253744, -0.045913550423797578]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_expandedData_rank2(self):
      arg0=Data(numpy.array([0.81754009937055727, 0.068794605163091882]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.073594571193924319, 0.49278314621057606, -0.79450929904989653, 0.55543766467178646, -0.15150111473618222], [0.03670305604595292, -0.67735370664124428, 0.65738905375957324, -0.63103985741826785, 0.9628578377269259], [-0.86786828643525094, -0.5752751384508854, 0.39109303732782164, -0.83914853897407826, 0.95198513521267536], [0.70762159671469527, -0.6304267832395658, -0.5483870867810221, -0.57642535770284908, 0.82988669590837638]])+(1.-msk_arg1)*numpy.array([[-0.38948009589738009, -0.83859298264283355, -0.21024311874563439, 0.50084730030558045, -0.46145224636212845], [0.17583332556646702, 0.38526563864204744, 0.30977882677375379, 0.87702995861563915, 0.94144902733384161], [0.91242354539656567, -0.12921860172255517, -0.77641969028149282, -0.59418535977868281, 0.72270161672714472], [-0.99183426227178062, -0.96151014763149401, -0.46360827099737434, 0.43715054312385027, -0.90366235547661167]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.060166513047014439, 0.40286998232113019, -0.64954321129608417, 0.45409256356992256, -0.12385823639616861], [0.030006220087011481, -0.5537638166364981, 0.53744191233571814, -0.51590038774051294, 0.78717489233499094], [-0.70951712513283027, -0.47031049385454793, 0.31973424060012035, -0.68603757993952585, 0.77828602204106401], [0.57850903049488434, -0.51539917501553545, -0.44832843342048723, -0.47125084421609625, 0.67846565183923746]], [[0.0050629094674330792, 0.033900821974582759, -0.054657953526542527, 0.038211114833805376, -0.010422459370043937], [0.0025249722489601636, -0.046598280804141169, 0.045224820391928426, -0.043412137833263531, 0.066239424774612268], [-0.059704656098882217, -0.039575826009871681, 0.026905091085001845, -0.05772889241190713, 0.065491441498088637], [0.048680548350864099, -0.043369961637204038, -0.037726073111638621, -0.039654954889161508, 0.057091727575119655]]])+(1.-msk_ref)*numpy.array([[[-0.31841559630279831, -0.68558339036127414, -0.17188218019128182, 0.40946275166129958, -0.37725571534566138], [0.14375079445626501, 0.31497010849948065, 0.25325661282350931, 0.71700715951758531, 0.7696723313588234], [0.74594283597154443, -0.10564138849278222, -0.63475423074598891, -0.48577035807799468, 0.59083755155437234], [-0.81086428133679489, -0.78607310164045086, -0.37901835194020567, 0.35738809846536562, -0.73878021189378096]], [[-0.026794129416143425, -0.057690673133453295, -0.01446359234236296, 0.034455592271522915, -0.031745425090104432], [0.012096384206858489, 0.026504197493286089, 0.021311112075786228, 0.060334929719165709, 0.064766614116608529], [0.062769817547065176, -0.0088895426852300067, -0.053413486033765389, -0.040876747219664181, 0.049717972373472079], [-0.068232846460213673, -0.066146710966614813, -0.031893747953608068, 0.030073599011036449, -0.062167094945763078]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[0.096587985349336769, -0.41147338952206325, 0.24409171143097264, 0.87622408202359137, 0.13972144257176877], [0.90926242262038559, 0.23018290769581995, -0.42857420242821154, -0.66176433450416416, 0.13873484339387732], [0.67546872313826256, -0.1473051146243205, 0.18477243685801326, -0.36311347294725893, -0.078844790086926464], [-0.10370292228364386, -0.61063338424861202, 0.73079465938438748, -0.97666680454888644, -0.64935172531905838]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.9652330546955592, -0.20265266885029432, 0.3716803443575416, 0.92942145427133416, 0.2223474274250592], [0.74680455546934854, -0.60335567874715323, -0.23996232680339369, 0.4017292028969488, -0.8708951237392375], [0.84719581080672524, -0.83137189315267035, 0.69592012701030304, 0.45226323247994227, -0.5411054267749229], [0.96631753795233633, 0.99859139869408753, -0.95454913298656785, 0.0076583508435648096, 0.67698130053309269]])+(1.-msk_arg1)*numpy.array([[-0.87833112768788291, -0.6595510056659506, 0.24220570379344464, 0.43217280424913507, -0.95579951001036889], [-0.42609761853809225, -0.82180511560918923, 0.71834570005375431, -0.26719977697253161, -0.5984655911967709], [0.69700867656891874, -0.5273527604562207, -0.0099409341444107735, 0.93986037339584372, -0.08491685305213803], [0.43714405229483755, 0.21144112509039981, -0.81170014133570789, -0.14134978605899451, 0.78010563049507331]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.093229916145630254, -0.019573813009916223, 0.035899855655442674, 0.089770945808518895, 0.021476090062594339], [0.072132347462491392, -0.05827690945926918, -0.023177477705678951, 0.038802214363811226, -0.084118005452534297], [0.081828936562219456, -0.080300536235680495, 0.06721752303197974, 0.04368319447281635, -0.052264283033782874], [0.09333466419854744, 0.096451931387038239, -0.092197977672128728, 0.00073970467907831874, 0.065388259937665311]], [[-0.39716771669431683, 0.083386180547522851, -0.15293657111152531, -0.38243219608355117, -0.091490049614100066], [-0.30729020174949057, 0.24826480622147623, 0.098738111967393447, -0.16530087678600419, 0.35835016848322077], [-0.34859853186153583, 0.34208741082890387, -0.28635261349755414, -0.18609428522472674, 0.22265048404386012], [-0.3976139526958628, -0.41089378756823425, 0.3927715672153298, -0.0031512075797507644, -0.27855979037340622]], [[0.23560538825038466, -0.049465836765722498, 0.090724091359485587, 0.22686407341375342, 0.05427316409245668], [0.18228880204896003, -0.14727412022698877, -0.058572815028398721, 0.098058768666916679, -0.21257828123039915], [0.20679347537696408, -0.20293098823524303, 0.16986833482120473, 0.11039370643333296, -0.13207934968607776], [0.23587010162454963, 0.24374788352748855, -0.23299753151564245, 0.0018693399641445674, 0.16524552425388822]], [[0.84576044728944333, -0.17756914873297999, 0.32567526854089923, 0.81438146058193106, 0.19482617048582959], [0.65436813606716615, -0.52867477574394528, -0.21026076952354866, 0.35200480203044804, -0.7630992803372354], [0.742333371618355, -0.7284680738979139, 0.60978197445134386, 0.39628393571275949, -0.47412960585384045], [0.84671069763558282, 0.87498983163738098, -0.83639893779757046, 0.0067104314377171717, 0.59318731860674612]], [[0.13486375482001853, -0.02831492323278207, 0.05193171388920749, 0.12986010634794204, 0.031066703311950935], [0.10434460980934589, -0.084301725818420936, -0.033527882463848378, 0.056130183751968474, -0.12168272301756533], [0.11837142082667494, -0.11616048022491351, 0.097234964060608081, 0.063190871264268769, -0.075604030812404827], [0.13501528038510036, 0.1395246307652982, -0.13337098186651442, 0.0010700358275835976, 0.094588803904595842]]], [[[0.87765014570575928, -0.18426445662930535, 0.33795497035091743, 0.84508800314611521, 0.20217216052391968], [0.67904131933, -0.54860864615940363, -0.21818872660687844, 0.36527726826343609, -0.79187221005941955], [0.77032331536796483, -0.75593522166649341, 0.63277402063567456, 0.41122596242683895, -0.49200683124240407], [0.8786362255791077, 0.90798163438446533, -0.86793565716955534, 0.0069634506412966125, 0.61555365739141921]], [[0.22218015113394224, -0.046647180568278865, 0.085554462397602574, 0.21393693281905327, 0.051180577363385424], [0.1719016440584189, -0.13888216450880478, -0.055235226121059751, 0.092471196029143693, -0.2004651718804086], [0.19500999511920977, -0.1913675997424602, 0.16018891835927587, 0.10410326589614372, -0.12455322050503934], [0.22242978064333463, 0.2298586717514409, -0.21972089496937211, 0.0017628214653264834, 0.15582952421240501]], [[-0.41367398657349558, 0.086851705922463357, -0.15929260714127641, -0.39832605848400554, -0.095292371370659393], [-0.3200611667300311, 0.2585826787995934, 0.10284166282258229, -0.172170772723681, 0.3732431830551623], [-0.36308626891701429, 0.35630454602913797, -0.29825341338718031, -0.19382835414769609, 0.2319038267096396], [-0.41413876812031558, -0.42797051224699079, 0.40909513334825914, -0.0032821716046962093, -0.2901367209347836]], [[-0.63875681008202823, 0.13410830853720779, -0.24596479573204708, -0.61505797015976194, -0.14714159733865723], [-0.49420861965485158, 0.39927926921541812, 0.15879850950311858, -0.26585005860598765, 0.57632733198421815], [-0.56064397193322835, 0.55017226759764393, -0.46053511971902661, -0.29929167706279108, 0.35808427264629861], [-0.63947448242273019, -0.66083217239837533, 0.63168657174238296, -0.0050680234493910709, -0.44800207981904566]], [[0.13391145668178223, -0.028114986276296862, 0.05156501436702593, 0.12894313990524325, 0.030847335522847089], [0.10360781304887423, -0.083706455601792851, -0.033291135829499233, 0.055733838050655356, -0.12082349860445452], [0.11753557813621994, -0.11534024939860703, 0.096548369835421602, 0.062744668730913514, -0.075070176643196082], [0.13406191229657449, 0.13853942131229713, -0.13242922447665287, 0.001062480104937332, 0.093920894710042002]]], [[[0.65198473898605414, -0.13688553946886944, 0.25105844761877838, 0.62779512297396523, 0.15018873289588225], [0.50444311951671861, -0.40754788992155933, -0.16208704648717481, 0.27135551172815403, -0.58826241721948191], [0.57225427257370376, -0.56156571112087428, 0.4700722795978669, 0.3054896681656098, -0.36549979170684183], [0.65271727350677422, 0.67451725701274701, -0.64476808403117258, 0.005172976465647558, 0.45727969465956853]], [[-0.14218376576111236, 0.029851774613917068, -0.054750415729194583, -0.13690853385574153, -0.03275291328327113], [-0.11000813064537711, 0.088877377417084097, 0.035347678055292563, -0.05917676628067195, 0.12828730602817015], [-0.1247962760201288, 0.12246533201629244, -0.10251259407862437, -0.0666206873008236, 0.07970759691492188], [-0.14234351569156006, -0.14709762044749308, 0.14060996944913212, -0.001128114248844576, -0.099722808073548788]], [[0.17834846365200246, -0.037444627459248876, 0.068676282959168486, 0.17173146697383296, 0.041083675994438434], [0.13798889757073687, -0.1114834990542321, -0.044338423877582002, 0.074228483776296469, -0.1609174142610599], [0.15653843445865898, -0.15361461063307871, 0.1285868577262318, 0.083565779566601106, -0.099981368302297757], [0.17854884626608888, 0.18451216616215843, -0.1763743694026319, 0.0014150521476790914, 0.12508748460680658]], [[-0.35048912669399601, 0.073585914388261162, -0.13496214066590007, -0.33748545209215614, -0.080737346573201849], [-0.27117479574930903, 0.21908657593232947, 0.087133553862085406, -0.14587328604824512, 0.31623375295378736], [-0.30762821312839894, 0.30188233543340359, -0.25269797421260864, -0.16422287303214539, 0.19648267074685097], [-0.35088291717571751, -0.362601990835071, 0.34660965077754757, -0.0027808503718553881, -0.24582103115692333]], [[-0.076103597582434179, 0.015978107136056876, -0.029305058730306911, -0.073280039464309263, -0.017530936241696909], [-0.058881648411941219, 0.047571451838574336, 0.018919779285584023, -0.031674254674198217, 0.068665543218948022], [-0.066796975865579714, 0.065549342399792945, -0.05486967633139455, -0.03565859962891587, 0.042663343788965552], [-0.07618910343716756, -0.078733729212645626, 0.075261226017983601, -0.00060382106467290358, -0.05337644853330617]]], [[[-0.10009748845669773, 0.021015673968355086, -0.038544337865268126, -0.096383720841051429, -0.023058077986229056], [-0.077445814776909053, 0.062569747062511227, 0.024884794527494685, -0.041660492307092477, 0.090314369334334552], [-0.087856681327118477, 0.086215694824417238, -0.072168950846973015, -0.046901018849617011, 0.056114214020097773], [-0.10020995253959321, -0.10355684621188818, 0.098989534554025677, -0.00079419336235107981, -0.070204939196663457]], [[-0.58940352677737495, 0.1237464850070685, -0.22696042653373513, -0.56753576801497141, -0.13577276208753655], [-0.45602379307852858, 0.36842912001899253, 0.14652900770812771, -0.24530926271646117, 0.53179763673450431], [-0.51732604507415747, 0.50766343268499059, -0.42495206232302524, -0.27616702822044392, 0.33041703798686067], [-0.5900657484586217, -0.60977324526612564, 0.5828795675071663, -0.004676444693369192, -0.41338738261754909]], [[0.70538716143279301, -0.14809748810578791, 0.27162201065464148, 0.67921623509876172, 0.16249031249009094], [0.54576078074093148, -0.44092910773766175, -0.17536318688137117, 0.29358155599583718, -0.63644550531254007], [0.61912617399038072, -0.6075621394782591, 0.50857471217723405, 0.33051155493226148, -0.39543695605102341], [0.70617969600503749, 0.7297652610728248, -0.69757940850658129, 0.0055966818961690817, 0.49473431893268111]], [[-0.94271108317447228, 0.19792413451932062, -0.36300785423730991, -0.90773508182236295, -0.217159351442898], [-0.72937921881280021, 0.58927746276840665, 0.23436323893118613, -0.39235557688733425, 0.85057435759960809], [-0.82742802536780724, 0.81197333027717677, -0.6796820866684079, -0.44171048608113539, 0.52847970809232536], [-0.94377026197145564, -0.97529107041255747, 0.93227645149890126, -0.0074796570464987118, -0.66118516353100498]], [[-0.62677574940154646, 0.13159286015845043, -0.24135127287575139, -0.60352142487963911, -0.14438168561871631], [-0.48493882657015391, 0.3917900509755155, 0.15581995092135942, -0.26086355101218378, 0.56551725117202867], [-0.55012806143042559, 0.53985277320045832, -0.45189693515839852, -0.29367791030922491, 0.3513677424558016], [-0.62747996047541432, -0.64843704763077747, 0.61983812640663916, -0.0049729633333674755, -0.43959897550990373]]]])+(1.-msk_ref)*numpy.array([[[[-0.084836234092983673, -0.063704702872403166, 0.023394160969527032, 0.041742700485197244, -0.092318749069784767], [-0.041155910536744542, -0.079376500466470379, 0.069383563952551089, -0.025808288143568937, -0.057804585754595877], [0.067322663840799338, -0.050935940700877751, -0.00096017480149906942, 0.090779219975979944, -0.0082019477585116921], [0.042222863318603476, 0.020422672292478818, -0.078400481359387944, -0.013652691064998048, 0.075348831209193265]], [[0.36140988623246934, 0.27138768786405426, -0.099661201901465518, -0.17782760862364674, 0.39328606408749373], [0.17532783136714794, 0.33815093644628413, -0.29558014004971767, 0.10994559791042692, 0.24625266532206078], [-0.2868005226741005, 0.21699162781873782, 0.0040904298674163128, -0.38672753351865979, 0.034941025352910195], [-0.17987314490716688, -0.087002396425305384, 0.33399300843094154, 0.05816167557791295, -0.32099270796505402]], [[-0.21439334816043149, -0.16099093374902101, 0.059120404757285125, 0.1054897994230941, -0.23330273818331601], [-0.10400689694562466, -0.20059581713177532, 0.17534223132520102, -0.065221250855199434, -0.14608049038776863], [0.17013404074594465, -0.12872243782760667, -0.0024264996285318172, 0.22941212704834449, -0.020727499990828784], [0.1067032398665175, 0.051611026090206055, -0.19812927666739533, -0.034502311189541807, 0.1904173184444804]], [[-0.76961488607106099, -0.57791447448738409, 0.2122264704672889, 0.37868021867875962, -0.83749454825743386], [-0.37335699465597827, -0.72008543302695316, 0.62943180160519496, -0.23412687929466486, -0.52438996326909648], [0.61073578778907911, -0.46207918843335888, -0.0087104858951433054, 0.82352829290912299, -0.074406191613941852], [0.38303614593411683, 0.18526980573437093, -0.71123121122029997, -0.1238540865337735, 0.68354733996198058]], [[-0.12272169221623944, -0.092153417961307452, 0.033841330333130616, 0.060383807649975793, -0.13354568634803848], [-0.059534973938537496, -0.11482379626577513, 0.10036829747673767, -0.037333538293457001, -0.083618475731579267], [0.097387057775248739, -0.073682488435147575, -0.0013889616591680253, 0.13131864718690853, -0.011864705207099632], [0.061078397598303427, 0.029542859016628474, -0.1134119146831337, -0.019749596015373603, 0.10899748405113086]]], [[[-0.79863348902437969, -0.59970494525353391, 0.22022854500370298, 0.39295849098221425, -0.86907257801140547], [-0.38743455290472267, -0.74723651034063743, 0.65316475150981346, -0.24295471653367082, -0.54416227330651723], [0.63376379784448389, -0.47950204854797113, -0.0090389178632566507, 0.854579720038805, -0.077211703527486311], [0.39747866002369653, 0.19225546964127693, -0.73804843695221511, -0.12852404890887456, 0.70932073548375374]], [[-0.20217681289094538, -0.15181736825789072, 0.055751613159687574, 0.099478792709122327, -0.22000871038842668], [-0.098080388797362389, -0.18916549107022265, 0.16535090196916249, -0.061504821599211917, -0.13775654993757064], [0.16043948386184903, -0.12138759178323009, -0.0022882331265731298, 0.21633979357633437, -0.019546408147919796], [0.10062308903915929, 0.048670132979783821, -0.18683949870976127, -0.032536304757241431, 0.17956698233723689]], [[0.37643006251670602, 0.28266654621400961, -0.10380311632683918, -0.18521811489223666, 0.40963101268396923], [0.18261444702152321, 0.35220447197363247, -0.30786443546827302, 0.11451493130499873, 0.25648691342788416], [-0.29871993764606758, 0.22600978871084049, 0.0042604279223322225, -0.40279990992200482, 0.036393172569533699], [-0.18734866355849639, -0.090618211546141803, 0.3478737406838176, 0.060578871823631905, -0.33433314839918316]], [[0.5812482141886639, 0.43646733233607998, -0.16028309638398161, -0.28599654819472725, 0.63251402666141798], [0.28197620696566983, 0.54384131542323277, -0.47537556414000065, 0.17682328258788846, 0.39604318368197228], [-0.46125548299325869, 0.34898324857224478, 0.0065785556684257182, -0.6219660745271357, 0.056194944748236025], [-0.28928634284934673, -0.13992419543226015, 0.53715420384796075, 0.093540247103636481, -0.51624608340752354]], [[-0.1218551314477461, -0.091502705481339944, 0.033602370384887378, 0.059957426316596549, -0.13260269533723321], [-0.059114586378386305, -0.11401300401432811, 0.099659578199622775, -0.037069919213163119, -0.083028030071308212], [0.096699389587962634, -0.073162202635222676, -0.0013791539417136764, 0.13039138171518344, -0.011780926309689263], [0.060647111635689206, 0.029334251376441842, -0.11261109199099752, -0.019610140432652665, 0.10822783247741592]]], [[[-0.59328520531192452, -0.44550607564173661, 0.16360237747816228, 0.29191921226124556, -0.64561267460288085], [-0.28781561432617964, -0.55510365210903134, 0.48522005278717079, -0.18048509217446446, -0.40424478872786829], [0.47080756077829777, -0.35621029574880142, -0.0067147900933267014, 0.6348462863459412, -0.057358678304047152], [0.29527713483107981, 0.14282186678373002, -0.54827805803917784, -0.095477359505135612, 0.52693695414347641]], [[0.12938266744217228, 0.097155236490208713, -0.035678138959957584, -0.063661264467432865, 0.14079415637994674], [0.062766358539903672, 0.12105609675366458, -0.10581599568630604, 0.03935989377453164, 0.088157042509952072], [-0.10267294299613051, 0.077681758826455422, 0.0014643504436152503, -0.13844624003393141, 0.012508686772381773], [-0.064393554730631006, -0.031146359167736632, 0.11956758236003361, 0.020821546437543366, -0.11491354931915458]], [[-0.16229138283113692, -0.121866846549051, 0.044752938110824911, 0.079853622184873838, -0.17660540461231089], [-0.078731095316679475, -0.15184693383349121, 0.13273048550540764, -0.04937115391913232, -0.11057994566109892], [0.12878799168081792, -0.097440254633296031, -0.0018368106265078076, 0.1736602914986323, -0.015690293868757366], [0.080772171800503917, 0.039068491934953181, -0.14997981311259251, -0.026117544419479246, 0.14414201835323157]], [[0.31893386617242953, 0.23949185625322059, -0.087948154272072768, -0.15692776786425933, 0.34706367952115341], [0.15472178608192302, 0.29840850961467635, -0.26084100192324872, 0.09702383898722898, 0.21731091925889398], [-0.25309324122331284, 0.1914888923175822, 0.0036096871215169839, -0.34127596426927237, 0.030834453423513885], [-0.15873289500701665, -0.076777121255450886, 0.29473925731218981, 0.051326011716233542, -0.28326686475477719]], [[0.069251833389364539, 0.052002160593353122, -0.01909665787345043, -0.034074574032301415, 0.075359811731954704], [0.033595577290175149, 0.064795051832568853, -0.056637815930584499, 0.021067310326672823, 0.047185893912157745], [-0.054955502792842816, 0.041579017699931935, 0.00078379086588402738, -0.074103093851415627, 0.0066952514537382034], [-0.034466531040934868, -0.016671031123496133, 0.063998327257142426, 0.011144694210653387, -0.061507264682013472]]], [[[0.091085504673921791, 0.068397366682675223, -0.025117439277146856, -0.044817582732152486, 0.099119202305350165], [0.044187568220501509, 0.085223592036320711, -0.074494548305464231, 0.027709397705589417, 0.062062630693313713], [-0.072281836617252043, 0.054688022333656515, 0.0010309039210046522, -0.097466267259745679, 0.0088061258126374761], [-0.045333115681888685, -0.021927062562815951, 0.084175676674559649, 0.014658385878485595, -0.080899233572263585]], [[0.5363383089909517, 0.4027438626743749, -0.14789888859170799, -0.26389914203886228, 0.58364308946079668], [0.26018943082818941, 0.50182163893726106, -0.43864586588426235, 0.16316110408321133, 0.36544306930883058], [-0.42561676702392509, 0.32201920081022967, 0.0060702662591941309, -0.57391012052786816, 0.051853065358969122], [-0.26693475205694889, -0.12911300978328494, 0.49565120429890003, 0.086312898224021092, -0.47635854122060384]], [[-0.64187969728537131, -0.48199635253227857, 0.17700263480468623, 0.31582957727644223, -0.69849317735779204], [-0.31138986400404378, -0.60057078954196463, 0.5249632011910228, -0.19526817000022553, -0.43735545787192026], [0.50937021838114571, -0.38538658095302031, -0.0072647815820272984, 0.68684494144469888, -0.062056782702231299], [0.31946253879871667, 0.1545200449902904, -0.59318612830968787, -0.10329766875703894, 0.57009702852148991]], [[0.85783685581474456, 0.6441615731407685, -0.23655427076745766, -0.42208883173893413, 0.93349765323121836], [0.4161553995234889, 0.80262977622395504, -0.70158439943293305, 0.26096515235193757, 0.58450147658661045], [-0.68074523688741417, 0.51504793542481142, 0.0097089803850525878, -0.9179304276066419, 0.082935471522779011], [-0.42694408468235029, -0.20650752799226216, 0.79276058329022536, 0.13805164387390689, -0.76190327334621755]], [[0.57034583316556098, 0.42828058346510506, -0.15727669164039007, -0.28063215607515152, 0.6206500608843436], [0.27668722375205218, 0.53364056969685525, -0.46645901970543219, 0.17350663618198101, 0.38861466418771345], [-0.45260378669238094, 0.34243742485401502, 0.0064551627379562729, -0.61029995502360557, 0.05514090503807078], [-0.28386024457061743, -0.13729965938085395, 0.52707888721806551, 0.091785727450887866, -0.50656293709308775]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_array_rank3(self):
      arg0=Data(0.63085697321,self.functionspace)
      arg1=numpy.array([[[0.7511494295457346, -0.63812308385015082], [-0.79031758533022489, 0.72347714731302926]], [[-0.1549764196296024, -0.42229395140139703], [0.84788909172282212, 0.61113423428304059]], [[0.055651654082436774, 0.84180242209460854], [0.67477676686322829, -0.31421937445641834]], [[-0.039714909406222576, 0.36656647335745873], [-0.54627624730718272, -0.40593534119731323]], [[-0.90428276936389351, -0.34475114430785903], [0.57493034903473594, 0.067723576808715213]], [[-0.65528381944986314, 0.75340720688533924], [0.90833199251890773, 0.61485294800008217]]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[0.47386785555184341, -0.40256439721330978], [-0.49857735975627532, 0.45641060334069861]], [[-0.09776795500649571, -0.26640708398609037], [0.53489674602226489, 0.3855382932649753]], [[0.035108234048591057, 0.53105692804367921], [0.42568762873594851, -0.19822748349360064]], [[-0.025054427539329675, 0.23125101586264965], [-0.34462217991287447, -0.25608714066681543]], [[-0.57047309080710695, -0.2174886634088331], [0.36269881979877788, 0.04272389068051935]], [[-0.41339036693180603, 0.47529219013048912], [0.57302757147053218, 0.38788426974474366]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_array_rank3(self):
      arg0=Data(numpy.array([-0.37351169248831639, 0.754681078152595]),self.functionspace)
      arg1=numpy.array([[[0.33151316835686417, -0.47169541761088807], [-0.27677524227489081, -0.10665081466068571]], [[-0.41169023354507717, 0.80275407203831062], [0.67347081713473345, -0.21720762663964099]], [[-0.57688106717229926, 0.53185800711579634], [0.78853005774813067, -0.23925046278153528]], [[0.65670628554389765, 0.52680954318837214], [0.70580059327607159, 0.99072646018628818]], [[-0.47118853616110168, -0.077236143350451369], [-0.74397504936769643, 0.43281688949676145]], [[0.061635190625851877, -0.97480843561904251], [0.69244102903953375, 0.35790333135525043]]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.12382404459513652, 0.17618375377082601], [0.10337878918095829, 0.03983532628917047]], [[0.15377111591233203, -0.29983803209891724], [-0.25154922474948371, 0.081129588247542628]], [[0.21547182376399163, -0.1986551844012841], [-0.29452519644741415, 0.089362845282144193]], [[-0.2452874761812168, -0.1967695240952857], [-0.26362477415380331, -0.37004791693713912]], [[0.17599442762262535, 0.028848602624097316], [0.27788337985840705, -0.16166216893346397]], [[-0.023021464367501947, 0.36410234863995655], [-0.25863482070490768, -0.13368107904170631]]], [[[0.25018671531734099, -0.35597960632222353], [-0.20887703824596029, -0.080487351793978881]], [[-0.31069482931669246, 0.60582330857725819], [0.50825568237954977, -0.16392248585537059]], [[-0.43536122573941038, 0.40138317423423975], [0.5950887141370873, -0.18055779720047635]], [[0.49560380760385458, 0.39757319403447677], [0.53265435269432693, 0.74768251312769196]], [[-0.35559707248320321, -0.058288655936067017], [-0.56146389237544325, 0.3266387168080685]], [[0.046514912113658609, -0.73566948118522346], [0.52257214235264771, 0.27010287198158583]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_Symbol_rank3(self):
      arg0=Data(0.371716733534,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[0.50405215756150867, 0.13008019207079569], [-0.53893233714559252, 0.25840133411586885]], [[-0.57594152824257216, -0.14990381581664569], [0.15935630688897007, 0.61623573647501284]], [[-0.0489993063575902, 0.38053461848990411], [0.69454122009365626, -0.10702911707257057]], [[-0.34118464231034862, 0.76582952390026526], [0.47018994301140093, 0.52337574033039158]], [[-0.29685296611554013, -0.074300728883793354], [-0.14670791850571141, 0.84435486865565568]], [[0.68276876614644699, -0.44728980928117434], [0.49799813782292346, -0.84947917546264229]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.18736462153943875, 0.048352984094008178], [-0.20033016795950745, 0.096052099858332199]], [[-0.21408710358480568, -0.055721756759619025], [0.05923540586478105, 0.22906513504930012]], [[-0.018213862104666405, 0.14145108538160581], [0.25817259363780837, -0.039784513791224815]], [[-0.1268240407715078, 0.28467164906796771], [0.17477746975665129, 0.19454752060645833]], [[-0.11034521490429454, -0.027618824239865673], [-0.054533788250489018, 0.31386083372005857]], [[0.25379657551087442, -0.16626510684896378], [0.18511424109746258, -0.31576562430797672]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_Symbol_rank3(self):
      arg0=Data(numpy.array([-0.86189841266156564, -0.49432560178388485]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[-0.29200271321703708, 0.6287222104032919], [-0.48896964245668983, -0.1427911534266304]], [[0.077520981194961314, -0.97539182610628794], [-0.08454111211744042, 0.14603074794410209]], [[0.81254997781561911, -0.27602273677717837], [0.21948100793507197, -0.081710300977030803]], [[-0.4655786061841225, 0.010235321903029559], [0.32540206939618765, -0.38111004402023085]], [[-0.36744905121002058, -0.9533430523784594], [-0.59893021573473826, 0.50857583445190535]], [[0.76622486584404048, -0.31849621110566662], [0.32793317954594192, 0.76799952375956071]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.25167667501463464, -0.54189467515166823], [0.42144215867311424, 0.12307146848052682]], [[-0.066815210639904241, 0.84068866664407549], [0.072865850338665356, -0.12586366985280278]], [[-0.70033553608747245, 0.2379035586867512], [-0.18917033234859904, 0.07042597871020162]], [[0.40128146163927936, -0.0088218077013013322], [-0.28046352708936279, 0.32847814199041636]], [[0.31670375397191508, 0.82168486356692594], [0.51621700223681999, -0.43834070443212841]], [[-0.66040799561279961, 0.27451137879069698], [-0.28264508690970752, -0.66193757045320378]]], [[[0.144344416933539, -0.31079348501250154], [0.24171021276145421, 0.070585322847034102]], [[-0.038320605680076475, 0.48216115141507315], [0.041790836122932613, -0.072186737356419076]], [[-0.40166425676318818, 0.13644510546341354], [-0.10849508132763806, 0.040391493702413107]], [[0.2301474246796687, -0.0050595816591668648], [-0.16085457377599191, 0.18839245185618347]], [[0.18163947336431094, 0.47126187807346759], [0.29606653931962645, -0.2514020554181795]], [[-0.37876456791013174, 0.15744083122069588], [-0.16210576632395049, -0.37964182675218183]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_constData_rank3(self):
      arg0=Data(-0.22015011451,self.functionspace)
      arg1=Data(numpy.array([[[0.21611300599883188, 0.85367468515937839], [0.20206252928537971, -0.33534565187777221]], [[-0.28753014455938852, 0.94369326713724155], [0.5960541281912437, -0.35610462964096889]], [[-0.43821404099900829, 0.71729335722593879], [0.96696190055936571, -0.11902778733421071]], [[-0.52697654397718829, -0.74626763346059466], [-0.894797071002152, 0.51441543102202125]], [[0.13108682526670457, -0.12314148797673896], [0.97800726769710322, -0.31679390749747505]], [[0.60304956080461203, -0.17187084537541608], [0.20646553622807273, -0.79892531291287883]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[-0.04757730301771268, -0.18793657969200497], [-0.044484088960328079, 0.073826383661274864]], [[0.063299794249785699, -0.20775418082244668], [-0.13122138457537647, 0.07839647499295023]], [[0.096472871305759725, -0.15791221473045161], [-0.21287677313481526, 0.026203981011481633]], [[0.11601394650058774, 0.1642909049613514], [0.19698967764421019, -0.11324861604513645]], [[-0.028858779593198887, 0.027109612678993505], [-0.21530841197499159, 0.069742215011594805]], [[-0.13276142986625553, 0.037837386290304122], [-0.045453411442949655, 0.17588349912259521]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_constData_rank3(self):
      arg0=Data(numpy.array([-0.6964528596908941, 0.14349836695029716]),self.functionspace)
      arg1=Data(numpy.array([[[0.18464657487749325, -0.13451960456249568], [-0.15051112471229877, -0.95360578587102784]], [[0.48599623445465157, 0.94785127929647262], [-0.80233042263825016, -0.29668787050614687]], [[0.97413159829837248, 0.8251455931670626], [0.30707955779124552, -0.98124009531244316]], [[-0.19974203930633738, -0.76739425913226422], [-0.6100504802439064, 0.089384999831634726]], [[-0.94203847780455074, 0.14571372034187569], [-0.62694687714414532, -0.2985007731314655]], [[0.04558018528447616, 0.32028111128322179], [0.88398088330267788, -0.87923676066261702]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[-0.12859763510555897, 0.093686563282038365], [0.10482390322117328, 0.66414147658765976]], [[-0.33847346728494832, -0.6601337340277007], [0.55878531726341296, 0.20662911584960766]], [[-0.67843673735016286, -0.57467500802253979], [-0.21386643617632811, 0.6833874704237165]], [[0.13911091447538965, 0.53445392628304045], [0.42487140152167191, -0.062252438746212092]], [[0.65608539180583625, -0.10148273722829852], [0.43663894546131565, 0.20789171706735196]], [[-0.031744450386614234, -0.22306069585817731], [-0.61565101408823253, 0.61234695630883784]]], [[[0.026496481957886047, -0.019303343577517874], [-0.021598100604067388, -0.13684087298684725]], [[0.069739665988236249, 0.13601511069079383], [-0.11513310540313063, -0.042574224911593309]], [[0.13978629355049932, 0.11840704511570777], [0.04406541506686313, -0.14080635126348953]], [[-0.028662656451781482, -0.11011982299251308], [-0.087541247672245096, 0.01282660150569217]], [[-0.1351809831692968, 0.020909680911311457], [-0.089965853034773433, -0.042834373477766439]], [[0.0065406821536142948, 0.045959816434168722], [0.12684981316921548, -0.12616903931775481]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_expandedData_rank3(self):
      arg0=Data(0.11946137623,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.60009572891304597, -0.90370197398073504], [0.24596895409604413, -0.62521236806063096]], [[0.73989407342463687, 0.98218131541014264], [-0.8161448228849888, -0.76997935470965029]], [[0.93799867068875042, 0.15725815651379604], [0.04143890929725047, -0.3138816191100573]], [[0.18267431352457963, -0.74895957924499701], [-0.18371856301252509, 0.46216104492375054]], [[0.19599041150492469, 0.47743431899460353], [0.6887260182789392, 0.83062239870725207]], [[0.3689738891803549, 0.46162556151726752], [-0.81813499745096308, -0.45772317035683363]]])+(1.-msk_arg1)*numpy.array([[[0.67627092574222458, 0.29414155783672391], [0.18312599623788461, -0.83022275044559457]], [[0.74444175782501998, -0.83170336216878682], [-0.68427091786799332, 0.52224497857627394]], [[0.74612578255800099, 0.2171108033711453], [-0.90597048931563728, 0.99802670455100317]], [[-0.13764985206780667, 0.34424923204205204], [-0.77946707677156035, -0.14045604097079534]], [[-0.67766328759702876, -0.0061437012722671547], [0.68403256260133172, -0.36106006739046248]], [[-0.73725728883864083, -0.70852286233785988], [0.94731906938117993, -0.29743929808334824]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.071688261645656878, -0.10795748151344513], [0.029383789766150491, -0.074688729924497987]], [[0.088388764275677739, 0.11733273164622091], [-0.097497783744775149, -0.091982793382250072]], [[0.11205461210232524, 0.018786275800520176], [0.0049503491341168805, -0.037496730192166879]], [[0.02182252489550443, -0.089471742077198421], [-0.021947272376461796, 0.055210394466454836]], [[0.023413284286249073, 0.057034960806495873], [0.082276157988963622, 0.099227294876975916]], [[0.044078128594395742, 0.055146424881768082], [-0.097735532737364253, -0.054680239863155122]]])+(1.-msk_ref)*numpy.array([[[0.080788255493456543, 0.035138555305591283], [0.021876483534055111, -0.099179552345630417]], [[0.088932036912806905, -0.099356428259745139], [-0.081743945562629497, 0.062388103869893227]], [[0.08913321282501406, 0.025936355365103254], [-0.10822848147735126, 0.11922564363988694]], [[-0.016443840765866784, 0.041124487025840865], [-0.093116209717052936, -0.016779071954178975]], [[-0.080954588956841503, -0.00073393500913062064], [0.081715471314442442, -0.043132732552136767]], [[-0.088073770360212786, -0.084641116225251653], [0.11316803975713453, -0.03553250789390186]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank1_expandedData_rank3(self):
      arg0=Data(numpy.array([-0.28740085940932802, 0.24764677399374602]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.67242579391337953, -0.89317712926090165], [-0.99701004112330294, 0.89526514770181609]], [[-0.91740715313646759, -0.35134117664448827], [-0.049093269571241605, 0.15259130609547955]], [[0.88272014101965879, -0.4251717004763671], [0.022740311589840356, -0.60226254447714189]], [[-0.63411600184138606, -0.57221024361603012], [-0.72835035653650548, 0.47257931918100105]], [[0.63109086308110807, -0.78774731648430785], [0.032191936052698766, 0.29192410722269302]], [[-0.8070392067870551, 0.25497980893334793], [-0.57544468016613104, -0.18247344197239346]]])+(1.-msk_arg1)*numpy.array([[[-0.72180000146745971, 0.67186892782174601], [0.15238074428723825, -0.38705902177094997]], [[0.029915034122554252, 0.44216489583112395], [0.42650164147091751, 0.11272393150830373]], [[0.36411532614042019, -0.93726403450581586], [0.037190266021924545, 0.10796202997472304]], [[-0.18544138361845919, -0.59619018146266511], [-0.18514933596332872, 0.1897906248393959]], [[0.72764377986819939, 0.18040101973592271], [-0.44782228345532848, 0.088378488666651434]], [[-0.17616843716837183, -0.57623381041958188], [0.6551520906908892, -0.97896984058814596]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.19325575105970497, 0.25669987455433962], [0.28654154265856674, -0.25729997284872091]], [[0.26366360423968577, 0.10097575611351045], [0.01410944786598865, -0.043854872510232658]], [[-0.25369452714697316, 0.1221947121134333], [-0.0065355850941560207, 0.17309077287277921]], [[0.18224548389442138, 0.164453715778068], [0.20932851841968214, -0.13581970247169484]], [[-0.18137605641488502, 0.22639925575498199], [-0.0092519900875957569, -0.083899239298102793]], [[0.23194376160762203, -0.073281416219470444], [0.16538329562227197, 0.052443024042244026]]], [[[0.16652407861283194, -0.22119243466645738], [-0.24690632032355803, 0.2217095256973893]], [[-0.22719292191303272, -0.087008508967174381], [-0.012157789834123318, 0.037788744694037743]], [[0.21860279526282306, -0.10529240001640756], [0.005631564804836558, -0.14914837623702917]], [[-0.15703678219383158, -0.14170602087768536], [-0.18037361613346031, 0.11703274385133573]], [[0.15628761633896529, -0.19508308164956931], [0.0079722291120638156, 0.072294063404704331]], [[-0.1998606560472859, 0.06314492711588536], [-0.14250701865500531, -0.045188959243998253]]]])+(1.-msk_ref)*numpy.array([[[[0.20744594074340214, -0.19309570726639358], [-0.043794356865585325, 0.11124109549910483]], [[-0.0085976065160814647, -0.12707857106250103], [-0.1225769382982308, -0.032396954791484724]], [[-0.10464705765686452, 0.26937048901042554], [-0.010688514416362645, -0.031028380198311035]], [[0.053296013022000051, 0.17134557052377317], [0.053212078274927077, -0.054545988686675741]], [[-0.20912544767797239, -0.051847408110423335], [0.12870450912770912, -0.025400053596093177]], [[0.050630960242988271, 0.16561009233529964], [-0.18829127390837958, 0.28135677352084598]]], [[[-0.17875144183209754, 0.16638617252169238], [0.037736599741500496, -0.095853918086750867]], [[0.0074083616943633928, 0.10950071002585861], [0.10562175561331, 0.0279157179899234]], [[0.090171985880355757, -0.23211041452572834], [0.0092100494042988389, 0.026736448437056269]], [[-0.045923960418048117, -0.14764457512597506], [-0.04585163575840262, 0.04700103597573372]], [[0.18019863470097505, 0.044675730562783365], [-0.110901743820225, 0.021886647608739072]], [[-0.043627545144267227, -0.14270244421653328], [0.16224630173485682, -0.24243872285882614]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_array_rank4(self):
      arg0=Data(-0.448440211752,self.functionspace)
      arg1=numpy.array([[[[-0.25515102245128873, 0.36730616433963759, 0.17450639786221234, -0.36562008126248435], [0.35434412138378724, -0.0087194181035530338, -0.82333920735532318, 0.93803882900265978], [-0.60591702633892441, 0.63607654180945716, 0.31940940747376056, 0.15366531796524296]], [[-0.86048365345962585, -0.011426882553942974, -0.43114257438679471, 0.9724566228920053], [-0.25770702633276033, 0.46387524920783285, -0.13014386176362192, 0.78555502852532433], [-0.71597477612394567, 0.93125380345187714, -0.01746761040089484, -0.30285112237192524]]], [[[-0.13998741907158574, 0.32762499088219132, -0.68181548853288021, -0.72915647255318494], [-0.063111239566042032, -0.34496963082412591, 0.52408052730259103, -0.97614011386580235], [0.21818049843376852, -0.99285280659954767, -0.94141588395163112, 0.23244192959824495]], [[0.38929568410850446, 0.33251835049575984, -0.28250822038803847, 0.68001471362974253], [-0.0057615102952528474, -0.27448682595580909, -0.41348165426752881, 0.36856267355724026], [0.20140002479968566, 0.3740960197975931, 0.96136038238721211, 0.11315580646280576]]], [[[-0.084662037656419331, -0.77212322168240699, 0.34567399161637669, 0.45692298418094368], [0.64724676592952934, 0.79532036096776637, 0.43505835701500573, -0.46844161826170061], [-0.68218528288427494, 0.34353392644124114, 0.4129843557441697, 0.40791753970232913]], [[0.89919955957474107, 0.80499016523171996, 0.96080359935353998, -0.26014772593819702], [-0.15237087519497927, 0.92286473964411764, 0.62941743892544189, -0.3482537369458607], [0.32637149670430898, -0.58702804227088135, 0.48043802099880728, -0.424008055959604]]]])
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.1144199785366858, -0.16471485411412445, -0.078255686009334413, 0.16395874666197513], [-0.15890215282626996, 0.0039101377007078053, 0.36921840848979187, -0.42065433110914857], [0.27171755959530958, -0.28524229909924004, -0.14323602232297705, -0.06890970772720606]], [[0.38587547176620018, 0.0051242736321505207, 0.19334166735313171, -0.43608865388890861], [0.11556619345853075, -0.20802031498107337, 0.05836174092744581, -0.35227446333440793], [0.32107188021380589, -0.41761165281441576, 0.0078331789069712287, 0.13581062144566713]]], [[[0.062775987851017831, -0.14692022028631643, 0.3057534820531857, 0.32698308295177891], [0.028301617634900023, 0.15469825429463233, -0.23501878263844897, 0.43774047936118315], [-0.097840908917702488, 0.44523512282964284, 0.42216873834556196, -0.10423630812898074]], [[-0.17457583901559048, -0.1491145995076017, 0.12668804617237148, -0.30494594217430571], [0.0025836928968120437, 0.12309093035463968, 0.18542180059512053, -0.16527832337373399], [-0.090315869767942716, -0.16775969833345258, -0.43111265344729277, -0.050743613811100416]]], [[[0.037965862093964173, 0.34625110102956386, -0.15501411799745879, -0.20490263978026216], [-0.29025147676895774, -0.35665363108272113, -0.19509766174409979, 0.21006805848652566], [0.30591931271042971, -0.15405442671715883, -0.18519879194000163, -0.18292662788129241]], [[-0.40323724090261637, -0.36098996015444473, -0.43086296954577319, 0.11666070130641476], [0.068329227537208714, -0.41384965926406664, -0.28225608959185672, 0.15617097953927772], [-0.14635810309175751, 0.26324697958006427, -0.21544772787021091, 0.19014226239889684]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_Symbol_rank4(self):
      arg0=Data(-0.879172796646,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=outer(arg0,arg1)
      s1=numpy.array([[[[-0.49773932599172688, 0.92242596386689457, 0.30442794094148296, 0.12621289087073317], [0.64194779132875657, 0.91111377887259293, -0.27248976286472604, 0.0035272897626432709], [-0.93189211248519221, -0.4498889301771507, -0.45532304928720202, -0.95567913295931661]], [[0.51499323757467774, 0.21218415657260881, 0.58084903155851619, 0.57428268034845442], [0.33688643171805666, -0.16774855352853502, -0.7980270549954509, 0.90323868587713618], [0.59868667656593511, 0.74838332836698207, 0.25746671456205572, -0.057395425763606012]]], [[[-0.48491601724669775, 0.23660205428951131, -0.11488735537946182, 0.32151898974381043], [0.53319046927247826, 0.80919493804629128, 0.46325345643577198, 0.5434392343833232], [0.0044753007299669356, -0.28726961790256889, -0.74805369309580083, -0.17115083203634773]], [[-0.66857685237334463, 0.45814268090894883, -0.90380351304172946, -0.50049179283194833], [-0.69929140021103686, -0.23428129607826298, -0.3454144249302491, 0.38579265061119905], [0.4283705066368142, -0.22837198046611928, 0.2671978584916237, 0.87318000431007059]]], [[[0.43544058365045069, -0.93641070098773649, -0.097797094411510965, 0.17888670161197373], [-0.44583370756073903, 0.78064613328917676, 0.94053032093569056, -0.26037863808064499], [-0.0048233408198046224, 0.61782731254210543, -0.63865216462639252, 0.98201943259298607]], [[-0.17344080624215374, 0.77305433973649773, 0.45598771798196092, -0.61905137239026087], [0.44138287124146314, -0.89286577205632511, 0.6663460437249773, 0.35774878756239747], [-0.91191590867527061, -0.14653490621709575, -0.42013075779317188, -0.73547657982287507]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.43759887523308844, -0.81097181435219734, -0.26764476421485789, -0.11096294023966148], [-0.56438303500354414, -0.80102644903457465, 0.23956558687532167, -0.0031010972052056395], [0.8192941947064174, 0.39553010892414575, 0.40030763861943969, 0.84020709602054078]], [[-0.45276804493256273, -0.18654653833801846, -0.51066666750470946, -0.50489371014759632], [-0.29618138632582269, 0.14747996493908655, 0.70160367773991761, -0.79410288150190766], [-0.52634903975146941, -0.65795826376401256, -0.22635773148490762, 0.05046049698330584]]], [[[0.42632497102145972, -0.20801408976201571, 0.10100583752828129, -0.28267074938802189], [-0.46876655601554229, -0.71142217671434593, -0.40727983685079333, -0.47777699150021685], [-0.0039345626585991353, 0.25255963336297177, 0.65766845740077484, 0.15047115564977054]], [[0.58779458107418481, -0.40278658203784373, 0.79459946217982502, 0.44001876920268268], [0.61479797599438135, 0.20597374227509221, 0.30367896596796823, -0.33917840356351237], [-0.3766116963207643, 0.20077843274209703, -0.23491308850803547, -0.7676761063650841]]], [[[-0.38282751570134921, 0.82326681479709396, 0.085980544997669495, -0.15727232173906619], [0.39196486751545095, -0.6863228441951188, -0.82688867258785748, 0.22891781542836645], [0.0042405500377268321, -0.54317696621223155, 0.5614856096589238, -0.86336477091398067]], [[0.15248443867653733, -0.6796483458258471, -0.40089199725465424, 0.54425312633219702], [-0.38805181330121735, 0.78498329784869214, -0.58583331479601652, -0.31452300205812617], [0.80173165973646832, 0.12882950330521609, 0.36936753328623456, 0.64661100155087692]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_constData_rank4(self):
      arg0=Data(-0.417410053051,self.functionspace)
      arg1=Data(numpy.array([[[[-0.94576928300862351, -0.62048842948887883, -0.39889033328627965, -0.017179219367599252], [-0.096071312182654411, 0.091364138729949085, 0.10187595078927769, 0.47143692177797614], [0.46966210476308135, 0.43389890996641811, 0.23972434500924034, 0.63268538631155091]], [[0.38944441855279255, -0.91092437832945938, 0.90538426583677034, -0.52350775380487624], [0.17979340245138054, 0.7143507616587641, -0.40349151482927992, -0.82811672312040918], [0.078986528205921003, -0.9170053419240094, 0.61367718689934736, 0.76517759465324175]]], [[[0.043013142565961893, 0.13675292997260069, 0.17643812054635499, -0.76496364495945546], [-0.36356111513980127, 0.69414565540107609, -0.5927406081713007, -0.063802585021933922], [-0.69540657661054039, -0.97942078046711467, 0.68334764317920427, -0.84217667903769078]], [[0.86049775957003605, 0.81374832485360726, 0.85720321784346187, 0.68990788631562938], [0.26635690263035317, -0.24450877678071303, -0.90021483752868448, 0.68721263455353809], [0.1715336960840641, 0.10045919693030037, 0.15564365826854853, -0.16002747995296263]]], [[[-0.86904748630712514, 0.74810020405840127, 0.36169408128210412, 0.27880264117974929], [0.97015926227527594, 0.67344800575323349, 0.6377297902349004, -0.14473893506001301], [0.054955764189549949, -0.15602273858521687, 0.50576167221108159, 0.63985303207983679]], [[-0.69541655011200687, 0.87626714412326745, 0.96026810922009309, -0.048868982009782957], [-0.66540954681536535, 0.56553627820198216, 0.87398572584185885, 0.43190709586551845], [0.15933540203546537, -0.5872633487310277, -0.44598472235855247, 0.1405540914022323]]]]),self.functionspace)
      res=outer(arg0,arg1)
      ref=Data(numpy.array([[[[0.39477360659474714, 0.25899810827055769, 0.16650083517860403, 0.0071707788676063939], [0.040101131514852277, -0.038136309994237734, -0.042524046023585463, -0.19678251052960069], [-0.19604168406525743, -0.18111376702790477, -0.10006335156795147, -0.26408924066497141]], [[-0.16255801540858286, 0.38022899308405611, -0.37791649443457365, 0.21851739928836489], [-0.075047573655471725, -0.29817718932109111, 0.16842141461056562, 0.34566424533020779], [-0.032969770928757094, 0.38276724842065918, -0.25615502713991728, -0.31939282037773636]]], [[[-0.017954118120353448, -0.057082047754759051, -0.073647045257493549, 0.31930351562470272], [0.15175406435782798, -0.28974337484616608, 0.24741588870233444, 0.02663184039880389], [0.29026969603510189, 0.40882007993414554, -0.28523617599178797, 0.35153301227553668]], [[-0.35918041547249674, -0.33966673144740245, -0.3578052406356283, -0.2879744874273914], [-0.11118004885746713, 0.10206042148750134, 0.37575872309025155, -0.28684946224639074], [-0.071599889182503512, -0.041932678720149365, -0.064967227654944892, 0.066797078896802822]]], [[[0.36274915736339763, -0.31226454586356933, -0.15097474565623839, -0.11637502524563091], [-0.40495422913435619, -0.28110396780862668, -0.26619482557422797, 0.060415486561962321], [-0.022939088445824762, 0.065125459590036058, -0.21111000642884961, -0.26708108806536324]], [[0.29027385907487807, -0.36576271511544456, -0.40082556241285561, 0.020398404373257625], [0.27774863423692187, -0.23606052788662107, -0.36481042818956999, -0.18028236379838028], [-0.066508198616544778, 0.24512962554879536, 0.18615850661967143, -0.058668690748757441]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_constData_rank0_expandedData_rank4(self):
      arg0=Data(0.0446296352099,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.83493992475885648, 0.26024474473146442, -0.27641842981279918, 0.39870398910379912], [-0.43615518465977599, 0.1307617326084376, 0.86325047901245844, -0.84740155321183575], [-0.34277425115768612, -0.99665619154375484, 0.62572624536880328, -0.36861771215142802]], [[0.16172507619941046, -0.055604708141027581, -0.90158191963756384, -0.67799237877879848], [0.44875857145027864, 0.91322040050478948, 0.35541011910030074, -0.42252559157082814], [0.71841452661281235, -0.42399254291293587, 0.011968355890833005, -0.53503912065223402]]], [[[0.21733911718324173, 0.82586341121125884, -0.20351620055918773, 0.26849828803684539], [-0.0027675040975294607, -0.90282176942342085, 0.45961809987326774, -0.11579856226490515], [0.46881150577395014, -0.84463688913836088, 0.90015960010663676, -0.84682738008523284]], [[0.58676349652971305, -0.56400170636377922, -0.41853456169608139, -0.3251869433384682], [-0.99772483767357212, -0.50911148213419288, 0.49774853321201795, -0.66890484092032088], [0.81808185587295612, 0.15836639889620208, 0.88622045509309211, 0.4269146725362889]]], [[[-0.010073445030529093, 0.68142450068881155, 0.72926994273213608, 0.78881989849172096], [0.31238844618913486, -0.15409616695845152, 0.27529205688076686, 0.075112836227879232], [0.08126758577522053, -0.30328648172851302, 0.82803294830929541, -0.089323169102796474]], [[-0.082243386649075045, 0.51484339977082172, 0.62765845253957075, 0.67404265033697675], [0.4548897701434913, -0.8102518961352132, -0.29182113685584543, 0.5064938009939024], [0.80135108524261534, -0.58862292165723362, 0.5819953900635888, 0.0037514290888855939]]]])+(1.-msk_arg1)*numpy.array([[[[-0.024467085503008024, 0.9104785955182606, 0.60676090282793549, 0.72485116145544759], [0.39311257256698462, 0.35984428315119699, 0.45151886370107963, 0.42708476399446682], [-0.35235399139098145, 0.6862957160449068, 0.21487591789820559, -0.50613288972477966]], [[-0.81610963170031847, 0.1774050944192096, -0.0095511449550185201, -0.48048006558641121], [-0.47063578677355622, -0.16145416553119429, 0.74620701119135546, -0.42746436133938515], [-0.03424527257764054, 0.74238485619319361, -0.04847517230856413, 0.50122429886182962]]], [[[-0.20315484399875117, 0.96514809821932857, 0.29423326020356955, -0.20935538616946547], [0.88318603351275926, 0.22679185747668562, -0.055942671603035565, 0.8146375864496469], [0.15353310928014818, 0.25602474775840256, 0.27815841161798804, 0.42634852514419119]], [[-0.2872372834622503, 0.10221917675293035, 0.67883549368623219, -0.77139665041407635], [0.09322742044113852, -0.16991436282799399, 0.98682491498336566, -0.4934527011871821], [0.72697976062816294, 0.066156633283350308, 0.80418716417699621, 0.94451153492052131]]], [[[0.9829866607602209, -0.78349886466420493, -0.81693247875887742, 0.62902855141793013], [0.44277257894516309, 0.91490629885114383, 0.69990829951854105, 0.75473359841841403], [0.44370819987613053, -0.34912347902903607, 0.82994105470259694, -0.40646982696858935]], [[0.066450599902513563, 0.6750221435230741, -0.89762862866476367, 0.61411954144575187], [0.32079212183711636, 0.25113487890755271, -0.51828827116657927, 0.56840005789848536], [-0.93967160610378597, -0.87552179468722069, 0.63506723414018063, 0.18149549290406863]]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.037263064264134484, 0.011614628022648007, -0.012336453687827108, 0.017794013590417962], [-0.019465446786254285, 0.0058358484257236319, 0.03852655397306165, -0.037819222196111744], [-0.015297889788499957, -0.044480402258244657, 0.027925934072044113, -0.016451274025210866]], [[0.0072177311550662608, -0.0024816178402847114, -0.040237272185228455, -0.030258552539962], [0.020027931341123167, 0.040756693340729649, 0.015861823965338792, -0.018857163018635732], [0.032062578252193011, -0.018922632521904612, 0.00053414335746963825, -0.023878600777712686]]], [[[0.0096997655167207712, 0.036857982775527862, -0.0090828537902529535, 0.011982980649555929], [-0.00012351269831452854, -0.040292606228886273, 0.020512588133192261, -0.0051680475917088046], [0.020922886484875877, -0.037695836247034757, 0.040173794583411318, -0.037793597058924149]], [[0.02618704080458218, -0.025171190412753206, -0.018679044811214136, -0.014512974656204769], [-0.04452809554518683, -0.022721459728799433, 0.022214335463494508, -0.029852979040382366], [0.036510694799414087, 0.0070678346122364398, 0.039551695626319508, 0.019053046101030778]]], [[[-0.00044957417701907556, 0.030411726888801647, 0.032547051513649644, 0.035204744315963134], [0.013941782397195606, -0.0068772557185931447, 0.012286184074760247, 0.0033522584804280956], [0.0036269427075339795, -0.013535565043624957, 0.036954808424787489, -0.0039864604528463124]], [[-0.0036704923445715679, 0.022977273121975131, 0.028012167773225339, 0.030082277600425492], [0.020301564502200407, -0.036161246552610743, -0.013023870884402581, 0.022604633574412541], [0.035764006609402162, -0.026270026269723468, 0.025974241952357285, 0.00016742491175261598]]]])+(1.-msk_ref)*numpy.array([[[[-0.0010919571006476659, 0.040634327584364298, 0.027079517752815172, 0.03234984291719889], [0.017544470710073558, 0.01605971908939096, 0.020151122177349014, 0.019060637220761574], [-0.015725430100517131, 0.030629127453172839, 0.0095898338311804256, -0.022588526236128465]], [[-0.036422675154037201, 0.0079175246482998327, -0.00042626411517895712, -0.021443650052730429], [-0.021004303480408573, -0.007205640510769315, 0.033302946700509012, -0.019077578511791909], [-0.0015283540228022688, 0.033132365317225507, -0.0021634292568662524, 0.022369457616520563]]], [[[-0.0090667265787799795, 0.043074207547017337, 0.013131523069492692, -0.0093434545139623043], [0.039416270498116329, 0.010121637867750704, -0.0024967010263083887, 0.036356978311487327], [0.0068521266598083573, 0.011426291097153549, 0.012414108441064478, 0.019027779149446446]], [[-0.012819295179590958, 0.0045620045699353321, 0.030296180450720758, -0.034427151110086984], [0.0041607057658441197, -0.0075832160299289164, 0.044041635971707259, -0.022022614047303258], [0.03244484152178518, 0.0029525464101483089, 0.035890579777669941, 0.042153205255006411]]], [[[0.043870336085885607, -0.034967268517301774, -0.036459398518094197, 0.028073314786367955], [0.019760778679250912, 0.040831934368928353, 0.031236652087864882, 0.033683485178037674], [0.019802535100094696, -0.015581253512262571, 0.037039966517062137, -0.018140600101422457]], [[0.0029656660331254404, 0.030125992024011561, -0.040060838251233959, 0.027407931109969495], [0.014316835375786994, 0.011208058034116069, -0.023131016475712668, 0.025367487237271863], [-0.041937200997473834, -0.039074218315171307, 0.028342818993410072, 0.0081000776405420489]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.668020810252)+(1.-msk_arg0)*(-0.199373680103)
      arg1=-0.414108752084
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(-0.27663326409916661)+(1.-msk_ref)*numpy.array(0.082562385865791135)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.89803697641820968, -0.11561484456563709])+(1.-msk_arg0)*numpy.array([-0.32840922337831913, -0.52183272187384655])
      arg1=0.777737979352
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.69843746342312674, -0.089918055595606172])+(1.-msk_ref)*numpy.array([-0.25541632579090445, -0.40584912667006418])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.17536854353567821, -0.78928042811432308, -0.97636831926807344, -0.27888945307416857, 0.010622511883872532], [0.75263624986742128, -0.5592719942188058, -0.013841027001748607, -0.052258497265722292, 0.58265643840157866], [0.98297043285336128, 0.77043885820702096, -0.18809135815129996, 0.91236177100903726, 0.6082829915489476], [-0.27330126762450457, 0.20962667467243068, -0.81369505041848655, 0.49080308139066298, -0.1249881775189563]])+(1.-msk_arg0)*numpy.array([[0.63107319493892855, 0.84913314407214524, -0.47663244422210194, 0.54298257454386545, -0.25310528901917007], [-0.74538586542084051, -0.63889727230382065, -0.34508951051308467, -0.30076076768983873, -0.5692494007765323], [0.46403709770754098, 0.07981428366460297, 0.071007166588557213, -0.58626153560215855, -0.90404379525966672], [0.93173030530645828, 0.32579774985788545, -0.65037365393573832, 0.99004422028261851, -0.44078045182846037]])
      arg1=-0.234522178058
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.041127812792920319, 0.18510376510025423, 0.22898002482198859, 0.06540576197247587, -0.0024912146234572496], [-0.17650989260462979, 0.13116168621127111, 0.0032460277990155029, 0.012255776600817243, -0.13664585699370058], [-0.23052836687979938, -0.18068499908755373, 0.04411159498760945, -0.21396906971427798, -0.14265585205395143], [0.064095208549426297, -0.049162104323322527, 0.19082953549950035, -0.11510420764552287, 0.029312499623298231]])+(1.-msk_ref)*numpy.array([[-0.14800066019136418, -0.19914055440939471, 0.11178087895227631, -0.12734145602979641, 0.059358803658881971], [0.17480951665245678, 0.14983557985627627, 0.080931143630643076, 0.070535070313143822, 0.1335016093285635], [-0.10882699085428094, -0.01871821964519536, -0.016652755366105595, 0.1374913322412929, 0.21201831992449846], [-0.21851142056350839, -0.076406797903203996, 0.15252704587282326, -0.23218732691483107, 0.10337279160838554]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.25740843478106368, 0.73423343206754943], [-0.6427051620706361, -0.89580613442810764]], [[-0.22740639543899688, -0.84905678784073912], [0.84197910761975647, -0.25666510559167866]], [[0.28439305888670252, 0.6223153710205096], [-0.77856279651648896, 0.7399236312518287]], [[0.048181875839945487, -0.38581915202623618], [0.7742861694449017, -0.83641627537952057]], [[0.48993888851279577, -0.8103853288250773], [0.11418354207494752, 0.35973611692648522]], [[-0.1081267874550631, -0.69353637113732813], [-0.82769375941263057, -0.18143269032927378]]])+(1.-msk_arg0)*numpy.array([[[0.58533346116207041, 0.28874612633378605], [-0.13007576734424164, 0.18467312201063169]], [[-0.83082036907737877, -0.060060583903163822], [0.34197864311618154, 0.6939585737259415]], [[-0.79098633992461753, 0.84652008035450477], [0.14677195874921889, -0.42095550171758123]], [[-0.96050106947722624, -0.43788444443669405], [-0.39609118372006691, -0.90094996813009498]], [[0.086117360299087897, -0.32697201394953446], [0.78954648940984051, -0.22063927607971023]], [[0.2610626132674041, 0.34277618940295862], [0.30158294235422911, -0.99775481707329794]]])
      arg1=-0.613278501288
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.15786305910143908, -0.4502895788139839], [0.39415725856478695, 0.54937864356673072]], [[0.13946345337815058, 0.52070827435543277], [-0.51636768523691223, 0.15740719129020936]], [[-0.17441214893076726, -0.38165263806798844], [0.47747582500628227, -0.45377925564174931]], [[-0.02954890860436972, 0.23661459132288482], [-0.47485306156525126, 0.51295611981770339]], [[-0.30046898726987104, 0.49699189992768461], [-0.070026311555487289, -0.22061842664786538]], [[0.066311834159534969, 0.42533094627986845], [0.50760678829806782, 0.11126876840979985]]])+(1.-msk_ref)*numpy.array([[[-0.35897242781523425, -0.17708179161072052], [0.079772671650772412, -0.1132560554948694]], [[0.5095242707873775, 0.036833864882618791], [-0.20972814972282014, -0.42558987405065324]], [[0.48509491708830665, -0.51915256619006867], [-0.090012086892835644, 0.25816295920232651]], [[0.58905465637458332, 0.26854511582149565], [0.24291420752526091, 0.55253324619036059]], [[-0.052813925659109515, 0.20052490667811296], [-0.48421188772252538, 0.13531332455944978]], [[-0.16010408820698105, -0.21021726771428267], [-0.18495433490104857, 0.61190157886766627]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.51373130732142314, -0.49826403224687299, -0.14925439016692477, 0.16168858249137918], [0.85919417540887144, -0.064282795632149003, 0.5515090118931254, -0.44099930119256969], [-0.71487224437524155, 0.0042457584859505459, 0.64173479370803466, 0.46078959800040109]], [[-0.40754479414590783, 0.725128391302996, 0.72850950188907149, -0.56409961600821523], [-0.022347866841244679, -0.32715203968889051, -0.0021558078400554237, -0.21285177956047807], [-0.9534140635323769, -0.42255393690504173, 0.27521318354608226, 0.57194316501709985]]], [[[0.98916441835287316, 0.48416942773906935, 0.68388501815994296, -0.097292654680867008], [0.24471140330887642, 0.71914389963411529, -0.44963477073130309, 0.527935384037723], [-0.68654030358372786, 0.88089482181212042, 0.22551553093490395, 0.50153512068782025]], [[-0.050412827743439026, -0.42713936461634372, -0.7302549257426092, -0.15609649997827968], [-0.41466745825868179, -0.39687267255639691, -0.35974562857433545, 0.60426450876362203], [0.26980797832737724, 0.06245476882456602, -0.8699550919775223, -0.26835386848971177]]], [[[0.44368144640849461, -0.45077538942190087, -0.4844355295838132, 0.50519709918605082], [-0.86012856869553977, -0.63594022089366686, 0.66789619211143192, 0.75734366205209036], [-0.50346727384050727, -0.072080239588779182, 0.83356371123426221, 0.1323134254267162]], [[0.68690502648922536, -0.34609252078420827, -0.394197615091467, -0.1394901997472644], [0.7938132612482065, 0.023217155424228419, 0.073861196778270521, -0.39694336802580144], [0.002027476811779616, 0.85655908725606755, 0.26981208768492859, -0.47583056631722642]]]])+(1.-msk_arg0)*numpy.array([[[[0.99538415389773904, 0.23823136326685446, -0.95985216090673009, 0.81660962284280081], [-0.13147899307334221, 0.96465576217747939, -0.053627611872602898, 0.63155894105576849], [-0.31614712525834432, -0.31560053874951, 0.3475495545383982, -0.49451553112039814]], [[0.25647863330323406, -0.84355062453784013, -0.023875899680008139, 0.98678089365882671], [-0.18117294237811077, 0.96325207513942224, 0.44001398623201249, 0.22639152697392451], [0.041655855711577594, 0.43114749731298319, -0.24049119940790997, -0.3179433057487997]]], [[[-0.018558917849502032, -0.71289989545026167, -0.029025319641343916, 0.84936380123980859], [0.5757942789674213, -0.4399905546039673, -0.2984122303891934, 0.69320778125482319], [-0.97557323408510177, 0.97758894999817825, -0.13294946758508241, -0.12124450243022733]], [[0.23090968905588616, 0.43492506953951815, 0.023440743934802777, -0.74633022318926501], [-0.26926132428283522, -0.43824040687843446, 0.91308299706660057, -0.9114403702994931], [0.65526167223687182, -0.67644618397949108, -0.32545238214563077, -0.95407218158865947]]], [[[0.54539062662972126, 0.73588406488339753, 0.84117839560109031, 0.37807152717999637], [0.68558935225302253, 0.43008087372839232, -0.27646314630559865, 0.9023041381229171], [0.62363434616615621, -0.42667050127974804, 0.60746106586901494, 0.8106742973298775]], [[0.42330331017042222, -0.74611903242880073, -0.5622261508844506, -0.27517600179999246], [0.6470216876552195, -0.66974562018521988, -0.6884238802306839, -0.10904474913681272], [0.30444409123221572, -0.89746722334726736, -0.94500306008683066, -0.79495475061134968]]]])
      arg1=-0.76173064645
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.39132488082748135, 0.37954298338612641, 0.11369164310733181, -0.12316314846472041], [-0.65447453466015737, 0.048966175472481346, -0.42010131615227608, 0.33592268278135434], [0.54454009683701776, -0.0032341243561731046, -0.48882905926059128, -0.35099755836221841]], [[0.31043935950203905, -0.55235251826638099, -0.55492801381883072, 0.42969196516405778], [0.017023055055756787, 0.2492017346796101, 0.0016421448996271027, 0.16213572364260689], [0.72624471094890908, 0.32187228351861241, -0.20963831621408227, -0.43566663682105666]]], [[[-0.75347685183713764, -0.36880669118294196, -0.52093617698035022, 0.074110796744880206], [-0.18640417543612378, -0.54779394755877064, 0.34250058457549182, -0.40214456136681187], [0.52295878926271877, -0.67100458207328417, -0.17178209116352866, -0.38203467169884381]], [[0.038400995866375444, 0.32536514433339164, 0.55625755665911503, 0.11890348783701614], [0.31586491104110764, 0.30231007742466937, 0.27402927021144119, -0.46028679488722324], [-0.20552100574864393, -0.04757371143061348, 0.66267145459438881, 0.2044133657219899]]], [[[-0.33796575499055326, 0.34336942878803367, 0.36900938911316017, -0.38482411294758651], [0.65518629066244971, 0.48441515556480208, -0.50875699817844355, -0.57689187727964608], [0.38350645196888022, 0.05490572749822184, -0.63495102461562225, -0.10078719108428839]], [[-0.52323660987729848, 0.26362927958841792, 0.3002724041726178, 0.10625396002690435], [-0.60467188865107102, -0.017685218810024477, -0.056262337169472605, 0.30236392833027997], [-0.001544391222498996, -0.65246730725806967, -0.20552413597222802, 0.36245472488142572]]]])+(1.-msk_ref)*numpy.array([[[[-0.75821461501447551, -0.1814681303458934, 0.73114880702378227, -0.62203657590522621], [0.10015157838833413, -0.73480785732503751, 0.040849795459280198, -0.48107780044160203], [0.24081895409630472, 0.24040260240158956, -0.26473914685189626, 0.37668763519983867]], [[-0.19536763514665006, 0.6425583625423954, 0.018187004497824698, -0.75166124803111245], [0.13800498251690224, -0.73373862589012495, -0.33517213817949237, -0.1724493641926193], [-0.031730541899602255, -0.32841826184345962, 0.18318951679049039, 0.24218715982243991]]], [[[0.014136896490911101, 0.54303769821536818, 0.022109475493814833, -0.64698643738951311], [-0.43860014833998712, 0.33515428959031413, 0.227309741162906, -0.52803761133931226], [0.743124030258829, -0.74465946284435602, 0.10127168388875005, 0.092355653214669153]], [[-0.17589098671607722, -0.33129575437758918, -0.017855533030723045, 0.56850260337503], [0.20510460260991051, 0.33382114843196126, -0.69552330161792086, 0.69427206246873618], [-0.4991328971868112, 0.51526978901124132, 0.24790705344043945, 0.72674601964136154]]], [[[-0.4154407545903539, -0.56054544445578403, -0.64075136306087788, -0.28798866880310592], [-0.52223442049083835, -0.32760578197085055, 0.21059045115492694, -0.6873127144267569], [-0.47504139365348436, 0.32500799676090997, -0.46272171039752807, -0.61751545656537676]], [[-0.32244310410048294, 0.56834173290053736, 0.42826488936423129, 0.20960999373859718], [-0.49285624840469228, 0.51016576422065252, 0.52439356731964248, 0.083062727251947591], [-0.23190439442215702, 0.68362828820787835, 0.71983779185704111, 0.60554139608157376]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.771414181912)+(1.-msk_arg0)*(0.183128004107)
      arg1=numpy.array(-0.536035695845)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.41350553778552301)+(1.-msk_ref)*numpy.array(-0.098163147110422982)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.84211476238897176, -0.63311767547216014])+(1.-msk_arg0)*numpy.array([0.7228079105361378, 0.53424529529876619])
      arg1=numpy.array(-0.42076964503)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.35433632964514156, 0.26639669957079831])+(1.-msk_ref)*numpy.array([-0.30413562794134791, -0.22479420326194333])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.3799622126195461, 0.37236705072111187, 0.6232161190449097, -0.67341084973043053, 0.38354033302504886], [0.52272884055284385, 0.42129933951125409, 0.27778957298543006, -0.62481768620975697, -0.43452070079407457], [0.57971563148932437, 0.58259832375302456, 0.10661588551405843, -0.14869063205343003, 0.013692674452755238], [-0.65472061118789893, -0.62775351208846275, 0.037602772251926764, -0.046802628906359445, -0.75513434598744889]])+(1.-msk_arg0)*numpy.array([[0.010910815632426019, -0.74920910177769695, 0.16281908062279138, 0.83277196168888179, 0.34638945889459172], [-0.85272886856179153, -0.7706397301146295, -0.7113468219626875, -0.76118910476955093, 0.68080399454121632], [0.69545723388366931, 0.64141052030309353, 0.98073714527561151, 0.85583897830349431, -0.26940020551838795], [0.63045657920698028, -0.28809893405512432, -0.95967764455003657, -0.22259961891726254, 0.60520236510218428]])
      arg1=numpy.array(0.885486966476)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.33645158702807043, 0.32972617015873018, 0.55184975071215336, -0.59629653051996545, 0.3396199660116257], [0.46286957531076389, 0.37305507412225103, 0.24597904630159007, -0.55326791756256188, -0.38476241721726179], [0.513330635946322, 0.51588322237419026, 0.094406977042018447, -0.13166361672042198, 0.012124684764116527], [-0.5797465678902225, -0.55586755311399938, 0.033296764732454416, -0.041443117893403977, -0.6686616213104225]])+(1.-msk_ref)*numpy.array([[0.00966138503613812, -0.66341489478949855, 0.14417417378512093, 0.73740871812233011, 0.30672335117590771], [-0.75508029904947116, -0.68239143686524806, -0.62988833949223244, -0.67402303129713126, 0.60284306389118758], [0.61581831634558604, 0.56796065588911349, 0.86842995968063885, 0.75783426069005966, -0.23855037075254473], [0.55826108381695705, -0.25510785116150159, -0.84978204626764597, -0.19710906129380706, 0.53589880637856058]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.4463432675487522, 0.76019878661702078], [-0.085592688955817842, 0.82447078133220941]], [[0.43940759438778776, -0.091163458108980633], [0.079709332561558099, -0.86851544097558375]], [[0.087497133213726874, -0.92480764823561001], [-0.28404925251082513, -0.1517033585351546]], [[-0.95839918828928683, 0.15658266453206582], [0.15208993384197234, -0.96379349512777934]], [[0.93942990915614644, -0.75969946811255618], [-0.8659092222707192, -0.60537319805077083]], [[0.327337965548961, -0.11291815862827148], [-0.84266681171659097, 0.21912726347102573]]])+(1.-msk_arg0)*numpy.array([[[0.56292885389644121, 0.18488731830320915], [0.54430645574055037, 0.59866881602820388]], [[0.91497718043284904, 0.0089825862396000211], [-0.60272668870203394, 0.68427682734608464]], [[0.45077649549700283, -0.26629280817811218], [0.95186464074863908, 0.65977924770938334]], [[0.0025375671708003367, 0.57653705835051272], [0.51442792009275506, 0.13344738186607263]], [[0.75394596315267992, 0.82486677200307179], [-0.210421866790929, 0.51981409505314136]], [[-0.56875483910531988, -0.50228072629755971], [0.67568239247773842, -0.21188179097900317]]])
      arg1=numpy.array(-0.325283555839)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.14518812519300994, -0.24728016445519885], [0.027841894217361048, -0.26818678743701413]], [[-0.14293206476507503, 0.02965397381625948], [-0.025928135129168733, 0.2825137909415249]], [[-0.028461378617470667, 0.30082472028508633], [0.092396550890101722, 0.049346607897018058]], [[0.31175149587985107, -0.050933765901719386], [-0.04947235448741924, 0.31350617518956198]], [[-0.30558110131172256, 0.24711774435657044], [0.28166603085391267, 0.19691794647151911]], [[-0.10647765739483614, 0.036730420157384688], [0.27410565690259836, -0.071278495443101927]]])+(1.-msk_ref)*numpy.array([[[-0.1831114992797489, -0.060140804327185715], [-0.17705393938935302, -0.19473712124751616]], [[-0.29762703076264446, -0.0029218875926466342], [0.19605708050000109, -0.22258399957729288]], [[-0.14663018134386127, 0.086620671538501434], [-0.30962591502003084, -0.21461533976362016]], [[-0.00082542887249798121, -0.18753802441315198], [-0.16733494307057895, -0.043408238890787147]], [[-0.24524622380468517, -0.2683155966905113], [0.068446773056011934, -0.16908697721406382]], [[0.18500659646475776, 0.16338366067941362], [-0.21978837124289144, 0.068921662387163921]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.31403475340174825, -0.44529723351568418, 0.51246391185524365, 0.015942733483527372], [-0.19794432119927841, 0.62979766487987732, -0.84218684969223756, 0.15189290257478816], [0.86628145613116603, 0.38957710836696569, 0.34806921903528654, 0.36452391358710057]], [[-0.64769996681692077, -0.047944207922439253, 0.94071577948782714, 0.52995994246574862], [0.26288341509963287, -0.15842925139810515, 0.91677042237312678, 0.15971871842058727], [0.44159924798644834, -0.14690277912758298, 0.77053530275794735, -0.35986252789064088]]], [[[-0.74464208984073332, 0.72496627953920334, 0.26229282682528532, -0.068267617453663831], [-0.76150062127382534, -0.20925073378676129, 0.63271920674188653, -0.96019340322931912], [-0.51401548152996024, -0.73307789292371095, 0.455130321312188, -0.42605394012282183]], [[-0.035194814480391523, 0.064715103894955606, 0.90184393500337712, 0.24845297240730058], [0.81903131284113417, -0.23449420225909279, 0.62678067988682762, -0.089125260315905397], [-0.41604544571592372, -0.44637819653861577, -0.41262781940722992, 0.44373825265876921]]], [[[-0.55741425141821543, -0.63116860073803238, -0.66473958174575998, -0.93061641622253921], [0.52565065522145038, 0.0082169278534585111, -0.99664732047156646, 0.57986628098319626], [0.72377726280554611, -0.064307043837671163, -0.46706787517406378, -0.057762943884072993]], [[-0.51171804164063572, -0.76248011577579433, -0.42796445301156694, 0.54372186320210147], [-0.28113071911784271, 0.35689467924681773, -0.99216842853938347, -0.24168176746071879], [0.23856810831436648, -0.36199751109310641, 0.42562845583970299, 0.93737052769639173]]]])+(1.-msk_arg0)*numpy.array([[[[0.82548941551363519, -0.68138599365956676, -0.51539648999145315, 0.27055203069465716], [-0.013577166973204546, 0.74914204467550438, 0.22588966745390393, 0.44467674134569868], [0.45398425526542119, 0.55326654311637169, -0.15622051962428807, 0.22712387279217716]], [[-0.76302079685336022, -0.010006803251566554, 0.90680167826394986, 0.93372627404849129], [-0.86287658390765731, -0.83203302829617942, -0.82197141311309196, -0.10756454509262281], [0.89889902656412257, 0.74650286195648552, 0.48209134818193422, 0.49202847344226019]]], [[[-0.8236922495556962, 0.4402185595395165, 0.59617768874750054, 0.43160190247889241], [0.090939262879082605, -0.56163456612849871, 0.65668545765448738, 0.69677153031341676], [-0.75813962516501898, 0.56072548596055438, 0.49834724021091015, -0.55130227447148217]], [[-0.23707881592363633, 0.33187755120540041, 0.51813023043316853, 0.63995125436627198], [0.76663867303590205, -0.39574894154855045, 0.93322035844601747, -0.28350509209773556], [0.14854930412492307, -0.24385741736058764, 0.44739292318740342, 0.877983527645539]]], [[[0.046091094481068406, -0.91843340666079021, -0.82002661750842343, 0.86971525588010068], [-0.91383310935561468, -0.27915760973248127, -0.48475397039245682, 0.12823696714625843], [0.90996461848980581, 0.4084380779897745, 0.89206120622010365, -0.32816874514194683]], [[0.60341747123153389, 0.38813934596379518, 0.84260955813589744, 0.69588078179445301], [-0.39269177831721969, 0.55534794867574577, -0.53195052385209407, -0.38118771487043079], [0.63206575369569284, -0.7867420000743337, -0.63506205778412328, 0.50965870349872211]]]])
      arg1=numpy.array(-0.185196674525)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.058158192015161848, 0.082467566822116423, -0.094906612289467437, -0.0029525412239815281], [0.036658630027138955, -0.11663643315912228, 0.15597020389136565, -0.028130060440742365], [-0.16043244487783462, -0.072148384940478288, -0.0644612618697155, -0.067508616581029787]], [[0.11995187994419834, 0.0088791078699525343, -0.17421743403397827, -0.09814681897591386], [-0.048685134264126427, 0.029340570506353563, -0.16978283352603138, -0.02957937551082641], [-0.081782712199661795, 0.027205906172852647, -0.1427005756745906, 0.066645343451368841]]], [[[0.1379052387495647, -0.13426134411314417, -0.048575759279704027, 0.012642935730137226], [0.14102738270834209, 0.038752540039144058, -0.11717749299645085, 0.1778246251785443], [0.095193957833518195, 0.13576358793698434, -0.084288621982337525, 0.07890377287885679]], [[0.0065179626022793371, -0.011985022032860851, -0.16701849770282004, -0.046012664265588428], [-0.15168187546970907, 0.043427546453686856, -0.11607769757131881, 0.016505701826646471], [0.077050232997701029, 0.082667757579247422, 0.076417299970563096, -0.08217884875176823]]], [[[0.10323126569528213, 0.11689032592103916, 0.12310755996419938, 0.17234706554243087], [-0.09734875330869848, -0.0015217477132692136, 0.18457576942520409, -0.10738930690704496], [-0.13404114216811672, 0.011909450667245447, 0.086499417259515418, 0.010697505118082368]], [[0.094768479606095093, 0.14120878183282187, 0.079257593512488803, -0.10069548093135777], [0.052064474287338569, -0.066095707752040392, 0.18374629353380867, 0.044758659626956847], [-0.044182020307449242, 0.067040735240631366, -0.078824974604560685, -0.17359790452675691]]]])+(1.-msk_ref)*numpy.array([[[[-0.15287789460839482, 0.12619042009340337, 0.095449716008077043, -0.050105336370532542], [0.0025144461729029381, -0.13873861542047528, -0.041834015221934556, -0.082352653735666539], [-0.084076374361690728, -0.10246312391088253, 0.028931520726925779, -0.04206258594626329]], [[0.14130891417036545, 0.0018532266848122476, -0.16793665526782495, -0.17292300087004153], [0.15980187386485956, 0.15408974993509877, 0.15222637226284458, 0.019920596047906927], [-0.16647311045309063, -0.13824984755745015, -0.089281714500383358, -0.091122037052930288]]], [[[0.15254506544941565, -0.081527013290735459, -0.11041012538180912, -0.079931237057588811], [-0.016841649068926021, 0.10401285394507391, -0.12161596296628709, -0.12903977031747296], [0.14040493740590096, -0.10384449532109434, -0.092292251645580928, 0.10209934788997602]], [[0.043906208309291206, -0.061462618832613371, -0.095955995646896192, -0.11851684416649069], [-0.14197893280821403, 0.073291387921428475, -0.17282930698287327, 0.052504200267295825], [-0.027510837126881686, 0.045161582753342373, -0.082855681580154411, -0.16259962960734572]]], [[[-0.0085359174230937845, 0.17009081268589335, 0.15186620258422989, -0.16106837317232078], [0.16923885292315027, 0.051699060990696315, 0.089774823279287527, -0.023749059866609593], [-0.16852242127937361, -0.075641373792932301, -0.16520676886438154, 0.060775760283205027]], [[-0.11175090902213367, -0.071882116124654588, -0.15604848808942492, -0.12887480665392315], [0.072725211457507086, -0.10284859329881565, 0.098515468029035644, 0.070594697163641584], [-0.11705647566533781, 0.14570200212261239, 0.11761138121837963, -0.094387097030490988]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.840030047569)+(1.-msk_arg0)*(0.855738055031)
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.973296912029)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.8175986513107627)+(1.-msk_ref)*numpy.array(0.83288720646701686)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.92012209161450209, 0.75807920608376977])+(1.-msk_arg0)*numpy.array([0.051429279613880885, -0.44315453961693407])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.933108797846)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.8585740187782871, -0.70737037666114999])+(1.-msk_ref)*numpy.array([-0.047989113274612742, 0.41351139972211481])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.62276020695162804, -0.34302831191538918, 0.81368593016229651, 0.65514163510995527, -0.7293305270095316], [0.76722549267397011, -0.073636533606815213, 0.73509318797465539, -0.038566919670141031, -0.32086840122188565], [0.018793260538951362, 0.6344141607080267, -0.79095153112213823, 0.34826069229122991, -0.62011963871489995], [0.013122248014268978, -0.22509348262239737, 0.33185662577985675, 0.95732324820562842, -0.15047734329431539]])+(1.-msk_arg0)*numpy.array([[0.88889729392496819, -0.93447767328554066, -0.46660288869533861, 0.57622221968840215, 0.04412049878624269], [0.63389790589665052, 0.33591145502130537, -0.19545589355733606, -0.8425425443015937, 0.28210362203779749], [-0.97489328450973645, 0.89408893428350944, -0.50014747028046269, 0.13993233565240493, -0.1174729194264359], [0.59546208980228621, 0.96258897976321633, 0.49296395430350581, -0.97989270368384118, 0.31334893950523868]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.50240362494)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.3128769854410654, 0.17233866736344125, -0.40879876087645239, -0.3291455323285527, 0.36641830054920344], [-0.38545686866600531, 0.036995261412101722, -0.36931348230738353, 0.019376160245059969, 0.16120544790267191], [-0.0094418022192165416, -0.31873197405316661, 0.39737691638783773, -0.1749674342313301, 0.31155035438703099], [-0.0065926649697343048, 0.11308778161992744, -0.16672597175225454, -0.48096267013812422, 0.075600362742449051]])+(1.-msk_ref)*numpy.array([[-0.44658522266752171, 0.46948497048442706, 0.23442298268815048, -0.28949613194259549, -0.022166298524382153], [-0.3184726057645384, -0.16876313266167223, 0.098197749439149828, 0.42329642842353865, -0.14172988232057598], [0.48978992006764066, -0.44919352160303916, 0.25127590207362227, -0.070302512678130133, 0.059018820552160435], [-0.29916231243119129, -0.48360819276061889, -0.24766687760698261, 0.49230164638330687, -0.15742764307862869]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.5780916775235736, -0.80862720946171529], [0.27812300557645608, 0.35727068748997803]], [[0.98460785899983572, 0.82716777247937667], [0.21959138582361182, -0.2133332213415069]], [[0.027580148732079479, -0.044918347577630691], [-0.17987231170312579, 0.39853259003367847]], [[-0.61401067431067169, -0.38058562293266074], [-0.18881681930349203, 0.86589244342473615]], [[-0.17987750397336444, 0.62329961438715054], [0.13793756999207507, -0.50370722718237615]], [[-0.17285156602683793, -0.024605406483061865], [0.69795318482215651, -0.6507960568903397]]])+(1.-msk_arg0)*numpy.array([[[0.63175399429249635, -0.14450159884784863], [0.93524859893745682, -0.94154333609531249]], [[0.50964770235734447, -0.9634515749003103], [0.6114356464652202, 0.6964200718640341]], [[-0.56198145245822517, -0.79194218605500488], [0.81455167241006232, -0.72677826920878563]], [[-0.41688908078919251, -0.63357109024065328], [-0.37492211638541795, -0.72785224431168416]], [[-0.27119868495781474, -0.11054047908371656], [0.29762339283780181, -0.36038575745130741]], [[0.85983156360789725, 0.19852460023140606], [0.57986460095756942, 0.28086406304360501]]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(-0.244561030322)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.14137869627570185, 0.19775870349230948], [-0.068018048800011804, -0.087374487436375631]], [[-0.24079671246007583, -0.20229300268665754], [-0.053703495566844395, 0.052172992413176668]], [[-0.0067450295903496091, 0.010985277363944224], [0.04398975787650497, -0.097465540835506348]], [[0.15016308313808488, 0.093076412070127554], [0.04617723587097293, -0.21176354811193263]], [[0.04399102770346424, -0.15243479589378728], [-0.033734154237366115, 0.12318715846032767]], [[0.042272757080283725, 0.0060175235609876664], [-0.17069214999658361, 0.15915935420255511]]])+(1.-msk_ref)*numpy.array([[[-0.15450240775417173, 0.035339459897397021], [-0.22872536096329205, 0.23026480836822302]], [[-0.12463996718971981, 0.23562270982291231], [-0.14953333167509358, -0.17031721031194527]], [[0.13743876303500194, 0.19367819697701877], [-0.19920759625506138, 0.1777416423332944]], [[0.10195482312776996, 0.15494679861144681], [0.091691339073698822, 0.17800429479099936]], [[0.066324629815237443, 0.027033893456994199], [-0.07278708360032328, 0.088136312155643257]], [[-0.21028129309926907, -0.048551380776843227], [-0.14181228425740178, -0.068688404638349382]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.10551317775244073, 0.27309311360157995, -0.60323761293718059, 0.96422500651875298], [0.41531228628182548, -0.24376478331342666, -0.80312081096243615, 0.5380456500809383], [-0.17239874404195876, 0.035522164814309587, 0.62989630492571203, -0.0084690481771898529]], [[0.51138390843465165, 0.44683066876643762, 0.91153361995621984, -0.60974247184207808], [0.074865870912510291, 0.17428218303627152, 0.56159047113077998, -0.14451104022062422], [0.49890756054527707, -0.68744243007116279, 0.41194632764543337, 0.8274297400387447]]], [[[0.33883281568708878, 0.078885382270569737, 0.4872488661297889, 0.95183538020015246], [0.55951253591256078, -0.13146354495324264, 0.16167955233008469, -0.45497317116862135], [-0.30340301957243554, -0.98527534469281264, 0.94127271568372817, -0.54359919524951628]], [[0.25022715568556264, -0.38811054900744635, 0.76696471791345133, -0.7797521772046041], [0.050897989872080718, -0.93207234540984873, -0.59279395691585668, -0.31283312589997103], [0.81467486632983799, 0.58977112953298461, 0.34668676875334281, 0.61646659525126668]]], [[[0.20622668886291451, -0.85844245399761188, 0.099593440407274469, -0.3845644425753747], [-0.071074574666799784, -0.056533951953412043, -0.69998884912787895, -0.4318296461914326], [0.71503179624432867, -0.87194856864517822, 0.6380658458946975, -0.70620252276787765]], [[-0.50718523145228223, 0.0043983818308508393, 0.82563725208598648, -0.071453175878386732], [-0.19909143813620034, 0.17917986410112463, -0.75494433112303327, -0.67306927735072408], [0.60163823160117325, 0.30485744340186871, -0.97300209829714346, 0.93814211414862259]]]])+(1.-msk_arg0)*numpy.array([[[[0.86959679081158647, 0.034322949151554205, 0.49099260764271113, 0.7014480226129276], [0.15714706561175418, 0.3395338691742209, -0.22451512719467526, 0.00098259052913518552], [0.43462351378311381, 0.66240156558315588, 0.33080014369606481, -0.19032453136411065]], [[-0.27939029715150476, 0.91596500890308841, 0.27164156462391564, 0.36087849900912272], [-0.76834410713028145, 0.2076051257774012, -0.47614990077243902, 0.14029061419445221], [0.0027023515323207903, -0.98203107942670886, -0.1131275665329099, 0.76078942079224832]]], [[[0.9952551318760039, 0.34236791138117129, 0.10074745977698951, 0.54899952670053831], [-0.1419709788097181, 0.5527566379629727, -0.61022261621539786, 0.45212818591080595], [0.078379644804734472, 0.83450720259337352, 0.79690818889886916, -0.44770552959089316]], [[0.7205583740881305, 0.21466726881352516, -0.28195045405859398, 0.80731927371327061], [-0.49848648044845767, -0.4434379376822839, -0.70927868021516804, 0.52993741026917784], [-0.51487495207375589, -0.9378856206072812, -0.57622740965515007, -0.46854788092288735]]], [[[0.10498405710099412, -0.95175335304127717, 0.65680478805958353, -0.44305581895582247], [-0.11999524741644052, 0.23151184001873704, -0.90742013870485105, 0.36546589170349764], [0.24622131321919039, 0.38365834193415593, 0.68486634091374521, -0.59804512198286885]], [[-0.81290546564331945, 0.62322159938062738, 0.83585876014694604, -0.30805175336516899], [-0.25222865567250219, -0.99413780755208414, 0.1059027117257707, 0.8860328688810819], [0.62442422431123101, -0.29843752595640027, 0.52379929303217754, 0.87402660544795285]]]])
      arg1=Symbol(shape=())
      res=outer(arg0,arg1)
      s1=numpy.array(0.737900179992)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.077858192855098068, 0.20151545768129583, -0.44512914316454244, 0.71150180586337874], [0.30645901080042187, -0.17987407748279044, -0.59262299096483873, 0.39702398203886219], [-0.12721306425902826, 0.026211811810199399, 0.46480059678124225, -0.0062493121743128566]], [[0.37735027807915594, 0.32971643090888708, 0.67262082223483544, -0.44992907972129159], [0.055243539621630512, 0.12860285423193638, 0.41439770972942969, -0.10663472258969024], [0.3681439787259384, -0.50726389288393658, 0.30397526931678109, 0.61056055410566978]]], [[[0.25002479568284081, 0.058209537776224153, 0.35954102601827309, 0.70235949837284462], [0.41286440095789312, -0.097006973483439146, 0.11930337076546314, -0.33572478489704716], [-0.22388114275274346, -0.72703485419091862, 0.6945653063249756, -0.40112194401835194]], [[0.18464266321936759, -0.28638684396955105, 0.56594340339617022, -0.57537927190875759], [0.037557635887860659, -0.68777635144388305, -0.43742276750662862, -0.23083961990917956], [0.60114873050008699, 0.4351922226367213, 0.25582022906408164, 0.4548908115952231]]], [[[0.15217471083118511, -0.6334448413179713, 0.073490017602592037, -0.28377017139505306], [-0.052445941439516149, -0.041716413322105489, -0.51652189776414781, -0.31864717365072054], [0.5276220911489925, -0.64341100574740873, 0.47082890253271209, -0.52110696866151696]], [[-0.3742520735781355, 0.0032455667446602177, 0.60923787692269527, -0.052725311341691505], [-0.1469096080356517, 0.13221685397123695, -0.55707355781994228, -0.49665794090446636], [0.44394895938882611, 0.22495436235826746, -0.71797842346646301, 0.69225523488873664]]]])+(1.-msk_ref)*numpy.array([[[[0.64167562846069914, 0.025326910356802478, 0.36230353355450345, 0.51759862214140528], [0.11595884800019381, 0.2505421031771799, -0.16566975276797161, 0.00072505372830769911], [0.32070876904949708, 0.48878623447107056, 0.24409748557484415, -0.14044050595054997]], [[-0.20616215055623066, 0.67589074493634616, 0.20044435942940955, 0.26629230937422549], [-0.56696125494754879, 0.15319185967849303, -0.35135109748335497, 0.10352046946533323], [0.0019940656821022993, -0.72464091026711752, -0.08347685170673852, 0.56138665053892789]]], [[[0.7343989409496815, 0.25263334343179467, 0.074341568703219507, 0.4051068495680798], [-0.10476041081739082, 0.40787922264488169, -0.45028337834078663, 0.33362546976322938], [0.057836354009155398, 0.61578301499861998, 0.58803869602590786, -0.33036199086872114]], [[0.53170015393467596, 0.15840301629598111, -0.20805129079878062, 0.59572103738437099], [-0.36783326364670416, -0.32721293403122415, -0.52337686579555753, 0.39104091042234246], [-0.37992631980881231, -0.69206596825841393, -0.42519830930110031, -0.34574156566806485]]], [[[0.077467754631157903, -0.70229897051754631, 0.48465637132904898, -0.32693096855418979], [-0.088544514666826252, 0.17083262842020205, -0.66958548367905502, 0.26967734726910064], [0.18168675134241025, 0.2831015595688065, 0.50536299623100167, -0.44129760315474681]], [[-0.59984308941502618, 0.45987533035812789, 0.6167803295606713, -0.22731144425513833], [-0.18611957041998511, -0.73357446712995122, 0.078145630044131414, 0.65380381342654925], [0.4607627475108837, -0.22021710411971984, 0.38651159260834539, 0.64494438947820698]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.791652550411)+(1.-msk_arg0)*(0.27657970676)
      arg1=Data(0.524788097784,self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.41544983603581132)+(1.-msk_ref)*numpy.array(0.14514573819592874)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.2777005478551926, -0.11730389760971582])+(1.-msk_arg0)*numpy.array([0.14278735863381065, -0.59771418621042871])
      arg1=Data(-0.929492640069,self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.25812061537453374, 0.10903310947963972])+(1.-msk_ref)*numpy.array([-0.13271979894502137, 0.55557093694743187])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.13317664602533652, -0.8278119096032821, 0.81800967280777881, -0.49731655159751642, -0.46639600919465596], [0.54488677268272023, 0.29640062843549808, -0.067602651839557826, 0.070422007044289581, 0.10857751872635713], [-0.10303795811028849, 0.92312867631464468, 0.83674986777882965, 0.56457088498604824, 0.24440215677678134], [0.88430294741407822, 0.78808073457631256, -0.48973872679301778, 0.44081702009759072, -0.80189819054052025]])+(1.-msk_arg0)*numpy.array([[-0.8024246446648049, 0.150768401322257, 0.36988293952586004, -0.03966117250769452, 0.56627044745398591], [0.26077699778899999, 0.62989660003720349, -0.99418330021016099, -0.90421489639872443, -0.2575086193377818], [-0.20703755181091976, 0.57518613221448489, -0.29410545024012147, 0.089755770194008822, 0.38574047734125716], [-0.34763454259115556, 0.48021712485292833, -0.84799225914637599, 0.96309475859407079, -0.78914836659374887]])
      arg1=Data(-0.355722959141,self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.047373990612636169, 0.29447170209642776, -0.29098482141732834, 0.17690691536417824, 0.16590776852238098], [-0.19382873517560922, -0.10543650863839409, 0.024047815358161487, -0.025050724734458879, -0.038623516257551525], [0.036652967362861402, -0.32837806440676748, -0.29765113902731405, -0.20083082585221726, -0.086939458429134253], [-0.31456686123144295, -0.28033841094567619, 0.17421130910086821, -0.15680873482893182, 0.28525359726906774]])+(1.-msk_ref)*numpy.array([[0.28544086908801036, -0.053631781863345038, -0.1315758537839937, 0.014108389647447693, -0.20143539924255741], [-0.09276436532946776, -0.22406868251823067, 0.35365382547954732, 0.32164999864653043, 0.091601728075207015], [0.073648010583535067, -0.20460691300833245, 0.10462006105897831, -0.031928188173412579, -0.13721674406038065], [0.12366158819023207, -0.17082425668297477, 0.30165031575240137, -0.34259491746048659, 0.28071819216619254]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.71614527258053018, -0.04214798739915282], [-0.85602591015480667, -0.63437029957306024]], [[0.48616819208344642, -0.68488217634113524], [0.16967198365605829, 0.51842248585142947]], [[0.12768328872289358, -0.6502475147161737], [0.25458052156920608, 0.82355211118303573]], [[0.34057178868714955, 0.67446046215066358], [0.051489994935964178, -0.39981816352451194]], [[0.83849369299165977, -0.50780980914803031], [-0.099483057187065294, 0.16116058957979695]], [[-0.59619809542046154, 0.17687119993864409], [-0.59048580388867711, 0.52200367122135582]]])+(1.-msk_arg0)*numpy.array([[[0.15979749909800622, 0.79913484946863145], [0.78599981054426094, -0.33299470529963315]], [[-0.63169398011950184, 0.97937647208491163], [0.41559571088800507, -0.89012278000386758]], [[-0.61202692897310218, 0.50430213793464462], [-0.674353480635832, -0.90845669378286154]], [[-0.88096226389709842, 0.27048679451685098], [-0.88479897767137117, -0.97502331724884006]], [[-0.46088647641014102, -0.54914351217085633], [0.8665650718293656, -0.99478510823778965]], [[-0.24029797246396223, -0.071361817533693195], [-0.72546360451150349, -0.61207451014397418]]])
      arg1=Data(-0.631902761531,self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.45253417540070151, 0.026633429630480432], [0.54092513656854391, 0.40086034413319321]], [[-0.30721102314585552, 0.43277893855303179], [-0.10721619502663338, -0.32759260044906235]], [[-0.080683422745301839, 0.41089320022754167], [-0.16087013461147476, -0.52040485332089326]], [[-0.21520825377081634, -0.42619342857619036], [-0.032536669991230949, 0.25264620164122104]], [[-0.52984648012739777, 0.32088642073295326], [0.062863618562010407, -0.10183782160536888]], [[0.37673922291545731, -0.1117653996764552], [0.37312961012185503, -0.32985556137387168]]])+(1.-msk_ref)*numpy.array([[[-0.10097648096570927, -0.50497551821454645], [-0.49667545084542514, 0.2104202738538973]], [[0.3991691704797517, -0.61887069728852517], [-0.26261607739039178, 0.56247104278571403]], [[0.38674150654917872, -0.31866991360667352], [0.42612582666153559, 0.57405629353232368]], [[0.55668248736080128, -0.17092135241275061], [0.55910691738996665, 0.61611992672624138]], [[0.29123543719566308, 0.34700530181736156], [-0.54758486193491496, 0.62860745702494858]], [[0.15184495239017518, 0.045093729567381551], [0.45842245508074203, 0.38677157322244959]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.33614499471732762, -0.38763051892321343, -0.67310246736468238, 0.54147114643914462], [0.79807775395112168, 0.65747863038277421, -0.82356537264935969, 0.97369681738798897], [-0.55352515849093664, -0.79137549467897039, 0.1987852106562471, -0.7374247699467178]], [[0.90001316105185181, 0.89817527500492433, -0.39221865336499651, 0.2541763072784724], [0.85319027771590972, -0.64517033625387277, -0.6040202851658163, -0.027827968094352507], [0.33853315146414942, 0.73257354535595054, -0.43164449645334657, 0.5741811015673548]]], [[[0.36107836699350915, -0.017101929430308527, 0.90242939202759342, -0.39777338170880627], [0.97430916784023713, 0.67115836087484748, 0.58825808307274419, 0.05087528266536756], [-0.96855449930468684, -0.92599616247707472, 0.017156628965631304, 0.98848943258679478]], [[-0.1015215647727179, 0.033919722549923037, -0.48391259268061093, -0.038419572012241332], [-0.31296886312169048, 0.6763235149893454, -0.43060770967905682, -0.56601062303268557], [-0.51970228140602592, -0.098958271363178474, -0.89913559217507522, -0.75328978108805522]]], [[[0.7928607866930224, 0.93404095404258913, 0.058432793246443682, -0.36756309946219856], [0.57169716993400965, 0.18136666084464381, 0.0030118825408098626, -0.61854504527165433], [-0.49261584481107556, 0.67908138854072941, -0.079794244062769026, 0.70216268024066131]], [[-0.23110518553918236, 0.221210552858077, -0.9089179127211533, -0.65776227434369638], [-0.36972745210471825, 0.67252662597655788, 0.42985612896673686, -0.42719967674258852], [0.61250180881279626, 0.40884633171139528, 0.73839275838093421, -0.58021136703013321]]]])+(1.-msk_arg0)*numpy.array([[[[-0.31009462545392563, 0.19911016528579473, 0.15798365667222125, -0.33924318891169092], [-0.46683321503505382, 0.96265767240208344, 0.16348606888355133, -0.04670429266016729], [-0.22730536037283411, -0.60736704221980231, 0.11646128721190485, 0.19817799892539867]], [[0.22650372920304318, -0.30264633978965572, -0.17506598233551052, -0.13724912529266642], [0.058668115865703419, 0.051609198623017383, -0.87565988190737931, -0.094481322335332907], [0.50273756152670601, -0.019114983721407741, -0.31015293298428226, -0.43861313303089045]]], [[[0.77531930002959948, -0.39709154910793831, -0.33698975134292386, 0.65281235409873872], [0.34828176289241219, 0.66750619882050244, 0.64624691865888506, -0.7148248388267775], [0.2906366405892904, 0.92778907252451792, 0.40218686572635032, 0.92138695697192396]], [[-0.98774207903531885, -0.44795498518746713, 0.54602942308871483, -0.62025586524331833], [-0.20252787584458476, -0.31858728562069905, -0.52867934500457392, -0.53999227776338721], [-0.040113310138338187, -0.98575276362842268, 0.24615633737850562, -0.054113937410190882]]], [[[-0.64735009236087149, 0.21069057561826776, 0.3280043377298274, -0.99782439129055489], [0.65027332347529598, -0.68432726910756392, 0.51695839859198411, -0.71134830648984892], [0.19173752494194862, 0.14071904346667186, -0.85589771591759933, -0.69208088909502607]], [[-0.73371064987758849, -0.32688482596206314, 0.15057874653305237, 0.16123884704326841], [0.40523773076670322, -0.80911074177084941, 0.087570240521975373, 0.51863272802496585], [0.78101380659994502, 0.185492143914324, 0.043925946630618329, -0.73751576574241118]]]])
      arg1=Data(0.880552249443,self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.29599323123725768, -0.34132892539048859, -0.59270189174342325, 0.47679363600532304], [0.70274916147186239, 0.57894428694407751, -0.72519234144953448, 0.85739092282622142], [-0.48740782343233829, -0.69684747199342112, 0.17504076439930547, -0.64934103997137049]], [[0.79250861349227075, 0.79089025879942942, -0.34536901749394633, 0.22381551912910561], [0.75127861824541242, -0.56810619086207059, -0.53187142081179917, -0.02450397990290264], [0.29809612803269342, 0.64506928324541857, -0.38008553231156911, 0.50559646057263929]]], [[[0.31794836828124234, -0.015059142429669011, 0.79463623111313331, -0.35026024603213129], [0.85793012939439584, 0.59099000440064298, 0.5179919783025736, 0.04479834459202417], [-0.85286284307061977, -0.81538800384452448, 0.015107308228540955, 0.87041659341467059]], [[-0.089395042227562629, 0.029868087991808047, -0.42611032201857585, -0.033830440558006082], [-0.27558543642733863, 0.59553819247488249, -0.37917258738527582, -0.49840192731991273], [-0.45762501293259517, -0.08713792844981097, -0.79173586824378417, -0.66331101121930924]]], [[[0.69815534921747391, 0.82247186315383614, 0.051453127534377988, -0.32365851404358109], [0.50340922898543528, 0.15970282118066795, 0.0026521199463674116, -0.54466123099561092], [-0.43377399025952373, 0.59796664423423229, -0.070263001102053524, 0.61829092756065118]], [[-0.20350019098440666, 0.19478744991964977, -0.80034971260540311, -0.5791940502719084], [-0.32556433963153913, 0.59219483331378808, 0.37851078129840471, -0.37617163631689371], [0.53933984553784908, 0.36001055706487795, 0.65019340436455431, -0.51090642439062561]]]])+(1.-msk_ref)*numpy.array([[[[-0.27305451998355529, 0.17532690392932046, 0.1391128642579125, -0.29872135310431469], [-0.41107103761369862, 0.84767037887695795, 0.14395802570796035, -0.041125569960541924], [-0.2001542463866896, -0.53481841526402496, 0.10255024842743875, 0.17450608274381887]], [[0.19944836825690682, -0.26649591528739036, -0.15415474454643513, -0.1208550260105046], [0.051660341396111906, 0.045444595939434639, -0.77106427876029882, -0.083195740912701099], [0.44268669068169419, -0.016831741913946766, -0.27310586281057025, -0.38622178092547427]]], [[[0.68270915367742724, -0.34965985680169365, -0.29673708358415807, 0.57483538686564883], [0.30668028975479333, 0.58777408488836003, 0.5690541779205146, -0.62944061978645638], [0.25592074764137812, 0.81696675481984904, 0.35414654931165929, 0.81132935756882019]], [[-0.86975850956378953, -0.39444776985590929, 0.48080743676268434, -0.54616769737005122], [-0.1783363766498072, -0.2805327509971603, -0.46552978647768706, -0.47549141486625446], [-0.035321865474907586, -0.86800681340739605, 0.21675351659322686, -0.04765014931274672]]], [[[-0.57002558000532488, 0.18552406029704943, 0.28882495741497266, -0.87863651229972162], [0.57259963773877232, -0.60258591616766632, 0.45520888074848337, -0.62637935141691348], [0.16883490889021455, 0.1239104702640073, -0.75366265904413743, -0.60941338368895026]], [[-0.64607056318979839, -0.28783916880959021, 0.13259245397794606, 0.14197922946150238], [0.35683299538568819, -0.7124642837145978, 0.077110172275866376, 0.45668321529700362], [0.68772346424741138, 0.16333552457771272, 0.038679091114492303, -0.6494211665239582]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.264591594097)+(1.-msk_arg0)*(-0.427834372758)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.12978686673)+(1.-msk_arg1)*(0.427514232563)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array(0.034340513960799644)+(1.-msk_ref)*numpy.array(-0.18290528353399987)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.21293714109293549, -0.97325505842674231])+(1.-msk_arg0)*numpy.array([-0.6125640006994324, 0.78735279410138892])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.233178513628)+(1.-msk_arg1)*(0.731625996957)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.04965236605634122, 0.22694216790531338])+(1.-msk_ref)*numpy.array([-0.44816774771145035, 0.57604777294099929])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.32941699506058231, 0.50568121904021801, -0.35213527917652732, 0.69364322007306822, -0.48890801916381332], [0.91487273015996218, 0.35548718050125561, 0.9280950949893787, 0.76483319225020319, -0.13978070240719553], [0.47902080752887843, 0.24418645006654782, 0.39670242841894154, -0.64460121466056619, 0.092516635563920646], [-0.66523647553523979, 0.58631041062426514, -0.45387928658778942, 0.89804929988327054, -0.34436500698567896]])+(1.-msk_arg0)*numpy.array([[0.45830684062404137, 0.02169027792878131, -0.48337085922610523, -0.4796207471344418, -0.87794907194563687], [-0.53017579701142581, 0.24506177783476679, 0.88874600023393135, 0.86671281296747926, 0.64119254334737619], [-0.14087777246442301, 0.94002508153033104, 0.89390247775514986, 0.51517305006431546, 0.83904630357552934], [-0.00086796920917087128, -0.77123745415412803, -0.77950764862612276, 0.43488999834439879, 0.21147604980506163]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.581754419525)+(1.-msk_arg1)*(0.905971318599)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.19163979274307527, -0.29418228404733882, 0.20485625493154649, -0.40353000885092577, 0.28442440088966742], [-0.5322312540732842, -0.20680623834101824, -0.5399234232493666, -0.44494508979082209, 0.081318041389667833], [-0.27867247182426697, -0.14205654651428778, -0.23078339096894263, 0.37499960545984318, -0.053821961618876771], [0.38700425967173191, -0.34108867259407061, 0.26404628090321297, -0.52244414915825133, 0.20033586474360976]])+(1.-msk_ref)*numpy.array([[0.41521285272297331, 0.019650769695910581, -0.43792013470526781, -0.43452264070869051, -0.79539667837310557], [-0.48032406590756543, 0.2220189420031087, 0.80517838573126754, 0.78521695001054748, 0.58090205397208527], [-0.12763122128084287, 0.85163576262989737, 0.80985000647049044, 0.46673200747328902, 0.76015188601569894], [-0.00078635520893561705, -0.69871901329273034, -0.70621157228359122, 0.39399786524546737, 0.19159123569393893]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.33745884555915162, -0.66375221821549824], [0.35050591871233383, 0.042176466180520178]], [[0.20748673291497721, -0.76371032387997184], [-0.32482792758474766, -0.70921662870826352]], [[0.50212745454878882, 0.36618836225151474], [0.19785234293440346, -0.77302966670008377]], [[0.64935634997914415, 0.57990195989895033], [-0.80742363450638099, -0.21732793929102523]], [[-0.1072984723427084, 0.25601710633311225], [0.55861416589548618, 0.51491586728123639]], [[-0.1317501155335572, 0.83648625632547402], [0.091416210119983576, -0.48358358241352173]]])+(1.-msk_arg0)*numpy.array([[[0.42886586178498498, -0.069787877992939507], [0.30684797181993506, 0.46417129346574049]], [[0.77136474775699138, 0.078418535081947294], [-0.49859534298556052, -0.72222304926459113]], [[0.4388243207897955, -0.11542252838195743], [-0.38927617211922882, 0.048609512811642386]], [[0.96883423294606907, -0.87835366478764509], [-0.51994722843107555, 0.07625890302971694]], [[-0.22900078843687521, 0.19274199077969256], [-0.7277710383793552, -0.51046116944335274]], [[0.81208765776357139, -0.22720534187686625], [0.60017241969465607, 0.43244947860361416]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.637368901774)+(1.-msk_arg1)*(-0.36037041184)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.21508577378804244, 0.42305502237423392], [-0.22340157247505449, -0.026881967930196909]], [[-0.13224559109074599, 0.48676521040503384], [0.20703521947029596, 0.45203262375982139]], [[-0.32004042425646079, -0.2333970742907589], [-0.12610493052956287, 0.49270506970354627]], [[-0.41387954364634211, -0.36961147531752869], [0.51462671519190484, 0.13851806999078145]], [[0.068388709479126703, -0.16317734189895694], [-0.35604329743234431, -0.32819136083517669]], [[0.083973426446253802, -0.53315032654342054], [-0.058265849448537937, 0.30822113683896346]]])+(1.-msk_ref)*numpy.array([[[-0.15455056723538993, 0.02514948633372573], [-0.11057892997688877, -0.16727360019035784]], [[-0.27797703182771805, -0.028259719783337627], [0.17967900909300136, 0.26026781770351548]], [[-0.15813930120824105, 0.041594864088571208], [0.14028361446594037, -0.017517430151252737]], [[-0.3491391915310551, 0.31653267192032503], [0.18737359684455307, -0.027481452291253424]], [[0.082525108440584458, -0.069458510596057663], [0.26226714882568447, 0.18395510186041289]], [[-0.29265236367809527, 0.081878082624318074], [-0.21628438206011838, -0.15584199670419457]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.13152428269210881, -0.3249269355703337, 0.50929908793572332, 0.54884977050839079], [0.93776551306946487, 0.10668163504602424, -0.84845010039488855, -0.9726067912742018], [0.1083611448838746, 0.84834672122815569, -0.92827615365710492, -0.18820948907656354]], [[-0.83470741457517228, -0.75974999350460037, -0.61856244293820262, 0.53074924989214667], [-0.42304235638745968, 0.60869483277582392, 0.24882199006295203, -0.83711417504221308], [0.34239912938615635, 0.32163181395462992, 0.57530808877203632, -0.48131089429291762]]], [[[0.27536846712519769, 0.31419276389072537, 0.86070347055427066, -0.28452411904104302], [0.17730288224396284, 0.16756821629920848, 0.66092392041239179, -0.71636704098355231], [-0.19092608481195517, 0.41713386566086319, -0.75212160203626044, 0.69628026918488284]], [[0.39790165641326158, 0.63795247582254455, 0.46050525883551074, -0.77107410798654508], [-0.89469611490295264, -0.91344706342184856, 0.083105874082978648, 0.73203700836399621], [0.83297065676902582, -0.3314079580353988, 0.52631137651761151, -0.065404468481846223]]], [[[0.069369776635408664, 0.14627252260314005, 0.3527723050915994, -0.90634105218740446], [0.2810853455317881, -0.89740910295554155, 0.23938138895920114, 0.98563119512268726], [0.26555452729196527, -0.024169136008622916, 0.34220225993250453, -0.97686411145602192]], [[-0.61998394665672785, 0.67363885378633603, -0.39612107107938477, -0.95171740177791797], [0.72427341015815094, -0.7096180514308017, -0.92874702670260212, -0.15181963621277306], [0.42525537412781644, 0.55610853161171336, 0.82710212515693238, -0.93985339008584168]]]])+(1.-msk_arg0)*numpy.array([[[[0.24363200118490158, -0.1513324253842474, 0.57133660366702466, 0.57053434632287159], [0.48129939414584433, -0.48919366668235287, 0.83801855845680429, 0.71261770393977741], [0.67852259714775998, 0.44877787453470797, 0.12516534181874928, 0.0025008086222404469]], [[0.097901205990076567, 0.72150826959730896, 0.52873140790976669, 0.37216804509190715], [0.80008770995803724, 0.057197703972295288, 0.62784333251223257, -0.10801727402453931], [0.41920817219938389, -0.72998152074020539, 0.76777140255742826, 0.072679087275026255]]], [[[0.80301108161918444, 0.093661774013343857, 0.4100575037780303, 0.017742627021203994], [-0.63220586939490575, 0.87175869618491353, 0.79014166286733545, -0.96226421942608464], [-0.14275240493695485, -0.23295462209719209, -0.65589256553715414, -0.0025931052289895895]], [[0.84256758603185, 0.68983388564910175, -0.35000681991373761, -0.50778119657276477], [0.047412329402986009, 0.44722652266486396, 0.38058248940630457, 0.049682360264559744], [0.61095978397269701, -0.6566569411821479, -0.2181239549048084, -0.33434575728366167]]], [[[0.71884062279606309, -0.56019619993759928, -0.11950311285462911, 0.89269367228125018], [-0.51800440731713415, -0.50124129173870924, -0.35877777621882045, 0.28245223081806858], [-0.85623006074111352, 0.0017039409286681018, 0.84801473994961341, 0.69242336798734838]], [[0.63349834503210944, 0.76984527473816655, 0.29278199906355118, 0.45303590963602947], [-0.88505218143270015, 0.012178144653693268, -0.83892758808741852, -0.92284598773509319], [-0.87068117618378538, -0.45190637812759382, -0.3925910200527376, 0.22267783977567768]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0844199344821)+(1.-msk_arg1)*(0.941872663068)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.011103271327677369, 0.02743031061232783, -0.042994995635343869, -0.046333861666852098], [-0.07916610317292809, -0.0090060566410321927, 0.071626101886695687, 0.082107401596245758], [-0.009147840751505619, -0.071617374624213276, 0.078365012073059251, 0.015888632736759226]], [[0.070465945250186693, 0.064138044674459355, 0.052219000905951218, -0.044805816902336273], [0.035713208009396515, -0.051385977902547973, -0.021005536098828367, 0.07066912381112847], [-0.028905312069518709, -0.027152136661419458, -0.048567471161176476, 0.040632234161744961]]], [[[-0.023246587953154593, -0.026524132542415327, -0.072660530592736042, 0.024019507488031472], [-0.014967897702528691, -0.014146097841267077, -0.055795154058888623, 0.060475658664990994], [0.016117967570755429, -0.035214413609368951, 0.063494056366498033, -0.058779934705789759]], [[-0.033590831764739752, -0.053855906211653766, -0.038875823779571529, 0.065094025677093298], [0.075530187401526228, 0.077113141246969275, -0.0070157924451654547, -0.061798516284585228], [-0.070319328269980483, 0.02797743810420586, -0.044431171922818015, 0.0055214409440761929]]], [[[-0.0058561919986014023, -0.012348316774693406, -0.029781014882943865, 0.076513252244128194], [-0.023729206453681276, 0.075759217675176599, -0.020208561172177762, -0.083206920915803742], [-0.022418095795421516, 0.0020403568783377087, -0.02888869236313989, 0.082466804287064474]], [[0.052339004156735223, -0.056868547901261682, 0.033440514867514023, 0.080343920703597707], [-0.061143113832702163, 0.059905909409127184, 0.078404763144709524, 0.012816603742183573], [-0.035900030822045237, -0.046946645803616044, -0.069823907215781231, 0.079342361613857368]]]])+(1.-msk_ref)*numpy.array([[[[0.22947032176470283, -0.14253587450525856, 0.53812632840430608, 0.53737070414310251], [0.45332274209734597, -0.4607581415942949, 0.78930677135443816, 0.67119513455943514], [0.63908188552763667, 0.42269161181417431, 0.1178898138226899, 0.0023554432768539851]], [[0.092210469603479797, 0.67956891531147867, 0.49799765921586775, 0.35053490773966883], [0.75358074206646097, 0.053872953761782826, 0.59134847158302517, -0.10173851754288017], [0.3948407175294632, -0.68754963893028576, 0.72314289555451294, 0.068454445481108447]]], [[[0.75633418591808432, 0.088217464517657301, 0.38622195309458718, 0.016711295362290466], [-0.59545742581444261, 0.82108568472870647, 0.74421283220613821, -0.90633036292626579], [-0.13445458779738592, -0.21941359028877122, -0.61776727738923365, -0.0024423749276449754]], [[0.7935913760709179, 0.64973567895113027, -0.32966185556424821, -0.47826522787204034], [0.044656376957065855, 0.42123043589716824, 0.35846024281431099, 0.046794456969903725], [0.57544631875804841, -0.61848722191356853, -0.20544499028519989, -0.31491112879837785]]], [[[0.67705633171466328, -0.52763348667601528, -0.1125567151493511, 0.84080376641583598], [-0.4878941906009488, -0.47210547028977462, -0.33792297953697298, 0.26603403483022003], [-0.80645968750943642, 0.0016048953801958397, 0.79872190143758515, 0.65217464157702321]], [[0.59667477328480667, 0.72509621906824828, 0.27576336115647193, 0.42670213867449464], [-0.83360645508049946, 0.011470261536206077, -0.79016296151343279, -0.86920340807002505], [-0.82007079809573435, -0.42563826382462472, -0.36977074955380512, 0.20973416995583236]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.809916642498)+(1.-msk_arg0)*(0.858853571425)
      arg1=numpy.array([0.18792098079616504, -0.0087948976168672477])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.15220032982133061, -0.0071231339489653553])+(1.-msk_ref)*numpy.array([0.16139660550243365, -0.0075535292285617128])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.69225378146116934, -0.05852530104787812])+(1.-msk_arg0)*numpy.array([-0.90322445384304317, 0.97425063291510239])
      arg1=numpy.array([-0.94088931049844371, -0.95517417934997439])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.65133418312893998, -0.6612229376090889], [0.055065830149651891, 0.055901856399617179]])+(1.-msk_ref)*numpy.array([[0.84983423360171428, 0.86273667646835761], [-0.91666200625616312, -0.93057904877587605]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.68854400539697558, -0.65841481073462838, 0.093032278271191648, 0.90655175598475313, -0.3675552924858676], [0.10025318514000503, 0.70593657398898335, -0.31748455253330365, -0.43299186356982067, 0.69988564454522106], [0.64562779777595503, 0.070769172172916894, -0.86753891003133443, -0.072516413535663293, -0.3535574126470018], [0.761728829327315, 0.31243787616296226, -0.3731441451355042, 0.59820441601917862, 0.8188916120757328]])+(1.-msk_arg0)*numpy.array([[-0.38284500969415047, -0.21780851164808479, 0.13254498807361514, -0.27159469939107406, 0.80967081260907348], [0.2985837063418848, -0.7067565097923707, -0.16488817957428337, -0.34413067911586759, 0.93265287047179957], [0.37419307563750981, -0.23034427559306847, 0.36688817263928009, -0.07634503898964029, -0.28028449653149945], [0.23430328570643932, -0.54406841361731795, 0.44301529307210474, 0.14940947211625, -0.041130763768178147]])
      arg1=numpy.array([0.9852682146970666, -0.49287947409927302])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.67840052293784547, 0.33936920727426834], [-0.64871518510261428, 0.32451914565405604], [0.091661746721457701, -0.045853700388562166], [0.8931966301495885, -0.44682075273353761], [-0.3621405468300089, 0.18116045926283889]], [[0.09877627674058724, -0.049412737168582729], [0.69553686794348935, -0.34794164733513266], [-0.31280743826838514, 0.15648161928725773], [-0.42661312039779303, 0.21341280200555737], [0.68957507949317576, -0.34495926841307928]], [[0.636116547673514, -0.31821668943168452], [0.069726615922399149, -0.034880672363028185], [-0.85475851306681194, 0.42759212173690064], [-0.071448117300517172, 0.035741851767023131], [-0.34834888075162557, 0.17426119160935391]], [[0.75050720375461022, -0.37544050480510194], [0.30783510845082501, -0.15399421609189462], [-0.36764706570232131, 0.18391509001761011], [0.58939179699511746, -0.29484267797139552], [0.80682787666026012, -0.40361486710419309]]])+(1.-msk_ref)*numpy.array([[[-0.37720501920703681, 0.18869644703958396], [-0.21459980341733373, 0.10735334467545342], [0.13059236376633479, -0.065328704016217848], [-0.26759362459023001, 0.13386345260402271], [0.79774291603166503, -0.39907012431229116]], [[0.29418503528510204, -0.14716578015639994], [-0.69634472462865893, 0.34834577686270135], [-0.16245908231380349, 0.081269999233759271], [-0.33906101983497994, 0.16961494814405451], [0.91891322862184444, -0.45968545631531799]], [[0.36868054358537372, -0.18443208633180533], [-0.22695089317927167, 0.11353196541608959], [0.36148325484977267, -0.18083164958369166], [-0.075220340266300825, 0.0376289026673024], [-0.27615540550485662, 0.13814647524862497]], [[0.23085158000564018, -0.11548328023872152], [-0.53605331455780003, 0.26816015357772943], [0.4364888868886504, -0.21835314466731429], [0.1472084038508088, -0.073640862042107291], [-0.040524834186999675, 0.020272509215361077]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.11711260368278764, 0.10980142262859216], [-0.91665357370244083, -0.093333775529722329]], [[0.47884791251865955, 0.81001994543770106], [-0.48359066455925381, -0.38107346610484627]], [[0.62252050489004418, 0.50277828007388337], [0.28603879958825051, 0.69684786424599521]], [[-0.34602034382597702, 0.91199419628949729], [0.69639120035850022, -0.6737254134675501]], [[-0.51849356148680248, -0.36035315909332022], [-0.7751326363239297, 0.29837003569811249]], [[0.56940063451215206, 0.23880928332203633], [0.87412068108193952, 0.84235747779910075]]])+(1.-msk_arg0)*numpy.array([[[-0.73797274562881965, 0.087647337119837188], [0.42606202197474197, 0.99157082131401553]], [[-0.64032420926864053, 0.15915908019775871], [0.12876515050279203, -0.30175036005486278]], [[-0.3189537853451887, 0.56434647528486237], [0.41060862455421976, 0.42765641045758773]], [[0.79174083353568236, -0.054672767013720813], [0.26963343963365571, -0.98947217950649047]], [[-0.43169389761932053, -0.23379247312147111], [0.54814164739271654, 0.61555291583611349]], [[0.35096141936800329, -0.80263157305871813], [-0.26078342225015771, -0.13421208138753649]]])
      arg1=numpy.array([0.13180785873603296, 0.57438234807218902])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.015436361522429887, 0.067267412292167261], [0.014472690402844927, 0.06306799895107755]], [[-0.12082214475245109, -0.52650963203197132], [-0.012302125100322251, -0.053609273143204531]]], [[[0.063115918009303745, 0.27504178836193383], [0.10676699454162163, 0.46526115824581316]], [[-0.063741050000290428, -0.27776594141533451], [-0.050228477588398018, -0.21888187224930933]]], [[[0.082053094768830853, 0.35756478932182822], [0.066270128515524038, 0.28878696906853379]], [[0.037702161689152566, 0.16429563734724961], [0.091850024851042406, 0.40025711251470475]]], [[[-0.045608200598807935, -0.19874797756751086], [0.12020800219260797, 0.5238333678929703]], [[0.091789832961869613, 0.39999481283872562], [-0.088802304125206247, -0.38697598494339785]]], [[[-0.068341526107995085, -0.29781354930710152], [-0.047497378288855564, -0.20698049365525237]], [[-0.10216857303027334, -0.44522250371912492], [0.039327515516361919, 0.17137848169866471]]], [[[0.075051478397985275, 0.32705367344488423], [0.031476940280964233, 0.13716783689594786]], [[0.1152159752502932, 0.50207948929830548], [0.11102933543898488, 0.48383526601441434]]]])+(1.-msk_ref)*numpy.array([[[[-0.09727060740688584, -0.42387851844756169], [0.011552607829680957, 0.050343083297166816]], [[0.056158322805235361, 0.24472250460623687], [0.13069682674252994, 0.56954077662621316]]], [[[-0.084399762920542978, -0.36779092284718951], [0.020978417559263121, 0.091418166200998491]], [[0.016972258767596036, 0.073960429495662491], [-0.039773068831658431, -0.17332008033994056]]], [[[-0.042040615482101612, -0.18320142415308244], [0.074385300492525258, 0.32415065360038281]], [[0.054121443581039388, 0.23584634591014464], [0.056368475737152653, 0.24563829320675309]]], [[[0.10435766394222021, 0.45476195903085737], [-0.0072063003512525554, -0.03140307229294468]], [[0.035539806321743563, 0.15487268817555999], [-0.13042020925962614, -0.56833535381704448]]], [[[-0.056900648274614878, -0.24795735456302051], [-0.030815685270742646, -0.13428626967311472]], [[0.072249376826875572, 0.31484288650558639], [0.081134711775079638, 0.35356272916062942]]], [[[0.046259473185855396, 0.20158604413934197], [-0.10579314899880343, -0.46101740757034126]], [[-0.034373304480648022, -0.14978939441034672], [-0.017690207064197366, -0.077089050447028948]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.254067258389)+(1.-msk_arg0)*(0.263270679776)
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([0.086029819078471848, 0.83425209198936989])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.021857360272945936, 0.21195614181680308])+(1.-msk_ref)*numpy.array([0.02264912894976729, 0.21963411536231719])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.82405756311567191, -0.86113596519171143])+(1.-msk_arg0)*numpy.array([0.76687280756177745, -0.30799700415619036])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.99441661760012701, -0.66752330528391268])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.81945653462128965, -0.55007762827517981], [0.85632791379976236, 0.57482832578362364]])+(1.-msk_ref)*numpy.array([[-0.76259106342509586, -0.51190547123599162], [0.30627733910397109, 0.20559517823188317]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.3424310295279609, -0.26832572848266145, 0.41401204013330717, -0.31330034444485921, -0.94812610177476819], [0.052831764714741603, 0.038019317469260017, -0.50820528889003369, -0.4936268761551732, 0.78359486103859699], [-0.6757233899231565, 0.57233381077891021, -0.80678252392568139, 0.46193075293023944, -0.57418497462518037], [-0.52839732781678639, 0.56850559614991192, 0.8565202361336417, -0.58127977123427166, 0.68963340783668836]])+(1.-msk_arg0)*numpy.array([[0.27788032216884639, -0.43830093584453977, 0.75390030825651921, 0.59691012726848469, 0.97888283225841199], [-0.81682616272870034, -0.23449564964363434, -0.26356216703327018, -0.82556685437593824, 0.56852460301604935], [-0.49044884799599742, -0.67296675456849653, 0.86317964179036899, 0.59429327303399226, 0.81029252192187129], [-0.38456885604240076, 0.6690785692129293, -0.028403477285916523, 0.093223787502714206, 0.99429369387340016]])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([-0.070606940989106137, -0.84084762315660422])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.024178007494719597, 0.28793231727365487], [0.018945658876834193, 0.22562105102641022], [-0.029232123686471861, -0.34812103990430798], [0.0221211789320848, 0.26343784996060526], [0.066944283718242292, 0.79722957913005044]], [[-0.0037302892935641, -0.044423463787559424], [-0.0026844277049981342, -0.031968452728063645], [0.035882820843010248, 0.42732320923880024], [0.034853483715325116, 0.41506498554129684], [-0.055327236112719044, -0.65888387642203383]], [[0.04771076151726307, 0.56818040632820943], [-0.040410739603736749, -0.48124552444560831], [0.056964446057862696, 0.67838116764719536], [-0.032615517413198784, -0.38841337566433243], [0.040541444620191512, 0.48280207116581803]], [[0.037308518943961207, 0.44430163717704585], [-0.040140441079333435, -0.47802657927388176], [-0.060476273768663297, -0.72020300473850607], [0.041042386505699337, 0.48876771403135194], [-0.048692905331241224, -0.57987661182886852]]])+(1.-msk_ref)*numpy.array([[[-0.01962027950940954, -0.23365500841766593], [0.030947088312645414, 0.36854430013219652], [-0.053230594576736977, -0.63391528229452543], [-0.042145998131845733, -0.50191046175181153], [-0.069115922372518776, -0.82309130285329057]], [[0.057673596670143351, 0.68682633746255728], [0.016557020496590198, 0.19717510964341375], [0.018609318374679041, 0.22161562170392915], [0.058290750169483853, 0.69417592725888211], [-0.040141783096009187, -0.47804256115209709]], [[0.034629092868628475, 0.41239274811732912], [0.047516123927448105, 0.56586249604233407], [-0.06094647403089036, -0.72580255015660078], [-0.041961230059333832, -0.49971008608859124], [-0.057212276279251559, -0.68133254111957609]], [[0.027153230524833843, 0.32336380854330699], [-0.047241591053492862, -0.56259312462771316], [0.0020054826446121244, 0.023882996365245504], [-0.006582246462985112, -0.078387000143313582], [-0.070204036169159531, -0.83604948921304878]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.56634170759099756, 0.84501833186687514], [-0.13265312491692072, -0.41121850298000506]], [[-0.23559563366484371, -0.10714654863049033], [-0.3290229474199895, -0.022927324448547282]], [[-0.32815393280986083, -0.91517125849315994], [-0.2205467236850982, -0.15844114516034935]], [[-0.74221507427104005, -0.5625403137170224], [-0.76223494838154915, 0.34994463731483338]], [[-0.83239993059800321, 0.22642915975839606], [0.81754451331270239, -0.52434757644970809]], [[0.23111026860334172, -0.81596178950749843], [-0.84042474033169845, -0.4474639138322476]]])+(1.-msk_arg0)*numpy.array([[[0.50566465771505298, -0.48174581395897231], [0.28983260843394332, 0.38795486493094788]], [[-0.52028826250910498, -0.70574604587399148], [0.19846007862913351, -0.11128359556555711]], [[0.86864751400151174, 0.3605964258813239], [0.82473549087445086, 0.011741257722523102]], [[0.96488267249014847, -0.41134421873214833], [-0.50431546682046213, -0.80601172902590945]], [[0.76716340040946074, 0.030747647008840584], [-0.19588562415873723, 0.61194433002776805]], [[0.53965096390623235, 0.9805552466619718], [0.23890285799743993, -0.51629601718021467]]])
      arg1=Symbol(shape=(2,))
      res=outer(arg0,arg1)
      s1=numpy.array([0.38087151448850998, -0.88335666622826348])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.21570342388819211, -0.50028172276360561], [0.32184341182869108, -0.74645257653969121]], [[-0.050523796588741093, 0.11718002219137247], [-0.15662141401569238, 0.36325260588379449]]], [[[-0.089731665800809213, 0.20811497353211159], [-0.040809068249111638, 0.094648617996094447]], [[-0.12531546828532478, 0.29064461394551916], [-0.0087323647858876443, 0.020253004890402486]]], [[[-0.12498448537465244, 0.28987696409661223], [-0.34856266323864549, 0.80842263193044217]], [[-0.083999964665422283, 0.19482141858203433], [-0.060345718914516108, 0.13996004178223456]]], [[[-0.28268857941381292, 0.65564063363242897], [-0.21425558124624383, 0.49692373614407037]], [[-0.29031357918615186, 0.6733253228849978], [0.13328394400123295, -0.30912592818288998]]], [[[-0.3170374222269921, 0.73530602766168995], [0.086240417001541086, -0.20001770770104363]], [[0.31137941694718074, -0.72218339577311697], [-0.19970905556078009, 0.46318592707748363]]], [[[0.088023318016801091, -0.20415279640456646], [-0.3107766025344757, 0.72078528614899184]], [[-0.32009384366374671, 0.74239479683516318], [-0.17042625854024426, 0.39527023118030519]]]])+(1.-msk_ref)*numpy.array([[[[0.19259326400724625, -0.44668224626862513], [-0.18348325776105376, 0.42555337618821903]], [[0.11038898452239128, -0.25602556675044985], [0.14776095695943545, -0.34270251613243835]]], [[[-0.19816297851243825, 0.45960010504773857], [-0.26879856533630458, 0.62342547428702821]], [[0.075587790712986852, -0.17531103343723042], [-0.042384751580780568, 0.098303105984684891]]], [[[0.33084309421443497, -0.76732557209584429], [0.13734090684456357, -0.31853525662035337]], [[0.3141182554617768, -0.72853559373898535], [0.0044719106107772872, -0.01037171827909486]]], [[[0.36749632477504379, -0.852335540872315], [-0.15666929556460624, 0.36336365773150014]], [[-0.19207939562788931, 0.44549042949787387], [-0.30698690792960065, 0.71199583389320587]]], [[[0.29219068617410648, -0.67767890383803964], [0.011710902883215217, -0.027161138956092856]], [[-0.0746072543398653, 0.17303687191890471], [0.2330721637603326, -0.54056510329061735]]], [[[0.20553767991815095, -0.47670427640307833], [0.37346556183579965, -0.86618001374395204]], [[0.090991293341118384, -0.21103643219302276], [-0.19664244598781414, 0.45607352852324468]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.754627721144)+(1.-msk_arg0)*(0.58366899685)
      arg1=Data(numpy.array([-0.23430637466555004, -0.19529124738101356]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.17681408556336167, -0.14737218897049123])+(1.-msk_ref)*numpy.array([-0.13675736665662333, -0.11398544645247927])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.42201616119905205, 0.27767742277886431])+(1.-msk_arg0)*numpy.array([0.36965951817707077, 0.016971039947976774])
      arg1=Data(numpy.array([-0.88941504482415179, -0.014645484245880747]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.37534752292937135, -0.0061806310403477865], [-0.24697047742751854, -0.0040667203207446247]])+(1.-msk_ref)*numpy.array([[-0.32878073692913373, -0.0054138426498021577], [-0.015094298256042233, -0.00024854909819430667]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.32565014483404586, 0.010119090846729684, 0.81211966143393899, -0.87604363255028828, -0.076988721786289904], [-0.36406220548242252, 0.40674278274405906, 0.60188613892112408, -0.30971097973586437, 0.62504329060164121], [0.80632167789462628, 0.035718851870698343, 0.62756915889350329, 0.15400741967310783, -0.80078011954154671], [-0.60228312653172322, -0.95589393004111911, -0.12966642233915548, -0.65230700307710321, 0.59350325591256503]])+(1.-msk_arg0)*numpy.array([[0.38013816651553789, 0.38544277320184328, -0.11994397447910532, 0.90332228918106083, 0.1620135444547588], [-0.29343253074237796, 0.61947664145034387, -0.085366125919957181, -0.92522686233194484, -0.20398584627625538], [0.92366281232068936, 0.60773805084890609, 0.27891181403544163, -0.87242066265766005, 0.9594846291366943], [-0.3720465977870473, 0.24525390349168763, 0.97924605524931319, -0.58203005863290769, -0.62666251303222253]])
      arg1=Data(numpy.array([0.2674613723521404, -0.54031949922685807]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.087098834643987197, 0.17595512317988546], [0.0027064659248223037, -0.0054675420989360657], [0.21721063916127695, -0.43880408877827143], [-0.23430783220225432, 0.47334345684044943], [-0.020591509184598226, 0.041598507601684057]], [[-0.097372577099875665, 0.19670990855368803], [0.10878798286705453, -0.21977105668640873], [0.16098129271557488, -0.32521081717344891], [-0.082835723672680231, 0.16734288147594181], [0.16717493628381266, -0.33772307777298632]], [[0.21565990252697725, -0.43567132521578444], [0.0095534131401797968, -0.019299592155734051], [0.16785050848353486, -0.3390878536635582], [0.041191035818181447, -0.083213211874994183], [-0.21417774972489312, 0.43267711318151209]], [[-0.16108747156671252, 0.32542531732040708], [-0.2556647023518786, 0.51648812959381074], [-0.034680759266822764, 0.070061296384830773], [-0.17446692623791391, 0.35245419324479299], [0.15873919532183822, -0.32068138202418695]]])+(1.-msk_ref)*numpy.array([[[0.10167227569967223, -0.20539606376869141], [0.1030910530837798, -0.2082622461970314], [-0.032080380019551616, 0.064808068225829227], [0.24160381914064355, -0.48808264693076986], [0.043332364939504292, -0.087539077207763577]], [[-0.078481867365118044, 0.15854731806759129], [0.16568607266240379, -0.33471530869118571], [-0.02283214119093737, 0.046124982408008171], [-0.24746244633636685, 0.49991811492643357], [-0.054558334385460011, 0.11021753030935316]], [[0.24704412337392909, -0.49907302820758626], [0.16254645311066332, -0.32837271929578776], [0.0745981365471442, -0.15070149168808439], [-0.2333388277027815, 0.47138589556235055], [0.25662507565968473, -0.51842825433100626]], [[-0.099508093623068478, 0.20102403140535369], [0.065595945602606176, -0.13251546631806083], [0.26191049380740122, -0.52910573819218509], [-0.15567055823215423, 0.31448218981551151], [-0.1676080157372393, 0.33859797522581492]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.8308383256533951, -0.0020690666107840983], [0.21524076447995588, -0.10475888330800176]], [[-0.087242765977021852, 0.70888825028190872], [0.78270518430092428, 0.29269417305487044]], [[-0.44144040991368905, -0.12863921174128756], [-0.24101563144741855, -0.92331317605137442]], [[0.047840171049458258, 0.80686689373407749], [-0.61618586167476153, -0.10436202485840806]], [[0.13957673157835604, -0.20644022281349739], [0.52298215287361449, 0.80141225819897266]], [[0.46802794923437463, -0.53954185089423423], [-0.059424631459731536, 0.46892840066385233]]])+(1.-msk_arg0)*numpy.array([[[0.24025366440042739, -0.94615132172696148], [-0.60120188385701789, -0.61359598051779263]], [[0.80388555717190679, -0.52479497642943729], [-0.59558320626551553, -0.37112136946130847]], [[0.18761274066806388, 0.78545898789654789], [-0.92703360405653634, -0.18394419220048652]], [[-0.45611388588357693, -0.73857176458521323], [-0.76968431314751107, 0.59284454646853635]], [[-0.85058232257517341, 0.84176506815894236], [-0.54396196879800685, -0.31031616416812891]], [[-0.92721672793037735, -0.55179586860640684], [0.092088579256883074, 0.22532501099469493]]])
      arg1=Data(numpy.array([0.59121762951075518, -0.29269610838823557]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.49120626539948509, 0.24318314461854629], [-0.0012232686569276269, 0.00060560774497254164]], [[0.12725413454992227, -0.063000134129791857], [-0.061935298659550622, 0.030662517463349404]]], [[[-0.051579461292896422, 0.025535618086499859], [0.41910723091969698, -0.20748883213966021]], [[0.46274910366817118, -0.22909476146017724], [0.17304595516511126, -0.085670445401073339]]], [[[-0.2609873527194273, 0.12920789006704428], [-0.076053769827816142, 0.037652196662805087]], [[-0.14249269029938061, 0.070544337385392661], [-0.54587902724114012, 0.27025017345381913]]], [[[0.028283952523249768, -0.014002631890803963], [0.47703393224416768, -0.23616679978326849]], [[-0.36429994447739461, 0.18035520375605432], [-0.061700668947730519, 0.030546358539572341]]], [[[0.082520224378614629, -0.040853566154534164], [-0.12205109916746805, 0.060424249832310935]], [[0.30919626869836975, -0.15307484090260826], [0.47380905555325792, -0.23457024918946712]]], [[[0.27670637469112702, -0.13698995935782812], [-0.31898665410753446, 0.15792180006932802]], [[-0.035132889746172728, 0.017393358370668535], [0.27723873745075239, -0.13725351798702887]]]])+(1.-msk_ref)*numpy.array([[[[0.14204220194809319, -0.070321312596018273], [-0.55938134158988206, 0.27693480981586704]], [[-0.35544115263134646, 0.17596945176062512], [-0.36276876107905687, 0.17959715562022152]]], [[[0.4752713135091074, -0.23529417417372556], [-0.31026804194376456, 0.1536054473025921]], [[-0.35211929138471326, 0.17432488669530419], [-0.21941349631369997, 0.10862578058103756]]], [[[0.11091995980378878, -0.054913519077593557], [0.46437720090211398, -0.22990078905588179]], [[-0.54807860986711743, 0.27133912825246864], [-0.10875104927504238, 0.053839749217700038]]], [[[-0.26966257039902747, 0.13350275937995873], [-0.43665664788164527, 0.21617708125952398]], [[-0.45505093509068528, 0.22528360314574855], [0.35050014743150681, -0.17352329163052907]]], [[[-0.50287926445664655, 0.2489621356815801], [0.49766634820188915, -0.24638135962728031]], [[-0.321599905736761, 0.15921555137837942], [-0.18346438697835152, 0.090828333621976162]]], [[[-0.54818687592971649, 0.27139272789769486], [-0.326231445411308, 0.16150850336580144]], [[0.054444391533267709, -0.02695396877549127], [0.13321611886976839, -0.065951753840683594]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.196322146119)+(1.-msk_arg0)*(-0.426019435555)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.4008017758092961, -0.69247468010822177])+(1.-msk_arg1)*numpy.array([0.043612026398168968, 0.10558868904036522])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.078686264795153185, 0.13594811533185513])+(1.-msk_ref)*numpy.array([-0.018579570869536998, -0.044982833705918678])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.084694054093183979, -0.71334990178012059])+(1.-msk_arg0)*numpy.array([0.338433690248783, -0.43486916422203836])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.29984942829353645, -0.5647027318292841])+(1.-msk_arg1)*numpy.array([-0.89076776806819269, 0.45228449535701198])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.025395463699703066, 0.047826963716118151], [0.21389756022201953, 0.40283063828538557]])+(1.-msk_ref)*numpy.array([[-0.30146582290199048, 0.15306831080598213], [0.38736743481574548, -0.19668458048649018]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.91685841531765533, -0.22384099195951723, 0.43367807439332395, 0.49498927216479727, -0.4553686876237637], [-0.48723087581788382, -0.028233650781919417, 0.46252772974398493, -0.49004398489623546, -0.60474523114706269], [-0.22700145502566982, -0.022434491114989852, -0.43737706246764052, -0.093536595481270579, 0.41370261256966478], [-0.66811934938107242, -0.47714142742132659, 0.98632342655140781, 0.19496584964968111, -0.19394544936355596]])+(1.-msk_arg0)*numpy.array([[-0.64866638345178118, -0.18375521953732088, -0.62929258930351617, -0.39782127311369653, -0.44126747720393511], [-0.21672598946323585, -0.90341157394747817, -0.71731330077304722, 0.22435617216144998, 0.93087894537294358], [0.80243746270150695, -0.82807947233128676, -0.71606775170573944, -0.79843563358641201, 0.87736535077078859], [-0.56915521568888772, 0.59242256876910404, -0.067116567370258728, 0.1996295935260104, -0.42950669257857754]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.90604925456086072, 0.69995302293583572])+(1.-msk_arg1)*numpy.array([0.76093140511844881, -0.11727039054567712])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.83071888373641367, -0.64175781940575283], [0.20281096390508421, -0.15667817897902017], [-0.39293369602346068, 0.30355427915259936], [-0.44848466106053758, 0.34646923737255897], [0.41258645997186855, -0.31873668945257766]], [[0.44145517183382893, -0.34103872439640254], [0.025581078244489752, -0.019762229213319219], [-0.41907290474826481, 0.32374768262595149], [0.4440039872172678, -0.34300776859964305], [0.54792896588003159, -0.42329325264741724]], [[0.20567449911023888, -0.15889035465605075], [0.020326753951188808, -0.015703089873964293], [0.3962851614108247, -0.30614339703702081], [0.084748762609965986, -0.065471222762241779], [-0.37483494372862536, 0.28957239426458975]], [[0.60534903846440791, -0.46765215828120543], [0.43231363463519801, -0.33397658449147721], [-0.89365760538281691, 0.69038006400708962], [-0.17664866273991842, 0.13646693583154795], [0.17572412982132102, -0.13575270356667005]]])+(1.-msk_ref)*numpy.array([[[-0.49359062261306635, 0.076069360121242324], [-0.13982511740038261, 0.021549046359948257], [-0.47884849420935149, 0.07379738771512373], [-0.30271470033641529, 0.046652656065421669], [-0.33577428146186344, 0.051747609386811146]], [[-0.16491361168794619, 0.025415541425751972], [-0.68743423836412398, 0.10594342810030563], [-0.54582621786738728, 0.084119610925264002], [0.17071965732980873, -0.026310335930706412], [0.70833502389781366, -0.10916453747463313]], [[0.61059986601314054, -0.094102154639487928], [-0.63011167643078969, 0.097109203123148233], [-0.54487844046545686, 0.083973544899697022], [-0.60755474856154745, 0.093632858576263689], [0.66761484916425684, -0.10288897733613538]], [[-0.43308807800463911, 0.066745054424944952], [0.45079293767735523, -0.069473626007626091], [-0.051071103915778009, 0.0078707860675954911], [0.15190442710497187, -0.023410640397270015], [-0.32682513109159467, 0.05036841758067187]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank3_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.13883456031409991, -0.3292702283004636], [0.30614442524975405, 0.14505263944761793]], [[-0.77383501589957482, 0.64758425412559206], [-0.27726204439273783, -0.9519754075218414]], [[0.607081071850609, -0.82120053072857457], [-0.29135529803867821, 0.68144743891650239]], [[0.23511839162296355, -0.47188904024163913], [0.6432406982842358, -0.5274484392482397]], [[0.018132367332716726, -0.49504783001903019], [0.63952767645173836, 0.76314424887778909]], [[0.98218793657759296, 0.022328732960420039], [-0.79791121219488392, -0.76829567228829099]]])+(1.-msk_arg0)*numpy.array([[[0.6479716279159895, -0.01948819418153902], [-0.56839741520435338, -0.78827860664172444]], [[-0.68170115669068876, 0.67613005216917665], [-0.40665000876541457, -0.26812785551717999]], [[-0.60645082773233461, -0.91576122624446632], [0.091252226719450436, -0.62939783050003162]], [[-0.91514322861177599, -0.45280213878396358], [0.23315706784438728, -0.29481090681351674]], [[0.96787113640604794, 0.88227228556255488], [0.70113912024859593, 0.86814108302189719]], [[0.90063169303264656, -0.90300469456878085], [0.82130049361510138, 0.3235079425689702]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.12214824086500831, -0.69411170222450136])+(1.-msk_arg1)*numpy.array([-0.65090991803994647, 0.19335409710404261])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.016958397313634199, 0.096366692987210095], [-0.040219779156121301, 0.22855031865748496]], [[0.037395002994886489, -0.2124984281366484], [0.01771792474135284, -0.10068273447914293]]], [[[-0.094522585911878801, 0.53712794012697795], [0.079101277453319574, -0.44949580896489877]], [[-0.033867070981208766, 0.19245082959568852], [-0.11628212137554232, 0.66077727059084868]]], [[[0.074153884988995597, -0.42138207617050105], [-0.10030820022590659, 0.57000489825167489]], [[-0.035588537122124751, 0.20223312187375386], [0.083237605905615963, -0.47300064180286039]]], [[[0.028719297931755101, -0.16319842703370216], [-0.057640416148993331, 0.32754370498321034]], [[0.078570719748198972, -0.44648089602614782], [-0.064426899001166688, 0.36610813400225217]]], [[[0.0022148367724094909, -0.012585888354671948], [-0.060469221580864187, 0.34361849197705463]], [[0.078117180663066041, -0.44390364412159627], [0.093216727526670026, -0.52970735363140065]]], [[[0.11997252865178533, -0.68174814056224364], [0.0027274154518598368, -0.015498634843673482]], [[-0.097463450936071436, 0.55383950972060614], [-0.09384596483421366, 0.53328301690374336]]]])+(1.-msk_ref)*numpy.array([[[[-0.42177115921900743, 0.1252879690647328], [0.012685058877452126, -0.0037681221901597339]], [[0.36997551494478309, -0.10990196901310938], [0.51309836324180802, -0.15241689825364341]]], [[[0.4437260440292729, -0.1318097116467096], [-0.44009975684178348, 0.13073251576208039]], [[0.26469252387643949, -0.07862744528218775], [0.17452708045891424, -0.05184361941196753]]], [[[0.39474485857451164, -0.11725975223418485], [0.59607806471894642, -0.17706618506338967]], [[-0.059396979414920094, 0.017643991906072732], [0.40968129026529571, -0.12169664923557687]]], [[[0.59567580393050312, -0.1769466926891084], [0.29473340304418222, -0.087551148711352667]], [[-0.15176424792102436, 0.045081874336477507], [0.19189534319126847, -0.057002896703351576]]], [[[-0.62999692207129054, 0.18714184969285508], [-0.57427978108443889, 0.17059096117486786]], [[-0.45637840729561374, 0.13556812153999004], [-0.56508164119689341, 0.16785863526662462]]], [[[-0.58623010149605825, 0.17414082782961263], [0.58777471173145202, -0.17459965739905839]], [[-0.53459263698517323, 0.15880181539405244], [-0.2105745283828401, 0.06255158614140971]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.135679058676)+(1.-msk_arg0)*(-0.629820830028)
      arg1=numpy.array([[-0.15137218142962316, -0.56892857471855862, 0.57687741496105271, -0.39999495525550022, -0.21860517149339675], [0.39059915618285679, 0.20078616449879449, -0.67471414803258778, -0.51436995307718436, -0.52869142419732817], [0.33338414638611669, 0.25052717773521804, -0.59775810180367017, -0.14479228654166709, 0.71189655719882228], [-0.94444987904345812, 0.92044245306127359, -0.54335603171718994, 0.094025208931912241, 0.3880528931344569]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.020538035086050812, 0.077191693471492615, -0.078270184633157325, 0.054270939004074564, 0.029660143889852865], [-0.052996125830392773, -0.02724247779429043, 0.091544580480203933, 0.069789231044550065, 0.07173235476498166], [-0.045233247159053046, -0.03399129164778137, 0.081103256568464663, 0.019645281141468216, -0.096589454755171444], [0.12814207055494686, -0.12488476559645874, 0.073722034909124351, -0.012757251839663067, -0.052650651256845288]])+(1.-msk_ref)*numpy.array([[0.095337352951132076, 0.3583230671558062, -0.36332941231509264, 0.25192515472597321, 0.13768209055835237], [-0.24600748475526601, -0.12645930878273987, 0.4249490247454204, 0.32396091078846023, 0.33298087161656914], [-0.20997227979503139, -0.15778723502573058, 0.37648050383386161, 0.091193198091303551, -0.44836728054893343], [0.59483420673885656, -0.57971382977992481, 0.34221694689676119, -0.059219035133039298, -0.24440379524865352]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.86660731561558046, 0.10293397253942249])+(1.-msk_arg0)*numpy.array([0.68626330708301464, -0.64806613946281844])
      arg1=numpy.array([[-0.61871824119745367, 0.1369785624298181, -0.69937967814104596, -0.1603390367364621, -0.47726573003741879], [0.12320225807433371, 0.79412062731893673, -0.20323882416546035, 0.60524988458207907, -0.062024186681118199], [0.29346975098615879, 0.55556808629568799, -0.79331957665352171, -0.35436718901142772, -0.64062927975535078], [0.45054591806262989, 0.89434005681699191, -0.53782149229694065, 0.67608265546193191, 0.5902052140526306]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.53618575412651859, 0.11870662428418587, -0.60608754546990051, -0.13895098221457336, -0.41360197314303782], [0.10676797814757631, 0.68819074511582456, -0.17612825183889655, 0.52451397775431541, -0.053750613922963479], [0.25432303311648791, 0.48145936790639132, -0.68749654874899713, -0.3070971984114324, -0.55517402043352726], [0.39044638861381298, 0.77504163588565911, -0.46608003971981732, 0.58589817518411813, 0.51147615621246922]], [[-0.063687126449058482, 0.014099747583640465, -0.071989928584400564, -0.016504334004428443, -0.049126857549679097], [0.012681697849418309, 0.081741990845436402, -0.020920179545592013, 0.062300774999060358, -0.0063843959286142352], [0.030208007289160427, 0.057186830138539854, -0.081659535518239873, -0.036476422502574643, -0.065942516690287292], [0.046376481157607639, 0.092057974849305793, -0.055360102719204517, 0.069591873491698333, 0.060752167295917453]]])+(1.-msk_ref)*numpy.array([[[-0.42460362635675086, 0.094003361252564144, -0.47995861082772856, -0.11003479760526946, -0.32752995825286829], [0.084549189066187302, 0.5449758479267317, -0.13947534759945213, 0.4153607874049105, -0.042564923470918448], [0.20139752184059012, 0.3812659922110605, -0.5444261162479429, -0.24318919905269412, -0.43964036813911678], [0.30919313172241331, 0.61375276504804011, -0.36908715592402058, 0.46397071899877179, 0.40503618205339681]], [[0.40097034198805875, -0.088771168143058882, 0.45324428803161615, 0.10391030054298601, 0.3092997591632537], [-0.079843211763335295, -0.51464268921437495, 0.13171220016587246, -0.39224195611142443, 0.040195775215753432], [-0.19018780857071457, -0.36004486489439247, 0.51412355540212529, 0.22965337613492687, 0.4151701441578961], [-0.29198355376957985, -0.57959150798834569, 0.3485438982330103, -0.43814627648298499, -0.38249201456191473]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.82198701632641225, -0.0055809644583895146, -0.81997433832370969, -0.013440107755883002, -0.0052025136098590163], [0.012338171120628516, -0.94554806940956548, -0.25511026330949549, -0.51527037834293887, 0.62279006579732687], [0.5635167688932019, 0.86230385418831657, 0.73570223287298075, 0.47639158369102952, 0.19187623056909731], [-0.27036352797170204, -0.40710182516029136, -0.80722681219478321, -0.90792685183815003, -0.31384743450951436]])+(1.-msk_arg0)*numpy.array([[0.89229130193725426, 0.18307842930067797, 0.29290544525661, 0.027650258983314302, 0.14508277074613707], [-0.1224177638677284, 0.96014567322005839, -0.95987661764430232, -0.32853655808001747, 0.38933705747241021], [-0.66540729123951414, -0.70473240520780589, -0.36230465711735671, -0.96645064654281287, -0.50720451022365531], [0.30009717637530686, -0.93720424349855591, -0.90344207829791445, 0.095855935879436416, -0.44243063558063711]])
      arg1=numpy.array([[0.87835048566625273, -0.87601670858751768, -0.81895933805461363, -0.61129077480889138, -0.0016464416367591994], [-0.079158624147162682, -0.29676575864334032, -0.8121801333168146, -0.54247405024734552, -0.51631405473326453], [-0.24550181580863573, -0.17128786684120079, 0.72400612674452747, -0.11041705408522606, 0.67698164797895566], [-0.036509002781882849, -0.60084796882118874, 0.48115497910310912, -0.34643162579331177, -0.50453128354835663]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.72199269500165819, 0.72007436054393781, 0.67317394278016551, 0.50247308009302138, 0.001353353648555269], [0.065067361279230143, 0.24393760049508348, 0.66760152450467614, 0.44590662599731978, 0.424403449337588], [0.20179930507925692, 0.14079640259771445, -0.59512363592477635, 0.090761384839067066, -0.55647012492995929], [0.030009926265732567, 0.49388922915711408, -0.39550314566356187, 0.2847622984469525, 0.41471816440724873]], [[-0.0049020428425125256, 0.0048890181155823009, 0.0045705829585490024, 0.0034115920879498112, 9.1887322575657513e-06], [0.00044178146794032893, 0.0016562391514554832, 0.0045327484578512002, 0.0030275283940290432, 0.0028815303888333277], [0.001370136908498085, 0.00095595149699409748, -0.0040406524610174617, 0.00061623365444971945, -0.0037782105163525134], [0.00020375544693693209, 0.0033533111588865856, -0.0026853088373516017, 0.0019334225908145693, 0.0028157711616290208]], [[-0.72022485830049465, 0.71831122098456379, 0.67152564133535508, 0.50124274859730855, 0.00135003989169023], [0.064908040457684946, 0.24334030658070671, 0.66596686741611733, 0.44481480040935001, 0.42336427539714022], [0.20130518897495533, 0.14045165527599332, -0.59366644471965579, 0.090539150863186507, -0.55510757885883877], [0.029936445398932865, 0.49267991566729918, -0.39453473562123026, 0.28406504313427783, 0.41370270539117571]], [[-0.011805125174786605, 0.011773758959370197, 0.011006901751140621, 0.0082158138836087111, 2.2128353011816019e-05], [0.0010639004383453086, 0.0039885637744228614, 0.010915788508965411, 0.0072909096900946133, 0.0069393165314919495], [0.0032995708587330052, 0.0023021273876210773, -0.0097307203593659346, 0.0014840171049925997, -0.0090987062975924175], [0.00049068493144833775, 0.0080754614458602072, -0.0064667747664254203, 0.0046560783807078473, 0.0067809548171038738]], [[-0.0045696303559049564, 0.0045574888488904607, 0.0042606471021502581, 0.0031802485755245205, 8.5656350230782907e-06], [0.00041182381946332843, 0.0015439278982821139, 0.0042253781972378382, 0.0028222286294071587, 0.0026861308967113017], [0.0012772265379895288, 0.000891127458445066, -0.0037666517280097161, 0.00057444622663892773, -0.0035220062372353024], [0.0001899385838551262, 0.0031259197352483804, -0.0025032153272353556, 0.0018023152480752904, 0.0026248308692599638]]], [[[0.01083723859603739, -0.010808444055082557, -0.010104460453754479, -0.007542210184053693, -2.0314078654462699e-05], [-0.00097667265040120971, -0.0036615467128846739, -0.01002081746563774, -0.0066931376604521808, -0.0063703711592845757], [-0.0030290434138719705, -0.0021133790119741663, 0.0089329114841574381, -0.0013623445079392132, 0.0083527154182894501], [-0.00045045432376637312, -0.0074133650567978935, 0.0059365724677165983, -0.0042743326806354244, -0.006224993312129971]], [[-0.83052260598667949, 0.82831590757544926, 0.77436542102247563, 0.57800481196842468, 0.0015567897110331862], [0.074848284239467086, 0.2806062901520755, 0.76795535707051754, 0.51293529091616519, 0.48819975766206297], [0.23213376797439825, 0.16196091180498012, -0.68458259538398514, 0.10440463232017708, -0.64011869027220758], [0.034521017096477785, 0.56813063692753385, -0.45495516157774463, 0.32756775495128299, 0.47705858111587868]], [[-0.22407622367634097, 0.22348085319127922, 0.20892493237088261, 0.15594655052016179, 0.00042002415947735609], [0.020194177449400059, 0.075707990828744728, 0.20719548766519372, 0.13839069779716881, 0.13171701445339637], [0.062630032873900315, 0.043697292811580531, -0.18470139363148436, 0.028168523741540826, -0.17270496647160757], [0.0093138213128532363, 0.153282483534949, -0.12274757341166896, 0.088378263274868377, 0.128711108593899]], [[-0.45258798706695413, 0.45138546086862624, 0.42198548796688357, 0.31498002881332571, 0.00084836260509248021], [0.040788094213415012, 0.15291460473538324, 0.41849236457677358, 0.27952080911217614, 0.26604133832618604], [0.12649981351559422, 0.088259563952820461, -0.37305891085025844, 0.05689463723400718, -0.34882858988534277], [0.018812007676344184, 0.30959916022108025, -0.24792490812404788, 0.17850595489247917, 0.25997002535981029]], [[0.54702795676119953, -0.54557450358077786, -0.51003974003236807, -0.38070582186452839, -0.0010253874952887204], [-0.049299204741037314, -0.18482276635187955, -0.50581771866766068, -0.33784744944688672, -0.32155526411941443], [-0.15289609202082347, -0.10667638186031521, 0.45090382331289203, -0.068766644378884939, 0.42161744508839655], [-0.022737444244723608, -0.37420214603633833, 0.29965854109433676, -0.21575417502209157, -0.31421707127789084]]], [[[0.49496522763842138, -0.49365010511969559, -0.4614973200354513, -0.34447260227452836, -0.00092779747131777881], [-0.044607212109440506, -0.16723248142883493, -0.45767712448594133, -0.30569322400379262, -0.290951627857437], [-0.13834439000189641, -0.096523585272962481, 0.40798959320195816, -0.062221861548812513, 0.38149051086909613], [-0.020573435283159542, -0.33858790598615962, 0.27113889916106015, -0.19522003040946587, -0.28431183871070981]], [[0.75740500911818942, -0.75539258414837984, -0.70619179362800577, -0.52711839114746928, -0.001419732969073578], [-0.068258786694342724, -0.25590225746927209, -0.70034605925426996, -0.46777746432543255, -0.44521959936809141], [-0.21169716198201677, -0.14770218775286259, 0.62431327354776089, -0.095213051305810245, 0.58376388426701165], [-0.03148185381138955, -0.51811351929573246, 0.41490179294250989, -0.29872932613429737, -0.43505927036232633]], [[0.64620441354972924, -0.64448744854187601, -0.60251021363895751, -0.44972798796155583, -0.0012112907884587881], [-0.058237176536220639, -0.21833123127414955, -0.59752273757625574, -0.39909937004262164, -0.37985340293096509], [-0.18061623406478455, -0.12601686609912122, 0.53265292405966713, -0.081234073237757498, 0.49805691003214791], [-0.026859754866597078, -0.44204519227894368, 0.35398679248410975, -0.25487052063395638, -0.37118479186079695]], [[0.41843877890233105, -0.41732698714381067, -0.39014533603439461, -0.29121378030692424, -0.00078435093879056575], [-0.037710502320269804, -0.14137670974537073, -0.38691577995318882, -0.25843007190862005, -0.2459676702163168], [-0.11695499883209939, -0.08160009815153782, 0.34491042532183369, -0.052601755262158906, 0.32250835941045775], [-0.017392581654241374, -0.28623891542426444, 0.22921818249575437, -0.16503711085233391, -0.24035445719126949]], [[0.1685345803081765, -0.16808678395932028, -0.15713883077528235, -0.11729216965199298, -0.00031591301511337011], [-0.015188658418393501, -0.056942295130462652, -0.15583806252393731, -0.10408787594301175, -0.099068394612065389], [-0.047105963015229846, -0.032866070231711078, 0.13891956650867204, -0.021186408128417322, 0.12989668677865757], [-0.0070052098356243684, -0.1152884434025082, 0.092322203709857359, -0.066471994507144722, -0.096807560891447084]]], [[[-0.237473936100386, 0.23684296789587969, 0.22141673590181513, 0.16527073049388713, 0.00044513776951372072], [0.021401604893812865, 0.080234637488012114, 0.21958388619206129, 0.14666519805797071, 0.13959248937905985], [0.066374737045481733, 0.046309991977934166, -0.19574485069977771, 0.029852744290723956, -0.18303114671968732], [0.0098707027948385321, 0.16244737662512782, -0.13008675765146716, 0.093662476550452262, 0.13640685779222486]], [[-0.35757808584515982, 0.35662800093688951, 0.33339984125409727, 0.24885759012834835, 0.00067026939534456755], [0.03222562036748744, 0.12081388198878236, 0.33064001463220399, 0.22084217595778996, 0.21019239403782256], [0.099944237295861271, 0.06973160321886579, -0.29474421562493036, 0.044950984246918138, -0.27560046449225473], [0.014862881667286663, 0.24460630475095976, -0.19587907017783757, 0.14103294715370429, 0.20539560638300047]], [[-0.70902806253410877, 0.7071441751024683, 0.66108593577497554, 0.49345030347306046, 0.0013290518339058898], [0.063898963828039126, 0.23955727731823004, 0.65561357994526626, 0.43789959827955738, 0.41678254849369595], [0.19817564816323585, 0.13826815871786702, -0.58443715770147708, 0.089131606581156, -0.54647773761242324], [0.029471045932029764, 0.48502059048523866, -0.38840119995305028, 0.2796488969325911, 0.40727117967128218]], [[-0.79747799126147101, 0.79535909238548297, 0.74355517358338064, 0.55500730872994031, 0.0014948485719980308], [0.071870240417772777, 0.26944160097840825, 0.73740015156782446, 0.49252675664496282, 0.46877539427376314], [0.22289769074768401, 0.15551685369920368, -0.65734460336669143, 0.10025060830484205, -0.61464981640173588], [0.033147503959505158, 0.54552600476516877, -0.43685352542333672, 0.31453457538369362, 0.45807749992592045]], [[-0.27566804652653937, 0.27493559657766126, 0.25702828721605059, 0.19185204141310383, 0.00051673148376652042], [0.024843731107889903, 0.093139172000482093, 0.25490065120107763, 0.17025408895811478, 0.16204384147924006], [0.077050115058967653, 0.053758257570718178, -0.22722746544794023, 0.034654109150746494, -0.21246895342821839], [0.011458256859594654, 0.18857459354478276, -0.15100925579298979, 0.108726676988191, 0.15834584897144408]]]])+(1.-msk_ref)*numpy.array([[[[0.78374449841236027, -0.78166208942434445, -0.73075029398642311, -0.54544944131645856, -0.0014691055516275699], [-0.070632551799833562, -0.26480150515026307, -0.72470126856483319, -0.48404487656237943, -0.46070254010644734], [-0.21905913485584755, -0.1528386737097901, 0.64602436944342312, -0.098524176945782582, 0.60406483606277028], [-0.032576665624677086, -0.5361314163658133, 0.42933040273750561, -0.30911792641135383, -0.45018887586543715]], [[0.1608070272912652, -0.16037976304935248, -0.14993378927216161, -0.11191415489800628, -0.00030142794879311162], [-0.014492236574465262, -0.054331408962646842, -0.14869266311685764, -0.09931529705566107, -0.094525966166430345], [-0.044946086828709382, -0.031359113619550717, 0.13254990448845566, -0.020214980829931194, 0.12394073677737169], [-0.0066840108846411943, -0.11000230238028597, 0.08808909782439775, -0.063424157910319759, -0.092368794925088121]], [[0.25727364009543341, -0.25659006408105683, -0.2398776495599452, -0.17905039657665645, -0.00048225172070397507], [-0.023185992051725324, -0.086924306672343257, -0.23789198357773444, -0.15889360322785537, -0.15123119809389254], [-0.071908818670734703, -0.050171148904176843, 0.21206533692261942, -0.032341756390756329, 0.19829161103182963], [-0.010693685715702209, -0.17599164183910002, 0.14093291339163105, -0.10147170960396128, -0.14777996025362036]], [[0.024286618406791784, -0.024222088866155438, -0.022644437794013716, -0.016902348237576709, -4.5524537657303756e-05], [-0.0021887564584318854, -0.0082056500838681053, -0.022456991027312662, -0.01499954798106656, -0.01427621733009988], [-0.0067881887879827031, -0.0047361538788586559, 0.020018956909992464, -0.0030530601416311233, 0.018718717893569038], [-0.0010094833821416031, -0.016613601947504227, 0.013304059783312148, -0.0095789241731956982, -0.013950420655296043]], [[0.12743352214667511, -0.12709493130178839, -0.11881688989338568, -0.088687759340826886, -0.0002388703145328296], [-0.011484552519722434, -0.043055598526555189, -0.11783334408657045, -0.078703638267764067, -0.074908273635874684], [-0.035618083660724664, -0.024850918316516788, 0.10504081490527493, -0.01601961214431067, 0.098218373233072892], [-0.0052968272807739896, -0.087172688113766633, 0.069807297526578757, -0.050261260144182598, -0.073198796545300496]]], [[[-0.10752570234739589, 0.10724000657605139, 0.10025517086324084, 0.074832849725075604, 0.00020155370351078393], [0.0096904217589416242, 0.036329400565627716, 0.099425275778437988, 0.06640846018754977, 0.063206012033926173], [0.030053783316760119, 0.020968677636373022, -0.088631211062600204, 0.0135170088539754, -0.082874579525073433], [0.0044693504815987743, 0.073554464767556518, -0.058901916615626208, 0.042409384962678889, 0.061763591533304643]], [[0.84334441838318952, -0.84110365241878182, -0.78632026497630036, -0.58692819251209416, -0.0015808238137436964], [-0.076003810472951086, -0.28493835912127136, -0.77981124087942977, -0.52085411217914934, -0.49573670567484834], [-0.23571750621632934, -0.16446130422267244, 0.6951513499785712, -0.10601645672963497, 0.65000100015637896], [-0.035053961054603892, -0.57690157752672488, 0.46197887133413784, -0.33262482657203868, -0.48442352890311707]], [[-0.84310809328755287, 0.84086795523888092, 0.78609991940007928, 0.58676372132072352, 0.0015803808294411693], [0.075982512403755115, 0.28485851263921486, 0.77959271928604257, 0.52070815651122737, 0.49559778849958114], [0.23565145258392778, 0.16441521826703945, -0.6949565520932891, 0.10598674840557479, -0.64981885446930565], [0.035044138103840133, 0.57673991603053187, -0.46184941390420736, 0.33253161721150076, 0.48428778194813499]], [[-0.2885702453487024, 0.28780351425992873, 0.26905808213195226, 0.20083136714178024, 0.00054091626842049775], [0.026006501919658587, 0.097498400900688217, 0.26683086554087598, 0.17822255731598935, 0.16962804243040447], [0.080656321568163603, 0.056274326212876459, -0.23786248090949194, 0.036276038902495304, -0.2224132205103441], [0.011994542112893573, 0.197400523605883, -0.1580770007375982, 0.11381545394819925, 0.16575697134067044]], [[0.34197439351876124, -0.34106576761813001, -0.31885121886773615, -0.23799815152412351, -0.00064102074215588553], [-0.030819385799020795, -0.1155419072287656, -0.31621182324311842, -0.21120525047844191, -0.20102019480149816], [-0.095582954571067871, -0.066688714056679144, 0.28188241497871119, -0.042989450932313884, 0.26357404278694962], [-0.014214307714350307, -0.23393238016911611, 0.18733146375220353, -0.13487866980175114, -0.19643272533949541]]], [[[-0.5844608174260929, 0.58290790514177493, 0.54494151477022601, 0.40675733862528823, 0.001095554269699891], [0.052672725672010312, 0.19747009959150452, 0.54043058250888909, 0.36096618834281424, 0.34355913658895182], [0.16335869825160643, 0.11397619549699801, -0.4817589556378884, 0.0734723128654972, -0.45046852460053921], [0.024293356646948552, 0.39980861938007123, -0.32016403131140486, 0.2305181297188286, 0.33571879473150723]], [[-0.61900205037902267, 0.61735736204510694, 0.57714718407462051, 0.43079641801241325, 0.0011603007747075873], [0.055785647588170659, 0.20914044687204042, 0.5723696588143552, 0.38229904219363198, 0.36386324563476824], [0.17301308513770361, 0.12071211038191382, -0.51023057908585845, 0.077814476101441751, -0.47709090506175361], [0.025729077342214777, 0.42343703423158108, -0.33908550570104568, 0.24414159288537116, 0.35555954495761488]], [[-0.31823047153817546, 0.31738493323387601, 0.2967127821669342, 0.22147349456613871, 0.00059651347266978126], [0.028679538179519487, 0.10751961642944766, 0.29425664471887758, 0.19654087476992826, 0.18706298656500756], [0.088946451198236229, 0.062058391864264706, -0.26231079150104153, 0.040004612920256456, -0.24527360384575861], [0.013227381734586688, 0.21769001732342103, -0.17432468972426088, 0.12551379139765426, 0.18279403369096722]], [[-0.8488823947633436, 0.84662691439671334, 0.79148378175515544, 0.59078236453971011, 0.0015912045843409354], [0.076502903486464893, 0.28680945931262458, 0.78493201495326348, 0.52427439659424546, 0.4989920520161048], [0.23726538861569055, 0.16554126965361773, -0.69971618929320634, 0.10671263331001947, -0.65426935138688069], [0.035284149343184031, 0.5806899079411737, -0.46501254064149344, 0.33480906873082394, 0.4876045851863845]], [[-0.44550332788706148, 0.4443196256268705, 0.41537986995107928, 0.31004943804118251, 0.0008350826239842832], [0.040149611190540065, 0.15052093126384694, 0.41194142673233802, 0.27514528496474749, 0.261876817252575], [0.12451962824623712, 0.086877978608445941, -0.36721917291438377, 0.056004027837635945, -0.34336814519356923], [0.018517530874738959, 0.30475279974482916, -0.24404397551766557, 0.17571168308648133, 0.25590054256465639]]], [[[0.26359050061632189, -0.26289014070470407, -0.24576738491637995, -0.18344663546442186, -0.00049409248625817436], [-0.023755279592317703, -0.089058566213742252, -0.24373296471649636, -0.16279493073610474, -0.15494438994833831], [-0.073674401719182256, -0.051403005186393909, 0.21727219431445524, -0.033135846154655882, 0.20316028101638656], [-0.010956248647121266, -0.18031277887407715, 0.14439325062776281, -0.10396315270767979, -0.15140841358587115]], [[-0.82319380244542961, 0.82100657666385946, 0.76753216687755232, 0.57290430816241311, 0.0015430520886434297], [0.074187798460228119, 0.2781301283256068, 0.7611786674297415, 0.5084089818796611, 0.48389172307396117], [0.23008534356245425, 0.16053171566338897, -0.67854161430392446, 0.10348333164328342, -0.63447007325652283], [0.034216392333081186, 0.56311726607670609, -0.45094048819589283, 0.32467718977559556, 0.47284885991929299]], [[-0.79353878824430191, 0.79143035583000543, 0.73988232641354446, 0.55226580803768721, 0.0014874646541099511], [0.071515231914716135, 0.26811067375639663, 0.73375770759602021, 0.49009388337814913, 0.46645984266264368], [0.22179667070006565, 0.15474866640623086, -0.65409759984649918, 0.099755412822289863, -0.61161370701965478], [0.032983769349848584, 0.54283133769289527, -0.43469565430430251, 0.31298090799483497, 0.45581479137524167]], [[0.084195107833696159, -0.083971401447680033, -0.078502113796428741, -0.058595849313772097, -0.00015782120396242417], [-0.007587824000554833, -0.028446759531728334, -0.077852286781768698, -0.051999357776767723, -0.049491766924163627], [-0.023532806314437795, -0.016418958780855586, 0.069400284861542538, -0.010584130056389692, 0.064892709440225974], [-0.0034996046296823279, -0.057594844372613481, 0.046121560824979195, -0.033207527708652602, -0.048362318364981026]], [[-0.38860916363588149, 0.3875766291596332, 0.36233270045020055, 0.27045376602327792, 0.0007284362197977971], [0.035022200393117951, 0.13129826321514301, 0.35933337258932485, 0.24000713883693556, 0.22843315539485409], [0.1086175244044152, 0.075782999793803998, -0.32032249081985659, 0.048851887417868153, -0.29951742079175647], [0.016152701305203677, 0.26583354873289339, -0.21287770321737673, 0.15327196438496837, 0.22322009645061405]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.871966327435)+(1.-msk_arg0)*(0.421982111576)
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[0.32398274675712746, -0.91115253337360302, -0.10695396226680187, -0.30758129496698361, 0.47976670604889704], [-0.1539010252015689, -0.07185794432350523, 0.16667408982619114, -0.71117924725670334, -0.15521765079425531], [0.64884133687323664, -0.34642653482565255, 0.2388205405039121, 0.62432707975566593, 0.56406107930529803], [0.80330443360880732, 0.5193551687248068, -0.41643480531373367, 0.74627938793961923, -0.18086586215540401]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.28250204584204569, -0.79449432825867883, -0.093260253682381541, -0.26820053215999529, 0.41834041269893968], [-0.13419651173345995, -0.062657707808779939, 0.14533419398427896, -0.6201243563782608, -0.13534456491612137], [0.56576779760123075, -0.30207227329788211, 0.20824346961918599, 0.54439219085263069, 0.49184226777074042], [0.70045441678594988, 0.45286021910724167, -0.36311712780543504, 0.65073049714198716, -0.15770894158197327]])+(1.-msk_ref)*numpy.array([[0.13671492359078327, -0.38449007000086588, -0.045132658838770871, -0.12979380433146548, 0.20245296768240253], [-0.064943479588277855, -0.030322767079147404, 0.07033348436987337, -0.30010492046645371, -0.06549907203603475], [0.27379943741159951, -0.14618580067170497, 0.10077799596957585, 0.26345485942940866, 0.23802368530311885], [0.3389801011326522, 0.21915859075643282, -0.17572803848005314, 0.3149165519484472, -0.076322158424361269]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.88204543507040856, 0.3316162385039283])+(1.-msk_arg0)*numpy.array([0.13864824415164678, -0.88295785230082879])
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.47031795499203799, 0.6472704042286983, -0.22175320045192071, 0.2441541371547824, 0.28019730682332145], [0.18473877880937928, 0.23668893252688639, 0.51922011601352303, 0.75100059572327105, -0.27326459689196514], [0.96311312572472585, -0.98620874537260694, -0.26233328718010607, -0.74251625431695989, -0.91589146006289113], [-0.11451434333834909, 0.29892330239426634, 0.61486206850621983, -0.44903168616193323, -0.070077360950145717]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.41484180523237696, -0.57092190530610143, 0.19559639817086991, -0.21535504213093024, -0.24714675540253331], [-0.16294799652929493, -0.20877039246702808, -0.4579757331264559, -0.66241664719286864, 0.2410317902549132], [-0.84950953600188694, 0.86988092188242283, 0.23138987842422709, 0.65493307258585298, 0.8078578813684445], [0.10100685379167626, -0.26366393431303398, -0.54233628072385998, 0.39606634898110155, 0.061811416327857339]], [[-0.15596507113531949, 0.21464537674523809, -0.073536962210073553, 0.080965476578441137, 0.092917976927680945], [0.061262378934575576, 0.078489893500076152, 0.17218182182797778, 0.24904399266796048, -0.09061897773760573], [0.31938395200659458, -0.32704283452014232, -0.086993977929037569, -0.24623044728461646, -0.30372448086392684], [-0.037974815792610701, 0.099127821141158909, 0.20389824635677728, -0.14890619873409672, -0.023238790842569395]]])+(1.-msk_ref)*numpy.array([[[-0.065208758652639309, 0.08974290503763567, -0.030745691877666972, 0.033851542418870922, 0.038848864607073756], [0.025613707308639887, 0.032816504904980393, 0.071988957413489305, 0.10412491395387226, -0.037887656547878519], [0.13353394380113748, -0.13673611091291046, -0.036372049650051419, -0.1029485749151041, -0.12698674277120797], [-0.015877212634040933, 0.041445191012976779, 0.085249546193836942, -0.062257454854805355, -0.009716103050518881]], [[0.41527093143828775, -0.57151248597566073, 0.19579872961186309, -0.21557781257254865, -0.24740241225319626], [-0.16311655537420738, -0.20898635152731537, -0.45844947850668744, -0.6631018730764624, 0.24128112158158127], [-0.85038829701264207, 0.87078075573449198, 0.231629235835563, 0.65561055721015893, 0.80869355651780062], [0.10111133865166844, -0.26393667708471258, -0.54289729146949695, 0.39647605322856033, 0.061875356119450629]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.82763319929521439, 0.22213088640836753, -0.40878221504339907, 0.030635030988743317, 0.21930855622565093], [-0.97929682999582401, -0.44905498476717298, 0.85103046752864397, 0.74119157366515487, -0.16977592036652522], [-0.59121898692726971, -0.76432211220764401, -0.39275255306377233, -0.073680487887200385, 0.75228830412779057], [-0.75145732785161279, -0.70555557387915901, -0.28780974908149215, 0.66284387393595345, -0.66632023823809816]])+(1.-msk_arg0)*numpy.array([[0.277179275256066, 0.7515933354845945, -0.099080160517141636, -0.23769729369617187, 0.3938748407232695], [-0.613694336424919, 0.017566129731441116, 0.60151121917495076, -0.20890148306604983, -0.5571886667714101], [0.34761437589554744, -0.048573009365031172, 0.89622876359930803, 0.69668867420839331, -0.41722081674867995], [-0.29056063672161336, 0.45537345668485707, -0.54956547167470871, 0.54761289129867863, 0.62245443255605704]])
      arg1=Symbol(shape=(4, 5))
      res=outer(arg0,arg1)
      s1=numpy.array([[-0.64398921752816163, -0.37493137030756207, 0.65918350625597943, -0.084683186061758065, -0.6119426038363931], [-0.97418212127916815, 0.79289389965190216, 0.5567626857095942, -0.95362473449008789, -0.53017211596159575], [0.30908720571089199, -0.63170909324743518, -0.34445135283782058, 0.24923697856327998, 0.58604176120619345], [0.26625885802882476, -0.24485289329567239, -0.085430451852495759, 0.48751549002951577, -0.49538853141975259]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.5329868564144542, -0.31030564952378636, 0.54556215420527321, -0.07008661620680473, -0.50646401499815796], [-0.80626546573047653, 0.65622531487056246, 0.46079528282202736, -0.78925148993308081, -0.43878804451040887], [0.25581083292372359, -0.5228234178682537, -0.28507937515073017, 0.20627679795100018, 0.48502761774768394], [0.22036467051108652, -0.20264838343498709, -0.070705078183916845, 0.40348400471910234, -0.40999999515308766]], [[-0.14304989572696156, -0.083283837628722657, 0.1464250165504164, -0.018810751183783034, -0.1359313530212225], [-0.21639593812292543, 0.17612622475746426, 0.1236741888957755, -0.21182950757322735, -0.11776760206754909], [0.068657814982045873, -0.14032210083527888, -0.076513284330426451, 0.055363230974004682, 0.13017797588905261], [0.059144316148022534, -0.05438939022742114, -0.018976741996262248, 0.10829224793806601, -0.11004109360080908]], [[0.26325133880522728, 0.15326527604358212, -0.26946249380739357, 0.034616980375257761, 0.25015125307566599], [0.39822832539217556, -0.32412092459410313, -0.22759468391787974, 0.38982483128503143, 0.21672493191702694], [-0.12634935259207317, 0.25823144240074369, 0.14080558698773971, -0.10188364416782175, -0.23956344925380249], [-0.10884188575994891, 0.10009150808118999, 0.034922449340421671, -0.1992876618822336, 0.20250602118086294]], [[-0.019728629635391792, -0.01148603414802416, 0.020194107141420403, -0.0025942720292274745, -0.018746880631860176], [-0.029844099474067017, 0.024290329186621558, 0.017056442130089375, -0.029214323292735961, -0.016241839201851103], [0.0094688961251772567, -0.01935242764750612, -0.010552277868301192, 0.0076353825618268355, 0.017953407515249448], [0.0081568483667404535, -0.0075010759737963841, -0.0026171645398835514, 0.014935052144546599, -0.015176243011512163]], [[-0.14123234552098779, -0.082225657505856323, 0.14456458304476119, -0.018571747271792328, -0.13420424894032484], [-0.21364647451857635, 0.17388841637278482, 0.12210282076328696, -0.20913806370209087, -0.11627128130263595], [0.067785468832276485, -0.13853920919471011, -0.075541128880834696, 0.054659801926756443, 0.12852397253806797], [0.058392845736592126, -0.053698334516347278, -0.018735629053475832, 0.10691631825601383, -0.10864294359641145]]], [[[0.63065659927681983, 0.36716910240818595, -0.64553631806201306, 0.082929975664226219, 0.59927345207637017], [0.95401346320729674, -0.7764784824521348, -0.54523593317536689, 0.93388167949175238, 0.51919587251336907], [-0.30268812074494367, 0.61863071249674972, 0.33732011792185079, -0.24407698302475722, -0.57390883899439493], [-0.26074645562593624, 0.23978366221975772, 0.083661770684259965, -0.47742237395976556, 0.48513241843565036]], [[0.28918656826733224, 0.16836480078219757, -0.29600963936055052, 0.038027406826998442, 0.27479587664413568], [0.4374613376314691, -0.35605295803016929, -0.25001705935025215, 0.42822994062004582, 0.23807643145711424], [-0.13879715045223268, 0.28367211724551167, 0.15467759700161965, -0.11192110761214991, -0.26316497415137441], [-0.11956486743625878, 0.10995241226908639, 0.038362970255275186, -0.21892126094896508, 0.22245668943052918]], [[-0.54805444487639698, -0.31907801936399971, 0.56098524751619694, -0.072067971425953115, -0.52078180024358134], [-0.82905866613025658, 0.674776866121368, 0.47382200872193941, -0.81156370363997843, -0.45119262371744728], [0.26304262918326254, -0.53760368496846056, -0.29313859584644436, 0.21210826239213476, 0.49873939403061673], [0.22659440043191359, -0.20837727225715724, -0.072703917381212776, 0.41489053540727477, -0.42159073350248039]], [[-0.47731938156308984, -0.27789597237469488, 0.48858126033598387, -0.062766463940093567, -0.45356670153024864], [-0.72205557950736543, 0.58768627723249478, 0.41266781117913215, -0.70681861764272369, -0.3929591049429601], [0.22909283240062148, -0.46821745692265454, -0.25530444026095572, 0.18473234835686594, 0.4343692152219174], [0.19734882198467169, -0.18148290129828568, -0.063320331047476572, 0.3613423732411159, -0.36717780517867643]], [[0.10933386211196205, 0.063654318468248844, -0.11191348646504204, 0.014377165853204676, 0.10389311877781157], [0.16539266624478466, -0.13461429156640498, -0.094524897392084778, 0.16190251698233793, 0.090010458940048055], [-0.0524755648230842, 0.10724899270998642, 0.058479545449535708, -0.042314437424952778, -0.099495779382000896], [-0.045204342677583699, 0.041570125313679374, 0.014504033590585587, -0.082768391012698589, 0.084105043860809789]]], [[[0.38073865277908486, 0.22166654492048987, -0.3897218047678257, 0.05006630747320609, 0.36179208629778786], [0.57595496682532843, -0.46877392809300983, -0.32916867100413216, 0.56380104943401632, 0.3134478212959016], [-0.18273822463257416, 0.37347841014249278, 0.20364617987050382, -0.14735363397099602, -0.34647901635739858], [-0.1574172923042135, 0.1447616795204783, 0.050508105196971437, -0.28822841412660177, 0.29288310568137405]], [[0.49221519898007243, 0.28656833688638217, -0.50382852983401094, 0.064725231639195843, 0.4677212635140775], [0.74458893661101699, -0.60602634013849754, -0.42554603193995771, 0.72887647131891764, 0.40522227150536283], [-0.23624218592530755, 0.48282922845165521, 0.2632717855537835, -0.19049733389573745, -0.44792467676700548], [-0.20350753276258657, 0.18714648058390121, 0.065296383406752992, -0.3726188690733041, 0.3786364086981881]], [[0.25292840932972654, 0.14725525291199365, -0.25889600501956506, 0.033259537527329942, 0.24034201998523599], [0.38261251528147477, -0.31141110339697509, -0.2186699662630858, 0.37453854913574403, 0.20822645210713908], [-0.12139478916230022, 0.24810535916653079, 0.1352841482333243, -0.097888459648628898, -0.23016939791572208], [-0.10457384626666542, 0.096166598966926753, 0.033553028074459389, -0.19147295336722836, 0.19456511047362063]], [[0.04744943974157137, 0.027625126288477769, -0.048568962348135972, 0.0062394984648729019, 0.045088229609629223], [0.071778213986836925, -0.058420809369137051, -0.041022546320470914, 0.07026353569853161, 0.03906334016823975], [-0.022773696116469989, 0.046544634193251985, 0.025379343730496825, -0.018363902180074172, -0.043179842887946518], [-0.01961808256385263, 0.018040880638617758, 0.0062945573729158693, -0.0359203791579423, 0.036500468688731066]], [[-0.48446555633084354, -0.2820564847329845, 0.49589604203032162, -0.063706170430538125, -0.46035726366362456], [-0.73286581592871902, 0.59648480712240004, 0.41884605663410468, -0.71740073428386275, -0.3988422820125912], [0.23252268981184449, -0.47522736246121733, -0.25912672408088727, 0.18749806392930438, 0.44087236268587088], [0.20030342476550675, -0.18419996785818424, -0.064268329744984906, 0.3667522012303332, -0.37267499818612237]]], [[[0.48393041656896335, 0.28174492565906412, -0.49534827617497518, 0.06363580071192966, 0.45984875387745405], [0.73205629369725944, -0.59582593110226323, -0.41838340005081898, 0.71660829475312515, 0.39840172156193615], [-0.23226584567662864, 0.47470242719128292, 0.2588404931783817, -0.18729095391297207, -0.44038537588545906], [-0.2000821699711626, 0.18399650091270206, 0.064197339066232331, -0.36634708742384953, 0.37226334206902201]], [[0.45437018194507267, 0.26453491814265145, -0.46509059704811379, 0.059748693939719308, 0.43175951503089316], [0.68733962564193996, -0.55943071039418213, -0.39282701623033456, 0.67283524680851459, 0.37406589153201175], [-0.21807820080405407, 0.44570587181087729, 0.24302957192494118, -0.17585053944212264, -0.41348503114498891], [-0.18786042137693698, 0.17275732364520063, 0.060275931483543509, -0.3439692713427544, 0.34952413957901735]], [[0.18534637510796667, 0.10790890361099946, -0.18971943953419165, 0.024372646531855901, 0.17612304726242725], [0.28037911188503312, -0.22820259430705978, -0.16024172887201599, 0.27446249555149677, 0.15258870366491062], [-0.088958311119951367, 0.18181203562004125, 0.099136457431033664, -0.071732832262126844, -0.16866853224403028], [-0.076631895120000695, 0.070471049781304848, 0.024587716911585299, -0.14031171085873562, 0.14257764892576788]], [[-0.42686430771935008, -0.24852096195477996, 0.43693574892139819, -0.056131731106414853, -0.40562240615336925], [-0.64573065118782869, 0.525564864065452, 0.36904673545873312, -0.63210431329055494, -0.35142133919680568], [0.2048765608174466, -0.41872450256869836, -0.22831746909750097, 0.16520520439897668, 0.38845419128616221], [0.17648805292558925, -0.16229924033653012, -0.056627051658007237, 0.32314665601494891, -0.32836525326971161]], [[0.4291030488461311, 0.24982435998627137, -0.43922731093110906, 0.056426120711431826, 0.40774974157630756], [0.6491172631380312, -0.52832125211359016, -0.37098224538410018, 0.63541946027517848, 0.35326441061472708], [-0.20595106054562962, 0.42092055350980395, 0.22951490748433181, -0.16607164293402848, -0.39049148594438543], [-0.17741366571477046, 0.16315043819406005, 0.056924039031143348, -0.32484143746123012, 0.33008740427603112]]]])+(1.-msk_ref)*numpy.array([[[[-0.17850046458717689, -0.10392320549261376, 0.18271200652478484, -0.02347242413897269, -0.16961780742968136], [-0.2700230943435768, 0.21977375646047018, 0.15432307771460618, -0.26432501277222092, -0.14695272286321009], [0.085672567669867625, -0.1750966686389907, -0.095474776340558579, 0.069083325085181599, 0.16243863064092121], [0.073801437298937414, -0.067868147508045332, -0.023679550729273018, 0.13512919020248707, -0.13731143410909391]], [[-0.48401800401810507, -0.28179591918727026, 0.49543793016336163, -0.063647318271619263, -0.4599319827425225], [-0.7321887899016678, 0.5959337707247605, 0.41845912402583491, -0.71673799499601598, -0.39847382901650097], [0.23230788389586232, -0.47478834444978857, -0.25888734119155854, 0.18732485204447796, 0.44046508203822915], [0.20011838320820352, -0.18402980277514791, -0.064208958259773338, 0.36641339325169031, -0.3723307186905867]], [[0.063806555043998694, 0.037148260352985123, -0.065312007610094674, 0.0083904236681019602, 0.060631371415387443], [0.096522120949269513, -0.078560054850572852, -0.055164136270061476, 0.094485291766394475, 0.052529538351187538], [-0.030624409955629955, 0.062589838359093874, 0.034128295329517853, -0.024694439842857168, -0.058065111770058034], [-0.026380970392606784, 0.024260063970821772, 0.00846446288259722, -0.048303113006717385, 0.049083175211420152]], [[0.15307449417595936, 0.089120172043904752, -0.15668613548619989, 0.020128964148449276, 0.14545710082929927], [0.23156045379525417, -0.18846873413546122, -0.13234098362418284, 0.22667401859002434, 0.12602047715724432], [-0.073469192313590989, 0.15015554186817803, 0.081875154379535159, -0.059242955293502451, -0.1393005406316504], [-0.063289009976084887, 0.058200870090058872, 0.020306587204579354, -0.11588111261497896, 0.1177525132465962]], [[-0.25365115048140763, -0.14767603376204819, 0.25963579853398022, -0.033354576422013955, -0.24102879561784213], [-0.38370582785428919, 0.31230095843584499, 0.21929481415452617, -0.37560879040705353, -0.20882145773029229], [0.12174167391897801, -0.24881431848627453, -0.1356707217359113, 0.098168175233960833, 0.23082710535227377], [0.10487266529726298, -0.096441394347464668, -0.033648905616318711, 0.19202008598550221, -0.19512107890908945]]], [[[0.39521253551574798, 0.23009325850578488, -0.40453718445401476, 0.051969591676518566, 0.37554571019151234], [0.59785005047543904, -0.48659449560224038, -0.34168210695270512, 0.58523409863128406, 0.3253636248960467], [-0.18968506760617829, 0.38767629279407201, 0.21138784441047193, -0.15295532217194385, -0.35965050976072571], [-0.16340155319525632, 0.15026483387280917, 0.052428184460098379, -0.29918549515053289, 0.30401713606216019]], [[-0.01131239814074894, -0.0065860930912096253, 0.011579302987718761, -0.0014875558324326083, -0.010749463167185957], [-0.01711260952444037, 0.013928077104553568, 0.0097801655668003085, -0.016751495801163974, -0.0093130721689740348], [0.0054294659538461562, -0.01109668388451548, -0.0060506771501195539, 0.0043781290993149848, 0.01029448560539023], [0.0046771376422796978, -0.0043011176887504899, -0.0015006824002565746, 0.0085637603439455614, -0.0087020592103874677]], [[-0.38736673937088706, -0.2255254256606366, 0.39650627450805298, -0.050937886491627289, -0.36809034169872273], [-0.5859814754690722, 0.47693457625599672, 0.33489900187229793, -0.57361597667852149, -0.31890447584462284], [0.18591942193853744, -0.37998010684316741, -0.2071913531919386, 0.14991883883907961, 0.3525106942705728], [0.1601576903090485, -0.14728176236479404, -0.051387375248461653, 0.29324603677432759, -0.29798175949958378]], [[0.13453030262017793, 0.078323719305236031, -0.13770441206955283, 0.017690443159059498, 0.12783571749272271], [0.20350808991164865, -0.16563671155130605, -0.11630855076057121, 0.19921362132344736, 0.11075374130464311], [-0.064568775669746559, 0.13196496644569877, 0.07195639845192793, -0.052065974456770456, -0.12242499305461364], [-0.0556218703216943, 0.051150132542479215, 0.01784654809098913, -0.10184270888483787, 0.10348739890749874]], [[0.35882349352967996, 0.20890751035244837, -0.36728957900847264, 0.047184511539706236, 0.34096748357222506], [0.54280323734808389, -0.44179149483822761, -0.31022185855859841, 0.53134889441077204, 0.29540589445201898], [-0.17221988806615249, 0.35198114745391479, 0.19192439005531381, -0.13887201979580849, -0.32653582759884803], [-0.14835641812115904, 0.13642925717053803, 0.047600879569371257, -0.27163810591995657, 0.27602487535561876]]], [[[-0.22385990993451385, -0.13033153429312558, 0.22914166312781098, -0.02943709287170455, -0.21272004631648403], [-0.33863971009705857, 0.27562131807888279, 0.19353871351486943, -0.33149366691832904, -0.18429544920721191], [0.10744315611049041, -0.21959116219674937, -0.11973624204309601, 0.086638356753366508, 0.20371654107041839], [0.092555406760351094, -0.08511438568919423, -0.02969685320317993, 0.1694673928060221, -0.1722041751752891]], [[0.03128049429397449, 0.018211544961193184, -0.032018526622645774, 0.0041133171896384517, 0.029723893827006683], [0.047318957300138971, -0.038513242813267928, -0.027043639147072026, 0.046320423159112403, 0.025752055153680984], [-0.015013295737606474, 0.030684011702283021, 0.016731038787189116, -0.012106190093866272, -0.028465811955367796], [-0.01293299400455661, 0.011893241879105674, 0.0041496141378901214, -0.023680094462801431, 0.024062511775980681]], [[-0.57716166019655013, -0.33602427844534066, 0.59077921879685313, -0.075895507141779589, -0.54844056323003176], [-0.87309003807457997, 0.71061431935045805, 0.49898673343173972, -0.85466591672976988, -0.47515549998308992], [0.2770128442186377, -0.56615585959558878, -0.30870721007394891, 0.22337334914099566, 0.52522748306338762], [0.2386288471285373, -0.21944420582209376, -0.076565228237492483, 0.43692540486466369, -0.44398145101560182]], [[-0.44865999416419555, -0.26121043929871157, 0.45924568303351843, -0.058997816625108918, -0.42633348135840876], [-0.67870165051150388, 0.55240019973640653, 0.38789025735572152, -0.66437955196423037, -0.36936490857154269], [0.21533755556149828, -0.44010457065994191, -0.23997535633786871, 0.17364058015895728, 0.40828865764549471], [0.18549953079634274, -0.17058623760625119, -0.05951842823813925, 0.33964652040471854, -0.34513157917287041]], [[0.26868570731444291, 0.15642917254442282, -0.27502508086737831, 0.035331588053567127, 0.2553151929759338], [0.40644906030205613, -0.33081184040781247, -0.23229298246694549, 0.3978720905956975, 0.22119884323887284], [-0.12895761641326561, 0.26356218383226293, 0.14371227476118323, -0.10398685576014491, -0.24450882225928289], [-0.11108873821335709, 0.10215772412409782, 0.03564336289710706, -0.2034016109277475, 0.20668640768687827]]], [[[0.18711791708683623, 0.10894029768347224, -0.19153277929412299, 0.024605600461719276, 0.17780643260778436], [0.28305897744168707, -0.2303837563355397, -0.16177332046261519, 0.2770858100469194, 0.15404714758584628], [-0.089808575293861062, 0.18354979635680779, 0.10008400440017826, -0.072418455185917732, -0.17028066728152727], [-0.077364343321624979, 0.071144612579119831, 0.024822726485676301, -0.14165281119462544, 0.14394040711390829]], [[-0.29325559605357532, -0.17073379411654471, 0.30017467183342944, -0.038562475160029676, -0.27866241880171039], [-0.44361668000748145, 0.36106283586882287, 0.2535349487447226, -0.43425539172493038, -0.24142630908335674], [0.14075010928163237, -0.28766355341134126, -0.1568540032015337, 0.11349590446205042, 0.2668678625621459], [0.12124721655354855, -0.1114995083993388, -0.038902760166220247, 0.22200161388215256, -0.22558678795464765]], [[0.35391423808429073, 0.20604933536872022, -0.36226449453575565, 0.046538955090947187, 0.33630252571519675], [0.53537685697785431, -0.43574710995019678, -0.30597754798287075, 0.52407922701071397, 0.29136428897721273], [-0.16986365599512407, 0.34716550579172928, 0.18929857019130839, -0.13697203768290822, -0.32206831691835869], [-0.14632667490018039, 0.13456269579495331, 0.046949626567700325, -0.26792168022679758, 0.2722484319319376]], [[-0.35265679737577027, -0.2053172517326996, 0.36097738575723753, -0.046373604363663293, -0.33510765859568908], [-0.53347468808516529, 0.4341989208814625, 0.30489042408864836, -0.52221719806805178, -0.29032908530766777], [0.16926013838277101, -0.34593204301289454, -0.18862600123926024, 0.13648538244958452, 0.32092402327589337], [0.14580678307904912, -0.13408460084049001, -0.046782816743897761, 0.26696976704795528, -0.271281146006977]], [[-0.40085394296871102, -0.23337769335225844, 0.41031169533687772, -0.052711424527110612, -0.38090638622785805], [-0.60638397950708056, 0.49354032238498396, 0.3465594016017518, -0.59358794297844819, -0.33000798359791916], [0.19239270124111055, -0.3932101251778336, -0.21440527137383181, 0.15513866206359259, 0.3647842919257534], [0.16573400638735586, -0.15240976875606654, -0.053176563430852801, 0.30345617770861033, -0.30835678721966053]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.301813613463)+(1.-msk_arg0)*(0.727110642101)
      arg1=Data(numpy.array([[-0.84614696308677417, -0.33247673122998478, 0.57230573144163022, -0.70955774937999205, 0.29481519101937215], [-0.17747847979304732, 0.35977904309928066, -0.78470207141906911, 0.79315989800765641, -0.48880985228402341], [-0.85970978747551619, -0.54651625396590786, -0.91193431861718222, -0.89861413456560735, -0.013520906220351403], [-0.46786512942225889, 0.69005135718166533, 0.59083978122721126, -0.52809667689026507, -0.5516605228719571]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.25537867244997542, 0.10034600364489327, -0.17272966081199209, 0.21415418830105959, -0.088979238105345637], [0.05356542129826225, -0.1085862130460596, 0.2368337676669019, -0.23938645487164698, 0.14752946781416357], [0.25947211748750598, 0.1649460454257213, 0.27523419194278392, 0.27121397906218569, 0.0040807935636588098], [0.14120806532427299, -0.20826689358605585, -0.17832348934988171, 0.15938676631006105, 0.16649865581288145]])+(1.-msk_ref)*numpy.array([[-0.61524246164179008, -0.24174736952825798, 0.4161295878665755, -0.51592699075938842, 0.21436326284320883], [-0.12904649140142246, 0.26159917104238201, -0.57056522700746204, 0.57671500272906817, -0.35541884555950498], [-0.62510413559179068, -0.39737778433975512, -0.66307714796362838, -0.65339190038498496, -0.0098311948036663883], [-0.34018971467086101, 0.50174368540299019, 0.42960589270690025, -0.38398471382505661, -0.40111823700717253]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.50837565399440865, 0.027194880772694496])+(1.-msk_arg0)*numpy.array([-0.95525264517241903, 0.39119361412161857])
      arg1=Data(numpy.array([[0.57395007622644179, 0.042614720839584175, 0.067192262504945255, 0.89561435895389252, -0.10709780675364455], [-0.27314154100005039, 0.054487430934573089, 0.72790883928028505, -0.48871374949030733, -0.68034067253660457], [0.51320482651384713, 0.11688658195513679, 0.51427970574258475, 0.39407629047254322, -0.36105065851902607], [-0.96414116432459473, 0.42030377683323783, 0.06595019391169199, 0.82912154649728698, 0.49091056947243605]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.29178224536175806, -0.021664286576612759, -0.034158910394315525, -0.45530853545996819, 0.054445917549750845], [0.13885850953894119, -0.027700083335838767, -0.3700511322174258, 0.24845017201319458, 0.34586863433979215], [-0.26090083931206409, -0.05942229254461371, -0.26144728174293858, -0.2003387918926697, 0.1835493646497218], [0.49014589495644645, -0.21367220742391727, -0.033527472960914483, -0.42150520844141376, -0.24956698180831727]], [[0.015608503892457002, 0.0011589022523941512, 0.001827285567669577, 0.024356125710064319, -0.0029125120856824389], [-0.0074280516415664161, 0.001481779187876141, 0.019795394097617793, -0.013290512149365394, -0.018501783474447749], [0.013956544069015435, 0.0031787166601977292, 0.013985775281486002, 0.010716857734746538, -0.0098187296113277488], [-0.026219704011854205, 0.011430111099293098, 0.0017935076603645462, 0.022547861603065794, 0.013350254406858457]]])+(1.-msk_ref)*numpy.array([[[-0.54826732851222004, -0.040707824805296992, -0.064185586492968502, -0.85553798544510617, 0.10230546319358352], [0.26091917954676891, -0.052049262528900436, -0.69533684416687758, 0.46684510193274703, 0.64989722705897401], [-0.4902402680426049, -0.11165621659780717, -0.49126704926909737, -0.37644241887363145, 0.34489459659154342], [0.92099839754068502, -0.40149629459590852, -0.062999097183777733, -0.79202055046098019, -0.46894362003164314]], [[0.22452560464440025, 0.016670606660020789, 0.026285184010318055, 0.35035861793838979, -0.041895978088456902], [-0.10685122659055797, 0.021315135031497727, 0.2847532895891271, -0.19118169793404066, -0.26614492652352695], [0.20076245086861011, 0.045725284437352728, 0.20118293675884424, 0.15416012830959494, -0.14124071198704816], [-0.37716586659556356, 0.16442015348836053, 0.025799294708336355, 0.3243470543203793, 0.19204107988242416]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.83545533635143987, 0.88593802697563584, -0.13137432742715816, 0.79834957810499785, -0.34659285335561085], [0.79205920222768489, -0.5442503557523124, 0.86098363592905813, -0.76446806321431415, -0.35901720438855866], [-0.9188798609420834, -0.017033504748293238, 0.96462600688012778, 0.7697190958071789, 0.6421550306644519], [-0.53342932023783485, 0.97574947391618738, 0.84854160434718096, 0.72296579812545336, 0.78585488081390209]])+(1.-msk_arg0)*numpy.array([[0.51969551986658802, -0.35447466539562233, -0.059544973375288857, -0.062774266876408635, -0.42472411284195855], [0.39001387745137195, 0.97309649704704704, 0.11415452203338505, 0.11381837680652263, -0.82397904811236433], [0.9895740872665495, -0.31498857505399092, -0.40291668499781208, 0.93169920182411015, 0.41199078249099985], [-0.12259217203566819, 0.2331181342170332, 0.30338795758460391, -0.12961221673806556, 0.98861933981363448]])
      arg1=Data(numpy.array([[-0.30478629701541937, -0.85322616191583101, -0.070080896897894318, -0.07976899175371166, 0.56375603635745741], [-0.94577255650508674, 0.086139062157043034, -0.61915925584671561, -0.36178122159045034, -0.63253379352393613], [-0.79337541122853206, 0.54284661040204329, 0.81227036754821502, 0.26828885205022979, -0.91953815164783226], [0.91196283696851133, 0.6331936479996243, 0.1362363900972261, 0.53126051613995862, 0.23306894578328841]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.25463533828832702, 0.71283235008723866, 0.058549459289640879, 0.066643429836012402, -0.47099298897517416], [0.79015072930691843, -0.071965339147409976, 0.517279904348525, 0.30225205216948448, 0.52845373322219225], [0.66282972104089521, -0.45352409748067812, -0.67861561312830165, -0.22414335312896641, 0.768233055772921], [-0.76190421869954095, -0.5290050121651213, -0.11381941911198401, -0.44384443320194866, -0.19471869449245272]], [[-0.27002177062705074, -0.75590550245170574, -0.062087331526403451, -0.070670383168119077, 0.49945291054613067], [-0.83789587267781962, 0.076313870772942369, -0.54853672950854215, -0.32051574165267888, -0.56038574103001015], [-0.70288144647478945, 0.48092845496999792, 0.71962120679644015, 0.23768729624493884, -0.81465381579970353], [0.80794255645898638, 0.56097033120229245, 0.12069699864501955, 0.47066389347909288, 0.20648464197653796]], [[0.040041094779414782, 0.11209201316494784, 0.0092068306954528802, 0.010479597641186394, -0.07406307010946149], [0.12425023350991971, -0.011316461356087699, 0.081341630807162002, 0.047528764662221085, 0.083098701699156036], [0.10422916104739342, -0.071316108337680997, -0.1067114732256573, -0.035246267494303277, 0.12080370621634612], [-0.11980850434530126, -0.083185389637099372, -0.017897964120127029, -0.069793992996491958, -0.030619275996436303]], [[-0.24332601163444462, -0.68117274639365022, -0.055949054471653784, -0.063683540912436762, 0.45007439378012193], [-0.75505712146912118, 0.068769083931435493, -0.4943055306850298, -0.28882788562304679, -0.50498308719698826], [-0.63339092483317772, 0.43338136239019942, 0.64847570523930897, 0.21418829184457513, -0.73411289541949643], [0.72806514614124795, 0.50550988173926459, 0.10876426455666836, 0.42413160892417934, 0.18607049453546493]], [[0.10563675234626491, 0.29572209001606431, 0.024289538021561568, 0.027647362461219117, -0.19539381323758065], [0.32779800898452871, -0.029855183338385865, 0.21459617316544985, 0.1253907858815127, 0.21923169234130982], [0.27497824755987804, -0.18814675563366581, -0.2815271043847466, -0.092986998755590466, 0.31870535174896658], [-0.3160798018191941, -0.21946039318683805, -0.047218559174665678, -0.18413109816412282, -0.080780030947614101]]], [[[-0.24140879126396328, -0.67580563312684261, -0.055508219288346809, -0.063181763970951627, 0.44652815640832944], [-0.74910785659425705, 0.068227236852748466, -0.49041078623783663, -0.28655214575388938, -0.50100421188062005], [-0.62840029528473251, 0.4299666531670453, 0.64336621931342752, 0.21250065412148639, -0.72832865481210196], [0.72232855711057531, 0.50152685569021993, 0.10790728645478857, 0.42078978058888372, 0.18460440326115896]], [[0.16588005057907193, 0.46436864215987111, 0.038141553068120108, 0.043414302139960843, -0.30682442334505977], [0.51473705033866746, -0.046881215223141225, 0.33697764526191198, 0.19689955855510877, 0.34425674215076196], [0.4317948498062657, -0.295444460830249, -0.44207843650517759, -0.14601630317271708, 0.50045896616215646], [-0.49633609845300036, -0.3446158681839, -0.074146703776826112, -0.28913872490632958, -0.12684785665737114]], [[-0.26241601418568961, -0.7346137631560874, -0.060338505420318503, -0.06867979655450572, 0.48538472195999799], [-0.81429469446167013, 0.074164322931490043, -0.53308598731803514, -0.31148771157580218, -0.54460124539623866], [-0.68308324621625327, 0.46738204837571612, 0.69935149440909461, 0.23099231131744, -0.79170730118123622], [0.78518507920532776, 0.54516936930190074, 0.11729730249175926, 0.45740661081172962, 0.20066854836264816]], [[0.23299939017364035, 0.65226415148357819, 0.053574607519855308, 0.060980846630518552, -0.43097348523956391], [0.72301291451269412, -0.065850562014292111, 0.47332747713835471, 0.27657018977656017, 0.48355188405284633], [0.60651016402373592, -0.41498889687650542, -0.62095475468596306, -0.20509825910883084, 0.70295754984188863], [-0.69716646370074919, -0.48405632172587898, -0.1041483692769362, -0.40613169783575104, -0.17817376557835249]], [[0.10942352429041677, 0.30632287136220132, 0.025160247685324832, 0.028638440416311549, -0.20239811613022901], [0.33954861922387636, -0.030925405284273878, 0.22228882510538819, 0.12988568277568113, 0.22709051423225335], [0.28483542216989066, -0.19489127247834664, -0.29161903656482718, -0.096320313631689122, 0.33013001653322727], [-0.32741034823469384, -0.22732741334141818, -0.048911207908695235, -0.19073166530659069, -0.083675761344904745]]], [[[0.28006199021858108, 0.78401233701336637, 0.064395924796233614, 0.073298120050140775, -0.51802406829340053], [0.86905135520423282, -0.079151449456545175, 0.56893297091343387, 0.33243347858649008, 0.58122256423444296], [0.72901668754454174, -0.49881081787911086, -0.74637888238007877, -0.24652522306422633, 0.84494508891710052], [-0.8379842848179736, -0.58182889122330528, -0.12518487518779056, -0.48816458919470462, -0.21416236049126602]], [[0.0051915788374268593, 0.014533431880361323, 0.0011937232900749318, 0.0013587454998034117, -0.0096027411221737257], [0.01610982133203483, -0.0014672501242655189, 0.010546452124414738, 0.0061624021558042636, 0.010774267375445901], [0.0135139638343403, -0.0092465803158780936, -0.013835811162530414, -0.0045698994353117308, 0.015662957472330138], [-0.01553392331377011, -0.010785507009790718, -0.0023205831976114305, -0.0090492285242507013, -0.0039699809946793421]], [[-0.2940047886417646, -0.82304414553452543, -0.06760185573319373, -0.076947243988236727, 0.54381373420606227], [-0.91231680459831188, 0.083091979564947543, -0.59725712059028868, -0.34898357514701078, -0.61015854746373377], [-0.76531055489025812, 0.52364395814053544, 0.78353712115508833, 0.25879840404366655, -0.88701041539798187], [0.87970306984800806, 0.6107950602517388, 0.13141716497125061, 0.51246771029716398, 0.2248243664986945]], [[-0.23459983295312686, -0.65674446986888302, -0.053942604593603345, -0.061399716206117248, 0.4339337865609012], [-0.72797919703233938, 0.066302881037197545, -0.47657870257097967, -0.27846991476261806, -0.48687333961872892], [-0.61067620416647439, 0.41783940212065268, 0.62522001286017692, 0.20650705261524888, -0.70778607464657395], [0.70195521028115226, 0.48738124220911994, 0.10486375100167097, 0.40892136412130414, 0.17939761820904515]], [[-0.19572005390604136, -0.54790347216877311, -0.045002800496459619, -0.051224059345677121, 0.36201877481439293], [-0.60733260502412101, 0.055314632100863098, -0.39759623092442686, -0.23231963144423851, -0.40618475757666528], [-0.50947001152588012, 0.34859168174881788, 0.52160350278074963, 0.17228303601524592, -0.59048604996854714], [0.58562152353835495, 0.4066084864477349, 0.087484883260498456, 0.34115161303266767, 0.14966639602639906]]], [[[0.16258194723474198, 0.45513585155989855, 0.037383205193901557, 0.042551119047239863, -0.30072399925413462], [0.50450281191610769, -0.045949101372356069, 0.3302777009652772, 0.19298471110780743, 0.33741207150693225], [0.42320970630504856, -0.28957029838017473, -0.43328883001058061, -0.14311313997654312, 0.49050861116625827], [-0.48646771620628038, -0.33776405723135444, -0.072672484961219808, -0.28338993599373941, -0.12432580931772833]], [[-0.29739506896965828, -0.83253497862089987, -0.068381398279694944, -0.077834551738508839, 0.55008465589286404], [-0.92283707445420604, 0.0840501445833685, -0.60414431816277081, -0.35300783663963753, -0.61719451626519095], [-0.77413564012427893, 0.52968229451697924, 0.79257238381287898, 0.26178270624558952, -0.89723886771623562], [0.88984725840313872, 0.61783836892270494, 0.13293258596560883, 0.51837716913600684, 0.22741690123424407]], [[-0.25862385345250039, -0.72399789630304678, -0.059466556687828624, -0.067687308239851543, 0.47837045155116459], [-0.80252736244436118, 0.073092577999698843, -0.52538238830257877, -0.30698641819104372, -0.53673123996060923], [-0.67321204429346304, 0.4606279337049789, 0.68924520084303675, 0.22765425294716546, -0.78026637845769298], [0.7738384087862672, 0.53729115393604532, 0.11560224502356863, 0.45079665069171193, 0.19776869717845769]], [[-0.22035006847945415, -0.61685333313099611, -0.050666091559133777, -0.057670252788884858, 0.40757633277321131], [-0.68376121115885047, 0.062275595822144654, -0.4476309655699825, -0.26155544961394145, -0.45730029887635321], [-0.57358328739194542, 0.39245953294901026, 0.58724369456815062, 0.19396366405065604, -0.66479463371287917], [0.65931794028969248, 0.45777735109401574, 0.09849425050037168, 0.38408318306366546, 0.16850087640647313]], [[-0.23951779911476295, -0.6705119437796685, -0.055073414879026103, -0.062686851507258209, 0.44303043275980752], [-0.74323997966936439, 0.067692802424844351, -0.48656932320824503, -0.28430753877367126, -0.49707976892051819], [-0.6234779392316786, 0.42659865831772847, 0.63832663287826696, 0.21083610385163193, -0.72262354456704303], [0.71667044655259748, 0.49759831878086463, 0.10706203210237189, 0.41749366959229928, 0.18315836860994791]]]])+(1.-msk_ref)*numpy.array([[[[-0.15839607307564069, -0.44341781378062139, -0.036420728146067942, -0.041455587638678751, 0.29298148639271593], [-0.49151376042846306, 0.044766084688524818, -0.32177429134746866, -0.18801608003241838, -0.32872497865860706], [-0.41231364678778004, 0.28211495140070508, 0.42213327093519415, 0.13942851444065429, -0.47787985775778163], [0.47394300065735895, 0.3290679020733861, 0.070801441576325203, 0.27609371011994766, 0.12112488694360368]], [[0.10803902065171155, 0.30244705825190527, 0.024841902478506196, 0.028276086660843098, -0.199837232352572], [0.33525241050750293, -0.030534115235610542, 0.21947627004286704, 0.12824227746969438, 0.22421720481082091], [0.28123148342834819, -0.19242537058341205, -0.28792926674743269, -0.09510160105988083, 0.32595297862387435], [-0.32326772148765553, -0.22445110650530029, -0.048292348794421701, -0.18831839369661746, -0.082617036570641594]], [[0.01814849194093603, 0.050805329094378057, 0.0041729651399014809, 0.0047498424901483973, -0.033568838175063176], [0.056316001696174268, -0.005129148162713479, 0.036867821404456345, 0.021542253207282844, 0.037664207894353235], [0.047241517738211787, -0.032323786963255469, -0.048366617409194554, -0.015975252552217743, 0.0547538747574325], [-0.054302802846542902, -0.037703498911539651, -0.0081121922210847953, -0.031633893288896049, -0.013878084171272551]], [[0.019132736349118284, 0.053560646794038221, 0.0043992769248064958, 0.0050074399768095358, -0.035389371879489359], [0.05937017886643358, -0.0054073164763297713, 0.038867268365520298, 0.022710550954992061, 0.039706845163018721], [0.049803559797640319, -0.034076797994331692, -0.050989676828270179, -0.016841635998566435, 0.057723333334580539], [-0.057247798509228072, -0.039748267043975168, -0.0085521395102417866, -0.033349489421068357, -0.014630732203203361]], [[0.12945008960625967, 0.36238572467325059, 0.029765046762126927, 0.033879814254892691, -0.23944078240122002], [0.4016924100118941, -0.036585336755688425, 0.26297186564738351, 0.15365720838288405, 0.26865235429701234], [0.3369656676846623, -0.23056004501227215, -0.34499081124472725, -0.11394874467242132, 0.39055002568295988], [-0.38733260687628662, -0.26893261040380378, -0.057862879920835344, -0.22563915140550492, -0.098990001228817709]]], [[[-0.11887088549302922, -0.33277004375174535, -0.027332522334417589, -0.0311110137742516, 0.21987267767638857], [-0.36886442194964564, 0.03359542963189309, -0.24148070213268358, -0.14109969702158554, -0.24669695743129583], [-0.30942742040781651, 0.21171771138423517, 0.31679671558633038, 0.10463637546508758, -0.3586326399886387], [0.35567816213764247, 0.24695430983391262, 0.053134082751796846, 0.20719897383656244, 0.090900123258443891]], [[-0.29658647797364546, -0.83027138934919176, -0.068195475281256232, -0.077622926448511598, 0.54858902416856947], [-0.92032796173833031, 0.083821619643936432, -0.60250170297869532, -0.35204803942706875, -0.61551641874202234], [-0.772030833509745, 0.52824213501609141, 0.79041744931628544, 0.26107094212685206, -0.89479935426962187], [0.88742784209114567, 0.6161585208208753, 0.13257115397394573, 0.5169677472751999, 0.22679857471216608]], [[-0.034792734058120535, -0.097399624699881296, -0.0080000512890500627, -0.0091059911267299884, 0.0643553008738212], [-0.10796421414013084, 0.0098331634689412944, -0.07067982891372819, -0.041298962431312022, -0.072206592869688793], [-0.090567390861833394, 0.061968395347888443, 0.09272433556934849, 0.03062638567267954, -0.10496943819282063], [0.10410468176635027, 0.072281918241972568, 0.015551999995102638, 0.060645790295166424, 0.026605874106716224]], [[-0.034690281599165727, -0.097112816798119131, -0.0079764939300635985, -0.0090791771609003494, 0.064165796973084763], [-0.10764629720956417, 0.0098042082343507975, -0.070471701485207622, -0.041177351400505935, -0.071993969654166545], [-0.090300701504238898, 0.061785920050883347, 0.092451294762375361, 0.030536201655642452, -0.10466033983222632], [0.10379812981162738, 0.072069073219517887, 0.015506204782846488, 0.060467209608445503, 0.026527529093061313]], [[0.25113752289245761, 0.70304048071997249, 0.057745190716787707, 0.065727977894106379, -0.46452316220541723], [0.77929677083985871, -0.070976782441452099, 0.5101742542625366, 0.29810014659102763, 0.52119459308675564], [0.6537247161398414, -0.44729423331009915, -0.66929376426225851, -0.22106439293150729, 0.75768017089778372], [-0.75143827031916521, -0.52173829934952587, -0.1122559310305771, -0.43774753438868647, -0.19204392809106624]]], [[[-0.3016086216803851, -0.84433050040979962, -0.069350239582554929, -0.078937327206852134, 0.55787836511943856], [-0.93591201436527227, 0.085240983812052432, -0.61270395547714962, -0.3580093221455472, -0.62593905139169714], [-0.78510374842619801, 0.53718693901434222, 0.80380170758018954, 0.26549169589139648, -0.90995112712367354], [0.9024547920141277, 0.62659202628220501, 0.13481600138295211, 0.5257216403599555, 0.23063898929367455]], [[0.096004201392869387, 0.26875649294065335, 0.022074681852373382, 0.025126321045995187, -0.1775767105703214], [0.29790754989870738, -0.02713282044533414, 0.19502809173064634, 0.11395695147006808, 0.19924091829559995], [0.24990419026574937, -0.17099048028342859, -0.2558558856325937, -0.084507923210172864, 0.2896440120953313], [-0.28725787451890639, -0.19944876491663996, -0.042912906387224892, -0.16734099296137331, -0.073414055121613878]], [[0.12280348442621132, 0.34377905671253306, 0.02823676265977303, 0.032140257723023313, -0.22714671331665276], [0.38106754322893543, -0.034706865373136261, 0.24946959485147086, 0.14576769049768312, 0.25485841923575486], [0.31966419065097607, -0.21872195672549011, -0.32727728381448118, -0.10809805488994705, 0.37049726379095999], [-0.36744504311255272, -0.25512428561368011, -0.054891914674043092, -0.21405372603333878, -0.093907367010937354]], [[-0.28396914965619241, -0.79495013403242876, -0.065294315702885894, -0.074320705947247184, 0.5252510490977671], [-0.88117553600293741, 0.080255695457594409, -0.57687018447439498, -0.33707127539077408, -0.58933123055302783], [-0.73918723738849845, 0.50576975362450738, 0.75679165311004848, 0.24996450931350586, -0.85673296193710291], [0.84967504729681309, 0.58994601644134648, 0.12693133591298367, 0.49497499884826424, 0.21715015075627661]], [[-0.12556914499991692, -0.35152131408949577, -0.028872683550634565, -0.032864089331129782, 0.23226229055293343], [-0.38964957561304403, 0.035488499621121036, -0.25508790630283351, -0.14905052857359943, -0.26059809254592697], [-0.32686335648116172, 0.22364779979212476, 0.33464790432044117, 0.11053253408978626, -0.37884124262771812], [0.37572028280536912, 0.26086994650769596, 0.056128136959905282, 0.21887443575107401, 0.096022257347609413]]], [[[0.037364414157828554, 0.10459884842691844, 0.0085913693689205848, 0.0097790539601828163, -0.069112076995279825], [0.11594431195368531, -0.010559974726947335, 0.075904078010236853, 0.044351545756490682, 0.077543691634060205], [0.097261614902197202, -0.066548745051386637, -0.099577988637946202, -0.032890113105793699, 0.1127281792801714], [-0.11179950499977977, -0.077624584627462273, -0.016701514972317543, -0.065128380590387683, -0.028572428297636714]], [[-0.071051212895153079, -0.19890249093097878, -0.016337127929093392, -0.018595598525999169, 0.13142175534924042], [-0.22047673376613944, 0.020080577453254922, -0.14433725050619306, -0.084337763371924834, -0.1474550977755221], [-0.18495019559926684, 0.12654738898296505, 0.18935495256262369, 0.062542996621179237, -0.214361018253522], [0.21259507502937181, 0.1476089218197493, 0.031759173071929249, 0.1238464603057252, 0.054332597784931061]], [[-0.092468492151282541, -0.25885854262139452, -0.021261700175549361, -0.02420095148674169, 0.17103679244648071], [-0.28693600425764765, 0.02613355413607853, -0.18784546205093827, -0.10976006591078974, -0.19190313572046855], [-0.24070054561046958, 0.16469312441160111, 0.2464330478169485, 0.081395606866237188, -0.27897680174955763], [0.27667854250093776, 0.19210332762215063, 0.041332480140296786, 0.16117804293704455, 0.070710311437588644]], [[0.03950402758755496, 0.11058853422482251, 0.0090833403979279047, 0.010339035848159039, -0.073069669571755536], [0.12258367757865166, -0.011164674793912362, 0.080250603664183892, 0.046891266104543569, 0.081984107140375212], [0.10283114575480438, -0.070359552522953867, -0.10528016292856741, -0.034773512840341185, 0.11918337821029902], [-0.11820152488222384, -0.082069632341693691, -0.017657900520893319, -0.068857853162308885, -0.030208582715776026]], [[-0.30131762773962623, -0.84351588490495, -0.06928333002474367, -0.078861167965153675, 0.55734012047966086], [-0.93500904042591215, 0.085158742761861514, -0.61211281475468116, -0.35766391244572121, -0.62533514136344748], [-0.78434627527312217, 0.53666865759573723, 0.80302619451569457, 0.26523554779325603, -0.90907320041552964], [0.90158409781837889, 0.62598748625957545, 0.13468593003651244, 0.52521442073533664, 0.23041646731133436]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.464030061953)+(1.-msk_arg0)*(-0.96738565521)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.67402898847849047, 0.20486008013328139, 0.86910684630548163, 0.11617789766533293, -0.67091974768759255], [0.63735266421821568, 0.52482421330068418, 0.96549957417820953, 0.93314176044328634, -0.66620391933821188], [-0.56446554507968827, 0.97270940295746322, 0.4273768968805165, 0.0795646430337138, -0.56522717749374385], [0.64133192382437154, -0.69241677052592632, 0.75734947668207875, -0.69971855592277477, -0.31147593247856209]])+(1.-msk_arg1)*numpy.array([[0.95121411016144797, 0.1130560179342579, 0.52012252957284932, -0.68225588101818579, 0.17730561916900478], [0.59947361921565534, 0.7649078291107767, 0.45338418670272795, -0.85303367434739097, -0.50029340792348398], [0.35754933425934743, 0.33821264868230383, -0.74573645227047036, -0.75507994872497042, -0.53190481570081594], [-0.93781246725424849, -0.38341219731292941, 0.42257095557524416, -0.18386027017029849, -0.38407080653131875]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.31276971328169517, 0.095061235675913722, 0.4032917037347844, 0.053910037051197068, -0.31132693208486845], [0.29575079626299683, 0.2435342122122757, 0.44802082722137121, 0.43300582890929579, -0.30913864596374629], [-0.26192898185358071, 0.4513664045164778, 0.19831572793668567, 0.036920386236191136, -0.26228240218986021], [0.29759729234456783, -0.32130219692434248, 0.35143292458474851, -0.32469044485440851, -0.14453419624485092]])+(1.-msk_ref)*numpy.array([[-0.92019088520370329, -0.10936876998478624, -0.5031590740604085, 0.66000455247977807, -0.17152291257225483], [-0.57992217990615635, -0.73996086143972717, -0.43859735851535431, 0.82521253997490041, 0.48397666622139479], [-0.34588809699244344, -0.32718206474590183, 0.72141474649378612, 0.73045351093337663, 0.51455708864618532], [0.90722632809902926, 0.37090745971314409, -0.40878908073195058, 0.17786378792581542, 0.37154458882340313]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.38164549218084232, -0.84943064650584477])+(1.-msk_arg0)*numpy.array([-0.063913606660673583, -0.89479132834762409])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.22046330966882177, 0.83542118723512249, 0.49831593535934582, 0.6272392679616996, -0.31521766907302196], [0.40162162896491904, 0.06623229899035965, 0.95859730892263073, 0.25588908389287979, -0.8272136207389138], [0.81517961958285157, 0.77730439461900702, -0.079977269227189174, 0.80967752989262998, -0.21695203317986378], [-0.28088513635915779, -0.40055183374451619, 0.96377618778594387, 0.97555828700940062, -0.27852718947653399]])+(1.-msk_arg1)*numpy.array([[0.18744921804733261, 0.84141914602805357, -0.67806681231531796, -0.27641832837831171, -0.7576319052308631], [0.35459885597183805, 0.60938573262227713, 0.59102751490644856, 0.58222541360655056, 0.7558014619438469], [0.06759826978510608, -0.45978302887178413, 0.20446940512042167, 0.11211057267987212, -0.37033917552328299], [0.53135465885908939, 0.33701500564107301, 0.54425303922014434, -0.18130551567932396, -0.078814758820267894]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.08413882832637494, 0.31883473018065195, 0.19018003041177434, 0.23938303913639408, -0.12030140245747134], [0.15327708425678815, 0.025277258346444516, 0.36584434176700836, 0.097658915366002963, -0.31570234942559938], [0.31110962713148921, 0.2966547182587026, -0.030522964277490346, 0.30900977940364144, -0.082798765482563541], [-0.10719854611207379, -0.15286880173336481, 0.36782083753974248, 0.37231742259680212, -0.10629864631351854]], [[-0.18726829166280554, -0.70963235917781053, -0.42328482713645388, -0.53279625689855925, 0.26775554843076249], [-0.34114971994240167, -0.056259744550949606, -0.8142619318569132, -0.21736002996491724, 0.7026606006626962], [-0.69243855128065024, -0.66026617445305746, 0.067935143505423304, -0.68776490767795218, 0.18428570580472917], [0.23859244297144178, 0.34024100309670602, -0.81866103027795278, -0.82866910643852965, 0.2365895306265082]]])+(1.-msk_ref)*numpy.array([[[-0.011980555591128053, -0.053778132335996882, 0.043337695531978013, 0.017666892313772321, 0.048422987584502107], [-0.022663691802908902, -0.038948040019446621, -0.037774700113366144, -0.037212126073097061, -0.048305997352241087], [-0.0043204492259873656, 0.029386391656564335, -0.013068377133008548, -0.0071653910447642046, 0.023669712395433262], [-0.033960792663636233, -0.021539844509288229, -0.034785174672592456, 0.011587889414538899, 0.0050373354942944565]], [[-0.16772793481429618, -0.75289455537156558, 0.60672830370006248, 0.24733672322925929, 0.67792245888006519], [-0.31729198136558878, -0.54527306916917739, -0.5288462951531363, -0.52097025123875018, -0.67628459409981101], [-0.060486345615016132, 0.41140986715587774, -0.18295745061415059, -0.10031556825003562, 0.3313762828056423], [-0.47545154102422327, -0.30155810457065774, -0.48699289992102451, 0.16223060321145327, 0.070522762738185127]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.7404773332238368, 0.54735446305705127, -0.60976156515517843, -0.91988988534917771, 0.94982879508028195], [-0.47925259590889246, 0.018461915276561269, 0.074865997893243552, -0.91906868677879183, -0.66380093527647133], [0.32921485725414845, 0.2327164935228383, -0.99951906447055583, -0.29839110112065903, -0.97521805679670415], [0.42585346098043475, 0.84140569875823989, -0.25614418347587686, 0.93757162920473247, -0.24299196943566526]])+(1.-msk_arg0)*numpy.array([[0.74735966835600798, 0.55356556314205907, -0.79664761259132577, -0.083573228323402304, -0.09886577387565576], [-0.11423197215038416, 0.50110483461189959, -0.32401081610725013, -0.95250271200920489, -0.08323616314293858], [0.47358512315705692, -0.86022323801341516, -0.11039972820750443, -0.083964682778386868, -0.78651382635546296], [-0.020880118242885715, 0.52612289356883424, -0.90535873207328499, -0.43573562176556124, 0.029851727443945597]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.94170691718388344, 0.88462212402533158, -0.10385925507555438, -0.97727211719508178, 0.67739494200640826], [0.27883639324142595, -0.36261004110559969, 0.90290987220864816, -0.69147390917696439, -0.98681627347326661], [0.064480872009119894, 0.44687978524198879, -0.86518312423678867, -0.41416292585597159, 0.16212940218186911], [0.78165062167133437, 0.89987109000799292, -0.91765215528596289, 0.6706855143282453, -0.49690520527620263]])+(1.-msk_arg1)*numpy.array([[-0.55087645570753607, 0.069507565325803045, -0.60607347319922145, 0.50137395826357256, 0.765268821355368], [-0.43076688393051321, 0.5987114326134837, -0.58866918847603333, -0.4089799468708637, -0.46204382982823478], [0.44673372515774679, -0.8896304761646654, 0.50531756290906094, 0.55178853719531307, -0.42163121911561885], [0.25546186385324576, -0.33184726578369128, -0.22570142190403009, 0.8324772605226114, -0.93173636328846476]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.69731262671476257, -0.65504263130908369, 0.076905424228960739, 0.72364785117462704, -0.50159560019622074], [-0.20647202887316415, 0.26850451623806032, -0.66858429431453503, 0.51202075626122012, 0.73071508256336892], [-0.04774662414926064, -0.33090435164762877, 0.6406484925851248, 0.30667825885801148, -0.12005314736480535], [-0.57879456784794381, -0.66633414497434584, 0.67950062077325601, -0.49662742108163649, 0.36794704126796574]], [[-0.51544748401229556, 0.48420186770427359, -0.056847826795385394, -0.53491425496794165, 0.37077514475947998], [0.1526223443034255, -0.1984762243484508, 0.49421174829167541, -0.37848133027551756, -0.54013829150292014], [0.035293893076002267, 0.24460164490217914, -0.47356184441264954, -0.22669392590003265, 0.088742251877017689], [0.42783995632312355, 0.4925484572918884, -0.502281002729694, 0.36710270957527896, -0.27198328182420972]], [[0.57421668373950274, -0.53940857091658456, 0.063329381930720949, 0.59590297576338802, -0.4130494000660288], [-0.1700237155651167, 0.22110566620553404, -0.55055973687200743, 0.42163421312371552, 0.60172263543365967], [-0.039317957438851685, -0.2724901172853651, 0.5275554159804714, 0.25254063389918535, -0.09886027803208991], [-0.47662050647483106, -0.54870680428117025, 0.55954901447519156, -0.4089582489436967, 0.30299369570297252]], [[0.8662666680808101, -0.81375494424700834, 0.095539078243902717, 0.89898273584153199, -0.62312875553838776], [-0.25649877781003355, 0.33356130913909071, -0.83057765882665402, 0.63607985503474551, 0.90776230866602603], [-0.059315301959684298, -0.4110801944111182, 0.79587320496022285, 0.3809842863815297, -0.14914119718481031], [-0.71903250075235725, -0.82778231381649214, 0.84413893591643019, -0.61695682088076387, 0.45709807231093563]], [[-0.89446034646753492, 0.84023956616434048, -0.098648511106349479, -0.92824119754096057, 0.64340922155942426], [0.2648468354170353, -0.34441745842734328, 0.8576097959860316, -0.65678182998300838, -0.93730651199872683], [0.061245788966148226, 0.42445928796213339, -0.82177584441762286, -0.39338387283270165, 0.15399517472149118], [0.7424342681558368, 0.85472347314987185, -0.87161244095808987, 0.63703641395219646, -0.47197487239661573]]], [[[0.45131548464573656, -0.4239574493375784, 0.04977481760412325, 0.46836019907512233, -0.32464328441212481], [-0.13363306529482616, 0.17378180350248884, -0.43272190012776102, 0.33139066597632993, 0.47293426074720257], [-0.03090262529683975, -0.21416829713643151, 0.41464125822704678, 0.19848865734569654, -0.077700936868817627], [-0.37460808952978658, -0.43126555586969523, 0.43978717756218783, -0.32142777378030224, 0.23814310954926121]], [[-0.017385713320400557, 0.016331818705527346, -0.0019174407678916509, -0.018042315029801255, 0.012506008028093443], [0.0051478538680451271, -0.0066944758573219808, 0.016669445563086825, -0.012765932727177739, -0.018218518434395363], [0.0011904403959911625, 0.0082502567339454921, -0.015972937538370174, -0.0076462408478456742, 0.0029932192869211952], [0.014430767553167622, 0.016613343823554404, -0.016941616344243293, 0.012382139142744985, -0.0091738218002915383]], [[-0.070501828077941503, 0.066228118073597117, -0.0077755267716802991, -0.073164452267052663, 0.0507138483011456], [0.020875364828972225, -0.027147162573480783, 0.06759724859056146, -0.051767884227675497, -0.073878985050868026], [0.0048274248279892773, 0.033456101060459861, -0.064772797956381301, -0.031006720734592753, 0.01213797948218065], [0.058519053795298634, 0.067369747128729174, -0.068700944324369309, 0.050211540302727384, -0.037201304051349943]], [[0.86549333970669629, -0.81302849382342701, 0.095453789172113335, 0.8981802013760134, -0.62257247978042551], [-0.25626979776253211, 0.33326353429172723, -0.82983619053040902, 0.63551201764907028, 0.9069519365530162], [-0.059262350359773175, -0.41071321737034316, 0.79516271781547765, 0.38064417637890996, -0.14900805675152104], [-0.71839061037929952, -0.82704334096384602, 0.84338536127839792, -0.61640605489521894, 0.45669001446674551]], [[0.6251059323829844, -0.58721299329427368, 0.068941870656270596, 0.64871414541371253, -0.44965539605540483], [-0.1850918586227765, 0.24070088442653678, -0.59935241764245983, 0.45900102763094675, 0.6550495652775965], [-0.042802463147096223, -0.29663921939978083, 0.57430936705379976, 0.27492173754003379, -0.10762164880413988], [-0.51886041372486702, -0.5973352711755634, 0.60913835893729185, -0.44520167168747043, 0.3298461400060903]]], [[[-0.3100239083159364, 0.29123074628486112, -0.034192009834220827, -0.3217325005608383, 0.22300847913732186], [0.091797083398237653, -0.11937661292150091, 0.29725134469253151, -0.22764348430466233, -0.32487457860757218], [0.021228061074105423, 0.14711946470820586, -0.2848311387443126, -0.13634858851563417, 0.053375407996004466], [0.25733099783614471, 0.29625093244411638, -0.30210472331142996, 0.22079963586199841, -0.1635885762238484]], [[-0.21915073169323523, 0.20586615879590053, -0.024169761661077063, -0.22742734033127973, 0.15764097563383772], [0.06488982770169989, -0.084385337282267422, 0.21012201942755063, -0.1609173835061927, -0.22964842291397289], [0.015005762433257314, 0.10399629664775466, -0.20134238292751963, -0.096382543852360975, 0.037730185972718586], [0.18190299183529962, 0.20941484468923452, -0.21355279185182438, 0.15607958115103057, -0.11563803698512404]], [[0.94125401686908639, -0.88419667781575539, 0.10380930546972694, 0.97680211231198755, -0.67706915871133155], [-0.27870229091301407, 0.36243564905349879, -0.90247563077121717, 0.69114135480635752, 0.98634167846631959], [-0.06444986086680117, -0.44666486487587553, 0.86476702693286767, 0.41396374018994891, -0.1620514283919923], [-0.78127469811576045, -0.89943831002888841, 0.91721082376081486, -0.67036295783532129, 0.49666622590821957]], [[0.28099696395144025, -0.26396336966361489, 0.030990677483566067, 0.29160930314435818, -0.20212862263885711], [-0.083202298411822181, 0.10819960944290732, -0.26942027098105203, 0.20632966115552098, 0.29445719444547341], [-0.019240518400021566, -0.13334495118692066, 0.25816294511202731, 0.12358253148951723, -0.048377970841082099], [-0.23323758969215713, -0.2685135254141327, 0.27381923706152445, -0.20012658912608067, 0.1482720913549532]], [[0.91836958984808159, -0.86269946879135684, 0.10128542091513537, 0.95305341509258856, -0.66060777902740553], [-0.27192628558110504, 0.35362385966197596, -0.88053401103787832, 0.67433784203317992, 0.96236104863196403], [-0.062882910701290901, -0.4358052357854208, 0.84374220519150256, 0.40389916375049806, -0.15811152054541372], [-0.76227980036025444, -0.87757053576512689, 0.89491095169328416, -0.65406462400488941, 0.48459092870162573]]], [[[-0.40102914991197242, 0.37671939317605085, -0.044228823228774618, -0.41617471342720264, 0.28847098050406983], [0.11874344310916275, -0.15441874099107733, 0.38450729403345485, -0.29446655740068106, -0.42023912541040576], [0.027459402512120146, 0.19030530318749433, -0.3684412278381019, -0.17637271538554869, 0.069043367045837814], [0.33286862251624616, 0.38321321811614012, -0.39078534630468265, 0.28561374750612623, -0.21160880144606428]], [[-0.79235756667857338, 0.74432609640253244, -0.087387769089357109, -0.82228232864547235, 0.56996396451419928], [0.23461453029452936, -0.3051021550132112, 0.75971351194143066, -0.58181008772413545, -0.83031283612777629], [0.054254573169374154, 0.37600719796246779, -0.72797001120229221, -0.34847904602960089, 0.13641660293209129], [0.65768528751218169, 0.75715666328051423, -0.77211775293539042, 0.56431861383038673, -0.4180988714620299]], [[0.24121274937565101, -0.22659081164316444, 0.026602944087740697, 0.25032256849267565, -0.17351077431092041], [-0.071422320270183554, 0.092880452899147981, -0.23127511196919251, 0.17711701986100617, 0.25276724860951749], [-0.016516400310588538, -0.11446565770268442, 0.22161162491474037, 0.10608542446935797, -0.04152850333930691], [-0.20021526025141548, -0.23049674558364464, 0.23505126203060153, -0.1717921934367069, 0.12727937807038589]], [[-0.88291768857745967, 0.82939660605298104, -0.097375490989177402, -0.91626261109495111, 0.63510627939199349], [0.26142909149293519, -0.33997288700537209, 0.84654267991169907, -0.64830631957961171, -0.92521094124607339], [0.060455436222132369, 0.41898180830799242, -0.81117115135112638, -0.38830740915098211, 0.15200792774564434], [0.73285344682928488, 0.84369360393303239, -0.86036462627469434, 0.62881571035274686, -0.4658842228711213]], [[0.22882721843770076, -0.21495607212327664, 0.025236964934930071, 0.23746927643179519, -0.16460153104389538], [-0.067755004344071709, 0.088111328025397201, -0.21939984807088425, 0.16802260700428889, 0.23978842976243309], [-0.015668334080425103, -0.108588199116938, 0.21023255128079912, 0.10063826502097994, -0.039396142739599418], [-0.18993482397052963, -0.21866144839926099, 0.22298210446981903, -0.16297119399859242, 0.120743974452898]]]])+(1.-msk_ref)*numpy.array([[[[-0.41170284524271727, 0.05194715097012572, -0.45295486992954404, 0.37470667517020256, 0.57193105253134091], [-0.32193779551305934, 0.44745277771896358, -0.4399476094708486, -0.30565511745766644, -0.34531292342636932], [0.33387076867733767, -0.66487393762582181, 0.37765396623018199, 0.41238449816093592, -0.31511016808678821], [0.19092189384696942, -0.24800926250094754, -0.16868013982167537, 0.62215992933809694, -0.69634217946249999]], [[-0.30494623542544375, 0.038476994542211623, -0.33550140349699065, 0.27754335755093779, 0.42362646604864407], [-0.23845771268594454, 0.33142603135427207, -0.32586699082311432, -0.22639721460337905, -0.2557715528551805], [0.24729640614149795, -0.49246879552643114, 0.27972640127732717, 0.30545113232785648, -0.23340052324801044], [0.141414890525242, -0.18369921856070162, -0.12494053471826788, 0.46083074352415798, -0.51577716464371315]], [[0.43885441327217983, -0.05537303597383661, 0.48282698547909264, -0.39941836686613807, -0.60964957952333165], [0.34316940966664811, -0.47696203342266419, 0.46896190360550516, 0.32581289827240084, 0.36808611394521601], [-0.35588935561094848, 0.70872199492506505, -0.40256003009197044, -0.43958102081190609, 0.33589150410242791], [-0.20351308394681855, 0.26436533203153678, 0.17980449891831313, -0.66319102213190551, 0.7422655493782796]], [[0.046038523810832531, -0.0058089716271771388, 0.050651516756435984, -0.041901440289369526, -0.063955985935913146], [0.036000579144885317, -0.050036247257637965, 0.049196984495459478, 0.034179774479531633, 0.038614494485654308], [-0.037334979612372421, 0.074349290907966686, -0.042231020060824159, -0.046114749405260065, 0.035237082143424081], [-0.021349772675729222, 0.027733547311837199, 0.018862596465702059, -0.06957281216769666, 0.077868215826323389]], [[0.054462827103403957, -0.0068719192361482145, 0.05991992295334754, -0.049568724384828837, -0.075658894246209418], [0.042588101339794973, -0.059192069113534591, 0.058199234875437295, 0.040434118947012515, 0.045680320800440231], [-0.044166675454075141, 0.087954005489387768, -0.049958611909964673, -0.05455300074553069, 0.041684896768001842], [-0.025256434865568552, 0.032808336740225062, 0.022314145741377817, -0.082303508595453864, 0.092116836604603208]]], [[[0.062927703946685593, -0.0079399862665381418, 0.06923296811158007, -0.057272936037292291, -0.087418166688623705], [0.049207350688458155, -0.068391987696420076, 0.067244842343783487, 0.0467185859010181, 0.052780177901195754], [-0.051031274450857103, 0.10162424377737506, -0.05772342177332785, -0.063031892813796217, 0.048163765679747894], [-0.029181912517169201, 0.037907567623183758, 0.025782318541243271, -0.095095519239847048, 0.10643408230266813]], [[-0.27604685522891431, 0.03483057702686234, -0.30370634755015541, 0.25124091443438096, 0.38347990615892502], [-0.21585936812828316, 0.30001719342003319, -0.29498497633240384, -0.20494182863630764, -0.23153239692952626], [0.22386042946073051, -0.44579813262420015, 0.25321707378803315, 0.27650390367199934, -0.21128144232214577], [0.12801317503582832, -0.1662902692369477, -0.11310007369488957, 0.41715837995235044, -0.46689759622755894]], [[0.1784899299880682, -0.022521202966841444, 0.19637436067223532, -0.16245058539190252, -0.2479553753487862], [0.13957312961430268, -0.19398897989383573, 0.19073518417531221, 0.13251392635712836, 0.14970719837996574], [-0.14474655887099352, 0.28824989661599476, -0.16372835595149154, -0.17878545425527914, 0.13661307540194648], [-0.082772406991369382, 0.10752210340953336, 0.073129701907691566, -0.26973163657265919, 0.30189265946589677]], [[0.52471131804344673, -0.066206144477984369, 0.5772866268990966, -0.47756005497684278, -0.72892062775707578], [0.41030662518756822, -0.57027426327525954, 0.56070899849967948, 0.38955450855187818, 0.4400980009785132], [-0.42551508475872857, 0.84737544123288411, -0.48131634909676252, -0.52558007813412777, 0.40160487967537428], [-0.24332811813514285, 0.31608542063180539, 0.21498121646792243, -0.79293684833378075, 0.88748141290985649]], [[0.045852842538876254, -0.0057855430471270026, 0.050447230491817811, -0.041732444585647607, -0.063698040462539726], [0.035855382627415486, -0.049834442480558405, 0.048998564609212369, 0.034041921579933562, 0.038458755598771099], [-0.037184401228682898, 0.074049427450972219, -0.042060695105290723, -0.045928760702392511, 0.035094964940463734], [-0.021263665376487929, 0.027621693153309427, 0.018786520375197058, -0.069292213069646669, 0.077554159940886935]]], [[[-0.26088689412057647, 0.032917748885167616, -0.28702738044727855, 0.23744324777199513, 0.36241992900983777], [-0.20400478777821374, 0.28354082754979465, -0.27878497012318698, -0.19368681850760458, -0.2188170840531628], [0.21156644624724233, -0.42131575861871423, 0.23931088026371147, 0.26131884234429464, -0.1996782728317304], [0.12098293825487071, -0.15715792823550204, -0.10688883568914295, 0.39424884595005028, -0.4412564803578759]], [[0.47387672847409035, -0.059792022910991276, 0.52135848558947107, -0.43129352983309327, -0.65830202345702438], [0.37055568372365505, -0.51502548719842156, 0.50638691542958281, 0.35181405417980888, 0.39746083939896348], [-0.384290731584992, 0.76528080884178484, -0.43468591019068004, -0.47466132216483797, 0.36269697255518141], [-0.2197542317127813, 0.28546272949834528, 0.1941536079745167, -0.71611628461929822, 0.80150127140284688]], [[0.060816610986025338, -0.0076736163203340157, 0.066910346714972271, -0.055351548722619083, -0.084485469883309891], [0.047556546906722265, -0.066097579435254211, 0.064988918411886282, 0.045151274976862957, 0.051009513232991549], [-0.049319281838541229, 0.098214962773691808, -0.05578692160363885, -0.060917304534379012, 0.046547971994163072], [-0.028202920336780832, 0.036635847948923002, 0.024917375634252208, -0.091905263300624152, 0.10286344126809512]], [[0.04625416685356535, -0.0058361806732790547, 0.050888766917567781, -0.042097705358945046, -0.064255553825293479], [0.03616920476065974, -0.050270615515184705, 0.049427421671800573, 0.034339871501733586, 0.038795363601238697], [-0.037509855519277276, 0.074697540721151395, -0.04242882887200685, -0.046330749486354586, 0.035402131562507461], [-0.021449774360413235, 0.027863450402402672, 0.018950948292802743, -0.06989868910000159, 0.07823294817660377]], [[0.43327194902766991, -0.054668661165049655, 0.47668516645846482, -0.39433755034886664, -0.60189450887474572], [0.33880411014740752, -0.47089481974759201, 0.4629964558858502, 0.32166838291605698, 0.3634038605421373], [-0.35136225153584916, 0.69970666985070351, -0.39743924992822288, -0.43398931372856936, 0.33161878345754398], [-0.20092428802711465, 0.26100246277712935, 0.17751728895560739, -0.65475487552755274, 0.73282353224453411]]], [[[0.011502365532395148, -0.0014513261827778707, 0.012654885784276169, -0.010468747532447042, -0.01597890347749387], [0.0089944634715885419, -0.012501165506337042, 0.012291482261323153, 0.0085395496496327508, 0.009647529800209307], [-0.0093278530043785623, 0.018575589534792936, -0.010551090463748032, -0.011521409901707079, 0.0088037097100261766], [-0.0053340739238037435, 0.0069290101481417968, 0.0047126723769435842, -0.017382223634225702, 0.019454765436659464]], [[-0.28982871487579265, 0.036569521394136266, -0.31886912943488771, 0.26378431768169075, 0.40262544664949751], [-0.22663631942715176, 0.31499579133934813, -0.3097123367958281, -0.21517371305932689, -0.24309183670485693], [0.23503684013477805, -0.46805496032677357, 0.26585913836886654, 0.29030858182731245, -0.22182983702006456], [0.13440433500695725, -0.17459244369702165, -0.11874668517474857, 0.43798534513641257, -0.49020783149662961]], [[0.49874080946839999, -0.062929281212870072, 0.54871391123889923, -0.45392329114807217, -0.69284280979751323], [0.38999855985448939, -0.54204862350872363, 0.53295679008927122, 0.37027356614240464, 0.41831541593557531], [-0.40445427898319303, 0.80543471991419424, -0.45749366804970981, -0.49956657040772129, 0.38172750594103011], [-0.23128462915125272, 0.30044081979190917, 0.20434075316217024, -0.75369055706659316, 0.84355565249341813]], [[0.24003649494373189, -0.030286922194649155, 0.26408780168007595, -0.21846649344103833, -0.33345488569107951], [0.18770047600547551, -0.26087989834798625, 0.25650413485483275, 0.17820713143942199, 0.20132895547314705], [-0.19465779749525614, 0.38764368867320281, -0.22018486246323776, -0.24043392133790925, 0.18371974141711578], [-0.11131383408348319, 0.14459767468745818, 0.098346149406723815, -0.36273999671951118, 0.40599072357908206]], [[-0.016444613811068137, 0.002074920895398116, -0.018092340132948626, 0.014966878749576324, 0.022844596276449941], [-0.012859135610971429, 0.017872570503951916, -0.017572792169035189, -0.012208757904028374, -0.013792806475189245], [0.013335773403427559, -0.026557006500295133, 0.015084602160600121, 0.016471841019047924, -0.012586420234897958], [0.0076259779320694302, -0.0099062141311933253, -0.0067375773303900788, 0.024850884284403487, -0.027813939966500327]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.993549884155)+(1.-msk_arg0)*(0.929529899002)
      arg1=numpy.array([[[-0.96404073390044864, 0.68299636489748128], [0.36400286263553872, -0.2902214898897415]], [[0.52744858859312793, -0.045387940631246293], [0.40639487510984895, 0.6685781294180575]], [[-0.20069163885522623, 0.19403923621031405], [0.20120512920242328, -0.87559023882949072]], [[-0.39214221655530723, -0.65080180395854148], [0.5640793644347486, -0.98168633861320709]], [[-0.54996581116145982, 0.084797595420068594], [0.079502859459630315, 0.053027222376161642]], [[0.71554985948411787, -0.18034726142179469], [0.3513773812994283, 0.62946438984268482]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.95782255948772366, -0.6785909592223428], [-0.36165500200371536, 0.28834952765931393]], [[-0.52404648409454735, 0.04509518315621968], [-0.40377358108667383, -0.66426572303203835]], [[0.19939715453553536, -0.19278766065832884], [-0.19990733281050782, 0.86994258035649974]], [[0.38961285383090466, 0.64660405693103029], [-0.5604409871885061, 0.97535434800593401]], [[0.54641846796881122, -0.084250641106252092], [-0.078990056806126058, -0.052685190648909572]], [[-0.71093447999774384, 0.17918400069333898], [-0.34911095648481871, -0.62540427160804857]]])+(1.-msk_ref)*numpy.array([[[-0.89610468601611992, 0.63486554208176271], [0.33835154414198393, -0.26976955218536774]], [[0.49027923328362022, -0.042189447870862755], [0.37775618721571325, 0.62146336111278955]], [[-0.18654887879560722, 0.18036527163696259], [0.18702618342617572, -0.8138873062661518]], [[-0.3645079149490027, -0.60493973510378218], [0.52432863465204005, -0.91250680318259625]], [[-0.51120966490336317, 0.078821900306413159], [0.073900284923865675, 0.049290388659660331]], [[0.66512498861703517, -0.16763817169465478], [0.32661578175077993, 0.58510597071571013]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.56879413112408872, -0.28005683913711432])+(1.-msk_arg0)*numpy.array([0.78833884726990999, 0.66356112587536176])
      arg1=numpy.array([[[-0.85252097033383101, 0.58175331209550607], [-0.70869360301507678, -0.2927221987776345]], [[0.36754488753156189, -0.66559884167391159], [-0.43601741806713212, -0.82616620732624724]], [[0.61801257801317044, 0.21090349649113005], [0.41668879754804911, 0.49086430937885894]], [[-0.097196959600152955, 0.49753981495824773], [-0.29445462587602544, -0.60285511181456664]], [[0.77183435706312098, -0.44132925471347328], [-0.47567793664135172, 0.59066187435301054]], [[-0.97049765916666608, -0.62489178685474744], [-0.052994325480985038, 0.76185763449074395]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.48490892458609641, 0.33089786968192419], [-0.40310076216016044, -0.1664986687144574]], [[0.20905737495261564, -0.37858871482711243], [-0.24800414846446295, -0.46991849006021652]], [[0.3515219273347594, 0.11996067103770461], [0.23701014255048389, 0.27920073835297393]], [[-0.055285060183672151, 0.2829977267488164], [-0.1674840630806225, -0.3429004495182818]], [[0.43901485249743755, -0.25102548997439167], [-0.27056281866681697, 0.33596500761074627]], [[-0.5520133728036658, -0.35543478095062531], [-0.030142861316464039, 0.43334015125041625]]], [[[0.23875432824979831, -0.16292399374301461], [0.1984744903770953, 0.081978853734930385]], [[-0.1029334594430954, 0.18640550773252029], [0.12210965991260675, 0.23137349662568665]], [[-0.17307864914534779, -0.059064966590271363], [-0.11669654754515159, -0.13746990692986583]], [[0.027220673279356636, -0.1393394279220716], [0.082464031792141235, 0.16883369707243914]], [[-0.21615749037652454, 0.12359727609379374], [0.13321685938304151, -0.16541889753010749]], [[0.271794506816185, 0.17500521862928395], [0.014841423286408105, -0.21336344098795673]]]])+(1.-msk_ref)*numpy.array([[[[-0.67207539902639746, 0.45861873545282344], [-0.55869069806846483, -0.23076428075467384]], [[0.2897499129565802, -0.52471742358939877], [-0.34372946874864535, -0.6512989155369272]], [[0.48720332334920813, 0.16626341930901098], [0.32849196632931393, 0.38696740382167011]], [[-0.076624139089324597, 0.39222996419506934], [-0.23213002033639851, -0.47525410391866818]], [[0.60846700733045289, -0.34791699592730801], [-0.37499539624357248, 0.46564170115373676]], [[-0.76508100590559558, -0.49262647091750589], [-0.041777485461526161, 0.60060196935821353]]], [[[-0.56569977490707279, 0.38602888275581471], [-0.47026152511735103, -0.19423907178959859]], [[0.24388849938017643, -0.44166551676247745], [-0.28932420883389448, -0.54821177869358217]], [[0.41008912207155424, 0.13994736158270465], [0.27649848764063417, 0.32571847378346752]], [[-0.06449612394393954, 0.330148079781514], [-0.19538864306550388, -0.40003121673539094]], [[0.51215927496209057, -0.29284893713940663], [-0.31564138719180435, 0.39194025835733515]], [[-0.64398451937603607, -0.41465389763560279], [-0.035164974281167806, 0.50553910969941784]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.503764813126)+(1.-msk_arg0)*(0.504959872725)
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[0.53122902975052488, -0.87578120213994959], [0.60096927110682286, 0.072479313822353397]], [[0.74249497746574855, 0.42379767715599459], [0.76536872628025643, -0.21428868343107266]], [[-0.3032764271638293, 0.24399051048458853], [-0.98010693463003928, 0.69059608075558088]], [[0.40867050690292572, -0.4077348015450375], [-0.74152107543587253, -0.29741642390738909]], [[-0.61647105895791832, 0.92930585828763923], [-0.83480343439680182, -0.10251756126917888]], [[-0.38593820060616069, -0.67881057166222747], [-0.13714869747101743, 0.46676412094242337]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.26761449289958278, -0.44118775363563056], [0.3027471725538271, 0.036512527983246314]], [[0.37404284357031059, 0.2134943576358847], [0.38556583336735101, -0.10795109856375291]], [[-0.15277999265582351, 0.12291383391887947], [-0.49374338676777357, 0.34789800556764755]], [[0.2058738215402165, -0.20540244610545857], [-0.37355222599622673, -0.1498279292104229]], [[-0.31055642781375903, 0.46815159203752532], [-0.4205445961261674, -0.051644740094940397]], [[-0.19442208550669496, -0.34196088078163506], [-0.069090687952013896, 0.23513934016066024]]])+(1.-msk_ref)*numpy.array([[[0.26824934325079747, -0.44223436436777913], [0.30346536664990381, 0.036599145082950986]], [[0.37493016932026185, 0.21400082111795909], [0.38648049461038575, -0.10820718631182165]], [[-0.15314242606122402, 0.12320541712047321], [-0.49491467296794611, 0.34872330904291332]], [[0.20636220715227585, -0.20588971349384824], [-0.37443838787520878, -0.15018335956268225]], [[-0.3112931474702072, 0.46926216792378067], [-0.42154223598363327, -0.051767254690590361]], [[-0.19488330465790935, -0.34277209987113094], [-0.069254588819402496, 0.23569715110381195]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.35055372625381676, -0.26852256401346986])+(1.-msk_arg0)*numpy.array([-0.62353202284885278, 0.91218379712365372])
      arg1=Symbol(shape=(6, 2, 2))
      res=outer(arg0,arg1)
      s1=numpy.array([[[0.29929520380858543, 0.67488170329686215], [-0.64306223942216434, -0.32374474916343798]], [[0.73815904667903953, 0.4505454275140357], [0.51889090097144197, -0.59558077853420865]], [[0.86008774297327095, -0.35853890388738141], [0.80231900123383459, 0.76486539049987434]], [[0.84503397296602523, -0.75666558985533827], [-0.82138727789061727, 0.29567457475611603]], [[0.023491717672938339, -0.77467215383340027], [-0.58054259240925266, -0.046426845453944043]], [[-0.0068639070766811638, -0.1806479263426819], [0.2669894885656019, 0.48758983052340943]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.10491904894499515, 0.2365822958712378], [-0.22542786424256378, -0.11348992817435041]], [[0.25876440438130238, 0.15794037846166412], [0.18189913885473921, -0.20878306120031603]], [[0.30150696320451514, -0.12568714876468062], [0.28125591552676132, 0.26812641272231158]], [[0.29622980803430721, -0.26525194205183106], [-0.28794037096203517, 0.10364982393926918]], [[0.008235109166351175, -0.27156421015136845], [-0.20351136901811429, -0.016275103672090157]], [[-0.0024061682023905242, -0.063326803719452165], [0.093594160087272546, 0.17092643197344817]]], [[[-0.080367515523615388, -0.18122096537505122], [0.17267672134988341, 0.08693277013126402]], [[-0.19821235986399427, -0.12098161340061379], [-0.13933391517211108, 0.15992687772914427]], [[-0.23095296601974097, 0.09627578577041869], [-0.21544075536803556, -0.20538361578219014]], [[-0.22691068909932627, 0.20318178428872], [0.22056101790723304, -0.079395294927104648]], [[-0.0063080562626179467, 0.20801695301718179], [0.15588878543275927, 0.012466655580350162]], [[0.0018431139273806265, 0.04850804436525339], [-0.071692702034280409, -0.13092887147903912]]]])+(1.-msk_ref)*numpy.array([[[[-0.18662014385972694, -0.42081035364037173], [0.40096989896461543, 0.20186521833257293]], [[-0.46026580355996227, -0.28092950180312787], [-0.32354509312058694, 0.3713636876093297]], [[-0.53629225020362781, 0.22356048801090933], [-0.50027158980940412, -0.47691806414546434]], [[-0.5269057425395085, 0.47180522586261947], [0.51216127092544939, -0.18436256570265536]], [[-0.014647838240801387, 0.48303289512441772], [0.36198689699485836, 0.028948624860388794]], [[0.0042798658641695622, 0.11263976693590301], [-0.16647649588469041, -0.30402787334679077]]], [[[0.27301223547101328, 0.61561615472261078], [-0.58659095534295003, -0.29531471459074965]], [[0.67333672208086259, 0.41098023884645296], [0.47332387234104373, -0.54327913605719635]], [[0.78455810324487141, -0.32705337876454432], [0.73186239304993661, 0.69769781619464155]], [[0.77082629815863579, -0.69021809090705166], [-0.74925616605532497, 0.26970955631395555]], [[0.021428764227857736, -0.70664338680971028], [-0.52956154633588171, -0.042349816174651715]], [[-0.0062611448203109422, -0.16478411139378168], [0.24354348547187307, 0.44477154304572242]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.564569713)+(1.-msk_arg0)*(0.765058255003)
      arg1=Data(numpy.array([[[-0.061916195813281893, -0.77770650262277941], [0.34046883403604578, 0.80949355749984009]], [[-0.64459103147321128, 0.51540888259634965], [-0.11655602739516113, -0.67199689966441034]], [[-0.88499197249500594, 0.25313378268983855], [0.99236121472568417, -0.6940495946852514]], [[0.22304924598053755, 0.25241118750820246], [-0.11946085671392392, 0.16792236720876286]], [[-0.24099441005664635, 0.6916611269224997], [-0.24961596509577566, 0.96472588150588923]], [[0.31899818052162288, 0.50443582458100011], [-0.98115542257236155, -0.52015867465733923]]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.034956008900342971, -0.43906953698380813], [0.19221839191710138, 0.45701554543285866]], [[-0.3639165736410655, 0.29098424492496039], [-0.065804002934881048, -0.37938909678028065]], [[-0.49963966391861803, 0.14291166704375177], [0.56025708618979631, -0.39183938047906863]], [[0.12592684878805024, 0.14250371168944045], [-0.067443981589688326, 0.094803882661295552]], [[-0.13605814492023302, 0.39049092391974266], [-0.14092561377428611, 0.5446550140452433]], [[0.18009671122454585, 0.28478918871050452], [-0.55393063532985976, -0.29366583366564192]]])+(1.-msk_ref)*numpy.array([[[-0.04736949672535657, -0.5949907798013595], [0.26047849205065005, 0.61930972853729993]], [[-0.49314968972971907, 0.39431782033240143], [-0.089172150929067273, -0.51411677542493028]], [[-0.67707041416902203, 0.19366209006709081], [0.75921413927105907, -0.53098837179569647]], [[0.17064566690968808, 0.19310926265835446], [-0.091394514578762576, 0.12847039323277168]], [[-0.1843747628235051, 0.52916105481699383], [-0.19097075467715691, 0.73807149946148609]], [[0.24405219133912365, 0.38592279171512733], [-0.75064105548030768, -0.39795168795821101]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.25980353350036678, 0.38094246378925556])+(1.-msk_arg0)*numpy.array([-0.20352129609671588, 0.1685951958778884])
      arg1=Data(numpy.array([[[-0.81679769816057579, -0.55786887196221646], [-0.50302600787280327, 0.8965691928576569]], [[0.64790141094910392, -0.79264082526829949], [0.8095029626250736, -0.64555155689440324]], [[0.6861970523964751, -0.33083887691935732], [-0.46426092136451613, -0.99503807327607685]], [[0.23706478599730141, 0.31436258162329533], [-0.55669207740676829, -0.94500743138959731]], [[-0.20834312393451926, 0.029866861841089998], [0.10057659973846467, -0.74860703974958365]], [[-0.62776180242682456, -0.2236049291303901], [0.55635192604811978, 0.57047326936893739]]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.21220692813708361, -0.14493630416564754], [-0.1306879342879376, 0.23293184433199107]], [[0.16832707592445043, -0.20593088720135103], [0.21031173006900947, -0.16771657553782901]], [[0.17827641889014056, -0.085953109242941972], [-0.12061662783663721, -0.25851440740452164]], [[0.061590269070607176, 0.081672509506029597], [-0.14463056878193811, -0.24551626985912281]], [[-0.054128279778692945, 0.0077595162408824513], [0.026130155999505186, -0.19449075413019137]], [[-0.16309473446704814, -0.058093350696174439], [0.14454219625703627, 0.14821097114955648]]], [[[-0.31115292755468243, -0.21251594255661949], [-0.19162396678913915, 0.34154127728474004]], [[0.24681315977948659, -0.30195054887765482], [0.30837405302709719, -0.24591800058634375]], [[0.26140159578483813, -0.12603057689093025], [-0.1768566992256686, -0.37905225519790253]], [[0.090308043655484616, 0.11975405636672908], [-0.21206765153929327, -0.35999345921270909]], [[-0.079366742945165991, 0.011377555935398125], [0.038313897703916529, -0.28517621013218758]], [[-0.23914112768925841, -0.085180612618352691], [0.21193807344266846, 0.21731749275931467]]]])+(1.-msk_ref)*numpy.array([[[[0.1662357261784545, 0.11353819587376314], [0.10237650509262973, -0.18247092417077676]], [[-0.13186173489925257, 0.16131928809777482], [-0.16475109214758632, 0.13138348955640178]], [[-0.13965571348147668, 0.067332757029809454], [0.094486984443161809, 0.20251143833872612]], [[-0.048247732505061362, -0.063979480056282706], [0.11329869312059876, 0.19232913725743914]], [[0.042402262615992066, -0.0060785424322401822], [-0.020469479935772944, 0.15235747499696095]], [[0.1277628956699178, 0.045508364990231288], [-0.11322946507521756, -0.11610345917049707]]], [[[-0.13770816791399065, -0.094054011742646534], [-0.084807768328987496, 0.15115725868791696]], [[0.10923306528852444, -0.13363543519692006], [0.13647831054750525, -0.10883689118388773]], [[0.11568952645961338, -0.055777845258239658], [-0.078272160975899549, -0.16775863886993686]], [[0.039967984030964729, 0.053000021025458158], [-0.093855609834062714, -0.15932371300118933]], [[-0.035125649789551454, 0.0050354094223563992], [0.016956731533638431, -0.12621155050214725]], [[-0.10583762404480676, -0.03769871682599947], [0.093798261949123232, 0.096179052592355388]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.989567049022)+(1.-msk_arg0)*(0.111704893141)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.98564278032184371, -0.98926209581241942], [0.71884820439034591, -0.35985766245982154]], [[-0.31829775311751174, -0.31295355476148701], [-0.23289595230410742, 0.96397526515271847]], [[-0.98050373305327909, 0.16502651443126082], [-0.15672073533714692, 0.11515718288678323]], [[0.68514762048745004, -0.080742832957046984], [0.37115655840142603, -0.018109035859751987]], [[0.120811315682819, -0.54008732129415926], [0.96170319243592362, -0.90135016688370628]], [[-0.039383107820948116, -0.0037324704980368217], [0.039849656968649283, 0.0067319961117462412]]])+(1.-msk_arg1)*numpy.array([[[-0.93341969370184841, -0.6562498098736802], [0.0040611019140541771, 0.75590407496308654]], [[0.16399251424059869, 0.35107616902065164], [-0.85195584055986484, 0.78978808695479863]], [[0.82449515840869503, -0.64524111675619156], [-0.58553323716380201, 0.865742756902522]], [[-0.60338001261042762, -0.92648566542986877], [0.0044011356577715599, 0.71778707833593858]], [[-0.44437597093742087, 0.64750491748878591], [-0.54543489906039855, -0.19085528281537423]], [[0.31818415050754689, -0.38866553388085512], [0.99116002770573752, 0.58200948995953605]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.97535961751305766, -0.97894117286254678], [0.7113484963134139, -0.35610328510836853]], [[-0.31497696826287164, -0.30968852566631128], [-0.2304661602507751, 0.95391815846750416]], [[-0.97027418567271895, 0.16330480089615126], [-0.15508567558815925, 0.11395575364297619]], [[0.67799950895030248, -0.079900446938992037], [0.36728430022251024, -0.017920105176370764]], [[0.11955089714872857, -0.53445261674733002], [0.95166979017398179, -0.89194642477871666]], [[-0.03897222578769613, -0.0036935298163044698], [0.039433907451010559, 0.0066617615263292037]]])+(1.-msk_ref)*numpy.array([[[-0.10426754714068533, -0.073306314885751844], [0.00045364495534419943, 0.084438183918610507]], [[0.018318766279172703, 0.039216925944809342], [-0.095167636130604596, 0.088223193857333626]], [[0.092100143565328751, -0.072076589997440529], [-0.06540692768789598, 0.096707702147405247]], [[-0.06740049983207301, -0.10349298225352706], [0.00049162838835048949, 0.080180328883518462]], [[-0.049638970348000057, 0.072329467616367535], [-0.060927747114922944, -0.021319468972289861]], [[0.035542726531610641, -0.043415841929757053], [0.11071742498051637, 0.065013307882987495]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank1_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.46339937678895926, -0.07398819985425642])+(1.-msk_arg0)*numpy.array([0.19125387973739794, 0.363847965390933])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.53932222036196209, -0.79546889428276879], [0.40819649651641421, -0.5403822480074254]], [[0.0075571699341703269, -0.69508479063866702], [0.87061594540981035, 0.38284731713654407]], [[0.40134777280270062, -0.10155047091049663], [0.12057619184612833, 0.50213788884001476]], [[0.071581753019605454, -0.64541201057082165], [0.45596858567208742, 0.050327024187448721]], [[0.27646171390120755, 0.27886795358673111], [0.27909761715969683, -0.99945859082854316]], [[0.14330865380306212, 0.65405782838481552], [0.24232918771382317, 0.66576254667772328]]])+(1.-msk_arg1)*numpy.array([[[0.57991223054867791, -0.57792205794002527], [-0.70572588770907418, 0.84975079195812264]], [[-0.88071566625166353, 0.72019098358391775], [-0.96333620472007286, 0.51504226327966207]], [[-0.13724807986659671, 0.81420229896465002], [0.45193448201464936, 0.97654411668289964]], [[0.28266570566132843, -0.90883007344716837], [-0.71593930216139778, -0.11859113719293135]], [[-0.41717200473919158, 0.3619607436530321], [-0.050599521071579057, -0.49442202374371336]], [[-0.88738130429442186, 0.27396039024978158], [-0.4578514852002773, 0.2201967143529151]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.24992158080417098, -0.36861978986563759], [0.18915800209314293, -0.25041279695445773]], [[0.0035019878377827896, -0.32210185879744252], [0.40344288652543669, 0.17741120816639958]], [[0.18598430779240829, -0.04705842493254947], [0.055874932157081864, 0.23269038475058654]], [[0.033170939738746373, -0.29908352347062794], [0.21129555843578848, 0.023321511644106615]], [[0.12811218592782714, 0.1292272358985036], [0.12933366185508705, -0.46314848811631831]], [[0.066409140860803711, 0.30308999005746357], [0.11229519456436038, 0.30851394921988734]]], [[[-0.039903480225982176, 0.058855311528037871], [-0.030201723964063738, 0.039981909763265752]], [[-0.00055914139942197193, 0.051428072405427676], [-0.064415306565283451, -0.028326183813964512]], [[-0.029694999225186914, 0.0075135365370196773], [-0.0089212153799765066, -0.037152278473889404]], [[-0.0052962050483325914, 0.047752872826451409], [-0.033736294843969047, -0.0037236059236509527]], [[-0.020454904540172805, -0.020632937882922565], [-0.020649930277258394, 0.073948141964275738]], [[-0.010603149318425405, -0.048392561322776681], [-0.017929500371089969, -0.049258572359070107]]]])+(1.-msk_ref)*numpy.array([[[[0.11091046399960303, -0.11052983576685112], [-0.13497281405547967, 0.16251813577191745]], [[-0.16844028811613795, 0.13773931976231693], [-0.18424178664421417, 0.098503831080965734]], [[-0.026249227760994875, 0.1557193485680981], [0.086434223072412983, 0.18676785105033478]], [[0.054060912876438431, -0.17381727756879528], [-0.13692616919485256, -0.022681015090618151]], [[-0.079785764424198546, 0.06922639653627613], [-0.0096773547177937142, -0.094560130268601064]], [[-0.16971511725274069, 0.052395987529642335], [-0.08756587288808286, 0.04211347592542259]]], [[[0.21099988519045412, -0.2102757649360191], [-0.25677692836665672, 0.30918009674329694]], [[-0.32044660325358776, 0.26204002406990329], [-0.35050791807482184, 0.1873970795846463]], [[-0.04993743461327349, 0.29624584989490804], [0.16443544177103536, 0.35531358996955892]], [[0.10284734189066667, -0.33067597310984442], [-0.26049305843482901, -0.043149143981045072]], [[-0.15178718514241152, 0.1316986801295448], [-0.01841053279164968, -0.17989444738361768]], [[-0.32287188209347778, 0.099679930590089033], [-0.16658833134133777, 0.08011812650307662]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.43804651728)+(1.-msk_arg0)*(-0.543337177571)
      arg1=numpy.array([[[[0.60795094604023503, -0.097794445632283677, 0.75426723610529667, -0.034864365796523078], [0.85978429070534101, 0.12902059279871558, -0.31201948276004643, 0.076558135122647863], [-0.97390792210779908, 0.49232230873386418, -0.97840440866285516, 0.36414936327706937]], [[0.79385832462423922, 0.42654756978458575, -0.68994913906275612, 0.98855386129816658], [-0.20652839986652571, 0.33057772983488531, -0.5920830737384537, 0.20621684716524569], [-0.031916346185909106, 0.48720143271352656, -0.74521372086186055, 0.26511369448021682]]], [[[0.92814304130352876, -0.60386390960393133, 0.8231201717346297, -0.14016735785423751], [-0.40984787457581096, 0.67837081242748476, -0.2357430543063852, -0.81200518114201992], [0.41040668039544603, 0.81899606955074167, -0.22180340249305619, -0.25792851582212095]], [[-0.77493277192916898, 0.73469674814970154, 0.013542845859844999, 0.5913689784999816], [-0.85270304587659984, 0.049194424212611887, -0.72045050859561766, -0.7655798577204973], [0.16786575391896652, 0.92315757404538923, 0.11199468205897722, -0.88181526809288391]]], [[[0.49210255594399932, -0.052030715984012987, 0.65375080354347515, -0.78565820813980913], [0.29470788038440965, 0.55718988716903617, 0.10815871903784302, 0.75227607016471398], [-0.44501413908595944, 0.67574867634473756, -0.026177234674506833, -0.4513298929006202]], [[-0.92238036282832137, 0.34283069911981556, -0.94496013493330766, -0.86792383195402123], [-0.47685078769718392, -0.13354947626753133, 0.20909536400421036, 0.71714871813267655], [-0.62375289750212426, 0.83122191743468754, 0.37425198819303973, 0.39703242187001364]]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.26631079459014878, 0.042838516318573112, -0.33040413587451362, 0.015272214014351066], [-0.37662551415573142, -0.056517021332908673, 0.13667904774661854, -0.033536024459945503], [0.42661697343095134, -0.21566007272023363, 0.4285866437063911, -0.15951436035333519]], [[-0.34774687431556989, -0.1868476773984856, 0.30222981746693656, -0.43303257608558993], [0.090469046280991255, -0.14480842324457782, 0.25935992839170596, -0.090332571705246292], [0.013980844291047782, -0.21341689081410087, 0.3264382750529829, -0.11613213055035512]]], [[[-0.4065698267808957, 0.26452048251322852, -0.3605649245314631, 0.061399822944421076], [0.17953243407264038, -0.29715797180842296, 0.10326642391191725, 0.35569604161276785], [-0.17977721701576746, -0.35875837593290316, 0.097160207982989369, 0.11298468806313997]], [[0.33945660186989079, -0.32183135178409039, -0.0059323964629681451, -0.25904712145948688], [0.37352359952049263, -0.021549446195941083, 0.31559083616308403, 0.33535959037436136], [-0.073533008874824168, -0.40438596021145301, -0.049058880429842147, 0.38627610707262416]]], [[[-0.2155638107759707, 0.022791873928393921, -0.28637326266137414, 0.34415484184827305], [-0.12909576061743061, -0.24407508953816315, -0.047378550188018537, -0.32953191256891434], [0.19493689376706644, -0.29600935422954072, 0.011466846481195113, 0.19770348772957794]], [[0.40404550554462532, -0.1501757937661832, 0.41393649607619598, 0.38019101185197457], [0.20888282681308795, 0.05850088296359144, -0.091593495981487277, -0.31414449835000358], [0.27323278439426069, -0.3641138660192636, -0.16393978001316453, -0.1739186696474963]]]])+(1.-msk_ref)*numpy.array([[[[-0.33032235112318914, 0.053135358071976641, -0.40982143119981596, 0.018943106109689686], [-0.46715276983182102, -0.07010168473980595, 0.16953178511004208, -0.041596881057647361], [0.52916038161219858, -0.26749701368275169, 0.53160348992600925, -0.19785588725728065]], [[-0.43133274149266626, -0.23175915266657404, 0.37487501788597705, -0.53711806487477098], [0.11221455787175624, -0.17961517069635238, 0.32170074617268041, -0.11204527970637809], [0.017341337455034401, -0.26471465135916988, 0.40490231978035029, -0.14404612649433127]]], [[[-0.50429462044412798, 0.32810171228125756, -0.44723179091214327, 0.076158136604121532], [0.22268558740554051, -0.36858408257097225, 0.12808796575882486, 0.44119260329482518], [-0.22298920738239134, -0.44499101287153464, 0.12051403468624665, 0.14014215180189726]], [[0.42104978510735341, -0.39918805751033343, -0.00735833164576881, -0.32131275168129203], [0.46330526625288276, -0.026729159603916548, 0.39144754592001552, 0.415967999099149], [-0.091207704945178214, -0.50158583073521723, -0.060850874452898815, 0.4791230189047016]]], [[[-0.26737761382214315, 0.028270222369757795, -0.35520711643215874, 0.42687731334626167], [-0.16012574793603021, -0.30274198066559083, -0.058766653131728595, -0.408739556717584], [0.24179272631020376, -0.36715937855256503, 0.014223064804663206, 0.24522431016209553]], [[0.50116354298615895, -0.18627266444449192, 0.51343197263188112, 0.4715752852006026], [0.25909076110994977, 0.072562395501300767, -0.11360928492125208, -0.38965356040894988], [0.33890813883060772, -0.45163377055421156, -0.20334501896518359, -0.21572247550307652]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.313942221653)+(1.-msk_arg0)*(-0.596470906615)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=outer(arg0,arg1)
      s1=numpy.array([[[[-0.071908451118721128, -0.9298321767693376, -0.010456652343436845, -0.72317081763727131], [-0.79540511358853139, -0.59944505462937503, 0.17764878529638262, -0.63254169781305403], [-0.65569459100668182, -0.35236871395185188, 0.099344584955546544, 0.6856630558112311]], [[0.73870761125198481, -0.81816368157535457, 0.36065679455650801, 0.61578751000019882], [-0.11784819310844452, 0.6644331501218268, -0.36233613510811025, -0.29997677507079534], [0.61997001750353298, -0.50775695834061607, -0.16292296834828557, 0.05812061330410323]]], [[[-0.3141236175539619, 0.94016715347619084, 0.53155827975628123, 0.089367349084946479], [-0.72735671464692464, 0.1069000044131192, 0.069826129147439575, -0.99954936782463943], [-0.24510026081969349, 0.64872629644380364, -0.83810769464459489, -0.40620634960910884]], [[-0.35932981076057602, 0.057782586427934213, 0.34081687168119346, -0.5617130132158703], [-0.39541678767602062, 0.29978328549458788, 0.67471415481702679, 0.18646399226812238], [0.97670338198479612, 0.77524556627085395, 0.85675520817167627, -0.28827259264981486]]], [[[0.61076372842157323, 0.059731591245643756, 0.12425642784480662, -0.037404647403425306], [0.25580035018912306, 0.26347904694714797, 0.85608478413826083, -0.23368001845785824], [0.28696432171183361, 0.45596972006885861, -0.24644317045359054, 0.16321766845884822]], [[0.15516257499678643, -0.7815985408636994, 0.67647283422838034, -0.72831757951246034], [0.39377049591829216, 0.84284937090106404, 0.16507706839250669, 0.83282935054500196], [0.83702929398835368, -0.86471393754351134, 0.34590990270438549, -0.98477709430128857]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.022575098899816428, -0.29191357933913886, -0.003282784667748553, -0.22703385312344992], [-0.24971124847390769, -0.18819111220907458, 0.055771454329851193, -0.19858154589940571], [-0.20585021662630107, -0.11062341689895175, 0.03118845971011042, 0.21525858304656226]], [[0.23191150862821266, -0.25685612386932516, 0.11322539533721422, 0.19332169895545123], [-0.036997523562222363, 0.2085936192889532, -0.11375261124089571, -0.094175375209940002], [0.19463476465312707, -0.15940634756107425, -0.051148398641514513, 0.018246514464508076]]], [[[-0.098616666368476213, 0.29515816488721736, 0.16687858728457833, 0.028056184114941152], [-0.22834798293026992, 0.033560424880138877, 0.021921370113956052, -0.31380074918642659], [-0.076947320409392631, 0.20366257475010061, -0.2631173916409531, -0.12752532384572002]], [[-0.11280879909622218, 0.018140393556025248, 0.10699680587231959, -0.17634543130022695], [-0.12413802480178683, 0.094114630662518795, 0.21182126074378621, 0.05853891999088761], [0.30662842963601988, 0.24338231540148314, 0.26897163346594394, -0.090500938178068827]]], [[[0.19174452180555956, 0.018752268458508813, 0.039009339012227931, -0.011742898105967516], [0.080306530237913856, 0.08271719735752564, 0.26876115905544412, -0.073362024150505703], [0.090090216693275327, 0.14314814692478059, -0.077368916443337085, 0.05124091744894585]], [[0.048712083511845972, -0.24537678235926424, 0.21237338446536141, -0.22864963898086421], [0.12362118430987702, 0.26460600401926687, 0.051824661595059482, 0.26146029656767938], [0.26277883614310099, -0.27147021464647048, 0.10859572334668666, -0.30916310881764436]]]])+(1.-msk_ref)*numpy.array([[[[0.042891299032087493, 0.55461784147770943, 0.0062370889034510553, 0.43135035323385029], [0.47443600922861817, 0.35755153520085731, -0.10596233202484485, 0.37729271996655034], [0.39110274716052129, 0.21017768627373778, -0.059256154655758189, -0.40897806453236035]], [[-0.44061759860711364, 0.48801083290898517, -0.21512128522609794, -0.36729933437221313], [0.070293018586371991, -0.39631504343844348, 0.216122963007428, 0.17892741899001921], [-0.36979407841465223, 0.30286225328166794, 0.097178810639162061, -0.034667254910537272]]], [[[0.1873655989516978, -0.56078235440389446, -0.31705904904511251, -0.053305023730506405], [0.43384711901819656, -0.063762742549475629, -0.041649254558012168, 0.59620211763313924], [0.14619517478277561, -0.38694636218503869, 0.49990685646594285, 0.24229026962424749]], [[0.21432977799827455, -0.034465631713248387, -0.20328734844148091, 0.33504547025049713], [0.23585460983603615, -0.17881200808707814, -0.40244736362990585, -0.11122034651928021], [-0.58257515174672703, -0.46241142576308852, -0.51102955576556264, 0.17194621469018581]]], [[[-0.36430279481937294, -0.035628156383865239, -0.074115344169373723, 0.022310783948347717], [-0.15257746678982428, -0.15715758600670757, -0.51062966733453463, 0.13938333246744497], [-0.17116586913770965, -0.27197267231860878, 0.14699618130960862, -0.097354590681289002]], [[-0.092549961781101808, 0.46620079027818723, -0.40349636473284162, 0.43442024695567749], [-0.23487264469875049, -0.50273512840151535, -0.098463668645478744, -0.49675847777543103], [-0.49926362184882012, 0.51577670628948713, -0.20632519327330423, 0.58739088625189662]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.647480014887)+(1.-msk_arg0)*(-0.378983139168)
      arg1=Data(numpy.array([[[[-0.15966497293401405, -0.82379668881748769, 0.42255757211308609, 0.27970108322437981], [-0.11267007778659455, 0.0049557419911394618, 0.85704943783706944, 0.58893235535539623], [0.99687634124774305, 0.58590098057034923, 0.11077319930465057, 0.31718343280306582]], [[-0.9128103218342134, -0.46964610822504937, -0.57260883670011609, 0.69637833743595912], [0.16045131981896832, -0.71296302378250265, 0.054788118352184778, -0.59976311458090792], [-0.50177368977017611, -0.022496865498684349, 0.43022221754422185, -0.90687847956880363]]], [[[-0.8375205296921322, 0.47187921735539762, 0.036701311258400704, 0.15982997815519395], [0.40454750391093253, 0.057429996683073803, 0.049686735148374073, -0.048948156089927863], [-0.51397104354091527, -0.91196540959389893, 0.36981298229808446, 0.09035240445060122]], [[0.69669588663419968, 0.99953429884768585, 0.98393221607933246, -0.51244956485403348], [-0.37301000438945131, 0.99808559202134495, -0.080812184802454778, -0.068534716172923504], [0.094817978807470338, 0.57847078543819253, -0.37507654657691658, 0.90495886400539605]]], [[[0.24152699525662746, 0.9003824620926717, 0.96762490705552207, -0.47904707822731774], [-0.93822028424107917, -0.25496865217469478, 0.78629828304891158, 0.7349206929295633], [0.99985514559762789, 0.0022368160793040381, -0.49421642858409776, 0.25409309561466964]], [[-0.51028035342680056, -0.30059759007213782, -0.36242607685938144, 0.097490979178828097], [-0.17210992271799341, 0.15143026193152109, -0.55920023737619973, 0.867565538988045], [0.16060374897895957, 0.67533415472816549, 0.48603174128372562, -0.54273022379048674]]]]),self.functionspace)
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.10337987905231617, 0.5333918923397607, -0.27359758308257631, -0.18110086153015112], [0.072951623642631885, -0.0032087438982012296, -0.55492238277000738, -0.38132193021319988], [-0.6454575082720132, -0.37935917562224825, -0.071723432734903156, -0.20536993379337451]], [[0.59102644077061428, 0.30408646914537751, 0.37075277811126389, -0.450891056290317], [-0.10388902294509304, 0.46162930925288037, -0.035474211686326758, 0.38833463035777632], [0.32488843612251322, 0.014566270808008602, -0.27856028782043496, 0.58718569145229682]]], [[[0.54227780503358813, -0.30553236267834061, -0.02376336555997741, -0.10348671663538224], [-0.26193642385492233, -0.037184775107341556, -0.032171168013576924, 0.031692952833818636], [0.33278597892357864, 0.59047937698067687, -0.2394465152839278, -0.058501376178790178]], [[-0.4510966630499213, -0.64717848269839429, -0.63707644591526602, 0.33180085188074548], [0.24151652319524144, -0.64624047398090756, 0.052324274618978983, 0.044374859047950146], [-0.061392746329856709, -0.37454827276746294, 0.24285456796154695, -0.58594277873872358]]], [[[-0.15638390248447684, -0.58297964996014195, -0.62651778922575541, 0.31017340934240245], [0.60747888360810065, 0.16508710670589877, -0.50911242401446821, -0.47584646119911239], [-0.64738622455682338, -0.0014482937083282164, 0.31999526053724303, -0.1645202013313789]], [[0.33039633083354675, 0.19463093209503271, 0.23466364164050435, -0.063123460650097524], [0.11143773532372042, -0.098048068249828368, 0.36207097802139493, -0.56173134809979874], [-0.10398771777987346, -0.43726536855738107, -0.3146958390821491, 0.35140697337972138]]]])+(1.-msk_ref)*numpy.array([[[[0.060510332657683796, 0.31220505516413888, -0.16014219515856606, -0.10600199454902588], [0.042700059769850397, -0.0018781426567080065, -0.32480728637354089, -0.22319543279010878], [-0.37779932516823894, -0.22204659285807737, -0.041981174808143305, -0.12020717305574337]], [[0.34593972123355332, 0.17798795639309697, 0.21700909444786562, -0.26391564836997394], [-0.060808344868618629, 0.27020096486370138, -0.02076377308221114, 0.22730010792096444], [0.19016376810094046, 0.0085259327081284829, -0.16304696654466649, 0.34369165303075977]]], [[[0.31740615946025186, -0.17883426710142078, -0.013909178152285363, -0.060572866854385642], [-0.1533166829746867, -0.021765000425351008, -0.018830434861532772, 0.018550525851439188], [0.19478635952251611, 0.34561951374039762, -0.14015288493635569, -0.034242037870052831]], [[-0.26403599416196338, -0.37880664628324029, -0.37289371997813298, 0.19420974475358466], [0.14136450240453091, -0.37825761082245973, 0.030626455479447413, 0.025973501877192745], [-0.035934415258006579, -0.21923067418226289, 0.14214768704995931, -0.34296415109854406]]], [[[-0.091534658856137124, -0.34122977193556603, -0.36671352481290898, 0.18155076551577951], [0.3555696685526446, 0.096628820190563638, -0.29799379163217377, -0.27852255124586361], [-0.37892824179172513, -0.00084771557947578569, 0.18729969353312909, -0.096296999016926302]], [[0.19338765019737314, 0.11392141831183186, 0.1373533723244599, -0.036947437329740596], [0.065226758793602652, -0.05738951603181891, 0.21192746138424376, -0.32879271139954436], [-0.060866112950172836, -0.25594025794615238, -0.18419783504692674, 0.20568560393339297]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_outer_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.690473237624)+(1.-msk_arg0)*(-0.0562045307337)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.56201639410078919, -0.67905807272448215, -0.25380207692191359, -0.32222609823072079], [0.76016816954232169, 0.24391957505617157, 0.26418043996577079, -0.32148058834963256], [-0.69353153778537813, -0.85059153048984104, -0.41192354690135802, -0.09464631974832094]], [[-0.64890091661083549, -0.087129640274469011, 0.92245871263029744, 0.17058762013903395], [-0.29767982521749548, -0.47635452775846954, 0.36511868715211371, 0.65030298443566781], [-0.52341553066884439, -0.136229235578611, 0.13799323750383441, -0.74164324953122218]]], [[[-0.81201740098239839, -0.11022566256500266, 0.82203378705157415, 0.13325227536636453], [0.18849371161493056, -0.50191772584346173, -0.64447769012620082, 0.058216229392509922], [0.67428521460060353, 0.6029321949871751, 0.50664817337072376, 0.1680297553997907]], [[-0.20918113073753042, -0.8713521135486364, 0.91086493006721159, -0.47669574703485496], [-0.17870237370162179, 0.19863142254470101, -0.44726017245596061, 0.21973053356979233], [-0.13665198792278255, -0.58338985283344003, -0.83782387884143894, 0.37637197375960674]]], [[[0.48442932734435407, -0.66084657381684431, 0.013136421713676016, -0.30866551598460745], [0.97879855698655449, -0.77731723293622768, 0.67126837966669139, -0.26330714312312908], [0.92670271865249032, -0.58770621430601544, 0.40426437257369874, 0.14919210195014121]], [[-0.63536521155437176, -0.82420572388288837, -0.53044375853468684, -0.53196527767767821], [-0.2730105313227722, -0.13733715172520444, -0.33175551206416021, -0.019053105080914845], [-0.66132172779628196, 0.58168822478577753, 0.36299207595924909, -0.016604553536709643]]]])+(1.-msk_arg1)*numpy.array([[[[-0.93858479936100658, 0.85293162859941574, -0.23719969973251875, -0.59966352839920201], [0.76964952963950251, 0.099205810203436107, 0.10815561430526488, -0.2269339495561149], [0.7272309695157666, 0.98459525877631671, -0.38356325840308836, -0.79663843013422331]], [[-0.61086241061772206, 0.50866661913662514, 0.8750602262829843, -0.41188300050386295], [0.60240578511221887, -0.66351955655808426, -0.42029065852688063, 0.12740508973692011], [-0.65658994729083653, -0.29334456722385105, -0.87068164399583092, -0.25083390343440404]]], [[[0.3684013559470114, -0.098457429179994449, -0.85367203809283088, -0.46399893153072269], [-0.80864851862858034, 0.43887697186794949, 0.57510730528110399, 0.85667058370089966], [0.025836965582203275, -0.23640860192184587, 0.21613477422384664, 0.67145006285814479]], [[-0.87221722248074096, 0.35929053249580289, 0.79083599288546269, 0.1352443609608609], [0.13264053413890675, -0.062105862078114304, 0.5600587428281163, -0.11056941015246968], [-0.15919859652552848, 0.73179978310420735, -0.80962177892350429, -0.72481111775677354]]], [[[-0.98213931444948832, -0.99397587509615715, -0.32204725603144935, 0.40306261750230998], [-0.089799448848834951, 0.65825162639669088, -0.52160276190906951, 0.84004216473803495], [-0.5391586472258616, -0.25865381238936069, -0.67372728245366553, -0.51690007321233233]], [[0.68035296315998606, -0.39722085397897633, 0.94591470622274665, -0.26720530940111842], [0.23930428363536183, -0.65515207918738083, -0.81974978522324471, -0.0056555485824274498], [0.922210890681332, 0.74667199058644851, -0.90112295821245314, 0.27807862829333208]]]])
      res=outer(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.38805727923252009, 0.46887142600876536, 0.17524354176796114, 0.22248849729230466], [-0.52487577716257261, -0.16841993870889735, -0.18240952370009017, 0.22197374267102901], [0.47886496628899961, 0.58731068795284702, 0.28442218508252926, 0.065350750825816498]], [[0.4480487167894443, 0.06016068481332433, -0.63693305388427934, -0.11778618637596644], [0.20553995269326114, 0.32891005303822701, -0.2521046820349328, -0.44901680709982467], [0.36140441608358453, 0.094062641349001844, -0.095280637469485757, 0.51208481566578368]]], [[[0.5606762838633168, 0.076107870100504432, -0.56759233038179224, -0.092007129992974279], [-0.13014986333051973, 0.34656075718399437, 0.44499459727785451, -0.040196748390905958], [-0.46557589520725107, -0.41630854474052059, -0.34982700460355326, -0.11602004922805698]], [[0.14443397259018523, 0.60164531495241469, -0.62892785730163714, 0.32914565581673255], [0.1233892065408471, -0.1371496814182942, 0.30882117933592157, -0.15171805291877657], [0.094354540528795092, 0.40281508048287579, 0.57849496618231977, -0.25987477527271896]]], [[[-0.33448548605145734, 0.45629687339602332, -0.0090703476314356775, 0.21312527816476468], [-0.67583420862417465, 0.53671674648638157, -0.46349285142305563, 0.18180653560174456], [-0.63986342646291849, 0.4057954125636003, -0.27913373018698395, -0.10301315366143916]], [[0.43870267469558466, 0.5690919946376245, 0.36625721933287175, 0.36730778758163984], [0.18850646546787436, 0.094827627797756095, 0.22906830251453819, 0.013155659152008956], [0.45662495450257551, -0.4016401518555745, -0.2506363139194282, 0.011464999839792421]]]])+(1.-msk_ref)*numpy.array([[[[0.052752718201846856, -0.047938621933340218, 0.013331697813635078, 0.033703807211777563], [-0.043257790642782727, -0.0055758160085382568, -0.0060788355482398722, 0.012754716142341149], [-0.040873675376629927, -0.055338714482125205, 0.021557992945225302, 0.044774689130106383]], [[0.034333235131611191, -0.028589368628459531, -0.049182349381939501, 0.023149690760498068], [-0.033857934463483952, 0.037292805308963943, 0.023622239234251004, -0.00716074328174548], [0.036903329871930554, 0.016487293744089832, 0.048936253219211259, 0.014098001834626889]]], [[[-0.02070582533265173, 0.0055337536043057302, 0.047980236301468374, 0.026078842207611346], [0.045449710518001646, -0.024666874253654842, -0.032323636214833418, -0.048148768150253408], [-0.001452154526129874, 0.01328723453242177, -0.012147753560480321, -0.037738535694039309]], [[0.049022559687360356, -0.020193755775979184, -0.044448565867428194, -0.0076013458421810825], [-0.0074549989775413907, 0.0034906308339108208, -0.031477838823946828, 0.0062145018111189141], [0.0089476824111771568, -0.041130463400377883, 0.045504412156159565, 0.040737668744070664]]], [[[0.05520067928372778, 0.055865947620374501, 0.01810051489931563, -0.022653945273004492], [0.0050471358826915131, -0.036996723766305049, 0.029316438462488603, -0.047214175665602633], [0.030303158758333136, 0.0145375161478203, 0.037866525752783084, 0.029052126051101926]], [[-0.038238919027672999, 0.022325611695518416, -0.053164692177332502, 0.015018149024436576], [-0.013449984964284022, 0.036822515169918905, 0.046073651997504191, 0.00031786745411684157], [-0.05183243034822968, -0.041966348842891112, 0.050647192999672894, -0.015629278810291056]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_float_rank0(self):
      arg0=0.190833616561
      arg1=0.400555937829
      res=maximum(arg0,arg1)
      ref=0.400555937829
      if isinstance(res,numpy.ndarray):
         self.assertEqual(res.shape,(),"wrong shape of result.")
      else: 
         self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_array_rank0(self):
      arg0=-0.607520185637
      arg1=numpy.array(-0.822580587394)
      res=maximum(arg0,arg1)
      ref=numpy.array(-0.607520185637)
      if isinstance(res,numpy.ndarray):
         self.assertEqual(res.shape,(),"wrong shape of result.")
      else: 
         self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      res=maximum(5.0,-1e20)
      self.assertTrue(Lsup(res-5.0)<=self.RES_TOL*5,"Wrong large result")
      self.assertTrue(Lsup(maximum(-numpy.inf,-5.)+5)<self.RES_TOL*5,"Wrong large result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_Symbol_rank0(self):
      arg0=0.516035627407
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.1743874735)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(0.516035627407)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_constData_rank0(self):
      arg0=0.66510948385
      arg1=Data(0.447127953255,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(0.66510948385,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_expandedData_rank0(self):
      arg0=-0.626379875424
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.808319082591)+(1.-msk_arg1)*(-0.732167960529)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.626379875424)+(1.-msk_ref)*(-0.626379875424)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_array_rank1(self):
      arg0=-0.34068193517
      arg1=numpy.array([-0.28498591942826468, -0.9244388369769494])
      res=maximum(arg0,arg1)
      ref=numpy.array([-0.28498591942826468, -0.34068193517048506])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
      arg1=numpy.array([-1e16, -numpy.inf])
      arg0=5.
      res=maximum(arg0,arg1)
      ref=numpy.array([5.,5.])
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*5,"wrong large result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_Symbol_rank1(self):
      arg0=0.954576259921
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([0.0049640892542059945, 0.29380146783316974])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.95457625992145734, 0.95457625992145734])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_constData_rank1(self):
      arg0=0.133219946847
      arg1=Data(numpy.array([0.73105505422959327, -0.57302577924904785]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.73105505422959327, 0.13321994684664973]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_expandedData_rank1(self):
      arg0=-0.989795574687
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.12744166984605543, 0.1251383582008796])+(1.-msk_arg1)*numpy.array([-0.75971755196106439, -0.044294049795981616])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.12744166984605543, 0.1251383582008796])+(1.-msk_ref)*numpy.array([-0.75971755196106439, -0.044294049795981616])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_array_rank2(self):
      arg0=-0.918302739699
      arg1=numpy.array([[0.19183086796477156, 0.41569819970182253, -0.57007179868513158, 0.18371653862222859, -0.95803703880242908], [-0.99708943520039761, -0.33564912838654082, 0.22798896799780199, -0.32560020734492867, -0.83553942115001578], [0.90329116559065703, -0.4744523361429227, -0.48030437204982679, -0.25285363526704185, 0.4839054158389946], [-0.75326661018214147, -0.89189805199917815, -0.29494941406721664, 0.33826278929980447, 0.52219823679588995]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[0.19183086796477156, 0.41569819970182253, -0.57007179868513158, 0.18371653862222859, -0.91830273969888498], [-0.91830273969888498, -0.33564912838654082, 0.22798896799780199, -0.32560020734492867, -0.83553942115001578], [0.90329116559065703, -0.4744523361429227, -0.48030437204982679, -0.25285363526704185, 0.4839054158389946], [-0.75326661018214147, -0.89189805199917815, -0.29494941406721664, 0.33826278929980447, 0.52219823679588995]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_Symbol_rank2(self):
      arg0=-0.277418552647
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.99289072506960552, 0.19782624907337709, 0.95613086199918595, 0.44091386643316266, -0.37059163780321391], [0.84363308798978487, -0.51183523461403269, -0.57551129897501507, 0.19000742902371703, -0.074561874521030802], [0.35669726023155235, -0.21740764792612688, -0.02741866226550882, 0.7913626340003177, -0.55431312092720475], [-0.79982049514168496, 0.49589465225392826, -0.21107892173219511, -0.28843484560692079, -0.85036340359415541]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.2774185526470867, 0.19782624907337709, 0.95613086199918595, 0.44091386643316266, -0.2774185526470867], [0.84363308798978487, -0.2774185526470867, -0.2774185526470867, 0.19000742902371703, -0.074561874521030802], [0.35669726023155235, -0.21740764792612688, -0.02741866226550882, 0.7913626340003177, -0.2774185526470867], [-0.2774185526470867, 0.49589465225392826, -0.21107892173219511, -0.2774185526470867, -0.2774185526470867]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_constData_rank2(self):
      arg0=-0.711360717248
      arg1=Data(numpy.array([[-0.49649726218321621, -0.1862706537495149, 0.6559357060844726, -0.60255638663804834, -0.72898391055117684], [0.877046256493895, -0.62092354207162237, -0.42516414792107171, -0.06686623727516694, 0.93735063523149598], [0.64862166841681268, 0.45648573520660918, -0.63173946887266208, 0.070770417261089902, 0.27864121106526873], [-0.25951227165524937, 0.85373113390321809, 0.5672699499488485, 0.22967921637760402, 0.88510133022334969]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[-0.49649726218321621, -0.1862706537495149, 0.6559357060844726, -0.60255638663804834, -0.71136071724766237], [0.877046256493895, -0.62092354207162237, -0.42516414792107171, -0.06686623727516694, 0.93735063523149598], [0.64862166841681268, 0.45648573520660918, -0.63173946887266208, 0.070770417261089902, 0.27864121106526873], [-0.25951227165524937, 0.85373113390321809, 0.5672699499488485, 0.22967921637760402, 0.88510133022334969]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_expandedData_rank2(self):
      arg0=0.959635515709
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.0064259026939066377, -0.7701474037173317, -0.19596310110827386, -0.73984521228212596, -0.61313716616198133], [0.43189572076819838, 0.088536215133996565, 0.067634491583320466, -0.70571296121285232, -0.21462215163400677], [-0.29878468443140771, 0.49093472354971013, 0.86357875479621882, 0.021213198562558766, -0.68952176426127321], [-0.64172570311955179, -0.10403523776207346, -0.1560226988237976, 0.18544285896364587, -0.99033629867964823]])+(1.-msk_arg1)*numpy.array([[0.59197813276731015, 0.69989892013318467, 0.37778322838437939, -0.96988982584154027, -0.12556098215489531], [-0.57102119315921684, -0.97261687381790707, 0.59394598916500385, -0.95926160849480113, -0.74824572871782435], [0.3774569754088497, -0.71634815703797106, -0.64965181170462061, -0.88835061435822782, 0.049571630549325674], [0.056229799936159397, 0.69426319270729175, 0.32265901320206658, -0.26876155184504102, 0.5465577920078144]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646]])+(1.-msk_ref)*numpy.array([[0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646], [0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646, 0.95963551570949646]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_array_rank3(self):
      arg0=0.417103350623
      arg1=numpy.array([[[-0.45176533427754739, -0.55230637452486797], [-0.2772690226072374, -0.43884388551060138]], [[0.43650997405556069, -0.91050440106967101], [-0.0097638249506593056, -0.99719694387315783]], [[-0.88346341044578236, -0.42616345838712633], [0.47323800588958997, 0.31718901666579002]], [[0.72726350287088914, 0.18527925932004718], [0.95182455621773032, 0.5402040160782311]], [[-0.60062553156664, 0.18455913185890904], [-0.041559661168358408, 0.65568567006769984]], [[0.46551889127092827, -0.88399518601736693], [-0.45129696041660194, -0.97227033198060986]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[0.41710335062270154, 0.41710335062270154], [0.41710335062270154, 0.41710335062270154]], [[0.43650997405556069, 0.41710335062270154], [0.41710335062270154, 0.41710335062270154]], [[0.41710335062270154, 0.41710335062270154], [0.47323800588958997, 0.41710335062270154]], [[0.72726350287088914, 0.41710335062270154], [0.95182455621773032, 0.5402040160782311]], [[0.41710335062270154, 0.41710335062270154], [0.41710335062270154, 0.65568567006769984]], [[0.46551889127092827, 0.41710335062270154], [0.41710335062270154, 0.41710335062270154]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_Symbol_rank3(self):
      arg0=-0.954672923464
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[0.3483462792197054, 0.45601206615028178], [0.64107739105983463, -0.95807507362286382]], [[0.026145251253471224, 0.14711601575188804], [-0.7393117552767503, -0.11682100905814674]], [[-0.88508316018799715, -0.1024334575510879], [0.91416954091266844, 0.3902047430084068]], [[0.63664321883211761, -0.22396125684499824], [0.50607888063060558, -0.8696015456531343]], [[-0.37313751414078422, -0.54165981780497363], [0.47720240432367467, -0.76737664842694775]], [[-0.68132241809142768, 0.73233776988106447], [0.86973993037218422, 0.7595371330514924]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.3483462792197054, 0.45601206615028178], [0.64107739105983463, -0.95467292346444288]], [[0.026145251253471224, 0.14711601575188804], [-0.7393117552767503, -0.11682100905814674]], [[-0.88508316018799715, -0.1024334575510879], [0.91416954091266844, 0.3902047430084068]], [[0.63664321883211761, -0.22396125684499824], [0.50607888063060558, -0.8696015456531343]], [[-0.37313751414078422, -0.54165981780497363], [0.47720240432367467, -0.76737664842694775]], [[-0.68132241809142768, 0.73233776988106447], [0.86973993037218422, 0.7595371330514924]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_constData_rank3(self):
      arg0=-0.143403966016
      arg1=Data(numpy.array([[[0.95668360633256144, -0.1800322431671213], [-0.42783838761912918, -0.89612594522855216]], [[0.60799134259365384, 0.37935166549606292], [-0.16348477876382117, 0.055255205348395009]], [[-0.84148244802025407, -0.08342603479235744], [-0.073128933987135269, 0.91858564423034572]], [[-0.44722942409971833, 0.12566700748073134], [-0.84487161060035754, 0.69960515111291777]], [[0.28894841908227131, -0.28216896408608538], [-0.4241999876693332, 0.41787923950246331]], [[-0.74860509777465034, 0.95471157706396692], [0.3975080494679184, -0.47708712009131093]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.95668360633256144, -0.14340396601550798], [-0.14340396601550798, -0.14340396601550798]], [[0.60799134259365384, 0.37935166549606292], [-0.14340396601550798, 0.055255205348395009]], [[-0.14340396601550798, -0.08342603479235744], [-0.073128933987135269, 0.91858564423034572]], [[-0.14340396601550798, 0.12566700748073134], [-0.14340396601550798, 0.69960515111291777]], [[0.28894841908227131, -0.14340396601550798], [-0.14340396601550798, 0.41787923950246331]], [[-0.14340396601550798, 0.95471157706396692], [0.3975080494679184, -0.14340396601550798]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_expandedData_rank3(self):
      arg0=-0.801969488354
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.9889605718250738, -0.37340300276544558], [-0.23838119100250488, 0.20477150811741951]], [[-0.20467295181293843, 0.35301722647248601], [-0.095474828781975996, -0.8510749109672644]], [[-0.70425456344214243, -0.46755265678910085], [0.66251795702833483, -0.92448552384625948]], [[-0.66717945104318677, -0.78744474882359006], [-0.16257558572897457, -0.26356489046284248]], [[-0.27934849406098761, 0.93892978662484139], [-0.18468624033742786, -0.70668348468539377]], [[-0.41995260750527241, -0.046679843759030781], [0.13620169917808034, 0.46624408802032424]]])+(1.-msk_arg1)*numpy.array([[[-0.56084457456829484, 0.032990993182650197], [0.42149270935344574, 0.1750548418363671]], [[-0.78410663312115148, -0.74664526632471984], [0.88029830669181575, -0.60878924835930359]], [[0.054733122499459386, 0.19985918328880037], [0.60155025489305403, 0.19928445576334952]], [[-0.086860661257179173, -0.14114143266705215], [0.57480201913819329, 0.38294532940281845]], [[0.39095132070017913, 0.096587125583033284], [0.50661117336330563, -0.043716480204347041]], [[0.74636288223324332, 0.82758235657560153], [-0.88560450972740812, -0.97248750693241459]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.80196948835439708, -0.37340300276544558], [-0.23838119100250488, 0.20477150811741951]], [[-0.20467295181293843, 0.35301722647248601], [-0.095474828781975996, -0.80196948835439708]], [[-0.70425456344214243, -0.46755265678910085], [0.66251795702833483, -0.80196948835439708]], [[-0.66717945104318677, -0.78744474882359006], [-0.16257558572897457, -0.26356489046284248]], [[-0.27934849406098761, 0.93892978662484139], [-0.18468624033742786, -0.70668348468539377]], [[-0.41995260750527241, -0.046679843759030781], [0.13620169917808034, 0.46624408802032424]]])+(1.-msk_ref)*numpy.array([[[-0.56084457456829484, 0.032990993182650197], [0.42149270935344574, 0.1750548418363671]], [[-0.78410663312115148, -0.74664526632471984], [0.88029830669181575, -0.60878924835930359]], [[0.054733122499459386, 0.19985918328880037], [0.60155025489305403, 0.19928445576334952]], [[-0.086860661257179173, -0.14114143266705215], [0.57480201913819329, 0.38294532940281845]], [[0.39095132070017913, 0.096587125583033284], [0.50661117336330563, -0.043716480204347041]], [[0.74636288223324332, 0.82758235657560153], [-0.80196948835439708, -0.80196948835439708]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_array_rank4(self):
      arg0=0.270905406087
      arg1=numpy.array([[[[-0.95360288151815653, -0.64078885487275894, 0.96879725589600185, 0.37349635624215671], [-0.23784772076621397, 0.65332164004684334, -0.2721768925317285, 0.22325285780446613], [0.065576671186904578, -0.0042823363828001781, 0.15081972618097739, -0.96500711704114939]], [[0.52806648530122047, -0.25468561493618047, 0.55916471457602035, 0.48281251134172609], [0.55407748333919904, -0.11022873429516356, -0.43913854875915792, -0.12104124446879827], [-0.17051966474671065, 0.390161734069155, -0.40998699913604164, -0.63105941814244515]]], [[[-0.94813299631664916, 0.33923647049967887, -0.6464524838010095, 0.099997153482365375], [-0.19694004792045505, -0.21476580508277232, 0.50450751987287878, 0.0044718532670784406], [-0.35156116891903766, 0.93361770074048556, 0.64503353549824793, 0.31429197499208672]], [[0.67485265775424352, -0.0069832167465539996, -0.27581854606767786, -0.049974746554111604], [0.88520110268538477, 0.59559353742053656, -0.88876809828391945, 0.14398781431221419], [0.49156586086617393, 0.34186756327969836, -0.39488984681038053, -0.49978462106847665]]], [[[0.4711536016262412, -0.84357230970236885, -0.60522103581380593, 0.74011510067186914], [0.058929816244633448, -0.51830971900097267, -0.92512610450551414, 0.79105987068184391], [-0.08685194216427039, 0.9276912197047904, 0.75649609887119307, -0.2819853018873526]], [[-0.23960953208469071, -0.026232616293173194, -0.60625185640538937, 0.41226223746307045], [-0.48188248446866555, -0.57075257849297079, -0.98430513472426018, 0.0026142817361640702], [-0.25427236937723752, -0.83104284375176407, -0.76917008485860205, 0.63675334398029215]]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[[0.27090540608696001, 0.27090540608696001, 0.96879725589600185, 0.37349635624215671], [0.27090540608696001, 0.65332164004684334, 0.27090540608696001, 0.27090540608696001], [0.27090540608696001, 0.27090540608696001, 0.27090540608696001, 0.27090540608696001]], [[0.52806648530122047, 0.27090540608696001, 0.55916471457602035, 0.48281251134172609], [0.55407748333919904, 0.27090540608696001, 0.27090540608696001, 0.27090540608696001], [0.27090540608696001, 0.390161734069155, 0.27090540608696001, 0.27090540608696001]]], [[[0.27090540608696001, 0.33923647049967887, 0.27090540608696001, 0.27090540608696001], [0.27090540608696001, 0.27090540608696001, 0.50450751987287878, 0.27090540608696001], [0.27090540608696001, 0.93361770074048556, 0.64503353549824793, 0.31429197499208672]], [[0.67485265775424352, 0.27090540608696001, 0.27090540608696001, 0.27090540608696001], [0.88520110268538477, 0.59559353742053656, 0.27090540608696001, 0.27090540608696001], [0.49156586086617393, 0.34186756327969836, 0.27090540608696001, 0.27090540608696001]]], [[[0.4711536016262412, 0.27090540608696001, 0.27090540608696001, 0.74011510067186914], [0.27090540608696001, 0.27090540608696001, 0.27090540608696001, 0.79105987068184391], [0.27090540608696001, 0.9276912197047904, 0.75649609887119307, 0.27090540608696001]], [[0.27090540608696001, 0.27090540608696001, 0.27090540608696001, 0.41226223746307045], [0.27090540608696001, 0.27090540608696001, 0.27090540608696001, 0.27090540608696001], [0.27090540608696001, 0.27090540608696001, 0.27090540608696001, 0.63675334398029215]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_Symbol_rank4(self):
      arg0=0.353401580302
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[-0.074589175407685948, 0.90104233790371513, 0.51151358320482165, 0.34042635800176368], [-0.77069325275751432, 0.71809715519512718, 0.76053478594611623, 0.45758260571055409], [-0.71024122512929955, -0.72118531031641853, 0.50390667571259962, 0.50204805556941356]], [[0.42183042301612583, -0.66538505104501611, -0.017057666144933759, -0.051306118703368098], [0.31222165692460968, -0.70310368247319088, -0.56746371028474152, 0.94129894601728492], [0.11142483414811877, -0.93362210033321413, 0.45587886579292758, 0.44507434385519118]]], [[[-0.0097583613027334426, 0.85209277666498373, 0.46992517586499605, 0.66074063590208398], [-0.17277498356081411, -0.09932228691815137, -0.50321737390269261, 0.95425990318493614], [-0.70104206593764151, 0.28297161453347353, -0.96791368879574313, -0.48721758409606686]], [[-0.046150958046365709, 0.10237443745619701, -0.13905807947799942, -0.39720574385032692], [0.65752541777313112, 0.50685244861699741, -0.56200626604121928, 0.39118546164001455], [-0.52590955373082471, 0.065148720258848991, -0.45477254310198667, -0.52903327237098385]]], [[[0.077120492993482559, -0.45795355685591566, 0.49594295266032584, -0.080271954488129849], [-0.67763184010244304, -0.41492698239532411, 0.15548079600238562, -0.092486466219969587], [-0.2018432259717875, 0.10685915175149963, -0.99636262057444425, 0.83639721625518804]], [[0.76208463161351037, 0.62163409104824519, -0.21443990356109865, -0.71119347731782123], [0.56204308504316747, 0.48054590873774083, 0.29921579200350279, 0.16272132140533691], [0.75211547917892352, 0.38904318222092527, 0.57327591456049376, 0.63678554842892932]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.35340158030175606, 0.90104233790371513, 0.51151358320482165, 0.35340158030175606], [0.35340158030175606, 0.71809715519512718, 0.76053478594611623, 0.45758260571055409], [0.35340158030175606, 0.35340158030175606, 0.50390667571259962, 0.50204805556941356]], [[0.42183042301612583, 0.35340158030175606, 0.35340158030175606, 0.35340158030175606], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.94129894601728492], [0.35340158030175606, 0.35340158030175606, 0.45587886579292758, 0.44507434385519118]]], [[[0.35340158030175606, 0.85209277666498373, 0.46992517586499605, 0.66074063590208398], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.95425990318493614], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.35340158030175606]], [[0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.35340158030175606], [0.65752541777313112, 0.50685244861699741, 0.35340158030175606, 0.39118546164001455], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.35340158030175606]]], [[[0.35340158030175606, 0.35340158030175606, 0.49594295266032584, 0.35340158030175606], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.35340158030175606], [0.35340158030175606, 0.35340158030175606, 0.35340158030175606, 0.83639721625518804]], [[0.76208463161351037, 0.62163409104824519, 0.35340158030175606, 0.35340158030175606], [0.56204308504316747, 0.48054590873774083, 0.35340158030175606, 0.35340158030175606], [0.75211547917892352, 0.38904318222092527, 0.57327591456049376, 0.63678554842892932]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_constData_rank4(self):
      arg0=-0.309869238816
      arg1=Data(numpy.array([[[[-0.82719032260804259, 0.55581165940832689, 0.28993332944093009, -0.86289948135541761], [0.48402181856866533, -0.32199675516574411, 0.12059109046218897, -0.7225649933438465], [-0.83246155788795284, 0.81949583430522099, 0.917821601053878, -0.38511162370320817]], [[0.040807935475384971, 0.064255792480005569, -0.31189093961899572, -0.84263132870438207], [-0.019662050520152574, 0.44586496426970701, -0.91681580233687932, -0.18241240078470145], [-0.5433480154993422, -0.59905565233432823, -0.92180233099065312, 0.86950034082755967]]], [[[0.59960822306615458, 0.13652846774954908, 0.43957232336106244, -0.43425247806437417], [-0.96917998667692262, 0.55216812010001926, 0.71335421685640177, 0.58559993798062027], [0.70518632845671458, -0.51213306047690121, 0.49991004029291486, -0.88896060694390422]], [[-0.01327086420919299, 0.94285928205783565, 0.36084019612625773, 0.75948720217972943], [0.15534832801909948, -0.087517839294278144, -0.82737480559591559, -0.17830159519382138], [-0.53216202742506535, 0.14079322663297011, 0.51872828089506218, -0.87286763824554892]]], [[[0.63517519655793753, 0.83248640824649534, -0.96421638668998866, -0.21027115956215714], [-0.65484094298333928, -0.20529300129283179, 0.30264230449111329, 0.57118632679487069], [0.94473501747170197, -0.073741122006270921, -0.16445760290671507, -0.94122715459958095]], [[-0.47320172714174435, 0.52414108821599092, -0.58822269419300688, 0.65154104107711608], [0.79073955181970002, 0.66222126451513996, 0.50406761329019711, 0.80692979137230991], [-0.44007779482511467, 0.83184141637396403, -0.59153030329212752, 0.67780297389993294]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.30986923881604533, 0.55581165940832689, 0.28993332944093009, -0.30986923881604533], [0.48402181856866533, -0.30986923881604533, 0.12059109046218897, -0.30986923881604533], [-0.30986923881604533, 0.81949583430522099, 0.917821601053878, -0.30986923881604533]], [[0.040807935475384971, 0.064255792480005569, -0.30986923881604533, -0.30986923881604533], [-0.019662050520152574, 0.44586496426970701, -0.30986923881604533, -0.18241240078470145], [-0.30986923881604533, -0.30986923881604533, -0.30986923881604533, 0.86950034082755967]]], [[[0.59960822306615458, 0.13652846774954908, 0.43957232336106244, -0.30986923881604533], [-0.30986923881604533, 0.55216812010001926, 0.71335421685640177, 0.58559993798062027], [0.70518632845671458, -0.30986923881604533, 0.49991004029291486, -0.30986923881604533]], [[-0.01327086420919299, 0.94285928205783565, 0.36084019612625773, 0.75948720217972943], [0.15534832801909948, -0.087517839294278144, -0.30986923881604533, -0.17830159519382138], [-0.30986923881604533, 0.14079322663297011, 0.51872828089506218, -0.30986923881604533]]], [[[0.63517519655793753, 0.83248640824649534, -0.30986923881604533, -0.21027115956215714], [-0.30986923881604533, -0.20529300129283179, 0.30264230449111329, 0.57118632679487069], [0.94473501747170197, -0.073741122006270921, -0.16445760290671507, -0.30986923881604533]], [[-0.30986923881604533, 0.52414108821599092, -0.30986923881604533, 0.65154104107711608], [0.79073955181970002, 0.66222126451513996, 0.50406761329019711, 0.80692979137230991], [-0.30986923881604533, 0.83184141637396403, -0.30986923881604533, 0.67780297389993294]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_float_rank0_expandedData_rank4(self):
      arg0=-0.246649004468
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.59710738457463131, 0.081437393836007343, -0.56546756227867667, 0.71443872053420754], [-0.57457972849842576, 0.99714958865565517, -0.39999443875374929, -0.68094907708339836], [0.34526888607366546, -0.29241451792195905, 0.96907276723625935, -0.2966708791455217]], [[-0.76911685555342713, 0.21001220920747699, 0.7216150128564951, 0.57784624578115795], [0.59505769173662104, -0.11354294660473241, -0.66879018252488964, 0.78630853516684396], [0.3616307103083698, -0.79300540303062417, 0.74450579924491178, -0.054555970573139767]]], [[[0.96283128026153286, -0.87620458144557722, 0.72154059555131611, 0.0082452389660727654], [-0.046764826200485832, -0.53421181085747382, -0.19205291797994817, 0.15732232154864123], [-0.64350186911984397, 0.86535072837345894, -0.089737099310619683, -0.68915912536904322]], [[-0.50600653327166056, 0.87271444603618842, 0.42777020885243378, 0.63522958101972282], [-0.62649776548253322, 0.4567028350414013, 0.97523051451214271, 0.81579778175168283], [0.20125503319935079, -0.46379240558699131, -0.31245213377702896, 0.2704000376928466]]], [[[-0.52356266147432984, 0.17857675536580886, 0.3664267659935363, -0.88731256350709842], [-0.37314717051082713, 0.54211837452172662, -0.72485428615154301, -0.60715112600284726], [-0.99925424756131687, -0.16075147245083277, -0.0025301737170317917, 0.14075502002330875]], [[-0.72510015564658348, -0.070820851711875132, -0.05397335487409638, 0.31296807923888648], [0.66626010161325921, -0.7189380142049282, -0.62806927909760879, 0.23068004843743117], [-0.70574206651459992, -0.10745983979108953, -0.16531954799426085, -0.43129597709381007]]]])+(1.-msk_arg1)*numpy.array([[[[0.87166570761445938, 0.7318590182568474, -0.80737962090297621, 0.53766528032150029], [-0.29770111995100423, -0.041729104475766832, 0.95558043277951188, 0.87068423363682523], [0.26116387820103548, 0.76929801350975846, -0.34600777484521661, 0.20956877787614303]], [[-0.27623529169912397, 0.74714116143240372, 0.67467631548270068, 0.80485965465015274], [-0.80196951310483411, 0.54024415368197976, -0.45263989703849994, -0.77389469864887639], [-0.11922685978923897, 0.39196315394661263, 0.17728969359335789, -0.38647232675584942]]], [[[0.96589541331221995, -0.80607305323690448, -0.7174848711693449, 0.37455766619796327], [0.91224675515514875, -0.8324612503371378, -0.95542285313177433, -0.11861636937415732], [0.47029221874868399, -0.40274109861783103, 0.9175933763798334, -0.43517134542523084]], [[-0.47437396776048946, -0.62388433551176026, -0.93178844269694805, 0.16109167169456717], [0.41540675061264087, 0.56983043087032437, 0.79225905558275778, -0.70471106926370264], [0.57876248922509843, -0.92178848481156983, -0.20102218593254717, -0.51116400848044763]]], [[[0.43214438484520512, -0.44676797608176422, -0.96373941940887264, -0.97610643146800369], [0.1482565940064382, -0.15856386047336768, -0.97338314492097022, -0.77655581838335452], [0.14918854112868329, 0.35190145831475594, 0.76106715747610876, 0.87042812467135966]], [[0.5937340223957881, 0.93249676000364778, 0.62934483950694187, -0.79475089188750059], [-0.7844570636858641, 0.063504498696464129, 0.80077845206462461, -0.17147808526676922], [0.77647533980748817, -0.41826398787923291, -0.93662968561496873, 0.5899829279078268]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.24664900446763838, 0.081437393836007343, -0.24664900446763838, 0.71443872053420754], [-0.24664900446763838, 0.99714958865565517, -0.24664900446763838, -0.24664900446763838], [0.34526888607366546, -0.24664900446763838, 0.96907276723625935, -0.24664900446763838]], [[-0.24664900446763838, 0.21001220920747699, 0.7216150128564951, 0.57784624578115795], [0.59505769173662104, -0.11354294660473241, -0.24664900446763838, 0.78630853516684396], [0.3616307103083698, -0.24664900446763838, 0.74450579924491178, -0.054555970573139767]]], [[[0.96283128026153286, -0.24664900446763838, 0.72154059555131611, 0.0082452389660727654], [-0.046764826200485832, -0.24664900446763838, -0.19205291797994817, 0.15732232154864123], [-0.24664900446763838, 0.86535072837345894, -0.089737099310619683, -0.24664900446763838]], [[-0.24664900446763838, 0.87271444603618842, 0.42777020885243378, 0.63522958101972282], [-0.24664900446763838, 0.4567028350414013, 0.97523051451214271, 0.81579778175168283], [0.20125503319935079, -0.24664900446763838, -0.24664900446763838, 0.2704000376928466]]], [[[-0.24664900446763838, 0.17857675536580886, 0.3664267659935363, -0.24664900446763838], [-0.24664900446763838, 0.54211837452172662, -0.24664900446763838, -0.24664900446763838], [-0.24664900446763838, -0.16075147245083277, -0.0025301737170317917, 0.14075502002330875]], [[-0.24664900446763838, -0.070820851711875132, -0.05397335487409638, 0.31296807923888648], [0.66626010161325921, -0.24664900446763838, -0.24664900446763838, 0.23068004843743117], [-0.24664900446763838, -0.10745983979108953, -0.16531954799426085, -0.24664900446763838]]]])+(1.-msk_ref)*numpy.array([[[[0.87166570761445938, 0.7318590182568474, -0.24664900446763838, 0.53766528032150029], [-0.24664900446763838, -0.041729104475766832, 0.95558043277951188, 0.87068423363682523], [0.26116387820103548, 0.76929801350975846, -0.24664900446763838, 0.20956877787614303]], [[-0.24664900446763838, 0.74714116143240372, 0.67467631548270068, 0.80485965465015274], [-0.24664900446763838, 0.54024415368197976, -0.24664900446763838, -0.24664900446763838], [-0.11922685978923897, 0.39196315394661263, 0.17728969359335789, -0.24664900446763838]]], [[[0.96589541331221995, -0.24664900446763838, -0.24664900446763838, 0.37455766619796327], [0.91224675515514875, -0.24664900446763838, -0.24664900446763838, -0.11861636937415732], [0.47029221874868399, -0.24664900446763838, 0.9175933763798334, -0.24664900446763838]], [[-0.24664900446763838, -0.24664900446763838, -0.24664900446763838, 0.16109167169456717], [0.41540675061264087, 0.56983043087032437, 0.79225905558275778, -0.24664900446763838], [0.57876248922509843, -0.24664900446763838, -0.20102218593254717, -0.24664900446763838]]], [[[0.43214438484520512, -0.24664900446763838, -0.24664900446763838, -0.24664900446763838], [0.1482565940064382, -0.15856386047336768, -0.24664900446763838, -0.24664900446763838], [0.14918854112868329, 0.35190145831475594, 0.76106715747610876, 0.87042812467135966]], [[0.5937340223957881, 0.93249676000364778, 0.62934483950694187, -0.24664900446763838], [-0.24664900446763838, 0.063504498696464129, 0.80077845206462461, -0.17147808526676922], [0.77647533980748817, -0.24664900446763838, -0.24664900446763838, 0.5899829279078268]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_float_rank0(self):
      arg0=numpy.array(-0.532218032885)
      arg1=-0.367677151081
      res=maximum(arg0,arg1)
      ref=numpy.array(-0.367677151081)
      if isinstance(res,numpy.ndarray):
          self.assertEqual(res.shape,(),"wrong shape of result.")
      else:
          self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_float_rank0(self):
      arg0=numpy.array([0.53113072385818016, -0.91297296718973731])
      arg1=0.192754148616
      res=maximum(arg0,arg1)
      ref=numpy.array([0.53113072385818016, 0.19275414861572537])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_float_rank0(self):
      arg0=numpy.array([[-0.83224495764259587, 0.67546682831068661, 0.79825688470140466, -0.51659524388186018, -0.57330212318607887], [0.43871883762893171, -0.51383061657367102, 0.046022377444661977, -0.97181480474218085, -0.064914882677348462], [0.54107834535109811, 0.52030930437850165, -0.46403217495914517, -0.40333646983529814, 0.14402492053142035], [-0.06466356036920895, -0.72560803049513556, -0.71248570650624976, 0.29239946340327383, 0.23126053673271052]])
      arg1=0.436559137295
      res=maximum(arg0,arg1)
      ref=numpy.array([[0.43655913729458451, 0.67546682831068661, 0.79825688470140466, 0.43655913729458451, 0.43655913729458451], [0.43871883762893171, 0.43655913729458451, 0.43655913729458451, 0.43655913729458451, 0.43655913729458451], [0.54107834535109811, 0.52030930437850165, 0.43655913729458451, 0.43655913729458451, 0.43655913729458451], [0.43655913729458451, 0.43655913729458451, 0.43655913729458451, 0.43655913729458451, 0.43655913729458451]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_float_rank0(self):
      arg0=numpy.array([[[0.14602536321923187, 0.3890404886953438], [0.96763189489480217, 0.31878780255153472]], [[-0.80681415429363024, 0.81150006186927182], [0.085441963781010877, 0.28294027876384797]], [[-0.90676678758245544, 0.035304671448607161], [-0.24178422686744017, -0.020920873167083132]], [[-0.85375981355743735, -0.44307345060683789], [0.0374669536660035, -0.0058344936858338858]], [[0.31755388514905247, 0.33655876978208488], [-0.60668072237969684, -0.046438959267768309]], [[-0.74363599042485151, -0.8956664888069259], [-0.53199684552981585, 0.75892708027917499]]])
      arg1=-0.149468998389
      res=maximum(arg0,arg1)
      ref=numpy.array([[[0.14602536321923187, 0.3890404886953438], [0.96763189489480217, 0.31878780255153472]], [[-0.1494689983892894, 0.81150006186927182], [0.085441963781010877, 0.28294027876384797]], [[-0.1494689983892894, 0.035304671448607161], [-0.1494689983892894, -0.020920873167083132]], [[-0.1494689983892894, -0.1494689983892894], [0.0374669536660035, -0.0058344936858338858]], [[0.31755388514905247, 0.33655876978208488], [-0.1494689983892894, -0.046438959267768309]], [[-0.1494689983892894, -0.1494689983892894], [-0.1494689983892894, 0.75892708027917499]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[0.2890956567711751, 0.0047389611017556454, 0.87892527211444582, -0.65150693309569196], [0.41621480995390114, -0.43854307594957254, -0.59815785643830077, -0.67517272581107557], [0.29083357354248873, -0.34488587447865737, -0.3534529402795854, -0.76014085074946847]], [[-0.19051880582497072, 0.85284977603728396, 0.31270253036076934, 0.37421013369344447], [0.71318255965443345, 0.92162734459590911, 0.67934988220751147, -0.42337007079808697], [0.42833893416405266, 0.30720360302238192, 0.27000152473334049, 0.95393479511777279]]], [[[-0.098035752002608278, 0.13176090528152407, 0.62934363744184929, -0.053289126771334905], [0.81346047837084501, -0.45301649520463716, -0.96631411831608749, -0.071951542993987649], [0.99690192197172367, -0.025819634205427322, 0.87468585164061108, 0.60501309254424518]], [[-0.88303485657993463, 0.46312758028512135, -0.77275723545859076, -0.067439349816685201], [0.14089517815207553, 0.56655437330646286, 0.35573146108388021, -0.9370989113947783], [-0.33072965554665301, -0.81929016865935234, -0.981755182164328, -0.80123180974034836]]], [[[0.32559736893252644, -0.15298445424344309, 0.5756357845038651, 0.54846826957121642], [0.49314026154864776, 0.40147202599848408, 0.65052530484204629, 0.78030715854096466], [0.11772129008710874, 0.97339684272768734, -0.46213970918704739, 0.64480146515364178]], [[-0.489873441461109, 0.67676986710802556, 0.57320598982638238, -0.081510204873377567], [0.14129642797645747, -0.88117171552134099, 0.62120016125873723, 0.62570138942740905], [-0.4571545505832082, 0.18504471982789994, 0.269612515446229, -0.92783580872646998]]]])
      arg1=0.44439427692
      res=maximum(arg0,arg1)
      ref=numpy.array([[[[0.44439427692047384, 0.44439427692047384, 0.87892527211444582, 0.44439427692047384], [0.44439427692047384, 0.44439427692047384, 0.44439427692047384, 0.44439427692047384], [0.44439427692047384, 0.44439427692047384, 0.44439427692047384, 0.44439427692047384]], [[0.44439427692047384, 0.85284977603728396, 0.44439427692047384, 0.44439427692047384], [0.71318255965443345, 0.92162734459590911, 0.67934988220751147, 0.44439427692047384], [0.44439427692047384, 0.44439427692047384, 0.44439427692047384, 0.95393479511777279]]], [[[0.44439427692047384, 0.44439427692047384, 0.62934363744184929, 0.44439427692047384], [0.81346047837084501, 0.44439427692047384, 0.44439427692047384, 0.44439427692047384], [0.99690192197172367, 0.44439427692047384, 0.87468585164061108, 0.60501309254424518]], [[0.44439427692047384, 0.46312758028512135, 0.44439427692047384, 0.44439427692047384], [0.44439427692047384, 0.56655437330646286, 0.44439427692047384, 0.44439427692047384], [0.44439427692047384, 0.44439427692047384, 0.44439427692047384, 0.44439427692047384]]], [[[0.44439427692047384, 0.44439427692047384, 0.5756357845038651, 0.54846826957121642], [0.49314026154864776, 0.44439427692047384, 0.65052530484204629, 0.78030715854096466], [0.44439427692047384, 0.97339684272768734, 0.44439427692047384, 0.64480146515364178]], [[0.44439427692047384, 0.67676986710802556, 0.57320598982638238, 0.44439427692047384], [0.44439427692047384, 0.44439427692047384, 0.62120016125873723, 0.62570138942740905], [0.44439427692047384, 0.44439427692047384, 0.44439427692047384, 0.44439427692047384]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_array_rank0(self):
      arg0=numpy.array(-0.595911728088)
      arg1=numpy.array(-0.259958577128)
      res=maximum(arg0,arg1)
      ref=numpy.array(-0.259958577128)
      if isinstance(res,numpy.ndarray):
          self.assertEqual(res.shape,(),"wrong shape of result.")
      else:
          self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_array_rank0(self):
      arg0=numpy.array([0.70448785703212291, 0.83900014998124073])
      arg1=numpy.array(-0.728326601622)
      res=maximum(arg0,arg1)
      ref=numpy.array([0.70448785703212291, 0.83900014998124073])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_array_rank0(self):
      arg0=numpy.array([[0.58907572104688044, -0.30767970415015888, -0.78138846520643046, -0.15168954291983616, 0.31446212443091293], [0.074998472598995392, -0.9250742963204186, 0.45334932675665041, -0.95916572344077733, -0.99342314865250936], [-0.79251531920902263, 0.44321789574157977, 0.62908133704628444, 0.24472622296386093, 0.29465980693370675], [-0.5523390670941164, -0.096556923965434915, -0.97293609728291464, 0.4811104330547229, 0.11676047360942654]])
      arg1=numpy.array(-0.637835089534)
      res=maximum(arg0,arg1)
      ref=numpy.array([[0.58907572104688044, -0.30767970415015888, -0.63783508953404722, -0.15168954291983616, 0.31446212443091293], [0.074998472598995392, -0.63783508953404722, 0.45334932675665041, -0.63783508953404722, -0.63783508953404722], [-0.63783508953404722, 0.44321789574157977, 0.62908133704628444, 0.24472622296386093, 0.29465980693370675], [-0.5523390670941164, -0.096556923965434915, -0.63783508953404722, 0.4811104330547229, 0.11676047360942654]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_array_rank0(self):
      arg0=numpy.array([[[0.26078403166817066, -0.88883099092846662], [0.2962776357981165, 0.33765471443325268]], [[0.61994569524479037, -0.86070803156387798], [0.40887497854892496, -0.95032693825538606]], [[-0.6230603384583091, 0.84641044210513128], [0.072929215696905469, 0.63605426925408093]], [[0.82476138834838242, -0.87239621000692358], [0.9136576981759601, 0.012874827321107896]], [[0.6527601185704035, 0.45834972014119924], [-0.20281376602514722, 0.78801993661046477]], [[-0.84382527553821407, 0.69859016456466971], [0.95609346718482136, -0.9642394043877307]]])
      arg1=numpy.array(-0.0639685246813)
      res=maximum(arg0,arg1)
      ref=numpy.array([[[0.26078403166817066, -0.063968524681262906], [0.2962776357981165, 0.33765471443325268]], [[0.61994569524479037, -0.063968524681262906], [0.40887497854892496, -0.063968524681262906]], [[-0.063968524681262906, 0.84641044210513128], [0.072929215696905469, 0.63605426925408093]], [[0.82476138834838242, -0.063968524681262906], [0.9136576981759601, 0.012874827321107896]], [[0.6527601185704035, 0.45834972014119924], [-0.063968524681262906, 0.78801993661046477]], [[-0.063968524681262906, 0.69859016456466971], [0.95609346718482136, -0.063968524681262906]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[-0.49178507725843357, -0.95412613874160401, -0.89953408681402758, -0.4182783638072558], [-0.12851206262547121, 0.97771672278447719, 0.095498689842355322, -0.63677823186616767], [-0.31171543678709579, -0.24066261014019164, 0.8884558662985651, -0.86534369235026132]], [[0.41073749486304023, -0.75082798331554312, -0.021514401962046348, -0.4552145355922177], [0.025376418086104691, 0.94693831924338046, 0.72528013847070816, 0.5590032330140231], [-0.63961957547864201, 0.38553731067491559, 0.47766722193193556, 0.22722302846704578]]], [[[-0.8552086040279212, -0.028628848798257334, 0.39615293686569841, 0.60472409304678898], [-0.76179395489150892, 0.34743550408059365, 0.84900306720985297, -0.32754549604906047], [0.96774618220062791, -0.17510456094746396, -0.83577780367198806, -0.28595508510654843]], [[-0.72033121788336163, -0.96380753221847604, 0.64121075422201179, 0.50807412144193731], [-0.22520904548844145, -0.79626857735444134, 0.6727388150821505, -0.97429723888611686], [-0.3912584771763985, 0.017073154831940274, -0.78645964556472814, 0.063579858350630403]]], [[[0.49275761913585359, 0.68061912084321108, 0.27488873542534376, -0.91020336944963454], [0.38754903931740148, -0.095266564605377857, -0.34983002073831537, -0.98686346316121343], [-0.57870251452427079, 0.019782802975875313, -0.99030465294393522, 0.36519674435393079]], [[0.36201122081142789, 0.89247761722577512, 0.88862029406919207, 0.65118385314611027], [0.62444020668337985, 0.29010577239708102, 0.50090883083435256, -0.86322012454452723], [-0.25882379276404111, 0.50333245341748101, -0.98127424266544994, 0.32881151993132085]]]])
      arg1=numpy.array(0.54009693064)
      res=maximum(arg0,arg1)
      ref=numpy.array([[[[0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.97771672278447719, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.8884558662985651, 0.54009693064018083]], [[0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.94693831924338046, 0.72528013847070816, 0.5590032330140231], [0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083]]], [[[0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.60472409304678898], [0.54009693064018083, 0.54009693064018083, 0.84900306720985297, 0.54009693064018083], [0.96774618220062791, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083]], [[0.54009693064018083, 0.54009693064018083, 0.64121075422201179, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.6727388150821505, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083]]], [[[0.54009693064018083, 0.68061912084321108, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083]], [[0.54009693064018083, 0.89247761722577512, 0.88862029406919207, 0.65118385314611027], [0.62444020668337985, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083], [0.54009693064018083, 0.54009693064018083, 0.54009693064018083, 0.54009693064018083]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(-0.12010332885)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.246027149377)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-0.12010332885)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([-0.95932830843055505, 0.18950193862059872])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.920292144762)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.92029214476235244, 0.18950193862059872])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[0.30088865432610801, -0.49045308105451935, 0.49998687647978324, -0.5043343548276058, -0.76352630779203201], [-0.65678898472721525, -0.38338721730936975, 0.72073131734926532, 0.34776513180427382, 0.94312963079951628], [0.96205564096436347, 0.15762892075645407, 0.48298080817733147, -0.60009884409183911, 0.35402981599768824], [0.44909106753608219, -0.02511293307884066, 0.69859396147005937, -0.42889283801171785, -0.58611648158759833]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.373183154648)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.37318315464849028, 0.37318315464849028, 0.49998687647978324, 0.37318315464849028, 0.37318315464849028], [0.37318315464849028, 0.37318315464849028, 0.72073131734926532, 0.37318315464849028, 0.94312963079951628], [0.96205564096436347, 0.37318315464849028, 0.48298080817733147, 0.37318315464849028, 0.37318315464849028], [0.44909106753608219, 0.37318315464849028, 0.69859396147005937, 0.37318315464849028, 0.37318315464849028]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[0.36396791109765414, -0.08330143784202626], [-0.20497744418340713, -0.63338393016692551]], [[-0.60496788043358785, -0.34910307601449686], [0.89311096736079221, 0.83537010849383497]], [[0.17354190245914469, -0.37208307861539325], [0.36343762172766292, 0.1496343652379688]], [[-0.95049259704684275, 0.20280821539940397], [0.18376026656017297, 0.34535160775388185]], [[-0.52413579876018224, -0.064824980130857357], [-0.5543617242149792, 0.11168558960890507]], [[-0.088753737311743963, 0.67201033300979862], [0.95061730590983862, 0.25784978966847749]]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.248549928852)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.36396791109765414, 0.24854992885239846], [0.24854992885239846, 0.24854992885239846]], [[0.24854992885239846, 0.24854992885239846], [0.89311096736079221, 0.83537010849383497]], [[0.24854992885239846, 0.24854992885239846], [0.36343762172766292, 0.24854992885239846]], [[0.24854992885239846, 0.24854992885239846], [0.24854992885239846, 0.34535160775388185]], [[0.24854992885239846, 0.24854992885239846], [0.24854992885239846, 0.24854992885239846]], [[0.24854992885239846, 0.67201033300979862], [0.95061730590983862, 0.25784978966847749]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[-0.99783734493539167, 0.93221628541660539, 0.91739007036958187, 0.17412987360539578], [0.8826971153609402, -0.10363858424513261, 0.34548814341091938, 0.51215656416203448], [0.016658545625886534, -0.012842449639625553, 0.17979122131168523, -0.67861237363219296]], [[0.51887494070803042, -0.073449427073168261, -0.069672952502191521, -0.82912309601618817], [0.87934065402431805, 0.80987886911230911, -0.18655551854370866, 0.76437067323631447], [0.7010984626936092, -0.65985831916807891, 0.68992206191709848, -0.82979817139792522]]], [[[0.51819922997563461, -0.54837890985730309, 0.44640834054324952, 0.3450034673133926], [-0.36058576044087731, 0.94431157210331707, -0.65233922911878395, 0.87237679742723495], [0.85356174139175089, 0.93268981921785055, 0.23511863710669312, -0.30862863582106992]], [[0.77404557587572698, 0.14724927186620507, 0.049443973895827664, 0.53935418395973755], [-0.69201435596394956, 0.17137097192210682, -0.67144154924437993, -0.70573887285393155], [-0.85460956391237519, -0.48604945173485148, 0.79847177238289135, -0.43722683723280498]]], [[[-0.70029815629498615, -0.0043327746067951534, 0.70002045844916161, 0.18344272718592047], [-0.10153380140243762, 0.095685144607773331, 0.65073337631981665, -0.23902831769201138], [-0.35840578736462159, 0.35575178107307925, -0.84190506332471604, -0.10043141709420578]], [[0.54995532515267564, -0.71750031361308975, 0.88839972599875372, -0.6533637185735155], [-0.017870534911850644, -0.87771051574766523, 0.37553987141326695, -0.11186989630308597], [0.59008553423755838, -0.58524730169973482, 0.91504377758136068, -0.5599777283515941]]]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.856304315214)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.85630431521365979, 0.93221628541660539, 0.91739007036958187, 0.17412987360539578], [0.8826971153609402, -0.10363858424513261, 0.34548814341091938, 0.51215656416203448], [0.016658545625886534, -0.012842449639625553, 0.17979122131168523, -0.67861237363219296]], [[0.51887494070803042, -0.073449427073168261, -0.069672952502191521, -0.82912309601618817], [0.87934065402431805, 0.80987886911230911, -0.18655551854370866, 0.76437067323631447], [0.7010984626936092, -0.65985831916807891, 0.68992206191709848, -0.82979817139792522]]], [[[0.51819922997563461, -0.54837890985730309, 0.44640834054324952, 0.3450034673133926], [-0.36058576044087731, 0.94431157210331707, -0.65233922911878395, 0.87237679742723495], [0.85356174139175089, 0.93268981921785055, 0.23511863710669312, -0.30862863582106992]], [[0.77404557587572698, 0.14724927186620507, 0.049443973895827664, 0.53935418395973755], [-0.69201435596394956, 0.17137097192210682, -0.67144154924437993, -0.70573887285393155], [-0.85460956391237519, -0.48604945173485148, 0.79847177238289135, -0.43722683723280498]]], [[[-0.70029815629498615, -0.0043327746067951534, 0.70002045844916161, 0.18344272718592047], [-0.10153380140243762, 0.095685144607773331, 0.65073337631981665, -0.23902831769201138], [-0.35840578736462159, 0.35575178107307925, -0.84190506332471604, -0.10043141709420578]], [[0.54995532515267564, -0.71750031361308975, 0.88839972599875372, -0.6533637185735155], [-0.017870534911850644, -0.85630431521365979, 0.37553987141326695, -0.11186989630308597], [0.59008553423755838, -0.58524730169973482, 0.91504377758136068, -0.5599777283515941]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_constData_rank0(self):
      arg0=numpy.array(0.275719067126)
      arg1=Data(0.379978099496,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(0.379978099496,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_constData_rank0(self):
      arg0=numpy.array([0.23412985120029073, 0.96842799591312634])
      arg1=Data(0.0282351777389,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.23412985120029073, 0.96842799591312634]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_constData_rank0(self):
      arg0=numpy.array([[0.75083228363058563, 0.14021129209291994, 0.66454891543379668, 0.52438137383900618, 0.59989233019891586], [-0.64926806752121502, -0.31973316364062088, -0.15677459149134809, 0.94726185071047286, 0.77006499756531577], [0.96436511217108922, 0.10798486901631477, 0.49614119848238003, -0.58809677000944571, 0.40684393221640747], [-0.33426541098798523, -0.99641768740611458, 0.019371391555169426, 0.41032148071109376, 0.55944773037045037]])
      arg1=Data(0.0504551040829,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.75083228363058563, 0.14021129209291994, 0.66454891543379668, 0.52438137383900618, 0.59989233019891586], [0.050455104082914204, 0.050455104082914204, 0.050455104082914204, 0.94726185071047286, 0.77006499756531577], [0.96436511217108922, 0.10798486901631477, 0.49614119848238003, 0.050455104082914204, 0.40684393221640747], [0.050455104082914204, 0.050455104082914204, 0.050455104082914204, 0.41032148071109376, 0.55944773037045037]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[0.68078035361717415, 0.77513835622840355], [0.88778377916351281, -0.59991037059023289]], [[-0.31479263876172614, 0.52689690629824115], [-0.32445682623377969, 0.70068635269972779]], [[0.7068855302553303, -0.90632512202385662], [-0.69310089072096459, 0.21580652641517606]], [[0.18682929631820056, 0.98487526483307875], [-0.80732797686141189, -0.74387425497735205]], [[-0.94959049904561454, -0.26401122319745207], [0.97666196551554485, 0.13397037446463989]], [[-0.32237452231000963, 0.23366086287779164], [-0.66314084754224023, 0.061657994330439836]]])
      arg1=Data(0.887177842883,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.88717784288295465, 0.88717784288295465], [0.88778377916351281, 0.88717784288295465]], [[0.88717784288295465, 0.88717784288295465], [0.88717784288295465, 0.88717784288295465]], [[0.88717784288295465, 0.88717784288295465], [0.88717784288295465, 0.88717784288295465]], [[0.88717784288295465, 0.98487526483307875], [0.88717784288295465, 0.88717784288295465]], [[0.88717784288295465, 0.88717784288295465], [0.97666196551554485, 0.88717784288295465]], [[0.88717784288295465, 0.88717784288295465], [0.88717784288295465, 0.88717784288295465]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[-0.44596394506288406, -0.99451216053143821, 0.80947207669238974, 0.53381678739762761], [0.24988481018214626, 0.97340528016730477, -0.88181308417483706, -0.10501736634476666], [0.67231055089947356, -0.41705694674616267, -0.99130432976066873, -0.76183835148666756]], [[0.87564340019720688, -0.56545607723354552, 0.35986228734119252, -0.95148055206041304], [0.54888663455682729, 0.97420887940931578, 0.058263938512163049, 0.76370352854831602], [-0.82673099109694004, 0.9659484007501713, -0.92707003031757185, -0.29800942664995955]]], [[[0.22672462889085043, 0.77216678282924223, -0.68765485895438716, -0.41035411530512622], [-0.10574234135684346, 0.41868370912220354, -0.42894043779354374, 0.14689038755897377], [0.80637358889951183, 0.0048710336014621269, 0.95227700237370771, 0.67742226677379058]], [[0.33943135111692935, 0.051854083713816834, -0.11097498559618657, -0.28695360058894792], [0.25170868190713924, 0.51759300366308403, 0.045719489002853608, 0.85414177693285209], [-0.62313668843495651, -0.65158905094111397, -0.35965515542399928, -0.91881459559837508]]], [[[0.50411396804110731, -0.35071960896573962, 0.85463428495185267, 0.047161400398034514], [0.96936117001305488, -0.23708858547871281, -0.97958490805427312, 0.19911749296919523], [0.3558068103726566, 0.83166317419688163, 0.094348126909065133, 0.53229454234478757]], [[-0.95064691521898426, 0.40080396732915236, 0.33234699751344388, -0.9871206815396758], [0.98903106268798591, 0.091842667447183279, 0.29485522713344281, -0.33705972294401088], [0.29805334379644233, -0.9604786924125317, -0.73602864043687255, -0.40470873942057994]]]])
      arg1=Data(-0.176207534638,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.17620753463766059, -0.17620753463766059, 0.80947207669238974, 0.53381678739762761], [0.24988481018214626, 0.97340528016730477, -0.17620753463766059, -0.10501736634476666], [0.67231055089947356, -0.17620753463766059, -0.17620753463766059, -0.17620753463766059]], [[0.87564340019720688, -0.17620753463766059, 0.35986228734119252, -0.17620753463766059], [0.54888663455682729, 0.97420887940931578, 0.058263938512163049, 0.76370352854831602], [-0.17620753463766059, 0.9659484007501713, -0.17620753463766059, -0.17620753463766059]]], [[[0.22672462889085043, 0.77216678282924223, -0.17620753463766059, -0.17620753463766059], [-0.10574234135684346, 0.41868370912220354, -0.17620753463766059, 0.14689038755897377], [0.80637358889951183, 0.0048710336014621269, 0.95227700237370771, 0.67742226677379058]], [[0.33943135111692935, 0.051854083713816834, -0.11097498559618657, -0.17620753463766059], [0.25170868190713924, 0.51759300366308403, 0.045719489002853608, 0.85414177693285209], [-0.17620753463766059, -0.17620753463766059, -0.17620753463766059, -0.17620753463766059]]], [[[0.50411396804110731, -0.17620753463766059, 0.85463428495185267, 0.047161400398034514], [0.96936117001305488, -0.17620753463766059, -0.17620753463766059, 0.19911749296919523], [0.3558068103726566, 0.83166317419688163, 0.094348126909065133, 0.53229454234478757]], [[-0.17620753463766059, 0.40080396732915236, 0.33234699751344388, -0.17620753463766059], [0.98903106268798591, 0.091842667447183279, 0.29485522713344281, -0.17620753463766059], [0.29805334379644233, -0.17620753463766059, -0.17620753463766059, -0.17620753463766059]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(-0.239808661002)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.499507234748)+(1.-msk_arg1)*(-0.671148207596)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.239808661002)+(1.-msk_ref)*(-0.239808661002)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([-0.097106554962706371, -0.53299957388591124])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.48432167856)+(1.-msk_arg1)*(-0.25985897402)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.097106554962706371, -0.48432167856004193])+(1.-msk_ref)*numpy.array([-0.097106554962706371, -0.25985897401999281])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[-0.13167949633164855, -0.0083337422827065577, -0.38573640576223611, -0.069870464900080531, 0.77840356126391153], [0.62218334866236424, 0.68483355583508021, -0.48863487618037715, -0.0082838053084859364, -0.33346342674536089], [-0.89360989325312534, -0.9751359066636649, -0.0084300747210315219, -0.96657583767552402, -0.10309752389903437], [-0.22842150326198207, -0.49737293664485316, -0.47025074030093283, -0.67836806796837679, 0.99564770637844346]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.0781036984237)+(1.-msk_arg1)*(-0.0843238721184)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.77840356126391153], [0.62218334866236424, 0.68483355583508021, 0.078103698423652057, 0.078103698423652057, 0.078103698423652057], [0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.078103698423652057], [0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.078103698423652057, 0.99564770637844346]])+(1.-msk_ref)*numpy.array([[-0.084323872118388099, -0.0083337422827065577, -0.084323872118388099, -0.069870464900080531, 0.77840356126391153], [0.62218334866236424, 0.68483355583508021, -0.084323872118388099, -0.0082838053084859364, -0.084323872118388099], [-0.084323872118388099, -0.084323872118388099, -0.0084300747210315219, -0.084323872118388099, -0.084323872118388099], [-0.084323872118388099, -0.084323872118388099, -0.084323872118388099, -0.084323872118388099, 0.99564770637844346]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[-0.70161399966991178, 0.33853877023786727], [-0.76494633110216448, -0.93684954048568492]], [[-0.65137354058954, 0.26553439129603973], [0.64552443308190122, -0.60830143039855877]], [[0.8023162924896885, -0.32038038872912455], [-0.47547406933615965, 0.27454091657539204]], [[-0.9672523328014897, -0.71919678369683115], [-0.35423678746927645, 0.81975223582448642]], [[-0.81403739453676183, 0.0020767234774008436], [0.94253026200138423, 0.98275512168780477]], [[-0.24210704831235974, 0.71604514999739655], [-0.55274691970980117, 0.99831912241667875]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.0102246658782)+(1.-msk_arg1)*(0.652230128332)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.010224665878218353, 0.33853877023786727], [0.010224665878218353, 0.010224665878218353]], [[0.010224665878218353, 0.26553439129603973], [0.64552443308190122, 0.010224665878218353]], [[0.8023162924896885, 0.010224665878218353], [0.010224665878218353, 0.27454091657539204]], [[0.010224665878218353, 0.010224665878218353], [0.010224665878218353, 0.81975223582448642]], [[0.010224665878218353, 0.010224665878218353], [0.94253026200138423, 0.98275512168780477]], [[0.010224665878218353, 0.71604514999739655], [0.010224665878218353, 0.99831912241667875]]])+(1.-msk_ref)*numpy.array([[[0.65223012833196758, 0.65223012833196758], [0.65223012833196758, 0.65223012833196758]], [[0.65223012833196758, 0.65223012833196758], [0.65223012833196758, 0.65223012833196758]], [[0.8023162924896885, 0.65223012833196758], [0.65223012833196758, 0.65223012833196758]], [[0.65223012833196758, 0.65223012833196758], [0.65223012833196758, 0.81975223582448642]], [[0.65223012833196758, 0.65223012833196758], [0.94253026200138423, 0.98275512168780477]], [[0.65223012833196758, 0.71604514999739655], [0.65223012833196758, 0.99831912241667875]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[0.11857521935136139, 0.093929496673029522, -0.86185255457483856, 0.66859958438889944], [-0.71414593797980697, 0.11258374469683963, 0.94835492072493288, -0.68730039028716172], [-0.69653609424045704, 0.079954029214261757, -0.091180257555399713, -0.1442422632927951]], [[0.26155223948894291, -0.27259910943079357, 0.74661493838533688, 0.53585345987130495], [-0.60022580287227467, -0.48790336997422057, 0.94579083650142803, -0.8863996264681393], [-0.48285648955978755, 0.43421403658329649, 0.12057557891417603, -0.48016460023246776]]], [[[0.82622400614598757, 0.89451467594773959, -0.42386713423727307, 0.4292564116088935], [0.15325746129096651, -0.40650587605155497, 0.48852383833911195, -0.12133478623760663], [0.79062049417854996, -0.18313225818980827, -0.97553885725628553, -0.8652842476948277]], [[-0.05398981985994844, 0.25155972652674774, 0.58065613688539863, 0.49996739942398416], [-0.96741564092273324, 0.25704723590663892, 0.21718381354462557, -0.87988046847383972], [0.17224341707309865, 0.2585605917314151, -0.61262203743428256, 0.088679962718663852]]], [[[0.95856449568455826, -0.21525011988362075, -0.87880681141864181, 0.74102075636768272], [0.41959007487547573, 0.80453495515335471, 0.48817109722463581, -0.11806636970967643], [0.90075904019500075, 0.47717994916605289, 0.35995395169862054, 0.40418049684201307]], [[0.28919635447948355, -0.28199908853065292, -0.051524269447178916, 0.39147412124693748], [-0.21236681046692141, -0.35585295155544783, -0.4325141024321022, 0.50366801462727206], [-0.69662947289754285, -0.10168687807009102, -0.74795243165887348, -0.26529343675798978]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.933797791441)+(1.-msk_arg1)*(-0.798477769938)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.11857521935136139, 0.093929496673029522, -0.86185255457483856, 0.66859958438889944], [-0.71414593797980697, 0.11258374469683963, 0.94835492072493288, -0.68730039028716172], [-0.69653609424045704, 0.079954029214261757, -0.091180257555399713, -0.1442422632927951]], [[0.26155223948894291, -0.27259910943079357, 0.74661493838533688, 0.53585345987130495], [-0.60022580287227467, -0.48790336997422057, 0.94579083650142803, -0.8863996264681393], [-0.48285648955978755, 0.43421403658329649, 0.12057557891417603, -0.48016460023246776]]], [[[0.82622400614598757, 0.89451467594773959, -0.42386713423727307, 0.4292564116088935], [0.15325746129096651, -0.40650587605155497, 0.48852383833911195, -0.12133478623760663], [0.79062049417854996, -0.18313225818980827, -0.93379779144110442, -0.8652842476948277]], [[-0.05398981985994844, 0.25155972652674774, 0.58065613688539863, 0.49996739942398416], [-0.93379779144110442, 0.25704723590663892, 0.21718381354462557, -0.87988046847383972], [0.17224341707309865, 0.2585605917314151, -0.61262203743428256, 0.088679962718663852]]], [[[0.95856449568455826, -0.21525011988362075, -0.87880681141864181, 0.74102075636768272], [0.41959007487547573, 0.80453495515335471, 0.48817109722463581, -0.11806636970967643], [0.90075904019500075, 0.47717994916605289, 0.35995395169862054, 0.40418049684201307]], [[0.28919635447948355, -0.28199908853065292, -0.051524269447178916, 0.39147412124693748], [-0.21236681046692141, -0.35585295155544783, -0.4325141024321022, 0.50366801462727206], [-0.69662947289754285, -0.10168687807009102, -0.74795243165887348, -0.26529343675798978]]]])+(1.-msk_ref)*numpy.array([[[[0.11857521935136139, 0.093929496673029522, -0.79847776993817776, 0.66859958438889944], [-0.71414593797980697, 0.11258374469683963, 0.94835492072493288, -0.68730039028716172], [-0.69653609424045704, 0.079954029214261757, -0.091180257555399713, -0.1442422632927951]], [[0.26155223948894291, -0.27259910943079357, 0.74661493838533688, 0.53585345987130495], [-0.60022580287227467, -0.48790336997422057, 0.94579083650142803, -0.79847776993817776], [-0.48285648955978755, 0.43421403658329649, 0.12057557891417603, -0.48016460023246776]]], [[[0.82622400614598757, 0.89451467594773959, -0.42386713423727307, 0.4292564116088935], [0.15325746129096651, -0.40650587605155497, 0.48852383833911195, -0.12133478623760663], [0.79062049417854996, -0.18313225818980827, -0.79847776993817776, -0.79847776993817776]], [[-0.05398981985994844, 0.25155972652674774, 0.58065613688539863, 0.49996739942398416], [-0.79847776993817776, 0.25704723590663892, 0.21718381354462557, -0.79847776993817776], [0.17224341707309865, 0.2585605917314151, -0.61262203743428256, 0.088679962718663852]]], [[[0.95856449568455826, -0.21525011988362075, -0.79847776993817776, 0.74102075636768272], [0.41959007487547573, 0.80453495515335471, 0.48817109722463581, -0.11806636970967643], [0.90075904019500075, 0.47717994916605289, 0.35995395169862054, 0.40418049684201307]], [[0.28919635447948355, -0.28199908853065292, -0.051524269447178916, 0.39147412124693748], [-0.21236681046692141, -0.35585295155544783, -0.4325141024321022, 0.50366801462727206], [-0.69662947289754285, -0.10168687807009102, -0.74795243165887348, -0.26529343675798978]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_array_rank1(self):
      arg0=numpy.array(0.174025779864)
      arg1=numpy.array([-0.80468496847716819, -0.55702002599440315])
      res=maximum(arg0,arg1)
      ref=numpy.array([0.17402577986423351, 0.17402577986423351])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_array_rank1(self):
      arg0=numpy.array([0.40031346790971001, 0.42660221126313203])
      arg1=numpy.array([-0.60356690978921446, 0.38041619399718063])
      res=maximum(arg0,arg1)
      ref=numpy.array([0.40031346790971001, 0.42660221126313203])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(0.155960285467)
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([-0.35545113165367748, 0.93267615556383454])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.15596028546704765, 0.93267615556383454])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([-0.35905112955801766, -0.57102630812379984])
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([0.50192775602939554, -0.7375470058295488])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.50192775602939554, -0.57102630812379984])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_constData_rank1(self):
      arg0=numpy.array(-0.232441816441)
      arg1=Data(numpy.array([0.7042628437324705, -0.052385211928024367]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.7042628437324705, -0.052385211928024367]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_constData_rank1(self):
      arg0=numpy.array([0.43161703003093366, -0.69479066934103839])
      arg1=Data(numpy.array([0.0397394354531031, 0.99784298068228838]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.43161703003093366, 0.99784298068228838]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(0.531197916018)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.10695530878160886, -0.091570793259940553])+(1.-msk_arg1)*numpy.array([0.19175738089028238, -0.89275588159527008])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.53119791601751842, 0.53119791601751842])+(1.-msk_ref)*numpy.array([0.53119791601751842, 0.53119791601751842])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([0.16613419849800959, 0.79547150938095146])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.76819288363166582, 0.064091176986368348])+(1.-msk_arg1)*numpy.array([-0.37616391703349095, -0.52746213013335952])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.16613419849800959, 0.79547150938095146])+(1.-msk_ref)*numpy.array([0.16613419849800959, 0.79547150938095146])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_array_rank2(self):
      arg0=numpy.array(0.386947484668)
      arg1=numpy.array([[0.87213326940203939, -0.38189636586633791, -0.72473189333280996, -0.21432483890478471, 0.88028312911208029], [0.2064306399504563, 0.30698465546256481, -0.43583358859083776, -0.70245985617074291, -0.31221616245670569], [0.33731000862177352, -0.22831708789872907, -0.776015520692938, 0.90598610188778328, -0.45012002694087161], [-0.97538226851945575, 0.79531267578818832, -0.52616844351177017, 0.30697695091792299, -0.82183015670659132]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[0.87213326940203939, 0.38694748466821971, 0.38694748466821971, 0.38694748466821971, 0.88028312911208029], [0.38694748466821971, 0.38694748466821971, 0.38694748466821971, 0.38694748466821971, 0.38694748466821971], [0.38694748466821971, 0.38694748466821971, 0.38694748466821971, 0.90598610188778328, 0.38694748466821971], [0.38694748466821971, 0.79531267578818832, 0.38694748466821971, 0.38694748466821971, 0.38694748466821971]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_array_rank2(self):
      arg0=numpy.array([[-0.83417152417968299, 0.83688796349341943, -0.27671759181162559, -0.67424349443288167, -0.42278111327171874], [0.99320754959522506, -0.82405717801872003, -0.76061490742008808, 0.5386983552951472, -0.90805452020755628], [-0.54835914387487272, -0.87502977528875259, 0.6410277115607339, 0.33439192634626114, 0.053433089463401418], [0.32291094963350808, -0.30495569379510146, 0.5766949489189841, 0.12613166199354442, -0.36970324165010404]])
      arg1=numpy.array([[0.71480188355203733, -0.80922817795802859, -0.19167944208480336, -0.40332523244781227, 0.40664695590425337], [-0.24805869736740882, 0.76973514026691303, 0.080936864295739319, 0.95644003803365796, -0.62680571189350576], [0.094574552319768079, 0.45665785511001311, -0.40597207626956666, 0.74668608783219925, -0.24298652271704668], [0.51320676722505087, 0.46491077097585576, -0.12305996770430627, -0.35470356431074013, -0.8913618557905405]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[0.71480188355203733, 0.83688796349341943, -0.19167944208480336, -0.40332523244781227, 0.40664695590425337], [0.99320754959522506, 0.76973514026691303, 0.080936864295739319, 0.95644003803365796, -0.62680571189350576], [0.094574552319768079, 0.45665785511001311, 0.6410277115607339, 0.74668608783219925, 0.053433089463401418], [0.51320676722505087, 0.46491077097585576, 0.5766949489189841, 0.12613166199354442, -0.36970324165010404]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(0.0585342737601)
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[0.46739135447000035, 0.7066282271423745, 0.31262155030569505, -0.59369434923157582, -0.22660499405718504], [-0.61766192494996508, -0.081760657275070869, -0.067509612259605278, 0.4190048649427387, -0.15975389590294209], [0.21426556410700281, -0.97743004027967473, 0.65277672017493171, -0.98564432738868168, -0.72243447254043125], [-0.59624081374608928, 0.51147747479299821, 0.28289272027701529, 0.040753837045484076, -0.24614723134031191]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.46739135447000035, 0.7066282271423745, 0.31262155030569505, 0.058534273760069366, 0.058534273760069366], [0.058534273760069366, 0.058534273760069366, 0.058534273760069366, 0.4190048649427387, 0.058534273760069366], [0.21426556410700281, 0.058534273760069366, 0.65277672017493171, 0.058534273760069366, 0.058534273760069366], [0.058534273760069366, 0.51147747479299821, 0.28289272027701529, 0.058534273760069366, 0.058534273760069366]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-0.1509922763531284, 0.26819583692593696, 0.13792062666516114, -0.67009659571616176, -0.28911428012327733], [0.97882363829635577, -0.14326364233543631, 0.75490549457928258, -0.10719645951684331, 0.74154785113308952], [0.60803030467184938, -0.27344716605735986, 0.009358524325833617, 0.95425171168249556, 0.30063797723589625], [-0.18029419108797407, 0.98381786341325128, 0.42691021623467118, -0.31226513329620165, -0.15802173113347839]])
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.73729156903222304, -0.35802223684371048, 0.36372497898024969, -0.39546935913517345, -0.70641158310236873], [-0.88099494570526349, 0.27267988330366366, 0.33437154258946, 0.11222753137279962, 0.51346492656917175], [-0.190230441087887, -0.37688552516163676, -0.98854058326426242, -0.20200240934513003, -0.51381327245111152], [0.80204114089677492, 0.61620728774239142, 0.045084039107120111, -0.23814078195863764, 0.88369890223672165]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.1509922763531284, 0.26819583692593696, 0.36372497898024969, -0.39546935913517345, -0.28911428012327733], [0.97882363829635577, 0.27267988330366366, 0.75490549457928258, 0.11222753137279962, 0.74154785113308952], [0.60803030467184938, -0.27344716605735986, 0.009358524325833617, 0.95425171168249556, 0.30063797723589625], [0.80204114089677492, 0.98381786341325128, 0.42691021623467118, -0.23814078195863764, 0.88369890223672165]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_constData_rank2(self):
      arg0=numpy.array(-0.304885788971)
      arg1=Data(numpy.array([[-0.22172548151150773, -0.93904306211217503, 0.25957365759321482, 0.96193812818412838, -0.3025989293684388], [0.15322972106506172, -0.17624868576712505, -0.96966588216584459, -0.34755630024821937, -0.80746601605652324], [-0.29018139429836909, 0.43809370308475382, 0.11270049169675955, -0.082055163241442131, 0.54761620786381182], [0.48410554687630114, 0.36722666263652659, -0.013480413011818015, 0.05569938806142205, 0.87738343744750513]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[-0.22172548151150773, -0.30488578897068686, 0.25957365759321482, 0.96193812818412838, -0.3025989293684388], [0.15322972106506172, -0.17624868576712505, -0.30488578897068686, -0.30488578897068686, -0.30488578897068686], [-0.29018139429836909, 0.43809370308475382, 0.11270049169675955, -0.082055163241442131, 0.54761620786381182], [0.48410554687630114, 0.36722666263652659, -0.013480413011818015, 0.05569938806142205, 0.87738343744750513]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-0.076997964172731859, -0.67651472726038619, 0.79667022354874106, -0.84019816840575978, 0.94360680273325759], [0.47038093139447756, -0.59450503868551907, -0.023296743257425589, -0.34991504516580862, 0.10421432583738932], [0.66486132511251816, -0.83990036625902253, -0.4366608776718115, 0.13311527467714845, 0.3382164722060923], [0.52398650977835115, -0.70218769667138736, 0.062518838539082466, 0.40994775955849438, -0.058568911794933065]])
      arg1=Data(numpy.array([[0.40801392810258319, 0.55970232701195477, 0.53761617047533083, -0.97909149343422142, 0.98483751837359579], [0.17961521617799492, -0.22784069507161275, 0.1643289876635754, -0.37631266069934499, 0.11329288954069172], [0.14297194831055426, 0.17614343920854547, -0.22505241632107609, -0.40785463004531297, -0.3909325837682216], [-0.047415577352520666, 0.92663082365757532, 0.078178606632032466, 0.91281457271967104, 0.81139969205422791]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.40801392810258319, 0.55970232701195477, 0.79667022354874106, -0.84019816840575978, 0.98483751837359579], [0.47038093139447756, -0.22784069507161275, 0.1643289876635754, -0.34991504516580862, 0.11329288954069172], [0.66486132511251816, 0.17614343920854547, -0.22505241632107609, 0.13311527467714845, 0.3382164722060923], [0.52398650977835115, 0.92663082365757532, 0.078178606632032466, 0.91281457271967104, 0.81139969205422791]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(-0.8135239605)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.61388846009494302, 0.28085229667463008, 0.96966170418990538, 0.4756731086027064, 0.089044479963028111], [-0.22360355015236388, 0.6233791983511312, 0.68326777995318166, 0.54220296868399953, -0.75022793841884283], [-0.35635379209712803, -0.23282610888785338, 0.96092420087535979, -0.42869505675150421, 0.65750802195711344], [-0.42871508206319087, -0.56475764881159818, -0.96280950832962331, 0.40149040331234409, -0.26763005910436588]])+(1.-msk_arg1)*numpy.array([[0.39944561956393798, -0.43585902666808818, -0.58310182711372915, 0.0015142487710770869, 0.46901395376714472], [-0.87900855153850288, 0.38639011731442729, -0.95185702388174454, 0.53002944784207284, -0.73959003057720962], [0.53242706460536771, -0.50214820452948183, -0.54088517606287101, -0.68854716037107155, 0.39914930452741304], [0.017706305731783445, -0.37994166987594036, -0.78515827077432565, 0.61126257987463384, 0.453222031968322]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.61388846009494302, 0.28085229667463008, 0.96966170418990538, 0.4756731086027064, 0.089044479963028111], [-0.22360355015236388, 0.6233791983511312, 0.68326777995318166, 0.54220296868399953, -0.75022793841884283], [-0.35635379209712803, -0.23282610888785338, 0.96092420087535979, -0.42869505675150421, 0.65750802195711344], [-0.42871508206319087, -0.56475764881159818, -0.81352396050002107, 0.40149040331234409, -0.26763005910436588]])+(1.-msk_ref)*numpy.array([[0.39944561956393798, -0.43585902666808818, -0.58310182711372915, 0.0015142487710770869, 0.46901395376714472], [-0.81352396050002107, 0.38639011731442729, -0.81352396050002107, 0.53002944784207284, -0.73959003057720962], [0.53242706460536771, -0.50214820452948183, -0.54088517606287101, -0.68854716037107155, 0.39914930452741304], [0.017706305731783445, -0.37994166987594036, -0.78515827077432565, 0.61126257987463384, 0.453222031968322]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-0.78653766932808744, -0.079549862437257701, -0.97814835106959053, 0.040719482339229174, -0.97962562375604301], [-0.40079581080394244, -0.09401787400557815, 0.62166778688084201, -0.75199001510197849, 0.20816228185381469], [0.0095398820105889737, -0.68544916461350902, 0.60254682946002269, -0.5420118338827502, 0.088366583114217079], [-0.14450048156635842, 0.54026628193149651, -0.61281491161900647, 0.94473957477532911, 0.041380925722077633]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.5466488143349244, -0.99031514955758526, -0.0376062612628143, -0.96730469425974897, 0.35607274289209556], [-0.8471312962686266, -0.54896276227621144, -0.63471248523310386, -0.24615168151838396, -0.25723265379342197], [0.049145120793863128, 0.41925950625254549, 0.053557161047377511, -0.3343699299117846, 0.084647011122490401], [0.14946012103736361, 0.80747938071502534, 0.91500285776142665, -0.39265567276306723, 0.92295370825448098]])+(1.-msk_arg1)*numpy.array([[0.54379962366779711, -0.78061481958224044, 0.050683010869727596, -0.93957509919839777, -0.83910628271728838], [0.85987841725590952, -0.55992543465972155, -0.49874145776952972, -0.054873430290065972, -0.44738464167107428], [-0.42793766792957233, 0.13909111370769045, 0.39594342406841165, 0.69552748643520901, -0.29658339184532245], [-0.053113369145769695, -0.88956129876827283, -0.56416742238479545, -0.71325747423089059, 0.21741601018126056]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.5466488143349244, -0.079549862437257701, -0.0376062612628143, 0.040719482339229174, 0.35607274289209556], [-0.40079581080394244, -0.09401787400557815, 0.62166778688084201, -0.24615168151838396, 0.20816228185381469], [0.049145120793863128, 0.41925950625254549, 0.60254682946002269, -0.3343699299117846, 0.088366583114217079], [0.14946012103736361, 0.80747938071502534, 0.91500285776142665, 0.94473957477532911, 0.92295370825448098]])+(1.-msk_ref)*numpy.array([[0.54379962366779711, -0.079549862437257701, 0.050683010869727596, 0.040719482339229174, -0.83910628271728838], [0.85987841725590952, -0.09401787400557815, 0.62166778688084201, -0.054873430290065972, 0.20816228185381469], [0.0095398820105889737, 0.13909111370769045, 0.60254682946002269, 0.69552748643520901, 0.088366583114217079], [-0.053113369145769695, 0.54026628193149651, -0.56416742238479545, 0.94473957477532911, 0.21741601018126056]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_array_rank3(self):
      arg0=numpy.array(0.206495354305)
      arg1=numpy.array([[[-0.28919465172476477, 0.73115741235130649], [0.66849766546201494, 0.32380315894628775]], [[0.34229286100657319, 0.89978338241535871], [-0.94517862805279029, -0.76245816228020713]], [[-0.33058799974889852, 0.91202625598008713], [-0.056699425352694677, -0.96047331967606175]], [[-0.18669649213118422, 0.89626981818651652], [0.12998045639700662, 0.48782120229237735]], [[-0.15134137439915119, -0.88244397684457598], [-0.52188376295396921, -0.81844180102396336]], [[-0.94590032799150481, -0.89329016632804104], [0.96506970958692784, -0.7503920073950836]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[0.20649535430469368, 0.73115741235130649], [0.66849766546201494, 0.32380315894628775]], [[0.34229286100657319, 0.89978338241535871], [0.20649535430469368, 0.20649535430469368]], [[0.20649535430469368, 0.91202625598008713], [0.20649535430469368, 0.20649535430469368]], [[0.20649535430469368, 0.89626981818651652], [0.20649535430469368, 0.48782120229237735]], [[0.20649535430469368, 0.20649535430469368], [0.20649535430469368, 0.20649535430469368]], [[0.20649535430469368, 0.20649535430469368], [0.96506970958692784, 0.20649535430469368]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_array_rank3(self):
      arg0=numpy.array([[[0.39235233572087225, 0.74304758249838065], [-0.66779588135394619, 0.9432836579138042]], [[0.49351107181892084, -0.54694467231291211], [0.99103217822194889, -0.54216696018531252]], [[-0.8175841098544685, 0.8326990222625672], [0.055153398295502587, -0.069716551616386457]], [[0.77626178147117608, 0.11964332061737948], [0.63436115410214011, 0.45842546157944164]], [[0.74397226514682946, -0.77625932725898372], [0.31696360097990373, 0.0024321261945943196]], [[-0.65200916938862652, -0.66937271743063431], [0.32954705135201956, -0.16907363126350994]]])
      arg1=numpy.array([[[-0.50482866899770862, -0.44765629013986286], [0.06780136207618459, 0.15558669698070049]], [[0.044218703621608846, 0.2249976385020942], [0.65123878442216898, 0.65553773857321596]], [[-0.17880284508873401, -0.12255713511715682], [-0.10992299072827882, 0.7685617836322296]], [[0.91958169502841858, -0.44956276558073194], [0.31549165750027508, -0.68010957917212078]], [[-0.8629768256097865, 0.026136425252384576], [-0.71708710148747556, 0.30781358079440957]], [[0.41046623722164766, -0.91807050647819977], [-0.80193690343803214, 0.39991444698935408]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[0.39235233572087225, 0.74304758249838065], [0.06780136207618459, 0.9432836579138042]], [[0.49351107181892084, 0.2249976385020942], [0.99103217822194889, 0.65553773857321596]], [[-0.17880284508873401, 0.8326990222625672], [0.055153398295502587, 0.7685617836322296]], [[0.91958169502841858, 0.11964332061737948], [0.63436115410214011, 0.45842546157944164]], [[0.74397226514682946, 0.026136425252384576], [0.31696360097990373, 0.30781358079440957]], [[0.41046623722164766, -0.66937271743063431], [0.32954705135201956, 0.39991444698935408]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(-0.92297263575)
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[0.37981875931431985, 0.87418870552988737], [0.45599418940704872, 0.91361702634679132]], [[0.88456872676409093, -0.51571215906629497], [-0.26515873138117541, 0.66143648685780843]], [[0.39450335922230106, -0.14722144919672209], [-0.54686554596220049, -0.78276847696479535]], [[-0.13878174646835451, 0.76489703719466484], [0.98137039775706092, 0.39004505195020789]], [[0.12259883986323894, 0.9145662451303298], [-0.19448278087007798, 0.49513630872017766]], [[-0.24780434378271909, -0.82875987188858025], [-0.43502860805814314, 0.21309349424065305]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.37981875931431985, 0.87418870552988737], [0.45599418940704872, 0.91361702634679132]], [[0.88456872676409093, -0.51571215906629497], [-0.26515873138117541, 0.66143648685780843]], [[0.39450335922230106, -0.14722144919672209], [-0.54686554596220049, -0.78276847696479535]], [[-0.13878174646835451, 0.76489703719466484], [0.98137039775706092, 0.39004505195020789]], [[0.12259883986323894, 0.9145662451303298], [-0.19448278087007798, 0.49513630872017766]], [[-0.24780434378271909, -0.82875987188858025], [-0.43502860805814314, 0.21309349424065305]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-0.6262965201288857, 0.084710159751576253], [-0.22354890140408368, -0.80318255693534746]], [[0.39505725142314208, 0.49024342679096233], [0.55974790145294517, -0.74633559665886362]], [[-0.79650062177428005, 0.14870059234679833], [-0.030425746887044802, 0.23615912733952005]], [[0.17001647445926582, 0.60460902992298449], [-0.29018815935114928, -0.36854365139823853]], [[-0.47996868823366334, -0.64200015630345253], [-0.27516754680725675, 0.34441943493978289]], [[0.60021679908106473, -0.45014014271083025], [0.20744651679018311, 0.85343196302909785]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[0.71779794274070441, -0.935435114645224], [0.099475933856508103, 0.74912698964974078]], [[-0.57148188291821378, 0.15972232967923117], [0.73333186260421135, -0.29643072117177716]], [[0.13824844479234577, -0.76005022768157326], [-0.81204372947270365, -0.40255252527070029]], [[-0.21006280244781217, -0.81074339822210306], [-0.84352401253734, 0.14503932771616901]], [[0.26035715420523409, -0.55753735229250756], [-0.64127181637220509, -0.78728049497639185]], [[0.54085431684653695, -0.079395370017745659], [-0.063435288341727958, 0.33333238514882124]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.71779794274070441, 0.084710159751576253], [0.099475933856508103, 0.74912698964974078]], [[0.39505725142314208, 0.49024342679096233], [0.73333186260421135, -0.29643072117177716]], [[0.13824844479234577, 0.14870059234679833], [-0.030425746887044802, 0.23615912733952005]], [[0.17001647445926582, 0.60460902992298449], [-0.29018815935114928, 0.14503932771616901]], [[0.26035715420523409, -0.55753735229250756], [-0.27516754680725675, 0.34441943493978289]], [[0.60021679908106473, -0.079395370017745659], [0.20744651679018311, 0.85343196302909785]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_constData_rank3(self):
      arg0=numpy.array(0.378728231479)
      arg1=Data(numpy.array([[[-0.67114944879927396, -0.16379812287341133], [-0.9257424893497439, -0.39645982000198798]], [[0.9478088084230436, -0.18879856392144134], [-0.16336027795938834, -0.77606729529959839]], [[0.1844003363969049, 0.78778704269394706], [-0.28578747268227223, 0.64431707984038233]], [[0.68238252707515246, 0.06834327476896096], [0.55758618402198645, -0.27105782153341762]], [[0.83653411523399557, -0.36510045139192182], [-0.013797618027125758, 0.11305598817590523]], [[0.9030275523114546, 0.86927059946280005], [0.79084757115115578, -0.51570522549785514]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.37872823147870549, 0.37872823147870549], [0.37872823147870549, 0.37872823147870549]], [[0.9478088084230436, 0.37872823147870549], [0.37872823147870549, 0.37872823147870549]], [[0.37872823147870549, 0.78778704269394706], [0.37872823147870549, 0.64431707984038233]], [[0.68238252707515246, 0.37872823147870549], [0.55758618402198645, 0.37872823147870549]], [[0.83653411523399557, 0.37872823147870549], [0.37872823147870549, 0.37872823147870549]], [[0.9030275523114546, 0.86927059946280005], [0.79084757115115578, 0.37872823147870549]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[0.18144247762408483, -0.78353539631767855], [0.4162761253699141, 0.86266675194423259]], [[0.012556149953617091, 0.4804386420113258], [-0.76805986046045316, 0.14356514697520395]], [[0.49194772346347282, -0.092267796433379345], [-0.26102733986309756, 0.41571439368388496]], [[0.10264134639034794, -0.080200667814620541], [-0.63587157924917848, 0.18913827649625636]], [[-0.36058395425965362, 0.77598983861399407], [-0.73989677505891915, -0.1411309114240975]], [[0.34868774774129974, 0.19384418036852469], [-0.637183036828326, -0.19633048544319909]]])
      arg1=Data(numpy.array([[[0.83338557285258363, 0.075765655696657497], [-0.034150038711153075, 0.48867409214454072]], [[-0.016093860376023894, 0.2815160832699517], [-0.61553678302662829, -0.14883913383260139]], [[-0.94569473686322292, 0.50994618929453117], [-0.42494790481694533, -0.1667068481277989]], [[0.19933276666126143, 0.57718400900736966], [-0.51588024821082401, -0.65798936279151676]], [[-0.90857839351892578, 0.80942308915069616], [-0.75446107420460651, 0.50588344731088331]], [[0.38538650065235491, 0.81707429971303891], [-0.97815096479177388, -0.71051070913370973]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.83338557285258363, 0.075765655696657497], [0.4162761253699141, 0.86266675194423259]], [[0.012556149953617091, 0.4804386420113258], [-0.61553678302662829, 0.14356514697520395]], [[0.49194772346347282, 0.50994618929453117], [-0.26102733986309756, 0.41571439368388496]], [[0.19933276666126143, 0.57718400900736966], [-0.51588024821082401, 0.18913827649625636]], [[-0.36058395425965362, 0.80942308915069616], [-0.73989677505891915, 0.50588344731088331]], [[0.38538650065235491, 0.81707429971303891], [-0.637183036828326, -0.19633048544319909]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(-0.609061899567)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.02221934064502995, -0.24823677577161973], [0.21094272664364166, -0.55842446834946635]], [[0.42482246714205973, 0.06867811566792148], [-0.09860813606919594, -0.38907891589509846]], [[0.040756602755701365, -0.56155860925355627], [-0.63505984155662887, 0.1755095121350585]], [[-0.66997146937465057, 0.29725734526569592], [-0.2396248349932053, -0.32831714430302616]], [[0.4825884611716833, 0.52633576472674903], [0.21070943365647499, 0.27856846579068084]], [[-0.0073100047866563322, -0.25318043364058052], [-0.34499905875957904, -0.1738541124162174]]])+(1.-msk_arg1)*numpy.array([[[-0.54275150799677596, 0.048651829704920102], [0.50592847793838391, -0.6263705525173533]], [[0.67364333189053371, 0.56674993507685767], [0.65430400632474917, 0.80805941287995409]], [[0.064865726857806116, 0.56140841973052491], [-0.22773339570038464, 0.11192902757904855]], [[0.39747464964694235, -0.071536903509030747], [0.0085715500683487544, -0.69734624746017571]], [[-0.39876917998500661, 0.63390636634158959], [-0.39861821315858226, -0.35254308646570043]], [[0.14082903951981685, -0.99563200824916698], [0.086951946864466167, 0.57962945075783878]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.02221934064502995, -0.24823677577161973], [0.21094272664364166, -0.55842446834946635]], [[0.42482246714205973, 0.06867811566792148], [-0.09860813606919594, -0.38907891589509846]], [[0.040756602755701365, -0.56155860925355627], [-0.60906189956736645, 0.1755095121350585]], [[-0.60906189956736645, 0.29725734526569592], [-0.2396248349932053, -0.32831714430302616]], [[0.4825884611716833, 0.52633576472674903], [0.21070943365647499, 0.27856846579068084]], [[-0.0073100047866563322, -0.25318043364058052], [-0.34499905875957904, -0.1738541124162174]]])+(1.-msk_ref)*numpy.array([[[-0.54275150799677596, 0.048651829704920102], [0.50592847793838391, -0.60906189956736645]], [[0.67364333189053371, 0.56674993507685767], [0.65430400632474917, 0.80805941287995409]], [[0.064865726857806116, 0.56140841973052491], [-0.22773339570038464, 0.11192902757904855]], [[0.39747464964694235, -0.071536903509030747], [0.0085715500683487544, -0.60906189956736645]], [[-0.39876917998500661, 0.63390636634158959], [-0.39861821315858226, -0.35254308646570043]], [[0.14082903951981685, -0.60906189956736645], [0.086951946864466167, 0.57962945075783878]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[0.38841144894036916, 0.92715868690960357], [-0.77447087138717952, 0.36220771166763188]], [[-0.16230793101547336, 0.49331094494024885], [-0.46865610524207124, -0.034121204544792283]], [[0.60386974342368327, 0.2626947818589247], [0.93855252670546929, 0.3634155608194618]], [[0.57612389585726764, 0.92358799548671633], [-0.299282269070823, 0.63505715881940072]], [[-0.42338854353484456, -0.42092239793600861], [0.95171309029484008, 0.66415309416591728]], [[-0.54765368059109276, 0.23554498111650468], [0.76618949130819791, -0.3948153453377552]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.9266725179072941, 0.53090206757564506], [-0.59713986747580439, 0.076267888409365936]], [[0.22324371017712408, 0.87148261272246397], [-0.67623763534981807, 0.52405697820419905]], [[-0.097525537051566191, 0.89706368891826105], [-0.3810691076053736, 0.013402757258889952]], [[0.5113244637433414, -0.55238850920540195], [-0.99749026942641228, 0.67337322595687987]], [[-0.7550449944751183, -0.80888632434803931], [0.5666662381666081, 0.79044507419751109]], [[0.54280750281353374, -0.52462690319031169], [-0.75601693543054327, -0.63628825071666228]]])+(1.-msk_arg1)*numpy.array([[[0.33073885193792507, 0.46494205056802484], [-0.0093783968795420058, -0.99272021120250731]], [[-0.50160691421559145, 0.7830226406524603], [0.7158546638991472, 0.10681849654155018]], [[-0.65696144464899331, 0.25217540916152914], [-0.93176795276651858, 0.2468866135972112]], [[0.63556079393859766, 0.91078215014392927], [0.11430987826908967, 0.4340935122722549]], [[0.50475199331880161, 0.74420618264664129], [-0.068579135846449146, 0.7950673667181396]], [[0.41364534013008858, -0.49894363315644497], [-0.46943702047349745, 0.72811686317254676]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.9266725179072941, 0.92715868690960357], [-0.59713986747580439, 0.36220771166763188]], [[0.22324371017712408, 0.87148261272246397], [-0.46865610524207124, 0.52405697820419905]], [[0.60386974342368327, 0.89706368891826105], [0.93855252670546929, 0.3634155608194618]], [[0.57612389585726764, 0.92358799548671633], [-0.299282269070823, 0.67337322595687987]], [[-0.42338854353484456, -0.42092239793600861], [0.95171309029484008, 0.79044507419751109]], [[0.54280750281353374, 0.23554498111650468], [0.76618949130819791, -0.3948153453377552]]])+(1.-msk_ref)*numpy.array([[[0.38841144894036916, 0.92715868690960357], [-0.0093783968795420058, 0.36220771166763188]], [[-0.16230793101547336, 0.7830226406524603], [0.7158546638991472, 0.10681849654155018]], [[0.60386974342368327, 0.2626947818589247], [0.93855252670546929, 0.3634155608194618]], [[0.63556079393859766, 0.92358799548671633], [0.11430987826908967, 0.63505715881940072]], [[0.50475199331880161, 0.74420618264664129], [0.95171309029484008, 0.7950673667181396]], [[0.41364534013008858, 0.23554498111650468], [0.76618949130819791, 0.72811686317254676]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_array_rank4(self):
      arg0=numpy.array(0.129171289856)
      arg1=numpy.array([[[[0.3673768442593579, 0.11571490160281228, -0.64517552668077371, 0.21138864531232571], [-0.17300389051526821, 0.38561202294913177, -0.7799425045145012, 0.91789518329451791], [0.44295986488382333, -0.56752817407673462, -0.67717388658505007, -0.13952018204273786]], [[-0.76623182406287338, 0.85573861149727271, -0.95164332684139752, 0.86720827147905721], [0.74534759751714708, 0.50799031619007984, -0.41362920053940533, -0.36619696053715045], [0.6430492090475719, 0.082871936976270044, -0.68180628674605592, -0.92951630862957102]]], [[[0.29463597351945991, -0.58173079797975968, -0.7230223183212805, 0.61668827498944934], [0.57368054758243181, 0.3467677103973561, -0.86059141654013738, -0.14937151897206813], [-0.75121283127075067, -0.69342076558479571, -0.29776459669994493, -0.29380111352128369]], [[-0.61904618645811826, 0.037206696855934807, -0.30720321824196484, -0.42828827933323077], [0.94459053163673512, 0.75017970241763399, -0.024543351104895805, 0.7317992761434462], [-0.68772557726122208, -0.26921105423467973, -0.33303964038314393, 0.88689290884788208]]], [[[0.19931943871727253, 0.50620323919947507, 0.026887179304865505, 0.9726645366564568], [0.39022216572404478, 0.20642036994395507, 0.58810510782489067, 0.022525598659116186], [0.29380513762421545, -0.51206092926883473, 0.23740794951394628, 0.83348785598584829]], [[0.82449928560496843, -0.88047679800160905, 0.0437070552148493, -0.33527652724059176], [-0.15159772508509795, -0.45358363740648411, -0.51667738655965545, 0.5776497411681254], [0.9659629370206908, -0.41135044415686139, -0.37504198978467462, -0.36072733216551578]]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[[0.3673768442593579, 0.1291712898560109, 0.1291712898560109, 0.21138864531232571], [0.1291712898560109, 0.38561202294913177, 0.1291712898560109, 0.91789518329451791], [0.44295986488382333, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109]], [[0.1291712898560109, 0.85573861149727271, 0.1291712898560109, 0.86720827147905721], [0.74534759751714708, 0.50799031619007984, 0.1291712898560109, 0.1291712898560109], [0.6430492090475719, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109]]], [[[0.29463597351945991, 0.1291712898560109, 0.1291712898560109, 0.61668827498944934], [0.57368054758243181, 0.3467677103973561, 0.1291712898560109, 0.1291712898560109], [0.1291712898560109, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109]], [[0.1291712898560109, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109], [0.94459053163673512, 0.75017970241763399, 0.1291712898560109, 0.7317992761434462], [0.1291712898560109, 0.1291712898560109, 0.1291712898560109, 0.88689290884788208]]], [[[0.19931943871727253, 0.50620323919947507, 0.1291712898560109, 0.9726645366564568], [0.39022216572404478, 0.20642036994395507, 0.58810510782489067, 0.1291712898560109], [0.29380513762421545, 0.1291712898560109, 0.23740794951394628, 0.83348785598584829]], [[0.82449928560496843, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109], [0.1291712898560109, 0.1291712898560109, 0.1291712898560109, 0.5776497411681254], [0.9659629370206908, 0.1291712898560109, 0.1291712898560109, 0.1291712898560109]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[-0.65015954456679181, 0.40240408318498599, 0.84464712425699462, -0.43969720683132807], [0.060269909487150608, -0.74687444707146433, -0.6437215821030513, 0.50694044142420136], [-0.95651303413075461, 0.30077769940421639, -0.88469041671271231, 0.35428303244641701]], [[-0.64479714996431903, -0.02806358507098361, 0.17146951356917417, 0.17856182601303572], [0.82338647213827665, 0.62775989519430597, 0.98131295897678883, -0.27925295851008336], [-0.077888587859346714, 0.39299547141403646, -0.76703658182228662, 0.69803559132588267]]], [[[-0.60438450369130425, 0.50974341740755325, 0.2628982550897998, 0.62253269815816936], [0.098454268427629721, -0.11354020704871637, 0.97520314368136374, 0.82424110975204568], [0.15014068269747205, -0.42103180617954483, 0.90299092501087008, -0.23139020793721432]], [[0.76941820515528692, -0.1765839158818503, -0.5913087365745302, 0.7161045583038077], [0.85854815816198493, 0.024439202303835561, -0.75033601915149672, 0.47128628281539831], [-0.41886779770164062, -0.18310623967189343, -0.6230995563003221, 0.9482826576576675]]], [[[0.094084288889770606, -0.1009961699861317, -0.24821425388692853, -0.70296524988401554], [0.67010687729768903, 0.57353374648941413, 0.81190592473146639, 0.99708169066638397], [-0.37503932645378679, -0.24423377176761285, -0.18549806495578736, -0.47985036549736715]], [[0.62433786823811555, 0.22620666330362238, -0.18049352036474309, 0.99051652854626893], [-0.3286392033550134, 0.83921438988823871, 0.68465908457587843, 0.44769792179940149], [0.11216755461815131, 0.21844427393884125, -0.42328115351224671, -0.85880777265173469]]]])
      arg1=numpy.array([[[[-0.73093615882935814, -0.70919589053874077, 0.78870394532867283, -0.50159810974536545], [-0.85519344422155075, -0.56155628934300195, -0.62373278444802516, -0.9772447843857226], [0.87755710000772535, -0.013588596244209494, -0.90114581352274459, 0.55410820192830745]], [[-0.72956039379052307, -0.35348269136869503, -0.42184790113450843, -0.89870920570023705], [-0.14489448657028414, -0.91755892231285974, 0.89607768435234414, 0.4058334012935787], [-0.55720951018176401, 0.90451728855028346, -0.54876588909958501, 0.70078916216098674]]], [[[0.84626531438079744, 0.45264260263683553, 0.12965000086402689, -0.021003962192429393], [-0.27005492904656037, 0.55590537183241318, -0.24978960230861946, 0.92321598749374068], [0.016189016341156437, -0.35579544797087137, 0.62297910324803074, 0.10682842803458792]], [[-0.5264656932595686, 0.044463460549419409, -0.73796856857767978, 0.16294987868437039], [0.78510668328973954, 0.756487852598553, 0.44081375563129299, -0.60856363687850457], [0.60854054966272164, 0.83451336734693471, 0.71676784780975211, 0.7477674423454328]]], [[[0.94511009490633557, -0.45500546126530228, -0.27748406592529928, -0.11960260520462684], [0.83727559558880982, 0.6973251313608686, 0.43086624118575867, 0.0099152218565310424], [0.18361709779587665, 0.53065002315173349, 0.45664732903855709, 0.71620430423422099]], [[-0.60506520502917138, 0.69610912235182143, -0.10835353752278154, -0.63873781536516083], [-0.80308848611516837, 0.54896014232641921, 0.48999568478728261, 0.075051256341975181], [0.00034354126399538387, 0.14982509954067424, 0.33039570291905651, -0.95215296115442083]]]])
      res=maximum(arg0,arg1)
      ref=numpy.array([[[[-0.65015954456679181, 0.40240408318498599, 0.84464712425699462, -0.43969720683132807], [0.060269909487150608, -0.56155628934300195, -0.62373278444802516, 0.50694044142420136], [0.87755710000772535, 0.30077769940421639, -0.88469041671271231, 0.55410820192830745]], [[-0.64479714996431903, -0.02806358507098361, 0.17146951356917417, 0.17856182601303572], [0.82338647213827665, 0.62775989519430597, 0.98131295897678883, 0.4058334012935787], [-0.077888587859346714, 0.90451728855028346, -0.54876588909958501, 0.70078916216098674]]], [[[0.84626531438079744, 0.50974341740755325, 0.2628982550897998, 0.62253269815816936], [0.098454268427629721, 0.55590537183241318, 0.97520314368136374, 0.92321598749374068], [0.15014068269747205, -0.35579544797087137, 0.90299092501087008, 0.10682842803458792]], [[0.76941820515528692, 0.044463460549419409, -0.5913087365745302, 0.7161045583038077], [0.85854815816198493, 0.756487852598553, 0.44081375563129299, 0.47128628281539831], [0.60854054966272164, 0.83451336734693471, 0.71676784780975211, 0.9482826576576675]]], [[[0.94511009490633557, -0.1009961699861317, -0.24821425388692853, -0.11960260520462684], [0.83727559558880982, 0.6973251313608686, 0.81190592473146639, 0.99708169066638397], [0.18361709779587665, 0.53065002315173349, 0.45664732903855709, 0.71620430423422099]], [[0.62433786823811555, 0.69610912235182143, -0.10835353752278154, 0.99051652854626893], [-0.3286392033550134, 0.83921438988823871, 0.68465908457587843, 0.44769792179940149], [0.11216755461815131, 0.21844427393884125, 0.33039570291905651, -0.85880777265173469]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(0.54447214019)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[0.82434914558439965, 0.030640814671103245, 0.10529857983193125, 0.19997385161819414], [0.28204526598017354, 0.99501508197753386, 0.067263102510887629, 0.945612931561016], [0.69177212052239145, 0.15433877465882562, -0.34415974069966504, 0.46076601729121092]], [[0.54514541152028295, -0.10818735160716719, -0.54790530814729199, -0.9614148723158118], [-0.056835322728886251, 0.31649018663403705, 0.99908563845353426, 0.17217399903284103], [0.89140208475766625, -0.31401352732944132, 0.93168762026073271, -0.54433006276087714]]], [[[0.19545745789279012, -0.53182407993275294, 0.90944127624204407, 0.8974512286069467], [0.44590078115621212, -0.52372172301239694, -0.066822932423527881, 0.0075688913505596034], [0.30168381753209683, 0.48339138025018724, -0.91094197954722289, -0.59987115584787709]], [[-0.73892869459997712, -0.99404245872149222, -0.9310583051851391, 0.73646990298896942], [0.50348458368583526, 0.77412103532204291, 0.55848954879843005, 0.19642606917414862], [0.31416604812741022, 0.50768353012385248, -0.94664565077452512, 0.32435400644970303]]], [[[-0.10775896169479915, -0.95686803118132291, 0.70433993252993576, -0.86326746229903017], [-0.37116743524580786, 0.0075152579945991782, 0.069692036911444655, -0.19749223358485568], [0.29171217123752013, 0.60639389298244173, -0.25174791429627064, 0.1591086976595617]], [[0.82370050143859186, 0.67318457269004317, -0.81977476642063407, -0.7307165497925121], [0.44164841381939346, -0.41778065946860976, 0.38428858791676279, 0.77095046504411346], [-0.52130145465959155, 0.38652434558975246, 0.28559952159740765, 0.4620838896553261]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.82434914558439965, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846], [0.54447214018957846, 0.99501508197753386, 0.54447214018957846, 0.945612931561016], [0.69177212052239145, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846]], [[0.54514541152028295, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846], [0.54447214018957846, 0.54447214018957846, 0.99908563845353426, 0.54447214018957846], [0.89140208475766625, 0.54447214018957846, 0.93168762026073271, 0.54447214018957846]]], [[[0.54447214018957846, 0.54447214018957846, 0.90944127624204407, 0.8974512286069467], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846]], [[0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.73646990298896942], [0.54447214018957846, 0.77412103532204291, 0.55848954879843005, 0.54447214018957846], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846]]], [[[0.54447214018957846, 0.54447214018957846, 0.70433993252993576, 0.54447214018957846], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846], [0.54447214018957846, 0.60639389298244173, 0.54447214018957846, 0.54447214018957846]], [[0.82370050143859186, 0.67318457269004317, 0.54447214018957846, 0.54447214018957846], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.77095046504411346], [0.54447214018957846, 0.54447214018957846, 0.54447214018957846, 0.54447214018957846]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[0.64035870487061941, -0.071502156034890074, 0.45870287482741956, -0.37050440874833046], [-0.25988335765654447, 0.42680078606482352, 0.11013240636419197, 0.13087052469793647], [-0.019889178150541476, -0.96397155283346359, -0.61014343630122569, 0.032007237991111159]], [[0.99029771186264992, 0.97213367201750378, 0.4758484907511531, -0.30758916017087645], [0.61920561114160155, 0.86345720477493981, 0.66440150764961925, -0.3446741419116508], [-0.41033588537740329, -0.36197294663242685, -0.83167286317164035, -0.12831037833172076]]], [[[0.19321103307000631, -0.34078216585226873, -0.78018494418415418, -0.29386078732302345], [-0.94164102465058108, -0.55556717835803959, 0.70474848050281214, -0.85546730500664658], [0.49961250716000172, 0.39678413866284501, 0.3150787805006765, 0.4308616101237055]], [[-0.57410334666083562, -0.62098075243495732, 0.35823364227533561, 0.43179776522545676], [-0.42835765597066544, 0.9438519022815326, -0.45189115695439286, -0.29829155143567099], [-0.9484619949380082, 0.93241452697201388, -0.4240568609770472, -0.15030179732130433]]], [[[-0.62214862186501874, -0.68626527658860526, 0.11837555713462589, -0.19551356382919916], [0.22367080470737677, 0.3144438225586601, -0.50523069137572274, -0.20317349094986836], [-0.29132101551971257, -0.0032855336079842257, -0.9206423961878889, 0.087371025588225182]], [[0.027805277588636157, -0.95709292181658534, -0.29963215574140944, -0.71458122527541601], [-0.54145518988377073, -0.029575892074154897, 0.93044197101731196, -0.11299239967562968], [0.094956034364700903, -0.70084591841267097, 0.85538135162366635, 0.88053528151004379]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[-0.26499460294772725, -0.25033497699890184, -0.97794180326004843, 0.377782894352507], [-0.85312111596170914, 0.6214282013811272, 0.89042813437483637, -0.47941616068389603], [-0.97425914007939873, 0.11103349509582849, 0.59853630796819668, -0.39131474118836218]], [[0.056847101073182804, -0.46017994784642191, 0.43494531047768947, 0.24615807931660227], [0.39908204176313289, 0.59236726121677363, -0.95090913141892552, 0.78833390908185597], [-0.50283442975408699, 0.81580846892625392, -0.61270374695955399, -0.62086199820911081]]], [[[0.18334852778114441, -0.3156194021042471, -0.76053679663695251, 0.14076166793207134], [-0.87501708908233389, -0.49164179184739409, 0.41759477539979772, -0.89279047688005031], [0.1578084605040464, -0.95852467056073309, 0.88167691429914496, 0.51601635901436382]], [[0.78955671979990072, 0.55980673412144366, -0.56376415759135656, 0.80750397548168018], [0.70439950213393177, 0.064904356306807465, -0.0025500668059383891, -0.93595437662341308], [-0.33579280547583923, 0.40104292385269447, -0.34647377314828653, -0.96767513771211866]]], [[[-0.78651350012593446, -0.72482037204131844, -0.27521751207727507, -0.99276193805650581], [0.48906016011055509, 0.81955986561954131, 0.52972943880001622, -0.39091273626154854], [-0.23189015507530963, -0.50679562801150646, 0.094728033171694204, -0.38017661241090139]], [[0.91968871624205017, -0.41799877488931947, -0.26457053738513392, -0.6320096150200083], [-0.24663202057308098, 0.061559275049066331, 0.53259487508798808, 0.34940042387313253], [0.18826045967397231, -0.15112354728140209, 0.93900739726492488, 0.4294116709555269]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.64035870487061941, -0.071502156034890074, 0.45870287482741956, 0.377782894352507], [-0.25988335765654447, 0.6214282013811272, 0.89042813437483637, 0.13087052469793647], [-0.019889178150541476, 0.11103349509582849, 0.59853630796819668, 0.032007237991111159]], [[0.99029771186264992, 0.97213367201750378, 0.4758484907511531, 0.24615807931660227], [0.61920561114160155, 0.86345720477493981, 0.66440150764961925, 0.78833390908185597], [-0.41033588537740329, 0.81580846892625392, -0.61270374695955399, -0.12831037833172076]]], [[[0.19321103307000631, -0.3156194021042471, -0.76053679663695251, 0.14076166793207134], [-0.87501708908233389, -0.49164179184739409, 0.70474848050281214, -0.85546730500664658], [0.49961250716000172, 0.39678413866284501, 0.88167691429914496, 0.51601635901436382]], [[0.78955671979990072, 0.55980673412144366, 0.35823364227533561, 0.80750397548168018], [0.70439950213393177, 0.9438519022815326, -0.0025500668059383891, -0.29829155143567099], [-0.33579280547583923, 0.93241452697201388, -0.34647377314828653, -0.15030179732130433]]], [[[-0.62214862186501874, -0.68626527658860526, 0.11837555713462589, -0.19551356382919916], [0.48906016011055509, 0.81955986561954131, 0.52972943880001622, -0.20317349094986836], [-0.23189015507530963, -0.0032855336079842257, 0.094728033171694204, 0.087371025588225182]], [[0.91968871624205017, -0.41799877488931947, -0.26457053738513392, -0.6320096150200083], [-0.24663202057308098, 0.061559275049066331, 0.93044197101731196, 0.34940042387313253], [0.18826045967397231, -0.15112354728140209, 0.93900739726492488, 0.88053528151004379]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_constData_rank4(self):
      arg0=numpy.array(-0.943242637062)
      arg1=Data(numpy.array([[[[0.46673989197448695, 0.6549649376118345, 0.9984111355226466, -0.70351419152946271], [0.71307312699624026, 0.5419548297460679, -0.17409109440608583, -0.97014228996198115], [-0.54280492332372599, 0.86380903975785861, -0.34701233365455697, 0.48567075553942263]], [[0.54700266237459316, -0.35248705368749378, 0.23377808128753208, 0.96721598543765519], [0.88430869613619123, -0.1219865051849538, -0.058637092978300664, -0.61754559413262333], [0.74079280717308227, -0.78739161180686512, 0.85037530898787361, 0.16508890280036592]]], [[[0.44817727199203761, 0.8926307678886285, -0.76111914551638038, 0.14362232256439045], [-0.48467388532021149, -0.8094428609146751, 0.53535210524492927, -0.56637649678780466], [0.98969180896200037, -0.3271162647098742, 0.69094018879758989, -0.61586242247168044]], [[-0.36044332609306107, -0.98997760997627204, 0.90220389283531444, -0.60156374176429561], [0.40405941425843839, 0.4234467099813175, -0.95866754182894565, 0.8344078032898532], [-0.48131027576552698, -0.59963997992966767, -0.83658860482593256, -0.75913735933586257]]], [[[0.88365651759965269, 0.89012413369555121, -0.4898888357384561, 0.67718373616908689], [0.32819567260296134, -0.89006474342304842, -0.58497923437075872, -0.025361716234509002], [-0.27583713562824563, -0.1438801449075906, 0.75117793183554871, -0.44679974659724797]], [[0.71819263148139534, -0.73238371283246728, -0.92822578406489709, -0.13704547005331813], [0.13259671454727084, -0.45302324436441288, -0.28335401748666711, 0.32261970225996106], [0.90169925985201749, 0.89781229478225999, -0.54573322813790259, -0.22215533008165655]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.46673989197448695, 0.6549649376118345, 0.9984111355226466, -0.70351419152946271], [0.71307312699624026, 0.5419548297460679, -0.17409109440608583, -0.94324263706204037], [-0.54280492332372599, 0.86380903975785861, -0.34701233365455697, 0.48567075553942263]], [[0.54700266237459316, -0.35248705368749378, 0.23377808128753208, 0.96721598543765519], [0.88430869613619123, -0.1219865051849538, -0.058637092978300664, -0.61754559413262333], [0.74079280717308227, -0.78739161180686512, 0.85037530898787361, 0.16508890280036592]]], [[[0.44817727199203761, 0.8926307678886285, -0.76111914551638038, 0.14362232256439045], [-0.48467388532021149, -0.8094428609146751, 0.53535210524492927, -0.56637649678780466], [0.98969180896200037, -0.3271162647098742, 0.69094018879758989, -0.61586242247168044]], [[-0.36044332609306107, -0.94324263706204037, 0.90220389283531444, -0.60156374176429561], [0.40405941425843839, 0.4234467099813175, -0.94324263706204037, 0.8344078032898532], [-0.48131027576552698, -0.59963997992966767, -0.83658860482593256, -0.75913735933586257]]], [[[0.88365651759965269, 0.89012413369555121, -0.4898888357384561, 0.67718373616908689], [0.32819567260296134, -0.89006474342304842, -0.58497923437075872, -0.025361716234509002], [-0.27583713562824563, -0.1438801449075906, 0.75117793183554871, -0.44679974659724797]], [[0.71819263148139534, -0.73238371283246728, -0.92822578406489709, -0.13704547005331813], [0.13259671454727084, -0.45302324436441288, -0.28335401748666711, 0.32261970225996106], [0.90169925985201749, 0.89781229478225999, -0.54573322813790259, -0.22215533008165655]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[0.6710452933322606, 0.73655023524342855, -0.96637183491308676, 0.47953689956820522], [0.98053583083570528, 0.1461687898181756, 0.012703818765450459, 0.48064837271561767], [0.52408663541340594, -0.6101131637614976, 0.71951147562469586, 0.97773169491505763]], [[0.85269224639001395, -0.64939328460945966, -0.40530815940692455, -0.043873643895695258], [0.013474207600136179, -0.83199072645059813, -0.24875392233750837, -0.14945963740023349], [-0.19859755265586743, -0.83519278737823521, -0.16490322565926241, 0.19988995509904828]]], [[[0.54261691324742345, -0.83216349466983819, -0.82229431953325571, -0.95593602291977064], [-0.038265980497722341, 0.38578299235089331, -0.43729585014806571, -0.94893987277299208], [-0.27213639583191407, 0.1045917845853499, 0.0051298010833620289, -0.87753324258171328]], [[0.47867910844244244, -0.7935953174942354, 0.48633491846960841, -0.16964491089477818], [-0.73725496052678019, -0.65544040345490262, 0.82473847295242608, -0.86310822408155241], [-0.22876295798043222, 0.087254884598592541, -0.66897239866347769, -0.88155102312679112]]], [[[0.80518718751035201, 0.9208930345840538, -0.84320779895756859, -0.22169550184009501], [0.34144334236516105, -0.65813010675475092, -0.67816684120052328, 0.03057411294794643], [0.83964290874070091, -0.65242174738552761, 0.65986798996685425, 0.55354844330886732]], [[0.41987573705519776, 0.86448705917430524, 0.89887374333241499, 0.50981938714297903], [-0.66180921979411456, 0.59715491866752002, -0.94380294634555173, 0.61085742396555731], [-0.061851722069981907, -0.82539430034923944, -0.05820899628782894, -0.12313347830576449]]]])
      arg1=Data(numpy.array([[[[0.74351019640477412, -0.98281371332345957, 0.047498607093644729, 0.15927637373867798], [-0.56718099478511252, -0.40521930963777275, -0.15323791375707096, -0.13397472408265321], [0.32026529445003682, 0.94984367824314786, -0.21650883898062334, 0.36878669403603381]], [[0.76460386081962728, 0.26035938179375706, -0.17228362074739612, -0.76176095860293325], [0.79650097864796598, -0.92064005521421444, 0.63695414023301522, 0.29763045353273676], [0.028233556083444356, 0.65356505734241077, 0.64710795344625938, 0.47088770373999034]]], [[[0.052592817909343559, 0.90142884270306523, -0.55744523632845322, -0.10133937858296571], [-0.38270754560875409, 0.84777761145662178, -0.19647228139194683, 0.2624309028567815], [-0.36151988444214433, 0.79453458152737544, 0.035112284546496175, 0.090453586873261216]], [[-0.042959217360417634, -0.40165574292355521, 0.87853656547492331, -0.74381165790643089], [-0.035497537775224242, -0.26990065440269873, 0.52801706764535417, 0.44592510638569061], [-0.46487843397425954, 0.88572704975088068, -0.70820902545471909, -0.83719918740365951]]], [[[-0.17444138264473819, 0.44713858293412612, 0.42308639570713846, 0.11624103502896643], [-0.53171945486995664, -0.43730113574911877, 0.023341839389971808, -0.9174119628732611], [-0.34478808240190162, -0.64263237508691695, 0.09480577060325146, -0.54979877472252747]], [[-0.83762097768252852, -0.28250723065451577, -0.70302270429800839, 0.20350206223682488], [-0.51785671081588647, -0.99895978214481795, -0.09100830119876635, -0.10432462166690804], [-0.2140802658780443, -0.32749288111075736, -0.34840221305540831, -0.33613300029766058]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.74351019640477412, 0.73655023524342855, 0.047498607093644729, 0.47953689956820522], [0.98053583083570528, 0.1461687898181756, 0.012703818765450459, 0.48064837271561767], [0.52408663541340594, 0.94984367824314786, 0.71951147562469586, 0.97773169491505763]], [[0.85269224639001395, 0.26035938179375706, -0.17228362074739612, -0.043873643895695258], [0.79650097864796598, -0.83199072645059813, 0.63695414023301522, 0.29763045353273676], [0.028233556083444356, 0.65356505734241077, 0.64710795344625938, 0.47088770373999034]]], [[[0.54261691324742345, 0.90142884270306523, -0.55744523632845322, -0.10133937858296571], [-0.038265980497722341, 0.84777761145662178, -0.19647228139194683, 0.2624309028567815], [-0.27213639583191407, 0.79453458152737544, 0.035112284546496175, 0.090453586873261216]], [[0.47867910844244244, -0.40165574292355521, 0.87853656547492331, -0.16964491089477818], [-0.035497537775224242, -0.26990065440269873, 0.82473847295242608, 0.44592510638569061], [-0.22876295798043222, 0.88572704975088068, -0.66897239866347769, -0.83719918740365951]]], [[[0.80518718751035201, 0.9208930345840538, 0.42308639570713846, 0.11624103502896643], [0.34144334236516105, -0.43730113574911877, 0.023341839389971808, 0.03057411294794643], [0.83964290874070091, -0.64263237508691695, 0.65986798996685425, 0.55354844330886732]], [[0.41987573705519776, 0.86448705917430524, 0.89887374333241499, 0.50981938714297903], [-0.51785671081588647, 0.59715491866752002, -0.09100830119876635, 0.61085742396555731], [-0.061851722069981907, -0.32749288111075736, -0.05820899628782894, -0.12313347830576449]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(0.816751245349)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.40706796204856954, -0.53751083415836831, -0.45864908744463273, -0.16181548971941506], [0.24911417445318973, -0.90143034728419491, 0.49231704616202521, -0.02953522449634538], [-0.54604157728790303, 0.96956023052870233, -0.35875149576930676, -0.65119882270566221]], [[-0.63683028644751638, 0.517070292194729, 0.88993945395848617, 0.45862552604255513], [-0.3937255872490002, 0.62928624666236899, 0.15644492847277935, 0.39426821818667213], [0.18687916068352117, -0.90059464422537161, 0.63405011651666454, -0.91397232745661849]]], [[[-0.59152682775898779, -0.29881608198483178, -0.12022357576247278, -0.14636655539718824], [0.10683837535157203, 0.4534643980627564, 0.56557067866858457, -0.0058893033942237771], [0.2438289343318536, -0.44513011643495815, -0.82351570680276276, -0.74628626035713075]], [[0.24049586413674917, 0.93536796522792276, 0.40179585898884218, 0.11001812792063603], [0.24888872211708435, 0.13170738221634015, -0.23386082553392895, 0.26287786471668606], [-0.12817923648221208, -0.81433650068603169, -0.85580088180765967, 0.37425524564540913]]], [[[-0.0015262470861385768, -0.13657548471526271, -0.17763500899807361, -0.29008714552601167], [-0.29605361382858919, -0.30804118243930012, -0.69034132273448701, 0.17159914379615593], [-0.055083234234476253, 0.27179546429245183, 0.73841704903963601, 0.56943419450749744]], [[-0.13221678354751099, -0.25975064422559724, 0.98055444051148144, 0.041900636495320498], [-0.67581584371274128, -0.044493982042192126, 0.38493910887288241, -0.053928330783549638], [0.24767507371629205, 0.02740918244429702, -0.40707529813083276, -0.53224165365256848]]]])+(1.-msk_arg1)*numpy.array([[[[-0.74622638578077849, 0.075132461697541508, -0.33313454924071673, -0.67257843324720423], [-0.89086147690801054, -0.37521216146728076, -0.3769720211792944, -0.25074902554592482], [0.03327796652289905, -0.831487975870429, -0.46430521993783014, 0.35666688446285999]], [[-0.3002282511139418, -0.9131223259696819, -0.35725566311385237, -0.08122060507717177], [0.49125425698890024, 0.04278065414817589, -0.10736165776646334, -0.58357962728965052], [-0.78389654631628258, 0.014252969571323604, -0.58349722415645489, 0.48015407778285857]]], [[[0.8629968548019411, -0.39330356017360191, -0.43953900522134193, -0.93850862679670422], [-0.67563892257080771, -0.30985847916271125, -0.73029188466858708, 0.45768180905096179], [0.89312962196038459, 0.24375888513531274, 0.79974754704490114, 0.062990767856061503]], [[0.19388446284468785, 0.2228516825113267, 0.58972079924045051, 0.16046525287644209], [0.16738918455502771, -0.87793601312414005, 0.55708100133014371, 0.79422379620746852], [0.48900265257984232, 0.20469753181175121, -0.076793936351922776, -0.66050327071342485]]], [[[0.37333906028974995, -0.13276337459811982, 0.55257492514565332, -0.87250909004301769], [-0.92159637562262575, 0.035611817197426232, 0.49669980616950649, 0.81933503740201186], [-0.96340332935349826, 0.85175864132768342, -0.56016253718759845, 0.56499606586394324]], [[-0.33109596385421125, 0.97878513835337344, -0.16451086770378209, 0.10562009554922169], [0.64934854776565909, 0.39252253014845206, 0.086082586528108962, -0.87179727093305726], [-0.64734193154791497, -0.11793903610185485, -0.80594955530268586, -0.7145408832676321]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.96956023052870233, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.81675124534906574, 0.88993945395848617, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]], [[[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.93536796522792276, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]], [[[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.81675124534906574, 0.98055444051148144, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]]])+(1.-msk_ref)*numpy.array([[[[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]], [[[0.8629968548019411, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.89312962196038459, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]], [[[0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81933503740201186], [0.81675124534906574, 0.85175864132768342, 0.81675124534906574, 0.81675124534906574]], [[0.81675124534906574, 0.97878513835337344, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574], [0.81675124534906574, 0.81675124534906574, 0.81675124534906574, 0.81675124534906574]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[0.42000806204653474, 0.43445211169835551, 0.67259254968505977, -0.44236725657295772], [0.57797003911910583, -0.56843078081208565, 0.73401747796917305, 0.2724166496453182], [-0.63816568661349526, 0.47652891249952689, 0.22262872392584887, 0.57248495666865962]], [[0.67126178307885032, 0.55524712080063265, -0.57633405842088048, -0.51172125052107154], [0.64644288887632562, -0.30158732813994793, 0.066495417813510427, 0.55087047201075623], [-0.70640833320964092, 0.79392204123529808, -0.30151094639813847, 0.63477282920213551]]], [[[-0.33887822678892188, -0.17257825839362329, -0.86588501606225043, 0.79230278052527692], [-0.39941731689877535, -0.079106652140733758, 0.34561234826387599, -0.3250977623978315], [-0.072143230372351264, -0.35332740106055804, 0.36569821506455535, -0.40358096819303957]], [[-0.46326111929557134, -0.65734587522945453, 0.36365229912540875, 0.29629501392799629], [0.76965355270422631, 0.86916618870287166, -0.30399370154030803, 0.052406314916716434], [-0.76844408403720332, -0.80603282987458247, 0.87923305663161355, -0.47575486213082696]]], [[[0.064241833879999932, 0.75416840871509683, -0.16640916907322456, 0.51810523833466604], [-0.52612642670199117, -0.54881438921402426, -0.60230840326054902, -0.27590086084382826], [0.56589884837057913, 0.63901913471463057, 0.14143396068435887, 0.4572178777562419]], [[-0.77613236188204149, -0.10681746398394076, 0.56066764451409656, -0.2652991439307546], [-0.47167240379595765, 0.13624117786721168, 0.90830564372286182, -0.50408842514981544], [-0.10789679373755079, 0.89455801147160074, 0.64339672677130522, -0.52438730821821777]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.58994058399593396, -0.84790900163215799, -0.2810393046159938, 0.95132339628179641], [-0.96691100251741302, -0.27351720946556934, -0.59482024886374374, 0.067410044082623743], [-0.47797733987261082, -0.15881452904936122, 0.048733333513018007, -0.2097978517034238]], [[-0.022424892496763604, 0.95039754974581636, -0.26755918731750183, 0.11642407168693469], [0.42843959365869888, 0.87961951831310414, 0.87592903677448142, -0.14801753665349993], [-0.44794994006371702, 0.90952976942348851, 0.86095504356940866, 0.40092005641620676]]], [[[-0.87293977911397436, -0.26515845603721533, 0.51886214635551764, -0.84860714710587803], [-0.96644396225751872, -0.7631489360116015, 0.32411191551565888, -0.61020033012558517], [0.62767467887627681, 0.37469207688125628, 0.97311127999171076, -0.48576153689962909]], [[0.80653641318766311, 0.7667470510008485, -0.1244805386140686, -0.58255916955809517], [-0.68905687561103779, 0.73624509848196351, 0.14977570034343146, 0.040371627889322026], [0.59631748792445949, -0.36976261889718165, -0.32387905826980279, -0.70956255815582425]]], [[[-0.47168955139676272, -0.62259149336107811, -0.71438890780775055, -0.70887906389104938], [-0.22646044424514478, 0.22578442905780038, 0.82289907678613572, 0.78508276520394693], [0.66158100390938746, 0.81084818404365477, 0.41404741540900059, 0.29086117891310748]], [[-0.48563213325230881, -0.66773681864261558, 0.13251259057679676, 0.38312110170916869], [0.4985513501920098, -0.054491390265769768, -0.034239376662770438, 0.15118777726900667], [0.637203043607808, -0.63220275381972035, -0.83486043539732502, -0.89196454166383954]]]])+(1.-msk_arg1)*numpy.array([[[[0.77783291145936362, 0.14573546502013346, 0.16427843684067489, 0.070174330346382652], [-0.75910085997762344, -0.13628190232977233, 0.050999095952770856, 0.11715267966221887], [0.24272200394062349, 0.5320807594862087, 0.56874924645624914, -0.97874608588909484]], [[-0.98167918496033835, -0.92884768363832904, 0.020720960458232351, 0.60484296418798422], [0.43656063194574424, 0.24417269636492955, 0.88220623197013071, 0.0051165565172857086], [0.59919244135295746, -0.24769803256244494, 0.46747145925858957, -0.32539081032403749]]], [[[0.36547407180069946, 0.69858798920753662, -0.11815619917568543, 0.7202806361724019], [0.89566175611285326, -0.56471605346156917, -0.96295426503805337, -0.79642127774673965], [0.0082356467622470841, 0.86729114253916273, -0.61543952427247706, 0.84048187872968816]], [[-0.70083513366236239, -0.63770108259677039, 0.8010185543717967, -0.171777821609852], [0.28780672248819994, 0.77948501197327458, 0.80566125829149815, -0.48883690350583775], [-0.31426640827493268, 0.010369391352117008, -0.17008192331534411, -0.24571331409278607]]], [[[-0.81997880316304705, -0.65143657046299919, 0.53349064661197132, 0.56466068808179881], [0.9458545705591126, -0.62032388429602836, -0.20671123052978779, -0.87582148391296899], [0.025494758959417885, -0.16261424915626144, -0.44378234236454994, -0.22992948225644549]], [[0.81002927637991107, -0.68316174531148488, -0.85225788243989031, -0.88222373281789257], [-0.26541667769836907, 0.17943188950419575, 0.63512022836563475, -0.38654668208377108], [-0.31161290145238318, 0.35849193061546902, -0.032927010158966707, 0.33048499755046112]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.58994058399593396, 0.43445211169835551, 0.67259254968505977, 0.95132339628179641], [0.57797003911910583, -0.27351720946556934, 0.73401747796917305, 0.2724166496453182], [-0.47797733987261082, 0.47652891249952689, 0.22262872392584887, 0.57248495666865962]], [[0.67126178307885032, 0.95039754974581636, -0.26755918731750183, 0.11642407168693469], [0.64644288887632562, 0.87961951831310414, 0.87592903677448142, 0.55087047201075623], [-0.44794994006371702, 0.90952976942348851, 0.86095504356940866, 0.63477282920213551]]], [[[-0.33887822678892188, -0.17257825839362329, 0.51886214635551764, 0.79230278052527692], [-0.39941731689877535, -0.079106652140733758, 0.34561234826387599, -0.3250977623978315], [0.62767467887627681, 0.37469207688125628, 0.97311127999171076, -0.40358096819303957]], [[0.80653641318766311, 0.7667470510008485, 0.36365229912540875, 0.29629501392799629], [0.76965355270422631, 0.86916618870287166, 0.14977570034343146, 0.052406314916716434], [0.59631748792445949, -0.36976261889718165, 0.87923305663161355, -0.47575486213082696]]], [[[0.064241833879999932, 0.75416840871509683, -0.16640916907322456, 0.51810523833466604], [-0.22646044424514478, 0.22578442905780038, 0.82289907678613572, 0.78508276520394693], [0.66158100390938746, 0.81084818404365477, 0.41404741540900059, 0.4572178777562419]], [[-0.48563213325230881, -0.10681746398394076, 0.56066764451409656, 0.38312110170916869], [0.4985513501920098, 0.13624117786721168, 0.90830564372286182, 0.15118777726900667], [0.637203043607808, 0.89455801147160074, 0.64339672677130522, -0.52438730821821777]]]])+(1.-msk_ref)*numpy.array([[[[0.77783291145936362, 0.43445211169835551, 0.67259254968505977, 0.070174330346382652], [0.57797003911910583, -0.13628190232977233, 0.73401747796917305, 0.2724166496453182], [0.24272200394062349, 0.5320807594862087, 0.56874924645624914, 0.57248495666865962]], [[0.67126178307885032, 0.55524712080063265, 0.020720960458232351, 0.60484296418798422], [0.64644288887632562, 0.24417269636492955, 0.88220623197013071, 0.55087047201075623], [0.59919244135295746, 0.79392204123529808, 0.46747145925858957, 0.63477282920213551]]], [[[0.36547407180069946, 0.69858798920753662, -0.11815619917568543, 0.79230278052527692], [0.89566175611285326, -0.079106652140733758, 0.34561234826387599, -0.3250977623978315], [0.0082356467622470841, 0.86729114253916273, 0.36569821506455535, 0.84048187872968816]], [[-0.46326111929557134, -0.63770108259677039, 0.8010185543717967, 0.29629501392799629], [0.76965355270422631, 0.86916618870287166, 0.80566125829149815, 0.052406314916716434], [-0.31426640827493268, 0.010369391352117008, 0.87923305663161355, -0.24571331409278607]]], [[[0.064241833879999932, 0.75416840871509683, 0.53349064661197132, 0.56466068808179881], [0.9458545705591126, -0.54881438921402426, -0.20671123052978779, -0.27590086084382826], [0.56589884837057913, 0.63901913471463057, 0.14143396068435887, 0.4572178777562419]], [[0.81002927637991107, -0.10681746398394076, 0.56066764451409656, -0.2652991439307546], [-0.26541667769836907, 0.17943188950419575, 0.90830564372286182, -0.38654668208377108], [-0.10789679373755079, 0.89455801147160074, 0.64339672677130522, 0.33048499755046112]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=-0.986368977956
      res=maximum(arg0,arg1)
      s0=numpy.array(0.917004108653)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.917004108653)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=-0.733232617521
      res=maximum(arg0,arg1)
      s0=numpy.array([0.87446212840176729, 0.69478198612538922])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.87446212840176729, 0.69478198612538922])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=-0.715390372158
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.10015069311152347, 0.85284430951934476, 0.98329958956907282, -0.057180446613754876, 0.86510058046773275], [0.36612085517932846, 0.96635272020442597, 0.59676599731596291, 0.33037821056673078, -0.35211979770395763], [-0.98100658956077491, 0.60495692444451987, 0.36107017587527679, 0.31905076888015071, 0.45131852744342105], [-0.74221976662814293, -0.85110644691487414, -0.0040792398492872639, 0.73620257954628232, 0.89473385139096373]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.10015069311152347, 0.85284430951934476, 0.98329958956907282, -0.057180446613754876, 0.86510058046773275], [0.36612085517932846, 0.96635272020442597, 0.59676599731596291, 0.33037821056673078, -0.35211979770395763], [-0.7153903721580166, 0.60495692444451987, 0.36107017587527679, 0.31905076888015071, 0.45131852744342105], [-0.7153903721580166, -0.7153903721580166, -0.0040792398492872639, 0.73620257954628232, 0.89473385139096373]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=0.464479738044
      res=maximum(arg0,arg1)
      s0=numpy.array([[[-0.34352756908292448, -0.61419356359664357], [0.55818002446357595, -0.41101495052891646]], [[-0.71040027802639782, -0.79504603230523996], [0.84186554823330906, 0.35508497777449599]], [[0.12920328146546312, 0.48205994463481305], [0.95050650098673106, -0.60135148204872468]], [[-0.24348281308306974, 0.15502314913856363], [0.90086027504291977, 0.0011363979118135248]], [[0.95363315495164236, 0.29965777748593347], [0.2258865378960544, -0.31912367702745836]], [[-0.1423878893499102, -0.47378802842834356], [0.35794091817558304, -0.93648558855847175]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.46447973804433973, 0.46447973804433973], [0.55818002446357595, 0.46447973804433973]], [[0.46447973804433973, 0.46447973804433973], [0.84186554823330906, 0.46447973804433973]], [[0.46447973804433973, 0.48205994463481305], [0.95050650098673106, 0.46447973804433973]], [[0.46447973804433973, 0.46447973804433973], [0.90086027504291977, 0.46447973804433973]], [[0.95363315495164236, 0.46447973804433973], [0.46447973804433973, 0.46447973804433973]], [[0.46447973804433973, 0.46447973804433973], [0.46447973804433973, 0.46447973804433973]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=0.525903738176
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[-0.19413905424603439, -0.63211402778026704, -0.49515700544447894, 0.27448546961970721], [0.38731263557039686, -0.45343907707182973, -0.18214645673257346, -0.23732767691311385], [0.99512219129028923, -0.79777607202760792, 0.19220487981433476, -0.78001727629227902]], [[0.040519495158340524, -0.34994277502182891, 0.45876588991715472, -0.33472722430214419], [-0.0017501847545955407, -0.67420388550667321, -0.39012005172372444, 0.96894201309850536], [0.88533663122940176, 0.7986914414082944, -0.40614644801536093, -0.96743479586304049]]], [[[0.60304500131116279, -0.66390835366580991, -0.23231715169180922, 0.75538886986552178], [-0.0094026018153718915, 0.64931261088799186, 0.82063279052147098, 0.66489594615650294], [-0.84183938247666856, 0.49762152875269217, 0.40679985917689154, 0.70053001183294716]], [[-0.0026706860103133945, 0.68125393416767577, -0.060972348753023375, -0.17121284596011876], [-0.97102579948957457, -0.73341280946258269, 0.21586532083260379, 0.82959603585223607], [0.61418236787689295, 0.41382494352608634, 0.69207793756699831, 0.62734783925624482]]], [[[0.62619841232383466, 0.79487182067585027, -0.42952146697606031, -0.92051357815300228], [-0.13654386398794482, -0.79818293777846794, -0.15827501140611089, -0.3251204465947175], [-0.76519771176625651, 0.91147718606064942, -0.025186900606159579, -0.9823182325003359]], [[0.80412070090660581, 0.2907015695560633, 0.19223847083813861, -0.70391356146192674], [0.80842793582848249, -0.74762359050177674, 0.51159052487731516, 0.45054313478058727], [0.059839263836910517, 0.65930489994926966, -0.49457259416276877, -0.5650773006137626]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.99512219129028923, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729]], [[0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.96894201309850536], [0.88533663122940176, 0.7986914414082944, 0.5259037381755729, 0.5259037381755729]]], [[[0.60304500131116279, 0.5259037381755729, 0.5259037381755729, 0.75538886986552178], [0.5259037381755729, 0.64931261088799186, 0.82063279052147098, 0.66489594615650294], [0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.70053001183294716]], [[0.5259037381755729, 0.68125393416767577, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.82959603585223607], [0.61418236787689295, 0.5259037381755729, 0.69207793756699831, 0.62734783925624482]]], [[[0.62619841232383466, 0.79487182067585027, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.91147718606064942, 0.5259037381755729, 0.5259037381755729]], [[0.80412070090660581, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.80842793582848249, 0.5259037381755729, 0.5259037381755729, 0.5259037381755729], [0.5259037381755729, 0.65930489994926966, 0.5259037381755729, 0.5259037381755729]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(0.163319519199)
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.697029784546)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(0.163319519199)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(-0.86461138312)
      res=maximum(arg0,arg1)
      s0=numpy.array([-0.53479441468903244, 0.47123278520533973])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.53479441468903244, 0.47123278520533973])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(0.240330748429)
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.21670951132207117, 0.89114065925724462, -0.39133530486105061, 0.20748149605074517, 0.37132800880875028], [0.56765962762910305, 0.25852724262971383, -0.66121631998813757, -0.19339893573669764, -0.21263300196532819], [0.29982161450753986, -0.5456649228670325, -0.20000772442829828, -0.35366191645422207, 0.24501905042773187], [0.62686496948551507, 0.58301282508729724, 0.79985475354344038, 0.45460470747630999, -0.74585968299614658]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.24033074842899693, 0.89114065925724462, 0.24033074842899693, 0.24033074842899693, 0.37132800880875028], [0.56765962762910305, 0.25852724262971383, 0.24033074842899693, 0.24033074842899693, 0.24033074842899693], [0.29982161450753986, 0.24033074842899693, 0.24033074842899693, 0.24033074842899693, 0.24501905042773187], [0.62686496948551507, 0.58301282508729724, 0.79985475354344038, 0.45460470747630999, 0.24033074842899693]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(0.56406146621)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[0.49814742907718523, -0.20616723253250879], [0.82062693577468382, -0.51987495842035991]], [[0.63825091040794346, -0.33593468894754364], [0.44682595942570003, 0.89941799493797059]], [[0.065960592588822253, -0.70075327485766215], [-0.84594644906740823, 0.37915175015845337]], [[0.63868567321922898, -0.091521295949379633], [0.74124018197041286, -0.091251444285183503]], [[-0.2899767943552749, -0.055025360307588755], [0.44164874750145877, -0.066547620706841171]], [[0.12381643413101662, 0.4275304450701769], [0.65430215999852193, 0.28775853527073747]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.56406146620986464, 0.56406146620986464], [0.82062693577468382, 0.56406146620986464]], [[0.63825091040794346, 0.56406146620986464], [0.56406146620986464, 0.89941799493797059]], [[0.56406146620986464, 0.56406146620986464], [0.56406146620986464, 0.56406146620986464]], [[0.63868567321922898, 0.56406146620986464], [0.74124018197041286, 0.56406146620986464]], [[0.56406146620986464, 0.56406146620986464], [0.56406146620986464, 0.56406146620986464]], [[0.56406146620986464, 0.56406146620986464], [0.65430215999852193, 0.56406146620986464]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(0.673716042599)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[0.92949562732749014, -0.33140075106618982, 0.8985665951811781, -0.77226331140467064], [0.22491766113324552, -0.82681218517906196, -0.39172215610702232, -0.16945280895218762], [0.82698314182604671, 0.98800458415997139, 0.48988568562712276, -0.24033341707920042]], [[-0.02074971973435713, -0.11811769358488022, -0.8081922603841234, 0.32953222027856199], [-0.83049130376397318, -0.076408561621631677, 0.86693245712853018, -0.38322565960826349], [-0.032701908245091227, 0.62434902947474868, -0.060463862774358113, -0.64361528687111447]]], [[[-0.72551194906831262, 0.32465942294217975, 0.96239077669546935, -0.3456104005150169], [-0.46819956359124104, -0.47688844710322731, 0.89035263520153096, 0.78148602550024338], [-0.54543222431933969, 0.49021417276933832, 0.66596718223072982, 0.24528067336774595]], [[0.82092327555344258, 0.12743161192466768, 0.97326825560320862, -0.34622099300303621], [-0.67228818938196255, 0.11244263900157248, 0.63074510030823983, 0.82133349344215989], [0.72747984047396108, -0.96008949636498753, -0.79681207230425155, -0.40330209587465182]]], [[[-0.48709602324947143, -0.7114627374732363, 0.25170065712457768, 0.77403207514055605], [0.72420011044427635, -0.44241645089163328, 0.88433681795000818, 0.78639271163316571], [-0.26810851303708882, 0.0013293407225201115, -0.89746526810324556, 0.40676874902078697]], [[-0.80835495704757432, 0.10175786779323448, -0.73594607340809448, 0.28757716016927781], [-0.55203099995639793, -0.8205215711447027, 0.84793054423581538, 0.24924223828120451], [0.6689253639882462, -0.50963140356565306, -0.63980941456055795, -0.2704595002396919]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.92949562732749014, 0.67371604259937956, 0.8985665951811781, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956], [0.82698314182604671, 0.98800458415997139, 0.67371604259937956, 0.67371604259937956]], [[0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.86693245712853018, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956]]], [[[0.67371604259937956, 0.67371604259937956, 0.96239077669546935, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.89035263520153096, 0.78148602550024338], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956]], [[0.82092327555344258, 0.67371604259937956, 0.97326825560320862, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.82133349344215989], [0.72747984047396108, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956]]], [[[0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.77403207514055605], [0.72420011044427635, 0.67371604259937956, 0.88433681795000818, 0.78639271163316571], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956]], [[0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.84793054423581538, 0.67371604259937956], [0.67371604259937956, 0.67371604259937956, 0.67371604259937956, 0.67371604259937956]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.166533972653)
      s1=numpy.array(0.430008514948)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(0.430008514948)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s0=numpy.array([0.98855944642845883, -0.70279545386154929])
      s1=numpy.array(0.942761064153)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.98855944642845883, 0.94276106415295402])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.8614007685112417, 0.82263019293461137, 0.1418462044227855, -0.20151175696136558, 0.30135292864272079], [-0.70846188345403349, -0.60533041631925211, 0.5431601660745895, 0.21139751570935839, -0.85400113567282943], [0.63686560832390393, -0.55432791732840991, 0.13475895427448137, -0.85230870242177037, 0.59339288766225207], [-0.33276615446339153, -0.49046931392216409, 0.97047097632721102, 0.5672078146297328, 0.30436669385545345]])
      s1=numpy.array(0.106783144869)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.1067831448692691, 0.82263019293461137, 0.1418462044227855, 0.1067831448692691, 0.30135292864272079], [0.1067831448692691, 0.1067831448692691, 0.5431601660745895, 0.21139751570935839, 0.1067831448692691], [0.63686560832390393, 0.1067831448692691, 0.13475895427448137, 0.1067831448692691, 0.59339288766225207], [0.1067831448692691, 0.1067831448692691, 0.97047097632721102, 0.5672078146297328, 0.30436669385545345]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s0=numpy.array([[[-0.79909899934883488, -0.40692994409128036], [-0.39615105661504368, -0.53390408798903644]], [[-0.16374438970800043, -0.24567498448514513], [0.22094989589252667, 0.81075447536539635]], [[-0.027615249313295731, 0.61952626682581768], [-0.7828540120805505, -0.52011020027155008]], [[-0.24843780128293691, 0.43600182922082897], [-0.53047801506438774, -0.46750623083443843]], [[0.85652955278665899, -0.91405058017376195], [0.31288219688658359, -0.65437941972793512]], [[-0.65139376325689069, 0.62494317062335347], [-0.16151041548850098, 0.61180187276593223]]])
      s1=numpy.array(-0.423928135739)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.42392813573946131, -0.40692994409128036], [-0.39615105661504368, -0.42392813573946131]], [[-0.16374438970800043, -0.24567498448514513], [0.22094989589252667, 0.81075447536539635]], [[-0.027615249313295731, 0.61952626682581768], [-0.42392813573946131, -0.42392813573946131]], [[-0.24843780128293691, 0.43600182922082897], [-0.42392813573946131, -0.42392813573946131]], [[0.85652955278665899, -0.42392813573946131], [0.31288219688658359, -0.42392813573946131]], [[-0.42392813573946131, 0.62494317062335347], [-0.16151041548850098, 0.61180187276593223]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[-0.3197610658358101, 0.09163128400803755, 0.84285416271000502, 0.8107182318956847], [-0.80234594024029171, 0.88981471872386875, 0.077137979027627868, 0.19190664421844761], [0.50690629441499446, 0.9721392850575814, 0.12774839226000223, 0.80806202926473958]], [[-0.20131529499656198, -0.83678241134985742, 0.9298155323060342, -0.93329898616677198], [0.54482984289836844, 0.071141501257063089, 0.61162501195689578, 0.71034656005263175], [-0.98097765053215502, 0.71752059792137901, 0.2085477530285158, 0.26205019049259848]]], [[[0.6168213512427394, 0.57594407295380168, 0.68704347550782052, 0.61691401043530614], [0.35213409335576284, 0.27472848818475315, -0.3687087676488221, -0.44614893287030677], [-0.87608963419014874, -0.014053908691249895, 0.57597178412796324, -0.26681554386471018]], [[0.15197681521073614, -0.87567767562025156, 0.13133830178541284, -0.9185946661028388], [0.428296691362821, 0.64886826324610847, -0.069826285233708152, 0.48355919520824586], [0.99989140373972729, 0.016264841745767367, -0.05030165567678524, -0.73853383570009656]]], [[[-0.057969768496793028, 0.88240125813848502, 0.97914735846770395, -0.89873980653677177], [0.8083904454657822, 0.0049221269856327687, 0.51901335144301863, 0.53003262470873147], [0.23853331008355982, -0.82285092330190102, 0.20465406057234414, -0.60712276206742044]], [[-0.80386246349242652, -0.88535276518185668, -0.61358743884420663, -0.18986074502330252], [-0.47083317022436844, 0.19064280065118289, -0.017269458122652681, -0.055598241359959388], [0.99387494359776829, -0.80889485214570378, -0.12065207499884267, -0.59817605022123899]]]])
      s1=numpy.array(0.203476883849)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.20347688384919493, 0.20347688384919493, 0.84285416271000502, 0.8107182318956847], [0.20347688384919493, 0.88981471872386875, 0.20347688384919493, 0.20347688384919493], [0.50690629441499446, 0.9721392850575814, 0.20347688384919493, 0.80806202926473958]], [[0.20347688384919493, 0.20347688384919493, 0.9298155323060342, 0.20347688384919493], [0.54482984289836844, 0.20347688384919493, 0.61162501195689578, 0.71034656005263175], [0.20347688384919493, 0.71752059792137901, 0.2085477530285158, 0.26205019049259848]]], [[[0.6168213512427394, 0.57594407295380168, 0.68704347550782052, 0.61691401043530614], [0.35213409335576284, 0.27472848818475315, 0.20347688384919493, 0.20347688384919493], [0.20347688384919493, 0.20347688384919493, 0.57597178412796324, 0.20347688384919493]], [[0.20347688384919493, 0.20347688384919493, 0.20347688384919493, 0.20347688384919493], [0.428296691362821, 0.64886826324610847, 0.20347688384919493, 0.48355919520824586], [0.99989140373972729, 0.20347688384919493, 0.20347688384919493, 0.20347688384919493]]], [[[0.20347688384919493, 0.88240125813848502, 0.97914735846770395, 0.20347688384919493], [0.8083904454657822, 0.20347688384919493, 0.51901335144301863, 0.53003262470873147], [0.23853331008355982, 0.20347688384919493, 0.20465406057234414, 0.20347688384919493]], [[0.20347688384919493, 0.20347688384919493, 0.20347688384919493, 0.20347688384919493], [0.20347688384919493, 0.20347688384919493, 0.20347688384919493, 0.20347688384919493], [0.99387494359776829, 0.20347688384919493, 0.20347688384919493, 0.20347688384919493]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(0.0788511607181,self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array(0.303633608874)
      sub=res.substitute({arg0:s0})
      ref=Data(0.303633608874,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(-0.162806571951,self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([0.48899532432066772, 0.98292801130723961])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.48899532432066772, 0.98292801130723961]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-0.587017634534,self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.96680749418452683, -0.9409372133034557, -0.7372427659579408, 0.58183265634894132, -0.47030168775580528], [0.95629410215028376, -0.43441571473057294, -0.24254920368993438, 0.016680928636627401, -0.92292483223332344], [-0.42555995336057428, 0.77948391551850871, -0.9687742377475006, -0.28777464721520696, -0.90969594156509759], [0.087728141355752021, -0.40374152482886361, 0.81089428261806384, 0.22370927269414698, 0.93473939721049248]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.58701763453389844, -0.58701763453389844, -0.58701763453389844, 0.58183265634894132, -0.47030168775580528], [0.95629410215028376, -0.43441571473057294, -0.24254920368993438, 0.016680928636627401, -0.58701763453389844], [-0.42555995336057428, 0.77948391551850871, -0.58701763453389844, -0.28777464721520696, -0.58701763453389844], [0.087728141355752021, -0.40374152482886361, 0.81089428261806384, 0.22370927269414698, 0.93473939721049248]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(0.547481898119,self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[0.95130050632782148, 0.059809648831347317], [-0.0042055570823034127, -0.01676446924521291]], [[-0.11956810738226498, 0.0040191935537194912], [-0.92067452177716436, -0.21619168762054319]], [[0.39748366071952357, 0.88176111652294176], [-0.38367950899579428, -0.53417355040157588]], [[0.30869825039087684, 0.78034270617056012], [-0.60852640470368158, 0.83832406492458933]], [[0.42530182038268727, -0.69673521783603443], [0.3202479389074957, 0.80561068799305002]], [[-0.56765351405396069, -0.68856733684006644], [-0.21245231302681078, -0.62249435800827047]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.95130050632782148, 0.54748189811873549], [0.54748189811873549, 0.54748189811873549]], [[0.54748189811873549, 0.54748189811873549], [0.54748189811873549, 0.54748189811873549]], [[0.54748189811873549, 0.88176111652294176], [0.54748189811873549, 0.54748189811873549]], [[0.54748189811873549, 0.78034270617056012], [0.54748189811873549, 0.83832406492458933]], [[0.54748189811873549, 0.54748189811873549], [0.54748189811873549, 0.80561068799305002]], [[0.54748189811873549, 0.54748189811873549], [0.54748189811873549, 0.54748189811873549]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(-0.518107061619,self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[0.31640786412133126, 0.64807746462822458, 0.073977040593609544, -0.16449622796920038], [-0.14690222245515772, -0.81301379067263757, -0.50432224582303009, -0.447562296962005], [-0.58599425890881518, 0.83080068562811737, -0.34042686204931627, -0.38686789778375408]], [[-0.21063890178960709, 0.76788516863594625, 0.62451258248714692, 0.32421120223336719], [0.001661874659981244, 0.86811936048353289, 0.34186278145656779, 0.8345881407727882], [0.3461223632547854, -0.30870389935021469, 0.69332650541969332, -0.20457936523355036]]], [[[-0.44008316860592944, 0.033272971528933715, -0.43037577665469273, -0.86290580509896109], [-0.16154802830657111, 0.20835433741116738, -0.38994782436231357, 0.078796771405668098], [0.71789054669369712, -0.45269067670741658, 0.06288990641906711, -0.09976863050329321]], [[-0.44845973597783084, 0.37672855875725775, -0.57888357458476558, 0.097784566934107531], [-0.62678286627981228, -0.053958966187036106, -0.45992917142127077, -0.65505084111324141], [-0.26841433130275139, -0.97725071573027411, 0.88433357499588494, -0.49330829768795237]]], [[[-0.47659572222145563, 0.04928606586223383, 0.46058577506833442, -0.18461027428713539], [-0.22513427689931298, -0.7662490489772833, 0.61821460314715604, 0.58597859687807463], [-0.093810622230459773, 0.4964372103067185, -0.8616945111843024, -0.57833522278885452]], [[0.73171927612181387, -0.65706298967009102, -0.08364118747949334, 0.32871476768255992], [0.22332591303785332, 0.26675993239669671, -0.86248306316787549, -0.27336126248224679], [0.43488279247381145, 0.62698861668571082, 0.36801312408896392, -0.5375278001288486]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.31640786412133126, 0.64807746462822458, 0.073977040593609544, -0.16449622796920038], [-0.14690222245515772, -0.5181070616193959, -0.50432224582303009, -0.447562296962005], [-0.5181070616193959, 0.83080068562811737, -0.34042686204931627, -0.38686789778375408]], [[-0.21063890178960709, 0.76788516863594625, 0.62451258248714692, 0.32421120223336719], [0.001661874659981244, 0.86811936048353289, 0.34186278145656779, 0.8345881407727882], [0.3461223632547854, -0.30870389935021469, 0.69332650541969332, -0.20457936523355036]]], [[[-0.44008316860592944, 0.033272971528933715, -0.43037577665469273, -0.5181070616193959], [-0.16154802830657111, 0.20835433741116738, -0.38994782436231357, 0.078796771405668098], [0.71789054669369712, -0.45269067670741658, 0.06288990641906711, -0.09976863050329321]], [[-0.44845973597783084, 0.37672855875725775, -0.5181070616193959, 0.097784566934107531], [-0.5181070616193959, -0.053958966187036106, -0.45992917142127077, -0.5181070616193959], [-0.26841433130275139, -0.5181070616193959, 0.88433357499588494, -0.49330829768795237]]], [[[-0.47659572222145563, 0.04928606586223383, 0.46058577506833442, -0.18461027428713539], [-0.22513427689931298, -0.5181070616193959, 0.61821460314715604, 0.58597859687807463], [-0.093810622230459773, 0.4964372103067185, -0.5181070616193959, -0.5181070616193959]], [[0.73171927612181387, -0.5181070616193959, -0.08364118747949334, 0.32871476768255992], [0.22332591303785332, 0.26675993239669671, -0.5181070616193959, -0.27336126248224679], [0.43488279247381145, 0.62698861668571082, 0.36801312408896392, -0.5181070616193959]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.263837473061)+(1.-msk_arg1)*(0.930322128729)
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.303145599651)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.263837473061)+(1.-msk_ref)*(0.930322128729)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.425613591683)+(1.-msk_arg1)*(-0.266798362013)
      res=maximum(arg0,arg1)
      s0=numpy.array([0.47786124208316028, -0.10829417858192092])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.47786124208316028, -0.10829417858192092])+(1.-msk_ref)*numpy.array([0.47786124208316028, -0.10829417858192092])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.518705356576)+(1.-msk_arg1)*(-0.425545159263)
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.47737943188442711, -0.50968653776702588, 0.12636167117822783, -0.16112767485523127, 0.61610406596900447], [0.70009109684603388, -0.44585520454126804, -0.95142957301589393, -0.83937244717880199, 0.83185554435457876], [-0.76477480506013906, -0.76358302849696003, -0.053281801624190273, 0.34653779013304975, -0.18229907430717285], [0.34674273157998892, 0.67907571493663266, 0.21299680560501799, -0.95864483251904531, 0.28186482517984324]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.51870535657631067, 0.51870535657631067, 0.51870535657631067, 0.51870535657631067, 0.61610406596900447], [0.70009109684603388, 0.51870535657631067, 0.51870535657631067, 0.51870535657631067, 0.83185554435457876], [0.51870535657631067, 0.51870535657631067, 0.51870535657631067, 0.51870535657631067, 0.51870535657631067], [0.51870535657631067, 0.67907571493663266, 0.51870535657631067, 0.51870535657631067, 0.51870535657631067]])+(1.-msk_ref)*numpy.array([[-0.42554515926296643, -0.42554515926296643, 0.12636167117822783, -0.16112767485523127, 0.61610406596900447], [0.70009109684603388, -0.42554515926296643, -0.42554515926296643, -0.42554515926296643, 0.83185554435457876], [-0.42554515926296643, -0.42554515926296643, -0.053281801624190273, 0.34653779013304975, -0.18229907430717285], [0.34674273157998892, 0.67907571493663266, 0.21299680560501799, -0.42554515926296643, 0.28186482517984324]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.39619680396)+(1.-msk_arg1)*(0.510543564256)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[0.33767055987597305, 0.97379547968070068], [0.80363858071540073, -0.95544434119662514]], [[0.75988251135815954, 0.24822206064809538], [0.98535571000331257, -0.061494240171396708]], [[0.56181380018510829, -0.70038885545135732], [0.47418443531364218, -0.7876625924625742]], [[0.77648037076526299, -0.85430842741265489], [-0.21299099629151108, 0.28476909129171646]], [[-0.89901746460087351, 0.80244629190134864], [-0.16039892085645224, 0.93087599509386121]], [[-0.24628855952915596, -0.36747393045803411], [-0.22990306443178077, 0.7827660973976549]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.39619680396043155, 0.97379547968070068], [0.80363858071540073, 0.39619680396043155]], [[0.75988251135815954, 0.39619680396043155], [0.98535571000331257, 0.39619680396043155]], [[0.56181380018510829, 0.39619680396043155], [0.47418443531364218, 0.39619680396043155]], [[0.77648037076526299, 0.39619680396043155], [0.39619680396043155, 0.39619680396043155]], [[0.39619680396043155, 0.80244629190134864], [0.39619680396043155, 0.93087599509386121]], [[0.39619680396043155, 0.39619680396043155], [0.39619680396043155, 0.7827660973976549]]])+(1.-msk_ref)*numpy.array([[[0.51054356425597924, 0.97379547968070068], [0.80363858071540073, 0.51054356425597924]], [[0.75988251135815954, 0.51054356425597924], [0.98535571000331257, 0.51054356425597924]], [[0.56181380018510829, 0.51054356425597924], [0.51054356425597924, 0.51054356425597924]], [[0.77648037076526299, 0.51054356425597924], [0.51054356425597924, 0.51054356425597924]], [[0.51054356425597924, 0.80244629190134864], [0.51054356425597924, 0.93087599509386121]], [[0.51054356425597924, 0.51054356425597924], [0.51054356425597924, 0.7827660973976549]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.867270234622)+(1.-msk_arg1)*(-0.184167666491)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[0.34778842100985696, 0.91466700155307601, 0.068070179806759334, 0.93587871845739112], [0.81660802101282726, -0.42862634669531841, 0.77826901708032148, -0.2395592743697541], [-0.99838763228477978, 0.90813063765068813, -0.21282461462487134, -0.56866622590996285]], [[-0.040514202639923669, -0.41228222367310474, -0.56155526462486827, -0.23846092550202469], [0.54370233421879144, 0.72784420886026124, -0.56680235853516159, -0.61068924291751481], [0.51942107564774043, -0.47875859482064098, -0.44532210559704266, -0.26453473762702484]]], [[[-0.45070324266714201, 0.47986144706095812, 0.015567101765326496, -0.55300155278252672], [-0.77777158426443727, -0.46817038952409518, 0.76605853896643961, -0.21029519272087338], [-0.75962168514124606, 0.69650112782742934, -0.40248744545665582, -0.53388556223025385]], [[0.15426085249022914, 0.37998253169049767, -0.63812447110396842, -0.78124765440264299], [-0.77292230448860999, 0.80781768514724206, 0.21895647166908949, -0.7094480538535668], [0.42737300022198133, -0.29447469391631875, -0.011906740714938024, -0.44324636230959058]]], [[[-0.84345759673746823, -0.62303509242689792, 0.33791243754168754, 0.056143570948844479], [0.45860774951591199, 0.41744652076944933, 0.30025926758975374, -0.3265961269776636], [-0.75573723130545112, -0.32649476937035216, 0.64656306851701828, 0.61570588213194255]], [[0.61027123987319132, 0.76728309078514756, 0.38269652894555595, 0.076546543524037558], [0.44125754685327867, 0.3603904711844681, 0.83200264991754014, -0.49749830527479832], [-0.98594801190851578, -0.044658872549786732, -0.46810940529491707, -0.34099272448468709]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.34778842100985696, 0.91466700155307601, 0.068070179806759334, 0.93587871845739112], [0.81660802101282726, -0.42862634669531841, 0.77826901708032148, -0.2395592743697541], [-0.86727023462218034, 0.90813063765068813, -0.21282461462487134, -0.56866622590996285]], [[-0.040514202639923669, -0.41228222367310474, -0.56155526462486827, -0.23846092550202469], [0.54370233421879144, 0.72784420886026124, -0.56680235853516159, -0.61068924291751481], [0.51942107564774043, -0.47875859482064098, -0.44532210559704266, -0.26453473762702484]]], [[[-0.45070324266714201, 0.47986144706095812, 0.015567101765326496, -0.55300155278252672], [-0.77777158426443727, -0.46817038952409518, 0.76605853896643961, -0.21029519272087338], [-0.75962168514124606, 0.69650112782742934, -0.40248744545665582, -0.53388556223025385]], [[0.15426085249022914, 0.37998253169049767, -0.63812447110396842, -0.78124765440264299], [-0.77292230448860999, 0.80781768514724206, 0.21895647166908949, -0.7094480538535668], [0.42737300022198133, -0.29447469391631875, -0.011906740714938024, -0.44324636230959058]]], [[[-0.84345759673746823, -0.62303509242689792, 0.33791243754168754, 0.056143570948844479], [0.45860774951591199, 0.41744652076944933, 0.30025926758975374, -0.3265961269776636], [-0.75573723130545112, -0.32649476937035216, 0.64656306851701828, 0.61570588213194255]], [[0.61027123987319132, 0.76728309078514756, 0.38269652894555595, 0.076546543524037558], [0.44125754685327867, 0.3603904711844681, 0.83200264991754014, -0.49749830527479832], [-0.86727023462218034, -0.044658872549786732, -0.46810940529491707, -0.34099272448468709]]]])+(1.-msk_ref)*numpy.array([[[[0.34778842100985696, 0.91466700155307601, 0.068070179806759334, 0.93587871845739112], [0.81660802101282726, -0.18416766649073457, 0.77826901708032148, -0.18416766649073457], [-0.18416766649073457, 0.90813063765068813, -0.18416766649073457, -0.18416766649073457]], [[-0.040514202639923669, -0.18416766649073457, -0.18416766649073457, -0.18416766649073457], [0.54370233421879144, 0.72784420886026124, -0.18416766649073457, -0.18416766649073457], [0.51942107564774043, -0.18416766649073457, -0.18416766649073457, -0.18416766649073457]]], [[[-0.18416766649073457, 0.47986144706095812, 0.015567101765326496, -0.18416766649073457], [-0.18416766649073457, -0.18416766649073457, 0.76605853896643961, -0.18416766649073457], [-0.18416766649073457, 0.69650112782742934, -0.18416766649073457, -0.18416766649073457]], [[0.15426085249022914, 0.37998253169049767, -0.18416766649073457, -0.18416766649073457], [-0.18416766649073457, 0.80781768514724206, 0.21895647166908949, -0.18416766649073457], [0.42737300022198133, -0.18416766649073457, -0.011906740714938024, -0.18416766649073457]]], [[[-0.18416766649073457, -0.18416766649073457, 0.33791243754168754, 0.056143570948844479], [0.45860774951591199, 0.41744652076944933, 0.30025926758975374, -0.18416766649073457], [-0.18416766649073457, -0.18416766649073457, 0.64656306851701828, 0.61570588213194255]], [[0.61027123987319132, 0.76728309078514756, 0.38269652894555595, 0.076546543524037558], [0.44125754685327867, 0.3603904711844681, 0.83200264991754014, -0.18416766649073457], [-0.18416766649073457, -0.044658872549786732, -0.18416766649073457, -0.18416766649073457]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([0.94106241283176484, 0.60425808753586785])
      res=maximum(arg0,arg1)
      s0=numpy.array(0.152421450799)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.94106241283176484, 0.60425808753586785])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.034752439445001171, 0.68085408257859159])
      res=maximum(arg0,arg1)
      s0=numpy.array([0.49509033009119663, -0.073919203029765068])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.49509033009119663, 0.68085408257859159])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.832622816062)
      s1=numpy.array([0.92124477972930863, -0.25183969524474414])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.92124477972930863, -0.25183969524474414])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s0=numpy.array([-0.83387519920792053, 0.83757127479595406])
      s1=numpy.array([0.023700736212592943, -0.74911518215668571])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.023700736212592943, 0.83757127479595406])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-0.20444554583372709, -0.7579455353730431]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array(0.269364792866)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.26936479286585557, 0.26936479286585557]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([0.79189883806353989, 0.51030782269672104]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([-0.021449956598355024, -0.8928594273678232])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.79189883806353989, 0.51030782269672104]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.91354659520570336, -0.7100777244328127])+(1.-msk_arg1)*numpy.array([-0.31113350629580583, -0.21776645657501925])
      res=maximum(arg0,arg1)
      s0=numpy.array(0.601790255505)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.60179025550480314, 0.60179025550480314])+(1.-msk_ref)*numpy.array([0.60179025550480314, 0.60179025550480314])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.64122684146842635, -0.65429758471804611])+(1.-msk_arg1)*numpy.array([0.36026003247482508, -0.4395755173829814])
      res=maximum(arg0,arg1)
      s0=numpy.array([0.05956187348421671, -0.39958095268037686])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.05956187348421671, -0.39958095268037686])+(1.-msk_ref)*numpy.array([0.36026003247482508, -0.39958095268037686])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-0.41909952453052179, -0.95594534974473211, 0.32059759793432763, 0.71268961072629944, 0.35710059122523963], [0.20954651130957025, -0.60789293663703381, 0.1052632207481321, 0.83984311406122658, 0.81600130912826918], [-0.39167915011286447, 0.7961363405839792, 0.14325859631932425, 0.37374712551337019, -0.55546564616509864], [-0.82503775909959853, -0.29694261438056713, 0.88448073236823621, -0.380823321238005, -0.41325694100420418]])
      res=maximum(arg0,arg1)
      s0=numpy.array(0.917201419953)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843], [0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843], [0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843], [0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843, 0.91720141995312843]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[0.23282724770560126, 0.90119165886227859, 0.44879639491772649, 0.57993715563417991, -0.88778077284719803], [-0.098116174000989709, -0.10763469613068244, 0.69142554364313624, 0.64695857344334162, -0.72343296045702021], [-0.8485413429579356, 0.84987016287292305, -0.037053313470545968, -0.34405217320242421, -0.7831835209615936], [0.18315904299735286, 0.55556339471233818, -0.47376736532833519, -0.15673803853155754, 0.2040480519251282]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.42967559325550519, 0.52471287102439446, -0.34684138483439919, 0.12047981989667123, 0.99943797195006256], [-0.83323480509064907, -0.32689970732899321, -0.20379032320512036, -0.67755387056710625, 0.67081376442907836], [0.47977870378710996, 0.27896962622954047, -0.52228456748219876, -0.39216003886053974, -0.39253205916633815], [0.06699134295942466, 0.43533022627635409, 0.022220650734660952, 0.31703340867534924, 0.84760350942064333]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.23282724770560126, 0.90119165886227859, 0.44879639491772649, 0.57993715563417991, 0.99943797195006256], [-0.098116174000989709, -0.10763469613068244, 0.69142554364313624, 0.64695857344334162, 0.67081376442907836], [0.47977870378710996, 0.84987016287292305, -0.037053313470545968, -0.34405217320242421, -0.39253205916633815], [0.18315904299735286, 0.55556339471233818, 0.022220650734660952, 0.31703340867534924, 0.84760350942064333]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s0=numpy.array(0.166413160055)
      s1=numpy.array([[0.55674342923379383, 0.59655050003377652, 0.8882982024657744, 0.58965616025464751, -0.39088936745926395], [0.75550554261223879, 0.16831498582554461, 0.7266049241086534, -0.27919117369941548, 0.60860624879720882], [0.11784583838409257, 0.65525340611548244, 0.22026806564170842, -0.51459503082157032, 0.98423242893885243], [0.34720556407559289, 0.5684198515052532, -0.36943957417928441, -0.78117042570196826, -0.59786143208149101]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.55674342923379383, 0.59655050003377652, 0.8882982024657744, 0.58965616025464751, 0.16641316005544349], [0.75550554261223879, 0.16831498582554461, 0.7266049241086534, 0.16641316005544349, 0.60860624879720882], [0.16641316005544349, 0.65525340611548244, 0.22026806564170842, 0.16641316005544349, 0.98423242893885243], [0.34720556407559289, 0.5684198515052532, 0.16641316005544349, 0.16641316005544349, 0.16641316005544349]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s0=numpy.array([[0.77311242286876292, 0.44908744692280189, 0.83226215240998358, 0.52087289713254514, 0.38641318195812535], [-0.98247040827721266, -0.12604666338806214, 0.8510608849491661, 0.95600963684451967, 0.86448510301290793], [-0.48121609014032507, -0.7704793822017717, 0.36678788811442398, 0.19832437648441004, -0.0214211514409548], [-0.14461403809964946, 0.94793132621096565, 0.046753157348639496, -0.039652758267353594, -0.92175826292896268]])
      s1=numpy.array([[-0.41521660807567051, -0.5504989614425273, -0.78358117611510147, 0.21750741385718664, -0.028037838778460555], [0.80571324980523062, -0.56610366108198229, 0.28300733959704338, -0.904023477840457, -0.34530644180596659], [-0.2927941485226444, -0.090832415033978542, -0.82095666495166841, -0.1793629803434289, -0.21667299855103739], [0.32528318239674658, -0.49986609951053218, -0.63688004623043692, -0.97178857833490007, -0.31844436113856789]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.77311242286876292, 0.44908744692280189, 0.83226215240998358, 0.52087289713254514, 0.38641318195812535], [0.80571324980523062, -0.12604666338806214, 0.8510608849491661, 0.95600963684451967, 0.86448510301290793], [-0.2927941485226444, -0.090832415033978542, 0.36678788811442398, 0.19832437648441004, -0.0214211514409548], [0.32528318239674658, 0.94793132621096565, 0.046753157348639496, -0.039652758267353594, -0.31844436113856789]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[0.55586423371079974, 0.1892755317946877, 0.10847290719935687, 0.98536651351785176, 0.40074483077480161], [0.40934295572734603, -0.40703498663658744, 0.90664485297735964, 0.37409425109294903, 0.41403477923560916], [0.43123719782172043, 0.58710861324703756, 0.22184998986325244, 0.46732912109877556, 0.14149017002364639], [0.67107738367105441, -0.73809081654075137, -0.60804272681684313, -0.73453608096490575, 0.9874358625015538]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.895231842742)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.55586423371079974, 0.1892755317946877, 0.10847290719935687, 0.98536651351785176, 0.40074483077480161], [0.40934295572734603, -0.40703498663658744, 0.90664485297735964, 0.37409425109294903, 0.41403477923560916], [0.43123719782172043, 0.58710861324703756, 0.22184998986325244, 0.46732912109877556, 0.14149017002364639], [0.67107738367105441, -0.73809081654075137, -0.60804272681684313, -0.73453608096490575, 0.9874358625015538]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[0.031464589677843602, 0.58682052183527667, -0.67221652467702797, -0.042723615419881344, -0.63221538060837301], [0.55748833630801475, -0.29991035566219959, 0.50987850203607521, -0.17227085001296349, -0.68007885242359722], [0.29317654387431946, 0.367577769111332, 0.33176880345126025, -0.72946453502795294, -0.98044004636102478], [-0.32683797691786798, 0.56652584111113891, -0.762238360123771, -0.20438852688104192, -0.10592808532685627]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.60950508272291426, -0.35350322683055491, 0.87782416243741301, -0.42352075787518917, -0.67320203638906162], [0.51816370523211086, 0.48670585227008223, 0.26768383062899237, 0.36693593965153748, -0.83242225321794527], [-0.006013485370434335, 0.62991771260382556, -0.79253727549210473, 0.37699346345269658, 0.57768486943772257], [0.9905387052666379, -0.31991906871887998, -0.36838059810883017, -0.31734079380928337, -0.55468831864149837]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.031464589677843602, 0.58682052183527667, 0.87782416243741301, -0.042723615419881344, -0.63221538060837301], [0.55748833630801475, 0.48670585227008223, 0.50987850203607521, 0.36693593965153748, -0.68007885242359722], [0.29317654387431946, 0.62991771260382556, 0.33176880345126025, 0.37699346345269658, 0.57768486943772257], [0.9905387052666379, 0.56652584111113891, -0.36838059810883017, -0.20438852688104192, -0.10592808532685627]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.4730468237134855, 0.080340250188812856, -0.81944086875643052, -0.098506767681945329, -0.5573341492542454], [-0.38440681213357553, 0.66568155755094494, -0.20620312454733902, 0.4569874848883233, 0.27044384864375748], [0.195540058404158, 0.34992187735181113, -0.20614423012646177, 0.09618657792128249, 0.66488703933604465], [0.7488248847520742, -0.12876223863389424, 0.62579378936737884, -0.96553767048637229, -0.73795071074858631]])+(1.-msk_arg1)*numpy.array([[0.57705445283252299, 0.95295912172459696, 0.14619043902616791, -0.80194733305638466, 0.78601648029803872], [-0.55324334911725881, 0.98107261205070739, -0.076270358399511906, 0.80642066967491277, -0.58671774610547422], [0.75602281001495553, -0.32236020811670429, 0.33247541870744768, 0.084067013568079219, -0.82826004746466086], [-0.79530514339776381, 0.97888215332810669, 0.44689990983720462, 0.21364765887818349, -0.82902352473958607]])
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.489204435704)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.4730468237134855, 0.080340250188812856, -0.48920443570417427, -0.098506767681945329, -0.48920443570417427], [-0.38440681213357553, 0.66568155755094494, -0.20620312454733902, 0.4569874848883233, 0.27044384864375748], [0.195540058404158, 0.34992187735181113, -0.20614423012646177, 0.09618657792128249, 0.66488703933604465], [0.7488248847520742, -0.12876223863389424, 0.62579378936737884, -0.48920443570417427, -0.48920443570417427]])+(1.-msk_ref)*numpy.array([[0.57705445283252299, 0.95295912172459696, 0.14619043902616791, -0.48920443570417427, 0.78601648029803872], [-0.48920443570417427, 0.98107261205070739, -0.076270358399511906, 0.80642066967491277, -0.48920443570417427], [0.75602281001495553, -0.32236020811670429, 0.33247541870744768, 0.084067013568079219, -0.48920443570417427], [-0.48920443570417427, 0.97888215332810669, 0.44689990983720462, 0.21364765887818349, -0.48920443570417427]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.39666351820237589, -0.7352975550560068, -0.11015957426400935, -0.58424108198582503, -0.026323516666580282], [0.3740296945948236, 0.82459869609560044, -0.72619689102671736, 0.52044070974717171, 0.61329955513119594], [-0.87825153338839335, -0.69110457403756254, -0.40618051038714875, -0.42682250646600828, 0.66659882816185978], [-0.03478776450699228, -0.99588509909633616, -0.71844653366126976, -0.57251535227464911, 0.15819170818429984]])+(1.-msk_arg1)*numpy.array([[-0.13687397567259652, -0.40647175964694093, 0.88049888245344388, 0.56621060585902838, 0.77997450369669186], [0.20844744008272698, 0.81816420914114296, -0.37991517645443307, -0.87711264870328653, -0.06635834242538019], [0.6151897845931098, 0.79990554114277668, 0.15042066460006742, -0.83744291638551838, -0.74662500252921915], [-0.11203359139317959, -0.63423100525926923, -0.89990092815647449, -0.22412842181570958, -0.50482283300657027]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[-0.18056793740176569, 0.53044230869190945, -0.14675373044471818, -0.20989896841938638, -0.090609562556853263], [0.026850958754019771, 0.71455594536067824, 0.90780734773692839, 0.80427290442132771, 0.71868468688995968], [0.61645391497678426, -0.81210161202835485, 0.82228572590035354, 0.31715205993520246, -0.49206959275863027], [0.46469955384515593, 0.55983185688761883, -0.28121005250833164, 0.28750026513484728, -0.0035877014730121282]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.39666351820237589, 0.53044230869190945, -0.11015957426400935, -0.20989896841938638, -0.026323516666580282], [0.3740296945948236, 0.82459869609560044, 0.90780734773692839, 0.80427290442132771, 0.71868468688995968], [0.61645391497678426, -0.69110457403756254, 0.82228572590035354, 0.31715205993520246, 0.66659882816185978], [0.46469955384515593, 0.55983185688761883, -0.28121005250833164, 0.28750026513484728, 0.15819170818429984]])+(1.-msk_ref)*numpy.array([[-0.13687397567259652, 0.53044230869190945, 0.88049888245344388, 0.56621060585902838, 0.77997450369669186], [0.20844744008272698, 0.81816420914114296, 0.90780734773692839, 0.80427290442132771, 0.71868468688995968], [0.61645391497678426, 0.79990554114277668, 0.82228572590035354, 0.31715205993520246, -0.49206959275863027], [0.46469955384515593, 0.55983185688761883, -0.28121005250833164, 0.28750026513484728, -0.0035877014730121282]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[0.29339754233379134, 0.0022108846355042289], [-0.70281282396389044, -0.17913328863274702]], [[0.95083613182725002, -0.33360040205006802], [-0.72360459828560386, -0.63771632185237914]], [[-0.23174499618243161, 0.25783144021821069], [0.52292264961498547, -0.037581142208750862]], [[-0.33492137217380225, -0.92909994539363883], [-0.29294994252202189, 0.032771080938205577]], [[-0.33963115491622209, -0.4560184398105811], [-0.38704925082990704, -0.36905793427294387]], [[-0.097828004568041749, 0.93545203751918193], [-0.26423008668138226, -0.47432814341661977]]])
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.716419255352)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.29339754233379134, 0.0022108846355042289], [-0.70281282396389044, -0.17913328863274702]], [[0.95083613182725002, -0.33360040205006802], [-0.7164192553517521, -0.63771632185237914]], [[-0.23174499618243161, 0.25783144021821069], [0.52292264961498547, -0.037581142208750862]], [[-0.33492137217380225, -0.7164192553517521], [-0.29294994252202189, 0.032771080938205577]], [[-0.33963115491622209, -0.4560184398105811], [-0.38704925082990704, -0.36905793427294387]], [[-0.097828004568041749, 0.93545203751918193], [-0.26423008668138226, -0.47432814341661977]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[-0.6764874487979744, -0.77301471366395913], [0.93472250718477645, 0.94134766765736844]], [[-0.2902843413852163, 0.66766850628321395], [-0.84489262052202574, -0.88419449853196341]], [[-0.18912439533545444, 0.41945117999069748], [0.86905852107269266, -0.35644548972390622]], [[0.79284675788109782, -0.90079211040313822], [-0.37667354978120082, 0.32001422801363355]], [[0.99719108917820387, 0.20935549823961486], [-0.11340161949072614, -0.12111595365420702]], [[0.22719684872202683, 0.84433437986117599], [0.43006222467797817, 0.034934288834081784]]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[[0.88636166698982444, -0.6840838071931834], [0.89183726534345209, -0.60038977276729177]], [[-0.52566599572237149, 0.85602758932345857], [-0.90144110979166236, 0.34377038179269848]], [[-0.40687327227202053, 0.92541998919612278], [0.33112275763245225, 0.82982998871415004]], [[-0.22107841184853938, -0.17412319620151218], [0.023732852133522497, 0.38468457701812753]], [[0.24155039570276626, -0.21673544374962583], [-0.39727152459399528, -0.93415688425435017]], [[-0.42630542124350135, -0.46625312593790635], [0.91604919497418402, 0.74853205663615796]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.88636166698982444, -0.6840838071931834], [0.93472250718477645, 0.94134766765736844]], [[-0.2902843413852163, 0.85602758932345857], [-0.84489262052202574, 0.34377038179269848]], [[-0.18912439533545444, 0.92541998919612278], [0.86905852107269266, 0.82982998871415004]], [[0.79284675788109782, -0.17412319620151218], [0.023732852133522497, 0.38468457701812753]], [[0.99719108917820387, 0.20935549823961486], [-0.11340161949072614, -0.12111595365420702]], [[0.22719684872202683, 0.84433437986117599], [0.91604919497418402, 0.74853205663615796]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s0=numpy.array(0.890464917105)
      s1=numpy.array([[[-0.47542536293629367, 0.08791282309219306], [0.8318730496857536, 0.66675946379122975]], [[0.12035381179926241, -0.62237239525026067], [-0.73265350863233625, 0.73451543217895621]], [[0.38525462023648638, 0.25181197027127888], [0.61968014849451936, 0.64752205758820613]], [[-0.96274486374913337, -0.33713474488128448], [0.67669977294816919, -0.22462212781416113]], [[0.3264230846866516, -0.27333933805359933], [0.66932275948701769, 0.038019088948836988]], [[-0.64261933418664063, 0.055528829058187457], [-0.76546743375303739, 0.1942798776976693]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]], [[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]], [[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]], [[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]], [[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]], [[0.89046491710527387, 0.89046491710527387], [0.89046491710527387, 0.89046491710527387]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s0=numpy.array([[[-0.18329481115264645, -0.97291108344147026], [-0.38503671080109481, 0.0254278700851025]], [[0.82486781226151185, 0.49859643168932766], [0.18958641241052931, -0.63059786697466569]], [[-0.96351285266090114, -0.11392388029447842], [-0.73545044716290309, -0.38415334680566304]], [[-0.70313895536108784, 0.86972453577345332], [0.9050989874114006, 0.20147143164852377]], [[-0.57693374895352312, -0.05702146868636282], [-0.024109084445333107, -0.088564104987937098]], [[-0.58633360287709313, 0.56710388407121859], [0.18820564679737162, -0.66919840474772596]]])
      s1=numpy.array([[[0.86414457908562214, 0.79359108960091551], [-0.10889162500414629, -0.50034676527335464]], [[-0.23814875115543277, -0.27663541315593543], [-0.32396929594753199, 0.4737724548108182]], [[-0.62312306630586245, -0.88502401546629295], [0.3881378252904939, 0.77539174161884361]], [[-0.54747818934173753, -0.84873885373197577], [-0.80592241488734295, -0.31580118674907953]], [[-0.9827146643857787, 0.86523259083675219], [0.96150783187292443, 0.98551482766924514]], [[0.35619038990324681, -0.73087960831237919], [0.32362013433909187, -0.080640095423787139]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.86414457908562214, 0.79359108960091551], [-0.10889162500414629, 0.0254278700851025]], [[0.82486781226151185, 0.49859643168932766], [0.18958641241052931, 0.4737724548108182]], [[-0.62312306630586245, -0.11392388029447842], [0.3881378252904939, 0.77539174161884361]], [[-0.54747818934173753, 0.86972453577345332], [0.9050989874114006, 0.20147143164852377]], [[-0.57693374895352312, 0.86523259083675219], [0.96150783187292443, 0.98551482766924514]], [[0.35619038990324681, 0.56710388407121859], [0.32362013433909187, -0.080640095423787139]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[0.32568852433408835, -0.55272129611930398], [-0.084149817978555586, -0.34274401865821558]], [[-0.47726176952042931, -0.34631937280026692], [-0.035216561520925671, -0.73249178043905627]], [[0.78009883802350832, 0.44360970602923189], [0.31506751179210712, -0.23113435042136166]], [[0.0023405479084084302, -0.46422556758653588], [0.6454266161015072, -0.96616888863495443]], [[0.9620354028039293, 0.60396559800614602], [-0.37745027478201343, 0.44192109227544019]], [[-0.54306608746890705, 0.075695529806122153], [-0.95678605433303199, 0.64704122328032687]]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array(0.633627188844)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.63362718884400193, 0.63362718884400193], [0.63362718884400193, 0.63362718884400193]], [[0.63362718884400193, 0.63362718884400193], [0.63362718884400193, 0.63362718884400193]], [[0.78009883802350832, 0.63362718884400193], [0.63362718884400193, 0.63362718884400193]], [[0.63362718884400193, 0.63362718884400193], [0.6454266161015072, 0.63362718884400193]], [[0.9620354028039293, 0.63362718884400193], [0.63362718884400193, 0.63362718884400193]], [[0.63362718884400193, 0.63362718884400193], [0.63362718884400193, 0.64704122328032687]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[0.85692258587753933, -0.10129388170266518], [0.8110137002800879, 0.67377608274648737]], [[-0.71752398637016057, -0.58951427845265014], [0.30156121045716056, 0.66789438832765269]], [[0.25674121894260526, 0.50337004809901376], [-0.27534576699195612, 0.0069744298369271984]], [[0.10411830060243932, -0.38454331052123525], [-0.14398299168325446, -0.36725433436330102]], [[-0.076454899963227296, 0.36804698909232925], [-0.76660768351290121, 0.42166893789969384]], [[0.76306470750109567, 0.90405926947092086], [-0.69746358167885059, -0.57398383152198784]]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[0.050483403914534986, 0.6274385153472164], [-0.82735136253802311, -0.50120424225576699]], [[0.71550161154713776, -0.045329688560096448], [0.36829020813589053, -0.44341986442180992]], [[0.72297168963475511, -0.91716624719047912], [0.57398208106426352, 0.22082803410655294]], [[-0.45336261658274313, -0.92069962652331472], [0.82130013830648618, 0.60823879246569246]], [[0.33707620838660768, 0.56989518375507564], [0.28427029363757916, -0.50013875183284107]], [[-0.6058190127748484, 0.76654727297616265], [0.91096982261018966, -0.82329820822579647]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.85692258587753933, 0.6274385153472164], [0.8110137002800879, 0.67377608274648737]], [[0.71550161154713776, -0.045329688560096448], [0.36829020813589053, 0.66789438832765269]], [[0.72297168963475511, 0.50337004809901376], [0.57398208106426352, 0.22082803410655294]], [[0.10411830060243932, -0.38454331052123525], [0.82130013830648618, 0.60823879246569246]], [[0.33707620838660768, 0.56989518375507564], [0.28427029363757916, 0.42166893789969384]], [[0.76306470750109567, 0.90405926947092086], [0.91096982261018966, -0.57398383152198784]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.57653628850840333, 0.19094416526641789], [0.84417250154613299, -0.37411027843502231]], [[0.32126078398907976, 0.44944398340492597], [-0.11146170994432114, 0.11597337950678299]], [[-0.70285763359083875, -0.15598356223672916], [0.76426158039335346, 0.12007672470791309]], [[-0.65693831034340433, 0.84391397019686054], [-0.64829210999691655, 0.2219351949329178]], [[0.66709305263693874, -0.51781394274008274], [-0.86385545861254398, 0.68197749050561574]], [[-0.41293796354927248, 0.90505950749329767], [0.25110585703000177, 0.24781032994023589]]])+(1.-msk_arg1)*numpy.array([[[0.11132526506740836, -0.52402219445034026], [0.1828750046221308, -0.53065346009666436]], [[-0.80156405080561766, 0.63718840971056601], [-0.32283315325508433, 0.093742341886728919]], [[-0.93420729238299116, 0.22189922926071115], [-0.87128641182657129, 0.40088846571558312]], [[-0.77881045840063501, -0.3430723237069313], [-0.8013638431535326, 0.6571865235565697]], [[-0.82407092416622141, -0.51730688792872059], [-0.93104098770035315, 0.97332671737783594]], [[0.52127502902327527, -0.84531224506343094], [-0.59547145607180307, -0.56618082649726631]]])
      res=maximum(arg0,arg1)
      s0=numpy.array(0.860369251457)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.90505950749329767], [0.86036925145669096, 0.86036925145669096]]])+(1.-msk_ref)*numpy.array([[[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.97332671737783594]], [[0.86036925145669096, 0.86036925145669096], [0.86036925145669096, 0.86036925145669096]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.010516777640972697, 0.60120064014605079], [-0.84460937064504482, -0.84625744776917022]], [[0.77962126254035158, 0.74164858425755464], [0.068202087346799223, 0.43265821004094707]], [[-0.55362398279753666, -0.27296959892978401], [-0.65328622442728768, -0.98672121384052991]], [[-0.63456176421577282, 0.62179697944598677], [-0.57625841517839804, 0.18574353119346165]], [[-0.08269199537899552, -0.99017991590928767], [0.52429151031264221, 0.75817259458900188]], [[-0.91460575509654629, 0.37417073831531877], [-0.11092444682659641, 0.059566798767184848]]])+(1.-msk_arg1)*numpy.array([[[-0.70061212578129428, 0.42367670762183929], [-0.12938353673895464, -0.10833729551639437]], [[-0.18825613846812339, -0.054063591344754025], [0.9480085478536513, 0.82998380686007667]], [[-0.28076760120864064, 0.2314742338361282], [-0.92727748765633589, 0.57998598065466922]], [[-0.038914120154450949, -0.84943755339331584], [0.16297527678184132, -0.4163279958605226]], [[-0.22292385049843944, -0.77914478579712099], [0.64661943234772634, -0.31154755140327883]], [[-0.90719447570461087, 0.14069859110425642], [-0.88855105894544573, -0.15904712129137155]]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[[-0.038394462994514766, 0.20434482159129064], [-0.92498889865024703, -0.77614747980675736]], [[0.65055268191055338, 0.03706977456722016], [0.17776082394317605, -0.70060252370400433]], [[-0.36252794984515258, -0.46726405611411126], [0.90858279094238226, 0.55782533354609654]], [[-0.9073925707193462, -0.22346212785707076], [0.63239830917035333, 0.34747287392663617]], [[-0.67968889804330246, 0.71036257956938531], [-0.27666689569899106, 0.40542747615974717]], [[0.39389220084882903, -0.76306927321997686], [-0.47819005673187309, -0.39173855327967866]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.010516777640972697, 0.60120064014605079], [-0.84460937064504482, -0.77614747980675736]], [[0.77962126254035158, 0.74164858425755464], [0.17776082394317605, 0.43265821004094707]], [[-0.36252794984515258, -0.27296959892978401], [0.90858279094238226, 0.55782533354609654]], [[-0.63456176421577282, 0.62179697944598677], [0.63239830917035333, 0.34747287392663617]], [[-0.08269199537899552, 0.71036257956938531], [0.52429151031264221, 0.75817259458900188]], [[0.39389220084882903, 0.37417073831531877], [-0.11092444682659641, 0.059566798767184848]]])+(1.-msk_ref)*numpy.array([[[-0.038394462994514766, 0.42367670762183929], [-0.12938353673895464, -0.10833729551639437]], [[0.65055268191055338, 0.03706977456722016], [0.9480085478536513, 0.82998380686007667]], [[-0.28076760120864064, 0.2314742338361282], [0.90858279094238226, 0.57998598065466922]], [[-0.038914120154450949, -0.22346212785707076], [0.63239830917035333, 0.34747287392663617]], [[-0.22292385049843944, 0.71036257956938531], [0.64661943234772634, 0.40542747615974717]], [[0.39389220084882903, 0.14069859110425642], [-0.47819005673187309, -0.15904712129137155]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[-0.22672291859395366, -0.95305535759181881, -0.63790479181274184, 0.57987784525884312], [-0.74794540413203126, 0.47005773853790078, -0.97876804068726875, 0.50784074089241993], [-0.084720347523811856, 0.086904799263365362, -0.96455200508452776, -0.18173771571574249]], [[0.91235685348610307, 0.32798518943235466, 0.63562798120801833, -0.047248426437298319], [-0.076374800002460663, -0.64182255571769198, 0.03315517509218191, -0.51358838089418168], [-0.91348077240620573, -0.84832022280125785, 0.008552131049955447, 0.91798535015558236]]], [[[-0.66753395132723825, -0.17623447837007822, -0.067291695444720245, -0.82210702472202102], [-0.3985041744402551, -0.041506375166231457, -0.9656316904147455, -0.16862736477613205], [-0.4851059142640104, 0.70744547107828404, -0.47413240299001425, 0.37066254940828669]], [[-0.059631009831195625, -0.84724579994833671, 0.24955596071221731, 0.17790317305792147], [0.44866580350344543, -0.24454945597196809, -0.14943377265514424, -0.86123841015287406], [0.63558026374371668, -0.99929837742030592, 0.31210228931905615, 0.33434525579893726]]], [[[-0.17964634256279277, -0.7893763716476343, -0.90750738207750836, 0.29169359971463216], [-0.32861007822736732, -0.66839272495110058, -0.95481155969639686, 0.82484410755679716], [-0.9067452758699881, -0.96011412793796813, -0.21841535705831494, 0.3739119351759197]], [[-0.47568953719081164, -0.28247777052015177, -0.21297521816609999, 0.80573677264279486], [-0.026756106052450379, 0.089541005915227956, -0.040373521220984809, 0.45967937074998177], [0.80916344237700244, 0.52963770268056076, 0.87782054362161399, 0.50646130790152677]]]])
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.0447842290384)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, 0.57987784525884312], [-0.044784229038412837, 0.47005773853790078, -0.044784229038412837, 0.50784074089241993], [-0.044784229038412837, 0.086904799263365362, -0.044784229038412837, -0.044784229038412837]], [[0.91235685348610307, 0.32798518943235466, 0.63562798120801833, -0.044784229038412837], [-0.044784229038412837, -0.044784229038412837, 0.03315517509218191, -0.044784229038412837], [-0.044784229038412837, -0.044784229038412837, 0.008552131049955447, 0.91798535015558236]]], [[[-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, -0.044784229038412837], [-0.044784229038412837, -0.041506375166231457, -0.044784229038412837, -0.044784229038412837], [-0.044784229038412837, 0.70744547107828404, -0.044784229038412837, 0.37066254940828669]], [[-0.044784229038412837, -0.044784229038412837, 0.24955596071221731, 0.17790317305792147], [0.44866580350344543, -0.044784229038412837, -0.044784229038412837, -0.044784229038412837], [0.63558026374371668, -0.044784229038412837, 0.31210228931905615, 0.33434525579893726]]], [[[-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, 0.29169359971463216], [-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, 0.82484410755679716], [-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, 0.3739119351759197]], [[-0.044784229038412837, -0.044784229038412837, -0.044784229038412837, 0.80573677264279486], [-0.026756106052450379, 0.089541005915227956, -0.040373521220984809, 0.45967937074998177], [0.80916344237700244, 0.52963770268056076, 0.87782054362161399, 0.50646130790152677]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[0.22859584632583263, -0.49964587357132872, 0.086470776924952197, 0.42290964650936536], [0.90937469309327956, 0.52077687187185928, 0.26403290901603027, 0.23331609740121895], [0.2963983419985563, -0.7647547061296065, 0.20664027121177853, 0.43665491999143158]], [[0.87939983689267698, 0.85421523030279389, 0.049255595757613735, 0.51481804264120412], [-0.18641300303264852, -0.6253462919532744, 0.33585610636840357, -0.39349689192316073], [0.57660421473602108, -0.42166143444700266, -0.91469900193095111, 0.10045159224477196]]], [[[-0.26395757504790129, 0.39147904987782045, -0.93606882339379793, -0.84160666236095771], [0.93471768322024862, -0.24903651446950037, 0.88677565016934268, -0.60610514893049938], [0.49273310369686851, 0.071189997025720153, 0.66662351893261484, -0.9527728713146697]], [[-0.40665175803980214, -0.15058994378663826, -0.60390247676056008, 0.72933869321036848], [-0.025687743718204148, -0.68564970759152288, 0.94289809230252786, 0.2900877373795363], [0.69609393571465406, -0.69112085533563472, 0.65144978792589914, -0.33446027303627068]]], [[[0.2387465130886075, 0.090950095918787222, -0.49309589516571251, 0.59865416201774324], [0.27584589277836713, 0.49849504887710827, -0.25120995254302203, 0.13990808487517103], [0.87000785561059812, -0.42191934000096465, 0.69837171372420315, 0.98016474174914903]], [[-0.20506044289881609, -0.40268835993916308, -0.46835003820343646, 0.68307210363587179], [-0.51517708190071332, -0.33359009958121311, 0.56420273334245685, 0.40583949265330466], [-0.0089796450496348879, 0.10175894115881468, 0.84086912438464467, -0.90042955897097188]]]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[-0.61680757786633, 0.15663517224128998, -0.62028829614332648, 0.65982719025153891], [0.95700546137100795, -0.08624570184920044, 0.018220798772857982, 0.91439775658040756], [0.06454215218124193, 0.14895550091837451, -0.81344811861198529, -0.73171642983234353]], [[-0.38274374999571026, -0.24846100370907909, -0.78256200168802881, 0.10660169114209683], [0.2408553375522946, -0.30389962377182278, 0.44611850858169744, -0.032042409294563479], [-0.34020222824520974, 0.1171858663552432, 0.99457455241720139, -0.45190701309699466]]], [[[-0.82444756608053904, -0.53701809683036617, 0.80526076129721846, 0.94179515034432382], [0.90570316863078637, 0.85026811413861325, -0.084328716034264062, 0.583858420302791], [-0.07503991344527372, -0.47842084538144136, 0.52532459198516146, 0.99015498512300959]], [[0.76475532370930455, 0.08568167696132778, 0.80666906071672861, 0.70540823249171347], [-0.92088313169158775, -0.71294774283312257, -0.50254617846594418, 0.69941402937188957], [0.68299307069766102, 0.20851158208497278, 0.97007859976731114, 0.89088997676979775]]], [[[0.74280261126787206, -0.79908745531265524, 0.28808238813408593, -0.9108026839182346], [-0.89323119175824472, 0.90400075232487431, -0.25196671766511436, -0.85718534216330911], [-0.44750354328292374, -0.63380481448691062, 0.02050948379246087, -0.83727980836704408]], [[-0.84104946225264698, 0.93174581258496203, -0.28512648036513899, -0.95377121876013171], [-0.29080440585024547, -0.40825991866604361, 0.97458611849762788, -0.034373434324477481], [0.31658722463091427, -0.5718901172479478, 0.36544453487551487, 0.55385738177950961]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.22859584632583263, 0.15663517224128998, 0.086470776924952197, 0.65982719025153891], [0.95700546137100795, 0.52077687187185928, 0.26403290901603027, 0.91439775658040756], [0.2963983419985563, 0.14895550091837451, 0.20664027121177853, 0.43665491999143158]], [[0.87939983689267698, 0.85421523030279389, 0.049255595757613735, 0.51481804264120412], [0.2408553375522946, -0.30389962377182278, 0.44611850858169744, -0.032042409294563479], [0.57660421473602108, 0.1171858663552432, 0.99457455241720139, 0.10045159224477196]]], [[[-0.26395757504790129, 0.39147904987782045, 0.80526076129721846, 0.94179515034432382], [0.93471768322024862, 0.85026811413861325, 0.88677565016934268, 0.583858420302791], [0.49273310369686851, 0.071189997025720153, 0.66662351893261484, 0.99015498512300959]], [[0.76475532370930455, 0.08568167696132778, 0.80666906071672861, 0.72933869321036848], [-0.025687743718204148, -0.68564970759152288, 0.94289809230252786, 0.69941402937188957], [0.69609393571465406, 0.20851158208497278, 0.97007859976731114, 0.89088997676979775]]], [[[0.74280261126787206, 0.090950095918787222, 0.28808238813408593, 0.59865416201774324], [0.27584589277836713, 0.90400075232487431, -0.25120995254302203, 0.13990808487517103], [0.87000785561059812, -0.42191934000096465, 0.69837171372420315, 0.98016474174914903]], [[-0.20506044289881609, 0.93174581258496203, -0.28512648036513899, 0.68307210363587179], [-0.29080440585024547, -0.33359009958121311, 0.97458611849762788, 0.40583949265330466], [0.31658722463091427, 0.10175894115881468, 0.84086912438464467, 0.55385738177950961]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s0=numpy.array(-0.937781883941)
      s1=numpy.array([[[[0.92592943775670555, 0.56445054840164621, -0.25798705131114419, -0.27720297339693656], [-0.5652057107656574, 0.71678840553271073, 0.26978498950813479, -0.81781280858706773], [-0.48007042378859066, 0.17887934717613962, 0.99045825227467987, -0.58914933265372804]], [[0.99922352095347478, 0.78262443546427352, -0.6520513942976085, -0.98291868592156351], [0.42952714706948503, -0.93783630911422833, 0.95894179727622619, 0.058853528582238912], [0.20846925403146854, -0.28138640206454313, -0.76485106306715434, -0.64242994143006227]]], [[[-0.76502499326976969, 0.22620747215356052, 0.15169968969400927, -0.28799122362178875], [-0.82468028483698341, 0.6853648721565524, 0.53618639251762268, -0.11504255580107769], [0.12016962029113687, -0.77624929571996182, 0.90332828825434164, 0.2649634762981079]], [[-0.70086154589199312, -0.72325037997451447, 0.90912045763140115, 0.59709754203889753], [-0.55693372356409254, 0.58996132388905598, 0.47997419252808293, -0.43712539184888555], [0.057999770468728284, -0.33362734244199643, 0.23278512046834376, -0.78673643505995416]]], [[[-0.67120184187944765, 0.039761256453554328, -0.82742387688294405, 0.83056912562260021], [0.0869773421732154, 0.036561727406758671, 0.67104424586236866, -0.68393955962414044], [0.29400476749493953, 0.60177470500994712, 0.84357347160566754, -0.35300399257861725]], [[-0.31642290502234882, -0.67819469512506481, -0.80321923418657626, -0.75130669583454446], [0.62276618423751628, 0.53085489780323525, 0.75723916853289319, -0.068956986552453969], [-0.65219707335512345, -0.52298443716825527, 0.74268126750312313, 0.064268626156129161]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.92592943775670555, 0.56445054840164621, -0.25798705131114419, -0.27720297339693656], [-0.5652057107656574, 0.71678840553271073, 0.26978498950813479, -0.81781280858706773], [-0.48007042378859066, 0.17887934717613962, 0.99045825227467987, -0.58914933265372804]], [[0.99922352095347478, 0.78262443546427352, -0.6520513942976085, -0.93778188394110451], [0.42952714706948503, -0.93778188394110451, 0.95894179727622619, 0.058853528582238912], [0.20846925403146854, -0.28138640206454313, -0.76485106306715434, -0.64242994143006227]]], [[[-0.76502499326976969, 0.22620747215356052, 0.15169968969400927, -0.28799122362178875], [-0.82468028483698341, 0.6853648721565524, 0.53618639251762268, -0.11504255580107769], [0.12016962029113687, -0.77624929571996182, 0.90332828825434164, 0.2649634762981079]], [[-0.70086154589199312, -0.72325037997451447, 0.90912045763140115, 0.59709754203889753], [-0.55693372356409254, 0.58996132388905598, 0.47997419252808293, -0.43712539184888555], [0.057999770468728284, -0.33362734244199643, 0.23278512046834376, -0.78673643505995416]]], [[[-0.67120184187944765, 0.039761256453554328, -0.82742387688294405, 0.83056912562260021], [0.0869773421732154, 0.036561727406758671, 0.67104424586236866, -0.68393955962414044], [0.29400476749493953, 0.60177470500994712, 0.84357347160566754, -0.35300399257861725]], [[-0.31642290502234882, -0.67819469512506481, -0.80321923418657626, -0.75130669583454446], [0.62276618423751628, 0.53085489780323525, 0.75723916853289319, -0.068956986552453969], [-0.65219707335512345, -0.52298443716825527, 0.74268126750312313, 0.064268626156129161]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[-0.41185158772809816, -0.59982439829929213, 0.11035526264174855, 0.39680396057713008], [0.60105426824013541, -0.32554905710316118, -0.38720375782902061, -0.1230271637826692], [-0.36164125397460012, 0.4041305639265893, 0.80684751796422649, -0.54389849602118989]], [[0.12472227670948288, 0.92001176547242602, -0.53583665251309198, -0.79032558890899129], [0.54303166570985795, 0.48730482750117066, 0.33536853423325219, -0.54405000648665203], [-0.90607601243187474, 0.77382684624419751, -0.09417864043275137, 0.75870003109114315]]], [[[0.63307410041973133, 0.8749566105768054, 0.37137382211609604, -0.77241284196092486], [0.15099008123405033, -0.73322216160913833, 0.28818404627765082, -0.83382706674975848], [-0.33206123027783407, -0.21371543078156408, 0.45839980899754873, 0.043354495720617336]], [[0.86256379359574797, 0.66782583537908891, -0.81777752559626249, 0.78678591492228267], [0.51236091465673694, -0.72912156504826964, 0.0011570183078764007, 0.29511700060869672], [-0.393514154152099, 0.72122139512956962, 0.22467497719317753, 0.0041654407953581885]]], [[[0.18146859353374323, 0.81309569349675392, -0.81543468728596191, -0.074289420750079715], [0.53380418923195894, 0.8477070735234058, 0.63008475217631887, -0.93684501515887053], [0.53706108758955562, -0.75639300486800209, -0.78351760363312195, -0.0026207614695039627]], [[-0.69087067123536605, 0.66220945531535946, -0.28646796185831502, -0.3640488631126737], [-0.70812099325081168, -0.081344895374056536, 0.64822549559257725, -0.025343484747048395], [-0.35262878521908503, 0.10534583481913495, -0.16963208595434542, 0.19534864838589616]]]])
      s1=numpy.array([[[[-0.044026313657740568, -0.78289814754355991, -0.70631058379045086, -0.52630435626334537], [0.9941097210486276, 0.7832949599807022, 0.28014575812793852, 0.05499629138443618], [0.91172429747596095, 0.51261079394706144, -0.81489687296645696, -0.16793882693190709]], [[-0.27046168009988736, 0.55398024095927823, 0.9492769973969879, -0.95147127755487593], [0.36824292958036708, -0.75966475779442799, -0.30187376724003312, 0.58898873841549482], [-0.99149051087675466, 0.95409690829694305, 0.34315692772672723, -0.55052047842092144]]], [[[-0.40959529608800405, -0.83174043986179091, -0.86670058576008313, -0.84047612074360467], [0.88661124543070624, 0.18009677741923502, -0.23825962030849501, 0.97781897017306307], [-0.25874554084403534, -0.49843208360473157, -0.85346777142566954, 0.98088469776303366]], [[0.46575327128539068, -0.97363923789027806, 0.31682900574101303, -0.58163959606138071], [-0.90383714625603595, 0.61983020748554929, -0.612894581271892, -0.43586899250922917], [-0.19735545140454702, 0.55375847781843635, -0.60601302849984973, -0.13122392524722182]]], [[[0.23212848220934368, 0.067639769889185519, 0.9949842666254769, -0.21102948960426482], [-0.48236152817826539, 0.38788767207762631, -0.023756490935391561, 0.77293934708851308], [0.56560004546728981, -0.64444227316016023, 0.42210933518530669, -0.52287550086514023]], [[-0.22095075813574039, -0.35204310530806149, -0.16366473997989539, -0.93633079033553201], [-0.16133682240062486, -0.80043300787665395, -0.47081233283657675, 0.62802945832598467], [-0.48214648563163132, -0.19446689423171892, 0.80158933715665071, 0.79093314251552416]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.044026313657740568, -0.59982439829929213, 0.11035526264174855, 0.39680396057713008], [0.9941097210486276, 0.7832949599807022, 0.28014575812793852, 0.05499629138443618], [0.91172429747596095, 0.51261079394706144, 0.80684751796422649, -0.16793882693190709]], [[0.12472227670948288, 0.92001176547242602, 0.9492769973969879, -0.79032558890899129], [0.54303166570985795, 0.48730482750117066, 0.33536853423325219, 0.58898873841549482], [-0.90607601243187474, 0.95409690829694305, 0.34315692772672723, 0.75870003109114315]]], [[[0.63307410041973133, 0.8749566105768054, 0.37137382211609604, -0.77241284196092486], [0.88661124543070624, 0.18009677741923502, 0.28818404627765082, 0.97781897017306307], [-0.25874554084403534, -0.21371543078156408, 0.45839980899754873, 0.98088469776303366]], [[0.86256379359574797, 0.66782583537908891, 0.31682900574101303, 0.78678591492228267], [0.51236091465673694, 0.61983020748554929, 0.0011570183078764007, 0.29511700060869672], [-0.19735545140454702, 0.72122139512956962, 0.22467497719317753, 0.0041654407953581885]]], [[[0.23212848220934368, 0.81309569349675392, 0.9949842666254769, -0.074289420750079715], [0.53380418923195894, 0.8477070735234058, 0.63008475217631887, 0.77293934708851308], [0.56560004546728981, -0.64444227316016023, 0.42210933518530669, -0.0026207614695039627]], [[-0.22095075813574039, 0.66220945531535946, -0.16366473997989539, -0.3640488631126737], [-0.16133682240062486, -0.081344895374056536, 0.64822549559257725, 0.62802945832598467], [-0.35262878521908503, 0.10534583481913495, 0.80158933715665071, 0.79093314251552416]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[-0.50685759473576719, 0.93542139233168409, 0.86091578348248921, 0.58494539092709275], [0.91744132050650751, -0.12090084241685783, -0.22425047634430584, -0.26763897748139587], [-0.47215008025389005, 0.63019999532793292, -0.40899409051397306, 0.22804192071663487]], [[-0.11719848975372549, -0.74613076576175752, 0.81912278588233622, 0.52233933993291282], [0.15105709321543803, -0.33275004001129749, -0.82835368404702359, 0.34215784767967183], [0.042821885170873708, 0.41013155832171266, 0.76046715154107147, -0.12793062147458967]]], [[[0.34105576406804095, 0.64203616232599203, 0.9792104464266489, 0.73330843417008174], [0.11804732097582948, 0.32724230927205689, -0.93531377686595452, -0.032105704410765101], [0.26531057306554207, -0.84810983561679953, 0.69312850590289665, -0.99315140646823163]], [[-0.01552242416667271, 0.70315763041634494, 0.15581981807010492, -0.82813194303662341], [-0.36750586241714367, 0.29968282135195801, -0.73956697022543305, -0.096361557223572314], [0.8611274306722938, 0.22786652896860193, -0.63216669039665208, 0.62383613485301503]]], [[[-0.020105085338268269, 0.79313636338363569, -0.18076384556191449, -0.88296710086038188], [0.64143630686143704, -0.45499259129938707, 0.11284017505285804, -0.15283896664140451], [0.53237462591611662, -0.80884202719114895, -0.620808123572411, -0.31888518763473628]], [[0.7599060453656159, 0.57588666706212166, 0.34391524109001259, 0.38132425560502026], [0.76217435250233434, -0.8090981677400646, 0.33451694900534701, 0.7541228619916136], [-0.56450367450664252, 0.34989342045913352, 0.59826501510005237, 0.43924192680866425]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array(0.483272599339)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.48327259933866151, 0.93542139233168409, 0.86091578348248921, 0.58494539092709275], [0.91744132050650751, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151], [0.48327259933866151, 0.63019999532793292, 0.48327259933866151, 0.48327259933866151]], [[0.48327259933866151, 0.48327259933866151, 0.81912278588233622, 0.52233933993291282], [0.48327259933866151, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151], [0.48327259933866151, 0.48327259933866151, 0.76046715154107147, 0.48327259933866151]]], [[[0.48327259933866151, 0.64203616232599203, 0.9792104464266489, 0.73330843417008174], [0.48327259933866151, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151], [0.48327259933866151, 0.48327259933866151, 0.69312850590289665, 0.48327259933866151]], [[0.48327259933866151, 0.70315763041634494, 0.48327259933866151, 0.48327259933866151], [0.48327259933866151, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151], [0.8611274306722938, 0.48327259933866151, 0.48327259933866151, 0.62383613485301503]]], [[[0.48327259933866151, 0.79313636338363569, 0.48327259933866151, 0.48327259933866151], [0.64143630686143704, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151], [0.53237462591611662, 0.48327259933866151, 0.48327259933866151, 0.48327259933866151]], [[0.7599060453656159, 0.57588666706212166, 0.48327259933866151, 0.48327259933866151], [0.76217435250233434, 0.48327259933866151, 0.48327259933866151, 0.7541228619916136], [0.48327259933866151, 0.48327259933866151, 0.59826501510005237, 0.48327259933866151]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[0.81291500942743689, -0.68582699458955609, 0.81352298902763742, 0.39781543546080167], [-0.058821412679685325, 0.084121425869268407, 0.70267202907861503, 0.64973936665382448], [-0.83557781519503593, 0.83519559824905709, 0.3509596814867979, 0.44069192607057572]], [[0.58645256429834958, -0.93225362283840862, 0.68570059820099405, -0.54348414064253125], [0.76917587572857382, 0.95347446589624063, -0.60997445954200913, 0.64429654703588657], [0.66222501403614586, 0.54611501765355408, -0.86310550948589104, 0.7061027344391495]]], [[[-0.66032843373230898, -0.70116000754862728, 0.98886901186760179, 0.60211469163098008], [-0.30672337825078611, -0.67158566276763154, -0.18554254910322188, -0.97453904134729852], [0.20690850500924474, -0.69845742100687702, 0.77379175383854171, 0.63817634019129077]], [[0.20347185863536654, -0.73037920605041928, -0.73059576888121858, -0.45781414409932242], [0.97450335589680859, 0.48952866118620597, 0.60198834990491656, -0.22026887284307417], [0.83793237855262115, 0.68924996250989179, 0.43909589165101726, -0.84649279842798819]]], [[[0.06762289208374539, 0.003485215342493575, -0.72219871508048983, -0.42979286443903852], [-0.18910628954926922, 0.3733743927355937, 0.4847674701588891, -0.94903484938792149], [0.92474942550438666, 0.29176241363103461, -0.68790225227442536, 0.8333730304177811]], [[0.75479445958320479, -0.20679565120324828, -0.6850961798536086, 0.53947605724380332], [-0.14225936952298301, 0.67875317157661486, -0.3548525982400359, 0.66203649107645157], [-0.87976283666119182, 0.0415683267122815, 0.56826126103931318, 0.089739757320835167]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[0.050681449842814974, -0.67051226777755391, 0.7906875326213878, 0.64802026846703398], [0.85274872767822263, -0.80912481523552904, 0.45310189322429273, -0.084009843756123193], [-0.20643249494243965, 0.5668766730144652, -0.20085534006470773, 0.5195238989056008]], [[0.22857005100313321, 0.048344182879428654, 0.37591809139105736, 0.09122669092525082], [0.091700487355204308, -0.9985039657422683, -0.4164303769076676, 0.9730391888230423], [0.36112212740007332, -0.77057750340490272, -0.89856339703576826, 0.021324460922719446]]], [[[-0.73448968801729797, 0.30575249723121178, 0.14892014298547984, -0.53107217197436607], [0.004812431898059355, -0.69992003674657433, -0.610488723221569, 0.64811312410943445], [0.22997105622251302, 0.11835539098936265, 0.8051190908659922, 0.31101048403753206]], [[0.84520047859214009, 0.25782980818567158, 0.013268176660491626, -0.33594843665530849], [0.64378241035921802, 0.69093768187104443, 0.39258728918190489, -0.18621134440879072], [-0.23302256553155276, -0.60443179190698082, -0.19660253238619974, 0.80036365200337034]]], [[[-0.07553608083911012, 0.60965867572354493, 0.85789932062393204, -0.81568652450890067], [0.80018980915321358, -0.23001896388500498, -0.24992976621779284, -0.6478071279090023], [-0.32424232929747721, 0.88731416443351541, -0.96115335133988533, 0.66653897919244987]], [[-0.67397552072494538, 0.20750517083663955, 0.84685381358684109, 0.66338214188449807], [-0.14628962409645374, 0.71081961187757159, -0.73926084700672257, -0.7162326018702152], [0.42110979033605678, 0.47593666296419634, -0.35853593738739398, -0.016857828821966292]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.81291500942743689, -0.67051226777755391, 0.81352298902763742, 0.64802026846703398], [0.85274872767822263, 0.084121425869268407, 0.70267202907861503, 0.64973936665382448], [-0.20643249494243965, 0.83519559824905709, 0.3509596814867979, 0.5195238989056008]], [[0.58645256429834958, 0.048344182879428654, 0.68570059820099405, 0.09122669092525082], [0.76917587572857382, 0.95347446589624063, -0.4164303769076676, 0.9730391888230423], [0.66222501403614586, 0.54611501765355408, -0.86310550948589104, 0.7061027344391495]]], [[[-0.66032843373230898, 0.30575249723121178, 0.98886901186760179, 0.60211469163098008], [0.004812431898059355, -0.67158566276763154, -0.18554254910322188, 0.64811312410943445], [0.22997105622251302, 0.11835539098936265, 0.8051190908659922, 0.63817634019129077]], [[0.84520047859214009, 0.25782980818567158, 0.013268176660491626, -0.33594843665530849], [0.97450335589680859, 0.69093768187104443, 0.60198834990491656, -0.18621134440879072], [0.83793237855262115, 0.68924996250989179, 0.43909589165101726, 0.80036365200337034]]], [[[0.06762289208374539, 0.60965867572354493, 0.85789932062393204, -0.42979286443903852], [0.80018980915321358, 0.3733743927355937, 0.4847674701588891, -0.6478071279090023], [0.92474942550438666, 0.88731416443351541, -0.68790225227442536, 0.8333730304177811]], [[0.75479445958320479, 0.20750517083663955, 0.84685381358684109, 0.66338214188449807], [-0.14225936952298301, 0.71081961187757159, -0.3548525982400359, 0.66203649107645157], [0.42110979033605678, 0.47593666296419634, 0.56826126103931318, 0.089739757320835167]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.72453400015814773, 0.74968555354910205, 0.8864462807329363, -0.12876802818277633], [0.64036866843529117, 0.23169677841426406, -0.27490410632243645, 0.52995296066672637], [-0.23465182365842896, -0.15412558193730219, -0.77218295750237198, 0.85143952116619448]], [[-0.79604804377473992, 0.46234931443574956, 0.97015610088983983, -0.91555740188161483], [-0.9372279642794441, 0.73653887107290172, 0.31312323353269611, 0.24774753287309492], [-0.21436562878991916, -0.7379793369973604, 0.55165617866786709, -0.48025841889867382]]], [[[0.035086815901544499, -0.97968557903833964, 0.31741740083453585, 0.54396195668473357], [0.55497611231869826, 0.36509009344306453, 0.39845329246782635, 0.84456796768111664], [0.9632050535497847, -0.44559189726273463, 0.2853969566976482, -0.033412748200378095]], [[0.72262134138721423, 0.12862427339658367, -0.4447384602871618, 0.50148443724376679], [-0.57321307543500222, -0.50047778281013655, -0.41258254604157951, 0.23232139132588125], [0.82816332075151888, -0.26837013198422977, 0.29243538695633298, 0.33535389672240479]]], [[[-0.47838691303255509, -0.65502321532395213, -0.060413524045078715, 0.35595272059037475], [0.36754116633167322, -0.77596556004081929, -0.075032377463689404, -0.80441957648586815], [-0.66004735163369821, -0.71714608090292464, 0.086736521512553288, 0.83967553771133496]], [[0.32380435178041012, -0.45076176762655806, -0.66376594567140113, -0.97260810637511352], [-0.70002953550259828, 0.70218111817212181, 0.3916836054380437, -0.099288820926793475], [-0.22602506461069805, -0.31956317606771778, -0.21834314763125384, -0.5491632419972341]]]])+(1.-msk_arg1)*numpy.array([[[[-0.89455511489685757, 0.94599383179617269, -0.58100108642594273, 0.74289184873547831], [0.79825297278664165, 0.49488990645198694, -0.81805774319192337, 0.7723478579687284], [0.18720418817675766, -0.6973485780893558, -0.54851329989453057, 0.83060779510514626]], [[-0.24324518631955816, -0.39189601657073014, -0.41943664184408846, -0.16449107937419249], [0.43816203077572458, 0.10547053767896508, 0.76319682823895252, 0.97067330199334001], [0.20407509728020412, -0.050720884235648134, 0.03555023147139158, 0.88567082924827845]]], [[[-0.39965892542940051, 0.41687140772441889, -0.074958423452218481, -0.24937935512687437], [-0.51038725497925874, -0.69682476026124873, 0.48908614254851068, -0.54496539335394378], [-0.43259036138330997, -0.55392539318519085, 0.35042405862448756, -0.58066340287718909]], [[-0.54195724470367024, 0.60173079206236246, -0.91270254249214222, 0.88695387596924524], [-0.2177174630230041, -0.55124617487945526, -0.57352866746381581, -0.45536381236459023], [-0.20681805646616924, -0.7302209979352523, 0.38046946864336317, -0.48623067156976862]]], [[[-0.19057597861210906, 0.27329850884929718, -0.97321381433423815, -0.4382382110386509], [0.56177584764962507, 0.67326301723647863, -0.96143448952855937, 0.98748707230701482], [0.74667864479466339, 0.3402524282291004, -0.81662031017514014, 0.44794940107754022]], [[0.47828453909135282, -0.092501250116698186, 0.70462443973450162, 0.079988260487481755], [0.22084791725555908, -0.18427573695410504, 0.79639900215468074, -0.52192450099142129], [-0.86805278096187766, 0.80672822476952732, -0.69960387443364214, 0.24071506414240629]]]])
      res=maximum(arg0,arg1)
      s0=numpy.array(0.603319875729)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.72453400015814773, 0.74968555354910205, 0.8864462807329363, 0.6033198757288365], [0.64036866843529117, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.85143952116619448]], [[0.6033198757288365, 0.6033198757288365, 0.97015610088983983, 0.6033198757288365], [0.6033198757288365, 0.73653887107290172, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]]], [[[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.84456796768111664], [0.9632050535497847, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]], [[0.72262134138721423, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.82816332075151888, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]]], [[[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.83967553771133496]], [[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.70218111817212181, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]]]])+(1.-msk_ref)*numpy.array([[[[0.6033198757288365, 0.94599383179617269, 0.6033198757288365, 0.74289184873547831], [0.79825297278664165, 0.6033198757288365, 0.6033198757288365, 0.7723478579687284], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.83060779510514626]], [[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.76319682823895252, 0.97067330199334001], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.88567082924827845]]], [[[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]], [[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.88695387596924524], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]]], [[[0.6033198757288365, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365], [0.6033198757288365, 0.67326301723647863, 0.6033198757288365, 0.98748707230701482], [0.74667864479466339, 0.6033198757288365, 0.6033198757288365, 0.6033198757288365]], [[0.6033198757288365, 0.6033198757288365, 0.70462443973450162, 0.6033198757288365], [0.6033198757288365, 0.6033198757288365, 0.79639900215468074, 0.6033198757288365], [0.6033198757288365, 0.80672822476952732, 0.6033198757288365, 0.6033198757288365]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.05190996520507829, 0.017161612088801625, 0.71858473475896756, 0.37348553899634163], [-0.2900675205518739, 0.60854666978599803, 0.70397814254347901, 0.34729973841370732], [0.75952611874240916, -0.85817450277367535, -0.7317856056362293, -0.31052335974622758]], [[0.79398744563549517, 0.36998417255058302, -0.22929438181474016, 0.72966719351334763], [-0.23421902838943631, 0.52858124363614434, -0.34289297309725453, 0.4839966640935327], [0.89912158559629063, 0.19283434949277334, 0.6880794645896311, -0.50054360169032353]]], [[[-0.72502571574959829, 0.12262090113765312, -0.0020542239853444944, 0.91548118836692938], [0.78806292974921677, -0.81050708718107689, -0.4555499714123219, 0.20236859401508678], [-0.86916961516845492, -0.46273000223993566, -0.87145807505307871, -0.48884808814567582]], [[-0.28116992348313641, 0.94502678380907934, -0.34178232083148097, 0.494848074318752], [-0.49963099873167938, -0.38212305995515927, -0.90334447714176291, -0.027869624718332098], [-0.58406375150638778, 0.049953828057895722, 0.06288075173468588, 0.011374517234280956]]], [[[0.6313092846086179, -0.32934442397022878, -0.12082067268034402, -0.81517538889628494], [0.18194989025213792, 0.55938774982922723, -0.47438800760526822, -0.49610384574944866], [-0.19817862014855248, 0.49727611559501272, 0.65656919046343964, -0.90170978231220733]], [[-0.01430383255025669, -0.42392546084720828, -0.63408974934462714, 0.2297719414510393], [0.14518598570413488, -0.7593693018435812, 0.24665523827371505, 0.4227370985616663], [0.84731050064349067, 0.29891687014594681, 0.74229800688242542, -0.61146599162534176]]]])+(1.-msk_arg1)*numpy.array([[[[-0.12183245485334804, -0.72533881666320732, -0.095314628080340524, -0.96204587984036749], [0.81161246893894945, -0.41231517619150382, 0.81643004907183769, 0.64054743409630088], [0.28623669454789491, -0.11921417764493447, 0.77875292789103367, 0.83227612622879255]], [[0.94598610956808082, 0.31619246495167364, -0.50035233986323946, -0.60942345828654809], [0.8550308542962981, 0.60309493959946914, -0.62253263594962727, 0.099524057753801021], [0.065643405866520288, -0.51140342686979157, 0.2778463538429532, 0.099989530376729752]]], [[[-0.15734782977223682, 0.69843204704279982, 0.79456354791214867, 0.70614031560047175], [0.75846189691326282, -0.75135735449830965, 0.34566943333871936, 0.92376655153080911], [-0.67437646803837725, 0.31275498490118347, 0.99946206793939596, 0.44245904480217879]], [[-0.46126594617388528, -0.81725777487357232, 0.7882110144947998, -0.32610455746185063], [-0.2660914567173136, 0.38181156282466966, -0.48839127021344608, -0.1193962417619181], [0.56725512112085696, 0.83756894062445353, -0.57986804950752746, 0.41187912397027371]]], [[[0.23564342400826011, 0.72328665933702796, -0.84957020571644293, 0.47666051314019398], [-0.27886381132232452, 0.027580773383707458, 0.11851498214281264, 0.29752771163794023], [-0.33671436751098494, -0.032465399786591131, -0.92877668000088254, -0.31021658992823364]], [[-0.27500029514744329, 0.48538238289279523, -0.69495807540982768, 0.61839139559063661], [0.66670303573354928, 0.93605953311897627, -0.4262911110981098, 0.34502064198694971], [-0.88906028071393473, 0.23035782924474724, 0.63598579428315882, -0.84795750149192473]]]])
      res=maximum(arg0,arg1)
      s0=numpy.array([[[[0.98325538221137898, -0.20186999833610031, 0.40857874358842761, -0.03668517494280743], [-0.74360096247307994, -0.90594758392189489, -0.93565726102554647, 0.14749657692620421], [0.83778447877286277, 0.0042404911944187162, -0.54558374198169668, 0.83393227278515081]], [[0.3861439064968375, 0.42073119766792422, -0.21121143187006264, 0.39579278884208868], [-0.073789476122656295, -0.94492877297639843, -0.088846534577811775, 0.28122228780170566], [0.83174417401553158, -0.30836633085770759, 0.93270177353542039, 0.17203339734052414]]], [[[0.0071722411065338054, -0.26670824412266647, -0.47151597155696856, 0.72438390893130444], [0.89834597644333214, 0.28086117009698075, 0.78702854436676284, -0.62291508588763289], [0.57885916508173674, -0.50880888737982843, 0.64618096057576913, -0.16496247850627377]], [[-0.66219205264588155, 0.71701284786986852, 0.11936010951822706, -0.37121227762466624], [0.61653883009191102, -0.21040229078419426, -0.17049828468535955, -0.77589529186195949], [0.24100195554851123, 0.79708451209204512, 0.9256059198368134, -0.71549478232361974]]], [[[0.058386909625657957, -0.63650298283992002, -0.86262180597088589, -0.67986038990692488], [0.61899027075381685, -0.34691779056299077, -0.94930033549704684, 0.20508395808553459], [-0.80551975631356143, 0.89331509581207391, 0.18099251426471263, -0.01524658111058308]], [[-0.79847150819719115, 0.09047607358432197, -0.63737569345251699, 0.43473428112791646], [0.78761887236842276, -0.20886021620156359, -0.27515812064151079, -0.80259839567017477], [-0.85468712569102934, 0.49900811497575015, -0.51677610308893063, -0.44832492704662719]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.98325538221137898, 0.017161612088801625, 0.71858473475896756, 0.37348553899634163], [-0.2900675205518739, 0.60854666978599803, 0.70397814254347901, 0.34729973841370732], [0.83778447877286277, 0.0042404911944187162, -0.54558374198169668, 0.83393227278515081]], [[0.79398744563549517, 0.42073119766792422, -0.21121143187006264, 0.72966719351334763], [-0.073789476122656295, 0.52858124363614434, -0.088846534577811775, 0.4839966640935327], [0.89912158559629063, 0.19283434949277334, 0.93270177353542039, 0.17203339734052414]]], [[[0.0071722411065338054, 0.12262090113765312, -0.0020542239853444944, 0.91548118836692938], [0.89834597644333214, 0.28086117009698075, 0.78702854436676284, 0.20236859401508678], [0.57885916508173674, -0.46273000223993566, 0.64618096057576913, -0.16496247850627377]], [[-0.28116992348313641, 0.94502678380907934, 0.11936010951822706, 0.494848074318752], [0.61653883009191102, -0.21040229078419426, -0.17049828468535955, -0.027869624718332098], [0.24100195554851123, 0.79708451209204512, 0.9256059198368134, 0.011374517234280956]]], [[[0.6313092846086179, -0.32934442397022878, -0.12082067268034402, -0.67986038990692488], [0.61899027075381685, 0.55938774982922723, -0.47438800760526822, 0.20508395808553459], [-0.19817862014855248, 0.89331509581207391, 0.65656919046343964, -0.01524658111058308]], [[-0.01430383255025669, 0.09047607358432197, -0.63408974934462714, 0.43473428112791646], [0.78761887236842276, -0.20886021620156359, 0.24665523827371505, 0.4227370985616663], [0.84731050064349067, 0.49900811497575015, 0.74229800688242542, -0.44832492704662719]]]])+(1.-msk_ref)*numpy.array([[[[0.98325538221137898, -0.20186999833610031, 0.40857874358842761, -0.03668517494280743], [0.81161246893894945, -0.41231517619150382, 0.81643004907183769, 0.64054743409630088], [0.83778447877286277, 0.0042404911944187162, 0.77875292789103367, 0.83393227278515081]], [[0.94598610956808082, 0.42073119766792422, -0.21121143187006264, 0.39579278884208868], [0.8550308542962981, 0.60309493959946914, -0.088846534577811775, 0.28122228780170566], [0.83174417401553158, -0.30836633085770759, 0.93270177353542039, 0.17203339734052414]]], [[[0.0071722411065338054, 0.69843204704279982, 0.79456354791214867, 0.72438390893130444], [0.89834597644333214, 0.28086117009698075, 0.78702854436676284, 0.92376655153080911], [0.57885916508173674, 0.31275498490118347, 0.99946206793939596, 0.44245904480217879]], [[-0.46126594617388528, 0.71701284786986852, 0.7882110144947998, -0.32610455746185063], [0.61653883009191102, 0.38181156282466966, -0.17049828468535955, -0.1193962417619181], [0.56725512112085696, 0.83756894062445353, 0.9256059198368134, 0.41187912397027371]]], [[[0.23564342400826011, 0.72328665933702796, -0.84957020571644293, 0.47666051314019398], [0.61899027075381685, 0.027580773383707458, 0.11851498214281264, 0.29752771163794023], [-0.33671436751098494, 0.89331509581207391, 0.18099251426471263, -0.01524658111058308]], [[-0.27500029514744329, 0.48538238289279523, -0.63737569345251699, 0.61839139559063661], [0.78761887236842276, 0.93605953311897627, -0.27515812064151079, 0.34502064198694971], [-0.85468712569102934, 0.49900811497575015, 0.63598579428315882, -0.44832492704662719]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_float_rank0(self):
      arg0=Data(-0.103724211678,self.functionspace)
      arg1=0.328283764361
      res=maximum(arg0,arg1)
      ref=Data(0.328283764361,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([-0.994662543839125, -0.74842601409030385]),self.functionspace)
      arg1=0.83441404241
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.83441404241015316, 0.83441404241015316]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[-0.93123334581207584, -0.6375770434441641, 0.14522839584946268, -0.73025618312998342, -0.43938340547287202], [-0.94178326076177643, 0.37494459873539854, -0.41021077539181738, -0.20418492678279287, 0.66837201836494731], [-0.8482840255895383, -0.37013598496106792, 0.9491945264624071, -0.84517384298961939, -0.98444834611902565], [-0.62650653009151203, -0.80707547020964321, -0.54928564740444141, 0.62760912325779228, -0.068170353902772929]]),self.functionspace)
      arg1=0.416476250509
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.41647625050863057, 0.41647625050863057, 0.41647625050863057, 0.41647625050863057, 0.41647625050863057], [0.41647625050863057, 0.41647625050863057, 0.41647625050863057, 0.41647625050863057, 0.66837201836494731], [0.41647625050863057, 0.41647625050863057, 0.9491945264624071, 0.41647625050863057, 0.41647625050863057], [0.41647625050863057, 0.41647625050863057, 0.41647625050863057, 0.62760912325779228, 0.41647625050863057]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[-0.30986069585890852, -0.71233438667691962], [-0.71108485594464965, 0.46803401869143135]], [[0.047347209617666586, 0.40377859059535814], [-0.076869071518825383, 0.66840655201248778]], [[-0.73984483745703389, 0.15706212028006727], [-0.75038361207139048, 0.71534375261818672]], [[-0.16981297613502533, 0.95326334909603228], [-0.578228141354864, 0.69445686433215403]], [[0.85239986273571344, 0.70437733963137261], [-0.64356702754466366, 0.98497620890820303]], [[-0.73348032264680829, -0.90903872026049592], [-0.30876237687286801, 0.85358440828017468]]]),self.functionspace)
      arg1=-0.707243668032
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[-0.30986069585890852, -0.70724366803240968], [-0.70724366803240968, 0.46803401869143135]], [[0.047347209617666586, 0.40377859059535814], [-0.076869071518825383, 0.66840655201248778]], [[-0.70724366803240968, 0.15706212028006727], [-0.70724366803240968, 0.71534375261818672]], [[-0.16981297613502533, 0.95326334909603228], [-0.578228141354864, 0.69445686433215403]], [[0.85239986273571344, 0.70437733963137261], [-0.64356702754466366, 0.98497620890820303]], [[-0.70724366803240968, -0.70724366803240968], [-0.30876237687286801, 0.85358440828017468]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[-0.44038422283869894, -0.99307493506995592, -0.64116938861589978, -0.52885072305198189], [-0.60664143188537656, -0.53692428608899201, 0.9427489146021919, -0.66511035190756762], [-0.95640611039066736, 0.98227091107399023, -0.35080572865502968, -0.062671325929018584]], [[-0.74010294573905111, -0.60760114920218666, 0.45863579037285751, -0.45688569472448681], [-0.33875049551517833, 0.35019781316828325, 0.45173979734058101, 0.8762043904815493], [-0.34188391703990084, 0.19998649935419754, -0.63879709781309413, 0.18545222105657011]]], [[[0.011754072910120206, -0.90550908100131511, -0.93322868838339446, -0.11741827984316866], [0.77558036495515026, -0.85219854425588326, -0.061841571000625484, -0.066269008250732009], [0.96157185984128013, -0.95342288705795242, 0.45172913312384599, 0.018267106882194906]], [[0.1591168098755007, -0.27340675847513096, -0.14469177879585371, 0.694740264377673], [-0.80538537088324014, 0.20415359879847728, -0.25347422230372807, -0.93986986572358044], [0.82523363323342358, 0.95118360314535688, -0.084226421580063437, 0.23148605050984217]]], [[[0.19213259368793234, 0.79589318105724693, -0.54294778796310839, -0.20471748468873829], [0.18053098336615636, -0.84012326762801948, -0.62575984457545952, -0.14368177341561639], [0.62597131428981112, 0.38509874618804685, 0.47371812469111485, -0.86705010549302908]], [[-0.36502653007689423, -0.79613146258086531, 0.15329391158627304, 0.68148041836662387], [0.73324139002880684, 0.38529515075962784, -0.9317611817520175, -0.6261241009923757], [0.80092298112981131, 0.34865093765311261, 0.89442927947537765, 0.92170715451548602]]]]),self.functionspace)
      arg1=0.635778225458
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.63577822545793317, 0.63577822545793317, 0.9427489146021919, 0.63577822545793317], [0.63577822545793317, 0.98227091107399023, 0.63577822545793317, 0.63577822545793317]], [[0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.8762043904815493], [0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317]]], [[[0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.77558036495515026, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.96157185984128013, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317]], [[0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.694740264377673], [0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.82523363323342358, 0.95118360314535688, 0.63577822545793317, 0.63577822545793317]]], [[[0.63577822545793317, 0.79589318105724693, 0.63577822545793317, 0.63577822545793317], [0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317]], [[0.63577822545793317, 0.63577822545793317, 0.63577822545793317, 0.68148041836662387], [0.73324139002880684, 0.63577822545793317, 0.63577822545793317, 0.63577822545793317], [0.80092298112981131, 0.63577822545793317, 0.89442927947537765, 0.92170715451548602]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_array_rank0(self):
      arg0=Data(0.131131173778,self.functionspace)
      arg1=numpy.array(-0.576805781524)
      res=maximum(arg0,arg1)
      ref=Data(0.131131173778,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([0.14808740913781926, 0.35567967234737252]),self.functionspace)
      arg1=numpy.array(-0.935584836565)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.14808740913781926, 0.35567967234737252]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[0.11219561462947825, -0.18427729976238316, -0.26893116897649416, 0.40378445902375137, -0.64010126735322159], [-0.54548479863007193, -0.6771466146796592, -0.65941588238170357, -0.080554716485880862, -0.51142929986801766], [0.1802163583855485, 0.4654264085214439, -0.88158680167000036, -0.67093289247607557, 0.47213709052956609], [-0.15939930729103469, -0.25730020832439515, -0.97874531075582616, 0.10703962128016098, 0.32041153354414065]]),self.functionspace)
      arg1=numpy.array(0.896240426157)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456], [0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456], [0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456], [0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456, 0.89624042615692456]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[0.11547252472671099, 0.27715619160036087], [-0.0011872311418676951, -0.38096891774653074]], [[0.37877267210850607, -0.8255429528666729], [-0.63956098319827714, -0.70194003551721074]], [[0.44309695940748495, -0.88721518381663578], [-0.53272604278013991, 0.79625941492663554]], [[-0.1703558707341577, -0.95114537821544309], [-0.32281622748150207, 0.31750595998728737]], [[0.30447422840521154, -0.82721331108465446], [0.65676818386047175, 0.40338692981661328]], [[0.39404755664916813, -0.78343567350875953], [-0.30036954834309015, -0.83068548144181831]]]),self.functionspace)
      arg1=numpy.array(0.597874377242)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.59787437724227765, 0.59787437724227765], [0.59787437724227765, 0.59787437724227765]], [[0.59787437724227765, 0.59787437724227765], [0.59787437724227765, 0.59787437724227765]], [[0.59787437724227765, 0.59787437724227765], [0.59787437724227765, 0.79625941492663554]], [[0.59787437724227765, 0.59787437724227765], [0.59787437724227765, 0.59787437724227765]], [[0.59787437724227765, 0.59787437724227765], [0.65676818386047175, 0.59787437724227765]], [[0.59787437724227765, 0.59787437724227765], [0.59787437724227765, 0.59787437724227765]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[-0.12333178650474741, -0.41601255505495294, 0.78421542610699757, -0.89389135345287363], [-0.89386090823390885, 0.56081437970269388, -0.39255299041023939, 0.32411039481285564], [0.8551210553300661, -0.90707027924543415, 0.16036883016043224, -0.10713182442360014]], [[0.62413380378897632, -0.33807365498251962, -0.14322028563106537, -0.28390387123110661], [0.10848574295659086, -0.3237100322279467, 0.40626470900918554, -0.86322529775615497], [-0.53241603186354824, 0.96450022170256, 0.7410412312711554, -0.90628249882883471]]], [[[0.56961012500639319, 0.46258567990715616, 0.2691574282574225, -0.70084123183330149], [0.32178078403959587, -0.16491236638222273, -0.67544384124087165, 0.49044876181851271], [0.87426425567196486, 0.99113280536628512, -0.33592835344070004, -0.48167757670424649]], [[0.37582959179360564, 0.079659313583228863, 0.13142551769766975, -0.50982437173132999], [-0.80549764829567927, -0.99459313338638555, -0.24095207228821169, -0.91245577244012499], [0.36688639267496503, 0.20893052084134811, -0.95250384603251925, -0.46421785538004756]]], [[[-0.69408638083069896, -0.60590039199066448, 0.65199154172599783, 0.5600386130626831], [-0.45945308229643844, -0.33990278016795927, -0.12156326670116702, 0.76710181803628164], [-0.30308185765731799, -0.47607829009357494, -0.66299346892125444, 0.85153168985207617]], [[0.97196327159277796, -0.76370007931813078, 0.50139581647673981, 0.63066011940195899], [-0.98705858372661903, 0.30251550651882297, -0.12436255937028684, 0.47348321672077809], [-0.11280446521491339, 0.3669717233616796, -0.58417675366941624, -0.8354338127149179]]]]),self.functionspace)
      arg1=numpy.array(-0.987163870919)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.12333178650474741, -0.41601255505495294, 0.78421542610699757, -0.89389135345287363], [-0.89386090823390885, 0.56081437970269388, -0.39255299041023939, 0.32411039481285564], [0.8551210553300661, -0.90707027924543415, 0.16036883016043224, -0.10713182442360014]], [[0.62413380378897632, -0.33807365498251962, -0.14322028563106537, -0.28390387123110661], [0.10848574295659086, -0.3237100322279467, 0.40626470900918554, -0.86322529775615497], [-0.53241603186354824, 0.96450022170256, 0.7410412312711554, -0.90628249882883471]]], [[[0.56961012500639319, 0.46258567990715616, 0.2691574282574225, -0.70084123183330149], [0.32178078403959587, -0.16491236638222273, -0.67544384124087165, 0.49044876181851271], [0.87426425567196486, 0.99113280536628512, -0.33592835344070004, -0.48167757670424649]], [[0.37582959179360564, 0.079659313583228863, 0.13142551769766975, -0.50982437173132999], [-0.80549764829567927, -0.98716387091897451, -0.24095207228821169, -0.91245577244012499], [0.36688639267496503, 0.20893052084134811, -0.95250384603251925, -0.46421785538004756]]], [[[-0.69408638083069896, -0.60590039199066448, 0.65199154172599783, 0.5600386130626831], [-0.45945308229643844, -0.33990278016795927, -0.12156326670116702, 0.76710181803628164], [-0.30308185765731799, -0.47607829009357494, -0.66299346892125444, 0.85153168985207617]], [[0.97196327159277796, -0.76370007931813078, 0.50139581647673981, 0.63066011940195899], [-0.98705858372661903, 0.30251550651882297, -0.12436255937028684, 0.47348321672077809], [-0.11280446521491339, 0.3669717233616796, -0.58417675366941624, -0.8354338127149179]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_Symbol_rank0(self):
      arg0=Data(0.664561779083,self.functionspace)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.829285050023)
      sub=res.substitute({arg1:s1})
      ref=Data(0.829285050023,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-0.69545419737927117, 0.99305327855079217]),self.functionspace)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.153020727326)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([0.15302072732624339, 0.99305327855079217]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[-0.59374607886600006, -0.50931590206448885, 0.87158812528115992, -0.083248463579309817, 0.64675557959416596], [-0.99510993900114997, -0.86534619373808885, -0.76055600732404205, -0.32388592463644716, 0.48179608280482089], [-0.0189717949163144, 0.53343311709904495, -0.01695952592096317, -0.14275237095244719, -0.56878877627022906], [-0.66684442729332871, 0.90441290480400194, -0.95338552917045272, -0.19153791835347533, 0.26360673254102074]]),self.functionspace)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.32156239679)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.3215623967904031, 0.3215623967904031, 0.87158812528115992, 0.3215623967904031, 0.64675557959416596], [0.3215623967904031, 0.3215623967904031, 0.3215623967904031, 0.3215623967904031, 0.48179608280482089], [0.3215623967904031, 0.53343311709904495, 0.3215623967904031, 0.3215623967904031, 0.3215623967904031], [0.3215623967904031, 0.90441290480400194, 0.3215623967904031, 0.3215623967904031, 0.3215623967904031]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[-0.63408955961464142, -0.76118818542823696], [-0.0029784442754923202, 0.54432467407787422]], [[-0.64191739678302429, 0.067130805893673484], [-0.19665248598291529, 0.1328475859290037]], [[-0.56971462838064468, 0.51091249756472101], [-0.47226268148108841, 0.63794524003575748]], [[0.55245668688973337, -0.63810367804480395], [0.74612529134962369, -0.17481247619555695]], [[0.025673774490274193, 0.10960736160784612], [-0.76951823716646683, 0.13670394008645115]], [[0.67172421750357691, -0.40880035103301604], [0.79132642447949686, 0.72197872430428278]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.186648816909)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.18664881690900659, -0.18664881690900659], [-0.0029784442754923202, 0.54432467407787422]], [[-0.18664881690900659, 0.067130805893673484], [-0.18664881690900659, 0.1328475859290037]], [[-0.18664881690900659, 0.51091249756472101], [-0.18664881690900659, 0.63794524003575748]], [[0.55245668688973337, -0.18664881690900659], [0.74612529134962369, -0.17481247619555695]], [[0.025673774490274193, 0.10960736160784612], [-0.18664881690900659, 0.13670394008645115]], [[0.67172421750357691, -0.18664881690900659], [0.79132642447949686, 0.72197872430428278]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[0.11691606484743811, -0.18910207642141397, -0.97467407734233369, 0.54785995546527189], [0.94659765351248715, -0.50895768482469506, 0.75673961085640173, 0.81000722351354781], [0.12962517455334921, -0.078976357655154539, -0.63070660955667046, 0.38159785488013642]], [[-0.02403526270024936, -0.1723643289467176, -0.52475964505263439, -0.90347327300706604], [-0.64581611506488201, 0.49661549530917881, -0.94000053808660566, -0.35771609596682397], [-0.78621989863997332, 0.020741795385072592, -0.44360981567644231, 0.22226902471522392]]], [[[-0.038463812991184509, -0.4099796075013018, -0.43560481702467024, -0.81522019261561196], [-0.16899899767684734, -0.83049119482277756, -0.92056631424361623, -0.37390489621232259], [0.072983530166874866, 0.1831674166669639, -0.85469662452253403, -0.29142005064818877]], [[-0.9144023718274068, 0.6401005232965018, -0.86961613870225341, -0.61116440564734287], [0.60983911653939904, -0.58546568971594759, -0.074285422547206714, 0.021363661195166195], [0.00850205556137551, 0.58012942485722929, -0.13639426106487806, -0.13725955595150907]]], [[[0.48995477809263677, -0.85159004979717401, 0.57236937634099427, 0.96218992321882535], [-0.41176815588149163, 0.51666573976104435, -0.56806312709864182, -0.22610154639166402], [-0.9520160415423875, 0.884347925828179, 0.13363791828118421, 0.11169377706342409]], [[0.72525326379313837, -0.65913525904766379, 0.18068049451919643, 0.85369735995883933], [-0.64521176966868765, -0.23404065953506392, 0.026544508868687089, 0.083906169344088344], [-0.020672344041770652, -0.9841740747373513, -0.99849728600899912, -0.28628548061458803]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.400014368083)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.11691606484743811, -0.18910207642141397, -0.40001436808308521, 0.54785995546527189], [0.94659765351248715, -0.40001436808308521, 0.75673961085640173, 0.81000722351354781], [0.12962517455334921, -0.078976357655154539, -0.40001436808308521, 0.38159785488013642]], [[-0.02403526270024936, -0.1723643289467176, -0.40001436808308521, -0.40001436808308521], [-0.40001436808308521, 0.49661549530917881, -0.40001436808308521, -0.35771609596682397], [-0.40001436808308521, 0.020741795385072592, -0.40001436808308521, 0.22226902471522392]]], [[[-0.038463812991184509, -0.40001436808308521, -0.40001436808308521, -0.40001436808308521], [-0.16899899767684734, -0.40001436808308521, -0.40001436808308521, -0.37390489621232259], [0.072983530166874866, 0.1831674166669639, -0.40001436808308521, -0.29142005064818877]], [[-0.40001436808308521, 0.6401005232965018, -0.40001436808308521, -0.40001436808308521], [0.60983911653939904, -0.40001436808308521, -0.074285422547206714, 0.021363661195166195], [0.00850205556137551, 0.58012942485722929, -0.13639426106487806, -0.13725955595150907]]], [[[0.48995477809263677, -0.40001436808308521, 0.57236937634099427, 0.96218992321882535], [-0.40001436808308521, 0.51666573976104435, -0.40001436808308521, -0.22610154639166402], [-0.40001436808308521, 0.884347925828179, 0.13363791828118421, 0.11169377706342409]], [[0.72525326379313837, -0.40001436808308521, 0.18068049451919643, 0.85369735995883933], [-0.40001436808308521, -0.23404065953506392, 0.026544508868687089, 0.083906169344088344], [-0.020672344041770652, -0.40001436808308521, -0.40001436808308521, -0.28628548061458803]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_constData_rank0(self):
      arg0=Data(0.585764561052,self.functionspace)
      arg1=Data(0.562102768003,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(0.585764561052,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([0.36817249063478297, 0.84922379654348235]),self.functionspace)
      arg1=Data(-0.000326831113467,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.36817249063478297, 0.84922379654348235]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[0.62061167912589932, -0.38731372384862972, -0.095757334081436607, -0.71510399904747191, -0.7960986251434321], [-0.62309855289829441, -0.87302083840022671, 0.19339618209313958, -0.46839248025963864, -0.50779431412333742], [-0.21582349308100612, 0.38788705235820564, 0.013156532615211658, 0.48974126868597168, -0.8747876083639794], [-0.03351051938462235, -0.01330223402014763, 0.41055032605572195, -0.83673851035555735, 0.75259533240721854]]),self.functionspace)
      arg1=Data(0.834120422885,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349], [0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349], [0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349], [0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349, 0.8341204228848349]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-0.34639348942196935, 0.54454548312732642], [0.54767137350158723, -0.1261592719343172]], [[0.50901738204046398, -0.84585881665715412], [-0.13374339923115808, -0.2770246163740675]], [[-0.54418624743208754, 0.12594979383328964], [0.36093139391063178, 0.82920727027033436]], [[0.27913464865114546, 0.80222365962388098], [0.20763542419244296, 0.90359922967678674]], [[0.84504710858705456, 0.65071402913489296], [-0.38961429927310487, -0.71287474316420352]], [[0.21051157156997924, 0.12062151771734198], [0.15420074047028498, -0.77490659965239894]]]),self.functionspace)
      arg1=Data(-0.719008102783,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[-0.34639348942196935, 0.54454548312732642], [0.54767137350158723, -0.1261592719343172]], [[0.50901738204046398, -0.71900810278299088], [-0.13374339923115808, -0.2770246163740675]], [[-0.54418624743208754, 0.12594979383328964], [0.36093139391063178, 0.82920727027033436]], [[0.27913464865114546, 0.80222365962388098], [0.20763542419244296, 0.90359922967678674]], [[0.84504710858705456, 0.65071402913489296], [-0.38961429927310487, -0.71287474316420352]], [[0.21051157156997924, 0.12062151771734198], [0.15420074047028498, -0.71900810278299088]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[0.24615935787720122, -0.78756263814918848, 0.14312411275206283, -0.61209069712580444], [0.66751834472618388, -0.97270217915095181, 0.44734491429921719, 0.56094524052401296], [-0.52616981543309271, -0.59090561139819875, 0.72854643636544969, 0.28084180850612128]], [[-0.13831139862668707, -0.41869220979295729, 0.45665100088168176, -0.17425379986376388], [-0.71060410977750221, -0.23490189062085109, 0.55707427536980814, -0.47235867785724728], [-0.7825415636663855, -0.50535912385584014, 0.18080270846310542, 0.81347475179814932]]], [[[0.11676945660942573, 0.0090598274192139616, -0.45702967566484753, 0.10306553976340282], [0.99323093060600787, 0.29434293524837996, 0.54768497876439315, 0.64082304875839924], [0.11154113644780606, -0.92134481397307355, 0.47282662127503539, 0.88748076184699709]], [[0.22357060873765233, -0.64706904541581722, 0.19015911386848727, -0.17771475973870299], [0.26760950005587758, 0.53275086600982924, -0.67459888863079587, -0.47669911211071847], [0.13841053267986525, -0.55447231310576828, -0.60380591334225908, 0.45443682452695322]]], [[[0.90802807850631129, 0.5777144475067848, 0.29643536414101779, -0.70268788049900732], [-0.40417344419730084, 0.60820239131834386, 0.98877684418314415, 0.02773308630434923], [-0.99593792390185087, -0.99197717949359321, -0.57269874716477709, 0.23365613209137837]], [[0.74741771939310353, -0.57587639566704985, 0.74979270157039379, 0.88416116732103767], [0.3503774673176725, 0.4043599318959008, 0.92900873297982289, -0.060730954440895157], [-0.82062848810850908, 0.36166456925688828, -0.98165811447801321, 0.10663254098157515]]]]),self.functionspace)
      arg1=Data(-0.218709768625,self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.24615935787720122, -0.21870976862466218, 0.14312411275206283, -0.21870976862466218], [0.66751834472618388, -0.21870976862466218, 0.44734491429921719, 0.56094524052401296], [-0.21870976862466218, -0.21870976862466218, 0.72854643636544969, 0.28084180850612128]], [[-0.13831139862668707, -0.21870976862466218, 0.45665100088168176, -0.17425379986376388], [-0.21870976862466218, -0.21870976862466218, 0.55707427536980814, -0.21870976862466218], [-0.21870976862466218, -0.21870976862466218, 0.18080270846310542, 0.81347475179814932]]], [[[0.11676945660942573, 0.0090598274192139616, -0.21870976862466218, 0.10306553976340282], [0.99323093060600787, 0.29434293524837996, 0.54768497876439315, 0.64082304875839924], [0.11154113644780606, -0.21870976862466218, 0.47282662127503539, 0.88748076184699709]], [[0.22357060873765233, -0.21870976862466218, 0.19015911386848727, -0.17771475973870299], [0.26760950005587758, 0.53275086600982924, -0.21870976862466218, -0.21870976862466218], [0.13841053267986525, -0.21870976862466218, -0.21870976862466218, 0.45443682452695322]]], [[[0.90802807850631129, 0.5777144475067848, 0.29643536414101779, -0.21870976862466218], [-0.21870976862466218, 0.60820239131834386, 0.98877684418314415, 0.02773308630434923], [-0.21870976862466218, -0.21870976862466218, -0.21870976862466218, 0.23365613209137837]], [[0.74741771939310353, -0.21870976862466218, 0.74979270157039379, 0.88416116732103767], [0.3503774673176725, 0.4043599318959008, 0.92900873297982289, -0.060730954440895157], [-0.21870976862466218, 0.36166456925688828, -0.21870976862466218, 0.10663254098157515]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_expandedData_rank0(self):
      arg0=Data(-0.0924750298109,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.719276974378)+(1.-msk_arg1)*(-0.901780055402)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.719276974378)+(1.-msk_ref)*(-0.0924750298109)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([0.19003810057983306, 0.86747179360557602]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.979758050952)+(1.-msk_arg1)*(0.498784837635)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.19003810057983306, 0.86747179360557602])+(1.-msk_ref)*numpy.array([0.49878483763491466, 0.86747179360557602])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[-0.21116738853853634, 0.39376908967419455, -0.90448828983244067, 0.39775680151484716, -0.62546967898667782], [0.71684884432650064, -0.2950221732323155, 0.061549355711304932, 0.92168965316615159, 0.54752726203495428], [-0.12485263858702367, 0.16626287261463624, -0.79329781788494325, -0.22207097268419607, 0.19795438035557633], [0.76599397431182026, 0.56414018544113698, 0.95805026779533731, -0.69615823883217742, 0.21821142022539242]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.243791427144)+(1.-msk_arg1)*(-0.440291613911)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.21116738853853634, 0.39376908967419455, -0.24379142714392676, 0.39775680151484716, -0.24379142714392676], [0.71684884432650064, -0.24379142714392676, 0.061549355711304932, 0.92168965316615159, 0.54752726203495428], [-0.12485263858702367, 0.16626287261463624, -0.24379142714392676, -0.22207097268419607, 0.19795438035557633], [0.76599397431182026, 0.56414018544113698, 0.95805026779533731, -0.24379142714392676, 0.21821142022539242]])+(1.-msk_ref)*numpy.array([[-0.21116738853853634, 0.39376908967419455, -0.44029161391063454, 0.39775680151484716, -0.44029161391063454], [0.71684884432650064, -0.2950221732323155, 0.061549355711304932, 0.92168965316615159, 0.54752726203495428], [-0.12485263858702367, 0.16626287261463624, -0.44029161391063454, -0.22207097268419607, 0.19795438035557633], [0.76599397431182026, 0.56414018544113698, 0.95805026779533731, -0.44029161391063454, 0.21821142022539242]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[0.71382052722853828, 0.59639246293382753], [0.79091055029858603, 0.6098069915009563]], [[-0.81877756445176075, -0.12572711561586747], [0.6629503717900096, 0.088668188470614639]], [[-0.25907968164793926, 0.078844106245125056], [0.53171494813158082, -0.48562454486265927]], [[-0.16221141560120533, -0.22108938742601714], [-0.19849691519790702, 0.1366342465866397]], [[0.97032753659399651, 0.10811990970766572], [-0.53083812183678813, 0.48681350528164535]], [[0.94501075888711217, 0.15734516046100699], [0.67336660412428007, 0.54871928770743117]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.154574403253)+(1.-msk_arg1)*(0.444360735115)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.71382052722853828, 0.59639246293382753], [0.79091055029858603, 0.6098069915009563]], [[0.15457440325278737, 0.15457440325278737], [0.6629503717900096, 0.15457440325278737]], [[0.15457440325278737, 0.15457440325278737], [0.53171494813158082, 0.15457440325278737]], [[0.15457440325278737, 0.15457440325278737], [0.15457440325278737, 0.15457440325278737]], [[0.97032753659399651, 0.15457440325278737], [0.15457440325278737, 0.48681350528164535]], [[0.94501075888711217, 0.15734516046100699], [0.67336660412428007, 0.54871928770743117]]])+(1.-msk_ref)*numpy.array([[[0.71382052722853828, 0.59639246293382753], [0.79091055029858603, 0.6098069915009563]], [[0.4443607351150709, 0.4443607351150709], [0.6629503717900096, 0.4443607351150709]], [[0.4443607351150709, 0.4443607351150709], [0.53171494813158082, 0.4443607351150709]], [[0.4443607351150709, 0.4443607351150709], [0.4443607351150709, 0.4443607351150709]], [[0.97032753659399651, 0.4443607351150709], [0.4443607351150709, 0.48681350528164535]], [[0.94501075888711217, 0.4443607351150709], [0.67336660412428007, 0.54871928770743117]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[0.17987454394191782, -0.47366581235947391, 0.57343836836209627, -0.38076754396458123], [0.57156205056895448, 0.22221196833792156, -0.63457115192030189, -0.90743873572672884], [0.50190781994193601, -0.72382204778829151, -0.16048375541143778, 0.24666469799623414]], [[0.1797380572435916, 0.16543463798169644, -0.38942568372144026, -0.60094923326812366], [0.58369106606630172, -0.72979782894371525, -0.71685913494409648, -0.43487862431086666], [0.94570907221553924, -0.71041646506714295, -0.57201057001577316, -0.27780439634546106]]], [[[0.21885917868948757, 0.691429511541092, -0.86074155940358898, -0.74572948424544871], [-0.67104106531707486, 0.65026368776100796, -0.77242606184409479, -0.17309140373321386], [-0.71424293657762106, 0.0160043137660697, -0.86608565479945288, 0.21568143007726359]], [[0.17793601295093553, 0.8241368231005648, -0.56689484436349358, -0.64591475368907791], [-0.83488920100836284, 0.89807531811166474, 0.12481763857472217, 0.19555968134032309], [0.90280696977547814, -0.3114337513979637, -0.4016548633642858, 0.84998160623596397]]], [[[-0.41113212039641711, 0.69589842214715314, 0.40122617777186043, 0.66162309921520612], [-0.64259349002452848, -0.61545179085033141, 0.35720959723129897, -0.74561290819100612], [0.25049896732519583, -0.84957718142891347, -0.69000881268819692, 0.55532528673635673]], [[0.65030548208217898, 0.61753581199201957, 0.55386412523479889, -0.59819873675646185], [0.59881331123370507, -0.73382421864055347, -0.25155846699294737, -0.83893004279709227], [-0.68286287600890794, -0.029611557116543841, 0.092485711989719332, 0.54588443381266383]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.647690225694)+(1.-msk_arg1)*(-0.116678653334)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.17987454394191782, -0.47366581235947391, 0.57343836836209627, -0.38076754396458123], [0.57156205056895448, 0.22221196833792156, -0.63457115192030189, -0.64769022569387036], [0.50190781994193601, -0.64769022569387036, -0.16048375541143778, 0.24666469799623414]], [[0.1797380572435916, 0.16543463798169644, -0.38942568372144026, -0.60094923326812366], [0.58369106606630172, -0.64769022569387036, -0.64769022569387036, -0.43487862431086666], [0.94570907221553924, -0.64769022569387036, -0.57201057001577316, -0.27780439634546106]]], [[[0.21885917868948757, 0.691429511541092, -0.64769022569387036, -0.64769022569387036], [-0.64769022569387036, 0.65026368776100796, -0.64769022569387036, -0.17309140373321386], [-0.64769022569387036, 0.0160043137660697, -0.64769022569387036, 0.21568143007726359]], [[0.17793601295093553, 0.8241368231005648, -0.56689484436349358, -0.64591475368907791], [-0.64769022569387036, 0.89807531811166474, 0.12481763857472217, 0.19555968134032309], [0.90280696977547814, -0.3114337513979637, -0.4016548633642858, 0.84998160623596397]]], [[[-0.41113212039641711, 0.69589842214715314, 0.40122617777186043, 0.66162309921520612], [-0.64259349002452848, -0.61545179085033141, 0.35720959723129897, -0.64769022569387036], [0.25049896732519583, -0.64769022569387036, -0.64769022569387036, 0.55532528673635673]], [[0.65030548208217898, 0.61753581199201957, 0.55386412523479889, -0.59819873675646185], [0.59881331123370507, -0.64769022569387036, -0.25155846699294737, -0.64769022569387036], [-0.64769022569387036, -0.029611557116543841, 0.092485711989719332, 0.54588443381266383]]]])+(1.-msk_ref)*numpy.array([[[[0.17987454394191782, -0.11667865333375405, 0.57343836836209627, -0.11667865333375405], [0.57156205056895448, 0.22221196833792156, -0.11667865333375405, -0.11667865333375405], [0.50190781994193601, -0.11667865333375405, -0.11667865333375405, 0.24666469799623414]], [[0.1797380572435916, 0.16543463798169644, -0.11667865333375405, -0.11667865333375405], [0.58369106606630172, -0.11667865333375405, -0.11667865333375405, -0.11667865333375405], [0.94570907221553924, -0.11667865333375405, -0.11667865333375405, -0.11667865333375405]]], [[[0.21885917868948757, 0.691429511541092, -0.11667865333375405, -0.11667865333375405], [-0.11667865333375405, 0.65026368776100796, -0.11667865333375405, -0.11667865333375405], [-0.11667865333375405, 0.0160043137660697, -0.11667865333375405, 0.21568143007726359]], [[0.17793601295093553, 0.8241368231005648, -0.11667865333375405, -0.11667865333375405], [-0.11667865333375405, 0.89807531811166474, 0.12481763857472217, 0.19555968134032309], [0.90280696977547814, -0.11667865333375405, -0.11667865333375405, 0.84998160623596397]]], [[[-0.11667865333375405, 0.69589842214715314, 0.40122617777186043, 0.66162309921520612], [-0.11667865333375405, -0.11667865333375405, 0.35720959723129897, -0.11667865333375405], [0.25049896732519583, -0.11667865333375405, -0.11667865333375405, 0.55532528673635673]], [[0.65030548208217898, 0.61753581199201957, 0.55386412523479889, -0.11667865333375405], [0.59881331123370507, -0.11667865333375405, -0.11667865333375405, -0.11667865333375405], [-0.11667865333375405, -0.029611557116543841, 0.092485711989719332, 0.54588443381266383]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_array_rank1(self):
      arg0=Data(-0.977109354346,self.functionspace)
      arg1=numpy.array([0.70276044813264082, -0.31317697172843362])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.70276044813264082, -0.31317697172843362]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([-0.62917253954533403, -0.91478760916921553]),self.functionspace)
      arg1=numpy.array([0.33852418316707711, -0.28298537896949161])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.33852418316707711, -0.28298537896949161]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_Symbol_rank1(self):
      arg0=Data(-0.897659014127,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([-0.82851210466879732, 0.41079203962342792])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.82851210466879732, 0.41079203962342792]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([-0.83340176844825109, -0.63521789570698273]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([0.60550672698957397, 0.38447082907767527])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([0.60550672698957397, 0.38447082907767527]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_constData_rank1(self):
      arg0=Data(-0.159726189832,self.functionspace)
      arg1=Data(numpy.array([0.1189612003237861, -0.63087634684158944]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([0.1189612003237861, -0.15972618983177433]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([-0.740289439487964, 0.50792620718681647]),self.functionspace)
      arg1=Data(numpy.array([-0.98869116031302773, -0.030691151152568041]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([-0.740289439487964, 0.50792620718681647]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_expandedData_rank1(self):
      arg0=Data(0.0787324456408,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.48016870047192817, -0.99662347188520961])+(1.-msk_arg1)*numpy.array([0.58774285667709059, 0.67262794612436538])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.078732445640770532, 0.078732445640770532])+(1.-msk_ref)*numpy.array([0.58774285667709059, 0.67262794612436538])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([0.17322811204814759, 0.25536817066436801]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.91972158516709168, -0.98465374807727835])+(1.-msk_arg1)*numpy.array([-0.19155853001762169, -0.43931509419045467])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.91972158516709168, 0.25536817066436801])+(1.-msk_ref)*numpy.array([0.17322811204814759, 0.25536817066436801])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_array_rank2(self):
      arg0=Data(-0.590060805237,self.functionspace)
      arg1=numpy.array([[-0.46148245190505666, 0.72542064767267411, -0.49628838214418636, 0.36379029454198486, -0.35637937725143498], [0.77716266154028713, -0.0294929702278659, 0.77367789791539621, 0.26018292128939491, -0.0013933338625782277], [-0.71102579607495486, 0.058714555627548126, 0.62191425731141292, -0.9329697856968644, 0.48541403689676299], [-0.78897231411198532, -0.72015354884882865, -0.62803607199062306, -0.96795550516592099, 0.86073050375759519]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[-0.46148245190505666, 0.72542064767267411, -0.49628838214418636, 0.36379029454198486, -0.35637937725143498], [0.77716266154028713, -0.0294929702278659, 0.77367789791539621, 0.26018292128939491, -0.0013933338625782277], [-0.59006080523717053, 0.058714555627548126, 0.62191425731141292, -0.59006080523717053, 0.48541403689676299], [-0.59006080523717053, -0.59006080523717053, -0.59006080523717053, -0.59006080523717053, 0.86073050375759519]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[-0.73760404253705514, 0.50224248181499931, 0.11399712803883166, -0.13254212815072308, 0.74172163669326041], [-0.87041848231795704, 0.84343778201486463, -0.75554993046891106, 0.9855020616978345, -0.54320886383165745], [0.015954104044625605, -0.72612141326347879, -0.34799202711515242, -0.59679759609203797, -0.2575746318844927], [-0.2477036236422252, -0.021811926970866979, 0.81844582541975552, -0.14903149292622531, 0.80053174022124529]]),self.functionspace)
      arg1=numpy.array([[-0.86954510950167463, 0.1017003127120133, -0.16951929662470433, 0.70137020593450439, 0.77429037367257458], [0.39247727833147628, -0.71486192220367606, -0.59117959653327024, 0.50948617730426826, -0.88092334234319924], [-0.95167080948153315, 0.021566922221860629, -0.37806539314914223, 0.3191199630251873, 0.21890366409298223], [0.8635367262140623, 0.52741696809771943, 0.40373578351947836, 0.40740580857358721, 0.50991869479490881]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[-0.73760404253705514, 0.50224248181499931, 0.11399712803883166, 0.70137020593450439, 0.77429037367257458], [0.39247727833147628, 0.84343778201486463, -0.59117959653327024, 0.9855020616978345, -0.54320886383165745], [0.015954104044625605, 0.021566922221860629, -0.34799202711515242, 0.3191199630251873, 0.21890366409298223], [0.8635367262140623, 0.52741696809771943, 0.81844582541975552, 0.40740580857358721, 0.80053174022124529]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_Symbol_rank2(self):
      arg0=Data(-0.637272359465,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.93357341361564083, 0.25698632067186034, -0.029720171711117738, -0.30875351618642743, -0.23637628888181861], [-0.2891471305478277, -0.22562638340862384, 0.18787873756952633, -0.15958639681320719, 0.065075144754314529], [0.95073268925174137, 0.71641948960869284, -0.27639016478885536, -0.24411084821371687, 0.69924204387635402], [0.16301313582709676, 0.74483426984036205, 0.62398172084669978, -0.62033142288846022, -0.28915587653777108]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.63727235946512772, 0.25698632067186034, -0.029720171711117738, -0.30875351618642743, -0.23637628888181861], [-0.2891471305478277, -0.22562638340862384, 0.18787873756952633, -0.15958639681320719, 0.065075144754314529], [0.95073268925174137, 0.71641948960869284, -0.27639016478885536, -0.24411084821371687, 0.69924204387635402], [0.16301313582709676, 0.74483426984036205, 0.62398172084669978, -0.62033142288846022, -0.28915587653777108]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-0.43026101754020685, 0.29811974564127852, 0.79886283172870032, 0.34469882516148398, 0.18440527003717522], [0.76150582417217327, 0.77351247227932407, -0.79713675943526252, 0.79860629299555597, 0.39057038072263506], [0.96977333965043799, -0.33335789993330733, -0.20605599405457475, -0.56915336090245372, 0.89461598602330206], [-0.67604219616426353, 0.88512102393172354, -0.56058686696277871, 0.54575207158615591, -0.32772855161181247]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.26119428722177185, 0.37209855196265185, 0.20415898126446064, -0.6225751456475801, 0.80103987500103524], [0.44532467101974893, -0.78274319336812548, -0.48351063332866051, 0.26100507852735033, -0.67050332562669701], [-0.12156255078406208, -0.27744404551457436, -0.71194253302697597, 0.57325927361819717, -0.54571219157485196], [-0.1389333385381808, -0.18545090923672225, -0.28684434823484395, 0.46339245987327771, -0.66451068494091325]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.26119428722177185, 0.37209855196265185, 0.79886283172870032, 0.34469882516148398, 0.80103987500103524], [0.76150582417217327, 0.77351247227932407, -0.48351063332866051, 0.79860629299555597, 0.39057038072263506], [0.96977333965043799, -0.27744404551457436, -0.20605599405457475, 0.57325927361819717, 0.89461598602330206], [-0.1389333385381808, 0.88512102393172354, -0.28684434823484395, 0.54575207158615591, -0.32772855161181247]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_constData_rank2(self):
      arg0=Data(0.471113579692,self.functionspace)
      arg1=Data(numpy.array([[0.20278512451424202, 0.46932554124551484, 0.51627101397671771, -0.15806811678484656, 0.90125732162226746], [-0.64525760065047266, -0.99646917035423299, -0.0079700168467822241, -0.1148252168651378, -0.60869273468514651], [-0.81920269598799123, 0.18492463625508537, 0.67703260540557975, 0.38678293776089578, -0.58337450118932721], [-0.85103881319765606, -0.18597068284684348, 0.55532783770806615, -0.3391026116648197, 0.59931507372202009]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.47111357969230805, 0.47111357969230805, 0.51627101397671771, 0.47111357969230805, 0.90125732162226746], [0.47111357969230805, 0.47111357969230805, 0.47111357969230805, 0.47111357969230805, 0.47111357969230805], [0.47111357969230805, 0.47111357969230805, 0.67703260540557975, 0.47111357969230805, 0.47111357969230805], [0.47111357969230805, 0.47111357969230805, 0.55532783770806615, 0.47111357969230805, 0.59931507372202009]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-0.17406452878186696, -0.34759214930565996, -0.13708153516362609, 0.93110590449668984, -0.47649961659515139], [-0.35702537789588118, -0.4057682901706241, 0.56892562984164674, 0.20451757373433055, -0.47729926624352892], [-0.55493234840963379, 0.020855045362278224, 0.95156666561071002, 0.14975165550539549, -0.17074783729098764], [-0.2364495010997254, -0.32021782391836329, -0.45843843696539488, -0.47665234638234, -0.41919606212785676]]),self.functionspace)
      arg1=Data(numpy.array([[0.64408124666207822, -0.67629542279010635, -0.4931164628479503, -0.96447891179315248, -0.45618690487636582], [0.18569576708094893, -0.085193091641874163, 0.08793086582533971, 0.64570776148383136, -0.5516531478806932], [0.87675126968759676, -0.39508497185268077, -0.97669171907295138, -0.68655710586099006, -0.028857220969970232], [-0.3670380390017618, 0.76258006170069592, -0.31454790037894464, 0.6627659629911522, 0.82734991520477563]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[0.64408124666207822, -0.34759214930565996, -0.13708153516362609, 0.93110590449668984, -0.45618690487636582], [0.18569576708094893, -0.085193091641874163, 0.56892562984164674, 0.64570776148383136, -0.47729926624352892], [0.87675126968759676, 0.020855045362278224, 0.95156666561071002, 0.14975165550539549, -0.028857220969970232], [-0.2364495010997254, 0.76258006170069592, -0.31454790037894464, 0.6627659629911522, 0.82734991520477563]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_expandedData_rank2(self):
      arg0=Data(-0.0165871765665,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.73128249610105489, 0.8419999761644652, 0.32706544558055151, 0.98621609382436559, 0.11814390137662678], [0.19794947116873596, 0.31254355907321152, -0.87988550364485918, -0.11555651603083028, -0.71600368849223273], [-0.1893391600405403, 0.90957761312463936, -0.52243082625432802, -0.61123719298866308, 0.33907244894623134], [0.69675393273476094, 0.80610078097181703, -0.025854445517779379, 0.94898341046945389, 0.46113499488993592]])+(1.-msk_arg1)*numpy.array([[0.23505303855840864, 0.99481465040860417, -0.086492983784499877, -0.65370229345838227, 0.10003757203241559], [-0.55539504622389524, -0.97706587559470259, 0.76222581775594289, 0.77064743585593676, 0.37822774452875341], [0.63726139395335868, 0.82448429030660098, -0.98743591219817395, -0.11783271519582783, 0.82622085914926746], [0.79063087780255503, -0.51677954263556924, -0.11079135634696624, 0.55532802164644224, 0.97336607909484085]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.016587176566458695, 0.8419999761644652, 0.32706544558055151, 0.98621609382436559, 0.11814390137662678], [0.19794947116873596, 0.31254355907321152, -0.016587176566458695, -0.016587176566458695, -0.016587176566458695], [-0.016587176566458695, 0.90957761312463936, -0.016587176566458695, -0.016587176566458695, 0.33907244894623134], [0.69675393273476094, 0.80610078097181703, -0.016587176566458695, 0.94898341046945389, 0.46113499488993592]])+(1.-msk_ref)*numpy.array([[0.23505303855840864, 0.99481465040860417, -0.016587176566458695, -0.016587176566458695, 0.10003757203241559], [-0.016587176566458695, -0.016587176566458695, 0.76222581775594289, 0.77064743585593676, 0.37822774452875341], [0.63726139395335868, 0.82448429030660098, -0.016587176566458695, -0.016587176566458695, 0.82622085914926746], [0.79063087780255503, -0.016587176566458695, -0.016587176566458695, 0.55532802164644224, 0.97336607909484085]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-0.58944550721294209, 0.11789616185996188, 0.72555020927912528, 0.95067825171691411, 0.44929807483372386], [-0.90050975945004685, -0.14027235979416108, -0.50271571837565965, 0.71764496858912841, 0.13936247694569537], [-0.55363895921566852, -0.29692903194729126, -0.12968512147399403, 0.16630251248490757, -0.15779583568993605], [-0.91804262372165168, -0.0058087569025269126, 0.57431208620955165, -0.71561067889580943, -0.22286757830799497]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.99165981962421723, 0.056234396987206559, -0.96929836288649107, 0.85741514845807765, 0.31290702472003784], [0.66879661516796074, 0.74410544323631544, 0.61758123318413549, 0.0023178475410641219, 0.035637036084378515], [0.80734776741943692, -0.93425673113467789, -0.3545773036750044, 0.17713706129968121, -0.78562855023306311], [0.95764289578713813, 0.82592763875748232, 0.56453388909804225, 0.33669515016119256, 0.68270953107335797]])+(1.-msk_arg1)*numpy.array([[0.46121428483489479, 0.77301745076266992, -0.75239767323970663, -0.84007824075301141, 0.54854476434581101], [-0.18306960903719038, 0.22306205318244277, 0.95307177391042264, -0.83728247335587391, -0.64823533348017937], [0.90595429846142239, -0.55842375993133087, 0.69860419850019628, -0.58794473533051539, -0.90321846168159303], [0.19112532041271257, -0.16066829098287294, 0.62710659839874872, -0.58348891958458182, -0.43880626183960802]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.58944550721294209, 0.11789616185996188, 0.72555020927912528, 0.95067825171691411, 0.44929807483372386], [0.66879661516796074, 0.74410544323631544, 0.61758123318413549, 0.71764496858912841, 0.13936247694569537], [0.80734776741943692, -0.29692903194729126, -0.12968512147399403, 0.17713706129968121, -0.15779583568993605], [0.95764289578713813, 0.82592763875748232, 0.57431208620955165, 0.33669515016119256, 0.68270953107335797]])+(1.-msk_ref)*numpy.array([[0.46121428483489479, 0.77301745076266992, 0.72555020927912528, 0.95067825171691411, 0.54854476434581101], [-0.18306960903719038, 0.22306205318244277, 0.95307177391042264, 0.71764496858912841, 0.13936247694569537], [0.90595429846142239, -0.29692903194729126, 0.69860419850019628, 0.16630251248490757, -0.15779583568993605], [0.19112532041271257, -0.0058087569025269126, 0.62710659839874872, -0.58348891958458182, -0.22286757830799497]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_array_rank3(self):
      arg0=Data(0.516978268542,self.functionspace)
      arg1=numpy.array([[[0.1203754261590837, -0.30872063802497229], [-0.54367292398629163, -0.28894544783644749]], [[0.98579636264447745, -0.1617667296901939], [-0.71399942465432775, 0.53397315922387301]], [[-0.30699998747323254, 0.077194107501915799], [0.79875258103307534, 0.80801781194196387]], [[0.15460773887563417, 0.90896321404086278], [0.20719915824628354, 0.51262923866513121]], [[-0.42605851894450963, 0.11121377720978609], [0.83365318051571125, -0.090277192445190391]], [[0.11506747491337666, 0.6855450860006127], [-0.55520344405285305, -0.29122646515827966]]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.51697826854212692, 0.51697826854212692], [0.51697826854212692, 0.51697826854212692]], [[0.98579636264447745, 0.51697826854212692], [0.51697826854212692, 0.53397315922387301]], [[0.51697826854212692, 0.51697826854212692], [0.79875258103307534, 0.80801781194196387]], [[0.51697826854212692, 0.90896321404086278], [0.51697826854212692, 0.51697826854212692]], [[0.51697826854212692, 0.51697826854212692], [0.83365318051571125, 0.51697826854212692]], [[0.51697826854212692, 0.6855450860006127], [0.51697826854212692, 0.51697826854212692]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[-0.019064560845273526, -0.66082512842015606], [0.7775245397942645, -0.97863728880489664]], [[0.14796990911746843, -0.14940640351421752], [-0.86829509897198331, 0.37449746403230755]], [[0.37150654719541598, -0.28378634170515826], [-0.50000172789557618, 0.12870803836646894]], [[0.011474562798716059, 0.62211999362561543], [0.20936671603779877, 0.1596384101994337]], [[0.48413175379125017, -0.70455876802613626], [0.95196006961287805, 0.20450770288369013]], [[-0.69630908911115341, -0.40070777001767066], [0.39061204191308385, -0.7494107073434304]]]),self.functionspace)
      arg1=numpy.array([[[-0.78977223364983429, -0.39796958371743218], [-0.72434391726505232, -0.52658036061654578]], [[0.30677953915987843, -0.27325740398129361], [-0.44704945638883409, 0.35675189799215068]], [[-0.85496603242765823, -0.49015497061116231], [0.38100286965101793, 0.71258135816204082]], [[-0.69502829292158985, -0.66075782445418518], [0.62742938866730125, -0.0042722353662665302]], [[0.47028792653016804, -0.43167155331296869], [0.46101712703457975, -0.26196273464811326]], [[-0.79525334450742657, 0.58902924204184304], [0.071218600732356085, -0.44930940165988953]]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[-0.019064560845273526, -0.39796958371743218], [0.7775245397942645, -0.52658036061654578]], [[0.30677953915987843, -0.14940640351421752], [-0.44704945638883409, 0.37449746403230755]], [[0.37150654719541598, -0.28378634170515826], [0.38100286965101793, 0.71258135816204082]], [[0.011474562798716059, 0.62211999362561543], [0.62742938866730125, 0.1596384101994337]], [[0.48413175379125017, -0.43167155331296869], [0.95196006961287805, 0.20450770288369013]], [[-0.69630908911115341, 0.58902924204184304], [0.39061204191308385, -0.44930940165988953]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_Symbol_rank3(self):
      arg0=Data(-0.301616866911,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[-0.075914420708843577, 0.66358144119505469], [-0.58087779325539213, -0.29018537386125676]], [[0.23348641898057276, -0.64142675232592739], [0.35482470582420178, -0.67699541518823403]], [[0.36917299179456542, -0.85774610863182055], [0.17824922067142834, 0.47555537122661984]], [[-0.97719261595724527, 0.54833659450404926], [-0.67194747717936876, 0.21998235181108461]], [[0.25195270131078606, 0.070395336399227793], [-0.42779349762042918, 0.3141743449583867]], [[0.20807098525982304, -0.99926821174044345], [-0.33669983271995707, 0.62342319037875926]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.075914420708843577, 0.66358144119505469], [-0.30161686691078327, -0.29018537386125676]], [[0.23348641898057276, -0.30161686691078327], [0.35482470582420178, -0.30161686691078327]], [[0.36917299179456542, -0.30161686691078327], [0.17824922067142834, 0.47555537122661984]], [[-0.30161686691078327, 0.54833659450404926], [-0.30161686691078327, 0.21998235181108461]], [[0.25195270131078606, 0.070395336399227793], [-0.30161686691078327, 0.3141743449583867]], [[0.20807098525982304, -0.30161686691078327], [-0.30161686691078327, 0.62342319037875926]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[0.25053145075842798, -0.3794274575500276], [-0.2203081336344006, -0.90636103624233622]], [[-0.89360594132235449, -0.044279048286900435], [0.50865438163476706, 0.30050515771416508]], [[0.17151373057705444, 0.3537454996156566], [-0.69271913159458465, -0.64664308190255659]], [[-0.14646556033688141, -0.056658533589058591], [-0.92991614862004801, 0.41532986353549406]], [[-0.63084142928204034, -0.88583877496366781], [-0.93048186777341169, -0.67144597532207717]], [[0.87416304298844172, -0.82914945635714843], [0.36425589944096037, -0.44121106899902984]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[0.34360762394944211, -0.750307485491571], [0.082385622892501509, 0.78367774116726041]], [[-0.36821289797486978, 0.94549455065763843], [-0.99726764906452803, -0.43481851107125213]], [[0.39446194425583214, -0.9533167476847999], [-0.62274186628347961, -0.20554677331882809]], [[-0.41826475650418771, -0.92753710539749479], [-0.52057150946775099, 0.60511201119510249]], [[-0.68353108574981913, -0.91712769524574145], [-0.014686165499424808, 0.35604294723483321]], [[0.58881786310683792, 0.14736266870876857], [0.72063758010693379, 0.80581576520467024]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.34360762394944211, -0.3794274575500276], [0.082385622892501509, 0.78367774116726041]], [[-0.36821289797486978, 0.94549455065763843], [0.50865438163476706, 0.30050515771416508]], [[0.39446194425583214, 0.3537454996156566], [-0.62274186628347961, -0.20554677331882809]], [[-0.14646556033688141, -0.056658533589058591], [-0.52057150946775099, 0.60511201119510249]], [[-0.63084142928204034, -0.88583877496366781], [-0.014686165499424808, 0.35604294723483321]], [[0.87416304298844172, 0.14736266870876857], [0.72063758010693379, 0.80581576520467024]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_constData_rank3(self):
      arg0=Data(-0.520111358301,self.functionspace)
      arg1=Data(numpy.array([[[-0.53755593069331176, 0.37344778982644522], [0.90325038776102318, -0.89380296608704612]], [[0.59616188236635415, 0.051917694202589626], [-0.04448028945702287, 0.41100328136339304]], [[0.33171935562963939, -0.24899622688538758], [0.62960041822469925, 0.33958207874297885]], [[0.77867517548268683, 0.42072327881397564], [-0.74938980192739946, 0.81416985659114172]], [[0.3116774006710874, -0.33738332950437711], [0.78527047728394561, -0.74961547941494744]], [[-0.20009722385279827, -0.6571543996337208], [0.17776822203863141, 0.44451143669045345]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[-0.52011135830068911, 0.37344778982644522], [0.90325038776102318, -0.52011135830068911]], [[0.59616188236635415, 0.051917694202589626], [-0.04448028945702287, 0.41100328136339304]], [[0.33171935562963939, -0.24899622688538758], [0.62960041822469925, 0.33958207874297885]], [[0.77867517548268683, 0.42072327881397564], [-0.52011135830068911, 0.81416985659114172]], [[0.3116774006710874, -0.33738332950437711], [0.78527047728394561, -0.52011135830068911]], [[-0.20009722385279827, -0.52011135830068911], [0.17776822203863141, 0.44451143669045345]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[0.22644209225706202, -0.82408753542802948], [-0.3229055658381601, 0.2590293226229694]], [[-0.26905946802022762, 0.2903861382640851], [0.12277829585192546, -0.77850587791871839]], [[-0.89889146053925439, 0.38226490597490859], [0.90580586744844571, 0.55103010631847082]], [[-0.039650312436662727, -0.88676021840684105], [-0.51601766497181489, -0.25197490301551628]], [[0.61158062010002601, 0.29776204958735475], [0.71009302472890323, -0.66290156645868592]], [[0.35864937589582269, 0.39150421091712273], [0.62113215392768795, -0.60699036535846251]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.19115277784536788, -0.12179826773639668], [0.93712678010940254, 0.88203929581384566]], [[-0.88435454431013327, 0.9789660191397036], [0.59264976401288116, -0.95487963435415701]], [[-0.63201606726111259, -0.55289610271892187], [-0.28138775032557017, 0.10882077764575948]], [[-0.49427255052853813, -0.052753620845127358], [0.041666359004336684, 0.79096925989660916]], [[-0.41422835506377953, 0.44371821729329164], [0.29584910796715769, 0.32801776168641106]], [[0.48887697683026987, 0.060894815731677765], [0.03438210218260096, 0.012054779031575036]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[0.22644209225706202, -0.12179826773639668], [0.93712678010940254, 0.88203929581384566]], [[-0.26905946802022762, 0.9789660191397036], [0.59264976401288116, -0.77850587791871839]], [[-0.63201606726111259, 0.38226490597490859], [0.90580586744844571, 0.55103010631847082]], [[-0.039650312436662727, -0.052753620845127358], [0.041666359004336684, 0.79096925989660916]], [[0.61158062010002601, 0.44371821729329164], [0.71009302472890323, 0.32801776168641106]], [[0.48887697683026987, 0.39150421091712273], [0.62113215392768795, 0.012054779031575036]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_expandedData_rank3(self):
      arg0=Data(-0.417160613179,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.59344857070579238, 0.57386185515692323], [0.22052027505720551, -0.02253631100635034]], [[-0.79319250300435051, -0.80838786337089852], [0.72156951045421569, 0.79754573800662776]], [[0.44658460292787949, -0.6036139053961691], [-0.0074518374538019572, 0.10259587076510246]], [[-0.67150694427957869, -0.24769554924870607], [0.34642765365536365, 0.40804680401868687]], [[-0.48546098050896314, -0.37273512567714118], [-0.93149112098916875, 0.56189739537971306]], [[0.38968682790960374, 0.6967969004356549], [0.31719546261205389, 0.66755203749869896]]])+(1.-msk_arg1)*numpy.array([[[-0.32798712079017855, 0.91458585203417941], [-0.1858316961301949, 0.10471165389886661]], [[0.25739129223381751, -0.20580522632299081], [-0.77836146736583722, 0.71646905903017921]], [[-0.12213192671753959, 0.2681111999511725], [-0.88598687196668746, -0.10299574975782755]], [[-0.47951968494409059, 0.38762496133793989], [-0.20961640133930182, -0.46499685779521527]], [[-0.63002780928919999, 0.99970479651183686], [-0.28892538074111696, -0.86014943997554583]], [[-0.87521791528084747, -0.2290352438643557], [0.61458437038138758, -0.054295041010556266]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.41716061317925801, 0.57386185515692323], [0.22052027505720551, -0.02253631100635034]], [[-0.41716061317925801, -0.41716061317925801], [0.72156951045421569, 0.79754573800662776]], [[0.44658460292787949, -0.41716061317925801], [-0.0074518374538019572, 0.10259587076510246]], [[-0.41716061317925801, -0.24769554924870607], [0.34642765365536365, 0.40804680401868687]], [[-0.41716061317925801, -0.37273512567714118], [-0.41716061317925801, 0.56189739537971306]], [[0.38968682790960374, 0.6967969004356549], [0.31719546261205389, 0.66755203749869896]]])+(1.-msk_ref)*numpy.array([[[-0.32798712079017855, 0.91458585203417941], [-0.1858316961301949, 0.10471165389886661]], [[0.25739129223381751, -0.20580522632299081], [-0.41716061317925801, 0.71646905903017921]], [[-0.12213192671753959, 0.2681111999511725], [-0.41716061317925801, -0.10299574975782755]], [[-0.41716061317925801, 0.38762496133793989], [-0.20961640133930182, -0.41716061317925801]], [[-0.41716061317925801, 0.99970479651183686], [-0.28892538074111696, -0.41716061317925801]], [[-0.41716061317925801, -0.2290352438643557], [0.61458437038138758, -0.054295041010556266]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[-0.83383467576998993, -0.83006279778742731], [0.61582380340209775, -0.4064088511087891]], [[-0.4159938969754351, -0.073787477564773329], [-0.021547740656737036, 0.5133460168591264]], [[0.91656606912919014, 0.53167536065528687], [-0.61521902289133901, 0.4664937027114231]], [[0.97834019232877312, -0.93183928685658612], [-0.89297362621765086, 0.98615396264285726]], [[0.34755706158837762, 0.58475703376154442], [-0.15101139759741611, 0.40395116766294437]], [[0.62168225352120166, 0.50496809316394398], [0.72198090495722789, 0.85904449954970996]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.19214098611675801, 0.07906975802244931], [0.70387607748340963, 0.73265073793599189]], [[-0.6142258277704653, 0.6255979194324861], [0.46214070326839796, -0.28059147251841399]], [[-0.57130450989061998, 0.5709484739777515], [0.48615540360013054, -0.60718602833024926]], [[0.48182675680934151, -0.66948554719719966], [0.8085115047175595, -0.95171986323783053]], [[0.5870376913405424, 0.59265808147737098], [0.49569538437672511, -0.32465377675760454]], [[0.41879820042914684, 0.82573065216442831], [0.14668313470902383, 0.78600006019282054]]])+(1.-msk_arg1)*numpy.array([[[0.63268720723055161, 0.78130831328720141], [-0.18520848313978489, -0.35202194720092805]], [[0.58179955935294925, 0.1226309312208731], [0.29154247836357272, -0.9060564605671757]], [[0.71741017447493505, 0.96466082998871294], [-0.81353604274588265, -0.098340216808310954]], [[-0.88020377622640522, -0.84721483807877607], [0.6939301069991286, -0.30953811263549613]], [[0.99593228905602715, 0.32599642612508051], [-0.40377432683868597, 0.0021172786571095337]], [[0.69285022412981334, 0.70887735927613194], [0.17846982139165757, 0.33981526774046644]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.19214098611675801, 0.07906975802244931], [0.70387607748340963, 0.73265073793599189]], [[-0.4159938969754351, 0.6255979194324861], [0.46214070326839796, 0.5133460168591264]], [[0.91656606912919014, 0.5709484739777515], [0.48615540360013054, 0.4664937027114231]], [[0.97834019232877312, -0.66948554719719966], [0.8085115047175595, 0.98615396264285726]], [[0.5870376913405424, 0.59265808147737098], [0.49569538437672511, 0.40395116766294437]], [[0.62168225352120166, 0.82573065216442831], [0.72198090495722789, 0.85904449954970996]]])+(1.-msk_ref)*numpy.array([[[0.63268720723055161, 0.78130831328720141], [0.61582380340209775, -0.35202194720092805]], [[0.58179955935294925, 0.1226309312208731], [0.29154247836357272, 0.5133460168591264]], [[0.91656606912919014, 0.96466082998871294], [-0.61521902289133901, 0.4664937027114231]], [[0.97834019232877312, -0.84721483807877607], [0.6939301069991286, 0.98615396264285726]], [[0.99593228905602715, 0.58475703376154442], [-0.15101139759741611, 0.40395116766294437]], [[0.69285022412981334, 0.70887735927613194], [0.72198090495722789, 0.85904449954970996]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_array_rank4(self):
      arg0=Data(-0.142135287883,self.functionspace)
      arg1=numpy.array([[[[-0.3214758956274788, -0.095246696795476593, 0.88428285109896976, 0.16221904765171113], [0.2694361196789572, -0.88687321541037512, 0.02982831555263199, 0.36044697686186544], [0.089838925108686407, 0.1168644540354129, -0.41060659412622824, 0.51404357016969393]], [[-0.7228284299326766, -0.74820755507542192, -0.53859530054671856, -0.10221620366876616], [0.59211782258097068, -0.9868578737653857, 0.51866928562956849, -0.47068533837586557], [0.54008589927595518, 0.30880228975280732, 0.011323333079816589, -0.23519250089433008]]], [[[-0.88835060064517424, 0.97805478155238479, 0.044980281282704704, 0.54172810913588032], [0.50306412490394248, -0.9089797473431025, -0.65257701723518702, -0.81985356457976932], [-0.55532857771618849, 0.90847532179586454, 0.73479685498652625, -0.95360986864558894]], [[-0.070414261238669784, -0.55257468908581853, -0.8087953736998108, -0.88978135385971924], [0.30925435765744647, 0.011985479496297913, -0.94329973754759822, -0.78626227567625295], [-0.80368619190217405, -0.0018149953637613958, -0.64327784734883697, -0.70628255506304627]]], [[[0.21889092381937481, 0.95982372912792258, -0.51545817953418194, -0.79191496743321532], [0.24699184622858739, -0.97716194806902634, -0.74023402440265662, -0.68483112762279252], [-0.80837443342334714, 0.39602906627714529, -0.93722656251851855, -0.36568401963574959]], [[-0.062719970600047636, -0.75215102861789918, 0.80346816982468328, -0.03123166762679852], [0.87032572315546886, -0.26645188225830663, 0.16577077009238161, -0.42846478915247221], [-0.83350849887379552, -0.09057758663591553, -0.72415237677358313, -0.11083229941883377]]]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.14213528788334617, -0.095246696795476593, 0.88428285109896976, 0.16221904765171113], [0.2694361196789572, -0.14213528788334617, 0.02982831555263199, 0.36044697686186544], [0.089838925108686407, 0.1168644540354129, -0.14213528788334617, 0.51404357016969393]], [[-0.14213528788334617, -0.14213528788334617, -0.14213528788334617, -0.10221620366876616], [0.59211782258097068, -0.14213528788334617, 0.51866928562956849, -0.14213528788334617], [0.54008589927595518, 0.30880228975280732, 0.011323333079816589, -0.14213528788334617]]], [[[-0.14213528788334617, 0.97805478155238479, 0.044980281282704704, 0.54172810913588032], [0.50306412490394248, -0.14213528788334617, -0.14213528788334617, -0.14213528788334617], [-0.14213528788334617, 0.90847532179586454, 0.73479685498652625, -0.14213528788334617]], [[-0.070414261238669784, -0.14213528788334617, -0.14213528788334617, -0.14213528788334617], [0.30925435765744647, 0.011985479496297913, -0.14213528788334617, -0.14213528788334617], [-0.14213528788334617, -0.0018149953637613958, -0.14213528788334617, -0.14213528788334617]]], [[[0.21889092381937481, 0.95982372912792258, -0.14213528788334617, -0.14213528788334617], [0.24699184622858739, -0.14213528788334617, -0.14213528788334617, -0.14213528788334617], [-0.14213528788334617, 0.39602906627714529, -0.14213528788334617, -0.14213528788334617]], [[-0.062719970600047636, -0.14213528788334617, 0.80346816982468328, -0.03123166762679852], [0.87032572315546886, -0.14213528788334617, 0.16577077009238161, -0.14213528788334617], [-0.14213528788334617, -0.09057758663591553, -0.14213528788334617, -0.11083229941883377]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[0.46361503964213124, 0.75212103924315676, 0.1517014593635575, 0.63148557170685327], [-0.9875146867756015, 0.42928007237587473, 0.85118517914010172, -0.15509137168645615], [0.15514320286516092, 0.0005342941540116275, -0.89094877398399008, 0.908865463056445]], [[0.45054003146551436, -0.23123539736983156, -0.076540616512914772, 0.10502324913621375], [-0.0071973112790566507, -0.81620273555925849, -0.79541946133102126, 0.26480060384338566], [-0.45249656775626268, 0.010265849576416963, -0.9086687652789831, -0.71275837340643822]]], [[[0.67609301986855086, 0.37746787067053145, -0.42950164086299925, -0.034365339088483715], [0.3373674099598567, 0.18896844133041868, 0.93458147614245779, -0.40631016965933098], [-0.19339720965663387, 0.66853942853358883, 0.56086919503548471, 0.49829638104085539]], [[-0.67730793306659365, -0.93402109411008638, -0.53909789907854089, -0.11494460733501222], [0.47127259923995135, 0.40243458962316581, 0.65232695613202951, 0.87100864021002966], [-0.62331578035775403, -0.78832076452515865, 0.98453152930704513, -0.32270285556445955]]], [[[0.030889419318346212, -0.74436621353272381, -0.41393005003246119, -0.27459836901005485], [0.2727885358054869, 0.39114640764062014, -0.62734280749087201, -0.68278841829167125], [0.45687670732427632, -0.41786887648402415, 0.14358785951974951, 0.20168949679051473]], [[-0.61804283685818384, -0.74947293782095592, 0.47882556396076903, -0.38116235218691275], [-0.16868935807808083, -0.31479531035901021, -0.629767051514182, 0.40666969603452707], [0.47141425542113868, 0.42602189213064201, -0.42510981826215244, 0.35951635388005387]]]]),self.functionspace)
      arg1=numpy.array([[[[-0.075878674569879179, -0.39617760413616954, -0.37441304528755759, 0.059028750333119939], [-0.45199615428664996, -0.67982487852801032, -0.43878514017544568, -0.2306154287295592], [-0.028641567153798952, 0.20315619172000665, 0.25175605070668317, -0.62452580082432019]], [[-0.5510023141107756, 0.47087051768615695, -0.20141109488796705, -0.75776728595767517], [0.13680386381556775, -0.43979686709904531, -0.97978303421397572, 0.84162201515236612], [-0.26616111363933515, -0.32677302519872753, -0.74540501454095254, -0.31553251505895985]]], [[[0.0019165847734914365, -0.34429378460591642, -0.6920165333737931, 0.54536804846096953], [-0.21558916120957838, 0.60934895052910298, -0.72555820711554286, 0.81566556247952771], [0.28566419298828816, -0.94399443344238465, -0.11385943971564827, 0.0098538097357632193]], [[0.23053049901245104, -0.63018177830499922, -0.58754298214345324, 0.32371305451881383], [0.34351012374192247, 0.50137166714297887, 0.95947548444362751, 0.42833198450872101], [-0.72876678207075196, -0.016030958882900848, 0.31136367053725356, 0.4736018523880543]]], [[[-0.49061444452775582, 0.81114387212475636, 0.38995779155236199, 0.68167382208379479], [-0.99234885343352053, 0.54948405195911665, 0.67748560641952316, 0.12361771392509757], [0.74957930366266456, 0.29970297400899271, 0.3159611266551301, -0.42046308727640236]], [[0.20950673516815188, -0.088341400215833366, 0.9638973190702973, 0.68199731178218692], [-0.57396477728598105, -0.89652900611840525, 0.019727515752113067, 0.57108671455067683], [0.70179048884165618, -0.72010473438276934, 0.82364341343487069, 0.81816644420876461]]]])
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.46361503964213124, 0.75212103924315676, 0.1517014593635575, 0.63148557170685327], [-0.45199615428664996, 0.42928007237587473, 0.85118517914010172, -0.15509137168645615], [0.15514320286516092, 0.20315619172000665, 0.25175605070668317, 0.908865463056445]], [[0.45054003146551436, 0.47087051768615695, -0.076540616512914772, 0.10502324913621375], [0.13680386381556775, -0.43979686709904531, -0.79541946133102126, 0.84162201515236612], [-0.26616111363933515, 0.010265849576416963, -0.74540501454095254, -0.31553251505895985]]], [[[0.67609301986855086, 0.37746787067053145, -0.42950164086299925, 0.54536804846096953], [0.3373674099598567, 0.60934895052910298, 0.93458147614245779, 0.81566556247952771], [0.28566419298828816, 0.66853942853358883, 0.56086919503548471, 0.49829638104085539]], [[0.23053049901245104, -0.63018177830499922, -0.53909789907854089, 0.32371305451881383], [0.47127259923995135, 0.50137166714297887, 0.95947548444362751, 0.87100864021002966], [-0.62331578035775403, -0.016030958882900848, 0.98453152930704513, 0.4736018523880543]]], [[[0.030889419318346212, 0.81114387212475636, 0.38995779155236199, 0.68167382208379479], [0.2727885358054869, 0.54948405195911665, 0.67748560641952316, 0.12361771392509757], [0.74957930366266456, 0.29970297400899271, 0.3159611266551301, 0.20168949679051473]], [[0.20950673516815188, -0.088341400215833366, 0.9638973190702973, 0.68199731178218692], [-0.16868935807808083, -0.31479531035901021, 0.019727515752113067, 0.57108671455067683], [0.70179048884165618, 0.42602189213064201, 0.82364341343487069, 0.81816644420876461]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_Symbol_rank4(self):
      arg0=Data(0.779736643798,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[0.63523675174647853, 0.38201082516767793, -0.20793716944450535, -0.63763646646312888], [0.834635387648758, -0.17680511068025861, 0.396315589707259, 0.11161680583935429], [-0.73869666308710835, -0.49009415248425725, 0.91186235080476585, -0.73184937821061569]], [[0.29296083296199282, 0.68080308442172255, -0.28515442484539988, -0.68360700469277513], [-0.27425080713688788, -0.21608242948540779, 0.2641923511851838, -0.98133514226068663], [-0.3814071825404457, -0.10623863191344207, 0.31035955573337604, 0.56608271252944742]]], [[[0.40557289599793145, 0.71289586399346105, 0.18676151577894973, -0.36486848211263023], [0.015987521664037052, -0.61454925338975319, 0.83473516520110924, 0.65255714442596346], [0.53737380711513572, 0.39006109295194147, 0.34144737973591632, -0.51867680380978776]], [[0.62905861891302006, 0.9272306847780154, -0.19029217178583258, 0.83209580335004119], [0.014784510552173336, 0.59641741385026537, -0.10011590287108874, -0.28753968515006356], [0.59953709339077244, -0.77203120330570019, 0.92668220579915639, -0.98693965215801116]]], [[[0.042998917634785805, 0.94576064365865564, -0.97301185415108771, 0.80265301874270079], [0.45826573366446288, 0.53177767011301391, 0.4221224976289133, 0.081570270788827948], [-0.23367502994461908, -0.051912331731638117, 0.49090314291706783, -0.00634503874320691]], [[0.83127420461334278, 0.16630541192993142, 0.54676273477515869, -0.42579432540758377], [0.90427503268496379, -0.9638633447444942, -0.94230249615905826, 0.61536244029307308], [0.4077827929061324, 0.89001033043558708, 0.26837782147501321, -0.034119015798711905]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.834635387648758, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.91186235080476585, 0.77973664379814167]], [[0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167]]], [[[0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.83473516520110924, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167]], [[0.77973664379814167, 0.9272306847780154, 0.77973664379814167, 0.83209580335004119], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.92668220579915639, 0.77973664379814167]]], [[[0.77973664379814167, 0.94576064365865564, 0.77973664379814167, 0.80265301874270079], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167]], [[0.83127420461334278, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.90427503268496379, 0.77973664379814167, 0.77973664379814167, 0.77973664379814167], [0.77973664379814167, 0.89001033043558708, 0.77973664379814167, 0.77973664379814167]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[0.11823709792393644, 0.57743427123415247, -0.47743748205799741, -0.25860465344987116], [-0.85342597746354443, 0.11397059023125844, -0.32681677820897659, -0.4288280260721582], [0.4662138324786389, 0.14147352071096919, 0.19750088925070242, 0.94387770790006087]], [[0.46496806851074601, 0.54406435609557624, -0.13736364411105972, -0.1802701617446667], [-0.48294385750438318, -0.90642315431975162, -0.14370791133432848, 0.46285396642950349], [0.57423763420356289, -0.60637963806576178, -0.44242702298232706, 0.60815013943825491]]], [[[-0.38932408289704612, 0.14626247172828211, -0.67169729383052701, -0.17668123573312444], [0.85536638615608918, 0.27777955127024501, -0.22927675627685118, 0.47133043947122588], [0.98633096485626948, -0.67344233684153321, -0.55759479090990816, 0.95392042633643581]], [[0.98926847688099429, -0.099775998668338861, -0.38220886856369973, 0.30538447323514117], [0.4769519856326716, -0.93678787932837371, 0.23713788866934005, -0.039710268152491013], [0.9479451531859, -0.29175471478610504, 0.70374847469318591, 0.45115262591873062]]], [[[0.81351288943358235, -0.3670301216083045, -0.26084469022562451, -0.19412768911684464], [-0.77387156646633315, -0.0018219771195073786, 0.62751947270548869, 0.63243293117888832], [0.96618766991196492, -0.90165237989619973, 0.11374318928241434, 0.26911135553743204]], [[0.82323210574075234, -0.53634841475736783, -0.5667782787281912, -0.2378237881048284], [0.1765664584729234, 0.18583021463594585, -0.11567401544857003, -0.14016246157527679], [0.15134244191084112, -0.94560346887402424, 0.94835435091970544, -0.77704296677274565]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[0.16899177524516928, -0.34732858643370546, -0.79882685938170694, -0.66038397595048615], [0.059068592737719916, -0.8830498377662197, 0.42039477387821811, 0.32883530186540688], [0.8004432686322831, 0.4513771244635616, 0.39893889542471173, 0.38953533377812555]], [[0.8549919640811019, 0.0085317390737900123, 0.9335898541916583, 0.098754781628490163], [0.66089186768623187, 0.84486059895315635, -0.49885852697655153, 0.68792401902966405], [-0.17004486075932879, -0.033475700920452045, -0.0082240694292592398, -0.97121871328749743]]], [[[-0.64603684287873531, -0.67737694403929827, -0.17922106732871224, -0.91211139825890175], [-0.4553044126204413, 0.074503564238266096, -0.21232591403526246, -0.059843420314997564], [-0.38994792354603125, -0.28399207815642069, -0.19705530292188755, -0.18417115963918018]], [[-0.018546350001871215, -0.35927915222869711, -0.27093146477076302, 0.21888297390394529], [0.16698354221314671, -0.77658313684702263, -0.72342182944478717, -0.41851758261220473], [-0.73791291651917956, 0.95640694294369277, 0.79377010505148649, 0.9488141093625535]]], [[[-0.7879905919921153, -0.47441581145629574, -0.75295386253986418, 0.50309712614614854], [0.85117753483747327, 0.16901189435505559, 0.27950826734646883, 0.927929405822288], [-0.11332190163979683, -0.90078094125877928, 0.015048983765326485, -0.35804541069235873]], [[-0.44383068491491673, 0.73699838067473666, -0.16448971670195967, 0.63214802470153497], [-0.030217824939371418, -0.55117960400873911, 0.7891485388245143, -0.020033647442706304], [0.74505558968503083, -0.051569338830459222, -0.54218896195541788, 0.23706541588721119]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.16899177524516928, 0.57743427123415247, -0.47743748205799741, -0.25860465344987116], [0.059068592737719916, 0.11397059023125844, 0.42039477387821811, 0.32883530186540688], [0.8004432686322831, 0.4513771244635616, 0.39893889542471173, 0.94387770790006087]], [[0.8549919640811019, 0.54406435609557624, 0.9335898541916583, 0.098754781628490163], [0.66089186768623187, 0.84486059895315635, -0.14370791133432848, 0.68792401902966405], [0.57423763420356289, -0.033475700920452045, -0.0082240694292592398, 0.60815013943825491]]], [[[-0.38932408289704612, 0.14626247172828211, -0.17922106732871224, -0.17668123573312444], [0.85536638615608918, 0.27777955127024501, -0.21232591403526246, 0.47133043947122588], [0.98633096485626948, -0.28399207815642069, -0.19705530292188755, 0.95392042633643581]], [[0.98926847688099429, -0.099775998668338861, -0.27093146477076302, 0.30538447323514117], [0.4769519856326716, -0.77658313684702263, 0.23713788866934005, -0.039710268152491013], [0.9479451531859, 0.95640694294369277, 0.79377010505148649, 0.9488141093625535]]], [[[0.81351288943358235, -0.3670301216083045, -0.26084469022562451, 0.50309712614614854], [0.85117753483747327, 0.16901189435505559, 0.62751947270548869, 0.927929405822288], [0.96618766991196492, -0.90078094125877928, 0.11374318928241434, 0.26911135553743204]], [[0.82323210574075234, 0.73699838067473666, -0.16448971670195967, 0.63214802470153497], [0.1765664584729234, 0.18583021463594585, 0.7891485388245143, -0.020033647442706304], [0.74505558968503083, -0.051569338830459222, 0.94835435091970544, 0.23706541588721119]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_constData_rank4(self):
      arg0=Data(-0.893686645203,self.functionspace)
      arg1=Data(numpy.array([[[[0.61638843267588261, -0.97285254339522864, 0.39019707596035991, -0.38329856435815723], [-0.39669633259426185, -0.43633677524387782, -0.062118904024617283, 0.52486356486440955], [0.59806243722015462, -0.95562790660740671, 0.89716362287428741, 0.25616048508546663]], [[0.24435399713825645, 0.58840030186856374, -0.11669001468268436, -0.10473414559607241], [0.53850954948156304, -0.21927908342820301, 0.5308446295654845, -0.14536822578932429], [0.90594574080059331, 0.77607397603230743, 0.34830152058308195, 0.53599148614980741]]], [[[0.84897407852148055, -0.54152647293785039, 0.93603932280166657, 0.8311250335877951], [-0.048354616437206177, 0.82001605920812848, 0.92696057539180776, -0.08827000636161686], [0.020618916247708174, 0.98375130765872387, 0.8200458168780016, -0.053631253048684435]], [[-0.37789023552666512, 0.052215678410592714, -0.18675922076149987, 0.43345944086720611], [-0.30827748070532701, -0.66668820782501603, 0.084841064566959279, -0.95519159649105756], [-0.6725560045726664, -0.53620416688473016, -0.25767276940639294, 0.58749559487682013]]], [[[-0.19257594002810241, 0.72549845431588622, 0.39704918849500959, 0.61577341276098552], [0.70773259021309665, -0.22540767589709043, -0.031031950353477278, -0.33370198061247858], [-0.8280003728298464, -0.81439343811994092, 0.95962098528552975, 0.50268932526386201]], [[-0.37240249982818407, -0.92063840650310702, 0.12378723541570058, -0.75766552223525885], [-0.8507935520807457, 0.40017389307358187, -0.23436141671947053, 0.5452149596180087], [0.49123509401676135, -0.2193283129711836, 0.29665843719288354, -0.61641191860539335]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.61638843267588261, -0.89368664520320795, 0.39019707596035991, -0.38329856435815723], [-0.39669633259426185, -0.43633677524387782, -0.062118904024617283, 0.52486356486440955], [0.59806243722015462, -0.89368664520320795, 0.89716362287428741, 0.25616048508546663]], [[0.24435399713825645, 0.58840030186856374, -0.11669001468268436, -0.10473414559607241], [0.53850954948156304, -0.21927908342820301, 0.5308446295654845, -0.14536822578932429], [0.90594574080059331, 0.77607397603230743, 0.34830152058308195, 0.53599148614980741]]], [[[0.84897407852148055, -0.54152647293785039, 0.93603932280166657, 0.8311250335877951], [-0.048354616437206177, 0.82001605920812848, 0.92696057539180776, -0.08827000636161686], [0.020618916247708174, 0.98375130765872387, 0.8200458168780016, -0.053631253048684435]], [[-0.37789023552666512, 0.052215678410592714, -0.18675922076149987, 0.43345944086720611], [-0.30827748070532701, -0.66668820782501603, 0.084841064566959279, -0.89368664520320795], [-0.6725560045726664, -0.53620416688473016, -0.25767276940639294, 0.58749559487682013]]], [[[-0.19257594002810241, 0.72549845431588622, 0.39704918849500959, 0.61577341276098552], [0.70773259021309665, -0.22540767589709043, -0.031031950353477278, -0.33370198061247858], [-0.8280003728298464, -0.81439343811994092, 0.95962098528552975, 0.50268932526386201]], [[-0.37240249982818407, -0.89368664520320795, 0.12378723541570058, -0.75766552223525885], [-0.8507935520807457, 0.40017389307358187, -0.23436141671947053, 0.5452149596180087], [0.49123509401676135, -0.2193283129711836, 0.29665843719288354, -0.61641191860539335]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[0.96485565509648352, 0.53935464926562759, 0.53077936146728266, -0.73952245406800854], [-0.60881766722021902, 0.66032495037649741, 0.93304470628387515, -0.29318048118189055], [-0.66535631878242762, 0.86695831494535347, -0.14571574174128665, -0.33755901373168684]], [[0.10130701439391632, -0.17559510038495674, 0.48296274631000302, -0.57408428494447028], [-0.27410670377127633, 0.17219315446797312, -0.60634684633721503, 0.20140988897507128], [0.063883370598835532, 0.062627983781888386, -0.2846290129863831, 0.15747009011324331]]], [[[-0.33786498745323823, -0.77773342159826253, -0.43944390559155377, 0.30699880066796359], [0.45721299355990963, -0.93225225636245979, 0.24951116488168035, -0.60399840561802765], [-0.98061260494043778, -0.75645258975794905, 0.27958959667741956, 0.087975846205211594]], [[0.66543448707063324, 0.85514480449431862, 0.82014945665654015, 0.81505258529464553], [0.36193943466925615, -0.43856446384821646, -0.28386557764570841, -0.44311355588507073], [0.8203258501802293, 0.88410934322225532, -0.42070109791391741, 0.095773374554660284]]], [[[-0.27073826415250957, 0.66740636068176817, -0.48896464867053013, -0.94994837491554485], [0.6540939867398663, 0.3623199797026595, -0.46325457322592967, -0.018711908974897273], [-0.35561156732560661, 0.41410104364264733, -0.17250699705434713, -0.25548148839396423]], [[-0.52110139543812939, -0.48803425309908843, 0.96222057216100421, -0.53953717021125525], [-0.50454727047867465, 0.66698964265585015, -0.15287504292958864, 0.50355653533991096], [0.83323461461179282, 0.62448988217093437, 0.4654771267287694, 0.72161921224557446]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-0.57504027263496882, -0.70962427655760041, 0.5000686248515589, 0.22836735474708636], [-0.2456640208529155, 0.55145194339939985, -0.18188128973631157, -0.010052160520559017], [-0.67026258406375816, -0.097417557973243252, 0.78510475463922025, -0.96873477555278131]], [[-0.10002520261013537, 0.63100857099364571, -0.37013228408685617, -0.59175793402432819], [0.77337490554492994, 0.46196627477364838, -0.64798814992967224, 0.60516599121325076], [-0.0017246851860892587, -0.30784264595534161, -0.65209147103284981, -0.88757120178936799]]], [[[0.80040151821525596, 0.43450571239861646, 0.0078842302106048034, -0.55464057489099483], [0.84844302837406849, 0.48335761669393862, 0.018983703352744907, 0.89616530091553726], [-0.57373660097290591, -0.84825568142179031, -0.35092918722312461, 0.92340153576769501]], [[-0.85244829995021609, 0.095947521825416882, -0.18971802853754727, -0.65909953891892914], [0.11749664922727288, 0.13980571669569475, -0.79988329726892982, 0.055016274898991968], [0.35743533014846873, 0.27824733541877245, 0.42999566458196914, 0.3568350967844669]]], [[[0.99608799667364067, -0.5403793610139318, -0.12575108132177171, 0.41721666984548333], [0.875733275967848, 0.9523625165119125, -0.20675560052583641, -0.41564394921784187], [-0.84876383654236665, 0.62662285663526829, 0.49660771467477982, 0.5918409282574939]], [[-0.4729461618005697, -0.64161728948155772, 0.65333699389309863, 0.49984898391509858], [0.43829686077623897, 0.33192534406508289, 0.61100656789543994, 0.073960447209926006], [0.78875551306588365, -0.25000292972836191, 0.99289944129181062, -0.54605870017388525]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      ref=Data(numpy.array([[[[0.96485565509648352, 0.53935464926562759, 0.53077936146728266, 0.22836735474708636], [-0.2456640208529155, 0.66032495037649741, 0.93304470628387515, -0.010052160520559017], [-0.66535631878242762, 0.86695831494535347, 0.78510475463922025, -0.33755901373168684]], [[0.10130701439391632, 0.63100857099364571, 0.48296274631000302, -0.57408428494447028], [0.77337490554492994, 0.46196627477364838, -0.60634684633721503, 0.60516599121325076], [0.063883370598835532, 0.062627983781888386, -0.2846290129863831, 0.15747009011324331]]], [[[0.80040151821525596, 0.43450571239861646, 0.0078842302106048034, 0.30699880066796359], [0.84844302837406849, 0.48335761669393862, 0.24951116488168035, 0.89616530091553726], [-0.57373660097290591, -0.75645258975794905, 0.27958959667741956, 0.92340153576769501]], [[0.66543448707063324, 0.85514480449431862, 0.82014945665654015, 0.81505258529464553], [0.36193943466925615, 0.13980571669569475, -0.28386557764570841, 0.055016274898991968], [0.8203258501802293, 0.88410934322225532, 0.42999566458196914, 0.3568350967844669]]], [[[0.99608799667364067, 0.66740636068176817, -0.12575108132177171, 0.41721666984548333], [0.875733275967848, 0.9523625165119125, -0.20675560052583641, -0.018711908974897273], [-0.35561156732560661, 0.62662285663526829, 0.49660771467477982, 0.5918409282574939]], [[-0.4729461618005697, -0.48803425309908843, 0.96222057216100421, 0.49984898391509858], [0.43829686077623897, 0.66698964265585015, 0.61100656789543994, 0.50355653533991096], [0.83323461461179282, 0.62448988217093437, 0.99289944129181062, 0.72161921224557446]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank0_expandedData_rank4(self):
      arg0=Data(0.216711670991,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.81879161478097595, 0.23338466982083461, -0.037340658437786667, 0.53960130549007368], [-0.8433547141558082, 0.51986550271704757, -0.22844534933352723, -0.89465705240856908], [-0.11648963460000861, -0.96923924737633538, 0.072867751072468012, 0.65625904184925732]], [[0.27849706772004201, -0.90647830830724296, 0.96202673895146806, 0.22494056179803157], [-0.2684069854476685, -0.42191439566230504, -0.95625432477352734, 0.72510262095158207], [-0.036924511893698497, -0.93553684683379412, -0.76943122405185593, 0.42971341646797767]]], [[[0.89494738055097778, 0.46650780268669378, 0.30405300775028121, -0.084542513395697849], [-0.027722135654347779, -0.59483396322557391, -0.028304510159490759, -0.88731646326202407], [0.34624637943920167, -0.39682309028436302, 0.89367998074843014, 0.56073508085183299]], [[0.63023798830300204, 0.18652798267686688, -0.3328850324391277, -0.21182328663126193], [-0.53239281368561087, 0.22479105625018425, 0.90626113618872939, -7.4251614887721473e-05], [-0.98623780619390233, -0.5666180903186695, -0.30927062686357099, -0.79727648923625138]]], [[[0.59536350392532866, 0.22068473036479674, -0.66623026326771462, 0.10843352912766102], [0.52432284313983835, -0.3431109607082834, -0.23886295117126921, -0.8119295310297503], [-0.54397735011730486, -0.7068782977204644, 0.71740443446430424, -0.31144890281105586]], [[-0.32351391151939302, -0.35563858193171405, 0.65601036166328086, -0.032917953499258745], [0.45870081001448093, 0.3006509634165655, -0.15553712418631416, 0.85594927731050774], [-0.84241444057987591, -0.64337027261577129, 0.32134231645586109, -0.39606743840399128]]]])+(1.-msk_arg1)*numpy.array([[[[0.55712979668912488, -0.80066523551532121, -0.48656514655750849, 0.76097556013292111], [-0.13438565652155154, -0.94821563688317712, -0.31012625194007182, -0.84105859136182715], [-0.20208116248301411, -0.50024164072503563, 0.87481453005108034, 0.13755575289800137]], [[-0.085598442928114249, -0.45841913232395637, -0.18238598324276789, 0.62303900657350142], [-0.79318456738858467, 0.34558408974177413, -0.8550879149504329, 0.8287523579134235], [-0.40328058936497357, -0.83102127169123086, -0.21653057906368112, -0.76688016358972666]]], [[[0.68068340217757584, 0.57860355650918649, -0.19514598498967861, 0.35797229868730107], [-0.940274604582233, -0.52273338186671348, -0.37314192021830439, 0.10804775393755039], [-0.20074283030723383, 0.95964123909155385, -0.98950080490821501, -0.58835700500290256]], [[0.99348893772020896, 0.85220955503331552, 0.41443782029517151, 0.83659124171927268], [-0.65635663807364564, 0.70812110348599355, 0.13053831014253792, 0.91230146888102226], [0.19243965515044836, 0.96567248884233092, -0.28016453566744892, -0.65082014990966219]]], [[[-0.013133698627046275, -0.9481821696335877, 0.63769890160711773, -0.35875665803034429], [-0.65944938772191586, 0.01066509243901459, 0.17762038374459466, -0.21321783232210256], [0.99670034374764271, 0.67813675209818114, 0.37840221867825585, 0.45475310270530489]], [[0.29249236353371622, -0.51219658392539991, 0.22458795099500506, -0.29060307474253499], [-0.86387736284336181, -0.17064143853256808, -0.45438045421185458, 0.9948613071200707], [-0.38789172342813538, 0.052527231540111297, 0.81849101524424928, 0.43381945218064333]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.21671167099079991, 0.23338466982083461, 0.21671167099079991, 0.53960130549007368], [0.21671167099079991, 0.51986550271704757, 0.21671167099079991, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.65625904184925732]], [[0.27849706772004201, 0.21671167099079991, 0.96202673895146806, 0.22494056179803157], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.72510262095158207], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.42971341646797767]]], [[[0.89494738055097778, 0.46650780268669378, 0.30405300775028121, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.34624637943920167, 0.21671167099079991, 0.89367998074843014, 0.56073508085183299]], [[0.63023798830300204, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.21671167099079991, 0.22479105625018425, 0.90626113618872939, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991]]], [[[0.59536350392532866, 0.22068473036479674, 0.21671167099079991, 0.21671167099079991], [0.52432284313983835, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.71740443446430424, 0.21671167099079991]], [[0.21671167099079991, 0.21671167099079991, 0.65601036166328086, 0.21671167099079991], [0.45870081001448093, 0.3006509634165655, 0.21671167099079991, 0.85594927731050774], [0.21671167099079991, 0.21671167099079991, 0.32134231645586109, 0.21671167099079991]]]])+(1.-msk_ref)*numpy.array([[[[0.55712979668912488, 0.21671167099079991, 0.21671167099079991, 0.76097556013292111], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.87481453005108034, 0.21671167099079991]], [[0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.62303900657350142], [0.21671167099079991, 0.34558408974177413, 0.21671167099079991, 0.8287523579134235], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991]]], [[[0.68068340217757584, 0.57860355650918649, 0.21671167099079991, 0.35797229868730107], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.21671167099079991, 0.95964123909155385, 0.21671167099079991, 0.21671167099079991]], [[0.99348893772020896, 0.85220955503331552, 0.41443782029517151, 0.83659124171927268], [0.21671167099079991, 0.70812110348599355, 0.21671167099079991, 0.91230146888102226], [0.21671167099079991, 0.96567248884233092, 0.21671167099079991, 0.21671167099079991]]], [[[0.21671167099079991, 0.21671167099079991, 0.63769890160711773, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.21671167099079991], [0.99670034374764271, 0.67813675209818114, 0.37840221867825585, 0.45475310270530489]], [[0.29249236353371622, 0.21671167099079991, 0.22458795099500506, 0.21671167099079991], [0.21671167099079991, 0.21671167099079991, 0.21671167099079991, 0.9948613071200707], [0.21671167099079991, 0.21671167099079991, 0.81849101524424928, 0.43381945218064333]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[0.9662251484946478, -0.26481381091852141, -0.066210160157844644, 0.40117951882974068], [0.2455027289465177, 0.67324997630420214, 0.79365479453824861, -0.037015541405008623], [-0.86030987461175457, -0.45616271741243231, 0.39179913013670342, -0.10731696095517629]], [[-0.19449522639452876, -0.67751174669485992, 0.28984158743095256, 0.67531831661784203], [-0.49980904983410723, -0.02577831866914404, 0.94104429684279367, 0.74763465415851904], [-0.28985388443946092, -0.90882129286702029, -0.90352359388616366, 0.0516463980384263]]], [[[-0.99639029481638874, 0.23581131279953027, 0.73436613098786441, 0.89956373143685453], [0.70388018442763145, 0.079895063328745408, -0.75367452457404527, 0.56811816966845319], [0.90284242548995652, 0.66397516482541619, -0.99647484543635345, 0.79709159927353346]], [[0.70723973142155505, 0.27467701034793879, -0.79995723813418396, -0.24824137685387848], [0.56082965044735578, 0.73559725370917817, 0.62475940100976302, 0.030745279822806282], [-0.36016041397070708, 0.7839330226446426, -0.69945965257975473, -0.89855265013928132]]], [[[0.33431193949654969, -0.46895691087357716, 0.94874414920913108, 0.35408330328067761], [-0.56725159045543427, -0.11646626979600483, -0.27664126953599744, -0.62696292984368651], [0.15060075336069922, -0.16895232031523566, -0.83212543422584218, 0.37516376063681012]], [[-0.21699302716045343, -0.24013090348080812, 0.73993829114859677, -0.63476480682303671], [-0.34968439152048214, 0.46819693714489508, 0.62165805854076095, -0.3791178743428667], [-0.41700973768159155, 0.79899832669176507, -0.86119824926464239, 0.46575159472502037]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.31931488193208657, -0.28905671753527695, 0.85561932051536327, 0.43258332189744197], [-0.022093948889682302, -0.87293800937103039, 0.56614871319438853, 0.86910428719314292], [0.68092117827772292, 0.53186005878844611, 0.28162238085357139, 0.97361457365289072]], [[-0.79666566403785954, 0.37011155709186694, -0.68233116747040423, 0.89387098751104865], [0.42703403901311821, 0.89550507893657039, 0.84501584710121325, -0.66998942788818261], [0.53860906470280701, -0.094095450408768766, -0.35365080444144259, 0.4361075442378417]]], [[[-0.16099982532347235, -0.8651116694760872, 0.52385238626653319, -0.28446906365453173], [0.14671592161598701, 0.71409402259173094, 0.3258739797599115, 0.70362092527950693], [-0.32717756310317503, -0.95828422715748962, 0.68727489580226919, 0.15249900882725109]], [[0.22017885162589113, 0.80937795601648843, 0.35945120041216438, 0.93900043111170084], [0.79190649660945756, 0.57203937052107467, 0.54292126003639929, 0.84571132339638178], [0.23760803956899457, 0.52713270420645153, -0.24975442892820454, -0.72715475618178438]]], [[[-0.84787512935758658, -0.62122949761588964, 0.68568216243118529, 0.34087206419913829], [-0.66077267614956314, 0.88586283683410505, 0.48865077882965502, -0.2823801328248261], [0.24010264814531568, -0.77577498913372622, -0.55288042007565297, -0.34642896895972042]], [[-0.42554067083401659, 0.87743453278083261, 0.50397543167721515, 0.97556159346990001], [0.84530378175248244, 0.66561045498275107, 0.80033289693830301, 0.92226186452120906], [-0.4442108477506006, -0.52692217686024989, -0.85590042003205369, 0.43455207703783549]]]])+(1.-msk_arg1)*numpy.array([[[[0.21605231242464118, 0.32170152972234489, 0.69540488109778908, 0.2547128534596883], [0.34796901405025005, -0.31607291081984523, -0.98038121857023519, -0.93906508361402574], [-0.28276602806278417, -0.62275848697078584, -0.26878568593014385, -0.22758564658522573]], [[0.85908277675070277, 0.70033299040044406, 0.93445045187257891, -0.75943811515314663], [0.50550467560364476, -0.89661957212631349, -0.20586932595053264, 0.74555292517790739], [-0.46974519139411886, -0.064153765304233668, -0.5695552407484461, -0.11199926962703399]]], [[[-0.046076882407277919, -0.4751273456616687, 0.54211989560690821, -0.95839866060661283], [-0.34429450468712863, 0.9945935413627196, 0.12045306605821482, 0.70763311498764381], [-0.93229388462703189, -0.8159573239034752, 0.43375216165599206, -0.58156200439008288]], [[-0.94359110518590672, 0.21676492225881594, -0.57979669038590131, 0.035244336548687416], [-0.13786684734995425, 0.75386861643073622, 0.022898320967623453, 0.66532275876665459], [-0.30607908894667135, -0.066287086564079223, -0.64210438699958949, 0.58486901207821607]]], [[[0.69742093261447113, 0.77935504242461695, 0.28045816177452632, 0.35225109286857292], [-0.96809103221557291, 0.77688458560421636, 0.92713086412266987, 0.49029829063110997], [-0.1905974439471938, 0.14508130554930188, 0.24154428575645048, 0.45490680510213011]], [[-0.021068579578790381, -0.055644755704836957, -0.71431963870922144, -0.65498904995738161], [0.87855331454880914, 0.53311774880814933, 0.18160214068170566, -0.33338460760283883], [-0.49933663079121393, -0.30118428496778038, 0.86142651573184215, -0.99665689200339136]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.9662251484946478, -0.26481381091852141, 0.85561932051536327, 0.43258332189744197], [0.2455027289465177, 0.67324997630420214, 0.79365479453824861, 0.86910428719314292], [0.68092117827772292, 0.53186005878844611, 0.39179913013670342, 0.97361457365289072]], [[-0.19449522639452876, 0.37011155709186694, 0.28984158743095256, 0.89387098751104865], [0.42703403901311821, 0.89550507893657039, 0.94104429684279367, 0.74763465415851904], [0.53860906470280701, -0.094095450408768766, -0.35365080444144259, 0.4361075442378417]]], [[[-0.16099982532347235, 0.23581131279953027, 0.73436613098786441, 0.89956373143685453], [0.70388018442763145, 0.71409402259173094, 0.3258739797599115, 0.70362092527950693], [0.90284242548995652, 0.66397516482541619, 0.68727489580226919, 0.79709159927353346]], [[0.70723973142155505, 0.80937795601648843, 0.35945120041216438, 0.93900043111170084], [0.79190649660945756, 0.73559725370917817, 0.62475940100976302, 0.84571132339638178], [0.23760803956899457, 0.7839330226446426, -0.24975442892820454, -0.72715475618178438]]], [[[0.33431193949654969, -0.46895691087357716, 0.94874414920913108, 0.35408330328067761], [-0.56725159045543427, 0.88586283683410505, 0.48865077882965502, -0.2823801328248261], [0.24010264814531568, -0.16895232031523566, -0.55288042007565297, 0.37516376063681012]], [[-0.21699302716045343, 0.87743453278083261, 0.73993829114859677, 0.97556159346990001], [0.84530378175248244, 0.66561045498275107, 0.80033289693830301, 0.92226186452120906], [-0.41700973768159155, 0.79899832669176507, -0.85590042003205369, 0.46575159472502037]]]])+(1.-msk_ref)*numpy.array([[[[0.9662251484946478, 0.32170152972234489, 0.69540488109778908, 0.40117951882974068], [0.34796901405025005, 0.67324997630420214, 0.79365479453824861, -0.037015541405008623], [-0.28276602806278417, -0.45616271741243231, 0.39179913013670342, -0.10731696095517629]], [[0.85908277675070277, 0.70033299040044406, 0.93445045187257891, 0.67531831661784203], [0.50550467560364476, -0.02577831866914404, 0.94104429684279367, 0.74763465415851904], [-0.28985388443946092, -0.064153765304233668, -0.5695552407484461, 0.0516463980384263]]], [[[-0.046076882407277919, 0.23581131279953027, 0.73436613098786441, 0.89956373143685453], [0.70388018442763145, 0.9945935413627196, 0.12045306605821482, 0.70763311498764381], [0.90284242548995652, 0.66397516482541619, 0.43375216165599206, 0.79709159927353346]], [[0.70723973142155505, 0.27467701034793879, -0.57979669038590131, 0.035244336548687416], [0.56082965044735578, 0.75386861643073622, 0.62475940100976302, 0.66532275876665459], [-0.30607908894667135, 0.7839330226446426, -0.64210438699958949, 0.58486901207821607]]], [[[0.69742093261447113, 0.77935504242461695, 0.94874414920913108, 0.35408330328067761], [-0.56725159045543427, 0.77688458560421636, 0.92713086412266987, 0.49029829063110997], [0.15060075336069922, 0.14508130554930188, 0.24154428575645048, 0.45490680510213011]], [[-0.021068579578790381, -0.055644755704836957, 0.73993829114859677, -0.63476480682303671], [0.87855331454880914, 0.53311774880814933, 0.62165805854076095, -0.33338460760283883], [-0.41700973768159155, 0.79899832669176507, 0.86142651573184215, 0.46575159472502037]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.318576793935)+(1.-msk_arg0)*(-0.920315318998)
      arg1=0.264453770493
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.318576793935)+(1.-msk_ref)*(0.264453770493)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.61984078319308633, -0.74126411397009218])+(1.-msk_arg0)*numpy.array([-0.36354960922197699, -0.45840304611368676])
      arg1=0.798333407454
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.79833340745371917, 0.79833340745371917])+(1.-msk_ref)*numpy.array([0.79833340745371917, 0.79833340745371917])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.56027127038237556, -0.38092190445101282, -0.77591749903008322, -0.36171347397940457, -0.3091925404043383], [0.34776167092224908, -0.37902110057843474, 0.8566813057073317, -0.87799841344488661, 0.61149184516634381], [0.11264186928512365, 0.34551824833197631, 0.4058993843871157, 0.42214558457038209, -0.88397115089856348], [0.57017035572765851, -0.17472794481465992, -0.37739320194928472, 0.24459442745468318, -0.0032651849035065794]])+(1.-msk_arg0)*numpy.array([[-0.63041245210130414, -0.40539069241410175, 0.95484833881413089, 0.76528251808922398, -0.45472655311970289], [-0.34559052794751488, -0.91180569915010246, -0.08113468153701886, 0.31746466254664774, 0.69806708955802943], [0.76702933573803689, -0.24228609093935383, 0.35224880600670572, 0.47231857975553981, -0.88896833667212327], [-0.087726728409511789, -0.23153089968162344, 0.39249534318815371, 0.23502532808353349, 0.68373987182917317]])
      arg1=0.989825243681
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427]])+(1.-msk_ref)*numpy.array([[0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427], [0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427, 0.98982524368093427]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.028875141511675473, 0.59607526974242653], [-0.85743668105869553, 0.70579283933075554]], [[0.53533801908700807, -0.63679610683946919], [-0.079395571307905088, 0.48032940038802163]], [[0.62755828198212904, -0.43131717689352711], [-0.69602024353252068, -0.39729826147508174]], [[0.871577527593145, 0.13860228070825675], [-0.41566762577178307, 0.94629933788596232]], [[0.67599858677777047, 0.69484194011263689], [0.60029405821428594, 0.28493844302972837]], [[-0.71484527552962907, 0.22995449367240717], [0.791550071610712, -0.87117943761246108]]])+(1.-msk_arg0)*numpy.array([[[0.78242950070530659, 0.014091229458666366], [0.25389636301837526, -0.32650460454438401]], [[-0.2924625700857808, 0.6366577134210214], [-0.93500357448199378, -0.040378324895707918]], [[-0.50017267851444713, -0.27455430871359354], [-0.54888413525358515, 0.39328658448761655]], [[0.32330904571084806, -0.58124263920494301], [0.15299156800247538, -0.28958015194809961]], [[-0.2894735013468257, 0.39047784429864718], [0.41819146976323429, 0.016153713585879137]], [[0.8615981317548056, -0.37210375853443756], [-0.84988813841877109, 0.90373148295773231]]])
      arg1=0.832320572018
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.871577527593145, 0.8323205720176674], [0.8323205720176674, 0.94629933788596232]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]]])+(1.-msk_ref)*numpy.array([[[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8323205720176674, 0.8323205720176674], [0.8323205720176674, 0.8323205720176674]], [[0.8615981317548056, 0.8323205720176674], [0.8323205720176674, 0.90373148295773231]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.42251039851841288, -0.5086459170688129, -0.45492102664037537, 0.31392311416403995], [0.65608273573528852, -0.85710626049138106, 0.21505967960738959, 0.70718137746639131], [0.83790464665837794, -0.60856345438902792, -0.60079149562707634, -0.1076802420164078]], [[0.46732380623178282, -0.35500225168117239, 0.25444444559611989, -0.48744596442284194], [0.90027866484871244, -0.31706597272192649, 0.59522974780728655, -0.47446522005210379], [-0.8633196459105501, 0.66961511233335314, 0.061156594494290673, 0.79799072926268422]]], [[[-0.76276876249566472, -0.099532420785620968, 0.10129392224865463, 0.39167447951002199], [-0.98828984273881293, -0.57023782466024131, -0.065377529996557548, -0.40442289015670507], [-0.82339803606408557, 0.014662422844692102, 0.21649789675243114, 0.77505805175113651]], [[-0.76191062946085264, 0.42107637324614444, 0.92357976930849417, -0.83786675093267315], [0.52554082210275488, -0.8888682988825547, 0.40287390701474046, -0.52290176387004972], [-0.34201346434522772, 0.38682001181478975, -0.819612138653941, -0.94002052598078389]]], [[[-0.51734417834945168, -0.94482365146869829, 0.64319147138071275, -0.35722472189352694], [-0.94118399117998175, -0.28909676588864341, -0.77521699548798018, 0.57822873832654831], [0.89952167291261254, 0.6107886611155009, -0.013437891648716782, -0.474928738027242]], [[0.26286809808854472, 0.25318532614933087, -0.74577774138101782, -0.5726071651639919], [0.97839123557228058, 0.9460156616470814, -0.35659193985301241, 0.12569654130562768], [-0.99682390578515179, -0.60347225844308694, 0.39905112336897774, -0.6498254841152844]]]])+(1.-msk_arg0)*numpy.array([[[[-0.43200273953647184, 0.38040962014976043, -0.6572991923365028, 0.45207958622632516], [-0.58836374817160375, 0.26862299998111294, 0.21139361003683721, 0.3086343700658376], [-0.77309660882283571, -0.70430093120909287, -0.56474331239969677, -0.017208266779719983]], [[0.67436506707034893, -0.30858059192041742, -0.031394042016385626, -0.63378689685176881], [-0.66569084269765399, 0.97185177189397032, -0.58300442478147785, 0.70372679431572527], [-0.022365773489303686, 0.85035019395735212, -0.97846447525858959, -0.64044194234297014]]], [[[-0.97775916672490459, -0.41628654841289214, -0.48245585793355605, -0.055238920243368517], [0.78673608724885491, -0.19279316198073482, -0.98976200752193, -0.47776310463081839], [0.84717802695622835, -0.68427094271230504, -0.4804198631320109, 0.96098954130826031]], [[0.53394958997155872, -0.22737455212877689, 0.30471343662030992, 0.18229541027562624], [0.37038417314131489, 0.87665381096600825, 0.44054927441029701, 0.35339162806138336], [-0.17989219181739258, 0.71059747268101714, -0.98051690316527695, 0.27992594956317007]]], [[[0.52813647387794704, 0.024570478697439002, 0.93464353314847237, -0.84326079071506377], [-0.53538275657360623, 0.93198072028257628, -0.74578726744677581, -0.35281011313225541], [0.56547473195301534, 0.11350722177621453, 0.12064243147442166, -0.56343163545267116]], [[-0.048739251574649201, 0.24929112934527575, 0.99164564508467357, -0.44909041546971151], [0.74190021533591577, 0.65438363932926258, 0.4220458996941816, -0.69453814291522642], [-0.30803415118241451, 0.24508007551774624, 0.53629014177782874, 0.31168691811329174]]]])
      arg1=0.734113764334
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.83790464665837794, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]], [[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.90027866484871244, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.79799072926268422]]], [[[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.77505805175113651]], [[0.7341137643336908, 0.7341137643336908, 0.92357976930849417, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]]], [[[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.89952167291261254, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]], [[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.97839123557228058, 0.9460156616470814, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]]]])+(1.-msk_ref)*numpy.array([[[[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]], [[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.97185177189397032, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.85035019395735212, 0.7341137643336908, 0.7341137643336908]]], [[[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.78673608724885491, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.84717802695622835, 0.7341137643336908, 0.7341137643336908, 0.96098954130826031]], [[0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.87665381096600825, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]]], [[[0.7341137643336908, 0.7341137643336908, 0.93464353314847237, 0.7341137643336908], [0.7341137643336908, 0.93198072028257628, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]], [[0.7341137643336908, 0.7341137643336908, 0.99164564508467357, 0.7341137643336908], [0.74190021533591577, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908], [0.7341137643336908, 0.7341137643336908, 0.7341137643336908, 0.7341137643336908]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.0285167629509)+(1.-msk_arg0)*(-0.0753791701623)
      arg1=numpy.array(0.369649061202)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.369649061202)+(1.-msk_ref)*(0.369649061202)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.32342379681372324, 0.54353331547928652])+(1.-msk_arg0)*numpy.array([0.22862832886262274, 0.31483883730737539])
      arg1=numpy.array(0.626949968399)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.62694996839906447, 0.62694996839906447])+(1.-msk_ref)*numpy.array([0.62694996839906447, 0.62694996839906447])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.71394651499733919, -0.92287830402130533, -0.90858139421470385, -0.13825598900369229, -0.080541309303345132], [-0.36949666654546265, 0.3898727048518853, -0.28621012865918916, -0.25638266490918404, 0.20407905334345466], [0.15975716776790327, 0.86409710321773558, 0.71725217347095049, 0.20875761848553376, -0.24238991057422754], [-0.78556146630181956, -0.076076120078814924, 0.5830295620999455, 0.62973766057396441, -0.44550377230379912]])+(1.-msk_arg0)*numpy.array([[-0.88705425678644301, 0.55511600368349057, 0.50879851258581632, 0.65698430346681214, -0.92944433639452151], [0.72904334195690801, -0.57898988094606052, 0.80587168656123565, 0.75367205014945649, -0.51428464418517805], [-0.32501367205410969, -0.97880582883393408, 0.053009572718297759, 0.094411264210482848, -0.63030483500737655], [-0.088813446479829139, -0.75746914620099148, -0.95633858232383417, 0.67276534899551677, 0.15202939959842365]])
      arg1=numpy.array(-0.940581531438)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.71394651499733919, -0.92287830402130533, -0.90858139421470385, -0.13825598900369229, -0.080541309303345132], [-0.36949666654546265, 0.3898727048518853, -0.28621012865918916, -0.25638266490918404, 0.20407905334345466], [0.15975716776790327, 0.86409710321773558, 0.71725217347095049, 0.20875761848553376, -0.24238991057422754], [-0.78556146630181956, -0.076076120078814924, 0.5830295620999455, 0.62973766057396441, -0.44550377230379912]])+(1.-msk_ref)*numpy.array([[-0.88705425678644301, 0.55511600368349057, 0.50879851258581632, 0.65698430346681214, -0.92944433639452151], [0.72904334195690801, -0.57898988094606052, 0.80587168656123565, 0.75367205014945649, -0.51428464418517805], [-0.32501367205410969, -0.94058153143782453, 0.053009572718297759, 0.094411264210482848, -0.63030483500737655], [-0.088813446479829139, -0.75746914620099148, -0.94058153143782453, 0.67276534899551677, 0.15202939959842365]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.18937114215158379, 0.97884605242301004], [-0.10174593321344649, -0.022379247683969705]], [[0.37143236733782325, 0.9436323979261656], [-0.1396616687095853, -0.36032181421557552]], [[-0.082984451143639948, 0.68652743342457567], [-0.935628845473649, 0.58410565742224385]], [[-0.12311403457235026, 0.051753925365314313], [-0.19479735727282055, -0.97762777864901618]], [[-0.43740194607254024, 0.047952734299117505], [-0.83030948972383301, 0.2720270466764092]], [[-0.13916673117721445, -0.38789957921110485], [-0.82523648008893313, -0.90290874072044436]]])+(1.-msk_arg0)*numpy.array([[[0.45196786956134583, 0.61219599413960424], [-0.065721595612748152, -0.26282718706162234]], [[-0.77537708674852834, 0.44532451113123472], [-0.069741033742395819, 0.85312396785377786]], [[0.77152934839087872, -0.89912864802471115], [-0.59570931290004303, -0.20558993214725563]], [[0.47614495941699153, -0.87515809599830097], [0.5635701134536959, 0.79401765404455427]], [[-0.22008420609503343, -0.65942311245997409], [0.9655251213411864, -0.30378791706072894]], [[0.44247117863110397, 0.10977233527612396], [0.050396835055231071, -0.17185541796448978]]])
      arg1=numpy.array(0.879188127312)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.87918812731204588, 0.97884605242301004], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.9436323979261656], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]]])+(1.-msk_ref)*numpy.array([[[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.9655251213411864, 0.87918812731204588]], [[0.87918812731204588, 0.87918812731204588], [0.87918812731204588, 0.87918812731204588]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.44580962530016999, -0.69348488146163345, -0.34481050040016448, 0.59519412601278909], [-0.29649588803121163, -0.90433699382125621, 0.58229774413551061, -0.36778950106866248], [-0.367202105893363, -0.11391186768016848, 0.091423849403935442, -0.80223166648482014]], [[-0.80093892706918934, -0.29174863765813419, -0.16211197355096374, 0.075768040268325176], [0.68362107850802523, 0.30569303924697744, -0.27076611907986936, -0.32562966437337582], [0.32515793722254704, 0.077836894356146047, -0.37435906797035168, -0.35187117123705081]]], [[[-0.51098052012853468, -0.13010630276624302, 0.87102220930593499, -0.64012790280688536], [-0.21267317186303125, -0.88402284628937533, 0.02431388699609216, 0.06052877315768912], [-0.28034820201167854, 0.95297399082866185, -0.30163718369713011, -0.50183508057815374]], [[-0.43680686484138076, -0.82955436525397341, -0.30888908734602927, 0.73714563499555674], [0.85936269560114997, -0.37602998325975867, -0.16401661856146643, 0.42565242322259933], [-0.27441411269419902, 0.29164819210771387, -0.85938658660708112, -0.99892556232036434]]], [[[-0.81554459002181479, -0.35510738606811576, -0.6024526618358339, -0.96050342654364496], [-0.56351464899865644, -0.15196137810022159, 0.10412313747497359, 0.2755721298497178], [0.69743953672566561, -0.36580894827046495, 0.95677556962500332, -0.99552242481327524]], [[-0.11222970781965746, -0.33762110851060645, 0.27106365544031208, -0.29815607031304237], [-0.056603466192955532, -0.93774716471967889, 0.74223460094575988, 0.42675210408069519], [-0.73797087885832946, -0.14895862461116716, -0.026765745522702833, 0.86420158258370283]]]])+(1.-msk_arg0)*numpy.array([[[[0.19372975479304544, -0.66808663318684403, 0.77537567177849098, 0.81919260999807397], [-0.10116256869273443, -0.57520810468712269, 0.15753839520629942, 0.79473404582834428], [-0.73645830163893855, 0.87616863458367211, -0.96733449165902963, -0.3516939203286118]], [[0.48878397140623631, 0.45609994336875803, 0.91436485388943489, -0.17781328406697527], [0.65055833166980692, 0.10117618648410831, -0.09041980822458906, 0.59283939012926501], [0.78272644881934017, 0.25217910521330689, -0.84476748260357493, -0.42169419337471803]]], [[[-0.45223518020548759, 0.38375427581487509, -0.033829350614732911, 0.70903023968037648], [-0.80644295317701453, 0.54199751717049516, -0.3972649595244353, -0.26862061395744208], [0.163939148868391, -0.19776011299147966, -0.29279137734597294, 0.38651062526683488]], [[0.73784058524755514, -0.62987052734714788, 0.96978901651430682, 0.68202116917743494], [-0.9741599743353393, -0.70746264080763765, -0.73637201007369524, -0.36338439617661566], [0.5758662884613357, 0.58540767089470402, -0.40111460235649776, 0.061425771822538389]]], [[[0.90108401417416362, 0.72943001699832255, -0.75909447019101606, 0.97176132609079291], [-0.31728686169672304, -0.40901210547887934, 0.96695071353736539, 0.032603047050764777], [-0.12055510695784455, 0.768561520375171, 0.948776815267379, -0.50392752129221652]], [[-0.87599954417448522, 0.95279874663930531, 0.90744334941927685, -0.50378095934118838], [0.9824024122546311, 0.69765742163802824, -0.38287651073484019, 0.40060850054520558], [0.91703259523128899, -0.39809207284495951, -0.84284247078113972, 0.36091797506547008]]]])
      arg1=numpy.array(-0.694396194523)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.44580962530016999, -0.69348488146163345, -0.34481050040016448, 0.59519412601278909], [-0.29649588803121163, -0.6943961945233823, 0.58229774413551061, -0.36778950106866248], [-0.367202105893363, -0.11391186768016848, 0.091423849403935442, -0.6943961945233823]], [[-0.6943961945233823, -0.29174863765813419, -0.16211197355096374, 0.075768040268325176], [0.68362107850802523, 0.30569303924697744, -0.27076611907986936, -0.32562966437337582], [0.32515793722254704, 0.077836894356146047, -0.37435906797035168, -0.35187117123705081]]], [[[-0.51098052012853468, -0.13010630276624302, 0.87102220930593499, -0.64012790280688536], [-0.21267317186303125, -0.6943961945233823, 0.02431388699609216, 0.06052877315768912], [-0.28034820201167854, 0.95297399082866185, -0.30163718369713011, -0.50183508057815374]], [[-0.43680686484138076, -0.6943961945233823, -0.30888908734602927, 0.73714563499555674], [0.85936269560114997, -0.37602998325975867, -0.16401661856146643, 0.42565242322259933], [-0.27441411269419902, 0.29164819210771387, -0.6943961945233823, -0.6943961945233823]]], [[[-0.6943961945233823, -0.35510738606811576, -0.6024526618358339, -0.6943961945233823], [-0.56351464899865644, -0.15196137810022159, 0.10412313747497359, 0.2755721298497178], [0.69743953672566561, -0.36580894827046495, 0.95677556962500332, -0.6943961945233823]], [[-0.11222970781965746, -0.33762110851060645, 0.27106365544031208, -0.29815607031304237], [-0.056603466192955532, -0.6943961945233823, 0.74223460094575988, 0.42675210408069519], [-0.6943961945233823, -0.14895862461116716, -0.026765745522702833, 0.86420158258370283]]]])+(1.-msk_ref)*numpy.array([[[[0.19372975479304544, -0.66808663318684403, 0.77537567177849098, 0.81919260999807397], [-0.10116256869273443, -0.57520810468712269, 0.15753839520629942, 0.79473404582834428], [-0.6943961945233823, 0.87616863458367211, -0.6943961945233823, -0.3516939203286118]], [[0.48878397140623631, 0.45609994336875803, 0.91436485388943489, -0.17781328406697527], [0.65055833166980692, 0.10117618648410831, -0.09041980822458906, 0.59283939012926501], [0.78272644881934017, 0.25217910521330689, -0.6943961945233823, -0.42169419337471803]]], [[[-0.45223518020548759, 0.38375427581487509, -0.033829350614732911, 0.70903023968037648], [-0.6943961945233823, 0.54199751717049516, -0.3972649595244353, -0.26862061395744208], [0.163939148868391, -0.19776011299147966, -0.29279137734597294, 0.38651062526683488]], [[0.73784058524755514, -0.62987052734714788, 0.96978901651430682, 0.68202116917743494], [-0.6943961945233823, -0.6943961945233823, -0.6943961945233823, -0.36338439617661566], [0.5758662884613357, 0.58540767089470402, -0.40111460235649776, 0.061425771822538389]]], [[[0.90108401417416362, 0.72943001699832255, -0.6943961945233823, 0.97176132609079291], [-0.31728686169672304, -0.40901210547887934, 0.96695071353736539, 0.032603047050764777], [-0.12055510695784455, 0.768561520375171, 0.948776815267379, -0.50392752129221652]], [[-0.6943961945233823, 0.95279874663930531, 0.90744334941927685, -0.50378095934118838], [0.9824024122546311, 0.69765742163802824, -0.38287651073484019, 0.40060850054520558], [0.91703259523128899, -0.39809207284495951, -0.6943961945233823, 0.36091797506547008]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.898401363569)+(1.-msk_arg0)*(-0.461611516853)
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.528111721902)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.898401363569)+(1.-msk_ref)*(-0.461611516853)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.27921448061888121, -0.32249539409259897])+(1.-msk_arg0)*numpy.array([-0.70841413542416731, 0.31689170070609274])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.00250798256919)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0025079825691880497, 0.0025079825691880497])+(1.-msk_ref)*numpy.array([0.0025079825691880497, 0.31689170070609274])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.42641848820329575, -0.45763524459085425, 0.2344734540420601, -0.96787976333155723, 0.97656103015032159], [0.82504846986485414, 0.85652985167315876, 0.26135179937225628, 0.070760946891457399, -0.090156190235288758], [0.13929321946239681, -0.41657937535911005, 0.34896897658333836, -0.33772282479090099, -0.36855043757355332], [-0.54862873652964095, -0.090044143750859007, 0.65954164408128668, 0.89804054387472365, 0.28478323180732534]])+(1.-msk_arg0)*numpy.array([[0.30357999812776781, 0.6947433594088992, -0.61066058551344859, -0.67498721750670931, 0.094688088720797259], [-0.77617047006506779, 0.85150466489009835, 0.10850223944373694, 0.14616996754542488, 0.32823487609690227], [-0.62185366333188341, -0.39685291408324463, -0.32113191680218578, -0.98631198793165442, -0.16480265791740822], [-0.88475577156552698, 0.29958085851021354, -0.99857897144759389, 0.24850774537797204, -0.95309401593682264]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.455749717973)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.42641848820329575, -0.45574971797293795, 0.2344734540420601, -0.45574971797293795, 0.97656103015032159], [0.82504846986485414, 0.85652985167315876, 0.26135179937225628, 0.070760946891457399, -0.090156190235288758], [0.13929321946239681, -0.41657937535911005, 0.34896897658333836, -0.33772282479090099, -0.36855043757355332], [-0.45574971797293795, -0.090044143750859007, 0.65954164408128668, 0.89804054387472365, 0.28478323180732534]])+(1.-msk_ref)*numpy.array([[0.30357999812776781, 0.6947433594088992, -0.45574971797293795, -0.45574971797293795, 0.094688088720797259], [-0.45574971797293795, 0.85150466489009835, 0.10850223944373694, 0.14616996754542488, 0.32823487609690227], [-0.45574971797293795, -0.39685291408324463, -0.32113191680218578, -0.45574971797293795, -0.16480265791740822], [-0.45574971797293795, 0.29958085851021354, -0.45574971797293795, 0.24850774537797204, -0.45574971797293795]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.59146727628596052, -0.69880068472985446], [-0.565323173304632, -0.99253459780838393]], [[-0.60501836732613423, 0.3625764440330661], [-0.6425334343620086, -0.057335382191270545]], [[0.27772683986831326, 0.17933249104967164], [-0.89658716007969419, 0.89419168028678264]], [[0.056090227699439943, -0.78275310715139979], [-0.16811940783507584, 0.06481262772531915]], [[0.76062101197939946, 0.37607298701045377], [0.76475505853012882, 0.23024333753573556]], [[-0.59217728233872013, 0.80566019828821389], [-0.032411211546716334, -0.44035092417812405]]])+(1.-msk_arg0)*numpy.array([[[-0.7955464873395941, 0.91123872126212579], [-0.028331950047604915, -0.087636398220037215]], [[0.53356700775760024, -0.43100126439479003], [0.51794560051482996, 0.19237163043374705]], [[0.3882676057097727, -0.86492964735620648], [0.94526767758073271, 0.59082462598647667]], [[-0.70426628860299356, -0.79562500016514548], [0.31134451536891028, -0.97255416094382263]], [[-0.27522179788979861, -0.088300508685171142], [-0.94750899935074195, 0.70706855388599421]], [[-0.82684870726394966, -0.61450321429161114], [0.39774041852061148, -0.57489247418150802]]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(0.414790844179)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.41479084417882084]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.41479084417882084]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.89419168028678264]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.41479084417882084]], [[0.76062101197939946, 0.41479084417882084], [0.76475505853012882, 0.41479084417882084]], [[0.41479084417882084, 0.80566019828821389], [0.41479084417882084, 0.41479084417882084]]])+(1.-msk_ref)*numpy.array([[[0.41479084417882084, 0.91123872126212579], [0.41479084417882084, 0.41479084417882084]], [[0.53356700775760024, 0.41479084417882084], [0.51794560051482996, 0.41479084417882084]], [[0.41479084417882084, 0.41479084417882084], [0.94526767758073271, 0.59082462598647667]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.41479084417882084]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.70706855388599421]], [[0.41479084417882084, 0.41479084417882084], [0.41479084417882084, 0.41479084417882084]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.26229435386837174, -0.017116628775625031, 0.70496262799237175, -0.29666710913320005], [-0.46458259211531261, -0.15839500174424948, -0.99482155870216071, 0.58111235343104362], [-0.24452460291422451, -0.78345408572424957, 0.11078458345530517, 0.1853232390393813]], [[-0.7195286876707454, -0.6310213470623609, 0.28283545439874924, 0.31076826190063045], [-0.20201988880735589, -0.14232763419418082, -0.76413979220304884, 0.86543879745088237], [-0.50873695339348446, 0.25699271487230346, 0.92206176319070998, 0.3336515646867344]]], [[[0.26385066597032991, -0.483967427087862, 0.74092852919156593, 0.94923669216624695], [-0.1876338737215657, -0.94191662896543593, -0.79183641068730481, 0.41003517358429353], [-0.02979135807280775, 0.18058122424506151, -0.21693636020847862, -0.33724384873918734]], [[-0.1314085748883016, 0.59820364481135146, -0.34212279685731528, 0.35858122991793095], [0.64329159332285513, -0.44653235038867578, -0.79440149065385968, -0.10817822047421877], [-0.2321630486471753, -0.31495540543079192, 0.57206783537205519, -0.35518023940138166]]], [[[-0.66276371131004019, -0.91281554320670044, -0.59705564040168513, -0.71808909818688194], [-0.38182391680868144, -0.71301467143963637, -0.61720893621899808, -0.94709795774501093], [0.62145095307532272, -0.70077833391697952, 0.99591552231169023, 0.58701095298918604]], [[-0.34001498732999047, 0.70912734246134246, 0.50991749391034347, -0.48305262805142668], [0.42647298317303628, -0.24659348312134188, 0.14711947973134887, -0.095371389015735497], [0.76017531047368569, -0.59159788878078379, 0.32466387353668136, -0.12706448315955621]]]])+(1.-msk_arg0)*numpy.array([[[[-0.37061676038540736, 0.37484370638231068, -0.98269574633406465, 0.89262597493077878], [-0.40312768261797749, -0.12243299290921228, 0.32687102327077588, -0.77782021310224891], [0.60499468143098456, -0.97474902820512765, 0.94525234907216538, 0.75821293847253135]], [[-0.70197426653520067, 0.39656820477115673, 0.26811912724684461, 0.49440907452822613], [-0.79016407456584026, 0.38674631743817023, -0.28080182157286138, 0.33625641716034949], [0.65268663396169813, -0.22232327581802558, -0.85446230497611242, -0.67250812530974402]]], [[[0.34442733633336919, -0.113701015818922, -0.81324035042088916, -0.78209319148462031], [-0.63182369698047358, 0.17461013527763036, -0.75677244659679554, 0.017784198785844252], [0.7474482995412941, 0.32317663366728966, 0.50995093222735499, 0.67982047479509511]], [[0.7528523996187424, 0.16872619171042103, -0.27194042913737859, 0.25211713456112794], [0.81816548887017171, -0.39986027775515565, -0.54257017324625911, 0.68819235584870153], [0.64479669008589435, 0.8861821440266171, -0.13594013870352928, 0.019005940280650835]]], [[[0.76419242456835001, -0.94637582068954962, 0.32041176124394544, -0.089178201833403969], [-0.53789044901791616, 0.01994584808566402, 0.66403281687876614, 0.42096967291751741], [0.10461792832675387, 0.30397131202338823, -0.67956784333797282, 0.056431392608151221]], [[-0.88678403236362913, -0.072578204652525846, -0.72079941061473418, 0.72022679113672949], [0.68678018903947824, -0.49815905771716507, 0.39334864307863637, -0.28393966821154182], [0.51686376326498262, -0.026598589920490134, 0.63586174987802391, -0.22462630160390185]]]])
      arg1=Symbol(shape=())
      res=maximum(arg0,arg1)
      s1=numpy.array(-0.38655297508)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.26229435386837174, -0.017116628775625031, 0.70496262799237175, -0.29666710913320005], [-0.38655297508002606, -0.15839500174424948, -0.38655297508002606, 0.58111235343104362], [-0.24452460291422451, -0.38655297508002606, 0.11078458345530517, 0.1853232390393813]], [[-0.38655297508002606, -0.38655297508002606, 0.28283545439874924, 0.31076826190063045], [-0.20201988880735589, -0.14232763419418082, -0.38655297508002606, 0.86543879745088237], [-0.38655297508002606, 0.25699271487230346, 0.92206176319070998, 0.3336515646867344]]], [[[0.26385066597032991, -0.38655297508002606, 0.74092852919156593, 0.94923669216624695], [-0.1876338737215657, -0.38655297508002606, -0.38655297508002606, 0.41003517358429353], [-0.02979135807280775, 0.18058122424506151, -0.21693636020847862, -0.33724384873918734]], [[-0.1314085748883016, 0.59820364481135146, -0.34212279685731528, 0.35858122991793095], [0.64329159332285513, -0.38655297508002606, -0.38655297508002606, -0.10817822047421877], [-0.2321630486471753, -0.31495540543079192, 0.57206783537205519, -0.35518023940138166]]], [[[-0.38655297508002606, -0.38655297508002606, -0.38655297508002606, -0.38655297508002606], [-0.38182391680868144, -0.38655297508002606, -0.38655297508002606, -0.38655297508002606], [0.62145095307532272, -0.38655297508002606, 0.99591552231169023, 0.58701095298918604]], [[-0.34001498732999047, 0.70912734246134246, 0.50991749391034347, -0.38655297508002606], [0.42647298317303628, -0.24659348312134188, 0.14711947973134887, -0.095371389015735497], [0.76017531047368569, -0.38655297508002606, 0.32466387353668136, -0.12706448315955621]]]])+(1.-msk_ref)*numpy.array([[[[-0.37061676038540736, 0.37484370638231068, -0.38655297508002606, 0.89262597493077878], [-0.38655297508002606, -0.12243299290921228, 0.32687102327077588, -0.38655297508002606], [0.60499468143098456, -0.38655297508002606, 0.94525234907216538, 0.75821293847253135]], [[-0.38655297508002606, 0.39656820477115673, 0.26811912724684461, 0.49440907452822613], [-0.38655297508002606, 0.38674631743817023, -0.28080182157286138, 0.33625641716034949], [0.65268663396169813, -0.22232327581802558, -0.38655297508002606, -0.38655297508002606]]], [[[0.34442733633336919, -0.113701015818922, -0.38655297508002606, -0.38655297508002606], [-0.38655297508002606, 0.17461013527763036, -0.38655297508002606, 0.017784198785844252], [0.7474482995412941, 0.32317663366728966, 0.50995093222735499, 0.67982047479509511]], [[0.7528523996187424, 0.16872619171042103, -0.27194042913737859, 0.25211713456112794], [0.81816548887017171, -0.38655297508002606, -0.38655297508002606, 0.68819235584870153], [0.64479669008589435, 0.8861821440266171, -0.13594013870352928, 0.019005940280650835]]], [[[0.76419242456835001, -0.38655297508002606, 0.32041176124394544, -0.089178201833403969], [-0.38655297508002606, 0.01994584808566402, 0.66403281687876614, 0.42096967291751741], [0.10461792832675387, 0.30397131202338823, -0.38655297508002606, 0.056431392608151221]], [[-0.38655297508002606, -0.072578204652525846, -0.38655297508002606, 0.72022679113672949], [0.68678018903947824, -0.38655297508002606, 0.39334864307863637, -0.28393966821154182], [0.51686376326498262, -0.026598589920490134, 0.63586174987802391, -0.22462630160390185]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.88000228503)+(1.-msk_arg0)*(-0.20984850813)
      arg1=Data(0.659456392175,self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.88000228503)+(1.-msk_ref)*(0.659456392175)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.71272356771458245, -0.3611606406674901])+(1.-msk_arg0)*numpy.array([-0.82598998478380836, -0.42990113071867553])
      arg1=Data(0.178686493024,self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.17868649302435347, 0.17868649302435347])+(1.-msk_ref)*numpy.array([0.17868649302435347, 0.17868649302435347])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.63593420649217469, 0.071252699353494764, 0.52730188526412869, 0.90707321993763812, -0.49826390881410676], [0.65603487673248218, -0.1221632218420734, -0.65505386558118928, -0.092624138241382603, -0.60221631581080404], [0.59133951441423194, 0.62499728002156107, 0.91980036751900829, -0.76085845352133918, -0.85482925516349528], [-0.080754965852170235, 0.044026814393698155, -0.39390549886182535, -0.39036051478539835, -0.22353088797836307]])+(1.-msk_arg0)*numpy.array([[-0.72561188067886895, 0.44063138647043476, -0.71568806092968162, 0.79302562670403565, -0.7203898051760802], [-0.47457497511194724, 0.49128310742058168, 0.24651099047400171, -0.8094160968572508, -0.24832575278907898], [-0.38354631430621122, 0.21584585668083234, 0.88500663492033116, -0.17117462898833979, 0.91967256531048047], [0.53756830959719037, -0.037911466866273003, 0.014786652808508993, -0.43313218614069782, 0.83859088561946526]])
      arg1=Data(0.24971708422,self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.63593420649217469, 0.24971708421996541, 0.52730188526412869, 0.90707321993763812, 0.24971708421996541], [0.65603487673248218, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541], [0.59133951441423194, 0.62499728002156107, 0.91980036751900829, 0.24971708421996541, 0.24971708421996541], [0.24971708421996541, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541]])+(1.-msk_ref)*numpy.array([[0.24971708421996541, 0.44063138647043476, 0.24971708421996541, 0.79302562670403565, 0.24971708421996541], [0.24971708421996541, 0.49128310742058168, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541], [0.24971708421996541, 0.24971708421996541, 0.88500663492033116, 0.24971708421996541, 0.91967256531048047], [0.53756830959719037, 0.24971708421996541, 0.24971708421996541, 0.24971708421996541, 0.83859088561946526]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.22561961094336969, -0.69644978225088083], [-0.44227121250995638, -0.50735454965999582]], [[0.16161200783821461, -0.4347630322634457], [0.26562304460337183, -0.15583347531179226]], [[0.59080340657647357, -0.30898005352956903], [-0.96090859496814618, 0.64957222823764948]], [[-0.78194949614475751, -0.96330346961526092], [-0.75472165908116562, -0.97849212182888756]], [[-0.75432107675432469, 0.082043851417474389], [0.47974376178170175, 0.6709257227867087]], [[0.54824661624696036, -0.54157677822383921], [-0.659240045201267, 0.18082823678245341]]])+(1.-msk_arg0)*numpy.array([[[-0.54709583559254926, -0.15868154997696871], [0.026457093040474833, 0.56251013754601731]], [[-0.37622675052380505, -0.23721419821220846], [0.52430722837144028, -0.02821186532212705]], [[-0.7164360240992329, 0.87804574359157361], [-0.36055956747943774, 0.8665138503873846]], [[-0.87103016729055427, -0.36978512045411382], [0.16675457478226341, 0.23700377551418117]], [[0.32530875953749505, 0.73065087014702934], [-0.31548638071883328, -0.89169514477602374]], [[-0.58353491789348433, 0.3017692417364648], [0.42105486881407872, -0.24670377062806792]]])
      arg1=Data(-0.751837483947,self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.22561961094336969, -0.69644978225088083], [-0.44227121250995638, -0.50735454965999582]], [[0.16161200783821461, -0.4347630322634457], [0.26562304460337183, -0.15583347531179226]], [[0.59080340657647357, -0.30898005352956903], [-0.75183748394658645, 0.64957222823764948]], [[-0.75183748394658645, -0.75183748394658645], [-0.75183748394658645, -0.75183748394658645]], [[-0.75183748394658645, 0.082043851417474389], [0.47974376178170175, 0.6709257227867087]], [[0.54824661624696036, -0.54157677822383921], [-0.659240045201267, 0.18082823678245341]]])+(1.-msk_ref)*numpy.array([[[-0.54709583559254926, -0.15868154997696871], [0.026457093040474833, 0.56251013754601731]], [[-0.37622675052380505, -0.23721419821220846], [0.52430722837144028, -0.02821186532212705]], [[-0.7164360240992329, 0.87804574359157361], [-0.36055956747943774, 0.8665138503873846]], [[-0.75183748394658645, -0.36978512045411382], [0.16675457478226341, 0.23700377551418117]], [[0.32530875953749505, 0.73065087014702934], [-0.31548638071883328, -0.75183748394658645]], [[-0.58353491789348433, 0.3017692417364648], [0.42105486881407872, -0.24670377062806792]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.46836650421512216, 0.55392434050869133, 0.64462355173707331, 0.47549964468485606], [-0.69289889981348418, -0.20461909725159777, -0.95905690844675684, 0.01420826477124737], [-0.77419156074874329, 0.93441370846146876, 0.67855558163879071, 0.16075772354444062]], [[-0.0065968567345067175, 0.20050959624333031, 0.72598290161102197, -0.97695441137457473], [0.34654853789303552, 0.48359989540054027, 0.45795932416242602, -0.13036567808708344], [-0.45679097037796268, 0.15641582786074903, 0.31786075279898829, -0.23870739994752288]]], [[[0.95042836806476449, 0.78075242319855431, -0.36462778215860348, -0.81076625675969294], [0.11060510010106617, -0.72824020802919986, -0.56633417520061813, -0.45562654021780635], [-0.42709566814202682, 0.79610178616158822, -0.85661543848488786, 0.070560494507247107]], [[0.20045514586255098, -0.19237619847201781, 0.49739688886609956, -0.96963620011390494], [0.17787451059406711, 0.95809663437507897, -0.68662219002349167, 0.20731333335407776], [-0.40982045954261337, -0.16017268023574838, -0.24073312330589358, -0.043607865804729862]]], [[[-0.89903746166893161, 0.21373862072105831, -0.01480721421090192, -0.12849469109133538], [0.89888673761286042, 0.87560850881946917, -0.81952018477772159, 0.37617546473368257], [0.92421029785719599, 0.26088854799923022, -0.75492467331593938, 0.8355004322022761]], [[-0.98145328336063553, -0.93694991169090569, 0.36611389422804419, 0.49453177652277347], [0.35699505986429214, -0.73480516245106497, 0.15750510738097434, 0.60146580130477645], [0.8080086070813568, -0.57380389192843828, 0.45232139162778928, -0.87668421492511817]]]])+(1.-msk_arg0)*numpy.array([[[[0.79049818853845744, -0.31643284915831793, 0.62698204144556136, 0.15055585522974413], [0.18428359666870175, -0.90655403355361042, -0.61799856818968246, -0.96209670683248194], [-0.22150547598602555, 0.11608945933719728, -0.93550584238462808, -0.51437923919154094]], [[0.11433307037970808, -0.6697957728739139, 0.026080474663143338, 0.89687571470160266], [0.74220257866685402, 0.80630222035973476, -0.88765397198480867, -0.30073269119267354], [-0.61692766321637516, 0.70275565435456011, -0.770480369255093, 0.45855506259202561]]], [[[-0.25841786485345164, -0.9569035410607365, -0.2354046937481602, -0.76468796690158114], [-0.12764743888647057, -0.7570280529729656, -0.31857315603959679, 0.15543454155327607], [-0.46592327565824387, -0.60197786685351118, 0.24325043755979214, -0.70586610361238722]], [[0.25903971611354648, -0.516785599192078, -0.4534446250373132, 0.19387204679020309], [-0.43563421185323126, 0.55496223606378137, -0.13284603494736147, -0.58152191275770404], [-0.19132818776329441, 0.16638009745376592, 0.92524151693459689, -0.22952254139222439]]], [[[0.3341414793432731, -0.40904633866616935, 0.86990541483951533, -0.38690851826482731], [0.34494363122687943, 0.41021617144572242, 0.33061965307913654, -0.43580755194659915], [-0.69103379639840234, 0.50350152095540768, -0.69843540817247174, 0.26288005317296004]], [[-0.2886477174481421, -0.78908077047169134, 0.11673976034770406, 0.33019842460207882], [-0.93492125398225467, -0.2334918373016639, 0.98554713583062536, 0.4728612818618716], [-0.93118935725322149, 0.039886713080873326, 0.023210062083922844, -0.17367472025873631]]]])
      arg1=Data(0.515617958919,self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.51561795891922024, 0.55392434050869133, 0.64462355173707331, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.93441370846146876, 0.67855558163879071, 0.51561795891922024]], [[0.51561795891922024, 0.51561795891922024, 0.72598290161102197, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]]], [[[0.95042836806476449, 0.78075242319855431, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.79610178616158822, 0.51561795891922024, 0.51561795891922024]], [[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.95809663437507897, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]]], [[[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.89888673761286042, 0.87560850881946917, 0.51561795891922024, 0.51561795891922024], [0.92421029785719599, 0.51561795891922024, 0.51561795891922024, 0.8355004322022761]], [[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.60146580130477645], [0.8080086070813568, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]]]])+(1.-msk_ref)*numpy.array([[[[0.79049818853845744, 0.51561795891922024, 0.62698204144556136, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]], [[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.89687571470160266], [0.74220257866685402, 0.80630222035973476, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.70275565435456011, 0.51561795891922024, 0.51561795891922024]]], [[[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]], [[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.55496223606378137, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.92524151693459689, 0.51561795891922024]]], [[[0.51561795891922024, 0.51561795891922024, 0.86990541483951533, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]], [[0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.98554713583062536, 0.51561795891922024], [0.51561795891922024, 0.51561795891922024, 0.51561795891922024, 0.51561795891922024]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.802743243229)+(1.-msk_arg0)*(0.99702688738)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.596467110586)+(1.-msk_arg1)*(0.684381079372)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.802743243229)+(1.-msk_ref)*(0.99702688738)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.30614519824934328, -0.51813576976876141])+(1.-msk_arg0)*numpy.array([0.5500576508538344, 0.12364229321815245])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.321480996125)+(1.-msk_arg1)*(0.0905449773774)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.32148099612549919, 0.32148099612549919])+(1.-msk_ref)*numpy.array([0.5500576508538344, 0.12364229321815245])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.35552309152320305, -0.11101213305383895, -0.74227520152636406, 0.46252596749984454, 0.42113714908898525], [0.93097659800329025, 0.83327502155685007, 0.52757195622578923, 0.061911816792792118, -0.70771644366942166], [0.073915923799455552, -0.5867632562644447, -0.47160386240000851, -0.37172839238353261, 0.91218541682748122], [0.8577488749130433, -0.56376354937296624, -0.81071810619576512, 0.81701750260161798, -0.06159461532096655]])+(1.-msk_arg0)*numpy.array([[-0.14196838411601753, 0.54924551192337701, 0.19495347583316969, -0.68202363369328567, 0.39461470689048062], [-0.3688063281037306, 0.48318268985280555, -0.77231524148262065, 0.98493877845621736, -0.42013126010835244], [-0.9491470463541829, 0.10891550790333016, 0.93107340408360417, -0.70621836778754332, 0.71341896718143105], [-0.45972826679918621, -0.21350972161348403, 0.96659767157606402, -0.12881865656272806, 0.4043650376973722]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.67091545012)+(1.-msk_arg1)*(-0.0757810945746)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.35552309152320305, -0.11101213305383895, -0.67091545012013643, 0.46252596749984454, 0.42113714908898525], [0.93097659800329025, 0.83327502155685007, 0.52757195622578923, 0.061911816792792118, -0.67091545012013643], [0.073915923799455552, -0.5867632562644447, -0.47160386240000851, -0.37172839238353261, 0.91218541682748122], [0.8577488749130433, -0.56376354937296624, -0.67091545012013643, 0.81701750260161798, -0.06159461532096655]])+(1.-msk_ref)*numpy.array([[-0.075781094574565167, 0.54924551192337701, 0.19495347583316969, -0.075781094574565167, 0.39461470689048062], [-0.075781094574565167, 0.48318268985280555, -0.075781094574565167, 0.98493877845621736, -0.075781094574565167], [-0.075781094574565167, 0.10891550790333016, 0.93107340408360417, -0.075781094574565167, 0.71341896718143105], [-0.075781094574565167, -0.075781094574565167, 0.96659767157606402, -0.075781094574565167, 0.4043650376973722]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.73259259492432927, 0.43516685226353369], [0.34658816662510294, 0.99694890396299152]], [[-0.82513842361582301, -0.60640821327530237], [-0.46651350541260106, -0.44412570768110937]], [[-0.095664979228247793, 0.53884106143660015], [-0.15097890528281876, -0.0053807492881323693]], [[0.83781924943391406, -0.99066586932757672], [-0.88389801870266527, 0.065863228211280322]], [[-0.1940964066265598, -0.44657666544066421], [-0.7618618185275754, -0.022638072362826334]], [[0.64174101237204484, -0.75430513988969916], [0.97134520182564943, 0.4795408665156784]]])+(1.-msk_arg0)*numpy.array([[[0.26927998366552952, 0.75523312878691362], [0.076434487275322427, -0.6819391552501628]], [[-0.48721751558695492, -0.18534329149598827], [-0.40788258318505388, -0.70764982356182471]], [[-0.82449663467055356, 0.16419130511566893], [0.25011445773188701, 0.1035128720077596]], [[-0.0022868899113452912, -0.47461046292673736], [-0.48883758713916237, 0.93798235093853721]], [[0.18720827904762127, -0.0030792936071819543], [-0.47477258458581706, -0.62011335136235024]], [[-0.63547863145971673, -0.81616381306744112], [-0.87756446408192956, 0.25672900864254466]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.043916106452)+(1.-msk_arg1)*(0.158271071418)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.043916106452033032, 0.43516685226353369], [0.34658816662510294, 0.99694890396299152]], [[-0.043916106452033032, -0.043916106452033032], [-0.043916106452033032, -0.043916106452033032]], [[-0.043916106452033032, 0.53884106143660015], [-0.043916106452033032, -0.0053807492881323693]], [[0.83781924943391406, -0.043916106452033032], [-0.043916106452033032, 0.065863228211280322]], [[-0.043916106452033032, -0.043916106452033032], [-0.043916106452033032, -0.022638072362826334]], [[0.64174101237204484, -0.043916106452033032], [0.97134520182564943, 0.4795408665156784]]])+(1.-msk_ref)*numpy.array([[[0.26927998366552952, 0.75523312878691362], [0.15827107141755392, 0.15827107141755392]], [[0.15827107141755392, 0.15827107141755392], [0.15827107141755392, 0.15827107141755392]], [[0.15827107141755392, 0.16419130511566893], [0.25011445773188701, 0.15827107141755392]], [[0.15827107141755392, 0.15827107141755392], [0.15827107141755392, 0.93798235093853721]], [[0.18720827904762127, 0.15827107141755392], [0.15827107141755392, 0.15827107141755392]], [[0.15827107141755392, 0.15827107141755392], [0.15827107141755392, 0.25672900864254466]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.97239764595471057, -0.43270302318573983, -0.17542209868837455, -0.71317358004463216], [0.72878255833928107, 0.25393968296339708, 0.7062928811864444, 0.70798565970935257], [-0.8868456954486732, -0.074127186200250605, -0.26518707102168726, -0.91047990906076226]], [[-0.5485021908717751, 0.4625407566939097, -0.013117973326893795, 0.98963345359662513], [-0.56450486895185792, 0.40832041648188855, -0.48341521803155008, -0.99848277869207203], [0.30951894134880442, 0.60735662998233275, 0.61717782998775084, 0.19184298106071851]]], [[[0.6509286048782652, -0.86979300962840544, -0.5411735404074498, 0.88956340514970922], [-0.96087197906551292, -0.21534927161261552, -0.26855994833000074, 0.87089013542946114], [-0.62559526136686361, 0.64612780085586707, -0.71008042900676327, -0.12675921354333219]], [[0.14266331123609066, 0.12940917183095224, 0.53440833399954135, 0.5541645660247565], [0.045871400215263813, 0.0028821188154224409, 0.61511017352453567, 0.23973103956187747], [-0.5445038209174482, 0.91013574526764818, -0.23848811565899952, -0.50332896814919081]]], [[[0.68813990143394066, -0.056178984894786232, -0.90169036885976483, -0.53603412124753502], [0.72461066752157022, -0.78363334606207768, -0.30673442696718567, -0.57239754291823752], [0.47800787094455832, 0.22707910699571876, 0.15163871045515576, 0.34159974670530202]], [[0.22466552974210119, 0.61881183505470827, 0.1234877438382167, -0.12282347214627087], [0.61057990868408396, 0.8011211522833841, 0.04817598302759829, -0.69530242501742423], [-0.89443503352680165, -0.28642491039227203, 0.85109648851133368, 0.76098655847489716]]]])+(1.-msk_arg0)*numpy.array([[[[-0.78310919117047861, 0.85334677407078319, -0.62426072225732399, 0.46836843422702801], [0.19400083911486732, 0.51787351106685575, 0.77734214592705597, -0.95555304880082459], [-0.57736565407128349, -0.962078230900135, 0.71908577835422061, 0.52860269178335706]], [[-0.23053457594726123, -0.266981616166589, 0.14706660164143481, 0.87362580720893002], [-0.13519752223577863, 0.68403053579261708, 0.81667447656236591, -0.63546890273087198], [-0.6941496632135975, -0.97716830228976104, -0.76264707972380985, 0.95414050486223734]]], [[[0.81051389375652416, -0.20260132938999731, -0.15605191092737858, 0.00027979793612331605], [0.35802562761073653, -0.90719191248436526, -0.43781707733763953, -0.011859015060990297], [-0.00026721880675317777, 0.03389014993537498, -0.86094631324460957, 0.63209497268625059]], [[0.84985041427879127, 0.63435047168217107, 0.63306132652062419, 0.15142825356810752], [-0.95169088382223621, -0.53852352077320687, 0.40757365469216311, 0.72549198576227814], [-0.40725622434408404, 0.2979281144336543, 0.66349993369849813, 0.90003860925876911]]], [[[-0.18468813635358217, -0.15991413502959206, -0.37936629946621503, 0.68121986886073937], [0.62629515765129828, -0.69478206811435461, -0.055458611202817476, 0.51186245363263772], [0.66956258449967776, 0.95206087000905248, 0.84125023317845504, -0.80845902992651286]], [[-0.24067863353083996, 0.34352122106482885, 0.4111960087158808, 0.49747272331840375], [-0.71349227438459728, -0.017613099557210354, -0.66930930912049247, -0.042255829832726244], [0.69452252329293929, -0.54976031507306566, -0.76373909421180697, 0.13579444083187586]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.0564655883036)+(1.-msk_arg1)*(0.980742034114)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.056465588303555192, -0.056465588303555192, -0.056465588303555192, -0.056465588303555192], [0.72878255833928107, 0.25393968296339708, 0.7062928811864444, 0.70798565970935257], [-0.056465588303555192, -0.056465588303555192, -0.056465588303555192, -0.056465588303555192]], [[-0.056465588303555192, 0.4625407566939097, -0.013117973326893795, 0.98963345359662513], [-0.056465588303555192, 0.40832041648188855, -0.056465588303555192, -0.056465588303555192], [0.30951894134880442, 0.60735662998233275, 0.61717782998775084, 0.19184298106071851]]], [[[0.6509286048782652, -0.056465588303555192, -0.056465588303555192, 0.88956340514970922], [-0.056465588303555192, -0.056465588303555192, -0.056465588303555192, 0.87089013542946114], [-0.056465588303555192, 0.64612780085586707, -0.056465588303555192, -0.056465588303555192]], [[0.14266331123609066, 0.12940917183095224, 0.53440833399954135, 0.5541645660247565], [0.045871400215263813, 0.0028821188154224409, 0.61511017352453567, 0.23973103956187747], [-0.056465588303555192, 0.91013574526764818, -0.056465588303555192, -0.056465588303555192]]], [[[0.68813990143394066, -0.056178984894786232, -0.056465588303555192, -0.056465588303555192], [0.72461066752157022, -0.056465588303555192, -0.056465588303555192, -0.056465588303555192], [0.47800787094455832, 0.22707910699571876, 0.15163871045515576, 0.34159974670530202]], [[0.22466552974210119, 0.61881183505470827, 0.1234877438382167, -0.056465588303555192], [0.61057990868408396, 0.8011211522833841, 0.04817598302759829, -0.056465588303555192], [-0.056465588303555192, -0.056465588303555192, 0.85109648851133368, 0.76098655847489716]]]])+(1.-msk_ref)*numpy.array([[[[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]], [[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]]], [[[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]], [[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]]], [[[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]], [[0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214], [0.98074203411363214, 0.98074203411363214, 0.98074203411363214, 0.98074203411363214]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.91472696079)+(1.-msk_arg0)*(0.0113171772217)
      arg1=numpy.array([0.14371887572797215, 0.37905569309172993])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.14371887572797215, 0.37905569309172993])+(1.-msk_ref)*numpy.array([0.14371887572797215, 0.37905569309172993])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.81630127664035768, -0.15929336428417074])+(1.-msk_arg0)*numpy.array([0.68019737932345881, -0.093124132522008241])
      arg1=numpy.array([-0.18398194604258977, -0.032707152944865703])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.18398194604258977, -0.032707152944865703])+(1.-msk_ref)*numpy.array([0.68019737932345881, -0.032707152944865703])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.421223972655)+(1.-msk_arg0)*(0.282155173841)
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([-0.011276680605555534, 0.73690706237065773])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.42122397265468803, 0.73690706237065773])+(1.-msk_ref)*numpy.array([0.28215517384063693, 0.73690706237065773])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.99594804363111522, -0.0046179410576308033])+(1.-msk_arg0)*numpy.array([-0.17470296678093145, 0.28032304613987402])
      arg1=Symbol(shape=(2,))
      res=maximum(arg0,arg1)
      s1=numpy.array([-0.62540376591546432, 0.037002191774947679])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.62540376591546432, 0.037002191774947679])+(1.-msk_ref)*numpy.array([-0.17470296678093145, 0.28032304613987402])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.548457814582)+(1.-msk_arg0)*(0.436923684914)
      arg1=Data(numpy.array([0.32727602859016058, -0.69475369769015094]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.54845781458158971, 0.54845781458158971])+(1.-msk_ref)*numpy.array([0.43692368491449796, 0.43692368491449796])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.99653128761363829, -0.95662233086782145])+(1.-msk_arg0)*numpy.array([-0.2807142353642802, -0.065215727549697844])
      arg1=Data(numpy.array([-0.79471881002574074, -0.20025122987772725]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.99653128761363829, -0.20025122987772725])+(1.-msk_ref)*numpy.array([-0.2807142353642802, -0.065215727549697844])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.29012502378)+(1.-msk_arg0)*(0.41584476555)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.51903312957260583, 0.43793002132024195])+(1.-msk_arg1)*numpy.array([-0.78491146092485953, -0.011835549572744242])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.29012502378044203, 0.43793002132024195])+(1.-msk_ref)*numpy.array([0.41584476555003325, 0.41584476555003325])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.82921401986738741, 0.80635465212532376])+(1.-msk_arg0)*numpy.array([0.88940997358724871, -0.066276300855594661])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.48900030716550646, 0.89815654363021591])+(1.-msk_arg1)*numpy.array([-0.28630310397292424, 0.90329307144140514])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.48900030716550646, 0.89815654363021591])+(1.-msk_ref)*numpy.array([0.88940997358724871, 0.90329307144140514])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.738866997633)+(1.-msk_arg0)*(-0.433750876359)
      arg1=numpy.array([[-0.72541252238085341, -0.27328120068358186, -0.25768712583393794, 0.21782093726317253, 0.21367510956324054], [0.98361068092360515, 0.35248978957852106, 0.42339790030821889, -0.94369935325387355, -0.37125013968130149], [0.40578636976051574, 0.092530880501699109, 0.53629223276669968, -0.79074006834632415, -0.15815695657904016], [-0.919681786301078, 0.94894235245350478, 0.5214898825250871, -0.66979099258767993, -0.38568393578428029]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.72541252238085341, -0.27328120068358186, -0.25768712583393794, 0.21782093726317253, 0.21367510956324054], [0.98361068092360515, 0.35248978957852106, 0.42339790030821889, -0.73886699763279085, -0.37125013968130149], [0.40578636976051574, 0.092530880501699109, 0.53629223276669968, -0.73886699763279085, -0.15815695657904016], [-0.73886699763279085, 0.94894235245350478, 0.5214898825250871, -0.66979099258767993, -0.38568393578428029]])+(1.-msk_ref)*numpy.array([[-0.43375087635858489, -0.27328120068358186, -0.25768712583393794, 0.21782093726317253, 0.21367510956324054], [0.98361068092360515, 0.35248978957852106, 0.42339790030821889, -0.43375087635858489, -0.37125013968130149], [0.40578636976051574, 0.092530880501699109, 0.53629223276669968, -0.43375087635858489, -0.15815695657904016], [-0.43375087635858489, 0.94894235245350478, 0.5214898825250871, -0.43375087635858489, -0.38568393578428029]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.59602224313129293, -0.11901786006980619, 0.47394054830502341, 0.71692130476721272, 0.09678329807748387], [-0.7741902289056608, 0.45979801568792888, -0.98432358068659065, 0.43810958790745702, -0.2181908164969264], [0.68635344198849357, -0.63580672602393418, 0.56914340993295376, 0.72582816628325864, 0.28625749548694635], [-0.96371518108689647, 0.96240821379092734, 0.28345092970524055, 0.30653314343609073, -0.46516206234910218]])+(1.-msk_arg0)*numpy.array([[-0.34558335496233084, 0.12813098391466982, 0.41371341344934742, -0.31603088740499929, 0.84942205473057886], [-0.047845479330066443, -0.9688999068869768, 0.13558507922573537, -0.0021241484260634547, -0.0031519468474074941], [-0.24144821576740583, 0.75686073650684005, 0.42949597298606812, -0.3727904968363922, -0.17834896282439594], [-0.94049628031166632, 0.38349145105884608, 0.023352370386710009, -0.42358610500863669, 0.37437019047424891]])
      arg1=numpy.array([[-0.18542847783609706, 0.22742315441537819, -0.89799604135669098, -0.029715367528657666, 0.26637098196110909], [0.08744313945977189, -0.3111398797675895, 0.33915365506764372, 0.17136460418220367, 0.74584076818085498], [0.31715314583046261, 0.67587032025034177, -0.10233387490482748, 0.66906369472410776, 0.47404774150329287], [0.20628235268347828, 0.6926711444575866, -0.17570076094413967, -0.73720093961838984, 0.18922410282454538]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.59602224313129293, 0.22742315441537819, 0.47394054830502341, 0.71692130476721272, 0.26637098196110909], [0.08744313945977189, 0.45979801568792888, 0.33915365506764372, 0.43810958790745702, 0.74584076818085498], [0.68635344198849357, 0.67587032025034177, 0.56914340993295376, 0.72582816628325864, 0.47404774150329287], [0.20628235268347828, 0.96240821379092734, 0.28345092970524055, 0.30653314343609073, 0.18922410282454538]])+(1.-msk_ref)*numpy.array([[-0.18542847783609706, 0.22742315441537819, 0.41371341344934742, -0.029715367528657666, 0.84942205473057886], [0.08744313945977189, -0.3111398797675895, 0.33915365506764372, 0.17136460418220367, 0.74584076818085498], [0.31715314583046261, 0.75686073650684005, 0.42949597298606812, 0.66906369472410776, 0.47404774150329287], [0.20628235268347828, 0.6926711444575866, 0.023352370386710009, -0.42358610500863669, 0.37437019047424891]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.0281439387505)+(1.-msk_arg0)*(-0.331756574484)
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.86392705072755782, -0.87803520589916362, -0.73987416131891992, 0.026677248414576971, 0.27461624498942339], [0.47420315909449617, 0.66220885687814213, 0.68616607165864241, -0.34840517229805057, -0.71087316775864284], [-0.69119971988312878, 0.75806628880188387, 0.69576624552429767, 0.26306967559165106, -0.90168911490526771], [0.79202428549607617, 0.96511960959462306, -0.55554712848984567, -0.021805693964552786, 0.35016106526768409]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.028143938750540443, 0.028143938750540443, 0.028143938750540443, 0.028143938750540443, 0.27461624498942339], [0.47420315909449617, 0.66220885687814213, 0.68616607165864241, 0.028143938750540443, 0.028143938750540443], [0.028143938750540443, 0.75806628880188387, 0.69576624552429767, 0.26306967559165106, 0.028143938750540443], [0.79202428549607617, 0.96511960959462306, 0.028143938750540443, 0.028143938750540443, 0.35016106526768409]])+(1.-msk_ref)*numpy.array([[-0.33175657448418594, -0.33175657448418594, -0.33175657448418594, 0.026677248414576971, 0.27461624498942339], [0.47420315909449617, 0.66220885687814213, 0.68616607165864241, -0.33175657448418594, -0.33175657448418594], [-0.33175657448418594, 0.75806628880188387, 0.69576624552429767, 0.26306967559165106, -0.33175657448418594], [0.79202428549607617, 0.96511960959462306, -0.33175657448418594, -0.021805693964552786, 0.35016106526768409]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.60779500060458513, -0.3464506555852036, -0.56869298426516113, 0.8943906522457854, 0.34349658895791069], [0.30375635917509225, -0.36655647672843839, 0.50134738778366272, 0.99305292743370677, -0.12238182614717275], [-0.91748314084262628, -0.99293764610348578, 0.21927490354305723, 0.090845027591937022, -0.82501040523539104], [0.25398300113992578, -0.68366940190167425, -0.42306797649754824, -0.48344811093714624, 0.30082584106755328]])+(1.-msk_arg0)*numpy.array([[-0.81097471552309597, -0.5561948977117499, 0.52396973860632445, -0.18657887233525283, 0.82067148550971036], [0.91966684473232951, -0.40397427112195161, 0.31280756336715032, -0.46298163200004661, -0.8548444303403997], [-0.2656237220702069, 0.97273724336591183, 0.84301635209528092, -0.84547521671833192, -0.093173833145636875], [0.63378079944689159, -0.66397844107006221, 0.64652748114382752, -0.69541994329850709, -0.99428707733673471]])
      arg1=Symbol(shape=(4, 5))
      res=maximum(arg0,arg1)
      s1=numpy.array([[-0.52497496298961899, 0.43994217237748923, 0.15574273657591542, -0.11358841725516267, -0.74750297163357793], [0.012207406063615922, -0.10049000400752028, -0.014486956599730094, 0.46315585392912695, 0.35154871907412133], [-0.87064810372102497, -0.90119248118803208, 0.67140076145965355, 0.34447833018828833, -0.021957569717256264], [0.19971753076162058, 0.31184185805084086, -0.33240999781700897, -0.67371940829365307, 0.64957291215668578]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.60779500060458513, 0.43994217237748923, 0.15574273657591542, 0.8943906522457854, 0.34349658895791069], [0.30375635917509225, -0.10049000400752028, 0.50134738778366272, 0.99305292743370677, 0.35154871907412133], [-0.87064810372102497, -0.90119248118803208, 0.67140076145965355, 0.34447833018828833, -0.021957569717256264], [0.25398300113992578, 0.31184185805084086, -0.33240999781700897, -0.48344811093714624, 0.64957291215668578]])+(1.-msk_ref)*numpy.array([[-0.52497496298961899, 0.43994217237748923, 0.52396973860632445, -0.11358841725516267, 0.82067148550971036], [0.91966684473232951, -0.10049000400752028, 0.31280756336715032, 0.46315585392912695, 0.35154871907412133], [-0.2656237220702069, 0.97273724336591183, 0.84301635209528092, 0.34447833018828833, -0.021957569717256264], [0.63378079944689159, 0.31184185805084086, 0.64652748114382752, -0.67371940829365307, 0.64957291215668578]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.968391321824)+(1.-msk_arg0)*(-0.7099346671)
      arg1=Data(numpy.array([[0.29701179879797723, 0.70347702823080427, -0.79471835800766311, -0.68731376369202835, 0.90311857651371241], [0.7103320021427153, 0.054717441367131858, -0.99745778238441463, 0.04873028739048979, -0.61006905413485013], [-0.93842733155605274, 0.50853628427338804, -0.30426091903544128, 0.58360725848963546, -0.31705926426858344], [0.69399270037069161, 0.86826926031660534, -0.47801614714304264, -0.70843001046225007, 0.41841801765930908]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918], [0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918], [0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918], [0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918, 0.9683913218243918]])+(1.-msk_ref)*numpy.array([[0.29701179879797723, 0.70347702823080427, -0.7099346670995712, -0.68731376369202835, 0.90311857651371241], [0.7103320021427153, 0.054717441367131858, -0.7099346670995712, 0.04873028739048979, -0.61006905413485013], [-0.7099346670995712, 0.50853628427338804, -0.30426091903544128, 0.58360725848963546, -0.31705926426858344], [0.69399270037069161, 0.86826926031660534, -0.47801614714304264, -0.70843001046225007, 0.41841801765930908]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.17637613197723212, 0.81368899697847774, -0.70066741436084534, 0.52296323461967109, 0.66661088326806484], [-0.67485549317596027, 0.59758445609776101, 0.45040429582876418, 0.4217012917259817, 0.31279429738001019], [-0.58077835069237804, 0.29166102420102469, 0.2526616156848065, -0.99562234283945816, 0.65468685772365487], [0.19825255328895963, -0.76510565916121398, -0.50483448612496407, -0.45238980645974602, -0.53086113668897883]])+(1.-msk_arg0)*numpy.array([[-0.22161254255821672, 0.68759782267067937, 0.42154665952894632, -0.985440698773, 0.65635042945475552], [-0.98478600172882391, -0.5752667375492353, 0.65357545821292073, -0.56915537584382703, 0.81813057385681121], [-0.40228961090376347, 0.33922609331886067, 0.39610970863817063, -0.73836203087494967, 0.024995750541199069], [0.88777276882702072, 0.17246350399093902, -0.61218504009487229, 0.31396178043500922, 0.11512731297688328]])
      arg1=Data(numpy.array([[-0.13197862175472586, 0.14177033935387584, 0.014728422246227613, 0.79013786976409039, -0.18503758822783389], [-0.55156124342170476, 0.42996511802016557, -0.48720125682385684, -0.15886085390437277, 0.30830039080251459], [-0.33140838037629905, 0.11411784911565048, -0.73051867757577305, 0.254789079889016, -0.43020531322659283], [0.95938676389248312, 0.30942507644975104, -0.50054369053870018, 0.38636057061586304, -0.98930316764198034]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.17637613197723212, 0.81368899697847774, 0.014728422246227613, 0.79013786976409039, 0.66661088326806484], [-0.55156124342170476, 0.59758445609776101, 0.45040429582876418, 0.4217012917259817, 0.31279429738001019], [-0.33140838037629905, 0.29166102420102469, 0.2526616156848065, 0.254789079889016, 0.65468685772365487], [0.95938676389248312, 0.30942507644975104, -0.50054369053870018, 0.38636057061586304, -0.53086113668897883]])+(1.-msk_ref)*numpy.array([[-0.13197862175472586, 0.68759782267067937, 0.42154665952894632, 0.79013786976409039, 0.65635042945475552], [-0.55156124342170476, 0.42996511802016557, 0.65357545821292073, -0.15886085390437277, 0.81813057385681121], [-0.33140838037629905, 0.33922609331886067, 0.39610970863817063, 0.254789079889016, 0.024995750541199069], [0.95938676389248312, 0.30942507644975104, -0.50054369053870018, 0.38636057061586304, 0.11512731297688328]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.886606128808)+(1.-msk_arg0)*(0.700768026575)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.64305532973646962, -0.60638128218256138, -0.61260373793718603, 0.70274481583329318, 0.67216573607284502], [0.056357513607530052, -0.48959189340698983, 0.77594105159309468, -0.33512945900811486, -0.25598250131831435], [-0.30715706143639299, 0.74424305314625716, 0.24844080090239506, 0.6650072844121715, -0.11577214960345117], [0.22799447655880756, 0.12096394541929056, -0.61783357756637769, -0.7264377493442018, -0.47973788724609601]])+(1.-msk_arg1)*numpy.array([[0.40953857832882945, -0.99145479187770569, -0.44359836716615697, -0.30133864079518013, 0.9989647760984417], [0.23049902934564859, 0.076896464188720426, 0.89322879768241381, 0.23892993150226904, 0.19317941564711605], [0.61204111604612521, -0.071423367004127769, 0.74464158577928607, -0.26337579380661036, -0.42258481952201143], [0.43093362538307511, 0.39300817159427748, -0.86217742540170916, -0.60481644540941271, 0.29651542402341824]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.64305532973646962, -0.60638128218256138, -0.61260373793718603, 0.70274481583329318, 0.67216573607284502], [0.056357513607530052, -0.48959189340698983, 0.77594105159309468, -0.33512945900811486, -0.25598250131831435], [-0.30715706143639299, 0.74424305314625716, 0.24844080090239506, 0.6650072844121715, -0.11577214960345117], [0.22799447655880756, 0.12096394541929056, -0.61783357756637769, -0.7264377493442018, -0.47973788724609601]])+(1.-msk_ref)*numpy.array([[0.70076802657454795, 0.70076802657454795, 0.70076802657454795, 0.70076802657454795, 0.9989647760984417], [0.70076802657454795, 0.70076802657454795, 0.89322879768241381, 0.70076802657454795, 0.70076802657454795], [0.70076802657454795, 0.70076802657454795, 0.74464158577928607, 0.70076802657454795, 0.70076802657454795], [0.70076802657454795, 0.70076802657454795, 0.70076802657454795, 0.70076802657454795, 0.70076802657454795]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.30612019189686168, 0.95135935967120422, -0.075353866244783019, -0.56798070953046609, 0.42529437925846336], [-0.7968326902743923, 0.25500403452815013, -0.34667163196195383, -0.15250832494823841, -0.98608167456642448], [0.75976092051693511, -0.71954763745402794, -0.85716569577642621, 0.5049444481642722, -0.5589267595072247], [-0.44267531322074372, 0.92213529484491441, 0.55119130907482328, -0.68170498176248873, 0.90555976103882951]])+(1.-msk_arg0)*numpy.array([[0.59519177758892727, 0.25976913209471464, -0.27191012388999081, 0.8616827024136533, 0.48496769325691225], [0.42123832877010292, 0.97608441953561509, -0.033557590270044457, -0.80335344330470848, 0.2498103208196969], [-0.28440764569673171, 0.48513587003581704, -0.26670168419543239, -0.023258098131885507, -0.28285209082596752], [-0.77293657173661967, -0.87258589181928414, 0.16777292094842156, 0.98554552171112531, -0.58041025857394812]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.53980428075498432, -0.8620985675765529, 0.10318753714451057, -0.34594469649839743, -0.26253653354933437], [0.5568000522030927, 0.95471851543455877, 0.76494137771653703, -0.89909752018530487, 0.065218938426729212], [-0.94889860454446717, -0.27132057498615136, 0.25050587653834056, -0.022689614047457729, -0.83525701623221682], [-0.72343888167391657, -0.94000414827540379, -0.12680345659866288, -0.3726106505034219, 0.12322122882315356]])+(1.-msk_arg1)*numpy.array([[0.42724124999901059, -0.22595557355760132, -0.80352821333452384, -0.92811140531522573, 0.63931956037221016], [-0.6877750864294403, -0.0056332903472737783, 0.21419300884938419, -0.69188811767615754, -0.9138285400178312], [-0.54256535713927923, 0.73733527645626329, -0.098012441663765548, 0.80054498308143573, 0.11936239919805236], [0.46067331791290411, -0.85622216822493269, -0.85577286610605174, -0.64462140755589581, -0.91167565524099992]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.53980428075498432, 0.95135935967120422, 0.10318753714451057, -0.34594469649839743, 0.42529437925846336], [0.5568000522030927, 0.95471851543455877, 0.76494137771653703, -0.15250832494823841, 0.065218938426729212], [0.75976092051693511, -0.27132057498615136, 0.25050587653834056, 0.5049444481642722, -0.5589267595072247], [-0.44267531322074372, 0.92213529484491441, 0.55119130907482328, -0.3726106505034219, 0.90555976103882951]])+(1.-msk_ref)*numpy.array([[0.59519177758892727, 0.25976913209471464, -0.27191012388999081, 0.8616827024136533, 0.63931956037221016], [0.42123832877010292, 0.97608441953561509, 0.21419300884938419, -0.69188811767615754, 0.2498103208196969], [-0.28440764569673171, 0.73733527645626329, -0.098012441663765548, 0.80054498308143573, 0.11936239919805236], [0.46067331791290411, -0.85622216822493269, 0.16777292094842156, 0.98554552171112531, -0.58041025857394812]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.718530103778)+(1.-msk_arg0)*(0.793689594671)
      arg1=numpy.array([[[-0.0024451589941487661, 0.37962439196941866], [0.63153612360410327, -0.47899847783370531]], [[-0.90520992078403451, -0.95918575086250835], [-0.28115016997012687, -0.51161918860583588]], [[-0.61627320654983975, -0.54619724559098004], [0.47194607858515991, 0.70563469194352724]], [[0.77071388199460955, 0.071543453256624145], [0.320642130841571, -0.23511721513687456]], [[0.565685956805559, 0.39919171271086351], [0.39452154791888394, -0.8334230961358382]], [[0.71553572930335529, -0.72729092650994009], [-0.57115890968262772, -0.19116184636450217]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.71853010377798143, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]], [[0.71853010377798143, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]], [[0.71853010377798143, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]], [[0.77071388199460955, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]], [[0.71853010377798143, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]], [[0.71853010377798143, 0.71853010377798143], [0.71853010377798143, 0.71853010377798143]]])+(1.-msk_ref)*numpy.array([[[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]], [[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]], [[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]], [[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]], [[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]], [[0.79368959467057976, 0.79368959467057976], [0.79368959467057976, 0.79368959467057976]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.23289574950911729, -0.39979931374519451], [0.15897379012575996, -0.50180377622002981]], [[-0.82406624777113491, 0.68978178267237378], [-0.49214098282971119, 0.91663604312644487]], [[0.51961880703793417, 0.33271371791298732], [-0.029330613495592406, -0.33267726604768266]], [[-0.44169585496973585, -0.19253039780834591], [-0.86233079085192776, 0.22210644263950541]], [[0.58157093465620235, 0.4651068757348702], [-0.80136699806451972, 0.4791116027753044]], [[-0.34358341468322973, 0.97285610062438943], [0.54582634844290179, -0.3844491351792334]]])+(1.-msk_arg0)*numpy.array([[[0.43243149290797156, 0.42118817946076303], [-0.18453847901320564, -0.66339724486601703]], [[0.44682412774501268, 0.06259230513593339], [-0.85030268678048393, 0.090377810940876158]], [[0.56249810268779799, 0.23332735750156597], [-0.6347229076098917, 0.17946284083046082]], [[-0.44724133701655133, -0.98766139526109908], [-0.19160273520718185, 0.37747814250070011]], [[-0.376248630682541, 0.97158262633526449], [0.2983965823325585, -0.33246515396523035]], [[0.79962535603610885, 0.42934691003447378], [0.76478449568109652, 0.91637336177037865]]])
      arg1=numpy.array([[[0.088752584503619891, -0.39447101553511321], [-0.22430239048448097, -0.72298239865701297]], [[-0.39702422374427049, -0.82719309363155413], [-0.88688183257968389, 0.23290068315861734]], [[0.19723100073754174, 0.17510622396992837], [-0.75586102931009358, 0.086122900287766813]], [[0.2038452039966705, 0.69127428790485257], [-0.35550974805881208, 0.95332996260066616]], [[0.86443048779177034, -0.38168990310894912], [0.38182538983725678, 0.73503462747021775]], [[-0.70625227491884734, -0.12131105742004333], [0.85069828133096981, -0.6433556681341337]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.088752584503619891, -0.39447101553511321], [0.15897379012575996, -0.50180377622002981]], [[-0.39702422374427049, 0.68978178267237378], [-0.49214098282971119, 0.91663604312644487]], [[0.51961880703793417, 0.33271371791298732], [-0.029330613495592406, 0.086122900287766813]], [[0.2038452039966705, 0.69127428790485257], [-0.35550974805881208, 0.95332996260066616]], [[0.86443048779177034, 0.4651068757348702], [0.38182538983725678, 0.73503462747021775]], [[-0.34358341468322973, 0.97285610062438943], [0.85069828133096981, -0.3844491351792334]]])+(1.-msk_ref)*numpy.array([[[0.43243149290797156, 0.42118817946076303], [-0.18453847901320564, -0.66339724486601703]], [[0.44682412774501268, 0.06259230513593339], [-0.85030268678048393, 0.23290068315861734]], [[0.56249810268779799, 0.23332735750156597], [-0.6347229076098917, 0.17946284083046082]], [[0.2038452039966705, 0.69127428790485257], [-0.19160273520718185, 0.95332996260066616]], [[0.86443048779177034, 0.97158262633526449], [0.38182538983725678, 0.73503462747021775]], [[0.79962535603610885, 0.42934691003447378], [0.85069828133096981, 0.91637336177037865]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.203757777432)+(1.-msk_arg0)*(0.0568880448082)
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[0.11497840867252918, -0.41553218391138258], [0.19359208872918665, -0.57277895936895629]], [[-0.22369972430343488, -0.98556145914479076], [-0.83554300333867504, 0.94244692999538016]], [[-0.76304378195713163, -0.059870648016089323], [0.93763163505012725, -0.99567896077857521]], [[0.75792904619750101, -0.60099011760915588], [-0.27282249216321008, 0.38868765288165363]], [[-0.56832397377108212, 0.8822442477483452], [-0.82115633602517257, 0.95299533317410079]], [[0.053060013277372198, -0.85972826581368156], [-0.80973562714846992, 0.73621330170361854]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.2037577774324375, 0.2037577774324375], [0.2037577774324375, 0.2037577774324375]], [[0.2037577774324375, 0.2037577774324375], [0.2037577774324375, 0.94244692999538016]], [[0.2037577774324375, 0.2037577774324375], [0.93763163505012725, 0.2037577774324375]], [[0.75792904619750101, 0.2037577774324375], [0.2037577774324375, 0.38868765288165363]], [[0.2037577774324375, 0.8822442477483452], [0.2037577774324375, 0.95299533317410079]], [[0.2037577774324375, 0.2037577774324375], [0.2037577774324375, 0.73621330170361854]]])+(1.-msk_ref)*numpy.array([[[0.11497840867252918, 0.056888044808183436], [0.19359208872918665, 0.056888044808183436]], [[0.056888044808183436, 0.056888044808183436], [0.056888044808183436, 0.94244692999538016]], [[0.056888044808183436, 0.056888044808183436], [0.93763163505012725, 0.056888044808183436]], [[0.75792904619750101, 0.056888044808183436], [0.056888044808183436, 0.38868765288165363]], [[0.056888044808183436, 0.8822442477483452], [0.056888044808183436, 0.95299533317410079]], [[0.056888044808183436, 0.056888044808183436], [0.056888044808183436, 0.73621330170361854]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.55545801472378975, 0.25527134442036647], [0.78910793500341025, -0.83985207456372057]], [[-0.90844432113247153, 0.21101734751492707], [0.56638940486904477, 0.49880272085421962]], [[-0.86313339067610917, -0.51652583560176568], [-0.9387503454813857, -0.55646359178576676]], [[-0.8829910672206398, 0.19789253550434793], [0.66076216251137176, -0.023729280056204782]], [[0.56173462177039446, 0.97220924571608536], [-0.54263105085032759, -0.27654261178757178]], [[0.50986267382579631, 0.88408919479913983], [-0.28277404053679889, -0.81076748480858418]]])+(1.-msk_arg0)*numpy.array([[[-0.56483192870131949, 0.48023538735888027], [0.74283336654455279, -0.9471486548877952]], [[-0.5133146462061744, 0.17554431111183777], [0.37994735470614027, -0.8687969527148276]], [[-0.31033321804098368, -0.28745609924459536], [-0.51959012239153735, -0.95292665330752069]], [[0.31325591384993179, -0.49336426195649152], [0.58796354735813416, 0.56742685056716602]], [[0.21142278321812791, -0.22729558697593899], [-0.87760180236765017, 0.19941157972526335]], [[-0.3967781123459253, -0.5732483748070536], [0.61075339342036838, -0.73582455298546701]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[-0.17924178832189619, -0.34205186644800545], [0.97121838264005578, 0.49642715620483902]], [[-0.38213977777433761, 0.86915739248574986], [-0.86895690834782036, -0.72958904407877356]], [[0.41408705627419584, 0.60785311575600476], [-0.94998066802533621, -0.82834567483076271]], [[-0.68102072384223344, 0.31059484376332147], [-0.99172450315559746, 0.11518485233335429]], [[0.24874731515194148, 0.27409432519442345], [0.054519844494301362, 0.0098498984905697196]], [[0.35235577698763976, -0.71374526581230846], [0.80535209894545789, 0.2815026219951855]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.17924178832189619, 0.25527134442036647], [0.97121838264005578, 0.49642715620483902]], [[-0.38213977777433761, 0.86915739248574986], [0.56638940486904477, 0.49880272085421962]], [[0.41408705627419584, 0.60785311575600476], [-0.9387503454813857, -0.55646359178576676]], [[-0.68102072384223344, 0.31059484376332147], [0.66076216251137176, 0.11518485233335429]], [[0.56173462177039446, 0.97220924571608536], [0.054519844494301362, 0.0098498984905697196]], [[0.50986267382579631, 0.88408919479913983], [0.80535209894545789, 0.2815026219951855]]])+(1.-msk_ref)*numpy.array([[[-0.17924178832189619, 0.48023538735888027], [0.97121838264005578, 0.49642715620483902]], [[-0.38213977777433761, 0.86915739248574986], [0.37994735470614027, -0.72958904407877356]], [[0.41408705627419584, 0.60785311575600476], [-0.51959012239153735, -0.82834567483076271]], [[0.31325591384993179, 0.31059484376332147], [0.58796354735813416, 0.56742685056716602]], [[0.24874731515194148, 0.27409432519442345], [0.054519844494301362, 0.19941157972526335]], [[0.35235577698763976, -0.5732483748070536], [0.80535209894545789, 0.2815026219951855]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.989176914801)+(1.-msk_arg0)*(-0.989986362534)
      arg1=Data(numpy.array([[[-0.49388719096314349, -0.79623559567830759], [-0.83658541823517818, 0.42072406054838662]], [[-0.48192235106585324, -0.14003155480478036], [0.47407230666875866, 0.95145307875088792]], [[0.81276956345717855, 0.54577402893520022], [-0.90122971458194412, -0.58274583646026556]], [[0.85522050991554255, 0.75964838308411031], [0.5778018835429104, -0.76867690348681994]], [[0.15025246865549424, 0.055620041059527336], [-0.54156080230040549, -0.63085875592520013]], [[-0.335005199709838, 0.74217379824147822], [-0.61569565476952914, -0.23786048005871696]]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.49388719096314349, -0.79623559567830759], [-0.83658541823517818, 0.42072406054838662]], [[-0.48192235106585324, -0.14003155480478036], [0.47407230666875866, 0.95145307875088792]], [[0.81276956345717855, 0.54577402893520022], [-0.90122971458194412, -0.58274583646026556]], [[0.85522050991554255, 0.75964838308411031], [0.5778018835429104, -0.76867690348681994]], [[0.15025246865549424, 0.055620041059527336], [-0.54156080230040549, -0.63085875592520013]], [[-0.335005199709838, 0.74217379824147822], [-0.61569565476952914, -0.23786048005871696]]])+(1.-msk_ref)*numpy.array([[[-0.49388719096314349, -0.79623559567830759], [-0.83658541823517818, 0.42072406054838662]], [[-0.48192235106585324, -0.14003155480478036], [0.47407230666875866, 0.95145307875088792]], [[0.81276956345717855, 0.54577402893520022], [-0.90122971458194412, -0.58274583646026556]], [[0.85522050991554255, 0.75964838308411031], [0.5778018835429104, -0.76867690348681994]], [[0.15025246865549424, 0.055620041059527336], [-0.54156080230040549, -0.63085875592520013]], [[-0.335005199709838, 0.74217379824147822], [-0.61569565476952914, -0.23786048005871696]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.9515053377069429, 0.47807174372932359], [-0.75564278236571569, -0.28735791360983409]], [[0.38398978657266181, 0.81107114048034323], [-0.83362651172912039, -0.7692532587204759]], [[-0.85019914335870261, -0.74233731831744643], [0.094460048711824784, 0.88107054995270251]], [[-0.16449632437481898, -0.44322035699164086], [-0.93906847021333362, 0.63056982121743488]], [[-0.15478314379855163, -0.51272224940380906], [-0.90580658840006745, -0.088880576181244342]], [[-0.57864595898321425, -0.98961581993571057], [0.69127717551907675, 0.12303841555427852]]])+(1.-msk_arg0)*numpy.array([[[0.54991250710512474, 0.75252556510707502], [0.4524255659499381, 0.034482370597328282]], [[0.048275902648243552, -0.42803062132180703], [-0.42953608825084499, -0.52649686005029883]], [[0.98434263591531024, 0.045527551414420664], [0.25896487569318261, -0.82768236289669472]], [[0.82974872968024926, -0.85936448047482017], [-0.48527758532198395, -0.76833200375575417]], [[0.89349009195765672, -0.19880662446189135], [0.2966368207713177, -0.18896189651548845]], [[0.1803928604184375, -0.82241408231310964], [0.22841835028060076, 0.39317537222153964]]])
      arg1=Data(numpy.array([[[0.68617253474215634, 0.15087223754864798], [-0.10726012723616618, -0.92665009980729041]], [[-0.041620269299473822, 0.015752588101144216], [-0.14427171039873188, -0.42796473013430236]], [[0.11932262149106454, 0.64034918365669058], [-0.48270730652849725, 0.053510918018123022]], [[0.33635000832920325, -0.86051716060430916], [-0.70908675709144253, -0.11679411206479129]], [[-0.071763389013920875, 0.16572479797356277], [0.77137834529942251, 0.26120416887974818]], [[0.61310060082682982, 0.015920493203621211], [-0.46407909287571503, -0.60491137260032635]]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.9515053377069429, 0.47807174372932359], [-0.10726012723616618, -0.28735791360983409]], [[0.38398978657266181, 0.81107114048034323], [-0.14427171039873188, -0.42796473013430236]], [[0.11932262149106454, 0.64034918365669058], [0.094460048711824784, 0.88107054995270251]], [[0.33635000832920325, -0.44322035699164086], [-0.70908675709144253, 0.63056982121743488]], [[-0.071763389013920875, 0.16572479797356277], [0.77137834529942251, 0.26120416887974818]], [[0.61310060082682982, 0.015920493203621211], [0.69127717551907675, 0.12303841555427852]]])+(1.-msk_ref)*numpy.array([[[0.68617253474215634, 0.75252556510707502], [0.4524255659499381, 0.034482370597328282]], [[0.048275902648243552, 0.015752588101144216], [-0.14427171039873188, -0.42796473013430236]], [[0.98434263591531024, 0.64034918365669058], [0.25896487569318261, 0.053510918018123022]], [[0.82974872968024926, -0.85936448047482017], [-0.48527758532198395, -0.11679411206479129]], [[0.89349009195765672, 0.16572479797356277], [0.77137834529942251, 0.26120416887974818]], [[0.61310060082682982, 0.015920493203621211], [0.22841835028060076, 0.39317537222153964]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.293076478313)+(1.-msk_arg0)*(0.415695253254)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.42476097366268384, -0.99781990217086425], [-0.42797078478453088, 0.56788496043163539]], [[0.88356209487076054, -0.21592478018874428], [0.59903633153983038, 0.99453929697875876]], [[-0.4031451864299096, -0.78793554032137436], [0.99404205252643041, -0.46900675116702062]], [[-0.35961855333700887, -0.11711382975673645], [0.18349118086242, -0.035040226820079123]], [[-0.67627961022952809, 0.4515888728838664], [0.93565887741312981, 0.31169707414443426]], [[0.96128404783088417, -0.8324773748092742], [0.23613968220346138, -0.11039468454161838]]])+(1.-msk_arg1)*numpy.array([[[-0.88023059794213743, 0.927763549181158], [-0.83970906479816909, 0.79761854094125706]], [[-0.40619318341717858, 0.69914276566700284], [0.44376855556648342, -0.91825223070551631]], [[0.39649333256863528, -0.092930299765284108], [-0.072283891404024025, 0.13436467337237956]], [[0.19751892281774874, -0.82813284163388401], [0.6214385672795204, 0.47316992672533109]], [[-0.97549578931282155, -0.50190475490773157], [0.6647972184776747, -0.63181839238397086]], [[-0.16594757687004513, -0.34293676516292471], [0.14039762775702891, -0.044223647723118154]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.29307647831298689, -0.29307647831298689], [-0.29307647831298689, 0.56788496043163539]], [[0.88356209487076054, -0.21592478018874428], [0.59903633153983038, 0.99453929697875876]], [[-0.29307647831298689, -0.29307647831298689], [0.99404205252643041, -0.29307647831298689]], [[-0.29307647831298689, -0.11711382975673645], [0.18349118086242, -0.035040226820079123]], [[-0.29307647831298689, 0.4515888728838664], [0.93565887741312981, 0.31169707414443426]], [[0.96128404783088417, -0.29307647831298689], [0.23613968220346138, -0.11039468454161838]]])+(1.-msk_ref)*numpy.array([[[0.41569525325398415, 0.927763549181158], [0.41569525325398415, 0.79761854094125706]], [[0.41569525325398415, 0.69914276566700284], [0.44376855556648342, 0.41569525325398415]], [[0.41569525325398415, 0.41569525325398415], [0.41569525325398415, 0.41569525325398415]], [[0.41569525325398415, 0.41569525325398415], [0.6214385672795204, 0.47316992672533109]], [[0.41569525325398415, 0.41569525325398415], [0.6647972184776747, 0.41569525325398415]], [[0.41569525325398415, 0.41569525325398415], [0.41569525325398415, 0.41569525325398415]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.36130298057679111, -0.72982256164520121], [-0.98427828362689751, -0.71392419224338233]], [[0.0028648066420755569, 0.40593075742981033], [-0.39308624937033798, 0.41173487980367285]], [[0.040429985666669666, 0.9956986678939499], [-0.3766616780070684, -0.14800180944610708]], [[-0.34619453852173332, -0.17504597230775909], [0.88567660560918915, -0.74291128943136053]], [[0.03679824663741571, -0.10882953030274356], [0.45259132917204359, 0.6036332348120923]], [[-0.74384731565805584, -0.98281302250512592], [-0.97516438572854058, -0.43418945070204695]]])+(1.-msk_arg0)*numpy.array([[[-0.56133979475379858, 0.5901398064050758], [0.26863333535059408, 0.21622822005148534]], [[-0.014772805904820441, -0.61818138105374354], [0.3201607812914451, -0.16980551408521727]], [[0.65927338717041262, 0.82573021065854024], [-0.9382351948264771, 0.46463417799389251]], [[0.55336804661738448, -0.16893054714418709], [-0.45486072472527739, -0.36714074972175692]], [[-0.45180298993654455, -0.029952989308233535], [0.71588478817998147, -0.59014771697946466]], [[0.86344487271841497, -0.66727303798019189], [-0.36584595815879517, 0.33547655315459779]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.64224348205869819, 0.049037997241984765], [-0.98116819130903643, 0.94181846056675522]], [[-0.16946566364672533, 0.3705330855361475], [-0.18152511908793034, 0.018261630850941124]], [[0.029393061582448254, -0.32804788747238334], [-0.08265934486633153, -0.62626548403291404]], [[-0.81665972655100605, 0.18713502632711143], [0.069041181401869434, -0.012080898549561292]], [[-0.49967653078908558, -0.88780398744026323], [-0.96151630421477496, 0.17188541642458732]], [[0.8559946449708169, 0.78065817299993023], [-0.37293727862758685, -0.60616899821341486]]])+(1.-msk_arg1)*numpy.array([[[-0.49032583607212854, -0.8136795614509551], [0.89577823564665859, 0.79840425740211107]], [[0.41626566605101467, -0.24782501100599741], [-0.59737466766066505, 0.053224551388946217]], [[-0.49418240106620037, -0.014671319634904867], [0.43041860621440775, -0.6139043809851934]], [[-0.70080342153969033, 0.3930154267575332], [-0.31944992543072104, -0.82077676018890178]], [[-0.25102242325231439, 0.045813485089310158], [0.89982702067370801, 0.27597754182222012]], [[-0.13163387797868897, 0.50410986652020684], [0.4718132434424589, 0.55301018967858151]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.64224348205869819, 0.049037997241984765], [-0.98116819130903643, 0.94181846056675522]], [[0.0028648066420755569, 0.40593075742981033], [-0.18152511908793034, 0.41173487980367285]], [[0.040429985666669666, 0.9956986678939499], [-0.08265934486633153, -0.14800180944610708]], [[-0.34619453852173332, 0.18713502632711143], [0.88567660560918915, -0.012080898549561292]], [[0.03679824663741571, -0.10882953030274356], [0.45259132917204359, 0.6036332348120923]], [[0.8559946449708169, 0.78065817299993023], [-0.37293727862758685, -0.43418945070204695]]])+(1.-msk_ref)*numpy.array([[[-0.49032583607212854, 0.5901398064050758], [0.89577823564665859, 0.79840425740211107]], [[0.41626566605101467, -0.24782501100599741], [0.3201607812914451, 0.053224551388946217]], [[0.65927338717041262, 0.82573021065854024], [0.43041860621440775, 0.46463417799389251]], [[0.55336804661738448, 0.3930154267575332], [-0.31944992543072104, -0.36714074972175692]], [[-0.25102242325231439, 0.045813485089310158], [0.89982702067370801, 0.27597754182222012]], [[0.86344487271841497, 0.50410986652020684], [0.4718132434424589, 0.55301018967858151]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.93122841147)+(1.-msk_arg0)*(0.690664190259)
      arg1=numpy.array([[[[-0.14129653506270778, -0.057224607125864191, 0.9211833065741597, 0.6473574724937774], [0.79342813565602577, 0.58188727876918134, 0.54967922043088868, -0.86099657295389598], [0.72149005255943988, 0.49970954592644179, 0.20671785867262726, -0.70256925121758917]], [[0.30291193430769958, 0.38342882633653308, -0.8482847466432486, 0.80705958071323347], [-0.29500788390136612, -0.45682358930611389, 0.42671080439327636, 0.82874473369979196], [0.33378375281988304, 0.57146717744436648, 0.71657597252409011, -0.35470560156724984]]], [[[0.64905652148048087, 0.066757786624187521, -0.033149691345921273, -0.09095042959743993], [0.23199263733313469, 0.37983826394739206, 0.64154881030545563, -0.51234130039277082], [0.69621394755055732, -0.82794221977727456, -0.39538559469645329, -0.1618575046203059]], [[-0.29009671679704652, -0.9022342552920628, 0.81807598072470888, -0.13568667169335469], [-0.5503101579327343, 0.5357357653203636, -0.15952645784198238, 0.70807762475485791], [-0.64948567730623208, 0.75852757462222153, -0.3563619745115234, -0.069412219258663477]]], [[[0.80762645906831954, 0.34639538845074114, -0.36919251697493238, 0.32457680007810064], [-0.9161626068958062, 0.90565137289941, -0.68875897981177348, -0.23023391707486063], [0.12566730654437941, 0.28534019901624386, 0.28644025853864852, -0.36938545264538325]], [[-0.59831408015871768, -0.17176820348331701, 0.54554002636113474, 0.58717693841655438], [-0.67460631161913742, -0.53038034535930567, 0.41207107058237002, 0.86424360099095976], [0.84577384964306712, 0.76585183685560199, -0.67847345259712255, -0.46920326976603643]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.14129653506270778, -0.057224607125864191, 0.9211833065741597, 0.6473574724937774], [0.79342813565602577, 0.58188727876918134, 0.54967922043088868, -0.86099657295389598], [0.72149005255943988, 0.49970954592644179, 0.20671785867262726, -0.70256925121758917]], [[0.30291193430769958, 0.38342882633653308, -0.8482847466432486, 0.80705958071323347], [-0.29500788390136612, -0.45682358930611389, 0.42671080439327636, 0.82874473369979196], [0.33378375281988304, 0.57146717744436648, 0.71657597252409011, -0.35470560156724984]]], [[[0.64905652148048087, 0.066757786624187521, -0.033149691345921273, -0.09095042959743993], [0.23199263733313469, 0.37983826394739206, 0.64154881030545563, -0.51234130039277082], [0.69621394755055732, -0.82794221977727456, -0.39538559469645329, -0.1618575046203059]], [[-0.29009671679704652, -0.9022342552920628, 0.81807598072470888, -0.13568667169335469], [-0.5503101579327343, 0.5357357653203636, -0.15952645784198238, 0.70807762475485791], [-0.64948567730623208, 0.75852757462222153, -0.3563619745115234, -0.069412219258663477]]], [[[0.80762645906831954, 0.34639538845074114, -0.36919251697493238, 0.32457680007810064], [-0.9161626068958062, 0.90565137289941, -0.68875897981177348, -0.23023391707486063], [0.12566730654437941, 0.28534019901624386, 0.28644025853864852, -0.36938545264538325]], [[-0.59831408015871768, -0.17176820348331701, 0.54554002636113474, 0.58717693841655438], [-0.67460631161913742, -0.53038034535930567, 0.41207107058237002, 0.86424360099095976], [0.84577384964306712, 0.76585183685560199, -0.67847345259712255, -0.46920326976603643]]]])+(1.-msk_ref)*numpy.array([[[[0.69066419025918635, 0.69066419025918635, 0.9211833065741597, 0.69066419025918635], [0.79342813565602577, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635], [0.72149005255943988, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635]], [[0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.80705958071323347], [0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.82874473369979196], [0.69066419025918635, 0.69066419025918635, 0.71657597252409011, 0.69066419025918635]]], [[[0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635], [0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635], [0.69621394755055732, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635]], [[0.69066419025918635, 0.69066419025918635, 0.81807598072470888, 0.69066419025918635], [0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.70807762475485791], [0.69066419025918635, 0.75852757462222153, 0.69066419025918635, 0.69066419025918635]]], [[[0.80762645906831954, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635], [0.69066419025918635, 0.90565137289941, 0.69066419025918635, 0.69066419025918635], [0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635]], [[0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.69066419025918635], [0.69066419025918635, 0.69066419025918635, 0.69066419025918635, 0.86424360099095976], [0.84577384964306712, 0.76585183685560199, 0.69066419025918635, 0.69066419025918635]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.44772485229861014, 0.33612133273895872, 0.85463791292335456, 0.041150956342052902], [0.86196525725452178, -0.39230412793663172, -0.55689630426568115, 0.24500138369530022], [0.250440568048236, 0.56208355979267344, -0.83931525120237538, -0.89962845216257148]], [[0.93199744061363954, 0.036161545737686263, 0.17600123827465097, -0.15644558799055064], [0.51769117284229615, 0.33689820823374972, 0.060403722261924919, -0.67401337318561683], [-0.5870077618091023, 0.35549507870374053, -0.91444103477854144, 0.23111635010592591]]], [[[0.30427728778811924, 0.11748924311521702, 0.10481933285039213, -0.33502229131967587], [0.26515755662112572, 0.099878676762299445, 0.69816433138192102, -0.98773857616342631], [-0.092028032496626722, -0.70240638229622876, 0.71611949446200152, 0.25170319313794276]], [[0.037527545049968802, 0.49361154856116607, 0.62722682321215739, -0.87892648689079045], [-0.95341541684169084, -0.010773509314439877, -0.58116207689853905, -0.58140329890782971], [0.27136771093638901, 0.082457582086216341, -0.21110397496607991, -0.29970993820392677]]], [[[-0.22265001693402553, 0.94823954643701369, -0.7266026099756171, -0.32353607416408758], [0.53071688333832867, -0.28644943651030919, -0.7169950181827689, 0.58843920390628113], [-0.68278304053745997, -0.73191962597309357, 0.83951460129804634, -0.28603981175880899]], [[-0.62530509820592028, 0.19163503831394335, 0.71630459577755268, -0.48358514127015995], [0.35290552000196262, 0.42122549901315964, 0.48162879235316169, 0.92685704379160105], [0.21890865144110738, -0.69229869309205228, -0.51436410927581733, 0.77525889246393453]]]])+(1.-msk_arg0)*numpy.array([[[[-0.47994795124062661, 0.74856862291517623, 0.5821395617608589, -0.31324499648561299], [0.13609834355782402, 0.8651786428977688, -0.22241596237712269, 0.59595179741831994], [0.52442180802467986, -0.67755617307904803, 0.041047440148369585, -0.63982185998775787]], [[0.89633393878486145, 0.60528407122761974, 0.087008352545255407, 0.61818598246202194], [-0.82215369326968912, -0.17257825389866843, -0.97707914343112701, 0.10612032693135687], [0.54832826138702373, 0.14418882636852293, -0.59559456960762192, -0.20767572929860978]]], [[[-0.80347236426212421, -0.91815707719608963, 0.91255277503919396, 0.045503228874384849], [0.75467835024620356, 0.25562178283718717, 0.24903527543586712, 0.56502967349932165], [-0.1335888822357898, -0.68279551726839105, 0.43491768196300673, 0.46168510826508036]], [[0.4356763541452715, 0.98085251814408414, -0.62755096901235152, -0.78345634103039363], [-0.60500930133859709, 0.56402299324965721, 0.59561490314477927, 0.82072216033311784], [0.95001617573414032, 0.20468480993367022, 0.30565824220619442, -0.070107462971154888]]], [[[0.23393156511221869, -0.37563984619602975, 0.36378548060529137, 0.43451806931870163], [-0.1642002012922239, -0.54728511822572523, 0.36670201211269826, 0.18918610945368441], [-0.33406116215906256, -0.25576174770698823, -0.02020947518693661, 0.87728898366655694]], [[-0.59338050831235423, -0.14657407934408662, 0.69570242811712601, 0.36366022997929148], [0.94456195322361602, -0.86212596260553642, 0.5145071925072755, -0.61650036986407497], [-0.47495276384029639, -0.9072767739797154, 0.60749045264710033, 0.80404847388366707]]]])
      arg1=numpy.array([[[[0.12235724577495133, 0.030550102850180449, 0.82361823715408589, -0.47577063319692625], [0.24762255252735432, -0.18117195942957376, 0.78421570529513662, -0.33120166549504804], [-0.31661878366910501, 0.050408417654134619, 0.93543377163183949, 0.7138338863269178]], [[0.60609599680976656, -0.18646992543326979, 0.032245229491007832, -0.16135181789862751], [-0.2197064057048792, 0.89130327750849525, 0.87528272382646333, -0.4054146695562737], [0.34864798718456469, 0.6865288809229968, -0.27667529617816933, 0.0036869563701744656]]], [[[-0.95770828799863805, 0.096015202531622768, -0.60118964454981816, -0.77935740289018307], [0.72274943044452389, 0.0073194320184981621, -0.059288992988969946, 0.79201968761767816], [-0.12866704041475518, 0.98722308476263909, -0.4127435552309402, 0.90334700524787515]], [[0.46963073206058481, 0.79351354921673289, 0.044478338561507069, -0.86621149843666823], [-0.86768077600190563, 0.87895872758421212, -0.68813225933821531, 0.051058982021920718], [-0.6400145105837145, -0.6791315301293348, 0.35602453634758469, -0.40925802590739879]]], [[[0.6864899883695792, 0.44223216849695901, -0.89165346027070291, 0.40133713829912065], [0.78750618231061065, -0.013479137470628899, 0.6577875955603778, -0.87184650021107624], [-0.69474956775331775, 0.93518928901459675, 0.90966513308817243, 0.77851041589827696]], [[-0.50347347741991966, 0.98138043351913606, -0.88658887741951498, 0.89654400325398464], [-0.78549232237218436, -0.34691444129238169, 0.30748561536051167, 0.62508666703593296], [0.44494445035343788, -0.87171691786051886, 0.31674335839547196, 0.8800806202145921]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.12235724577495133, 0.33612133273895872, 0.85463791292335456, 0.041150956342052902], [0.86196525725452178, -0.18117195942957376, 0.78421570529513662, 0.24500138369530022], [0.250440568048236, 0.56208355979267344, 0.93543377163183949, 0.7138338863269178]], [[0.93199744061363954, 0.036161545737686263, 0.17600123827465097, -0.15644558799055064], [0.51769117284229615, 0.89130327750849525, 0.87528272382646333, -0.4054146695562737], [0.34864798718456469, 0.6865288809229968, -0.27667529617816933, 0.23111635010592591]]], [[[0.30427728778811924, 0.11748924311521702, 0.10481933285039213, -0.33502229131967587], [0.72274943044452389, 0.099878676762299445, 0.69816433138192102, 0.79201968761767816], [-0.092028032496626722, 0.98722308476263909, 0.71611949446200152, 0.90334700524787515]], [[0.46963073206058481, 0.79351354921673289, 0.62722682321215739, -0.86621149843666823], [-0.86768077600190563, 0.87895872758421212, -0.58116207689853905, 0.051058982021920718], [0.27136771093638901, 0.082457582086216341, 0.35602453634758469, -0.29970993820392677]]], [[[0.6864899883695792, 0.94823954643701369, -0.7266026099756171, 0.40133713829912065], [0.78750618231061065, -0.013479137470628899, 0.6577875955603778, 0.58843920390628113], [-0.68278304053745997, 0.93518928901459675, 0.90966513308817243, 0.77851041589827696]], [[-0.50347347741991966, 0.98138043351913606, 0.71630459577755268, 0.89654400325398464], [0.35290552000196262, 0.42122549901315964, 0.48162879235316169, 0.92685704379160105], [0.44494445035343788, -0.69229869309205228, 0.31674335839547196, 0.8800806202145921]]]])+(1.-msk_ref)*numpy.array([[[[0.12235724577495133, 0.74856862291517623, 0.82361823715408589, -0.31324499648561299], [0.24762255252735432, 0.8651786428977688, 0.78421570529513662, 0.59595179741831994], [0.52442180802467986, 0.050408417654134619, 0.93543377163183949, 0.7138338863269178]], [[0.89633393878486145, 0.60528407122761974, 0.087008352545255407, 0.61818598246202194], [-0.2197064057048792, 0.89130327750849525, 0.87528272382646333, 0.10612032693135687], [0.54832826138702373, 0.6865288809229968, -0.27667529617816933, 0.0036869563701744656]]], [[[-0.80347236426212421, 0.096015202531622768, 0.91255277503919396, 0.045503228874384849], [0.75467835024620356, 0.25562178283718717, 0.24903527543586712, 0.79201968761767816], [-0.12866704041475518, 0.98722308476263909, 0.43491768196300673, 0.90334700524787515]], [[0.46963073206058481, 0.98085251814408414, 0.044478338561507069, -0.78345634103039363], [-0.60500930133859709, 0.87895872758421212, 0.59561490314477927, 0.82072216033311784], [0.95001617573414032, 0.20468480993367022, 0.35602453634758469, -0.070107462971154888]]], [[[0.6864899883695792, 0.44223216849695901, 0.36378548060529137, 0.43451806931870163], [0.78750618231061065, -0.013479137470628899, 0.6577875955603778, 0.18918610945368441], [-0.33406116215906256, 0.93518928901459675, 0.90966513308817243, 0.87728898366655694]], [[-0.50347347741991966, 0.98138043351913606, 0.69570242811712601, 0.89654400325398464], [0.94456195322361602, -0.34691444129238169, 0.5145071925072755, 0.62508666703593296], [0.44494445035343788, -0.87171691786051886, 0.60749045264710033, 0.8800806202145921]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.147628731348)+(1.-msk_arg0)*(-0.395886944621)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[-0.31246093656771246, -0.52851142694973907, -0.044369590572535866, 0.67803010838222266], [-0.22235874254659382, -0.063647479223127856, 0.79517805608645387, -0.72719113756206477], [-0.89593891443313867, -0.73511734912568638, 0.47069114235682252, 0.21516590863893703]], [[-0.51977924692542388, -0.86198523567687713, 0.34545085226258321, 0.7381145911671303], [-0.16249995804664596, 0.87418344846586082, 0.49817375014892251, -0.13299998301700322], [0.28444788400540522, 0.6551380435429055, 0.32808612031418782, -0.27680377998420758]]], [[[0.62723660838275141, 0.90966290798967298, 0.31717572742596234, -0.83189379285591203], [0.16331236766697166, 0.11990847060725351, -0.47457646415191568, 0.24632866404588305], [0.8696338458087387, 0.77443690949473587, 0.54428098898198174, 0.25747336261162634]], [[-0.78877997567582514, -0.39156751222429831, 0.9832167479288465, 0.63939630264312863], [0.21069734048661237, 0.59234304867311915, -0.52502861329730433, -0.13801231722134299], [-0.54079587627861625, -0.5178931297743965, 0.46012276057985746, 0.96189327472595143]]], [[[0.14801737575488616, -0.63897615575085798, 0.84981249623306998, -0.53133061937546344], [0.21629302411047413, 0.37064127356125764, -0.73411858270565933, -0.67049515437618168], [-0.3475565652979713, 0.32949869075680982, -0.77385422607040844, 0.59428634814319592]], [[0.17015668069439971, 0.90181120005832049, -0.52618968462948779, 0.90564479664001474], [-0.52256578400000553, -0.13757787583585723, -0.16045224278530701, 0.020370641941540368], [-0.53394248004825062, 0.51159324501715497, 0.49168103358201365, -0.38080632331793129]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.14762873134834154, -0.14762873134834154, -0.044369590572535866, 0.67803010838222266], [-0.14762873134834154, -0.063647479223127856, 0.79517805608645387, -0.14762873134834154], [-0.14762873134834154, -0.14762873134834154, 0.47069114235682252, 0.21516590863893703]], [[-0.14762873134834154, -0.14762873134834154, 0.34545085226258321, 0.7381145911671303], [-0.14762873134834154, 0.87418344846586082, 0.49817375014892251, -0.13299998301700322], [0.28444788400540522, 0.6551380435429055, 0.32808612031418782, -0.14762873134834154]]], [[[0.62723660838275141, 0.90966290798967298, 0.31717572742596234, -0.14762873134834154], [0.16331236766697166, 0.11990847060725351, -0.14762873134834154, 0.24632866404588305], [0.8696338458087387, 0.77443690949473587, 0.54428098898198174, 0.25747336261162634]], [[-0.14762873134834154, -0.14762873134834154, 0.9832167479288465, 0.63939630264312863], [0.21069734048661237, 0.59234304867311915, -0.14762873134834154, -0.13801231722134299], [-0.14762873134834154, -0.14762873134834154, 0.46012276057985746, 0.96189327472595143]]], [[[0.14801737575488616, -0.14762873134834154, 0.84981249623306998, -0.14762873134834154], [0.21629302411047413, 0.37064127356125764, -0.14762873134834154, -0.14762873134834154], [-0.14762873134834154, 0.32949869075680982, -0.14762873134834154, 0.59428634814319592]], [[0.17015668069439971, 0.90181120005832049, -0.14762873134834154, 0.90564479664001474], [-0.14762873134834154, -0.13757787583585723, -0.14762873134834154, 0.020370641941540368], [-0.14762873134834154, 0.51159324501715497, 0.49168103358201365, -0.14762873134834154]]]])+(1.-msk_ref)*numpy.array([[[[-0.31246093656771246, -0.39588694462092167, -0.044369590572535866, 0.67803010838222266], [-0.22235874254659382, -0.063647479223127856, 0.79517805608645387, -0.39588694462092167], [-0.39588694462092167, -0.39588694462092167, 0.47069114235682252, 0.21516590863893703]], [[-0.39588694462092167, -0.39588694462092167, 0.34545085226258321, 0.7381145911671303], [-0.16249995804664596, 0.87418344846586082, 0.49817375014892251, -0.13299998301700322], [0.28444788400540522, 0.6551380435429055, 0.32808612031418782, -0.27680377998420758]]], [[[0.62723660838275141, 0.90966290798967298, 0.31717572742596234, -0.39588694462092167], [0.16331236766697166, 0.11990847060725351, -0.39588694462092167, 0.24632866404588305], [0.8696338458087387, 0.77443690949473587, 0.54428098898198174, 0.25747336261162634]], [[-0.39588694462092167, -0.39156751222429831, 0.9832167479288465, 0.63939630264312863], [0.21069734048661237, 0.59234304867311915, -0.39588694462092167, -0.13801231722134299], [-0.39588694462092167, -0.39588694462092167, 0.46012276057985746, 0.96189327472595143]]], [[[0.14801737575488616, -0.39588694462092167, 0.84981249623306998, -0.39588694462092167], [0.21629302411047413, 0.37064127356125764, -0.39588694462092167, -0.39588694462092167], [-0.3475565652979713, 0.32949869075680982, -0.39588694462092167, 0.59428634814319592]], [[0.17015668069439971, 0.90181120005832049, -0.39588694462092167, 0.90564479664001474], [-0.39588694462092167, -0.13757787583585723, -0.16045224278530701, 0.020370641941540368], [-0.39588694462092167, 0.51159324501715497, 0.49168103358201365, -0.38080632331793129]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.64955332739259841, -0.54371345505728952, -0.074718693307241013, 0.87395481865855773], [-0.11909148189432006, 0.13960911061237846, 0.9077075416009921, 0.79192333723239505], [0.61658737833032506, -0.87369804673653051, -0.19274386793515874, -0.83213737268194454]], [[0.58522426723080589, 0.34826670665588377, -0.11382049604378719, -0.73593426108570403], [0.69296638421621459, -0.3312517742066905, 0.0028459003084224666, 0.3353789028876224], [-0.3659007052100911, -0.45367418338194199, -0.6246292716319779, -0.97176440101919703]]], [[[-0.25285667227548325, -0.43168644077625551, 0.65933752295432568, -0.52256307781629907], [0.061386042066885471, 0.23623654928523408, 0.30211329760814887, 0.62586947950432581], [0.037748361945424191, -0.8821606052252875, -0.63610687132281263, 0.041836663053908207]], [[0.96588456303838166, -0.87318845106503518, -0.51728513906267648, -0.093125133561687612], [-0.23679375964289973, -0.23817502217249387, -0.11305766142284157, 0.5812403472686698], [0.0085361080479586793, 0.71749226785448172, 0.09676777236777645, 0.25687058900660875]]], [[[-0.19460738710411496, 0.68107263183300426, -0.37356156601552892, 0.99110820557417334], [0.25959302053768729, 0.33518124938257987, -0.73860067592390033, -0.81340757338264913], [-0.89335058994314887, -0.41019948734861567, -0.56423461750966419, 0.28507638239104005]], [[-0.28714030121450107, 0.43856209923596623, -0.3405210683554416, 0.77809420413775832], [0.42074853010469382, 0.68511114627268488, -0.9355512593381321, 0.15801324909253811], [-0.67265695222025257, -0.89407688041035316, 0.6323791858446135, -0.79502794053364489]]]])+(1.-msk_arg0)*numpy.array([[[[0.32211198310318467, -0.46711770508518669, -0.73206449664509865, 0.14611010683737735], [0.9897549808034567, -0.72550655355298233, 0.26447850234234216, 0.50706633406852841], [0.46839424906923477, -0.61995983075698136, 0.21284753009175206, -0.01055741110099051]], [[0.71195804705528665, 0.10144467362497833, 0.44189417094735695, 0.51148237843236877], [-0.35890766031535959, -0.15729126509587066, -0.011551900826927586, -0.6925377541638511], [-0.7548382347584508, -0.27363777983285731, -0.41845609324264643, -0.11520181280857411]]], [[[-0.057651344836884499, 0.3698634611032281, -0.33797348832774898, -0.99343837155583747], [-0.91224977527553497, 0.080518362391644116, 0.27699109082965601, 0.13492958714393621], [-0.90840582016768168, 0.015434393467110175, -0.17174623894477548, 0.37750940976182523]], [[0.16542463166845467, -0.75211940358628504, 0.66066165060930504, -0.84612390383538028], [0.29964428794483977, -0.28609922412188138, 0.2217935750654092, -0.042811647684336318], [0.02077035904048663, -0.73684058395684238, -0.70412799016464112, 0.33690823207449361]]], [[[0.75588046416563315, 0.38126119245659229, 0.54347752547017092, 0.095545005025334584], [-0.46869536480744278, -0.37765524563931208, 0.99973226631757517, 0.92018628104095601], [0.16694744107224668, -0.93550286704346775, -0.28996693142521845, 0.83501516004999576]], [[-0.58465259240942347, 0.084966669167076292, -0.16550726505346125, 0.92497093601090374], [-0.57508491165671449, -0.95640467896010306, 0.79046520273480336, -0.72219279015851368], [0.95124065585898476, -0.53101431048835401, -0.63566078997953634, 0.87339384061328573]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=maximum(arg0,arg1)
      s1=numpy.array([[[[-0.1051978068715782, -0.076175634332572795, 0.74222108364999362, 0.13811225467403943], [0.631589162126726, -0.77197902790331874, 0.58954221740743828, -0.66466807552729623], [-0.91321057861211541, -0.79122895997322584, -0.79000311426487668, 0.79291715962337128]], [[-0.036825834289621495, -0.17918899694231394, 0.71001761983990397, 0.98556597842863702], [-0.050695926774718902, 0.79786073024279225, -0.96398096252990473, -0.27155374512924912], [0.26231494261173793, 0.71834923833273678, 0.15560342048601838, 0.4674254660334658]]], [[[-0.18853733520870963, 0.023665839348501194, 0.90617130592536266, 0.50132640890137736], [0.53929686830817292, 0.37791701785493448, -0.65004364480986432, 0.50124220046629309], [0.47061231405153303, -0.64505137876845886, -0.10653989494870375, 0.27973053908449219]], [[-0.13655540883185391, 0.25663897744727038, 0.77054206091284527, -0.62320251062333942], [-0.82797118851206108, -0.0091261025533584395, -0.9243215554855797, -0.40616594451594445], [0.25507602726706757, -0.98478784749705173, 0.94747898987626922, 0.22931050723354907]]], [[[-0.35411061935900712, 0.48385818609704989, 0.21246680982957167, 0.60424636685034172], [-0.97492289595354031, -0.03669036824230032, -0.50272170251252568, -0.25756512141893118], [-0.5135661362691617, -0.74813853495589955, 0.75044249880887159, -0.22865135469733411]], [[-0.33824491009103919, -0.8323510313215341, -0.60884697142486566, -0.26214309593321872], [-0.37243779295626678, -0.49956292623391407, -0.68671308414696042, 0.95754049164931665], [-0.7246997733546876, -0.62006965200478859, -0.86162222813506828, -0.92745293853297772]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.1051978068715782, -0.076175634332572795, 0.74222108364999362, 0.87395481865855773], [0.631589162126726, 0.13960911061237846, 0.9077075416009921, 0.79192333723239505], [0.61658737833032506, -0.79122895997322584, -0.19274386793515874, 0.79291715962337128]], [[0.58522426723080589, 0.34826670665588377, 0.71001761983990397, 0.98556597842863702], [0.69296638421621459, 0.79786073024279225, 0.0028459003084224666, 0.3353789028876224], [0.26231494261173793, 0.71834923833273678, 0.15560342048601838, 0.4674254660334658]]], [[[-0.18853733520870963, 0.023665839348501194, 0.90617130592536266, 0.50132640890137736], [0.53929686830817292, 0.37791701785493448, 0.30211329760814887, 0.62586947950432581], [0.47061231405153303, -0.64505137876845886, -0.10653989494870375, 0.27973053908449219]], [[0.96588456303838166, 0.25663897744727038, 0.77054206091284527, -0.093125133561687612], [-0.23679375964289973, -0.0091261025533584395, -0.11305766142284157, 0.5812403472686698], [0.25507602726706757, 0.71749226785448172, 0.94747898987626922, 0.25687058900660875]]], [[[-0.19460738710411496, 0.68107263183300426, 0.21246680982957167, 0.99110820557417334], [0.25959302053768729, 0.33518124938257987, -0.50272170251252568, -0.25756512141893118], [-0.5135661362691617, -0.41019948734861567, 0.75044249880887159, 0.28507638239104005]], [[-0.28714030121450107, 0.43856209923596623, -0.3405210683554416, 0.77809420413775832], [0.42074853010469382, 0.68511114627268488, -0.68671308414696042, 0.95754049164931665], [-0.67265695222025257, -0.62006965200478859, 0.6323791858446135, -0.79502794053364489]]]])+(1.-msk_ref)*numpy.array([[[[0.32211198310318467, -0.076175634332572795, 0.74222108364999362, 0.14611010683737735], [0.9897549808034567, -0.72550655355298233, 0.58954221740743828, 0.50706633406852841], [0.46839424906923477, -0.61995983075698136, 0.21284753009175206, 0.79291715962337128]], [[0.71195804705528665, 0.10144467362497833, 0.71001761983990397, 0.98556597842863702], [-0.050695926774718902, 0.79786073024279225, -0.011551900826927586, -0.27155374512924912], [0.26231494261173793, 0.71834923833273678, 0.15560342048601838, 0.4674254660334658]]], [[[-0.057651344836884499, 0.3698634611032281, 0.90617130592536266, 0.50132640890137736], [0.53929686830817292, 0.37791701785493448, 0.27699109082965601, 0.50124220046629309], [0.47061231405153303, 0.015434393467110175, -0.10653989494870375, 0.37750940976182523]], [[0.16542463166845467, 0.25663897744727038, 0.77054206091284527, -0.62320251062333942], [0.29964428794483977, -0.0091261025533584395, 0.2217935750654092, -0.042811647684336318], [0.25507602726706757, -0.73684058395684238, 0.94747898987626922, 0.33690823207449361]]], [[[0.75588046416563315, 0.48385818609704989, 0.54347752547017092, 0.60424636685034172], [-0.46869536480744278, -0.03669036824230032, 0.99973226631757517, 0.92018628104095601], [0.16694744107224668, -0.74813853495589955, 0.75044249880887159, 0.83501516004999576]], [[-0.33824491009103919, 0.084966669167076292, -0.16550726505346125, 0.92497093601090374], [-0.37243779295626678, -0.49956292623391407, 0.79046520273480336, 0.95754049164931665], [0.95124065585898476, -0.53101431048835401, -0.63566078997953634, 0.87339384061328573]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.0881956698806)+(1.-msk_arg0)*(-0.731128324845)
      arg1=Data(numpy.array([[[[-0.64189758046957146, -0.53815399874264136, 0.033693340777282366, 0.99925795797181571], [-0.2419875717107054, -0.50747686174550699, -0.81027767900895475, -0.84877637041813614], [-0.20071783395542631, -0.8069228315827619, 0.66582338261558349, 0.64096730572944427]], [[0.81185199455197243, -0.90778080246183657, 0.017632995172836674, -0.32090381670185386], [0.13011082833732179, 0.6578464151231187, -0.6959246201621212, -0.6545688299236001], [0.14663475574733353, 0.91800376327950861, 0.56376151063798741, -0.96804446803081667]]], [[[-0.0021218076221873883, -0.10724622444534382, 0.38123000588899658, 0.12283867161727002], [0.3563313122155094, -0.042825459741448135, -0.39224174048110716, -0.93819757083393496], [-0.56950271363830596, 0.037025945267049831, -0.10711115625494805, -0.12244688771221512]], [[0.55557589199957658, 0.95005362115253589, 0.084514241828244518, -0.67046064450650178], [0.01747628454518857, 0.18620488628187792, 0.66507212829733886, 0.76247784353876535], [0.37735898180876082, 0.32486295087694095, 0.47501482765124581, 0.52248847470936344]]], [[[0.0036882590128266557, 0.8282751649691007, -0.80873751745324807, 0.47849609936161519], [0.39166314221529275, 0.48831283571527884, -0.86334428729172585, 0.79143886820146481], [0.83249453703468279, -0.98930182274409639, -0.083316120352270762, 0.294221269622853]], [[0.18930386704305269, -0.99776611169212037, 0.14587140405481969, -0.95686905824011759], [-0.15749539134547641, -0.56249207552408076, 0.45265834859712295, -0.46177078880637601], [0.66396367131331657, 0.96630757924567945, -0.86681340389547001, 0.6623848566435715]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.088195669880560379, 0.088195669880560379, 0.088195669880560379, 0.99925795797181571], [0.088195669880560379, 0.088195669880560379, 0.088195669880560379, 0.088195669880560379], [0.088195669880560379, 0.088195669880560379, 0.66582338261558349, 0.64096730572944427]], [[0.81185199455197243, 0.088195669880560379, 0.088195669880560379, 0.088195669880560379], [0.13011082833732179, 0.6578464151231187, 0.088195669880560379, 0.088195669880560379], [0.14663475574733353, 0.91800376327950861, 0.56376151063798741, 0.088195669880560379]]], [[[0.088195669880560379, 0.088195669880560379, 0.38123000588899658, 0.12283867161727002], [0.3563313122155094, 0.088195669880560379, 0.088195669880560379, 0.088195669880560379], [0.088195669880560379, 0.088195669880560379, 0.088195669880560379, 0.088195669880560379]], [[0.55557589199957658, 0.95005362115253589, 0.088195669880560379, 0.088195669880560379], [0.088195669880560379, 0.18620488628187792, 0.66507212829733886, 0.76247784353876535], [0.37735898180876082, 0.32486295087694095, 0.47501482765124581, 0.52248847470936344]]], [[[0.088195669880560379, 0.8282751649691007, 0.088195669880560379, 0.47849609936161519], [0.39166314221529275, 0.48831283571527884, 0.088195669880560379, 0.79143886820146481], [0.83249453703468279, 0.088195669880560379, 0.088195669880560379, 0.294221269622853]], [[0.18930386704305269, 0.088195669880560379, 0.14587140405481969, 0.088195669880560379], [0.088195669880560379, 0.088195669880560379, 0.45265834859712295, 0.088195669880560379], [0.66396367131331657, 0.96630757924567945, 0.088195669880560379, 0.6623848566435715]]]])+(1.-msk_ref)*numpy.array([[[[-0.64189758046957146, -0.53815399874264136, 0.033693340777282366, 0.99925795797181571], [-0.2419875717107054, -0.50747686174550699, -0.73112832484494983, -0.73112832484494983], [-0.20071783395542631, -0.73112832484494983, 0.66582338261558349, 0.64096730572944427]], [[0.81185199455197243, -0.73112832484494983, 0.017632995172836674, -0.32090381670185386], [0.13011082833732179, 0.6578464151231187, -0.6959246201621212, -0.6545688299236001], [0.14663475574733353, 0.91800376327950861, 0.56376151063798741, -0.73112832484494983]]], [[[-0.0021218076221873883, -0.10724622444534382, 0.38123000588899658, 0.12283867161727002], [0.3563313122155094, -0.042825459741448135, -0.39224174048110716, -0.73112832484494983], [-0.56950271363830596, 0.037025945267049831, -0.10711115625494805, -0.12244688771221512]], [[0.55557589199957658, 0.95005362115253589, 0.084514241828244518, -0.67046064450650178], [0.01747628454518857, 0.18620488628187792, 0.66507212829733886, 0.76247784353876535], [0.37735898180876082, 0.32486295087694095, 0.47501482765124581, 0.52248847470936344]]], [[[0.0036882590128266557, 0.8282751649691007, -0.73112832484494983, 0.47849609936161519], [0.39166314221529275, 0.48831283571527884, -0.73112832484494983, 0.79143886820146481], [0.83249453703468279, -0.73112832484494983, -0.083316120352270762, 0.294221269622853]], [[0.18930386704305269, -0.73112832484494983, 0.14587140405481969, -0.73112832484494983], [-0.15749539134547641, -0.56249207552408076, 0.45265834859712295, -0.46177078880637601], [0.66396367131331657, 0.96630757924567945, -0.73112832484494983, 0.6623848566435715]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.10531364581559632, -0.26135514113778791, 0.75823435654818261, 0.41913136584038457], [0.43115964993714972, 0.43325049456887954, 0.99798790086677469, 0.12148464100767264], [-0.792314005480238, 0.94449170983715258, -0.51282378136184414, 0.78279449919260413]], [[-0.256730957467878, -0.54685067746437355, 0.13799031452254362, 0.54420562233980974], [0.79271302319406733, 0.61289153297761811, 0.22488152217118107, 0.38057207678407945], [-0.58355124854227669, 0.33339550351240987, -0.45440247482861396, -0.32858088081907244]]], [[[0.42804301050211202, 0.71528000472301367, -0.29947441761599003, -0.65907060023841413], [0.21382369966318948, -0.23392820079386456, 0.1149254861227258, 0.26438763174771629], [0.54292796867386128, 0.93595972935560612, 0.63111118015829226, -0.48720925111434954]], [[-0.64752714499820963, 0.23062867134246989, -0.44111125359638725, 0.87281600565752915], [0.17646081091253429, 0.16503304920470341, -0.76446719475275016, -0.9496673356542491], [0.81402347205529568, -0.7279756234241106, 0.32595508977473875, 0.78908310868157816]]], [[[-0.7994446077113897, 0.54116981041814904, -0.29869702018833433, -0.60523743501453087], [-0.46193843131786938, -0.87737364655870431, -0.53149381910526228, 0.32118149490277959], [0.88124482945730831, 0.76961174010593836, -0.51624564499591163, 0.46112358500778217]], [[-0.72956420221936336, -0.49297722281654144, -0.93821464396887477, 0.6257868729024445], [-0.22970749405216329, -0.44455529448948572, 0.85363272062066464, 0.84754506932043783], [-0.66852456480381584, 0.79050538702573658, 0.32041213027833426, 0.036833079887540254]]]])+(1.-msk_arg0)*numpy.array([[[[-0.19914272243996511, -0.22066958304650086, -0.051623157097562489, -0.30649365550154917], [0.70526559900670605, 0.79190691746271002, -0.1840028427567888, 0.11449103902153879], [-0.5924052950405061, 0.3079650735182351, -0.9438123347143772, -0.065661960754274995]], [[-0.0036167079524380696, 0.84054224471953476, -0.53140279065734641, 0.35467617149678676], [0.29362960794600701, -0.14052271099644154, 0.1293556777292062, 0.45589620449734247], [0.45309220884092505, 0.070251984236403331, -0.79210333449212667, 0.10845805646166129]]], [[[0.30350245072854709, -0.61792951212226188, -0.98924055757045237, 0.75886074238090151], [0.19835957943587212, -0.32993508643372804, -0.81430829784396175, 0.67280893395841113], [-0.23121853491576294, 0.53086442636291209, -0.63883554946929699, 0.17420740192224771]], [[0.89221420493213976, -0.2045207552720627, -0.20771044164770736, 0.42648308785419431], [0.87113955270066956, 0.24540728624391916, -0.45094960000767226, -0.16188150623057007], [-0.45960021467049605, -0.95157979026703687, -0.82635297970091592, -0.97341062097802844]]], [[[0.48800175644884614, -0.50576269437461963, -0.17133398261855226, -0.72602653879188628], [-0.85605984598767382, -0.58287114441973764, 0.005984177454671169, 0.26177073655897831], [0.64181206014940528, 0.87931494110685526, 0.78403977073172282, 0.024419280675760158]], [[-0.68551675053970906, 0.14152572212171521, -0.77906301525452615, -0.12878542896599199], [0.33305465516919974, 0.0082682574899826111, -0.39785724636599062, 0.047898579856563428], [-0.84404774609808642, 0.67721243023535527, -0.59693854720693174, -0.37505322327010027]]]])
      arg1=Data(numpy.array([[[[-0.82759551572398471, 0.058473671021340712, -0.10837091814903643, 0.57740785960292684], [0.27913511962597259, 0.0011338106694573913, 0.0009601979643438785, -0.04802408596147334], [0.92688524933994465, 0.78439774647519878, -0.31909062602360128, 0.72602592282456158]], [[0.94140041851607315, 0.94570499934150676, 0.26198355391860817, 0.3065174789316123], [-0.48677124302537544, -0.85999326805316434, -0.99216619030997077, -0.083870140182316755], [-0.26968066595677942, -0.028588804387562616, 0.57648293580256471, 0.70249263700215847]]], [[[-0.59432588710527101, -0.015338113227458816, -0.67625911560986385, 0.87073185203582937], [0.26418569454285712, 0.76837073130429445, 0.32369932417835279, 0.14645995660195821], [-0.50777857923006087, 0.27528983484000102, 0.57195763489050599, 0.75155872411421054]], [[0.30020184045055109, -0.83179186646902536, 0.66761313528171162, 0.93305194275955561], [-0.44456663805664576, 0.55650037021524135, 0.62058565622477313, -0.61150812573877777], [-0.31226647099785221, 0.14513696705524182, 0.68032486017657678, -0.22086700354642508]]], [[[0.66284368284982187, 0.61236796493579271, 0.43440212926366417, 0.083118805833193532], [-0.85956397424801834, -0.10084123824854152, 0.67438359152521032, 0.85048470938007048], [0.66201451496804764, -0.61354777895468482, -0.80926632689236633, 0.8510794761665228]], [[-0.73695063130269212, 0.44757527421684307, 0.68262028916181294, 0.10862811437247544], [0.65035504442237335, -0.49416276490023092, 0.99800064674598987, -0.91779706704154007], [-0.69460250652998123, 0.81518826983336323, 0.33775413898516593, -0.57214687266614961]]]]),self.functionspace)
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.10531364581559632, 0.058473671021340712, 0.75823435654818261, 0.57740785960292684], [0.43115964993714972, 0.43325049456887954, 0.99798790086677469, 0.12148464100767264], [0.92688524933994465, 0.94449170983715258, -0.31909062602360128, 0.78279449919260413]], [[0.94140041851607315, 0.94570499934150676, 0.26198355391860817, 0.54420562233980974], [0.79271302319406733, 0.61289153297761811, 0.22488152217118107, 0.38057207678407945], [-0.26968066595677942, 0.33339550351240987, 0.57648293580256471, 0.70249263700215847]]], [[[0.42804301050211202, 0.71528000472301367, -0.29947441761599003, 0.87073185203582937], [0.26418569454285712, 0.76837073130429445, 0.32369932417835279, 0.26438763174771629], [0.54292796867386128, 0.93595972935560612, 0.63111118015829226, 0.75155872411421054]], [[0.30020184045055109, 0.23062867134246989, 0.66761313528171162, 0.93305194275955561], [0.17646081091253429, 0.55650037021524135, 0.62058565622477313, -0.61150812573877777], [0.81402347205529568, 0.14513696705524182, 0.68032486017657678, 0.78908310868157816]]], [[[0.66284368284982187, 0.61236796493579271, 0.43440212926366417, 0.083118805833193532], [-0.46193843131786938, -0.10084123824854152, 0.67438359152521032, 0.85048470938007048], [0.88124482945730831, 0.76961174010593836, -0.51624564499591163, 0.8510794761665228]], [[-0.72956420221936336, 0.44757527421684307, 0.68262028916181294, 0.6257868729024445], [0.65035504442237335, -0.44455529448948572, 0.99800064674598987, 0.84754506932043783], [-0.66852456480381584, 0.81518826983336323, 0.33775413898516593, 0.036833079887540254]]]])+(1.-msk_ref)*numpy.array([[[[-0.19914272243996511, 0.058473671021340712, -0.051623157097562489, 0.57740785960292684], [0.70526559900670605, 0.79190691746271002, 0.0009601979643438785, 0.11449103902153879], [0.92688524933994465, 0.78439774647519878, -0.31909062602360128, 0.72602592282456158]], [[0.94140041851607315, 0.94570499934150676, 0.26198355391860817, 0.35467617149678676], [0.29362960794600701, -0.14052271099644154, 0.1293556777292062, 0.45589620449734247], [0.45309220884092505, 0.070251984236403331, 0.57648293580256471, 0.70249263700215847]]], [[[0.30350245072854709, -0.015338113227458816, -0.67625911560986385, 0.87073185203582937], [0.26418569454285712, 0.76837073130429445, 0.32369932417835279, 0.67280893395841113], [-0.23121853491576294, 0.53086442636291209, 0.57195763489050599, 0.75155872411421054]], [[0.89221420493213976, -0.2045207552720627, 0.66761313528171162, 0.93305194275955561], [0.87113955270066956, 0.55650037021524135, 0.62058565622477313, -0.16188150623057007], [-0.31226647099785221, 0.14513696705524182, 0.68032486017657678, -0.22086700354642508]]], [[[0.66284368284982187, 0.61236796493579271, 0.43440212926366417, 0.083118805833193532], [-0.85605984598767382, -0.10084123824854152, 0.67438359152521032, 0.85048470938007048], [0.66201451496804764, 0.87931494110685526, 0.78403977073172282, 0.8510794761665228]], [[-0.68551675053970906, 0.44757527421684307, 0.68262028916181294, 0.10862811437247544], [0.65035504442237335, 0.0082682574899826111, 0.99800064674598987, 0.047898579856563428], [-0.69460250652998123, 0.81518826983336323, 0.33775413898516593, -0.37505322327010027]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.430656784848)+(1.-msk_arg0)*(-0.700783789461)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.68835738514383604, -0.76863401076143667, 0.92032350741511615, -0.1469001502609133], [0.06288453409673278, 0.38073779065461522, -0.90058807506723526, 0.38778286232709025], [0.96668256007501041, 0.057601266285067343, 0.79648117729052714, 0.49143175520310778]], [[0.98909339096938509, 0.7602637867878832, -0.912321941323607, -0.54315061194825787], [0.80423923526891117, 0.47661901140330509, 0.99031475342531228, -0.89450136434961736], [-0.36910767598659655, 0.55360242758997402, -0.24636032220916326, 0.013256286221418945]]], [[[0.15603828628588978, -0.70571626749096739, -0.22412221738086946, -0.44778376773528183], [0.83165578331141399, 0.65163198519388876, 0.6039897710150941, -0.3822861567244713], [0.31330097532750978, -0.81103293228454709, 0.17174182343570354, -0.035823123408156521]], [[0.62310089072276864, 0.54271156487897554, 0.44186419167763291, 0.22536200856774924], [0.81691779953483756, 0.97875287193551719, -0.10259001428076586, 0.99985268669145078], [-0.6545383935440594, -0.91966644139989562, 0.54410275610147973, -0.25257465340702723]]], [[[0.22718370645623831, -0.8974233572367345, -0.75452612517607376, -0.58709992502677699], [-0.063069442929713171, -0.2742728542659314, 0.0030934546784433525, 0.88865344687736392], [-0.28059615259505599, -0.38818143127085736, -0.26036533554867081, -0.91372878625635412]], [[-0.19770833412921429, -0.2520416533993588, -0.32329826656736227, -0.2921042427795626], [-0.77757026759743275, 0.19691427600790279, -0.82662487726058442, 0.76440912595714972], [-0.8785601665435594, 0.6685997731242812, 0.17858501472408173, -0.072430031752663382]]]])+(1.-msk_arg1)*numpy.array([[[[0.218050144488529, -0.97758051946393643, 0.850873231285731, -0.62549319560829364], [0.64439251587171076, 0.02043882543426645, -0.40088447997559573, 0.32825901856835582], [0.40086412906080149, 0.44806775103789498, 0.7575462104983206, -0.71189398852218688]], [[-0.47405190774543682, -0.13001057892794665, -0.51504190270399475, 0.35420972217429236], [-0.83399490871466253, 0.0058345121935481359, 0.14938712239679286, 0.94294724195590929], [0.52941947608322826, 0.92556345140725682, -0.88319470923943211, 0.76396950935303654]]], [[[0.89036261550337015, 0.87671711452257739, 0.25741308722544365, 0.62226771243047474], [-0.015121664468682905, 0.93979205811046596, -0.71174182978124589, 0.97248180642636251], [-0.64050451914857787, -0.61134599587804939, -0.9143086087295571, -0.10806644675596133]], [[0.21434619581078618, 0.85410384206731016, -0.051783648108333358, 0.80705842847270559], [0.059772809014316586, 0.63262662785196522, 0.16149018829346895, -0.34656541265527929], [-0.88512172547595824, -0.86523726523371947, -0.82269875427618322, -0.72165361398578187]]], [[[0.93903641427310136, 0.28209257258291887, -0.51351863703745515, 0.68881576173912262], [-0.19081765518314353, -0.044663078887927421, 0.065339976778740816, 0.97860023971151544], [0.065270643177932852, 0.84667221338805354, 0.015860276910488746, -0.23037023584540139]], [[0.56220085723088831, 0.2868468697603257, 0.73939367184777383, 0.48844672936175515], [-0.54331103170726092, -0.55778174945812209, -0.95425197901750436, 0.59928943529546808], [0.79804805060136541, 0.64738539292254482, 0.47278529075708753, -0.84128123298524726]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.68835738514383604, -0.43065678484805492, 0.92032350741511615, -0.1469001502609133], [0.06288453409673278, 0.38073779065461522, -0.43065678484805492, 0.38778286232709025], [0.96668256007501041, 0.057601266285067343, 0.79648117729052714, 0.49143175520310778]], [[0.98909339096938509, 0.7602637867878832, -0.43065678484805492, -0.43065678484805492], [0.80423923526891117, 0.47661901140330509, 0.99031475342531228, -0.43065678484805492], [-0.36910767598659655, 0.55360242758997402, -0.24636032220916326, 0.013256286221418945]]], [[[0.15603828628588978, -0.43065678484805492, -0.22412221738086946, -0.43065678484805492], [0.83165578331141399, 0.65163198519388876, 0.6039897710150941, -0.3822861567244713], [0.31330097532750978, -0.43065678484805492, 0.17174182343570354, -0.035823123408156521]], [[0.62310089072276864, 0.54271156487897554, 0.44186419167763291, 0.22536200856774924], [0.81691779953483756, 0.97875287193551719, -0.10259001428076586, 0.99985268669145078], [-0.43065678484805492, -0.43065678484805492, 0.54410275610147973, -0.25257465340702723]]], [[[0.22718370645623831, -0.43065678484805492, -0.43065678484805492, -0.43065678484805492], [-0.063069442929713171, -0.2742728542659314, 0.0030934546784433525, 0.88865344687736392], [-0.28059615259505599, -0.38818143127085736, -0.26036533554867081, -0.43065678484805492]], [[-0.19770833412921429, -0.2520416533993588, -0.32329826656736227, -0.2921042427795626], [-0.43065678484805492, 0.19691427600790279, -0.43065678484805492, 0.76440912595714972], [-0.43065678484805492, 0.6685997731242812, 0.17858501472408173, -0.072430031752663382]]]])+(1.-msk_ref)*numpy.array([[[[0.218050144488529, -0.70078378946110842, 0.850873231285731, -0.62549319560829364], [0.64439251587171076, 0.02043882543426645, -0.40088447997559573, 0.32825901856835582], [0.40086412906080149, 0.44806775103789498, 0.7575462104983206, -0.70078378946110842]], [[-0.47405190774543682, -0.13001057892794665, -0.51504190270399475, 0.35420972217429236], [-0.70078378946110842, 0.0058345121935481359, 0.14938712239679286, 0.94294724195590929], [0.52941947608322826, 0.92556345140725682, -0.70078378946110842, 0.76396950935303654]]], [[[0.89036261550337015, 0.87671711452257739, 0.25741308722544365, 0.62226771243047474], [-0.015121664468682905, 0.93979205811046596, -0.70078378946110842, 0.97248180642636251], [-0.64050451914857787, -0.61134599587804939, -0.70078378946110842, -0.10806644675596133]], [[0.21434619581078618, 0.85410384206731016, -0.051783648108333358, 0.80705842847270559], [0.059772809014316586, 0.63262662785196522, 0.16149018829346895, -0.34656541265527929], [-0.70078378946110842, -0.70078378946110842, -0.70078378946110842, -0.70078378946110842]]], [[[0.93903641427310136, 0.28209257258291887, -0.51351863703745515, 0.68881576173912262], [-0.19081765518314353, -0.044663078887927421, 0.065339976778740816, 0.97860023971151544], [0.065270643177932852, 0.84667221338805354, 0.015860276910488746, -0.23037023584540139]], [[0.56220085723088831, 0.2868468697603257, 0.73939367184777383, 0.48844672936175515], [-0.54331103170726092, -0.55778174945812209, -0.70078378946110842, 0.59928943529546808], [0.79804805060136541, 0.64738539292254482, 0.47278529075708753, -0.70078378946110842]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_maximum_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.56285787359423467, 0.37775535632450929, -0.3363819504506349, -0.39614224287220079], [-0.65956942938945873, -0.050567052731754325, -0.54905395376446253, -0.94001597388873392], [-0.78068340465775754, -0.89097864337441868, -0.83984842944522442, 0.74631234877918584]], [[0.08573606948206014, 0.55431246693940839, -0.8641772485747512, -0.42863009943253139], [0.80323187807206664, -0.74856933876086118, -0.01644211276713814, -0.91644648328349509], [0.40224351184954954, -0.91717203862964691, -0.84862948625343493, -0.25037544559678815]]], [[[0.82728992326268469, -0.95042129406079257, -0.93015204068307433, -0.28209813227511105], [-0.51972916795137514, 0.74506459052200502, -0.58415751918737313, -0.72089184286465779], [0.70832242911523857, 0.32910818999400115, 0.51617654973055238, 0.88129235546352791]], [[0.58177700099759, 0.63909107674454901, 0.86508533718829983, -0.45384546311830021], [0.34909094815239139, -0.85407855551520706, -0.83626668667446791, -0.19144948150684948], [-0.54080937187207279, -0.29228298836204991, 0.073340450796397505, -0.39919385201564572]]], [[[-0.35454323448815428, 0.78785581682131256, -0.37407581640252885, -0.9491936944611421], [-0.90501286696617278, 0.69422301523334684, 0.080299003664048518, -0.10604345961307726], [0.37557752598770766, -0.11425319383355337, 0.43620438053394817, -0.63938084921875493]], [[-0.6542747934342823, -0.77225386259596918, -0.30885873461055691, -0.93491787154442219], [0.44906982794974248, 0.47078393816377861, -0.53577916133628367, 0.57388498777390184], [0.53885977209128866, 0.79843035696797116, 0.29270123507386536, 0.76740426807290363]]]])+(1.-msk_arg0)*numpy.array([[[[-0.89333258741389199, 0.57101135857695096, 0.82736606705903304, -0.55893684971502733], [-0.92021692928061039, -0.55348474011111159, 0.29566092180536008, -0.13507617727452992], [0.13917734283523964, -0.83620085111886411, 0.23398407194295245, 0.31146881246574765]], [[-0.029507824602079902, 0.1574243100965087, 0.31486576471582972, -0.95685787951979218], [-0.48899322354200181, -0.94149220476950357, -0.24568012939396544, -0.15845295863431397], [0.29158080714097045, 0.23100712186933658, 0.7981595934007133, 0.38286456647335521]]], [[[0.61716197962362473, -0.45981943122067626, 0.77169576646882287, -0.61914888379614541], [0.85941618082643645, 0.3953325518632147, -0.071709454511042603, -0.16346196683480119], [-0.27195889282127017, 0.640204102093195, 0.36381437568430219, -0.23698452461448793]], [[-0.19651750243903776, -0.85806312069582069, -0.16697127876879536, -0.13043639826490505], [0.30397572395834271, -0.72931842874315134, -0.042718535847001471, 0.29178523953544122], [-0.26391565154853391, -0.45797052486847045, -0.31300470368371225, -0.25426548079345146]]], [[[0.25564008776185876, -0.33973920939409541, -0.42791398656409929, -0.52664151938223003], [-0.10999100193161926, -0.29629666691577783, -0.40782973073033224, -0.7964414558802948], [0.76976134456727685, -0.91549509332347023, 0.72186262412866609, -0.56773143000653792]], [[-0.42862609106683847, -0.35583358749489524, 0.87176384608629998, -0.6521142543462386], [0.067116251967670904, 0.48376703343805683, -0.38447080115823962, -0.52461174025121959], [0.95864693409401736, -0.18283658347090248, 0.65543503798526181, 0.34294213784697214]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.91334699106125039, 0.08034469059439342, 0.95336683070903305, -0.30302709745042145], [0.67698272338736065, -0.081495809822069276, -0.96880332310388173, -0.65951169314669], [0.14645181982635735, 0.95081750997466497, 0.070403435387265789, 0.86128470750091801]], [[0.31458850366898372, 0.37355625042636609, 0.59835306604159055, -0.55972405824577054], [0.58982641275618297, -0.92890562173951086, -0.30083832088441054, 0.89547691587223865], [-0.63027121919601004, -0.64536171128018927, 0.24414107153983289, 0.87244828606233771]]], [[[-0.89466900572920127, 0.5598293778988026, -0.89619884347773127, -0.88632252361082808], [-0.43917654443748066, -0.26084937566420874, 0.085241904688784231, 0.91415146709412731], [0.21564731418211958, 0.48337780597622526, 0.19274924619906364, 0.68742605559501802]], [[-0.52271461311172263, 0.86796787718622181, 0.60388060730113802, 0.59164210921511518], [0.06016452677678541, -0.69422396793023222, -0.94949047918679574, -0.023652022509516479], [0.47311564439251907, -0.83556354001059763, 0.30169446600775252, -0.41514884541574859]]], [[[0.94784819259638442, -0.96878672195987425, 0.41736597890415839, -0.54187648571449287], [0.087381032350053456, 0.45323520162423536, 0.04915039358163531, 0.44902577072276673], [-0.36295093024222025, -0.3956861841715571, -0.26069456297899429, 0.96853061131817464]], [[0.11413660171644269, -0.36860481482212748, -0.65069198361079228, 0.46860889975506259], [-0.96249981724552258, -0.64237067775573453, -0.20746509600546581, -0.41040214652163631], [-0.38498880402659741, 0.97133351909870091, -0.20699459203726933, 0.52388252445004002]]]])+(1.-msk_arg1)*numpy.array([[[[0.59326071032227223, -0.80869719454086364, 0.94238621327406347, -0.84308193696275091], [0.70828574326935856, 0.91353233044236459, -0.15648092096926169, 0.18059659161430264], [-0.37080639325253117, -0.096042972458373566, -0.50397349081083131, 0.90885407233921245]], [[-0.36356977188789052, 0.07834805962528657, -0.50427040211387508, -0.17978606506745876], [0.0037874599835847622, 0.5356158232271877, -0.95158338448852087, 0.77887517249080407], [-0.97788888695396725, 0.025164090720837473, 0.61062564247754847, 0.50036684617541782]]], [[[-0.51188738745996476, -0.56019687946102437, 0.5290236210334176, -0.59831305037402949], [0.16156648393526041, 0.013553898362512085, 0.47562501938041502, 0.88759773681313248], [0.8561694563853337, 0.60259960885555874, -0.082171173450925172, 0.63858906210461575]], [[-0.46713128615175425, -0.055061913628714487, -0.26470592863123388, 0.12914283139830651], [0.30407054362431873, -0.51481475425475165, -0.61454326667132175, -0.83597993953409167], [-0.361452844057218, -0.14442685496647223, 0.78922625506892952, -0.60792243592735828]]], [[[-0.49703903944354133, 0.095413976577233273, 0.16265112247299029, 0.61673952395802845], [-0.88844924670122549, -0.36852319019343116, 0.90935427930023738, -0.29773383810366583], [-0.80667363405005532, -0.16023550925636854, 0.40614983587173814, -0.81931798357574137]], [[0.75510295592173726, -0.43123287953825495, 0.70147960136393661, 0.14788978483969961], [-0.23220064658101736, 0.5689017497000739, 0.012773766156810806, 0.088171873737747708], [-0.67756995292205335, -0.43998855003188519, 0.96781348177706006, 0.8176918424069477]]]])
      res=maximum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.56285787359423467, 0.37775535632450929, 0.95336683070903305, -0.30302709745042145], [0.67698272338736065, -0.050567052731754325, -0.54905395376446253, -0.65951169314669], [0.14645181982635735, 0.95081750997466497, 0.070403435387265789, 0.86128470750091801]], [[0.31458850366898372, 0.55431246693940839, 0.59835306604159055, -0.42863009943253139], [0.80323187807206664, -0.74856933876086118, -0.01644211276713814, 0.89547691587223865], [0.40224351184954954, -0.64536171128018927, 0.24414107153983289, 0.87244828606233771]]], [[[0.82728992326268469, 0.5598293778988026, -0.89619884347773127, -0.28209813227511105], [-0.43917654443748066, 0.74506459052200502, 0.085241904688784231, 0.91415146709412731], [0.70832242911523857, 0.48337780597622526, 0.51617654973055238, 0.88129235546352791]], [[0.58177700099759, 0.86796787718622181, 0.86508533718829983, 0.59164210921511518], [0.34909094815239139, -0.69422396793023222, -0.83626668667446791, -0.023652022509516479], [0.47311564439251907, -0.29228298836204991, 0.30169446600775252, -0.39919385201564572]]], [[[0.94784819259638442, 0.78785581682131256, 0.41736597890415839, -0.54187648571449287], [0.087381032350053456, 0.69422301523334684, 0.080299003664048518, 0.44902577072276673], [0.37557752598770766, -0.11425319383355337, 0.43620438053394817, 0.96853061131817464]], [[0.11413660171644269, -0.36860481482212748, -0.30885873461055691, 0.46860889975506259], [0.44906982794974248, 0.47078393816377861, -0.20746509600546581, 0.57388498777390184], [0.53885977209128866, 0.97133351909870091, 0.29270123507386536, 0.76740426807290363]]]])+(1.-msk_ref)*numpy.array([[[[0.59326071032227223, 0.57101135857695096, 0.94238621327406347, -0.55893684971502733], [0.70828574326935856, 0.91353233044236459, 0.29566092180536008, 0.18059659161430264], [0.13917734283523964, -0.096042972458373566, 0.23398407194295245, 0.90885407233921245]], [[-0.029507824602079902, 0.1574243100965087, 0.31486576471582972, -0.17978606506745876], [0.0037874599835847622, 0.5356158232271877, -0.24568012939396544, 0.77887517249080407], [0.29158080714097045, 0.23100712186933658, 0.7981595934007133, 0.50036684617541782]]], [[[0.61716197962362473, -0.45981943122067626, 0.77169576646882287, -0.59831305037402949], [0.85941618082643645, 0.3953325518632147, 0.47562501938041502, 0.88759773681313248], [0.8561694563853337, 0.640204102093195, 0.36381437568430219, 0.63858906210461575]], [[-0.19651750243903776, -0.055061913628714487, -0.16697127876879536, 0.12914283139830651], [0.30407054362431873, -0.51481475425475165, -0.042718535847001471, 0.29178523953544122], [-0.26391565154853391, -0.14442685496647223, 0.78922625506892952, -0.25426548079345146]]], [[[0.25564008776185876, 0.095413976577233273, 0.16265112247299029, 0.61673952395802845], [-0.10999100193161926, -0.29629666691577783, 0.90935427930023738, -0.29773383810366583], [0.76976134456727685, -0.16023550925636854, 0.72186262412866609, -0.56773143000653792]], [[0.75510295592173726, -0.35583358749489524, 0.87176384608629998, 0.14788978483969961], [0.067116251967670904, 0.5689017497000739, 0.012773766156810806, 0.088171873737747708], [0.95864693409401736, -0.18283658347090248, 0.96781348177706006, 0.8176918424069477]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_float_rank0(self):
      arg0=0.576130683834
      arg1=0.412116338185
      res=minimum(arg0,arg1)
      ref=0.412116338185
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_array_rank0(self):
      arg0=0.289003350398
      arg1=numpy.array(-0.445379036457)
      res=minimum(arg0,arg1)
      ref=numpy.array(-0.445379036457)
      if isinstance(res,numpy.ndarray):
         self.assertEqual(res.shape,(),"wrong shape of result.")
      else:
         self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_Symbol_rank0(self):
      arg0=-0.795460602902
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.057865339538)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-0.795460602902)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_constData_rank0(self):
      arg0=0.371238578658
      arg1=Data(-0.645135030501,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(-0.645135030501,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_expandedData_rank0(self):
      arg0=-0.911968300013
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.806502526946)+(1.-msk_arg1)*(-0.438461765499)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.911968300013)+(1.-msk_ref)*(-0.911968300013)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_array_rank1(self):
      arg0=-0.819071116627
      arg1=numpy.array([0.31932661296889409, -0.069005867155198297])
      res=minimum(arg0,arg1)
      ref=numpy.array([-0.81907111662704235, -0.81907111662704235])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_Symbol_rank1(self):
      arg0=0.961907855854
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([0.71421469083190137, 0.66166660438927982])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.71421469083190137, 0.66166660438927982])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_constData_rank1(self):
      arg0=-0.968249647669
      arg1=Data(numpy.array([-0.52214145626118125, -0.58128985653909826]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.96824964766889465, -0.96824964766889465]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_expandedData_rank1(self):
      arg0=0.925021455714
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.51802917099944845, -0.73552775153957839])+(1.-msk_arg1)*numpy.array([0.94684306680258179, 0.90506150125803431])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.51802917099944845, -0.73552775153957839])+(1.-msk_ref)*numpy.array([0.92502145571355165, 0.90506150125803431])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_array_rank2(self):
      arg0=0.478671800384
      arg1=numpy.array([[0.90441638908409461, 0.65105243799339574, 0.55757907837341425, 0.96796203872288622, 0.65429378508429648], [-0.9839204249726301, -0.074489762658563885, 0.60404519273294666, -0.49171021040278884, -0.96236949107971359], [-0.34775021026631392, 0.03117292968711971, -0.12859207137734652, 0.82761350470691464, -0.47178241252823083], [0.34614973425604645, 0.54814360606236878, 0.73720876600461183, 0.76149194774572737, 0.22429834323918452]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[0.47867180038398538, 0.47867180038398538, 0.47867180038398538, 0.47867180038398538, 0.47867180038398538], [-0.9839204249726301, -0.074489762658563885, 0.47867180038398538, -0.49171021040278884, -0.96236949107971359], [-0.34775021026631392, 0.03117292968711971, -0.12859207137734652, 0.47867180038398538, -0.47178241252823083], [0.34614973425604645, 0.47867180038398538, 0.47867180038398538, 0.47867180038398538, 0.22429834323918452]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_Symbol_rank2(self):
      arg0=0.68364346717
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[-0.042844872804917999, 0.15831472887466269, 0.60399494084203398, -0.35880039162979438, 0.62579400882782488], [0.29706029923208788, 0.86074671446204842, 0.71410623852035271, 0.77752871203854967, -0.83366440037832001], [-0.82757199785724689, 0.65944198579793434, -0.40611818515717935, 0.37528061111072764, 0.55885961648126026], [0.42438562655850331, -0.63223079701355545, -0.34676968192525748, 0.32019111161752223, 0.17119274535725859]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.042844872804917999, 0.15831472887466269, 0.60399494084203398, -0.35880039162979438, 0.62579400882782488], [0.29706029923208788, 0.68364346717028135, 0.68364346717028135, 0.68364346717028135, -0.83366440037832001], [-0.82757199785724689, 0.65944198579793434, -0.40611818515717935, 0.37528061111072764, 0.55885961648126026], [0.42438562655850331, -0.63223079701355545, -0.34676968192525748, 0.32019111161752223, 0.17119274535725859]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_constData_rank2(self):
      arg0=-0.720838996247
      arg1=Data(numpy.array([[-0.3194234988515039, 0.61804206873773371, 0.38173572485452434, 0.83348152044490842, 0.62513540816554869], [-0.8843675267723512, 0.47081586165207501, -0.8954880719672702, 0.87135071329587555, 0.45445544130117765], [0.45909558655893501, -0.2196943692231903, -0.84367859877127671, -0.8839183602265086, 0.57583179655742933], [0.70220506269758887, -0.96222604760788122, -0.73678170743805582, -0.49608249020194473, -0.80887113413205292]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.7208389962473174, -0.7208389962473174, -0.7208389962473174, -0.7208389962473174, -0.7208389962473174], [-0.8843675267723512, -0.7208389962473174, -0.8954880719672702, -0.7208389962473174, -0.7208389962473174], [-0.7208389962473174, -0.7208389962473174, -0.84367859877127671, -0.8839183602265086, -0.7208389962473174], [-0.7208389962473174, -0.96222604760788122, -0.73678170743805582, -0.7208389962473174, -0.80887113413205292]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_expandedData_rank2(self):
      arg0=0.495699222644
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.49930708441927374, 0.7877563342214875, 0.18718421840595245, 0.28640212239203855, -0.098555309394831481], [0.13795575274883176, -0.40917614030441851, 0.32903723135509, -0.38623631617861931, 0.029402757771769261], [-0.96910292054156155, -0.56109509853717054, -0.10246680739939507, 0.3287515340044147, 0.17887298858214229], [-0.81089405837278794, 0.042371551758132675, -0.51600300197393367, -0.55464369981571227, -0.47783840203259986]])+(1.-msk_arg1)*numpy.array([[-0.051685226236429349, -0.13272660137191084, 0.78576887405219753, -0.7748999659405067, 0.14209476128681842], [0.84845388952803136, -0.26771507129818128, 0.04001228027127457, -0.65756325918272207, 0.83341994005464581], [0.69877677820696116, -0.34424091482753227, -0.33461693696891515, -0.77592869887258065, 0.065093052535560192], [0.93191889936701977, -0.88228320287439876, 0.96359621418279051, -0.61215698455214462, 0.57613852010423794]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.4956992226436987, 0.4956992226436987, 0.18718421840595245, 0.28640212239203855, -0.098555309394831481], [0.13795575274883176, -0.40917614030441851, 0.32903723135509, -0.38623631617861931, 0.029402757771769261], [-0.96910292054156155, -0.56109509853717054, -0.10246680739939507, 0.3287515340044147, 0.17887298858214229], [-0.81089405837278794, 0.042371551758132675, -0.51600300197393367, -0.55464369981571227, -0.47783840203259986]])+(1.-msk_ref)*numpy.array([[-0.051685226236429349, -0.13272660137191084, 0.4956992226436987, -0.7748999659405067, 0.14209476128681842], [0.4956992226436987, -0.26771507129818128, 0.04001228027127457, -0.65756325918272207, 0.4956992226436987], [0.4956992226436987, -0.34424091482753227, -0.33461693696891515, -0.77592869887258065, 0.065093052535560192], [0.4956992226436987, -0.88228320287439876, 0.4956992226436987, -0.61215698455214462, 0.4956992226436987]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_array_rank3(self):
      arg0=0.637928830253
      arg1=numpy.array([[[-0.35653583772730757, 0.61571708422109794], [0.07350669790974762, 0.61876388475252808]], [[-0.053463270495726967, 0.83419890725283885], [-0.96099905287190679, -0.80575372226068542]], [[0.24300609133706175, 0.2756588695642721], [-0.87654405373062816, -0.3200825567458685]], [[-0.34218532069750118, -0.4170463384581613], [0.53608434420038287, -0.26824412326699676]], [[0.56718420178173146, -0.75344847265520709], [0.4436615786631195, -0.52972537223344784]], [[-0.75515269605520174, 0.50931856772376993], [-0.84961959248176089, 0.02703055814068378]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[-0.35653583772730757, 0.61571708422109794], [0.07350669790974762, 0.61876388475252808]], [[-0.053463270495726967, 0.63792883025270131], [-0.96099905287190679, -0.80575372226068542]], [[0.24300609133706175, 0.2756588695642721], [-0.87654405373062816, -0.3200825567458685]], [[-0.34218532069750118, -0.4170463384581613], [0.53608434420038287, -0.26824412326699676]], [[0.56718420178173146, -0.75344847265520709], [0.4436615786631195, -0.52972537223344784]], [[-0.75515269605520174, 0.50931856772376993], [-0.84961959248176089, 0.02703055814068378]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_Symbol_rank3(self):
      arg0=0.894495643181
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[-0.90613729250969199, 0.37131529339144365], [0.057353325379353004, -0.92305472462035376]], [[0.49314842983606955, -0.75156438593974673], [-0.8085183922979069, -0.87823087137433742]], [[0.73630032314683747, -0.55890645086741619], [0.55483238176143312, 0.10055303634870172]], [[0.85448195138501726, 0.79787340913783145], [0.1772091871909165, 0.85948634932974799]], [[-0.68928255735554345, -0.089513122712289839], [-0.39245577916628349, 0.6237358069671366]], [[0.90829705735042054, -0.60449884852496827], [-0.089718244027576555, -0.79325304007628383]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.90613729250969199, 0.37131529339144365], [0.057353325379353004, -0.92305472462035376]], [[0.49314842983606955, -0.75156438593974673], [-0.8085183922979069, -0.87823087137433742]], [[0.73630032314683747, -0.55890645086741619], [0.55483238176143312, 0.10055303634870172]], [[0.85448195138501726, 0.79787340913783145], [0.1772091871909165, 0.85948634932974799]], [[-0.68928255735554345, -0.089513122712289839], [-0.39245577916628349, 0.6237358069671366]], [[0.89449564318064589, -0.60449884852496827], [-0.089718244027576555, -0.79325304007628383]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_constData_rank3(self):
      arg0=-0.310043459727
      arg1=Data(numpy.array([[[0.38697328105644235, 0.70221233643550196], [0.34544746758392653, -0.11055874729875126]], [[0.42778446857838048, -0.8880788764748333], [0.73132093952186783, 0.8033154117025938]], [[-0.5661335107684673, 0.49843795591687612], [0.22595639372635068, 0.044169583330319506]], [[-0.9124201688421103, 0.67544031532457383], [0.25484397741761766, 0.047857857156642147]], [[0.91698387844722751, -0.63314665729291408], [-0.56451465364624975, 0.97099413120842981]], [[-0.96623694702119445, -0.36559666676379576], [0.48002112676354947, -0.49901854944236645]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.31004345972712621, -0.31004345972712621], [-0.31004345972712621, -0.31004345972712621]], [[-0.31004345972712621, -0.8880788764748333], [-0.31004345972712621, -0.31004345972712621]], [[-0.5661335107684673, -0.31004345972712621], [-0.31004345972712621, -0.31004345972712621]], [[-0.9124201688421103, -0.31004345972712621], [-0.31004345972712621, -0.31004345972712621]], [[-0.31004345972712621, -0.63314665729291408], [-0.56451465364624975, -0.31004345972712621]], [[-0.96623694702119445, -0.36559666676379576], [-0.31004345972712621, -0.49901854944236645]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_expandedData_rank3(self):
      arg0=0.180877844009
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.72765769710886885, -0.77950756222280115], [-0.76657464297493383, 0.26360833458842103]], [[-0.63889942876567685, -0.96415887226926067], [0.40819049952211262, 0.53747956489837434]], [[-0.17320920691384134, -0.66907140584029845], [0.12483196580198208, -0.70507661237760755]], [[0.071507590554227729, 0.062086896243514822], [-0.37700203793464682, -0.06698451825887064]], [[0.99908694441096602, -0.49471711706132759], [-0.25036019181025893, 0.69865642814709528]], [[-0.47164774628773198, 0.19617691072048715], [-0.50986442038570967, 0.18436333118518289]]])+(1.-msk_arg1)*numpy.array([[[0.10851375096232618, -0.13477534514489853], [0.18640776317948937, -0.18353749086282534]], [[0.64663017185804406, 0.25612468253827858], [-0.64860384050774189, -0.26232530692787082]], [[0.51631302366268272, 0.74589200206144768], [0.2374091871954056, 0.055785511654019571]], [[-0.8453176382142269, -0.10030284644757259], [-0.56959526008388495, 0.13512355223289974]], [[0.99581050667376392, 0.48594409950935535], [-0.92853699670508494, -0.8971336427464387]], [[0.69034433856458577, 0.14940275380447643], [-0.23670374220161006, 0.79042782527452959]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.72765769710886885, -0.77950756222280115], [-0.76657464297493383, 0.18087784400928597]], [[-0.63889942876567685, -0.96415887226926067], [0.18087784400928597, 0.18087784400928597]], [[-0.17320920691384134, -0.66907140584029845], [0.12483196580198208, -0.70507661237760755]], [[0.071507590554227729, 0.062086896243514822], [-0.37700203793464682, -0.06698451825887064]], [[0.18087784400928597, -0.49471711706132759], [-0.25036019181025893, 0.18087784400928597]], [[-0.47164774628773198, 0.18087784400928597], [-0.50986442038570967, 0.18087784400928597]]])+(1.-msk_ref)*numpy.array([[[0.10851375096232618, -0.13477534514489853], [0.18087784400928597, -0.18353749086282534]], [[0.18087784400928597, 0.18087784400928597], [-0.64860384050774189, -0.26232530692787082]], [[0.18087784400928597, 0.18087784400928597], [0.18087784400928597, 0.055785511654019571]], [[-0.8453176382142269, -0.10030284644757259], [-0.56959526008388495, 0.13512355223289974]], [[0.18087784400928597, 0.18087784400928597], [-0.92853699670508494, -0.8971336427464387]], [[0.18087784400928597, 0.14940275380447643], [-0.23670374220161006, 0.18087784400928597]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_array_rank4(self):
      arg0=-0.603085006827
      arg1=numpy.array([[[[-0.70607704048865649, -0.56029094543871594, 0.2759474860161828, -0.41979392009896799], [-0.75548953091054183, 0.40511972963574361, 0.71460850587655478, 0.27090479688119151], [-0.061794010531437848, -0.0015136391740222521, -0.30206698891426353, 0.013381419945716289]], [[0.31994705777693921, 0.47258343222484944, 0.37643022651957669, 0.85194373648799337], [0.45941386337903189, -0.45892812095380631, 0.41582039496693923, 0.21413609093107877], [-0.0060070961090430419, 0.51567534162223505, 0.51312188502507339, -0.68845438476518916]]], [[[-0.42927693241618514, -0.87617183460935077, 0.75076927409777072, -0.55153508970034371], [-0.44024321061727245, -0.83467085878218961, -0.39572065240267684, 0.13922892068583947], [0.9347261071909736, 0.2983629795170144, 0.35380597203291431, 0.96801563109079991]], [[0.52944817085309537, -0.029814060272752352, -0.055395760818877315, -0.11553414997981881], [-0.72927205003089735, 0.14151430463091752, 0.26469336301284674, -0.84290814078559761], [0.53244003607890944, 0.56077533799253754, -0.54463232950117257, -0.49240028005267633]]], [[[0.5938398944802763, -0.98169089874803594, -0.44777749260327093, -0.99372717001525279], [0.61554748714269025, -0.99945790763342135, -0.9487593851596996, -0.26853211882521277], [0.59884043432358047, 0.62939900229019474, 0.12091425374976561, -0.16087608097193984]], [[-0.48710949684942784, 0.52733606804478717, -0.5988443576649074, 0.60717746327021271], [-0.6025481691604162, 0.59859173620695305, 0.72649190091814697, 0.44926682636298154], [0.04851453171235165, -0.9672918515015474, -0.230126974255626, 0.82817952796288097]]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[[-0.70607704048865649, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.75548953091054183, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254]], [[-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.68845438476518916]]], [[[-0.60308500682709254, -0.87617183460935077, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.83467085878218961, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254]], [[-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.72927205003089735, -0.60308500682709254, -0.60308500682709254, -0.84290814078559761], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254]]], [[[-0.60308500682709254, -0.98169089874803594, -0.60308500682709254, -0.99372717001525279], [-0.60308500682709254, -0.99945790763342135, -0.9487593851596996, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254]], [[-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.60308500682709254, -0.60308500682709254, -0.60308500682709254], [-0.60308500682709254, -0.9672918515015474, -0.60308500682709254, -0.60308500682709254]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_Symbol_rank4(self):
      arg0=-0.547775254003
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[0.94563717317463669, 0.085018636247385926, -0.35111838060064526, -0.94546995829506186], [-0.96270429050300343, 0.95303333299008575, -0.9560559260847874, -0.16757586875723862], [-0.66709058162177493, -0.66884130878394332, 0.59879979128821748, 0.41353125589037254]], [[-0.85530360776685899, -0.17834180470063354, -0.90733689234716053, 0.44908792593729885], [-0.77887513684760812, -0.76726305470368539, 0.28965951946167401, 0.82018644596730872], [0.42184983776004148, -0.17127589026081358, 0.62146272010891135, -0.35457623802143323]]], [[[0.30492139420967157, 0.53679161911737228, 0.14844065168563914, 0.15314985247092672], [0.45593167482004393, -0.17221199826759803, -0.35870795795696986, -0.9035950989665984], [-0.55530638040637248, -0.82169652829431894, -0.57091196698301427, 0.88874079482303636]], [[0.56907815087550495, -0.044310109383250973, 0.84443479742483718, 0.3680280178088573], [-0.42946285345553381, 0.090792786230326161, -0.76195332771501989, -0.041045557504301167], [-0.22323466109737455, 0.86476347571916201, -0.84170195238191647, -0.077947108514334618]]], [[[-0.44709122841949833, -0.49793909400969549, -0.4733167256854105, 0.45477465087062696], [-0.56750632242349131, 0.71902291396422968, -0.85248801895660709, 0.47326705718436779], [-0.59998638332003851, 0.68199746749165957, 0.51104336317144705, 0.9082862989279763]], [[-0.40033650496772477, -0.35997993320302424, 0.59694076516233641, -0.7837152570650634], [-0.72552882811112562, 0.34832450287472927, 0.80670270090898688, -0.60074212372300662], [0.78278479447822669, -0.87601559729427003, 0.86401769869353129, 0.16604791649833772]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.94546995829506186], [-0.96270429050300343, -0.5477752540032248, -0.9560559260847874, -0.5477752540032248], [-0.66709058162177493, -0.66884130878394332, -0.5477752540032248, -0.5477752540032248]], [[-0.85530360776685899, -0.5477752540032248, -0.90733689234716053, -0.5477752540032248], [-0.77887513684760812, -0.76726305470368539, -0.5477752540032248, -0.5477752540032248], [-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.5477752540032248]]], [[[-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.5477752540032248], [-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.9035950989665984], [-0.55530638040637248, -0.82169652829431894, -0.57091196698301427, -0.5477752540032248]], [[-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.5477752540032248], [-0.5477752540032248, -0.5477752540032248, -0.76195332771501989, -0.5477752540032248], [-0.5477752540032248, -0.5477752540032248, -0.84170195238191647, -0.5477752540032248]]], [[[-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.5477752540032248], [-0.56750632242349131, -0.5477752540032248, -0.85248801895660709, -0.5477752540032248], [-0.59998638332003851, -0.5477752540032248, -0.5477752540032248, -0.5477752540032248]], [[-0.5477752540032248, -0.5477752540032248, -0.5477752540032248, -0.7837152570650634], [-0.72552882811112562, -0.5477752540032248, -0.5477752540032248, -0.60074212372300662], [-0.5477752540032248, -0.87601559729427003, -0.5477752540032248, -0.5477752540032248]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_constData_rank4(self):
      arg0=-0.581792947884
      arg1=Data(numpy.array([[[[0.87747226519015875, 0.6231426348902005, -0.3932757823780173, 0.67597306605245411], [0.92013106431870528, 0.60546278260755204, 0.58894428300734991, 0.2230598410169915], [0.7187674472486103, 0.65740312497866116, 0.066181623539462109, -0.032620866916025282]], [[-0.89637094500210379, -0.15642889276419836, 0.73313064244502302, 0.30877465905136403], [0.81586268596284639, -0.2219566251724403, 0.53103247253907337, -0.44215014316465151], [0.11755306734261484, 0.31433786581830625, -0.80401777975618272, 0.5146635941296418]]], [[[0.25999133341202407, 0.41054339747028967, 0.97464606783013896, 0.76696477751629222], [-0.86950974391029412, 0.087870184538882512, -0.084176512771706236, -0.99250158443436454], [-0.56396487277988316, -0.44981899824167204, 0.37907997865231202, -0.13846599928934089]], [[0.37241467604039702, 0.28102865211565753, -0.24431836156486675, -0.10719573858498843], [0.56675159336001268, 0.068143087890095044, 0.19812077662531657, -0.4370159846273598], [-0.16895140057040403, -0.76247762793663942, -0.22872027382334315, 0.92369238513446073]]], [[[-0.63173384125877408, 0.33275973707979567, 0.12580695086856153, -0.19807779864906205], [0.50013208954093913, -0.4238507591084324, -0.44016338346600081, -0.079960057043174881], [0.87220448346661161, -0.16431705938389696, 0.23619941013837775, 0.39692507332646398]], [[0.059734965680461105, -0.64303318483765759, -0.33036924826159075, -0.8397929579564527], [0.84754199125021534, 0.3277818253539122, 0.24711249515614564, 0.86244655028076544], [-0.18611073492048535, -0.53349032076852021, -0.81648674433785762, -0.76088310405458248]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652]], [[-0.89637094500210379, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.80401777975618272, -0.58179294788395652]]], [[[-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.86950974391029412, -0.58179294788395652, -0.58179294788395652, -0.99250158443436454], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652]], [[-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.76247762793663942, -0.58179294788395652, -0.58179294788395652]]], [[[-0.63173384125877408, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652]], [[-0.58179294788395652, -0.64303318483765759, -0.58179294788395652, -0.8397929579564527], [-0.58179294788395652, -0.58179294788395652, -0.58179294788395652, -0.58179294788395652], [-0.58179294788395652, -0.58179294788395652, -0.81648674433785762, -0.76088310405458248]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_float_rank0_expandedData_rank4(self):
      arg0=-0.10290430648
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.85187225610746631, 0.64558385225605552, 0.36117210026029056, -0.03422553197228817], [0.24323566750290659, -0.57877387738099317, 0.52697125903685293, -0.93181939519771917], [0.53731967233777023, -0.72710062296494615, 0.97368033040394164, -0.53409182293530799]], [[0.82844548806310003, 0.5213969124995439, -0.27770318177172659, -0.17619831855758017], [0.48771787591444626, 0.93796688405206363, 0.025437621104899666, 0.19876953601259006], [-0.040481365114872991, 0.89537512768984895, -0.81935917804915803, -0.99907977569170137]]], [[[-0.48987287243545707, 0.61994560528887011, 0.55888316071892397, -0.77637054376351999], [-0.66089259237922282, -0.88068129189395061, 0.93239068267859482, -0.78746314792351146], [-0.14315793732359605, 0.41361971509445605, 0.95568183192572898, -0.36880629855877523]], [[0.2172025525332788, -0.34936830028158439, 0.57184924406041859, -0.23412108331402326], [-0.23445523796533108, 0.19578577497365091, 0.14009352718615009, 0.14133228086993066], [0.10020812115061251, -0.33883877747695057, 0.20560447740398247, -0.77600027604688715]]], [[[-0.4942915852751073, -0.29235022442187919, -0.51116999803212382, 0.37093582472300968], [-0.27376810487262837, -0.8987464689482163, -0.87245236617626798, 0.21392735696162135], [0.89481650900006993, 0.72356062177150782, 0.026226541387195601, 0.045736178732056709]], [[0.3686576166289619, -0.23135459106690015, 0.81774316603276498, -0.44173564156250134], [-0.98798277603952345, 0.11479044811880423, -0.63649413700127844, 0.88953686149105238], [0.22524856598321619, 0.69039006747762377, 0.28184364889788771, -0.7480606303977928]]]])+(1.-msk_arg1)*numpy.array([[[[-0.8455279216269902, -0.43871740189967201, -0.093114977650723629, 0.93535789029724681], [0.19668699619808239, -0.63153668349117908, -0.30409206579554415, -0.16200487448713541], [0.31762815788635312, 0.64257458918316024, 0.30369800481028641, 0.096271411370706828]], [[-0.3519302369911308, -0.73908595879854055, 0.45561936679827331, 0.14072878601025618], [0.50517314547955894, 0.76234026364558671, 0.26197719351314808, -0.80972622054330379], [-0.45574391278975557, -0.50005275856572107, 0.12476127070454868, 0.53226113313979795]]], [[[0.80337843968115918, -0.5107867211640178, -0.48889196777448052, 0.84244453195505953], [-0.58857634025677297, 0.088235543304351349, 0.74319890254648469, -0.50361561223490159], [0.0011302964934849502, -0.04113483334572976, 0.2937309133365209, 0.82522969217995268]], [[0.71359750060924054, 0.722636991660494, -0.80351833294633779, 0.87106029082868752], [-0.32816347153535119, -0.65528138827757654, 0.55150801541119088, -0.81367209614811142], [0.45181211779440011, -0.5765470015831522, -0.90927890301572711, -0.22101251340165429]]], [[[-0.94593773582688456, 0.95215960064055705, 0.38466411752637231, 0.9820597265068487], [0.14778124466184317, -0.47015621926212225, 0.088603825063333419, -0.64973922336995482], [0.92791653906193416, 0.19050873211481267, 0.0016011818744656736, -0.46740459879994556]], [[0.38875790905781193, 0.66071108129954692, 0.57881914215991492, 0.56584702828234157], [0.098816526092418977, 0.82921905428706588, 0.6807498250186037, 0.56130783810878215], [-0.50006331453622477, -0.42728851210218832, -0.91864985916115116, -0.58556436060393446]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.57877387738099317, -0.1029043064799362, -0.93181939519771917], [-0.1029043064799362, -0.72710062296494615, -0.1029043064799362, -0.53409182293530799]], [[-0.1029043064799362, -0.1029043064799362, -0.27770318177172659, -0.17619831855758017], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.1029043064799362, -0.81935917804915803, -0.99907977569170137]]], [[[-0.48987287243545707, -0.1029043064799362, -0.1029043064799362, -0.77637054376351999], [-0.66089259237922282, -0.88068129189395061, -0.1029043064799362, -0.78746314792351146], [-0.14315793732359605, -0.1029043064799362, -0.1029043064799362, -0.36880629855877523]], [[-0.1029043064799362, -0.34936830028158439, -0.1029043064799362, -0.23412108331402326], [-0.23445523796533108, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.33883877747695057, -0.1029043064799362, -0.77600027604688715]]], [[[-0.4942915852751073, -0.29235022442187919, -0.51116999803212382, -0.1029043064799362], [-0.27376810487262837, -0.8987464689482163, -0.87245236617626798, -0.1029043064799362], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362]], [[-0.1029043064799362, -0.23135459106690015, -0.1029043064799362, -0.44173564156250134], [-0.98798277603952345, -0.1029043064799362, -0.63649413700127844, -0.1029043064799362], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.7480606303977928]]]])+(1.-msk_ref)*numpy.array([[[[-0.8455279216269902, -0.43871740189967201, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.63153668349117908, -0.30409206579554415, -0.16200487448713541], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362]], [[-0.3519302369911308, -0.73908595879854055, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.80972622054330379], [-0.45574391278975557, -0.50005275856572107, -0.1029043064799362, -0.1029043064799362]]], [[[-0.1029043064799362, -0.5107867211640178, -0.48889196777448052, -0.1029043064799362], [-0.58857634025677297, -0.1029043064799362, -0.1029043064799362, -0.50361561223490159], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362]], [[-0.1029043064799362, -0.1029043064799362, -0.80351833294633779, -0.1029043064799362], [-0.32816347153535119, -0.65528138827757654, -0.1029043064799362, -0.81367209614811142], [-0.1029043064799362, -0.5765470015831522, -0.90927890301572711, -0.22101251340165429]]], [[[-0.94593773582688456, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.47015621926212225, -0.1029043064799362, -0.64973922336995482], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.46740459879994556]], [[-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.1029043064799362, -0.1029043064799362, -0.1029043064799362, -0.1029043064799362], [-0.50006331453622477, -0.42728851210218832, -0.91864985916115116, -0.58556436060393446]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_float_rank0(self):
      arg0=numpy.array(0.857550779512)
      arg1=-0.931803549177
      res=minimum(arg0,arg1)
      ref=numpy.array(-0.931803549177)
      if isinstance(res,numpy.ndarray):
          self.assertEqual(res.shape,(),"wrong shape of result.")
      else:
          self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_float_rank0(self):
      arg0=numpy.array([-0.3787763125444279, 0.61640691487825783])
      arg1=0.94517726501
      res=minimum(arg0,arg1)
      ref=numpy.array([-0.3787763125444279, 0.61640691487825783])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_float_rank0(self):
      arg0=numpy.array([[0.73487364184043158, 0.85512732743605779, -0.33783827568479152, 0.052087941059582255, -0.9768667842826757], [0.49681750240627554, -0.1673990705734647, 0.34496840412171048, -0.55168843110085031, 0.96005491316714942], [-0.17374074046696575, -0.00020542667606160414, 0.83993824499041558, 0.26937612547408407, 0.62423339248770437], [0.46916929579476263, 0.027976087899955537, 0.56428053215234608, 0.0037181200269902348, -0.82795822733434399]])
      arg1=0.774188926122
      res=minimum(arg0,arg1)
      ref=numpy.array([[0.73487364184043158, 0.77418892612235446, -0.33783827568479152, 0.052087941059582255, -0.9768667842826757], [0.49681750240627554, -0.1673990705734647, 0.34496840412171048, -0.55168843110085031, 0.77418892612235446], [-0.17374074046696575, -0.00020542667606160414, 0.77418892612235446, 0.26937612547408407, 0.62423339248770437], [0.46916929579476263, 0.027976087899955537, 0.56428053215234608, 0.0037181200269902348, -0.82795822733434399]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_float_rank0(self):
      arg0=numpy.array([[[-0.46087902964363803, -0.57129195776970931], [0.0076277080182611812, -0.41537987414122468]], [[-0.13192465918260998, -0.812803248157564], [0.36381513982996605, -0.9997332266509058]], [[0.70174669839708037, 0.024330102842379686], [0.81487919329588299, -0.52678709995947504]], [[-0.72411149617151471, 0.70679642938999798], [-0.3498991701900156, -0.96024889494342114]], [[-0.06909535466653538, 0.1962226565582319], [-0.86504927743940008, 0.26157135071251991]], [[0.84244033106704874, 0.92704263636113993], [-0.95355033204340445, 0.26248819447807525]]])
      arg1=0.943913650475
      res=minimum(arg0,arg1)
      ref=numpy.array([[[-0.46087902964363803, -0.57129195776970931], [0.0076277080182611812, -0.41537987414122468]], [[-0.13192465918260998, -0.812803248157564], [0.36381513982996605, -0.9997332266509058]], [[0.70174669839708037, 0.024330102842379686], [0.81487919329588299, -0.52678709995947504]], [[-0.72411149617151471, 0.70679642938999798], [-0.3498991701900156, -0.96024889494342114]], [[-0.06909535466653538, 0.1962226565582319], [-0.86504927743940008, 0.26157135071251991]], [[0.84244033106704874, 0.92704263636113993], [-0.95355033204340445, 0.26248819447807525]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_float_rank0(self):
      arg0=numpy.array([[[[-0.83625307245463865, -0.98092419056114721, -0.41276929014027663, 0.22870779277970277], [0.061813298478873957, 0.68068220259497325, -0.59242034530039533, -0.23160001258621188], [0.84245839390738309, -0.69364898209393511, 0.27911312262812582, 0.52734351530235135]], [[-0.29691958348502956, -0.27360051966730414, -0.75953910304979599, -0.50280490074627293], [0.20366134391546309, 0.61433073050478693, 0.12105359319633124, -0.47500250514633802], [0.10283925179231024, 0.31934023713999782, 0.32672660137224585, -0.88915465274000738]]], [[[0.64044241730765195, -0.58502981647791041, 0.29232033967124815, 0.49761920262791626], [0.19279011387703693, -0.90954455094873077, 0.46668774191172258, 0.53332298417661672], [-0.65415679726684228, 0.048987662787866482, -0.5670428493706543, 0.41710263147312898]], [[0.88536193649022077, -0.58797937643338116, -0.58729792385186452, 0.51901779786057145], [-0.2132527448536401, 0.47436958605338297, 0.45876189062645278, -0.25260854832951041], [-0.9043252274086746, -0.21672385749114187, -0.051211493465172175, -0.75228882024605115]]], [[[-0.52436255644704577, -0.72862396222744175, 0.69400005913482521, -0.23999690561865661], [-0.29737752504935466, 0.55286786193362603, 0.95036177416639966, -0.95894560840181464], [0.56248187365701585, -0.14169630012641865, -0.55808511422967433, -0.98783111835212356]], [[-0.74629979771557209, -0.98792590326574592, 0.28466555035646102, -0.6908222095481058], [0.52849233626894532, 0.51479121086295399, 0.79278677375459461, -0.64298385481196307], [0.81646839664183801, 0.91859825266245809, -0.65957056952356319, -0.10172208823781093]]]])
      arg1=0.805915464943
      res=minimum(arg0,arg1)
      ref=numpy.array([[[[-0.83625307245463865, -0.98092419056114721, -0.41276929014027663, 0.22870779277970277], [0.061813298478873957, 0.68068220259497325, -0.59242034530039533, -0.23160001258621188], [0.80591546494255284, -0.69364898209393511, 0.27911312262812582, 0.52734351530235135]], [[-0.29691958348502956, -0.27360051966730414, -0.75953910304979599, -0.50280490074627293], [0.20366134391546309, 0.61433073050478693, 0.12105359319633124, -0.47500250514633802], [0.10283925179231024, 0.31934023713999782, 0.32672660137224585, -0.88915465274000738]]], [[[0.64044241730765195, -0.58502981647791041, 0.29232033967124815, 0.49761920262791626], [0.19279011387703693, -0.90954455094873077, 0.46668774191172258, 0.53332298417661672], [-0.65415679726684228, 0.048987662787866482, -0.5670428493706543, 0.41710263147312898]], [[0.80591546494255284, -0.58797937643338116, -0.58729792385186452, 0.51901779786057145], [-0.2132527448536401, 0.47436958605338297, 0.45876189062645278, -0.25260854832951041], [-0.9043252274086746, -0.21672385749114187, -0.051211493465172175, -0.75228882024605115]]], [[[-0.52436255644704577, -0.72862396222744175, 0.69400005913482521, -0.23999690561865661], [-0.29737752504935466, 0.55286786193362603, 0.80591546494255284, -0.95894560840181464], [0.56248187365701585, -0.14169630012641865, -0.55808511422967433, -0.98783111835212356]], [[-0.74629979771557209, -0.98792590326574592, 0.28466555035646102, -0.6908222095481058], [0.52849233626894532, 0.51479121086295399, 0.79278677375459461, -0.64298385481196307], [0.80591546494255284, 0.80591546494255284, -0.65957056952356319, -0.10172208823781093]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_array_rank0(self):
      arg0=numpy.array(0.389872782207)
      arg1=numpy.array(0.770887964108)
      res=minimum(arg0,arg1)
      ref=numpy.array(0.389872782207)
      diff=add(arg1, -arg0)
      out=add(arg0,mult(whereNegative(diff),diff))
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_array_rank0(self):
      arg0=numpy.array([0.62053337242222106, 0.9378071268909014])
      arg1=numpy.array(-0.00670569008167)
      res=minimum(arg0,arg1)
      ref=numpy.array([-0.0067056900816702125, -0.0067056900816702125])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_array_rank0(self):
      arg0=numpy.array([[-0.53581508774705489, -0.81694620054214262, 0.10622104587034653, 0.71457914938406719, 0.13736693490304352], [-0.96860495698991578, 0.11246557480273189, -0.25211716028434417, -0.75842206367277676, 0.46930135143129736], [-0.7392131609723176, 0.62472685015314267, -0.85230854677453771, -0.68801606930350068, -0.072796679374400153], [-0.17581662031575007, -0.40120307594212457, 0.087091851442376189, 0.48178507094534528, 0.68553402521876916]])
      arg1=numpy.array(0.909515806974)
      res=minimum(arg0,arg1)
      ref=numpy.array([[-0.53581508774705489, -0.81694620054214262, 0.10622104587034653, 0.71457914938406719, 0.13736693490304352], [-0.96860495698991578, 0.11246557480273189, -0.25211716028434417, -0.75842206367277676, 0.46930135143129736], [-0.7392131609723176, 0.62472685015314267, -0.85230854677453771, -0.68801606930350068, -0.072796679374400153], [-0.17581662031575007, -0.40120307594212457, 0.087091851442376189, 0.48178507094534528, 0.68553402521876916]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_array_rank0(self):
      arg0=numpy.array([[[-0.60423663694916696, 0.81178484746809043], [0.40721555680054, 0.96464766417481118]], [[0.83267825176593901, -0.114463937229067], [0.074552643403738905, 0.90263895861717436]], [[-0.83917862228856999, 0.29580567543478775], [0.7356101992386741, 0.7081218375203]], [[0.32914154435747078, -0.45511477742165662], [-0.27307419370718855, -0.71710400533929675]], [[0.49887556634883579, 0.10645599573484876], [-0.54797030132655555, -0.024678156116086036]], [[-0.23798655863607632, -0.71155424564324576], [-0.50580795685584201, -0.6367245911624706]]])
      arg1=numpy.array(-0.203164817525)
      res=minimum(arg0,arg1)
      ref=numpy.array([[[-0.60423663694916696, -0.20316481752504534], [-0.20316481752504534, -0.20316481752504534]], [[-0.20316481752504534, -0.20316481752504534], [-0.20316481752504534, -0.20316481752504534]], [[-0.83917862228856999, -0.20316481752504534], [-0.20316481752504534, -0.20316481752504534]], [[-0.20316481752504534, -0.45511477742165662], [-0.27307419370718855, -0.71710400533929675]], [[-0.20316481752504534, -0.20316481752504534], [-0.54797030132655555, -0.20316481752504534]], [[-0.23798655863607632, -0.71155424564324576], [-0.50580795685584201, -0.6367245911624706]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_array_rank0(self):
      arg0=numpy.array([[[[-0.88409627233633015, -0.24180880044828168, -0.16109379951963732, 0.054780850074784615], [0.088497394752413916, 0.12739685401428691, 0.91613533694130633, -0.12330788977174723], [0.59156881341898626, 0.097883418078583251, -0.42019448331551068, 0.93499807156448056]], [[-0.28124888615332799, -0.070680388991419818, -0.98156318945166121, -0.30427047986189004], [-0.69697011679708165, 0.45690333366086189, 0.29894632104366292, -0.045313440763791402], [0.014105507577271581, -0.73962116497441976, 0.64459249185291112, -0.65033503935731996]]], [[[0.080118695849563437, -0.98565794524601902, -0.30068448750058363, 0.39824415379508893], [-0.26040900506020015, 0.34311381611429415, 0.85968572379571007, 0.10451592223118], [-0.5900350618330874, 0.39587445892497297, -0.39705501036035828, -0.76595207715516089]], [[0.87009079679190982, 0.44293188122529714, 0.2692810520962956, 0.84644072410474203], [0.080851462622369974, 0.065149905551311393, -0.33718206162615094, 0.032794151371295488], [-0.68745989338873525, -0.67986231166762789, 0.5482539578904575, -0.25593960686285988]]], [[[-0.46618713532480949, -0.3468711581159647, -0.58787716751812269, -0.9476946148329235], [-0.28418210099892649, -0.53141237500870053, 0.9814823709551237, -0.21408035592456343], [-0.38038762215406852, 0.53764354138724557, -0.8732296453732491, -0.68309253564894012]], [[-0.93251515388706663, 0.13721073537317197, 0.90279717503452539, -0.69626879529176033], [0.93285049558560673, 0.62824431407597237, 0.77017740778577104, -0.68709117985967616], [-0.36529411543321433, -0.56693930195860442, 0.77404342041484209, -0.77208297857019681]]]])
      arg1=numpy.array(0.860202631207)
      res=minimum(arg0,arg1)
      ref=numpy.array([[[[-0.88409627233633015, -0.24180880044828168, -0.16109379951963732, 0.054780850074784615], [0.088497394752413916, 0.12739685401428691, 0.86020263120695017, -0.12330788977174723], [0.59156881341898626, 0.097883418078583251, -0.42019448331551068, 0.86020263120695017]], [[-0.28124888615332799, -0.070680388991419818, -0.98156318945166121, -0.30427047986189004], [-0.69697011679708165, 0.45690333366086189, 0.29894632104366292, -0.045313440763791402], [0.014105507577271581, -0.73962116497441976, 0.64459249185291112, -0.65033503935731996]]], [[[0.080118695849563437, -0.98565794524601902, -0.30068448750058363, 0.39824415379508893], [-0.26040900506020015, 0.34311381611429415, 0.85968572379571007, 0.10451592223118], [-0.5900350618330874, 0.39587445892497297, -0.39705501036035828, -0.76595207715516089]], [[0.86020263120695017, 0.44293188122529714, 0.2692810520962956, 0.84644072410474203], [0.080851462622369974, 0.065149905551311393, -0.33718206162615094, 0.032794151371295488], [-0.68745989338873525, -0.67986231166762789, 0.5482539578904575, -0.25593960686285988]]], [[[-0.46618713532480949, -0.3468711581159647, -0.58787716751812269, -0.9476946148329235], [-0.28418210099892649, -0.53141237500870053, 0.86020263120695017, -0.21408035592456343], [-0.38038762215406852, 0.53764354138724557, -0.8732296453732491, -0.68309253564894012]], [[-0.93251515388706663, 0.13721073537317197, 0.86020263120695017, -0.69626879529176033], [0.86020263120695017, 0.62824431407597237, 0.77017740778577104, -0.68709117985967616], [-0.36529411543321433, -0.56693930195860442, 0.77404342041484209, -0.77208297857019681]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_Symbol_rank0(self):
      arg0=numpy.array(-0.589989553137)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.16116509338)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-0.589989553137)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_Symbol_rank0(self):
      arg0=numpy.array([-0.795584704645244, 0.58480072643935732])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.0189026195962)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.795584704645244, -0.018902619596217862])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_Symbol_rank0(self):
      arg0=numpy.array([[0.98871260359194446, -0.89072605894894341, 0.86498378368955287, -0.60271164558937085, -0.85794843534870391], [0.73977614650450874, 0.50606460005050824, -0.92461110212529762, -0.46905554358711132, -0.74239222415318573], [-0.44477632358946262, -0.86477850743482176, 0.62820722849292232, 0.76481483697824726, 0.27628667776844007], [0.25855739349300677, -0.46378715831138706, 0.77272492541599114, -0.6692293239612197, 0.065184783984226113]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.975377330914)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218], [-0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218], [-0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218], [-0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218, -0.97537733091367218]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_Symbol_rank0(self):
      arg0=numpy.array([[[-0.63726268067598002, 0.51533896886654662], [0.31931361643960399, 0.12421954089616039]], [[-0.41826300303511332, 0.21808917362194213], [-0.22685813005782784, 0.032987671390316775]], [[0.78595611950831179, -0.05026676711440059], [0.57031203644261153, -0.39560310730377735]], [[-0.58121613037110054, 0.14484792357611753], [0.32869116918354946, 0.19159927226108109]], [[-0.22055670271672012, 0.72397474215090418], [0.50472056892009531, -0.75047666407904323]], [[0.65171254846182203, 0.64945524580665048], [-0.49810161852076185, 0.82100479948739613]]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.316954281904)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.63726268067598002, 0.31695428190447883], [0.31695428190447883, 0.12421954089616039]], [[-0.41826300303511332, 0.21808917362194213], [-0.22685813005782784, 0.032987671390316775]], [[0.31695428190447883, -0.05026676711440059], [0.31695428190447883, -0.39560310730377735]], [[-0.58121613037110054, 0.14484792357611753], [0.31695428190447883, 0.19159927226108109]], [[-0.22055670271672012, 0.31695428190447883], [0.31695428190447883, -0.75047666407904323]], [[0.31695428190447883, 0.31695428190447883], [-0.49810161852076185, 0.31695428190447883]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_Symbol_rank0(self):
      arg0=numpy.array([[[[0.89366137430438575, -0.27017051148467597, 0.73597991268948793, 0.14234748208450343], [0.29588439802036781, 0.47617291526058381, -0.99353965547121326, 0.8865470784923104], [-0.44726996204385472, -0.05154657705644583, 0.34578572400888508, -0.24586567781595514]], [[0.84064422867891953, 0.91380313292900595, -0.045085556251011072, -0.11526776421649476], [-0.51543692650579809, 0.88189412631353892, 0.8588840179928483, -0.83196921065495255], [-0.91533703660193755, -0.38344485839633347, -0.31245129554028872, 0.56882947284639318]]], [[[0.62201865500174236, 0.67596286539329653, -0.94676984138511511, 0.10026539518463529], [-0.13157968507011919, 0.53112399233900565, 0.90819122064811375, 0.61032450809679606], [0.93829396112967411, 0.31555099430715061, -0.64497924720427791, -0.4235531122043672]], [[-0.71707188532302801, -0.39203083210553547, -0.87158656439122639, -0.20777519996785609], [-0.68776369081670508, 0.96395711280383067, 0.48153973742632372, 0.8375398015736395], [-0.89563764605418528, 0.32660506676409984, -0.13244365803586344, 0.0062243303918554371]]], [[[0.64249386078279791, -0.82116997542869918, -0.46581380139791939, 0.76412840190660636], [-0.64956086181368144, 0.34646174960891241, -0.19906807034835605, -0.78425281673753666], [0.36115886780697504, 0.92896226684364169, 0.090263153575377952, 0.23731132376303443]], [[0.92603801339530523, 0.20437981024603813, 0.062390915054103324, -0.074960421645424491], [0.90353414862632642, -0.089160016434783662, 0.87220847106964872, -0.82916751824668045], [-0.12220813955361276, 0.24581722647389337, 0.74155215669888563, 0.79823296235192376]]]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.45127628568)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.4512762856797452, -0.27017051148467597, 0.4512762856797452, 0.14234748208450343], [0.29588439802036781, 0.4512762856797452, -0.99353965547121326, 0.4512762856797452], [-0.44726996204385472, -0.05154657705644583, 0.34578572400888508, -0.24586567781595514]], [[0.4512762856797452, 0.4512762856797452, -0.045085556251011072, -0.11526776421649476], [-0.51543692650579809, 0.4512762856797452, 0.4512762856797452, -0.83196921065495255], [-0.91533703660193755, -0.38344485839633347, -0.31245129554028872, 0.4512762856797452]]], [[[0.4512762856797452, 0.4512762856797452, -0.94676984138511511, 0.10026539518463529], [-0.13157968507011919, 0.4512762856797452, 0.4512762856797452, 0.4512762856797452], [0.4512762856797452, 0.31555099430715061, -0.64497924720427791, -0.4235531122043672]], [[-0.71707188532302801, -0.39203083210553547, -0.87158656439122639, -0.20777519996785609], [-0.68776369081670508, 0.4512762856797452, 0.4512762856797452, 0.4512762856797452], [-0.89563764605418528, 0.32660506676409984, -0.13244365803586344, 0.0062243303918554371]]], [[[0.4512762856797452, -0.82116997542869918, -0.46581380139791939, 0.4512762856797452], [-0.64956086181368144, 0.34646174960891241, -0.19906807034835605, -0.78425281673753666], [0.36115886780697504, 0.4512762856797452, 0.090263153575377952, 0.23731132376303443]], [[0.4512762856797452, 0.20437981024603813, 0.062390915054103324, -0.074960421645424491], [0.4512762856797452, -0.089160016434783662, 0.4512762856797452, -0.82916751824668045], [-0.12220813955361276, 0.24581722647389337, 0.4512762856797452, 0.4512762856797452]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_constData_rank0(self):
      arg0=numpy.array(-0.664721644752)
      arg1=Data(-0.591271306444,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(-0.664721644752,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_constData_rank0(self):
      arg0=numpy.array([0.042987704765046875, -0.75971623114729758])
      arg1=Data(0.259779720844,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([0.042987704765046875, -0.75971623114729758]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_constData_rank0(self):
      arg0=numpy.array([[-0.40694840135770916, 0.014819351600515729, 0.22945217906940396, 0.24172192708217133, 0.93718430559874277], [-0.67233903053984378, 0.6303475818609956, 0.47580560616279821, 0.011270456829029385, 0.12521216351981534], [-0.34833156966200152, 0.77212022808581104, 0.062723415721219977, 0.23428298086606669, 0.88603401165303297], [-0.8932351864413437, 0.8135967320357218, 0.38936600077325179, -0.64561296346977559, 0.91200200131622999]])
      arg1=Data(-0.342763784081,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.40694840135770916, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094], [-0.67233903053984378, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094], [-0.34833156966200152, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094, -0.34276378408100094], [-0.8932351864413437, -0.34276378408100094, -0.34276378408100094, -0.64561296346977559, -0.34276378408100094]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_constData_rank0(self):
      arg0=numpy.array([[[-0.94517666245515719, 0.58346090245069915], [0.1183278909908978, -0.42796547087031644]], [[-0.14342455990554237, 0.2264971166960672], [0.52502555931526484, -0.68340476112619131]], [[0.19183738551063034, -0.42115644937505303], [0.47811884751893574, -0.93686472499067253]], [[0.4432181165912128, 0.35719501497132122], [0.94125080725572152, 0.46228585982612369]], [[0.72278047347309027, 0.50193474124708115], [-0.49919230064208908, -0.46046094534597826]], [[-0.87157976779416679, -0.057714519068007064], [-0.32308528611964626, -0.45683830566307004]]])
      arg1=Data(-0.305838120197,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.94517666245515719, -0.30583812019708456], [-0.30583812019708456, -0.42796547087031644]], [[-0.30583812019708456, -0.30583812019708456], [-0.30583812019708456, -0.68340476112619131]], [[-0.30583812019708456, -0.42115644937505303], [-0.30583812019708456, -0.93686472499067253]], [[-0.30583812019708456, -0.30583812019708456], [-0.30583812019708456, -0.30583812019708456]], [[-0.30583812019708456, -0.30583812019708456], [-0.49919230064208908, -0.46046094534597826]], [[-0.87157976779416679, -0.30583812019708456], [-0.32308528611964626, -0.45683830566307004]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_constData_rank0(self):
      arg0=numpy.array([[[[0.3052119132493083, 0.11354358001719866, -0.78678845999523728, -0.89713683920060361], [0.22313671955092262, -0.77524459473219154, -0.19689450126311048, -0.65921975688805134], [-0.32637449117625894, 0.93571751315249174, -0.033965491584570184, -0.095406238741338578]], [[-0.69558665665303332, 0.59193970427477627, 0.11950863130350098, -0.077954679843484698], [0.12794466376386215, 0.43377077999781344, 0.65045339412460534, 0.55334364692466331], [0.52135873564148949, 0.88714742787211742, 0.98151745723995343, 0.40097573933625963]]], [[[-0.77163074313229774, -0.18221408708888043, -0.75097864636239975, -0.17309967927973546], [0.41837092940580578, -0.43013478261701144, -0.86866401698486473, 0.61023733125886293], [0.40079818259749067, 0.38964383635805655, -0.86726724319815718, -0.63749061316478639]], [[-0.59777777836614443, -0.67040010737276701, 0.91135630031305537, -0.25724680669481215], [-0.59690625496551508, -0.31209228496860009, -0.78429011435216345, -0.70538350697491015], [0.24029207385364049, -0.063254594454844382, -0.087946871394203985, -0.23820668895538444]]], [[[-0.82311664743046276, 0.010040840100980608, -0.4060817484849244, -0.98411327160277295], [-0.84677009076546872, 0.77599780734071033, -0.29779780944522383, -0.49165390908579165], [-0.65193081384960094, 0.87337389590410597, 0.7267891901788428, 0.75166395878354964]], [[0.48654688816567582, -0.62133642270161471, -0.32769997280633167, 0.37264222787251011], [0.21125896682256107, -0.048121022159131721, 0.4442699768969478, -0.59237144025170818], [0.18742833104058154, -0.48877147524805387, -0.0059717660824947227, -0.94205779052527672]]]])
      arg1=Data(-0.619822489751,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.6198224897512703, -0.6198224897512703, -0.78678845999523728, -0.89713683920060361], [-0.6198224897512703, -0.77524459473219154, -0.6198224897512703, -0.65921975688805134], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703]], [[-0.69558665665303332, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703]]], [[[-0.77163074313229774, -0.6198224897512703, -0.75097864636239975, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.86866401698486473, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.86726724319815718, -0.63749061316478639]], [[-0.6198224897512703, -0.67040010737276701, -0.6198224897512703, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.78429011435216345, -0.70538350697491015], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703]]], [[[-0.82311664743046276, -0.6198224897512703, -0.6198224897512703, -0.98411327160277295], [-0.84677009076546872, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703], [-0.65193081384960094, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703]], [[-0.6198224897512703, -0.62133642270161471, -0.6198224897512703, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.6198224897512703], [-0.6198224897512703, -0.6198224897512703, -0.6198224897512703, -0.94205779052527672]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_expandedData_rank0(self):
      arg0=numpy.array(0.241014004012)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.457147006796)+(1.-msk_arg1)*(0.227139154357)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.457147006796)+(1.-msk_ref)*(0.227139154357)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_expandedData_rank0(self):
      arg0=numpy.array([0.43114521020975749, 0.25123749545997365])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.99599266468)+(1.-msk_arg1)*(-0.382405565433)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.99599266468023795, -0.99599266468023795])+(1.-msk_ref)*numpy.array([-0.3824055654327454, -0.3824055654327454])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_expandedData_rank0(self):
      arg0=numpy.array([[0.32956811869988312, -0.72127093781956364, 0.15029370486346361, -0.98944296051175962, 0.87237931970997074], [-0.85087910536284106, -0.86252552109077363, -0.82444622757962271, -0.39010362623932537, 0.37533351790526615], [0.070658492002692475, 0.20863035530184848, 0.87001918006677337, 0.2481886335923762, -0.7581726725142397], [-0.48197484007023994, 0.82658729741930448, 0.90458054560265877, 0.73803797500734669, -0.69060406577633482]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.706056822366)+(1.-msk_arg1)*(-0.0808000064869)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.70605682236550793, -0.72127093781956364, -0.70605682236550793, -0.98944296051175962, -0.70605682236550793], [-0.85087910536284106, -0.86252552109077363, -0.82444622757962271, -0.70605682236550793, -0.70605682236550793], [-0.70605682236550793, -0.70605682236550793, -0.70605682236550793, -0.70605682236550793, -0.7581726725142397], [-0.70605682236550793, -0.70605682236550793, -0.70605682236550793, -0.70605682236550793, -0.70605682236550793]])+(1.-msk_ref)*numpy.array([[-0.080800006486851039, -0.72127093781956364, -0.080800006486851039, -0.98944296051175962, -0.080800006486851039], [-0.85087910536284106, -0.86252552109077363, -0.82444622757962271, -0.39010362623932537, -0.080800006486851039], [-0.080800006486851039, -0.080800006486851039, -0.080800006486851039, -0.080800006486851039, -0.7581726725142397], [-0.48197484007023994, -0.080800006486851039, -0.080800006486851039, -0.080800006486851039, -0.69060406577633482]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_expandedData_rank0(self):
      arg0=numpy.array([[[0.21007160385608681, 0.11305037671936513], [0.1329485349489512, -0.37626478532743368]], [[-0.95527489621779949, 0.37237241085840544], [-0.33392087721063235, 0.62050805087855454]], [[-0.8205233599259667, 0.12652200692553306], [-0.5949694037862463, 0.19068506824209219]], [[-0.60519441327651968, -0.18185610588607126], [0.89518944895353969, -0.85635279227786532]], [[-0.52971437192678517, 0.24588120319937623], [0.15760323116388042, -0.870908007371749]], [[0.6586753617957728, -0.25550368902454612], [-0.95372243692221703, -0.071797083499290704]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.890951276982)+(1.-msk_arg1)*(0.623818404953)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.89095127698233023, -0.89095127698233023], [-0.89095127698233023, -0.89095127698233023]], [[-0.95527489621779949, -0.89095127698233023], [-0.89095127698233023, -0.89095127698233023]], [[-0.89095127698233023, -0.89095127698233023], [-0.89095127698233023, -0.89095127698233023]], [[-0.89095127698233023, -0.89095127698233023], [-0.89095127698233023, -0.89095127698233023]], [[-0.89095127698233023, -0.89095127698233023], [-0.89095127698233023, -0.89095127698233023]], [[-0.89095127698233023, -0.89095127698233023], [-0.95372243692221703, -0.89095127698233023]]])+(1.-msk_ref)*numpy.array([[[0.21007160385608681, 0.11305037671936513], [0.1329485349489512, -0.37626478532743368]], [[-0.95527489621779949, 0.37237241085840544], [-0.33392087721063235, 0.62050805087855454]], [[-0.8205233599259667, 0.12652200692553306], [-0.5949694037862463, 0.19068506824209219]], [[-0.60519441327651968, -0.18185610588607126], [0.62381840495341323, -0.85635279227786532]], [[-0.52971437192678517, 0.24588120319937623], [0.15760323116388042, -0.870908007371749]], [[0.62381840495341323, -0.25550368902454612], [-0.95372243692221703, -0.071797083499290704]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_expandedData_rank0(self):
      arg0=numpy.array([[[[0.5319546215297335, 0.014486017479623303, -0.54561251957725654, 0.70676057147864357], [0.95665516215206936, 0.42356636884393906, 0.39029118354891779, 0.44603954002421675], [0.92990219203288915, -0.90909317758589392, 0.46611081644715724, 0.29055430681920802]], [[-0.91155951283019276, 0.74250070159423065, -0.94260663726745997, 0.21713004866508911], [0.37717440578553885, 0.55086890867978577, 0.48044217937659273, 0.87125216315341114], [0.0012827116164471075, 0.10391331945326088, -0.89512521613322171, -0.63242089633718823]]], [[[0.99287218328082161, -0.60268837233942274, -0.45797847239881295, 0.78879545032188858], [-0.19319080141923073, 0.75615504801389899, 0.47818809342018209, -0.16124654985220377], [0.85197759536734896, 0.15549246283537355, 0.75129715031855548, -0.44165321471408414]], [[0.25971006353169246, 0.052116379050871187, 0.68949102137257401, -0.59886239358865256], [0.14346581310485762, -0.3117872771788206, 0.44953749857580161, 0.92090364630578914], [0.40751637305011257, -0.54069505182237121, -0.71213156573192404, 0.58153046320690138]]], [[[-0.42132810212390703, 0.48730929096977937, 0.19411109887127287, 0.099214783457927114], [-0.84318150658342517, -0.66492817632386658, -0.14704164995798408, -0.18152600563982224], [-0.21604880586706798, -0.56126901733033363, -0.24748282523677911, -0.84868545378868609]], [[0.13568526650716017, 0.96708538831757584, -0.15109262038747517, -0.46535759104069307], [0.049465625840152461, -0.22606211316804803, 0.052449869862204679, -0.23571546724487868], [-0.33605192256366734, 0.36904095940403669, -0.33972902717567965, -0.55168459126114211]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.778218875313)+(1.-msk_arg1)*(0.921171099156)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.90909317758589392, -0.77821887531322309, -0.77821887531322309]], [[-0.91155951283019276, -0.77821887531322309, -0.94260663726745997, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.89512521613322171, -0.77821887531322309]]], [[[-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309]], [[-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309]]], [[[-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.84318150658342517, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.84868545378868609]], [[-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309], [-0.77821887531322309, -0.77821887531322309, -0.77821887531322309, -0.77821887531322309]]]])+(1.-msk_ref)*numpy.array([[[[0.5319546215297335, 0.014486017479623303, -0.54561251957725654, 0.70676057147864357], [0.92117109915615503, 0.42356636884393906, 0.39029118354891779, 0.44603954002421675], [0.92117109915615503, -0.90909317758589392, 0.46611081644715724, 0.29055430681920802]], [[-0.91155951283019276, 0.74250070159423065, -0.94260663726745997, 0.21713004866508911], [0.37717440578553885, 0.55086890867978577, 0.48044217937659273, 0.87125216315341114], [0.0012827116164471075, 0.10391331945326088, -0.89512521613322171, -0.63242089633718823]]], [[[0.92117109915615503, -0.60268837233942274, -0.45797847239881295, 0.78879545032188858], [-0.19319080141923073, 0.75615504801389899, 0.47818809342018209, -0.16124654985220377], [0.85197759536734896, 0.15549246283537355, 0.75129715031855548, -0.44165321471408414]], [[0.25971006353169246, 0.052116379050871187, 0.68949102137257401, -0.59886239358865256], [0.14346581310485762, -0.3117872771788206, 0.44953749857580161, 0.92090364630578914], [0.40751637305011257, -0.54069505182237121, -0.71213156573192404, 0.58153046320690138]]], [[[-0.42132810212390703, 0.48730929096977937, 0.19411109887127287, 0.099214783457927114], [-0.84318150658342517, -0.66492817632386658, -0.14704164995798408, -0.18152600563982224], [-0.21604880586706798, -0.56126901733033363, -0.24748282523677911, -0.84868545378868609]], [[0.13568526650716017, 0.92117109915615503, -0.15109262038747517, -0.46535759104069307], [0.049465625840152461, -0.22606211316804803, 0.052449869862204679, -0.23571546724487868], [-0.33605192256366734, 0.36904095940403669, -0.33972902717567965, -0.55168459126114211]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_array_rank1(self):
      arg0=numpy.array(0.960254008827)
      arg1=numpy.array([-0.66609491116549435, -0.87378120113921232])
      res=minimum(arg0,arg1)
      ref=numpy.array([-0.66609491116549435, -0.87378120113921232])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_array_rank1(self):
      arg0=numpy.array([-0.38894166493230808, 0.20208487199063119])
      arg1=numpy.array([0.89827294190446416, -0.96322664887318887])
      res=minimum(arg0,arg1)
      ref=numpy.array([-0.38894166493230808, -0.96322664887318887])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_Symbol_rank1(self):
      arg0=numpy.array(-0.160174690351)
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([0.97799166757184608, -0.66553740967946506])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.16017469035123133, -0.66553740967946506])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_Symbol_rank1(self):
      arg0=numpy.array([0.95775374773539945, -0.20046400376073947])
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([-0.1337716266998179, 0.82122082364612647])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.1337716266998179, -0.20046400376073947])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_constData_rank1(self):
      arg0=numpy.array(-0.731395791021)
      arg1=Data(numpy.array([0.53510826895876917, 0.51633606525927722]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.7313957910208817, -0.7313957910208817]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_constData_rank1(self):
      arg0=numpy.array([0.1714742838097425, 0.66214623250405524])
      arg1=Data(numpy.array([0.79240518333092513, 0.38008637184861516]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([0.1714742838097425, 0.38008637184861516]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_expandedData_rank1(self):
      arg0=numpy.array(0.770633081927)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.8170469154149016, -0.29762829699380955])+(1.-msk_arg1)*numpy.array([-0.57189641056897145, -0.86898445344861885])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.8170469154149016, -0.29762829699380955])+(1.-msk_ref)*numpy.array([-0.57189641056897145, -0.86898445344861885])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank1_expandedData_rank1(self):
      arg0=numpy.array([0.41049279682053763, -0.53167200359367706])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.54148250128142039, 0.16845646697398187])+(1.-msk_arg1)*numpy.array([0.85939525034344966, 0.39108855742598903])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.41049279682053763, -0.53167200359367706])+(1.-msk_ref)*numpy.array([0.41049279682053763, -0.53167200359367706])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_array_rank2(self):
      arg0=numpy.array(0.886650683606)
      arg1=numpy.array([[-0.65399170339400814, 0.47980443873712031, -0.25085945423758371, -0.028943356192464353, 0.81546730268100753], [0.57632529126086918, -0.36854370936412906, 0.81416498341382249, 0.27632464907736343, 0.58458812649896674], [0.4716175113202945, 0.35448013516386245, 0.018767823745010626, 0.56206535311293981, -0.28073247599362805], [-0.78383718255395873, -0.77266624598659006, -0.46461735860239917, 0.44552501249197496, -0.28151470416065716]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[-0.65399170339400814, 0.47980443873712031, -0.25085945423758371, -0.028943356192464353, 0.81546730268100753], [0.57632529126086918, -0.36854370936412906, 0.81416498341382249, 0.27632464907736343, 0.58458812649896674], [0.4716175113202945, 0.35448013516386245, 0.018767823745010626, 0.56206535311293981, -0.28073247599362805], [-0.78383718255395873, -0.77266624598659006, -0.46461735860239917, 0.44552501249197496, -0.28151470416065716]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_array_rank2(self):
      arg0=numpy.array([[-0.33033503295609346, 0.14920008371124904, 0.76638872345598585, 0.67290758513209004, -0.13428745331417624], [-0.763692946784162, 0.60795873984430937, 0.56376621800116311, -0.90622233301415278, -0.95353924031714676], [0.11895544876173392, 0.73118052844668591, 0.89037593914185598, -0.17977961042776203, -0.95894626264533112], [-0.58212225400480455, 0.81630235390327277, -0.9290666495511597, 0.025047408496705836, -0.51157939784496564]])
      arg1=numpy.array([[0.61873938162772646, 0.49746335505769168, 0.063416602568223901, -0.061568244302447672, -0.99370907587651902], [-0.79347164177425111, 0.78351121005795665, 0.4527345542965644, -0.6309610156799359, 0.040831744033480666], [0.86011562640382211, -0.03710348102207317, 0.40344479967307523, -0.34196234788019897, -0.097843135118964364], [0.22327526174792123, -0.6270958938205442, -0.58788184415622369, -0.34140040204976252, -0.18752742848795667]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[-0.33033503295609346, 0.14920008371124904, 0.063416602568223901, -0.061568244302447672, -0.99370907587651902], [-0.79347164177425111, 0.60795873984430937, 0.4527345542965644, -0.90622233301415278, -0.95353924031714676], [0.11895544876173392, -0.03710348102207317, 0.40344479967307523, -0.34196234788019897, -0.95894626264533112], [-0.58212225400480455, -0.6270958938205442, -0.9290666495511597, -0.34140040204976252, -0.51157939784496564]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_Symbol_rank2(self):
      arg0=numpy.array(0.203246356478)
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[-0.32037576109780996, -0.8787290144206672, 0.84060668061176402, 0.3241617509588024, 0.2259347027143821], [0.88103810623886347, -0.59359665904081949, -0.39307862499963164, -0.29764874721864154, -0.96528851244823177], [0.18444279757222182, 0.17735089159515716, 0.36860980503725616, 0.77170115358695845, 0.095302526478923122], [-0.43568260137806281, 0.11297439463310988, -0.54068232357999091, -0.70272655235656978, 0.29203349722429217]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.32037576109780996, -0.8787290144206672, 0.20324635647778155, 0.20324635647778155, 0.20324635647778155], [0.20324635647778155, -0.59359665904081949, -0.39307862499963164, -0.29764874721864154, -0.96528851244823177], [0.18444279757222182, 0.17735089159515716, 0.20324635647778155, 0.20324635647778155, 0.095302526478923122], [-0.43568260137806281, 0.11297439463310988, -0.54068232357999091, -0.70272655235656978, 0.20324635647778155]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-0.53439659774438875, -0.24668292621735244, 0.68055249629635561, 0.41707868616411181, -0.00041925758239558242], [-0.66528377308490416, 0.24682268114094663, -0.60077771172865191, -0.3150360888718835, 0.82152579393086445], [0.35992942166911246, -0.56732347916232584, -0.16823553440068606, 0.073337608593839088, -0.092060179662548869], [-0.59966185643431813, -0.067626242642157441, 0.82673143899721313, -0.92572491543047186, 0.56736705313405489]])
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[-0.17728702639090388, 0.21961702857075149, 0.022580477526591647, 0.24849767598710204, -0.33421612539071366], [-0.97833291250145016, -0.16078774568966603, 0.41579955046001049, -0.23068221787073195, 0.11344518992589414], [-0.51801569046234897, 0.94581413237604428, 0.24625772962811432, 0.88458664730384307, -0.8957264822848725], [0.63026013284802063, -0.13974391670976938, -0.9846496556298785, -0.71814084258291455, -0.14419332897551995]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.53439659774438875, -0.24668292621735244, 0.022580477526591647, 0.24849767598710204, -0.33421612539071366], [-0.97833291250145016, -0.16078774568966603, -0.60077771172865191, -0.3150360888718835, 0.11344518992589414], [-0.51801569046234897, -0.56732347916232584, -0.16823553440068606, 0.073337608593839088, -0.8957264822848725], [-0.59966185643431813, -0.13974391670976938, -0.9846496556298785, -0.92572491543047186, -0.14419332897551995]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_constData_rank2(self):
      arg0=numpy.array(-0.372822602531)
      arg1=Data(numpy.array([[0.72628613344987181, 0.50327993112189007, 0.9193025455046322, 0.82868336373458495, 0.67610666756669557], [0.97218830704717463, -0.44810794051580394, -0.94732714933736517, 0.57150664067167622, 0.27183454929052608], [-0.9405370053109674, -0.96009187309070909, 0.73298247999572808, 0.22203108993302645, 0.35811406887935404], [-0.013576202508727597, -0.14720442130805989, 0.90781108331407112, -0.25737497489237193, 0.54985552366240609]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.37282260253057853, -0.37282260253057853, -0.37282260253057853, -0.37282260253057853, -0.37282260253057853], [-0.37282260253057853, -0.44810794051580394, -0.94732714933736517, -0.37282260253057853, -0.37282260253057853], [-0.9405370053109674, -0.96009187309070909, -0.37282260253057853, -0.37282260253057853, -0.37282260253057853], [-0.37282260253057853, -0.37282260253057853, -0.37282260253057853, -0.37282260253057853, -0.37282260253057853]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_constData_rank2(self):
      arg0=numpy.array([[0.78275358721638888, 0.013870539089549494, -0.32339075804088613, -0.027328231721210017, -0.099374020448430134], [-0.77823473646082952, 0.52402046446606088, 0.55663042341320157, 0.42491774012211403, 0.21493436248644882], [0.83326564594425867, -0.1260860351322981, 0.50885191862449197, -0.97006016463034173, 0.56992486824713295], [-0.71316955615260413, 0.42719756593962277, 0.29616224873037789, -0.072683279822802316, -0.8517359579542032]])
      arg1=Data(numpy.array([[0.66470003290035251, 0.18486555639964108, -0.98204547383856067, -0.15942552938494137, -0.14875789909341375], [-0.33159592694847673, 0.077392701340451175, 0.45890670062481798, -0.22535620257887934, -0.3310921066635728], [0.13803969146973882, -0.039958188696183949, 0.049577345690025254, 0.091617517260889958, -0.67195731085357058], [-0.34948904665046587, -0.17370967665836745, -0.37287470045721727, -0.14888793861918637, -0.29632405958706576]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[0.66470003290035251, 0.013870539089549494, -0.98204547383856067, -0.15942552938494137, -0.14875789909341375], [-0.77823473646082952, 0.077392701340451175, 0.45890670062481798, -0.22535620257887934, -0.3310921066635728], [0.13803969146973882, -0.1260860351322981, 0.049577345690025254, -0.97006016463034173, -0.67195731085357058], [-0.71316955615260413, -0.17370967665836745, -0.37287470045721727, -0.14888793861918637, -0.8517359579542032]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_expandedData_rank2(self):
      arg0=numpy.array(-0.637530102226)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.59807216885365033, -0.66094264631987287, 0.1467742033353252, 0.14926516531057832, 0.036308193213772766], [-0.98363286133358918, -0.58648307185312598, 0.094047680117578469, -0.07815794946211474, 0.59519966122415746], [0.67160795944833729, 0.87835497110541016, 0.53893624842926147, -0.50569954639711234, -0.88259518993263697], [-0.56480487916923394, -0.43716018144810365, 0.47779476696933854, 0.75402797028101309, -0.78550418646809828]])+(1.-msk_arg1)*numpy.array([[-0.67836057167916808, 0.25817773105522512, -0.55590752578847713, -0.37042155216568817, -0.38068486553076886], [-0.29957095557543933, -0.49327680200822166, 0.97116509867177037, -0.15461941909736865, 0.18640933745498134], [-0.062767091537753616, 0.37485538624605508, 0.85725224712282655, 0.76653901471472929, -0.24928692146813658], [0.59860836449271515, -0.69948672203405149, -0.45656755189509557, -0.30014445425305292, -0.30942585188514604]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.63753010222647744, -0.66094264631987287, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744], [-0.98363286133358918, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744], [-0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.88259518993263697], [-0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.78550418646809828]])+(1.-msk_ref)*numpy.array([[-0.67836057167916808, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744], [-0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744], [-0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744], [-0.63753010222647744, -0.69948672203405149, -0.63753010222647744, -0.63753010222647744, -0.63753010222647744]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-0.13611954010555682, -0.58442826873562659, 0.78399043478976482, -0.4804011172459175, -0.68188345765257696], [-0.18552675805111951, -0.29045979536132926, 0.15992135022530518, 0.16384701397334345, -0.65137403965964369], [0.51543557690911634, -0.97819571140752926, -0.96836272151235891, -0.13965398445259325, -0.31589396478709886], [-0.26685094001247323, 0.9990205085652557, 0.46747732907560868, -0.59994040792857128, -0.896108124510266]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.34894152894459185, 0.90762321494679665, -0.97878939335657056, 0.2745456947117868, 0.48803358804154273], [0.38716880605185122, -0.058351115663058506, 0.45162240402813536, -0.03794281391886134, 0.55481142352719881], [0.93601145392657181, -0.73735739898988362, -0.88202008227545714, -0.72566359273837544, -0.35070506745617847], [0.64345030423445149, 0.42641364278345173, -0.94607743162746849, -0.0096308587427624825, 0.56327698410566085]])+(1.-msk_arg1)*numpy.array([[-0.36711228363198467, 0.60993685278791365, 0.67388569157974287, -0.39140988838288182, -0.1945456360227622], [-0.33055765121925584, 0.35776313895404321, -0.87249443881683586, -0.58774954006405578, -0.29392346801996938], [0.29587592498579318, 0.057559557483068513, 0.52394023815596058, 0.49672424405480009, -0.97022178907476664], [0.42099470426519003, -0.093764421364879169, -0.9094771825968202, 0.73952907119560729, 0.89047956672450312]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.34894152894459185, -0.58442826873562659, -0.97878939335657056, -0.4804011172459175, -0.68188345765257696], [-0.18552675805111951, -0.29045979536132926, 0.15992135022530518, -0.03794281391886134, -0.65137403965964369], [0.51543557690911634, -0.97819571140752926, -0.96836272151235891, -0.72566359273837544, -0.35070506745617847], [-0.26685094001247323, 0.42641364278345173, -0.94607743162746849, -0.59994040792857128, -0.896108124510266]])+(1.-msk_ref)*numpy.array([[-0.36711228363198467, -0.58442826873562659, 0.67388569157974287, -0.4804011172459175, -0.68188345765257696], [-0.33055765121925584, -0.29045979536132926, -0.87249443881683586, -0.58774954006405578, -0.65137403965964369], [0.29587592498579318, -0.97819571140752926, -0.96836272151235891, -0.13965398445259325, -0.97022178907476664], [-0.26685094001247323, -0.093764421364879169, -0.9094771825968202, -0.59994040792857128, -0.896108124510266]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_array_rank3(self):
      arg0=numpy.array(-0.160330065964)
      arg1=numpy.array([[[0.66642437288411904, -0.9539277688704344], [-0.21909074438146647, 0.28178316529445091]], [[-0.96337470673707815, -0.04634602269617405], [-0.046220251565481041, 0.06720708926727248]], [[0.93281550648799061, 0.74175233295480569], [-0.49306838408655018, 0.69539856077920503]], [[-0.5700912419893196, -0.079304552093697911], [0.35674517284883289, -0.37426284214721428]], [[-0.2261370585629694, 0.37269786708046104], [-0.23891963570307384, 0.37569228452350978]], [[-0.8689417221552671, -0.62092135302941887], [0.079810053424291816, -0.59449663162803823]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[-0.16033006596359911, -0.9539277688704344], [-0.21909074438146647, -0.16033006596359911]], [[-0.96337470673707815, -0.16033006596359911], [-0.16033006596359911, -0.16033006596359911]], [[-0.16033006596359911, -0.16033006596359911], [-0.49306838408655018, -0.16033006596359911]], [[-0.5700912419893196, -0.16033006596359911], [-0.16033006596359911, -0.37426284214721428]], [[-0.2261370585629694, -0.16033006596359911], [-0.23891963570307384, -0.16033006596359911]], [[-0.8689417221552671, -0.62092135302941887], [-0.16033006596359911, -0.59449663162803823]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_array_rank3(self):
      arg0=numpy.array([[[-0.8423526318709591, -0.16053619313971046], [0.73189603499057876, -0.38237117980202173]], [[0.38584994653117577, 0.85015131457011228], [0.32983458172008606, 0.57811782533087275]], [[-0.98579258243383894, -0.42979707397457023], [-0.43261722389463531, -0.23370476519492356]], [[-0.52471160167446484, -0.9508706003642684], [0.42661463243079112, 0.16754764921086163]], [[-0.068683949091622454, -0.82327987329609287], [0.21978392262225999, 0.34359441114911049]], [[-0.93043809811342815, 0.35820386607448906], [0.94948379953751361, 0.019561305096486903]]])
      arg1=numpy.array([[[0.95683035066787214, 0.11003778351929938], [0.47613188450434341, -0.30465302048878229]], [[0.11097248204969623, 0.69505330674919286], [0.062351207203307446, 0.50933617475673865]], [[0.13215588609064022, 0.13770293801069466], [-0.53759649782456798, 0.79212030026190883]], [[-0.67581481944096478, -0.83475159690761158], [0.54132413076965058, 0.92842304332122683]], [[-0.74564446845387677, -0.31318928847970628], [0.2820014665884687, -0.48198054219113651]], [[-0.16623544819263958, -0.0028432350581752974], [-0.38154927399718419, 0.66989103426736252]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[-0.8423526318709591, -0.16053619313971046], [0.47613188450434341, -0.38237117980202173]], [[0.11097248204969623, 0.69505330674919286], [0.062351207203307446, 0.50933617475673865]], [[-0.98579258243383894, -0.42979707397457023], [-0.53759649782456798, -0.23370476519492356]], [[-0.67581481944096478, -0.9508706003642684], [0.42661463243079112, 0.16754764921086163]], [[-0.74564446845387677, -0.82327987329609287], [0.21978392262225999, -0.48198054219113651]], [[-0.93043809811342815, -0.0028432350581752974], [-0.38154927399718419, 0.019561305096486903]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_Symbol_rank3(self):
      arg0=numpy.array(0.896077820679)
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[-0.13407891426785024, 0.35941408966353339], [-0.17623567667626605, 0.462324243535569]], [[0.097414939769117304, -0.78403684020647879], [0.31148226843972959, -0.17649123732371041]], [[-0.11728231950995904, -0.89056481693829115], [0.76862475619318849, -0.59952777138883229]], [[-0.47067349303461903, 0.34379315765597962], [0.002563574623446474, -0.97015530056465438]], [[0.66443615945504408, -0.29218908378811426], [-0.79694154414896401, -0.78470612910429249]], [[0.23003804111150772, -0.88133758909420479], [-0.28925987133241393, 0.58846783012958759]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.13407891426785024, 0.35941408966353339], [-0.17623567667626605, 0.462324243535569]], [[0.097414939769117304, -0.78403684020647879], [0.31148226843972959, -0.17649123732371041]], [[-0.11728231950995904, -0.89056481693829115], [0.76862475619318849, -0.59952777138883229]], [[-0.47067349303461903, 0.34379315765597962], [0.002563574623446474, -0.97015530056465438]], [[0.66443615945504408, -0.29218908378811426], [-0.79694154414896401, -0.78470612910429249]], [[0.23003804111150772, -0.88133758909420479], [-0.28925987133241393, 0.58846783012958759]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_Symbol_rank3(self):
      arg0=numpy.array([[[-0.6169460602519139, 0.42542731525881083], [-0.0079415155986617414, 0.10077917957244931]], [[-0.35432608632047269, 0.39021506466324984], [0.19503059476766937, -0.74995285274596113]], [[-0.29159418539768445, -0.40390091558891061], [-0.99127321296267534, -0.95791710373188188]], [[-0.54425524422243932, 0.24977462655224492], [0.29684020670760614, 0.2074103656041737]], [[-0.25279454250510769, 0.82422460089896465], [0.83248642845685983, -0.09446195878071606]], [[-0.58848561784093723, -0.33463955761609387], [0.91898347172175598, 0.19936818335889805]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[0.61018293290776149, -0.06838792129666027], [-0.03149897359799092, 0.59286158578224124]], [[0.27944460998704912, -0.18840017105808826], [-0.54632540449287559, 0.035040878758546778]], [[-0.24439859219744808, -0.16874144655682177], [0.60035062445144827, -0.9529838700016946]], [[-0.64138103644471767, 0.84612124058148908], [-0.52114573487060301, -0.41240503058669975]], [[0.57575267747266334, -0.033226343559272342], [0.46738635544184293, 0.63157516163943117]], [[0.60779079921134316, -0.81655716012556812], [0.58545056057439138, 0.23063555966509619]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-0.6169460602519139, -0.06838792129666027], [-0.03149897359799092, 0.10077917957244931]], [[-0.35432608632047269, -0.18840017105808826], [-0.54632540449287559, -0.74995285274596113]], [[-0.29159418539768445, -0.40390091558891061], [-0.99127321296267534, -0.95791710373188188]], [[-0.64138103644471767, 0.24977462655224492], [-0.52114573487060301, -0.41240503058669975]], [[-0.25279454250510769, -0.033226343559272342], [0.46738635544184293, -0.09446195878071606]], [[-0.58848561784093723, -0.81655716012556812], [0.58545056057439138, 0.19936818335889805]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_constData_rank3(self):
      arg0=numpy.array(-0.415889907293)
      arg1=Data(numpy.array([[[0.078285035507941636, -0.88829449384195969], [0.78284628623875596, 0.48487975776181758]], [[-0.64877397556696148, 0.27779634599817737], [-0.46090863581088648, -0.55154952339707375]], [[0.092940243150796631, 0.28795449293522557], [-0.7542371081895396, -0.9093143329877087]], [[0.63399239655683126, 0.81538932920917251], [-0.68950618704829258, 0.13515238776853478]], [[-0.98817482146142543, -0.65010930256058352], [0.86444673534538152, -0.95207803410803282]], [[-0.92838098819484083, -0.055470455907630667], [-0.39027240185255807, 0.58151262225085487]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.41588990729265674, -0.88829449384195969], [-0.41588990729265674, -0.41588990729265674]], [[-0.64877397556696148, -0.41588990729265674], [-0.46090863581088648, -0.55154952339707375]], [[-0.41588990729265674, -0.41588990729265674], [-0.7542371081895396, -0.9093143329877087]], [[-0.41588990729265674, -0.41588990729265674], [-0.68950618704829258, -0.41588990729265674]], [[-0.98817482146142543, -0.65010930256058352], [-0.41588990729265674, -0.95207803410803282]], [[-0.92838098819484083, -0.41588990729265674], [-0.41588990729265674, -0.41588990729265674]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_constData_rank3(self):
      arg0=numpy.array([[[0.23230794457336801, 0.6295482391959788], [-0.86598061725124631, -0.71081480028995569]], [[-0.0070531167247123161, -0.16394097673965735], [0.29985683892413628, -0.14835484085662687]], [[0.082331089547942993, -0.077703177546636137], [0.15993152398681709, -0.67249786082949115]], [[0.71817030563728057, 0.91126316805303298], [0.6552162285397114, -0.052120819881793956]], [[0.47085218410118945, 0.47762222399359455], [0.69638513430354543, -0.29164000136522406]], [[0.49701961970062092, 0.43153336108405438], [-0.13926013494181588, -0.074842289140794493]]])
      arg1=Data(numpy.array([[[-0.71759635999776594, 0.7956629210514985], [-0.70190580432881067, 0.50873386933266262]], [[-0.83072489665769789, 0.96369912618839826], [0.83008627283250647, -0.75029334468040298]], [[-0.4968341939778611, 0.41495249799406775], [0.023558072093853744, -0.45220570760800594]], [[-0.98470889446903631, -0.45939707055229384], [0.25767031211634528, 0.26171833423275404]], [[-0.58834348362746614, -0.78118404391334062], [0.45620441373827947, 0.75119227847384784]], [[-0.49513587087712052, -0.77230564464754425], [0.69120761611386072, 0.32178715271392666]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.71759635999776594, 0.6295482391959788], [-0.86598061725124631, -0.71081480028995569]], [[-0.83072489665769789, -0.16394097673965735], [0.29985683892413628, -0.75029334468040298]], [[-0.4968341939778611, -0.077703177546636137], [0.023558072093853744, -0.67249786082949115]], [[-0.98470889446903631, -0.45939707055229384], [0.25767031211634528, -0.052120819881793956]], [[-0.58834348362746614, -0.78118404391334062], [0.45620441373827947, -0.29164000136522406]], [[-0.49513587087712052, -0.77230564464754425], [-0.13926013494181588, -0.074842289140794493]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_expandedData_rank3(self):
      arg0=numpy.array(0.421175117364)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.2829539473674072, 0.26460463438963866], [0.4944241195463408, -0.14183289114499043]], [[-0.23141500621564015, 0.72498785591645865], [-0.58704992796483135, -0.073035863218186448]], [[-0.36205001010349336, 0.23509533190089771], [0.99239767418126235, -0.22859926662165808]], [[-0.37580413579582972, 0.34666574092921598], [0.34165651594416047, -0.4852533001340793]], [[0.78319458318008017, 0.69711733836445733], [-0.78801005334209351, -0.87515019413759942]], [[-0.7984610368004843, -0.59209920431386887], [0.95189109002620209, 0.27537361386975356]]])+(1.-msk_arg1)*numpy.array([[[0.27446575000248274, 0.017349123112922138], [-0.77096715555414685, -0.097087488470648964]], [[0.85944675368687107, -0.70393045517842956], [0.19314904950952894, 0.052600456260221362]], [[-0.26236400545090666, 0.43790436341387151], [0.43014747797369157, 0.77978061312029023]], [[-0.39532661171030092, -0.56484594847043423], [0.59568394714667661, -0.63797433280420091]], [[0.020636830350132973, -0.41371359474401759], [0.73147374907031248, 0.36429486148697654]], [[-0.80644341652331408, 0.31189793837408408], [0.89659030970463793, 0.55651979999695933]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.2829539473674072, 0.26460463438963866], [0.42117511736446667, -0.14183289114499043]], [[-0.23141500621564015, 0.42117511736446667], [-0.58704992796483135, -0.073035863218186448]], [[-0.36205001010349336, 0.23509533190089771], [0.42117511736446667, -0.22859926662165808]], [[-0.37580413579582972, 0.34666574092921598], [0.34165651594416047, -0.4852533001340793]], [[0.42117511736446667, 0.42117511736446667], [-0.78801005334209351, -0.87515019413759942]], [[-0.7984610368004843, -0.59209920431386887], [0.42117511736446667, 0.27537361386975356]]])+(1.-msk_ref)*numpy.array([[[0.27446575000248274, 0.017349123112922138], [-0.77096715555414685, -0.097087488470648964]], [[0.42117511736446667, -0.70393045517842956], [0.19314904950952894, 0.052600456260221362]], [[-0.26236400545090666, 0.42117511736446667], [0.42117511736446667, 0.42117511736446667]], [[-0.39532661171030092, -0.56484594847043423], [0.42117511736446667, -0.63797433280420091]], [[0.020636830350132973, -0.41371359474401759], [0.42117511736446667, 0.36429486148697654]], [[-0.80644341652331408, 0.31189793837408408], [0.42117511736446667, 0.42117511736446667]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank3_expandedData_rank3(self):
      arg0=numpy.array([[[-0.44437682880277785, 0.17119541519503967], [0.020929249805486272, 0.58170300942349695]], [[0.73028470694488234, -0.7016569695632946], [-0.62783764903554995, -0.36781622192912433]], [[-0.01675601911778446, 0.7692551787251134], [0.8011270201656655, 0.11025975693234935]], [[-0.84459689494534174, -0.11099094965229672], [0.4562713252772248, -0.87014121921591037]], [[-0.85802171684943973, -0.027261182455522226], [-0.94235114333802628, 0.0098853109764975855]], [[-0.056015196275054357, 0.77622858573743847], [0.10644091959164026, 0.89772237308605818]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.8593066865690393, -0.58245592888380959], [0.91937667904714337, -0.81385389061758473]], [[-0.3365880437476958, -0.068098054719098799], [0.18688186984410238, -0.66851156981592008]], [[0.069434941986412424, 0.40467055700243026], [0.3233186730856723, 0.29236614723618914]], [[-0.83403672060967904, -0.6957481066212301], [-0.51549137067202033, -0.17926352650150323]], [[-0.93561865200237371, -0.80417566586104572], [0.21767801216519467, 0.17338459780994175]], [[-0.68175254457359746, -0.8359966716618179], [-0.62548811235394464, -0.77067444521092821]]])+(1.-msk_arg1)*numpy.array([[[0.080984278450838998, 0.92431525231552847], [0.72290054114827584, 0.027666007066041942]], [[-0.4284361911118022, 0.83516056306889208], [-0.60050207373833175, -0.6528086645912512]], [[-0.52881328986023135, -0.72290003842147654], [-0.43345053521437826, 0.81521539489200001]], [[0.97056103386445125, 0.42677224349907683], [0.35309634734929873, -0.86105870683644969]], [[-0.64117624992120859, 0.73828461790461919], [-0.35162808537783885, -0.91996400261609557]], [[0.94379663317243123, -0.29093618357265916], [0.25221741055218727, -0.3748647916959682]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.44437682880277785, -0.58245592888380959], [0.020929249805486272, -0.81385389061758473]], [[-0.3365880437476958, -0.7016569695632946], [-0.62783764903554995, -0.66851156981592008]], [[-0.01675601911778446, 0.40467055700243026], [0.3233186730856723, 0.11025975693234935]], [[-0.84459689494534174, -0.6957481066212301], [-0.51549137067202033, -0.87014121921591037]], [[-0.93561865200237371, -0.80417566586104572], [-0.94235114333802628, 0.0098853109764975855]], [[-0.68175254457359746, -0.8359966716618179], [-0.62548811235394464, -0.77067444521092821]]])+(1.-msk_ref)*numpy.array([[[-0.44437682880277785, 0.17119541519503967], [0.020929249805486272, 0.027666007066041942]], [[-0.4284361911118022, -0.7016569695632946], [-0.62783764903554995, -0.6528086645912512]], [[-0.52881328986023135, -0.72290003842147654], [-0.43345053521437826, 0.11025975693234935]], [[-0.84459689494534174, -0.11099094965229672], [0.35309634734929873, -0.87014121921591037]], [[-0.85802171684943973, -0.027261182455522226], [-0.94235114333802628, -0.91996400261609557]], [[-0.056015196275054357, -0.29093618357265916], [0.10644091959164026, -0.3748647916959682]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_array_rank4(self):
      arg0=numpy.array(-0.425021300795)
      arg1=numpy.array([[[[0.61485980119305039, 0.3536734485821571, 0.052250192878319668, 0.050101524233152839], [-0.039055350717554393, 0.7713984496570816, -0.73726824203828367, -0.41761983874039488], [-0.39471456087181944, 0.47715720673594375, 0.58378594752525248, 0.9847168762336389]], [[-0.68846166272752263, 0.89123145679985138, -0.54696246032404061, -0.31473352128891041], [0.61406524479214819, 0.87161245947951183, -0.68639513801258745, 0.9875941844426499], [0.39234126544038617, -0.27317552546725499, 0.34483861351735268, 0.25840962926583155]]], [[[-0.17803493909693047, -0.95925086177480923, 0.80696656451276971, -0.16348233631392994], [-0.30672770197390054, -0.22062718838574868, -0.73072807724460676, 0.34145064687010396], [-0.2031521866358974, -0.97455493027649043, -0.84632780030579613, 0.21522265188033973]], [[-0.49543951705969769, -0.68786374270050077, 0.0083934267287057995, -0.46851701005951418], [0.31110904898927538, -0.50460034026401313, -0.76732460633745703, 0.24531335380988151], [0.75868219019798255, 0.59950483231628771, 0.034430641577910315, -0.8574307841533626]]], [[[-0.040193080937767522, 0.3050590978518708, -0.51149457708662482, -0.63023453461760548], [-0.58444008961073712, -0.89533794260129529, 0.001718281287224066, -0.15026721804901189], [0.51164427280872116, -0.3004974735745658, -0.16293466115535393, 0.74220646871136609]], [[0.50215439753218849, -0.022004013369893771, 0.20858271078395774, -0.65255125579857287], [-0.60876611946674219, 0.50167757288927173, 0.13001533342573723, -0.34671537042375489], [-0.23089939867254539, 0.39558498022357491, 0.35653763449420883, -0.28348035905940594]]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[[-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.73726824203828367, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381]], [[-0.68846166272752263, -0.4250213007947381, -0.54696246032404061, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.68639513801258745, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381]]], [[[-0.4250213007947381, -0.95925086177480923, -0.4250213007947381, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.73072807724460676, -0.4250213007947381], [-0.4250213007947381, -0.97455493027649043, -0.84632780030579613, -0.4250213007947381]], [[-0.49543951705969769, -0.68786374270050077, -0.4250213007947381, -0.46851701005951418], [-0.4250213007947381, -0.50460034026401313, -0.76732460633745703, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.8574307841533626]]], [[[-0.4250213007947381, -0.4250213007947381, -0.51149457708662482, -0.63023453461760548], [-0.58444008961073712, -0.89533794260129529, -0.4250213007947381, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381]], [[-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.65255125579857287], [-0.60876611946674219, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381], [-0.4250213007947381, -0.4250213007947381, -0.4250213007947381, -0.4250213007947381]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[0.042003939301036075, -0.28174094403147287, 0.39166430802192709, -0.72286091356523685], [-0.33886759908862429, 0.25462030836845817, -0.78150236199695322, -0.059614808287742793], [-0.91591074314123899, -0.45794579861759788, -0.4794446505723069, 0.51761381411325758]], [[-0.37116726176147496, 0.36874577496090977, 0.04653712013143485, 0.66047333430625699], [-0.11772512279714653, 0.41125989291869902, -0.051596578298892526, 0.31531054601415121], [0.94777892963159904, 0.34055291308435898, 0.85891030072275276, 0.085100918532384107]]], [[[-0.081424966162158219, 0.017887385295465297, 0.81032953717541512, 0.28349857468643225], [0.59854856200751305, -0.70570895384304211, 0.47548664744321822, -0.30265312351112916], [0.0038629985751263973, 0.28479936753094237, 0.87288390787059145, 0.9256013729582182]], [[0.69848851185160865, 0.39861043628990434, -0.1057564500882362, -0.56551442610005487], [-0.053463682328375617, 0.54600713350550056, -0.33231542828693561, 0.46378044816114583], [-0.60774604129762033, -0.26185686498031457, 0.57006282462040714, 0.46411501898722873]]], [[[-0.88481620176041575, 0.30210577801645266, 0.34849551054895844, 0.06626118165337691], [-0.76290211249462025, 0.19755714842723759, 0.73700013120301588, -0.71744858881488205], [0.15791315216699831, -0.82314659794161793, 0.82294870945863874, -0.40651539260931102]], [[-0.041507420283407814, 0.15798803074352841, -0.28927616509900411, 0.64378065731080558], [0.0080178331193350161, 0.20000920176907111, 0.24165479179411586, 0.11678446509431195], [0.71631630922307332, 0.96181563579772433, 0.33502329535286735, -0.62034315202089352]]]])
      arg1=numpy.array([[[[-0.34043458800732118, 0.00057191896702213363, -0.76297413648161472, -0.5050662500035803], [0.4353622860421511, 0.20101382044109961, 0.61932945299685871, -0.3381915344269435], [0.81634370838873238, 0.92937038910446157, -0.28919701417780952, -0.39864650332866391]], [[0.51428419957766658, -0.79550542226298426, -0.260123277813191, -0.27241781897892281], [-0.14877045824181745, 0.08042370779557606, 0.30500099902824629, 0.44964628041638699], [0.21700093634831807, -0.29956762605653742, 0.71766894400698145, -0.32820710507925344]]], [[[0.84141687607897753, 0.22081991062484074, -0.19074104132815517, 0.537673379263784], [-0.44222009045488586, 0.26203424208040871, -0.17734796805122111, 0.9927355338030146], [0.9797124453041044, -0.72244264377131451, 0.98779150207768684, -0.26095818585381969]], [[-0.42451198440543014, -0.053120518772907355, 0.61116466119905111, 0.039629276722799256], [0.12611917229199521, -0.58173761969395499, -0.36454834186256813, -0.85777456539054131], [0.71915215975897673, 0.13301874138712888, 0.47141499862787462, 0.52747331468349512]]], [[[0.91143202507268772, 0.6538661838232469, 0.52091089894188869, -0.70207947258922232], [0.32509302860839639, 0.36200994883335547, 0.32722934083113109, 0.0016335040878923124], [0.24852406000762395, -0.20983527910110555, 0.47273912630193338, -0.36377154614083507]], [[-0.91101787425493375, 0.46886398300907817, 0.66985160031839097, -0.2793849302445901], [0.45054535807781981, -0.4937162343641992, -0.59936511558946393, -0.56881807522634475], [0.99668071551760007, -0.15821676346947111, 0.34842119194446042, -0.39580568445408937]]]])
      res=minimum(arg0,arg1)
      ref=numpy.array([[[[-0.34043458800732118, -0.28174094403147287, -0.76297413648161472, -0.72286091356523685], [-0.33886759908862429, 0.20101382044109961, -0.78150236199695322, -0.3381915344269435], [-0.91591074314123899, -0.45794579861759788, -0.4794446505723069, -0.39864650332866391]], [[-0.37116726176147496, -0.79550542226298426, -0.260123277813191, -0.27241781897892281], [-0.14877045824181745, 0.08042370779557606, -0.051596578298892526, 0.31531054601415121], [0.21700093634831807, -0.29956762605653742, 0.71766894400698145, -0.32820710507925344]]], [[[-0.081424966162158219, 0.017887385295465297, -0.19074104132815517, 0.28349857468643225], [-0.44222009045488586, -0.70570895384304211, -0.17734796805122111, -0.30265312351112916], [0.0038629985751263973, -0.72244264377131451, 0.87288390787059145, -0.26095818585381969]], [[-0.42451198440543014, -0.053120518772907355, -0.1057564500882362, -0.56551442610005487], [-0.053463682328375617, -0.58173761969395499, -0.36454834186256813, -0.85777456539054131], [-0.60774604129762033, -0.26185686498031457, 0.47141499862787462, 0.46411501898722873]]], [[[-0.88481620176041575, 0.30210577801645266, 0.34849551054895844, -0.70207947258922232], [-0.76290211249462025, 0.19755714842723759, 0.32722934083113109, -0.71744858881488205], [0.15791315216699831, -0.82314659794161793, 0.47273912630193338, -0.40651539260931102]], [[-0.91101787425493375, 0.15798803074352841, -0.28927616509900411, -0.2793849302445901], [0.0080178331193350161, -0.4937162343641992, -0.59936511558946393, -0.56881807522634475], [0.71631630922307332, -0.15821676346947111, 0.33502329535286735, -0.62034315202089352]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_Symbol_rank4(self):
      arg0=numpy.array(-0.100250978974)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[-0.9202576531584965, -0.31389104223317932, -0.56282863661484761, 0.58866839112871783], [0.97870514853610624, 0.31431246497322851, -0.74813390088164122, 0.88784585830810303], [0.18057891520591873, 0.070522550318576238, 0.97217514528791482, 0.23915238327768407]], [[0.31771529857768943, -0.58226133885328202, 0.3484389117947253, -0.49522391743193284], [0.93013634598070416, 0.79309492449199603, 0.095097761264621106, 0.82751900039793536], [-0.047492310624078771, -0.8841068904752476, -0.48001273400968603, -0.97605805091958775]]], [[[-0.62046631586968815, -0.57889569283560149, -0.63052457742428136, -0.14171955215843224], [-0.43679217074784349, 0.33364885698662405, -0.33833466053365835, 0.023649251084093326], [0.42162167127597616, -0.10157452709138637, -0.81987219386266807, 0.19159299788432782]], [[-0.93087975048159355, 0.94497913568264091, -0.44581541911151579, -0.48718368010892155], [-0.43520533138739648, 0.92516681886481189, 0.85627287462192281, -0.86558938514188322], [-0.96992689194935067, 0.51267178255250534, -0.30430734699648965, 0.6379173384831438]]], [[[0.94377675387235582, -0.1812657672252822, 0.78861701031980957, -0.52714118747564775], [-0.81772037112203266, -0.99145864106124604, -0.52756517562249527, -0.06703579559092443], [0.65202540881573712, 0.43515790814081434, 0.44775444096369355, 0.22595929479900301]], [[-0.81894108161107959, 0.83708163989499162, 0.29610698715584305, 0.28876865628271431], [-0.6941081536729401, 0.6825130132689361, 0.54881394533772276, -0.84435947489566909], [-0.55978332173928314, 0.80814861449338915, 0.47178274139759191, -0.77282375706342332]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.9202576531584965, -0.31389104223317932, -0.56282863661484761, -0.10025097897447721], [-0.10025097897447721, -0.10025097897447721, -0.74813390088164122, -0.10025097897447721], [-0.10025097897447721, -0.10025097897447721, -0.10025097897447721, -0.10025097897447721]], [[-0.10025097897447721, -0.58226133885328202, -0.10025097897447721, -0.49522391743193284], [-0.10025097897447721, -0.10025097897447721, -0.10025097897447721, -0.10025097897447721], [-0.10025097897447721, -0.8841068904752476, -0.48001273400968603, -0.97605805091958775]]], [[[-0.62046631586968815, -0.57889569283560149, -0.63052457742428136, -0.14171955215843224], [-0.43679217074784349, -0.10025097897447721, -0.33833466053365835, -0.10025097897447721], [-0.10025097897447721, -0.10157452709138637, -0.81987219386266807, -0.10025097897447721]], [[-0.93087975048159355, -0.10025097897447721, -0.44581541911151579, -0.48718368010892155], [-0.43520533138739648, -0.10025097897447721, -0.10025097897447721, -0.86558938514188322], [-0.96992689194935067, -0.10025097897447721, -0.30430734699648965, -0.10025097897447721]]], [[[-0.10025097897447721, -0.1812657672252822, -0.10025097897447721, -0.52714118747564775], [-0.81772037112203266, -0.99145864106124604, -0.52756517562249527, -0.10025097897447721], [-0.10025097897447721, -0.10025097897447721, -0.10025097897447721, -0.10025097897447721]], [[-0.81894108161107959, -0.10025097897447721, -0.10025097897447721, -0.10025097897447721], [-0.6941081536729401, -0.10025097897447721, -0.10025097897447721, -0.84435947489566909], [-0.55978332173928314, -0.10025097897447721, -0.10025097897447721, -0.77282375706342332]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[0.70445318135805635, 0.56505898183842018, -0.17970743649454235, 0.63606342447069619], [-0.85723402359082357, -0.78099195166922519, -0.55695409649330263, 0.86334658016750665], [0.43504417176378762, 0.24956759878900625, -0.268034708587787, -0.090793010880092062]], [[0.68780610493749106, -0.14314154390078304, -0.9387548997947528, -0.89233861518475588], [0.66136382438745667, -0.91236547961722914, -0.29641086436424668, 0.86585711347671257], [-0.18308913503565183, 0.26524153391301919, 0.3698831236421396, -0.63078100847404706]]], [[[0.81894812564036856, 0.79298576369700369, -0.43059267970878667, -0.15469303242090815], [-0.76188763355885269, 0.38672127253359112, 0.46954044163906583, -0.42981850144569989], [-0.27127681996642505, 0.73910896016295369, 0.31754535362315561, -0.75994079659692271]], [[0.47823267817974635, -0.99549628561268078, 0.027456509878385216, -0.62397996334457639], [-0.54471304428356615, 0.24846318666456324, -0.94266857232150159, -0.29159565261457177], [0.19295139234129288, -0.062231191143994335, 0.088509854763827223, 0.73248314188730834]]], [[[-0.20972680189375081, -0.65626213527766164, -0.67689012906431079, 0.0090845973299429339], [0.82675065698606431, -0.97259030620601328, -0.23798941112429661, -0.14790400323964437], [0.074558251968207534, -0.94480496258497215, -0.89800727906629541, -0.53405768954931765]], [[-0.71024864234252494, -0.26678465296397835, -0.46646004764065396, -0.61964027472269656], [0.8215367388911734, 0.80174719844867592, 0.7795206525592604, 0.86654531975743487], [0.25155276811754534, -0.9641659788010859, 0.5259203902194689, 0.28214533113819962]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[-0.77811688231575027, 0.67479581175472347, 0.41122986892670199, -0.95268575172648995], [0.72029115589804626, 0.97158610198700091, 0.68662540338992128, -0.58661014252443344], [-0.39996595180915606, 0.17300212437628204, 0.83920700119950609, 0.24863272558186722]], [[0.19626514969237108, -0.058554962674822075, 0.59425086301018437, 0.34341760314877368], [-0.16795248484325698, -0.13435530592963096, -0.46720047172015833, -0.96841834768150536], [-0.4094184746615408, -0.49324831423073801, -0.80305255952076737, 0.27903179028181202]]], [[[0.59210854414084513, 0.16229364612933384, 0.32983932364370339, -0.069921810341125212], [-0.86525595241422848, 0.78663994253262359, 0.073152643257254013, -0.065465301649834329], [-0.95737070161960691, -0.15722480301738595, 0.58479204407788887, 0.34806632765301249]], [[-0.053031486310931797, -0.74754296765970407, 0.8895618667923979, -0.18033426633519434], [0.99127585832599152, -0.9430927446690911, 0.63173692588662744, 0.74573345854646034], [-0.84900413790932805, 0.037917824805533185, -0.021823519588779039, -0.25279198270242631]]], [[[-0.55365751580690814, 0.5655307194960888, 0.81403512429854596, 0.496623639607636], [0.8070680684310394, -0.10432783046120275, -0.4092176979965263, 0.46822488023699216], [-0.12632401070818977, 0.30546570052106348, 0.50204623844809193, -0.65430212823342404]], [[0.42021004855658339, 0.73702738579454241, 0.25419395310148363, -0.88826396308206013], [0.022199128509628796, -0.0504396902219999, -0.034733483183756242, -0.37356837890252637], [0.74297212749789154, 0.0040409026736771825, 0.0082558710862761231, -0.32333739708445952]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-0.77811688231575027, 0.56505898183842018, -0.17970743649454235, -0.95268575172648995], [-0.85723402359082357, -0.78099195166922519, -0.55695409649330263, -0.58661014252443344], [-0.39996595180915606, 0.17300212437628204, -0.268034708587787, -0.090793010880092062]], [[0.19626514969237108, -0.14314154390078304, -0.9387548997947528, -0.89233861518475588], [-0.16795248484325698, -0.91236547961722914, -0.46720047172015833, -0.96841834768150536], [-0.4094184746615408, -0.49324831423073801, -0.80305255952076737, -0.63078100847404706]]], [[[0.59210854414084513, 0.16229364612933384, -0.43059267970878667, -0.15469303242090815], [-0.86525595241422848, 0.38672127253359112, 0.073152643257254013, -0.42981850144569989], [-0.95737070161960691, -0.15722480301738595, 0.31754535362315561, -0.75994079659692271]], [[-0.053031486310931797, -0.99549628561268078, 0.027456509878385216, -0.62397996334457639], [-0.54471304428356615, -0.9430927446690911, -0.94266857232150159, -0.29159565261457177], [-0.84900413790932805, -0.062231191143994335, -0.021823519588779039, -0.25279198270242631]]], [[[-0.55365751580690814, -0.65626213527766164, -0.67689012906431079, 0.0090845973299429339], [0.8070680684310394, -0.97259030620601328, -0.4092176979965263, -0.14790400323964437], [-0.12632401070818977, -0.94480496258497215, -0.89800727906629541, -0.65430212823342404]], [[-0.71024864234252494, -0.26678465296397835, -0.46646004764065396, -0.88826396308206013], [0.022199128509628796, -0.0504396902219999, -0.034733483183756242, -0.37356837890252637], [0.25155276811754534, -0.9641659788010859, 0.0082558710862761231, -0.32333739708445952]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_constData_rank4(self):
      arg0=numpy.array(-0.728265524667)
      arg1=Data(numpy.array([[[[-0.91504547999740171, 0.012810369887270801, 0.21587944606381781, 0.34119766573266341], [-0.32280199779375085, -0.054237935216707722, -0.79623690920579104, 0.37088241127304933], [0.12150991633148944, -0.69101227785116848, 0.73073892595521994, -0.39934370895848681]], [[-0.3693789717505116, 0.75661694648430133, -0.85836142330610854, -0.32592487428594308], [0.22624952359253991, -0.60247521964053385, -0.70161823705348025, -0.24954872237120762], [0.75171081324418454, -0.15657093890028562, -0.25090153278953364, 0.59505994298610987]]], [[[-0.05826875212801208, 0.29442028374780427, -0.53038798155388278, 0.091061125683532085], [-0.079005593570049548, 0.00038909916899121377, -0.98829583577709301, 0.88259853324762538], [0.039576704346908853, -0.89938608943741927, 0.17757413934391741, -0.75009089150150232]], [[0.55508806275619582, 0.03370053177355703, -0.32048468296258648, 0.26936484745950495], [-0.66575518682742252, -0.014717889405908791, 0.095144932559580564, -0.44264133289207175], [0.93538260370364279, 0.1969856382674855, 0.95006635748916191, 0.26793304941124396]]], [[[-0.99713043387118305, 0.91152513517539258, 0.16558703416532627, -0.8732226182327969], [0.41787238147618089, -0.24825788959565842, -0.93904745746514684, 0.65940703339290052], [0.39049965031397704, -0.01633210611410818, 0.76623765746407324, 0.79548430500716738]], [[-0.9295138582981306, 0.87668478493227453, 0.11974242787104439, 0.99380782515202259], [0.023477675853753821, 0.12563754123099624, 0.59918906448149012, 0.24490444444099158], [0.11158552855467185, -0.90242799905815874, -0.32273327237013638, -0.63910566525377743]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.91504547999740171, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.79623690920579104, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475]], [[-0.72826552466718475, -0.72826552466718475, -0.85836142330610854, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475]]], [[[-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.98829583577709301, -0.72826552466718475], [-0.72826552466718475, -0.89938608943741927, -0.72826552466718475, -0.75009089150150232]], [[-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475]]], [[[-0.99713043387118305, -0.72826552466718475, -0.72826552466718475, -0.8732226182327969], [-0.72826552466718475, -0.72826552466718475, -0.93904745746514684, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475]], [[-0.9295138582981306, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.72826552466718475, -0.72826552466718475, -0.72826552466718475], [-0.72826552466718475, -0.90242799905815874, -0.72826552466718475, -0.72826552466718475]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[0.89616945966184547, -0.14645510856890098, -0.86944537131732735, 0.71847451412555885], [-0.70029804236386828, -0.59541381010443239, 0.46801127601195169, -0.98906614917578506], [-0.0038988657105638591, 0.89334894552295396, 0.80207365883677695, 0.045257584587148569]], [[-0.89527264919235794, 0.25135871481774141, -0.13871252779034937, -0.2268380251899389], [0.82776928299769925, -0.68463706276658498, 0.67371921923813471, -0.84777953655418981], [-0.62874608627117512, 0.42722345466709055, -0.94537803283246236, 0.36509199072064469]]], [[[-0.061019954251117836, -0.90043209976059879, -0.022522952319181444, -0.30808636639781128], [-0.059270354706197015, -0.98946793146803502, -0.084217982787760493, 0.50140625510490033], [-0.36014449522505387, -0.0012891593072639207, -0.7676490938631908, 0.97777264426399246]], [[-0.58620170107081848, 0.51110054858214671, -0.82865872761639325, -0.28662262522030013], [0.98406191940826915, -0.19441394216509345, -0.80699650983760751, 0.83474185695992764], [0.21626770610979862, 0.56723393086742901, -0.45682530021332535, 0.30071795628255216]]], [[[0.95678003089117025, 0.64036149833716083, 0.69037514189719995, 0.86338039107159026], [0.44798670324388978, -0.8821382997160776, 0.53761088703496585, -0.59796439421036895], [0.77175483432437963, 0.55279794412778749, 0.51171466465772752, -0.78060131818837952]], [[-0.0019563325668305875, 0.45035895276343862, 0.71256826749792679, 0.059872047958682284], [-0.0045286660467727113, 0.052392141396692571, -0.93729392803827971, -0.12521252410639683], [0.74090180294982066, 0.40862302453986343, -0.91445707601869186, 0.97035478267697162]]]])
      arg1=Data(numpy.array([[[[-0.75590562416235296, 0.14642023157793571, -0.12653964381022997, 0.42197150408388007], [0.58565806704221313, -0.45752468513384636, 0.37530077420954244, -0.004886124862624408], [-0.94573442434973387, 0.69031368457162623, -0.22430619417909337, 0.37189041044203774]], [[0.72030982570688162, 0.084943716693110405, 0.70869964682131847, 0.65056340832307979], [-0.50256410430250154, -0.80187490102990533, 0.36305309609681768, -0.49170901485046947], [0.15916719884739439, 0.55444505244047804, -0.49309231895000694, -0.012241059835174317]]], [[[-0.32488113260609497, -0.9506670587376318, 0.22143799462804314, -0.83450193323794597], [-0.66841387730291268, -0.68891638572947711, 0.90173799584959169, 0.20367991669827634], [-0.88038622593071425, -0.35055128187852258, 0.46266021883328223, 0.51901972460544465]], [[0.53282314089226923, -0.40039628012906325, 0.74925467656589073, -0.66283838835894349], [0.8539284273772747, 0.23870884717634011, -0.52872969622654864, 0.004765180524383128], [-0.15473284316236779, 0.52492084303138831, 0.85159883021105176, -0.87240307400493955]]], [[[-0.83306261910311052, 0.79895454199389393, -0.92709032553525006, -0.80730790080570647], [0.42263888236227642, -0.016657249528685192, 0.97554582357090669, 0.061298475762013194], [-0.97792819096272021, -0.79595674466849009, -0.15669485807783312, -0.55958160117965061]], [[0.51306876856470796, 0.28658084223168268, -0.57526824659959508, 0.47201153027836873], [-0.95345014094079827, 0.65039071677434834, -0.48813003978909841, -0.12064204080928587], [-0.58226564935141245, -0.22086805263933829, -0.11701890565008766, 0.27034100986868181]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.75590562416235296, -0.14645510856890098, -0.86944537131732735, 0.42197150408388007], [-0.70029804236386828, -0.59541381010443239, 0.37530077420954244, -0.98906614917578506], [-0.94573442434973387, 0.69031368457162623, -0.22430619417909337, 0.045257584587148569]], [[-0.89527264919235794, 0.084943716693110405, -0.13871252779034937, -0.2268380251899389], [-0.50256410430250154, -0.80187490102990533, 0.36305309609681768, -0.84777953655418981], [-0.62874608627117512, 0.42722345466709055, -0.94537803283246236, -0.012241059835174317]]], [[[-0.32488113260609497, -0.9506670587376318, -0.022522952319181444, -0.83450193323794597], [-0.66841387730291268, -0.98946793146803502, -0.084217982787760493, 0.20367991669827634], [-0.88038622593071425, -0.35055128187852258, -0.7676490938631908, 0.51901972460544465]], [[-0.58620170107081848, -0.40039628012906325, -0.82865872761639325, -0.66283838835894349], [0.8539284273772747, -0.19441394216509345, -0.80699650983760751, 0.004765180524383128], [-0.15473284316236779, 0.52492084303138831, -0.45682530021332535, -0.87240307400493955]]], [[[-0.83306261910311052, 0.64036149833716083, -0.92709032553525006, -0.80730790080570647], [0.42263888236227642, -0.8821382997160776, 0.53761088703496585, -0.59796439421036895], [-0.97792819096272021, -0.79595674466849009, -0.15669485807783312, -0.78060131818837952]], [[-0.0019563325668305875, 0.28658084223168268, -0.57526824659959508, 0.059872047958682284], [-0.95345014094079827, 0.052392141396692571, -0.93729392803827971, -0.12521252410639683], [-0.58226564935141245, -0.22086805263933829, -0.91445707601869186, 0.27034100986868181]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank0_expandedData_rank4(self):
      arg0=numpy.array(0.281482687583)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.96139931694264602, -0.46542502700306065, 0.21705020250000717, 0.25998144094215436], [-0.67599497210438408, -0.88726153965521437, 0.50714519594861973, -0.94351230306971434], [0.4735621454256087, 0.66535053046165382, -0.92160757851582442, 0.56806474443196353]], [[-0.79648092981517116, -0.91914568320849033, 0.061585600611920865, 0.40796935385442001], [0.51063761899195481, -0.63004783356942196, 0.29473857980068674, 0.3993733500273533], [-0.86738854332012272, -0.90515907494131387, -0.53950068927270856, 0.63224096754179016]]], [[[-0.89351049172945252, -0.10390460426084824, 0.69035315350800897, -0.93521456718548146], [0.39827705680524073, 0.73667161808316028, 0.28704469449069014, 0.88865905114776966], [0.87149264051148667, -0.74166315186312604, -0.7737992814615251, -0.7013002656683156]], [[0.01183038355658983, 0.69969112055301053, -0.69259784860387152, 0.72657661951184838], [-0.0011547332973642632, 0.42540737329381795, -0.89384884283793609, -0.62524545534864617], [0.59186830563376747, -0.3950350803692193, 0.67372642149590822, 0.97846778898779418]]], [[[0.64847624613338084, -0.88588135991859018, 0.0098973918958247253, -0.22092271944289599], [0.019301469314388697, -0.29040469916195244, 0.74571810999576105, -0.82485168516556739], [-0.96894898013431607, 0.44051204266918909, -0.87294543885774289, 0.30028502270655122]], [[-0.70101408967227208, -0.51776039315507005, 0.90350367325208669, 0.10904456854021327], [0.61537494807808057, 0.64923447601306727, -0.47834048264925055, -0.64570185204968511], [0.48998647902070847, 0.36405054395814584, 0.13174866605530777, -0.15696623732480997]]]])+(1.-msk_arg1)*numpy.array([[[[-0.95592431340402562, -0.60487536368214267, -0.70132740049521569, 0.33572363884270229], [-0.098366265904404449, 0.9871748496737538, 0.24895594396041809, -0.45423489233820269], [0.78592718677440909, -0.19086024983760552, 0.16439749870129217, -0.04056652690832907]], [[-0.031753883057961563, 0.67732913621272894, 0.93957129305443887, -0.91173472602648853], [-0.27159715341342894, -0.53046432854904779, -0.12734481648913021, 0.00035975804336385586], [0.3983625491254712, 0.91201218719416577, -0.18752057545397216, 0.58097395184941303]]], [[[0.31110931733325153, 0.59709777785508633, 0.015313248760234988, -0.84817068169239951], [0.67194461729786537, -0.60439213536887459, -0.48302160342844225, -0.30697988179917512], [0.67323921500232453, -0.68065095783053819, 0.32806852285911003, -0.35698994357893921]], [[0.033347651443349058, -0.49997367580630692, 0.84884845716150847, -0.34517141483336999], [-0.78606121659755734, -0.038076895417745726, 0.15363901358486598, -0.95959538054192728], [0.70460148255700306, -0.6314651277352985, 0.57195126378779571, 0.61365414684478425]]], [[[-0.52421196371811485, 0.051881060064812523, 0.84475480281804982, -0.038368254608132757], [0.55468719070482808, 0.36163262036185251, 0.14013678078593461, 0.69111743548010685], [0.48452729217682733, -0.77957351851064516, 0.70582041801806028, -0.22776541561739805]], [[-0.99620405759506614, -0.065100342142452972, -0.98034381283504834, -0.24442864167348999], [-0.79840506656885291, -0.11423424587912767, 0.62928672738040414, 0.51459561217101402], [0.90827122726638509, 0.036154935543965427, -0.75332876638410395, -0.77542998162694277]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.28148268758292794, -0.46542502700306065, 0.21705020250000717, 0.25998144094215436], [-0.67599497210438408, -0.88726153965521437, 0.28148268758292794, -0.94351230306971434], [0.28148268758292794, 0.28148268758292794, -0.92160757851582442, 0.28148268758292794]], [[-0.79648092981517116, -0.91914568320849033, 0.061585600611920865, 0.28148268758292794], [0.28148268758292794, -0.63004783356942196, 0.28148268758292794, 0.28148268758292794], [-0.86738854332012272, -0.90515907494131387, -0.53950068927270856, 0.28148268758292794]]], [[[-0.89351049172945252, -0.10390460426084824, 0.28148268758292794, -0.93521456718548146], [0.28148268758292794, 0.28148268758292794, 0.28148268758292794, 0.28148268758292794], [0.28148268758292794, -0.74166315186312604, -0.7737992814615251, -0.7013002656683156]], [[0.01183038355658983, 0.28148268758292794, -0.69259784860387152, 0.28148268758292794], [-0.0011547332973642632, 0.28148268758292794, -0.89384884283793609, -0.62524545534864617], [0.28148268758292794, -0.3950350803692193, 0.28148268758292794, 0.28148268758292794]]], [[[0.28148268758292794, -0.88588135991859018, 0.0098973918958247253, -0.22092271944289599], [0.019301469314388697, -0.29040469916195244, 0.28148268758292794, -0.82485168516556739], [-0.96894898013431607, 0.28148268758292794, -0.87294543885774289, 0.28148268758292794]], [[-0.70101408967227208, -0.51776039315507005, 0.28148268758292794, 0.10904456854021327], [0.28148268758292794, 0.28148268758292794, -0.47834048264925055, -0.64570185204968511], [0.28148268758292794, 0.28148268758292794, 0.13174866605530777, -0.15696623732480997]]]])+(1.-msk_ref)*numpy.array([[[[-0.95592431340402562, -0.60487536368214267, -0.70132740049521569, 0.28148268758292794], [-0.098366265904404449, 0.28148268758292794, 0.24895594396041809, -0.45423489233820269], [0.28148268758292794, -0.19086024983760552, 0.16439749870129217, -0.04056652690832907]], [[-0.031753883057961563, 0.28148268758292794, 0.28148268758292794, -0.91173472602648853], [-0.27159715341342894, -0.53046432854904779, -0.12734481648913021, 0.00035975804336385586], [0.28148268758292794, 0.28148268758292794, -0.18752057545397216, 0.28148268758292794]]], [[[0.28148268758292794, 0.28148268758292794, 0.015313248760234988, -0.84817068169239951], [0.28148268758292794, -0.60439213536887459, -0.48302160342844225, -0.30697988179917512], [0.28148268758292794, -0.68065095783053819, 0.28148268758292794, -0.35698994357893921]], [[0.033347651443349058, -0.49997367580630692, 0.28148268758292794, -0.34517141483336999], [-0.78606121659755734, -0.038076895417745726, 0.15363901358486598, -0.95959538054192728], [0.28148268758292794, -0.6314651277352985, 0.28148268758292794, 0.28148268758292794]]], [[[-0.52421196371811485, 0.051881060064812523, 0.28148268758292794, -0.038368254608132757], [0.28148268758292794, 0.28148268758292794, 0.14013678078593461, 0.28148268758292794], [0.28148268758292794, -0.77957351851064516, 0.28148268758292794, -0.22776541561739805]], [[-0.99620405759506614, -0.065100342142452972, -0.98034381283504834, -0.24442864167348999], [-0.79840506656885291, -0.11423424587912767, 0.28148268758292794, 0.28148268758292794], [0.28148268758292794, 0.036154935543965427, -0.75332876638410395, -0.77542998162694277]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[0.97758867065194388, 0.1574715083841578, 0.92126133529285936, 0.0017485576090787713], [0.30821540583119145, -0.0025962813909445881, -0.96197375828255094, -0.17194860271430423], [0.094454498954737121, -0.07250961702271419, 0.62778955379908385, 0.0079850761766036005]], [[-0.60879110851107665, 0.47215584168603542, -0.98584787335963897, -0.73779857765254264], [-0.83454070683860015, 0.71015247657599923, -0.12985974578869364, 0.1442477762775396], [0.83715255131333088, -0.98841700808434618, 0.50984203618760593, 0.44426127969468876]]], [[[-0.23720255382404654, 0.87338277179418444, 0.65518040493836915, 0.43693499615282794], [-0.74945847124111942, -0.20205597361726579, 0.16075177649972994, 0.76542786079844416], [0.0084223442203108512, 0.62485445223452429, -0.80756840753979398, 0.69416166740918994]], [[0.50369752501143816, -0.28619664928209976, 0.73165224082732983, -0.83834270633927233], [-0.87567827976335533, 0.20958630766347386, -0.96710268290637158, 0.97297089353810251], [-0.74444429732585271, 0.73028907717698144, 0.79344172504077126, 0.30570201766123484]]], [[[-0.92244419927829013, 0.59913714085120406, -0.35711739481389837, 0.88880258561481273], [-0.89231644998518367, 0.19489484911023292, -0.39451485599809466, 0.27240452235133139], [0.2987117668996726, 0.3206929594555108, -0.32011263224255404, -0.84191955653278927]], [[-0.10047467343115413, 0.36870094503673023, -0.11718151966864498, 0.94218134444430746], [-0.86040589643937704, -0.38260047523296281, 0.5015039443115934, -0.5357932441680231], [-0.57857336437949636, 0.044525865869085024, -0.3671500832314345, 0.98066964135088841]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.7291751279524481, -0.82295874146889636, -0.51901123107040248, -0.94385821506324263], [0.52294198124270164, -0.14066371662637622, -0.097520794836683411, 0.075409404431268978], [0.23559899515853866, -0.29600101634192733, -0.1751856203151525, -0.50967621684885756]], [[-0.8768845479609868, 0.99854052839455631, -0.30669679984325215, -0.37134456828559181], [0.68518337298456267, 0.13654662776898308, -0.47464629422009064, -0.011280137872295093], [0.40164036996810104, 0.37774591678565561, -0.74261475554766676, 0.98647492200385689]]], [[[0.68809646007526037, 0.90499229150499949, -0.68585979550179599, 0.62601311739091314], [-0.5805994620782815, 0.58051780093835159, -0.56955547233437276, 0.45211084783869149], [0.39755039928225622, -0.32712115094789707, 0.57617901672016059, 0.015699026079915912]], [[0.32301557183066287, 0.28929822753850409, -0.66402611654355659, -0.80524061708853534], [-0.68700827200906578, -0.63985275312192713, -0.30966086791544556, -0.95007157363421046], [-0.61111269316547867, 0.62577720815804883, -0.76699263429637488, 0.78025394665315306]]], [[[-0.47340018072776724, -0.92358854884009567, -0.86082413472578834, 0.30344956406647272], [0.92125667910051856, 0.073668347431324888, -0.60350141920277145, -0.028444018908313051], [-0.60429126796395538, -0.28222700986008209, 0.48016499084307274, -0.059464320230215417]], [[0.065766576024333556, -0.52347734065420548, 0.016237375223726147, 0.42922281621395175], [-0.99857921802796556, -0.0077971875052467166, -0.96806290472645085, 0.15635839223982595], [0.19379650848173302, -0.41071589816893872, -0.55531682359218126, -0.63323305108665906]]]])+(1.-msk_arg1)*numpy.array([[[[-0.31178601958278973, 0.12684778530306096, -0.080846647401125349, 0.048217540170149675], [-0.023908426556572104, -0.85794324333380501, 0.6724681026582755, -0.13821299320362268], [-0.071069979997626875, -0.14530184611306152, -0.055119495841453148, 0.95397372024346017]], [[-0.47831340013473711, -0.66383344642512765, 0.81086102238370406, 0.45600927493737786], [0.44566995864355974, -0.22137428274745985, -0.24935027071035054, 0.77905263525564439], [-0.56872754605053299, -0.71165324113234418, 0.57765580895481872, -0.51244917020295588]]], [[[-0.71609928733020745, -0.87099442425058093, 0.97220643808598095, -0.82807409932896592], [-0.44989238496978312, 0.19505344002723879, -0.81516005237704814, 0.1311739770256346], [0.79987025852551774, 0.56139062013262508, -0.40098839998976055, 0.58213997350561786]], [[0.78581917651153765, 0.27526409545213815, 0.24194305054027732, -0.57780081412091744], [-0.15028899104477866, -0.81143553920292733, 0.89191984931908475, 0.50772063858105621], [-0.8909661319611808, 0.15236586097218385, -0.99310008767165203, -0.81001930174950521]]], [[[0.96631073454664596, -0.30422026751622155, -0.63386350345029441, -0.080745092712922295], [-0.32769338763082501, 0.72745103771464192, -0.11565640992324844, -0.68262761307055708], [-0.43871134404200429, 0.86969922871936189, 0.15221660186189334, -0.10926022795256984]], [[-0.29081627930669951, 0.93426803597386665, -0.53586220853287103, 0.83151004347999669], [0.25571746811416229, -0.69085800408630038, -0.72177795263395961, -0.23374635001601196], [0.35949139491405213, 0.16688540524134865, 0.23071452816086579, -0.83712306643881917]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.7291751279524481, -0.82295874146889636, -0.51901123107040248, -0.94385821506324263], [0.30821540583119145, -0.14066371662637622, -0.96197375828255094, -0.17194860271430423], [0.094454498954737121, -0.29600101634192733, -0.1751856203151525, -0.50967621684885756]], [[-0.8768845479609868, 0.47215584168603542, -0.98584787335963897, -0.73779857765254264], [-0.83454070683860015, 0.13654662776898308, -0.47464629422009064, -0.011280137872295093], [0.40164036996810104, -0.98841700808434618, -0.74261475554766676, 0.44426127969468876]]], [[[-0.23720255382404654, 0.87338277179418444, -0.68585979550179599, 0.43693499615282794], [-0.74945847124111942, -0.20205597361726579, -0.56955547233437276, 0.45211084783869149], [0.0084223442203108512, -0.32712115094789707, -0.80756840753979398, 0.015699026079915912]], [[0.32301557183066287, -0.28619664928209976, -0.66402611654355659, -0.83834270633927233], [-0.87567827976335533, -0.63985275312192713, -0.96710268290637158, -0.95007157363421046], [-0.74444429732585271, 0.62577720815804883, -0.76699263429637488, 0.30570201766123484]]], [[[-0.92244419927829013, -0.92358854884009567, -0.86082413472578834, 0.30344956406647272], [-0.89231644998518367, 0.073668347431324888, -0.60350141920277145, -0.028444018908313051], [-0.60429126796395538, -0.28222700986008209, -0.32011263224255404, -0.84191955653278927]], [[-0.10047467343115413, -0.52347734065420548, -0.11718151966864498, 0.42922281621395175], [-0.99857921802796556, -0.38260047523296281, -0.96806290472645085, -0.5357932441680231], [-0.57857336437949636, -0.41071589816893872, -0.55531682359218126, -0.63323305108665906]]]])+(1.-msk_ref)*numpy.array([[[[-0.31178601958278973, 0.12684778530306096, -0.080846647401125349, 0.0017485576090787713], [-0.023908426556572104, -0.85794324333380501, -0.96197375828255094, -0.17194860271430423], [-0.071069979997626875, -0.14530184611306152, -0.055119495841453148, 0.0079850761766036005]], [[-0.60879110851107665, -0.66383344642512765, -0.98584787335963897, -0.73779857765254264], [-0.83454070683860015, -0.22137428274745985, -0.24935027071035054, 0.1442477762775396], [-0.56872754605053299, -0.98841700808434618, 0.50984203618760593, -0.51244917020295588]]], [[[-0.71609928733020745, -0.87099442425058093, 0.65518040493836915, -0.82807409932896592], [-0.74945847124111942, -0.20205597361726579, -0.81516005237704814, 0.1311739770256346], [0.0084223442203108512, 0.56139062013262508, -0.80756840753979398, 0.58213997350561786]], [[0.50369752501143816, -0.28619664928209976, 0.24194305054027732, -0.83834270633927233], [-0.87567827976335533, -0.81143553920292733, -0.96710268290637158, 0.50772063858105621], [-0.8909661319611808, 0.15236586097218385, -0.99310008767165203, -0.81001930174950521]]], [[[-0.92244419927829013, -0.30422026751622155, -0.63386350345029441, -0.080745092712922295], [-0.89231644998518367, 0.19489484911023292, -0.39451485599809466, -0.68262761307055708], [-0.43871134404200429, 0.3206929594555108, -0.32011263224255404, -0.84191955653278927]], [[-0.29081627930669951, 0.36870094503673023, -0.53586220853287103, 0.83151004347999669], [-0.86040589643937704, -0.69085800408630038, -0.72177795263395961, -0.5357932441680231], [-0.57857336437949636, 0.044525865869085024, -0.3671500832314345, -0.83712306643881917]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_float_rank0(self):
      arg0=Symbol(shape=())
      arg1=0.597119985048
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.531875505796)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-0.531875505796)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_float_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=-0.421981970969
      res=minimum(arg0,arg1)
      s0=numpy.array([0.60250056430372534, 0.71187054876987443])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.42198197096909018, -0.42198197096909018])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_float_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=0.914021229368
      res=minimum(arg0,arg1)
      s0=numpy.array([[-0.82857048793293164, 0.34941752541287441, -0.11666402825236566, 0.34404682426784472, 0.041468065236192064], [0.41520887543141027, 0.10004321752818179, 0.92980871100238383, -0.46037318713712216, -0.020525473146714823], [-0.097653358288753767, 0.27028960599480034, -0.83372310099316871, 0.79592781736696039, -0.93738380994181214], [-0.42294022191322744, -0.2137923958072101, -0.40942413446004333, 0.15148152017591165, 0.0052467450489508405]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.82857048793293164, 0.34941752541287441, -0.11666402825236566, 0.34404682426784472, 0.041468065236192064], [0.41520887543141027, 0.10004321752818179, 0.91402122936829566, -0.46037318713712216, -0.020525473146714823], [-0.097653358288753767, 0.27028960599480034, -0.83372310099316871, 0.79592781736696039, -0.93738380994181214], [-0.42294022191322744, -0.2137923958072101, -0.40942413446004333, 0.15148152017591165, 0.0052467450489508405]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_float_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-0.248207009526
      res=minimum(arg0,arg1)
      s0=numpy.array([[[0.36109304578796597, 0.02227143963453182], [0.12405253606018318, -0.021449710420084589]], [[-0.93540719356601354, 0.49495762109712693], [-0.091653356140455555, -0.48933834147138233]], [[-0.58529273952226069, 0.54368194611753795], [0.1069475654738592, 0.90279184311110838]], [[0.67698381045313738, 0.081252032257523199], [-0.94814337698548434, -0.18197033827102604]], [[0.55484973316147013, 0.25201101525810432], [0.51549643768427056, 0.7530743051610953]], [[-0.19259978091138308, 0.72350065761339444], [-0.667051849941805, -0.092222994803684344]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.24820700952594299, -0.24820700952594299], [-0.24820700952594299, -0.24820700952594299]], [[-0.93540719356601354, -0.24820700952594299], [-0.24820700952594299, -0.48933834147138233]], [[-0.58529273952226069, -0.24820700952594299], [-0.24820700952594299, -0.24820700952594299]], [[-0.24820700952594299, -0.24820700952594299], [-0.94814337698548434, -0.24820700952594299]], [[-0.24820700952594299, -0.24820700952594299], [-0.24820700952594299, -0.24820700952594299]], [[-0.24820700952594299, -0.24820700952594299], [-0.667051849941805, -0.24820700952594299]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_float_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=0.886954035708
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[0.92548497368491067, 0.98831591446042499, 0.48654142045232773, -0.74521173022346465], [-0.24246325346943887, 0.76267735091273203, 0.91156880880396507, -0.2484865758175383], [-0.83832754346905514, -0.95520204007347576, 0.47782100446710318, -0.37660558415536438]], [[-0.034610770126496382, -0.96759192194927257, -0.16876255258294837, 0.52850226757301688], [0.11356938101928082, -0.88555132067857456, -0.98337800716450108, -0.53286540445610786], [0.51827033106573195, -0.3413008365597725, -0.69606433470149831, 0.21413833883488076]]], [[[-0.51022569887443137, 0.56180423344767583, 0.17579900712383312, 0.59744508855144929], [-0.17069333894904193, 0.87474759557217041, 0.99733379065050642, -0.91815284569707312], [0.73993018739467065, -0.013161852804395968, 0.31715834740386972, 0.95730230725043119]], [[-0.38075183868269424, -0.55060852505461999, 0.72379701903501026, -0.78841575037190892], [-0.057971522698722744, 0.4462698865196828, 0.040839105265164433, -0.038458403418057729], [0.76063269931623823, -0.53502535309600074, 0.23252452647007593, -0.77444009925638913]]], [[[-0.14150054932851885, 0.39939035045421512, 0.32472269303493628, 0.34044462015325605], [-0.85952976216465737, -0.93787779329929233, -0.21452254050887842, -0.76927435538762978], [0.90114203324775155, -0.41849180752937265, 0.11670965587363091, -0.76962324640144453]], [[0.34583273085537591, -0.66711536862245935, -0.58243919396568655, 0.0052765386938227188], [0.91899052027327843, -0.23479536331319006, -0.50877726275669533, 0.42071581068856734], [-0.58230229639892439, -0.2527939465370852, 0.33008872978453674, 0.46396057036326521]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.88695403570788689, 0.88695403570788689, 0.48654142045232773, -0.74521173022346465], [-0.24246325346943887, 0.76267735091273203, 0.88695403570788689, -0.2484865758175383], [-0.83832754346905514, -0.95520204007347576, 0.47782100446710318, -0.37660558415536438]], [[-0.034610770126496382, -0.96759192194927257, -0.16876255258294837, 0.52850226757301688], [0.11356938101928082, -0.88555132067857456, -0.98337800716450108, -0.53286540445610786], [0.51827033106573195, -0.3413008365597725, -0.69606433470149831, 0.21413833883488076]]], [[[-0.51022569887443137, 0.56180423344767583, 0.17579900712383312, 0.59744508855144929], [-0.17069333894904193, 0.87474759557217041, 0.88695403570788689, -0.91815284569707312], [0.73993018739467065, -0.013161852804395968, 0.31715834740386972, 0.88695403570788689]], [[-0.38075183868269424, -0.55060852505461999, 0.72379701903501026, -0.78841575037190892], [-0.057971522698722744, 0.4462698865196828, 0.040839105265164433, -0.038458403418057729], [0.76063269931623823, -0.53502535309600074, 0.23252452647007593, -0.77444009925638913]]], [[[-0.14150054932851885, 0.39939035045421512, 0.32472269303493628, 0.34044462015325605], [-0.85952976216465737, -0.93787779329929233, -0.21452254050887842, -0.76927435538762978], [0.88695403570788689, -0.41849180752937265, 0.11670965587363091, -0.76962324640144453]], [[0.34583273085537591, -0.66711536862245935, -0.58243919396568655, 0.0052765386938227188], [0.88695403570788689, -0.23479536331319006, -0.50877726275669533, 0.42071581068856734], [-0.58230229639892439, -0.2527939465370852, 0.33008872978453674, 0.46396057036326521]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_array_rank0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-0.56543962449)
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.581570681648)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-0.581570681648)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_array_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(0.195344263765)
      res=minimum(arg0,arg1)
      s0=numpy.array([0.060399406668079525, -0.8264769825553302])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.060399406668079525, -0.8264769825553302])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_array_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-0.612537816899)
      res=minimum(arg0,arg1)
      s0=numpy.array([[0.84589922366685011, -0.24223854103488884, 0.92635178850265598, -0.31902911877981022, 0.42046893270693997], [-0.91977946147270884, 0.82941925583271381, 0.16382018013974453, 0.16022686926552088, -0.9963162524255269], [0.44954931085742933, -0.2617468986842475, -0.881044712391851, -0.89487787535519403, -0.36868674822372172], [-0.06705215759490013, 0.16463318063814358, -0.92096565373086081, -0.95089579996155016, -0.49413748885000741]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.61253781689944753, -0.61253781689944753, -0.61253781689944753, -0.61253781689944753, -0.61253781689944753], [-0.91977946147270884, -0.61253781689944753, -0.61253781689944753, -0.61253781689944753, -0.9963162524255269], [-0.61253781689944753, -0.61253781689944753, -0.881044712391851, -0.89487787535519403, -0.61253781689944753], [-0.61253781689944753, -0.61253781689944753, -0.92096565373086081, -0.95089579996155016, -0.61253781689944753]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_array_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(0.534382566569)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[-0.093088257947719688, -0.78691314607759333], [0.18255880012637471, 0.85624604662574066]], [[-0.38620129258710967, 0.88538435143979077], [-0.83370033212624772, 0.3226125243052449]], [[-0.86747050218089417, 0.27609646660629927], [0.73764777116013147, 0.044847040012168993]], [[-0.067847859332550664, 0.57773862831028855], [0.29365191480072217, 0.27486555771361987]], [[0.16694014900679677, 0.99496178243527078], [0.99875943717243043, 0.75800483780736805]], [[0.74025237816059519, -0.48452441082690045], [0.22379926154093877, -0.87473373064924598]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.093088257947719688, -0.78691314607759333], [0.18255880012637471, 0.53438256656887351]], [[-0.38620129258710967, 0.53438256656887351], [-0.83370033212624772, 0.3226125243052449]], [[-0.86747050218089417, 0.27609646660629927], [0.53438256656887351, 0.044847040012168993]], [[-0.067847859332550664, 0.53438256656887351], [0.29365191480072217, 0.27486555771361987]], [[0.16694014900679677, 0.53438256656887351], [0.53438256656887351, 0.53438256656887351]], [[0.53438256656887351, -0.48452441082690045], [0.22379926154093877, -0.87473373064924598]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_array_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(-0.528164850395)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[-0.45359674406052508, -0.79357352810685255, -0.074377222528491993, 0.40258324094229492], [-0.073935617449695457, -0.98655186621991886, -0.0059205638075845446, -0.74856010342340262], [-0.1726746082212367, 0.86582786904054942, -0.96320767639282745, -0.38894878060941873]], [[-0.86097809555007099, 0.15785962449166546, -0.40962317049992847, -0.8128306492341999], [0.20780645338788983, -0.14214567748824236, 0.9083781714461785, -0.37409231827155676], [0.11587137036079298, -0.22868726936984385, -0.34387404848269432, -0.10882122233171043]]], [[[0.55288892585522897, -0.077659442065239892, -0.39104313065518337, 0.1458445374102435], [0.68350579559663505, -0.18380870402607541, -0.5017488025685819, -0.97317952319938561], [0.07109944832127546, -0.6250084404945051, 0.46470322150748511, -0.87134845880807044]], [[-0.7019290555822868, 0.59287333263375319, -0.50530826894135616, 0.18254367250979819], [-0.2468068515077555, -0.54271759588948676, -0.51373804843689319, -0.75433799389884038], [-0.83278986799759669, -0.3137840316547702, 0.069063836225355946, 0.26366442342375596]]], [[[0.0073924358987338401, 0.53608031671068113, -0.33574171736313185, -0.43598515343766131], [-0.79451710161342337, 0.76985127600319836, 0.001118058728945881, -0.42426724511392289], [0.75581965924344985, -0.77785585410678482, -0.60611359390119746, -0.40814385019068289]], [[0.70260302924114826, -0.57773846359079983, 0.86083882274685575, 0.0094648270969444059], [0.60505129550730929, -0.98840563575066032, -0.13609544142976393, -0.62366626480816345], [0.74097291534420706, -0.23319084157718284, 0.53642380290135616, -0.25334298285962675]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.52816485039461214, -0.79357352810685255, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.98655186621991886, -0.52816485039461214, -0.74856010342340262], [-0.52816485039461214, -0.52816485039461214, -0.96320767639282745, -0.52816485039461214]], [[-0.86097809555007099, -0.52816485039461214, -0.52816485039461214, -0.8128306492341999], [-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214]]], [[[-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.97317952319938561], [-0.52816485039461214, -0.6250084404945051, -0.52816485039461214, -0.87134845880807044]], [[-0.7019290555822868, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.54271759588948676, -0.52816485039461214, -0.75433799389884038], [-0.83278986799759669, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214]]], [[[-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214], [-0.79451710161342337, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.77785585410678482, -0.60611359390119746, -0.52816485039461214]], [[-0.52816485039461214, -0.57773846359079983, -0.52816485039461214, -0.52816485039461214], [-0.52816485039461214, -0.98840563575066032, -0.52816485039461214, -0.62366626480816345], [-0.52816485039461214, -0.52816485039461214, -0.52816485039461214, -0.52816485039461214]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_Symbol_rank0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s0=numpy.array(0.83339629583)
      s1=numpy.array(-0.0469714860124)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.0469714860124)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_Symbol_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s0=numpy.array([-0.99592379373190654, -0.39592212671138438])
      s1=numpy.array(-0.737716840951)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-0.99592379373190654, -0.73771684095131729])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_Symbol_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s0=numpy.array([[-0.097137321531767684, 0.11060589153767597, -0.54321030100177037, 0.18070666115420209, -0.5867586451523219], [0.91579339586764541, -0.23765693635780494, 0.93499276617866922, -0.24678887075868472, -0.63910334808085345], [-0.45731190243105768, 0.35051986794031276, 0.15645461767397095, 0.1216196456605243, -0.44280780023887845], [-0.85057921370330236, -0.35991231271213309, -0.51471731123552789, 0.422118298471583, 0.65078424537387125]])
      s1=numpy.array(-0.598837596568)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889], [-0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.63910334808085345], [-0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889], [-0.85057921370330236, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889, -0.59883759656848889]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_Symbol_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s0=numpy.array([[[0.01882018567984689, 0.18194776793691081], [-0.2453900867359291, -0.33329690550810542]], [[0.95499715417889397, -0.73359965655017034], [-0.37401780068583057, -0.51479169693950522]], [[-0.563476378825577, -0.27035422273374166], [0.72514271745781422, -0.85233427437325071]], [[0.5132632491205551, 0.15681814908221448], [0.083571946014736609, -0.67062544662832102]], [[-0.81161813749582268, 0.34111478607846624], [-0.47767677588967139, 0.40193355742917802]], [[-0.36388516365109447, 0.51587462524631689], [0.26255144075686476, 0.69676093161410702]]])
      s1=numpy.array(-0.564080325705)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.56408032570486832, -0.56408032570486832], [-0.56408032570486832, -0.56408032570486832]], [[-0.56408032570486832, -0.73359965655017034], [-0.56408032570486832, -0.56408032570486832]], [[-0.56408032570486832, -0.56408032570486832], [-0.56408032570486832, -0.85233427437325071]], [[-0.56408032570486832, -0.56408032570486832], [-0.56408032570486832, -0.67062544662832102]], [[-0.81161813749582268, -0.56408032570486832], [-0.56408032570486832, -0.56408032570486832]], [[-0.56408032570486832, -0.56408032570486832], [-0.56408032570486832, -0.56408032570486832]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_Symbol_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[0.66961975188075096, 0.075726492937726153, 0.37179645800679895, -0.26675747956247431], [0.66945514192756939, 0.24053153856188314, 0.85723998772643251, -0.79431620657608204], [-0.79729851104661353, -0.1026593773556066, 0.5550144164011277, 0.5034433832117573]], [[-0.32025486569563144, -0.34628077498046506, -0.97312154050498534, 0.99377676961715755], [0.24433414043889212, -0.16282562685024438, -0.21957143816740943, -0.95755901744625072], [0.48046257270491477, 0.46644193655000876, -0.74215636548958441, -0.27164831911789666]]], [[[0.51186974566416898, -0.463801213202087, 0.059463793931349374, -0.75088124606027806], [0.53795137212219091, -0.082850171150558038, -0.62182860311876942, 0.11878096326694276], [-0.6070594461713521, -0.11458536295839705, -0.25381060462161376, -0.38306129474119621]], [[0.18441714952794608, -0.50550235877416227, -0.66578437766516352, 0.50314522138510198], [0.51254022926387766, 0.0020127365743212167, 0.6416282278486376, 0.020615909383862485], [0.61291670373564244, -0.4743320122336121, -0.66404404042326171, 0.20891164522870453]]], [[[0.25979876417911263, -0.42656378623868574, -0.17411009994956061, 0.50330060618383143], [0.17600985825828963, 0.42214602317588712, 0.35602515364762422, -0.48558406996743608], [-0.52452784939484243, 0.38115215139121128, -0.32985680004899764, 0.66375611522028555]], [[-0.68993874892025908, 0.24946662498160022, 0.39836716842210862, 0.76911114790948942], [0.54375068598036247, 0.87246072947068343, 0.43040817283892996, 0.17538324270539163], [0.79315390131967023, 0.49910137681442257, 0.059236611001377781, -0.12711202669569399]]]])
      s1=numpy.array(0.0151140415673)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.015114041567315439, 0.015114041567315439, 0.015114041567315439, -0.26675747956247431], [0.015114041567315439, 0.015114041567315439, 0.015114041567315439, -0.79431620657608204], [-0.79729851104661353, -0.1026593773556066, 0.015114041567315439, 0.015114041567315439]], [[-0.32025486569563144, -0.34628077498046506, -0.97312154050498534, 0.015114041567315439], [0.015114041567315439, -0.16282562685024438, -0.21957143816740943, -0.95755901744625072], [0.015114041567315439, 0.015114041567315439, -0.74215636548958441, -0.27164831911789666]]], [[[0.015114041567315439, -0.463801213202087, 0.015114041567315439, -0.75088124606027806], [0.015114041567315439, -0.082850171150558038, -0.62182860311876942, 0.015114041567315439], [-0.6070594461713521, -0.11458536295839705, -0.25381060462161376, -0.38306129474119621]], [[0.015114041567315439, -0.50550235877416227, -0.66578437766516352, 0.015114041567315439], [0.015114041567315439, 0.0020127365743212167, 0.015114041567315439, 0.015114041567315439], [0.015114041567315439, -0.4743320122336121, -0.66404404042326171, 0.015114041567315439]]], [[[0.015114041567315439, -0.42656378623868574, -0.17411009994956061, 0.015114041567315439], [0.015114041567315439, 0.015114041567315439, 0.015114041567315439, -0.48558406996743608], [-0.52452784939484243, 0.015114041567315439, -0.32985680004899764, 0.015114041567315439]], [[-0.68993874892025908, 0.015114041567315439, 0.015114041567315439, 0.015114041567315439], [0.015114041567315439, 0.015114041567315439, 0.015114041567315439, 0.015114041567315439], [0.015114041567315439, 0.015114041567315439, 0.015114041567315439, -0.12711202669569399]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_constData_rank0(self):
      arg0=Symbol(shape=())
      arg1=Data(-0.279204298557,self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.939692468018)
      sub=res.substitute({arg0:s0})
      ref=Data(-0.939692468018,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_constData_rank0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(0.75991638667,self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([0.21366572118752147, 0.13317679628341095])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.21366572118752147, 0.13317679628341095]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_constData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-0.944633887938,self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[-0.80031929590838602, 0.15190614883299736, 0.43550250751788711, 0.92660296799271569, -0.72052524093732284], [-0.88839025628550305, 0.068710492122759925, -0.20140233918945882, 0.021878397795129123, -0.75900292491230514], [-0.43213161666636379, -0.13680997824622199, 0.23739335194543432, -0.36873275853355114, 0.24208071915043661], [-0.40071487580316889, -0.40103150553644928, 0.35952092137926162, -0.55686429026219719, -0.82372080654973034]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775], [-0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775], [-0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775], [-0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775, -0.94463388793756775]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_constData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(0.0814648047068,self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[-0.68810570818751415, -0.24926167744258865], [0.18727032641340235, 0.23034302544562402]], [[-0.9405585414628117, 0.36213227424055194], [-0.20335759040621948, 0.3305936310143438]], [[-0.036256771712500857, -0.21841894795415562], [-0.27260104878412772, 0.78176640940499453]], [[0.60362090422854342, 0.26448972768414603], [-0.83284180714485379, -0.08197399143442774]], [[0.10887593576535615, 0.48371526108561946], [0.672572377072435, 0.31836134212363509]], [[0.15564330277494398, 0.67103902104729718], [-0.17443908619814974, 0.13536494359801998]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-0.68810570818751415, -0.24926167744258865], [0.081464804706753657, 0.081464804706753657]], [[-0.9405585414628117, 0.081464804706753657], [-0.20335759040621948, 0.081464804706753657]], [[-0.036256771712500857, -0.21841894795415562], [-0.27260104878412772, 0.081464804706753657]], [[0.081464804706753657, 0.081464804706753657], [-0.83284180714485379, -0.08197399143442774]], [[0.081464804706753657, 0.081464804706753657], [0.081464804706753657, 0.081464804706753657]], [[0.081464804706753657, 0.081464804706753657], [-0.17443908619814974, 0.081464804706753657]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_constData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(0.494027727923,self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[-0.062887591304893631, 0.55143753025829945, 0.27738807493901074, 0.13308372207335117], [-0.19462763774398439, -0.52370009739182799, 0.68401886125708256, 0.87104428874141004], [0.92445550324260761, 0.29997498060945671, -0.81919911073405483, 0.9964846610442768]], [[-0.68964061727203307, 0.94565263781547682, -0.99224499843348157, -0.36234714336475293], [0.54684264049826559, 0.036184123767978305, 0.76230499190199752, 0.90060610699539079], [-0.11860107847568213, 0.49598907165710471, 0.28544731671537305, -0.10998909235750509]]], [[[0.21515276024134389, -0.45183308556089719, 0.14037773628124484, -0.37065859103968135], [-0.6533679896527802, 0.38885539083279719, -0.10558579854726391, -0.67119483414009151], [-0.56327709260725123, 0.60929463130498918, -0.85617082530804733, 0.21550458218056168]], [[-0.64735057009430497, 0.331565481616932, 0.83344788455309415, 0.88193919047806513], [-0.057262006041648661, 0.77780010806268085, -0.088351431027552607, 0.97511830760797635], [-0.6701317510427125, -0.64225389671854249, 0.68238185166065479, -0.13261186574518935]]], [[[0.26601102749114469, 0.31398463229435225, 0.82884441352359928, -0.2418342127269093], [-0.25496458510329556, 0.0088863261838609819, -0.92216452316987851, 0.92747225255615695], [0.69548529313498864, -0.45219810809852801, 0.05864994082755226, 0.27565329249961223]], [[0.81903267474144403, -0.99134970166858372, -0.090398022434032388, -0.341278234963039], [-0.62899868227247357, 0.36702485332171175, 0.2622630946393234, 0.45598827150606636], [0.54330588190838336, -0.057616006986790369, -0.052830126536661526, -0.86234441908395576]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.062887591304893631, 0.49402772792289995, 0.27738807493901074, 0.13308372207335117], [-0.19462763774398439, -0.52370009739182799, 0.49402772792289995, 0.49402772792289995], [0.49402772792289995, 0.29997498060945671, -0.81919911073405483, 0.49402772792289995]], [[-0.68964061727203307, 0.49402772792289995, -0.99224499843348157, -0.36234714336475293], [0.49402772792289995, 0.036184123767978305, 0.49402772792289995, 0.49402772792289995], [-0.11860107847568213, 0.49402772792289995, 0.28544731671537305, -0.10998909235750509]]], [[[0.21515276024134389, -0.45183308556089719, 0.14037773628124484, -0.37065859103968135], [-0.6533679896527802, 0.38885539083279719, -0.10558579854726391, -0.67119483414009151], [-0.56327709260725123, 0.49402772792289995, -0.85617082530804733, 0.21550458218056168]], [[-0.64735057009430497, 0.331565481616932, 0.49402772792289995, 0.49402772792289995], [-0.057262006041648661, 0.49402772792289995, -0.088351431027552607, 0.49402772792289995], [-0.6701317510427125, -0.64225389671854249, 0.49402772792289995, -0.13261186574518935]]], [[[0.26601102749114469, 0.31398463229435225, 0.49402772792289995, -0.2418342127269093], [-0.25496458510329556, 0.0088863261838609819, -0.92216452316987851, 0.49402772792289995], [0.49402772792289995, -0.45219810809852801, 0.05864994082755226, 0.27565329249961223]], [[0.49402772792289995, -0.99134970166858372, -0.090398022434032388, -0.341278234963039], [-0.62899868227247357, 0.36702485332171175, 0.2622630946393234, 0.45598827150606636], [0.49402772792289995, -0.057616006986790369, -0.052830126536661526, -0.86234441908395576]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_expandedData_rank0(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.293736161217)+(1.-msk_arg1)*(0.905849755844)
      res=minimum(arg0,arg1)
      s0=numpy.array(0.308975055267)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.293736161217)+(1.-msk_ref)*(0.308975055267)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_expandedData_rank0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.034829446549)+(1.-msk_arg1)*(0.597347566292)
      res=minimum(arg0,arg1)
      s0=numpy.array([-0.98999679471041691, -0.32607498351924757])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.98999679471041691, -0.32607498351924757])+(1.-msk_ref)*numpy.array([-0.98999679471041691, -0.32607498351924757])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_expandedData_rank0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.186413924441)+(1.-msk_arg1)*(0.904549504825)
      res=minimum(arg0,arg1)
      s0=numpy.array([[-0.39066533569118933, -0.066618973730152353, 0.75334684948382313, -0.7075160489608161, 0.21594904704516749], [-0.88301709688979968, -0.54318272440748516, -0.86338514069722216, -0.75520906987779113, 0.10222014749340125], [0.4717059727828099, 0.0063810856137724858, 0.034607182999305985, 0.79599622527786296, -0.6137994947786225], [0.28695590708433172, 0.88226417740727547, -0.76318173229985153, -0.86273294936408385, 0.59114782761744511]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.39066533569118933, -0.1864139244411569, -0.1864139244411569, -0.7075160489608161, -0.1864139244411569], [-0.88301709688979968, -0.54318272440748516, -0.86338514069722216, -0.75520906987779113, -0.1864139244411569], [-0.1864139244411569, -0.1864139244411569, -0.1864139244411569, -0.1864139244411569, -0.6137994947786225], [-0.1864139244411569, -0.1864139244411569, -0.76318173229985153, -0.86273294936408385, -0.1864139244411569]])+(1.-msk_ref)*numpy.array([[-0.39066533569118933, -0.066618973730152353, 0.75334684948382313, -0.7075160489608161, 0.21594904704516749], [-0.88301709688979968, -0.54318272440748516, -0.86338514069722216, -0.75520906987779113, 0.10222014749340125], [0.4717059727828099, 0.0063810856137724858, 0.034607182999305985, 0.79599622527786296, -0.6137994947786225], [0.28695590708433172, 0.88226417740727547, -0.76318173229985153, -0.86273294936408385, 0.59114782761744511]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_expandedData_rank0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.569371397098)+(1.-msk_arg1)*(0.116694793666)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[0.20982228454318963, -0.59014706719496712], [0.43086820035363904, -0.81249064378077374]], [[0.60486172689449003, 0.18113953680701189], [-0.36638313224706121, 0.71215169027032443]], [[0.8371030001413351, -0.1935468968462184], [0.6061095141191597, 0.1276230679409267]], [[0.31902352470296202, -0.76871869065393272], [0.13905543087792616, -0.86635463708253146]], [[-0.71707072421660056, 0.062883724391742835], [-0.0034796731678696791, -0.67920540601869894]], [[0.7284296130155663, -0.81027905280497259], [-0.67113037665723518, -0.92539595801606778]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.5693713970976737, -0.59014706719496712], [-0.5693713970976737, -0.81249064378077374]], [[-0.5693713970976737, -0.5693713970976737], [-0.5693713970976737, -0.5693713970976737]], [[-0.5693713970976737, -0.5693713970976737], [-0.5693713970976737, -0.5693713970976737]], [[-0.5693713970976737, -0.76871869065393272], [-0.5693713970976737, -0.86635463708253146]], [[-0.71707072421660056, -0.5693713970976737], [-0.5693713970976737, -0.67920540601869894]], [[-0.5693713970976737, -0.81027905280497259], [-0.67113037665723518, -0.92539595801606778]]])+(1.-msk_ref)*numpy.array([[[0.11669479366586466, -0.59014706719496712], [0.11669479366586466, -0.81249064378077374]], [[0.11669479366586466, 0.11669479366586466], [-0.36638313224706121, 0.11669479366586466]], [[0.11669479366586466, -0.1935468968462184], [0.11669479366586466, 0.11669479366586466]], [[0.11669479366586466, -0.76871869065393272], [0.11669479366586466, -0.86635463708253146]], [[-0.71707072421660056, 0.062883724391742835], [-0.0034796731678696791, -0.67920540601869894]], [[0.11669479366586466, -0.81027905280497259], [-0.67113037665723518, -0.92539595801606778]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_expandedData_rank0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.788905904689)+(1.-msk_arg1)*(-0.0143682050296)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[-0.748743505471551, 0.52307474191878622, 0.64301486100432892, -0.40283042592464047], [-0.26754273028219377, -0.55276391210043907, -0.73911400375875269, -0.72941956090882898], [0.12933590614746282, -0.68072540739913023, -0.47467885917496733, 0.73419669166194468]], [[-0.049535147922386003, 0.31874698341063645, -0.99962106638642179, 0.81273446813966488], [0.3779537328022895, -0.33443588586548234, 0.99739269708225975, 0.93579855348434049], [-0.0040279343435163373, -0.49370348139273146, 0.43766788494757258, 0.46797354922042556]]], [[[-0.91484379823793294, -0.24066449642720555, 0.79660636120861605, -0.040286788600157886], [-0.26345041029301375, 0.10494230551281203, 0.0022040631511319742, -0.94031488785552431], [0.852070837310686, -0.60396106622626666, -0.24336307906539445, -0.93509448454055111]], [[0.19437600234066843, -0.2298639205445121, -0.16696338047033144, -0.3748466357664153], [0.9783235930048193, 0.45312567395943271, 0.52318047229699305, -0.84793149262782652], [-0.076561393913761266, -0.13014346356790019, -0.20808063085398198, 0.96281249243761735]]], [[[-0.23226076463981427, 0.93630790512905837, 0.37092503495152362, -0.61991743686342837], [0.87669810159045558, 0.9237586907654145, 0.79794289333568713, 0.76162098954405399], [0.33872192451333483, -0.77363395428832771, -0.02955805998347838, -0.76832260627687843]], [[0.58561166285524546, -0.17714136735321007, -0.9380349905188754, -0.2136071633963228], [0.42610278208373442, 0.51195549889983205, 0.27787693352593434, 0.67545350662787107], [-0.81367258723790781, -0.3977098468894944, 0.85109989441283118, -0.6695319163344251]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.748743505471551, 0.52307474191878622, 0.64301486100432892, -0.40283042592464047], [-0.26754273028219377, -0.55276391210043907, -0.73911400375875269, -0.72941956090882898], [0.12933590614746282, -0.68072540739913023, -0.47467885917496733, 0.73419669166194468]], [[-0.049535147922386003, 0.31874698341063645, -0.99962106638642179, 0.78890590468914668], [0.3779537328022895, -0.33443588586548234, 0.78890590468914668, 0.78890590468914668], [-0.0040279343435163373, -0.49370348139273146, 0.43766788494757258, 0.46797354922042556]]], [[[-0.91484379823793294, -0.24066449642720555, 0.78890590468914668, -0.040286788600157886], [-0.26345041029301375, 0.10494230551281203, 0.0022040631511319742, -0.94031488785552431], [0.78890590468914668, -0.60396106622626666, -0.24336307906539445, -0.93509448454055111]], [[0.19437600234066843, -0.2298639205445121, -0.16696338047033144, -0.3748466357664153], [0.78890590468914668, 0.45312567395943271, 0.52318047229699305, -0.84793149262782652], [-0.076561393913761266, -0.13014346356790019, -0.20808063085398198, 0.78890590468914668]]], [[[-0.23226076463981427, 0.78890590468914668, 0.37092503495152362, -0.61991743686342837], [0.78890590468914668, 0.78890590468914668, 0.78890590468914668, 0.76162098954405399], [0.33872192451333483, -0.77363395428832771, -0.02955805998347838, -0.76832260627687843]], [[0.58561166285524546, -0.17714136735321007, -0.9380349905188754, -0.2136071633963228], [0.42610278208373442, 0.51195549889983205, 0.27787693352593434, 0.67545350662787107], [-0.81367258723790781, -0.3977098468894944, 0.78890590468914668, -0.6695319163344251]]]])+(1.-msk_ref)*numpy.array([[[[-0.748743505471551, -0.014368205029622105, -0.014368205029622105, -0.40283042592464047], [-0.26754273028219377, -0.55276391210043907, -0.73911400375875269, -0.72941956090882898], [-0.014368205029622105, -0.68072540739913023, -0.47467885917496733, -0.014368205029622105]], [[-0.049535147922386003, -0.014368205029622105, -0.99962106638642179, -0.014368205029622105], [-0.014368205029622105, -0.33443588586548234, -0.014368205029622105, -0.014368205029622105], [-0.014368205029622105, -0.49370348139273146, -0.014368205029622105, -0.014368205029622105]]], [[[-0.91484379823793294, -0.24066449642720555, -0.014368205029622105, -0.040286788600157886], [-0.26345041029301375, -0.014368205029622105, -0.014368205029622105, -0.94031488785552431], [-0.014368205029622105, -0.60396106622626666, -0.24336307906539445, -0.93509448454055111]], [[-0.014368205029622105, -0.2298639205445121, -0.16696338047033144, -0.3748466357664153], [-0.014368205029622105, -0.014368205029622105, -0.014368205029622105, -0.84793149262782652], [-0.076561393913761266, -0.13014346356790019, -0.20808063085398198, -0.014368205029622105]]], [[[-0.23226076463981427, -0.014368205029622105, -0.014368205029622105, -0.61991743686342837], [-0.014368205029622105, -0.014368205029622105, -0.014368205029622105, -0.014368205029622105], [-0.014368205029622105, -0.77363395428832771, -0.02955805998347838, -0.76832260627687843]], [[-0.014368205029622105, -0.17714136735321007, -0.9380349905188754, -0.2136071633963228], [-0.014368205029622105, -0.014368205029622105, -0.014368205029622105, -0.014368205029622105], [-0.81367258723790781, -0.3977098468894944, -0.014368205029622105, -0.6695319163344251]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_array_rank1(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([0.10802126302989601, -0.43728044938464605])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.255683171272)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.10802126302989601, -0.43728044938464605])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_array_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([0.25999408394411572, 0.90400606994892074])
      res=minimum(arg0,arg1)
      s0=numpy.array([-0.38705482982591022, 0.32164070458406369])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-0.38705482982591022, 0.32164070458406369])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_Symbol_rank1(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s0=numpy.array(0.837958194008)
      s1=numpy.array([0.2624578944916014, 0.91915393159266756])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.2624578944916014, 0.83795819400777272])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_Symbol_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s0=numpy.array([0.65936212278832218, -0.056411708954108741])
      s1=numpy.array([-0.13814703275130502, 0.66012264571695156])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-0.13814703275130502, -0.056411708954108741])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_constData_rank1(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-0.96257101464203343, 0.072526136301492405]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array(0.419227436825)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-0.96257101464203343, 0.072526136301492405]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_constData_rank1(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-0.97203088284258987, 0.36809116951364351]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([0.80153249212192756, 0.25189027814104614])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-0.97203088284258987, 0.25189027814104614]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_expandedData_rank1(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.82656655352454123, -0.50044739148595441])+(1.-msk_arg1)*numpy.array([0.99288374828396919, 0.92407527964430347])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.581640605016)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.82656655352454123, -0.50044739148595441])+(1.-msk_ref)*numpy.array([0.58164060501620485, 0.58164060501620485])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank1_expandedData_rank1(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.28717564562252873, -0.419641529363592])+(1.-msk_arg1)*numpy.array([0.31880398872825766, -0.37869095943655795])
      res=minimum(arg0,arg1)
      s0=numpy.array([-0.95628698708945215, 0.54045315801178773])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.95628698708945215, -0.419641529363592])+(1.-msk_ref)*numpy.array([-0.95628698708945215, -0.37869095943655795])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_array_rank2(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-0.70808053338868726, -0.82119997048401316, -0.86653249236258212, -0.54431722634312729, -0.18629759295114945], [0.20495976499005875, 0.90825149218715939, -0.42665906047215008, 0.51344190246978161, 0.30636084018120102], [0.8941199530421402, 0.44403863939209853, -0.21014742576102563, 0.45676218717470229, -0.48189397950818091], [-0.73483335511122228, 0.11900803807855653, 0.54964499131871025, -0.89469127576019525, 0.81689723647614065]])
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.857677391731)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.85767739173067103, -0.85767739173067103, -0.86653249236258212, -0.85767739173067103, -0.85767739173067103], [-0.85767739173067103, -0.85767739173067103, -0.85767739173067103, -0.85767739173067103, -0.85767739173067103], [-0.85767739173067103, -0.85767739173067103, -0.85767739173067103, -0.85767739173067103, -0.85767739173067103], [-0.85767739173067103, -0.85767739173067103, -0.85767739173067103, -0.89469127576019525, -0.85767739173067103]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[-0.15761774740339551, -0.61712486855024773, 0.90964583444523606, 0.49009086212528308, -0.63526818983508648], [-0.90571320620562723, -0.95028888060724315, -0.93181101559212332, 0.16065443836333615, 0.6442024669087496], [-0.59210561974607168, -0.61893139561931676, 0.9412179956704636, 0.13100506472840223, 0.76189753454116604], [0.59022610106155327, -0.48587266107393057, -0.033263770580520768, 0.94360059670957597, -0.80943019498550961]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[0.51670777233138798, 0.93489687907648578, 0.4419112997412884, -0.35179632593046661, 0.60691473460439282], [-0.91999022897134308, 0.20210088622407896, 0.037292793705377081, 0.73042638978282093, -0.34454389289968201], [0.1923047841931711, 0.65596218960853458, -0.50791482324023085, -0.09657157306056563, -0.92842763863945543], [-0.55407896300662629, -0.29829055200290666, 0.67555193549758141, -0.73916385017619857, -0.10534966142033042]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.15761774740339551, -0.61712486855024773, 0.4419112997412884, -0.35179632593046661, -0.63526818983508648], [-0.91999022897134308, -0.95028888060724315, -0.93181101559212332, 0.16065443836333615, -0.34454389289968201], [-0.59210561974607168, -0.61893139561931676, -0.50791482324023085, -0.09657157306056563, -0.92842763863945543], [-0.55407896300662629, -0.48587266107393057, -0.033263770580520768, -0.73916385017619857, -0.80943019498550961]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_Symbol_rank2(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s0=numpy.array(0.553744053649)
      s1=numpy.array([[0.78508661484966735, -0.13282021484060946, 0.080762052008032992, 0.31675423801256142, 0.14154360973828073], [0.3233923757821513, 0.76731835699484852, 0.88247585339559564, -0.16884945044945954, -0.93909175597530292], [0.63753886346886302, 0.85318880780881323, -0.21407908853045621, 0.9718202246394636, -0.81999239895477283], [-0.60182794091406921, 0.81031177379846175, -0.40194636026423214, -0.96022110409797823, -0.53718276541033338]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.55374405364888002, -0.13282021484060946, 0.080762052008032992, 0.31675423801256142, 0.14154360973828073], [0.3233923757821513, 0.55374405364888002, 0.55374405364888002, -0.16884945044945954, -0.93909175597530292], [0.55374405364888002, 0.55374405364888002, -0.21407908853045621, 0.55374405364888002, -0.81999239895477283], [-0.60182794091406921, 0.55374405364888002, -0.40194636026423214, -0.96022110409797823, -0.53718276541033338]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s0=numpy.array([[-0.2160407683970067, 0.86142823109778233, -0.20040152737300576, -0.59160311077949168, -0.084238400478369169], [0.489989604384508, 0.26502120699648879, -0.08202339658126534, -0.52350143495457635, 0.25159434223235722], [-0.50930471227441187, -0.82355283079705077, 0.20584721779704962, 0.87149794723569074, -0.68553004657688499], [0.84188076660872069, -0.82317118978757775, -0.2815158862573468, -0.045441482511855202, -0.6183268516584044]])
      s1=numpy.array([[-0.5541478036764893, 0.2493217340600753, -0.25751469103349778, 0.74781576682290907, 0.58842601404000905], [-0.80986164315824305, -0.44599033376942354, -0.014580929698750689, 0.47355214011885294, -0.73674403198211791], [0.78244509959728026, 0.40165287280854245, 0.2471650391278466, -0.32921210105073584, 0.92792275010812064], [-0.71433695514678464, -0.03367814974313621, 0.021148443388798732, 0.79448859068214173, 0.73705684952688122]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.5541478036764893, 0.2493217340600753, -0.25751469103349778, -0.59160311077949168, -0.084238400478369169], [-0.80986164315824305, -0.44599033376942354, -0.08202339658126534, -0.52350143495457635, -0.73674403198211791], [-0.50930471227441187, -0.82355283079705077, 0.20584721779704962, -0.32921210105073584, -0.68553004657688499], [-0.71433695514678464, -0.82317118978757775, -0.2815158862573468, -0.045441482511855202, -0.6183268516584044]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_constData_rank2(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[0.99906786880902154, -0.44159308764307803, -0.52888876553356123, 0.20110420513552296, -0.52462322519261284], [0.68733652186888183, 0.19502609708897412, 0.50324475331187091, -0.23073274429057533, -0.95923286033003108], [-0.15310927371424388, 0.64360368092845066, 0.016385518666198973, 0.25947078188689821, -0.17503879449265081], [0.32624578285984907, 0.076317233913973714, 0.78120128702405278, 0.51010004879539994, 0.57167281131945447]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array(0.563414382256)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.5634143822558999, -0.44159308764307803, -0.52888876553356123, 0.20110420513552296, -0.52462322519261284], [0.5634143822558999, 0.19502609708897412, 0.50324475331187091, -0.23073274429057533, -0.95923286033003108], [-0.15310927371424388, 0.5634143822558999, 0.016385518666198973, 0.25947078188689821, -0.17503879449265081], [0.32624578285984907, 0.076317233913973714, 0.5634143822558999, 0.51010004879539994, 0.5634143822558999]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[1.5646820216153046e-05, -0.98915096740318864, 0.19454815391558067, -0.14550920357650043, 0.85205323396704835], [-0.19295264257115941, -0.99903727050689395, -0.42391999202575503, -0.57719822722874992, -0.40054195497573808], [0.43775624689419712, 0.079765453818418042, -0.40706622049607155, 0.8642630592772873, 0.77594007264993436], [0.084946492056182432, -0.91970638389595916, 0.039287554388367507, 0.49496187943842962, -0.019758686779552725]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[0.63001442397994678, -0.0024790187730288071, 0.067032948223904265, -0.21873627007538943, 0.46540529920452012], [-0.43059212000384739, -0.35206174914493249, 0.19012862391627849, 0.75269996268980499, -0.48063235316222053], [-0.65427342508348096, 0.65343784164054464, -0.54840701455638485, -0.82854386365321253, 0.18955278662017361], [-0.66284686577165264, 0.65520157988919703, -0.27937386078489546, 0.45498722759554022, -0.60546486094530483]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[1.5646820216153046e-05, -0.98915096740318864, 0.067032948223904265, -0.21873627007538943, 0.46540529920452012], [-0.43059212000384739, -0.99903727050689395, -0.42391999202575503, -0.57719822722874992, -0.48063235316222053], [-0.65427342508348096, 0.079765453818418042, -0.54840701455638485, -0.82854386365321253, 0.18955278662017361], [-0.66284686577165264, -0.91970638389595916, -0.27937386078489546, 0.45498722759554022, -0.60546486094530483]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_expandedData_rank2(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[0.1821759528173359, -0.069893890636752998, 0.5566613903782629, 0.30826869254908917, 0.13574343836443625], [0.51552309524498652, -0.48739954866362267, -0.52269003131510505, 0.057116845576091535, -0.22065743173462216], [0.22984280093533682, 0.89083114702180688, -0.46764072509277943, -0.2808250300741697, -0.55024440282587528], [0.030819701345027095, -0.50842963845241029, -0.76362083964204386, 0.63185527486276993, 0.11984189558752467]])+(1.-msk_arg1)*numpy.array([[0.66557712795475976, 0.90196609934745076, 0.82525312396517458, 0.8275544120167595, -0.51183904370284949], [0.36445709204720167, -0.60041803695667229, 0.020959508473389787, 0.88506936385882562, 0.036685288356346213], [0.16081362929870702, -0.23443663323744324, 0.64281514128929729, 0.35620003935176792, 0.79021675559972526], [0.20062618513106489, -0.77028277451061178, 0.39985684007695377, 0.91854308553109165, 0.87833396162892141]])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.940100819606)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.1821759528173359, -0.069893890636752998, 0.5566613903782629, 0.30826869254908917, 0.13574343836443625], [0.51552309524498652, -0.48739954866362267, -0.52269003131510505, 0.057116845576091535, -0.22065743173462216], [0.22984280093533682, 0.89083114702180688, -0.46764072509277943, -0.2808250300741697, -0.55024440282587528], [0.030819701345027095, -0.50842963845241029, -0.76362083964204386, 0.63185527486276993, 0.11984189558752467]])+(1.-msk_ref)*numpy.array([[0.66557712795475976, 0.90196609934745076, 0.82525312396517458, 0.8275544120167595, -0.51183904370284949], [0.36445709204720167, -0.60041803695667229, 0.020959508473389787, 0.88506936385882562, 0.036685288356346213], [0.16081362929870702, -0.23443663323744324, 0.64281514128929729, 0.35620003935176792, 0.79021675559972526], [0.20062618513106489, -0.77028277451061178, 0.39985684007695377, 0.91854308553109165, 0.87833396162892141]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.75385858642949888, -0.63713761817652403, 0.88494606551117139, 0.14852944876247043, 0.73314308303735243], [-0.68910878899737815, -0.34559443311969584, 0.35754066202044243, -0.22653487734945332, -0.37729422946987712], [-0.91914619245150098, -0.023463532139722965, 0.061599834762514849, -0.19448782363028072, 0.33440011113641943], [-0.33485348203346033, -0.11294315458683113, -0.64382096377006515, -0.43331792555460802, -0.49206897919141035]])+(1.-msk_arg1)*numpy.array([[-0.92563442087633918, -0.3437682590714648, 0.74084654694512664, 0.50577029727614242, 0.050787286047057956], [-0.69070016532606648, -0.89541108830879046, 0.32279400724886576, -0.28143681089833694, -0.58019870187125955], [0.82172186416984339, 0.61285664676996388, -0.050948399676204126, 0.38702202486933102, -0.23491005952379895], [0.72042800285121023, 0.67738376101150433, 0.28841104445563182, -0.52048099541822368, 0.70705763871301697]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[0.60912904475631202, -0.029723786493243587, -0.71439142059107819, 0.64391876532895576, 0.35340090699270754], [0.035933509239176153, 0.98464717327947615, 0.31041025526901733, -0.46401814181314016, -0.91982693157130391], [0.87328267388118497, -0.928767033856714, -0.33539718376298455, 0.57028238486548766, 0.44007159510222182], [-0.32104479736619651, 0.40193036796783344, 0.57898364989774831, 0.1188995212985049, 0.42637939587703011]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.75385858642949888, -0.63713761817652403, -0.71439142059107819, 0.14852944876247043, 0.35340090699270754], [-0.68910878899737815, -0.34559443311969584, 0.31041025526901733, -0.46401814181314016, -0.91982693157130391], [-0.91914619245150098, -0.928767033856714, -0.33539718376298455, -0.19448782363028072, 0.33440011113641943], [-0.33485348203346033, -0.11294315458683113, -0.64382096377006515, -0.43331792555460802, -0.49206897919141035]])+(1.-msk_ref)*numpy.array([[-0.92563442087633918, -0.3437682590714648, -0.71439142059107819, 0.50577029727614242, 0.050787286047057956], [-0.69070016532606648, -0.89541108830879046, 0.31041025526901733, -0.46401814181314016, -0.91982693157130391], [0.82172186416984339, -0.928767033856714, -0.33539718376298455, 0.38702202486933102, -0.23491005952379895], [-0.32104479736619651, 0.40193036796783344, 0.28841104445563182, -0.52048099541822368, 0.42637939587703011]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_array_rank3(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[-0.75233499220445221, -0.46112388504099933], [0.82719348900310341, 0.45953661558500136]], [[0.82643293271788387, 0.34035688759844374], [0.72912088026214827, -0.3098119531411665]], [[0.55427045987071888, -0.16073666721247992], [0.4620603729023538, 0.88307606447140508]], [[0.31470218277427842, 0.046553703022481407], [-0.69268196813109273, 0.25312652260358792]], [[-0.72264074096278264, -0.27877143843863927], [0.68535564716225417, 0.79208138559882024]], [[-0.28415295682376507, -0.95688682412938642], [0.97819407538658809, -0.56382635113250945]]])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.0647377774171)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.75233499220445221, -0.46112388504099933], [0.064737777417098297, 0.064737777417098297]], [[0.064737777417098297, 0.064737777417098297], [0.064737777417098297, -0.3098119531411665]], [[0.064737777417098297, -0.16073666721247992], [0.064737777417098297, 0.064737777417098297]], [[0.064737777417098297, 0.046553703022481407], [-0.69268196813109273, 0.064737777417098297]], [[-0.72264074096278264, -0.27877143843863927], [0.064737777417098297, 0.064737777417098297]], [[-0.28415295682376507, -0.95688682412938642], [0.064737777417098297, -0.56382635113250945]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_array_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([[[0.74643860451888577, -0.99178109629592703], [0.02575023687437672, -0.3577748071913478]], [[0.3555018679365729, 0.18368495393700468], [-0.443383706407948, -0.3796647247224838]], [[0.27411997499535157, -0.89549230692372639], [-0.97860014563345765, -0.66048624522897081]], [[-0.8456078797189952, -0.51284578613153498], [0.32157427446725784, -0.86337860907575692]], [[-0.58827796959062884, 0.94925047400320239], [0.744796119854918, 0.61085006729128577]], [[0.21783451028688328, 0.72175836456968656], [0.41348928791037176, -0.45967815264504064]]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[[-0.65740023961930527, -0.82860950438472192], [-0.43663136580970541, 0.75836865672741793]], [[-0.78056009558678552, 0.31264409273690452], [-0.84881732194632153, -0.56065898065582953]], [[0.69175646573867899, 0.19809126334137894], [-0.69387176459593247, -0.18543661239163844]], [[0.081625812770727002, 0.25480666219549319], [-0.91206446781298389, -0.072878786790836481]], [[0.93826056892038312, -0.44327380689293761], [-0.6611325806757431, -0.030520237756865631]], [[-0.53959204385467441, 0.29153585836939544], [0.92406312973963667, 0.45536263836177771]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-0.65740023961930527, -0.99178109629592703], [-0.43663136580970541, -0.3577748071913478]], [[-0.78056009558678552, 0.18368495393700468], [-0.84881732194632153, -0.56065898065582953]], [[0.27411997499535157, -0.89549230692372639], [-0.97860014563345765, -0.66048624522897081]], [[-0.8456078797189952, -0.51284578613153498], [-0.91206446781298389, -0.86337860907575692]], [[-0.58827796959062884, -0.44327380689293761], [-0.6611325806757431, -0.030520237756865631]], [[-0.53959204385467441, 0.29153585836939544], [0.41348928791037176, -0.45967815264504064]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_Symbol_rank3(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.597982471631)
      s1=numpy.array([[[0.64323982778723998, 0.26682376753962367], [0.71598414738768312, -0.60804662428170175]], [[-0.97202236599458725, -0.45346337501581169], [-0.65716212242926764, -0.11730889743599016]], [[0.90987420997666013, 0.029657982972106867], [-0.73164422357884051, -0.9529060945153649]], [[0.26907733539704659, -0.094426102477775276], [-0.15173943298903314, 0.44863811261417341]], [[0.57286626847519329, -0.90612308321086421], [-0.15147520773063383, -0.51925250195261619]], [[0.12831907136118725, 0.26553057704805116], [-0.73422922358282361, 0.14145706160326843]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.59798247163114748, -0.59798247163114748], [-0.59798247163114748, -0.60804662428170175]], [[-0.97202236599458725, -0.59798247163114748], [-0.65716212242926764, -0.59798247163114748]], [[-0.59798247163114748, -0.59798247163114748], [-0.73164422357884051, -0.9529060945153649]], [[-0.59798247163114748, -0.59798247163114748], [-0.59798247163114748, -0.59798247163114748]], [[-0.59798247163114748, -0.90612308321086421], [-0.59798247163114748, -0.59798247163114748]], [[-0.59798247163114748, -0.59798247163114748], [-0.73422922358282361, -0.59798247163114748]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_Symbol_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s0=numpy.array([[[-0.69830331527963763, -0.93324718177548882], [0.6906579283431713, -0.058404185247855178]], [[-0.84571971833057602, -0.17135972857460713], [-0.73170565625285433, -0.91804404263878991]], [[0.13493255506303203, -0.46152400366323199], [-0.010807188476345431, -0.76905715659165552]], [[0.78385280941253122, -0.61680981988182904], [0.48478097100094319, -0.77957391192737435]], [[0.15587311575795249, 0.065330336281540502], [0.84599968014576565, 0.53482048146408578]], [[-0.098660829498766089, 0.68716513611085728], [0.88683923383570207, -0.55316786301826504]]])
      s1=numpy.array([[[0.3892617078163263, 0.87408204103211817], [0.4780588325552666, -0.28189022099605499]], [[-0.11453995007251572, 0.69005974266399472], [0.59575896832598185, 0.82577476557811025]], [[0.1553098246491984, -0.16249470526125576], [0.97665346402124498, -0.11218746013560943]], [[-0.86049579316139702, -0.082719636090183846], [-0.28281095389592048, -0.92025207705426726]], [[0.66504204902543584, -0.53112384752013786], [0.37524576184604141, 0.99340609956043324]], [[0.46404953521408498, 0.51384942840855019], [0.6997888093765301, 0.7163841817206591]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-0.69830331527963763, -0.93324718177548882], [0.4780588325552666, -0.28189022099605499]], [[-0.84571971833057602, -0.17135972857460713], [-0.73170565625285433, -0.91804404263878991]], [[0.13493255506303203, -0.46152400366323199], [-0.010807188476345431, -0.76905715659165552]], [[-0.86049579316139702, -0.61680981988182904], [-0.28281095389592048, -0.92025207705426726]], [[0.15587311575795249, -0.53112384752013786], [0.37524576184604141, 0.53482048146408578]], [[-0.098660829498766089, 0.51384942840855019], [0.6997888093765301, -0.55316786301826504]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_constData_rank3(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[0.30515263849005758, -0.35636360965679614], [-0.077435654699747447, -0.45836458103211819]], [[-0.029683390485448768, 0.7759015023739011], [0.86175012376182569, 0.5368841535280191]], [[0.464038863537652, 0.32731482139425783], [0.091898206352594158, -0.37304464166521867]], [[-0.55157866907591258, 0.66823448349646575], [0.43244015505774613, -0.45302183429681753]], [[-0.10761378445506686, 0.13192542172979271], [-0.40065056166263546, 0.04570695267114977]], [[0.91101978919223181, 0.75382789904263992], [0.42236963047239118, 0.98444448917374294]]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array(0.274940952503)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.27494095250320472, -0.35636360965679614], [-0.077435654699747447, -0.45836458103211819]], [[-0.029683390485448768, 0.27494095250320472], [0.27494095250320472, 0.27494095250320472]], [[0.27494095250320472, 0.27494095250320472], [0.091898206352594158, -0.37304464166521867]], [[-0.55157866907591258, 0.27494095250320472], [0.27494095250320472, -0.45302183429681753]], [[-0.10761378445506686, 0.13192542172979271], [-0.40065056166263546, 0.04570695267114977]], [[0.27494095250320472, 0.27494095250320472], [0.27494095250320472, 0.27494095250320472]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_constData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([[[0.21001661054042553, 0.35252720412832428], [0.79423559066903993, -0.74884575366578532]], [[-0.7594753530563243, 0.27546596802435008], [0.79102400477685553, -0.058517145659837899]], [[-0.62038231203127725, 0.24046375660873309], [0.37979198093209643, -0.40202885774007191]], [[0.077888389457567486, -0.31364376677877681], [0.73754224127803347, 0.56008226319306265]], [[-0.32111451542809166, 0.27084327931367547], [-0.74565870802083634, 0.36490120412037896]], [[0.65489076840669802, 0.69246188119016683], [0.82161716841191401, 0.47655868299765669]]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[0.054586189539438612, 0.36966757664655425], [-0.31088584199915781, -0.43395152634905965]], [[0.13709472414102186, 0.87893759810542926], [0.39583426081277695, -0.59373926301848789]], [[-0.60030736016866415, -0.9282044284252593], [0.83186390527938148, 0.093038637508932709]], [[0.89521451783283079, 0.43530602867945323], [0.92048555804563947, 0.24751061798192042]], [[-0.50798450541561757, 0.5395195119202103], [-0.95380417939962925, -0.26292885863499782]], [[0.34690946703237668, 0.39255620242620393], [0.82955418403976688, 0.52255000205537772]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.054586189539438612, 0.35252720412832428], [-0.31088584199915781, -0.74884575366578532]], [[-0.7594753530563243, 0.27546596802435008], [0.39583426081277695, -0.59373926301848789]], [[-0.62038231203127725, -0.9282044284252593], [0.37979198093209643, -0.40202885774007191]], [[0.077888389457567486, -0.31364376677877681], [0.73754224127803347, 0.24751061798192042]], [[-0.50798450541561757, 0.27084327931367547], [-0.95380417939962925, -0.26292885863499782]], [[0.34690946703237668, 0.39255620242620393], [0.82161716841191401, 0.47655868299765669]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_expandedData_rank3(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.013443029238729753, 0.155466670359107], [-0.79444670541052975, -0.090937875507068888]], [[0.71319989098181513, 0.88748835068920484], [-0.42950136717655241, 0.28400137380393908]], [[0.53563083898294805, 0.32801646871022627], [0.53013991663198068, -0.44662188920984613]], [[0.44363344153685502, 0.01153305780694347], [-0.72321948496492716, 0.78001196453245725]], [[-0.44976848703046901, 0.13626456865097691], [0.23844173593371343, 0.83392514108826421]], [[0.51691820163451085, -0.12080360094835885], [-0.70201017485712014, -0.079696081306193323]]])+(1.-msk_arg1)*numpy.array([[[-0.46331132111775108, 0.07723697330694379], [-0.31519839959715545, 0.11730041554212711]], [[0.87379282272834624, 0.71266585392123649], [0.32924837763429338, 0.93654660622062869]], [[0.013195071487158039, 0.54670358016535392], [0.48479532296587169, -0.44194732972788531]], [[-0.25538041084510898, 0.18978785428940648], [-0.0054185165291433801, 0.69644027411568032]], [[-0.75541233296045895, -0.27855778874339476], [-0.26837487707348928, -0.077942992766691477]], [[-0.14843499612967892, 0.56121586269398605], [0.80999769115642284, -0.28477359413322634]]])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.379858935835)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.013443029238729753, 0.155466670359107], [-0.79444670541052975, -0.090937875507068888]], [[0.37985893583535302, 0.37985893583535302], [-0.42950136717655241, 0.28400137380393908]], [[0.37985893583535302, 0.32801646871022627], [0.37985893583535302, -0.44662188920984613]], [[0.37985893583535302, 0.01153305780694347], [-0.72321948496492716, 0.37985893583535302]], [[-0.44976848703046901, 0.13626456865097691], [0.23844173593371343, 0.37985893583535302]], [[0.37985893583535302, -0.12080360094835885], [-0.70201017485712014, -0.079696081306193323]]])+(1.-msk_ref)*numpy.array([[[-0.46331132111775108, 0.07723697330694379], [-0.31519839959715545, 0.11730041554212711]], [[0.37985893583535302, 0.37985893583535302], [0.32924837763429338, 0.37985893583535302]], [[0.013195071487158039, 0.37985893583535302], [0.37985893583535302, -0.44194732972788531]], [[-0.25538041084510898, 0.18978785428940648], [-0.0054185165291433801, 0.37985893583535302]], [[-0.75541233296045895, -0.27855778874339476], [-0.26837487707348928, -0.077942992766691477]], [[-0.14843499612967892, 0.37985893583535302], [0.37985893583535302, -0.28477359413322634]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank3_expandedData_rank3(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.6078150761407406, 0.34668713082215286], [-0.36074456547295108, -0.39966212795093781]], [[-0.5391598652376306, -0.46396242702578738], [-0.82893323767164517, 0.74864915035055657]], [[0.75496479769815128, 0.75204489197311952], [0.58299956582946755, 0.55327056179741629]], [[-0.33569447438923494, -0.19954531244981188], [-0.16733266496413268, -0.81677106334913208]], [[-0.92466054869390057, 0.97682360256510825], [0.5857917531271517, 0.227043165884113]], [[0.88574234603959923, 0.45806711003646505], [0.90733237126071375, -0.67787823718481488]]])+(1.-msk_arg1)*numpy.array([[[0.1475968956894258, 0.22202788288127739], [-0.15853980667737089, -0.40783106072937114]], [[-0.74612133447809037, -0.29220039460230773], [0.72192104921974298, 0.33044698885711621]], [[0.1187444095523984, -0.66020982571205189], [0.66150010802499692, 0.17867465834235485]], [[0.43724641409722853, 0.78780304723699834], [-0.99106683575169652, -0.21835121136282232]], [[0.34047406503108002, 0.73911353651647316], [-0.51451332543614692, 0.84960472557229183]], [[0.52485332450578404, 0.40013876611501753], [0.94519657668679891, 0.15625178308914434]]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[[-0.20135630861469656, -0.4234821507204638], [-0.49814130837842496, 0.65281493152379411]], [[0.61772474453098214, -0.014224601736481324], [0.90707866436741691, -0.58751272646404851]], [[0.22102008650448357, -0.81245084015201163], [-0.53510527061266044, -0.76549775739399872]], [[0.92809545164041563, 0.57485939133914821], [0.79305178061759385, -0.22794646822300169]], [[0.83628603372420129, -0.47071200230895305], [-0.76587170884717004, -0.16136880921939323]], [[0.54698602603780566, -0.38846395198407158], [0.034206328546134257, 0.19265006449550537]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.6078150761407406, -0.4234821507204638], [-0.49814130837842496, -0.39966212795093781]], [[-0.5391598652376306, -0.46396242702578738], [-0.82893323767164517, -0.58751272646404851]], [[0.22102008650448357, -0.81245084015201163], [-0.53510527061266044, -0.76549775739399872]], [[-0.33569447438923494, -0.19954531244981188], [-0.16733266496413268, -0.81677106334913208]], [[-0.92466054869390057, -0.47071200230895305], [-0.76587170884717004, -0.16136880921939323]], [[0.54698602603780566, -0.38846395198407158], [0.034206328546134257, -0.67787823718481488]]])+(1.-msk_ref)*numpy.array([[[-0.20135630861469656, -0.4234821507204638], [-0.49814130837842496, -0.40783106072937114]], [[-0.74612133447809037, -0.29220039460230773], [0.72192104921974298, -0.58751272646404851]], [[0.1187444095523984, -0.81245084015201163], [-0.53510527061266044, -0.76549775739399872]], [[0.43724641409722853, 0.57485939133914821], [-0.99106683575169652, -0.22794646822300169]], [[0.34047406503108002, -0.47071200230895305], [-0.76587170884717004, -0.16136880921939323]], [[0.52485332450578404, -0.38846395198407158], [0.034206328546134257, 0.15625178308914434]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_array_rank4(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[-0.053119268720309476, -0.92241077701902618, 0.3828209332719954, 0.38331704025454671], [-0.5510070335457784, 0.25502431156654715, 0.8750375853648793, -0.86847802906334493], [0.39234781585988965, -0.38013566975891999, -0.038980729273602632, 0.72663088591219327]], [[-0.54098183850003601, -0.4082000652613007, 0.33227524270115349, 0.63441135029477325], [-0.43882814452532926, 0.46802126860379967, -0.086509573182057986, -0.74603157109211415], [0.23322290877322982, -0.22911037055438288, 0.35182240303554035, -0.4971611244552061]]], [[[-0.24927795462848024, -0.0035978195496921828, -0.37692789359142775, 0.5245643979117125], [0.58418870997019368, -0.85531118591859823, 0.71029463768053747, -0.83795038606840433], [0.52515997634509404, 0.1346560570400881, 0.51184839569902008, 0.74798050348858691]], [[-0.18909808474565737, 0.875039430951013, 0.56443846933378827, 0.72768321467109032], [-0.72942478590716964, 0.98250971875176218, -0.60389462701200514, -0.35566065088398879], [0.82744767496026972, 0.75268285881232444, -0.70185988034355873, -0.48419480538779203]]], [[[-0.044055073358089558, -0.20971328268622114, -0.61857017700182126, -0.017707631698052451], [-0.066662177415284907, -0.29485578595575901, -0.86095351678592813, 0.75204300111556366], [0.00019969664388042041, -0.75634004673159261, -0.59270951750239198, 0.4801126235096802]], [[0.45535924267844963, -0.96538423206684687, 0.50574855625276438, -0.62510443961863671], [0.230512887331348, -0.88036368562618939, 0.78148449977280476, 0.9939394259200085], [0.39243678085307065, 0.67210505075538873, -0.91180484237006643, -0.12400395555630195]]]])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.194484871384)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.053119268720309476, -0.92241077701902618, 0.19448487138447623, 0.19448487138447623], [-0.5510070335457784, 0.19448487138447623, 0.19448487138447623, -0.86847802906334493], [0.19448487138447623, -0.38013566975891999, -0.038980729273602632, 0.19448487138447623]], [[-0.54098183850003601, -0.4082000652613007, 0.19448487138447623, 0.19448487138447623], [-0.43882814452532926, 0.19448487138447623, -0.086509573182057986, -0.74603157109211415], [0.19448487138447623, -0.22911037055438288, 0.19448487138447623, -0.4971611244552061]]], [[[-0.24927795462848024, -0.0035978195496921828, -0.37692789359142775, 0.19448487138447623], [0.19448487138447623, -0.85531118591859823, 0.19448487138447623, -0.83795038606840433], [0.19448487138447623, 0.1346560570400881, 0.19448487138447623, 0.19448487138447623]], [[-0.18909808474565737, 0.19448487138447623, 0.19448487138447623, 0.19448487138447623], [-0.72942478590716964, 0.19448487138447623, -0.60389462701200514, -0.35566065088398879], [0.19448487138447623, 0.19448487138447623, -0.70185988034355873, -0.48419480538779203]]], [[[-0.044055073358089558, -0.20971328268622114, -0.61857017700182126, -0.017707631698052451], [-0.066662177415284907, -0.29485578595575901, -0.86095351678592813, 0.19448487138447623], [0.00019969664388042041, -0.75634004673159261, -0.59270951750239198, 0.19448487138447623]], [[0.19448487138447623, -0.96538423206684687, 0.19448487138447623, -0.62510443961863671], [0.19448487138447623, -0.88036368562618939, 0.19448487138447623, 0.19448487138447623], [0.19448487138447623, 0.19448487138447623, -0.91180484237006643, -0.12400395555630195]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array([[[[0.13438719170942681, 0.55325255665983653, -0.56793379984286818, 0.14748542006341059], [-0.27654354801362047, 0.57972677985669874, -0.5903455518753693, 0.86790294925717593], [-0.51899222080596186, 0.69588506579634712, -0.38613688718524508, 0.91129225159092964]], [[0.58501352054307532, -0.12636887698543053, -0.65997757689059133, -0.63108045786607758], [-0.53496023065077392, -0.97039202864863316, 0.30168880318821611, -0.85198585395584181], [0.46344280096690471, -0.1131378767946527, -0.73179323733731816, 0.29069085083060986]]], [[[-0.64687525849972505, 0.8135233981002532, 0.25577168917939885, -0.269872244345853], [0.53378142324564171, -0.35960748636479956, 0.13359614337550285, -0.18554447218986181], [-0.67540948250614274, 0.39871343727715014, 0.68695420484573111, -0.22681290619297911]], [[0.68969855167760241, 0.8100992328455805, -0.50024444504854815, -0.31461725481032921], [0.082198645206028376, -0.44950302897321359, 0.35378151539905867, 0.32113177218687561], [0.35423821152812041, -0.68274833787931, -0.041195996327842987, 0.83655605095769059]]], [[[0.29993587789867937, -0.97042548758468272, -0.65194059275675986, -0.3127934095983127], [-0.69578271858527407, 0.57514096037341234, 0.68402484605910585, -0.49192346119431418], [0.95966848146278716, -0.90286565545597552, -0.49732293555120366, -0.0830768002135156]], [[-0.27636308754859251, 0.13392417173934534, -0.075851548325129636, 0.56651336672099117], [-0.58365817185789237, 0.39260147313501359, -0.81159229332436089, -0.49800878559760231], [-0.89990186047829512, 0.81053474877003318, -0.99503866066177582, 0.57429878390007238]]]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[0.60592503592684688, -0.038525229351737744, 0.63143323852524924, 0.11920238562721197], [0.096656563418305197, 0.096480075254608666, -0.037336412987487577, -0.5578875048957721], [0.65813587880623836, 0.5618299430624305, -0.68741076224373798, 0.1380662136924482]], [[-0.21954862069258385, -0.60097237448926832, -0.60228589501166119, 0.99248047001029094], [-0.036735558721638917, -0.53701069689419656, -0.63210183737965386, -0.21271854355504916], [0.44631967723409449, 0.83363387260876221, 0.28694751487177239, -0.8828410361557808]]], [[[0.62591268493090069, -0.41122840927898308, 0.79619893492046923, -0.57876412637139718], [0.72349055157666675, 0.929606655800159, 0.18827120673200248, 0.044954018627960668], [0.18820503061495386, -0.37638548922776338, -0.23162271366455989, 0.33594135531668234]], [[-0.61094811453720688, -0.04178672915854631, 0.31242927150407529, -0.75523472796109359], [0.77406174960822183, 0.95816195427536255, 0.5568789528307907, -0.32041137403714792], [-0.016345731986376677, -0.18178972555623929, -0.67343612867713532, 0.35646602822470785]]], [[[-0.60686695951153613, 0.78186194180459045, 0.79669312033452844, -0.85729930405372778], [0.4453454269511794, 0.98725000074070168, -0.83215987340465691, -0.45012678469591538], [0.08714519207895921, 0.14711146959689625, 0.011389703550661823, 0.20434472845024132]], [[-0.54273698085065658, 0.82524269430670261, -0.88464324376025649, 0.70534802057184187], [0.62149667618964521, -0.20453913126235745, 0.57183787374238237, 0.54170817919705549], [0.79289172658516871, 0.94795991504588439, -0.61142932891876378, 0.52883854527635421]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.13438719170942681, -0.038525229351737744, -0.56793379984286818, 0.11920238562721197], [-0.27654354801362047, 0.096480075254608666, -0.5903455518753693, -0.5578875048957721], [-0.51899222080596186, 0.5618299430624305, -0.68741076224373798, 0.1380662136924482]], [[-0.21954862069258385, -0.60097237448926832, -0.65997757689059133, -0.63108045786607758], [-0.53496023065077392, -0.97039202864863316, -0.63210183737965386, -0.85198585395584181], [0.44631967723409449, -0.1131378767946527, -0.73179323733731816, -0.8828410361557808]]], [[[-0.64687525849972505, -0.41122840927898308, 0.25577168917939885, -0.57876412637139718], [0.53378142324564171, -0.35960748636479956, 0.13359614337550285, -0.18554447218986181], [-0.67540948250614274, -0.37638548922776338, -0.23162271366455989, -0.22681290619297911]], [[-0.61094811453720688, -0.04178672915854631, -0.50024444504854815, -0.75523472796109359], [0.082198645206028376, -0.44950302897321359, 0.35378151539905867, -0.32041137403714792], [-0.016345731986376677, -0.68274833787931, -0.67343612867713532, 0.35646602822470785]]], [[[-0.60686695951153613, -0.97042548758468272, -0.65194059275675986, -0.85729930405372778], [-0.69578271858527407, 0.57514096037341234, -0.83215987340465691, -0.49192346119431418], [0.08714519207895921, -0.90286565545597552, -0.49732293555120366, -0.0830768002135156]], [[-0.54273698085065658, 0.13392417173934534, -0.88464324376025649, 0.56651336672099117], [-0.58365817185789237, -0.20453913126235745, -0.81159229332436089, -0.49800878559760231], [-0.89990186047829512, 0.81053474877003318, -0.99503866066177582, 0.52883854527635421]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_Symbol_rank4(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.673897987823)
      s1=numpy.array([[[[-0.68175202601835028, -0.056378823805597333, 0.62539633200608757, 0.18726729603943371], [0.27916206127919341, 0.5092229383380289, -0.85603996261238269, -0.99830273294482996], [-0.7523920814542906, 0.00041591434089749413, 0.43732034474060644, 0.7113220596030867]], [[0.2816499957120906, -0.5259183793941351, 0.2861865611396246, -0.72660604453973043], [0.11229417482910908, 0.67101590523293408, -0.73067545559749147, 0.063616079097956524], [0.86754698969824928, -0.55967270627170596, 0.75343385665344798, 0.6983566324418149]]], [[[-0.7285053806726709, 0.88126046081945675, -0.61520361548880964, -0.61050172639141631], [0.83041805712492089, 0.059987032504245086, 0.35490870155530563, 0.98739110458053458], [-0.91413630779672816, 0.22260481456808456, 0.99330882776652785, 0.96487416375557689]], [[0.1210855445747494, -0.81783233268424493, 0.64436593941403375, -0.84412126016367361], [-0.80411105553883955, -0.025298360335949122, 0.71079618274963807, 0.34611220417349275], [0.92222544106202187, 0.66591685550103286, -0.58287963876413706, 0.38764650832928438]]], [[[-0.27339420671151604, -0.15103387150681491, -0.62639198169191435, -0.11260323200650957], [-0.50323787353117888, -0.92472506069700455, -0.068001896200677647, -0.6061540428336174], [0.92926307113328521, -0.73195282835004494, -0.44113554497665342, -0.6209881509817281]], [[0.46739689877133395, -0.82204615584291596, -0.03722946033110075, -0.038860048028877658], [0.5015367765601122, 0.24907896027823351, -0.66088912147651224, -0.64614513719936695], [0.56943294915660192, -0.87934443681243435, -0.82419123337134326, -0.29527595751615121]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.68175202601835028, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.67389798782309862, -0.85603996261238269, -0.99830273294482996], [-0.7523920814542906, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862]], [[-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.72660604453973043], [-0.67389798782309862, -0.67389798782309862, -0.73067545559749147, -0.67389798782309862], [-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862]]], [[[-0.7285053806726709, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.91413630779672816, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862]], [[-0.67389798782309862, -0.81783233268424493, -0.67389798782309862, -0.84412126016367361], [-0.80411105553883955, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862]]], [[[-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.92472506069700455, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.73195282835004494, -0.67389798782309862, -0.67389798782309862]], [[-0.67389798782309862, -0.82204615584291596, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.67389798782309862, -0.67389798782309862, -0.67389798782309862], [-0.67389798782309862, -0.87934443681243435, -0.82419123337134326, -0.67389798782309862]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[0.26357025164799763, -0.78663845904346674, 0.45258322041580601, 0.27593296390008826], [0.84188423022013481, -0.14831041384477106, -0.67477666207603093, -0.973055888740745], [0.74914383570015075, -0.47418357690140267, -0.1875193459012503, 0.70352796772900827]], [[-0.23612377674656027, -0.98023158749668671, 0.12675481236541297, 0.78616605607281365], [-0.83581846289419892, 0.33765482600050722, 0.9189700153418765, -0.36827865901202483], [-0.22409456757030566, 0.63066404070172055, 0.80763879515070713, -0.80877370680899086]]], [[[0.25577051769878256, 0.20297911562432969, -0.37403500818132351, 0.71673443579176577], [-0.50562779530403268, -0.32479347571219863, -0.18611249578073363, 0.34312155603339023], [0.37240188028582177, -0.72489718388596458, -0.67462797737732361, -0.17365577631279172]], [[-0.7077175716464843, -0.32882515980132077, 0.46446055916410356, -0.61519757745998649], [-0.93757009237949562, -0.017337466979359339, 0.57364717521806874, 0.2204693955804049], [-0.96187183459970216, 0.19548083233536162, 0.28249660347814998, -0.85771203814934704]]], [[[0.19707429136566379, -0.71407876670775905, 0.20393071128570561, -0.39266521734502002], [-0.7446557350404599, 0.033243680191036962, 0.82626996995832225, -0.14826913809106279], [0.60304197251543745, -0.90571303892110322, -0.29153631186422158, -0.72456685241121477]], [[-0.42658801415702907, -0.76884025047604987, 0.61548132977439995, 0.0016102503587689387], [-0.64953516029139702, -0.60490920379094959, 0.58054143550434123, 0.29071794579918553], [-0.28363847962442423, -0.52485677135922093, -0.83655723625291167, 0.18327051195699617]]]])
      s1=numpy.array([[[[0.054701662456412503, 0.70572233553282082, -0.69397786363840841, 0.19326217604600693], [-0.4917785568152544, -0.7851918340139874, -0.12942311925112615, 0.11905949837529994], [-0.298430624132074, -0.016375538740589279, -0.32326357580183984, -0.30029358680761775]], [[-0.31866104697935449, -0.28890328187430914, -0.39118436829265968, -0.36572827674279185], [-0.93203775560128777, 0.034669656983532526, 0.90897833854307142, 0.12001980821635261], [0.44373188831491395, -0.068100092817357405, -0.69886972180623852, 0.93194760816901767]]], [[[-0.83429064346307324, 0.086729808545102927, -0.6411007209605204, 0.26995595067358624], [0.99542307093051674, 0.37389388120435685, -0.50078798850159223, 0.49657661309945866], [-0.7438017109180286, -0.92624961680093376, -0.15873360062503261, -0.95175843250678227]], [[0.51952862478864925, 0.050817235432958796, 0.70895087413643521, -0.057872664827281817], [-0.83789158896075011, -0.8163157088671753, 0.53791336908082776, 0.47722483170510133], [-0.85496920094368711, 0.78710846829540038, 0.87194757411704105, -0.98206730803265097]]], [[[0.86609577438670371, 0.21592755142298015, 0.62392923564764402, -0.20025031836009388], [-0.20175253772572588, 0.0082588851376395045, 0.57491302492109408, 0.68891634930153955], [-0.96815088595769572, 0.26000171017817042, -0.91688511831124364, 0.66116210277385923]], [[0.094312828417370431, 0.79094067441470806, 0.068756532197648212, 0.629501311302054], [-0.89276924100547661, 0.48445921526170466, -0.87341392387400396, -0.79720919132264778], [-0.50743458704014643, -0.84941824196085469, -0.79842560171712829, -0.11100515531765742]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.054701662456412503, -0.78663845904346674, -0.69397786363840841, 0.19326217604600693], [-0.4917785568152544, -0.7851918340139874, -0.67477666207603093, -0.973055888740745], [-0.298430624132074, -0.47418357690140267, -0.32326357580183984, -0.30029358680761775]], [[-0.31866104697935449, -0.98023158749668671, -0.39118436829265968, -0.36572827674279185], [-0.93203775560128777, 0.034669656983532526, 0.90897833854307142, -0.36827865901202483], [-0.22409456757030566, -0.068100092817357405, -0.69886972180623852, -0.80877370680899086]]], [[[-0.83429064346307324, 0.086729808545102927, -0.6411007209605204, 0.26995595067358624], [-0.50562779530403268, -0.32479347571219863, -0.50078798850159223, 0.34312155603339023], [-0.7438017109180286, -0.92624961680093376, -0.67462797737732361, -0.95175843250678227]], [[-0.7077175716464843, -0.32882515980132077, 0.46446055916410356, -0.61519757745998649], [-0.93757009237949562, -0.8163157088671753, 0.53791336908082776, 0.2204693955804049], [-0.96187183459970216, 0.19548083233536162, 0.28249660347814998, -0.98206730803265097]]], [[[0.19707429136566379, -0.71407876670775905, 0.20393071128570561, -0.39266521734502002], [-0.7446557350404599, 0.0082588851376395045, 0.57491302492109408, -0.14826913809106279], [-0.96815088595769572, -0.90571303892110322, -0.91688511831124364, -0.72456685241121477]], [[-0.42658801415702907, -0.76884025047604987, 0.068756532197648212, 0.0016102503587689387], [-0.89276924100547661, -0.60490920379094959, -0.87341392387400396, -0.79720919132264778], [-0.50743458704014643, -0.84941824196085469, -0.83655723625291167, -0.11100515531765742]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_constData_rank4(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[-0.71623988650501502, -0.97809647811478762, -0.41088406583620718, -0.50434103349055981], [-0.6681545419944781, 0.79606910981899959, 0.14336414183030954, -0.55744193380568574], [0.49332855934927067, 0.81979073147707004, -0.35221241536279346, -0.29610978571799884]], [[0.12561645801743504, 0.046278003590296546, 0.071274968867319322, 0.27669323505850896], [-0.17442228216960531, 0.27781614674485433, -0.41900168936673343, -0.45889775008029998], [0.18511023186124276, -0.21421467673375028, 0.75711718005708217, 0.70433789515563916]]], [[[0.40477170479892077, -0.79288875889985722, -0.24838366607126616, 0.71551187509153569], [0.77174598951688633, -0.17533285787080932, 0.77758436546693632, -0.17257670750668663], [0.17667931139484749, 0.49357232008446417, 0.48520492062617238, 0.86002148878765139]], [[0.87804337891457651, 0.70957124488683299, -0.64766406077979832, -0.80331875277749099], [0.98007750401636029, -0.97457588797496619, 0.14363768233785112, 0.63354793415119182], [0.95581840855515976, -0.43887796576372851, -0.7722717567839732, -0.7556141305138302]]], [[[-0.26188712062195729, -0.85855686433801437, 0.36441510530353982, 0.25181980179936114], [-0.58605631751836995, -0.53044262712366996, 0.085197242770796322, -0.37452649564689278], [-0.32555359236457737, 0.32890962328887441, 0.98477413167721073, 0.49770248964385422]], [[0.70439412151119796, 0.073225696435214926, -0.6271192581796452, 0.55276549840617029], [-0.043414834586503526, -0.21742505125938627, -0.70643298766455609, 0.70648027719443673], [-0.0033353168889800333, -0.6557398910004959, 0.43594407920422062, -0.89235761347169595]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array(-0.446183806183)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.71623988650501502, -0.97809647811478762, -0.44618380618268927, -0.50434103349055981], [-0.6681545419944781, -0.44618380618268927, -0.44618380618268927, -0.55744193380568574], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927]], [[-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.45889775008029998], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927]]], [[[-0.44618380618268927, -0.79288875889985722, -0.44618380618268927, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927]], [[-0.44618380618268927, -0.44618380618268927, -0.64766406077979832, -0.80331875277749099], [-0.44618380618268927, -0.97457588797496619, -0.44618380618268927, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.7722717567839732, -0.7556141305138302]]], [[[-0.44618380618268927, -0.85855686433801437, -0.44618380618268927, -0.44618380618268927], [-0.58605631751836995, -0.53044262712366996, -0.44618380618268927, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.44618380618268927, -0.44618380618268927]], [[-0.44618380618268927, -0.44618380618268927, -0.6271192581796452, -0.44618380618268927], [-0.44618380618268927, -0.44618380618268927, -0.70643298766455609, -0.44618380618268927], [-0.44618380618268927, -0.6557398910004959, -0.44618380618268927, -0.89235761347169595]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(numpy.array([[[[-0.17300875012077244, 0.67822759414656364, 0.5935447002295704, -0.68733237636656952], [-0.36480762227014907, 0.10153124480094577, 0.58745550704138561, -0.053922254581248552], [-0.905408204530181, 0.37084631210451269, 0.14013834282753734, -0.66700102284422336]], [[0.65948242918790334, -0.017148738521686857, -0.88227132985762879, 0.69132571840504298], [-0.61219735633040351, -0.72178269592924438, 0.35604692410748418, -0.49408956197804255], [0.94286041107526475, 0.83133282322922231, -0.53710983939223822, -0.42939055516195834]]], [[[-0.27052320099413452, 0.92419060656715746, -0.49468050029238886, 0.63406410305544858], [-0.10455789959194051, -0.36724922031900431, 0.72995418874837337, -0.94411291348111348], [-0.42153594282091333, 0.018368578784250245, 0.069732252881098189, 0.14632261604402408]], [[-0.7200005962006899, 0.94497467957072034, -0.94001900052358556, 0.1656990104425855], [-0.61834077710899926, -0.26679803751456599, -0.31024081767630474, 0.62470469377980775], [0.66456896289612644, 0.11254821792500658, 0.0087506831843955357, -0.6888316639800931]]], [[[0.29271925308725977, -0.38072891167693212, 0.13992879014213622, -0.033136690796282009], [-0.079481263615651221, -0.54541478113479802, -0.36844824135770393, -0.28466123501693152], [0.3880143820700781, 0.60581816283893963, 0.83866644477483332, 0.3658434484483768]], [[-0.79818626146478522, 0.72973231254160154, -0.38334816505750902, 0.74931891869908918], [-0.10472192155407511, 0.32340748763042737, -0.9733026511151559, 0.38151924414321381], [0.16685146063995404, -0.071374758989875442, -0.91491461674939001, -0.7345879366753163]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[-0.65055817495331558, 0.64080112000377176, -0.27614758128774786, 0.34731358448573535], [-0.79543694811639032, -0.71089822437189198, 0.81015288889571124, -0.51987645916561043], [0.78610029741781351, 0.4882168342204003, 0.10620412201617646, 0.94423687618599117]], [[0.822633216760011, -0.42632467643266447, 0.24029783434781149, 0.68836320623684122], [-0.35951614161175627, -0.23200670144796187, -0.023790378413287128, 0.40971825187591482], [-0.2807995964783081, 0.97704594829862401, 0.18660648942711155, 0.97246895433064839]]], [[[-0.5228509035886959, -0.18471417404942181, -0.26459335063398504, 0.52413117800127984], [0.93834818349937876, 0.4156989628245209, -0.77463887644261042, -0.73762788538767765], [-0.065942932991239989, -0.23504651378885066, -0.61899584152219456, 0.49756636928643561]], [[-0.39709799276445934, 0.57986371483229004, 0.94152108972024018, -0.86048741907772919], [0.4238997414165282, 0.91076871530998638, 0.13992527669542554, 0.71789886940040182], [0.85911838731503987, 0.19371400798994776, -0.49566825708580775, 0.61304272602805088]]], [[[-0.11701488856899567, 0.17747907781544714, 0.73299482361771418, -0.30678825648889552], [-0.95635410346269367, -0.15739261077626487, 0.0073975234657399636, -0.6833048980765557], [-0.46606158731755865, 0.20257219663303383, -0.057185674829863808, -0.24300444348168115]], [[-0.33739457955018737, -0.36096591178371784, 0.97155971010381448, -0.95130351297090199], [0.00066588787013621697, 0.49817225413197441, -0.37899161215359878, 0.65255229667389769], [0.23824513832572847, 0.3865746627501867, 0.31354968001144501, -0.26283047959961658]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.65055817495331558, 0.64080112000377176, -0.27614758128774786, -0.68733237636656952], [-0.79543694811639032, -0.71089822437189198, 0.58745550704138561, -0.51987645916561043], [-0.905408204530181, 0.37084631210451269, 0.10620412201617646, -0.66700102284422336]], [[0.65948242918790334, -0.42632467643266447, -0.88227132985762879, 0.68836320623684122], [-0.61219735633040351, -0.72178269592924438, -0.023790378413287128, -0.49408956197804255], [-0.2807995964783081, 0.83133282322922231, -0.53710983939223822, -0.42939055516195834]]], [[[-0.5228509035886959, -0.18471417404942181, -0.49468050029238886, 0.52413117800127984], [-0.10455789959194051, -0.36724922031900431, -0.77463887644261042, -0.94411291348111348], [-0.42153594282091333, -0.23504651378885066, -0.61899584152219456, 0.14632261604402408]], [[-0.7200005962006899, 0.57986371483229004, -0.94001900052358556, -0.86048741907772919], [-0.61834077710899926, -0.26679803751456599, -0.31024081767630474, 0.62470469377980775], [0.66456896289612644, 0.11254821792500658, -0.49566825708580775, -0.6888316639800931]]], [[[-0.11701488856899567, -0.38072891167693212, 0.13992879014213622, -0.30678825648889552], [-0.95635410346269367, -0.54541478113479802, -0.36844824135770393, -0.6833048980765557], [-0.46606158731755865, 0.20257219663303383, -0.057185674829863808, -0.24300444348168115]], [[-0.79818626146478522, -0.36096591178371784, -0.38334816505750902, -0.95130351297090199], [-0.10472192155407511, 0.32340748763042737, -0.9733026511151559, 0.38151924414321381], [0.16685146063995404, -0.071374758989875442, -0.91491461674939001, -0.7345879366753163]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank0_expandedData_rank4(self):
      arg0=Symbol(shape=())
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[0.84304592054929284, -0.93854044327826047, -0.016150108174167732, -0.4816289566732086], [0.58693713840565964, -0.072060904262374548, -0.51650899791924809, 0.065637866936331335], [-0.84372265586634509, 0.50617350903531588, 0.028367373854239375, 0.95657994218771547]], [[-0.74443080976101905, -0.82382940477998789, -0.55820751048077422, -0.65336508255641434], [-0.50033753438124662, -0.11237620842294183, 0.53728001451496366, -0.087145875343469292], [0.26083640003653707, 0.50271420734311834, -0.6808514308219229, -0.34957764340005304]]], [[[0.81889407616075882, -0.74632079299917753, 0.36918283641195115, -0.36242798553780919], [0.48842834845041039, 0.048188955723841964, -0.67015041429224032, -0.90932233077831248], [-0.77416707710251553, 0.17507317160262525, 0.58930201402691207, 0.7030366755743378]], [[-0.36778491835789517, 0.3543683879661883, -0.093834960329447714, -0.22245410576982727], [-0.81928340656137899, 0.30198307170635896, 0.46068926120376674, 0.059606098767489346], [-0.9387621160557198, -0.95812419219088363, -0.53681876508019077, -0.36559768687315564]]], [[[0.44874233217351467, 0.80089288654136248, 0.32293094857495008, -0.32929652507994378], [0.89174448138373652, -0.62263529006676999, -0.71434457255195727, -0.22728977404721196], [-0.48393427524918597, 0.56002972143236662, -0.38808490772161441, -0.37523520331305527]], [[0.83652365410688034, -0.20801638533955491, 0.38992326136100885, -0.38094282209944152], [-0.77327806146259315, -0.20087619939263801, 0.89561910128626776, -0.24444410497100821], [-0.38424093301367046, 0.73384086064049314, -0.39451311257683819, -0.53522662448144454]]]])+(1.-msk_arg1)*numpy.array([[[[0.8065770212348371, 0.0056019701672249678, 0.60401375853649109, 0.45757343634212555], [-0.090667876964254379, -0.23603061534056358, 0.032716366472212988, -0.54806352318739826], [-0.49773348618594859, -0.74942755924857885, 0.1778970365539061, -0.29459180918446548]], [[0.29935694848509975, -0.91376532891721385, -0.77904661039849588, 0.3960438814618048], [-0.020111025319112574, 0.26125760316173863, 0.30769295226359827, -0.90687882876021919], [-0.58704265680243273, 0.67822959449697806, -0.44794684915332783, 0.05121167389162018]]], [[[-0.6870388454086771, 0.35685497133542277, 0.39045951037721061, 0.14024609533855092], [-0.18964212852861673, 0.31813361863443701, -0.84389207428729129, -0.48496590426750918], [0.13403188484190287, -0.96904376943006798, 0.35405767543512434, -0.42380349881637835]], [[-0.028337093825541393, -0.39007842784678526, -0.58292003230481337, 0.85793410258037239], [0.9862437219773259, 0.090801190169059298, -0.7818546020296564, 0.41540278423343047], [0.65112428417909052, -0.055217378565429298, -0.40698130804725663, 0.73961312562780956]]], [[[-0.64943794192347615, -0.39033613237532006, 0.80611413259473874, -0.24624399381929662], [-0.047428801430705825, 0.48603041365657229, -0.59643898075402313, -0.17065265489188408], [0.71990104610279659, -0.37555295759927021, 0.8847613884086083, -0.035629393697421907]], [[-0.20040238694865464, 0.9145441049081704, -0.30894587640822868, 0.25227047156325866], [-0.96209304193060552, -0.83695063007170378, -0.6535630764085989, -0.7484454225740631], [0.32538567365852011, 0.60202193441711827, -0.57735930636458854, 0.26124925237693808]]]])
      res=minimum(arg0,arg1)
      s0=numpy.array(0.447485901524)
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.44748590152369294, -0.93854044327826047, -0.016150108174167732, -0.4816289566732086], [0.44748590152369294, -0.072060904262374548, -0.51650899791924809, 0.065637866936331335], [-0.84372265586634509, 0.44748590152369294, 0.028367373854239375, 0.44748590152369294]], [[-0.74443080976101905, -0.82382940477998789, -0.55820751048077422, -0.65336508255641434], [-0.50033753438124662, -0.11237620842294183, 0.44748590152369294, -0.087145875343469292], [0.26083640003653707, 0.44748590152369294, -0.6808514308219229, -0.34957764340005304]]], [[[0.44748590152369294, -0.74632079299917753, 0.36918283641195115, -0.36242798553780919], [0.44748590152369294, 0.048188955723841964, -0.67015041429224032, -0.90932233077831248], [-0.77416707710251553, 0.17507317160262525, 0.44748590152369294, 0.44748590152369294]], [[-0.36778491835789517, 0.3543683879661883, -0.093834960329447714, -0.22245410576982727], [-0.81928340656137899, 0.30198307170635896, 0.44748590152369294, 0.059606098767489346], [-0.9387621160557198, -0.95812419219088363, -0.53681876508019077, -0.36559768687315564]]], [[[0.44748590152369294, 0.44748590152369294, 0.32293094857495008, -0.32929652507994378], [0.44748590152369294, -0.62263529006676999, -0.71434457255195727, -0.22728977404721196], [-0.48393427524918597, 0.44748590152369294, -0.38808490772161441, -0.37523520331305527]], [[0.44748590152369294, -0.20801638533955491, 0.38992326136100885, -0.38094282209944152], [-0.77327806146259315, -0.20087619939263801, 0.44748590152369294, -0.24444410497100821], [-0.38424093301367046, 0.44748590152369294, -0.39451311257683819, -0.53522662448144454]]]])+(1.-msk_ref)*numpy.array([[[[0.44748590152369294, 0.0056019701672249678, 0.44748590152369294, 0.44748590152369294], [-0.090667876964254379, -0.23603061534056358, 0.032716366472212988, -0.54806352318739826], [-0.49773348618594859, -0.74942755924857885, 0.1778970365539061, -0.29459180918446548]], [[0.29935694848509975, -0.91376532891721385, -0.77904661039849588, 0.3960438814618048], [-0.020111025319112574, 0.26125760316173863, 0.30769295226359827, -0.90687882876021919], [-0.58704265680243273, 0.44748590152369294, -0.44794684915332783, 0.05121167389162018]]], [[[-0.6870388454086771, 0.35685497133542277, 0.39045951037721061, 0.14024609533855092], [-0.18964212852861673, 0.31813361863443701, -0.84389207428729129, -0.48496590426750918], [0.13403188484190287, -0.96904376943006798, 0.35405767543512434, -0.42380349881637835]], [[-0.028337093825541393, -0.39007842784678526, -0.58292003230481337, 0.44748590152369294], [0.44748590152369294, 0.090801190169059298, -0.7818546020296564, 0.41540278423343047], [0.44748590152369294, -0.055217378565429298, -0.40698130804725663, 0.44748590152369294]]], [[[-0.64943794192347615, -0.39033613237532006, 0.44748590152369294, -0.24624399381929662], [-0.047428801430705825, 0.44748590152369294, -0.59643898075402313, -0.17065265489188408], [0.44748590152369294, -0.37555295759927021, 0.44748590152369294, -0.035629393697421907]], [[-0.20040238694865464, 0.44748590152369294, -0.30894587640822868, 0.25227047156325866], [-0.96209304193060552, -0.83695063007170378, -0.6535630764085989, -0.7484454225740631], [0.32538567365852011, 0.44748590152369294, -0.57735930636458854, 0.26124925237693808]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.31799507401849869, 0.48678880131185931, -0.53684812424887007, -0.4020415802164734], [0.22603016819040334, 0.75671800119642163, -0.78855901263695771, -0.93018883123678253], [-0.48519359826425212, -0.90795906541520477, 0.35534750941625171, -0.68636077955926256]], [[-0.70415449291547683, 0.35620079202298705, -0.65421858709741532, 0.249324357781054], [0.33976025493103079, 0.47292602797148686, -0.94472132793369545, -0.22712616001263553], [0.78894809439244895, -0.36744388355292346, -0.52168914585234294, -0.712051622923626]]], [[[0.90605082875140441, 0.17904346809134752, -0.12549704094203595, -0.95786097439885598], [-0.30911495342056905, -0.33623822753705457, -0.60878302838853493, 0.55138544366509845], [-0.64868461431907076, 0.10986749692847431, 0.65236748253654198, 0.10213663975515952]], [[0.67089081408919737, -0.51279237661809352, 0.45469242126736131, 0.90206014890542874], [-0.078261784514981336, 0.45400633394255219, 0.95628090067815474, 0.3578702780776315], [-0.18771391078933441, 0.98276144953644762, 0.31644836301378132, -0.056174714140197146]]], [[[-0.004370372101097475, -0.76490253507886963, -0.84331966165529182, -0.082603138002831145], [-0.9944040451425169, 0.12438244421100197, -0.47926712507882074, 0.98256673641260206], [-0.093351209950814029, -0.50913778834314272, 0.36550718685543915, 0.53127316550991543]], [[0.59974623194028132, -0.66316686628207688, -0.61865358714661234, 0.037203150887942904], [0.82123680433620461, 0.32997882157440706, -0.59030435096127087, 0.69718132499071772], [0.48995760403408406, 0.10694465925852259, 0.35079613716396341, -0.39578802346959407]]]])+(1.-msk_arg1)*numpy.array([[[[0.092891334392726899, 0.51506725871734571, -0.57829797039558661, -0.9498811326855543], [0.92891473990037654, 0.19324100438602732, -0.24519149908445792, -0.83517958372228351], [-0.11196661118191598, 0.21696570818995875, 0.042775479824969542, 0.18018091905280786]], [[-0.24787741717078426, 0.15880201887408552, 0.76760858301235602, 0.37347572780334382], [-0.12325877606002478, 0.5587065846291599, 0.71424078059830376, -0.98673639352808284], [-0.034551028999261435, 0.67999894398162075, -0.91505086185222861, 0.93010278060685736]]], [[[-0.74803814743992225, -0.24878429194359786, 0.31495724127649782, 0.9922478251627791], [0.85794444389781011, -0.57988549740033402, 0.1387665049953426, 0.33857356477131284], [-0.66025755835807853, 0.25432080671336177, -0.56807083242948764, 0.40853682243698031]], [[-0.39137368305661657, 0.86747247544720651, -0.63450060962317312, -0.90185980874639005], [-0.48200572113986895, 0.85585241560215741, -0.88714824579162266, 0.30983497625720413], [0.8542537672184527, -0.84043674852562855, -0.82960690893498423, 0.020988462892144399]]], [[[0.86044901383170957, -0.1199171517248987, 0.95702931666603441, -0.94015078390103257], [0.6084337815583365, -0.42345447439769157, -0.50820219804017341, 0.97497856500730662], [-0.54353195700696033, -0.33686397683045444, -0.64197151920481677, -0.6399056824919549]], [[0.79647555515073853, -0.45180270006532686, -0.050480034952806285, -0.362600332907556], [0.87327416408699032, 0.98241816425527762, -0.36545524139363605, -0.94053767061554572], [-0.78566576513235997, 0.55087517249548035, -0.49994903052066242, -0.3009921698178597]]]])
      res=minimum(arg0,arg1)
      s0=numpy.array([[[[0.50995663066569219, 0.27865952115453796, 0.25889358037505406, 0.072134136210141531], [-0.93105421599074178, -0.48139292155563984, 0.64605765547323424, 0.010268895442930415], [-0.33895373182789013, 0.05569247022173851, 0.39375147879636341, -0.95174276485452891]], [[0.68947167915536833, 0.89586435850612078, -0.99634076457600829, -0.097767108991461571], [-0.38534845992595379, -0.69630183152252356, -0.84997664211676827, 0.4593568145425686], [0.99519820406086801, -0.32679265720987893, 0.077886536831188957, 0.57901972372401689]]], [[[-0.47598220912153688, -0.32074587923862219, -0.63786066988988233, -0.93892798355157869], [0.10506875318511377, 0.10527584547697888, -0.34927550650699324, -0.33554692001078501], [0.5670263088663785, -0.35439585127293438, 0.30339704483342778, 0.5950650960530468]], [[0.2447332486204068, -0.34119377015818753, 0.099904599903200442, -0.67508744023966583], [0.58014163620696868, 0.4885125695766479, -0.67151749494339508, -0.59203159821170748], [0.26463629056571825, -0.23637956445319053, 0.42560986318195293, -0.077033283284174692]]], [[[0.34356507185523166, -0.79533530261730978, -0.75685869734758482, -0.79056473976967956], [-0.82647610591453935, 0.3448181985703902, -0.14383043099167692, 0.53853155776407347], [-0.021437901370136903, 0.49098630886244465, -0.93144066495554245, 0.53269565893580384]], [[0.62770129871031544, -0.54158615196480597, 0.67675156607233711, 0.4654699702003311], [-0.70528540676313378, 0.96685929674101412, 0.040673805444765287, -0.8255032100691817], [-0.56432757904759234, -0.073345492218191977, -0.44790361496813014, 0.70146264023511251]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.31799507401849869, 0.27865952115453796, -0.53684812424887007, -0.4020415802164734], [-0.93105421599074178, -0.48139292155563984, -0.78855901263695771, -0.93018883123678253], [-0.48519359826425212, -0.90795906541520477, 0.35534750941625171, -0.95174276485452891]], [[-0.70415449291547683, 0.35620079202298705, -0.99634076457600829, -0.097767108991461571], [-0.38534845992595379, -0.69630183152252356, -0.94472132793369545, -0.22712616001263553], [0.78894809439244895, -0.36744388355292346, -0.52168914585234294, -0.712051622923626]]], [[[-0.47598220912153688, -0.32074587923862219, -0.63786066988988233, -0.95786097439885598], [-0.30911495342056905, -0.33623822753705457, -0.60878302838853493, -0.33554692001078501], [-0.64868461431907076, -0.35439585127293438, 0.30339704483342778, 0.10213663975515952]], [[0.2447332486204068, -0.51279237661809352, 0.099904599903200442, -0.67508744023966583], [-0.078261784514981336, 0.45400633394255219, -0.67151749494339508, -0.59203159821170748], [-0.18771391078933441, -0.23637956445319053, 0.31644836301378132, -0.077033283284174692]]], [[[-0.004370372101097475, -0.79533530261730978, -0.84331966165529182, -0.79056473976967956], [-0.9944040451425169, 0.12438244421100197, -0.47926712507882074, 0.53853155776407347], [-0.093351209950814029, -0.50913778834314272, -0.93144066495554245, 0.53127316550991543]], [[0.59974623194028132, -0.66316686628207688, -0.61865358714661234, 0.037203150887942904], [-0.70528540676313378, 0.32997882157440706, -0.59030435096127087, -0.8255032100691817], [-0.56432757904759234, -0.073345492218191977, -0.44790361496813014, -0.39578802346959407]]]])+(1.-msk_ref)*numpy.array([[[[0.092891334392726899, 0.27865952115453796, -0.57829797039558661, -0.9498811326855543], [-0.93105421599074178, -0.48139292155563984, -0.24519149908445792, -0.83517958372228351], [-0.33895373182789013, 0.05569247022173851, 0.042775479824969542, -0.95174276485452891]], [[-0.24787741717078426, 0.15880201887408552, -0.99634076457600829, -0.097767108991461571], [-0.38534845992595379, -0.69630183152252356, -0.84997664211676827, -0.98673639352808284], [-0.034551028999261435, -0.32679265720987893, -0.91505086185222861, 0.57901972372401689]]], [[[-0.74803814743992225, -0.32074587923862219, -0.63786066988988233, -0.93892798355157869], [0.10506875318511377, -0.57988549740033402, -0.34927550650699324, -0.33554692001078501], [-0.66025755835807853, -0.35439585127293438, -0.56807083242948764, 0.40853682243698031]], [[-0.39137368305661657, -0.34119377015818753, -0.63450060962317312, -0.90185980874639005], [-0.48200572113986895, 0.4885125695766479, -0.88714824579162266, -0.59203159821170748], [0.26463629056571825, -0.84043674852562855, -0.82960690893498423, -0.077033283284174692]]], [[[0.34356507185523166, -0.79533530261730978, -0.75685869734758482, -0.94015078390103257], [-0.82647610591453935, -0.42345447439769157, -0.50820219804017341, 0.53853155776407347], [-0.54353195700696033, -0.33686397683045444, -0.93144066495554245, -0.6399056824919549]], [[0.62770129871031544, -0.54158615196480597, -0.050480034952806285, -0.362600332907556], [-0.70528540676313378, 0.96685929674101412, -0.36545524139363605, -0.94053767061554572], [-0.78566576513235997, -0.073345492218191977, -0.49994903052066242, -0.3009921698178597]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_float_rank0(self):
      arg0=Data(-0.29611572534,self.functionspace)
      arg1=-0.797756770219
      res=minimum(arg0,arg1)
      ref=Data(-0.797756770219,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_float_rank0(self):
      arg0=Data(numpy.array([-0.25887636523261937, 0.58795283912157514]),self.functionspace)
      arg1=0.297005924794
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.25887636523261937, 0.29700592479356924]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_float_rank0(self):
      arg0=Data(numpy.array([[0.92166039196513494, -0.88756772726825806, 0.51584477639977289, -0.091769134949095443, -0.59410155596129877], [-0.21264133241776295, -0.99912379000472296, -0.39510039872407354, 0.63706725072139769, -0.014024596004296619], [-0.71763440840753634, 0.18493934212184127, -0.048531213740846457, 0.65025127044880593, 0.28319580428582536], [0.3890980206496224, -0.55389622293052998, -0.11386564567836932, 0.47865626831071628, 0.77724870834768001]]),self.functionspace)
      arg1=-0.792380770389
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.79238077038907662, -0.88756772726825806, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662], [-0.79238077038907662, -0.99912379000472296, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662], [-0.79238077038907662, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662], [-0.79238077038907662, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662, -0.79238077038907662]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_float_rank0(self):
      arg0=Data(numpy.array([[[0.74944567586463773, -0.33978343253328824], [-0.37890850997240322, 0.24248607098113717]], [[-0.89701702176207077, 0.87276965314348209], [0.40649570156709047, 0.23333768985973946]], [[-0.099164468164592234, 0.80557277646808578], [0.16263513747592051, 0.90057254013938648]], [[-0.69543374467469943, 0.6643337919851946], [-0.79741770598340134, -0.99292456633157555]], [[0.16172944706257919, -0.35955193794626594], [-0.5330239044023739, 0.21493737787509892]], [[0.2790541359506078, -0.6732090415846117], [0.87628168163683506, 0.87903061493575962]]]),self.functionspace)
      arg1=-0.871289677371
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.87128967737138519, -0.87128967737138519], [-0.87128967737138519, -0.87128967737138519]], [[-0.89701702176207077, -0.87128967737138519], [-0.87128967737138519, -0.87128967737138519]], [[-0.87128967737138519, -0.87128967737138519], [-0.87128967737138519, -0.87128967737138519]], [[-0.87128967737138519, -0.87128967737138519], [-0.87128967737138519, -0.99292456633157555]], [[-0.87128967737138519, -0.87128967737138519], [-0.87128967737138519, -0.87128967737138519]], [[-0.87128967737138519, -0.87128967737138519], [-0.87128967737138519, -0.87128967737138519]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_float_rank0(self):
      arg0=Data(numpy.array([[[[-0.1388269187667055, 0.63673847966886488, -0.66759240330085645, -0.34685771879646921], [-0.8392936053248905, 0.105179246187733, 0.0075929780086834153, -0.61673063456035648], [0.85000062095565854, -0.063763121690259439, -0.34883169739146491, 0.48037150690786268]], [[0.29358432415872904, -0.4707085862025322, -0.65438978095826439, -0.71740395392272682], [-0.47514822799320178, -0.97629638396916341, 0.17825044054295769, -0.25246398111694068], [-0.6220915922021002, 0.84788597777577634, 0.82889468506100461, -0.012674439024131567]]], [[[0.35116363603933731, -0.021538026059338744, -0.17384192774016105, -0.45553109503155098], [0.20056679586272153, 0.30744588054944422, -0.77874328797484016, -0.88012265977233417], [-0.42260697201030206, -0.61038803434718636, 0.47895312009091717, -0.038151577343234599]], [[-0.53415574143168665, 0.84695135128253862, 0.18589602384578963, -0.61915445579529371], [0.52952255634814915, -0.98555762331258112, 0.96914190429825697, -0.7938040908090811], [-0.21437472718479267, -0.97693491984729741, -0.20188828071516474, -0.97583229805026028]]], [[[0.17231265722644173, -0.068428660408735631, -0.63970341301842293, -0.82382393634357487], [-0.66840533565477145, -0.80976059948291401, -0.5982127225211793, 0.72748975461801502], [-0.10852873189382928, -0.42677679737230956, 0.46920906990042588, 0.86994684336813899]], [[-0.11514183406335321, -0.092372123984953092, 0.83938403886434299, 0.90243772022516411], [-0.75932711388683383, -0.29960560855890805, -0.14267592964077958, -0.014233063379661903], [0.96335057859736772, 0.8315593009623734, -0.77043221707324938, -0.26892059588084694]]]]),self.functionspace)
      arg1=-0.456782141397
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.45678214139666196, -0.45678214139666196, -0.66759240330085645, -0.45678214139666196], [-0.8392936053248905, -0.45678214139666196, -0.45678214139666196, -0.61673063456035648], [-0.45678214139666196, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196]], [[-0.45678214139666196, -0.4707085862025322, -0.65438978095826439, -0.71740395392272682], [-0.47514822799320178, -0.97629638396916341, -0.45678214139666196, -0.45678214139666196], [-0.6220915922021002, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196]]], [[[-0.45678214139666196, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196], [-0.45678214139666196, -0.45678214139666196, -0.77874328797484016, -0.88012265977233417], [-0.45678214139666196, -0.61038803434718636, -0.45678214139666196, -0.45678214139666196]], [[-0.53415574143168665, -0.45678214139666196, -0.45678214139666196, -0.61915445579529371], [-0.45678214139666196, -0.98555762331258112, -0.45678214139666196, -0.7938040908090811], [-0.45678214139666196, -0.97693491984729741, -0.45678214139666196, -0.97583229805026028]]], [[[-0.45678214139666196, -0.45678214139666196, -0.63970341301842293, -0.82382393634357487], [-0.66840533565477145, -0.80976059948291401, -0.5982127225211793, -0.45678214139666196], [-0.45678214139666196, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196]], [[-0.45678214139666196, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196], [-0.75932711388683383, -0.45678214139666196, -0.45678214139666196, -0.45678214139666196], [-0.45678214139666196, -0.45678214139666196, -0.77043221707324938, -0.45678214139666196]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_array_rank0(self):
      arg0=Data(-0.703477260169,self.functionspace)
      arg1=numpy.array(-0.111524267259)
      res=minimum(arg0,arg1)
      ref=Data(-0.703477260169,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_array_rank0(self):
      arg0=Data(numpy.array([-0.051626260426405057, -0.86657522504242901]),self.functionspace)
      arg1=numpy.array(-0.129380536671)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.12938053667112936, -0.86657522504242901]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_array_rank0(self):
      arg0=Data(numpy.array([[-0.6309979859705459, 0.78447705956664482, 0.917224542144363, -0.83129287572552357, 0.5512981316358605], [0.43503920838346177, 0.46800695003772042, -0.69428086176864978, 0.088050921116030967, -0.15187099091557288], [0.045904757583457245, 0.43220580296973821, 0.21222982060715156, 0.31117897079074908, -0.44179607670877008], [0.22348517314633543, 0.44230213315739264, -0.82108511449959209, -0.35816040215571188, -0.44975983281622223]]),self.functionspace)
      arg1=numpy.array(0.14849378456)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.6309979859705459, 0.14849378455981843, 0.14849378455981843, -0.83129287572552357, 0.14849378455981843], [0.14849378455981843, 0.14849378455981843, -0.69428086176864978, 0.088050921116030967, -0.15187099091557288], [0.045904757583457245, 0.14849378455981843, 0.14849378455981843, 0.14849378455981843, -0.44179607670877008], [0.14849378455981843, 0.14849378455981843, -0.82108511449959209, -0.35816040215571188, -0.44975983281622223]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_array_rank0(self):
      arg0=Data(numpy.array([[[-0.39477601512904448, 0.93108074593242995], [0.5196584332324099, 0.079546360600516008]], [[-0.011408950668585494, -0.19261328272506151], [-0.56101138822270258, 0.0080905002789830593]], [[-0.12985789454999708, 0.18948827014477776], [0.97067644642834749, 0.91441994772822888]], [[0.70947246023344901, 0.057294420615167985], [-0.55510575011556451, -0.67986242508465433]], [[0.53473055660037172, 0.62107497512061527], [0.31696913866088416, -0.84521363185821374]], [[0.57739437704357788, 0.28991965886080817], [0.16266973488424896, 0.5671198696134534]]]),self.functionspace)
      arg1=numpy.array(-0.578089688994)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.57808968899364288]], [[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.57808968899364288]], [[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.57808968899364288]], [[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.67986242508465433]], [[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.84521363185821374]], [[-0.57808968899364288, -0.57808968899364288], [-0.57808968899364288, -0.57808968899364288]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_array_rank0(self):
      arg0=Data(numpy.array([[[[0.58465167980803057, 0.0072059833259232597, 0.10727877587063395, 0.63113722764806979], [0.93653531533863443, 0.16810528871356945, 0.21643745838889594, -0.93449191563091616], [-0.62020651520280623, 0.074389445958584322, -0.8665042207324325, 0.99896909552637947]], [[-0.16598165682557431, 0.051228620323296115, 0.12782685941597638, 0.99340798955327081], [-0.38550610833728238, 0.040786943724632208, -0.32308295140597143, 0.87369291428889384], [0.0012250072168713544, 0.42329577041092215, 0.91221785716084169, 0.95683884961498467]]], [[[0.78554940016237707, -0.038755571687487222, -0.22988059146204809, -0.89249557409600633], [-0.95357834899939875, -0.55681052529943598, -0.85504105863507385, 0.67389329257018593], [-0.057504850827395648, 0.76153677924906082, -0.60495762331902969, 0.51113529335496688]], [[-0.049075872488488104, -0.038400746530552032, 0.69878393264314442, 0.011406572165138851], [-0.90312910067746444, -0.20667382966347003, -0.1155681279992582, 0.39031082462470401], [-0.71121787314583318, -0.87280260307269142, 0.50415662027435815, -0.93354815046253092]]], [[[0.23763626249632419, -0.56488234036028739, 0.8658467087211914, -0.98353794563121855], [0.9906136361815927, -0.44094476246559688, 0.6015899220857992, 0.76861991342931102], [0.4688721921620318, -0.89478060385034408, -0.91040519788606256, 0.13095510541648592]], [[0.54226227573912822, -0.25705182063540133, 0.2900354262376923, -0.2164301562016222], [-0.53559360744049722, -0.24938270106067995, 0.68768814954490987, -0.20605799444486284], [0.013941991501521089, -0.50250813872002276, 0.78373202627834027, -0.1225349799036779]]]]),self.functionspace)
      arg1=numpy.array(0.119932171092)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[0.11993217109155174, 0.0072059833259232597, 0.10727877587063395, 0.11993217109155174], [0.11993217109155174, 0.11993217109155174, 0.11993217109155174, -0.93449191563091616], [-0.62020651520280623, 0.074389445958584322, -0.8665042207324325, 0.11993217109155174]], [[-0.16598165682557431, 0.051228620323296115, 0.11993217109155174, 0.11993217109155174], [-0.38550610833728238, 0.040786943724632208, -0.32308295140597143, 0.11993217109155174], [0.0012250072168713544, 0.11993217109155174, 0.11993217109155174, 0.11993217109155174]]], [[[0.11993217109155174, -0.038755571687487222, -0.22988059146204809, -0.89249557409600633], [-0.95357834899939875, -0.55681052529943598, -0.85504105863507385, 0.11993217109155174], [-0.057504850827395648, 0.11993217109155174, -0.60495762331902969, 0.11993217109155174]], [[-0.049075872488488104, -0.038400746530552032, 0.11993217109155174, 0.011406572165138851], [-0.90312910067746444, -0.20667382966347003, -0.1155681279992582, 0.11993217109155174], [-0.71121787314583318, -0.87280260307269142, 0.11993217109155174, -0.93354815046253092]]], [[[0.11993217109155174, -0.56488234036028739, 0.11993217109155174, -0.98353794563121855], [0.11993217109155174, -0.44094476246559688, 0.11993217109155174, 0.11993217109155174], [0.11993217109155174, -0.89478060385034408, -0.91040519788606256, 0.11993217109155174]], [[0.11993217109155174, -0.25705182063540133, 0.11993217109155174, -0.2164301562016222], [-0.53559360744049722, -0.24938270106067995, 0.11993217109155174, -0.20605799444486284], [0.013941991501521089, -0.50250813872002276, 0.11993217109155174, -0.1225349799036779]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_Symbol_rank0(self):
      arg0=Data(-0.577484238555,self.functionspace)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.670085306694)
      sub=res.substitute({arg1:s1})
      ref=Data(-0.670085306694,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_Symbol_rank0(self):
      arg0=Data(numpy.array([-0.67345552691005262, -0.56545369692434355]),self.functionspace)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.601730413138)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.67345552691005262, -0.601730413137614]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_Symbol_rank0(self):
      arg0=Data(numpy.array([[-0.25187655315255375, 0.19251904284405219, 0.36308572579677834, -0.16990277969663081, 0.14141160615729831], [0.88306547196238583, 0.57676120460414393, 0.49187884361353618, 0.66608179555896219, 0.016825044011488544], [0.17718789350321495, -0.28150366113610503, -0.8669892277286162, 0.98326962254829309, -0.65779040297830926], [0.27731590110229365, 0.39393978730236623, 0.4069891446851972, 0.5781891383100366, -0.02021853699522036]]),self.functionspace)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.0221859756614)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.25187655315255375, -0.022185975661384028, -0.022185975661384028, -0.16990277969663081, -0.022185975661384028], [-0.022185975661384028, -0.022185975661384028, -0.022185975661384028, -0.022185975661384028, -0.022185975661384028], [-0.022185975661384028, -0.28150366113610503, -0.8669892277286162, -0.022185975661384028, -0.65779040297830926], [-0.022185975661384028, -0.022185975661384028, -0.022185975661384028, -0.022185975661384028, -0.022185975661384028]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_Symbol_rank0(self):
      arg0=Data(numpy.array([[[0.12664500420310976, -0.14819216611347263], [0.17423785846616058, -0.95672800973789052]], [[-0.58234967921786129, 0.78973644891399908], [-0.87709059801762979, -0.11727066253858465]], [[0.85154534915794255, -0.098030220820856329], [0.84113006911918808, 0.11572139189608044]], [[0.22765492804984078, -0.10733315138068122], [-0.36519586237375856, -0.79755432275386173]], [[-0.15679550122061081, 0.42997061804572323], [-0.41504174790737114, -0.44243061519699545]], [[-0.66709566150309496, 0.86699131866230306], [-0.14832685290599978, -0.63235360123551887]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.565672893661)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[0.12664500420310976, -0.14819216611347263], [0.17423785846616058, -0.95672800973789052]], [[-0.58234967921786129, 0.56567289366129869], [-0.87709059801762979, -0.11727066253858465]], [[0.56567289366129869, -0.098030220820856329], [0.56567289366129869, 0.11572139189608044]], [[0.22765492804984078, -0.10733315138068122], [-0.36519586237375856, -0.79755432275386173]], [[-0.15679550122061081, 0.42997061804572323], [-0.41504174790737114, -0.44243061519699545]], [[-0.66709566150309496, 0.56567289366129869], [-0.14832685290599978, -0.63235360123551887]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_Symbol_rank0(self):
      arg0=Data(numpy.array([[[[-0.9541863689499861, -0.99452543825324491, 0.74768092528772501, 0.49640721987564951], [0.30204583860763168, 0.90896995989732177, -0.58655329932787725, 0.084434374299792836], [-0.79945374308156913, 0.70952163839696114, 0.17967023953501404, 0.54378300791853529]], [[0.84620486487271607, -0.95008728903801809, 0.7634747742235084, 0.20053407910473009], [-0.71258812405135741, -0.99516980144395273, 0.90748224968545421, -0.76048226893068516], [0.85692181946960932, -0.51724519708257244, 0.56008959884748788, 0.77357453889094563]]], [[[0.7804397680682853, 0.9067708757320716, -0.22225489389335684, -0.47719711593190639], [0.34301910486397458, -0.5639153727173345, 0.71784137013248595, 0.044586791739296139], [0.48333383385389794, 0.052068712240846526, -0.75605251529799555, -0.42196396856421292]], [[0.74947858526231337, -0.31459726954395251, -0.86806180576192249, 0.41696201616015816], [0.32172204497208723, 0.020442862260954042, -0.8995750982113726, -0.17721390926367553], [0.89527769077648944, -0.34209525277023478, -0.065650656480902958, -0.15883513602310662]]], [[[-0.88254885544736306, -0.11818400177591082, 0.32582821622342895, -0.87982773837565365], [-0.98403502120725839, -0.74472860819036857, 0.30602215380770814, -0.93224025778126474], [0.82343767299364012, -0.97534433330863601, -0.27556638425556046, 0.88097517765454936]], [[0.23609418186757014, -0.90060249576058671, 0.64312426149714019, -0.93624918763932241], [0.84516356184121433, -0.27586959311413217, 0.49833262139425716, 0.1494914320171099], [0.86331148784161549, 0.33425294557488705, -0.67391279981618157, 0.83420406228176258]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.566451588966)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.9541863689499861, -0.99452543825324491, 0.56645158896558789, 0.49640721987564951], [0.30204583860763168, 0.56645158896558789, -0.58655329932787725, 0.084434374299792836], [-0.79945374308156913, 0.56645158896558789, 0.17967023953501404, 0.54378300791853529]], [[0.56645158896558789, -0.95008728903801809, 0.56645158896558789, 0.20053407910473009], [-0.71258812405135741, -0.99516980144395273, 0.56645158896558789, -0.76048226893068516], [0.56645158896558789, -0.51724519708257244, 0.56008959884748788, 0.56645158896558789]]], [[[0.56645158896558789, 0.56645158896558789, -0.22225489389335684, -0.47719711593190639], [0.34301910486397458, -0.5639153727173345, 0.56645158896558789, 0.044586791739296139], [0.48333383385389794, 0.052068712240846526, -0.75605251529799555, -0.42196396856421292]], [[0.56645158896558789, -0.31459726954395251, -0.86806180576192249, 0.41696201616015816], [0.32172204497208723, 0.020442862260954042, -0.8995750982113726, -0.17721390926367553], [0.56645158896558789, -0.34209525277023478, -0.065650656480902958, -0.15883513602310662]]], [[[-0.88254885544736306, -0.11818400177591082, 0.32582821622342895, -0.87982773837565365], [-0.98403502120725839, -0.74472860819036857, 0.30602215380770814, -0.93224025778126474], [0.56645158896558789, -0.97534433330863601, -0.27556638425556046, 0.56645158896558789]], [[0.23609418186757014, -0.90060249576058671, 0.56645158896558789, -0.93624918763932241], [0.56645158896558789, -0.27586959311413217, 0.49833262139425716, 0.1494914320171099], [0.56645158896558789, 0.33425294557488705, -0.67391279981618157, 0.56645158896558789]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_constData_rank0(self):
      arg0=Data(-0.0682904128536,self.functionspace)
      arg1=Data(0.59973017062,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(-0.0682904128536,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_constData_rank0(self):
      arg0=Data(numpy.array([-0.046535259888465186, 0.48670114343582571]),self.functionspace)
      arg1=Data(-0.348298274913,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.34829827491324861, -0.34829827491324861]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_constData_rank0(self):
      arg0=Data(numpy.array([[0.0043433043409331074, -0.57853974767086314, 0.66921930943734331, 0.6516414165234734, -0.10393719679290947], [-0.53471960534791219, -0.013287726154472157, 0.48621509573775468, 0.40828070350173196, 0.60537705279057641], [-0.83716110570575375, 0.31558661264530308, -0.37910809975832604, 0.57779622112999607, 0.93389423604887267], [-0.49975805605893742, 0.1709728139617237, -0.45509192065283166, 0.59113241384052828, 0.6466938264587454]]),self.functionspace)
      arg1=Data(-0.168612415903,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.16861241590337417, -0.57853974767086314, -0.16861241590337417, -0.16861241590337417, -0.16861241590337417], [-0.53471960534791219, -0.16861241590337417, -0.16861241590337417, -0.16861241590337417, -0.16861241590337417], [-0.83716110570575375, -0.16861241590337417, -0.37910809975832604, -0.16861241590337417, -0.16861241590337417], [-0.49975805605893742, -0.16861241590337417, -0.45509192065283166, -0.16861241590337417, -0.16861241590337417]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_constData_rank0(self):
      arg0=Data(numpy.array([[[-0.96748467197599286, 0.64903315279047846], [-0.47705259350894802, -0.49164181292261167]], [[-0.7015432941767461, -0.92535952728804727], [-0.36711963241188994, 0.92953764813788631]], [[-0.4385844921920905, -0.65550129890521247], [-0.74532935198119943, -0.40457816408817449]], [[-0.92662104366440556, -0.29287316117065054], [0.96615565125929637, -0.36691971273969104]], [[0.72210396163733281, 0.081839009617667768], [-0.2691125588227703, -0.51493797950128295]], [[-0.54949192982790951, -0.6774418688721966], [-0.17061276299902639, 0.82709731250799012]]]),self.functionspace)
      arg1=Data(0.499190483375,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.96748467197599286, 0.49919048337529159], [-0.47705259350894802, -0.49164181292261167]], [[-0.7015432941767461, -0.92535952728804727], [-0.36711963241188994, 0.49919048337529159]], [[-0.4385844921920905, -0.65550129890521247], [-0.74532935198119943, -0.40457816408817449]], [[-0.92662104366440556, -0.29287316117065054], [0.49919048337529159, -0.36691971273969104]], [[0.49919048337529159, 0.081839009617667768], [-0.2691125588227703, -0.51493797950128295]], [[-0.54949192982790951, -0.6774418688721966], [-0.17061276299902639, 0.49919048337529159]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_constData_rank0(self):
      arg0=Data(numpy.array([[[[-0.37733922304651357, 0.93859055187304907, -0.66898461244429197, -0.46585811774138564], [0.19019525471841159, 0.17346337979132165, -0.64718916649618929, -0.67366242394796338], [0.21369470981955363, -0.51799021616039864, -0.6056442944578373, -0.38855873336390934]], [[0.99912850246702045, -0.22170809303306993, -0.10146419048978084, 0.57504962195842402], [-0.65839444610623876, -0.11875165595273174, 0.68203592892318099, 0.35593988456162529], [0.99397536248303098, 0.74338026360660292, -0.19639175275880572, 0.57004920258862946]]], [[[-0.65958353056467245, 0.65427773671661527, -0.14175338875473731, 0.39791966909561971], [0.40653156970772342, -0.99830720962539998, -0.47147437649596791, -0.18366519810361259], [-0.94376751485499333, 0.53108747554058211, -0.87104507659859154, 0.49459590756485095]], [[-0.83564421015096224, -0.94951507050099071, 0.8807711293336058, -0.078107576456112771], [-0.1222045997717498, -0.28558604389610087, 0.73718369414227958, -0.3309739040944093], [-0.1342824947722332, -0.40766565005604072, 0.14915899465306381, -0.089308100175077065]]], [[[0.95785187074356348, -0.89162565354404499, -0.29472586924251343, 0.7678226475120653], [-0.37455223108752689, -0.23254090056192767, -0.42146652737226464, 0.52324509319178381], [0.51533338332973488, 0.50126475193545161, -0.95767232046665685, -0.033673825475154961]], [[-0.33046373133706775, 0.47620518964597536, 0.78139637666038175, 0.66138116717578854], [0.79787867018082315, -0.72738889276213015, 0.28490828339394891, -0.55618525346623437], [-0.50670753528575885, -0.54352326092270364, -0.90122107838353926, -0.81241061207885834]]]]),self.functionspace)
      arg1=Data(-0.145562749107,self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.37733922304651357, -0.14556274910733213, -0.66898461244429197, -0.46585811774138564], [-0.14556274910733213, -0.14556274910733213, -0.64718916649618929, -0.67366242394796338], [-0.14556274910733213, -0.51799021616039864, -0.6056442944578373, -0.38855873336390934]], [[-0.14556274910733213, -0.22170809303306993, -0.14556274910733213, -0.14556274910733213], [-0.65839444610623876, -0.14556274910733213, -0.14556274910733213, -0.14556274910733213], [-0.14556274910733213, -0.14556274910733213, -0.19639175275880572, -0.14556274910733213]]], [[[-0.65958353056467245, -0.14556274910733213, -0.14556274910733213, -0.14556274910733213], [-0.14556274910733213, -0.99830720962539998, -0.47147437649596791, -0.18366519810361259], [-0.94376751485499333, -0.14556274910733213, -0.87104507659859154, -0.14556274910733213]], [[-0.83564421015096224, -0.94951507050099071, -0.14556274910733213, -0.14556274910733213], [-0.14556274910733213, -0.28558604389610087, -0.14556274910733213, -0.3309739040944093], [-0.14556274910733213, -0.40766565005604072, -0.14556274910733213, -0.14556274910733213]]], [[[-0.14556274910733213, -0.89162565354404499, -0.29472586924251343, -0.14556274910733213], [-0.37455223108752689, -0.23254090056192767, -0.42146652737226464, -0.14556274910733213], [-0.14556274910733213, -0.14556274910733213, -0.95767232046665685, -0.14556274910733213]], [[-0.33046373133706775, -0.14556274910733213, -0.14556274910733213, -0.14556274910733213], [-0.14556274910733213, -0.72738889276213015, -0.14556274910733213, -0.55618525346623437], [-0.50670753528575885, -0.54352326092270364, -0.90122107838353926, -0.81241061207885834]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_expandedData_rank0(self):
      arg0=Data(0.847648908098,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.311113868368)+(1.-msk_arg1)*(0.434427772754)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.311113868368)+(1.-msk_ref)*(0.434427772754)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_expandedData_rank0(self):
      arg0=Data(numpy.array([0.46831305963737124, 0.11263222450309307]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.149120762552)+(1.-msk_arg1)*(0.152164813827)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.14912076255240625, 0.11263222450309307])+(1.-msk_ref)*numpy.array([0.15216481382704261, 0.11263222450309307])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_expandedData_rank0(self):
      arg0=Data(numpy.array([[-0.54224621195320832, 0.47554128544693075, 0.79438328624548626, 0.42268775103957679, 0.5508013258342499], [-0.052795411003615067, -0.39861061718913571, 0.16757651909234195, -0.1655995184462824, 0.64613941746480097], [-0.85102433746626382, -0.41445863472983424, -0.24645592071424027, -0.24779944984550029, -0.59625567262833989], [-0.93188639663803285, -0.37478830578694033, 0.9991234193984555, -0.93683631624508812, 0.52050343962722456]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.345473349033)+(1.-msk_arg1)*(0.211321735562)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.54224621195320832, -0.34547334903252191, -0.34547334903252191, -0.34547334903252191, -0.34547334903252191], [-0.34547334903252191, -0.39861061718913571, -0.34547334903252191, -0.34547334903252191, -0.34547334903252191], [-0.85102433746626382, -0.41445863472983424, -0.34547334903252191, -0.34547334903252191, -0.59625567262833989], [-0.93188639663803285, -0.37478830578694033, -0.34547334903252191, -0.93683631624508812, -0.34547334903252191]])+(1.-msk_ref)*numpy.array([[-0.54224621195320832, 0.21132173556238087, 0.21132173556238087, 0.21132173556238087, 0.21132173556238087], [-0.052795411003615067, -0.39861061718913571, 0.16757651909234195, -0.1655995184462824, 0.21132173556238087], [-0.85102433746626382, -0.41445863472983424, -0.24645592071424027, -0.24779944984550029, -0.59625567262833989], [-0.93188639663803285, -0.37478830578694033, 0.21132173556238087, -0.93683631624508812, 0.21132173556238087]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_expandedData_rank0(self):
      arg0=Data(numpy.array([[[0.77625068038328937, 0.75783014853553654], [-0.040642761077081202, 0.79204260731199394]], [[-0.54776326748801329, 0.65058903205233287], [-0.01282121102231315, 0.15475725458670797]], [[0.31470843587449004, -0.07753563902895122], [0.11099859076132779, -0.81518098747217427]], [[-0.55118665017984303, -0.67078063912337416], [-0.98004112137157251, -0.18816152794055951]], [[-0.046829245664697039, 0.16829625958590699], [-0.038355227747033371, 0.56440637140131744]], [[0.59007303065181005, 0.50473600523369155], [-0.12313540236032172, -0.92117335984586246]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.73895651651)+(1.-msk_arg1)*(0.719199756325)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.73895651650968386, 0.73895651650968386], [-0.040642761077081202, 0.73895651650968386]], [[-0.54776326748801329, 0.65058903205233287], [-0.01282121102231315, 0.15475725458670797]], [[0.31470843587449004, -0.07753563902895122], [0.11099859076132779, -0.81518098747217427]], [[-0.55118665017984303, -0.67078063912337416], [-0.98004112137157251, -0.18816152794055951]], [[-0.046829245664697039, 0.16829625958590699], [-0.038355227747033371, 0.56440637140131744]], [[0.59007303065181005, 0.50473600523369155], [-0.12313540236032172, -0.92117335984586246]]])+(1.-msk_ref)*numpy.array([[[0.71919975632513666, 0.71919975632513666], [-0.040642761077081202, 0.71919975632513666]], [[-0.54776326748801329, 0.65058903205233287], [-0.01282121102231315, 0.15475725458670797]], [[0.31470843587449004, -0.07753563902895122], [0.11099859076132779, -0.81518098747217427]], [[-0.55118665017984303, -0.67078063912337416], [-0.98004112137157251, -0.18816152794055951]], [[-0.046829245664697039, 0.16829625958590699], [-0.038355227747033371, 0.56440637140131744]], [[0.59007303065181005, 0.50473600523369155], [-0.12313540236032172, -0.92117335984586246]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_expandedData_rank0(self):
      arg0=Data(numpy.array([[[[0.99268289857921577, 0.41088037150125367, 0.26177708839700098, 0.82376151099681261], [-0.54331004634703661, 0.031496969431629163, 0.52688351495449215, -0.26722050954674859], [0.38313164658014309, 0.53282042430453891, -0.078299137887749559, -0.056014546456052861]], [[-0.76995708990435641, 0.061549978967386787, 0.69527255728927351, 0.10969825231765773], [0.37086234261002815, -0.95794861186225044, -0.86426578495087525, -0.97802222289390506], [-0.4471798391315418, -0.6251092573946726, 0.27276076837895524, -0.91339059646578202]]], [[[-0.66459163952874545, -0.15129505705337598, 0.56364833895809929, 0.97582274938998448], [0.83601385850089827, -0.84277453397822333, -0.031356869131770715, 0.44962798226714273], [0.89692171300867551, 0.27995650006365791, -0.074995651859347978, -0.84552021003774147]], [[-0.77755385628089391, -0.95235277479154123, 0.22063465156856288, -0.29165804377398152], [-0.51877939160718611, 0.81455229280273311, -0.28420396833550776, 0.43845728843849896], [-0.71947115048083421, 0.13592865235020035, -0.37959986478412966, 0.062674966057193471]]], [[[0.17452369197943218, -0.37523426512393177, -0.95494844959095526, -0.41924989267453139], [0.45425791383688785, 0.67211643814140909, -0.60261482866032257, -0.68685820529638919], [0.95173027347869144, 0.69422535407147024, -0.075058683952093475, 0.10782056487000879]], [[-0.5324481902328142, -0.46623598923291354, 0.046695183173332211, 0.92082584868164785], [-0.83964862951040442, 0.60757410631852449, 0.14647609641995141, 0.88180327423933225], [-0.76119594211399955, -0.76985823613463378, -0.048217420980172454, 0.34485508040439328]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.971083689041)+(1.-msk_arg1)*(0.961141977621)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]], [[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97802222289390506], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]]], [[[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]], [[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]]], [[[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]], [[-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234], [-0.97108368904079234, -0.97108368904079234, -0.97108368904079234, -0.97108368904079234]]]])+(1.-msk_ref)*numpy.array([[[[0.96114197762050035, 0.41088037150125367, 0.26177708839700098, 0.82376151099681261], [-0.54331004634703661, 0.031496969431629163, 0.52688351495449215, -0.26722050954674859], [0.38313164658014309, 0.53282042430453891, -0.078299137887749559, -0.056014546456052861]], [[-0.76995708990435641, 0.061549978967386787, 0.69527255728927351, 0.10969825231765773], [0.37086234261002815, -0.95794861186225044, -0.86426578495087525, -0.97802222289390506], [-0.4471798391315418, -0.6251092573946726, 0.27276076837895524, -0.91339059646578202]]], [[[-0.66459163952874545, -0.15129505705337598, 0.56364833895809929, 0.96114197762050035], [0.83601385850089827, -0.84277453397822333, -0.031356869131770715, 0.44962798226714273], [0.89692171300867551, 0.27995650006365791, -0.074995651859347978, -0.84552021003774147]], [[-0.77755385628089391, -0.95235277479154123, 0.22063465156856288, -0.29165804377398152], [-0.51877939160718611, 0.81455229280273311, -0.28420396833550776, 0.43845728843849896], [-0.71947115048083421, 0.13592865235020035, -0.37959986478412966, 0.062674966057193471]]], [[[0.17452369197943218, -0.37523426512393177, -0.95494844959095526, -0.41924989267453139], [0.45425791383688785, 0.67211643814140909, -0.60261482866032257, -0.68685820529638919], [0.95173027347869144, 0.69422535407147024, -0.075058683952093475, 0.10782056487000879]], [[-0.5324481902328142, -0.46623598923291354, 0.046695183173332211, 0.92082584868164785], [-0.83964862951040442, 0.60757410631852449, 0.14647609641995141, 0.88180327423933225], [-0.76119594211399955, -0.76985823613463378, -0.048217420980172454, 0.34485508040439328]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_array_rank1(self):
      arg0=Data(-0.333406435888,self.functionspace)
      arg1=numpy.array([-0.09652682757478126, 0.26251725837796158])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.33340643588845387, -0.33340643588845387]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_array_rank1(self):
      arg0=Data(numpy.array([0.53558157958236996, 0.54103401750141189]),self.functionspace)
      arg1=numpy.array([0.033108086171252271, -0.32851105843468731])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([0.033108086171252271, -0.32851105843468731]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_Symbol_rank1(self):
      arg0=Data(0.322575974804,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([-0.036119662426258303, 0.72279085689234224])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.036119662426258303, 0.32257597480446365]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_Symbol_rank1(self):
      arg0=Data(numpy.array([-0.34173936416674877, 0.54962083223933789]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([0.97437676937769035, -0.18212143168456918])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-0.34173936416674877, -0.18212143168456918]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_constData_rank1(self):
      arg0=Data(-0.924549249904,self.functionspace)
      arg1=Data(numpy.array([-0.51873551296636755, -0.75625179387206831]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.9245492499043575, -0.9245492499043575]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_constData_rank1(self):
      arg0=Data(numpy.array([-0.68795188857449396, -0.29420892408496324]),self.functionspace)
      arg1=Data(numpy.array([-0.28574289295395161, 0.6744015597648072]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([-0.68795188857449396, -0.29420892408496324]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_expandedData_rank1(self):
      arg0=Data(-0.750967691228,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.048427606313035332, 0.1772855405310334])+(1.-msk_arg1)*numpy.array([-0.58305150173003639, -0.30318806576525592])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.75096769122823592, -0.75096769122823592])+(1.-msk_ref)*numpy.array([-0.75096769122823592, -0.75096769122823592])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank1_expandedData_rank1(self):
      arg0=Data(numpy.array([-0.72405478212841956, 0.56611255103978775]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([-0.10702258655126662, 0.720896326748794])+(1.-msk_arg1)*numpy.array([0.63801284539466074, -0.9382170945092474])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.72405478212841956, 0.56611255103978775])+(1.-msk_ref)*numpy.array([-0.72405478212841956, -0.9382170945092474])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_array_rank2(self):
      arg0=Data(0.910609441281,self.functionspace)
      arg1=numpy.array([[0.48920816916927046, 0.38845789826158716, 0.18954063794801335, 0.023657833905485814, -0.24335189451304085], [0.41796465128268401, -0.048936612595969597, 0.64823155567840685, 0.29039777666930822, -0.53281597530118585], [-0.10651289123882024, 0.83991176563396963, -0.72137841171371009, 0.55448443855013263, -0.80348654064522074], [-0.73024847464826581, -0.25234530262628896, -0.80021064888998783, 0.97549310512084197, 0.36123493587136224]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[0.48920816916927046, 0.38845789826158716, 0.18954063794801335, 0.023657833905485814, -0.24335189451304085], [0.41796465128268401, -0.048936612595969597, 0.64823155567840685, 0.29039777666930822, -0.53281597530118585], [-0.10651289123882024, 0.83991176563396963, -0.72137841171371009, 0.55448443855013263, -0.80348654064522074], [-0.73024847464826581, -0.25234530262628896, -0.80021064888998783, 0.91060944128075971, 0.36123493587136224]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[0.08078793489159497, -0.33015773315187524, 0.7208408272666289, 0.90090565140641932, 0.45038382747708505], [-0.44631629717055077, 0.92019630193275503, 0.55457196852919588, -0.96327169927714973, 0.25167968115551953], [0.82864955108820237, -0.80653937904066941, -0.35846366619861936, 0.26239388808589048, -0.91303681812093873], [0.21705894462164244, 0.47242401589506522, -0.38614229297848901, 0.15755257819143931, 0.58667580805751429]]),self.functionspace)
      arg1=numpy.array([[-0.15136529414604083, 0.65675934597948293, 0.79594791912751184, 0.0039503927380593051, 0.4030018796340924], [-0.069724849720067317, -0.24367772766093632, -0.2670851341110807, -0.14093561399556664, 0.67627965935820122], [-0.094422971120954768, 0.070547327429800744, 0.2858023558445848, -0.43318476417767449, 0.2047025555027675], [0.88303425731739571, 0.47446500843168438, 0.82298295629449925, 0.80823799516835959, -0.24838912878339392]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.15136529414604083, -0.33015773315187524, 0.7208408272666289, 0.0039503927380593051, 0.4030018796340924], [-0.44631629717055077, -0.24367772766093632, -0.2670851341110807, -0.96327169927714973, 0.25167968115551953], [-0.094422971120954768, -0.80653937904066941, -0.35846366619861936, -0.43318476417767449, -0.91303681812093873], [0.21705894462164244, 0.47242401589506522, -0.38614229297848901, 0.15755257819143931, -0.24838912878339392]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_Symbol_rank2(self):
      arg0=Data(-0.926661461929,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[0.43098175620867751, -0.70233831493858268, 0.038906006113785363, 0.46274771951940252, -0.27511520037965354], [0.13794990254380113, -0.53095534110851106, -0.010621276586338491, 0.47614717447299126, -0.81163007785983843], [0.91706713656948158, -0.81508726911005946, -0.42600984542134857, 0.76262042951360942, 0.024316253770477969], [-0.97794375929078803, -0.95623205854989979, 0.48181728354246656, 0.31527125271695233, -0.34831807752665567]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235], [-0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235], [-0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235], [-0.97794375929078803, -0.95623205854989979, -0.92666146192871235, -0.92666146192871235, -0.92666146192871235]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[0.2808029066908555, 0.9010611993886799, 0.29037274882955422, -0.090647209177802779, 0.58688177321052426], [0.27681430209205726, 0.26745048963252316, -0.026491798155260149, 0.37988382126599496, 0.69110282748420682], [0.95204816041604667, -0.94013987918904895, 0.25669760465156632, -0.17061329058727837, -0.94278722803704551], [0.82828265608883456, -0.84477111418043416, -0.96794629884558936, 0.5563029974480862, -0.78503331015363309]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[0.54435813159522839, 0.28651670072192514, 0.28150164559776458, -0.21272229466796522, -0.87755341582737212], [0.072266768633033829, 0.32834501740621636, 0.71177094914392391, 0.80437239514182646, 0.40229318372167988], [0.99092440178789865, 0.7820092191132233, 0.10249034285539338, -0.2066555610571561, -0.68586662194202974], [-0.53419830792837786, -0.37645032766098607, -0.42746664296356496, 0.34563004991437407, 0.12853450941106326]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.2808029066908555, 0.28651670072192514, 0.28150164559776458, -0.21272229466796522, -0.87755341582737212], [0.072266768633033829, 0.26745048963252316, -0.026491798155260149, 0.37988382126599496, 0.40229318372167988], [0.95204816041604667, -0.94013987918904895, 0.10249034285539338, -0.2066555610571561, -0.94278722803704551], [-0.53419830792837786, -0.84477111418043416, -0.96794629884558936, 0.34563004991437407, -0.78503331015363309]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_constData_rank2(self):
      arg0=Data(0.0239593304379,self.functionspace)
      arg1=Data(numpy.array([[-0.59888776552553891, 0.35901840582658107, 0.17754465465295954, -0.48822681164526882, 0.45276081231428589], [0.83489996452941728, -0.30010523751131601, -0.64228605313824461, 0.95333586170524698, 0.10209629399219056], [-0.54322049612252821, -0.92654073718452046, 0.75062980899156462, 0.88949655954583196, -0.3868042904873723], [-0.17063584074434668, -0.79809397702897344, -0.41996162827215389, 0.088624383432029719, 0.89769190209046035]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.59888776552553891, 0.02395933043787446, 0.02395933043787446, -0.48822681164526882, 0.02395933043787446], [0.02395933043787446, -0.30010523751131601, -0.64228605313824461, 0.02395933043787446, 0.02395933043787446], [-0.54322049612252821, -0.92654073718452046, 0.02395933043787446, 0.02395933043787446, -0.3868042904873723], [-0.17063584074434668, -0.79809397702897344, -0.41996162827215389, 0.02395933043787446, 0.02395933043787446]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-0.47049212755913183, -0.85815760292450549, -0.64814433099547597, 0.69566639600669289, 0.81114832798074943], [0.86045847392291086, -0.9507762296416713, 0.026978084565971505, 0.21635857059107022, -0.50046578534540043], [-0.37253935557470541, -0.91069795235443007, 0.91705556814935663, -0.79767296009783273, 0.8050421099408025], [-0.78804909207842289, -0.50434135046043482, -0.58071301654969454, -0.40529046283604075, 0.84807168898735696]]),self.functionspace)
      arg1=Data(numpy.array([[0.40161578060172576, 0.31441352033987457, 0.047543882582621766, -0.20104789816683488, 0.86293270121963683], [0.50656495929015333, 0.60433274181199637, 0.34562048971548243, -0.23424953925520531, -0.73111290581640276], [-0.76921531534692766, 0.28376156860966839, -0.84056992787629148, 0.32326868123239572, 0.032180307079568538], [0.091253746570855609, -0.99341071581959839, -0.34932331687068707, -0.047001517360826695, -0.82435184257320859]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[-0.47049212755913183, -0.85815760292450549, -0.64814433099547597, -0.20104789816683488, 0.81114832798074943], [0.50656495929015333, -0.9507762296416713, 0.026978084565971505, -0.23424953925520531, -0.73111290581640276], [-0.76921531534692766, -0.91069795235443007, -0.84056992787629148, -0.79767296009783273, 0.032180307079568538], [-0.78804909207842289, -0.99341071581959839, -0.58071301654969454, -0.40529046283604075, -0.82435184257320859]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_expandedData_rank2(self):
      arg0=Data(0.189657622991,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.18376557892890588, 0.12110246401885338, -0.64761301931505133, -0.69595937001166641, 0.35069437237070078], [0.32938697739593326, -0.91748075155543973, -0.72839042080982397, 0.50415374667448254, 0.40160094055295659], [0.02236767999635525, -0.76495302251454933, 0.86629740539658662, 0.18671842475226774, -0.2658514625358972], [0.95540045082863534, -0.92875987145830741, 0.66616191088909549, 0.11035961552319518, 0.10163562178107366]])+(1.-msk_arg1)*numpy.array([[0.35008579380261962, -0.60954248283174572, -0.037442952446151034, -0.14410574074733562, -0.96681555558116083], [0.29126400064373015, -0.79197966253541474, -0.91254474981822042, -0.41773510590652774, -0.17378599502988767], [0.048161799918185366, -0.27715116449573607, -0.90483097123086886, -0.80243310930260559, -0.6787797463078884], [-0.65044916557513721, -0.12487664924483943, -0.30825616699309388, 0.78378671040517389, 0.54441120182034552]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.18376557892890588, 0.12110246401885338, -0.64761301931505133, -0.69595937001166641, 0.18965762299059485], [0.18965762299059485, -0.91748075155543973, -0.72839042080982397, 0.18965762299059485, 0.18965762299059485], [0.02236767999635525, -0.76495302251454933, 0.18965762299059485, 0.18671842475226774, -0.2658514625358972], [0.18965762299059485, -0.92875987145830741, 0.18965762299059485, 0.11035961552319518, 0.10163562178107366]])+(1.-msk_ref)*numpy.array([[0.18965762299059485, -0.60954248283174572, -0.037442952446151034, -0.14410574074733562, -0.96681555558116083], [0.18965762299059485, -0.79197966253541474, -0.91254474981822042, -0.41773510590652774, -0.17378599502988767], [0.048161799918185366, -0.27715116449573607, -0.90483097123086886, -0.80243310930260559, -0.6787797463078884], [-0.65044916557513721, -0.12487664924483943, -0.30825616699309388, 0.18965762299059485, 0.18965762299059485]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-0.78654644680753871, 0.55961885541804057, 0.78635098458761354, -0.47414005397332759, -0.83773565056572608], [0.31776536222415586, 0.69745302712485069, -0.50695368166781773, 0.71939099429230158, -0.39845136350516897], [-0.60099546595039843, 0.83429249800028593, 0.0050669949631381161, 0.32053992884667837, 0.1385215907570978], [-0.38958871724035449, 0.22024255605896847, 0.20855802880863794, -0.94225915133387361, 0.082766981560819186]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.94587560739078214, -0.06012667850573572, -0.18884011568186154, -0.60124634429146795, -0.44576092212698071], [0.29806772015174787, -0.29488756301924868, -0.61373344769163674, 0.95034093102437733, 0.50416904107236293], [0.7033899516286104, 0.69091894086205641, 0.87867773693996498, -0.82966547623571274, -0.3515708581381416], [0.50229506332467166, 0.22165018627630495, 0.48988537522316977, -0.66859426709231107, 0.46826446395382471]])+(1.-msk_arg1)*numpy.array([[-0.82446139999685264, -0.93255187838673748, 0.91641818901164207, -0.58093848925466629, -0.3148674142340484], [-0.053487594740966982, 0.88279305640627359, 0.63682069571190603, 0.34983672903592811, -0.79327143310207404], [-0.69326917294834955, -0.80882632387270115, -0.87048032759728811, -0.49214313684585131, 0.84403839932037616], [0.24217467723805708, -0.40468462039855391, -0.954439477554897, 0.96941174619882831, 0.89530244951755122]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.94587560739078214, -0.06012667850573572, -0.18884011568186154, -0.60124634429146795, -0.83773565056572608], [0.29806772015174787, -0.29488756301924868, -0.61373344769163674, 0.71939099429230158, -0.39845136350516897], [-0.60099546595039843, 0.69091894086205641, 0.0050669949631381161, -0.82966547623571274, -0.3515708581381416], [-0.38958871724035449, 0.22024255605896847, 0.20855802880863794, -0.94225915133387361, 0.082766981560819186]])+(1.-msk_ref)*numpy.array([[-0.82446139999685264, -0.93255187838673748, 0.78635098458761354, -0.58093848925466629, -0.83773565056572608], [-0.053487594740966982, 0.69745302712485069, -0.50695368166781773, 0.34983672903592811, -0.79327143310207404], [-0.69326917294834955, -0.80882632387270115, -0.87048032759728811, -0.49214313684585131, 0.1385215907570978], [-0.38958871724035449, -0.40468462039855391, -0.954439477554897, -0.94225915133387361, 0.082766981560819186]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_array_rank3(self):
      arg0=Data(-0.399707245146,self.functionspace)
      arg1=numpy.array([[[0.32117886095005566, 0.56385535645967932], [-0.31072551227289691, -0.47028934103651188]], [[0.63314052806338372, 0.39229120262840222], [-0.96294339222657377, 0.38844528467320361]], [[-0.07479702462209481, -0.30265890482966595], [0.23289771129855019, -0.38794173062652981]], [[-0.67550413520579045, 0.25973443342653324], [-0.81714722110875426, 0.36919583400258849]], [[0.93260213643622958, -0.66295130418161108], [-0.72853296567395098, -0.93602246828388491]], [[-0.61740804378795633, -0.76118555874949156], [-0.28823968696345581, 0.64899458607866034]]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.39970724514590361, -0.39970724514590361], [-0.39970724514590361, -0.47028934103651188]], [[-0.39970724514590361, -0.39970724514590361], [-0.96294339222657377, -0.39970724514590361]], [[-0.39970724514590361, -0.39970724514590361], [-0.39970724514590361, -0.39970724514590361]], [[-0.67550413520579045, -0.39970724514590361], [-0.81714722110875426, -0.39970724514590361]], [[-0.39970724514590361, -0.66295130418161108], [-0.72853296567395098, -0.93602246828388491]], [[-0.61740804378795633, -0.76118555874949156], [-0.39970724514590361, -0.39970724514590361]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_array_rank3(self):
      arg0=Data(numpy.array([[[0.44305388856965466, 0.0013166608880175179], [-0.076147766778611459, -0.14487786772472755]], [[-0.67723537207517381, -0.070777211678995089], [-0.63470560980709556, -0.30698248123253213]], [[-0.39598874813071938, -0.90648087138003741], [0.1214487144962384, 0.34356420008573307]], [[-0.69848939714645186, 0.68144645566325046], [-0.66662066848117507, -0.7147542146013306]], [[-0.96447338478373013, 0.73254823315118878], [-0.75980707780766821, -0.62572827865652214]], [[-0.48777657969769672, 0.73666769252686093], [-0.67894536034535968, -0.57970185442492972]]]),self.functionspace)
      arg1=numpy.array([[[-0.70005976804998915, 0.48549864818248989], [0.036760051800379889, 0.32565531260532921]], [[0.10129372494847333, -0.71024810419330509], [-0.069828207153450395, -0.3699999105876941]], [[0.63841369462640207, 0.89143029501478876], [-0.33109302170189769, -0.99066238438389886]], [[0.59337989328951224, -0.50689562938629584], [-0.45502338271299769, -0.28400466841449035]], [[-0.5042825245311775, 0.77442066705951818], [-0.048179651918959898, -0.72572413224436638]], [[-0.63067040118636131, 0.21983466534382412], [0.21907146521294329, -0.15512888807256653]]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.70005976804998915, 0.0013166608880175179], [-0.076147766778611459, -0.14487786772472755]], [[-0.67723537207517381, -0.71024810419330509], [-0.63470560980709556, -0.3699999105876941]], [[-0.39598874813071938, -0.90648087138003741], [-0.33109302170189769, -0.99066238438389886]], [[-0.69848939714645186, -0.50689562938629584], [-0.66662066848117507, -0.7147542146013306]], [[-0.96447338478373013, 0.73254823315118878], [-0.75980707780766821, -0.72572413224436638]], [[-0.63067040118636131, 0.21983466534382412], [-0.67894536034535968, -0.57970185442492972]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_Symbol_rank3(self):
      arg0=Data(0.735285914782,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[-0.50798052449578912, 0.62222845873339949], [0.22149507872711749, -0.9178571706795664]], [[-0.36454601635958839, 0.89930989593928157], [0.35425600101013721, -0.44918488310946203]], [[0.89680176582067639, -0.5164279445805271], [-0.60268416402483815, -0.72632320918973936]], [[0.36878517133648359, 0.2511995535679139], [0.34112716491032491, -0.22182004306242997]], [[-0.47728030650052622, -0.060545528874624388], [0.97640332332281266, -0.94778945971937301]], [[-0.041532480208000155, 0.57289838456355313], [0.80747225113914278, 0.19379790915075201]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.50798052449578912, 0.62222845873339949], [0.22149507872711749, -0.9178571706795664]], [[-0.36454601635958839, 0.73528591478210648], [0.35425600101013721, -0.44918488310946203]], [[0.73528591478210648, -0.5164279445805271], [-0.60268416402483815, -0.72632320918973936]], [[0.36878517133648359, 0.2511995535679139], [0.34112716491032491, -0.22182004306242997]], [[-0.47728030650052622, -0.060545528874624388], [0.73528591478210648, -0.94778945971937301]], [[-0.041532480208000155, 0.57289838456355313], [0.73528591478210648, 0.19379790915075201]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_Symbol_rank3(self):
      arg0=Data(numpy.array([[[-0.58304628381912016, -0.23531993551482411], [0.82429648900917041, 0.93971287567890083]], [[0.44346626533189881, 0.41170755163605244], [-0.38567798172131917, -0.6881885901007434]], [[-0.57572602837295839, 0.54500820570609054], [-0.28812087775606088, -0.40261980486737436]], [[-0.93720928987828711, 0.90381854676073181], [-0.0779019533319254, 0.22947721190382819]], [[-0.31246286902471021, -0.73201401313048886], [-0.52642080365671462, 0.55688071035190978]], [[-0.8652530494148063, -0.26773432394272167], [0.079778791504450774, -0.67502109329347326]]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[-0.8489210073261384, 0.0080443560518137502], [0.63860428467272534, 0.28177676821052344]], [[0.36957256668628924, -0.76390300823156276], [-0.52919616471996678, -0.88533617012197618]], [[0.27355677942036816, -0.8753918126064173], [-0.77586298355575201, 0.16388210647867396]], [[-0.25390968924469814, -0.84159655473354289], [-0.50499244634457141, 0.845403210723918]], [[0.29331245627059244, 0.98563128344924755], [-0.39953268163563149, 0.032502708251956891]], [[0.96960747123959212, -0.58932304761752197], [0.19703691771735588, 0.88509923006814151]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-0.8489210073261384, -0.23531993551482411], [0.63860428467272534, 0.28177676821052344]], [[0.36957256668628924, -0.76390300823156276], [-0.52919616471996678, -0.88533617012197618]], [[-0.57572602837295839, -0.8753918126064173], [-0.77586298355575201, -0.40261980486737436]], [[-0.93720928987828711, -0.84159655473354289], [-0.50499244634457141, 0.22947721190382819]], [[-0.31246286902471021, -0.73201401313048886], [-0.52642080365671462, 0.032502708251956891]], [[-0.8652530494148063, -0.58932304761752197], [0.079778791504450774, -0.67502109329347326]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_constData_rank3(self):
      arg0=Data(0.306106140788,self.functionspace)
      arg1=Data(numpy.array([[[-0.76628841233666734, -0.96371501470546184], [-0.34126495594876216, 0.18894152239541517]], [[-0.36483136112890335, 0.8943400498524452], [-0.54631863424863458, -0.84010175892825467]], [[0.0081249961006295557, -0.67664073631754462], [0.44319699434566351, 0.96423211852501933]], [[0.91667848543544794, 0.89893697272059359], [-0.20338371318227799, -0.30427474472848037]], [[0.44194604199111009, -0.59352681992452183], [-0.11023929944091049, -0.64282414795628573]], [[-0.014833179963952947, 0.57301070365369355], [-0.36132359965492999, -0.98894708526171526]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[-0.76628841233666734, -0.96371501470546184], [-0.34126495594876216, 0.18894152239541517]], [[-0.36483136112890335, 0.30610614078839071], [-0.54631863424863458, -0.84010175892825467]], [[0.0081249961006295557, -0.67664073631754462], [0.30610614078839071, 0.30610614078839071]], [[0.30610614078839071, 0.30610614078839071], [-0.20338371318227799, -0.30427474472848037]], [[0.30610614078839071, -0.59352681992452183], [-0.11023929944091049, -0.64282414795628573]], [[-0.014833179963952947, 0.30610614078839071], [-0.36132359965492999, -0.98894708526171526]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_constData_rank3(self):
      arg0=Data(numpy.array([[[0.24567939374194059, -0.38142740695929578], [-0.45193989939619628, -0.60115687085650782]], [[0.40594139345453262, 0.97508967903353594], [0.34438709659068212, 0.31053499742808777]], [[0.93059078021899566, 0.96998574489152789], [-0.84363978388948002, 0.6629663278454232]], [[-0.41723438500109711, 0.42734365580775457], [-0.088489927594940898, 0.90823902949603252]], [[-0.046089968165372808, 0.36520442413194498], [-0.65188308833513231, -0.5383045342958277]], [[-0.40865875951128161, 0.45119724930116223], [0.069688167864556672, 0.30197762054058819]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.44671276211928546, 0.26562039078771993], [-0.28157505639816005, -0.4851959967926136]], [[0.17825021121860729, 0.75683931430668094], [0.12694088292947425, -0.42472824719280333]], [[0.44726690408557213, -0.21945987825453672], [-0.14909412802808797, -0.63216133393393603]], [[-0.69495213737893424, 0.34469639671173802], [0.3878750427664559, 0.60867075359136424]], [[0.67057924207194852, -0.028638569332069874], [0.073464850844077612, 0.29022832698390588]], [[-0.75093446070075354, 0.32452516889914951], [0.40790244242845186, 0.50041267619106833]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[0.24567939374194059, -0.38142740695929578], [-0.45193989939619628, -0.60115687085650782]], [[0.17825021121860729, 0.75683931430668094], [0.12694088292947425, -0.42472824719280333]], [[0.44726690408557213, -0.21945987825453672], [-0.84363978388948002, -0.63216133393393603]], [[-0.69495213737893424, 0.34469639671173802], [-0.088489927594940898, 0.60867075359136424]], [[-0.046089968165372808, -0.028638569332069874], [-0.65188308833513231, -0.5383045342958277]], [[-0.75093446070075354, 0.32452516889914951], [0.069688167864556672, 0.30197762054058819]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_expandedData_rank3(self):
      arg0=Data(-0.836569215512,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.0024226368124502518, 0.64626783949599176], [-0.71898521427264317, -0.18483317016051592]], [[-0.47307147238988212, 0.79763262909599852], [0.52364063135954586, -0.61937346851712216]], [[-0.76282149769301033, -0.64904752516245057], [0.93907088642565162, 0.96685235581600226]], [[0.12189103305672844, -0.51070310777104777], [-0.090621388334109154, 0.22410552394163807]], [[0.91340775337554625, 0.35287893605146636], [-0.74762018439877886, -0.15696287119314145]], [[0.11546929065407152, -0.29744659369830906], [0.93497363688637014, -0.11682432676327692]]])+(1.-msk_arg1)*numpy.array([[[-0.94931185014972086, 0.99193538177716722], [-0.035269459879945231, 0.35464262245808165]], [[-0.60551379563530783, 0.51916345808629627], [0.26545394272858713, -0.81998328654200936]], [[-0.95242636787230794, 0.52460143031151452], [0.99799721328684132, -0.79436151790394627]], [[-0.95708432366448304, 0.90933789353263106], [-0.022305122342574091, -0.89613876437123396]], [[0.058129126978553103, -0.69704018907759346], [0.39959135809831436, -0.64652472369812308]], [[-0.75606632025522269, 0.33381184296675093], [-0.54165429915660934, -0.63654269092587668]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]]])+(1.-msk_ref)*numpy.array([[[-0.94931185014972086, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.95242636787230794, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.95708432366448304, -0.8365692155115747], [-0.8365692155115747, -0.89613876437123396]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]], [[-0.8365692155115747, -0.8365692155115747], [-0.8365692155115747, -0.8365692155115747]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank3_expandedData_rank3(self):
      arg0=Data(numpy.array([[[0.75964452604296873, -0.89348107946885191], [-0.41218941014767307, 0.40844033266000968]], [[0.86046959206135343, -0.91223626329510155], [-0.11862402483165457, -0.62819770978089395]], [[0.56560345509134691, 0.10568049331599827], [0.78793384183324888, 0.21337503906297495]], [[0.66193858822265805, -0.064401115240066087], [0.24647023480484376, -0.47588212296961552]], [[-0.74179072374435751, 0.088069147157366023], [-0.79176467880000967, 0.21210987755503719]], [[0.49387369092153666, -0.31959746689300417], [-0.34097200906052572, 0.13922605308003688]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.31595471950209397, 0.82453578732293487], [0.75794522324307767, 0.63644182349283529]], [[0.72925822129227158, 0.52087049163865173], [-0.45476973061941273, -0.89335455668133612]], [[0.61143698481419473, -0.65384306592233199], [0.378636765400447, -0.023818683640318028]], [[-0.938532389456745, -0.00027962213071797137], [-0.43930821567776546, 0.44332698824814143]], [[0.5329108455500724, 0.47486313435035754], [-0.61132171802401558, 0.097006988244661585]], [[-0.3391099827768016, -0.76579771581643419], [0.41020559026703696, 0.86559375559218066]]])+(1.-msk_arg1)*numpy.array([[[-0.26331725482265234, -0.57792952276439524], [-0.2050465665484682, -0.21770528604664929]], [[-0.31732965952574066, -0.42106371843051038], [-0.66318456931694159, 0.5182526477227194]], [[0.77683969846260958, 0.73693704099147594], [0.76609864599594135, 0.93451258070155774]], [[0.78588350599736012, 0.068555366305665499], [0.19918094727315583, -0.20734889615792107]], [[0.75154151939412506, -0.01180570902421052], [-0.068804421156342555, -0.26082347483593238]], [[-0.98873689234072804, 0.35189554141420087], [-0.50861156667682361, 0.44100584053254077]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.31595471950209397, -0.89348107946885191], [-0.41218941014767307, 0.40844033266000968]], [[0.72925822129227158, -0.91223626329510155], [-0.45476973061941273, -0.89335455668133612]], [[0.56560345509134691, -0.65384306592233199], [0.378636765400447, -0.023818683640318028]], [[-0.938532389456745, -0.064401115240066087], [-0.43930821567776546, -0.47588212296961552]], [[-0.74179072374435751, 0.088069147157366023], [-0.79176467880000967, 0.097006988244661585]], [[-0.3391099827768016, -0.76579771581643419], [-0.34097200906052572, 0.13922605308003688]]])+(1.-msk_ref)*numpy.array([[[-0.26331725482265234, -0.89348107946885191], [-0.41218941014767307, -0.21770528604664929]], [[-0.31732965952574066, -0.91223626329510155], [-0.66318456931694159, -0.62819770978089395]], [[0.56560345509134691, 0.10568049331599827], [0.76609864599594135, 0.21337503906297495]], [[0.66193858822265805, -0.064401115240066087], [0.19918094727315583, -0.47588212296961552]], [[-0.74179072374435751, -0.01180570902421052], [-0.79176467880000967, -0.26082347483593238]], [[-0.98873689234072804, -0.31959746689300417], [-0.50861156667682361, 0.13922605308003688]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_array_rank4(self):
      arg0=Data(-0.441965249048,self.functionspace)
      arg1=numpy.array([[[[0.90170112707882399, -0.64427343648076119, 0.35920636506172676, 0.057389178295796262], [0.22997258478639138, 0.44784729359623054, -0.928355620464278, 0.49366772307302731], [0.36717630946052582, -0.16851022366846236, -0.87888599683564927, 0.28360278891652557]], [[0.5831228061573368, 0.79071167169113088, 0.7172198061891184, 0.068659884110793046], [-0.51441433806109282, 0.51207829378963354, -0.37426160656263474, 0.9633698226222962], [-0.57999017047476609, -0.3669237698440464, 0.85356673680899697, 0.86349829166070169]]], [[[-0.95543429065165419, -0.54201040689187741, 0.67156808962188919, -0.055940012634396297], [-0.45225528034059992, -0.95114412168177509, 0.088098067745095543, 0.22717635789586366], [0.095818180743235004, -0.31854132927367074, -0.14782415215257427, 0.63959521433800393]], [[-0.18612216897636258, -0.63008696815824217, -0.9871454720776065, -0.80139763362000016], [0.80414211200737284, 0.18635900004782258, -0.61604752040682942, 0.1648537654269675], [-0.24712466266261268, 0.79298079666599364, 0.013292716674855543, -0.65192047085780369]]], [[[-0.07948997363400423, 0.83622357362129462, -0.64710935873163944, 0.78088026623203199], [-0.079437465004570385, -0.24452858641521313, -0.19057591654920092, -0.60089415009256708], [-0.069520201782470314, 0.36883323379128519, 0.042285991306272752, -0.77443470266431436]], [[-0.84569211427906454, -0.44482174237235217, 0.11489913371520433, 0.54835343548387994], [0.97320324483020282, -0.61405933933254664, 0.33173263918038032, -0.57050877022505531], [-0.12077190543524385, 0.36150632743457489, -0.068101080694213945, -0.4546305711521279]]]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.4419652490476238, -0.64427343648076119, -0.4419652490476238, -0.4419652490476238], [-0.4419652490476238, -0.4419652490476238, -0.928355620464278, -0.4419652490476238], [-0.4419652490476238, -0.4419652490476238, -0.87888599683564927, -0.4419652490476238]], [[-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.4419652490476238], [-0.51441433806109282, -0.4419652490476238, -0.4419652490476238, -0.4419652490476238], [-0.57999017047476609, -0.4419652490476238, -0.4419652490476238, -0.4419652490476238]]], [[[-0.95543429065165419, -0.54201040689187741, -0.4419652490476238, -0.4419652490476238], [-0.45225528034059992, -0.95114412168177509, -0.4419652490476238, -0.4419652490476238], [-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.4419652490476238]], [[-0.4419652490476238, -0.63008696815824217, -0.9871454720776065, -0.80139763362000016], [-0.4419652490476238, -0.4419652490476238, -0.61604752040682942, -0.4419652490476238], [-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.65192047085780369]]], [[[-0.4419652490476238, -0.4419652490476238, -0.64710935873163944, -0.4419652490476238], [-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.60089415009256708], [-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.77443470266431436]], [[-0.84569211427906454, -0.44482174237235217, -0.4419652490476238, -0.4419652490476238], [-0.4419652490476238, -0.61405933933254664, -0.4419652490476238, -0.57050877022505531], [-0.4419652490476238, -0.4419652490476238, -0.4419652490476238, -0.4546305711521279]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[-0.51530089384428823, -0.5345844623433913, 0.87401774844575941, 0.45002410870429377], [0.904555542052909, -0.18275561484024849, -0.21404331798026099, -0.28848183617848155], [0.24054399907052826, 0.35436917676168878, -0.45959352447418755, 0.43151245238610403]], [[0.068395691230318967, 0.72778171656693558, -0.81685869439276226, 0.28666012215553627], [0.069979226420893381, -0.44552834059566093, 0.9000378385296457, 0.25755405964297506], [-0.93840728377736071, 0.085503990101015503, -0.52327551969733488, -0.47600827962065573]]], [[[0.35891500889634043, -0.76794687894658797, 0.27297737299087221, -0.14444722654289421], [-0.65807224946127207, 0.33656135158116518, 0.53691155421768966, -0.82123315518174667], [0.74691672155002986, -0.22265611186493373, 0.014444817111401553, -0.68448298226507109]], [[0.10529759492705582, 0.47149678183606492, 0.89842029505228127, 0.52171041358953962], [0.53282707947023966, 0.61961053067985539, -0.10855051050934916, 0.363477700849552], [-0.38195165309676948, 0.96497264562810536, -0.61516337011164413, 0.24928063284391966]]], [[[-0.41137557106048472, -0.14139620422554033, -0.040691550605413074, -0.40176139538677047], [-0.76064522142761892, -0.51725323830086323, 0.66334491900051384, -0.64739820840737905], [0.019124520418513979, 0.39115651057203404, -0.17472681319897276, -0.02651555586260379]], [[-0.95082839249290152, -0.20596353818634294, -0.57721748029229958, 0.60568524787826306], [0.9835866552981587, -0.69093164788788619, -0.92689157655664034, -0.36153457781800702], [-0.39218315735539644, -0.2610778320289322, -0.85950149433400092, 0.52441170091953859]]]]),self.functionspace)
      arg1=numpy.array([[[[0.43303953044031118, -0.73571584733951711, 0.0042665769615761739, -0.10257787561659071], [0.19809834639869695, -0.259483377199629, 0.57019413308355027, 0.1244648016566452], [0.029029608792817463, 0.70934806279242157, -0.89042703551266111, 0.39381556732884193]], [[0.99499159811475391, 0.24566810789927018, -0.21693845637681508, -0.23380543644165863], [0.5178154149165739, -0.75797274290326455, -0.27361700039045589, -0.527160043141083], [-0.46223530755517839, 0.20849539411532669, 0.39518075099797745, -0.41269654461030192]]], [[[0.67953505185008312, 0.054156715132414712, 0.19072473654602673, 0.25942335630497637], [0.72590396510832367, -0.97370415191925996, 0.40403187319501321, 0.066046555101955784], [-0.67087636019860319, -0.33754965949797699, 0.66566569311518031, 0.25311273314057092]], [[-0.49337017567894459, 0.41205367414414917, -0.31007193815224943, 0.80676899243211464], [-0.85418776374264738, -0.98351831500831199, -0.59719845307986663, -0.95737176975883198], [-0.64198375664141705, 0.57548823578699504, -0.25900425755775469, 0.14619726680373546]]], [[[-0.56765617368135124, 0.39274800205686411, 0.63136199848034624, -0.46983677240895538], [0.60607116354140467, 0.15641956236114596, 0.026688254748447271, 0.42814938916885259], [-0.63078578929266249, -0.56088664335956318, -0.93644570551476702, -0.36325409946308973]], [[-0.4432069513096375, -0.48070579866519259, -0.3600050021296981, -0.18010675378602881], [0.7073427260537446, 0.44691840750064449, 0.62666275491474166, 0.48451269233352146], [-0.60540524473384205, -0.29303927106857852, -0.3105015501989401, 0.90154427673081505]]]])
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.51530089384428823, -0.73571584733951711, 0.0042665769615761739, -0.10257787561659071], [0.19809834639869695, -0.259483377199629, -0.21404331798026099, -0.28848183617848155], [0.029029608792817463, 0.35436917676168878, -0.89042703551266111, 0.39381556732884193]], [[0.068395691230318967, 0.24566810789927018, -0.81685869439276226, -0.23380543644165863], [0.069979226420893381, -0.75797274290326455, -0.27361700039045589, -0.527160043141083], [-0.93840728377736071, 0.085503990101015503, -0.52327551969733488, -0.47600827962065573]]], [[[0.35891500889634043, -0.76794687894658797, 0.19072473654602673, -0.14444722654289421], [-0.65807224946127207, -0.97370415191925996, 0.40403187319501321, -0.82123315518174667], [-0.67087636019860319, -0.33754965949797699, 0.014444817111401553, -0.68448298226507109]], [[-0.49337017567894459, 0.41205367414414917, -0.31007193815224943, 0.52171041358953962], [-0.85418776374264738, -0.98351831500831199, -0.59719845307986663, -0.95737176975883198], [-0.64198375664141705, 0.57548823578699504, -0.61516337011164413, 0.14619726680373546]]], [[[-0.56765617368135124, -0.14139620422554033, -0.040691550605413074, -0.46983677240895538], [-0.76064522142761892, -0.51725323830086323, 0.026688254748447271, -0.64739820840737905], [-0.63078578929266249, -0.56088664335956318, -0.93644570551476702, -0.36325409946308973]], [[-0.95082839249290152, -0.48070579866519259, -0.57721748029229958, -0.18010675378602881], [0.7073427260537446, -0.69093164788788619, -0.92689157655664034, -0.36153457781800702], [-0.60540524473384205, -0.29303927106857852, -0.85950149433400092, 0.52441170091953859]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_Symbol_rank4(self):
      arg0=Data(-0.245549496412,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[-0.15122997161907215, -0.14411909127500055, 0.18889048155883081, -0.58151885974657858], [-0.38911810296942995, -0.50666314346719643, 0.81336886483195059, 0.6886912058038106], [-0.048257306725079774, 0.43614122989493698, 0.92475711357784895, 0.83187442424649816]], [[0.44158070163135554, -0.68560852878910072, -0.32756832255953183, 0.56827326962347047], [-0.004385576552391024, -0.42425434421977415, 0.36254840063460336, 0.30239035101805123], [0.67284231550084539, -0.8281090849130448, 0.54662750719384223, 0.19890340968609599]]], [[[0.94816681433039118, -0.8784800828818462, -0.68175632397400188, -0.14436301228417059], [-0.55105084315707131, 0.7629183662814869, -0.043762900419689554, 0.8301272992471147], [-0.24889924886877957, -0.75522885944489904, 0.14613789520603682, 0.72490716348836592]], [[-0.3069706216525212, 0.43845127208636114, 0.99078445440101914, -0.10893078944454482], [-0.53659329746893891, -0.11600731298385503, -0.033209302221220538, 0.73320700965472829], [-0.39727295094865989, 0.99281743066728056, -0.45102312902656094, 0.20416358829056458]]], [[[0.57785996409695883, 0.15041265653715419, -0.081284878438162123, -0.78488862960175965], [0.13701152873544742, 0.39917975007870155, -0.014568497246839618, 0.34476888942036465], [0.91312437548828851, 0.25845595007223587, 0.81742541216091569, -0.067147556164976807]], [[0.53745791273467014, -0.62965565607069074, -0.9376484478247713, -0.27225069021039783], [-0.79655585688205455, -0.6486112305000229, -0.73647014621643092, -0.64782815701805685], [-0.804970968268927, 0.3644103347094414, 0.19144398343347535, 0.36177697278121701]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-0.24554949641155566, -0.24554949641155566, -0.24554949641155566, -0.58151885974657858], [-0.38911810296942995, -0.50666314346719643, -0.24554949641155566, -0.24554949641155566], [-0.24554949641155566, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566]], [[-0.24554949641155566, -0.68560852878910072, -0.32756832255953183, -0.24554949641155566], [-0.24554949641155566, -0.42425434421977415, -0.24554949641155566, -0.24554949641155566], [-0.24554949641155566, -0.8281090849130448, -0.24554949641155566, -0.24554949641155566]]], [[[-0.24554949641155566, -0.8784800828818462, -0.68175632397400188, -0.24554949641155566], [-0.55105084315707131, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566], [-0.24889924886877957, -0.75522885944489904, -0.24554949641155566, -0.24554949641155566]], [[-0.3069706216525212, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566], [-0.53659329746893891, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566], [-0.39727295094865989, -0.24554949641155566, -0.45102312902656094, -0.24554949641155566]]], [[[-0.24554949641155566, -0.24554949641155566, -0.24554949641155566, -0.78488862960175965], [-0.24554949641155566, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566], [-0.24554949641155566, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566]], [[-0.24554949641155566, -0.62965565607069074, -0.9376484478247713, -0.27225069021039783], [-0.79655585688205455, -0.6486112305000229, -0.73647014621643092, -0.64782815701805685], [-0.804970968268927, -0.24554949641155566, -0.24554949641155566, -0.24554949641155566]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[0.40914241600458334, -0.37791162517693788, 0.49259995412141144, 0.23978302005126362], [0.18921143337929625, -0.84110079966033169, -0.58646715803166916, -0.44544857124273429], [-0.58233159278890945, 0.17275523921687519, 0.47341770754381818, -0.73063062026277548]], [[0.49298914468370914, 0.67407680403351411, -0.17673468745876164, -0.7958926990687949], [-0.53202595962536825, 0.70445367969174866, -0.16219170388349569, -0.41442772296018782], [-0.72338166890664013, -0.6908730598177022, 0.11256297990948316, -0.67630393093529784]]], [[[0.27947113329550111, 0.12737358655557163, -0.20590196552130924, -0.42740646460398413], [0.20855944355235412, -0.87493560411088867, 0.7276918084137598, -0.94842451718572529], [-0.18497308467553975, 0.72423375427898096, -0.11398471626404505, -0.61173175853942841]], [[-0.26491181118302221, -0.24880447906360814, 0.68679168804110069, -0.9903966966633091], [-0.54770853720656043, 0.45287764230535754, -0.50891801092371258, -0.93378099413211135], [-0.25872884524060202, -0.75263355426673706, -0.53400164188402344, -0.54935391468697148]]], [[[0.04341257079775529, 0.58511671525784936, 0.17492085756800813, -0.89852950852798319], [0.23022268311038596, -0.2801213673896461, -0.9723213478734829, 0.10165141603531724], [0.18341813300362775, 0.62524828110800046, 0.99555596036690641, -0.51888659708961349]], [[-0.89670219019045727, 0.47596688585209734, 0.63016480796777796, -0.93031551729978057], [0.96703472729607487, -0.87693163411531705, 0.71268438754280172, 0.32084709407062606], [0.54120818881957633, 0.64299280894448763, 0.97934482156284308, -0.69788311345021214]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[0.59439169626434296, -0.081487991000120497, -0.46334955845116688, 0.40429459926177524], [-0.9631981694718148, 0.5356317808247939, 0.98143116107245421, -0.14114548021055939], [0.78932087290152797, -0.1457069174141099, -0.18812532250207292, -0.41882657262714407]], [[0.33532439366035538, 0.99749909772237677, 0.47991527839831494, -0.095025052900759022], [-0.18041389552138476, 0.065107043783313223, -0.74260164906098303, -0.63940438632402019], [0.37975293390454468, 0.94420135325670596, -0.5683302470015994, 0.89372433163565068]]], [[[-0.71327043542463064, 0.028888232150588999, 0.82938971769083558, -0.61410987208004308], [-0.11490745104312783, -0.13251043235873761, 0.94692653301160257, 0.72630396850483736], [0.31762426225040818, 0.0024370145795582498, -0.39243151118532782, 0.58374834767852946]], [[0.60148892660279496, 0.43338097817768162, 0.63997177071306677, 0.14919962262103859], [0.87971400376804465, -0.25543434569474122, 0.16723514445623033, 0.41970705914099482], [0.71331012432438001, -0.61189482059545419, 0.40377678364247482, -0.04070256812090367]]], [[[-0.39003876239894142, 0.1289297363946782, -0.74276812594474206, 0.45309930259518394], [-0.36186572196874867, -0.32932537598843292, -0.35060940514117345, -0.93856585173611617], [-0.55548866305999622, 0.29838209255040327, -0.14612330066369661, 0.78027652540055814]], [[-0.91709305957125076, 0.36499993011017384, -0.032342905989356741, -0.64718409898222751], [0.067050591976438367, 0.79987410328492237, 0.91284096110843893, -0.63043571030482659], [-0.52745963363171744, 0.64166490147571675, 0.31846801128441693, 0.49678731756539318]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.40914241600458334, -0.37791162517693788, -0.46334955845116688, 0.23978302005126362], [-0.9631981694718148, -0.84110079966033169, -0.58646715803166916, -0.44544857124273429], [-0.58233159278890945, -0.1457069174141099, -0.18812532250207292, -0.73063062026277548]], [[0.33532439366035538, 0.67407680403351411, -0.17673468745876164, -0.7958926990687949], [-0.53202595962536825, 0.065107043783313223, -0.74260164906098303, -0.63940438632402019], [-0.72338166890664013, -0.6908730598177022, -0.5683302470015994, -0.67630393093529784]]], [[[-0.71327043542463064, 0.028888232150588999, -0.20590196552130924, -0.61410987208004308], [-0.11490745104312783, -0.87493560411088867, 0.7276918084137598, -0.94842451718572529], [-0.18497308467553975, 0.0024370145795582498, -0.39243151118532782, -0.61173175853942841]], [[-0.26491181118302221, -0.24880447906360814, 0.63997177071306677, -0.9903966966633091], [-0.54770853720656043, -0.25543434569474122, -0.50891801092371258, -0.93378099413211135], [-0.25872884524060202, -0.75263355426673706, -0.53400164188402344, -0.54935391468697148]]], [[[-0.39003876239894142, 0.1289297363946782, -0.74276812594474206, -0.89852950852798319], [-0.36186572196874867, -0.32932537598843292, -0.9723213478734829, -0.93856585173611617], [-0.55548866305999622, 0.29838209255040327, -0.14612330066369661, -0.51888659708961349]], [[-0.91709305957125076, 0.36499993011017384, -0.032342905989356741, -0.93031551729978057], [0.067050591976438367, -0.87693163411531705, 0.71268438754280172, -0.63043571030482659], [-0.52745963363171744, 0.64166490147571675, 0.31846801128441693, -0.69788311345021214]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_constData_rank4(self):
      arg0=Data(0.891735083193,self.functionspace)
      arg1=Data(numpy.array([[[[-0.72203768857693551, -0.41411712052199667, -0.035331134676287679, 0.55758258245386427], [0.75338709881292676, -0.36350254911788116, -0.75993093691310132, 0.70447797605003237], [-0.037788491392551915, 0.56878553125366382, 0.81116121000923824, 0.2223985611830317]], [[-0.41515674664945834, 0.037076606947510582, -0.85516490079085417, -0.048103352600587845], [0.78899433337234859, -0.39667640020196338, -0.39907228581187004, 0.37033536518374444], [0.6502243364215694, -0.54408171649819681, -0.11018474323398109, 0.28802582231668872]]], [[[0.11960730057388647, -0.27231559614579326, 0.97477423712713884, 0.87436535045988562], [0.40588083121866858, 0.67917533822992349, 0.53624575670919006, 0.42863165673791803], [-0.81421658977971934, -0.47090725645970055, 0.52414977704280918, -0.86989526260792149]], [[0.5262247411090677, -0.83468582885633391, 0.47908955620427496, -0.19151822890455361], [0.68839087823497791, -0.80199977977182701, -0.99698818178604953, 0.6450566248973959], [-0.10957877883728395, 0.66604278801404204, 0.20364558370243446, 0.4630430950022415]]], [[[0.57174367066436149, -0.92783012055691327, -0.71868586438947601, -0.56230882090472978], [0.051114429961445662, -0.065326037484948474, -0.02928983521946793, 0.4244404756507314], [-0.15803743255916403, 0.46586183224671363, -0.15594140495277231, -0.11096137267486283]], [[0.074437673391905435, -0.17916258499215765, 0.58674546072894329, -0.29877668964767268], [0.18909123865906041, -0.70642032548817313, -0.29342586282114747, 0.71513650161087261], [-0.028468734471830848, 0.6230463797833472, -0.097661383995800044, 0.17315221205431386]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.72203768857693551, -0.41411712052199667, -0.035331134676287679, 0.55758258245386427], [0.75338709881292676, -0.36350254911788116, -0.75993093691310132, 0.70447797605003237], [-0.037788491392551915, 0.56878553125366382, 0.81116121000923824, 0.2223985611830317]], [[-0.41515674664945834, 0.037076606947510582, -0.85516490079085417, -0.048103352600587845], [0.78899433337234859, -0.39667640020196338, -0.39907228581187004, 0.37033536518374444], [0.6502243364215694, -0.54408171649819681, -0.11018474323398109, 0.28802582231668872]]], [[[0.11960730057388647, -0.27231559614579326, 0.89173508319332551, 0.87436535045988562], [0.40588083121866858, 0.67917533822992349, 0.53624575670919006, 0.42863165673791803], [-0.81421658977971934, -0.47090725645970055, 0.52414977704280918, -0.86989526260792149]], [[0.5262247411090677, -0.83468582885633391, 0.47908955620427496, -0.19151822890455361], [0.68839087823497791, -0.80199977977182701, -0.99698818178604953, 0.6450566248973959], [-0.10957877883728395, 0.66604278801404204, 0.20364558370243446, 0.4630430950022415]]], [[[0.57174367066436149, -0.92783012055691327, -0.71868586438947601, -0.56230882090472978], [0.051114429961445662, -0.065326037484948474, -0.02928983521946793, 0.4244404756507314], [-0.15803743255916403, 0.46586183224671363, -0.15594140495277231, -0.11096137267486283]], [[0.074437673391905435, -0.17916258499215765, 0.58674546072894329, -0.29877668964767268], [0.18909123865906041, -0.70642032548817313, -0.29342586282114747, 0.71513650161087261], [-0.028468734471830848, 0.6230463797833472, -0.097661383995800044, 0.17315221205431386]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-0.47777935053857146, 0.51316552051709263, 0.88742512685450414, 0.48952191876435913], [0.41202779986075044, 0.77404073221013903, -0.94676425093241035, 0.50237644935067216], [-0.37965342477781316, 0.67131426768130131, -0.14063757901097174, -0.26750582781962473]], [[-0.050107785374164715, -0.83992581313093662, -0.80250312368621834, -0.33568592401079411], [-0.11397976003118249, -0.44669983253036549, 0.8690123529393472, 0.90063236093865551], [0.15837327302030935, -0.95854666045623516, -0.80616209677280914, -0.31842107377317364]]], [[[0.85022897247710905, 0.5839959379551829, 0.44612811860910351, 0.10978570557810263], [0.6430376987942843, 0.085773987365875382, -0.98381281067446569, -0.5785782108545614], [-0.97395881411588148, 0.23293947833417428, -0.80154114989688474, -0.5411078331058059]], [[-0.20202996036682364, -0.62298638721157129, -0.7576426160764187, -0.96110793936858419], [0.039902022303325957, 0.95970890710541568, -0.49433734854466183, 0.024279241207748603], [-0.33419375034649512, 0.80948391343505643, 0.42134649611306618, -0.062467245134910199]]], [[[-0.75572791191304689, 0.70458156902591251, 0.15154171655867099, -0.4583951436461946], [0.67235681790284341, 0.53943912513522818, -0.89569430105247161, 0.36935947260022051], [-0.057387342010895237, 0.37282610862137999, -0.62070663534803949, -0.74749869011929881]], [[0.0063662944109017516, 0.95287811076359397, -0.48218175270003694, 0.99726134245386144], [-0.79471912871921724, 0.7810353581622016, -0.93026670417550372, 0.95100361471790817], [-0.26211300902795065, 0.083282449448966567, -0.48644073147660105, 0.79058224860134874]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[0.9044762237957813, -0.4165617794514338, -0.84316547505942374, -0.83502739048194075], [0.4765574192484292, 0.71882262037480693, 0.56463189469898589, -0.42463654489820257], [-0.43922183033040563, 0.38657471796018816, 0.61866675565004203, 0.93824806697394769]], [[-0.72104247828474777, 0.30022085686185962, -0.10640971824214818, -0.095386702945631674], [0.29431326865861096, 0.38141853130311043, 0.75349472607164469, -0.11359897246925343], [-0.30881692673047323, -0.054713691342018533, -0.076942269721663736, -0.86385624349593293]]], [[[-0.91975215252728471, -0.97113742068953313, 0.91998252036130057, 0.98586686095690501], [0.76005823212864088, -0.43964985948023272, 0.20604133120413848, -0.64800567651972929], [0.093176759927668718, -0.35216616072049511, 0.38818593035522286, -0.23699608167896336]], [[-0.33979970436946694, 0.37786671793745219, 0.45905605365501434, 0.32329028459669162], [0.91064201311612525, -0.5820740522897403, 0.28626480691507727, -0.25257010865511065], [-0.36624442568743398, 0.39826792488190055, -0.61074895830358722, 0.48219677250559201]]], [[[-0.46963234204880111, 0.00085481749717941646, -0.98783853145909517, 0.26145880073622019], [-0.21110894864995977, 0.82609873032705305, -0.19904218310269717, 0.82411923550522492], [-0.16294712128624456, -0.34713332785339834, -0.040962843188556031, 0.5372056695231342]], [[0.3040740909862587, 0.30653728349056175, 0.74174999512063966, 0.48061529794500668], [0.010756035221834992, -0.9065960753598894, -0.93890917258128948, -0.9725300746483847], [0.6506929930879044, 0.74106769304325515, -0.87987031844181507, 0.092640477359948514]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      ref=Data(numpy.array([[[[-0.47777935053857146, -0.4165617794514338, -0.84316547505942374, -0.83502739048194075], [0.41202779986075044, 0.71882262037480693, -0.94676425093241035, -0.42463654489820257], [-0.43922183033040563, 0.38657471796018816, -0.14063757901097174, -0.26750582781962473]], [[-0.72104247828474777, -0.83992581313093662, -0.80250312368621834, -0.33568592401079411], [-0.11397976003118249, -0.44669983253036549, 0.75349472607164469, -0.11359897246925343], [-0.30881692673047323, -0.95854666045623516, -0.80616209677280914, -0.86385624349593293]]], [[[-0.91975215252728471, -0.97113742068953313, 0.44612811860910351, 0.10978570557810263], [0.6430376987942843, -0.43964985948023272, -0.98381281067446569, -0.64800567651972929], [-0.97395881411588148, -0.35216616072049511, -0.80154114989688474, -0.5411078331058059]], [[-0.33979970436946694, -0.62298638721157129, -0.7576426160764187, -0.96110793936858419], [0.039902022303325957, -0.5820740522897403, -0.49433734854466183, -0.25257010865511065], [-0.36624442568743398, 0.39826792488190055, -0.61074895830358722, -0.062467245134910199]]], [[[-0.75572791191304689, 0.00085481749717941646, -0.98783853145909517, -0.4583951436461946], [-0.21110894864995977, 0.53943912513522818, -0.89569430105247161, 0.36935947260022051], [-0.16294712128624456, -0.34713332785339834, -0.62070663534803949, -0.74749869011929881]], [[0.0063662944109017516, 0.30653728349056175, -0.48218175270003694, 0.48061529794500668], [-0.79471912871921724, -0.9065960753598894, -0.93890917258128948, -0.9725300746483847], [-0.26211300902795065, 0.083282449448966567, -0.87987031844181507, 0.092640477359948514]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank0_expandedData_rank4(self):
      arg0=Data(0.316000715921,self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.37755969956550639, 0.36096931524775444, 0.95451965100827829, -0.43778927185741034], [-0.1629892982850687, -0.67234249843048977, -0.70971458497585305, -0.93063861231689038], [0.83603147762023577, -0.033608855259837922, -0.27179761381437872, -0.73676617519650112]], [[0.15778727712231966, 0.73747423055335704, 0.27387137708130549, 0.44356861336674069], [-0.35716705733248988, -0.84396471006743834, -0.15553398284535724, -0.93650145850970135], [0.42487581822832898, 0.23786348031884486, 0.8396087607535061, 0.38507604571186871]]], [[[0.39829311097167919, 0.67632063438127732, -0.95575880406143243, -0.75973639782870861], [0.94163396413932121, 0.11518518289272595, 0.80397065121148126, -0.79049563434914361], [0.727812873273602, 0.49635152562076179, -0.62036081501552331, -0.56579689878028372]], [[-0.26378624581050802, 0.090099480628977657, -0.49537633544458504, -0.52696913511146759], [-0.76634891512661563, 0.021743249235644413, 0.52889001340348663, -0.23091101336365827], [0.26967534066181731, 0.60851799360922665, 0.66612069814599839, -0.26596843544371485]]], [[[-0.4421738685329899, 0.54351668881464921, 0.085141505067702905, -0.71225260085711728], [0.57563600579702867, -0.38484962942553413, -0.75521288610876103, 0.28529838559586151], [0.721887966315087, -0.62430636161445729, -0.6901704195658962, 0.57637611174539227]], [[-0.2272238299492273, 0.52802390990066561, -0.53954880773503922, 0.74273270379966472], [-0.23081372311262394, 0.13968851302108121, -0.22489242015891531, -0.096494606150010931], [0.65044538305087429, -0.15595510601618234, 0.45761158985282946, 0.27521900803800459]]]])+(1.-msk_arg1)*numpy.array([[[[-0.85637723195299276, -0.22531301543043658, 0.73380568704308136, -0.19605666398270749], [-0.53037531460080878, 0.13437937725542914, -0.77731053360350244, 0.31686759234243222], [-0.53057977927535838, 0.40334641708084429, 0.18747944506803416, 0.2240578584733679]], [[-0.7582667750656864, -0.71197977040865568, -0.32083171472023597, -0.93452911580057973], [-0.2319980929469394, 0.22952685272066708, 0.99718674382924788, -0.56515226645193151], [0.23463548081498375, 0.2756888531629198, 0.22671245584002064, 0.76467896098677191]]], [[[-0.12889979409763397, 0.57877838294149542, -0.054266377341127159, 0.32225886284062533], [0.96897693705886301, 0.75226902690537178, 0.27650958731641029, -0.40548580796679912], [-0.5794149922830012, -0.93179222846167664, 0.93389510265410469, 0.45423395359675678]], [[0.51861877705472748, -0.93602090371304869, -0.76487812793156018, -0.058905480629213081], [-0.33342462067329492, -0.994038389912965, 0.13028734936504827, 0.6869314181732129], [-0.39062165766102863, -0.70044510997957499, -0.26533438821902733, 0.15795503030560276]]], [[[0.51904407201881719, 0.99197957687365435, 0.82080152139235829, 0.50162858513889907], [-0.98462565456527695, -0.15183193504837877, 0.86675561015521496, 0.82372301903200351], [-0.71821744090184803, -0.74296552709816277, -0.95671144435889177, -0.51378186028823158]], [[-0.8369732970991226, 0.81681701960603115, -0.98096813889999024, 0.20834643803841346], [-0.56513579029019856, -0.66870035965430974, -0.4843056575612692, -0.69365171409366488], [0.63435591930774482, -0.26430268204553498, 0.72043513688760585, -0.10195564698579229]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.37755969956550639, 0.31600071592090728, 0.31600071592090728, -0.43778927185741034], [-0.1629892982850687, -0.67234249843048977, -0.70971458497585305, -0.93063861231689038], [0.31600071592090728, -0.033608855259837922, -0.27179761381437872, -0.73676617519650112]], [[0.15778727712231966, 0.31600071592090728, 0.27387137708130549, 0.31600071592090728], [-0.35716705733248988, -0.84396471006743834, -0.15553398284535724, -0.93650145850970135], [0.31600071592090728, 0.23786348031884486, 0.31600071592090728, 0.31600071592090728]]], [[[0.31600071592090728, 0.31600071592090728, -0.95575880406143243, -0.75973639782870861], [0.31600071592090728, 0.11518518289272595, 0.31600071592090728, -0.79049563434914361], [0.31600071592090728, 0.31600071592090728, -0.62036081501552331, -0.56579689878028372]], [[-0.26378624581050802, 0.090099480628977657, -0.49537633544458504, -0.52696913511146759], [-0.76634891512661563, 0.021743249235644413, 0.31600071592090728, -0.23091101336365827], [0.26967534066181731, 0.31600071592090728, 0.31600071592090728, -0.26596843544371485]]], [[[-0.4421738685329899, 0.31600071592090728, 0.085141505067702905, -0.71225260085711728], [0.31600071592090728, -0.38484962942553413, -0.75521288610876103, 0.28529838559586151], [0.31600071592090728, -0.62430636161445729, -0.6901704195658962, 0.31600071592090728]], [[-0.2272238299492273, 0.31600071592090728, -0.53954880773503922, 0.31600071592090728], [-0.23081372311262394, 0.13968851302108121, -0.22489242015891531, -0.096494606150010931], [0.31600071592090728, -0.15595510601618234, 0.31600071592090728, 0.27521900803800459]]]])+(1.-msk_ref)*numpy.array([[[[-0.85637723195299276, -0.22531301543043658, 0.31600071592090728, -0.19605666398270749], [-0.53037531460080878, 0.13437937725542914, -0.77731053360350244, 0.31600071592090728], [-0.53057977927535838, 0.31600071592090728, 0.18747944506803416, 0.2240578584733679]], [[-0.7582667750656864, -0.71197977040865568, -0.32083171472023597, -0.93452911580057973], [-0.2319980929469394, 0.22952685272066708, 0.31600071592090728, -0.56515226645193151], [0.23463548081498375, 0.2756888531629198, 0.22671245584002064, 0.31600071592090728]]], [[[-0.12889979409763397, 0.31600071592090728, -0.054266377341127159, 0.31600071592090728], [0.31600071592090728, 0.31600071592090728, 0.27650958731641029, -0.40548580796679912], [-0.5794149922830012, -0.93179222846167664, 0.31600071592090728, 0.31600071592090728]], [[0.31600071592090728, -0.93602090371304869, -0.76487812793156018, -0.058905480629213081], [-0.33342462067329492, -0.994038389912965, 0.13028734936504827, 0.31600071592090728], [-0.39062165766102863, -0.70044510997957499, -0.26533438821902733, 0.15795503030560276]]], [[[0.31600071592090728, 0.31600071592090728, 0.31600071592090728, 0.31600071592090728], [-0.98462565456527695, -0.15183193504837877, 0.31600071592090728, 0.31600071592090728], [-0.71821744090184803, -0.74296552709816277, -0.95671144435889177, -0.51378186028823158]], [[-0.8369732970991226, 0.31600071592090728, -0.98096813889999024, 0.20834643803841346], [-0.56513579029019856, -0.66870035965430974, -0.4843056575612692, -0.69365171409366488], [0.31600071592090728, -0.26430268204553498, 0.31600071592090728, -0.10195564698579229]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-0.082948971486332068, -0.8185207403445125, -0.60916521073513796, 0.69890586240725083], [-0.85580002362796836, 0.46734043198688302, -0.67397330493613916, -0.52932217312961649], [0.5751081669961573, 0.052941628165801058, 0.098163203354192152, 0.38179995083187723]], [[0.53767215178340888, -0.73150448350908204, 0.79051300480353959, -0.76741350463770286], [-0.63252343652595422, -0.20704378252566169, 0.73994977677913876, 0.010608838986240876], [0.42470265198407309, 0.064509699095955719, 0.69780923538137096, -0.73368376961204418]]], [[[0.67114701541099153, -0.096186192218104649, -0.35213406260979152, 0.47414640994359991], [0.57776297104209728, -0.12129927169254184, 0.66004288476383932, -0.34976968314726742], [-0.079196098179713292, 0.8095559688460725, -0.45381584972272249, -0.42312632509663017]], [[0.45437467710092028, 0.30748161382877059, -0.029464813833504078, 0.69702146761293315], [-0.65011341281663348, -0.087301283591635359, 0.86654367992566983, 0.33315766268509006], [0.9455067082307691, 0.23015920988881899, 0.18234716907032289, -0.50456272603383279]]], [[[0.0078729928569665208, 0.69731173748581221, 0.49970395768415377, 0.082420781284541222], [0.92931850420460083, -0.0061160337028260958, 0.76986905856890631, 0.36344781002822146], [0.76550152853294962, 0.56796799078432691, 0.30688505811574607, 0.17074010306728082]], [[0.59020456928225928, 0.24362230608362645, -0.32466678999008902, 0.30563086098209458], [-0.95962944437257724, -0.29191868772168217, -0.12704158563474799, -0.67874623436275505], [0.16919381995510241, -0.96468755364254899, -0.22719249912901507, -0.28389379495055422]]]]),self.functionspace)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.77522513090163669, -0.80635267202421312, 0.94233668188620756, -0.45658996926920992], [0.25112652901834354, 0.52640525584683284, 0.51104938355418983, 0.33201070386050224], [0.61637556743500754, 0.3042010867044147, -0.047839405067248153, -0.2640132345768671]], [[0.10264903345536336, -0.73683207607320234, 0.069246745195582937, -0.32287172580414225], [0.30995571522493481, -0.82084745507109091, -0.33981780426883557, 0.14491188736164284], [-0.71602311035524302, 0.2766751304240469, -0.86451180045316023, -0.84988304495821465]]], [[[0.31804733675909169, -0.81564972895920818, 0.96671061401348335, -0.70055538066741985], [0.053083457750787044, 0.91371825554084962, 0.38663635014232711, 0.00090513077608278536], [-0.54612064201970534, 0.68147787479274302, -0.77298036810153237, 0.83912530419823095]], [[0.44035593042649568, 0.81527025451484381, -0.64131114335863293, 0.51552585289312369], [0.89486573235115974, 0.76998778596631912, 0.11117369822532019, -0.97206597240432502], [0.15307161716384332, 0.87640722766535317, -0.19680515819738842, 0.4572939716025195]]], [[[0.41287051892656246, 0.63382278248957458, 0.22942170089067293, -0.51085554706807135], [0.72864808970530914, -0.19935178364826323, -0.15568291665146261, -0.485912841653515], [-0.81435667376152643, -0.015893633225117165, -0.70747754430419785, -0.91438960906100375]], [[0.49161162358807431, -0.37469690458832061, -0.36420300133574735, 0.48516242994294201], [-0.88510878690266148, -0.59765120583551856, -0.085076081801722436, 0.47172114387495845], [-0.10243328873293822, 0.71598263259232064, -0.34597076174440833, 0.50550262127338264]]]])+(1.-msk_arg1)*numpy.array([[[[-0.33593578300648619, -0.56242545797659882, -0.43259300047183369, 0.60975351860299476], [-0.02246403277780229, -0.34695041342876953, 0.21608419785673472, 0.71236574239944783], [-0.14760964716830305, 0.79084556506300907, -0.56753645222226501, 0.37044570460434123]], [[-0.15520476820418772, -0.89403655751177724, 0.43653625711027777, -0.50213469071532391], [-0.65169030098169367, -0.76540354918033926, 0.053850360299739553, 0.87753524596482224], [0.24770890750127994, 0.42745906433629433, -0.84900220123156545, 0.47162957032782726]]], [[[0.16370780834360921, -0.23123623323395481, -0.99531518184273637, 0.64911640690557815], [-0.16939552664705682, -0.44508487152560727, 0.93675404476766655, 0.70544654458716871], [-0.070725470702952986, 0.27145346226473599, 0.58243235416032579, 0.35080534465830149]], [[0.08882593105537917, 0.58501183479991914, 0.47653867211448286, 0.17035838850009011], [-0.12480704343941573, 0.7815321741368928, 0.5965737714975663, 0.23566973902026289], [-0.28987700695566376, -0.8017050606090701, 0.68223074082642809, -0.70544535100287553]]], [[[-0.77126412665483546, 0.022612276143459287, 0.90170123505951372, -0.87611945096268884], [0.13590453833353489, 0.90858638661704583, 0.64915192636583363, 0.1040140355630772], [-0.11166416716567751, -0.36494170175989593, -0.36908225517180071, -0.1520350621282216]], [[0.61894169405811095, -0.87237289574196764, 0.94300015667511072, -0.86980426118376197], [0.063922248082339816, 0.36973018852956674, 0.27399321809408206, -0.8269329208347509], [-0.19172428640293182, 0.20528097568175507, -0.7586224706371929, 0.24071100688645974]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.77522513090163669, -0.8185207403445125, -0.60916521073513796, -0.45658996926920992], [-0.85580002362796836, 0.46734043198688302, -0.67397330493613916, -0.52932217312961649], [0.5751081669961573, 0.052941628165801058, -0.047839405067248153, -0.2640132345768671]], [[0.10264903345536336, -0.73683207607320234, 0.069246745195582937, -0.76741350463770286], [-0.63252343652595422, -0.82084745507109091, -0.33981780426883557, 0.010608838986240876], [-0.71602311035524302, 0.064509699095955719, -0.86451180045316023, -0.84988304495821465]]], [[[0.31804733675909169, -0.81564972895920818, -0.35213406260979152, -0.70055538066741985], [0.053083457750787044, -0.12129927169254184, 0.38663635014232711, -0.34976968314726742], [-0.54612064201970534, 0.68147787479274302, -0.77298036810153237, -0.42312632509663017]], [[0.44035593042649568, 0.30748161382877059, -0.64131114335863293, 0.51552585289312369], [-0.65011341281663348, -0.087301283591635359, 0.11117369822532019, -0.97206597240432502], [0.15307161716384332, 0.23015920988881899, -0.19680515819738842, -0.50456272603383279]]], [[[0.0078729928569665208, 0.63382278248957458, 0.22942170089067293, -0.51085554706807135], [0.72864808970530914, -0.19935178364826323, -0.15568291665146261, -0.485912841653515], [-0.81435667376152643, -0.015893633225117165, -0.70747754430419785, -0.91438960906100375]], [[0.49161162358807431, -0.37469690458832061, -0.36420300133574735, 0.30563086098209458], [-0.95962944437257724, -0.59765120583551856, -0.12704158563474799, -0.67874623436275505], [-0.10243328873293822, -0.96468755364254899, -0.34597076174440833, -0.28389379495055422]]]])+(1.-msk_ref)*numpy.array([[[[-0.33593578300648619, -0.8185207403445125, -0.60916521073513796, 0.60975351860299476], [-0.85580002362796836, -0.34695041342876953, -0.67397330493613916, -0.52932217312961649], [-0.14760964716830305, 0.052941628165801058, -0.56753645222226501, 0.37044570460434123]], [[-0.15520476820418772, -0.89403655751177724, 0.43653625711027777, -0.76741350463770286], [-0.65169030098169367, -0.76540354918033926, 0.053850360299739553, 0.010608838986240876], [0.24770890750127994, 0.064509699095955719, -0.84900220123156545, -0.73368376961204418]]], [[[0.16370780834360921, -0.23123623323395481, -0.99531518184273637, 0.47414640994359991], [-0.16939552664705682, -0.44508487152560727, 0.66004288476383932, -0.34976968314726742], [-0.079196098179713292, 0.27145346226473599, -0.45381584972272249, -0.42312632509663017]], [[0.08882593105537917, 0.30748161382877059, -0.029464813833504078, 0.17035838850009011], [-0.65011341281663348, -0.087301283591635359, 0.5965737714975663, 0.23566973902026289], [-0.28987700695566376, -0.8017050606090701, 0.18234716907032289, -0.70544535100287553]]], [[[-0.77126412665483546, 0.022612276143459287, 0.49970395768415377, -0.87611945096268884], [0.13590453833353489, -0.0061160337028260958, 0.64915192636583363, 0.1040140355630772], [-0.11166416716567751, -0.36494170175989593, -0.36908225517180071, -0.1520350621282216]], [[0.59020456928225928, -0.87237289574196764, -0.32466678999008902, -0.86980426118376197], [-0.95962944437257724, -0.29191868772168217, -0.12704158563474799, -0.8269329208347509], [-0.19172428640293182, -0.96468755364254899, -0.7586224706371929, -0.28389379495055422]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.802185224287)+(1.-msk_arg0)*(-0.823984932181)
      arg1=0.702168831355
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.802185224287)+(1.-msk_ref)*(-0.823984932181)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.31499734364793808, -0.64546172566180404])+(1.-msk_arg0)*numpy.array([0.57336113182024961, 0.7643120118283262])
      arg1=0.594809736858
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.31499734364793808, -0.64546172566180404])+(1.-msk_ref)*numpy.array([0.57336113182024961, 0.59480973685824345])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.98383676749216908, 0.86440260904723765, 0.73344359288963279, -0.17397728468125617, -0.5938265296569154], [-0.58067394710115927, -0.44194706748457513, 0.27967035693140119, 0.074730365268826482, 0.69729235073669682], [-0.42381209475255077, -0.32932440466462043, 0.87791272277942922, 0.23848484063958675, -0.58763030313364073], [0.58014841638267201, 0.39037463608931611, 0.81425418742887401, -0.10613404447735553, 0.51799412509184828]])+(1.-msk_arg0)*numpy.array([[0.61907871601423992, -0.43702600031341965, 0.076002585444592263, -0.3435730095806635, -0.028739878240547023], [0.25304979868147393, 0.46903784589121167, 0.020380446238655203, -0.16308379559145014, 0.19405790849438431], [0.76223526305495692, 0.31307173153337819, 0.41301691085010406, -0.12908655648683509, -0.76242941180774193], [0.87655686472521355, 0.84110825321307225, -0.94080344664271731, -0.012273803170802733, 0.89276957902139253]])
      arg1=0.971999246082
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.97199924608175192, 0.86440260904723765, 0.73344359288963279, -0.17397728468125617, -0.5938265296569154], [-0.58067394710115927, -0.44194706748457513, 0.27967035693140119, 0.074730365268826482, 0.69729235073669682], [-0.42381209475255077, -0.32932440466462043, 0.87791272277942922, 0.23848484063958675, -0.58763030313364073], [0.58014841638267201, 0.39037463608931611, 0.81425418742887401, -0.10613404447735553, 0.51799412509184828]])+(1.-msk_ref)*numpy.array([[0.61907871601423992, -0.43702600031341965, 0.076002585444592263, -0.3435730095806635, -0.028739878240547023], [0.25304979868147393, 0.46903784589121167, 0.020380446238655203, -0.16308379559145014, 0.19405790849438431], [0.76223526305495692, 0.31307173153337819, 0.41301691085010406, -0.12908655648683509, -0.76242941180774193], [0.87655686472521355, 0.84110825321307225, -0.94080344664271731, -0.012273803170802733, 0.89276957902139253]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.35818675832618307, 0.66251667998983321], [0.44019858334827289, 0.80797363314062398]], [[-0.50875035066170571, 0.1850621473253049], [0.51643121442949136, 0.78957724033603993]], [[0.024803462659992004, 0.36090008797224926], [-0.40840720414424614, 0.33362856444192657]], [[-0.91448475681065045, 0.58046892641875081], [0.077155098664183575, 0.28778717742664117]], [[-0.0924686868337945, -0.00030014203367789705], [0.40742006235432759, -0.1675532175983081]], [[-0.44504569133159877, -0.54854390964934829], [0.6396789050300129, 0.59932475896449144]]])+(1.-msk_arg0)*numpy.array([[[0.41018693806518547, 0.27067191793728607], [-0.67161859940333368, 0.4779122230495445]], [[0.39418228244142606, -0.67720841536523468], [0.10786860697717726, -0.49729127516348415]], [[0.85375250311878537, -0.69445649759543193], [0.96360749355350905, 0.58162118701559451]], [[0.79545219381900667, -0.59196899563762573], [0.9399625831374443, -0.81064564899123104]], [[0.064606030286278093, -0.067628344496847514], [-0.25937635968343487, -0.31843859488575843]], [[-0.31806705339180064, -0.36476927543472448], [-0.85350703236145065, 0.96598062699500153]]])
      arg1=-0.105078713676
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.35818675832618307, -0.10507871367577559], [-0.10507871367577559, -0.10507871367577559]], [[-0.50875035066170571, -0.10507871367577559], [-0.10507871367577559, -0.10507871367577559]], [[-0.10507871367577559, -0.10507871367577559], [-0.40840720414424614, -0.10507871367577559]], [[-0.91448475681065045, -0.10507871367577559], [-0.10507871367577559, -0.10507871367577559]], [[-0.10507871367577559, -0.10507871367577559], [-0.10507871367577559, -0.1675532175983081]], [[-0.44504569133159877, -0.54854390964934829], [-0.10507871367577559, -0.10507871367577559]]])+(1.-msk_ref)*numpy.array([[[-0.10507871367577559, -0.10507871367577559], [-0.67161859940333368, -0.10507871367577559]], [[-0.10507871367577559, -0.67720841536523468], [-0.10507871367577559, -0.49729127516348415]], [[-0.10507871367577559, -0.69445649759543193], [-0.10507871367577559, -0.10507871367577559]], [[-0.10507871367577559, -0.59196899563762573], [-0.10507871367577559, -0.81064564899123104]], [[-0.10507871367577559, -0.10507871367577559], [-0.25937635968343487, -0.31843859488575843]], [[-0.31806705339180064, -0.36476927543472448], [-0.85350703236145065, -0.10507871367577559]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_float_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.1673905629670811, 0.4823761924849661, 0.26764137614442407, -0.60861245207012837], [0.81207772906574993, -0.19481378571026675, -0.68696933857396947, -0.83490575205488016], [0.91048162991973736, -0.011872095205975652, 0.3228228503728281, 0.9846389748649369]], [[0.47865443995711709, -0.052374822305633728, 0.55524952723542609, 0.89862533426637237], [0.25000787318830997, -0.83925180949611278, -0.57694886610954743, 0.22989642774423347], [-0.99098202123805956, -0.782526244818877, 0.76293059825092313, 0.25321959328143429]]], [[[-0.79558118032687841, -0.65200692367995861, -0.63808693821221318, -0.34957058264418084], [-0.92018607122064067, -0.8233939374948176, -0.33026462204352747, 0.4978099437902499], [0.18337802396535441, -0.20296321990869104, 0.72854075222480108, -0.59755758924975333]], [[0.51028788055695884, 0.10055515756334144, 0.058927234239805504, 0.34214029017102621], [0.20495462641135176, -0.22772555958250384, -0.87845202723777782, -0.97332774435546465], [0.82216539419002954, 0.55513989920065088, 0.14055973874252681, 0.16104095379698213]]], [[[-0.82240318552779113, 0.69191749483779263, 0.58090156909485091, -0.53896089228036015], [0.62423861157834315, -0.99300034048831809, -0.47207283729237037, -0.5513640483443949], [0.17882096863182828, 0.71019678377287399, -0.064032963761873063, -0.85638868871581564]], [[-0.25414488117093148, -0.37884680134470416, 0.84221144197438447, -0.34840820199457445], [0.85176550954687125, 0.79796351752116323, 0.29891957756077292, -0.28400221389535885], [0.5317500607729484, 0.13516570259352556, -0.41149304720465829, -0.8735268753644041]]]])+(1.-msk_arg0)*numpy.array([[[[0.0065215397792657281, 0.12834491205802556, -0.015697763382179764, -0.25915206650850653], [-0.71979894074050899, -0.92007415267088888, -0.12122704948084029, -0.19160146136619138], [-0.93957001515002592, -0.55520004603224482, -0.99293749286636968, -0.086066806389853756]], [[0.16051813794273717, 0.43096892110838736, 0.81330510865915295, 0.55946134760795507], [0.79197637782615571, 0.70474500001109952, -0.46497326823377239, -0.68931545158187846], [-0.43158350324336059, -0.50574491215569806, 0.23216889455744449, -0.65669272586291272]]], [[[-0.95311899689549695, 0.48656523140642793, -0.51145914130788617, -0.97835361498153417], [-0.76029447855376331, 0.8491047318765681, -0.63012145578386503, -0.48488780240026674], [0.48001806190712526, 0.85277368570324508, -0.5591782811307755, -0.70829693233712288]], [[-0.1435407512454181, 0.23197093520841183, 0.62351793318353677, -0.26581074459525], [0.049100018055762362, -0.59491369065388855, 0.91424634055756115, -0.54961406544295288], [0.35654242968961802, 0.45509805445077878, -0.55366692200073819, -0.86384046944165038]]], [[[-0.40639958646080232, -0.51600064003387924, 0.14685762017526827, 0.60425898325870597], [0.15549336728605834, 0.61482215207579549, -0.08570713597168611, -0.55072667475672521], [-0.20776110369317746, 0.61201617495459937, 0.0016843880876091788, 0.83913217927551509]], [[0.024554926743849403, 0.79386330058413157, -0.50359963472303293, 0.61453042322610285], [0.74107463400371909, 0.49356232454200022, -0.83323707925421742, -0.85429233597972942], [-0.029915917258621638, -0.46919535249699251, -0.77590589476588923, 0.35658003104386915]]]])
      arg1=0.225526822619
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.1673905629670811, 0.22552682261871149, 0.22552682261871149, -0.60861245207012837], [0.22552682261871149, -0.19481378571026675, -0.68696933857396947, -0.83490575205488016], [0.22552682261871149, -0.011872095205975652, 0.22552682261871149, 0.22552682261871149]], [[0.22552682261871149, -0.052374822305633728, 0.22552682261871149, 0.22552682261871149], [0.22552682261871149, -0.83925180949611278, -0.57694886610954743, 0.22552682261871149], [-0.99098202123805956, -0.782526244818877, 0.22552682261871149, 0.22552682261871149]]], [[[-0.79558118032687841, -0.65200692367995861, -0.63808693821221318, -0.34957058264418084], [-0.92018607122064067, -0.8233939374948176, -0.33026462204352747, 0.22552682261871149], [0.18337802396535441, -0.20296321990869104, 0.22552682261871149, -0.59755758924975333]], [[0.22552682261871149, 0.10055515756334144, 0.058927234239805504, 0.22552682261871149], [0.20495462641135176, -0.22772555958250384, -0.87845202723777782, -0.97332774435546465], [0.22552682261871149, 0.22552682261871149, 0.14055973874252681, 0.16104095379698213]]], [[[-0.82240318552779113, 0.22552682261871149, 0.22552682261871149, -0.53896089228036015], [0.22552682261871149, -0.99300034048831809, -0.47207283729237037, -0.5513640483443949], [0.17882096863182828, 0.22552682261871149, -0.064032963761873063, -0.85638868871581564]], [[-0.25414488117093148, -0.37884680134470416, 0.22552682261871149, -0.34840820199457445], [0.22552682261871149, 0.22552682261871149, 0.22552682261871149, -0.28400221389535885], [0.22552682261871149, 0.13516570259352556, -0.41149304720465829, -0.8735268753644041]]]])+(1.-msk_ref)*numpy.array([[[[0.0065215397792657281, 0.12834491205802556, -0.015697763382179764, -0.25915206650850653], [-0.71979894074050899, -0.92007415267088888, -0.12122704948084029, -0.19160146136619138], [-0.93957001515002592, -0.55520004603224482, -0.99293749286636968, -0.086066806389853756]], [[0.16051813794273717, 0.22552682261871149, 0.22552682261871149, 0.22552682261871149], [0.22552682261871149, 0.22552682261871149, -0.46497326823377239, -0.68931545158187846], [-0.43158350324336059, -0.50574491215569806, 0.22552682261871149, -0.65669272586291272]]], [[[-0.95311899689549695, 0.22552682261871149, -0.51145914130788617, -0.97835361498153417], [-0.76029447855376331, 0.22552682261871149, -0.63012145578386503, -0.48488780240026674], [0.22552682261871149, 0.22552682261871149, -0.5591782811307755, -0.70829693233712288]], [[-0.1435407512454181, 0.22552682261871149, 0.22552682261871149, -0.26581074459525], [0.049100018055762362, -0.59491369065388855, 0.22552682261871149, -0.54961406544295288], [0.22552682261871149, 0.22552682261871149, -0.55366692200073819, -0.86384046944165038]]], [[[-0.40639958646080232, -0.51600064003387924, 0.14685762017526827, 0.22552682261871149], [0.15549336728605834, 0.22552682261871149, -0.08570713597168611, -0.55072667475672521], [-0.20776110369317746, 0.22552682261871149, 0.0016843880876091788, 0.22552682261871149]], [[0.024554926743849403, 0.22552682261871149, -0.50359963472303293, 0.22552682261871149], [0.22552682261871149, 0.22552682261871149, -0.83323707925421742, -0.85429233597972942], [-0.029915917258621638, -0.46919535249699251, -0.77590589476588923, 0.22552682261871149]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.462133770027)+(1.-msk_arg0)*(0.99788331378)
      arg1=numpy.array(0.908337733011)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(0.462133770027)+(1.-msk_ref)*(0.908337733011)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.5733632905595285, 0.6185641476203565])+(1.-msk_arg0)*numpy.array([-0.57891950666832592, -0.94424966124896148])
      arg1=numpy.array(0.239828052473)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.5733632905595285, 0.23982805247344641])+(1.-msk_ref)*numpy.array([-0.57891950666832592, -0.94424966124896148])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.47489425311934097, 0.48485384309032153, -0.41298654790185441, 0.61628910965045813, -0.051589397258991809], [-0.041217574062325291, -0.50403021400870363, -0.51648306146470246, 0.65144771240193, 0.13772917467182766], [-0.63237436710276507, -0.36065888688746162, 0.99022504045723836, 0.3420571281089948, 0.44702312852105663], [0.93857189128770857, 0.92228180949743788, 0.61695947141749552, 0.068784573992198617, -0.17921548948849697]])+(1.-msk_arg0)*numpy.array([[0.39940690921861766, 0.95037421923313525, -0.50038788053702765, 0.83968383150250459, -0.053875737417968805], [0.59451864194975013, 0.10567824093127354, -0.30821210325749182, -0.79172745325260951, -0.73086269168476448], [-0.3164850272328712, -0.16817246386270313, 0.70363824093253502, 0.70929526377113761, -0.44059322736590101], [0.23117511818942793, 0.94921332798368763, 0.22627184611548135, -0.24690800725411743, 0.186957271620938]])
      arg1=numpy.array(-0.949988230451)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338]])+(1.-msk_ref)*numpy.array([[-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338], [-0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338, -0.94998823045081338]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.73794431904656754, 0.82355104070281926], [-0.50256159560105451, 0.79946678939692362]], [[0.59921594903633513, 0.34868340968016698], [0.78046383177587297, 0.31486385139022754]], [[-0.7791322226161963, -0.69843267401804199], [0.92008699903449398, -0.055041092543384407]], [[0.96946957784223708, 0.25591085275570125], [-0.13557629412674466, -0.67420645331323414]], [[0.0071872882500698676, -0.62002453139022129], [0.40690757676771372, -0.87014099230175734]], [[-0.040965080772459306, -0.46099765218654332], [-0.89900694430032946, -0.48472600774799046]]])+(1.-msk_arg0)*numpy.array([[[0.80833301324171236, 0.2674952859099724], [0.71850258587004778, 0.41103346168326471]], [[0.58859616213957899, 0.10197618336054082], [0.8435722774885317, 0.73753382196389694]], [[0.72709746124741192, -0.86283974734421531], [0.64353494729470673, -0.29573499097342193]], [[0.46098346490248554, 0.64150182208622963], [0.012077818122483208, -0.14382055035546171]], [[0.61343195695553732, -0.69238020541238821], [0.36434239110549527, 0.15131686557122093]], [[-0.87222563567199884, 0.16839624403313125], [0.68506532285248589, 0.17364984720840893]]])
      arg1=numpy.array(-0.706034523686)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.73794431904656754, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.7791322226161963, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.87014099230175734]], [[-0.70603452368611097, -0.70603452368611097], [-0.89900694430032946, -0.70603452368611097]]])+(1.-msk_ref)*numpy.array([[[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.86283974734421531], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.70603452368611097, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]], [[-0.87222563567199884, -0.70603452368611097], [-0.70603452368611097, -0.70603452368611097]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_array_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.003572883152234585, -0.83248851627990117, -0.90059434128670612, -0.5545936894966883], [0.18266188687930596, -0.67278350237110263, -0.2259850606001792, 0.64561879250800192], [-0.03375412750509299, 0.11592092713001323, -0.54064533415290694, -0.38526693665320977]], [[-0.82574461299081436, 0.6573736905587424, -0.0068627178436753145, -0.13869449105647647], [0.67225844133198986, 0.34593382971626685, 0.27340450146826267, 0.85160802149470505], [-0.66999148400103103, -0.73276437740140854, 0.63553345158765873, 0.4596323856921023]]], [[[0.075990498475986934, 0.55235878286543105, -0.72626608553487371, 0.43754421080024053], [-0.043789935840196081, -0.89420819788538308, 0.0081478352868700199, -0.10555379301806322], [0.054254094230939742, 0.56992485311953622, 0.50341369164178551, 0.043647549383019513]], [[-0.77506453965638866, 0.83308193461012325, 0.84523474681226451, 0.26073073370767563], [-0.98297600612437219, 0.43720681360806002, -0.41707679842475742, 0.5982142439324114], [-0.36040280392867774, -0.96719255289859274, 0.8904467550533588, 0.84756694743012551]]], [[[-0.64937319147417805, 0.58831554830922328, -0.35634380136295274, -0.93930859763560992], [-0.68526847870442387, -0.29528295261473736, -0.4018362720052826, 0.51449899473279914], [-0.9621007507783661, 0.38641494717541702, -0.36988826598461011, 0.78887945477223465]], [[0.65362213702130934, -0.43446121264100857, -0.32276035405760539, 0.20061429545458442], [-0.033249637634963802, 0.18143851012971979, -0.42348143688431228, -0.28782314589036329], [-0.38241249258873178, -0.14806811190300029, 0.51438624882026995, -0.60230580855814098]]]])+(1.-msk_arg0)*numpy.array([[[[0.616531518223294, -0.88751393989696181, -0.034786720749853117, 0.41864688199010436], [0.64913739343864263, -0.8394089948517216, 0.60394421876557081, 0.99308681736162474], [-0.98479628762575788, -0.58230884762427926, -0.15846222737518278, -0.59007729262231923]], [[-0.26546171374352978, 0.6402636132864834, 0.086209503624699035, 0.018751916551482228], [-0.78597083872371343, 0.57140347860684781, 0.085834016205685204, -0.7222132440798763], [0.22271838932504751, -0.072533664224724514, 0.041360865451035567, 0.32854489275958554]]], [[[0.43862328123174721, -0.67606573245299528, 0.70186633219862271, -0.017264169432553045], [-0.56333048313503276, 0.71392662651080996, -0.68201580778163073, -0.43008865680379227], [0.75828789750863845, 0.66146308220586469, -0.43210049831356745, 0.72595395947562369]], [[0.45894812979278532, 0.49822692657848378, 0.45001204304095488, 0.6088261732425917], [0.2596755550790224, -0.6512896582296519, -0.014857341412102487, 0.58058436759554821], [-0.057701880520455218, -0.82045427404409166, -0.93527611591253845, 0.46524710162916016]]], [[[0.999161503495551, 0.90016356716091117, -0.20055873229761856, 0.93853783232131449], [-0.32761194245027903, -0.72191519213242406, 0.20971406207416754, -0.83636736103877651], [-0.45359327429169061, 0.3734719973424343, 0.92418407152812487, -0.29321828753943602]], [[-0.36734823413473694, 0.41689911801280255, 0.18137743926892891, -0.21532480896170347], [0.73752810368759891, -0.52161649336255822, 0.5062463901789771, -0.8865676309480528], [0.99751166299242722, 0.20175756810895229, 0.11999531795492135, -0.030929223769755376]]]])
      arg1=numpy.array(-0.279029648621)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.27902964862139656, -0.83248851627990117, -0.90059434128670612, -0.5545936894966883], [-0.27902964862139656, -0.67278350237110263, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.27902964862139656, -0.54064533415290694, -0.38526693665320977]], [[-0.82574461299081436, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.66999148400103103, -0.73276437740140854, -0.27902964862139656, -0.27902964862139656]]], [[[-0.27902964862139656, -0.27902964862139656, -0.72626608553487371, -0.27902964862139656], [-0.27902964862139656, -0.89420819788538308, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656]], [[-0.77506453965638866, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.98297600612437219, -0.27902964862139656, -0.41707679842475742, -0.27902964862139656], [-0.36040280392867774, -0.96719255289859274, -0.27902964862139656, -0.27902964862139656]]], [[[-0.64937319147417805, -0.27902964862139656, -0.35634380136295274, -0.93930859763560992], [-0.68526847870442387, -0.29528295261473736, -0.4018362720052826, -0.27902964862139656], [-0.9621007507783661, -0.27902964862139656, -0.36988826598461011, -0.27902964862139656]], [[-0.27902964862139656, -0.43446121264100857, -0.32276035405760539, -0.27902964862139656], [-0.27902964862139656, -0.27902964862139656, -0.42348143688431228, -0.28782314589036329], [-0.38241249258873178, -0.27902964862139656, -0.27902964862139656, -0.60230580855814098]]]])+(1.-msk_ref)*numpy.array([[[[-0.27902964862139656, -0.88751393989696181, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.8394089948517216, -0.27902964862139656, -0.27902964862139656], [-0.98479628762575788, -0.58230884762427926, -0.27902964862139656, -0.59007729262231923]], [[-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.78597083872371343, -0.27902964862139656, -0.27902964862139656, -0.7222132440798763], [-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656]]], [[[-0.27902964862139656, -0.67606573245299528, -0.27902964862139656, -0.27902964862139656], [-0.56333048313503276, -0.27902964862139656, -0.68201580778163073, -0.43008865680379227], [-0.27902964862139656, -0.27902964862139656, -0.43210049831356745, -0.27902964862139656]], [[-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.6512896582296519, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.82045427404409166, -0.93527611591253845, -0.27902964862139656]]], [[[-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.32761194245027903, -0.72191519213242406, -0.27902964862139656, -0.83636736103877651], [-0.45359327429169061, -0.27902964862139656, -0.27902964862139656, -0.29321828753943602]], [[-0.36734823413473694, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656], [-0.27902964862139656, -0.52161649336255822, -0.27902964862139656, -0.8865676309480528], [-0.27902964862139656, -0.27902964862139656, -0.27902964862139656, -0.27902964862139656]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.464503999428)+(1.-msk_arg0)*(-0.301387163516)
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.766070273853)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.464503999428)+(1.-msk_ref)*(-0.301387163516)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.80903563869513762, 0.0091520499531714261])+(1.-msk_arg0)*numpy.array([-0.78217910923348621, 0.58741291418538633])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.587405001344)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.58740500134434681, 0.0091520499531714261])+(1.-msk_ref)*numpy.array([-0.78217910923348621, 0.58740500134434681])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.9269347882409027, -0.19538549178858533, -0.13919359797025921, 0.77233394566993585, -0.075171294241806352], [-0.89734773018794778, 0.049295542075726395, 0.74841793385927136, -0.21345291958553436, -0.75687230107319992], [-0.64330341780597533, 0.09799933500085678, -0.27217028913747954, 0.96173234744090541, 0.27222187779384499], [-0.8390698132447223, -0.13235036751328821, 0.68497031165535893, 0.6975148405360847, -0.78213671344025526]])+(1.-msk_arg0)*numpy.array([[-0.15358623180906905, -0.43005149074565341, 0.2247048122698998, -0.66290882187063471, -0.48453554133010601], [-0.096833608837384233, 0.71119760015660716, -0.64154537675575996, 0.018398441835853374, -0.66943187577305041], [0.33256159724417311, 0.46056275610099551, -0.26194917115081084, -0.24918380170351218, 0.50002345452734898], [-0.48138013102149113, 0.72238055051609495, 0.81803470813958423, -0.80117391759568601, -0.83412401173248196]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.0941877697202)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.9269347882409027, -0.19538549178858533, -0.13919359797025921, 0.094187769720183168, -0.075171294241806352], [-0.89734773018794778, 0.049295542075726395, 0.094187769720183168, -0.21345291958553436, -0.75687230107319992], [-0.64330341780597533, 0.094187769720183168, -0.27217028913747954, 0.094187769720183168, 0.094187769720183168], [-0.8390698132447223, -0.13235036751328821, 0.094187769720183168, 0.094187769720183168, -0.78213671344025526]])+(1.-msk_ref)*numpy.array([[-0.15358623180906905, -0.43005149074565341, 0.094187769720183168, -0.66290882187063471, -0.48453554133010601], [-0.096833608837384233, 0.094187769720183168, -0.64154537675575996, 0.018398441835853374, -0.66943187577305041], [0.094187769720183168, 0.094187769720183168, -0.26194917115081084, -0.24918380170351218, 0.094187769720183168], [-0.48138013102149113, 0.094187769720183168, 0.094187769720183168, -0.80117391759568601, -0.83412401173248196]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.20049443744332951, -0.46441399548117746], [0.013656240883040738, -0.98251065335079191]], [[-0.66451037110371192, 0.331849773665295], [0.7768479346731112, -0.92370765096659335]], [[-0.56648091702302983, -0.34305649733566512], [0.37992807031152731, 0.77640142050816352]], [[-0.62301251327821805, 0.36944721163419025], [0.55327419391728005, 0.081666578517416877]], [[-0.6989086464993346, -0.30016051318200399], [0.12652551706991999, -0.16375044253049054]], [[0.81063186905735396, -0.64950401432853644], [-0.63378104054700635, -0.23509524811118476]]])+(1.-msk_arg0)*numpy.array([[[0.89210144755925858, -0.9150803174635751], [0.346681249750735, -0.046585865339404009]], [[-0.42663188715513001, -0.20162958114203744], [-0.43738867003469317, 0.035233145378010811]], [[0.90452813540187216, -0.19987191130631121], [0.4435082682668896, 0.64122226380008174]], [[-0.68755877232233642, 0.84316084321287432], [-0.020093182977176882, -0.28814303461665314]], [[-0.078358076680761224, -0.35385765156307403], [-0.96971904443666634, 0.65972858388346234]], [[-0.78287511391799391, 0.30683082303880327], [0.42990174476743626, -0.536201036004059]]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(0.260804688284)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.20049443744332951, -0.46441399548117746], [0.013656240883040738, -0.98251065335079191]], [[-0.66451037110371192, 0.26080468828381198], [0.26080468828381198, -0.92370765096659335]], [[-0.56648091702302983, -0.34305649733566512], [0.26080468828381198, 0.26080468828381198]], [[-0.62301251327821805, 0.26080468828381198], [0.26080468828381198, 0.081666578517416877]], [[-0.6989086464993346, -0.30016051318200399], [0.12652551706991999, -0.16375044253049054]], [[0.26080468828381198, -0.64950401432853644], [-0.63378104054700635, -0.23509524811118476]]])+(1.-msk_ref)*numpy.array([[[0.26080468828381198, -0.9150803174635751], [0.26080468828381198, -0.046585865339404009]], [[-0.42663188715513001, -0.20162958114203744], [-0.43738867003469317, 0.035233145378010811]], [[0.26080468828381198, -0.19987191130631121], [0.26080468828381198, 0.26080468828381198]], [[-0.68755877232233642, 0.26080468828381198], [-0.020093182977176882, -0.28814303461665314]], [[-0.078358076680761224, -0.35385765156307403], [-0.96971904443666634, 0.26080468828381198]], [[-0.78287511391799391, 0.26080468828381198], [0.26080468828381198, -0.536201036004059]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_Symbol_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.83683260642815305, 0.88555629059408481, -0.43129030730203199, 0.36869268409932232], [0.79676981334775632, 0.49049832075929456, 0.68166994203145115, -0.43720876830422362], [-0.83939225967065267, 0.72045713364835873, 0.27850921139011975, 0.62375287761035048]], [[-0.38386831573194025, -0.83486292448383081, 0.8614958815260374, -0.42261869554728571], [0.094141384917348248, 0.40515693029884225, -0.13466413015654499, -0.16890798762973214], [0.38279410670965519, 0.2206863364941567, 0.55818876613609913, -0.13701826760514924]]], [[[-0.28812636970205974, 0.47062498032102829, -0.90295378665134729, -0.79453759342064223], [0.0086586586196699677, 0.2908221401535156, -0.92269828778117957, 0.50355535760715342], [0.5332385024756785, 0.43840702634398787, 0.87659781203428699, 0.053975743440553225]], [[-0.085519656043533621, 0.79744368664803789, 0.95236104534837662, -0.11254672171837399], [-0.17755678902613781, 0.45513031451093333, 0.29107417100211364, 0.15949745864721421], [0.48687791162595628, -0.028598327197644213, 0.97820290197771631, 0.048321324382919473]]], [[[-0.84181935475915481, -0.29256940254504382, 0.064969879604321656, 0.2580001983349145], [-0.26208462920420983, -0.1875760318203552, -0.41318481368818416, -0.37106205354116595], [0.0067900165741914265, 0.83615819494144694, 0.22964684342589803, 0.29215662345288496]], [[-0.1182788433919999, -0.72992344113813967, -0.47368052279964079, -0.71319722267262198], [-0.35527861810436123, -0.44522413459062737, 0.42625564388227555, -0.95543212174322067], [-0.22304843697119292, -0.31204169804826742, 0.39307613383462181, -0.9757230429712902]]]])+(1.-msk_arg0)*numpy.array([[[[-0.64663236767076815, 0.95991464621704159, -0.70104691378501394, -0.92282708395961133], [0.09713421082105711, 0.53425490471230286, 0.77322041872670577, 0.86293428848230502], [0.50030625190145162, -0.47518901092111565, 0.013239611106141913, 0.91924807363620031]], [[0.24304263128689363, 0.26786204936914282, -0.91279738818009482, -0.020468370256350399], [0.86190418181134087, -0.99502860422099837, -0.030031763343616458, -0.21081191509741304], [0.059327023366198661, 0.65324376323541156, 0.17390350413745215, -0.36713925129607961]]], [[[0.17158434026975855, -0.19327634665047166, 0.32386926224180823, 0.88851269780683273], [0.62825020286853794, 0.33122620756673404, -0.88344534235428895, 0.98925546836162104], [-0.94799801810694895, 0.65737518155752994, 0.16117809764797109, -0.42880688469323203]], [[0.089997594501926903, 0.38393825794048064, -0.95701858088764769, -0.84092437110970497], [-0.020791050130631428, 0.65074577665374589, -0.72879666138800725, -0.74851767174102002], [-0.30587455655307472, -0.58294308612344747, 0.80948012751436016, 0.9168362542354207]]], [[[-0.52944762849197935, 0.27408771629566919, 0.76521073963216546, 0.83050944470128285], [0.24674013444477505, 0.52213030368824742, 0.097966671637116143, -0.80546812665923007], [0.20746242905553092, 0.53768718800286264, -0.36238802628050726, 0.94790145004268589]], [[0.067910897322539299, -0.48790665361198293, 0.86035867328108262, 0.20414019742515466], [-0.77010340655818221, 0.68957994368684528, 0.070862669621646335, 0.61588573513969114], [-0.22445338531382375, -0.088109229698033031, -0.15534367376287328, 0.59706372275977038]]]])
      arg1=Symbol(shape=())
      res=minimum(arg0,arg1)
      s1=numpy.array(-0.395311299465)
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.39531129946463328, -0.39531129946463328, -0.43129030730203199, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.43720876830422362], [-0.83939225967065267, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]], [[-0.39531129946463328, -0.83486292448383081, -0.39531129946463328, -0.42261869554728571], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]]], [[[-0.39531129946463328, -0.39531129946463328, -0.90295378665134729, -0.79453759342064223], [-0.39531129946463328, -0.39531129946463328, -0.92269828778117957, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]], [[-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]]], [[[-0.84181935475915481, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.41318481368818416, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]], [[-0.39531129946463328, -0.72992344113813967, -0.47368052279964079, -0.71319722267262198], [-0.39531129946463328, -0.44522413459062737, -0.39531129946463328, -0.95543212174322067], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.9757230429712902]]]])+(1.-msk_ref)*numpy.array([[[[-0.64663236767076815, -0.39531129946463328, -0.70104691378501394, -0.92282708395961133], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.47518901092111565, -0.39531129946463328, -0.39531129946463328]], [[-0.39531129946463328, -0.39531129946463328, -0.91279738818009482, -0.39531129946463328], [-0.39531129946463328, -0.99502860422099837, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]]], [[[-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.88344534235428895, -0.39531129946463328], [-0.94799801810694895, -0.39531129946463328, -0.39531129946463328, -0.42880688469323203]], [[-0.39531129946463328, -0.39531129946463328, -0.95701858088764769, -0.84092437110970497], [-0.39531129946463328, -0.39531129946463328, -0.72879666138800725, -0.74851767174102002], [-0.39531129946463328, -0.58294308612344747, -0.39531129946463328, -0.39531129946463328]]], [[[-0.52944762849197935, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.80546812665923007], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]], [[-0.39531129946463328, -0.48790665361198293, -0.39531129946463328, -0.39531129946463328], [-0.77010340655818221, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328], [-0.39531129946463328, -0.39531129946463328, -0.39531129946463328, -0.39531129946463328]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.52005123366)+(1.-msk_arg0)*(-0.958999719052)
      arg1=Data(-0.0708731017333,self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.0708731017333)+(1.-msk_ref)*(-0.958999719052)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.034425264741442252, -0.25794327443852638])+(1.-msk_arg0)*numpy.array([0.55328368359564517, -0.7545462490824173])
      arg1=Data(-0.480407268311,self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.48040726831111624, -0.48040726831111624])+(1.-msk_ref)*numpy.array([-0.48040726831111624, -0.7545462490824173])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.27849143638905716, -0.81816127855938081, 0.69606228158862371, 0.088242144542835854, 0.72827996193193223], [0.22446314701273384, 0.52486458845088135, -0.56328987644667716, 0.96401440200654265, -0.45785103334420252], [-0.63305172879430938, 0.35865428691855961, -0.569393829831361, -0.12161223217288253, -0.61491458388035025], [-0.79391210001212853, 0.51704887878501915, 0.88799473490616476, 0.31718933752579814, 0.33400216210835976]])+(1.-msk_arg0)*numpy.array([[0.88709645024753669, 0.40173931175005051, -0.11125074388760958, 0.22067298812180125, -0.14045418809412369], [0.49181919797271267, 0.20790408021387274, 0.48665659642663606, -0.73508616755061795, 0.48899223794148439], [0.096462720018308801, 0.99484725712415867, 0.5212821515664845, 0.66472970224333516, 0.65483542392463612], [-0.68749624980452695, 0.046250707599986507, 0.30102899693285412, -0.74255558225134988, -0.24410315816412154]])
      arg1=Data(0.882879392343,self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.27849143638905716, -0.81816127855938081, 0.69606228158862371, 0.088242144542835854, 0.72827996193193223], [0.22446314701273384, 0.52486458845088135, -0.56328987644667716, 0.8828793923426248, -0.45785103334420252], [-0.63305172879430938, 0.35865428691855961, -0.569393829831361, -0.12161223217288253, -0.61491458388035025], [-0.79391210001212853, 0.51704887878501915, 0.8828793923426248, 0.31718933752579814, 0.33400216210835976]])+(1.-msk_ref)*numpy.array([[0.8828793923426248, 0.40173931175005051, -0.11125074388760958, 0.22067298812180125, -0.14045418809412369], [0.49181919797271267, 0.20790408021387274, 0.48665659642663606, -0.73508616755061795, 0.48899223794148439], [0.096462720018308801, 0.8828793923426248, 0.5212821515664845, 0.66472970224333516, 0.65483542392463612], [-0.68749624980452695, 0.046250707599986507, 0.30102899693285412, -0.74255558225134988, -0.24410315816412154]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.1103051566256672, -0.19648336038552183], [0.14839247071435313, -0.40700556791885134]], [[-0.71066715925814417, -0.83992804777778951], [-0.29459995067111877, -0.81474982136381691]], [[-0.32120619159672992, 0.17237728295360344], [-0.99051659837636619, -0.62127719001414072]], [[0.95449221429340092, 0.82360801918298709], [0.5318330818379351, -0.70200136085562281]], [[-0.79497949850504268, -0.49818711248341052], [0.034872855252941148, 0.11110350266570124]], [[-0.22756949133965776, -0.38114513438097375], [0.2265504559910787, 0.50417573362442281]]])+(1.-msk_arg0)*numpy.array([[[-0.94514200940537618, -0.77537348277307783], [0.63636770768104811, 0.022128213807887764]], [[0.62061270420486903, -0.31049755146640301], [-0.8668020566821506, 0.075675575276644702]], [[0.53418336467311667, -0.90449984407341888], [0.26414066251050228, 0.4789955701240205]], [[-0.79364470705171386, -0.89799383156450374], [-0.73319936648412321, -0.62470096724911706]], [[-0.6611890645524221, 0.0015673250623819968], [0.23300764861253742, 0.49118920845054381]], [[0.77418383504698474, 0.85778927929979853], [-0.23024965361662053, -0.13327488849319868]]])
      arg1=Data(0.60832751022,self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.1103051566256672, -0.19648336038552183], [0.14839247071435313, -0.40700556791885134]], [[-0.71066715925814417, -0.83992804777778951], [-0.29459995067111877, -0.81474982136381691]], [[-0.32120619159672992, 0.17237728295360344], [-0.99051659837636619, -0.62127719001414072]], [[0.60832751021962284, 0.60832751021962284], [0.5318330818379351, -0.70200136085562281]], [[-0.79497949850504268, -0.49818711248341052], [0.034872855252941148, 0.11110350266570124]], [[-0.22756949133965776, -0.38114513438097375], [0.2265504559910787, 0.50417573362442281]]])+(1.-msk_ref)*numpy.array([[[-0.94514200940537618, -0.77537348277307783], [0.60832751021962284, 0.022128213807887764]], [[0.60832751021962284, -0.31049755146640301], [-0.8668020566821506, 0.075675575276644702]], [[0.53418336467311667, -0.90449984407341888], [0.26414066251050228, 0.4789955701240205]], [[-0.79364470705171386, -0.89799383156450374], [-0.73319936648412321, -0.62470096724911706]], [[-0.6611890645524221, 0.0015673250623819968], [0.23300764861253742, 0.49118920845054381]], [[0.60832751021962284, 0.60832751021962284], [-0.23024965361662053, -0.13327488849319868]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_constData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.076054737410755635, -0.72933991048518987, -0.30374092730161983, -0.96623484277144489], [0.26241998633761421, -0.15173597932636973, -0.13766545284134324, 0.71621334182239549], [-0.35659865699692372, 0.67632274098233558, -0.86123445121824838, 0.66935017428129973]], [[-0.23799031523212033, 0.63264528280808618, 0.017178832261615318, -0.048992885430059196], [0.26189284562651527, -0.098324061946928332, -0.84769621042425403, 0.95598541040480378], [-0.5666491522221766, -0.91220870779368557, 0.044186763535234652, -0.22773955761163256]]], [[[-0.81391008905988627, 0.94134815974881603, -0.71181797425869231, 0.58931705578025761], [0.27434744450909032, 0.28236916304783, 0.91023791064138959, 0.68979277440267328], [-0.720953929625195, 0.82242068121848089, 0.27635478930198754, -0.95091791893578104]], [[-0.25203499022073439, -0.68568413257267435, -0.29924710780745079, 0.69965309436670231], [-0.53071819612057314, -0.039075518371336493, -0.78639467261753437, -0.92015810457855163], [-0.42005878303025823, 0.81364725175999397, -0.51827954915638941, -0.74688756593230687]]], [[[-0.19357706524175278, -0.50241444585477191, -0.56723586186680675, -0.83952868893443555], [-0.54123827126039425, 0.41236143271136139, -0.78452155659763756, -0.84584424443890915], [0.028483192381424383, 0.27603692837961313, -0.21670730721198272, 0.61097851846601636]], [[0.89295301820646777, 0.16387018597327541, -0.24406195263139097, -0.80560101508762272], [0.078848859642367763, 0.0024368363763078715, -0.7783690225546902, -0.50753078234011828], [0.67282794405846125, 0.62648284872330318, -0.11026758119417712, 0.92501906189262328]]]])+(1.-msk_arg0)*numpy.array([[[[-0.07611528984507876, -0.47812530896237626, -0.25942623749172178, 0.55493254138929649], [-0.22946271939066754, 0.73840875153960073, -0.040983059976963832, 0.17588271052000692], [-0.65716349160644816, -0.59828010440641055, 0.4403478331348325, 0.77972835168314947]], [[0.75921663312389209, -0.2784580521221387, -0.74420894259584824, 0.66040845029143846], [0.51546181862148099, 0.63248490527752654, -0.61848381965575561, 0.41091980869937217], [0.97079564136341667, -0.59000592327596468, -0.27591995603626063, -0.51995789329755993]]], [[[-0.65043240731215168, -0.75853074956668132, 0.53056801836881573, -0.025730918022197624], [0.29260532688272711, 0.68060539297868883, -0.59529254688050703, -0.29787661868788451], [0.51479365939487498, 0.37299502583565203, -0.8838683779838945, -0.056242853801407922]], [[0.97478509415738723, -0.6088450671522132, -0.3576635906904837, -0.41783175536232986], [-0.96931154248893847, -0.10227178975707663, -0.65210198192227931, 0.36335439111687462], [-0.77482950956053132, 0.2204388099696537, -0.067043895866304659, -0.6424907017287933]]], [[[0.62323183107976821, 0.88333740893801105, -0.78349398705810236, 0.4997741844267396], [0.64250440974075218, -0.4126927948016792, 0.40236419396109602, 0.82279414179119925], [-0.43795775664127023, -0.44903791714375796, -0.45277914810155551, 0.92971899356307874]], [[-0.38092442486833433, 0.0091969461730894153, 0.059546651251002336, 0.51321128033471131], [-0.64020797134164131, -0.54586469622389844, -0.82586172506752953, 0.095536359033647766], [-0.39064177768555197, -0.74606450260273638, 0.72964185657906278, 0.28114058336683456]]]])
      arg1=Data(0.971581327027,self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.076054737410755635, -0.72933991048518987, -0.30374092730161983, -0.96623484277144489], [0.26241998633761421, -0.15173597932636973, -0.13766545284134324, 0.71621334182239549], [-0.35659865699692372, 0.67632274098233558, -0.86123445121824838, 0.66935017428129973]], [[-0.23799031523212033, 0.63264528280808618, 0.017178832261615318, -0.048992885430059196], [0.26189284562651527, -0.098324061946928332, -0.84769621042425403, 0.95598541040480378], [-0.5666491522221766, -0.91220870779368557, 0.044186763535234652, -0.22773955761163256]]], [[[-0.81391008905988627, 0.94134815974881603, -0.71181797425869231, 0.58931705578025761], [0.27434744450909032, 0.28236916304783, 0.91023791064138959, 0.68979277440267328], [-0.720953929625195, 0.82242068121848089, 0.27635478930198754, -0.95091791893578104]], [[-0.25203499022073439, -0.68568413257267435, -0.29924710780745079, 0.69965309436670231], [-0.53071819612057314, -0.039075518371336493, -0.78639467261753437, -0.92015810457855163], [-0.42005878303025823, 0.81364725175999397, -0.51827954915638941, -0.74688756593230687]]], [[[-0.19357706524175278, -0.50241444585477191, -0.56723586186680675, -0.83952868893443555], [-0.54123827126039425, 0.41236143271136139, -0.78452155659763756, -0.84584424443890915], [0.028483192381424383, 0.27603692837961313, -0.21670730721198272, 0.61097851846601636]], [[0.89295301820646777, 0.16387018597327541, -0.24406195263139097, -0.80560101508762272], [0.078848859642367763, 0.0024368363763078715, -0.7783690225546902, -0.50753078234011828], [0.67282794405846125, 0.62648284872330318, -0.11026758119417712, 0.92501906189262328]]]])+(1.-msk_ref)*numpy.array([[[[-0.07611528984507876, -0.47812530896237626, -0.25942623749172178, 0.55493254138929649], [-0.22946271939066754, 0.73840875153960073, -0.040983059976963832, 0.17588271052000692], [-0.65716349160644816, -0.59828010440641055, 0.4403478331348325, 0.77972835168314947]], [[0.75921663312389209, -0.2784580521221387, -0.74420894259584824, 0.66040845029143846], [0.51546181862148099, 0.63248490527752654, -0.61848381965575561, 0.41091980869937217], [0.97079564136341667, -0.59000592327596468, -0.27591995603626063, -0.51995789329755993]]], [[[-0.65043240731215168, -0.75853074956668132, 0.53056801836881573, -0.025730918022197624], [0.29260532688272711, 0.68060539297868883, -0.59529254688050703, -0.29787661868788451], [0.51479365939487498, 0.37299502583565203, -0.8838683779838945, -0.056242853801407922]], [[0.97158132702703259, -0.6088450671522132, -0.3576635906904837, -0.41783175536232986], [-0.96931154248893847, -0.10227178975707663, -0.65210198192227931, 0.36335439111687462], [-0.77482950956053132, 0.2204388099696537, -0.067043895866304659, -0.6424907017287933]]], [[[0.62323183107976821, 0.88333740893801105, -0.78349398705810236, 0.4997741844267396], [0.64250440974075218, -0.4126927948016792, 0.40236419396109602, 0.82279414179119925], [-0.43795775664127023, -0.44903791714375796, -0.45277914810155551, 0.92971899356307874]], [[-0.38092442486833433, 0.0091969461730894153, 0.059546651251002336, 0.51321128033471131], [-0.64020797134164131, -0.54586469622389844, -0.82586172506752953, 0.095536359033647766], [-0.39064177768555197, -0.74606450260273638, 0.72964185657906278, 0.28114058336683456]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.990889660926)+(1.-msk_arg0)*(0.638489586459)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.021440027298)+(1.-msk_arg1)*(-0.0214498921701)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(-0.021440027298)+(1.-msk_ref)*(-0.0214498921701)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.61872992563211127, -0.86423515014777452])+(1.-msk_arg0)*numpy.array([-0.99897758157836147, -0.24165908859903573])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.810450971302)+(1.-msk_arg1)*(-0.883898898489)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.61872992563211127, -0.86423515014777452])+(1.-msk_ref)*numpy.array([-0.99897758157836147, -0.8838988984892151])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.57809290836321137, 0.61131024795215816, -0.16254350429937503, 0.53006905966067874, -0.50349392968088313], [-0.6440698094038757, 0.69454530962563577, -0.43817952396547577, 0.66034062958614959, -0.80884372520734327], [0.5626787379919842, 0.16259434796566108, 0.36622353383061745, -0.63845551344583629, 0.71558766822651343], [-0.50993166335705009, 0.26665578154492375, 0.53861866283245163, 0.77572810173574358, 0.74206103313474725]])+(1.-msk_arg0)*numpy.array([[-0.99107016113317403, 0.11075671480706628, 0.12612840476437825, 0.61863078266044269, 0.48431224547242668], [-0.25461635661047466, 0.69516433698361269, -0.80164710660605976, -0.96382939052091254, 0.69153703842540004], [0.8032011468446667, 0.51162994103725934, 0.84668546301153458, 0.67855415544892428, 0.6376083288170471], [-0.30647246938944672, -0.77463775878238184, 0.15940259391181444, 0.16031964275027333, 0.052373723628484692]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.530634180494)+(1.-msk_arg1)*(-0.0731398927261)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.57809290836321137, -0.53063418049374378, -0.53063418049374378, -0.53063418049374378, -0.53063418049374378], [-0.6440698094038757, -0.53063418049374378, -0.53063418049374378, -0.53063418049374378, -0.80884372520734327], [-0.53063418049374378, -0.53063418049374378, -0.53063418049374378, -0.63845551344583629, -0.53063418049374378], [-0.53063418049374378, -0.53063418049374378, -0.53063418049374378, -0.53063418049374378, -0.53063418049374378]])+(1.-msk_ref)*numpy.array([[-0.99107016113317403, -0.073139892726098443, -0.073139892726098443, -0.073139892726098443, -0.073139892726098443], [-0.25461635661047466, -0.073139892726098443, -0.80164710660605976, -0.96382939052091254, -0.073139892726098443], [-0.073139892726098443, -0.073139892726098443, -0.073139892726098443, -0.073139892726098443, -0.073139892726098443], [-0.30647246938944672, -0.77463775878238184, -0.073139892726098443, -0.073139892726098443, -0.073139892726098443]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.62662566987321666, -0.97785256576949564], [0.23964451398543463, -0.074653930720721906]], [[0.14613932634279547, 0.72739918144327609], [-0.70895254628118209, -0.090214115825494767]], [[0.26908483072350275, 0.5667442102134046], [0.51818108101124261, -0.097385193899757638]], [[-0.63710850656554974, -0.12166014794356905], [-0.097239376342243888, -0.72150207112372833]], [[-0.85135658001837689, 0.31875222504542222], [-0.18443479840717103, -0.6307702882058337]], [[0.88840209950027615, -0.46887359427357933], [0.41615845048775046, 0.83151132811969752]]])+(1.-msk_arg0)*numpy.array([[[0.48802916573201882, -0.25179150075753687], [0.75058606225722668, -0.919263471295797]], [[-0.85078803369522671, 0.98392726478149406], [-0.030340361270548843, -0.69847653668336207]], [[0.79450282770036718, -0.75478589838619703], [-0.46878127116264201, 0.38353402057374253]], [[-0.1467064753395606, -0.039916975353142758], [-0.83351905954235894, -0.35369261795507212]], [[0.64105874338970148, 0.95392297007217874], [-0.49197871404718452, -0.84675547319521094]], [[-0.63511776113388918, 0.11504612909116263], [-0.82667947607167047, 0.45816753353235207]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(0.570228458514)+(1.-msk_arg1)*(0.495811657368)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.62662566987321666, -0.97785256576949564], [0.23964451398543463, -0.074653930720721906]], [[0.14613932634279547, 0.57022845851385418], [-0.70895254628118209, -0.090214115825494767]], [[0.26908483072350275, 0.5667442102134046], [0.51818108101124261, -0.097385193899757638]], [[-0.63710850656554974, -0.12166014794356905], [-0.097239376342243888, -0.72150207112372833]], [[-0.85135658001837689, 0.31875222504542222], [-0.18443479840717103, -0.6307702882058337]], [[0.57022845851385418, -0.46887359427357933], [0.41615845048775046, 0.57022845851385418]]])+(1.-msk_ref)*numpy.array([[[0.48802916573201882, -0.25179150075753687], [0.49581165736751709, -0.919263471295797]], [[-0.85078803369522671, 0.49581165736751709], [-0.030340361270548843, -0.69847653668336207]], [[0.49581165736751709, -0.75478589838619703], [-0.46878127116264201, 0.38353402057374253]], [[-0.1467064753395606, -0.039916975353142758], [-0.83351905954235894, -0.35369261795507212]], [[0.49581165736751709, 0.49581165736751709], [-0.49197871404718452, -0.84675547319521094]], [[-0.63511776113388918, 0.11504612909116263], [-0.82667947607167047, 0.45816753353235207]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_expandedData_rank0(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.74750281784624617, -0.16810413074236652, 0.20491321566244536, -0.67356012035003787], [0.31772253789818516, -0.35715430243987512, 0.15241924652764038, -0.97738248216292445], [-0.55346404106743519, 0.42450603037819601, -0.51001429198785986, -0.86305438174304494]], [[0.2102550993356751, -0.12159077685239006, 0.2810710925465103, 0.26771615139810101], [0.33821864945390079, -0.4101445566744355, 0.18302102112191521, 0.66164182553960393], [0.15911549848063955, -0.232374949064861, -0.69248034637240941, 0.32538142102794243]]], [[[0.17369006334931103, -0.014029816006901896, -0.52212087160686727, -0.29886260222462568], [0.59017180540161807, 0.76006263042563549, 0.27009318877981769, -0.65645032190576025], [0.64387813999459609, -0.26057823323071072, 0.80988595315353384, 0.66596183778084383]], [[-0.62477851498697512, 0.93258445510388577, -0.96982815821406509, 0.66789797870433265], [-0.69624572930402762, 0.80593925908451625, 0.31046995739879102, 0.11972949289855328], [0.55970746818155703, -0.7733626102525597, -0.18865486562809819, -0.51856611048655021]]], [[[0.96340818941619988, -0.16263669505051692, -0.78159060340714115, -0.92535685332446582], [-0.71386749338910782, 0.53129450882747764, -0.24287648064530121, -0.12452046034889408], [0.87891446407436891, 0.0054348401841746163, 0.30230039171882339, 0.94174537336542774]], [[0.46455818744441468, -0.68657738004796465, 0.88842688515073753, -0.74607666323782773], [0.21447623068514954, -0.1116454759172465, -0.2807036222457886, -0.062950816540038534], [-0.42874253139276197, -0.51941738551773486, -0.48575320893944385, -0.61951429900930566]]]])+(1.-msk_arg0)*numpy.array([[[[0.64369186385254595, 0.8481169719642494, -0.16704691612962619, -0.80504153152296865], [-0.2793021349186966, -0.67077158740474463, 0.89097378085746404, -0.7890292463009545], [0.66037859190841042, 0.22743352408456663, 0.91195275727738956, 0.29974658727634296]], [[-0.18198705147094629, 0.87058398804650672, -0.31452429840530094, 0.33689241114523871], [-0.86670614218442843, -0.99834601812736801, 0.14948038401416186, 0.97301727980592845], [-0.90466859748611794, 0.84088229266876513, 0.77977499163625152, -0.12989656380369863]]], [[[-0.95994738469792229, 0.30947246860769484, -0.92321912041941823, 0.15245956593492815], [-0.89799544869786252, 0.28223362758996506, 0.85846999177486771, -0.87647083674033777], [0.21471919709356713, 0.70497308908502121, -0.36837722065914291, 0.51843208878191005]], [[-0.52840754288074843, -0.45618921814156166, -0.8830374929111553, 0.45740248927322602], [-0.11569412689515457, -0.19863429186345183, -0.15282914387081026, -0.44633295838014164], [0.25332031687368994, -0.46813445792309416, -0.84384770385119756, 0.016806455416438215]]], [[[-0.18051459322469965, -0.50301089439637248, -0.62781185314946919, -0.40007293245258246], [-0.47548142248719616, -0.22781247155660211, 0.90269966673649549, 0.70524383707492877], [-0.23146222053393495, -0.26427861043705847, 0.74880558221941307, -0.68387223179284606]], [[0.36599886499265866, -0.63511510971353458, -0.86357579314386146, 0.036272527918985498], [0.61488686718146202, -0.0029840665914477871, -0.023348354989592535, 0.61838170040130724], [-0.70102889151426417, -0.39329490866743155, -0.27387847945258437, -0.65904674572060951]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(-0.66378266073)+(1.-msk_arg1)*(0.176867584007)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.74750281784624617, -0.66378266073015757, -0.66378266073015757, -0.67356012035003787], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.97738248216292445], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.86305438174304494]], [[-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.69248034637240941, -0.66378266073015757]]], [[[-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757]], [[-0.66378266073015757, -0.66378266073015757, -0.96982815821406509, -0.66378266073015757], [-0.69624572930402762, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.7733626102525597, -0.66378266073015757, -0.66378266073015757]]], [[[-0.66378266073015757, -0.66378266073015757, -0.78159060340714115, -0.92535685332446582], [-0.71386749338910782, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757]], [[-0.66378266073015757, -0.68657738004796465, -0.66378266073015757, -0.74607666323782773], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757], [-0.66378266073015757, -0.66378266073015757, -0.66378266073015757, -0.66378266073015757]]]])+(1.-msk_ref)*numpy.array([[[[0.176867584006833, 0.176867584006833, -0.16704691612962619, -0.80504153152296865], [-0.2793021349186966, -0.67077158740474463, 0.176867584006833, -0.7890292463009545], [0.176867584006833, 0.176867584006833, 0.176867584006833, 0.176867584006833]], [[-0.18198705147094629, 0.176867584006833, -0.31452429840530094, 0.176867584006833], [-0.86670614218442843, -0.99834601812736801, 0.14948038401416186, 0.176867584006833], [-0.90466859748611794, 0.176867584006833, 0.176867584006833, -0.12989656380369863]]], [[[-0.95994738469792229, 0.176867584006833, -0.92321912041941823, 0.15245956593492815], [-0.89799544869786252, 0.176867584006833, 0.176867584006833, -0.87647083674033777], [0.176867584006833, 0.176867584006833, -0.36837722065914291, 0.176867584006833]], [[-0.52840754288074843, -0.45618921814156166, -0.8830374929111553, 0.176867584006833], [-0.11569412689515457, -0.19863429186345183, -0.15282914387081026, -0.44633295838014164], [0.176867584006833, -0.46813445792309416, -0.84384770385119756, 0.016806455416438215]]], [[[-0.18051459322469965, -0.50301089439637248, -0.62781185314946919, -0.40007293245258246], [-0.47548142248719616, -0.22781247155660211, 0.176867584006833, 0.176867584006833], [-0.23146222053393495, -0.26427861043705847, 0.176867584006833, -0.68387223179284606]], [[0.176867584006833, -0.63511510971353458, -0.86357579314386146, 0.036272527918985498], [0.176867584006833, -0.0029840665914477871, -0.023348354989592535, 0.176867584006833], [-0.70102889151426417, -0.39329490866743155, -0.27387847945258437, -0.65904674572060951]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.842402847791)+(1.-msk_arg0)*(-0.906601537829)
      arg1=numpy.array([-0.83377825260090077, -0.37098755558629559])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.84240284779105989, -0.84240284779105989])+(1.-msk_ref)*numpy.array([-0.90660153782895114, -0.90660153782895114])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_array_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.73587872494774897, -0.35869074864056683])+(1.-msk_arg0)*numpy.array([-0.05725872410823718, 0.0057164705554890727])
      arg1=numpy.array([0.0047780275563675811, 0.49289913890538206])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.0047780275563675811, -0.35869074864056683])+(1.-msk_ref)*numpy.array([-0.05725872410823718, 0.0057164705554890727])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.819029587075)+(1.-msk_arg0)*(0.028940260487)
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([-0.35903829548951482, 0.18086240337267423])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.81902958707530527, -0.81902958707530527])+(1.-msk_ref)*numpy.array([-0.35903829548951482, 0.028940260486997271])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_Symbol_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([-0.22052650710667998, -0.81217051509134408])+(1.-msk_arg0)*numpy.array([0.010833285880480714, 0.36815158077045895])
      arg1=Symbol(shape=(2,))
      res=minimum(arg0,arg1)
      s1=numpy.array([-0.65775475123840144, -0.89427609294130983])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.65775475123840144, -0.89427609294130983])+(1.-msk_ref)*numpy.array([-0.65775475123840144, -0.89427609294130983])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.619316501359)+(1.-msk_arg0)*(-0.927367468503)
      arg1=Data(numpy.array([0.96164988567854826, -0.99965820160885044]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.61931650135891236, -0.99965820160885044])+(1.-msk_ref)*numpy.array([-0.92736746850307972, -0.99965820160885044])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_constData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.57054897177490749, 0.416033158291641])+(1.-msk_arg0)*numpy.array([0.2617796914222692, 0.52730548338587258])
      arg1=Data(numpy.array([-0.1749926103514714, 0.50274541958270724]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.1749926103514714, 0.416033158291641])+(1.-msk_ref)*numpy.array([-0.1749926103514714, 0.50274541958270724])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.941509969475)+(1.-msk_arg0)*(0.242172275309)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.48825215447838954, 0.31988160282319056])+(1.-msk_arg1)*numpy.array([-0.79394085108267487, -0.41735178266651318])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([-0.9415099694746234, -0.9415099694746234])+(1.-msk_ref)*numpy.array([-0.79394085108267487, -0.41735178266651318])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank1_expandedData_rank1(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([0.47303262562639325, 0.97495522515615107])+(1.-msk_arg0)*numpy.array([-0.077894892719945652, -0.80065799694397244])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([0.50527520338016552, 0.63581449187935135])+(1.-msk_arg1)*numpy.array([-0.74058292461788722, -0.086272320749931186])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([0.47303262562639325, 0.63581449187935135])+(1.-msk_ref)*numpy.array([-0.74058292461788722, -0.80065799694397244])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.837334269788)+(1.-msk_arg0)*(0.367325728134)
      arg1=numpy.array([[-0.26930492594254019, -0.70861001514038446, 0.17806025039496376, 0.159191588435871, 0.61003454305661009], [-0.25923503090502953, -0.54123612211598116, -0.3966703187476599, -0.29699036099242981, -0.98304899367567167], [-0.72076074463771977, -0.042036317413082891, 0.9894280426739952, 0.74933245243739166, -0.86976425309626104], [-0.38359630193550109, -0.63973145949886545, 0.065755683172032953, -0.78659151700231655, 0.054022302543793366]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429], [-0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.98304899367567167], [-0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.86976425309626104], [-0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429, -0.83733426978787429]])+(1.-msk_ref)*numpy.array([[-0.26930492594254019, -0.70861001514038446, 0.17806025039496376, 0.159191588435871, 0.3673257281343052], [-0.25923503090502953, -0.54123612211598116, -0.3966703187476599, -0.29699036099242981, -0.98304899367567167], [-0.72076074463771977, -0.042036317413082891, 0.3673257281343052, 0.3673257281343052, -0.86976425309626104], [-0.38359630193550109, -0.63973145949886545, 0.065755683172032953, -0.78659151700231655, 0.054022302543793366]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_array_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.75815278427330535, -0.27270067098681383, 0.92671089144236896, -0.31300472038705407, -0.12807238722747716], [0.30270779135884918, 0.17037287970182469, -0.94548742814506781, 0.33674841464466998, 0.34429077909496564], [0.32867448512654152, 0.67725765973494978, 0.82883690795141973, -0.51356925221380267, -0.98912544405895142], [0.045243314512963595, 0.30354161910270827, 0.37200578160967734, -0.61446540812665851, 0.8887845172980926]])+(1.-msk_arg0)*numpy.array([[0.064220508829170964, 0.60798774433734448, 0.14851265280731241, -0.18471909240954298, 0.16966453665990699], [0.6438102319203407, -0.17704175423046764, 0.61167617222284676, -0.34971119244540216, 0.732954609990915], [0.42324588735524404, 0.52133339938382339, -0.92319854295578074, -0.40006301004744138, -0.69059244059516223], [-0.24369347835723598, -0.27531182231117768, 0.48239945839028486, 0.65638123709950325, 0.90625207376964823]])
      arg1=numpy.array([[0.448486011755586, -0.67331511464486593, -0.37806173423897516, -0.76306330811466561, 0.64755678844523312], [-0.55651629951659332, -0.25795556875335279, 0.80210907489549799, 0.67098877934185142, 0.22173805112838307], [-0.73253370512115845, 0.41073071032983166, -0.75720113901150432, -0.45769873369275227, 0.61081635920859223], [0.29817938927724419, -0.96429389230646612, 0.55058157545006536, 0.97293629823480265, 0.51849193033631291]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.448486011755586, -0.67331511464486593, -0.37806173423897516, -0.76306330811466561, -0.12807238722747716], [-0.55651629951659332, -0.25795556875335279, -0.94548742814506781, 0.33674841464466998, 0.22173805112838307], [-0.73253370512115845, 0.41073071032983166, -0.75720113901150432, -0.51356925221380267, -0.98912544405895142], [0.045243314512963595, -0.96429389230646612, 0.37200578160967734, -0.61446540812665851, 0.51849193033631291]])+(1.-msk_ref)*numpy.array([[0.064220508829170964, -0.67331511464486593, -0.37806173423897516, -0.76306330811466561, 0.16966453665990699], [-0.55651629951659332, -0.25795556875335279, 0.61167617222284676, -0.34971119244540216, 0.22173805112838307], [-0.73253370512115845, 0.41073071032983166, -0.92319854295578074, -0.45769873369275227, -0.69059244059516223], [-0.24369347835723598, -0.96429389230646612, 0.48239945839028486, 0.65638123709950325, 0.51849193033631291]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.859605992769)+(1.-msk_arg0)*(0.869361633334)
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[0.13332880450632789, 0.89473288500333981, 0.35062037960508441, 0.41445506895352713, 0.38463070446237624], [0.77859383335568078, 0.46959943513374824, -0.32593475780836045, -0.11145173556234211, -0.5557834390954326], [-0.44133862186780948, -0.33371031080468971, -0.82856874049197016, 0.21980095391139831, 0.90641242951381118], [-0.24846022299415638, -0.098009251176446099, -0.27658871946811181, 0.93972591845461872, -0.26391052012410987]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[0.13332880450632789, 0.85960599276904959, 0.35062037960508441, 0.41445506895352713, 0.38463070446237624], [0.77859383335568078, 0.46959943513374824, -0.32593475780836045, -0.11145173556234211, -0.5557834390954326], [-0.44133862186780948, -0.33371031080468971, -0.82856874049197016, 0.21980095391139831, 0.85960599276904959], [-0.24846022299415638, -0.098009251176446099, -0.27658871946811181, 0.85960599276904959, -0.26391052012410987]])+(1.-msk_ref)*numpy.array([[0.13332880450632789, 0.86936163333377214, 0.35062037960508441, 0.41445506895352713, 0.38463070446237624], [0.77859383335568078, 0.46959943513374824, -0.32593475780836045, -0.11145173556234211, -0.5557834390954326], [-0.44133862186780948, -0.33371031080468971, -0.82856874049197016, 0.21980095391139831, 0.86936163333377214], [-0.24846022299415638, -0.098009251176446099, -0.27658871946811181, 0.86936163333377214, -0.26391052012410987]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.27242546526914935, 0.34579525753942142, -0.20921211548333751, -0.204921892076527, 0.54925870679838118], [-0.90916866468608926, -0.99618785777917873, 0.46458588225216935, -0.15043008134246727, -0.86774002401367811], [-0.76111604203236016, -0.3148157285450135, -0.4996214518080806, -0.87759025150145575, 0.18539301061160063], [-0.77408903750734548, 0.27979849406514035, -0.995123856152202, -0.028278043015043552, 0.79036518866749872]])+(1.-msk_arg0)*numpy.array([[0.53080487080010719, -0.94451426807550187, 0.67964531927122396, -0.73447815116031645, -0.52965880413861521], [-0.34789345411150729, -0.38165465337811089, -0.23157822333450673, 0.083417736026888401, -0.35817959989090364], [0.32046552262620853, -0.96670425160937534, -0.087508352848618465, 0.66257872645153415, -0.9182459506356031], [0.37711922253551955, 0.90310930098428388, 0.35844377057926535, -0.91860174550110929, -0.9580820441213882]])
      arg1=Symbol(shape=(4, 5))
      res=minimum(arg0,arg1)
      s1=numpy.array([[0.51480842046743813, -0.51157235852842864, -0.33425256657163493, 0.84829118904077094, -0.47731027107317647], [-0.013889839922651026, 0.93951452603260566, 0.058573868506020199, 0.71262854238998408, 0.71329526859614578], [0.60501478670611464, -0.32111794650686853, -0.17806979653665889, 0.25398176735776734, -0.65004946700562294], [-0.88618412862605633, 0.71384981761871669, 0.3030213171230578, 0.75124792477286229, -0.45301944251299497]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.27242546526914935, -0.51157235852842864, -0.33425256657163493, -0.204921892076527, -0.47731027107317647], [-0.90916866468608926, -0.99618785777917873, 0.058573868506020199, -0.15043008134246727, -0.86774002401367811], [-0.76111604203236016, -0.32111794650686853, -0.4996214518080806, -0.87759025150145575, -0.65004946700562294], [-0.88618412862605633, 0.27979849406514035, -0.995123856152202, -0.028278043015043552, -0.45301944251299497]])+(1.-msk_ref)*numpy.array([[0.51480842046743813, -0.94451426807550187, -0.33425256657163493, -0.73447815116031645, -0.52965880413861521], [-0.34789345411150729, -0.38165465337811089, -0.23157822333450673, 0.083417736026888401, -0.35817959989090364], [0.32046552262620853, -0.96670425160937534, -0.17806979653665889, 0.25398176735776734, -0.9182459506356031], [-0.88618412862605633, 0.71384981761871669, 0.3030213171230578, -0.91860174550110929, -0.9580820441213882]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.0279144982409)+(1.-msk_arg0)*(0.951595483286)
      arg1=Data(numpy.array([[-0.20525723047497046, 0.44406498165406472, -0.50663137104109146, -0.0094296875132247049, 0.21937718429591624], [0.49438849701232823, -0.11336717468638846, -0.53383364430959035, -0.96570584014918293, -0.98776298518457151], [0.27459905386577899, 0.60017419185598575, 0.49029888171329694, 0.18988750499695306, -0.1529071923195835], [0.7429715793928906, 0.048216176997676996, 0.064002471983303444, -0.27681595931854019, -0.91276991952999031]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.20525723047497046, -0.027914498240869223, -0.50663137104109146, -0.027914498240869223, -0.027914498240869223], [-0.027914498240869223, -0.11336717468638846, -0.53383364430959035, -0.96570584014918293, -0.98776298518457151], [-0.027914498240869223, -0.027914498240869223, -0.027914498240869223, -0.027914498240869223, -0.1529071923195835], [-0.027914498240869223, -0.027914498240869223, -0.027914498240869223, -0.27681595931854019, -0.91276991952999031]])+(1.-msk_ref)*numpy.array([[-0.20525723047497046, 0.44406498165406472, -0.50663137104109146, -0.0094296875132247049, 0.21937718429591624], [0.49438849701232823, -0.11336717468638846, -0.53383364430959035, -0.96570584014918293, -0.98776298518457151], [0.27459905386577899, 0.60017419185598575, 0.49029888171329694, 0.18988750499695306, -0.1529071923195835], [0.7429715793928906, 0.048216176997676996, 0.064002471983303444, -0.27681595931854019, -0.91276991952999031]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_constData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[-0.27792738422539265, 0.23444343102606546, 0.16596951294627971, 0.17766898950827636, 0.44478466282317286], [0.55950970958455937, 0.56401204651204973, 0.050792478353166359, 0.028728181883849402, 0.86208502573296308], [-0.78403835802146005, 0.82607783691130243, -0.52989316961523825, 0.38521532888490162, 0.69440766176590718], [-0.881352053729062, -0.6836179857944733, 0.63676578558739894, 0.76552557956090461, 0.40601714876946682]])+(1.-msk_arg0)*numpy.array([[-0.14772602208893515, -0.0057983182761083274, -0.61304420546599636, -0.56544454729009885, -0.68792446085798953], [0.10442431600863156, 0.35302531028405149, -0.11187210982409512, 0.86478686155186524, -0.73529197897456022], [0.16884780642383035, -0.92809027234487651, 0.65142594109899665, 0.63031501470674778, 0.36522828323811884], [0.025123879513389236, 0.031699123872386492, 0.85233492028813496, 0.77979846694746491, -0.98379846966027729]])
      arg1=Data(numpy.array([[0.37595351260671905, 0.23289713071327633, 0.1381696607688716, 0.58924415425351695, -0.10594593177841927], [0.95143615675180548, 0.1498326531994012, -0.11091188264571294, -0.37546274438281557, -0.51751054960208309], [-0.94743480613147391, 0.32366686928449773, 0.51629592716499451, 0.82465031835797342, 0.91991475132391565], [0.35634689986603174, 0.25070301849541243, 0.97452003970305356, -0.6982231861464967, 0.75616699116538988]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.27792738422539265, 0.23289713071327633, 0.1381696607688716, 0.17766898950827636, -0.10594593177841927], [0.55950970958455937, 0.1498326531994012, -0.11091188264571294, -0.37546274438281557, -0.51751054960208309], [-0.94743480613147391, 0.32366686928449773, -0.52989316961523825, 0.38521532888490162, 0.69440766176590718], [-0.881352053729062, -0.6836179857944733, 0.63676578558739894, -0.6982231861464967, 0.40601714876946682]])+(1.-msk_ref)*numpy.array([[-0.14772602208893515, -0.0057983182761083274, -0.61304420546599636, -0.56544454729009885, -0.68792446085798953], [0.10442431600863156, 0.1498326531994012, -0.11187210982409512, -0.37546274438281557, -0.73529197897456022], [-0.94743480613147391, -0.92809027234487651, 0.51629592716499451, 0.63031501470674778, 0.36522828323811884], [0.025123879513389236, 0.031699123872386492, 0.85233492028813496, -0.6982231861464967, -0.98379846966027729]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.515781668257)+(1.-msk_arg0)*(0.051010474103)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.4716970375401659, -0.80091422265073131, -0.91800698021859883, 0.30140546864420625, -0.54205382857510109], [0.48468330239335145, 0.8121797189082931, -0.98220426187303023, -0.40905203022702019, -0.50401385876780669], [0.10234807318411132, -0.10939804858183644, 0.38168880078563849, 0.88251628140862848, -0.44641947628160983], [0.26030678583841205, 0.27045518367821075, -0.70924156725516885, 0.98157706245208898, 0.19058740609834102]])+(1.-msk_arg1)*numpy.array([[-0.0137301895429216, 0.27262376862289628, -0.75802785888055135, 0.76610381404644157, 0.82096961069783703], [0.75822457249477826, 0.036715084201798254, -0.70869577770163361, 0.15654024859068061, 0.0060195628618959773], [0.44714767436082048, 0.92251496067612471, 0.014229541294706438, 0.067954707485034627, -0.23637805650611576], [-0.88316013612057875, -0.90332534164080647, 0.7384105903555731, -0.70788207126646396, 0.10817711546041053]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.4716970375401659, -0.80091422265073131, -0.91800698021859883, 0.30140546864420625, -0.54205382857510109], [0.48468330239335145, 0.51578166825729332, -0.98220426187303023, -0.40905203022702019, -0.50401385876780669], [0.10234807318411132, -0.10939804858183644, 0.38168880078563849, 0.51578166825729332, -0.44641947628160983], [0.26030678583841205, 0.27045518367821075, -0.70924156725516885, 0.51578166825729332, 0.19058740609834102]])+(1.-msk_ref)*numpy.array([[-0.0137301895429216, 0.051010474102958669, -0.75802785888055135, 0.051010474102958669, 0.051010474102958669], [0.051010474102958669, 0.036715084201798254, -0.70869577770163361, 0.051010474102958669, 0.0060195628618959773], [0.051010474102958669, 0.051010474102958669, 0.014229541294706438, 0.051010474102958669, -0.23637805650611576], [-0.88316013612057875, -0.90332534164080647, 0.051010474102958669, -0.70788207126646396, 0.051010474102958669]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[0.48958556868934022, -0.92296741349090583, 0.756389119775708, 0.44596469123090987, -0.90874783171417195], [-0.87047452465149666, -0.69235262479968007, 0.23455567017497381, -0.60619013984908521, -0.26655399724861706], [-0.64015777846090582, -0.3979247904074954, -0.31877828435919642, -0.68896577604504028, 0.24470700495248154], [0.87685592432440673, -0.88284138475013219, 0.20116425269259652, 0.97804970927411294, -0.55832579222530243]])+(1.-msk_arg0)*numpy.array([[-0.67094039460485955, 0.49329579545631663, -0.25798550389191366, -0.3033729691562701, -0.45980255214895571], [-0.43235301323149433, 0.20259735692549641, 0.12957480859334303, 0.85504708724770251, 0.21519665932683529], [-0.24828164256662055, 0.31034514326190044, -0.07956987101411972, 0.81880318506813232, -0.20052058236191028], [0.53861651708347291, -0.42490387836762156, 0.57690197647907993, 0.64952738513893138, 0.87440192262236582]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[-0.65301063658808034, -0.14802803725456748, 0.81648431650889242, -0.55784261539989055, -0.67549296495256828], [0.03612600926348386, 0.99386456975773863, 0.066782075434601351, 0.02913802662680598, -0.94467733879185878], [0.47858022222548935, 0.21226410302106258, -0.87590896713901212, 0.65125411844531955, 0.74715254306631351], [0.70005687768137892, 0.83177496391627193, 0.76507320584197003, 0.018246533054664393, -0.15202050754910323]])+(1.-msk_arg1)*numpy.array([[0.73598880205409301, 0.56550154595645119, -0.58776334393975427, 0.79848113338752902, 0.73365579037605566], [0.77185086956647275, -0.99151219641012367, 0.034446020166434721, -0.83131844368372021, 0.52267028767700108], [0.2688697063427461, -0.096653256973761925, -0.45575687556295952, -0.22184422552763849, -0.71388218415699822], [0.081321870523604511, 0.31496661153999228, 0.60830091695300137, 0.57070604534672276, -0.28326204332472593]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[-0.65301063658808034, -0.92296741349090583, 0.756389119775708, -0.55784261539989055, -0.90874783171417195], [-0.87047452465149666, -0.69235262479968007, 0.066782075434601351, -0.60619013984908521, -0.94467733879185878], [-0.64015777846090582, -0.3979247904074954, -0.87590896713901212, -0.68896577604504028, 0.24470700495248154], [0.70005687768137892, -0.88284138475013219, 0.20116425269259652, 0.018246533054664393, -0.55832579222530243]])+(1.-msk_ref)*numpy.array([[-0.67094039460485955, 0.49329579545631663, -0.58776334393975427, -0.3033729691562701, -0.45980255214895571], [-0.43235301323149433, -0.99151219641012367, 0.034446020166434721, -0.83131844368372021, 0.21519665932683529], [-0.24828164256662055, -0.096653256973761925, -0.45575687556295952, -0.22184422552763849, -0.71388218415699822], [0.081321870523604511, -0.42490387836762156, 0.57690197647907993, 0.57070604534672276, -0.28326204332472593]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.456000460704)+(1.-msk_arg0)*(-0.167260028739)
      arg1=numpy.array([[[0.13937174506790306, 0.84327753669433614], [0.85706887732388015, 0.071511216613709516]], [[-0.086281789429336841, 0.29597330487755391], [-0.8635381630195289, 0.86708569963266102]], [[-0.5640393169443878, 0.30046483949713854], [0.30853449887358297, 0.74430016909887087]], [[-0.15438015325021492, 0.37472319156518563], [0.27825695617780677, 0.73412418348958575]], [[0.48563694458782325, 0.36987117832247174], [-0.46961645536814811, -0.81933933145747218]], [[-0.9252348288823391, -0.8922391966247214], [0.61619531785506343, 0.84230576327925855]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.45600046070363742, -0.45600046070363742], [-0.45600046070363742, -0.45600046070363742]], [[-0.45600046070363742, -0.45600046070363742], [-0.8635381630195289, -0.45600046070363742]], [[-0.5640393169443878, -0.45600046070363742], [-0.45600046070363742, -0.45600046070363742]], [[-0.45600046070363742, -0.45600046070363742], [-0.45600046070363742, -0.45600046070363742]], [[-0.45600046070363742, -0.45600046070363742], [-0.46961645536814811, -0.81933933145747218]], [[-0.9252348288823391, -0.8922391966247214], [-0.45600046070363742, -0.45600046070363742]]])+(1.-msk_ref)*numpy.array([[[-0.1672600287386532, -0.1672600287386532], [-0.1672600287386532, -0.1672600287386532]], [[-0.1672600287386532, -0.1672600287386532], [-0.8635381630195289, -0.1672600287386532]], [[-0.5640393169443878, -0.1672600287386532], [-0.1672600287386532, -0.1672600287386532]], [[-0.1672600287386532, -0.1672600287386532], [-0.1672600287386532, -0.1672600287386532]], [[-0.1672600287386532, -0.1672600287386532], [-0.46961645536814811, -0.81933933145747218]], [[-0.9252348288823391, -0.8922391966247214], [-0.1672600287386532, -0.1672600287386532]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_array_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.50902118805999708, -0.18317307708008745], [0.10634106921471487, -0.45135218679343447]], [[0.73113512494233701, 0.16943536778692336], [-0.89040735378621916, 0.76655626991914971]], [[-0.46819770910180702, -0.19921185032515232], [-0.47207487512874158, 0.42589679438989436]], [[0.48135158601495442, -0.99765967147455603], [0.30932976351634345, -0.029823241097793884]], [[0.56168625607256217, 0.29967354622543074], [0.33181448671884461, -0.043048866655934992]], [[0.45605232382230443, 0.57174744726102467], [-0.82577795333017545, -0.94230549246517747]]])+(1.-msk_arg0)*numpy.array([[[-0.46031607927851792, 0.23300367312345105], [0.85507510560237776, 0.16191785677978521]], [[0.97738994108064192, -0.87426038087742031], [-0.49508150161214437, 0.37811213070530836]], [[0.67965742738835355, -0.85732509592015749], [-0.010796832324976302, -0.89825123009871333]], [[0.78515665696733361, -0.3438282520674607], [0.94790509512478405, -0.3378903605036776]], [[-0.37250074683817957, 0.81121471125884814], [-0.59258769261980171, -0.67826483726896925]], [[-0.43350164058606944, -0.30490510988148256], [0.30060848608294211, 0.1564514970888804]]])
      arg1=numpy.array([[[-0.055628046074645887, 0.28723878423994198], [0.96421895538402325, -0.25542888003843944]], [[0.2814582475473506, -0.24188293210233724], [-0.040754722253135922, -0.1419313705266001]], [[-0.51293930277570521, -0.73451472898444914], [0.93564899737256568, -0.90101279293833469]], [[0.97020682652671342, -0.050083267760595573], [0.66526331308884856, 0.24803233515044121]], [[-0.45338256545319178, -0.17053125666171609], [0.80912438817255627, 0.78524510835846861]], [[0.82393877660358239, 0.51396256792410622], [-0.49877066821748661, -0.088871848844500168]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.055628046074645887, -0.18317307708008745], [0.10634106921471487, -0.45135218679343447]], [[0.2814582475473506, -0.24188293210233724], [-0.89040735378621916, -0.1419313705266001]], [[-0.51293930277570521, -0.73451472898444914], [-0.47207487512874158, -0.90101279293833469]], [[0.48135158601495442, -0.99765967147455603], [0.30932976351634345, -0.029823241097793884]], [[-0.45338256545319178, -0.17053125666171609], [0.33181448671884461, -0.043048866655934992]], [[0.45605232382230443, 0.51396256792410622], [-0.82577795333017545, -0.94230549246517747]]])+(1.-msk_ref)*numpy.array([[[-0.46031607927851792, 0.23300367312345105], [0.85507510560237776, -0.25542888003843944]], [[0.2814582475473506, -0.87426038087742031], [-0.49508150161214437, -0.1419313705266001]], [[-0.51293930277570521, -0.85732509592015749], [-0.010796832324976302, -0.90101279293833469]], [[0.78515665696733361, -0.3438282520674607], [0.66526331308884856, -0.3378903605036776]], [[-0.45338256545319178, -0.17053125666171609], [-0.59258769261980171, -0.67826483726896925]], [[-0.43350164058606944, -0.30490510988148256], [-0.49877066821748661, -0.088871848844500168]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.0446486886865)+(1.-msk_arg0)*(-0.640751123045)
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[-0.42266661301793351, -0.1443671382523648], [-0.9327749656440476, 0.31740018398045611]], [[-0.521748018188537, -0.64299222077188545], [0.24323303412316366, -0.16837358246291689]], [[-0.11120152654490378, 0.35219220307127608], [-0.56214515818364119, -0.28598853253000289]], [[-0.87395643326804251, 0.19933073494750864], [-0.69461102177173628, -0.390911340478719]], [[-0.2344170362629987, 0.5953469840943828], [-0.14418326995138919, 0.62232359749236221]], [[-0.54191984459071829, 0.22459637187862547], [0.83275857299063438, -0.41088458255413496]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.42266661301793351, -0.1443671382523648], [-0.9327749656440476, 0.044648688686522275]], [[-0.521748018188537, -0.64299222077188545], [0.044648688686522275, -0.16837358246291689]], [[-0.11120152654490378, 0.044648688686522275], [-0.56214515818364119, -0.28598853253000289]], [[-0.87395643326804251, 0.044648688686522275], [-0.69461102177173628, -0.390911340478719]], [[-0.2344170362629987, 0.044648688686522275], [-0.14418326995138919, 0.044648688686522275]], [[-0.54191984459071829, 0.044648688686522275], [0.044648688686522275, -0.41088458255413496]]])+(1.-msk_ref)*numpy.array([[[-0.64075112304476622, -0.64075112304476622], [-0.9327749656440476, -0.64075112304476622]], [[-0.64075112304476622, -0.64299222077188545], [-0.64075112304476622, -0.64075112304476622]], [[-0.64075112304476622, -0.64075112304476622], [-0.64075112304476622, -0.64075112304476622]], [[-0.87395643326804251, -0.64075112304476622], [-0.69461102177173628, -0.64075112304476622]], [[-0.64075112304476622, -0.64075112304476622], [-0.64075112304476622, -0.64075112304476622]], [[-0.64075112304476622, -0.64075112304476622], [-0.64075112304476622, -0.64075112304476622]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_Symbol_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.81244489536094577, 0.52678238193648874], [-0.79049540350596392, 0.12223236282324579]], [[-0.31307255156704894, -0.31787834371428736], [-0.56305626868848946, 0.10883267824209164]], [[-0.80076798146242334, -0.54062003277886284], [0.98450947722970295, 0.61795266162989848]], [[-0.16458126147107666, 0.13096749198874114], [-0.8970305285261948, -0.3815896738866873]], [[-0.88422122283470372, 0.26215720913368479], [0.49469204030460157, -0.24479622538410362]], [[-0.7267522930979482, 0.70822905980141115], [0.040740998448109966, 0.70477595260366055]]])+(1.-msk_arg0)*numpy.array([[[-0.4538139983224061, -0.34985009609894613], [-0.3088613934743476, 0.27484184108413379]], [[0.30733579715453985, -0.3502618500927448], [0.52406700282454799, -0.77896058653913314]], [[-0.36831658214222673, -0.81157455455134953], [0.94697499468347601, -0.32867899829332536]], [[0.37982545643748655, -0.57316243825394952], [-0.25834937595984608, 0.98437577069827498]], [[-0.91695924303201015, -0.1755499485945895], [-0.60256068476890357, -0.94735997635225622]], [[0.39479196290165741, -0.1475063107220751], [0.23837329939222873, -0.59962827241797023]]])
      arg1=Symbol(shape=(6, 2, 2))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[0.12537173880330488, 0.075720290215295361], [-0.89587602199001726, 0.96278185071965461]], [[-0.50834481303721057, -0.59494766571941837], [-0.33293779444566063, 0.39113550155636156]], [[0.0084630848966877892, -0.97751658844180578], [-0.041724840350798598, 0.79302871351403681]], [[0.049091708050443161, -0.66794959354366079], [0.72304229335899706, 0.64058509134950659]], [[0.77921204607903394, 0.42194982913745038], [-0.71795885296052608, 0.42267391907132024]], [[0.61109946919282976, -0.31525864658930081], [0.89013243215991378, 0.24112597802899804]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.81244489536094577, 0.075720290215295361], [-0.89587602199001726, 0.12223236282324579]], [[-0.50834481303721057, -0.59494766571941837], [-0.56305626868848946, 0.10883267824209164]], [[-0.80076798146242334, -0.97751658844180578], [-0.041724840350798598, 0.61795266162989848]], [[-0.16458126147107666, -0.66794959354366079], [-0.8970305285261948, -0.3815896738866873]], [[-0.88422122283470372, 0.26215720913368479], [-0.71795885296052608, -0.24479622538410362]], [[-0.7267522930979482, -0.31525864658930081], [0.040740998448109966, 0.24112597802899804]]])+(1.-msk_ref)*numpy.array([[[-0.4538139983224061, -0.34985009609894613], [-0.89587602199001726, 0.27484184108413379]], [[-0.50834481303721057, -0.59494766571941837], [-0.33293779444566063, -0.77896058653913314]], [[-0.36831658214222673, -0.97751658844180578], [-0.041724840350798598, -0.32867899829332536]], [[0.049091708050443161, -0.66794959354366079], [-0.25834937595984608, 0.64058509134950659]], [[-0.91695924303201015, -0.1755499485945895], [-0.71795885296052608, -0.94735997635225622]], [[0.39479196290165741, -0.31525864658930081], [0.23837329939222873, -0.59962827241797023]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.679107826024)+(1.-msk_arg0)*(-0.113570924181)
      arg1=Data(numpy.array([[[0.33329203989625777, -0.91954307756499953], [0.34450633089160454, -0.10471102182382985]], [[-0.41494252311686997, -0.28271397592119341], [-0.82409222082895606, 0.72533393026724324]], [[-0.032671245195534437, -0.27677672184833524], [0.65217184883685819, 0.32602715966453233]], [[-0.2205689520694718, -0.15326419961899562], [0.61091020137658147, 0.61190314349809927]], [[0.95970298571009693, -0.010972522520193762], [-0.11566440298399749, 0.23492362121363808]], [[0.07360441194930134, -0.13172715363179521], [0.9928059463094443, 0.21564006861807883]]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.67910782602413255, -0.91954307756499953], [-0.67910782602413255, -0.67910782602413255]], [[-0.67910782602413255, -0.67910782602413255], [-0.82409222082895606, -0.67910782602413255]], [[-0.67910782602413255, -0.67910782602413255], [-0.67910782602413255, -0.67910782602413255]], [[-0.67910782602413255, -0.67910782602413255], [-0.67910782602413255, -0.67910782602413255]], [[-0.67910782602413255, -0.67910782602413255], [-0.67910782602413255, -0.67910782602413255]], [[-0.67910782602413255, -0.67910782602413255], [-0.67910782602413255, -0.67910782602413255]]])+(1.-msk_ref)*numpy.array([[[-0.11357092418082804, -0.91954307756499953], [-0.11357092418082804, -0.11357092418082804]], [[-0.41494252311686997, -0.28271397592119341], [-0.82409222082895606, -0.11357092418082804]], [[-0.11357092418082804, -0.27677672184833524], [-0.11357092418082804, -0.11357092418082804]], [[-0.2205689520694718, -0.15326419961899562], [-0.11357092418082804, -0.11357092418082804]], [[-0.11357092418082804, -0.11357092418082804], [-0.11566440298399749, -0.11357092418082804]], [[-0.11357092418082804, -0.13172715363179521], [-0.11357092418082804, -0.11357092418082804]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_constData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[-0.25214048068745654, 0.052389293687070237], [-0.8949647052646803, -0.27585834032294332]], [[0.22184475555399641, -0.56715818139855378], [0.77652646549636484, 0.31123320652071751]], [[0.47345789745894407, 0.93447830642024599], [-0.76094394758857975, -0.51989943557816498]], [[-0.058824415433247124, 0.68066678084345833], [-0.4830602950457461, -0.39110225661972353]], [[0.059813400688766283, -0.79442499243782705], [-0.47324005118901247, -0.9076200020735361]], [[0.67127181461617536, -0.091174963242821816], [-0.59972841400200205, 0.22232690585568493]]])+(1.-msk_arg0)*numpy.array([[[0.35031203271240385, -0.030671131974060462], [-0.35757486391234861, -0.65170562549894928]], [[-0.9227500536624742, 0.99582109272603003], [-0.55773348830849478, -0.95042016392778961]], [[0.33426027164849459, 0.52038109936391019], [-0.70292077134722342, 0.52842010844803444]], [[0.38671137833529357, -0.60570395854628201], [0.42787075706226285, -0.29013024206172378]], [[-0.63227960362939917, -0.079247152879650562], [-0.072590521312945633, 0.12814260574044756]], [[-0.20752759218194905, -0.11683262699701902], [-0.068631605741806379, -0.58348069043497119]]])
      arg1=Data(numpy.array([[[-0.15981045445506359, 0.43654189802682497], [-0.88239882561497707, -0.25366011510066722]], [[-0.83386402274848104, 0.59235390541543764], [-0.80971771629326095, 0.068403152150757096]], [[0.61564510864046884, 0.63119731998001893], [0.02740626311603056, 0.35235715499437892]], [[0.55802775336809862, 0.029925210418192405], [-0.070883024844496711, -0.25458667417329606]], [[0.50988304366259896, 0.87819033789609091], [-0.76742563926932972, -0.52216527308067895]], [[-0.99883361025339701, -0.77982220168608851], [0.95362183785557164, 0.28776070357810934]]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.25214048068745654, 0.052389293687070237], [-0.8949647052646803, -0.27585834032294332]], [[-0.83386402274848104, -0.56715818139855378], [-0.80971771629326095, 0.068403152150757096]], [[0.47345789745894407, 0.63119731998001893], [-0.76094394758857975, -0.51989943557816498]], [[-0.058824415433247124, 0.029925210418192405], [-0.4830602950457461, -0.39110225661972353]], [[0.059813400688766283, -0.79442499243782705], [-0.76742563926932972, -0.9076200020735361]], [[-0.99883361025339701, -0.77982220168608851], [-0.59972841400200205, 0.22232690585568493]]])+(1.-msk_ref)*numpy.array([[[-0.15981045445506359, -0.030671131974060462], [-0.88239882561497707, -0.65170562549894928]], [[-0.9227500536624742, 0.59235390541543764], [-0.80971771629326095, -0.95042016392778961]], [[0.33426027164849459, 0.52038109936391019], [-0.70292077134722342, 0.35235715499437892]], [[0.38671137833529357, -0.60570395854628201], [-0.070883024844496711, -0.29013024206172378]], [[-0.63227960362939917, -0.079247152879650562], [-0.76742563926932972, -0.52216527308067895]], [[-0.99883361025339701, -0.77982220168608851], [-0.068631605741806379, -0.58348069043497119]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.114792422685)+(1.-msk_arg0)*(-0.785613561753)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[-0.3308745126288275, 0.40363749209476207], [0.71828623724867402, -0.6355958244786617]], [[-0.73314814963148223, 0.6693898248186656], [0.9888546361728856, -0.57987104287119529]], [[-0.64946657181007827, 0.33326711353828786], [-0.39892579649162974, 0.38714634266813563]], [[0.51779091113984754, 0.93508546917815383], [-0.15673085305270607, 0.86475721978994069]], [[0.31256147138595969, 0.83354861625901422], [-0.36251535352822462, 0.46543041668694785]], [[0.25756344688370003, 0.43790014025549251], [0.75555702045698303, 0.76267793272936046]]])+(1.-msk_arg1)*numpy.array([[[0.85118686570877067, 0.57467342820461553], [0.84658625434126344, -0.72178766140974959]], [[0.19328282140220798, 0.065066104776074996], [0.5439633716390091, 0.18315925686668888]], [[0.97577415800903644, 0.97309035476408812], [-0.37530219846764346, -0.0024991607999924614]], [[0.21722963310672871, 0.0906245651131381], [-0.67755086532158559, 0.27698112112210249]], [[0.2936457534156538, 0.87538922827156451], [0.88543814629738971, -0.56496547293896393]], [[0.20165980916866411, 0.053935389314735405], [0.63176295036156516, 0.095597905287107432]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[-0.3308745126288275, -0.11479242268464551], [-0.11479242268464551, -0.6355958244786617]], [[-0.73314814963148223, -0.11479242268464551], [-0.11479242268464551, -0.57987104287119529]], [[-0.64946657181007827, -0.11479242268464551], [-0.39892579649162974, -0.11479242268464551]], [[-0.11479242268464551, -0.11479242268464551], [-0.15673085305270607, -0.11479242268464551]], [[-0.11479242268464551, -0.11479242268464551], [-0.36251535352822462, -0.11479242268464551]], [[-0.11479242268464551, -0.11479242268464551], [-0.11479242268464551, -0.11479242268464551]]])+(1.-msk_ref)*numpy.array([[[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]], [[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]], [[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]], [[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]], [[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]], [[-0.78561356175317187, -0.78561356175317187], [-0.78561356175317187, -0.78561356175317187]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank3_expandedData_rank3(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[0.70599769281292724, -0.62682631245719378], [0.16240992398018328, -0.30461390116360687]], [[-0.66439851208005218, 0.46530295777225938], [-0.36745772150478584, -0.27272767458596392]], [[0.88754891209809683, 0.46974918792912979], [0.73864747310244594, -0.85151003919293977]], [[-0.21533682034437662, -0.65561908253956425], [-0.026146585653293197, 0.37445084878522317]], [[-0.65005710266960848, -0.33540146795034365], [-0.037980818728829968, -0.30364243761179033]], [[0.31272912236735406, 0.16550088425772569], [-0.89170233888417894, -0.060949202001112202]]])+(1.-msk_arg0)*numpy.array([[[0.03286881168483613, 0.67897982415725555], [0.86674048211321231, -0.7715805177911832]], [[0.57397638092698355, -0.46435045979941525], [-0.29987021473606057, 0.87217680942595788]], [[0.52593281930871338, -0.22040994139329739], [0.12885850657281939, 0.51524829386818638]], [[-0.81786035801465506, -0.69917257706297731], [-0.46237380435922737, -0.056518365979367546]], [[0.57018001026326126, 0.31171572340356191], [0.55939816262581243, -0.26197875119045522]], [[0.0072844237960489888, 0.64459875521538446], [-0.40207692212912938, 0.64490365474220956]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[0.70453547216543, -0.12231449560661978], [0.29420749022638937, -0.93357418232171252]], [[-0.02053427845359046, -0.28702503130392953], [0.52220540192354958, 0.92383968724995458]], [[0.97144381493916687, -0.60696351581127272], [0.68922464624268009, -0.49710998060183753]], [[0.58241855005131193, 0.604412405951545], [-0.43975941517157091, 0.20218472702182466]], [[-0.2257267484009271, 0.35309477884304297], [0.30535733178859248, 0.63998611311098386]], [[0.29815046744726392, -0.4864327918438307], [0.65496056346310016, 0.41955651331429822]]])+(1.-msk_arg1)*numpy.array([[[-0.38605355398936614, 0.78789415276076147], [0.89324602443799805, -0.13988696926738986]], [[0.74453111726612797, 0.03448364798448833], [-0.72607655515611658, -0.97274438747404868]], [[0.97026316417312142, 0.0037806894980239036], [0.95950452063026637, -0.021164822335755895]], [[-0.35203247359910783, -0.74465227741444173], [0.7287982213029951, 0.19487117344269222]], [[0.29936219789706842, -0.34653236125488474], [0.89506213449943783, 0.96596505828086432]], [[0.27105399968338606, -0.076712170200837315], [0.73041163689677102, 0.20392573047329665]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[0.70453547216543, -0.62682631245719378], [0.16240992398018328, -0.93357418232171252]], [[-0.66439851208005218, -0.28702503130392953], [-0.36745772150478584, -0.27272767458596392]], [[0.88754891209809683, -0.60696351581127272], [0.68922464624268009, -0.85151003919293977]], [[-0.21533682034437662, -0.65561908253956425], [-0.43975941517157091, 0.20218472702182466]], [[-0.65005710266960848, -0.33540146795034365], [-0.037980818728829968, -0.30364243761179033]], [[0.29815046744726392, -0.4864327918438307], [-0.89170233888417894, -0.060949202001112202]]])+(1.-msk_ref)*numpy.array([[[-0.38605355398936614, 0.67897982415725555], [0.86674048211321231, -0.7715805177911832]], [[0.57397638092698355, -0.46435045979941525], [-0.72607655515611658, -0.97274438747404868]], [[0.52593281930871338, -0.22040994139329739], [0.12885850657281939, -0.021164822335755895]], [[-0.81786035801465506, -0.74465227741444173], [-0.46237380435922737, -0.056518365979367546]], [[0.29936219789706842, -0.34653236125488474], [0.55939816262581243, -0.26197875119045522]], [[0.0072844237960489888, -0.076712170200837315], [-0.40207692212912938, 0.20392573047329665]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.758366898322)+(1.-msk_arg0)*(-0.816907957339)
      arg1=numpy.array([[[[-0.089336930207332133, -0.3488365704255485, -0.75783339384575088, 0.41855694747691374], [0.08692745527314738, 0.40313277705580841, -0.25416741545283172, 0.026893797246153373], [-0.32798951844355484, 0.51029639997479248, -0.2839293463233914, -0.78086184089955268]], [[0.62944259459658491, -0.76427579770952625, 0.81531302514649262, 0.061876143318732035], [0.37762165583704688, 0.20952038685705632, 0.74407804412193745, 0.21476614230824298], [0.71099068148543521, -0.51421400497578773, -0.65582151123549526, 0.061174309790876347]]], [[[0.51374975530413702, -0.4508956749476476, 0.39023088117997751, -0.37827753651300733], [-0.55435950583055438, -0.044318978933388653, -0.99117935367048782, -0.13920995803715375], [-0.80924381779847909, -0.94708607824384528, 0.83447951476568161, 0.84398795898910062]], [[-0.028577073800450714, -0.16455399905080426, 0.020668146918353392, 0.88907591993932011], [0.18505213419131517, 0.16856443645261043, -0.52487575282311805, 0.78255342105905967], [0.15120940626236146, -0.11671747557258083, 0.69155772564727247, 0.8051395342733616]]], [[[0.091550377384957082, 0.5750401476966438, 0.099762805806505206, -0.12024001715072496], [-0.58381802062603549, 0.66699112831631924, 0.0062305482940636647, -0.35721485573505074], [-0.88802900331983858, -0.84609101198132519, 0.4139953198063071, -0.11375810156048205]], [[-0.68954916248695231, 0.32623442994355756, -0.25993997016456682, -0.5582295558308441], [0.52507211113380481, 0.20142225363651134, 0.046540894108333619, -0.050245880461216474], [0.93120134587043113, 0.16805624381737783, -0.82317750595266204, 0.98122107375241496]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.78086184089955268]], [[-0.75836689832230575, -0.76427579770952625, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575]]], [[[-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.99117935367048782, -0.75836689832230575], [-0.80924381779847909, -0.94708607824384528, -0.75836689832230575, -0.75836689832230575]], [[-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575]]], [[[-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.88802900331983858, -0.84609101198132519, -0.75836689832230575, -0.75836689832230575]], [[-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.75836689832230575, -0.75836689832230575], [-0.75836689832230575, -0.75836689832230575, -0.82317750595266204, -0.75836689832230575]]]])+(1.-msk_ref)*numpy.array([[[[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549]], [[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549]]], [[[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.99117935367048782, -0.81690795733923549], [-0.81690795733923549, -0.94708607824384528, -0.81690795733923549, -0.81690795733923549]], [[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549]]], [[[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.88802900331983858, -0.84609101198132519, -0.81690795733923549, -0.81690795733923549]], [[-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.81690795733923549, -0.81690795733923549], [-0.81690795733923549, -0.81690795733923549, -0.82317750595266204, -0.81690795733923549]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_array_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.49606432274244439, -0.12284044252066506, 0.11396427127974107, -0.23891306807966939], [-0.60255849157853825, 0.097766837584110533, 0.99911269434311079, 0.91586023591892651], [-0.53888661521823655, 0.68331848676079954, 0.41233676219180926, -0.031508906318399532]], [[0.049667557510936655, -0.57787470309708544, 0.27191409276622469, -0.76634432702232913], [0.34954140309705539, 0.83629446151104436, 0.8980545228768857, -0.77493605422725254], [-0.45349761024232205, 0.39464559276929512, -0.37779168388472262, 0.48344610692479173]]], [[[0.62542242125573533, -0.76083586876152443, -0.66647782332284722, 0.41050924047271775], [-0.27837179693769332, 0.45619739372057944, -0.85214198840226585, 0.7017477830032397], [-0.7408609838764566, -0.51764398965413672, -0.030243582412935455, 0.75281934842242348]], [[-0.7788969655058573, 0.75281945130855465, 0.3382442492127915, 0.52493679565792006], [-0.78481524699129213, -0.22418514822507762, -0.30086761755155833, 0.19532894016325453], [0.53547505421768959, -0.073834424193617965, -0.99035088191485099, -0.28041462699302677]]], [[[0.98346573898461132, -0.68600843141448697, -0.2663344236460039, -0.75643928230429225], [0.91941455886367041, 0.7760704919158643, -0.82827072579253436, -0.6761780995175477], [0.38532138796094029, -0.38656159426822456, 0.24837951918329226, 0.28428982821685977]], [[0.28366600539943065, 0.92891105091657833, 0.084516164128678017, 0.63344723710600226], [0.62889618468874087, 0.16867908081195115, -0.36401389394896411, -0.45267654295944948], [0.45807884772662488, -0.60800911998886464, 0.99247440611006787, -0.29140746081807567]]]])+(1.-msk_arg0)*numpy.array([[[[0.80306832083438295, 0.45380343230866615, -0.81859733571389515, 0.49222046967396005], [0.38143740035274099, 0.58486130070634368, 0.30403435196519601, -0.36650642825728474], [0.29887879077490775, -0.40845032290228556, -0.5431598285703183, 0.13475156717942238]], [[0.75417450356940652, 0.89249294152043523, -0.40593907203447244, -0.61986107758979259], [0.19160048053597145, 0.342064487447997, 0.27070576607926489, -0.63845009908674477], [0.99735490677681127, -0.07027588824433173, -0.49305637448540329, -0.72580147612898527]]], [[[-0.34547620865449669, 0.83311260564863066, 0.80112622267541678, -0.56300568757995695], [-0.29508013079198014, 0.54309669725669019, -0.29035907077685996, -0.57918385294842811], [-0.36638640241290688, 0.71079186227365532, 0.68408201846508088, 0.098941840697029271]], [[-0.0083776891277580479, -0.66934245974582929, -0.40196203264670105, -0.21809863804953045], [0.7086148720548755, 0.92918702222597904, 0.32724460264843902, 0.084196985144037173], [-0.75061680086990212, 0.90576411892179953, -0.043172076885423127, -0.50670707732414688]]], [[[-0.18429522436961276, -0.98517895664828359, -0.31809447394197532, -0.65766694559763561], [0.10277198103867979, 0.13507862514731039, 0.77391686457442033, 0.84397781207882416], [0.88095026569445523, 0.30627922070338309, 0.071650944921933135, 0.70303512474291341]], [[0.44076384652016931, -0.3696173929940656, -0.18349170631666301, 0.022647439106598943], [-0.6163482466651351, -0.71247536265046474, -0.59464770203407813, -0.65488370258744655], [-0.72810003445735783, 0.126782641507865, -0.10780968972322191, -0.44575083371064461]]]])
      arg1=numpy.array([[[[-0.40646020267158511, -0.94392299321327577, 0.63653845256210473, 0.018123141345770977], [-0.58007359110062051, -0.035141670205105724, -0.56085284601427232, 0.4982263447559756], [0.89111944510230678, -0.6978849999061898, 0.69660183851440616, -0.84311137937700176]], [[0.075304243529505488, -0.37445167860256556, -0.99852100531274823, -0.95361378824481613], [0.59232224338855555, -0.65407829559440844, 0.53093039161054434, -0.32542186245998606], [0.4259892094597022, -0.79571852348154337, 0.57065703382798039, -0.1757223919746298]]], [[[0.34601371301863448, 0.80747948770295519, -0.86511166399201556, -0.74056228089333587], [0.97480864356061359, -0.11752105298069071, 0.75068672969592742, 0.071689919914833755], [0.17921987443575671, 0.61188638379964888, -0.62475728387574914, -0.020086980543206234]], [[0.62270356873520427, -0.7023606282123791, 0.13646179507218359, 0.46457079389295264], [-0.20088755948476478, 0.64693768372202198, 0.28881302912352691, -0.82453766358705738], [-0.11379651691536408, 0.019035208347425225, -0.36105451312547676, 0.989384055038947]]], [[[-0.42022042513581503, 0.22479463994542725, 0.89805854180185252, -0.061610380722787683], [-0.25620732047220529, -0.029958018671728093, -0.78948055498350711, -0.4710653685201871], [-0.89181028184506328, -0.61622699261807412, -0.47912802119186182, -0.55122670074272984]], [[0.96354450197573982, -0.34221151585365006, -0.53914756727179403, -0.79966242791688713], [0.054418810355942426, 0.49431247670409828, -0.8952893167318865, 0.90166797356140105], [-0.96945322437856607, -0.35752184536471265, 0.66612719206177506, 0.24007800416472835]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.40646020267158511, -0.94392299321327577, 0.11396427127974107, -0.23891306807966939], [-0.60255849157853825, -0.035141670205105724, -0.56085284601427232, 0.4982263447559756], [-0.53888661521823655, -0.6978849999061898, 0.41233676219180926, -0.84311137937700176]], [[0.049667557510936655, -0.57787470309708544, -0.99852100531274823, -0.95361378824481613], [0.34954140309705539, -0.65407829559440844, 0.53093039161054434, -0.77493605422725254], [-0.45349761024232205, -0.79571852348154337, -0.37779168388472262, -0.1757223919746298]]], [[[0.34601371301863448, -0.76083586876152443, -0.86511166399201556, -0.74056228089333587], [-0.27837179693769332, -0.11752105298069071, -0.85214198840226585, 0.071689919914833755], [-0.7408609838764566, -0.51764398965413672, -0.62475728387574914, -0.020086980543206234]], [[-0.7788969655058573, -0.7023606282123791, 0.13646179507218359, 0.46457079389295264], [-0.78481524699129213, -0.22418514822507762, -0.30086761755155833, -0.82453766358705738], [-0.11379651691536408, -0.073834424193617965, -0.99035088191485099, -0.28041462699302677]]], [[[-0.42022042513581503, -0.68600843141448697, -0.2663344236460039, -0.75643928230429225], [-0.25620732047220529, -0.029958018671728093, -0.82827072579253436, -0.6761780995175477], [-0.89181028184506328, -0.61622699261807412, -0.47912802119186182, -0.55122670074272984]], [[0.28366600539943065, -0.34221151585365006, -0.53914756727179403, -0.79966242791688713], [0.054418810355942426, 0.16867908081195115, -0.8952893167318865, -0.45267654295944948], [-0.96945322437856607, -0.60800911998886464, 0.66612719206177506, -0.29140746081807567]]]])+(1.-msk_ref)*numpy.array([[[[-0.40646020267158511, -0.94392299321327577, -0.81859733571389515, 0.018123141345770977], [-0.58007359110062051, -0.035141670205105724, -0.56085284601427232, -0.36650642825728474], [0.29887879077490775, -0.6978849999061898, -0.5431598285703183, -0.84311137937700176]], [[0.075304243529505488, -0.37445167860256556, -0.99852100531274823, -0.95361378824481613], [0.19160048053597145, -0.65407829559440844, 0.27070576607926489, -0.63845009908674477], [0.4259892094597022, -0.79571852348154337, -0.49305637448540329, -0.72580147612898527]]], [[[-0.34547620865449669, 0.80747948770295519, -0.86511166399201556, -0.74056228089333587], [-0.29508013079198014, -0.11752105298069071, -0.29035907077685996, -0.57918385294842811], [-0.36638640241290688, 0.61188638379964888, -0.62475728387574914, -0.020086980543206234]], [[-0.0083776891277580479, -0.7023606282123791, -0.40196203264670105, -0.21809863804953045], [-0.20088755948476478, 0.64693768372202198, 0.28881302912352691, -0.82453766358705738], [-0.75061680086990212, 0.019035208347425225, -0.36105451312547676, -0.50670707732414688]]], [[[-0.42022042513581503, -0.98517895664828359, -0.31809447394197532, -0.65766694559763561], [-0.25620732047220529, -0.029958018671728093, -0.78948055498350711, -0.4710653685201871], [-0.89181028184506328, -0.61622699261807412, -0.47912802119186182, -0.55122670074272984]], [[0.44076384652016931, -0.3696173929940656, -0.53914756727179403, -0.79966242791688713], [-0.6163482466651351, -0.71247536265046474, -0.8952893167318865, -0.65488370258744655], [-0.96945322437856607, -0.35752184536471265, -0.10780968972322191, -0.44575083371064461]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.601933950643)+(1.-msk_arg0)*(0.205154176993)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[0.43548460722462612, 0.066997691064892173, 0.74930355563116779, -0.85328470894941222], [-0.99123944712255652, 0.28620324901470151, -0.76197287927399038, -0.69452425443778676], [-0.48366257643746091, -0.53183861133971355, -0.57791305534313042, -0.78569921642875751]], [[0.75855312617769388, -0.6227534016161993, 0.99104716840900586, -0.34351180164469741], [-0.4617108022429619, -0.12043918455102709, 0.53784633906065316, -0.70994815455369631], [0.57298664908617791, -0.79189430466636757, 0.77095038466448695, -0.79328481621815761]]], [[[0.30428659116868784, -0.1672465647261927, 0.3927523697659272, -0.84775230258279821], [-0.89915158651691085, 0.057013556220236294, 0.2650999819760711, 0.88278578903655491], [0.97846072555367192, 0.50605429755841924, 0.16659190317959149, 0.13894529883666218]], [[-0.69994773486339446, -0.63268706304695943, -0.46862520558069476, -0.069476741091367034], [-0.30521998245380133, 0.053166301813267314, 0.11559592316217748, 0.37240181934668271], [0.42414890894182333, 0.072370322417571131, 0.75135844031365306, -0.24136886703150462]]], [[[-0.64838170653307703, 0.2727973934404615, -0.1043017257958343, 0.53358153808248021], [-0.054020825819784735, -0.19121086276848143, 0.76574226438525628, 0.80378847123383346], [0.092989189766207625, -0.13097494533695642, -0.78699523743127742, -0.72532981427404275]], [[-0.06361553543847176, 0.61042906547399278, -0.89290628739228284, -0.29744685756904077], [-0.40777493561718936, -0.065876360611297757, 0.55475259815695077, -0.32566525179616512], [-0.31956645642332804, -0.99059676647867834, -0.19180860354263185, -0.13156313162155153]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[0.43548460722462612, 0.066997691064892173, 0.60193395064346289, -0.85328470894941222], [-0.99123944712255652, 0.28620324901470151, -0.76197287927399038, -0.69452425443778676], [-0.48366257643746091, -0.53183861133971355, -0.57791305534313042, -0.78569921642875751]], [[0.60193395064346289, -0.6227534016161993, 0.60193395064346289, -0.34351180164469741], [-0.4617108022429619, -0.12043918455102709, 0.53784633906065316, -0.70994815455369631], [0.57298664908617791, -0.79189430466636757, 0.60193395064346289, -0.79328481621815761]]], [[[0.30428659116868784, -0.1672465647261927, 0.3927523697659272, -0.84775230258279821], [-0.89915158651691085, 0.057013556220236294, 0.2650999819760711, 0.60193395064346289], [0.60193395064346289, 0.50605429755841924, 0.16659190317959149, 0.13894529883666218]], [[-0.69994773486339446, -0.63268706304695943, -0.46862520558069476, -0.069476741091367034], [-0.30521998245380133, 0.053166301813267314, 0.11559592316217748, 0.37240181934668271], [0.42414890894182333, 0.072370322417571131, 0.60193395064346289, -0.24136886703150462]]], [[[-0.64838170653307703, 0.2727973934404615, -0.1043017257958343, 0.53358153808248021], [-0.054020825819784735, -0.19121086276848143, 0.60193395064346289, 0.60193395064346289], [0.092989189766207625, -0.13097494533695642, -0.78699523743127742, -0.72532981427404275]], [[-0.06361553543847176, 0.60193395064346289, -0.89290628739228284, -0.29744685756904077], [-0.40777493561718936, -0.065876360611297757, 0.55475259815695077, -0.32566525179616512], [-0.31956645642332804, -0.99059676647867834, -0.19180860354263185, -0.13156313162155153]]]])+(1.-msk_ref)*numpy.array([[[[0.20515417699288374, 0.066997691064892173, 0.20515417699288374, -0.85328470894941222], [-0.99123944712255652, 0.20515417699288374, -0.76197287927399038, -0.69452425443778676], [-0.48366257643746091, -0.53183861133971355, -0.57791305534313042, -0.78569921642875751]], [[0.20515417699288374, -0.6227534016161993, 0.20515417699288374, -0.34351180164469741], [-0.4617108022429619, -0.12043918455102709, 0.20515417699288374, -0.70994815455369631], [0.20515417699288374, -0.79189430466636757, 0.20515417699288374, -0.79328481621815761]]], [[[0.20515417699288374, -0.1672465647261927, 0.20515417699288374, -0.84775230258279821], [-0.89915158651691085, 0.057013556220236294, 0.20515417699288374, 0.20515417699288374], [0.20515417699288374, 0.20515417699288374, 0.16659190317959149, 0.13894529883666218]], [[-0.69994773486339446, -0.63268706304695943, -0.46862520558069476, -0.069476741091367034], [-0.30521998245380133, 0.053166301813267314, 0.11559592316217748, 0.20515417699288374], [0.20515417699288374, 0.072370322417571131, 0.20515417699288374, -0.24136886703150462]]], [[[-0.64838170653307703, 0.20515417699288374, -0.1043017257958343, 0.20515417699288374], [-0.054020825819784735, -0.19121086276848143, 0.20515417699288374, 0.20515417699288374], [0.092989189766207625, -0.13097494533695642, -0.78699523743127742, -0.72532981427404275]], [[-0.06361553543847176, 0.20515417699288374, -0.89290628739228284, -0.29744685756904077], [-0.40777493561718936, -0.065876360611297757, 0.20515417699288374, -0.32566525179616512], [-0.31956645642332804, -0.99059676647867834, -0.19180860354263185, -0.13156313162155153]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.42986240064603209, 0.86525916844801642, 0.094903705533901039, 0.9234643790954753], [-0.51666486958850166, -0.063117651748206827, -0.43858474898716016, 0.13324554778679487], [-0.39782602560542135, -0.97693529476347041, -0.16715722151784074, 0.20602394009443459]], [[0.96692144650157386, -0.51514987063564344, -0.65058610049277776, 0.49673570931219468], [0.5537975101259609, 0.31719309623587688, 0.22898406863311993, 0.15162621893638217], [0.059587378896713172, -0.64836398161622877, -0.088859007312100058, -0.63955986732362269]]], [[[0.71268538298492734, -0.69692161061354296, 0.0025366044082435479, -0.22414751849728987], [-0.82655538227315017, 0.94496410326959235, 0.21649067366672159, -0.17332920555681985], [0.73083496418159255, -0.10416492307677494, 0.55837768833957879, -0.11794801753380879]], [[-0.35012584885439879, 0.3651816956758942, -0.51941953613071168, -0.30416977338429763], [0.098538244376603856, -0.74627432179191611, 0.61731578225384598, 0.3337858439539354], [0.55360910687237297, 0.85622411966452194, -0.5274792129410919, -0.71469517662626614]]], [[[-0.049726404909461897, -0.092096909301278007, -0.1581130771802215, -0.58447771731001086], [-0.84925739366824415, -0.092104808665342652, -0.95867441973213063, 0.064677058892707695], [0.56999889522399894, -0.37478276808373301, 0.96404382097085417, -0.79725378150825299]], [[0.61325075891056269, 0.13784075530023165, -0.098252774376548446, -0.1679999112025925], [-0.78467916590277542, -0.45785008771721269, -0.54026137119608908, 0.600621944405626], [0.34467140205795865, 0.6152783280261036, 0.44808586225502123, 0.29559662152836275]]]])+(1.-msk_arg0)*numpy.array([[[[-0.77078969341947223, -0.10622855953164612, 0.7602092438387491, -0.62490523380901997], [0.85693806223336844, -0.64462624663083123, -0.026230771478372716, -0.10232707469915869], [0.75736018093892365, -0.041507700706797213, -0.27034894370965357, 0.19881059574133841]], [[-0.12312770249842253, 0.61877257034774757, -0.15965664903489585, 0.77928338151462695], [0.41931628445869329, -0.45598014086947569, -0.13953320304468431, 0.36532030908820201], [-0.26921071221922444, 0.77440282806827043, -0.95417925042355245, 0.8340172223024811]]], [[[0.40978254953795679, -0.020237614365529843, 0.8685724207795209, 0.69179849283311179], [-0.86723821682993352, 0.52712076011416098, 0.62746962389073535, 0.03025195740988762], [-0.22304094308960609, -0.77384764423383001, -0.055913257749240497, -0.4596016368270146]], [[-0.52708763115339297, 0.635252503045169, 0.35142677461668459, -0.57832410339393259], [-0.55381796512233961, 0.35727572419499332, -0.60693497923891182, -0.033459788966754722], [0.74867688695939738, 0.56100131895256289, -0.15265112556355076, -0.6524143271358458]]], [[[0.14105850794168906, -0.73553645888122299, -0.14745125329023123, -0.72654092973880569], [-0.15633192820893727, -0.80988484684352202, -0.85459703429019185, -0.059797604723019759], [0.92996263398065526, 0.55457356750257225, -0.69008062905103995, 0.54222206617935509]], [[-0.74650467452200697, 0.083769818397442464, -0.9725921667528159, 0.62128547480092777], [-0.7852030499122995, 0.21682826535645683, -0.48725934169117169, -0.59218988379492332], [0.1240233135801454, -0.63353971005458587, -0.22359072524046675, -0.8985099110825685]]]])
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=minimum(arg0,arg1)
      s1=numpy.array([[[[0.58594087089960389, 0.8529298296380714, 0.31943867033532403, -0.92218828514607099], [0.080385285293333331, -0.79525944407654614, 0.30610772683601373, -0.98222983772034422], [0.088647240867722576, 0.7380503982178972, -0.41330488740048699, -0.10228622213653193]], [[0.33639052631831046, -0.91217505324026771, 0.92345866731237503, -0.71625128230230328], [-0.89184626079843032, -0.52076563483123062, -0.64138939488896751, -0.33793494773193644], [-0.064339513096321577, -0.70568769554733901, 0.23262906739334044, 0.27461686067168101]]], [[[-0.31042932971486059, 0.54600937448254139, 0.88976034705360219, 0.14811022832910448], [-0.19894195401013204, -0.28784680643607152, 0.69523246866521937, 0.013489098725809168], [0.45598506161210328, -0.86842449583834913, -0.49051282169496102, 0.3940256751351574]], [[-0.872777132401787, 0.5231551994101955, 0.10865673664739761, -0.3250125010624445], [0.64755108050343213, -0.58469372474879067, 0.93697003002595602, -0.35426963599279837], [0.52648270661856111, 0.15061060879359078, 0.98710782859935509, 0.93487985818366637]]], [[[-0.047764095288784558, 0.61090218381136552, 0.91706192737596059, -0.26032527986140641], [-0.33251511296742953, 0.58626053543453316, 0.56591727122712698, 0.51407826051273053], [0.40554211941619456, 0.46263741420461124, -0.28261240023098777, -0.72767183369249921]], [[0.29491639862233532, 0.66327909005331631, 0.40044279392285342, -0.38397983343368347], [-0.88478103112317319, 0.9595925491569639, 0.59434659243498267, 0.82639408330585984], [-0.91279563370190453, 0.39762039678657302, -0.29514731756234958, 0.36613681146552191]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.42986240064603209, 0.8529298296380714, 0.094903705533901039, -0.92218828514607099], [-0.51666486958850166, -0.79525944407654614, -0.43858474898716016, -0.98222983772034422], [-0.39782602560542135, -0.97693529476347041, -0.41330488740048699, -0.10228622213653193]], [[0.33639052631831046, -0.91217505324026771, -0.65058610049277776, -0.71625128230230328], [-0.89184626079843032, -0.52076563483123062, -0.64138939488896751, -0.33793494773193644], [-0.064339513096321577, -0.70568769554733901, -0.088859007312100058, -0.63955986732362269]]], [[[-0.31042932971486059, -0.69692161061354296, 0.0025366044082435479, -0.22414751849728987], [-0.82655538227315017, -0.28784680643607152, 0.21649067366672159, -0.17332920555681985], [0.45598506161210328, -0.86842449583834913, -0.49051282169496102, -0.11794801753380879]], [[-0.872777132401787, 0.3651816956758942, -0.51941953613071168, -0.3250125010624445], [0.098538244376603856, -0.74627432179191611, 0.61731578225384598, -0.35426963599279837], [0.52648270661856111, 0.15061060879359078, -0.5274792129410919, -0.71469517662626614]]], [[[-0.049726404909461897, -0.092096909301278007, -0.1581130771802215, -0.58447771731001086], [-0.84925739366824415, -0.092104808665342652, -0.95867441973213063, 0.064677058892707695], [0.40554211941619456, -0.37478276808373301, -0.28261240023098777, -0.79725378150825299]], [[0.29491639862233532, 0.13784075530023165, -0.098252774376548446, -0.38397983343368347], [-0.88478103112317319, -0.45785008771721269, -0.54026137119608908, 0.600621944405626], [-0.91279563370190453, 0.39762039678657302, -0.29514731756234958, 0.29559662152836275]]]])+(1.-msk_ref)*numpy.array([[[[-0.77078969341947223, -0.10622855953164612, 0.31943867033532403, -0.92218828514607099], [0.080385285293333331, -0.79525944407654614, -0.026230771478372716, -0.98222983772034422], [0.088647240867722576, -0.041507700706797213, -0.41330488740048699, -0.10228622213653193]], [[-0.12312770249842253, -0.91217505324026771, -0.15965664903489585, -0.71625128230230328], [-0.89184626079843032, -0.52076563483123062, -0.64138939488896751, -0.33793494773193644], [-0.26921071221922444, -0.70568769554733901, -0.95417925042355245, 0.27461686067168101]]], [[[-0.31042932971486059, -0.020237614365529843, 0.8685724207795209, 0.14811022832910448], [-0.86723821682993352, -0.28784680643607152, 0.62746962389073535, 0.013489098725809168], [-0.22304094308960609, -0.86842449583834913, -0.49051282169496102, -0.4596016368270146]], [[-0.872777132401787, 0.5231551994101955, 0.10865673664739761, -0.57832410339393259], [-0.55381796512233961, -0.58469372474879067, -0.60693497923891182, -0.35426963599279837], [0.52648270661856111, 0.15061060879359078, -0.15265112556355076, -0.6524143271358458]]], [[[-0.047764095288784558, -0.73553645888122299, -0.14745125329023123, -0.72654092973880569], [-0.33251511296742953, -0.80988484684352202, -0.85459703429019185, -0.059797604723019759], [0.40554211941619456, 0.46263741420461124, -0.69008062905103995, -0.72767183369249921]], [[-0.74650467452200697, 0.083769818397442464, -0.9725921667528159, -0.38397983343368347], [-0.88478103112317319, 0.21682826535645683, -0.48725934169117169, -0.59218988379492332], [-0.91279563370190453, -0.63353971005458587, -0.29514731756234958, -0.8985099110825685]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(0.915014570808)+(1.-msk_arg0)*(0.00693309937149)
      arg1=Data(numpy.array([[[[-0.024616493564646014, 0.73986506071275615, 0.40045069363770303, -0.349587964480369], [0.096004295226797431, 0.49415258135071571, -0.25174172967066832, 0.54249134677714772], [-0.87581721391988365, 0.10847159986262644, -0.3688371682325351, 0.30383154728572248]], [[-0.84813991148188839, 0.92539057923751389, -0.04808129585241594, -0.76044944158127881], [-0.37956815177630943, -0.0021700374885920137, 0.12804567444566484, -0.84584319411046383], [-0.66698396570131679, -0.86375301071471444, 0.6266795734119297, 0.72969896277602908]]], [[[0.37505603318395497, 0.15695183669002444, -0.50506507527916145, 0.78544822702085604], [0.43781850470970518, 0.82020434557550126, -0.67497282695188909, 0.57574879486380559], [0.49745792521920018, 0.41403102534370628, 0.19334686956486169, -0.094285235742264328]], [[0.91078143137671685, -0.64662819523078952, -0.49160107656731356, 0.21244530928296457], [-0.12892029937078608, 0.0065983102969269236, -0.72815551209722429, -0.87074291789406821], [-0.33951262082135991, -0.28909708798195943, 0.78297670060616809, -0.1813889085841629]]], [[[0.15302183077978571, 0.9821284263110488, 0.79282893507840635, 0.64017446726074545], [0.83261679997442761, 0.34583690439257486, -0.87367948663003148, 0.013024284133244635], [-0.55472238642899518, -0.98196829116013173, 0.34590028258804728, 0.56193507184117908]], [[-0.91806403728542074, 0.46759895278200192, -0.22691484440602205, -0.1360765618692974], [-0.2630245734727592, 0.89706296333461988, -0.92618670589312568, 0.56460258310291511], [-0.63907522579916498, 0.99651590418151725, -0.47525732685460365, -0.19631796941274882]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.024616493564646014, 0.73986506071275615, 0.40045069363770303, -0.349587964480369], [0.096004295226797431, 0.49415258135071571, -0.25174172967066832, 0.54249134677714772], [-0.87581721391988365, 0.10847159986262644, -0.3688371682325351, 0.30383154728572248]], [[-0.84813991148188839, 0.91501457080784343, -0.04808129585241594, -0.76044944158127881], [-0.37956815177630943, -0.0021700374885920137, 0.12804567444566484, -0.84584319411046383], [-0.66698396570131679, -0.86375301071471444, 0.6266795734119297, 0.72969896277602908]]], [[[0.37505603318395497, 0.15695183669002444, -0.50506507527916145, 0.78544822702085604], [0.43781850470970518, 0.82020434557550126, -0.67497282695188909, 0.57574879486380559], [0.49745792521920018, 0.41403102534370628, 0.19334686956486169, -0.094285235742264328]], [[0.91078143137671685, -0.64662819523078952, -0.49160107656731356, 0.21244530928296457], [-0.12892029937078608, 0.0065983102969269236, -0.72815551209722429, -0.87074291789406821], [-0.33951262082135991, -0.28909708798195943, 0.78297670060616809, -0.1813889085841629]]], [[[0.15302183077978571, 0.91501457080784343, 0.79282893507840635, 0.64017446726074545], [0.83261679997442761, 0.34583690439257486, -0.87367948663003148, 0.013024284133244635], [-0.55472238642899518, -0.98196829116013173, 0.34590028258804728, 0.56193507184117908]], [[-0.91806403728542074, 0.46759895278200192, -0.22691484440602205, -0.1360765618692974], [-0.2630245734727592, 0.89706296333461988, -0.92618670589312568, 0.56460258310291511], [-0.63907522579916498, 0.91501457080784343, -0.47525732685460365, -0.19631796941274882]]]])+(1.-msk_ref)*numpy.array([[[[-0.024616493564646014, 0.0069330993714913358, 0.0069330993714913358, -0.349587964480369], [0.0069330993714913358, 0.0069330993714913358, -0.25174172967066832, 0.0069330993714913358], [-0.87581721391988365, 0.0069330993714913358, -0.3688371682325351, 0.0069330993714913358]], [[-0.84813991148188839, 0.0069330993714913358, -0.04808129585241594, -0.76044944158127881], [-0.37956815177630943, -0.0021700374885920137, 0.0069330993714913358, -0.84584319411046383], [-0.66698396570131679, -0.86375301071471444, 0.0069330993714913358, 0.0069330993714913358]]], [[[0.0069330993714913358, 0.0069330993714913358, -0.50506507527916145, 0.0069330993714913358], [0.0069330993714913358, 0.0069330993714913358, -0.67497282695188909, 0.0069330993714913358], [0.0069330993714913358, 0.0069330993714913358, 0.0069330993714913358, -0.094285235742264328]], [[0.0069330993714913358, -0.64662819523078952, -0.49160107656731356, 0.0069330993714913358], [-0.12892029937078608, 0.0065983102969269236, -0.72815551209722429, -0.87074291789406821], [-0.33951262082135991, -0.28909708798195943, 0.0069330993714913358, -0.1813889085841629]]], [[[0.0069330993714913358, 0.0069330993714913358, 0.0069330993714913358, 0.0069330993714913358], [0.0069330993714913358, 0.0069330993714913358, -0.87367948663003148, 0.0069330993714913358], [-0.55472238642899518, -0.98196829116013173, 0.0069330993714913358, 0.0069330993714913358]], [[-0.91806403728542074, 0.0069330993714913358, -0.22691484440602205, -0.1360765618692974], [-0.2630245734727592, 0.0069330993714913358, -0.92618670589312568, 0.0069330993714913358], [-0.63907522579916498, 0.0069330993714913358, -0.47525732685460365, -0.19631796941274882]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_constData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[-0.34701240595496552, -0.066590066066970044, 0.68893642710771608, -0.17424522506882512], [0.024355088381233791, 0.05633425284930893, 0.88879426240819681, -0.92863593547037859], [0.83669776930697304, -0.39579600459575404, 0.47192813756614016, 0.99172878988195778]], [[-0.91551508139749727, 0.68326775998294154, -0.89927562350378354, 0.57943460559444593], [0.44460535034711812, 0.53502178865053773, 0.14278367326090602, -0.58322836856114124], [0.1419481361577406, -0.41584721501837718, -0.34887760440988136, -0.82956487366913989]]], [[[0.43035841120515195, -0.33252375929641009, -0.067318115758067121, 0.84803550650774739], [-0.62015844837686718, 0.38772843909572074, 0.64565778860413992, 0.89579458002448398], [0.34136535785486344, -0.050471545430222475, 0.81193883786827192, 0.53984856950978832]], [[-0.27538461578377293, 0.084308717068411809, -0.11883741328977337, -0.85222725709749736], [-0.63763814472991376, 0.059471248494016082, 0.73094946640875458, -0.13046786055282689], [0.69664180936494846, -0.42460910739161783, 0.52027742705763869, -0.76923727859653113]]], [[[-0.73466423193399621, -0.011639215664459579, 0.95320371554189598, 0.62649241111665455], [-0.25811374588837577, 0.61082264991855828, -0.24422765522426393, 0.32506948963370585], [0.20347634345487453, 0.97972000096265566, 0.60645379455271597, 0.8589946844017029]], [[-0.041699531836799197, 0.88375542257324491, 0.67422122172347598, -0.054675511226085494], [0.65426125902067622, 0.45574875234198231, -0.3229715640720443, 0.28227990718457141], [0.26764386074170599, 0.67423488792039055, 0.30435538461696798, 0.093625119111805377]]]])+(1.-msk_arg0)*numpy.array([[[[0.99213262869399821, 0.47146052239570513, -0.39451256286827108, -0.12052296338715451], [0.30982502841033499, 0.91165464856342027, 0.69003585651378629, 0.58749700860257104], [0.13762751362008552, -0.36280676873926487, 0.44729807825768031, 0.034791000996944543]], [[-0.67726750805352709, -0.68750211316933463, -0.90630896884080681, -0.89677743485238182], [0.98934429876382213, -0.046251856573284122, 0.7758619897125183, 0.4195696059348375], [0.16092756713974654, 0.18650696278593371, 0.13872078227177354, 0.36678306618403123]]], [[[0.3335529154575263, -0.7349642744095326, -0.49715646607759001, 0.63213686477707398], [-0.6617700431351845, -0.20753978747261148, 0.97303348087248631, -0.96573015784394056], [-0.51630494384942915, -0.99455947865587224, 0.12992473826799666, -0.97846484067113382]], [[-0.8046496501788245, -0.88598910169886835, 0.25950100249661379, 0.68437657086603343], [0.40057865795459313, 0.41091540387964853, 0.92851419973982474, 0.4518258714544543], [0.48341844439544968, -0.17590981446341858, 0.5237804617238333, -0.95206524255248937]]], [[[-0.19617319531924671, 0.98343738990959761, -0.18690278454110754, -0.88243885345113027], [0.37831438577629273, -0.34195429513329212, -0.2069020831166215, 0.45040781392162299], [-0.88278705514420563, -0.71402473105050923, 0.89699923794244452, 0.17595933142076903]], [[0.7193053321972116, -0.85703390263672685, -0.83884713109593068, 0.65941444232361102], [0.38261507706130127, -0.71870105969512688, 0.6223634029597902, -0.7273911403112685], [-0.86420470101157987, -0.10232739528406842, -0.78869794636170787, 0.79418519099703522]]]])
      arg1=Data(numpy.array([[[[-0.59070702028860333, 0.47242671894939003, -0.99417581760700324, 0.25914160458610813], [-0.43354412582670943, -0.76322412365052394, 0.13514439555568769, 0.39346038940016315], [0.32243253754739754, 0.31773533228571216, -0.59218535240200554, 0.50303263671827736]], [[0.89049748904122983, 0.17458866658741901, 0.43564362337626239, -0.72584448709673288], [0.87461640678970753, 0.79807509864035886, 0.51505568184861983, -0.53337386976464263], [-0.4396377432223153, 0.99980566879211419, 0.32344354369863737, 0.075629494773711059]]], [[[0.77629624350318815, -0.097059670735997416, -0.21145608563375506, 0.2166695317184284], [0.82464631495795238, 0.75425365270193701, -0.29174907876180001, 0.60917873956124002], [0.21721508803541867, 0.28524117459309739, -0.59985368595416277, 0.11934295191090305]], [[-0.91434634851775698, 0.3617103235489485, 0.78303293404161955, -0.11004975165444675], [-0.89770389762359248, 0.60492008175175105, -0.77381993927717874, 0.57716284532775708], [-0.27910751265213629, 0.95374373631541665, -0.4433049708535739, -0.66837888038023663]]], [[[0.42163029229014648, -0.43572007771569798, 0.22863778408918156, 0.045561253608975338], [0.86892968183383967, -0.35152860041100142, -0.030501286783599202, -0.8628259117493855], [0.45870319970757767, -0.37563694895442001, -0.64243884527119111, -0.31114603092393911]], [[-0.60575845350360313, -0.26221713284580184, 0.95348527655101001, 0.96594382139883628], [0.19168896809532643, -0.26791303190916049, -0.72106038473599887, -0.71790845704998341], [-0.79579865588188592, 0.38819717867276826, 0.47952089305368006, 0.87339431832908687]]]]),self.functionspace)
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.59070702028860333, -0.066590066066970044, -0.99417581760700324, -0.17424522506882512], [-0.43354412582670943, -0.76322412365052394, 0.13514439555568769, -0.92863593547037859], [0.32243253754739754, -0.39579600459575404, -0.59218535240200554, 0.50303263671827736]], [[-0.91551508139749727, 0.17458866658741901, -0.89927562350378354, -0.72584448709673288], [0.44460535034711812, 0.53502178865053773, 0.14278367326090602, -0.58322836856114124], [-0.4396377432223153, -0.41584721501837718, -0.34887760440988136, -0.82956487366913989]]], [[[0.43035841120515195, -0.33252375929641009, -0.21145608563375506, 0.2166695317184284], [-0.62015844837686718, 0.38772843909572074, -0.29174907876180001, 0.60917873956124002], [0.21721508803541867, -0.050471545430222475, -0.59985368595416277, 0.11934295191090305]], [[-0.91434634851775698, 0.084308717068411809, -0.11883741328977337, -0.85222725709749736], [-0.89770389762359248, 0.059471248494016082, -0.77381993927717874, -0.13046786055282689], [-0.27910751265213629, -0.42460910739161783, -0.4433049708535739, -0.76923727859653113]]], [[[-0.73466423193399621, -0.43572007771569798, 0.22863778408918156, 0.045561253608975338], [-0.25811374588837577, -0.35152860041100142, -0.24422765522426393, -0.8628259117493855], [0.20347634345487453, -0.37563694895442001, -0.64243884527119111, -0.31114603092393911]], [[-0.60575845350360313, -0.26221713284580184, 0.67422122172347598, -0.054675511226085494], [0.19168896809532643, -0.26791303190916049, -0.72106038473599887, -0.71790845704998341], [-0.79579865588188592, 0.38819717867276826, 0.30435538461696798, 0.093625119111805377]]]])+(1.-msk_ref)*numpy.array([[[[-0.59070702028860333, 0.47146052239570513, -0.99417581760700324, -0.12052296338715451], [-0.43354412582670943, -0.76322412365052394, 0.13514439555568769, 0.39346038940016315], [0.13762751362008552, -0.36280676873926487, -0.59218535240200554, 0.034791000996944543]], [[-0.67726750805352709, -0.68750211316933463, -0.90630896884080681, -0.89677743485238182], [0.87461640678970753, -0.046251856573284122, 0.51505568184861983, -0.53337386976464263], [-0.4396377432223153, 0.18650696278593371, 0.13872078227177354, 0.075629494773711059]]], [[[0.3335529154575263, -0.7349642744095326, -0.49715646607759001, 0.2166695317184284], [-0.6617700431351845, -0.20753978747261148, -0.29174907876180001, -0.96573015784394056], [-0.51630494384942915, -0.99455947865587224, -0.59985368595416277, -0.97846484067113382]], [[-0.91434634851775698, -0.88598910169886835, 0.25950100249661379, -0.11004975165444675], [-0.89770389762359248, 0.41091540387964853, -0.77381993927717874, 0.4518258714544543], [-0.27910751265213629, -0.17590981446341858, -0.4433049708535739, -0.95206524255248937]]], [[[-0.19617319531924671, -0.43572007771569798, -0.18690278454110754, -0.88243885345113027], [0.37831438577629273, -0.35152860041100142, -0.2069020831166215, -0.8628259117493855], [-0.88278705514420563, -0.71402473105050923, -0.64243884527119111, -0.31114603092393911]], [[-0.60575845350360313, -0.85703390263672685, -0.83884713109593068, 0.65941444232361102], [0.19168896809532643, -0.71870105969512688, -0.72106038473599887, -0.7273911403112685], [-0.86420470101157987, -0.10232739528406842, -0.78869794636170787, 0.79418519099703522]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank0_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*(-0.281440687124)+(1.-msk_arg0)*(-0.35596270117)
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.35379270015815001, 0.39138893129730512, 0.17268454564435354, 0.27495962726829037], [-0.42478195929874141, 0.81114791137829712, 0.49891790941554914, -0.84889097868629948], [-0.80354923791611466, -0.99875111821489648, -0.15295293690319189, -0.23948710705684717]], [[0.7805413687322833, -0.98188721215648256, 0.067676870968989888, -0.81348894976268782], [0.51781452911506731, -0.68957839475146754, -0.35693520061661244, 0.50816586186118617], [-0.037127168597973492, 0.87527048404958374, 0.3907745056832348, 0.3424864780213035]]], [[[-0.74587202908015726, -0.91209200550772151, -0.45967987413130373, 0.47364848085989331], [0.55813495312028105, 0.69155459403636499, 0.57371625967095441, 0.30866830662451483], [0.67024978892614429, 0.62918201641780858, -0.29910907170343304, 0.72463645868347082]], [[-0.13317748932441864, -0.9204063014033288, 0.75421236332449348, -0.64526129483169914], [0.16143967775323942, -0.50063348653036743, -0.0032020169286099964, -0.098411289042768857], [0.79703082376106371, -0.8478212917704453, 0.25159317951323068, 0.96570013411217848]]], [[[-0.67936202530106371, 0.65848615916060815, 0.0342330141319378, 0.1830705561479844], [0.59696897864236109, -0.90892311891528177, -0.85226027785125735, -0.094758937987674985], [0.046843830140058218, -0.46623548865208053, 0.95940015221237096, -0.074574115874788127]], [[-0.35689244415995436, 0.9386540295859791, 0.98024546961490633, 0.66425792601153422], [0.32479805893954339, 0.18451300545005034, -0.17544017380326538, -0.52561231707548273], [-0.3534490724726933, -0.12495276741935912, -0.35564363665390397, -0.73327365076131912]]]])+(1.-msk_arg1)*numpy.array([[[[0.27368905393342446, 0.51017421645331118, -0.42724661619890125, 0.28046826996439989], [0.40050816583205218, -0.46169248751741998, -0.84630948383699978, -0.7238308756095988], [-0.5900363110555269, 0.19871173292364341, 0.0032822419523403923, -0.6289290678175401]], [[0.55188643485228916, 0.53505012324679879, -0.10508508660536475, 0.86026174516162857], [-0.65932929728702394, 0.03617752320813894, -0.66465550799693784, -0.070428965950132483], [0.94883643499055936, 0.82888540594303439, 0.65244299870764255, -0.86131623082268183]]], [[[-0.63797413232474165, 0.46987977398290481, 0.127332908906991, 0.11442327661551732], [0.10848855516345557, 0.45352945434542136, 0.4833214761528497, 0.73064138639748299], [-0.73347934704847217, -0.49295671859578061, -0.31440523153896116, -0.7835557756250735]], [[0.41901609701772569, -0.44948463644971337, -0.24302629954154664, -0.023138045232491855], [0.733124095901178, -0.56727159154999107, -0.019172829153542015, -0.24666017927879302], [-0.17010065342109293, 0.074460303015782214, -0.31188620889066887, -0.56017651115588873]]], [[[-0.62539591562958363, 0.62376910063026658, 0.84867780557098604, 0.66461784742102359], [-0.17708382487177809, 0.44481741630262528, 0.34179971179574986, 0.97256706794332581], [0.9287135448908217, -0.4095713814718418, 0.79776224226010961, 0.3794798609396457]], [[0.53675571412073686, 0.51971846190698212, 0.9097391503559551, 0.80384587348141245], [0.70665590624068031, -0.98930795849510522, -0.47071997860101367, 0.80774878578135545], [0.89959467660483683, 0.88160223458711506, 0.82429762924456917, 0.032127226351953198]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.35379270015815001, -0.28144068712411241, -0.28144068712411241, -0.28144068712411241], [-0.42478195929874141, -0.28144068712411241, -0.28144068712411241, -0.84889097868629948], [-0.80354923791611466, -0.99875111821489648, -0.28144068712411241, -0.28144068712411241]], [[-0.28144068712411241, -0.98188721215648256, -0.28144068712411241, -0.81348894976268782], [-0.28144068712411241, -0.68957839475146754, -0.35693520061661244, -0.28144068712411241], [-0.28144068712411241, -0.28144068712411241, -0.28144068712411241, -0.28144068712411241]]], [[[-0.74587202908015726, -0.91209200550772151, -0.45967987413130373, -0.28144068712411241], [-0.28144068712411241, -0.28144068712411241, -0.28144068712411241, -0.28144068712411241], [-0.28144068712411241, -0.28144068712411241, -0.29910907170343304, -0.28144068712411241]], [[-0.28144068712411241, -0.9204063014033288, -0.28144068712411241, -0.64526129483169914], [-0.28144068712411241, -0.50063348653036743, -0.28144068712411241, -0.28144068712411241], [-0.28144068712411241, -0.8478212917704453, -0.28144068712411241, -0.28144068712411241]]], [[[-0.67936202530106371, -0.28144068712411241, -0.28144068712411241, -0.28144068712411241], [-0.28144068712411241, -0.90892311891528177, -0.85226027785125735, -0.28144068712411241], [-0.28144068712411241, -0.46623548865208053, -0.28144068712411241, -0.28144068712411241]], [[-0.35689244415995436, -0.28144068712411241, -0.28144068712411241, -0.28144068712411241], [-0.28144068712411241, -0.28144068712411241, -0.28144068712411241, -0.52561231707548273], [-0.3534490724726933, -0.28144068712411241, -0.35564363665390397, -0.73327365076131912]]]])+(1.-msk_ref)*numpy.array([[[[-0.35596270116966267, -0.35596270116966267, -0.42724661619890125, -0.35596270116966267], [-0.35596270116966267, -0.46169248751741998, -0.84630948383699978, -0.7238308756095988], [-0.5900363110555269, -0.35596270116966267, -0.35596270116966267, -0.6289290678175401]], [[-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.65932929728702394, -0.35596270116966267, -0.66465550799693784, -0.35596270116966267], [-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.86131623082268183]]], [[[-0.63797413232474165, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.73347934704847217, -0.49295671859578061, -0.35596270116966267, -0.7835557756250735]], [[-0.35596270116966267, -0.44948463644971337, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.56727159154999107, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.56017651115588873]]], [[[-0.62539591562958363, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.4095713814718418, -0.35596270116966267, -0.35596270116966267]], [[-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267], [-0.35596270116966267, -0.98930795849510522, -0.47071997860101367, -0.35596270116966267], [-0.35596270116966267, -0.35596270116966267, -0.35596270116966267, -0.35596270116966267]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_minimum_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=whereNegative(self.functionspace.getX()[0]-0.5)
      arg0=msk_arg0*numpy.array([[[[0.16571615363407544, -0.15953462441407429, -0.68043226043641081, -0.87681401094494094], [-0.9695983155906287, -0.15364919368303798, 0.030160758861827119, -0.92415648368275738], [0.82219196541332606, 0.95377813518040711, 0.19171337543821187, 0.10581899828603314]], [[-0.88867133240432428, 0.52390121671003653, 0.27634854506335915, -0.83253250212204444], [0.68836708148447934, -0.944819458613956, -0.65875103578172434, 0.75220212840386891], [0.43162549126481475, -0.89619209121093757, 0.30709465760721777, -0.58977699061810096]]], [[[-0.95992219762575615, -0.066575866315797372, -0.39911206297446955, 0.47436127044029619], [-0.89976461972455346, 0.31866787036320132, 0.72722010061752695, -0.23250235959439425], [0.30897331797821015, 0.69432613471155391, -0.52945320471416024, -0.4294587659101694]], [[0.046053965984596434, -0.33603355979512473, -0.45371629141797443, 0.53619070384652145], [0.17824930535811601, -0.41289723392230493, 0.82622334829024346, -0.62092390819188248], [0.13528973033119573, 0.65860517582594169, -0.59994769111313673, 0.86347249694863781]]], [[[0.062965273551715883, 0.3984423920467417, 0.8412369286949386, 0.48653611314147449], [0.076552434980462714, -0.74637079543318352, -0.93111256601427783, 0.24019128438975446], [0.54932891961257702, -0.36479323325406687, -0.36227281864741689, -0.6426381946129549]], [[0.87535402737995538, -0.22622773167577637, 0.53698671630387973, -0.015222102108113145], [-0.23230268554196098, -0.59313355157251668, -0.27386335124317984, 0.40869078283118898], [0.49847726683133531, 0.30018332336192199, -0.89742548476289596, 0.91923945312301303]]]])+(1.-msk_arg0)*numpy.array([[[[-0.98224484945839463, 0.7590398139904484, 0.90218789449350356, 0.794570940224119], [0.065804782544962137, -0.43702907186543016, -0.97674631897757735, 0.32386280340723728], [0.52537049868532804, -0.91155503921923597, 0.97099132032305335, -0.04553695938314517]], [[0.98725809756636784, -0.33391372704158195, -0.63493116997531973, -0.058358115695967161], [0.62008950317384048, -0.19140831812052861, 0.87762021206781871, -0.35590907029535046], [-0.69105597053150425, -0.94527231725182959, -0.8102802593916536, -0.40046146143927785]]], [[[0.23812505816617868, 0.67420998385603159, 0.60705754082266328, -0.69124361116174593], [0.24845222081689045, 0.39236925649498011, 0.84439909529847723, 0.19820436834791244], [-0.97817921648061867, 0.7658360887578135, 0.91830949704569953, 0.69788275060624128]], [[0.86705579380032538, 0.50278661317081963, -0.65318848328350376, 0.87473434196921507], [-0.95695137577762912, -0.47131199534772339, 0.52066934994631886, 0.68123222673178385], [0.08060465442413145, -0.71465794226684465, -0.12366819719187494, 0.21266832142581893]]], [[[0.26114660171599891, 0.86748987236737696, 0.33557193919758266, -0.77240321798355382], [-0.55175199625157334, 0.73517151916955004, 0.44786064200667841, -0.028816980538964643], [0.17183266111472473, -0.36898894101335911, 0.036378238609670488, 0.17012681736007407]], [[-0.30590119814148453, 0.4568227076734801, 0.36372090687649461, -0.32049948781332871], [-0.2367582269308024, 0.38591976993568977, 0.95772294509922817, 0.20975307783256003], [-0.18061661907409143, 0.56011139300635548, -0.85781971479507502, 0.51683029807570202]]]])
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*numpy.array([[[[-0.25546937210977871, 0.24832683088632401, -0.14545464691490784, 0.080235186525601687], [0.11069137789143491, -0.016207251388733024, 0.77248522205512682, 0.7916830124200489], [0.24188876407985682, -0.37726568082801926, -0.031244555386071182, 0.23719843766370108]], [[-0.52530895845960357, -0.040442550882596295, -0.96671783461884542, -0.31844929396282939], [0.63028509118943798, -0.82987066268733845, 0.40226273254697853, 0.6416560128654627], [-0.51768106525739577, -0.95314812231221246, 0.91646994008831739, 0.10500079998103473]]], [[[-0.43762630469563502, 0.88953292259189332, 0.20989650270724391, -0.2252897487980432], [0.22990139512492824, 0.3938910226926513, -0.89967417979417186, -0.20081505765042973], [0.51252688708846761, -0.63589317933525402, 0.43780388779762469, -0.8219515227692169]], [[0.45878322925691384, 0.65986650945619796, 0.86195023375372393, 0.92947598086978256], [0.74115675870795483, -0.3216339775383199, 0.46769531478658299, -0.22253987779723783], [0.16061925701180435, 0.5771814412103291, -0.079603474945781683, -0.71267492351189787]]], [[[0.17583785687054077, 0.59324083913910819, 0.64345037699888441, 0.30816424930249453], [-0.62579483160181737, 0.22078302970647767, 0.32405747132008256, 0.87513873105183126], [-0.093324441995394247, 0.54868508307347397, -0.56595771086234792, -0.34881472760328158]], [[0.52000927142362841, -0.39276100837897543, -0.20324203866299317, -0.63660389564486386], [-0.88829184611129675, 0.1589791241084153, 0.030944332643213501, -0.61769836226507069], [-0.59576399350712039, 0.32459356457722199, -0.96040815278083791, 0.973733761332366]]]])+(1.-msk_arg1)*numpy.array([[[[0.72410480548498746, 0.85139119034874633, 0.60917416945078062, -0.95323666037912491], [0.40303015222384286, 0.83523232097517108, -0.87374826353837798, 0.022982393583445671], [-0.89361829233112866, -0.94599386362372728, -0.93231635029418403, 0.34529834386005431]], [[-0.16748037734067323, 0.96004226924202563, -0.42974360142484525, -0.90439036915169324], [0.25068095981494043, 0.1834607986753285, 0.35750719638369866, -0.071025360246698588], [-0.019474311003289335, 0.60750499913984646, 0.5413467925025135, -0.73372966368662751]]], [[[0.64667907904540933, -0.88151193083904777, 0.095207905980526197, 0.43753183700227116], [0.14858701799370544, 0.093928805343780786, -0.95743778636458687, -0.14774514961802643], [0.45962714702945218, 0.99081133934136334, -0.79683179728428799, 0.043961512901157684]], [[0.35756731123310392, -0.48264940908142151, 0.099071925139986972, 0.96376061087137077], [-0.66314226659986986, -0.052455192302455655, -0.54481953495951108, -0.44645464432144211], [-0.43854372305478373, -0.35203765642805251, 0.59303138116541465, 0.053016146326937852]]], [[[0.9701121331518916, -0.73394722395651057, 0.021272086611772822, -0.53271581135173629], [0.72587623540675916, 0.27049855862575178, -0.42218196424565946, -0.68367437973492895], [0.56588978128030054, -0.12810113550265623, 0.49704499320239148, 0.16474024394444542]], [[-0.39535356597941762, -0.88290828211650996, -0.21519102192100914, 0.41118028396703177], [0.50973881288865242, -0.57773555361305662, -0.057544577579257794, -0.38940814537914492], [0.63601981762837778, -0.07334659860804571, 0.13001360256183525, -0.94605829289307453]]]])
      res=minimum(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*numpy.array([[[[-0.25546937210977871, -0.15953462441407429, -0.68043226043641081, -0.87681401094494094], [-0.9695983155906287, -0.15364919368303798, 0.030160758861827119, -0.92415648368275738], [0.24188876407985682, -0.37726568082801926, -0.031244555386071182, 0.10581899828603314]], [[-0.88867133240432428, -0.040442550882596295, -0.96671783461884542, -0.83253250212204444], [0.63028509118943798, -0.944819458613956, -0.65875103578172434, 0.6416560128654627], [-0.51768106525739577, -0.95314812231221246, 0.30709465760721777, -0.58977699061810096]]], [[[-0.95992219762575615, -0.066575866315797372, -0.39911206297446955, -0.2252897487980432], [-0.89976461972455346, 0.31866787036320132, -0.89967417979417186, -0.23250235959439425], [0.30897331797821015, -0.63589317933525402, -0.52945320471416024, -0.8219515227692169]], [[0.046053965984596434, -0.33603355979512473, -0.45371629141797443, 0.53619070384652145], [0.17824930535811601, -0.41289723392230493, 0.46769531478658299, -0.62092390819188248], [0.13528973033119573, 0.5771814412103291, -0.59994769111313673, -0.71267492351189787]]], [[[0.062965273551715883, 0.3984423920467417, 0.64345037699888441, 0.30816424930249453], [-0.62579483160181737, -0.74637079543318352, -0.93111256601427783, 0.24019128438975446], [-0.093324441995394247, -0.36479323325406687, -0.56595771086234792, -0.6426381946129549]], [[0.52000927142362841, -0.39276100837897543, -0.20324203866299317, -0.63660389564486386], [-0.88829184611129675, -0.59313355157251668, -0.27386335124317984, -0.61769836226507069], [-0.59576399350712039, 0.30018332336192199, -0.96040815278083791, 0.91923945312301303]]]])+(1.-msk_ref)*numpy.array([[[[-0.98224484945839463, 0.7590398139904484, 0.60917416945078062, -0.95323666037912491], [0.065804782544962137, -0.43702907186543016, -0.97674631897757735, 0.022982393583445671], [-0.89361829233112866, -0.94599386362372728, -0.93231635029418403, -0.04553695938314517]], [[-0.16748037734067323, -0.33391372704158195, -0.63493116997531973, -0.90439036915169324], [0.25068095981494043, -0.19140831812052861, 0.35750719638369866, -0.35590907029535046], [-0.69105597053150425, -0.94527231725182959, -0.8102802593916536, -0.73372966368662751]]], [[[0.23812505816617868, -0.88151193083904777, 0.095207905980526197, -0.69124361116174593], [0.14858701799370544, 0.093928805343780786, -0.95743778636458687, -0.14774514961802643], [-0.97817921648061867, 0.7658360887578135, -0.79683179728428799, 0.043961512901157684]], [[0.35756731123310392, -0.48264940908142151, -0.65318848328350376, 0.87473434196921507], [-0.95695137577762912, -0.47131199534772339, -0.54481953495951108, -0.44645464432144211], [-0.43854372305478373, -0.71465794226684465, -0.12366819719187494, 0.053016146326937852]]], [[[0.26114660171599891, -0.73394722395651057, 0.021272086611772822, -0.77240321798355382], [-0.55175199625157334, 0.27049855862575178, -0.42218196424565946, -0.68367437973492895], [0.17183266111472473, -0.36898894101335911, 0.036378238609670488, 0.16474024394444542]], [[-0.39535356597941762, -0.88290828211650996, -0.21519102192100914, -0.32049948781332871], [-0.2367582269308024, -0.57773555361305662, -0.057544577579257794, -0.38940814537914492], [-0.18061661907409143, -0.07334659860804571, -0.85781971479507502, -0.94605829289307453]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_constData_rank0(self):
      arg0=4.35251522982
      arg1=Data(2.94566651719,self.functionspace)
      res=power(arg0,arg1)
      ref=Data(76.1230031099,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_power_float_rank0_expandedData_rank0(self):
      arg0=3.41819405792
      msk_arg1=whereNegative(self.functionspace.getX()[0]-0.5)
      arg1=msk_arg1*(3.18794922769)+(1.-msk_arg1)*(0.35955973515)
      res=power(arg0,arg1)
      msk_ref=whereNegative(self.functionspace.getX()[0]-0.5)
      ref=msk_ref*(50.3172415941)+(1.-msk_ref)*(1.55572132713)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[1.0, -1.0, 5.0], [-1.0, -6.0, 6.0]])
      arg1=numpy.array([4.0, 0.0, -7.0])
      res=tensor_mult(arg0,arg1)
      ref=numpy.array([-31.0, -46.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[-1.0, 5.0, -3.0], [5.0, 7.0, 0.0]])
      arg1=numpy.array([[6.0, -6.0], [-5.0, -2.0], [0.0, 2.0]])
      res=tensor_mult(arg0,arg1)
      ref=numpy.array([[-31.0, -10.0], [-5.0, -44.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-6.0, -2.0, 6.0], [-7.0, -1.0, -5.0]])
      arg1=Symbol(shape=(3,))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([4.0, -1.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-64.0, 8.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[6.0, 2.0, 4.0], [0.0, -1.0, 3.0]])
      arg1=Symbol(shape=(3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[1.0, -3.0], [-3.0, -5.0], [1.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[4.0, -28.0], [6.0, 5.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[2.0, -4.0, 3.0], [-3.0, 3.0, 7.0]])
      arg1=Data(numpy.array([-5.0, 4.0, -4.0]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([-38.0, -1.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-5.0, 0.0, -6.0], [-6.0, 2.0, 4.0]])
      arg1=Data(numpy.array([[1.0, 4.0], [5.0, -6.0], [7.0, -3.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-47.0, -2.0], [32.0, -48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[3.0, 5.0, -3.0], [4.0, -5.0, -2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, -4.0, 5.0])+(1.-msk_arg1)*numpy.array([-7.0, -3.0, -5.0])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-26.0, 22.0])+(1.-msk_ref)*numpy.array([-21.0, -3.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[6.0, 7.0, 4.0], [-1.0, -4.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, -2.0], [-2.0, -3.0], [-5.0, -5.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 7.0], [-6.0, -6.0], [-4.0, 4.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-16.0, -53.0], [-15.0, -6.0]])+(1.-msk_ref)*numpy.array([[-70.0, 16.0], [10.0, 33.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[-7.0, 4.0, 2.0], [7.0, -2.0, 7.0]], [[3.0, 5.0, 0.0], [7.0, 0.0, -4.0]], [[0.0, -2.0, -4.0], [-3.0, -7.0, 0.0]], [[-1.0, -2.0, 5.0], [-5.0, 5.0, 4.0]], [[-5.0, -2.0, -1.0], [-5.0, 0.0, -2.0]]], [[[7.0, 0.0, -1.0], [5.0, -1.0, 7.0]], [[5.0, -6.0, -7.0], [-6.0, -2.0, 0.0]], [[-5.0, 4.0, 7.0], [-5.0, -4.0, 3.0]], [[0.0, -5.0, 1.0], [1.0, -4.0, -3.0]], [[-3.0, 4.0, -6.0], [-4.0, -6.0, -2.0]]], [[[0.0, 5.0, 0.0], [7.0, -7.0, -2.0]], [[1.0, -6.0, 3.0], [0.0, -5.0, -5.0]], [[-6.0, -4.0, 4.0], [0.0, 0.0, 2.0]], [[-5.0, -3.0, -7.0], [-1.0, 3.0, -4.0]], [[5.0, -3.0, 3.0], [-6.0, -1.0, 0.0]]], [[[2.0, -7.0, 0.0], [-4.0, 5.0, 1.0]], [[-2.0, -4.0, -6.0], [6.0, 2.0, 4.0]], [[0.0, 1.0, -1.0], [4.0, -4.0, -7.0]], [[-5.0, 6.0, 6.0], [0.0, 2.0, 1.0]], [[-5.0, 2.0, -7.0], [-7.0, -4.0, -3.0]]]])
      arg1=numpy.array([[0.0, 2.0, -5.0], [-2.0, -5.0, 1.0]])
      res=tensor_mult(arg0,arg1)
      ref=numpy.array([[1.0, -8.0, 57.0, -40.0, 9.0], [7.0, 45.0, 6.0, 0.0, 74.0], [29.0, -7.0, -26.0, 12.0, -4.0], [-30.0, 4.0, 12.0, -27.0, 70.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[6.0, 6.0, 2.0], [0.0, 5.0, 0.0]], [[-6.0, -2.0, -4.0], [-2.0, 4.0, -3.0]], [[-5.0, -6.0, 0.0], [5.0, 0.0, 7.0]], [[1.0, -7.0, -7.0], [2.0, 7.0, 4.0]], [[-6.0, -5.0, 2.0], [5.0, 0.0, 7.0]]], [[[-5.0, 7.0, 7.0], [-6.0, 0.0, -2.0]], [[0.0, -4.0, -4.0], [-2.0, 6.0, 4.0]], [[-7.0, -4.0, 0.0], [3.0, 0.0, 3.0]], [[-1.0, 1.0, 4.0], [0.0, 0.0, -4.0]], [[0.0, 6.0, 3.0], [2.0, 6.0, -7.0]]], [[[-6.0, 7.0, -1.0], [7.0, -1.0, 7.0]], [[-6.0, 2.0, 2.0], [-7.0, -2.0, 7.0]], [[-1.0, -5.0, 1.0], [3.0, 0.0, 0.0]], [[-2.0, 0.0, -7.0], [-5.0, -3.0, -2.0]], [[2.0, 6.0, -5.0], [-2.0, -3.0, -7.0]]], [[[-4.0, 1.0, 0.0], [-4.0, 3.0, 2.0]], [[1.0, 0.0, 6.0], [-6.0, -5.0, -7.0]], [[-4.0, -6.0, 2.0], [-1.0, 6.0, 0.0]], [[7.0, 4.0, -7.0], [-2.0, 2.0, 1.0]], [[-7.0, -1.0, -2.0], [0.0, -7.0, 3.0]]]])
      arg1=numpy.array([[[3.0, -4.0], [-1.0, -1.0], [-6.0, -7.0]], [[-2.0, 1.0], [1.0, 2.0], [3.0, 1.0]]])
      res=tensor_mult(arg0,arg1)
      ref=numpy.array([[[5.0, -34.0], [7.0, 57.0], [2.0, 38.0], [67.0, 72.0], [-14.0, 27.0]], [[-58.0, -44.0], [50.0, 46.0], [-14.0, 38.0], [-40.0, -29.0], [-43.0, -20.0]], [[-13.0, 36.0], [1.0, 4.0], [-10.0, 5.0], [37.0, 44.0], [10.0, 6.0]], [[4.0, 19.0], [-47.0, -69.0], [-10.0, 19.0], [68.0, 20.0], [-6.0, 32.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[-1.0, -6.0, -4.0], [-2.0, 3.0, 3.0]], [[3.0, -6.0, 2.0], [-2.0, -1.0, -3.0]], [[4.0, -4.0, -4.0], [3.0, 1.0, 1.0]], [[6.0, -6.0, -6.0], [-4.0, -5.0, 0.0]], [[-2.0, 1.0, 7.0], [2.0, -1.0, -6.0]]], [[[-4.0, 6.0, 0.0], [2.0, -6.0, 1.0]], [[0.0, -7.0, 1.0], [7.0, 6.0, 4.0]], [[3.0, 4.0, -3.0], [2.0, 1.0, 7.0]], [[1.0, -6.0, 0.0], [2.0, 4.0, 6.0]], [[-5.0, 6.0, 3.0], [-3.0, 0.0, 0.0]]], [[[-1.0, 5.0, -5.0], [0.0, -6.0, -5.0]], [[-5.0, -2.0, 6.0], [-5.0, 4.0, -6.0]], [[0.0, 4.0, -5.0], [6.0, 4.0, 0.0]], [[2.0, -2.0, -4.0], [7.0, -5.0, -4.0]], [[-7.0, 3.0, -7.0], [2.0, 0.0, -2.0]]], [[[6.0, -7.0, 1.0], [7.0, -6.0, -4.0]], [[6.0, 7.0, 4.0], [-6.0, 7.0, 0.0]], [[4.0, 0.0, -6.0], [0.0, 1.0, 7.0]], [[-5.0, 7.0, 1.0], [2.0, -1.0, -1.0]], [[-1.0, 0.0, 6.0], [7.0, -2.0, 4.0]]]])
      arg1=numpy.array([[[[-2.0, 2.0, -6.0, 0.0, 0.0], [-2.0, 1.0, -4.0, 4.0, -3.0], [0.0, -7.0, 2.0, 2.0, 0.0], [3.0, 5.0, -1.0, -1.0, 0.0]], [[-5.0, -3.0, -1.0, 2.0, -6.0], [-3.0, -7.0, 2.0, -5.0, 3.0], [2.0, 6.0, -3.0, -4.0, 0.0], [5.0, 5.0, -1.0, 2.0, 7.0]], [[-1.0, 2.0, -7.0, 5.0, 2.0], [2.0, 0.0, -2.0, 4.0, 1.0], [3.0, -6.0, -6.0, 5.0, 7.0], [0.0, -3.0, 0.0, -6.0, -2.0]]], [[[4.0, 5.0, -2.0, 3.0, 3.0], [2.0, -5.0, 4.0, 4.0, -6.0], [3.0, 0.0, 0.0, -2.0, 6.0], [-1.0, 0.0, 4.0, -7.0, 0.0]], [[0.0, -2.0, -1.0, 3.0, 0.0], [4.0, 3.0, -3.0, -1.0, -4.0], [2.0, 7.0, 6.0, -4.0, 0.0], [-2.0, 5.0, -4.0, -1.0, 6.0]], [[1.0, -1.0, 0.0, -4.0, 2.0], [-3.0, 7.0, -4.0, 7.0, -4.0], [-1.0, 4.0, 0.0, -7.0, -7.0], [-2.0, 6.0, -6.0, 1.0, -4.0]]]])
      res=tensor_mult(arg0,arg1)
      ref=numpy.array([[[[31.0, -11.0, 41.0, -41.0, 28.0], [11.0, 81.0, -29.0, 20.0, -31.0], [-27.0, 28.0, 58.0, -27.0, -61.0], [-43.0, 10.0, -31.0, 27.0, -28.0]], [[11.0, 23.0, -21.0, 1.0, 28.0], [17.0, 31.0, -21.0, 22.0, 3.0], [-11.0, -88.0, 6.0, 69.0, 23.0], [-11.0, -44.0, 17.0, -15.0, -40.0]], [[29.0, 24.0, 1.0, -20.0, 27.0], [3.0, 27.0, -11.0, 38.0, -54.0], [-10.0, -17.0, 50.0, -13.0, -17.0], [-15.0, 23.0, 2.0, -9.0, -18.0]], [[8.0, 8.0, 25.0, -69.0, 12.0], [-34.0, 53.0, -25.0, 19.0, 2.0], [-52.0, -77.0, 36.0, 34.0, -66.0], [2.0, -7.0, 4.0, 51.0, -60.0]], [[-6.0, 25.0, -41.0, 64.0, 2.0], [33.0, -64.0, 31.0, -18.0, 32.0], [33.0, -53.0, -55.0, 69.0, 103.0], [11.0, -67.0, 49.0, -57.0, 11.0]]], [[[-13.0, -5.0, 20.0, -4.0, -28.0], [-33.0, -67.0, 50.0, -25.0, 38.0], [5.0, 26.0, -62.0, -19.0, 5.0], [26.0, -14.0, 24.0, 9.0, 2.0]], [[66.0, 42.0, -20.0, 14.0, 73.0], [49.0, 60.0, -22.0, 89.0, -102.0], [18.0, 10.0, 51.0, -33.0, 21.0], [-62.0, 16.0, -13.0, -71.0, -31.0]], [[-8.0, -11.0, -6.0, -26.0, -10.0], [-37.0, 17.0, -21.0, 36.0, -44.0], [0.0, 56.0, 18.0, -82.0, -58.0], [11.0, 91.0, -45.0, 15.0, 12.0]], [[42.0, 16.0, -8.0, -18.0, 54.0], [18.0, 87.0, -44.0, 80.0, -73.0], [-4.0, 9.0, 44.0, -36.0, -30.0], [-49.0, 31.0, -39.0, -25.0, -42.0]], [[-35.0, -37.0, 9.0, 18.0, -39.0], [-8.0, -32.0, 14.0, -50.0, 54.0], [12.0, 53.0, -46.0, -13.0, 3.0], [18.0, -4.0, -13.0, 20.0, 36.0]]], [[[-23.0, -10.0, 42.0, -13.0, -50.0], [-32.0, -89.0, 62.0, -78.0, 57.0], [-12.0, 5.0, -23.0, 12.0, 0.0], [44.0, -25.0, 50.0, 42.0, 29.0]], [[-12.0, -19.0, -4.0, 47.0, -3.0], [52.0, 4.0, -4.0, -52.0, 53.0], [13.0, -9.0, -16.0, 64.0, 54.0], [-16.0, -69.0, 7.0, -10.0, 22.0]], [[9.0, 0.0, 15.0, 13.0, -16.0], [6.0, -46.0, 30.0, -20.0, -45.0], [19.0, 82.0, 42.0, -69.0, 1.0], [6.0, 55.0, 4.0, -8.0, 62.0]], [[34.0, 51.0, 9.0, -2.0, 17.0], [0.0, -62.0, 55.0, 7.0, -22.0], [-1.0, -53.0, 4.0, 26.0, 42.0], [7.0, -37.0, 72.0, -30.0, -20.0]], [[12.0, -25.0, 84.0, -15.0, -30.0], [1.0, -52.0, 64.0, -77.0, 19.0], [-7.0, 101.0, 19.0, -51.0, -23.0], [-4.0, -11.0, 24.0, 39.0, 43.0]]], [[[46.0, 86.0, -44.0, 10.0, 57.0], [13.0, -26.0, 22.0, 69.0, -40.0], [2.0, -148.0, -9.0, 83.0, 77.0], [-4.0, -62.0, 77.0, -73.0, -71.0]], [[-75.0, -45.0, -66.0, 37.0, -52.0], [-9.0, 8.0, -63.0, -26.0, 15.0], [22.0, 25.0, 9.0, -12.0, -8.0], [45.0, 88.0, -65.0, 19.0, 83.0]], [[5.0, -13.0, 17.0, -55.0, 2.0], [-37.0, 56.0, -35.0, 40.0, -50.0], [-23.0, 43.0, 50.0, -75.0, -91.0], [-4.0, 85.0, -50.0, 38.0, -10.0]], [[-19.0, -16.0, 13.0, 26.0, -36.0], [-6.0, -74.0, 47.0, -49.0, 33.0], [22.0, 60.0, -43.0, -26.0, 26.0], [22.0, -4.0, 16.0, -1.0, 45.0]], [[28.0, 45.0, -48.0, 29.0, 41.0], [8.0, -14.0, 10.0, 78.0, -41.0], [31.0, -27.0, -50.0, -6.0, 56.0], [-14.0, -9.0, 13.0, -78.0, -40.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[-1.0, -6.0, 7.0], [2.0, 2.0, 1.0]], [[-7.0, 0.0, -4.0], [-2.0, 6.0, -7.0]], [[0.0, -3.0, -7.0], [4.0, -3.0, 4.0]], [[-6.0, 0.0, 7.0], [-6.0, 0.0, -7.0]], [[2.0, -1.0, 0.0], [6.0, -4.0, -5.0]]], [[[6.0, -3.0, 2.0], [-4.0, -5.0, 0.0]], [[4.0, 4.0, 1.0], [-1.0, 7.0, 0.0]], [[3.0, -2.0, -3.0], [-7.0, 3.0, 0.0]], [[-7.0, 6.0, -2.0], [-2.0, 2.0, 0.0]], [[2.0, 1.0, 1.0], [-2.0, 7.0, -1.0]]], [[[-4.0, -2.0, 0.0], [7.0, -5.0, 1.0]], [[-2.0, 2.0, -3.0], [-7.0, 1.0, 3.0]], [[4.0, -1.0, 6.0], [1.0, -7.0, 1.0]], [[3.0, -6.0, 5.0], [2.0, -6.0, 7.0]], [[-6.0, -5.0, 7.0], [4.0, 7.0, -2.0]]], [[[4.0, 4.0, 7.0], [-2.0, 0.0, -6.0]], [[-3.0, 5.0, -3.0], [1.0, -1.0, 0.0]], [[1.0, 1.0, 1.0], [5.0, 6.0, 2.0]], [[4.0, -7.0, -1.0], [-7.0, 4.0, -2.0]], [[1.0, -7.0, 6.0], [-7.0, -1.0, -7.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[-7.0, 3.0, 7.0], [0.0, 0.0, -7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[31.0, 70.0, -86.0, 140.0, 18.0], [-37.0, -9.0, -48.0, 53.0, 3.0], [15.0, -22.0, 4.0, -53.0, 90.0], [75.0, 15.0, -11.0, -42.0, 63.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[-7.0, -3.0, 6.0], [4.0, 5.0, -3.0]], [[-7.0, 3.0, -1.0], [0.0, 7.0, -1.0]], [[3.0, -1.0, 2.0], [6.0, -7.0, 4.0]], [[-4.0, 1.0, 0.0], [7.0, -2.0, -4.0]], [[6.0, -2.0, -5.0], [7.0, 5.0, 6.0]]], [[[-4.0, -4.0, -3.0], [-1.0, 0.0, 0.0]], [[4.0, -2.0, -1.0], [6.0, 3.0, -1.0]], [[1.0, -2.0, -6.0], [0.0, 7.0, 4.0]], [[-3.0, 6.0, -1.0], [-1.0, 1.0, 5.0]], [[-4.0, 2.0, 1.0], [0.0, 5.0, -2.0]]], [[[0.0, -7.0, -3.0], [-7.0, -2.0, -5.0]], [[7.0, -4.0, -6.0], [1.0, 0.0, -4.0]], [[-2.0, -1.0, -4.0], [-2.0, 7.0, 0.0]], [[-4.0, 4.0, -2.0], [-1.0, -3.0, -2.0]], [[-4.0, -4.0, 2.0], [-5.0, -7.0, -5.0]]], [[[6.0, 0.0, 1.0], [-5.0, 6.0, 7.0]], [[2.0, -1.0, 1.0], [0.0, -1.0, -3.0]], [[-1.0, 1.0, -4.0], [1.0, 6.0, 0.0]], [[7.0, 0.0, 1.0], [-5.0, 0.0, 6.0]], [[-3.0, -7.0, -2.0], [1.0, 0.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[2.0, -4.0], [-4.0, 6.0], [-5.0, 7.0]], [[-4.0, 0.0], [-1.0, 6.0], [-4.0, 4.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-41.0, 70.0], [-24.0, 77.0], [-33.0, -30.0], [-22.0, -6.0], [-12.0, -17.0]], [[27.0, -29.0], [-2.0, -21.0], [17.0, 0.0], [-42.0, 67.0], [-18.0, 57.0]], [[93.0, -95.0], [72.0, -110.0], [21.0, 16.0], [1.0, 0.0], [45.0, -56.0]], [[-7.0, 47.0], [16.0, -25.0], [4.0, 18.0], [5.0, 3.0], [28.0, -44.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[0.0, 5.0, 4.0], [0.0, -4.0, -2.0]], [[7.0, 7.0, 1.0], [-2.0, 3.0, 7.0]], [[-1.0, -2.0, -5.0], [6.0, 0.0, 7.0]], [[6.0, -5.0, 0.0], [-1.0, 2.0, 0.0]], [[-5.0, -5.0, 7.0], [6.0, 5.0, 4.0]]], [[[4.0, -2.0, 2.0], [6.0, -7.0, 7.0]], [[-7.0, 5.0, 5.0], [-7.0, 3.0, -4.0]], [[7.0, -7.0, -5.0], [-3.0, -6.0, -7.0]], [[6.0, 4.0, -4.0], [-4.0, 1.0, 0.0]], [[-1.0, -4.0, 2.0], [5.0, 1.0, 6.0]]], [[[7.0, 5.0, 0.0], [3.0, -6.0, 7.0]], [[-6.0, -6.0, -1.0], [-6.0, 5.0, 4.0]], [[7.0, 4.0, 6.0], [7.0, -6.0, 2.0]], [[-4.0, -5.0, -1.0], [-1.0, 2.0, 4.0]], [[-2.0, -5.0, -3.0], [-1.0, -7.0, -4.0]]], [[[-4.0, 3.0, 0.0], [-6.0, -5.0, -7.0]], [[1.0, -4.0, -5.0], [3.0, 4.0, -2.0]], [[0.0, 6.0, -4.0], [-4.0, -5.0, -5.0]], [[-2.0, 3.0, -5.0], [3.0, -7.0, 1.0]], [[4.0, 7.0, -1.0], [-1.0, -2.0, 5.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[[0.0, 0.0, 6.0, 5.0, 4.0], [-2.0, -4.0, -4.0, 2.0, 7.0], [-5.0, -1.0, 0.0, -7.0, -7.0], [-4.0, -3.0, 5.0, -5.0, 2.0]], [[-3.0, -5.0, 1.0, 2.0, 4.0], [4.0, 2.0, -4.0, -6.0, 0.0], [4.0, 2.0, 0.0, -7.0, 0.0], [0.0, 0.0, 7.0, 2.0, 6.0]], [[4.0, -3.0, -3.0, 5.0, 2.0], [-6.0, 1.0, -4.0, 2.0, 2.0], [-7.0, -3.0, -7.0, -5.0, 0.0], [-5.0, -4.0, -6.0, 0.0, -5.0]]], [[[-5.0, 7.0, 7.0, 0.0, 0.0], [3.0, 5.0, 1.0, 0.0, 7.0], [0.0, -7.0, -1.0, -1.0, -5.0], [6.0, -6.0, 5.0, 0.0, 3.0]], [[3.0, 5.0, 6.0, 1.0, -7.0], [-7.0, 6.0, -4.0, 2.0, -7.0], [-1.0, 6.0, 6.0, 4.0, 6.0], [7.0, -7.0, -6.0, -4.0, -7.0]], [[0.0, 6.0, 7.0, 3.0, 4.0], [-3.0, 3.0, 6.0, -7.0, -1.0], [-7.0, 5.0, -1.0, -7.0, 1.0], [-5.0, -4.0, 4.0, -1.0, -2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-11.0, -69.0, -45.0, 20.0, 48.0], [30.0, -16.0, -32.0, -16.0, 38.0], [10.0, -36.0, -50.0, -57.0, -26.0], [-38.0, 20.0, 27.0, 28.0, 42.0]], [[2.0, 5.0, 99.0, 78.0, 65.0], [-40.0, 16.0, -32.0, -69.0, 9.0], [-66.0, 71.0, 6.0, -138.0, -14.0], [-59.0, -62.0, 78.0, -40.0, 10.0]], [[-44.0, 109.0, 98.0, -13.0, 6.0], [21.0, 46.0, 80.0, -49.0, 18.0], [-17.0, 5.0, 22.0, -9.0, -16.0], [30.0, -41.0, 69.0, -6.0, 15.0]], [[26.0, 28.0, 36.0, 22.0, -10.0], [-49.0, -27.0, -13.0, 46.0, 21.0], [-52.0, 3.0, 13.0, 2.0, -25.0], [-16.0, -26.0, -22.0, -48.0, -35.0]], [[28.0, 95.0, 44.0, 17.0, -45.0], [-81.0, 89.0, 22.0, 16.0, -18.0], [-77.0, -18.0, -29.0, 21.0, 39.0], [36.0, -100.0, -86.0, -9.0, -100.0]]], [[[-37.0, 53.0, 65.0, 40.0, 89.0], [18.0, -9.0, 60.0, -39.0, 116.0], [-84.0, -63.0, -69.0, -107.0, -93.0], [-74.0, -35.0, 94.0, -3.0, 39.0]], [[49.0, -98.0, -111.0, -9.0, -35.0], [-26.0, 14.0, -55.0, 0.0, -105.0], [45.0, 49.0, -6.0, 36.0, 98.0], [2.0, 38.0, -99.0, 37.0, -43.0]], [[-2.0, -43.0, -56.0, -31.0, 4.0], [42.0, -119.0, -1.0, 83.0, 67.0], [27.0, -56.0, 9.0, 53.0, -77.0], [-28.0, 87.0, 9.0, -18.0, 44.0]], [[-5.0, -31.0, 30.0, 19.0, 25.0], [9.0, -34.0, -32.0, -18.0, -1.0], [13.0, 48.0, 38.0, -42.0, -16.0], [-21.0, 15.0, 56.0, -26.0, 37.0]], [[-2.0, 90.0, 67.0, 16.0, 1.0], [-36.0, 47.0, 49.0, -14.0, 19.0], [-68.0, -12.0, -19.0, -18.0, -6.0], [1.0, -66.0, -2.0, -13.0, -40.0]]], [[[-48.0, 8.0, 81.0, 60.0, 118.0], [36.0, -18.0, 21.0, -77.0, 105.0], [-58.0, -19.0, -46.0, -160.0, -93.0], [-87.0, -25.0, 149.0, -8.0, 81.0]], [[59.0, 40.0, -23.0, -30.0, -69.0], [-71.0, 23.0, 50.0, 4.0, -125.0], [-20.0, 89.0, 39.0, 87.0, 106.0], [8.0, 7.0, -110.0, -6.0, -104.0]], [[-41.0, -7.0, 55.0, 73.0, 106.0], [23.0, -9.0, -25.0, -24.0, 150.0], [-69.0, -92.0, -87.0, -152.0, -118.0], [-68.0, -53.0, 106.0, -5.0, 67.0]], [[22.0, 55.0, 7.0, -21.0, -36.0], [-35.0, 24.0, 55.0, -4.0, -55.0], [-23.0, 36.0, 16.0, 49.0, 49.0], [9.0, -8.0, -50.0, -2.0, -58.0]], [[-13.0, -32.0, -85.0, -54.0, -1.0], [60.0, -64.0, 43.0, 34.0, 26.0], [46.0, -54.0, -16.0, 65.0, -27.0], [-12.0, 89.0, -6.0, 32.0, 35.0]]], [[[6.0, -124.0, -142.0, -40.0, 3.0], [58.0, -59.0, -24.0, 13.0, -28.0], [86.0, -13.0, -17.0, 42.0, 21.0], [-20.0, 111.0, -27.0, 53.0, 41.0]], [[-11.0, 64.0, 48.0, -30.0, -58.0], [-1.0, 16.0, 7.0, 38.0, -8.0], [24.0, -1.0, 58.0, 73.0, 0.0], [77.0, -21.0, -10.0, -27.0, -12.0]], [[-29.0, -101.0, -75.0, -28.0, 31.0], [86.0, -57.0, -22.0, -19.0, 4.0], [92.0, -3.0, 7.0, -3.0, -15.0], [-14.0, 95.0, 56.0, 37.0, 89.0]], [[-65.0, -8.0, -8.0, -33.0, 47.0], [101.0, -15.0, 53.0, -53.0, 45.0], [57.0, -35.0, -11.0, -20.0, -42.0], [-3.0, 53.0, 102.0, 43.0, 95.0]], [[-26.0, -19.0, 50.0, 42.0, 76.0], [22.0, -5.0, -3.0, -75.0, 28.0], [-18.0, 33.0, -9.0, -114.0, -30.0], [-56.0, -8.0, 102.0, -3.0, 56.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[-1.0, -2.0, -5.0], [-5.0, 6.0, 0.0]], [[-6.0, 1.0, 7.0], [1.0, 3.0, 1.0]], [[-2.0, 3.0, 0.0], [6.0, -1.0, 1.0]], [[4.0, -2.0, 4.0], [7.0, 6.0, -5.0]], [[5.0, -2.0, 4.0], [-1.0, 3.0, 4.0]]], [[[-4.0, -5.0, -5.0], [0.0, 5.0, -5.0]], [[-3.0, 1.0, 0.0], [7.0, 6.0, -3.0]], [[3.0, 0.0, 6.0], [2.0, -5.0, -2.0]], [[-2.0, -6.0, -2.0], [3.0, -1.0, -1.0]], [[0.0, -5.0, 3.0], [-2.0, 2.0, 4.0]]], [[[0.0, -6.0, 7.0], [-5.0, 0.0, -7.0]], [[6.0, -5.0, -6.0], [-2.0, -6.0, -4.0]], [[4.0, 4.0, 0.0], [6.0, 0.0, -3.0]], [[0.0, -2.0, -5.0], [-7.0, -1.0, -6.0]], [[-2.0, 7.0, 3.0], [-2.0, -7.0, 0.0]]], [[[-7.0, -6.0, 2.0], [3.0, 4.0, 3.0]], [[5.0, 3.0, -2.0], [7.0, 4.0, 2.0]], [[5.0, 1.0, 1.0], [6.0, -6.0, -3.0]], [[-6.0, -6.0, 5.0], [7.0, 2.0, -2.0]], [[-5.0, 2.0, -1.0], [-6.0, 4.0, -1.0]]]])
      arg1=Data(numpy.array([[-1.0, 4.0, 5.0], [6.0, -5.0, 6.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-92.0, 42.0, 61.0, -10.0, 10.0], [-96.0, 1.0, 52.0, -15.0, -3.0], [-61.0, -62.0, 30.0, -106.0, 68.0], [9.0, 31.0, 52.0, 27.0, -54.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[-3.0, -6.0, 5.0], [6.0, 2.0, -3.0]], [[2.0, 4.0, 3.0], [-5.0, -7.0, -3.0]], [[1.0, -7.0, 7.0], [0.0, 3.0, 5.0]], [[5.0, 3.0, -2.0], [-7.0, 0.0, 0.0]], [[7.0, -7.0, -4.0], [-5.0, -2.0, 5.0]]], [[[-6.0, 4.0, -2.0], [0.0, -5.0, -1.0]], [[5.0, -1.0, 4.0], [3.0, -1.0, 7.0]], [[-4.0, -1.0, -4.0], [7.0, 6.0, 7.0]], [[2.0, -4.0, 4.0], [3.0, 1.0, 1.0]], [[-5.0, 0.0, 5.0], [5.0, 6.0, 0.0]]], [[[0.0, 1.0, 1.0], [5.0, 2.0, 1.0]], [[0.0, 1.0, 6.0], [-6.0, 6.0, 4.0]], [[2.0, -4.0, -6.0], [6.0, 6.0, 5.0]], [[-4.0, -6.0, -4.0], [-2.0, 4.0, 7.0]], [[3.0, 5.0, 6.0], [-3.0, -1.0, 0.0]]], [[[-2.0, 2.0, -1.0], [0.0, 4.0, 3.0]], [[3.0, -5.0, -5.0], [7.0, 1.0, 7.0]], [[-1.0, 2.0, -6.0], [-6.0, 7.0, 5.0]], [[3.0, -4.0, 7.0], [0.0, -7.0, -3.0]], [[-4.0, 4.0, 4.0], [-2.0, 1.0, -1.0]]]])
      arg1=Data(numpy.array([[[7.0, 6.0], [7.0, -3.0], [-4.0, -3.0]], [[1.0, 7.0], [-4.0, 4.0], [-4.0, 5.0]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[-73.0, 20.0], [65.0, -87.0], [-102.0, 43.0], [57.0, -22.0], [-1.0, 57.0]], [[18.0, -67.0], [-9.0, 73.0], [-64.0, 99.0], [-35.0, 42.0], [-74.0, 14.0]], [[-4.0, 42.0], [-63.0, -19.0], [-28.0, 133.0], [-100.0, 43.0], [33.0, -40.0]], [[-24.0, 16.0], [-19.0, 136.0], [-23.0, 17.0], [5.0, -34.0], [-18.0, -63.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[3.0, -7.0, -1.0], [2.0, 7.0, 0.0]], [[-3.0, 7.0, -6.0], [5.0, -5.0, 5.0]], [[5.0, 7.0, 3.0], [3.0, 0.0, -4.0]], [[4.0, 7.0, -5.0], [6.0, -3.0, 1.0]], [[-3.0, 3.0, 6.0], [-2.0, 4.0, -4.0]]], [[[3.0, 0.0, -1.0], [0.0, -4.0, 1.0]], [[-5.0, 6.0, -7.0], [-6.0, 1.0, -3.0]], [[3.0, 7.0, 1.0], [-6.0, -4.0, 7.0]], [[7.0, 3.0, 1.0], [-7.0, 6.0, -4.0]], [[-4.0, 6.0, 5.0], [1.0, 6.0, 5.0]]], [[[-4.0, -6.0, -4.0], [-5.0, 2.0, 1.0]], [[0.0, -7.0, 3.0], [4.0, 1.0, 3.0]], [[-7.0, -2.0, -1.0], [4.0, 7.0, 5.0]], [[-5.0, 6.0, 4.0], [-4.0, 3.0, -5.0]], [[6.0, -5.0, 5.0], [-5.0, 4.0, -1.0]]], [[[2.0, 6.0, -3.0], [-2.0, 3.0, -2.0]], [[-1.0, 0.0, -5.0], [-1.0, -6.0, -5.0]], [[-2.0, 5.0, 4.0], [-7.0, -5.0, -7.0]], [[2.0, 1.0, -5.0], [1.0, 0.0, 4.0]], [[-4.0, -1.0, -6.0], [0.0, 7.0, -2.0]]]])
      arg1=Data(numpy.array([[[[7.0, 7.0, -6.0, 0.0, 3.0], [2.0, 0.0, -2.0, -6.0, 1.0], [0.0, -3.0, -2.0, 1.0, -6.0], [0.0, -1.0, -3.0, 0.0, -1.0]], [[2.0, 3.0, 1.0, 7.0, -1.0], [6.0, 7.0, -3.0, -2.0, -4.0], [3.0, -1.0, -3.0, 6.0, 6.0], [4.0, 7.0, -6.0, -5.0, -6.0]], [[-7.0, 4.0, 4.0, -4.0, -7.0], [2.0, -2.0, 1.0, -1.0, 3.0], [-3.0, 4.0, 6.0, -5.0, -4.0], [3.0, -2.0, 5.0, 7.0, -6.0]]], [[[-6.0, -1.0, -3.0, 1.0, 3.0], [4.0, 0.0, 0.0, 0.0, 6.0], [6.0, 0.0, -4.0, -1.0, 6.0], [-2.0, -4.0, 0.0, -6.0, 3.0]], [[3.0, -1.0, 2.0, 7.0, 6.0], [5.0, -3.0, 5.0, -1.0, -7.0], [-5.0, -4.0, 1.0, -7.0, 1.0], [-5.0, 4.0, 1.0, -3.0, -2.0]], [[2.0, 2.0, 3.0, -2.0, -5.0], [5.0, 2.0, 4.0, 4.0, -5.0], [0.0, -1.0, -3.0, 0.0, 0.0], [0.0, 4.0, 4.0, 7.0, -1.0]]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[23.0, -13.0, -21.0, 6.0, 71.0], [5.0, -68.0, 49.0, -10.0, -9.0], [-41.0, -34.0, 8.0, -85.0, -37.0], [-70.0, -30.0, 35.0, -5.0, 37.0]], [[0.0, -14.0, -9.0, 33.0, -14.0], [44.0, 86.0, -26.0, 35.0, -9.0], [94.0, -7.0, -91.0, 99.0, 109.0], [25.0, 44.0, -48.0, -57.0, 17.0]], [[2.0, 57.0, -32.0, 48.0, 16.0], [50.0, 35.0, -44.0, -63.0, 24.0], [30.0, -6.0, -13.0, 29.0, 18.0], [31.0, 10.0, -58.0, -60.0, -52.0]], [[34.0, 28.0, -58.0, 52.0, 35.0], [54.0, 70.0, -45.0, -26.0, 13.0], [87.0, -28.0, -89.0, 86.0, 71.0], [16.0, 23.0, -78.0, -90.0, 7.0]], [[-41.0, 2.0, 47.0, 31.0, -16.0], [16.0, -11.0, 7.0, -14.0, -17.0], [-41.0, 18.0, 57.0, -41.0, 4.0], [14.0, 20.0, 9.0, -1.0, -61.0]]], [[[18.0, 23.0, -27.0, -26.0, -13.0], [-11.0, 16.0, -23.0, -9.0, 23.0], [23.0, 2.0, -19.0, 36.0, -18.0], [17.0, -13.0, -14.0, 12.0, 10.0]], [[59.0, -46.0, 19.0, 77.0, 31.0], [-22.0, 47.0, -22.0, 12.0, -78.0], [-2.0, -20.0, -16.0, 65.0, 59.0], [10.0, 77.0, -67.0, -67.0, -6.0]], [[66.0, 70.0, 24.0, -3.0, -82.0], [41.0, 73.0, -18.0, -1.0, -65.0], [2.0, -3.0, -22.0, 74.0, -20.0], [63.0, 80.0, -22.0, 69.0, -68.0]], [[100.0, 55.0, -14.0, 60.0, 46.0], [16.0, -7.0, -8.0, -71.0, -66.0], [-66.0, -40.0, 29.0, -15.0, -64.0], [-1.0, 48.0, -44.0, -12.0, -60.0]], [[-29.0, 13.0, 74.0, 55.0, -39.0], [97.0, 24.0, 45.0, 21.0, -74.0], [-21.0, -3.0, 7.0, -36.0, 52.0], [7.0, 76.0, 27.0, 16.0, -76.0]]], [[[26.0, -57.0, 24.0, -19.0, 14.0], [-57.0, -38.0, 36.0, 42.0, -41.0], [-46.0, -7.0, 21.0, -29.0, -24.0], [-36.0, 2.0, 34.0, 33.0, 44.0]], [[-50.0, -8.0, 4.0, -56.0, -11.0], [0.0, -52.0, 41.0, 22.0, 39.0], [-11.0, 12.0, 15.0, -68.0, -29.0], [-32.0, -55.0, 70.0, 50.0, 31.0]], [[-39.0, -60.0, 53.0, 33.0, 17.0], [48.0, -23.0, 74.0, 60.0, -52.0], [-14.0, -14.0, -10.0, -67.0, 65.0], [-54.0, 27.0, 55.0, -7.0, 18.0]], [[-28.0, -10.0, 55.0, 53.0, -18.0], [8.0, 15.0, -9.0, -9.0, -37.0], [-33.0, 18.0, 50.0, -6.0, 29.0], [29.0, 47.0, -18.0, -22.0, -68.0]], [[37.0, 46.0, -1.0, -30.0, 2.0], [-13.0, -59.0, 24.0, -39.0, -12.0], [-80.0, -8.0, 60.0, -72.0, -112.0], [-15.0, -19.0, 37.0, 71.0, -28.0]]], [[[64.0, 15.0, -12.0, 77.0, 43.0], [31.0, 35.0, -18.0, -32.0, -54.0], [0.0, -34.0, -23.0, 34.0, 27.0], [4.0, 58.0, -62.0, -62.0, -30.0]], [[6.0, -30.0, -38.0, -13.0, 18.0], [-71.0, 18.0, -53.0, -3.0, 45.0], [39.0, 12.0, -15.0, 67.0, 14.0], [17.0, -29.0, -48.0, -46.0, 45.0]], [[-19.0, 15.0, 23.0, -9.0, -55.0], [-54.0, 28.0, -60.0, -25.0, 18.0], [-14.0, 44.0, 57.0, 50.0, -21.0], [71.0, 9.0, -37.0, 11.0, -56.0]], [[53.0, 4.0, -22.0, 20.0, 23.0], [24.0, 25.0, 4.0, 7.0, -31.0], [24.0, -31.0, -53.0, 32.0, 20.0], [-13.0, 27.0, -21.0, -18.0, 21.0]], [[29.0, -66.0, 7.0, 70.0, 83.0], [-1.0, -20.0, 32.0, 17.0, -57.0], [-20.0, -37.0, -12.0, -29.0, 49.0], [-57.0, 29.0, -13.0, -72.0, 34.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[6.0, -5.0, 2.0], [-7.0, -7.0, 6.0]], [[-4.0, 7.0, 0.0], [4.0, -4.0, 7.0]], [[-5.0, 0.0, 7.0], [2.0, -6.0, -1.0]], [[0.0, 5.0, 0.0], [5.0, 0.0, 5.0]], [[-5.0, 0.0, -2.0], [3.0, 3.0, 1.0]]], [[[7.0, 1.0, 2.0], [-6.0, -3.0, 0.0]], [[7.0, -7.0, 7.0], [-7.0, 6.0, -2.0]], [[0.0, -7.0, 7.0], [6.0, 1.0, 4.0]], [[5.0, 5.0, 0.0], [-2.0, 7.0, -6.0]], [[6.0, -5.0, 7.0], [0.0, -6.0, -1.0]]], [[[6.0, -4.0, 0.0], [-2.0, -1.0, -4.0]], [[0.0, 6.0, 6.0], [-4.0, -2.0, 4.0]], [[0.0, -6.0, 6.0], [5.0, -5.0, -6.0]], [[7.0, 2.0, 0.0], [0.0, 0.0, -2.0]], [[5.0, 3.0, 7.0], [-5.0, 4.0, 2.0]]], [[[0.0, 7.0, 1.0], [7.0, 2.0, 3.0]], [[6.0, -6.0, -7.0], [0.0, -7.0, 0.0]], [[-2.0, 4.0, 0.0], [-6.0, -3.0, -6.0]], [[-2.0, 0.0, 0.0], [-3.0, 5.0, -1.0]], [[-1.0, 0.0, -6.0], [5.0, 4.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, -2.0, 1.0], [2.0, -5.0, 1.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -5.0, 1.0], [2.0, -6.0, -3.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[27.0, 29.0, 50.0, 5.0, 0.0], [-11.0, -39.0, 32.0, -65.0, 34.0], [-7.0, 0.0, 47.0, -20.0, -37.0], [-6.0, 28.0, -7.0, -28.0, -15.0]])+(1.-msk_ref)*numpy.array([[13.0, -8.0, 70.0, -30.0, 3.0], [-25.0, -30.0, 36.0, -73.0, 47.0], [10.0, -32.0, 94.0, -32.0, -68.0], [-41.0, 41.0, 12.0, -25.0, -13.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[-3.0, 1.0, -6.0], [-6.0, 7.0, -4.0]], [[5.0, 2.0, 5.0], [7.0, -3.0, -2.0]], [[-7.0, 6.0, 5.0], [0.0, 1.0, 7.0]], [[-6.0, 7.0, -1.0], [5.0, 4.0, -6.0]], [[-4.0, -4.0, 4.0], [-6.0, 7.0, -2.0]]], [[[5.0, 4.0, -1.0], [-4.0, 3.0, 0.0]], [[2.0, -2.0, 5.0], [-5.0, -3.0, 6.0]], [[-6.0, 0.0, -6.0], [-2.0, 0.0, 0.0]], [[-5.0, -3.0, -3.0], [4.0, -1.0, 2.0]], [[6.0, 6.0, -2.0], [-6.0, 2.0, 6.0]]], [[[-2.0, 5.0, 1.0], [5.0, 7.0, 5.0]], [[2.0, -1.0, 5.0], [0.0, -6.0, 3.0]], [[-3.0, -2.0, 6.0], [-7.0, -6.0, 1.0]], [[5.0, 3.0, -5.0], [0.0, 0.0, 4.0]], [[-4.0, -2.0, 2.0], [4.0, 0.0, -7.0]]], [[[7.0, 0.0, 6.0], [4.0, -5.0, 1.0]], [[-2.0, 2.0, -3.0], [3.0, 2.0, 3.0]], [[4.0, 3.0, -4.0], [4.0, -5.0, 3.0]], [[-5.0, -3.0, 1.0], [-3.0, 0.0, 6.0]], [[0.0, -4.0, 4.0], [-2.0, -1.0, 4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, 6.0], [4.0, 3.0], [-5.0, -6.0]], [[-6.0, -3.0], [0.0, -7.0], [7.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, -2.0], [3.0, -6.0], [-3.0, 7.0]], [[2.0, -5.0], [3.0, -3.0], [5.0, -1.0]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[27.0, -34.0], [-48.0, -6.0], [13.0, -19.0], [-69.0, -88.0], [-34.0, -103.0]], [[70.0, 39.0], [49.0, 48.0], [12.0, 6.0], [-32.0, -14.0], [142.0, 106.0]], [[10.0, -37.0], [2.0, 39.0], [-4.0, 9.0], [90.0, 93.0], [-111.0, -96.0]], [[-12.0, 35.0], [16.0, 7.0], [49.0, 98.0], [18.0, 0.0], [4.0, 1.0]]])+(1.-msk_ref)*numpy.array([[[28.0, -29.0], [-44.0, -11.0], [83.0, 3.0], [52.0, -68.0], [-1.0, 71.0]], [[-14.0, -30.0], [-22.0, 71.0], [50.0, -20.0], [45.0, -12.0], [12.0, -44.0]], [[80.0, -70.0], [-33.0, 52.0], [-33.0, 112.0], [14.0, -67.0], [-15.0, 21.0]], [[-62.0, 22.0], [54.0, -53.0], [5.0, -62.0], [42.0, 44.0], [-11.0, 61.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[-5.0, -7.0, 4.0], [-7.0, 3.0, -7.0]], [[3.0, -2.0, 7.0], [-6.0, 2.0, 6.0]], [[-1.0, 6.0, -4.0], [0.0, -1.0, 0.0]], [[7.0, -7.0, 0.0], [-1.0, -1.0, -5.0]], [[2.0, -1.0, 7.0], [7.0, -4.0, 0.0]]], [[[1.0, 2.0, -5.0], [1.0, 0.0, -4.0]], [[0.0, 6.0, 7.0], [0.0, 3.0, -6.0]], [[0.0, -2.0, -5.0], [-4.0, 0.0, -5.0]], [[4.0, 0.0, 7.0], [-6.0, -2.0, -6.0]], [[3.0, -3.0, 0.0], [-5.0, -1.0, -1.0]]], [[[5.0, -4.0, -7.0], [0.0, 7.0, -3.0]], [[6.0, 5.0, -4.0], [-2.0, -6.0, 2.0]], [[5.0, 0.0, 5.0], [3.0, 3.0, 0.0]], [[-1.0, -3.0, 7.0], [4.0, -3.0, -7.0]], [[-6.0, 1.0, -2.0], [-6.0, -1.0, -1.0]]], [[[-6.0, -1.0, 2.0], [6.0, 0.0, -5.0]], [[5.0, 7.0, 2.0], [2.0, 3.0, 1.0]], [[3.0, -6.0, 1.0], [2.0, -3.0, 0.0]], [[-1.0, 7.0, 7.0], [1.0, 0.0, -7.0]], [[-6.0, 0.0, -6.0], [3.0, 0.0, 6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, -5.0, -6.0, -7.0, 0.0], [7.0, 3.0, 6.0, 5.0, 3.0], [2.0, -1.0, -3.0, -5.0, -3.0], [5.0, 4.0, 0.0, 4.0, -7.0]], [[4.0, -7.0, 6.0, 0.0, -1.0], [4.0, 4.0, 1.0, -7.0, 7.0], [0.0, 7.0, -4.0, -5.0, -4.0], [4.0, 4.0, -7.0, 6.0, 2.0]], [[-2.0, -5.0, -2.0, -6.0, 2.0], [-5.0, 5.0, 1.0, 0.0, -2.0], [-5.0, 3.0, 4.0, -4.0, -2.0], [-7.0, 7.0, -7.0, -1.0, -6.0]]], [[[2.0, 5.0, 3.0, 6.0, -7.0], [-1.0, 5.0, 1.0, -5.0, -1.0], [-7.0, -3.0, 6.0, 0.0, -5.0], [-4.0, 0.0, 7.0, -2.0, 4.0]], [[-7.0, -6.0, 0.0, 1.0, 0.0], [-2.0, 7.0, 4.0, 5.0, 0.0], [0.0, -5.0, -4.0, 3.0, -3.0], [-6.0, 0.0, -3.0, 2.0, -1.0]], [[5.0, 3.0, 1.0, -1.0, 0.0], [-2.0, 5.0, -5.0, -2.0, 1.0], [0.0, 0.0, -5.0, 2.0, -4.0], [2.0, 0.0, -2.0, -3.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, -1.0, -2.0, 3.0, 4.0], [-4.0, 0.0, 6.0, 7.0, -5.0], [-1.0, 1.0, -5.0, 0.0, 4.0], [-4.0, -5.0, 3.0, -1.0, 3.0]], [[7.0, 6.0, -3.0, 1.0, -1.0], [-7.0, -2.0, 0.0, 0.0, 6.0], [4.0, 5.0, -7.0, 5.0, 1.0], [-2.0, 0.0, -5.0, -5.0, -4.0]], [[2.0, 2.0, -2.0, 1.0, 3.0], [0.0, 0.0, 7.0, 6.0, -2.0], [1.0, -6.0, 2.0, -6.0, -4.0], [2.0, -2.0, -6.0, -7.0, -6.0]]], [[[4.0, 7.0, -5.0, -7.0, 0.0], [7.0, 0.0, -7.0, 6.0, 0.0], [3.0, 5.0, -6.0, -6.0, -6.0], [0.0, -6.0, -1.0, -2.0, 4.0]], [[0.0, 2.0, 0.0, 0.0, 7.0], [3.0, 2.0, -2.0, 2.0, 0.0], [-3.0, -3.0, -7.0, -5.0, 6.0], [-3.0, -2.0, -1.0, 1.0, 0.0]], [[-1.0, 7.0, 6.0, -1.0, 6.0], [2.0, -1.0, 5.0, 0.0, 5.0], [5.0, 0.0, -4.0, 0.0, -6.0], [0.0, 6.0, 6.0, 2.0, 3.0]]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-121.0, -20.0, -48.0, -21.0, 64.0], [-68.0, -72.0, 7.0, 88.0, -72.0], [19.0, -26.0, 40.0, 39.0, 89.0], [-85.0, -20.0, -23.0, -25.0, -62.0]], [[-9.0, -60.0, -56.0, -103.0, 58.0], [-32.0, 50.0, -5.0, 57.0, -7.0], [13.0, 12.0, -47.0, -15.0, -15.0], [-18.0, 53.0, -95.0, -9.0, -69.0]], [[36.0, -11.0, 50.0, 30.0, -14.0], [39.0, -6.0, -8.0, -52.0, 47.0], [18.0, 36.0, -33.0, -12.0, -10.0], [53.0, -8.0, -11.0, 34.0, 44.0]], [[-27.0, 0.0, -92.0, -51.0, 14.0], [34.0, -44.0, 55.0, 94.0, -32.0], [21.0, -48.0, 30.0, -13.0, 35.0], [7.0, 0.0, 55.0, 1.0, -86.0]], [[30.0, 21.0, -11.0, -18.0, -34.0], [-24.0, 44.0, 9.0, -38.0, -22.0], [-80.0, 11.0, 84.0, -45.0, -39.0], [-47.0, 53.0, 19.0, -27.0, -26.0]]], [[[3.0, -1.0, 15.0, 33.0, -19.0], [47.0, -29.0, 24.0, -6.0, 22.0], [20.0, -5.0, -5.0, -3.0, 10.0], [36.0, -23.0, 36.0, 31.0, 15.0]], [[-41.0, -113.0, 16.0, -33.0, 8.0], [-5.0, 50.0, 55.0, -15.0, 22.0], [-35.0, 48.0, 22.0, -61.0, -23.0], [-55.0, 73.0, -88.0, 53.0, -57.0]], [[-31.0, 4.0, -19.0, 11.0, 20.0], [31.0, -78.0, 14.0, 44.0, -5.0], [53.0, -17.0, -11.0, 20.0, 58.0], [33.0, -43.0, 31.0, 16.0, -10.0]], [[-30.0, -91.0, -62.0, -102.0, 56.0], [15.0, -27.0, 47.0, 52.0, -2.0], [15.0, 45.0, 18.0, -66.0, 34.0], [-5.0, 65.0, -73.0, 35.0, -116.0]], [[-11.0, -16.0, -52.0, -51.0, 38.0], [18.0, -40.0, 11.0, 58.0, -8.0], [41.0, -4.0, -18.0, -5.0, 35.0], [27.0, 0.0, -9.0, 5.0, -50.0]]], [[[-51.0, -13.0, -43.0, 17.0, -10.0], [46.0, -2.0, 62.0, 94.0, -2.0], [45.0, -89.0, -40.0, 38.0, 6.0], [10.0, -45.0, 62.0, 26.0, -20.0]], [[94.0, -13.0, -2.0, -38.0, 1.0], [92.0, -24.0, 1.0, -29.0, 65.0], [46.0, 53.0, -52.0, -53.0, -10.0], [126.0, 16.0, -7.0, 44.0, -2.0]], [[-10.0, -53.0, -31.0, -44.0, -11.0], [1.0, 76.0, 50.0, 25.0, 2.0], [-36.0, -14.0, 11.0, -36.0, -49.0], [-40.0, 55.0, -23.0, 15.0, -56.0]], [[-35.0, 8.0, -21.0, -7.0, -11.0], [-38.0, -16.0, 25.0, -5.0, -49.0], [-65.0, 4.0, 114.0, -31.0, 18.0], [-78.0, 33.0, 23.0, -22.0, -50.0]], [[-20.0, 6.0, 27.0, 18.0, 37.0], [-18.0, -66.0, -42.0, -10.0, -2.0], [40.0, 30.0, -21.0, 28.0, 55.0], [16.0, -34.0, -30.0, -3.0, 29.0]]], [[[-39.0, 42.0, 39.0, 71.0, -37.0], [-52.0, -7.0, -4.0, -43.0, -40.0], [-64.0, -13.0, 91.0, 17.0, 8.0], [-82.0, -14.0, 45.0, -29.0, 32.0]], [[27.0, -89.0, 15.0, -33.0, -17.0], [43.0, 89.0, 48.0, -21.0, 59.0], [-14.0, 29.0, -40.0, -57.0, -70.0], [15.0, 62.0, -60.0, 59.0, -24.0]], [[8.0, 50.0, -50.0, -18.0, -6.0], [-4.0, -21.0, 3.0, 32.0, -37.0], [-13.0, -33.0, 43.0, 2.0, 12.0], [-6.0, -5.0, 58.0, -35.0, -28.0]], [[-22.0, -95.0, 30.0, -22.0, 0.0], [-1.0, 30.0, 44.0, -45.0, 24.0], [-44.0, 68.0, 44.0, -72.0, -16.0], [-44.0, 73.0, -77.0, 50.0, -45.0]], [[30.0, 93.0, 63.0, 90.0, -33.0], [-27.0, -3.0, -69.0, -57.0, -3.0], [-3.0, -21.0, -18.0, 66.0, -9.0], [12.0, -66.0, 51.0, -42.0, 114.0]]]])+(1.-msk_ref)*numpy.array([[[[-42.0, -121.0, 16.0, 38.0, -22.0], [15.0, 27.0, 6.0, -47.0, -60.0], [-84.0, -108.0, 131.0, -32.0, 59.0], [33.0, 11.0, -42.0, 15.0, -60.0]], [[-42.0, 3.0, 52.0, 50.0, 85.0], [-22.0, 2.0, 135.0, 31.0, -11.0], [2.0, -85.0, 11.0, -26.0, -6.0], [0.0, 39.0, 17.0, -16.0, -31.0]], [[38.0, 27.0, -8.0, -1.0, -29.0], [-41.0, -14.0, -32.0, -33.0, 49.0], [24.0, 56.0, -38.0, 59.0, 12.0], [-13.0, 15.0, -8.0, -2.0, -3.0]], [[-76.0, -93.0, -18.0, 26.0, -2.0], [1.0, 17.0, 26.0, 41.0, -102.0], [-60.0, -30.0, 47.0, -24.0, 51.0], [-11.0, -57.0, 28.0, 19.0, 30.0]], [[27.0, 47.0, -50.0, -37.0, 2.0], [36.0, -6.0, 20.0, 90.0, -30.0], [34.0, 2.0, -3.0, -69.0, -87.0], [20.0, -58.0, -34.0, -64.0, -4.0]]], [[[8.0, -20.0, -27.0, -3.0, -37.0], [-19.0, 0.0, -56.0, -17.0, -3.0], [-15.0, 46.0, -19.0, 34.0, 44.0], [-18.0, -25.0, -2.0, 14.0, 17.0]], [[62.0, 14.0, -68.0, 19.0, 0.0], [-45.0, 0.0, 13.0, 48.0, -8.0], [-8.0, -21.0, -25.0, -27.0, 32.0], [-7.0, -56.0, -111.0, -88.0, -84.0]], [[-35.0, -85.0, 6.0, 26.0, -43.0], [-24.0, 9.0, -32.0, -54.0, -27.0], [-50.0, 0.0, 48.0, 44.0, 72.0], [-6.0, 4.0, 14.0, 43.0, 7.0]], [[-20.0, -78.0, -28.0, 67.0, -13.0], [-76.0, 2.0, 89.0, 30.0, -64.0], [-39.0, -62.0, 68.0, 4.0, 48.0], [4.0, -30.0, -58.0, -55.0, -72.0]], [[-52.0, -65.0, 22.0, 42.0, 2.0], [-31.0, 5.0, 50.0, -11.0, -38.0], [-32.0, -34.0, 47.0, 20.0, 39.0], [-3.0, 11.0, 24.0, 19.0, -2.0]]], [[[-59.0, -50.0, -2.0, 7.0, 34.0], [23.0, 25.0, -48.0, 7.0, -50.0], [-64.0, 6.0, -48.0, -13.0, 104.0], [-47.0, -43.0, 52.0, 65.0, 64.0]], [[-7.0, 4.0, 3.0, 31.0, -23.0], [-87.0, -24.0, 44.0, -6.0, 18.0], [32.0, 63.0, -27.0, 91.0, 9.0], [-24.0, 14.0, 37.0, -1.0, 20.0]], [[2.0, 32.0, -35.0, -1.0, 56.0], [10.0, 6.0, 38.0, 89.0, -35.0], [0.0, -19.0, -54.0, -63.0, 0.0], [-19.0, -59.0, -21.0, -43.0, -3.0]], [[20.0, -30.0, -65.0, -20.0, -43.0], [30.0, 7.0, -14.0, 53.0, -62.0], [-18.0, -29.0, 65.0, -66.0, -35.0], [33.0, -69.0, -73.0, -58.0, -38.0]], [[4.0, -43.0, 37.0, 24.0, -44.0], [-30.0, -3.0, -11.0, -92.0, 35.0], [-12.0, -16.0, 66.0, 58.0, 21.0], [21.0, 66.0, -10.0, 24.0, -37.0]]], [[[50.0, 11.0, -49.0, -54.0, -47.0], [63.0, 7.0, -89.0, 6.0, -5.0], [-3.0, 7.0, 25.0, -53.0, -39.0], [30.0, -40.0, -61.0, -25.0, -17.0]], [[40.0, 68.0, -39.0, 9.0, 46.0], [-44.0, -9.0, 29.0, 65.0, 18.0], [27.0, 29.0, -107.0, -4.0, 19.0], [-39.0, -41.0, -31.0, -53.0, -14.0]], [[-44.0, -29.0, 0.0, -10.0, 0.0], [35.0, 6.0, 17.0, 33.0, -53.0], [-11.0, -14.0, 38.0, -33.0, -28.0], [11.0, -23.0, 34.0, 13.0, 35.0]], [[78.0, 15.0, -80.0, 11.0, -32.0], [-52.0, -7.0, 1.0, 41.0, -2.0], [4.0, -3.0, -8.0, -13.0, 11.0], [4.0, -57.0, -123.0, -99.0, -90.0]], [[18.0, 57.0, 45.0, -51.0, -6.0], [57.0, -6.0, -69.0, -60.0, 72.0], [39.0, 45.0, -24.0, 18.0, -54.0], [12.0, 60.0, 51.0, 54.0, 48.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([-2.0, -1.0, -4.0])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[-5.0, -2.0, 4.0], [-7.0, 6.0, 4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-4.0, -8.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[6.0, -5.0], [2.0, 0.0], [1.0, -6.0]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[-1.0, 0.0, 0.0], [1.0, -4.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-6.0, 5.0], [-2.0, -5.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3,))
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[-5.0, -5.0, 2.0], [2.0, -7.0, -2.0]])
      s1=numpy.array([4.0, 2.0, 5.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-20.0, -16.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3, 2))
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[0.0, -5.0, 0.0], [-7.0, -5.0, -7.0]])
      s1=numpy.array([[-6.0, 6.0], [-3.0, -5.0], [1.0, -3.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[15.0, 25.0], [50.0, 4.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([-6.0, 6.0, -2.0]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[0.0, -7.0, -1.0], [-4.0, 6.0, 2.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-40.0, 56.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[7.0, 0.0], [7.0, 4.0], [-4.0, -5.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[-3.0, 6.0, -7.0], [7.0, 6.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[49.0, 59.0], [119.0, 59.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, -1.0, -1.0])+(1.-msk_arg1)*numpy.array([3.0, -1.0, -7.0])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[7.0, 5.0, 6.0], [0.0, 6.0, 4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([10.0, -10.0])+(1.-msk_ref)*numpy.array([-26.0, -34.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, 1.0], [6.0, 3.0], [3.0, -1.0]])+(1.-msk_arg1)*numpy.array([[0.0, -6.0], [6.0, -7.0], [0.0, 6.0]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[0.0, 2.0, -2.0], [-1.0, 5.0, 7.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 8.0], [45.0, 7.0]])+(1.-msk_ref)*numpy.array([[12.0, -26.0], [30.0, 13.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[-6.0, -2.0, 0.0], [6.0, 0.0, -5.0]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[5.0, -6.0, 0.0], [-4.0, 0.0, -5.0]], [[-3.0, 0.0, -2.0], [7.0, 5.0, 2.0]], [[-6.0, 2.0, -7.0], [0.0, 3.0, 7.0]], [[-7.0, -4.0, -6.0], [1.0, 4.0, 5.0]], [[6.0, 5.0, 5.0], [-3.0, -6.0, -7.0]]], [[[-4.0, 6.0, -6.0], [1.0, 7.0, 7.0]], [[0.0, 5.0, 3.0], [-5.0, 2.0, -7.0]], [[-3.0, 0.0, -5.0], [5.0, -2.0, -5.0]], [[2.0, 0.0, -3.0], [-5.0, -2.0, 0.0]], [[5.0, 6.0, -2.0], [-6.0, 7.0, 3.0]]], [[[0.0, 6.0, 3.0], [-7.0, 0.0, 7.0]], [[-4.0, -7.0, -1.0], [-5.0, 2.0, -6.0]], [[4.0, 1.0, 0.0], [4.0, -6.0, 0.0]], [[-5.0, 0.0, 0.0], [-1.0, 4.0, 5.0]], [[1.0, -2.0, -2.0], [1.0, -2.0, 0.0]]], [[[-2.0, 7.0, -3.0], [-2.0, -4.0, -5.0]], [[-1.0, -3.0, 1.0], [-4.0, 5.0, 2.0]], [[-5.0, -1.0, 3.0], [4.0, 6.0, -1.0]], [[7.0, 1.0, 0.0], [-7.0, 7.0, 3.0]], [[1.0, -2.0, 1.0], [-6.0, -7.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-17.0, 50.0, -3.0, 31.0, -29.0], [-17.0, -5.0, 73.0, -42.0, -93.0], [-89.0, 38.0, -2.0, -1.0, 4.0], [11.0, -22.0, 61.0, -101.0, -38.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[-6.0, 0.0], [5.0, -4.0], [-3.0, -2.0]], [[1.0, 3.0], [-2.0, -2.0], [-3.0, -4.0]]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[2.0, 1.0, 1.0], [6.0, -2.0, -5.0]], [[0.0, 1.0, 1.0], [5.0, -3.0, 0.0]], [[-4.0, 1.0, 1.0], [2.0, 6.0, -7.0]], [[-2.0, 0.0, -7.0], [-1.0, 0.0, 7.0]], [[-2.0, 0.0, 7.0], [-1.0, 4.0, 0.0]]], [[[2.0, 7.0, -2.0], [-4.0, -2.0, -3.0]], [[-3.0, 2.0, -2.0], [-3.0, 0.0, 7.0]], [[-2.0, 4.0, 7.0], [-2.0, 1.0, 0.0]], [[1.0, -3.0, 3.0], [-6.0, -5.0, -3.0]], [[0.0, 5.0, 7.0], [0.0, 2.0, -1.0]]], [[[-5.0, 1.0, 2.0], [-6.0, 6.0, -6.0]], [[0.0, 7.0, 7.0], [0.0, -7.0, 3.0]], [[0.0, 0.0, 0.0], [-3.0, -2.0, 4.0]], [[-4.0, -4.0, 0.0], [0.0, 0.0, -6.0]], [[-3.0, -5.0, -1.0], [-3.0, 4.0, -2.0]]], [[[-4.0, 0.0, -5.0], [2.0, -4.0, -2.0]], [[0.0, -2.0, 0.0], [4.0, 0.0, 7.0]], [[2.0, 3.0, 1.0], [-2.0, -2.0, -1.0]], [[3.0, 0.0, -4.0], [6.0, -2.0, 2.0]], [[-4.0, 2.0, -3.0], [6.0, 4.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[15.0, 36.0], [13.0, 15.0], [37.0, 16.0], [11.0, -17.0], [-18.0, -25.0]], [[38.0, -20.0], [10.0, -41.0], [7.0, -38.0], [-17.0, 10.0], [3.0, -34.0]], [[29.0, -14.0], [19.0, -40.0], [-11.0, -21.0], [22.0, 40.0], [-9.0, 13.0]], [[55.0, 32.0], [-27.0, -8.0], [5.0, -12.0], [-2.0, 22.0], [38.0, 4.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[[7.0, 6.0, 2.0, -2.0, 1.0], [4.0, 2.0, 5.0, -6.0, 3.0], [-4.0, -5.0, -1.0, -4.0, -3.0], [1.0, -7.0, -7.0, -7.0, 7.0]], [[1.0, -7.0, 0.0, -6.0, -5.0], [-3.0, 5.0, 2.0, -2.0, 4.0], [7.0, -7.0, -4.0, 2.0, 0.0], [-6.0, 2.0, -3.0, 2.0, 2.0]], [[-1.0, 1.0, 4.0, 6.0, 0.0], [-5.0, -2.0, -1.0, 0.0, 5.0], [6.0, -2.0, -3.0, 4.0, 5.0], [-5.0, -7.0, 2.0, -5.0, -7.0]]], [[[-1.0, 0.0, -2.0, 3.0, 3.0], [4.0, 0.0, 4.0, 0.0, 2.0], [-4.0, 0.0, 7.0, 1.0, -1.0], [5.0, -4.0, -7.0, -4.0, -5.0]], [[-6.0, 5.0, 0.0, 0.0, -5.0], [0.0, -4.0, 2.0, 0.0, 4.0], [7.0, 0.0, -1.0, 3.0, 0.0], [3.0, 3.0, -1.0, 4.0, -4.0]], [[-1.0, 6.0, 4.0, 1.0, -7.0], [-3.0, -6.0, 4.0, -6.0, -7.0], [-6.0, -2.0, -6.0, -3.0, -7.0], [-2.0, 6.0, 7.0, -3.0, 0.0]]]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-6.0, -3.0, 2.0], [4.0, -6.0, -4.0]], [[7.0, -7.0, 0.0], [-7.0, -1.0, -6.0]], [[7.0, 6.0, 6.0], [5.0, 0.0, 6.0]], [[3.0, -4.0, 4.0], [-1.0, -7.0, -7.0]], [[-6.0, -7.0, 7.0], [6.0, -5.0, 0.0]]], [[[6.0, -6.0, -6.0], [3.0, 0.0, -5.0]], [[1.0, 1.0, 1.0], [6.0, 7.0, 5.0]], [[-1.0, -5.0, 4.0], [-3.0, -1.0, 0.0]], [[-2.0, 2.0, -5.0], [0.0, 0.0, -3.0]], [[-2.0, -2.0, 4.0], [-5.0, 4.0, 5.0]]], [[[-2.0, -4.0, -3.0], [-3.0, -7.0, 0.0]], [[1.0, 1.0, 3.0], [-6.0, 7.0, 0.0]], [[0.0, -3.0, 1.0], [1.0, 0.0, -5.0]], [[-6.0, 5.0, -4.0], [1.0, -3.0, -3.0]], [[-4.0, 7.0, 0.0], [0.0, 0.0, 7.0]]], [[[-6.0, -5.0, 0.0], [2.0, -1.0, -7.0]], [[2.0, 2.0, 3.0], [5.0, 4.0, -5.0]], [[-3.0, -4.0, -4.0], [2.0, -2.0, 4.0]], [[1.0, -2.0, -2.0], [0.0, 4.0, 7.0]], [[-3.0, 0.0, -5.0], [4.0, -1.0, 7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-11.0, -67.0, -28.0, 50.0, 79.0], [3.0, 17.0, -50.0, 66.0, -8.0], [-19.0, 55.0, 70.0, 24.0, 52.0], [12.0, -36.0, 5.0, -2.0, -58.0]], [[61.0, 50.0, 4.0, 1.0, 68.0], [39.0, 19.0, -33.0, 8.0, 17.0], [-20.0, 26.0, 9.0, -34.0, 28.0], [23.0, -74.0, -20.0, -21.0, 74.0]], [[38.0, 42.0, 52.0, 7.0, -50.0], [-18.0, -4.0, 85.0, -90.0, 43.0], [-6.0, -101.0, -50.0, -5.0, -38.0], [-46.0, -63.0, -48.0, -105.0, -6.0]], [[63.0, -27.0, -4.0, 32.0, 104.0], [21.0, 48.0, -43.0, 32.0, 32.0], [-19.0, 19.0, 43.0, -5.0, 61.0], [-5.0, -116.0, -36.0, -52.0, 18.0]], [[-32.0, -5.0, 4.0, 114.0, 72.0], [-14.0, -41.0, -37.0, 50.0, -19.0], [-42.0, 65.0, 60.0, 29.0, 47.0], [16.0, -60.0, 40.0, -51.0, -115.0]]], [[[44.0, 42.0, -38.0, -8.0, 80.0], [99.0, 24.0, 16.0, 6.0, 5.0], [-84.0, 34.0, 87.0, -42.0, -16.0], [97.0, -54.0, -92.0, -21.0, 57.0]], [[-46.0, 65.0, 14.0, 21.0, -56.0], [5.0, -53.0, 64.0, -38.0, 17.0], [4.0, -24.0, -3.0, 14.0, -39.0], [31.0, 15.0, -22.0, -21.0, -56.0]], [[-7.0, 28.0, 20.0, 47.0, 20.0], [-21.0, -31.0, -33.0, 16.0, -13.0], [-2.0, 32.0, -11.0, 4.0, 26.0], [-9.0, -22.0, 52.0, -15.0, -26.0]], [[-4.0, -49.0, -36.0, -41.0, 9.0], [20.0, 34.0, -13.0, 26.0, -2.0], [10.0, 12.0, 27.0, 1.0, 2.0], [17.0, 35.0, -23.0, 52.0, 25.0]], [[-44.0, 56.0, 42.0, 30.0, -62.0], [-57.0, -68.0, -10.0, -14.0, -23.0], [36.0, 6.0, -71.0, 12.0, -4.0], [-33.0, 44.0, 94.0, 11.0, -37.0]]], [[[30.0, -22.0, -10.0, 1.0, 44.0], [7.0, 10.0, -41.0, 20.0, -71.0], [-75.0, 44.0, 13.0, -36.0, -6.0], [1.0, 18.0, 48.0, 5.0, 42.0]], [[-31.0, 37.0, 26.0, -8.0, -57.0], [-38.0, -27.0, -6.0, -8.0, 38.0], [94.0, -18.0, -63.0, 25.0, 18.0], [-29.0, 19.0, 31.0, 32.0, -10.0]], [[0.0, -8.0, -18.0, 22.0, 53.0], [23.0, 13.0, -23.0, 36.0, 30.0], [11.0, 29.0, 46.0, 14.0, 39.0], [28.0, -47.0, -31.0, 0.0, -18.0]], [[-13.0, -108.0, -42.0, -42.0, 8.0], [-6.0, 51.0, -30.0, 44.0, -7.0], [28.0, 9.0, 26.0, 19.0, 18.0], [-14.0, 49.0, -6.0, 65.0, 3.0]], [[-28.0, -31.0, 20.0, -27.0, -88.0], [-58.0, -15.0, 22.0, -32.0, -33.0], [23.0, -43.0, -66.0, 9.0, -37.0], [-60.0, 84.0, 56.0, 21.0, -14.0]]], [[[-36.0, -48.0, -44.0, 41.0, 79.0], [20.0, 9.0, -62.0, 88.0, 11.0], [16.0, 79.0, 83.0, 34.0, 65.0], [45.0, -21.0, -5.0, 41.0, -58.0]], [[-11.0, -9.0, -14.0, 12.0, 22.0], [22.0, 22.0, 19.0, 14.0, 90.0], [62.0, -20.0, 42.0, 40.0, 39.0], [22.0, -69.0, -88.0, -14.0, -44.0]], [[-15.0, 20.0, -10.0, 16.0, 5.0], [16.0, -34.0, 1.0, 2.0, -77.0], [-86.0, 43.0, 23.0, -28.0, -41.0], [37.0, 51.0, 41.0, 5.0, -3.0]], [[-24.0, 80.0, 22.0, 5.0, -58.0], [-1.0, -62.0, 39.0, -44.0, -48.0], [-44.0, -1.0, -33.0, -25.0, -62.0], [21.0, 57.0, 40.0, -6.0, 1.0]], [[-21.0, 14.0, -6.0, -5.0, -35.0], [8.0, -34.0, 32.0, -24.0, -79.0], [-83.0, 11.0, 5.0, -28.0, -69.0], [25.0, 79.0, 33.0, 5.0, -2.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[1.0, -3.0, 5.0], [6.0, 0.0, 2.0]], [[-5.0, -6.0, -1.0], [-7.0, 6.0, -2.0]], [[6.0, -3.0, 5.0], [7.0, -4.0, 2.0]], [[-6.0, 1.0, -2.0], [-5.0, -6.0, 4.0]], [[-3.0, 6.0, 4.0], [0.0, 2.0, 3.0]]], [[[2.0, -2.0, 1.0], [-7.0, -2.0, 7.0]], [[-3.0, -3.0, -7.0], [5.0, 0.0, 3.0]], [[-1.0, 5.0, 5.0], [3.0, -2.0, 4.0]], [[0.0, 2.0, -3.0], [6.0, 7.0, 2.0]], [[4.0, 4.0, -5.0], [1.0, -4.0, 5.0]]], [[[3.0, 4.0, -7.0], [-4.0, -2.0, 6.0]], [[7.0, -2.0, -2.0], [2.0, -4.0, 5.0]], [[-6.0, 0.0, -1.0], [6.0, 1.0, 7.0]], [[-7.0, -4.0, -6.0], [6.0, -6.0, 2.0]], [[-6.0, 0.0, 5.0], [7.0, 6.0, 1.0]]], [[[7.0, 3.0, 3.0], [7.0, -1.0, 0.0]], [[0.0, 2.0, 3.0], [0.0, -2.0, -3.0]], [[-6.0, 0.0, 0.0], [5.0, -5.0, 0.0]], [[-6.0, 7.0, 1.0], [-2.0, -4.0, 2.0]], [[3.0, 4.0, 6.0], [-7.0, 7.0, -5.0]]]])
      s1=numpy.array([[3.0, 6.0, -3.0], [-6.0, 3.0, 4.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-58.0, 4.0, -61.0, 22.0, 33.0], [55.0, -24.0, 4.0, 14.0, 53.0], [96.0, 11.0, -20.0, -73.0, -53.0], [-15.0, -15.0, -63.0, 29.0, 58.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3, 2))
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[2.0, -6.0, 3.0], [0.0, 0.0, -6.0]], [[6.0, -2.0, -4.0], [4.0, 2.0, 2.0]], [[-4.0, 0.0, -3.0], [3.0, 6.0, 2.0]], [[3.0, -2.0, 1.0], [6.0, -2.0, 1.0]], [[3.0, -5.0, -2.0], [3.0, -5.0, 5.0]]], [[[3.0, -2.0, 7.0], [0.0, -7.0, 6.0]], [[0.0, -6.0, -6.0], [7.0, -7.0, 5.0]], [[-6.0, -1.0, 2.0], [-3.0, 2.0, 6.0]], [[-6.0, 7.0, 1.0], [-1.0, -5.0, 5.0]], [[2.0, 0.0, -7.0], [-4.0, -1.0, 6.0]]], [[[-2.0, 0.0, -3.0], [-5.0, 0.0, -6.0]], [[1.0, 1.0, -3.0], [0.0, -6.0, 6.0]], [[-6.0, -6.0, 6.0], [1.0, 0.0, -3.0]], [[-5.0, -1.0, 0.0], [7.0, 4.0, 1.0]], [[2.0, 4.0, 0.0], [-1.0, -4.0, 6.0]]], [[[-2.0, 0.0, -3.0], [1.0, 6.0, 5.0]], [[-5.0, -7.0, 5.0], [-4.0, 0.0, -1.0]], [[-7.0, 4.0, 0.0], [0.0, 7.0, -1.0]], [[4.0, -6.0, -6.0], [4.0, 0.0, -2.0]], [[-5.0, -5.0, 0.0], [-7.0, -5.0, -3.0]]]])
      s1=numpy.array([[[7.0, -3.0], [2.0, -6.0], [2.0, 3.0]], [[2.0, -4.0], [7.0, -1.0], [-7.0, -1.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[50.0, 45.0], [38.0, -38.0], [0.0, -17.0], [10.0, -17.0], [-57.0, 3.0]], [[-60.0, 25.0], [-94.0, -8.0], [-74.0, 34.0], [-98.0, -17.0], [-57.0, -16.0]], [[12.0, 23.0], [-81.0, -18.0], [-19.0, 71.0], [-2.0, -12.0], [-50.0, -28.0]], [[-11.0, -18.0], [-40.0, 89.0], [15.0, -9.0], [26.0, -8.0], [-73.0, 81.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-5.0, -4.0, 3.0], [7.0, -3.0, 2.0]], [[-6.0, -1.0, -6.0], [7.0, -4.0, 0.0]], [[4.0, 1.0, 4.0], [1.0, -2.0, 0.0]], [[-2.0, 6.0, 3.0], [-2.0, -7.0, -7.0]], [[3.0, 1.0, 6.0], [2.0, 1.0, -2.0]]], [[[0.0, -4.0, -1.0], [6.0, -5.0, -5.0]], [[5.0, 1.0, -2.0], [0.0, -4.0, -7.0]], [[-1.0, 1.0, 0.0], [0.0, 0.0, 6.0]], [[-6.0, -7.0, -5.0], [-1.0, -4.0, 2.0]], [[1.0, -2.0, 0.0], [4.0, 3.0, -4.0]]], [[[0.0, -2.0, 4.0], [1.0, 5.0, -1.0]], [[6.0, 0.0, 7.0], [7.0, -1.0, 3.0]], [[-6.0, 6.0, 2.0], [1.0, 1.0, -2.0]], [[-6.0, 1.0, 5.0], [0.0, -1.0, -4.0]], [[-3.0, 6.0, 6.0], [0.0, 3.0, -1.0]]], [[[-3.0, -5.0, 6.0], [3.0, 0.0, -7.0]], [[6.0, -4.0, 3.0], [0.0, 3.0, 6.0]], [[2.0, -3.0, 3.0], [-2.0, 0.0, -4.0]], [[0.0, -7.0, 0.0], [2.0, 5.0, 3.0]], [[-3.0, 0.0, -3.0], [-5.0, 1.0, 7.0]]]])
      s1=numpy.array([[[[0.0, 5.0, -5.0, -1.0, -2.0], [0.0, -3.0, 7.0, 4.0, -2.0], [3.0, 2.0, -2.0, -2.0, -6.0], [0.0, -2.0, -2.0, 7.0, -6.0]], [[5.0, 0.0, 4.0, -5.0, 0.0], [-4.0, -1.0, 6.0, 0.0, 4.0], [-3.0, 2.0, 0.0, -2.0, -3.0], [3.0, -4.0, -1.0, -7.0, 4.0]], [[6.0, -4.0, 7.0, -4.0, -4.0], [-7.0, 6.0, 7.0, 0.0, 7.0], [1.0, -5.0, 0.0, 6.0, 3.0], [1.0, 3.0, 1.0, 6.0, 1.0]]], [[[5.0, 2.0, -6.0, 5.0, -2.0], [-7.0, 3.0, 6.0, -7.0, -4.0], [5.0, 4.0, -1.0, -1.0, 7.0], [6.0, -7.0, 0.0, -7.0, 1.0]], [[1.0, 1.0, -7.0, -2.0, -7.0], [-5.0, 4.0, 0.0, 7.0, -4.0], [0.0, 3.0, 4.0, -6.0, -3.0], [5.0, 6.0, 7.0, 7.0, 0.0]], [[6.0, 1.0, 1.0, -1.0, 5.0], [-3.0, -7.0, 5.0, 0.0, 1.0], [2.0, -6.0, 0.0, 7.0, -2.0], [-3.0, 7.0, -5.0, 1.0, 3.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[42.0, -24.0, 11.0, 52.0, 15.0], [-45.0, 32.0, 14.0, -90.0, 1.0], [39.0, -26.0, -9.0, 61.0, 105.0], [12.0, -18.0, -14.0, -57.0, 30.0]], [[-10.0, 4.0, -30.0, 78.0, 50.0], [17.0, -12.0, -48.0, -101.0, -46.0], [14.0, 32.0, -11.0, -5.0, 82.0], [13.0, -75.0, -21.0, -148.0, 33.0]], [[32.0, 4.0, 20.0, -16.0, -12.0], [-29.0, 6.0, 68.0, -5.0, 28.0], [18.0, -12.0, -17.0, 25.0, -2.0], [3.0, -19.0, -19.0, 24.0, -15.0]], [[-11.0, -40.0, 109.0, -29.0, 10.0], [25.0, 33.0, -4.0, -43.0, 78.0], [-45.0, 6.0, -22.0, 5.0, 24.0], [-5.0, -88.0, -13.0, -80.0, 16.0]], [[40.0, -6.0, 10.0, -22.0, -51.0], [-59.0, 50.0, 71.0, 5.0, 26.0], [18.0, 1.0, -4.0, 6.0, 12.0], [32.0, -14.0, 16.0, 41.0, -12.0]]], [[[-31.0, 6.0, -29.0, 69.0, 2.0], [21.0, 31.0, -20.0, -77.0, -32.0], [31.0, 36.0, -26.0, -9.0, 76.0], [13.0, -94.0, -7.0, -60.0, -26.0]], [[-53.0, 22.0, -14.0, 13.0, -9.0], [51.0, 5.0, -8.0, -8.0, -11.0], [-4.0, 52.0, -26.0, -49.0, -13.0], [2.0, -93.0, -6.0, -19.0, -49.0]], [[41.0, 1.0, 15.0, -10.0, 32.0], [-22.0, -40.0, 29.0, -4.0, 12.0], [6.0, -36.0, 2.0, 42.0, -9.0], [-15.0, 40.0, -29.0, -8.0, 28.0]], [[-62.0, -14.0, 3.0, 62.0, 72.0], [84.0, -38.0, -115.0, -45.0, -29.0], [-3.0, -29.0, -3.0, 35.0, 43.0], [-58.0, 22.0, -24.0, -42.0, 8.0]], [[-11.0, 12.0, -62.0, 27.0, -51.0], [-23.0, 51.0, -1.0, -3.0, -42.0], [21.0, 47.0, 6.0, -48.0, 27.0], [45.0, -32.0, 41.0, 10.0, -22.0]]], [[[18.0, -10.0, -22.0, -10.0, -58.0], [-49.0, 56.0, 17.0, 28.0, -5.0], [13.0, 1.0, 19.0, -10.0, 12.0], [32.0, 36.0, 46.0, 65.0, -6.0]], [[94.0, 18.0, -13.0, 0.0, -32.0], [-102.0, 20.0, 148.0, -32.0, 16.0], [66.0, -16.0, -23.0, 50.0, 31.0], [35.0, -25.0, -27.0, 31.0, -13.0]], [[36.0, -37.0, 53.0, -27.0, -15.0], [-44.0, 45.0, 4.0, -24.0, 40.0], [-33.0, 9.0, 15.0, -9.0, 32.0], [37.0, -21.0, 25.0, -74.0, 57.0]], [[10.0, -55.0, 72.0, -13.0, -21.0], [-22.0, 71.0, -21.0, -31.0, 51.0], [-24.0, -14.0, 8.0, 18.0, 59.0], [15.0, -11.0, 29.0, -30.0, 33.0]], [[63.0, -37.0, 59.0, -56.0, -44.0], [-78.0, 58.0, 52.0, 9.0, 59.0], [-23.0, -9.0, 18.0, 5.0, 11.0], [42.0, 11.0, 32.0, -7.0, 45.0]]], [[[-16.0, -40.0, 12.0, 26.0, -59.0], [-22.0, 108.0, -26.0, -33.0, 9.0], [13.0, 8.0, 3.0, 0.0, 86.0], [30.0, -26.0, 52.0, 22.0, -14.0]], [[37.0, 27.0, -40.0, -10.0, -15.0], [-38.0, -26.0, 69.0, 45.0, -13.0], [45.0, -38.0, 0.0, 38.0, -36.0], [-12.0, 73.0, -14.0, 115.0, -31.0]], [[-31.0, -10.0, 7.0, -5.0, -32.0], [17.0, 37.0, -15.0, 22.0, 9.0], [0.0, -1.0, -2.0, -6.0, 0.0], [-6.0, 3.0, 22.0, 63.0, -35.0]], [[-2.0, 12.0, -72.0, 32.0, -24.0], [-20.0, 12.0, -15.0, 21.0, -53.0], [37.0, -9.0, 18.0, 3.0, 14.0], [7.0, 65.0, 27.0, 73.0, -17.0]], [[0.0, -5.0, 24.0, -19.0, 56.0], [30.0, -69.0, -37.0, 30.0, 8.0], [-23.0, -50.0, 15.0, 36.0, -43.0], [-49.0, 87.0, -25.0, 10.0, 31.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[-2.0, 0.0, 5.0], [-5.0, -5.0, 3.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[6.0, -4.0, -3.0], [-3.0, -4.0, 0.0]], [[-6.0, 1.0, -6.0], [6.0, 7.0, 5.0]], [[2.0, -4.0, -4.0], [0.0, 3.0, -7.0]], [[3.0, 5.0, 4.0], [-5.0, 0.0, -5.0]], [[2.0, -4.0, -4.0], [1.0, -7.0, 3.0]]], [[[-4.0, -3.0, 2.0], [-2.0, 0.0, 4.0]], [[-1.0, -4.0, -6.0], [7.0, -5.0, -4.0]], [[7.0, -1.0, 0.0], [6.0, -3.0, 3.0]], [[-1.0, -3.0, 0.0], [2.0, 3.0, -4.0]], [[-6.0, 2.0, 7.0], [7.0, 4.0, -6.0]]], [[[4.0, -5.0, 5.0], [-6.0, -7.0, -6.0]], [[2.0, 5.0, -5.0], [-6.0, -1.0, -1.0]], [[2.0, 2.0, 4.0], [1.0, -3.0, 6.0]], [[6.0, 2.0, 1.0], [7.0, 7.0, -4.0]], [[-3.0, 0.0, 2.0], [-5.0, 2.0, 4.0]]], [[[-5.0, 0.0, -3.0], [-3.0, 1.0, -3.0]], [[2.0, -6.0, 0.0], [1.0, 5.0, 5.0]], [[0.0, -3.0, 4.0], [-7.0, 6.0, 2.0]], [[-6.0, -1.0, 7.0], [4.0, -4.0, 6.0]], [[-4.0, -2.0, -7.0], [7.0, 0.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[8.0, -68.0, -60.0, 24.0, 15.0], [40.0, -50.0, -20.0, -35.0, -26.0], [64.0, 3.0, 44.0, -89.0, 43.0], [-4.0, -19.0, 31.0, 65.0, -56.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[-5.0, -3.0], [-1.0, -1.0], [6.0, 4.0]], [[5.0, -5.0], [0.0, -2.0], [-3.0, 1.0]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-3.0, -2.0, 1.0], [6.0, -6.0, 5.0]], [[0.0, -1.0, -3.0], [2.0, -2.0, 4.0]], [[-1.0, 0.0, -1.0], [-3.0, 4.0, -4.0]], [[2.0, 5.0, 2.0], [1.0, 6.0, -1.0]], [[6.0, 2.0, -6.0], [6.0, 0.0, 2.0]]], [[[5.0, 6.0, -6.0], [2.0, 2.0, 2.0]], [[0.0, -1.0, -1.0], [5.0, 2.0, 7.0]], [[-2.0, 4.0, -4.0], [1.0, -7.0, 3.0]], [[-4.0, 2.0, 6.0], [5.0, -7.0, 6.0]], [[-5.0, 0.0, 0.0], [-5.0, 1.0, -2.0]]], [[[-5.0, -4.0, 6.0], [7.0, 7.0, 0.0]], [[5.0, 4.0, 2.0], [-5.0, -2.0, -4.0]], [[-2.0, 4.0, 1.0], [-6.0, -7.0, 7.0]], [[-5.0, 6.0, 1.0], [-6.0, -1.0, 4.0]], [[4.0, -7.0, -7.0], [5.0, -7.0, 5.0]]], [[[0.0, -4.0, 4.0], [6.0, 0.0, 7.0]], [[-6.0, -1.0, 7.0], [6.0, 4.0, 3.0]], [[4.0, 0.0, 3.0], [6.0, 5.0, 5.0]], [[0.0, -2.0, 0.0], [-2.0, 1.0, 7.0]], [[6.0, 0.0, 4.0], [4.0, -6.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[38.0, 2.0], [-19.0, -13.0], [-4.0, 2.0], [5.0, -21.0], [-44.0, -72.0]], [[-63.0, -57.0], [-1.0, -25.0], [-22.0, -2.0], [61.0, 29.0], [6.0, 36.0]], [[100.0, -6.0], [-30.0, 14.0], [-39.0, 57.0], [-17.0, 49.0], [-45.0, -39.0]], [[37.0, -3.0], [94.0, 12.0], [13.0, -35.0], [-29.0, 17.0], [20.0, -12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[[6.0, -5.0, -7.0, 1.0, 1.0], [5.0, -3.0, 4.0, -5.0, -1.0], [0.0, 2.0, 0.0, 5.0, 4.0], [-5.0, 0.0, 6.0, -2.0, -7.0]], [[4.0, 1.0, 3.0, -7.0, 0.0], [4.0, -5.0, 1.0, -5.0, -1.0], [3.0, -4.0, 0.0, -4.0, 7.0], [4.0, 3.0, 6.0, -1.0, -3.0]], [[0.0, -2.0, 4.0, 3.0, 7.0], [2.0, -2.0, 3.0, -7.0, -4.0], [0.0, -5.0, -1.0, -5.0, -7.0], [0.0, -7.0, -7.0, 4.0, 0.0]]], [[[-1.0, -6.0, 0.0, -5.0, -4.0], [-2.0, 0.0, -1.0, -5.0, 4.0], [-1.0, 2.0, -6.0, 0.0, -2.0], [-6.0, 5.0, -3.0, 3.0, 0.0]], [[6.0, 5.0, 7.0, 5.0, -3.0], [-2.0, 6.0, 0.0, 2.0, 0.0], [-2.0, 6.0, -3.0, 0.0, 0.0], [4.0, -6.0, 6.0, -1.0, 7.0]], [[-1.0, 6.0, 2.0, 1.0, 1.0], [-5.0, -3.0, 6.0, 0.0, -5.0], [-6.0, 3.0, -5.0, 7.0, -5.0], [7.0, 7.0, -2.0, -3.0, -7.0]]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-7.0, 0.0, 7.0], [2.0, -7.0, 1.0]], [[1.0, -2.0, -5.0], [-6.0, -3.0, 3.0]], [[-3.0, 1.0, -7.0], [7.0, -4.0, 5.0]], [[-1.0, -7.0, 6.0], [-3.0, -5.0, 5.0]], [[2.0, 4.0, 6.0], [-4.0, 7.0, -7.0]]], [[[-2.0, -2.0, 7.0], [0.0, 1.0, -7.0]], [[-4.0, -4.0, 5.0], [-3.0, -2.0, 4.0]], [[-5.0, 2.0, -5.0], [-3.0, 5.0, -5.0]], [[4.0, 0.0, -7.0], [0.0, 4.0, 2.0]], [[-3.0, -5.0, 0.0], [2.0, -2.0, 6.0]]], [[[-2.0, 7.0, 5.0], [0.0, 4.0, 3.0]], [[7.0, -7.0, 5.0], [5.0, 3.0, 1.0]], [[6.0, 6.0, -6.0], [-2.0, 4.0, 4.0]], [[-2.0, 2.0, 7.0], [5.0, 4.0, 5.0]], [[-5.0, 1.0, -5.0], [-1.0, 4.0, 3.0]]], [[[5.0, 6.0, -5.0], [6.0, 5.0, 2.0]], [[3.0, -2.0, -3.0], [-7.0, -5.0, 3.0]], [[1.0, 2.0, 1.0], [-2.0, -3.0, -5.0]], [[5.0, -1.0, 3.0], [-2.0, 2.0, -7.0]], [[7.0, 2.0, -7.0], [-5.0, 3.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-87.0, -20.0, 30.0, -30.0, 56.0], [-16.0, -38.0, -3.0, -38.0, -18.0], [6.0, -84.0, -3.0, -63.0, -86.0], [2.0, 10.0, -141.0, 52.0, -7.0]], [[-17.0, 42.0, -48.0, 18.0, 2.0], [-10.0, -10.0, 11.0, 64.0, -18.0], [-12.0, 14.0, 35.0, 59.0, 22.0], [32.0, 38.0, 23.0, -44.0, -43.0]], [[-50.0, -2.0, -22.0, -81.0, -63.0], [-56.0, -21.0, -9.0, 16.0, 33.0], [-26.0, 30.0, -48.0, 51.0, 5.0], [-4.0, 146.0, -18.0, -13.0, -45.0]], [[-66.0, 9.0, -15.0, 61.0, 73.0], [-30.0, -19.0, 40.0, 3.0, -53.0], [-38.0, -25.0, 2.0, 28.0, -114.0], [10.0, -13.0, -121.0, 14.0, -42.0]], [[81.0, -1.0, 57.0, 40.0, 32.0], [67.0, 25.0, -8.0, -38.0, -11.0], [44.0, -29.0, 32.0, -85.0, 37.0], [9.0, -141.0, 62.0, 18.0, 72.0]]], [[[-7.0, -43.0, 29.0, 31.0, 37.0], [29.0, 29.0, -31.0, -27.0, 11.0], [34.0, -46.0, 25.0, -86.0, -36.0], [-43.0, -110.0, -53.0, 54.0, 76.0]], [[-53.0, 38.0, 30.0, 48.0, 53.0], [-36.0, -2.0, 22.0, 16.0, -44.0], [-29.0, -23.0, -1.0, -1.0, -93.0], [42.0, -22.0, -94.0, 13.0, -2.0]], [[16.0, 50.0, 46.0, 1.0, -48.0], [-6.0, 60.0, -60.0, 75.0, 36.0], [29.0, 16.0, 33.0, -43.0, 60.0], [36.0, -39.0, 66.0, -11.0, 99.0]], [[46.0, 26.0, -24.0, 5.0, -55.0], [-12.0, 20.0, 7.0, 37.0, 14.0], [-20.0, 73.0, -15.0, 69.0, 55.0], [10.0, 39.0, 93.0, -46.0, -14.0]], [[-58.0, 24.0, 4.0, 18.0, 1.0], [-65.0, 4.0, 17.0, 26.0, -14.0], [-49.0, 24.0, -36.0, 47.0, -81.0], [17.0, 49.0, -78.0, 1.0, -20.0]]], [[[37.0, 45.0, 89.0, -13.0, 24.0], [5.0, -24.0, 32.0, -52.0, -40.0], [-5.0, -24.0, -32.0, -42.0, -9.0], [75.0, -17.0, 13.0, 4.0, 0.0]], [[26.0, -61.0, -27.0, 62.0, 14.0], [-4.0, 19.0, 37.0, -54.0, -5.0], [-38.0, 48.0, -49.0, 45.0, -71.0], [-74.0, -42.0, -34.0, 22.0, -14.0]], [[82.0, 44.0, -12.0, -20.0, -36.0], [18.0, -24.0, 38.0, 0.0, -16.0], [-12.0, 50.0, -14.0, 64.0, 92.0], [50.0, 54.0, 136.0, -64.0, -60.0]], [[10.0, 18.0, 86.0, 5.0, 20.0], [-31.0, -9.0, 40.0, -66.0, -33.0], [-37.0, 2.0, -74.0, -18.0, -78.0], [39.0, -7.0, -50.0, 26.0, 1.0]], [[-4.0, 80.0, 52.0, 1.0, -45.0], [-52.0, 35.0, -15.0, 68.0, 5.0], [-22.0, 42.0, -16.0, 17.0, 9.0], [72.0, 30.0, 32.0, -27.0, 39.0]]], [[[76.0, -8.0, 2.0, -55.0, -67.0], [7.0, -11.0, 17.0, -40.0, 23.0], [-10.0, 59.0, -56.0, 40.0, 75.0], [-3.0, 67.0, 109.0, -29.0, -32.0]], [[-16.0, 24.0, -68.0, 21.0, 28.0], [10.0, -32.0, 26.0, 41.0, -32.0], [-7.0, -6.0, 45.0, 59.0, 18.0], [20.0, 31.0, 12.0, -41.0, -71.0]], [[3.0, -38.0, -28.0, -20.0, 20.0], [50.0, -18.0, -19.0, -18.0, 10.0], [44.0, -48.0, 45.0, -43.0, 40.0], [-32.0, -28.0, 9.0, 12.0, 1.0]], [[47.0, -52.0, -26.0, 34.0, 21.0], [62.0, 17.0, -12.0, -27.0, 11.0], [37.0, -14.0, 38.0, -35.0, 31.0], [-58.0, -95.0, 35.0, 16.0, 31.0]], [[68.0, 56.0, -40.0, 17.0, -26.0], [8.0, -14.0, 44.0, 35.0, -26.0], [-25.0, 64.0, 3.0, 97.0, 76.0], [50.0, 47.0, 126.0, -77.0, -69.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, -6.0, -7.0], [0.0, 1.0, 5.0]])+(1.-msk_arg1)*numpy.array([[2.0, -5.0, 4.0], [0.0, -6.0, 1.0]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-7.0, 6.0, 5.0], [-7.0, 6.0, 0.0]], [[3.0, 0.0, -2.0], [-7.0, 7.0, 1.0]], [[0.0, -5.0, 2.0], [3.0, -2.0, 4.0]], [[6.0, -1.0, -3.0], [-4.0, -3.0, -1.0]], [[0.0, -5.0, 2.0], [1.0, 3.0, -6.0]]], [[[-5.0, 7.0, -4.0], [6.0, -7.0, 7.0]], [[-6.0, 4.0, 4.0], [0.0, -1.0, -4.0]], [[-1.0, -5.0, 0.0], [-1.0, 3.0, 6.0]], [[-5.0, 2.0, 6.0], [5.0, -1.0, -7.0]], [[6.0, 5.0, 5.0], [-1.0, -4.0, -5.0]]], [[[3.0, 5.0, -1.0], [5.0, -6.0, -4.0]], [[7.0, -5.0, -3.0], [0.0, 0.0, -1.0]], [[6.0, 4.0, 6.0], [0.0, -6.0, -2.0]], [[5.0, 2.0, 3.0], [-4.0, 7.0, 5.0]], [[2.0, 3.0, 4.0], [-4.0, 0.0, 3.0]]], [[[-7.0, 5.0, -2.0], [-5.0, -3.0, 4.0]], [[2.0, -1.0, 0.0], [5.0, 0.0, -5.0]], [[6.0, 0.0, 0.0], [6.0, -4.0, -6.0]], [[-1.0, -4.0, 7.0], [5.0, 0.0, 1.0]], [[2.0, 0.0, 3.0], [-1.0, 0.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-65.0, 26.0, 34.0, 19.0, -11.0], [14.0, -73.0, 63.0, -90.0, -94.0], [-49.0, 46.0, -82.0, -1.0, -31.0], [1.0, -19.0, -34.0, -20.0, 9.0]])+(1.-msk_ref)*numpy.array([[-60.0, -43.0, 49.0, 22.0, 9.0], [-12.0, -14.0, 11.0, 3.0, 26.0], [9.0, 26.0, 50.0, -25.0, 8.0], [-25.0, 4.0, 30.0, 47.0, 22.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, 0.0], [-7.0, -7.0], [7.0, 0.0]], [[-1.0, 6.0], [-2.0, 6.0], [-3.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -3.0], [-7.0, 6.0], [5.0, -4.0]], [[0.0, 7.0], [3.0, 7.0], [-1.0, 4.0]]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-3.0, 5.0, 6.0], [-1.0, 4.0, -1.0]], [[-7.0, 4.0, -6.0], [-7.0, -3.0, 6.0]], [[3.0, 4.0, 0.0], [-7.0, -2.0, 0.0]], [[-2.0, 5.0, -4.0], [-4.0, 2.0, 3.0]], [[0.0, 3.0, 2.0], [1.0, 1.0, 3.0]]], [[[5.0, 2.0, -4.0], [-5.0, 1.0, 4.0]], [[0.0, 6.0, -3.0], [-5.0, 2.0, -4.0]], [[0.0, 6.0, -3.0], [7.0, 7.0, 1.0]], [[-6.0, -3.0, 0.0], [7.0, 6.0, -4.0]], [[-6.0, 5.0, 0.0], [6.0, -4.0, 1.0]]], [[[-2.0, 2.0, -6.0], [0.0, -7.0, -1.0]], [[7.0, 0.0, -7.0], [-3.0, -4.0, 2.0]], [[-6.0, 0.0, 7.0], [2.0, -4.0, -1.0]], [[3.0, 0.0, 4.0], [-4.0, 1.0, 4.0]], [[-2.0, -7.0, -7.0], [-6.0, -5.0, -6.0]]], [[[7.0, 4.0, -4.0], [1.0, 3.0, 1.0]], [[7.0, -3.0, -4.0], [-7.0, -2.0, -2.0]], [[2.0, 5.0, 4.0], [-7.0, 6.0, -5.0]], [[-4.0, 6.0, 0.0], [4.0, -6.0, 1.0]], [[6.0, 7.0, -4.0], [-6.0, -2.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-15.0, -21.0], [-117.0, -64.0], [1.0, -82.0], [-84.0, -35.0], [-19.0, 3.0]], [[-21.0, -22.0], [-50.0, -76.0], [-87.0, 46.0], [-22.0, 83.0], [-72.0, -19.0]], [[-51.0, -60.0], [-2.0, -34.0], [22.0, -16.0], [36.0, -2.0], [22.0, -41.0]], [[-24.0, 0.0], [52.0, -41.0], [15.0, -61.0], [-61.0, -50.0], [-49.0, -73.0]]])+(1.-msk_ref)*numpy.array([[[2.0, 32.0], [-87.0, 23.0], [-28.0, -48.0], [-56.0, 50.0], [-11.0, 36.0]], [[-25.0, 1.0], [-47.0, 11.0], [-37.0, 150.0], [31.0, 75.0], [-60.0, 66.0]], [[-68.0, -11.0], [-35.0, -34.0], [12.0, -28.0], [25.0, -30.0], [1.0, -109.0]], [[-26.0, 51.0], [11.0, -94.0], [12.0, -19.0], [-69.0, 38.0], [-69.0, 8.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, -3.0, 5.0, 1.0, 3.0], [-2.0, -3.0, 7.0, -2.0, -6.0], [0.0, 3.0, 0.0, -5.0, 4.0], [0.0, -5.0, -4.0, -6.0, 2.0]], [[3.0, -1.0, -7.0, -3.0, -3.0], [0.0, -4.0, 2.0, -2.0, -6.0], [0.0, -1.0, 6.0, 7.0, -3.0], [2.0, 7.0, -5.0, 6.0, 0.0]], [[4.0, 5.0, 0.0, 7.0, 1.0], [7.0, 0.0, 1.0, 6.0, -1.0], [-3.0, -5.0, 1.0, -1.0, 6.0], [0.0, -4.0, 2.0, 5.0, -7.0]]], [[[0.0, 0.0, 4.0, 2.0, -6.0], [-3.0, 0.0, 3.0, -3.0, 0.0], [-1.0, -6.0, -1.0, -5.0, 7.0], [0.0, 0.0, 4.0, -4.0, -3.0]], [[7.0, 3.0, -6.0, -5.0, 4.0], [6.0, -1.0, 7.0, 6.0, -6.0], [-6.0, 0.0, 5.0, -7.0, 3.0], [-3.0, -5.0, 3.0, 7.0, -5.0]], [[-5.0, -1.0, 4.0, -1.0, 2.0], [-1.0, 7.0, -5.0, -4.0, 0.0], [0.0, 5.0, -3.0, -6.0, -5.0], [0.0, -5.0, 5.0, 1.0, 2.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, -4.0, -5.0, 7.0, 0.0], [-4.0, -7.0, -6.0, 6.0, 7.0], [6.0, 1.0, 1.0, 4.0, 1.0], [-7.0, 0.0, 1.0, 0.0, 6.0]], [[0.0, -1.0, 7.0, 4.0, -1.0], [2.0, 5.0, -7.0, -6.0, 1.0], [-2.0, 7.0, 2.0, -4.0, -5.0], [1.0, 7.0, 3.0, 6.0, -2.0]], [[-4.0, -6.0, -1.0, 0.0, 7.0], [3.0, 4.0, 4.0, -3.0, -4.0], [-5.0, -4.0, 3.0, 0.0, 1.0], [-3.0, -3.0, -2.0, 2.0, -1.0]]], [[[-6.0, -3.0, 3.0, -6.0, 2.0], [5.0, 0.0, -5.0, 4.0, -7.0], [-4.0, 6.0, 0.0, 2.0, 3.0], [0.0, 6.0, -3.0, -2.0, 7.0]], [[-5.0, 0.0, 1.0, 0.0, 1.0], [-4.0, -3.0, 2.0, 0.0, 2.0], [1.0, 0.0, 7.0, -4.0, -2.0], [-7.0, 7.0, -5.0, 3.0, 3.0]], [[7.0, -3.0, 4.0, 4.0, 0.0], [-4.0, -2.0, -3.0, -5.0, -5.0], [3.0, 5.0, 5.0, -7.0, -6.0], [-7.0, -6.0, 4.0, 2.0, -5.0]]]])
      res=tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-3.0, -6.0, 6.0], [-3.0, -6.0, 1.0]], [[-7.0, -5.0, -4.0], [-1.0, 4.0, 3.0]], [[-5.0, 6.0, 5.0], [-7.0, 4.0, 2.0]], [[-7.0, 0.0, 0.0], [-2.0, 6.0, -1.0]], [[5.0, 0.0, -2.0], [1.0, 4.0, 5.0]]], [[[-5.0, 4.0, -4.0], [6.0, 0.0, 3.0]], [[-2.0, -3.0, 5.0], [-7.0, 1.0, -2.0]], [[-2.0, -2.0, 5.0], [-7.0, 3.0, -3.0]], [[-4.0, 7.0, 5.0], [0.0, -1.0, -6.0]], [[6.0, -5.0, -4.0], [3.0, 0.0, 0.0]]], [[[5.0, 7.0, -6.0], [-1.0, -3.0, -5.0]], [[1.0, -2.0, 3.0], [-5.0, 3.0, 6.0]], [[4.0, 0.0, -7.0], [6.0, -1.0, 7.0]], [[-4.0, 0.0, -1.0], [-6.0, -5.0, 4.0]], [[7.0, 6.0, 2.0], [1.0, -6.0, 0.0]]], [[[-3.0, 1.0, 0.0], [6.0, -2.0, 0.0]], [[0.0, -5.0, 1.0], [0.0, -7.0, -4.0]], [[7.0, -2.0, -1.0], [7.0, -5.0, -6.0]], [[7.0, 4.0, -6.0], [3.0, 3.0, 6.0]], [[-1.0, 0.0, -7.0], [-7.0, 5.0, -3.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-32.0, 26.0, 55.0, 80.0, 11.0], [20.0, 46.0, -83.0, 23.0, 84.0], [21.0, -10.0, -60.0, 18.0, -2.0], [6.0, -26.0, 29.0, -17.0, -7.0]], [[3.0, 15.0, -16.0, -45.0, 18.0], [10.0, 58.0, -53.0, 15.0, 52.0], [-11.0, 25.0, -22.0, -37.0, -47.0], [-22.0, -19.0, 68.0, 27.0, 3.0]], [[71.0, 44.0, -111.0, -24.0, 34.0], [88.0, 1.0, -21.0, 65.0, -35.0], [-32.0, 6.0, 62.0, 57.0, -55.0], [0.0, 17.0, -6.0, 149.0, -40.0]], [[68.0, 40.0, -83.0, -40.0, 13.0], [57.0, 8.0, -8.0, 60.0, 6.0], [-34.0, -14.0, 35.0, 9.0, -19.0], [-18.0, 10.0, 33.0, 91.0, -40.0]], [[-20.0, -18.0, 25.0, -32.0, 33.0], [-8.0, 16.0, 39.0, -21.0, -52.0], [-19.0, 44.0, 2.0, -86.0, 2.0], [-12.0, -62.0, 17.0, -11.0, 11.0]]], [[[-4.0, -12.0, -17.0, -36.0, -61.0], [-39.0, 20.0, -28.0, -52.0, 10.0], [6.0, -20.0, 5.0, 9.0, -29.0], [8.0, 54.0, 31.0, 13.0, 6.0]], [[34.0, 39.0, -31.0, 25.0, 50.0], [68.0, 3.0, -19.0, 75.0, 19.0], [-14.0, 4.0, 5.0, 24.0, -5.0], [-9.0, -26.0, -2.0, 52.0, -27.0]], [[56.0, 45.0, -54.0, 13.0, 53.0], [81.0, -10.0, 2.0, 89.0, 1.0], [-26.0, -2.0, 24.0, 23.0, 3.0], [-13.0, -24.0, -6.0, 71.0, -39.0]], [[76.0, 33.0, -87.0, 21.0, -44.0], [43.0, -57.0, 14.0, 42.0, -17.0], [-9.0, -74.0, 60.0, 107.0, 20.0], [17.0, 84.0, -42.0, 78.0, -50.0]], [[-49.0, -33.0, 77.0, -1.0, 11.0], [-49.0, 2.0, 37.0, -35.0, -2.0], [9.0, 25.0, -37.0, -76.0, 36.0], [-10.0, -49.0, 5.0, -98.0, 31.0]]], [[[-14.0, -56.0, -30.0, -40.0, -28.0], [-62.0, -75.0, 44.0, -55.0, -48.0], [37.0, 19.0, 37.0, 86.0, -28.0], [23.0, 88.0, -105.0, -40.0, 60.0]], [[-6.0, 17.0, 5.0, -3.0, 66.0], [46.0, 44.0, -18.0, 29.0, -15.0], [-22.0, 50.0, -7.0, -54.0, -28.0], [-13.0, -76.0, 31.0, 44.0, -7.0]], [[-82.0, -57.0, 78.0, -35.0, -21.0], [-88.0, 38.0, -3.0, -102.0, -11.0], [21.0, 46.0, -39.0, -78.0, -22.0], [3.0, -22.0, 26.0, -83.0, 58.0]], [[-47.0, -12.0, 2.0, -2.0, 11.0], [-15.0, 45.0, -102.0, -26.0, 55.0], [39.0, 49.0, -32.0, 62.0, -99.0], [15.0, 29.0, -5.0, 12.0, 50.0]], [[-37.0, -35.0, 33.0, 35.0, -25.0], [-39.0, -39.0, 24.0, -53.0, -44.0], [29.0, -1.0, 7.0, 42.0, 11.0], [30.0, 29.0, -68.0, -42.0, 27.0]]], [[[-2.0, 2.0, 14.0, 16.0, -56.0], [-24.0, 7.0, -15.0, -26.0, 24.0], [6.0, -46.0, -10.0, 6.0, 21.0], [8.0, 32.0, 25.0, -14.0, -14.0]], [[-40.0, -7.0, 61.0, 61.0, -20.0], [-31.0, -1.0, -38.0, -10.0, 71.0], [39.0, -20.0, -52.0, 37.0, 20.0], [11.0, 16.0, -14.0, -78.0, 20.0]], [[-36.0, -33.0, 83.0, 51.0, -48.0], [-66.0, -50.0, 60.0, -43.0, 1.0], [26.0, -44.0, -27.0, -12.0, 92.0], [11.0, 10.0, -37.0, -128.0, 13.0]], [[-42.0, -52.0, 25.0, -62.0, 9.0], [-53.0, 2.0, 51.0, -73.0, -78.0], [-3.0, 59.0, 12.0, -73.0, -20.0], [-1.0, -28.0, -9.0, -33.0, 44.0]], [[25.0, -14.0, -75.0, -86.0, 46.0], [7.0, -23.0, 15.0, 23.0, -17.0], [-2.0, 59.0, 34.0, 30.0, -65.0], [-15.0, 23.0, -38.0, 31.0, 37.0]]]])+(1.-msk_ref)*numpy.array([[[[16.0, -12.0, -44.0, -23.0, 36.0], [23.0, 31.0, 84.0, -17.0, -47.0], [-27.0, -82.0, -34.0, 23.0, 30.0], [32.0, -126.0, 10.0, -34.0, -56.0]], [[-12.0, 51.0, 17.0, -51.0, -21.0], [-27.0, -10.0, 65.0, -19.0, -38.0], [5.0, -17.0, 14.0, -47.0, -15.0], [7.0, -19.0, -19.0, -18.0, -38.0]], [[-9.0, -1.0, 53.0, 39.0, 19.0], [-12.0, 69.0, 45.0, -119.0, -2.0], [-29.0, -15.0, 60.0, -88.0, -71.0], [-16.0, 1.0, 12.0, 76.0, -94.0]], [[-60.0, 37.0, 31.0, -41.0, 2.0], [-2.0, 33.0, 67.0, -45.0, -18.0], [-31.0, -24.0, 30.0, -49.0, -19.0], [14.0, 36.0, -35.0, 20.0, -33.0]], [[42.0, -26.0, 4.0, 49.0, -8.0], [-57.0, -65.0, -50.0, 15.0, 19.0], [55.0, 44.0, 52.0, -29.0, -32.0], [-92.0, 10.0, 6.0, 16.0, 26.0]]], [[[-24.0, 13.0, 87.0, -43.0, -20.0], [34.0, 33.0, -53.0, -33.0, -72.0], [-33.0, 90.0, 6.0, -45.0, -29.0], [30.0, 58.0, 9.0, 10.0, -7.0]], [[-7.0, 8.0, -44.0, 8.0, 25.0], [-14.0, 20.0, 96.0, -27.0, 24.0], [-8.0, -95.0, 4.0, 0.0, 7.0], [3.0, -59.0, -13.0, 5.0, -47.0]], [[-24.0, 10.0, -39.0, 8.0, 26.0], [-16.0, 21.0, 96.0, -28.0, 34.0], [-11.0, -93.0, 15.0, -5.0, 4.0], [-3.0, -32.0, -24.0, 15.0, -38.0]], [[-77.0, -3.0, 39.0, -24.0, 27.0], [73.0, 98.0, 11.0, -51.0, -13.0], [-82.0, -5.0, -12.0, 2.0, 4.0], [69.0, 63.0, -12.0, 37.0, -16.0]], [[28.0, -4.0, -52.0, 4.0, -17.0], [-31.0, -83.0, -32.0, 90.0, 32.0], [54.0, 5.0, -16.0, 50.0, 36.0], [-35.0, -5.0, -10.0, -44.0, 71.0]]], [[[35.0, 27.0, 4.0, 49.0, -54.0], [3.0, -5.0, -89.0, 27.0, 92.0], [32.0, 47.0, -45.0, 37.0, -3.0], [46.0, 70.0, 36.0, 13.0, 31.0]], [[50.0, -23.0, -10.0, 53.0, 16.0], [-60.0, -26.0, 33.0, -41.0, 4.0], [36.0, -25.0, 57.0, -52.0, -43.0], [-81.0, -68.0, 13.0, 25.0, -49.0]], [[66.0, -13.0, 32.0, 20.0, -38.0], [-31.0, -67.0, -105.0, 34.0, -23.0], [55.0, 103.0, 11.0, -17.0, -25.0], [-49.0, 8.0, 33.0, -15.0, 35.0]], [[73.0, 28.0, 14.0, 24.0, -24.0], [-13.0, 31.0, 28.0, -65.0, -12.0], [12.0, -16.0, -22.0, -36.0, -37.0], [38.0, -92.0, 57.0, 3.0, -100.0]], [[51.0, -49.0, 2.0, 67.0, 4.0], [19.0, 7.0, -93.0, 4.0, 28.0], [10.0, 47.0, -17.0, 30.0, -6.0], [-7.0, 0.0, 48.0, 20.0, 17.0]]], [[[-41.0, -7.0, 38.0, -53.0, 9.0], [52.0, 32.0, -23.0, 0.0, -66.0], [-46.0, 40.0, -15.0, 4.0, 14.0], [36.0, 29.0, -8.0, -12.0, 16.0]], [[3.0, 11.0, -59.0, -36.0, 5.0], [37.0, 8.0, 37.0, 47.0, -3.0], [-14.0, -59.0, -76.0, 76.0, 64.0], [69.0, -63.0, 2.0, -57.0, 8.0]], [[-20.0, -23.0, -56.0, -25.0, 4.0], [44.0, -36.0, -59.0, 115.0, 22.0], [0.0, 9.0, -65.0, 112.0, 83.0], [29.0, 32.0, -17.0, -55.0, 111.0]], [[68.0, -23.0, 35.0, 71.0, -37.0], [-59.0, -74.0, -121.0, 18.0, 32.0], [73.0, 107.0, 48.0, -36.0, -52.0], [-90.0, 49.0, 31.0, 27.0, 40.0]], [[19.0, 76.0, -16.0, 23.0, -58.0], [-60.0, -30.0, 32.0, 2.0, 95.0], [53.0, -30.0, -2.0, -17.0, -21.0], [14.0, 32.0, -3.0, 9.0, -18.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-7.0, 2.0, 3.0], [5.0, -1.0, -4.0]]),self.functionspace)
      arg1=numpy.array([-3.0, -4.0, -2.0])
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([7.0, -3.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[0.0, 3.0, -2.0], [-7.0, -3.0, -7.0]]),self.functionspace)
      arg1=numpy.array([[2.0, 6.0], [-4.0, -1.0], [-2.0, 3.0]])
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-8.0, -9.0], [12.0, -60.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[0.0, -1.0, -5.0], [1.0, -7.0, -2.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([-2.0, -1.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-19.0, -3.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-3.0, 0.0, -4.0], [1.0, -5.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[4.0, 1.0], [5.0, 2.0], [4.0, 3.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-28.0, -15.0], [-21.0, -9.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[5.0, -6.0, 5.0], [-3.0, 7.0, -5.0]]),self.functionspace)
      arg1=Data(numpy.array([-1.0, 3.0, -6.0]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([-53.0, 54.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[6.0, 3.0, -3.0], [-7.0, -7.0, -2.0]]),self.functionspace)
      arg1=Data(numpy.array([[5.0, -4.0], [3.0, 3.0], [3.0, -6.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[30.0, 3.0], [-62.0, 19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[6.0, 6.0, -5.0], [5.0, -6.0, -6.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, -2.0, 0.0])+(1.-msk_arg1)*numpy.array([-1.0, -6.0, -7.0])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, 32.0])+(1.-msk_ref)*numpy.array([-7.0, 73.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-3.0, 5.0, -2.0], [-1.0, -1.0, -5.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 4.0], [2.0, 6.0], [7.0, 1.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 0.0], [-1.0, 4.0], [-3.0, 1.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[11.0, 16.0], [-32.0, -15.0]])+(1.-msk_ref)*numpy.array([[13.0, 18.0], [20.0, -9.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[7.0, -6.0, 1.0], [-7.0, -2.0, 2.0]], [[0.0, -7.0, -1.0], [2.0, 0.0, -5.0]], [[3.0, 4.0, 3.0], [-2.0, 5.0, 7.0]], [[0.0, 0.0, 4.0], [0.0, 6.0, -7.0]], [[2.0, 3.0, -2.0], [-1.0, -7.0, 0.0]]], [[[3.0, 2.0, 1.0], [3.0, 0.0, 3.0]], [[-4.0, 0.0, -2.0], [-5.0, 3.0, -5.0]], [[-7.0, -7.0, -2.0], [3.0, 0.0, 5.0]], [[-1.0, 7.0, -3.0], [-4.0, 0.0, 6.0]], [[-7.0, 6.0, -1.0], [-4.0, -7.0, 2.0]]], [[[-3.0, 5.0, 0.0], [-4.0, -2.0, -4.0]], [[-1.0, 7.0, -6.0], [3.0, 0.0, 3.0]], [[-1.0, -1.0, -7.0], [1.0, 3.0, 6.0]], [[-2.0, 7.0, -4.0], [-2.0, -2.0, -3.0]], [[-6.0, -7.0, 0.0], [6.0, 4.0, -6.0]]], [[[-7.0, -4.0, 4.0], [-2.0, -7.0, 5.0]], [[-3.0, 1.0, -4.0], [1.0, -6.0, 7.0]], [[4.0, 0.0, 7.0], [-2.0, 6.0, -3.0]], [[-5.0, 2.0, 7.0], [3.0, 4.0, -2.0]], [[-7.0, 5.0, 5.0], [-5.0, 1.0, 3.0]]]]),self.functionspace)
      arg1=numpy.array([[-3.0, -4.0, 1.0], [-2.0, 5.0, -3.0]])
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[2.0, 38.0, -14.0, 55.0, -53.0], [-31.0, 50.0, 26.0, -38.0, -37.0], [-1.0, -46.0, -5.0, -23.0, 72.0], [-5.0, -52.0, 38.0, 34.0, 12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[-5.0, -4.0, 7.0], [0.0, 5.0, -2.0]], [[1.0, 2.0, -7.0], [6.0, 0.0, 5.0]], [[-2.0, 5.0, 2.0], [6.0, 4.0, 5.0]], [[4.0, -4.0, 0.0], [2.0, 3.0, 0.0]], [[5.0, 4.0, -5.0], [-1.0, -5.0, 0.0]]], [[[5.0, 5.0, -7.0], [4.0, -7.0, -3.0]], [[2.0, 7.0, 5.0], [3.0, -3.0, -1.0]], [[3.0, -5.0, -1.0], [-3.0, 6.0, -5.0]], [[-5.0, -1.0, 2.0], [-7.0, 3.0, -2.0]], [[-4.0, 2.0, 6.0], [-6.0, 4.0, 2.0]]], [[[-5.0, 2.0, -4.0], [6.0, -1.0, -4.0]], [[-3.0, 0.0, 6.0], [-5.0, -3.0, -4.0]], [[4.0, 0.0, 7.0], [1.0, -1.0, 4.0]], [[7.0, 5.0, 6.0], [-4.0, 3.0, 1.0]], [[1.0, 0.0, -4.0], [0.0, -5.0, 5.0]]], [[[-3.0, 1.0, 4.0], [6.0, 6.0, 3.0]], [[-3.0, 5.0, 3.0], [-2.0, -6.0, 0.0]], [[3.0, 4.0, 7.0], [7.0, 5.0, -4.0]], [[-7.0, -6.0, -6.0], [-7.0, -6.0, 2.0]], [[6.0, 4.0, -3.0], [0.0, -7.0, -3.0]]]]),self.functionspace)
      arg1=numpy.array([[[3.0, 6.0], [0.0, 7.0], [3.0, -4.0]], [[-6.0, -3.0], [-3.0, -2.0], [-1.0, -5.0]]])
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[-7.0, -86.0], [-59.0, 5.0], [-53.0, -36.0], [-9.0, -16.0], [21.0, 91.0]], [[-6.0, 110.0], [13.0, 43.0], [11.0, 9.0], [26.0, -20.0], [28.0, -34.0]], [[-56.0, 4.0], [52.0, -1.0], [26.0, -25.0], [53.0, 54.0], [1.0, 7.0]], [[-54.0, -72.0], [30.0, 23.0], [-23.0, 7.0], [19.0, -37.0], [33.0, 105.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[2.0, 6.0, -1.0], [-2.0, -2.0, -2.0]], [[-4.0, -6.0, 3.0], [1.0, 6.0, 0.0]], [[1.0, 2.0, -6.0], [-7.0, -5.0, -5.0]], [[5.0, 2.0, 6.0], [3.0, 0.0, -1.0]], [[-7.0, -1.0, 4.0], [1.0, -3.0, -5.0]]], [[[2.0, -6.0, -7.0], [-2.0, 5.0, 7.0]], [[6.0, -3.0, 6.0], [-2.0, -5.0, -4.0]], [[2.0, 7.0, -4.0], [-1.0, 6.0, -5.0]], [[0.0, -3.0, -7.0], [7.0, 0.0, -7.0]], [[-5.0, -5.0, -5.0], [0.0, -3.0, 6.0]]], [[[-6.0, -7.0, -3.0], [6.0, 3.0, 3.0]], [[-5.0, 6.0, -5.0], [3.0, -5.0, 6.0]], [[3.0, -3.0, -6.0], [-5.0, -6.0, -6.0]], [[-1.0, -3.0, 1.0], [-4.0, 1.0, 2.0]], [[6.0, -6.0, 1.0], [2.0, -6.0, 5.0]]], [[[6.0, 6.0, 5.0], [2.0, 6.0, 1.0]], [[6.0, -6.0, -1.0], [7.0, -4.0, 6.0]], [[-3.0, -2.0, 0.0], [5.0, 1.0, 0.0]], [[-3.0, 2.0, 4.0], [3.0, 4.0, -1.0]], [[-5.0, 0.0, 2.0], [1.0, 4.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[[[6.0, 0.0, -1.0, 6.0, -4.0], [-2.0, -4.0, 1.0, 1.0, -2.0], [0.0, -7.0, -1.0, 2.0, 6.0], [3.0, -3.0, -3.0, 4.0, 0.0]], [[-1.0, 6.0, 0.0, 3.0, 2.0], [0.0, 4.0, 5.0, 7.0, -5.0], [-4.0, 0.0, 7.0, 4.0, 2.0], [-5.0, 1.0, -6.0, 1.0, -7.0]], [[4.0, 6.0, -7.0, -4.0, -1.0], [-7.0, 6.0, 1.0, 7.0, 0.0], [-3.0, -4.0, -7.0, 1.0, 0.0], [1.0, -4.0, -5.0, 2.0, 3.0]]], [[[-5.0, 3.0, 1.0, 5.0, -1.0], [-1.0, 0.0, 0.0, 7.0, 7.0], [4.0, 0.0, 4.0, -7.0, -7.0], [-7.0, 5.0, 6.0, -5.0, -2.0]], [[-4.0, 7.0, 4.0, -3.0, -4.0], [-5.0, -3.0, 4.0, 0.0, 6.0], [6.0, -1.0, 0.0, 3.0, -7.0], [0.0, -7.0, -4.0, 4.0, 4.0]], [[6.0, -6.0, -6.0, 6.0, 0.0], [0.0, 6.0, -2.0, -1.0, -3.0], [-7.0, 2.0, -3.0, 1.0, -4.0], [-5.0, 3.0, 0.0, -6.0, -3.0]]]])
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[8.0, 22.0, 7.0, 18.0, 15.0], [15.0, 4.0, 27.0, 25.0, -54.0], [-27.0, -12.0, 45.0, 33.0, 60.0], [-1.0, 2.0, -41.0, 26.0, -43.0]], [[-35.0, 27.0, 8.0, -67.0, -24.0], [-44.0, -8.0, -7.0, -18.0, 81.0], [55.0, 10.0, -55.0, -18.0, -85.0], [14.0, -43.0, 15.0, 3.0, 73.0]], [[5.0, -50.0, 44.0, -14.0, 33.0], [72.0, -47.0, -5.0, -71.0, -76.0], [-13.0, 12.0, 42.0, 33.0, 114.0], [61.0, 8.0, -7.0, 39.0, -23.0]], [[31.0, 63.0, -38.0, 21.0, -25.0], [-55.0, 18.0, 23.0, 83.0, 4.0], [-7.0, -61.0, -18.0, 2.0, 17.0], [-5.0, -25.0, -39.0, 25.0, 1.0]], [[-48.0, 30.0, -2.0, -77.0, 33.0], [0.0, 27.0, -10.0, 26.0, 23.0], [13.0, 26.0, -9.0, -35.0, -10.0], [6.0, 15.0, 25.0, -8.0, 20.0]]], [[[22.0, -91.0, 23.0, 39.0, -31.0], [22.0, -47.0, -29.0, -110.0, 21.0], [18.0, 23.0, -24.0, 9.0, -49.0], [8.0, -8.0, 33.0, -24.0, 24.0]], [[69.0, 1.0, -46.0, -16.0, -14.0], [-27.0, -9.0, -15.0, 17.0, -29.0], [-16.0, -69.0, -65.0, 1.0, 95.0], [73.0, -32.0, -22.0, 47.0, 35.0]], [[-60.0, 87.0, 79.0, -4.0, -13.0], [-5.0, -52.0, 67.0, 21.0, 5.0], [51.0, -14.0, 86.0, 48.0, 11.0], [-1.0, -45.0, -58.0, 66.0, -20.0]], [[-102.0, 3.0, 98.0, 12.0, -6.0], [42.0, -96.0, -8.0, -14.0, 85.0], [110.0, 14.0, 77.0, -75.0, -27.0], [-6.0, 39.0, 95.0, -10.0, 7.0]], [[3.0, -117.0, -8.0, 20.0, 27.0], [60.0, 15.0, -59.0, -81.0, -1.0], [-25.0, 70.0, -13.0, -38.0, -43.0], [-25.0, 69.0, 82.0, -83.0, -10.0]]], [[[-65.0, -39.0, 27.0, -6.0, -5.0], [12.0, -13.0, -38.0, -37.0, 98.0], [58.0, 57.0, -7.0, -73.0, -125.0], [-43.0, 41.0, 99.0, -73.0, 31.0]], [[-15.0, -56.0, -13.0, 74.0, 54.0], [67.0, 65.0, -12.0, 17.0, -47.0], [-69.0, 72.0, 76.0, -21.0, -28.0], [-101.0, 109.0, 42.0, -95.0, -101.0]], [[10.0, -75.0, 46.0, -10.0, 17.0], [71.0, -78.0, -30.0, -89.0, -44.0], [16.0, -3.0, 16.0, -1.0, 113.0], [83.0, 11.0, 33.0, 34.0, 7.0]], [[29.0, -29.0, -18.0, -30.0, -3.0], [-6.0, 7.0, -15.0, -45.0, -11.0], [-15.0, 6.0, -49.0, 20.0, 1.0], [31.0, -25.0, -12.0, 7.0, 30.0]], [[90.0, -96.0, -65.0, 72.0, -15.0], [9.0, 6.0, -57.0, -20.0, -19.0], [-42.0, -30.0, -62.0, -38.0, 32.0], [10.0, 39.0, 49.0, -44.0, 2.0]]], [[[22.0, 108.0, -21.0, 32.0, -43.0], [-79.0, 18.0, 63.0, 96.0, 5.0], [-2.0, -66.0, 6.0, 46.0, -12.0], [-26.0, -61.0, -91.0, 48.0, -10.0]], [[55.0, -85.0, -44.0, 105.0, -26.0], [8.0, -6.0, -53.0, 0.0, 25.0], [-11.0, -22.0, -31.0, -68.0, -21.0], [-32.0, 61.0, 81.0, -71.0, -9.0]], [[-45.0, 10.0, 12.0, -2.0, -1.0], [-4.0, 1.0, -9.0, 18.0, 57.0], [34.0, 20.0, 9.0, -46.0, -64.0], [-34.0, 25.0, 47.0, -35.0, 8.0]], [[-41.0, 79.0, 0.0, -31.0, -7.0], [-45.0, 26.0, 29.0, 61.0, 44.0], [23.0, -1.0, 4.0, -4.0, -59.0], [-31.0, -21.0, -21.0, 5.0, 11.0]], [[-43.0, 43.0, 8.0, -45.0, 1.0], [-25.0, 20.0, 13.0, 16.0, 41.0], [22.0, 23.0, -5.0, -3.0, -65.0], [-20.0, -16.0, -5.0, -5.0, 20.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[-5.0, 5.0, 7.0], [-3.0, -6.0, 0.0]], [[5.0, 0.0, 0.0], [3.0, -6.0, -3.0]], [[0.0, 4.0, 5.0], [3.0, 0.0, -3.0]], [[7.0, 3.0, -4.0], [3.0, 7.0, 5.0]], [[7.0, -5.0, -7.0], [0.0, 3.0, 0.0]]], [[[4.0, -6.0, 6.0], [-5.0, -4.0, -4.0]], [[4.0, 2.0, -3.0], [6.0, 1.0, 7.0]], [[0.0, -2.0, 4.0], [-1.0, 0.0, -1.0]], [[0.0, 5.0, 7.0], [-1.0, 1.0, -4.0]], [[3.0, -6.0, -6.0], [3.0, -3.0, 0.0]]], [[[7.0, 3.0, -6.0], [2.0, -2.0, 4.0]], [[-5.0, -3.0, 6.0], [1.0, 6.0, -7.0]], [[4.0, -5.0, 6.0], [-3.0, 2.0, -1.0]], [[-4.0, 1.0, 0.0], [-3.0, -5.0, 6.0]], [[4.0, 7.0, 0.0], [2.0, 2.0, -3.0]]], [[[-3.0, 7.0, -1.0], [-6.0, -1.0, -2.0]], [[0.0, -4.0, -5.0], [-6.0, -3.0, -5.0]], [[-7.0, -4.0, -7.0], [-7.0, -4.0, -1.0]], [[0.0, -3.0, -5.0], [2.0, 7.0, 2.0]], [[7.0, 7.0, -6.0], [0.0, 1.0, 6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[3.0, -3.0, 2.0], [-1.0, 7.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-55.0, -30.0, -5.0, 50.0, 43.0], [19.0, 1.0, 15.0, 7.0, -9.0], [-16.0, 47.0, 56.0, -47.0, 3.0], [-33.0, -13.0, -44.0, 46.0, -5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[-4.0, 0.0, 0.0], [1.0, -3.0, 7.0]], [[5.0, 5.0, -4.0], [4.0, 6.0, 2.0]], [[0.0, 6.0, -1.0], [-1.0, 3.0, 1.0]], [[-7.0, 0.0, 1.0], [0.0, 5.0, -7.0]], [[-1.0, 5.0, 6.0], [-5.0, 4.0, -3.0]]], [[[1.0, -7.0, 1.0], [4.0, 0.0, 0.0]], [[-6.0, 7.0, -4.0], [0.0, 0.0, -5.0]], [[2.0, 0.0, -7.0], [-4.0, -5.0, -6.0]], [[4.0, -5.0, -3.0], [-7.0, -5.0, -6.0]], [[4.0, -6.0, -1.0], [5.0, 7.0, -3.0]]], [[[3.0, 0.0, -5.0], [0.0, 0.0, 1.0]], [[0.0, 0.0, -4.0], [0.0, -7.0, -3.0]], [[-3.0, 7.0, -3.0], [-5.0, 4.0, 1.0]], [[6.0, 1.0, 1.0], [-1.0, -6.0, -1.0]], [[6.0, -1.0, 4.0], [-5.0, -3.0, -2.0]]], [[[4.0, 6.0, 0.0], [3.0, 1.0, -1.0]], [[-6.0, 0.0, 5.0], [-6.0, 7.0, 7.0]], [[-3.0, 1.0, 7.0], [6.0, -6.0, 7.0]], [[-3.0, -7.0, -2.0], [7.0, -6.0, 3.0]], [[7.0, -5.0, 5.0], [-4.0, -4.0, 1.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[0.0, 4.0], [-5.0, 7.0], [-5.0, 7.0]], [[2.0, -6.0], [-4.0, -3.0], [4.0, -2.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[42.0, -27.0], [-13.0, -19.0], [-35.0, 30.0], [-53.0, -22.0], [-93.0, 97.0]], [[38.0, -62.0], [-35.0, 7.0], [23.0, 10.0], [22.0, 29.0], [5.0, -78.0]], [[29.0, -25.0], [36.0, -1.0], [-42.0, 32.0], [8.0, 64.0], [-21.0, 88.0]], [[-32.0, 39.0], [-37.0, 12.0], [24.0, 12.0], [95.0, -105.0], [12.0, 62.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[6.0, -5.0, 7.0], [-2.0, 2.0, 5.0]], [[-2.0, -4.0, 5.0], [0.0, 7.0, 5.0]], [[-2.0, 2.0, 6.0], [0.0, 1.0, 6.0]], [[-3.0, 4.0, -2.0], [0.0, 1.0, -7.0]], [[0.0, 1.0, 4.0], [1.0, -4.0, 3.0]]], [[[-6.0, 4.0, 3.0], [-5.0, 0.0, -7.0]], [[4.0, 3.0, -3.0], [-2.0, -7.0, -4.0]], [[5.0, 1.0, 7.0], [-7.0, -6.0, 2.0]], [[0.0, -5.0, -3.0], [-3.0, 4.0, 0.0]], [[5.0, 3.0, 6.0], [-2.0, 4.0, 0.0]]], [[[-2.0, 6.0, -6.0], [6.0, -2.0, 7.0]], [[-1.0, -4.0, -4.0], [-1.0, 5.0, 0.0]], [[-6.0, 1.0, 7.0], [1.0, 2.0, -4.0]], [[7.0, -5.0, 0.0], [-2.0, 1.0, -4.0]], [[-6.0, -4.0, 1.0], [-5.0, 3.0, 3.0]]], [[[-1.0, -2.0, -7.0], [3.0, 7.0, 1.0]], [[4.0, 7.0, 4.0], [6.0, 2.0, 7.0]], [[0.0, -6.0, 7.0], [0.0, 7.0, -1.0]], [[-6.0, 6.0, 5.0], [-7.0, -3.0, 5.0]], [[3.0, 2.0, -7.0], [-3.0, -3.0, 0.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[[1.0, 2.0, 0.0, 2.0, 3.0], [1.0, -1.0, 2.0, -7.0, -4.0], [-6.0, 2.0, -7.0, -1.0, -5.0], [1.0, 3.0, -7.0, 0.0, -2.0]], [[-4.0, 1.0, 4.0, -3.0, 3.0], [2.0, -7.0, -6.0, 5.0, 7.0], [-2.0, 0.0, -4.0, -1.0, 4.0], [-7.0, -5.0, 4.0, 0.0, -1.0]], [[-5.0, 2.0, 4.0, -5.0, 1.0], [6.0, 4.0, 5.0, 2.0, 6.0], [3.0, -1.0, 2.0, 2.0, -5.0], [0.0, 6.0, -2.0, -5.0, 2.0]]], [[[0.0, -7.0, -1.0, -7.0, 6.0], [3.0, -4.0, 2.0, -3.0, -1.0], [3.0, 4.0, 3.0, 6.0, -2.0], [0.0, 2.0, 5.0, 0.0, -4.0]], [[0.0, 3.0, 7.0, -3.0, 5.0], [4.0, -4.0, -2.0, 0.0, 0.0], [7.0, -7.0, 5.0, 0.0, -5.0], [-1.0, 0.0, -3.0, 1.0, -1.0]], [[-4.0, 1.0, -5.0, 0.0, -3.0], [4.0, 7.0, -1.0, 7.0, 4.0], [0.0, -3.0, -4.0, 4.0, -1.0], [-7.0, -7.0, 6.0, 4.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-29.0, 46.0, -1.0, 0.0, -7.0], [60.0, 92.0, 64.0, -12.0, 5.0], [3.0, -32.0, -24.0, 21.0, -96.0], [4.0, 46.0, -62.0, -13.0, 13.0]], [[-31.0, 28.0, 28.0, -38.0, 7.0], [68.0, 57.0, 26.0, 39.0, 30.0], [84.0, -73.0, 55.0, 36.0, -71.0], [-16.0, 9.0, -3.0, 2.0, 11.0]], [[-64.0, 19.0, 9.0, -43.0, -7.0], [66.0, 50.0, 6.0, 78.0, 82.0], [33.0, -35.0, -1.0, 36.0, -23.0], [-59.0, -22.0, 43.0, -5.0, 13.0]], [[19.0, -10.0, 50.0, -11.0, 27.0], [-31.0, -86.0, -35.0, -12.0, 0.0], [11.0, 10.0, 34.0, -33.0, 43.0], [17.0, 8.0, -4.0, -17.0, -3.0]], [[-36.0, -7.0, -24.0, -18.0, -16.0], [25.0, 42.0, 21.0, 31.0, 42.0], [-15.0, 19.0, -25.0, 25.0, -1.0], [-24.0, 0.0, 31.0, -12.0, 7.0]]], [[[-9.0, 26.0, 68.0, -4.0, -12.0], [-23.0, -39.0, -24.0, 34.0, 47.0], [22.0, -14.0, 45.0, -50.0, 48.0], [15.0, 19.0, -15.0, -43.0, 34.0]], [[23.0, -6.0, -27.0, 49.0, -17.0], [-58.0, -29.0, -11.0, -41.0, -27.0], [-94.0, 64.0, -71.0, -41.0, 50.0], [18.0, 3.0, -23.0, -8.0, -2.0]], [[-42.0, 58.0, -13.0, 39.0, -53.0], [12.0, 82.0, 35.0, 19.0, 44.0], [-74.0, 11.0, -84.0, -26.0, -14.0], [-10.0, 24.0, -50.0, -33.0, 37.0]], [[35.0, 22.0, -1.0, 39.0, -16.0], [-21.0, 19.0, 1.0, -22.0, -50.0], [20.0, -37.0, 25.0, -19.0, -19.0], [31.0, 1.0, -41.0, 19.0, 7.0]], [[-37.0, 51.0, 66.0, -27.0, 38.0], [57.0, -10.0, 10.0, -2.0, 39.0], [4.0, -32.0, -21.0, -8.0, -59.0], [-20.0, 32.0, -57.0, -26.0, 3.0]]], [[[-24.0, -51.0, -55.0, -28.0, 11.0], [12.0, -31.0, -61.0, 63.0, 36.0], [-14.0, 19.0, -42.0, 48.0, 55.0], [-91.0, -109.0, 128.0, 56.0, -36.0]], [[35.0, 8.0, 4.0, 22.0, 0.0], [-16.0, -3.0, -10.0, -18.0, -47.0], [34.0, -37.0, 37.0, -9.0, -14.0], [22.0, -9.0, -21.0, 25.0, -3.0]], [[-29.0, -2.0, 65.0, -63.0, 20.0], [33.0, -13.0, 19.0, 30.0, 56.0], [72.0, -17.0, 81.0, 9.0, -9.0], [13.0, 49.0, 7.0, -49.0, 19.0]], [[43.0, 22.0, 9.0, 40.0, 11.0], [-21.0, 4.0, 42.0, -96.0, -77.0], [-31.0, 11.0, -14.0, -30.0, -52.0], [69.0, 70.0, -106.0, -15.0, -2.0]], [[-7.0, 33.0, -1.0, 21.0, -53.0], [1.0, 67.0, -2.0, 60.0, 19.0], [53.0, -63.0, 48.0, -6.0, 1.0], [-2.0, -23.0, 8.0, 10.0, 35.0]]], [[[38.0, -17.0, 5.0, -3.0, 34.0], [-6.0, -46.0, -34.0, -19.0, -51.0], [47.0, -35.0, 41.0, 11.0, -10.0], [-1.0, -36.0, 13.0, 46.0, -29.0]], [[-72.0, -6.0, 17.0, -81.0, 62.0], [96.0, -20.0, -13.0, 46.0, 79.0], [6.0, -7.0, -48.0, 61.0, -41.0], [-96.0, -36.0, 58.0, 10.0, -33.0]], [[-7.0, 28.0, 58.0, -38.0, 27.0], [54.0, 35.0, 58.0, -23.0, -4.0], [82.0, -53.0, 77.0, 16.0, -93.0], [42.0, 79.0, -65.0, -32.0, 13.0]], [[-75.0, 49.0, 5.0, 3.0, -67.0], [23.0, 59.0, -36.0, 138.0, 123.0], [-3.0, -39.0, -28.0, -12.0, 53.0], [-80.0, -67.0, 60.0, -8.0, 47.0]], [[30.0, 6.0, -38.0, 65.0, -25.0], [-56.0, -21.0, -41.0, -16.0, -37.0], [-73.0, 22.0, -67.0, -37.0, 49.0], [-8.0, -49.0, -5.0, 32.0, -7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[0.0, 6.0, -6.0], [0.0, 3.0, 7.0]], [[7.0, 0.0, 4.0], [-7.0, -6.0, -6.0]], [[5.0, -3.0, 2.0], [-5.0, -1.0, 4.0]], [[4.0, -3.0, 1.0], [0.0, -3.0, -2.0]], [[1.0, 5.0, -7.0], [4.0, 0.0, -1.0]]], [[[0.0, -2.0, 1.0], [-1.0, -3.0, 7.0]], [[0.0, 6.0, -3.0], [-7.0, 6.0, 3.0]], [[6.0, -1.0, 7.0], [-6.0, -5.0, 2.0]], [[6.0, 0.0, 0.0], [-7.0, -3.0, -4.0]], [[0.0, 1.0, 0.0], [-4.0, -5.0, -4.0]]], [[[3.0, 5.0, -2.0], [7.0, 3.0, -2.0]], [[-5.0, 0.0, 4.0], [-4.0, 5.0, 0.0]], [[3.0, -4.0, 5.0], [2.0, -1.0, 0.0]], [[-4.0, 0.0, 4.0], [5.0, 0.0, 4.0]], [[0.0, 6.0, -6.0], [-5.0, -7.0, -1.0]]], [[[1.0, -3.0, 0.0], [4.0, 4.0, 5.0]], [[-7.0, 1.0, -7.0], [3.0, 3.0, -3.0]], [[5.0, 5.0, -5.0], [0.0, 3.0, 0.0]], [[0.0, -6.0, -5.0], [-1.0, 6.0, 5.0]], [[-5.0, -4.0, -4.0], [4.0, 0.0, -6.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[-1.0, -6.0, -6.0], [0.0, 6.0, -7.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-31.0, -25.0, -33.0, 4.0, 18.0], [-61.0, -3.0, -86.0, 4.0, -8.0], [11.0, 11.0, -15.0, -48.0, -35.0], [6.0, 82.0, 13.0, 67.0, 95.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[-1.0, 2.0, -7.0], [-6.0, 2.0, 6.0]], [[0.0, 3.0, 6.0], [-6.0, 5.0, -3.0]], [[5.0, 5.0, -2.0], [2.0, 0.0, -2.0]], [[-1.0, 1.0, 5.0], [3.0, 1.0, -7.0]], [[-6.0, 0.0, -7.0], [-2.0, -2.0, 7.0]]], [[[-7.0, 4.0, 1.0], [1.0, 2.0, -2.0]], [[6.0, -5.0, -3.0], [1.0, 0.0, 7.0]], [[-4.0, 4.0, 2.0], [7.0, 0.0, 3.0]], [[-5.0, -7.0, 4.0], [-4.0, -5.0, 3.0]], [[0.0, 6.0, 5.0], [-2.0, 7.0, 3.0]]], [[[-2.0, 1.0, 3.0], [0.0, -6.0, -7.0]], [[7.0, -7.0, -6.0], [-2.0, 6.0, 3.0]], [[4.0, 2.0, -1.0], [1.0, 4.0, -4.0]], [[1.0, 0.0, -6.0], [0.0, -7.0, 0.0]], [[-6.0, 0.0, 1.0], [1.0, -6.0, -6.0]]], [[[-6.0, -4.0, 1.0], [-5.0, 4.0, -4.0]], [[-7.0, 3.0, 7.0], [7.0, 7.0, 0.0]], [[3.0, 3.0, -2.0], [-4.0, 0.0, 0.0]], [[5.0, 3.0, -4.0], [-2.0, -4.0, -6.0]], [[5.0, 1.0, 7.0], [-1.0, 7.0, 2.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.0, -7.0], [4.0, 7.0], [5.0, 1.0]], [[-3.0, 1.0], [2.0, -6.0], [-3.0, 0.0]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[-23.0, -4.0], [79.0, -9.0], [10.0, 0.0], [43.0, 16.0], [-54.0, 45.0]], [[28.0, 67.0], [-59.0, -79.0], [-4.0, 65.0], [-15.0, 16.0], [60.0, 3.0]], [[28.0, 60.0], [-49.0, -142.0], [20.0, -38.0], [-44.0, 29.0], [8.0, 80.0]], [[24.0, -14.0], [40.0, 42.0], [14.0, -6.0], [8.0, 4.0], [50.0, -64.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[0.0, 1.0, -6.0], [-6.0, 6.0, -2.0]], [[5.0, -2.0, 0.0], [-7.0, 0.0, 4.0]], [[-3.0, -4.0, -6.0], [-7.0, -6.0, 2.0]], [[-4.0, -2.0, -7.0], [0.0, 6.0, 3.0]], [[-3.0, 3.0, -4.0], [0.0, 5.0, 4.0]]], [[[-5.0, 1.0, -6.0], [-4.0, 5.0, 4.0]], [[2.0, 7.0, 6.0], [-7.0, 0.0, -3.0]], [[-6.0, -6.0, 7.0], [-2.0, 6.0, 6.0]], [[-5.0, 7.0, 6.0], [0.0, 3.0, -7.0]], [[7.0, 5.0, -2.0], [1.0, -1.0, -1.0]]], [[[-2.0, 0.0, 6.0], [0.0, -3.0, 0.0]], [[0.0, 7.0, -1.0], [-6.0, 3.0, 2.0]], [[-3.0, 0.0, -2.0], [4.0, -5.0, 0.0]], [[-6.0, 3.0, 3.0], [0.0, 3.0, 6.0]], [[-2.0, 4.0, -2.0], [-4.0, -5.0, -7.0]]], [[[-4.0, 1.0, 0.0], [0.0, 3.0, 5.0]], [[5.0, 1.0, -7.0], [6.0, 0.0, 0.0]], [[-7.0, 3.0, 6.0], [-6.0, -2.0, 2.0]], [[0.0, -6.0, 6.0], [5.0, 7.0, 0.0]], [[-5.0, -4.0, -5.0], [6.0, -2.0, -5.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-2.0, 7.0, 4.0, -6.0, 4.0], [3.0, 1.0, 7.0, 2.0, -5.0], [-6.0, -2.0, -2.0, 0.0, -1.0], [-3.0, -2.0, 1.0, -5.0, 6.0]], [[-1.0, 0.0, 6.0, 7.0, -1.0], [7.0, 0.0, -5.0, -6.0, 2.0], [-1.0, 2.0, -4.0, -2.0, 6.0], [7.0, -1.0, -4.0, 2.0, 4.0]], [[3.0, -6.0, 4.0, -3.0, -1.0], [0.0, 0.0, -3.0, -7.0, -6.0], [7.0, 3.0, 1.0, 2.0, -6.0], [-5.0, -2.0, 2.0, 0.0, 3.0]]], [[[1.0, -7.0, 0.0, -5.0, 6.0], [-5.0, -2.0, 4.0, -2.0, 4.0], [-5.0, -4.0, 2.0, 4.0, -7.0], [-5.0, 1.0, -3.0, -6.0, 6.0]], [[-3.0, 4.0, 2.0, -1.0, -1.0], [-3.0, -1.0, -3.0, 0.0, 5.0], [1.0, 7.0, -6.0, -5.0, 4.0], [5.0, -2.0, 4.0, 5.0, 2.0]], [[0.0, 7.0, 6.0, -4.0, 3.0], [6.0, -5.0, 0.0, 5.0, -2.0], [-4.0, -2.0, 3.0, -6.0, -1.0], [-4.0, 5.0, 0.0, -4.0, 4.0]]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-43.0, 88.0, -18.0, 57.0, -43.0], [7.0, 16.0, -29.0, 38.0, 48.0], [1.0, 54.0, -64.0, -56.0, 110.0], [105.0, -17.0, 26.0, 76.0, -46.0]], [[-15.0, 112.0, 32.0, -25.0, -8.0], [60.0, -1.0, 17.0, 56.0, -65.0], [-9.0, 6.0, -4.0, -48.0, 28.0], [-10.0, 5.0, 34.0, -3.0, -4.0]], [[3.0, 54.0, -60.0, 41.0, -32.0], [28.0, 7.0, 7.0, 84.0, -19.0], [1.0, -38.0, 44.0, -14.0, 38.0], [8.0, 37.0, -2.0, 11.0, -98.0]], [[-29.0, 59.0, -26.0, 13.0, -4.0], [-26.0, -25.0, -15.0, 68.0, 82.0], [-29.0, 19.0, -18.0, -58.0, 55.0], [51.0, 27.0, 14.0, 34.0, -29.0]], [[-24.0, 51.0, 24.0, 30.0, -4.0], [21.0, -28.0, -39.0, 24.0, 62.0], [-24.0, 27.0, -28.0, -63.0, 61.0], [59.0, 21.0, -3.0, 30.0, 8.0]]], [[[-28.0, 77.0, -4.0, 54.0, -32.0], [21.0, -22.0, -53.0, 54.0, 64.0], [-4.0, 37.0, -26.0, -79.0, 91.0], [81.0, 27.0, 11.0, 60.0, -42.0]], [[0.0, 6.0, 56.0, 66.0, -56.0], [72.0, 31.0, -67.0, -81.0, -54.0], [70.0, 62.0, -49.0, -12.0, 56.0], [60.0, -45.0, 7.0, 58.0, 4.0]], [[19.0, -4.0, 16.0, -47.0, -25.0], [-32.0, -38.0, -59.0, 9.0, -14.0], [83.0, 59.0, 21.0, -48.0, -40.0], [-43.0, 20.0, 62.0, 36.0, -15.0]], [[12.0, -108.0, 10.0, 86.0, -57.0], [-17.0, 27.0, -97.0, -129.0, 32.0], [96.0, 77.0, -51.0, 25.0, 30.0], [77.0, -50.0, -9.0, 82.0, -6.0]], [[-21.0, 43.0, 42.0, -1.0, 29.0], [48.0, 11.0, 37.0, -9.0, -12.0], [-63.0, -19.0, -31.0, 1.0, 25.0], [18.0, -17.0, -24.0, -32.0, 56.0]]], [[[31.0, -62.0, 10.0, -3.0, -11.0], [3.0, 1.0, -23.0, -46.0, -41.0], [51.0, 1.0, 28.0, 27.0, -46.0], [-39.0, -2.0, -2.0, -5.0, 0.0]], [[-25.0, 74.0, 56.0, 71.0, -39.0], [82.0, -1.0, -65.0, -13.0, 7.0], [11.0, 52.0, -53.0, -67.0, 100.0], [91.0, -7.0, 0.0, 57.0, 3.0]], [[19.0, -57.0, -30.0, 9.0, 19.0], [-14.0, -6.0, 16.0, 0.0, 18.0], [-21.0, -51.0, 42.0, 37.0, -33.0], [-26.0, 24.0, -39.0, -34.0, -10.0]], [[9.0, -6.0, 48.0, 21.0, -15.0], [30.0, -39.0, -75.0, -21.0, 21.0], [33.0, 36.0, 3.0, -51.0, 12.0], [15.0, 27.0, 0.0, 27.0, 15.0]], [[5.0, -43.0, -44.0, 99.0, -50.0], [15.0, 46.0, -29.0, -41.0, 3.0], [37.0, 1.0, -13.0, 39.0, 53.0], [67.0, -25.0, -30.0, 45.0, -64.0]]], [[[-2.0, 19.0, 26.0, 8.0, -5.0], [16.0, -32.0, -42.0, 11.0, 27.0], [6.0, 21.0, 1.0, -47.0, 17.0], [14.0, 26.0, 4.0, 17.0, 6.0]], [[-26.0, 35.0, -2.0, -32.0, 62.0], [-8.0, -7.0, 75.0, 41.0, 43.0], [-110.0, -53.0, -9.0, 8.0, 1.0], [-3.0, 9.0, -31.0, -59.0, 49.0]], [[29.0, -37.0, 22.0, 69.0, -65.0], [48.0, -3.0, -100.0, -52.0, -33.0], [101.0, 44.0, 14.0, -20.0, 21.0], [24.0, 7.0, 3.0, 59.0, -44.0]], [[8.0, -43.0, 2.0, -92.0, 23.0], [-88.0, -17.0, 11.0, -16.0, 7.0], [30.0, 35.0, -2.0, 9.0, -79.0], [-62.0, -15.0, 49.0, -7.0, 38.0]], [[11.0, -90.0, -98.0, 9.0, 12.0], [-97.0, 10.0, 30.0, 12.0, 71.0], [-13.0, -41.0, 30.0, 62.0, -34.0], [-8.0, 9.0, -25.0, -9.0, -49.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[0.0, -5.0, -6.0], [3.0, 1.0, -4.0]], [[1.0, 7.0, -2.0], [0.0, 7.0, -4.0]], [[5.0, -6.0, -6.0], [-2.0, 1.0, 5.0]], [[1.0, 5.0, 6.0], [-4.0, -4.0, 0.0]], [[4.0, -7.0, 4.0], [-1.0, 3.0, 6.0]]], [[[2.0, 5.0, -1.0], [-6.0, 1.0, 2.0]], [[6.0, 2.0, 6.0], [5.0, 7.0, 4.0]], [[-7.0, 1.0, 6.0], [0.0, 6.0, 0.0]], [[-5.0, -3.0, 1.0], [7.0, 6.0, 2.0]], [[3.0, -3.0, -4.0], [-3.0, -7.0, -1.0]]], [[[-4.0, -4.0, 4.0], [0.0, -4.0, 0.0]], [[-2.0, 7.0, 4.0], [2.0, -7.0, -1.0]], [[2.0, 6.0, 1.0], [2.0, 0.0, 6.0]], [[-2.0, 0.0, -6.0], [1.0, 4.0, 3.0]], [[-6.0, 7.0, 6.0], [1.0, 3.0, -7.0]]], [[[-7.0, 0.0, 3.0], [-6.0, -5.0, -5.0]], [[-5.0, -5.0, 7.0], [-7.0, 6.0, -6.0]], [[-1.0, -3.0, 5.0], [-7.0, -6.0, -1.0]], [[7.0, 1.0, -2.0], [4.0, 0.0, 0.0]], [[-3.0, -4.0, 2.0], [-1.0, -3.0, 1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 1.0, -3.0], [5.0, 4.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-1.0, 0.0, 7.0], [0.0, -5.0, -2.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[16.0, 20.0, 1.0, -54.0, -8.0], [-20.0, 23.0, 42.0, 86.0, -53.0], [-12.0, -17.0, 27.0, 61.0, 8.0], [-44.0, -36.0, -76.0, -8.0, -8.0]])+(1.-msk_ref)*numpy.array([[-39.0, -42.0, -62.0, 61.0, -3.0], [-18.0, -7.0, 19.0, -22.0, 6.0], [52.0, 67.0, -7.0, -66.0, 47.0], [63.0, 36.0, 68.0, -21.0, 30.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[-2.0, 7.0, -3.0], [-5.0, -1.0, -4.0]], [[2.0, -2.0, 4.0], [-7.0, 2.0, -2.0]], [[-2.0, 0.0, 2.0], [-3.0, 5.0, 4.0]], [[0.0, 0.0, 2.0], [2.0, 0.0, -3.0]], [[6.0, 6.0, -6.0], [4.0, 6.0, -1.0]]], [[[-1.0, 0.0, 1.0], [-2.0, 7.0, 2.0]], [[-6.0, -3.0, 1.0], [0.0, 7.0, -5.0]], [[0.0, 2.0, -3.0], [0.0, -6.0, 3.0]], [[6.0, -3.0, 4.0], [2.0, -2.0, 3.0]], [[-6.0, -1.0, 1.0], [5.0, -3.0, -1.0]]], [[[5.0, -5.0, -1.0], [4.0, -5.0, -4.0]], [[-2.0, 4.0, 7.0], [-4.0, 0.0, 5.0]], [[5.0, 2.0, 1.0], [-7.0, -7.0, 2.0]], [[4.0, 6.0, 7.0], [-3.0, -1.0, -1.0]], [[2.0, 5.0, -4.0], [-7.0, 6.0, -6.0]]], [[[2.0, -4.0, -4.0], [-5.0, -7.0, 1.0]], [[-7.0, 3.0, 7.0], [-3.0, -6.0, 1.0]], [[-4.0, -7.0, -5.0], [-1.0, 1.0, 7.0]], [[-5.0, 4.0, 3.0], [-5.0, 1.0, 3.0]], [[6.0, -5.0, 0.0], [-1.0, -3.0, -6.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-2.0, -2.0], [-7.0, -3.0], [-5.0, 3.0]], [[0.0, -7.0], [-1.0, -4.0], [5.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, -5.0], [0.0, -7.0], [-5.0, -1.0]], [[3.0, 3.0], [0.0, 3.0], [2.0, 2.0]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-49.0, 9.0], [-22.0, 53.0], [9.0, 15.0], [-25.0, -11.0], [-35.0, -101.0]], [[0.0, -7.0], [-4.0, -9.0], [22.0, 12.0], [6.0, 6.0], [12.0, -6.0]], [[15.0, -10.0], [-34.0, 46.0], [-12.0, 66.0], [-89.0, 19.0], [-55.0, -12.0]], [[56.0, 60.0], [-31.0, 72.0], [116.0, 24.0], [-19.0, 41.0], [-4.0, 16.0]]])+(1.-msk_ref)*numpy.array([[[-8.0, -62.0], [-45.0, -19.0], [-11.0, 22.0], [-10.0, -2.0], [40.0, -38.0]], [[-7.0, 23.0], [-15.0, 61.0], [21.0, -23.0], [-8.0, -7.0], [8.0, 40.0]], [[9.0, 0.0], [-37.0, -27.0], [-22.0, -78.0], [-46.0, -83.0], [-13.0, -56.0]], [[7.0, -12.0], [-42.0, -18.0], [36.0, 88.0], [-24.0, -12.0], [-15.0, -19.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-2.0, 0.0, -7.0], [-6.0, 0.0, 3.0]], [[-7.0, 0.0, -4.0], [-6.0, 5.0, 3.0]], [[6.0, 5.0, -3.0], [-7.0, -3.0, -4.0]], [[7.0, 0.0, 3.0], [0.0, 4.0, 6.0]], [[-5.0, 0.0, -1.0], [0.0, 0.0, -6.0]]], [[[-5.0, 1.0, 7.0], [-1.0, 1.0, -6.0]], [[6.0, -6.0, 1.0], [6.0, -3.0, -6.0]], [[-7.0, 6.0, -6.0], [-3.0, 0.0, -6.0]], [[0.0, 0.0, 7.0], [2.0, 0.0, -3.0]], [[-4.0, -5.0, 4.0], [-7.0, 2.0, -4.0]]], [[[4.0, -1.0, 0.0], [-3.0, 2.0, -4.0]], [[3.0, -2.0, -7.0], [3.0, 2.0, 5.0]], [[-3.0, 7.0, 4.0], [5.0, 0.0, -1.0]], [[2.0, 6.0, 5.0], [0.0, -6.0, 1.0]], [[-4.0, 4.0, 2.0], [-1.0, -4.0, -3.0]]], [[[4.0, -5.0, 5.0], [-3.0, -7.0, -3.0]], [[-6.0, -7.0, 5.0], [-6.0, 7.0, 1.0]], [[-2.0, 1.0, -7.0], [0.0, 5.0, 0.0]], [[3.0, 2.0, 1.0], [0.0, -2.0, 1.0]], [[-1.0, -1.0, 5.0], [7.0, 1.0, 3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, -4.0, -5.0, -6.0, -4.0], [0.0, 0.0, -5.0, 1.0, 6.0], [7.0, -1.0, -5.0, 4.0, 5.0], [-5.0, 5.0, 0.0, 1.0, -3.0]], [[3.0, 1.0, 3.0, 4.0, -5.0], [5.0, 7.0, -2.0, -1.0, 0.0], [-5.0, -3.0, -5.0, 6.0, 3.0], [6.0, -5.0, 0.0, -4.0, 0.0]], [[6.0, -1.0, -4.0, -6.0, 1.0], [-1.0, -3.0, 2.0, -7.0, -4.0], [0.0, -2.0, 7.0, 5.0, 1.0], [4.0, 0.0, 0.0, 6.0, 4.0]]], [[[6.0, -2.0, 7.0, 1.0, -1.0], [-7.0, -5.0, 3.0, 4.0, 0.0], [-7.0, 4.0, -4.0, 0.0, 0.0], [0.0, -6.0, -7.0, -2.0, 1.0]], [[0.0, 0.0, 2.0, 0.0, 2.0], [4.0, -5.0, 3.0, 2.0, 0.0], [1.0, -3.0, -4.0, -2.0, 2.0], [6.0, -3.0, 6.0, -7.0, 5.0]], [[-4.0, -2.0, -7.0, -4.0, -1.0], [-1.0, 5.0, 0.0, -2.0, -1.0], [4.0, 0.0, 4.0, 4.0, -5.0], [0.0, -1.0, 4.0, 1.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, 7.0, 3.0, 3.0, -6.0], [2.0, 6.0, -4.0, 0.0, -4.0], [-7.0, 7.0, 2.0, 5.0, -3.0], [-6.0, 1.0, -3.0, -7.0, 5.0]], [[5.0, -5.0, 0.0, 5.0, -3.0], [-4.0, 5.0, 6.0, 4.0, -7.0], [-6.0, 3.0, 6.0, 5.0, 2.0], [-1.0, 3.0, -4.0, 5.0, 0.0]], [[0.0, 6.0, 0.0, -1.0, 6.0], [-6.0, -1.0, 2.0, -1.0, 6.0], [3.0, 1.0, 1.0, -1.0, 2.0], [-3.0, -2.0, -4.0, 1.0, 3.0]]], [[[4.0, -6.0, 2.0, 2.0, 7.0], [-4.0, -3.0, 7.0, -5.0, 1.0], [4.0, -7.0, 4.0, 5.0, -4.0], [3.0, 1.0, -5.0, -7.0, -6.0]], [[-7.0, -3.0, -6.0, -2.0, 1.0], [0.0, 0.0, 0.0, 2.0, -1.0], [5.0, 3.0, 7.0, 6.0, 1.0], [-6.0, 5.0, -5.0, -2.0, -2.0]], [[-2.0, -6.0, -5.0, -7.0, -5.0], [0.0, -2.0, -1.0, 6.0, 5.0], [6.0, -6.0, 0.0, 7.0, -2.0], [1.0, -5.0, 2.0, -5.0, -6.0]]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-76.0, 21.0, -25.0, 36.0, 4.0], [46.0, 66.0, -22.0, 17.0, 13.0], [40.0, -8.0, -3.0, -31.0, -32.0], [-18.0, 23.0, 54.0, -29.0, -10.0]], [[-23.0, 38.0, -2.0, 48.0, 37.0], [63.0, 32.0, 24.0, 1.0, -29.0], [10.0, -24.0, 23.0, -46.0, -44.0], [49.0, -17.0, 84.0, -51.0, 42.0]], [[-71.0, 6.0, -30.0, 11.0, -47.0], [69.0, 74.0, -76.0, -4.0, 52.0], [47.0, -34.0, -52.0, 29.0, 56.0], [-30.0, 60.0, 15.0, -1.0, -76.0]], [[-55.0, -43.0, -81.0, -84.0, -23.0], [7.0, 1.0, -17.0, -18.0, 24.0], [77.0, -25.0, -6.0, 59.0, 16.0], [1.0, 17.0, 48.0, 3.0, 47.0]], [[53.0, 33.0, 71.0, 60.0, 25.0], [7.0, -27.0, 23.0, 14.0, -20.0], [-59.0, 7.0, -6.0, -49.0, 4.0], [21.0, -19.0, -24.0, -17.0, -25.0]]], [[[98.0, 28.0, 37.0, 15.0, 31.0], [15.0, -44.0, 37.0, -45.0, -52.0], [-56.0, -19.0, 45.0, -5.0, 17.0], [65.0, -21.0, -11.0, 22.0, 11.0]], [[6.0, -31.0, 26.0, -36.0, 1.0], [-79.0, -90.0, -7.0, 35.0, 38.0], [3.0, 43.0, -29.0, -25.0, 37.0], [-80.0, 39.0, -84.0, 39.0, -59.0]], [[37.0, 58.0, 98.0, 123.0, 1.0], [63.0, 45.0, 2.0, 29.0, -12.0], [-82.0, -11.0, -49.0, -46.0, 7.0], [47.0, -41.0, -3.0, -67.0, -42.0]], [[66.0, -5.0, 7.0, -28.0, 8.0], [-18.0, -46.0, 20.0, -35.0, -25.0], [-26.0, -6.0, 29.0, 23.0, 22.0], [28.0, -9.0, -26.0, 35.0, 12.0]], [[11.0, 29.0, -28.0, -11.0, 60.0], [32.0, -42.0, 23.0, -43.0, -36.0], [32.0, -23.0, 77.0, -46.0, -7.0], [18.0, 45.0, 45.0, 36.0, 7.0]]], [[[-33.0, -3.0, -12.0, -15.0, 0.0], [28.0, -22.0, -21.0, 5.0, 28.0], [40.0, -19.0, -27.0, -10.0, 41.0], [-14.0, 41.0, 17.0, -4.0, -29.0]], [[-71.0, -23.0, -3.0, -1.0, -13.0], [-21.0, 7.0, -10.0, 60.0, 41.0], [32.0, 23.0, -54.0, -19.0, -19.0], [-43.0, -4.0, 11.0, -46.0, 6.0]], [[100.0, 7.0, 62.0, 31.0, -23.0], [-3.0, 7.0, 24.0, -16.0, -33.0], [-95.0, -6.0, -16.0, 46.0, 15.0], [73.0, -79.0, -39.0, -18.0, 24.0]], [[30.0, -9.0, -31.0, -22.0, -46.0], [0.0, 62.0, -30.0, -53.0, -9.0], [-18.0, -12.0, 23.0, 85.0, 16.0], [10.0, -3.0, -32.0, 51.0, -10.0]], [[58.0, 26.0, 30.0, 39.0, -6.0], [12.0, 32.0, 1.0, -28.0, -29.0], [-57.0, -4.0, 22.0, 14.0, 1.0], [28.0, -19.0, -29.0, 19.0, -19.0]]], [[[-19.0, -14.0, -69.0, -65.0, 6.0], [-34.0, -15.0, -30.0, -46.0, 7.0], [55.0, 10.0, 68.0, 13.0, 11.0], [-72.0, 87.0, -33.0, 106.0, -48.0]], [[11.0, 22.0, -46.0, -32.0, 83.0], [29.0, -64.0, 57.0, -46.0, -57.0], [46.0, -28.0, 100.0, -51.0, -37.0], [50.0, 19.0, 88.0, 16.0, 73.0]], [[-25.0, 16.0, 51.0, 58.0, 6.0], [32.0, 3.0, 9.0, 56.0, 16.0], [-14.0, -2.0, -64.0, -47.0, -4.0], [18.0, -30.0, 30.0, -83.0, 3.0]], [[-13.0, -13.0, -24.0, -20.0, -26.0], [0.0, 26.0, -23.0, -12.0, 13.0], [13.0, -5.0, -6.0, 37.0, 13.0], [-11.0, 10.0, -8.0, 16.0, -9.0]], [[64.0, -22.0, 12.0, -33.0, 6.0], [-58.0, -47.0, 41.0, -11.0, -29.0], [-38.0, 19.0, 25.0, 25.0, -16.0], [25.0, -48.0, -31.0, 15.0, 53.0]]]])+(1.-msk_ref)*numpy.array([[[[-16.0, -38.0, -33.0, -32.0, -87.0], [62.0, 7.0, -51.0, 55.0, -25.0], [-13.0, 3.0, -35.0, -12.0, 10.0], [18.0, -9.0, 70.0, 34.0, -13.0]], [[-16.0, -70.0, -78.0, -60.0, -34.0], [34.0, -26.0, -25.0, 62.0, 8.0], [56.0, -14.0, -7.0, -10.0, 36.0], [9.0, 5.0, 48.0, 62.0, -39.0]], [[-16.0, 74.0, 42.0, 66.0, -101.0], [38.0, 93.0, -45.0, 28.0, -101.0], [-148.0, 118.0, -10.0, -23.0, 19.0], [-39.0, 25.0, 16.0, 55.0, 93.0]], [[-89.0, 19.0, -33.0, -32.0, -50.0], [-4.0, 27.0, -28.0, 41.0, 16.0], [16.0, 28.0, 45.0, 98.0, -23.0], [-69.0, -9.0, -41.0, -84.0, 0.0]], [[47.0, -5.0, 15.0, 28.0, 54.0], [-4.0, -17.0, 24.0, -35.0, -16.0], [-4.0, 0.0, -11.0, -66.0, 25.0], [27.0, 27.0, 7.0, 64.0, 8.0]]], [[[41.0, 41.0, 7.0, 21.0, 93.0], [-52.0, -17.0, 39.0, -32.0, 23.0], [15.0, 21.0, 6.0, -68.0, 48.0], [-7.0, 18.0, -29.0, 82.0, 36.0]], [[-15.0, 87.0, 78.0, 47.0, 57.0], [6.0, -1.0, -10.0, -97.0, 3.0], [-30.0, 10.0, -20.0, -31.0, -43.0], [-3.0, 7.0, -25.0, -77.0, 39.0]], [[79.0, -61.0, 3.0, 51.0, -3.0], [10.0, 15.0, 37.0, 9.0, -83.0], [-53.0, 20.0, 4.0, -56.0, 45.0], [39.0, 50.0, 24.0, 124.0, 1.0]], [[14.0, 48.0, 19.0, 18.0, 71.0], [-50.0, -7.0, 31.0, -35.0, 29.0], [11.0, 11.0, 15.0, -18.0, 12.0], [-18.0, 3.0, -44.0, 8.0, 27.0]], [[-31.0, 81.0, -18.0, -31.0, 36.0], [16.0, -24.0, -51.0, -9.0, 46.0], [28.0, 40.0, -48.0, -100.0, 48.0], [-20.0, -4.0, 33.0, 72.0, 54.0]]], [[[-51.0, 69.0, 14.0, 25.0, -20.0], [24.0, 36.0, -39.0, -9.0, -34.0], [-48.0, 76.0, 4.0, -16.0, 8.0], [-48.0, 28.0, -11.0, 4.0, 58.0]], [[-43.0, -65.0, -22.0, -27.0, -56.0], [44.0, -4.0, -22.0, 18.0, -14.0], [22.0, -37.0, 13.0, 74.0, -47.0], [7.0, -1.0, 12.0, -88.0, -58.0]], [[78.0, -56.0, 6.0, 39.0, 61.0], [-78.0, 0.0, 98.0, -7.0, -13.0], [5.0, -25.0, 60.0, 34.0, 13.0], [13.0, 20.0, -62.0, 30.0, -27.0]], [[56.0, 26.0, 37.0, 36.0, -11.0], [-50.0, 35.0, 37.0, 13.0, -9.0], [-59.0, 13.0, 3.0, 6.0, 8.0], [4.0, -25.0, -18.0, 28.0, 31.0]], [[78.0, 0.0, 25.0, 33.0, 28.0], [-32.0, 3.0, 40.0, -7.0, -12.0], [-32.0, -1.0, -14.0, -52.0, 30.0], [32.0, -2.0, 7.0, 80.0, 18.0]]], [[[-10.0, 140.0, 63.0, 11.0, 8.0], [10.0, 9.0, -54.0, -42.0, 38.0], [-48.0, 36.0, -78.0, -88.0, -1.0], [-4.0, -44.0, 32.0, 2.0, 85.0]], [[-68.0, 32.0, -77.0, -91.0, 47.0], [10.0, -60.0, -51.0, 17.0, 95.0], [116.0, -1.0, -24.0, -51.0, 43.0], [-31.0, -13.0, 23.0, 35.0, 1.0]], [[-16.0, -76.0, -36.0, -4.0, -28.0], [34.0, 0.0, 0.0, 21.0, -46.0], [12.0, -3.0, 30.0, 32.0, -1.0], [2.0, 40.0, 5.0, 2.0, -41.0]], [[1.0, 17.0, 16.0, 15.0, -25.0], [-8.0, 25.0, 1.0, 9.0, -13.0], [-34.0, 16.0, 5.0, 19.0, -7.0], [-10.0, -8.0, -9.0, -11.0, 16.0]], [[17.0, -35.0, -10.0, -22.0, 74.0], [-56.0, -43.0, 54.0, -24.0, 62.0], [79.0, -69.0, 32.0, 47.0, -22.0], [10.0, -17.0, -47.0, -59.0, -52.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 1.0, 0.0], [7.0, 7.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 3.0, -1.0], [0.0, -5.0, -2.0]])
      arg1=numpy.array([-1.0, 5.0, 0.0])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([9.0, 28.0])+(1.-msk_ref)*numpy.array([19.0, -25.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -5.0, 5.0], [7.0, -2.0, 0.0]])+(1.-msk_arg0)*numpy.array([[1.0, -2.0, -1.0], [6.0, 2.0, 0.0]])
      arg1=numpy.array([[1.0, 6.0], [-5.0, 3.0], [0.0, 6.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[19.0, -21.0], [17.0, 36.0]])+(1.-msk_ref)*numpy.array([[11.0, -6.0], [-4.0, 42.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 6.0, -2.0], [-1.0, -6.0, -1.0]])+(1.-msk_arg0)*numpy.array([[1.0, -7.0, -1.0], [7.0, 3.0, -7.0]])
      arg1=Symbol(shape=(3,))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([-3.0, 0.0, 5.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-28.0, -2.0])+(1.-msk_ref)*numpy.array([-8.0, -56.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -2.0, -6.0], [-2.0, 0.0, 1.0]])+(1.-msk_arg0)*numpy.array([[-6.0, 7.0, 1.0], [-7.0, -2.0, -1.0]])
      arg1=Symbol(shape=(3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[-1.0, 7.0], [-1.0, 0.0], [-5.0, 1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[32.0, -6.0], [-3.0, -13.0]])+(1.-msk_ref)*numpy.array([[-6.0, -41.0], [14.0, -50.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, -5.0, 5.0], [-4.0, -7.0, 6.0]])+(1.-msk_arg0)*numpy.array([[5.0, -2.0, -3.0], [5.0, -4.0, -6.0]])
      arg1=Data(numpy.array([2.0, 1.0, 6.0]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([29.0, 21.0])+(1.-msk_ref)*numpy.array([-10.0, -30.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 1.0, -7.0], [-1.0, -4.0, 3.0]])+(1.-msk_arg0)*numpy.array([[0.0, -2.0, -4.0], [5.0, -5.0, -3.0]])
      arg1=Data(numpy.array([[5.0, -6.0], [-7.0, 5.0], [0.0, 0.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-27.0, 29.0], [23.0, -14.0]])+(1.-msk_ref)*numpy.array([[14.0, -10.0], [60.0, -55.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 0.0, 2.0], [2.0, 2.0, 5.0]])+(1.-msk_arg0)*numpy.array([[5.0, 2.0, 3.0], [5.0, 5.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, 0.0, 6.0])+(1.-msk_arg1)*numpy.array([-2.0, -2.0, 1.0])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([16.0, 32.0])+(1.-msk_ref)*numpy.array([-11.0, -13.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -4.0, -6.0], [2.0, 0.0, -5.0]])+(1.-msk_arg0)*numpy.array([[-6.0, -6.0, -3.0], [-7.0, -2.0, -2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 2.0], [0.0, 4.0], [3.0, -2.0]])+(1.-msk_arg1)*numpy.array([[2.0, 1.0], [-7.0, 1.0], [-7.0, 5.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, 4.0], [-11.0, 14.0]])+(1.-msk_ref)*numpy.array([[51.0, -27.0], [14.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, -7.0, 3.0], [7.0, -2.0, -6.0]], [[0.0, -1.0, -7.0], [4.0, 1.0, 7.0]], [[0.0, -7.0, 5.0], [-5.0, 0.0, 3.0]], [[-6.0, 4.0, -6.0], [4.0, -6.0, 0.0]], [[-2.0, 6.0, 0.0], [1.0, 0.0, -7.0]]], [[[1.0, -1.0, 1.0], [3.0, -2.0, 6.0]], [[-5.0, 1.0, -3.0], [0.0, -6.0, 5.0]], [[2.0, 4.0, -4.0], [-7.0, -5.0, 0.0]], [[6.0, 1.0, -5.0], [6.0, -4.0, 3.0]], [[-1.0, -7.0, 7.0], [-3.0, 1.0, 4.0]]], [[[-7.0, -6.0, 3.0], [-2.0, -4.0, 0.0]], [[4.0, -5.0, -6.0], [3.0, -2.0, 0.0]], [[2.0, 3.0, -7.0], [5.0, 0.0, 7.0]], [[-2.0, -6.0, 6.0], [-4.0, 6.0, -2.0]], [[7.0, 0.0, -5.0], [-4.0, 3.0, 1.0]]], [[[1.0, -3.0, -4.0], [2.0, 2.0, -1.0]], [[5.0, 5.0, 1.0], [7.0, 0.0, 7.0]], [[-2.0, 7.0, -7.0], [-7.0, 3.0, 3.0]], [[0.0, -3.0, 1.0], [3.0, 2.0, -2.0]], [[-1.0, 5.0, -3.0], [7.0, -4.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, -7.0, -2.0], [0.0, -4.0, 4.0]], [[2.0, 6.0, -5.0], [0.0, 0.0, 2.0]], [[4.0, 0.0, 0.0], [0.0, 7.0, -1.0]], [[-1.0, 5.0, 5.0], [-6.0, -1.0, 4.0]], [[0.0, -7.0, -4.0], [2.0, -7.0, -6.0]]], [[[0.0, 5.0, 0.0], [-1.0, 6.0, 3.0]], [[4.0, -6.0, -4.0], [3.0, 2.0, 3.0]], [[2.0, 0.0, 2.0], [1.0, 2.0, 4.0]], [[4.0, -3.0, -3.0], [6.0, -6.0, -7.0]], [[2.0, -3.0, 1.0], [2.0, -6.0, 7.0]]], [[[6.0, 0.0, 6.0], [-2.0, 0.0, -3.0]], [[3.0, 5.0, -5.0], [6.0, 0.0, -4.0]], [[5.0, 1.0, 1.0], [-2.0, 5.0, -4.0]], [[-2.0, -5.0, 0.0], [5.0, -6.0, 7.0]], [[-1.0, 2.0, -5.0], [6.0, 4.0, 0.0]]], [[[4.0, 7.0, 6.0], [-3.0, 1.0, 7.0]], [[5.0, 0.0, 7.0], [-1.0, 5.0, 0.0]], [[-6.0, -1.0, 7.0], [-7.0, -4.0, 0.0]], [[2.0, 3.0, 2.0], [3.0, 3.0, -1.0]], [[7.0, 3.0, 7.0], [-7.0, 1.0, 0.0]]]])
      arg1=numpy.array([[1.0, -7.0, -1.0], [3.0, 1.0, -6.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[94.0, -15.0, 11.0, -22.0, 1.0], [-22.0, -45.0, -48.0, 0.0, 9.0], [22.0, 52.0, -39.0, 40.0, -3.0], [40.0, -52.0, -80.0, 43.0, -28.0]])+(1.-msk_ref)*numpy.array([[18.0, -47.0, 17.0, -84.0, 88.0], [-50.0, 43.0, -19.0, 82.0, -20.0], [12.0, 15.0, 20.0, 0.0, 12.0], [-101.0, 0.0, -31.0, -3.0, -41.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_array_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 5.0, 6.0], [5.0, -2.0, 0.0]], [[-6.0, -5.0, 1.0], [4.0, -1.0, 3.0]], [[6.0, -3.0, -7.0], [7.0, 3.0, 7.0]], [[0.0, 0.0, 2.0], [-5.0, -4.0, 7.0]], [[-5.0, 3.0, -1.0], [1.0, 0.0, -5.0]]], [[[0.0, -6.0, 0.0], [-2.0, 4.0, -1.0]], [[4.0, -4.0, 3.0], [-1.0, 3.0, -6.0]], [[-1.0, 5.0, 3.0], [5.0, -4.0, -1.0]], [[7.0, -6.0, 2.0], [6.0, -1.0, 4.0]], [[0.0, -1.0, -2.0], [-4.0, -4.0, 0.0]]], [[[-2.0, 5.0, -2.0], [-1.0, 6.0, -7.0]], [[-7.0, 0.0, -1.0], [4.0, 3.0, -3.0]], [[-1.0, 2.0, 7.0], [6.0, -7.0, 3.0]], [[1.0, 7.0, -5.0], [5.0, 7.0, -1.0]], [[1.0, 3.0, -3.0], [-7.0, -2.0, 1.0]]], [[[6.0, 4.0, 0.0], [1.0, 5.0, 5.0]], [[-2.0, -1.0, -5.0], [-4.0, 5.0, 3.0]], [[7.0, 0.0, 0.0], [3.0, -6.0, -6.0]], [[4.0, 0.0, 4.0], [-4.0, -6.0, 0.0]], [[1.0, 0.0, -4.0], [-3.0, 5.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -7.0, 5.0], [-4.0, 6.0, -3.0]], [[5.0, -5.0, -4.0], [6.0, -7.0, -3.0]], [[7.0, 2.0, 0.0], [0.0, -3.0, 2.0]], [[0.0, -1.0, 7.0], [4.0, 0.0, 3.0]], [[-4.0, -2.0, -6.0], [7.0, -1.0, -7.0]]], [[[-4.0, -4.0, 1.0], [-5.0, -3.0, -1.0]], [[-1.0, 6.0, -6.0], [5.0, 5.0, 1.0]], [[6.0, 4.0, -1.0], [-1.0, 3.0, 5.0]], [[-6.0, -6.0, -5.0], [0.0, 0.0, -3.0]], [[4.0, -4.0, -1.0], [0.0, -5.0, 0.0]]], [[[-5.0, 2.0, 6.0], [5.0, -4.0, 2.0]], [[6.0, -3.0, 2.0], [4.0, 1.0, 6.0]], [[-6.0, -4.0, 5.0], [-5.0, 7.0, 2.0]], [[2.0, 4.0, -7.0], [6.0, 1.0, 5.0]], [[-6.0, 5.0, -6.0], [3.0, 4.0, -6.0]]], [[[-3.0, -6.0, -6.0], [-3.0, -6.0, 0.0]], [[-2.0, -5.0, -6.0], [-4.0, -1.0, 5.0]], [[3.0, 5.0, 3.0], [-3.0, 3.0, 7.0]], [[6.0, 3.0, -6.0], [-4.0, 7.0, 3.0]], [[-6.0, 5.0, 5.0], [0.0, -5.0, 5.0]]]])
      arg1=numpy.array([[[7.0, -4.0], [5.0, 2.0], [-6.0, 7.0]], [[5.0, -2.0], [-3.0, 7.0], [-5.0, 0.0]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[41.0, 16.0], [-65.0, 6.0], [60.0, -72.0], [-60.0, -4.0], [16.0, 17.0]], [[-47.0, 20.0], [6.0, 20.0], [42.0, -3.0], [20.0, -45.0], [-1.0, -36.0]], [[35.0, 48.0], [-17.0, 34.0], [-3.0, -4.0], [81.0, 14.0], [6.0, -19.0]], [[27.0, 17.0], [-39.0, 14.0], [112.0, -76.0], [2.0, -22.0], [21.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[-88.0, 71.0], [100.0, -119.0], [58.0, -45.0], [-42.0, 39.0], [71.0, -51.0]], [[-65.0, 4.0], [64.0, -1.0], [29.0, 0.0], [-27.0, -23.0], [29.0, -66.0]], [[-34.0, 28.0], [2.0, -17.0], [-148.0, 110.0], [78.0, -54.0], [52.0, 14.0]], [[-12.0, -78.0], [-45.0, -43.0], [-31.0, 46.0], [37.0, -3.0], [-57.0, 34.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_array_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, -1.0, -5.0], [6.0, 0.0, -3.0]], [[4.0, -2.0, -2.0], [3.0, -1.0, -4.0]], [[5.0, 3.0, 7.0], [-4.0, 2.0, 1.0]], [[3.0, 4.0, 0.0], [7.0, -6.0, -6.0]], [[-4.0, -5.0, 7.0], [-5.0, -3.0, -3.0]]], [[[7.0, -4.0, 0.0], [-2.0, 2.0, 0.0]], [[1.0, -5.0, 0.0], [0.0, -4.0, -6.0]], [[-6.0, 3.0, 7.0], [3.0, -4.0, 5.0]], [[-1.0, 2.0, -1.0], [1.0, 0.0, -2.0]], [[1.0, -2.0, 6.0], [-2.0, 6.0, 7.0]]], [[[2.0, 6.0, 7.0], [-4.0, 3.0, -6.0]], [[-7.0, -3.0, 0.0], [-4.0, -5.0, -2.0]], [[1.0, 7.0, -7.0], [-2.0, -1.0, 7.0]], [[4.0, 2.0, 6.0], [-3.0, 1.0, 7.0]], [[7.0, 0.0, -3.0], [0.0, -5.0, -3.0]]], [[[3.0, -6.0, -3.0], [-6.0, 6.0, 0.0]], [[0.0, -2.0, -7.0], [-1.0, 0.0, 5.0]], [[-3.0, -2.0, -5.0], [7.0, 3.0, -7.0]], [[-2.0, -3.0, -2.0], [-3.0, 0.0, -2.0]], [[-1.0, -5.0, 2.0], [3.0, -3.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 1.0, 0.0], [-7.0, -1.0, -1.0]], [[-2.0, 3.0, 0.0], [-4.0, 6.0, -4.0]], [[-5.0, 6.0, -5.0], [-4.0, 0.0, 0.0]], [[5.0, 1.0, -7.0], [7.0, -5.0, 3.0]], [[5.0, 2.0, 5.0], [-1.0, 4.0, 0.0]]], [[[-1.0, 4.0, -4.0], [-5.0, -1.0, 2.0]], [[0.0, 6.0, 4.0], [0.0, -6.0, -2.0]], [[-5.0, 5.0, -7.0], [-3.0, -4.0, -7.0]], [[-6.0, -6.0, 0.0], [0.0, 5.0, 4.0]], [[-6.0, 6.0, 0.0], [5.0, -4.0, -3.0]]], [[[0.0, 3.0, -3.0], [-3.0, -1.0, 1.0]], [[-3.0, -2.0, -7.0], [7.0, 4.0, -2.0]], [[0.0, 2.0, -1.0], [-6.0, 7.0, -6.0]], [[0.0, -3.0, -4.0], [-7.0, -6.0, -4.0]], [[-2.0, -2.0, 0.0], [0.0, -5.0, -7.0]]], [[[5.0, 6.0, -4.0], [4.0, 2.0, 5.0]], [[6.0, 3.0, 5.0], [-1.0, 7.0, -4.0]], [[4.0, -1.0, -5.0], [0.0, 7.0, 0.0]], [[-3.0, -1.0, -2.0], [6.0, -3.0, -2.0]], [[0.0, -2.0, 6.0], [-1.0, 5.0, 7.0]]]])
      arg1=numpy.array([[[[-3.0, -3.0, 7.0, 4.0, -1.0], [7.0, -1.0, -4.0, -7.0, -3.0], [3.0, -3.0, -1.0, -6.0, 2.0], [-4.0, -6.0, 3.0, 3.0, 7.0]], [[7.0, -4.0, 4.0, 6.0, 2.0], [0.0, -3.0, 0.0, -2.0, -1.0], [0.0, -6.0, 7.0, -7.0, 7.0], [-1.0, 6.0, -1.0, 4.0, 0.0]], [[-1.0, -7.0, 2.0, 2.0, -2.0], [6.0, -6.0, -4.0, -6.0, 4.0], [-6.0, -5.0, -2.0, 7.0, -1.0], [6.0, 0.0, 3.0, -4.0, -6.0]]], [[[1.0, 0.0, -2.0, 6.0, -7.0], [6.0, -4.0, -6.0, 4.0, -1.0], [-7.0, 0.0, -7.0, 6.0, 1.0], [1.0, -5.0, 2.0, 1.0, 0.0]], [[2.0, -6.0, -5.0, 0.0, -4.0], [0.0, 2.0, -6.0, -6.0, -7.0], [5.0, 6.0, 4.0, 7.0, -7.0], [1.0, 7.0, 1.0, 1.0, -7.0]], [[-4.0, -6.0, -5.0, -5.0, 3.0], [-6.0, 3.0, 4.0, 0.0, 7.0], [7.0, 4.0, 0.0, 6.0, 5.0], [0.0, -2.0, 0.0, 0.0, -7.0]]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[25.0, 66.0, -32.0, 23.0, -40.0], [3.0, 3.0, -16.0, 77.0, -37.0], [-42.0, 28.0, -36.0, 8.0, -17.0], [-11.0, -12.0, -11.0, 13.0, 30.0]], [[-7.0, 40.0, 35.0, 38.0, -33.0], [58.0, -12.0, -36.0, 6.0, -42.0], [-30.0, -12.0, -39.0, -37.0, -14.0], [-24.0, -50.0, 13.0, 14.0, 75.0]], [[-5.0, -94.0, 54.0, 23.0, 10.0], [47.0, -33.0, -32.0, -111.0, 7.0], [18.0, -52.0, 38.0, -6.0, 11.0], [17.0, 20.0, 27.0, -3.0, -28.0]], [[38.0, 47.0, 83.0, 108.0, -38.0], [99.0, -73.0, -42.0, 35.0, -20.0], [-112.0, -93.0, -48.0, -82.0, 53.0], [-15.0, -59.0, 13.0, 26.0, 105.0]], [[-29.0, 19.0, 6.0, -47.0, 18.0], [2.0, -18.0, 24.0, -6.0, 50.0], [-55.0, -23.0, -22.0, 39.0, -49.0], [55.0, 4.0, 1.0, -68.0, -28.0]]], [[[-47.0, -17.0, 27.0, -8.0, -9.0], [37.0, 17.0, -28.0, -61.0, -29.0], [45.0, 15.0, -13.0, -12.0, -30.0], [-24.0, -42.0, 23.0, 5.0, 35.0]], [[-22.0, 77.0, 37.0, 4.0, -13.0], [43.0, -12.0, -4.0, 27.0, -12.0], [-59.0, -21.0, -52.0, -35.0, -35.0], [-3.0, -52.0, 4.0, -21.0, 77.0]], [[7.0, -49.0, -27.0, 1.0, 8.0], [-12.0, -50.0, 22.0, 30.0, 103.0], [-66.0, -39.0, -24.0, 84.0, 58.0], [62.0, 1.0, 2.0, -35.0, -91.0]], [[27.0, 14.0, 7.0, 22.0, -6.0], [5.0, -9.0, -6.0, 13.0, -18.0], [-18.0, -12.0, 10.0, -21.0, 4.0], [-3.0, 17.0, -6.0, 10.0, 13.0]], [[-41.0, -115.0, -50.0, -43.0, -6.0], [-11.0, 10.0, -24.0, -83.0, 32.0], [60.0, 43.0, 11.0, 122.0, -27.0], [38.0, 20.0, 25.0, -25.0, -120.0]]], [[[55.0, -61.0, 75.0, 64.0, -6.0], [68.0, -58.0, -54.0, -102.0, -43.0], [-35.0, -83.0, 66.0, -44.0, -16.0], [27.0, 77.0, 16.0, 1.0, -7.0]], [[-6.0, 75.0, -18.0, -60.0, 43.0], [-61.0, 16.0, 74.0, 69.0, 49.0], [-32.0, 1.0, -6.0, -8.0, -14.0], [22.0, 13.0, -31.0, -42.0, 0.0]], [[21.0, -18.0, -5.0, -15.0, 66.0], [-89.0, 47.0, 70.0, 19.0, 20.0], [103.0, 12.0, 72.0, -81.0, 98.0], [-56.0, 25.0, -30.0, 56.0, 7.0]], [[-33.0, -110.0, 14.0, -13.0, 26.0], [4.0, -11.0, 0.0, -86.0, 55.0], [51.0, -20.0, 23.0, 35.0, 41.0], [16.0, -4.0, 23.0, -6.0, -64.0]], [[-16.0, 48.0, 83.0, 37.0, 10.0], [49.0, -8.0, 2.0, -1.0, -19.0], [-7.0, -48.0, -21.0, -116.0, 37.0], [-51.0, -71.0, 7.0, 28.0, 123.0]]], [[[-42.0, 0.0, -27.0, -66.0, 9.0], [-33.0, 69.0, 0.0, -51.0, -51.0], [99.0, 78.0, 27.0, 9.0, -81.0], [-24.0, 18.0, 0.0, -3.0, -3.0]], [[-28.0, 27.0, -45.0, -57.0, 32.0], [-78.0, 67.0, 54.0, 42.0, 10.0], [84.0, 67.0, 7.0, -11.0, 17.0], [-41.0, -17.0, -21.0, 19.0, 7.0]], [[41.0, 76.0, -33.0, 43.0, -73.0], [33.0, -4.0, -56.0, 65.0, -86.0], [-62.0, 36.0, -38.0, 18.0, -64.0], [-6.0, 6.0, -5.0, 13.0, 37.0]], [[-8.0, 44.0, -14.0, -38.0, 15.0], [-32.0, 29.0, 26.0, 20.0, -10.0], [13.0, 26.0, 6.0, -11.0, -36.0], [-4.0, 13.0, -15.0, -13.0, 12.0]], [[-61.0, -9.0, -44.0, -42.0, -4.0], [-13.0, 4.0, 20.0, 35.0, 76.0], [-9.0, 29.0, -71.0, 88.0, 15.0], [21.0, -72.0, 11.0, -31.0, -40.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, 8.0, 28.0, -31.0, 52.0], [-36.0, 20.0, 44.0, -24.0, 6.0], [37.0, -16.0, 52.0, -62.0, 2.0], [-9.0, 36.0, -16.0, -4.0, 14.0]], [[51.0, -18.0, -4.0, 6.0, 0.0], [-14.0, 9.0, -20.0, -44.0, -63.0], [24.0, 8.0, 75.0, -15.0, -49.0], [7.0, 100.0, -11.0, 8.0, -28.0]], [[58.0, 26.0, -13.0, -18.0, 55.0], [-89.0, 33.0, 64.0, 37.0, -7.0], [43.0, 4.0, 85.0, -71.0, 33.0], [-20.0, 86.0, -44.0, 25.0, -5.0]], [[-16.0, 42.0, 21.0, 39.0, -9.0], [17.0, 5.0, 8.0, 63.0, 5.0], [4.0, -4.0, -53.0, -61.0, 81.0], [-61.0, -100.0, 2.0, 49.0, 91.0]], [[1.0, -82.0, 35.0, 36.0, -20.0], [59.0, -29.0, -58.0, -97.0, -24.0], [12.0, -28.0, 22.0, 13.0, -10.0], [11.0, 15.0, 30.0, 6.0, -23.0]]], [[[20.0, 9.0, 6.0, -28.0, 62.0], [-73.0, 37.0, 64.0, 9.0, 9.0], [65.0, 1.0, 68.0, -75.0, 42.0], [-30.0, 44.0, -30.0, 23.0, 10.0]], [[34.0, -4.0, 72.0, 54.0, 22.0], [36.0, -60.0, 12.0, 0.0, 38.0], [-68.0, -100.0, 10.0, -68.0, 70.0], [12.0, -2.0, 0.0, 2.0, 32.0]], [[74.0, 110.0, 32.0, 13.0, 45.0], [-53.0, 15.0, 62.0, 79.0, -36.0], [-21.0, -32.0, 59.0, -142.0, 22.0], [-34.0, 61.0, -51.0, 26.0, 84.0]], [[-30.0, -12.0, -111.0, -80.0, -14.0], [-66.0, 46.0, 10.0, 24.0, 17.0], [35.0, 100.0, -16.0, 137.0, -69.0], [35.0, 27.0, -7.0, -37.0, -105.0]], [[69.0, 36.0, 7.0, 57.0, -10.0], [6.0, -49.0, 6.0, 74.0, 14.0], [-94.0, -54.0, -3.0, -22.0, 48.0], [19.0, 25.0, -18.0, 7.0, 7.0]]], [[[15.0, 9.0, 12.0, -11.0, 40.0], [-42.0, 22.0, 40.0, 6.0, 2.0], [41.0, -5.0, 44.0, -61.0, 33.0], [-25.0, 24.0, -19.0, 20.0, 18.0]], [[25.0, 54.0, -67.0, 14.0, -58.0], [-9.0, 25.0, -34.0, 71.0, -66.0], [-10.0, 72.0, -30.0, 41.0, -44.0], [-17.0, 3.0, -10.0, 22.0, 7.0]], [[47.0, -7.0, 13.0, 4.0, 2.0], [-6.0, 20.0, -26.0, -64.0, -91.0], [41.0, 11.0, 86.0, -44.0, -70.0], [-7.0, 103.0, -10.0, 13.0, -1.0]], [[-20.0, 100.0, 44.0, -48.0, 63.0], [-42.0, 37.0, 78.0, 38.0, 8.0], [15.0, -14.0, 12.0, -115.0, -2.0], [-34.0, -17.0, -29.0, -9.0, 94.0]], [[10.0, 86.0, 38.0, 15.0, -3.0], [28.0, -23.0, 10.0, 48.0, -6.0], [-80.0, -40.0, -32.0, -51.0, -18.0], [5.0, -21.0, -9.0, -19.0, 70.0]]], [[[19.0, -53.0, 8.0, 47.0, -6.0], [5.0, 4.0, -20.0, -19.0, -20.0], [56.0, 1.0, 25.0, -32.0, 71.0], [-44.0, -10.0, 7.0, 61.0, 10.0]], [[27.0, -83.0, 51.0, 66.0, -43.0], [90.0, -39.0, -96.0, -124.0, -77.0], [2.0, -35.0, 40.0, -3.0, -42.0], [9.0, 44.0, 35.0, 16.0, -9.0]], [[0.0, -15.0, -21.0, 0.0, -24.0], [-2.0, 43.0, -38.0, -38.0, -80.0], [77.0, 61.0, 27.0, -3.0, -43.0], [-38.0, 19.0, 5.0, 35.0, 9.0]], [[12.0, 57.0, -16.0, 24.0, -31.0], [15.0, -18.0, -6.0, 77.0, 3.0], [-68.0, -1.0, -54.0, 14.0, 6.0], [4.0, -35.0, -5.0, -2.0, 26.0]], [[-39.0, -106.0, -54.0, -41.0, -8.0], [-12.0, 5.0, -20.0, -66.0, 41.0], [45.0, 40.0, 1.0, 127.0, -21.0], [42.0, 14.0, 23.0, -28.0, -120.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, 3.0, -1.0], [-4.0, 5.0, -5.0]], [[-1.0, -2.0, 0.0], [2.0, -3.0, 4.0]], [[-6.0, 5.0, 0.0], [-4.0, -2.0, 1.0]], [[5.0, 3.0, -4.0], [0.0, 5.0, -2.0]], [[-5.0, 2.0, 6.0], [4.0, 7.0, -5.0]]], [[[0.0, -6.0, 3.0], [-4.0, 3.0, -4.0]], [[-6.0, -6.0, 2.0], [5.0, -1.0, 1.0]], [[0.0, 1.0, 7.0], [-4.0, 4.0, 2.0]], [[-3.0, 0.0, -5.0], [5.0, 6.0, 7.0]], [[5.0, -3.0, -3.0], [3.0, -1.0, -4.0]]], [[[-2.0, 1.0, 0.0], [1.0, 0.0, 4.0]], [[0.0, 2.0, 2.0], [2.0, -3.0, -3.0]], [[4.0, -4.0, -3.0], [-1.0, 7.0, -7.0]], [[5.0, 2.0, -3.0], [-4.0, -7.0, 0.0]], [[-5.0, 0.0, -3.0], [7.0, 5.0, -1.0]]], [[[0.0, -1.0, 7.0], [-6.0, 3.0, 5.0]], [[-6.0, 1.0, 6.0], [-6.0, -2.0, 0.0]], [[-2.0, 5.0, -6.0], [0.0, -1.0, -3.0]], [[7.0, 0.0, 5.0], [-4.0, 4.0, -7.0]], [[2.0, 1.0, -4.0], [2.0, 7.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -7.0, 2.0], [3.0, 0.0, 4.0]], [[7.0, -2.0, 0.0], [0.0, -3.0, -6.0]], [[-2.0, -2.0, 5.0], [5.0, -3.0, -7.0]], [[7.0, 0.0, 2.0], [-1.0, -4.0, -1.0]], [[4.0, 7.0, 0.0], [4.0, 2.0, -2.0]]], [[[-3.0, 3.0, 7.0], [-2.0, 0.0, 6.0]], [[-3.0, 2.0, -2.0], [-2.0, -1.0, -7.0]], [[4.0, 5.0, -2.0], [6.0, -2.0, -6.0]], [[-5.0, 6.0, 6.0], [7.0, 0.0, 5.0]], [[-1.0, -1.0, 4.0], [-5.0, 2.0, 0.0]]], [[[-4.0, 4.0, -7.0], [7.0, -1.0, 5.0]], [[4.0, -2.0, -4.0], [0.0, 3.0, 7.0]], [[6.0, 6.0, -3.0], [-1.0, 3.0, -3.0]], [[-7.0, 0.0, 2.0], [6.0, -2.0, -1.0]], [[2.0, -4.0, 4.0], [3.0, -7.0, 2.0]]], [[[7.0, -3.0, 6.0], [-4.0, 3.0, 5.0]], [[-7.0, 4.0, 0.0], [2.0, -7.0, -3.0]], [[5.0, -3.0, -6.0], [7.0, 0.0, 0.0]], [[1.0, 2.0, 3.0], [-7.0, 6.0, -4.0]], [[7.0, 6.0, 5.0], [-7.0, -7.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[2.0, -1.0, 7.0], [-3.0, 1.0, 4.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-25.0, 7.0, -3.0, -24.0, 5.0], [26.0, -4.0, 72.0, -22.0, -34.0], [8.0, -9.0, -27.0, -8.0, -51.0], [91.0, 45.0, -64.0, 37.0, -32.0]])+(1.-msk_ref)*numpy.array([[20.0, -11.0, -13.0, 23.0, -17.0], [70.0, -45.0, -55.0, 25.0, 44.0], [-63.0, 13.0, -21.0, -24.0, 28.0], [94.0, -43.0, -50.0, 32.0, 57.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, -7.0, 0.0], [3.0, 7.0, 5.0]], [[-2.0, 0.0, 2.0], [5.0, 6.0, 0.0]], [[-4.0, 4.0, -2.0], [-3.0, 0.0, -6.0]], [[-6.0, -4.0, -3.0], [0.0, -5.0, 3.0]], [[5.0, 7.0, 4.0], [0.0, 2.0, -6.0]]], [[[-7.0, -3.0, 3.0], [5.0, 1.0, -5.0]], [[4.0, -4.0, -4.0], [-3.0, 4.0, -3.0]], [[-3.0, -6.0, 6.0], [-2.0, -5.0, 7.0]], [[-4.0, 4.0, 4.0], [3.0, -2.0, -1.0]], [[5.0, 0.0, 6.0], [2.0, 2.0, 5.0]]], [[[5.0, -1.0, -3.0], [5.0, -7.0, 5.0]], [[4.0, -7.0, -7.0], [-5.0, -5.0, 2.0]], [[-1.0, -3.0, 0.0], [-6.0, -1.0, -7.0]], [[0.0, -4.0, -6.0], [-5.0, -6.0, -6.0]], [[5.0, 0.0, -7.0], [6.0, 0.0, 3.0]]], [[[-3.0, 7.0, 5.0], [-2.0, -3.0, 0.0]], [[2.0, -2.0, -5.0], [6.0, 1.0, -4.0]], [[3.0, 1.0, 0.0], [7.0, -5.0, 0.0]], [[-7.0, -2.0, 6.0], [-2.0, 6.0, -4.0]], [[2.0, -7.0, 2.0], [-4.0, -1.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 7.0, 0.0], [-2.0, -1.0, -1.0]], [[-4.0, -1.0, 5.0], [-6.0, -7.0, 0.0]], [[6.0, 1.0, -5.0], [-2.0, -7.0, -2.0]], [[3.0, 0.0, 1.0], [0.0, 2.0, 6.0]], [[4.0, -2.0, -3.0], [5.0, 4.0, -4.0]]], [[[1.0, 6.0, 0.0], [-1.0, -6.0, 0.0]], [[-7.0, 7.0, 4.0], [3.0, -3.0, 2.0]], [[1.0, -1.0, 0.0], [-1.0, 3.0, -1.0]], [[4.0, 3.0, -4.0], [1.0, -2.0, 7.0]], [[-4.0, 4.0, -6.0], [4.0, 1.0, 3.0]]], [[[2.0, 3.0, -1.0], [-1.0, -5.0, 0.0]], [[-5.0, -4.0, 6.0], [-1.0, 5.0, 0.0]], [[4.0, -1.0, -1.0], [6.0, 6.0, 1.0]], [[5.0, 1.0, 5.0], [3.0, 1.0, 6.0]], [[7.0, 7.0, 3.0], [2.0, -6.0, -1.0]]], [[[2.0, 0.0, 0.0], [0.0, 0.0, 1.0]], [[-5.0, 0.0, 2.0], [-4.0, -6.0, 0.0]], [[0.0, -4.0, 0.0], [-4.0, -1.0, -3.0]], [[-5.0, -3.0, -3.0], [-5.0, -4.0, 5.0]], [[2.0, 0.0, 7.0], [6.0, -6.0, -3.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[7.0, -4.0], [-6.0, 5.0], [1.0, 1.0]], [[1.0, 7.0], [2.0, 4.0], [-7.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[59.0, -26.0], [5.0, 69.0], [-15.0, 37.0], [-52.0, -31.0], [43.0, 51.0]], [[14.0, 75.0], [74.0, -33.0], [-40.0, -74.0], [-42.0, 57.0], [12.0, -12.0]], [[-6.0, -41.0], [34.0, -121.0], [52.0, -29.0], [43.0, -61.0], [13.0, 3.0]], [[-66.0, 26.0], [57.0, 39.0], [12.0, 22.0], [7.0, 50.0], [66.0, -65.0]]])+(1.-msk_ref)*numpy.array([[[-88.0, 49.0], [-37.0, -54.0], [29.0, -58.0], [-16.0, -27.0], [78.0, 38.0]], [[-42.0, -5.0], [-104.0, 68.0], [25.0, 0.0], [-46.0, -34.0], [-73.0, 50.0]], [[-16.0, -21.0], [4.0, 19.0], [44.0, 40.0], [-3.0, -9.0], [7.0, 4.0]], [[7.0, -12.0], [-49.0, -30.0], [39.0, -40.0], [-68.0, -69.0], [36.0, 29.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, 2.0, 6.0], [-7.0, 7.0, 2.0]], [[-3.0, 0.0, -6.0], [3.0, -6.0, -6.0]], [[0.0, 6.0, -1.0], [-6.0, 0.0, 0.0]], [[-5.0, 4.0, -6.0], [5.0, -7.0, 0.0]], [[2.0, -6.0, 3.0], [0.0, 7.0, 5.0]]], [[[-3.0, 0.0, -3.0], [-4.0, 4.0, 4.0]], [[3.0, 2.0, 5.0], [0.0, 6.0, 1.0]], [[-1.0, -3.0, 0.0], [-3.0, -6.0, -5.0]], [[2.0, -4.0, -1.0], [-1.0, 7.0, -5.0]], [[-4.0, 2.0, 0.0], [0.0, 3.0, -5.0]]], [[[-5.0, -6.0, -1.0], [-3.0, -2.0, 3.0]], [[1.0, 5.0, -5.0], [6.0, 3.0, 6.0]], [[0.0, 1.0, -2.0], [0.0, -5.0, 4.0]], [[-2.0, -4.0, -1.0], [4.0, 3.0, 1.0]], [[-6.0, -4.0, 1.0], [0.0, -7.0, -5.0]]], [[[-2.0, 4.0, -6.0], [0.0, 5.0, 0.0]], [[1.0, -7.0, 2.0], [3.0, -5.0, -7.0]], [[1.0, 5.0, -6.0], [0.0, 5.0, 1.0]], [[2.0, 1.0, -5.0], [3.0, 3.0, 6.0]], [[-4.0, 4.0, 7.0], [4.0, 6.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -4.0, 1.0], [-7.0, -4.0, 6.0]], [[4.0, -6.0, 6.0], [7.0, 7.0, 3.0]], [[6.0, 6.0, 0.0], [-7.0, 3.0, -3.0]], [[5.0, 3.0, -5.0], [2.0, -5.0, 7.0]], [[-5.0, 0.0, -3.0], [-5.0, -2.0, -5.0]]], [[[5.0, 3.0, 1.0], [-5.0, 1.0, -5.0]], [[-4.0, 0.0, -3.0], [2.0, 7.0, 7.0]], [[5.0, -5.0, -2.0], [-4.0, 2.0, -6.0]], [[-1.0, 1.0, 2.0], [0.0, 5.0, -2.0]], [[-5.0, 6.0, -5.0], [-6.0, 6.0, -6.0]]], [[[-7.0, -1.0, 1.0], [-1.0, -4.0, 1.0]], [[5.0, -2.0, 3.0], [0.0, -3.0, -1.0]], [[1.0, 0.0, -2.0], [7.0, 0.0, 5.0]], [[0.0, -1.0, -4.0], [-6.0, -2.0, -2.0]], [[0.0, 4.0, -4.0], [1.0, 5.0, -2.0]]], [[[1.0, 4.0, 0.0], [0.0, 3.0, 5.0]], [[-4.0, -2.0, 3.0], [-7.0, 2.0, -1.0]], [[7.0, -3.0, -5.0], [0.0, -2.0, 2.0]], [[6.0, 2.0, 0.0], [-4.0, 3.0, -2.0]], [[6.0, -6.0, 0.0], [-7.0, 4.0, 3.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=tensor_mult(arg0,arg1)
      s1=numpy.array([[[[3.0, -5.0, 0.0, 5.0, 3.0], [-2.0, -2.0, 0.0, 5.0, 0.0], [0.0, 0.0, -6.0, -7.0, -4.0], [4.0, -4.0, -4.0, 4.0, 5.0]], [[7.0, -6.0, 6.0, 2.0, 5.0], [7.0, 1.0, 0.0, 7.0, 7.0], [-3.0, 0.0, -2.0, -6.0, -7.0], [4.0, 0.0, -2.0, 7.0, -7.0]], [[1.0, -1.0, 0.0, 4.0, 2.0], [2.0, -7.0, -4.0, -4.0, 3.0], [3.0, -2.0, -2.0, 2.0, 5.0], [1.0, 7.0, -6.0, 0.0, -1.0]]], [[[-7.0, 2.0, -4.0, 0.0, 0.0], [-5.0, -5.0, -7.0, 7.0, 7.0], [5.0, 7.0, 0.0, -5.0, 5.0], [6.0, 0.0, -6.0, 3.0, 6.0]], [[5.0, 5.0, 0.0, 0.0, 1.0], [1.0, 3.0, 2.0, 0.0, -3.0], [-5.0, -1.0, -1.0, 0.0, 1.0], [5.0, -6.0, 0.0, 7.0, -4.0]], [[2.0, 2.0, 4.0, 2.0, -2.0], [3.0, 0.0, 4.0, 5.0, 0.0], [-4.0, -3.0, -6.0, 7.0, -4.0], [2.0, -3.0, -4.0, 1.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[102.0, 17.0, 48.0, 22.0, 19.0], [78.0, 20.0, 47.0, -59.0, -38.0], [-66.0, -74.0, -23.0, 63.0, -12.0], [3.0, 2.0, 2.0, 36.0, -88.0]], [[-78.0, -15.0, -36.0, -51.0, -15.0], [-45.0, 15.0, -33.0, 0.0, 21.0], [51.0, 57.0, 72.0, -48.0, 15.0], [-42.0, 24.0, 54.0, -51.0, -3.0]], [[83.0, -47.0, 60.0, 8.0, 28.0], [70.0, 43.0, 46.0, 4.0, -3.0], [-51.0, -40.0, -10.0, -8.0, -77.0], [-13.0, -7.0, 30.0, 24.0, -77.0]], [[-63.0, -18.0, 4.0, -41.0, -14.0], [-6.0, 10.0, -25.0, 62.0, 66.0], [30.0, 54.0, 41.0, -26.0, -20.0], [-15.0, 20.0, 18.0, -26.0, 11.0]], [[12.0, 68.0, -16.0, 20.0, -21.0], [-18.0, -10.0, 22.0, -19.0, -54.0], [-28.0, -28.0, -43.0, 63.0, 36.0], [32.0, -44.0, -34.0, 20.0, 51.0]]], [[[44.0, 38.0, 32.0, -19.0, -19.0], [36.0, 59.0, 64.0, -11.0, -49.0], [-65.0, -38.0, -4.0, 63.0, -35.0], [-11.0, -45.0, 38.0, 8.0, -28.0]], [[60.0, 0.0, 16.0, 41.0, 33.0], [27.0, -21.0, -4.0, 14.0, 11.0], [-25.0, -19.0, -44.0, -16.0, 1.0], [57.0, -16.0, -50.0, 69.0, -22.0]], [[-43.0, -23.0, -26.0, -21.0, -14.0], [-25.0, -4.0, -11.0, -72.0, -24.0], [44.0, 0.0, 48.0, 5.0, 24.0], [-74.0, 55.0, 48.0, -81.0, -8.0]], [[9.0, 38.0, -40.0, -12.0, 1.0], [-37.0, 25.0, 5.0, -46.0, -59.0], [-11.0, 3.0, 21.0, -22.0, 37.0], [10.0, -42.0, 32.0, 21.0, -25.0]], [[7.0, 13.0, -8.0, -26.0, 11.0], [10.0, 19.0, -14.0, -31.0, 5.0], [-1.0, 12.0, 47.0, -19.0, 25.0], [-3.0, 13.0, 32.0, 14.0, -76.0]]], [[[-41.0, 52.0, -12.0, -35.0, -55.0], [-12.0, 20.0, 33.0, -69.0, -60.0], [-2.0, -26.0, 28.0, 105.0, 28.0], [-67.0, 16.0, 44.0, -82.0, 26.0]], [[18.0, 9.0, 30.0, 7.0, 9.0], [14.0, 17.0, 8.0, 132.0, 53.0], [-39.0, 31.0, -45.0, -35.0, -55.0], [82.0, -75.0, -44.0, 84.0, 35.0]], [[-12.0, -21.0, 22.0, 2.0, -12.0], [10.0, 0.0, 14.0, 35.0, 16.0], [0.0, -3.0, -17.0, 18.0, -38.0], [-15.0, 4.0, -6.0, -24.0, 39.0]], [[-46.0, 60.0, -36.0, -20.0, -27.0], [-40.0, -4.0, -14.0, -1.0, -12.0], [10.0, 24.0, 13.0, 23.0, 50.0], [16.0, -20.0, -6.0, -2.0, 37.0]], [[-90.0, 8.0, -44.0, -44.0, -33.0], [-36.0, -20.0, -38.0, -87.0, -4.0], [70.0, 20.0, 79.0, 33.0, 70.0], [-84.0, 88.0, 46.0, -106.0, -5.0]]], [[[41.0, 17.0, 24.0, -26.0, 7.0], [25.0, 65.0, 34.0, 42.0, -5.0], [-55.0, 7.0, 11.0, -22.0, -45.0], [27.0, -64.0, 36.0, 55.0, -52.0]], [[-104.0, 2.0, -82.0, -15.0, -19.0], [-88.0, -53.0, -67.0, -66.0, -7.0], [95.0, 43.0, 51.0, -25.0, 93.0], [-43.0, 61.0, 8.0, -78.0, 48.0]], [[59.0, -2.0, 34.0, -7.0, 19.0], [29.0, 60.0, 38.0, 69.0, 2.0], [-62.0, 4.0, -15.0, -42.0, -68.0], [45.0, -79.0, 18.0, 75.0, -38.0]], [[14.0, 22.0, 18.0, 4.0, -8.0], [-1.0, 26.0, 29.0, 88.0, 4.0], [-42.0, 10.0, -43.0, -3.0, -46.0], [52.0, -79.0, -22.0, 51.0, 50.0]], [[23.0, 25.0, 4.0, 14.0, 30.0], [33.0, -39.0, -48.0, 3.0, 59.0], [3.0, 11.0, 2.0, -9.0, 53.0], [59.0, 32.0, -54.0, 65.0, -61.0]]]])+(1.-msk_ref)*numpy.array([[[[14.0, 1.0, 28.0, 8.0, -34.0], [23.0, 12.0, 61.0, -51.0, -62.0], [-24.0, -65.0, -26.0, 103.0, -30.0], [-65.0, 13.0, 20.0, -71.0, 37.0]], [[-32.0, 65.0, -52.0, 38.0, -5.0], [-57.0, -70.0, -47.0, 18.0, 4.0], [24.0, 21.0, -49.0, 6.0, 86.0], [81.0, -25.0, -94.0, 47.0, 88.0]], [[118.0, -71.0, 52.0, 36.0, 57.0], [59.0, 38.0, 43.0, 8.0, -16.0], [-56.0, -43.0, -33.0, -64.0, -86.0], [15.0, -33.0, 18.0, 63.0, -84.0]], [[6.0, -45.0, 38.0, 25.0, 1.0], [7.0, 3.0, 24.0, 115.0, 35.0], [-17.0, 8.0, -63.0, -24.0, -89.0], [28.0, -46.0, -36.0, 19.0, 83.0]], [[-3.0, -2.0, 0.0, -47.0, -13.0], [12.0, 50.0, 23.0, -73.0, -38.0], [-4.0, -12.0, 68.0, 19.0, -2.0], [-73.0, 26.0, 88.0, -54.0, -74.0]]], [[[67.0, -59.0, 18.0, 25.0, 43.0], [24.0, 14.0, 13.0, -18.0, -14.0], [-16.0, -23.0, -9.0, -61.0, -40.0], [-2.0, -4.0, 18.0, 28.0, -61.0]], [[20.0, 76.0, 20.0, -18.0, -25.0], [20.0, 40.0, 40.0, 41.0, -16.0], [-62.0, -8.0, -19.0, 61.0, -10.0], [42.0, -68.0, -6.0, 46.0, 9.0]], [[4.0, -3.0, -38.0, -5.0, 0.0], [-45.0, 25.0, 16.0, -60.0, -75.0], [3.0, -8.0, 18.0, -31.0, 11.0], [-28.0, -28.0, 50.0, -19.0, -6.0]], [[27.0, 18.0, -2.0, 1.0, 15.0], [12.0, 4.0, -6.0, -16.0, -2.0], [-14.0, -3.0, 7.0, -9.0, 20.0], [23.0, -6.0, -2.0, 36.0, -46.0]], [[82.0, 0.0, 36.0, -45.0, 23.0], [60.0, 99.0, 50.0, -35.0, -33.0], [-69.0, -20.0, 58.0, -23.0, -47.0], [-19.0, -33.0, 98.0, 40.0, -158.0]]], [[[-38.0, 20.0, 2.0, -31.0, -30.0], [13.0, -1.0, -1.0, -48.0, 1.0], [17.0, -8.0, 40.0, 69.0, 27.0], [-55.0, 56.0, 26.0, -65.0, -13.0]], [[-13.0, -33.0, -16.0, 31.0, 10.0], [-24.0, -42.0, -22.0, -6.0, 4.0], [34.0, 0.0, -23.0, -24.0, 10.0], [-2.0, 22.0, -30.0, -16.0, 42.0]], [[-38.0, 21.0, -8.0, 7.0, -11.0], [-26.0, -23.0, -21.0, 87.0, 43.0], [9.0, 38.0, -32.0, -11.0, 1.0], [54.0, -33.0, -54.0, 30.0, 79.0]], [[17.0, -16.0, 10.0, -22.0, -11.0], [7.0, 51.0, 46.0, -43.0, -55.0], [-21.0, -26.0, 24.0, 14.0, -37.0], [-58.0, -10.0, 70.0, -41.0, -29.0]], [[38.0, 3.0, 12.0, -12.0, 21.0], [14.0, 42.0, 11.0, 41.0, 8.0], [-36.0, 16.0, 7.0, -51.0, -30.0], [39.0, -52.0, 18.0, 64.0, -50.0]]], [[[56.0, -4.0, 44.0, 23.0, 16.0], [44.0, 11.0, 26.0, 58.0, 19.0], [-47.0, -18.0, -47.0, 4.0, -49.0], [45.0, -37.0, -32.0, 58.0, -5.0]], [[34.0, 23.0, 12.0, -14.0, -12.0], [34.0, 26.0, 37.0, -100.0, -60.0], [-26.0, -54.0, 26.0, 74.0, 16.0], [-55.0, 28.0, 48.0, -38.0, -65.0]], [[-11.0, -18.0, -10.0, 13.0, -10.0], [-41.0, 12.0, 24.0, 44.0, -30.0], [-4.0, 6.0, -36.0, -27.0, -42.0], [5.0, -57.0, 0.0, -5.0, 81.0]], [[71.0, -39.0, 20.0, 30.0, 35.0], [19.0, 19.0, 26.0, 6.0, -23.0], [-33.0, -25.0, -31.0, -48.0, -47.0], [19.0, -36.0, 4.0, 45.0, -32.0]], [[51.0, 18.0, 4.0, 24.0, -14.0], [-6.0, 29.0, 69.0, -46.0, -103.0], [-49.0, -62.0, -46.0, 50.0, -25.0], [-16.0, -57.0, 18.0, -8.0, 32.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 2.0, 3.0], [4.0, 0.0, 6.0]], [[0.0, -6.0, 1.0], [7.0, 7.0, 5.0]], [[-2.0, -7.0, 4.0], [-2.0, -3.0, 2.0]], [[0.0, 0.0, 4.0], [-6.0, -5.0, 6.0]], [[-3.0, -5.0, 2.0], [7.0, 5.0, 1.0]]], [[[-6.0, -1.0, 1.0], [-7.0, 2.0, -1.0]], [[-6.0, -1.0, -7.0], [2.0, 5.0, -2.0]], [[-1.0, -6.0, -1.0], [3.0, -7.0, -6.0]], [[-5.0, -2.0, 6.0], [1.0, -5.0, -6.0]], [[0.0, 0.0, -2.0], [-7.0, -3.0, 6.0]]], [[[0.0, -7.0, 4.0], [-1.0, -5.0, -5.0]], [[-2.0, 1.0, 7.0], [2.0, 4.0, -3.0]], [[-5.0, -4.0, 7.0], [-7.0, 4.0, 2.0]], [[-6.0, 4.0, 1.0], [-2.0, 0.0, 0.0]], [[5.0, 0.0, -1.0], [0.0, -2.0, 3.0]]], [[[6.0, 2.0, -6.0], [-2.0, -7.0, 3.0]], [[-1.0, 4.0, 6.0], [0.0, 7.0, 7.0]], [[6.0, -1.0, 6.0], [0.0, -4.0, 2.0]], [[5.0, 2.0, 0.0], [0.0, -7.0, 5.0]], [[0.0, 4.0, 4.0], [-2.0, -2.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, -7.0, 4.0], [-3.0, -2.0, -3.0]], [[-5.0, -2.0, 5.0], [-1.0, -5.0, -3.0]], [[0.0, -7.0, 5.0], [6.0, -3.0, -2.0]], [[2.0, -7.0, -4.0], [0.0, -2.0, 6.0]], [[1.0, -3.0, -6.0], [0.0, 5.0, -4.0]]], [[[-6.0, -1.0, -5.0], [1.0, -7.0, 2.0]], [[4.0, 0.0, 7.0], [2.0, 2.0, 3.0]], [[0.0, -7.0, 0.0], [3.0, 1.0, 3.0]], [[4.0, -1.0, 3.0], [-1.0, -7.0, -3.0]], [[6.0, 0.0, -7.0], [5.0, 3.0, 4.0]]], [[[6.0, -5.0, -1.0], [-3.0, 1.0, -6.0]], [[4.0, 4.0, 0.0], [-3.0, 2.0, 1.0]], [[3.0, 5.0, 0.0], [6.0, -7.0, 7.0]], [[-7.0, -3.0, -5.0], [-5.0, -4.0, 3.0]], [[-4.0, 0.0, 3.0], [6.0, -4.0, -1.0]]], [[[0.0, -3.0, 0.0], [0.0, 1.0, -7.0]], [[0.0, 3.0, -3.0], [2.0, -3.0, 1.0]], [[-4.0, -1.0, 2.0], [-4.0, 2.0, -5.0]], [[7.0, -2.0, 7.0], [-4.0, 1.0, -2.0]], [[-2.0, 0.0, -7.0], [5.0, 0.0, 7.0]]]])
      arg1=Data(numpy.array([[1.0, -3.0, 0.0], [1.0, -3.0, 1.0]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 9.0, 28.0, 15.0, 5.0], [-17.0, -18.0, 35.0, 11.0, 8.0], [30.0, -18.0, -10.0, -20.0, 14.0], [22.0, -27.0, 23.0, 25.0, -1.0]])+(1.-msk_ref)*numpy.array([[16.0, 12.0, 34.0, 35.0, -9.0], [21.0, 3.0, 24.0, 24.0, 6.0], [9.0, -16.0, 22.0, 12.0, 13.0], [-1.0, 3.0, -16.0, 4.0, 10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_constData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, -5.0, 1.0], [-3.0, -7.0, 3.0]], [[-4.0, -7.0, -3.0], [0.0, -2.0, -7.0]], [[3.0, -3.0, -2.0], [-7.0, 3.0, 0.0]], [[-4.0, -7.0, -2.0], [-2.0, -3.0, -4.0]], [[7.0, 4.0, 0.0], [3.0, 2.0, 0.0]]], [[[-3.0, 5.0, -6.0], [-1.0, 6.0, 4.0]], [[-7.0, -3.0, 1.0], [-1.0, -5.0, 1.0]], [[1.0, -7.0, -3.0], [-3.0, 4.0, -7.0]], [[-4.0, 2.0, -4.0], [-2.0, -5.0, 0.0]], [[7.0, -6.0, -7.0], [-5.0, -1.0, -3.0]]], [[[-1.0, -7.0, 5.0], [3.0, 0.0, -6.0]], [[4.0, 7.0, -2.0], [-4.0, 4.0, -3.0]], [[-4.0, 0.0, -5.0], [0.0, 6.0, 6.0]], [[2.0, 5.0, 2.0], [-5.0, -4.0, 0.0]], [[-1.0, -3.0, -2.0], [3.0, 4.0, 1.0]]], [[[-2.0, -1.0, 7.0], [-4.0, -4.0, 0.0]], [[-3.0, -6.0, 2.0], [2.0, -1.0, 3.0]], [[-7.0, -2.0, -7.0], [1.0, 1.0, 4.0]], [[-1.0, 0.0, 2.0], [-5.0, 7.0, 7.0]], [[-2.0, 1.0, 7.0], [6.0, 1.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, -7.0, 0.0], [0.0, 3.0, -7.0]], [[7.0, 5.0, -2.0], [0.0, 1.0, -2.0]], [[7.0, -7.0, -7.0], [-3.0, 3.0, -1.0]], [[-3.0, -1.0, 0.0], [0.0, 0.0, 5.0]], [[-5.0, -1.0, -5.0], [1.0, 7.0, 2.0]]], [[[-2.0, 5.0, -4.0], [-4.0, 2.0, 5.0]], [[0.0, 0.0, -3.0], [-5.0, 7.0, -4.0]], [[-4.0, -5.0, 5.0], [-2.0, 0.0, 7.0]], [[4.0, 6.0, 6.0], [0.0, 2.0, -1.0]], [[0.0, -5.0, -5.0], [-3.0, -2.0, -1.0]]], [[[-1.0, 0.0, 0.0], [-1.0, 2.0, 2.0]], [[-1.0, -5.0, -7.0], [7.0, -3.0, 0.0]], [[-4.0, 0.0, 6.0], [1.0, 3.0, -2.0]], [[7.0, -3.0, -3.0], [1.0, 2.0, 5.0]], [[-4.0, 5.0, 4.0], [-6.0, -3.0, -7.0]]], [[[-2.0, 0.0, 5.0], [-2.0, 0.0, 0.0]], [[-5.0, 0.0, -4.0], [-4.0, 2.0, 7.0]], [[1.0, -6.0, 6.0], [-4.0, 3.0, -7.0]], [[-4.0, -2.0, 4.0], [6.0, 1.0, -6.0]], [[-1.0, 2.0, -2.0], [7.0, 0.0, 0.0]]]])
      arg1=Data(numpy.array([[[7.0, -1.0], [-1.0, -4.0], [4.0, -4.0]], [[6.0, -5.0], [4.0, -2.0], [1.0, -6.0]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-41.0, 28.0], [-48.0, 90.0], [-14.0, 46.0], [-57.0, 80.0], [71.0, -42.0]], [[-28.0, -24.0], [-67.0, 24.0], [-7.0, 88.0], [-78.0, 32.0], [-10.0, 90.0]], [[32.0, 30.0], [2.0, 6.0], [-18.0, -24.0], [-29.0, 3.0], [23.0, -8.0]], [[-25.0, 6.0], [4.0, -7.0], [-61.0, 12.0], [6.0, -38.0], [49.0, -38.0]]])+(1.-msk_ref)*numpy.array([[[19.0, 63.0], [38.0, -9.0], [21.0, 64.0], [-15.0, -23.0], [-18.0, -2.0]], [[-46.0, -16.0], [-18.0, 47.0], [-8.0, -28.0], [53.0, -50.0], [-42.0, 65.0]], [[-3.0, -10.0], [0.0, 20.0], [12.0, -19.0], [59.0, -22.0], [-72.0, 46.0]], [[-6.0, -8.0], [-60.0, -5.0], [18.0, 55.0], [24.0, 0.0], [25.0, -34.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_constData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, -3.0, -2.0], [0.0, 5.0, 3.0]], [[3.0, 1.0, -5.0], [-6.0, -3.0, -3.0]], [[-2.0, 4.0, 4.0], [7.0, 7.0, 6.0]], [[-2.0, -5.0, -4.0], [-4.0, -4.0, 2.0]], [[-3.0, -4.0, -2.0], [-4.0, -5.0, 2.0]]], [[[-1.0, -1.0, 6.0], [-3.0, -5.0, 5.0]], [[1.0, 4.0, -2.0], [5.0, -3.0, 0.0]], [[2.0, 7.0, 0.0], [1.0, 5.0, -5.0]], [[-3.0, -4.0, 3.0], [-5.0, 3.0, 5.0]], [[7.0, 3.0, -5.0], [7.0, -4.0, -1.0]]], [[[-6.0, -3.0, -1.0], [1.0, -7.0, -7.0]], [[4.0, 2.0, -4.0], [-3.0, 7.0, -6.0]], [[5.0, 7.0, -3.0], [3.0, -6.0, -6.0]], [[0.0, -5.0, -7.0], [7.0, -1.0, -1.0]], [[3.0, -1.0, -5.0], [-3.0, -7.0, -1.0]]], [[[-5.0, 6.0, 1.0], [2.0, -6.0, 5.0]], [[-4.0, -7.0, 1.0], [6.0, 0.0, -6.0]], [[-2.0, 7.0, 2.0], [-1.0, 5.0, -4.0]], [[-4.0, 1.0, -7.0], [-1.0, 6.0, -3.0]], [[5.0, 0.0, 2.0], [-2.0, 0.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -5.0, 5.0], [6.0, 5.0, -1.0]], [[-1.0, 0.0, -7.0], [-6.0, 0.0, 0.0]], [[-5.0, -1.0, -6.0], [0.0, -7.0, 7.0]], [[-6.0, 3.0, 3.0], [-2.0, -5.0, -3.0]], [[0.0, -4.0, 6.0], [-1.0, -5.0, 5.0]]], [[[-2.0, -4.0, 0.0], [-2.0, -1.0, 7.0]], [[-1.0, -4.0, 0.0], [-1.0, -3.0, 1.0]], [[3.0, 3.0, 7.0], [-4.0, -3.0, -3.0]], [[-2.0, 6.0, 4.0], [-5.0, -5.0, -5.0]], [[-3.0, -5.0, 5.0], [7.0, -2.0, 6.0]]], [[[6.0, 1.0, -5.0], [0.0, 0.0, -3.0]], [[1.0, 2.0, -4.0], [2.0, 4.0, 6.0]], [[0.0, -3.0, 4.0], [3.0, 0.0, 2.0]], [[-4.0, 0.0, 1.0], [-1.0, -3.0, 7.0]], [[-6.0, -6.0, -6.0], [6.0, 5.0, -6.0]]], [[[2.0, -3.0, -5.0], [3.0, 5.0, -3.0]], [[0.0, 3.0, 7.0], [3.0, -6.0, 2.0]], [[-1.0, 2.0, 7.0], [7.0, 4.0, -6.0]], [[-7.0, -5.0, -1.0], [-2.0, 7.0, 7.0]], [[-2.0, -3.0, 0.0], [0.0, -6.0, 7.0]]]])
      arg1=Data(numpy.array([[[[-6.0, 5.0, -2.0, -2.0, -1.0], [5.0, -4.0, 2.0, -5.0, -6.0], [-6.0, 0.0, 7.0, 3.0, 3.0], [-7.0, 5.0, 6.0, -4.0, 0.0]], [[-1.0, 5.0, -6.0, -6.0, 0.0], [-4.0, 5.0, 0.0, 3.0, -6.0], [7.0, 0.0, -1.0, -6.0, 4.0], [-3.0, 5.0, 6.0, -4.0, 4.0]], [[4.0, -3.0, 5.0, 5.0, -3.0], [2.0, 6.0, -2.0, 2.0, -2.0], [4.0, -6.0, -4.0, -2.0, -5.0], [3.0, 0.0, -3.0, -5.0, 0.0]]], [[[0.0, -5.0, -6.0, 0.0, 3.0], [5.0, 2.0, 0.0, 0.0, -1.0], [-4.0, -5.0, 0.0, 7.0, 0.0], [6.0, 5.0, 4.0, -7.0, 7.0]], [[-2.0, -3.0, 5.0, -6.0, -5.0], [-2.0, 5.0, -6.0, 2.0, -6.0], [-5.0, 4.0, 0.0, 4.0, -4.0], [4.0, 7.0, -6.0, -5.0, 6.0]], [[6.0, -6.0, 3.0, -7.0, 0.0], [-6.0, 0.0, -4.0, 5.0, 4.0], [0.0, -3.0, 6.0, 0.0, 6.0], [-4.0, 1.0, -5.0, -6.0, 3.0]]]]),self.functionspace)
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-9.0, -32.0, 38.0, -47.0, -21.0], [-10.0, -10.0, -34.0, 2.0, -8.0], [-66.0, 23.0, 43.0, 48.0, 2.0], [-3.0, 33.0, -45.0, -29.0, 27.0]], [[-51.0, 92.0, -25.0, 2.0, 9.0], [-5.0, -64.0, 46.0, -43.0, -2.0], [8.0, 57.0, 22.0, -41.0, 32.0], [-75.0, -34.0, 48.0, 84.0, -65.0]], [[46.0, -94.0, 11.0, -84.0, -24.0], [-33.0, 101.0, -78.0, 74.0, -45.0], [-7.0, -49.0, 2.0, 39.0, -2.0], [60.0, 100.0, -44.0, -148.0, 125.0]], [[21.0, -3.0, 24.0, 24.0, 22.0], [-22.0, -69.0, 20.0, -11.0, 86.0], [-3.0, 22.0, 19.0, -12.0, 22.0], [-31.0, -81.0, -32.0, 84.0, -66.0]], [[36.0, -6.0, 25.0, 36.0, 22.0], [-25.0, -53.0, 20.0, -1.0, 88.0], [23.0, 6.0, 3.0, -29.0, 17.0], [-25.0, -88.0, -32.0, 79.0, -68.0]]], [[[71.0, -28.0, 46.0, 33.0, -1.0], [-24.0, 4.0, -4.0, 29.0, 53.0], [60.0, -56.0, 0.0, -50.0, 13.0], [-30.0, -55.0, -37.0, -6.0, -40.0]], [[-12.0, 15.0, -81.0, -18.0, 35.0], [16.0, -1.0, 24.0, -3.0, -13.0], [9.0, -25.0, 11.0, 6.0, 41.0], [-7.0, 29.0, 74.0, -30.0, 33.0]], [[-59.0, 55.0, -42.0, -41.0, -24.0], [7.0, 54.0, -6.0, -4.0, -105.0], [8.0, 30.0, -23.0, -9.0, -16.0], [11.0, 80.0, 53.0, -38.0, 50.0]], [[58.0, -58.0, 105.0, -8.0, -36.0], [-54.0, 15.0, -50.0, 40.0, 43.0], [7.0, 4.0, 1.0, -14.0, -22.0], [4.0, -34.0, -114.0, 3.0, -18.0]], [[-63.0, 48.0, -122.0, -26.0, 49.0], [62.0, -49.0, 52.0, -49.0, -37.0], [-49.0, -18.0, 60.0, 46.0, 68.0], [-43.0, 56.0, 132.0, -38.0, 34.0]]], [[[7.0, 16.0, -37.0, 116.0, 47.0], [41.0, -30.0, 60.0, -30.0, 69.0], [42.0, -6.0, -77.0, -19.0, -39.0], [54.0, -96.0, 30.0, 111.0, -68.0]], [[-92.0, 72.0, -5.0, -40.0, -36.0], [11.0, -1.0, -2.0, -38.0, -91.0], [-49.0, 85.0, 6.0, 15.0, -24.0], [-12.0, 58.0, 24.0, 18.0, 11.0]], [[-73.0, 108.0, -133.0, 11.0, 43.0], [54.0, -27.0, 76.0, -52.0, -57.0], [25.0, -3.0, 4.0, -24.0, 46.0], [-47.0, 27.0, 159.0, 12.0, -5.0]], [[-27.0, -30.0, -55.0, 8.0, 47.0], [49.0, -58.0, 24.0, -36.0, 39.0], [-86.0, 6.0, 27.0, 89.0, 13.0], [36.0, 2.0, 30.0, 17.0, 20.0]], [[-29.0, 67.0, -45.0, 24.0, 38.0], [14.0, -88.0, 62.0, -47.0, 39.0], [2.0, 20.0, 36.0, -24.0, 52.0], [-75.0, -55.0, 62.0, 79.0, -70.0]]], [[[70.0, -20.0, -48.0, -20.0, 38.0], [-55.0, 30.0, 4.0, 58.0, 46.0], [98.0, -55.0, -15.0, -63.0, 58.0], [-12.0, -22.0, 22.0, -23.0, 17.0]], [[-1.0, -52.0, 1.0, 97.0, 19.0], [76.0, -1.0, 14.0, -29.0, 34.0], [-45.0, -18.0, -61.0, 70.0, -81.0], [112.0, -31.0, -15.0, 33.0, -4.0]], [[-21.0, 33.0, -9.0, -30.0, -32.0], [-25.0, 78.0, -22.0, 25.0, -79.0], [48.0, 25.0, -53.0, -39.0, -32.0], [29.0, 51.0, 10.0, -24.0, 39.0]], [[-35.0, 11.0, -6.0, -48.0, -8.0], [-37.0, 7.0, -18.0, 6.0, -15.0], [-23.0, 80.0, -19.0, 13.0, -15.0], [34.0, 19.0, -22.0, 42.0, 24.0]], [[-10.0, 17.0, 18.0, -14.0, -17.0], [7.0, -12.0, -2.0, -11.0, -24.0], [-14.0, -8.0, 39.0, -3.0, 17.0], [-49.0, 17.0, 6.0, -28.0, -8.0]]]])+(1.-msk_ref)*numpy.array([[[[9.0, -79.0, 41.0, 32.0, -22.0], [56.0, 42.0, -36.0, 0.0, -20.0], [-64.0, -37.0, -21.0, 82.0, -71.0], [90.0, 39.0, -46.0, -66.0, 49.0]], [[-22.0, 46.0, 3.0, -33.0, 4.0], [-49.0, -50.0, 12.0, -9.0, 26.0], [2.0, 72.0, 21.0, -31.0, 32.0], [-50.0, -35.0, -9.0, 81.0, -42.0]], [[63.0, -33.0, -28.0, -21.0, 58.0], [-61.0, -56.0, 16.0, 31.0, 118.0], [34.0, -13.0, 32.0, -25.0, 81.0], [-36.0, -72.0, -11.0, 47.0, -25.0]], [[37.0, 19.0, -13.0, 60.0, 16.0], [-18.0, 28.0, 24.0, 20.0, 32.0], [102.0, -19.0, -75.0, -76.0, -19.0], [22.0, -63.0, 10.0, 54.0, -41.0]], [[68.0, -48.0, 50.0, 49.0, 4.0], [3.0, -11.0, -2.0, 15.0, 63.0], [25.0, -66.0, 10.0, -15.0, 4.0], [-16.0, -55.0, -41.0, -12.0, -38.0]]], [[[60.0, -59.0, 56.0, -15.0, 1.0], [-44.0, -21.0, -26.0, 31.0, 72.0], [-3.0, -15.0, 32.0, 0.0, 24.0], [-18.0, -40.0, -73.0, 1.0, -15.0]], [[22.0, -17.0, 20.0, 37.0, 13.0], [6.0, -33.0, 12.0, -8.0, 53.0], [-3.0, -10.0, 3.0, 2.0, -1.0], [-3.0, -50.0, -21.0, 36.0, -38.0]], [[-5.0, 56.0, 11.0, 50.0, -21.0], [21.0, 22.0, 22.0, -13.0, -40.0], [62.0, -25.0, -28.0, -63.0, -20.0], [-33.0, -14.0, 32.0, 2.0, -43.0]], [[2.0, 78.0, -22.0, 53.0, 0.0], [-11.0, 27.0, 38.0, 1.0, -17.0], [115.0, -4.0, -66.0, -105.0, -12.0], [-22.0, -45.0, 47.0, 54.0, -56.0]], [[83.0, -120.0, 27.0, 31.0, 19.0], [18.0, 21.0, -28.0, 36.0, 67.0], [-15.0, -91.0, 0.0, 52.0, -10.0], [61.0, -13.0, -53.0, -68.0, 35.0]]], [[[-75.0, 68.0, -52.0, -22.0, 9.0], [34.0, -49.0, 34.0, -52.0, -44.0], [-49.0, 39.0, 43.0, 22.0, 29.0], [-48.0, 32.0, 72.0, 15.0, -5.0]], [[4.0, -31.0, -8.0, -100.0, -3.0], [-45.0, 6.0, -38.0, 31.0, -12.0], [-36.0, 12.0, 57.0, 29.0, 51.0], [-21.0, 59.0, -16.0, -62.0, 64.0]], [[31.0, -54.0, 26.0, 24.0, -3.0], [23.0, 15.0, -16.0, 9.0, 15.0], [-17.0, -45.0, -1.0, 31.0, -20.0], [31.0, 2.0, -28.0, -41.0, 15.0]], [[76.0, -51.0, 25.0, -18.0, 13.0], [-59.0, 5.0, -20.0, 51.0, 69.0], [47.0, -34.0, 10.0, -33.0, 37.0], [-15.0, -39.0, -48.0, -9.0, -4.0]], [[-28.0, -51.0, -11.0, 30.0, 17.0], [38.0, -5.0, -6.0, -20.0, 24.0], [-79.0, 44.0, -48.0, 92.0, -68.0], [122.0, -1.0, -30.0, 47.0, 30.0]]], [[[-57.0, -2.0, -13.0, -20.0, -3.0], [35.0, -22.0, -4.0, -34.0, -29.0], [-90.0, 44.0, 19.0, 75.0, -19.0], [30.0, 42.0, 6.0, 1.0, 30.0]], [[49.0, -15.0, -25.0, 39.0, 18.0], [17.0, 33.0, 14.0, 21.0, 9.0], [67.0, -87.0, -19.0, -35.0, 13.0], [-2.0, -10.0, 35.0, -50.0, 3.0]], [[-12.0, -27.0, -15.0, 43.0, -19.0], [64.0, 90.0, -16.0, 3.0, -75.0], [0.0, -43.0, -73.0, 36.0, -82.0], [104.0, 62.0, 19.0, -72.0, 63.0]], [[71.0, -110.0, 107.0, -52.0, -31.0], [-83.0, 28.0, -82.0, 67.0, 62.0], [-24.0, 23.0, 2.0, 25.0, -22.0], [49.0, -14.0, -154.0, -10.0, 29.0]], [[69.0, -49.0, 13.0, 9.0, 32.0], [-28.0, -37.0, 4.0, 24.0, 94.0], [21.0, -45.0, 31.0, -12.0, 48.0], [-29.0, -60.0, -29.0, 8.0, -27.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -4.0, 1.0], [3.0, -4.0, -3.0]], [[-4.0, -1.0, 7.0], [-4.0, -2.0, -2.0]], [[-6.0, -5.0, -2.0], [-5.0, -3.0, 0.0]], [[2.0, 0.0, 3.0], [0.0, 2.0, -5.0]], [[-2.0, -2.0, 0.0], [-6.0, 0.0, 0.0]]], [[[7.0, 6.0, 2.0], [-4.0, 7.0, -6.0]], [[7.0, -7.0, -1.0], [0.0, 7.0, 1.0]], [[-6.0, -4.0, 7.0], [-6.0, -3.0, -1.0]], [[7.0, -4.0, 6.0], [2.0, -7.0, -3.0]], [[6.0, 5.0, -1.0], [-6.0, -1.0, 0.0]]], [[[-7.0, -7.0, 2.0], [7.0, 3.0, 2.0]], [[-3.0, -3.0, 2.0], [3.0, -1.0, 2.0]], [[-2.0, -1.0, -2.0], [5.0, -4.0, 7.0]], [[-5.0, -7.0, -6.0], [7.0, -3.0, -3.0]], [[0.0, -2.0, -5.0], [7.0, 0.0, -1.0]]], [[[-4.0, 6.0, -4.0], [-1.0, 0.0, 0.0]], [[6.0, -5.0, 5.0], [2.0, 2.0, -1.0]], [[0.0, 0.0, -1.0], [1.0, 1.0, -6.0]], [[-1.0, 3.0, -1.0], [-1.0, 6.0, 1.0]], [[0.0, -7.0, -1.0], [3.0, 2.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 7.0, -3.0], [-6.0, -7.0, 6.0]], [[5.0, -7.0, 1.0], [-7.0, 0.0, 1.0]], [[-2.0, -1.0, 6.0], [0.0, 0.0, 1.0]], [[-6.0, 5.0, -6.0], [-1.0, 3.0, 3.0]], [[5.0, -5.0, -2.0], [3.0, 7.0, 5.0]]], [[[-6.0, 0.0, 4.0], [3.0, -4.0, -5.0]], [[4.0, 4.0, 0.0], [4.0, 7.0, 3.0]], [[-7.0, 7.0, 3.0], [0.0, 0.0, 3.0]], [[-5.0, 6.0, 6.0], [5.0, -5.0, -3.0]], [[0.0, 4.0, -3.0], [-4.0, -2.0, -5.0]]], [[[0.0, -6.0, 4.0], [6.0, 0.0, 5.0]], [[-5.0, 7.0, 5.0], [4.0, 0.0, -2.0]], [[7.0, -4.0, 7.0], [7.0, 5.0, 2.0]], [[-3.0, -4.0, -5.0], [-7.0, 0.0, 6.0]], [[5.0, 6.0, 3.0], [-4.0, -7.0, 0.0]]], [[[-4.0, -1.0, 0.0], [0.0, 0.0, -6.0]], [[-6.0, 0.0, 5.0], [1.0, 0.0, 0.0]], [[5.0, 0.0, -6.0], [-3.0, 3.0, 0.0]], [[3.0, 0.0, -3.0], [0.0, 5.0, 2.0]], [[-4.0, 1.0, 2.0], [6.0, 2.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 6.0, 6.0], [0.0, 4.0, 4.0]])+(1.-msk_arg1)*numpy.array([[1.0, -1.0, 7.0], [-4.0, 5.0, -6.0]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-64.0, 8.0, -72.0, 12.0, -18.0], [73.0, 5.0, -16.0, -7.0, 38.0], [-31.0, -11.0, -12.0, -117.0, -46.0], [0.0, 22.0, -26.0, 37.0, -36.0]])+(1.-msk_ref)*numpy.array([[-82.0, 41.0, 35.0, -52.0, -11.0], [20.0, 1.0, -11.0, 4.0, 11.0], [-20.0, 19.0, 45.0, -42.0, 1.0], [33.0, 25.0, -10.0, -5.0, 25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -4.0, -5.0], [5.0, 0.0, -6.0]], [[-5.0, -5.0, 6.0], [5.0, -5.0, 4.0]], [[-2.0, 1.0, 0.0], [6.0, 4.0, -1.0]], [[0.0, 0.0, 0.0], [-5.0, 2.0, 4.0]], [[-1.0, -1.0, 4.0], [-1.0, 1.0, 1.0]]], [[[-3.0, 1.0, 0.0], [0.0, -5.0, -6.0]], [[6.0, 6.0, 2.0], [-3.0, -6.0, 7.0]], [[-4.0, -3.0, 3.0], [-5.0, 3.0, -6.0]], [[0.0, -4.0, -5.0], [4.0, 3.0, -1.0]], [[-3.0, -1.0, 5.0], [2.0, -3.0, -2.0]]], [[[7.0, 0.0, 3.0], [-1.0, -2.0, 0.0]], [[1.0, 7.0, -6.0], [0.0, 0.0, -5.0]], [[4.0, -7.0, -3.0], [0.0, 4.0, 6.0]], [[1.0, 1.0, -6.0], [3.0, 5.0, 4.0]], [[0.0, 3.0, -6.0], [-7.0, -1.0, 7.0]]], [[[7.0, 0.0, -1.0], [-1.0, 4.0, 3.0]], [[-5.0, 0.0, 0.0], [1.0, -5.0, -3.0]], [[-6.0, 0.0, -6.0], [-5.0, -2.0, -2.0]], [[-4.0, 6.0, 7.0], [-7.0, -4.0, 0.0]], [[7.0, -6.0, 4.0], [-7.0, -7.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, 1.0, 7.0], [3.0, -2.0, -1.0]], [[-7.0, -3.0, 2.0], [7.0, -3.0, 7.0]], [[0.0, 3.0, 3.0], [0.0, 3.0, -6.0]], [[6.0, 6.0, 6.0], [-4.0, -2.0, -1.0]], [[-1.0, 1.0, -6.0], [6.0, 0.0, 6.0]]], [[[3.0, -5.0, -4.0], [2.0, 4.0, -6.0]], [[-5.0, 2.0, -6.0], [-2.0, -5.0, 7.0]], [[3.0, -7.0, -4.0], [7.0, -3.0, -4.0]], [[-3.0, -1.0, -1.0], [0.0, 2.0, 0.0]], [[-5.0, -3.0, -7.0], [-2.0, -3.0, 1.0]]], [[[3.0, 0.0, 2.0], [-2.0, 1.0, 0.0]], [[0.0, 6.0, -7.0], [-6.0, -1.0, -1.0]], [[0.0, -5.0, -4.0], [-3.0, 1.0, -3.0]], [[-6.0, -3.0, 0.0], [1.0, -2.0, -3.0]], [[-1.0, -2.0, 4.0], [5.0, 7.0, -1.0]]], [[[-4.0, 5.0, -3.0], [-2.0, -5.0, -7.0]], [[-1.0, 5.0, 7.0], [-5.0, 0.0, -3.0]], [[-3.0, 6.0, 6.0], [0.0, -1.0, 1.0]], [[4.0, 6.0, 4.0], [7.0, 7.0, -3.0]], [[7.0, -6.0, 3.0], [6.0, 0.0, 4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 0.0], [-2.0, -6.0], [-3.0, 6.0]], [[1.0, 5.0], [3.0, 0.0], [5.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, 5.0], [7.0, -2.0], [6.0, 2.0]], [[-7.0, 3.0], [6.0, 0.0], [-7.0, -7.0]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-22.0, 1.0], [27.0, 103.0], [21.0, 21.0], [21.0, -13.0], [2.0, 28.0]], [[-32.0, -24.0], [-34.0, -18.0], [-9.0, -7.0], [31.0, 11.0], [-15.0, 40.0]], [[-51.0, 13.0], [-26.0, -93.0], [45.0, 42.0], [49.0, -15.0], [37.0, -68.0]], [[-6.0, -2.0], [-4.0, -4.0], [27.0, -67.0], [-32.0, -29.0], [-58.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[-26.0, 63.0], [-76.0, -53.0], [99.0, 42.0], [59.0, 25.0], [-106.0, -43.0]], [[-28.0, 65.0], [-52.0, -96.0], [-133.0, 70.0], [20.0, -15.0], [-39.0, -46.0]], [[11.0, 13.0], [43.0, -37.0], [-11.0, 14.0], [23.0, 0.0], [31.0, 29.0]], [[78.0, 7.0], [140.0, 5.0], [86.0, -22.0], [52.0, 58.0], [-143.0, 43.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_mult_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -3.0, -5.0], [0.0, 4.0, -6.0]], [[7.0, -1.0, 2.0], [-5.0, 2.0, -3.0]], [[7.0, -1.0, 0.0], [0.0, -2.0, 0.0]], [[0.0, 7.0, 2.0], [-1.0, 5.0, 2.0]], [[0.0, 6.0, -7.0], [-7.0, 7.0, -7.0]]], [[[2.0, -6.0, 2.0], [-7.0, -5.0, 3.0]], [[-4.0, -1.0, 1.0], [3.0, 1.0, -5.0]], [[5.0, 6.0, -6.0], [-2.0, -7.0, -5.0]], [[7.0, 6.0, 4.0], [-4.0, -3.0, 4.0]], [[-4.0, 6.0, -1.0], [-4.0, -7.0, 4.0]]], [[[7.0, 4.0, 0.0], [2.0, -5.0, 4.0]], [[2.0, 3.0, 0.0], [2.0, 6.0, 2.0]], [[-2.0, 6.0, -1.0], [0.0, 7.0, -7.0]], [[3.0, -5.0, -6.0], [-5.0, -2.0, 7.0]], [[7.0, -6.0, 2.0], [-5.0, 5.0, -7.0]]], [[[4.0, -4.0, -1.0], [-1.0, 0.0, 5.0]], [[0.0, 7.0, 0.0], [2.0, 6.0, -3.0]], [[0.0, 5.0, 6.0], [0.0, -6.0, 0.0]], [[0.0, -2.0, -3.0], [-7.0, 5.0, -5.0]], [[0.0, 5.0, 5.0], [-6.0, 3.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, -7.0, -2.0], [-1.0, 6.0, -3.0]], [[0.0, 3.0, 0.0], [-4.0, -5.0, 1.0]], [[-5.0, 7.0, 4.0], [3.0, -5.0, 2.0]], [[0.0, -1.0, -3.0], [3.0, 5.0, -4.0]], [[5.0, -6.0, 2.0], [4.0, 4.0, 3.0]]], [[[-4.0, 3.0, 7.0], [-6.0, -7.0, -6.0]], [[-1.0, 0.0, -5.0], [6.0, 2.0, 6.0]], [[2.0, 4.0, 6.0], [-2.0, -2.0, -7.0]], [[-4.0, 2.0, -4.0], [-5.0, 1.0, -7.0]], [[-7.0, -3.0, -5.0], [1.0, 2.0, 7.0]]], [[[-6.0, 6.0, -7.0], [-3.0, -1.0, 3.0]], [[4.0, 2.0, 4.0], [0.0, -2.0, -7.0]], [[4.0, -1.0, 1.0], [0.0, -4.0, 0.0]], [[-5.0, -6.0, 7.0], [0.0, 3.0, 3.0]], [[7.0, -5.0, 7.0], [4.0, -3.0, 4.0]]], [[[7.0, -2.0, 5.0], [5.0, 1.0, 5.0]], [[-5.0, -1.0, 1.0], [-2.0, -5.0, -5.0]], [[-6.0, 5.0, 0.0], [-2.0, 6.0, -5.0]], [[-4.0, 2.0, -6.0], [1.0, -2.0, 1.0]], [[0.0, 0.0, -2.0], [-3.0, -2.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-1.0, -5.0, 2.0, -7.0, -3.0], [-5.0, 7.0, -1.0, -2.0, -6.0], [6.0, -1.0, -1.0, -3.0, -4.0], [6.0, -1.0, -5.0, 0.0, 6.0]], [[-5.0, -1.0, -5.0, -6.0, 5.0], [3.0, 0.0, -3.0, -1.0, 3.0], [7.0, -3.0, 5.0, -1.0, -1.0], [4.0, 6.0, 0.0, -5.0, 0.0]], [[-5.0, 0.0, 3.0, -1.0, 2.0], [-5.0, -6.0, 0.0, 0.0, 1.0], [-4.0, -6.0, 7.0, 4.0, 0.0], [-2.0, 4.0, 1.0, 0.0, -6.0]]], [[[4.0, -2.0, 0.0, -6.0, 5.0], [1.0, -7.0, 0.0, 5.0, -1.0], [-6.0, 0.0, -3.0, -3.0, 4.0], [2.0, -6.0, 0.0, -1.0, 2.0]], [[4.0, -3.0, 6.0, -3.0, 0.0], [-4.0, 2.0, -7.0, 1.0, 7.0], [7.0, 3.0, 1.0, 7.0, -1.0], [-2.0, -1.0, 6.0, -2.0, -1.0]], [[3.0, 7.0, 7.0, 1.0, -4.0], [-4.0, 2.0, -6.0, -7.0, -1.0], [0.0, -6.0, 0.0, -7.0, 4.0], [0.0, -3.0, -7.0, -2.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -1.0, 3.0, -4.0, 4.0], [7.0, 4.0, 5.0, 6.0, -1.0], [-2.0, 3.0, -5.0, 7.0, -4.0], [-6.0, 1.0, -5.0, -5.0, 6.0]], [[-6.0, -4.0, -4.0, 3.0, 6.0], [-6.0, 0.0, 6.0, -2.0, 6.0], [-7.0, -3.0, 7.0, -4.0, 3.0], [3.0, -4.0, 1.0, 2.0, -4.0]], [[5.0, 3.0, -2.0, 2.0, -7.0], [1.0, 1.0, 1.0, 2.0, 2.0], [-7.0, 6.0, 2.0, 0.0, -4.0], [6.0, 6.0, 2.0, 5.0, -3.0]]], [[[7.0, -5.0, -4.0, -4.0, 3.0], [-6.0, -2.0, -2.0, 1.0, -6.0], [-5.0, 5.0, -5.0, 4.0, -4.0], [5.0, 1.0, -1.0, 0.0, 2.0]], [[-7.0, -1.0, -4.0, -5.0, 7.0], [7.0, -6.0, -4.0, -7.0, 3.0], [-6.0, -6.0, 7.0, -1.0, 2.0], [1.0, 0.0, 5.0, 1.0, -5.0]], [[-7.0, 0.0, 0.0, -4.0, -1.0], [-1.0, -1.0, 7.0, -2.0, 6.0], [2.0, 0.0, 0.0, 3.0, -7.0], [1.0, -6.0, 1.0, 6.0, 0.0]]]])
      res=tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[34.0, -71.0, -10.0, -23.0, -13.0], [4.0, 54.0, 13.0, 41.0, -4.0], [51.0, 83.0, -50.0, 41.0, -41.0], [14.0, -28.0, 41.0, 19.0, 20.0]], [[-33.0, -51.0, 16.0, -24.0, -35.0], [-49.0, 70.0, 0.0, -15.0, -21.0], [71.0, 8.0, 19.0, 38.0, -61.0], [20.0, 32.0, 0.0, 12.0, 3.0]], [[-10.0, -28.0, 7.0, -37.0, -26.0], [-30.0, 45.0, 10.0, -15.0, -59.0], [21.0, -10.0, -14.0, -34.0, -25.0], [42.0, -11.0, -47.0, 9.0, 44.0]], [[-23.0, -6.0, 15.0, -51.0, 26.0], [-18.0, 9.0, -68.0, -21.0, 57.0], [82.0, -30.0, 57.0, 25.0, -8.0], [12.0, 45.0, 18.0, -48.0, -9.0]], [[-16.0, -62.0, -58.0, -15.0, 9.0], [46.0, 91.0, -25.0, 15.0, 74.0], [161.0, 87.0, 9.0, 85.0, -69.0], [10.0, 64.0, 84.0, -23.0, -14.0]]], [[[-21.0, 46.0, 31.0, 80.0, -79.0], [-37.0, 47.0, 33.0, -59.0, -59.0], [-31.0, -29.0, -2.0, -27.0, -13.0], [-20.0, 8.0, -59.0, 41.0, 6.0]], [[5.0, -23.0, -29.0, 7.0, 44.0], [31.0, -63.0, 30.0, 60.0, 31.0], [-46.0, 34.0, -2.0, 50.0, 8.0], [-26.0, -2.0, 62.0, 10.0, -50.0]], [[-56.0, -41.0, -115.0, -37.0, 13.0], [69.0, 61.0, 56.0, 2.0, -60.0], [59.0, 22.0, -18.0, -53.0, -47.0], [76.0, 41.0, -38.0, -4.0, 44.0]], [[-73.0, 4.0, 6.0, -52.0, -19.0], [-45.0, 55.0, -28.0, -71.0, -41.0], [71.0, -82.0, 60.0, -48.0, -31.0], [56.0, 60.0, -77.0, -28.0, 33.0]], [[-53.0, 71.0, -55.0, 42.0, 4.0], [51.0, 0.0, 11.0, -53.0, -8.0], [-3.0, -53.0, 32.0, -63.0, 17.0], [8.0, 55.0, -51.0, -20.0, 1.0]]], [[[-27.0, 0.0, -8.0, -66.0, -7.0], [-17.0, 33.0, -8.0, -41.0, -71.0], [23.0, -58.0, 2.0, -94.0, -3.0], [72.0, -2.0, -93.0, -20.0, 71.0]], [[21.0, -21.0, 39.0, -60.0, 11.0], [-31.0, 16.0, -65.0, -5.0, 35.0], [63.0, -5.0, 13.0, 13.0, -1.0], [16.0, -8.0, 12.0, -33.0, 20.0]], [[-16.0, -66.0, -44.0, -49.0, 62.0], [33.0, -8.0, -23.0, 54.0, 85.0], [83.0, 53.0, 32.0, 94.0, -33.0], [0.0, 48.0, 100.0, -30.0, -48.0]], [[45.0, 55.0, 50.0, 58.0, -99.0], [-25.0, 102.0, -16.0, -77.0, -55.0], [23.0, 0.0, -57.0, -76.0, 3.0], [4.0, -46.0, -82.0, 20.0, 81.0]], [[-8.0, -83.0, 31.0, -7.0, -44.0], [-60.0, 68.0, 18.0, 21.0, -11.0], [57.0, 56.0, -3.0, 92.0, -75.0], [-6.0, 11.0, 46.0, 39.0, -20.0]]], [[[32.0, 21.0, 60.0, 8.0, -59.0], [-48.0, 51.0, -22.0, -44.0, -41.0], [6.0, -16.0, -28.0, -44.0, 4.0], [8.0, -41.0, -56.0, 11.0, 53.0]], [[-12.0, -50.0, -20.0, -75.0, 57.0], [11.0, -8.0, -45.0, 30.0, 64.0], [79.0, 15.0, 35.0, 50.0, -17.0], [20.0, 33.0, 57.0, -43.0, -17.0]], [[-79.0, 13.0, -43.0, -18.0, 37.0], [9.0, -48.0, 27.0, -11.0, -21.0], [-31.0, -69.0, 61.0, -23.0, 1.0], [20.0, 60.0, -30.0, -13.0, -30.0]], [[2.0, -34.0, -4.0, 37.0, -31.0], [2.0, 67.0, 1.0, 7.0, 38.0], [75.0, 69.0, -5.0, 81.0, -51.0], [-26.0, 28.0, 62.0, 17.0, -26.0]], [[-47.0, 33.0, 43.0, -3.0, -15.0], [-48.0, 28.0, -66.0, -67.0, 42.0], [72.0, -66.0, 81.0, 19.0, -12.0], [-8.0, 68.0, -12.0, -35.0, -20.0]]]])+(1.-msk_ref)*numpy.array([[[[40.0, 15.0, 30.0, -63.0, 38.0], [133.0, -9.0, -57.0, 9.0, -46.0], [14.0, -14.0, -36.0, 51.0, 0.0], [-71.0, 39.0, -13.0, -66.0, 38.0]], [[-18.0, 13.0, 24.0, 46.0, -30.0], [-30.0, 37.0, 53.0, 23.0, 33.0], [31.0, 1.0, 6.0, -20.0, 8.0], [-15.0, -22.0, -17.0, 7.0, 5.0]], [[-10.0, -21.0, -43.0, 54.0, -34.0], [-128.0, 6.0, 49.0, -2.0, 34.0], [-48.0, 33.0, 32.0, -40.0, -11.0], [87.0, -18.0, 14.0, 66.0, -39.0]], [[5.0, -25.0, -22.0, -30.0, 63.0], [24.0, -35.0, -63.0, -28.0, -39.0], [-25.0, -30.0, 7.0, -1.0, 35.0], [-5.0, 13.0, 11.0, -36.0, -6.0]], [[55.0, 1.0, 3.0, -82.0, 7.0], [74.0, -13.0, -12.0, 16.0, -31.0], [-20.0, 41.0, -55.0, 80.0, -75.0], [-9.0, 27.0, -8.0, -5.0, 36.0]]], [[[42.0, 50.0, 14.0, 122.0, -108.0], [-46.0, 51.0, 3.0, 39.0, 15.0], [-2.0, 33.0, 36.0, -75.0, 49.0], [32.0, 56.0, 2.0, 18.0, -34.0]], [[-45.0, -46.0, -25.0, -64.0, 57.0], [-40.0, -39.0, 12.0, -36.0, -3.0], [7.0, -15.0, -21.0, 33.0, -38.0], [14.0, -61.0, 5.0, 18.0, 11.0]], [[67.0, 12.0, -6.0, 62.0, -23.0], [1.0, 37.0, 3.0, 42.0, -2.0], [-66.0, 32.0, 26.0, -29.0, 33.0], [17.0, 62.0, -9.0, -16.0, -16.0]], [[-49.0, 8.0, 4.0, 57.0, 23.0], [0.0, -9.0, -55.0, -34.0, -1.0], [27.0, -73.0, 58.0, -78.0, 109.0], [-25.0, 1.0, 17.0, -37.0, -35.0]], [[-105.0, -3.0, -11.0, -33.0, -1.0], [-35.0, -54.0, -19.0, -73.0, 21.0], [67.0, -49.0, 13.0, -14.0, -10.0], [17.0, -66.0, 38.0, 48.0, -23.0]]], [[[-142.0, -23.0, -12.0, 33.0, 42.0], [-77.0, -22.0, 30.0, -64.0, 61.0], [46.0, -87.0, 66.0, -68.0, 59.0], [-1.0, -93.0, 23.0, 24.0, -40.0]], [[95.0, 2.0, 4.0, 36.0, -7.0], [13.0, 39.0, -5.0, 56.0, -32.0], [-52.0, 42.0, -12.0, 1.0, 19.0], [-3.0, 62.0, -27.0, -40.0, 14.0]], [[63.0, 7.0, 30.0, 3.0, -25.0], [7.0, 41.0, 31.0, 56.0, -20.0], [16.0, 45.0, -53.0, 36.0, -31.0], [-25.0, 14.0, -39.0, -21.0, 45.0]], [[-1.0, 47.0, -17.0, -11.0, -87.0], [26.0, -34.0, -45.0, -31.0, 10.0], [-9.0, 27.0, 18.0, -5.0, -41.0], [60.0, 43.0, 51.0, 69.0, -42.0]], [[128.0, 17.0, 23.0, -46.0, -64.0], [37.0, 41.0, 44.0, 83.0, -32.0], [-22.0, 116.0, -97.0, 100.0, -121.0], [6.0, 49.0, -41.0, 11.0, 64.0]]], [[[72.0, -10.0, -5.0, -69.0, -2.0], [38.0, 12.0, 49.0, 44.0, -6.0], [-56.0, 76.0, -57.0, 91.0, -107.0], [13.0, 20.0, -22.0, 17.0, 40.0]], [[37.0, 27.0, 15.0, 72.0, -69.0], [-46.0, 20.0, -41.0, 17.0, -32.0], [40.0, 14.0, -5.0, -49.0, 46.0], [13.0, 33.0, -2.0, -7.0, -8.0]], [[-87.0, -10.0, -54.0, 37.0, 47.0], [-13.0, -51.0, -55.0, -80.0, 36.0], [-59.0, -79.0, 117.0, -91.0, 94.0], [42.0, 2.0, 62.0, 16.0, -90.0]], [[-52.0, -25.0, -4.0, 12.0, 26.0], [-67.0, -13.0, -1.0, -27.0, -2.0], [45.0, -37.0, 3.0, -27.0, 31.0], [-2.0, -53.0, 0.0, -2.0, -2.0]], [[-52.0, 11.0, 24.0, -2.0, -14.0], [-3.0, 11.0, 47.0, -3.0, 38.0], [51.0, -15.0, -3.0, 5.0, -19.0], [-24.0, -45.0, -6.0, 18.0, 10.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_float_rank0_offset0(self):
      arg0=7.0
      arg1=6.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=42.0
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_array_rank0_offset0(self):
      arg0=0.0
      arg1=numpy.array(-6.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-0.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_array_rank1_offset0(self):
      arg0=-2.0
      arg1=numpy.array([-7.0, -5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([14.0, 10.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_array_rank2_offset0(self):
      arg0=-3.0
      arg1=numpy.array([[0.0, -4.0, 2.0, -7.0, -3.0], [-7.0, 4.0, -1.0, 3.0, 0.0], [3.0, 4.0, -3.0, 7.0, -4.0], [5.0, 4.0, 4.0, 0.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-0.0, 12.0, -6.0, 21.0, 9.0], [21.0, -12.0, 3.0, -9.0, -0.0], [-9.0, -12.0, 9.0, -21.0, 12.0], [-15.0, -12.0, -12.0, -0.0, -9.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_array_rank3_offset0(self):
      arg0=-6.0
      arg1=numpy.array([[[-4.0, 7.0], [-1.0, -3.0]], [[3.0, -1.0], [-5.0, -6.0]], [[-6.0, 6.0], [5.0, -5.0]], [[0.0, 6.0], [7.0, 7.0]], [[-6.0, 7.0], [4.0, 5.0]], [[6.0, 1.0], [-5.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[24.0, -42.0], [6.0, 18.0]], [[-18.0, 6.0], [30.0, 36.0]], [[36.0, -36.0], [-30.0, 30.0]], [[-0.0, -36.0], [-42.0, -42.0]], [[36.0, -42.0], [-24.0, -30.0]], [[-36.0, -6.0], [30.0, -12.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_array_rank4_offset0(self):
      arg0=7.0
      arg1=numpy.array([[[[0.0, -7.0, -6.0, 5.0], [-5.0, 2.0, 6.0, 0.0], [0.0, 1.0, -1.0, -1.0]], [[0.0, -2.0, -1.0, 3.0], [-4.0, 0.0, 2.0, -3.0], [-3.0, -6.0, -3.0, -2.0]]], [[[6.0, 7.0, 5.0, 5.0], [5.0, 1.0, 0.0, -3.0], [0.0, 0.0, -1.0, 0.0]], [[-6.0, 1.0, -7.0, -3.0], [7.0, 7.0, -5.0, 6.0], [-7.0, 1.0, 0.0, 0.0]]], [[[2.0, -2.0, 6.0, -6.0], [-7.0, 2.0, 0.0, -2.0], [3.0, -6.0, -3.0, 1.0]], [[-6.0, 1.0, 2.0, 3.0], [-2.0, 5.0, 4.0, 1.0], [4.0, 0.0, -2.0, -2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[0.0, -49.0, -42.0, 35.0], [-35.0, 14.0, 42.0, 0.0], [0.0, 7.0, -7.0, -7.0]], [[0.0, -14.0, -7.0, 21.0], [-28.0, 0.0, 14.0, -21.0], [-21.0, -42.0, -21.0, -14.0]]], [[[42.0, 49.0, 35.0, 35.0], [35.0, 7.0, 0.0, -21.0], [0.0, 0.0, -7.0, 0.0]], [[-42.0, 7.0, -49.0, -21.0], [49.0, 49.0, -35.0, 42.0], [-49.0, 7.0, 0.0, 0.0]]], [[[14.0, -14.0, 42.0, -42.0], [-49.0, 14.0, 0.0, -14.0], [21.0, -42.0, -21.0, 7.0]], [[-42.0, 7.0, 14.0, 21.0], [-14.0, 35.0, 28.0, 7.0], [28.0, 0.0, -14.0, -14.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_Symbol_rank0_offset0(self):
      arg0=-4.0
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(0.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-0.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_Symbol_rank1_offset0(self):
      arg0=-3.0
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([4.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-12.0, -12.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_Symbol_rank2_offset0(self):
      arg0=-3.0
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[0.0, 3.0, 4.0, 7.0, -6.0], [3.0, 1.0, 2.0, 1.0, 7.0], [7.0, -6.0, 6.0, 7.0, 7.0], [0.0, 4.0, 6.0, 0.0, 1.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.0, -9.0, -12.0, -21.0, 18.0], [-9.0, -3.0, -6.0, -3.0, -21.0], [-21.0, 18.0, -18.0, -21.0, -21.0], [-0.0, -12.0, -18.0, -0.0, -3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_Symbol_rank3_offset0(self):
      arg0=-3.0
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-2.0, 6.0], [3.0, 4.0]], [[-5.0, 7.0], [7.0, 1.0]], [[1.0, 0.0], [0.0, -7.0]], [[0.0, 5.0], [2.0, 7.0]], [[0.0, -1.0], [-6.0, 0.0]], [[4.0, -1.0], [5.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[6.0, -18.0], [-9.0, -12.0]], [[15.0, -21.0], [-21.0, -3.0]], [[-3.0, -0.0], [-0.0, 21.0]], [[-0.0, -15.0], [-6.0, -21.0]], [[-0.0, 3.0], [18.0, -0.0]], [[-12.0, 3.0], [-15.0, 18.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_Symbol_rank4_offset0(self):
      arg0=7.0
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[3.0, -4.0, 1.0, 4.0], [-1.0, -1.0, -6.0, 0.0], [-6.0, -4.0, -1.0, -6.0]], [[-4.0, 2.0, 2.0, 7.0], [4.0, -7.0, 0.0, 3.0], [0.0, 3.0, 2.0, 5.0]]], [[[-5.0, -3.0, -2.0, -6.0], [0.0, -5.0, -5.0, 0.0], [-7.0, 1.0, -3.0, -5.0]], [[1.0, -4.0, 1.0, -7.0], [4.0, -3.0, 4.0, -6.0], [4.0, -2.0, 0.0, 3.0]]], [[[-7.0, 5.0, -3.0, -6.0], [-2.0, 5.0, -3.0, -2.0], [5.0, 7.0, -5.0, 5.0]], [[-2.0, -3.0, -6.0, -6.0], [1.0, 4.0, 3.0, 3.0], [-1.0, -4.0, 2.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[21.0, -28.0, 7.0, 28.0], [-7.0, -7.0, -42.0, 0.0], [-42.0, -28.0, -7.0, -42.0]], [[-28.0, 14.0, 14.0, 49.0], [28.0, -49.0, 0.0, 21.0], [0.0, 21.0, 14.0, 35.0]]], [[[-35.0, -21.0, -14.0, -42.0], [0.0, -35.0, -35.0, 0.0], [-49.0, 7.0, -21.0, -35.0]], [[7.0, -28.0, 7.0, -49.0], [28.0, -21.0, 28.0, -42.0], [28.0, -14.0, 0.0, 21.0]]], [[[-49.0, 35.0, -21.0, -42.0], [-14.0, 35.0, -21.0, -14.0], [35.0, 49.0, -35.0, 35.0]], [[-14.0, -21.0, -42.0, -42.0], [7.0, 28.0, 21.0, 21.0], [-7.0, -28.0, 14.0, -28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_constData_rank0_offset0(self):
      arg0=7.0
      arg1=Data(-4.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-28.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_constData_rank1_offset0(self):
      arg0=3.0
      arg1=Data(numpy.array([-1.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-3.0, -18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_constData_rank2_offset0(self):
      arg0=-1.0
      arg1=Data(numpy.array([[-7.0, 0.0, -7.0, -1.0, -4.0], [0.0, -4.0, -1.0, 3.0, 0.0], [4.0, -5.0, 6.0, -3.0, 0.0], [0.0, -2.0, 3.0, -2.0, 4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[7.0, -0.0, 7.0, 1.0, 4.0], [-0.0, 4.0, 1.0, -3.0, -0.0], [-4.0, 5.0, -6.0, 3.0, -0.0], [-0.0, 2.0, -3.0, 2.0, -4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_constData_rank3_offset0(self):
      arg0=-6.0
      arg1=Data(numpy.array([[[-6.0, 2.0], [4.0, 2.0]], [[-5.0, -2.0], [-4.0, -3.0]], [[-6.0, 2.0], [-1.0, -4.0]], [[0.0, 0.0], [0.0, 3.0]], [[5.0, 5.0], [-7.0, -6.0]], [[0.0, 5.0], [6.0, 3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[36.0, -12.0], [-24.0, -12.0]], [[30.0, 12.0], [24.0, 18.0]], [[36.0, -12.0], [6.0, 24.0]], [[-0.0, -0.0], [-0.0, -18.0]], [[-30.0, -30.0], [42.0, 36.0]], [[-0.0, -30.0], [-36.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_constData_rank4_offset0(self):
      arg0=-6.0
      arg1=Data(numpy.array([[[[7.0, 1.0, -7.0, 0.0], [-6.0, 0.0, -5.0, 4.0], [-6.0, 7.0, 0.0, -5.0]], [[2.0, 3.0, 5.0, -4.0], [7.0, -3.0, 0.0, -6.0], [2.0, -6.0, 7.0, 2.0]]], [[[3.0, 0.0, 1.0, -2.0], [-4.0, -1.0, 0.0, -5.0], [7.0, 3.0, 0.0, -6.0]], [[-7.0, -1.0, 1.0, -7.0], [7.0, 4.0, -1.0, -3.0], [5.0, -2.0, 2.0, 6.0]]], [[[-3.0, -1.0, 7.0, 0.0], [0.0, 6.0, 5.0, 0.0], [2.0, -7.0, 3.0, -4.0]], [[-5.0, 3.0, 5.0, 2.0], [-5.0, 1.0, 3.0, -2.0], [2.0, -3.0, 5.0, -5.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-42.0, -6.0, 42.0, -0.0], [36.0, -0.0, 30.0, -24.0], [36.0, -42.0, -0.0, 30.0]], [[-12.0, -18.0, -30.0, 24.0], [-42.0, 18.0, -0.0, 36.0], [-12.0, 36.0, -42.0, -12.0]]], [[[-18.0, -0.0, -6.0, 12.0], [24.0, 6.0, -0.0, 30.0], [-42.0, -18.0, -0.0, 36.0]], [[42.0, 6.0, -6.0, 42.0], [-42.0, -24.0, 6.0, 18.0], [-30.0, 12.0, -12.0, -36.0]]], [[[18.0, 6.0, -42.0, -0.0], [-0.0, -36.0, -30.0, -0.0], [-12.0, 42.0, -18.0, 24.0]], [[30.0, -18.0, -30.0, -12.0], [30.0, -6.0, -18.0, 12.0], [-12.0, 18.0, -30.0, 30.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_expandedData_rank0_offset0(self):
      arg0=7.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-6.0)+(1-msk_arg1)*(-3.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-42.0)+(1.-msk_ref)*numpy.array(-21.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_expandedData_rank1_offset0(self):
      arg0=-6.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 2.0])+(1.-msk_arg1)*numpy.array([0.0, -5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-42.0, -12.0])+(1.-msk_ref)*numpy.array([-0.0, 30.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_expandedData_rank2_offset0(self):
      arg0=-1.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 6.0, 6.0, 7.0, 0.0], [7.0, -4.0, 4.0, -2.0, 7.0], [-1.0, -6.0, 5.0, -5.0, 7.0], [-2.0, 0.0, -4.0, 3.0, 3.0]])+(1.-msk_arg1)*numpy.array([[-5.0, 4.0, 5.0, -2.0, 7.0], [-6.0, 1.0, -3.0, 3.0, 0.0], [-4.0, 0.0, 5.0, -3.0, 0.0], [-2.0, 6.0, -7.0, 0.0, -1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-1.0, -6.0, -6.0, -7.0, -0.0], [-7.0, 4.0, -4.0, 2.0, -7.0], [1.0, 6.0, -5.0, 5.0, -7.0], [2.0, -0.0, 4.0, -3.0, -3.0]])+(1.-msk_ref)*numpy.array([[5.0, -4.0, -5.0, 2.0, -7.0], [6.0, -1.0, 3.0, -3.0, -0.0], [4.0, -0.0, -5.0, 3.0, -0.0], [2.0, -6.0, 7.0, -0.0, 1.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_expandedData_rank3_offset0(self):
      arg0=7.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -5.0], [-6.0, 0.0]], [[3.0, -7.0], [4.0, -1.0]], [[1.0, -6.0], [-6.0, -6.0]], [[0.0, -4.0], [-6.0, 7.0]], [[-3.0, 7.0], [-6.0, 1.0]], [[-2.0, 3.0], [-6.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -4.0], [2.0, 1.0]], [[0.0, 0.0], [-7.0, -4.0]], [[-6.0, -1.0], [3.0, -3.0]], [[-4.0, -6.0], [5.0, 1.0]], [[-5.0, 7.0], [-4.0, 0.0]], [[7.0, -7.0], [2.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -35.0], [-42.0, 0.0]], [[21.0, -49.0], [28.0, -7.0]], [[7.0, -42.0], [-42.0, -42.0]], [[0.0, -28.0], [-42.0, 49.0]], [[-21.0, 49.0], [-42.0, 7.0]], [[-14.0, 21.0], [-42.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, -28.0], [14.0, 7.0]], [[0.0, 0.0], [-49.0, -28.0]], [[-42.0, -7.0], [21.0, -21.0]], [[-28.0, -42.0], [35.0, 7.0]], [[-35.0, 49.0], [-28.0, 0.0]], [[49.0, -49.0], [14.0, -21.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_float_rank0_expandedData_rank4_offset0(self):
      arg0=3.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, -4.0, -6.0, 7.0], [7.0, 3.0, 3.0, 6.0], [0.0, -7.0, 6.0, 1.0]], [[-7.0, 1.0, 4.0, 4.0], [5.0, 4.0, 1.0, -7.0], [7.0, 0.0, -6.0, 2.0]]], [[[0.0, -7.0, -3.0, 2.0], [1.0, -7.0, 2.0, -6.0], [0.0, 5.0, -1.0, -3.0]], [[-7.0, -1.0, 3.0, 0.0], [5.0, 0.0, 0.0, -6.0], [-5.0, -3.0, 0.0, -5.0]]], [[[-7.0, 0.0, 2.0, -3.0], [0.0, 6.0, -4.0, -1.0], [-4.0, -2.0, 4.0, 7.0]], [[-7.0, 6.0, -5.0, -1.0], [6.0, 2.0, 2.0, -3.0], [-5.0, -7.0, 2.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -4.0, 6.0, -6.0], [7.0, 7.0, -3.0, -2.0], [3.0, -5.0, 0.0, 0.0]], [[3.0, 1.0, 0.0, -1.0], [-3.0, -3.0, 0.0, -3.0], [-3.0, -6.0, 6.0, -2.0]]], [[[-2.0, 3.0, 0.0, -2.0], [0.0, 5.0, -4.0, -3.0], [-5.0, -5.0, -7.0, -1.0]], [[-4.0, -5.0, 2.0, -1.0], [2.0, -1.0, 0.0, 1.0], [6.0, 6.0, 3.0, -4.0]]], [[[-7.0, -6.0, 3.0, -2.0], [-6.0, 7.0, 3.0, -3.0], [-1.0, 2.0, -3.0, 1.0]], [[3.0, -3.0, 0.0, -5.0], [2.0, -6.0, -7.0, 5.0], [-2.0, 0.0, 3.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[9.0, -12.0, -18.0, 21.0], [21.0, 9.0, 9.0, 18.0], [0.0, -21.0, 18.0, 3.0]], [[-21.0, 3.0, 12.0, 12.0], [15.0, 12.0, 3.0, -21.0], [21.0, 0.0, -18.0, 6.0]]], [[[0.0, -21.0, -9.0, 6.0], [3.0, -21.0, 6.0, -18.0], [0.0, 15.0, -3.0, -9.0]], [[-21.0, -3.0, 9.0, 0.0], [15.0, 0.0, 0.0, -18.0], [-15.0, -9.0, 0.0, -15.0]]], [[[-21.0, 0.0, 6.0, -9.0], [0.0, 18.0, -12.0, -3.0], [-12.0, -6.0, 12.0, 21.0]], [[-21.0, 18.0, -15.0, -3.0], [18.0, 6.0, 6.0, -9.0], [-15.0, -21.0, 6.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -12.0, 18.0, -18.0], [21.0, 21.0, -9.0, -6.0], [9.0, -15.0, 0.0, 0.0]], [[9.0, 3.0, 0.0, -3.0], [-9.0, -9.0, 0.0, -9.0], [-9.0, -18.0, 18.0, -6.0]]], [[[-6.0, 9.0, 0.0, -6.0], [0.0, 15.0, -12.0, -9.0], [-15.0, -15.0, -21.0, -3.0]], [[-12.0, -15.0, 6.0, -3.0], [6.0, -3.0, 0.0, 3.0], [18.0, 18.0, 9.0, -12.0]]], [[[-21.0, -18.0, 9.0, -6.0], [-18.0, 21.0, 9.0, -9.0], [-3.0, 6.0, -9.0, 3.0]], [[9.0, -9.0, 0.0, -15.0], [6.0, -18.0, -21.0, 15.0], [-6.0, 0.0, 9.0, 3.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_float_rank0_offset0(self):
      arg0=numpy.array(4.0)
      arg1=-5.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-20.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_array_rank0_offset0(self):
      arg0=numpy.array(-3.0)
      arg1=numpy.array(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(21.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank1_offset1(self):
      arg0=numpy.array([-4.0, 0.0, -6.0])
      arg1=numpy.array([-4.0, 4.0, -5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array(46.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank2_offset2(self):
      arg0=numpy.array([[-4.0, -6.0, 0.0], [5.0, 0.0, 0.0]])
      arg1=numpy.array([[1.0, 1.0, 4.0], [5.0, -3.0, 2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array(15.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank3_offset3(self):
      arg0=numpy.array([[[-4.0, 6.0, -7.0], [-1.0, 0.0, -3.0], [0.0, -7.0, 5.0], [4.0, -4.0, -3.0]], [[0.0, 6.0, 5.0], [-4.0, 0.0, -1.0], [0.0, 7.0, 6.0], [-2.0, -6.0, 6.0]]])
      arg1=numpy.array([[[-1.0, -6.0, -6.0], [-7.0, 3.0, 0.0], [5.0, 0.0, 1.0], [-6.0, 3.0, -7.0]], [[4.0, 1.0, -3.0], [-3.0, 7.0, -1.0], [5.0, 5.0, -6.0], [6.0, -1.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array(10.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank4_offset4(self):
      arg0=numpy.array([[[[0.0, 5.0], [-6.0, -7.0], [-5.0, 1.0]], [[1.0, 3.0], [-6.0, 1.0], [-3.0, -1.0]]], [[[-4.0, 0.0], [5.0, 7.0], [-3.0, -2.0]], [[5.0, -3.0], [6.0, 0.0], [6.0, -2.0]]], [[[-4.0, -3.0], [3.0, -5.0], [4.0, 6.0]], [[-4.0, -3.0], [5.0, 6.0], [3.0, 2.0]]], [[[-6.0, -2.0], [-2.0, 5.0], [-4.0, 5.0]], [[2.0, -6.0], [-3.0, -2.0], [-1.0, 3.0]]]])
      arg1=numpy.array([[[[-5.0, -5.0], [-6.0, -1.0], [-4.0, -7.0]], [[-7.0, -4.0], [-2.0, -1.0], [-7.0, -2.0]]], [[[2.0, 1.0], [3.0, -6.0], [-7.0, 2.0]], [[7.0, 3.0], [3.0, 7.0], [-3.0, 7.0]]], [[[-5.0, 6.0], [6.0, -5.0], [-2.0, 7.0]], [[5.0, 4.0], [-3.0, -7.0], [-2.0, 4.0]]], [[[-1.0, 6.0], [0.0, 2.0], [1.0, 7.0]], [[-5.0, 1.0], [2.0, -2.0], [5.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      ref=numpy.array(29.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_array_rank1_offset0(self):
      arg0=numpy.array(-7.0)
      arg1=numpy.array([0.0, -6.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-0.0, 42.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank2_offset1(self):
      arg0=numpy.array([-5.0, -6.0, -2.0])
      arg1=numpy.array([[6.0, 6.0], [0.0, 7.0], [-1.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([-28.0, -60.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank3_offset2(self):
      arg0=numpy.array([[4.0, 2.0, 5.0], [-5.0, 6.0, -6.0]])
      arg1=numpy.array([[[4.0, 3.0], [-2.0, -5.0], [-6.0, 6.0]], [[-7.0, 5.0], [-1.0, -6.0], [-6.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([47.0, -29.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank4_offset3(self):
      arg0=numpy.array([[[-5.0, -4.0, 7.0], [-6.0, -3.0, 1.0], [4.0, -7.0, 1.0], [7.0, 5.0, 2.0]], [[2.0, 0.0, 0.0], [5.0, -2.0, -4.0], [-4.0, 0.0, -5.0], [1.0, -2.0, 1.0]]])
      arg1=numpy.array([[[[0.0, -2.0], [-1.0, 0.0], [3.0, -1.0]], [[-6.0, 5.0], [-4.0, -1.0], [-7.0, -4.0]], [[0.0, -2.0], [-4.0, 3.0], [2.0, -6.0]], [[3.0, -5.0], [6.0, 0.0], [-5.0, 0.0]]], [[[5.0, 3.0], [0.0, -2.0], [-6.0, -5.0]], [[-5.0, 0.0], [-1.0, 1.0], [-7.0, -3.0]], [[0.0, -4.0], [1.0, 2.0], [-6.0, -5.0]], [[-1.0, 3.0], [6.0, 0.0], [6.0, -3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([175.0, -41.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_array_rank2_offset0(self):
      arg0=numpy.array(1.0)
      arg1=numpy.array([[0.0, -7.0, 0.0, -2.0, -2.0], [7.0, 1.0, 2.0, -4.0, 3.0], [-7.0, 4.0, 4.0, 5.0, -6.0], [-3.0, 7.0, 4.0, 3.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[0.0, -7.0, 0.0, -2.0, -2.0], [7.0, 1.0, 2.0, -4.0, 3.0], [-7.0, 4.0, 4.0, 5.0, -6.0], [-3.0, 7.0, 4.0, 3.0, -4.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank3_offset1(self):
      arg0=numpy.array([0.0, 2.0, 4.0])
      arg1=numpy.array([[[7.0, -5.0, -6.0, 6.0, 0.0], [3.0, 3.0, -6.0, 4.0, 1.0], [-7.0, 3.0, 7.0, 3.0, 5.0], [5.0, -2.0, 6.0, -1.0, 4.0]], [[-7.0, 0.0, 5.0, 7.0, 5.0], [0.0, 0.0, 5.0, -7.0, 2.0], [1.0, -4.0, 5.0, 5.0, 3.0], [-2.0, 0.0, 7.0, -7.0, 0.0]], [[-2.0, 2.0, 2.0, 4.0, 3.0], [-3.0, 4.0, 0.0, 3.0, -7.0], [5.0, -4.0, -6.0, -6.0, -2.0], [-6.0, 1.0, -1.0, -1.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[-22.0, 8.0, 18.0, 30.0, 22.0], [-12.0, 16.0, 10.0, -2.0, -24.0], [22.0, -24.0, -14.0, -14.0, -2.0], [-28.0, 4.0, 10.0, -18.0, 0.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank4_offset2(self):
      arg0=numpy.array([[2.0, -6.0, -3.0], [-5.0, -5.0, -7.0]])
      arg1=numpy.array([[[[-5.0, -2.0, 2.0, -2.0, 6.0], [1.0, -6.0, -7.0, -1.0, -5.0], [5.0, 0.0, 2.0, 5.0, 2.0], [-1.0, -5.0, 7.0, 0.0, 3.0]], [[7.0, 4.0, 6.0, -6.0, 0.0], [5.0, -5.0, 3.0, 4.0, -2.0], [7.0, -4.0, 4.0, 5.0, 7.0], [-7.0, 0.0, 7.0, 0.0, 4.0]], [[5.0, 6.0, 6.0, 4.0, 6.0], [5.0, -4.0, 0.0, -6.0, 2.0], [-5.0, -1.0, -2.0, -1.0, 1.0], [0.0, 6.0, -3.0, -1.0, 5.0]]], [[[3.0, -2.0, 6.0, -4.0, -4.0], [3.0, -2.0, 4.0, -6.0, -1.0], [-5.0, -6.0, 0.0, -6.0, -2.0], [-5.0, -4.0, 7.0, -6.0, -2.0]], [[-7.0, 1.0, 7.0, 4.0, 3.0], [1.0, -1.0, -5.0, 5.0, -7.0], [-2.0, 6.0, 0.0, -6.0, 6.0], [0.0, -2.0, -4.0, 3.0, 2.0]], [[-4.0, 7.0, 6.0, -6.0, 1.0], [0.0, 3.0, -3.0, 3.0, -4.0], [-2.0, 0.0, -3.0, -5.0, -7.0], [6.0, -7.0, 0.0, 7.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[-19.0, -90.0, -157.0, 62.0, -8.0], [-63.0, 24.0, -6.0, -24.0, 64.0], [32.0, 27.0, 7.0, 78.0, -12.0], [23.0, 51.0, -34.0, -31.0, -26.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_array_rank3_offset0(self):
      arg0=numpy.array(6.0)
      arg1=numpy.array([[[-2.0, 6.0], [-7.0, 5.0]], [[5.0, -4.0], [-5.0, 1.0]], [[-5.0, 4.0], [1.0, -1.0]], [[4.0, -1.0], [-4.0, 4.0]], [[2.0, 1.0], [6.0, -2.0]], [[3.0, 6.0], [3.0, -6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-12.0, 36.0], [-42.0, 30.0]], [[30.0, -24.0], [-30.0, 6.0]], [[-30.0, 24.0], [6.0, -6.0]], [[24.0, -6.0], [-24.0, 24.0]], [[12.0, 6.0], [36.0, -12.0]], [[18.0, 36.0], [18.0, -36.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank4_offset1(self):
      arg0=numpy.array([-3.0, 5.0, 0.0])
      arg1=numpy.array([[[[-7.0, 0.0], [-7.0, 2.0]], [[7.0, 4.0], [0.0, 6.0]], [[-7.0, 6.0], [-6.0, -4.0]], [[-4.0, 3.0], [-2.0, -1.0]], [[7.0, 0.0], [-4.0, 1.0]], [[6.0, -5.0], [-1.0, 4.0]]], [[[-6.0, 6.0], [-2.0, -2.0]], [[-1.0, 0.0], [5.0, 6.0]], [[2.0, 0.0], [-6.0, 0.0]], [[-5.0, 7.0], [0.0, 0.0]], [[3.0, 2.0], [3.0, 1.0]], [[6.0, 4.0], [-1.0, 0.0]]], [[[-6.0, 5.0], [2.0, 4.0]], [[-7.0, 7.0], [6.0, -7.0]], [[-7.0, -4.0], [-5.0, 3.0]], [[1.0, 2.0], [3.0, 0.0]], [[0.0, 6.0], [-7.0, 3.0]], [[-3.0, 6.0], [-5.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-9.0, 30.0], [11.0, -16.0]], [[-26.0, -12.0], [25.0, 12.0]], [[31.0, -18.0], [-12.0, 12.0]], [[-13.0, 26.0], [6.0, 3.0]], [[-6.0, 10.0], [27.0, 2.0]], [[12.0, 35.0], [-2.0, -12.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_array_rank4_offset0(self):
      arg0=numpy.array(6.0)
      arg1=numpy.array([[[[-3.0, 5.0, 5.0, 0.0], [-3.0, -2.0, 4.0, 4.0], [-1.0, 1.0, 6.0, 3.0]], [[-6.0, -5.0, -5.0, -3.0], [5.0, 3.0, -2.0, 7.0], [0.0, 6.0, -5.0, -1.0]]], [[[2.0, -4.0, 7.0, 7.0], [3.0, -5.0, -7.0, -7.0], [0.0, 5.0, -1.0, 6.0]], [[4.0, 4.0, -3.0, 0.0], [0.0, -4.0, -4.0, -4.0], [-3.0, -4.0, 5.0, 1.0]]], [[[2.0, -3.0, -7.0, -5.0], [-7.0, 3.0, 5.0, 3.0], [-3.0, -2.0, 1.0, 7.0]], [[1.0, 2.0, -1.0, 0.0], [0.0, 5.0, 4.0, 3.0], [6.0, -3.0, -5.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-18.0, 30.0, 30.0, 0.0], [-18.0, -12.0, 24.0, 24.0], [-6.0, 6.0, 36.0, 18.0]], [[-36.0, -30.0, -30.0, -18.0], [30.0, 18.0, -12.0, 42.0], [0.0, 36.0, -30.0, -6.0]]], [[[12.0, -24.0, 42.0, 42.0], [18.0, -30.0, -42.0, -42.0], [0.0, 30.0, -6.0, 36.0]], [[24.0, 24.0, -18.0, 0.0], [0.0, -24.0, -24.0, -24.0], [-18.0, -24.0, 30.0, 6.0]]], [[[12.0, -18.0, -42.0, -30.0], [-42.0, 18.0, 30.0, 18.0], [-18.0, -12.0, 6.0, 42.0]], [[6.0, 12.0, -6.0, 0.0], [0.0, 30.0, 24.0, 18.0], [36.0, -18.0, -30.0, 24.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_Symbol_rank0_offset0(self):
      arg0=numpy.array(-1.0)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(4.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-4.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank1_offset1(self):
      arg0=numpy.array([-2.0, 1.0, 1.0])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-3.0, -2.0, -1.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(3.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank2_offset2(self):
      arg0=numpy.array([[5.0, -5.0, 0.0], [-3.0, 7.0, -3.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[0.0, 2.0, 7.0], [7.0, 0.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-31.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[6.0, -1.0, 5.0], [6.0, -7.0, -2.0], [-2.0, 7.0, -1.0], [0.0, -6.0, -5.0]], [[-7.0, 6.0, 3.0], [-7.0, 3.0, 1.0], [3.0, 0.0, 6.0], [0.0, 5.0, -6.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-2.0, 0.0, 5.0], [-5.0, 0.0, 7.0], [3.0, 7.0, 2.0], [-7.0, 0.0, 5.0]], [[-7.0, 2.0, -2.0], [1.0, 3.0, 7.0], [3.0, -3.0, 2.0], [5.0, 0.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(100.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank4_offset4(self):
      arg0=numpy.array([[[[7.0, -2.0], [3.0, 5.0], [1.0, 4.0]], [[0.0, 4.0], [-5.0, 4.0], [-6.0, 5.0]]], [[[5.0, -2.0], [-4.0, 3.0], [-7.0, 5.0]], [[-2.0, -4.0], [-4.0, 5.0], [-5.0, 4.0]]], [[[7.0, 0.0], [4.0, -2.0], [3.0, 6.0]], [[6.0, 0.0], [-5.0, 2.0], [7.0, -6.0]]], [[[-3.0, 7.0], [0.0, -5.0], [0.0, -3.0]], [[-2.0, -6.0], [7.0, -3.0], [-1.0, -5.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[3.0, 1.0], [-6.0, -5.0], [6.0, 3.0]], [[-4.0, -5.0], [-2.0, -7.0], [0.0, 4.0]]], [[[7.0, -1.0], [-4.0, -7.0], [-6.0, 0.0]], [[4.0, -7.0], [-3.0, 0.0], [3.0, 5.0]]], [[[-6.0, -2.0], [-3.0, 3.0], [4.0, 3.0]], [[-3.0, -5.0], [-1.0, 4.0], [4.0, 7.0]]], [[[-3.0, -2.0], [-7.0, 1.0], [0.0, -5.0]], [[-4.0, 3.0], [7.0, 3.0], [-2.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(60.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_Symbol_rank1_offset0(self):
      arg0=numpy.array(-7.0)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([1.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-7.0, 49.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank2_offset1(self):
      arg0=numpy.array([7.0, -7.0, 7.0])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-6.0, 2.0], [6.0, -7.0], [-7.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-133.0, 49.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank3_offset2(self):
      arg0=numpy.array([[-2.0, 0.0, 2.0], [7.0, 0.0, 0.0]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[7.0, -1.0], [5.0, 2.0], [-1.0, -3.0]], [[1.0, 6.0], [-6.0, -1.0], [-7.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-9.0, 38.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[-6.0, -4.0, -3.0], [1.0, 5.0, 7.0], [3.0, 7.0, 3.0], [-4.0, -2.0, -3.0]], [[0.0, -3.0, 7.0], [7.0, -7.0, -4.0], [5.0, -6.0, -7.0], [7.0, 1.0, -2.0]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-6.0, 0.0], [-2.0, 4.0], [2.0, 5.0]], [[-6.0, -1.0], [3.0, 0.0], [1.0, 4.0]], [[2.0, 6.0], [6.0, -5.0], [0.0, -4.0]], [[3.0, -7.0], [0.0, 0.0], [6.0, -2.0]]], [[[-4.0, 0.0], [-6.0, -1.0], [-6.0, -7.0]], [[-1.0, -6.0], [-3.0, 0.0], [-1.0, 6.0]], [[4.0, -2.0], [3.0, 4.0], [0.0, 7.0]], [[-5.0, -2.0], [-6.0, 0.0], [-2.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([31.0, -216.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_Symbol_rank2_offset0(self):
      arg0=numpy.array(-6.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[0.0, 6.0, 4.0, 2.0, 4.0], [-7.0, -3.0, -2.0, 7.0, 5.0], [5.0, 5.0, 1.0, -7.0, 0.0], [4.0, -6.0, -1.0, 3.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-0.0, -36.0, -24.0, -12.0, -24.0], [42.0, 18.0, 12.0, -42.0, -30.0], [-30.0, -30.0, -6.0, 42.0, -0.0], [-24.0, 36.0, 6.0, -18.0, -42.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank3_offset1(self):
      arg0=numpy.array([6.0, -6.0, -3.0])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[1.0, -3.0, -1.0, 7.0, -7.0], [6.0, -5.0, -6.0, -4.0, -7.0], [5.0, 3.0, -3.0, -6.0, 4.0], [-2.0, -6.0, -5.0, 0.0, 3.0]], [[0.0, -4.0, -1.0, 0.0, 4.0], [0.0, -7.0, -5.0, 7.0, 5.0], [6.0, 6.0, 0.0, -6.0, -1.0], [-2.0, 7.0, 0.0, -3.0, 4.0]], [[-2.0, 1.0, 4.0, -3.0, 2.0], [0.0, 3.0, 0.0, 1.0, 2.0], [7.0, 7.0, 4.0, -6.0, 0.0], [-6.0, -2.0, 7.0, -7.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[12.0, 3.0, -12.0, 51.0, -72.0], [36.0, 3.0, -6.0, -69.0, -78.0], [-27.0, -39.0, -30.0, 18.0, 30.0], [18.0, -72.0, -51.0, 39.0, -27.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank4_offset2(self):
      arg0=numpy.array([[0.0, 4.0, -2.0], [3.0, 3.0, -1.0]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[2.0, 0.0, -6.0, -5.0, -1.0], [1.0, 2.0, 1.0, -1.0, -2.0], [-5.0, -6.0, 6.0, 0.0, 4.0], [5.0, 0.0, -3.0, 7.0, 5.0]], [[-1.0, 0.0, -7.0, 5.0, 2.0], [-5.0, -3.0, 0.0, 0.0, -7.0], [5.0, 2.0, -5.0, 5.0, -7.0], [3.0, -6.0, 4.0, -6.0, 0.0]], [[6.0, 6.0, -3.0, -3.0, 1.0], [6.0, 0.0, -5.0, -2.0, -4.0], [-4.0, 5.0, -6.0, 2.0, -3.0], [3.0, -7.0, 5.0, 6.0, -7.0]]], [[[5.0, 3.0, -1.0, 4.0, 1.0], [-3.0, -1.0, -2.0, 5.0, 1.0], [2.0, 2.0, 3.0, 3.0, 2.0], [-3.0, 7.0, 0.0, 1.0, -1.0]], [[5.0, 4.0, -2.0, -2.0, 3.0], [-2.0, -5.0, 1.0, 5.0, 0.0], [0.0, 4.0, 1.0, -2.0, 7.0], [-3.0, -3.0, 0.0, 6.0, 2.0]], [[-5.0, -3.0, 1.0, 2.0, 2.0], [1.0, 0.0, -6.0, -4.0, -5.0], [2.0, 0.0, 0.0, 3.0, -3.0], [-1.0, 3.0, 3.0, -3.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[19.0, 12.0, -32.0, 30.0, 16.0], [-48.0, -30.0, 13.0, 38.0, -12.0], [32.0, 16.0, 4.0, 16.0, 8.0], [-11.0, -1.0, 3.0, -12.0, 11.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_Symbol_rank3_offset0(self):
      arg0=numpy.array(7.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[5.0, 7.0], [0.0, 6.0]], [[-2.0, 1.0], [3.0, -3.0]], [[0.0, 7.0], [7.0, -1.0]], [[5.0, -2.0], [3.0, 0.0]], [[-5.0, 2.0], [5.0, -1.0]], [[1.0, -6.0], [7.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[35.0, 49.0], [0.0, 42.0]], [[-14.0, 7.0], [21.0, -21.0]], [[0.0, 49.0], [49.0, -7.0]], [[35.0, -14.0], [21.0, 0.0]], [[-35.0, 14.0], [35.0, -7.0]], [[7.0, -42.0], [49.0, 35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank4_offset1(self):
      arg0=numpy.array([-3.0, -6.0, 4.0])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[-4.0, -3.0], [1.0, 5.0]], [[4.0, 4.0], [-2.0, -6.0]], [[-1.0, 4.0], [5.0, 5.0]], [[-3.0, 7.0], [0.0, -5.0]], [[-1.0, 0.0], [4.0, 7.0]], [[-1.0, 0.0], [-1.0, 1.0]]], [[[0.0, 5.0], [7.0, 0.0]], [[-7.0, 0.0], [4.0, 0.0]], [[-3.0, -2.0], [5.0, 3.0]], [[3.0, -2.0], [1.0, -6.0]], [[6.0, 0.0], [-2.0, -2.0]], [[4.0, -3.0], [-6.0, 1.0]]], [[[-7.0, 0.0], [-7.0, 0.0]], [[3.0, -2.0], [3.0, 0.0]], [[-7.0, 4.0], [-4.0, 7.0]], [[-2.0, 4.0], [-5.0, 7.0]], [[-1.0, 1.0], [7.0, -7.0]], [[2.0, -5.0], [4.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-16.0, -21.0], [-73.0, -15.0]], [[42.0, -20.0], [-6.0, 18.0]], [[-7.0, 16.0], [-61.0, -5.0]], [[-17.0, 7.0], [-26.0, 79.0]], [[-37.0, 4.0], [28.0, -37.0]], [[-13.0, -2.0], [55.0, 7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_Symbol_rank4_offset0(self):
      arg0=numpy.array(0.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[5.0, -5.0, 2.0, 1.0], [-6.0, 4.0, -3.0, -7.0], [-1.0, 0.0, -1.0, 4.0]], [[0.0, 1.0, -2.0, 6.0], [4.0, 4.0, -7.0, 3.0], [4.0, 0.0, 1.0, -3.0]]], [[[6.0, 7.0, 3.0, -1.0], [-3.0, 5.0, -4.0, 2.0], [6.0, -1.0, -3.0, -7.0]], [[7.0, 0.0, 2.0, 0.0], [-2.0, 1.0, -5.0, 5.0], [-3.0, -4.0, -1.0, -4.0]]], [[[-4.0, 6.0, 5.0, -1.0], [2.0, -2.0, 6.0, 0.0], [2.0, -1.0, 0.0, -6.0]], [[-4.0, -7.0, -6.0, -5.0], [1.0, -3.0, -7.0, 7.0], [-3.0, -5.0, 4.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0]], [[0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]]], [[[0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, -0.0, -0.0]], [[0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, -0.0]]], [[[-0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0], [0.0, -0.0, 0.0, -0.0]], [[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_constData_rank0_offset0(self):
      arg0=numpy.array(3.0)
      arg1=Data(-6.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-18.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank1_offset1(self):
      arg0=numpy.array([-4.0, -7.0, -5.0])
      arg1=Data(numpy.array([-1.0, -3.0, 4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(5.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank2_offset2(self):
      arg0=numpy.array([[2.0, 5.0, 3.0], [-3.0, -2.0, 0.0]])
      arg1=Data(numpy.array([[3.0, 5.0, -1.0], [3.0, 7.0, 3.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(5.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank3_offset3(self):
      arg0=numpy.array([[[5.0, -7.0, 6.0], [4.0, -4.0, -2.0], [-2.0, 3.0, -7.0], [1.0, -7.0, 7.0]], [[6.0, -2.0, -6.0], [6.0, -4.0, -1.0], [5.0, 2.0, -6.0], [1.0, -2.0, 2.0]]])
      arg1=Data(numpy.array([[[5.0, -6.0, 4.0], [-4.0, -4.0, -7.0], [-3.0, -6.0, -6.0], [0.0, 4.0, 6.0]], [[-1.0, 1.0, 1.0], [3.0, 2.0, -7.0], [-1.0, -6.0, -2.0], [-4.0, -2.0, 1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(149.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank4_offset4(self):
      arg0=numpy.array([[[[-7.0, 6.0], [4.0, -1.0], [-6.0, 3.0]], [[1.0, -7.0], [-1.0, 3.0], [-7.0, -1.0]]], [[[4.0, -6.0], [4.0, 6.0], [6.0, 1.0]], [[3.0, -4.0], [-3.0, 6.0], [0.0, -7.0]]], [[[0.0, 7.0], [1.0, -4.0], [-6.0, -6.0]], [[-6.0, -1.0], [6.0, 0.0], [-7.0, -6.0]]], [[[-7.0, -3.0], [-5.0, 0.0], [0.0, 3.0]], [[3.0, 0.0], [0.0, -1.0], [0.0, -4.0]]]])
      arg1=Data(numpy.array([[[[-4.0, 4.0], [-3.0, 3.0], [5.0, 1.0]], [[-3.0, 0.0], [0.0, 5.0], [-7.0, -7.0]]], [[[5.0, -6.0], [-6.0, 1.0], [-3.0, -7.0]], [[-6.0, 3.0], [5.0, -4.0], [-4.0, -3.0]]], [[[4.0, 5.0], [7.0, -5.0], [7.0, 5.0]], [[0.0, -2.0], [-1.0, 4.0], [-2.0, -5.0]]], [[[2.0, 3.0], [6.0, 0.0], [-7.0, 2.0]], [[0.0, -4.0], [5.0, 1.0], [-6.0, 7.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(-3.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_constData_rank1_offset0(self):
      arg0=numpy.array(-5.0)
      arg1=Data(numpy.array([-2.0, 2.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([10.0, -10.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank2_offset1(self):
      arg0=numpy.array([4.0, 3.0, 1.0])
      arg1=Data(numpy.array([[-1.0, 0.0], [-2.0, 1.0], [0.0, 1.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-10.0, 4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank3_offset2(self):
      arg0=numpy.array([[-3.0, 1.0, 3.0], [-3.0, 4.0, 7.0]])
      arg1=Data(numpy.array([[[1.0, 7.0], [-6.0, -3.0], [2.0, 2.0]], [[-4.0, -2.0], [0.0, 0.0], [-6.0, 5.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-33.0, 23.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank4_offset3(self):
      arg0=numpy.array([[[6.0, 5.0, 1.0], [-2.0, 5.0, 1.0], [0.0, -4.0, -6.0], [-6.0, 2.0, -1.0]], [[-7.0, -5.0, 0.0], [6.0, 6.0, -5.0], [-7.0, -2.0, 7.0], [-2.0, 7.0, 1.0]]])
      arg1=Data(numpy.array([[[[7.0, 5.0], [-2.0, 6.0], [6.0, -3.0]], [[0.0, 1.0], [-1.0, 0.0], [-2.0, -4.0]], [[-4.0, -4.0], [-5.0, -2.0], [-7.0, 0.0]], [[1.0, -7.0], [1.0, -6.0], [7.0, -3.0]]], [[[-3.0, -1.0], [-5.0, -1.0], [-5.0, -1.0]], [[-2.0, 0.0], [-7.0, 0.0], [5.0, -5.0]], [[-2.0, -7.0], [2.0, -6.0], [7.0, -4.0]], [[-1.0, -5.0], [3.0, -1.0], [-6.0, 7.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([125.0, 172.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_constData_rank2_offset0(self):
      arg0=numpy.array(3.0)
      arg1=Data(numpy.array([[6.0, -3.0, -5.0, -3.0, 0.0], [1.0, 6.0, 6.0, 3.0, 0.0], [-6.0, 3.0, 7.0, -5.0, 1.0], [6.0, -1.0, 2.0, 0.0, -2.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[18.0, -9.0, -15.0, -9.0, 0.0], [3.0, 18.0, 18.0, 9.0, 0.0], [-18.0, 9.0, 21.0, -15.0, 3.0], [18.0, -3.0, 6.0, 0.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank3_offset1(self):
      arg0=numpy.array([-6.0, 0.0, -1.0])
      arg1=Data(numpy.array([[[-7.0, 6.0, -2.0, -4.0, -7.0], [1.0, 1.0, -1.0, -7.0, 7.0], [5.0, -4.0, -4.0, 0.0, -3.0], [5.0, -2.0, -4.0, 0.0, -2.0]], [[2.0, -7.0, 0.0, 3.0, -1.0], [1.0, -7.0, -4.0, 6.0, -2.0], [-4.0, 7.0, 5.0, 1.0, 0.0], [7.0, -1.0, -5.0, 7.0, 6.0]], [[-3.0, -5.0, 4.0, -1.0, 6.0], [5.0, -3.0, -7.0, -1.0, -4.0], [7.0, 2.0, 7.0, 3.0, -5.0], [5.0, -6.0, 6.0, 5.0, -3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[45.0, -31.0, 8.0, 25.0, 36.0], [-11.0, -3.0, 13.0, 43.0, -38.0], [-37.0, 22.0, 17.0, -3.0, 23.0], [-35.0, 18.0, 18.0, -5.0, 15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank4_offset2(self):
      arg0=numpy.array([[-2.0, 0.0, 0.0], [6.0, 3.0, 0.0]])
      arg1=Data(numpy.array([[[[1.0, -5.0, -1.0, -5.0, -4.0], [0.0, -4.0, -2.0, 7.0, 2.0], [4.0, 5.0, 0.0, 2.0, -4.0], [1.0, -2.0, 1.0, -5.0, 0.0]], [[-1.0, -1.0, 4.0, 6.0, -1.0], [-7.0, 4.0, 2.0, -4.0, 1.0], [-3.0, 1.0, -3.0, -1.0, -6.0], [1.0, -4.0, 4.0, 0.0, 0.0]], [[0.0, 1.0, 1.0, -7.0, 6.0], [-7.0, -1.0, 3.0, 3.0, 3.0], [3.0, -6.0, 0.0, 6.0, 6.0], [7.0, 0.0, 2.0, 7.0, 6.0]]], [[[-3.0, -2.0, -5.0, -7.0, 0.0], [0.0, -3.0, 1.0, -1.0, 0.0], [-3.0, -5.0, 2.0, 4.0, -1.0], [-7.0, 3.0, 5.0, -1.0, -5.0]], [[-2.0, 0.0, 4.0, -3.0, 0.0], [-3.0, -6.0, 0.0, -7.0, -3.0], [4.0, -6.0, 4.0, 4.0, 2.0], [-5.0, 4.0, -4.0, -1.0, -6.0]], [[0.0, 1.0, -5.0, -7.0, 4.0], [3.0, -3.0, -2.0, -4.0, 2.0], [-1.0, -3.0, 4.0, -3.0, 2.0], [2.0, 1.0, 3.0, 0.0, -2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-26.0, -2.0, -16.0, -41.0, 8.0], [-9.0, -28.0, 10.0, -41.0, -13.0], [-14.0, -58.0, 24.0, 32.0, 8.0], [-59.0, 34.0, 16.0, 1.0, -48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_constData_rank3_offset0(self):
      arg0=numpy.array(0.0)
      arg1=Data(numpy.array([[[0.0, -1.0], [-3.0, -1.0]], [[5.0, 2.0], [-4.0, 6.0]], [[-5.0, 4.0], [0.0, 7.0]], [[-2.0, -2.0], [5.0, 6.0]], [[0.0, 5.0], [1.0, -6.0]], [[-1.0, -3.0], [4.0, 7.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, -0.0], [-0.0, -0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[-0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, -0.0]], [[-0.0, -0.0], [0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank4_offset1(self):
      arg0=numpy.array([-2.0, 0.0, 1.0])
      arg1=Data(numpy.array([[[[0.0, 6.0], [5.0, 2.0]], [[-2.0, 2.0], [3.0, 0.0]], [[-2.0, 1.0], [4.0, -6.0]], [[-5.0, 5.0], [-6.0, 4.0]], [[-2.0, -5.0], [5.0, 2.0]], [[-2.0, 2.0], [-7.0, 0.0]]], [[[-1.0, -4.0], [-4.0, -2.0]], [[3.0, -6.0], [-7.0, 4.0]], [[-6.0, -3.0], [-6.0, -3.0]], [[5.0, -2.0], [4.0, -5.0]], [[4.0, -6.0], [7.0, 6.0]], [[-7.0, -4.0], [-4.0, 5.0]]], [[[3.0, -2.0], [-3.0, 0.0]], [[4.0, 4.0], [-4.0, 1.0]], [[-4.0, -3.0], [0.0, 2.0]], [[6.0, -3.0], [0.0, -6.0]], [[-1.0, 2.0], [5.0, -3.0]], [[6.0, 7.0], [6.0, 4.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[3.0, -14.0], [-13.0, -4.0]], [[8.0, 0.0], [-10.0, 1.0]], [[0.0, -5.0], [-8.0, 14.0]], [[16.0, -13.0], [12.0, -14.0]], [[3.0, 12.0], [-5.0, -7.0]], [[10.0, 3.0], [20.0, 4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_constData_rank4_offset0(self):
      arg0=numpy.array(5.0)
      arg1=Data(numpy.array([[[[0.0, -3.0, -4.0, -5.0], [7.0, 4.0, 4.0, -2.0], [-6.0, 1.0, 6.0, 3.0]], [[2.0, 0.0, 6.0, 7.0], [3.0, 7.0, -1.0, -2.0], [6.0, 5.0, 3.0, 5.0]]], [[[-2.0, 1.0, 4.0, 1.0], [6.0, -6.0, -1.0, -4.0], [1.0, 3.0, 7.0, 7.0]], [[7.0, -6.0, 0.0, 5.0], [-1.0, -7.0, 3.0, -3.0], [-1.0, 1.0, 2.0, -3.0]]], [[[3.0, -4.0, 6.0, -2.0], [4.0, -2.0, 5.0, 6.0], [4.0, -3.0, -2.0, 1.0]], [[1.0, 0.0, -2.0, -4.0], [0.0, -7.0, -6.0, 5.0], [7.0, 7.0, -2.0, 2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, -15.0, -20.0, -25.0], [35.0, 20.0, 20.0, -10.0], [-30.0, 5.0, 30.0, 15.0]], [[10.0, 0.0, 30.0, 35.0], [15.0, 35.0, -5.0, -10.0], [30.0, 25.0, 15.0, 25.0]]], [[[-10.0, 5.0, 20.0, 5.0], [30.0, -30.0, -5.0, -20.0], [5.0, 15.0, 35.0, 35.0]], [[35.0, -30.0, 0.0, 25.0], [-5.0, -35.0, 15.0, -15.0], [-5.0, 5.0, 10.0, -15.0]]], [[[15.0, -20.0, 30.0, -10.0], [20.0, -10.0, 25.0, 30.0], [20.0, -15.0, -10.0, 5.0]], [[5.0, 0.0, -10.0, -20.0], [0.0, -35.0, -30.0, 25.0], [35.0, 35.0, -10.0, 10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_expandedData_rank0_offset0(self):
      arg0=numpy.array(5.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(6.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(5.0)+(1.-msk_ref)*numpy.array(30.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank1_offset1(self):
      arg0=numpy.array([1.0, -7.0, 5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, -2.0, -6.0])+(1.-msk_arg1)*numpy.array([-3.0, -1.0, -2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-12.0)+(1.-msk_ref)*numpy.array(-6.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank2_offset2(self):
      arg0=numpy.array([[-6.0, 0.0, 0.0], [7.0, -3.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 2.0, 6.0], [7.0, 1.0, 2.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 7.0, -6.0], [-7.0, -2.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(98.0)+(1.-msk_ref)*numpy.array(-7.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[-5.0, 1.0, -7.0], [5.0, 1.0, -5.0], [7.0, -5.0, -4.0], [-7.0, 0.0, -7.0]], [[2.0, -1.0, 7.0], [2.0, -6.0, -2.0], [-3.0, -7.0, 4.0], [-6.0, 0.0, 6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, 2.0, -2.0], [-5.0, 4.0, 7.0], [1.0, -1.0, 1.0], [2.0, 6.0, -6.0]], [[-7.0, -5.0, -4.0], [0.0, -6.0, 3.0], [-5.0, 3.0, -4.0], [7.0, 3.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 6.0, -1.0], [7.0, 1.0, -5.0], [7.0, -1.0, 3.0], [2.0, -6.0, -2.0]], [[5.0, -1.0, -3.0], [-5.0, -3.0, 1.0], [5.0, 4.0, 2.0], [-7.0, 1.0, -2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-81.0)+(1.-msk_ref)*numpy.array(117.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank4_offset4(self):
      arg0=numpy.array([[[[7.0, -1.0], [-7.0, 3.0], [5.0, 4.0]], [[-4.0, 6.0], [-2.0, -4.0], [1.0, -1.0]]], [[[1.0, 0.0], [3.0, -3.0], [0.0, 6.0]], [[6.0, -7.0], [-1.0, 7.0], [7.0, 2.0]]], [[[0.0, 4.0], [-4.0, 5.0], [5.0, -3.0]], [[2.0, -3.0], [0.0, 5.0], [4.0, 4.0]]], [[[-7.0, 7.0], [-5.0, -1.0], [2.0, -6.0]], [[-3.0, -5.0], [-6.0, 3.0], [6.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, 5.0], [-7.0, 0.0], [-5.0, 4.0]], [[6.0, 3.0], [6.0, 4.0], [-3.0, -6.0]]], [[[-4.0, 3.0], [1.0, 1.0], [-2.0, -5.0]], [[5.0, 2.0], [0.0, 6.0], [3.0, -1.0]]], [[[-4.0, -1.0], [7.0, -6.0], [7.0, -1.0]], [[-1.0, 0.0], [-6.0, -3.0], [-6.0, -4.0]]], [[[0.0, 0.0], [-5.0, 5.0], [-3.0, 7.0]], [[6.0, 1.0], [-6.0, -3.0], [-2.0, -2.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -3.0], [3.0, -1.0], [-4.0, -7.0]], [[7.0, -7.0], [2.0, 6.0], [-7.0, 6.0]]], [[[6.0, 0.0], [-1.0, -7.0], [-3.0, -7.0]], [[-7.0, 0.0], [4.0, -4.0], [-6.0, -1.0]]], [[[-6.0, 0.0], [-7.0, 7.0], [-7.0, -6.0]], [[5.0, -5.0], [-7.0, 1.0], [3.0, 0.0]]], [[[-2.0, -5.0], [0.0, 2.0], [7.0, 6.0]], [[4.0, 2.0], [1.0, 3.0], [7.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-81.0)+(1.-msk_ref)*numpy.array(-197.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_expandedData_rank1_offset0(self):
      arg0=numpy.array(-3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 7.0])+(1.-msk_arg1)*numpy.array([0.0, -2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, -21.0])+(1.-msk_ref)*numpy.array([-0.0, 6.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank2_offset1(self):
      arg0=numpy.array([-6.0, -6.0, -4.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 5.0], [7.0, -4.0], [-1.0, 0.0]])+(1.-msk_arg1)*numpy.array([[1.0, 6.0], [6.0, -1.0], [1.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-14.0, -6.0])+(1.-msk_ref)*numpy.array([-46.0, -50.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank3_offset2(self):
      arg0=numpy.array([[-1.0, -3.0, -5.0], [5.0, -7.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, 7.0], [3.0, 0.0], [-2.0, 3.0]], [[-7.0, 0.0], [6.0, -5.0], [3.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, -4.0], [0.0, 6.0], [-6.0, -3.0]], [[2.0, -7.0], [-4.0, 3.0], [-4.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-58.0, 28.0])+(1.-msk_ref)*numpy.array([42.0, -80.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[-5.0, 3.0, 6.0], [-2.0, 7.0, -4.0], [0.0, 4.0, 5.0], [0.0, -7.0, -5.0]], [[0.0, -6.0, 3.0], [-1.0, -3.0, 5.0], [-6.0, 0.0, -2.0], [-6.0, -7.0, 6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -4.0], [-5.0, 6.0], [2.0, 6.0]], [[-6.0, 0.0], [1.0, 6.0], [0.0, -5.0]], [[-2.0, -4.0], [-1.0, -4.0], [-7.0, -6.0]], [[2.0, 2.0], [-3.0, 5.0], [-7.0, -1.0]]], [[[6.0, 6.0], [-2.0, -6.0], [4.0, -5.0]], [[2.0, 5.0], [7.0, -5.0], [0.0, -3.0]], [[2.0, 1.0], [7.0, 4.0], [1.0, 4.0]], [[-2.0, -6.0], [3.0, -1.0], [4.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, 2.0], [-1.0, -5.0], [3.0, -3.0]], [[6.0, 5.0], [-6.0, 0.0], [6.0, 2.0]], [[3.0, -1.0], [6.0, 4.0], [4.0, 0.0]], [[-7.0, 4.0], [-2.0, -1.0], [-4.0, -2.0]]], [[[-3.0, -6.0], [-3.0, -6.0], [1.0, 4.0]], [[-3.0, -7.0], [-5.0, -5.0], [2.0, 4.0]], [[7.0, -4.0], [-6.0, -4.0], [-2.0, -7.0]], [[-4.0, 7.0], [-6.0, 2.0], [5.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([35.0, 123.0])+(1.-msk_ref)*numpy.array([97.0, 50.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_expandedData_rank2_offset0(self):
      arg0=numpy.array(-6.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, -3.0, -5.0, 4.0, 3.0], [-3.0, -7.0, 1.0, 0.0, 4.0], [-1.0, 5.0, -3.0, 5.0, 3.0], [-1.0, -3.0, -1.0, 0.0, -7.0]])+(1.-msk_arg1)*numpy.array([[0.0, 4.0, 5.0, -5.0, -6.0], [4.0, 2.0, 0.0, 1.0, 0.0], [5.0, 1.0, 0.0, -6.0, 5.0], [1.0, 0.0, -3.0, -1.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-18.0, 18.0, 30.0, -24.0, -18.0], [18.0, 42.0, -6.0, -0.0, -24.0], [6.0, -30.0, 18.0, -30.0, -18.0], [6.0, 18.0, 6.0, -0.0, 42.0]])+(1.-msk_ref)*numpy.array([[-0.0, -24.0, -30.0, 30.0, 36.0], [-24.0, -12.0, -0.0, -6.0, -0.0], [-30.0, -6.0, -0.0, 36.0, -30.0], [-6.0, -0.0, 18.0, 6.0, -18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank3_offset1(self):
      arg0=numpy.array([6.0, 7.0, 7.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, -5.0, -7.0, -5.0, 5.0], [2.0, 2.0, 0.0, 0.0, -3.0], [-2.0, 4.0, -4.0, 4.0, -6.0], [-3.0, 0.0, -5.0, -6.0, 4.0]], [[-2.0, 7.0, 6.0, 5.0, 5.0], [-6.0, 0.0, -3.0, 2.0, 5.0], [1.0, 1.0, -2.0, 6.0, 7.0], [-1.0, -6.0, -4.0, 5.0, -2.0]], [[-5.0, -5.0, 5.0, -2.0, 4.0], [0.0, -4.0, -3.0, 0.0, 4.0], [0.0, -6.0, 5.0, 3.0, 4.0], [-5.0, -4.0, -4.0, 1.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 7.0, 7.0, -3.0, 6.0], [3.0, 1.0, 5.0, -5.0, 0.0], [6.0, -5.0, -7.0, 1.0, 4.0], [-1.0, 0.0, 3.0, 5.0, 0.0]], [[3.0, 7.0, -7.0, 4.0, 7.0], [2.0, -5.0, 7.0, -7.0, 1.0], [-2.0, -3.0, 4.0, 1.0, 6.0], [1.0, 6.0, -4.0, 3.0, 5.0]], [[1.0, 0.0, -6.0, -5.0, 3.0], [-5.0, 2.0, 4.0, -1.0, 5.0], [-6.0, -5.0, 7.0, -6.0, -3.0], [-6.0, -1.0, 5.0, -2.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-13.0, -16.0, 35.0, -9.0, 93.0], [-30.0, -16.0, -42.0, 14.0, 45.0], [-5.0, -11.0, -3.0, 87.0, 41.0], [-60.0, -70.0, -86.0, 6.0, 24.0]])+(1.-msk_ref)*numpy.array([[34.0, 91.0, -49.0, -25.0, 106.0], [-3.0, -15.0, 107.0, -86.0, 42.0], [-20.0, -86.0, 35.0, -29.0, 45.0], [-41.0, 35.0, 25.0, 37.0, 63.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank4_offset2(self):
      arg0=numpy.array([[-6.0, -2.0, 0.0], [-2.0, 0.0, 0.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, 3.0, 7.0, -7.0, 3.0], [-2.0, -2.0, 0.0, -7.0, 2.0], [-2.0, -6.0, 4.0, 2.0, -2.0], [-4.0, 4.0, 3.0, 7.0, -3.0]], [[5.0, 1.0, 0.0, -1.0, 7.0], [5.0, 2.0, 0.0, 4.0, 0.0], [5.0, -7.0, 3.0, 5.0, -4.0], [3.0, -1.0, 7.0, 0.0, -2.0]], [[4.0, -7.0, 3.0, 0.0, 0.0], [3.0, 2.0, 4.0, 3.0, -2.0], [5.0, -2.0, -6.0, 5.0, -4.0], [-4.0, -6.0, 0.0, -7.0, -2.0]]], [[[5.0, 7.0, -2.0, 5.0, 3.0], [7.0, 0.0, 5.0, 1.0, -2.0], [0.0, 7.0, 1.0, 5.0, -4.0], [5.0, 7.0, 0.0, 2.0, 5.0]], [[6.0, 6.0, 4.0, -4.0, -6.0], [-3.0, 0.0, -4.0, 7.0, 4.0], [7.0, 3.0, 0.0, -6.0, -7.0], [6.0, 3.0, 0.0, -7.0, 4.0]], [[-2.0, 2.0, 0.0, 2.0, 0.0], [-4.0, 0.0, 0.0, 2.0, 3.0], [-7.0, -6.0, 1.0, -7.0, -4.0], [7.0, 4.0, 0.0, 0.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, 7.0, 2.0, -7.0, 0.0], [1.0, -7.0, 0.0, -4.0, 5.0], [-7.0, -3.0, 0.0, 3.0, -3.0], [5.0, -4.0, 7.0, -1.0, 3.0]], [[5.0, 4.0, 5.0, 4.0, 3.0], [2.0, 4.0, -7.0, 1.0, -5.0], [-1.0, -7.0, 5.0, 0.0, -3.0], [-2.0, 1.0, 1.0, 0.0, 6.0]], [[-3.0, 0.0, 7.0, -5.0, -5.0], [3.0, -4.0, 3.0, 5.0, -7.0], [-7.0, -2.0, 7.0, 7.0, -4.0], [0.0, -3.0, 0.0, -7.0, -4.0]]], [[[-3.0, -4.0, 0.0, -7.0, -1.0], [-7.0, 6.0, -7.0, 6.0, 6.0], [7.0, 0.0, 6.0, -3.0, 0.0], [0.0, 4.0, 1.0, 1.0, -2.0]], [[0.0, 0.0, 4.0, 4.0, 7.0], [0.0, -5.0, 2.0, -7.0, 4.0], [-3.0, -4.0, -3.0, 2.0, 7.0], [-1.0, -7.0, -7.0, 7.0, 0.0]], [[-7.0, 5.0, -3.0, 1.0, 3.0], [-6.0, -1.0, -6.0, -5.0, -6.0], [-2.0, 6.0, -3.0, 4.0, 0.0], [0.0, -1.0, -2.0, 6.0, -7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-62.0, -34.0, -38.0, 34.0, -38.0], [-12.0, 8.0, -10.0, 32.0, -8.0], [2.0, 36.0, -32.0, -32.0, 28.0], [8.0, -36.0, -32.0, -46.0, 12.0]])+(1.-msk_ref)*numpy.array([[-46.0, -42.0, -22.0, 48.0, -4.0], [4.0, 22.0, 28.0, 10.0, -32.0], [30.0, 32.0, -22.0, -12.0, 24.0], [-26.0, 14.0, -46.0, 4.0, -26.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_expandedData_rank3_offset0(self):
      arg0=numpy.array(-5.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 0.0], [2.0, -6.0]], [[-4.0, 0.0], [-1.0, -5.0]], [[-2.0, 6.0], [-2.0, -4.0]], [[-4.0, 4.0], [-2.0, -6.0]], [[0.0, 3.0], [2.0, -1.0]], [[6.0, 7.0], [-3.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, 4.0], [5.0, 5.0]], [[-5.0, -1.0], [-1.0, 2.0]], [[2.0, -2.0], [0.0, 6.0]], [[-5.0, 6.0], [-2.0, 0.0]], [[4.0, 4.0], [0.0, 5.0]], [[-5.0, -3.0], [4.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[25.0, -0.0], [-10.0, 30.0]], [[20.0, -0.0], [5.0, 25.0]], [[10.0, -30.0], [10.0, 20.0]], [[20.0, -20.0], [10.0, 30.0]], [[-0.0, -15.0], [-10.0, 5.0]], [[-30.0, -35.0], [15.0, -35.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, -20.0], [-25.0, -25.0]], [[25.0, 5.0], [5.0, -10.0]], [[-10.0, 10.0], [-0.0, -30.0]], [[25.0, -30.0], [10.0, -0.0]], [[-20.0, -20.0], [-0.0, -25.0]], [[25.0, 15.0], [-20.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank4_offset1(self):
      arg0=numpy.array([-5.0, -6.0, -3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, 0.0], [-5.0, 0.0]], [[6.0, -3.0], [4.0, -4.0]], [[-3.0, 6.0], [0.0, 4.0]], [[-6.0, -4.0], [-7.0, 7.0]], [[7.0, -7.0], [-7.0, 1.0]], [[-4.0, 5.0], [0.0, 0.0]]], [[[0.0, 2.0], [0.0, 2.0]], [[-2.0, 0.0], [-5.0, -1.0]], [[5.0, 5.0], [-7.0, -3.0]], [[-7.0, 0.0], [7.0, -5.0]], [[1.0, -7.0], [5.0, 2.0]], [[-2.0, 4.0], [-3.0, -3.0]]], [[[-6.0, 7.0], [1.0, 6.0]], [[0.0, 5.0], [4.0, -1.0]], [[-6.0, -3.0], [5.0, -1.0]], [[0.0, 5.0], [2.0, -1.0]], [[-3.0, 5.0], [7.0, 2.0]], [[6.0, 0.0], [-5.0, 1.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, -2.0], [-6.0, -2.0]], [[-3.0, 0.0], [4.0, 0.0]], [[-4.0, 1.0], [2.0, 3.0]], [[0.0, 2.0], [-4.0, -2.0]], [[2.0, 0.0], [3.0, 4.0]], [[0.0, 7.0], [2.0, -7.0]]], [[[2.0, 2.0], [7.0, 5.0]], [[-2.0, 7.0], [0.0, 3.0]], [[4.0, 7.0], [0.0, 1.0]], [[-3.0, 5.0], [6.0, 4.0]], [[-4.0, 0.0], [4.0, -5.0]], [[-2.0, 6.0], [3.0, -1.0]]], [[[7.0, -1.0], [4.0, 7.0]], [[-6.0, 0.0], [-3.0, 1.0]], [[-1.0, 1.0], [6.0, -3.0]], [[-4.0, 1.0], [0.0, -1.0]], [[1.0, 7.0], [4.0, 4.0]], [[-4.0, -3.0], [-2.0, 7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -33.0], [22.0, -30.0]], [[-18.0, 0.0], [-2.0, 29.0]], [[3.0, -51.0], [27.0, 1.0]], [[72.0, 5.0], [-13.0, -2.0]], [[-32.0, 62.0], [-16.0, -23.0]], [[14.0, -49.0], [33.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-38.0, 1.0], [-24.0, -41.0]], [[45.0, -42.0], [-11.0, -21.0]], [[-1.0, -50.0], [-28.0, -12.0]], [[30.0, -43.0], [-16.0, -11.0]], [[11.0, -21.0], [-51.0, -2.0]], [[24.0, -62.0], [-22.0, 20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank0_expandedData_rank4_offset0(self):
      arg0=numpy.array(-1.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -3.0, 4.0, -1.0], [7.0, 1.0, -3.0, 1.0], [2.0, -3.0, 0.0, 5.0]], [[3.0, 4.0, 4.0, 1.0], [-7.0, 0.0, 6.0, -6.0], [4.0, -4.0, -2.0, 1.0]]], [[[-7.0, 5.0, 0.0, -5.0], [-5.0, 0.0, 5.0, -5.0], [-4.0, 1.0, -6.0, -1.0]], [[-7.0, -5.0, -3.0, 2.0], [-2.0, 0.0, -7.0, -7.0], [0.0, -6.0, -1.0, 5.0]]], [[[-2.0, 6.0, 5.0, -5.0], [2.0, 4.0, -6.0, -4.0], [-5.0, 4.0, -6.0, 7.0]], [[5.0, 4.0, 4.0, -2.0], [7.0, 3.0, -1.0, -2.0], [-1.0, 6.0, 6.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[-3.0, -7.0, 0.0, -5.0], [4.0, 5.0, -1.0, 5.0], [5.0, 2.0, 7.0, 1.0]], [[7.0, 0.0, -6.0, 6.0], [-5.0, 0.0, 0.0, 5.0], [-5.0, 7.0, -5.0, 1.0]]], [[[0.0, -1.0, 5.0, 5.0], [4.0, 1.0, 5.0, 2.0], [0.0, -2.0, 3.0, 5.0]], [[-5.0, 4.0, 2.0, -4.0], [0.0, 0.0, -2.0, -6.0], [7.0, 6.0, 0.0, 5.0]]], [[[-3.0, 5.0, 0.0, -6.0], [7.0, -2.0, -5.0, -4.0], [-2.0, 7.0, -4.0, -2.0]], [[-4.0, 1.0, 6.0, 0.0], [1.0, 3.0, 7.0, 0.0], [-3.0, -1.0, -2.0, -3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 3.0, -4.0, 1.0], [-7.0, -1.0, 3.0, -1.0], [-2.0, 3.0, -0.0, -5.0]], [[-3.0, -4.0, -4.0, -1.0], [7.0, -0.0, -6.0, 6.0], [-4.0, 4.0, 2.0, -1.0]]], [[[7.0, -5.0, -0.0, 5.0], [5.0, -0.0, -5.0, 5.0], [4.0, -1.0, 6.0, 1.0]], [[7.0, 5.0, 3.0, -2.0], [2.0, -0.0, 7.0, 7.0], [-0.0, 6.0, 1.0, -5.0]]], [[[2.0, -6.0, -5.0, 5.0], [-2.0, -4.0, 6.0, 4.0], [5.0, -4.0, 6.0, -7.0]], [[-5.0, -4.0, -4.0, 2.0], [-7.0, -3.0, 1.0, 2.0], [1.0, -6.0, -6.0, -0.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 7.0, -0.0, 5.0], [-4.0, -5.0, 1.0, -5.0], [-5.0, -2.0, -7.0, -1.0]], [[-7.0, -0.0, 6.0, -6.0], [5.0, -0.0, -0.0, -5.0], [5.0, -7.0, 5.0, -1.0]]], [[[-0.0, 1.0, -5.0, -5.0], [-4.0, -1.0, -5.0, -2.0], [-0.0, 2.0, -3.0, -5.0]], [[5.0, -4.0, -2.0, 4.0], [-0.0, -0.0, 2.0, 6.0], [-7.0, -6.0, -0.0, -5.0]]], [[[3.0, -5.0, -0.0, 6.0], [-7.0, 2.0, 5.0, 4.0], [2.0, -7.0, 4.0, 2.0]], [[4.0, -1.0, -6.0, -0.0], [-1.0, -3.0, -7.0, -0.0], [3.0, 1.0, 2.0, 3.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_float_rank0_offset0(self):
      arg0=numpy.array([4.0, -3.0])
      arg1=3.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([12.0, -9.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank0_offset0(self):
      arg0=numpy.array([3.0, 5.0])
      arg1=numpy.array(4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([12.0, 20.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank1_offset1(self):
      arg0=numpy.array([[0.0, 3.0, -3.0], [3.0, 5.0, 0.0]])
      arg1=numpy.array([3.0, 3.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([12.0, 24.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank2_offset2(self):
      arg0=numpy.array([[[-3.0, -7.0, 5.0], [2.0, -2.0, 0.0]], [[-1.0, 0.0, 0.0], [-6.0, -5.0, -1.0]]])
      arg1=numpy.array([[-3.0, -4.0, 0.0], [-1.0, -7.0, -5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([49.0, 49.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank3_offset3(self):
      arg0=numpy.array([[[[4.0, -5.0, 6.0], [7.0, 5.0, 0.0], [0.0, -5.0, -3.0], [-3.0, -2.0, -2.0]], [[4.0, 1.0, -4.0], [-1.0, -1.0, 0.0], [1.0, 0.0, 6.0], [-5.0, 2.0, -6.0]]], [[[1.0, 2.0, 6.0], [2.0, 7.0, -6.0], [7.0, -6.0, 0.0], [-3.0, 0.0, -6.0]], [[2.0, 1.0, 3.0], [4.0, -4.0, 7.0], [5.0, 0.0, -4.0], [1.0, 0.0, 4.0]]]])
      arg1=numpy.array([[[-1.0, 7.0, 2.0], [-3.0, -4.0, -6.0], [7.0, -5.0, -5.0], [0.0, -1.0, -4.0]], [[6.0, -3.0, -5.0], [-2.0, 2.0, -1.0], [4.0, -1.0, -7.0], [-4.0, 5.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([33.0, 133.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank1_offset0(self):
      arg0=numpy.array([-2.0, 0.0])
      arg1=numpy.array([-2.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[4.0, 2.0], [0.0, 0.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank2_offset1(self):
      arg0=numpy.array([[-3.0, 4.0, 6.0], [-1.0, 2.0, -1.0]])
      arg1=numpy.array([[-4.0, -1.0], [-6.0, -7.0], [7.0, -7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[30.0, -67.0], [-15.0, -6.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank3_offset2(self):
      arg0=numpy.array([[[-4.0, 7.0, 7.0], [-7.0, -7.0, -5.0]], [[-6.0, 0.0, -3.0], [-3.0, 4.0, 4.0]]])
      arg1=numpy.array([[[-2.0, 7.0], [2.0, -6.0], [-1.0, 7.0]], [[3.0, 6.0], [7.0, 0.0], [5.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[-80.0, -63.0], [54.0, -81.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank4_offset3(self):
      arg0=numpy.array([[[[-6.0, -2.0, -4.0], [1.0, -5.0, -2.0], [0.0, -3.0, -2.0], [0.0, -5.0, 5.0]], [[-3.0, 0.0, 7.0], [-4.0, 4.0, -7.0], [-1.0, 0.0, 5.0], [0.0, -1.0, 6.0]]], [[[-4.0, 4.0, 4.0], [-4.0, -7.0, 5.0], [3.0, 4.0, 5.0], [0.0, -3.0, 0.0]], [[-7.0, -6.0, -5.0], [-5.0, -6.0, 0.0], [-7.0, -6.0, -7.0], [-4.0, -6.0, -1.0]]]])
      arg1=numpy.array([[[[-7.0, 0.0], [3.0, -4.0], [7.0, 5.0]], [[7.0, -1.0], [7.0, -4.0], [-7.0, 4.0]], [[-6.0, 1.0], [1.0, 7.0], [1.0, -4.0]], [[-7.0, 3.0], [1.0, -7.0], [7.0, -1.0]]], [[[7.0, -4.0], [-1.0, 3.0], [-6.0, 6.0]], [[-7.0, -2.0], [-5.0, 0.0], [-4.0, 6.0]], [[4.0, 0.0], [-1.0, -1.0], [5.0, 0.0]], [[0.0, -4.0], [0.0, 3.0], [1.0, -6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([[19.0, -3.0], [-62.0, 88.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank2_offset0(self):
      arg0=numpy.array([0.0, -5.0])
      arg1=numpy.array([[-4.0, 0.0, -6.0, -3.0, 0.0], [1.0, 5.0, -4.0, 6.0, 0.0], [7.0, -6.0, 0.0, -4.0, 5.0], [-6.0, 2.0, -6.0, 0.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[20.0, 0.0, 30.0, 15.0, 0.0], [-5.0, -25.0, 20.0, -30.0, 0.0], [-35.0, 30.0, 0.0, 20.0, -25.0], [30.0, -10.0, 30.0, 0.0, 30.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank3_offset1(self):
      arg0=numpy.array([[-6.0, 3.0, -1.0], [2.0, -2.0, -4.0]])
      arg1=numpy.array([[[7.0, 2.0, -5.0, -1.0, -4.0], [-5.0, 7.0, -1.0, -2.0, -7.0], [4.0, 3.0, -2.0, -6.0, -4.0], [0.0, 0.0, -4.0, -3.0, 7.0]], [[4.0, 0.0, 0.0, -2.0, 0.0], [6.0, 7.0, 7.0, 0.0, 7.0], [0.0, 0.0, 2.0, 5.0, 0.0], [-7.0, 0.0, 3.0, -3.0, 0.0]], [[-2.0, -7.0, -4.0, 4.0, -5.0], [-3.0, -5.0, -4.0, -5.0, -3.0], [4.0, -5.0, -1.0, 0.0, 7.0], [1.0, 4.0, -5.0, 6.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-28.0, -5.0, 34.0, -4.0, 29.0], [51.0, -16.0, 31.0, 17.0, 66.0], [-28.0, -13.0, 19.0, 51.0, 17.0], [-22.0, -4.0, 38.0, 3.0, -37.0]], [[14.0, 32.0, 6.0, -14.0, 12.0], [-10.0, 20.0, 0.0, 16.0, -16.0], [-8.0, 26.0, -4.0, -22.0, -36.0], [10.0, -16.0, 6.0, -24.0, 34.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank4_offset2(self):
      arg0=numpy.array([[[-3.0, 7.0, 0.0], [7.0, 0.0, -7.0]], [[7.0, 4.0, -2.0], [-1.0, 6.0, 1.0]]])
      arg1=numpy.array([[[[0.0, -6.0, 1.0, -1.0, -6.0], [1.0, 5.0, -2.0, -5.0, -7.0], [6.0, -4.0, 0.0, -7.0, -5.0], [0.0, 0.0, 4.0, -1.0, -6.0]], [[-4.0, 1.0, 1.0, 5.0, -4.0], [5.0, -4.0, 3.0, -3.0, 7.0], [-2.0, -7.0, -3.0, 7.0, -3.0], [7.0, -1.0, 0.0, -2.0, -6.0]], [[1.0, 3.0, 0.0, -7.0, 2.0], [0.0, 0.0, 5.0, -6.0, 0.0], [-2.0, -5.0, 1.0, -2.0, -4.0], [-7.0, 0.0, 6.0, 3.0, -7.0]]], [[[-6.0, 4.0, 1.0, 1.0, 0.0], [7.0, -4.0, -2.0, 0.0, 3.0], [-1.0, -3.0, -3.0, -4.0, 4.0], [-7.0, 5.0, -2.0, 0.0, 0.0]], [[-6.0, 0.0, -7.0, -7.0, -6.0], [-2.0, -1.0, 6.0, 0.0, 3.0], [-5.0, -3.0, 0.0, -5.0, -7.0], [5.0, -7.0, -5.0, -3.0, -2.0]], [[0.0, 3.0, -6.0, 5.0, 0.0], [2.0, -2.0, -4.0, 6.0, 4.0], [-1.0, -3.0, 6.0, 6.0, 2.0], [0.0, 3.0, 4.0, 1.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[-70.0, 32.0, 53.0, 10.0, -10.0], [67.0, -57.0, 41.0, -48.0, 63.0], [-32.0, -37.0, -84.0, 0.0, 8.0], [0.0, 7.0, -54.0, -18.0, -17.0]], [[-48.0, -45.0, -38.0, -11.0, -98.0], [10.0, 15.0, 22.0, -29.0, -2.0], [8.0, -64.0, -5.0, -37.0, -83.0], [79.0, -48.0, -8.0, -38.0, -65.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_array_rank3_offset0(self):
      arg0=numpy.array([1.0, 5.0])
      arg1=numpy.array([[[4.0, 3.0], [6.0, 0.0]], [[0.0, 0.0], [5.0, 4.0]], [[-5.0, 1.0], [-7.0, -2.0]], [[4.0, -7.0], [7.0, -2.0]], [[5.0, 2.0], [-3.0, 1.0]], [[0.0, 4.0], [-1.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[4.0, 3.0], [6.0, 0.0]], [[0.0, 0.0], [5.0, 4.0]], [[-5.0, 1.0], [-7.0, -2.0]], [[4.0, -7.0], [7.0, -2.0]], [[5.0, 2.0], [-3.0, 1.0]], [[0.0, 4.0], [-1.0, -5.0]]], [[[20.0, 15.0], [30.0, 0.0]], [[0.0, 0.0], [25.0, 20.0]], [[-25.0, 5.0], [-35.0, -10.0]], [[20.0, -35.0], [35.0, -10.0]], [[25.0, 10.0], [-15.0, 5.0]], [[0.0, 20.0], [-5.0, -25.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank4_offset1(self):
      arg0=numpy.array([[-7.0, 4.0, -6.0], [-2.0, -3.0, -7.0]])
      arg1=numpy.array([[[[3.0, 1.0], [-1.0, 2.0]], [[2.0, 2.0], [0.0, 1.0]], [[-5.0, 2.0], [-2.0, 1.0]], [[-1.0, 7.0], [7.0, 5.0]], [[1.0, 0.0], [-5.0, -6.0]], [[-3.0, -1.0], [-7.0, -7.0]]], [[[-7.0, 4.0], [-2.0, 5.0]], [[5.0, -1.0], [-2.0, 7.0]], [[5.0, -2.0], [0.0, 5.0]], [[7.0, -3.0], [0.0, 0.0]], [[1.0, 2.0], [4.0, 7.0]], [[3.0, 0.0], [5.0, 5.0]]], [[[-7.0, -6.0], [-7.0, 3.0]], [[1.0, -6.0], [4.0, -7.0]], [[1.0, -7.0], [-5.0, -1.0]], [[0.0, -1.0], [-3.0, -7.0]], [[5.0, 4.0], [-1.0, -5.0]], [[4.0, 6.0], [2.0, -7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[-7.0, 45.0], [41.0, -12.0]], [[0.0, 18.0], [-32.0, 63.0]], [[49.0, 20.0], [44.0, 19.0]], [[35.0, -55.0], [-31.0, 7.0]], [[-33.0, -16.0], [57.0, 100.0]], [[9.0, -29.0], [57.0, 111.0]]], [[[64.0, 28.0], [57.0, -40.0]], [[-26.0, 41.0], [-22.0, 26.0]], [[-12.0, 51.0], [39.0, -10.0]], [[-19.0, 2.0], [7.0, 39.0]], [[-40.0, -34.0], [5.0, 26.0]], [[-31.0, -40.0], [-15.0, 48.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank0_offset0(self):
      arg0=numpy.array([-1.0, -3.0])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(6.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-6.0, -18.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank1_offset1(self):
      arg0=numpy.array([[-4.0, 2.0, 2.0], [-4.0, -1.0, 4.0]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([3.0, -5.0, -3.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-28.0, -19.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[-2.0, -2.0, -6.0], [7.0, 6.0, 2.0]], [[-5.0, -5.0, 1.0], [5.0, 1.0, -1.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[1.0, 6.0, -4.0], [-4.0, 1.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-18.0, -55.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[[-1.0, -7.0, 2.0], [-2.0, 4.0, -7.0], [-7.0, -2.0, 3.0], [1.0, -7.0, -6.0]], [[-1.0, 6.0, -7.0], [-6.0, 5.0, -6.0], [-4.0, -6.0, 0.0], [2.0, -4.0, 5.0]]], [[[-7.0, 6.0, 4.0], [-1.0, 5.0, 0.0], [6.0, -6.0, 0.0], [2.0, 2.0, -7.0]], [[0.0, -2.0, -6.0], [0.0, 7.0, -5.0], [3.0, 5.0, -4.0], [0.0, -6.0, 0.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-4.0, 1.0, 0.0], [2.0, 3.0, 7.0], [-5.0, 0.0, -7.0], [3.0, 4.0, -6.0]], [[-5.0, 0.0, -1.0], [5.0, -3.0, 4.0], [-6.0, 5.0, 7.0], [4.0, -4.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-53.0, 41.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank1_offset0(self):
      arg0=numpy.array([-1.0, -6.0])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-4.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[4.0, -4.0], [24.0, -24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank2_offset1(self):
      arg0=numpy.array([[5.0, 4.0, -3.0], [2.0, -6.0, 6.0]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-1.0, 0.0], [-3.0, -5.0], [5.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-32.0, -2.0], [46.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[7.0, -5.0, -5.0], [-1.0, 0.0, 2.0]], [[-2.0, 1.0, 4.0], [7.0, 7.0, 0.0]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[6.0, -2.0], [5.0, 7.0], [5.0, -7.0]], [[4.0, -1.0], [5.0, 6.0], [0.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-12.0, -25.0], [76.0, 18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[[2.0, 1.0, -3.0], [-7.0, 2.0, 6.0], [-4.0, -5.0, 6.0], [3.0, 0.0, -1.0]], [[7.0, -4.0, 6.0], [-1.0, -7.0, 6.0], [-6.0, -4.0, 1.0], [-1.0, -7.0, 0.0]]], [[[6.0, 4.0, -5.0], [-6.0, -3.0, 1.0], [3.0, -3.0, -7.0], [-6.0, 0.0, -2.0]], [[-6.0, 5.0, 5.0], [0.0, 4.0, 0.0], [3.0, -4.0, -2.0], [7.0, 3.0, 6.0]]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-7.0, 4.0], [-5.0, -2.0], [0.0, -1.0]], [[-3.0, -5.0], [-6.0, -4.0], [-3.0, -5.0]], [[0.0, 7.0], [-1.0, -5.0], [0.0, 4.0]], [[-7.0, -7.0], [4.0, 4.0], [-5.0, 0.0]]], [[[4.0, -4.0], [-5.0, -4.0], [6.0, 3.0]], [[0.0, -5.0], [-5.0, -4.0], [3.0, -3.0]], [[1.0, -5.0], [-4.0, 7.0], [-3.0, 2.0]], [[-5.0, -1.0], [-5.0, -1.0], [6.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[145.0, 39.0], [-2.0, 30.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank2_offset0(self):
      arg0=numpy.array([4.0, 4.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-4.0, 2.0, 6.0, 2.0, 4.0], [4.0, -2.0, 0.0, 6.0, 5.0], [4.0, -2.0, 3.0, 1.0, 6.0], [3.0, -4.0, 5.0, 6.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-16.0, 8.0, 24.0, 8.0, 16.0], [16.0, -8.0, 0.0, 24.0, 20.0], [16.0, -8.0, 12.0, 4.0, 24.0], [12.0, -16.0, 20.0, 24.0, -24.0]], [[-16.0, 8.0, 24.0, 8.0, 16.0], [16.0, -8.0, 0.0, 24.0, 20.0], [16.0, -8.0, 12.0, 4.0, 24.0], [12.0, -16.0, 20.0, 24.0, -24.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank3_offset1(self):
      arg0=numpy.array([[3.0, 3.0, 6.0], [-6.0, 0.0, -2.0]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[0.0, 6.0, -6.0, 0.0, -7.0], [-6.0, 4.0, -2.0, -4.0, 1.0], [-6.0, -5.0, -6.0, 6.0, -5.0], [0.0, -3.0, 6.0, -2.0, 6.0]], [[0.0, 4.0, 5.0, 5.0, -4.0], [-6.0, 5.0, -5.0, 3.0, -6.0], [-4.0, -7.0, -6.0, 5.0, 0.0], [-6.0, -6.0, 0.0, -1.0, 1.0]], [[0.0, 3.0, -2.0, 0.0, -5.0], [0.0, 3.0, 1.0, 0.0, 7.0], [-5.0, 5.0, -7.0, -5.0, -2.0], [-1.0, -3.0, -2.0, 6.0, -2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.0, 48.0, -15.0, 15.0, -63.0], [-36.0, 45.0, -15.0, -3.0, 27.0], [-60.0, -6.0, -78.0, 3.0, -27.0], [-24.0, -45.0, 6.0, 27.0, 9.0]], [[0.0, -42.0, 40.0, 0.0, 52.0], [36.0, -30.0, 10.0, 24.0, -20.0], [46.0, 20.0, 50.0, -26.0, 34.0], [2.0, 24.0, -32.0, 0.0, -32.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[7.0, 1.0, 6.0], [-7.0, -6.0, 0.0]], [[0.0, -7.0, -4.0], [0.0, 5.0, -2.0]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[4.0, -3.0, -4.0, 7.0, 0.0], [-7.0, 2.0, 2.0, 5.0, -1.0], [2.0, 2.0, 2.0, -1.0, 3.0], [-3.0, -7.0, -4.0, -5.0, 2.0]], [[-5.0, -4.0, 7.0, -5.0, -6.0], [3.0, 5.0, -7.0, 4.0, 1.0], [5.0, 4.0, -6.0, -3.0, -4.0], [5.0, 6.0, -1.0, -2.0, 7.0]], [[0.0, -3.0, 0.0, -1.0, -4.0], [-7.0, -7.0, -4.0, 0.0, -7.0], [0.0, 2.0, -2.0, 5.0, 4.0], [-1.0, 6.0, 4.0, -7.0, 0.0]]], [[[0.0, 2.0, -6.0, 0.0, -4.0], [5.0, -7.0, -2.0, -5.0, -1.0], [0.0, 4.0, 0.0, 5.0, 2.0], [6.0, -7.0, -2.0, -1.0, -3.0]], [[0.0, -7.0, -6.0, 0.0, -4.0], [0.0, 5.0, 7.0, -1.0, 2.0], [1.0, 3.0, -7.0, 0.0, 3.0], [-2.0, 4.0, 4.0, -1.0, 0.0]], [[2.0, 2.0, -7.0, 6.0, -4.0], [-3.0, 6.0, -5.0, -2.0, 0.0], [-6.0, 5.0, -3.0, -3.0, 3.0], [0.0, 4.0, -1.0, 7.0, -6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[23.0, -15.0, 57.0, 38.0, 22.0], [-123.0, -4.0, -45.0, 80.0, -53.0], [13.0, -16.0, 38.0, -15.0, 9.0], [-52.0, 18.0, -15.0, -66.0, 42.0]], [[31.0, 1.0, -65.0, 27.0, 46.0], [13.0, 6.0, 110.0, -29.0, 31.0], [-18.0, -31.0, 21.0, 7.0, 21.0], [-41.0, -54.0, 13.0, 23.0, -37.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_Symbol_rank3_offset0(self):
      arg0=numpy.array([-6.0, 0.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[1.0, 4.0], [-3.0, -5.0]], [[1.0, -1.0], [6.0, 2.0]], [[1.0, 0.0], [5.0, -3.0]], [[5.0, -6.0], [0.0, 5.0]], [[6.0, -7.0], [0.0, 7.0]], [[1.0, -2.0], [7.0, 4.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-6.0, -24.0], [18.0, 30.0]], [[-6.0, 6.0], [-36.0, -12.0]], [[-6.0, 0.0], [-30.0, 18.0]], [[-30.0, 36.0], [0.0, -30.0]], [[-36.0, 42.0], [0.0, -42.0]], [[-6.0, 12.0], [-42.0, -24.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank4_offset1(self):
      arg0=numpy.array([[6.0, 5.0, 1.0], [5.0, 2.0, -4.0]])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[6.0, -5.0], [-3.0, -1.0]], [[-2.0, -4.0], [-1.0, 0.0]], [[0.0, -4.0], [5.0, -7.0]], [[-5.0, -1.0], [0.0, 0.0]], [[4.0, -5.0], [2.0, 3.0]], [[-2.0, 6.0], [1.0, -1.0]]], [[[-5.0, -6.0], [-1.0, -2.0]], [[7.0, 6.0], [-1.0, 5.0]], [[-1.0, -6.0], [4.0, 3.0]], [[-2.0, 2.0], [5.0, -1.0]], [[-5.0, -1.0], [6.0, -1.0]], [[-4.0, 4.0], [5.0, 3.0]]], [[[-3.0, 1.0], [-6.0, -5.0]], [[-2.0, -2.0], [1.0, -3.0]], [[4.0, -2.0], [-7.0, -7.0]], [[5.0, 2.0], [-1.0, 6.0]], [[-4.0, 0.0], [-4.0, 6.0]], [[-5.0, 2.0], [6.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[8.0, -59.0], [-29.0, -21.0]], [[21.0, 4.0], [-10.0, 22.0]], [[-1.0, -56.0], [43.0, -34.0]], [[-35.0, 6.0], [24.0, 1.0]], [[-5.0, -35.0], [38.0, 19.0]], [[-37.0, 58.0], [37.0, 12.0]]], [[[32.0, -41.0], [7.0, 11.0]], [[12.0, 0.0], [-11.0, 22.0]], [[-18.0, -24.0], [61.0, -1.0]], [[-49.0, -9.0], [14.0, -26.0]], [[26.0, -27.0], [38.0, -11.0]], [[2.0, 30.0], [-9.0, -11.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank0_offset0(self):
      arg0=numpy.array([0.0, -5.0])
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-0.0, 35.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank1_offset1(self):
      arg0=numpy.array([[-4.0, 0.0, 7.0], [-1.0, -7.0, -4.0]])
      arg1=Data(numpy.array([-5.0, -6.0, -3.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-1.0, 59.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank2_offset2(self):
      arg0=numpy.array([[[-1.0, -6.0, 7.0], [-2.0, -7.0, 4.0]], [[-4.0, 1.0, -7.0], [-2.0, -4.0, 7.0]]])
      arg1=Data(numpy.array([[0.0, -5.0, -2.0], [-1.0, 7.0, 0.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-31.0, -17.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank3_offset3(self):
      arg0=numpy.array([[[[-3.0, 3.0, 5.0], [-4.0, -2.0, 7.0], [-1.0, -1.0, -1.0], [-6.0, -3.0, 6.0]], [[-7.0, -7.0, 1.0], [-5.0, -1.0, -5.0], [4.0, 2.0, 0.0], [-7.0, 1.0, 1.0]]], [[[7.0, -7.0, 3.0], [1.0, -7.0, 0.0], [-4.0, -7.0, 4.0], [-6.0, 0.0, 1.0]], [[-3.0, -3.0, 5.0], [-1.0, 1.0, 2.0], [5.0, 0.0, -6.0], [-3.0, 5.0, -5.0]]]])
      arg1=Data(numpy.array([[[-2.0, -5.0, 0.0], [2.0, 7.0, -4.0], [-2.0, -3.0, 5.0], [-2.0, -3.0, -3.0]], [[3.0, 0.0, 2.0], [-5.0, 0.0, 3.0], [5.0, -6.0, -7.0], [3.0, 6.0, 0.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-72.0, 132.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank1_offset0(self):
      arg0=numpy.array([-7.0, 0.0])
      arg1=Data(numpy.array([-3.0, 4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[21.0, -28.0], [0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank2_offset1(self):
      arg0=numpy.array([[1.0, -2.0, -6.0], [3.0, -1.0, 2.0]])
      arg1=Data(numpy.array([[1.0, 3.0], [4.0, -1.0], [-1.0, 2.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-1.0, -7.0], [-3.0, 14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank3_offset2(self):
      arg0=numpy.array([[[-1.0, -6.0, 0.0], [-2.0, 0.0, 7.0]], [[-5.0, 5.0, 3.0], [-1.0, -4.0, -3.0]]])
      arg1=Data(numpy.array([[[1.0, 0.0], [-3.0, -4.0], [-2.0, -5.0]], [[6.0, 0.0], [3.0, -1.0], [3.0, -6.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[26.0, -18.0], [-53.0, -13.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank4_offset3(self):
      arg0=numpy.array([[[[-5.0, -5.0, 6.0], [-7.0, -7.0, 5.0], [1.0, 6.0, -7.0], [0.0, 1.0, -4.0]], [[-6.0, -7.0, 3.0], [2.0, -3.0, -7.0], [-3.0, -2.0, -4.0], [4.0, -4.0, -4.0]]], [[[-3.0, 0.0, 6.0], [-2.0, 0.0, 7.0], [-4.0, -6.0, -3.0], [1.0, -5.0, -6.0]], [[5.0, 7.0, -5.0], [3.0, -2.0, 6.0], [-7.0, -1.0, -7.0], [-4.0, 6.0, 5.0]]]])
      arg1=Data(numpy.array([[[[7.0, -7.0], [5.0, 4.0], [4.0, -4.0]], [[4.0, 0.0], [6.0, -7.0], [-5.0, 6.0]], [[-4.0, -6.0], [-3.0, -5.0], [0.0, 1.0]], [[1.0, -5.0], [-7.0, -4.0], [0.0, -5.0]]], [[[4.0, 1.0], [-2.0, -4.0], [-7.0, 7.0]], [[1.0, 4.0], [-4.0, 0.0], [3.0, 2.0]], [[2.0, 4.0], [1.0, 5.0], [4.0, -2.0]], [[7.0, 3.0], [-3.0, 5.0], [-4.0, 2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[-166.0, 50.0], [-9.0, 110.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank2_offset0(self):
      arg0=numpy.array([-4.0, -3.0])
      arg1=Data(numpy.array([[5.0, -4.0, 2.0, 7.0, 4.0], [3.0, -1.0, 7.0, -6.0, 2.0], [6.0, 0.0, 6.0, -4.0, 6.0], [-6.0, -5.0, -6.0, 3.0, -4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-20.0, 16.0, -8.0, -28.0, -16.0], [-12.0, 4.0, -28.0, 24.0, -8.0], [-24.0, 0.0, -24.0, 16.0, -24.0], [24.0, 20.0, 24.0, -12.0, 16.0]], [[-15.0, 12.0, -6.0, -21.0, -12.0], [-9.0, 3.0, -21.0, 18.0, -6.0], [-18.0, 0.0, -18.0, 12.0, -18.0], [18.0, 15.0, 18.0, -9.0, 12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank3_offset1(self):
      arg0=numpy.array([[0.0, 6.0, 4.0], [-5.0, -3.0, -4.0]])
      arg1=Data(numpy.array([[[0.0, 7.0, -6.0, 2.0, -7.0], [6.0, -4.0, -2.0, -2.0, 6.0], [-7.0, -5.0, -6.0, 7.0, -5.0], [-1.0, 0.0, 3.0, -2.0, -7.0]], [[-7.0, -1.0, -4.0, 3.0, 0.0], [6.0, -6.0, 2.0, 4.0, 5.0], [0.0, -3.0, -6.0, 1.0, -1.0], [-7.0, -3.0, 0.0, 5.0, -5.0]], [[-6.0, -7.0, -3.0, 4.0, 1.0], [0.0, 3.0, -1.0, 3.0, 0.0], [-7.0, 6.0, -5.0, -4.0, -7.0], [6.0, 3.0, 3.0, 7.0, 5.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-66.0, -34.0, -36.0, 34.0, 4.0], [36.0, -24.0, 8.0, 36.0, 30.0], [-28.0, 6.0, -56.0, -10.0, -34.0], [-18.0, -6.0, 12.0, 58.0, -10.0]], [[45.0, -4.0, 54.0, -35.0, 31.0], [-48.0, 26.0, 8.0, -14.0, -45.0], [63.0, 10.0, 68.0, -22.0, 56.0], [2.0, -3.0, -27.0, -33.0, 30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank4_offset2(self):
      arg0=numpy.array([[[2.0, 4.0, 4.0], [7.0, -3.0, 1.0]], [[4.0, 3.0, -4.0], [-3.0, -5.0, 0.0]]])
      arg1=Data(numpy.array([[[[-5.0, -4.0, 0.0, 2.0, 2.0], [-2.0, 0.0, 0.0, 7.0, 5.0], [-4.0, -1.0, 1.0, 4.0, -2.0], [0.0, 6.0, -4.0, 7.0, 0.0]], [[0.0, 5.0, 1.0, 0.0, 1.0], [3.0, 3.0, -6.0, 2.0, -4.0], [-6.0, 3.0, 2.0, -4.0, 1.0], [-7.0, -1.0, 5.0, 3.0, -3.0]], [[1.0, 4.0, 2.0, 7.0, 3.0], [5.0, -3.0, 3.0, 3.0, -7.0], [-3.0, 2.0, 7.0, -4.0, -6.0], [5.0, -7.0, 4.0, 3.0, 2.0]]], [[[2.0, 1.0, 0.0, -6.0, 0.0], [-7.0, 1.0, -4.0, 3.0, -6.0], [-1.0, 4.0, 2.0, 3.0, -1.0], [0.0, -2.0, 0.0, 6.0, 3.0]], [[0.0, -3.0, 5.0, -7.0, -2.0], [-1.0, 7.0, -2.0, -1.0, 1.0], [2.0, -7.0, 3.0, 7.0, 3.0], [-2.0, 5.0, 0.0, -4.0, 3.0]], [[7.0, -1.0, -2.0, 4.0, 6.0], [2.0, 1.0, 2.0, 0.0, 6.0], [2.0, 6.0, -1.0, -2.0, -2.0], [3.0, -4.0, 5.0, 5.0, -7.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[15.0, 43.0, -5.0, 15.0, 32.0], [-16.0, -13.0, -32.0, 58.0, -73.0], [-55.0, 73.0, 42.0, -26.0, -42.0], [1.0, -53.0, 33.0, 97.0, 1.0]], [[-30.0, -5.0, -30.0, 33.0, 9.0], [7.0, -17.0, -8.0, 18.0, 49.0], [-29.0, 20.0, -39.0, -24.0, 7.0], [-31.0, 30.0, -17.0, 27.0, -41.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_constData_rank3_offset0(self):
      arg0=numpy.array([-4.0, -3.0])
      arg1=Data(numpy.array([[[3.0, 0.0], [7.0, -4.0]], [[-4.0, 3.0], [-4.0, -3.0]], [[4.0, -1.0], [1.0, 2.0]], [[4.0, 5.0], [-7.0, 1.0]], [[-5.0, 4.0], [4.0, 2.0]], [[0.0, -4.0], [-4.0, 2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-12.0, 0.0], [-28.0, 16.0]], [[16.0, -12.0], [16.0, 12.0]], [[-16.0, 4.0], [-4.0, -8.0]], [[-16.0, -20.0], [28.0, -4.0]], [[20.0, -16.0], [-16.0, -8.0]], [[0.0, 16.0], [16.0, -8.0]]], [[[-9.0, 0.0], [-21.0, 12.0]], [[12.0, -9.0], [12.0, 9.0]], [[-12.0, 3.0], [-3.0, -6.0]], [[-12.0, -15.0], [21.0, -3.0]], [[15.0, -12.0], [-12.0, -6.0]], [[0.0, 12.0], [12.0, -6.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank4_offset1(self):
      arg0=numpy.array([[1.0, -4.0, 7.0], [-3.0, 5.0, 1.0]])
      arg1=Data(numpy.array([[[[-1.0, 4.0], [0.0, -6.0]], [[4.0, 2.0], [7.0, -2.0]], [[-5.0, 3.0], [5.0, 2.0]], [[0.0, 1.0], [0.0, 3.0]], [[-3.0, 5.0], [-4.0, 2.0]], [[7.0, -6.0], [4.0, -3.0]]], [[[-3.0, 2.0], [1.0, -3.0]], [[2.0, -4.0], [5.0, -1.0]], [[5.0, -7.0], [-3.0, 4.0]], [[-5.0, -5.0], [3.0, -5.0]], [[0.0, -6.0], [0.0, -1.0]], [[3.0, -2.0], [2.0, 5.0]]], [[[-1.0, 1.0], [-5.0, -5.0]], [[-5.0, -6.0], [4.0, -4.0]], [[-2.0, 0.0], [3.0, -7.0]], [[1.0, 6.0], [1.0, 4.0]], [[2.0, 3.0], [2.0, 3.0]], [[-1.0, 0.0], [4.0, -2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[4.0, 3.0], [-39.0, -29.0]], [[-39.0, -24.0], [15.0, -26.0]], [[-39.0, 31.0], [38.0, -63.0]], [[27.0, 63.0], [-5.0, 51.0]], [[11.0, 50.0], [10.0, 27.0]], [[-12.0, 2.0], [24.0, -37.0]]], [[[-13.0, -1.0], [0.0, -2.0]], [[-7.0, -32.0], [8.0, -3.0]], [[38.0, -44.0], [-27.0, 7.0]], [[-24.0, -22.0], [16.0, -30.0]], [[11.0, -42.0], [14.0, -8.0]], [[-7.0, 8.0], [2.0, 32.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank0_offset0(self):
      arg0=numpy.array([-4.0, -2.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-7.0)+(1-msk_arg1)*(4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([28.0, 14.0])+(1.-msk_ref)*numpy.array([-16.0, -8.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank1_offset1(self):
      arg0=numpy.array([[4.0, -1.0, -3.0], [-5.0, 6.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, -3.0, -1.0])+(1.-msk_arg1)*numpy.array([-5.0, -3.0, 1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([26.0, -50.0])+(1.-msk_ref)*numpy.array([-20.0, 14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[1.0, 1.0, 7.0], [-4.0, -3.0, -3.0]], [[-4.0, -1.0, 0.0], [4.0, -4.0, -4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, -2.0, -1.0], [-7.0, 6.0, -4.0]])+(1.-msk_arg1)*numpy.array([[0.0, 7.0, -6.0], [3.0, -6.0, 1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, -6.0])+(1.-msk_ref)*numpy.array([-32.0, 25.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[[0.0, 6.0, -1.0], [3.0, 3.0, -1.0], [-5.0, -4.0, -7.0], [-2.0, -6.0, 3.0]], [[1.0, 7.0, 3.0], [0.0, 5.0, 5.0], [-2.0, 7.0, 2.0], [5.0, 6.0, 1.0]]], [[[-2.0, 0.0, -1.0], [0.0, 7.0, -5.0], [-5.0, 1.0, -6.0], [4.0, -2.0, 1.0]], [[-3.0, -1.0, 1.0], [-4.0, 4.0, -1.0], [-5.0, 7.0, -5.0], [3.0, 0.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 7.0, -2.0], [-5.0, -6.0, 2.0], [3.0, -6.0, -1.0], [3.0, 0.0, 0.0]], [[4.0, -7.0, 0.0], [-4.0, 5.0, -4.0], [4.0, -4.0, -5.0], [0.0, 4.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 2.0, 5.0], [-6.0, -2.0, 6.0], [0.0, -6.0, -2.0], [-4.0, 6.0, -3.0]], [[0.0, -2.0, 4.0], [5.0, 4.0, 6.0], [-2.0, 6.0, 7.0], [-5.0, -7.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-41.0, -33.0])+(1.-msk_ref)*numpy.array([16.0, -61.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank1_offset0(self):
      arg0=numpy.array([-3.0, 3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, -3.0])+(1.-msk_arg1)*numpy.array([-6.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 9.0], [-3.0, -9.0]])+(1.-msk_ref)*numpy.array([[18.0, -12.0], [-18.0, 12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank2_offset1(self):
      arg0=numpy.array([[-7.0, 0.0, 1.0], [3.0, 0.0, -4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 3.0], [5.0, -5.0], [0.0, -2.0]])+(1.-msk_arg1)*numpy.array([[0.0, 1.0], [3.0, -2.0], [-3.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -23.0], [0.0, 17.0]])+(1.-msk_ref)*numpy.array([[-3.0, -1.0], [12.0, -21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[-7.0, 0.0, 2.0], [6.0, 4.0, 7.0]], [[3.0, 0.0, 2.0], [4.0, -7.0, -3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, -7.0], [7.0, -7.0], [6.0, -1.0]], [[2.0, 7.0], [0.0, 1.0], [-4.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, 0.0], [-2.0, 5.0], [1.0, -2.0]], [[4.0, 4.0], [2.0, -4.0], [-1.0, -7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 58.0], [29.0, 13.0]])+(1.-msk_ref)*numpy.array([[62.0, -45.0], [-8.0, 61.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[[6.0, 0.0, 6.0], [0.0, 0.0, -2.0], [-4.0, -6.0, -7.0], [6.0, 7.0, 5.0]], [[3.0, 1.0, -1.0], [0.0, 5.0, 6.0], [7.0, 1.0, -6.0], [-5.0, -4.0, 2.0]]], [[[-1.0, 7.0, 7.0], [-4.0, -5.0, 6.0], [-4.0, 4.0, 3.0], [6.0, 6.0, -1.0]], [[-1.0, 2.0, -7.0], [7.0, -6.0, 0.0], [-5.0, -6.0, -3.0], [-2.0, 4.0, 6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, -3.0], [-7.0, -6.0], [-2.0, -6.0]], [[5.0, -6.0], [-6.0, 2.0], [-5.0, -2.0]], [[-6.0, -4.0], [0.0, -2.0], [0.0, -2.0]], [[-2.0, -7.0], [-7.0, -1.0], [-4.0, 1.0]]], [[[-7.0, -7.0], [-6.0, 0.0], [6.0, 1.0]], [[1.0, -2.0], [-1.0, 0.0], [0.0, -5.0]], [[0.0, -1.0], [0.0, -2.0], [5.0, 4.0]], [[1.0, -1.0], [-7.0, 0.0], [5.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 0.0], [1.0, 4.0], [3.0, 5.0]], [[4.0, -3.0], [-2.0, -3.0], [6.0, -7.0]], [[-6.0, -7.0], [0.0, -6.0], [-6.0, 3.0]], [[2.0, -6.0], [7.0, -4.0], [-6.0, -7.0]]], [[[2.0, -1.0], [5.0, -6.0], [2.0, -2.0]], [[-1.0, -1.0], [7.0, -6.0], [4.0, 6.0]], [[-4.0, 7.0], [-1.0, 0.0], [5.0, 3.0]], [[1.0, 3.0], [6.0, 6.0], [-3.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-58.0, -146.0], [-164.0, -175.0]])+(1.-msk_ref)*numpy.array([[95.0, -19.0], [81.0, 20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank2_offset0(self):
      arg0=numpy.array([5.0, -5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 2.0, -5.0, 2.0, -6.0], [-4.0, 4.0, 3.0, -7.0, 3.0], [5.0, 3.0, 7.0, 5.0, 1.0], [1.0, 4.0, 3.0, -2.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 0.0, 1.0, 1.0, 2.0], [-5.0, 5.0, -6.0, -2.0, -6.0], [-5.0, 5.0, -7.0, 7.0, 3.0], [3.0, -7.0, 5.0, -3.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[15.0, 10.0, -25.0, 10.0, -30.0], [-20.0, 20.0, 15.0, -35.0, 15.0], [25.0, 15.0, 35.0, 25.0, 5.0], [5.0, 20.0, 15.0, -10.0, 20.0]], [[-15.0, -10.0, 25.0, -10.0, 30.0], [20.0, -20.0, -15.0, 35.0, -15.0], [-25.0, -15.0, -35.0, -25.0, -5.0], [-5.0, -20.0, -15.0, 10.0, -20.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, 0.0, 5.0, 5.0, 10.0], [-25.0, 25.0, -30.0, -10.0, -30.0], [-25.0, 25.0, -35.0, 35.0, 15.0], [15.0, -35.0, 25.0, -15.0, -20.0]], [[30.0, 0.0, -5.0, -5.0, -10.0], [25.0, -25.0, 30.0, 10.0, 30.0], [25.0, -25.0, 35.0, -35.0, -15.0], [-15.0, 35.0, -25.0, 15.0, 20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank3_offset1(self):
      arg0=numpy.array([[3.0, 2.0, -4.0], [1.0, -2.0, 0.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, 0.0, 0.0, -3.0, -1.0], [6.0, 5.0, -7.0, 4.0, 3.0], [2.0, 0.0, -1.0, 5.0, 1.0], [-5.0, -7.0, -4.0, -3.0, 1.0]], [[-5.0, 4.0, 7.0, 5.0, -2.0], [0.0, -4.0, 7.0, -1.0, -4.0], [-4.0, -1.0, 7.0, 4.0, 0.0], [0.0, 3.0, -1.0, -5.0, -7.0]], [[-6.0, 6.0, 0.0, -5.0, -3.0], [0.0, 2.0, 6.0, -5.0, 4.0], [-2.0, 7.0, 1.0, 5.0, 0.0], [-2.0, 0.0, 4.0, -2.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 5.0, -4.0, 7.0, 7.0], [1.0, 3.0, -4.0, -6.0, -5.0], [1.0, -1.0, -7.0, 6.0, -1.0], [-1.0, 5.0, 5.0, -4.0, -3.0]], [[0.0, 0.0, -3.0, 1.0, 2.0], [3.0, -3.0, -3.0, 0.0, 1.0], [7.0, 1.0, 2.0, -6.0, 7.0], [2.0, 0.0, -5.0, -7.0, 4.0]], [[5.0, 0.0, -5.0, -5.0, 0.0], [0.0, -2.0, -2.0, 1.0, -2.0], [6.0, 5.0, 1.0, -7.0, -6.0], [2.0, 0.0, 5.0, -4.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-4.0, -16.0, 14.0, 21.0, 5.0], [18.0, -1.0, -31.0, 30.0, -15.0], [6.0, -30.0, 7.0, 3.0, 3.0], [-7.0, -15.0, -30.0, -11.0, -35.0]], [[4.0, -8.0, -14.0, -13.0, 3.0], [6.0, 13.0, -21.0, 6.0, 11.0], [10.0, 2.0, -15.0, -3.0, 1.0], [-5.0, -13.0, -2.0, 7.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, 15.0, 2.0, 43.0, 25.0], [9.0, 11.0, -10.0, -22.0, -5.0], [-7.0, -21.0, -21.0, 34.0, 35.0], [-7.0, 15.0, -15.0, -10.0, -17.0]], [[0.0, 5.0, 2.0, 5.0, 3.0], [-5.0, 9.0, 2.0, -6.0, -7.0], [-13.0, -3.0, -11.0, 18.0, -15.0], [-5.0, 5.0, 15.0, 10.0, -11.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[6.0, 2.0, -1.0], [4.0, 1.0, -6.0]], [[0.0, -4.0, -2.0], [5.0, 6.0, 2.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[4.0, 7.0, 3.0, 0.0, 1.0], [-2.0, 5.0, 0.0, -1.0, -4.0], [-1.0, 3.0, 0.0, 3.0, 0.0], [-7.0, 1.0, -6.0, -2.0, 4.0]], [[-4.0, -4.0, 0.0, 6.0, 2.0], [6.0, -2.0, -7.0, 5.0, 0.0], [-3.0, 1.0, -4.0, 0.0, 2.0], [-7.0, -1.0, -7.0, 2.0, 0.0]], [[5.0, 1.0, 0.0, -1.0, 4.0], [2.0, -1.0, 2.0, 0.0, -2.0], [-4.0, -6.0, -7.0, 2.0, 0.0], [-7.0, -7.0, -6.0, -4.0, -2.0]]], [[[5.0, -3.0, 6.0, 6.0, 3.0], [0.0, -1.0, 0.0, 6.0, 6.0], [-1.0, -4.0, 7.0, 6.0, -2.0], [0.0, 2.0, -2.0, 6.0, 0.0]], [[3.0, -2.0, -3.0, -2.0, -7.0], [-1.0, -2.0, -3.0, -1.0, 1.0], [5.0, -6.0, 6.0, -3.0, 0.0], [-4.0, -5.0, 5.0, -1.0, 0.0]], [[4.0, 0.0, 0.0, -3.0, 0.0], [-1.0, -6.0, 4.0, 3.0, 6.0], [2.0, -4.0, -2.0, -4.0, 0.0], [3.0, 5.0, -2.0, -3.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -5.0, -3.0, -1.0, -7.0], [-6.0, 6.0, 1.0, -2.0, -4.0], [-5.0, 6.0, -5.0, -6.0, -4.0], [-1.0, 1.0, -7.0, 3.0, 7.0]], [[1.0, 7.0, -3.0, 6.0, 0.0], [-1.0, 2.0, 6.0, 5.0, 5.0], [-7.0, 1.0, 5.0, -1.0, -7.0], [1.0, -4.0, 0.0, 1.0, 3.0]], [[-4.0, -4.0, 1.0, 3.0, -5.0], [0.0, 0.0, -3.0, -2.0, 6.0], [-6.0, -5.0, 2.0, -6.0, 0.0], [-2.0, -6.0, -4.0, -4.0, -2.0]]], [[[-1.0, -2.0, 3.0, 0.0, -2.0], [0.0, 3.0, -6.0, 2.0, 3.0], [4.0, 6.0, -1.0, 3.0, -3.0], [3.0, 7.0, 5.0, -3.0, 3.0]], [[-1.0, 5.0, -7.0, 1.0, 1.0], [-4.0, 5.0, -3.0, 0.0, -5.0], [-1.0, 1.0, 0.0, 7.0, -1.0], [-5.0, -6.0, -7.0, -5.0, 5.0]], [[-6.0, 0.0, 6.0, 3.0, 2.0], [3.0, 4.0, 2.0, 0.0, -1.0], [5.0, 0.0, -4.0, -3.0, -6.0], [7.0, 1.0, -6.0, 0.0, -3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[10.0, 19.0, 39.0, 53.0, 11.0], [3.0, 57.0, -43.0, 9.0, -33.0], [-19.0, 28.0, 45.0, 61.0, -4.0], [-71.0, -16.0, -35.0, 37.0, 62.0]], [[57.0, -13.0, 12.0, -10.0, -43.0], [-36.0, -19.0, 14.0, 10.0, 52.0], [49.0, -56.0, 97.0, 0.0, -18.0], [24.0, 8.0, 56.0, 18.0, -8.0]]])+(1.-msk_ref)*numpy.array([[[61.0, -15.0, -56.0, -14.0, -56.0], [-60.0, 33.0, -18.0, 8.0, -7.0], [-53.0, 68.0, -2.0, 5.0, -15.0], [-37.0, 20.0, 11.0, 7.0, 85.0]], [[-19.0, 0.0, -5.0, -18.0, 10.0], [-14.0, 45.0, -62.0, -6.0, -49.0], [64.0, 42.0, -37.0, 67.0, -5.0], [-1.0, 29.0, -21.0, -41.0, 31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank1_expandedData_rank3_offset0(self):
      arg0=numpy.array([-6.0, 6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, -1.0], [-5.0, 3.0]], [[2.0, 3.0], [2.0, -2.0]], [[0.0, 0.0], [-5.0, -1.0]], [[-2.0, 0.0], [-6.0, -4.0]], [[-3.0, -3.0], [7.0, -7.0]], [[5.0, 0.0], [-5.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 7.0], [3.0, 6.0]], [[-7.0, -4.0], [-7.0, -4.0]], [[0.0, 3.0], [3.0, 0.0]], [[-7.0, -3.0], [7.0, -2.0]], [[-6.0, 0.0], [6.0, -6.0]], [[7.0, 0.0], [2.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-42.0, 6.0], [30.0, -18.0]], [[-12.0, -18.0], [-12.0, 12.0]], [[0.0, 0.0], [30.0, 6.0]], [[12.0, 0.0], [36.0, 24.0]], [[18.0, 18.0], [-42.0, 42.0]], [[-30.0, 0.0], [30.0, 18.0]]], [[[42.0, -6.0], [-30.0, 18.0]], [[12.0, 18.0], [12.0, -12.0]], [[0.0, 0.0], [-30.0, -6.0]], [[-12.0, 0.0], [-36.0, -24.0]], [[-18.0, -18.0], [42.0, -42.0]], [[30.0, 0.0], [-30.0, -18.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -42.0], [-18.0, -36.0]], [[42.0, 24.0], [42.0, 24.0]], [[0.0, -18.0], [-18.0, 0.0]], [[42.0, 18.0], [-42.0, 12.0]], [[36.0, 0.0], [-36.0, 36.0]], [[-42.0, 0.0], [-12.0, 30.0]]], [[[0.0, 42.0], [18.0, 36.0]], [[-42.0, -24.0], [-42.0, -24.0]], [[0.0, 18.0], [18.0, 0.0]], [[-42.0, -18.0], [42.0, -12.0]], [[-36.0, 0.0], [36.0, -36.0]], [[42.0, 0.0], [12.0, -30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank4_offset1(self):
      arg0=numpy.array([[-2.0, -6.0, 2.0], [-5.0, 6.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, 2.0], [5.0, 4.0]], [[-3.0, 0.0], [-4.0, 0.0]], [[-1.0, -1.0], [-2.0, 5.0]], [[-7.0, 0.0], [-4.0, -4.0]], [[-1.0, 0.0], [7.0, -2.0]], [[-2.0, 3.0], [-3.0, -2.0]]], [[[0.0, -4.0], [0.0, -3.0]], [[-6.0, -1.0], [1.0, -3.0]], [[3.0, -7.0], [6.0, -6.0]], [[7.0, 5.0], [-2.0, 3.0]], [[6.0, 1.0], [-6.0, 7.0]], [[7.0, -3.0], [2.0, 0.0]]], [[[7.0, 5.0], [2.0, -2.0]], [[-4.0, -2.0], [-7.0, 0.0]], [[-4.0, -2.0], [5.0, -1.0]], [[-6.0, 6.0], [-4.0, -2.0]], [[6.0, -2.0], [5.0, -1.0]], [[-4.0, 0.0], [-5.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, 7.0], [2.0, -7.0]], [[3.0, 4.0], [5.0, 3.0]], [[6.0, -5.0], [6.0, -7.0]], [[-2.0, 6.0], [5.0, 3.0]], [[-3.0, -7.0], [-5.0, 0.0]], [[7.0, 4.0], [4.0, -2.0]]], [[[2.0, -4.0], [-2.0, -6.0]], [[-6.0, -7.0], [0.0, 6.0]], [[5.0, 6.0], [3.0, 5.0]], [[2.0, 6.0], [-4.0, 6.0]], [[-3.0, -4.0], [-6.0, -2.0]], [[2.0, 1.0], [6.0, 2.0]]], [[[2.0, 3.0], [7.0, 5.0]], [[-4.0, 3.0], [4.0, 1.0]], [[2.0, -7.0], [-4.0, -6.0]], [[-4.0, 0.0], [-2.0, 7.0]], [[-7.0, 6.0], [2.0, 4.0]], [[6.0, 4.0], [4.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 30.0], [-6.0, 6.0]], [[34.0, 2.0], [-12.0, 18.0]], [[-24.0, 40.0], [-22.0, 24.0]], [[-40.0, -18.0], [12.0, -14.0]], [[-22.0, -10.0], [32.0, -40.0]], [[-46.0, 12.0], [-16.0, 10.0]]], [[[-4.0, -19.0], [-19.0, -44.0]], [[-33.0, -12.0], [5.0, -18.0]], [[11.0, -43.0], [61.0, -64.0]], [[59.0, 48.0], [-4.0, 32.0]], [[59.0, 0.0], [-56.0, 49.0]], [[40.0, -33.0], [12.0, 19.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 16.0], [22.0, 60.0]], [[22.0, 40.0], [-2.0, -40.0]], [[-38.0, -40.0], [-38.0, -28.0]], [[-16.0, -48.0], [10.0, -28.0]], [[10.0, 50.0], [50.0, 20.0]], [[-14.0, -6.0], [-36.0, -6.0]]], [[[48.0, -50.0], [-1.0, 14.0]], [[-63.0, -53.0], [-13.0, 24.0]], [[6.0, 40.0], [-24.0, 47.0]], [[10.0, 6.0], [-55.0, 42.0]], [[-24.0, 29.0], [-5.0, 0.0]], [[-5.0, -2.0], [28.0, 25.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_float_rank0_offset0(self):
      arg0=numpy.array([[-4.0, 0.0, -3.0, -4.0, -6.0], [3.0, 3.0, -3.0, 2.0, -3.0], [-4.0, 2.0, 5.0, -6.0, 4.0], [-3.0, 5.0, 1.0, -6.0, 0.0]])
      arg1=0.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-0.0, 0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0, -0.0], [-0.0, 0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0, 0.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank0_offset0(self):
      arg0=numpy.array([[-6.0, -7.0, -2.0, 4.0, -3.0], [6.0, -2.0, 4.0, 6.0, 4.0], [-5.0, 0.0, -1.0, -3.0, -7.0], [0.0, -5.0, 7.0, -3.0, 2.0]])
      arg1=numpy.array(-2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[12.0, 14.0, 4.0, -8.0, 6.0], [-12.0, 4.0, -8.0, -12.0, -8.0], [10.0, -0.0, 2.0, 6.0, 14.0], [-0.0, 10.0, -14.0, 6.0, -4.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank1_offset1(self):
      arg0=numpy.array([[[6.0, -7.0, 0.0], [0.0, 1.0, 6.0], [-1.0, 6.0, 6.0], [-4.0, 2.0, 4.0], [-6.0, 6.0, 5.0]], [[-5.0, 6.0, 4.0], [6.0, -5.0, -7.0], [-3.0, 1.0, -6.0], [-3.0, 2.0, 0.0], [1.0, -1.0, 5.0]], [[4.0, 0.0, -6.0], [-1.0, 5.0, -6.0], [0.0, -6.0, -3.0], [0.0, 4.0, 7.0], [-1.0, -4.0, 0.0]], [[-2.0, -4.0, -1.0], [0.0, 0.0, 5.0], [-4.0, -4.0, 0.0], [-5.0, 1.0, -7.0], [6.0, 4.0, 0.0]]])
      arg1=numpy.array([-7.0, -5.0, -5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[-7.0, -35.0, -53.0, -2.0, -13.0], [-15.0, 18.0, 46.0, 11.0, -27.0], [2.0, 12.0, 45.0, -55.0, 27.0], [39.0, -25.0, 48.0, 65.0, -62.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank2_offset2(self):
      arg0=numpy.array([[[[-4.0, 1.0, -1.0], [-1.0, -4.0, 1.0]], [[0.0, -7.0, 0.0], [0.0, 6.0, 2.0]], [[-6.0, 7.0, 1.0], [7.0, -5.0, 6.0]], [[-2.0, 0.0, 4.0], [2.0, -3.0, -3.0]], [[3.0, 5.0, -4.0], [4.0, 4.0, 2.0]]], [[[-3.0, -3.0, 7.0], [7.0, -1.0, 5.0]], [[-3.0, -7.0, -7.0], [7.0, -5.0, 5.0]], [[2.0, -4.0, -1.0], [-4.0, -2.0, 0.0]], [[6.0, -5.0, -3.0], [6.0, -1.0, 0.0]], [[-6.0, -1.0, 5.0], [0.0, 6.0, 3.0]]], [[[4.0, 6.0, -2.0], [3.0, -6.0, -4.0]], [[2.0, -4.0, 1.0], [0.0, -6.0, -4.0]], [[-7.0, -1.0, -1.0], [-1.0, 5.0, -6.0]], [[6.0, -7.0, -7.0], [0.0, -3.0, -4.0]], [[2.0, 3.0, 3.0], [1.0, 6.0, -1.0]]], [[[-2.0, 5.0, -2.0], [7.0, 1.0, -6.0]], [[-7.0, -4.0, 1.0], [3.0, 0.0, 5.0]], [[-3.0, -4.0, 6.0], [-7.0, -3.0, 0.0]], [[6.0, -7.0, 7.0], [-5.0, 6.0, -6.0]], [[1.0, -1.0, -2.0], [7.0, -1.0, 0.0]]]])
      arg1=numpy.array([[2.0, 1.0, 3.0], [-7.0, -6.0, -1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[20.0, -45.0, -27.0, 15.0, -55.0], [-36.0, -58.0, 37.0, -38.0, -37.0], [27.0, 43.0, -35.0, 6.0, -26.0], [-54.0, -41.0, 75.0, 31.0, -48.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank1_offset0(self):
      arg0=numpy.array([[5.0, -7.0, -6.0, -3.0, -3.0], [5.0, 6.0, -4.0, 7.0, 4.0], [6.0, 2.0, 3.0, 6.0, 7.0], [1.0, 5.0, -5.0, -5.0, 7.0]])
      arg1=numpy.array([-6.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-30.0, 20.0], [42.0, -28.0], [36.0, -24.0], [18.0, -12.0], [18.0, -12.0]], [[-30.0, 20.0], [-36.0, 24.0], [24.0, -16.0], [-42.0, 28.0], [-24.0, 16.0]], [[-36.0, 24.0], [-12.0, 8.0], [-18.0, 12.0], [-36.0, 24.0], [-42.0, 28.0]], [[-6.0, 4.0], [-30.0, 20.0], [30.0, -20.0], [30.0, -20.0], [-42.0, 28.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank2_offset1(self):
      arg0=numpy.array([[[-5.0, -2.0, 7.0], [-6.0, 2.0, 1.0], [-6.0, 3.0, -5.0], [-1.0, 4.0, -5.0], [-5.0, 7.0, -5.0]], [[2.0, 2.0, -6.0], [-7.0, 0.0, 2.0], [-5.0, 1.0, -2.0], [-6.0, 0.0, -3.0], [-7.0, -2.0, -1.0]], [[-4.0, 7.0, -3.0], [-3.0, -2.0, 5.0], [-5.0, -1.0, -4.0], [-3.0, 3.0, 7.0], [-3.0, -5.0, -1.0]], [[-2.0, -6.0, -3.0], [-6.0, 5.0, -2.0], [-1.0, -5.0, -7.0], [5.0, 2.0, 5.0], [1.0, -1.0, 4.0]]])
      arg1=numpy.array([[3.0, 7.0], [5.0, 0.0], [4.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[3.0, 0.0], [-4.0, -37.0], [-23.0, -67.0], [-3.0, -32.0], [0.0, -60.0]], [[-8.0, -16.0], [-13.0, -39.0], [-18.0, -45.0], [-30.0, -57.0], [-35.0, -54.0]], [[11.0, -43.0], [1.0, 4.0], [-36.0, -55.0], [34.0, 14.0], [-38.0, -26.0]], [[-48.0, -29.0], [-1.0, -52.0], [-56.0, -42.0], [45.0, 60.0], [14.0, 27.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank3_offset2(self):
      arg0=numpy.array([[[[-6.0, -4.0, 0.0], [0.0, -1.0, 2.0]], [[0.0, -3.0, 4.0], [2.0, -2.0, 1.0]], [[-3.0, 7.0, -3.0], [5.0, -4.0, -3.0]], [[3.0, 5.0, 1.0], [-5.0, -6.0, 3.0]], [[-5.0, 0.0, 1.0], [-7.0, 2.0, 6.0]]], [[[-2.0, 6.0, 6.0], [0.0, 3.0, -2.0]], [[-1.0, 4.0, -7.0], [6.0, -5.0, 1.0]], [[-6.0, -3.0, -6.0], [7.0, -6.0, 5.0]], [[-4.0, -2.0, 5.0], [3.0, -2.0, 5.0]], [[-7.0, -3.0, -6.0], [-7.0, 4.0, 1.0]]], [[[6.0, -1.0, 0.0], [0.0, 2.0, -7.0]], [[1.0, 5.0, -5.0], [3.0, 7.0, -5.0]], [[-3.0, 6.0, -6.0], [6.0, 7.0, 0.0]], [[0.0, -7.0, 2.0], [-6.0, 1.0, -6.0]], [[3.0, -4.0, -4.0], [1.0, -5.0, 3.0]]], [[[-3.0, -1.0, -2.0], [-7.0, 0.0, 6.0]], [[-1.0, 2.0, -7.0], [-4.0, 6.0, 0.0]], [[-6.0, 1.0, 7.0], [3.0, -4.0, 1.0]], [[4.0, 0.0, 3.0], [7.0, -3.0, 2.0]], [[-2.0, 3.0, 6.0], [0.0, 3.0, 2.0]]]])
      arg1=numpy.array([[[0.0, -4.0], [0.0, 1.0], [5.0, 1.0]], [[-7.0, -1.0], [4.0, -7.0], [7.0, -1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[10.0, 25.0], [5.0, 12.0], [-87.0, 42.0], [37.0, 38.0], [104.0, 8.0]], [[28.0, 1.0], [-90.0, 29.0], [-68.0, 45.0], [31.0, 25.0], [42.0, -3.0]], [[-41.0, -32.0], [-53.0, -51.0], [-44.0, -43.0], [14.0, 0.0], [-26.0, 11.0]], [[81.0, 10.0], [17.0, -39.0], [5.0, 56.0], [-32.0, -1.0], [56.0, -6.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_array_rank2_offset0(self):
      arg0=numpy.array([[6.0, -4.0, 0.0, 4.0, -6.0], [1.0, -1.0, 7.0, 6.0, 6.0], [-2.0, -7.0, 1.0, -3.0, -7.0], [0.0, -6.0, 7.0, -2.0, 6.0]])
      arg1=numpy.array([[3.0, -6.0, -1.0, 2.0, -6.0], [-7.0, -6.0, 0.0, -2.0, 5.0], [-2.0, 6.0, 7.0, -6.0, -4.0], [0.0, 6.0, 4.0, 0.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[18.0, -36.0, -6.0, 12.0, -36.0], [-42.0, -36.0, 0.0, -12.0, 30.0], [-12.0, 36.0, 42.0, -36.0, -24.0], [0.0, 36.0, 24.0, 0.0, 0.0]], [[-12.0, 24.0, 4.0, -8.0, 24.0], [28.0, 24.0, 0.0, 8.0, -20.0], [8.0, -24.0, -28.0, 24.0, 16.0], [0.0, -24.0, -16.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[12.0, -24.0, -4.0, 8.0, -24.0], [-28.0, -24.0, 0.0, -8.0, 20.0], [-8.0, 24.0, 28.0, -24.0, -16.0], [0.0, 24.0, 16.0, 0.0, 0.0]], [[-18.0, 36.0, 6.0, -12.0, 36.0], [42.0, 36.0, 0.0, 12.0, -30.0], [12.0, -36.0, -42.0, 36.0, 24.0], [0.0, -36.0, -24.0, 0.0, 0.0]]], [[[3.0, -6.0, -1.0, 2.0, -6.0], [-7.0, -6.0, 0.0, -2.0, 5.0], [-2.0, 6.0, 7.0, -6.0, -4.0], [0.0, 6.0, 4.0, 0.0, 0.0]], [[-3.0, 6.0, 1.0, -2.0, 6.0], [7.0, 6.0, 0.0, 2.0, -5.0], [2.0, -6.0, -7.0, 6.0, 4.0], [0.0, -6.0, -4.0, 0.0, 0.0]], [[21.0, -42.0, -7.0, 14.0, -42.0], [-49.0, -42.0, 0.0, -14.0, 35.0], [-14.0, 42.0, 49.0, -42.0, -28.0], [0.0, 42.0, 28.0, 0.0, 0.0]], [[18.0, -36.0, -6.0, 12.0, -36.0], [-42.0, -36.0, 0.0, -12.0, 30.0], [-12.0, 36.0, 42.0, -36.0, -24.0], [0.0, 36.0, 24.0, 0.0, 0.0]], [[18.0, -36.0, -6.0, 12.0, -36.0], [-42.0, -36.0, 0.0, -12.0, 30.0], [-12.0, 36.0, 42.0, -36.0, -24.0], [0.0, 36.0, 24.0, 0.0, 0.0]]], [[[-6.0, 12.0, 2.0, -4.0, 12.0], [14.0, 12.0, 0.0, 4.0, -10.0], [4.0, -12.0, -14.0, 12.0, 8.0], [0.0, -12.0, -8.0, 0.0, 0.0]], [[-21.0, 42.0, 7.0, -14.0, 42.0], [49.0, 42.0, 0.0, 14.0, -35.0], [14.0, -42.0, -49.0, 42.0, 28.0], [0.0, -42.0, -28.0, 0.0, 0.0]], [[3.0, -6.0, -1.0, 2.0, -6.0], [-7.0, -6.0, 0.0, -2.0, 5.0], [-2.0, 6.0, 7.0, -6.0, -4.0], [0.0, 6.0, 4.0, 0.0, 0.0]], [[-9.0, 18.0, 3.0, -6.0, 18.0], [21.0, 18.0, 0.0, 6.0, -15.0], [6.0, -18.0, -21.0, 18.0, 12.0], [0.0, -18.0, -12.0, 0.0, 0.0]], [[-21.0, 42.0, 7.0, -14.0, 42.0], [49.0, 42.0, 0.0, 14.0, -35.0], [14.0, -42.0, -49.0, 42.0, 28.0], [0.0, -42.0, -28.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-18.0, 36.0, 6.0, -12.0, 36.0], [42.0, 36.0, 0.0, 12.0, -30.0], [12.0, -36.0, -42.0, 36.0, 24.0], [0.0, -36.0, -24.0, 0.0, 0.0]], [[21.0, -42.0, -7.0, 14.0, -42.0], [-49.0, -42.0, 0.0, -14.0, 35.0], [-14.0, 42.0, 49.0, -42.0, -28.0], [0.0, 42.0, 28.0, 0.0, 0.0]], [[-6.0, 12.0, 2.0, -4.0, 12.0], [14.0, 12.0, 0.0, 4.0, -10.0], [4.0, -12.0, -14.0, 12.0, 8.0], [0.0, -12.0, -8.0, 0.0, 0.0]], [[18.0, -36.0, -6.0, 12.0, -36.0], [-42.0, -36.0, 0.0, -12.0, 30.0], [-12.0, 36.0, 42.0, -36.0, -24.0], [0.0, 36.0, 24.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank3_offset1(self):
      arg0=numpy.array([[[-1.0, -2.0, -7.0], [-3.0, 0.0, -2.0], [0.0, 1.0, -5.0], [0.0, -6.0, 5.0], [6.0, -1.0, -4.0]], [[0.0, 1.0, 6.0], [0.0, -7.0, -7.0], [1.0, 3.0, 7.0], [6.0, 7.0, 0.0], [7.0, 0.0, 0.0]], [[6.0, 3.0, -3.0], [0.0, 2.0, 0.0], [4.0, -1.0, 6.0], [3.0, 3.0, 0.0], [7.0, 0.0, -6.0]], [[3.0, 5.0, -4.0], [-6.0, 0.0, 6.0], [-5.0, -2.0, -5.0], [-2.0, 4.0, 0.0], [-3.0, -5.0, -6.0]]])
      arg1=numpy.array([[[-7.0, 4.0, -3.0, 4.0, 5.0], [-6.0, -2.0, 5.0, -1.0, 7.0], [-2.0, -7.0, -2.0, 4.0, -5.0], [5.0, 3.0, -1.0, 3.0, 4.0]], [[3.0, -2.0, -1.0, 4.0, 3.0], [-6.0, 2.0, -3.0, -4.0, 4.0], [2.0, 2.0, -1.0, -1.0, -2.0], [2.0, -7.0, 7.0, 6.0, -2.0]], [[7.0, -4.0, 0.0, -4.0, -3.0], [-3.0, 6.0, 4.0, -6.0, 0.0], [1.0, 2.0, 6.0, 4.0, 4.0], [0.0, 0.0, 5.0, -7.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[-48.0, 28.0, 5.0, 16.0, 10.0], [39.0, -44.0, -27.0, 51.0, -15.0], [-9.0, -11.0, -38.0, -30.0, -19.0], [-9.0, 11.0, -48.0, 34.0, -14.0]], [[7.0, -4.0, 9.0, -4.0, -9.0], [24.0, -6.0, -23.0, 15.0, -21.0], [4.0, 17.0, -6.0, -20.0, 7.0], [-15.0, -9.0, -7.0, 5.0, -16.0]], [[-32.0, 18.0, -1.0, 24.0, 18.0], [9.0, -28.0, -23.0, 26.0, 4.0], [-3.0, -8.0, -31.0, -21.0, -22.0], [2.0, -7.0, -18.0, 41.0, -12.0]], [[17.0, -8.0, 6.0, -44.0, -33.0], [21.0, 18.0, 38.0, -6.0, -24.0], [-7.0, -2.0, 36.0, 26.0, 32.0], [-12.0, 42.0, -17.0, -71.0, 22.0]], [[-73.0, 42.0, -17.0, 36.0, 39.0], [-18.0, -38.0, 17.0, 22.0, 38.0], [-18.0, -52.0, -35.0, 9.0, -44.0], [28.0, 25.0, -33.0, 40.0, 18.0]]], [[[45.0, -26.0, -1.0, -20.0, -15.0], [-24.0, 38.0, 21.0, -40.0, 4.0], [8.0, 14.0, 35.0, 23.0, 22.0], [2.0, -7.0, 37.0, -36.0, 10.0]], [[-70.0, 42.0, 7.0, 0.0, 0.0], [63.0, -56.0, -7.0, 70.0, -28.0], [-21.0, -28.0, -35.0, -21.0, -14.0], [-14.0, 49.0, -84.0, 7.0, 0.0]], [[51.0, -30.0, -6.0, -12.0, -7.0], [-45.0, 46.0, 24.0, -55.0, 19.0], [11.0, 13.0, 37.0, 29.0, 17.0], [11.0, -18.0, 55.0, -28.0, 12.0]], [[-21.0, 10.0, -25.0, 52.0, 51.0], [-78.0, 2.0, 9.0, -34.0, 70.0], [2.0, -28.0, -19.0, 17.0, -44.0], [44.0, -31.0, 43.0, 60.0, 10.0]], [[-49.0, 28.0, -21.0, 28.0, 35.0], [-42.0, -14.0, 35.0, -7.0, 49.0], [-14.0, -49.0, -14.0, 28.0, -35.0], [35.0, 21.0, -7.0, 21.0, 28.0]]], [[[-54.0, 30.0, -21.0, 48.0, 48.0], [-45.0, -24.0, 9.0, 0.0, 54.0], [-9.0, -42.0, -33.0, 9.0, -48.0], [36.0, -3.0, 0.0, 57.0, 12.0]], [[6.0, -4.0, -2.0, 8.0, 6.0], [-12.0, 4.0, -6.0, -8.0, 8.0], [4.0, 4.0, -2.0, -2.0, -4.0], [4.0, -14.0, 14.0, 12.0, -4.0]], [[11.0, -6.0, -11.0, -12.0, -1.0], [-36.0, 26.0, 47.0, -36.0, 24.0], [-4.0, -18.0, 29.0, 41.0, 6.0], [18.0, 19.0, 19.0, -36.0, 30.0]], [[-12.0, 6.0, -12.0, 24.0, 24.0], [-36.0, 0.0, 6.0, -15.0, 33.0], [0.0, -15.0, -9.0, 9.0, -21.0], [21.0, -12.0, 18.0, 27.0, 6.0]], [[-91.0, 52.0, -21.0, 52.0, 53.0], [-24.0, -50.0, 11.0, 29.0, 49.0], [-20.0, -61.0, -50.0, 4.0, -59.0], [35.0, 21.0, -37.0, 63.0, 16.0]]], [[[-34.0, 18.0, -14.0, 48.0, 42.0], [-36.0, -20.0, -16.0, 1.0, 41.0], [0.0, -19.0, -35.0, -9.0, -41.0], [25.0, -26.0, 12.0, 67.0, -6.0]], [[84.0, -48.0, 18.0, -48.0, -48.0], [18.0, 48.0, -6.0, -30.0, -42.0], [18.0, 54.0, 48.0, 0.0, 54.0], [-30.0, -18.0, 36.0, -60.0, -12.0]], [[-6.0, 4.0, 17.0, -8.0, -16.0], [57.0, -24.0, -39.0, 43.0, -43.0], [1.0, 21.0, -18.0, -38.0, 9.0], [-29.0, -1.0, -34.0, 8.0, -26.0]], [[26.0, -16.0, 2.0, 8.0, 2.0], [-12.0, 12.0, -22.0, -14.0, 2.0], [12.0, 22.0, 0.0, -12.0, 2.0], [-2.0, -34.0, 30.0, 18.0, -16.0]], [[-36.0, 22.0, 14.0, -8.0, -12.0], [66.0, -40.0, -24.0, 59.0, -41.0], [-10.0, -1.0, -25.0, -31.0, 1.0], [-25.0, 26.0, -62.0, 3.0, -14.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank4_offset2(self):
      arg0=numpy.array([[[[-1.0, -7.0, 5.0], [2.0, 1.0, 0.0]], [[-4.0, 0.0, 0.0], [-1.0, 3.0, 2.0]], [[-5.0, 3.0, 3.0], [4.0, -5.0, 3.0]], [[0.0, -6.0, -7.0], [-2.0, -2.0, 5.0]], [[-3.0, 7.0, 7.0], [-7.0, 0.0, 7.0]]], [[[-2.0, -3.0, -1.0], [0.0, 0.0, -4.0]], [[0.0, -1.0, 0.0], [-7.0, 4.0, -4.0]], [[5.0, -2.0, 3.0], [-7.0, 1.0, 0.0]], [[-7.0, 3.0, -2.0], [-7.0, 5.0, -6.0]], [[5.0, -1.0, -1.0], [6.0, -4.0, -6.0]]], [[[-6.0, -1.0, 0.0], [5.0, 5.0, 4.0]], [[-1.0, 0.0, -4.0], [-4.0, -3.0, -7.0]], [[-6.0, -1.0, 0.0], [4.0, -3.0, -6.0]], [[6.0, -1.0, 6.0], [-5.0, 5.0, -4.0]], [[6.0, -1.0, 6.0], [2.0, -4.0, 1.0]]], [[[4.0, 7.0, -6.0], [0.0, 4.0, -5.0]], [[3.0, -2.0, -3.0], [-2.0, -3.0, 4.0]], [[4.0, 7.0, 1.0], [7.0, 3.0, 1.0]], [[3.0, -5.0, -7.0], [-1.0, -4.0, -7.0]], [[-5.0, 6.0, 6.0], [5.0, 7.0, 0.0]]]])
      arg1=numpy.array([[[[6.0, 3.0, -6.0, 7.0, -5.0], [7.0, 7.0, 0.0, 5.0, 2.0], [1.0, -5.0, -3.0, -3.0, 1.0], [1.0, -7.0, 2.0, -2.0, -1.0]], [[7.0, -7.0, -7.0, -7.0, 0.0], [7.0, 1.0, -1.0, 0.0, -4.0], [1.0, 1.0, 0.0, -4.0, 7.0], [-4.0, 0.0, -5.0, 6.0, 4.0]], [[-4.0, 2.0, 1.0, 4.0, 6.0], [-7.0, 4.0, -3.0, 5.0, -5.0], [5.0, -2.0, 3.0, -6.0, 4.0], [3.0, 5.0, -5.0, -3.0, 4.0]]], [[[3.0, 6.0, -1.0, -2.0, -6.0], [-5.0, -3.0, 4.0, -3.0, 0.0], [-1.0, -4.0, 0.0, 3.0, 0.0], [3.0, 7.0, 5.0, 1.0, 3.0]], [[3.0, -1.0, -6.0, 7.0, 0.0], [2.0, 7.0, 3.0, 0.0, 3.0], [0.0, 2.0, -3.0, 2.0, 4.0], [6.0, 3.0, -3.0, -3.0, -7.0]], [[-7.0, 7.0, 0.0, 3.0, 4.0], [-3.0, 6.0, 0.0, 4.0, -2.0], [2.0, 3.0, 6.0, 3.0, -3.0], [-3.0, 6.0, -2.0, -2.0, -7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[[-66.0, 67.0, 52.0, 65.0, 23.0], [-99.0, 7.0, 3.0, 14.0, 4.0], [15.0, -18.0, 15.0, 9.0, -26.0], [54.0, 49.0, 15.0, -56.0, -8.0]], [[-32.0, -7.0, 7.0, 1.0, 34.0], [-23.0, 8.0, 5.0, -9.0, -3.0], [1.0, 36.0, 15.0, 21.0, 2.0], [5.0, 42.0, -26.0, -6.0, -34.0]], [[-45.0, 20.0, 38.0, -78.0, 31.0], [-74.0, -49.0, -11.0, -10.0, -58.0], [15.0, 5.0, 57.0, -4.0, -1.0], [-35.0, 81.0, -11.0, 32.0, 55.0]], [[-61.0, 53.0, 49.0, 19.0, -10.0], [-2.0, -12.0, 13.0, -9.0, 43.0], [-29.0, 27.0, 15.0, 71.0, -93.0], [-30.0, -25.0, 51.0, -21.0, -79.0]], [[-67.0, -37.0, -17.0, -7.0, 127.0], [-7.0, 77.0, -56.0, 69.0, -83.0], [60.0, 57.0, 72.0, -61.0, 53.0], [-52.0, 49.0, -125.0, 6.0, -11.0]]], [[[-1.0, -15.0, 32.0, -9.0, -12.0], [-16.0, -45.0, 6.0, -31.0, 21.0], [-18.0, -3.0, -21.0, 12.0, -15.0], [19.0, -15.0, 24.0, -3.0, 14.0]], [[12.0, -67.0, -10.0, 37.0, 26.0], [48.0, 24.0, -15.0, 5.0, 24.0], [-2.0, 23.0, -36.0, -21.0, 21.0], [19.0, -61.0, -34.0, -17.0, -25.0]], [[-14.0, -8.0, -12.0, 82.0, 35.0], [37.0, 73.0, -32.0, 61.0, 6.0], [25.0, -3.0, -9.0, -44.0, 7.0], [7.0, -66.0, -33.0, -41.0, -29.0]], [[23.0, -135.0, -4.0, -47.0, 41.0], [49.0, -34.0, -10.0, -48.0, 11.0], [-19.0, 62.0, -36.0, -8.0, 44.0], [2.0, -31.0, -57.0, 28.0, -3.0]], [[75.0, 18.0, -6.0, -20.0, -91.0], [15.0, -52.0, 16.0, -22.0, 19.0], [-19.0, -74.0, -42.0, -13.0, -4.0], [18.0, -46.0, 74.0, 17.0, 75.0]]], [[[-41.0, 42.0, 8.0, 2.0, 16.0], [-76.0, 1.0, 36.0, -29.0, -1.0], [-4.0, 31.0, 27.0, 59.0, -5.0], [31.0, 116.0, -5.0, -12.0, -46.0]], [[38.0, -81.0, 24.0, -57.0, -23.0], [56.0, -74.0, -13.0, -41.0, 23.0], [-31.0, 2.0, -42.0, -12.0, -8.0], [-22.0, -92.0, 21.0, 33.0, 43.0]], [[2.0, -26.0, 57.0, -82.0, -18.0], [-57.0, -112.0, 8.0, -66.0, -5.0], [-23.0, -11.0, -9.0, 10.0, -7.0], [10.0, 25.0, 34.0, 31.0, 77.0]], [[33.0, -26.0, -48.0, 106.0, 20.0], [40.0, 91.0, -22.0, 59.0, 9.0], [32.0, -25.0, -39.0, -67.0, 55.0], [55.0, -56.0, -45.0, -48.0, -8.0]], [[-8.0, 60.0, -1.0, 44.0, -2.0], [-28.0, 37.0, -21.0, 58.0, -28.0], [35.0, -56.0, 18.0, -49.0, 4.0], [7.0, -4.0, 7.0, -24.0, 41.0]]], [[[144.0, -88.0, -103.0, -32.0, -76.0], [142.0, 9.0, 23.0, -30.0, 32.0], [-29.0, -8.0, -72.0, -11.0, 60.0], [-3.0, -76.0, 1.0, 50.0, 7.0]], [[-27.0, 36.0, 13.0, 18.0, -5.0], [20.0, 16.0, -6.0, 22.0, 12.0], [-4.0, 3.0, 15.0, 17.0, -47.0], [-34.0, -35.0, 22.0, -10.0, -36.0]], [[92.0, 11.0, -97.0, -7.0, -52.0], [38.0, 45.0, 27.0, 8.0, -18.0], [11.0, -34.0, -12.0, -16.0, 66.0], [15.0, 41.0, -8.0, 27.0, 21.0]], [[45.0, -21.0, 35.0, -19.0, -79.0], [53.0, -79.0, 10.0, -45.0, 63.0], [-50.0, -31.0, -60.0, 21.0, -55.0], [-4.0, -117.0, 87.0, 10.0, 23.0]], [[24.0, -22.0, -53.0, -14.0, 31.0], [-46.0, 29.0, 17.0, -10.0, -43.0], [26.0, 13.0, 12.0, -16.0, 89.0], [46.0, 121.0, -66.0, 12.0, 19.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank0_offset0(self):
      arg0=numpy.array([[2.0, 4.0, -1.0, -5.0, 0.0], [-4.0, -3.0, 0.0, 7.0, -7.0], [-4.0, -3.0, 2.0, 7.0, 0.0], [-5.0, 4.0, 6.0, 1.0, 7.0]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[14.0, 28.0, -7.0, -35.0, 0.0], [-28.0, -21.0, 0.0, 49.0, -49.0], [-28.0, -21.0, 14.0, 49.0, 0.0], [-35.0, 28.0, 42.0, 7.0, 49.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[0.0, 4.0, -3.0], [-4.0, -7.0, 5.0], [-5.0, -1.0, 6.0], [-4.0, 6.0, -2.0], [-6.0, -2.0, 0.0]], [[6.0, -3.0, -3.0], [-5.0, 4.0, -3.0], [-1.0, 7.0, 3.0], [-3.0, -1.0, 1.0], [7.0, 3.0, 6.0]], [[-1.0, 4.0, -6.0], [-4.0, 4.0, -6.0], [6.0, -4.0, -3.0], [1.0, -2.0, 0.0], [-4.0, -3.0, -7.0]], [[-1.0, 4.0, 5.0], [-6.0, 4.0, 6.0], [-7.0, 0.0, 7.0], [-1.0, 1.0, -6.0], [-7.0, -5.0, -2.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([0.0, -2.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[13.0, -21.0, -40.0, 2.0, 4.0], [27.0, 13.0, -35.0, -5.0, -48.0], [34.0, 34.0, 29.0, 4.0, 55.0], [-43.0, -50.0, -49.0, 40.0, 24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[[6.0, -6.0, 0.0], [4.0, -1.0, -2.0]], [[-6.0, 7.0, -2.0], [4.0, -3.0, -7.0]], [[-2.0, -6.0, 0.0], [5.0, -6.0, 2.0]], [[0.0, 2.0, -4.0], [0.0, -5.0, 5.0]], [[2.0, -4.0, -7.0], [6.0, 3.0, -6.0]]], [[[4.0, -5.0, -3.0], [-2.0, 7.0, 1.0]], [[-3.0, -4.0, -4.0], [-1.0, 0.0, 0.0]], [[0.0, 2.0, -1.0], [7.0, 7.0, 3.0]], [[-6.0, -6.0, -3.0], [-2.0, 0.0, -6.0]], [[1.0, 5.0, 1.0], [-2.0, -4.0, -6.0]]], [[[0.0, 5.0, 0.0], [6.0, 5.0, 3.0]], [[-2.0, 0.0, -1.0], [0.0, 5.0, 6.0]], [[7.0, 1.0, -3.0], [-5.0, 5.0, 0.0]], [[0.0, -3.0, -4.0], [-2.0, 5.0, 5.0]], [[7.0, 6.0, 0.0], [-7.0, 0.0, 7.0]]], [[[-4.0, 2.0, 0.0], [-6.0, 5.0, 1.0]], [[0.0, 2.0, -3.0], [7.0, -1.0, 2.0]], [[7.0, 5.0, -5.0], [-4.0, -3.0, -3.0]], [[2.0, 7.0, 4.0], [-3.0, 0.0, 4.0]], [[3.0, 2.0, 4.0], [2.0, -6.0, 7.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[6.0, -5.0, 2.0], [-4.0, 0.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[62.0, -49.0, -14.0, -48.0, 30.0], [45.0, -2.0, -58.0, 32.0, 27.0], [-67.0, -50.0, 51.0, -15.0, -2.0], [-16.0, -56.0, 41.0, -27.0, -34.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank1_offset0(self):
      arg0=numpy.array([[-7.0, 5.0, 0.0, -1.0, 0.0], [0.0, -1.0, -5.0, 0.0, 0.0], [3.0, -6.0, 1.0, 7.0, -1.0], [-3.0, 4.0, 7.0, 7.0, -3.0]])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-3.0, 7.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[21.0, -49.0], [-15.0, 35.0], [0.0, 0.0], [3.0, -7.0], [0.0, 0.0]], [[0.0, 0.0], [3.0, -7.0], [15.0, -35.0], [0.0, 0.0], [0.0, 0.0]], [[-9.0, 21.0], [18.0, -42.0], [-3.0, 7.0], [-21.0, 49.0], [3.0, -7.0]], [[9.0, -21.0], [-12.0, 28.0], [-21.0, 49.0], [-21.0, 49.0], [9.0, -21.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[0.0, -1.0, 1.0], [-2.0, -1.0, 2.0], [-5.0, 7.0, 0.0], [-5.0, 6.0, 0.0], [-7.0, -6.0, -7.0]], [[1.0, 1.0, -7.0], [-5.0, 3.0, 0.0], [6.0, -1.0, 7.0], [-4.0, 1.0, -5.0], [-7.0, 1.0, 1.0]], [[4.0, 2.0, -5.0], [7.0, 7.0, -4.0], [0.0, -6.0, 5.0], [-3.0, 6.0, 2.0], [3.0, -7.0, 6.0]], [[6.0, -3.0, -2.0], [-2.0, -7.0, 7.0], [-1.0, -7.0, 1.0], [6.0, -5.0, 7.0], [-1.0, 7.0, 7.0]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-4.0, -4.0], [-5.0, -4.0], [4.0, -1.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[9.0, 3.0], [21.0, 10.0], [-15.0, -8.0], [-10.0, -4.0], [30.0, 59.0]], [[-37.0, -1.0], [5.0, 8.0], [9.0, -27.0], [-9.0, 17.0], [27.0, 23.0]], [[-46.0, -19.0], [-79.0, -52.0], [50.0, 19.0], [-10.0, -14.0], [47.0, 10.0]], [[-17.0, -10.0], [71.0, 29.0], [43.0, 31.0], [29.0, -11.0], [-3.0, -31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[[4.0, 1.0, 3.0], [2.0, 5.0, -1.0]], [[-2.0, 3.0, -5.0], [-2.0, 3.0, -2.0]], [[4.0, 0.0, 3.0], [3.0, 5.0, -2.0]], [[-2.0, -6.0, -7.0], [0.0, 2.0, 5.0]], [[-2.0, -1.0, -2.0], [-4.0, -7.0, -1.0]]], [[[1.0, -3.0, -7.0], [0.0, 6.0, 3.0]], [[-4.0, 4.0, -6.0], [2.0, -5.0, -3.0]], [[-4.0, -2.0, -6.0], [7.0, 2.0, -2.0]], [[-1.0, -3.0, 5.0], [0.0, 5.0, -5.0]], [[0.0, 2.0, 4.0], [0.0, 2.0, -1.0]]], [[[0.0, -5.0, 0.0], [-7.0, -7.0, -4.0]], [[1.0, 6.0, 7.0], [5.0, -3.0, 4.0]], [[-5.0, -4.0, -7.0], [-5.0, 0.0, -7.0]], [[1.0, -7.0, 5.0], [-4.0, 7.0, -7.0]], [[-6.0, 2.0, -7.0], [0.0, 0.0, 7.0]]], [[[1.0, 0.0, 6.0], [5.0, 2.0, 1.0]], [[7.0, -7.0, -1.0], [0.0, -5.0, -2.0]], [[-6.0, 0.0, 0.0], [-2.0, 4.0, 3.0]], [[1.0, -2.0, 3.0], [1.0, -3.0, -2.0]], [[0.0, 4.0, 3.0], [-4.0, 0.0, 5.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[4.0, -6.0], [-5.0, 0.0], [-3.0, 0.0]], [[-4.0, 5.0], [1.0, 4.0], [4.0, -7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-5.0, 13.0], [-5.0, 28.0], [-8.0, 25.0], [65.0, -15.0], [8.0, -29.0]], [[58.0, -3.0], [-43.0, 35.0], [-22.0, 81.0], [-19.0, 61.0], [-24.0, 15.0]], [[30.0, -35.0], [-54.0, -21.0], [13.0, 54.0], [19.0, 51.0], [15.0, -13.0]], [[-28.0, 20.0], [53.0, -48.0], [0.0, 21.0], [-10.0, 1.0], [7.0, -55.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_Symbol_rank2_offset0(self):
      arg0=numpy.array([[-4.0, -2.0, 6.0, 2.0, 2.0], [7.0, -2.0, 0.0, 2.0, -2.0], [6.0, 4.0, -6.0, -4.0, -3.0], [-6.0, -1.0, 6.0, 0.0, -3.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-4.0, -5.0, -2.0, 6.0, 4.0], [3.0, -4.0, 0.0, 7.0, -5.0], [0.0, 3.0, -5.0, -4.0, 0.0], [-2.0, -6.0, -2.0, -2.0, -5.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[16.0, 20.0, 8.0, -24.0, -16.0], [-12.0, 16.0, 0.0, -28.0, 20.0], [0.0, -12.0, 20.0, 16.0, 0.0], [8.0, 24.0, 8.0, 8.0, 20.0]], [[8.0, 10.0, 4.0, -12.0, -8.0], [-6.0, 8.0, 0.0, -14.0, 10.0], [0.0, -6.0, 10.0, 8.0, 0.0], [4.0, 12.0, 4.0, 4.0, 10.0]], [[-24.0, -30.0, -12.0, 36.0, 24.0], [18.0, -24.0, 0.0, 42.0, -30.0], [0.0, 18.0, -30.0, -24.0, 0.0], [-12.0, -36.0, -12.0, -12.0, -30.0]], [[-8.0, -10.0, -4.0, 12.0, 8.0], [6.0, -8.0, 0.0, 14.0, -10.0], [0.0, 6.0, -10.0, -8.0, 0.0], [-4.0, -12.0, -4.0, -4.0, -10.0]], [[-8.0, -10.0, -4.0, 12.0, 8.0], [6.0, -8.0, 0.0, 14.0, -10.0], [0.0, 6.0, -10.0, -8.0, 0.0], [-4.0, -12.0, -4.0, -4.0, -10.0]]], [[[-28.0, -35.0, -14.0, 42.0, 28.0], [21.0, -28.0, 0.0, 49.0, -35.0], [0.0, 21.0, -35.0, -28.0, 0.0], [-14.0, -42.0, -14.0, -14.0, -35.0]], [[8.0, 10.0, 4.0, -12.0, -8.0], [-6.0, 8.0, 0.0, -14.0, 10.0], [0.0, -6.0, 10.0, 8.0, 0.0], [4.0, 12.0, 4.0, 4.0, 10.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-8.0, -10.0, -4.0, 12.0, 8.0], [6.0, -8.0, 0.0, 14.0, -10.0], [0.0, 6.0, -10.0, -8.0, 0.0], [-4.0, -12.0, -4.0, -4.0, -10.0]], [[8.0, 10.0, 4.0, -12.0, -8.0], [-6.0, 8.0, 0.0, -14.0, 10.0], [0.0, -6.0, 10.0, 8.0, 0.0], [4.0, 12.0, 4.0, 4.0, 10.0]]], [[[-24.0, -30.0, -12.0, 36.0, 24.0], [18.0, -24.0, 0.0, 42.0, -30.0], [0.0, 18.0, -30.0, -24.0, 0.0], [-12.0, -36.0, -12.0, -12.0, -30.0]], [[-16.0, -20.0, -8.0, 24.0, 16.0], [12.0, -16.0, 0.0, 28.0, -20.0], [0.0, 12.0, -20.0, -16.0, 0.0], [-8.0, -24.0, -8.0, -8.0, -20.0]], [[24.0, 30.0, 12.0, -36.0, -24.0], [-18.0, 24.0, 0.0, -42.0, 30.0], [0.0, -18.0, 30.0, 24.0, 0.0], [12.0, 36.0, 12.0, 12.0, 30.0]], [[16.0, 20.0, 8.0, -24.0, -16.0], [-12.0, 16.0, 0.0, -28.0, 20.0], [0.0, -12.0, 20.0, 16.0, 0.0], [8.0, 24.0, 8.0, 8.0, 20.0]], [[12.0, 15.0, 6.0, -18.0, -12.0], [-9.0, 12.0, 0.0, -21.0, 15.0], [0.0, -9.0, 15.0, 12.0, 0.0], [6.0, 18.0, 6.0, 6.0, 15.0]]], [[[24.0, 30.0, 12.0, -36.0, -24.0], [-18.0, 24.0, 0.0, -42.0, 30.0], [0.0, -18.0, 30.0, 24.0, 0.0], [12.0, 36.0, 12.0, 12.0, 30.0]], [[4.0, 5.0, 2.0, -6.0, -4.0], [-3.0, 4.0, 0.0, -7.0, 5.0], [0.0, -3.0, 5.0, 4.0, 0.0], [2.0, 6.0, 2.0, 2.0, 5.0]], [[-24.0, -30.0, -12.0, 36.0, 24.0], [18.0, -24.0, 0.0, 42.0, -30.0], [0.0, 18.0, -30.0, -24.0, 0.0], [-12.0, -36.0, -12.0, -12.0, -30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[12.0, 15.0, 6.0, -18.0, -12.0], [-9.0, 12.0, 0.0, -21.0, 15.0], [0.0, -9.0, 15.0, 12.0, 0.0], [6.0, 18.0, 6.0, 6.0, 15.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank3_offset1(self):
      arg0=numpy.array([[[6.0, 7.0, 0.0], [4.0, 6.0, -7.0], [-1.0, -4.0, -7.0], [-2.0, -7.0, 3.0], [-3.0, -2.0, -6.0]], [[1.0, -2.0, 2.0], [-4.0, 4.0, -2.0], [-6.0, -7.0, 0.0], [1.0, 0.0, 0.0], [-1.0, 3.0, -4.0]], [[-1.0, -6.0, 7.0], [4.0, -6.0, 4.0], [-7.0, -6.0, 7.0], [-3.0, -7.0, -4.0], [2.0, 6.0, -1.0]], [[-1.0, 3.0, -5.0], [-5.0, -6.0, -2.0], [-6.0, 0.0, -4.0], [6.0, 4.0, -4.0], [3.0, -6.0, 6.0]]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[7.0, -7.0, -6.0, 2.0, 7.0], [4.0, 0.0, 1.0, 2.0, 6.0], [3.0, 4.0, -5.0, -4.0, -4.0], [2.0, 2.0, 0.0, 3.0, -1.0]], [[6.0, 0.0, 6.0, 7.0, 3.0], [-6.0, -6.0, -7.0, -5.0, -6.0], [2.0, 4.0, -3.0, 5.0, 0.0], [-1.0, 6.0, -1.0, -7.0, 2.0]], [[5.0, 0.0, -5.0, -5.0, 6.0], [-6.0, 6.0, 0.0, 3.0, 3.0], [1.0, -1.0, -1.0, 1.0, 1.0], [-1.0, 7.0, 4.0, 3.0, -1.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[84.0, -42.0, 6.0, 61.0, 63.0], [-18.0, -42.0, -43.0, -23.0, -6.0], [32.0, 52.0, -51.0, 11.0, -24.0], [5.0, 54.0, -7.0, -31.0, 8.0]], [[29.0, -28.0, 47.0, 85.0, 4.0], [22.0, -78.0, -38.0, -43.0, -33.0], [17.0, 47.0, -31.0, 7.0, -23.0], [9.0, -5.0, -34.0, -51.0, 15.0]], [[-66.0, 7.0, 17.0, 5.0, -61.0], [62.0, -18.0, 27.0, -3.0, -3.0], [-18.0, -13.0, 24.0, -23.0, -3.0], [9.0, -75.0, -24.0, 4.0, 0.0]], [[-41.0, 14.0, -45.0, -68.0, -17.0], [16.0, 60.0, 47.0, 40.0, 39.0], [-17.0, -39.0, 28.0, -24.0, 11.0], [0.0, -25.0, 19.0, 52.0, -15.0]], [[-63.0, 21.0, 36.0, 10.0, -63.0], [36.0, -24.0, 11.0, -14.0, -24.0], [-19.0, -14.0, 27.0, -4.0, 6.0], [2.0, -60.0, -22.0, -13.0, 5.0]]], [[[5.0, -7.0, -28.0, -22.0, 13.0], [4.0, 24.0, 15.0, 18.0, 24.0], [1.0, -6.0, -1.0, -12.0, -2.0], [2.0, 4.0, 10.0, 23.0, -7.0]], [[-14.0, 28.0, 58.0, 30.0, -28.0], [-28.0, -36.0, -32.0, -34.0, -54.0], [-6.0, 2.0, 10.0, 34.0, 14.0], [-10.0, 2.0, -12.0, -46.0, 14.0]], [[-84.0, 42.0, -6.0, -61.0, -63.0], [18.0, 42.0, 43.0, 23.0, 6.0], [-32.0, -52.0, 51.0, -11.0, 24.0], [-5.0, -54.0, 7.0, 31.0, -8.0]], [[7.0, -7.0, -6.0, 2.0, 7.0], [4.0, 0.0, 1.0, 2.0, 6.0], [3.0, 4.0, -5.0, -4.0, -4.0], [2.0, 2.0, 0.0, 3.0, -1.0]], [[-9.0, 7.0, 44.0, 39.0, -22.0], [2.0, -42.0, -22.0, -29.0, -36.0], [-1.0, 12.0, 0.0, 15.0, 0.0], [-1.0, -12.0, -19.0, -36.0, 11.0]]], [[[-8.0, 7.0, -65.0, -79.0, 17.0], [-10.0, 78.0, 41.0, 49.0, 51.0], [-8.0, -35.0, 16.0, -19.0, 11.0], [-3.0, 11.0, 34.0, 60.0, -18.0]], [[12.0, -28.0, -80.0, -54.0, 34.0], [28.0, 60.0, 46.0, 50.0, 72.0], [4.0, -12.0, -6.0, -42.0, -12.0], [10.0, 0.0, 22.0, 66.0, -20.0]], [[-50.0, 49.0, -29.0, -91.0, -25.0], [-34.0, 78.0, 35.0, 37.0, 15.0], [-26.0, -59.0, 46.0, 5.0, 35.0], [-15.0, -1.0, 34.0, 42.0, -12.0]], [[-83.0, 21.0, -4.0, -35.0, -66.0], [54.0, 18.0, 46.0, 17.0, 12.0], [-27.0, -36.0, 40.0, -27.0, 8.0], [5.0, -76.0, -9.0, 28.0, -7.0]], [[45.0, -14.0, 29.0, 51.0, 26.0], [-22.0, -42.0, -40.0, -29.0, -27.0], [17.0, 33.0, -27.0, 21.0, -9.0], [-1.0, 33.0, -10.0, -39.0, 11.0]]], [[[-14.0, 7.0, 49.0, 44.0, -28.0], [8.0, -48.0, -22.0, -32.0, -39.0], [-2.0, 13.0, 1.0, 14.0, -1.0], [0.0, -19.0, -23.0, -39.0, 12.0]], [[-81.0, 35.0, 4.0, -42.0, -65.0], [28.0, 24.0, 37.0, 14.0, 0.0], [-29.0, -42.0, 45.0, -12.0, 18.0], [-2.0, -60.0, -2.0, 21.0, -5.0]], [[-62.0, 42.0, 56.0, 8.0, -66.0], [0.0, -24.0, -6.0, -24.0, -48.0], [-22.0, -20.0, 34.0, 20.0, 20.0], [-8.0, -40.0, -16.0, -30.0, 10.0]], [[46.0, -42.0, 8.0, 60.0, 30.0], [24.0, -48.0, -22.0, -20.0, 0.0], [22.0, 44.0, -38.0, -8.0, -28.0], [12.0, 8.0, -20.0, -22.0, 6.0]], [[15.0, -21.0, -84.0, -66.0, 39.0], [12.0, 72.0, 45.0, 54.0, 72.0], [3.0, -18.0, -3.0, -36.0, -6.0], [6.0, 12.0, 30.0, 69.0, -21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[[3.0, 4.0, -5.0], [7.0, 7.0, 1.0]], [[5.0, 7.0, 3.0], [0.0, -3.0, -6.0]], [[1.0, 6.0, -5.0], [-1.0, 0.0, 0.0]], [[-3.0, -6.0, 0.0], [-2.0, 6.0, -1.0]], [[-3.0, 6.0, -3.0], [4.0, -7.0, -3.0]]], [[[2.0, 4.0, -7.0], [-3.0, 6.0, 0.0]], [[5.0, -6.0, -4.0], [0.0, 0.0, 4.0]], [[6.0, 0.0, 0.0], [4.0, 1.0, 5.0]], [[3.0, 7.0, -2.0], [-2.0, -2.0, 4.0]], [[-5.0, -1.0, 0.0], [-1.0, 0.0, -3.0]]], [[[1.0, -1.0, 0.0], [3.0, 0.0, 6.0]], [[6.0, -5.0, -1.0], [0.0, 0.0, -1.0]], [[-5.0, 6.0, 4.0], [6.0, 5.0, -2.0]], [[7.0, 0.0, -6.0], [-7.0, 0.0, -3.0]], [[1.0, 1.0, -4.0], [0.0, 7.0, 2.0]]], [[[1.0, 4.0, -7.0], [3.0, 4.0, 0.0]], [[-2.0, 7.0, 0.0], [-2.0, 0.0, -3.0]], [[-1.0, -2.0, 7.0], [-4.0, 6.0, 6.0]], [[6.0, -5.0, -6.0], [-7.0, -4.0, -6.0]], [[1.0, 3.0, -4.0], [-3.0, 6.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-7.0, -5.0, 7.0, 2.0, 7.0], [3.0, 5.0, 6.0, -5.0, 0.0], [2.0, 6.0, -1.0, 0.0, -4.0], [0.0, 6.0, -1.0, 0.0, 7.0]], [[-5.0, 7.0, 0.0, 3.0, 0.0], [-6.0, -5.0, 0.0, -5.0, 4.0], [-7.0, 7.0, -3.0, 3.0, 2.0], [-6.0, 7.0, 6.0, 0.0, 3.0]], [[4.0, -3.0, 4.0, -6.0, -5.0], [0.0, 5.0, -2.0, 5.0, 5.0], [3.0, 0.0, 0.0, -7.0, 4.0], [-2.0, 1.0, 5.0, 2.0, 7.0]]], [[[1.0, 4.0, 3.0, -2.0, 7.0], [-7.0, 2.0, -7.0, -1.0, 0.0], [7.0, 0.0, -1.0, 4.0, 4.0], [-1.0, 7.0, -5.0, 3.0, -3.0]], [[4.0, 0.0, 6.0, 0.0, 0.0], [1.0, -5.0, -7.0, 0.0, 7.0], [6.0, 2.0, 6.0, -6.0, 2.0], [0.0, 1.0, 6.0, -4.0, -5.0]], [[0.0, 0.0, 0.0, -1.0, 4.0], [6.0, 4.0, 5.0, 1.0, -3.0], [0.0, 5.0, -5.0, 3.0, -4.0], [7.0, 3.0, 0.0, 0.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-26.0, 56.0, 64.0, 33.0, 99.0], [-51.0, -47.0, -65.0, -66.0, 37.0], [54.0, 65.0, 15.0, 36.0, 14.0], [-14.0, 100.0, 3.0, -17.0, -61.0]], [[-70.0, 15.0, 29.0, 19.0, -4.0], [-66.0, -4.0, 15.0, -51.0, 40.0], [-48.0, 43.0, -14.0, 0.0, 24.0], [-90.0, 61.0, 34.0, 18.0, 110.0]], [[-58.0, 48.0, -16.0, 52.0, 25.0], [-26.0, -52.0, 23.0, -59.0, -1.0], [-62.0, 48.0, -18.0, 49.0, -16.0], [-25.0, 36.0, 15.0, -13.0, -7.0]], [[73.0, -35.0, 9.0, -19.0, -39.0], [41.0, -23.0, -51.0, 46.0, 21.0], [58.0, -53.0, 64.0, -65.0, 8.0], [31.0, -71.0, 13.0, -30.0, -60.0]], [[-45.0, 82.0, -63.0, 25.0, 10.0], [-98.0, -29.0, -6.0, -37.0, -31.0], [-71.0, -5.0, -46.0, 88.0, 26.0], [-55.0, 33.0, -38.0, 34.0, 8.0]]], [[[-41.0, 27.0, 13.0, 64.0, 28.0], [9.0, -81.0, 5.0, -62.0, 23.0], [-30.0, 52.0, 25.0, 13.0, -28.0], [-7.0, 18.0, 38.0, -47.0, -44.0]], [[-21.0, -55.0, 19.0, 12.0, 71.0], [75.0, 51.0, 58.0, -11.0, -56.0], [40.0, 8.0, -7.0, 22.0, -64.0], [72.0, -4.0, -61.0, -8.0, -23.0]], [[-34.0, -14.0, 60.0, -1.0, 90.0], [21.0, 53.0, 26.0, -29.0, -8.0], [46.0, 63.0, -29.0, 25.0, -26.0], [31.0, 80.0, -20.0, 8.0, 10.0]], [[-74.0, 32.0, -5.0, 39.0, 33.0], [3.0, -8.0, 70.0, -54.0, -8.0], [-75.0, 83.0, -54.0, 51.0, -34.0], [-8.0, 61.0, 27.0, -2.0, 32.0]], [[39.0, 14.0, -38.0, -8.0, -54.0], [-20.0, -34.0, -38.0, 28.0, 5.0], [-10.0, -52.0, 24.0, -16.0, 26.0], [-14.0, -53.0, 4.0, -3.0, -26.0]]], [[[1.0, 0.0, 16.0, -13.0, 52.0], [24.0, 40.0, 15.0, 3.0, -22.0], [30.0, 29.0, -31.0, 27.0, -18.0], [45.0, 38.0, -22.0, 9.0, -23.0]], [[-21.0, -62.0, 38.0, 4.0, 43.0], [42.0, 46.0, 33.0, -11.0, -22.0], [44.0, -4.0, 14.0, -11.0, -34.0], [25.0, -3.0, -41.0, -2.0, 23.0]], [[47.0, 79.0, 29.0, -26.0, -21.0], [-100.0, -56.0, -125.0, 7.0, 85.0], [32.0, 12.0, 21.0, -22.0, 90.0], [-64.0, 57.0, 61.0, 6.0, -26.0]], [[-80.0, -45.0, 4.0, 67.0, 18.0], [52.0, -21.0, 88.0, -61.0, -21.0], [-53.0, 27.0, 15.0, 5.0, -68.0], [-2.0, -22.0, -2.0, -33.0, 37.0]], [[0.0, 14.0, 33.0, 27.0, 35.0], [16.0, -47.0, -25.0, -28.0, 27.0], [25.0, 37.0, 28.0, -5.0, -12.0], [16.0, 22.0, 27.0, -36.0, -59.0]]], [[[-36.0, 56.0, 12.0, 50.0, 63.0], [-38.0, -64.0, -29.0, -63.0, 9.0], [-2.0, 42.0, 8.0, 49.0, -4.0], [-13.0, 52.0, -3.0, -21.0, -59.0]], [[-23.0, 51.0, -20.0, 24.0, -40.0], [-52.0, -61.0, -13.0, -26.0, 37.0], [-67.0, 22.0, -2.0, 4.0, 26.0], [-61.0, 14.0, 54.0, -6.0, 22.0]], [[65.0, -46.0, 45.0, -48.0, -46.0], [79.0, 26.0, -4.0, 60.0, 51.0], [41.0, 22.0, 17.0, -89.0, 0.0], [44.0, -17.0, 80.0, -22.0, 0.0]], [[-64.0, -75.0, -27.0, 53.0, -1.0], [57.0, 7.0, 95.0, -34.0, -60.0], [-44.0, -37.0, 22.0, 5.0, -70.0], [7.0, -76.0, -55.0, -17.0, 44.0]], [[-17.0, 16.0, 18.0, 41.0, 6.0], [12.0, -66.0, -7.0, -37.0, 34.0], [-16.0, 39.0, 29.0, -11.0, -14.0], [-7.0, 8.0, 48.0, -41.0, -33.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank0_offset0(self):
      arg0=numpy.array([[-7.0, 6.0, 1.0, 0.0, -6.0], [1.0, 3.0, -4.0, 5.0, -3.0], [-2.0, -7.0, -2.0, 0.0, -6.0], [6.0, -1.0, 3.0, -2.0, 1.0]])
      arg1=Data(-6.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[42.0, -36.0, -6.0, -0.0, 36.0], [-6.0, -18.0, 24.0, -30.0, 18.0], [12.0, 42.0, 12.0, -0.0, 36.0], [-36.0, 6.0, -18.0, 12.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank1_offset1(self):
      arg0=numpy.array([[[-7.0, 7.0, -5.0], [-3.0, 0.0, -6.0], [7.0, -6.0, -4.0], [3.0, -6.0, -4.0], [-4.0, -3.0, -7.0]], [[7.0, 4.0, 6.0], [0.0, 5.0, -3.0], [6.0, -3.0, -2.0], [-7.0, 3.0, 4.0], [5.0, 4.0, 5.0]], [[-3.0, 3.0, 1.0], [-1.0, -7.0, 6.0], [-5.0, -2.0, 7.0], [3.0, 3.0, 4.0], [4.0, 4.0, -6.0]], [[1.0, 0.0, -1.0], [-7.0, 0.0, 0.0], [4.0, -7.0, -5.0], [-4.0, -4.0, 3.0], [-7.0, -1.0, -4.0]]])
      arg1=Data(numpy.array([1.0, 2.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[37.0, 33.0, 19.0, 15.0, 32.0], [-21.0, 28.0, 12.0, -25.0, -17.0], [-3.0, -51.0, -51.0, -15.0, 48.0], [7.0, -7.0, 20.0, -30.0, 15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank2_offset2(self):
      arg0=numpy.array([[[[0.0, -6.0, 5.0], [-3.0, 1.0, 2.0]], [[-3.0, -7.0, 2.0], [-5.0, 1.0, -1.0]], [[5.0, 0.0, -7.0], [0.0, 0.0, -4.0]], [[4.0, 1.0, -2.0], [0.0, 2.0, 6.0]], [[7.0, 4.0, -7.0], [4.0, 7.0, 1.0]]], [[[0.0, -7.0, -4.0], [-4.0, 0.0, -6.0]], [[4.0, 2.0, 1.0], [1.0, 2.0, -4.0]], [[-3.0, 1.0, 6.0], [0.0, -6.0, 3.0]], [[0.0, -1.0, -2.0], [-2.0, -2.0, 0.0]], [[3.0, 1.0, -1.0], [7.0, 0.0, -4.0]]], [[[-6.0, 2.0, 3.0], [-7.0, -2.0, -5.0]], [[-5.0, 3.0, 1.0], [2.0, -4.0, 2.0]], [[-1.0, -3.0, -7.0], [3.0, 6.0, 0.0]], [[-2.0, -6.0, -4.0], [-5.0, 1.0, 1.0]], [[-7.0, 3.0, -6.0], [-4.0, -2.0, 5.0]]], [[[3.0, -4.0, 4.0], [-7.0, 1.0, 2.0]], [[-6.0, 6.0, 6.0], [-1.0, -3.0, 7.0]], [[-1.0, -5.0, -7.0], [0.0, 7.0, 7.0]], [[-2.0, -7.0, 0.0], [-2.0, 1.0, 0.0]], [[6.0, -1.0, -2.0], [5.0, -7.0, 6.0]]]])
      arg1=Data(numpy.array([[-4.0, 4.0, -4.0], [2.0, -2.0, -5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-62.0, -31.0, 28.0, -38.0, 5.0], [10.0, 6.0, -11.0, 4.0, 30.0], [35.0, 30.0, 14.0, -17.0, 35.0], [-70.0, -7.0, -37.0, -26.0, -26.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank1_offset0(self):
      arg0=numpy.array([[-2.0, 4.0, 2.0, -7.0, -2.0], [-3.0, -1.0, -4.0, -5.0, 0.0], [-7.0, 1.0, -3.0, -5.0, 3.0], [-5.0, -5.0, 0.0, -1.0, -3.0]])
      arg1=Data(numpy.array([2.0, 0.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-4.0, 0.0], [8.0, 0.0], [4.0, 0.0], [-14.0, 0.0], [-4.0, 0.0]], [[-6.0, 0.0], [-2.0, 0.0], [-8.0, 0.0], [-10.0, 0.0], [0.0, 0.0]], [[-14.0, 0.0], [2.0, 0.0], [-6.0, 0.0], [-10.0, 0.0], [6.0, 0.0]], [[-10.0, 0.0], [-10.0, 0.0], [0.0, 0.0], [-2.0, 0.0], [-6.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank2_offset1(self):
      arg0=numpy.array([[[-7.0, -5.0, 0.0], [5.0, -6.0, 6.0], [0.0, -3.0, -7.0], [2.0, -2.0, 5.0], [2.0, 1.0, 3.0]], [[-1.0, -4.0, -4.0], [0.0, -5.0, 4.0], [-2.0, -7.0, -3.0], [-6.0, -4.0, -5.0], [0.0, -4.0, -3.0]], [[-2.0, 2.0, 3.0], [-6.0, 5.0, 5.0], [3.0, -6.0, 3.0], [-1.0, 0.0, 3.0], [6.0, -1.0, 6.0]], [[-7.0, 1.0, -7.0], [7.0, -2.0, 5.0], [0.0, 5.0, -5.0], [7.0, 4.0, 3.0], [-3.0, 4.0, 3.0]]])
      arg1=Data(numpy.array([[-7.0, -3.0], [1.0, 6.0], [5.0, -4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[44.0, -9.0], [-11.0, -75.0], [-38.0, 10.0], [9.0, -38.0], [2.0, -12.0]], [[-17.0, -5.0], [15.0, -46.0], [-8.0, -24.0], [13.0, 14.0], [-19.0, -12.0]], [[31.0, 6.0], [72.0, 28.0], [-12.0, -57.0], [22.0, -9.0], [-13.0, -48.0]], [[15.0, 55.0], [-26.0, -53.0], [-20.0, 50.0], [-30.0, -9.0], [40.0, 21.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank3_offset2(self):
      arg0=numpy.array([[[[1.0, 5.0, -7.0], [2.0, -1.0, -6.0]], [[6.0, 0.0, 7.0], [-1.0, -7.0, -6.0]], [[2.0, 0.0, 2.0], [7.0, 7.0, -2.0]], [[4.0, 0.0, 2.0], [5.0, 0.0, 0.0]], [[-3.0, 0.0, 2.0], [0.0, -3.0, -6.0]]], [[[0.0, -6.0, -4.0], [-6.0, 6.0, 6.0]], [[-1.0, 3.0, -4.0], [-5.0, 3.0, 4.0]], [[3.0, -3.0, -1.0], [0.0, 5.0, -2.0]], [[-5.0, -7.0, -1.0], [0.0, -3.0, 1.0]], [[-4.0, 4.0, -5.0], [-2.0, 4.0, 5.0]]], [[[0.0, -1.0, -7.0], [7.0, 5.0, -5.0]], [[1.0, -6.0, 7.0], [6.0, -6.0, 1.0]], [[2.0, 3.0, 1.0], [0.0, 5.0, 4.0]], [[-6.0, 4.0, -4.0], [0.0, 4.0, 6.0]], [[1.0, -4.0, 2.0], [2.0, 0.0, -3.0]]], [[[-4.0, -7.0, -7.0], [0.0, 5.0, 6.0]], [[-4.0, 0.0, 3.0], [-3.0, -2.0, 7.0]], [[-5.0, 4.0, -6.0], [-1.0, 0.0, 0.0]], [[-2.0, 1.0, 0.0], [0.0, 0.0, -5.0]], [[6.0, 0.0, -3.0], [2.0, 5.0, -3.0]]]])
      arg1=Data(numpy.array([[[-5.0, 6.0], [0.0, -5.0], [-2.0, -5.0]], [[3.0, -4.0], [0.0, 3.0], [1.0, 4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[9.0, -19.0], [-53.0, -40.0], [5.0, -13.0], [-9.0, -6.0], [5.0, -61.0]], [[-4.0, 116.0], [2.0, 44.0], [-15.0, 45.0], [28.0, 5.0], [29.0, 21.0]], [[30.0, 7.0], [0.0, -37.0], [-8.0, 23.0], [44.0, 0.0], [-6.0, -4.0]], [[40.0, 85.0], [12.0, -5.0], [34.0, -16.0], [5.0, -37.0], [-21.0, 46.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_constData_rank2_offset0(self):
      arg0=numpy.array([[0.0, -6.0, 0.0, 0.0, 0.0], [0.0, 7.0, -3.0, 6.0, -1.0], [-3.0, -3.0, -3.0, -1.0, 3.0], [-2.0, -3.0, 1.0, -2.0, 6.0]])
      arg1=Data(numpy.array([[5.0, 0.0, -3.0, -5.0, -1.0], [-2.0, -2.0, 0.0, -4.0, 6.0], [-2.0, 5.0, 5.0, 0.0, -1.0], [3.0, -6.0, 3.0, -7.0, 7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-30.0, 0.0, 18.0, 30.0, 6.0], [12.0, 12.0, 0.0, 24.0, -36.0], [12.0, -30.0, -30.0, 0.0, 6.0], [-18.0, 36.0, -18.0, 42.0, -42.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[35.0, 0.0, -21.0, -35.0, -7.0], [-14.0, -14.0, 0.0, -28.0, 42.0], [-14.0, 35.0, 35.0, 0.0, -7.0], [21.0, -42.0, 21.0, -49.0, 49.0]], [[-15.0, 0.0, 9.0, 15.0, 3.0], [6.0, 6.0, 0.0, 12.0, -18.0], [6.0, -15.0, -15.0, 0.0, 3.0], [-9.0, 18.0, -9.0, 21.0, -21.0]], [[30.0, 0.0, -18.0, -30.0, -6.0], [-12.0, -12.0, 0.0, -24.0, 36.0], [-12.0, 30.0, 30.0, 0.0, -6.0], [18.0, -36.0, 18.0, -42.0, 42.0]], [[-5.0, 0.0, 3.0, 5.0, 1.0], [2.0, 2.0, 0.0, 4.0, -6.0], [2.0, -5.0, -5.0, 0.0, 1.0], [-3.0, 6.0, -3.0, 7.0, -7.0]]], [[[-15.0, 0.0, 9.0, 15.0, 3.0], [6.0, 6.0, 0.0, 12.0, -18.0], [6.0, -15.0, -15.0, 0.0, 3.0], [-9.0, 18.0, -9.0, 21.0, -21.0]], [[-15.0, 0.0, 9.0, 15.0, 3.0], [6.0, 6.0, 0.0, 12.0, -18.0], [6.0, -15.0, -15.0, 0.0, 3.0], [-9.0, 18.0, -9.0, 21.0, -21.0]], [[-15.0, 0.0, 9.0, 15.0, 3.0], [6.0, 6.0, 0.0, 12.0, -18.0], [6.0, -15.0, -15.0, 0.0, 3.0], [-9.0, 18.0, -9.0, 21.0, -21.0]], [[-5.0, 0.0, 3.0, 5.0, 1.0], [2.0, 2.0, 0.0, 4.0, -6.0], [2.0, -5.0, -5.0, 0.0, 1.0], [-3.0, 6.0, -3.0, 7.0, -7.0]], [[15.0, 0.0, -9.0, -15.0, -3.0], [-6.0, -6.0, 0.0, -12.0, 18.0], [-6.0, 15.0, 15.0, 0.0, -3.0], [9.0, -18.0, 9.0, -21.0, 21.0]]], [[[-10.0, 0.0, 6.0, 10.0, 2.0], [4.0, 4.0, 0.0, 8.0, -12.0], [4.0, -10.0, -10.0, 0.0, 2.0], [-6.0, 12.0, -6.0, 14.0, -14.0]], [[-15.0, 0.0, 9.0, 15.0, 3.0], [6.0, 6.0, 0.0, 12.0, -18.0], [6.0, -15.0, -15.0, 0.0, 3.0], [-9.0, 18.0, -9.0, 21.0, -21.0]], [[5.0, 0.0, -3.0, -5.0, -1.0], [-2.0, -2.0, 0.0, -4.0, 6.0], [-2.0, 5.0, 5.0, 0.0, -1.0], [3.0, -6.0, 3.0, -7.0, 7.0]], [[-10.0, 0.0, 6.0, 10.0, 2.0], [4.0, 4.0, 0.0, 8.0, -12.0], [4.0, -10.0, -10.0, 0.0, 2.0], [-6.0, 12.0, -6.0, 14.0, -14.0]], [[30.0, 0.0, -18.0, -30.0, -6.0], [-12.0, -12.0, 0.0, -24.0, 36.0], [-12.0, 30.0, 30.0, 0.0, -6.0], [18.0, -36.0, 18.0, -42.0, 42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank3_offset1(self):
      arg0=numpy.array([[[-2.0, -7.0, 6.0], [0.0, -6.0, 0.0], [2.0, 1.0, -1.0], [4.0, 5.0, -3.0], [1.0, 7.0, -5.0]], [[1.0, 0.0, 6.0], [7.0, -1.0, -6.0], [0.0, 3.0, 4.0], [-2.0, 7.0, 0.0], [2.0, 2.0, -1.0]], [[-6.0, 5.0, -1.0], [-7.0, -4.0, -4.0], [-5.0, -4.0, 0.0], [1.0, 4.0, 6.0], [0.0, 0.0, -7.0]], [[-3.0, -4.0, -7.0], [3.0, -4.0, -1.0], [3.0, 5.0, 4.0], [0.0, 3.0, 0.0], [-4.0, -5.0, 7.0]]])
      arg1=Data(numpy.array([[[0.0, 5.0, -7.0, -2.0, -6.0], [5.0, 0.0, 5.0, -3.0, -3.0], [-3.0, -3.0, -4.0, -6.0, 0.0], [-7.0, -2.0, 7.0, -5.0, 7.0]], [[0.0, -7.0, 0.0, 4.0, 2.0], [0.0, 0.0, 6.0, 5.0, 2.0], [-1.0, 4.0, -3.0, -7.0, -7.0], [0.0, 0.0, -6.0, -5.0, 4.0]], [[0.0, 6.0, 2.0, -5.0, 0.0], [7.0, -1.0, 7.0, -3.0, 6.0], [0.0, 1.0, -2.0, -3.0, 3.0], [-7.0, 0.0, 0.0, 2.0, 4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[0.0, 75.0, 26.0, -54.0, -2.0], [32.0, -6.0, -10.0, -47.0, 28.0], [13.0, -16.0, 17.0, 43.0, 67.0], [-28.0, 4.0, 28.0, 57.0, -18.0]], [[0.0, 42.0, 0.0, -24.0, -12.0], [0.0, 0.0, -36.0, -30.0, -12.0], [6.0, -24.0, 18.0, 42.0, 42.0], [0.0, 0.0, 36.0, 30.0, -24.0]], [[0.0, -3.0, -16.0, 5.0, -10.0], [3.0, 1.0, 9.0, 2.0, -10.0], [-7.0, -3.0, -9.0, -16.0, -10.0], [-7.0, -4.0, 8.0, -17.0, 14.0]], [[0.0, -33.0, -34.0, 27.0, -14.0], [-1.0, 3.0, 29.0, 22.0, -20.0], [-17.0, 5.0, -25.0, -50.0, -44.0], [-7.0, -8.0, -2.0, -51.0, 36.0]], [[0.0, -74.0, -17.0, 51.0, 8.0], [-30.0, 5.0, 12.0, 47.0, -19.0], [-10.0, 20.0, -15.0, -40.0, -64.0], [28.0, -2.0, -35.0, -50.0, 15.0]]], [[[0.0, 41.0, 5.0, -32.0, -6.0], [47.0, -6.0, 47.0, -21.0, 33.0], [-3.0, 3.0, -16.0, -24.0, 18.0], [-49.0, -2.0, 7.0, 7.0, 31.0]], [[0.0, 6.0, -61.0, 12.0, -44.0], [-7.0, 6.0, -13.0, -8.0, -59.0], [-20.0, -31.0, -13.0, -17.0, -11.0], [-7.0, -14.0, 55.0, -42.0, 21.0]], [[0.0, 3.0, 8.0, -8.0, 6.0], [28.0, -4.0, 46.0, 3.0, 30.0], [-3.0, 16.0, -17.0, -33.0, -9.0], [-28.0, 0.0, -18.0, -7.0, 28.0]], [[0.0, -59.0, 14.0, 32.0, 26.0], [-10.0, 0.0, 32.0, 41.0, 20.0], [-1.0, 34.0, -13.0, -37.0, -49.0], [14.0, 4.0, -56.0, -25.0, 14.0]], [[0.0, -10.0, -16.0, 9.0, -8.0], [3.0, 1.0, 15.0, 7.0, -8.0], [-8.0, 1.0, -12.0, -23.0, -17.0], [-7.0, -4.0, 2.0, -22.0, 18.0]]], [[[0.0, -71.0, 40.0, 37.0, 46.0], [-37.0, 1.0, -7.0, 46.0, 22.0], [13.0, 37.0, 11.0, 4.0, -38.0], [49.0, 12.0, -72.0, 3.0, -26.0]], [[0.0, -31.0, 41.0, 18.0, 34.0], [-63.0, 4.0, -87.0, 13.0, -11.0], [25.0, 1.0, 48.0, 82.0, 16.0], [77.0, 14.0, -25.0, 47.0, -81.0]], [[0.0, 3.0, 35.0, -6.0, 22.0], [-25.0, 0.0, -49.0, -5.0, 7.0], [19.0, -1.0, 32.0, 58.0, 28.0], [35.0, 10.0, -11.0, 45.0, -51.0]], [[0.0, 13.0, 5.0, -16.0, 2.0], [47.0, -6.0, 71.0, -1.0, 41.0], [-7.0, 19.0, -28.0, -52.0, -10.0], [-49.0, -2.0, -17.0, -13.0, 47.0]], [[0.0, -42.0, -14.0, 35.0, 0.0], [-49.0, 7.0, -49.0, 21.0, -42.0], [0.0, -7.0, 14.0, 21.0, -21.0], [49.0, 0.0, 0.0, -14.0, -28.0]]], [[[0.0, -29.0, 7.0, 25.0, 10.0], [-64.0, 7.0, -88.0, 10.0, -41.0], [13.0, -14.0, 38.0, 67.0, 7.0], [70.0, 6.0, 3.0, 21.0, -65.0]], [[0.0, 37.0, -23.0, -17.0, -26.0], [8.0, 1.0, -16.0, -26.0, -23.0], [-5.0, -26.0, 2.0, 13.0, 25.0], [-14.0, -6.0, 45.0, 3.0, 1.0]], [[0.0, 4.0, -13.0, -6.0, -8.0], [43.0, -4.0, 73.0, 4.0, 25.0], [-14.0, 15.0, -35.0, -65.0, -23.0], [-49.0, -6.0, -9.0, -32.0, 57.0]], [[0.0, -21.0, 0.0, 12.0, 6.0], [0.0, 0.0, 18.0, 15.0, 6.0], [-3.0, 12.0, -9.0, -21.0, -21.0], [0.0, 0.0, -18.0, -15.0, 12.0]], [[0.0, 57.0, 42.0, -47.0, 14.0], [29.0, -7.0, -1.0, -34.0, 44.0], [17.0, -1.0, 17.0, 38.0, 56.0], [-21.0, 8.0, 2.0, 59.0, -20.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank4_offset2(self):
      arg0=numpy.array([[[[-7.0, -7.0, 1.0], [0.0, 0.0, -6.0]], [[1.0, 4.0, 1.0], [-5.0, 3.0, -6.0]], [[7.0, 6.0, -6.0], [0.0, -5.0, 4.0]], [[-1.0, -5.0, 5.0], [6.0, 0.0, 5.0]], [[0.0, -3.0, 3.0], [-4.0, 0.0, 7.0]]], [[[5.0, -6.0, -6.0], [0.0, -1.0, -3.0]], [[3.0, 0.0, 6.0], [-1.0, -2.0, 4.0]], [[1.0, -7.0, 5.0], [-4.0, 0.0, -2.0]], [[0.0, -7.0, 2.0], [1.0, 6.0, 0.0]], [[1.0, 3.0, -3.0], [-5.0, -5.0, 5.0]]], [[[-1.0, -5.0, -5.0], [7.0, 3.0, -2.0]], [[-1.0, 0.0, 7.0], [-3.0, 0.0, -7.0]], [[-2.0, 7.0, -6.0], [-3.0, -5.0, -2.0]], [[7.0, 6.0, 4.0], [-3.0, -1.0, 6.0]], [[-5.0, 7.0, 2.0], [5.0, 2.0, -3.0]]], [[[6.0, -2.0, -6.0], [-6.0, 1.0, 5.0]], [[-2.0, -7.0, 3.0], [6.0, 0.0, 3.0]], [[-3.0, 0.0, -7.0], [-5.0, -6.0, -4.0]], [[7.0, -2.0, 5.0], [-4.0, 3.0, 4.0]], [[-4.0, -6.0, -6.0], [2.0, 4.0, 7.0]]]])
      arg1=Data(numpy.array([[[[1.0, 5.0, -3.0, -3.0, 1.0], [0.0, 7.0, -1.0, 0.0, 4.0], [6.0, 2.0, 0.0, -4.0, -3.0], [3.0, 0.0, 2.0, 6.0, -4.0]], [[0.0, 0.0, -1.0, 3.0, 0.0], [-1.0, 3.0, -6.0, 4.0, -1.0], [0.0, 5.0, -5.0, -4.0, 2.0], [-4.0, -2.0, 0.0, 0.0, 5.0]], [[-6.0, 0.0, -2.0, 4.0, 4.0], [-6.0, -2.0, 3.0, 2.0, -3.0], [5.0, -5.0, -2.0, 3.0, 3.0], [-4.0, -1.0, -2.0, -4.0, -2.0]]], [[[3.0, -5.0, 5.0, 0.0, -7.0], [1.0, 6.0, 5.0, 0.0, 0.0], [-7.0, 6.0, 1.0, -2.0, 3.0], [-6.0, 0.0, 6.0, 0.0, -7.0]], [[6.0, 0.0, -3.0, 2.0, 4.0], [4.0, 7.0, 0.0, 3.0, 2.0], [-4.0, -2.0, 4.0, -3.0, -1.0], [7.0, -7.0, 0.0, 5.0, 2.0]], [[6.0, -1.0, 0.0, -6.0, -3.0], [0.0, -7.0, -6.0, 7.0, -3.0], [4.0, -1.0, 6.0, 1.0, -5.0], [-2.0, -2.0, -2.0, 4.0, -1.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-49.0, -29.0, 26.0, 40.0, 15.0], [1.0, -30.0, 88.0, -68.0, -6.0], [-61.0, -48.0, -3.0, 53.0, 40.0], [15.0, 25.0, -4.0, -70.0, -3.0]], [[-38.0, 36.0, -43.0, 55.0, 70.0], [-3.0, 50.0, -11.0, -15.0, 21.0], [10.0, -13.0, -51.0, -22.0, 20.0], [46.0, -18.0, -18.0, -7.0, 61.0]], [[37.0, 31.0, 0.0, -61.0, -49.0], [10.0, 16.0, -85.0, 25.0, 18.0], [48.0, 80.0, -14.0, -51.0, -42.0], [-22.0, 21.0, 18.0, 57.0, 0.0]], [[17.0, -40.0, 28.0, -22.0, -38.0], [-19.0, -31.0, 46.0, 25.0, -29.0], [-3.0, -21.0, 51.0, 32.0, 1.0], [-49.0, -5.0, 14.0, -6.0, -78.0]], [[12.0, 13.0, -23.0, -39.0, 19.0], [-19.0, -88.0, -35.0, 43.0, -27.0], [71.0, -61.0, 47.0, 36.0, -44.0], [10.0, -11.0, -44.0, 16.0, 0.0]]], [[[17.0, 28.0, 6.0, -41.0, -14.0], [38.0, 43.0, 31.0, -60.0, 51.0], [-8.0, 15.0, 20.0, -14.0, -29.0], [62.0, 31.0, 28.0, 37.0, -37.0]], [[-24.0, 16.0, -20.0, -13.0, 14.0], [-45.0, -39.0, -14.0, 34.0, -22.0], [79.0, -30.0, 3.0, 18.0, -12.0], [-31.0, 0.0, -20.0, 0.0, -25.0]], [[-53.0, 27.0, -26.0, 8.0, 55.0], [-27.0, -34.0, 48.0, -32.0, 2.0], [51.0, -80.0, 9.0, 45.0, -4.0], [39.0, 13.0, -28.0, -22.0, -19.0]], [[27.0, -5.0, -10.0, -1.0, 25.0], [20.0, 23.0, 53.0, -6.0, 13.0], [-21.0, -51.0, 56.0, 14.0, -11.0], [56.0, -30.0, 2.0, 22.0, -34.0]], [[4.0, 25.0, -10.0, -46.0, -11.0], [-10.0, -78.0, -83.0, 26.0, -15.0], [66.0, 7.0, -4.0, 5.0, -41.0], [-12.0, 22.0, -32.0, 13.0, 37.0]]], [[[56.0, -38.0, 44.0, -14.0, -52.0], [54.0, 65.0, 63.0, -35.0, 28.0], [-100.0, 36.0, 42.0, -16.0, 6.0], [20.0, -2.0, 54.0, 21.0, -52.0]], [[-94.0, 17.0, -26.0, 73.0, 69.0], [-45.0, 10.0, 49.0, -35.0, -4.0], [22.0, -48.0, -59.0, 24.0, 50.0], [1.0, 7.0, -20.0, -62.0, 18.0]], [[-17.0, 7.0, 11.0, 5.0, -19.0], [6.0, -20.0, -61.0, -13.0, -1.0], [-9.0, 55.0, -58.0, -19.0, 8.0], [-23.0, 31.0, -6.0, -21.0, 68.0]], [[4.0, 44.0, -47.0, -25.0, 22.0], [-37.0, -8.0, -82.0, 71.0, -10.0], [111.0, 2.0, -9.0, -25.0, -35.0], [-20.0, -21.0, -24.0, 45.0, 7.0]], [[-8.0, -47.0, 23.0, 66.0, -15.0], [-6.0, 47.0, 12.0, 17.0, -20.0], [-75.0, 44.0, -44.0, -21.0, 63.0], [-61.0, -24.0, 22.0, -40.0, 23.0]]], [[[60.0, 55.0, -37.0, -76.0, 13.0], [36.0, -16.0, -72.0, 18.0, 31.0], [64.0, -11.0, 50.0, -20.0, -84.0], [83.0, -7.0, -22.0, 85.0, 17.0]], [[16.0, -43.0, 37.0, -21.0, -41.0], [-5.0, -26.0, 65.0, -1.0, -19.0], [-27.0, -21.0, 53.0, 36.0, 4.0], [-32.0, 5.0, 20.0, -12.0, -78.0]], [[-36.0, 14.0, 16.0, -7.0, -8.0], [13.0, -51.0, -19.0, -60.0, 9.0], [-10.0, 15.0, -39.0, 15.0, -1.0], [15.0, 57.0, -14.0, -36.0, 53.0]], [[7.0, 51.0, -58.0, -25.0, 55.0], [-20.0, 2.0, -24.0, 39.0, 9.0], [99.0, -55.0, 32.0, -2.0, -45.0], [46.0, -30.0, -28.0, 53.0, -18.0]], [[104.0, -37.0, 28.0, -64.0, -47.0], [60.0, -43.0, -10.0, 25.0, -5.0], [-56.0, -11.0, 102.0, 13.0, -51.0], [38.0, -24.0, 2.0, 48.0, -15.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank0_offset0(self):
      arg0=numpy.array([[-5.0, 3.0, 3.0, 0.0, -1.0], [-7.0, 0.0, 0.0, 0.0, -3.0], [2.0, -5.0, -1.0, 1.0, -6.0], [-6.0, -3.0, 0.0, 1.0, 0.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[15.0, -9.0, -9.0, -0.0, 3.0], [21.0, -0.0, -0.0, -0.0, 9.0], [-6.0, 15.0, 3.0, -3.0, 18.0], [18.0, 9.0, -0.0, -3.0, -0.0]])+(1.-msk_ref)*numpy.array([[10.0, -6.0, -6.0, -0.0, 2.0], [14.0, -0.0, -0.0, -0.0, 6.0], [-4.0, 10.0, 2.0, -2.0, 12.0], [12.0, 6.0, -0.0, -2.0, -0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[-1.0, 6.0, -2.0], [5.0, 6.0, 1.0], [-1.0, 0.0, -3.0], [-1.0, -5.0, -6.0], [-6.0, -1.0, -7.0]], [[3.0, -1.0, 6.0], [-2.0, -2.0, -4.0], [2.0, -1.0, 7.0], [4.0, 1.0, 3.0], [-4.0, 1.0, 5.0]], [[-7.0, 2.0, 2.0], [2.0, 2.0, -3.0], [-5.0, -6.0, -1.0], [-6.0, 6.0, 5.0], [2.0, 4.0, -3.0]], [[1.0, 0.0, 2.0], [7.0, 6.0, -6.0], [-1.0, -5.0, -6.0], [0.0, -1.0, -5.0], [-5.0, -1.0, 5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-2.0, -2.0, 2.0])+(1.-msk_arg1)*numpy.array([0.0, 0.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-14.0, -20.0, -4.0, 0.0, 0.0], [8.0, 0.0, 12.0, -4.0, 16.0], [14.0, -14.0, 20.0, 10.0, -18.0], [2.0, -38.0, 0.0, -8.0, 22.0]])+(1.-msk_ref)*numpy.array([[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[[5.0, -3.0, 1.0], [1.0, -2.0, 1.0]], [[-7.0, 5.0, 1.0], [3.0, 2.0, -2.0]], [[-6.0, 2.0, -4.0], [-6.0, 1.0, -5.0]], [[5.0, -1.0, 0.0], [-3.0, 1.0, 2.0]], [[-1.0, 4.0, 0.0], [0.0, 4.0, -4.0]]], [[[3.0, 4.0, 3.0], [-5.0, -2.0, 5.0]], [[3.0, -2.0, -5.0], [0.0, -4.0, 0.0]], [[5.0, 5.0, -7.0], [-6.0, 2.0, -2.0]], [[3.0, -3.0, 0.0], [-2.0, 3.0, -6.0]], [[-3.0, 1.0, 2.0], [6.0, 5.0, -3.0]]], [[[4.0, 0.0, 0.0], [-1.0, 2.0, -1.0]], [[-4.0, 1.0, 1.0], [6.0, 4.0, -4.0]], [[-1.0, -2.0, -4.0], [-6.0, 3.0, -7.0]], [[0.0, -3.0, 0.0], [6.0, -1.0, -3.0]], [[-6.0, 0.0, 3.0], [1.0, 7.0, -7.0]]], [[[0.0, 0.0, -4.0], [-4.0, 1.0, -6.0]], [[-1.0, 2.0, 3.0], [-1.0, 5.0, -5.0]], [[5.0, 3.0, 3.0], [4.0, -2.0, 1.0]], [[1.0, -4.0, 0.0], [5.0, 3.0, 5.0]], [[5.0, 6.0, -3.0], [1.0, 5.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-1.0, 1.0, 1.0], [6.0, -7.0, -7.0]])+(1.-msk_arg1)*numpy.array([[3.0, 6.0, -7.0], [7.0, 6.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 31.0, -4.0, -45.0, 5.0], [-47.0, 18.0, -43.0, 3.0, 28.0], [-17.0, 42.0, -13.0, 61.0, 15.0], [7.0, 0.0, 32.0, -31.0, -66.0]])+(1.-msk_ref)*numpy.array([[-11.0, 27.0, -34.0, 2.0, 29.0], [-15.0, 8.0, 56.0, -29.0, 43.0], [13.0, 37.0, -39.0, 6.0, -18.0], [-18.0, -9.0, 32.0, 52.0, 129.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank1_offset0(self):
      arg0=numpy.array([[7.0, -2.0, 0.0, -1.0, 5.0], [-4.0, 7.0, 1.0, -2.0, -1.0], [-1.0, 2.0, -7.0, -5.0, 6.0], [0.0, -7.0, -5.0, 3.0, -5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, -5.0])+(1.-msk_arg1)*numpy.array([0.0, -7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-35.0, -35.0], [10.0, 10.0], [0.0, 0.0], [5.0, 5.0], [-25.0, -25.0]], [[20.0, 20.0], [-35.0, -35.0], [-5.0, -5.0], [10.0, 10.0], [5.0, 5.0]], [[5.0, 5.0], [-10.0, -10.0], [35.0, 35.0], [25.0, 25.0], [-30.0, -30.0]], [[0.0, 0.0], [35.0, 35.0], [25.0, 25.0], [-15.0, -15.0], [25.0, 25.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -49.0], [0.0, 14.0], [0.0, 0.0], [0.0, 7.0], [0.0, -35.0]], [[0.0, 28.0], [0.0, -49.0], [0.0, -7.0], [0.0, 14.0], [0.0, 7.0]], [[0.0, 7.0], [0.0, -14.0], [0.0, 49.0], [0.0, 35.0], [0.0, -42.0]], [[0.0, 0.0], [0.0, 49.0], [0.0, 35.0], [0.0, -21.0], [0.0, 35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[-6.0, -2.0, -7.0], [5.0, -2.0, 3.0], [7.0, -7.0, -3.0], [5.0, -7.0, -1.0], [6.0, 0.0, 3.0]], [[-6.0, -3.0, -5.0], [6.0, 5.0, 5.0], [-2.0, 5.0, -4.0], [5.0, 0.0, 1.0], [-6.0, -6.0, 6.0]], [[-7.0, 7.0, -1.0], [2.0, -3.0, -4.0], [-1.0, 2.0, 7.0], [5.0, -5.0, -4.0], [-2.0, 6.0, -7.0]], [[5.0, 1.0, 6.0], [-1.0, -1.0, -5.0], [-5.0, -3.0, 5.0], [6.0, -6.0, -6.0], [0.0, 0.0, -5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 7.0], [3.0, -3.0], [-2.0, 4.0]])+(1.-msk_arg1)*numpy.array([[3.0, 4.0], [-4.0, -6.0], [-4.0, -2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, -64.0], [-12.0, 53.0], [-15.0, 58.0], [-19.0, 52.0], [-6.0, 54.0]], [[1.0, -53.0], [5.0, 47.0], [23.0, -45.0], [-2.0, 39.0], [-30.0, 0.0]], [[23.0, -74.0], [-1.0, 7.0], [-8.0, 15.0], [-7.0, 34.0], [32.0, -60.0]], [[-9.0, 56.0], [7.0, -24.0], [-19.0, -6.0], [-6.0, 36.0], [10.0, -20.0]]])+(1.-msk_ref)*numpy.array([[[18.0, 2.0], [11.0, 26.0], [61.0, 76.0], [47.0, 64.0], [6.0, 18.0]], [[14.0, 4.0], [-22.0, -16.0], [-10.0, -30.0], [11.0, 18.0], [-18.0, 0.0]], [[-45.0, -68.0], [34.0, 34.0], [-39.0, -30.0], [51.0, 58.0], [-2.0, -30.0]], [[-13.0, 2.0], [21.0, 12.0], [-23.0, -12.0], [66.0, 72.0], [20.0, 10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[[-4.0, 1.0, 0.0], [0.0, -5.0, 1.0]], [[6.0, -4.0, -2.0], [-2.0, 3.0, -1.0]], [[2.0, 7.0, 0.0], [5.0, -2.0, 0.0]], [[7.0, 7.0, 3.0], [3.0, -1.0, -4.0]], [[-2.0, -1.0, -5.0], [0.0, 1.0, -3.0]]], [[[0.0, -5.0, -6.0], [6.0, 0.0, -1.0]], [[-2.0, -5.0, -1.0], [-7.0, -3.0, -4.0]], [[7.0, -7.0, -3.0], [-5.0, -2.0, 3.0]], [[-2.0, 4.0, -5.0], [4.0, 7.0, -7.0]], [[3.0, 0.0, 2.0], [7.0, -2.0, -1.0]]], [[[-3.0, -7.0, 5.0], [-4.0, -2.0, -4.0]], [[6.0, -6.0, 6.0], [0.0, 5.0, -7.0]], [[0.0, 3.0, -1.0], [-5.0, -6.0, 1.0]], [[1.0, -1.0, 0.0], [5.0, 7.0, 1.0]], [[4.0, 5.0, -3.0], [-7.0, 0.0, 0.0]]], [[[1.0, -6.0, -4.0], [1.0, 1.0, 1.0]], [[-3.0, -1.0, 4.0], [1.0, -6.0, 6.0]], [[6.0, 7.0, -7.0], [-6.0, 0.0, 2.0]], [[1.0, 4.0, -7.0], [6.0, -6.0, 6.0]], [[-3.0, 4.0, 0.0], [-3.0, -7.0, -7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 7.0], [-1.0, 5.0], [6.0, 6.0]], [[6.0, 7.0], [-4.0, -2.0], [7.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, -2.0], [2.0, -2.0], [4.0, 2.0]], [[-4.0, -7.0], [4.0, 4.0], [-7.0, -6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[14.0, -10.0], [-21.0, -13.0], [37.0, 88.0], [26.0, 113.0], [-60.0, -60.0]], [[-2.0, -22.0], [-65.0, -100.0], [9.0, -26.0], [-93.0, -31.0], [64.0, 83.0]], [[-16.0, -62.0], [-9.0, 17.0], [-8.0, -11.0], [13.0, 26.0], [-53.0, -14.0]], [[-6.0, -39.0], [88.0, 35.0], [-53.0, -1.0], [59.0, 57.0], [-52.0, -29.0]]])+(1.-msk_ref)*numpy.array([[[-1.0, -20.0], [-25.0, 24.0], [-26.0, -61.0], [-4.0, -23.0], [15.0, 18.0]], [[-51.0, -38.0], [42.0, 73.0], [-77.0, 3.0], [61.0, 28.0], [-39.0, -53.0]], [[60.0, 74.0], [45.0, 74.0], [-9.0, -3.0], [-7.0, -13.0], [2.0, 25.0]], [[-41.0, -7.0], [-38.0, -51.0], [-40.0, -10.0], [-116.0, -126.0], [59.0, 33.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank2_expandedData_rank2_offset0(self):
      arg0=numpy.array([[6.0, 7.0, -6.0, 5.0, 0.0], [2.0, -4.0, -1.0, 0.0, 0.0], [6.0, 2.0, -2.0, 5.0, -1.0], [5.0, -4.0, 5.0, 1.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 5.0, 0.0, -2.0, 6.0], [-3.0, -7.0, -2.0, -3.0, 0.0], [0.0, -6.0, 6.0, 0.0, -6.0], [-3.0, -6.0, 6.0, -3.0, -3.0]])+(1.-msk_arg1)*numpy.array([[4.0, 0.0, 6.0, -4.0, 5.0], [-7.0, 0.0, 3.0, -2.0, 1.0], [0.0, 3.0, -7.0, 4.0, -5.0], [-4.0, -5.0, 6.0, 1.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-24.0, 30.0, 0.0, -12.0, 36.0], [-18.0, -42.0, -12.0, -18.0, 0.0], [0.0, -36.0, 36.0, 0.0, -36.0], [-18.0, -36.0, 36.0, -18.0, -18.0]], [[-28.0, 35.0, 0.0, -14.0, 42.0], [-21.0, -49.0, -14.0, -21.0, 0.0], [0.0, -42.0, 42.0, 0.0, -42.0], [-21.0, -42.0, 42.0, -21.0, -21.0]], [[24.0, -30.0, 0.0, 12.0, -36.0], [18.0, 42.0, 12.0, 18.0, 0.0], [0.0, 36.0, -36.0, 0.0, 36.0], [18.0, 36.0, -36.0, 18.0, 18.0]], [[-20.0, 25.0, 0.0, -10.0, 30.0], [-15.0, -35.0, -10.0, -15.0, 0.0], [0.0, -30.0, 30.0, 0.0, -30.0], [-15.0, -30.0, 30.0, -15.0, -15.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-8.0, 10.0, 0.0, -4.0, 12.0], [-6.0, -14.0, -4.0, -6.0, 0.0], [0.0, -12.0, 12.0, 0.0, -12.0], [-6.0, -12.0, 12.0, -6.0, -6.0]], [[16.0, -20.0, 0.0, 8.0, -24.0], [12.0, 28.0, 8.0, 12.0, 0.0], [0.0, 24.0, -24.0, 0.0, 24.0], [12.0, 24.0, -24.0, 12.0, 12.0]], [[4.0, -5.0, 0.0, 2.0, -6.0], [3.0, 7.0, 2.0, 3.0, 0.0], [0.0, 6.0, -6.0, 0.0, 6.0], [3.0, 6.0, -6.0, 3.0, 3.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-24.0, 30.0, 0.0, -12.0, 36.0], [-18.0, -42.0, -12.0, -18.0, 0.0], [0.0, -36.0, 36.0, 0.0, -36.0], [-18.0, -36.0, 36.0, -18.0, -18.0]], [[-8.0, 10.0, 0.0, -4.0, 12.0], [-6.0, -14.0, -4.0, -6.0, 0.0], [0.0, -12.0, 12.0, 0.0, -12.0], [-6.0, -12.0, 12.0, -6.0, -6.0]], [[8.0, -10.0, 0.0, 4.0, -12.0], [6.0, 14.0, 4.0, 6.0, 0.0], [0.0, 12.0, -12.0, 0.0, 12.0], [6.0, 12.0, -12.0, 6.0, 6.0]], [[-20.0, 25.0, 0.0, -10.0, 30.0], [-15.0, -35.0, -10.0, -15.0, 0.0], [0.0, -30.0, 30.0, 0.0, -30.0], [-15.0, -30.0, 30.0, -15.0, -15.0]], [[4.0, -5.0, 0.0, 2.0, -6.0], [3.0, 7.0, 2.0, 3.0, 0.0], [0.0, 6.0, -6.0, 0.0, 6.0], [3.0, 6.0, -6.0, 3.0, 3.0]]], [[[-20.0, 25.0, 0.0, -10.0, 30.0], [-15.0, -35.0, -10.0, -15.0, 0.0], [0.0, -30.0, 30.0, 0.0, -30.0], [-15.0, -30.0, 30.0, -15.0, -15.0]], [[16.0, -20.0, 0.0, 8.0, -24.0], [12.0, 28.0, 8.0, 12.0, 0.0], [0.0, 24.0, -24.0, 0.0, 24.0], [12.0, 24.0, -24.0, 12.0, 12.0]], [[-20.0, 25.0, 0.0, -10.0, 30.0], [-15.0, -35.0, -10.0, -15.0, 0.0], [0.0, -30.0, 30.0, 0.0, -30.0], [-15.0, -30.0, 30.0, -15.0, -15.0]], [[-4.0, 5.0, 0.0, -2.0, 6.0], [-3.0, -7.0, -2.0, -3.0, 0.0], [0.0, -6.0, 6.0, 0.0, -6.0], [-3.0, -6.0, 6.0, -3.0, -3.0]], [[-12.0, 15.0, 0.0, -6.0, 18.0], [-9.0, -21.0, -6.0, -9.0, 0.0], [0.0, -18.0, 18.0, 0.0, -18.0], [-9.0, -18.0, 18.0, -9.0, -9.0]]]])+(1.-msk_ref)*numpy.array([[[[24.0, 0.0, 36.0, -24.0, 30.0], [-42.0, 0.0, 18.0, -12.0, 6.0], [0.0, 18.0, -42.0, 24.0, -30.0], [-24.0, -30.0, 36.0, 6.0, -24.0]], [[28.0, 0.0, 42.0, -28.0, 35.0], [-49.0, 0.0, 21.0, -14.0, 7.0], [0.0, 21.0, -49.0, 28.0, -35.0], [-28.0, -35.0, 42.0, 7.0, -28.0]], [[-24.0, 0.0, -36.0, 24.0, -30.0], [42.0, 0.0, -18.0, 12.0, -6.0], [0.0, -18.0, 42.0, -24.0, 30.0], [24.0, 30.0, -36.0, -6.0, 24.0]], [[20.0, 0.0, 30.0, -20.0, 25.0], [-35.0, 0.0, 15.0, -10.0, 5.0], [0.0, 15.0, -35.0, 20.0, -25.0], [-20.0, -25.0, 30.0, 5.0, -20.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[8.0, 0.0, 12.0, -8.0, 10.0], [-14.0, 0.0, 6.0, -4.0, 2.0], [0.0, 6.0, -14.0, 8.0, -10.0], [-8.0, -10.0, 12.0, 2.0, -8.0]], [[-16.0, 0.0, -24.0, 16.0, -20.0], [28.0, 0.0, -12.0, 8.0, -4.0], [0.0, -12.0, 28.0, -16.0, 20.0], [16.0, 20.0, -24.0, -4.0, 16.0]], [[-4.0, 0.0, -6.0, 4.0, -5.0], [7.0, 0.0, -3.0, 2.0, -1.0], [0.0, -3.0, 7.0, -4.0, 5.0], [4.0, 5.0, -6.0, -1.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[24.0, 0.0, 36.0, -24.0, 30.0], [-42.0, 0.0, 18.0, -12.0, 6.0], [0.0, 18.0, -42.0, 24.0, -30.0], [-24.0, -30.0, 36.0, 6.0, -24.0]], [[8.0, 0.0, 12.0, -8.0, 10.0], [-14.0, 0.0, 6.0, -4.0, 2.0], [0.0, 6.0, -14.0, 8.0, -10.0], [-8.0, -10.0, 12.0, 2.0, -8.0]], [[-8.0, 0.0, -12.0, 8.0, -10.0], [14.0, 0.0, -6.0, 4.0, -2.0], [0.0, -6.0, 14.0, -8.0, 10.0], [8.0, 10.0, -12.0, -2.0, 8.0]], [[20.0, 0.0, 30.0, -20.0, 25.0], [-35.0, 0.0, 15.0, -10.0, 5.0], [0.0, 15.0, -35.0, 20.0, -25.0], [-20.0, -25.0, 30.0, 5.0, -20.0]], [[-4.0, 0.0, -6.0, 4.0, -5.0], [7.0, 0.0, -3.0, 2.0, -1.0], [0.0, -3.0, 7.0, -4.0, 5.0], [4.0, 5.0, -6.0, -1.0, 4.0]]], [[[20.0, 0.0, 30.0, -20.0, 25.0], [-35.0, 0.0, 15.0, -10.0, 5.0], [0.0, 15.0, -35.0, 20.0, -25.0], [-20.0, -25.0, 30.0, 5.0, -20.0]], [[-16.0, 0.0, -24.0, 16.0, -20.0], [28.0, 0.0, -12.0, 8.0, -4.0], [0.0, -12.0, 28.0, -16.0, 20.0], [16.0, 20.0, -24.0, -4.0, 16.0]], [[20.0, 0.0, 30.0, -20.0, 25.0], [-35.0, 0.0, 15.0, -10.0, 5.0], [0.0, 15.0, -35.0, 20.0, -25.0], [-20.0, -25.0, 30.0, 5.0, -20.0]], [[4.0, 0.0, 6.0, -4.0, 5.0], [-7.0, 0.0, 3.0, -2.0, 1.0], [0.0, 3.0, -7.0, 4.0, -5.0], [-4.0, -5.0, 6.0, 1.0, -4.0]], [[12.0, 0.0, 18.0, -12.0, 15.0], [-21.0, 0.0, 9.0, -6.0, 3.0], [0.0, 9.0, -21.0, 12.0, -15.0], [-12.0, -15.0, 18.0, 3.0, -12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank3_offset1(self):
      arg0=numpy.array([[[-4.0, 7.0, -5.0], [-6.0, -3.0, 0.0], [1.0, -1.0, -5.0], [-4.0, 5.0, 7.0], [5.0, 7.0, -7.0]], [[1.0, 0.0, 0.0], [-1.0, -2.0, 2.0], [-4.0, 6.0, 6.0], [0.0, 1.0, 6.0], [2.0, 2.0, -7.0]], [[3.0, 6.0, 0.0], [2.0, -4.0, -1.0], [3.0, 6.0, -4.0], [7.0, -5.0, -6.0], [-6.0, 7.0, -2.0]], [[4.0, -7.0, 3.0], [-3.0, -7.0, 2.0], [7.0, 5.0, -4.0], [4.0, -3.0, 0.0], [-7.0, 6.0, 3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, -3.0, -3.0, -4.0, -5.0], [-6.0, 2.0, 6.0, -6.0, 7.0], [5.0, -4.0, 2.0, 5.0, -5.0], [1.0, 0.0, 7.0, -3.0, -2.0]], [[5.0, -7.0, 2.0, 5.0, -7.0], [6.0, -6.0, 3.0, 2.0, 0.0], [2.0, -1.0, 3.0, 2.0, 0.0], [0.0, 0.0, -4.0, -7.0, -5.0]], [[2.0, -5.0, -5.0, -4.0, 1.0], [3.0, 0.0, -7.0, 6.0, -4.0], [-1.0, -7.0, 0.0, 1.0, 0.0], [0.0, -2.0, -2.0, 6.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, -4.0, 5.0, 0.0, 4.0], [-6.0, 0.0, 2.0, 0.0, 0.0], [0.0, 7.0, 2.0, 1.0, 2.0], [-5.0, 2.0, 4.0, 2.0, 0.0]], [[-5.0, -2.0, 7.0, 7.0, 3.0], [3.0, 4.0, 5.0, 0.0, -6.0], [-5.0, -3.0, 7.0, 0.0, -7.0], [-5.0, -6.0, 0.0, 3.0, 0.0]], [[6.0, 0.0, 4.0, 2.0, 0.0], [6.0, 0.0, 1.0, 1.0, -5.0], [1.0, 5.0, -4.0, -6.0, 0.0], [-7.0, -7.0, -6.0, 4.0, -4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[49.0, -12.0, 51.0, 71.0, -34.0], [51.0, -50.0, 32.0, 8.0, -8.0], [-1.0, 44.0, 13.0, -11.0, 20.0], [-4.0, 10.0, -46.0, -67.0, -12.0]], [[21.0, 39.0, 12.0, 9.0, 51.0], [18.0, 6.0, -45.0, 30.0, -42.0], [-36.0, 27.0, -21.0, -36.0, 30.0], [-6.0, 0.0, -30.0, 39.0, 27.0]], [[-21.0, 29.0, 20.0, 11.0, -3.0], [-27.0, 8.0, 38.0, -38.0, 27.0], [8.0, 32.0, -1.0, -2.0, -5.0], [1.0, 10.0, 21.0, -26.0, 18.0]], [[63.0, -58.0, -13.0, 13.0, -8.0], [75.0, -38.0, -58.0, 76.0, -56.0], [-17.0, -38.0, 7.0, -3.0, 20.0], [-4.0, -14.0, -62.0, 19.0, -38.0]], [[-9.0, -29.0, 34.0, 43.0, -81.0], [-9.0, -32.0, 100.0, -58.0, 63.0], [46.0, 22.0, 31.0, 32.0, -25.0], [5.0, 14.0, 21.0, -106.0, -24.0]]], [[[-6.0, -3.0, -3.0, -4.0, -5.0], [-6.0, 2.0, 6.0, -6.0, 7.0], [5.0, -4.0, 2.0, 5.0, -5.0], [1.0, 0.0, 7.0, -3.0, -2.0]], [[0.0, 7.0, -11.0, -14.0, 21.0], [0.0, 10.0, -26.0, 14.0, -15.0], [-11.0, -8.0, -8.0, -7.0, 5.0], [-1.0, -4.0, -3.0, 29.0, 6.0]], [[66.0, -60.0, -6.0, 22.0, -16.0], [78.0, -44.0, -48.0, 72.0, -52.0], [-14.0, -32.0, 10.0, -2.0, 20.0], [-4.0, -12.0, -64.0, 6.0, -40.0]], [[17.0, -37.0, -28.0, -19.0, -1.0], [24.0, -6.0, -39.0, 38.0, -24.0], [-4.0, -43.0, 3.0, 8.0, 0.0], [0.0, -12.0, -16.0, 29.0, -23.0]], [[-16.0, 15.0, 33.0, 30.0, -31.0], [-21.0, -8.0, 67.0, -50.0, 42.0], [21.0, 39.0, 10.0, 7.0, -10.0], [2.0, 14.0, 20.0, -62.0, 7.0]]], [[[12.0, -51.0, 3.0, 18.0, -57.0], [18.0, -30.0, 36.0, -6.0, 21.0], [27.0, -18.0, 24.0, 27.0, -15.0], [3.0, 0.0, -3.0, -51.0, -36.0]], [[-34.0, 27.0, -9.0, -24.0, 17.0], [-39.0, 28.0, 7.0, -26.0, 18.0], [3.0, 3.0, -8.0, 1.0, -10.0], [2.0, 2.0, 32.0, 16.0, 19.0]], [[4.0, -31.0, 23.0, 34.0, -61.0], [6.0, -30.0, 64.0, -30.0, 37.0], [31.0, 10.0, 24.0, 23.0, -15.0], [3.0, 8.0, 5.0, -75.0, -24.0]], [[-79.0, 44.0, -1.0, -29.0, -6.0], [-90.0, 44.0, 69.0, -88.0, 73.0], [31.0, 19.0, -1.0, 19.0, -35.0], [7.0, 12.0, 81.0, -22.0, 29.0]], [[67.0, -21.0, 42.0, 67.0, -21.0], [72.0, -54.0, -1.0, 38.0, -34.0], [-14.0, 31.0, 9.0, -18.0, 30.0], [-6.0, 4.0, -66.0, -43.0, -17.0]]], [[[-53.0, 22.0, -41.0, -63.0, 32.0], [-57.0, 50.0, -18.0, -20.0, 16.0], [3.0, -30.0, -13.0, 9.0, -20.0], [4.0, -6.0, 50.0, 55.0, 18.0]], [[-13.0, 48.0, -15.0, -31.0, 66.0], [-18.0, 36.0, -53.0, 16.0, -29.0], [-31.0, 5.0, -27.0, -27.0, 15.0], [-3.0, -4.0, 3.0, 70.0, 35.0]], [[-25.0, -36.0, 9.0, 13.0, -74.0], [-24.0, -16.0, 85.0, -56.0, 65.0], [49.0, -5.0, 29.0, 41.0, -35.0], [7.0, 8.0, 37.0, -80.0, -27.0]], [[-39.0, 9.0, -18.0, -31.0, 1.0], [-42.0, 26.0, 15.0, -30.0, 28.0], [14.0, -13.0, -1.0, 14.0, -20.0], [4.0, 0.0, 40.0, 9.0, 7.0]], [[78.0, -36.0, 18.0, 46.0, -4.0], [87.0, -50.0, -45.0, 72.0, -61.0], [-26.0, 1.0, 4.0, -20.0, 35.0], [-7.0, -6.0, -79.0, -3.0, -25.0]]]])+(1.-msk_ref)*numpy.array([[[[-89.0, 2.0, 9.0, 39.0, 5.0], [15.0, 28.0, 22.0, -5.0, -17.0], [-40.0, -74.0, 61.0, 26.0, -57.0], [20.0, -15.0, 14.0, -7.0, 20.0]], [[-21.0, 30.0, -51.0, -21.0, -33.0], [27.0, -12.0, -27.0, 0.0, 18.0], [15.0, -33.0, -33.0, -6.0, 9.0], [45.0, 6.0, -24.0, -21.0, 0.0]], [[-19.0, -2.0, -22.0, -17.0, 1.0], [-39.0, -4.0, -8.0, -5.0, 31.0], [0.0, -15.0, 15.0, 31.0, 9.0], [35.0, 43.0, 34.0, -21.0, 20.0]], [[-7.0, 6.0, 43.0, 49.0, -1.0], [81.0, 20.0, 24.0, 7.0, -65.0], [-18.0, -8.0, -1.0, -46.0, -43.0], [-54.0, -87.0, -58.0, 35.0, -28.0]], [[-47.0, -34.0, 46.0, 35.0, 41.0], [-51.0, 28.0, 38.0, -7.0, -7.0], [-42.0, -21.0, 87.0, 47.0, -39.0], [-11.0, 17.0, 62.0, 3.0, 28.0]]], [[[6.0, -4.0, 5.0, 0.0, 4.0], [-6.0, 0.0, 2.0, 0.0, 0.0], [0.0, 7.0, 2.0, 1.0, 2.0], [-5.0, 2.0, 4.0, 2.0, 0.0]], [[16.0, 8.0, -11.0, -10.0, -10.0], [12.0, -8.0, -10.0, 2.0, 2.0], [12.0, 9.0, -24.0, -13.0, 12.0], [1.0, -4.0, -16.0, 0.0, -8.0]], [[-18.0, 4.0, 46.0, 54.0, 2.0], [78.0, 24.0, 28.0, 6.0, -66.0], [-24.0, -16.0, 10.0, -40.0, -50.0], [-52.0, -86.0, -52.0, 34.0, -24.0]], [[31.0, -2.0, 31.0, 19.0, 3.0], [39.0, 4.0, 11.0, 6.0, -36.0], [1.0, 27.0, -17.0, -36.0, -7.0], [-47.0, -48.0, -36.0, 27.0, -24.0]], [[-40.0, -12.0, -4.0, 0.0, 14.0], [-48.0, 8.0, 7.0, -7.0, 23.0], [-17.0, -27.0, 46.0, 44.0, -10.0], [29.0, 41.0, 50.0, -18.0, 28.0]]], [[[-12.0, -24.0, 57.0, 42.0, 30.0], [0.0, 24.0, 36.0, 0.0, -36.0], [-30.0, 3.0, 48.0, 3.0, -36.0], [-45.0, -30.0, 12.0, 24.0, 0.0]], [[26.0, 0.0, -22.0, -30.0, -4.0], [-30.0, -16.0, -17.0, -1.0, 29.0], [19.0, 21.0, -20.0, 8.0, 32.0], [17.0, 35.0, 14.0, -12.0, 4.0]], [[-36.0, -24.0, 41.0, 34.0, 30.0], [-24.0, 24.0, 32.0, -4.0, -16.0], [-34.0, -17.0, 64.0, 27.0, -36.0], [-17.0, -2.0, 36.0, 8.0, 16.0]], [[31.0, -18.0, -24.0, -47.0, 13.0], [-93.0, -20.0, -17.0, -6.0, 60.0], [19.0, 34.0, 3.0, 43.0, 49.0], [32.0, 86.0, 64.0, -25.0, 24.0]], [[-83.0, 10.0, 11.0, 45.0, -3.0], [45.0, 28.0, 21.0, -2.0, -32.0], [-37.0, -73.0, 45.0, 6.0, -61.0], [9.0, -40.0, -12.0, 1.0, 8.0]]], [[[77.0, -2.0, -17.0, -43.0, -5.0], [-27.0, -28.0, -24.0, 3.0, 27.0], [38.0, 64.0, -53.0, -14.0, 57.0], [-6.0, 29.0, -2.0, -1.0, -12.0]], [[29.0, 26.0, -56.0, -45.0, -33.0], [9.0, -28.0, -39.0, 2.0, 32.0], [37.0, 10.0, -63.0, -15.0, 43.0], [36.0, 22.0, -24.0, -19.0, -8.0]], [[-7.0, -38.0, 54.0, 27.0, 43.0], [-51.0, 20.0, 35.0, -4.0, -10.0], [-29.0, 14.0, 65.0, 31.0, -21.0], [-32.0, 12.0, 52.0, 13.0, 16.0]], [[39.0, -10.0, -1.0, -21.0, 7.0], [-33.0, -12.0, -7.0, 0.0, 18.0], [15.0, 37.0, -13.0, 4.0, 29.0], [-5.0, 26.0, 16.0, -1.0, 0.0]], [[-54.0, 16.0, 19.0, 48.0, -10.0], [78.0, 24.0, 19.0, 3.0, -51.0], [-27.0, -52.0, 16.0, -25.0, -56.0], [-16.0, -71.0, -46.0, 16.0, -12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[[-6.0, 7.0, -2.0], [-2.0, -5.0, 4.0]], [[-2.0, -3.0, 3.0], [-3.0, 2.0, -6.0]], [[1.0, 7.0, -7.0], [-5.0, 5.0, -5.0]], [[4.0, 5.0, -5.0], [6.0, -6.0, -6.0]], [[4.0, -7.0, -1.0], [4.0, 1.0, -5.0]]], [[[-6.0, 7.0, 6.0], [-1.0, 7.0, -2.0]], [[0.0, -6.0, -3.0], [3.0, 2.0, -7.0]], [[0.0, 2.0, 1.0], [-1.0, 0.0, 3.0]], [[-2.0, -1.0, -4.0], [-5.0, -3.0, -2.0]], [[4.0, 7.0, 4.0], [4.0, 2.0, -2.0]]], [[[4.0, -1.0, 0.0], [4.0, 0.0, 2.0]], [[-5.0, 3.0, 4.0], [-7.0, -6.0, -3.0]], [[1.0, 7.0, -6.0], [7.0, 2.0, -2.0]], [[0.0, 0.0, 3.0], [5.0, -5.0, 2.0]], [[-3.0, -7.0, 0.0], [-7.0, -5.0, -7.0]]], [[[0.0, 5.0, 7.0], [3.0, -7.0, -2.0]], [[3.0, -5.0, 7.0], [5.0, -7.0, 0.0]], [[-6.0, 3.0, 5.0], [-4.0, 5.0, 2.0]], [[7.0, 0.0, -3.0], [1.0, 6.0, 0.0]], [[-1.0, 4.0, 5.0], [-2.0, -2.0, -4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, 2.0, -2.0, 4.0, -4.0], [1.0, 1.0, 3.0, 4.0, 2.0], [5.0, -5.0, -3.0, 2.0, -1.0], [-3.0, 6.0, -3.0, -1.0, -7.0]], [[0.0, -4.0, -4.0, -4.0, 2.0], [6.0, 7.0, -7.0, -7.0, 4.0], [0.0, 4.0, -5.0, 1.0, -3.0], [-7.0, 2.0, -5.0, -6.0, 5.0]], [[-7.0, -7.0, -4.0, -3.0, -1.0], [1.0, -3.0, -4.0, -1.0, -6.0], [7.0, 2.0, -6.0, 0.0, 0.0], [-3.0, -5.0, -3.0, -6.0, -7.0]]], [[[0.0, 0.0, -4.0, 5.0, 7.0], [-1.0, -3.0, -3.0, 0.0, 0.0], [2.0, -3.0, 5.0, -3.0, 7.0], [0.0, -6.0, -2.0, 6.0, 2.0]], [[5.0, 7.0, 7.0, 3.0, 7.0], [0.0, 5.0, 3.0, 4.0, -7.0], [5.0, -5.0, 7.0, -1.0, -4.0], [-4.0, 1.0, 1.0, -3.0, -4.0]], [[-1.0, 2.0, -1.0, 0.0, -1.0], [0.0, -3.0, 5.0, -7.0, 6.0], [-1.0, -7.0, -3.0, -6.0, -3.0], [-1.0, 6.0, -6.0, 6.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[2.0, -5.0, 3.0, 3.0, 7.0], [-6.0, -7.0, -4.0, 7.0, -7.0], [3.0, 3.0, 0.0, -1.0, 3.0], [3.0, -7.0, -2.0, -4.0, -3.0]], [[-5.0, -4.0, -1.0, -5.0, 4.0], [-1.0, 6.0, -7.0, 6.0, 5.0], [6.0, -5.0, -1.0, 0.0, -4.0], [-7.0, 1.0, -6.0, -3.0, 7.0]], [[6.0, 0.0, -3.0, 1.0, -2.0], [1.0, 4.0, -4.0, 5.0, 6.0], [-1.0, 4.0, -4.0, 5.0, -2.0], [1.0, 0.0, 0.0, 1.0, -7.0]]], [[[-5.0, 4.0, -2.0, 2.0, -4.0], [7.0, -5.0, 0.0, 2.0, 7.0], [-3.0, -7.0, 2.0, -1.0, 1.0], [6.0, -1.0, 4.0, -4.0, -4.0]], [[2.0, -2.0, 4.0, 7.0, 0.0], [3.0, -5.0, 4.0, -5.0, 6.0], [0.0, -7.0, 0.0, -2.0, -2.0], [-4.0, -2.0, 7.0, -6.0, -3.0]], [[-2.0, -2.0, -6.0, -4.0, -3.0], [5.0, 0.0, 5.0, 2.0, -4.0], [-4.0, -6.0, 1.0, -6.0, -1.0], [2.0, 0.0, 4.0, 6.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[27.0, -53.0, -39.0, -71.0, -13.0], [36.0, 18.0, -48.0, -119.0, 87.0], [-77.0, 57.0, -62.0, -18.0, -21.0], [-9.0, 19.0, -36.0, 3.0, 131.0]], [[9.0, -11.0, 36.0, -14.0, -2.0], [-14.0, 5.0, -12.0, 60.0, -84.0], [21.0, 45.0, 20.0, 36.0, 0.0], [16.0, -49.0, 56.0, -58.0, -72.0]], [[72.0, 48.0, 58.0, -13.0, 22.0], [41.0, 126.0, -13.0, 17.0, 7.0], [-24.0, 34.0, 29.0, 49.0, -62.0], [-46.0, 60.0, 28.0, -76.0, 17.0]], [[-17.0, -31.0, -68.0, 23.0, 5.0], [23.0, 24.0, -69.0, 4.0, 64.0], [-27.0, 44.0, -1.0, 37.0, 65.0], [-2.0, -19.0, -4.0, 14.0, 32.0]], [[-11.0, 40.0, 20.0, 70.0, 11.0], [-43.0, -34.0, 31.0, 105.0, -51.0], [31.0, -32.0, 71.0, 18.0, 56.0], [41.0, -38.0, 49.0, 35.0, -82.0]]], [[[37.0, -37.0, 15.0, -54.0, 76.0], [43.0, 69.0, -77.0, -37.0, -81.0], [47.0, 52.0, -3.0, 3.0, -44.0], [-75.0, -51.0, -14.0, -111.0, -7.0]], [[38.0, 45.0, 45.0, 54.0, 33.0], [-42.0, -11.0, 16.0, 102.0, -62.0], [2.0, 0.0, 98.0, 25.0, 52.0], [50.0, -55.0, 77.0, 24.0, -53.0]], [[-10.0, -9.0, -11.0, -16.0, -7.0], [14.0, 5.0, 0.0, -36.0, 20.0], [2.0, -8.0, -30.0, -13.0, -22.0], [-20.0, 23.0, -29.0, -6.0, 19.0]], [[29.0, 3.0, 25.0, -26.0, -44.0], [-7.0, 9.0, 13.0, 5.0, 25.0], [-61.0, 42.0, -5.0, 25.0, -12.0], [39.0, 21.0, 42.0, -1.0, 27.0]], [[-44.0, -38.0, -52.0, 2.0, 38.0], [46.0, 45.0, -69.0, -15.0, -14.0], [68.0, 8.0, -31.0, 13.0, 1.0], [-79.0, -16.0, -53.0, -64.0, -33.0]]], [[[-30.0, 16.0, -22.0, 40.0, 8.0], [-6.0, -21.0, 17.0, 9.0, 16.0], [26.0, -50.0, 7.0, -17.0, 21.0], [-7.0, 10.0, -27.0, 38.0, -13.0]], [[-20.0, -98.0, -29.0, -97.0, -66.0], [24.0, 4.0, -64.0, -48.0, 2.0], [-38.0, 117.0, -92.0, 38.0, -20.0], [9.0, -26.0, 14.0, -79.0, 14.0]], [[47.0, 26.0, -18.0, 35.0, 81.0], [30.0, 63.0, -47.0, -17.0, 40.0], [-11.0, -6.0, 53.0, -2.0, 25.0], [-40.0, -2.0, -20.0, 17.0, 64.0]], [[-48.0, -52.0, -69.0, 1.0, -5.0], [-2.0, -55.0, -32.0, -37.0, 29.0], [4.0, 2.0, -34.0, -22.0, 49.0], [9.0, -38.0, -36.0, 39.0, 21.0]], [[3.0, -27.0, 34.0, -34.0, -79.0], [-38.0, -35.0, 11.0, 66.0, -41.0], [-47.0, 82.0, -5.0, 55.0, 16.0], [85.0, -37.0, 95.0, -24.0, -50.0]]], [[[-82.0, -122.0, -107.0, -47.0, -23.0], [34.0, -24.0, -103.0, -56.0, 15.0], [22.0, 74.0, -95.0, 15.0, 40.0], [-26.0, -62.0, -47.0, -45.0, -2.0]], [[-105.0, -72.0, -83.0, 15.0, -43.0], [-25.0, -103.0, -20.0, 12.0, -7.0], [39.0, -1.0, -50.0, -7.0, 75.0], [33.0, -64.0, -22.0, 36.0, -57.0]], [[30.0, -20.0, 29.0, -56.0, 30.0], [21.0, 31.0, -22.0, -44.0, -53.0], [20.0, 25.0, -18.0, -14.0, -57.0], [-40.0, -14.0, -11.0, -69.0, 6.0]], [[2.0, 77.0, 36.0, 60.0, 24.0], [3.0, 43.0, 48.0, 55.0, -10.0], [46.0, -74.0, 44.0, 5.0, -24.0], [-36.0, 57.0, -8.0, -1.0, -50.0]], [[-34.0, -75.0, -36.0, -51.0, -17.0], [30.0, 20.0, -71.0, -17.0, -26.0], [20.0, 75.0, -59.0, 34.0, -5.0], [-28.0, -37.0, -6.0, -83.0, -28.0]]]])+(1.-msk_ref)*numpy.array([[[[-67.0, -4.0, -59.0, -110.0, -14.0], [18.0, 111.0, -17.0, 19.0, 5.0], [16.0, -36.0, 1.0, -16.0, -38.0], [-53.0, 61.0, -57.0, 63.0, 100.0]], [[60.0, 18.0, 38.0, 44.0, -2.0], [-27.0, 13.0, -5.0, -45.0, 32.0], [6.0, 64.0, -21.0, 52.0, -1.0], [-20.0, 10.0, 0.0, -16.0, -24.0]], [[-30.0, -53.0, 77.0, 6.0, 84.0], [-65.0, 7.0, -30.0, -31.0, 1.0], [87.0, -30.0, 6.0, -11.0, -21.0], [-113.0, -5.0, -49.0, -72.0, 105.0]], [[-77.0, 8.0, 22.0, -24.0, 52.0], [-40.0, -18.0, -85.0, 63.0, -3.0], [53.0, 3.0, 21.0, 13.0, 26.0], [20.0, -17.0, -80.0, -60.0, 58.0]], [[29.0, 32.0, 48.0, 81.0, 1.0], [-12.0, -99.0, 16.0, -26.0, -15.0], [-21.0, 38.0, 14.0, 15.0, 49.0], [70.0, -41.0, 37.0, -48.0, -68.0]]], [[[12.0, -12.0, -1.0, 8.0, -16.0], [39.0, 78.0, -31.0, -11.0, 156.0], [29.0, -59.0, -35.0, 35.0, -71.0], [-99.0, 36.0, 7.0, -41.0, 10.0]], [[15.0, 46.0, 59.0, 75.0, -9.0], [-5.0, -73.0, 27.0, -69.0, 13.0], [-14.0, 25.0, 17.0, 20.0, 36.0], [35.0, -13.0, 34.0, -51.0, -32.0]], [[-5.0, -18.0, -21.0, -23.0, 1.0], [7.0, 21.0, -3.0, 21.0, -3.0], [2.0, -17.0, -5.0, -12.0, -14.0], [-13.0, 3.0, -4.0, 17.0, 8.0]], [[0.0, 4.0, 17.0, -28.0, 16.0], [-45.0, 32.0, 9.0, -39.0, -60.0], [15.0, 51.0, 5.0, 5.0, 9.0], [-25.0, 24.0, -39.0, 33.0, 58.0]], [[-15.0, -32.0, 5.0, 11.0, 38.0], [-3.0, 0.0, -83.0, 84.0, 79.0], [46.0, -37.0, -17.0, 20.0, -22.0], [-21.0, -29.0, -28.0, -73.0, -11.0]]], [[[-11.0, -4.0, -7.0, 17.0, 2.0], [15.0, -54.0, 1.0, 34.0, -13.0], [-14.0, -23.0, 11.0, -20.0, 18.0], [47.0, -33.0, 22.0, -17.0, -37.0]], [[28.0, 3.0, -22.0, -70.0, 6.0], [-51.0, 134.0, -56.0, 13.0, 1.0], [32.0, 95.0, -36.0, 62.0, -27.0], [-56.0, 57.0, -90.0, 61.0, 57.0]], [[-96.0, -5.0, 20.0, -2.0, 25.0], [26.0, -34.0, -31.0, 19.0, 61.0], [38.0, -107.0, 29.0, -30.0, -8.0], [-22.0, -11.0, -10.0, -83.0, 56.0]], [[-21.0, 26.0, -51.0, -30.0, -32.0], [33.0, 12.0, -22.0, 54.0, 15.0], [-26.0, 0.0, 0.0, 8.0, 7.0], [57.0, 5.0, -7.0, 25.0, -28.0]], [[68.0, 39.0, 34.0, 5.0, 0.0], [-74.0, 39.0, 6.0, -66.0, -65.0], [-2.0, 152.0, -14.0, 62.0, 29.0], [4.0, 31.0, -43.0, 49.0, 10.0]]], [[[-8.0, 10.0, -48.0, -53.0, 0.0], [-8.0, 78.0, -101.0, 102.0, 54.0], [22.0, 43.0, -29.0, 58.0, -15.0], [14.0, 16.0, -75.0, 10.0, -3.0]], [[34.0, 39.0, -45.0, 2.0, -33.0], [8.0, -13.0, -33.0, 71.0, -11.0], [-43.0, 76.0, -13.0, 41.0, 34.0], [109.0, -17.0, -5.0, 32.0, -92.0]], [[29.0, -12.0, -20.0, -9.0, -30.0], [35.0, 75.0, 13.0, -28.0, 81.0], [-1.0, -32.0, -29.0, 13.0, -56.0], [-74.0, 39.0, 21.0, 18.0, 3.0]], [[3.0, -43.0, 52.0, 62.0, 51.0], [-20.0, -96.0, 8.0, 6.0, -24.0], [21.0, -40.0, 14.0, -35.0, 16.0], [0.0, -62.0, 32.0, -71.0, -22.0]], [[22.0, -7.0, -2.0, -20.0, 19.0], [-33.0, 71.0, -72.0, 40.0, 47.0], [38.0, 49.0, -32.0, 56.0, -23.0], [-38.0, 17.0, -60.0, -7.0, 14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_float_rank0_offset0(self):
      arg0=numpy.array([[[-4.0, -1.0], [-5.0, 6.0]], [[6.0, 4.0], [-3.0, -2.0]], [[4.0, -2.0], [-2.0, -6.0]], [[-5.0, -6.0], [0.0, 2.0]], [[-5.0, -1.0], [-6.0, 2.0]], [[6.0, -7.0], [3.0, -7.0]]])
      arg1=2.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-8.0, -2.0], [-10.0, 12.0]], [[12.0, 8.0], [-6.0, -4.0]], [[8.0, -4.0], [-4.0, -12.0]], [[-10.0, -12.0], [0.0, 4.0]], [[-10.0, -2.0], [-12.0, 4.0]], [[12.0, -14.0], [6.0, -14.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank0_offset0(self):
      arg0=numpy.array([[[5.0, -2.0], [-7.0, 1.0]], [[-4.0, -5.0], [0.0, 6.0]], [[2.0, -4.0], [-2.0, 7.0]], [[-7.0, -7.0], [6.0, -7.0]], [[1.0, 3.0], [6.0, 0.0]], [[6.0, -7.0], [1.0, -2.0]]])
      arg1=numpy.array(4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[20.0, -8.0], [-28.0, 4.0]], [[-16.0, -20.0], [0.0, 24.0]], [[8.0, -16.0], [-8.0, 28.0]], [[-28.0, -28.0], [24.0, -28.0]], [[4.0, 12.0], [24.0, 0.0]], [[24.0, -28.0], [4.0, -8.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank1_offset1(self):
      arg0=numpy.array([[[[5.0, -2.0, 7.0], [0.0, -3.0, 6.0]], [[4.0, 4.0, -5.0], [0.0, -1.0, -2.0]]], [[[-4.0, 5.0, 0.0], [5.0, 6.0, -6.0]], [[4.0, -5.0, 0.0], [5.0, -3.0, 4.0]]], [[[3.0, 0.0, 5.0], [7.0, -4.0, 6.0]], [[0.0, 6.0, 0.0], [-4.0, -3.0, -3.0]]], [[[6.0, 2.0, 0.0], [-3.0, -6.0, -2.0]], [[0.0, 1.0, 0.0], [-2.0, 4.0, -5.0]]], [[[5.0, 3.0, 0.0], [2.0, -3.0, -5.0]], [[6.0, 1.0, 3.0], [-1.0, 2.0, 0.0]]], [[[1.0, -4.0, 1.0], [6.0, -6.0, -3.0]], [[0.0, 0.0, 7.0], [2.0, 3.0, -6.0]]]])
      arg1=numpy.array([6.0, -4.0, -2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[24.0, 0.0], [18.0, 8.0]], [[-44.0, 18.0], [44.0, 34.0]], [[8.0, 46.0], [-24.0, -6.0]], [[28.0, 10.0], [-4.0, -18.0]], [[18.0, 34.0], [26.0, -14.0]], [[20.0, 66.0], [-14.0, 12.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_array_rank1_offset0(self):
      arg0=numpy.array([[[2.0, -5.0], [-1.0, -2.0]], [[-1.0, 7.0], [-3.0, -4.0]], [[-2.0, -7.0], [0.0, 6.0]], [[-1.0, 3.0], [5.0, 5.0]], [[6.0, -3.0], [3.0, 0.0]], [[-7.0, 0.0], [-4.0, 0.0]]])
      arg1=numpy.array([1.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[2.0, 8.0], [-5.0, -20.0]], [[-1.0, -4.0], [-2.0, -8.0]]], [[[-1.0, -4.0], [7.0, 28.0]], [[-3.0, -12.0], [-4.0, -16.0]]], [[[-2.0, -8.0], [-7.0, -28.0]], [[0.0, 0.0], [6.0, 24.0]]], [[[-1.0, -4.0], [3.0, 12.0]], [[5.0, 20.0], [5.0, 20.0]]], [[[6.0, 24.0], [-3.0, -12.0]], [[3.0, 12.0], [0.0, 0.0]]], [[[-7.0, -28.0], [0.0, 0.0]], [[-4.0, -16.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank2_offset1(self):
      arg0=numpy.array([[[[4.0, -3.0, -4.0], [4.0, -2.0, 0.0]], [[-4.0, -3.0, -4.0], [2.0, 0.0, 7.0]]], [[[0.0, -6.0, 2.0], [6.0, -1.0, -7.0]], [[-1.0, -5.0, -2.0], [6.0, -6.0, -4.0]]], [[[2.0, 0.0, 4.0], [-2.0, -3.0, 1.0]], [[0.0, 0.0, -5.0], [7.0, 7.0, -6.0]]], [[[7.0, -5.0, 2.0], [-3.0, -3.0, 6.0]], [[6.0, 1.0, -3.0], [-2.0, -2.0, 1.0]]], [[[-6.0, 4.0, 2.0], [-3.0, -6.0, 0.0]], [[2.0, -2.0, -7.0], [7.0, 4.0, 4.0]]], [[[6.0, 2.0, 6.0], [4.0, -7.0, 4.0]], [[6.0, 7.0, -2.0], [-1.0, -5.0, 2.0]]]])
      arg1=numpy.array([[-1.0, -3.0], [-3.0, -6.0], [-3.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[17.0, -18.0], [2.0, 0.0]], [[25.0, 6.0], [-23.0, 36.0]]], [[[12.0, 48.0], [18.0, -54.0]], [[22.0, 21.0], [24.0, -6.0]]], [[[-14.0, 18.0], [8.0, 30.0]], [[15.0, -30.0], [-10.0, -99.0]]], [[[2.0, 21.0], [-6.0, 63.0]], [[0.0, -42.0], [5.0, 24.0]]], [[[-12.0, 6.0], [21.0, 45.0]], [[25.0, -36.0], [-31.0, -21.0]]], [[[-30.0, 6.0], [5.0, 54.0]], [[-21.0, -72.0], [10.0, 45.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[5.0, -6.0], [0.0, -3.0]], [[-1.0, -2.0], [-7.0, 2.0]], [[7.0, -7.0], [5.0, 7.0]], [[-1.0, 1.0], [6.0, 5.0]], [[4.0, -2.0], [4.0, -5.0]], [[-2.0, 0.0], [-4.0, 6.0]]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(5.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[25.0, -30.0], [0.0, -15.0]], [[-5.0, -10.0], [-35.0, 10.0]], [[35.0, -35.0], [25.0, 35.0]], [[-5.0, 5.0], [30.0, 25.0]], [[20.0, -10.0], [20.0, -25.0]], [[-10.0, 0.0], [-20.0, 30.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[[-5.0, -2.0, -2.0], [4.0, -6.0, -4.0]], [[-2.0, -6.0, 0.0], [-3.0, -6.0, 7.0]]], [[[0.0, 2.0, -6.0], [-5.0, 2.0, 1.0]], [[4.0, 2.0, -7.0], [-7.0, 1.0, -5.0]]], [[[3.0, 7.0, 6.0], [-4.0, -2.0, 2.0]], [[-3.0, 4.0, 3.0], [7.0, 2.0, 5.0]]], [[[-2.0, 3.0, -3.0], [0.0, 3.0, 3.0]], [[-3.0, -1.0, -1.0], [0.0, 4.0, -5.0]]], [[[-1.0, 7.0, 2.0], [6.0, 5.0, 5.0]], [[-3.0, -7.0, 2.0], [-7.0, 1.0, -2.0]]], [[[6.0, 2.0, -7.0], [-4.0, 6.0, 6.0]], [[5.0, -3.0, -2.0], [5.0, 6.0, 3.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-5.0, -7.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[53.0, 50.0], [52.0, 8.0]], [[28.0, 4.0], [15.0, 63.0]], [[-106.0, 20.0], [-34.0, -84.0]], [[10.0, -42.0], [29.0, 7.0]], [[-58.0, -100.0], [50.0, 42.0]], [[5.0, -64.0], [10.0, -88.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_Symbol_rank1_offset0(self):
      arg0=numpy.array([[[0.0, -4.0], [-1.0, 2.0]], [[2.0, 1.0], [6.0, -6.0]], [[4.0, -1.0], [0.0, 4.0]], [[-7.0, 0.0], [5.0, -5.0]], [[-2.0, 3.0], [-1.0, 6.0]], [[6.0, -4.0], [-7.0, -6.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-2.0, -5.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.0, 0.0], [8.0, 20.0]], [[2.0, 5.0], [-4.0, -10.0]]], [[[-4.0, -10.0], [-2.0, -5.0]], [[-12.0, -30.0], [12.0, 30.0]]], [[[-8.0, -20.0], [2.0, 5.0]], [[0.0, 0.0], [-8.0, -20.0]]], [[[14.0, 35.0], [0.0, 0.0]], [[-10.0, -25.0], [10.0, 25.0]]], [[[4.0, 10.0], [-6.0, -15.0]], [[2.0, 5.0], [-12.0, -30.0]]], [[[-12.0, -30.0], [8.0, 20.0]], [[14.0, 35.0], [12.0, 30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[[0.0, 0.0, 5.0], [0.0, -7.0, 0.0]], [[2.0, -5.0, 1.0], [1.0, 6.0, 0.0]]], [[[-3.0, -1.0, 5.0], [-7.0, -1.0, -3.0]], [[0.0, 0.0, -4.0], [-7.0, 6.0, 6.0]]], [[[-3.0, 1.0, 6.0], [-4.0, -1.0, 5.0]], [[-7.0, -2.0, -3.0], [0.0, -2.0, -6.0]]], [[[1.0, 2.0, 2.0], [-1.0, 5.0, -3.0]], [[-3.0, -1.0, -2.0], [0.0, -3.0, -3.0]]], [[[-2.0, -7.0, -6.0], [-2.0, 0.0, -4.0]], [[1.0, 3.0, -4.0], [2.0, -4.0, 0.0]]], [[[5.0, 1.0, 1.0], [2.0, 1.0, -2.0]], [[-7.0, -3.0, -2.0], [4.0, 2.0, -7.0]]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[2.0, 2.0], [7.0, -6.0], [-7.0, 6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-35.0, 30.0], [-49.0, 42.0]], [[-38.0, 40.0], [44.0, -34.0]]], [[[-48.0, 30.0], [0.0, -26.0]], [[28.0, -24.0], [-14.0, -14.0]]], [[[-41.0, 24.0], [-50.0, 28.0]], [[-7.0, -20.0], [28.0, -24.0]]], [[[2.0, 2.0], [54.0, -50.0]], [[1.0, -12.0], [0.0, 0.0]]], [[[-11.0, 2.0], [24.0, -28.0]], [[51.0, -40.0], [-24.0, 28.0]]], [[[10.0, 10.0], [25.0, -14.0]], [[-21.0, -8.0], [71.0, -46.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank0_offset0(self):
      arg0=numpy.array([[[-5.0, 4.0], [-3.0, -4.0]], [[4.0, 0.0], [2.0, 7.0]], [[1.0, 1.0], [3.0, -6.0]], [[4.0, 2.0], [2.0, -2.0]], [[6.0, -3.0], [0.0, 0.0]], [[0.0, -6.0], [-1.0, -4.0]]])
      arg1=Data(2.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-10.0, 8.0], [-6.0, -8.0]], [[8.0, 0.0], [4.0, 14.0]], [[2.0, 2.0], [6.0, -12.0]], [[8.0, 4.0], [4.0, -4.0]], [[12.0, -6.0], [0.0, 0.0]], [[0.0, -12.0], [-2.0, -8.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank1_offset1(self):
      arg0=numpy.array([[[[-2.0, -3.0, -1.0], [7.0, -4.0, -1.0]], [[-4.0, 0.0, 6.0], [-3.0, 1.0, 4.0]]], [[[7.0, -2.0, -4.0], [1.0, 0.0, -3.0]], [[-6.0, -2.0, -2.0], [0.0, 6.0, 2.0]]], [[[-6.0, 7.0, -4.0], [3.0, 5.0, 5.0]], [[3.0, 5.0, -4.0], [-1.0, 0.0, 7.0]]], [[[1.0, -1.0, 4.0], [0.0, -3.0, 0.0]], [[3.0, -4.0, 0.0], [-3.0, 7.0, -1.0]]], [[[-4.0, -4.0, 2.0], [0.0, -4.0, -1.0]], [[4.0, 6.0, -5.0], [0.0, 2.0, -7.0]]], [[[-7.0, 3.0, 5.0], [-4.0, -7.0, -5.0]], [[-1.0, -4.0, 0.0], [-1.0, 1.0, -6.0]]]])
      arg1=Data(numpy.array([-5.0, 0.0, -3.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[13.0, -32.0], [2.0, 3.0]], [[-23.0, 4.0], [36.0, -6.0]], [[42.0, -30.0], [-3.0, -16.0]], [[-17.0, 0.0], [-15.0, 18.0]], [[14.0, 3.0], [-5.0, 21.0]], [[20.0, 35.0], [5.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_constData_rank1_offset0(self):
      arg0=numpy.array([[[-4.0, 6.0], [-6.0, -1.0]], [[-1.0, 4.0], [-3.0, 3.0]], [[-6.0, -6.0], [-3.0, 4.0]], [[-2.0, 4.0], [-2.0, 0.0]], [[5.0, 5.0], [3.0, 7.0]], [[3.0, 0.0], [0.0, 0.0]]])
      arg1=Data(numpy.array([-2.0, -4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[8.0, 16.0], [-12.0, -24.0]], [[12.0, 24.0], [2.0, 4.0]]], [[[2.0, 4.0], [-8.0, -16.0]], [[6.0, 12.0], [-6.0, -12.0]]], [[[12.0, 24.0], [12.0, 24.0]], [[6.0, 12.0], [-8.0, -16.0]]], [[[4.0, 8.0], [-8.0, -16.0]], [[4.0, 8.0], [0.0, 0.0]]], [[[-10.0, -20.0], [-10.0, -20.0]], [[-6.0, -12.0], [-14.0, -28.0]]], [[[-6.0, -12.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank2_offset1(self):
      arg0=numpy.array([[[[3.0, 3.0, 0.0], [4.0, 0.0, 3.0]], [[5.0, 1.0, -4.0], [-2.0, -5.0, 7.0]]], [[[-7.0, -6.0, 1.0], [-6.0, -5.0, -4.0]], [[2.0, 0.0, -2.0], [6.0, -7.0, 6.0]]], [[[-2.0, 3.0, 0.0], [0.0, -6.0, -1.0]], [[-1.0, -6.0, -4.0], [-4.0, 0.0, -2.0]]], [[[4.0, 0.0, 6.0], [-5.0, -1.0, 5.0]], [[7.0, -4.0, -2.0], [6.0, 5.0, -3.0]]], [[[1.0, -1.0, 3.0], [3.0, -1.0, 3.0]], [[-6.0, 0.0, -3.0], [-5.0, -3.0, -6.0]]], [[[2.0, 0.0, -3.0], [2.0, -3.0, 2.0]], [[0.0, 7.0, 7.0], [0.0, -1.0, -5.0]]]])
      arg1=Data(numpy.array([[-1.0, 4.0], [0.0, 3.0], [-1.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-3.0, 21.0], [-7.0, 31.0]], [[-1.0, 3.0], [-5.0, 12.0]]], [[[6.0, -41.0], [10.0, -59.0]], [[0.0, -2.0], [-12.0, 33.0]]], [[[2.0, 1.0], [1.0, -23.0]], [[5.0, -42.0], [6.0, -26.0]]], [[[-10.0, 46.0], [0.0, 2.0]], [[-5.0, 6.0], [-3.0, 24.0]]], [[[-4.0, 16.0], [-6.0, 24.0]], [[9.0, -39.0], [11.0, -59.0]]], [[[1.0, -7.0], [-4.0, 9.0]], [[-7.0, 56.0], [5.0, -28.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[-1.0, 3.0], [-1.0, 0.0]], [[-1.0, -2.0], [1.0, 0.0]], [[-5.0, 2.0], [-3.0, 0.0]], [[5.0, -2.0], [-4.0, 0.0]], [[-5.0, 6.0], [6.0, -6.0]], [[5.0, 3.0], [-7.0, 4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, -9.0], [3.0, -0.0]], [[3.0, 6.0], [-3.0, -0.0]], [[15.0, -6.0], [9.0, -0.0]], [[-15.0, 6.0], [12.0, -0.0]], [[15.0, -18.0], [-18.0, 18.0]], [[-15.0, -9.0], [21.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[7.0, -21.0], [7.0, -0.0]], [[7.0, 14.0], [-7.0, -0.0]], [[35.0, -14.0], [21.0, -0.0]], [[-35.0, 14.0], [28.0, -0.0]], [[35.0, -42.0], [-42.0, 42.0]], [[-35.0, -21.0], [49.0, -28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[[-5.0, 0.0, 6.0], [-2.0, 7.0, -1.0]], [[-3.0, -3.0, -1.0], [4.0, 6.0, 0.0]]], [[[-6.0, 3.0, 7.0], [-3.0, -1.0, 0.0]], [[2.0, -2.0, -5.0], [-4.0, 7.0, 6.0]]], [[[0.0, -1.0, 1.0], [5.0, 7.0, -1.0]], [[-6.0, -4.0, 0.0], [-7.0, 1.0, -1.0]]], [[[7.0, -4.0, -5.0], [1.0, -6.0, -4.0]], [[1.0, 4.0, -7.0], [-1.0, -1.0, 5.0]]], [[[4.0, -1.0, 0.0], [3.0, -2.0, 6.0]], [[4.0, -3.0, 4.0], [3.0, 1.0, 4.0]]], [[[-5.0, 4.0, -6.0], [0.0, 0.0, 2.0]], [[-1.0, 0.0, 7.0], [-4.0, 1.0, 2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, -3.0, -4.0])+(1.-msk_arg1)*numpy.array([3.0, -2.0, 6.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, -11.0], [22.0, -30.0]], [[-19.0, 12.0], [20.0, -33.0]], [[-1.0, -32.0], [30.0, 22.0]], [[11.0, 31.0], [13.0, -14.0]], [[-9.0, -27.0], [-19.0, -28.0]], [[27.0, -8.0], [-25.0, 1.0]]])+(1.-msk_ref)*numpy.array([[[21.0, -26.0], [-9.0, 0.0]], [[18.0, -7.0], [-20.0, 10.0]], [[8.0, -5.0], [-10.0, -29.0]], [[-1.0, -9.0], [-47.0, 29.0]], [[14.0, 49.0], [42.0, 31.0]], [[-59.0, 12.0], [39.0, -2.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank3_expandedData_rank1_offset0(self):
      arg0=numpy.array([[[-1.0, 3.0], [-5.0, 5.0]], [[0.0, -1.0], [-2.0, 4.0]], [[-5.0, 5.0], [-7.0, 3.0]], [[-5.0, -4.0], [4.0, -5.0]], [[5.0, -3.0], [1.0, 2.0]], [[-2.0, -6.0], [6.0, 4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, 1.0])+(1.-msk_arg1)*numpy.array([5.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, -1.0], [9.0, 3.0]], [[-15.0, -5.0], [15.0, 5.0]]], [[[0.0, 0.0], [-3.0, -1.0]], [[-6.0, -2.0], [12.0, 4.0]]], [[[-15.0, -5.0], [15.0, 5.0]], [[-21.0, -7.0], [9.0, 3.0]]], [[[-15.0, -5.0], [-12.0, -4.0]], [[12.0, 4.0], [-15.0, -5.0]]], [[[15.0, 5.0], [-9.0, -3.0]], [[3.0, 1.0], [6.0, 2.0]]], [[[-6.0, -2.0], [-18.0, -6.0]], [[18.0, 6.0], [12.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[-5.0, 0.0], [15.0, 0.0]], [[-25.0, 0.0], [25.0, 0.0]]], [[[0.0, 0.0], [-5.0, 0.0]], [[-10.0, 0.0], [20.0, 0.0]]], [[[-25.0, 0.0], [25.0, 0.0]], [[-35.0, 0.0], [15.0, 0.0]]], [[[-25.0, 0.0], [-20.0, 0.0]], [[20.0, 0.0], [-25.0, 0.0]]], [[[25.0, 0.0], [-15.0, 0.0]], [[5.0, 0.0], [10.0, 0.0]]], [[[-10.0, 0.0], [-30.0, 0.0]], [[30.0, 0.0], [20.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[[6.0, 3.0, 4.0], [-7.0, 2.0, 7.0]], [[5.0, 3.0, -4.0], [7.0, -6.0, 6.0]]], [[[-4.0, 6.0, 0.0], [0.0, -2.0, 0.0]], [[3.0, 3.0, 7.0], [-1.0, -5.0, 5.0]]], [[[6.0, 7.0, -1.0], [2.0, 5.0, 6.0]], [[0.0, -4.0, -5.0], [-5.0, 3.0, -3.0]]], [[[7.0, 5.0, -6.0], [0.0, -4.0, 3.0]], [[0.0, 3.0, 2.0], [7.0, 0.0, 6.0]]], [[[4.0, 0.0, 0.0], [-7.0, -5.0, -4.0]], [[-2.0, 2.0, -7.0], [2.0, 5.0, 0.0]]], [[[2.0, 5.0, 3.0], [5.0, -5.0, 1.0]], [[-4.0, -1.0, 4.0], [5.0, -2.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, -2.0], [7.0, -4.0], [0.0, 2.0]])+(1.-msk_arg1)*numpy.array([[2.0, 7.0], [-4.0, -2.0], [-3.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-21.0, -16.0], [63.0, 20.0]], [[-14.0, -30.0], [-91.0, 22.0]]], [[[70.0, -16.0], [-14.0, 8.0]], [[0.0, -4.0], [-28.0, 32.0]]], [[[7.0, -42.0], [21.0, -12.0]], [[-28.0, 6.0], [56.0, -8.0]]], [[[-14.0, -46.0], [-28.0, 22.0]], [[21.0, -8.0], [-49.0, -2.0]]], [[[-28.0, -8.0], [14.0, 26.0]], [[28.0, -18.0], [21.0, -24.0]]], [[[21.0, -18.0], [-70.0, 12.0]], [[21.0, 20.0], [-49.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, 20.0], [-43.0, -81.0]], [[10.0, 45.0], [20.0, 37.0]]], [[[-32.0, -40.0], [8.0, 4.0]], [[-27.0, -13.0], [3.0, -17.0]]], [[[-13.0, 32.0], [-34.0, -20.0]], [[31.0, 28.0], [-13.0, -29.0]]], [[[12.0, 63.0], [7.0, -4.0]], [[-18.0, -14.0], [-4.0, 25.0]]], [[[8.0, 28.0], [18.0, -23.0]], [[9.0, 10.0], [-16.0, 4.0]]], [[[-25.0, -8.0], [27.0, 41.0]], [[-16.0, -42.0], [24.0, 47.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_float_rank0_offset0(self):
      arg0=numpy.array([[[[5.0, 0.0, 3.0, -6.0], [-1.0, 7.0, 0.0, -6.0], [-1.0, 0.0, 0.0, 6.0]], [[5.0, -1.0, 7.0, -4.0], [2.0, 5.0, 0.0, 1.0], [0.0, 1.0, -1.0, -5.0]]], [[[2.0, 0.0, 3.0, 6.0], [-7.0, -1.0, 0.0, -6.0], [-3.0, 5.0, -4.0, 3.0]], [[-5.0, -2.0, 7.0, 4.0], [0.0, 1.0, -1.0, -1.0], [6.0, -2.0, 0.0, 3.0]]], [[[-4.0, 2.0, -2.0, 7.0], [-7.0, 6.0, -5.0, -4.0], [3.0, 1.0, -7.0, 0.0]], [[6.0, 0.0, -4.0, 1.0], [6.0, 0.0, -2.0, -2.0], [-5.0, 5.0, -4.0, 5.0]]]])
      arg1=6.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[30.0, 0.0, 18.0, -36.0], [-6.0, 42.0, 0.0, -36.0], [-6.0, 0.0, 0.0, 36.0]], [[30.0, -6.0, 42.0, -24.0], [12.0, 30.0, 0.0, 6.0], [0.0, 6.0, -6.0, -30.0]]], [[[12.0, 0.0, 18.0, 36.0], [-42.0, -6.0, 0.0, -36.0], [-18.0, 30.0, -24.0, 18.0]], [[-30.0, -12.0, 42.0, 24.0], [0.0, 6.0, -6.0, -6.0], [36.0, -12.0, 0.0, 18.0]]], [[[-24.0, 12.0, -12.0, 42.0], [-42.0, 36.0, -30.0, -24.0], [18.0, 6.0, -42.0, 0.0]], [[36.0, 0.0, -24.0, 6.0], [36.0, 0.0, -12.0, -12.0], [-30.0, 30.0, -24.0, 30.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_array_rank0_offset0(self):
      arg0=numpy.array([[[[-6.0, 0.0, -7.0, -5.0], [-6.0, 0.0, 0.0, 6.0], [-7.0, 0.0, 3.0, 7.0]], [[-7.0, -1.0, 2.0, 5.0], [-3.0, 6.0, 4.0, -4.0], [0.0, 3.0, -2.0, -3.0]]], [[[-6.0, 6.0, 6.0, 1.0], [4.0, 3.0, 2.0, 3.0], [6.0, 7.0, -7.0, -3.0]], [[-5.0, 4.0, -5.0, 0.0], [-2.0, 6.0, -1.0, -2.0], [-1.0, 4.0, 5.0, -6.0]]], [[[-3.0, -1.0, -5.0, -7.0], [5.0, 5.0, 0.0, -3.0], [-1.0, 6.0, -6.0, -2.0]], [[1.0, -1.0, 5.0, 2.0], [-5.0, 6.0, 1.0, -2.0], [6.0, -6.0, 3.0, -3.0]]]])
      arg1=numpy.array(-1.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[6.0, -0.0, 7.0, 5.0], [6.0, -0.0, -0.0, -6.0], [7.0, -0.0, -3.0, -7.0]], [[7.0, 1.0, -2.0, -5.0], [3.0, -6.0, -4.0, 4.0], [-0.0, -3.0, 2.0, 3.0]]], [[[6.0, -6.0, -6.0, -1.0], [-4.0, -3.0, -2.0, -3.0], [-6.0, -7.0, 7.0, 3.0]], [[5.0, -4.0, 5.0, -0.0], [2.0, -6.0, 1.0, 2.0], [1.0, -4.0, -5.0, 6.0]]], [[[3.0, 1.0, 5.0, 7.0], [-5.0, -5.0, -0.0, 3.0], [1.0, -6.0, 6.0, 2.0]], [[-1.0, 1.0, -5.0, -2.0], [5.0, -6.0, -1.0, 2.0], [-6.0, 6.0, -3.0, 3.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[[5.0, -7.0, 4.0, 3.0], [3.0, -5.0, 5.0, -5.0], [-3.0, 7.0, 5.0, -2.0]], [[-6.0, -1.0, 3.0, 1.0], [1.0, 4.0, -4.0, 6.0], [-7.0, 7.0, -6.0, -2.0]]], [[[-6.0, 0.0, 4.0, -6.0], [6.0, 0.0, -7.0, 6.0], [3.0, 0.0, 7.0, -4.0]], [[-3.0, 4.0, 1.0, 2.0], [5.0, -3.0, 3.0, 7.0], [-7.0, 6.0, 5.0, 5.0]]], [[[0.0, -3.0, 4.0, -5.0], [1.0, -6.0, 0.0, 4.0], [-5.0, 4.0, -1.0, -7.0]], [[-7.0, 0.0, 0.0, 1.0], [5.0, 1.0, -2.0, -3.0], [3.0, -1.0, -4.0, 0.0]]]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-6.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-30.0, 42.0, -24.0, -18.0], [-18.0, 30.0, -30.0, 30.0], [18.0, -42.0, -30.0, 12.0]], [[36.0, 6.0, -18.0, -6.0], [-6.0, -24.0, 24.0, -36.0], [42.0, -42.0, 36.0, 12.0]]], [[[36.0, -0.0, -24.0, 36.0], [-36.0, -0.0, 42.0, -36.0], [-18.0, -0.0, -42.0, 24.0]], [[18.0, -24.0, -6.0, -12.0], [-30.0, 18.0, -18.0, -42.0], [42.0, -36.0, -30.0, -30.0]]], [[[-0.0, 18.0, -24.0, 30.0], [-6.0, 36.0, -0.0, -24.0], [30.0, -24.0, 6.0, 42.0]], [[42.0, -0.0, -0.0, -6.0], [-30.0, -6.0, 12.0, 18.0], [-18.0, 6.0, 24.0, -0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_constData_rank0_offset0(self):
      arg0=numpy.array([[[[1.0, 1.0, -5.0, 6.0], [1.0, 6.0, 7.0, 0.0], [2.0, -6.0, -5.0, 5.0]], [[0.0, -2.0, 0.0, 2.0], [3.0, -3.0, -6.0, -4.0], [-2.0, 0.0, -4.0, 2.0]]], [[[0.0, 1.0, 4.0, -4.0], [-6.0, -3.0, -6.0, 3.0], [0.0, -1.0, -2.0, 5.0]], [[6.0, -1.0, 6.0, 4.0], [0.0, 3.0, 0.0, 7.0], [-1.0, 6.0, -1.0, 5.0]]], [[[-2.0, -4.0, -3.0, -7.0], [0.0, -2.0, -3.0, 5.0], [3.0, -2.0, 1.0, 0.0]], [[-2.0, 7.0, -5.0, -6.0], [1.0, -5.0, -7.0, -6.0], [-1.0, 5.0, -2.0, 3.0]]]])
      arg1=Data(6.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[6.0, 6.0, -30.0, 36.0], [6.0, 36.0, 42.0, 0.0], [12.0, -36.0, -30.0, 30.0]], [[0.0, -12.0, 0.0, 12.0], [18.0, -18.0, -36.0, -24.0], [-12.0, 0.0, -24.0, 12.0]]], [[[0.0, 6.0, 24.0, -24.0], [-36.0, -18.0, -36.0, 18.0], [0.0, -6.0, -12.0, 30.0]], [[36.0, -6.0, 36.0, 24.0], [0.0, 18.0, 0.0, 42.0], [-6.0, 36.0, -6.0, 30.0]]], [[[-12.0, -24.0, -18.0, -42.0], [0.0, -12.0, -18.0, 30.0], [18.0, -12.0, 6.0, 0.0]], [[-12.0, 42.0, -30.0, -36.0], [6.0, -30.0, -42.0, -36.0], [-6.0, 30.0, -12.0, 18.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_array_rank4_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[[-1.0, 2.0, 0.0, -7.0], [3.0, 1.0, 3.0, 7.0], [2.0, 4.0, 0.0, -2.0]], [[2.0, 0.0, 7.0, -5.0], [-1.0, 0.0, -3.0, -3.0], [0.0, -1.0, -7.0, 2.0]]], [[[0.0, 4.0, -2.0, 4.0], [-1.0, 6.0, -7.0, 2.0], [0.0, -1.0, 5.0, 0.0]], [[-3.0, 3.0, 1.0, 5.0], [3.0, -5.0, -1.0, 0.0], [5.0, 0.0, 5.0, 0.0]]], [[[-6.0, -3.0, -4.0, -3.0], [0.0, -7.0, -3.0, -7.0], [3.0, 0.0, 0.0, 3.0]], [[-2.0, 3.0, 7.0, -4.0], [0.0, 3.0, 4.0, -1.0], [3.0, 2.0, 4.0, 4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0]]], [[[0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0]]], [[[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0]]], [[[0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0]]], [[[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_float_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=-2.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(4.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-8.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_array_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-49.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([-4.0, -3.0, -6.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([2.0, -3.0, 7.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-41.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[1.0, -1.0, -2.0], [-7.0, 7.0, -1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-5.0, 6.0, -2.0], [-2.0, 5.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(43.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[5.0, -5.0, 1.0], [-3.0, -3.0, 4.0], [4.0, 4.0, -3.0], [0.0, 7.0, -3.0]], [[0.0, 6.0, -2.0], [-4.0, -6.0, -4.0], [6.0, -6.0, -2.0], [-7.0, 0.0, 7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-1.0, -3.0, 2.0], [1.0, 4.0, 0.0], [-3.0, -5.0, -7.0], [7.0, -4.0, 0.0]], [[-2.0, 7.0, -7.0], [-7.0, -3.0, 6.0], [-5.0, -3.0, 0.0], [-4.0, -5.0, -3.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(31.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=numpy.array([[[[-3.0, 1.0], [-1.0, 5.0], [-5.0, -1.0]], [[0.0, 5.0], [6.0, 0.0], [0.0, 3.0]]], [[[5.0, -1.0], [7.0, -2.0], [5.0, -5.0]], [[3.0, 2.0], [4.0, 5.0], [0.0, 4.0]]], [[[6.0, 4.0], [-7.0, -4.0], [-6.0, 4.0]], [[-5.0, -2.0], [-2.0, 0.0], [-2.0, -1.0]]], [[[-4.0, -5.0], [0.0, 4.0], [-2.0, -4.0]], [[-5.0, 6.0], [-2.0, 1.0], [-6.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[0.0, -4.0], [-3.0, 6.0], [-4.0, 4.0]], [[0.0, 4.0], [6.0, 2.0], [5.0, -4.0]]], [[[-6.0, 4.0], [3.0, 1.0], [-4.0, 1.0]], [[-6.0, -1.0], [0.0, -5.0], [4.0, -4.0]]], [[[-2.0, -6.0], [-5.0, -5.0], [-7.0, 5.0]], [[2.0, 0.0], [0.0, 6.0], [7.0, -6.0]]], [[[-6.0, 2.0], [5.0, -7.0], [5.0, -6.0]], [[7.0, 0.0], [3.0, 3.0], [-2.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(30.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_array_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([7.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[0.0, -7.0], [-1.0, 7.0], [-7.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-7.0, 1.0, 2.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-15.0, 62.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[-3.0, -7.0], [-1.0, -2.0], [5.0, -1.0]], [[-3.0, -1.0], [-7.0, -7.0], [0.0, 3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-4.0, -6.0, -3.0], [0.0, -2.0, 5.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([17.0, 72.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[[3.0, -6.0], [3.0, -3.0], [-6.0, -1.0]], [[0.0, -3.0], [-5.0, 0.0], [-6.0, -6.0]], [[-1.0, 6.0], [5.0, -4.0], [-2.0, -7.0]], [[3.0, -7.0], [-7.0, -2.0], [0.0, 7.0]]], [[[5.0, 3.0], [0.0, 0.0], [3.0, -2.0]], [[4.0, -5.0], [-2.0, -4.0], [0.0, 2.0]], [[-4.0, 5.0], [3.0, -6.0], [1.0, -4.0]], [[5.0, 7.0], [-4.0, 4.0], [5.0, 3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[3.0, -5.0, -7.0], [2.0, -4.0, -5.0], [-1.0, -3.0, -5.0], [-7.0, 7.0, 0.0]], [[6.0, 3.0, 1.0], [-3.0, -2.0, -6.0], [-5.0, 5.0, 1.0], [-7.0, 7.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([30.0, 63.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_array_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-6.0, 0.0, 3.0, -5.0, -3.0], [7.0, 2.0, 5.0, 0.0, -2.0], [-3.0, 7.0, -5.0, 5.0, 0.0], [4.0, 6.0, 0.0, 2.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-6.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[36.0, -0.0, -18.0, 30.0, 18.0], [-42.0, -12.0, -30.0, -0.0, 12.0], [18.0, -42.0, 30.0, -30.0, -0.0], [-24.0, -36.0, -0.0, -12.0, -24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[3.0, -3.0, 1.0, -4.0, 7.0], [5.0, 1.0, 5.0, 5.0, 4.0], [0.0, 5.0, 3.0, -1.0, -6.0], [2.0, -7.0, -6.0, 4.0, 3.0]], [[-3.0, 0.0, 4.0, -1.0, 0.0], [0.0, -2.0, 4.0, 2.0, -2.0], [3.0, -7.0, 0.0, 0.0, 5.0], [-5.0, 2.0, -2.0, 0.0, 7.0]], [[-6.0, 1.0, 5.0, -2.0, -4.0], [5.0, 7.0, -2.0, 0.0, -3.0], [4.0, 0.0, 2.0, 4.0, -5.0], [7.0, 4.0, 0.0, 1.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 0.0, 1.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-21.0, 16.0, 0.0, 18.0, -39.0], [-20.0, 2.0, -27.0, -25.0, -23.0], [4.0, -25.0, -13.0, 9.0, 25.0], [-3.0, 39.0, 30.0, -19.0, -14.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[[7.0, -6.0, -7.0, 6.0, 0.0], [-7.0, -3.0, 7.0, 5.0, -1.0], [4.0, 6.0, -5.0, -6.0, -7.0], [5.0, 0.0, 3.0, 6.0, 1.0]], [[-7.0, 0.0, 1.0, -2.0, -1.0], [6.0, -7.0, 4.0, 2.0, 2.0], [0.0, 7.0, -7.0, -4.0, 0.0], [-3.0, 0.0, -6.0, 0.0, 1.0]], [[2.0, -5.0, 0.0, 3.0, 1.0], [7.0, 1.0, -6.0, 0.0, 6.0], [-4.0, 4.0, 7.0, -6.0, -6.0], [0.0, 2.0, 4.0, -1.0, -5.0]]], [[[0.0, 3.0, 3.0, -1.0, 6.0], [1.0, -7.0, -1.0, -4.0, -5.0], [5.0, 5.0, -2.0, -3.0, -2.0], [-6.0, -7.0, 1.0, 7.0, 5.0]], [[6.0, 2.0, -4.0, -6.0, 7.0], [-3.0, -7.0, 7.0, 7.0, -7.0], [-2.0, 0.0, 3.0, 1.0, 7.0], [5.0, -2.0, 7.0, 4.0, -7.0]], [[-5.0, -7.0, 2.0, -6.0, -3.0], [1.0, -1.0, 3.0, 5.0, 3.0], [-6.0, 0.0, 0.0, 0.0, -5.0], [6.0, 3.0, 0.0, 2.0, 3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[0.0, -6.0, 7.0], [4.0, 2.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[33.0, -68.0, 12.0, -25.0, 30.0], [18.0, 0.0, -35.0, 21.0, 17.0], [-54.0, 6.0, 89.0, -28.0, -71.0], [46.0, 3.0, 82.0, 43.0, -14.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_array_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[7.0, 6.0], [-3.0, -3.0]], [[6.0, 7.0], [1.0, 3.0]], [[-4.0, -3.0], [6.0, 1.0]], [[4.0, 1.0], [-6.0, 5.0]], [[3.0, 7.0], [-1.0, 0.0]], [[4.0, 4.0], [7.0, 5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.0, 0.0], [-0.0, -0.0]], [[0.0, 0.0], [0.0, 0.0]], [[-0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[[-6.0, 0.0], [2.0, -2.0]], [[-3.0, -1.0], [3.0, -3.0]], [[-3.0, 4.0], [1.0, -4.0]], [[0.0, -7.0], [3.0, 6.0]], [[-7.0, 5.0], [-1.0, -7.0]], [[-4.0, 1.0], [7.0, 6.0]]], [[[6.0, -1.0], [-2.0, -7.0]], [[-3.0, 6.0], [-2.0, 6.0]], [[7.0, 0.0], [-1.0, -7.0]], [[-3.0, -7.0], [3.0, -4.0]], [[-3.0, 0.0], [3.0, -1.0]], [[6.0, -2.0], [-2.0, -4.0]]], [[[-2.0, -5.0], [1.0, 6.0]], [[-4.0, 3.0], [2.0, 1.0]], [[3.0, -5.0], [-2.0, 3.0]], [[-7.0, 6.0], [4.0, 0.0]], [[-5.0, -3.0], [-7.0, 1.0]], [[-1.0, 2.0], [1.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([5.0, -7.0, 4.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-80.0, -13.0], [28.0, 63.0]], [[-10.0, -35.0], [37.0, -53.0]], [[-52.0, 0.0], [4.0, 41.0]], [[-7.0, 38.0], [10.0, 58.0]], [[-34.0, 13.0], [-54.0, -24.0]], [[-66.0, 27.0], [53.0, 54.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_array_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[0.0, 5.0, -7.0, 0.0], [2.0, -2.0, 6.0, -3.0], [-2.0, 6.0, 5.0, 3.0]], [[-4.0, -6.0, 1.0, 3.0], [-6.0, 2.0, 1.0, -3.0], [0.0, 1.0, 1.0, 3.0]]], [[[5.0, 3.0, 2.0, -3.0], [-4.0, 2.0, -4.0, -4.0], [-5.0, 1.0, 3.0, 4.0]], [[4.0, 0.0, 5.0, -3.0], [7.0, 6.0, -2.0, -4.0], [1.0, 7.0, -4.0, 2.0]]], [[[2.0, -5.0, -1.0, 7.0], [7.0, 3.0, -4.0, 0.0], [-3.0, -1.0, 1.0, 3.0]], [[6.0, 0.0, 4.0, -4.0], [-5.0, -7.0, 6.0, 4.0], [-2.0, 0.0, 4.0, 7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-5.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.0, -25.0, 35.0, -0.0], [-10.0, 10.0, -30.0, 15.0], [10.0, -30.0, -25.0, -15.0]], [[20.0, 30.0, -5.0, -15.0], [30.0, -10.0, -5.0, 15.0], [-0.0, -5.0, -5.0, -15.0]]], [[[-25.0, -15.0, -10.0, 15.0], [20.0, -10.0, 20.0, 20.0], [25.0, -5.0, -15.0, -20.0]], [[-20.0, -0.0, -25.0, 15.0], [-35.0, -30.0, 10.0, 20.0], [-5.0, -35.0, 20.0, -10.0]]], [[[-10.0, 25.0, 5.0, -35.0], [-35.0, -15.0, 20.0, -0.0], [15.0, 5.0, -5.0, -15.0]], [[-30.0, -0.0, -20.0, 20.0], [25.0, 35.0, -30.0, -20.0], [10.0, -0.0, -20.0, -35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      s1=numpy.array(-5.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-0.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([1.0, -7.0, -1.0])
      s1=numpy.array([-5.0, 0.0, -4.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-1.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-1.0, 4.0, -3.0], [-2.0, 3.0, 0.0]])
      s1=numpy.array([[7.0, -6.0, 3.0], [-6.0, -6.0, 7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-46.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[5.0, 7.0, -5.0], [-5.0, -6.0, -2.0], [6.0, -5.0, 6.0], [-7.0, -4.0, 1.0]], [[4.0, -3.0, -5.0], [1.0, 6.0, -3.0], [0.0, 4.0, 0.0], [-6.0, -6.0, 0.0]]])
      s1=numpy.array([[[0.0, 6.0, 4.0], [-7.0, 5.0, 1.0], [-5.0, -1.0, 0.0], [-2.0, 2.0, -2.0]], [[4.0, 1.0, -7.0], [-2.0, -1.0, 2.0], [-1.0, -2.0, 3.0], [4.0, 4.0, 2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-18.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[2.0, 0.0], [-3.0, -1.0], [0.0, 0.0]], [[4.0, -3.0], [6.0, 0.0], [-7.0, 6.0]]], [[[0.0, -3.0], [5.0, -1.0], [-6.0, 3.0]], [[1.0, -5.0], [2.0, -7.0], [-2.0, -1.0]]], [[[1.0, -6.0], [0.0, -3.0], [0.0, -6.0]], [[-7.0, -2.0], [0.0, -1.0], [-6.0, -1.0]]], [[[-2.0, 4.0], [6.0, 4.0], [-7.0, -1.0]], [[-1.0, 3.0], [-7.0, -7.0], [3.0, 7.0]]]])
      s1=numpy.array([[[[1.0, -1.0], [7.0, 0.0], [3.0, -5.0]], [[1.0, 0.0], [-5.0, -1.0], [-2.0, 2.0]]], [[[-7.0, -2.0], [3.0, 5.0], [7.0, 0.0]], [[0.0, -5.0], [-1.0, -6.0], [-1.0, -4.0]]], [[[-2.0, 2.0], [6.0, -2.0], [1.0, -7.0]], [[0.0, -5.0], [-4.0, 1.0], [-6.0, 7.0]]], [[[5.0, -3.0], [-5.0, -6.0], [7.0, -4.0]], [[0.0, 0.0], [-3.0, 0.0], [6.0, 5.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(51.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(1.0)
      s1=numpy.array([-1.0, 0.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-1.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([2.0, 2.0, -7.0])
      s1=numpy.array([[2.0, 5.0], [5.0, -1.0], [-2.0, 7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([28.0, -41.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[2.0, 0.0, 2.0], [0.0, -7.0, 0.0]])
      s1=numpy.array([[[2.0, 3.0], [3.0, 0.0], [6.0, -6.0]], [[2.0, -3.0], [0.0, 4.0], [-7.0, -2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([16.0, -34.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[6.0, 3.0, 7.0], [6.0, -6.0, -5.0], [3.0, -1.0, -4.0], [-7.0, 5.0, -5.0]], [[6.0, 0.0, 3.0], [-3.0, 5.0, 6.0], [1.0, -5.0, 6.0], [4.0, -7.0, 6.0]]])
      s1=numpy.array([[[[-6.0, -5.0], [6.0, -6.0], [-2.0, -4.0]], [[5.0, -3.0], [0.0, 4.0], [6.0, -3.0]], [[1.0, 1.0], [3.0, 4.0], [3.0, -7.0]], [[7.0, -6.0], [-6.0, -6.0], [-1.0, 6.0]]], [[[-2.0, 2.0], [-4.0, -5.0], [-1.0, 0.0]], [[7.0, 0.0], [4.0, -2.0], [0.0, -7.0]], [[2.0, -3.0], [7.0, -7.0], [-3.0, 0.0]], [[-5.0, -4.0], [-7.0, 5.0], [0.0, 6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-156.0, -117.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-4.0)
      s1=numpy.array([[0.0, 3.0, -1.0, 7.0, 0.0], [-7.0, 0.0, -4.0, 2.0, 0.0], [-6.0, -5.0, 3.0, -4.0, -6.0], [1.0, -5.0, 2.0, 1.0, 5.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-0.0, -12.0, 4.0, -28.0, -0.0], [28.0, -0.0, 16.0, -8.0, -0.0], [24.0, 20.0, -12.0, 16.0, 24.0], [-4.0, 20.0, -8.0, -4.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([0.0, 2.0, -4.0])
      s1=numpy.array([[[1.0, -1.0, -3.0, -6.0, 3.0], [-1.0, 0.0, -7.0, -7.0, -5.0], [-2.0, 4.0, 2.0, 5.0, 4.0], [4.0, -3.0, 4.0, 0.0, -2.0]], [[-7.0, 3.0, 4.0, 0.0, 5.0], [-1.0, 7.0, 0.0, -2.0, 3.0], [-6.0, -1.0, 1.0, 2.0, 2.0], [4.0, 0.0, 7.0, -4.0, -7.0]], [[3.0, -7.0, -1.0, 4.0, 7.0], [7.0, 4.0, 0.0, 7.0, -1.0], [-4.0, -2.0, 6.0, 5.0, 1.0], [6.0, 0.0, 1.0, 2.0, 6.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-26.0, 34.0, 12.0, -16.0, -18.0], [-30.0, -2.0, 0.0, -32.0, 10.0], [4.0, 6.0, -22.0, -16.0, 0.0], [-16.0, 0.0, 10.0, -16.0, -38.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-1.0, 0.0, 7.0], [7.0, -3.0, 6.0]])
      s1=numpy.array([[[[-7.0, -1.0, -6.0, 6.0, -5.0], [-1.0, 0.0, -1.0, 0.0, 4.0], [-2.0, -5.0, -3.0, 2.0, 2.0], [-1.0, 0.0, 5.0, -7.0, 2.0]], [[6.0, 0.0, -1.0, -4.0, -7.0], [7.0, 3.0, -1.0, 7.0, 1.0], [4.0, 4.0, 2.0, 4.0, 2.0], [-4.0, 1.0, -2.0, -3.0, -6.0]], [[6.0, -5.0, -4.0, 4.0, -5.0], [2.0, -6.0, 4.0, -6.0, 0.0], [6.0, -6.0, 6.0, 2.0, 5.0], [2.0, 7.0, 2.0, 2.0, 0.0]]], [[[-6.0, -5.0, -6.0, 7.0, 6.0], [0.0, -3.0, 1.0, 3.0, 5.0], [-6.0, 0.0, -6.0, -3.0, 1.0], [3.0, -4.0, -7.0, 3.0, 6.0]], [[-6.0, -7.0, 2.0, -3.0, 3.0], [-1.0, 0.0, -7.0, -7.0, -1.0], [1.0, 0.0, -4.0, -3.0, -6.0], [4.0, -1.0, 0.0, -3.0, -6.0]], [[0.0, -3.0, 6.0, -1.0, -4.0], [-3.0, 4.0, 0.0, -7.0, 0.0], [-1.0, -1.0, -5.0, -1.0, -6.0], [6.0, 4.0, 0.0, -3.0, -2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[25.0, -66.0, -34.0, 74.0, -21.0], [0.0, -39.0, 57.0, -42.0, 34.0], [-7.0, -43.0, -15.0, -6.0, 22.0], [60.0, 48.0, -40.0, 33.0, 46.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-6.0)
      s1=numpy.array([[[4.0, 3.0], [4.0, -5.0]], [[3.0, 7.0], [0.0, -6.0]], [[6.0, 4.0], [0.0, 4.0]], [[-7.0, -6.0], [1.0, -4.0]], [[0.0, -4.0], [0.0, 3.0]], [[-1.0, 4.0], [-5.0, -2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-24.0, -18.0], [-24.0, 30.0]], [[-18.0, -42.0], [-0.0, 36.0]], [[-36.0, -24.0], [-0.0, -24.0]], [[42.0, 36.0], [-6.0, 24.0]], [[-0.0, 24.0], [-0.0, -18.0]], [[6.0, -24.0], [30.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-7.0, 7.0, 0.0])
      s1=numpy.array([[[[5.0, -4.0], [5.0, 6.0]], [[-1.0, -5.0], [2.0, -2.0]], [[3.0, 1.0], [7.0, -6.0]], [[1.0, 5.0], [-5.0, -1.0]], [[6.0, -6.0], [7.0, 4.0]], [[-2.0, 0.0], [1.0, 7.0]]], [[[3.0, -4.0], [-1.0, 2.0]], [[4.0, -2.0], [-6.0, 3.0]], [[1.0, -7.0], [4.0, -6.0]], [[2.0, 0.0], [-1.0, 3.0]], [[-3.0, 0.0], [6.0, -6.0]], [[-1.0, 4.0], [-5.0, 0.0]]], [[[-4.0, -4.0], [-3.0, 1.0]], [[1.0, -2.0], [-1.0, 4.0]], [[-7.0, -4.0], [2.0, -4.0]], [[-7.0, 7.0], [-7.0, -1.0]], [[-1.0, 2.0], [5.0, -5.0]], [[1.0, -4.0], [-4.0, -6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-14.0, 0.0], [-42.0, -28.0]], [[35.0, 21.0], [-56.0, 35.0]], [[-14.0, -56.0], [-21.0, 0.0]], [[7.0, -35.0], [28.0, 28.0]], [[-63.0, 42.0], [-7.0, -70.0]], [[7.0, 28.0], [-42.0, -49.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_Symbol_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-5.0)
      s1=numpy.array([[[[-3.0, 0.0, -3.0, 0.0], [-1.0, -7.0, -7.0, -2.0], [-2.0, -6.0, -5.0, 3.0]], [[5.0, -5.0, 4.0, 0.0], [-5.0, 4.0, 0.0, -2.0], [-6.0, 3.0, 2.0, 4.0]]], [[[0.0, 4.0, 6.0, 5.0], [6.0, 0.0, 0.0, 2.0], [7.0, 3.0, 6.0, 1.0]], [[6.0, 6.0, 1.0, -7.0], [-4.0, -1.0, 4.0, 5.0], [5.0, -5.0, 5.0, -5.0]]], [[[2.0, -4.0, -7.0, 0.0], [-6.0, 7.0, 0.0, 1.0], [-2.0, 5.0, -3.0, 1.0]], [[1.0, 2.0, 7.0, 0.0], [3.0, 3.0, 6.0, -6.0], [-5.0, -3.0, -7.0, 7.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[15.0, -0.0, 15.0, -0.0], [5.0, 35.0, 35.0, 10.0], [10.0, 30.0, 25.0, -15.0]], [[-25.0, 25.0, -20.0, -0.0], [25.0, -20.0, -0.0, 10.0], [30.0, -15.0, -10.0, -20.0]]], [[[-0.0, -20.0, -30.0, -25.0], [-30.0, -0.0, -0.0, -10.0], [-35.0, -15.0, -30.0, -5.0]], [[-30.0, -30.0, -5.0, 35.0], [20.0, 5.0, -20.0, -25.0], [-25.0, 25.0, -25.0, 25.0]]], [[[-10.0, 20.0, 35.0, -0.0], [30.0, -35.0, -0.0, -5.0], [10.0, -25.0, 15.0, -5.0]], [[-5.0, -10.0, -35.0, -0.0], [-15.0, -15.0, -30.0, 30.0], [25.0, 15.0, 35.0, -35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_constData_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(-4.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=Data(-0.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([-3.0, 6.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([3.0, 0.0, -3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(9.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[0.0, 4.0, -2.0], [-6.0, -5.0, -6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, 0.0, -5.0], [-2.0, 2.0, -2.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(24.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[4.0, -3.0, -6.0], [1.0, 7.0, 5.0], [1.0, 2.0, -2.0], [4.0, 0.0, 0.0]], [[-6.0, 0.0, -1.0], [4.0, 0.0, 0.0], [-7.0, -6.0, 7.0], [4.0, 0.0, -6.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-3.0, 6.0, 4.0], [-5.0, -1.0, -6.0], [-6.0, 4.0, -6.0], [-5.0, 2.0, 4.0]], [[-2.0, 4.0, 7.0], [-1.0, 3.0, 3.0], [3.0, -4.0, 0.0], [-1.0, 6.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-72.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Data(numpy.array([[[[4.0, 4.0], [1.0, -7.0], [-2.0, -1.0]], [[4.0, -1.0], [-7.0, 3.0], [-7.0, -7.0]]], [[[7.0, 3.0], [1.0, -5.0], [-3.0, -1.0]], [[-4.0, 3.0], [-4.0, -7.0], [2.0, 3.0]]], [[[-6.0, -3.0], [-4.0, 2.0], [7.0, -5.0]], [[-3.0, 2.0], [-2.0, 6.0], [7.0, -7.0]]], [[[-2.0, -2.0], [5.0, -4.0], [-5.0, -2.0]], [[0.0, 1.0], [-1.0, 1.0], [-4.0, 6.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[-6.0, -3.0], [-6.0, 0.0], [-2.0, -7.0]], [[-2.0, -2.0], [-6.0, -7.0], [-3.0, -4.0]]], [[[-1.0, -4.0], [0.0, -2.0], [-3.0, -1.0]], [[4.0, 6.0], [-3.0, 3.0], [-2.0, 1.0]]], [[[3.0, 0.0], [6.0, -3.0], [0.0, 1.0]], [[3.0, 0.0], [5.0, -5.0], [-5.0, -7.0]]], [[[2.0, 3.0], [0.0, -3.0], [6.0, 6.0]], [[-7.0, 0.0], [-1.0, 0.0], [-5.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-75.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_constData_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([1.0, -4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(6.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([6.0, -24.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[7.0, -6.0], [-2.0, -7.0], [7.0, 1.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([2.0, -5.0, -4.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-4.0, 19.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[-5.0, 2.0], [2.0, -7.0], [-3.0, 7.0]], [[4.0, 7.0], [-7.0, -5.0], [-3.0, -1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[3.0, 2.0, 0.0], [2.0, 2.0, 6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-35.0, -10.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[[-1.0, -4.0], [7.0, 1.0], [4.0, 3.0]], [[0.0, 2.0], [0.0, 3.0], [-5.0, 4.0]], [[-1.0, -5.0], [-1.0, -1.0], [1.0, -3.0]], [[7.0, 2.0], [3.0, 5.0], [1.0, 2.0]]], [[[0.0, -7.0], [6.0, 1.0], [-5.0, 5.0]], [[2.0, 0.0], [-6.0, -7.0], [-2.0, -7.0]], [[-6.0, 2.0], [-1.0, 6.0], [-5.0, -1.0]], [[4.0, -5.0], [-3.0, -6.0], [-4.0, -3.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[4.0, -4.0, -6.0], [1.0, -6.0, 4.0], [3.0, 6.0, 4.0], [2.0, 6.0, 1.0]], [[-7.0, 6.0, 5.0], [2.0, 7.0, -2.0], [2.0, 2.0, 5.0], [-1.0, 1.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-97.0, 35.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_constData_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[-4.0, 6.0, -6.0, 3.0, -6.0], [-7.0, -3.0, -6.0, 0.0, 1.0], [4.0, -5.0, -2.0, 2.0, 5.0], [-7.0, 7.0, 3.0, 4.0, 4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(7.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-28.0, 42.0, -42.0, 21.0, -42.0], [-49.0, -21.0, -42.0, 0.0, 7.0], [28.0, -35.0, -14.0, 14.0, 35.0], [-49.0, 49.0, 21.0, 28.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[-2.0, 3.0, 7.0, -2.0, 0.0], [3.0, -2.0, -4.0, -3.0, 4.0], [7.0, -4.0, -2.0, -5.0, -4.0], [-7.0, 5.0, 2.0, -3.0, -6.0]], [[-5.0, -5.0, -5.0, 2.0, -7.0], [1.0, 3.0, -4.0, 4.0, -3.0], [1.0, 5.0, 1.0, -3.0, 0.0], [-4.0, 4.0, -6.0, 5.0, -2.0]], [[1.0, 6.0, -6.0, 6.0, 0.0], [0.0, -2.0, -2.0, 3.0, 0.0], [6.0, -3.0, -1.0, 7.0, -7.0], [6.0, 2.0, -2.0, -2.0, 4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([0.0, 1.0, -5.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-10.0, -35.0, 25.0, -28.0, -7.0], [1.0, 13.0, 6.0, -11.0, -3.0], [-29.0, 20.0, 6.0, -38.0, 35.0], [-34.0, -6.0, 4.0, 15.0, -22.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[[2.0, 5.0, 0.0, 4.0, -2.0], [-7.0, -5.0, -1.0, 3.0, -4.0], [2.0, -1.0, 7.0, -5.0, 2.0], [2.0, -6.0, 0.0, -5.0, -2.0]], [[7.0, -6.0, -7.0, -7.0, 0.0], [-6.0, -1.0, -5.0, 4.0, -6.0], [2.0, -5.0, -4.0, -6.0, 3.0], [7.0, -7.0, 2.0, -6.0, 5.0]], [[0.0, 4.0, -3.0, 0.0, 6.0], [3.0, 5.0, -3.0, -4.0, 4.0], [7.0, 5.0, 0.0, -7.0, -7.0], [-4.0, 0.0, 0.0, 4.0, 2.0]]], [[[-7.0, 3.0, 7.0, 4.0, 1.0], [1.0, -5.0, 6.0, 2.0, -7.0], [-1.0, -6.0, 4.0, 1.0, -7.0], [-3.0, 5.0, 6.0, 4.0, 4.0]], [[-6.0, -3.0, 1.0, 3.0, -2.0], [3.0, 5.0, 2.0, 7.0, 1.0], [2.0, -3.0, -6.0, 6.0, 0.0], [-6.0, -2.0, 2.0, -7.0, -1.0]], [[2.0, 7.0, -3.0, 3.0, -6.0], [2.0, -4.0, -2.0, 1.0, -7.0], [6.0, -3.0, 7.0, -7.0, 5.0], [-5.0, 2.0, 0.0, -4.0, 7.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, 3.0, 1.0], [7.0, -7.0, 6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[12.0, 35.0, 0.0, -24.0, 5.0], [32.0, -57.0, 5.0, -42.0, -84.0], [14.0, -42.0, 51.0, -67.0, -31.0], [-6.0, 82.0, 34.0, 74.0, 108.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_constData_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[5.0, -3.0], [-7.0, -7.0]], [[-6.0, 3.0], [0.0, 6.0]], [[-2.0, 4.0], [-1.0, -5.0]], [[2.0, 4.0], [0.0, -1.0]], [[0.0, -3.0], [3.0, -7.0]], [[1.0, 0.0], [4.0, 5.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(4.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[20.0, -12.0], [-28.0, -28.0]], [[-24.0, 12.0], [0.0, 24.0]], [[-8.0, 16.0], [-4.0, -20.0]], [[8.0, 16.0], [0.0, -4.0]], [[0.0, -12.0], [12.0, -28.0]], [[4.0, 0.0], [16.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[[-3.0, -2.0], [5.0, 6.0]], [[2.0, 3.0], [-3.0, -1.0]], [[-5.0, 0.0], [6.0, 5.0]], [[-6.0, 2.0], [-6.0, 5.0]], [[3.0, 3.0], [-4.0, 3.0]], [[-7.0, 5.0], [5.0, 4.0]]], [[[6.0, -5.0], [-2.0, 3.0]], [[6.0, 3.0], [-7.0, -3.0]], [[7.0, -1.0], [-5.0, -5.0]], [[0.0, -6.0], [2.0, -7.0]], [[0.0, 3.0], [4.0, -1.0]], [[7.0, 4.0], [-5.0, 3.0]]], [[[2.0, -7.0], [-3.0, -2.0]], [[-3.0, -7.0], [-5.0, 5.0]], [[-7.0, 2.0], [-5.0, -7.0]], [[-7.0, 0.0], [-6.0, 0.0]], [[7.0, 7.0], [0.0, 7.0]], [[-1.0, 2.0], [7.0, -5.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([6.0, -7.0, -1.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-62.0, 30.0], [47.0, 17.0]], [[-27.0, 4.0], [36.0, 10.0]], [[-72.0, 5.0], [76.0, 72.0]], [[-29.0, 54.0], [-44.0, 79.0]], [[11.0, -10.0], [-52.0, 18.0]], [[-90.0, 0.0], [58.0, 8.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_constData_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[-6.0, -6.0, -3.0, -3.0], [3.0, -4.0, 3.0, -7.0], [5.0, -6.0, -3.0, -2.0]], [[4.0, 0.0, -1.0, 3.0], [3.0, -3.0, -4.0, 4.0], [3.0, -2.0, -1.0, 0.0]]], [[[-6.0, 2.0, -5.0, 0.0], [-4.0, 1.0, -5.0, -7.0], [5.0, 7.0, 1.0, -7.0]], [[-7.0, 0.0, -3.0, -2.0], [-3.0, -3.0, 6.0, -3.0], [4.0, 7.0, 1.0, 3.0]]], [[[5.0, 3.0, 3.0, -5.0], [0.0, -1.0, -3.0, 5.0], [5.0, 4.0, 5.0, -6.0]], [[-1.0, 5.0, -2.0, -2.0], [-2.0, 3.0, -5.0, 0.0], [2.0, -3.0, 5.0, 2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0]], [[0.0, 0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, -0.0]], [[-0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, -0.0], [0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[-0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(5.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(3.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-9.0)+(1.-msk_ref)*numpy.array(15.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 2.0, 7.0])+(1.-msk_arg1)*numpy.array([-3.0, -7.0, -2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([6.0, 4.0, 6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(62.0)+(1.-msk_ref)*numpy.array(-58.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 5.0, -5.0], [0.0, 6.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 0.0, -2.0], [-4.0, 5.0, -7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, -4.0, 4.0], [5.0, -3.0, 2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-16.0)+(1.-msk_ref)*numpy.array(-43.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-2.0, -3.0, 2.0], [5.0, 3.0, 0.0], [-2.0, -1.0, -1.0], [5.0, -3.0, -5.0]], [[7.0, 0.0, 7.0], [-2.0, 0.0, -1.0], [-4.0, 2.0, -4.0], [0.0, -2.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 1.0, 2.0], [-2.0, 3.0, 3.0], [4.0, -4.0, -3.0], [-5.0, 0.0, -5.0]], [[3.0, 1.0, -2.0], [6.0, 6.0, 7.0], [-5.0, -4.0, 2.0], [-5.0, 1.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-2.0, 6.0, 0.0], [-7.0, 2.0, 3.0], [7.0, 2.0, 1.0], [-2.0, 5.0, -4.0]], [[0.0, 3.0, 0.0], [-3.0, 6.0, 7.0], [0.0, -7.0, 5.0], [2.0, 0.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-103.0)+(1.-msk_ref)*numpy.array(187.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-1.0, 0.0], [1.0, 0.0], [5.0, -1.0]], [[4.0, 1.0], [-3.0, -4.0], [7.0, -3.0]]], [[[-7.0, -7.0], [2.0, 2.0], [-2.0, 5.0]], [[-3.0, 2.0], [2.0, 4.0], [3.0, 0.0]]], [[[4.0, 5.0], [0.0, 0.0], [-4.0, 0.0]], [[-4.0, 5.0], [5.0, 6.0], [-6.0, 0.0]]], [[[5.0, 2.0], [-1.0, 7.0], [-3.0, -1.0]], [[-3.0, -6.0], [-6.0, 0.0], [-7.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 6.0], [1.0, 6.0], [-6.0, 7.0]], [[-7.0, 0.0], [-5.0, 7.0], [3.0, 6.0]]], [[[7.0, -4.0], [0.0, -4.0], [-7.0, -7.0]], [[4.0, 7.0], [7.0, -5.0], [2.0, 0.0]]], [[[-4.0, 5.0], [-1.0, -5.0], [-4.0, -2.0]], [[3.0, -7.0], [-2.0, 5.0], [-6.0, -6.0]]], [[[3.0, -7.0], [6.0, 4.0], [2.0, -6.0]], [[1.0, 3.0], [3.0, 0.0], [-2.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[-7.0, 6.0], [4.0, -7.0], [4.0, 6.0]], [[0.0, 6.0], [-3.0, -4.0], [0.0, 1.0]]], [[[0.0, 2.0], [-7.0, -5.0], [4.0, -6.0]], [[-1.0, 7.0], [6.0, -4.0], [2.0, -5.0]]], [[[0.0, -6.0], [5.0, -6.0], [6.0, 0.0]], [[7.0, -5.0], [-7.0, 2.0], [-4.0, 6.0]]], [[[-6.0, -6.0], [3.0, 2.0], [-7.0, 7.0]], [[6.0, -6.0], [0.0, -3.0], [7.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-158.0)+(1.-msk_ref)*numpy.array(158.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, -7.0])+(1.-msk_arg1)*numpy.array([-1.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(2.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([14.0, -14.0])+(1.-msk_ref)*numpy.array([-2.0, -2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 0.0], [-5.0, -7.0], [6.0, -5.0]])+(1.-msk_arg1)*numpy.array([[0.0, -4.0], [0.0, -3.0], [-4.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, 1.0, -7.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-41.0, 28.0])+(1.-msk_ref)*numpy.array([28.0, 9.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -2.0], [6.0, -4.0], [0.0, -1.0]], [[6.0, -6.0], [-7.0, -5.0], [2.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, -2.0], [-2.0, 2.0], [4.0, 7.0]], [[3.0, -5.0], [-4.0, 3.0], [-5.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, 5.0, -4.0], [3.0, 7.0, -1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-3.0, -55.0])+(1.-msk_ref)*numpy.array([-40.0, 5.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -5.0], [5.0, -2.0], [-5.0, -5.0]], [[3.0, -2.0], [0.0, -6.0], [2.0, 1.0]], [[6.0, -6.0], [-4.0, -4.0], [-3.0, 2.0]], [[0.0, 1.0], [5.0, -2.0], [-6.0, 4.0]]], [[[-7.0, 6.0], [-3.0, -7.0], [3.0, 0.0]], [[4.0, -6.0], [5.0, 5.0], [2.0, 6.0]], [[-7.0, -7.0], [5.0, -1.0], [-6.0, -2.0]], [[-2.0, 1.0], [-7.0, 4.0], [0.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -4.0], [-5.0, 4.0], [-6.0, -4.0]], [[1.0, -1.0], [-5.0, 0.0], [0.0, -1.0]], [[0.0, 0.0], [-3.0, 5.0], [0.0, 0.0]], [[4.0, 0.0], [5.0, 0.0], [-1.0, -1.0]]], [[[4.0, -3.0], [7.0, -2.0], [-3.0, 5.0]], [[0.0, 2.0], [0.0, 1.0], [4.0, -4.0]], [[0.0, 1.0], [-7.0, -4.0], [-1.0, 4.0]], [[3.0, 0.0], [-1.0, 0.0], [-7.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[1.0, -4.0, -1.0], [0.0, -6.0, 6.0], [3.0, 4.0, 7.0], [-4.0, -7.0, 0.0]], [[-5.0, 5.0, 7.0], [6.0, -1.0, 6.0], [0.0, -5.0, -1.0], [-3.0, -5.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([37.0, -42.0])+(1.-msk_ref)*numpy.array([40.0, 41.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 6.0, -7.0, 0.0, 3.0], [-2.0, -4.0, -1.0, 4.0, 6.0], [-2.0, 3.0, -7.0, -3.0, 0.0], [6.0, 6.0, 4.0, 0.0, -5.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -2.0, 6.0, 5.0, 5.0], [6.0, 3.0, -5.0, 6.0, 7.0], [5.0, -3.0, -2.0, 7.0, 6.0], [-7.0, 0.0, -5.0, 2.0, 1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-0.0, 0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0, -0.0]])+(1.-msk_ref)*numpy.array([[-0.0, -0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 5.0, -1.0, 1.0, 2.0], [-4.0, -2.0, 2.0, -2.0, 3.0], [-7.0, 3.0, 1.0, 4.0, -6.0], [1.0, 5.0, 5.0, 2.0, -6.0]], [[-3.0, -6.0, 2.0, 2.0, -7.0], [0.0, 5.0, -3.0, 5.0, 0.0], [7.0, 6.0, 5.0, -2.0, 5.0], [-7.0, -7.0, 4.0, 4.0, -3.0]], [[-6.0, -5.0, 4.0, 6.0, -7.0], [0.0, 2.0, -4.0, 4.0, -1.0], [-1.0, 2.0, -2.0, -4.0, 5.0], [4.0, -1.0, -6.0, 0.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[4.0, -4.0, -6.0, 0.0, 3.0], [-3.0, -2.0, 2.0, 5.0, -2.0], [-6.0, 6.0, 0.0, 0.0, 4.0], [0.0, 6.0, 4.0, 7.0, -3.0]], [[6.0, -5.0, 5.0, 0.0, -3.0], [0.0, -7.0, -3.0, -2.0, 1.0], [5.0, 1.0, -1.0, 7.0, 3.0], [-4.0, -5.0, 5.0, -5.0, 2.0]], [[1.0, 6.0, -3.0, -7.0, -5.0], [6.0, -6.0, -7.0, 0.0, 7.0], [-1.0, 0.0, 0.0, -4.0, 3.0], [-1.0, 0.0, 5.0, -7.0, 5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 2.0, -5.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-11.0, -12.0, -11.0, -31.0, 11.0], [20.0, 10.0, 4.0, 0.0, -10.0], [54.0, -13.0, 15.0, -4.0, 15.0], [-39.0, -34.0, 13.0, -2.0, 44.0]])+(1.-msk_ref)*numpy.array([[-13.0, -20.0, 55.0, 35.0, 4.0], [-15.0, 26.0, 19.0, -29.0, -23.0], [45.0, -28.0, -2.0, 34.0, -29.0], [-3.0, -40.0, -35.0, -10.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-1.0, -1.0, 5.0, -7.0, -1.0], [-3.0, 4.0, 4.0, -5.0, 3.0], [7.0, 2.0, -1.0, -6.0, 5.0], [-7.0, 0.0, 7.0, 4.0, -3.0]], [[5.0, -3.0, -4.0, 1.0, 0.0], [6.0, 3.0, -4.0, 4.0, -3.0], [5.0, -5.0, -6.0, 7.0, -2.0], [1.0, 5.0, -5.0, 5.0, 0.0]], [[3.0, 2.0, 7.0, -5.0, -2.0], [2.0, 3.0, -1.0, 1.0, 5.0], [-4.0, -4.0, -7.0, 1.0, -4.0], [-4.0, -2.0, -6.0, -6.0, 0.0]]], [[[0.0, 6.0, 7.0, -3.0, 4.0], [-1.0, -6.0, -2.0, 0.0, -6.0], [-7.0, -3.0, 0.0, 2.0, 5.0], [0.0, -5.0, -5.0, 6.0, 7.0]], [[-2.0, -1.0, -7.0, -1.0, 1.0], [4.0, 5.0, 3.0, -6.0, 0.0], [-6.0, -4.0, -2.0, 4.0, 6.0], [-2.0, -7.0, -1.0, -6.0, 4.0]], [[4.0, 0.0, 4.0, 4.0, 5.0], [-6.0, -4.0, 2.0, -1.0, 1.0], [-4.0, 4.0, -1.0, 2.0, -3.0], [4.0, 3.0, -7.0, 4.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, -3.0, 6.0, -6.0, 0.0], [0.0, 3.0, -6.0, -1.0, 0.0], [0.0, 3.0, -6.0, -2.0, 5.0], [-3.0, 5.0, 0.0, -7.0, -6.0]], [[5.0, 5.0, -7.0, 3.0, -3.0], [-6.0, -4.0, 3.0, -7.0, 5.0], [-2.0, 5.0, -6.0, 1.0, 7.0], [-1.0, 1.0, 0.0, 4.0, 4.0]], [[0.0, 5.0, -7.0, 1.0, -6.0], [-7.0, 3.0, 5.0, -3.0, -1.0], [-1.0, -2.0, -2.0, -5.0, 6.0], [-7.0, 6.0, 3.0, -3.0, -1.0]]], [[[-5.0, -7.0, 0.0, 4.0, -5.0], [-3.0, -5.0, 4.0, -2.0, -5.0], [-4.0, -2.0, -6.0, 4.0, -7.0], [-4.0, -5.0, 0.0, -5.0, 2.0]], [[4.0, 2.0, 0.0, 0.0, 3.0], [1.0, -6.0, -1.0, 2.0, -5.0], [-4.0, 2.0, -6.0, -4.0, 1.0], [-3.0, 2.0, 2.0, 4.0, 0.0]], [[0.0, -2.0, -1.0, 2.0, -5.0], [5.0, 4.0, 2.0, -5.0, -6.0], [1.0, 4.0, 1.0, -2.0, 6.0], [-5.0, 0.0, 0.0, 3.0, -3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[6.0, -7.0, 4.0], [6.0, -7.0, -1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-19.0, 66.0, 173.0, -84.0, -2.0], [-80.0, -52.0, 13.0, -11.0, 22.0], [-5.0, 37.0, 23.0, -99.0, 19.0], [-55.0, -27.0, 37.0, 39.0, -4.0]])+(1.-msk_ref)*numpy.array([[-51.0, -87.0, 58.0, -31.0, -49.0], [-16.0, 66.0, -8.0, 10.0, -28.0], [13.0, -55.0, 3.0, 15.0, -50.0], [-37.0, 3.0, -2.0, -143.0, -53.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, -3.0], [2.0, -1.0]], [[-6.0, 0.0], [6.0, -6.0]], [[-1.0, -5.0], [-2.0, 1.0]], [[-2.0, -2.0], [4.0, -6.0]], [[1.0, 0.0], [0.0, -2.0]], [[7.0, 0.0], [1.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, -5.0], [1.0, -4.0]], [[0.0, -3.0], [5.0, -6.0]], [[-6.0, 5.0], [0.0, -6.0]], [[-1.0, 4.0], [-5.0, 2.0]], [[-7.0, -7.0], [-7.0, 6.0]], [[-6.0, 2.0], [-5.0, 6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-4.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 12.0], [-8.0, 4.0]], [[24.0, -0.0], [-24.0, 24.0]], [[4.0, 20.0], [8.0, -4.0]], [[8.0, 8.0], [-16.0, 24.0]], [[-4.0, -0.0], [-0.0, 8.0]], [[-28.0, -0.0], [-4.0, -28.0]]])+(1.-msk_ref)*numpy.array([[[12.0, 20.0], [-4.0, 16.0]], [[-0.0, 12.0], [-20.0, 24.0]], [[24.0, -20.0], [-0.0, 24.0]], [[4.0, -16.0], [20.0, -8.0]], [[28.0, 28.0], [28.0, -24.0]], [[24.0, -8.0], [20.0, -24.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, 7.0], [4.0, -6.0]], [[0.0, -4.0], [-2.0, -7.0]], [[2.0, 4.0], [5.0, 5.0]], [[-4.0, -7.0], [-6.0, 5.0]], [[1.0, 1.0], [3.0, 7.0]], [[-1.0, 4.0], [-7.0, 0.0]]], [[[7.0, 0.0], [-3.0, 7.0]], [[-4.0, 1.0], [-5.0, 0.0]], [[-6.0, -1.0], [-6.0, 3.0]], [[2.0, -1.0], [0.0, 1.0]], [[-4.0, -1.0], [6.0, 0.0]], [[7.0, -7.0], [-5.0, 3.0]]], [[[1.0, 0.0], [-1.0, 0.0]], [[4.0, -4.0], [2.0, -6.0]], [[4.0, -6.0], [-4.0, -2.0]], [[6.0, -6.0], [-4.0, 7.0]], [[-3.0, 5.0], [4.0, -5.0]], [[-3.0, 0.0], [2.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, -7.0], [0.0, 1.0]], [[3.0, -3.0], [-4.0, -4.0]], [[-6.0, 1.0], [-6.0, 7.0]], [[0.0, -4.0], [5.0, -6.0]], [[0.0, 1.0], [2.0, -7.0]], [[-4.0, -2.0], [0.0, 4.0]]], [[[1.0, 2.0], [-1.0, 0.0]], [[-6.0, 7.0], [6.0, 5.0]], [[5.0, 5.0], [-1.0, -7.0]], [[-2.0, -3.0], [6.0, 1.0]], [[6.0, 1.0], [-5.0, 2.0]], [[0.0, -6.0], [2.0, -2.0]]], [[[1.0, 0.0], [-6.0, 5.0]], [[-5.0, 7.0], [4.0, 2.0]], [[-5.0, 1.0], [-4.0, -3.0]], [[6.0, 0.0], [5.0, 4.0]], [[-4.0, -7.0], [3.0, 6.0]], [[-2.0, 0.0], [0.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([0.0, -3.0, 5.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-16.0, 0.0], [4.0, -21.0]], [[32.0, -23.0], [25.0, -30.0]], [[38.0, -27.0], [-2.0, -19.0]], [[24.0, -27.0], [-20.0, 32.0]], [[-3.0, 28.0], [2.0, -25.0]], [[-36.0, 21.0], [25.0, 16.0]]])+(1.-msk_ref)*numpy.array([[[2.0, -6.0], [-27.0, 25.0]], [[-7.0, 14.0], [2.0, -5.0]], [[-40.0, -10.0], [-17.0, 6.0]], [[36.0, 9.0], [7.0, 17.0]], [[-38.0, -38.0], [30.0, 24.0]], [[-10.0, 18.0], [-6.0, 16.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank0_expandedData_rank4_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, -3.0, 3.0, 0.0], [1.0, -2.0, 4.0, 2.0], [0.0, -7.0, 1.0, -4.0]], [[-2.0, -3.0, 2.0, 1.0], [1.0, -4.0, -4.0, 7.0], [0.0, -5.0, -7.0, 5.0]]], [[[-5.0, 7.0, -6.0, 7.0], [0.0, -2.0, -6.0, 0.0], [0.0, 0.0, 6.0, -6.0]], [[1.0, 0.0, 0.0, 2.0], [-1.0, 3.0, 2.0, 7.0], [0.0, 0.0, -4.0, -4.0]]], [[[-2.0, -4.0, 0.0, 2.0], [0.0, 5.0, 0.0, -4.0], [-2.0, 6.0, 4.0, 7.0]], [[-7.0, 1.0, 1.0, 7.0], [-7.0, -5.0, -4.0, 7.0], [-1.0, -5.0, -6.0, 1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, 5.0, 4.0, 0.0], [0.0, 6.0, 4.0, -4.0], [3.0, -7.0, -1.0, 4.0]], [[-4.0, -7.0, 2.0, -6.0], [4.0, 4.0, 3.0, 7.0], [-7.0, -3.0, 0.0, 0.0]]], [[[2.0, -4.0, 2.0, 1.0], [2.0, 5.0, 0.0, -1.0], [7.0, -3.0, -7.0, -4.0]], [[4.0, 7.0, 2.0, 4.0], [-3.0, 3.0, 7.0, 2.0], [-1.0, -6.0, -3.0, 1.0]]], [[[2.0, 0.0, 6.0, 4.0], [7.0, -2.0, -1.0, -6.0], [0.0, 5.0, 3.0, 6.0]], [[0.0, 7.0, -2.0, 0.0], [-6.0, -2.0, 1.0, 5.0], [7.0, -3.0, 0.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-4.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[16.0, 12.0, -12.0, -0.0], [-4.0, 8.0, -16.0, -8.0], [-0.0, 28.0, -4.0, 16.0]], [[8.0, 12.0, -8.0, -4.0], [-4.0, 16.0, 16.0, -28.0], [-0.0, 20.0, 28.0, -20.0]]], [[[20.0, -28.0, 24.0, -28.0], [-0.0, 8.0, 24.0, -0.0], [-0.0, -0.0, -24.0, 24.0]], [[-4.0, -0.0, -0.0, -8.0], [4.0, -12.0, -8.0, -28.0], [-0.0, -0.0, 16.0, 16.0]]], [[[8.0, 16.0, -0.0, -8.0], [-0.0, -20.0, -0.0, 16.0], [8.0, -24.0, -16.0, -28.0]], [[28.0, -4.0, -4.0, -28.0], [28.0, 20.0, 16.0, -28.0], [4.0, 20.0, 24.0, -4.0]]]])+(1.-msk_ref)*numpy.array([[[[28.0, -20.0, -16.0, -0.0], [-0.0, -24.0, -16.0, 16.0], [-12.0, 28.0, 4.0, -16.0]], [[16.0, 28.0, -8.0, 24.0], [-16.0, -16.0, -12.0, -28.0], [28.0, 12.0, -0.0, -0.0]]], [[[-8.0, 16.0, -8.0, -4.0], [-8.0, -20.0, -0.0, 4.0], [-28.0, 12.0, 28.0, 16.0]], [[-16.0, -28.0, -8.0, -16.0], [12.0, -12.0, -28.0, -8.0], [4.0, 24.0, 12.0, -4.0]]], [[[-8.0, -0.0, -24.0, -16.0], [-28.0, 8.0, 4.0, 24.0], [-0.0, -20.0, -12.0, -24.0]], [[-0.0, -28.0, 8.0, -0.0], [24.0, 8.0, -4.0, -20.0], [-28.0, 12.0, -0.0, -16.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_float_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=-1.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-1.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([1.0, -0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([1.0, -4.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-7.0, 28.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([-3.0, 5.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[0.0, 0.0, 6.0], [-2.0, -4.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-6.0, -21.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[5.0, 3.0, 0.0], [2.0, -2.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-3.0, -4.0, -4.0], [-4.0, -7.0, 1.0]], [[6.0, -6.0, 3.0], [6.0, -3.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-16.0, 20.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=numpy.array([[[7.0, 3.0, -5.0], [1.0, 1.0, 6.0], [-7.0, -6.0, -5.0], [5.0, 4.0, 0.0]], [[2.0, 0.0, 3.0], [-3.0, 6.0, -7.0], [4.0, 0.0, -2.0], [3.0, 6.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[6.0, 4.0, 7.0], [-6.0, -5.0, 5.0], [7.0, -3.0, 3.0], [-7.0, -2.0, -7.0]], [[4.0, 0.0, 7.0], [-2.0, 7.0, -6.0], [5.0, -7.0, 7.0], [-3.0, 1.0, -2.0]]], [[[2.0, -5.0, -6.0], [7.0, 1.0, 0.0], [-4.0, -5.0, -4.0], [-2.0, -5.0, 4.0]], [[6.0, 7.0, 3.0], [-5.0, 7.0, -3.0], [0.0, -2.0, 7.0], [1.0, -3.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([69.0, 157.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([2.0, 2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([2.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[4.0, 4.0], [0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[4.0, 6.0], [1.0, 5.0], [-1.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, -3.0, 1.0], [2.0, -2.0, -2.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-32.0, -53.0], [8.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[[-5.0, 2.0], [3.0, -1.0], [-4.0, 3.0]], [[-2.0, 2.0], [-6.0, 2.0], [7.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-7.0, 0.0, 0.0], [-6.0, 6.0, -1.0]], [[4.0, -5.0, -5.0], [7.0, 7.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[4.0, -11.0], [-85.0, 32.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=numpy.array([[[[-2.0, 5.0], [-7.0, -3.0], [-4.0, -5.0]], [[-1.0, 6.0], [3.0, 5.0], [-3.0, 0.0]], [[0.0, 6.0], [0.0, 5.0], [-5.0, -6.0]], [[4.0, -5.0], [-6.0, 2.0], [5.0, 5.0]]], [[[7.0, -4.0], [-6.0, 6.0], [-5.0, 1.0]], [[-6.0, 0.0], [-1.0, -1.0], [0.0, 7.0]], [[5.0, 2.0], [-5.0, -3.0], [-4.0, 1.0]], [[0.0, -2.0], [1.0, -2.0], [4.0, 6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-1.0, 5.0, -5.0], [7.0, 6.0, 1.0], [-2.0, -6.0, 4.0], [-6.0, 0.0, -5.0]], [[-3.0, -3.0, 5.0], [4.0, 6.0, -7.0], [0.0, 5.0, 2.0], [-4.0, -7.0, 0.0]]], [[[-3.0, -2.0, -1.0], [-1.0, 4.0, -7.0], [0.0, -4.0, 7.0], [0.0, -2.0, 1.0]], [[2.0, -1.0, -1.0], [6.0, -4.0, 4.0], [-7.0, -4.0, -4.0], [-3.0, 4.0, -3.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-172.0, -31.0], [26.0, -60.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[-5.0, -7.0, 5.0, -1.0, -6.0], [-1.0, -4.0, 2.0, 2.0, -3.0], [-1.0, 4.0, 7.0, 3.0, 3.0], [0.0, -5.0, -1.0, 6.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-2.0, 3.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[10.0, 14.0, -10.0, 2.0, 12.0], [2.0, 8.0, -4.0, -4.0, 6.0], [2.0, -8.0, -14.0, -6.0, -6.0], [0.0, 10.0, 2.0, -12.0, 12.0]], [[-15.0, -21.0, 15.0, -3.0, -18.0], [-3.0, -12.0, 6.0, 6.0, -9.0], [-3.0, 12.0, 21.0, 9.0, 9.0], [0.0, -15.0, -3.0, 18.0, -18.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[5.0, 1.0, 1.0, -4.0, -7.0], [1.0, 1.0, 4.0, 0.0, -5.0], [0.0, 6.0, -3.0, -1.0, -2.0], [-2.0, -5.0, 0.0, 0.0, 5.0]], [[-5.0, 0.0, 0.0, 6.0, 6.0], [-5.0, -1.0, -1.0, -6.0, 2.0], [-4.0, 3.0, 3.0, 0.0, -5.0], [-4.0, 0.0, 6.0, 5.0, -2.0]], [[-6.0, 4.0, -6.0, -6.0, -5.0], [4.0, 4.0, -3.0, 5.0, -4.0], [1.0, 1.0, -2.0, 1.0, 0.0], [-2.0, 1.0, 7.0, 0.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-6.0, -5.0, 5.0], [-3.0, 1.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-35.0, 14.0, -36.0, -36.0, -13.0], [39.0, 19.0, -34.0, 55.0, 0.0], [25.0, -46.0, -7.0, 11.0, 37.0], [22.0, 35.0, 5.0, -25.0, -15.0]], [[-20.0, -3.0, -3.0, 18.0, 27.0], [-8.0, -4.0, -13.0, -6.0, 17.0], [-4.0, -15.0, 12.0, 3.0, 1.0], [2.0, 15.0, 6.0, 5.0, -17.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[[[-3.0, -5.0, -4.0, 4.0, 1.0], [2.0, 6.0, -7.0, -5.0, -5.0], [0.0, -1.0, 7.0, -4.0, 0.0], [-7.0, -6.0, -7.0, 0.0, -7.0]], [[-7.0, -3.0, -6.0, -5.0, -7.0], [0.0, -5.0, -4.0, 3.0, -2.0], [-2.0, -7.0, -2.0, 6.0, -3.0], [-5.0, -1.0, -6.0, 5.0, -6.0]], [[4.0, -1.0, -4.0, 5.0, 2.0], [0.0, -5.0, 3.0, -2.0, 4.0], [6.0, 4.0, 3.0, 0.0, -3.0], [3.0, 5.0, -5.0, 4.0, 0.0]]], [[[-2.0, 6.0, 6.0, 2.0, 2.0], [6.0, -2.0, -7.0, 4.0, -7.0], [3.0, 6.0, -4.0, 7.0, -6.0], [4.0, -5.0, 4.0, 0.0, 0.0]], [[3.0, -5.0, -2.0, -1.0, -6.0], [7.0, 7.0, 7.0, -7.0, 1.0], [2.0, -5.0, -6.0, 0.0, -5.0], [7.0, 7.0, 0.0, -5.0, -1.0]], [[3.0, 1.0, -5.0, -1.0, -2.0], [3.0, 0.0, -4.0, -3.0, -5.0], [-2.0, 0.0, 7.0, 5.0, 5.0], [-5.0, -5.0, 2.0, -6.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[3.0, -1.0, -5.0], [1.0, 2.0, -5.0]], [[-1.0, -2.0, -3.0], [-7.0, -1.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-33.0, -16.0, 41.0, -3.0, 0.0], [11.0, 60.0, -5.0, -3.0, -13.0], [-11.0, -20.0, -43.0, -36.0, -23.0], [12.0, -8.0, 4.0, -5.0, 8.0]], [[22.0, -21.0, -22.0, -24.0, -5.0], [-45.0, 26.0, 40.0, -22.0, 35.0], [-41.0, -34.0, 36.0, -47.0, 72.0], [-37.0, 11.0, 10.0, -29.0, 10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_array_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[-7.0, -5.0], [1.0, 1.0]], [[1.0, 6.0], [-7.0, 4.0]], [[-1.0, 7.0], [-7.0, 5.0]], [[-6.0, -5.0], [-7.0, -1.0]], [[-2.0, 7.0], [4.0, 3.0]], [[6.0, 7.0], [4.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, 5.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[-35.0, -25.0], [5.0, 5.0]], [[5.0, 30.0], [-35.0, 20.0]], [[-5.0, 35.0], [-35.0, 25.0]], [[-30.0, -25.0], [-35.0, -5.0]], [[-10.0, 35.0], [20.0, 15.0]], [[30.0, 35.0], [20.0, 10.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[[-2.0, -5.0], [2.0, -4.0]], [[4.0, 0.0], [-1.0, 2.0]], [[-7.0, 5.0], [-3.0, -5.0]], [[-5.0, -5.0], [4.0, 0.0]], [[7.0, 3.0], [7.0, -5.0]], [[0.0, 0.0], [-1.0, -6.0]]], [[[3.0, 0.0], [-4.0, 2.0]], [[5.0, -7.0], [4.0, -7.0]], [[0.0, -7.0], [-2.0, -5.0]], [[-4.0, 7.0], [-5.0, -3.0]], [[0.0, -5.0], [-7.0, 0.0]], [[0.0, -3.0], [2.0, -7.0]]], [[[0.0, 7.0], [-6.0, 0.0]], [[-1.0, -6.0], [-4.0, 0.0]], [[1.0, -6.0], [7.0, -6.0]], [[2.0, -4.0], [7.0, 0.0]], [[-2.0, 0.0], [4.0, -4.0]], [[-4.0, 5.0], [-3.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-5.0, 6.0, 2.0], [0.0, 4.0, 2.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[28.0, 39.0], [-46.0, 32.0]], [[8.0, -54.0], [21.0, -52.0]], [[37.0, -79.0], [17.0, -17.0]], [[5.0, 59.0], [-36.0, -18.0]], [[-39.0, -45.0], [-69.0, 17.0]], [[-8.0, -8.0], [11.0, -14.0]]], [[[12.0, 14.0], [-28.0, 8.0]], [[18.0, -40.0], [8.0, -28.0]], [[2.0, -40.0], [6.0, -32.0]], [[-12.0, 20.0], [-6.0, -12.0]], [[-4.0, -20.0], [-20.0, -8.0]], [[-8.0, -2.0], [2.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-4.0, 4.0])
      s1=numpy.array(1.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-4.0, 4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-5.0, 3.0, -4.0], [7.0, 2.0, 6.0]])
      s1=numpy.array([-1.0, 4.0, -6.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([41.0, -35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-2.0, 7.0, 4.0], [0.0, 1.0, 2.0]], [[5.0, 7.0, 0.0], [0.0, -4.0, 0.0]]])
      s1=numpy.array([[-1.0, -3.0, 3.0], [2.0, -5.0, -3.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-18.0, -6.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-2.0, -4.0, 6.0], [2.0, -4.0, -6.0], [2.0, 3.0, -3.0], [3.0, 3.0, 6.0]], [[-3.0, -1.0, 6.0], [3.0, -1.0, 6.0], [7.0, -2.0, -1.0], [5.0, -2.0, -5.0]]], [[[2.0, -1.0, 0.0], [-6.0, -5.0, -1.0], [5.0, 6.0, 4.0], [0.0, -6.0, 6.0]], [[-2.0, 2.0, -4.0], [1.0, 3.0, 4.0], [-3.0, -5.0, 1.0], [2.0, 2.0, 1.0]]]])
      s1=numpy.array([[[0.0, 5.0, 6.0], [-4.0, 1.0, 0.0], [-7.0, 5.0, -1.0], [-6.0, 6.0, 4.0]], [[7.0, 5.0, -7.0], [5.0, -6.0, -5.0], [0.0, -6.0, -3.0], [3.0, 2.0, 7.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-54.0, 28.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, 7.0])
      s1=numpy.array([-7.0, -1.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.0, 0.0], [-49.0, -7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[0.0, -1.0, 4.0], [0.0, -2.0, 0.0]])
      s1=numpy.array([[0.0, 1.0], [-3.0, 0.0], [5.0, -2.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[23.0, -8.0], [6.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-7.0, -7.0, 5.0], [4.0, 1.0, 1.0]], [[3.0, 4.0, 0.0], [-6.0, 5.0, -5.0]]])
      s1=numpy.array([[[2.0, -1.0], [2.0, 2.0], [0.0, 2.0]], [[4.0, 6.0], [1.0, -1.0], [0.0, 0.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-11.0, 26.0], [-5.0, -36.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[0.0, -2.0, 0.0], [-2.0, -4.0, -1.0], [1.0, 7.0, -6.0], [-4.0, 7.0, -4.0]], [[-2.0, -3.0, 4.0], [-7.0, -6.0, 5.0], [-6.0, 1.0, -7.0], [7.0, 5.0, 6.0]]], [[[3.0, 7.0, 3.0], [2.0, -4.0, -5.0], [-6.0, 3.0, 5.0], [7.0, 5.0, -3.0]], [[0.0, 0.0, 2.0], [-1.0, 3.0, 0.0], [2.0, 3.0, 0.0], [0.0, -4.0, 4.0]]]])
      s1=numpy.array([[[[-5.0, 3.0], [0.0, 5.0], [-2.0, 5.0]], [[-3.0, 2.0], [-1.0, -6.0], [6.0, 0.0]], [[1.0, -5.0], [-4.0, 4.0], [-2.0, -1.0]], [[-4.0, -5.0], [0.0, 6.0], [-3.0, 2.0]]], [[[1.0, 3.0], [-4.0, 0.0], [-4.0, 5.0]], [[-4.0, 0.0], [7.0, -7.0], [2.0, -2.0]], [[2.0, -6.0], [2.0, 4.0], [-7.0, -1.0]], [[5.0, 3.0], [-5.0, 2.0], [3.0, -4.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[74.0, 193.0], [-41.0, 78.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-1.0, -2.0])
      s1=numpy.array([[0.0, 3.0, -7.0, 3.0, -5.0], [-7.0, -7.0, -6.0, 3.0, -3.0], [4.0, -3.0, 0.0, 6.0, 4.0], [-6.0, 5.0, 4.0, -2.0, -3.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.0, -3.0, 7.0, -3.0, 5.0], [7.0, 7.0, 6.0, -3.0, 3.0], [-4.0, 3.0, 0.0, -6.0, -4.0], [6.0, -5.0, -4.0, 2.0, 3.0]], [[0.0, -6.0, 14.0, -6.0, 10.0], [14.0, 14.0, 12.0, -6.0, 6.0], [-8.0, 6.0, 0.0, -12.0, -8.0], [12.0, -10.0, -8.0, 4.0, 6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-6.0, -5.0, 0.0], [0.0, -4.0, -3.0]])
      s1=numpy.array([[[7.0, 0.0, 3.0, 4.0, -3.0], [-2.0, -3.0, 5.0, 0.0, 0.0], [3.0, -3.0, -7.0, 4.0, 1.0], [-5.0, 0.0, -3.0, 1.0, 3.0]], [[3.0, 3.0, -1.0, 3.0, 3.0], [0.0, 0.0, 2.0, 2.0, 5.0], [-5.0, 2.0, 0.0, 2.0, -1.0], [-7.0, 0.0, 3.0, -4.0, -3.0]], [[7.0, -1.0, 2.0, 5.0, -3.0], [5.0, 7.0, -4.0, -2.0, -2.0], [5.0, -6.0, 0.0, -4.0, -6.0], [6.0, -2.0, -7.0, -2.0, 5.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-57.0, -15.0, -13.0, -39.0, 3.0], [12.0, 18.0, -40.0, -10.0, -25.0], [7.0, 8.0, 42.0, -34.0, -1.0], [65.0, 0.0, 3.0, 14.0, -3.0]], [[-33.0, -9.0, -2.0, -27.0, -3.0], [-15.0, -21.0, 4.0, -2.0, -14.0], [5.0, 10.0, 0.0, 4.0, 22.0], [10.0, 6.0, 9.0, 22.0, -3.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[3.0, 0.0, -6.0], [0.0, -1.0, -3.0]], [[-1.0, 3.0, 4.0], [0.0, 3.0, 0.0]]])
      s1=numpy.array([[[[-6.0, 3.0, 0.0, 0.0, -2.0], [-5.0, -1.0, 5.0, -1.0, 2.0], [-3.0, -3.0, -5.0, -1.0, 0.0], [-4.0, 0.0, 4.0, -7.0, -4.0]], [[-1.0, 2.0, 0.0, -5.0, -2.0], [-4.0, 6.0, -3.0, 0.0, -1.0], [1.0, -3.0, -1.0, 0.0, 2.0], [7.0, -7.0, 0.0, 7.0, -6.0]], [[5.0, 5.0, -6.0, 1.0, 2.0], [-5.0, -6.0, -4.0, 6.0, -1.0], [0.0, 0.0, 3.0, -5.0, -2.0], [-3.0, 2.0, 5.0, -2.0, 7.0]]], [[[-2.0, -1.0, 5.0, 4.0, 7.0], [3.0, 4.0, -5.0, -1.0, 3.0], [3.0, -5.0, -4.0, 0.0, -6.0], [1.0, 1.0, -1.0, 7.0, 3.0]], [[-5.0, -1.0, 5.0, 4.0, -1.0], [-1.0, 1.0, 1.0, 5.0, -1.0], [3.0, 4.0, 1.0, 2.0, 0.0], [0.0, -6.0, -1.0, -5.0, 0.0]], [[-6.0, 1.0, -2.0, 6.0, -5.0], [1.0, 0.0, -4.0, 4.0, 7.0], [-5.0, -5.0, -6.0, 0.0, 3.0], [-5.0, -3.0, 6.0, 5.0, 0.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-25.0, -23.0, 37.0, -28.0, -2.0], [13.0, 32.0, 50.0, -56.0, -8.0], [3.0, 2.0, -16.0, 25.0, 3.0], [21.0, 3.0, -35.0, -19.0, -54.0]], [[8.0, 20.0, -9.0, 1.0, 1.0], [-30.0, -2.0, -27.0, 40.0, -12.0], [15.0, 6.0, 17.0, -13.0, -2.0], [13.0, -31.0, 13.0, 5.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-3.0, 0.0])
      s1=numpy.array([[[3.0, 6.0], [6.0, 5.0]], [[-4.0, 2.0], [2.0, -7.0]], [[-6.0, -4.0], [2.0, 3.0]], [[-3.0, -5.0], [-2.0, 1.0]], [[-6.0, 1.0], [2.0, -2.0]], [[-2.0, 6.0], [3.0, 0.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-9.0, -18.0], [-18.0, -15.0]], [[12.0, -6.0], [-6.0, 21.0]], [[18.0, 12.0], [-6.0, -9.0]], [[9.0, 15.0], [6.0, -3.0]], [[18.0, -3.0], [-6.0, 6.0]], [[6.0, -18.0], [-9.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-3.0, 1.0, 1.0], [6.0, -6.0, -4.0]])
      s1=numpy.array([[[[-6.0, -7.0], [5.0, 1.0]], [[-2.0, 2.0], [0.0, 1.0]], [[0.0, 3.0], [-4.0, -6.0]], [[0.0, -2.0], [4.0, 3.0]], [[-1.0, -2.0], [-4.0, 3.0]], [[-7.0, 3.0], [-5.0, 5.0]]], [[[0.0, -6.0], [0.0, 4.0]], [[2.0, 7.0], [0.0, 4.0]], [[0.0, -6.0], [-6.0, 2.0]], [[7.0, -4.0], [-6.0, 5.0]], [[-1.0, 1.0], [2.0, 7.0]], [[-4.0, -1.0], [2.0, -3.0]]], [[[-5.0, -1.0], [-2.0, -6.0]], [[0.0, 4.0], [-7.0, -5.0]], [[-4.0, 3.0], [-5.0, -1.0]], [[1.0, 3.0], [-5.0, 0.0]], [[-3.0, 2.0], [0.0, 3.0]], [[4.0, -7.0], [-1.0, 6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[13.0, 14.0], [-17.0, -5.0]], [[8.0, 5.0], [-7.0, -4.0]], [[-4.0, -12.0], [1.0, 19.0]], [[8.0, 5.0], [-23.0, -4.0]], [[-1.0, 9.0], [14.0, 1.0]], [[21.0, -17.0], [16.0, -12.0]]], [[[-16.0, -2.0], [38.0, 6.0]], [[-24.0, -46.0], [28.0, 2.0]], [[16.0, 42.0], [32.0, -44.0]], [[-46.0, 0.0], [80.0, -12.0]], [[12.0, -26.0], [-36.0, -36.0]], [[-34.0, 52.0], [-38.0, 24.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(-2.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-5.0, 2.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([10.0, -4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([2.0, -5.0, 5.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, -3.0, 0.0], [3.0, 3.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([1.0, -9.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[-5.0, -3.0, -7.0], [-2.0, -5.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[1.0, -2.0, -6.0], [-4.0, -5.0, -6.0]], [[-3.0, 6.0, 4.0], [-4.0, 3.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([46.0, -33.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Data(numpy.array([[[-7.0, -6.0, 0.0], [-4.0, 2.0, 1.0], [0.0, 4.0, -7.0], [4.0, 4.0, 4.0]], [[7.0, 4.0, 1.0], [6.0, 0.0, -5.0], [6.0, 1.0, -4.0], [-2.0, -7.0, 2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[3.0, -1.0, 2.0], [0.0, 0.0, 5.0], [1.0, -2.0, 7.0], [-3.0, 1.0, -1.0]], [[-3.0, 2.0, -1.0], [-3.0, 7.0, 3.0], [7.0, 6.0, -1.0], [-2.0, 0.0, -3.0]]], [[[2.0, 1.0, 2.0], [-1.0, -5.0, 7.0], [4.0, -5.0, 7.0], [6.0, 1.0, 5.0]], [[-3.0, 5.0, -4.0], [-6.0, 1.0, 3.0], [-3.0, -3.0, -5.0], [2.0, 0.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-76.0, -97.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([2.0, 3.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.0, 0.0], [0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[3.0, -7.0], [-3.0, 0.0], [0.0, -6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-6.0, -1.0, -5.0], [2.0, 6.0, -3.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-15.0, 72.0], [-12.0, 4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[[7.0, 0.0], [-7.0, -2.0], [2.0, 0.0]], [[0.0, 0.0], [0.0, -3.0], [-3.0, -1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-4.0, 0.0, -7.0], [-2.0, -3.0, 5.0]], [[-6.0, 6.0, 5.0], [-3.0, -1.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-57.0, 4.0], [-80.0, -11.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Data(numpy.array([[[[-1.0, 3.0], [-4.0, 1.0], [-1.0, -3.0]], [[1.0, -4.0], [-5.0, -3.0], [-4.0, 6.0]], [[3.0, -7.0], [1.0, 4.0], [2.0, -1.0]], [[0.0, 1.0], [7.0, 1.0], [0.0, 3.0]]], [[[4.0, -2.0], [2.0, -5.0], [-5.0, -3.0]], [[-4.0, 0.0], [7.0, -2.0], [6.0, -5.0]], [[5.0, -7.0], [6.0, -6.0], [-3.0, -1.0]], [[7.0, -6.0], [-6.0, 1.0], [-2.0, -3.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-3.0, -6.0, 3.0], [0.0, 1.0, -3.0], [-5.0, 0.0, -4.0], [-2.0, -6.0, -3.0]], [[0.0, 2.0, -3.0], [-2.0, 4.0, -2.0], [4.0, -5.0, -2.0], [-2.0, -2.0, 2.0]]], [[[6.0, 2.0, 2.0], [4.0, -7.0, -7.0], [4.0, 5.0, 7.0], [4.0, -5.0, -1.0]], [[-7.0, 3.0, -3.0], [-1.0, -2.0, -3.0], [-5.0, -4.0, 1.0], [-4.0, 1.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-1.0, -14.0], [-64.0, 83.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[3.0, 6.0, 6.0, -2.0, -3.0], [6.0, 3.0, 1.0, -2.0, 5.0], [3.0, 4.0, 7.0, 1.0, 6.0], [-2.0, 3.0, 3.0, -2.0, -4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[12.0, 24.0, 24.0, -8.0, -12.0], [24.0, 12.0, 4.0, -8.0, 20.0], [12.0, 16.0, 28.0, 4.0, 24.0], [-8.0, 12.0, 12.0, -8.0, -16.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[3.0, 0.0, 2.0, 0.0, 5.0], [-1.0, 0.0, 2.0, -7.0, 0.0], [3.0, 2.0, -2.0, 4.0, 0.0], [-5.0, -2.0, -1.0, 0.0, 3.0]], [[1.0, 6.0, -5.0, 1.0, 3.0], [-1.0, 1.0, 1.0, -3.0, 0.0], [-1.0, 0.0, -7.0, -3.0, -5.0], [-1.0, 4.0, -3.0, -4.0, -2.0]], [[1.0, 7.0, 4.0, 2.0, -4.0], [-2.0, -7.0, 3.0, 1.0, -1.0], [-5.0, 6.0, 5.0, 7.0, 7.0], [7.0, 5.0, 7.0, -7.0, -2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, -4.0, -6.0], [3.0, -3.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-31.0, -66.0, -18.0, -16.0, -23.0], [23.0, 38.0, -36.0, 55.0, 6.0], [13.0, -50.0, 12.0, -58.0, -22.0], [-3.0, -32.0, -23.0, 58.0, -1.0]], [[0.0, -60.0, -3.0, -15.0, 30.0], [12.0, 39.0, -15.0, -18.0, 6.0], [42.0, -30.0, -15.0, -21.0, -27.0], [-54.0, -48.0, -36.0, 54.0, 27.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[[[-7.0, -5.0, 5.0, -6.0, 6.0], [-4.0, -6.0, 2.0, 1.0, 5.0], [3.0, 3.0, 6.0, 0.0, 3.0], [2.0, 0.0, 0.0, -3.0, -2.0]], [[2.0, 5.0, -6.0, -7.0, 2.0], [-3.0, -7.0, 4.0, -2.0, -2.0], [-6.0, -1.0, 7.0, 5.0, 4.0], [0.0, 2.0, 0.0, 5.0, 4.0]], [[0.0, 2.0, -3.0, 0.0, 6.0], [2.0, -7.0, -2.0, 3.0, 7.0], [6.0, -3.0, 2.0, 0.0, 2.0], [3.0, 0.0, 6.0, 2.0, 3.0]]], [[[-6.0, 3.0, -4.0, 4.0, 1.0], [-7.0, -6.0, -4.0, -5.0, -6.0], [6.0, -7.0, 7.0, 6.0, 1.0], [0.0, 0.0, 7.0, -7.0, 0.0]], [[-7.0, 2.0, 7.0, 2.0, -1.0], [1.0, 6.0, 2.0, 1.0, 1.0], [4.0, 0.0, -4.0, -6.0, -6.0], [5.0, 6.0, 3.0, -1.0, -2.0]], [[3.0, 2.0, -1.0, 0.0, -1.0], [-1.0, 0.0, -4.0, 0.0, 7.0], [5.0, -2.0, 3.0, 6.0, 0.0], [-6.0, -3.0, -4.0, -5.0, 0.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[3.0, 0.0, 5.0], [4.0, -3.0, -3.0]], [[6.0, -6.0, 7.0], [2.0, -3.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-33.0, -5.0, -34.0, -8.0, 58.0], [-30.0, -95.0, -14.0, -5.0, 2.0], [36.0, -28.0, 59.0, 24.0, 41.0], [24.0, -9.0, 61.0, -9.0, 15.0]], [[-24.0, -32.0, 9.0, 8.0, 64.0], [-16.0, -73.0, -68.0, 26.0, 125.0], [131.0, -25.0, 55.0, 42.0, 28.0], [-24.0, -51.0, 19.0, -80.0, -9.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_constData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[-4.0, 7.0], [4.0, -1.0]], [[2.0, 0.0], [-5.0, -3.0]], [[-4.0, 7.0], [2.0, -5.0]], [[-2.0, 1.0], [1.0, 2.0]], [[6.0, 6.0], [4.0, -1.0]], [[2.0, -4.0], [1.0, -2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([7.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-28.0, 49.0], [28.0, -7.0]], [[14.0, 0.0], [-35.0, -21.0]], [[-28.0, 49.0], [14.0, -35.0]], [[-14.0, 7.0], [7.0, 14.0]], [[42.0, 42.0], [28.0, -7.0]], [[14.0, -28.0], [7.0, -14.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[[3.0, -7.0], [-4.0, 2.0]], [[-5.0, -5.0], [-7.0, -5.0]], [[1.0, -2.0], [7.0, 3.0]], [[2.0, 0.0], [6.0, -4.0]], [[6.0, 4.0], [-5.0, 4.0]], [[0.0, 7.0], [2.0, 0.0]]], [[[2.0, -4.0], [-1.0, 4.0]], [[-1.0, 0.0], [-4.0, -3.0]], [[-5.0, 5.0], [-1.0, 2.0]], [[-3.0, -4.0], [4.0, -6.0]], [[-6.0, -6.0], [0.0, 5.0]], [[-4.0, 1.0], [5.0, -5.0]]], [[[4.0, 3.0], [5.0, 0.0]], [[-3.0, -1.0], [7.0, 4.0]], [[-5.0, -7.0], [-1.0, 5.0]], [[-6.0, -6.0], [7.0, -3.0]], [[-4.0, 0.0], [3.0, -7.0]], [[2.0, 0.0], [-5.0, 4.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, -4.0, 3.0], [-4.0, -6.0, -2.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-17.0, 74.0], [47.0, -30.0]], [[30.0, 32.0], [86.0, 59.0]], [[-2.0, -27.0], [-48.0, -14.0]], [[-20.0, -2.0], [-37.0, 43.0]], [[-30.0, -4.0], [44.0, -69.0]], [[22.0, -53.0], [-49.0, 32.0]]], [[[-32.0, 46.0], [12.0, -32.0]], [[32.0, 22.0], [38.0, 30.0]], [[36.0, -8.0], [-20.0, -34.0]], [[22.0, 36.0], [-62.0, 58.0]], [[20.0, 20.0], [14.0, -32.0]], [[20.0, -34.0], [-28.0, 22.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(3.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, -1.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([4.0, -1.0])+(1.-msk_ref)*numpy.array([12.0, -3.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 5.0, -4.0])+(1.-msk_arg1)*numpy.array([0.0, 1.0, 7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[7.0, -1.0, -3.0], [3.0, -4.0, 3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([7.0, -32.0])+(1.-msk_ref)*numpy.array([-22.0, 17.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -5.0, 4.0], [6.0, 2.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 6.0, 0.0], [7.0, 3.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[3.0, 0.0, -3.0], [0.0, 3.0, 6.0]], [[0.0, -3.0, 4.0], [-4.0, 0.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-21.0, 7.0])+(1.-msk_ref)*numpy.array([-39.0, -16.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, -6.0, -4.0], [-4.0, 5.0, 2.0], [4.0, 4.0, -5.0], [3.0, 7.0, 0.0]], [[0.0, 4.0, 5.0], [0.0, 4.0, -2.0], [0.0, -2.0, 5.0], [0.0, 0.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 2.0, 2.0], [3.0, -1.0, -6.0], [-1.0, 7.0, -1.0], [6.0, -6.0, 5.0]], [[4.0, -6.0, 7.0], [3.0, 1.0, -5.0], [-3.0, 4.0, 0.0], [0.0, -2.0, -4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-5.0, -4.0, 1.0], [4.0, -5.0, 5.0], [5.0, 6.0, 4.0], [1.0, -1.0, 5.0]], [[1.0, 0.0, -2.0], [4.0, -4.0, 7.0], [-5.0, -1.0, -6.0], [0.0, 0.0, -2.0]]], [[[0.0, -2.0, 0.0], [-3.0, 3.0, 0.0], [-3.0, 1.0, 0.0], [-7.0, 2.0, 3.0]], [[-2.0, 3.0, 4.0], [-2.0, 0.0, 1.0], [3.0, -4.0, 7.0], [0.0, 1.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-50.0, 103.0])+(1.-msk_ref)*numpy.array([33.0, -89.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 2.0])+(1.-msk_arg1)*numpy.array([-2.0, -4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([5.0, -4.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[10.0, 10.0], [-8.0, -8.0]])+(1.-msk_ref)*numpy.array([[-10.0, -20.0], [8.0, 16.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, -2.0], [2.0, 5.0], [-5.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 6.0], [4.0, 0.0], [5.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, -1.0, 0.0], [0.0, -2.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-23.0, 9.0], [-9.0, -10.0]])+(1.-msk_ref)*numpy.array([[38.0, -42.0], [-3.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 0.0], [-3.0, 0.0], [6.0, 6.0]], [[1.0, 7.0], [5.0, -3.0], [-2.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, -7.0], [-1.0, -4.0], [-7.0, 5.0]], [[3.0, 2.0], [-2.0, 5.0], [-1.0, 3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[4.0, 5.0, 0.0], [-6.0, -5.0, -7.0]], [[0.0, 2.0, -7.0], [6.0, -4.0, -7.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-36.0, -62.0], [-48.0, -23.0]])+(1.-msk_ref)*numpy.array([[-2.0, -106.0], [80.0, -72.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, -4.0], [5.0, 2.0], [-7.0, -2.0]], [[0.0, 0.0], [4.0, 2.0], [4.0, -5.0]], [[6.0, -5.0], [3.0, -2.0], [1.0, 0.0]], [[1.0, 4.0], [0.0, -6.0], [4.0, -3.0]]], [[[-4.0, -4.0], [-5.0, -3.0], [-4.0, -4.0]], [[5.0, 3.0], [6.0, -1.0], [2.0, -2.0]], [[7.0, -3.0], [-3.0, -2.0], [1.0, 5.0]], [[5.0, 1.0], [6.0, 0.0], [-4.0, -2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, -5.0], [0.0, -5.0], [-2.0, -3.0]], [[-4.0, 1.0], [7.0, 7.0], [7.0, 4.0]], [[-4.0, 0.0], [-6.0, 0.0], [-7.0, -4.0]], [[1.0, 1.0], [-3.0, 7.0], [-7.0, -3.0]]], [[[0.0, 1.0], [-7.0, -5.0], [0.0, -7.0]], [[7.0, 7.0], [1.0, -2.0], [2.0, -6.0]], [[7.0, 5.0], [0.0, -5.0], [-2.0, 6.0]], [[-6.0, 6.0], [-7.0, -7.0], [-1.0, 6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[0.0, -3.0, -4.0], [7.0, 6.0, -1.0], [3.0, 7.0, 6.0], [-2.0, -1.0, 0.0]], [[-7.0, -7.0, 0.0], [3.0, 3.0, 7.0], [-2.0, -4.0, 7.0], [-2.0, 0.0, -6.0]]], [[[0.0, -6.0, 1.0], [4.0, -5.0, -6.0], [-1.0, -7.0, -7.0], [5.0, 6.0, 6.0]], [[0.0, -7.0, 1.0], [4.0, 1.0, 0.0], [7.0, -1.0, -4.0], [0.0, -4.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[205.0, 88.0], [-29.0, -32.0]])+(1.-msk_ref)*numpy.array([[-3.0, 44.0], [102.0, 163.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 6.0, -4.0, -3.0, 4.0], [7.0, 0.0, 7.0, -6.0, -1.0], [-4.0, -1.0, 0.0, 0.0, 2.0], [-6.0, 2.0, 4.0, 0.0, -1.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -6.0, -3.0, 3.0, -3.0], [-6.0, 3.0, -6.0, 0.0, 4.0], [-3.0, 3.0, -3.0, -2.0, -7.0], [2.0, 0.0, 2.0, 7.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-3.0, 4.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[12.0, -18.0, 12.0, 9.0, -12.0], [-21.0, 0.0, -21.0, 18.0, 3.0], [12.0, 3.0, 0.0, 0.0, -6.0], [18.0, -6.0, -12.0, 0.0, 3.0]], [[-16.0, 24.0, -16.0, -12.0, 16.0], [28.0, 0.0, 28.0, -24.0, -4.0], [-16.0, -4.0, 0.0, 0.0, 8.0], [-24.0, 8.0, 16.0, 0.0, -4.0]]])+(1.-msk_ref)*numpy.array([[[21.0, 18.0, 9.0, -9.0, 9.0], [18.0, -9.0, 18.0, 0.0, -12.0], [9.0, -9.0, 9.0, 6.0, 21.0], [-6.0, 0.0, -6.0, -21.0, 18.0]], [[-28.0, -24.0, -12.0, 12.0, -12.0], [-24.0, 12.0, -24.0, 0.0, 16.0], [-12.0, 12.0, -12.0, -8.0, -28.0], [8.0, 0.0, 8.0, 28.0, -24.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -7.0, 7.0, 4.0, 6.0], [-3.0, 6.0, 1.0, -4.0, 0.0], [7.0, 3.0, 5.0, 7.0, -6.0], [-5.0, -6.0, 4.0, 5.0, 6.0]], [[-5.0, -2.0, 7.0, 0.0, 4.0], [-6.0, -7.0, -4.0, 3.0, 4.0], [4.0, -5.0, 0.0, 7.0, -2.0], [2.0, 0.0, -5.0, 7.0, -3.0]], [[3.0, 4.0, 3.0, 3.0, 3.0], [0.0, 5.0, 6.0, 1.0, -4.0], [7.0, -4.0, -2.0, 2.0, -3.0], [-7.0, 0.0, -4.0, 4.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -3.0, -3.0, 2.0, 2.0], [0.0, -4.0, 7.0, -1.0, -5.0], [-2.0, 5.0, 7.0, 0.0, 4.0], [-4.0, 3.0, 7.0, -4.0, 0.0]], [[0.0, 0.0, 7.0, -3.0, 2.0], [5.0, -7.0, -6.0, 1.0, -5.0], [6.0, 0.0, -3.0, -5.0, 7.0], [5.0, 5.0, 6.0, 5.0, 0.0]], [[0.0, 4.0, 3.0, -7.0, -4.0], [-7.0, 2.0, -3.0, 3.0, -6.0], [4.0, 2.0, -7.0, 0.0, 2.0], [4.0, 5.0, 5.0, 0.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[7.0, 7.0, -3.0], [-4.0, -4.0, 2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-44.0, -75.0, 89.0, 19.0, 61.0], [-63.0, -22.0, -39.0, -10.0, 40.0], [56.0, -2.0, 41.0, 92.0, -47.0], [0.0, -42.0, 5.0, 72.0, 0.0]], [[26.0, 44.0, -50.0, -10.0, -34.0], [36.0, 14.0, 24.0, 6.0, -24.0], [-30.0, 0.0, -24.0, -52.0, 26.0], [-2.0, 24.0, -4.0, -40.0, 2.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, -33.0, 19.0, 14.0, 40.0], [56.0, -83.0, 16.0, -9.0, -52.0], [16.0, 29.0, 49.0, -35.0, 71.0], [-5.0, 41.0, 76.0, 7.0, -12.0]], [[20.0, 20.0, -10.0, -10.0, -24.0], [-34.0, 48.0, -10.0, 6.0, 28.0], [-8.0, -16.0, -30.0, 20.0, -40.0], [4.0, -22.0, -42.0, -4.0, 8.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, 7.0, -2.0, 2.0, 4.0], [-7.0, -3.0, 0.0, 6.0, -5.0], [0.0, -4.0, -2.0, 4.0, -7.0], [6.0, 3.0, -2.0, 2.0, 4.0]], [[5.0, 0.0, 3.0, -2.0, 1.0], [0.0, 0.0, -7.0, 4.0, -4.0], [-1.0, -5.0, 2.0, -4.0, 3.0], [-5.0, 6.0, 0.0, -2.0, 1.0]], [[-2.0, 0.0, -7.0, -7.0, -5.0], [3.0, -2.0, 2.0, 3.0, -7.0], [-3.0, -5.0, -5.0, 6.0, 3.0], [-1.0, 6.0, 0.0, 4.0, 7.0]]], [[[-5.0, 0.0, 0.0, 0.0, -5.0], [-3.0, -3.0, -2.0, -5.0, 6.0], [3.0, 3.0, 5.0, 6.0, -7.0], [2.0, -3.0, 7.0, 7.0, 4.0]], [[-6.0, 4.0, 3.0, -7.0, 0.0], [5.0, 4.0, 6.0, 0.0, -7.0], [5.0, 3.0, 4.0, -1.0, 1.0], [5.0, -6.0, 6.0, -2.0, -2.0]], [[3.0, -4.0, -2.0, 7.0, 6.0], [0.0, 5.0, -7.0, 2.0, -6.0], [4.0, 4.0, 4.0, -4.0, 2.0], [7.0, 0.0, 3.0, -2.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, 4.0, 4.0, -3.0, -2.0], [-3.0, -5.0, 1.0, 7.0, 1.0], [1.0, 2.0, 0.0, 0.0, -7.0], [-1.0, 1.0, 1.0, -1.0, 1.0]], [[-3.0, -7.0, 0.0, 0.0, 5.0], [0.0, -5.0, -2.0, 2.0, 4.0], [-7.0, 0.0, -2.0, 6.0, -7.0], [3.0, -1.0, 4.0, -7.0, -4.0]], [[3.0, 5.0, 1.0, 2.0, -5.0], [-1.0, 3.0, -3.0, 6.0, 0.0], [-1.0, 5.0, -6.0, -4.0, 7.0], [7.0, -3.0, 6.0, -4.0, 7.0]]], [[[-3.0, 0.0, 4.0, 3.0, 1.0], [1.0, 6.0, 4.0, 1.0, 7.0], [-1.0, 1.0, 0.0, 3.0, 0.0], [-1.0, 0.0, -3.0, -7.0, -1.0]], [[5.0, 5.0, -3.0, 2.0, 7.0], [-4.0, 6.0, 7.0, 0.0, -1.0], [7.0, 7.0, 0.0, 2.0, 2.0], [-2.0, -6.0, -3.0, 7.0, 0.0]], [[5.0, 7.0, 2.0, -7.0, 7.0], [7.0, 7.0, 0.0, 4.0, 0.0], [2.0, -3.0, 7.0, -6.0, 0.0], [4.0, -6.0, 3.0, 7.0, 5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[0.0, 5.0, -7.0], [-1.0, 5.0, 7.0]], [[-5.0, -2.0, -7.0], [-5.0, 7.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[35.0, -8.0, 65.0, 53.0, 87.0], [7.0, 72.0, -66.0, 18.0, -54.0], [66.0, 50.0, 88.0, -101.0, 20.0], [54.0, -39.0, 44.0, -69.0, -16.0]], [[-63.0, 13.0, 84.0, -41.0, 8.0], [64.0, 47.0, 87.0, -44.0, 33.0], [23.0, 51.0, 24.0, -71.0, 40.0], [-23.0, -96.0, 2.0, -73.0, -135.0]]])+(1.-msk_ref)*numpy.array([[[27.0, 4.0, -12.0, -56.0, 143.0], [35.0, 27.0, 42.0, -5.0, 8.0], [22.0, -22.0, 81.0, 23.0, -74.0], [-15.0, -56.0, -13.0, 84.0, -33.0]], [[40.0, -41.0, -78.0, 35.0, 44.0], [-46.0, -9.0, 49.0, -106.0, -55.0], [60.0, 14.0, 11.0, 45.0, 14.0], [-79.0, 6.0, -76.0, 96.0, -66.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank1_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, -2.0], [2.0, 1.0]], [[-3.0, -5.0], [1.0, 0.0]], [[0.0, 1.0], [-3.0, -7.0]], [[-3.0, 2.0], [1.0, 0.0]], [[4.0, -7.0], [5.0, 3.0]], [[5.0, 6.0], [0.0, -1.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 4.0], [5.0, -6.0]], [[6.0, -4.0], [7.0, 5.0]], [[-7.0, -2.0], [0.0, 7.0]], [[-2.0, -4.0], [-1.0, 6.0]], [[6.0, 4.0], [2.0, 2.0]], [[-4.0, -1.0], [-6.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, -3.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[21.0, 14.0], [-14.0, -7.0]], [[21.0, 35.0], [-7.0, 0.0]], [[0.0, -7.0], [21.0, 49.0]], [[21.0, -14.0], [-7.0, 0.0]], [[-28.0, 49.0], [-35.0, -21.0]], [[-35.0, -42.0], [0.0, 7.0]]], [[[9.0, 6.0], [-6.0, -3.0]], [[9.0, 15.0], [-3.0, 0.0]], [[0.0, -3.0], [9.0, 21.0]], [[9.0, -6.0], [-3.0, 0.0]], [[-12.0, 21.0], [-15.0, -9.0]], [[-15.0, -18.0], [0.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -28.0], [-35.0, 42.0]], [[-42.0, 28.0], [-49.0, -35.0]], [[49.0, 14.0], [0.0, -49.0]], [[14.0, 28.0], [7.0, -42.0]], [[-42.0, -28.0], [-14.0, -14.0]], [[28.0, 7.0], [42.0, -28.0]]], [[[-9.0, -12.0], [-15.0, 18.0]], [[-18.0, 12.0], [-21.0, -15.0]], [[21.0, 6.0], [0.0, -21.0]], [[6.0, 12.0], [3.0, -18.0]], [[-18.0, -12.0], [-6.0, -6.0]], [[12.0, 3.0], [18.0, -12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, 3.0], [0.0, 7.0]], [[-5.0, -2.0], [-6.0, 6.0]], [[-2.0, 7.0], [1.0, 6.0]], [[1.0, 1.0], [-7.0, -6.0]], [[1.0, -7.0], [0.0, -5.0]], [[5.0, 7.0], [-2.0, 5.0]]], [[[0.0, 5.0], [2.0, -5.0]], [[1.0, -2.0], [1.0, 7.0]], [[4.0, 0.0], [-3.0, -7.0]], [[-6.0, -7.0], [6.0, -4.0]], [[7.0, -1.0], [0.0, 2.0]], [[-5.0, -3.0], [0.0, 2.0]]], [[[0.0, 0.0], [7.0, -2.0]], [[-1.0, 2.0], [0.0, -3.0]], [[4.0, -1.0], [1.0, -1.0]], [[6.0, 2.0], [3.0, -4.0]], [[4.0, 1.0], [-3.0, 5.0]], [[5.0, 7.0], [6.0, -2.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, -4.0], [-5.0, 3.0]], [[4.0, -3.0], [-7.0, -4.0]], [[-1.0, 0.0], [6.0, -6.0]], [[0.0, -7.0], [0.0, -3.0]], [[1.0, 2.0], [1.0, 4.0]], [[-7.0, -5.0], [-3.0, 3.0]]], [[[3.0, 3.0], [6.0, -2.0]], [[-3.0, 0.0], [5.0, -6.0]], [[6.0, -5.0], [4.0, 2.0]], [[-1.0, 4.0], [-4.0, 4.0]], [[-1.0, 6.0], [-6.0, 6.0]], [[-6.0, 5.0], [-5.0, 0.0]]], [[[1.0, 1.0], [5.0, -2.0]], [[-1.0, 2.0], [3.0, 1.0]], [[-7.0, 5.0], [6.0, -1.0]], [[1.0, -5.0], [3.0, -4.0]], [[-3.0, 1.0], [-3.0, -7.0]], [[-2.0, 0.0], [4.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[3.0, -4.0, 0.0], [7.0, 3.0, -2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-9.0, -11.0], [-8.0, 41.0]], [[-19.0, 2.0], [-22.0, -10.0]], [[-22.0, 21.0], [15.0, 46.0]], [[27.0, 31.0], [-45.0, -2.0]], [[-25.0, -17.0], [0.0, -23.0]], [[35.0, 33.0], [-6.0, 7.0]]], [[[-21.0, 36.0], [-8.0, 38.0]], [[-30.0, -24.0], [-39.0, 69.0]], [[-10.0, 51.0], [-4.0, 23.0]], [[-23.0, -18.0], [-37.0, -46.0]], [[20.0, -54.0], [6.0, -39.0]], [[10.0, 26.0], [-26.0, 45.0]]]])+(1.-msk_ref)*numpy.array([[[[-9.0, -24.0], [-39.0, 17.0]], [[24.0, -9.0], [-41.0, 12.0]], [[-27.0, 20.0], [2.0, -26.0]], [[4.0, -37.0], [16.0, -25.0]], [[7.0, -18.0], [27.0, -12.0]], [[3.0, -35.0], [11.0, 9.0]]], [[[14.0, -21.0], [-27.0, 19.0]], [[21.0, -25.0], [-40.0, -48.0]], [[25.0, -25.0], [42.0, -34.0]], [[-5.0, -27.0], [-18.0, -1.0]], [[10.0, 30.0], [-5.0, 60.0]], [[-63.0, -20.0], [-44.0, 19.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_float_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=4.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[7.0, -5.0, -3.0, -6.0, -4.0], [0.0, -3.0, 0.0, 7.0, 0.0], [3.0, -2.0, 3.0, 7.0, -7.0], [1.0, 1.0, 4.0, 4.0, 6.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[28.0, -20.0, -12.0, -24.0, -16.0], [0.0, -12.0, 0.0, 28.0, 0.0], [12.0, -8.0, 12.0, 28.0, -28.0], [4.0, 4.0, 16.0, 16.0, 24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-5.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[0.0, -5.0, -7.0, 0.0, -1.0], [-6.0, 3.0, 6.0, 0.0, 6.0], [-7.0, -2.0, 0.0, -7.0, 0.0], [-3.0, 6.0, 7.0, -7.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.0, 25.0, 35.0, -0.0, 5.0], [30.0, -15.0, -30.0, -0.0, -30.0], [35.0, 10.0, -0.0, 35.0, -0.0], [15.0, -30.0, -35.0, 35.0, -0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([3.0, 2.0, -4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[0.0, -5.0, -1.0], [5.0, -4.0, 6.0], [-1.0, -3.0, 5.0], [-7.0, -5.0, 4.0], [-3.0, 2.0, -2.0]], [[7.0, 5.0, 7.0], [5.0, 1.0, -2.0], [2.0, -6.0, 1.0], [-2.0, 3.0, 4.0], [-4.0, -3.0, 4.0]], [[-2.0, -7.0, -1.0], [3.0, 6.0, 5.0], [2.0, -7.0, 1.0], [-3.0, 7.0, 5.0], [-7.0, 1.0, 2.0]], [[2.0, 0.0, -7.0], [5.0, -2.0, 2.0], [-1.0, 1.0, 5.0], [7.0, 2.0, 7.0], [0.0, -4.0, -6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-6.0, -17.0, -29.0, -47.0, 3.0], [3.0, 25.0, -10.0, -16.0, -34.0], [-16.0, 1.0, -12.0, -15.0, -27.0], [34.0, 3.0, -21.0, -3.0, 16.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[0.0, -2.0, -2.0], [2.0, 3.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[6.0, 3.0, -2.0], [-7.0, -4.0, -4.0]], [[-5.0, 4.0, 1.0], [7.0, 3.0, 2.0]], [[-7.0, 3.0, -6.0], [-3.0, 4.0, 7.0]], [[-6.0, 0.0, 1.0], [7.0, -6.0, 0.0]], [[6.0, 5.0, -5.0], [2.0, -4.0, -2.0]]], [[[-4.0, 6.0, -2.0], [0.0, -1.0, 1.0]], [[-1.0, -3.0, 2.0], [4.0, -1.0, 7.0]], [[-1.0, 5.0, 0.0], [-5.0, -5.0, -3.0]], [[0.0, 5.0, 2.0], [6.0, -4.0, 5.0]], [[-5.0, 0.0, -5.0], [-4.0, 0.0, 0.0]]], [[[-6.0, 3.0, 6.0], [0.0, -5.0, 6.0]], [[2.0, -7.0, -1.0], [-3.0, 3.0, -6.0]], [[5.0, 0.0, -2.0], [6.0, 3.0, -7.0]], [[5.0, -2.0, 3.0], [-1.0, -5.0, 5.0]], [[4.0, 3.0, 7.0], [-7.0, -2.0, -5.0]]], [[[-3.0, -4.0, 7.0], [-4.0, -3.0, 4.0]], [[2.0, 0.0, -7.0], [0.0, 5.0, -3.0]], [[-3.0, -2.0, -2.0], [6.0, -3.0, -5.0]], [[1.0, 4.0, -3.0], [6.0, 5.0, 7.0]], [[2.0, -5.0, -5.0], [-6.0, 0.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-40.0, 19.0, 33.0, -6.0, -14.0], [-8.0, 28.0, -44.0, 1.0, 2.0], [-15.0, 1.0, 4.0, -4.0, -55.0], [-11.0, 20.0, -4.0, 46.0, 23.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([7.0, 7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[1.0, -4.0, 0.0, -6.0, -5.0], [5.0, -6.0, 0.0, -3.0, 4.0], [-2.0, -6.0, 7.0, 7.0, 2.0], [5.0, -6.0, -7.0, 0.0, 2.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[7.0, 7.0], [-28.0, -28.0], [0.0, 0.0], [-42.0, -42.0], [-35.0, -35.0]], [[35.0, 35.0], [-42.0, -42.0], [0.0, 0.0], [-21.0, -21.0], [28.0, 28.0]], [[-14.0, -14.0], [-42.0, -42.0], [49.0, 49.0], [49.0, 49.0], [14.0, 14.0]], [[35.0, 35.0], [-42.0, -42.0], [-49.0, -49.0], [0.0, 0.0], [14.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([[1.0, 7.0], [-6.0, 4.0], [-5.0, -5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[6.0, 3.0, -4.0], [5.0, -2.0, -3.0], [5.0, -3.0, -6.0], [5.0, 0.0, -6.0], [5.0, -5.0, 3.0]], [[-4.0, 6.0, -6.0], [1.0, -6.0, -1.0], [5.0, -3.0, -5.0], [-5.0, 3.0, 2.0], [-6.0, 4.0, -3.0]], [[-6.0, 7.0, 0.0], [-6.0, 3.0, -4.0], [4.0, -3.0, 3.0], [4.0, -3.0, -6.0], [-3.0, 7.0, 2.0]], [[-6.0, -4.0, 5.0], [-3.0, -2.0, 4.0], [5.0, 1.0, 0.0], [-7.0, 0.0, 0.0], [-5.0, 0.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[8.0, 74.0], [32.0, 42.0], [53.0, 53.0], [35.0, 65.0], [20.0, 0.0]], [[-10.0, 26.0], [42.0, -12.0], [48.0, 48.0], [-33.0, -33.0], [-15.0, -11.0]], [[-48.0, -14.0], [-4.0, -10.0], [7.0, 1.0], [52.0, 46.0], [-55.0, -3.0]], [[-7.0, -83.0], [-11.0, -49.0], [-1.0, 39.0], [-7.0, -49.0], [5.0, -25.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[-2.0, 4.0], [4.0, -5.0], [-7.0, 3.0]], [[-5.0, -7.0], [-7.0, 0.0], [-1.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[4.0, -7.0, -7.0], [6.0, 0.0, -3.0]], [[4.0, -5.0, 6.0], [5.0, 0.0, 6.0]], [[2.0, 6.0, -1.0], [-3.0, -1.0, 0.0]], [[7.0, 0.0, -3.0], [-7.0, -1.0, 0.0]], [[6.0, 5.0, -4.0], [7.0, 0.0, 6.0]]], [[[0.0, 3.0, 1.0], [-4.0, 0.0, -7.0]], [[6.0, 5.0, 0.0], [2.0, -2.0, -4.0]], [[0.0, -4.0, -2.0], [1.0, -1.0, -1.0]], [[-2.0, 7.0, 3.0], [-7.0, 0.0, -6.0]], [[-7.0, 7.0, 1.0], [4.0, 0.0, -1.0]]], [[[-1.0, 2.0, 0.0], [-6.0, -4.0, 3.0]], [[7.0, 4.0, 6.0], [-5.0, -7.0, -7.0]], [[-4.0, -4.0, 3.0], [3.0, 7.0, 7.0]], [[4.0, -5.0, -5.0], [3.0, -5.0, -1.0]], [[-6.0, 1.0, 4.0], [5.0, 1.0, 0.0]]], [[[-4.0, 0.0, 5.0], [0.0, -3.0, 0.0]], [[3.0, 7.0, 7.0], [6.0, -1.0, 6.0]], [[7.0, -7.0, -7.0], [-7.0, 0.0, -6.0]], [[-3.0, -2.0, 2.0], [3.0, -1.0, 4.0]], [[-6.0, 1.0, 0.0], [1.0, -7.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-14.0, -24.0], [-101.0, 48.0], [49.0, -4.0], [49.0, 68.0], [-5.0, -38.0]], [[32.0, -12.0], [16.0, -31.0], [1.0, 3.0], [52.0, -9.0], [16.0, -92.0]], [[65.0, 40.0], [41.0, 33.0], [-100.0, 20.0], [28.0, 1.0], [-44.0, -52.0]], [[-6.0, -1.0], [-56.0, -20.0], [48.0, 67.0], [-28.0, -1.0], [54.0, -12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_array_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[5.0, -7.0, -6.0, -1.0, -4.0], [-7.0, -5.0, -2.0, -2.0, 2.0], [0.0, -7.0, -7.0, 6.0, -2.0], [3.0, 2.0, -2.0, 3.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-1.0, -6.0, -2.0, 0.0, 0.0], [6.0, 5.0, -4.0, -3.0, 5.0], [-1.0, -2.0, 0.0, 7.0, -6.0], [0.0, -5.0, 0.0, 2.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-5.0, 7.0, 6.0, 1.0, 4.0], [7.0, 5.0, 2.0, 2.0, -2.0], [0.0, 7.0, 7.0, -6.0, 2.0], [-3.0, -2.0, 2.0, -3.0, 0.0]], [[-30.0, 42.0, 36.0, 6.0, 24.0], [42.0, 30.0, 12.0, 12.0, -12.0], [0.0, 42.0, 42.0, -36.0, 12.0], [-18.0, -12.0, 12.0, -18.0, 0.0]], [[-10.0, 14.0, 12.0, 2.0, 8.0], [14.0, 10.0, 4.0, 4.0, -4.0], [0.0, 14.0, 14.0, -12.0, 4.0], [-6.0, -4.0, 4.0, -6.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[30.0, -42.0, -36.0, -6.0, -24.0], [-42.0, -30.0, -12.0, -12.0, 12.0], [0.0, -42.0, -42.0, 36.0, -12.0], [18.0, 12.0, -12.0, 18.0, 0.0]], [[25.0, -35.0, -30.0, -5.0, -20.0], [-35.0, -25.0, -10.0, -10.0, 10.0], [0.0, -35.0, -35.0, 30.0, -10.0], [15.0, 10.0, -10.0, 15.0, 0.0]], [[-20.0, 28.0, 24.0, 4.0, 16.0], [28.0, 20.0, 8.0, 8.0, -8.0], [0.0, 28.0, 28.0, -24.0, 8.0], [-12.0, -8.0, 8.0, -12.0, 0.0]], [[-15.0, 21.0, 18.0, 3.0, 12.0], [21.0, 15.0, 6.0, 6.0, -6.0], [0.0, 21.0, 21.0, -18.0, 6.0], [-9.0, -6.0, 6.0, -9.0, 0.0]], [[25.0, -35.0, -30.0, -5.0, -20.0], [-35.0, -25.0, -10.0, -10.0, 10.0], [0.0, -35.0, -35.0, 30.0, -10.0], [15.0, 10.0, -10.0, 15.0, 0.0]]], [[[-5.0, 7.0, 6.0, 1.0, 4.0], [7.0, 5.0, 2.0, 2.0, -2.0], [0.0, 7.0, 7.0, -6.0, 2.0], [-3.0, -2.0, 2.0, -3.0, 0.0]], [[-10.0, 14.0, 12.0, 2.0, 8.0], [14.0, 10.0, 4.0, 4.0, -4.0], [0.0, 14.0, 14.0, -12.0, 4.0], [-6.0, -4.0, 4.0, -6.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[35.0, -49.0, -42.0, -7.0, -28.0], [-49.0, -35.0, -14.0, -14.0, 14.0], [0.0, -49.0, -49.0, 42.0, -14.0], [21.0, 14.0, -14.0, 21.0, 0.0]], [[-30.0, 42.0, 36.0, 6.0, 24.0], [42.0, 30.0, 12.0, 12.0, -12.0], [0.0, 42.0, 42.0, -36.0, 12.0], [-18.0, -12.0, 12.0, -18.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-25.0, 35.0, 30.0, 5.0, 20.0], [35.0, 25.0, 10.0, 10.0, -10.0], [0.0, 35.0, 35.0, -30.0, 10.0], [-15.0, -10.0, 10.0, -15.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[10.0, -14.0, -12.0, -2.0, -8.0], [-14.0, -10.0, -4.0, -4.0, 4.0], [0.0, -14.0, -14.0, 12.0, -4.0], [6.0, 4.0, -4.0, 6.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([[[5.0, -5.0, -5.0, 0.0, 5.0], [5.0, 0.0, 0.0, -3.0, 0.0], [-4.0, 2.0, 7.0, 7.0, 3.0], [3.0, 7.0, -3.0, 0.0, 4.0]], [[0.0, -1.0, 1.0, -1.0, -5.0], [-5.0, 6.0, 0.0, 6.0, 6.0], [-7.0, -5.0, 6.0, 3.0, -2.0], [-2.0, 7.0, 5.0, -1.0, 4.0]], [[-2.0, -6.0, -1.0, 6.0, -1.0], [-1.0, -2.0, 0.0, -5.0, 4.0], [5.0, 1.0, 5.0, 7.0, -5.0], [-2.0, 7.0, 0.0, 0.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, -1.0, -1.0], [-6.0, -3.0, 0.0], [-1.0, -6.0, -3.0], [3.0, 5.0, 0.0], [1.0, 6.0, -3.0]], [[3.0, -6.0, 0.0], [2.0, 2.0, 7.0], [1.0, 2.0, 2.0], [4.0, 0.0, -4.0], [6.0, -3.0, 4.0]], [[6.0, 7.0, -5.0], [-7.0, -7.0, -1.0], [5.0, 2.0, 4.0], [0.0, 4.0, 0.0], [3.0, 7.0, -7.0]], [[0.0, 0.0, -6.0], [-5.0, 4.0, -6.0], [0.0, -1.0, -4.0], [0.0, 0.0, 5.0], [-4.0, -1.0, 5.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[27.0, -18.0, -25.0, -5.0, 31.0], [31.0, -4.0, 0.0, -16.0, -10.0], [-18.0, 14.0, 24.0, 25.0, 22.0], [19.0, 21.0, -20.0, 1.0, 14.0]], [[-30.0, 33.0, 27.0, 3.0, -15.0], [-15.0, -18.0, 0.0, 0.0, -18.0], [45.0, 3.0, -60.0, -51.0, -12.0], [-12.0, -63.0, 3.0, 3.0, -36.0]], [[1.0, 29.0, 2.0, -12.0, 28.0], [28.0, -30.0, 0.0, -18.0, -48.0], [31.0, 25.0, -58.0, -46.0, 24.0], [15.0, -70.0, -27.0, 6.0, -34.0]], [[15.0, -20.0, -10.0, -5.0, -10.0], [-10.0, 30.0, 0.0, 21.0, 30.0], [-47.0, -19.0, 51.0, 36.0, -1.0], [-1.0, 56.0, 16.0, -5.0, 32.0]], [[11.0, 7.0, 4.0, -24.0, -22.0], [-22.0, 42.0, 0.0, 48.0, 24.0], [-61.0, -31.0, 28.0, 4.0, 6.0], [-3.0, 28.0, 27.0, -6.0, 22.0]]], [[[15.0, -9.0, -21.0, 6.0, 45.0], [45.0, -36.0, 0.0, -45.0, -36.0], [30.0, 36.0, -15.0, 3.0, 21.0], [21.0, -21.0, -39.0, 6.0, -12.0]], [[-4.0, -54.0, -15.0, 40.0, -7.0], [-7.0, -2.0, 0.0, -29.0, 40.0], [13.0, 1.0, 61.0, 69.0, -33.0], [-12.0, 77.0, 4.0, -2.0, 30.0]], [[1.0, -19.0, -5.0, 10.0, -7.0], [-7.0, 8.0, 0.0, -1.0, 20.0], [-8.0, -6.0, 29.0, 27.0, -11.0], [-5.0, 35.0, 7.0, -2.0, 16.0]], [[28.0, 4.0, -16.0, -24.0, 24.0], [24.0, 8.0, 0.0, 8.0, -16.0], [-36.0, 4.0, 8.0, 0.0, 32.0], [20.0, 0.0, -12.0, 0.0, 8.0]], [[22.0, -51.0, -37.0, 27.0, 41.0], [41.0, -26.0, 0.0, -56.0, -2.0], [17.0, 31.0, 44.0, 61.0, 4.0], [16.0, 49.0, -33.0, 3.0, 20.0]]], [[[40.0, -7.0, -18.0, -37.0, 0.0], [0.0, 52.0, 0.0, 49.0, 22.0], [-98.0, -28.0, 59.0, 28.0, 29.0], [14.0, 56.0, 17.0, -7.0, 42.0]], [[-33.0, 48.0, 29.0, 1.0, 1.0], [1.0, -40.0, 0.0, -16.0, -46.0], [72.0, 20.0, -96.0, -77.0, -2.0], [-5.0, -105.0, -14.0, 7.0, -58.0]], [[17.0, -51.0, -27.0, 22.0, 11.0], [11.0, 4.0, 0.0, -23.0, 28.0], [-14.0, 4.0, 67.0, 69.0, -9.0], [3.0, 77.0, -5.0, -2.0, 36.0]], [[0.0, -4.0, 4.0, -4.0, -20.0], [-20.0, 24.0, 0.0, 24.0, 24.0], [-28.0, -20.0, 24.0, 12.0, -8.0], [-8.0, 28.0, 20.0, -4.0, 16.0]], [[29.0, 20.0, -1.0, -49.0, -13.0], [-13.0, 56.0, 0.0, 68.0, 14.0], [-96.0, -36.0, 28.0, -7.0, 30.0], [9.0, 21.0, 26.0, -7.0, 26.0]]], [[[12.0, 36.0, 6.0, -36.0, 6.0], [6.0, 12.0, 0.0, 30.0, -24.0], [-30.0, -6.0, -30.0, -42.0, 30.0], [12.0, -42.0, 0.0, 0.0, -12.0]], [[-13.0, 57.0, 35.0, -40.0, -39.0], [-39.0, 36.0, 0.0, 69.0, 0.0], [-38.0, -36.0, -41.0, -65.0, 7.0], [-11.0, -49.0, 35.0, -4.0, -16.0]], [[8.0, 25.0, 3.0, -23.0, 9.0], [9.0, 2.0, 0.0, 14.0, -22.0], [-13.0, 1.0, -26.0, -31.0, 22.0], [10.0, -35.0, -5.0, 1.0, -12.0]], [[-10.0, -30.0, -5.0, 30.0, -5.0], [-5.0, -10.0, 0.0, -25.0, 20.0], [25.0, 5.0, 25.0, 35.0, -25.0], [-10.0, 35.0, 0.0, 0.0, 10.0]], [[-30.0, -9.0, 14.0, 31.0, -20.0], [-20.0, -16.0, 0.0, -19.0, 14.0], [48.0, 2.0, -9.0, 4.0, -35.0], [-20.0, 0.0, 7.0, 1.0, -10.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[[0.0, -5.0, 3.0, -3.0, -3.0], [3.0, -5.0, 1.0, 6.0, 0.0], [0.0, 6.0, 2.0, 0.0, -5.0], [0.0, -6.0, 3.0, 7.0, 2.0]], [[5.0, 3.0, 6.0, 3.0, -4.0], [-2.0, -1.0, -5.0, 1.0, -3.0], [-1.0, 5.0, -4.0, -6.0, -1.0], [0.0, -6.0, 7.0, 7.0, 4.0]], [[7.0, -7.0, -3.0, -6.0, -5.0], [5.0, 3.0, 5.0, -4.0, 7.0], [-4.0, 1.0, 0.0, 2.0, 6.0], [1.0, 7.0, -7.0, -5.0, 1.0]]], [[[-7.0, 1.0, -7.0, 6.0, 7.0], [2.0, 7.0, 6.0, -1.0, 2.0], [4.0, -1.0, -2.0, 7.0, 7.0], [0.0, 3.0, -3.0, -6.0, 2.0]], [[6.0, 5.0, 0.0, 6.0, 2.0], [-4.0, 0.0, -1.0, -6.0, 0.0], [-3.0, 5.0, -4.0, 0.0, 3.0], [0.0, -6.0, 5.0, -5.0, -7.0]], [[4.0, -3.0, -1.0, 0.0, 1.0], [-5.0, 5.0, 7.0, 1.0, 2.0], [-5.0, -4.0, -1.0, 3.0, 6.0], [7.0, -6.0, 7.0, -7.0, 5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[0.0, 3.0, 0.0], [-1.0, -7.0, -5.0]], [[1.0, 7.0, 0.0], [2.0, 1.0, -5.0]], [[-6.0, 4.0, -6.0], [-4.0, 1.0, 0.0]], [[-5.0, 6.0, -3.0], [2.0, 5.0, -4.0]], [[-2.0, 0.0, -3.0], [4.0, -1.0, -2.0]]], [[[-1.0, 4.0, -5.0], [1.0, -6.0, 7.0]], [[6.0, -3.0, 0.0], [-6.0, 1.0, -1.0]], [[-6.0, 1.0, 2.0], [-5.0, -3.0, 2.0]], [[2.0, 3.0, 0.0], [-2.0, 2.0, -5.0]], [[4.0, 1.0, -1.0], [0.0, 3.0, -6.0]]], [[[-1.0, 7.0, 3.0], [3.0, -7.0, -6.0]], [[6.0, 3.0, -6.0], [4.0, -2.0, -2.0]], [[0.0, -2.0, 0.0], [0.0, 5.0, 0.0]], [[3.0, 7.0, 0.0], [5.0, -2.0, 6.0]], [[5.0, 1.0, 4.0], [-7.0, 6.0, 3.0]]], [[[-6.0, -1.0, 1.0], [3.0, 0.0, -6.0]], [[0.0, -7.0, -5.0], [3.0, -7.0, 2.0]], [[4.0, 2.0, -5.0], [-2.0, -6.0, 0.0]], [[0.0, 7.0, -7.0], [5.0, -3.0, 6.0]], [[2.0, -3.0, 6.0], [3.0, 3.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-40.0, -12.0, 30.0, -39.0, -38.0], [45.0, -35.0, -49.0, 41.0, -21.0], [39.0, 1.0, 23.0, -40.0, -61.0], [-35.0, 51.0, -46.0, 97.0, 34.0]], [[7.0, 38.0, 36.0, 36.0, -20.0], [14.0, -23.0, -58.0, 0.0, -27.0], [23.0, 64.0, -29.0, -43.0, -25.0], [-35.0, -18.0, 16.0, 74.0, 2.0]], [[12.0, 85.0, 52.0, 48.0, 6.0], [-68.0, -20.0, -81.0, -10.0, -62.0], [1.0, -13.0, -24.0, -64.0, -35.0], [-6.0, -48.0, 69.0, 35.0, -17.0]], [[9.0, 103.0, 20.0, 93.0, 26.0], [-38.0, 4.0, -71.0, -48.0, -43.0], [19.0, 36.0, -54.0, -40.0, 6.0], [-31.0, -27.0, 39.0, 13.0, -40.0]], [[-63.0, 36.0, -23.0, 42.0, 45.0], [1.0, 19.0, -6.0, 0.0, -17.0], [41.0, -16.0, -6.0, 16.0, 5.0], [-17.0, 21.0, -16.0, -4.0, -2.0]]], [[[-30.0, 2.0, 22.0, 15.0, 14.0], [-45.0, 28.0, 15.0, 60.0, -31.0], [3.0, -50.0, -3.0, -6.0, 2.0], [44.0, -56.0, 76.0, 21.0, 88.0]], [[29.0, -37.0, 43.0, -57.0, -47.0], [13.0, -74.0, -23.0, 32.0, -5.0], [-19.0, 36.0, 33.0, -27.0, -72.0], [-7.0, -36.0, 13.0, 59.0, -24.0]], [[44.0, -7.0, 15.0, -39.0, -35.0], [-18.0, 10.0, -14.0, -18.0, 5.0], [-30.0, -47.0, 4.0, -31.0, 9.0], [16.0, 35.0, -11.0, -14.0, 15.0]], [[21.0, 22.0, 43.0, 3.0, -33.0], [13.0, -52.0, -62.0, 0.0, -23.0], [8.0, 59.0, -7.0, -47.0, -51.0], [-35.0, -18.0, 8.0, 72.0, -27.0]], [[-8.0, 23.0, 27.0, 15.0, -11.0], [23.0, -54.0, -51.0, 5.0, -22.0], [24.0, 67.0, -2.0, -26.0, -54.0], [-43.0, -19.0, -1.0, 67.0, -40.0]]], [[[-31.0, -9.0, 15.0, -18.0, -39.0], [62.0, -2.0, -38.0, 22.0, -6.0], [44.0, 18.0, -2.0, -33.0, -20.0], [-39.0, 72.0, -61.0, 86.0, 54.0]], [[-75.0, 21.0, 28.0, 39.0, 22.0], [8.0, -33.0, -27.0, 69.0, -47.0], [53.0, 39.0, 2.0, -8.0, -59.0], [-20.0, -60.0, 45.0, 93.0, 30.0]], [[20.0, 19.0, -12.0, 24.0, 18.0], [-16.0, 2.0, 5.0, -32.0, 6.0], [-13.0, 15.0, -12.0, 12.0, 17.0], [0.0, -18.0, 11.0, -39.0, -43.0]], [[12.0, -17.0, 10.0, 30.0, 0.0], [-17.0, 43.0, 42.0, 38.0, 1.0], [-11.0, 14.0, -30.0, 11.0, 43.0], [42.0, -69.0, 75.0, 8.0, 88.0]], [[130.0, -36.0, 55.0, -42.0, -73.0], [-20.0, -48.0, -7.0, -11.0, 17.0], [-78.0, 64.0, -7.0, -38.0, -15.0], [25.0, -83.0, 66.0, 13.0, -23.0]]], [[[-43.0, 41.0, -42.0, 27.0, 32.0], [25.0, 25.0, -20.0, -50.0, 4.0], [39.0, -19.0, -8.0, 11.0, 22.0], [-41.0, 94.0, -83.0, -30.0, -39.0]], [[-125.0, -24.0, -50.0, -15.0, 62.0], [13.0, 23.0, 49.0, 54.0, -4.0], [50.0, -86.0, 48.0, 59.0, -11.0], [9.0, 46.0, -44.0, -21.0, 32.0]], [[-47.0, -11.0, 53.0, -24.0, -21.0], [3.0, -51.0, -37.0, 84.0, -45.0], [28.0, 1.0, 28.0, -36.0, -84.0], [-5.0, -41.0, 37.0, 109.0, 49.0]], [[-43.0, 42.0, 22.0, 75.0, 42.0], [-57.0, 37.0, 5.0, 54.0, -48.0], [20.0, -16.0, -32.0, -3.0, 13.0], [35.0, -94.0, 110.0, 27.0, 82.0]], [[24.0, -43.0, -51.0, -15.0, 3.0], [36.0, 32.0, 62.0, -36.0, 57.0], [-18.0, 15.0, -2.0, 51.0, 59.0], [6.0, 39.0, -51.0, -70.0, -17.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[4.0, 7.0, -7.0, -7.0, 7.0], [-2.0, -2.0, -7.0, 0.0, -3.0], [0.0, -5.0, 3.0, -7.0, 7.0], [3.0, 7.0, -4.0, -4.0, 2.0]])
      s1=numpy.array(7.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[28.0, 49.0, -49.0, -49.0, 49.0], [-14.0, -14.0, -49.0, 0.0, -21.0], [0.0, -35.0, 21.0, -49.0, 49.0], [21.0, 49.0, -28.0, -28.0, 14.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, 5.0, -6.0], [-4.0, -2.0, 3.0], [1.0, -5.0, -7.0], [0.0, 3.0, -1.0], [-7.0, -7.0, 6.0]], [[0.0, 4.0, -2.0], [5.0, 3.0, 1.0], [-6.0, 0.0, -5.0], [6.0, 6.0, 5.0], [4.0, -2.0, 7.0]], [[-2.0, 5.0, 6.0], [-5.0, -1.0, 5.0], [1.0, 1.0, -1.0], [0.0, 0.0, 4.0], [7.0, 1.0, 3.0]], [[-5.0, -1.0, -3.0], [0.0, -2.0, 5.0], [-6.0, -7.0, -7.0], [0.0, 0.0, 2.0], [3.0, -7.0, 0.0]]])
      s1=numpy.array([-6.0, 0.0, -6.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[6.0, 6.0, 36.0, 6.0, 6.0], [12.0, -36.0, 66.0, -66.0, -66.0], [-24.0, 0.0, 0.0, -24.0, -60.0], [48.0, -30.0, 78.0, -12.0, -18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, 3.0, -3.0], [3.0, 2.0, 4.0]], [[-6.0, 1.0, -5.0], [-3.0, -1.0, 3.0]], [[-3.0, -7.0, -2.0], [0.0, -5.0, 5.0]], [[0.0, 5.0, 0.0], [2.0, 6.0, 3.0]], [[1.0, -4.0, 6.0], [-2.0, 5.0, 2.0]]], [[[0.0, 2.0, -3.0], [-6.0, -3.0, -6.0]], [[3.0, -3.0, -6.0], [0.0, 5.0, 5.0]], [[0.0, 1.0, 7.0], [-1.0, 4.0, -7.0]], [[2.0, 2.0, -7.0], [1.0, 0.0, -5.0]], [[4.0, -3.0, -5.0], [7.0, 0.0, -6.0]]], [[[-5.0, 2.0, -6.0], [-7.0, -1.0, 2.0]], [[7.0, -2.0, 3.0], [-5.0, 7.0, 1.0]], [[0.0, 1.0, 3.0], [-4.0, 3.0, -6.0]], [[6.0, 7.0, 5.0], [-3.0, 0.0, -1.0]], [[-2.0, 0.0, 0.0], [2.0, -3.0, 3.0]]], [[[-1.0, -7.0, -7.0], [-6.0, 7.0, 6.0]], [[0.0, -2.0, -4.0], [4.0, 4.0, -4.0]], [[4.0, 0.0, -1.0], [-6.0, 5.0, -3.0]], [[-1.0, 2.0, -2.0], [-6.0, 6.0, -7.0]], [[1.0, -4.0, -1.0], [3.0, 2.0, -2.0]]]])
      s1=numpy.array([[-1.0, 2.0, 2.0], [-4.0, 0.0, 6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[17.0, 28.0, 15.0, 20.0, 23.0], [-14.0, 9.0, -22.0, -46.0, -84.0], [37.0, 21.0, -12.0, 24.0, 12.0], [33.0, -52.0, 0.0, -17.0, -35.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[0.0, -7.0, -1.0, 2.0, 0.0], [0.0, -7.0, -5.0, 7.0, -2.0], [-2.0, 4.0, -6.0, -3.0, -5.0], [3.0, -6.0, 1.0, -5.0, 3.0]])
      s1=numpy.array([-4.0, 2.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.0, 0.0], [28.0, -14.0], [4.0, -2.0], [-8.0, 4.0], [0.0, 0.0]], [[0.0, 0.0], [28.0, -14.0], [20.0, -10.0], [-28.0, 14.0], [8.0, -4.0]], [[8.0, -4.0], [-16.0, 8.0], [24.0, -12.0], [12.0, -6.0], [20.0, -10.0]], [[-12.0, 6.0], [24.0, -12.0], [-4.0, 2.0], [20.0, -10.0], [-12.0, 6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, 1.0, -5.0], [4.0, 6.0, 3.0], [-7.0, 3.0, 0.0], [1.0, 1.0, 5.0], [7.0, 5.0, -2.0]], [[0.0, -3.0, -7.0], [-1.0, -1.0, -1.0], [5.0, -5.0, 2.0], [1.0, 7.0, 0.0], [7.0, 1.0, 1.0]], [[-1.0, -5.0, -5.0], [-4.0, -5.0, -3.0], [3.0, -2.0, -5.0], [6.0, -7.0, -1.0], [-1.0, 2.0, -3.0]], [[0.0, 4.0, -3.0], [-2.0, 1.0, 2.0], [-2.0, 0.0, -6.0], [5.0, 4.0, 0.0], [-7.0, -4.0, 1.0]]])
      s1=numpy.array([[-3.0, -1.0], [2.0, -4.0], [-5.0, -4.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[12.0, 11.0], [-15.0, -40.0], [27.0, -5.0], [-26.0, -25.0], [-1.0, -19.0]], [[29.0, 40.0], [6.0, 9.0], [-35.0, 7.0], [11.0, -29.0], [-24.0, -15.0]], [[18.0, 41.0], [17.0, 36.0], [12.0, 25.0], [-27.0, 26.0], [22.0, 5.0]], [[23.0, -4.0], [-2.0, -10.0], [36.0, 26.0], [-7.0, -21.0], [8.0, 19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-6.0, 3.0, 6.0], [4.0, 7.0, -5.0]], [[6.0, -3.0, 6.0], [-2.0, 1.0, -1.0]], [[-7.0, 5.0, -1.0], [5.0, 5.0, 6.0]], [[3.0, 1.0, -4.0], [-1.0, 0.0, 7.0]], [[2.0, -2.0, -4.0], [0.0, -4.0, -1.0]]], [[[-7.0, 0.0, 0.0], [2.0, 2.0, -3.0]], [[0.0, 6.0, -5.0], [0.0, 0.0, 0.0]], [[-3.0, -1.0, -7.0], [7.0, 7.0, 6.0]], [[1.0, -6.0, -4.0], [-2.0, -4.0, 1.0]], [[0.0, -7.0, 7.0], [5.0, -3.0, -6.0]]], [[[1.0, 0.0, 2.0], [5.0, 6.0, -5.0]], [[1.0, 6.0, 2.0], [-2.0, -5.0, 7.0]], [[3.0, 1.0, 2.0], [4.0, -3.0, -5.0]], [[-2.0, -3.0, -1.0], [5.0, 6.0, -4.0]], [[-5.0, -1.0, 1.0], [1.0, -1.0, -3.0]]], [[[1.0, 0.0, 7.0], [-2.0, 2.0, -6.0]], [[2.0, 3.0, -4.0], [-1.0, -5.0, 0.0]], [[-7.0, 5.0, 6.0], [5.0, -2.0, 5.0]], [[6.0, -7.0, 1.0], [-2.0, 0.0, -3.0]], [[0.0, 5.0, 5.0], [1.0, -1.0, 5.0]]]])
      s1=numpy.array([[[-3.0, -1.0], [-5.0, -7.0], [6.0, -5.0]], [[4.0, 6.0], [5.0, 5.0], [2.0, -2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[80.0, 24.0], [28.0, -20.0], [47.0, 20.0], [-28.0, -10.0], [-42.0, 14.0]], [[33.0, 35.0], [-60.0, -17.0], [47.0, 110.0], [-23.0, 27.0], [70.0, 41.0]], [[49.0, 59.0], [-40.0, -104.0], [-11.0, -1.0], [57.0, 96.0], [19.0, 14.0]], [[29.0, -26.0], [-74.0, -34.0], [52.0, -48.0], [9.0, 32.0], [14.0, -69.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-4.0, -2.0, -2.0, -5.0, -5.0], [2.0, -2.0, -5.0, -7.0, 1.0], [-1.0, 1.0, -2.0, -6.0, 3.0], [0.0, 0.0, 0.0, 3.0, 2.0]])
      s1=numpy.array([[0.0, 5.0, 3.0, -5.0, -3.0], [-2.0, -4.0, 6.0, -7.0, -1.0], [4.0, 0.0, 0.0, 1.0, 3.0], [-1.0, -4.0, -3.0, -2.0, 0.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0, -20.0, -12.0, 20.0, 12.0], [8.0, 16.0, -24.0, 28.0, 4.0], [-16.0, 0.0, 0.0, -4.0, -12.0], [4.0, 16.0, 12.0, 8.0, 0.0]], [[0.0, -10.0, -6.0, 10.0, 6.0], [4.0, 8.0, -12.0, 14.0, 2.0], [-8.0, 0.0, 0.0, -2.0, -6.0], [2.0, 8.0, 6.0, 4.0, 0.0]], [[0.0, -10.0, -6.0, 10.0, 6.0], [4.0, 8.0, -12.0, 14.0, 2.0], [-8.0, 0.0, 0.0, -2.0, -6.0], [2.0, 8.0, 6.0, 4.0, 0.0]], [[0.0, -25.0, -15.0, 25.0, 15.0], [10.0, 20.0, -30.0, 35.0, 5.0], [-20.0, 0.0, 0.0, -5.0, -15.0], [5.0, 20.0, 15.0, 10.0, 0.0]], [[0.0, -25.0, -15.0, 25.0, 15.0], [10.0, 20.0, -30.0, 35.0, 5.0], [-20.0, 0.0, 0.0, -5.0, -15.0], [5.0, 20.0, 15.0, 10.0, 0.0]]], [[[0.0, 10.0, 6.0, -10.0, -6.0], [-4.0, -8.0, 12.0, -14.0, -2.0], [8.0, 0.0, 0.0, 2.0, 6.0], [-2.0, -8.0, -6.0, -4.0, 0.0]], [[0.0, -10.0, -6.0, 10.0, 6.0], [4.0, 8.0, -12.0, 14.0, 2.0], [-8.0, 0.0, 0.0, -2.0, -6.0], [2.0, 8.0, 6.0, 4.0, 0.0]], [[0.0, -25.0, -15.0, 25.0, 15.0], [10.0, 20.0, -30.0, 35.0, 5.0], [-20.0, 0.0, 0.0, -5.0, -15.0], [5.0, 20.0, 15.0, 10.0, 0.0]], [[0.0, -35.0, -21.0, 35.0, 21.0], [14.0, 28.0, -42.0, 49.0, 7.0], [-28.0, 0.0, 0.0, -7.0, -21.0], [7.0, 28.0, 21.0, 14.0, 0.0]], [[0.0, 5.0, 3.0, -5.0, -3.0], [-2.0, -4.0, 6.0, -7.0, -1.0], [4.0, 0.0, 0.0, 1.0, 3.0], [-1.0, -4.0, -3.0, -2.0, 0.0]]], [[[0.0, -5.0, -3.0, 5.0, 3.0], [2.0, 4.0, -6.0, 7.0, 1.0], [-4.0, 0.0, 0.0, -1.0, -3.0], [1.0, 4.0, 3.0, 2.0, 0.0]], [[0.0, 5.0, 3.0, -5.0, -3.0], [-2.0, -4.0, 6.0, -7.0, -1.0], [4.0, 0.0, 0.0, 1.0, 3.0], [-1.0, -4.0, -3.0, -2.0, 0.0]], [[0.0, -10.0, -6.0, 10.0, 6.0], [4.0, 8.0, -12.0, 14.0, 2.0], [-8.0, 0.0, 0.0, -2.0, -6.0], [2.0, 8.0, 6.0, 4.0, 0.0]], [[0.0, -30.0, -18.0, 30.0, 18.0], [12.0, 24.0, -36.0, 42.0, 6.0], [-24.0, 0.0, 0.0, -6.0, -18.0], [6.0, 24.0, 18.0, 12.0, 0.0]], [[0.0, 15.0, 9.0, -15.0, -9.0], [-6.0, -12.0, 18.0, -21.0, -3.0], [12.0, 0.0, 0.0, 3.0, 9.0], [-3.0, -12.0, -9.0, -6.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 15.0, 9.0, -15.0, -9.0], [-6.0, -12.0, 18.0, -21.0, -3.0], [12.0, 0.0, 0.0, 3.0, 9.0], [-3.0, -12.0, -9.0, -6.0, 0.0]], [[0.0, 10.0, 6.0, -10.0, -6.0], [-4.0, -8.0, 12.0, -14.0, -2.0], [8.0, 0.0, 0.0, 2.0, 6.0], [-2.0, -8.0, -6.0, -4.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-5.0, 1.0, -3.0], [5.0, 3.0, 3.0], [0.0, 3.0, 0.0], [-2.0, -5.0, 5.0], [4.0, -5.0, 3.0]], [[-3.0, 6.0, 2.0], [4.0, 7.0, 2.0], [3.0, 0.0, -2.0], [-6.0, 6.0, -5.0], [3.0, 7.0, 5.0]], [[0.0, 2.0, -5.0], [-4.0, 0.0, 6.0], [-5.0, 2.0, 1.0], [-1.0, 0.0, 4.0], [-6.0, 0.0, 6.0]], [[-4.0, -7.0, 1.0], [0.0, 2.0, -5.0], [-4.0, -4.0, 1.0], [6.0, -1.0, -2.0], [-1.0, -4.0, 0.0]]])
      s1=numpy.array([[[2.0, -4.0, 0.0, 0.0, -2.0], [-5.0, -3.0, 6.0, 0.0, -7.0], [3.0, -2.0, -5.0, -4.0, -4.0], [7.0, -2.0, 3.0, 0.0, -3.0]], [[-3.0, 3.0, -3.0, 5.0, 2.0], [-2.0, -3.0, 4.0, 7.0, 1.0], [-4.0, -4.0, 2.0, -5.0, 7.0], [5.0, 1.0, 0.0, 7.0, -4.0]], [[-6.0, 3.0, -1.0, 5.0, 5.0], [2.0, -5.0, 7.0, -2.0, 4.0], [-1.0, 7.0, 0.0, 1.0, 0.0], [-7.0, -7.0, -1.0, 3.0, 7.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[5.0, 14.0, 0.0, -10.0, -3.0], [17.0, 27.0, -47.0, 13.0, 24.0], [-16.0, -15.0, 27.0, 12.0, 27.0], [-9.0, 32.0, -12.0, -2.0, -10.0]], [[-17.0, -2.0, -12.0, 30.0, 11.0], [-25.0, -39.0, 63.0, 15.0, -20.0], [0.0, -1.0, -19.0, -32.0, 1.0], [29.0, -28.0, 12.0, 30.0, -6.0]], [[-9.0, 9.0, -9.0, 15.0, 6.0], [-6.0, -9.0, 12.0, 21.0, 3.0], [-12.0, -12.0, 6.0, -15.0, 21.0], [15.0, 3.0, 0.0, 21.0, -12.0]], [[-19.0, 8.0, 10.0, 0.0, 19.0], [30.0, -4.0, 3.0, -45.0, 29.0], [9.0, 59.0, 0.0, 38.0, -27.0], [-74.0, -36.0, -11.0, -20.0, 61.0]], [[5.0, -22.0, 12.0, -10.0, -3.0], [-4.0, -12.0, 25.0, -41.0, -21.0], [29.0, 33.0, -30.0, 12.0, -51.0], [-18.0, -34.0, 9.0, -26.0, 29.0]]], [[[-36.0, 36.0, -20.0, 40.0, 28.0], [7.0, -19.0, 20.0, 38.0, 35.0], [-35.0, -4.0, 27.0, -16.0, 54.0], [-5.0, -2.0, -11.0, 48.0, -1.0]], [[-25.0, 11.0, -23.0, 45.0, 16.0], [-30.0, -43.0, 66.0, 45.0, -13.0], [-18.0, -22.0, -6.0, -49.0, 33.0], [49.0, -15.0, 10.0, 55.0, -26.0]], [[18.0, -18.0, 2.0, -10.0, -16.0], [-19.0, 1.0, 4.0, 4.0, -29.0], [11.0, -20.0, -15.0, -14.0, -12.0], [35.0, 8.0, 11.0, -6.0, -23.0]], [[0.0, 27.0, -13.0, 5.0, -1.0], [8.0, 25.0, -47.0, 52.0, 28.0], [-37.0, -47.0, 42.0, -11.0, 66.0], [23.0, 53.0, -13.0, 27.0, -41.0]], [[-45.0, 24.0, -26.0, 60.0, 33.0], [-19.0, -55.0, 81.0, 39.0, 6.0], [-24.0, 1.0, -1.0, -42.0, 37.0], [21.0, -34.0, 4.0, 64.0, -2.0]]], [[[24.0, -9.0, -1.0, -15.0, -21.0], [-14.0, 19.0, -27.0, 24.0, -18.0], [-3.0, -43.0, 4.0, -15.0, 14.0], [45.0, 37.0, 5.0, -1.0, -43.0]], [[-44.0, 34.0, -6.0, 30.0, 38.0], [32.0, -18.0, 18.0, -12.0, 52.0], [-18.0, 50.0, 20.0, 22.0, 16.0], [-70.0, -34.0, -18.0, 18.0, 54.0]], [[-22.0, 29.0, -7.0, 15.0, 19.0], [23.0, 4.0, -15.0, 12.0, 41.0], [-24.0, 9.0, 29.0, 11.0, 34.0], [-32.0, 5.0, -16.0, 17.0, 14.0]], [[-26.0, 16.0, -4.0, 20.0, 22.0], [13.0, -17.0, 22.0, -8.0, 23.0], [-7.0, 30.0, 5.0, 8.0, 4.0], [-35.0, -26.0, -7.0, 12.0, 31.0]], [[-48.0, 42.0, -6.0, 30.0, 42.0], [42.0, -12.0, 6.0, -12.0, 66.0], [-24.0, 54.0, 30.0, 30.0, 24.0], [-84.0, -30.0, -24.0, 18.0, 60.0]]], [[[7.0, -2.0, 20.0, -30.0, -1.0], [36.0, 28.0, -45.0, -51.0, 25.0], [15.0, 43.0, 6.0, 52.0, -33.0], [-70.0, -6.0, -13.0, -46.0, 47.0]], [[24.0, -9.0, -1.0, -15.0, -21.0], [-14.0, 19.0, -27.0, 24.0, -18.0], [-3.0, -43.0, 4.0, -15.0, 14.0], [45.0, 37.0, 5.0, -1.0, -43.0]], [[-2.0, 7.0, 11.0, -15.0, 5.0], [30.0, 19.0, -33.0, -30.0, 28.0], [3.0, 31.0, 12.0, 37.0, -12.0], [-55.0, -3.0, -13.0, -25.0, 35.0]], [[27.0, -33.0, 5.0, -15.0, -24.0], [-32.0, -5.0, 18.0, -3.0, -51.0], [24.0, -22.0, -32.0, -21.0, -31.0], [51.0, 1.0, 20.0, -13.0, -28.0]], [[10.0, -8.0, 12.0, -20.0, -6.0], [13.0, 15.0, -22.0, -28.0, 3.0], [13.0, 18.0, -3.0, 24.0, -24.0], [-27.0, -2.0, -3.0, -28.0, 19.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[3.0, -7.0, -5.0], [2.0, 2.0, 3.0]], [[-7.0, 6.0, 0.0], [6.0, 0.0, 0.0]], [[-6.0, 0.0, 1.0], [0.0, 7.0, -6.0]], [[-1.0, 5.0, 0.0], [0.0, 7.0, -6.0]], [[0.0, 4.0, -3.0], [3.0, -4.0, -7.0]]], [[[0.0, -2.0, 2.0], [-3.0, 3.0, 0.0]], [[-5.0, 6.0, 7.0], [-2.0, 7.0, -5.0]], [[6.0, -7.0, 1.0], [-7.0, 2.0, -5.0]], [[-5.0, 3.0, 1.0], [-5.0, -4.0, -7.0]], [[7.0, 3.0, -5.0], [-1.0, -2.0, -3.0]]], [[[3.0, 0.0, 6.0], [-6.0, 1.0, 6.0]], [[1.0, -7.0, -6.0], [3.0, -4.0, 3.0]], [[7.0, -6.0, 0.0], [2.0, 6.0, -1.0]], [[1.0, 5.0, 3.0], [-4.0, -2.0, 3.0]], [[-7.0, -7.0, -2.0], [3.0, -5.0, -2.0]]], [[[0.0, -2.0, -1.0], [-5.0, 3.0, -5.0]], [[-4.0, -4.0, 1.0], [-1.0, 6.0, 3.0]], [[5.0, -3.0, 7.0], [3.0, -7.0, 2.0]], [[6.0, -2.0, 7.0], [4.0, 0.0, -5.0]], [[7.0, 4.0, -1.0], [-1.0, -1.0, 0.0]]]])
      s1=numpy.array([[[[5.0, 4.0, 0.0, -1.0, -3.0], [3.0, 0.0, 2.0, 4.0, -7.0], [-2.0, -2.0, -3.0, -6.0, 4.0], [-4.0, -3.0, 6.0, 3.0, 2.0]], [[-2.0, -1.0, 0.0, 2.0, -7.0], [-1.0, 0.0, 3.0, -2.0, 5.0], [-2.0, -2.0, 0.0, 6.0, -7.0], [5.0, -6.0, -4.0, -7.0, 5.0]], [[-3.0, 5.0, 7.0, -1.0, 2.0], [4.0, -3.0, 0.0, -7.0, 0.0], [0.0, 3.0, 0.0, 3.0, 6.0], [0.0, -1.0, -7.0, 1.0, -7.0]]], [[[-1.0, 0.0, -7.0, -3.0, 4.0], [3.0, -7.0, -2.0, -6.0, 4.0], [6.0, 7.0, 0.0, 3.0, -2.0], [-1.0, 3.0, -5.0, -1.0, 5.0]], [[-4.0, -5.0, -4.0, -2.0, -5.0], [-3.0, 0.0, 0.0, 2.0, 3.0], [7.0, 3.0, -3.0, -4.0, -7.0], [-3.0, -6.0, 7.0, -7.0, -2.0]], [[1.0, 4.0, -1.0, 2.0, 0.0], [4.0, -4.0, -6.0, -7.0, -4.0], [-1.0, -3.0, -3.0, 6.0, -1.0], [-5.0, 7.0, -5.0, 4.0, 5.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[37.0, -4.0, -60.0, -16.0, 28.0], [8.0, -11.0, -37.0, 32.0, -54.0], [31.0, 4.0, -24.0, -59.0, 10.0], [-70.0, 53.0, 70.0, 49.0, 27.0]], [[-53.0, -34.0, -42.0, 1.0, 3.0], [-9.0, -42.0, -8.0, -76.0, 103.0], [38.0, 44.0, 21.0, 96.0, -82.0], [52.0, 3.0, -96.0, -69.0, 46.0]], [[-67.0, -78.0, -15.0, -21.0, -15.0], [-59.0, 21.0, 24.0, 25.0, 87.0], [67.0, 54.0, 15.0, -25.0, -61.0], [33.0, -67.0, 36.0, -90.0, -63.0]], [[-49.0, -68.0, -22.0, -15.0, -67.0], [-53.0, 24.0, 49.0, 42.0, 77.0], [47.0, 31.0, 0.0, -28.0, -82.0], [38.0, -111.0, 53.0, -111.0, -21.0]], [[7.0, -27.0, -19.0, -4.0, -2.0], [-23.0, 16.0, 48.0, 36.0, 48.0], [-11.0, 13.0, 33.0, -2.0, -17.0], [64.0, -37.0, -3.0, -34.0, 29.0]]], [[[-11.0, -3.0, 23.0, -3.0, -9.0], [-8.0, 15.0, 0.0, 14.0, -13.0], [7.0, -2.0, -9.0, -27.0, 11.0], [-16.0, -17.0, 30.0, -2.0, -45.0]], [[-89.0, -46.0, 40.0, -8.0, -56.0], [-40.0, 13.0, 42.0, -20.0, 98.0], [40.0, 41.0, 9.0, 23.0, -60.0], [56.0, -111.0, -19.0, -117.0, -78.0]], [[35.0, 6.0, 53.0, -14.0, -5.0], [-18.0, 66.0, 35.0, 112.0, -79.0], [-21.0, -23.0, -9.0, -134.0, 84.0], [-33.0, -45.0, 131.0, 41.0, -94.0]], [[-20.0, -26.0, 65.0, 19.0, -4.0], [-45.0, 60.0, 51.0, 38.0, 46.0], [-47.0, -19.0, 48.0, 10.0, 10.0], [87.0, -44.0, -17.0, -30.0, -54.0]], [[50.0, -2.0, -17.0, 5.0, -46.0], [-11.0, 34.0, 43.0, 80.0, -32.0], [-37.0, -39.0, -6.0, -52.0, -4.0], [9.0, -46.0, 71.0, -2.0, 48.0]]], [[[5.0, 61.0, 74.0, 19.0, -26.0], [36.0, 0.0, -18.0, -34.0, -66.0], [-41.0, -45.0, -30.0, 14.0, 47.0], [-39.0, 3.0, -17.0, 38.0, -38.0]], [[53.0, 13.0, -50.0, -4.0, 66.0], [19.0, -15.0, -43.0, 13.0, -54.0], [-1.0, -6.0, 0.0, -23.0, 36.0], [-45.0, 99.0, 18.0, 83.0, 47.0]], [[20.0, 0.0, -37.0, -39.0, -1.0], [11.0, -10.0, -2.0, 47.0, -49.0], [53.0, 33.0, -36.0, -102.0, 25.0], [-73.0, -22.0, 103.0, 15.0, -23.0]], [[1.0, 36.0, 54.0, 28.0, -38.0], [16.0, 7.0, 7.0, -28.0, -16.0], [-53.0, -46.0, -6.0, 47.0, 6.0], [16.0, -15.0, -44.0, 1.0, 5.0]], [[0.0, -14.0, -13.0, -8.0, 103.0], [-6.0, -7.0, -29.0, -14.0, 19.0], [13.0, 34.0, 42.0, 11.0, 40.0], [15.0, 90.0, -40.0, 50.0, -20.0]]], [[[-5.0, -38.0, 21.0, -4.0, -23.0], [-46.0, 58.0, 34.0, 82.0, -1.0], [0.0, -10.0, 6.0, -72.0, 2.0], [11.0, -55.0, 86.0, -23.0, -59.0]], [[-35.0, -25.0, -13.0, -8.0, 8.0], [-13.0, -8.0, -36.0, -18.0, 10.0], [49.0, 21.0, -15.0, -6.0, -25.0], [-36.0, 17.0, 17.0, -12.0, -37.0]], [[37.0, 101.0, 54.0, -9.0, 67.0], [84.0, -50.0, -17.0, -69.0, -67.0], [-37.0, 11.0, 0.0, 22.0, 124.0], [-27.0, 61.0, -81.0, 97.0, -15.0]], [[4.0, 41.0, 26.0, -39.0, 26.0], [40.0, -29.0, 28.0, -10.0, -16.0], [21.0, 56.0, -3.0, -45.0, 77.0], [-13.0, -36.0, 0.0, 15.0, -52.0]], [[35.0, 24.0, 4.0, 7.0, -50.0], [13.0, 10.0, 28.0, 31.0, -36.0], [-35.0, -35.0, -18.0, -20.0, 3.0], [-4.0, -41.0, 31.0, 0.0, 38.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-4.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-5.0, 0.0, -5.0, -6.0, 0.0], [-4.0, 1.0, -1.0, -4.0, 7.0], [3.0, -3.0, 6.0, -3.0, 3.0], [0.0, -5.0, -2.0, 6.0, 3.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[20.0, -0.0, 20.0, 24.0, -0.0], [16.0, -4.0, 4.0, 16.0, -28.0], [-12.0, 12.0, -24.0, 12.0, -12.0], [-0.0, 20.0, 8.0, -24.0, -12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([-5.0, -4.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-2.0, -1.0, 4.0], [1.0, -1.0, 2.0], [-5.0, 7.0, -6.0], [-3.0, 0.0, -4.0], [-4.0, -7.0, 4.0]], [[-7.0, -6.0, -4.0], [0.0, 0.0, -6.0], [5.0, 5.0, -7.0], [7.0, 4.0, 4.0], [5.0, 3.0, -5.0]], [[7.0, 6.0, 0.0], [6.0, 4.0, -4.0], [0.0, 1.0, 7.0], [6.0, 1.0, -7.0], [-5.0, 3.0, 5.0]], [[5.0, 4.0, 6.0], [-5.0, 4.0, 0.0], [2.0, -7.0, -2.0], [4.0, 6.0, -6.0], [0.0, -4.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-10.0, -13.0, 33.0, 39.0, 24.0], [83.0, 36.0, -3.0, -75.0, -7.0], [-59.0, -22.0, -46.0, 8.0, -17.0], [-77.0, 9.0, 30.0, -8.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[0.0, 1.0, -6.0], [3.0, -5.0, 6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[5.0, 7.0, 2.0], [-4.0, 1.0, 0.0]], [[0.0, -4.0, 4.0], [7.0, -2.0, -4.0]], [[1.0, 5.0, -6.0], [6.0, 0.0, 2.0]], [[-7.0, 0.0, -6.0], [-5.0, -1.0, -2.0]], [[-1.0, -6.0, 2.0], [0.0, 5.0, 6.0]]], [[[1.0, -1.0, 0.0], [4.0, 6.0, -2.0]], [[0.0, -3.0, -6.0], [-6.0, -4.0, -6.0]], [[0.0, 7.0, 0.0], [-2.0, -7.0, 3.0]], [[-2.0, -1.0, 0.0], [6.0, 6.0, 2.0]], [[2.0, -5.0, -3.0], [0.0, 0.0, -7.0]]], [[[2.0, 0.0, 7.0], [5.0, 6.0, 0.0]], [[3.0, -5.0, -4.0], [5.0, 1.0, 0.0]], [[7.0, 4.0, 5.0], [-2.0, -6.0, 0.0]], [[3.0, 5.0, -1.0], [3.0, -5.0, 0.0]], [[-6.0, -3.0, 0.0], [7.0, -4.0, 5.0]]], [[[5.0, -5.0, 3.0], [-7.0, 4.0, -6.0]], [[3.0, -2.0, 2.0], [7.0, 0.0, -1.0]], [[4.0, 7.0, 4.0], [-7.0, -2.0, -7.0]], [[-3.0, -6.0, -4.0], [-7.0, -2.0, 2.0]], [[-5.0, -5.0, -1.0], [7.0, -6.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-22.0, -21.0, 71.0, 14.0, -7.0], [-31.0, -1.0, 54.0, -1.0, -29.0], [-57.0, 29.0, -2.0, 45.0, 68.0], [-100.0, 1.0, -70.0, 19.0, 76.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([-2.0, 6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-2.0, 6.0, 0.0, 1.0, -1.0], [6.0, -3.0, -6.0, -7.0, 3.0], [7.0, 2.0, 1.0, 4.0, -6.0], [-2.0, -6.0, -1.0, 5.0, 4.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[4.0, -12.0], [-12.0, 36.0], [0.0, 0.0], [-2.0, 6.0], [2.0, -6.0]], [[-12.0, 36.0], [6.0, -18.0], [12.0, -36.0], [14.0, -42.0], [-6.0, 18.0]], [[-14.0, 42.0], [-4.0, 12.0], [-2.0, 6.0], [-8.0, 24.0], [12.0, -36.0]], [[4.0, -12.0], [12.0, -36.0], [2.0, -6.0], [-10.0, 30.0], [-8.0, 24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([[-1.0, -4.0], [-5.0, 4.0], [-6.0, -6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, -7.0, -6.0], [6.0, 7.0, 2.0], [4.0, 5.0, -4.0], [7.0, -6.0, -2.0], [-1.0, 7.0, -4.0]], [[6.0, 5.0, -5.0], [7.0, -2.0, 1.0], [-5.0, 0.0, -3.0], [-3.0, 5.0, 0.0], [-3.0, -1.0, -4.0]], [[2.0, -1.0, -2.0], [7.0, 0.0, -2.0], [-7.0, -2.0, -2.0], [1.0, -5.0, -7.0], [7.0, 7.0, 7.0]], [[7.0, 0.0, -4.0], [0.0, -5.0, 1.0], [-4.0, 7.0, 3.0], [3.0, 1.0, 3.0], [0.0, 0.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[69.0, 0.0], [-53.0, -8.0], [-5.0, 28.0], [35.0, -40.0], [-10.0, 56.0]], [[-1.0, 26.0], [-3.0, -42.0], [23.0, 38.0], [-22.0, 32.0], [32.0, 32.0]], [[15.0, 0.0], [5.0, -16.0], [29.0, 32.0], [66.0, 18.0], [-84.0, -42.0]], [[17.0, -4.0], [19.0, -26.0], [-49.0, 26.0], [-26.0, -26.0], [0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[-7.0, -4.0], [6.0, -6.0], [7.0, 1.0]], [[5.0, 5.0], [4.0, 0.0], [-7.0, 2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[1.0, -4.0, 4.0], [3.0, 2.0, 0.0]], [[7.0, -1.0, -5.0], [1.0, 6.0, -6.0]], [[-7.0, -3.0, -6.0], [7.0, 0.0, -4.0]], [[-7.0, -7.0, -6.0], [0.0, 6.0, -2.0]], [[0.0, -2.0, -1.0], [-3.0, -7.0, -2.0]]], [[[-6.0, -5.0, -6.0], [1.0, -1.0, -3.0]], [[-7.0, -6.0, -6.0], [3.0, -7.0, 0.0]], [[-6.0, -2.0, -3.0], [-3.0, 1.0, -4.0]], [[6.0, 5.0, 7.0], [0.0, -2.0, -3.0]], [[-1.0, -7.0, 5.0], [-6.0, 7.0, 3.0]]], [[[-5.0, 7.0, -2.0], [4.0, 3.0, -1.0]], [[-6.0, 4.0, 7.0], [-2.0, -2.0, -7.0]], [[4.0, -7.0, 6.0], [5.0, -3.0, 0.0]], [[0.0, -6.0, -5.0], [-2.0, -6.0, -2.0]], [[4.0, -5.0, -4.0], [4.0, -6.0, -1.0]]], [[[5.0, -4.0, -5.0], [-7.0, -4.0, -1.0]], [[1.0, 7.0, 2.0], [-4.0, 2.0, 1.0]], [[3.0, 6.0, 6.0], [-4.0, 5.0, -1.0]], [[0.0, 6.0, -6.0], [4.0, 0.0, 7.0]], [[-4.0, -6.0, 0.0], [-5.0, 7.0, 3.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[20.0, 39.0], [-19.0, -34.0], [52.0, 67.0], [3.0, 60.0], [-48.0, -8.0]], [[-8.0, 47.0], [-42.0, 73.0], [26.0, 10.0], [50.0, -53.0], [-23.0, 27.0]], [[102.0, -6.0], [146.0, -17.0], [-15.0, 57.0], [-91.0, 17.0], [-83.0, 28.0]], [[-138.0, -38.0], [30.0, -62.0], [64.0, -64.0], [-35.0, -8.0], [-26.0, 33.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_constData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[-4.0, -2.0, 0.0, 0.0, -1.0], [7.0, -6.0, 2.0, 0.0, 0.0], [0.0, -5.0, -2.0, 2.0, -7.0], [-6.0, -7.0, 4.0, 2.0, -7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[2.0, -1.0, 4.0, 2.0, 7.0], [2.0, -6.0, -1.0, -6.0, -4.0], [-6.0, 0.0, 0.0, 4.0, -7.0], [0.0, -1.0, 3.0, 0.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-8.0, -4.0, 0.0, 0.0, -2.0], [14.0, -12.0, 4.0, 0.0, 0.0], [0.0, -10.0, -4.0, 4.0, -14.0], [-12.0, -14.0, 8.0, 4.0, -14.0]], [[4.0, 2.0, 0.0, 0.0, 1.0], [-7.0, 6.0, -2.0, 0.0, 0.0], [0.0, 5.0, 2.0, -2.0, 7.0], [6.0, 7.0, -4.0, -2.0, 7.0]], [[-16.0, -8.0, 0.0, 0.0, -4.0], [28.0, -24.0, 8.0, 0.0, 0.0], [0.0, -20.0, -8.0, 8.0, -28.0], [-24.0, -28.0, 16.0, 8.0, -28.0]], [[-8.0, -4.0, 0.0, 0.0, -2.0], [14.0, -12.0, 4.0, 0.0, 0.0], [0.0, -10.0, -4.0, 4.0, -14.0], [-12.0, -14.0, 8.0, 4.0, -14.0]], [[-28.0, -14.0, 0.0, 0.0, -7.0], [49.0, -42.0, 14.0, 0.0, 0.0], [0.0, -35.0, -14.0, 14.0, -49.0], [-42.0, -49.0, 28.0, 14.0, -49.0]]], [[[-8.0, -4.0, 0.0, 0.0, -2.0], [14.0, -12.0, 4.0, 0.0, 0.0], [0.0, -10.0, -4.0, 4.0, -14.0], [-12.0, -14.0, 8.0, 4.0, -14.0]], [[24.0, 12.0, 0.0, 0.0, 6.0], [-42.0, 36.0, -12.0, 0.0, 0.0], [0.0, 30.0, 12.0, -12.0, 42.0], [36.0, 42.0, -24.0, -12.0, 42.0]], [[4.0, 2.0, 0.0, 0.0, 1.0], [-7.0, 6.0, -2.0, 0.0, 0.0], [0.0, 5.0, 2.0, -2.0, 7.0], [6.0, 7.0, -4.0, -2.0, 7.0]], [[24.0, 12.0, 0.0, 0.0, 6.0], [-42.0, 36.0, -12.0, 0.0, 0.0], [0.0, 30.0, 12.0, -12.0, 42.0], [36.0, 42.0, -24.0, -12.0, 42.0]], [[16.0, 8.0, 0.0, 0.0, 4.0], [-28.0, 24.0, -8.0, 0.0, 0.0], [0.0, 20.0, 8.0, -8.0, 28.0], [24.0, 28.0, -16.0, -8.0, 28.0]]], [[[24.0, 12.0, 0.0, 0.0, 6.0], [-42.0, 36.0, -12.0, 0.0, 0.0], [0.0, 30.0, 12.0, -12.0, 42.0], [36.0, 42.0, -24.0, -12.0, 42.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-16.0, -8.0, 0.0, 0.0, -4.0], [28.0, -24.0, 8.0, 0.0, 0.0], [0.0, -20.0, -8.0, 8.0, -28.0], [-24.0, -28.0, 16.0, 8.0, -28.0]], [[28.0, 14.0, 0.0, 0.0, 7.0], [-49.0, 42.0, -14.0, 0.0, 0.0], [0.0, 35.0, 14.0, -14.0, 49.0], [42.0, 49.0, -28.0, -14.0, 49.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[4.0, 2.0, 0.0, 0.0, 1.0], [-7.0, 6.0, -2.0, 0.0, 0.0], [0.0, 5.0, 2.0, -2.0, 7.0], [6.0, 7.0, -4.0, -2.0, 7.0]], [[-12.0, -6.0, 0.0, 0.0, -3.0], [21.0, -18.0, 6.0, 0.0, 0.0], [0.0, -15.0, -6.0, 6.0, -21.0], [-18.0, -21.0, 12.0, 6.0, -21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[24.0, 12.0, 0.0, 0.0, 6.0], [-42.0, 36.0, -12.0, 0.0, 0.0], [0.0, 30.0, 12.0, -12.0, 42.0], [36.0, 42.0, -24.0, -12.0, 42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([[[5.0, 6.0, -7.0, 4.0, 4.0], [7.0, 6.0, -1.0, -7.0, 1.0], [-6.0, 3.0, 3.0, -7.0, -3.0], [6.0, 7.0, -5.0, -6.0, -2.0]], [[-2.0, 0.0, 1.0, 0.0, -5.0], [-3.0, -3.0, -2.0, 4.0, 2.0], [-1.0, -4.0, 6.0, -5.0, 2.0], [-4.0, -2.0, -5.0, -3.0, -4.0]], [[-2.0, -3.0, -6.0, -5.0, 3.0], [-7.0, 4.0, -5.0, 1.0, -1.0], [-6.0, 5.0, -6.0, 6.0, 2.0], [3.0, -7.0, 4.0, 2.0, 3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-1.0, 3.0, -4.0], [6.0, -4.0, -7.0], [0.0, -1.0, 4.0], [-7.0, 7.0, 0.0], [-4.0, -4.0, 3.0]], [[7.0, 4.0, -7.0], [7.0, -7.0, 4.0], [5.0, 3.0, 4.0], [3.0, -6.0, 6.0], [-3.0, 5.0, 1.0]], [[-5.0, 3.0, -6.0], [-1.0, -3.0, 1.0], [-2.0, -2.0, -1.0], [-5.0, 0.0, -4.0], [6.0, -5.0, -1.0]], [[1.0, 5.0, -6.0], [6.0, -4.0, 4.0], [0.0, 3.0, 0.0], [6.0, 1.0, 2.0], [0.0, 6.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-3.0, 6.0, 34.0, 16.0, -31.0], [12.0, -31.0, 15.0, 15.0, 9.0], [27.0, -35.0, 39.0, -32.0, 1.0], [-30.0, 15.0, -26.0, -11.0, -22.0]], [[52.0, 57.0, -4.0, 59.0, 23.0], [103.0, 20.0, 37.0, -65.0, 5.0], [10.0, -1.0, 36.0, -64.0, -40.0], [31.0, 99.0, -38.0, -38.0, -17.0]], [[-6.0, -12.0, -25.0, -20.0, 17.0], [-25.0, 19.0, -18.0, 0.0, -6.0], [-23.0, 24.0, -30.0, 29.0, 6.0], [16.0, -26.0, 21.0, 11.0, 16.0]], [[-49.0, -42.0, 56.0, -28.0, -63.0], [-70.0, -63.0, -7.0, 77.0, 7.0], [35.0, -49.0, 21.0, 14.0, 35.0], [-70.0, -63.0, 0.0, 21.0, -14.0]], [[-18.0, -33.0, 6.0, -31.0, 13.0], [-37.0, 0.0, -3.0, 15.0, -15.0], [10.0, 19.0, -54.0, 66.0, 10.0], [1.0, -41.0, 52.0, 42.0, 33.0]]], [[[41.0, 63.0, -3.0, 63.0, -13.0], [86.0, 2.0, 20.0, -40.0, 22.0], [-4.0, -30.0, 87.0, -111.0, -27.0], [5.0, 90.0, -83.0, -68.0, -51.0]], [[41.0, 30.0, -80.0, 8.0, 75.0], [42.0, 79.0, -13.0, -73.0, -11.0], [-59.0, 69.0, -45.0, 10.0, -27.0], [82.0, 35.0, 16.0, -13.0, 26.0]], [[11.0, 18.0, -56.0, 0.0, 17.0], [-2.0, 37.0, -31.0, -19.0, 7.0], [-57.0, 23.0, 9.0, -26.0, -1.0], [30.0, 1.0, -24.0, -31.0, -10.0]], [[15.0, 0.0, -63.0, -18.0, 60.0], [-3.0, 60.0, -21.0, -39.0, -15.0], [-48.0, 63.0, -63.0, 45.0, -9.0], [60.0, -9.0, 39.0, 12.0, 36.0]], [[-27.0, -21.0, 20.0, -17.0, -34.0], [-43.0, -29.0, -12.0, 42.0, 6.0], [7.0, -24.0, 15.0, 2.0, 21.0], [-35.0, -38.0, -6.0, 5.0, -11.0]]], [[[-19.0, -12.0, 74.0, 10.0, -53.0], [-2.0, -63.0, 29.0, 41.0, 7.0], [63.0, -57.0, 39.0, -16.0, 9.0], [-60.0, 1.0, -14.0, 9.0, -20.0]], [[-1.0, -9.0, -2.0, -9.0, 14.0], [-5.0, 7.0, 2.0, -4.0, -8.0], [3.0, 14.0, -27.0, 28.0, -1.0], [9.0, -8.0, 24.0, 17.0, 17.0]], [[-4.0, -9.0, 18.0, -3.0, -1.0], [-1.0, -10.0, 11.0, 5.0, -5.0], [20.0, -3.0, -12.0, 18.0, 0.0], [-7.0, -3.0, 16.0, 16.0, 9.0]], [[-17.0, -18.0, 59.0, 0.0, -32.0], [-7.0, -46.0, 25.0, 31.0, -1.0], [54.0, -35.0, 9.0, 11.0, 7.0], [-42.0, -7.0, 9.0, 22.0, -2.0]], [[42.0, 39.0, -41.0, 29.0, 46.0], [64.0, 47.0, 9.0, -63.0, -3.0], [-25.0, 33.0, -6.0, -23.0, -30.0], [53.0, 59.0, -9.0, -23.0, 5.0]]], [[[7.0, 24.0, 34.0, 34.0, -39.0], [34.0, -33.0, 19.0, 7.0, 17.0], [25.0, -47.0, 69.0, -68.0, -5.0], [-32.0, 39.0, -54.0, -33.0, -40.0]], [[30.0, 24.0, -70.0, 4.0, 56.0], [26.0, 64.0, -18.0, -54.0, -6.0], [-56.0, 54.0, -30.0, 2.0, -18.0], [64.0, 22.0, 6.0, -16.0, 16.0]], [[-6.0, 0.0, 3.0, 0.0, -15.0], [-9.0, -9.0, -6.0, 12.0, 6.0], [-3.0, -12.0, 18.0, -15.0, 6.0], [-12.0, -6.0, -15.0, -9.0, -12.0]], [[24.0, 30.0, -53.0, 14.0, 25.0], [25.0, 41.0, -18.0, -36.0, 6.0], [-49.0, 24.0, 12.0, -35.0, -12.0], [38.0, 26.0, -27.0, -35.0, -10.0]], [[-24.0, -18.0, -30.0, -30.0, -12.0], [-60.0, 6.0, -42.0, 30.0, 6.0], [-42.0, 6.0, 0.0, 6.0, 24.0], [-6.0, -54.0, -6.0, -6.0, -6.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[[-7.0, 5.0, 4.0, -6.0, 4.0], [-2.0, 6.0, 7.0, -4.0, 7.0], [6.0, -5.0, -7.0, 0.0, 0.0], [1.0, 7.0, -3.0, 6.0, 4.0]], [[-7.0, 0.0, -6.0, -2.0, -4.0], [0.0, 1.0, 4.0, 7.0, -6.0], [1.0, 1.0, -4.0, -3.0, 4.0], [7.0, 6.0, -3.0, -5.0, -1.0]], [[0.0, -4.0, -3.0, -6.0, 0.0], [-3.0, -1.0, 2.0, -7.0, 1.0], [-5.0, 6.0, 6.0, 1.0, 1.0], [0.0, 0.0, -5.0, 0.0, -5.0]]], [[[-7.0, 2.0, 1.0, 2.0, 6.0], [-1.0, 1.0, -3.0, 1.0, 0.0], [5.0, 1.0, -3.0, 6.0, 5.0], [1.0, 4.0, 7.0, -4.0, -4.0]], [[-3.0, 3.0, -5.0, 4.0, 6.0], [4.0, 0.0, 3.0, 3.0, -2.0], [0.0, 1.0, -2.0, 1.0, 6.0], [7.0, -1.0, 2.0, -3.0, 3.0]], [[0.0, -4.0, 7.0, 1.0, 3.0], [-2.0, 6.0, 7.0, 0.0, -4.0], [6.0, 1.0, 0.0, -6.0, -1.0], [7.0, 0.0, -5.0, 4.0, 2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-2.0, 6.0, 0.0], [2.0, -3.0, 7.0]], [[0.0, 4.0, 3.0], [0.0, -3.0, -6.0]], [[0.0, -7.0, -6.0], [4.0, 3.0, -7.0]], [[0.0, 6.0, -5.0], [1.0, -5.0, 2.0]], [[-1.0, 5.0, -3.0], [2.0, -1.0, -1.0]]], [[[-2.0, -7.0, 7.0], [-1.0, -5.0, 0.0]], [[0.0, 6.0, -1.0], [-4.0, -7.0, 5.0]], [[7.0, 7.0, -7.0], [7.0, 6.0, -5.0]], [[-1.0, -7.0, -4.0], [3.0, 4.0, 0.0]], [[1.0, -3.0, 5.0], [7.0, -5.0, 7.0]]], [[[0.0, -7.0, -5.0], [-3.0, 6.0, -1.0]], [[7.0, 1.0, 3.0], [-2.0, -2.0, 4.0]], [[-7.0, 0.0, -1.0], [0.0, 5.0, 4.0]], [[-7.0, -2.0, -6.0], [0.0, 4.0, -6.0]], [[0.0, 0.0, -5.0], [2.0, 0.0, -2.0]]], [[[7.0, 0.0, -2.0], [-3.0, -2.0, -6.0]], [[-3.0, 0.0, 3.0], [-1.0, -1.0, 7.0]], [[-2.0, 1.0, -2.0], [-7.0, 1.0, 7.0]], [[-1.0, 0.0, 6.0], [3.0, 6.0, 0.0]], [[-3.0, 7.0, 4.0], [4.0, 4.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-33.0, -43.0, 22.0, -1.0, -17.0], [-24.0, 38.0, 44.0, 43.0, -72.0], [46.0, 22.0, -10.0, -51.0, 9.0], [70.0, 33.0, -39.0, -13.0, -17.0]], [[-19.0, 3.0, -60.0, -44.0, -52.0], [-9.0, -35.0, -29.0, -2.0, 9.0], [-47.0, 13.0, 8.0, 24.0, 7.0], [-35.0, 27.0, -3.0, -35.0, -40.0]], [[12.0, 69.0, 0.0, 63.0, 49.0], [40.0, -39.0, -92.0, 6.0, 58.0], [1.0, -43.0, -26.0, 84.0, 11.0], [-73.0, -29.0, 120.0, -18.0, 16.0]], [[-34.0, -1.0, 19.0, 2.0, -42.0], [-10.0, 24.0, 10.0, 63.0, -39.0], [48.0, -26.0, -47.0, -34.0, -8.0], [22.0, 45.0, -6.0, -11.0, 4.0]], [[-39.0, 12.0, -25.0, 13.0, -21.0], [7.0, -2.0, -9.0, 59.0, -34.0], [18.0, -8.0, -35.0, -1.0, 22.0], [22.0, 32.0, 20.0, -40.0, -7.0]]], [[[85.0, -55.0, 37.0, -38.0, -16.0], [-36.0, -27.0, -40.0, -106.0, 45.0], [-59.0, 39.0, 97.0, 17.0, -56.0], [-87.0, -55.0, -25.0, 42.0, -47.0]], [[7.0, -45.0, 33.0, -37.0, -75.0], [-31.0, 33.0, 48.0, 24.0, -43.0], [21.0, -6.0, -4.0, -80.0, -44.0], [24.0, 27.0, -80.0, 27.0, 4.0]], [[-165.0, 115.0, -51.0, 19.0, 63.0], [34.0, 33.0, 25.0, 95.0, 8.0], [89.0, -62.0, -152.0, 50.0, 97.0], [70.0, 113.0, 79.0, -59.0, 36.0]], [[23.0, 29.0, 33.0, 66.0, 66.0], [27.0, -6.0, -40.0, -2.0, 23.0], [22.0, -19.0, -6.0, 39.0, 7.0], [-19.0, -41.0, 73.0, 5.0, 23.0]], [[-20.0, -44.0, 88.0, -29.0, 49.0], [-58.0, 47.0, 18.0, -68.0, 12.0], [55.0, 31.0, 24.0, 9.0, -9.0], [1.0, 22.0, -15.0, 36.0, -47.0]]], [[[52.0, 36.0, 17.0, 61.0, 43.0], [44.0, -11.0, -18.0, 1.0, 29.0], [-3.0, -35.0, -5.0, 10.0, -11.0], [-17.0, -60.0, 42.0, 25.0, 60.0]], [[-36.0, -3.0, 49.0, -70.0, 12.0], [-37.0, 62.0, 87.0, -50.0, 34.0], [42.0, -16.0, -25.0, -38.0, -19.0], [26.0, 49.0, -77.0, 67.0, 22.0]], [[34.0, -32.0, -22.0, 72.0, 14.0], [29.0, -17.0, -8.0, 50.0, -76.0], [-13.0, 38.0, 33.0, -20.0, 25.0], [56.0, -54.0, 16.0, -41.0, 0.0]], [[51.0, 25.0, -60.0, 92.0, -14.0], [60.0, -74.0, -99.0, 68.0, -27.0], [-50.0, -5.0, 13.0, 40.0, 16.0], [-35.0, -65.0, 95.0, -68.0, 4.0]], [[-14.0, 32.0, 3.0, 32.0, 6.0], [17.0, -5.0, -30.0, 37.0, 3.0], [23.0, -30.0, -36.0, 19.0, 7.0], [-12.0, 8.0, 49.0, -16.0, 13.0]]], [[[-22.0, 55.0, -1.0, -50.0, -20.0], [-1.0, 5.0, 6.0, -23.0, 75.0], [1.0, -58.0, -48.0, 14.0, -23.0], [-52.0, 39.0, -6.0, 36.0, 32.0]], [[31.0, -60.0, 32.0, 1.0, -3.0], [-20.0, 20.0, 34.0, -13.0, -44.0], [4.0, 38.0, 44.0, -46.0, -15.0], [38.0, -24.0, -50.0, 17.0, -12.0]], [[53.0, -41.0, 29.0, 19.0, -27.0], [7.0, 26.0, 59.0, 25.0, -52.0], [6.0, 0.0, 17.0, -88.0, -34.0], [54.0, -37.0, -69.0, 36.0, 46.0]], [[-32.0, -5.0, -49.0, 0.0, 50.0], [5.0, -9.0, 14.0, -17.0, -13.0], [-21.0, 50.0, 22.0, 30.0, 57.0], [44.0, -1.0, 6.0, -36.0, -28.0]], [[-68.0, -31.0, -47.0, 9.0, 23.0], [-4.0, 19.0, 50.0, 49.0, -87.0], [19.0, 59.0, -3.0, -19.0, 71.0], [113.0, 33.0, -21.0, -61.0, -33.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(6.0)+(1-msk_arg1)*(5.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[2.0, 3.0, -3.0, 5.0, -7.0], [1.0, 0.0, -7.0, -4.0, -3.0], [2.0, 5.0, 3.0, 4.0, 2.0], [-5.0, 1.0, 2.0, 5.0, 4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[12.0, 18.0, -18.0, 30.0, -42.0], [6.0, 0.0, -42.0, -24.0, -18.0], [12.0, 30.0, 18.0, 24.0, 12.0], [-30.0, 6.0, 12.0, 30.0, 24.0]])+(1.-msk_ref)*numpy.array([[10.0, 15.0, -15.0, 25.0, -35.0], [5.0, 0.0, -35.0, -20.0, -15.0], [10.0, 25.0, 15.0, 20.0, 10.0], [-25.0, 5.0, 10.0, 25.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-2.0, -5.0, 1.0])+(1.-msk_arg1)*numpy.array([-1.0, 3.0, 1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-7.0, -2.0, 7.0], [7.0, 0.0, -2.0], [6.0, 5.0, -4.0], [6.0, -5.0, 3.0], [1.0, -1.0, 0.0]], [[-4.0, 1.0, -5.0], [2.0, 7.0, -2.0], [3.0, 0.0, 1.0], [-3.0, 3.0, 3.0], [3.0, 6.0, -7.0]], [[-5.0, -6.0, 1.0], [7.0, -5.0, -6.0], [0.0, -3.0, 4.0], [0.0, 0.0, -2.0], [6.0, 3.0, -6.0]], [[-1.0, 0.0, 3.0], [1.0, -4.0, -7.0], [-2.0, -1.0, 4.0], [-2.0, 5.0, -1.0], [-7.0, 5.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[31.0, -16.0, -41.0, 16.0, 3.0], [-2.0, -41.0, -5.0, -6.0, -43.0], [41.0, 5.0, 19.0, -2.0, -33.0], [5.0, 11.0, 13.0, -22.0, -7.0]])+(1.-msk_ref)*numpy.array([[8.0, -9.0, 5.0, -18.0, -4.0], [2.0, 17.0, -2.0, 15.0, 8.0], [-12.0, -28.0, -5.0, -2.0, -3.0], [4.0, -20.0, 3.0, 16.0, 26.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 6.0, 6.0], [-2.0, -1.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-1.0, -5.0, -2.0], [-1.0, 4.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[3.0, 3.0, 5.0], [0.0, 6.0, -3.0]], [[-1.0, -2.0, 7.0], [-7.0, -5.0, -7.0]], [[6.0, 2.0, 0.0], [-3.0, 7.0, -1.0]], [[-2.0, 0.0, -2.0], [-7.0, -2.0, 2.0]], [[-5.0, -2.0, -5.0], [-4.0, -5.0, -2.0]]], [[[-2.0, 1.0, -3.0], [0.0, 6.0, 6.0]], [[-1.0, 3.0, -3.0], [-3.0, 1.0, 3.0]], [[-4.0, -1.0, -1.0], [-6.0, -5.0, -4.0]], [[0.0, -3.0, 6.0], [-4.0, 2.0, -6.0]], [[-4.0, 0.0, 6.0], [2.0, 7.0, 3.0]]], [[[-7.0, -6.0, 3.0], [-3.0, -1.0, -5.0]], [[0.0, 6.0, -4.0], [4.0, 7.0, 0.0]], [[0.0, 7.0, 4.0], [3.0, 4.0, -3.0]], [[-2.0, 3.0, -6.0], [0.0, 4.0, -6.0]], [[-4.0, 2.0, 1.0], [-1.0, 7.0, -7.0]]], [[[-7.0, -5.0, 1.0], [0.0, -2.0, 2.0]], [[5.0, 6.0, 3.0], [-4.0, 7.0, -1.0]], [[1.0, -5.0, -1.0], [2.0, 5.0, 5.0]], [[2.0, 3.0, -4.0], [4.0, 1.0, 0.0]], [[-6.0, 5.0, 0.0], [-5.0, 7.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[12.0, 27.0, -29.0, 24.0, -7.0], [18.0, 23.0, 13.0, 0.0, 61.0], [11.0, -3.0, 44.0, -34.0, 9.0], [28.0, 21.0, -31.0, -27.0, 73.0]])+(1.-msk_ref)*numpy.array([[-22.0, -58.0, 9.0, 17.0, -3.0], [63.0, 17.0, -27.0, -21.0, 36.0], [0.0, 2.0, -48.0, -21.0, -21.0], [34.0, -15.0, 74.0, -9.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 2.0])+(1.-msk_arg1)*numpy.array([-6.0, 6.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[5.0, 5.0, 7.0, 5.0, -7.0], [4.0, 7.0, -6.0, 6.0, -6.0], [4.0, 1.0, 0.0, 3.0, -1.0], [7.0, 4.0, 4.0, 2.0, 7.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-20.0, 10.0], [-20.0, 10.0], [-28.0, 14.0], [-20.0, 10.0], [28.0, -14.0]], [[-16.0, 8.0], [-28.0, 14.0], [24.0, -12.0], [-24.0, 12.0], [24.0, -12.0]], [[-16.0, 8.0], [-4.0, 2.0], [0.0, 0.0], [-12.0, 6.0], [4.0, -2.0]], [[-28.0, 14.0], [-16.0, 8.0], [-16.0, 8.0], [-8.0, 4.0], [-28.0, 14.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, 30.0], [-30.0, 30.0], [-42.0, 42.0], [-30.0, 30.0], [42.0, -42.0]], [[-24.0, 24.0], [-42.0, 42.0], [36.0, -36.0], [-36.0, 36.0], [36.0, -36.0]], [[-24.0, 24.0], [-6.0, 6.0], [0.0, 0.0], [-18.0, 18.0], [6.0, -6.0]], [[-42.0, 42.0], [-24.0, 24.0], [-24.0, 24.0], [-12.0, 12.0], [-42.0, 42.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 0.0], [2.0, -5.0], [7.0, -1.0]])+(1.-msk_arg1)*numpy.array([[1.0, 6.0], [5.0, -1.0], [-2.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, 1.0, 6.0], [-1.0, 3.0, -7.0], [0.0, 0.0, -4.0], [-7.0, 7.0, 2.0], [-3.0, 3.0, 1.0]], [[1.0, -1.0, 3.0], [-2.0, 4.0, -2.0], [-3.0, -4.0, 6.0], [6.0, 1.0, 5.0], [2.0, 7.0, 7.0]], [[2.0, -1.0, -7.0], [6.0, -5.0, 3.0], [2.0, 3.0, -1.0], [-4.0, -3.0, 4.0], [5.0, 3.0, 1.0]], [[5.0, 0.0, -1.0], [7.0, -4.0, 0.0], [1.0, 6.0, 5.0], [0.0, -6.0, 7.0], [-5.0, 5.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[52.0, -11.0], [-47.0, -8.0], [-28.0, 4.0], [0.0, -37.0], [1.0, -16.0]], [[23.0, 2.0], [-14.0, -18.0], [22.0, 14.0], [61.0, -10.0], [71.0, -42.0]], [[-43.0, 12.0], [35.0, 22.0], [7.0, -14.0], [6.0, 11.0], [33.0, -16.0]], [[13.0, 1.0], [20.0, 20.0], [51.0, -35.0], [37.0, 23.0], [18.0, -29.0]]])+(1.-msk_ref)*numpy.array([[[-5.0, 41.0], [28.0, -44.0], [8.0, -20.0], [24.0, -39.0], [10.0, -16.0]], [[-10.0, 22.0], [22.0, -26.0], [-35.0, 16.0], [1.0, 60.0], [23.0, 40.0]], [[11.0, -22.0], [-25.0, 56.0], [19.0, 4.0], [-27.0, -1.0], [18.0, 32.0]], [[7.0, 25.0], [-13.0, 46.0], [21.0, 25.0], [-44.0, 41.0], [12.0, -15.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 0.0], [3.0, 2.0], [0.0, 0.0]], [[5.0, -4.0], [7.0, 5.0], [4.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[4.0, 6.0], [-4.0, 5.0], [3.0, 2.0]], [[1.0, 0.0], [3.0, 4.0], [-2.0, -6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-7.0, -4.0, -7.0], [-5.0, 0.0, 2.0]], [[5.0, 0.0, -5.0], [6.0, -4.0, -1.0]], [[-4.0, 1.0, -1.0], [-1.0, 7.0, -3.0]], [[-7.0, 2.0, -5.0], [-3.0, 0.0, 4.0]], [[-2.0, 5.0, 0.0], [-6.0, 5.0, 4.0]]], [[[3.0, -6.0, -6.0], [7.0, -1.0, -2.0]], [[-4.0, 2.0, -2.0], [-4.0, -1.0, -1.0]], [[3.0, 5.0, 3.0], [2.0, -3.0, -3.0]], [[1.0, -5.0, 1.0], [4.0, 6.0, 6.0]], [[-4.0, 6.0, -4.0], [5.0, 1.0, 1.0]]], [[[7.0, -7.0, -3.0], [3.0, -3.0, 2.0]], [[-3.0, 2.0, 0.0], [4.0, -5.0, 0.0]], [[2.0, 5.0, -6.0], [3.0, 4.0, 2.0]], [[5.0, -1.0, 0.0], [-2.0, -6.0, -6.0]], [[3.0, 6.0, 6.0], [-5.0, -3.0, 1.0]]], [[[-5.0, -6.0, 2.0], [-5.0, -2.0, 3.0]], [[-4.0, -5.0, -2.0], [-6.0, 2.0, -1.0]], [[4.0, -5.0, 0.0], [1.0, 1.0, -5.0]], [[2.0, 0.0, -6.0], [-3.0, 4.0, 5.0]], [[-5.0, -7.0, 2.0], [-3.0, 6.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, 12.0], [-27.0, -44.0], [55.0, 41.0], [42.0, 16.0], [46.0, 59.0]], [[-13.0, -45.0], [-5.0, 15.0], [-23.0, -13.0], [66.0, 4.0], [74.0, -3.0]], [[-54.0, -41.0], [6.0, -37.0], [56.0, 18.0], [-104.0, -24.0], [-39.0, 17.0]], [[-20.0, -2.0], [-15.0, 24.0], [-43.0, -9.0], [23.0, 32.0], [31.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[-42.0, -88.0], [1.0, 10.0], [3.0, 25.0], [-62.0, -66.0], [-27.0, 9.0]], [[26.0, -16.0], [-35.0, -16.0], [0.0, 55.0], [37.0, -29.0], [-46.0, -4.0]], [[37.0, -23.0], [-31.0, -28.0], [-19.0, 29.0], [16.0, 37.0], [-10.0, 42.0]], [[-7.0, -82.0], [0.0, -39.0], [50.0, 33.0], [-11.0, -14.0], [29.0, -37.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank2_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 1.0, 7.0, 0.0, 1.0], [-7.0, 5.0, -6.0, 4.0, 4.0], [5.0, 5.0, -6.0, 0.0, 0.0], [0.0, 3.0, -4.0, -6.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 4.0, 4.0, 0.0, 4.0], [-2.0, 4.0, 0.0, 5.0, -4.0], [0.0, -7.0, 3.0, 0.0, 1.0], [6.0, 0.0, 4.0, -4.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-1.0, 5.0, 7.0, -3.0, 0.0], [2.0, 5.0, -1.0, -4.0, 6.0], [2.0, 1.0, 3.0, -6.0, 4.0], [3.0, 2.0, -6.0, 7.0, 0.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, -1.0, -7.0, 0.0, -1.0], [7.0, -5.0, 6.0, -4.0, -4.0], [-5.0, -5.0, 6.0, 0.0, 0.0], [0.0, -3.0, 4.0, 6.0, 0.0]], [[15.0, 5.0, 35.0, 0.0, 5.0], [-35.0, 25.0, -30.0, 20.0, 20.0], [25.0, 25.0, -30.0, 0.0, 0.0], [0.0, 15.0, -20.0, -30.0, 0.0]], [[21.0, 7.0, 49.0, 0.0, 7.0], [-49.0, 35.0, -42.0, 28.0, 28.0], [35.0, 35.0, -42.0, 0.0, 0.0], [0.0, 21.0, -28.0, -42.0, 0.0]], [[-9.0, -3.0, -21.0, 0.0, -3.0], [21.0, -15.0, 18.0, -12.0, -12.0], [-15.0, -15.0, 18.0, 0.0, 0.0], [0.0, -9.0, 12.0, 18.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[6.0, 2.0, 14.0, 0.0, 2.0], [-14.0, 10.0, -12.0, 8.0, 8.0], [10.0, 10.0, -12.0, 0.0, 0.0], [0.0, 6.0, -8.0, -12.0, 0.0]], [[15.0, 5.0, 35.0, 0.0, 5.0], [-35.0, 25.0, -30.0, 20.0, 20.0], [25.0, 25.0, -30.0, 0.0, 0.0], [0.0, 15.0, -20.0, -30.0, 0.0]], [[-3.0, -1.0, -7.0, 0.0, -1.0], [7.0, -5.0, 6.0, -4.0, -4.0], [-5.0, -5.0, 6.0, 0.0, 0.0], [0.0, -3.0, 4.0, 6.0, 0.0]], [[-12.0, -4.0, -28.0, 0.0, -4.0], [28.0, -20.0, 24.0, -16.0, -16.0], [-20.0, -20.0, 24.0, 0.0, 0.0], [0.0, -12.0, 16.0, 24.0, 0.0]], [[18.0, 6.0, 42.0, 0.0, 6.0], [-42.0, 30.0, -36.0, 24.0, 24.0], [30.0, 30.0, -36.0, 0.0, 0.0], [0.0, 18.0, -24.0, -36.0, 0.0]]], [[[6.0, 2.0, 14.0, 0.0, 2.0], [-14.0, 10.0, -12.0, 8.0, 8.0], [10.0, 10.0, -12.0, 0.0, 0.0], [0.0, 6.0, -8.0, -12.0, 0.0]], [[3.0, 1.0, 7.0, 0.0, 1.0], [-7.0, 5.0, -6.0, 4.0, 4.0], [5.0, 5.0, -6.0, 0.0, 0.0], [0.0, 3.0, -4.0, -6.0, 0.0]], [[9.0, 3.0, 21.0, 0.0, 3.0], [-21.0, 15.0, -18.0, 12.0, 12.0], [15.0, 15.0, -18.0, 0.0, 0.0], [0.0, 9.0, -12.0, -18.0, 0.0]], [[-18.0, -6.0, -42.0, 0.0, -6.0], [42.0, -30.0, 36.0, -24.0, -24.0], [-30.0, -30.0, 36.0, 0.0, 0.0], [0.0, -18.0, 24.0, 36.0, 0.0]], [[12.0, 4.0, 28.0, 0.0, 4.0], [-28.0, 20.0, -24.0, 16.0, 16.0], [20.0, 20.0, -24.0, 0.0, 0.0], [0.0, 12.0, -16.0, -24.0, 0.0]]], [[[9.0, 3.0, 21.0, 0.0, 3.0], [-21.0, 15.0, -18.0, 12.0, 12.0], [15.0, 15.0, -18.0, 0.0, 0.0], [0.0, 9.0, -12.0, -18.0, 0.0]], [[6.0, 2.0, 14.0, 0.0, 2.0], [-14.0, 10.0, -12.0, 8.0, 8.0], [10.0, 10.0, -12.0, 0.0, 0.0], [0.0, 6.0, -8.0, -12.0, 0.0]], [[-18.0, -6.0, -42.0, 0.0, -6.0], [42.0, -30.0, 36.0, -24.0, -24.0], [-30.0, -30.0, 36.0, 0.0, 0.0], [0.0, -18.0, 24.0, 36.0, 0.0]], [[21.0, 7.0, 49.0, 0.0, 7.0], [-49.0, 35.0, -42.0, 28.0, 28.0], [35.0, 35.0, -42.0, 0.0, 0.0], [0.0, 21.0, -28.0, -42.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, -4.0, -4.0, 0.0, -4.0], [2.0, -4.0, 0.0, -5.0, 4.0], [0.0, 7.0, -3.0, 0.0, -1.0], [-6.0, 0.0, -4.0, 4.0, -4.0]], [[-30.0, 20.0, 20.0, 0.0, 20.0], [-10.0, 20.0, 0.0, 25.0, -20.0], [0.0, -35.0, 15.0, 0.0, 5.0], [30.0, 0.0, 20.0, -20.0, 20.0]], [[-42.0, 28.0, 28.0, 0.0, 28.0], [-14.0, 28.0, 0.0, 35.0, -28.0], [0.0, -49.0, 21.0, 0.0, 7.0], [42.0, 0.0, 28.0, -28.0, 28.0]], [[18.0, -12.0, -12.0, 0.0, -12.0], [6.0, -12.0, 0.0, -15.0, 12.0], [0.0, 21.0, -9.0, 0.0, -3.0], [-18.0, 0.0, -12.0, 12.0, -12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-12.0, 8.0, 8.0, 0.0, 8.0], [-4.0, 8.0, 0.0, 10.0, -8.0], [0.0, -14.0, 6.0, 0.0, 2.0], [12.0, 0.0, 8.0, -8.0, 8.0]], [[-30.0, 20.0, 20.0, 0.0, 20.0], [-10.0, 20.0, 0.0, 25.0, -20.0], [0.0, -35.0, 15.0, 0.0, 5.0], [30.0, 0.0, 20.0, -20.0, 20.0]], [[6.0, -4.0, -4.0, 0.0, -4.0], [2.0, -4.0, 0.0, -5.0, 4.0], [0.0, 7.0, -3.0, 0.0, -1.0], [-6.0, 0.0, -4.0, 4.0, -4.0]], [[24.0, -16.0, -16.0, 0.0, -16.0], [8.0, -16.0, 0.0, -20.0, 16.0], [0.0, 28.0, -12.0, 0.0, -4.0], [-24.0, 0.0, -16.0, 16.0, -16.0]], [[-36.0, 24.0, 24.0, 0.0, 24.0], [-12.0, 24.0, 0.0, 30.0, -24.0], [0.0, -42.0, 18.0, 0.0, 6.0], [36.0, 0.0, 24.0, -24.0, 24.0]]], [[[-12.0, 8.0, 8.0, 0.0, 8.0], [-4.0, 8.0, 0.0, 10.0, -8.0], [0.0, -14.0, 6.0, 0.0, 2.0], [12.0, 0.0, 8.0, -8.0, 8.0]], [[-6.0, 4.0, 4.0, 0.0, 4.0], [-2.0, 4.0, 0.0, 5.0, -4.0], [0.0, -7.0, 3.0, 0.0, 1.0], [6.0, 0.0, 4.0, -4.0, 4.0]], [[-18.0, 12.0, 12.0, 0.0, 12.0], [-6.0, 12.0, 0.0, 15.0, -12.0], [0.0, -21.0, 9.0, 0.0, 3.0], [18.0, 0.0, 12.0, -12.0, 12.0]], [[36.0, -24.0, -24.0, 0.0, -24.0], [12.0, -24.0, 0.0, -30.0, 24.0], [0.0, 42.0, -18.0, 0.0, -6.0], [-36.0, 0.0, -24.0, 24.0, -24.0]], [[-24.0, 16.0, 16.0, 0.0, 16.0], [-8.0, 16.0, 0.0, 20.0, -16.0], [0.0, -28.0, 12.0, 0.0, 4.0], [24.0, 0.0, 16.0, -16.0, 16.0]]], [[[-18.0, 12.0, 12.0, 0.0, 12.0], [-6.0, 12.0, 0.0, 15.0, -12.0], [0.0, -21.0, 9.0, 0.0, 3.0], [18.0, 0.0, 12.0, -12.0, 12.0]], [[-12.0, 8.0, 8.0, 0.0, 8.0], [-4.0, 8.0, 0.0, 10.0, -8.0], [0.0, -14.0, 6.0, 0.0, 2.0], [12.0, 0.0, 8.0, -8.0, 8.0]], [[36.0, -24.0, -24.0, 0.0, -24.0], [12.0, -24.0, 0.0, -30.0, 24.0], [0.0, 42.0, -18.0, 0.0, -6.0], [-36.0, 0.0, -24.0, 24.0, -24.0]], [[-42.0, 28.0, 28.0, 0.0, 28.0], [-14.0, 28.0, 0.0, 35.0, -28.0], [0.0, -49.0, 21.0, 0.0, 7.0], [42.0, 0.0, 28.0, -28.0, 28.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 4.0, 0.0, -2.0, -5.0], [4.0, -1.0, -3.0, 5.0, -3.0], [0.0, 1.0, 3.0, -1.0, 5.0], [1.0, 1.0, 5.0, -1.0, 1.0]], [[7.0, 0.0, -4.0, -3.0, -4.0], [1.0, -1.0, -2.0, -4.0, -6.0], [-3.0, -2.0, 4.0, 0.0, -7.0], [-1.0, 1.0, 4.0, -4.0, 3.0]], [[4.0, -1.0, 2.0, -2.0, -4.0], [-1.0, -5.0, -4.0, 4.0, -7.0], [-2.0, 3.0, 5.0, -3.0, -3.0], [6.0, 6.0, -1.0, -2.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 5.0, 1.0, -1.0, -1.0], [6.0, -2.0, 1.0, 6.0, 1.0], [-3.0, 1.0, 0.0, 3.0, 5.0], [4.0, -6.0, -5.0, -3.0, 2.0]], [[1.0, 0.0, 4.0, 7.0, 3.0], [-6.0, 1.0, 0.0, 5.0, -7.0], [-4.0, 0.0, -5.0, 7.0, 2.0], [4.0, 4.0, -6.0, 5.0, 0.0]], [[-3.0, 5.0, 7.0, -5.0, 6.0], [3.0, -5.0, -2.0, -5.0, 4.0], [-4.0, -4.0, 0.0, 1.0, 2.0], [-3.0, 0.0, 2.0, 3.0, 6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[1.0, -7.0, -1.0], [0.0, 1.0, -4.0], [1.0, 1.0, -4.0], [1.0, 4.0, -6.0], [-1.0, -5.0, 6.0]], [[-4.0, -7.0, -6.0], [-2.0, -5.0, -6.0], [4.0, -4.0, -6.0], [-2.0, 6.0, -1.0], [-3.0, 1.0, -2.0]], [[7.0, 1.0, -4.0], [2.0, -3.0, -6.0], [-4.0, -6.0, 0.0], [2.0, 1.0, 7.0], [0.0, -7.0, 3.0]], [[-2.0, 3.0, -7.0], [-3.0, 0.0, 0.0], [-3.0, 2.0, -1.0], [-3.0, 2.0, 2.0], [5.0, 1.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-54.0, 5.0, 26.0, 21.0, 27.0], [-2.0, 11.0, 15.0, 29.0, 46.0], [23.0, 12.0, -30.0, 2.0, 57.0], [2.0, -12.0, -22.0, 29.0, -22.0]], [[-9.0, 4.0, -12.0, 5.0, 12.0], [5.0, 19.0, 14.0, -20.0, 22.0], [5.0, -14.0, -16.0, 12.0, 5.0], [-25.0, -23.0, 8.0, 4.0, -5.0]], [[-10.0, 8.0, -12.0, 3.0, 7.0], [9.0, 18.0, 11.0, -15.0, 19.0], [5.0, -13.0, -13.0, 11.0, 10.0], [-24.0, -22.0, 13.0, 3.0, -4.0]], [[3.0, 10.0, -28.0, -2.0, 3.0], [14.0, 25.0, 13.0, -35.0, 15.0], [0.0, -25.0, -11.0, 17.0, -5.0], [-39.0, -31.0, 27.0, -5.0, 1.0]], [[-10.0, -10.0, 32.0, 5.0, 1.0], [-15.0, -24.0, -11.0, 39.0, -9.0], [3.0, 27.0, 7.0, -17.0, 12.0], [40.0, 30.0, -31.0, 9.0, -4.0]]], [[[-69.0, -10.0, 16.0, 41.0, 72.0], [-17.0, 41.0, 50.0, -16.0, 96.0], [33.0, -8.0, -70.0, 22.0, 47.0], [-33.0, -47.0, -42.0, 44.0, -37.0]], [[-57.0, -2.0, 8.0, 31.0, 54.0], [-7.0, 37.0, 40.0, -14.0, 78.0], [27.0, -10.0, -56.0, 20.0, 43.0], [-33.0, -43.0, -24.0, 34.0, -29.0]], [[-56.0, 22.0, 4.0, 16.0, 20.0], [18.0, 30.0, 20.0, 12.0, 54.0], [24.0, -6.0, -34.0, 14.0, 66.0], [-28.0, -36.0, 10.0, 24.0, -20.0]], [[40.0, -7.0, -26.0, -12.0, -10.0], [-1.0, 1.0, -2.0, -38.0, -23.0], [-16.0, -17.0, 13.0, 5.0, -49.0], [-14.0, -2.0, 15.0, -20.0, 14.0]], [[2.0, -10.0, -8.0, 7.0, 19.0], [-9.0, 12.0, 15.0, -27.0, 17.0], [1.0, -11.0, -15.0, 9.0, -16.0], [-16.0, -14.0, -9.0, 3.0, -4.0]]], [[[-16.0, 32.0, -12.0, -9.0, -23.0], [33.0, 12.0, -7.0, 15.0, 1.0], [5.0, -7.0, 5.0, 5.0, 40.0], [-18.0, -16.0, 43.0, -3.0, 2.0]], [[-47.0, 14.0, 0.0, 17.0, 26.0], [11.0, 31.0, 24.0, -2.0, 54.0], [21.0, -10.0, -36.0, 16.0, 49.0], [-31.0, -37.0, 4.0, 22.0, -19.0]], [[-38.0, -16.0, 24.0, 26.0, 44.0], [-22.0, 10.0, 24.0, 4.0, 48.0], [18.0, 8.0, -36.0, 4.0, 22.0], [2.0, -10.0, -44.0, 28.0, -22.0]], [[33.0, 1.0, 10.0, -21.0, -42.0], [2.0, -38.0, -36.0, 34.0, -61.0], [-17.0, 21.0, 45.0, -23.0, -18.0], [43.0, 45.0, 7.0, -20.0, 19.0]], [[-37.0, -3.0, 34.0, 15.0, 16.0], [-10.0, -8.0, 2.0, 40.0, 21.0], [15.0, 23.0, -13.0, -9.0, 40.0], [25.0, 11.0, -31.0, 22.0, -15.0]]], [[[-5.0, -1.0, -26.0, 9.0, 26.0], [2.0, 34.0, 28.0, -50.0, 37.0], [5.0, -29.0, -29.0, 23.0, -10.0], [-47.0, -41.0, 9.0, 4.0, -7.0]], [[3.0, -12.0, 0.0, 6.0, 15.0], [-12.0, 3.0, 9.0, -15.0, 9.0], [0.0, -3.0, -9.0, 3.0, -15.0], [-3.0, -3.0, -15.0, 3.0, -3.0]], [[13.0, -11.0, -10.0, 2.0, 11.0], [-9.0, 6.0, 9.0, -27.0, 4.0], [-4.0, -10.0, -6.0, 6.0, -26.0], [-11.0, -7.0, -6.0, -3.0, 1.0]], [[25.0, -14.0, -4.0, -4.0, -1.0], [-12.0, -9.0, -3.0, -15.0, -17.0], [-10.0, -1.0, 9.0, -3.0, -35.0], [7.0, 11.0, -9.0, -9.0, 7.0]], [[2.0, 20.0, -4.0, -13.0, -29.0], [21.0, -6.0, -17.0, 21.0, -21.0], [-3.0, 3.0, 19.0, -5.0, 18.0], [4.0, 6.0, 29.0, -9.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[-1.0, 0.0, -34.0, -45.0, -28.0], [45.0, -4.0, 3.0, -24.0, 46.0], [29.0, 5.0, 35.0, -47.0, -11.0], [-21.0, -34.0, 35.0, -41.0, -4.0]], [[13.0, -20.0, -24.0, 27.0, -21.0], [-18.0, 21.0, 8.0, 25.0, -23.0], [12.0, 16.0, -5.0, 3.0, -6.0], [16.0, 4.0, -14.0, -7.0, -24.0]], [[16.0, -15.0, -23.0, 26.0, -22.0], [-12.0, 19.0, 9.0, 31.0, -22.0], [9.0, 17.0, -5.0, 6.0, -1.0], [20.0, -2.0, -19.0, -10.0, -22.0]], [[25.0, -25.0, -25.0, 57.0, -25.0], [-36.0, 32.0, 13.0, 56.0, -51.0], [5.0, 25.0, -20.0, 25.0, 1.0], [38.0, 10.0, -41.0, -1.0, -34.0]], [[-26.0, 25.0, 21.0, -64.0, 22.0], [42.0, -33.0, -13.0, -61.0, 58.0], [-1.0, -25.0, 25.0, -32.0, -3.0], [-42.0, -14.0, 47.0, -4.0, 34.0]]], [[[-1.0, -50.0, -74.0, -15.0, -53.0], [0.0, 31.0, 8.0, -29.0, 21.0], [64.0, 20.0, 35.0, -67.0, -46.0], [-26.0, -4.0, 50.0, -41.0, -44.0]], [[7.0, -40.0, -64.0, -3.0, -49.0], [0.0, 29.0, 10.0, -7.0, 9.0], [50.0, 22.0, 25.0, -47.0, -32.0], [-10.0, -8.0, 28.0, -37.0, -40.0]], [[26.0, -10.0, -54.0, -2.0, -52.0], [30.0, 18.0, 16.0, 34.0, 8.0], [28.0, 28.0, 20.0, -22.0, 0.0], [18.0, -40.0, -8.0, -50.0, -28.0]], [[3.0, -15.0, 15.0, 49.0, 14.0], [-51.0, 15.0, 0.0, 23.0, -48.0], [-14.0, 2.0, -30.0, 35.0, 0.0], [19.0, 36.0, -28.0, 33.0, -10.0]], [[-2.0, -25.0, -13.0, 20.0, -6.0], [-30.0, 17.0, 1.0, -3.0, -18.0], [13.0, 5.0, -5.0, -4.0, -17.0], [-2.0, 22.0, 5.0, 8.0, -18.0]]], [[[34.0, 15.0, -17.0, 20.0, -28.0], [24.0, 7.0, 15.0, 67.0, -16.0], [-9.0, 23.0, -5.0, 24.0, 29.0], [44.0, -38.0, -49.0, -28.0, -10.0]], [[21.0, -20.0, -52.0, 7.0, -47.0], [12.0, 23.0, 14.0, 27.0, -1.0], [30.0, 26.0, 15.0, -21.0, -8.0], [14.0, -24.0, -4.0, -39.0, -32.0]], [[-18.0, -20.0, -28.0, -38.0, -14.0], [12.0, 2.0, -4.0, -54.0, 38.0], [36.0, -4.0, 30.0, -54.0, -32.0], [-40.0, 0.0, 56.0, -18.0, -8.0]], [[-14.0, 45.0, 55.0, -30.0, 43.0], [27.0, -38.0, -12.0, -18.0, 23.0], [-38.0, -26.0, -5.0, 20.0, 26.0], [-9.0, -8.0, -2.0, 20.0, 46.0]], [[-16.0, 15.0, -7.0, -64.0, -3.0], [51.0, -22.0, -6.0, -50.0, 61.0], [16.0, -12.0, 35.0, -46.0, -8.0], [-37.0, -28.0, 48.0, -26.0, 18.0]]], [[[18.0, -45.0, -39.0, 58.0, -31.0], [-51.0, 42.0, 12.0, 38.0, -51.0], [22.0, 26.0, -15.0, 8.0, -18.0], [25.0, 24.0, -22.0, 0.0, -46.0]], [[-9.0, -15.0, -3.0, 3.0, 3.0], [-18.0, 6.0, -3.0, -18.0, -3.0], [9.0, -3.0, 0.0, -9.0, -15.0], [-12.0, 18.0, 15.0, 9.0, -6.0]], [[-4.0, -20.0, -2.0, 22.0, 3.0], [-33.0, 13.0, -1.0, -3.0, -21.0], [5.0, 1.0, -10.0, 4.0, -13.0], [-1.0, 26.0, 1.0, 16.0, -12.0]], [[-13.0, -5.0, 19.0, 7.0, 21.0], [-24.0, -2.0, -7.0, -18.0, -9.0], [-7.0, -11.0, -10.0, 7.0, -7.0], [-10.0, 26.0, 7.0, 25.0, 6.0]], [[16.0, 25.0, 9.0, 2.0, -2.0], [24.0, -9.0, 5.0, 35.0, -2.0], [-19.0, 5.0, -5.0, 22.0, 27.0], [24.0, -26.0, -31.0, -10.0, 10.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, 2.0, -1.0, 2.0, 5.0], [-5.0, 4.0, -2.0, 1.0, 5.0], [-5.0, 1.0, 0.0, -5.0, 4.0], [-2.0, -4.0, 6.0, -1.0, 0.0]], [[-4.0, -7.0, 7.0, -5.0, 0.0], [-1.0, 0.0, 1.0, 3.0, 3.0], [0.0, -7.0, 0.0, -2.0, -3.0], [1.0, 6.0, 2.0, 4.0, 0.0]], [[-1.0, -1.0, 7.0, 7.0, 5.0], [-1.0, 2.0, 1.0, 3.0, -6.0], [6.0, -5.0, 4.0, 7.0, -5.0], [-3.0, -6.0, 4.0, 0.0, -2.0]]], [[[0.0, 6.0, -7.0, 1.0, -6.0], [-3.0, 7.0, 3.0, -7.0, 1.0], [-2.0, -2.0, 0.0, -2.0, 3.0], [3.0, -4.0, 1.0, -7.0, -3.0]], [[-5.0, 4.0, 2.0, -6.0, -5.0], [6.0, 1.0, -1.0, 7.0, 2.0], [-2.0, 4.0, 6.0, 4.0, -5.0], [6.0, 7.0, -3.0, -2.0, 0.0]], [[4.0, -5.0, -5.0, -2.0, -7.0], [4.0, -3.0, 7.0, 4.0, -5.0], [6.0, -6.0, -4.0, -1.0, -5.0], [-3.0, -5.0, -3.0, 1.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, -7.0, 2.0, 0.0, -5.0], [-6.0, 3.0, -6.0, -2.0, 0.0], [-7.0, 1.0, 6.0, -2.0, -3.0], [6.0, 5.0, 2.0, 0.0, 0.0]], [[4.0, 0.0, 3.0, 0.0, -6.0], [4.0, 7.0, 6.0, 1.0, -7.0], [-5.0, 5.0, -3.0, -4.0, 0.0], [5.0, -4.0, -6.0, -1.0, -5.0]], [[4.0, 0.0, -2.0, 0.0, 1.0], [7.0, 1.0, -1.0, 2.0, -5.0], [-1.0, -6.0, 3.0, 0.0, 6.0], [5.0, 3.0, 4.0, 0.0, 1.0]]], [[[-3.0, 6.0, 5.0, 1.0, 4.0], [1.0, -2.0, 6.0, 1.0, -1.0], [-1.0, -4.0, 3.0, -2.0, -4.0], [0.0, -5.0, 1.0, -4.0, 4.0]], [[1.0, 0.0, 7.0, -3.0, 6.0], [-4.0, -2.0, 7.0, -4.0, 6.0], [5.0, 3.0, 3.0, 0.0, -4.0], [-4.0, 5.0, 3.0, -7.0, -2.0]], [[-3.0, 5.0, -4.0, -2.0, 6.0], [-5.0, 3.0, -1.0, 2.0, 6.0], [-2.0, 7.0, 5.0, -1.0, 0.0], [5.0, -1.0, -6.0, -1.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, 1.0, -7.0], [-2.0, -2.0, 2.0]], [[0.0, -2.0, -6.0], [-6.0, -1.0, 0.0]], [[7.0, -5.0, 7.0], [6.0, -2.0, 1.0]], [[-6.0, -5.0, 0.0], [-1.0, 1.0, 2.0]], [[4.0, 0.0, -7.0], [2.0, -2.0, -3.0]]], [[[0.0, 1.0, 5.0], [5.0, 4.0, -3.0]], [[2.0, 4.0, 0.0], [7.0, 0.0, -6.0]], [[-7.0, 0.0, 3.0], [6.0, 0.0, 2.0]], [[0.0, 2.0, 1.0], [7.0, 0.0, -6.0]], [[-5.0, 3.0, 4.0], [4.0, -1.0, 5.0]]], [[[6.0, -7.0, 1.0], [1.0, -5.0, -5.0]], [[-6.0, -7.0, -2.0], [6.0, -4.0, -4.0]], [[-2.0, -1.0, 2.0], [-3.0, -1.0, 4.0]], [[-1.0, 1.0, -4.0], [7.0, -3.0, 1.0]], [[0.0, 4.0, -6.0], [3.0, 4.0, 2.0]]], [[[-6.0, 2.0, 4.0], [-2.0, -3.0, 4.0]], [[0.0, 3.0, 4.0], [-2.0, -4.0, 7.0]], [[7.0, -4.0, 3.0], [-5.0, -4.0, -7.0]], [[-4.0, -4.0, -6.0], [-1.0, -4.0, 7.0]], [[-7.0, -5.0, -5.0], [-6.0, 4.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[31.0, -40.0, -37.0, -58.0, -52.0], [33.0, -56.0, 14.0, -15.0, 4.0], [3.0, 7.0, -48.0, -32.0, 6.0], [8.0, 52.0, -58.0, 29.0, 12.0]], [[19.0, -20.0, -16.0, -32.0, 11.0], [20.0, -55.0, -25.0, 11.0, 22.0], [-22.0, 52.0, -30.0, -30.0, 23.0], [-8.0, 41.0, -31.0, 36.0, 30.0]], [[13.0, 65.0, -44.0, 104.0, 37.0], [-63.0, 79.0, 15.0, -39.0, -25.0], [5.0, -19.0, 12.0, 3.0, 31.0], [-37.0, -143.0, 69.0, -64.0, -36.0]], [[35.0, 11.0, -30.0, 2.0, -43.0], [52.0, -36.0, 17.0, 1.0, -54.0], [42.0, 23.0, -2.0, 44.0, -27.0], [4.0, -5.0, -56.0, -7.0, -5.0]], [[-3.0, 34.0, -56.0, -21.0, 4.0], [-43.0, 23.0, -28.0, -57.0, 75.0], [-80.0, 45.0, -28.0, -78.0, 82.0], [16.0, 19.0, 13.0, -17.0, 20.0]]], [[[-41.0, 49.0, 30.0, 17.0, -4.0], [-9.0, 58.0, -4.0, -1.0, 1.0], [-6.0, -8.0, 56.0, 42.0, -18.0], [34.0, -1.0, 24.0, -42.0, -13.0]], [[-44.0, 48.0, 7.0, 3.0, 10.0], [-59.0, 75.0, -21.0, -59.0, 59.0], [-60.0, -4.0, 24.0, -26.0, 47.0], [39.0, 18.0, 45.0, -41.0, 3.0]], [[19.0, 9.0, -24.0, 9.0, -70.0], [22.0, 14.0, 49.0, -32.0, -57.0], [53.0, -46.0, 4.0, 42.0, -35.0], [17.0, -24.0, -30.0, -33.0, -32.0]], [[-33.0, 57.0, 2.0, 16.0, 5.0], [-48.0, 69.0, -18.0, -64.0, 37.0], [-44.0, 3.0, 28.0, -5.0, 40.0], [38.0, 8.0, 33.0, -47.0, 1.0]], [[19.0, -40.0, -1.0, 3.0, -59.0], [20.0, 0.0, 65.0, 1.0, -63.0], [73.0, -88.0, -10.0, 30.0, -57.0], [-8.0, -34.0, -16.0, -4.0, -40.0]]], [[[20.0, 71.0, -40.0, 95.0, 89.0], [-77.0, 43.0, -45.0, -74.0, 19.0], [-46.0, 58.0, -6.0, -26.0, 93.0], [-34.0, -86.0, 57.0, -36.0, 15.0]], [[46.0, 79.0, -87.0, 47.0, -28.0], [-19.0, 22.0, -3.0, -119.0, -21.0], [-10.0, 49.0, -16.0, 6.0, 65.0], [17.0, -38.0, -28.0, -60.0, 2.0]], [[27.0, -41.0, 8.0, 10.0, -5.0], [28.0, -38.0, 25.0, 31.0, -50.0], [54.0, -27.0, -14.0, 24.0, -39.0], [-30.0, -25.0, -18.0, 25.0, -11.0]], [[21.0, 20.0, -80.0, -12.0, -59.0], [-27.0, 31.0, 30.0, -76.0, 18.0], [-21.0, -20.0, -38.0, -52.0, 44.0], [15.0, -20.0, -7.0, -37.0, -17.0]], [[-22.0, 2.0, -37.0, -87.0, -82.0], [25.0, 7.0, 17.0, 9.0, 49.0], [-38.0, 0.0, -8.0, -42.0, -3.0], [49.0, 66.0, -31.0, -11.0, -5.0]]], [[[31.0, -74.0, 36.0, 14.0, -11.0], [28.0, -45.0, 43.0, 21.0, -76.0], [88.0, -72.0, -18.0, 42.0, -61.0], [-34.0, -21.0, -21.0, 38.0, -18.0]], [[32.0, -88.0, 20.0, 21.0, 3.0], [3.0, -31.0, 54.0, 35.0, -60.0], [78.0, -95.0, -36.0, 3.0, -50.0], [-60.0, -61.0, 11.0, 41.0, -30.0]], [[-9.0, 28.0, 48.0, 88.0, 149.0], [-71.0, 16.0, -75.0, -17.0, 27.0], [-41.0, 56.0, 16.0, -5.0, 65.0], [-45.0, -43.0, 74.0, 13.0, 37.0]], [[78.0, -31.0, -102.0, -21.0, -73.0], [37.0, -60.0, 48.0, -27.0, -40.0], [36.0, -2.0, -76.0, -35.0, 8.0], [-26.0, -31.0, -66.0, 10.0, -13.0]], [[15.0, 11.0, -8.0, -52.0, -37.0], [83.0, -73.0, -25.0, 29.0, -13.0], [3.0, 87.0, 8.0, 39.0, -21.0], [33.0, 85.0, -87.0, 20.0, 32.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, 33.0, -25.0, 0.0, 4.0], [-19.0, -1.0, 15.0, 7.0, 30.0], [25.0, 58.0, -56.0, 8.0, -11.0], [-42.0, -52.0, -64.0, 19.0, -16.0]], [[-15.0, -36.0, -31.0, -3.0, -24.0], [-52.0, -6.0, -49.0, -16.0, 44.0], [17.0, 47.0, -33.0, 20.0, -8.0], [-36.0, 15.0, -21.0, 33.0, -18.0]], [[-22.0, -8.0, -3.0, 10.0, 20.0], [-4.0, -12.0, -58.0, 11.0, -12.0], [-49.0, -83.0, 95.0, -7.0, 5.0], [65.0, 35.0, 66.0, -6.0, 60.0]], [[-16.0, 46.0, -33.0, -8.0, 74.0], [1.0, -47.0, 5.0, 6.0, 54.0], [69.0, -10.0, -11.0, 32.0, 18.0], [-55.0, -2.0, 8.0, 0.0, 19.0]], [[-31.0, -31.0, 30.0, 14.0, -49.0], [-48.0, -4.0, -16.0, -18.0, 3.0], [-27.0, 11.0, -12.0, -9.0, -54.0], [-18.0, -18.0, -6.0, 9.0, 5.0]]], [[[22.0, 15.0, 58.0, -1.0, 25.0], [43.0, -15.0, 62.0, -6.0, -31.0], [11.0, -54.0, 24.0, -11.0, -6.0], [-1.0, 9.0, 49.0, -46.0, 12.0]], [[11.0, -2.0, 75.0, 19.0, -42.0], [41.0, 2.0, 60.0, -5.0, -71.0], [-29.0, -48.0, -9.0, -28.0, -34.0], [2.0, -35.0, 23.0, -26.0, 8.0]], [[-5.0, 95.0, 2.0, 2.0, 74.0], [59.0, -24.0, 73.0, 30.0, -9.0], [36.0, -35.0, -5.0, 0.0, 15.0], [-17.0, -58.0, -8.0, -26.0, 27.0]], [[9.0, 12.0, 63.0, 19.0, -19.0], [52.0, -17.0, 59.0, -1.0, -62.0], [-6.0, -66.0, -12.0, -16.0, -22.0], [-15.0, -34.0, 35.0, -24.0, 19.0]], [[5.0, 84.0, -16.0, -3.0, 51.0], [53.0, 19.0, 56.0, 39.0, -21.0], [-3.0, 2.0, 7.0, -15.0, 27.0], [34.0, -55.0, -41.0, -17.0, 7.0]]], [[[-23.0, -61.0, -21.0, 26.0, -43.0], [-11.0, -37.0, -103.0, -6.0, -17.0], [-24.0, -89.0, 23.0, 19.0, 4.0], [1.0, 36.0, 74.0, 43.0, 50.0]], [[-40.0, 58.0, -11.0, 26.0, 46.0], [36.0, -85.0, 8.0, 15.0, 5.0], [61.0, -93.0, -35.0, 32.0, -2.0], [-85.0, -54.0, 40.0, 15.0, 65.0]], [[2.0, 16.0, -49.0, -8.0, 24.0], [3.0, 9.0, -25.0, 16.0, 18.0], [7.0, 18.0, 5.0, 10.0, 34.0], [17.0, 6.0, -20.0, 16.0, -3.0]], [[-38.0, 54.0, 19.0, 14.0, 11.0], [-4.0, -5.0, 36.0, 16.0, -6.0], [-18.0, -2.0, -4.0, -17.0, -37.0], [-4.0, -72.0, -32.0, -9.0, 25.0]], [[-19.0, 28.0, 59.0, -13.0, 18.0], [-49.0, 14.0, 74.0, -17.0, 35.0], [-1.0, 70.0, 1.0, -24.0, -64.0], [-16.0, -31.0, -45.0, -46.0, -22.0]]], [[[21.0, 50.0, -61.0, -1.0, 20.0], [62.0, 22.0, 7.0, 40.0, -26.0], [7.0, 7.0, -25.0, 4.0, 62.0], [26.0, -35.0, -43.0, 23.0, -8.0]], [[9.0, 23.0, -65.0, -4.0, -4.0], [19.0, 58.0, -33.0, 39.0, -21.0], [-51.0, 36.0, 20.0, -15.0, 48.0], [86.0, -17.0, -58.0, 26.0, -11.0]], [[21.0, -114.0, -29.0, 21.0, -94.0], [9.0, -7.0, -120.0, -15.0, -48.0], [-33.0, -72.0, 1.0, 19.0, 33.0], [18.0, 72.0, 75.0, 59.0, 11.0]], [[-58.0, 57.0, -69.0, -3.0, 52.0], [-54.0, -15.0, -35.0, 21.0, 77.0], [21.0, 53.0, -10.0, 19.0, -4.0], [-23.0, -44.0, -63.0, 29.0, 18.0]], [[-8.0, 8.0, -17.0, -16.0, 54.0], [-30.0, -60.0, 10.0, -25.0, 84.0], [107.0, 27.0, -53.0, 47.0, -1.0], [-113.0, 21.0, 8.0, 2.0, -12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_float_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=6.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-6.0, 5.0], [-5.0, 0.0]], [[-2.0, 0.0], [-2.0, 0.0]], [[-1.0, 3.0], [7.0, 6.0]], [[6.0, -4.0], [1.0, -4.0]], [[0.0, -2.0], [2.0, -2.0]], [[0.0, 4.0], [0.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-36.0, 30.0], [-30.0, 0.0]], [[-12.0, 0.0], [-12.0, 0.0]], [[-6.0, 18.0], [42.0, 36.0]], [[36.0, -24.0], [6.0, -24.0]], [[0.0, -12.0], [12.0, -12.0]], [[0.0, 24.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[7.0, -5.0], [0.0, 4.0]], [[-3.0, 6.0], [-1.0, 5.0]], [[1.0, -5.0], [0.0, -2.0]], [[-7.0, -1.0], [1.0, -1.0]], [[6.0, -2.0], [3.0, -6.0]], [[-6.0, 2.0], [2.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-49.0, 35.0], [-0.0, -28.0]], [[21.0, -42.0], [7.0, -35.0]], [[-7.0, 35.0], [-0.0, 14.0]], [[49.0, 7.0], [-7.0, 7.0]], [[-42.0, 14.0], [-21.0, 42.0]], [[42.0, -14.0], [-14.0, -49.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=numpy.array([4.0, -5.0, 5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[6.0, -2.0, 0.0], [-3.0, 7.0, -5.0]], [[5.0, 6.0, 0.0], [4.0, 4.0, -5.0]]], [[[-3.0, 4.0, 7.0], [5.0, 1.0, -7.0]], [[-1.0, 2.0, -5.0], [-7.0, 1.0, -7.0]]], [[[5.0, 0.0, -5.0], [2.0, -1.0, 1.0]], [[1.0, 7.0, -1.0], [7.0, 4.0, -2.0]]], [[[6.0, -4.0, 7.0], [2.0, -4.0, -2.0]], [[-2.0, -4.0, -2.0], [0.0, 0.0, 3.0]]], [[[-1.0, -5.0, 7.0], [-6.0, 2.0, 2.0]], [[7.0, 5.0, 5.0], [2.0, -6.0, -1.0]]], [[[-1.0, -7.0, -1.0], [1.0, 3.0, 7.0]], [[6.0, -6.0, 6.0], [0.0, 1.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[34.0, -72.0], [-10.0, -29.0]], [[3.0, -20.0], [-39.0, -68.0]], [[-5.0, 18.0], [-36.0, -2.0]], [[79.0, 18.0], [2.0, 15.0]], [[56.0, -24.0], [28.0, 33.0]], [[26.0, 24.0], [84.0, -5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_array_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([-5.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-4.0, 1.0], [2.0, 4.0]], [[1.0, -3.0], [2.0, -7.0]], [[-1.0, 1.0], [-2.0, -6.0]], [[1.0, 4.0], [-5.0, -3.0]], [[-3.0, 5.0], [6.0, 6.0]], [[5.0, -2.0], [6.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[20.0, 4.0], [-5.0, -1.0]], [[-10.0, -2.0], [-20.0, -4.0]]], [[[-5.0, -1.0], [15.0, 3.0]], [[-10.0, -2.0], [35.0, 7.0]]], [[[5.0, 1.0], [-5.0, -1.0]], [[10.0, 2.0], [30.0, 6.0]]], [[[-5.0, -1.0], [-20.0, -4.0]], [[25.0, 5.0], [15.0, 3.0]]], [[[15.0, 3.0], [-25.0, -5.0]], [[-30.0, -6.0], [-30.0, -6.0]]], [[[-25.0, -5.0], [10.0, 2.0]], [[-30.0, -6.0], [-30.0, -6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=numpy.array([[-3.0, -6.0], [-1.0, 4.0], [2.0, 2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[5.0, 4.0, 0.0], [6.0, 4.0, 7.0]], [[6.0, -1.0, -4.0], [-7.0, -7.0, 7.0]]], [[[-4.0, 3.0, -5.0], [-5.0, 0.0, -1.0]], [[6.0, -1.0, 2.0], [-7.0, -1.0, -7.0]]], [[[6.0, -7.0, -4.0], [0.0, 0.0, -6.0]], [[-7.0, -7.0, -7.0], [5.0, 0.0, 6.0]]], [[[-5.0, -3.0, 6.0], [0.0, -5.0, -1.0]], [[-6.0, -1.0, 4.0], [7.0, -3.0, -6.0]]], [[[-2.0, -5.0, -3.0], [-4.0, 0.0, 4.0]], [[3.0, 7.0, -5.0], [4.0, -5.0, -4.0]]], [[[1.0, -3.0, 3.0], [3.0, -7.0, 0.0]], [[-1.0, 2.0, -7.0], [-5.0, 6.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-19.0, -14.0], [-8.0, -6.0]], [[-25.0, -48.0], [42.0, 28.0]]], [[[-1.0, 26.0], [13.0, 28.0]], [[-13.0, -36.0], [8.0, 24.0]]], [[[-19.0, -72.0], [-12.0, -12.0]], [[14.0, 0.0], [-3.0, -18.0]]], [[[30.0, 30.0], [3.0, -22.0]], [[27.0, 40.0], [-30.0, -66.0]]], [[[5.0, -14.0], [20.0, 32.0]], [[-26.0, 0.0], [-15.0, -52.0]]], [[[6.0, -12.0], [-2.0, -46.0]], [[-13.0, 0.0], [17.0, 62.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[4.0, -4.0], [-7.0, -5.0]], [[-3.0, 1.0], [-5.0, -2.0]], [[-3.0, 3.0], [-5.0, -2.0]], [[-1.0, 0.0], [-2.0, -3.0]], [[-2.0, -1.0], [4.0, -4.0]], [[3.0, 2.0], [-7.0, 1.0]]])
      s1=numpy.array(0.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.0, -0.0], [-0.0, -0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[-0.0, -0.0], [0.0, -0.0]], [[0.0, 0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[5.0, 2.0, 5.0], [-3.0, 2.0, 5.0]], [[0.0, 1.0, 7.0], [-3.0, 0.0, -3.0]]], [[[1.0, 7.0, 0.0], [-5.0, 3.0, -5.0]], [[-5.0, 6.0, 0.0], [-6.0, 3.0, -5.0]]], [[[-3.0, -5.0, -2.0], [6.0, 6.0, 0.0]], [[-5.0, -1.0, 5.0], [5.0, 7.0, -1.0]]], [[[5.0, 0.0, -6.0], [-7.0, -6.0, -6.0]], [[0.0, -5.0, -4.0], [7.0, -5.0, 1.0]]], [[[-4.0, 0.0, -7.0], [-5.0, 0.0, -1.0]], [[-2.0, -1.0, 1.0], [6.0, 4.0, 7.0]]], [[[3.0, -1.0, 0.0], [-3.0, 3.0, -4.0]], [[3.0, 1.0, -5.0], [0.0, 0.0, -4.0]]]])
      s1=numpy.array([0.0, 6.0, -7.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-23.0, -23.0], [-43.0, 21.0]], [[42.0, 53.0], [36.0, 53.0]], [[-16.0, 36.0], [-41.0, 49.0]], [[42.0, 6.0], [-2.0, -37.0]], [[49.0, 7.0], [-13.0, -25.0]], [[-6.0, 46.0], [41.0, 28.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-2.0, 7.0], [-7.0, 6.0]], [[0.0, 1.0], [-1.0, -1.0]], [[-6.0, -4.0], [4.0, 0.0]], [[0.0, 1.0], [0.0, 2.0]], [[-1.0, 5.0], [0.0, -5.0]], [[-1.0, 5.0], [3.0, -7.0]]])
      s1=numpy.array([6.0, 2.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-12.0, -4.0], [42.0, 14.0]], [[-42.0, -14.0], [36.0, 12.0]]], [[[0.0, 0.0], [6.0, 2.0]], [[-6.0, -2.0], [-6.0, -2.0]]], [[[-36.0, -12.0], [-24.0, -8.0]], [[24.0, 8.0], [0.0, 0.0]]], [[[0.0, 0.0], [6.0, 2.0]], [[0.0, 0.0], [12.0, 4.0]]], [[[-6.0, -2.0], [30.0, 10.0]], [[0.0, 0.0], [-30.0, -10.0]]], [[[-6.0, -2.0], [30.0, 10.0]], [[18.0, 6.0], [-42.0, -14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[3.0, 6.0, 7.0], [1.0, 0.0, -5.0]], [[-2.0, 0.0, 7.0], [7.0, -2.0, 3.0]]], [[[-1.0, 7.0, -3.0], [0.0, 2.0, 0.0]], [[-5.0, -6.0, 3.0], [2.0, 7.0, -2.0]]], [[[-6.0, -7.0, 7.0], [6.0, 3.0, -5.0]], [[3.0, 3.0, 0.0], [-2.0, 0.0, 1.0]]], [[[-1.0, 2.0, 5.0], [3.0, 0.0, 0.0]], [[0.0, -1.0, 0.0], [5.0, 6.0, -3.0]]], [[[-4.0, 1.0, -6.0], [-3.0, -2.0, 0.0]], [[-1.0, 5.0, -5.0], [-7.0, 3.0, 5.0]]], [[[-5.0, 4.0, 5.0], [6.0, 6.0, -3.0]], [[-3.0, -5.0, 0.0], [5.0, -3.0, 7.0]]]])
      s1=numpy.array([[6.0, 3.0], [-2.0, 4.0], [3.0, -2.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[27.0, 19.0], [-9.0, 13.0]], [[9.0, -20.0], [55.0, 7.0]]], [[[-29.0, 31.0], [-4.0, 8.0]], [[-9.0, -45.0], [-8.0, 38.0]]], [[[-1.0, -60.0], [15.0, 40.0]], [[12.0, 21.0], [-9.0, -8.0]]], [[[5.0, -5.0], [18.0, 9.0]], [[2.0, -4.0], [9.0, 45.0]]], [[[-44.0, 4.0], [-14.0, -17.0]], [[-31.0, 27.0], [-33.0, -19.0]]], [[[-23.0, -9.0], [15.0, 48.0]], [[-8.0, -29.0], [57.0, -11.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(1.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-2.0, 7.0], [-1.0, 0.0]], [[3.0, 0.0], [0.0, 6.0]], [[-7.0, -5.0], [7.0, -6.0]], [[3.0, 1.0], [0.0, 0.0]], [[-7.0, -5.0], [6.0, -4.0]], [[7.0, -2.0], [5.0, -4.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-2.0, 7.0], [-1.0, 0.0]], [[3.0, 0.0], [0.0, 6.0]], [[-7.0, -5.0], [7.0, -6.0]], [[3.0, 1.0], [0.0, 0.0]], [[-7.0, -5.0], [6.0, -4.0]], [[7.0, -2.0], [5.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Data(numpy.array([-2.0, 0.0, 6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[5.0, 3.0, -4.0], [4.0, -6.0, 0.0]], [[-6.0, 6.0, -4.0], [3.0, 3.0, -1.0]]], [[[-7.0, -7.0, 3.0], [-7.0, 3.0, 6.0]], [[-5.0, -3.0, 7.0], [-1.0, 7.0, 2.0]]], [[[-6.0, 6.0, 5.0], [3.0, 6.0, 6.0]], [[-6.0, -3.0, -4.0], [-2.0, 7.0, 2.0]]], [[[-2.0, -2.0, 1.0], [-7.0, 6.0, -1.0]], [[-1.0, -5.0, -3.0], [-3.0, 3.0, -6.0]]], [[[-2.0, -6.0, 2.0], [2.0, 3.0, -6.0]], [[0.0, 4.0, 4.0], [-3.0, 4.0, 6.0]]], [[[5.0, -2.0, 0.0], [-2.0, 5.0, 0.0]], [[7.0, 1.0, -1.0], [-3.0, -4.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-34.0, -8.0], [-12.0, -12.0]], [[32.0, 50.0], [52.0, 14.0]], [[42.0, 30.0], [-12.0, 16.0]], [[10.0, 8.0], [-16.0, -30.0]], [[16.0, -40.0], [24.0, 42.0]], [[-10.0, 4.0], [-20.0, 6.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_constData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([1.0, -1.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-7.0, -7.0], [5.0, 6.0]], [[7.0, -5.0], [-4.0, 3.0]], [[0.0, 6.0], [2.0, 7.0]], [[5.0, 1.0], [-6.0, 4.0]], [[-7.0, -1.0], [5.0, 4.0]], [[5.0, 4.0], [-5.0, 3.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-7.0, 7.0], [-7.0, 7.0]], [[5.0, -5.0], [6.0, -6.0]]], [[[7.0, -7.0], [-5.0, 5.0]], [[-4.0, 4.0], [3.0, -3.0]]], [[[0.0, 0.0], [6.0, -6.0]], [[2.0, -2.0], [7.0, -7.0]]], [[[5.0, -5.0], [1.0, -1.0]], [[-6.0, 6.0], [4.0, -4.0]]], [[[-7.0, 7.0], [-1.0, 1.0]], [[5.0, -5.0], [4.0, -4.0]]], [[[5.0, -5.0], [4.0, -4.0]], [[-5.0, 5.0], [3.0, -3.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Data(numpy.array([[6.0, -2.0], [-2.0, -6.0], [-1.0, -6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[7.0, -4.0, 7.0], [7.0, -4.0, -5.0]], [[2.0, -3.0, -1.0], [-2.0, 3.0, 5.0]]], [[[6.0, 5.0, 3.0], [4.0, -1.0, 0.0]], [[-6.0, 1.0, -7.0], [-1.0, 4.0, 7.0]]], [[[-4.0, 2.0, 3.0], [-1.0, 7.0, 6.0]], [[2.0, -4.0, 0.0], [4.0, 4.0, 0.0]]], [[[4.0, 0.0, 6.0], [-4.0, 7.0, -1.0]], [[3.0, 0.0, 3.0], [6.0, 6.0, 2.0]]], [[[-7.0, 7.0, -7.0], [5.0, -1.0, 0.0]], [[0.0, -2.0, -4.0], [1.0, 4.0, 1.0]]], [[[-6.0, -3.0, 2.0], [5.0, 2.0, -2.0]], [[5.0, -3.0, -1.0], [0.0, -1.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[43.0, -32.0], [55.0, 40.0]], [[19.0, 20.0], [-23.0, -44.0]]], [[[23.0, -60.0], [26.0, -2.0]], [[-31.0, 48.0], [-21.0, -64.0]]], [[[-31.0, -22.0], [-26.0, -76.0]], [[20.0, 20.0], [16.0, -32.0]]], [[[18.0, -44.0], [-37.0, -28.0]], [[15.0, -24.0], [22.0, -60.0]]], [[[-49.0, 14.0], [32.0, -4.0]], [[8.0, 36.0], [-3.0, -32.0]]], [[[-32.0, 18.0], [28.0, -10.0]], [[37.0, 14.0], [3.0, 12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-7.0, 5.0], [5.0, 7.0]], [[-7.0, 0.0], [7.0, 1.0]], [[4.0, -2.0], [3.0, -3.0]], [[-7.0, 0.0], [-5.0, 5.0]], [[-7.0, 0.0], [7.0, 5.0]], [[-5.0, 6.0], [1.0, -7.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[28.0, -20.0], [-20.0, -28.0]], [[28.0, -0.0], [-28.0, -4.0]], [[-16.0, 8.0], [-12.0, 12.0]], [[28.0, -0.0], [20.0, -20.0]], [[28.0, -0.0], [-28.0, -20.0]], [[20.0, -24.0], [-4.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[-14.0, 10.0], [10.0, 14.0]], [[-14.0, 0.0], [14.0, 2.0]], [[8.0, -4.0], [6.0, -6.0]], [[-14.0, 0.0], [-10.0, 10.0]], [[-14.0, 0.0], [14.0, 10.0]], [[-10.0, 12.0], [2.0, -14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -2.0, -4.0])+(1.-msk_arg1)*numpy.array([-3.0, -6.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[7.0, -6.0, 4.0], [-2.0, 7.0, -7.0]], [[-4.0, 0.0, -2.0], [-3.0, 6.0, -6.0]]], [[[7.0, -3.0, 1.0], [3.0, 7.0, -1.0]], [[-1.0, -7.0, -4.0], [-3.0, 0.0, 2.0]]], [[[0.0, -1.0, 1.0], [-4.0, -6.0, 3.0]], [[5.0, -5.0, 6.0], [0.0, -1.0, -6.0]]], [[[0.0, 0.0, -5.0], [-6.0, -5.0, 2.0]], [[3.0, -3.0, 4.0], [-3.0, 7.0, -5.0]]], [[[7.0, 4.0, 1.0], [4.0, 7.0, 6.0]], [[2.0, 5.0, -7.0], [3.0, 3.0, -2.0]]], [[[-2.0, -5.0, 4.0], [-1.0, 0.0, -6.0]], [[4.0, 0.0, 6.0], [4.0, 5.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[38.0, 2.0], [-16.0, -6.0]], [[44.0, 8.0], [24.0, -26.0]], [[-2.0, -24.0], [16.0, 26.0]], [[20.0, -34.0], [8.0, -12.0]], [[30.0, -14.0], [30.0, 20.0]], [[-18.0, 18.0], [0.0, 42.0]]])+(1.-msk_ref)*numpy.array([[[15.0, -36.0], [12.0, -27.0]], [[-3.0, -51.0], [45.0, 9.0]], [[6.0, 48.0], [15.0, 6.0]], [[0.0, 48.0], [9.0, -33.0]], [[-45.0, -54.0], [-36.0, -27.0]], [[36.0, 3.0], [-12.0, -42.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank3_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, -7.0])+(1.-msk_arg1)*numpy.array([-4.0, -2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-4.0, 3.0], [0.0, 1.0]], [[0.0, -2.0], [5.0, 3.0]], [[2.0, -5.0], [-3.0, 6.0]], [[1.0, 3.0], [0.0, -6.0]], [[3.0, -6.0], [3.0, 6.0]], [[-5.0, 1.0], [0.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 28.0], [-3.0, -21.0]], [[0.0, 0.0], [-1.0, -7.0]]], [[[0.0, 0.0], [2.0, 14.0]], [[-5.0, -35.0], [-3.0, -21.0]]], [[[-2.0, -14.0], [5.0, 35.0]], [[3.0, 21.0], [-6.0, -42.0]]], [[[-1.0, -7.0], [-3.0, -21.0]], [[0.0, 0.0], [6.0, 42.0]]], [[[-3.0, -21.0], [6.0, 42.0]], [[-3.0, -21.0], [-6.0, -42.0]]], [[[5.0, 35.0], [-1.0, -7.0]], [[0.0, 0.0], [-7.0, -49.0]]]])+(1.-msk_ref)*numpy.array([[[[16.0, 8.0], [-12.0, -6.0]], [[0.0, 0.0], [-4.0, -2.0]]], [[[0.0, 0.0], [8.0, 4.0]], [[-20.0, -10.0], [-12.0, -6.0]]], [[[-8.0, -4.0], [20.0, 10.0]], [[12.0, 6.0], [-24.0, -12.0]]], [[[-4.0, -2.0], [-12.0, -6.0]], [[0.0, 0.0], [24.0, 12.0]]], [[[-12.0, -6.0], [24.0, 12.0]], [[-12.0, -6.0], [-24.0, -12.0]]], [[[20.0, 10.0], [-4.0, -2.0]], [[0.0, 0.0], [-28.0, -14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 5.0], [-1.0, 0.0], [-2.0, 7.0]])+(1.-msk_arg1)*numpy.array([[4.0, 1.0], [-1.0, -6.0], [4.0, -2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[2.0, -6.0, 2.0], [-5.0, 1.0, -3.0]], [[2.0, 6.0, 2.0], [1.0, 2.0, 1.0]]], [[[5.0, -3.0, -3.0], [7.0, -5.0, 1.0]], [[0.0, 0.0, -5.0], [-2.0, -7.0, -7.0]]], [[[0.0, 6.0, 4.0], [-6.0, 5.0, -6.0]], [[7.0, -7.0, 0.0], [6.0, 6.0, -5.0]]], [[[1.0, -5.0, 1.0], [-2.0, 0.0, 2.0]], [[-4.0, -7.0, -5.0], [7.0, -7.0, -6.0]]], [[[-1.0, 4.0, 0.0], [-4.0, -1.0, 0.0]], [[0.0, 5.0, 4.0], [-2.0, -1.0, -5.0]]], [[[-1.0, -2.0, -6.0], [0.0, 4.0, -7.0]], [[-3.0, -5.0, 6.0], [-2.0, 2.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, 24.0], [5.0, -46.0]], [[-10.0, 24.0], [-4.0, 12.0]]], [[[9.0, 4.0], [3.0, 42.0]], [[10.0, -35.0], [21.0, -59.0]]], [[[-14.0, 28.0], [7.0, -72.0]], [[7.0, 35.0], [4.0, -5.0]]], [[[3.0, 12.0], [-4.0, 4.0]], [[17.0, -55.0], [19.0, -7.0]]], [[[-4.0, -5.0], [1.0, -20.0]], [[-13.0, 28.0], [11.0, -45.0]]], [[[14.0, -47.0], [10.0, -49.0]], [[-7.0, 27.0], [-6.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[22.0, 34.0], [-33.0, -5.0]], [[10.0, -38.0], [6.0, -13.0]]], [[[11.0, 29.0], [37.0, 35.0]], [[-20.0, 10.0], [-29.0, 54.0]]], [[[10.0, -44.0], [-53.0, -24.0]], [[35.0, 49.0], [-2.0, -20.0]]], [[[13.0, 29.0], [0.0, -6.0]], [[-29.0, 48.0], [11.0, 61.0]]], [[[-8.0, -25.0], [-15.0, 2.0]], [[11.0, -38.0], [-27.0, 14.0]]], [[[-26.0, 23.0], [-32.0, -10.0]], [[17.0, 15.0], [-2.0, -18.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_float_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=-5.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[7.0, 0.0, -2.0, -4.0], [2.0, -3.0, -5.0, -3.0], [3.0, 5.0, -6.0, -1.0]], [[-3.0, -1.0, -5.0, 4.0], [1.0, -3.0, -6.0, 3.0], [-3.0, -6.0, 6.0, 5.0]]], [[[-4.0, -1.0, -2.0, -7.0], [3.0, 2.0, 0.0, 4.0], [4.0, -2.0, 7.0, -5.0]], [[6.0, -3.0, -7.0, -1.0], [-1.0, -3.0, -5.0, 5.0], [-5.0, -2.0, 1.0, -6.0]]], [[[0.0, -3.0, -3.0, 3.0], [-5.0, 7.0, -4.0, -1.0], [4.0, -4.0, 0.0, 5.0]], [[1.0, 5.0, 6.0, 2.0], [-2.0, -3.0, 5.0, -1.0], [7.0, 7.0, -1.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-35.0, -0.0, 10.0, 20.0], [-10.0, 15.0, 25.0, 15.0], [-15.0, -25.0, 30.0, 5.0]], [[15.0, 5.0, 25.0, -20.0], [-5.0, 15.0, 30.0, -15.0], [15.0, 30.0, -30.0, -25.0]]], [[[20.0, 5.0, 10.0, 35.0], [-15.0, -10.0, -0.0, -20.0], [-20.0, 10.0, -35.0, 25.0]], [[-30.0, 15.0, 35.0, 5.0], [5.0, 15.0, 25.0, -25.0], [25.0, 10.0, -5.0, 30.0]]], [[[-0.0, 15.0, 15.0, -15.0], [25.0, -35.0, 20.0, 5.0], [-20.0, 20.0, -0.0, -25.0]], [[-5.0, -25.0, -30.0, -10.0], [10.0, 15.0, -25.0, 5.0], [-35.0, -35.0, 5.0, 30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_array_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[4.0, 0.0, 2.0, 6.0], [-3.0, 2.0, 4.0, 6.0], [2.0, 6.0, -1.0, -1.0]], [[-5.0, -4.0, -5.0, 0.0], [-2.0, -4.0, -7.0, -2.0], [0.0, 0.0, -3.0, -5.0]]], [[[-6.0, 7.0, 0.0, -4.0], [-2.0, 2.0, -6.0, 6.0], [-3.0, -6.0, 3.0, 2.0]], [[3.0, -5.0, -1.0, 1.0], [0.0, 5.0, -5.0, 0.0], [1.0, -2.0, 5.0, 7.0]]], [[[4.0, 2.0, 1.0, 1.0], [0.0, -2.0, 6.0, -5.0], [0.0, -3.0, -6.0, -7.0]], [[-5.0, -3.0, -1.0, -3.0], [3.0, -4.0, -3.0, 0.0], [-2.0, -6.0, -4.0, 3.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0]], [[-0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0]]], [[[-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]], [[0.0, -0.0, -0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0]], [[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[4.0, 1.0, 2.0, 6.0], [4.0, 6.0, -2.0, -2.0], [-6.0, 1.0, -5.0, -5.0]], [[3.0, -6.0, -7.0, 5.0], [3.0, -4.0, 1.0, -5.0], [-5.0, 5.0, -5.0, -7.0]]], [[[7.0, -1.0, -2.0, -3.0], [0.0, 0.0, -7.0, -6.0], [-6.0, 2.0, -1.0, 3.0]], [[2.0, 1.0, -3.0, 2.0], [2.0, -7.0, 4.0, 7.0], [3.0, 1.0, 0.0, 6.0]]], [[[0.0, -5.0, 5.0, 1.0], [0.0, -1.0, -5.0, -5.0], [5.0, 6.0, -6.0, -6.0]], [[-1.0, 4.0, -5.0, 3.0], [0.0, -5.0, -1.0, 0.0], [2.0, -1.0, 5.0, -5.0]]]])
      s1=numpy.array(0.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, -0.0]], [[0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0]]], [[[0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0]], [[0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0]]], [[[0.0, -0.0, 0.0, 0.0], [0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0]], [[-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, -0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_constData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(-1.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[7.0, 2.0, 6.0, -4.0], [4.0, -7.0, -4.0, -6.0], [6.0, -2.0, -7.0, 0.0]], [[6.0, -4.0, -1.0, -3.0], [4.0, 3.0, -2.0, -1.0], [-3.0, -4.0, 0.0, 4.0]]], [[[5.0, 5.0, 1.0, -5.0], [-5.0, -6.0, 0.0, 7.0], [-6.0, 0.0, 7.0, 5.0]], [[-3.0, 7.0, 5.0, -2.0], [-7.0, -7.0, -4.0, -6.0], [-6.0, 6.0, 3.0, 2.0]]], [[[4.0, 1.0, 6.0, 2.0], [7.0, -6.0, 1.0, 0.0], [3.0, -4.0, 1.0, -7.0]], [[-6.0, 0.0, 4.0, -1.0], [-2.0, -1.0, -7.0, 5.0], [0.0, 1.0, -7.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-7.0, -2.0, -6.0, 4.0], [-4.0, 7.0, 4.0, 6.0], [-6.0, 2.0, 7.0, -0.0]], [[-6.0, 4.0, 1.0, 3.0], [-4.0, -3.0, 2.0, 1.0], [3.0, 4.0, -0.0, -4.0]]], [[[-5.0, -5.0, -1.0, 5.0], [5.0, 6.0, -0.0, -7.0], [6.0, -0.0, -7.0, -5.0]], [[3.0, -7.0, -5.0, 2.0], [7.0, 7.0, 4.0, 6.0], [6.0, -6.0, -3.0, -2.0]]], [[[-4.0, -1.0, -6.0, -2.0], [-7.0, 6.0, -1.0, -0.0], [-3.0, 4.0, -1.0, 7.0]], [[6.0, -0.0, -4.0, 1.0], [2.0, 1.0, 7.0, -5.0], [-0.0, -1.0, 7.0, 2.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_Symbol_rank4_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(2.0)+(1-msk_arg1)*(4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-6.0, -7.0, -4.0, 1.0], [0.0, 2.0, 0.0, 7.0], [5.0, -6.0, -1.0, -1.0]], [[-3.0, -1.0, -5.0, 4.0], [-2.0, 7.0, 0.0, -4.0], [-1.0, 1.0, 6.0, -4.0]]], [[[-4.0, -4.0, 1.0, 0.0], [-1.0, 0.0, 5.0, -5.0], [-7.0, 6.0, -6.0, -2.0]], [[-4.0, -4.0, -7.0, -7.0], [-2.0, 3.0, 1.0, -3.0], [-1.0, 1.0, -5.0, 0.0]]], [[[4.0, 1.0, 1.0, -7.0], [6.0, 3.0, -7.0, 6.0], [0.0, 7.0, 6.0, 1.0]], [[-7.0, -6.0, 0.0, 3.0], [0.0, -1.0, -5.0, 1.0], [-6.0, -4.0, 2.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-12.0, -14.0, -8.0, 2.0], [0.0, 4.0, 0.0, 14.0], [10.0, -12.0, -2.0, -2.0]], [[-6.0, -2.0, -10.0, 8.0], [-4.0, 14.0, 0.0, -8.0], [-2.0, 2.0, 12.0, -8.0]]], [[[-8.0, -8.0, 2.0, 0.0], [-2.0, 0.0, 10.0, -10.0], [-14.0, 12.0, -12.0, -4.0]], [[-8.0, -8.0, -14.0, -14.0], [-4.0, 6.0, 2.0, -6.0], [-2.0, 2.0, -10.0, 0.0]]], [[[8.0, 2.0, 2.0, -14.0], [12.0, 6.0, -14.0, 12.0], [0.0, 14.0, 12.0, 2.0]], [[-14.0, -12.0, 0.0, 6.0], [0.0, -2.0, -10.0, 2.0], [-12.0, -8.0, 4.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[-24.0, -28.0, -16.0, 4.0], [0.0, 8.0, 0.0, 28.0], [20.0, -24.0, -4.0, -4.0]], [[-12.0, -4.0, -20.0, 16.0], [-8.0, 28.0, 0.0, -16.0], [-4.0, 4.0, 24.0, -16.0]]], [[[-16.0, -16.0, 4.0, 0.0], [-4.0, 0.0, 20.0, -20.0], [-28.0, 24.0, -24.0, -8.0]], [[-16.0, -16.0, -28.0, -28.0], [-8.0, 12.0, 4.0, -12.0], [-4.0, 4.0, -20.0, 0.0]]], [[[16.0, 4.0, 4.0, -28.0], [24.0, 12.0, -28.0, 24.0], [0.0, 28.0, 24.0, 4.0]], [[-28.0, -24.0, 0.0, 12.0], [0.0, -4.0, -20.0, 4.0], [-24.0, -16.0, 8.0, 8.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_float_rank0_offset0(self):
      arg0=Data(2.0,self.functionspace)
      arg1=4.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(8.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_array_rank0_offset0(self):
      arg0=Data(5.0,self.functionspace)
      arg1=numpy.array(6.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(30.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank1_offset1(self):
      arg0=Data(numpy.array([-7.0, -7.0, 5.0]),self.functionspace)
      arg1=numpy.array([-6.0, -1.0, 7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(84.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank2_offset2(self):
      arg0=Data(numpy.array([[1.0, -3.0, 5.0], [-1.0, 0.0, -2.0]]),self.functionspace)
      arg1=numpy.array([[-5.0, -3.0, 3.0], [5.0, -7.0, -2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(18.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[6.0, -2.0, 2.0], [-5.0, -5.0, -3.0], [-7.0, 2.0, 3.0], [-5.0, 2.0, 3.0]], [[7.0, 6.0, 3.0], [0.0, 1.0, 7.0], [2.0, 1.0, 2.0], [3.0, 4.0, 0.0]]]),self.functionspace)
      arg1=numpy.array([[[-4.0, -1.0, 5.0], [-2.0, 3.0, -2.0], [0.0, 6.0, 3.0], [-6.0, -1.0, -3.0]], [[7.0, 0.0, 3.0], [-4.0, -5.0, 5.0], [-4.0, -4.0, 0.0], [3.0, 5.0, -4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(134.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank4_offset4(self):
      arg0=Data(numpy.array([[[[2.0, 5.0], [-7.0, 6.0], [-3.0, 0.0]], [[3.0, -2.0], [0.0, 7.0], [-3.0, -6.0]]], [[[2.0, -7.0], [-6.0, -1.0], [-2.0, 3.0]], [[-6.0, -4.0], [0.0, -7.0], [0.0, 1.0]]], [[[-2.0, -6.0], [-6.0, 1.0], [2.0, 1.0]], [[3.0, 0.0], [-1.0, -1.0], [3.0, -3.0]]], [[[0.0, -5.0], [-2.0, -2.0], [5.0, 4.0]], [[-5.0, 4.0], [-2.0, 0.0], [-1.0, -1.0]]]]),self.functionspace)
      arg1=numpy.array([[[[2.0, 1.0], [4.0, 7.0], [-5.0, 0.0]], [[0.0, 0.0], [-5.0, 2.0], [-6.0, 1.0]]], [[[-7.0, 6.0], [-4.0, -6.0], [-5.0, 2.0]], [[7.0, -2.0], [-5.0, -6.0], [5.0, 0.0]]], [[[2.0, 6.0], [5.0, -7.0], [0.0, 1.0]], [[2.0, 1.0], [2.0, -3.0], [4.0, 7.0]]], [[[-6.0, -1.0], [4.0, 5.0], [-3.0, 7.0]], [[-4.0, -1.0], [5.0, -7.0], [-4.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(-6.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_array_rank1_offset0(self):
      arg0=Data(-6.0,self.functionspace)
      arg1=numpy.array([-5.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([30.0, -24.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank2_offset1(self):
      arg0=Data(numpy.array([7.0, -5.0, 3.0]),self.functionspace)
      arg1=numpy.array([[-7.0, 6.0], [3.0, 7.0], [3.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-55.0, 16.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank3_offset2(self):
      arg0=Data(numpy.array([[-2.0, 4.0, 5.0], [0.0, -3.0, 0.0]]),self.functionspace)
      arg1=numpy.array([[[4.0, -1.0], [1.0, 6.0], [-2.0, -1.0]], [[7.0, 5.0], [-7.0, 0.0], [7.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([7.0, 21.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[0.0, 6.0, 2.0], [-2.0, 3.0, -2.0], [3.0, -7.0, -4.0], [2.0, 1.0, -5.0]], [[-1.0, 1.0, -1.0], [-7.0, 0.0, -3.0], [-4.0, -4.0, 0.0], [3.0, 2.0, 0.0]]]),self.functionspace)
      arg1=numpy.array([[[[0.0, -4.0], [4.0, -7.0], [-2.0, -1.0]], [[-2.0, -2.0], [5.0, 0.0], [-5.0, -7.0]], [[-5.0, -3.0], [0.0, 6.0], [-4.0, 7.0]], [[2.0, 1.0], [0.0, 6.0], [-3.0, -3.0]]], [[[-7.0, 0.0], [5.0, -3.0], [-2.0, -1.0]], [[-3.0, 4.0], [2.0, -6.0], [-6.0, 4.0]], [[-4.0, -1.0], [-1.0, 2.0], [1.0, -7.0]], [[7.0, 3.0], [4.0, 5.0], [0.0, 5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([171.0, -109.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_array_rank2_offset0(self):
      arg0=Data(0.0,self.functionspace)
      arg1=numpy.array([[5.0, -3.0, 1.0, 0.0, 6.0], [-5.0, -6.0, -7.0, -2.0, -2.0], [-6.0, -6.0, -6.0, 6.0, -6.0], [-4.0, 7.0, -5.0, -5.0, -3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[0.0, -0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0, -0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank3_offset1(self):
      arg0=Data(numpy.array([3.0, -2.0, -3.0]),self.functionspace)
      arg1=numpy.array([[[-6.0, -2.0, -3.0, 5.0, -4.0], [-5.0, 6.0, 6.0, 0.0, -1.0], [0.0, -5.0, 6.0, -5.0, 0.0], [1.0, -2.0, -4.0, 7.0, 4.0]], [[-6.0, -2.0, -6.0, -3.0, -7.0], [-5.0, 2.0, -2.0, 0.0, 4.0], [4.0, -3.0, 0.0, -1.0, -6.0], [7.0, 0.0, -3.0, 6.0, 0.0]], [[0.0, -6.0, -1.0, 5.0, -4.0], [0.0, 3.0, 2.0, 2.0, 0.0], [3.0, -6.0, 1.0, 0.0, 6.0], [1.0, 0.0, -1.0, -3.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-6.0, 16.0, 6.0, 6.0, 14.0], [-5.0, 5.0, 16.0, -6.0, -11.0], [-17.0, 9.0, 15.0, -13.0, -6.0], [-14.0, -6.0, -3.0, 18.0, 27.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank4_offset2(self):
      arg0=Data(numpy.array([[0.0, -2.0, -1.0], [-3.0, 6.0, 7.0]]),self.functionspace)
      arg1=numpy.array([[[[1.0, -3.0, 7.0, 0.0, -7.0], [2.0, 4.0, -4.0, -7.0, 1.0], [6.0, 6.0, 2.0, 1.0, 0.0], [7.0, -2.0, 4.0, 0.0, -7.0]], [[6.0, -7.0, 4.0, 4.0, 3.0], [-3.0, 2.0, -6.0, 0.0, -5.0], [3.0, 1.0, 1.0, 1.0, -6.0], [7.0, 5.0, -6.0, 2.0, -5.0]], [[0.0, -4.0, 0.0, 0.0, -5.0], [7.0, 0.0, -4.0, -6.0, -6.0], [-4.0, -2.0, 3.0, 2.0, -3.0], [-4.0, 0.0, 5.0, -2.0, -6.0]]], [[[-1.0, 7.0, -5.0, 7.0, 7.0], [5.0, -3.0, 5.0, -4.0, 2.0], [-2.0, 4.0, 5.0, 5.0, 7.0], [-3.0, 3.0, 7.0, -2.0, -3.0]], [[-7.0, -4.0, 5.0, 5.0, -3.0], [-3.0, 2.0, -6.0, -5.0, 5.0], [-6.0, -2.0, 0.0, -1.0, -6.0], [7.0, -1.0, 0.0, 1.0, 2.0]], [[1.0, 4.0, 0.0, -7.0, 3.0], [-1.0, 0.0, 0.0, 1.0, -2.0], [-3.0, 2.0, 0.0, 2.0, -1.0], [5.0, -7.0, -4.0, 0.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-44.0, 1.0, 37.0, -48.0, -19.0], [-41.0, 17.0, -35.0, -5.0, 26.0], [-53.0, -10.0, -20.0, -11.0, -49.0], [76.0, -74.0, -42.0, 10.0, 37.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_array_rank3_offset0(self):
      arg0=Data(6.0,self.functionspace)
      arg1=numpy.array([[[-6.0, -2.0], [0.0, 7.0]], [[-3.0, 7.0], [-5.0, 3.0]], [[-6.0, -3.0], [0.0, 3.0]], [[-7.0, 0.0], [2.0, -1.0]], [[6.0, 1.0], [5.0, 4.0]], [[7.0, -7.0], [-4.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-36.0, -12.0], [0.0, 42.0]], [[-18.0, 42.0], [-30.0, 18.0]], [[-36.0, -18.0], [0.0, 18.0]], [[-42.0, 0.0], [12.0, -6.0]], [[36.0, 6.0], [30.0, 24.0]], [[42.0, -42.0], [-24.0, 6.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank4_offset1(self):
      arg0=Data(numpy.array([-1.0, -5.0, 3.0]),self.functionspace)
      arg1=numpy.array([[[[1.0, -2.0], [3.0, 3.0]], [[-6.0, 5.0], [7.0, 7.0]], [[-4.0, 6.0], [-5.0, 5.0]], [[-1.0, 4.0], [-3.0, 2.0]], [[5.0, -1.0], [-2.0, 3.0]], [[-4.0, 2.0], [-5.0, -2.0]]], [[[-2.0, 4.0], [1.0, -6.0]], [[-1.0, 0.0], [-5.0, 0.0]], [[1.0, -5.0], [-7.0, 0.0]], [[3.0, -2.0], [1.0, 4.0]], [[-7.0, -4.0], [-3.0, -2.0]], [[6.0, 4.0], [1.0, -5.0]]], [[[7.0, 7.0], [2.0, -2.0]], [[6.0, -2.0], [-7.0, 3.0]], [[0.0, -5.0], [-4.0, 6.0]], [[-2.0, -2.0], [5.0, -3.0]], [[-4.0, -4.0], [1.0, -3.0]], [[3.0, -3.0], [-5.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[30.0, 3.0], [-2.0, 21.0]], [[29.0, -11.0], [-3.0, 2.0]], [[-1.0, 4.0], [28.0, 13.0]], [[-20.0, 0.0], [13.0, -31.0]], [[18.0, 9.0], [20.0, -2.0]], [[-17.0, -31.0], [-15.0, 27.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_array_rank4_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      arg1=numpy.array([[[[5.0, -7.0, 6.0, -7.0], [6.0, 3.0, -6.0, -5.0], [0.0, -7.0, 6.0, 6.0]], [[6.0, 3.0, 0.0, -1.0], [6.0, 0.0, 0.0, -3.0], [-5.0, -3.0, 7.0, 0.0]]], [[[-7.0, -2.0, 0.0, 0.0], [7.0, -2.0, 6.0, 2.0], [-3.0, -7.0, 1.0, -1.0]], [[2.0, -7.0, 7.0, -3.0], [-5.0, -1.0, 2.0, -2.0], [-2.0, -4.0, -7.0, 2.0]]], [[[3.0, 6.0, -6.0, 7.0], [5.0, 0.0, 5.0, -5.0], [-7.0, 6.0, -2.0, -6.0]], [[6.0, -2.0, 3.0, 2.0], [4.0, 1.0, 1.0, -4.0], [-4.0, 3.0, -7.0, -1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-35.0, 49.0, -42.0, 49.0], [-42.0, -21.0, 42.0, 35.0], [-0.0, 49.0, -42.0, -42.0]], [[-42.0, -21.0, -0.0, 7.0], [-42.0, -0.0, -0.0, 21.0], [35.0, 21.0, -49.0, -0.0]]], [[[49.0, 14.0, -0.0, -0.0], [-49.0, 14.0, -42.0, -14.0], [21.0, 49.0, -7.0, 7.0]], [[-14.0, 49.0, -49.0, 21.0], [35.0, 7.0, -14.0, 14.0], [14.0, 28.0, 49.0, -14.0]]], [[[-21.0, -42.0, 42.0, -49.0], [-35.0, -0.0, -35.0, 35.0], [49.0, -42.0, 14.0, 42.0]], [[-42.0, 14.0, -21.0, -14.0], [-28.0, -7.0, -7.0, 28.0], [28.0, -21.0, 49.0, 7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_Symbol_rank0_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(1.0)
      sub=res.substitute({arg1:s1})
      ref=Data(-7.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([2.0, 3.0, -2.0]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([1.0, -7.0, -5.0])
      sub=res.substitute({arg1:s1})
      ref=Data(-9.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[-3.0, 0.0, 4.0], [0.0, -6.0, -6.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[2.0, 5.0, -3.0], [-6.0, 2.0, 1.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(-36.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[5.0, -6.0, 4.0], [-3.0, -7.0, 2.0], [-4.0, -7.0, -5.0], [-5.0, -2.0, 4.0]], [[3.0, 7.0, 3.0], [7.0, 2.0, -7.0], [-5.0, -6.0, -7.0], [0.0, 5.0, -4.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[7.0, 0.0, -7.0], [3.0, 6.0, 0.0], [0.0, 7.0, -7.0], [2.0, -3.0, 3.0]], [[7.0, -4.0, 3.0], [1.0, -6.0, 4.0], [2.0, -1.0, -4.0], [-3.0, 6.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(-11.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-2.0, 4.0], [-6.0, 2.0], [-1.0, 7.0]], [[-4.0, 4.0], [-1.0, 7.0], [0.0, -3.0]]], [[[7.0, 0.0], [4.0, 7.0], [-4.0, 1.0]], [[0.0, 2.0], [5.0, -1.0], [2.0, -1.0]]], [[[-6.0, -1.0], [-4.0, 7.0], [-2.0, 0.0]], [[0.0, -4.0], [4.0, 1.0], [-7.0, 7.0]]], [[[-6.0, 3.0], [1.0, -5.0], [3.0, 6.0]], [[-4.0, 0.0], [-4.0, 3.0], [5.0, 4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[7.0, 6.0], [0.0, 7.0], [-2.0, 7.0]], [[3.0, -4.0], [0.0, 4.0], [3.0, 0.0]]], [[[0.0, -1.0], [-6.0, -2.0], [3.0, -5.0]], [[7.0, -6.0], [-2.0, 1.0], [0.0, 4.0]]], [[[7.0, -6.0], [-5.0, 4.0], [1.0, 0.0]], [[-4.0, 7.0], [0.0, 2.0], [2.0, 1.0]]], [[[0.0, 0.0], [-5.0, 3.0], [-6.0, 0.0]], [[2.0, -5.0], [-2.0, 7.0], [6.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(-5.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_Symbol_rank1_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-5.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([5.0, -4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([-1.0, -6.0, 6.0]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-4.0, -5.0], [4.0, 3.0], [6.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([16.0, -49.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[0.0, -6.0, 0.0], [1.0, 6.0, -7.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-1.0, -3.0], [3.0, -7.0], [0.0, 0.0]], [[7.0, 2.0], [1.0, 5.0], [6.0, 6.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-47.0, 32.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[-3.0, 2.0, 2.0], [0.0, 7.0, 1.0], [3.0, 0.0, 0.0], [7.0, 0.0, 7.0]], [[-6.0, -7.0, -6.0], [-3.0, 5.0, 7.0], [-3.0, 6.0, -5.0], [1.0, -1.0, 7.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-4.0, 3.0], [-1.0, 1.0], [2.0, 7.0]], [[6.0, -7.0], [1.0, 4.0], [7.0, 0.0]], [[1.0, -2.0], [-7.0, -3.0], [1.0, 3.0]], [[0.0, -5.0], [1.0, -6.0], [0.0, -7.0]]], [[[6.0, 0.0], [3.0, 0.0], [-3.0, -5.0]], [[-4.0, 2.0], [5.0, -4.0], [3.0, 6.0]], [[1.0, -5.0], [-5.0, 0.0], [-6.0, 2.0]], [[-2.0, 3.0], [0.0, -7.0], [-5.0, 2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([10.0, 20.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_Symbol_rank2_offset0(self):
      arg0=Data(0.0,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-1.0, -7.0, 0.0, -3.0, -2.0], [6.0, 7.0, -4.0, -5.0, 4.0], [5.0, -7.0, 5.0, -3.0, -7.0], [1.0, 0.0, -5.0, 6.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-0.0, -0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0, -0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([-3.0, -5.0, -4.0]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[4.0, 1.0, -7.0, 0.0, -6.0], [4.0, 0.0, -5.0, 1.0, 2.0], [3.0, 7.0, 7.0, -6.0, 5.0], [0.0, -6.0, 4.0, -6.0, -3.0]], [[2.0, 0.0, -1.0, 1.0, -6.0], [3.0, -1.0, 1.0, 7.0, 0.0], [1.0, -7.0, -5.0, 5.0, -3.0], [-3.0, 7.0, 2.0, -4.0, -3.0]], [[0.0, 2.0, 4.0, -1.0, 0.0], [5.0, 0.0, -4.0, -2.0, -2.0], [-1.0, 4.0, -1.0, -4.0, -7.0], [0.0, -7.0, 6.0, 0.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-22.0, -11.0, 10.0, -1.0, 48.0], [-47.0, 5.0, 26.0, -30.0, 2.0], [-10.0, -2.0, 8.0, 9.0, 28.0], [15.0, 11.0, -46.0, 38.0, 20.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[-2.0, 6.0, 7.0], [-6.0, 4.0, -7.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[2.0, 5.0, 6.0, -4.0, 7.0], [2.0, -5.0, 7.0, -1.0, 0.0], [-3.0, 0.0, 6.0, -6.0, -3.0], [-2.0, -5.0, 4.0, 6.0, -2.0]], [[-2.0, 3.0, -4.0, -3.0, 6.0], [-4.0, 5.0, 5.0, 0.0, -6.0], [-2.0, 0.0, 5.0, -7.0, 1.0], [1.0, -7.0, -3.0, -5.0, 0.0]], [[-2.0, 0.0, -4.0, -5.0, 0.0], [-3.0, -3.0, 1.0, -6.0, -4.0], [-2.0, -2.0, -5.0, -3.0, 6.0], [4.0, 2.0, -4.0, 3.0, 0.0]]], [[[7.0, 0.0, -6.0, 0.0, 6.0], [-6.0, -7.0, 3.0, 6.0, -1.0], [5.0, -7.0, 0.0, -1.0, 1.0], [-6.0, -2.0, 6.0, 0.0, -7.0]], [[6.0, 7.0, 0.0, -4.0, 5.0], [7.0, -3.0, -3.0, -2.0, -2.0], [1.0, -4.0, -6.0, 5.0, 6.0], [-4.0, -6.0, -2.0, -3.0, 0.0]], [[3.0, -7.0, 0.0, -4.0, 7.0], [-3.0, 0.0, 1.0, 4.0, 0.0], [5.0, -6.0, -4.0, 2.0, 0.0], [1.0, 3.0, -3.0, -3.0, 2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-69.0, 85.0, -28.0, -33.0, -43.0], [36.0, 49.0, -14.0, -112.0, -66.0], [-81.0, 54.0, -13.0, -39.0, 72.0], [51.0, -51.0, -77.0, -12.0, 32.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_Symbol_rank3_offset0(self):
      arg0=Data(-5.0,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-6.0, -2.0], [3.0, 4.0]], [[-3.0, 4.0], [3.0, 3.0]], [[2.0, -4.0], [3.0, -1.0]], [[0.0, -6.0], [-2.0, 6.0]], [[-4.0, -3.0], [-1.0, 7.0]], [[-4.0, 2.0], [0.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[30.0, 10.0], [-15.0, -20.0]], [[15.0, -20.0], [-15.0, -15.0]], [[-10.0, 20.0], [-15.0, 5.0]], [[-0.0, 30.0], [10.0, -30.0]], [[20.0, 15.0], [5.0, -35.0]], [[20.0, -10.0], [-0.0, -25.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([-5.0, -3.0, 2.0]),self.functionspace)
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[7.0, -2.0], [3.0, -3.0]], [[4.0, 1.0], [-4.0, -6.0]], [[2.0, 6.0], [-6.0, 6.0]], [[1.0, 1.0], [-5.0, 0.0]], [[0.0, 6.0], [3.0, 1.0]], [[-3.0, -5.0], [-1.0, 7.0]]], [[[4.0, 0.0], [-4.0, 1.0]], [[-7.0, 0.0], [0.0, -5.0]], [[-5.0, 5.0], [-4.0, 4.0]], [[2.0, 3.0], [5.0, -4.0]], [[-3.0, 1.0], [7.0, 5.0]], [[-5.0, 2.0], [2.0, -7.0]]], [[[0.0, 5.0], [-4.0, 4.0]], [[3.0, 1.0], [-3.0, -1.0]], [[2.0, -4.0], [0.0, 1.0]], [[-4.0, 0.0], [-7.0, 6.0]], [[-6.0, 7.0], [1.0, -2.0]], [[-2.0, 5.0], [2.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-47.0, 20.0], [-11.0, 20.0]], [[7.0, -3.0], [14.0, 43.0]], [[9.0, -53.0], [42.0, -40.0]], [[-19.0, -14.0], [-4.0, 24.0]], [[-3.0, -19.0], [-34.0, -24.0]], [[26.0, 29.0], [3.0, -14.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_Symbol_rank4_offset0(self):
      arg0=Data(6.0,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-5.0, 6.0, -1.0, -2.0], [5.0, 0.0, -6.0, 0.0], [0.0, 2.0, -2.0, 1.0]], [[-1.0, -6.0, -7.0, -6.0], [6.0, -3.0, 5.0, 5.0], [-6.0, -5.0, 5.0, 1.0]]], [[[7.0, -4.0, 0.0, 2.0], [-3.0, -7.0, 1.0, -5.0], [-6.0, -6.0, 2.0, -5.0]], [[3.0, 6.0, 0.0, -3.0], [5.0, 7.0, -4.0, 2.0], [4.0, 6.0, -7.0, -6.0]]], [[[-6.0, -2.0, -1.0, 1.0], [-4.0, 2.0, -4.0, 1.0], [5.0, -3.0, -5.0, -7.0]], [[3.0, -4.0, 6.0, 5.0], [-5.0, -1.0, 6.0, 6.0], [2.0, -1.0, 2.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-30.0, 36.0, -6.0, -12.0], [30.0, 0.0, -36.0, 0.0], [0.0, 12.0, -12.0, 6.0]], [[-6.0, -36.0, -42.0, -36.0], [36.0, -18.0, 30.0, 30.0], [-36.0, -30.0, 30.0, 6.0]]], [[[42.0, -24.0, 0.0, 12.0], [-18.0, -42.0, 6.0, -30.0], [-36.0, -36.0, 12.0, -30.0]], [[18.0, 36.0, 0.0, -18.0], [30.0, 42.0, -24.0, 12.0], [24.0, 36.0, -42.0, -36.0]]], [[[-36.0, -12.0, -6.0, 6.0], [-24.0, 12.0, -24.0, 6.0], [30.0, -18.0, -30.0, -42.0]], [[18.0, -24.0, 36.0, 30.0], [-30.0, -6.0, 36.0, 36.0], [12.0, -6.0, 12.0, 36.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_constData_rank0_offset0(self):
      arg0=Data(-4.0,self.functionspace)
      arg1=Data(4.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-16.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank1_offset1(self):
      arg0=Data(numpy.array([7.0, 5.0, -4.0]),self.functionspace)
      arg1=Data(numpy.array([7.0, -2.0, 0.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(39.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[0.0, 5.0, 7.0], [-5.0, 2.0, -6.0]]),self.functionspace)
      arg1=Data(numpy.array([[0.0, -1.0, -5.0], [-7.0, -7.0, 4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(-43.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[4.0, -2.0, 4.0], [-3.0, -7.0, -6.0], [5.0, -2.0, 0.0], [-3.0, -3.0, -3.0]], [[-3.0, -1.0, 7.0], [2.0, 7.0, 6.0], [3.0, 7.0, 4.0], [0.0, -3.0, 3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[5.0, -3.0, 6.0], [-6.0, -3.0, -3.0], [3.0, -5.0, 0.0], [7.0, -4.0, -5.0]], [[2.0, 7.0, -4.0], [4.0, -4.0, -2.0], [-4.0, 3.0, -5.0], [4.0, -3.0, 3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(72.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[3.0, -3.0], [-1.0, -4.0], [5.0, -7.0]], [[7.0, -1.0], [7.0, -3.0], [5.0, -4.0]]], [[[6.0, -3.0], [7.0, -7.0], [3.0, 0.0]], [[-2.0, -4.0], [0.0, -1.0], [-1.0, 5.0]]], [[[7.0, -1.0], [5.0, -4.0], [6.0, 6.0]], [[6.0, 1.0], [-2.0, 0.0], [1.0, -2.0]]], [[[2.0, 7.0], [-4.0, -2.0], [4.0, -2.0]], [[-6.0, -7.0], [4.0, -5.0], [3.0, 4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.0, -7.0], [-7.0, -4.0], [0.0, 0.0]], [[2.0, 1.0], [-6.0, 0.0], [-4.0, 6.0]]], [[[-6.0, -6.0], [-5.0, 0.0], [5.0, 5.0]], [[6.0, -3.0], [7.0, -4.0], [0.0, 2.0]]], [[[-2.0, 4.0], [4.0, 0.0], [6.0, -2.0]], [[3.0, 2.0], [-7.0, 2.0], [-4.0, 4.0]]], [[[1.0, -6.0], [-6.0, -5.0], [7.0, -2.0]], [[-5.0, 2.0], [-2.0, 0.0], [2.0, -7.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(10.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_constData_rank1_offset0(self):
      arg0=Data(0.0,self.functionspace)
      arg1=Data(numpy.array([-2.0, 4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-0.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank2_offset1(self):
      arg0=Data(numpy.array([6.0, -5.0, 0.0]),self.functionspace)
      arg1=Data(numpy.array([[-3.0, -2.0], [7.0, 4.0], [2.0, -5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-53.0, -32.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[0.0, -6.0, 7.0], [0.0, -5.0, 4.0]]),self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 3.0], [7.0, -1.0], [0.0, 1.0]], [[-4.0, -3.0], [0.0, 2.0], [3.0, 1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-30.0, 7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[4.0, -6.0, -5.0], [3.0, 2.0, 6.0], [-7.0, -3.0, 2.0], [7.0, 1.0, 5.0]], [[-5.0, -6.0, -3.0], [6.0, 0.0, 1.0], [4.0, 7.0, 4.0], [-5.0, -3.0, -5.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.0, -3.0], [4.0, 3.0], [-3.0, 7.0]], [[6.0, 6.0], [-3.0, -7.0], [6.0, 0.0]], [[-2.0, -3.0], [-7.0, -7.0], [3.0, -6.0]], [[-1.0, 3.0], [3.0, 7.0], [7.0, 3.0]]], [[[-3.0, 3.0], [0.0, -1.0], [-1.0, 6.0]], [[0.0, 2.0], [2.0, -4.0], [-5.0, -5.0]], [[0.0, 1.0], [0.0, 4.0], [-3.0, 7.0]], [[7.0, 2.0], [6.0, -2.0], [0.0, 1.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([63.0, 43.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_constData_rank2_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      arg1=Data(numpy.array([[0.0, -7.0, -1.0, 7.0, 6.0], [0.0, 7.0, -4.0, -4.0, -5.0], [-5.0, 2.0, -2.0, -6.0, -6.0], [2.0, -3.0, 0.0, 3.0, 4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-0.0, 7.0, 1.0, -7.0, -6.0], [-0.0, -7.0, 4.0, 4.0, 5.0], [5.0, -2.0, 2.0, 6.0, 6.0], [-2.0, 3.0, -0.0, -3.0, -4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank3_offset1(self):
      arg0=Data(numpy.array([0.0, -2.0, 0.0]),self.functionspace)
      arg1=Data(numpy.array([[[3.0, 0.0, -2.0, -4.0, -2.0], [2.0, -3.0, -3.0, -7.0, 0.0], [7.0, -3.0, -3.0, 6.0, -1.0], [0.0, 3.0, -3.0, -5.0, 3.0]], [[0.0, -1.0, 0.0, 1.0, -3.0], [4.0, -4.0, 5.0, -4.0, 0.0], [-1.0, -7.0, -2.0, 2.0, 0.0], [3.0, -4.0, 4.0, 0.0, 6.0]], [[-3.0, -5.0, -5.0, -4.0, 4.0], [-3.0, 0.0, 2.0, -7.0, 5.0], [-1.0, -5.0, 4.0, -4.0, -4.0], [0.0, -1.0, 6.0, 1.0, 0.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[0.0, 2.0, 0.0, -2.0, 6.0], [-8.0, 8.0, -10.0, 8.0, 0.0], [2.0, 14.0, 4.0, -4.0, 0.0], [-6.0, 8.0, -8.0, 0.0, -12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[3.0, -5.0, 5.0], [0.0, 5.0, -7.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[0.0, 5.0, -4.0, 1.0, 6.0], [7.0, 0.0, -6.0, 7.0, -1.0], [-6.0, -5.0, 0.0, 2.0, 6.0], [-5.0, 5.0, 7.0, 6.0, -2.0]], [[-4.0, -7.0, 0.0, -6.0, 0.0], [-4.0, 5.0, 5.0, -2.0, -1.0], [5.0, -1.0, 1.0, -6.0, -2.0], [1.0, -2.0, 3.0, -4.0, -7.0]], [[4.0, 3.0, -7.0, 1.0, -3.0], [-5.0, 1.0, 3.0, 6.0, 4.0], [0.0, -5.0, 1.0, -2.0, 4.0], [0.0, -3.0, -6.0, 0.0, -4.0]]], [[[5.0, 0.0, 0.0, -1.0, -4.0], [7.0, 5.0, 1.0, -6.0, 1.0], [-5.0, -5.0, -3.0, 4.0, -2.0], [-5.0, 3.0, 2.0, 4.0, 5.0]], [[2.0, 7.0, 2.0, -2.0, -6.0], [6.0, 0.0, 6.0, -6.0, 4.0], [-4.0, 0.0, 4.0, 5.0, -3.0], [3.0, 1.0, 2.0, -4.0, 0.0]], [[-4.0, 2.0, 6.0, 1.0, -6.0], [-2.0, -2.0, -7.0, 6.0, 3.0], [-5.0, 7.0, 0.0, -6.0, -4.0], [-6.0, -1.0, -2.0, 0.0, -1.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[78.0, 86.0, -79.0, 21.0, 15.0], [60.0, -6.0, 51.0, -11.0, 21.0], [-28.0, -84.0, 20.0, 93.0, 61.0], [37.0, 22.0, 0.0, 18.0, 16.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_constData_rank3_offset0(self):
      arg0=Data(6.0,self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 5.0], [-4.0, 3.0]], [[-4.0, 2.0], [5.0, 7.0]], [[0.0, -6.0], [-7.0, 0.0]], [[-2.0, -3.0], [-5.0, -4.0]], [[-2.0, 5.0], [4.0, -3.0]], [[4.0, 1.0], [-7.0, -1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-12.0, 30.0], [-24.0, 18.0]], [[-24.0, 12.0], [30.0, 42.0]], [[0.0, -36.0], [-42.0, 0.0]], [[-12.0, -18.0], [-30.0, -24.0]], [[-12.0, 30.0], [24.0, -18.0]], [[24.0, 6.0], [-42.0, -6.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank4_offset1(self):
      arg0=Data(numpy.array([0.0, -7.0, 7.0]),self.functionspace)
      arg1=Data(numpy.array([[[[-7.0, 6.0], [3.0, -5.0]], [[2.0, 3.0], [-2.0, 1.0]], [[-2.0, 4.0], [6.0, -1.0]], [[5.0, 5.0], [4.0, 4.0]], [[4.0, 0.0], [5.0, 7.0]], [[-1.0, 6.0], [-2.0, -7.0]]], [[[3.0, -2.0], [3.0, -6.0]], [[0.0, -1.0], [-6.0, 0.0]], [[-3.0, 6.0], [5.0, 6.0]], [[4.0, 3.0], [-6.0, -3.0]], [[-5.0, 0.0], [-7.0, 0.0]], [[0.0, -4.0], [-3.0, 7.0]]], [[[-1.0, 6.0], [0.0, -1.0]], [[1.0, -6.0], [-4.0, -1.0]], [[5.0, 2.0], [-1.0, -6.0]], [[1.0, 3.0], [1.0, -6.0]], [[-7.0, -1.0], [2.0, 7.0]], [[2.0, -6.0], [5.0, -4.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-28.0, 56.0], [-21.0, 35.0]], [[7.0, -35.0], [14.0, -7.0]], [[56.0, -28.0], [-42.0, -84.0]], [[-21.0, 0.0], [49.0, -21.0]], [[-14.0, -7.0], [63.0, 49.0]], [[14.0, -14.0], [56.0, -77.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_constData_rank4_offset0(self):
      arg0=Data(-5.0,self.functionspace)
      arg1=Data(numpy.array([[[[5.0, 0.0, -2.0, -5.0], [-3.0, -5.0, -4.0, 6.0], [5.0, -4.0, -1.0, 3.0]], [[-6.0, 0.0, 6.0, 2.0], [6.0, -3.0, 4.0, 6.0], [-3.0, 2.0, 1.0, 3.0]]], [[[0.0, 4.0, 4.0, 4.0], [-4.0, -6.0, -6.0, 0.0], [6.0, 7.0, 7.0, -2.0]], [[-2.0, -5.0, 2.0, 3.0], [4.0, 2.0, -7.0, 6.0], [-7.0, 3.0, 5.0, 2.0]]], [[[-4.0, -1.0, -3.0, 6.0], [2.0, 7.0, 5.0, 2.0], [-6.0, -7.0, 0.0, -6.0]], [[-5.0, -1.0, 4.0, 4.0], [-4.0, 4.0, -6.0, 0.0], [-5.0, 3.0, -5.0, -2.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-25.0, -0.0, 10.0, 25.0], [15.0, 25.0, 20.0, -30.0], [-25.0, 20.0, 5.0, -15.0]], [[30.0, -0.0, -30.0, -10.0], [-30.0, 15.0, -20.0, -30.0], [15.0, -10.0, -5.0, -15.0]]], [[[-0.0, -20.0, -20.0, -20.0], [20.0, 30.0, 30.0, -0.0], [-30.0, -35.0, -35.0, 10.0]], [[10.0, 25.0, -10.0, -15.0], [-20.0, -10.0, 35.0, -30.0], [35.0, -15.0, -25.0, -10.0]]], [[[20.0, 5.0, 15.0, -30.0], [-10.0, -35.0, -25.0, -10.0], [30.0, 35.0, -0.0, 30.0]], [[25.0, 5.0, -20.0, -20.0], [20.0, -20.0, 30.0, -0.0], [25.0, -15.0, 25.0, 10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_expandedData_rank0_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-5.0)+(1-msk_arg1)*(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(-0.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([-6.0, -1.0, -3.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, -4.0, 3.0])+(1.-msk_arg1)*numpy.array([4.0, 4.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(13.0)+(1.-msk_ref)*numpy.array(-28.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[5.0, 0.0, -1.0], [3.0, 6.0, 1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 7.0, 0.0], [6.0, 6.0, -5.0]])+(1.-msk_arg1)*numpy.array([[0.0, 6.0, -4.0], [-4.0, -1.0, 3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(49.0)+(1.-msk_ref)*numpy.array(-11.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[1.0, -6.0, -6.0], [7.0, 0.0, 5.0], [1.0, 5.0, 0.0], [6.0, -5.0, -3.0]], [[7.0, -5.0, 6.0], [-5.0, -2.0, 1.0], [7.0, 5.0, 7.0], [4.0, -4.0, -2.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 5.0, 1.0], [5.0, 4.0, -2.0], [5.0, 4.0, 3.0], [-2.0, 3.0, 0.0]], [[2.0, 2.0, 0.0], [-7.0, 7.0, -6.0], [1.0, 1.0, -1.0], [1.0, -4.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 3.0, 0.0], [5.0, 2.0, 4.0], [-5.0, -1.0, 2.0], [-7.0, -1.0, 0.0]], [[-5.0, -7.0, 7.0], [0.0, 1.0, -5.0], [-6.0, -6.0, 4.0], [4.0, 1.0, -2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(40.0)+(1.-msk_ref)*numpy.array(-2.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-5.0, 7.0], [-2.0, 7.0], [3.0, 7.0]], [[5.0, -4.0], [0.0, 0.0], [2.0, 5.0]]], [[[4.0, 6.0], [4.0, -4.0], [-7.0, 1.0]], [[0.0, -6.0], [-6.0, 2.0], [2.0, 3.0]]], [[[-5.0, 6.0], [-6.0, 0.0], [-6.0, 4.0]], [[-2.0, 3.0], [7.0, -6.0], [0.0, 6.0]]], [[[-5.0, -4.0], [0.0, -3.0], [-7.0, 4.0]], [[-3.0, 6.0], [0.0, 0.0], [5.0, 2.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, 5.0], [0.0, -5.0], [-4.0, -5.0]], [[0.0, -3.0], [4.0, 5.0], [6.0, -6.0]]], [[[-4.0, -3.0], [-6.0, 3.0], [-1.0, 0.0]], [[5.0, 1.0], [0.0, -6.0], [2.0, 5.0]]], [[[6.0, 7.0], [3.0, -1.0], [5.0, 6.0]], [[3.0, 1.0], [-7.0, 0.0], [5.0, -5.0]]], [[[7.0, -4.0], [0.0, 5.0], [6.0, -6.0]], [[0.0, -4.0], [-2.0, 1.0], [-2.0, 2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, 4.0], [-5.0, 4.0], [0.0, -5.0]], [[-4.0, 0.0], [0.0, 3.0], [-5.0, -3.0]]], [[[1.0, -1.0], [-2.0, -4.0], [4.0, 3.0]], [[-5.0, 2.0], [0.0, -4.0], [4.0, 0.0]]], [[[-1.0, 0.0], [0.0, 4.0], [7.0, 0.0]], [[-4.0, -6.0], [-7.0, 2.0], [-4.0, 4.0]]], [[[3.0, 5.0], [-4.0, -6.0], [0.0, 2.0]], [[4.0, 0.0], [-2.0, 0.0], [-4.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-374.0)+(1.-msk_ref)*numpy.array(-145.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_expandedData_rank1_offset0(self):
      arg0=Data(1.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, -3.0])+(1.-msk_arg1)*numpy.array([0.0, 5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([1.0, -3.0])+(1.-msk_ref)*numpy.array([0.0, 5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([7.0, 3.0, -1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -2.0], [4.0, -7.0], [6.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-7.0, 7.0], [0.0, 6.0], [-6.0, -7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-29.0, -40.0])+(1.-msk_ref)*numpy.array([-43.0, 74.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[-6.0, -5.0, 4.0], [-6.0, -2.0, 1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 6.0], [-3.0, 5.0], [5.0, -3.0]], [[-3.0, -5.0], [-5.0, -2.0], [-5.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 1.0], [5.0, 1.0], [-3.0, 6.0]], [[-4.0, -4.0], [0.0, -7.0], [-3.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([34.0, -44.0])+(1.-msk_ref)*numpy.array([-34.0, 48.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[-1.0, 4.0, 3.0], [4.0, 0.0, 3.0], [3.0, -4.0, -6.0], [7.0, 5.0, -4.0]], [[1.0, 4.0, -1.0], [5.0, -3.0, -5.0], [2.0, -5.0, 3.0], [-5.0, -2.0, 1.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-1.0, -1.0], [-4.0, -7.0], [0.0, 6.0]], [[-3.0, 6.0], [5.0, 7.0], [-1.0, -5.0]], [[3.0, -1.0], [-6.0, -2.0], [-3.0, 6.0]], [[5.0, 0.0], [2.0, -2.0], [0.0, 0.0]]], [[[-4.0, 5.0], [6.0, -3.0], [7.0, -6.0]], [[-4.0, 2.0], [2.0, -6.0], [1.0, -5.0]], [[3.0, 0.0], [1.0, -7.0], [-3.0, 0.0]], [[7.0, -6.0], [-4.0, 7.0], [-5.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, 0.0], [0.0, -4.0], [0.0, -1.0]], [[2.0, 5.0], [-4.0, 0.0], [-5.0, 5.0]], [[-7.0, 3.0], [-4.0, -3.0], [7.0, -5.0]], [[-3.0, -7.0], [-3.0, -3.0], [1.0, 1.0]]], [[[-4.0, 1.0], [-3.0, 7.0], [0.0, -6.0]], [[6.0, 5.0], [-4.0, -5.0], [3.0, -1.0]], [[-2.0, 6.0], [-7.0, 4.0], [-3.0, -5.0]], [[0.0, -2.0], [-1.0, -4.0], [1.0, -3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, 67.0])+(1.-msk_ref)*numpy.array([-65.0, 71.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_expandedData_rank2_offset0(self):
      arg0=Data(-2.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 2.0, -5.0, -7.0, 0.0], [-7.0, -2.0, -2.0, -3.0, -5.0], [-7.0, -2.0, -2.0, 0.0, 0.0], [7.0, 2.0, 2.0, -6.0, -2.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 2.0, 0.0, -1.0, 0.0], [-1.0, 4.0, 1.0, -2.0, -1.0], [-7.0, -2.0, -7.0, 1.0, 3.0], [4.0, 6.0, 4.0, -7.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-2.0, -4.0, 10.0, 14.0, -0.0], [14.0, 4.0, 4.0, 6.0, 10.0], [14.0, 4.0, 4.0, -0.0, -0.0], [-14.0, -4.0, -4.0, 12.0, 4.0]])+(1.-msk_ref)*numpy.array([[4.0, -4.0, -0.0, 2.0, -0.0], [2.0, -8.0, -2.0, 4.0, 2.0], [14.0, 4.0, 14.0, -2.0, -6.0], [-8.0, -12.0, -8.0, 14.0, -0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([-3.0, -7.0, -3.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 7.0, 6.0, 7.0, 1.0], [5.0, -5.0, 4.0, 1.0, 1.0], [-1.0, 2.0, -2.0, -7.0, 5.0], [-4.0, 0.0, -5.0, 0.0, -6.0]], [[-2.0, 1.0, -4.0, -1.0, 3.0], [4.0, 4.0, 4.0, 3.0, -4.0], [3.0, -2.0, -1.0, -6.0, 0.0], [-5.0, -6.0, -6.0, -4.0, -7.0]], [[3.0, -1.0, -5.0, 4.0, -2.0], [-5.0, 4.0, 5.0, 6.0, -3.0], [-5.0, 3.0, 1.0, 0.0, 0.0], [2.0, 7.0, 6.0, 4.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, -4.0, 3.0, -3.0, -6.0], [-2.0, -4.0, -1.0, 6.0, -1.0], [6.0, 0.0, 0.0, 0.0, 5.0], [-4.0, -4.0, 5.0, -7.0, 0.0]], [[-2.0, 1.0, 1.0, -3.0, -3.0], [1.0, 6.0, -6.0, -2.0, 7.0], [2.0, 3.0, 0.0, 5.0, -5.0], [4.0, 0.0, -4.0, -1.0, 2.0]], [[-3.0, 7.0, -5.0, -5.0, 5.0], [4.0, 4.0, -7.0, -7.0, -3.0], [7.0, -5.0, 6.0, -5.0, 0.0], [2.0, 7.0, 3.0, -3.0, 7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-7.0, -25.0, 25.0, -26.0, -18.0], [-28.0, -25.0, -55.0, -42.0, 34.0], [-3.0, -1.0, 10.0, 63.0, -15.0], [41.0, 21.0, 39.0, 16.0, 88.0]])+(1.-msk_ref)*numpy.array([[26.0, -16.0, -1.0, 45.0, 24.0], [-13.0, -42.0, 66.0, 17.0, -37.0], [-53.0, -6.0, -18.0, -20.0, 20.0], [-22.0, -9.0, 4.0, 37.0, -35.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[-2.0, -1.0, 3.0], [3.0, 2.0, 3.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, 4.0, 0.0, -3.0, 6.0], [5.0, -4.0, 4.0, 4.0, 2.0], [-7.0, -2.0, -3.0, -3.0, 5.0], [-5.0, -4.0, -4.0, 1.0, -3.0]], [[-5.0, 7.0, -6.0, 1.0, 2.0], [-5.0, 4.0, 4.0, 6.0, -6.0], [7.0, 6.0, 6.0, 3.0, -3.0], [7.0, 4.0, 7.0, -6.0, -3.0]], [[2.0, -3.0, -3.0, -5.0, -6.0], [2.0, -4.0, 5.0, -4.0, -1.0], [1.0, -3.0, 1.0, -3.0, -2.0], [3.0, 7.0, 1.0, 0.0, -1.0]]], [[[4.0, -4.0, 0.0, 5.0, 2.0], [1.0, -3.0, 0.0, 0.0, -5.0], [-7.0, -5.0, -5.0, 0.0, -1.0], [6.0, -3.0, 7.0, 1.0, -3.0]], [[5.0, 4.0, 0.0, -4.0, 1.0], [1.0, 7.0, -7.0, 3.0, 5.0], [0.0, 4.0, 3.0, 7.0, -4.0], [-1.0, 4.0, -3.0, 4.0, 5.0]], [[-6.0, 1.0, 5.0, -2.0, 2.0], [1.0, 6.0, -4.0, 1.0, -6.0], [-3.0, -6.0, 2.0, -7.0, 0.0], [3.0, 6.0, -5.0, -5.0, -1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, -4.0, 7.0, -4.0, 2.0], [5.0, -7.0, -5.0, -3.0, 4.0], [0.0, 6.0, -7.0, -2.0, -7.0], [-6.0, 7.0, -5.0, 4.0, -2.0]], [[6.0, 5.0, 6.0, -3.0, 0.0], [0.0, -7.0, 6.0, -5.0, -3.0], [-3.0, 1.0, 6.0, 5.0, -4.0], [3.0, 0.0, 2.0, -5.0, 4.0]], [[-3.0, -3.0, 5.0, -5.0, 6.0], [3.0, -6.0, 7.0, 1.0, -5.0], [2.0, 5.0, 5.0, 5.0, 1.0], [1.0, -4.0, 3.0, 5.0, -6.0]]], [[[3.0, -7.0, 2.0, 6.0, 0.0], [1.0, 6.0, -4.0, -1.0, -7.0], [2.0, 4.0, -2.0, -7.0, 4.0], [6.0, 0.0, 1.0, 0.0, 4.0]], [[1.0, 1.0, 5.0, -1.0, 3.0], [5.0, 5.0, 6.0, 4.0, -5.0], [-3.0, 1.0, -3.0, -2.0, 0.0], [2.0, 1.0, -2.0, -5.0, -4.0]], [[-5.0, 7.0, -1.0, -5.0, -6.0], [-3.0, 3.0, 6.0, 7.0, -1.0], [-6.0, -6.0, 4.0, 1.0, -7.0], [-2.0, -6.0, -1.0, 2.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[23.0, -25.0, 12.0, -9.0, -18.0], [9.0, 15.0, -23.0, -17.0, -24.0], [-20.0, -36.0, 0.0, -13.0, -24.0], [37.0, 42.0, 4.0, 0.0, 4.0]])+(1.-msk_ref)*numpy.array([[-9.0, -4.0, 8.0, -3.0, 2.0], [3.0, 40.0, 43.0, 40.0, -54.0], [-9.0, -2.0, 23.0, -8.0, 12.0], [28.0, -42.0, 13.0, 8.0, -29.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_expandedData_rank3_offset0(self):
      arg0=Data(4.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, 0.0], [-5.0, 4.0]], [[-4.0, 6.0], [0.0, 0.0]], [[-3.0, 0.0], [2.0, -4.0]], [[6.0, 7.0], [2.0, -2.0]], [[3.0, 2.0], [-6.0, 3.0]], [[5.0, 0.0], [6.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 0.0], [-6.0, 5.0]], [[2.0, 3.0], [7.0, -2.0]], [[1.0, 7.0], [0.0, 0.0]], [[-7.0, -7.0], [0.0, -7.0]], [[6.0, -7.0], [3.0, 0.0]], [[0.0, 5.0], [6.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-16.0, 0.0], [-20.0, 16.0]], [[-16.0, 24.0], [0.0, 0.0]], [[-12.0, 0.0], [8.0, -16.0]], [[24.0, 28.0], [8.0, -8.0]], [[12.0, 8.0], [-24.0, 12.0]], [[20.0, 0.0], [24.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, 0.0], [-24.0, 20.0]], [[8.0, 12.0], [28.0, -8.0]], [[4.0, 28.0], [0.0, 0.0]], [[-28.0, -28.0], [0.0, -28.0]], [[24.0, -28.0], [12.0, 0.0]], [[0.0, 20.0], [24.0, -12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([-6.0, -1.0, -4.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, 0.0], [-7.0, 7.0]], [[2.0, 3.0], [-4.0, 5.0]], [[-7.0, -1.0], [4.0, 4.0]], [[-4.0, -4.0], [0.0, -6.0]], [[4.0, -3.0], [-2.0, -2.0]], [[-5.0, 3.0], [-5.0, -6.0]]], [[[-7.0, 3.0], [-1.0, -6.0]], [[0.0, -1.0], [-2.0, -3.0]], [[0.0, 5.0], [7.0, 3.0]], [[-6.0, 5.0], [-5.0, 0.0]], [[-2.0, 6.0], [2.0, 1.0]], [[4.0, -4.0], [-7.0, 5.0]]], [[[-4.0, 3.0], [-5.0, -7.0]], [[-7.0, 6.0], [-6.0, 3.0]], [[5.0, 3.0], [4.0, 2.0]], [[-1.0, 7.0], [4.0, -5.0]], [[6.0, -6.0], [3.0, 2.0]], [[-2.0, 3.0], [-3.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, -3.0], [7.0, 5.0]], [[-5.0, 3.0], [7.0, 4.0]], [[7.0, -2.0], [0.0, 0.0]], [[2.0, -4.0], [7.0, 2.0]], [[-3.0, 6.0], [4.0, -2.0]], [[-4.0, -2.0], [0.0, 0.0]]], [[[1.0, 7.0], [-5.0, -5.0]], [[5.0, 3.0], [7.0, 0.0]], [[-3.0, 0.0], [-5.0, -4.0]], [[1.0, 0.0], [5.0, -4.0]], [[0.0, -1.0], [0.0, 2.0]], [[-5.0, 6.0], [4.0, 3.0]]], [[[7.0, -6.0], [-4.0, 6.0]], [[5.0, -1.0], [7.0, -4.0]], [[3.0, -6.0], [5.0, 2.0]], [[-6.0, 2.0], [5.0, -5.0]], [[0.0, 7.0], [-7.0, 0.0]], [[5.0, 0.0], [-6.0, -2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[47.0, -15.0], [63.0, -8.0]], [[16.0, -41.0], [50.0, -39.0]], [[22.0, -11.0], [-47.0, -35.0]], [[34.0, -9.0], [-11.0, 56.0]], [[-46.0, 36.0], [-2.0, 3.0]], [[34.0, -26.0], [49.0, 3.0]]])+(1.-msk_ref)*numpy.array([[[13.0, 35.0], [-21.0, -49.0]], [[5.0, -17.0], [-77.0, -8.0]], [[-51.0, 36.0], [-15.0, -4.0]], [[11.0, 16.0], [-67.0, 12.0]], [[18.0, -63.0], [4.0, 10.0]], [[9.0, 6.0], [20.0, 5.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank0_expandedData_rank4_offset0(self):
      arg0=Data(4.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, 3.0, 1.0, 0.0], [7.0, -4.0, -4.0, 0.0], [-4.0, 0.0, 4.0, -6.0]], [[-7.0, -7.0, -5.0, -5.0], [-6.0, -1.0, 3.0, -1.0], [2.0, 3.0, 4.0, -3.0]]], [[[7.0, 1.0, -1.0, 1.0], [7.0, -3.0, 7.0, 3.0], [-3.0, -3.0, -5.0, 0.0]], [[-6.0, 5.0, -5.0, 0.0], [-6.0, -6.0, 0.0, -6.0], [-2.0, -2.0, -3.0, 7.0]]], [[[-7.0, -5.0, 4.0, -4.0], [-2.0, 2.0, -2.0, -5.0], [-6.0, -3.0, -2.0, 2.0]], [[-5.0, 0.0, 1.0, -6.0], [-7.0, 2.0, 6.0, 0.0], [3.0, 1.0, 0.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[2.0, 5.0, 4.0, -6.0], [-4.0, 0.0, -2.0, -4.0], [-6.0, 0.0, 5.0, -4.0]], [[-3.0, 0.0, 1.0, -5.0], [-3.0, -5.0, 3.0, -7.0], [0.0, -6.0, -1.0, -7.0]]], [[[2.0, 7.0, 1.0, 2.0], [-2.0, -1.0, 4.0, -7.0], [4.0, -3.0, -6.0, 2.0]], [[-4.0, 6.0, -7.0, -3.0], [0.0, 2.0, 6.0, -5.0], [1.0, -2.0, -7.0, 7.0]]], [[[-2.0, -1.0, 3.0, -3.0], [-5.0, -6.0, 5.0, -1.0], [6.0, 1.0, -5.0, -7.0]], [[0.0, 1.0, 3.0, -2.0], [-3.0, -6.0, 4.0, 1.0], [-6.0, -6.0, -1.0, -7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 12.0, 4.0, 0.0], [28.0, -16.0, -16.0, 0.0], [-16.0, 0.0, 16.0, -24.0]], [[-28.0, -28.0, -20.0, -20.0], [-24.0, -4.0, 12.0, -4.0], [8.0, 12.0, 16.0, -12.0]]], [[[28.0, 4.0, -4.0, 4.0], [28.0, -12.0, 28.0, 12.0], [-12.0, -12.0, -20.0, 0.0]], [[-24.0, 20.0, -20.0, 0.0], [-24.0, -24.0, 0.0, -24.0], [-8.0, -8.0, -12.0, 28.0]]], [[[-28.0, -20.0, 16.0, -16.0], [-8.0, 8.0, -8.0, -20.0], [-24.0, -12.0, -8.0, 8.0]], [[-20.0, 0.0, 4.0, -24.0], [-28.0, 8.0, 24.0, 0.0], [12.0, 4.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, 20.0, 16.0, -24.0], [-16.0, 0.0, -8.0, -16.0], [-24.0, 0.0, 20.0, -16.0]], [[-12.0, 0.0, 4.0, -20.0], [-12.0, -20.0, 12.0, -28.0], [0.0, -24.0, -4.0, -28.0]]], [[[8.0, 28.0, 4.0, 8.0], [-8.0, -4.0, 16.0, -28.0], [16.0, -12.0, -24.0, 8.0]], [[-16.0, 24.0, -28.0, -12.0], [0.0, 8.0, 24.0, -20.0], [4.0, -8.0, -28.0, 28.0]]], [[[-8.0, -4.0, 12.0, -12.0], [-20.0, -24.0, 20.0, -4.0], [24.0, 4.0, -20.0, -28.0]], [[0.0, 4.0, 12.0, -8.0], [-12.0, -24.0, 16.0, 4.0], [-24.0, -24.0, -4.0, -28.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_float_rank0_offset0(self):
      arg0=Data(numpy.array([3.0, -7.0]),self.functionspace)
      arg1=2.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([6.0, -14.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank0_offset0(self):
      arg0=Data(numpy.array([-2.0, 2.0]),self.functionspace)
      arg1=numpy.array(5.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-10.0, 10.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank1_offset1(self):
      arg0=Data(numpy.array([[7.0, -5.0, -1.0], [-7.0, 0.0, -4.0]]),self.functionspace)
      arg1=numpy.array([-4.0, -7.0, -3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([10.0, 40.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[0.0, 5.0, 2.0], [1.0, 0.0, 6.0]], [[-4.0, 7.0, 0.0], [7.0, -5.0, 1.0]]]),self.functionspace)
      arg1=numpy.array([[3.0, 5.0, 1.0], [0.0, -4.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([63.0, 49.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[[6.0, -6.0, -4.0], [4.0, 0.0, 4.0], [5.0, 7.0, 5.0], [0.0, 2.0, 0.0]], [[5.0, -5.0, -6.0], [0.0, 1.0, 4.0], [-6.0, -7.0, -5.0], [7.0, -1.0, 0.0]]], [[[3.0, 2.0, 4.0], [-7.0, -7.0, -6.0], [-6.0, 0.0, -5.0], [-5.0, 0.0, -5.0]], [[6.0, -6.0, 0.0], [-1.0, -3.0, -7.0], [-7.0, 3.0, 3.0], [7.0, 5.0, -2.0]]]]),self.functionspace)
      arg1=numpy.array([[[-4.0, 0.0, -1.0], [2.0, 3.0, -2.0], [-3.0, -4.0, 6.0], [-3.0, 0.0, 4.0]], [[-4.0, 3.0, 4.0], [-4.0, -1.0, 4.0], [-3.0, 6.0, -5.0], [0.0, 6.0, -7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-82.0, -51.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank1_offset0(self):
      arg0=Data(numpy.array([3.0, -3.0]),self.functionspace)
      arg1=numpy.array([-3.0, 7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-9.0, 21.0], [9.0, -21.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank2_offset1(self):
      arg0=Data(numpy.array([[1.0, 0.0, 2.0], [5.0, 3.0, 1.0]]),self.functionspace)
      arg1=numpy.array([[3.0, -1.0], [-4.0, -2.0], [1.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[5.0, -9.0], [4.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[-7.0, -7.0, 0.0], [6.0, 5.0, 7.0]], [[2.0, -7.0, 3.0], [2.0, 3.0, -2.0]]]),self.functionspace)
      arg1=numpy.array([[[-3.0, 0.0], [3.0, 6.0], [-6.0, 7.0]], [[0.0, 6.0], [3.0, 5.0], [-3.0, -5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-6.0, -16.0], [-30.0, 16.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[[5.0, -7.0, -3.0], [3.0, -2.0, -6.0], [2.0, -2.0, -3.0], [1.0, -4.0, 4.0]], [[0.0, -4.0, 6.0], [4.0, -4.0, 0.0], [0.0, -7.0, -2.0], [3.0, 4.0, 5.0]]], [[[0.0, 2.0, 0.0], [-1.0, 7.0, 5.0], [5.0, -5.0, 6.0], [6.0, 5.0, -7.0]], [[0.0, -3.0, -1.0], [-3.0, -6.0, 1.0], [0.0, 3.0, -3.0], [2.0, 7.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[[[-6.0, -1.0], [-5.0, 4.0], [-6.0, 0.0]], [[-6.0, 0.0], [7.0, 7.0], [-2.0, 2.0]], [[-5.0, -1.0], [-2.0, -3.0], [-5.0, 4.0]], [[7.0, -1.0], [6.0, 0.0], [6.0, -2.0]]], [[[6.0, 4.0], [4.0, -2.0], [0.0, 1.0]], [[6.0, -7.0], [7.0, -2.0], [0.0, 0.0]], [[0.0, 5.0], [4.0, 0.0], [0.0, 0.0]], [[-6.0, 2.0], [-4.0, 0.0], [-4.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[-83.0, -101.0], [-80.0, 151.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank2_offset0(self):
      arg0=Data(numpy.array([1.0, -2.0]),self.functionspace)
      arg1=numpy.array([[-1.0, -3.0, 0.0, -4.0, 1.0], [1.0, -7.0, 0.0, -3.0, -1.0], [0.0, -3.0, 3.0, 4.0, 2.0], [1.0, -1.0, -1.0, -2.0, 1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-1.0, -3.0, 0.0, -4.0, 1.0], [1.0, -7.0, 0.0, -3.0, -1.0], [0.0, -3.0, 3.0, 4.0, 2.0], [1.0, -1.0, -1.0, -2.0, 1.0]], [[2.0, 6.0, 0.0, 8.0, -2.0], [-2.0, 14.0, 0.0, 6.0, 2.0], [0.0, 6.0, -6.0, -8.0, -4.0], [-2.0, 2.0, 2.0, 4.0, -2.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank3_offset1(self):
      arg0=Data(numpy.array([[3.0, -6.0, 2.0], [0.0, 5.0, 4.0]]),self.functionspace)
      arg1=numpy.array([[[1.0, 6.0, 0.0, 6.0, 1.0], [0.0, 5.0, 6.0, 5.0, 1.0], [6.0, 0.0, -3.0, 0.0, -5.0], [-6.0, 6.0, -6.0, 0.0, -7.0]], [[-2.0, 4.0, -2.0, -2.0, 2.0], [6.0, -4.0, -2.0, 1.0, 1.0], [0.0, 5.0, 6.0, 0.0, -2.0], [6.0, -1.0, 7.0, 7.0, 0.0]], [[-3.0, 0.0, -2.0, 1.0, -1.0], [6.0, 4.0, 4.0, -6.0, 0.0], [2.0, -7.0, -5.0, 7.0, 2.0], [-5.0, 5.0, -7.0, 1.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[9.0, -6.0, 8.0, 32.0, -11.0], [-24.0, 47.0, 38.0, -3.0, -3.0], [22.0, -44.0, -55.0, 14.0, 1.0], [-64.0, 34.0, -74.0, -40.0, -19.0]], [[-22.0, 20.0, -18.0, -6.0, 6.0], [54.0, -4.0, 6.0, -19.0, 5.0], [8.0, -3.0, 10.0, 28.0, -2.0], [10.0, 15.0, 7.0, 39.0, 4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[-2.0, 2.0, -7.0], [-2.0, 2.0, -5.0]], [[5.0, 5.0, -1.0], [6.0, 0.0, -3.0]]]),self.functionspace)
      arg1=numpy.array([[[[4.0, 0.0, 7.0, -6.0, -3.0], [6.0, 0.0, -2.0, 5.0, -3.0], [7.0, 7.0, 5.0, -5.0, -3.0], [4.0, 6.0, -5.0, 0.0, 1.0]], [[-5.0, -4.0, 6.0, 3.0, 5.0], [5.0, 2.0, 7.0, -5.0, -4.0], [-4.0, 3.0, 0.0, 1.0, 2.0], [-2.0, 1.0, 3.0, -7.0, 5.0]], [[6.0, -7.0, -1.0, -6.0, 0.0], [-3.0, 5.0, 7.0, 5.0, 3.0], [6.0, -6.0, 5.0, -2.0, -3.0], [-3.0, -7.0, 6.0, 5.0, 4.0]]], [[[5.0, -3.0, 5.0, 3.0, -3.0], [3.0, -7.0, 0.0, 0.0, 0.0], [4.0, 2.0, 6.0, -7.0, -2.0], [-1.0, 0.0, 4.0, 0.0, -2.0]], [[7.0, -6.0, -2.0, -7.0, 0.0], [-6.0, -2.0, -3.0, -4.0, -5.0], [0.0, -4.0, 2.0, -5.0, -2.0], [6.0, 3.0, -1.0, 1.0, -2.0]], [[3.0, 6.0, 4.0, 2.0, -2.0], [3.0, 0.0, 4.0, 0.0, 4.0], [-1.0, 0.0, 0.0, 5.0, 6.0], [-7.0, -2.0, -3.0, 4.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-71.0, 5.0, -29.0, 30.0, 32.0], [-14.0, -21.0, -57.0, -63.0, -53.0], [-67.0, 22.0, -53.0, 5.0, 1.0], [58.0, 55.0, -21.0, -67.0, -40.0]], [[10.0, -49.0, 84.0, 3.0, -2.0], [67.0, -37.0, 6.0, -5.0, -50.0], [36.0, 68.0, 56.0, -75.0, -32.0], [28.0, 48.0, 17.0, -52.0, 2.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_array_rank3_offset0(self):
      arg0=Data(numpy.array([-4.0, 0.0]),self.functionspace)
      arg1=numpy.array([[[4.0, -5.0], [6.0, 4.0]], [[7.0, -4.0], [7.0, 5.0]], [[-1.0, -6.0], [4.0, -1.0]], [[-5.0, -2.0], [-6.0, -1.0]], [[6.0, -3.0], [0.0, -1.0]], [[-2.0, 1.0], [1.0, -1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-16.0, 20.0], [-24.0, -16.0]], [[-28.0, 16.0], [-28.0, -20.0]], [[4.0, 24.0], [-16.0, 4.0]], [[20.0, 8.0], [24.0, 4.0]], [[-24.0, 12.0], [0.0, 4.0]], [[8.0, -4.0], [-4.0, 4.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank4_offset1(self):
      arg0=Data(numpy.array([[-2.0, 3.0, -5.0], [6.0, 3.0, -1.0]]),self.functionspace)
      arg1=numpy.array([[[[1.0, -1.0], [-3.0, -3.0]], [[-1.0, -3.0], [1.0, 5.0]], [[-7.0, 2.0], [-2.0, -3.0]], [[1.0, -1.0], [-1.0, 0.0]], [[-2.0, -3.0], [-7.0, 0.0]], [[-7.0, 0.0], [-4.0, 5.0]]], [[[-5.0, 6.0], [2.0, 6.0]], [[-1.0, 6.0], [2.0, 0.0]], [[3.0, 5.0], [-2.0, -4.0]], [[-7.0, 5.0], [-1.0, -3.0]], [[5.0, 0.0], [2.0, -1.0]], [[-7.0, 1.0], [-6.0, -2.0]]], [[[-3.0, 0.0], [-3.0, 6.0]], [[5.0, -7.0], [2.0, -2.0]], [[-1.0, 5.0], [4.0, -2.0]], [[2.0, 4.0], [4.0, -5.0]], [[-4.0, 7.0], [2.0, 1.0]], [[-2.0, -1.0], [-7.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-2.0, 20.0], [27.0, -6.0]], [[-26.0, 59.0], [-6.0, 0.0]], [[28.0, -14.0], [-22.0, 4.0]], [[-33.0, -3.0], [-21.0, 16.0]], [[39.0, -29.0], [10.0, -8.0]], [[3.0, 8.0], [25.0, -26.0]]], [[[-6.0, 12.0], [-9.0, -6.0]], [[-14.0, 7.0], [10.0, 32.0]], [[-32.0, 22.0], [-22.0, -28.0]], [[-17.0, 5.0], [-13.0, -4.0]], [[7.0, -25.0], [-38.0, -4.0]], [[-61.0, 4.0], [-35.0, 22.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([1.0, -5.0]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-1.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-1.0, 5.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[-7.0, 4.0, -5.0], [3.0, 4.0, -2.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-5.0, -6.0, 3.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-4.0, -45.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[-2.0, -7.0, -4.0], [5.0, 4.0, -4.0]], [[-2.0, -1.0, -5.0], [0.0, 1.0, -1.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-1.0, 3.0, 2.0], [5.0, 3.0, 6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-14.0, -14.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[[2.0, 1.0, 1.0], [4.0, 3.0, -1.0], [-4.0, -3.0, 2.0], [2.0, 4.0, -4.0]], [[-6.0, 0.0, 7.0], [0.0, 3.0, 0.0], [-5.0, -7.0, 3.0], [-3.0, -6.0, -1.0]]], [[[-4.0, 0.0, -1.0], [-6.0, 3.0, -4.0], [-4.0, 4.0, 3.0], [4.0, -3.0, 0.0]], [[4.0, -7.0, -3.0], [7.0, -7.0, 4.0], [1.0, -4.0, 5.0], [-5.0, -3.0, -6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-4.0, 5.0, 3.0], [-2.0, 0.0, -6.0], [6.0, -1.0, -6.0], [0.0, 1.0, -4.0]], [[1.0, -2.0, 0.0], [5.0, -7.0, 0.0], [-7.0, 3.0, -2.0], [-6.0, 0.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-23.0, 61.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([-6.0, 2.0]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-1.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[6.0, -36.0], [-2.0, 12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[-4.0, -1.0, 6.0], [4.0, -3.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-6.0, 0.0], [-5.0, 2.0], [2.0, -7.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[41.0, -44.0], [-15.0, 15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[-7.0, 5.0, 5.0], [7.0, -1.0, -1.0]], [[-2.0, 3.0, 5.0], [-7.0, 0.0, 3.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[5.0, 4.0], [-4.0, 1.0], [-6.0, -2.0]], [[0.0, -6.0], [0.0, -6.0], [0.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-85.0, -76.0], [-52.0, 48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-4.0, 3.0, 4.0], [3.0, -5.0, -6.0], [-5.0, -6.0, 5.0], [-5.0, 4.0, 0.0]], [[-6.0, -4.0, -5.0], [-6.0, 0.0, 6.0], [3.0, 2.0, -4.0], [1.0, 2.0, -1.0]]], [[[2.0, 4.0, 0.0], [-5.0, 2.0, 3.0], [-1.0, 0.0, 0.0], [1.0, -6.0, 5.0]], [[-4.0, -2.0, 3.0], [-2.0, -6.0, -2.0], [7.0, -6.0, -2.0], [6.0, 1.0, 0.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[1.0, 6.0], [0.0, -7.0], [4.0, -3.0]], [[-5.0, -4.0], [7.0, 4.0], [-5.0, 4.0]], [[3.0, -5.0], [-6.0, 1.0], [4.0, -7.0]], [[-4.0, 2.0], [-2.0, 5.0], [-7.0, 4.0]]], [[[-4.0, 0.0], [-4.0, 5.0], [5.0, 2.0]], [[-7.0, 3.0], [7.0, 0.0], [6.0, 5.0]], [[-7.0, 7.0], [6.0, -7.0], [5.0, -5.0]], [[3.0, 2.0], [-2.0, -5.0], [2.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[106.0, -115.0], [-84.0, 109.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([6.0, -6.0]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-4.0, 3.0, 0.0, 4.0, -6.0], [-2.0, 3.0, -3.0, 1.0, 1.0], [6.0, -2.0, 4.0, -4.0, 1.0], [-4.0, -7.0, 2.0, 4.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-24.0, 18.0, 0.0, 24.0, -36.0], [-12.0, 18.0, -18.0, 6.0, 6.0], [36.0, -12.0, 24.0, -24.0, 6.0], [-24.0, -42.0, 12.0, 24.0, -12.0]], [[24.0, -18.0, 0.0, -24.0, 36.0], [12.0, -18.0, 18.0, -6.0, -6.0], [-36.0, 12.0, -24.0, 24.0, -6.0], [24.0, 42.0, -12.0, -24.0, 12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[7.0, 2.0, -1.0], [-5.0, 2.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[5.0, -4.0, -2.0, 6.0, -7.0], [-4.0, 3.0, 4.0, -3.0, -1.0], [-4.0, 2.0, -6.0, 7.0, 3.0], [-5.0, 2.0, -5.0, 6.0, -6.0]], [[2.0, -2.0, 5.0, 7.0, -7.0], [-1.0, 2.0, 0.0, -4.0, 1.0], [0.0, 6.0, 4.0, 6.0, -2.0], [3.0, 5.0, 7.0, 2.0, 7.0]], [[4.0, -5.0, 4.0, -5.0, 2.0], [-2.0, 6.0, 5.0, -5.0, 5.0], [5.0, 3.0, -7.0, 1.0, -6.0], [4.0, -3.0, 0.0, 1.0, -2.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[35.0, -27.0, -8.0, 61.0, -65.0], [-28.0, 19.0, 23.0, -24.0, -10.0], [-33.0, 23.0, -27.0, 60.0, 23.0], [-33.0, 27.0, -21.0, 45.0, -26.0]], [[-21.0, 16.0, 20.0, -16.0, 21.0], [18.0, -11.0, -20.0, 7.0, 7.0], [20.0, 2.0, 38.0, -23.0, -19.0], [31.0, 0.0, 39.0, -26.0, 44.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[-5.0, 6.0, 6.0], [-6.0, 6.0, -4.0]], [[-2.0, -4.0, 2.0], [6.0, -5.0, -1.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-7.0, 3.0, 6.0, -5.0, -3.0], [2.0, 6.0, 0.0, 3.0, 4.0], [6.0, 1.0, -2.0, -6.0, 1.0], [-4.0, -3.0, 3.0, 1.0, 6.0]], [[7.0, 1.0, 2.0, -5.0, -2.0], [2.0, -6.0, 0.0, 0.0, 0.0], [-1.0, 7.0, 0.0, -1.0, 0.0], [1.0, -5.0, -4.0, 6.0, -3.0]], [[0.0, -1.0, -2.0, 2.0, -2.0], [-6.0, 2.0, -3.0, -3.0, 0.0], [5.0, 7.0, -7.0, -4.0, 7.0], [0.0, 0.0, 3.0, -3.0, 4.0]]], [[[-2.0, 0.0, 1.0, 1.0, 0.0], [2.0, 0.0, -3.0, 4.0, -5.0], [1.0, 5.0, 1.0, 3.0, 2.0], [0.0, -6.0, 0.0, 3.0, -4.0]], [[6.0, 3.0, -2.0, 0.0, -7.0], [1.0, 2.0, 3.0, -2.0, 1.0], [2.0, 7.0, -2.0, 0.0, 0.0], [-4.0, 5.0, 2.0, 3.0, -6.0]], [[-7.0, -1.0, 0.0, 2.0, -5.0], [-7.0, 3.0, -3.0, -4.0, 0.0], [-2.0, -1.0, 4.0, 0.0, 1.0], [0.0, -3.0, 7.0, 0.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[153.0, 7.0, -48.0, -7.0, -31.0], [-12.0, -54.0, 30.0, -53.0, 16.0], [8.0, 95.0, -66.0, -18.0, 21.0], [2.0, 63.0, -37.0, 13.0, -64.0]], [[-49.0, -26.0, -8.0, 38.0, 50.0], [-10.0, 3.0, -36.0, 26.0, -43.0], [0.0, -20.0, 2.0, 26.0, 23.0], [24.0, -32.0, -1.0, -29.0, 7.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_Symbol_rank3_offset0(self):
      arg0=Data(numpy.array([7.0, -3.0]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[0.0, 5.0], [-7.0, 6.0]], [[7.0, -1.0], [3.0, 5.0]], [[0.0, 5.0], [4.0, -4.0]], [[-1.0, 3.0], [7.0, -2.0]], [[7.0, 3.0], [2.0, 6.0]], [[6.0, 0.0], [5.0, -7.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[0.0, 35.0], [-49.0, 42.0]], [[49.0, -7.0], [21.0, 35.0]], [[0.0, 35.0], [28.0, -28.0]], [[-7.0, 21.0], [49.0, -14.0]], [[49.0, 21.0], [14.0, 42.0]], [[42.0, 0.0], [35.0, -49.0]]], [[[0.0, -15.0], [21.0, -18.0]], [[-21.0, 3.0], [-9.0, -15.0]], [[0.0, -15.0], [-12.0, 12.0]], [[3.0, -9.0], [-21.0, 6.0]], [[-21.0, -9.0], [-6.0, -18.0]], [[-18.0, 0.0], [-15.0, 21.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([[-6.0, 2.0, -7.0], [-7.0, 2.0, 4.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[-3.0, 0.0], [2.0, 3.0]], [[0.0, -4.0], [-5.0, 5.0]], [[0.0, -3.0], [3.0, 2.0]], [[-5.0, 0.0], [2.0, 3.0]], [[-4.0, -2.0], [-7.0, -5.0]], [[-4.0, 7.0], [0.0, -5.0]]], [[[1.0, -7.0], [3.0, -3.0]], [[5.0, -7.0], [0.0, -1.0]], [[-4.0, -6.0], [7.0, 2.0]], [[2.0, -7.0], [-4.0, 4.0]], [[-4.0, 2.0], [5.0, 5.0]], [[7.0, -2.0], [0.0, 4.0]]], [[[-3.0, 7.0], [0.0, -2.0]], [[-3.0, 2.0], [0.0, -6.0]], [[-2.0, 0.0], [-5.0, -4.0]], [[-7.0, -1.0], [-1.0, -2.0]], [[7.0, -1.0], [-2.0, 4.0]], [[4.0, -3.0], [-4.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[41.0, -63.0], [-6.0, -10.0]], [[31.0, -4.0], [30.0, 10.0]], [[6.0, 6.0], [31.0, 20.0]], [[83.0, -7.0], [-13.0, 4.0]], [[-33.0, 23.0], [66.0, 12.0]], [[10.0, -25.0], [28.0, 3.0]]], [[[11.0, 14.0], [-8.0, -35.0]], [[-2.0, 22.0], [35.0, -61.0]], [[-16.0, 9.0], [-27.0, -26.0]], [[11.0, -18.0], [-26.0, -21.0]], [[48.0, 14.0], [51.0, 61.0]], [[58.0, -65.0], [-16.0, 63.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank0_offset0(self):
      arg0=Data(numpy.array([2.0, 0.0]),self.functionspace)
      arg1=Data(3.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([6.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[2.0, 0.0, 6.0], [5.0, 7.0, 2.0]]),self.functionspace)
      arg1=Data(numpy.array([6.0, 4.0, -4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-12.0, 50.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[0.0, 3.0, -5.0], [1.0, 2.0, 1.0]], [[-3.0, 7.0, 1.0], [-1.0, 5.0, -4.0]]]),self.functionspace)
      arg1=Data(numpy.array([[4.0, -6.0, 2.0], [7.0, -7.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-30.0, -114.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[7.0, -3.0, 6.0], [0.0, 2.0, -2.0], [2.0, -6.0, -2.0], [2.0, -5.0, 7.0]], [[0.0, -5.0, 6.0], [6.0, -1.0, 2.0], [-7.0, 3.0, -7.0], [4.0, -7.0, 7.0]]], [[[-1.0, -3.0, 0.0], [-6.0, 7.0, -6.0], [-2.0, 5.0, 0.0], [-6.0, 0.0, -5.0]], [[5.0, -3.0, 5.0], [-7.0, -2.0, -2.0], [5.0, -6.0, -7.0], [-6.0, 3.0, 2.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 4.0, 3.0], [-2.0, -3.0, -3.0], [6.0, -6.0, -5.0], [7.0, -3.0, 7.0]], [[5.0, -2.0, 4.0], [5.0, 0.0, -7.0], [7.0, -4.0, -6.0], [0.0, -3.0, 6.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([222.0, 14.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank1_offset0(self):
      arg0=Data(numpy.array([6.0, -5.0]),self.functionspace)
      arg1=Data(numpy.array([2.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[12.0, -36.0], [-10.0, 30.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[7.0, 5.0, 3.0], [-4.0, -5.0, -6.0]]),self.functionspace)
      arg1=Data(numpy.array([[0.0, -3.0], [-6.0, -3.0], [6.0, -7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-12.0, -57.0], [-6.0, 69.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[-7.0, -6.0, 4.0], [4.0, -6.0, 6.0]], [[4.0, 2.0, -3.0], [-3.0, 0.0, -3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.0, -1.0], [-7.0, -1.0], [0.0, 5.0]], [[3.0, 5.0], [-5.0, 4.0], [-6.0, 0.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[48.0, 29.0], [-5.0, -36.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[0.0, 5.0, -5.0], [5.0, 6.0, 4.0], [3.0, -2.0, 0.0], [-3.0, 4.0, 4.0]], [[1.0, 7.0, -1.0], [0.0, -1.0, -5.0], [-2.0, 2.0, -1.0], [-1.0, -3.0, -6.0]]], [[[6.0, -6.0, -6.0], [-5.0, -6.0, 5.0], [-5.0, 3.0, 2.0], [5.0, 1.0, 6.0]], [[1.0, -3.0, 0.0], [-3.0, -1.0, 0.0], [-4.0, 0.0, -3.0], [7.0, -4.0, 2.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-7.0, -2.0], [-7.0, -3.0], [1.0, 4.0]], [[5.0, -4.0], [5.0, 5.0], [7.0, 1.0]], [[-7.0, 5.0], [-1.0, 3.0], [-1.0, 5.0]], [[2.0, 7.0], [5.0, 5.0], [4.0, -3.0]]], [[[3.0, -4.0], [5.0, -6.0], [-6.0, 4.0]], [[-3.0, 7.0], [-3.0, -2.0], [-6.0, 7.0]], [[1.0, 0.0], [5.0, 0.0], [-1.0, -6.0]], [[5.0, -5.0], [3.0, 0.0], [-3.0, -6.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[144.0, -61.0], [59.0, -41.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank2_offset0(self):
      arg0=Data(numpy.array([5.0, -2.0]),self.functionspace)
      arg1=Data(numpy.array([[0.0, -2.0, 3.0, -7.0, 1.0], [0.0, 1.0, 0.0, -7.0, -7.0], [-1.0, -1.0, 7.0, 0.0, 3.0], [-5.0, 1.0, 3.0, 6.0, -4.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, -10.0, 15.0, -35.0, 5.0], [0.0, 5.0, 0.0, -35.0, -35.0], [-5.0, -5.0, 35.0, 0.0, 15.0], [-25.0, 5.0, 15.0, 30.0, -20.0]], [[0.0, 4.0, -6.0, 14.0, -2.0], [0.0, -2.0, 0.0, 14.0, 14.0], [2.0, 2.0, -14.0, 0.0, -6.0], [10.0, -2.0, -6.0, -12.0, 8.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[0.0, 6.0, -4.0], [-6.0, -6.0, -3.0]]),self.functionspace)
      arg1=Data(numpy.array([[[-1.0, -2.0, -1.0, 3.0, 3.0], [0.0, 0.0, 5.0, -2.0, 3.0], [3.0, -6.0, 3.0, 3.0, 6.0], [-2.0, -6.0, 7.0, 5.0, 2.0]], [[0.0, -5.0, -6.0, -7.0, 3.0], [5.0, 5.0, 5.0, -5.0, 4.0], [3.0, 4.0, 2.0, -6.0, -5.0], [1.0, 0.0, -1.0, 5.0, -2.0]], [[5.0, 2.0, 6.0, -7.0, -2.0], [0.0, -2.0, -6.0, -5.0, 5.0], [0.0, 0.0, 0.0, -4.0, -5.0], [3.0, 0.0, -4.0, 4.0, 3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-20.0, -38.0, -60.0, -14.0, 26.0], [30.0, 38.0, 54.0, -10.0, 4.0], [18.0, 24.0, 12.0, -20.0, -10.0], [-6.0, 0.0, 10.0, 14.0, -24.0]], [[-9.0, 36.0, 24.0, 45.0, -30.0], [-30.0, -24.0, -42.0, 57.0, -57.0], [-36.0, 12.0, -30.0, 30.0, 9.0], [-3.0, 36.0, -24.0, -72.0, -9.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[-3.0, 4.0, -1.0], [1.0, -7.0, -7.0]], [[2.0, 5.0, 1.0], [0.0, -1.0, -6.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-1.0, -1.0, 0.0, 6.0, 3.0], [-3.0, 4.0, -6.0, 7.0, 3.0], [7.0, -3.0, -4.0, 3.0, 6.0], [0.0, 2.0, 0.0, 7.0, -1.0]], [[4.0, -4.0, 2.0, -2.0, -7.0], [1.0, 7.0, 2.0, -6.0, -7.0], [-1.0, 5.0, -1.0, 5.0, -2.0], [-6.0, -6.0, -5.0, -5.0, -2.0]], [[0.0, 7.0, -7.0, 2.0, 7.0], [-3.0, 4.0, 6.0, -5.0, 1.0], [5.0, 6.0, 7.0, -5.0, 5.0], [-3.0, 0.0, 2.0, -3.0, -5.0]]], [[[-6.0, -7.0, 2.0, 6.0, 1.0], [0.0, -1.0, -6.0, -4.0, -2.0], [5.0, -1.0, 6.0, 5.0, -4.0], [-2.0, -1.0, 4.0, -3.0, 0.0]], [[7.0, -1.0, -3.0, 0.0, -4.0], [0.0, -6.0, -6.0, 7.0, 5.0], [1.0, 3.0, 1.0, -2.0, -1.0], [1.0, 6.0, 0.0, 7.0, 7.0]], [[-3.0, 6.0, 1.0, -4.0, -7.0], [-6.0, -3.0, 6.0, 7.0, 0.0], [2.0, -3.0, -1.0, -3.0, 4.0], [-7.0, -4.0, -5.0, -6.0, 6.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-15.0, -62.0, 31.0, 6.0, 34.0], [58.0, 74.0, 14.0, -142.0, -75.0], [-46.0, 22.0, 7.0, 56.0, -56.0], [19.0, -45.0, 17.0, -48.0, -91.0]], [[29.0, -50.0, 0.0, 28.0, 24.0], [32.0, 71.0, -26.0, -70.0, -33.0], [1.0, 40.0, -1.0, 46.0, -16.0], [8.0, -8.0, 7.0, 15.0, -60.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_constData_rank3_offset0(self):
      arg0=Data(numpy.array([6.0, 3.0]),self.functionspace)
      arg1=Data(numpy.array([[[2.0, 0.0], [0.0, 2.0]], [[4.0, 4.0], [-1.0, 4.0]], [[0.0, 7.0], [3.0, -5.0]], [[-5.0, -7.0], [-6.0, -1.0]], [[5.0, 2.0], [-6.0, 7.0]], [[4.0, 7.0], [0.0, -4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[12.0, 0.0], [0.0, 12.0]], [[24.0, 24.0], [-6.0, 24.0]], [[0.0, 42.0], [18.0, -30.0]], [[-30.0, -42.0], [-36.0, -6.0]], [[30.0, 12.0], [-36.0, 42.0]], [[24.0, 42.0], [0.0, -24.0]]], [[[6.0, 0.0], [0.0, 6.0]], [[12.0, 12.0], [-3.0, 12.0]], [[0.0, 21.0], [9.0, -15.0]], [[-15.0, -21.0], [-18.0, -3.0]], [[15.0, 6.0], [-18.0, 21.0]], [[12.0, 21.0], [0.0, -12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank4_offset1(self):
      arg0=Data(numpy.array([[3.0, -3.0, 0.0], [-6.0, -7.0, 6.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[-2.0, 7.0], [-6.0, -3.0]], [[4.0, -4.0], [3.0, 4.0]], [[4.0, -6.0], [5.0, 4.0]], [[-7.0, 3.0], [2.0, 6.0]], [[-3.0, 3.0], [0.0, -2.0]], [[-7.0, -7.0], [4.0, 2.0]]], [[[4.0, 7.0], [-3.0, -1.0]], [[-1.0, -5.0], [0.0, 6.0]], [[2.0, 5.0], [4.0, 6.0]], [[0.0, 2.0], [-3.0, -1.0]], [[4.0, -1.0], [-6.0, -3.0]], [[-6.0, -7.0], [-3.0, 1.0]]], [[[0.0, 5.0], [4.0, 1.0]], [[0.0, -3.0], [1.0, 7.0]], [[3.0, 4.0], [-1.0, 4.0]], [[4.0, 7.0], [2.0, 4.0]], [[4.0, 1.0], [-7.0, -7.0]], [[-6.0, -1.0], [-2.0, 0.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-18.0, 0.0], [-9.0, -6.0]], [[15.0, 3.0], [9.0, -6.0]], [[6.0, -33.0], [3.0, -6.0]], [[-21.0, 3.0], [15.0, 21.0]], [[-21.0, 12.0], [18.0, 3.0]], [[-3.0, 0.0], [21.0, 3.0]]], [[[-16.0, -61.0], [81.0, 31.0]], [[-17.0, 41.0], [-12.0, -24.0]], [[-20.0, 25.0], [-64.0, -42.0]], [[66.0, 10.0], [21.0, -5.0]], [[14.0, -5.0], [0.0, -9.0]], [[48.0, 85.0], [-15.0, -19.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([2.0, 1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-6.0, -3.0])+(1.-msk_ref)*numpy.array([14.0, 7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[6.0, -6.0, 1.0], [-3.0, 2.0, -2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, 5.0, -3.0])+(1.-msk_arg1)*numpy.array([5.0, 6.0, -4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-27.0, 13.0])+(1.-msk_ref)*numpy.array([-10.0, 5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[7.0, 0.0, -7.0], [7.0, 0.0, -3.0]], [[4.0, 6.0, 3.0], [0.0, 0.0, -7.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 6.0, -4.0], [0.0, 4.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 2.0, -1.0], [4.0, -1.0, -3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-5.0, -16.0])+(1.-msk_ref)*numpy.array([16.0, 14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[5.0, -2.0, 4.0], [-1.0, 2.0, 3.0], [-5.0, 4.0, 6.0], [-2.0, 1.0, 4.0]], [[-5.0, 1.0, -4.0], [-2.0, -7.0, 1.0], [7.0, 0.0, -7.0], [6.0, 0.0, 0.0]]], [[[-3.0, 0.0, 6.0], [0.0, -4.0, 5.0], [2.0, 1.0, 5.0], [0.0, 1.0, 3.0]], [[-7.0, -4.0, -4.0], [7.0, -3.0, -3.0], [-2.0, -3.0, 0.0], [-1.0, -6.0, -3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, 4.0, 6.0], [-3.0, 2.0, -4.0], [-3.0, 6.0, 1.0], [-7.0, -6.0, 3.0]], [[-5.0, 7.0, -6.0], [-5.0, 3.0, -6.0], [-2.0, 5.0, -5.0], [0.0, -3.0, -2.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, -2.0, -2.0], [1.0, 3.0, 6.0], [-7.0, -3.0, 0.0], [0.0, 7.0, 1.0]], [[3.0, 2.0, 3.0], [-5.0, 4.0, -7.0], [0.0, -4.0, -7.0], [-5.0, -5.0, 7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([116.0, 46.0])+(1.-msk_ref)*numpy.array([27.0, -45.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([-7.0, -3.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, -5.0])+(1.-msk_arg1)*numpy.array([-2.0, -3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[21.0, 35.0], [9.0, 15.0]])+(1.-msk_ref)*numpy.array([[14.0, 21.0], [6.0, 9.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[0.0, 3.0, 4.0], [-5.0, -6.0, 1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 0.0], [6.0, 1.0], [-3.0, 0.0]])+(1.-msk_arg1)*numpy.array([[6.0, 3.0], [6.0, -6.0], [-5.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 3.0], [-4.0, -6.0]])+(1.-msk_ref)*numpy.array([[-2.0, 6.0], [-71.0, 27.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[6.0, 3.0, 2.0], [-7.0, -2.0, 7.0]], [[-6.0, -4.0, -7.0], [3.0, 6.0, 2.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -3.0], [1.0, 2.0], [7.0, 5.0]], [[0.0, -1.0], [-7.0, 4.0], [-4.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, -5.0], [1.0, 0.0], [2.0, 4.0]], [[-5.0, 6.0], [-6.0, -5.0], [0.0, -4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 32.0], [-103.0, 6.0]])+(1.-msk_ref)*numpy.array([[48.0, -82.0], [-63.0, -18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-7.0, 6.0, -7.0], [0.0, 2.0, 6.0], [0.0, -2.0, 0.0], [6.0, 6.0, 7.0]], [[-4.0, 6.0, -4.0], [6.0, 0.0, -2.0], [7.0, -2.0, 0.0], [0.0, 7.0, -3.0]]], [[[3.0, -2.0, 5.0], [-6.0, -7.0, -5.0], [-6.0, -6.0, 7.0], [6.0, 4.0, -5.0]], [[-3.0, -7.0, 0.0], [6.0, -5.0, -7.0], [1.0, 2.0, 1.0], [4.0, 0.0, -7.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, 0.0], [6.0, 4.0], [-7.0, -2.0]], [[6.0, 2.0], [5.0, -2.0], [-3.0, -5.0]], [[-6.0, 2.0], [-4.0, -6.0], [5.0, -2.0]], [[2.0, -4.0], [3.0, 6.0], [0.0, 5.0]]], [[[6.0, -2.0], [1.0, 1.0], [-3.0, -3.0]], [[3.0, -4.0], [3.0, 4.0], [-2.0, -3.0]], [[-5.0, 2.0], [-2.0, 3.0], [-5.0, 7.0]], [[4.0, 2.0], [2.0, -2.0], [-7.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -5.0], [3.0, 0.0], [-3.0, -7.0]], [[-3.0, 7.0], [1.0, -6.0], [1.0, 1.0]], [[-4.0, 3.0], [0.0, 1.0], [5.0, -3.0]], [[-1.0, 3.0], [-4.0, 2.0], [-7.0, 1.0]]], [[[3.0, 0.0], [0.0, -3.0], [-6.0, 7.0]], [[-1.0, 3.0], [-3.0, 7.0], [-3.0, -2.0]], [[7.0, -2.0], [0.0, 4.0], [6.0, -5.0]], [[-1.0, -5.0], [0.0, -7.0], [2.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[149.0, 47.0], [53.0, -49.0]])+(1.-msk_ref)*numpy.array([[-19.0, 33.0], [91.0, -45.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([2.0, 5.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 3.0, 3.0, 4.0, -1.0], [5.0, -3.0, 1.0, -3.0, -2.0], [0.0, -7.0, -7.0, 2.0, -4.0], [-1.0, -3.0, -2.0, -3.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-5.0, 0.0, -1.0, 6.0, 0.0], [1.0, 3.0, -4.0, 3.0, 4.0], [0.0, 6.0, -1.0, 6.0, 5.0], [-2.0, -4.0, 0.0, 2.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 6.0, 6.0, 8.0, -2.0], [10.0, -6.0, 2.0, -6.0, -4.0], [0.0, -14.0, -14.0, 4.0, -8.0], [-2.0, -6.0, -4.0, -6.0, -8.0]], [[20.0, 15.0, 15.0, 20.0, -5.0], [25.0, -15.0, 5.0, -15.0, -10.0], [0.0, -35.0, -35.0, 10.0, -20.0], [-5.0, -15.0, -10.0, -15.0, -20.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 0.0, -2.0, 12.0, 0.0], [2.0, 6.0, -8.0, 6.0, 8.0], [0.0, 12.0, -2.0, 12.0, 10.0], [-4.0, -8.0, 0.0, 4.0, 8.0]], [[-25.0, 0.0, -5.0, 30.0, 0.0], [5.0, 15.0, -20.0, 15.0, 20.0], [0.0, 30.0, -5.0, 30.0, 25.0], [-10.0, -20.0, 0.0, 10.0, 20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[2.0, 2.0, -6.0], [2.0, -5.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, -4.0, 6.0, 5.0, 5.0], [4.0, -1.0, 3.0, -4.0, 3.0], [-4.0, 2.0, -7.0, -5.0, -7.0], [3.0, -3.0, 1.0, 3.0, 0.0]], [[7.0, -1.0, 6.0, 1.0, 4.0], [4.0, 7.0, -5.0, -1.0, 3.0], [4.0, -7.0, -6.0, 7.0, -1.0], [-2.0, -3.0, 4.0, 2.0, 0.0]], [[-4.0, 4.0, 2.0, -4.0, 0.0], [7.0, 7.0, -7.0, -7.0, -2.0], [0.0, 5.0, -6.0, 2.0, -3.0], [-7.0, -6.0, -2.0, -1.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-4.0, -4.0, 6.0, 3.0, 3.0], [-3.0, 1.0, 6.0, -5.0, 5.0], [-4.0, 5.0, -1.0, 2.0, 7.0], [-5.0, 2.0, 3.0, 0.0, 0.0]], [[-6.0, 0.0, -5.0, -2.0, 5.0], [3.0, -1.0, -3.0, 6.0, -2.0], [0.0, 1.0, -1.0, 3.0, 7.0], [-2.0, -4.0, 7.0, -7.0, 2.0]], [[-4.0, 3.0, -5.0, 3.0, 6.0], [-3.0, -7.0, 2.0, -1.0, 1.0], [2.0, -5.0, -2.0, 0.0, -5.0], [-5.0, -6.0, 6.0, 2.0, -1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[46.0, -34.0, 12.0, 36.0, 18.0], [-26.0, -30.0, 38.0, 32.0, 24.0], [0.0, -40.0, 10.0, -8.0, 2.0], [44.0, 24.0, 22.0, 16.0, 30.0]], [[-27.0, -3.0, -18.0, 5.0, -10.0], [-12.0, -37.0, 31.0, -3.0, -9.0], [-28.0, 39.0, 16.0, -45.0, -9.0], [16.0, 9.0, -18.0, -4.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[4.0, -26.0, 32.0, -16.0, -20.0], [18.0, 42.0, -6.0, 8.0, 0.0], [-20.0, 42.0, 8.0, 10.0, 58.0], [16.0, 32.0, -16.0, -26.0, 10.0]], [[22.0, -8.0, 37.0, 16.0, -19.0], [-21.0, 7.0, 27.0, -40.0, 20.0], [-8.0, 5.0, 3.0, -11.0, -21.0], [0.0, 24.0, -29.0, 35.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[-7.0, 0.0, -6.0], [-5.0, 1.0, 3.0]], [[6.0, 6.0, -4.0], [5.0, 5.0, 3.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, -7.0, 2.0, 0.0, 1.0], [0.0, 2.0, 7.0, -2.0, -3.0], [4.0, -4.0, 5.0, 7.0, 5.0], [1.0, 0.0, 4.0, -5.0, -1.0]], [[3.0, -3.0, -2.0, 2.0, 4.0], [7.0, -2.0, 6.0, -2.0, -5.0], [4.0, 6.0, -7.0, 6.0, -4.0], [4.0, -6.0, -2.0, -6.0, 5.0]], [[6.0, -5.0, 6.0, -4.0, -5.0], [-6.0, -6.0, -4.0, 3.0, 4.0], [2.0, -6.0, 6.0, -3.0, -2.0], [-3.0, -3.0, 0.0, 3.0, 5.0]]], [[[-1.0, 0.0, -3.0, -5.0, 7.0], [3.0, -1.0, -2.0, -2.0, 7.0], [2.0, -3.0, -1.0, 5.0, -7.0], [-7.0, -4.0, -4.0, 2.0, 6.0]], [[0.0, 2.0, -3.0, -2.0, -2.0], [-4.0, -4.0, -2.0, -4.0, -1.0], [7.0, 7.0, -2.0, 4.0, 3.0], [1.0, -4.0, 7.0, -7.0, 2.0]], [[6.0, 6.0, -6.0, -5.0, 0.0], [5.0, 1.0, -4.0, 3.0, 1.0], [-7.0, 0.0, 4.0, 0.0, -5.0], [5.0, -4.0, 5.0, -4.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, -5.0, -2.0, -1.0, 0.0], [-5.0, -2.0, -3.0, 4.0, -2.0], [7.0, 0.0, 3.0, -2.0, 5.0], [-2.0, 1.0, 3.0, 0.0, 1.0]], [[6.0, 7.0, 6.0, -7.0, 0.0], [3.0, -1.0, 7.0, 0.0, -1.0], [2.0, 6.0, 5.0, 3.0, 1.0], [4.0, -3.0, -4.0, -3.0, -4.0]], [[-5.0, 4.0, 7.0, 0.0, -2.0], [-7.0, 0.0, 5.0, 3.0, 6.0], [0.0, 0.0, -5.0, 2.0, 4.0], [0.0, 5.0, 0.0, 1.0, -4.0]]], [[[4.0, 3.0, 6.0, 0.0, 6.0], [-7.0, -4.0, 1.0, -4.0, 4.0], [-5.0, -1.0, -5.0, 1.0, 2.0], [0.0, -1.0, -2.0, -3.0, -5.0]], [[0.0, 7.0, 5.0, -7.0, -6.0], [-7.0, -4.0, 0.0, -4.0, -1.0], [-3.0, -1.0, 0.0, 0.0, 7.0], [-4.0, -7.0, 7.0, -7.0, 5.0]], [[-7.0, -1.0, -3.0, -3.0, 0.0], [5.0, 1.0, 6.0, 7.0, -3.0], [-5.0, -5.0, -1.0, -4.0, 1.0], [5.0, 3.0, -6.0, 0.0, -6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-55.0, 99.0, -56.0, 32.0, -14.0], [32.0, 26.0, -29.0, 11.0, -36.0], [-64.0, 86.0, -56.0, -52.0, 0.0], [62.0, 22.0, 14.0, -12.0, -33.0]], [[43.0, -12.0, -72.0, -22.0, 75.0], [76.0, 2.0, 62.0, -57.0, -31.0], [64.0, 56.0, -39.0, 135.0, -21.0], [27.0, -76.0, 42.0, -115.0, 62.0]]])+(1.-msk_ref)*numpy.array([[[-60.0, 0.0, -62.0, -9.0, -24.0], [120.0, 33.0, 4.0, -9.0, -52.0], [-42.0, -11.0, 31.0, -15.0, -59.0], [25.0, -30.0, -22.0, 2.0, 29.0]], [[97.0, 43.0, 42.0, -92.0, 8.0], [-39.0, -55.0, 27.0, -7.0, -36.0], [-1.0, 11.0, 40.0, -9.0, 68.0], [7.0, -63.0, 1.0, -72.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank1_expandedData_rank3_offset0(self):
      arg0=Data(numpy.array([7.0, 7.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, -6.0], [-2.0, -2.0]], [[4.0, 6.0], [-1.0, -3.0]], [[-3.0, -1.0], [1.0, 6.0]], [[0.0, 3.0], [7.0, 5.0]], [[-7.0, -7.0], [4.0, 4.0]], [[-5.0, -3.0], [7.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -6.0], [4.0, 5.0]], [[-5.0, 3.0], [-2.0, 0.0]], [[0.0, -7.0], [-7.0, 3.0]], [[0.0, 3.0], [-5.0, -4.0]], [[-6.0, 6.0], [2.0, 0.0]], [[-6.0, 3.0], [-2.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[21.0, -42.0], [-14.0, -14.0]], [[28.0, 42.0], [-7.0, -21.0]], [[-21.0, -7.0], [7.0, 42.0]], [[0.0, 21.0], [49.0, 35.0]], [[-49.0, -49.0], [28.0, 28.0]], [[-35.0, -21.0], [49.0, 35.0]]], [[[21.0, -42.0], [-14.0, -14.0]], [[28.0, 42.0], [-7.0, -21.0]], [[-21.0, -7.0], [7.0, 42.0]], [[0.0, 21.0], [49.0, 35.0]], [[-49.0, -49.0], [28.0, 28.0]], [[-35.0, -21.0], [49.0, 35.0]]]])+(1.-msk_ref)*numpy.array([[[[-35.0, -42.0], [28.0, 35.0]], [[-35.0, 21.0], [-14.0, 0.0]], [[0.0, -49.0], [-49.0, 21.0]], [[0.0, 21.0], [-35.0, -28.0]], [[-42.0, 42.0], [14.0, 0.0]], [[-42.0, 21.0], [-14.0, 0.0]]], [[[-35.0, -42.0], [28.0, 35.0]], [[-35.0, 21.0], [-14.0, 0.0]], [[0.0, -49.0], [-49.0, 21.0]], [[0.0, 21.0], [-35.0, -28.0]], [[-42.0, 42.0], [14.0, 0.0]], [[-42.0, 21.0], [-14.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([[-1.0, 7.0, -2.0], [-4.0, -2.0, -5.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -2.0], [3.0, 6.0]], [[5.0, 2.0], [3.0, -4.0]], [[6.0, -1.0], [-5.0, -4.0]], [[6.0, 0.0], [0.0, 0.0]], [[-4.0, 0.0], [4.0, 5.0]], [[0.0, -1.0], [-4.0, 0.0]]], [[[4.0, 6.0], [6.0, 1.0]], [[-4.0, 2.0], [-5.0, 2.0]], [[-3.0, 6.0], [0.0, 0.0]], [[-1.0, -2.0], [-5.0, -4.0]], [[-1.0, -3.0], [5.0, 0.0]], [[-7.0, -4.0], [5.0, -1.0]]], [[[-1.0, 6.0], [-2.0, -6.0]], [[-2.0, 0.0], [-4.0, -3.0]], [[6.0, -2.0], [2.0, -3.0]], [[6.0, 1.0], [-5.0, -3.0]], [[6.0, 0.0], [0.0, 2.0]], [[6.0, -2.0], [0.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -6.0], [6.0, 7.0]], [[5.0, -3.0], [-7.0, -2.0]], [[0.0, -1.0], [-5.0, -2.0]], [[-5.0, 0.0], [-3.0, -5.0]], [[7.0, -3.0], [3.0, 1.0]], [[2.0, 5.0], [2.0, 3.0]]], [[[2.0, -3.0], [-6.0, 5.0]], [[7.0, -4.0], [6.0, -4.0]], [[6.0, -5.0], [4.0, 0.0]], [[6.0, 3.0], [-2.0, -1.0]], [[6.0, 5.0], [-4.0, -6.0]], [[7.0, 2.0], [-6.0, -2.0]]], [[[5.0, -3.0], [0.0, 4.0]], [[-7.0, 0.0], [1.0, 2.0]], [[-4.0, -6.0], [-3.0, 0.0]], [[5.0, -5.0], [2.0, 7.0]], [[-2.0, 7.0], [5.0, -7.0]], [[-6.0, -7.0], [-4.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[32.0, 32.0], [43.0, 13.0]], [[-29.0, 12.0], [-30.0, 24.0]], [[-39.0, 47.0], [1.0, 10.0]], [[-25.0, -16.0], [-25.0, -22.0]], [[-15.0, -21.0], [31.0, -9.0]], [[-61.0, -23.0], [39.0, -7.0]]], [[[5.0, -34.0], [-14.0, 4.0]], [[-2.0, -12.0], [18.0, 27.0]], [[-48.0, 2.0], [10.0, 31.0]], [[-52.0, -1.0], [35.0, 23.0]], [[-12.0, 6.0], [-26.0, -30.0]], [[-16.0, 22.0], [6.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, -9.0], [-48.0, 20.0]], [[58.0, -25.0], [47.0, -30.0]], [[50.0, -22.0], [39.0, 2.0]], [[37.0, 31.0], [-15.0, -16.0]], [[39.0, 24.0], [-41.0, -29.0]], [[59.0, 23.0], [-36.0, -19.0]]], [[[-29.0, 45.0], [-12.0, -58.0]], [[1.0, 20.0], [11.0, 6.0]], [[8.0, 44.0], [27.0, 8.0]], [[-17.0, 19.0], [6.0, -13.0]], [[-30.0, -33.0], [-29.0, 43.0]], [[8.0, 11.0], [24.0, -13.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_float_rank0_offset0(self):
      arg0=Data(numpy.array([[4.0, -1.0, 6.0, -6.0, 0.0], [-1.0, -2.0, -1.0, 7.0, -1.0], [0.0, 4.0, 1.0, 6.0, -7.0], [0.0, 5.0, -2.0, 6.0, -1.0]]),self.functionspace)
      arg1=-3.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-12.0, 3.0, -18.0, 18.0, -0.0], [3.0, 6.0, 3.0, -21.0, 3.0], [-0.0, -12.0, -3.0, -18.0, 21.0], [-0.0, -15.0, 6.0, -18.0, 3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank0_offset0(self):
      arg0=Data(numpy.array([[-7.0, -5.0, 6.0, -5.0, 4.0], [-4.0, 3.0, -4.0, 1.0, -5.0], [4.0, -2.0, -3.0, 0.0, -5.0], [2.0, -5.0, -2.0, 4.0, 4.0]]),self.functionspace)
      arg1=numpy.array(-4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[28.0, 20.0, -24.0, 20.0, -16.0], [16.0, -12.0, 16.0, -4.0, 20.0], [-16.0, 8.0, 12.0, -0.0, 20.0], [-8.0, 20.0, 8.0, -16.0, -16.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[0.0, 3.0, 5.0], [6.0, -5.0, -2.0], [4.0, 3.0, 7.0], [-2.0, -2.0, -7.0], [3.0, 4.0, 6.0]], [[-1.0, 7.0, 5.0], [7.0, 7.0, 0.0], [4.0, 5.0, 0.0], [2.0, -1.0, 2.0], [3.0, -2.0, -5.0]], [[3.0, -5.0, -6.0], [2.0, -6.0, -2.0], [4.0, 7.0, 0.0], [0.0, 7.0, -5.0], [6.0, -1.0, 7.0]], [[7.0, 1.0, -6.0], [0.0, -1.0, -3.0], [-5.0, 3.0, 3.0], [7.0, -1.0, 7.0], [-3.0, -4.0, 6.0]]]),self.functionspace)
      arg1=numpy.array([5.0, 2.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[6.0, 20.0, 26.0, -14.0, 23.0], [9.0, 49.0, 30.0, 8.0, 11.0], [5.0, -2.0, 34.0, 14.0, 28.0], [37.0, -2.0, -19.0, 33.0, -23.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[[-3.0, -2.0, 0.0], [3.0, 1.0, 5.0]], [[5.0, 4.0, -6.0], [4.0, -6.0, 2.0]], [[0.0, 0.0, 1.0], [6.0, -6.0, 7.0]], [[-2.0, -7.0, -5.0], [-1.0, 0.0, -3.0]], [[-5.0, -3.0, -2.0], [4.0, -1.0, 7.0]]], [[[-1.0, 6.0, 0.0], [-3.0, -2.0, 5.0]], [[4.0, 2.0, -4.0], [1.0, -6.0, 7.0]], [[3.0, -1.0, -5.0], [2.0, -7.0, 0.0]], [[1.0, 0.0, -1.0], [-2.0, 0.0, -5.0]], [[-5.0, -1.0, -2.0], [0.0, -6.0, -1.0]]], [[[1.0, 0.0, 7.0], [-1.0, 3.0, -6.0]], [[-7.0, -3.0, 3.0], [6.0, -1.0, 2.0]], [[0.0, -1.0, 7.0], [-7.0, -3.0, 4.0]], [[-6.0, 5.0, 1.0], [-5.0, 1.0, 2.0]], [[6.0, 3.0, 0.0], [-4.0, 6.0, -1.0]]], [[[-5.0, 0.0, -3.0], [-6.0, -1.0, -6.0]], [[-7.0, 0.0, 5.0], [4.0, 6.0, -3.0]], [[6.0, -2.0, 1.0], [3.0, 0.0, 0.0]], [[-3.0, 5.0, 4.0], [-2.0, 4.0, -4.0]], [[-2.0, -3.0, -2.0], [7.0, -6.0, -7.0]]]]),self.functionspace)
      arg1=numpy.array([[0.0, 1.0, -5.0], [-4.0, 0.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[6.0, 26.0, -1.0, 10.0, 19.0], [38.0, 46.0, 16.0, -7.0, 5.0], [-55.0, -34.0, 8.0, 28.0, 15.0], [15.0, -53.0, -19.0, -23.0, -49.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank1_offset0(self):
      arg0=Data(numpy.array([[-5.0, -4.0, 6.0, -2.0, 3.0], [7.0, 0.0, 6.0, -2.0, 3.0], [-6.0, -6.0, -2.0, -2.0, 0.0], [-6.0, -5.0, 7.0, 6.0, -5.0]]),self.functionspace)
      arg1=numpy.array([-5.0, -3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[25.0, 15.0], [20.0, 12.0], [-30.0, -18.0], [10.0, 6.0], [-15.0, -9.0]], [[-35.0, -21.0], [0.0, 0.0], [-30.0, -18.0], [10.0, 6.0], [-15.0, -9.0]], [[30.0, 18.0], [30.0, 18.0], [10.0, 6.0], [10.0, 6.0], [0.0, 0.0]], [[30.0, 18.0], [25.0, 15.0], [-35.0, -21.0], [-30.0, -18.0], [25.0, 15.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[-3.0, -3.0, -6.0], [-5.0, 1.0, 0.0], [-7.0, 3.0, 0.0], [7.0, -6.0, 7.0], [-4.0, -1.0, -5.0]], [[4.0, -1.0, 0.0], [3.0, 0.0, 6.0], [0.0, -3.0, -7.0], [4.0, -1.0, 6.0], [-1.0, 7.0, -3.0]], [[-6.0, 1.0, 0.0], [-4.0, -6.0, 4.0], [0.0, 0.0, 0.0], [3.0, -3.0, 1.0], [-1.0, 0.0, -4.0]], [[3.0, -1.0, -7.0], [7.0, -6.0, 7.0], [6.0, 0.0, -1.0], [3.0, 4.0, 3.0], [-2.0, -6.0, -3.0]]]),self.functionspace)
      arg1=numpy.array([[0.0, -1.0], [0.0, -4.0], [4.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-24.0, 51.0], [0.0, 1.0], [0.0, -5.0], [28.0, -25.0], [-20.0, 38.0]], [[0.0, 0.0], [24.0, -39.0], [-28.0, 54.0], [24.0, -36.0], [-12.0, -9.0]], [[0.0, 2.0], [16.0, 4.0], [0.0, 0.0], [4.0, 3.0], [-16.0, 25.0]], [[-28.0, 43.0], [28.0, -25.0], [-4.0, 0.0], [12.0, -37.0], [-12.0, 44.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[[5.0, 0.0, -6.0], [-7.0, 2.0, -6.0]], [[-7.0, 4.0, 0.0], [4.0, 0.0, 1.0]], [[3.0, -5.0, 0.0], [-6.0, -4.0, -6.0]], [[-2.0, -6.0, 6.0], [-3.0, 2.0, 2.0]], [[6.0, -6.0, 1.0], [1.0, -4.0, -3.0]]], [[[0.0, 0.0, 4.0], [-3.0, 6.0, -6.0]], [[-6.0, -5.0, 4.0], [6.0, -6.0, -6.0]], [[7.0, -2.0, 0.0], [0.0, -6.0, -1.0]], [[-7.0, -4.0, -2.0], [-5.0, -3.0, 6.0]], [[-5.0, 2.0, 0.0], [-6.0, 2.0, 2.0]]], [[[-2.0, -4.0, 2.0], [3.0, 6.0, -6.0]], [[6.0, 0.0, -5.0], [-6.0, 0.0, 4.0]], [[0.0, 0.0, -5.0], [4.0, 2.0, -2.0]], [[2.0, -1.0, 4.0], [0.0, -5.0, -3.0]], [[4.0, -3.0, -1.0], [7.0, 7.0, -2.0]]], [[[-6.0, 6.0, -3.0], [6.0, -2.0, 3.0]], [[1.0, -1.0, 2.0], [3.0, -3.0, -3.0]], [[2.0, -6.0, 2.0], [-7.0, 7.0, -7.0]], [[1.0, -3.0, 1.0], [-6.0, -6.0, -2.0]], [[6.0, -3.0, -2.0], [-7.0, 5.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[[4.0, 6.0], [0.0, 4.0], [-1.0, 4.0]], [[-4.0, 4.0], [4.0, 7.0], [-2.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[74.0, 10.0], [-46.0, -13.0], [32.0, -36.0], [2.0, -16.0], [9.0, 1.0]], [[44.0, 64.0], [-64.0, -40.0], [6.0, -5.0], [-30.0, -125.0], [8.0, -38.0]], [[14.0, 52.0], [45.0, -20.0], [1.0, 16.0], [-10.0, -2.0], [21.0, 91.0]], [[-59.0, -23.0], [-16.0, 10.0], [76.0, 38.0], [7.0, -62.0], [74.0, 23.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_array_rank2_offset0(self):
      arg0=Data(numpy.array([[1.0, 4.0, 6.0, 1.0, 3.0], [-1.0, 2.0, 0.0, -3.0, -3.0], [7.0, 4.0, -6.0, -3.0, 4.0], [-7.0, -6.0, 4.0, 0.0, 4.0]]),self.functionspace)
      arg1=numpy.array([[-7.0, 6.0, 7.0, 5.0, -1.0], [-3.0, -1.0, 6.0, -7.0, 3.0], [-1.0, -1.0, 3.0, 6.0, 7.0], [-4.0, 7.0, 0.0, 6.0, -7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-7.0, 6.0, 7.0, 5.0, -1.0], [-3.0, -1.0, 6.0, -7.0, 3.0], [-1.0, -1.0, 3.0, 6.0, 7.0], [-4.0, 7.0, 0.0, 6.0, -7.0]], [[-28.0, 24.0, 28.0, 20.0, -4.0], [-12.0, -4.0, 24.0, -28.0, 12.0], [-4.0, -4.0, 12.0, 24.0, 28.0], [-16.0, 28.0, 0.0, 24.0, -28.0]], [[-42.0, 36.0, 42.0, 30.0, -6.0], [-18.0, -6.0, 36.0, -42.0, 18.0], [-6.0, -6.0, 18.0, 36.0, 42.0], [-24.0, 42.0, 0.0, 36.0, -42.0]], [[-7.0, 6.0, 7.0, 5.0, -1.0], [-3.0, -1.0, 6.0, -7.0, 3.0], [-1.0, -1.0, 3.0, 6.0, 7.0], [-4.0, 7.0, 0.0, 6.0, -7.0]], [[-21.0, 18.0, 21.0, 15.0, -3.0], [-9.0, -3.0, 18.0, -21.0, 9.0], [-3.0, -3.0, 9.0, 18.0, 21.0], [-12.0, 21.0, 0.0, 18.0, -21.0]]], [[[7.0, -6.0, -7.0, -5.0, 1.0], [3.0, 1.0, -6.0, 7.0, -3.0], [1.0, 1.0, -3.0, -6.0, -7.0], [4.0, -7.0, 0.0, -6.0, 7.0]], [[-14.0, 12.0, 14.0, 10.0, -2.0], [-6.0, -2.0, 12.0, -14.0, 6.0], [-2.0, -2.0, 6.0, 12.0, 14.0], [-8.0, 14.0, 0.0, 12.0, -14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[21.0, -18.0, -21.0, -15.0, 3.0], [9.0, 3.0, -18.0, 21.0, -9.0], [3.0, 3.0, -9.0, -18.0, -21.0], [12.0, -21.0, 0.0, -18.0, 21.0]], [[21.0, -18.0, -21.0, -15.0, 3.0], [9.0, 3.0, -18.0, 21.0, -9.0], [3.0, 3.0, -9.0, -18.0, -21.0], [12.0, -21.0, 0.0, -18.0, 21.0]]], [[[-49.0, 42.0, 49.0, 35.0, -7.0], [-21.0, -7.0, 42.0, -49.0, 21.0], [-7.0, -7.0, 21.0, 42.0, 49.0], [-28.0, 49.0, 0.0, 42.0, -49.0]], [[-28.0, 24.0, 28.0, 20.0, -4.0], [-12.0, -4.0, 24.0, -28.0, 12.0], [-4.0, -4.0, 12.0, 24.0, 28.0], [-16.0, 28.0, 0.0, 24.0, -28.0]], [[42.0, -36.0, -42.0, -30.0, 6.0], [18.0, 6.0, -36.0, 42.0, -18.0], [6.0, 6.0, -18.0, -36.0, -42.0], [24.0, -42.0, 0.0, -36.0, 42.0]], [[21.0, -18.0, -21.0, -15.0, 3.0], [9.0, 3.0, -18.0, 21.0, -9.0], [3.0, 3.0, -9.0, -18.0, -21.0], [12.0, -21.0, 0.0, -18.0, 21.0]], [[-28.0, 24.0, 28.0, 20.0, -4.0], [-12.0, -4.0, 24.0, -28.0, 12.0], [-4.0, -4.0, 12.0, 24.0, 28.0], [-16.0, 28.0, 0.0, 24.0, -28.0]]], [[[49.0, -42.0, -49.0, -35.0, 7.0], [21.0, 7.0, -42.0, 49.0, -21.0], [7.0, 7.0, -21.0, -42.0, -49.0], [28.0, -49.0, 0.0, -42.0, 49.0]], [[42.0, -36.0, -42.0, -30.0, 6.0], [18.0, 6.0, -36.0, 42.0, -18.0], [6.0, 6.0, -18.0, -36.0, -42.0], [24.0, -42.0, 0.0, -36.0, 42.0]], [[-28.0, 24.0, 28.0, 20.0, -4.0], [-12.0, -4.0, 24.0, -28.0, 12.0], [-4.0, -4.0, 12.0, 24.0, 28.0], [-16.0, 28.0, 0.0, 24.0, -28.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-28.0, 24.0, 28.0, 20.0, -4.0], [-12.0, -4.0, 24.0, -28.0, 12.0], [-4.0, -4.0, 12.0, 24.0, 28.0], [-16.0, 28.0, 0.0, 24.0, -28.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank3_offset1(self):
      arg0=Data(numpy.array([[[0.0, -5.0, 0.0], [-5.0, -3.0, 7.0], [5.0, 6.0, -4.0], [3.0, -4.0, 7.0], [0.0, -1.0, -4.0]], [[4.0, -5.0, -6.0], [-7.0, 2.0, -6.0], [3.0, -6.0, -5.0], [1.0, 6.0, 2.0], [5.0, -3.0, 2.0]], [[0.0, 2.0, -5.0], [5.0, 4.0, -3.0], [-6.0, 7.0, 2.0], [0.0, -4.0, -7.0], [1.0, 1.0, -4.0]], [[2.0, -5.0, 5.0], [6.0, 7.0, -5.0], [0.0, 6.0, 4.0], [1.0, 0.0, 4.0], [0.0, -7.0, 3.0]]]),self.functionspace)
      arg1=numpy.array([[[-7.0, -2.0, 7.0, 0.0, -4.0], [-6.0, -2.0, 5.0, 6.0, -3.0], [0.0, -2.0, 6.0, -1.0, -3.0], [0.0, -5.0, 2.0, -7.0, 0.0]], [[-1.0, 7.0, 3.0, 3.0, 3.0], [-2.0, -7.0, -2.0, 2.0, 4.0], [-6.0, 2.0, 3.0, -2.0, -3.0], [-4.0, 5.0, -1.0, 3.0, 1.0]], [[6.0, -1.0, -7.0, -2.0, -4.0], [-1.0, -3.0, 2.0, -1.0, -1.0], [-5.0, -3.0, 1.0, -3.0, -7.0], [-4.0, -2.0, 4.0, 0.0, 1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[5.0, -35.0, -15.0, -15.0, -15.0], [10.0, 35.0, 10.0, -10.0, -20.0], [30.0, -10.0, -15.0, 10.0, 15.0], [20.0, -25.0, 5.0, -15.0, -5.0]], [[80.0, -18.0, -93.0, -23.0, -17.0], [29.0, 10.0, -5.0, -43.0, -4.0], [-17.0, -17.0, -32.0, -10.0, -25.0], [-16.0, -4.0, 21.0, 26.0, 4.0]], [[-65.0, 36.0, 81.0, 26.0, 14.0], [-38.0, -40.0, 5.0, 46.0, 13.0], [-16.0, 14.0, 44.0, -5.0, -5.0], [-8.0, 13.0, -12.0, -17.0, 2.0]], [[25.0, -41.0, -40.0, -26.0, -52.0], [-17.0, 1.0, 37.0, 3.0, -32.0], [-11.0, -35.0, 13.0, -16.0, -46.0], [-12.0, -49.0, 38.0, -33.0, 3.0]], [[-23.0, -3.0, 25.0, 5.0, 13.0], [6.0, 19.0, -6.0, 2.0, 0.0], [26.0, 10.0, -7.0, 14.0, 31.0], [20.0, 3.0, -15.0, -3.0, -5.0]]], [[[-59.0, -37.0, 55.0, -3.0, -7.0], [-8.0, 45.0, 18.0, 20.0, -26.0], [60.0, 0.0, 3.0, 24.0, 45.0], [44.0, -33.0, -11.0, -43.0, -11.0]], [[11.0, 34.0, -1.0, 18.0, 58.0], [44.0, 18.0, -51.0, -32.0, 35.0], [18.0, 36.0, -42.0, 21.0, 57.0], [16.0, 57.0, -40.0, 55.0, -4.0]], [[-45.0, -43.0, 38.0, -8.0, -10.0], [-1.0, 51.0, 17.0, 11.0, -28.0], [61.0, -3.0, -5.0, 24.0, 44.0], [44.0, -35.0, -8.0, -39.0, -11.0]], [[-1.0, 38.0, 11.0, 14.0, 6.0], [-20.0, -50.0, -3.0, 16.0, 19.0], [-46.0, 4.0, 26.0, -19.0, -35.0], [-32.0, 21.0, 4.0, 11.0, 8.0]], [[-20.0, -33.0, 12.0, -13.0, -37.0], [-26.0, 5.0, 35.0, 22.0, -29.0], [8.0, -22.0, 23.0, -5.0, -20.0], [4.0, -44.0, 21.0, -44.0, -1.0]]], [[[-32.0, 19.0, 41.0, 16.0, 26.0], [1.0, 1.0, -14.0, 9.0, 13.0], [13.0, 19.0, 1.0, 11.0, 29.0], [12.0, 20.0, -22.0, 6.0, -3.0]], [[-57.0, 21.0, 68.0, 18.0, 4.0], [-35.0, -29.0, 11.0, 41.0, 4.0], [-9.0, 7.0, 39.0, -4.0, -6.0], [-4.0, 1.0, -6.0, -23.0, 1.0]], [[47.0, 59.0, -35.0, 17.0, 37.0], [20.0, -43.0, -40.0, -24.0, 44.0], [-52.0, 20.0, -13.0, -14.0, -17.0], [-36.0, 61.0, -11.0, 63.0, 9.0]], [[-38.0, -21.0, 37.0, 2.0, 16.0], [15.0, 49.0, -6.0, -1.0, -9.0], [59.0, 13.0, -19.0, 29.0, 61.0], [44.0, -6.0, -24.0, -12.0, -11.0]], [[-32.0, 9.0, 38.0, 11.0, 15.0], [-4.0, 3.0, -5.0, 12.0, 5.0], [14.0, 12.0, 5.0, 9.0, 22.0], [12.0, 8.0, -15.0, -4.0, -3.0]]], [[[21.0, -44.0, -36.0, -25.0, -43.0], [-7.0, 16.0, 30.0, -3.0, -31.0], [5.0, -29.0, 2.0, -7.0, -26.0], [0.0, -45.0, 29.0, -29.0, 0.0]], [[-79.0, 42.0, 98.0, 31.0, 17.0], [-45.0, -46.0, 6.0, 55.0, 15.0], [-17.0, 17.0, 52.0, -5.0, -4.0], [-8.0, 15.0, -15.0, -21.0, 2.0]], [[18.0, 38.0, -10.0, 10.0, 2.0], [-16.0, -54.0, -4.0, 8.0, 20.0], [-56.0, 0.0, 22.0, -24.0, -46.0], [-40.0, 22.0, 10.0, 18.0, 10.0]], [[17.0, -6.0, -21.0, -8.0, -20.0], [-10.0, -14.0, 13.0, 2.0, -7.0], [-20.0, -14.0, 10.0, -13.0, -31.0], [-16.0, -13.0, 18.0, -7.0, 4.0]], [[25.0, -52.0, -42.0, -27.0, -33.0], [11.0, 40.0, 20.0, -17.0, -31.0], [27.0, -23.0, -18.0, 5.0, 0.0], [16.0, -41.0, 19.0, -21.0, -4.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-6.0, 7.0, 1.0], [0.0, 0.0, -5.0]], [[-4.0, 7.0, -5.0], [2.0, 0.0, -7.0]], [[3.0, -1.0, 4.0], [-1.0, 1.0, -5.0]], [[6.0, 2.0, 2.0], [-6.0, 1.0, 7.0]], [[7.0, 7.0, 7.0], [4.0, -2.0, 7.0]]], [[[-6.0, 0.0, -6.0], [-4.0, -7.0, 1.0]], [[0.0, -7.0, -7.0], [-4.0, 3.0, -3.0]], [[-3.0, 4.0, 0.0], [5.0, 0.0, 3.0]], [[-4.0, -7.0, -5.0], [1.0, 4.0, -4.0]], [[7.0, 7.0, 7.0], [-1.0, 1.0, 3.0]]], [[[-4.0, -2.0, 4.0], [2.0, 5.0, 7.0]], [[1.0, 0.0, -7.0], [0.0, -3.0, -1.0]], [[4.0, 0.0, -3.0], [-7.0, 5.0, -7.0]], [[-5.0, -1.0, 0.0], [-7.0, 3.0, 0.0]], [[0.0, -1.0, -7.0], [7.0, 5.0, -5.0]]], [[[-1.0, 7.0, -6.0], [-3.0, 5.0, 5.0]], [[4.0, 0.0, 4.0], [5.0, -7.0, 3.0]], [[4.0, -6.0, 5.0], [6.0, 5.0, 1.0]], [[-7.0, -1.0, -7.0], [0.0, -5.0, -7.0]], [[6.0, 0.0, 4.0], [4.0, 4.0, -6.0]]]]),self.functionspace)
      arg1=numpy.array([[[[1.0, 0.0, 2.0, -6.0, -4.0], [2.0, 6.0, 1.0, 0.0, 3.0], [-4.0, 4.0, -5.0, 3.0, -1.0], [-6.0, -7.0, 3.0, -7.0, -1.0]], [[5.0, 1.0, -7.0, 2.0, -5.0], [4.0, 5.0, 0.0, 7.0, -1.0], [0.0, 4.0, -6.0, 0.0, -3.0], [0.0, -7.0, -4.0, 7.0, -2.0]], [[-1.0, 3.0, -7.0, -3.0, -2.0], [6.0, -2.0, -4.0, 1.0, -5.0], [4.0, -3.0, 4.0, 0.0, 1.0], [0.0, 5.0, 0.0, 0.0, -6.0]]], [[[1.0, 1.0, -6.0, 6.0, -3.0], [-7.0, -1.0, -3.0, -7.0, 7.0], [7.0, 6.0, -2.0, 5.0, -4.0], [-4.0, 5.0, 4.0, -5.0, 0.0]], [[-1.0, 6.0, 0.0, -5.0, 4.0], [7.0, 7.0, -2.0, -7.0, -6.0], [-2.0, 0.0, -5.0, 0.0, 2.0], [-1.0, -4.0, 7.0, 4.0, -5.0]], [[4.0, 7.0, -7.0, 6.0, -3.0], [-2.0, 7.0, -3.0, 0.0, 4.0], [6.0, 0.0, -7.0, 5.0, -2.0], [3.0, 0.0, -3.0, 0.0, 3.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[8.0, -25.0, -33.0, 17.0, 2.0], [32.0, -38.0, 5.0, 50.0, -50.0], [-2.0, 1.0, 27.0, -43.0, -4.0], [21.0, -2.0, -31.0, 91.0, -29.0]], [[10.0, -55.0, 15.0, 23.0, 6.0], [-10.0, -30.0, 31.0, 30.0, -8.0], [-32.0, 39.0, 3.0, -37.0, -16.0], [-5.0, -36.0, -11.0, 67.0, -1.0]], [[-28.0, -19.0, 26.0, -73.0, 7.0], [50.0, -22.0, 3.0, -3.0, -43.0], [-35.0, -10.0, 39.0, -21.0, 20.0], [-30.0, -3.0, 31.0, -19.0, -45.0]], [[35.0, 57.0, -29.0, -37.0, -37.0], [67.0, 104.0, -7.0, 51.0, -14.0], [-18.0, -10.0, -76.0, 23.0, 2.0], [8.0, -80.0, -28.0, 6.0, -6.0]], [[69.0, 69.0, -157.0, 27.0, -118.0], [28.0, 94.0, -50.0, 42.0, 47.0], [74.0, 59.0, -96.0, 76.0, -55.0], [-35.0, -35.0, -26.0, -28.0, -32.0]]], [[[7.0, -57.0, 47.0, 71.0, 17.0], [-71.0, -62.0, 41.0, 71.0, 30.0], [-8.0, -30.0, 42.0, -33.0, 0.0], [62.0, 20.0, -86.0, 34.0, 80.0]], [[-47.0, -35.0, 143.0, -50.0, 82.0], [-15.0, -17.0, 43.0, -49.0, -16.0], [-80.0, -31.0, 28.0, -35.0, 42.0], [4.0, -18.0, 42.0, -17.0, 32.0]], [[34.0, 30.0, -85.0, 74.0, -32.0], [-31.0, 18.0, -27.0, -7.0, 34.0], [65.0, 34.0, -40.0, 31.0, -35.0], [7.0, 18.0, -14.0, 24.0, 4.0]], [[-53.0, -25.0, 98.0, -13.0, 86.0], [-37.0, -50.0, 17.0, -89.0, -13.0], [-29.0, -23.0, 48.0, -27.0, 32.0], [4.0, 41.0, 60.0, -10.0, 16.0]], [[45.0, 54.0, -99.0, -42.0, -79.0], [92.0, 92.0, -29.0, 56.0, -22.0], [9.0, 29.0, -73.0, 31.0, -21.0], [-30.0, -72.0, -13.0, 9.0, -59.0]]], [[[7.0, 91.0, -83.0, 37.0, 11.0], [15.0, 40.0, -57.0, -59.0, -18.0], [78.0, -24.0, -30.0, 33.0, 2.0], [32.0, 52.0, 18.0, 24.0, -20.0]], [[7.0, -46.0, 58.0, 24.0, 1.0], [-59.0, -8.0, 38.0, 14.0, 52.0], [-32.0, 25.0, -11.0, -2.0, -12.0], [-6.0, -30.0, -15.0, -19.0, 53.0]], [[-33.0, -35.0, 120.0, -124.0, 52.0], [88.0, 23.0, 48.0, 11.0, -80.0], [-129.0, -17.0, 6.0, -58.0, 45.0], [-22.0, -98.0, 40.0, 27.0, -32.0]], [[-20.0, 10.0, 39.0, -29.0, 58.0], [56.0, -7.0, 10.0, 21.0, -81.0], [-35.0, -66.0, 30.0, -50.0, 42.0], [55.0, -5.0, -18.0, 75.0, -8.0]], [[-16.0, -20.0, 49.0, 6.0, 33.0], [-50.0, 2.0, 12.0, -98.0, 35.0], [-19.0, 59.0, -26.0, 10.0, -12.0], [-48.0, -13.0, 82.0, -22.0, 4.0]]], [[[52.0, 51.0, -26.0, 25.0, -5.0], [36.0, 114.0, 7.0, 29.0, -11.0], [-21.0, 24.0, -115.0, 7.0, -14.0], [28.0, -107.0, -23.0, 91.0, 13.0]], [[24.0, -4.0, -71.0, 47.0, -76.0], [-58.0, -17.0, -22.0, 18.0, 81.0], [67.0, 34.0, 0.0, 52.0, -40.0], [-28.0, 45.0, -26.0, -81.0, 16.0]], [[-26.0, 52.0, -28.0, -34.0, 3.0], [5.0, 20.0, -47.0, -114.0, 9.0], [42.0, 13.0, -8.0, 47.0, 3.0], [-50.0, 49.0, 92.0, -80.0, -44.0]], [[-28.0, -101.0, 91.0, 44.0, 48.0], [-81.0, -117.0, 52.0, 21.0, 17.0], [-32.0, -11.0, 87.0, -56.0, 7.0], [26.0, 41.0, -31.0, 22.0, 55.0]], [[-22.0, -2.0, 2.0, -80.0, -10.0], [48.0, 10.0, -12.0, -52.0, -22.0], [-24.0, 36.0, 0.0, 8.0, 2.0], [-74.0, -18.0, 80.0, -46.0, -68.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[2.0, 1.0, 1.0, 6.0, 4.0], [5.0, -6.0, -7.0, 7.0, 6.0], [-7.0, -5.0, -4.0, -4.0, 0.0], [2.0, 2.0, -5.0, -1.0, 6.0]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(6.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[12.0, 6.0, 6.0, 36.0, 24.0], [30.0, -36.0, -42.0, 42.0, 36.0], [-42.0, -30.0, -24.0, -24.0, 0.0], [12.0, 12.0, -30.0, -6.0, 36.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[3.0, 5.0, 0.0], [-3.0, 7.0, -2.0], [5.0, 0.0, -1.0], [7.0, -3.0, -6.0], [0.0, -3.0, -2.0]], [[-5.0, 0.0, 0.0], [-3.0, -4.0, 0.0], [4.0, -4.0, -7.0], [-4.0, -7.0, 7.0], [-4.0, -4.0, -6.0]], [[-4.0, 4.0, 0.0], [-6.0, 0.0, 2.0], [-3.0, 4.0, 5.0], [7.0, -5.0, 3.0], [0.0, 0.0, 0.0]], [[-6.0, -7.0, 3.0], [2.0, 3.0, 4.0], [0.0, -3.0, 7.0], [-1.0, 0.0, -4.0], [1.0, -7.0, -6.0]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-3.0, -7.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-44.0, -40.0, -15.0, 0.0, 21.0], [15.0, 37.0, 16.0, 61.0, 40.0], [-16.0, 18.0, -19.0, 14.0, 0.0], [67.0, -27.0, 21.0, 3.0, 46.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[[7.0, -3.0, 7.0], [-6.0, 2.0, 0.0]], [[4.0, 3.0, 2.0], [7.0, -4.0, 1.0]], [[-4.0, 7.0, -3.0], [2.0, 2.0, -7.0]], [[1.0, 6.0, -4.0], [-3.0, -1.0, 4.0]], [[1.0, 6.0, 1.0], [-7.0, -2.0, -6.0]]], [[[-4.0, 4.0, -5.0], [2.0, -3.0, -3.0]], [[-1.0, 7.0, 0.0], [-1.0, -3.0, 3.0]], [[-1.0, -2.0, -1.0], [5.0, 2.0, 4.0]], [[7.0, 0.0, 4.0], [-2.0, 1.0, -6.0]], [[-7.0, -7.0, 6.0], [0.0, -5.0, 3.0]]], [[[6.0, 6.0, -7.0], [-2.0, 3.0, -6.0]], [[0.0, 7.0, 1.0], [-6.0, 2.0, -6.0]], [[-3.0, 0.0, 5.0], [-2.0, -1.0, -3.0]], [[-2.0, -3.0, -5.0], [-4.0, -7.0, -7.0]], [[3.0, -7.0, 0.0], [-6.0, -7.0, 1.0]]], [[[0.0, 3.0, -2.0], [6.0, -2.0, -1.0]], [[7.0, -2.0, -4.0], [6.0, -2.0, 5.0]], [[-4.0, 3.0, 7.0], [-5.0, 2.0, 3.0]], [[-2.0, 3.0, -5.0], [7.0, -4.0, -4.0]], [[0.0, 6.0, 5.0], [7.0, -6.0, 6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[6.0, 5.0, 6.0], [3.0, -2.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[47.0, 87.0, -54.0, 33.0, -17.0], [-43.0, 53.0, 17.0, 16.0, -10.0], [-30.0, -23.0, -13.0, -104.0, -14.0], [18.0, 65.0, 35.0, -26.0, 135.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[-4.0, 7.0, -5.0, -7.0, 0.0], [4.0, 1.0, -2.0, 2.0, 0.0], [-3.0, -2.0, -5.0, -6.0, 2.0], [-4.0, -2.0, 6.0, 7.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-7.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[28.0, -24.0], [-49.0, 42.0], [35.0, -30.0], [49.0, -42.0], [0.0, 0.0]], [[-28.0, 24.0], [-7.0, 6.0], [14.0, -12.0], [-14.0, 12.0], [0.0, 0.0]], [[21.0, -18.0], [14.0, -12.0], [35.0, -30.0], [42.0, -36.0], [-14.0, 12.0]], [[28.0, -24.0], [14.0, -12.0], [-42.0, 36.0], [-49.0, 42.0], [21.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[0.0, 4.0, 6.0], [0.0, -3.0, 6.0], [-1.0, 5.0, 7.0], [0.0, -2.0, -6.0], [0.0, 6.0, -5.0]], [[7.0, 2.0, -2.0], [7.0, -6.0, 2.0], [-1.0, 6.0, 6.0], [0.0, 0.0, 1.0], [-1.0, -6.0, 6.0]], [[-4.0, 0.0, 0.0], [1.0, 0.0, 0.0], [2.0, -2.0, 4.0], [3.0, 3.0, 0.0], [5.0, 6.0, -5.0]], [[-6.0, -3.0, 4.0], [5.0, 7.0, 5.0], [5.0, 4.0, -7.0], [-7.0, -1.0, -3.0], [-1.0, -4.0, 4.0]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-3.0, -1.0], [6.0, 6.0], [1.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[30.0, -12.0], [-12.0, -54.0], [40.0, -11.0], [-18.0, 24.0], [31.0, 66.0]], [[-11.0, 17.0], [-55.0, -55.0], [45.0, 1.0], [1.0, -6.0], [-27.0, -71.0]], [[12.0, 4.0], [-3.0, -1.0], [-14.0, -38.0], [9.0, 15.0], [16.0, 61.0]], [[4.0, -36.0], [32.0, 7.0], [2.0, 61.0], [12.0, 19.0], [-17.0, -47.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[[4.0, -6.0, -5.0], [0.0, -7.0, -7.0]], [[4.0, 4.0, -4.0], [-6.0, 6.0, 4.0]], [[-4.0, 7.0, 3.0], [-7.0, 7.0, 1.0]], [[4.0, 6.0, -5.0], [7.0, -2.0, -6.0]], [[2.0, 0.0, -2.0], [4.0, -4.0, -2.0]]], [[[0.0, -3.0, 0.0], [3.0, -6.0, -3.0]], [[6.0, -6.0, 7.0], [-5.0, 2.0, 7.0]], [[6.0, -5.0, -4.0], [-3.0, -2.0, 0.0]], [[-2.0, -3.0, 5.0], [-2.0, -5.0, -3.0]], [[6.0, 7.0, 1.0], [3.0, 6.0, 0.0]]], [[[0.0, 4.0, 0.0], [2.0, 0.0, -4.0]], [[2.0, 3.0, 0.0], [1.0, 0.0, -4.0]], [[3.0, -7.0, -5.0], [0.0, 2.0, 7.0]], [[-5.0, -1.0, -5.0], [-7.0, 7.0, -1.0]], [[0.0, -3.0, 0.0], [5.0, -5.0, 5.0]]], [[[-2.0, 7.0, 7.0], [2.0, 3.0, -2.0]], [[4.0, -4.0, 3.0], [3.0, -3.0, -2.0]], [[7.0, -2.0, 7.0], [-2.0, 4.0, -5.0]], [[-4.0, 0.0, -1.0], [-1.0, -5.0, -3.0]], [[4.0, 3.0, -3.0], [-2.0, -2.0, 3.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-3.0, 2.0], [0.0, 3.0], [-3.0, 5.0]], [[-5.0, -6.0], [-1.0, -4.0], [6.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-32.0, 28.0], [48.0, -8.0], [37.0, 37.0], [-66.0, -3.0], [-28.0, -4.0]], [[-27.0, 12.0], [26.0, 16.0], [11.0, 3.0], [-12.0, 59.0], [-42.0, -4.0]], [[-34.0, 20.0], [-35.0, 27.0], [46.0, -83.0], [52.0, -19.0], [10.0, -44.0]], [[-40.0, 38.0], [-45.0, 15.0], [-66.0, 64.0], [7.0, 28.0], [27.0, 7.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([[-6.0, 0.0, -5.0, 3.0, 5.0], [-4.0, -7.0, -6.0, -5.0, 0.0], [5.0, 3.0, 6.0, -3.0, -6.0], [-2.0, 0.0, 4.0, 7.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-2.0, -5.0, 2.0, 0.0, 3.0], [0.0, 2.0, -7.0, 0.0, 4.0], [1.0, 2.0, -2.0, 6.0, -6.0], [-1.0, 0.0, -3.0, -7.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[12.0, 30.0, -12.0, 0.0, -18.0], [0.0, -12.0, 42.0, 0.0, -24.0], [-6.0, -12.0, 12.0, -36.0, 36.0], [6.0, 0.0, 18.0, 42.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[10.0, 25.0, -10.0, 0.0, -15.0], [0.0, -10.0, 35.0, 0.0, -20.0], [-5.0, -10.0, 10.0, -30.0, 30.0], [5.0, 0.0, 15.0, 35.0, 10.0]], [[-6.0, -15.0, 6.0, 0.0, 9.0], [0.0, 6.0, -21.0, 0.0, 12.0], [3.0, 6.0, -6.0, 18.0, -18.0], [-3.0, 0.0, -9.0, -21.0, -6.0]], [[-10.0, -25.0, 10.0, 0.0, 15.0], [0.0, 10.0, -35.0, 0.0, 20.0], [5.0, 10.0, -10.0, 30.0, -30.0], [-5.0, 0.0, -15.0, -35.0, -10.0]]], [[[8.0, 20.0, -8.0, 0.0, -12.0], [0.0, -8.0, 28.0, 0.0, -16.0], [-4.0, -8.0, 8.0, -24.0, 24.0], [4.0, 0.0, 12.0, 28.0, 8.0]], [[14.0, 35.0, -14.0, 0.0, -21.0], [0.0, -14.0, 49.0, 0.0, -28.0], [-7.0, -14.0, 14.0, -42.0, 42.0], [7.0, 0.0, 21.0, 49.0, 14.0]], [[12.0, 30.0, -12.0, 0.0, -18.0], [0.0, -12.0, 42.0, 0.0, -24.0], [-6.0, -12.0, 12.0, -36.0, 36.0], [6.0, 0.0, 18.0, 42.0, 12.0]], [[10.0, 25.0, -10.0, 0.0, -15.0], [0.0, -10.0, 35.0, 0.0, -20.0], [-5.0, -10.0, 10.0, -30.0, 30.0], [5.0, 0.0, 15.0, 35.0, 10.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-10.0, -25.0, 10.0, 0.0, 15.0], [0.0, 10.0, -35.0, 0.0, 20.0], [5.0, 10.0, -10.0, 30.0, -30.0], [-5.0, 0.0, -15.0, -35.0, -10.0]], [[-6.0, -15.0, 6.0, 0.0, 9.0], [0.0, 6.0, -21.0, 0.0, 12.0], [3.0, 6.0, -6.0, 18.0, -18.0], [-3.0, 0.0, -9.0, -21.0, -6.0]], [[-12.0, -30.0, 12.0, 0.0, 18.0], [0.0, 12.0, -42.0, 0.0, 24.0], [6.0, 12.0, -12.0, 36.0, -36.0], [-6.0, 0.0, -18.0, -42.0, -12.0]], [[6.0, 15.0, -6.0, 0.0, -9.0], [0.0, -6.0, 21.0, 0.0, -12.0], [-3.0, -6.0, 6.0, -18.0, 18.0], [3.0, 0.0, 9.0, 21.0, 6.0]], [[12.0, 30.0, -12.0, 0.0, -18.0], [0.0, -12.0, 42.0, 0.0, -24.0], [-6.0, -12.0, 12.0, -36.0, 36.0], [6.0, 0.0, 18.0, 42.0, 12.0]]], [[[4.0, 10.0, -4.0, 0.0, -6.0], [0.0, -4.0, 14.0, 0.0, -8.0], [-2.0, -4.0, 4.0, -12.0, 12.0], [2.0, 0.0, 6.0, 14.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-8.0, -20.0, 8.0, 0.0, 12.0], [0.0, 8.0, -28.0, 0.0, 16.0], [4.0, 8.0, -8.0, 24.0, -24.0], [-4.0, 0.0, -12.0, -28.0, -8.0]], [[-14.0, -35.0, 14.0, 0.0, 21.0], [0.0, 14.0, -49.0, 0.0, 28.0], [7.0, 14.0, -14.0, 42.0, -42.0], [-7.0, 0.0, -21.0, -49.0, -14.0]], [[6.0, 15.0, -6.0, 0.0, -9.0], [0.0, -6.0, 21.0, 0.0, -12.0], [-3.0, -6.0, 6.0, -18.0, 18.0], [3.0, 0.0, 9.0, 21.0, 6.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[[0.0, -6.0, -3.0], [0.0, 6.0, 5.0], [1.0, 5.0, -4.0], [-7.0, 4.0, 2.0], [-4.0, 5.0, -3.0]], [[-1.0, -7.0, -6.0], [-7.0, 2.0, 0.0], [6.0, 6.0, 6.0], [-5.0, 6.0, -6.0], [3.0, 1.0, -7.0]], [[-4.0, -7.0, 5.0], [2.0, 2.0, 5.0], [-6.0, 1.0, 3.0], [-4.0, -7.0, 4.0], [0.0, 7.0, 0.0]], [[-4.0, 0.0, -1.0], [4.0, -6.0, 0.0], [-6.0, -4.0, -1.0], [-6.0, 5.0, 0.0], [-2.0, 1.0, -4.0]]]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[1.0, 2.0, -2.0, 2.0, 1.0], [0.0, -3.0, -4.0, -4.0, -5.0], [-3.0, -5.0, 4.0, 0.0, -7.0], [-7.0, 3.0, -6.0, 2.0, -5.0]], [[-7.0, 3.0, 1.0, 0.0, 6.0], [5.0, 4.0, 0.0, 1.0, 7.0], [5.0, -1.0, -7.0, -1.0, -1.0], [-6.0, -5.0, 1.0, -7.0, -6.0]], [[3.0, 0.0, -3.0, -5.0, -7.0], [6.0, -1.0, -3.0, -7.0, 1.0], [4.0, 0.0, 0.0, -5.0, 0.0], [7.0, -5.0, 0.0, -1.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[33.0, -18.0, 3.0, 15.0, -15.0], [-48.0, -21.0, 9.0, 15.0, -45.0], [-42.0, 6.0, 42.0, 21.0, 6.0], [15.0, 45.0, -6.0, 45.0, 30.0]], [[-27.0, 18.0, -9.0, -25.0, 1.0], [60.0, 19.0, -15.0, -29.0, 47.0], [50.0, -6.0, -42.0, -31.0, -6.0], [-1.0, -55.0, 6.0, -47.0, -26.0]], [[-46.0, 17.0, 15.0, 22.0, 59.0], [1.0, 21.0, 8.0, 29.0, 26.0], [6.0, -10.0, -31.0, 15.0, -12.0], [-65.0, -2.0, -1.0, -29.0, -43.0]], [[-29.0, -2.0, 12.0, -24.0, 3.0], [32.0, 35.0, 22.0, 18.0, 65.0], [49.0, 31.0, -56.0, -14.0, 45.0], [39.0, -51.0, 46.0, -44.0, 15.0]], [[-48.0, 7.0, 22.0, 7.0, 47.0], [7.0, 35.0, 25.0, 42.0, 52.0], [25.0, 15.0, -51.0, 10.0, 23.0], [-23.0, -22.0, 29.0, -40.0, -16.0]]], [[[30.0, -23.0, 13.0, 28.0, -1.0], [-71.0, -19.0, 22.0, 39.0, -50.0], [-56.0, 12.0, 45.0, 37.0, 14.0], [7.0, 62.0, -1.0, 53.0, 35.0]], [[-21.0, -8.0, 16.0, -14.0, 5.0], [10.0, 29.0, 28.0, 30.0, 49.0], [31.0, 33.0, -42.0, -2.0, 47.0], [37.0, -31.0, 44.0, -28.0, 23.0]], [[-18.0, 30.0, -24.0, -18.0, 0.0], [66.0, 0.0, -42.0, -60.0, 18.0], [36.0, -36.0, -18.0, -36.0, -48.0], [-36.0, -42.0, -30.0, -36.0, -54.0]], [[-65.0, 8.0, 34.0, 20.0, 73.0], [-6.0, 45.0, 38.0, 68.0, 61.0], [21.0, 19.0, -62.0, 24.0, 29.0], [-43.0, -15.0, 36.0, -46.0, -23.0]], [[-25.0, 9.0, 16.0, 41.0, 58.0], [-37.0, 2.0, 9.0, 38.0, -15.0], [-32.0, -16.0, 5.0, 34.0, -22.0], [-76.0, 39.0, -17.0, 6.0, -35.0]]], [[[60.0, -29.0, -14.0, -33.0, -81.0], [-5.0, -21.0, 1.0, -26.0, -24.0], [-3.0, 27.0, 33.0, -18.0, 35.0], [105.0, -2.0, 17.0, 36.0, 72.0]], [[3.0, 10.0, -17.0, -21.0, -21.0], [40.0, -3.0, -23.0, -41.0, 9.0], [24.0, -12.0, -6.0, -27.0, -16.0], [9.0, -29.0, -10.0, -15.0, -12.0]], [[-4.0, -9.0, 4.0, -27.0, -21.0], [23.0, 19.0, 15.0, 4.0, 40.0], [35.0, 29.0, -31.0, -16.0, 41.0], [57.0, -38.0, 37.0, -22.0, 30.0]], [[57.0, -29.0, -11.0, -28.0, -74.0], [-11.0, -20.0, 4.0, -19.0, -25.0], [-7.0, 27.0, 33.0, -13.0, 35.0], [98.0, 3.0, 17.0, 37.0, 70.0]], [[-49.0, 21.0, 7.0, 0.0, 42.0], [35.0, 28.0, 0.0, 7.0, 49.0], [35.0, -7.0, -49.0, -7.0, -7.0], [-42.0, -35.0, 7.0, -49.0, -42.0]]], [[[-7.0, -8.0, 11.0, -3.0, 3.0], [-6.0, 13.0, 19.0, 23.0, 19.0], [8.0, 20.0, -16.0, 5.0, 28.0], [21.0, -7.0, 24.0, -7.0, 18.0]], [[46.0, -10.0, -14.0, 8.0, -32.0], [-30.0, -36.0, -16.0, -22.0, -62.0], [-42.0, -14.0, 58.0, 6.0, -22.0], [8.0, 42.0, -30.0, 50.0, 16.0]], [[19.0, -24.0, 11.0, -7.0, -23.0], [-26.0, 3.0, 27.0, 27.0, 1.0], [-6.0, 34.0, 4.0, 9.0, 46.0], [59.0, 7.0, 32.0, 17.0, 52.0]], [[-41.0, 3.0, 17.0, -12.0, 24.0], [25.0, 38.0, 24.0, 29.0, 65.0], [43.0, 25.0, -59.0, -5.0, 37.0], [12.0, -43.0, 41.0, -47.0, 0.0]], [[-21.0, -1.0, 17.0, 16.0, 32.0], [-19.0, 14.0, 20.0, 37.0, 13.0], [-5.0, 9.0, -15.0, 19.0, 13.0], [-20.0, 9.0, 13.0, -7.0, -4.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[[1.0, -5.0, -6.0], [0.0, -5.0, -2.0]], [[0.0, 5.0, 5.0], [-1.0, 6.0, 5.0]], [[2.0, 6.0, 4.0], [0.0, -7.0, 6.0]], [[-4.0, -1.0, -7.0], [-4.0, 1.0, -3.0]], [[-1.0, 6.0, -6.0], [5.0, 0.0, 1.0]]], [[[5.0, -1.0, -5.0], [5.0, 0.0, 4.0]], [[1.0, -2.0, -2.0], [0.0, 7.0, 4.0]], [[4.0, -1.0, 3.0], [-1.0, 7.0, 7.0]], [[2.0, -6.0, 5.0], [-6.0, 2.0, 6.0]], [[2.0, 0.0, -7.0], [3.0, 4.0, -4.0]]], [[[0.0, -4.0, 3.0], [7.0, 0.0, 0.0]], [[-3.0, 4.0, -7.0], [-1.0, -2.0, 7.0]], [[2.0, -1.0, -2.0], [3.0, 3.0, -2.0]], [[4.0, 7.0, -4.0], [3.0, -5.0, 1.0]], [[-5.0, 0.0, 6.0], [3.0, 0.0, 7.0]]], [[[-6.0, -1.0, -5.0], [-2.0, 0.0, 5.0]], [[-4.0, -3.0, -7.0], [-5.0, 4.0, 5.0]], [[-2.0, 6.0, -4.0], [-6.0, -6.0, 1.0]], [[-3.0, -1.0, 0.0], [-3.0, 7.0, -7.0]], [[-5.0, -7.0, 6.0], [-6.0, 5.0, -4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-4.0, -7.0, 3.0, 3.0, 2.0], [4.0, 6.0, -2.0, -2.0, 6.0], [3.0, 1.0, 6.0, 0.0, 2.0], [-7.0, 0.0, 0.0, -3.0, 7.0]], [[-3.0, -3.0, 4.0, -6.0, 1.0], [-4.0, 5.0, 5.0, 5.0, 0.0], [-4.0, 3.0, 1.0, -3.0, 0.0], [-7.0, -7.0, -2.0, 0.0, 0.0]], [[-6.0, 6.0, -3.0, -6.0, 6.0], [6.0, 5.0, 5.0, 7.0, 6.0], [2.0, -7.0, 7.0, -6.0, 4.0], [-1.0, -4.0, -7.0, 2.0, 1.0]]], [[[7.0, 0.0, -7.0, 3.0, -6.0], [0.0, 2.0, -7.0, 5.0, -1.0], [4.0, -1.0, 7.0, -1.0, 0.0], [0.0, 0.0, 0.0, -3.0, 2.0]], [[4.0, -5.0, -7.0, -2.0, 1.0], [-2.0, 0.0, 2.0, 2.0, -7.0], [-5.0, -2.0, -2.0, 4.0, 6.0], [-5.0, 1.0, 0.0, 6.0, -1.0]], [[1.0, 2.0, -3.0, 3.0, 3.0], [-1.0, 7.0, 1.0, 0.0, 0.0], [7.0, 0.0, 2.0, 1.0, -1.0], [-6.0, -3.0, -1.0, 1.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[25.0, -7.0, 42.0, 73.0, -50.0], [0.0, -63.0, -69.0, -79.0, 5.0], [22.0, 38.0, -35.0, 29.0, -50.0], [71.0, 60.0, 54.0, -47.0, -4.0]], [[-23.0, -5.0, -45.0, -60.0, 62.0], [-7.0, 83.0, 74.0, 67.0, -11.0], [-9.0, -31.0, 31.0, -15.0, 51.0], [-100.0, -64.0, -50.0, 54.0, 22.0]], [[-72.0, 39.0, 49.0, -22.0, 45.0], [16.0, 104.0, 38.0, 40.0, 85.0], [67.0, 6.0, 72.0, -64.0, -28.0], [-61.0, -83.0, -46.0, -34.0, 55.0]], [[34.0, -22.0, 35.0, 13.0, -35.0], [-53.0, -93.0, -5.0, -64.0, -69.0], [-64.0, 44.0, -110.0, 50.0, -27.0], [55.0, 45.0, 54.0, 13.0, -59.0]], [[58.0, -45.0, 1.0, 15.0, -59.0], [-65.0, 11.0, -32.0, 15.0, -47.0], [-12.0, 54.0, -5.0, 14.0, -27.0], [-35.0, -21.0, 29.0, -23.0, 2.0]]], [[[52.0, -54.0, -21.0, 78.0, -39.0], [-10.0, 38.0, -71.0, -25.0, -5.0], [57.0, 32.0, 37.0, 32.0, -14.0], [-47.0, 15.0, 33.0, -36.0, 60.0]], [[46.0, -40.0, -60.0, 25.0, 7.0], [-18.0, 14.0, -4.0, -12.0, -55.0], [0.0, -5.0, -16.0, 50.0, 32.0], [-50.0, 17.0, 14.0, 39.0, 18.0]], [[-3.0, -28.0, -64.0, 4.0, 59.0], [17.0, 81.0, 30.0, 17.0, -6.0], [32.0, -33.0, 37.0, 21.0, 55.0], [-101.0, -19.0, -26.0, 46.0, 57.0]], [[-48.0, 36.0, -23.0, 8.0, 84.0], [52.0, 37.0, 43.0, -25.0, 34.0], [48.0, -49.0, 7.0, 8.0, 30.0], [-23.0, 6.0, -29.0, 40.0, 35.0]], [[67.0, -84.0, -10.0, 37.0, -64.0], [-38.0, -45.0, -56.0, -30.0, -61.0], [-44.0, 40.0, -32.0, 51.0, 4.0], [-3.0, 44.0, 53.0, -9.0, -11.0]]], [[[43.0, 30.0, -74.0, 27.0, -28.0], [34.0, 9.0, -54.0, 36.0, 11.0], [50.0, -40.0, 66.0, -13.0, 12.0], [25.0, 16.0, -13.0, -15.0, 17.0]], [[34.0, -9.0, 28.0, 31.0, -19.0], [-73.0, 14.0, 1.0, -32.0, -45.0], [16.0, 63.0, -52.0, 30.0, -53.0], [-32.0, -23.0, 34.0, -7.0, 7.0]], [[38.0, -42.0, -28.0, 21.0, -30.0], [-4.0, -11.0, -36.0, -2.0, -24.0], [-11.0, 4.0, 8.0, 22.0, 16.0], [-8.0, 24.0, 18.0, -3.0, 5.0]], [[-11.0, -46.0, 63.0, 16.0, -29.0], [-27.0, 52.0, -23.0, 4.0, 32.0], [20.0, 60.0, 36.0, -19.0, -39.0], [-54.0, -41.0, 13.0, -58.0, 40.0]], [[12.0, 85.0, -75.0, -21.0, 29.0], [9.0, 55.0, 26.0, 67.0, 3.0], [58.0, -50.0, 47.0, -32.0, 7.0], [-13.0, -45.0, -49.0, 25.0, 12.0]]], [[[48.0, 25.0, -8.0, 27.0, -16.0], [-55.0, -35.0, 1.0, -38.0, -64.0], [3.0, 28.0, -76.0, 40.0, -37.0], [24.0, 12.0, 32.0, 19.0, -26.0]], [[53.0, -15.0, -11.0, 40.0, -4.0], [-59.0, -49.0, 6.0, -73.0, -89.0], [-19.0, 33.0, -109.0, 77.0, -17.0], [6.0, 38.0, 50.0, 42.0, -24.0]], [[-51.0, 4.0, 111.0, -21.0, 11.0], [-45.0, -7.0, 45.0, -36.0, 12.0], [-25.0, 62.0, -62.0, -11.0, -57.0], [0.0, -35.0, 15.0, -19.0, -19.0]], [[15.0, -25.0, -20.0, -47.0, -3.0], [-15.0, -78.0, 29.0, 0.0, -64.0], [-101.0, -17.0, -68.0, 27.0, 43.0], [35.0, 35.0, 9.0, 53.0, -69.0]], [[-21.0, 59.0, -42.0, -49.0, 48.0], [38.0, -75.0, 53.0, -3.0, -23.0], [-52.0, -72.0, -55.0, 7.0, 48.0], [77.0, 42.0, -24.0, 71.0, -66.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[7.0, -4.0, -1.0, 0.0, -1.0], [-5.0, 2.0, -5.0, -2.0, 2.0], [-2.0, -7.0, 5.0, -3.0, -1.0], [-7.0, 0.0, 4.0, 3.0, 1.0]]),self.functionspace)
      arg1=Data(-5.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-35.0, 20.0, 5.0, -0.0, 5.0], [25.0, -10.0, 25.0, 10.0, -10.0], [10.0, 35.0, -25.0, 15.0, 5.0], [35.0, -0.0, -20.0, -15.0, -5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[0.0, -4.0, 3.0], [2.0, 5.0, 1.0], [-6.0, -7.0, 7.0], [6.0, -7.0, 5.0], [-2.0, 1.0, 4.0]], [[7.0, -6.0, 0.0], [3.0, 3.0, -5.0], [4.0, -2.0, 0.0], [4.0, -4.0, -4.0], [5.0, 7.0, 4.0]], [[-5.0, -4.0, 7.0], [-1.0, -1.0, -6.0], [7.0, -4.0, 6.0], [3.0, -6.0, 1.0], [4.0, 0.0, 6.0]], [[3.0, 1.0, 6.0], [5.0, -7.0, 1.0], [-3.0, -3.0, -2.0], [-6.0, 3.0, 6.0], [0.0, 4.0, 4.0]]]),self.functionspace)
      arg1=Data(numpy.array([-6.0, 5.0, 2.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-14.0, 15.0, 15.0, -61.0, 25.0], [-72.0, -13.0, -34.0, -52.0, 13.0], [24.0, -11.0, -50.0, -46.0, -12.0], [-1.0, -63.0, -1.0, 63.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[7.0, 4.0, 0.0], [2.0, 5.0, 7.0]], [[-1.0, 0.0, 3.0], [6.0, 0.0, -2.0]], [[2.0, 5.0, -7.0], [-3.0, 0.0, 7.0]], [[-5.0, 7.0, 2.0], [-4.0, 7.0, 7.0]], [[3.0, -1.0, 0.0], [4.0, -5.0, 5.0]]], [[[-4.0, -1.0, 2.0], [1.0, 2.0, 2.0]], [[0.0, 0.0, -5.0], [-4.0, 7.0, -6.0]], [[-1.0, 6.0, 1.0], [7.0, 0.0, 7.0]], [[0.0, 6.0, -7.0], [-7.0, 5.0, -1.0]], [[7.0, 4.0, -2.0], [5.0, 6.0, -1.0]]], [[[-3.0, -2.0, 3.0], [-4.0, 3.0, 0.0]], [[7.0, -7.0, 4.0], [7.0, 0.0, -7.0]], [[0.0, 3.0, -7.0], [-7.0, -1.0, 3.0]], [[0.0, 0.0, -7.0], [0.0, 1.0, 5.0]], [[-4.0, 0.0, 5.0], [4.0, -5.0, 2.0]]], [[[0.0, 2.0, -5.0], [7.0, 0.0, 2.0]], [[1.0, 0.0, 6.0], [-1.0, 0.0, -5.0]], [[2.0, 0.0, 5.0], [-1.0, -5.0, 1.0]], [[-2.0, -2.0, -6.0], [-5.0, 4.0, -3.0]], [[2.0, 0.0, 0.0], [0.0, -7.0, -4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[-5.0, 0.0, -2.0], [0.0, 0.0, -6.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-77.0, 11.0, -38.0, -21.0, -45.0], [4.0, 46.0, -39.0, 20.0, -25.0], [9.0, -1.0, -4.0, -16.0, -2.0], [-2.0, 13.0, -26.0, 40.0, 14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[-4.0, 7.0, 6.0, 4.0, 6.0], [0.0, -3.0, -6.0, 3.0, -7.0], [5.0, 4.0, -5.0, -4.0, -5.0], [6.0, -3.0, -4.0, -6.0, 5.0]]),self.functionspace)
      arg1=Data(numpy.array([2.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-8.0, 24.0], [14.0, -42.0], [12.0, -36.0], [8.0, -24.0], [12.0, -36.0]], [[0.0, 0.0], [-6.0, 18.0], [-12.0, 36.0], [6.0, -18.0], [-14.0, 42.0]], [[10.0, -30.0], [8.0, -24.0], [-10.0, 30.0], [-8.0, 24.0], [-10.0, 30.0]], [[12.0, -36.0], [-6.0, 18.0], [-8.0, 24.0], [-12.0, 36.0], [10.0, -30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[3.0, 0.0, -5.0], [-7.0, -7.0, 3.0], [-4.0, -4.0, -4.0], [-4.0, -2.0, -7.0], [5.0, -7.0, 0.0]], [[-4.0, 3.0, -6.0], [6.0, 6.0, -2.0], [-7.0, 3.0, 3.0], [0.0, -5.0, -6.0], [-2.0, -3.0, -3.0]], [[5.0, 1.0, 4.0], [-6.0, 1.0, 7.0], [0.0, -2.0, 4.0], [6.0, -1.0, -6.0], [-2.0, -2.0, 6.0]], [[-5.0, 2.0, -2.0], [-7.0, 3.0, -6.0], [7.0, 1.0, -3.0], [-4.0, -1.0, 7.0], [-5.0, -6.0, -4.0]]]),self.functionspace)
      arg1=Data(numpy.array([[1.0, -7.0], [6.0, -5.0], [3.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-12.0, -46.0], [-40.0, 99.0], [-40.0, 28.0], [-37.0, 3.0], [-37.0, 0.0]], [[-4.0, -17.0], [36.0, -82.0], [20.0, 49.0], [-48.0, -5.0], [-29.0, 14.0]], [[23.0, -20.0], [21.0, 72.0], [0.0, 30.0], [-18.0, -67.0], [4.0, 54.0]], [[1.0, 15.0], [-7.0, 4.0], [4.0, -69.0], [11.0, 68.0], [-53.0, 45.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[-2.0, -1.0, 6.0], [3.0, 0.0, -1.0]], [[-4.0, -3.0, -4.0], [0.0, 2.0, -4.0]], [[-6.0, 2.0, 3.0], [7.0, -1.0, 0.0]], [[-4.0, 3.0, 4.0], [0.0, 0.0, 4.0]], [[5.0, 1.0, -5.0], [5.0, 0.0, 6.0]]], [[[2.0, 1.0, 1.0], [-5.0, 7.0, 6.0]], [[-7.0, 1.0, 4.0], [0.0, 6.0, 1.0]], [[-3.0, -2.0, 3.0], [2.0, 7.0, 0.0]], [[-4.0, -7.0, -5.0], [-7.0, -6.0, 0.0]], [[-1.0, 2.0, 6.0], [4.0, 4.0, 6.0]]], [[[6.0, 0.0, 1.0], [0.0, 0.0, 3.0]], [[-4.0, -5.0, 0.0], [7.0, 0.0, 4.0]], [[-1.0, 7.0, 0.0], [0.0, -4.0, 4.0]], [[-2.0, -3.0, -1.0], [-4.0, 0.0, 2.0]], [[-2.0, 7.0, 6.0], [-4.0, 3.0, 2.0]]], [[[7.0, -3.0, 4.0], [7.0, -7.0, -2.0]], [[-1.0, -3.0, -2.0], [2.0, 3.0, 1.0]], [[-7.0, 3.0, -5.0], [-3.0, -5.0, -3.0]], [[7.0, -3.0, 6.0], [-5.0, 1.0, -6.0]], [[6.0, -4.0, 7.0], [4.0, 0.0, -2.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[-7.0, -5.0], [6.0, -3.0], [-2.0, 6.0]], [[0.0, -1.0], [0.0, -6.0], [-5.0, 7.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[1.0, 39.0], [38.0, -35.0], [48.0, 41.0], [18.0, 63.0], [-49.0, -21.0]], [[-40.0, -2.0], [42.0, 27.0], [3.0, -5.0], [-4.0, 54.0], [-23.0, 49.0]], [[-59.0, -3.0], [-22.0, 56.0], [29.0, 36.0], [-12.0, 31.0], [34.0, 25.0]], [[-65.0, 19.0], [-12.0, -11.0], [92.0, 8.0], [-49.0, -33.0], [-70.0, 6.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_constData_rank2_offset0(self):
      arg0=Data(numpy.array([[-4.0, 5.0, -4.0, 3.0, 2.0], [7.0, 5.0, 7.0, -2.0, 3.0], [1.0, 5.0, -2.0, 0.0, -4.0], [3.0, -7.0, 3.0, -6.0, -5.0]]),self.functionspace)
      arg1=Data(numpy.array([[-4.0, -1.0, 3.0, -5.0, 5.0], [6.0, 1.0, 3.0, 0.0, -2.0], [4.0, 3.0, 0.0, 0.0, -4.0], [-5.0, -5.0, 5.0, 5.0, -7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[16.0, 4.0, -12.0, 20.0, -20.0], [-24.0, -4.0, -12.0, 0.0, 8.0], [-16.0, -12.0, 0.0, 0.0, 16.0], [20.0, 20.0, -20.0, -20.0, 28.0]], [[-20.0, -5.0, 15.0, -25.0, 25.0], [30.0, 5.0, 15.0, 0.0, -10.0], [20.0, 15.0, 0.0, 0.0, -20.0], [-25.0, -25.0, 25.0, 25.0, -35.0]], [[16.0, 4.0, -12.0, 20.0, -20.0], [-24.0, -4.0, -12.0, 0.0, 8.0], [-16.0, -12.0, 0.0, 0.0, 16.0], [20.0, 20.0, -20.0, -20.0, 28.0]], [[-12.0, -3.0, 9.0, -15.0, 15.0], [18.0, 3.0, 9.0, 0.0, -6.0], [12.0, 9.0, 0.0, 0.0, -12.0], [-15.0, -15.0, 15.0, 15.0, -21.0]], [[-8.0, -2.0, 6.0, -10.0, 10.0], [12.0, 2.0, 6.0, 0.0, -4.0], [8.0, 6.0, 0.0, 0.0, -8.0], [-10.0, -10.0, 10.0, 10.0, -14.0]]], [[[-28.0, -7.0, 21.0, -35.0, 35.0], [42.0, 7.0, 21.0, 0.0, -14.0], [28.0, 21.0, 0.0, 0.0, -28.0], [-35.0, -35.0, 35.0, 35.0, -49.0]], [[-20.0, -5.0, 15.0, -25.0, 25.0], [30.0, 5.0, 15.0, 0.0, -10.0], [20.0, 15.0, 0.0, 0.0, -20.0], [-25.0, -25.0, 25.0, 25.0, -35.0]], [[-28.0, -7.0, 21.0, -35.0, 35.0], [42.0, 7.0, 21.0, 0.0, -14.0], [28.0, 21.0, 0.0, 0.0, -28.0], [-35.0, -35.0, 35.0, 35.0, -49.0]], [[8.0, 2.0, -6.0, 10.0, -10.0], [-12.0, -2.0, -6.0, 0.0, 4.0], [-8.0, -6.0, 0.0, 0.0, 8.0], [10.0, 10.0, -10.0, -10.0, 14.0]], [[-12.0, -3.0, 9.0, -15.0, 15.0], [18.0, 3.0, 9.0, 0.0, -6.0], [12.0, 9.0, 0.0, 0.0, -12.0], [-15.0, -15.0, 15.0, 15.0, -21.0]]], [[[-4.0, -1.0, 3.0, -5.0, 5.0], [6.0, 1.0, 3.0, 0.0, -2.0], [4.0, 3.0, 0.0, 0.0, -4.0], [-5.0, -5.0, 5.0, 5.0, -7.0]], [[-20.0, -5.0, 15.0, -25.0, 25.0], [30.0, 5.0, 15.0, 0.0, -10.0], [20.0, 15.0, 0.0, 0.0, -20.0], [-25.0, -25.0, 25.0, 25.0, -35.0]], [[8.0, 2.0, -6.0, 10.0, -10.0], [-12.0, -2.0, -6.0, 0.0, 4.0], [-8.0, -6.0, 0.0, 0.0, 8.0], [10.0, 10.0, -10.0, -10.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[16.0, 4.0, -12.0, 20.0, -20.0], [-24.0, -4.0, -12.0, 0.0, 8.0], [-16.0, -12.0, 0.0, 0.0, 16.0], [20.0, 20.0, -20.0, -20.0, 28.0]]], [[[-12.0, -3.0, 9.0, -15.0, 15.0], [18.0, 3.0, 9.0, 0.0, -6.0], [12.0, 9.0, 0.0, 0.0, -12.0], [-15.0, -15.0, 15.0, 15.0, -21.0]], [[28.0, 7.0, -21.0, 35.0, -35.0], [-42.0, -7.0, -21.0, 0.0, 14.0], [-28.0, -21.0, 0.0, 0.0, 28.0], [35.0, 35.0, -35.0, -35.0, 49.0]], [[-12.0, -3.0, 9.0, -15.0, 15.0], [18.0, 3.0, 9.0, 0.0, -6.0], [12.0, 9.0, 0.0, 0.0, -12.0], [-15.0, -15.0, 15.0, 15.0, -21.0]], [[24.0, 6.0, -18.0, 30.0, -30.0], [-36.0, -6.0, -18.0, 0.0, 12.0], [-24.0, -18.0, 0.0, 0.0, 24.0], [30.0, 30.0, -30.0, -30.0, 42.0]], [[20.0, 5.0, -15.0, 25.0, -25.0], [-30.0, -5.0, -15.0, 0.0, 10.0], [-20.0, -15.0, 0.0, 0.0, 20.0], [25.0, 25.0, -25.0, -25.0, 35.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[[-1.0, 4.0, 0.0], [-3.0, 6.0, -1.0], [2.0, 7.0, 6.0], [-3.0, 7.0, 0.0], [6.0, 7.0, 0.0]], [[-1.0, 0.0, 2.0], [-4.0, -1.0, -7.0], [-2.0, 1.0, -5.0], [-1.0, -4.0, -1.0], [-4.0, 1.0, -5.0]], [[1.0, -4.0, -7.0], [-2.0, -3.0, -7.0], [3.0, 7.0, 3.0], [1.0, 0.0, -4.0], [-2.0, 6.0, -1.0]], [[6.0, 4.0, 0.0], [-7.0, 0.0, 0.0], [5.0, -4.0, -6.0], [1.0, 7.0, -5.0], [6.0, -7.0, -3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[-6.0, 3.0, 0.0, -7.0, 0.0], [5.0, -2.0, -5.0, -5.0, -5.0], [2.0, -1.0, 5.0, -2.0, -2.0], [0.0, -7.0, -3.0, 5.0, 6.0]], [[2.0, -6.0, 0.0, -3.0, 6.0], [6.0, -1.0, 6.0, 6.0, 2.0], [-2.0, 7.0, 1.0, -7.0, 3.0], [-7.0, 7.0, -6.0, -4.0, 4.0]], [[-3.0, -2.0, -1.0, 0.0, -7.0], [-6.0, 7.0, -1.0, -3.0, 7.0], [-4.0, 1.0, -4.0, -7.0, 6.0], [5.0, 7.0, -3.0, -1.0, -3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[14.0, -27.0, 0.0, -5.0, 24.0], [19.0, -2.0, 29.0, 29.0, 13.0], [-10.0, 29.0, -1.0, -26.0, 14.0], [-28.0, 35.0, -21.0, -21.0, 10.0]], [[33.0, -43.0, 1.0, 3.0, 43.0], [27.0, -7.0, 52.0, 54.0, 20.0], [-14.0, 44.0, -5.0, -29.0, 18.0], [-47.0, 56.0, -24.0, -38.0, 9.0]], [[-16.0, -48.0, -6.0, -35.0, 0.0], [16.0, 31.0, 26.0, 14.0, 46.0], [-34.0, 53.0, -7.0, -95.0, 53.0], [-19.0, 77.0, -66.0, -24.0, 22.0]], [[32.0, -51.0, 0.0, 0.0, 42.0], [27.0, -1.0, 57.0, 57.0, 29.0], [-20.0, 52.0, -8.0, -43.0, 27.0], [-49.0, 70.0, -33.0, -43.0, 10.0]], [[-22.0, -24.0, 0.0, -63.0, 42.0], [72.0, -19.0, 12.0, 12.0, -16.0], [-2.0, 43.0, 37.0, -61.0, 9.0], [-49.0, 7.0, -60.0, 2.0, 64.0]]], [[[0.0, -7.0, -2.0, 7.0, -14.0], [-17.0, 16.0, 3.0, -1.0, 19.0], [-10.0, 3.0, -13.0, -12.0, 14.0], [10.0, 21.0, -3.0, -7.0, -12.0]], [[43.0, 8.0, 7.0, 31.0, 43.0], [16.0, -40.0, 21.0, 35.0, -31.0], [22.0, -10.0, 7.0, 64.0, -37.0], [-28.0, -28.0, 39.0, -9.0, -7.0]], [[29.0, -2.0, 5.0, 11.0, 41.0], [26.0, -32.0, 21.0, 31.0, -23.0], [14.0, 4.0, 11.0, 32.0, -23.0], [-32.0, -14.0, 15.0, -9.0, 7.0]], [[1.0, 23.0, 1.0, 19.0, -17.0], [-23.0, -1.0, -18.0, -16.0, -10.0], [10.0, -28.0, -5.0, 37.0, -16.0], [23.0, -28.0, 30.0, 12.0, -19.0]], [[41.0, -8.0, 5.0, 25.0, 41.0], [16.0, -28.0, 31.0, 41.0, -13.0], [10.0, 6.0, 1.0, 36.0, -19.0], [-32.0, 0.0, 21.0, -19.0, -5.0]]], [[[7.0, 41.0, 7.0, 5.0, 25.0], [23.0, -47.0, -22.0, -8.0, -62.0], [38.0, -36.0, 29.0, 75.0, -56.0], [-7.0, -84.0, 42.0, 28.0, 11.0]], [[27.0, 26.0, 7.0, 23.0, 31.0], [14.0, -42.0, -1.0, 13.0, -45.0], [30.0, -26.0, 15.0, 74.0, -47.0], [-14.0, -56.0, 45.0, 9.0, -3.0]], [[-13.0, -39.0, -3.0, -42.0, 21.0], [39.0, 8.0, 24.0, 18.0, 20.0], [-20.0, 49.0, 10.0, -76.0, 33.0], [-34.0, 49.0, -60.0, -16.0, 37.0]], [[6.0, 11.0, 4.0, -7.0, 28.0], [29.0, -30.0, -1.0, 7.0, -33.0], [18.0, -5.0, 21.0, 26.0, -26.0], [-20.0, -35.0, 9.0, 9.0, 18.0]], [[27.0, -40.0, 1.0, -4.0, 43.0], [32.0, -9.0, 47.0, 49.0, 15.0], [-12.0, 43.0, 0.0, -31.0, 16.0], [-47.0, 49.0, -27.0, -33.0, 15.0]]], [[[-28.0, -6.0, 0.0, -54.0, 24.0], [54.0, -16.0, -6.0, -6.0, -22.0], [4.0, 22.0, 34.0, -40.0, 0.0], [-28.0, -14.0, -42.0, 14.0, 52.0]], [[42.0, -21.0, 0.0, 49.0, 0.0], [-35.0, 14.0, 35.0, 35.0, 35.0], [-14.0, 7.0, -35.0, 14.0, 14.0], [0.0, 49.0, 21.0, -35.0, -42.0]], [[-20.0, 51.0, 6.0, -23.0, 18.0], [37.0, -48.0, -43.0, -31.0, -75.0], [42.0, -39.0, 45.0, 60.0, -58.0], [-2.0, -105.0, 27.0, 47.0, 32.0]], [[23.0, -29.0, 5.0, -28.0, 77.0], [77.0, -44.0, 42.0, 52.0, -26.0], [8.0, 43.0, 32.0, -16.0, -11.0], [-74.0, 7.0, -30.0, -18.0, 49.0]], [[-41.0, 66.0, 3.0, -21.0, -21.0], [6.0, -26.0, -69.0, -63.0, -65.0], [38.0, -58.0, 35.0, 58.0, -51.0], [34.0, -112.0, 33.0, 61.0, 17.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-7.0, 0.0, -7.0], [6.0, 6.0, -3.0]], [[-7.0, 7.0, -1.0], [6.0, 5.0, -2.0]], [[0.0, -2.0, -6.0], [-6.0, -1.0, -5.0]], [[-4.0, 2.0, -5.0], [1.0, 5.0, -2.0]], [[-7.0, -1.0, -1.0], [-7.0, 2.0, 1.0]]], [[[-5.0, 7.0, 2.0], [7.0, 5.0, 2.0]], [[5.0, 2.0, -5.0], [4.0, 6.0, -3.0]], [[3.0, 0.0, 1.0], [-6.0, -2.0, -1.0]], [[6.0, -6.0, -6.0], [4.0, 4.0, -1.0]], [[-2.0, -7.0, 6.0], [4.0, 7.0, -4.0]]], [[[7.0, 0.0, 7.0], [-6.0, -6.0, 4.0]], [[-7.0, -2.0, -2.0], [-1.0, -4.0, -5.0]], [[5.0, 3.0, -3.0], [7.0, 0.0, -4.0]], [[5.0, -3.0, 5.0], [0.0, -4.0, -7.0]], [[0.0, -4.0, 6.0], [2.0, -7.0, 7.0]]], [[[2.0, 7.0, -2.0], [6.0, -2.0, 5.0]], [[-3.0, -5.0, 2.0], [-6.0, 2.0, -5.0]], [[-1.0, 7.0, -2.0], [3.0, 7.0, 5.0]], [[-3.0, -5.0, -4.0], [-2.0, -3.0, 0.0]], [[-3.0, 0.0, 7.0], [-5.0, -1.0, 6.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-3.0, 4.0, -2.0, 4.0, -7.0], [-5.0, -3.0, -6.0, -4.0, -4.0], [7.0, -5.0, 4.0, -6.0, 2.0], [-6.0, 3.0, -4.0, -7.0, 3.0]], [[7.0, -5.0, 5.0, 6.0, 7.0], [-2.0, 2.0, -3.0, 1.0, -6.0], [-7.0, -5.0, 7.0, 5.0, 3.0], [0.0, 2.0, -4.0, -4.0, -4.0]], [[-2.0, 4.0, 5.0, -4.0, -6.0], [-5.0, 3.0, -6.0, -4.0, 3.0], [0.0, -3.0, -4.0, -2.0, 6.0], [-3.0, -5.0, -3.0, -5.0, -6.0]]], [[[1.0, -2.0, -1.0, -4.0, 4.0], [1.0, 7.0, -2.0, 3.0, 1.0], [0.0, 0.0, 6.0, 2.0, 0.0], [-2.0, 0.0, 2.0, 0.0, -4.0]], [[0.0, -4.0, 3.0, -5.0, 2.0], [5.0, -7.0, -1.0, -3.0, 0.0], [5.0, 2.0, 4.0, -2.0, 5.0], [-6.0, -3.0, -1.0, 0.0, 6.0]], [[-1.0, 3.0, 0.0, 7.0, 2.0], [-3.0, 0.0, -4.0, 5.0, 0.0], [2.0, 5.0, 3.0, -5.0, -1.0], [-5.0, 3.0, 3.0, 5.0, -4.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[44.0, -101.0, -9.0, -75.0, 121.0], [115.0, 0.0, 78.0, 41.0, 13.0], [-25.0, 53.0, 51.0, 71.0, -23.0], [30.0, -13.0, 46.0, 69.0, 45.0]], [[80.0, -105.0, 53.0, -45.0, 134.0], [63.0, 39.0, 18.0, 32.0, -11.0], [-77.0, 3.0, 75.0, 91.0, 28.0], [13.0, -23.0, 4.0, 16.0, -29.0]], [[-3.0, -13.0, -37.0, 6.0, -14.0], [38.0, -57.0, 75.0, -18.0, -12.0], [-1.0, 1.0, -45.0, 17.0, -42.0], [61.0, 14.0, 0.0, 13.0, 82.0]], [[39.0, -74.0, 7.0, -27.0, 82.0], [73.0, -27.0, 49.0, 16.0, -10.0], [-21.0, 25.0, 38.0, 46.0, -5.0], [17.0, -4.0, 14.0, 35.0, 44.0]], [[8.0, -18.0, 17.0, -5.0, 26.0], [42.0, -47.0, 59.0, 9.0, 24.0], [-30.0, 52.0, -62.0, 16.0, -14.0], [42.0, -21.0, 22.0, 63.0, 25.0]]], [[[65.0, -75.0, 63.0, -25.0, 114.0], [27.0, 49.0, -30.0, 35.0, -9.0], [-55.0, 4.0, 89.0, 55.0, 46.0], [-30.0, -20.0, 1.0, 7.0, -61.0]], [[16.0, -51.0, -11.0, -15.0, 31.0], [39.0, -40.0, -8.0, -19.0, -43.0], [45.0, -23.0, 93.0, 1.0, 19.0], [-44.0, 17.0, -20.0, -33.0, 69.0]], [[-16.0, 33.0, -1.0, 35.0, -57.0], [-33.0, -34.0, -6.0, -33.0, -15.0], [9.0, -27.0, -39.0, -23.0, 3.0], [8.0, 7.0, -28.0, -31.0, 19.0]], [[-43.0, 3.0, -64.0, -31.0, -26.0], [39.0, -48.0, 10.0, -11.0, -2.0], [102.0, 21.0, 43.0, -49.0, -21.0], [-45.0, 21.0, 19.0, 7.0, 90.0]], [[-47.0, 3.0, 16.0, -153.0, -49.0], [45.0, -11.0, -2.0, -52.0, 72.0], [62.0, 21.0, -41.0, -21.0, 50.0], [-36.0, -83.0, 7.0, -8.0, 28.0]]], [[[-45.0, 104.0, 9.0, 82.0, -119.0], [-118.0, 0.0, -82.0, -36.0, -13.0], [27.0, -48.0, -48.0, -76.0, 22.0], [-35.0, 16.0, -43.0, -64.0, -49.0]], [[15.0, -23.0, -17.0, -43.0, 25.0], [43.0, 32.0, 86.0, 18.0, 33.0], [-65.0, 18.0, -71.0, 67.0, -47.0], [99.0, -18.0, 29.0, 42.0, -1.0]], [[23.0, -33.0, -17.0, -6.0, 24.0], [3.0, 31.0, -19.0, -4.0, -40.0], [6.0, -51.0, 83.0, 25.0, 5.0], [-15.0, 24.0, -21.0, -52.0, 9.0]], [[-39.0, 50.0, -12.0, -47.0, -108.0], [-43.0, 22.0, -19.0, -66.0, 13.0], [22.0, -68.0, -58.0, -12.0, 18.0], [14.0, -25.0, -40.0, -83.0, 1.0]], [[-45.0, 89.0, -13.0, 28.0, -56.0], [-76.0, 73.0, -49.0, 34.0, 44.0], [7.0, 23.0, -47.0, -49.0, -18.0], [-15.0, 4.0, 30.0, 21.0, -98.0]]], [[[48.0, -24.0, 9.0, 79.0, 77.0], [-33.0, 58.0, -51.0, 56.0, -50.0], [-35.0, -18.0, 108.0, 18.0, -2.0], [-31.0, 51.0, -1.0, -7.0, -66.0]], [[-31.0, 10.0, 3.0, -71.0, -56.0], [34.0, -51.0, 51.0, -50.0, 42.0], [14.0, 13.0, -98.0, -2.0, 6.0], [37.0, -50.0, -3.0, 6.0, 55.0]], [[54.0, -66.0, 45.0, 34.0, 104.0], [24.0, -17.0, -36.0, 32.0, -41.0], [-11.0, 15.0, 114.0, 12.0, 37.0], [-61.0, 15.0, -4.0, 14.0, -9.0]], [[-20.0, 13.0, -46.0, -3.0, -4.0], [28.0, -6.0, 64.0, 26.0, 28.0], [-1.0, 46.0, -55.0, 3.0, -60.0], [52.0, 10.0, 43.0, 61.0, 25.0]], [[-16.0, 48.0, 43.0, 27.0, -31.0], [-48.0, 2.0, -37.0, 2.0, 28.0], [-14.0, 22.0, -56.0, -34.0, 25.0], [-17.0, -23.0, 0.0, 16.0, -61.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[1.0, 6.0, 0.0, 3.0, -2.0], [6.0, 3.0, 2.0, -7.0, 5.0], [2.0, 7.0, 1.0, 2.0, -4.0], [5.0, 4.0, -3.0, -1.0, 7.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(-4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-4.0, -24.0, -0.0, -12.0, 8.0], [-24.0, -12.0, -8.0, 28.0, -20.0], [-8.0, -28.0, -4.0, -8.0, 16.0], [-20.0, -16.0, 12.0, 4.0, -28.0]])+(1.-msk_ref)*numpy.array([[-4.0, -24.0, -0.0, -12.0, 8.0], [-24.0, -12.0, -8.0, 28.0, -20.0], [-8.0, -28.0, -4.0, -8.0, 16.0], [-20.0, -16.0, 12.0, 4.0, -28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[-4.0, -6.0, 3.0], [3.0, 5.0, -4.0], [1.0, 1.0, 6.0], [-7.0, -2.0, -7.0], [-7.0, -2.0, -6.0]], [[-4.0, -5.0, 7.0], [-7.0, 0.0, 2.0], [2.0, -6.0, -6.0], [7.0, 5.0, 3.0], [-5.0, -4.0, 3.0]], [[4.0, 7.0, -7.0], [-4.0, -1.0, 6.0], [-6.0, -1.0, -7.0], [7.0, -1.0, -1.0], [0.0, 5.0, -2.0]], [[-7.0, -3.0, 7.0], [-5.0, -7.0, 2.0], [-2.0, 0.0, 0.0], [-6.0, -3.0, 2.0], [7.0, -2.0, -4.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, -5.0, 2.0])+(1.-msk_arg1)*numpy.array([1.0, -3.0, -4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[52.0, -45.0, 3.0, 24.0, 26.0], [55.0, 32.0, 10.0, -47.0, 46.0], [-65.0, 33.0, 15.0, -25.0, -29.0], [57.0, 59.0, 8.0, 43.0, -26.0]])+(1.-msk_ref)*numpy.array([[2.0, 4.0, -26.0, 27.0, 23.0], [-17.0, -15.0, 44.0, -20.0, -5.0], [11.0, -25.0, 25.0, 14.0, -7.0], [-26.0, 8.0, -2.0, -5.0, 29.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[-3.0, -7.0, -3.0], [-4.0, 3.0, -4.0]], [[-7.0, -2.0, -5.0], [0.0, -2.0, 7.0]], [[2.0, -4.0, 3.0], [-1.0, -1.0, 4.0]], [[1.0, 1.0, 7.0], [-1.0, 0.0, -2.0]], [[-2.0, 1.0, -2.0], [-5.0, 0.0, 6.0]]], [[[-3.0, -5.0, 0.0], [-1.0, -1.0, 1.0]], [[-7.0, -4.0, -6.0], [1.0, 7.0, -2.0]], [[1.0, -2.0, -4.0], [4.0, -2.0, 0.0]], [[0.0, 1.0, -5.0], [-1.0, -7.0, -7.0]], [[-5.0, 0.0, 2.0], [0.0, 0.0, 1.0]]], [[[0.0, -6.0, 2.0], [0.0, -1.0, -4.0]], [[-7.0, -1.0, 7.0], [3.0, -7.0, -4.0]], [[4.0, 1.0, -3.0], [7.0, -5.0, 4.0]], [[0.0, 2.0, -4.0], [7.0, 0.0, 0.0]], [[4.0, -3.0, -5.0], [1.0, -1.0, -6.0]]], [[[6.0, -2.0, -4.0], [-7.0, 3.0, -2.0]], [[-1.0, 3.0, 1.0], [4.0, 0.0, 7.0]], [[6.0, 1.0, -1.0], [-3.0, -1.0, -7.0]], [[-2.0, -6.0, -1.0], [-7.0, -4.0, -3.0]], [[-6.0, -5.0, -1.0], [-4.0, -4.0, 4.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, -5.0, 5.0], [-4.0, -1.0, 0.0]])+(1.-msk_arg1)*numpy.array([[0.0, -2.0, 0.0], [1.0, 6.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[27.0, -27.0, 44.0, 36.0, 1.0], [24.0, -35.0, -22.0, -19.0, 0.0], [41.0, 21.0, -35.0, -58.0, -5.0], [27.0, -28.0, 15.0, 53.0, 28.0]])+(1.-msk_ref)*numpy.array([[8.0, 27.0, 21.0, -13.0, 23.0], [8.0, 41.0, -4.0, -80.0, 5.0], [-14.0, -57.0, -5.0, 3.0, -29.0], [5.0, 33.0, -46.0, -34.0, 2.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[2.0, 5.0, 1.0, -7.0, -5.0], [7.0, 2.0, 2.0, -5.0, 0.0], [-6.0, 0.0, 5.0, -5.0, 0.0], [6.0, 5.0, -5.0, 1.0, 1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, -2.0])+(1.-msk_arg1)*numpy.array([0.0, 4.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[14.0, -4.0], [35.0, -10.0], [7.0, -2.0], [-49.0, 14.0], [-35.0, 10.0]], [[49.0, -14.0], [14.0, -4.0], [14.0, -4.0], [-35.0, 10.0], [0.0, 0.0]], [[-42.0, 12.0], [0.0, 0.0], [35.0, -10.0], [-35.0, 10.0], [0.0, 0.0]], [[42.0, -12.0], [35.0, -10.0], [-35.0, 10.0], [7.0, -2.0], [7.0, -2.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 8.0], [0.0, 20.0], [0.0, 4.0], [0.0, -28.0], [0.0, -20.0]], [[0.0, 28.0], [0.0, 8.0], [0.0, 8.0], [0.0, -20.0], [0.0, 0.0]], [[0.0, -24.0], [0.0, 0.0], [0.0, 20.0], [0.0, -20.0], [0.0, 0.0]], [[0.0, 24.0], [0.0, 20.0], [0.0, -20.0], [0.0, 4.0], [0.0, 4.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[4.0, 1.0, -3.0], [-3.0, -6.0, 3.0], [-1.0, -4.0, 1.0], [-3.0, 6.0, -7.0], [-5.0, -1.0, -4.0]], [[1.0, 7.0, -2.0], [7.0, 4.0, -5.0], [-3.0, -6.0, 1.0], [-6.0, 0.0, -7.0], [-3.0, -3.0, -5.0]], [[7.0, 6.0, 4.0], [5.0, 5.0, -6.0], [7.0, -3.0, 2.0], [4.0, -5.0, -4.0], [-1.0, 7.0, 1.0]], [[-5.0, -1.0, 0.0], [-2.0, -4.0, 6.0], [-1.0, 5.0, 0.0], [6.0, 0.0, -6.0], [-6.0, 0.0, -4.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 0.0], [-6.0, -4.0], [-6.0, 4.0]])+(1.-msk_arg1)*numpy.array([[4.0, -6.0], [-3.0, 4.0], [-3.0, -3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -16.0], [36.0, 36.0], [24.0, 20.0], [24.0, -52.0], [60.0, -12.0]], [[-36.0, -36.0], [-36.0, -36.0], [48.0, 28.0], [78.0, -28.0], [66.0, -8.0]], [[-102.0, -8.0], [-24.0, -44.0], [-36.0, 20.0], [30.0, 4.0], [-42.0, -24.0]], [[36.0, 4.0], [0.0, 40.0], [-24.0, -20.0], [0.0, -24.0], [60.0, -16.0]]])+(1.-msk_ref)*numpy.array([[[22.0, -11.0], [-3.0, -15.0], [5.0, -13.0], [-9.0, 63.0], [-5.0, 38.0]], [[-11.0, 28.0], [31.0, -11.0], [3.0, -9.0], [-3.0, 57.0], [12.0, 21.0]], [[-2.0, -30.0], [23.0, 8.0], [31.0, -60.0], [43.0, -32.0], [-28.0, 31.0]], [[-17.0, 26.0], [-14.0, -22.0], [-19.0, 26.0], [42.0, -18.0], [-12.0, 48.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[-1.0, 7.0, -2.0], [2.0, -2.0, 6.0]], [[-3.0, 5.0, 0.0], [-7.0, 0.0, -6.0]], [[0.0, 0.0, 1.0], [4.0, 2.0, 3.0]], [[7.0, -5.0, 0.0], [2.0, 0.0, -5.0]], [[0.0, -4.0, 1.0], [7.0, -6.0, -3.0]]], [[[0.0, 0.0, -1.0], [-6.0, 1.0, -2.0]], [[0.0, 3.0, 5.0], [-3.0, -3.0, 0.0]], [[-2.0, 7.0, -2.0], [6.0, -3.0, -6.0]], [[-6.0, -3.0, 6.0], [5.0, 7.0, 2.0]], [[-5.0, -5.0, 0.0], [-6.0, -1.0, -7.0]]], [[[-5.0, -7.0, -5.0], [5.0, -7.0, -3.0]], [[-3.0, 2.0, 4.0], [-1.0, -1.0, 1.0]], [[-1.0, 6.0, -3.0], [0.0, -3.0, -4.0]], [[6.0, 3.0, 7.0], [-6.0, -1.0, 0.0]], [[7.0, 1.0, -3.0], [-1.0, -3.0, 5.0]]], [[[5.0, -6.0, -7.0], [6.0, -6.0, 4.0]], [[7.0, -6.0, -5.0], [-3.0, -5.0, 7.0]], [[1.0, -2.0, -2.0], [0.0, 1.0, 0.0]], [[-2.0, -7.0, 7.0], [-3.0, 4.0, 0.0]], [[3.0, -2.0, 3.0], [-2.0, -4.0, -1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 0.0], [4.0, -6.0], [-6.0, -2.0]], [[-3.0, 2.0], [-1.0, -2.0], [2.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 4.0], [2.0, -3.0], [-4.0, -1.0]], [[5.0, -2.0], [1.0, 0.0], [6.0, -6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[49.0, -48.0], [32.0, -26.0], [-14.0, -7.0], [-43.0, 49.0], [-43.0, 57.0]], [[19.0, -6.0], [-6.0, -28.0], [15.0, -2.0], [-60.0, -4.0], [-10.0, 41.0]], [[-7.0, 85.0], [-7.0, -23.0], [38.0, -12.0], [-17.0, -42.0], [31.0, -11.0]], [[9.0, 62.0], [27.0, 29.0], [2.0, 14.0], [-63.0, 14.0], [-21.0, 13.0]]])+(1.-msk_ref)*numpy.array([[[72.0, -63.0], [-43.0, 23.0], [36.0, -27.0], [-72.0, 69.0], [-1.0, 15.0]], [[-37.0, 25.0], [-32.0, -8.0], [25.0, -3.0], [50.0, -43.0], [-53.0, 49.0]], [[36.0, 14.0], [6.0, -26.0], [3.0, 5.0], [-89.0, 20.0], [-6.0, 0.0]], [[34.0, 9.0], [-12.0, 15.0], [-1.0, 12.0], [-41.0, 12.0], [-54.0, 25.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank2_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([[2.0, -1.0, 0.0, -2.0, -1.0], [-5.0, 3.0, -4.0, -7.0, 7.0], [-1.0, 7.0, -5.0, -1.0, -4.0], [7.0, -3.0, 2.0, 2.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, 4.0, 1.0, -2.0, -4.0], [7.0, 5.0, -5.0, 0.0, -4.0], [4.0, 4.0, 1.0, 5.0, -3.0], [-3.0, -4.0, 0.0, -5.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-2.0, -1.0, 4.0, 4.0, 4.0], [-4.0, 2.0, 3.0, 3.0, 7.0], [3.0, -4.0, -1.0, 2.0, -7.0], [-4.0, -7.0, -3.0, -7.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 8.0, 2.0, -4.0, -8.0], [14.0, 10.0, -10.0, 0.0, -8.0], [8.0, 8.0, 2.0, 10.0, -6.0], [-6.0, -8.0, 0.0, -10.0, -12.0]], [[-5.0, -4.0, -1.0, 2.0, 4.0], [-7.0, -5.0, 5.0, 0.0, 4.0], [-4.0, -4.0, -1.0, -5.0, 3.0], [3.0, 4.0, 0.0, 5.0, 6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-10.0, -8.0, -2.0, 4.0, 8.0], [-14.0, -10.0, 10.0, 0.0, 8.0], [-8.0, -8.0, -2.0, -10.0, 6.0], [6.0, 8.0, 0.0, 10.0, 12.0]], [[-5.0, -4.0, -1.0, 2.0, 4.0], [-7.0, -5.0, 5.0, 0.0, 4.0], [-4.0, -4.0, -1.0, -5.0, 3.0], [3.0, 4.0, 0.0, 5.0, 6.0]]], [[[-25.0, -20.0, -5.0, 10.0, 20.0], [-35.0, -25.0, 25.0, 0.0, 20.0], [-20.0, -20.0, -5.0, -25.0, 15.0], [15.0, 20.0, 0.0, 25.0, 30.0]], [[15.0, 12.0, 3.0, -6.0, -12.0], [21.0, 15.0, -15.0, 0.0, -12.0], [12.0, 12.0, 3.0, 15.0, -9.0], [-9.0, -12.0, 0.0, -15.0, -18.0]], [[-20.0, -16.0, -4.0, 8.0, 16.0], [-28.0, -20.0, 20.0, 0.0, 16.0], [-16.0, -16.0, -4.0, -20.0, 12.0], [12.0, 16.0, 0.0, 20.0, 24.0]], [[-35.0, -28.0, -7.0, 14.0, 28.0], [-49.0, -35.0, 35.0, 0.0, 28.0], [-28.0, -28.0, -7.0, -35.0, 21.0], [21.0, 28.0, 0.0, 35.0, 42.0]], [[35.0, 28.0, 7.0, -14.0, -28.0], [49.0, 35.0, -35.0, 0.0, -28.0], [28.0, 28.0, 7.0, 35.0, -21.0], [-21.0, -28.0, 0.0, -35.0, -42.0]]], [[[-5.0, -4.0, -1.0, 2.0, 4.0], [-7.0, -5.0, 5.0, 0.0, 4.0], [-4.0, -4.0, -1.0, -5.0, 3.0], [3.0, 4.0, 0.0, 5.0, 6.0]], [[35.0, 28.0, 7.0, -14.0, -28.0], [49.0, 35.0, -35.0, 0.0, -28.0], [28.0, 28.0, 7.0, 35.0, -21.0], [-21.0, -28.0, 0.0, -35.0, -42.0]], [[-25.0, -20.0, -5.0, 10.0, 20.0], [-35.0, -25.0, 25.0, 0.0, 20.0], [-20.0, -20.0, -5.0, -25.0, 15.0], [15.0, 20.0, 0.0, 25.0, 30.0]], [[-5.0, -4.0, -1.0, 2.0, 4.0], [-7.0, -5.0, 5.0, 0.0, 4.0], [-4.0, -4.0, -1.0, -5.0, 3.0], [3.0, 4.0, 0.0, 5.0, 6.0]], [[-20.0, -16.0, -4.0, 8.0, 16.0], [-28.0, -20.0, 20.0, 0.0, 16.0], [-16.0, -16.0, -4.0, -20.0, 12.0], [12.0, 16.0, 0.0, 20.0, 24.0]]], [[[35.0, 28.0, 7.0, -14.0, -28.0], [49.0, 35.0, -35.0, 0.0, -28.0], [28.0, 28.0, 7.0, 35.0, -21.0], [-21.0, -28.0, 0.0, -35.0, -42.0]], [[-15.0, -12.0, -3.0, 6.0, 12.0], [-21.0, -15.0, 15.0, 0.0, 12.0], [-12.0, -12.0, -3.0, -15.0, 9.0], [9.0, 12.0, 0.0, 15.0, 18.0]], [[10.0, 8.0, 2.0, -4.0, -8.0], [14.0, 10.0, -10.0, 0.0, -8.0], [8.0, 8.0, 2.0, 10.0, -6.0], [-6.0, -8.0, 0.0, -10.0, -12.0]], [[10.0, 8.0, 2.0, -4.0, -8.0], [14.0, 10.0, -10.0, 0.0, -8.0], [8.0, 8.0, 2.0, 10.0, -6.0], [-6.0, -8.0, 0.0, -10.0, -12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-4.0, -2.0, 8.0, 8.0, 8.0], [-8.0, 4.0, 6.0, 6.0, 14.0], [6.0, -8.0, -2.0, 4.0, -14.0], [-8.0, -14.0, -6.0, -14.0, 0.0]], [[2.0, 1.0, -4.0, -4.0, -4.0], [4.0, -2.0, -3.0, -3.0, -7.0], [-3.0, 4.0, 1.0, -2.0, 7.0], [4.0, 7.0, 3.0, 7.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[4.0, 2.0, -8.0, -8.0, -8.0], [8.0, -4.0, -6.0, -6.0, -14.0], [-6.0, 8.0, 2.0, -4.0, 14.0], [8.0, 14.0, 6.0, 14.0, 0.0]], [[2.0, 1.0, -4.0, -4.0, -4.0], [4.0, -2.0, -3.0, -3.0, -7.0], [-3.0, 4.0, 1.0, -2.0, 7.0], [4.0, 7.0, 3.0, 7.0, 0.0]]], [[[10.0, 5.0, -20.0, -20.0, -20.0], [20.0, -10.0, -15.0, -15.0, -35.0], [-15.0, 20.0, 5.0, -10.0, 35.0], [20.0, 35.0, 15.0, 35.0, 0.0]], [[-6.0, -3.0, 12.0, 12.0, 12.0], [-12.0, 6.0, 9.0, 9.0, 21.0], [9.0, -12.0, -3.0, 6.0, -21.0], [-12.0, -21.0, -9.0, -21.0, 0.0]], [[8.0, 4.0, -16.0, -16.0, -16.0], [16.0, -8.0, -12.0, -12.0, -28.0], [-12.0, 16.0, 4.0, -8.0, 28.0], [16.0, 28.0, 12.0, 28.0, 0.0]], [[14.0, 7.0, -28.0, -28.0, -28.0], [28.0, -14.0, -21.0, -21.0, -49.0], [-21.0, 28.0, 7.0, -14.0, 49.0], [28.0, 49.0, 21.0, 49.0, 0.0]], [[-14.0, -7.0, 28.0, 28.0, 28.0], [-28.0, 14.0, 21.0, 21.0, 49.0], [21.0, -28.0, -7.0, 14.0, -49.0], [-28.0, -49.0, -21.0, -49.0, 0.0]]], [[[2.0, 1.0, -4.0, -4.0, -4.0], [4.0, -2.0, -3.0, -3.0, -7.0], [-3.0, 4.0, 1.0, -2.0, 7.0], [4.0, 7.0, 3.0, 7.0, 0.0]], [[-14.0, -7.0, 28.0, 28.0, 28.0], [-28.0, 14.0, 21.0, 21.0, 49.0], [21.0, -28.0, -7.0, 14.0, -49.0], [-28.0, -49.0, -21.0, -49.0, 0.0]], [[10.0, 5.0, -20.0, -20.0, -20.0], [20.0, -10.0, -15.0, -15.0, -35.0], [-15.0, 20.0, 5.0, -10.0, 35.0], [20.0, 35.0, 15.0, 35.0, 0.0]], [[2.0, 1.0, -4.0, -4.0, -4.0], [4.0, -2.0, -3.0, -3.0, -7.0], [-3.0, 4.0, 1.0, -2.0, 7.0], [4.0, 7.0, 3.0, 7.0, 0.0]], [[8.0, 4.0, -16.0, -16.0, -16.0], [16.0, -8.0, -12.0, -12.0, -28.0], [-12.0, 16.0, 4.0, -8.0, 28.0], [16.0, 28.0, 12.0, 28.0, 0.0]]], [[[-14.0, -7.0, 28.0, 28.0, 28.0], [-28.0, 14.0, 21.0, 21.0, 49.0], [21.0, -28.0, -7.0, 14.0, -49.0], [-28.0, -49.0, -21.0, -49.0, 0.0]], [[6.0, 3.0, -12.0, -12.0, -12.0], [12.0, -6.0, -9.0, -9.0, -21.0], [-9.0, 12.0, 3.0, -6.0, 21.0], [12.0, 21.0, 9.0, 21.0, 0.0]], [[-4.0, -2.0, 8.0, 8.0, 8.0], [-8.0, 4.0, 6.0, 6.0, 14.0], [6.0, -8.0, -2.0, 4.0, -14.0], [-8.0, -14.0, -6.0, -14.0, 0.0]], [[-4.0, -2.0, 8.0, 8.0, 8.0], [-8.0, 4.0, 6.0, 6.0, 14.0], [6.0, -8.0, -2.0, 4.0, -14.0], [-8.0, -14.0, -6.0, -14.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[[5.0, -7.0, 0.0], [2.0, 0.0, -3.0], [-1.0, -5.0, 3.0], [-2.0, -4.0, -4.0], [2.0, -1.0, 5.0]], [[-6.0, -7.0, -4.0], [7.0, 2.0, 5.0], [5.0, 0.0, 4.0], [4.0, -2.0, -2.0], [-1.0, -5.0, -3.0]], [[0.0, 4.0, 7.0], [3.0, 1.0, 2.0], [3.0, 3.0, 5.0], [0.0, 7.0, 5.0], [-1.0, 2.0, -7.0]], [[-5.0, 0.0, 0.0], [1.0, 5.0, 0.0], [-7.0, 5.0, -4.0], [-3.0, 5.0, 5.0], [-6.0, -5.0, 6.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 1.0, -5.0, -5.0, -2.0], [5.0, 7.0, -7.0, -5.0, -3.0], [3.0, 5.0, 4.0, 0.0, 2.0], [-2.0, 0.0, -3.0, -5.0, 6.0]], [[3.0, 4.0, 6.0, -4.0, -7.0], [-3.0, 6.0, 3.0, 5.0, 1.0], [7.0, 4.0, -4.0, -4.0, -3.0], [3.0, -2.0, -1.0, 6.0, -3.0]], [[5.0, -3.0, -1.0, 1.0, 0.0], [1.0, 7.0, -3.0, -4.0, 7.0], [-6.0, -6.0, 6.0, -5.0, -4.0], [-3.0, -1.0, -5.0, 7.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, 4.0, -4.0, -7.0, 6.0], [-7.0, 2.0, 4.0, 4.0, -1.0], [4.0, -1.0, 6.0, 6.0, -5.0], [1.0, -7.0, 7.0, -5.0, 3.0]], [[-2.0, 3.0, 1.0, -1.0, -2.0], [-6.0, -5.0, 6.0, 4.0, 7.0], [-1.0, -1.0, 7.0, -7.0, -4.0], [-6.0, 0.0, -6.0, -5.0, 0.0]], [[5.0, 5.0, 1.0, 2.0, 5.0], [7.0, -6.0, -1.0, 1.0, 0.0], [7.0, -2.0, -1.0, -2.0, -1.0], [2.0, -3.0, 1.0, 7.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-26.0, -23.0, -67.0, 3.0, 39.0], [46.0, -7.0, -56.0, -60.0, -22.0], [-34.0, -3.0, 48.0, 28.0, 31.0], [-31.0, 14.0, -8.0, -67.0, 51.0]], [[-17.0, 11.0, -7.0, -13.0, -4.0], [7.0, -7.0, -5.0, 2.0, -27.0], [24.0, 28.0, -10.0, 15.0, 16.0], [5.0, 3.0, 9.0, -31.0, 6.0]], [[1.0, -30.0, -28.0, 28.0, 37.0], [13.0, -16.0, -17.0, -32.0, 19.0], [-56.0, -43.0, 34.0, 5.0, 1.0], [-22.0, 7.0, -7.0, -4.0, 15.0]], [[-30.0, -6.0, -10.0, 22.0, 32.0], [-2.0, -66.0, 14.0, 6.0, -26.0], [-10.0, -2.0, -16.0, 36.0, 24.0], [4.0, 12.0, 30.0, -42.0, -8.0]], [[20.0, -17.0, -21.0, -1.0, 3.0], [18.0, 43.0, -32.0, -35.0, 28.0], [-31.0, -24.0, 42.0, -21.0, -13.0], [-22.0, -3.0, -30.0, 19.0, 25.0]]], [[[-35.0, -22.0, -8.0, 54.0, 61.0], [-13.0, -112.0, 33.0, 11.0, -17.0], [-43.0, -34.0, -20.0, 48.0, 25.0], [3.0, 18.0, 45.0, -40.0, -23.0]], [[24.0, 0.0, -28.0, -38.0, -28.0], [34.0, 96.0, -58.0, -45.0, 16.0], [5.0, 13.0, 50.0, -33.0, -12.0], [-23.0, -9.0, -48.0, 12.0, 46.0]], [[15.0, -7.0, -29.0, -21.0, -10.0], [29.0, 63.0, -47.0, -41.0, 13.0], [-9.0, 1.0, 44.0, -20.0, -6.0], [-22.0, -4.0, -35.0, 3.0, 38.0]], [[-20.0, 2.0, -30.0, -14.0, 6.0], [24.0, 2.0, -28.0, -22.0, -28.0], [10.0, 24.0, 12.0, 18.0, 22.0], [-8.0, 6.0, 0.0, -46.0, 26.0]], [[-29.0, -12.0, -22.0, 22.0, 37.0], [7.0, -58.0, 1.0, -8.0, -23.0], [-20.0, -7.0, -2.0, 35.0, 25.0], [-4.0, 13.0, 23.0, -46.0, 3.0]]], [[[47.0, -5.0, 17.0, -9.0, -28.0], [-5.0, 73.0, -9.0, -8.0, 53.0], [-14.0, -26.0, 26.0, -51.0, -40.0], [-9.0, -15.0, -39.0, 73.0, 2.0]], [[10.0, 1.0, -11.0, -17.0, -13.0], [14.0, 41.0, -24.0, -18.0, 6.0], [4.0, 7.0, 20.0, -14.0, -5.0], [-9.0, -4.0, -20.0, 5.0, 19.0]], [[31.0, 0.0, -2.0, -22.0, -27.0], [11.0, 74.0, -27.0, -20.0, 29.0], [0.0, -3.0, 30.0, -37.0, -23.0], [-12.0, -11.0, -37.0, 38.0, 19.0]], [[46.0, 13.0, 37.0, -23.0, -49.0], [-16.0, 77.0, 6.0, 15.0, 42.0], [19.0, -2.0, 2.0, -53.0, -41.0], [6.0, -19.0, -32.0, 77.0, -11.0]], [[-28.0, 28.0, 24.0, -10.0, -12.0], [-18.0, -44.0, 34.0, 43.0, -44.0], [53.0, 45.0, -54.0, 27.0, 20.0], [29.0, 3.0, 36.0, -32.0, -26.0]]], [[[5.0, -5.0, 25.0, 25.0, 10.0], [-25.0, -35.0, 35.0, 25.0, 15.0], [-15.0, -25.0, -20.0, 0.0, -10.0], [10.0, 0.0, 15.0, 25.0, -30.0]], [[14.0, 21.0, 25.0, -25.0, -37.0], [-10.0, 37.0, 8.0, 20.0, 2.0], [38.0, 25.0, -16.0, -20.0, -13.0], [13.0, -10.0, -8.0, 25.0, -9.0]], [[2.0, 25.0, 69.0, 11.0, -21.0], [-54.0, -47.0, 76.0, 76.0, -2.0], [38.0, 9.0, -72.0, 0.0, -13.0], [41.0, -6.0, 36.0, 37.0, -65.0]], [[43.0, 2.0, 40.0, 0.0, -29.0], [-25.0, 44.0, 21.0, 20.0, 49.0], [-4.0, -25.0, -2.0, -45.0, -41.0], [6.0, -15.0, -21.0, 80.0, -23.0]], [[21.0, -44.0, -6.0, 56.0, 47.0], [-9.0, -30.0, 9.0, -19.0, 55.0], [-89.0, -86.0, 32.0, -10.0, -21.0], [-21.0, 4.0, -7.0, 42.0, -9.0]]]])+(1.-msk_ref)*numpy.array([[[[44.0, -1.0, -27.0, -28.0, 44.0], [7.0, 45.0, -22.0, -8.0, -54.0], [27.0, 2.0, -19.0, 79.0, 3.0], [47.0, -35.0, 77.0, 10.0, 15.0]], [[-3.0, -7.0, -11.0, -20.0, -3.0], [-35.0, 22.0, 11.0, 5.0, -2.0], [-13.0, 4.0, 15.0, 18.0, -7.0], [-4.0, -5.0, 11.0, -31.0, 6.0]], [[19.0, -4.0, 2.0, 18.0, 19.0], [58.0, 5.0, -37.0, -21.0, -34.0], [22.0, 0.0, -44.0, 23.0, 22.0], [35.0, -2.0, 26.0, 51.0, -3.0]], [[-24.0, -40.0, 0.0, 10.0, -24.0], [10.0, 40.0, -28.0, -28.0, -26.0], [-32.0, 14.0, -36.0, 24.0, 30.0], [14.0, 26.0, 6.0, 2.0, -6.0]], [[39.0, 30.0, -4.0, -3.0, 39.0], [27.0, -21.0, -3.0, 9.0, -9.0], [44.0, -11.0, 0.0, 9.0, -11.0], [18.0, -29.0, 25.0, 30.0, 6.0]]], [[[-42.0, -65.0, 13.0, 41.0, -42.0], [56.0, 47.0, -62.0, -56.0, -43.0], [-45.0, 21.0, -81.0, 21.0, 62.0], [28.0, 54.0, -4.0, 37.0, -18.0]], [[63.0, 59.0, -21.0, -41.0, 63.0], [-26.0, -26.0, 35.0, 41.0, 7.0], [61.0, -19.0, 51.0, 18.0, -48.0], [5.0, -64.0, 42.0, -10.0, 21.0]], [[50.0, 40.0, -16.0, -27.0, 50.0], [-7.0, -14.0, 16.0, 24.0, -5.0], [48.0, -13.0, 26.0, 22.0, -29.0], [13.0, -47.0, 39.0, 3.0, 15.0]], [[18.0, 0.0, -20.0, -30.0, 18.0], [-30.0, 30.0, 6.0, 6.0, -18.0], [4.0, 2.0, 12.0, 42.0, -10.0], [12.0, -22.0, 38.0, -24.0, 12.0]], [[-11.0, -34.0, -4.0, 6.0, -11.0], [16.0, 41.0, -31.0, -27.0, -34.0], [-20.0, 12.0, -38.0, 35.0, 28.0], [23.0, 16.0, 20.0, 9.0, -3.0]]], [[[27.0, 47.0, 11.0, 10.0, 27.0], [25.0, -62.0, 17.0, 23.0, 28.0], [45.0, -18.0, 21.0, -42.0, -23.0], [-10.0, -21.0, -17.0, 29.0, 0.0]], [[26.0, 25.0, -9.0, -18.0, 26.0], [-13.0, -11.0, 16.0, 18.0, 4.0], [25.0, -8.0, 23.0, 7.0, -21.0], [1.0, -27.0, 17.0, -6.0, 9.0]], [[37.0, 46.0, -4.0, -14.0, 37.0], [-4.0, -39.0, 25.0, 29.0, 18.0], [44.0, -16.0, 34.0, -13.0, -32.0], [-5.0, -36.0, 8.0, 5.0, 9.0]], [[11.0, 46.0, 12.0, 3.0, 11.0], [-7.0, -65.0, 37.0, 33.0, 49.0], [28.0, -17.0, 44.0, -59.0, -33.0], [-32.0, -15.0, -37.0, 0.0, 0.0]], [[-45.0, -33.0, -1.0, -9.0, -45.0], [-54.0, 30.0, 15.0, -3.0, 15.0], [-55.0, 13.0, 15.0, -6.0, 4.0], [-27.0, 28.0, -26.0, -54.0, -3.0]]], [[[-30.0, -20.0, 20.0, 35.0, -30.0], [35.0, -10.0, -20.0, -20.0, 5.0], [-20.0, 5.0, -30.0, -30.0, 25.0], [-5.0, 35.0, -35.0, 25.0, -15.0]], [[-4.0, 19.0, 1.0, -12.0, -4.0], [-37.0, -23.0, 34.0, 24.0, 34.0], [-1.0, -6.0, 41.0, -29.0, -25.0], [-29.0, -7.0, -23.0, -30.0, 3.0]], [[-72.0, -33.0, 29.0, 36.0, -72.0], [-9.0, -15.0, 6.0, -12.0, 42.0], [-61.0, 10.0, -3.0, -69.0, 19.0], [-45.0, 61.0, -83.0, -18.0, -21.0]], [[-3.0, 28.0, 22.0, 26.0, -3.0], [26.0, -61.0, 13.0, 13.0, 38.0], [18.0, -12.0, 12.0, -63.0, -10.0], [-23.0, 6.0, -46.0, 25.0, -9.0]], [[4.0, -9.0, 25.0, 59.0, 4.0], [114.0, -23.0, -60.0, -38.0, -29.0], [23.0, -1.0, -77.0, -13.0, 44.0], [36.0, 24.0, -6.0, 97.0, -18.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-5.0, -3.0, -5.0], [0.0, 0.0, 0.0]], [[6.0, 3.0, 0.0], [0.0, -7.0, -2.0]], [[4.0, -6.0, 4.0], [2.0, 2.0, 7.0]], [[3.0, 3.0, 4.0], [1.0, -6.0, 4.0]], [[4.0, 6.0, -1.0], [-6.0, 6.0, 7.0]]], [[[1.0, -2.0, 2.0], [5.0, 7.0, -5.0]], [[-3.0, -3.0, 0.0], [6.0, -5.0, -4.0]], [[5.0, 0.0, -3.0], [-3.0, -7.0, 4.0]], [[-5.0, 1.0, 0.0], [-2.0, 0.0, 0.0]], [[5.0, -3.0, -1.0], [1.0, -6.0, 7.0]]], [[[4.0, -3.0, 4.0], [3.0, 0.0, 7.0]], [[-3.0, 7.0, -6.0], [1.0, 1.0, -4.0]], [[2.0, -2.0, -3.0], [7.0, -2.0, -1.0]], [[-2.0, 7.0, 2.0], [-7.0, -4.0, -3.0]], [[-3.0, -5.0, -1.0], [-4.0, -3.0, 3.0]]], [[[7.0, -7.0, 3.0], [0.0, 0.0, -2.0]], [[-5.0, -7.0, -3.0], [-3.0, 6.0, 0.0]], [[-3.0, -4.0, 2.0], [6.0, -4.0, -4.0]], [[5.0, 2.0, -4.0], [3.0, -3.0, 0.0]], [[7.0, 2.0, 0.0], [5.0, -1.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, 3.0, 0.0, -1.0, 0.0], [-6.0, -1.0, -2.0, -2.0, -4.0], [6.0, 2.0, 3.0, -6.0, -4.0], [-4.0, 1.0, -2.0, -7.0, -6.0]], [[6.0, -7.0, -3.0, 3.0, 0.0], [7.0, -4.0, 4.0, 1.0, 6.0], [4.0, 4.0, 4.0, -2.0, -1.0], [-2.0, 4.0, -3.0, -4.0, 5.0]], [[4.0, -5.0, -6.0, 1.0, 0.0], [3.0, -6.0, -6.0, 1.0, 4.0], [-2.0, 7.0, -3.0, -5.0, -4.0], [0.0, 5.0, -1.0, -7.0, 0.0]]], [[[-3.0, -4.0, -2.0, 2.0, 2.0], [6.0, 1.0, -2.0, -5.0, -1.0], [6.0, 1.0, -6.0, 0.0, 5.0], [0.0, -3.0, 5.0, -5.0, 6.0]], [[1.0, -1.0, -1.0, -3.0, 1.0], [6.0, 0.0, 3.0, 0.0, 3.0], [2.0, -2.0, 1.0, -7.0, 6.0], [-7.0, 7.0, -6.0, 6.0, 1.0]], [[-2.0, -6.0, -2.0, -4.0, 0.0], [-3.0, -2.0, -4.0, 4.0, -2.0], [1.0, -5.0, 6.0, 0.0, 6.0], [4.0, 5.0, 5.0, 6.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, 1.0, -3.0, -3.0, -7.0], [4.0, 3.0, 4.0, 1.0, -1.0], [0.0, -5.0, 2.0, -4.0, -7.0], [-1.0, -2.0, 1.0, -4.0, 6.0]], [[-7.0, 6.0, -2.0, -4.0, 1.0], [1.0, 6.0, 5.0, 4.0, 1.0], [-1.0, 2.0, -5.0, -2.0, 7.0], [0.0, 0.0, 4.0, -1.0, -5.0]], [[7.0, -1.0, 0.0, 7.0, -5.0], [-1.0, -7.0, -7.0, -2.0, 4.0], [-4.0, -5.0, -2.0, 5.0, -6.0], [0.0, -4.0, 5.0, -1.0, -3.0]]], [[[2.0, 0.0, 2.0, 4.0, 4.0], [-6.0, 5.0, 2.0, 0.0, -5.0], [-7.0, 0.0, 7.0, 2.0, 2.0], [6.0, 0.0, -3.0, -1.0, -7.0]], [[-5.0, 1.0, 0.0, -3.0, 7.0], [7.0, 3.0, -6.0, 0.0, 5.0], [-2.0, 0.0, 0.0, -6.0, 0.0], [1.0, -7.0, -7.0, 7.0, -3.0]], [[7.0, -2.0, 4.0, -5.0, -5.0], [-4.0, -4.0, -1.0, 7.0, 7.0], [-5.0, -1.0, 1.0, 0.0, -7.0], [-7.0, -1.0, 0.0, 4.0, -6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-23.0, 31.0, 39.0, -9.0, 0.0], [-6.0, 47.0, 28.0, 2.0, -18.0], [-32.0, -57.0, -12.0, 61.0, 43.0], [26.0, -42.0, 24.0, 82.0, 15.0]], [[-3.0, 16.0, 2.0, 32.0, -7.0], [-51.0, -14.0, -13.0, -17.0, -23.0], [32.0, 48.0, 11.0, 7.0, -81.0], [11.0, -41.0, 11.0, -108.0, -34.0]], [[-50.0, -18.0, -26.0, -48.0, 6.0], [-51.0, -16.0, -82.0, 8.0, -46.0], [15.0, -25.0, 8.0, -46.0, 38.0], [10.0, 43.0, 39.0, 12.0, -19.0]], [[8.0, -54.0, -37.0, 14.0, -4.0], [-27.0, -46.0, -54.0, 12.0, -5.0], [20.0, 39.0, 21.0, -2.0, -38.0], [40.0, 10.0, 42.0, -78.0, 9.0]], [[30.0, -49.0, -20.0, -45.0, -6.0], [-6.0, -42.0, 24.0, 55.0, 26.0], [33.0, -28.0, 123.0, -73.0, 30.0], [-42.0, 118.0, -56.0, 63.0, -3.0]]], [[[-5.0, 10.0, -13.0, 4.0, 17.0], [73.0, 10.0, 9.0, -47.0, 18.0], [33.0, 24.0, -64.0, -61.0, 27.0], [-69.0, 12.0, -40.0, -26.0, 6.0]], [[-24.0, 17.0, 10.0, 37.0, 7.0], [15.0, 29.0, -17.0, -43.0, -19.0], [-8.0, 18.0, -86.0, 59.0, -9.0], [37.0, -88.0, 55.0, -51.0, 22.0]], [[-33.0, 25.0, 23.0, -9.0, -13.0], [-111.0, 2.0, -23.0, 18.0, -58.0], [8.0, -20.0, 59.0, 34.0, -41.0], [45.0, -30.0, 40.0, -17.0, -43.0]], [[27.0, -14.0, 1.0, 4.0, -4.0], [25.0, -1.0, 18.0, 21.0, 28.0], [-38.0, -8.0, 1.0, 28.0, 9.0], [18.0, 5.0, -3.0, 41.0, 23.0]], [[-60.0, 1.0, 5.0, -23.0, -4.0], [-105.0, 0.0, -64.0, 9.0, -75.0], [21.0, -31.0, 36.0, 23.0, -2.0], [56.0, -22.0, 76.0, -15.0, -24.0]]], [[[-37.0, -41.0, -35.0, -31.0, 6.0], [-36.0, -27.0, -78.0, 6.0, -35.0], [29.0, -8.0, 12.0, -38.0, 28.0], [18.0, 38.0, 47.0, -17.0, 0.0]], [[33.0, -9.0, 20.0, 33.0, 3.0], [73.0, 20.0, 87.0, -14.0, 40.0], [26.0, -1.0, 8.0, 27.0, 16.0], [-25.0, -21.0, -30.0, 12.0, 48.0]], [[-51.0, 15.0, 14.0, 13.0, 12.0], [-2.0, 33.0, -10.0, -48.0, -43.0], [47.0, -9.0, -43.0, 21.0, 23.0], [6.0, -61.0, 47.0, -38.0, 15.0]], [[79.0, -15.0, -9.0, 35.0, -18.0], [10.0, -39.0, 34.0, 36.0, 59.0], [-41.0, 54.0, 36.0, 16.0, -84.0], [10.0, 14.0, -45.0, -35.0, -8.0]], [[-22.0, 32.0, 26.0, -24.0, -11.0], [-71.0, 19.0, -21.0, 32.0, -33.0], [-63.0, -46.0, 13.0, 54.0, 1.0], [55.0, -22.0, 35.0, 68.0, -25.0]]], [[[-47.0, 67.0, 7.0, -17.0, 0.0], [-76.0, 7.0, -52.0, -26.0, -54.0], [6.0, 17.0, -28.0, -43.0, -45.0], [-22.0, -16.0, -6.0, -54.0, -83.0]], [[-24.0, 55.0, 39.0, -43.0, 0.0], [-10.0, 48.0, 24.0, 15.0, -13.0], [-58.0, -74.0, -10.0, 17.0, 60.0], [-8.0, 3.0, -17.0, 135.0, -17.0]], [[-21.0, 13.0, 0.0, 33.0, 8.0], [20.0, 21.0, -30.0, -42.0, -14.0], [-14.0, 26.0, -95.0, 44.0, -10.0], [32.0, -75.0, 50.0, -55.0, 18.0]], [[-31.0, 12.0, 15.0, 12.0, 3.0], [-28.0, 14.0, 7.0, -27.0, -36.0], [58.0, -1.0, 14.0, 7.0, -9.0], [-3.0, -37.0, 21.0, -48.0, -5.0]], [[-25.0, -12.0, -15.0, 12.0, 9.0], [-4.0, -10.0, -19.0, -37.0, -24.0], [78.0, 29.0, -2.0, -39.0, -11.0], [-25.0, -7.0, 11.0, -88.0, -3.0]]]])+(1.-msk_ref)*numpy.array([[[[-44.0, -18.0, 21.0, -8.0, 57.0], [-18.0, 2.0, 0.0, -7.0, -18.0], [23.0, 44.0, 15.0, 1.0, 44.0], [5.0, 30.0, -42.0, 28.0, 0.0]], [[36.0, 21.0, -32.0, 1.0, -78.0], [-14.0, 23.0, 83.0, 4.0, -52.0], [21.0, -22.0, -5.0, 12.0, -7.0], [1.0, 39.0, 67.0, -84.0, 54.0]], [[137.0, -48.0, 32.0, 7.0, -67.0], [-20.0, -64.0, -57.0, 21.0, 55.0], [-63.0, -59.0, 51.0, 8.0, -139.0], [-39.0, -45.0, -20.0, 26.0, -20.0]], [[85.0, 3.0, 3.0, 9.0, -96.0], [-53.0, -30.0, 33.0, 35.0, 9.0], [-34.0, -33.0, -6.0, 40.0, -50.0], [-31.0, 16.0, 74.0, -46.0, -22.0]], [[-18.0, 33.0, -8.0, -120.0, -34.0], [73.0, 15.0, -2.0, 79.0, 107.0], [-7.0, -10.0, -55.0, -81.0, -41.0], [-83.0, -53.0, -1.0, 55.0, -21.0]]], [[[-26.0, 4.0, -9.0, 43.0, 75.0], [39.0, 43.0, -47.0, -46.0, -20.0], [-30.0, -14.0, 38.0, -22.0, 12.0], [71.0, -54.0, -61.0, 20.0, -16.0]], [[12.0, -18.0, 11.0, 80.0, 27.0], [-70.0, 4.0, 19.0, -43.0, -83.0], [-9.0, 13.0, 47.0, 60.0, 40.0], [62.0, 45.0, 2.0, -42.0, -6.0]], [[66.0, -7.0, -5.0, -47.0, -101.0], [-24.0, -16.0, 73.0, 39.0, -9.0], [27.0, -14.0, -1.0, 1.0, -51.0], [-58.0, 47.0, 48.0, -47.0, 57.0]], [[-41.0, 1.0, 9.0, 3.0, 28.0], [-7.0, -19.0, -19.0, -1.0, 16.0], [13.0, 27.0, -29.0, 14.0, 38.0], [-7.0, 10.0, 5.0, 21.0, -21.0]], [[125.0, -32.0, 21.0, -23.0, -106.0], [-58.0, -37.0, 43.0, 44.0, 2.0], [-23.0, -33.0, 41.0, 19.0, -97.0], [-54.0, 29.0, 27.0, -31.0, 17.0]]], [[[128.0, -32.0, 28.0, 5.0, -74.0], [-37.0, -47.0, -28.0, 33.0, 43.0], [-69.0, -53.0, 43.0, 16.0, -116.0], [-35.0, -31.0, 3.0, 8.0, -36.0]], [[-140.0, 54.0, -19.0, -40.0, 89.0], [18.0, 99.0, 65.0, 9.0, -42.0], [28.0, 63.0, -26.0, -36.0, 136.0], [38.0, 27.0, -15.0, 1.0, -21.0]], [[22.0, -7.0, 8.0, 20.0, 18.0], [-43.0, 48.0, 46.0, -7.0, -68.0], [-26.0, 2.0, 68.0, 7.0, 11.0], [45.0, 23.0, -28.0, -28.0, -6.0]], [[-62.0, 40.0, -34.0, -9.0, -30.0], [23.0, -13.0, 26.0, 1.0, 11.0], [57.0, 17.0, -95.0, 14.0, 58.0], [-23.0, 27.0, 85.0, -34.0, 26.0]], [[38.0, -41.0, 23.0, 0.0, -31.0], [-25.0, -73.0, -23.0, 0.0, 20.0], [28.0, 7.0, -4.0, 27.0, -37.0], [-45.0, 28.0, 5.0, 13.0, 29.0]]], [[[98.0, -34.0, -15.0, 38.0, -61.0], [26.0, -34.0, -26.0, -41.0, -16.0], [5.0, -62.0, 41.0, 1.0, -102.0], [7.0, -24.0, -6.0, -32.0, 80.0]], [[-38.0, -38.0, 23.0, -8.0, 73.0], [36.0, -33.0, -76.0, -27.0, 31.0], [28.0, 26.0, 10.0, -23.0, -2.0], [-7.0, -20.0, -81.0, 75.0, 17.0]], [[28.0, -25.0, 13.0, 95.0, 23.0], [-66.0, -13.0, -6.0, -51.0, -71.0], [-18.0, 1.0, 48.0, 66.0, 21.0], [63.0, 30.0, 1.0, -36.0, -10.0]], [[9.0, 18.0, -13.0, -30.0, -22.0], [-13.0, 61.0, 82.0, 21.0, -49.0], [-1.0, -1.0, 29.0, -20.0, 9.0], [10.0, 27.0, 5.0, -42.0, 20.0]], [[43.0, 18.0, -15.0, -6.0, -34.0], [-7.0, 55.0, 54.0, 15.0, -35.0], [-35.0, -31.0, 39.0, -16.0, -25.0], [22.0, -7.0, 7.0, -42.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[4.0, 2.0], [-1.0, 6.0]], [[3.0, 3.0], [7.0, 2.0]], [[-2.0, -4.0], [1.0, -6.0]], [[-6.0, -5.0], [-6.0, 1.0]], [[6.0, 4.0], [-1.0, 5.0]], [[-3.0, 5.0], [0.0, 5.0]]]),self.functionspace)
      arg1=-5.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-20.0, -10.0], [5.0, -30.0]], [[-15.0, -15.0], [-35.0, -10.0]], [[10.0, 20.0], [-5.0, 30.0]], [[30.0, 25.0], [30.0, -5.0]], [[-30.0, -20.0], [5.0, -25.0]], [[15.0, -25.0], [-0.0, -25.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[-4.0, -3.0], [3.0, 0.0]], [[3.0, -6.0], [5.0, -4.0]], [[-7.0, -3.0], [4.0, -6.0]], [[-3.0, 0.0], [-4.0, -6.0]], [[7.0, -2.0], [0.0, -1.0]], [[-4.0, -1.0], [0.0, 0.0]]]),self.functionspace)
      arg1=numpy.array(-4.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[16.0, 12.0], [-12.0, -0.0]], [[-12.0, 24.0], [-20.0, 16.0]], [[28.0, 12.0], [-16.0, 24.0]], [[12.0, -0.0], [16.0, 24.0]], [[-28.0, 8.0], [-0.0, 4.0]], [[16.0, 4.0], [-0.0, -0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[[2.0, 6.0, -1.0], [4.0, -1.0, 3.0]], [[-5.0, 7.0, -6.0], [-2.0, 0.0, -7.0]]], [[[6.0, -2.0, 2.0], [3.0, 1.0, 6.0]], [[2.0, -3.0, 0.0], [2.0, 7.0, 7.0]]], [[[0.0, -3.0, -5.0], [-4.0, 3.0, 2.0]], [[-1.0, 1.0, 6.0], [-2.0, -1.0, 3.0]]], [[[2.0, 0.0, 5.0], [-5.0, 7.0, 3.0]], [[-1.0, -5.0, -5.0], [-2.0, 5.0, -1.0]]], [[[-5.0, -3.0, -4.0], [6.0, 3.0, 4.0]], [[-3.0, 7.0, -4.0], [-5.0, 2.0, 5.0]]], [[[6.0, 0.0, -5.0], [-6.0, -5.0, 0.0]], [[-7.0, -3.0, 0.0], [0.0, 3.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([-7.0, -6.0, -5.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-45.0, -37.0], [23.0, 49.0]], [[-40.0, -57.0], [4.0, -91.0]], [[43.0, 0.0], [-29.0, 5.0]], [[-39.0, -22.0], [62.0, -11.0]], [[73.0, -80.0], [-1.0, -2.0]], [[-17.0, 72.0], [67.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_array_rank1_offset0(self):
      arg0=Data(numpy.array([[[0.0, -2.0], [-4.0, -2.0]], [[2.0, -4.0], [0.0, 5.0]], [[7.0, 3.0], [5.0, -4.0]], [[-3.0, 6.0], [2.0, 5.0]], [[-1.0, 0.0], [2.0, 2.0]], [[-6.0, 1.0], [4.0, 4.0]]]),self.functionspace)
      arg1=numpy.array([-3.0, -7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 0.0], [6.0, 14.0]], [[12.0, 28.0], [6.0, 14.0]]], [[[-6.0, -14.0], [12.0, 28.0]], [[0.0, 0.0], [-15.0, -35.0]]], [[[-21.0, -49.0], [-9.0, -21.0]], [[-15.0, -35.0], [12.0, 28.0]]], [[[9.0, 21.0], [-18.0, -42.0]], [[-6.0, -14.0], [-15.0, -35.0]]], [[[3.0, 7.0], [0.0, 0.0]], [[-6.0, -14.0], [-6.0, -14.0]]], [[[18.0, 42.0], [-3.0, -7.0]], [[-12.0, -28.0], [-12.0, -28.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-6.0, -4.0, 3.0], [3.0, 6.0, -4.0]], [[6.0, 5.0, 6.0], [-3.0, -4.0, 5.0]]], [[[0.0, 5.0, 0.0], [0.0, 5.0, 0.0]], [[5.0, -5.0, 7.0], [-3.0, 4.0, 1.0]]], [[[4.0, -6.0, 0.0], [-6.0, 0.0, -7.0]], [[7.0, -1.0, -1.0], [6.0, 5.0, -3.0]]], [[[4.0, 6.0, 5.0], [0.0, 0.0, -5.0]], [[1.0, 3.0, -5.0], [-1.0, -6.0, 4.0]]], [[[0.0, 6.0, -1.0], [-7.0, -2.0, -5.0]], [[-6.0, -7.0, 5.0], [-6.0, 7.0, -6.0]]], [[[3.0, 3.0, 2.0], [0.0, 7.0, -4.0]], [[-1.0, -3.0, -1.0], [-6.0, -6.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[4.0, -5.0], [-4.0, 1.0], [5.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[7.0, 44.0], [-32.0, -33.0]], [[34.0, 11.0], [29.0, 41.0]]], [[[-20.0, 5.0], [-20.0, 5.0]], [[75.0, 12.0], [-23.0, 25.0]]], [[[40.0, -26.0], [-59.0, -12.0]], [[27.0, -42.0], [-11.0, -43.0]]], [[[17.0, 16.0], [-25.0, -30.0]], [[-33.0, -32.0], [40.0, 23.0]]], [[[-29.0, 0.0], [-45.0, 3.0]], [[29.0, 53.0], [-82.0, 1.0]]], [[[10.0, 0.0], [-48.0, -17.0]], [[3.0, -4.0], [0.0, 24.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[4.0, 5.0], [-6.0, 7.0]], [[6.0, -7.0], [-3.0, 2.0]], [[7.0, 1.0], [-7.0, 6.0]], [[-7.0, 0.0], [-2.0, -7.0]], [[4.0, -3.0], [-6.0, -2.0]], [[3.0, -3.0], [-7.0, 4.0]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-7.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-28.0, -35.0], [42.0, -49.0]], [[-42.0, 49.0], [21.0, -14.0]], [[-49.0, -7.0], [49.0, -42.0]], [[49.0, -0.0], [14.0, 49.0]], [[-28.0, 21.0], [42.0, 14.0]], [[-21.0, 21.0], [49.0, -28.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[[6.0, -6.0, -3.0], [-5.0, -5.0, -6.0]], [[-1.0, -7.0, 5.0], [0.0, 2.0, -1.0]]], [[[6.0, -4.0, 6.0], [-5.0, -7.0, -3.0]], [[2.0, -3.0, 3.0], [0.0, 0.0, -3.0]]], [[[3.0, -6.0, 1.0], [3.0, 4.0, 7.0]], [[-2.0, -7.0, 7.0], [-4.0, -1.0, -2.0]]], [[[3.0, 3.0, -6.0], [6.0, 3.0, -2.0]], [[3.0, 2.0, -3.0], [3.0, 0.0, 6.0]]], [[[-2.0, -1.0, 6.0], [-6.0, 6.0, 5.0]], [[-7.0, 5.0, -1.0], [-4.0, 1.0, -2.0]]], [[[4.0, -2.0, 0.0], [6.0, 3.0, -5.0]], [[-5.0, 5.0, 5.0], [-6.0, 7.0, 5.0]]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-6.0, -6.0, 5.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-15.0, 30.0], [73.0, -17.0]], [[18.0, 57.0], [21.0, -15.0]], [[23.0, -7.0], [89.0, 20.0]], [[-66.0, -64.0], [-45.0, 12.0]], [[48.0, 25.0], [7.0, 8.0]], [[-12.0, -79.0], [25.0, 19.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[[-2.0, -4.0], [5.0, 1.0]], [[1.0, -5.0], [-5.0, 3.0]], [[-3.0, -3.0], [5.0, -1.0]], [[3.0, 4.0], [-2.0, 7.0]], [[2.0, 6.0], [-5.0, 5.0]], [[2.0, -1.0], [-7.0, 0.0]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([6.0, -4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-12.0, 8.0], [-24.0, 16.0]], [[30.0, -20.0], [6.0, -4.0]]], [[[6.0, -4.0], [-30.0, 20.0]], [[-30.0, 20.0], [18.0, -12.0]]], [[[-18.0, 12.0], [-18.0, 12.0]], [[30.0, -20.0], [-6.0, 4.0]]], [[[18.0, -12.0], [24.0, -16.0]], [[-12.0, 8.0], [42.0, -28.0]]], [[[12.0, -8.0], [36.0, -24.0]], [[-30.0, 20.0], [30.0, -20.0]]], [[[12.0, -8.0], [-6.0, 4.0]], [[-42.0, 28.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-5.0, -2.0, 7.0], [-7.0, 3.0, -1.0]], [[5.0, 1.0, -4.0], [3.0, -3.0, -2.0]]], [[[5.0, 5.0, 5.0], [5.0, 3.0, 4.0]], [[5.0, 4.0, 4.0], [-2.0, 7.0, -2.0]]], [[[-2.0, -7.0, -7.0], [1.0, 4.0, 1.0]], [[-6.0, -2.0, 3.0], [0.0, 0.0, -4.0]]], [[[2.0, -3.0, 0.0], [-6.0, 7.0, 0.0]], [[0.0, 3.0, -1.0], [0.0, 6.0, -6.0]]], [[[-4.0, 6.0, 5.0], [2.0, 5.0, 6.0]], [[5.0, -3.0, 0.0], [0.0, -4.0, -4.0]]], [[[-7.0, -7.0, 4.0], [0.0, 3.0, 1.0]], [[2.0, -6.0, 5.0], [0.0, -5.0, -4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[0.0, 1.0], [-7.0, 1.0], [6.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[56.0, -49.0], [-27.0, 2.0]], [[-31.0, 30.0], [9.0, 12.0]]], [[[-5.0, -20.0], [3.0, -16.0]], [[-4.0, -15.0], [-61.0, 17.0]]], [[[7.0, 33.0], [-22.0, -1.0]], [[32.0, -26.0], [-24.0, 24.0]]], [[[21.0, -1.0], [-49.0, 1.0]], [[-27.0, 9.0], [-78.0, 42.0]]], [[[-12.0, -28.0], [1.0, -29.0]], [[21.0, 2.0], [4.0, 20.0]]], [[[73.0, -38.0], [-15.0, -3.0]], [[72.0, -34.0], [11.0, 19.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[1.0, -7.0], [0.0, 2.0]], [[-2.0, 3.0], [-4.0, -6.0]], [[-2.0, -4.0], [7.0, 4.0]], [[5.0, -2.0], [6.0, -2.0]], [[-4.0, 7.0], [3.0, -3.0]], [[-1.0, 1.0], [0.0, -5.0]]]),self.functionspace)
      arg1=Data(-6.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-6.0, 42.0], [-0.0, -12.0]], [[12.0, -18.0], [24.0, 36.0]], [[12.0, 24.0], [-42.0, -24.0]], [[-30.0, 12.0], [-36.0, 12.0]], [[24.0, -42.0], [-18.0, 18.0]], [[6.0, -6.0], [-0.0, 30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[-1.0, -2.0, -2.0], [-2.0, 6.0, -2.0]], [[-2.0, 1.0, -2.0], [-1.0, 4.0, 1.0]]], [[[-2.0, 0.0, 0.0], [-3.0, 4.0, 2.0]], [[0.0, 3.0, 4.0], [6.0, -7.0, 5.0]]], [[[-5.0, 0.0, 2.0], [0.0, 1.0, -7.0]], [[-5.0, -7.0, -7.0], [7.0, 7.0, 5.0]]], [[[0.0, -7.0, 3.0], [-1.0, 7.0, -4.0]], [[0.0, 7.0, 0.0], [4.0, 2.0, -6.0]]], [[[0.0, -3.0, -2.0], [-4.0, -1.0, 6.0]], [[-5.0, -1.0, -3.0], [7.0, 0.0, 4.0]]], [[[0.0, -5.0, 3.0], [-7.0, 1.0, -2.0]], [[-5.0, 3.0, 5.0], [-6.0, 1.0, 6.0]]]]),self.functionspace)
      arg1=Data(numpy.array([-3.0, 5.0, -2.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-3.0, 40.0], [15.0, 21.0]], [[6.0, 25.0], [7.0, -63.0]], [[11.0, 19.0], [-6.0, 4.0]], [[-41.0, 46.0], [35.0, 10.0]], [[-11.0, -5.0], [16.0, -29.0]], [[-31.0, 30.0], [20.0, 11.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[[6.0, 4.0], [0.0, 5.0]], [[7.0, -2.0], [-6.0, 0.0]], [[4.0, -4.0], [5.0, 4.0]], [[1.0, -5.0], [-3.0, -5.0]], [[6.0, 3.0], [5.0, 5.0]], [[1.0, 0.0], [-5.0, -7.0]]]),self.functionspace)
      arg1=Data(numpy.array([3.0, 0.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[18.0, 0.0], [12.0, 0.0]], [[0.0, 0.0], [15.0, 0.0]]], [[[21.0, 0.0], [-6.0, 0.0]], [[-18.0, 0.0], [0.0, 0.0]]], [[[12.0, 0.0], [-12.0, 0.0]], [[15.0, 0.0], [12.0, 0.0]]], [[[3.0, 0.0], [-15.0, 0.0]], [[-9.0, 0.0], [-15.0, 0.0]]], [[[18.0, 0.0], [9.0, 0.0]], [[15.0, 0.0], [15.0, 0.0]]], [[[3.0, 0.0], [0.0, 0.0]], [[-15.0, 0.0], [-21.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[0.0, 3.0, 6.0], [2.0, -6.0, -4.0]], [[0.0, 2.0, 4.0], [-3.0, 7.0, 5.0]]], [[[2.0, 0.0, 0.0], [-2.0, 3.0, -6.0]], [[-7.0, 6.0, -3.0], [7.0, -3.0, 6.0]]], [[[0.0, 1.0, -7.0], [-2.0, 7.0, -2.0]], [[-6.0, -1.0, 2.0], [2.0, -7.0, 5.0]]], [[[-3.0, 7.0, 5.0], [2.0, -2.0, 3.0]], [[7.0, -3.0, -4.0], [1.0, -1.0, 6.0]]], [[[1.0, -3.0, 0.0], [6.0, 7.0, 4.0]], [[-7.0, -3.0, 7.0], [-3.0, -7.0, -4.0]]], [[[0.0, -5.0, -4.0], [-2.0, 0.0, -1.0]], [[0.0, -7.0, 3.0], [-6.0, -5.0, 0.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[-1.0, 3.0], [2.0, 0.0], [-7.0, -2.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-36.0, -12.0], [14.0, 14.0]], [[-24.0, -8.0], [-18.0, -19.0]]], [[[-2.0, 6.0], [50.0, 6.0]], [[40.0, -15.0], [-55.0, 9.0]]], [[[51.0, 14.0], [30.0, -2.0]], [[-10.0, -22.0], [-51.0, -4.0]]], [[[-18.0, -19.0], [-27.0, 0.0]], [[15.0, 29.0], [-45.0, -9.0]]], [[[-7.0, 3.0], [-20.0, 10.0]], [[-48.0, -35.0], [17.0, -1.0]]], [[[18.0, 8.0], [9.0, -4.0]], [[-35.0, -6.0], [-4.0, -18.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[4.0, 0.0], [-7.0, 7.0]], [[-1.0, -6.0], [-3.0, -1.0]], [[1.0, -2.0], [6.0, -5.0]], [[0.0, 3.0], [-5.0, 6.0]], [[-4.0, 1.0], [0.0, 6.0]], [[0.0, 4.0], [-4.0, 4.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-7.0)+(1-msk_arg1)*(-1.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-28.0, -0.0], [49.0, -49.0]], [[7.0, 42.0], [21.0, 7.0]], [[-7.0, 14.0], [-42.0, 35.0]], [[-0.0, -21.0], [35.0, -42.0]], [[28.0, -7.0], [-0.0, -42.0]], [[-0.0, -28.0], [28.0, -28.0]]])+(1.-msk_ref)*numpy.array([[[-4.0, -0.0], [7.0, -7.0]], [[1.0, 6.0], [3.0, 1.0]], [[-1.0, 2.0], [-6.0, 5.0]], [[-0.0, -3.0], [5.0, -6.0]], [[4.0, -1.0], [-0.0, -6.0]], [[-0.0, -4.0], [4.0, -4.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[-1.0, -2.0, -4.0], [0.0, 6.0, 7.0]], [[0.0, 4.0, -1.0], [-5.0, 0.0, -7.0]]], [[[6.0, -6.0, -7.0], [-3.0, -6.0, 3.0]], [[-4.0, -2.0, 1.0], [-4.0, -6.0, 5.0]]], [[[4.0, 1.0, -1.0], [1.0, -1.0, 3.0]], [[4.0, 0.0, 4.0], [2.0, -5.0, -6.0]]], [[[5.0, -1.0, 1.0], [3.0, 4.0, -6.0]], [[-7.0, 6.0, 0.0], [2.0, -1.0, 4.0]]], [[[2.0, 6.0, 5.0], [7.0, -5.0, 2.0]], [[-1.0, -5.0, 0.0], [-7.0, 3.0, 5.0]]], [[[1.0, 1.0, 6.0], [6.0, -3.0, 7.0]], [[-6.0, 2.0, 0.0], [7.0, 0.0, -5.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, -2.0, -3.0])+(1.-msk_arg1)*numpy.array([-3.0, -2.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[22.0, -33.0], [-5.0, 51.0]], [[-3.0, 21.0], [25.0, 21.0]], [[-23.0, -13.0], [-36.0, 16.0]], [[-31.0, -8.0], [30.0, -22.0]], [[-39.0, -38.0], [16.0, 21.0]], [[-26.0, -51.0], [32.0, -27.0]]])+(1.-msk_ref)*numpy.array([[[7.0, -12.0], [-8.0, 15.0]], [[-6.0, 21.0], [16.0, 24.0]], [[-14.0, -1.0], [-12.0, 4.0]], [[-13.0, -17.0], [9.0, -4.0]], [[-18.0, -11.0], [13.0, 15.0]], [[-5.0, -12.0], [14.0, -21.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank3_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[[0.0, -2.0], [-7.0, -4.0]], [[-3.0, 3.0], [-2.0, -3.0]], [[4.0, -3.0], [-5.0, -2.0]], [[1.0, 0.0], [0.0, -5.0]], [[3.0, 5.0], [6.0, 0.0]], [[6.0, -7.0], [2.0, 1.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, 2.0])+(1.-msk_arg1)*numpy.array([6.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [-2.0, -4.0]], [[-7.0, -14.0], [-4.0, -8.0]]], [[[-3.0, -6.0], [3.0, 6.0]], [[-2.0, -4.0], [-3.0, -6.0]]], [[[4.0, 8.0], [-3.0, -6.0]], [[-5.0, -10.0], [-2.0, -4.0]]], [[[1.0, 2.0], [0.0, 0.0]], [[0.0, 0.0], [-5.0, -10.0]]], [[[3.0, 6.0], [5.0, 10.0]], [[6.0, 12.0], [0.0, 0.0]]], [[[6.0, 12.0], [-7.0, -14.0]], [[2.0, 4.0], [1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [-12.0, 2.0]], [[-42.0, 7.0], [-24.0, 4.0]]], [[[-18.0, 3.0], [18.0, -3.0]], [[-12.0, 2.0], [-18.0, 3.0]]], [[[24.0, -4.0], [-18.0, 3.0]], [[-30.0, 5.0], [-12.0, 2.0]]], [[[6.0, -1.0], [0.0, 0.0]], [[0.0, 0.0], [-30.0, 5.0]]], [[[18.0, -3.0], [30.0, -5.0]], [[36.0, -6.0], [0.0, 0.0]]], [[[36.0, -6.0], [-42.0, 7.0]], [[12.0, -2.0], [6.0, -1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-2.0, 0.0, 1.0], [1.0, -4.0, 1.0]], [[-4.0, -6.0, -1.0], [5.0, 1.0, 0.0]]], [[[-5.0, 6.0, -4.0], [-6.0, 3.0, 5.0]], [[0.0, 4.0, 6.0], [-1.0, 2.0, -6.0]]], [[[6.0, 4.0, 2.0], [7.0, 1.0, -5.0]], [[6.0, -1.0, 5.0], [-3.0, -7.0, 4.0]]], [[[1.0, 0.0, -7.0], [7.0, 0.0, -3.0]], [[-4.0, -7.0, -5.0], [-1.0, 4.0, -2.0]]], [[[7.0, 6.0, 5.0], [4.0, -2.0, -4.0]], [[5.0, 7.0, 6.0], [-5.0, 2.0, 0.0]]], [[[6.0, -7.0, -1.0], [-2.0, 0.0, -2.0]], [[6.0, 0.0, -7.0], [-5.0, -7.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -1.0], [-1.0, -7.0], [6.0, -3.0]])+(1.-msk_arg1)*numpy.array([[0.0, 5.0], [-1.0, 3.0], [0.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[16.0, -1.0], [5.0, 24.0]], [[20.0, 49.0], [-26.0, -12.0]]], [[[-5.0, -25.0], [57.0, -30.0]], [[32.0, -46.0], [-33.0, 5.0]]], [[[-22.0, -40.0], [-66.0, 1.0]], [[1.0, -14.0], [46.0, 40.0]]], [[[-47.0, 20.0], [-53.0, 2.0]], [[-3.0, 68.0], [-11.0, -21.0]]], [[[-11.0, -64.0], [-42.0, 22.0]], [[4.0, -72.0], [23.0, -9.0]]], [[[-29.0, 46.0], [-2.0, 8.0]], [[-72.0, 15.0], [32.0, 54.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -10.0], [4.0, -7.0]], [[6.0, -38.0], [-1.0, 28.0]]], [[[-6.0, -7.0], [-3.0, -21.0]], [[-4.0, 12.0], [-2.0, 1.0]]], [[[-4.0, 42.0], [-1.0, 38.0]], [[1.0, 27.0], [7.0, -36.0]]], [[[0.0, 5.0], [0.0, 35.0]], [[7.0, -41.0], [-4.0, 7.0]]], [[[-6.0, 53.0], [2.0, 14.0]], [[-7.0, 46.0], [-2.0, -19.0]]], [[[7.0, 9.0], [0.0, -10.0]], [[0.0, 30.0], [7.0, -46.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-3.0, 4.0, 5.0, -5.0], [4.0, 1.0, -1.0, 2.0], [7.0, -3.0, 2.0, 3.0]], [[-7.0, 1.0, 5.0, 7.0], [-7.0, 0.0, -4.0, 5.0], [-6.0, 6.0, -6.0, 5.0]]], [[[-2.0, -7.0, 0.0, -1.0], [-1.0, 0.0, -6.0, -1.0], [0.0, 2.0, 6.0, -5.0]], [[6.0, -4.0, 0.0, 7.0], [3.0, 4.0, 2.0, -7.0], [5.0, -7.0, -6.0, -3.0]]], [[[-1.0, 3.0, -5.0, -2.0], [-7.0, -7.0, 0.0, -7.0], [-7.0, 7.0, 6.0, 1.0]], [[7.0, -7.0, -5.0, -6.0], [-7.0, 7.0, 7.0, 0.0], [7.0, -5.0, -4.0, 3.0]]]]),self.functionspace)
      arg1=-3.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[9.0, -12.0, -15.0, 15.0], [-12.0, -3.0, 3.0, -6.0], [-21.0, 9.0, -6.0, -9.0]], [[21.0, -3.0, -15.0, -21.0], [21.0, -0.0, 12.0, -15.0], [18.0, -18.0, 18.0, -15.0]]], [[[6.0, 21.0, -0.0, 3.0], [3.0, -0.0, 18.0, 3.0], [-0.0, -6.0, -18.0, 15.0]], [[-18.0, 12.0, -0.0, -21.0], [-9.0, -12.0, -6.0, 21.0], [-15.0, 21.0, 18.0, 9.0]]], [[[3.0, -9.0, 15.0, 6.0], [21.0, 21.0, -0.0, 21.0], [21.0, -21.0, -18.0, -3.0]], [[-21.0, 21.0, 15.0, 18.0], [21.0, -21.0, -21.0, -0.0], [-21.0, 15.0, 12.0, -9.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[[4.0, 0.0, -5.0, 6.0], [7.0, -1.0, -5.0, -5.0], [2.0, -1.0, -7.0, -3.0]], [[2.0, -3.0, 4.0, 6.0], [6.0, 2.0, 3.0, 6.0], [5.0, -1.0, 0.0, 0.0]]], [[[2.0, 7.0, 3.0, 2.0], [0.0, -1.0, -2.0, -3.0], [-7.0, 7.0, -3.0, -6.0]], [[2.0, 2.0, 6.0, -6.0], [6.0, 4.0, -3.0, 0.0], [-4.0, -1.0, 2.0, 6.0]]], [[[7.0, 2.0, 3.0, -1.0], [0.0, -4.0, -7.0, -7.0], [2.0, 1.0, -2.0, -6.0]], [[-7.0, -6.0, 0.0, 6.0], [4.0, -6.0, -4.0, 1.0], [5.0, 1.0, 7.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array(7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[28.0, 0.0, -35.0, 42.0], [49.0, -7.0, -35.0, -35.0], [14.0, -7.0, -49.0, -21.0]], [[14.0, -21.0, 28.0, 42.0], [42.0, 14.0, 21.0, 42.0], [35.0, -7.0, 0.0, 0.0]]], [[[14.0, 49.0, 21.0, 14.0], [0.0, -7.0, -14.0, -21.0], [-49.0, 49.0, -21.0, -42.0]], [[14.0, 14.0, 42.0, -42.0], [42.0, 28.0, -21.0, 0.0], [-28.0, -7.0, 14.0, 42.0]]], [[[49.0, 14.0, 21.0, -7.0], [0.0, -28.0, -49.0, -49.0], [14.0, 7.0, -14.0, -42.0]], [[-49.0, -42.0, 0.0, 42.0], [28.0, -42.0, -28.0, 7.0], [35.0, 7.0, 49.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-1.0, 3.0, 0.0, -6.0], [-7.0, -1.0, -3.0, -5.0], [1.0, -2.0, 5.0, 6.0]], [[-1.0, 5.0, 5.0, 0.0], [-3.0, 4.0, 4.0, -4.0], [1.0, 4.0, 0.0, -2.0]]], [[[-5.0, 0.0, 3.0, 0.0], [0.0, 0.0, -2.0, -7.0], [6.0, -7.0, 1.0, -2.0]], [[-1.0, -6.0, -6.0, -3.0], [1.0, -6.0, 6.0, 4.0], [0.0, 4.0, -4.0, 7.0]]], [[[1.0, 1.0, -2.0, 0.0], [-2.0, -3.0, -2.0, 2.0], [-4.0, 3.0, 0.0, -1.0]], [[-6.0, 3.0, -7.0, -6.0], [0.0, -5.0, -1.0, -6.0], [2.0, -4.0, 2.0, 7.0]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(2.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-2.0, 6.0, 0.0, -12.0], [-14.0, -2.0, -6.0, -10.0], [2.0, -4.0, 10.0, 12.0]], [[-2.0, 10.0, 10.0, 0.0], [-6.0, 8.0, 8.0, -8.0], [2.0, 8.0, 0.0, -4.0]]], [[[-10.0, 0.0, 6.0, 0.0], [0.0, 0.0, -4.0, -14.0], [12.0, -14.0, 2.0, -4.0]], [[-2.0, -12.0, -12.0, -6.0], [2.0, -12.0, 12.0, 8.0], [0.0, 8.0, -8.0, 14.0]]], [[[2.0, 2.0, -4.0, 0.0], [-4.0, -6.0, -4.0, 4.0], [-8.0, 6.0, 0.0, -2.0]], [[-12.0, 6.0, -14.0, -12.0], [0.0, -10.0, -2.0, -12.0], [4.0, -8.0, 4.0, 14.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[7.0, 2.0, 5.0, -3.0], [-6.0, -6.0, 1.0, -2.0], [-5.0, 4.0, 7.0, -1.0]], [[-1.0, -7.0, -2.0, -1.0], [4.0, 6.0, 0.0, 4.0], [7.0, 6.0, -7.0, -1.0]]], [[[-7.0, 7.0, 0.0, 7.0], [6.0, 0.0, -1.0, 0.0], [0.0, -3.0, -5.0, -6.0]], [[3.0, 1.0, 4.0, -6.0], [1.0, 2.0, -3.0, -4.0], [-3.0, 6.0, 2.0, -6.0]]], [[[4.0, 1.0, -7.0, -2.0], [-7.0, 2.0, 4.0, 3.0], [5.0, 6.0, 1.0, 1.0]], [[-5.0, -4.0, 5.0, 1.0], [6.0, -5.0, -6.0, 6.0], [6.0, -1.0, 7.0, -2.0]]]]),self.functionspace)
      arg1=Data(-6.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-42.0, -12.0, -30.0, 18.0], [36.0, 36.0, -6.0, 12.0], [30.0, -24.0, -42.0, 6.0]], [[6.0, 42.0, 12.0, 6.0], [-24.0, -36.0, -0.0, -24.0], [-42.0, -36.0, 42.0, 6.0]]], [[[42.0, -42.0, -0.0, -42.0], [-36.0, -0.0, 6.0, -0.0], [-0.0, 18.0, 30.0, 36.0]], [[-18.0, -6.0, -24.0, 36.0], [-6.0, -12.0, 18.0, 24.0], [18.0, -36.0, -12.0, 36.0]]], [[[-24.0, -6.0, 42.0, 12.0], [42.0, -12.0, -24.0, -18.0], [-30.0, -36.0, -6.0, -6.0]], [[30.0, 24.0, -30.0, -6.0], [-36.0, 30.0, 36.0, -36.0], [-36.0, 6.0, -42.0, 12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_constData_rank4_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-4.0, 7.0, -3.0, -5.0], [6.0, -3.0, -6.0, -2.0], [0.0, 0.0, -5.0, 4.0]], [[-1.0, 3.0, -5.0, 0.0], [3.0, 0.0, -5.0, -2.0], [-6.0, 1.0, 0.0, 1.0]]], [[[3.0, 5.0, -1.0, -2.0], [-6.0, -6.0, -3.0, -5.0], [-3.0, -2.0, -3.0, -5.0]], [[0.0, -1.0, -3.0, -7.0], [-2.0, 4.0, -7.0, 3.0], [-5.0, -6.0, 6.0, -5.0]]], [[[7.0, -2.0, -4.0, -3.0], [-2.0, 3.0, 1.0, 7.0], [-5.0, -1.0, 5.0, -6.0]], [[-5.0, 2.0, 5.0, 3.0], [0.0, 7.0, 5.0, -5.0], [-6.0, -5.0, -5.0, -3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-7.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[12.0, -21.0, 9.0, 15.0], [-18.0, 9.0, 18.0, 6.0], [-0.0, -0.0, 15.0, -12.0]], [[3.0, -9.0, 15.0, -0.0], [-9.0, -0.0, 15.0, 6.0], [18.0, -3.0, -0.0, -3.0]]], [[[-9.0, -15.0, 3.0, 6.0], [18.0, 18.0, 9.0, 15.0], [9.0, 6.0, 9.0, 15.0]], [[-0.0, 3.0, 9.0, 21.0], [6.0, -12.0, 21.0, -9.0], [15.0, 18.0, -18.0, 15.0]]], [[[-21.0, 6.0, 12.0, 9.0], [6.0, -9.0, -3.0, -21.0], [15.0, 3.0, -15.0, 18.0]], [[15.0, -6.0, -15.0, -9.0], [-0.0, -21.0, -15.0, 15.0], [18.0, 15.0, 15.0, 9.0]]]])+(1.-msk_ref)*numpy.array([[[[28.0, -49.0, 21.0, 35.0], [-42.0, 21.0, 42.0, 14.0], [-0.0, -0.0, 35.0, -28.0]], [[7.0, -21.0, 35.0, -0.0], [-21.0, -0.0, 35.0, 14.0], [42.0, -7.0, -0.0, -7.0]]], [[[-21.0, -35.0, 7.0, 14.0], [42.0, 42.0, 21.0, 35.0], [21.0, 14.0, 21.0, 35.0]], [[-0.0, 7.0, 21.0, 49.0], [14.0, -28.0, 49.0, -21.0], [35.0, 42.0, -42.0, 35.0]]], [[[-49.0, 14.0, 28.0, 21.0], [14.0, -21.0, -7.0, -49.0], [35.0, 7.0, -35.0, 42.0]], [[35.0, -14.0, -35.0, -21.0], [-0.0, -49.0, -35.0, 35.0], [42.0, 35.0, 35.0, 21.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-6.0)+(1-msk_arg0)*(-3.0)
      arg1=5.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-30.0)+(1.-msk_ref)*numpy.array(-15.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-3.0)+(1-msk_arg0)*(4.0)
      arg1=numpy.array(-5.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(15.0)+(1.-msk_ref)*numpy.array(-20.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, -2.0, -6.0])+(1.-msk_arg0)*numpy.array([0.0, 2.0, 3.0])
      arg1=numpy.array([-4.0, -6.0, -1.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(30.0)+(1.-msk_ref)*numpy.array(-15.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -7.0, 0.0], [3.0, -4.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-3.0, -4.0, -7.0], [-1.0, -6.0, 5.0]])
      arg1=numpy.array([[0.0, -7.0, 6.0], [-5.0, 7.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(42.0)+(1.-msk_ref)*numpy.array(-81.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, -5.0, 5.0], [0.0, -3.0, -3.0], [-6.0, -3.0, -3.0], [-2.0, -6.0, 2.0]], [[7.0, -4.0, -7.0], [6.0, 0.0, 0.0], [-3.0, 4.0, -3.0], [-4.0, -2.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, -2.0, 2.0], [0.0, 7.0, -3.0], [-7.0, 1.0, -2.0], [0.0, 4.0, 5.0]], [[-5.0, -3.0, 0.0], [-4.0, 5.0, 3.0], [-1.0, -7.0, 3.0], [6.0, 0.0, 7.0]]])
      arg1=numpy.array([[[4.0, 4.0, 0.0], [5.0, -7.0, -6.0], [3.0, -7.0, -4.0], [7.0, 3.0, 0.0]], [[-5.0, 6.0, -6.0], [-5.0, 2.0, 5.0], [5.0, 5.0, -1.0], [1.0, -4.0, 5.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-10.0)+(1.-msk_ref)*numpy.array(-13.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -5.0], [-1.0, 0.0], [-4.0, -2.0]], [[5.0, 0.0], [3.0, 3.0], [-3.0, 0.0]]], [[[7.0, -6.0], [-5.0, 4.0], [-3.0, -2.0]], [[-6.0, -3.0], [5.0, -5.0], [0.0, 4.0]]], [[[1.0, -6.0], [6.0, -5.0], [-1.0, -5.0]], [[-5.0, -5.0], [0.0, 2.0], [-4.0, -5.0]]], [[[3.0, 0.0], [4.0, -4.0], [6.0, 4.0]], [[3.0, -2.0], [0.0, 0.0], [-1.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -2.0], [6.0, -1.0], [5.0, -7.0]], [[3.0, 3.0], [7.0, -5.0], [-2.0, -3.0]]], [[[0.0, -5.0], [5.0, 1.0], [-1.0, 5.0]], [[2.0, 0.0], [-3.0, 1.0], [-1.0, 5.0]]], [[[-2.0, 1.0], [-5.0, 5.0], [1.0, -7.0]], [[0.0, 0.0], [2.0, -1.0], [-5.0, 2.0]]], [[[-1.0, 0.0], [0.0, 0.0], [5.0, -4.0]], [[2.0, 6.0], [1.0, -5.0], [0.0, 6.0]]]])
      arg1=numpy.array([[[[0.0, 3.0], [-2.0, -2.0], [1.0, 1.0]], [[0.0, -1.0], [-2.0, 4.0], [-2.0, -3.0]]], [[[-7.0, -2.0], [-2.0, -3.0], [3.0, 7.0]], [[1.0, 7.0], [5.0, -5.0], [0.0, 2.0]]], [[[1.0, -3.0], [4.0, 3.0], [2.0, -5.0]], [[7.0, -4.0], [6.0, 4.0], [-1.0, -2.0]]], [[[3.0, 3.0], [7.0, -2.0], [-7.0, 6.0]], [[6.0, -7.0], [-5.0, -7.0], [-4.0, -4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(95.0)+(1.-msk_ref)*numpy.array(-71.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-2.0)+(1-msk_arg0)*(-4.0)
      arg1=numpy.array([-1.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([2.0, -0.0])+(1.-msk_ref)*numpy.array([4.0, -0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, 5.0, 2.0])+(1.-msk_arg0)*numpy.array([5.0, 2.0, -2.0])
      arg1=numpy.array([[-5.0, -2.0], [6.0, -7.0], [-2.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([11.0, -31.0])+(1.-msk_ref)*numpy.array([-9.0, -34.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 5.0, -2.0], [-4.0, 2.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-7.0, -5.0, 2.0], [-2.0, 2.0, 2.0]])
      arg1=numpy.array([[[2.0, 7.0], [1.0, -1.0], [-6.0, -4.0]], [[-1.0, 0.0], [-6.0, 6.0], [7.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, 36.0])+(1.-msk_ref)*numpy.array([-27.0, -40.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, 2.0, -3.0], [3.0, -7.0, -2.0], [6.0, -5.0, -2.0], [-1.0, -5.0, -1.0]], [[-7.0, -2.0, -4.0], [5.0, -5.0, -2.0], [4.0, -3.0, 2.0], [2.0, 0.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -5.0, 7.0], [5.0, 0.0, -6.0], [-1.0, 2.0, -5.0], [-5.0, 1.0, 0.0]], [[-3.0, 0.0, -2.0], [-3.0, 5.0, 0.0], [5.0, -2.0, 0.0], [0.0, 5.0, 2.0]]])
      arg1=numpy.array([[[[-2.0, 7.0], [4.0, -1.0], [-7.0, 1.0]], [[-3.0, 0.0], [-2.0, -2.0], [-4.0, -1.0]], [[6.0, 6.0], [-6.0, -4.0], [-2.0, 6.0]], [[0.0, 7.0], [4.0, -2.0], [7.0, 1.0]]], [[[-3.0, -1.0], [-6.0, -6.0], [4.0, 6.0]], [[0.0, -1.0], [0.0, -1.0], [5.0, 0.0]], [[1.0, -4.0], [1.0, 2.0], [-4.0, 5.0]], [[1.0, -3.0], [7.0, 0.0], [0.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([75.0, 100.0])+(1.-msk_ref)*numpy.array([-29.0, -76.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(-5.0)
      arg1=numpy.array([[3.0, -3.0, 2.0, 1.0, -7.0], [2.0, 0.0, -1.0, 5.0, 6.0], [-6.0, 6.0, 7.0, 0.0, -3.0], [-5.0, 7.0, -6.0, 2.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -0.0, 0.0, 0.0, -0.0], [0.0, 0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0, 0.0]])+(1.-msk_ref)*numpy.array([[-15.0, 15.0, -10.0, -5.0, 35.0], [-10.0, -0.0, 5.0, -25.0, -30.0], [30.0, -30.0, -35.0, -0.0, 15.0], [25.0, -35.0, 30.0, -10.0, -0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, 0.0, 5.0])+(1.-msk_arg0)*numpy.array([3.0, 3.0, 0.0])
      arg1=numpy.array([[[7.0, 5.0, -7.0, -6.0, -1.0], [-6.0, 4.0, -5.0, 3.0, -2.0], [-5.0, 2.0, -2.0, -7.0, 3.0], [5.0, -1.0, 5.0, -2.0, 3.0]], [[7.0, -6.0, 7.0, -4.0, -7.0], [-6.0, 6.0, 6.0, 3.0, 4.0], [-4.0, -7.0, 4.0, 3.0, -1.0], [4.0, -2.0, -4.0, 6.0, 3.0]], [[3.0, -5.0, -7.0, 3.0, -5.0], [0.0, 5.0, 7.0, 2.0, -1.0], [-5.0, 4.0, -2.0, 0.0, -6.0], [-6.0, 4.0, 3.0, -1.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-13.0, -45.0, -7.0, 39.0, -21.0], [24.0, 9.0, 55.0, -2.0, 3.0], [-5.0, 12.0, -2.0, 28.0, -42.0], [-50.0, 24.0, -5.0, 3.0, 8.0]])+(1.-msk_ref)*numpy.array([[42.0, -3.0, 0.0, -30.0, -24.0], [-36.0, 30.0, 3.0, 18.0, 6.0], [-27.0, -15.0, 6.0, -12.0, 6.0], [27.0, -9.0, 3.0, 12.0, 18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 3.0, -7.0], [-3.0, 4.0, -4.0]])+(1.-msk_arg0)*numpy.array([[-4.0, -4.0, -3.0], [4.0, 1.0, 2.0]])
      arg1=numpy.array([[[[-3.0, -6.0, -6.0, 0.0, 4.0], [-4.0, 1.0, 7.0, 2.0, 4.0], [0.0, 3.0, -3.0, -6.0, -4.0], [1.0, 0.0, 0.0, 4.0, 0.0]], [[0.0, -5.0, -6.0, -4.0, 1.0], [0.0, 0.0, -7.0, -2.0, -6.0], [-6.0, -6.0, -2.0, -4.0, 4.0], [1.0, 3.0, -6.0, 0.0, -2.0]], [[6.0, -5.0, -7.0, -5.0, 5.0], [-7.0, -1.0, 3.0, 2.0, 5.0], [-7.0, -2.0, -2.0, -6.0, 6.0], [5.0, 4.0, 7.0, -5.0, -5.0]]], [[[5.0, 0.0, -1.0, -1.0, 5.0], [6.0, -3.0, -7.0, -7.0, -4.0], [-4.0, -1.0, 1.0, -6.0, -2.0], [-1.0, -5.0, 3.0, 3.0, -6.0]], [[-3.0, 2.0, -6.0, 7.0, 1.0], [1.0, -7.0, 6.0, 5.0, 7.0], [-2.0, -3.0, -6.0, 6.0, -6.0], [-3.0, 6.0, -7.0, 1.0, 1.0]], [[1.0, 0.0, 0.0, -3.0, -2.0], [-1.0, -6.0, 2.0, -3.0, -7.0], [0.0, 5.0, -3.0, -4.0, 6.0], [0.0, 6.0, 6.0, 4.0, -6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-64.0, 46.0, 28.0, 66.0, -47.0], [51.0, 9.0, -26.0, 27.0, 3.0], [35.0, -42.0, 2.0, 106.0, -60.0], [-44.0, -4.0, -128.0, 2.0, 75.0]])+(1.-msk_ref)*numpy.array([[13.0, 61.0, 59.0, 28.0, -18.0], [60.0, -32.0, -27.0, -35.0, -30.0], [27.0, 21.0, 18.0, 32.0, -20.0], [-30.0, -26.0, 20.0, 20.0, -12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(7.0)+(1-msk_arg0)*(6.0)
      arg1=numpy.array([[[-5.0, 5.0], [3.0, 0.0]], [[-2.0, -2.0], [6.0, 3.0]], [[0.0, 1.0], [4.0, -4.0]], [[1.0, -4.0], [7.0, 0.0]], [[5.0, 4.0], [-2.0, -6.0]], [[6.0, 2.0], [2.0, 3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-35.0, 35.0], [21.0, 0.0]], [[-14.0, -14.0], [42.0, 21.0]], [[0.0, 7.0], [28.0, -28.0]], [[7.0, -28.0], [49.0, 0.0]], [[35.0, 28.0], [-14.0, -42.0]], [[42.0, 14.0], [14.0, 21.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, 30.0], [18.0, 0.0]], [[-12.0, -12.0], [36.0, 18.0]], [[0.0, 6.0], [24.0, -24.0]], [[6.0, -24.0], [42.0, 0.0]], [[30.0, 24.0], [-12.0, -36.0]], [[36.0, 12.0], [12.0, 18.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, 2.0, -1.0])+(1.-msk_arg0)*numpy.array([6.0, 6.0, 7.0])
      arg1=numpy.array([[[[-3.0, 2.0], [3.0, -6.0]], [[-5.0, 6.0], [2.0, 7.0]], [[-6.0, 1.0], [1.0, -5.0]], [[-3.0, -3.0], [7.0, -4.0]], [[-1.0, -5.0], [-3.0, 1.0]], [[-3.0, -7.0], [7.0, 0.0]]], [[[5.0, 7.0], [4.0, -2.0]], [[-4.0, 7.0], [0.0, 6.0]], [[1.0, 3.0], [0.0, -2.0]], [[6.0, -5.0], [-4.0, 0.0]], [[-1.0, -7.0], [0.0, 2.0]], [[6.0, -4.0], [-1.0, -5.0]]], [[[2.0, 7.0], [7.0, 1.0]], [[-1.0, -5.0], [4.0, 6.0]], [[5.0, -7.0], [-5.0, 7.0]], [[4.0, -2.0], [-1.0, 0.0]], [[7.0, -6.0], [2.0, -4.0]], [[-5.0, -7.0], [0.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[17.0, 1.0], [-8.0, 13.0]], [[8.0, 1.0], [-10.0, -15.0]], [[15.0, 10.0], [2.0, 4.0]], [[17.0, 1.0], [-28.0, 12.0]], [[-6.0, 7.0], [7.0, 5.0]], [[26.0, 20.0], [-23.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[26.0, 103.0], [91.0, -41.0]], [[-61.0, 43.0], [40.0, 120.0]], [[5.0, -25.0], [-29.0, 7.0]], [[46.0, -62.0], [11.0, -24.0]], [[37.0, -114.0], [-4.0, -10.0]], [[-17.0, -115.0], [36.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_array_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(0.0)
      arg1=numpy.array([[[[-2.0, -1.0, -2.0, -6.0], [4.0, -1.0, 6.0, -1.0], [-4.0, -2.0, -5.0, 4.0]], [[-1.0, -1.0, -2.0, 6.0], [5.0, 4.0, -6.0, -3.0], [-2.0, -1.0, 4.0, -7.0]]], [[[0.0, 3.0, -2.0, 2.0], [-6.0, 0.0, 1.0, -4.0], [-7.0, -7.0, -5.0, 2.0]], [[6.0, -3.0, 5.0, 6.0], [-5.0, -3.0, 0.0, -2.0], [-3.0, -5.0, 0.0, 0.0]]], [[[-6.0, 3.0, -5.0, 5.0], [7.0, 6.0, 5.0, 4.0], [0.0, -5.0, -1.0, 0.0]], [[-6.0, 5.0, 3.0, -7.0], [-5.0, -2.0, 0.0, 2.0], [-1.0, 6.0, -1.0, -5.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 5.0, 10.0, 30.0], [-20.0, 5.0, -30.0, 5.0], [20.0, 10.0, 25.0, -20.0]], [[5.0, 5.0, 10.0, -30.0], [-25.0, -20.0, 30.0, 15.0], [10.0, 5.0, -20.0, 35.0]]], [[[-0.0, -15.0, 10.0, -10.0], [30.0, -0.0, -5.0, 20.0], [35.0, 35.0, 25.0, -10.0]], [[-30.0, 15.0, -25.0, -30.0], [25.0, 15.0, -0.0, 10.0], [15.0, 25.0, -0.0, -0.0]]], [[[30.0, -15.0, 25.0, -25.0], [-35.0, -30.0, -25.0, -20.0], [-0.0, 25.0, 5.0, -0.0]], [[30.0, -25.0, -15.0, 35.0], [25.0, 10.0, -0.0, -10.0], [5.0, -30.0, 5.0, 25.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, -0.0, -0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0]], [[-0.0, -0.0, -0.0, 0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, -0.0]]], [[[0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0]], [[0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, 0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(4.0)+(1-msk_arg0)*(3.0)
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-4.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-16.0)+(1.-msk_ref)*numpy.array(-12.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, -6.0, 0.0])+(1.-msk_arg0)*numpy.array([7.0, -3.0, 4.0])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-4.0, 1.0, 0.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(10.0)+(1.-msk_ref)*numpy.array(-31.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 0.0, 6.0], [0.0, 3.0, -1.0]])+(1.-msk_arg0)*numpy.array([[0.0, -7.0, 2.0], [5.0, -2.0, -1.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-1.0, 3.0, -2.0], [-6.0, -7.0, 6.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-36.0)+(1.-msk_ref)*numpy.array(-47.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 4.0, -3.0], [-1.0, 0.0, 4.0], [-5.0, 0.0, -4.0], [0.0, -1.0, 0.0]], [[-1.0, 5.0, 5.0], [1.0, -6.0, 0.0], [0.0, 0.0, -7.0], [4.0, 3.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 5.0, -6.0], [5.0, -7.0, -1.0], [0.0, -6.0, -5.0], [0.0, -3.0, 5.0]], [[-1.0, 0.0, 7.0], [-7.0, 5.0, -3.0], [-1.0, -2.0, 6.0], [4.0, -7.0, -2.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[4.0, 4.0, 3.0], [2.0, 3.0, 1.0], [4.0, -7.0, -7.0], [0.0, -2.0, -4.0]], [[-7.0, -3.0, -3.0], [-5.0, 4.0, 0.0], [-7.0, -4.0, -1.0], [3.0, -1.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(3.0)+(1.-msk_ref)*numpy.array(158.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, 3.0], [5.0, -5.0], [-7.0, -4.0]], [[-1.0, 1.0], [4.0, -5.0], [7.0, 3.0]]], [[[-6.0, -7.0], [-2.0, 0.0], [0.0, -1.0]], [[4.0, -1.0], [-6.0, 4.0], [-3.0, -5.0]]], [[[0.0, -1.0], [-5.0, -7.0], [4.0, 7.0]], [[1.0, -1.0], [-4.0, 1.0], [5.0, -1.0]]], [[[5.0, 6.0], [-3.0, 4.0], [3.0, -5.0]], [[4.0, -3.0], [0.0, -2.0], [4.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, 0.0], [-4.0, -7.0], [0.0, 1.0]], [[-2.0, -7.0], [-7.0, -2.0], [5.0, -2.0]]], [[[-1.0, -4.0], [-1.0, 0.0], [1.0, -4.0]], [[-1.0, 3.0], [5.0, -3.0], [-1.0, 4.0]]], [[[-1.0, -5.0], [0.0, 0.0], [5.0, 2.0]], [[5.0, 0.0], [-5.0, 6.0], [0.0, 0.0]]], [[[1.0, 1.0], [4.0, 6.0], [0.0, -3.0]], [[0.0, 5.0], [7.0, 0.0], [0.0, -7.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[-3.0, 6.0], [5.0, -6.0], [-4.0, -4.0]], [[3.0, -6.0], [0.0, -5.0], [-4.0, -3.0]]], [[[5.0, -4.0], [-7.0, 0.0], [0.0, -5.0]], [[1.0, 5.0], [1.0, -2.0], [-3.0, -4.0]]], [[[-6.0, 2.0], [-4.0, -7.0], [-2.0, 3.0]], [[-4.0, -2.0], [-5.0, 5.0], [-4.0, -1.0]]], [[[-6.0, -1.0], [-2.0, 0.0], [-6.0, -3.0]], [[0.0, -6.0], [0.0, -7.0], [-4.0, -7.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(157.0)+(1.-msk_ref)*numpy.array(155.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(3.0)+(1-msk_arg0)*(-1.0)
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([2.0, 2.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 6.0])+(1.-msk_ref)*numpy.array([-2.0, -2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, 3.0, 0.0])+(1.-msk_arg0)*numpy.array([0.0, -1.0, 0.0])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[3.0, 4.0], [-6.0, 1.0], [2.0, -2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-27.0, -9.0])+(1.-msk_ref)*numpy.array([6.0, -1.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, -7.0, -1.0], [2.0, 5.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-4.0, -4.0, -1.0], [1.0, -7.0, 1.0]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-4.0, 0.0], [-2.0, -5.0], [2.0, 7.0]], [[-2.0, 4.0], [-5.0, 3.0], [-1.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-44.0, 51.0])+(1.-msk_ref)*numpy.array([54.0, -4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, -2.0, 7.0], [7.0, 7.0, 2.0], [6.0, 5.0, 3.0], [-1.0, -7.0, 1.0]], [[2.0, -5.0, -7.0], [-6.0, -2.0, -6.0], [4.0, -2.0, -6.0], [-2.0, -2.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, -5.0, 0.0], [-3.0, 1.0, -7.0], [-7.0, 0.0, 5.0], [-1.0, 0.0, 5.0]], [[2.0, 5.0, 4.0], [3.0, -7.0, 7.0], [4.0, -1.0, 6.0], [7.0, 6.0, -2.0]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[4.0, -2.0], [1.0, 6.0], [-1.0, -6.0]], [[-6.0, 1.0], [-4.0, -1.0], [-4.0, -4.0]], [[-6.0, 4.0], [-7.0, 7.0], [-3.0, 0.0]], [[-5.0, -3.0], [-3.0, -5.0], [5.0, 0.0]]], [[[-1.0, 6.0], [0.0, -4.0], [-7.0, 7.0]], [[-6.0, 5.0], [-6.0, -7.0], [7.0, -1.0]], [[0.0, 0.0], [6.0, -3.0], [4.0, -4.0]], [[-2.0, 0.0], [-3.0, 1.0], [0.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-121.0, 77.0])+(1.-msk_ref)*numpy.array([111.0, 27.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(3.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-4.0, 6.0, -7.0, 3.0, -6.0], [2.0, -2.0, 0.0, 0.0, 3.0], [0.0, -6.0, 5.0, 4.0, -3.0], [0.0, 0.0, 6.0, 7.0, -6.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-0.0, 0.0, -0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0, -0.0]])+(1.-msk_ref)*numpy.array([[-12.0, 18.0, -21.0, 9.0, -18.0], [6.0, -6.0, 0.0, 0.0, 9.0], [0.0, -18.0, 15.0, 12.0, -9.0], [0.0, 0.0, 18.0, 21.0, -18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, -7.0, 0.0])+(1.-msk_arg0)*numpy.array([0.0, 3.0, 7.0])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-2.0, 1.0, 4.0, -7.0, -5.0], [-7.0, 4.0, 5.0, -4.0, 0.0], [6.0, 7.0, 2.0, 2.0, 5.0], [2.0, -4.0, 0.0, -7.0, -3.0]], [[-4.0, 2.0, -1.0, -2.0, -4.0], [-4.0, -1.0, 0.0, 0.0, 0.0], [6.0, 6.0, -2.0, -1.0, -5.0], [0.0, 3.0, -4.0, 0.0, -6.0]], [[4.0, 7.0, -2.0, -4.0, -1.0], [5.0, 0.0, -3.0, -7.0, 1.0], [-7.0, -5.0, 7.0, -5.0, 0.0], [-7.0, 1.0, 0.0, 3.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, -15.0, 3.0, 21.0, 33.0], [35.0, 3.0, -5.0, 4.0, 0.0], [-48.0, -49.0, 12.0, 5.0, 30.0], [-2.0, -17.0, 28.0, 7.0, 45.0]])+(1.-msk_ref)*numpy.array([[16.0, 55.0, -17.0, -34.0, -19.0], [23.0, -3.0, -21.0, -49.0, 7.0], [-31.0, -17.0, 43.0, -38.0, -15.0], [-49.0, 16.0, -12.0, 21.0, 3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, -3.0, 1.0], [2.0, 4.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-4.0, -3.0, 6.0], [-7.0, 1.0, 7.0]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[0.0, -6.0, 3.0, 3.0, 1.0], [-2.0, 7.0, 6.0, -5.0, -5.0], [-5.0, 0.0, -5.0, -5.0, 0.0], [-1.0, -2.0, 0.0, -4.0, 1.0]], [[4.0, -5.0, 0.0, 4.0, -6.0], [-3.0, -1.0, 0.0, -3.0, 3.0], [-5.0, -3.0, -1.0, 0.0, 6.0], [4.0, 3.0, -4.0, -2.0, -5.0]], [[-5.0, 2.0, -5.0, -2.0, 0.0], [2.0, 4.0, 3.0, 5.0, 4.0], [6.0, -2.0, 0.0, -7.0, -6.0], [-5.0, -4.0, 0.0, 1.0, -7.0]]], [[[-6.0, 3.0, 1.0, 2.0, 4.0], [-4.0, 7.0, -1.0, -7.0, -3.0], [3.0, 4.0, 4.0, 6.0, -1.0], [1.0, 1.0, 0.0, -3.0, 0.0]], [[6.0, -6.0, 1.0, 1.0, -6.0], [-5.0, 2.0, 3.0, 0.0, -2.0], [4.0, -4.0, 5.0, 2.0, 3.0], [-5.0, -7.0, 0.0, 0.0, -7.0]], [[-4.0, 2.0, 6.0, -2.0, 6.0], [1.0, 6.0, 3.0, 0.0, -1.0], [0.0, 0.0, -5.0, 7.0, -5.0], [-1.0, 2.0, 2.0, -7.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-25.0, -3.0, 37.0, -10.0, 34.0], [-16.0, 73.0, 40.0, -10.0, -34.0], [33.0, -1.0, -4.0, 38.0, -39.0], [-42.0, -33.0, 22.0, -42.0, -33.0]])+(1.-msk_ref)*numpy.array([[-22.0, 38.0, -6.0, -63.0, 22.0], [59.0, -6.0, 25.0, 108.0, 47.0], [54.0, -35.0, -35.0, -13.0, -79.0], [-57.0, -25.0, 26.0, 0.0, -59.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(0.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-1.0, 2.0], [-4.0, 2.0]], [[-1.0, 3.0], [0.0, 2.0]], [[7.0, -4.0], [-2.0, 4.0]], [[0.0, -6.0], [4.0, -4.0]], [[3.0, 2.0], [0.0, -5.0]], [[6.0, 0.0], [5.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 12.0], [-24.0, 12.0]], [[-6.0, 18.0], [0.0, 12.0]], [[42.0, -24.0], [-12.0, 24.0]], [[0.0, -36.0], [24.0, -24.0]], [[18.0, 12.0], [0.0, -30.0]], [[36.0, 0.0], [30.0, -36.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[0.0, -0.0], [-0.0, 0.0]], [[0.0, -0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, -0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, -4.0, 6.0])+(1.-msk_arg0)*numpy.array([4.0, 5.0, 2.0])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[-7.0, -3.0], [0.0, -1.0]], [[-6.0, -4.0], [-6.0, 5.0]], [[-5.0, 0.0], [6.0, -4.0]], [[7.0, -2.0], [7.0, 3.0]], [[-6.0, -6.0], [-2.0, 0.0]], [[-5.0, 4.0], [-4.0, -5.0]]], [[[-3.0, 6.0], [-3.0, 2.0]], [[0.0, 2.0], [-3.0, 3.0]], [[1.0, 3.0], [5.0, 0.0]], [[-7.0, 5.0], [-7.0, -2.0]], [[5.0, 5.0], [1.0, 3.0]], [[-7.0, -5.0], [-1.0, -1.0]]], [[[-2.0, -1.0], [-3.0, -3.0]], [[0.0, 2.0], [1.0, 5.0]], [[-7.0, -2.0], [7.0, 0.0]], [[3.0, 3.0], [7.0, -1.0]], [[6.0, 4.0], [-6.0, -5.0]], [[1.0, -7.0], [-6.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[49.0, -9.0], [-6.0, -19.0]], [[42.0, 32.0], [60.0, -17.0]], [[-11.0, -24.0], [-20.0, 28.0]], [[-3.0, 12.0], [21.0, -19.0]], [[58.0, 46.0], [-26.0, -42.0]], [[69.0, -50.0], [-4.0, 39.0]]])+(1.-msk_ref)*numpy.array([[[-47.0, 16.0], [-21.0, 0.0]], [[-24.0, -2.0], [-37.0, 45.0]], [[-29.0, 11.0], [63.0, -16.0]], [[-1.0, 23.0], [7.0, 0.0]], [[13.0, 9.0], [-15.0, 5.0]], [[-53.0, -23.0], [-33.0, -25.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_Symbol_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(7.0)+(1-msk_arg0)*(-5.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[4.0, -1.0, -5.0, -5.0], [-4.0, 7.0, -7.0, 0.0], [0.0, 7.0, -2.0, -2.0]], [[2.0, -5.0, -3.0, 3.0], [6.0, -1.0, 4.0, 0.0], [-3.0, -5.0, -7.0, 7.0]]], [[[-3.0, 2.0, 1.0, 1.0], [0.0, 5.0, 2.0, -4.0], [2.0, 6.0, -1.0, -6.0]], [[2.0, -3.0, 5.0, 1.0], [7.0, 0.0, 3.0, -7.0], [1.0, 0.0, -4.0, -4.0]]], [[[-4.0, -5.0, 2.0, -5.0], [5.0, -7.0, 5.0, -6.0], [-6.0, -5.0, 7.0, -4.0]], [[6.0, 1.0, 0.0, -2.0], [0.0, -6.0, -6.0, -3.0], [7.0, 1.0, -7.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[28.0, -7.0, -35.0, -35.0], [-28.0, 49.0, -49.0, 0.0], [0.0, 49.0, -14.0, -14.0]], [[14.0, -35.0, -21.0, 21.0], [42.0, -7.0, 28.0, 0.0], [-21.0, -35.0, -49.0, 49.0]]], [[[-21.0, 14.0, 7.0, 7.0], [0.0, 35.0, 14.0, -28.0], [14.0, 42.0, -7.0, -42.0]], [[14.0, -21.0, 35.0, 7.0], [49.0, 0.0, 21.0, -49.0], [7.0, 0.0, -28.0, -28.0]]], [[[-28.0, -35.0, 14.0, -35.0], [35.0, -49.0, 35.0, -42.0], [-42.0, -35.0, 49.0, -28.0]], [[42.0, 7.0, 0.0, -14.0], [0.0, -42.0, -42.0, -21.0], [49.0, 7.0, -49.0, 49.0]]]])+(1.-msk_ref)*numpy.array([[[[-20.0, 5.0, 25.0, 25.0], [20.0, -35.0, 35.0, -0.0], [-0.0, -35.0, 10.0, 10.0]], [[-10.0, 25.0, 15.0, -15.0], [-30.0, 5.0, -20.0, -0.0], [15.0, 25.0, 35.0, -35.0]]], [[[15.0, -10.0, -5.0, -5.0], [-0.0, -25.0, -10.0, 20.0], [-10.0, -30.0, 5.0, 30.0]], [[-10.0, 15.0, -25.0, -5.0], [-35.0, -0.0, -15.0, 35.0], [-5.0, -0.0, 20.0, 20.0]]], [[[20.0, 25.0, -10.0, 25.0], [-25.0, 35.0, -25.0, 30.0], [30.0, 25.0, -35.0, 20.0]], [[-30.0, -5.0, -0.0, 10.0], [-0.0, 30.0, 30.0, 15.0], [-35.0, -5.0, 35.0, -35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(-5.0)
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(35.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, 3.0, 2.0])+(1.-msk_arg0)*numpy.array([1.0, -4.0, -5.0])
      arg1=Data(numpy.array([3.0, 3.0, -3.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(0.0)+(1.-msk_ref)*numpy.array(6.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, -5.0, -3.0], [3.0, 5.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -1.0, 0.0], [0.0, 1.0, 0.0]])
      arg1=Data(numpy.array([[-3.0, -3.0, 3.0], [0.0, 6.0, -2.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(24.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 7.0, 0.0], [-6.0, 4.0, 5.0], [3.0, -3.0, 3.0], [6.0, -7.0, 7.0]], [[-3.0, 4.0, 2.0], [-5.0, -3.0, -2.0], [3.0, -6.0, -7.0], [4.0, 2.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[6.0, 4.0, 4.0], [-4.0, 1.0, 0.0], [6.0, 0.0, -6.0], [5.0, -4.0, -7.0]], [[-1.0, 4.0, 3.0], [-1.0, -5.0, 4.0], [1.0, 1.0, -6.0], [-7.0, 6.0, 6.0]]])
      arg1=Data(numpy.array([[[-6.0, 0.0, 5.0], [3.0, -4.0, -6.0], [6.0, 0.0, -3.0], [5.0, 2.0, 7.0]], [[-5.0, 0.0, -3.0], [5.0, 0.0, 3.0], [2.0, 4.0, 7.0], [-7.0, -1.0, -3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-103.0)+(1.-msk_ref)*numpy.array(-18.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -7.0], [0.0, -7.0], [-7.0, 0.0]], [[-5.0, 5.0], [2.0, -5.0], [0.0, 1.0]]], [[[4.0, 6.0], [-2.0, 0.0], [-5.0, 5.0]], [[5.0, -5.0], [3.0, -2.0], [-7.0, 1.0]]], [[[1.0, 7.0], [-7.0, 6.0], [0.0, 0.0]], [[4.0, -4.0], [-2.0, 2.0], [-3.0, -7.0]]], [[[5.0, -5.0], [-1.0, -1.0], [0.0, 4.0]], [[1.0, 5.0], [7.0, 0.0], [5.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 3.0], [-4.0, 2.0], [-1.0, -6.0]], [[6.0, 4.0], [5.0, 7.0], [0.0, -5.0]]], [[[7.0, -2.0], [-2.0, 0.0], [-6.0, -7.0]], [[-2.0, 0.0], [7.0, 4.0], [3.0, 0.0]]], [[[0.0, -5.0], [-5.0, 0.0], [5.0, 4.0]], [[-3.0, 6.0], [0.0, 0.0], [-7.0, 5.0]]], [[[-2.0, 3.0], [0.0, 6.0], [-4.0, -4.0]], [[-5.0, 0.0], [-7.0, -3.0], [-5.0, 1.0]]]])
      arg1=Data(numpy.array([[[[-7.0, 6.0], [-4.0, 1.0], [-4.0, 0.0]], [[-4.0, 3.0], [0.0, -6.0], [-1.0, -5.0]]], [[[2.0, -6.0], [-7.0, -6.0], [1.0, -6.0]], [[-1.0, 2.0], [6.0, -1.0], [-1.0, 1.0]]], [[[-6.0, -5.0], [-5.0, -1.0], [-7.0, 6.0]], [[0.0, 0.0], [-6.0, -2.0], [6.0, -2.0]]], [[[-5.0, -4.0], [7.0, 0.0], [-3.0, 3.0]], [[6.0, 7.0], [5.0, -4.0], [6.0, -3.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(62.0)+(1.-msk_ref)*numpy.array(51.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-6.0)+(1-msk_arg0)*(1.0)
      arg1=Data(numpy.array([0.0, -5.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-0.0, 30.0])+(1.-msk_ref)*numpy.array([0.0, -5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, 6.0, -1.0])+(1.-msk_arg0)*numpy.array([4.0, -5.0, -2.0])
      arg1=Data(numpy.array([[0.0, -3.0], [0.0, 1.0], [6.0, -5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-6.0, 14.0])+(1.-msk_ref)*numpy.array([-12.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 1.0, 7.0], [-2.0, 0.0, -3.0]])+(1.-msk_arg0)*numpy.array([[1.0, 3.0, 1.0], [7.0, 6.0, -2.0]])
      arg1=Data(numpy.array([[[-2.0, -6.0], [-5.0, -3.0], [2.0, 1.0]], [[-6.0, -3.0], [3.0, -3.0], [-5.0, -7.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([40.0, 43.0])+(1.-msk_ref)*numpy.array([-29.0, -39.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, -5.0, -3.0], [5.0, 0.0, -7.0], [4.0, 5.0, 0.0], [-2.0, 6.0, 0.0]], [[-4.0, -1.0, -2.0], [-3.0, 2.0, -5.0], [3.0, -6.0, 1.0], [6.0, -3.0, -1.0]]])+(1.-msk_arg0)*numpy.array([[[5.0, -2.0, 0.0], [7.0, 3.0, -3.0], [-5.0, -4.0, 2.0], [6.0, -3.0, -5.0]], [[5.0, -5.0, -4.0], [-5.0, -4.0, 5.0], [5.0, 4.0, 1.0], [0.0, 4.0, 4.0]]])
      arg1=Data(numpy.array([[[[3.0, -3.0], [0.0, -1.0], [-2.0, 6.0]], [[-7.0, 7.0], [-6.0, -3.0], [-2.0, 5.0]], [[-6.0, -7.0], [-3.0, -5.0], [-7.0, 2.0]], [[6.0, -1.0], [0.0, -4.0], [1.0, 1.0]]], [[[-5.0, -4.0], [0.0, 5.0], [0.0, 0.0]], [[0.0, -6.0], [4.0, 4.0], [1.0, 5.0]], [[3.0, 4.0], [-1.0, 2.0], [1.0, 4.0]], [[-3.0, 2.0], [2.0, 7.0], [6.0, 0.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-39.0, -99.0])+(1.-msk_ref)*numpy.array([21.0, 126.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(-6.0)
      arg1=Data(numpy.array([[6.0, 6.0, -5.0, -1.0, -2.0], [-4.0, 5.0, 0.0, -6.0, 6.0], [2.0, 7.0, 4.0, -5.0, 2.0], [1.0, 4.0, -4.0, 0.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 0.0, -0.0, -0.0, -0.0], [-0.0, 0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, 0.0, 0.0]])+(1.-msk_ref)*numpy.array([[-36.0, -36.0, 30.0, 6.0, 12.0], [24.0, -30.0, -0.0, 36.0, -36.0], [-12.0, -42.0, -24.0, 30.0, -12.0], [-6.0, -24.0, 24.0, -0.0, -30.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, 2.0, 5.0])+(1.-msk_arg0)*numpy.array([-7.0, 3.0, 5.0])
      arg1=Data(numpy.array([[[5.0, 5.0, 0.0, 3.0, 1.0], [6.0, -5.0, -7.0, -5.0, -5.0], [-1.0, -2.0, 5.0, 6.0, -6.0], [1.0, -7.0, 4.0, -1.0, 2.0]], [[3.0, 1.0, 7.0, -2.0, 1.0], [6.0, 3.0, 0.0, -1.0, 2.0], [0.0, -1.0, -3.0, -3.0, 5.0], [5.0, -4.0, -2.0, 5.0, -5.0]], [[2.0, -1.0, -6.0, 6.0, -4.0], [3.0, 0.0, -7.0, -2.0, 4.0], [-1.0, 3.0, 4.0, -3.0, -7.0], [-7.0, -3.0, -1.0, 2.0, 5.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[51.0, 32.0, -16.0, 47.0, -11.0], [69.0, -29.0, -84.0, -47.0, -11.0], [-12.0, -1.0, 49.0, 21.0, -67.0], [-18.0, -72.0, 19.0, 13.0, 29.0]])+(1.-msk_ref)*numpy.array([[-16.0, -37.0, -9.0, 3.0, -24.0], [-9.0, 44.0, 14.0, 22.0, 61.0], [2.0, 26.0, -24.0, -66.0, 22.0], [-27.0, 22.0, -39.0, 32.0, -4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 5.0, 7.0], [-5.0, -7.0, -1.0]])+(1.-msk_arg0)*numpy.array([[7.0, -4.0, 1.0], [-1.0, -4.0, -1.0]])
      arg1=Data(numpy.array([[[[-2.0, 0.0, 1.0, 3.0, -4.0], [0.0, -7.0, -5.0, -1.0, -5.0], [-6.0, -2.0, -1.0, 7.0, 1.0], [2.0, -5.0, 7.0, 4.0, 6.0]], [[6.0, 3.0, 4.0, -6.0, 4.0], [-4.0, 1.0, -6.0, -6.0, -1.0], [-6.0, 1.0, 0.0, 2.0, 0.0], [0.0, -3.0, 0.0, 7.0, 3.0]], [[-4.0, -6.0, 3.0, -2.0, 4.0], [2.0, 4.0, 3.0, -3.0, -1.0], [-1.0, -2.0, 7.0, -7.0, 2.0], [0.0, -5.0, -6.0, -6.0, -1.0]]], [[[0.0, 0.0, 7.0, -5.0, 7.0], [3.0, -1.0, -4.0, 0.0, -5.0], [7.0, -7.0, -6.0, 2.0, 7.0], [-1.0, 7.0, -7.0, 2.0, -4.0]], [[7.0, -2.0, 2.0, 5.0, -7.0], [-7.0, -1.0, -1.0, 2.0, 3.0], [-6.0, 4.0, -2.0, 1.0, 2.0], [-2.0, -7.0, 7.0, 1.0, -1.0]], [[-7.0, 0.0, -4.0, -4.0, 1.0], [-3.0, -7.0, -6.0, -5.0, -1.0], [-7.0, 0.0, -4.0, 7.0, 7.0], [7.0, 4.0, 5.0, -4.0, -5.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-44.0, -13.0, -2.0, -44.0, 53.0], [31.0, 38.0, 14.0, -62.0, -17.0], [-35.0, -6.0, 95.0, -49.0, -40.0], [16.0, -50.0, -47.0, -12.0, 52.0]])+(1.-msk_ref)*numpy.array([[-63.0, -10.0, -17.0, 32.0, -20.0], [46.0, -37.0, 6.0, 11.0, -38.0], [5.0, -29.0, 18.0, 21.0, -13.0], [16.0, -11.0, 17.0, -8.0, 42.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(3.0)
      arg1=Data(numpy.array([[[-6.0, -3.0], [2.0, -2.0]], [[1.0, 6.0], [4.0, 0.0]], [[-7.0, 7.0], [1.0, 4.0]], [[-3.0, 1.0], [-2.0, -3.0]], [[7.0, -6.0], [-6.0, 2.0]], [[5.0, 0.0], [-3.0, 0.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-36.0, -18.0], [12.0, -12.0]], [[6.0, 36.0], [24.0, 0.0]], [[-42.0, 42.0], [6.0, 24.0]], [[-18.0, 6.0], [-12.0, -18.0]], [[42.0, -36.0], [-36.0, 12.0]], [[30.0, 0.0], [-18.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, -9.0], [6.0, -6.0]], [[3.0, 18.0], [12.0, 0.0]], [[-21.0, 21.0], [3.0, 12.0]], [[-9.0, 3.0], [-6.0, -9.0]], [[21.0, -18.0], [-18.0, 6.0]], [[15.0, 0.0], [-9.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([4.0, -5.0, -5.0])+(1.-msk_arg0)*numpy.array([-5.0, -7.0, 0.0])
      arg1=Data(numpy.array([[[[1.0, -1.0], [-7.0, -4.0]], [[-6.0, 6.0], [4.0, 2.0]], [[-5.0, -1.0], [6.0, -5.0]], [[0.0, -5.0], [-2.0, -3.0]], [[-6.0, 0.0], [5.0, 6.0]], [[-1.0, -7.0], [7.0, -2.0]]], [[[-4.0, 0.0], [2.0, -6.0]], [[5.0, -2.0], [1.0, 7.0]], [[3.0, -7.0], [3.0, 2.0]], [[-2.0, 0.0], [3.0, -6.0]], [[7.0, 3.0], [-7.0, 5.0]], [[-7.0, -6.0], [-4.0, 3.0]]], [[[-4.0, 7.0], [0.0, -7.0]], [[-6.0, -4.0], [-2.0, -2.0]], [[0.0, 5.0], [-4.0, 6.0]], [[6.0, 1.0], [-7.0, 3.0]], [[3.0, 3.0], [-4.0, -2.0]], [[-5.0, 6.0], [-5.0, -5.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[44.0, -39.0], [-38.0, 49.0]], [[-19.0, 54.0], [21.0, -17.0]], [[-35.0, 6.0], [29.0, -60.0]], [[-20.0, -25.0], [12.0, 3.0]], [[-74.0, -30.0], [75.0, 9.0]], [[56.0, -28.0], [73.0, 2.0]]])+(1.-msk_ref)*numpy.array([[[23.0, 5.0], [21.0, 62.0]], [[-5.0, -16.0], [-27.0, -59.0]], [[4.0, 54.0], [-51.0, 11.0]], [[14.0, 25.0], [-11.0, 57.0]], [[-19.0, -21.0], [24.0, -65.0]], [[54.0, 77.0], [-7.0, -11.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_constData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(2.0)+(1-msk_arg0)*(3.0)
      arg1=Data(numpy.array([[[[5.0, 1.0, -5.0, -7.0], [7.0, 4.0, 7.0, 2.0], [-4.0, 5.0, -3.0, 2.0]], [[5.0, 1.0, -6.0, -1.0], [-4.0, -6.0, -5.0, 0.0], [4.0, 2.0, 7.0, 0.0]]], [[[-6.0, 6.0, 0.0, 0.0], [5.0, 7.0, 6.0, -3.0], [-3.0, -6.0, 3.0, -1.0]], [[-2.0, 6.0, 7.0, -5.0], [-2.0, -7.0, 1.0, 5.0], [-5.0, 6.0, 3.0, 0.0]]], [[[1.0, 2.0, -6.0, 7.0], [2.0, 6.0, -5.0, -4.0], [-2.0, -7.0, 7.0, 5.0]], [[-2.0, -2.0, 3.0, 3.0], [4.0, 5.0, -4.0, -3.0], [2.0, -3.0, 5.0, -6.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 2.0, -10.0, -14.0], [14.0, 8.0, 14.0, 4.0], [-8.0, 10.0, -6.0, 4.0]], [[10.0, 2.0, -12.0, -2.0], [-8.0, -12.0, -10.0, 0.0], [8.0, 4.0, 14.0, 0.0]]], [[[-12.0, 12.0, 0.0, 0.0], [10.0, 14.0, 12.0, -6.0], [-6.0, -12.0, 6.0, -2.0]], [[-4.0, 12.0, 14.0, -10.0], [-4.0, -14.0, 2.0, 10.0], [-10.0, 12.0, 6.0, 0.0]]], [[[2.0, 4.0, -12.0, 14.0], [4.0, 12.0, -10.0, -8.0], [-4.0, -14.0, 14.0, 10.0]], [[-4.0, -4.0, 6.0, 6.0], [8.0, 10.0, -8.0, -6.0], [4.0, -6.0, 10.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[15.0, 3.0, -15.0, -21.0], [21.0, 12.0, 21.0, 6.0], [-12.0, 15.0, -9.0, 6.0]], [[15.0, 3.0, -18.0, -3.0], [-12.0, -18.0, -15.0, 0.0], [12.0, 6.0, 21.0, 0.0]]], [[[-18.0, 18.0, 0.0, 0.0], [15.0, 21.0, 18.0, -9.0], [-9.0, -18.0, 9.0, -3.0]], [[-6.0, 18.0, 21.0, -15.0], [-6.0, -21.0, 3.0, 15.0], [-15.0, 18.0, 9.0, 0.0]]], [[[3.0, 6.0, -18.0, 21.0], [6.0, 18.0, -15.0, -12.0], [-6.0, -21.0, 21.0, 15.0]], [[-6.0, -6.0, 9.0, 9.0], [12.0, 15.0, -12.0, -9.0], [6.0, -9.0, 15.0, -18.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-3.0)+(1-msk_arg0)*(2.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(5.0)+(1-msk_arg1)*(2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-15.0)+(1.-msk_ref)*numpy.array(4.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, 7.0, 0.0])+(1.-msk_arg0)*numpy.array([-5.0, -6.0, 6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, 0.0, -6.0])+(1.-msk_arg1)*numpy.array([-2.0, -2.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-25.0)+(1.-msk_ref)*numpy.array(22.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, -6.0, 6.0], [0.0, 7.0, -5.0]])+(1.-msk_arg0)*numpy.array([[0.0, -7.0, 2.0], [-1.0, 5.0, -2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, -3.0, -1.0], [-7.0, 0.0, 0.0]])+(1.-msk_arg1)*numpy.array([[6.0, 3.0, 6.0], [1.0, 5.0, 1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(47.0)+(1.-msk_ref)*numpy.array(13.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 4.0, 1.0], [0.0, 0.0, 2.0], [0.0, -4.0, 2.0], [1.0, -2.0, -4.0]], [[-7.0, 6.0, 7.0], [-7.0, -3.0, 6.0], [1.0, 5.0, -4.0], [-4.0, 5.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, -7.0, -3.0], [4.0, 5.0, 0.0], [-2.0, -1.0, 7.0], [2.0, 5.0, 5.0]], [[7.0, 4.0, 2.0], [-3.0, -7.0, -4.0], [-5.0, -2.0, 4.0], [2.0, 3.0, 2.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, -2.0, -2.0], [2.0, 3.0, 1.0], [2.0, -7.0, 3.0], [-1.0, 3.0, -7.0]], [[-1.0, 0.0, -5.0], [5.0, -3.0, 5.0], [-6.0, -6.0, -3.0], [-4.0, -3.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 0.0, -3.0], [6.0, 0.0, -7.0], [-2.0, 1.0, 2.0], [-3.0, -7.0, 0.0]], [[-1.0, 5.0, 5.0], [3.0, -3.0, -4.0], [-2.0, 4.0, 0.0], [-5.0, -7.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-17.0)+(1.-msk_ref)*numpy.array(32.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, -7.0], [-7.0, 3.0], [3.0, -2.0]], [[-7.0, 4.0], [1.0, -3.0], [6.0, -7.0]]], [[[-4.0, 0.0], [-4.0, 5.0], [-4.0, -4.0]], [[0.0, 6.0], [5.0, 2.0], [1.0, 0.0]]], [[[0.0, 2.0], [-5.0, -7.0], [-5.0, 0.0]], [[-2.0, -4.0], [0.0, 1.0], [0.0, -7.0]]], [[[-6.0, 0.0], [-4.0, -3.0], [7.0, 3.0]], [[-5.0, -3.0], [-3.0, 2.0], [-3.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 5.0], [-4.0, -1.0], [4.0, 0.0]], [[-2.0, 7.0], [-4.0, -5.0], [-5.0, 7.0]]], [[[-5.0, 3.0], [-7.0, -6.0], [-6.0, -4.0]], [[6.0, -3.0], [1.0, -2.0], [1.0, 2.0]]], [[[-1.0, -4.0], [6.0, -2.0], [1.0, 4.0]], [[0.0, 0.0], [-2.0, -6.0], [4.0, -7.0]]], [[[-3.0, -7.0], [0.0, -2.0], [-6.0, -1.0]], [[0.0, -3.0], [4.0, 1.0], [-1.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, -5.0], [-7.0, 1.0], [-4.0, -6.0]], [[-6.0, -3.0], [-2.0, -2.0], [5.0, -7.0]]], [[[0.0, 0.0], [0.0, 0.0], [2.0, 2.0]], [[4.0, -6.0], [-1.0, -7.0], [4.0, -7.0]]], [[[3.0, -1.0], [5.0, -3.0], [-7.0, -3.0]], [[-4.0, -4.0], [0.0, 2.0], [-5.0, 0.0]]], [[[2.0, 1.0], [-1.0, -6.0], [0.0, 3.0]], [[6.0, 0.0], [-4.0, -4.0], [-3.0, 2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, 0.0], [-1.0, 0.0], [-5.0, 3.0]], [[-5.0, -2.0], [1.0, 6.0], [2.0, 6.0]]], [[[-5.0, -5.0], [6.0, 6.0], [3.0, -4.0]], [[6.0, 6.0], [3.0, 6.0], [-7.0, -2.0]]], [[[5.0, -7.0], [-3.0, 7.0], [-2.0, 7.0]], [[-7.0, 7.0], [7.0, 2.0], [7.0, 4.0]]], [[[-5.0, 5.0], [-4.0, -2.0], [0.0, -5.0]], [[3.0, 7.0], [5.0, -4.0], [-3.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(207.0)+(1.-msk_ref)*numpy.array(-110.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(0.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, 1.0])+(1.-msk_arg1)*numpy.array([1.0, 3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-18.0, 6.0])+(1.-msk_ref)*numpy.array([0.0, 0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, -7.0, -1.0])+(1.-msk_arg0)*numpy.array([2.0, 3.0, 6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 1.0], [-5.0, 2.0], [-4.0, 0.0]])+(1.-msk_arg1)*numpy.array([[5.0, 2.0], [0.0, -7.0], [-4.0, -6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([24.0, -11.0])+(1.-msk_ref)*numpy.array([-14.0, -53.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 1.0, 0.0], [-2.0, 4.0, -5.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -1.0, -1.0], [-3.0, 3.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 5.0], [-7.0, 1.0], [-4.0, 0.0]], [[1.0, -3.0], [2.0, 4.0], [2.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, 7.0], [0.0, 2.0], [5.0, 7.0]], [[1.0, -7.0], [-4.0, 3.0], [-1.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([9.0, 73.0])+(1.-msk_ref)*numpy.array([13.0, -14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-6.0, -4.0, 6.0], [1.0, -2.0, -5.0], [4.0, 4.0, 7.0], [2.0, -6.0, -5.0]], [[2.0, -1.0, -2.0], [2.0, 3.0, 3.0], [7.0, 2.0, -6.0], [5.0, -2.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, 3.0, 5.0], [3.0, 6.0, -5.0], [-4.0, -2.0, -5.0], [-1.0, 0.0, 6.0]], [[-2.0, 5.0, 5.0], [-4.0, 3.0, 5.0], [-6.0, 0.0, 7.0], [5.0, 3.0, 6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -7.0], [6.0, -1.0], [-1.0, -5.0]], [[5.0, 1.0], [4.0, 6.0], [-5.0, 4.0]], [[-1.0, 0.0], [2.0, -7.0], [0.0, 1.0]], [[5.0, 0.0], [0.0, -3.0], [-7.0, 1.0]]], [[[2.0, 0.0], [-5.0, 3.0], [3.0, 0.0]], [[2.0, -1.0], [-5.0, 6.0], [-4.0, 0.0]], [[-2.0, -3.0], [-4.0, -7.0], [0.0, -4.0]], [[0.0, 0.0], [2.0, 2.0], [-1.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, 5.0], [5.0, 0.0], [-2.0, 0.0]], [[-4.0, -4.0], [0.0, -1.0], [-6.0, 5.0]], [[6.0, 7.0], [-7.0, 2.0], [7.0, 0.0]], [[7.0, -3.0], [-5.0, -3.0], [3.0, -3.0]]], [[[-6.0, -3.0], [5.0, -6.0], [3.0, -7.0]], [[3.0, -6.0], [-2.0, -2.0], [-4.0, 5.0]], [[4.0, -6.0], [1.0, -3.0], [6.0, 0.0]], [[-2.0, 6.0], [-6.0, -7.0], [-7.0, 7.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([22.0, -43.0])+(1.-msk_ref)*numpy.array([-42.0, -14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(7.0)+(1-msk_arg0)*(4.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, -5.0, 0.0, 2.0, 3.0], [-7.0, -2.0, 5.0, -1.0, 5.0], [-4.0, 2.0, 6.0, 2.0, 2.0], [0.0, -7.0, 0.0, -2.0, -7.0]])+(1.-msk_arg1)*numpy.array([[5.0, 0.0, 0.0, -4.0, -4.0], [0.0, 6.0, -7.0, 6.0, 1.0], [0.0, 0.0, -4.0, -4.0, -6.0], [0.0, -7.0, 5.0, -1.0, -3.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-14.0, -35.0, 0.0, 14.0, 21.0], [-49.0, -14.0, 35.0, -7.0, 35.0], [-28.0, 14.0, 42.0, 14.0, 14.0], [0.0, -49.0, 0.0, -14.0, -49.0]])+(1.-msk_ref)*numpy.array([[20.0, 0.0, 0.0, -16.0, -16.0], [0.0, 24.0, -28.0, 24.0, 4.0], [0.0, 0.0, -16.0, -16.0, -24.0], [0.0, -28.0, 20.0, -4.0, -12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, 0.0, 7.0])+(1.-msk_arg0)*numpy.array([5.0, -5.0, -5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 3.0, 7.0, 1.0, -7.0], [-5.0, -2.0, 2.0, -5.0, 0.0], [-1.0, 5.0, -1.0, 2.0, 2.0], [0.0, -4.0, 5.0, -5.0, -6.0]], [[-2.0, -6.0, 0.0, -2.0, -1.0], [0.0, 3.0, 3.0, 2.0, -5.0], [5.0, -1.0, -6.0, 1.0, -1.0], [-2.0, -5.0, 1.0, -7.0, 4.0]], [[0.0, -7.0, 2.0, 0.0, 2.0], [-1.0, -3.0, -2.0, 0.0, 2.0], [-5.0, -4.0, 5.0, -4.0, -5.0], [7.0, -1.0, 4.0, -2.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 2.0, -5.0, -5.0, 7.0], [5.0, 0.0, 0.0, 2.0, -2.0], [3.0, -1.0, 7.0, -4.0, 2.0], [1.0, -1.0, 5.0, 0.0, -2.0]], [[0.0, 0.0, 5.0, -3.0, 7.0], [-4.0, -7.0, -3.0, 1.0, 5.0], [1.0, 6.0, -4.0, 0.0, -2.0], [-5.0, 0.0, 7.0, 0.0, 0.0]], [[1.0, -7.0, -1.0, 2.0, 3.0], [-1.0, 1.0, -4.0, 5.0, 4.0], [5.0, -7.0, 0.0, -6.0, 0.0], [-2.0, -6.0, -4.0, -3.0, 2.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, -67.0, -28.0, -6.0, 56.0], [23.0, -9.0, -26.0, 30.0, 14.0], [-29.0, -58.0, 41.0, -40.0, -47.0], [49.0, 17.0, -2.0, 16.0, 71.0]])+(1.-msk_ref)*numpy.array([[10.0, 45.0, -45.0, -20.0, -15.0], [50.0, 30.0, 35.0, -20.0, -55.0], [-15.0, 0.0, 55.0, 10.0, 20.0], [40.0, 25.0, 10.0, 15.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, 2.0, 5.0], [-7.0, -6.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 3.0, -1.0], [2.0, 2.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 2.0, -3.0, 4.0, -1.0], [-7.0, -7.0, -5.0, 3.0, 4.0], [7.0, -6.0, -2.0, -2.0, -3.0], [1.0, 6.0, -7.0, -6.0, -6.0]], [[-6.0, -7.0, -7.0, 6.0, 0.0], [0.0, 2.0, 1.0, -4.0, -1.0], [3.0, -5.0, -6.0, 3.0, 7.0], [-3.0, -7.0, -7.0, 1.0, -7.0]], [[-4.0, 5.0, 6.0, 4.0, -7.0], [3.0, 6.0, 5.0, -6.0, 0.0], [0.0, 0.0, -1.0, 7.0, 2.0], [-6.0, 7.0, -6.0, 2.0, 6.0]]], [[[1.0, -1.0, -5.0, -4.0, -3.0], [0.0, 1.0, -7.0, 7.0, 7.0], [-7.0, 5.0, -1.0, -6.0, -1.0], [6.0, -6.0, -7.0, -1.0, 7.0]], [[-4.0, 3.0, 5.0, -3.0, 0.0], [-5.0, 2.0, 0.0, -4.0, -6.0], [-7.0, -1.0, 0.0, 0.0, 0.0], [4.0, 6.0, 4.0, 0.0, -7.0]], [[7.0, 5.0, -3.0, -6.0, 1.0], [-5.0, -5.0, -6.0, 4.0, 0.0], [-3.0, 3.0, 4.0, 1.0, 1.0], [2.0, -1.0, 5.0, 4.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, 5.0, -7.0, -5.0, 2.0], [2.0, 3.0, -5.0, -2.0, 5.0], [4.0, -4.0, 2.0, 0.0, 7.0], [-2.0, 6.0, -3.0, -3.0, 4.0]], [[5.0, -2.0, -1.0, -4.0, 3.0], [0.0, -1.0, 6.0, -2.0, -6.0], [3.0, 4.0, 5.0, 0.0, 1.0], [-2.0, 3.0, 4.0, -5.0, -1.0]], [[0.0, 5.0, -6.0, -6.0, -4.0], [2.0, 7.0, -1.0, 1.0, 5.0], [-1.0, -3.0, -3.0, -6.0, 4.0], [2.0, 5.0, 0.0, 5.0, 0.0]]], [[[2.0, 0.0, 3.0, -3.0, -5.0], [-5.0, 0.0, 1.0, -3.0, -6.0], [6.0, 0.0, -3.0, 2.0, 0.0], [-7.0, 4.0, 0.0, -4.0, -1.0]], [[1.0, 7.0, -2.0, 1.0, -4.0], [0.0, -3.0, -2.0, -4.0, -5.0], [-2.0, 1.0, 4.0, 0.0, 6.0], [-6.0, -7.0, -3.0, 0.0, -6.0]], [[3.0, -6.0, 6.0, -1.0, -5.0], [0.0, 0.0, 7.0, 1.0, 7.0], [-2.0, 1.0, 7.0, 0.0, 0.0], [0.0, 1.0, 2.0, -6.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-15.0, 14.0, 0.0, 106.0, -21.0], [-4.0, -34.0, 41.0, -42.0, 13.0], [146.0, -81.0, -24.0, 69.0, 10.0], [-95.0, 69.0, -68.0, -23.0, -33.0]])+(1.-msk_ref)*numpy.array([[13.0, -46.0, 64.0, 11.0, -35.0], [-22.0, -31.0, 70.0, -7.0, -42.0], [-10.0, 41.0, 38.0, 10.0, -24.0], [-24.0, -28.0, 29.0, -37.0, -37.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(7.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 3.0], [3.0, 1.0]], [[7.0, -2.0], [-7.0, 4.0]], [[3.0, 6.0], [3.0, -1.0]], [[-2.0, 7.0], [2.0, -4.0]], [[-7.0, 1.0], [-3.0, -3.0]], [[6.0, -3.0], [1.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -4.0], [0.0, 6.0]], [[1.0, 4.0], [-5.0, -7.0]], [[4.0, 5.0], [0.0, -7.0]], [[5.0, -2.0], [-1.0, -2.0]], [[2.0, 5.0], [-1.0, 7.0]], [[2.0, -7.0], [4.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[25.0, -15.0], [-15.0, -5.0]], [[-35.0, 10.0], [35.0, -20.0]], [[-15.0, -30.0], [-15.0, 5.0]], [[10.0, -35.0], [-10.0, 20.0]], [[35.0, -5.0], [15.0, 15.0]], [[-30.0, 15.0], [-5.0, 30.0]]])+(1.-msk_ref)*numpy.array([[[14.0, -28.0], [0.0, 42.0]], [[7.0, 28.0], [-35.0, -49.0]], [[28.0, 35.0], [0.0, -49.0]], [[35.0, -14.0], [-7.0, -14.0]], [[14.0, 35.0], [-7.0, 49.0]], [[14.0, -49.0], [28.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, -1.0, -7.0])+(1.-msk_arg0)*numpy.array([-6.0, -4.0, 4.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, 6.0], [-1.0, 3.0]], [[-5.0, -2.0], [6.0, 2.0]], [[-5.0, -7.0], [-1.0, 2.0]], [[3.0, 7.0], [4.0, 4.0]], [[-5.0, 0.0], [-5.0, -6.0]], [[3.0, 6.0], [-6.0, -7.0]]], [[[0.0, 1.0], [-2.0, 1.0]], [[-1.0, -4.0], [-5.0, -7.0]], [[5.0, 5.0], [0.0, -4.0]], [[4.0, 7.0], [-6.0, 3.0]], [[0.0, 6.0], [7.0, 5.0]], [[-3.0, 4.0], [-6.0, -6.0]]], [[[0.0, -1.0], [1.0, -6.0]], [[6.0, 2.0], [-6.0, 6.0]], [[1.0, 7.0], [-1.0, 4.0]], [[2.0, 7.0], [1.0, -2.0]], [[-5.0, -5.0], [-6.0, 3.0]], [[0.0, -2.0], [0.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, -2.0], [7.0, 3.0]], [[2.0, 0.0], [0.0, 4.0]], [[4.0, 3.0], [0.0, -6.0]], [[-7.0, 5.0], [0.0, 0.0]], [[-4.0, 7.0], [4.0, -6.0]], [[7.0, 5.0], [7.0, -3.0]]], [[[-3.0, 7.0], [7.0, 4.0]], [[4.0, -4.0], [-3.0, -3.0]], [[-1.0, 4.0], [-6.0, 3.0]], [[6.0, 7.0], [5.0, 3.0]], [[6.0, -6.0], [0.0, -5.0]], [[-7.0, -6.0], [5.0, 5.0]]], [[[0.0, 2.0], [-7.0, 5.0]], [[5.0, -4.0], [0.0, 7.0]], [[3.0, 5.0], [7.0, -4.0]], [[-5.0, -5.0], [-4.0, 7.0]], [[-1.0, 0.0], [-1.0, -4.0]], [[-4.0, 2.0], [5.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-18.0, -30.0], [1.0, 23.0]], [[-11.0, 2.0], [11.0, -47.0]], [[18.0, -12.0], [13.0, -36.0]], [[-36.0, -98.0], [-25.0, -13.0]], [[65.0, 29.0], [65.0, 10.0]], [[-15.0, -26.0], [42.0, 6.0]]])+(1.-msk_ref)*numpy.array([[[54.0, -8.0], [-98.0, -14.0]], [[-8.0, 0.0], [12.0, 16.0]], [[-8.0, -14.0], [52.0, 8.0]], [[-2.0, -78.0], [-36.0, 16.0]], [[-4.0, -18.0], [-28.0, 40.0]], [[-30.0, 2.0], [-42.0, 14.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank0_expandedData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-6.0)+(1-msk_arg0)*(-7.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, -5.0, 3.0, 5.0], [-3.0, 3.0, 1.0, 7.0], [-3.0, -6.0, 0.0, -2.0]], [[-3.0, 3.0, 0.0, -5.0], [3.0, -1.0, 0.0, 3.0], [-7.0, -4.0, -5.0, -7.0]]], [[[1.0, 3.0, -4.0, 0.0], [-4.0, -7.0, -5.0, -2.0], [-3.0, -5.0, -4.0, 5.0]], [[5.0, 0.0, 2.0, -5.0], [-4.0, 4.0, -1.0, -4.0], [-5.0, 3.0, -3.0, -5.0]]], [[[-3.0, 7.0, -6.0, -1.0], [-5.0, 7.0, -3.0, 1.0], [-1.0, -2.0, 1.0, -1.0]], [[-6.0, -7.0, 3.0, -4.0], [3.0, 6.0, 0.0, 2.0], [6.0, -3.0, 1.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 2.0, 6.0, -7.0], [3.0, 0.0, 4.0, -5.0], [-4.0, 1.0, 4.0, 2.0]], [[-4.0, -1.0, 6.0, 2.0], [-2.0, -5.0, -2.0, -2.0], [-3.0, -1.0, -7.0, 0.0]]], [[[4.0, -7.0, -5.0, -4.0], [-4.0, 5.0, 7.0, 3.0], [4.0, -7.0, 7.0, -5.0]], [[4.0, 2.0, 4.0, 0.0], [0.0, 5.0, 3.0, 0.0], [-7.0, 5.0, 2.0, 7.0]]], [[[1.0, -2.0, 4.0, 6.0], [0.0, 2.0, 0.0, 3.0], [6.0, -5.0, 0.0, -5.0]], [[-2.0, 4.0, -5.0, 0.0], [-4.0, 0.0, 5.0, 0.0], [1.0, -5.0, -1.0, 0.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-12.0, 30.0, -18.0, -30.0], [18.0, -18.0, -6.0, -42.0], [18.0, 36.0, -0.0, 12.0]], [[18.0, -18.0, -0.0, 30.0], [-18.0, 6.0, -0.0, -18.0], [42.0, 24.0, 30.0, 42.0]]], [[[-6.0, -18.0, 24.0, -0.0], [24.0, 42.0, 30.0, 12.0], [18.0, 30.0, 24.0, -30.0]], [[-30.0, -0.0, -12.0, 30.0], [24.0, -24.0, 6.0, 24.0], [30.0, -18.0, 18.0, 30.0]]], [[[18.0, -42.0, 36.0, 6.0], [30.0, -42.0, 18.0, -6.0], [6.0, 12.0, -6.0, 6.0]], [[36.0, 42.0, -18.0, 24.0], [-18.0, -36.0, -0.0, -12.0], [-36.0, 18.0, -6.0, -30.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -14.0, -42.0, 49.0], [-21.0, -0.0, -28.0, 35.0], [28.0, -7.0, -28.0, -14.0]], [[28.0, 7.0, -42.0, -14.0], [14.0, 35.0, 14.0, 14.0], [21.0, 7.0, 49.0, -0.0]]], [[[-28.0, 49.0, 35.0, 28.0], [28.0, -35.0, -49.0, -21.0], [-28.0, 49.0, -49.0, 35.0]], [[-28.0, -14.0, -28.0, -0.0], [-0.0, -35.0, -21.0, -0.0], [49.0, -35.0, -14.0, -49.0]]], [[[-7.0, 14.0, -28.0, -42.0], [-0.0, -14.0, -0.0, -21.0], [-42.0, 35.0, -0.0, 35.0]], [[14.0, -28.0, 35.0, -0.0], [28.0, -0.0, -35.0, -0.0], [-7.0, 35.0, 7.0, -0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, -3.0])+(1.-msk_arg0)*numpy.array([-7.0, 7.0])
      arg1=-6.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-42.0, 18.0])+(1.-msk_ref)*numpy.array([42.0, -42.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, 0.0])+(1.-msk_arg0)*numpy.array([-2.0, -3.0])
      arg1=numpy.array(-2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([2.0, -0.0])+(1.-msk_ref)*numpy.array([4.0, 6.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 2.0, 3.0], [-5.0, 5.0, -7.0]])+(1.-msk_arg0)*numpy.array([[6.0, -6.0, -7.0], [-2.0, 0.0, 7.0]])
      arg1=numpy.array([6.0, -1.0, 3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([25.0, -56.0])+(1.-msk_ref)*numpy.array([21.0, 9.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, -6.0, -1.0], [-4.0, -4.0, 0.0]], [[0.0, -6.0, 3.0], [2.0, 0.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 2.0, -3.0], [5.0, 2.0, -7.0]], [[1.0, -3.0, -3.0], [2.0, -7.0, -5.0]]])
      arg1=numpy.array([[6.0, -2.0, -3.0], [4.0, -4.0, 2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([21.0, 5.0])+(1.-msk_ref)*numpy.array([21.0, 47.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, 1.0, 0.0], [-5.0, 4.0, 1.0], [-1.0, 0.0, -7.0], [-3.0, 6.0, -3.0]], [[-6.0, -2.0, 2.0], [-5.0, -5.0, 5.0], [-7.0, 5.0, -4.0], [6.0, 0.0, 6.0]]], [[[7.0, -6.0, 7.0], [0.0, -2.0, 6.0], [3.0, 5.0, 7.0], [-4.0, 3.0, 2.0]], [[7.0, 7.0, -1.0], [0.0, 0.0, -5.0], [1.0, 3.0, -4.0], [-1.0, 1.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, 0.0, 2.0], [0.0, 0.0, 3.0], [-7.0, -2.0, 4.0], [-5.0, 4.0, 0.0]], [[0.0, 1.0, 6.0], [1.0, 1.0, 0.0], [3.0, 0.0, -2.0], [2.0, 2.0, 5.0]]], [[[2.0, 0.0, -1.0], [1.0, 1.0, -6.0], [0.0, 4.0, -5.0], [2.0, -3.0, 3.0]], [[6.0, -6.0, 4.0], [-6.0, 0.0, -7.0], [0.0, 0.0, 3.0], [4.0, -2.0, -6.0]]]])
      arg1=numpy.array([[[1.0, -5.0, 7.0], [-7.0, -4.0, -1.0], [-2.0, 1.0, 7.0], [-7.0, 0.0, -5.0]], [[6.0, 0.0, 0.0], [-4.0, -6.0, 0.0], [-2.0, -6.0, -7.0], [6.0, -2.0, 7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([100.0, 245.0])+(1.-msk_ref)*numpy.array([124.0, -45.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, -1.0])+(1.-msk_arg0)*numpy.array([-7.0, -6.0])
      arg1=numpy.array([-4.0, 7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, -14.0], [4.0, -7.0]])+(1.-msk_ref)*numpy.array([[28.0, -49.0], [24.0, -42.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 4.0, 0.0], [-2.0, 0.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 6.0, 6.0], [-1.0, 6.0, -7.0]])
      arg1=numpy.array([[0.0, 7.0], [1.0, -3.0], [7.0, 2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, -33.0], [0.0, -14.0]])+(1.-msk_ref)*numpy.array([[48.0, -41.0], [-43.0, -39.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[5.0, 3.0, 1.0], [-6.0, 0.0, -6.0]], [[-3.0, 0.0, 5.0], [-5.0, 3.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, 7.0, 1.0], [2.0, 7.0, -6.0]], [[-6.0, -2.0, -5.0], [-4.0, 3.0, 2.0]]])
      arg1=numpy.array([[[6.0, -4.0], [-6.0, -7.0], [-2.0, 6.0]], [[-6.0, 6.0], [5.0, 2.0], [-2.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[58.0, -53.0], [25.0, 30.0]])+(1.-msk_ref)*numpy.array([[-3.0, -3.0], [21.0, -16.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, -1.0, 3.0], [-5.0, -7.0, 0.0], [5.0, -5.0, -7.0], [-3.0, 7.0, -5.0]], [[1.0, -5.0, 0.0], [-1.0, -6.0, -7.0], [-2.0, -2.0, 0.0], [4.0, 1.0, -2.0]]], [[[7.0, -5.0, 2.0], [4.0, -6.0, 0.0], [-5.0, 2.0, 3.0], [-1.0, 6.0, 2.0]], [[0.0, 5.0, -4.0], [-7.0, -4.0, 3.0], [1.0, -2.0, 0.0], [-4.0, -4.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, 1.0, 1.0], [2.0, 2.0, 0.0], [7.0, 0.0, 0.0], [-6.0, 4.0, -3.0]], [[1.0, -2.0, -1.0], [0.0, 3.0, -1.0], [0.0, 5.0, 2.0], [-4.0, -5.0, -3.0]]], [[[-6.0, 0.0, -3.0], [0.0, 1.0, 4.0], [2.0, -3.0, -3.0], [4.0, 7.0, 6.0]], [[-6.0, -5.0, -6.0], [0.0, 3.0, 5.0], [2.0, -7.0, 6.0], [3.0, -2.0, 1.0]]]])
      arg1=numpy.array([[[[0.0, -2.0], [-2.0, 5.0], [4.0, 1.0]], [[3.0, 7.0], [1.0, -2.0], [-1.0, 1.0]], [[1.0, -1.0], [0.0, -4.0], [-6.0, -3.0]], [[-3.0, 1.0], [1.0, 6.0], [5.0, -3.0]]], [[[7.0, 7.0], [-6.0, 7.0], [0.0, 5.0]], [[4.0, -4.0], [-2.0, -7.0], [1.0, 2.0]], [[-6.0, 7.0], [-1.0, -6.0], [-4.0, 3.0]], [[6.0, 3.0], [-7.0, 4.0], [6.0, -2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[87.0, 83.0], [-51.0, 96.0]])+(1.-msk_ref)*numpy.array([[16.0, -43.0], [26.0, 13.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, 2.0])+(1.-msk_arg0)*numpy.array([-5.0, 3.0])
      arg1=numpy.array([[2.0, 2.0, -7.0, 1.0, -6.0], [-5.0, 5.0, 1.0, -6.0, -3.0], [5.0, 7.0, 1.0, -5.0, 6.0], [-2.0, -1.0, 4.0, 5.0, -2.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -12.0, 42.0, -6.0, 36.0], [30.0, -30.0, -6.0, 36.0, 18.0], [-30.0, -42.0, -6.0, 30.0, -36.0], [12.0, 6.0, -24.0, -30.0, 12.0]], [[4.0, 4.0, -14.0, 2.0, -12.0], [-10.0, 10.0, 2.0, -12.0, -6.0], [10.0, 14.0, 2.0, -10.0, 12.0], [-4.0, -2.0, 8.0, 10.0, -4.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -10.0, 35.0, -5.0, 30.0], [25.0, -25.0, -5.0, 30.0, 15.0], [-25.0, -35.0, -5.0, 25.0, -30.0], [10.0, 5.0, -20.0, -25.0, 10.0]], [[6.0, 6.0, -21.0, 3.0, -18.0], [-15.0, 15.0, 3.0, -18.0, -9.0], [15.0, 21.0, 3.0, -15.0, 18.0], [-6.0, -3.0, 12.0, 15.0, -6.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, -3.0, 0.0], [-7.0, -2.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 1.0, -4.0], [-1.0, -7.0, -6.0]])
      arg1=numpy.array([[[0.0, 2.0, 3.0, 7.0, -5.0], [3.0, -5.0, -1.0, -5.0, -6.0], [7.0, 3.0, 5.0, -3.0, -3.0], [1.0, -4.0, -2.0, 7.0, 0.0]], [[3.0, -7.0, -2.0, 0.0, -3.0], [7.0, 3.0, 3.0, 7.0, -7.0], [-6.0, 6.0, 5.0, -4.0, -2.0], [6.0, 7.0, 0.0, 1.0, 5.0]], [[-2.0, -5.0, 0.0, 0.0, 0.0], [1.0, 0.0, -1.0, 6.0, 6.0], [-2.0, -3.0, -7.0, 0.0, -5.0], [4.0, -3.0, 7.0, 3.0, 7.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, 15.0, -3.0, -21.0, 24.0], [-30.0, 6.0, -6.0, -6.0, 39.0], [-3.0, -27.0, -30.0, 21.0, 15.0], [-21.0, -9.0, 6.0, -24.0, -15.0]], [[-20.0, -35.0, -17.0, -49.0, 41.0], [-28.0, 29.0, -6.0, 63.0, 98.0], [-51.0, -54.0, -94.0, 29.0, -10.0], [9.0, -7.0, 63.0, -30.0, 39.0]]])+(1.-msk_ref)*numpy.array([[[11.0, 9.0, -8.0, -14.0, 7.0], [-3.0, 13.0, 9.0, -7.0, -19.0], [-12.0, 12.0, 23.0, 2.0, 24.0], [-12.0, 27.0, -24.0, -25.0, -23.0]], [[-9.0, 77.0, 11.0, -7.0, 26.0], [-58.0, -16.0, -14.0, -80.0, 19.0], [47.0, -27.0, 2.0, 31.0, 47.0], [-67.0, -27.0, -40.0, -32.0, -77.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -1.0, -3.0], [3.0, 5.0, 0.0]], [[1.0, 2.0, -5.0], [5.0, -5.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[-5.0, 0.0, -6.0], [2.0, -6.0, 5.0]], [[-4.0, -7.0, -1.0], [-5.0, 5.0, -3.0]]])
      arg1=numpy.array([[[[-5.0, -2.0, -4.0, -1.0, 4.0], [6.0, -4.0, -6.0, 5.0, -7.0], [0.0, 1.0, -2.0, -1.0, 0.0], [4.0, 5.0, -3.0, 2.0, -4.0]], [[-6.0, -3.0, -1.0, 2.0, -2.0], [0.0, -4.0, 0.0, 0.0, 4.0], [-3.0, 0.0, 4.0, -2.0, 6.0], [0.0, 5.0, 0.0, 3.0, -7.0]], [[-7.0, 3.0, 1.0, -2.0, 2.0], [7.0, -2.0, -2.0, 7.0, -6.0], [-6.0, -7.0, -3.0, 4.0, -5.0], [-5.0, 0.0, 5.0, 3.0, 2.0]]], [[[7.0, 0.0, 7.0, 3.0, 2.0], [0.0, 6.0, 5.0, -2.0, 0.0], [-1.0, 2.0, 1.0, -1.0, -2.0], [3.0, 4.0, 7.0, 6.0, 3.0]], [[-3.0, 0.0, -7.0, -2.0, 0.0], [3.0, -2.0, 2.0, 7.0, -2.0], [-2.0, -4.0, 0.0, -2.0, 7.0], [-7.0, 0.0, -1.0, 0.0, 4.0]], [[5.0, -7.0, 0.0, 6.0, 0.0], [-2.0, -7.0, -2.0, 3.0, 7.0], [-5.0, 3.0, 1.0, -3.0, -7.0], [0.0, -1.0, 4.0, -4.0, 6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[33.0, -6.0, -16.0, 3.0, 2.0], [-6.0, 18.0, 31.0, 8.0, 4.0], [8.0, 7.0, 8.0, -23.0, 38.0], [-11.0, 7.0, 1.0, 6.0, 30.0]], [[53.0, -2.0, 59.0, 20.0, 0.0], [-38.0, 59.0, 25.0, -84.0, 20.0], [44.0, 57.0, 23.0, -11.0, 13.0], [79.0, 38.0, 0.0, 35.0, -51.0]]])+(1.-msk_ref)*numpy.array([[[124.0, -43.0, 70.0, 65.0, -28.0], [-100.0, 21.0, 30.0, -98.0, 118.0], [21.0, 80.0, 35.0, -24.0, -51.0], [58.0, -22.0, 25.0, -36.0, 20.0]], [[4.0, 47.0, -48.0, -51.0, -14.0], [-10.0, 27.0, 17.0, 9.0, -25.0], [37.0, -36.0, -25.0, 18.0, 29.0], [-61.0, -72.0, -45.0, -50.0, 50.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, 6.0])+(1.-msk_arg0)*numpy.array([2.0, 0.0])
      arg1=numpy.array([[[-5.0, 3.0], [4.0, 7.0]], [[1.0, 1.0], [5.0, 7.0]], [[-4.0, 4.0], [-3.0, 0.0]], [[-7.0, 1.0], [5.0, -2.0]], [[6.0, 5.0], [0.0, 5.0]], [[5.0, 5.0], [-6.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-35.0, 21.0], [28.0, 49.0]], [[7.0, 7.0], [35.0, 49.0]], [[-28.0, 28.0], [-21.0, 0.0]], [[-49.0, 7.0], [35.0, -14.0]], [[42.0, 35.0], [0.0, 35.0]], [[35.0, 35.0], [-42.0, 0.0]]], [[[-30.0, 18.0], [24.0, 42.0]], [[6.0, 6.0], [30.0, 42.0]], [[-24.0, 24.0], [-18.0, 0.0]], [[-42.0, 6.0], [30.0, -12.0]], [[36.0, 30.0], [0.0, 30.0]], [[30.0, 30.0], [-36.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-10.0, 6.0], [8.0, 14.0]], [[2.0, 2.0], [10.0, 14.0]], [[-8.0, 8.0], [-6.0, 0.0]], [[-14.0, 2.0], [10.0, -4.0]], [[12.0, 10.0], [0.0, 10.0]], [[10.0, 10.0], [-12.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, 4.0, 2.0], [-2.0, -1.0, 0.0]])+(1.-msk_arg0)*numpy.array([[6.0, 7.0, -7.0], [7.0, 6.0, -7.0]])
      arg1=numpy.array([[[[3.0, -5.0], [0.0, 1.0]], [[-7.0, -2.0], [-3.0, -3.0]], [[-6.0, -3.0], [0.0, 6.0]], [[-4.0, -6.0], [-1.0, 7.0]], [[0.0, 5.0], [6.0, 0.0]], [[4.0, -5.0], [-7.0, -7.0]]], [[[6.0, -6.0], [4.0, -2.0]], [[1.0, -5.0], [-4.0, 6.0]], [[-3.0, -3.0], [-4.0, -7.0]], [[-1.0, 0.0], [3.0, -6.0]], [[7.0, 4.0], [3.0, 7.0]], [[3.0, -3.0], [-6.0, -4.0]]], [[[2.0, 7.0], [0.0, -2.0]], [[7.0, 7.0], [-6.0, -3.0]], [[-4.0, 3.0], [-5.0, 6.0]], [[-6.0, 0.0], [-3.0, 1.0]], [[-7.0, -6.0], [6.0, 0.0]], [[-4.0, 5.0], [-2.0, -2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 25.0], [16.0, -19.0]], [[67.0, 8.0], [-7.0, 39.0]], [[22.0, 15.0], [-26.0, -58.0]], [[12.0, 42.0], [13.0, -71.0]], [[14.0, -31.0], [-18.0, 28.0]], [[-24.0, 33.0], [21.0, 29.0]]], [[[-12.0, 16.0], [-4.0, 0.0]], [[13.0, 9.0], [10.0, 0.0]], [[15.0, 9.0], [4.0, -5.0]], [[9.0, 12.0], [-1.0, -8.0]], [[-7.0, -14.0], [-15.0, -7.0]], [[-11.0, 13.0], [20.0, 18.0]]]])+(1.-msk_ref)*numpy.array([[[[46.0, -121.0], [28.0, 6.0]], [[-84.0, -96.0], [-4.0, 45.0]], [[-29.0, -60.0], [7.0, -55.0]], [[11.0, -36.0], [36.0, -7.0]], [[98.0, 100.0], [15.0, 49.0]], [[73.0, -86.0], [-70.0, -56.0]]], [[[43.0, -120.0], [24.0, 9.0]], [[-92.0, -93.0], [-3.0, 36.0]], [[-32.0, -60.0], [11.0, -42.0]], [[8.0, -42.0], [32.0, 6.0]], [[91.0, 101.0], [18.0, 42.0]], [[74.0, -88.0], [-71.0, -59.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, -1.0])+(1.-msk_arg0)*numpy.array([-7.0, 0.0])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-6.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([36.0, 6.0])+(1.-msk_ref)*numpy.array([42.0, -0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, -1.0, 1.0], [7.0, 4.0, -4.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 0.0, -5.0], [4.0, 1.0, 7.0]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([7.0, 0.0, 6.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-43.0, 25.0])+(1.-msk_ref)*numpy.array([-65.0, 70.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-4.0, -7.0, -2.0], [3.0, 0.0, 4.0]], [[0.0, 0.0, 3.0], [-2.0, 5.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, -2.0, -3.0], [6.0, 4.0, -1.0]], [[2.0, 2.0, -7.0], [6.0, 7.0, -6.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[0.0, -6.0, 7.0], [6.0, 5.0, 5.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([66.0, 34.0])+(1.-msk_ref)*numpy.array([42.0, -20.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, 5.0, -5.0], [3.0, 4.0, -7.0], [-6.0, 0.0, -3.0], [-7.0, -1.0, 1.0]], [[-6.0, 0.0, -6.0], [6.0, 5.0, 5.0], [-4.0, 2.0, 4.0], [7.0, 5.0, 4.0]]], [[[-6.0, 2.0, 7.0], [-2.0, 2.0, -4.0], [-4.0, -3.0, -4.0], [0.0, 1.0, -3.0]], [[2.0, -3.0, 2.0], [-5.0, 3.0, -5.0], [-4.0, -6.0, -1.0], [0.0, 3.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, 1.0, 7.0], [6.0, -7.0, -5.0], [2.0, -7.0, -6.0], [7.0, -1.0, -6.0]], [[6.0, 2.0, -4.0], [-2.0, -7.0, -1.0], [7.0, 5.0, 4.0], [-4.0, -1.0, -4.0]]], [[[4.0, -7.0, 6.0], [-3.0, 2.0, 2.0], [3.0, -7.0, -1.0], [0.0, 0.0, -7.0]], [[-3.0, 0.0, 2.0], [4.0, 6.0, 7.0], [-1.0, 2.0, 4.0], [-5.0, 5.0, -3.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[5.0, -7.0, 4.0], [5.0, -4.0, 2.0], [7.0, 5.0, 2.0], [-7.0, -6.0, -4.0]], [[6.0, -5.0, -2.0], [2.0, -2.0, 7.0], [-6.0, -4.0, 0.0], [-3.0, 7.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-34.0, -11.0])+(1.-msk_ref)*numpy.array([52.0, 172.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, -1.0])+(1.-msk_arg0)*numpy.array([3.0, -7.0])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([3.0, 4.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 4.0], [-3.0, -4.0]])+(1.-msk_ref)*numpy.array([[9.0, 12.0], [-21.0, -28.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 0.0, 6.0], [1.0, -7.0, -4.0]])+(1.-msk_arg0)*numpy.array([[-3.0, 0.0, -3.0], [6.0, -7.0, -5.0]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-5.0, 0.0], [-4.0, 5.0], [5.0, -4.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[60.0, -24.0], [3.0, -19.0]])+(1.-msk_ref)*numpy.array([[0.0, 12.0], [-27.0, -15.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 4.0, -2.0], [-6.0, 2.0, -4.0]], [[-6.0, -7.0, 4.0], [4.0, 4.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 4.0, 1.0], [4.0, -2.0, 4.0]], [[7.0, 5.0, 4.0], [3.0, 4.0, 2.0]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-7.0, 3.0], [1.0, -6.0], [-1.0, -6.0]], [[5.0, -4.0], [1.0, -6.0], [-7.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[55.0, -21.0], [34.0, -40.0]])+(1.-msk_ref)*numpy.array([[42.0, -55.0], [-43.0, -69.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 6.0, -6.0], [4.0, -1.0, -4.0], [0.0, 3.0, 1.0], [3.0, -5.0, 5.0]], [[7.0, -2.0, 1.0], [-1.0, -1.0, 6.0], [-5.0, 1.0, 0.0], [-2.0, -4.0, 3.0]]], [[[7.0, -1.0, 0.0], [5.0, 0.0, 0.0], [7.0, -3.0, 0.0], [6.0, -6.0, 2.0]], [[0.0, -2.0, -2.0], [-1.0, 5.0, 2.0], [-5.0, 0.0, 5.0], [-5.0, 1.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -6.0, 0.0], [-5.0, 0.0, -2.0], [-2.0, 7.0, 3.0], [-2.0, -7.0, -6.0]], [[1.0, -1.0, 2.0], [0.0, 6.0, -5.0], [-1.0, -6.0, 7.0], [6.0, -4.0, 0.0]]], [[[-3.0, 0.0, 0.0], [5.0, 5.0, 0.0], [5.0, 0.0, -1.0], [0.0, 7.0, 7.0]], [[-3.0, -1.0, 5.0], [-6.0, 2.0, 0.0], [7.0, 3.0, -4.0], [-1.0, 0.0, -5.0]]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-3.0, -3.0], [-6.0, 6.0], [-7.0, -1.0]], [[-1.0, 3.0], [4.0, -7.0], [-4.0, 3.0]], [[7.0, 0.0], [-4.0, 7.0], [5.0, 1.0]], [[0.0, 6.0], [7.0, 2.0], [2.0, 2.0]]], [[[4.0, -3.0], [-7.0, 1.0], [2.0, -1.0]], [[-6.0, -5.0], [-7.0, -2.0], [-3.0, -3.0]], [[-5.0, -5.0], [0.0, -2.0], [-3.0, 7.0]], [[3.0, -4.0], [0.0, 0.0], [6.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[79.0, 118.0], [-33.0, 60.0]])+(1.-msk_ref)*numpy.array([[-37.0, 8.0], [88.0, -40.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, 3.0])+(1.-msk_arg0)*numpy.array([0.0, -6.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-1.0, -1.0, -4.0, -4.0, -4.0], [4.0, 0.0, 5.0, 4.0, 0.0], [6.0, -3.0, -2.0, 1.0, 3.0], [5.0, 4.0, -3.0, 0.0, -6.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-7.0, -7.0, -28.0, -28.0, -28.0], [28.0, 0.0, 35.0, 28.0, 0.0], [42.0, -21.0, -14.0, 7.0, 21.0], [35.0, 28.0, -21.0, 0.0, -42.0]], [[-3.0, -3.0, -12.0, -12.0, -12.0], [12.0, 0.0, 15.0, 12.0, 0.0], [18.0, -9.0, -6.0, 3.0, 9.0], [15.0, 12.0, -9.0, 0.0, -18.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[6.0, 6.0, 24.0, 24.0, 24.0], [-24.0, 0.0, -30.0, -24.0, 0.0], [-36.0, 18.0, 12.0, -6.0, -18.0], [-30.0, -24.0, 18.0, 0.0, 36.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -4.0, 3.0], [-7.0, 7.0, -5.0]])+(1.-msk_arg0)*numpy.array([[0.0, -5.0, -7.0], [7.0, 4.0, 1.0]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-1.0, 7.0, -7.0, -1.0, -6.0], [-6.0, -7.0, 5.0, 5.0, 5.0], [3.0, -4.0, -3.0, 0.0, 6.0], [5.0, -6.0, -1.0, 4.0, -6.0]], [[2.0, 0.0, 7.0, -5.0, -7.0], [-1.0, -6.0, 2.0, 4.0, -7.0], [-2.0, 2.0, 3.0, 2.0, 1.0], [0.0, -7.0, -4.0, 6.0, 0.0]], [[-2.0, -7.0, 0.0, -1.0, -5.0], [6.0, -1.0, 5.0, 5.0, 4.0], [-7.0, 4.0, -7.0, -2.0, 1.0], [4.0, -4.0, 7.0, 6.0, -1.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-8.0, -63.0, 14.0, 23.0, 49.0], [58.0, 63.0, -23.0, -31.0, 10.0], [-31.0, 28.0, -15.0, -14.0, -37.0], [-18.0, 52.0, 43.0, -30.0, 33.0]], [[31.0, -14.0, 98.0, -23.0, 18.0], [5.0, 12.0, -46.0, -32.0, -104.0], [0.0, 22.0, 77.0, 24.0, -40.0], [-55.0, 13.0, -56.0, -16.0, 47.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 49.0, -35.0, 32.0, 70.0], [-37.0, 37.0, -45.0, -55.0, 7.0], [59.0, -38.0, 34.0, 4.0, -12.0], [-28.0, 63.0, -29.0, -72.0, 7.0]], [[-1.0, 42.0, -21.0, -28.0, -75.0], [-40.0, -74.0, 48.0, 56.0, 11.0], [6.0, -16.0, -16.0, 6.0, 47.0], [39.0, -74.0, -16.0, 58.0, -43.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -4.0, 4.0], [-6.0, 6.0, 5.0]], [[-6.0, 6.0, -4.0], [7.0, 0.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-6.0, 4.0, 1.0], [0.0, 4.0, 6.0]], [[-7.0, 2.0, 4.0], [6.0, 3.0, -4.0]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[4.0, 7.0, 3.0, 1.0, 5.0], [0.0, -7.0, 5.0, 4.0, 7.0], [-5.0, -2.0, 3.0, 0.0, 1.0], [0.0, -5.0, 0.0, 0.0, -5.0]], [[-7.0, -1.0, -3.0, -1.0, -7.0], [6.0, 0.0, -5.0, 2.0, -1.0], [7.0, 2.0, 3.0, 1.0, -5.0], [2.0, 0.0, 7.0, 1.0, 2.0]], [[5.0, 3.0, 1.0, 7.0, 7.0], [0.0, 0.0, -7.0, 5.0, 6.0], [-2.0, 0.0, -3.0, -7.0, -3.0], [2.0, 5.0, 7.0, 0.0, -2.0]]], [[[-6.0, -5.0, 5.0, -3.0, 2.0], [2.0, 6.0, -3.0, -2.0, 7.0], [-1.0, 0.0, 1.0, 0.0, -4.0], [0.0, -6.0, -1.0, 3.0, -5.0]], [[4.0, 7.0, 1.0, 3.0, 7.0], [7.0, 7.0, 6.0, 0.0, 0.0], [-7.0, -3.0, -4.0, 1.0, -7.0], [0.0, -6.0, 2.0, 0.0, 4.0]], [[4.0, -7.0, -5.0, -7.0, 5.0], [0.0, 1.0, 6.0, 5.0, -7.0], [-1.0, -7.0, 0.0, 1.0, 0.0], [0.0, 5.0, 0.0, 5.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[128.0, 53.0, -33.0, 33.0, 111.0], [6.0, 11.0, 76.0, 49.0, -49.0], [-77.0, -61.0, -54.0, -21.0, -10.0], [0.0, 45.0, 18.0, 3.0, 38.0]], [[-128.0, -95.0, -5.0, -61.0, -86.0], [50.0, 84.0, -53.0, -46.0, -23.0], [73.0, 24.0, 19.0, 34.0, -52.0], [4.0, -32.0, 7.0, 27.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-7.0, -57.0, -55.0, -33.0, 7.0], [52.0, 76.0, 3.0, 19.0, -82.0], [22.0, -34.0, -25.0, 7.0, -57.0], [10.0, 41.0, 43.0, 34.0, 52.0]], [[-62.0, -20.0, 30.0, 38.0, -8.0], [45.0, 102.0, -97.0, -36.0, 43.0], [18.0, 37.0, -33.0, -27.0, -74.0], [12.0, -19.0, 42.0, 0.0, 13.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, -7.0])+(1.-msk_arg0)*numpy.array([3.0, -4.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-1.0, -4.0], [0.0, -5.0]], [[-3.0, -6.0], [-2.0, 7.0]], [[-6.0, -2.0], [-6.0, -1.0]], [[1.0, -4.0], [4.0, 2.0]], [[0.0, 5.0], [2.0, -3.0]], [[6.0, -7.0], [-5.0, -7.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 20.0], [0.0, 25.0]], [[15.0, 30.0], [10.0, -35.0]], [[30.0, 10.0], [30.0, 5.0]], [[-5.0, 20.0], [-20.0, -10.0]], [[0.0, -25.0], [-10.0, 15.0]], [[-30.0, 35.0], [25.0, 35.0]]], [[[7.0, 28.0], [0.0, 35.0]], [[21.0, 42.0], [14.0, -49.0]], [[42.0, 14.0], [42.0, 7.0]], [[-7.0, 28.0], [-28.0, -14.0]], [[0.0, -35.0], [-14.0, 21.0]], [[-42.0, 49.0], [35.0, 49.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -12.0], [0.0, -15.0]], [[-9.0, -18.0], [-6.0, 21.0]], [[-18.0, -6.0], [-18.0, -3.0]], [[3.0, -12.0], [12.0, 6.0]], [[0.0, 15.0], [6.0, -9.0]], [[18.0, -21.0], [-15.0, -21.0]]], [[[4.0, 16.0], [0.0, 20.0]], [[12.0, 24.0], [8.0, -28.0]], [[24.0, 8.0], [24.0, 4.0]], [[-4.0, 16.0], [-16.0, -8.0]], [[0.0, -20.0], [-8.0, 12.0]], [[-24.0, 28.0], [20.0, 28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, -4.0, -7.0], [-2.0, 1.0, -6.0]])+(1.-msk_arg0)*numpy.array([[7.0, 0.0, 5.0], [2.0, 7.0, 1.0]])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[1.0, -6.0], [4.0, -4.0]], [[5.0, 0.0], [-5.0, 0.0]], [[3.0, 2.0], [-6.0, -4.0]], [[2.0, -7.0], [-3.0, 5.0]], [[-5.0, -3.0], [6.0, -7.0]], [[-5.0, -5.0], [0.0, -4.0]]], [[[-7.0, -2.0], [-5.0, -2.0]], [[-1.0, 2.0], [-6.0, -4.0]], [[1.0, -7.0], [0.0, 4.0]], [[4.0, 1.0], [-1.0, 4.0]], [[6.0, -3.0], [0.0, -7.0]], [[1.0, 5.0], [-5.0, 1.0]]], [[[-7.0, 6.0], [6.0, 3.0]], [[-5.0, -4.0], [4.0, 0.0]], [[-5.0, 2.0], [-4.0, 0.0]], [[0.0, 5.0], [-7.0, 0.0]], [[2.0, 3.0], [6.0, 1.0]], [[-5.0, 4.0], [4.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[78.0, -40.0], [-18.0, -17.0]], [[44.0, 20.0], [-9.0, 16.0]], [[34.0, 16.0], [22.0, -20.0]], [[-14.0, -46.0], [50.0, -11.0]], [[-43.0, -12.0], [-36.0, 14.0]], [[26.0, -53.0], [-8.0, -43.0]]], [[[33.0, -26.0], [-49.0, -12.0]], [[19.0, 26.0], [-20.0, -4.0]], [[25.0, -23.0], [36.0, 12.0]], [[0.0, -15.0], [47.0, -6.0]], [[4.0, -15.0], [-48.0, 1.0]], [[41.0, -9.0], [-29.0, -21.0]]]])+(1.-msk_ref)*numpy.array([[[[-28.0, -12.0], [58.0, -13.0]], [[10.0, -20.0], [-15.0, 0.0]], [[-4.0, 24.0], [-62.0, -28.0]], [[14.0, -24.0], [-56.0, 35.0]], [[-25.0, -6.0], [72.0, -44.0]], [[-60.0, -15.0], [20.0, -3.0]]], [[[-54.0, -20.0], [-21.0, -19.0]], [[-2.0, 10.0], [-48.0, -28.0]], [[8.0, -43.0], [-16.0, 20.0]], [[32.0, -2.0], [-20.0, 38.0]], [[34.0, -24.0], [18.0, -62.0]], [[-8.0, 29.0], [-31.0, 4.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, 5.0])+(1.-msk_arg0)*numpy.array([0.0, -5.0])
      arg1=Data(7.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-49.0, 35.0])+(1.-msk_ref)*numpy.array([0.0, -35.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 3.0, -4.0], [-2.0, 4.0, -4.0]])+(1.-msk_arg0)*numpy.array([[5.0, 5.0, -5.0], [4.0, 7.0, -6.0]])
      arg1=Data(numpy.array([6.0, -3.0, -1.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([25.0, -20.0])+(1.-msk_ref)*numpy.array([20.0, 9.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 3.0, -5.0], [3.0, 5.0, 7.0]], [[0.0, -4.0, 6.0], [0.0, 1.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -2.0, -4.0], [1.0, 0.0, -2.0]], [[2.0, -6.0, -4.0], [-5.0, 5.0, -1.0]]])
      arg1=Data(numpy.array([[2.0, 7.0, 0.0], [1.0, 6.0, -5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([21.0, -37.0])+(1.-msk_ref)*numpy.array([1.0, -8.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 0.0, -6.0], [-2.0, -2.0, 4.0], [-4.0, 7.0, -5.0], [1.0, -5.0, -7.0]], [[5.0, 6.0, -3.0], [7.0, -5.0, -5.0], [7.0, 2.0, -6.0], [3.0, 0.0, 7.0]]], [[[5.0, 1.0, -4.0], [0.0, -5.0, 4.0], [4.0, -2.0, 6.0], [-2.0, -4.0, 0.0]], [[-1.0, -3.0, -2.0], [0.0, -3.0, 5.0], [1.0, 5.0, 6.0], [0.0, 7.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, -2.0, 0.0], [-5.0, -6.0, 7.0], [-2.0, -3.0, 4.0], [-3.0, -2.0, -1.0]], [[-6.0, 2.0, -5.0], [5.0, 3.0, 0.0], [3.0, 1.0, -7.0], [2.0, -2.0, 5.0]]], [[[6.0, 0.0, -4.0], [-6.0, -5.0, 1.0], [-3.0, 7.0, 7.0], [0.0, -5.0, -4.0]], [[2.0, 5.0, -6.0], [1.0, -3.0, 5.0], [-6.0, 1.0, 6.0], [7.0, 7.0, 6.0]]]])
      arg1=Data(numpy.array([[[-5.0, 3.0, -6.0], [-7.0, 6.0, 0.0], [0.0, 4.0, 7.0], [-4.0, -3.0, -6.0]], [[2.0, 3.0, 5.0], [4.0, -7.0, 1.0], [5.0, 2.0, -3.0], [7.0, 6.0, -1.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([206.0, 67.0])+(1.-msk_ref)*numpy.array([41.0, 180.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, 6.0])+(1.-msk_arg0)*numpy.array([-5.0, -2.0])
      arg1=Data(numpy.array([-2.0, -3.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[14.0, 21.0], [-12.0, -18.0]])+(1.-msk_ref)*numpy.array([[10.0, 15.0], [4.0, 6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 0.0, 5.0], [-3.0, -5.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-6.0, 0.0, 7.0], [0.0, -1.0, -2.0]])
      arg1=Data(numpy.array([[1.0, -5.0], [0.0, 0.0], [-3.0, 5.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-21.0, 55.0], [3.0, 5.0]])+(1.-msk_ref)*numpy.array([[-27.0, 65.0], [6.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -5.0, -1.0], [0.0, -2.0, 3.0]], [[2.0, -6.0, 1.0], [0.0, -4.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, -5.0, 0.0], [0.0, -4.0, -1.0]], [[-6.0, -7.0, -3.0], [-1.0, 0.0, -6.0]]])
      arg1=Data(numpy.array([[[-5.0, -7.0], [5.0, 0.0], [-1.0, 0.0]], [[-4.0, -1.0], [-6.0, 1.0], [-4.0, -4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, -14.0], [-25.0, -26.0]])+(1.-msk_ref)*numpy.array([[-32.0, -49.0], [26.0, 67.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 3.0, -2.0], [-3.0, -6.0, 6.0], [1.0, 3.0, 6.0], [-2.0, -7.0, -2.0]], [[-7.0, -3.0, -3.0], [-1.0, 2.0, -6.0], [2.0, -5.0, 4.0], [3.0, -5.0, 0.0]]], [[[-6.0, -2.0, 5.0], [7.0, 4.0, 3.0], [7.0, 2.0, 3.0], [2.0, 0.0, 2.0]], [[5.0, -1.0, 0.0], [-6.0, 0.0, 7.0], [0.0, 4.0, -2.0], [-5.0, 6.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, -6.0, -4.0], [4.0, 7.0, 3.0], [0.0, -2.0, 6.0], [2.0, 5.0, -4.0]], [[6.0, 0.0, 4.0], [-5.0, 2.0, 1.0], [-2.0, 3.0, -6.0], [-6.0, -5.0, -3.0]]], [[[0.0, -5.0, 7.0], [1.0, 6.0, 5.0], [5.0, 2.0, -1.0], [-1.0, -3.0, 5.0]], [[-1.0, 4.0, -2.0], [3.0, 5.0, -7.0], [2.0, 0.0, 3.0], [-2.0, 0.0, 0.0]]]])
      arg1=Data(numpy.array([[[[-2.0, 3.0], [1.0, 2.0], [2.0, 3.0]], [[3.0, -7.0], [-2.0, 1.0], [0.0, -1.0]], [[-6.0, 7.0], [0.0, -5.0], [0.0, 2.0]], [[-7.0, -2.0], [0.0, -1.0], [1.0, 0.0]]], [[[6.0, 5.0], [-2.0, -5.0], [-6.0, 5.0]], [[-2.0, 5.0], [-2.0, 1.0], [-7.0, -5.0]], [[-5.0, -2.0], [-1.0, -3.0], [-7.0, -1.0]], [[1.0, -1.0], [7.0, -5.0], [2.0, -1.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-45.0, 60.0], [15.0, -81.0]])+(1.-msk_ref)*numpy.array([[-35.0, 43.0], [-20.0, 48.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([4.0, 0.0])+(1.-msk_arg0)*numpy.array([0.0, -5.0])
      arg1=Data(numpy.array([[6.0, 1.0, -6.0, 6.0, 0.0], [-2.0, -3.0, 1.0, 2.0, 0.0], [4.0, 1.0, 4.0, 7.0, -6.0], [6.0, 0.0, 5.0, -5.0, 0.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[24.0, 4.0, -24.0, 24.0, 0.0], [-8.0, -12.0, 4.0, 8.0, 0.0], [16.0, 4.0, 16.0, 28.0, -24.0], [24.0, 0.0, 20.0, -20.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-30.0, -5.0, 30.0, -30.0, 0.0], [10.0, 15.0, -5.0, -10.0, 0.0], [-20.0, -5.0, -20.0, -35.0, 30.0], [-30.0, 0.0, -25.0, 25.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 1.0, 7.0], [1.0, 0.0, 1.0]])+(1.-msk_arg0)*numpy.array([[3.0, 4.0, -5.0], [-1.0, 3.0, 6.0]])
      arg1=Data(numpy.array([[[7.0, -3.0, 2.0, 4.0, 6.0], [-2.0, 5.0, -7.0, -6.0, 1.0], [3.0, -1.0, -5.0, -7.0, 6.0], [-3.0, -1.0, -4.0, -6.0, 1.0]], [[-1.0, 1.0, -6.0, -5.0, -4.0], [5.0, -3.0, -2.0, -7.0, 0.0], [-7.0, 0.0, 4.0, 5.0, 6.0], [-5.0, 4.0, 4.0, -3.0, -6.0]], [[-6.0, 1.0, 6.0, 6.0, 0.0], [5.0, 7.0, -3.0, -3.0, 7.0], [-4.0, -2.0, 3.0, -3.0, -7.0], [3.0, 2.0, -1.0, -4.0, -2.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-85.0, 26.0, 24.0, 13.0, -40.0], [52.0, 16.0, 19.0, 8.0, 43.0], [-53.0, -8.0, 55.0, 26.0, -79.0], [34.0, 24.0, 21.0, 5.0, -26.0]], [[1.0, -2.0, 8.0, 10.0, 6.0], [3.0, 12.0, -10.0, -9.0, 8.0], [-1.0, -3.0, -2.0, -10.0, -1.0], [0.0, 1.0, -5.0, -10.0, -1.0]]])+(1.-msk_ref)*numpy.array([[[47.0, -10.0, -48.0, -38.0, 2.0], [-11.0, -32.0, -14.0, -31.0, -32.0], [1.0, 7.0, -14.0, 14.0, 77.0], [-44.0, 3.0, 9.0, -10.0, -11.0]], [[-46.0, 12.0, 16.0, 17.0, -18.0], [47.0, 28.0, -17.0, -33.0, 41.0], [-48.0, -11.0, 35.0, 4.0, -30.0], [6.0, 25.0, 10.0, -27.0, -31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -2.0, -7.0], [-5.0, -3.0, -4.0]], [[6.0, -4.0, -1.0], [-5.0, 2.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 4.0, 3.0], [-1.0, 5.0, -7.0]], [[-2.0, 1.0, -7.0], [2.0, 0.0, 0.0]]])
      arg1=Data(numpy.array([[[[0.0, 4.0, -4.0, 0.0, -4.0], [6.0, -5.0, -3.0, 5.0, 0.0], [1.0, 0.0, -5.0, 3.0, 4.0], [-6.0, 6.0, -2.0, -2.0, -2.0]], [[2.0, 5.0, 5.0, -1.0, 0.0], [-5.0, -5.0, 4.0, -3.0, -1.0], [0.0, -5.0, 5.0, 0.0, -2.0], [0.0, 0.0, 7.0, 3.0, -1.0]], [[-5.0, 1.0, -6.0, -6.0, -3.0], [-1.0, 4.0, -5.0, -3.0, -4.0], [-1.0, 4.0, 6.0, 3.0, 3.0], [2.0, -1.0, 0.0, 7.0, 5.0]]], [[[4.0, 1.0, 1.0, -6.0, 0.0], [-5.0, 1.0, 1.0, 1.0, 6.0], [-7.0, 6.0, -1.0, 0.0, -7.0], [-6.0, 0.0, -2.0, 4.0, 0.0]], [[1.0, -2.0, 2.0, -4.0, -2.0], [5.0, -6.0, 7.0, -2.0, -7.0], [0.0, 6.0, 7.0, 4.0, 6.0], [6.0, 6.0, 4.0, -7.0, -7.0]], [[6.0, 4.0, -2.0, -1.0, -1.0], [7.0, 4.0, 3.0, -2.0, 5.0], [0.0, 0.0, -4.0, -4.0, 4.0], [0.0, -5.0, -3.0, -4.0, 3.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-16.0, -32.0, 29.0, 90.0, 31.0], [-1.0, -21.0, -11.0, 36.0, 1.0], [42.0, -66.0, -52.0, -17.0, -16.0], [-2.0, 9.0, -4.0, -38.0, -24.0]], [[-21.0, -6.0, -39.0, 32.0, -25.0], [92.0, -31.0, -20.0, 36.0, -36.0], [42.0, -2.0, -37.0, 23.0, 76.0], [4.0, 49.0, -22.0, -65.0, -27.0]]])+(1.-msk_ref)*numpy.array([[[-48.0, -24.0, 33.0, -29.0, -4.0], [-54.0, -57.0, 20.0, -28.0, -92.0], [2.0, 16.0, 112.0, 51.0, 2.0], [54.0, 50.0, 75.0, 26.0, -41.0]], [[45.0, -8.0, 57.0, 29.0, 29.0], [-20.0, -21.0, 47.0, 10.0, 39.0], [-9.0, -21.0, -29.0, -27.0, -45.0], [-14.0, -5.0, 7.0, -34.0, -32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, 3.0])+(1.-msk_arg0)*numpy.array([6.0, -1.0])
      arg1=Data(numpy.array([[[5.0, 2.0], [2.0, 5.0]], [[-7.0, -4.0], [-4.0, 4.0]], [[4.0, -1.0], [2.0, -5.0]], [[-5.0, -1.0], [0.0, 2.0]], [[-5.0, 4.0], [-7.0, -2.0]], [[2.0, -2.0], [7.0, -4.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[15.0, 6.0], [6.0, 15.0]], [[-21.0, -12.0], [-12.0, 12.0]], [[12.0, -3.0], [6.0, -15.0]], [[-15.0, -3.0], [0.0, 6.0]], [[-15.0, 12.0], [-21.0, -6.0]], [[6.0, -6.0], [21.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[30.0, 12.0], [12.0, 30.0]], [[-42.0, -24.0], [-24.0, 24.0]], [[24.0, -6.0], [12.0, -30.0]], [[-30.0, -6.0], [0.0, 12.0]], [[-30.0, 24.0], [-42.0, -12.0]], [[12.0, -12.0], [42.0, -24.0]]], [[[-5.0, -2.0], [-2.0, -5.0]], [[7.0, 4.0], [4.0, -4.0]], [[-4.0, 1.0], [-2.0, 5.0]], [[5.0, 1.0], [0.0, -2.0]], [[5.0, -4.0], [7.0, 2.0]], [[-2.0, 2.0], [-7.0, 4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 6.0, -6.0], [-3.0, 5.0, 2.0]])+(1.-msk_arg0)*numpy.array([[5.0, 3.0, 6.0], [3.0, 6.0, 3.0]])
      arg1=Data(numpy.array([[[[4.0, 6.0], [6.0, 1.0]], [[1.0, 2.0], [-4.0, 0.0]], [[6.0, -2.0], [-5.0, -5.0]], [[-4.0, 2.0], [3.0, -1.0]], [[2.0, -1.0], [2.0, -1.0]], [[0.0, 0.0], [0.0, 4.0]]], [[[-3.0, 3.0], [-1.0, -5.0]], [[-3.0, 0.0], [-4.0, 2.0]], [[3.0, -6.0], [1.0, -6.0]], [[7.0, 5.0], [2.0, -7.0]], [[6.0, -1.0], [-2.0, 4.0]], [[3.0, -7.0], [-5.0, -2.0]]], [[[-3.0, -7.0], [2.0, -5.0]], [[3.0, 2.0], [-7.0, 4.0]], [[6.0, 1.0], [-6.0, 4.0]], [[-1.0, -5.0], [4.0, -3.0]], [[0.0, -7.0], [-1.0, -6.0]], [[1.0, -5.0], [0.0, 4.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[16.0, 84.0], [6.0, 4.0]], [[-32.0, -4.0], [2.0, -12.0]], [[6.0, -50.0], [22.0, -80.0]], [[32.0, 68.0], [0.0, -28.0]], [[44.0, 32.0], [2.0, 56.0]], [[12.0, -12.0], [-30.0, -20.0]]], [[[-33.0, -17.0], [-19.0, -38.0]], [[-12.0, -2.0], [-22.0, 18.0]], [[9.0, -22.0], [8.0, -7.0]], [[45.0, 9.0], [9.0, -38.0]], [[24.0, -16.0], [-18.0, 11.0]], [[17.0, -45.0], [-25.0, -14.0]]]])+(1.-msk_ref)*numpy.array([[[[-7.0, -3.0], [39.0, -40.0]], [[14.0, 22.0], [-74.0, 30.0]], [[75.0, -22.0], [-58.0, -19.0]], [[-5.0, -5.0], [45.0, -44.0]], [[28.0, -50.0], [-2.0, -29.0]], [[15.0, -51.0], [-15.0, 38.0]]], [[[-15.0, 15.0], [18.0, -42.0]], [[-6.0, 12.0], [-57.0, 24.0]], [[54.0, -39.0], [-27.0, -39.0]], [[27.0, 21.0], [33.0, -54.0]], [[42.0, -30.0], [-9.0, 3.0]], [[21.0, -57.0], [-30.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([4.0, 0.0])+(1.-msk_arg0)*numpy.array([1.0, 1.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-2.0)+(1-msk_arg1)*(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-8.0, -0.0])+(1.-msk_ref)*numpy.array([0.0, 0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 4.0, -5.0], [-3.0, -5.0, 6.0]])+(1.-msk_arg0)*numpy.array([[2.0, 5.0, -4.0], [3.0, -3.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-7.0, -3.0, 3.0])+(1.-msk_arg1)*numpy.array([-7.0, 1.0, -3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, 54.0])+(1.-msk_ref)*numpy.array([3.0, -30.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 1.0, -7.0], [5.0, -4.0, 4.0]], [[3.0, 7.0, 2.0], [-1.0, 5.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 5.0, -5.0], [5.0, -1.0, 0.0]], [[-4.0, 6.0, 1.0], [-2.0, 3.0, -1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 2.0, 1.0], [2.0, -4.0, 6.0]])+(1.-msk_arg1)*numpy.array([[7.0, -1.0, 3.0], [-6.0, -2.0, 4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([43.0, 3.0])+(1.-msk_ref)*numpy.array([-55.0, -29.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 6.0, -7.0], [7.0, 2.0, -7.0], [0.0, 1.0, 1.0], [-5.0, -3.0, 5.0]], [[0.0, 3.0, -7.0], [-2.0, -3.0, 4.0], [6.0, -6.0, 0.0], [-6.0, 4.0, 6.0]]], [[[0.0, -4.0, 4.0], [7.0, 0.0, 0.0], [7.0, 4.0, 3.0], [-5.0, -4.0, 3.0]], [[-5.0, -3.0, -2.0], [6.0, 0.0, -6.0], [7.0, 3.0, -2.0], [6.0, 0.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 4.0, 3.0], [-3.0, 4.0, -1.0], [5.0, -2.0, -7.0], [6.0, -7.0, 0.0]], [[0.0, 6.0, -6.0], [-6.0, -1.0, 0.0], [-4.0, -4.0, 2.0], [3.0, -2.0, 6.0]]], [[[-7.0, -5.0, 5.0], [-6.0, -2.0, 5.0], [-3.0, -3.0, 6.0], [3.0, -7.0, -3.0]], [[-2.0, 6.0, -5.0], [-7.0, -3.0, 6.0], [-7.0, 7.0, 7.0], [1.0, -3.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, 0.0, 6.0], [2.0, 3.0, -6.0], [-6.0, 5.0, 7.0], [5.0, -5.0, -5.0]], [[0.0, 6.0, -3.0], [4.0, -5.0, 1.0], [-5.0, -6.0, -6.0], [5.0, 4.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 3.0, 0.0], [-2.0, 1.0, 0.0], [-6.0, 0.0, 0.0], [7.0, 3.0, -2.0]], [[1.0, -4.0, -4.0], [6.0, 5.0, 5.0], [0.0, -4.0, 0.0], [7.0, -5.0, 4.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([17.0, 12.0])+(1.-msk_ref)*numpy.array([61.0, -41.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, -3.0])+(1.-msk_arg0)*numpy.array([1.0, 7.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -4.0])+(1.-msk_arg1)*numpy.array([1.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[42.0, -28.0], [-18.0, 12.0]])+(1.-msk_ref)*numpy.array([[1.0, 0.0], [7.0, 0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, 0.0, -5.0], [6.0, -2.0, 7.0]])+(1.-msk_arg0)*numpy.array([[0.0, 6.0, -5.0], [3.0, -2.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, -1.0], [4.0, 7.0], [2.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-3.0, -4.0], [2.0, 5.0], [0.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, 31.0], [-6.0, -62.0]])+(1.-msk_ref)*numpy.array([[12.0, 30.0], [-13.0, -22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, -1.0, 7.0], [2.0, 1.0, 3.0]], [[0.0, 0.0, -1.0], [2.0, 0.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 3.0, 4.0], [-4.0, 4.0, 0.0]], [[-3.0, 3.0, 4.0], [-1.0, -3.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, -3.0], [2.0, 0.0], [7.0, -3.0]], [[6.0, 1.0], [-1.0, -6.0], [-5.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, -4.0], [-3.0, 1.0], [4.0, 5.0]], [[-4.0, -5.0], [7.0, -5.0], [0.0, -6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, -40.0], [-20.0, 10.0]])+(1.-msk_ref)*numpy.array([[51.0, 51.0], [-10.0, 55.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, -5.0, -5.0], [-1.0, 2.0, 1.0], [4.0, -6.0, 1.0], [-5.0, 0.0, -2.0]], [[-1.0, 6.0, -1.0], [-4.0, 0.0, -3.0], [2.0, 4.0, 6.0], [-3.0, 4.0, -1.0]]], [[[0.0, 1.0, 6.0], [-1.0, -3.0, 2.0], [6.0, 4.0, 5.0], [1.0, 6.0, 0.0]], [[0.0, 2.0, 0.0], [-4.0, 0.0, 1.0], [7.0, -6.0, -6.0], [1.0, 7.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 2.0, 7.0], [-5.0, 0.0, -4.0], [4.0, -1.0, 1.0], [3.0, -5.0, 3.0]], [[-7.0, 0.0, -3.0], [-4.0, -5.0, -6.0], [0.0, -4.0, 1.0], [-3.0, -5.0, -1.0]]], [[[-4.0, -2.0, 6.0], [2.0, -5.0, 0.0], [4.0, -5.0, 4.0], [5.0, -1.0, 0.0]], [[-6.0, 0.0, -2.0], [-7.0, 3.0, -4.0], [-1.0, -1.0, -6.0], [4.0, 2.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, -3.0], [-5.0, 6.0], [-2.0, -1.0]], [[-3.0, 1.0], [-5.0, -5.0], [-1.0, 5.0]], [[-5.0, 6.0], [3.0, -3.0], [-6.0, -7.0]], [[6.0, 2.0], [-4.0, -6.0], [3.0, -7.0]]], [[[-5.0, 0.0], [0.0, -5.0], [-2.0, -3.0]], [[0.0, 5.0], [1.0, -5.0], [7.0, -4.0]], [[2.0, 0.0], [2.0, -1.0], [-7.0, -5.0]], [[-3.0, -7.0], [-1.0, 5.0], [2.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -5.0], [7.0, 7.0], [5.0, 7.0]], [[-6.0, 0.0], [6.0, 4.0], [-5.0, -4.0]], [[-1.0, -1.0], [0.0, -7.0], [0.0, 6.0]], [[2.0, 0.0], [2.0, 0.0], [-5.0, -6.0]]], [[[-2.0, 3.0], [-2.0, -1.0], [-2.0, 0.0]], [[0.0, -1.0], [2.0, 4.0], [-3.0, -6.0]], [[-1.0, -1.0], [-5.0, 1.0], [2.0, -7.0]], [[4.0, 2.0], [3.0, 0.0], [-1.0, 1.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-115.0, -35.0], [-36.0, 39.0]])+(1.-msk_ref)*numpy.array([[76.0, 81.0], [7.0, 163.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, -4.0])+(1.-msk_arg0)*numpy.array([1.0, 6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 6.0, -1.0, -7.0, 0.0], [6.0, -2.0, 0.0, 5.0, 5.0], [-3.0, 4.0, -6.0, -7.0, 0.0], [-4.0, -3.0, 5.0, -2.0, 3.0]])+(1.-msk_arg1)*numpy.array([[1.0, 2.0, -5.0, -7.0, -3.0], [0.0, -3.0, 1.0, 3.0, -1.0], [-5.0, 7.0, -1.0, 7.0, -5.0], [6.0, 5.0, 6.0, -1.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[21.0, -42.0, 7.0, 49.0, 0.0], [-42.0, 14.0, 0.0, -35.0, -35.0], [21.0, -28.0, 42.0, 49.0, 0.0], [28.0, 21.0, -35.0, 14.0, -21.0]], [[12.0, -24.0, 4.0, 28.0, 0.0], [-24.0, 8.0, 0.0, -20.0, -20.0], [12.0, -16.0, 24.0, 28.0, 0.0], [16.0, 12.0, -20.0, 8.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[1.0, 2.0, -5.0, -7.0, -3.0], [0.0, -3.0, 1.0, 3.0, -1.0], [-5.0, 7.0, -1.0, 7.0, -5.0], [6.0, 5.0, 6.0, -1.0, 0.0]], [[6.0, 12.0, -30.0, -42.0, -18.0], [0.0, -18.0, 6.0, 18.0, -6.0], [-30.0, 42.0, -6.0, 42.0, -30.0], [36.0, 30.0, 36.0, -6.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 1.0, -4.0], [-1.0, 5.0, -4.0]])+(1.-msk_arg0)*numpy.array([[5.0, -4.0, 3.0], [-4.0, -3.0, -5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 3.0, 0.0, -4.0, -3.0], [7.0, -4.0, 0.0, 0.0, -5.0], [-7.0, 7.0, 1.0, 7.0, 5.0], [-6.0, 5.0, 1.0, -6.0, 0.0]], [[1.0, -7.0, 3.0, -4.0, -7.0], [-4.0, -5.0, 4.0, 1.0, 2.0], [6.0, -1.0, -4.0, -1.0, 2.0], [1.0, -3.0, 5.0, -7.0, 2.0]], [[3.0, -7.0, 0.0, -4.0, -1.0], [-4.0, -6.0, 7.0, 2.0, -7.0], [-3.0, 5.0, -7.0, 7.0, 0.0], [0.0, 3.0, -2.0, -2.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, -1.0, 2.0, 4.0, 4.0], [0.0, 2.0, 2.0, 6.0, -2.0], [-3.0, 6.0, 3.0, -3.0, -6.0], [7.0, -2.0, -4.0, 4.0, 1.0]], [[0.0, -7.0, 4.0, 2.0, -5.0], [7.0, -5.0, -5.0, -1.0, -4.0], [-1.0, 7.0, -1.0, 6.0, 7.0], [-7.0, 5.0, -2.0, -2.0, 6.0]], [[-3.0, 7.0, 4.0, 1.0, -1.0], [6.0, -6.0, -7.0, 0.0, -4.0], [5.0, 3.0, -3.0, -2.0, 5.0], [1.0, 7.0, -7.0, -6.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-11.0, 9.0, 3.0, 28.0, 9.0], [-16.0, 35.0, -24.0, -7.0, 50.0], [46.0, -49.0, 20.0, -57.0, -18.0], [25.0, -35.0, 9.0, 25.0, -22.0]], [[-7.0, -10.0, 15.0, 0.0, -28.0], [-11.0, 3.0, -8.0, -3.0, 43.0], [49.0, -32.0, 7.0, -40.0, 5.0], [11.0, -32.0, 32.0, -21.0, -14.0]]])+(1.-msk_ref)*numpy.array([[[-19.0, 44.0, 6.0, 15.0, 37.0], [-10.0, 12.0, 9.0, 34.0, -6.0], [4.0, 11.0, 10.0, -45.0, -43.0], [66.0, -9.0, -33.0, 10.0, -19.0]], [[23.0, -10.0, -40.0, -27.0, 4.0], [-51.0, 37.0, 42.0, -21.0, 40.0], [-10.0, -60.0, 6.0, 4.0, -22.0], [-12.0, -42.0, 57.0, 20.0, -22.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, 4.0, -3.0], [-6.0, 3.0, -1.0]], [[0.0, -3.0, -5.0], [-1.0, -6.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 1.0, 6.0], [-6.0, -6.0, -1.0]], [[-2.0, 6.0, -7.0], [-3.0, -6.0, -2.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, -6.0, -5.0, -7.0, 5.0], [2.0, -6.0, 4.0, -5.0, 5.0], [-1.0, -5.0, -5.0, 5.0, -1.0], [0.0, 4.0, -2.0, -3.0, -4.0]], [[-1.0, -2.0, -7.0, -1.0, -3.0], [6.0, -5.0, 0.0, -3.0, 3.0], [-4.0, 0.0, 3.0, 0.0, 5.0], [-7.0, -1.0, -6.0, 5.0, -2.0]], [[-2.0, 3.0, 0.0, -6.0, 4.0], [-3.0, -5.0, 0.0, 5.0, -4.0], [0.0, 5.0, 4.0, -7.0, 5.0], [7.0, -6.0, 0.0, -7.0, -4.0]]], [[[3.0, 7.0, 0.0, 1.0, 6.0], [2.0, 6.0, -4.0, 2.0, -5.0], [6.0, -4.0, 0.0, -2.0, 1.0], [-3.0, 2.0, -7.0, -3.0, 6.0]], [[6.0, 1.0, -6.0, 7.0, -5.0], [2.0, -5.0, 0.0, 3.0, 4.0], [2.0, 1.0, -5.0, 0.0, 5.0], [4.0, 0.0, -3.0, 2.0, 2.0]], [[5.0, -4.0, -3.0, 7.0, 4.0], [-4.0, -6.0, 4.0, 3.0, 4.0], [6.0, 5.0, 5.0, 3.0, -5.0], [0.0, 3.0, 6.0, 6.0, -1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, -5.0, 7.0, 7.0, -4.0], [0.0, -3.0, 0.0, -2.0, 1.0], [-5.0, -5.0, -7.0, 2.0, 1.0], [3.0, 0.0, -5.0, -1.0, -2.0]], [[-4.0, 5.0, -4.0, 1.0, 3.0], [6.0, 5.0, -6.0, -2.0, -3.0], [5.0, 0.0, -7.0, 3.0, 0.0], [0.0, -6.0, -5.0, 4.0, 0.0]], [[-2.0, -7.0, 6.0, -5.0, -2.0], [-3.0, 2.0, 5.0, 2.0, -5.0], [2.0, -3.0, 1.0, 1.0, 3.0], [-3.0, 5.0, -5.0, -5.0, -2.0]]], [[[1.0, 4.0, -6.0, 0.0, -7.0], [-7.0, 5.0, -7.0, 1.0, 2.0], [0.0, 0.0, 5.0, -3.0, -1.0], [5.0, 4.0, 0.0, -4.0, 0.0]], [[3.0, 0.0, 6.0, 5.0, 4.0], [-1.0, 7.0, 0.0, 2.0, -7.0], [5.0, -2.0, 6.0, 5.0, -4.0], [6.0, -1.0, -2.0, 4.0, 2.0]], [[-7.0, 0.0, -3.0, -1.0, -5.0], [2.0, 0.0, -5.0, -7.0, -4.0], [7.0, 7.0, -6.0, -6.0, 7.0], [1.0, 3.0, -4.0, 0.0, 2.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-24.0, -34.0, -28.0, 43.0, -94.0], [25.0, -32.0, 8.0, -18.0, 47.0], [-49.0, 22.0, -5.0, 15.0, 22.0], [-19.0, -13.0, 9.0, 68.0, -13.0]], [[-41.0, -10.0, 66.0, -31.0, 1.0], [-5.0, 82.0, -8.0, -45.0, -20.0], [-24.0, -42.0, -14.0, 28.0, -56.0], [-35.0, 22.0, 25.0, -7.0, 11.0]]])+(1.-msk_ref)*numpy.array([[[-25.0, -51.0, 21.0, -72.0, 22.0], [34.0, -49.0, 71.0, 3.0, -1.0], [-10.0, -3.0, -47.0, -1.0, 39.0], [-91.0, 3.0, -9.0, -24.0, -22.0]], [[-9.0, 77.0, -92.0, -1.0, 47.0], [80.0, -35.0, -40.0, -23.0, 59.0], [-18.0, 29.0, -74.0, -2.0, -10.0], [-38.0, -83.0, 35.0, 49.0, 2.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank1_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([6.0, -2.0])+(1.-msk_arg0)*numpy.array([-3.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 4.0], [3.0, 6.0]], [[-1.0, 3.0], [6.0, -7.0]], [[-7.0, 5.0], [-5.0, -6.0]], [[-6.0, 4.0], [5.0, 2.0]], [[-7.0, 0.0], [-2.0, -5.0]], [[-2.0, 5.0], [0.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, -6.0], [-7.0, 2.0]], [[3.0, 6.0], [2.0, 0.0]], [[-4.0, 4.0], [6.0, 1.0]], [[-3.0, -4.0], [-4.0, -4.0]], [[6.0, 7.0], [0.0, -4.0]], [[7.0, 0.0], [-1.0, -1.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, 24.0], [18.0, 36.0]], [[-6.0, 18.0], [36.0, -42.0]], [[-42.0, 30.0], [-30.0, -36.0]], [[-36.0, 24.0], [30.0, 12.0]], [[-42.0, 0.0], [-12.0, -30.0]], [[-12.0, 30.0], [0.0, 18.0]]], [[[-8.0, -8.0], [-6.0, -12.0]], [[2.0, -6.0], [-12.0, 14.0]], [[14.0, -10.0], [10.0, 12.0]], [[12.0, -8.0], [-10.0, -4.0]], [[14.0, 0.0], [4.0, 10.0]], [[4.0, -10.0], [0.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[21.0, 18.0], [21.0, -6.0]], [[-9.0, -18.0], [-6.0, 0.0]], [[12.0, -12.0], [-18.0, -3.0]], [[9.0, 12.0], [12.0, 12.0]], [[-18.0, -21.0], [0.0, 12.0]], [[-21.0, 0.0], [3.0, 3.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 2.0, 0.0], [-5.0, -1.0, 4.0]])+(1.-msk_arg0)*numpy.array([[1.0, 0.0, -1.0], [-6.0, -3.0, 0.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, -7.0], [-1.0, 7.0]], [[0.0, 0.0], [4.0, -4.0]], [[-4.0, -6.0], [-5.0, 4.0]], [[-1.0, -6.0], [-3.0, 1.0]], [[0.0, -3.0], [-4.0, 0.0]], [[-5.0, 6.0], [6.0, -4.0]]], [[[-7.0, 5.0], [0.0, 5.0]], [[3.0, -1.0], [7.0, -3.0]], [[-6.0, 5.0], [5.0, 0.0]], [[-4.0, 1.0], [-2.0, 5.0]], [[-5.0, 2.0], [-5.0, -3.0]], [[3.0, 7.0], [-3.0, -2.0]]], [[[6.0, -3.0], [6.0, -6.0]], [[-1.0, 4.0], [4.0, -2.0]], [[4.0, 5.0], [-2.0, -3.0]], [[0.0, -3.0], [3.0, 3.0]], [[-6.0, -2.0], [-4.0, -3.0]], [[5.0, 0.0], [2.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, 7.0], [-2.0, -2.0]], [[-2.0, 4.0], [0.0, -5.0]], [[5.0, 5.0], [5.0, -7.0]], [[-3.0, 0.0], [0.0, 0.0]], [[-3.0, -4.0], [-6.0, -5.0]], [[0.0, -5.0], [0.0, 7.0]]], [[[6.0, -7.0], [4.0, -5.0]], [[0.0, 1.0], [-2.0, 0.0]], [[-3.0, -2.0], [-1.0, -1.0]], [[-4.0, 1.0], [0.0, -4.0]], [[-7.0, -2.0], [-7.0, 1.0]], [[-2.0, 6.0], [-3.0, -4.0]]], [[[6.0, 0.0], [0.0, 2.0]], [[3.0, 6.0], [-2.0, -7.0]], [[-3.0, 1.0], [-3.0, 2.0]], [[0.0, 4.0], [-4.0, -5.0]], [[0.0, 4.0], [3.0, 7.0]], [[2.0, 4.0], [0.0, 6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-14.0, 10.0], [0.0, 10.0]], [[6.0, -2.0], [14.0, -6.0]], [[-12.0, 10.0], [10.0, 0.0]], [[-8.0, 2.0], [-4.0, 10.0]], [[-10.0, 4.0], [-10.0, -6.0]], [[6.0, 14.0], [-6.0, -4.0]]], [[[61.0, 18.0], [29.0, -64.0]], [[-7.0, 17.0], [-11.0, 15.0]], [[42.0, 45.0], [12.0, -32.0]], [[9.0, 17.0], [29.0, 2.0]], [[-19.0, 5.0], [9.0, -9.0]], [[42.0, -37.0], [-19.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[-11.0, 7.0], [-2.0, -4.0]], [[-5.0, -2.0], [2.0, 2.0]], [[8.0, 4.0], [8.0, -9.0]], [[-3.0, -4.0], [4.0, 5.0]], [[-3.0, -8.0], [-9.0, -12.0]], [[-2.0, -9.0], [0.0, 1.0]]], [[[12.0, -21.0], [0.0, 27.0]], [[12.0, -27.0], [6.0, 30.0]], [[-21.0, -24.0], [-27.0, 45.0]], [[30.0, -3.0], [0.0, 12.0]], [[39.0, 30.0], [57.0, 27.0]], [[6.0, 12.0], [9.0, -30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -3.0, -5.0, 5.0, 0.0], [-1.0, 3.0, 0.0, -3.0, 7.0], [0.0, 7.0, 0.0, -2.0, -3.0], [-5.0, 2.0, 5.0, 6.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-7.0, 2.0, -1.0, 0.0, 2.0], [-6.0, -4.0, -2.0, 3.0, -5.0], [0.0, -1.0, 6.0, 0.0, -1.0], [-5.0, -2.0, -7.0, -3.0, 7.0]])
      arg1=-7.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-21.0, 21.0, 35.0, -35.0, -0.0], [7.0, -21.0, -0.0, 21.0, -49.0], [-0.0, -49.0, -0.0, 14.0, 21.0], [35.0, -14.0, -35.0, -42.0, -35.0]])+(1.-msk_ref)*numpy.array([[49.0, -14.0, 7.0, -0.0, -14.0], [42.0, 28.0, 14.0, -21.0, 35.0], [-0.0, 7.0, -42.0, -0.0, 7.0], [35.0, 14.0, 49.0, 21.0, -49.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, -5.0, 7.0, 2.0, 7.0], [6.0, 1.0, 6.0, -7.0, -6.0], [-4.0, 4.0, -6.0, -7.0, -3.0], [7.0, -2.0, 4.0, 0.0, -5.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 3.0, 1.0, 5.0, -4.0], [-4.0, 6.0, -5.0, -5.0, -1.0], [-4.0, 6.0, 0.0, -2.0, 0.0], [0.0, -3.0, 6.0, 1.0, 7.0]])
      arg1=numpy.array(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, -0.0, -0.0], [0.0, -0.0, 0.0, 0.0, -0.0]])+(1.-msk_ref)*numpy.array([[-0.0, 0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0, -0.0], [-0.0, 0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 1.0, -5.0], [0.0, 5.0, 4.0], [0.0, -5.0, -3.0], [6.0, 7.0, 5.0], [0.0, -7.0, 5.0]], [[2.0, 0.0, -6.0], [-3.0, 2.0, 5.0], [0.0, 2.0, 5.0], [4.0, -6.0, 0.0], [-6.0, 0.0, 0.0]], [[-3.0, 2.0, -2.0], [-7.0, -4.0, -2.0], [5.0, 0.0, -1.0], [-3.0, -3.0, 6.0], [5.0, 7.0, 2.0]], [[-3.0, 0.0, -5.0], [0.0, 0.0, 4.0], [6.0, 2.0, -6.0], [0.0, -7.0, -3.0], [5.0, -3.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 5.0, -7.0], [-6.0, -1.0, 2.0], [-4.0, -6.0, 1.0], [-4.0, -3.0, 2.0], [-4.0, -3.0, -1.0]], [[-4.0, -5.0, 7.0], [2.0, -2.0, 0.0], [2.0, 3.0, -1.0], [-5.0, 5.0, -5.0], [0.0, 0.0, 1.0]], [[-4.0, -7.0, -1.0], [7.0, -5.0, 3.0], [0.0, 0.0, -5.0], [-1.0, 7.0, 6.0], [0.0, -6.0, -1.0]], [[3.0, 5.0, 0.0], [1.0, -5.0, 7.0], [-3.0, -6.0, 6.0], [0.0, -1.0, -1.0], [-4.0, 3.0, 2.0]]])
      arg1=numpy.array([0.0, 6.0, -7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[41.0, 2.0, -9.0, 7.0, -77.0], [42.0, -23.0, -23.0, -36.0, 0.0], [26.0, -10.0, 7.0, -60.0, 28.0], [35.0, -28.0, 54.0, -21.0, 17.0]])+(1.-msk_ref)*numpy.array([[79.0, -20.0, -43.0, -32.0, -11.0], [-79.0, -12.0, 25.0, 65.0, -7.0], [-35.0, -51.0, 35.0, 0.0, -29.0], [30.0, -79.0, -78.0, 1.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 1.0, 3.0], [2.0, -3.0, 4.0]], [[-6.0, 7.0, -4.0], [-4.0, 3.0, 5.0]], [[-3.0, -5.0, 3.0], [-3.0, 2.0, 3.0]], [[-6.0, 6.0, 3.0], [4.0, 3.0, 2.0]], [[-5.0, 6.0, 5.0], [0.0, -5.0, 6.0]]], [[[-4.0, 6.0, -4.0], [-2.0, -4.0, 0.0]], [[4.0, -1.0, -4.0], [7.0, -5.0, -3.0]], [[-1.0, 3.0, -6.0], [-3.0, -2.0, 4.0]], [[5.0, -3.0, -1.0], [-3.0, 2.0, -1.0]], [[0.0, -6.0, 1.0], [3.0, 5.0, 2.0]]], [[[-5.0, -3.0, -4.0], [4.0, 3.0, 0.0]], [[4.0, -1.0, -2.0], [-3.0, -1.0, 6.0]], [[2.0, -3.0, 5.0], [-4.0, 4.0, -4.0]], [[4.0, -6.0, 3.0], [-2.0, 4.0, -1.0]], [[-1.0, -6.0, 1.0], [-5.0, -6.0, 0.0]]], [[[-5.0, 3.0, 6.0], [2.0, 7.0, 2.0]], [[5.0, -2.0, 0.0], [0.0, -6.0, 5.0]], [[4.0, 3.0, 0.0], [5.0, 0.0, 2.0]], [[5.0, -4.0, -5.0], [-1.0, -4.0, 2.0]], [[1.0, -6.0, -4.0], [5.0, -4.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -5.0, -1.0], [5.0, -6.0, 6.0]], [[-4.0, 0.0, -7.0], [-7.0, 0.0, -1.0]], [[0.0, 5.0, 2.0], [-2.0, 5.0, 2.0]], [[-2.0, -5.0, 5.0], [-1.0, 7.0, -2.0]], [[4.0, 3.0, -7.0], [-6.0, 0.0, 3.0]]], [[[-4.0, -4.0, -3.0], [-4.0, -5.0, -7.0]], [[1.0, 2.0, 6.0], [1.0, 3.0, -5.0]], [[5.0, 0.0, 0.0], [5.0, 3.0, -4.0]], [[6.0, -7.0, -4.0], [-4.0, 7.0, 7.0]], [[6.0, 4.0, 7.0], [-2.0, 6.0, 4.0]]], [[[2.0, 3.0, 1.0], [0.0, 7.0, -6.0]], [[-1.0, 6.0, -6.0], [0.0, -5.0, 0.0]], [[6.0, -2.0, -3.0], [1.0, -2.0, -5.0]], [[4.0, 4.0, 7.0], [7.0, -5.0, -7.0]], [[4.0, 2.0, 0.0], [-1.0, -3.0, -2.0]]], [[[3.0, -4.0, 4.0], [0.0, 5.0, -2.0]], [[-5.0, 0.0, -5.0], [-5.0, 6.0, -5.0]], [[6.0, -3.0, 3.0], [-7.0, -6.0, 3.0]], [[-4.0, 2.0, 6.0], [-6.0, -2.0, -7.0]], [[-2.0, 5.0, -5.0], [-4.0, 0.0, -3.0]]]])
      arg1=numpy.array([[5.0, -6.0, -3.0], [-3.0, 5.0, 0.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-36.0, -33.0, 25.0, -72.0, -101.0], [-58.0, -8.0, -6.0, 65.0, 49.0], [8.0, 36.0, 45.0, 73.0, 13.0], [-32.0, 7.0, -13.0, 47.0, 18.0]])+(1.-msk_ref)*numpy.array([[-12.0, 22.0, -5.0, 43.0, 41.0], [0.0, -13.0, 25.0, 131.0, 21.0], [24.0, -48.0, 38.0, -71.0, -4.0], [52.0, 35.0, 30.0, -42.0, -13.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 1.0, 5.0, 7.0, 2.0], [0.0, 5.0, 5.0, 3.0, 2.0], [7.0, 0.0, 3.0, 0.0, 4.0], [-1.0, 5.0, -3.0, -6.0, -4.0]])+(1.-msk_arg0)*numpy.array([[2.0, 4.0, 3.0, -3.0, -1.0], [1.0, 6.0, 6.0, -2.0, -3.0], [-2.0, -4.0, 2.0, 0.0, 7.0], [-4.0, 5.0, -6.0, -6.0, 0.0]])
      arg1=numpy.array([6.0, -3.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[36.0, -18.0], [6.0, -3.0], [30.0, -15.0], [42.0, -21.0], [12.0, -6.0]], [[0.0, 0.0], [30.0, -15.0], [30.0, -15.0], [18.0, -9.0], [12.0, -6.0]], [[42.0, -21.0], [0.0, 0.0], [18.0, -9.0], [0.0, 0.0], [24.0, -12.0]], [[-6.0, 3.0], [30.0, -15.0], [-18.0, 9.0], [-36.0, 18.0], [-24.0, 12.0]]])+(1.-msk_ref)*numpy.array([[[12.0, -6.0], [24.0, -12.0], [18.0, -9.0], [-18.0, 9.0], [-6.0, 3.0]], [[6.0, -3.0], [36.0, -18.0], [36.0, -18.0], [-12.0, 6.0], [-18.0, 9.0]], [[-12.0, 6.0], [-24.0, 12.0], [12.0, -6.0], [0.0, 0.0], [42.0, -21.0]], [[-24.0, 12.0], [30.0, -15.0], [-36.0, 18.0], [-36.0, 18.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 1.0, -3.0], [-6.0, -3.0, -3.0], [-4.0, 4.0, -1.0], [-5.0, 3.0, -5.0], [-3.0, 1.0, 5.0]], [[-6.0, -3.0, 4.0], [2.0, -6.0, -1.0], [6.0, 5.0, 0.0], [-1.0, -2.0, 7.0], [1.0, -3.0, -3.0]], [[-5.0, -5.0, -1.0], [-3.0, 7.0, -7.0], [0.0, 1.0, 0.0], [-7.0, 4.0, 1.0], [-1.0, 7.0, -6.0]], [[-5.0, 1.0, -2.0], [-2.0, -7.0, -2.0], [7.0, 6.0, -6.0], [-4.0, 0.0, 3.0], [-6.0, -1.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, -4.0, 0.0], [-1.0, -5.0, -5.0], [-4.0, -1.0, 6.0], [3.0, -7.0, -7.0], [-6.0, 7.0, -2.0]], [[1.0, -3.0, -4.0], [1.0, -4.0, 1.0], [-5.0, 5.0, 7.0], [6.0, -7.0, -7.0], [-1.0, 2.0, 7.0]], [[7.0, -7.0, -4.0], [-5.0, -1.0, 3.0], [-7.0, 7.0, 3.0], [-6.0, -7.0, 6.0], [3.0, 0.0, 7.0]], [[-7.0, 0.0, -7.0], [0.0, -5.0, 0.0], [6.0, 6.0, 7.0], [5.0, 6.0, -5.0], [7.0, 5.0, -7.0]]])
      arg1=numpy.array([[-3.0, 5.0], [4.0, -1.0], [2.0, -7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[13.0, -5.0], [0.0, -6.0], [26.0, -17.0], [17.0, 7.0], [23.0, -51.0]], [[14.0, -55.0], [-32.0, 23.0], [2.0, 25.0], [9.0, -52.0], [-21.0, 29.0]], [[-7.0, -13.0], [23.0, 27.0], [4.0, -1.0], [39.0, -46.0], [19.0, 30.0]], [[15.0, -12.0], [-26.0, 11.0], [-9.0, 71.0], [18.0, -41.0], [16.0, -36.0]]])+(1.-msk_ref)*numpy.array([[[-25.0, 19.0], [-27.0, 35.0], [20.0, -61.0], [-51.0, 71.0], [42.0, -23.0]], [[-23.0, 36.0], [-17.0, 2.0], [49.0, -79.0], [-60.0, 86.0], [25.0, -56.0]], [[-57.0, 70.0], [17.0, -45.0], [55.0, -63.0], [2.0, -65.0], [5.0, -34.0]], [[7.0, 14.0], [-20.0, 5.0], [20.0, -25.0], [-1.0, 54.0], [-15.0, 79.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -4.0, 0.0], [-1.0, 5.0, -3.0]], [[-2.0, 2.0, 7.0], [6.0, 0.0, 7.0]], [[-3.0, 6.0, 0.0], [-3.0, 4.0, 0.0]], [[0.0, 0.0, -5.0], [-7.0, 6.0, 4.0]], [[-2.0, 5.0, 5.0], [4.0, -6.0, 6.0]]], [[[5.0, 1.0, 0.0], [-5.0, 7.0, -3.0]], [[5.0, 0.0, 5.0], [-3.0, 7.0, 6.0]], [[-3.0, -1.0, 1.0], [-5.0, 1.0, -4.0]], [[2.0, 6.0, -2.0], [-3.0, -6.0, -5.0]], [[5.0, 2.0, 2.0], [-6.0, 1.0, 3.0]]], [[[-2.0, 5.0, 0.0], [-7.0, 4.0, 0.0]], [[2.0, -6.0, 3.0], [4.0, 2.0, -1.0]], [[0.0, -4.0, 7.0], [4.0, -3.0, -3.0]], [[4.0, 4.0, -2.0], [1.0, -2.0, -7.0]], [[5.0, 1.0, 7.0], [0.0, 7.0, -7.0]]], [[[-4.0, 7.0, 6.0], [0.0, 2.0, 5.0]], [[3.0, -1.0, -5.0], [-7.0, 7.0, -3.0]], [[-5.0, 2.0, -5.0], [5.0, 7.0, 3.0]], [[-5.0, -4.0, 1.0], [-4.0, -1.0, -6.0]], [[3.0, -1.0, 2.0], [0.0, 3.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 0.0, -1.0], [6.0, 1.0, -1.0]], [[0.0, 1.0, 7.0], [5.0, 7.0, 0.0]], [[-1.0, 6.0, 6.0], [1.0, -6.0, 5.0]], [[-2.0, 0.0, 2.0], [0.0, -4.0, -7.0]], [[0.0, -6.0, 4.0], [7.0, 0.0, 2.0]]], [[[0.0, 7.0, -7.0], [6.0, -1.0, -5.0]], [[4.0, 1.0, 7.0], [0.0, 1.0, 2.0]], [[-4.0, 7.0, -6.0], [-7.0, -3.0, 3.0]], [[1.0, 6.0, 2.0], [5.0, -5.0, 2.0]], [[-3.0, 7.0, -2.0], [0.0, 5.0, -5.0]]], [[[1.0, -4.0, 4.0], [-1.0, -6.0, 7.0]], [[-1.0, 0.0, -5.0], [3.0, -1.0, 6.0]], [[-6.0, 2.0, -2.0], [7.0, 7.0, 0.0]], [[5.0, -6.0, -1.0], [-4.0, 0.0, 3.0]], [[4.0, -1.0, 0.0], [-3.0, 5.0, -4.0]]], [[[1.0, -1.0, -7.0], [0.0, -1.0, 7.0]], [[6.0, 3.0, 0.0], [-7.0, -2.0, -2.0]], [[0.0, -1.0, -5.0], [-5.0, -7.0, -1.0]], [[-5.0, 4.0, -1.0], [-7.0, 4.0, 0.0]], [[-6.0, 1.0, 6.0], [3.0, 4.0, 3.0]]]])
      arg1=numpy.array([[[3.0, 6.0], [2.0, 6.0], [2.0, 0.0]], [[-3.0, 4.0], [-5.0, -4.0], [1.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-21.0, -15.0], [1.0, 3.0], [-8.0, -10.0], [-15.0, -64.0], [38.0, 40.0]], [[-6.0, -3.0], [5.0, -28.0], [-3.0, -36.0], [48.0, 75.0], [39.0, 5.0]], [[5.0, -26.0], [-23.0, -13.0], [6.0, 13.0], [16.0, 81.0], [-11.0, 29.0]], [[9.0, -5.0], [-20.0, -35.0], [-68.0, -35.0], [-10.0, -48.0], [-1.0, -9.0]]])+(1.-msk_ref)*numpy.array([[[-47.0, -19.0], [-34.0, -2.0], [53.0, 43.0], [11.0, 25.0], [-23.0, -14.0]], [[-18.0, 85.0], [25.0, 20.0], [29.0, -7.0], [31.0, 76.0], [-29.0, 19.0]], [[43.0, -19.0], [-11.0, -8.0], [-74.0, -24.0], [16.0, -31.0], [-10.0, -2.0]], [[-1.0, -17.0], [53.0, 40.0], [37.0, 5.0], [-8.0, -50.0], [-30.0, -43.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -4.0, -7.0, -6.0, -2.0], [0.0, 6.0, -2.0, 7.0, 7.0], [-2.0, 4.0, 1.0, 0.0, 2.0], [-7.0, -4.0, -5.0, 2.0, 7.0]])+(1.-msk_arg0)*numpy.array([[7.0, -3.0, 5.0, -3.0, -2.0], [2.0, -1.0, -3.0, -5.0, 3.0], [5.0, 5.0, 7.0, 1.0, 7.0], [5.0, 7.0, -2.0, -5.0, -1.0]])
      arg1=numpy.array([[0.0, 1.0, 5.0, -6.0, -1.0], [-6.0, 0.0, 1.0, 5.0, 0.0], [-4.0, 1.0, 0.0, -2.0, 7.0], [0.0, -3.0, 7.0, 3.0, -4.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 3.0, 15.0, -18.0, -3.0], [-18.0, 0.0, 3.0, 15.0, 0.0], [-12.0, 3.0, 0.0, -6.0, 21.0], [0.0, -9.0, 21.0, 9.0, -12.0]], [[0.0, -4.0, -20.0, 24.0, 4.0], [24.0, 0.0, -4.0, -20.0, 0.0], [16.0, -4.0, 0.0, 8.0, -28.0], [0.0, 12.0, -28.0, -12.0, 16.0]], [[0.0, -7.0, -35.0, 42.0, 7.0], [42.0, 0.0, -7.0, -35.0, 0.0], [28.0, -7.0, 0.0, 14.0, -49.0], [0.0, 21.0, -49.0, -21.0, 28.0]], [[0.0, -6.0, -30.0, 36.0, 6.0], [36.0, 0.0, -6.0, -30.0, 0.0], [24.0, -6.0, 0.0, 12.0, -42.0], [0.0, 18.0, -42.0, -18.0, 24.0]], [[0.0, -2.0, -10.0, 12.0, 2.0], [12.0, 0.0, -2.0, -10.0, 0.0], [8.0, -2.0, 0.0, 4.0, -14.0], [0.0, 6.0, -14.0, -6.0, 8.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 6.0, 30.0, -36.0, -6.0], [-36.0, 0.0, 6.0, 30.0, 0.0], [-24.0, 6.0, 0.0, -12.0, 42.0], [0.0, -18.0, 42.0, 18.0, -24.0]], [[0.0, -2.0, -10.0, 12.0, 2.0], [12.0, 0.0, -2.0, -10.0, 0.0], [8.0, -2.0, 0.0, 4.0, -14.0], [0.0, 6.0, -14.0, -6.0, 8.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]]], [[[0.0, -2.0, -10.0, 12.0, 2.0], [12.0, 0.0, -2.0, -10.0, 0.0], [8.0, -2.0, 0.0, 4.0, -14.0], [0.0, 6.0, -14.0, -6.0, 8.0]], [[0.0, 4.0, 20.0, -24.0, -4.0], [-24.0, 0.0, 4.0, 20.0, 0.0], [-16.0, 4.0, 0.0, -8.0, 28.0], [0.0, -12.0, 28.0, 12.0, -16.0]], [[0.0, 1.0, 5.0, -6.0, -1.0], [-6.0, 0.0, 1.0, 5.0, 0.0], [-4.0, 1.0, 0.0, -2.0, 7.0], [0.0, -3.0, 7.0, 3.0, -4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 2.0, 10.0, -12.0, -2.0], [-12.0, 0.0, 2.0, 10.0, 0.0], [-8.0, 2.0, 0.0, -4.0, 14.0], [0.0, -6.0, 14.0, 6.0, -8.0]]], [[[0.0, -7.0, -35.0, 42.0, 7.0], [42.0, 0.0, -7.0, -35.0, 0.0], [28.0, -7.0, 0.0, 14.0, -49.0], [0.0, 21.0, -49.0, -21.0, 28.0]], [[0.0, -4.0, -20.0, 24.0, 4.0], [24.0, 0.0, -4.0, -20.0, 0.0], [16.0, -4.0, 0.0, 8.0, -28.0], [0.0, 12.0, -28.0, -12.0, 16.0]], [[0.0, -5.0, -25.0, 30.0, 5.0], [30.0, 0.0, -5.0, -25.0, 0.0], [20.0, -5.0, 0.0, 10.0, -35.0], [0.0, 15.0, -35.0, -15.0, 20.0]], [[0.0, 2.0, 10.0, -12.0, -2.0], [-12.0, 0.0, 2.0, 10.0, 0.0], [-8.0, 2.0, 0.0, -4.0, 14.0], [0.0, -6.0, 14.0, 6.0, -8.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]], [[0.0, -3.0, -15.0, 18.0, 3.0], [18.0, 0.0, -3.0, -15.0, 0.0], [12.0, -3.0, 0.0, 6.0, -21.0], [0.0, 9.0, -21.0, -9.0, 12.0]], [[0.0, 5.0, 25.0, -30.0, -5.0], [-30.0, 0.0, 5.0, 25.0, 0.0], [-20.0, 5.0, 0.0, -10.0, 35.0], [0.0, -15.0, 35.0, 15.0, -20.0]], [[0.0, -3.0, -15.0, 18.0, 3.0], [18.0, 0.0, -3.0, -15.0, 0.0], [12.0, -3.0, 0.0, 6.0, -21.0], [0.0, 9.0, -21.0, -9.0, 12.0]], [[0.0, -2.0, -10.0, 12.0, 2.0], [12.0, 0.0, -2.0, -10.0, 0.0], [8.0, -2.0, 0.0, 4.0, -14.0], [0.0, 6.0, -14.0, -6.0, 8.0]]], [[[0.0, 2.0, 10.0, -12.0, -2.0], [-12.0, 0.0, 2.0, 10.0, 0.0], [-8.0, 2.0, 0.0, -4.0, 14.0], [0.0, -6.0, 14.0, 6.0, -8.0]], [[0.0, -1.0, -5.0, 6.0, 1.0], [6.0, 0.0, -1.0, -5.0, 0.0], [4.0, -1.0, 0.0, 2.0, -7.0], [0.0, 3.0, -7.0, -3.0, 4.0]], [[0.0, -3.0, -15.0, 18.0, 3.0], [18.0, 0.0, -3.0, -15.0, 0.0], [12.0, -3.0, 0.0, 6.0, -21.0], [0.0, 9.0, -21.0, -9.0, 12.0]], [[0.0, -5.0, -25.0, 30.0, 5.0], [30.0, 0.0, -5.0, -25.0, 0.0], [20.0, -5.0, 0.0, 10.0, -35.0], [0.0, 15.0, -35.0, -15.0, 20.0]], [[0.0, 3.0, 15.0, -18.0, -3.0], [-18.0, 0.0, 3.0, 15.0, 0.0], [-12.0, 3.0, 0.0, -6.0, 21.0], [0.0, -9.0, 21.0, 9.0, -12.0]]], [[[0.0, 5.0, 25.0, -30.0, -5.0], [-30.0, 0.0, 5.0, 25.0, 0.0], [-20.0, 5.0, 0.0, -10.0, 35.0], [0.0, -15.0, 35.0, 15.0, -20.0]], [[0.0, 5.0, 25.0, -30.0, -5.0], [-30.0, 0.0, 5.0, 25.0, 0.0], [-20.0, 5.0, 0.0, -10.0, 35.0], [0.0, -15.0, 35.0, 15.0, -20.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]], [[0.0, 1.0, 5.0, -6.0, -1.0], [-6.0, 0.0, 1.0, 5.0, 0.0], [-4.0, 1.0, 0.0, -2.0, 7.0], [0.0, -3.0, 7.0, 3.0, -4.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]]], [[[0.0, 5.0, 25.0, -30.0, -5.0], [-30.0, 0.0, 5.0, 25.0, 0.0], [-20.0, 5.0, 0.0, -10.0, 35.0], [0.0, -15.0, 35.0, 15.0, -20.0]], [[0.0, 7.0, 35.0, -42.0, -7.0], [-42.0, 0.0, 7.0, 35.0, 0.0], [-28.0, 7.0, 0.0, -14.0, 49.0], [0.0, -21.0, 49.0, 21.0, -28.0]], [[0.0, -2.0, -10.0, 12.0, 2.0], [12.0, 0.0, -2.0, -10.0, 0.0], [8.0, -2.0, 0.0, 4.0, -14.0], [0.0, 6.0, -14.0, -6.0, 8.0]], [[0.0, -5.0, -25.0, 30.0, 5.0], [30.0, 0.0, -5.0, -25.0, 0.0], [20.0, -5.0, 0.0, 10.0, -35.0], [0.0, 15.0, -35.0, -15.0, 20.0]], [[0.0, -1.0, -5.0, 6.0, 1.0], [6.0, 0.0, -1.0, -5.0, 0.0], [4.0, -1.0, 0.0, 2.0, -7.0], [0.0, 3.0, -7.0, -3.0, 4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, -5.0, 2.0], [-2.0, 2.0, -3.0], [1.0, 0.0, -5.0], [-3.0, 4.0, -7.0], [-5.0, 0.0, -3.0]], [[0.0, -6.0, -5.0], [4.0, 3.0, -4.0], [-1.0, 7.0, 0.0], [-2.0, 1.0, 7.0], [-3.0, 4.0, -4.0]], [[-3.0, -5.0, -5.0], [0.0, -1.0, -5.0], [-7.0, 7.0, -7.0], [-5.0, -4.0, -6.0], [6.0, 1.0, 3.0]], [[4.0, 7.0, -6.0], [3.0, 3.0, 1.0], [1.0, 3.0, 1.0], [7.0, 4.0, 4.0], [0.0, -7.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 5.0, -5.0], [-6.0, -6.0, 3.0], [7.0, -7.0, -4.0], [3.0, -1.0, -3.0], [-4.0, -1.0, 1.0]], [[3.0, -5.0, 0.0], [-4.0, -6.0, 7.0], [5.0, 3.0, -7.0], [3.0, 6.0, 6.0], [4.0, -7.0, -1.0]], [[-6.0, -3.0, -3.0], [-3.0, 2.0, -4.0], [5.0, -3.0, -3.0], [-2.0, 2.0, -3.0], [-7.0, 0.0, 1.0]], [[0.0, 2.0, 2.0], [-4.0, -4.0, 2.0], [-7.0, 5.0, 5.0], [-1.0, 5.0, 0.0], [0.0, -3.0, -6.0]]])
      arg1=numpy.array([[[-5.0, 3.0, 0.0, -1.0, -7.0], [7.0, -7.0, -5.0, 2.0, 2.0], [-6.0, -5.0, -1.0, -2.0, 7.0], [-4.0, -4.0, 7.0, 0.0, -5.0]], [[2.0, 0.0, -2.0, 0.0, -1.0], [2.0, 4.0, 3.0, 2.0, -2.0], [-6.0, -1.0, 3.0, 3.0, 4.0], [2.0, -7.0, 4.0, 1.0, -1.0]], [[0.0, -4.0, -5.0, 5.0, -5.0], [-3.0, -2.0, 0.0, 5.0, -1.0], [0.0, 1.0, -5.0, 4.0, 0.0], [-6.0, 5.0, 3.0, 3.0, 6.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-30.0, 4.0, 0.0, 6.0, -33.0], [12.0, -52.0, -35.0, 8.0, 16.0], [6.0, -13.0, -29.0, -15.0, 8.0], [-38.0, 29.0, 14.0, 1.0, -3.0]], [[14.0, 6.0, 11.0, -13.0, 27.0], [-1.0, 28.0, 16.0, -15.0, -5.0], [0.0, 5.0, 23.0, -2.0, -6.0], [30.0, -21.0, -15.0, -7.0, -10.0]], [[-5.0, 23.0, 25.0, -26.0, 18.0], [22.0, 3.0, -5.0, -23.0, 7.0], [-6.0, -10.0, 24.0, -22.0, 7.0], [26.0, -29.0, -8.0, -15.0, -35.0]], [[23.0, 19.0, 27.0, -32.0, 52.0], [8.0, 51.0, 27.0, -33.0, -7.0], [-6.0, 4.0, 50.0, -10.0, -5.0], [62.0, -51.0, -26.0, -17.0, -31.0]], [[25.0, -3.0, 15.0, -10.0, 50.0], [-26.0, 41.0, 25.0, -25.0, -7.0], [30.0, 22.0, 20.0, -2.0, -35.0], [38.0, 5.0, -44.0, -9.0, 7.0]]], [[[-12.0, 20.0, 37.0, -25.0, 31.0], [3.0, -14.0, -18.0, -37.0, 17.0], [36.0, 1.0, 7.0, -38.0, -24.0], [18.0, 17.0, -39.0, -21.0, -24.0]], [[-14.0, 28.0, 14.0, -24.0, -11.0], [46.0, -8.0, -11.0, -6.0, 6.0], [-42.0, -27.0, 25.0, -15.0, 40.0], [14.0, -57.0, 28.0, -9.0, -47.0]], [[19.0, -3.0, -14.0, 1.0, 0.0], [7.0, 35.0, 26.0, 12.0, -16.0], [-36.0, -2.0, 22.0, 23.0, 21.0], [18.0, -45.0, 21.0, 7.0, -2.0]], [[12.0, -34.0, -37.0, 37.0, -22.0], [-33.0, 4.0, 13.0, 33.0, -13.0], [6.0, 16.0, -30.0, 35.0, -10.0], [-32.0, 36.0, 11.0, 22.0, 51.0]], [[23.0, 7.0, 12.0, -17.0, 37.0], [-1.0, 45.0, 27.0, -18.0, -10.0], [-6.0, 7.0, 35.0, 2.0, -5.0], [44.0, -36.0, -17.0, -8.0, -13.0]]], [[[5.0, 11.0, 35.0, -22.0, 51.0], [-16.0, 11.0, 0.0, -41.0, 9.0], [48.0, 15.0, 13.0, -29.0, -41.0], [32.0, 22.0, -56.0, -20.0, -10.0]], [[-2.0, 20.0, 27.0, -25.0, 26.0], [13.0, 6.0, -3.0, -27.0, 7.0], [6.0, -4.0, 22.0, -23.0, -4.0], [28.0, -18.0, -19.0, -16.0, -29.0]], [[49.0, 7.0, 21.0, -28.0, 77.0], [-14.0, 91.0, 56.0, -35.0, -21.0], [0.0, 21.0, 63.0, 7.0, -21.0], [84.0, -56.0, -42.0, -14.0, -14.0]], [[17.0, 9.0, 38.0, -25.0, 69.0], [-25.0, 31.0, 13.0, -48.0, 4.0], [54.0, 23.0, 23.0, -26.0, -51.0], [48.0, 18.0, -69.0, -22.0, -7.0]], [[-28.0, 6.0, -17.0, 9.0, -58.0], [35.0, -44.0, -27.0, 29.0, 7.0], [-42.0, -28.0, -18.0, 3.0, 46.0], [-40.0, -16.0, 55.0, 10.0, -13.0]]], [[[-6.0, 36.0, 16.0, -34.0, -5.0], [60.0, 12.0, 1.0, -8.0, 0.0], [-66.0, -33.0, 47.0, -11.0, 56.0], [34.0, -95.0, 38.0, -11.0, -63.0]], [[-9.0, 5.0, -11.0, 2.0, -29.0], [24.0, -11.0, -6.0, 17.0, -1.0], [-36.0, -17.0, 1.0, 7.0, 33.0], [-12.0, -28.0, 36.0, 6.0, -12.0]], [[1.0, -1.0, -11.0, 4.0, -15.0], [10.0, 3.0, 4.0, 13.0, -5.0], [-24.0, -7.0, 3.0, 11.0, 19.0], [-4.0, -20.0, 22.0, 6.0, -2.0]], [[-27.0, 5.0, -28.0, 13.0, -73.0], [45.0, -41.0, -23.0, 42.0, 2.0], [-66.0, -35.0, -15.0, 14.0, 65.0], [-44.0, -36.0, 77.0, 16.0, -15.0]], [[-14.0, 24.0, 44.0, -30.0, 37.0], [4.0, -16.0, -21.0, -44.0, 20.0], [42.0, 1.0, 9.0, -45.0, -28.0], [22.0, 19.0, -46.0, -25.0, -29.0]]]])+(1.-msk_ref)*numpy.array([[[[-25.0, 41.0, 15.0, -32.0, -29.0], [74.0, -19.0, -20.0, -1.0, 9.0], [-72.0, -45.0, 33.0, -19.0, 69.0], [12.0, -88.0, 54.0, -10.0, -70.0]], [[18.0, -30.0, -3.0, 21.0, 33.0], [-63.0, 12.0, 12.0, -9.0, -3.0], [72.0, 39.0, -27.0, 6.0, -66.0], [-6.0, 81.0, -57.0, 3.0, 54.0]], [[-49.0, 37.0, 34.0, -27.0, -22.0], [47.0, -69.0, -56.0, -20.0, 32.0], [0.0, -32.0, -8.0, -51.0, 21.0], [-18.0, 1.0, 9.0, -19.0, -52.0]], [[-17.0, 21.0, 17.0, -18.0, -5.0], [28.0, -19.0, -18.0, -11.0, 11.0], [-12.0, -17.0, 9.0, -21.0, 17.0], [4.0, -20.0, 8.0, -10.0, -32.0]], [[18.0, -16.0, -3.0, 9.0, 24.0], [-33.0, 22.0, 17.0, -5.0, -7.0], [30.0, 22.0, -4.0, 9.0, -32.0], [8.0, 28.0, -29.0, 2.0, 27.0]]], [[[-25.0, 9.0, 10.0, -3.0, -16.0], [11.0, -41.0, -30.0, -4.0, 16.0], [12.0, -10.0, -18.0, -21.0, 1.0], [-22.0, 23.0, 1.0, -5.0, -10.0]], [[8.0, -40.0, -23.0, 39.0, -1.0], [-61.0, -10.0, 2.0, 15.0, -3.0], [60.0, 33.0, -49.0, 18.0, -52.0], [-38.0, 93.0, -31.0, 15.0, 68.0]], [[-19.0, 43.0, 29.0, -40.0, -3.0], [62.0, -9.0, -16.0, -19.0, 11.0], [-48.0, -35.0, 39.0, -29.0, 47.0], [28.0, -76.0, 26.0, -18.0, -70.0]], [[-3.0, -15.0, -42.0, 27.0, -57.0], [15.0, -9.0, 3.0, 48.0, -12.0], [-54.0, -15.0, -15.0, 36.0, 45.0], [-36.0, -24.0, 63.0, 24.0, 15.0]], [[-34.0, 16.0, 19.0, -9.0, -16.0], [17.0, -54.0, -41.0, -11.0, 23.0], [18.0, -14.0, -20.0, -33.0, 0.0], [-24.0, 28.0, -3.0, -10.0, -19.0]]], [[[24.0, -6.0, 21.0, -9.0, 60.0], [-39.0, 36.0, 21.0, -33.0, -3.0], [54.0, 30.0, 12.0, -9.0, -54.0], [36.0, 30.0, -63.0, -12.0, 15.0]], [[19.0, 7.0, 16.0, -17.0, 39.0], [-5.0, 37.0, 21.0, -22.0, -6.0], [6.0, 9.0, 29.0, -4.0, -13.0], [40.0, -22.0, -25.0, -10.0, -11.0]], [[-31.0, 27.0, 21.0, -20.0, -17.0], [38.0, -41.0, -34.0, -11.0, 19.0], [-12.0, -25.0, 1.0, -31.0, 23.0], [-8.0, -14.0, 14.0, -12.0, -40.0]], [[14.0, 6.0, 11.0, -13.0, 27.0], [-1.0, 28.0, 16.0, -15.0, -5.0], [0.0, 5.0, 23.0, -2.0, -6.0], [30.0, -21.0, -15.0, -7.0, -10.0]], [[35.0, -25.0, -5.0, 12.0, 44.0], [-52.0, 47.0, 35.0, -9.0, -15.0], [42.0, 36.0, 2.0, 18.0, -49.0], [22.0, 33.0, -46.0, 3.0, 41.0]]], [[[4.0, -8.0, -14.0, 10.0, -12.0], [-2.0, 4.0, 6.0, 14.0, -6.0], [-12.0, 0.0, -4.0, 14.0, 8.0], [-8.0, -4.0, 14.0, 8.0, 10.0]], [[12.0, -20.0, -2.0, 14.0, 22.0], [-42.0, 8.0, 8.0, -6.0, -2.0], [48.0, 26.0, -18.0, 4.0, -44.0], [-4.0, 54.0, -38.0, 2.0, 36.0]], [[45.0, -41.0, -35.0, 32.0, 19.0], [-54.0, 59.0, 50.0, 21.0, -29.0], [12.0, 35.0, -3.0, 49.0, -29.0], [8.0, 18.0, -14.0, 20.0, 60.0]], [[15.0, -3.0, -10.0, 1.0, 2.0], [3.0, 27.0, 20.0, 8.0, -12.0], [-24.0, 0.0, 16.0, 17.0, 13.0], [14.0, -31.0, 13.0, 5.0, 0.0]], [[-6.0, 24.0, 36.0, -30.0, 33.0], [12.0, 0.0, -9.0, -36.0, 12.0], [18.0, -3.0, 21.0, -33.0, -12.0], [30.0, -9.0, -30.0, -21.0, -33.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, -7.0, 6.0], [-3.0, -3.0, -2.0]], [[-3.0, -5.0, 0.0], [1.0, 5.0, 4.0]], [[-6.0, 7.0, -7.0], [0.0, 4.0, -7.0]], [[1.0, 0.0, 0.0], [6.0, 3.0, 5.0]], [[6.0, 0.0, -4.0], [6.0, -1.0, -6.0]]], [[[-5.0, 6.0, -5.0], [1.0, -6.0, 2.0]], [[-5.0, -2.0, -6.0], [-2.0, 2.0, -1.0]], [[-7.0, -4.0, -1.0], [4.0, -1.0, 3.0]], [[6.0, -2.0, 0.0], [5.0, 0.0, 3.0]], [[-1.0, 7.0, 0.0], [0.0, 0.0, 4.0]]], [[[-4.0, 4.0, -6.0], [-6.0, -7.0, 0.0]], [[1.0, 3.0, 1.0], [4.0, -6.0, -3.0]], [[3.0, 0.0, 7.0], [0.0, 2.0, -3.0]], [[-2.0, 1.0, 6.0], [-6.0, 3.0, 0.0]], [[-3.0, 4.0, 4.0], [0.0, 3.0, 7.0]]], [[[0.0, 2.0, -7.0], [-2.0, 6.0, 5.0]], [[6.0, 3.0, -6.0], [2.0, -7.0, 6.0]], [[6.0, 4.0, -7.0], [-7.0, -1.0, -2.0]], [[-2.0, 7.0, 6.0], [-2.0, 1.0, 0.0]], [[7.0, 7.0, 2.0], [-1.0, 1.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 6.0, 0.0], [-5.0, 0.0, 5.0]], [[-5.0, -4.0, 3.0], [1.0, -4.0, -5.0]], [[4.0, 2.0, 3.0], [2.0, 3.0, 6.0]], [[5.0, -6.0, -7.0], [0.0, -4.0, 0.0]], [[-5.0, 0.0, 4.0], [7.0, 5.0, 4.0]]], [[[-2.0, -2.0, 3.0], [3.0, -5.0, -2.0]], [[-1.0, 5.0, 0.0], [1.0, 0.0, 4.0]], [[1.0, 7.0, 1.0], [0.0, 2.0, 4.0]], [[0.0, -4.0, -6.0], [2.0, -6.0, 4.0]], [[-2.0, -3.0, 6.0], [-4.0, -4.0, 0.0]]], [[[-2.0, -6.0, -4.0], [2.0, -6.0, -4.0]], [[-6.0, -3.0, -3.0], [0.0, 0.0, -7.0]], [[-5.0, -3.0, 4.0], [-1.0, 4.0, -1.0]], [[-5.0, -5.0, 0.0], [1.0, -1.0, -1.0]], [[0.0, -3.0, -6.0], [5.0, -4.0, -5.0]]], [[[7.0, 5.0, 0.0], [1.0, -4.0, 6.0]], [[0.0, -5.0, -4.0], [4.0, -6.0, -5.0]], [[-7.0, 1.0, 0.0], [7.0, 2.0, 6.0]], [[4.0, 0.0, 6.0], [-3.0, 3.0, 6.0]], [[0.0, 1.0, -2.0], [2.0, 4.0, 5.0]]]])
      arg1=numpy.array([[[[4.0, -6.0, -7.0, 2.0, 4.0], [6.0, 7.0, 0.0, 1.0, 0.0], [-3.0, -6.0, -5.0, 0.0, -6.0], [4.0, -1.0, 3.0, 6.0, -2.0]], [[4.0, 2.0, -3.0, 5.0, 7.0], [0.0, -2.0, -6.0, 4.0, 5.0], [5.0, -1.0, -4.0, 5.0, -6.0], [4.0, -3.0, -7.0, 3.0, -7.0]], [[3.0, -3.0, -1.0, 0.0, -7.0], [-1.0, -1.0, 1.0, 5.0, 2.0], [-2.0, -7.0, 7.0, 2.0, 2.0], [7.0, -7.0, 5.0, -5.0, 7.0]]], [[[2.0, 5.0, 7.0, -4.0, 1.0], [7.0, -7.0, -6.0, 6.0, -1.0], [4.0, 2.0, -3.0, 5.0, -3.0], [2.0, 2.0, 7.0, -5.0, -1.0]], [[-1.0, 2.0, 3.0, -1.0, -6.0], [0.0, 7.0, -3.0, 5.0, 0.0], [5.0, 7.0, 0.0, -6.0, 2.0], [-7.0, 4.0, -1.0, 6.0, -2.0]], [[-2.0, 7.0, 6.0, 4.0, -5.0], [6.0, 6.0, -6.0, -5.0, -6.0], [-5.0, 6.0, -4.0, -7.0, -2.0], [0.0, -1.0, 0.0, 1.0, 4.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-5.0, -73.0, -34.0, -26.0, -62.0], [-33.0, 3.0, 87.0, -20.0, -8.0], [-67.0, -80.0, 82.0, -6.0, 55.0], [33.0, -38.0, 64.0, -50.0, 90.0]], [[-43.0, 51.0, 82.0, -24.0, -96.0], [13.0, 41.0, -15.0, -12.0, -50.0], [-7.0, 84.0, 16.0, -78.0, 47.0], [-65.0, 36.0, 28.0, -4.0, 46.0]], [[-7.0, 30.0, -2.0, -9.0, 85.0], [-71.0, -63.0, -19.0, 42.0, 63.0], [122.0, 64.0, -19.0, 46.0, 2.0], [-73.0, 57.0, -106.0, 37.0, -122.0]], [[3.0, 65.0, 74.0, -5.0, -33.0], [78.0, 16.0, -75.0, 27.0, -36.0], [11.0, 57.0, -43.0, -23.0, -28.0], [-5.0, 18.0, 42.0, -1.0, 6.0]], [[37.0, -38.0, -35.0, -35.0, 94.0], [46.0, -39.0, -1.0, 47.0, 22.0], [39.0, -39.0, -52.0, 70.0, -52.0], [15.0, 36.0, 41.0, 14.0, -68.0]]], [[[-7.0, 64.0, 23.0, 30.0, 84.0], [-6.0, -79.0, -41.0, -40.0, 7.0], [19.0, 31.0, -45.0, 47.0, -35.0], [13.0, -2.0, -69.0, -26.0, -48.0]], [[-50.0, 31.0, 33.0, -18.0, -1.0], [-44.0, -3.0, 18.0, -40.0, -14.0], [24.0, 78.0, 1.0, -37.0, 42.0], [-88.0, 58.0, -47.0, 15.0, -24.0]], [[-44.0, 76.0, 105.0, -37.0, -54.0], [5.0, -57.0, -16.0, -24.0, -44.0], [-1.0, 72.0, 20.0, -17.0, 44.0], [-36.0, 27.0, 31.0, -72.0, 45.0]], [[20.0, 6.0, 17.0, -6.0, 0.0], [89.0, 29.0, -36.0, 13.0, -33.0], [-23.0, -6.0, -49.0, -6.0, -45.0], [26.0, 7.0, 67.0, 8.0, 9.0]], [[16.0, 48.0, 10.0, 49.0, 25.0], [18.0, 3.0, -66.0, 7.0, 11.0], [18.0, 23.0, -39.0, 7.0, -44.0], [24.0, -24.0, -52.0, 19.0, -31.0]]], [[[-23.0, 6.0, -41.0, 43.0, 90.0], [-60.0, -37.0, 27.0, -89.0, 14.0], [-15.0, 1.0, -20.0, 20.0, -8.0], [-5.0, -6.0, -105.0, 6.0, -42.0]], [[39.0, -16.0, -25.0, -5.0, 73.0], [15.0, -88.0, -5.0, 27.0, 31.0], [11.0, -68.0, -10.0, 94.0, -40.0], [73.0, -30.0, 21.0, -49.0, -20.0]], [[37.0, -56.0, -40.0, -8.0, -34.0], [-7.0, 10.0, 19.0, 63.0, 32.0], [2.0, -71.0, 46.0, 23.0, 6.0], [47.0, -41.0, 42.0, -8.0, 27.0]], [[-1.0, -28.0, -28.0, 22.0, -67.0], [-60.0, 41.0, 27.0, 11.0, 23.0], [-10.0, -22.0, 66.0, -31.0, 42.0], [5.0, -43.0, -28.0, 9.0, 39.0]], [[-1.0, 69.0, 56.0, 39.0, -65.0], [20.0, 30.0, -71.0, 13.0, -14.0], [1.0, 49.0, -1.0, -39.0, -6.0], [11.0, -32.0, -20.0, -1.0, 28.0]]], [[[-33.0, 62.0, 35.0, 32.0, 0.0], [23.0, 89.0, -55.0, -34.0, -32.0], [21.0, 115.0, -71.0, -85.0, -18.0], [-87.0, 58.0, -69.0, 92.0, -53.0]], [[17.0, 26.0, -16.0, 50.0, 101.0], [92.0, 15.0, -51.0, -65.0, -35.0], [-48.0, -6.0, -114.0, 13.0, -98.0], [47.0, -3.0, -12.0, 29.0, -39.0]], [[10.0, -58.0, -111.0, 53.0, 110.0], [-18.0, 71.0, 26.0, -50.0, 25.0], [-7.0, -24.0, -66.0, -9.0, -51.0], [-16.0, 15.0, -93.0, 110.0, -88.0]], [[33.0, 0.0, -24.0, 38.0, -9.0], [-32.0, -13.0, -27.0, 49.0, 49.0], [26.0, -34.0, 30.0, 31.0, -10.0], [51.0, -61.0, -40.0, -5.0, -3.0]], [[67.0, -65.0, -100.0, 36.0, 76.0], [9.0, 23.0, -13.0, 64.0, 64.0], [31.0, -82.0, -30.0, 56.0, -67.0], [61.0, -36.0, -26.0, 60.0, -66.0]]]])+(1.-msk_ref)*numpy.array([[[[28.0, -14.0, -65.0, 82.0, 36.0], [31.0, 95.0, -36.0, -25.0, 5.0], [-33.0, -22.0, -59.0, -30.0, -67.0], [38.0, -39.0, -59.0, 84.0, -29.0]], [[-11.0, -25.0, 9.0, -50.0, -19.0], [-56.0, -95.0, 63.0, 5.0, 15.0], [-2.0, -43.0, 79.0, 50.0, 59.0], [15.0, -13.0, 39.0, -91.0, 46.0]], [[22.0, 29.0, 22.0, 31.0, -37.0], [71.0, 64.0, -66.0, 24.0, -22.0], [-15.0, 14.0, -37.0, -34.0, -42.0], [28.0, -21.0, 24.0, 29.0, 15.0]], [[-21.0, -29.0, -22.0, -16.0, 51.0], [37.0, 26.0, 41.0, -74.0, -44.0], [-51.0, -3.0, -50.0, -20.0, -16.0], [-25.0, 46.0, 26.0, 23.0, -9.0]], [[-7.0, 91.0, 119.0, -27.0, -91.0], [39.0, -29.0, -77.0, 62.0, -23.0], [40.0, 75.0, 16.0, -15.0, 19.0], [-13.0, 7.0, 49.0, -51.0, 37.0]]], [[[8.0, -10.0, 11.0, -29.0, 0.0], [-6.0, -81.0, 24.0, 8.0, 5.0], [-13.0, -48.0, 38.0, 55.0, 15.0], [46.0, -25.0, 49.0, -80.0, 38.0]], [[10.0, 49.0, 23.0, 35.0, 12.0], [25.0, 0.0, -60.0, 5.0, 0.0], [12.0, 27.0, -34.0, 2.0, -35.0], [18.0, -16.0, -31.0, 8.0, -18.0]], [[25.0, 37.0, 1.0, 51.0, 14.0], [29.0, 30.0, -71.0, 24.0, 13.0], [20.0, 18.0, -42.0, -3.0, -50.0], [25.0, -25.0, -43.0, 38.0, -32.0]], [[-32.0, 36.0, 38.0, -6.0, 32.0], [44.0, -18.0, 0.0, -84.0, -58.0], [-50.0, 32.0, -48.0, -14.0, -14.0], [-12.0, 30.0, 18.0, -24.0, 12.0]], [[-6.0, -40.0, -23.0, 1.0, -51.0], [-46.0, -14.0, 60.0, -28.0, 1.0], [-57.0, -63.0, 76.0, 1.0, 46.0], [42.0, -55.0, 21.0, -55.0, 79.0]]], [[[-26.0, -18.0, 8.0, -52.0, 36.0], [-18.0, -78.0, 62.0, -44.0, -16.0], [-18.0, -16.0, 16.0, 36.0, 30.0], [-14.0, 32.0, 36.0, -60.0, 12.0]], [[-31.0, -10.0, 12.0, -55.0, 11.0], [-75.0, -75.0, 57.0, 2.0, 21.0], [44.0, 18.0, 49.0, 28.0, 62.0], [-57.0, 43.0, -12.0, -37.0, -16.0]], [[-24.0, 8.0, 39.0, -29.0, -89.0], [-47.0, -4.0, 22.0, 22.0, 0.0], [13.0, 25.0, 72.0, -29.0, 69.0], [-34.0, 1.0, 15.0, -31.0, 48.0]], [[-35.0, 16.0, 48.0, -42.0, -43.0], [-29.0, -45.0, 33.0, -19.0, -20.0], [-6.0, 24.0, 46.0, -7.0, 57.0], [-31.0, 19.0, 28.0, -57.0, 42.0]], [[-6.0, -6.0, 8.0, -51.0, 75.0], [11.0, -81.0, 24.0, -7.0, -2.0], [22.0, -3.0, -25.0, 57.0, -7.0], [-16.0, 50.0, 30.0, -33.0, -38.0]]], [[[42.0, 7.0, -33.0, 63.0, 58.0], [85.0, 40.0, -60.0, -17.0, -12.0], [-42.0, -37.0, -82.0, 12.0, -95.0], [78.0, -42.0, -3.0, 34.0, -18.0]], [[-8.0, -25.0, -1.0, -55.0, 58.0], [2.0, -86.0, 50.0, -21.0, -7.0], [-6.0, -31.0, 0.0, 58.0, 8.0], [2.0, 32.0, 49.0, -56.0, -5.0]], [[-24.0, 125.0, 137.0, -15.0, -56.0], [43.0, -50.0, -90.0, 19.0, -38.0], [34.0, 105.0, -14.0, -14.0, 7.0], [-24.0, 20.0, 19.0, -56.0, 20.0]], [[13.0, -9.0, -10.0, 41.0, -77.0], [33.0, 100.0, -21.0, 1.0, -21.0], [-51.0, -15.0, 7.0, -63.0, -9.0], [31.0, -46.0, 18.0, 33.0, 55.0]], [[-12.0, 61.0, 55.0, 13.0, -26.0], [46.0, 44.0, -62.0, 1.0, -31.0], [12.0, 75.0, -44.0, -48.0, -18.0], [-34.0, 26.0, -7.0, 32.0, -11.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, 6.0, -2.0, -2.0, 7.0], [6.0, 4.0, -6.0, 3.0, -1.0], [-3.0, 3.0, -4.0, 0.0, 3.0], [-3.0, 5.0, 3.0, -1.0, 0.0]])+(1.-msk_arg0)*numpy.array([[5.0, -6.0, -1.0, -2.0, -1.0], [6.0, -7.0, -5.0, -4.0, 3.0], [1.0, -4.0, -7.0, 6.0, 5.0], [3.0, -1.0, -7.0, -2.0, -7.0]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-15.0, 18.0, -6.0, -6.0, 21.0], [18.0, 12.0, -18.0, 9.0, -3.0], [-9.0, 9.0, -12.0, 0.0, 9.0], [-9.0, 15.0, 9.0, -3.0, 0.0]])+(1.-msk_ref)*numpy.array([[15.0, -18.0, -3.0, -6.0, -3.0], [18.0, -21.0, -15.0, -12.0, 9.0], [3.0, -12.0, -21.0, 18.0, 15.0], [9.0, -3.0, -21.0, -6.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 3.0, 2.0], [0.0, 3.0, 7.0], [-4.0, 2.0, 0.0], [4.0, 7.0, 3.0], [6.0, 2.0, 5.0]], [[-3.0, 1.0, 6.0], [5.0, -2.0, 5.0], [0.0, 4.0, -5.0], [-1.0, -7.0, 5.0], [-3.0, 6.0, -2.0]], [[1.0, 0.0, -1.0], [5.0, 3.0, -4.0], [4.0, 7.0, -5.0], [-1.0, 4.0, -3.0], [4.0, -5.0, 4.0]], [[6.0, 1.0, -3.0], [-4.0, 0.0, -3.0], [0.0, 6.0, 5.0], [-4.0, 2.0, -7.0], [2.0, -3.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 0.0, -7.0], [7.0, -1.0, 2.0], [-7.0, -4.0, 4.0], [7.0, 7.0, -5.0], [3.0, -5.0, 7.0]], [[-6.0, 6.0, -6.0], [2.0, -4.0, 5.0], [0.0, 0.0, -3.0], [6.0, 0.0, -1.0], [1.0, -2.0, 3.0]], [[3.0, -7.0, 7.0], [-7.0, -1.0, -6.0], [5.0, 6.0, -1.0], [-3.0, -3.0, -4.0], [-2.0, -4.0, 2.0]], [[7.0, 6.0, 2.0], [5.0, 7.0, 7.0], [-5.0, -6.0, 0.0], [-1.0, 6.0, -3.0], [-2.0, -7.0, 0.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-4.0, 1.0, 6.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[11.0, 45.0, 18.0, 9.0, 8.0], [49.0, 8.0, -26.0, 27.0, 6.0], [-10.0, -41.0, -39.0, -10.0, 3.0], [-41.0, -2.0, 36.0, -24.0, -11.0]])+(1.-msk_ref)*numpy.array([[-50.0, -17.0, 48.0, -51.0, 25.0], [-6.0, 18.0, -18.0, -30.0, 12.0], [23.0, -9.0, -20.0, -15.0, 16.0], [-10.0, 29.0, 14.0, -8.0, 1.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 0.0, -2.0], [-2.0, -7.0, -6.0]], [[-7.0, -6.0, 3.0], [-2.0, 0.0, 0.0]], [[2.0, -4.0, -5.0], [-4.0, -4.0, -7.0]], [[7.0, 3.0, 1.0], [7.0, -3.0, -2.0]], [[3.0, -1.0, 3.0], [4.0, -2.0, -1.0]]], [[[-1.0, 2.0, 1.0], [7.0, 0.0, 0.0]], [[4.0, -6.0, -4.0], [5.0, -6.0, 0.0]], [[-5.0, 0.0, 7.0], [6.0, 1.0, -6.0]], [[2.0, -3.0, -2.0], [1.0, 5.0, 5.0]], [[-4.0, -5.0, 0.0], [6.0, 6.0, 4.0]]], [[[-5.0, 1.0, -4.0], [1.0, 5.0, 1.0]], [[6.0, 2.0, 7.0], [-1.0, -2.0, -3.0]], [[-6.0, 1.0, -4.0], [4.0, -1.0, -3.0]], [[0.0, -1.0, 5.0], [-5.0, 1.0, 2.0]], [[7.0, 0.0, 4.0], [7.0, 3.0, 4.0]]], [[[5.0, -1.0, 6.0], [1.0, 0.0, -1.0]], [[0.0, -1.0, 6.0], [-2.0, -4.0, 0.0]], [[0.0, -7.0, -7.0], [5.0, -4.0, 1.0]], [[-6.0, -3.0, -3.0], [-5.0, 2.0, 0.0]], [[2.0, 1.0, 0.0], [-1.0, 5.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 3.0, -2.0], [1.0, 0.0, 0.0]], [[0.0, 4.0, -2.0], [5.0, 7.0, 3.0]], [[4.0, 5.0, -7.0], [1.0, 7.0, -4.0]], [[7.0, 7.0, 1.0], [2.0, 4.0, -3.0]], [[2.0, 7.0, 6.0], [6.0, 2.0, -6.0]]], [[[7.0, -1.0, 2.0], [-6.0, 1.0, 6.0]], [[-3.0, 5.0, -5.0], [6.0, 4.0, 6.0]], [[0.0, 5.0, -3.0], [5.0, -4.0, -6.0]], [[3.0, 0.0, 4.0], [7.0, 7.0, -2.0]], [[-7.0, 3.0, 6.0], [7.0, 2.0, 6.0]]], [[[-5.0, -5.0, 0.0], [0.0, 6.0, 7.0]], [[2.0, 7.0, -7.0], [3.0, -4.0, -6.0]], [[-6.0, 0.0, 7.0], [-4.0, -4.0, -4.0]], [[-7.0, 0.0, 0.0], [-1.0, 0.0, 7.0]], [[0.0, 7.0, -1.0], [-3.0, -1.0, -4.0]]], [[[-6.0, 3.0, 6.0], [0.0, 2.0, -5.0]], [[2.0, -4.0, -4.0], [-2.0, -2.0, 7.0]], [[-2.0, -4.0, -3.0], [-1.0, -1.0, -2.0]], [[7.0, -1.0, -4.0], [3.0, -5.0, -3.0]], [[1.0, 5.0, 3.0], [2.0, 0.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-6.0, 0.0, 5.0], [-4.0, 1.0, -7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[75.0, 65.0, 24.0, -54.0, -14.0], [-17.0, -70.0, 84.0, -56.0, -22.0], [4.0, 22.0, 20.0, 32.0, -75.0], [3.0, 34.0, -66.0, 43.0, -24.0]])+(1.-msk_ref)*numpy.array([[-14.0, -44.0, -28.0, -20.0, 38.0], [-49.0, -69.0, 3.0, -5.0, 4.0], [-13.0, -21.0, 111.0, -3.0, 34.0], [103.0, -75.0, 14.0, -58.0, 1.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, -5.0, 4.0, 7.0, -2.0], [-2.0, 6.0, -7.0, 0.0, 6.0], [1.0, -2.0, -3.0, 1.0, -7.0], [-7.0, 0.0, 4.0, -4.0, 3.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -5.0, 5.0, 7.0, 5.0], [2.0, -3.0, -7.0, 0.0, 4.0], [-2.0, -2.0, -7.0, 7.0, -1.0], [-4.0, 4.0, -1.0, 3.0, 7.0]])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([1.0, -1.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-7.0, 7.0], [-5.0, 5.0], [4.0, -4.0], [7.0, -7.0], [-2.0, 2.0]], [[-2.0, 2.0], [6.0, -6.0], [-7.0, 7.0], [0.0, 0.0], [6.0, -6.0]], [[1.0, -1.0], [-2.0, 2.0], [-3.0, 3.0], [1.0, -1.0], [-7.0, 7.0]], [[-7.0, 7.0], [0.0, 0.0], [4.0, -4.0], [-4.0, 4.0], [3.0, -3.0]]])+(1.-msk_ref)*numpy.array([[[-5.0, 5.0], [-5.0, 5.0], [5.0, -5.0], [7.0, -7.0], [5.0, -5.0]], [[2.0, -2.0], [-3.0, 3.0], [-7.0, 7.0], [0.0, 0.0], [4.0, -4.0]], [[-2.0, 2.0], [-2.0, 2.0], [-7.0, 7.0], [7.0, -7.0], [-1.0, 1.0]], [[-4.0, 4.0], [4.0, -4.0], [-1.0, 1.0], [3.0, -3.0], [7.0, -7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, -1.0, -7.0], [-3.0, 4.0, -6.0], [2.0, -1.0, -3.0], [-5.0, 7.0, -6.0], [-4.0, -4.0, -6.0]], [[0.0, 7.0, -3.0], [3.0, -4.0, 3.0], [0.0, -7.0, -7.0], [6.0, -6.0, 5.0], [0.0, 0.0, -4.0]], [[6.0, 6.0, 6.0], [4.0, 4.0, 0.0], [3.0, -5.0, -5.0], [0.0, -5.0, -5.0], [4.0, 5.0, 1.0]], [[-2.0, -5.0, -2.0], [3.0, -4.0, 4.0], [0.0, -4.0, -3.0], [4.0, 4.0, 7.0], [7.0, 6.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, -3.0, 4.0], [-5.0, -2.0, -2.0], [0.0, 5.0, -1.0], [-7.0, -1.0, -7.0], [3.0, 6.0, 2.0]], [[3.0, -7.0, 3.0], [-2.0, -6.0, 7.0], [2.0, -3.0, 5.0], [-1.0, 3.0, 0.0], [-4.0, 4.0, 5.0]], [[-7.0, -3.0, -1.0], [2.0, -4.0, -3.0], [5.0, 1.0, 2.0], [-5.0, -4.0, 7.0], [-6.0, 6.0, 2.0]], [[-7.0, -1.0, -3.0], [7.0, 1.0, -7.0], [-2.0, -1.0, 6.0], [3.0, 4.0, 4.0], [0.0, -6.0, 3.0]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[2.0, -7.0], [0.0, 0.0], [2.0, -7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-24.0, 84.0], [-18.0, 63.0], [-2.0, 7.0], [-22.0, 77.0], [-20.0, 70.0]], [[-6.0, 21.0], [12.0, -42.0], [-14.0, 49.0], [22.0, -77.0], [-8.0, 28.0]], [[24.0, -84.0], [8.0, -28.0], [-4.0, 14.0], [-10.0, 35.0], [10.0, -35.0]], [[-8.0, 28.0], [14.0, -49.0], [-6.0, 21.0], [22.0, -77.0], [14.0, -49.0]]])+(1.-msk_ref)*numpy.array([[[10.0, -35.0], [-14.0, 49.0], [-2.0, 7.0], [-28.0, 98.0], [10.0, -35.0]], [[12.0, -42.0], [10.0, -35.0], [14.0, -49.0], [-2.0, 7.0], [2.0, -7.0]], [[-16.0, 56.0], [-2.0, 7.0], [14.0, -49.0], [4.0, -14.0], [-8.0, 28.0]], [[-20.0, 70.0], [0.0, 0.0], [8.0, -28.0], [14.0, -49.0], [6.0, -21.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, -2.0, -7.0], [6.0, 4.0, 7.0]], [[-3.0, 3.0, 5.0], [-6.0, 2.0, -4.0]], [[1.0, 7.0, -3.0], [2.0, -6.0, 4.0]], [[1.0, -7.0, -6.0], [0.0, 7.0, 2.0]], [[5.0, 0.0, -2.0], [6.0, -5.0, -1.0]]], [[[-3.0, 4.0, 4.0], [-1.0, -2.0, -1.0]], [[7.0, 4.0, 4.0], [2.0, 3.0, -5.0]], [[3.0, 4.0, 4.0], [5.0, 5.0, -6.0]], [[3.0, 6.0, -1.0], [-4.0, -7.0, 1.0]], [[-2.0, -1.0, 0.0], [-3.0, 0.0, -5.0]]], [[[-4.0, -5.0, 5.0], [3.0, 0.0, 0.0]], [[0.0, -6.0, 1.0], [-5.0, 0.0, -7.0]], [[-4.0, -6.0, 0.0], [-7.0, -1.0, 1.0]], [[6.0, -5.0, 0.0], [-1.0, -1.0, 2.0]], [[-7.0, -1.0, 6.0], [2.0, 0.0, 1.0]]], [[[-1.0, 0.0, 1.0], [6.0, -1.0, -1.0]], [[-3.0, 7.0, 0.0], [-7.0, 1.0, 6.0]], [[-2.0, -6.0, 1.0], [-2.0, -2.0, -6.0]], [[0.0, 0.0, 7.0], [4.0, -6.0, 1.0]], [[1.0, -7.0, -6.0], [-5.0, -5.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, -5.0, -7.0], [-7.0, 1.0, 0.0]], [[0.0, 7.0, -4.0], [5.0, -2.0, -6.0]], [[-6.0, 0.0, -6.0], [-6.0, -1.0, -4.0]], [[5.0, 3.0, 2.0], [-2.0, -4.0, -3.0]], [[6.0, -6.0, -1.0], [2.0, 4.0, 0.0]]], [[[1.0, 5.0, -4.0], [1.0, -4.0, 0.0]], [[-7.0, 6.0, -6.0], [1.0, 7.0, 0.0]], [[-1.0, 4.0, 3.0], [6.0, 4.0, 5.0]], [[1.0, -7.0, -5.0], [1.0, 0.0, 0.0]], [[5.0, -7.0, -5.0], [-4.0, 0.0, 5.0]]], [[[-3.0, -6.0, -6.0], [-6.0, -6.0, -7.0]], [[-1.0, 0.0, -2.0], [-1.0, 0.0, -6.0]], [[0.0, 7.0, 6.0], [-4.0, 4.0, -7.0]], [[4.0, 7.0, 0.0], [5.0, 4.0, -6.0]], [[-4.0, -5.0, -3.0], [5.0, -2.0, 0.0]]], [[[3.0, -4.0, -2.0], [0.0, 6.0, 5.0]], [[0.0, -1.0, -3.0], [-5.0, -2.0, 3.0]], [[4.0, -7.0, -5.0], [0.0, 3.0, 0.0]], [[6.0, 2.0, 3.0], [6.0, 4.0, 6.0]], [[2.0, -3.0, 4.0], [-4.0, 1.0, 1.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[5.0, -6.0], [0.0, -4.0], [0.0, -6.0]], [[-2.0, 3.0], [-1.0, 3.0], [7.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[13.0, 118.0], [-33.0, -44.0], [35.0, -20.0], [12.0, 83.0], [11.0, -17.0]], [[-18.0, -33.0], [-7.0, -77.0], [-42.0, -40.0], [37.0, -67.0], [-39.0, -3.0]], [[-26.0, 23.0], [-39.0, -11.0], [2.0, 26.0], [47.0, -18.0], [-32.0, 18.0]], [[-23.0, 13.0], [40.0, -16.0], [-46.0, 6.0], [5.0, -46.0], [27.0, 30.0]]])+(1.-msk_ref)*numpy.array([[[-2.0, 62.0], [-50.0, -7.0], [-45.0, 43.0], [12.0, -78.0], [22.0, 12.0]], [[7.0, -11.0], [-44.0, 78.0], [14.0, 12.0], [3.0, 55.0], [68.0, 26.0]], [[-46.0, 28.0], [-45.0, 3.0], [-45.0, -78.0], [-36.0, -37.0], [-28.0, 71.0]], [[44.0, 38.0], [33.0, 7.0], [17.0, 43.0], [56.0, -20.0], [24.0, -31.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -3.0, 5.0, 3.0, -1.0], [7.0, 3.0, 0.0, -7.0, -7.0], [-4.0, -5.0, 5.0, 7.0, 7.0], [6.0, -2.0, 7.0, 0.0, -1.0]])+(1.-msk_arg0)*numpy.array([[7.0, 4.0, 3.0, -3.0, 6.0], [2.0, -3.0, 0.0, -6.0, -6.0], [-3.0, -4.0, 4.0, -1.0, -2.0], [-5.0, 1.0, 1.0, 5.0, 4.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[0.0, -7.0, 0.0, -2.0, 4.0], [3.0, -2.0, 0.0, -5.0, 5.0], [7.0, 0.0, -6.0, 7.0, -3.0], [7.0, -1.0, 0.0, 4.0, 0.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 21.0, 0.0, 6.0, -12.0], [-9.0, 6.0, 0.0, 15.0, -15.0], [-21.0, 0.0, 18.0, -21.0, 9.0], [-21.0, 3.0, 0.0, -12.0, 0.0]], [[0.0, -35.0, 0.0, -10.0, 20.0], [15.0, -10.0, 0.0, -25.0, 25.0], [35.0, 0.0, -30.0, 35.0, -15.0], [35.0, -5.0, 0.0, 20.0, 0.0]], [[0.0, -21.0, 0.0, -6.0, 12.0], [9.0, -6.0, 0.0, -15.0, 15.0], [21.0, 0.0, -18.0, 21.0, -9.0], [21.0, -3.0, 0.0, 12.0, 0.0]], [[0.0, 7.0, 0.0, 2.0, -4.0], [-3.0, 2.0, 0.0, 5.0, -5.0], [-7.0, 0.0, 6.0, -7.0, 3.0], [-7.0, 1.0, 0.0, -4.0, 0.0]]], [[[0.0, -49.0, 0.0, -14.0, 28.0], [21.0, -14.0, 0.0, -35.0, 35.0], [49.0, 0.0, -42.0, 49.0, -21.0], [49.0, -7.0, 0.0, 28.0, 0.0]], [[0.0, -21.0, 0.0, -6.0, 12.0], [9.0, -6.0, 0.0, -15.0, 15.0], [21.0, 0.0, -18.0, 21.0, -9.0], [21.0, -3.0, 0.0, 12.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 49.0, 0.0, 14.0, -28.0], [-21.0, 14.0, 0.0, 35.0, -35.0], [-49.0, 0.0, 42.0, -49.0, 21.0], [-49.0, 7.0, 0.0, -28.0, 0.0]], [[0.0, 49.0, 0.0, 14.0, -28.0], [-21.0, 14.0, 0.0, 35.0, -35.0], [-49.0, 0.0, 42.0, -49.0, 21.0], [-49.0, 7.0, 0.0, -28.0, 0.0]]], [[[0.0, 28.0, 0.0, 8.0, -16.0], [-12.0, 8.0, 0.0, 20.0, -20.0], [-28.0, 0.0, 24.0, -28.0, 12.0], [-28.0, 4.0, 0.0, -16.0, 0.0]], [[0.0, 35.0, 0.0, 10.0, -20.0], [-15.0, 10.0, 0.0, 25.0, -25.0], [-35.0, 0.0, 30.0, -35.0, 15.0], [-35.0, 5.0, 0.0, -20.0, 0.0]], [[0.0, -35.0, 0.0, -10.0, 20.0], [15.0, -10.0, 0.0, -25.0, 25.0], [35.0, 0.0, -30.0, 35.0, -15.0], [35.0, -5.0, 0.0, 20.0, 0.0]], [[0.0, -49.0, 0.0, -14.0, 28.0], [21.0, -14.0, 0.0, -35.0, 35.0], [49.0, 0.0, -42.0, 49.0, -21.0], [49.0, -7.0, 0.0, 28.0, 0.0]], [[0.0, -49.0, 0.0, -14.0, 28.0], [21.0, -14.0, 0.0, -35.0, 35.0], [49.0, 0.0, -42.0, 49.0, -21.0], [49.0, -7.0, 0.0, 28.0, 0.0]]], [[[0.0, -42.0, 0.0, -12.0, 24.0], [18.0, -12.0, 0.0, -30.0, 30.0], [42.0, 0.0, -36.0, 42.0, -18.0], [42.0, -6.0, 0.0, 24.0, 0.0]], [[0.0, 14.0, 0.0, 4.0, -8.0], [-6.0, 4.0, 0.0, 10.0, -10.0], [-14.0, 0.0, 12.0, -14.0, 6.0], [-14.0, 2.0, 0.0, -8.0, 0.0]], [[0.0, -49.0, 0.0, -14.0, 28.0], [21.0, -14.0, 0.0, -35.0, 35.0], [49.0, 0.0, -42.0, 49.0, -21.0], [49.0, -7.0, 0.0, 28.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 7.0, 0.0, 2.0, -4.0], [-3.0, 2.0, 0.0, 5.0, -5.0], [-7.0, 0.0, 6.0, -7.0, 3.0], [-7.0, 1.0, 0.0, -4.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -49.0, 0.0, -14.0, 28.0], [21.0, -14.0, 0.0, -35.0, 35.0], [49.0, 0.0, -42.0, 49.0, -21.0], [49.0, -7.0, 0.0, 28.0, 0.0]], [[0.0, -28.0, 0.0, -8.0, 16.0], [12.0, -8.0, 0.0, -20.0, 20.0], [28.0, 0.0, -24.0, 28.0, -12.0], [28.0, -4.0, 0.0, 16.0, 0.0]], [[0.0, -21.0, 0.0, -6.0, 12.0], [9.0, -6.0, 0.0, -15.0, 15.0], [21.0, 0.0, -18.0, 21.0, -9.0], [21.0, -3.0, 0.0, 12.0, 0.0]], [[0.0, 21.0, 0.0, 6.0, -12.0], [-9.0, 6.0, 0.0, 15.0, -15.0], [-21.0, 0.0, 18.0, -21.0, 9.0], [-21.0, 3.0, 0.0, -12.0, 0.0]], [[0.0, -42.0, 0.0, -12.0, 24.0], [18.0, -12.0, 0.0, -30.0, 30.0], [42.0, 0.0, -36.0, 42.0, -18.0], [42.0, -6.0, 0.0, 24.0, 0.0]]], [[[0.0, -14.0, 0.0, -4.0, 8.0], [6.0, -4.0, 0.0, -10.0, 10.0], [14.0, 0.0, -12.0, 14.0, -6.0], [14.0, -2.0, 0.0, 8.0, 0.0]], [[0.0, 21.0, 0.0, 6.0, -12.0], [-9.0, 6.0, 0.0, 15.0, -15.0], [-21.0, 0.0, 18.0, -21.0, 9.0], [-21.0, 3.0, 0.0, -12.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 42.0, 0.0, 12.0, -24.0], [-18.0, 12.0, 0.0, 30.0, -30.0], [-42.0, 0.0, 36.0, -42.0, 18.0], [-42.0, 6.0, 0.0, -24.0, 0.0]], [[0.0, 42.0, 0.0, 12.0, -24.0], [-18.0, 12.0, 0.0, 30.0, -30.0], [-42.0, 0.0, 36.0, -42.0, 18.0], [-42.0, 6.0, 0.0, -24.0, 0.0]]], [[[0.0, 21.0, 0.0, 6.0, -12.0], [-9.0, 6.0, 0.0, 15.0, -15.0], [-21.0, 0.0, 18.0, -21.0, 9.0], [-21.0, 3.0, 0.0, -12.0, 0.0]], [[0.0, 28.0, 0.0, 8.0, -16.0], [-12.0, 8.0, 0.0, 20.0, -20.0], [-28.0, 0.0, 24.0, -28.0, 12.0], [-28.0, 4.0, 0.0, -16.0, 0.0]], [[0.0, -28.0, 0.0, -8.0, 16.0], [12.0, -8.0, 0.0, -20.0, 20.0], [28.0, 0.0, -24.0, 28.0, -12.0], [28.0, -4.0, 0.0, 16.0, 0.0]], [[0.0, 7.0, 0.0, 2.0, -4.0], [-3.0, 2.0, 0.0, 5.0, -5.0], [-7.0, 0.0, 6.0, -7.0, 3.0], [-7.0, 1.0, 0.0, -4.0, 0.0]], [[0.0, 14.0, 0.0, 4.0, -8.0], [-6.0, 4.0, 0.0, 10.0, -10.0], [-14.0, 0.0, 12.0, -14.0, 6.0], [-14.0, 2.0, 0.0, -8.0, 0.0]]], [[[0.0, 35.0, 0.0, 10.0, -20.0], [-15.0, 10.0, 0.0, 25.0, -25.0], [-35.0, 0.0, 30.0, -35.0, 15.0], [-35.0, 5.0, 0.0, -20.0, 0.0]], [[0.0, -7.0, 0.0, -2.0, 4.0], [3.0, -2.0, 0.0, -5.0, 5.0], [7.0, 0.0, -6.0, 7.0, -3.0], [7.0, -1.0, 0.0, 4.0, 0.0]], [[0.0, -7.0, 0.0, -2.0, 4.0], [3.0, -2.0, 0.0, -5.0, 5.0], [7.0, 0.0, -6.0, 7.0, -3.0], [7.0, -1.0, 0.0, 4.0, 0.0]], [[0.0, -35.0, 0.0, -10.0, 20.0], [15.0, -10.0, 0.0, -25.0, 25.0], [35.0, 0.0, -30.0, 35.0, -15.0], [35.0, -5.0, 0.0, 20.0, 0.0]], [[0.0, -28.0, 0.0, -8.0, 16.0], [12.0, -8.0, 0.0, -20.0, 20.0], [28.0, 0.0, -24.0, 28.0, -12.0], [28.0, -4.0, 0.0, 16.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, -6.0, 0.0], [0.0, -5.0, 2.0], [-7.0, 6.0, -4.0], [2.0, 6.0, -2.0], [-2.0, 1.0, 3.0]], [[7.0, 0.0, -1.0], [-5.0, 2.0, 0.0], [5.0, -5.0, -1.0], [4.0, 5.0, 7.0], [5.0, 7.0, -6.0]], [[3.0, 5.0, 5.0], [4.0, 2.0, 5.0], [1.0, -4.0, -7.0], [1.0, 0.0, 7.0], [0.0, 3.0, -7.0]], [[4.0, 1.0, -6.0], [0.0, -2.0, 3.0], [-2.0, 0.0, 6.0], [4.0, -3.0, 2.0], [-5.0, -2.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 5.0, -6.0], [2.0, -5.0, 1.0], [-1.0, 6.0, -4.0], [-7.0, -1.0, -7.0], [3.0, 7.0, -2.0]], [[1.0, 4.0, 6.0], [1.0, -1.0, -6.0], [-3.0, -6.0, 2.0], [-2.0, 0.0, -6.0], [-6.0, -4.0, 7.0]], [[7.0, -2.0, -4.0], [1.0, 1.0, -3.0], [1.0, -3.0, 1.0], [4.0, -7.0, -4.0], [-5.0, 4.0, -7.0]], [[6.0, 0.0, 0.0], [-5.0, 3.0, 2.0], [0.0, -4.0, -5.0], [-1.0, 2.0, -4.0], [-4.0, -3.0, 3.0]]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-7.0, 5.0, 4.0, -6.0, -1.0], [2.0, -5.0, -6.0, 2.0, 6.0], [4.0, 2.0, 7.0, 3.0, 6.0], [6.0, 1.0, -7.0, 5.0, -7.0]], [[-6.0, -6.0, 5.0, 0.0, -1.0], [-6.0, 6.0, -3.0, 2.0, -4.0], [-3.0, 4.0, -1.0, -6.0, -7.0], [-3.0, 3.0, 2.0, 0.0, 0.0]], [[-3.0, -2.0, 7.0, -2.0, 1.0], [7.0, -3.0, 2.0, -1.0, 7.0], [6.0, -7.0, -3.0, 1.0, -1.0], [-2.0, 4.0, 2.0, 7.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[43.0, 31.0, -34.0, 6.0, 7.0], [34.0, -31.0, 24.0, -14.0, 18.0], [14.0, -26.0, -1.0, 33.0, 36.0], [12.0, -19.0, -5.0, -5.0, 7.0]], [[24.0, 26.0, -11.0, -4.0, 7.0], [44.0, -36.0, 19.0, -12.0, 34.0], [27.0, -34.0, -1.0, 32.0, 33.0], [11.0, -7.0, -6.0, 14.0, 2.0]], [[25.0, -63.0, -26.0, 50.0, -3.0], [-78.0, 83.0, 16.0, 2.0, -94.0], [-70.0, 38.0, -43.0, -61.0, -80.0], [-52.0, -5.0, 53.0, -63.0, 45.0]], [[-44.0, -22.0, 24.0, -8.0, -10.0], [-46.0, 32.0, -34.0, 18.0, -26.0], [-22.0, 42.0, 14.0, -32.0, -28.0], [-2.0, 12.0, -6.0, -4.0, -16.0]], [[-1.0, -22.0, 18.0, 6.0, 4.0], [11.0, 7.0, 15.0, -5.0, 5.0], [7.0, -21.0, -24.0, -9.0, -22.0], [-21.0, 13.0, 22.0, 11.0, 17.0]]], [[[-46.0, 37.0, 21.0, -40.0, -8.0], [7.0, -32.0, -44.0, 15.0, 35.0], [22.0, 21.0, 52.0, 20.0, 43.0], [44.0, 3.0, -51.0, 28.0, -50.0]], [[23.0, -37.0, -10.0, 30.0, 3.0], [-22.0, 37.0, 24.0, -6.0, -38.0], [-26.0, -2.0, -37.0, -27.0, -44.0], [-36.0, 1.0, 39.0, -25.0, 35.0]], [[-2.0, 57.0, -12.0, -28.0, -1.0], [33.0, -52.0, -17.0, 1.0, 43.0], [29.0, -3.0, 43.0, 44.0, 66.0], [47.0, -14.0, -47.0, 18.0, -36.0]], [[-79.0, -24.0, 90.0, -38.0, -2.0], [27.0, -11.0, -25.0, 11.0, 53.0], [43.0, -21.0, 2.0, -11.0, -18.0], [-5.0, 47.0, -4.0, 69.0, -21.0]], [[-59.0, -5.0, 13.0, -18.0, -18.0], [-74.0, 35.0, -63.0, 30.0, -40.0], [-37.0, 80.0, 46.0, -33.0, -13.0], [21.0, 2.0, -33.0, -17.0, -41.0]]], [[[-66.0, -25.0, 72.0, -28.0, -3.0], [11.0, 0.0, -23.0, 11.0, 33.0], [27.0, -9.0, 1.0, -16.0, -22.0], [-7.0, 38.0, -1.0, 50.0, -16.0]], [[-55.0, -2.0, 61.0, -34.0, -1.0], [31.0, -23.0, -20.0, 7.0, 51.0], [40.0, -19.0, 11.0, 5.0, 5.0], [8.0, 30.0, -14.0, 55.0, -23.0]], [[38.0, 43.0, -65.0, 8.0, -4.0], [-23.0, -8.0, -8.0, 1.0, -27.0], [-26.0, 35.0, 32.0, 20.0, 41.0], [32.0, -39.0, -29.0, -44.0, -14.0]], [[-28.0, -9.0, 53.0, -20.0, 6.0], [51.0, -26.0, 8.0, -5.0, 55.0], [46.0, -47.0, -14.0, 10.0, -1.0], [-8.0, 29.0, 7.0, 54.0, 0.0]], [[3.0, -4.0, -34.0, 14.0, -10.0], [-67.0, 39.0, -23.0, 13.0, -61.0], [-51.0, 61.0, 18.0, -25.0, -14.0], [5.0, -19.0, -8.0, -49.0, -7.0]]], [[[-16.0, 26.0, -21.0, -12.0, -11.0], [-40.0, 4.0, -39.0, 16.0, -22.0], [-23.0, 54.0, 45.0, 0.0, 23.0], [33.0, -17.0, -38.0, -22.0, -34.0]], [[3.0, 6.0, 11.0, -6.0, 5.0], [33.0, -21.0, 12.0, -7.0, 29.0], [24.0, -29.0, -7.0, 15.0, 11.0], [0.0, 6.0, 2.0, 21.0, 3.0]], [[-4.0, -22.0, 34.0, 0.0, 8.0], [38.0, -8.0, 24.0, -10.0, 30.0], [28.0, -46.0, -32.0, 0.0, -18.0], [-24.0, 22.0, 26.0, 32.0, 20.0]], [[-16.0, 34.0, 15.0, -28.0, 1.0], [40.0, -44.0, -11.0, 0.0, 50.0], [37.0, -18.0, 25.0, 32.0, 43.0], [29.0, 3.0, -30.0, 34.0, -26.0]], [[56.0, -7.0, -51.0, 36.0, 4.0], [-19.0, 22.0, 30.0, -11.0, -43.0], [-32.0, 3.0, -24.0, -6.0, -13.0], [-18.0, -23.0, 25.0, -46.0, 32.0]]]])+(1.-msk_ref)*numpy.array([[[[-26.0, -8.0, -9.0, 0.0, -13.0], [-68.0, 38.0, -39.0, 20.0, -50.0], [-43.0, 66.0, 27.0, -30.0, -17.0], [9.0, -7.0, -16.0, -32.0, -20.0]], [[13.0, 38.0, -10.0, -14.0, 4.0], [41.0, -43.0, 5.0, -7.0, 39.0], [29.0, -23.0, 16.0, 37.0, 46.0], [25.0, -9.0, -22.0, 17.0, -13.0]], [[-17.0, -33.0, -2.0, 14.0, -9.0], [-66.0, 53.0, -20.0, 14.0, -58.0], [-46.0, 50.0, -1.0, -43.0, -44.0], [-16.0, 1.0, 11.0, -33.0, 3.0]], [[76.0, -15.0, -82.0, 56.0, 1.0], [-57.0, 50.0, 31.0, -9.0, -87.0], [-67.0, 31.0, -27.0, -22.0, -28.0], [-25.0, -38.0, 33.0, -84.0, 42.0]], [[-57.0, -23.0, 33.0, -14.0, -12.0], [-50.0, 33.0, -43.0, 22.0, -24.0], [-21.0, 48.0, 20.0, -35.0, -29.0], [1.0, 16.0, -11.0, 1.0, -23.0]]], [[[-49.0, -31.0, 66.0, -18.0, 1.0], [20.0, 1.0, -6.0, 4.0, 32.0], [28.0, -24.0, -15.0, -15.0, -28.0], [-18.0, 37.0, 13.0, 47.0, -1.0]], [[17.0, 23.0, -43.0, 6.0, -6.0], [-34.0, 7.0, -15.0, 6.0, -32.0], [-29.0, 40.0, 26.0, 3.0, 19.0], [21.0, -26.0, -21.0, -37.0, -13.0]], [[51.0, 17.0, -28.0, 14.0, 11.0], [44.0, -27.0, 40.0, -20.0, 20.0], [18.0, -44.0, -21.0, 29.0, 22.0], [-4.0, -13.0, 13.0, -1.0, 23.0]], [[32.0, 2.0, -50.0, 24.0, -4.0], [-46.0, 28.0, 0.0, 2.0, -54.0], [-44.0, 38.0, 4.0, -12.0, -6.0], [0.0, -26.0, 2.0, -52.0, 8.0]], [[45.0, -20.0, 5.0, 22.0, 17.0], [61.0, -15.0, 62.0, -27.0, 29.0], [30.0, -77.0, -59.0, 13.0, -15.0], [-38.0, 10.0, 48.0, 19.0, 49.0]]], [[[-25.0, 55.0, -10.0, -34.0, -9.0], [-2.0, -35.0, -44.0, 14.0, 22.0], [10.0, 34.0, 63.0, 29.0, 60.0], [56.0, -15.0, -61.0, 7.0, -53.0]], [[-4.0, 5.0, -12.0, 0.0, -5.0], [-25.0, 10.0, -15.0, 7.0, -19.0], [-17.0, 27.0, 15.0, -6.0, 2.0], [9.0, -8.0, -11.0, -16.0, -10.0]], [[8.0, 21.0, -4.0, -8.0, 3.0], [27.0, -26.0, 5.0, -5.0, 25.0], [19.0, -17.0, 7.0, 22.0, 26.0], [13.0, -4.0, -11.0, 12.0, -6.0]], [[26.0, 70.0, -47.0, -16.0, -1.0], [22.0, -50.0, -11.0, -2.0, 24.0], [13.0, 8.0, 47.0, 50.0, 77.0], [53.0, -33.0, -50.0, -8.0, -32.0]], [[32.0, -35.0, -49.0, 44.0, -6.0], [-83.0, 70.0, 4.0, 5.0, -95.0], [-74.0, 55.0, -18.0, -46.0, -51.0], [-28.0, -21.0, 29.0, -74.0, 28.0]]], [[[-42.0, 30.0, 24.0, -36.0, -6.0], [12.0, -30.0, -36.0, 12.0, 36.0], [24.0, 12.0, 42.0, 18.0, 36.0], [36.0, 6.0, -42.0, 30.0, -42.0]], [[11.0, -47.0, 9.0, 26.0, 4.0], [-14.0, 37.0, 25.0, -6.0, -28.0], [-17.0, -12.0, -44.0, -31.0, -53.0], [-43.0, 12.0, 45.0, -11.0, 37.0]], [[39.0, 34.0, -55.0, 10.0, -1.0], [-11.0, -9.0, 2.0, -3.0, -19.0], [-18.0, 19.0, 19.0, 19.0, 33.0], [22.0, -32.0, -18.0, -35.0, -5.0]], [[7.0, -9.0, -22.0, 14.0, -5.0], [-42.0, 29.0, -8.0, 6.0, -42.0], [-34.0, 34.0, 3.0, -19.0, -16.0], [-4.0, -11.0, 3.0, -33.0, 3.0]], [[37.0, -8.0, -10.0, 18.0, 10.0], [31.0, -7.0, 39.0, -17.0, 9.0], [11.0, -41.0, -34.0, 9.0, -6.0], [-21.0, -1.0, 28.0, 1.0, 31.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -2.0, -5.0], [-2.0, -5.0, 2.0]], [[0.0, -3.0, -3.0], [-6.0, 5.0, -1.0]], [[2.0, 0.0, 2.0], [-5.0, 0.0, 2.0]], [[1.0, 1.0, -7.0], [5.0, 0.0, -5.0]], [[5.0, 0.0, 5.0], [3.0, -3.0, 6.0]]], [[[-1.0, 0.0, -7.0], [-2.0, 4.0, 2.0]], [[4.0, -6.0, -1.0], [-5.0, -2.0, -4.0]], [[-1.0, -1.0, -4.0], [3.0, -6.0, 1.0]], [[0.0, 0.0, 4.0], [0.0, -3.0, 1.0]], [[-1.0, 1.0, 3.0], [5.0, -4.0, 0.0]]], [[[-1.0, -6.0, 3.0], [-5.0, -2.0, 5.0]], [[-6.0, -6.0, 2.0], [6.0, 0.0, 3.0]], [[6.0, -3.0, 3.0], [4.0, -6.0, 0.0]], [[-7.0, -3.0, -1.0], [7.0, -3.0, 5.0]], [[-6.0, 3.0, 3.0], [3.0, -3.0, 2.0]]], [[[-4.0, -6.0, 2.0], [0.0, 4.0, 1.0]], [[-1.0, -2.0, 5.0], [-6.0, -6.0, 2.0]], [[-4.0, 0.0, 2.0], [4.0, 0.0, -5.0]], [[-2.0, 0.0, 4.0], [-2.0, -5.0, -6.0]], [[7.0, 0.0, -2.0], [-4.0, 7.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-2.0, 6.0, 2.0], [-3.0, 5.0, -2.0]], [[4.0, 2.0, 0.0], [-6.0, 6.0, 4.0]], [[0.0, 3.0, 1.0], [6.0, -3.0, 3.0]], [[-5.0, 1.0, 0.0], [-6.0, 0.0, 7.0]], [[2.0, -5.0, -5.0], [-2.0, 5.0, -3.0]]], [[[1.0, 5.0, 5.0], [-4.0, -5.0, 0.0]], [[-2.0, -4.0, 7.0], [3.0, 3.0, 6.0]], [[5.0, 4.0, 7.0], [-6.0, -5.0, -4.0]], [[5.0, -1.0, -4.0], [6.0, -5.0, -3.0]], [[0.0, 5.0, 5.0], [7.0, 6.0, -3.0]]], [[[1.0, 3.0, -5.0], [-5.0, 1.0, 1.0]], [[7.0, -3.0, 2.0], [1.0, 4.0, 3.0]], [[7.0, -4.0, -5.0], [2.0, 0.0, -7.0]], [[6.0, -4.0, 6.0], [4.0, -3.0, 2.0]], [[-2.0, -7.0, -2.0], [5.0, -1.0, 2.0]]], [[[1.0, -5.0, -1.0], [3.0, -7.0, 5.0]], [[-5.0, 5.0, 1.0], [-6.0, 6.0, -1.0]], [[0.0, 2.0, 0.0], [-1.0, -5.0, -2.0]], [[-5.0, 6.0, -4.0], [3.0, 4.0, 2.0]], [[1.0, -7.0, -4.0], [-7.0, 0.0, -4.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-4.0, 4.0, -2.0, 1.0, -6.0], [5.0, 3.0, -7.0, -3.0, -6.0], [6.0, 0.0, 1.0, -5.0, 7.0], [7.0, -1.0, 0.0, -3.0, -1.0]], [[6.0, 7.0, 3.0, 4.0, -6.0], [-5.0, 1.0, -7.0, 5.0, -3.0], [2.0, -2.0, 1.0, 0.0, -4.0], [-4.0, 4.0, 0.0, -7.0, -3.0]], [[3.0, -1.0, 0.0, 7.0, 0.0], [6.0, 1.0, 5.0, -4.0, -4.0], [-2.0, -4.0, -6.0, -3.0, 0.0], [0.0, 6.0, -2.0, 3.0, 6.0]]], [[[-5.0, -4.0, 6.0, 3.0, 1.0], [5.0, 3.0, 4.0, -4.0, 7.0], [1.0, 0.0, -7.0, 2.0, -1.0], [6.0, -3.0, 3.0, 0.0, -2.0]], [[1.0, -7.0, -3.0, 5.0, -4.0], [6.0, 0.0, -2.0, 3.0, 2.0], [5.0, 7.0, -6.0, 4.0, -2.0], [-6.0, -3.0, -2.0, -2.0, -7.0]], [[4.0, -5.0, 2.0, 4.0, -6.0], [3.0, -6.0, -6.0, -5.0, -4.0], [2.0, -5.0, 2.0, 2.0, 4.0], [-2.0, 2.0, 0.0, 1.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-38.0, 48.0, -11.0, -60.0, -18.0], [-24.0, -7.0, -63.0, -25.0, -42.0], [19.0, -21.0, 82.0, -35.0, 70.0], [64.0, -19.0, 14.0, -7.0, 15.0]], [[4.0, -24.0, -62.0, -30.0, -2.0], [-6.0, -18.0, -22.0, 41.0, -7.0], [17.0, 58.0, 25.0, 15.0, 4.0], [-52.0, -29.0, -22.0, 1.0, -35.0]], [[31.0, 16.0, -30.0, 9.0, -29.0], [3.0, -19.0, -36.0, -4.0, -63.0], [7.0, -18.0, 29.0, -22.0, 27.0], [-20.0, 29.0, -19.0, 2.0, 26.0]], [[-64.0, 23.0, 21.0, -49.0, 23.0], [-32.0, 42.0, 1.0, 35.0, 74.0], [17.0, 51.0, -1.0, 16.0, -22.0], [43.0, -64.0, 29.0, -36.0, -71.0]], [[1.0, -6.0, 29.0, 58.0, -51.0], [70.0, -7.0, -28.0, -86.0, -59.0], [20.0, -71.0, -16.0, -34.0, 62.0], [59.0, 37.0, 5.0, 12.0, 58.0]]], [[[5.0, -27.0, -18.0, -28.0, -24.0], [-27.0, -28.0, -56.0, 41.0, 20.0], [30.0, 46.0, 35.0, 42.0, -5.0], [-47.0, -43.0, 0.0, -24.0, -59.0]], [[-48.0, 29.0, -58.0, -68.0, 39.0], [-5.0, 14.0, 17.0, -4.0, -25.0], [-9.0, 22.0, 43.0, -43.0, 45.0], [42.0, -21.0, -9.0, 27.0, 20.0]], [[-31.0, 18.0, 37.0, -50.0, 33.0], [-42.0, -5.0, 12.0, -21.0, 30.0], [-25.0, -29.0, 39.0, 1.0, 10.0], [49.0, -16.0, 29.0, 11.0, 19.0]], [[13.0, 12.0, 11.0, 17.0, 6.0], [9.0, -2.0, 20.0, -30.0, -26.0], [-21.0, -42.0, -4.0, -22.0, 10.0], [16.0, 35.0, -2.0, 19.0, 48.0]], [[-10.0, 8.0, 47.0, 19.0, 21.0], [9.0, 16.0, 43.0, -36.0, 18.0], [-25.0, -42.0, -29.0, -10.0, -8.0], [43.0, 20.0, 17.0, 13.0, 34.0]]], [[[20.0, -40.0, -30.0, -9.0, 15.0], [21.0, -51.0, 18.0, -50.0, -47.0], [-29.0, -39.0, 32.0, -12.0, 46.0], [-11.0, 26.0, -17.0, 63.0, 76.0]], [[-24.0, -107.0, 36.0, 14.0, 60.0], [51.0, -22.0, 100.0, -59.0, 76.0], [-40.0, -11.0, -60.0, 42.0, -12.0], [12.0, -18.0, 14.0, 69.0, 33.0]], [[-59.0, 26.0, 21.0, -3.0, 10.0], [47.0, 30.0, 22.0, -79.0, -23.0], [-2.0, -48.0, -7.0, -55.0, 62.0], [114.0, 6.0, 18.0, 24.0, 55.0]], [[-11.0, -80.0, 66.0, 0.0, 49.0], [6.0, -34.0, 69.0, -52.0, 78.0], [-44.0, -36.0, -25.0, 50.0, -18.0], [13.0, -13.0, 29.0, 50.0, 32.0]], [[41.0, -7.0, 52.0, 29.0, 21.0], [-24.0, -15.0, 42.0, -10.0, 22.0], [-44.0, -49.0, -20.0, 19.0, -43.0], [-22.0, 40.0, 9.0, 14.0, 36.0]]], [[[-6.0, -93.0, -20.0, 10.0, 38.0], [49.0, -22.0, 66.0, -19.0, 38.0], [-18.0, 27.0, -44.0, 32.0, -8.0], [-30.0, -18.0, -12.0, 53.0, 9.0]], [[39.0, 33.0, -18.0, -14.0, 24.0], [-25.0, -30.0, 22.0, -31.0, -70.0], [-52.0, -68.0, 49.0, -42.0, 27.0], [-3.0, 63.0, -16.0, 46.0, 97.0]], [[-18.0, -9.0, 22.0, 2.0, 58.0], [-3.0, 32.0, 84.0, 13.0, 64.0], [-34.0, 17.0, -54.0, 12.0, -52.0], [6.0, -6.0, 8.0, 13.0, -7.0]], [[1.0, 61.0, -5.0, -29.0, 66.0], [-44.0, 28.0, 72.0, 13.0, -4.0], [-59.0, -21.0, 6.0, -38.0, -26.0], [16.0, 35.0, -4.0, 22.0, 47.0]], [[13.0, -28.0, -49.0, 36.0, -104.0], [60.0, -23.0, -119.0, -1.0, -68.0], [87.0, 32.0, 15.0, 1.0, 59.0], [-27.0, -18.0, -22.0, -36.0, -45.0]]]])+(1.-msk_ref)*numpy.array([[[[62.0, 19.0, -15.0, 44.0, -35.0], [-19.0, 5.0, -28.0, 65.0, -17.0], [14.0, 25.0, -21.0, 14.0, -53.0], [-82.0, 28.0, -23.0, -42.0, -39.0]], [[48.0, -8.0, -48.0, 40.0, -90.0], [28.0, -28.0, -102.0, 20.0, -76.0], [60.0, 18.0, 20.0, 0.0, 30.0], [-60.0, 12.0, -30.0, -34.0, -28.0]], [[0.0, 2.0, 60.0, 34.0, -18.0], [12.0, 4.0, -4.0, -37.0, 11.0], [1.0, -46.0, -21.0, 3.0, 0.0], [36.0, 15.0, 22.0, -9.0, 15.0]], [[84.0, -24.0, -9.0, 9.0, -24.0], [-39.0, -74.0, -38.0, 9.0, -43.0], [-20.0, -37.0, 52.0, 27.0, -5.0], [-89.0, 41.0, -18.0, 15.0, 35.0]], [[-50.0, -34.0, -52.0, -46.0, 14.0], [16.0, 8.0, -4.0, 27.0, 31.0], [29.0, 80.0, 5.0, 15.0, 14.0], [-2.0, -67.0, -6.0, 1.0, -57.0]]], [[[56.0, 85.0, 4.0, 19.0, -20.0], [-40.0, 1.0, -23.0, 3.0, -79.0], [-23.0, -65.0, 34.0, -48.0, 1.0], [-7.0, 76.0, -12.0, -13.0, 57.0]], [[17.0, -106.0, 13.0, 79.0, -9.0], [103.0, -30.0, 47.0, -75.0, -1.0], [-4.0, -29.0, -75.0, 19.0, 17.0], [-10.0, 22.0, -11.0, 55.0, 47.0]], [[34.0, 120.0, -27.0, 11.0, -16.0], [-25.0, 32.0, -18.0, 6.0, -106.0], [-15.0, -51.0, 31.0, -86.0, 19.0], [21.0, 78.0, -22.0, -16.0, 60.0]], [[-85.0, 43.0, 32.0, -46.0, 20.0], [-3.0, 46.0, 4.0, -28.0, 33.0], [11.0, -2.0, 10.0, -27.0, 31.0], [111.0, -42.0, 36.0, -13.0, -12.0]], [[4.0, -25.0, 33.0, 94.0, -29.0], [67.0, 49.0, 24.0, 10.0, 38.0], [31.0, 27.0, -116.0, 17.0, -51.0], [-8.0, 5.0, -1.0, -35.0, -50.0]]], [[[29.0, 38.0, -24.0, -28.0, -39.0], [-56.0, -20.0, -81.0, 50.0, -32.0], [24.0, 16.0, 65.0, 6.0, 2.0], [-43.0, -5.0, -7.0, -40.0, -34.0]], [[-29.0, -42.0, -23.0, 44.0, -57.0], [100.0, 5.0, -40.0, -51.0, -38.0], [59.0, 11.0, -33.0, -17.0, 64.0], [37.0, -16.0, -9.0, 1.0, -7.0]], [[-105.0, 32.0, -28.0, -66.0, 26.0], [14.0, 60.0, 4.0, 6.0, 32.0], [32.0, 63.0, 5.0, -30.0, 35.0], [91.0, -73.0, 16.0, -15.0, -50.0]], [[-45.0, -15.0, 13.0, 37.0, -8.0], [94.0, 20.0, 26.0, -97.0, -34.0], [9.0, -47.0, -40.0, -48.0, 68.0], [96.0, 15.0, 6.0, 36.0, 61.0]], [[-58.0, -78.0, 20.0, -26.0, 51.0], [38.0, -12.0, 63.0, -54.0, 66.0], [-18.0, 5.0, -22.0, 26.0, 19.0], [46.0, -46.0, 21.0, 53.0, 14.0]]], [[[-39.0, -18.0, 32.0, -32.0, 25.0], [12.0, -24.0, 19.0, -82.0, 0.0], [-24.0, -60.0, 33.0, -14.0, 58.0], [77.0, -5.0, 25.0, 48.0, 66.0]], [[85.0, 1.0, -31.0, 30.0, -24.0], [-41.0, -21.0, -25.0, 83.0, -15.0], [0.0, 33.0, -2.0, 32.0, -65.0], [-125.0, 29.0, -32.0, -30.0, -37.0]], [[4.0, 63.0, 11.0, -28.0, 19.0], [-51.0, 11.0, 4.0, 9.0, -15.0], [-26.0, -29.0, 35.0, -26.0, -5.0], [20.0, 22.0, 7.0, -6.0, 25.0]], [[41.0, -24.0, 38.0, 28.0, -31.0], [-34.0, -16.0, -35.0, 51.0, 49.0], [17.0, 22.0, -16.0, 63.0, -62.0], [-69.0, -12.0, 9.0, -45.0, -65.0]], [[-39.0, 7.0, -73.0, -92.0, 53.0], [-31.0, -5.0, 18.0, 26.0, -2.0], [-15.0, 50.0, 59.0, -15.0, 26.0], [1.0, -40.0, -13.0, 30.0, -2.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 7.0, 6.0, 0.0, -1.0], [-7.0, -6.0, -5.0, 7.0, -3.0], [3.0, 7.0, 1.0, 0.0, 0.0], [4.0, 2.0, -3.0, 4.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -1.0, 1.0, 4.0, 1.0], [0.0, 2.0, 0.0, 3.0, 1.0], [5.0, 2.0, 1.0, 4.0, -5.0], [0.0, -6.0, 2.0, 7.0, 3.0]])
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-0.0, -49.0, -42.0, -0.0, 7.0], [49.0, 42.0, 35.0, -49.0, 21.0], [-21.0, -49.0, -7.0, -0.0, -0.0], [-28.0, -14.0, 21.0, -28.0, 14.0]])+(1.-msk_ref)*numpy.array([[35.0, 7.0, -7.0, -28.0, -7.0], [-0.0, -14.0, -0.0, -21.0, -7.0], [-35.0, -14.0, -7.0, -28.0, 35.0], [-0.0, 42.0, -14.0, -49.0, -21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[5.0, 3.0, 3.0], [-4.0, -1.0, 6.0], [5.0, 1.0, 2.0], [-2.0, -5.0, -2.0], [0.0, 2.0, 5.0]], [[-3.0, -1.0, -4.0], [5.0, 2.0, 2.0], [0.0, 4.0, -7.0], [-2.0, 5.0, -2.0], [1.0, 4.0, 1.0]], [[0.0, -5.0, -1.0], [-7.0, 1.0, 3.0], [-3.0, 6.0, 6.0], [0.0, -1.0, -2.0], [1.0, 0.0, -5.0]], [[2.0, -7.0, 2.0], [-7.0, 0.0, -6.0], [-6.0, -7.0, -5.0], [-3.0, -4.0, -3.0], [1.0, 0.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, -5.0, 5.0], [0.0, -6.0, 2.0], [7.0, 4.0, -3.0], [2.0, 4.0, 7.0], [4.0, 5.0, 1.0]], [[-7.0, 6.0, 0.0], [0.0, -5.0, 1.0], [-3.0, 6.0, 0.0], [0.0, -5.0, 7.0], [-5.0, 1.0, 0.0]], [[7.0, -1.0, -4.0], [4.0, 3.0, 1.0], [-2.0, 0.0, 7.0], [-5.0, 6.0, 6.0], [7.0, 4.0, 4.0]], [[7.0, 7.0, 7.0], [3.0, -5.0, 0.0], [0.0, -2.0, 5.0], [0.0, -3.0, 0.0], [3.0, 2.0, -1.0]]])
      arg1=Data(numpy.array([-1.0, -6.0, 4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-11.0, 34.0, -3.0, 24.0, 8.0], [-7.0, -9.0, -52.0, -36.0, -21.0], [26.0, 13.0, -9.0, -2.0, -21.0], [48.0, -17.0, 28.0, 15.0, -25.0]])+(1.-msk_ref)*numpy.array([[54.0, 44.0, -43.0, 2.0, -30.0], [-29.0, 34.0, -33.0, 58.0, -1.0], [-17.0, -18.0, 30.0, -7.0, -15.0], [-21.0, 27.0, 32.0, 18.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 0.0, -6.0], [7.0, 2.0, -3.0]], [[0.0, -3.0, -1.0], [7.0, 7.0, -3.0]], [[7.0, -3.0, -2.0], [6.0, 3.0, -3.0]], [[3.0, -6.0, 2.0], [-2.0, -3.0, -1.0]], [[-4.0, 0.0, -1.0], [-2.0, -7.0, 1.0]]], [[[4.0, -5.0, 2.0], [-4.0, 3.0, 4.0]], [[4.0, -7.0, 3.0], [0.0, 6.0, -3.0]], [[4.0, 3.0, -5.0], [6.0, -1.0, 6.0]], [[-1.0, -5.0, 3.0], [-1.0, -2.0, -2.0]], [[-1.0, 7.0, -5.0], [1.0, -5.0, 6.0]]], [[[-7.0, -5.0, 1.0], [0.0, 0.0, 0.0]], [[7.0, 4.0, 0.0], [2.0, 0.0, 4.0]], [[7.0, -3.0, -4.0], [-7.0, -5.0, -4.0]], [[-7.0, 6.0, 6.0], [-5.0, -1.0, 3.0]], [[-5.0, 5.0, -7.0], [5.0, 4.0, -5.0]]], [[[1.0, -5.0, -5.0], [4.0, 2.0, -6.0]], [[0.0, 1.0, -7.0], [-5.0, 7.0, -7.0]], [[2.0, 0.0, 1.0], [-3.0, -5.0, 0.0]], [[7.0, 2.0, 2.0], [3.0, 3.0, 6.0]], [[7.0, 6.0, 4.0], [4.0, -7.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -7.0, -4.0], [-3.0, 2.0, 3.0]], [[-7.0, -6.0, 0.0], [-3.0, -6.0, 1.0]], [[2.0, -7.0, 6.0], [-3.0, -5.0, 0.0]], [[-2.0, 6.0, -3.0], [-7.0, 2.0, -6.0]], [[5.0, -6.0, 6.0], [4.0, 1.0, -1.0]]], [[[-6.0, -6.0, 2.0], [-2.0, 7.0, 4.0]], [[-1.0, 3.0, 2.0], [-1.0, 4.0, -5.0]], [[2.0, -1.0, -1.0], [-4.0, -4.0, 4.0]], [[5.0, 0.0, -5.0], [-2.0, -5.0, -1.0]], [[5.0, -1.0, 5.0], [-3.0, -6.0, 3.0]]], [[[-3.0, 0.0, 1.0], [-6.0, -4.0, 1.0]], [[6.0, 0.0, 0.0], [0.0, 0.0, 0.0]], [[-4.0, 7.0, 6.0], [-7.0, -4.0, -2.0]], [[0.0, 4.0, -4.0], [-6.0, -6.0, 4.0]], [[4.0, -1.0, 5.0], [6.0, 7.0, 0.0]]], [[[-4.0, 0.0, -2.0], [0.0, -1.0, 3.0]], [[7.0, -3.0, -4.0], [4.0, 7.0, -5.0]], [[3.0, 7.0, 2.0], [-5.0, 4.0, 2.0]], [[5.0, 0.0, 3.0], [-2.0, 3.0, -1.0]], [[-2.0, -2.0, -2.0], [6.0, 2.0, -3.0]]]])
      arg1=Data(numpy.array([[2.0, -5.0, 0.0], [-3.0, -3.0, 7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-62.0, -48.0, -19.0, 44.0, 26.0], [64.0, 4.0, 20.0, 18.0, 17.0], [11.0, 16.0, 37.0, -5.0, -97.0], [-33.0, -60.0, 28.0, 28.0, 14.0]])+(1.-msk_ref)*numpy.array([[63.0, 50.0, 63.0, -61.0, 18.0], [31.0, -61.0, 61.0, 24.0, 63.0], [31.0, 12.0, -24.0, 44.0, -26.0], [16.0, -39.0, -12.0, 0.0, -39.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 0.0, 3.0, 0.0, 6.0], [-6.0, -7.0, -5.0, -4.0, -7.0], [3.0, 7.0, 4.0, 1.0, -1.0], [-3.0, 2.0, 5.0, 3.0, 6.0]])+(1.-msk_arg0)*numpy.array([[-7.0, -2.0, -1.0, 6.0, 6.0], [2.0, 3.0, 7.0, 7.0, -4.0], [7.0, 0.0, 1.0, 2.0, -5.0], [-7.0, -5.0, 5.0, 6.0, -4.0]])
      arg1=Data(numpy.array([5.0, 0.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[30.0, 0.0], [0.0, 0.0], [15.0, 0.0], [0.0, 0.0], [30.0, 0.0]], [[-30.0, 0.0], [-35.0, 0.0], [-25.0, 0.0], [-20.0, 0.0], [-35.0, 0.0]], [[15.0, 0.0], [35.0, 0.0], [20.0, 0.0], [5.0, 0.0], [-5.0, 0.0]], [[-15.0, 0.0], [10.0, 0.0], [25.0, 0.0], [15.0, 0.0], [30.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, 0.0], [-10.0, 0.0], [-5.0, 0.0], [30.0, 0.0], [30.0, 0.0]], [[10.0, 0.0], [15.0, 0.0], [35.0, 0.0], [35.0, 0.0], [-20.0, 0.0]], [[35.0, 0.0], [0.0, 0.0], [5.0, 0.0], [10.0, 0.0], [-25.0, 0.0]], [[-35.0, 0.0], [-25.0, 0.0], [25.0, 0.0], [30.0, 0.0], [-20.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, 3.0, -5.0], [-4.0, 5.0, 0.0], [-3.0, 1.0, 0.0], [-3.0, -5.0, 5.0], [6.0, -6.0, 4.0]], [[-4.0, 5.0, 6.0], [-2.0, 0.0, 0.0], [-2.0, -1.0, -2.0], [-2.0, -5.0, -3.0], [4.0, 4.0, 4.0]], [[-7.0, 1.0, -6.0], [5.0, -4.0, 2.0], [-2.0, 2.0, 4.0], [-2.0, -7.0, 6.0], [2.0, -4.0, -7.0]], [[3.0, 0.0, -4.0], [-6.0, -3.0, 5.0], [-3.0, -1.0, -4.0], [-1.0, 2.0, 5.0], [0.0, 0.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-5.0, 7.0, 3.0], [0.0, -5.0, 0.0], [0.0, -7.0, 7.0], [5.0, 4.0, 3.0], [3.0, 7.0, 4.0]], [[-2.0, -6.0, 7.0], [-4.0, -5.0, 2.0], [4.0, -1.0, 1.0], [3.0, 6.0, 2.0], [-6.0, -7.0, -3.0]], [[6.0, -7.0, 2.0], [0.0, 3.0, 2.0], [6.0, -7.0, 7.0], [1.0, 3.0, 1.0], [-1.0, 0.0, -6.0]], [[-3.0, -2.0, 1.0], [-6.0, -1.0, -6.0], [-3.0, 1.0, -1.0], [-1.0, -4.0, 6.0], [7.0, -2.0, -6.0]]])
      arg1=Data(numpy.array([[3.0, 6.0], [7.0, 7.0], [7.0, 7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-2.0, 10.0], [23.0, 11.0], [-2.0, -11.0], [-9.0, -18.0], [4.0, 22.0]], [[65.0, 53.0], [-6.0, -12.0], [-27.0, -33.0], [-62.0, -68.0], [68.0, 80.0]], [[-56.0, -77.0], [1.0, 16.0], [36.0, 30.0], [-13.0, -19.0], [-71.0, -65.0]], [[-19.0, -10.0], [-4.0, -22.0], [-44.0, -53.0], [46.0, 43.0], [0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[55.0, 40.0], [-35.0, -35.0], [0.0, 0.0], [64.0, 79.0], [86.0, 95.0]], [[1.0, -5.0], [-33.0, -45.0], [12.0, 24.0], [65.0, 74.0], [-88.0, -106.0]], [[-17.0, 1.0], [35.0, 35.0], [18.0, 36.0], [31.0, 34.0], [-45.0, -48.0]], [[-16.0, -25.0], [-67.0, -85.0], [-9.0, -18.0], [11.0, 8.0], [-35.0, -14.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, -3.0, 7.0], [-3.0, 0.0, 6.0]], [[-1.0, 7.0, -3.0], [-7.0, -7.0, 2.0]], [[2.0, -6.0, 1.0], [-6.0, -2.0, 6.0]], [[0.0, 7.0, 4.0], [3.0, 7.0, 7.0]], [[-4.0, 4.0, -5.0], [-4.0, -3.0, -5.0]]], [[[-3.0, 0.0, -5.0], [6.0, 4.0, 1.0]], [[7.0, 1.0, -4.0], [0.0, 1.0, 0.0]], [[-7.0, 4.0, 4.0], [5.0, -3.0, 5.0]], [[6.0, -4.0, -4.0], [7.0, 0.0, -5.0]], [[0.0, 4.0, 1.0], [7.0, 3.0, 6.0]]], [[[7.0, 1.0, 0.0], [-6.0, -5.0, -7.0]], [[-7.0, -4.0, -2.0], [5.0, 5.0, 1.0]], [[0.0, -7.0, 7.0], [-5.0, -7.0, 4.0]], [[0.0, 0.0, 6.0], [-2.0, 3.0, 1.0]], [[7.0, 7.0, 0.0], [-7.0, -2.0, 0.0]]], [[[-1.0, 0.0, -4.0], [0.0, -7.0, 5.0]], [[6.0, -2.0, -4.0], [-1.0, -7.0, -1.0]], [[1.0, 7.0, -6.0], [2.0, -4.0, -1.0]], [[0.0, 2.0, -2.0], [3.0, 3.0, 4.0]], [[-2.0, -2.0, 7.0], [-5.0, -7.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 1.0, 0.0], [6.0, -5.0, 0.0]], [[1.0, -6.0, 5.0], [-7.0, 1.0, -7.0]], [[3.0, 5.0, -1.0], [-7.0, 3.0, 4.0]], [[4.0, -5.0, 3.0], [-6.0, -7.0, -2.0]], [[1.0, -4.0, 5.0], [4.0, -2.0, 3.0]]], [[[3.0, -3.0, -4.0], [2.0, -4.0, -1.0]], [[5.0, 2.0, 4.0], [2.0, 5.0, 0.0]], [[-3.0, 1.0, 3.0], [-6.0, -4.0, 5.0]], [[-4.0, 0.0, 1.0], [-1.0, 4.0, -5.0]], [[-1.0, -3.0, -2.0], [0.0, 4.0, 7.0]]], [[[-5.0, -2.0, 6.0], [-6.0, -5.0, -7.0]], [[4.0, -1.0, -1.0], [-2.0, -6.0, -3.0]], [[-6.0, -4.0, 7.0], [0.0, -7.0, -5.0]], [[2.0, -2.0, 0.0], [1.0, -1.0, 5.0]], [[7.0, -7.0, -4.0], [-6.0, 5.0, 3.0]]], [[[-7.0, 0.0, 1.0], [-5.0, -6.0, 4.0]], [[-4.0, -1.0, -2.0], [6.0, -3.0, 2.0]], [[0.0, 5.0, -5.0], [-2.0, 3.0, 5.0]], [[-2.0, 6.0, -5.0], [-6.0, -4.0, 0.0]], [[4.0, -7.0, 1.0], [-6.0, 4.0, 0.0]]]])
      arg1=Data(numpy.array([[[-5.0, 5.0], [-4.0, -6.0], [-2.0, -4.0]], [[-3.0, 1.0], [-7.0, -3.0], [1.0, -3.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, -26.0], [55.0, -27.0], [50.0, 24.0], [-87.0, -97.0], [42.0, -4.0]], [[-20.0, -4.0], [-38.0, 42.0], [22.0, -76.0], [-32.0, 92.0], [-54.0, -48.0]], [[7.0, 59.0], [6.0, -16.0], [82.0, 18.0], [-26.0, -38.0], [-28.0, -8.0]], [[67.0, 17.0], [37.0, 81.0], [0.0, 4.0], [-30.0, -22.0], [71.0, -19.0]]])+(1.-msk_ref)*numpy.array([[[33.0, -5.0], [16.0, 32.0], [-29.0, -39.0], [59.0, 59.0], [6.0, 10.0]], [[26.0, 66.0], [-82.0, -16.0], [56.0, -42.0], [-12.0, -22.0], [0.0, -12.0]], [[67.0, -7.0], [31.0, 55.0], [76.0, 2.0], [7.0, 11.0], [-13.0, 63.0]], [[94.0, -38.0], [33.0, 3.0], [-20.0, -36.0], [42.0, -20.0], [-4.0, 40.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, -1.0, 6.0, 6.0, -7.0], [6.0, 1.0, 5.0, 2.0, -4.0], [0.0, -6.0, 2.0, -2.0, 0.0], [-5.0, -7.0, 0.0, 0.0, -3.0]])+(1.-msk_arg0)*numpy.array([[-2.0, -5.0, 5.0, 0.0, 7.0], [0.0, 0.0, 6.0, 6.0, 4.0], [1.0, -6.0, -7.0, -6.0, 6.0], [3.0, -4.0, 5.0, -3.0, 0.0]])
      arg1=Data(numpy.array([[-5.0, 5.0, -4.0, 6.0, -7.0], [-5.0, 2.0, 0.0, 6.0, 2.0], [0.0, 3.0, -3.0, 1.0, -7.0], [3.0, -2.0, -7.0, -2.0, 0.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-5.0, 5.0, -4.0, 6.0, -7.0], [-5.0, 2.0, 0.0, 6.0, 2.0], [0.0, 3.0, -3.0, 1.0, -7.0], [3.0, -2.0, -7.0, -2.0, 0.0]], [[5.0, -5.0, 4.0, -6.0, 7.0], [5.0, -2.0, 0.0, -6.0, -2.0], [0.0, -3.0, 3.0, -1.0, 7.0], [-3.0, 2.0, 7.0, 2.0, 0.0]], [[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]], [[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]], [[35.0, -35.0, 28.0, -42.0, 49.0], [35.0, -14.0, 0.0, -42.0, -14.0], [0.0, -21.0, 21.0, -7.0, 49.0], [-21.0, 14.0, 49.0, 14.0, 0.0]]], [[[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]], [[-5.0, 5.0, -4.0, 6.0, -7.0], [-5.0, 2.0, 0.0, 6.0, 2.0], [0.0, 3.0, -3.0, 1.0, -7.0], [3.0, -2.0, -7.0, -2.0, 0.0]], [[-25.0, 25.0, -20.0, 30.0, -35.0], [-25.0, 10.0, 0.0, 30.0, 10.0], [0.0, 15.0, -15.0, 5.0, -35.0], [15.0, -10.0, -35.0, -10.0, 0.0]], [[-10.0, 10.0, -8.0, 12.0, -14.0], [-10.0, 4.0, 0.0, 12.0, 4.0], [0.0, 6.0, -6.0, 2.0, -14.0], [6.0, -4.0, -14.0, -4.0, 0.0]], [[20.0, -20.0, 16.0, -24.0, 28.0], [20.0, -8.0, 0.0, -24.0, -8.0], [0.0, -12.0, 12.0, -4.0, 28.0], [-12.0, 8.0, 28.0, 8.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[30.0, -30.0, 24.0, -36.0, 42.0], [30.0, -12.0, 0.0, -36.0, -12.0], [0.0, -18.0, 18.0, -6.0, 42.0], [-18.0, 12.0, 42.0, 12.0, 0.0]], [[-10.0, 10.0, -8.0, 12.0, -14.0], [-10.0, 4.0, 0.0, 12.0, 4.0], [0.0, 6.0, -6.0, 2.0, -14.0], [6.0, -4.0, -14.0, -4.0, 0.0]], [[10.0, -10.0, 8.0, -12.0, 14.0], [10.0, -4.0, 0.0, -12.0, -4.0], [0.0, -6.0, 6.0, -2.0, 14.0], [-6.0, 4.0, 14.0, 4.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[25.0, -25.0, 20.0, -30.0, 35.0], [25.0, -10.0, 0.0, -30.0, -10.0], [0.0, -15.0, 15.0, -5.0, 35.0], [-15.0, 10.0, 35.0, 10.0, 0.0]], [[35.0, -35.0, 28.0, -42.0, 49.0], [35.0, -14.0, 0.0, -42.0, -14.0], [0.0, -21.0, 21.0, -7.0, 49.0], [-21.0, 14.0, 49.0, 14.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[15.0, -15.0, 12.0, -18.0, 21.0], [15.0, -6.0, 0.0, -18.0, -6.0], [0.0, -9.0, 9.0, -3.0, 21.0], [-9.0, 6.0, 21.0, 6.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[10.0, -10.0, 8.0, -12.0, 14.0], [10.0, -4.0, 0.0, -12.0, -4.0], [0.0, -6.0, 6.0, -2.0, 14.0], [-6.0, 4.0, 14.0, 4.0, 0.0]], [[25.0, -25.0, 20.0, -30.0, 35.0], [25.0, -10.0, 0.0, -30.0, -10.0], [0.0, -15.0, 15.0, -5.0, 35.0], [-15.0, 10.0, 35.0, 10.0, 0.0]], [[-25.0, 25.0, -20.0, 30.0, -35.0], [-25.0, 10.0, 0.0, 30.0, 10.0], [0.0, 15.0, -15.0, 5.0, -35.0], [15.0, -10.0, -35.0, -10.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-35.0, 35.0, -28.0, 42.0, -49.0], [-35.0, 14.0, 0.0, 42.0, 14.0], [0.0, 21.0, -21.0, 7.0, -49.0], [21.0, -14.0, -49.0, -14.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]], [[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]], [[-20.0, 20.0, -16.0, 24.0, -28.0], [-20.0, 8.0, 0.0, 24.0, 8.0], [0.0, 12.0, -12.0, 4.0, -28.0], [12.0, -8.0, -28.0, -8.0, 0.0]]], [[[-5.0, 5.0, -4.0, 6.0, -7.0], [-5.0, 2.0, 0.0, 6.0, 2.0], [0.0, 3.0, -3.0, 1.0, -7.0], [3.0, -2.0, -7.0, -2.0, 0.0]], [[30.0, -30.0, 24.0, -36.0, 42.0], [30.0, -12.0, 0.0, -36.0, -12.0], [0.0, -18.0, 18.0, -6.0, 42.0], [-18.0, 12.0, 42.0, 12.0, 0.0]], [[35.0, -35.0, 28.0, -42.0, 49.0], [35.0, -14.0, 0.0, -42.0, -14.0], [0.0, -21.0, 21.0, -7.0, 49.0], [-21.0, 14.0, 49.0, 14.0, 0.0]], [[30.0, -30.0, 24.0, -36.0, 42.0], [30.0, -12.0, 0.0, -36.0, -12.0], [0.0, -18.0, 18.0, -6.0, 42.0], [-18.0, 12.0, 42.0, 12.0, 0.0]], [[-30.0, 30.0, -24.0, 36.0, -42.0], [-30.0, 12.0, 0.0, 36.0, 12.0], [0.0, 18.0, -18.0, 6.0, -42.0], [18.0, -12.0, -42.0, -12.0, 0.0]]], [[[-15.0, 15.0, -12.0, 18.0, -21.0], [-15.0, 6.0, 0.0, 18.0, 6.0], [0.0, 9.0, -9.0, 3.0, -21.0], [9.0, -6.0, -21.0, -6.0, 0.0]], [[20.0, -20.0, 16.0, -24.0, 28.0], [20.0, -8.0, 0.0, -24.0, -8.0], [0.0, -12.0, 12.0, -4.0, 28.0], [-12.0, 8.0, 28.0, 8.0, 0.0]], [[-25.0, 25.0, -20.0, 30.0, -35.0], [-25.0, 10.0, 0.0, 30.0, 10.0], [0.0, 15.0, -15.0, 5.0, -35.0], [15.0, -10.0, -35.0, -10.0, 0.0]], [[15.0, -15.0, 12.0, -18.0, 21.0], [15.0, -6.0, 0.0, -18.0, -6.0], [0.0, -9.0, 9.0, -3.0, 21.0], [-9.0, 6.0, 21.0, 6.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[5.0, -1.0, 0.0], [0.0, -2.0, 5.0], [-1.0, -6.0, -5.0], [-1.0, -6.0, 3.0], [-1.0, -3.0, 1.0]], [[6.0, 0.0, 5.0], [3.0, 0.0, -7.0], [-1.0, 3.0, 4.0], [5.0, -6.0, -1.0], [-3.0, 5.0, -6.0]], [[1.0, -4.0, -3.0], [-3.0, 4.0, 1.0], [5.0, 5.0, 1.0], [-3.0, -6.0, -4.0], [-2.0, 0.0, 6.0]], [[2.0, -4.0, -1.0], [5.0, -4.0, 0.0], [5.0, -7.0, 3.0], [1.0, 7.0, -1.0], [0.0, -7.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[6.0, 0.0, -6.0], [4.0, -2.0, -4.0], [4.0, 5.0, 4.0], [5.0, -1.0, 0.0], [4.0, -2.0, -2.0]], [[6.0, 0.0, -1.0], [3.0, 3.0, 2.0], [7.0, 2.0, -5.0], [4.0, -2.0, 5.0], [0.0, 5.0, -6.0]], [[-4.0, -1.0, -1.0], [7.0, -7.0, 7.0], [3.0, 0.0, 5.0], [-2.0, -6.0, 3.0], [7.0, 1.0, -7.0]], [[-7.0, 4.0, 2.0], [6.0, -2.0, 5.0], [5.0, -4.0, 0.0], [4.0, 0.0, -6.0], [5.0, 7.0, 5.0]]])
      arg1=Data(numpy.array([[[-6.0, -7.0, 1.0, -5.0, -5.0], [0.0, -5.0, 2.0, -1.0, 6.0], [0.0, 0.0, -3.0, 1.0, -6.0], [0.0, 3.0, -4.0, 4.0, 0.0]], [[-1.0, 2.0, -2.0, -3.0, -3.0], [2.0, 4.0, 2.0, 6.0, -1.0], [3.0, -2.0, -1.0, 2.0, 6.0], [3.0, 4.0, -6.0, 2.0, 2.0]], [[-5.0, -1.0, 3.0, 4.0, 6.0], [1.0, 4.0, 3.0, -1.0, -7.0], [-5.0, -3.0, 1.0, 1.0, 4.0], [-6.0, -3.0, -1.0, -6.0, 0.0]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-29.0, -37.0, 7.0, -22.0, -22.0], [-2.0, -29.0, 8.0, -11.0, 31.0], [-3.0, 2.0, -14.0, 3.0, -36.0], [-3.0, 11.0, -14.0, 18.0, -2.0]], [[-23.0, -9.0, 19.0, 26.0, 36.0], [1.0, 12.0, 11.0, -17.0, -33.0], [-31.0, -11.0, 7.0, 1.0, 8.0], [-36.0, -23.0, 7.0, -34.0, -4.0]], [[37.0, 0.0, -4.0, 3.0, -7.0], [-17.0, -39.0, -29.0, -30.0, 35.0], [7.0, 27.0, 4.0, -18.0, -50.0], [12.0, -12.0, 45.0, 14.0, -12.0]], [[-3.0, -8.0, 20.0, 35.0, 41.0], [-9.0, -7.0, -5.0, -38.0, -21.0], [-33.0, 3.0, 12.0, -10.0, -18.0], [-36.0, -36.0, 37.0, -34.0, -12.0]], [[4.0, 0.0, 8.0, 18.0, 20.0], [-5.0, -3.0, -5.0, -18.0, -10.0], [-14.0, 3.0, 7.0, -6.0, -8.0], [-15.0, -18.0, 21.0, -16.0, -6.0]]], [[[-61.0, -47.0, 21.0, -10.0, 0.0], [5.0, -10.0, 27.0, -11.0, 1.0], [-25.0, -15.0, -13.0, 11.0, -16.0], [-30.0, 3.0, -29.0, -6.0, 0.0]], [[17.0, -14.0, -18.0, -43.0, -57.0], [-7.0, -43.0, -15.0, 4.0, 67.0], [35.0, 21.0, -16.0, -4.0, -46.0], [42.0, 30.0, -5.0, 54.0, 0.0]], [[-17.0, 9.0, 5.0, 12.0, 20.0], [10.0, 33.0, 16.0, 15.0, -37.0], [-11.0, -18.0, 4.0, 9.0, 40.0], [-15.0, -3.0, -18.0, -22.0, 6.0]], [[-19.0, -46.0, 14.0, -11.0, -13.0], [-13.0, -53.0, -5.0, -40.0, 43.0], [-13.0, 15.0, -10.0, -8.0, -70.0], [-12.0, -6.0, 17.0, 14.0, -12.0]], [[43.0, 37.0, -31.0, -24.0, -36.0], [4.0, 11.0, -14.0, 39.0, 19.0], [45.0, 8.0, -2.0, 1.0, 24.0], [51.0, 29.0, -12.0, 34.0, 10.0]]], [[[13.0, -12.0, 0.0, -5.0, -11.0], [-11.0, -33.0, -15.0, -22.0, 31.0], [3.0, 17.0, -2.0, -10.0, -42.0], [6.0, -4.0, 23.0, 14.0, -8.0]], [[9.0, 28.0, -8.0, 7.0, 9.0], [9.0, 35.0, 5.0, 26.0, -29.0], [7.0, -11.0, 6.0, 6.0, 46.0], [6.0, 4.0, -13.0, -10.0, 8.0]], [[-40.0, -26.0, -2.0, -36.0, -34.0], [11.0, -1.0, 23.0, 24.0, 18.0], [10.0, -13.0, -19.0, 16.0, 4.0], [9.0, 32.0, -51.0, 24.0, 10.0]], [[44.0, 13.0, -3.0, 17.0, 9.0], [-16.0, -25.0, -30.0, -29.0, 16.0], [2.0, 24.0, 11.0, -19.0, -34.0], [6.0, -21.0, 52.0, 0.0, -12.0]], [[-18.0, 8.0, 16.0, 34.0, 46.0], [6.0, 34.0, 14.0, -4.0, -54.0], [-30.0, -18.0, 12.0, 4.0, 36.0], [-36.0, -24.0, 2.0, -44.0, 0.0]]], [[[-3.0, -21.0, 7.0, -2.0, -4.0], [-9.0, -30.0, -7.0, -25.0, 23.0], [-7.0, 11.0, -3.0, -7.0, -40.0], [-6.0, -7.0, 17.0, 6.0, -8.0]], [[-26.0, -43.0, 13.0, -13.0, -13.0], [-8.0, -41.0, 2.0, -29.0, 34.0], [-12.0, 8.0, -11.0, -3.0, -54.0], [-12.0, -1.0, 4.0, 12.0, -8.0]], [[-38.0, -52.0, 28.0, 8.0, 14.0], [-11.0, -41.0, 5.0, -50.0, 16.0], [-36.0, 5.0, -5.0, -6.0, -60.0], [-39.0, -22.0, 19.0, -12.0, -14.0]], [[-8.0, 8.0, -16.0, -30.0, -32.0], [13.0, 19.0, 13.0, 42.0, 6.0], [26.0, -11.0, -11.0, 14.0, 32.0], [27.0, 34.0, -45.0, 24.0, 14.0]], [[22.0, -11.0, 5.0, 9.0, 3.0], [-17.0, -40.0, -23.0, -39.0, 28.0], [-6.0, 23.0, 4.0, -17.0, -54.0], [-3.0, -19.0, 45.0, 4.0, -14.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -36.0, -12.0, -54.0, -66.0], [-6.0, -54.0, -6.0, 0.0, 78.0], [30.0, 18.0, -24.0, 0.0, -60.0], [36.0, 36.0, -18.0, 60.0, 0.0]], [[-2.0, -28.0, -4.0, -30.0, -38.0], [-8.0, -44.0, -8.0, -12.0, 54.0], [14.0, 16.0, -14.0, -4.0, -52.0], [18.0, 16.0, 0.0, 36.0, -4.0]], [[-49.0, -22.0, 6.0, -19.0, -11.0], [14.0, 16.0, 30.0, 22.0, -9.0], [-5.0, -22.0, -13.0, 18.0, 22.0], [-9.0, 20.0, -50.0, 2.0, 10.0]], [[-29.0, -37.0, 7.0, -22.0, -22.0], [-2.0, -29.0, 8.0, -11.0, 31.0], [-3.0, 2.0, -14.0, 3.0, -36.0], [-3.0, 11.0, -14.0, 18.0, -2.0]], [[-12.0, -30.0, 2.0, -22.0, -26.0], [-6.0, -36.0, -2.0, -14.0, 40.0], [4.0, 10.0, -12.0, -2.0, -44.0], [6.0, 10.0, -2.0, 24.0, -4.0]]], [[[-31.0, -41.0, 3.0, -34.0, -36.0], [-1.0, -34.0, 9.0, -5.0, 43.0], [5.0, 3.0, -19.0, 5.0, -40.0], [6.0, 21.0, -23.0, 30.0, 0.0]], [[-31.0, -17.0, 3.0, -16.0, -12.0], [8.0, 5.0, 18.0, 13.0, 1.0], [-1.0, -12.0, -10.0, 11.0, 8.0], [-3.0, 15.0, -32.0, 6.0, 6.0]], [[-19.0, -40.0, -12.0, -61.0, -71.0], [-1.0, -47.0, 3.0, 10.0, 75.0], [31.0, 11.0, -28.0, 6.0, -50.0], [36.0, 44.0, -35.0, 62.0, 4.0]], [[-47.0, -37.0, 23.0, 6.0, 16.0], [1.0, -8.0, 19.0, -21.0, -9.0], [-31.0, -11.0, -5.0, 5.0, -16.0], [-36.0, -11.0, -9.0, -18.0, -4.0]], [[25.0, 16.0, -28.0, -39.0, -51.0], [4.0, -4.0, -8.0, 36.0, 37.0], [45.0, 8.0, -11.0, 4.0, 6.0], [51.0, 38.0, -24.0, 46.0, 10.0]]], [[[30.0, 27.0, -5.0, 19.0, 17.0], [-3.0, 12.0, -13.0, -1.0, -16.0], [2.0, 5.0, 12.0, -7.0, 14.0], [3.0, -13.0, 23.0, -12.0, -2.0]], [[-70.0, -70.0, 42.0, 14.0, 28.0], [-7.0, -35.0, 21.0, -56.0, 0.0], [-56.0, -7.0, -7.0, 0.0, -56.0], [-63.0, -28.0, 7.0, -28.0, -14.0]], [[-43.0, -26.0, 18.0, 5.0, 15.0], [5.0, 5.0, 21.0, -8.0, -17.0], [-25.0, -15.0, -4.0, 8.0, 2.0], [-30.0, -6.0, -17.0, -18.0, 0.0]], [[3.0, -1.0, 19.0, 40.0, 46.0], [-9.0, -2.0, -7.0, -37.0, -27.0], [-33.0, 3.0, 15.0, -11.0, -12.0], [-36.0, -39.0, 41.0, -38.0, -12.0]], [[-8.0, -40.0, -16.0, -66.0, -80.0], [-5.0, -59.0, -5.0, 6.0, 90.0], [38.0, 19.0, -29.0, 2.0, -64.0], [45.0, 46.0, -27.0, 72.0, 2.0]]], [[[28.0, 55.0, -9.0, 31.0, 35.0], [10.0, 59.0, 0.0, 29.0, -60.0], [2.0, -14.0, 19.0, 3.0, 74.0], [0.0, -11.0, 2.0, -32.0, 8.0]], [[-59.0, -51.0, 25.0, -4.0, 6.0], [1.0, -18.0, 23.0, -23.0, 3.0], [-31.0, -11.0, -11.0, 7.0, -28.0], [-36.0, -5.0, -17.0, -10.0, -4.0]], [[-26.0, -43.0, 13.0, -13.0, -13.0], [-8.0, -41.0, 2.0, -29.0, 34.0], [-12.0, 8.0, -11.0, -3.0, -54.0], [-12.0, -1.0, 4.0, 12.0, -8.0]], [[6.0, -22.0, -14.0, -44.0, -56.0], [-6.0, -44.0, -10.0, 2.0, 66.0], [30.0, 18.0, -18.0, -2.0, -48.0], [36.0, 30.0, -10.0, 52.0, 0.0]], [[-62.0, -26.0, 6.0, -26.0, -16.0], [19.0, 23.0, 39.0, 32.0, -12.0], [-4.0, -29.0, -17.0, 24.0, 32.0], [-9.0, 28.0, -67.0, 4.0, 14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -1.0, 5.0], [-5.0, 0.0, 5.0]], [[4.0, 6.0, 0.0], [0.0, -5.0, -2.0]], [[-1.0, -4.0, 1.0], [6.0, 0.0, 0.0]], [[-6.0, 6.0, 7.0], [7.0, 7.0, 2.0]], [[7.0, 1.0, 7.0], [5.0, -6.0, 2.0]]], [[[-3.0, 2.0, 4.0], [-5.0, 5.0, 0.0]], [[0.0, 3.0, 1.0], [7.0, 7.0, 6.0]], [[-3.0, -1.0, 3.0], [2.0, 6.0, -7.0]], [[3.0, 2.0, 1.0], [4.0, 7.0, -2.0]], [[-2.0, -6.0, -7.0], [0.0, -6.0, -7.0]]], [[[-1.0, -5.0, -5.0], [-4.0, 7.0, 0.0]], [[3.0, -4.0, 1.0], [1.0, -1.0, -7.0]], [[3.0, 6.0, 5.0], [5.0, 4.0, 0.0]], [[0.0, 1.0, 2.0], [3.0, 4.0, -4.0]], [[-6.0, -6.0, 5.0], [-7.0, 0.0, 7.0]]], [[[-2.0, 0.0, 7.0], [0.0, -7.0, -2.0]], [[-1.0, 0.0, -2.0], [-1.0, 4.0, 1.0]], [[-5.0, -7.0, -1.0], [-7.0, -3.0, -1.0]], [[-6.0, 0.0, -4.0], [-5.0, 0.0, -7.0]], [[4.0, 3.0, 1.0], [-2.0, 0.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, 7.0, 2.0], [5.0, -5.0, 1.0]], [[2.0, 4.0, 1.0], [-6.0, -6.0, -7.0]], [[0.0, -5.0, 5.0], [-7.0, -5.0, 0.0]], [[4.0, 2.0, -5.0], [5.0, 2.0, 0.0]], [[-6.0, -4.0, -4.0], [6.0, 3.0, 1.0]]], [[[-4.0, -1.0, -4.0], [-5.0, 1.0, 1.0]], [[-1.0, -1.0, 1.0], [0.0, -1.0, 4.0]], [[-6.0, 4.0, 5.0], [-4.0, 1.0, 1.0]], [[-7.0, -1.0, -3.0], [0.0, 0.0, 1.0]], [[4.0, 1.0, 0.0], [6.0, 0.0, -4.0]]], [[[-4.0, -2.0, -6.0], [-3.0, 6.0, -3.0]], [[2.0, -2.0, -4.0], [-1.0, -5.0, 4.0]], [[3.0, 0.0, -3.0], [7.0, 7.0, -3.0]], [[-1.0, 1.0, -5.0], [4.0, -6.0, 6.0]], [[-7.0, 7.0, 1.0], [4.0, -3.0, -6.0]]], [[[3.0, -4.0, -7.0], [5.0, 6.0, -2.0]], [[-6.0, 0.0, -1.0], [-2.0, -2.0, -3.0]], [[6.0, 7.0, 7.0], [0.0, 0.0, -7.0]], [[2.0, 3.0, -6.0], [-6.0, 7.0, 3.0]], [[2.0, 0.0, -3.0], [5.0, -5.0, 6.0]]]])
      arg1=Data(numpy.array([[[[-4.0, -3.0, -1.0, -6.0, -3.0], [2.0, 4.0, 5.0, -3.0, -5.0], [0.0, -5.0, 3.0, -1.0, 6.0], [1.0, -2.0, 7.0, -5.0, -3.0]], [[7.0, -3.0, -1.0, 0.0, -6.0], [-2.0, -1.0, 6.0, -6.0, -7.0], [5.0, -1.0, 0.0, -2.0, -3.0], [-7.0, 0.0, -5.0, 2.0, 7.0]], [[3.0, 0.0, 0.0, 2.0, 2.0], [6.0, 3.0, 2.0, 5.0, 0.0], [7.0, -3.0, 4.0, 1.0, 5.0], [4.0, 7.0, 6.0, -3.0, 6.0]]], [[[-3.0, -4.0, 0.0, 0.0, -1.0], [1.0, 1.0, -1.0, -1.0, -6.0], [0.0, 0.0, 7.0, 7.0, 4.0], [-5.0, -6.0, 3.0, -7.0, -4.0]], [[-4.0, 5.0, 6.0, 7.0, 3.0], [6.0, 0.0, 4.0, -2.0, -2.0], [0.0, 4.0, -4.0, -7.0, 2.0], [-2.0, 6.0, -5.0, -5.0, -6.0]], [[5.0, 4.0, -6.0, 4.0, -7.0], [-6.0, -3.0, -5.0, 3.0, 4.0], [-7.0, 2.0, -4.0, -6.0, 4.0], [-7.0, -1.0, 5.0, -6.0, -5.0]]]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, 25.0, -35.0, -6.0, -32.0], [9.0, 20.0, 14.0, 33.0, 27.0], [-5.0, -34.0, -17.0, -64.0, 64.0], [23.0, 48.0, 87.0, -42.0, 0.0]], [[36.0, -63.0, -28.0, -67.0, -49.0], [-22.0, 16.0, 46.0, -44.0, -60.0], [44.0, -50.0, 40.0, 31.0, -12.0], [-14.0, -36.0, 13.0, 29.0, 70.0]], [[-39.0, -9.0, 5.0, 8.0, 23.0], [18.0, 9.0, -33.0, 26.0, -3.0], [-13.0, 6.0, 43.0, 52.0, 35.0], [1.0, -27.0, 37.0, -48.0, -43.0]], [[48.0, 15.0, 30.0, 107.0, -4.0], [55.0, -8.0, 31.0, 2.0, -60.0], [65.0, 35.0, 23.0, -11.0, 31.0], [-83.0, 59.0, -34.0, -75.0, 22.0]], [[19.0, -66.0, -56.0, -62.0, -50.0], [11.0, 47.0, 16.0, 21.0, -52.0], [40.0, -77.0, 100.0, 63.0, 90.0], [1.0, -33.0, 141.0, -71.0, 34.0]]], [[[33.0, 48.0, 31.0, 61.0, 25.0], [39.0, -7.0, 30.0, 12.0, 21.0], [38.0, 21.0, -48.0, -67.0, -14.0], [14.0, 94.0, -47.0, 17.0, 37.0]], [[5.0, 22.0, 3.0, 75.0, -44.0], [13.0, -11.0, 11.0, -16.0, -53.0], [-20.0, 34.0, 1.0, -41.0, 62.0], [-108.0, 1.0, 7.0, -117.0, -73.0]], [[-51.0, 6.0, 82.0, 38.0, 86.0], [94.0, 21.0, 42.0, -5.0, -30.0], [65.0, 17.0, 21.0, 22.0, -8.0], [43.0, 58.0, -57.0, 2.0, 11.0]], [[-45.0, -4.0, 49.0, 25.0, 12.0], [66.0, 23.0, 63.0, -40.0, -75.0], [31.0, 4.0, 21.0, -15.0, 39.0], [-27.0, 21.0, -16.0, -65.0, -37.0]], [[-66.0, -34.0, 14.0, -72.0, 59.0], [-28.0, -2.0, -49.0, -2.0, 36.0], [-30.0, -1.0, 18.0, 91.0, -69.0], [73.0, -74.0, -31.0, 91.0, -7.0]]], [[[-62.0, 69.0, 48.0, 45.0, 48.0], [16.0, -18.0, -13.0, -2.0, 50.0], [-60.0, 53.0, -79.0, -71.0, -18.0], [20.0, 33.0, -59.0, 3.0, -88.0]], [[-71.0, -34.0, 37.0, -51.0, 62.0], [57.0, 41.0, 23.0, 0.0, -19.0], [36.0, -32.0, 52.0, 62.0, 9.0], [81.0, -4.0, 20.0, 14.0, 6.0]], [[14.0, -27.0, 15.0, 20.0, -28.0], [53.0, 26.0, 72.0, -33.0, -95.0], [65.0, -20.0, 48.0, -3.0, 53.0], [-52.0, 23.0, 16.0, -73.0, 19.0]], [[-32.0, -11.0, 47.0, 16.0, 35.0], [61.0, 20.0, 43.0, -19.0, -49.0], [47.0, 1.0, 29.0, 17.0, 11.0], [6.0, 24.0, -24.0, -21.0, 3.0]], [[53.0, 92.0, -30.0, 74.0, 22.0], [-19.0, -31.0, -84.0, 107.0, 142.0], [-44.0, 35.0, -75.0, -68.0, 7.0], [42.0, 82.0, 32.0, 10.0, -1.0]]], [[[47.0, -37.0, -28.0, -31.0, 13.0], [8.0, 19.0, -14.0, 49.0, 16.0], [63.0, -43.0, 58.0, 70.0, 1.0], [54.0, 13.0, 53.0, 36.0, 100.0]], [[-10.0, 31.0, 19.0, 34.0, 5.0], [3.0, -14.0, 3.0, -11.0, 7.0], [-21.0, 29.0, -38.0, -42.0, -8.0], [-19.0, 17.0, -37.0, -8.0, -34.0]], [[-4.0, 45.0, 0.0, 3.0, 60.0], [-21.0, -20.0, -69.0, 62.0, 118.0], [-35.0, 21.0, -52.0, -4.0, -52.0], [88.0, 28.0, -17.0, 84.0, 11.0]], [[-8.0, 10.0, 48.0, 0.0, 64.0], [1.0, -20.0, 2.0, -18.0, 32.0], [21.0, 28.0, -41.0, 9.0, -104.0], [52.0, 21.0, -116.0, 119.0, 49.0]], [[-21.0, -41.0, 35.0, -50.0, 23.0], [48.0, 35.0, 77.0, -44.0, -57.0], [71.0, -40.0, 30.0, 19.0, -16.0], [46.0, 18.0, -22.0, 39.0, 58.0]]]])+(1.-msk_ref)*numpy.array([[[[37.0, -83.0, -50.0, -69.0, -86.0], [-19.0, 29.0, 51.0, -45.0, -100.0], [42.0, -66.0, 80.0, 45.0, 45.0], [-56.0, -61.0, 71.0, -43.0, 45.0]], [[30.0, -52.0, 0.0, -80.0, 9.0], [2.0, 22.0, 53.0, -28.0, -18.0], [76.0, -55.0, 20.0, 33.0, -59.0], [69.0, 10.0, -23.0, 109.0, 123.0]], [[21.0, 18.0, -25.0, -25.0, 32.0], [3.0, 13.0, -33.0, 72.0, 87.0], [10.0, -30.0, -9.0, 1.0, 2.0], [100.0, 47.0, 59.0, 49.0, 53.0]], [[-40.0, -28.0, 6.0, -20.0, -33.0], [-9.0, 4.0, 25.0, -58.0, -68.0], [-25.0, 1.0, 19.0, 8.0, 17.0], [-59.0, -61.0, -7.0, -46.0, -60.0]], [[-41.0, 25.0, 22.0, 53.0, 30.0], [-10.0, -29.0, -61.0, 13.0, 20.0], [-55.0, 60.0, -8.0, 25.0, -10.0], [-37.0, -35.0, -38.0, -29.0, -81.0]]], [[[13.0, 44.0, 5.0, 27.0, 11.0], [-35.0, -35.0, -30.0, 4.0, 59.0], [-40.0, 39.0, -71.0, -46.0, -55.0], [3.0, 15.0, -62.0, 54.0, -10.0]], [[24.0, 17.0, -28.0, 17.0, -20.0], [-24.0, -12.0, -33.0, 28.0, 30.0], [-26.0, 7.0, -11.0, -13.0, 16.0], [-16.0, -1.0, 29.0, -19.0, -12.0]], [[80.0, 31.0, 2.0, 57.0, 4.0], [6.0, -20.0, 7.0, 24.0, 28.0], [48.0, 17.0, -34.0, -38.0, -33.0], [-3.0, 76.0, -44.0, 40.0, 81.0]], [[17.0, 28.0, 2.0, 40.0, 14.0], [-36.0, -39.0, -52.0, 15.0, 46.0], [-33.0, 47.0, -37.0, 0.0, -50.0], [-19.0, -8.0, -57.0, 36.0, -9.0]], [[-47.0, -55.0, 19.0, -40.0, 4.0], [36.0, 33.0, 40.0, -36.0, -79.0], [33.0, -29.0, 70.0, 60.0, 29.0], [-5.0, -40.0, 21.0, -36.0, -9.0]]], [[[-46.0, 48.0, 60.0, 42.0, 54.0], [11.0, -26.0, -2.0, -24.0, 28.0], [-31.0, 58.0, -69.0, -43.0, -60.0], [10.0, 23.0, -108.0, 43.0, -47.0]], [[9.0, -5.0, -54.0, -39.0, -44.0], [-71.0, -15.0, -49.0, 9.0, 36.0], [-66.0, -8.0, -13.0, 2.0, 0.0], [-13.0, -60.0, 42.0, 6.0, -30.0]], [[-85.0, -14.0, 57.0, 13.0, 20.0], [55.0, 19.0, 45.0, -54.0, -83.0], [0.0, 16.0, 30.0, 12.0, 33.0], [-37.0, -24.0, -26.0, -72.0, -82.0]], [[38.0, -22.0, -72.0, -22.0, -77.0], [-102.0, -34.0, -67.0, -2.0, 10.0], [-72.0, 7.0, 5.0, 28.0, -6.0], [-78.0, -99.0, 30.0, -12.0, -30.0]], [[50.0, -55.0, 18.0, -1.0, 10.0], [0.0, -10.0, 23.0, -32.0, -56.0], [84.0, 1.0, 47.0, 79.0, -72.0], [-24.0, -15.0, -81.0, 69.0, 108.0]]], [[[-110.0, 5.0, 49.0, 2.0, 28.0], [25.0, 6.0, 6.0, -43.0, -37.0], [-55.0, 30.0, 0.0, 3.0, 19.0], [-20.0, -47.0, -26.0, -55.0, -125.0]], [[20.0, 4.0, 12.0, 8.0, 33.0], [-14.0, -20.0, -23.0, 10.0, 34.0], [14.0, 19.0, -16.0, 23.0, -65.0], [25.0, 8.0, -59.0, 75.0, 47.0]], [[11.0, -67.0, 29.0, -50.0, 3.0], [82.0, 59.0, 121.0, -46.0, -107.0], [133.0, -72.0, 74.0, 29.0, 22.0], [34.0, 44.0, 14.0, 5.0, 108.0]], [[0.0, 56.0, 19.0, 37.0, -30.0], [-20.0, -28.0, 35.0, -53.0, 3.0], [-48.0, 39.0, -100.0, -123.0, -25.0], [-48.0, 29.0, -75.0, 3.0, -54.0]], [[18.0, -27.0, -68.0, -29.0, -74.0], [-75.0, -14.0, -51.0, 2.0, -6.0], [-63.0, -9.0, 25.0, 29.0, 31.0], [-67.0, -91.0, 66.0, -47.0, -44.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, -7.0, 2.0, 6.0, -6.0], [-3.0, 4.0, -6.0, -2.0, -3.0], [-2.0, 0.0, 0.0, -5.0, 6.0], [2.0, -3.0, 3.0, -3.0, 6.0]])+(1.-msk_arg0)*numpy.array([[0.0, -3.0, -7.0, -6.0, -2.0], [-7.0, 0.0, 6.0, 2.0, -4.0], [0.0, -1.0, 5.0, 7.0, 2.0], [7.0, 5.0, 2.0, -6.0, -3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-7.0)+(1-msk_arg1)*(-6.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[28.0, 49.0, -14.0, -42.0, 42.0], [21.0, -28.0, 42.0, 14.0, 21.0], [14.0, -0.0, -0.0, 35.0, -42.0], [-14.0, 21.0, -21.0, 21.0, -42.0]])+(1.-msk_ref)*numpy.array([[-0.0, 18.0, 42.0, 36.0, 12.0], [42.0, -0.0, -36.0, -12.0, 24.0], [-0.0, 6.0, -30.0, -42.0, -12.0], [-42.0, -30.0, -12.0, 36.0, 18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, 0.0, -4.0], [4.0, 3.0, -7.0], [4.0, 0.0, -2.0], [-7.0, 4.0, 5.0], [-7.0, -1.0, -3.0]], [[0.0, 4.0, 2.0], [-7.0, 4.0, 2.0], [0.0, -5.0, 0.0], [0.0, 6.0, -5.0], [5.0, -4.0, -5.0]], [[-5.0, 1.0, -3.0], [-6.0, 3.0, 2.0], [0.0, -2.0, -7.0], [-2.0, -5.0, -6.0], [-2.0, -5.0, -1.0]], [[-3.0, -1.0, 4.0], [0.0, 6.0, -5.0], [-3.0, -6.0, 5.0], [3.0, -5.0, -4.0], [-2.0, 4.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, 0.0, 7.0], [-7.0, 1.0, 0.0], [7.0, -7.0, 4.0], [0.0, 0.0, 2.0], [-4.0, 4.0, -7.0]], [[3.0, 2.0, 4.0], [0.0, -3.0, -4.0], [-6.0, -7.0, -4.0], [6.0, -4.0, -3.0], [6.0, -2.0, 0.0]], [[-5.0, 5.0, 7.0], [3.0, 5.0, 0.0], [3.0, -7.0, 3.0], [6.0, 2.0, -3.0], [0.0, -3.0, 1.0]], [[-1.0, -1.0, 6.0], [4.0, 7.0, 6.0], [3.0, -7.0, 4.0], [2.0, 7.0, 0.0], [7.0, 3.0, -3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, 4.0, -3.0])+(1.-msk_arg1)*numpy.array([-2.0, 7.0, 2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[28.0, 49.0, 22.0, -27.0, -23.0], [10.0, -18.0, -20.0, 39.0, 19.0], [-7.0, -18.0, 13.0, -10.0, -25.0], [-28.0, 39.0, -51.0, 4.0, 23.0]])+(1.-msk_ref)*numpy.array([[22.0, 21.0, -55.0, 4.0, 22.0], [16.0, -29.0, -45.0, -46.0, -26.0], [59.0, 29.0, -49.0, -4.0, -19.0], [7.0, 53.0, -47.0, 45.0, 1.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, 2.0, -6.0], [2.0, -4.0, -6.0]], [[2.0, -3.0, 1.0], [3.0, 1.0, -3.0]], [[0.0, 0.0, 4.0], [0.0, 5.0, 0.0]], [[0.0, 0.0, 7.0], [5.0, -4.0, 5.0]], [[-4.0, -5.0, -2.0], [4.0, -4.0, -1.0]]], [[[-1.0, -4.0, 0.0], [5.0, 0.0, -2.0]], [[3.0, 4.0, 2.0], [-7.0, 3.0, -2.0]], [[-2.0, 0.0, -4.0], [6.0, 3.0, -7.0]], [[0.0, 7.0, -1.0], [0.0, 2.0, 2.0]], [[-5.0, 5.0, 6.0], [6.0, -4.0, 2.0]]], [[[-2.0, 4.0, -5.0], [2.0, 4.0, 4.0]], [[5.0, 3.0, 4.0], [-6.0, -7.0, -7.0]], [[-7.0, -1.0, 4.0], [0.0, -2.0, -3.0]], [[-2.0, -7.0, -3.0], [0.0, -2.0, -1.0]], [[6.0, 6.0, -5.0], [3.0, -7.0, -1.0]]], [[[-2.0, 4.0, -6.0], [6.0, 6.0, 2.0]], [[7.0, 2.0, 6.0], [2.0, 5.0, -4.0]], [[0.0, 5.0, 0.0], [6.0, 4.0, 7.0]], [[2.0, -4.0, -2.0], [5.0, 2.0, -4.0]], [[5.0, 0.0, -5.0], [-3.0, 2.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, -5.0, -5.0], [0.0, 3.0, -2.0]], [[1.0, -2.0, -1.0], [-6.0, 7.0, 2.0]], [[7.0, -3.0, 1.0], [7.0, -6.0, 1.0]], [[4.0, 4.0, -5.0], [-7.0, -7.0, -1.0]], [[0.0, -5.0, 2.0], [7.0, 1.0, 0.0]]], [[[4.0, -6.0, 2.0], [-4.0, -5.0, -7.0]], [[2.0, 1.0, 2.0], [-5.0, 6.0, -1.0]], [[-2.0, 1.0, 0.0], [-4.0, -5.0, -6.0]], [[7.0, 3.0, -6.0], [2.0, 0.0, -6.0]], [[4.0, 2.0, -4.0], [-1.0, 3.0, -2.0]]], [[[7.0, 0.0, 7.0], [4.0, 2.0, 1.0]], [[-6.0, 3.0, -3.0], [-7.0, -2.0, 0.0]], [[-5.0, -3.0, 1.0], [-2.0, -3.0, 1.0]], [[6.0, -5.0, 6.0], [0.0, 4.0, -2.0]], [[-3.0, 1.0, -4.0], [0.0, 6.0, -6.0]]], [[[-1.0, -2.0, -4.0], [-7.0, -7.0, -7.0]], [[-6.0, -2.0, 5.0], [0.0, -3.0, -3.0]], [[0.0, 6.0, 7.0], [2.0, -5.0, 7.0]], [[1.0, 6.0, 4.0], [-4.0, -6.0, -1.0]], [[-1.0, 7.0, 0.0], [6.0, -7.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 3.0, 0.0], [-4.0, -2.0, 5.0]])+(1.-msk_arg1)*numpy.array([[0.0, -1.0, -7.0], [-7.0, 7.0, 7.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-12.0, -44.0, -10.0, 13.0, -16.0], [-39.0, 15.0, -59.0, 27.0, 24.0], [22.0, -3.0, 7.0, -16.0, -3.0], [-8.0, -53.0, 18.0, -62.0, -7.0]])+(1.-msk_ref)*numpy.array([[47.0, 114.0, -88.0, 24.0, -51.0], [-64.0, 55.0, -50.0, -17.0, 40.0], [-56.0, 53.0, -4.0, -23.0, 27.0], [-19.0, -75.0, -55.0, -55.0, -119.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 6.0, 7.0, 7.0, 1.0], [1.0, -7.0, -3.0, 6.0, 0.0], [-5.0, -6.0, 1.0, -3.0, -7.0], [4.0, 3.0, 1.0, 0.0, -1.0]])+(1.-msk_arg0)*numpy.array([[1.0, -2.0, 4.0, 6.0, 7.0], [0.0, -5.0, 4.0, -7.0, 0.0], [-2.0, -1.0, 4.0, -3.0, -6.0], [3.0, 1.0, 3.0, 4.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, -7.0])+(1.-msk_arg1)*numpy.array([3.0, -7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-24.0, 42.0], [24.0, -42.0], [28.0, -49.0], [28.0, -49.0], [4.0, -7.0]], [[4.0, -7.0], [-28.0, 49.0], [-12.0, 21.0], [24.0, -42.0], [0.0, 0.0]], [[-20.0, 35.0], [-24.0, 42.0], [4.0, -7.0], [-12.0, 21.0], [-28.0, 49.0]], [[16.0, -28.0], [12.0, -21.0], [4.0, -7.0], [0.0, 0.0], [-4.0, 7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, -7.0], [-6.0, 14.0], [12.0, -28.0], [18.0, -42.0], [21.0, -49.0]], [[0.0, 0.0], [-15.0, 35.0], [12.0, -28.0], [-21.0, 49.0], [0.0, 0.0]], [[-6.0, 14.0], [-3.0, 7.0], [12.0, -28.0], [-9.0, 21.0], [-18.0, 42.0]], [[9.0, -21.0], [3.0, -7.0], [9.0, -21.0], [12.0, -28.0], [12.0, -28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 0.0, -1.0], [-2.0, -4.0, 5.0], [1.0, 6.0, 6.0], [-7.0, -6.0, 0.0], [-6.0, 7.0, 3.0]], [[3.0, -2.0, 5.0], [-1.0, 2.0, -5.0], [7.0, 0.0, 5.0], [-1.0, 2.0, 4.0], [-2.0, 3.0, -1.0]], [[-6.0, 6.0, 3.0], [-2.0, 6.0, -7.0], [-5.0, 3.0, -4.0], [-3.0, 0.0, -6.0], [3.0, 4.0, 2.0]], [[-4.0, 3.0, 6.0], [-7.0, -6.0, -3.0], [5.0, 5.0, 0.0], [7.0, 5.0, 3.0], [0.0, 3.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, 5.0, -3.0], [-1.0, -3.0, 6.0], [5.0, -3.0, 7.0], [7.0, 2.0, 3.0], [7.0, -1.0, 3.0]], [[-5.0, -4.0, 4.0], [3.0, 6.0, -4.0], [-2.0, 0.0, 0.0], [7.0, 1.0, 2.0], [-5.0, 5.0, 5.0]], [[1.0, 6.0, 2.0], [3.0, 3.0, -5.0], [4.0, -2.0, -3.0], [-3.0, 0.0, 5.0], [1.0, 5.0, -2.0]], [[7.0, 6.0, 3.0], [-1.0, -1.0, 2.0], [2.0, 5.0, 4.0], [-6.0, 0.0, 0.0], [-5.0, 2.0, 6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, -3.0], [6.0, 3.0], [-7.0, -1.0]])+(1.-msk_arg1)*numpy.array([[0.0, 2.0], [4.0, 3.0], [-7.0, 6.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, -2.0], [-53.0, -11.0], [-9.0, 9.0], [-15.0, 3.0], [39.0, 36.0]], [[-56.0, -20.0], [50.0, 14.0], [-56.0, -26.0], [-13.0, 5.0], [31.0, 16.0]], [[33.0, 33.0], [91.0, 31.0], [61.0, 28.0], [51.0, 15.0], [1.0, 1.0]], [[-12.0, 15.0], [6.0, 6.0], [15.0, 0.0], [-12.0, -9.0], [53.0, 14.0]]])+(1.-msk_ref)*numpy.array([[[41.0, -1.0], [-54.0, 25.0], [-61.0, 43.0], [-13.0, 38.0], [-25.0, 29.0]], [[-44.0, 2.0], [52.0, 0.0], [0.0, -4.0], [-10.0, 29.0], [-15.0, 35.0]], [[10.0, 32.0], [47.0, -15.0], [13.0, -16.0], [-35.0, 24.0], [34.0, 5.0]], [[3.0, 50.0], [-18.0, 7.0], [-8.0, 43.0], [0.0, -12.0], [-34.0, 32.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 3.0, -4.0], [-6.0, -2.0, 2.0]], [[2.0, -3.0, 1.0], [5.0, -1.0, -5.0]], [[4.0, -2.0, 3.0], [-2.0, 4.0, -2.0]], [[-3.0, -4.0, -2.0], [1.0, -7.0, 0.0]], [[3.0, -3.0, -2.0], [-6.0, 0.0, 1.0]]], [[[0.0, -5.0, -2.0], [0.0, 1.0, 3.0]], [[0.0, 5.0, 3.0], [4.0, 3.0, -6.0]], [[4.0, -7.0, 3.0], [2.0, 0.0, 7.0]], [[2.0, -7.0, 7.0], [-5.0, 2.0, -2.0]], [[7.0, 0.0, -5.0], [0.0, 0.0, -4.0]]], [[[1.0, 3.0, 6.0], [4.0, 1.0, 2.0]], [[-2.0, 4.0, 2.0], [0.0, -5.0, 6.0]], [[-1.0, -1.0, -2.0], [1.0, -6.0, -2.0]], [[0.0, 5.0, -2.0], [-3.0, -4.0, 0.0]], [[-6.0, 1.0, -1.0], [0.0, -1.0, -6.0]]], [[[5.0, -6.0, -5.0], [7.0, -4.0, 0.0]], [[5.0, -6.0, 4.0], [-7.0, 7.0, 4.0]], [[-7.0, 0.0, -3.0], [3.0, -1.0, -6.0]], [[6.0, -6.0, -7.0], [-6.0, -4.0, 4.0]], [[0.0, -5.0, -2.0], [4.0, -1.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, -6.0, 6.0], [0.0, 2.0, 5.0]], [[0.0, 3.0, 7.0], [-7.0, -4.0, -1.0]], [[2.0, 2.0, 0.0], [3.0, 0.0, -1.0]], [[-1.0, -3.0, 3.0], [-2.0, 3.0, 0.0]], [[-2.0, -6.0, 0.0], [-3.0, -6.0, 2.0]]], [[[-4.0, 7.0, -5.0], [-1.0, 7.0, 4.0]], [[3.0, 0.0, 2.0], [0.0, 4.0, 6.0]], [[-2.0, -2.0, 0.0], [3.0, -3.0, -5.0]], [[5.0, -4.0, 1.0], [-5.0, -3.0, 5.0]], [[6.0, 0.0, 7.0], [0.0, -6.0, 5.0]]], [[[7.0, 2.0, 7.0], [6.0, 0.0, 7.0]], [[3.0, 5.0, -2.0], [-3.0, 0.0, -7.0]], [[3.0, -5.0, 3.0], [-6.0, -2.0, 0.0]], [[6.0, -5.0, -3.0], [0.0, 1.0, 0.0]], [[1.0, -4.0, -6.0], [3.0, 6.0, -5.0]]], [[[-1.0, -3.0, -6.0], [3.0, 6.0, -4.0]], [[3.0, 0.0, 3.0], [0.0, -4.0, 7.0]], [[0.0, 1.0, -5.0], [-5.0, -6.0, 4.0]], [[4.0, -5.0, -5.0], [-4.0, -5.0, 7.0]], [[6.0, -3.0, -4.0], [6.0, 6.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 3.0], [6.0, 0.0], [6.0, 6.0]], [[-3.0, 2.0], [7.0, 4.0], [3.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, 6.0], [-2.0, -3.0], [-4.0, 5.0]], [[-4.0, 3.0], [-5.0, -6.0], [1.0, -3.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, -50.0], [-47.0, 33.0], [38.0, 48.0], [-91.0, -47.0], [-6.0, -18.0]], [[-26.0, -17.0], [39.0, 56.0], [-5.0, 13.0], [25.0, 52.0], [-35.0, 3.0]], [[56.0, 45.0], [17.0, -32.0], [-70.0, -31.0], [-1.0, -34.0], [-31.0, -10.0]], [[-110.0, -17.0], [75.0, 41.0], [-59.0, -19.0], [-70.0, -64.0], [-67.0, -2.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 27.0], [13.0, 32.0], [-3.0, 18.0], [-20.0, -6.0], [42.0, 27.0]], [[-49.0, -127.0], [-1.0, -14.0], [-12.0, 36.0], [79.0, 35.0], [49.0, 92.0]], [[0.0, 68.0], [24.0, 5.0], [53.0, 42.0], [59.0, 30.0], [-8.0, -24.0]], [[-23.0, -42.0], [36.0, 36.0], [72.0, -19.0], [106.0, 11.0], [10.0, 7.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank2_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 0.0, -7.0, -4.0, 1.0], [2.0, -5.0, -7.0, -1.0, 2.0], [0.0, -4.0, 5.0, -7.0, 1.0], [-1.0, -7.0, -6.0, -1.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-3.0, -1.0, -1.0, -5.0, 3.0], [3.0, -2.0, -5.0, 6.0, 3.0], [6.0, 3.0, -3.0, 2.0, -6.0], [4.0, 3.0, 6.0, 2.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, 6.0, 6.0, 0.0, 1.0], [-1.0, 4.0, -5.0, -3.0, -6.0], [-1.0, 2.0, 5.0, 4.0, 0.0], [-4.0, 6.0, 6.0, -1.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-6.0, -7.0, -4.0, -1.0, 7.0], [0.0, 7.0, -5.0, 0.0, 0.0], [-2.0, 0.0, 4.0, -1.0, -3.0], [-7.0, 7.0, 2.0, 0.0, -1.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[25.0, 30.0, 30.0, 0.0, 5.0], [-5.0, 20.0, -25.0, -15.0, -30.0], [-5.0, 10.0, 25.0, 20.0, 0.0], [-20.0, 30.0, 30.0, -5.0, -30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-35.0, -42.0, -42.0, 0.0, -7.0], [7.0, -28.0, 35.0, 21.0, 42.0], [7.0, -14.0, -35.0, -28.0, 0.0], [28.0, -42.0, -42.0, 7.0, 42.0]], [[-20.0, -24.0, -24.0, 0.0, -4.0], [4.0, -16.0, 20.0, 12.0, 24.0], [4.0, -8.0, -20.0, -16.0, 0.0], [16.0, -24.0, -24.0, 4.0, 24.0]], [[5.0, 6.0, 6.0, 0.0, 1.0], [-1.0, 4.0, -5.0, -3.0, -6.0], [-1.0, 2.0, 5.0, 4.0, 0.0], [-4.0, 6.0, 6.0, -1.0, -6.0]]], [[[10.0, 12.0, 12.0, 0.0, 2.0], [-2.0, 8.0, -10.0, -6.0, -12.0], [-2.0, 4.0, 10.0, 8.0, 0.0], [-8.0, 12.0, 12.0, -2.0, -12.0]], [[-25.0, -30.0, -30.0, 0.0, -5.0], [5.0, -20.0, 25.0, 15.0, 30.0], [5.0, -10.0, -25.0, -20.0, 0.0], [20.0, -30.0, -30.0, 5.0, 30.0]], [[-35.0, -42.0, -42.0, 0.0, -7.0], [7.0, -28.0, 35.0, 21.0, 42.0], [7.0, -14.0, -35.0, -28.0, 0.0], [28.0, -42.0, -42.0, 7.0, 42.0]], [[-5.0, -6.0, -6.0, 0.0, -1.0], [1.0, -4.0, 5.0, 3.0, 6.0], [1.0, -2.0, -5.0, -4.0, 0.0], [4.0, -6.0, -6.0, 1.0, 6.0]], [[10.0, 12.0, 12.0, 0.0, 2.0], [-2.0, 8.0, -10.0, -6.0, -12.0], [-2.0, 4.0, 10.0, 8.0, 0.0], [-8.0, 12.0, 12.0, -2.0, -12.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-20.0, -24.0, -24.0, 0.0, -4.0], [4.0, -16.0, 20.0, 12.0, 24.0], [4.0, -8.0, -20.0, -16.0, 0.0], [16.0, -24.0, -24.0, 4.0, 24.0]], [[25.0, 30.0, 30.0, 0.0, 5.0], [-5.0, 20.0, -25.0, -15.0, -30.0], [-5.0, 10.0, 25.0, 20.0, 0.0], [-20.0, 30.0, 30.0, -5.0, -30.0]], [[-35.0, -42.0, -42.0, 0.0, -7.0], [7.0, -28.0, 35.0, 21.0, 42.0], [7.0, -14.0, -35.0, -28.0, 0.0], [28.0, -42.0, -42.0, 7.0, 42.0]], [[5.0, 6.0, 6.0, 0.0, 1.0], [-1.0, 4.0, -5.0, -3.0, -6.0], [-1.0, 2.0, 5.0, 4.0, 0.0], [-4.0, 6.0, 6.0, -1.0, -6.0]]], [[[-5.0, -6.0, -6.0, 0.0, -1.0], [1.0, -4.0, 5.0, 3.0, 6.0], [1.0, -2.0, -5.0, -4.0, 0.0], [4.0, -6.0, -6.0, 1.0, 6.0]], [[-35.0, -42.0, -42.0, 0.0, -7.0], [7.0, -28.0, 35.0, 21.0, 42.0], [7.0, -14.0, -35.0, -28.0, 0.0], [28.0, -42.0, -42.0, 7.0, 42.0]], [[-30.0, -36.0, -36.0, 0.0, -6.0], [6.0, -24.0, 30.0, 18.0, 36.0], [6.0, -12.0, -30.0, -24.0, 0.0], [24.0, -36.0, -36.0, 6.0, 36.0]], [[-5.0, -6.0, -6.0, 0.0, -1.0], [1.0, -4.0, 5.0, 3.0, 6.0], [1.0, -2.0, -5.0, -4.0, 0.0], [4.0, -6.0, -6.0, 1.0, 6.0]], [[25.0, 30.0, 30.0, 0.0, 5.0], [-5.0, 20.0, -25.0, -15.0, -30.0], [-5.0, 10.0, 25.0, 20.0, 0.0], [-20.0, 30.0, 30.0, -5.0, -30.0]]]])+(1.-msk_ref)*numpy.array([[[[18.0, 21.0, 12.0, 3.0, -21.0], [0.0, -21.0, 15.0, 0.0, 0.0], [6.0, 0.0, -12.0, 3.0, 9.0], [21.0, -21.0, -6.0, 0.0, 3.0]], [[6.0, 7.0, 4.0, 1.0, -7.0], [0.0, -7.0, 5.0, 0.0, 0.0], [2.0, 0.0, -4.0, 1.0, 3.0], [7.0, -7.0, -2.0, 0.0, 1.0]], [[6.0, 7.0, 4.0, 1.0, -7.0], [0.0, -7.0, 5.0, 0.0, 0.0], [2.0, 0.0, -4.0, 1.0, 3.0], [7.0, -7.0, -2.0, 0.0, 1.0]], [[30.0, 35.0, 20.0, 5.0, -35.0], [0.0, -35.0, 25.0, 0.0, 0.0], [10.0, 0.0, -20.0, 5.0, 15.0], [35.0, -35.0, -10.0, 0.0, 5.0]], [[-18.0, -21.0, -12.0, -3.0, 21.0], [0.0, 21.0, -15.0, 0.0, 0.0], [-6.0, 0.0, 12.0, -3.0, -9.0], [-21.0, 21.0, 6.0, 0.0, -3.0]]], [[[-18.0, -21.0, -12.0, -3.0, 21.0], [0.0, 21.0, -15.0, 0.0, 0.0], [-6.0, 0.0, 12.0, -3.0, -9.0], [-21.0, 21.0, 6.0, 0.0, -3.0]], [[12.0, 14.0, 8.0, 2.0, -14.0], [0.0, -14.0, 10.0, 0.0, 0.0], [4.0, 0.0, -8.0, 2.0, 6.0], [14.0, -14.0, -4.0, 0.0, 2.0]], [[30.0, 35.0, 20.0, 5.0, -35.0], [0.0, -35.0, 25.0, 0.0, 0.0], [10.0, 0.0, -20.0, 5.0, 15.0], [35.0, -35.0, -10.0, 0.0, 5.0]], [[-36.0, -42.0, -24.0, -6.0, 42.0], [0.0, 42.0, -30.0, 0.0, 0.0], [-12.0, 0.0, 24.0, -6.0, -18.0], [-42.0, 42.0, 12.0, 0.0, -6.0]], [[-18.0, -21.0, -12.0, -3.0, 21.0], [0.0, 21.0, -15.0, 0.0, 0.0], [-6.0, 0.0, 12.0, -3.0, -9.0], [-21.0, 21.0, 6.0, 0.0, -3.0]]], [[[-36.0, -42.0, -24.0, -6.0, 42.0], [0.0, 42.0, -30.0, 0.0, 0.0], [-12.0, 0.0, 24.0, -6.0, -18.0], [-42.0, 42.0, 12.0, 0.0, -6.0]], [[-18.0, -21.0, -12.0, -3.0, 21.0], [0.0, 21.0, -15.0, 0.0, 0.0], [-6.0, 0.0, 12.0, -3.0, -9.0], [-21.0, 21.0, 6.0, 0.0, -3.0]], [[18.0, 21.0, 12.0, 3.0, -21.0], [0.0, -21.0, 15.0, 0.0, 0.0], [6.0, 0.0, -12.0, 3.0, 9.0], [21.0, -21.0, -6.0, 0.0, 3.0]], [[-12.0, -14.0, -8.0, -2.0, 14.0], [0.0, 14.0, -10.0, 0.0, 0.0], [-4.0, 0.0, 8.0, -2.0, -6.0], [-14.0, 14.0, 4.0, 0.0, -2.0]], [[36.0, 42.0, 24.0, 6.0, -42.0], [0.0, -42.0, 30.0, 0.0, 0.0], [12.0, 0.0, -24.0, 6.0, 18.0], [42.0, -42.0, -12.0, 0.0, 6.0]]], [[[-24.0, -28.0, -16.0, -4.0, 28.0], [0.0, 28.0, -20.0, 0.0, 0.0], [-8.0, 0.0, 16.0, -4.0, -12.0], [-28.0, 28.0, 8.0, 0.0, -4.0]], [[-18.0, -21.0, -12.0, -3.0, 21.0], [0.0, 21.0, -15.0, 0.0, 0.0], [-6.0, 0.0, 12.0, -3.0, -9.0], [-21.0, 21.0, 6.0, 0.0, -3.0]], [[-36.0, -42.0, -24.0, -6.0, 42.0], [0.0, 42.0, -30.0, 0.0, 0.0], [-12.0, 0.0, 24.0, -6.0, -18.0], [-42.0, 42.0, 12.0, 0.0, -6.0]], [[-12.0, -14.0, -8.0, -2.0, 14.0], [0.0, 14.0, -10.0, 0.0, 0.0], [-4.0, 0.0, 8.0, -2.0, -6.0], [-14.0, 14.0, 4.0, 0.0, -2.0]], [[-30.0, -35.0, -20.0, -5.0, 35.0], [0.0, 35.0, -25.0, 0.0, 0.0], [-10.0, 0.0, 20.0, -5.0, -15.0], [-35.0, 35.0, 10.0, 0.0, -5.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 7.0, 4.0], [0.0, 5.0, -3.0], [-2.0, 7.0, -3.0], [0.0, -4.0, 0.0], [0.0, -2.0, 2.0]], [[-5.0, -1.0, 4.0], [-6.0, 4.0, -3.0], [-3.0, 7.0, -1.0], [-1.0, 3.0, 5.0], [-4.0, -3.0, 6.0]], [[-6.0, -3.0, 7.0], [7.0, 6.0, 2.0], [2.0, 5.0, -6.0], [6.0, -7.0, -5.0], [-4.0, 0.0, -2.0]], [[4.0, -1.0, -3.0], [-7.0, 7.0, -1.0], [-6.0, -4.0, 7.0], [-7.0, -3.0, 0.0], [1.0, 5.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 0.0, -1.0], [-7.0, -7.0, 6.0], [-5.0, 5.0, -5.0], [-1.0, -5.0, -2.0], [-5.0, -3.0, 5.0]], [[-5.0, 7.0, 3.0], [1.0, -1.0, 4.0], [1.0, 2.0, 2.0], [2.0, -2.0, -7.0], [-1.0, 6.0, 1.0]], [[7.0, -1.0, -3.0], [-4.0, 2.0, 6.0], [-3.0, 0.0, -7.0], [-2.0, -1.0, -6.0], [-7.0, 0.0, -4.0]], [[-3.0, -1.0, 2.0], [-7.0, 0.0, -6.0], [6.0, -1.0, 5.0], [0.0, -3.0, -2.0], [0.0, 5.0, 7.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-2.0, -1.0, -6.0, 0.0, -6.0], [-1.0, -7.0, -5.0, 5.0, -6.0], [1.0, -5.0, -7.0, 6.0, -5.0], [4.0, -1.0, -1.0, -1.0, 0.0]], [[-4.0, 7.0, -7.0, 1.0, 2.0], [1.0, -7.0, -1.0, -5.0, -6.0], [0.0, 2.0, 5.0, 7.0, -1.0], [7.0, 4.0, -2.0, -3.0, 5.0]], [[-4.0, 0.0, 0.0, 4.0, 7.0], [-3.0, 0.0, -5.0, -4.0, -3.0], [-1.0, 7.0, -2.0, -3.0, 6.0], [0.0, 2.0, 7.0, 5.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, -4.0, -5.0, -7.0, 4.0], [1.0, 2.0, 2.0, -3.0, -3.0], [0.0, -6.0, 5.0, 2.0, -6.0], [0.0, -4.0, 1.0, -7.0, 0.0]], [[-7.0, -7.0, -2.0, 7.0, -6.0], [-1.0, 2.0, -7.0, 3.0, -4.0], [6.0, -6.0, -3.0, 7.0, 4.0], [7.0, 0.0, -5.0, -6.0, -5.0]], [[-6.0, 5.0, -4.0, 6.0, -1.0], [-1.0, -3.0, 7.0, 5.0, 0.0], [-7.0, 5.0, 2.0, -1.0, -4.0], [-6.0, -1.0, -2.0, 4.0, 0.0]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-30.0, 56.0, -7.0, 23.0, 84.0], [2.0, 0.0, 8.0, -86.0, -12.0], [-11.0, 77.0, 76.0, -5.0, 52.0], [21.0, 43.0, 21.0, 6.0, 39.0]], [[-8.0, 35.0, -35.0, -7.0, -11.0], [14.0, -35.0, 10.0, -13.0, -21.0], [3.0, -11.0, 31.0, 44.0, -23.0], [35.0, 14.0, -31.0, -30.0, 22.0]], [[-12.0, 51.0, -37.0, -5.0, 5.0], [18.0, -35.0, 18.0, -33.0, -21.0], [1.0, 3.0, 55.0, 46.0, -15.0], [41.0, 24.0, -33.0, -34.0, 32.0]], [[16.0, -28.0, 28.0, -4.0, -8.0], [-4.0, 28.0, 4.0, 20.0, 24.0], [0.0, -8.0, -20.0, -28.0, 4.0], [-28.0, -16.0, 8.0, 12.0, -20.0]], [[0.0, -14.0, 14.0, 6.0, 10.0], [-8.0, 14.0, -8.0, 2.0, 6.0], [-2.0, 10.0, -14.0, -20.0, 14.0], [-14.0, -4.0, 18.0, 16.0, -8.0]]], [[[-2.0, -2.0, 37.0, 15.0, 56.0], [-8.0, 42.0, 6.0, -36.0, 24.0], [-9.0, 51.0, 22.0, -49.0, 50.0], [-27.0, 9.0, 35.0, 28.0, -1.0]], [[8.0, 34.0, 8.0, -8.0, 23.0], [19.0, 14.0, 41.0, -38.0, 21.0], [-3.0, 17.0, 68.0, 1.0, 8.0], [4.0, 16.0, -23.0, -21.0, 17.0]], [[-18.0, 52.0, -31.0, 3.0, 25.0], [13.0, -28.0, 13.0, -46.0, -21.0], [-2.0, 22.0, 58.0, 34.0, 2.0], [37.0, 29.0, -18.0, -23.0, 34.0]], [[-30.0, 22.0, -15.0, 23.0, 47.0], [-11.0, -14.0, -23.0, -40.0, -27.0], [-6.0, 46.0, 12.0, 0.0, 32.0], [17.0, 23.0, 30.0, 17.0, 20.0]], [[-4.0, -17.0, 45.0, 21.0, 60.0], [-17.0, 49.0, -7.0, -29.0, 24.0], [-10.0, 56.0, 1.0, -63.0, 59.0], [-37.0, 4.0, 52.0, 43.0, -9.0]]], [[[-4.0, -15.0, 57.0, 25.0, 79.0], [-18.0, 63.0, -2.0, -43.0, 33.0], [-13.0, 73.0, 13.0, -78.0, 75.0], [-45.0, 8.0, 61.0, 50.0, -8.0]], [[-46.0, 35.0, -84.0, 14.0, -16.0], [-7.0, -91.0, -51.0, -3.0, -84.0], [5.0, -9.0, -23.0, 78.0, -29.0], [70.0, 21.0, -5.0, -15.0, 32.0]], [[0.0, 33.0, -47.0, -19.0, -44.0], [21.0, -49.0, 15.0, 9.0, -24.0], [8.0, -42.0, 23.0, 65.0, -51.0], [43.0, 6.0, -54.0, -47.0, 19.0]], [[36.0, -55.0, 13.0, -27.0, -85.0], [2.0, 7.0, 2.0, 85.0, 21.0], [11.0, -79.0, -67.0, 2.0, -53.0], [-25.0, -44.0, -27.0, -10.0, -40.0]], [[16.0, 4.0, 24.0, -8.0, 10.0], [10.0, 28.0, 30.0, -12.0, 30.0], [-2.0, 6.0, 32.0, -18.0, 8.0], [-16.0, 0.0, -10.0, -6.0, -2.0]]], [[[8.0, -11.0, -17.0, -13.0, -47.0], [4.0, -21.0, -4.0, 37.0, -9.0], [7.0, -43.0, -27.0, 26.0, -37.0], [9.0, -14.0, -23.0, -16.0, -8.0]], [[-10.0, 56.0, -7.0, 3.0, 49.0], [17.0, 0.0, 33.0, -66.0, 3.0], [-6.0, 42.0, 86.0, 10.0, 22.0], [21.0, 33.0, -14.0, -19.0, 34.0]], [[0.0, -22.0, 64.0, 24.0, 77.0], [-19.0, 70.0, -1.0, -38.0, 39.0], [-13.0, 71.0, 8.0, -85.0, 76.0], [-52.0, 4.0, 63.0, 53.0, -13.0]], [[26.0, -14.0, 63.0, -3.0, 36.0], [4.0, 70.0, 38.0, -20.0, 60.0], [-7.0, 29.0, 34.0, -63.0, 38.0], [-49.0, -5.0, 13.0, 16.0, -15.0]], [[-50.0, 34.0, -41.0, 33.0, 53.0], [-17.0, -42.0, -45.0, -48.0, -57.0], [-6.0, 54.0, 4.0, 20.0, 32.0], [39.0, 33.0, 38.0, 19.0, 32.0]]]])+(1.-msk_ref)*numpy.array([[[[13.0, 23.0, 39.0, 43.0, -27.0], [-6.0, -11.0, -21.0, 16.0, 21.0], [7.0, 37.0, -37.0, -13.0, 46.0], [6.0, 29.0, -5.0, 45.0, 0.0]], [[20.0, 107.0, 25.0, 36.0, 8.0], [-6.0, -46.0, 77.0, 30.0, 49.0], [-84.0, 114.0, -2.0, -69.0, -10.0], [-85.0, 22.0, 16.0, 115.0, 35.0]], [[0.0, -40.0, 35.0, 40.0, -45.0], [-5.0, 15.0, -80.0, 5.0, -5.0], [65.0, -25.0, -50.0, 30.0, 70.0], [65.0, 25.0, -20.0, -15.0, -25.0]], [[48.0, 29.0, 23.0, -40.0, 28.0], [6.0, -6.0, 19.0, -22.0, 23.0], [-16.0, 26.0, 6.0, -35.0, -6.0], [-23.0, 6.0, 28.0, 29.0, 25.0]], [[-4.0, 66.0, 11.0, 44.0, -7.0], [-7.0, -31.0, 46.0, 31.0, 27.0], [-53.0, 73.0, -6.0, -36.0, -2.0], [-51.0, 15.0, 0.0, 73.0, 15.0]]], [[[-62.0, -14.0, -1.0, 102.0, -65.0], [-15.0, -5.0, -38.0, 51.0, -13.0], [21.0, 3.0, -40.0, 36.0, 46.0], [31.0, 17.0, -46.0, 5.0, -35.0]], [[-18.0, 23.0, -19.0, 10.0, 6.0], [-2.0, -12.0, 37.0, 14.0, 1.0], [-34.0, 20.0, 16.0, -9.0, -26.0], [-31.0, -8.0, -2.0, 15.0, 5.0]], [[-27.0, -8.0, -17.0, 19.0, -10.0], [-3.0, 0.0, 2.0, 13.0, -11.0], [-2.0, -8.0, 3.0, 14.0, -6.0], [2.0, -6.0, -13.0, -11.0, -10.0]], [[54.0, -29.0, 22.0, -70.0, 27.0], [11.0, 21.0, -31.0, -47.0, 2.0], [37.0, -35.0, 2.0, -3.0, 8.0], [28.0, -1.0, 26.0, -30.0, 10.0]], [[-47.0, -33.0, -11.0, 55.0, -41.0], [-8.0, 7.0, -37.0, 26.0, -21.0], [29.0, -25.0, -21.0, 39.0, 26.0], [36.0, 3.0, -33.0, -25.0, -30.0]]], [[[18.0, -36.0, -21.0, -74.0, 37.0], [11.0, 21.0, 0.0, -39.0, -17.0], [15.0, -51.0, 32.0, 10.0, -34.0], [11.0, -25.0, 18.0, -55.0, 5.0]], [[-46.0, 32.0, -8.0, 78.0, -34.0], [-12.0, -22.0, 20.0, 48.0, 4.0], [-30.0, 42.0, -14.0, 0.0, 8.0], [-22.0, 10.0, -26.0, 40.0, -10.0]], [[45.0, -23.0, 43.0, -21.0, -5.0], [4.0, 15.0, -55.0, -26.0, 9.0], [49.0, -17.0, -29.0, 1.0, 46.0], [42.0, 19.0, 11.0, -7.0, 0.0]], [[45.0, -15.0, 36.0, -29.0, 4.0], [5.0, 12.0, -39.0, -27.0, 10.0], [36.0, -12.0, -19.0, -5.0, 32.0], [29.0, 14.0, 15.0, -4.0, 5.0]], [[31.0, 8.0, 51.0, 25.0, -24.0], [-3.0, -2.0, -42.0, 1.0, 21.0], [28.0, 22.0, -43.0, -10.0, 58.0], [24.0, 32.0, 1.0, 33.0, 0.0]]], [[[-2.0, 29.0, 9.0, 26.0, -8.0], [-4.0, -14.0, 15.0, 16.0, 13.0], [-20.0, 34.0, -8.0, -15.0, 6.0], [-19.0, 10.0, -2.0, 35.0, 5.0]], [[43.0, -2.0, 59.0, 13.0, -22.0], [-1.0, 4.0, -56.0, -9.0, 21.0], [42.0, 12.0, -47.0, -8.0, 66.0], [36.0, 34.0, 5.0, 25.0, 0.0]], [[-29.0, 8.0, -48.0, -19.0, 25.0], [2.0, -5.0, 54.0, 4.0, -14.0], [-41.0, -5.0, 43.0, 0.0, -60.0], [-37.0, -29.0, 1.0, -16.0, 5.0]], [[33.0, 11.0, 14.0, -33.0, 20.0], [5.0, 0.0, 7.0, -19.0, 12.0], [-4.0, 8.0, 5.0, -19.0, -4.0], [-9.0, 2.0, 19.0, 10.0, 15.0]], [[-77.0, 0.0, -38.0, 77.0, -37.0], [-12.0, -11.0, 14.0, 50.0, -20.0], [-19.0, 5.0, -1.0, 28.0, -8.0], [-7.0, -7.0, -39.0, -2.0, -25.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -3.0, -7.0], [7.0, 1.0, 0.0]], [[3.0, -4.0, 7.0], [7.0, 1.0, 3.0]], [[6.0, 0.0, 2.0], [0.0, -1.0, 3.0]], [[-3.0, 5.0, 1.0], [-6.0, -7.0, 0.0]], [[-7.0, 2.0, -6.0], [0.0, 7.0, -1.0]]], [[[4.0, 5.0, -4.0], [5.0, -7.0, -6.0]], [[6.0, -1.0, -7.0], [-6.0, 4.0, -7.0]], [[-7.0, -6.0, -5.0], [6.0, -4.0, 1.0]], [[3.0, -6.0, -6.0], [0.0, -1.0, -4.0]], [[-2.0, 0.0, 3.0], [0.0, 4.0, -6.0]]], [[[3.0, -6.0, -2.0], [5.0, 0.0, -5.0]], [[-1.0, 4.0, 7.0], [5.0, -7.0, -5.0]], [[-1.0, 4.0, -2.0], [-3.0, 2.0, 7.0]], [[7.0, 6.0, -2.0], [-6.0, 0.0, 0.0]], [[-3.0, 5.0, 3.0], [3.0, 0.0, -5.0]]], [[[-4.0, -7.0, 7.0], [5.0, -3.0, -2.0]], [[0.0, 3.0, 0.0], [-7.0, 4.0, -5.0]], [[1.0, -6.0, 5.0], [6.0, 1.0, 1.0]], [[7.0, 2.0, 4.0], [4.0, -5.0, 2.0]], [[3.0, -4.0, 5.0], [4.0, 5.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -5.0, -5.0], [6.0, 0.0, 5.0]], [[6.0, 5.0, 4.0], [-6.0, 4.0, -5.0]], [[4.0, -7.0, -5.0], [2.0, 7.0, -1.0]], [[-2.0, -5.0, 0.0], [7.0, -7.0, 6.0]], [[7.0, -4.0, 2.0], [-2.0, 0.0, -4.0]]], [[[-1.0, -1.0, 5.0], [7.0, 5.0, -5.0]], [[2.0, 4.0, 0.0], [2.0, -4.0, 4.0]], [[-7.0, 0.0, -4.0], [5.0, 1.0, -1.0]], [[-5.0, 1.0, -7.0], [-7.0, -3.0, 4.0]], [[2.0, -6.0, 6.0], [-6.0, 3.0, 0.0]]], [[[-1.0, -2.0, 5.0], [-1.0, 5.0, -5.0]], [[-4.0, 0.0, -3.0], [-7.0, 4.0, 0.0]], [[5.0, -1.0, 0.0], [6.0, 6.0, -1.0]], [[2.0, -6.0, -6.0], [6.0, 4.0, 4.0]], [[2.0, -2.0, -6.0], [4.0, 5.0, 4.0]]], [[[1.0, -2.0, -7.0], [2.0, 0.0, -1.0]], [[0.0, 4.0, 5.0], [7.0, -2.0, 7.0]], [[5.0, 7.0, 1.0], [3.0, -7.0, -5.0]], [[-7.0, -2.0, 2.0], [0.0, -3.0, -5.0]], [[-2.0, 7.0, 6.0], [-7.0, -6.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, 1.0, -6.0, -2.0, -1.0], [0.0, 0.0, 3.0, 0.0, -5.0], [-5.0, -7.0, -2.0, -6.0, 6.0], [2.0, -4.0, -2.0, -2.0, 0.0]], [[3.0, -6.0, 2.0, -4.0, 2.0], [5.0, 2.0, 4.0, -3.0, 1.0], [-6.0, 2.0, 2.0, 7.0, 0.0], [3.0, 7.0, -1.0, -2.0, 4.0]], [[7.0, 4.0, 0.0, -3.0, 0.0], [1.0, -4.0, 4.0, -1.0, -4.0], [0.0, 0.0, 5.0, -1.0, -4.0], [-1.0, 6.0, 4.0, -3.0, -6.0]]], [[[0.0, -4.0, 1.0, 6.0, 0.0], [0.0, 4.0, -7.0, 5.0, 6.0], [-7.0, 0.0, -6.0, -3.0, 0.0], [7.0, -6.0, 0.0, -3.0, -7.0]], [[0.0, 0.0, 3.0, 3.0, -4.0], [4.0, -5.0, -5.0, 6.0, 2.0], [0.0, -2.0, 3.0, -7.0, 2.0], [0.0, -3.0, 0.0, 1.0, 5.0]], [[-4.0, 2.0, 0.0, 2.0, -7.0], [7.0, -5.0, -4.0, -7.0, 6.0], [-7.0, 0.0, 1.0, -5.0, 0.0], [1.0, 6.0, 7.0, 5.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, 7.0, -2.0, -4.0, 0.0], [-4.0, 6.0, 3.0, 0.0, 7.0], [6.0, 2.0, 1.0, 1.0, 5.0], [-1.0, -5.0, -5.0, -7.0, -4.0]], [[2.0, 6.0, -3.0, 0.0, 6.0], [7.0, -6.0, -1.0, 0.0, 0.0], [5.0, 4.0, 6.0, 4.0, -4.0], [5.0, 1.0, -6.0, 1.0, -4.0]], [[2.0, -7.0, 0.0, 2.0, 3.0], [0.0, -1.0, 4.0, -3.0, -1.0], [4.0, 2.0, -6.0, 0.0, 7.0], [-2.0, -5.0, -3.0, 0.0, 0.0]]], [[[-6.0, 3.0, 0.0, -3.0, -3.0], [0.0, 2.0, 3.0, -5.0, -2.0], [7.0, 6.0, 2.0, 1.0, -5.0], [1.0, -2.0, -7.0, 3.0, 6.0]], [[5.0, 2.0, -1.0, 0.0, 0.0], [-3.0, 7.0, -3.0, 3.0, 0.0], [-7.0, 7.0, -2.0, 2.0, 7.0], [6.0, 3.0, -2.0, -2.0, -6.0]], [[7.0, -1.0, 6.0, -6.0, -5.0], [0.0, 4.0, 0.0, 2.0, 1.0], [-2.0, -7.0, 6.0, 0.0, -2.0], [-7.0, 4.0, -4.0, 5.0, -6.0]]]])
      res=generalTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-94.0, -32.0, -32.0, 66.0, -16.0], [-18.0, 45.0, -76.0, 57.0, 39.0], [-61.0, -50.0, -92.0, -78.0, 66.0], [59.0, -132.0, -37.0, -5.0, -14.0]], [[7.0, 33.0, -16.0, 40.0, -36.0], [12.0, -28.0, -45.0, 25.0, 15.0], [-61.0, -31.0, -15.0, -96.0, -8.0], [39.0, -25.0, 47.0, -24.0, -93.0]], [[-34.0, 20.0, -39.0, -15.0, -23.0], [19.0, -18.0, 19.0, -29.0, -22.0], [-51.0, -40.0, -2.0, -46.0, 26.0], [13.0, 9.0, 17.0, -4.0, -8.0]], [[40.0, -5.0, 1.0, -74.0, 41.0], [-2.0, 17.0, 92.0, -88.0, -34.0], [27.0, 45.0, 36.0, 119.0, -36.0], [-34.0, 110.0, 5.0, 4.0, 21.0]], [[10.0, -45.0, 67.0, 43.0, -10.0], [25.0, -2.0, -68.0, 49.0, 69.0], [30.0, 39.0, 8.0, 18.0, -4.0], [-3.0, -21.0, -19.0, 30.0, 76.0]]], [[[-13.0, -74.0, -30.0, -19.0, 76.0], [-49.0, 111.0, 40.0, 14.0, -19.0], [-43.0, -4.0, -75.0, 79.0, 26.0], [56.0, -50.0, -71.0, -58.0, -44.0]], [[-60.0, -6.0, -32.0, -25.0, 25.0], [-45.0, 17.0, 36.0, 53.0, -73.0], [67.0, -52.0, -8.0, -11.0, 72.0], [-33.0, -91.0, -88.0, -2.0, 79.0]], [[-15.0, -13.0, 24.0, 79.0, 4.0], [-44.0, 47.0, -91.0, 22.0, 83.0], [22.0, 45.0, -70.0, 10.0, -30.0], [16.0, -62.0, 7.0, 24.0, -53.0]], [[-62.0, 7.0, -33.0, 25.0, 17.0], [-68.0, 37.0, -18.0, 46.0, -23.0], [49.0, -31.0, -55.0, -27.0, 40.0], [-10.0, -111.0, -52.0, 3.0, -5.0]], [[57.0, -2.0, 24.0, -5.0, 28.0], [-23.0, -2.0, 10.0, 63.0, -30.0], [52.0, 6.0, 25.0, 11.0, -16.0], [-13.0, -22.0, -26.0, -31.0, -16.0]]], [[[-30.0, 1.0, -25.0, 44.0, 20.0], [-67.0, 41.0, -38.0, 80.0, -13.0], [21.0, -33.0, -63.0, -48.0, 26.0], [20.0, -126.0, -43.0, -28.0, -62.0]], [[87.0, -27.0, -2.0, -36.0, 72.0], [-36.0, 60.0, 61.0, -1.0, -33.0], [-19.0, 29.0, -11.0, 86.0, -48.0], [33.0, 35.0, -9.0, -74.0, -111.0]], [[-24.0, -7.0, 17.0, -6.0, -48.0], [75.0, -41.0, -12.0, -62.0, 45.0], [-47.0, 11.0, 31.0, -4.0, 6.0], [-2.0, 74.0, 39.0, 46.0, 80.0]], [[-38.0, -13.0, -36.0, -68.0, 5.0], [28.0, -4.0, 79.0, -46.0, -57.0], [-29.0, -37.0, 24.0, 20.0, 50.0], [-8.0, 38.0, -28.0, -2.0, 78.0]], [[74.0, -43.0, 31.0, -15.0, 48.0], [-7.0, 35.0, 22.0, 32.0, -4.0], [-1.0, 31.0, 8.0, 66.0, -30.0], [22.0, 17.0, -22.0, -47.0, -34.0]]], [[[60.0, 42.0, 6.0, 32.0, 16.0], [-54.0, 3.0, -24.0, 35.0, -3.0], [41.0, 20.0, -12.0, -16.0, -58.0], [-3.0, -24.0, 29.0, -27.0, -126.0]], [[29.0, 0.0, 11.0, -52.0, 25.0], [-4.0, -17.0, 61.0, 15.0, -61.0], [66.0, -2.0, 55.0, 39.0, 8.0], [-45.0, 21.0, -38.0, -6.0, 66.0]], [[7.0, 35.0, -9.0, 48.0, -24.0], [-14.0, -18.0, -52.0, 42.0, 13.0], [-18.0, -21.0, -21.0, -83.0, -12.0], [22.0, -49.0, 31.0, -17.0, -88.0]], [[-16.0, -1.0, -49.0, -21.0, 3.0], [8.0, 19.0, 34.0, -34.0, -23.0], [-89.0, -35.0, -27.0, -19.0, 16.0], [46.0, 13.0, 14.0, -37.0, -63.0]], [[13.0, 27.0, -7.0, 30.0, -17.0], [-9.0, -27.0, -32.0, 71.0, -17.0], [-5.0, -39.0, 0.0, -88.0, 8.0], [15.0, -61.0, 4.0, -30.0, -55.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, 18.0, 45.0, -58.0, -88.0], [-35.0, 67.0, 3.0, -5.0, -2.0], [-13.0, -29.0, 42.0, -14.0, -55.0], [-44.0, 28.0, -17.0, 38.0, 26.0]], [[69.0, 39.0, -61.0, 32.0, 85.0], [-1.0, -2.0, -1.0, 20.0, 45.0], [17.0, 67.0, -38.0, 28.0, 106.0], [64.0, -41.0, -18.0, -88.0, -74.0]], [[12.0, 42.0, 0.0, -26.0, -58.0], [-86.0, 120.0, -16.0, 24.0, 28.0], [-64.0, 38.0, -24.0, -8.0, 54.0], [22.0, 11.0, 13.0, -48.0, -12.0]], [[-55.0, -43.0, 62.0, -49.0, -81.0], [-6.0, 7.0, 41.0, -44.0, -22.0], [49.0, -73.0, 32.0, -29.0, -86.0], [-100.0, -6.0, -19.0, 74.0, 76.0]], [[15.0, 9.0, -26.0, 6.0, 8.0], [-56.0, 44.0, 27.0, -4.0, 47.0], [24.0, 18.0, -57.0, -11.0, 83.0], [-5.0, -61.0, 13.0, -79.0, 0.0]]], [[[-49.0, -12.0, -30.0, 23.0, 13.0], [-18.0, 24.0, 24.0, -45.0, -31.0], [33.0, 116.0, -63.0, 12.0, 44.0], [58.0, -40.0, -43.0, -8.0, 50.0]], [[14.0, 32.0, 12.0, -38.0, -2.0], [32.0, -20.0, 20.0, -14.0, 14.0], [66.0, -24.0, 62.0, 12.0, -52.0], [-32.0, -6.0, -56.0, 24.0, -12.0]], [[-75.0, -3.0, 7.0, 11.0, -22.0], [25.0, -25.0, -25.0, -12.0, -56.0], [-28.0, 22.0, 19.0, 0.0, -79.0], [33.0, 44.0, 14.0, 57.0, 58.0]], [[18.0, -11.0, 34.0, 3.0, -14.0], [36.0, -48.0, -56.0, 55.0, -10.0], [-89.0, -111.0, 59.0, -14.0, -72.0], [-29.0, 82.0, 79.0, 41.0, -32.0]], [[61.0, -76.0, 11.0, 22.0, 0.0], [-59.0, 51.0, 9.0, 21.0, 20.0], [-57.0, -23.0, -88.0, -22.0, 127.0], [-32.0, -25.0, 44.0, -44.0, -38.0]]], [[[-3.0, -42.0, -27.0, 47.0, 31.0], [-25.0, 14.0, 1.0, -5.0, -15.0], [-28.0, 64.0, -85.0, 0.0, 88.0], [45.0, -25.0, 19.0, -33.0, 6.0]], [[36.0, -20.0, 4.0, 31.0, 12.0], [4.0, -7.0, -57.0, 56.0, -11.0], [-113.0, -28.0, -8.0, -3.0, 22.0], [27.0, 61.0, 70.0, -1.0, -50.0]], [[10.0, 60.0, -19.0, -32.0, -19.0], [-45.0, 86.0, 16.0, -14.0, 22.0], [27.0, 91.0, -7.0, 19.0, 43.0], [39.0, -24.0, -69.0, -35.0, -10.0]], [[-2.0, 42.0, 34.0, -62.0, -92.0], [-62.0, 110.0, -6.0, 8.0, 12.0], [-36.0, 4.0, 30.0, -8.0, -18.0], [-18.0, 30.0, -22.0, 10.0, 4.0]], [[23.0, 62.0, 21.0, -56.0, -62.0], [-37.0, 89.0, -19.0, 21.0, 16.0], [-37.0, 15.0, 48.0, 8.0, -17.0], [6.0, 41.0, -34.0, 6.0, -30.0]]], [[[-32.0, 51.0, -2.0, -18.0, -34.0], [-18.0, 25.0, -17.0, 9.0, 9.0], [-16.0, -1.0, 29.0, -5.0, -44.0], [12.0, 20.0, 18.0, -8.0, 22.0]], [[15.0, -1.0, 32.0, -53.0, -17.0], [34.0, -1.0, 43.0, -42.0, -12.0], [89.0, 5.0, 54.0, 19.0, -44.0], [-44.0, -13.0, -112.0, 64.0, -4.0]], [[-47.0, 70.0, -54.0, 3.0, 61.0], [50.0, -76.0, 42.0, -49.0, 23.0], [149.0, 44.0, 31.0, 22.0, -50.0], [24.0, -70.0, -57.0, -30.0, 42.0]], [[-85.0, -76.0, -7.0, 62.0, 19.0], [23.0, -73.0, -2.0, -25.0, -56.0], [-13.0, -4.0, -55.0, -21.0, -24.0], [10.0, -6.0, 67.0, 28.0, 84.0]], [[7.0, -44.0, -29.0, 59.0, 96.0], [75.0, -128.0, 8.0, -7.0, -9.0], [46.0, -27.0, -16.0, 7.0, 3.0], [3.0, -29.0, 23.0, -3.0, -8.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[7.0, 7.0], [-1.0, -5.0]], [[-6.0, -6.0], [4.0, -3.0]], [[-4.0, -6.0], [-4.0, 7.0]], [[-3.0, -7.0], [-6.0, 0.0]], [[-4.0, 6.0], [0.0, -2.0]], [[-4.0, 1.0], [-5.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, 5.0], [7.0, -1.0]], [[6.0, 7.0], [-2.0, 5.0]], [[3.0, 5.0], [0.0, -2.0]], [[5.0, 0.0], [-1.0, -6.0]], [[6.0, 4.0], [5.0, 0.0]], [[-3.0, 0.0], [7.0, 7.0]]])
      arg1=-5.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-35.0, -35.0], [5.0, 25.0]], [[30.0, 30.0], [-20.0, 15.0]], [[20.0, 30.0], [20.0, -35.0]], [[15.0, 35.0], [30.0, -0.0]], [[20.0, -30.0], [-0.0, 10.0]], [[20.0, -5.0], [25.0, -25.0]]])+(1.-msk_ref)*numpy.array([[[-5.0, -25.0], [-35.0, 5.0]], [[-30.0, -35.0], [10.0, -25.0]], [[-15.0, -25.0], [-0.0, 10.0]], [[-25.0, -0.0], [5.0, 30.0]], [[-30.0, -20.0], [-25.0, -0.0]], [[15.0, -0.0], [-35.0, -35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, -1.0], [-4.0, 5.0]], [[-3.0, 5.0], [6.0, 2.0]], [[0.0, -2.0], [2.0, 3.0]], [[6.0, 7.0], [2.0, 5.0]], [[0.0, 4.0], [1.0, 3.0]], [[-5.0, 0.0], [-6.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -6.0], [-6.0, -4.0]], [[-2.0, -2.0], [7.0, -5.0]], [[5.0, 3.0], [2.0, 1.0]], [[1.0, 0.0], [0.0, 2.0]], [[-5.0, -7.0], [4.0, 7.0]], [[3.0, -4.0], [5.0, -2.0]]])
      arg1=numpy.array(6.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, -6.0], [-24.0, 30.0]], [[-18.0, 30.0], [36.0, 12.0]], [[0.0, -12.0], [12.0, 18.0]], [[36.0, 42.0], [12.0, 30.0]], [[0.0, 24.0], [6.0, 18.0]], [[-30.0, 0.0], [-36.0, 6.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -36.0], [-36.0, -24.0]], [[-12.0, -12.0], [42.0, -30.0]], [[30.0, 18.0], [12.0, 6.0]], [[6.0, 0.0], [0.0, 12.0]], [[-30.0, -42.0], [24.0, 42.0]], [[18.0, -24.0], [30.0, -12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -7.0, 2.0], [-2.0, -4.0, 1.0]], [[3.0, -4.0, 3.0], [5.0, 5.0, -2.0]]], [[[7.0, -7.0, -6.0], [3.0, -2.0, 0.0]], [[-3.0, -3.0, -4.0], [6.0, 2.0, -2.0]]], [[[1.0, -6.0, -3.0], [6.0, -1.0, -7.0]], [[-7.0, 6.0, -5.0], [-7.0, 6.0, 2.0]]], [[[-4.0, 5.0, 7.0], [2.0, 1.0, -3.0]], [[2.0, 0.0, -4.0], [7.0, -4.0, -2.0]]], [[[4.0, -1.0, -4.0], [-6.0, -5.0, 4.0]], [[-6.0, 0.0, -7.0], [-5.0, 7.0, 0.0]]], [[[2.0, 0.0, 1.0], [7.0, -7.0, 6.0]], [[0.0, -3.0, 7.0], [3.0, 4.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, -5.0, -3.0], [-1.0, 0.0, 7.0]], [[1.0, 1.0, 3.0], [2.0, -1.0, -6.0]]], [[[-6.0, 6.0, 3.0], [-4.0, 7.0, 0.0]], [[-1.0, 7.0, -3.0], [4.0, 7.0, -6.0]]], [[[0.0, -2.0, -7.0], [-5.0, 1.0, 5.0]], [[4.0, -2.0, 0.0], [5.0, 6.0, -6.0]]], [[[4.0, 7.0, 7.0], [0.0, 0.0, 3.0]], [[-7.0, -2.0, -5.0], [6.0, -5.0, 7.0]]], [[[-1.0, 0.0, -2.0], [-2.0, 7.0, 2.0]], [[0.0, 5.0, -2.0], [-4.0, 5.0, 1.0]]], [[[3.0, -3.0, -4.0], [-3.0, 0.0, 1.0]], [[2.0, 3.0, -7.0], [0.0, -5.0, 3.0]]]])
      arg1=numpy.array([-1.0, 0.0, -7.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-14.0, -5.0], [-24.0, 9.0]], [[35.0, -3.0], [31.0, 8.0]], [[20.0, 43.0], [42.0, -7.0]], [[-45.0, 19.0], [26.0, 7.0]], [[24.0, -22.0], [55.0, 5.0]], [[-9.0, -49.0], [-49.0, -3.0]]])+(1.-msk_ref)*numpy.array([[[28.0, -48.0], [-22.0, 40.0]], [[-15.0, 4.0], [22.0, 38.0]], [[49.0, -30.0], [-4.0, 37.0]], [[-53.0, -21.0], [42.0, -55.0]], [[15.0, -12.0], [14.0, -3.0]], [[25.0, -4.0], [47.0, -21.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 0.0], [-5.0, 1.0]], [[6.0, 5.0], [6.0, -6.0]], [[1.0, -6.0], [7.0, -7.0]], [[0.0, -5.0], [-5.0, -1.0]], [[-6.0, 6.0], [0.0, -2.0]], [[3.0, -4.0], [0.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[6.0, -5.0], [-2.0, 4.0]], [[0.0, 6.0], [6.0, 6.0]], [[1.0, 1.0], [6.0, -2.0]], [[-3.0, -7.0], [6.0, -5.0]], [[2.0, 6.0], [-7.0, -3.0]], [[7.0, 2.0], [-6.0, 1.0]]])
      arg1=numpy.array([6.0, -6.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-6.0, 6.0], [0.0, 0.0]], [[-30.0, 30.0], [6.0, -6.0]]], [[[36.0, -36.0], [30.0, -30.0]], [[36.0, -36.0], [-36.0, 36.0]]], [[[6.0, -6.0], [-36.0, 36.0]], [[42.0, -42.0], [-42.0, 42.0]]], [[[0.0, 0.0], [-30.0, 30.0]], [[-30.0, 30.0], [-6.0, 6.0]]], [[[-36.0, 36.0], [36.0, -36.0]], [[0.0, 0.0], [-12.0, 12.0]]], [[[18.0, -18.0], [-24.0, 24.0]], [[0.0, 0.0], [12.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[36.0, -36.0], [-30.0, 30.0]], [[-12.0, 12.0], [24.0, -24.0]]], [[[0.0, 0.0], [36.0, -36.0]], [[36.0, -36.0], [36.0, -36.0]]], [[[6.0, -6.0], [6.0, -6.0]], [[36.0, -36.0], [-12.0, 12.0]]], [[[-18.0, 18.0], [-42.0, 42.0]], [[36.0, -36.0], [-30.0, 30.0]]], [[[12.0, -12.0], [36.0, -36.0]], [[-42.0, 42.0], [-18.0, 18.0]]], [[[42.0, -42.0], [12.0, -12.0]], [[-36.0, 36.0], [6.0, -6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -7.0, 4.0], [-6.0, -1.0, -5.0]], [[-1.0, 6.0, 1.0], [6.0, -3.0, 3.0]]], [[[-7.0, 7.0, -6.0], [-1.0, -2.0, 5.0]], [[7.0, -6.0, -4.0], [0.0, -6.0, 3.0]]], [[[-1.0, 3.0, 0.0], [-3.0, 3.0, 3.0]], [[0.0, 7.0, 5.0], [5.0, 5.0, 2.0]]], [[[5.0, -3.0, -1.0], [5.0, 4.0, -6.0]], [[-2.0, -5.0, 6.0], [6.0, 7.0, -2.0]]], [[[-6.0, 4.0, 1.0], [3.0, -3.0, 7.0]], [[2.0, -5.0, 0.0], [0.0, 0.0, -1.0]]], [[[0.0, 3.0, 1.0], [3.0, 5.0, 6.0]], [[-5.0, -2.0, -6.0], [4.0, 0.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 6.0, 7.0], [0.0, 0.0, 4.0]], [[-2.0, -5.0, 5.0], [1.0, -7.0, 4.0]]], [[[-5.0, 5.0, -1.0], [7.0, 0.0, 2.0]], [[2.0, 0.0, -2.0], [-3.0, 5.0, -1.0]]], [[[5.0, 3.0, -7.0], [-1.0, 0.0, 0.0]], [[-7.0, 3.0, -5.0], [7.0, -1.0, 0.0]]], [[[0.0, 4.0, 2.0], [2.0, 1.0, -2.0]], [[7.0, 2.0, -6.0], [-2.0, -4.0, 3.0]]], [[[3.0, -2.0, 0.0], [7.0, -4.0, -2.0]], [[-1.0, -1.0, -4.0], [-3.0, 1.0, 0.0]]], [[[-4.0, 4.0, -3.0], [-1.0, -3.0, -2.0]], [[-7.0, -2.0, 7.0], [-6.0, -5.0, 3.0]]]])
      arg1=numpy.array([[0.0, -1.0], [5.0, -3.0], [-7.0, -5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-63.0, -5.0], [30.0, 34.0]], [[23.0, -22.0], [-36.0, -12.0]]], [[[77.0, 16.0], [-45.0, -18.0]], [[-2.0, 31.0], [-51.0, 3.0]]], [[[15.0, -8.0], [-6.0, -21.0]], [[0.0, -46.0], [11.0, -30.0]]], [[[-8.0, 9.0], [62.0, 13.0]], [[-67.0, -13.0], [49.0, -17.0]]], [[[13.0, -11.0], [-64.0, -29.0]], [[-25.0, 13.0], [7.0, 5.0]]], [[[8.0, -14.0], [-17.0, -48.0]], [[32.0, 41.0], [42.0, 26.0]]]])+(1.-msk_ref)*numpy.array([[[[-19.0, -47.0], [-28.0, -20.0]], [[-60.0, -8.0], [-63.0, 0.0]]], [[[32.0, -5.0], [-14.0, -17.0]], [[14.0, 8.0], [32.0, -7.0]]], [[[64.0, 21.0], [0.0, 1.0]], [[50.0, 23.0], [-5.0, -4.0]]], [[[6.0, -22.0], [19.0, 5.0]], [[52.0, 17.0], [-41.0, -1.0]]], [[[-10.0, 3.0], [-6.0, 15.0]], [[23.0, 24.0], [5.0, 0.0]]], [[[41.0, 7.0], [-1.0, 20.0]], [[-59.0, -22.0], [-46.0, 6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 0.0], [-7.0, 4.0]], [[0.0, -3.0], [-7.0, 3.0]], [[-5.0, -5.0], [7.0, -7.0]], [[7.0, -3.0], [6.0, 0.0]], [[-6.0, -1.0], [0.0, -6.0]], [[5.0, -3.0], [3.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[5.0, 6.0], [-5.0, -5.0]], [[3.0, 6.0], [-7.0, 5.0]], [[-6.0, 0.0], [0.0, -3.0]], [[-4.0, -4.0], [-3.0, -3.0]], [[3.0, -5.0], [-3.0, -7.0]], [[-1.0, 2.0], [3.0, 0.0]]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-49.0, 0.0], [-49.0, 28.0]], [[0.0, -21.0], [-49.0, 21.0]], [[-35.0, -35.0], [49.0, -49.0]], [[49.0, -21.0], [42.0, 0.0]], [[-42.0, -7.0], [0.0, -42.0]], [[35.0, -21.0], [21.0, 7.0]]])+(1.-msk_ref)*numpy.array([[[35.0, 42.0], [-35.0, -35.0]], [[21.0, 42.0], [-49.0, 35.0]], [[-42.0, 0.0], [0.0, -21.0]], [[-28.0, -28.0], [-21.0, -21.0]], [[21.0, -35.0], [-21.0, -49.0]], [[-7.0, 14.0], [21.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, 7.0, -6.0], [-6.0, 3.0, 0.0]], [[2.0, 7.0, 3.0], [-3.0, 1.0, 0.0]]], [[[1.0, 1.0, 2.0], [7.0, 5.0, -3.0]], [[0.0, -2.0, 7.0], [5.0, 5.0, 2.0]]], [[[-2.0, 0.0, 4.0], [-2.0, 1.0, 2.0]], [[-4.0, -3.0, -6.0], [-1.0, 5.0, -2.0]]], [[[7.0, -2.0, 0.0], [5.0, -3.0, -7.0]], [[-1.0, 4.0, 2.0], [-6.0, 6.0, -5.0]]], [[[-2.0, -2.0, 1.0], [6.0, 6.0, 6.0]], [[2.0, 7.0, 2.0], [-3.0, 5.0, -4.0]]], [[[-4.0, -3.0, 6.0], [-1.0, 2.0, -1.0]], [[0.0, -1.0, -4.0], [0.0, -4.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 2.0, 7.0], [-3.0, 3.0, -5.0]], [[-1.0, 5.0, -7.0], [2.0, 0.0, -4.0]]], [[[5.0, 1.0, 0.0], [3.0, -7.0, 3.0]], [[-5.0, 7.0, -2.0], [3.0, -6.0, 2.0]]], [[[2.0, 7.0, -5.0], [-7.0, -7.0, -1.0]], [[-7.0, 5.0, -2.0], [-5.0, 7.0, 6.0]]], [[[4.0, -2.0, 3.0], [-5.0, 6.0, 3.0]], [[-2.0, 6.0, -1.0], [-1.0, 5.0, 4.0]]], [[[7.0, 5.0, -5.0], [-5.0, 5.0, 1.0]], [[7.0, 3.0, -3.0], [1.0, 1.0, 0.0]]], [[[3.0, 7.0, -1.0], [-3.0, 7.0, -6.0]], [[0.0, -4.0, -1.0], [-3.0, 0.0, -1.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-6.0, 0.0, 6.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 36.0], [6.0, 18.0]], [[6.0, -60.0], [42.0, -18.0]], [[36.0, 24.0], [-12.0, -6.0]], [[-42.0, -72.0], [18.0, 6.0]], [[18.0, 0.0], [0.0, -6.0]], [[60.0, 0.0], [-24.0, 42.0]]])+(1.-msk_ref)*numpy.array([[[84.0, -12.0], [-36.0, -36.0]], [[-30.0, 0.0], [18.0, -6.0]], [[-42.0, 36.0], [30.0, 66.0]], [[-6.0, 48.0], [6.0, 30.0]], [[-72.0, 36.0], [-60.0, -6.0]], [[-24.0, -18.0], [-6.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-4.0, 6.0], [7.0, 7.0]], [[-7.0, 3.0], [1.0, 7.0]], [[-5.0, -7.0], [0.0, 1.0]], [[7.0, -4.0], [-7.0, 1.0]], [[-4.0, 5.0], [7.0, -6.0]], [[1.0, -3.0], [5.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, -1.0], [-1.0, 7.0]], [[1.0, 6.0], [-1.0, 0.0]], [[7.0, -4.0], [-3.0, 0.0]], [[4.0, -1.0], [7.0, 0.0]], [[-3.0, 2.0], [-1.0, 0.0]], [[-3.0, -1.0], [-4.0, 4.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-7.0, -5.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[28.0, 20.0], [-42.0, -30.0]], [[-49.0, -35.0], [-49.0, -35.0]]], [[[49.0, 35.0], [-21.0, -15.0]], [[-7.0, -5.0], [-49.0, -35.0]]], [[[35.0, 25.0], [49.0, 35.0]], [[0.0, 0.0], [-7.0, -5.0]]], [[[-49.0, -35.0], [28.0, 20.0]], [[49.0, 35.0], [-7.0, -5.0]]], [[[28.0, 20.0], [-35.0, -25.0]], [[-49.0, -35.0], [42.0, 30.0]]], [[[-7.0, -5.0], [21.0, 15.0]], [[-35.0, -25.0], [42.0, 30.0]]]])+(1.-msk_ref)*numpy.array([[[[-49.0, -35.0], [7.0, 5.0]], [[7.0, 5.0], [-49.0, -35.0]]], [[[-7.0, -5.0], [-42.0, -30.0]], [[7.0, 5.0], [0.0, 0.0]]], [[[-49.0, -35.0], [28.0, 20.0]], [[21.0, 15.0], [0.0, 0.0]]], [[[-28.0, -20.0], [7.0, 5.0]], [[-49.0, -35.0], [0.0, 0.0]]], [[[21.0, 15.0], [-14.0, -10.0]], [[7.0, 5.0], [0.0, 0.0]]], [[[21.0, 15.0], [7.0, 5.0]], [[28.0, 20.0], [-28.0, -20.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, 0.0, 7.0], [-5.0, -5.0, -1.0]], [[2.0, -3.0, 4.0], [4.0, 7.0, 6.0]]], [[[5.0, 0.0, 2.0], [-1.0, 2.0, 3.0]], [[-4.0, -1.0, 5.0], [7.0, 4.0, -5.0]]], [[[-1.0, 4.0, -3.0], [5.0, 3.0, -4.0]], [[4.0, -5.0, 1.0], [7.0, 2.0, 0.0]]], [[[1.0, -1.0, 7.0], [-5.0, -7.0, 2.0]], [[-1.0, -6.0, 4.0], [0.0, -2.0, 1.0]]], [[[5.0, -7.0, -1.0], [4.0, 0.0, -6.0]], [[-3.0, -6.0, -2.0], [-2.0, -5.0, 7.0]]], [[[5.0, 0.0, 2.0], [0.0, 7.0, -7.0]], [[0.0, 0.0, 0.0], [0.0, 4.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -2.0, -4.0], [1.0, -2.0, -1.0]], [[-2.0, 4.0, 7.0], [3.0, 7.0, 4.0]]], [[[6.0, 6.0, 6.0], [-6.0, -4.0, -4.0]], [[6.0, -6.0, 3.0], [7.0, -1.0, -2.0]]], [[[-6.0, 3.0, -4.0], [7.0, 0.0, 4.0]], [[0.0, 0.0, 1.0], [-7.0, -7.0, 5.0]]], [[[1.0, -1.0, 0.0], [-1.0, 4.0, 5.0]], [[4.0, -6.0, -3.0], [5.0, 3.0, 4.0]]], [[[0.0, -3.0, -6.0], [3.0, 3.0, 7.0]], [[-3.0, 4.0, 0.0], [5.0, -6.0, 0.0]]], [[[5.0, -6.0, -1.0], [-7.0, -5.0, 2.0]], [[1.0, 7.0, 5.0], [3.0, -4.0, 4.0]]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[1.0, 0.0], [6.0, 3.0], [-7.0, 5.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-54.0, 35.0], [-28.0, -20.0]], [[-44.0, 11.0], [4.0, 51.0]]], [[[-9.0, 10.0], [-10.0, 21.0]], [[-45.0, 22.0], [66.0, -13.0]]], [[[44.0, -3.0], [51.0, -11.0]], [[-33.0, -10.0], [19.0, 6.0]]], [[[-54.0, 32.0], [-61.0, -11.0]], [[-65.0, 2.0], [-19.0, -1.0]]], [[[-30.0, -26.0], [46.0, -30.0]], [[-25.0, -28.0], [-81.0, 20.0]]], [[[-9.0, 10.0], [91.0, -14.0]], [[0.0, 0.0], [-4.0, 32.0]]]])+(1.-msk_ref)*numpy.array([[[[18.0, -26.0], [-4.0, -11.0]], [[-27.0, 47.0], [17.0, 41.0]]], [[[0.0, 48.0], [-2.0, -32.0]], [[-51.0, -3.0], [15.0, -13.0]]], [[[40.0, -11.0], [-21.0, 20.0]], [[-7.0, 5.0], [-84.0, 4.0]]], [[[-5.0, -3.0], [-12.0, 37.0]], [[-11.0, -33.0], [-5.0, 29.0]]], [[[24.0, -39.0], [-28.0, 44.0]], [[21.0, 12.0], [-31.0, -18.0]]], [[[-24.0, -23.0], [-51.0, -5.0]], [[8.0, 46.0], [-49.0, 8.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, -4.0], [-2.0, 2.0]], [[-1.0, -2.0], [-2.0, 2.0]], [[-5.0, 4.0], [5.0, 7.0]], [[0.0, 0.0], [7.0, 4.0]], [[3.0, 3.0], [6.0, 0.0]], [[5.0, 3.0], [0.0, -7.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 0.0], [7.0, -6.0]], [[7.0, 0.0], [4.0, 2.0]], [[-6.0, -7.0], [-2.0, 7.0]], [[6.0, -7.0], [2.0, 7.0]], [[1.0, 6.0], [-2.0, 4.0]], [[0.0, 2.0], [-3.0, -5.0]]])
      arg1=Data(-1.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-1.0, 4.0], [2.0, -2.0]], [[1.0, 2.0], [2.0, -2.0]], [[5.0, -4.0], [-5.0, -7.0]], [[-0.0, -0.0], [-7.0, -4.0]], [[-3.0, -3.0], [-6.0, -0.0]], [[-5.0, -3.0], [-0.0, 7.0]]])+(1.-msk_ref)*numpy.array([[[-2.0, -0.0], [-7.0, 6.0]], [[-7.0, -0.0], [-4.0, -2.0]], [[6.0, 7.0], [2.0, -7.0]], [[-6.0, 7.0], [-2.0, -7.0]], [[-1.0, -6.0], [2.0, -4.0]], [[-0.0, -2.0], [3.0, 5.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 0.0, 5.0], [3.0, -3.0, 4.0]], [[7.0, 5.0, -1.0], [-6.0, -2.0, 7.0]]], [[[7.0, 5.0, 4.0], [-6.0, 3.0, -4.0]], [[4.0, 0.0, 2.0], [1.0, -3.0, 0.0]]], [[[-6.0, 5.0, 5.0], [6.0, -3.0, 5.0]], [[5.0, -4.0, -2.0], [4.0, 5.0, 2.0]]], [[[-6.0, 5.0, -7.0], [0.0, 2.0, -4.0]], [[1.0, -5.0, -1.0], [5.0, 7.0, -4.0]]], [[[6.0, -6.0, 6.0], [4.0, 4.0, 6.0]], [[-5.0, 7.0, 7.0], [0.0, -6.0, 2.0]]], [[[0.0, 7.0, 2.0], [0.0, -7.0, 0.0]], [[-1.0, 2.0, -4.0], [-6.0, 3.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -5.0, -2.0], [1.0, 7.0, -6.0]], [[5.0, 6.0, 7.0], [0.0, -6.0, -4.0]]], [[[3.0, -6.0, -7.0], [0.0, 6.0, 0.0]], [[-5.0, 6.0, -1.0], [-4.0, 0.0, -2.0]]], [[[2.0, 3.0, -5.0], [3.0, 3.0, -4.0]], [[0.0, 7.0, -5.0], [-5.0, -7.0, 0.0]]], [[[4.0, 7.0, 4.0], [-2.0, 2.0, -3.0]], [[-2.0, -2.0, 0.0], [-2.0, 5.0, -2.0]]], [[[1.0, 3.0, 0.0], [7.0, 4.0, 2.0]], [[6.0, 1.0, -2.0], [5.0, 3.0, -5.0]]], [[[2.0, 0.0, 0.0], [3.0, -1.0, -4.0]], [[-4.0, -2.0, 0.0], [0.0, -5.0, 2.0]]]])
      arg1=Data(numpy.array([-4.0, 2.0, -6.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-58.0, -42.0], [-12.0, -22.0]], [[-42.0, 54.0], [-28.0, -10.0]], [[4.0, -60.0], [-16.0, -18.0]], [[76.0, 28.0], [-8.0, 18.0]], [[-72.0, -44.0], [-8.0, -24.0]], [[2.0, -14.0], [32.0, 6.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, 46.0], [-50.0, 12.0]], [[18.0, 12.0], [38.0, 28.0]], [[28.0, 18.0], [44.0, 6.0]], [[-26.0, 30.0], [4.0, 30.0]], [[2.0, -32.0], [-10.0, 16.0]], [[-8.0, 10.0], [12.0, -22.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -5.0], [-7.0, -5.0]], [[2.0, -6.0], [-3.0, 5.0]], [[-6.0, -2.0], [1.0, -3.0]], [[-7.0, 3.0], [0.0, -4.0]], [[-4.0, 3.0], [-3.0, 3.0]], [[2.0, -6.0], [-3.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -4.0], [-2.0, 7.0]], [[5.0, 0.0], [3.0, -6.0]], [[-1.0, -5.0], [4.0, -1.0]], [[-2.0, 6.0], [7.0, 5.0]], [[-5.0, -1.0], [4.0, -7.0]], [[6.0, -3.0], [0.0, -3.0]]])
      arg1=Data(numpy.array([2.0, -4.0]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [-10.0, 20.0]], [[-14.0, 28.0], [-10.0, 20.0]]], [[[4.0, -8.0], [-12.0, 24.0]], [[-6.0, 12.0], [10.0, -20.0]]], [[[-12.0, 24.0], [-4.0, 8.0]], [[2.0, -4.0], [-6.0, 12.0]]], [[[-14.0, 28.0], [6.0, -12.0]], [[0.0, 0.0], [-8.0, 16.0]]], [[[-8.0, 16.0], [6.0, -12.0]], [[-6.0, 12.0], [6.0, -12.0]]], [[[4.0, -8.0], [-12.0, 24.0]], [[-6.0, 12.0], [0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [-8.0, 16.0]], [[-4.0, 8.0], [14.0, -28.0]]], [[[10.0, -20.0], [0.0, 0.0]], [[6.0, -12.0], [-12.0, 24.0]]], [[[-2.0, 4.0], [-10.0, 20.0]], [[8.0, -16.0], [-2.0, 4.0]]], [[[-4.0, 8.0], [12.0, -24.0]], [[14.0, -28.0], [10.0, -20.0]]], [[[-10.0, 20.0], [-2.0, 4.0]], [[8.0, -16.0], [-14.0, 28.0]]], [[[12.0, -24.0], [-6.0, 12.0]], [[0.0, 0.0], [-6.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, -6.0, 0.0], [4.0, -4.0, 7.0]], [[-7.0, -3.0, 5.0], [1.0, -2.0, 1.0]]], [[[2.0, 0.0, -3.0], [0.0, 0.0, 7.0]], [[5.0, 7.0, -4.0], [2.0, -1.0, 0.0]]], [[[1.0, 2.0, 6.0], [-2.0, 5.0, 7.0]], [[-3.0, -2.0, 3.0], [-7.0, 4.0, -6.0]]], [[[-2.0, 7.0, 5.0], [4.0, -2.0, -6.0]], [[5.0, -4.0, -4.0], [5.0, -6.0, 2.0]]], [[[-4.0, 0.0, 7.0], [-3.0, 0.0, -3.0]], [[-4.0, -7.0, -7.0], [-1.0, 0.0, -6.0]]], [[[5.0, 1.0, -6.0], [0.0, 4.0, 1.0]], [[1.0, 4.0, -3.0], [3.0, 0.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[3.0, -2.0, 7.0], [0.0, 2.0, -4.0]], [[-4.0, 0.0, 2.0], [0.0, 4.0, 5.0]]], [[[-1.0, 6.0, -3.0], [0.0, -1.0, -1.0]], [[-3.0, 5.0, 6.0], [-1.0, 7.0, 3.0]]], [[[-6.0, 1.0, -3.0], [0.0, -4.0, -7.0]], [[1.0, -4.0, -7.0], [4.0, 7.0, -4.0]]], [[[1.0, 0.0, -7.0], [3.0, 0.0, 6.0]], [[-2.0, -1.0, 0.0], [-4.0, 0.0, 2.0]]], [[[7.0, 4.0, -5.0], [0.0, -5.0, 2.0]], [[-3.0, 1.0, -5.0], [-6.0, 7.0, 2.0]]], [[[7.0, 3.0, 7.0], [-3.0, 0.0, 1.0]], [[0.0, 2.0, 0.0], [-3.0, -6.0, 4.0]]]])
      arg1=Data(numpy.array([[4.0, -4.0], [-6.0, 1.0], [5.0, -7.0]]),self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 22.0], [75.0, -69.0]], [[15.0, -10.0], [21.0, -13.0]]], [[[-7.0, 13.0], [35.0, -49.0]], [[-42.0, 15.0], [14.0, -9.0]]], [[[22.0, -44.0], [-3.0, -36.0]], [[15.0, -11.0], [-82.0, 74.0]]], [[[-25.0, -20.0], [-2.0, 24.0]], [[24.0, 4.0], [66.0, -40.0]]], [[[19.0, -33.0], [-27.0, 33.0]], [[-9.0, 58.0], [-34.0, 46.0]]], [[[-16.0, 23.0], [-19.0, -3.0]], [[-35.0, 21.0], [-3.0, 9.0]]]])+(1.-msk_ref)*numpy.array([[[[59.0, -63.0], [-32.0, 30.0]], [[-6.0, 2.0], [1.0, -31.0]]], [[[-55.0, 31.0], [1.0, 6.0]], [[-12.0, -25.0], [-31.0, -10.0]]], [[[-45.0, 46.0], [-11.0, 45.0]], [[-7.0, 41.0], [-46.0, 19.0]]], [[[-31.0, 45.0], [42.0, -54.0]], [[-2.0, 7.0], [-6.0, 2.0]]], [[[-21.0, 11.0], [40.0, -19.0]], [[-43.0, 48.0], [-56.0, 17.0]]], [[[45.0, -74.0], [-7.0, 5.0]], [[-12.0, 2.0], [44.0, -22.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[7.0, 6.0], [5.0, 1.0]], [[-6.0, -1.0], [-6.0, -2.0]], [[2.0, -4.0], [0.0, 4.0]], [[6.0, 3.0], [7.0, 6.0]], [[-5.0, 4.0], [-2.0, -6.0]], [[-5.0, -5.0], [6.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 0.0], [1.0, 3.0]], [[4.0, -7.0], [-4.0, 7.0]], [[-2.0, 0.0], [-5.0, -5.0]], [[2.0, -2.0], [-3.0, 0.0]], [[4.0, -5.0], [6.0, 5.0]], [[1.0, 1.0], [3.0, -1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(2.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 6.0], [5.0, 1.0]], [[-6.0, -1.0], [-6.0, -2.0]], [[2.0, -4.0], [0.0, 4.0]], [[6.0, 3.0], [7.0, 6.0]], [[-5.0, 4.0], [-2.0, -6.0]], [[-5.0, -5.0], [6.0, -5.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0], [2.0, 6.0]], [[8.0, -14.0], [-8.0, 14.0]], [[-4.0, 0.0], [-10.0, -10.0]], [[4.0, -4.0], [-6.0, 0.0]], [[8.0, -10.0], [12.0, 10.0]], [[2.0, 2.0], [6.0, -2.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, -5.0, 3.0], [1.0, 0.0, -3.0]], [[6.0, -6.0, 0.0], [1.0, -4.0, 3.0]]], [[[-5.0, -6.0, -4.0], [-4.0, -3.0, -5.0]], [[-4.0, -5.0, -1.0], [-7.0, -6.0, -3.0]]], [[[-3.0, -7.0, -6.0], [6.0, -3.0, -5.0]], [[-3.0, -1.0, 1.0], [5.0, -1.0, -5.0]]], [[[0.0, 6.0, 4.0], [6.0, -3.0, 0.0]], [[-1.0, -3.0, -2.0], [6.0, 4.0, -6.0]]], [[[4.0, 7.0, -5.0], [-4.0, 4.0, 4.0]], [[5.0, -5.0, -2.0], [-2.0, 2.0, 7.0]]], [[[7.0, -4.0, 1.0], [0.0, 5.0, 5.0]], [[-2.0, -3.0, -3.0], [7.0, -2.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 6.0, 2.0], [-6.0, -1.0, -6.0]], [[1.0, 2.0, -2.0], [0.0, 0.0, -1.0]]], [[[-4.0, 0.0, -7.0], [2.0, 1.0, -2.0]], [[-6.0, 1.0, 6.0], [7.0, -5.0, 4.0]]], [[[5.0, 7.0, -1.0], [-1.0, 5.0, -4.0]], [[4.0, -1.0, 0.0], [2.0, -2.0, 5.0]]], [[[-7.0, -7.0, -2.0], [1.0, -4.0, -3.0]], [[3.0, -4.0, 3.0], [-3.0, -2.0, 1.0]]], [[[-1.0, 7.0, 2.0], [0.0, -6.0, 7.0]], [[-3.0, -7.0, 0.0], [3.0, 7.0, 5.0]]], [[[0.0, 1.0, 3.0], [5.0, 4.0, 2.0]], [[-4.0, -6.0, 4.0], [-4.0, 1.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, 4.0, -5.0])+(1.-msk_arg1)*numpy.array([7.0, -4.0, 2.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-40.0, 20.0], [6.0, -26.0]], [[-29.0, -7.0], [-35.0, -44.0]], [[-13.0, 43.0], [-24.0, 46.0]], [[4.0, 18.0], [-7.0, 76.0]], [[73.0, -24.0], [15.0, -37.0]], [[14.0, -5.0], [-7.0, 42.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, -50.0], [-5.0, -2.0]], [[-42.0, 6.0], [-34.0, 77.0]], [[5.0, -35.0], [32.0, 32.0]], [[-25.0, 17.0], [43.0, -11.0]], [[-31.0, 38.0], [7.0, 3.0]], [[2.0, 23.0], [4.0, -36.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank3_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 0.0], [3.0, 6.0]], [[1.0, 0.0], [-5.0, 2.0]], [[-2.0, -7.0], [2.0, -1.0]], [[7.0, -3.0], [4.0, -4.0]], [[-5.0, -5.0], [-1.0, 1.0]], [[-2.0, -3.0], [-5.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 3.0], [-4.0, 0.0]], [[-4.0, 5.0], [-3.0, -2.0]], [[7.0, 7.0], [-6.0, 1.0]], [[4.0, 0.0], [3.0, -3.0]], [[-3.0, 5.0], [0.0, 7.0]], [[-6.0, -3.0], [6.0, 5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, -6.0])+(1.-msk_arg1)*numpy.array([-5.0, 0.0])
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, -6.0], [0.0, 0.0]], [[-9.0, -18.0], [-18.0, -36.0]]], [[[-3.0, -6.0], [0.0, 0.0]], [[15.0, 30.0], [-6.0, -12.0]]], [[[6.0, 12.0], [21.0, 42.0]], [[-6.0, -12.0], [3.0, 6.0]]], [[[-21.0, -42.0], [9.0, 18.0]], [[-12.0, -24.0], [12.0, 24.0]]], [[[15.0, 30.0], [15.0, 30.0]], [[3.0, 6.0], [-3.0, -6.0]]], [[[6.0, 12.0], [9.0, 18.0]], [[15.0, 30.0], [0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 0.0], [-15.0, 0.0]], [[20.0, 0.0], [0.0, 0.0]]], [[[20.0, 0.0], [-25.0, 0.0]], [[15.0, 0.0], [10.0, 0.0]]], [[[-35.0, 0.0], [-35.0, 0.0]], [[30.0, 0.0], [-5.0, 0.0]]], [[[-20.0, 0.0], [0.0, 0.0]], [[-15.0, 0.0], [15.0, 0.0]]], [[[15.0, 0.0], [-25.0, 0.0]], [[0.0, 0.0], [-35.0, 0.0]]], [[[30.0, 0.0], [15.0, 0.0]], [[-30.0, 0.0], [-25.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, -2.0, 6.0], [-6.0, -3.0, -5.0]], [[-1.0, 4.0, 0.0], [-4.0, -4.0, 3.0]]], [[[-7.0, 5.0, 3.0], [-1.0, 7.0, 0.0]], [[0.0, -3.0, -2.0], [-7.0, 1.0, 2.0]]], [[[-4.0, 2.0, -1.0], [1.0, 1.0, -4.0]], [[-5.0, -4.0, 3.0], [0.0, -4.0, -2.0]]], [[[0.0, 0.0, -3.0], [1.0, -1.0, 4.0]], [[7.0, 0.0, -5.0], [5.0, 2.0, -4.0]]], [[[5.0, -4.0, 2.0], [3.0, -1.0, 7.0]], [[-2.0, 6.0, -4.0], [-3.0, 4.0, 1.0]]], [[[-6.0, -2.0, -3.0], [-6.0, -3.0, 2.0]], [[-1.0, 0.0, 6.0], [3.0, 4.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[3.0, 2.0, -6.0], [-1.0, 3.0, 5.0]], [[-6.0, -4.0, -4.0], [0.0, 0.0, -1.0]]], [[[-4.0, 4.0, 7.0], [0.0, 1.0, -6.0]], [[-5.0, 0.0, 1.0], [3.0, 3.0, -5.0]]], [[[-5.0, 0.0, 1.0], [0.0, 0.0, 1.0]], [[4.0, 6.0, 5.0], [1.0, -2.0, 0.0]]], [[[2.0, 5.0, -5.0], [4.0, 3.0, -6.0]], [[0.0, 0.0, -6.0], [-2.0, 4.0, -1.0]]], [[[-6.0, 2.0, -4.0], [-3.0, 6.0, -3.0]], [[-5.0, 6.0, 1.0], [-1.0, 2.0, -4.0]]], [[[-5.0, -4.0, -2.0], [1.0, 4.0, -3.0]], [[-5.0, 2.0, -2.0], [-6.0, -2.0, 2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 4.0], [5.0, -1.0], [6.0, 4.0]])+(1.-msk_arg1)*numpy.array([[3.0, 1.0], [4.0, -6.0], [-4.0, 5.0]])
      res=generalTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[54.0, -2.0], [-21.0, -41.0]], [[24.0, -8.0], [14.0, 0.0]]], [[[71.0, -21.0], [39.0, -11.0]], [[-27.0, -5.0], [45.0, -21.0]]], [[[20.0, -22.0], [-23.0, -13.0]], [[18.0, -4.0], [-32.0, -4.0]]], [[[-18.0, -12.0], [15.0, 21.0]], [[-58.0, 8.0], [-34.0, 2.0]]], [[[-28.0, 32.0], [25.0, 41.0]], [[14.0, -30.0], [38.0, -12.0]]], [[[-4.0, -34.0], [21.0, -13.0]], [[40.0, 20.0], [8.0, 8.0]]]])+(1.-msk_ref)*numpy.array([[[[41.0, -39.0], [-11.0, 6.0]], [[-18.0, -2.0], [4.0, -5.0]]], [[[-24.0, 7.0], [28.0, -36.0]], [[-19.0, 0.0], [41.0, -40.0]]], [[[-19.0, 0.0], [-4.0, 5.0]], [[16.0, -7.0], [-5.0, 13.0]]], [[[46.0, -53.0], [48.0, -44.0]], [[24.0, -30.0], [14.0, -31.0]]], [[[6.0, -38.0], [27.0, -54.0]], [[5.0, -36.0], [21.0, -33.0]]], [[[-23.0, 9.0], [31.0, -38.0]], [[1.0, -27.0], [-34.0, 16.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 4.0, -7.0, 3.0], [1.0, 5.0, -6.0, 3.0], [7.0, -1.0, -3.0, -2.0]], [[4.0, -3.0, -3.0, 5.0], [-5.0, 3.0, 4.0, -7.0], [1.0, -4.0, -6.0, -3.0]]], [[[5.0, 6.0, -3.0, 2.0], [-2.0, 0.0, 6.0, 6.0], [-7.0, 4.0, -2.0, 7.0]], [[-2.0, -7.0, 1.0, -5.0], [-7.0, 4.0, -1.0, -6.0], [-6.0, -4.0, -4.0, 3.0]]], [[[6.0, -7.0, 3.0, -6.0], [4.0, -3.0, 4.0, -3.0], [0.0, 1.0, 7.0, 0.0]], [[3.0, 5.0, -7.0, 2.0], [2.0, 2.0, 6.0, -5.0], [-4.0, 4.0, -1.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -2.0, 6.0, -4.0], [-2.0, 2.0, 0.0, -7.0], [0.0, 4.0, 0.0, 0.0]], [[-3.0, -6.0, -3.0, -2.0], [-7.0, 1.0, -7.0, 5.0], [1.0, 2.0, 3.0, 5.0]]], [[[-3.0, 6.0, 1.0, -2.0], [0.0, 0.0, -1.0, -6.0], [1.0, 7.0, 4.0, 3.0]], [[-5.0, 1.0, 4.0, 4.0], [-2.0, 2.0, -4.0, 7.0], [0.0, 4.0, -2.0, 7.0]]], [[[-5.0, -4.0, 0.0, 2.0], [-2.0, -7.0, -1.0, 4.0], [-6.0, -5.0, 6.0, 1.0]], [[2.0, 2.0, 6.0, 5.0], [1.0, 1.0, -3.0, 2.0], [0.0, 1.0, -4.0, -1.0]]]])
      arg1=-7.0
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-21.0, -28.0, 49.0, -21.0], [-7.0, -35.0, 42.0, -21.0], [-49.0, 7.0, 21.0, 14.0]], [[-28.0, 21.0, 21.0, -35.0], [35.0, -21.0, -28.0, 49.0], [-7.0, 28.0, 42.0, 21.0]]], [[[-35.0, -42.0, 21.0, -14.0], [14.0, -0.0, -42.0, -42.0], [49.0, -28.0, 14.0, -49.0]], [[14.0, 49.0, -7.0, 35.0], [49.0, -28.0, 7.0, 42.0], [42.0, 28.0, 28.0, -21.0]]], [[[-42.0, 49.0, -21.0, 42.0], [-28.0, 21.0, -28.0, 21.0], [-0.0, -7.0, -49.0, -0.0]], [[-21.0, -35.0, 49.0, -14.0], [-14.0, -14.0, -42.0, 35.0], [28.0, -28.0, 7.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[28.0, 14.0, -42.0, 28.0], [14.0, -14.0, -0.0, 49.0], [-0.0, -28.0, -0.0, -0.0]], [[21.0, 42.0, 21.0, 14.0], [49.0, -7.0, 49.0, -35.0], [-7.0, -14.0, -21.0, -35.0]]], [[[21.0, -42.0, -7.0, 14.0], [-0.0, -0.0, 7.0, 42.0], [-7.0, -49.0, -28.0, -21.0]], [[35.0, -7.0, -28.0, -28.0], [14.0, -14.0, 28.0, -49.0], [-0.0, -28.0, 14.0, -49.0]]], [[[35.0, 28.0, -0.0, -14.0], [14.0, 49.0, 7.0, -28.0], [42.0, 35.0, -42.0, -7.0]], [[-14.0, -14.0, -42.0, -35.0], [-7.0, -7.0, 21.0, -14.0], [-0.0, -7.0, 28.0, 7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -2.0, 1.0, 0.0], [3.0, 1.0, 0.0, -4.0], [1.0, 0.0, -1.0, -7.0]], [[2.0, 0.0, -1.0, -3.0], [1.0, 2.0, -6.0, 3.0], [-1.0, 1.0, -6.0, -1.0]]], [[[3.0, 0.0, -6.0, 0.0], [-4.0, 0.0, 3.0, 1.0], [-2.0, 1.0, 0.0, -1.0]], [[0.0, 3.0, -6.0, 2.0], [-7.0, 3.0, -4.0, -2.0], [6.0, 0.0, -6.0, -4.0]]], [[[4.0, -5.0, 3.0, 7.0], [2.0, 7.0, -3.0, 3.0], [4.0, 5.0, -2.0, -5.0]], [[-2.0, -7.0, 3.0, -1.0], [1.0, 1.0, -1.0, -7.0], [-6.0, -5.0, -5.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 5.0, -5.0, 1.0], [-3.0, 5.0, 6.0, 1.0], [4.0, 0.0, -1.0, 6.0]], [[-4.0, 0.0, 2.0, 1.0], [7.0, 6.0, -1.0, -1.0], [4.0, 7.0, -3.0, -7.0]]], [[[1.0, -2.0, 6.0, 5.0], [2.0, 5.0, 7.0, 6.0], [-1.0, 0.0, -2.0, -6.0]], [[5.0, -1.0, -2.0, 4.0], [5.0, 4.0, 1.0, 3.0], [-2.0, 7.0, -1.0, -1.0]]], [[[0.0, -7.0, 6.0, -4.0], [-6.0, 3.0, -5.0, 0.0], [0.0, 0.0, 4.0, -3.0]], [[-3.0, -6.0, 4.0, 0.0], [-5.0, -4.0, -3.0, 5.0], [5.0, -2.0, 1.0, 6.0]]]])
      arg1=numpy.array(0.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0], [0.0, 0.0, -0.0, -0.0]], [[0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]]], [[[0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, -0.0]], [[0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0]]], [[[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, -0.0]], [[-0.0, -0.0, 0.0, -0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0]], [[-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0]]], [[[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]], [[0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]]], [[[0.0, -0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[-0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 7.0, 2.0, -5.0], [-2.0, 0.0, 5.0, 7.0], [2.0, -2.0, -7.0, 7.0]], [[-6.0, -7.0, -6.0, -2.0], [4.0, 2.0, 0.0, 7.0], [-4.0, -6.0, -1.0, 2.0]]], [[[0.0, 6.0, 0.0, 0.0], [6.0, -2.0, 2.0, -5.0], [-1.0, 3.0, -7.0, -4.0]], [[-6.0, -7.0, -3.0, -7.0], [4.0, 5.0, 4.0, -1.0], [-2.0, -6.0, 2.0, 5.0]]], [[[5.0, 0.0, -5.0, 0.0], [4.0, -1.0, 0.0, 6.0], [-2.0, -1.0, 4.0, -5.0]], [[-1.0, -1.0, 3.0, 3.0], [-3.0, -1.0, 4.0, 5.0], [-4.0, -4.0, -5.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, -3.0, -1.0, 1.0], [0.0, 7.0, -5.0, 7.0], [0.0, 0.0, -2.0, -1.0]], [[-6.0, 3.0, 6.0, 3.0], [2.0, -5.0, -7.0, -6.0], [-1.0, 4.0, -5.0, 7.0]]], [[[-2.0, -6.0, 4.0, -3.0], [-1.0, 6.0, -6.0, -7.0], [-4.0, -7.0, 3.0, -4.0]], [[3.0, -2.0, -6.0, 1.0], [7.0, 5.0, -1.0, 7.0], [-7.0, -7.0, 1.0, 6.0]]], [[[6.0, -7.0, -4.0, 3.0], [-6.0, -3.0, -2.0, 4.0], [-7.0, -1.0, 0.0, 4.0]], [[1.0, 7.0, 0.0, -3.0], [-2.0, 4.0, -2.0, 1.0], [-2.0, 0.0, 4.0, -3.0]]]])
      arg1=Symbol(shape=())
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-2.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-8.0, -14.0, -4.0, 10.0], [4.0, -0.0, -10.0, -14.0], [-4.0, 4.0, 14.0, -14.0]], [[12.0, 14.0, 12.0, 4.0], [-8.0, -4.0, -0.0, -14.0], [8.0, 12.0, 2.0, -4.0]]], [[[-0.0, -12.0, -0.0, -0.0], [-12.0, 4.0, -4.0, 10.0], [2.0, -6.0, 14.0, 8.0]], [[12.0, 14.0, 6.0, 14.0], [-8.0, -10.0, -8.0, 2.0], [4.0, 12.0, -4.0, -10.0]]], [[[-10.0, -0.0, 10.0, -0.0], [-8.0, 2.0, -0.0, -12.0], [4.0, 2.0, -8.0, 10.0]], [[2.0, 2.0, -6.0, -6.0], [6.0, 2.0, -8.0, -10.0], [8.0, 8.0, 10.0, 12.0]]]])+(1.-msk_ref)*numpy.array([[[[-10.0, 6.0, 2.0, -2.0], [-0.0, -14.0, 10.0, -14.0], [-0.0, -0.0, 4.0, 2.0]], [[12.0, -6.0, -12.0, -6.0], [-4.0, 10.0, 14.0, 12.0], [2.0, -8.0, 10.0, -14.0]]], [[[4.0, 12.0, -8.0, 6.0], [2.0, -12.0, 12.0, 14.0], [8.0, 14.0, -6.0, 8.0]], [[-6.0, 4.0, 12.0, -2.0], [-14.0, -10.0, 2.0, -14.0], [14.0, 14.0, -2.0, -12.0]]], [[[-12.0, 14.0, 8.0, -6.0], [12.0, 6.0, 4.0, -8.0], [14.0, 2.0, -0.0, -8.0]], [[-2.0, -14.0, -0.0, 6.0], [4.0, -8.0, 4.0, -2.0], [4.0, -0.0, -8.0, 6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, 1.0, 4.0, -7.0], [0.0, -6.0, 2.0, 1.0], [-6.0, 4.0, -6.0, 3.0]], [[-4.0, 7.0, 5.0, -5.0], [-2.0, -7.0, 4.0, 0.0], [5.0, 5.0, -6.0, 0.0]]], [[[-7.0, 4.0, 7.0, -6.0], [0.0, 0.0, 5.0, -5.0], [-1.0, -3.0, 3.0, -3.0]], [[4.0, 4.0, 0.0, -5.0], [-1.0, 0.0, -4.0, -5.0], [1.0, -5.0, 1.0, 4.0]]], [[[5.0, 4.0, 1.0, -3.0], [0.0, 7.0, -1.0, 3.0], [1.0, -6.0, -5.0, -5.0]], [[-4.0, -1.0, 1.0, 6.0], [-3.0, 0.0, -2.0, 0.0], [-6.0, -3.0, 0.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[3.0, -3.0, -3.0, 0.0], [-3.0, -4.0, -4.0, 6.0], [0.0, -6.0, -5.0, 5.0]], [[6.0, 1.0, 2.0, 7.0], [0.0, 7.0, 1.0, 5.0], [-2.0, -4.0, 6.0, 5.0]]], [[[-3.0, -1.0, 5.0, -5.0], [3.0, 6.0, 6.0, 0.0], [6.0, 1.0, 1.0, 5.0]], [[7.0, -7.0, -6.0, 7.0], [-3.0, 6.0, 6.0, -2.0], [-2.0, 1.0, -2.0, 5.0]]], [[[-6.0, 4.0, -6.0, 0.0], [-1.0, -7.0, 0.0, 0.0], [4.0, -7.0, -4.0, -2.0]], [[3.0, -3.0, 6.0, -2.0], [5.0, -5.0, 3.0, -2.0], [0.0, -4.0, 4.0, 6.0]]]])
      arg1=Data(0.0,self.functionspace)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0]]], [[[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, -0.0]], [[0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, -0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [0.0, -0.0, -0.0, -0.0]], [[-0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]]], [[[-0.0, -0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0]], [[0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0], [0.0, -0.0, -0.0, -0.0]], [[0.0, -0.0, 0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorProduct_expandedData_rank4_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 0.0, -2.0, 6.0], [7.0, -4.0, 1.0, -5.0], [5.0, 4.0, -6.0, -4.0]], [[7.0, -7.0, -4.0, 6.0], [2.0, -3.0, 0.0, -2.0], [-6.0, 4.0, -2.0, 1.0]]], [[[1.0, 3.0, -7.0, 2.0], [-7.0, -6.0, -4.0, 1.0], [0.0, -3.0, 0.0, -4.0]], [[-3.0, -3.0, -1.0, -7.0], [-3.0, 6.0, 7.0, 2.0], [5.0, 2.0, -7.0, -7.0]]], [[[7.0, -6.0, 7.0, -6.0], [7.0, 1.0, -3.0, -3.0], [-1.0, -1.0, -3.0, -1.0]], [[4.0, -1.0, -3.0, 0.0], [-4.0, -2.0, -4.0, -4.0], [-6.0, 6.0, -7.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 7.0, -2.0, 7.0], [-6.0, -1.0, 3.0, -4.0], [0.0, 6.0, -2.0, 1.0]], [[3.0, 2.0, -2.0, 5.0], [-3.0, 7.0, 7.0, 5.0], [6.0, -2.0, -5.0, 0.0]]], [[[2.0, 6.0, -4.0, 6.0], [4.0, 2.0, 4.0, -2.0], [1.0, 3.0, 4.0, -7.0]], [[4.0, 0.0, -5.0, -2.0], [3.0, -5.0, 4.0, -2.0], [-3.0, 3.0, 2.0, -3.0]]], [[[-2.0, -6.0, 0.0, 3.0], [7.0, -2.0, -3.0, -3.0], [-7.0, 6.0, -3.0, 2.0]], [[6.0, -3.0, -1.0, 5.0], [-3.0, -1.0, -2.0, 3.0], [0.0, -4.0, 0.0, -4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(6.0)+(1-msk_arg1)*(1.0)
      res=generalTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, 0.0, -12.0, 36.0], [42.0, -24.0, 6.0, -30.0], [30.0, 24.0, -36.0, -24.0]], [[42.0, -42.0, -24.0, 36.0], [12.0, -18.0, 0.0, -12.0], [-36.0, 24.0, -12.0, 6.0]]], [[[6.0, 18.0, -42.0, 12.0], [-42.0, -36.0, -24.0, 6.0], [0.0, -18.0, 0.0, -24.0]], [[-18.0, -18.0, -6.0, -42.0], [-18.0, 36.0, 42.0, 12.0], [30.0, 12.0, -42.0, -42.0]]], [[[42.0, -36.0, 42.0, -36.0], [42.0, 6.0, -18.0, -18.0], [-6.0, -6.0, -18.0, -6.0]], [[24.0, -6.0, -18.0, 0.0], [-24.0, -12.0, -24.0, -24.0], [-36.0, 36.0, -42.0, 42.0]]]])+(1.-msk_ref)*numpy.array([[[[-7.0, 7.0, -2.0, 7.0], [-6.0, -1.0, 3.0, -4.0], [0.0, 6.0, -2.0, 1.0]], [[3.0, 2.0, -2.0, 5.0], [-3.0, 7.0, 7.0, 5.0], [6.0, -2.0, -5.0, 0.0]]], [[[2.0, 6.0, -4.0, 6.0], [4.0, 2.0, 4.0, -2.0], [1.0, 3.0, 4.0, -7.0]], [[4.0, 0.0, -5.0, -2.0], [3.0, -5.0, 4.0, -2.0], [-3.0, 3.0, 2.0, -3.0]]], [[[-2.0, -6.0, 0.0, 3.0], [7.0, -2.0, -3.0, -3.0], [-7.0, 6.0, -3.0, 2.0]], [[6.0, -3.0, -1.0, 5.0], [-3.0, -1.0, -2.0, 3.0], [0.0, -4.0, 0.0, -4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[0.0, 7.0, 0.0], [6.0, 6.0, 4.0]])
      arg1=numpy.array([7.0, -2.0, 0.0])
      res=matrix_mult(arg0,arg1)
      ref=numpy.array([-14.0, 30.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[6.0, 3.0, 3.0], [7.0, 3.0, 6.0]])
      arg1=numpy.array([[-6.0, -6.0], [-5.0, -1.0], [-1.0, 7.0]])
      res=matrix_mult(arg0,arg1)
      ref=numpy.array([[-54.0, -18.0], [-63.0, -3.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-1.0, 3.0, 2.0], [-1.0, -5.0, 1.0]])
      arg1=Symbol(shape=(3,))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([0.0, 0.0, -2.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-4.0, -2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-3.0, 2.0, -7.0], [5.0, 1.0, 2.0]])
      arg1=Symbol(shape=(3, 2))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([[-5.0, -3.0], [-7.0, 2.0], [6.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-41.0, 27.0], [-20.0, -17.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[-4.0, -7.0, 7.0], [-2.0, 4.0, 1.0]])
      arg1=Data(numpy.array([3.0, 4.0, 7.0]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([9.0, 17.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[5.0, -3.0, 2.0], [-6.0, -3.0, -3.0]])
      arg1=Data(numpy.array([[0.0, 0.0], [3.0, 6.0], [-7.0, -6.0]]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[-23.0, -30.0], [12.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[1.0, 4.0, 7.0], [-3.0, -6.0, -2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, -1.0, 1.0])+(1.-msk_arg1)*numpy.array([-4.0, 0.0, -3.0])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([8.0, -11.0])+(1.-msk_ref)*numpy.array([-25.0, 18.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-7.0, -5.0, -4.0], [1.0, 4.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, 1.0], [-1.0, 0.0], [-4.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 4.0], [7.0, 3.0], [-1.0, -5.0]])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-21.0, 9.0], [-10.0, -11.0]])+(1.-msk_ref)*numpy.array([[11.0, -23.0], [19.0, 1.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([4.0, -5.0, -1.0])
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[2.0, 7.0, -6.0], [0.0, 3.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-21.0, -9.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[5.0, -5.0], [6.0, 0.0], [5.0, 4.0]])
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-5.0, 4.0, 4.0], [4.0, 0.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[19.0, 41.0], [20.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3,))
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-7.0, -4.0, 4.0], [1.0, -6.0, 6.0]])
      s1=numpy.array([-2.0, 0.0, 4.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([30.0, 22.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3, 2))
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-6.0, 2.0, 7.0], [-2.0, 2.0, 5.0]])
      s1=numpy.array([[-6.0, -4.0], [-4.0, 1.0], [2.0, 2.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[42.0, 40.0], [14.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([6.0, 4.0, 2.0]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[1.0, -2.0, -5.0], [-7.0, 6.0, 6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-12.0, -6.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[6.0, 0.0], [7.0, -3.0], [6.0, 7.0]]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-2.0, -1.0, 4.0], [5.0, 2.0, 5.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[5.0, 31.0], [74.0, 29.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, -1.0, -6.0])+(1.-msk_arg1)*numpy.array([-1.0, -1.0, -1.0])
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-7.0, 2.0, 5.0], [0.0, -5.0, -7.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-67.0, 47.0])+(1.-msk_ref)*numpy.array([0.0, 12.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 5.0], [-7.0, 7.0], [0.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -6.0], [0.0, 7.0], [5.0, 6.0]])
      res=matrix_mult(arg0,arg1)
      s0=numpy.array([[-6.0, -1.0, 4.0], [-5.0, 4.0, 6.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[19.0, -37.0], [-18.0, 3.0]])+(1.-msk_ref)*numpy.array([[44.0, 53.0], [50.0, 94.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-7.0, -3.0, -3.0], [-1.0, 7.0, -7.0]]),self.functionspace)
      arg1=numpy.array([2.0, -6.0, -7.0])
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([25.0, 5.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[-1.0, -3.0, 2.0], [-2.0, 7.0, 0.0]]),self.functionspace)
      arg1=numpy.array([[0.0, -1.0], [5.0, -1.0], [-1.0, -4.0]])
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[-17.0, -4.0], [35.0, -5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[-2.0, -3.0, 0.0], [3.0, 6.0, 6.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([7.0, 7.0, -3.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-35.0, 45.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-6.0, 7.0, -5.0], [3.0, -1.0, 6.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([[0.0, -7.0], [-6.0, 7.0], [0.0, -1.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-42.0, 96.0], [6.0, -34.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[0.0, -2.0, 3.0], [-6.0, -7.0, -1.0]]),self.functionspace)
      arg1=Data(numpy.array([0.0, -3.0, -4.0]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([-6.0, 25.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-5.0, 5.0, -2.0], [3.0, 5.0, -5.0]]),self.functionspace)
      arg1=Data(numpy.array([[5.0, -4.0], [-6.0, 6.0], [-4.0, -2.0]]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[-47.0, 54.0], [5.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[-7.0, 2.0, 2.0], [4.0, -5.0, 2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, 5.0, 4.0])+(1.-msk_arg1)*numpy.array([-4.0, 0.0, -4.0])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-3.0, -5.0])+(1.-msk_ref)*numpy.array([20.0, -24.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[2.0, -3.0, 3.0], [6.0, -1.0, 6.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 0.0], [0.0, 7.0], [1.0, -3.0]])+(1.-msk_arg1)*numpy.array([[6.0, 3.0], [5.0, 5.0], [-3.0, -4.0]])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[11.0, -30.0], [30.0, -25.0]])+(1.-msk_ref)*numpy.array([[-12.0, -21.0], [13.0, -11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 5.0, 0.0], [-7.0, -1.0, 2.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 0.0, 6.0], [-7.0, -3.0, 7.0]])
      arg1=numpy.array([-4.0, 6.0, 6.0])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([38.0, 34.0])+(1.-msk_ref)*numpy.array([56.0, 52.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, -1.0, -6.0], [2.0, -7.0, 2.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 0.0, 0.0], [3.0, 2.0, -7.0]])
      arg1=numpy.array([[3.0, 5.0], [-7.0, -2.0], [7.0, 6.0]])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-44.0, -49.0], [69.0, 36.0]])+(1.-msk_ref)*numpy.array([[-15.0, -25.0], [-54.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, -6.0, 0.0], [0.0, 2.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 7.0, 7.0], [-1.0, -5.0, 1.0]])
      arg1=Symbol(shape=(3,))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([3.0, 0.0, 0.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, 0.0])+(1.-msk_ref)*numpy.array([-3.0, -3.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, -1.0, 4.0], [0.0, 2.0, 3.0]])+(1.-msk_arg0)*numpy.array([[3.0, 3.0, -3.0], [0.0, -6.0, 0.0]])
      arg1=Symbol(shape=(3, 2))
      res=matrix_mult(arg0,arg1)
      s1=numpy.array([[0.0, -6.0], [3.0, -1.0], [4.0, -3.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[13.0, 1.0], [18.0, -11.0]])+(1.-msk_ref)*numpy.array([[-3.0, -12.0], [-18.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, -2.0, 4.0], [-6.0, 2.0, -7.0]])+(1.-msk_arg0)*numpy.array([[4.0, -3.0, -1.0], [7.0, 3.0, 0.0]])
      arg1=Data(numpy.array([0.0, -1.0, -1.0]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-2.0, 5.0])+(1.-msk_ref)*numpy.array([4.0, -3.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, 6.0, -1.0], [3.0, 3.0, 5.0]])+(1.-msk_arg0)*numpy.array([[0.0, -2.0, 0.0], [4.0, -3.0, -3.0]])
      arg1=Data(numpy.array([[-6.0, 0.0], [6.0, 4.0], [-4.0, 4.0]]),self.functionspace)
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[70.0, 20.0], [-20.0, 32.0]])+(1.-msk_ref)*numpy.array([[-12.0, -8.0], [-30.0, -24.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -6.0, -6.0], [0.0, -3.0, -7.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 0.0, -2.0], [3.0, 0.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -5.0, -1.0])+(1.-msk_arg1)*numpy.array([-1.0, 0.0, 4.0])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([36.0, 22.0])+(1.-msk_ref)*numpy.array([-4.0, 1.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 0.0, 0.0], [-1.0, -7.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 0.0, 7.0], [3.0, 5.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 0.0], [-1.0, 7.0], [1.0, -4.0]])+(1.-msk_arg1)*numpy.array([[0.0, 4.0], [0.0, -4.0], [-7.0, -4.0]])
      res=matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 0.0], [5.0, -49.0]])+(1.-msk_ref)*numpy.array([[-49.0, -36.0], [49.0, 20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[7.0, 5.0], [-5.0, -5.0], [-1.0, 6.0]])
      arg1=numpy.array([0.0, 4.0, 3.0])
      res=transposed_matrix_mult(arg0,arg1)
      ref=numpy.array([-23.0, -2.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[-1.0, 3.0], [-7.0, 0.0], [3.0, 0.0]])
      arg1=numpy.array([[0.0, 2.0], [-3.0, 7.0], [1.0, 5.0]])
      res=transposed_matrix_mult(arg0,arg1)
      ref=numpy.array([[24.0, -36.0], [0.0, 6.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[0.0, -2.0], [0.0, 3.0], [2.0, 7.0]])
      arg1=Symbol(shape=(3,))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([-5.0, 2.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([12.0, 58.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-3.0, -6.0], [7.0, 0.0], [-7.0, 5.0]])
      arg1=Symbol(shape=(3, 2))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([[-1.0, 7.0], [7.0, 2.0], [-6.0, 4.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[94.0, -35.0], [-24.0, -22.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[0.0, -4.0], [5.0, -6.0], [-4.0, 0.0]])
      arg1=Data(numpy.array([3.0, -4.0, -2.0]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([-12.0, 12.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-1.0, -5.0], [-4.0, 2.0], [7.0, -6.0]])
      arg1=Data(numpy.array([[7.0, -7.0], [0.0, 2.0], [2.0, 0.0]]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[7.0, -1.0], [-47.0, 39.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[6.0, 3.0], [-5.0, -2.0], [5.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 0.0, 4.0])+(1.-msk_arg1)*numpy.array([2.0, -4.0, 3.0])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([20.0, 28.0])+(1.-msk_ref)*numpy.array([47.0, 35.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-2.0, -3.0], [5.0, 7.0], [-2.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 3.0], [-2.0, 2.0], [6.0, -5.0]])+(1.-msk_arg1)*numpy.array([[-7.0, 3.0], [6.0, 3.0], [5.0, -3.0]])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, 14.0], [-59.0, 40.0]])+(1.-msk_ref)*numpy.array([[34.0, 15.0], [28.0, 33.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([-3.0, 1.0, 7.0])
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[-7.0, 0.0], [2.0, 0.0], [6.0, 2.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([65.0, 14.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([[0.0, 2.0], [6.0, -3.0], [1.0, -6.0]])
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[-4.0, -2.0], [5.0, -2.0], [-7.0, 3.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[23.0, 19.0], [-9.0, -16.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3,))
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[-1.0, 4.0], [7.0, 7.0], [0.0, 7.0]])
      s1=numpy.array([-3.0, -4.0, -7.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-25.0, -89.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3, 2))
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[0.0, 2.0], [-6.0, 0.0], [7.0, 7.0]])
      s1=numpy.array([[-4.0, 5.0], [-5.0, -1.0], [4.0, 1.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[58.0, 13.0], [20.0, 17.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([-5.0, -6.0, 0.0]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[7.0, -6.0], [5.0, 2.0], [3.0, -5.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-65.0, 18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([[0.0, -7.0], [-6.0, -2.0], [-1.0, -3.0]]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[-6.0, -5.0], [-7.0, -7.0], [1.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[41.0, 53.0], [43.0, 52.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, 2.0, 1.0])+(1.-msk_arg1)*numpy.array([-5.0, 3.0, 6.0])
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[1.0, 0.0], [2.0, -7.0], [4.0, 5.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([9.0, -9.0])+(1.-msk_ref)*numpy.array([25.0, 9.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 0.0], [0.0, -7.0], [4.0, 6.0]])+(1.-msk_arg1)*numpy.array([[2.0, 0.0], [-3.0, -3.0], [1.0, 6.0]])
      res=transposed_matrix_mult(arg0,arg1)
      s0=numpy.array([[-4.0, 6.0], [-6.0, -7.0], [3.0, 4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[20.0, 60.0], [4.0, 73.0]])+(1.-msk_ref)*numpy.array([[13.0, 36.0], [37.0, 45.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[-7.0, 1.0], [5.0, 0.0], [3.0, -3.0]]),self.functionspace)
      arg1=numpy.array([-2.0, 0.0, 7.0])
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([35.0, -23.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[1.0, -7.0], [7.0, 0.0], [-2.0, 4.0]]),self.functionspace)
      arg1=numpy.array([[3.0, -1.0], [2.0, 7.0], [1.0, -4.0]])
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[15.0, 56.0], [-17.0, -9.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[7.0, 4.0], [0.0, 0.0], [-4.0, 7.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([0.0, 1.0, -6.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([24.0, -42.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-7.0, 0.0], [-2.0, 0.0], [-5.0, -4.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([[5.0, 3.0], [0.0, 5.0], [-7.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.0, -21.0], [28.0, 8.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[-7.0, 5.0], [1.0, -7.0], [2.0, 1.0]]),self.functionspace)
      arg1=Data(numpy.array([-2.0, 5.0, -3.0]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([13.0, -48.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[5.0, 1.0], [-7.0, 4.0], [6.0, 7.0]]),self.functionspace)
      arg1=Data(numpy.array([[-7.0, 1.0], [-5.0, 0.0], [-6.0, -2.0]]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      ref=Data(numpy.array([[-36.0, -7.0], [-69.0, -13.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[5.0, -7.0], [0.0, 0.0], [5.0, -7.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -3.0, -2.0])+(1.-msk_arg1)*numpy.array([-5.0, -6.0, 4.0])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([20.0, -28.0])+(1.-msk_ref)*numpy.array([-5.0, 7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-3.0, 1.0], [0.0, 0.0], [-2.0, 4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, -1.0], [7.0, 2.0], [3.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -1.0], [0.0, -1.0], [-6.0, 4.0]])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[12.0, 15.0], [6.0, -25.0]])+(1.-msk_ref)*numpy.array([[24.0, -5.0], [-28.0, 15.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, -3.0], [6.0, -4.0], [6.0, 5.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0], [1.0, -3.0], [-2.0, -3.0]])
      arg1=numpy.array([1.0, 1.0, 3.0])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, 8.0])+(1.-msk_ref)*numpy.array([-5.0, -11.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -2.0], [4.0, -5.0], [-2.0, -5.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 7.0], [-4.0, 6.0], [6.0, -5.0]])
      arg1=numpy.array([[-3.0, 3.0], [2.0, 3.0], [5.0, -2.0]])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-2.0, 16.0], [-29.0, -11.0]])+(1.-msk_ref)*numpy.array([[28.0, -30.0], [-34.0, 49.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 0.0], [0.0, 5.0], [-1.0, -2.0]])+(1.-msk_arg0)*numpy.array([[1.0, -6.0], [1.0, 4.0], [0.0, -6.0]])
      arg1=Symbol(shape=(3,))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([3.0, 2.0, 1.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, 8.0])+(1.-msk_ref)*numpy.array([5.0, -16.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 3.0], [0.0, 3.0], [-1.0, 0.0]])+(1.-msk_arg0)*numpy.array([[5.0, -6.0], [0.0, -6.0], [4.0, -1.0]])
      arg1=Symbol(shape=(3, 2))
      res=transposed_matrix_mult(arg0,arg1)
      s1=numpy.array([[2.0, 4.0], [-2.0, 5.0], [6.0, 1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-2.0, 7.0], [0.0, 27.0]])+(1.-msk_ref)*numpy.array([[34.0, 24.0], [-6.0, -55.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 2.0], [0.0, 3.0], [3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 1.0], [5.0, -1.0], [-6.0, 6.0]])
      arg1=Data(numpy.array([6.0, 4.0, -5.0]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-15.0, 24.0])+(1.-msk_ref)*numpy.array([44.0, -28.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, 4.0], [0.0, -7.0], [5.0, 0.0]])+(1.-msk_arg0)*numpy.array([[0.0, -5.0], [1.0, 2.0], [1.0, 7.0]])
      arg1=Data(numpy.array([[-3.0, 6.0], [-1.0, 0.0], [0.0, 1.0]]),self.functionspace)
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-3.0, 11.0], [-5.0, 24.0]])+(1.-msk_ref)*numpy.array([[-1.0, 1.0], [13.0, -23.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, 7.0], [2.0, -5.0], [-6.0, 0.0]])+(1.-msk_arg0)*numpy.array([[0.0, 7.0], [0.0, -3.0], [6.0, -3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 6.0, 7.0])+(1.-msk_arg1)*numpy.array([5.0, -2.0, 7.0])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-65.0, -65.0])+(1.-msk_ref)*numpy.array([42.0, 20.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_matrix_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, -4.0], [-3.0, -5.0], [-5.0, 6.0]])+(1.-msk_arg0)*numpy.array([[2.0, 1.0], [2.0, -5.0], [-5.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, -3.0], [-4.0, 7.0], [0.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 1.0], [5.0, 6.0], [0.0, 2.0]])
      res=transposed_matrix_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[40.0, 0.0], [36.0, -23.0]])+(1.-msk_ref)*numpy.array([[6.0, 4.0], [-27.0, -25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[-5.0, 6.0], [-1.0, -1.0], [1.0, -7.0]])
      arg1=numpy.array([-7.0, 0.0, -2.0])
      res=transposed_tensor_mult(arg0,arg1)
      ref=numpy.array([33.0, -28.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[7.0, 5.0], [3.0, -7.0], [1.0, 2.0]])
      arg1=numpy.array([[-2.0, 5.0], [0.0, -1.0], [0.0, 3.0]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=numpy.array([[-14.0, 35.0], [-10.0, 38.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-5.0, 0.0], [-1.0, 1.0], [-2.0, 2.0]])
      arg1=Symbol(shape=(3,))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([-6.0, 5.0, -2.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([29.0, 1.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[5.0, 4.0], [0.0, 3.0], [-3.0, 7.0]])
      arg1=Symbol(shape=(3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[3.0, 5.0], [-7.0, 7.0], [-6.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[33.0, 25.0], [-51.0, 41.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[-4.0, -2.0], [-1.0, -7.0], [-7.0, 0.0]])
      arg1=Data(numpy.array([-6.0, 7.0, 1.0]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([10.0, -37.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-1.0, 0.0], [2.0, -6.0], [-2.0, 0.0]])
      arg1=Data(numpy.array([[5.0, 0.0], [7.0, -4.0], [-7.0, -5.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[23.0, 2.0], [-42.0, 24.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[7.0, 0.0], [-3.0, 4.0], [2.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, -7.0, -6.0])+(1.-msk_arg1)*numpy.array([0.0, -7.0, -6.0])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, -70.0])+(1.-msk_ref)*numpy.array([9.0, -70.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-4.0, -2.0], [4.0, -4.0], [1.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, -7.0], [6.0, -1.0], [7.0, 4.0]])+(1.-msk_arg1)*numpy.array([[7.0, -3.0], [-3.0, -6.0], [-3.0, 0.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, 28.0], [-8.0, 34.0]])+(1.-msk_ref)*numpy.array([[-43.0, -12.0], [-14.0, 30.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[-1.0, -6.0, 4.0, 4.0, -2.0], [5.0, 0.0, -2.0, -2.0, -4.0], [-1.0, -2.0, -7.0, -6.0, 3.0], [3.0, -2.0, 4.0, -1.0, 5.0]], [[6.0, 3.0, 6.0, 4.0, 7.0], [6.0, 7.0, 4.0, 5.0, -2.0], [-5.0, 6.0, 0.0, -7.0, 7.0], [1.0, -1.0, -2.0, 0.0, 7.0]], [[-5.0, 7.0, -4.0, 2.0, -1.0], [-1.0, 0.0, 0.0, 7.0, -7.0], [-2.0, 3.0, -4.0, 5.0, 4.0], [-3.0, -2.0, 2.0, 0.0, 6.0]]], [[[-5.0, -3.0, 2.0, -3.0, 3.0], [6.0, 1.0, 0.0, -7.0, -3.0], [-1.0, -7.0, 2.0, 5.0, 0.0], [1.0, -4.0, 5.0, 2.0, 7.0]], [[1.0, -3.0, -7.0, 7.0, -6.0], [6.0, -4.0, 2.0, 3.0, 7.0], [2.0, -1.0, 2.0, 1.0, 1.0], [-7.0, 3.0, -3.0, 7.0, 2.0]], [[6.0, -4.0, 3.0, 3.0, 5.0], [-3.0, -6.0, -7.0, 5.0, 0.0], [-7.0, 0.0, -3.0, -4.0, -6.0], [4.0, 4.0, 1.0, -2.0, -4.0]]]])
      arg1=numpy.array([[-7.0, -5.0, -5.0], [6.0, -6.0, -6.0]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=numpy.array([[-70.0, 16.0, -2.0, -136.0, 8.0], [-42.0, 31.0, 24.0, -136.0, 13.0], [66.0, -67.0, 87.0, 100.0, -46.0], [13.0, -37.0, 14.0, -11.0, -46.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[-1.0, 4.0, 6.0, 6.0, 0.0], [0.0, -1.0, -4.0, -3.0, -3.0], [-1.0, 3.0, -3.0, -1.0, -6.0], [-7.0, 0.0, -3.0, 5.0, 2.0]], [[5.0, 0.0, -2.0, 0.0, 1.0], [7.0, -2.0, 0.0, -7.0, 3.0], [5.0, -7.0, 0.0, 6.0, -1.0], [-4.0, 3.0, -2.0, -1.0, 0.0]], [[5.0, 4.0, 3.0, 5.0, -7.0], [3.0, -7.0, 6.0, -3.0, 3.0], [7.0, -2.0, -7.0, 0.0, 0.0], [-6.0, 7.0, -6.0, 0.0, 0.0]]], [[[6.0, -6.0, 4.0, 2.0, 7.0], [4.0, 6.0, -7.0, 6.0, 5.0], [-1.0, 6.0, -5.0, -2.0, 6.0], [5.0, 4.0, 6.0, 6.0, -1.0]], [[7.0, 3.0, 0.0, 2.0, -7.0], [0.0, 4.0, 3.0, -2.0, -6.0], [-7.0, 3.0, 5.0, -6.0, -6.0], [5.0, 5.0, 0.0, -2.0, 7.0]], [[6.0, -3.0, -2.0, 7.0, 7.0], [-3.0, -6.0, -2.0, -7.0, 3.0], [7.0, 4.0, -2.0, 5.0, 2.0], [-3.0, -6.0, -7.0, 0.0, -3.0]]]])
      arg1=numpy.array([[[3.0, 5.0], [-5.0, 1.0], [0.0, 0.0]], [[5.0, 3.0], [-5.0, 3.0], [-4.0, 6.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=numpy.array([[[-57.0, 75.0], [-21.0, -7.0], [56.0, 28.0], [-10.0, 84.0], [37.0, 43.0]], [[-3.0, 1.0], [41.0, -13.0], [-54.0, -44.0], [94.0, -52.0], [19.0, 3.0]], [[-26.0, 18.0], [43.0, 59.0], [-51.0, -27.0], [-33.0, 7.0], [39.0, -19.0]], [[11.0, -27.0], [4.0, -6.0], [59.0, -41.0], [60.0, 36.0], [-22.0, 10.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[3.0, 6.0, -5.0, 0.0, -4.0], [-2.0, 3.0, 0.0, 2.0, 1.0], [4.0, 1.0, -1.0, 7.0, 2.0], [5.0, -3.0, -5.0, 0.0, -1.0]], [[5.0, 0.0, -2.0, -2.0, 3.0], [5.0, 7.0, 2.0, 1.0, 7.0], [4.0, -7.0, -2.0, 4.0, -7.0], [5.0, 0.0, 5.0, 5.0, 0.0]], [[0.0, -4.0, 0.0, 0.0, -7.0], [-2.0, 6.0, -3.0, 1.0, 3.0], [-2.0, 5.0, -2.0, 2.0, 2.0], [4.0, -3.0, -3.0, -6.0, -4.0]]], [[[-3.0, -5.0, -1.0, 0.0, -7.0], [-4.0, 0.0, 1.0, -1.0, 4.0], [3.0, 2.0, 7.0, 2.0, 6.0], [-4.0, 4.0, 3.0, 7.0, 4.0]], [[-7.0, -3.0, 2.0, -3.0, 0.0], [-5.0, 5.0, 1.0, 6.0, -1.0], [3.0, 1.0, 5.0, -6.0, 0.0], [2.0, 0.0, 2.0, 6.0, 0.0]], [[5.0, 4.0, 0.0, -7.0, -6.0], [0.0, 3.0, 5.0, -2.0, -4.0], [4.0, -3.0, 1.0, 4.0, -2.0], [-6.0, -6.0, -6.0, 3.0, -7.0]]]])
      arg1=numpy.array([[[[2.0, 0.0, 4.0, -2.0, -3.0], [4.0, -1.0, 0.0, 2.0, -4.0], [-6.0, -4.0, 3.0, 5.0, 5.0], [-2.0, 0.0, 4.0, -4.0, 3.0]], [[0.0, 4.0, 1.0, -5.0, 4.0], [1.0, -2.0, 7.0, 6.0, 1.0], [7.0, -3.0, 1.0, 2.0, -5.0], [-2.0, -3.0, 0.0, -2.0, -3.0]], [[5.0, 1.0, 5.0, 4.0, -3.0], [-3.0, 0.0, -1.0, -7.0, 7.0], [-1.0, -5.0, 4.0, 5.0, -4.0], [3.0, 0.0, -4.0, -6.0, 0.0]]], [[[-2.0, 5.0, -3.0, 0.0, 5.0], [-7.0, 3.0, 3.0, 3.0, -3.0], [-3.0, 3.0, -2.0, -7.0, 7.0], [-2.0, -1.0, 0.0, 0.0, -7.0]], [[3.0, 4.0, -7.0, -3.0, 0.0], [7.0, -4.0, -2.0, 4.0, 2.0], [-7.0, -5.0, -6.0, -4.0, -3.0], [3.0, 0.0, 1.0, 0.0, 2.0]], [[5.0, -3.0, -7.0, -7.0, -3.0], [6.0, -4.0, -1.0, -5.0, 7.0], [3.0, -4.0, -6.0, -4.0, -4.0], [-5.0, -5.0, 1.0, 5.0, 1.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=numpy.array([[[[16.0, -38.0, 40.0, -45.0, -19.0], [19.0, -14.0, 35.0, -26.0, 23.0], [90.0, -21.0, 32.0, 54.0, -30.0], [-56.0, -37.0, 10.0, 3.0, 6.0]], [[13.0, -53.0, 12.0, -47.0, -43.0], [74.0, -25.0, -9.0, -7.0, -15.0], [16.0, -20.0, 6.0, 41.0, 4.0], [-43.0, -15.0, 41.0, 20.0, 51.0]], [[-2.0, -5.0, -33.0, 14.0, 2.0], [-1.0, -2.0, -21.0, -17.0, 25.0], [5.0, 13.0, -27.0, -30.0, -28.0], [22.0, 7.0, -18.0, 24.0, 2.0]], [[-44.0, 1.0, 68.0, 68.0, 13.0], [-65.0, 44.0, -1.0, 11.0, -57.0], [-14.0, 49.0, 58.0, 36.0, 47.0], [30.0, 41.0, -10.0, -31.0, -7.0]], [[-59.0, -12.0, 15.0, 7.0, 28.0], [21.0, 1.0, 13.0, 68.0, -51.0], [55.0, 45.0, 13.0, 24.0, -32.0], [25.0, 28.0, 6.0, 22.0, 22.0]]], [[[-21.0, -22.0, 34.0, -14.0, 12.0], [-4.0, 0.0, 35.0, 8.0, 1.0], [96.0, 16.0, 29.0, 38.0, -40.0], [-19.0, -11.0, -5.0, 10.0, -3.0]], [[66.0, 45.0, -7.0, -53.0, -8.0], [54.0, -49.0, 30.0, 11.0, 68.0], [-1.0, -100.0, -8.0, 27.0, -71.0], [-2.0, -36.0, -4.0, -47.0, 1.0]], [[11.0, -1.0, -58.0, -60.0, 7.0], [41.0, -25.0, 13.0, 15.0, 15.0], [22.0, -13.0, -48.0, -42.0, -14.0], [-37.0, -32.0, 18.0, 39.0, -6.0]], [[19.0, 30.0, -11.0, -9.0, -4.0], [43.0, -23.0, -7.0, 34.0, 1.0], [-51.0, -41.0, -11.0, 8.0, -16.0], [27.0, 8.0, 8.0, -26.0, 20.0]], [[-14.0, 59.0, 49.0, 6.0, 48.0], [-57.0, 17.0, 64.0, 51.0, -18.0], [23.0, -7.0, 44.0, 26.0, 5.0], [2.0, -5.0, -13.0, -56.0, -52.0]]], [[[21.0, 29.0, -48.0, -73.0, 13.0], [50.0, -31.0, 29.0, 47.0, -1.0], [-12.0, -40.0, -40.0, -31.0, 4.0], [-39.0, -35.0, 31.0, 8.0, -11.0]], [[11.0, 0.0, 30.0, 71.0, -27.0], [-43.0, 27.0, -47.0, -50.0, -1.0], [-82.0, 5.0, 24.0, 10.0, 43.0], [41.0, 34.0, -18.0, -35.0, 9.0]], [[-6.0, 42.0, -79.0, -18.0, 33.0], [-8.0, 2.0, -2.0, 36.0, -16.0], [-59.0, 12.0, -63.0, -92.0, 43.0], [-4.0, -6.0, 10.0, 25.0, -35.0]], [[22.0, -8.0, 50.0, -36.0, -13.0], [-6.0, -1.0, 40.0, -14.0, 0.0], [32.0, -30.0, 41.0, 47.0, 23.0], [-58.0, -34.0, 18.0, -28.0, -13.0]], [[-8.0, 10.0, 7.0, 53.0, -4.0], [-59.0, 38.0, -31.0, -24.0, -33.0], [-87.0, 29.0, 7.0, -28.0, 87.0], [14.0, 25.0, -2.0, -16.0, -17.0]]], [[[14.0, 30.0, 85.0, 17.0, -9.0], [19.0, -11.0, 21.0, 38.0, -13.0], [-19.0, -53.0, 68.0, 99.0, -26.0], [36.0, 19.0, 0.0, -84.0, 26.0]], [[-59.0, 35.0, 3.0, 36.0, 56.0], [-67.0, 39.0, 21.0, 57.0, -63.0], [-9.0, 63.0, 7.0, -34.0, 49.0], [19.0, 26.0, -6.0, 0.0, -43.0]], [[-55.0, 58.0, -11.0, 9.0, 77.0], [-49.0, 20.0, 49.0, 88.0, -43.0], [27.0, 43.0, -4.0, -35.0, 1.0], [21.0, 12.0, -12.0, -2.0, -53.0]], [[-11.0, 64.0, -109.0, -88.0, 64.0], [34.0, -25.0, 47.0, 102.0, -25.0], [-13.0, -6.0, -87.0, -105.0, 18.0], [-39.0, -37.0, 33.0, 41.0, -49.0]], [[-65.0, 37.0, 13.0, 35.0, 56.0], [-62.0, 41.0, 23.0, 73.0, -85.0], [-23.0, 64.0, 15.0, -25.0, 67.0], [17.0, 31.0, 5.0, -7.0, -38.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[3.0, -5.0, 6.0, -3.0, -4.0], [-1.0, 7.0, 0.0, -5.0, -1.0], [-5.0, -7.0, 5.0, 2.0, 7.0], [1.0, -7.0, 7.0, 3.0, -7.0]], [[0.0, 5.0, 7.0, -5.0, 3.0], [-7.0, 4.0, 0.0, 7.0, 2.0], [-5.0, 0.0, -1.0, 0.0, 6.0], [-2.0, 1.0, -3.0, -2.0, -4.0]], [[-2.0, 0.0, -5.0, -5.0, 7.0], [-5.0, 3.0, 4.0, 3.0, -4.0], [6.0, -4.0, 2.0, -3.0, 3.0], [0.0, 4.0, -3.0, 7.0, -2.0]]], [[[1.0, 6.0, 0.0, -7.0, -6.0], [5.0, -4.0, 3.0, 0.0, -1.0], [-7.0, -5.0, 4.0, -1.0, -7.0], [-5.0, -4.0, -7.0, 4.0, 5.0]], [[0.0, 0.0, 4.0, 3.0, 6.0], [-6.0, 0.0, 5.0, -2.0, 4.0], [-6.0, -2.0, 0.0, 0.0, -3.0], [-4.0, 2.0, -2.0, -5.0, -3.0]], [[-3.0, 1.0, -4.0, 3.0, 6.0], [3.0, 0.0, -1.0, 1.0, -5.0], [-5.0, 6.0, 6.0, 7.0, -5.0], [6.0, -1.0, 1.0, -6.0, -2.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[-6.0, -5.0, 7.0], [6.0, -6.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-17.0, 38.0, -118.0, -61.0, -32.0], [63.0, -65.0, 19.0, 25.0, -47.0], [106.0, -22.0, -5.0, -60.0, -60.0], [-20.0, 32.0, -81.0, 113.0, 102.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[-5.0, 5.0, -2.0, -4.0, 0.0], [1.0, -7.0, 4.0, -3.0, -7.0], [2.0, -5.0, -4.0, -1.0, -7.0], [-6.0, -4.0, -7.0, 3.0, -5.0]], [[-6.0, 5.0, -5.0, -4.0, 2.0], [-1.0, 0.0, 0.0, 1.0, -3.0], [0.0, -1.0, -1.0, -2.0, 2.0], [4.0, -1.0, 6.0, 5.0, -5.0]], [[2.0, 6.0, 2.0, 7.0, -2.0], [6.0, 3.0, 5.0, 5.0, -3.0], [-4.0, -1.0, -4.0, 5.0, 7.0], [-3.0, -6.0, 1.0, 2.0, 1.0]]], [[[-2.0, 1.0, 2.0, 2.0, -4.0], [-2.0, 7.0, -2.0, 2.0, 0.0], [0.0, -4.0, -4.0, 0.0, 5.0], [4.0, -4.0, -1.0, 1.0, -1.0]], [[-2.0, 1.0, -7.0, 0.0, -1.0], [-7.0, 4.0, 4.0, -2.0, -6.0], [-3.0, 7.0, -7.0, 5.0, 3.0], [-6.0, 2.0, -1.0, 5.0, 5.0]], [[6.0, 4.0, 6.0, 0.0, -5.0], [7.0, 2.0, -7.0, -2.0, 3.0], [-4.0, -3.0, 7.0, 6.0, 1.0], [1.0, 6.0, 6.0, -4.0, 6.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[1.0, -7.0], [1.0, 3.0], [4.0, 1.0]], [[-7.0, 1.0], [-1.0, -3.0], [0.0, -2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[13.0, 11.0], [26.0, -24.0], [-6.0, 12.0], [6.0, 25.0], [23.0, 13.0]], [[45.0, 1.0], [-48.0, 43.0], [34.0, -23.0], [6.0, 41.0], [-16.0, 49.0]], [[-11.0, -1.0], [11.0, 12.0], [14.0, 24.0], [12.0, -21.0], [-15.0, 56.0]], [[-36.0, 71.0], [-3.0, -3.0], [11.0, 58.0], [4.0, -10.0], [-4.0, -7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[-1.0, 7.0, 6.0, -7.0, 0.0], [4.0, 1.0, -1.0, -3.0, 0.0], [7.0, -3.0, -3.0, 0.0, -7.0], [-3.0, 1.0, 3.0, 2.0, 0.0]], [[6.0, 0.0, -2.0, -6.0, 1.0], [-5.0, -5.0, -4.0, 3.0, 2.0], [4.0, 2.0, 2.0, 7.0, -6.0], [-2.0, 0.0, -7.0, 0.0, 3.0]], [[-6.0, 0.0, 5.0, 2.0, -1.0], [0.0, 6.0, 4.0, 1.0, 2.0], [-7.0, 5.0, 0.0, 4.0, 5.0], [-3.0, 4.0, -4.0, 1.0, 0.0]]], [[[-1.0, 0.0, 5.0, -3.0, 2.0], [-1.0, 3.0, 7.0, -2.0, -1.0], [-3.0, 0.0, 1.0, 0.0, -6.0], [-1.0, -2.0, -6.0, -4.0, 2.0]], [[4.0, 7.0, 6.0, -3.0, 6.0], [-1.0, 0.0, 2.0, -7.0, 2.0], [6.0, -4.0, 3.0, 0.0, 0.0], [5.0, -6.0, -1.0, -5.0, 7.0]], [[6.0, 1.0, -6.0, 2.0, 5.0], [0.0, -4.0, -5.0, -6.0, 1.0], [-7.0, -3.0, -7.0, -2.0, -1.0], [-7.0, 6.0, 7.0, 5.0, 4.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[[-7.0, 0.0, 6.0, 7.0, -4.0], [-4.0, 7.0, -1.0, -5.0, 5.0], [-2.0, -2.0, 2.0, 6.0, 1.0], [7.0, -6.0, -5.0, -2.0, -7.0]], [[7.0, 7.0, 2.0, -4.0, -1.0], [1.0, 2.0, 3.0, -1.0, -4.0], [-7.0, -1.0, 4.0, -5.0, 0.0], [-7.0, -4.0, -6.0, 1.0, 5.0]], [[1.0, 6.0, -3.0, -2.0, -1.0], [-5.0, 4.0, 0.0, 6.0, 1.0], [4.0, -5.0, -5.0, 3.0, -6.0], [-5.0, 5.0, 2.0, 4.0, 3.0]]], [[[2.0, 0.0, -3.0, 0.0, 2.0], [-3.0, -7.0, 4.0, 0.0, -5.0], [6.0, 5.0, 1.0, 1.0, 5.0], [5.0, 0.0, -5.0, 3.0, 7.0]], [[7.0, 2.0, -1.0, 3.0, 4.0], [-6.0, 6.0, 5.0, -3.0, 6.0], [-7.0, 5.0, 1.0, 0.0, 0.0], [7.0, 7.0, -7.0, 4.0, 5.0]], [[7.0, 0.0, -2.0, -2.0, 0.0], [6.0, -2.0, -5.0, -7.0, -4.0], [-3.0, 3.0, -3.0, 0.0, 0.0], [-1.0, 4.0, 7.0, -3.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[111.0, 14.0, 11.0, -19.0, 18.0], [55.0, 0.0, 5.0, -91.0, -30.0], [-116.0, 59.0, 37.0, -55.0, 30.0], [-2.0, 4.0, -24.0, -21.0, 50.0]], [[7.0, 14.0, 33.0, 68.0, 0.0], [-64.0, 89.0, 23.0, -63.0, 73.0], [-66.0, 24.0, 18.0, 42.0, 7.0], [97.0, 11.0, -77.0, 11.0, -11.0]], [[-41.0, 28.0, 8.0, 70.0, 7.0], [-138.0, 71.0, 68.0, 26.0, 78.0], [28.0, 2.0, 8.0, 66.0, 1.0], [104.0, 15.0, -117.0, 63.0, 10.0]], [[-4.0, -36.0, -52.0, -42.0, 14.0], [51.0, -54.0, -48.0, 48.0, -20.0], [61.0, -14.0, -60.0, -9.0, -34.0], [-55.0, 63.0, 125.0, -11.0, -5.0]], [[87.0, 13.0, -17.0, 6.0, 28.0], [-6.0, 10.0, 16.0, -60.0, 1.0], [-56.0, 59.0, 2.0, -6.0, 16.0], [45.0, 53.0, -25.0, 12.0, 61.0]]], [[[-72.0, -37.0, 18.0, 45.0, -17.0], [-12.0, 19.0, -28.0, -12.0, 39.0], [28.0, -13.0, -14.0, 48.0, -1.0], [51.0, -11.0, 22.0, -20.0, -65.0]], [[-58.0, 1.0, -23.0, 23.0, 1.0], [-72.0, 8.0, 16.0, 64.0, 32.0], [87.0, -24.0, -33.0, 52.0, -20.0], [31.0, 28.0, -6.0, 38.0, -5.0]], [[-24.0, 0.0, -39.0, 17.0, 26.0], [-83.0, -26.0, 52.0, 62.0, 12.0], [89.0, 16.0, -14.0, 33.0, 10.0], [55.0, 36.0, -47.0, 58.0, 43.0]], [[-52.0, 13.0, 10.0, -44.0, -24.0], [22.0, -27.0, -1.0, 81.0, -34.0], [44.0, -65.0, 10.0, -32.0, -19.0], [-100.0, -62.0, 16.0, -3.0, -28.0]], [[35.0, 30.0, -3.0, -8.0, 2.0], [-11.0, 29.0, 7.0, -3.0, 7.0], [-29.0, -4.0, -4.0, -5.0, -17.0], [-16.0, 20.0, -10.0, 12.0, 22.0]]], [[[-41.0, -2.0, 88.0, 79.0, -7.0], [-58.0, 100.0, 58.0, -50.0, 91.0], [-109.0, 11.0, 89.0, -2.0, 34.0], [90.0, -79.0, -149.0, -2.0, -62.0]], [[-9.0, 36.0, -19.0, -45.0, -11.0], [-5.0, -15.0, 4.0, 76.0, -30.0], [49.0, -50.0, -18.0, -13.0, -33.0], [-85.0, -5.0, 20.0, 21.0, 17.0]], [[9.0, 20.0, -6.0, -6.0, 24.0], [-49.0, 8.0, 63.0, 53.0, 18.0], [-2.0, 3.0, 27.0, -27.0, 2.0], [-2.0, 3.0, -72.0, 44.0, 32.0]], [[39.0, 73.0, 6.0, -32.0, -11.0], [-25.0, 34.0, 31.0, 31.0, -16.0], [-27.0, -33.0, 14.0, -23.0, -24.0], [-67.0, -16.0, -48.0, 29.0, 41.0]], [[-7.0, -12.0, -49.0, -33.0, 17.0], [9.0, 3.0, -30.0, 78.0, 28.0], [43.0, -38.0, -66.0, -3.0, -67.0], [-61.0, 87.0, 104.0, 13.0, -11.0]]], [[[-12.0, -22.0, -1.0, 22.0, 35.0], [-44.0, 14.0, 53.0, 33.0, 53.0], [-12.0, 22.0, 26.0, -18.0, 10.0], [45.0, 18.0, -58.0, 30.0, -1.0]], [[-7.0, 12.0, -6.0, -31.0, -36.0], [54.0, -11.0, -69.0, -5.0, -41.0], [26.0, -44.0, -44.0, 16.0, -33.0], [-71.0, -4.0, 97.0, -34.0, -21.0]], [[-44.0, -75.0, 21.0, 40.0, -17.0], [67.0, 13.0, -88.0, -78.0, 35.0], [-23.0, 7.0, -30.0, 35.0, -3.0], [46.0, 11.0, 105.0, -72.0, -94.0]], [[-21.0, -4.0, 16.0, -13.0, -37.0], [59.0, 6.0, -68.0, -24.0, -19.0], [-4.0, -39.0, -25.0, 11.0, -24.0], [-51.0, -22.0, 82.0, -47.0, -49.0]], [[102.0, 35.0, -15.0, 1.0, 29.0], [-21.0, 26.0, 32.0, -52.0, 4.0], [-70.0, 54.0, 9.0, -13.0, 10.0], [34.0, 53.0, -49.0, 25.0, 76.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[0.0, 0.0, 3.0, 2.0, 0.0], [1.0, -6.0, -2.0, 0.0, 1.0], [-5.0, -6.0, -3.0, 1.0, -6.0], [-2.0, 2.0, 6.0, 6.0, 5.0]], [[2.0, -1.0, -5.0, 0.0, 1.0], [0.0, 6.0, 0.0, 7.0, 0.0], [1.0, 7.0, 1.0, 0.0, -1.0], [2.0, 1.0, -3.0, 3.0, 0.0]], [[1.0, 6.0, 6.0, 0.0, -7.0], [-7.0, 0.0, 3.0, 4.0, -1.0], [-6.0, 2.0, -4.0, 3.0, 0.0], [-5.0, 1.0, 3.0, 7.0, 0.0]]], [[[-7.0, -2.0, 2.0, 3.0, -5.0], [5.0, -6.0, 3.0, -7.0, -6.0], [2.0, 0.0, 3.0, 0.0, 2.0], [-1.0, -5.0, -7.0, 6.0, 2.0]], [[2.0, -2.0, -4.0, -3.0, 3.0], [5.0, 3.0, 0.0, -2.0, -6.0], [0.0, -5.0, -6.0, -5.0, -6.0], [-7.0, -1.0, 1.0, 1.0, -7.0]], [[2.0, -3.0, -4.0, 5.0, -6.0], [6.0, -7.0, -6.0, -3.0, 4.0], [0.0, -7.0, 5.0, 5.0, 0.0], [1.0, -1.0, 0.0, 2.0, 0.0]]]])
      arg1=Data(numpy.array([[6.0, 4.0, -3.0], [-6.0, 2.0, -1.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[49.0, -11.0, -36.0, -17.0, 67.0], [1.0, 37.0, -33.0, 57.0, 29.0], [-20.0, -17.0, -37.0, -18.0, -64.0], [2.0, 42.0, 59.0, -9.0, 4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[-1.0, 3.0, 4.0, -3.0, 0.0], [7.0, 2.0, 6.0, -6.0, -4.0], [-5.0, -2.0, 4.0, 5.0, -7.0], [0.0, -4.0, -3.0, 4.0, 3.0]], [[1.0, 0.0, -3.0, -2.0, -5.0], [5.0, 0.0, 4.0, -5.0, -6.0], [0.0, 1.0, -6.0, 1.0, 0.0], [7.0, 5.0, -5.0, 5.0, 5.0]], [[-3.0, -2.0, 0.0, -6.0, -1.0], [3.0, -2.0, 2.0, -4.0, 6.0], [-7.0, -4.0, 1.0, 5.0, -1.0], [7.0, 0.0, -3.0, -2.0, -2.0]]], [[[6.0, -7.0, -4.0, -2.0, 2.0], [-2.0, -7.0, -3.0, -1.0, 6.0], [3.0, 2.0, -1.0, -3.0, 0.0], [-6.0, 0.0, -3.0, 3.0, -6.0]], [[-4.0, -2.0, 2.0, 6.0, 3.0], [1.0, 0.0, 5.0, 3.0, -1.0], [-3.0, -3.0, 0.0, -7.0, 0.0], [7.0, -4.0, -7.0, 6.0, -4.0]], [[-6.0, -6.0, -5.0, 0.0, 2.0], [-6.0, 2.0, -5.0, 0.0, 2.0], [6.0, 5.0, -2.0, 3.0, 2.0], [-4.0, 0.0, 1.0, 2.0, 0.0]]]])
      arg1=Data(numpy.array([[[-6.0, 7.0], [0.0, 5.0], [-4.0, 0.0]], [[1.0, -3.0], [0.0, 4.0], [-2.0, -4.0]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[36.0, -12.0], [-5.0, 58.0], [-18.0, 53.0], [40.0, -1.0], [2.0, -27.0]], [[-44.0, 108.0], [-15.0, 27.0], [-37.0, 111.0], [51.0, -52.0], [2.0, -88.0]], [[49.0, -80.0], [20.0, -47.0], [-25.0, 9.0], [-59.0, 9.0], [42.0, -57.0]], [[-26.0, 97.0], [24.0, -19.0], [25.0, -69.0], [-17.0, 60.0], [-16.0, 48.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[7.0, 2.0, -1.0, 0.0, 2.0], [6.0, 0.0, 5.0, -1.0, 0.0], [-7.0, -3.0, 6.0, 0.0, 4.0], [-4.0, -1.0, 4.0, 6.0, -7.0]], [[-4.0, -5.0, 4.0, -7.0, -1.0], [6.0, -2.0, -5.0, 3.0, -2.0], [-7.0, 0.0, 3.0, 6.0, -3.0], [-3.0, -4.0, 0.0, 2.0, -1.0]], [[-3.0, -3.0, 7.0, 2.0, 1.0], [-2.0, 1.0, 2.0, 0.0, -4.0], [-6.0, 0.0, -6.0, -1.0, -6.0], [6.0, -1.0, 0.0, 1.0, -4.0]]], [[[3.0, 5.0, 0.0, -1.0, -5.0], [7.0, 4.0, -2.0, -5.0, 0.0], [-2.0, -4.0, 1.0, -4.0, 2.0], [0.0, 6.0, -3.0, 0.0, -4.0]], [[1.0, 0.0, 4.0, 3.0, 3.0], [-2.0, 6.0, 4.0, 5.0, 0.0], [0.0, 0.0, 0.0, 7.0, -4.0], [4.0, 7.0, 5.0, 5.0, 5.0]], [[-3.0, -3.0, 3.0, 2.0, 3.0], [-4.0, 4.0, 7.0, 4.0, 3.0], [-2.0, -1.0, 4.0, 2.0, -7.0], [-2.0, -5.0, -3.0, 5.0, -2.0]]]])
      arg1=Data(numpy.array([[[[-3.0, 5.0, -6.0, -5.0, 0.0], [0.0, 3.0, 4.0, 7.0, -1.0], [-3.0, 5.0, -7.0, 4.0, 0.0], [-1.0, -1.0, 2.0, -7.0, 3.0]], [[-2.0, -2.0, 1.0, -7.0, 2.0], [5.0, 2.0, -3.0, 6.0, -1.0], [2.0, -7.0, -3.0, -2.0, 4.0], [5.0, 1.0, 4.0, 6.0, 0.0]], [[-2.0, 2.0, 7.0, -7.0, -7.0], [4.0, -6.0, -2.0, 6.0, -4.0], [-3.0, 1.0, -4.0, -6.0, -5.0], [1.0, 0.0, 3.0, -1.0, -5.0]]], [[[-5.0, 0.0, -1.0, 0.0, 7.0], [5.0, 3.0, -2.0, -3.0, 2.0], [-5.0, 7.0, 1.0, 0.0, -1.0], [7.0, 2.0, 6.0, 1.0, 2.0]], [[-4.0, -1.0, -3.0, -4.0, -4.0], [-6.0, -1.0, 4.0, 0.0, -4.0], [4.0, -2.0, -3.0, 0.0, -1.0], [7.0, 1.0, -2.0, 6.0, 1.0]], [[-7.0, 1.0, -5.0, 5.0, -5.0], [0.0, -2.0, -2.0, 4.0, 6.0], [-7.0, 1.0, 1.0, 6.0, 4.0], [1.0, -4.0, 1.0, -5.0, -5.0]]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-5.0, 33.0, -58.0, -5.0, 45.0], [-23.0, 45.0, 50.0, -14.0, -7.0], [-10.0, 76.0, -28.0, 36.0, -17.0], [-5.0, 8.0, 2.0, -46.0, 58.0]], [[6.0, 11.0, -28.0, 31.0, 61.0], [-12.0, 35.0, 25.0, -61.0, 7.0], [-11.0, 74.0, 15.0, 18.0, -22.0], [2.0, 15.0, 2.0, -21.0, 46.0]], [[-56.0, 0.0, 32.0, -73.0, -72.0], [24.0, -47.0, -20.0, 71.0, -29.0], [-15.0, -31.0, -42.0, -36.0, -11.0], [59.0, -3.0, 30.0, 33.0, -49.0]], [[-11.0, 17.0, -11.0, 33.0, -57.0], [-50.0, -36.0, 27.0, -19.0, -3.0], [-17.0, 40.0, 5.0, 14.0, -32.0], [-17.0, -14.0, -32.0, -37.0, -19.0]], [[-14.0, 14.0, -25.0, -7.0, -71.0], [-44.0, -26.0, 25.0, 41.0, -9.0], [5.0, -20.0, -26.0, 22.0, 5.0], [-17.0, -22.0, -30.0, -23.0, -21.0]]], [[[-25.0, 12.0, -25.0, -70.0, 103.0], [69.0, 73.0, -4.0, 29.0, -6.0], [-15.0, 35.0, -43.0, 0.0, 13.0], [53.0, 28.0, 72.0, 11.0, 60.0]], [[-70.0, 4.0, -37.0, 3.0, -27.0], [-22.0, -12.0, 12.0, -2.0, 6.0], [-31.0, 35.0, -8.0, 22.0, -7.0], [65.0, -4.0, 11.0, 7.0, -11.0]], [[-64.0, 42.0, -66.0, 15.0, -89.0], [-51.0, -31.0, 37.0, 51.0, 14.0], [-54.0, 47.0, -35.0, 60.0, -4.0], [-7.0, -38.0, -17.0, -80.0, -30.0]], [[-26.0, -12.0, -21.0, -16.0, -69.0], [-40.0, -25.0, 9.0, 42.0, -8.0], [26.0, -67.0, -18.0, 14.0, 28.0], [20.0, -17.0, -26.0, 30.0, -28.0]], [[-9.0, -1.0, -45.0, 57.0, 9.0], [-26.0, 14.0, 8.0, -24.0, 36.0], [-13.0, 13.0, 25.0, 46.0, 24.0], [-11.0, -14.0, -17.0, -23.0, 5.0]]], [[[71.0, -35.0, 5.0, 116.0, 24.0], [-69.0, -1.0, 13.0, -129.0, 22.0], [49.0, -8.0, 90.0, 10.0, -4.0], [-50.0, 4.0, -74.0, 21.0, 15.0]], [[36.0, -16.0, 27.0, 10.0, -23.0], [-20.0, -19.0, -2.0, -13.0, -11.0], [36.0, -44.0, 16.0, -18.0, 0.0], [-26.0, -1.0, -31.0, 22.0, -12.0]], [[-45.0, 16.0, -96.0, 11.0, 35.0], [-4.0, 55.0, 17.0, 37.0, 41.0], [-27.0, 14.0, -22.0, 78.0, 57.0], [14.0, -17.0, 16.0, -37.0, 30.0]], [[-32.0, -19.0, -28.0, -53.0, -47.0], [-36.0, -5.0, 16.0, 50.0, -26.0], [49.0, -83.0, -37.0, 6.0, 34.0], [52.0, -3.0, -15.0, 65.0, -6.0]], [[61.0, 11.0, -24.0, 24.0, 101.0], [-5.0, 66.0, 31.0, -60.0, 1.0], [23.0, 50.0, 12.0, 16.0, -8.0], [-46.0, 21.0, -9.0, -27.0, 77.0]]], [[[4.0, -8.0, 61.0, -27.0, -54.0], [-15.0, -54.0, 1.0, -18.0, -45.0], [18.0, -3.0, -1.0, -58.0, -54.0], [21.0, 13.0, -12.0, 38.0, -28.0]], [[-10.0, -11.0, -7.0, -13.0, 38.0], [-36.0, 16.0, 36.0, -75.0, -37.0], [31.0, 45.0, 3.0, -20.0, -44.0], [66.0, 36.0, -4.0, 57.0, 46.0]], [[4.0, 12.0, -21.0, -55.0, -26.0], [-45.0, 4.0, 48.0, 25.0, -48.0], [44.0, -14.0, -49.0, -2.0, -14.0], [7.0, 7.0, -23.0, 14.0, 26.0]], [[-79.0, 28.0, -67.0, -46.0, -48.0], [-16.0, 1.0, 26.0, 80.0, -2.0], [-32.0, 12.0, -62.0, 44.0, 18.0], [45.0, -19.0, 18.0, -26.0, -7.0]], [[45.0, -48.0, 12.0, 40.0, -12.0], [-71.0, -12.0, 15.0, -75.0, -16.0], [85.0, -72.0, 47.0, -14.0, 7.0], [3.0, 11.0, -66.0, 83.0, 6.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[1.0, -1.0, 5.0, 5.0, 0.0], [-2.0, -4.0, 6.0, 3.0, -4.0], [-1.0, 2.0, -5.0, -5.0, 4.0], [1.0, -3.0, -6.0, 4.0, -7.0]], [[-6.0, 0.0, 2.0, 4.0, 1.0], [-2.0, 0.0, 7.0, 4.0, -3.0], [2.0, 6.0, 2.0, 0.0, 7.0], [-4.0, -5.0, 7.0, 6.0, 2.0]], [[-6.0, 4.0, -2.0, 4.0, -5.0], [-5.0, -1.0, 7.0, 4.0, 7.0], [-1.0, -7.0, 0.0, 1.0, -6.0], [-7.0, -4.0, -5.0, 0.0, -3.0]]], [[[-2.0, -1.0, 4.0, 0.0, 1.0], [7.0, -1.0, 0.0, -3.0, -4.0], [6.0, 7.0, -4.0, -6.0, 0.0], [6.0, 5.0, 0.0, 7.0, 7.0]], [[-2.0, 0.0, 0.0, -2.0, -7.0], [-5.0, 0.0, -4.0, 6.0, -2.0], [6.0, 0.0, -1.0, -6.0, -1.0], [4.0, -7.0, -2.0, 5.0, 4.0]], [[4.0, 0.0, -3.0, 7.0, 1.0], [5.0, 6.0, -2.0, 3.0, 1.0], [5.0, 3.0, 4.0, 0.0, -3.0], [-5.0, 1.0, 6.0, -7.0, -6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, -1.0, 0.0], [-5.0, 5.0, 6.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 0.0, -3.0], [-2.0, -1.0, 3.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[33.0, 2.0, -25.0, 43.0, -35.0], [-34.0, 29.0, -21.0, 68.0, 7.0], [25.0, -17.0, 22.0, -15.0, -18.0], [-33.0, -58.0, 1.0, -46.0, -74.0]])+(1.-msk_ref)*numpy.array([[32.0, -6.0, -31.0, -9.0, 23.0], [29.0, 39.0, -47.0, -15.0, 8.0], [4.0, 8.0, 41.0, 35.0, -6.0], [-14.0, 24.0, 59.0, -56.0, 1.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[-1.0, 3.0, 2.0, 2.0, -5.0], [-6.0, -7.0, 0.0, 0.0, -7.0], [2.0, -2.0, 7.0, 0.0, 6.0], [0.0, 5.0, 4.0, 6.0, 2.0]], [[5.0, -5.0, 0.0, -2.0, 0.0], [-6.0, -6.0, -7.0, 0.0, -1.0], [6.0, -5.0, 1.0, -5.0, 5.0], [1.0, 1.0, -7.0, -3.0, 1.0]], [[-6.0, 0.0, 5.0, 0.0, -4.0], [-3.0, 4.0, -7.0, 6.0, 1.0], [0.0, -7.0, -4.0, 1.0, -4.0], [0.0, 2.0, 1.0, -2.0, 3.0]]], [[[-7.0, 3.0, 7.0, 7.0, 1.0], [-7.0, -3.0, -4.0, 6.0, -2.0], [0.0, -7.0, 7.0, -3.0, -7.0], [7.0, -7.0, -1.0, 0.0, 5.0]], [[0.0, 3.0, 5.0, 4.0, -1.0], [-1.0, 7.0, 2.0, 4.0, -5.0], [-2.0, 3.0, 4.0, 1.0, -2.0], [-2.0, -3.0, 0.0, 6.0, 3.0]], [[-5.0, -5.0, -7.0, 5.0, 2.0], [-6.0, 2.0, 6.0, 2.0, -5.0], [0.0, 1.0, 4.0, -3.0, -2.0], [-7.0, -6.0, 0.0, 1.0, 7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, -4.0], [-7.0, -7.0], [-2.0, -5.0]], [[-6.0, 0.0], [-7.0, 1.0], [-6.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, -4.0], [0.0, -5.0], [6.0, 0.0]], [[-1.0, -5.0], [-7.0, -7.0], [-1.0, 0.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[46.0, -1.0], [35.0, 26.0], [-39.0, -28.0], [-80.0, 10.0], [-18.0, 39.0]], [[115.0, 80.0], [-30.0, 57.0], [37.0, 86.0], [-88.0, -26.0], [61.0, 25.0]], [[-22.0, -52.0], [58.0, 81.0], [-72.0, -11.0], [62.0, 31.0], [59.0, -41.0]], [[7.0, -9.0], [103.0, -40.0], [65.0, 28.0], [-5.0, 13.0], [-100.0, -27.0]]])+(1.-msk_ref)*numpy.array([[[-27.0, 14.0], [-10.0, -23.0], [1.0, -78.0], [-34.0, -61.0], [-35.0, 22.0]], [[-16.0, 96.0], [-45.0, 24.0], [-58.0, 41.0], [0.0, -58.0], [27.0, 78.0]], [[20.0, -24.0], [-63.0, 47.0], [-42.0, -96.0], [5.0, 33.0], [17.0, 0.0]], [[14.0, -26.0], [61.0, 31.0], [19.0, 24.0], [-37.0, -51.0], [-9.0, -59.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[0.0, 5.0, 1.0, -4.0, -7.0], [7.0, 4.0, -3.0, -2.0, 4.0], [3.0, -2.0, 6.0, 7.0, -4.0], [-7.0, 3.0, -1.0, 4.0, -5.0]], [[-5.0, 0.0, -5.0, 4.0, -6.0], [-6.0, 7.0, 3.0, -5.0, -5.0], [-1.0, 6.0, 1.0, -6.0, -2.0], [0.0, -7.0, -1.0, 5.0, 4.0]], [[-3.0, -1.0, 0.0, 4.0, 5.0], [0.0, -3.0, 1.0, 3.0, 4.0], [3.0, -6.0, 4.0, 6.0, -4.0], [5.0, 0.0, -5.0, -7.0, 0.0]]], [[[0.0, -6.0, -5.0, 4.0, 6.0], [0.0, 0.0, -5.0, 4.0, 6.0], [-6.0, -4.0, -5.0, 6.0, -5.0], [6.0, -2.0, 2.0, 6.0, 0.0]], [[5.0, 0.0, -2.0, 4.0, 5.0], [7.0, 4.0, -3.0, -2.0, -6.0], [-6.0, 2.0, -4.0, -1.0, 5.0], [-5.0, -1.0, -4.0, 4.0, 6.0]], [[0.0, -2.0, -3.0, 5.0, -7.0], [5.0, -2.0, -7.0, 3.0, 0.0], [-5.0, 4.0, -4.0, -5.0, 3.0], [0.0, 6.0, -2.0, 1.0, 4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, 3.0, 2.0, -7.0, -6.0], [7.0, -6.0, 1.0, 5.0, 4.0], [0.0, -4.0, 2.0, -3.0, -5.0], [-1.0, -1.0, 5.0, -3.0, -3.0]], [[5.0, -5.0, 3.0, -7.0, -5.0], [0.0, 4.0, 2.0, 5.0, -2.0], [5.0, -6.0, -4.0, 1.0, 0.0], [3.0, 3.0, 6.0, 2.0, 7.0]], [[0.0, -2.0, 6.0, 1.0, 5.0], [3.0, 5.0, 6.0, 0.0, 3.0], [6.0, -7.0, -4.0, -2.0, -4.0], [5.0, 0.0, -5.0, 2.0, 5.0]]], [[[5.0, -2.0, -3.0, 6.0, -2.0], [3.0, -1.0, -5.0, 5.0, -2.0], [5.0, 3.0, -5.0, 6.0, 2.0], [-2.0, 2.0, -3.0, 3.0, -3.0]], [[-2.0, 0.0, 3.0, 5.0, -3.0], [2.0, 0.0, 6.0, -7.0, -4.0], [7.0, 6.0, -1.0, -6.0, -1.0], [1.0, 1.0, -5.0, -4.0, 5.0]], [[6.0, 5.0, 6.0, -2.0, 4.0], [-2.0, 7.0, 5.0, 6.0, -4.0], [4.0, 1.0, 0.0, -5.0, 1.0], [-7.0, -5.0, 0.0, 0.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, -5.0, 1.0, -1.0, -3.0], [2.0, -7.0, 4.0, -6.0, -6.0], [-3.0, -4.0, 0.0, 7.0, 1.0], [0.0, -6.0, 7.0, 0.0, 1.0]], [[1.0, 0.0, -6.0, 5.0, 6.0], [-6.0, -7.0, 3.0, -4.0, 5.0], [-4.0, -2.0, -7.0, 3.0, 1.0], [-4.0, -7.0, -7.0, 0.0, -4.0]], [[0.0, -5.0, -4.0, -1.0, 6.0], [0.0, -7.0, 4.0, 5.0, 3.0], [7.0, -7.0, 3.0, -7.0, -4.0], [-7.0, 2.0, -5.0, 3.0, 7.0]]], [[[6.0, 5.0, -4.0, 0.0, 3.0], [2.0, -6.0, 1.0, -1.0, -3.0], [-1.0, -5.0, 3.0, -4.0, 0.0], [1.0, -5.0, -7.0, 0.0, -4.0]], [[-6.0, -2.0, 2.0, -4.0, 0.0], [-3.0, -4.0, -6.0, -6.0, 5.0], [5.0, -4.0, 4.0, 1.0, 5.0], [2.0, -2.0, -3.0, 6.0, 0.0]], [[-2.0, 0.0, 6.0, 7.0, -1.0], [2.0, 7.0, 0.0, -1.0, 4.0], [-2.0, 1.0, 2.0, 2.0, -2.0], [0.0, -7.0, -6.0, 2.0, 5.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-35.0, 31.0, -18.0, 57.0, -5.0], [1.0, -35.0, 2.0, -60.0, -19.0], [-8.0, 81.0, 27.0, -29.0, 7.0], [-25.0, -10.0, -40.0, -36.0, -25.0]], [[-32.0, 19.0, 10.0, -68.0, -31.0], [18.0, -43.0, 19.0, -17.0, 37.0], [-44.0, -33.0, 44.0, -39.0, -35.0], [16.0, -7.0, 48.0, -35.0, -2.0]], [[-62.0, 23.0, -22.0, -6.0, 23.0], [-6.0, -42.0, -11.0, -49.0, 44.0], [-76.0, -4.0, 49.0, -11.0, -16.0], [13.0, -13.0, 0.0, -20.0, -33.0]], [[54.0, -23.0, 58.0, 38.0, 24.0], [-6.0, 91.0, 57.0, 22.0, -56.0], [112.0, 5.0, -64.0, -17.0, 13.0], [-3.0, 3.0, -48.0, 24.0, 68.0]], [[-66.0, -48.0, -47.0, 171.0, 42.0], [8.0, -12.0, -24.0, -112.0, -5.0], [37.0, 70.0, -45.0, 46.0, 15.0], [56.0, 41.0, -139.0, 17.0, 11.0]]], [[[0.0, 76.0, 47.0, 18.0, -13.0], [53.0, -31.0, 62.0, -14.0, -8.0], [39.0, 55.0, 31.0, -94.0, -37.0], [-53.0, -43.0, -36.0, -61.0, -28.0]], [[23.0, -27.0, 11.0, -56.0, -94.0], [31.0, -25.0, 14.0, 15.0, -15.0], [37.0, -15.0, -12.0, -13.0, -14.0], [20.0, 31.0, 57.0, -20.0, 42.0]], [[-52.0, -51.0, -27.0, -30.0, -1.0], [-25.0, -9.0, -19.0, -46.0, 35.0], [-53.0, -53.0, 6.0, 33.0, -3.0], [73.0, 34.0, 28.0, 14.0, 35.0]], [[13.0, 20.0, -1.0, 60.0, 62.0], [-3.0, 24.0, -11.0, 17.0, -1.0], [11.0, 20.0, -14.0, 16.0, 11.0], [-29.0, -22.0, -57.0, 22.0, -36.0]], [[25.0, 17.0, -19.0, 17.0, 27.0], [46.0, -30.0, -48.0, 67.0, 50.0], [-13.0, -32.0, -12.0, 47.0, -18.0], [-17.0, -13.0, -18.0, 28.0, -75.0]]], [[[-47.0, -5.0, -9.0, -67.0, 12.0], [10.0, -36.0, -12.0, -8.0, 79.0], [-79.0, -86.0, 34.0, 9.0, -38.0], [50.0, 1.0, 42.0, 1.0, -13.0]], [[26.0, 4.0, 20.0, -56.0, -30.0], [-48.0, 38.0, 26.0, 10.0, -54.0], [4.0, 18.0, 14.0, -32.0, 28.0], [-28.0, -6.0, 58.0, -14.0, 40.0]], [[-24.0, -5.0, 18.0, -87.0, -15.0], [39.0, -35.0, 13.0, 14.0, 76.0], [-40.0, -101.0, 21.0, -11.0, -56.0], [51.0, 3.0, 51.0, -7.0, 4.0]], [[-14.0, 2.0, -19.0, 40.0, -11.0], [93.0, -77.0, -30.0, 12.0, 70.0], [9.0, -27.0, -15.0, 28.0, -51.0], [27.0, 11.0, -44.0, 1.0, -56.0]], [[-35.0, 31.0, 10.0, 27.0, 21.0], [-51.0, 22.0, 38.0, -72.0, -46.0], [-12.0, 74.0, 36.0, -57.0, 24.0], [-28.0, -22.0, -22.0, -35.0, 18.0]]], [[[26.0, -43.0, -17.0, 65.0, 70.0], [-26.0, 61.0, -37.0, 30.0, -5.0], [25.0, -19.0, -59.0, 77.0, 32.0], [15.0, 14.0, -53.0, 69.0, 3.0]], [[-1.0, 78.0, 24.0, -1.0, 48.0], [1.0, -2.0, 23.0, 13.0, 10.0], [-28.0, 24.0, 45.0, -52.0, -12.0], [-63.0, -59.0, -16.0, -25.0, -57.0]], [[-1.0, -2.0, -65.0, 5.0, -14.0], [-20.0, -39.0, -77.0, 16.0, 3.0], [-61.0, 25.0, 16.0, 58.0, 31.0], [-21.0, 8.0, 28.0, 13.0, -55.0]], [[61.0, -6.0, -19.0, -16.0, -104.0], [31.0, -38.0, -29.0, 53.0, -47.0], [45.0, 46.0, -18.0, 14.0, 17.0], [-39.0, 22.0, 47.0, -14.0, -10.0]], [[22.0, -15.0, 44.0, 29.0, 8.0], [-31.0, 74.0, 59.0, -23.0, -68.0], [78.0, 36.0, -32.0, -37.0, 23.0], [-5.0, 3.0, -31.0, -1.0, 73.0]]]])+(1.-msk_ref)*numpy.array([[[[-35.0, 5.0, 52.0, -42.0, -48.0], [15.0, 36.0, -57.0, -25.0, -9.0], [24.0, 11.0, 46.0, 11.0, 32.0], [51.0, 19.0, 35.0, 21.0, -1.0]], [[3.0, -50.0, 21.0, -18.0, -37.0], [-6.0, -6.0, 10.0, -27.0, -23.0], [-12.0, 15.0, -25.0, 62.0, 13.0], [1.0, 12.0, 94.0, -7.0, 12.0]], [[-10.0, -26.0, 29.0, -39.0, -45.0], [22.0, 45.0, -4.0, 34.0, -38.0], [18.0, 36.0, 6.0, 4.0, -8.0], [11.0, 79.0, 101.0, -18.0, 26.0]], [[-34.0, 12.0, -22.0, 39.0, 67.0], [-26.0, -33.0, -8.0, -5.0, 84.0], [30.0, -51.0, 22.0, -46.0, -6.0], [-32.0, -59.0, -146.0, 46.0, 17.0]], [[-35.0, 30.0, -47.0, -97.0, 40.0], [5.0, -49.0, -50.0, 62.0, 6.0], [113.0, -52.0, 81.0, -135.0, 6.0], [5.0, 103.0, -47.0, 31.0, -7.0]]], [[[-9.0, -49.0, 87.0, -30.0, -62.0], [39.0, 0.0, -32.0, -65.0, -17.0], [28.0, -39.0, 80.0, 48.0, 26.0], [38.0, -49.0, 40.0, 52.0, 56.0]], [[15.0, -13.0, -30.0, 4.0, 14.0], [-50.0, -86.0, 1.0, -89.0, 14.0], [-37.0, -27.0, -46.0, 70.0, 47.0], [1.0, -73.0, -6.0, 11.0, -55.0]], [[-16.0, -9.0, -53.0, -20.0, 25.0], [-39.0, -14.0, 14.0, 41.0, 8.0], [8.0, 29.0, -59.0, -16.0, -5.0], [-30.0, 79.0, 39.0, -29.0, -23.0]], [[11.0, 19.0, 14.0, 3.0, 3.0], [46.0, 33.0, 5.0, 52.0, -14.0], [27.0, -12.0, 54.0, -62.0, -35.0], [-1.0, 16.0, -34.0, 3.0, 38.0]], [[95.0, 2.0, -18.0, -9.0, 0.0], [68.0, -33.0, 59.0, 46.0, -85.0], [0.0, -40.0, 41.0, -45.0, -47.0], [-14.0, 1.0, 19.0, -24.0, 28.0]]], [[[30.0, -48.0, -21.0, -22.0, -10.0], [8.0, -10.0, 51.0, 48.0, -46.0], [2.0, 18.0, -36.0, 5.0, -30.0], [-35.0, 72.0, 103.0, -37.0, 27.0]], [[-52.0, 16.0, 30.0, 58.0, -10.0], [-46.0, 58.0, -30.0, -54.0, 62.0], [-54.0, 54.0, -56.0, 72.0, 30.0], [18.0, -54.0, -28.0, 2.0, -32.0]], [[45.0, -67.0, -28.0, -17.0, 1.0], [0.0, -59.0, 62.0, 13.0, -40.0], [-1.0, -17.0, -34.0, 25.0, -21.0], [-45.0, 26.0, 86.0, -20.0, 30.0]], [[95.0, -33.0, -37.0, -74.0, 2.0], [55.0, -116.0, 46.0, 17.0, -97.0], [44.0, -89.0, 64.0, -46.0, -18.0], [-14.0, 19.0, 52.0, 2.0, 24.0]], [[-96.0, 5.0, 72.0, -1.0, -42.0], [-15.0, 101.0, -73.0, -16.0, 54.0], [16.0, 56.0, 13.0, 25.0, 29.0], [41.0, 24.0, 8.0, 24.0, 11.0]]], [[[17.0, 50.0, -61.0, 22.0, 69.0], [13.0, -2.0, 28.0, 91.0, 14.0], [25.0, -17.0, 13.0, -113.0, -52.0], [-39.0, 32.0, -101.0, -15.0, 4.0]], [[-4.0, -23.0, 87.0, 8.0, -63.0], [59.0, 86.0, -5.0, 12.0, -28.0], [4.0, 22.0, 51.0, 19.0, -21.0], [24.0, 1.0, 51.0, 6.0, 69.0]], [[32.0, 48.0, -3.0, 3.0, -25.0], [16.0, 39.0, -1.0, 9.0, -48.0], [-46.0, 45.0, -22.0, 9.0, 2.0], [33.0, 15.0, 35.0, -43.0, -50.0]], [[43.0, 37.0, -8.0, 19.0, -7.0], [-20.0, -59.0, -15.0, -110.0, -14.0], [-69.0, -22.0, -20.0, 74.0, 55.0], [43.0, -118.0, -32.0, 5.0, -84.0]], [[-75.0, 13.0, 7.0, 29.0, 35.0], [-44.0, 11.0, -44.0, -26.0, 96.0], [21.0, -8.0, 4.0, -9.0, 21.0], [-4.0, -38.0, -105.0, 44.0, -1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([5.0, 0.0, -7.0])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[1.0, 3.0], [6.0, 7.0], [1.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-2.0, 15.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([[7.0, -2.0], [-5.0, -4.0], [-7.0, 4.0]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[4.0, -4.0], [-7.0, 3.0], [-5.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[98.0, 0.0], [-15.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3,))
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[-2.0, 0.0], [3.0, -1.0], [-7.0, 0.0]])
      s1=numpy.array([-1.0, 4.0, 3.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-7.0, -4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[6.0, -2.0], [7.0, -6.0], [4.0, -2.0]])
      s1=numpy.array([[3.0, 4.0], [2.0, -2.0], [6.0, 1.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[56.0, 14.0], [-30.0, 2.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([2.0, -3.0, 0.0]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[0.0, 3.0], [1.0, 3.0], [7.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-3.0, -3.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([[-6.0, 3.0], [-4.0, -5.0], [-3.0, -5.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[0.0, 2.0], [3.0, 5.0], [1.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-15.0, -20.0], [-29.0, -14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, 0.0, 0.0])+(1.-msk_arg1)*numpy.array([-3.0, 3.0, -1.0])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[4.0, -7.0], [-3.0, -5.0], [-4.0, -3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-4.0, 7.0])+(1.-msk_ref)*numpy.array([-17.0, 9.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 3.0], [-6.0, -4.0], [4.0, 1.0]])+(1.-msk_arg1)*numpy.array([[0.0, 3.0], [-1.0, -5.0], [-4.0, 2.0]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[-1.0, -4.0], [7.0, -2.0], [-3.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-50.0, -34.0], [32.0, -3.0]])+(1.-msk_ref)*numpy.array([[5.0, -44.0], [-2.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[7.0, 0.0, 3.0], [2.0, 0.0, -5.0]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[2.0, 4.0, 1.0, -4.0, -7.0], [-3.0, -1.0, -7.0, -4.0, -2.0], [7.0, -7.0, 6.0, -7.0, 1.0], [6.0, 7.0, -6.0, 1.0, 0.0]], [[-2.0, -6.0, -3.0, -1.0, 1.0], [-2.0, -5.0, -5.0, -3.0, 4.0], [1.0, 5.0, 3.0, -3.0, 0.0], [5.0, 6.0, 0.0, 7.0, 3.0]], [[7.0, -5.0, -7.0, -7.0, 1.0], [1.0, 0.0, 5.0, 0.0, -3.0], [-4.0, -6.0, -6.0, -4.0, 5.0], [0.0, 4.0, 2.0, -7.0, 1.0]]], [[[3.0, -5.0, -4.0, 1.0, 0.0], [-4.0, -3.0, -3.0, 0.0, 0.0], [-3.0, -1.0, 2.0, 7.0, 1.0], [0.0, -2.0, -6.0, -1.0, 6.0]], [[5.0, -6.0, 0.0, 2.0, 3.0], [-6.0, 3.0, 0.0, -2.0, 6.0], [-4.0, -2.0, -3.0, -2.0, 1.0], [2.0, -2.0, -7.0, -2.0, 6.0]], [[-7.0, 0.0, 2.0, -1.0, 7.0], [-3.0, 2.0, -6.0, 7.0, -7.0], [-1.0, 0.0, -6.0, 5.0, 3.0], [-3.0, 3.0, 6.0, -4.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[76.0, 3.0, -32.0, -42.0, -81.0], [-11.0, -23.0, -10.0, -63.0, 12.0], [36.0, -69.0, 58.0, -72.0, 9.0], [57.0, 42.0, -78.0, 4.0, -15.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[[1.0, -4.0], [1.0, -2.0], [0.0, 6.0]], [[-3.0, 0.0], [-7.0, -2.0], [7.0, 0.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[1.0, 6.0, -2.0, -3.0, 2.0], [-5.0, -6.0, 0.0, 1.0, -4.0], [5.0, 4.0, 0.0, 4.0, -2.0], [5.0, 7.0, 0.0, 0.0, 7.0]], [[3.0, -6.0, -1.0, 7.0, -7.0], [0.0, 4.0, 7.0, -5.0, -5.0], [6.0, 5.0, -4.0, -1.0, 3.0], [-6.0, 1.0, 0.0, 7.0, -4.0]], [[7.0, -3.0, 6.0, -1.0, -6.0], [0.0, -3.0, -6.0, -2.0, -4.0], [0.0, 5.0, -7.0, -2.0, 2.0], [-3.0, 6.0, 2.0, 1.0, -4.0]]], [[[-7.0, 5.0, -3.0, -7.0, 6.0], [4.0, -3.0, -4.0, 1.0, -3.0], [-7.0, 0.0, -1.0, 5.0, 6.0], [3.0, -2.0, 1.0, -1.0, 3.0]], [[7.0, -3.0, 7.0, 4.0, 3.0], [-1.0, -4.0, 1.0, -7.0, -6.0], [0.0, -1.0, 1.0, -2.0, -5.0], [3.0, 0.0, -4.0, 6.0, 0.0]], [[6.0, 5.0, 7.0, 7.0, -4.0], [6.0, 0.0, 0.0, -5.0, 5.0], [-1.0, 2.0, 6.0, -3.0, 6.0], [6.0, 0.0, 4.0, 4.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[18.0, 18.0], [41.0, -24.0], [6.0, 32.0], [46.0, -16.0], [-72.0, -36.0]], [[32.0, 22.0], [35.0, 6.0], [12.0, -52.0], [7.0, 8.0], [77.0, 14.0]], [[25.0, -32.0], [30.0, 6.0], [34.0, -36.0], [-19.0, -22.0], [60.0, 24.0]], [[11.0, -32.0], [14.0, 6.0], [53.0, 20.0], [-4.0, -20.0], [1.0, -44.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[[[-2.0, -4.0, -1.0, 1.0, -2.0], [-7.0, 0.0, -2.0, -5.0, 0.0], [0.0, -7.0, 2.0, -6.0, 0.0], [3.0, -5.0, -7.0, -7.0, -3.0]], [[5.0, 7.0, 1.0, 0.0, 0.0], [2.0, -4.0, 6.0, 7.0, 0.0], [0.0, 4.0, -5.0, 6.0, -5.0], [1.0, 0.0, 2.0, 0.0, 7.0]], [[5.0, 4.0, 0.0, 1.0, 2.0], [6.0, 5.0, 0.0, -6.0, 1.0], [0.0, 2.0, 3.0, -3.0, 0.0], [-1.0, 2.0, -3.0, 7.0, 1.0]]], [[[2.0, 7.0, 2.0, 5.0, 2.0], [4.0, -6.0, 2.0, -1.0, -5.0], [7.0, 6.0, 4.0, 2.0, -5.0], [1.0, 6.0, -5.0, 5.0, -4.0]], [[6.0, -7.0, 0.0, -5.0, 1.0], [0.0, -3.0, 1.0, -2.0, 0.0], [6.0, 6.0, 0.0, 6.0, 4.0], [5.0, 0.0, 6.0, 3.0, -7.0]], [[2.0, 3.0, 2.0, 5.0, 4.0], [-3.0, 4.0, 0.0, -7.0, 7.0], [7.0, 4.0, 2.0, -7.0, 3.0], [2.0, -3.0, 5.0, -5.0, 0.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[5.0, 2.0, 2.0, 2.0, -1.0], [0.0, -7.0, -6.0, 3.0, 6.0], [5.0, 6.0, 0.0, -5.0, -7.0], [3.0, -1.0, -1.0, -3.0, 0.0]], [[4.0, -1.0, -3.0, -5.0, 0.0], [3.0, 6.0, 6.0, 5.0, 0.0], [-5.0, 5.0, 0.0, 3.0, 3.0], [-1.0, 3.0, 3.0, 0.0, -7.0]], [[-2.0, 7.0, -5.0, 0.0, -7.0], [3.0, -2.0, 6.0, 7.0, -7.0], [-2.0, 6.0, 2.0, -1.0, -3.0], [7.0, -1.0, 1.0, 2.0, 0.0]]], [[[4.0, -3.0, 2.0, 7.0, -7.0], [4.0, -3.0, -3.0, 6.0, 1.0], [5.0, -2.0, 6.0, 5.0, -1.0], [7.0, 5.0, 0.0, -1.0, -1.0]], [[0.0, -7.0, 0.0, -1.0, 7.0], [3.0, -4.0, -2.0, 0.0, 5.0], [4.0, -4.0, -2.0, 5.0, 6.0], [0.0, -1.0, 5.0, -1.0, 3.0]], [[-2.0, -7.0, 3.0, -3.0, 0.0], [5.0, 7.0, -6.0, -1.0, -6.0], [6.0, 3.0, -2.0, 2.0, -3.0], [-4.0, 6.0, -1.0, 1.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[4.0, 22.0, 3.0, 13.0, -14.0], [-17.0, -58.0, 22.0, 25.0, -36.0], [14.0, -7.0, -4.0, 22.0, -46.0], [21.0, 1.0, -51.0, -19.0, -5.0]], [[-36.0, 20.0, -23.0, -6.0, -31.0], [35.0, 50.0, -23.0, 7.0, -27.0], [-112.0, -92.0, 4.0, -38.0, -29.0], [-54.0, 7.0, -99.0, 34.0, 55.0]], [[-34.0, -26.0, 5.0, 22.0, 2.0], [-51.0, -13.0, -18.0, -24.0, 6.0], [35.0, -12.0, 18.0, -32.0, 14.0], [16.0, -17.0, 0.0, -54.0, -40.0]], [[-27.0, 4.0, 1.0, 27.0, -3.0], [13.0, -31.0, -21.0, -29.0, -56.0], [22.0, -10.0, 51.0, -13.0, -23.0], [-3.0, 41.0, -80.0, 33.0, -62.0]], [[-5.0, -122.0, -13.0, -78.0, -19.0], [-63.0, -14.0, -5.0, 40.0, 28.0], [-7.0, -7.0, -51.0, 55.0, 63.0], [32.0, -51.0, 105.0, -56.0, -25.0]]], [[[66.0, 55.0, 21.0, 33.0, 37.0], [25.0, -10.0, 29.0, -42.0, 18.0], [81.0, 80.0, 20.0, 0.0, -8.0], [29.0, 15.0, 20.0, 25.0, -13.0]], [[18.0, 90.0, 21.0, 31.0, 28.0], [16.0, 24.0, 40.0, 51.0, 62.0], [4.0, 55.0, -48.0, 5.0, -10.0], [-22.0, -8.0, 93.0, -27.0, 101.0]], [[42.0, 65.0, -6.0, -35.0, -8.0], [96.0, 6.0, 40.0, 85.0, -21.0], [-75.0, 24.0, -48.0, 78.0, -41.0], [-43.0, 42.0, 9.0, 93.0, 92.0]], [[64.0, 90.0, 12.0, 35.0, 16.0], [58.0, -25.0, 36.0, -21.0, -30.0], [35.0, 45.0, 24.0, 10.0, -58.0], [11.0, 38.0, -67.0, 63.0, 9.0]], [[-27.0, -98.0, -16.0, -51.0, -43.0], [-62.0, -80.0, -5.0, 43.0, -54.0], [-5.0, -44.0, -17.0, 59.0, -3.0], [39.0, -20.0, -26.0, -41.0, -64.0]]], [[[1.0, -38.0, 12.0, 38.0, 24.0], [-55.0, -8.0, -26.0, -103.0, 15.0], [101.0, 19.0, 61.0, -62.0, 34.0], [49.0, -17.0, -10.0, -42.0, -100.0]], [[21.0, 58.0, 1.0, 37.0, 4.0], [-13.0, 46.0, 10.0, -42.0, 37.0], [-17.0, -34.0, 3.0, -73.0, -22.0], [1.0, -39.0, -49.0, -37.0, 59.0]], [[6.0, 58.0, 8.0, 32.0, 6.0], [42.0, -28.0, 10.0, 0.0, -42.0], [16.0, 20.0, 26.0, 8.0, -44.0], [-10.0, 46.0, -58.0, 48.0, -8.0]], [[64.0, 43.0, 22.0, 4.0, 31.0], [49.0, -54.0, 43.0, 23.0, -12.0], [79.0, 113.0, -4.0, 77.0, -14.0], [23.0, 47.0, 59.0, 58.0, -20.0]], [[42.0, -21.0, 2.0, -60.0, 0.0], [42.0, -51.0, 36.0, 84.0, -19.0], [8.0, 73.0, -48.0, 124.0, 5.0], [8.0, 32.0, 90.0, 56.0, 1.0]]], [[[30.0, 46.0, 2.0, 25.0, 6.0], [59.0, -19.0, 2.0, -43.0, -56.0], [21.0, 15.0, 52.0, -3.0, -42.0], [0.0, 53.0, -99.0, 83.0, -37.0]], [[28.0, 81.0, 26.0, 58.0, 33.0], [9.0, -20.0, 29.0, -13.0, 16.0], [71.0, 65.0, 12.0, -11.0, -26.0], [13.0, 15.0, 15.0, -8.0, 10.0]], [[50.0, -9.0, 2.0, -30.0, 5.0], [22.0, -26.0, 25.0, 17.0, -6.0], [23.0, 47.0, -16.0, 58.0, 2.0], [22.0, 10.0, 35.0, 34.0, -10.0]], [[10.0, 23.0, 3.0, 4.0, 11.0], [26.0, 23.0, 3.0, -1.0, 14.0], [-6.0, 17.0, -2.0, -3.0, 4.0], [-15.0, 10.0, 19.0, 22.0, 22.0]], [[-33.0, -98.0, -23.0, -55.0, -27.0], [3.0, -3.0, -41.0, -5.0, -44.0], [-38.0, -44.0, 17.0, 23.0, 31.0], [-7.0, 15.0, -26.0, 39.0, -66.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3))
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[3.0, 1.0, -3.0, 1.0, -3.0], [4.0, -6.0, -6.0, 5.0, 1.0], [-5.0, 4.0, 4.0, 3.0, -1.0], [3.0, 6.0, 6.0, -5.0, -1.0]], [[0.0, 5.0, 0.0, -3.0, 5.0], [-3.0, 6.0, 6.0, -6.0, -7.0], [6.0, 0.0, 5.0, 1.0, 5.0], [-3.0, 6.0, -1.0, 6.0, -7.0]], [[1.0, 6.0, 5.0, 3.0, -7.0], [3.0, -1.0, -3.0, 6.0, 4.0], [7.0, 0.0, 1.0, 0.0, 2.0], [0.0, 5.0, 5.0, 4.0, -1.0]]], [[[-4.0, -4.0, -7.0, 6.0, -2.0], [7.0, 4.0, -5.0, 2.0, -4.0], [-1.0, 7.0, -6.0, 1.0, 3.0], [-2.0, 6.0, -3.0, 6.0, 2.0]], [[-7.0, -7.0, 7.0, 4.0, -5.0], [7.0, 0.0, -7.0, -5.0, 7.0], [-5.0, -2.0, 3.0, -7.0, 0.0], [-4.0, 0.0, 0.0, 0.0, -2.0]], [[6.0, 7.0, -7.0, 3.0, -4.0], [-5.0, -6.0, -1.0, -4.0, 0.0], [-1.0, 2.0, -5.0, 6.0, -1.0], [0.0, 4.0, -1.0, -3.0, -5.0]]]])
      s1=numpy.array([[-2.0, 4.0, 0.0], [6.0, 4.0, 4.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-34.0, -6.0, -36.0, 50.0, -22.0], [30.0, 36.0, -26.0, -58.0, -26.0], [4.0, 34.0, -32.0, 0.0, 36.0], [-46.0, 64.0, -38.0, 58.0, -42.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[7.0, 7.0, -2.0, 4.0, 4.0], [0.0, -6.0, 6.0, 1.0, -5.0], [-4.0, 0.0, 0.0, 0.0, -1.0], [6.0, 0.0, 7.0, -6.0, 1.0]], [[2.0, -3.0, -5.0, 7.0, 5.0], [-3.0, -4.0, -1.0, -7.0, 0.0], [-4.0, 3.0, 2.0, 2.0, 1.0], [-3.0, -3.0, 6.0, 3.0, 5.0]], [[3.0, 1.0, 3.0, -5.0, 3.0], [-1.0, -5.0, 0.0, -4.0, -4.0], [-4.0, 5.0, 4.0, 4.0, 1.0], [-3.0, 5.0, 6.0, -2.0, 0.0]]], [[[6.0, 1.0, -4.0, -4.0, -7.0], [3.0, -3.0, 6.0, 6.0, 2.0], [3.0, 0.0, 2.0, -3.0, -6.0], [-6.0, 5.0, -6.0, -2.0, 5.0]], [[7.0, 5.0, -2.0, 3.0, 3.0], [0.0, -7.0, -2.0, -1.0, 0.0], [-5.0, -4.0, 5.0, 7.0, 1.0], [5.0, 1.0, 0.0, 2.0, -5.0]], [[1.0, 1.0, -1.0, 1.0, -4.0], [0.0, 5.0, 5.0, -2.0, 6.0], [7.0, 0.0, 2.0, 4.0, 0.0], [-2.0, -5.0, 1.0, 7.0, 0.0]]]])
      s1=numpy.array([[[-3.0, -1.0], [5.0, 0.0], [-4.0, -6.0]], [[-6.0, 0.0], [4.0, 4.0], [-2.0, 2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-33.0, 5.0], [-28.0, 9.0], [-13.0, -26.0], [77.0, 40.0], [63.0, -18.0]], [[-29.0, 6.0], [-2.0, 18.0], [-77.0, -4.0], [-58.0, 15.0], [7.0, 41.0]], [[-44.0, 22.0], [-21.0, -46.0], [-2.0, 0.0], [32.0, 12.0], [44.0, -1.0]], [[39.0, 28.0], [-51.0, -36.0], [19.0, -41.0], [47.0, 40.0], [-28.0, -21.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[4.0, 0.0, 1.0, 2.0, 3.0], [0.0, -6.0, 2.0, -3.0, 1.0], [1.0, 5.0, 3.0, -7.0, -1.0], [1.0, -3.0, 2.0, -3.0, 2.0]], [[1.0, 0.0, 2.0, -7.0, 5.0], [0.0, 7.0, 7.0, 3.0, -4.0], [-7.0, 2.0, 0.0, -6.0, -4.0], [-6.0, -4.0, 3.0, 2.0, -7.0]], [[7.0, 5.0, -3.0, 0.0, -2.0], [-5.0, -4.0, 7.0, -5.0, 1.0], [-2.0, -4.0, -3.0, -4.0, -5.0], [-6.0, -2.0, -3.0, -5.0, 2.0]]], [[[0.0, 7.0, 4.0, 1.0, 4.0], [5.0, 0.0, 0.0, 0.0, -5.0], [-3.0, 2.0, 5.0, -6.0, -7.0], [-1.0, -5.0, 1.0, -2.0, 2.0]], [[4.0, -6.0, 2.0, -1.0, 0.0], [3.0, 7.0, 0.0, -3.0, -3.0], [-5.0, -7.0, 0.0, 2.0, -1.0], [3.0, -4.0, 0.0, -1.0, 0.0]], [[6.0, 7.0, -3.0, -1.0, 5.0], [-3.0, 7.0, 0.0, 0.0, 7.0], [5.0, -3.0, -7.0, -6.0, -3.0], [-6.0, 1.0, 7.0, -6.0, -4.0]]]])
      s1=numpy.array([[[[3.0, 2.0, 3.0, -7.0, 4.0], [-7.0, -2.0, 4.0, 3.0, 3.0], [0.0, 0.0, -1.0, 2.0, 2.0], [-3.0, -6.0, -7.0, -5.0, 5.0]], [[6.0, 4.0, -4.0, 6.0, 1.0], [2.0, -3.0, 1.0, 0.0, 0.0], [-3.0, -7.0, 1.0, 5.0, -1.0], [3.0, 4.0, 6.0, -6.0, -3.0]], [[-7.0, -2.0, 7.0, 7.0, 6.0], [0.0, -7.0, -1.0, 0.0, -5.0], [-5.0, 0.0, 2.0, -1.0, -6.0], [-3.0, 3.0, -5.0, 0.0, -4.0]]], [[[-3.0, -6.0, 5.0, -6.0, 2.0], [-6.0, -6.0, -3.0, 6.0, -5.0], [-3.0, 6.0, 2.0, -2.0, -1.0], [6.0, -4.0, -7.0, 3.0, 5.0]], [[-6.0, 2.0, -4.0, -5.0, 1.0], [-1.0, 5.0, -5.0, 5.0, -6.0], [-6.0, 7.0, 5.0, 0.0, -7.0], [-2.0, 0.0, -7.0, -7.0, -3.0]], [[2.0, 5.0, 0.0, 0.0, 7.0], [-1.0, -6.0, 7.0, -4.0, 3.0], [2.0, 5.0, 7.0, -3.0, -6.0], [-5.0, 6.0, 1.0, 1.0, -4.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-43.0, 36.0, 41.0, 7.0, 105.0], [-36.0, -76.0, 32.0, 8.0, -29.0], [-50.0, 51.0, 73.0, -12.0, -99.0], [-68.0, 37.0, -79.0, -48.0, -47.0]], [[-6.0, -29.0, 94.0, 23.0, 87.0], [-43.0, -149.0, 53.0, -16.0, -3.0], [4.0, 35.0, 43.0, -40.0, -37.0], [4.0, 29.0, -25.0, 70.0, 5.0]], [[6.0, -19.0, -14.0, -50.0, -23.0], [-26.0, 17.0, -34.0, 49.0, -23.0], [-21.0, 9.0, -8.0, 16.0, 18.0], [47.0, -41.0, -25.0, -22.0, 37.0]], [[-35.0, -37.0, 43.0, -57.0, -5.0], [-32.0, 12.0, -4.0, 11.0, 4.0], [22.0, 43.0, -19.0, -30.0, 23.0], [-14.0, -50.0, -57.0, 41.0, 43.0]], [[51.0, 31.0, -5.0, -29.0, 48.0], [-40.0, -61.0, 42.0, 13.0, 14.0], [-7.0, 14.0, 41.0, 10.0, -21.0], [11.0, 10.0, -4.0, -28.0, 8.0]]], [[[-4.0, -29.0, -22.0, -80.0, -38.0], [-30.0, 38.0, -46.0, 57.0, -27.0], [-14.0, 36.0, -6.0, 4.0, 22.0], [54.0, -53.0, -34.0, -9.0, 48.0]], [[24.0, 73.0, -102.0, 21.0, 15.0], [42.0, 12.0, 1.0, -11.0, -19.0], [-29.0, 35.0, 89.0, 6.0, -86.0], [2.0, 94.0, 62.0, -54.0, -84.0]], [[-1.0, 18.0, 27.0, 77.0, 57.0], [0.0, -74.0, 8.0, 6.0, -29.0], [-56.0, -49.0, 19.0, 32.0, -45.0], [-6.0, 37.0, -7.0, -52.0, -39.0]], [[62.0, 10.0, -44.0, 19.0, -42.0], [30.0, 17.0, 11.0, -24.0, 34.0], [34.0, -42.0, -19.0, 14.0, 42.0], [39.0, 15.0, 85.0, 18.0, 5.0]], [[19.0, 43.0, 13.0, 21.0, 42.0], [11.0, -24.0, 78.0, -70.0, 62.0], [54.0, 12.0, 21.0, -30.0, -16.0], [-77.0, 43.0, 27.0, 32.0, -31.0]]], [[[24.0, 11.0, 22.0, -20.0, 9.0], [-3.0, -4.0, 68.0, -60.0, 73.0], [80.0, 21.0, -8.0, -40.0, 29.0], [-51.0, 2.0, 22.0, 68.0, 14.0]], [[85.0, -15.0, 17.0, -28.0, -26.0], [-33.0, -17.0, 34.0, 4.0, 58.0], [44.0, -66.0, -63.0, 29.0, 97.0], [44.0, -60.0, 29.0, 15.0, 78.0]], [[1.0, -53.0, 13.0, -72.0, -45.0], [-44.0, 27.0, -49.0, 67.0, -22.0], [-14.0, -5.0, -48.0, 20.0, 61.0], [65.0, -89.0, -48.0, -7.0, 80.0]], [[-35.0, -20.0, -63.0, 11.0, -110.0], [77.0, 142.0, -64.0, -23.0, -1.0], [32.0, -10.0, -51.0, -10.0, 44.0], [5.0, -6.0, 55.0, 33.0, -13.0]], [[29.0, 17.0, -53.0, -5.0, -74.0], [45.0, 104.0, 2.0, -38.0, 54.0], [58.0, -36.0, -53.0, 6.0, 64.0], [-19.0, -15.0, 61.0, 12.0, 7.0]]], [[[-18.0, -28.0, -32.0, -94.0, -79.0], [-10.0, 115.0, -50.0, 36.0, 2.0], [21.0, 27.0, -48.0, -2.0, 60.0], [15.0, -80.0, -33.0, 1.0, 57.0]], [[22.0, 9.0, -16.0, 33.0, -35.0], [46.0, 36.0, 28.0, -63.0, 53.0], [63.0, -25.0, -28.0, -17.0, 37.0], [-24.0, 22.0, 71.0, 53.0, -12.0]], [[56.0, 51.0, -22.0, -23.0, 44.0], [-21.0, -40.0, 60.0, -16.0, 37.0], [17.0, 20.0, 46.0, -1.0, -24.0], [-17.0, 29.0, 19.0, -18.0, -10.0]], [[38.0, -8.0, -58.0, 15.0, -87.0], [44.0, 78.0, -36.0, -2.0, 14.0], [19.0, -63.0, -56.0, 31.0, 67.0], [50.0, -17.0, 73.0, -2.0, 16.0]], [[-64.0, -60.0, 58.0, -54.0, -11.0], [-36.0, 15.0, -35.0, 34.0, -26.0], [-3.0, 41.0, -29.0, -25.0, 21.0], [-1.0, -66.0, -84.0, 34.0, 49.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[-4.0, -7.0, -6.0], [-7.0, 4.0, -4.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[0.0, 6.0, 2.0, -7.0, 1.0], [-2.0, 0.0, 7.0, 3.0, -6.0], [5.0, -3.0, 0.0, 2.0, -2.0], [-1.0, -3.0, -5.0, 7.0, 0.0]], [[-1.0, 1.0, 7.0, -6.0, 7.0], [3.0, 2.0, -6.0, 2.0, -6.0], [4.0, 2.0, 0.0, -4.0, 0.0], [-1.0, 5.0, 2.0, 0.0, -1.0]], [[5.0, -3.0, -7.0, 1.0, 3.0], [-2.0, 3.0, 6.0, -2.0, -1.0], [7.0, 3.0, -2.0, 1.0, 0.0], [4.0, 0.0, -5.0, 3.0, 6.0]]], [[[6.0, 6.0, -5.0, 2.0, 4.0], [6.0, 6.0, 6.0, 2.0, -1.0], [-6.0, 5.0, 1.0, 5.0, -2.0], [1.0, -5.0, -5.0, -1.0, -7.0]], [[-7.0, 1.0, 4.0, -2.0, 0.0], [-1.0, -5.0, -5.0, -6.0, 1.0], [-3.0, 0.0, 7.0, 1.0, 0.0], [0.0, -7.0, -7.0, -1.0, 4.0]], [[-7.0, -4.0, 5.0, -7.0, 6.0], [-3.0, 7.0, -6.0, -1.0, 1.0], [-1.0, -5.0, 4.0, 0.0, -6.0], [-7.0, 2.0, -4.0, 6.0, -4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-65.0, -35.0, 16.0, 70.0, -123.0], [-35.0, -122.0, -60.0, -48.0, 79.0], [-56.0, -35.0, 17.0, -17.0, 46.0], [8.0, -24.0, 59.0, -67.0, 52.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[[-7.0, 7.0], [1.0, 5.0], [1.0, 4.0]], [[-7.0, 2.0], [2.0, 7.0], [-7.0, 6.0]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-5.0, -2.0, 5.0, -5.0, -4.0], [-2.0, 2.0, -2.0, 0.0, 0.0], [0.0, -2.0, 3.0, 0.0, -6.0], [4.0, -1.0, 6.0, 2.0, -1.0]], [[-5.0, 6.0, -5.0, -3.0, -7.0], [-2.0, 3.0, -1.0, -5.0, 6.0], [0.0, -4.0, 2.0, 0.0, 4.0], [-2.0, 4.0, 4.0, 4.0, 2.0]], [[7.0, 0.0, 2.0, 6.0, 0.0], [0.0, 7.0, -5.0, -2.0, -5.0], [-5.0, 4.0, -6.0, -2.0, 0.0], [0.0, -4.0, 1.0, 2.0, 5.0]]], [[[0.0, 7.0, -6.0, 2.0, -6.0], [3.0, -4.0, 5.0, 3.0, -2.0], [0.0, -7.0, -5.0, 0.0, 2.0], [5.0, 5.0, 5.0, -5.0, 2.0]], [[4.0, 2.0, 5.0, -1.0, 0.0], [-2.0, 6.0, -5.0, 7.0, 7.0], [-3.0, 5.0, 6.0, 0.0, 6.0], [-4.0, 4.0, 2.0, -6.0, -3.0]], [[-1.0, -5.0, -7.0, -6.0, -1.0], [-2.0, -3.0, -6.0, -7.0, -5.0], [2.0, 6.0, 4.0, 2.0, -3.0], [0.0, -3.0, -6.0, 7.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[52.0, -10.0], [10.0, 14.0], [63.0, -1.0], [64.0, -65.0], [70.0, -81.0]], [[1.0, -44.0], [57.0, 73.0], [5.0, -100.0], [35.0, -20.0], [64.0, 25.0]], [[-25.0, -29.0], [31.0, 39.0], [-6.0, 63.0], [-16.0, 4.0], [65.0, 6.0]], [[-73.0, 0.0], [1.0, 17.0], [-26.0, 54.0], [-34.0, 32.0], [29.0, -24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[[[-4.0, -1.0, 4.0, -1.0, 5.0], [0.0, 5.0, 0.0, -5.0, 3.0], [-5.0, -1.0, 2.0, -2.0, -2.0], [-3.0, 7.0, -3.0, 3.0, 2.0]], [[-6.0, 1.0, 0.0, -6.0, -3.0], [1.0, -1.0, 5.0, -1.0, -1.0], [2.0, 0.0, -5.0, 7.0, -3.0], [1.0, 3.0, -6.0, -5.0, -7.0]], [[-2.0, 5.0, -4.0, -2.0, -4.0], [-2.0, 1.0, -1.0, -6.0, -2.0], [0.0, -6.0, -2.0, -5.0, 3.0], [4.0, 7.0, -1.0, 1.0, -1.0]]], [[[-4.0, -6.0, 0.0, -6.0, 0.0], [4.0, 1.0, 7.0, -4.0, -5.0], [1.0, 1.0, 1.0, 0.0, -6.0], [-5.0, -1.0, 2.0, -4.0, -5.0]], [[4.0, 2.0, 3.0, -5.0, 4.0], [6.0, 5.0, 0.0, 1.0, 0.0], [1.0, 0.0, -6.0, 6.0, 2.0], [2.0, 0.0, 5.0, 0.0, 0.0]], [[-5.0, -5.0, -6.0, -1.0, -4.0], [2.0, 0.0, -7.0, -1.0, -2.0], [0.0, -5.0, 3.0, 3.0, 0.0], [0.0, 5.0, 7.0, 3.0, -1.0]]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-5.0, 0.0, -5.0, -6.0, -4.0], [7.0, 0.0, 7.0, 4.0, 3.0], [3.0, -1.0, -3.0, 4.0, -4.0], [3.0, 1.0, -3.0, 4.0, -5.0]], [[6.0, 2.0, 7.0, -3.0, 0.0], [0.0, -7.0, -5.0, -1.0, -3.0], [-5.0, -3.0, 2.0, -5.0, 0.0], [-7.0, 5.0, 7.0, 3.0, 7.0]], [[0.0, 2.0, 5.0, 2.0, -7.0], [-4.0, -2.0, 0.0, 2.0, -4.0], [4.0, 0.0, 5.0, -7.0, -4.0], [-2.0, -2.0, -1.0, 4.0, -4.0]]], [[[5.0, 7.0, -1.0, -4.0, 0.0], [3.0, 4.0, -4.0, -2.0, 0.0], [-7.0, -2.0, -6.0, -2.0, 6.0], [-5.0, 2.0, -5.0, -2.0, 0.0]], [[-3.0, 0.0, 2.0, -2.0, -5.0], [-7.0, -4.0, 5.0, 5.0, 0.0], [-3.0, -7.0, 3.0, 4.0, 5.0], [-6.0, -6.0, 4.0, 1.0, 1.0]], [[0.0, -7.0, 3.0, 1.0, 7.0], [0.0, -2.0, -5.0, 4.0, 0.0], [-7.0, -1.0, 5.0, 2.0, 0.0], [3.0, -5.0, 1.0, -6.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-48.0, -25.0, -29.0, -46.0, -55.0], [8.0, -41.0, 65.0, -4.0, -46.0], [39.0, 10.0, -17.0, 34.0, -44.0], [-10.0, -22.0, -26.0, -65.0, -77.0]], [[-9.0, 5.0, 34.0, -51.0, 14.0], [12.0, 7.0, 106.0, -35.0, -27.0], [11.0, 30.0, -28.0, -17.0, -42.0], [-25.0, -22.0, -49.0, -57.0, -44.0]], [[-35.0, 32.0, -52.0, -54.0, -70.0], [11.0, -18.0, 2.0, -9.0, -33.0], [40.0, -41.0, -59.0, 55.0, 14.0], [51.0, 37.0, -3.0, -32.0, -62.0]], [[41.0, 28.0, -44.0, 53.0, -41.0], [-33.0, -39.0, -52.0, 34.0, -1.0], [18.0, -15.0, 10.0, -28.0, 47.0], [39.0, -28.0, 23.0, 18.0, 26.0]], [[-25.0, -76.0, -45.0, 36.0, -40.0], [-2.0, -52.0, -42.0, 50.0, -12.0], [15.0, 11.0, 57.0, 34.0, -23.0], [-26.0, -42.0, 43.0, 2.0, -8.0]]], [[[-60.0, -59.0, 23.0, 18.0, 23.0], [-22.0, -1.0, 25.0, -30.0, 14.0], [-39.0, 20.0, 67.0, -36.0, -58.0], [-66.0, 18.0, -46.0, 5.0, 3.0]], [[24.0, -39.0, 8.0, 44.0, 21.0], [-15.0, -11.0, 9.0, 1.0, -5.0], [-14.0, 26.0, 61.0, -69.0, -17.0], [-43.0, -49.0, 18.0, 11.0, 33.0]], [[63.0, 47.0, 73.0, 27.0, 90.0], [-1.0, 61.0, -18.0, -4.0, 56.0], [-44.0, 14.0, -10.0, -34.0, 35.0], [4.0, 13.0, -9.0, 47.0, 74.0]], [[-6.0, 7.0, -1.0, -19.0, 19.0], [25.0, 46.0, -49.0, -22.0, 11.0], [-19.0, -38.0, -11.0, 17.0, 23.0], [15.0, 61.0, 41.0, 39.0, 19.0]], [[14.0, -26.0, 28.0, 23.0, 40.0], [5.0, 14.0, -11.0, 12.0, 20.0], [-21.0, 21.0, 29.0, -7.0, -9.0], [-28.0, -16.0, 13.0, 20.0, 31.0]]], [[[61.0, 83.0, 29.0, 83.0, 30.0], [-73.0, 2.0, -29.0, -2.0, 55.0], [-35.0, 1.0, 13.0, -100.0, 57.0], [31.0, 6.0, -61.0, 45.0, 79.0]], [[7.0, 1.0, -19.0, 67.0, -20.0], [-55.0, -39.0, -22.0, 10.0, 12.0], [-10.0, 4.0, 50.0, -64.0, 9.0], [-4.0, -19.0, -25.0, 17.0, 30.0]], [[1.0, 47.0, -53.0, -3.0, -49.0], [-4.0, -3.0, -72.0, 5.0, -1.0], [16.0, -58.0, -35.0, 28.0, 57.0], [67.0, 51.0, 30.0, 25.0, 0.0]], [[42.0, -34.0, 44.0, 30.0, 71.0], [29.0, 36.0, -46.0, 37.0, 37.0], [-28.0, 26.0, 27.0, 22.0, 6.0], [-27.0, -24.0, 55.0, 44.0, 58.0]], [[20.0, -42.0, 15.0, -49.0, 16.0], [62.0, 7.0, 46.0, 25.0, -34.0], [31.0, 34.0, -24.0, 58.0, -30.0], [-24.0, -62.0, 53.0, -40.0, -34.0]]], [[[15.0, -17.0, -16.0, 100.0, 8.0], [-53.0, -15.0, -89.0, 15.0, 39.0], [-40.0, -11.0, 85.0, -72.0, 27.0], [-11.0, 6.0, 16.0, 71.0, 79.0]], [[-37.0, -5.0, 24.0, -4.0, -6.0], [-29.0, -30.0, 76.0, -7.0, 2.0], [1.0, 38.0, 4.0, -8.0, -47.0], [-28.0, -19.0, -92.0, -47.0, -36.0]], [[3.0, 38.0, -2.0, -28.0, -20.0], [15.0, -8.0, -6.0, 37.0, 9.0], [28.0, -1.0, -65.0, 87.0, 20.0], [45.0, 3.0, -15.0, -22.0, -30.0]], [[0.0, 63.0, 39.0, -17.0, 23.0], [-19.0, 24.0, 39.0, -32.0, 23.0], [-15.0, 0.0, -41.0, -19.0, 9.0], [19.0, 37.0, -75.0, -9.0, -1.0]], [[-15.0, -11.0, -7.0, -35.0, -30.0], [23.0, -31.0, 32.0, 42.0, -16.0], [40.0, 24.0, -40.0, 88.0, -21.0], [8.0, -37.0, -11.0, -51.0, -56.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, -3.0, 3.0], [6.0, -4.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-6.0, -4.0, 3.0], [-1.0, 5.0, -6.0]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-3.0, -5.0, -2.0, -7.0, -6.0], [-2.0, -4.0, 0.0, 5.0, 2.0], [2.0, -3.0, -3.0, 0.0, -1.0], [3.0, 3.0, 7.0, -2.0, -6.0]], [[0.0, 6.0, 4.0, -6.0, 5.0], [6.0, 0.0, 5.0, 1.0, -2.0], [-3.0, -2.0, 5.0, 0.0, -1.0], [3.0, -6.0, 1.0, 5.0, -6.0]], [[2.0, 1.0, 6.0, 5.0, -3.0], [-4.0, 4.0, -1.0, -5.0, -4.0], [0.0, 1.0, -6.0, 6.0, 1.0], [-4.0, -5.0, -5.0, -4.0, -6.0]]], [[[3.0, 1.0, -1.0, 3.0, -4.0], [-1.0, -4.0, 4.0, 7.0, 4.0], [-1.0, 2.0, 6.0, -1.0, -2.0], [7.0, -1.0, -1.0, -6.0, 7.0]], [[6.0, 2.0, -4.0, -6.0, 5.0], [0.0, -1.0, -6.0, 7.0, 0.0], [3.0, -7.0, -6.0, 5.0, -1.0], [4.0, 1.0, -2.0, 0.0, -4.0]], [[-2.0, -3.0, 1.0, -6.0, -1.0], [3.0, 0.0, -2.0, 0.0, 6.0], [0.0, -5.0, -2.0, 1.0, -3.0], [-1.0, 1.0, 0.0, -7.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-4.0, -22.0, 25.0, 59.0, -61.0], [-17.0, 0.0, 20.0, -14.0, 44.0], [-13.0, 30.0, 23.0, -3.0, -15.0], [-6.0, -8.0, -30.0, -94.0, 80.0]])+(1.-msk_ref)*numpy.array([[63.0, 36.0, -11.0, 84.0, 42.0], [-41.0, 35.0, -45.0, -21.0, -56.0], [16.0, 22.0, -44.0, 38.0, 28.0], [-23.0, -9.0, -70.0, 28.0, 3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 4.0], [-5.0, -1.0], [1.0, 4.0]], [[5.0, 6.0], [6.0, 6.0], [-6.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 6.0], [4.0, -5.0], [7.0, -4.0]], [[2.0, 0.0], [6.0, -7.0], [-2.0, 1.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[-1.0, 3.0, -6.0, 0.0, 4.0], [0.0, 6.0, 0.0, -3.0, -6.0], [0.0, 3.0, 1.0, -2.0, -6.0], [-1.0, 1.0, -6.0, 1.0, 4.0]], [[-7.0, 0.0, 3.0, 7.0, -3.0], [-4.0, 1.0, 1.0, -5.0, 0.0], [-2.0, 3.0, 0.0, 5.0, 6.0], [-3.0, 3.0, -4.0, -5.0, 0.0]], [[0.0, -3.0, 3.0, -1.0, 1.0], [0.0, 0.0, -6.0, 7.0, -1.0], [-4.0, 7.0, 7.0, -5.0, -2.0], [0.0, -4.0, 5.0, 0.0, 5.0]]], [[[-3.0, 7.0, 2.0, -6.0, 6.0], [7.0, 2.0, 1.0, 4.0, -1.0], [-5.0, 1.0, 3.0, 0.0, 6.0], [0.0, -7.0, -1.0, -1.0, 0.0]], [[6.0, -1.0, -2.0, 4.0, 3.0], [5.0, -7.0, -1.0, -3.0, -1.0], [0.0, 3.0, 0.0, -2.0, -6.0], [-6.0, -6.0, 0.0, -2.0, 4.0]], [[5.0, -1.0, 3.0, 6.0, 2.0], [-2.0, 3.0, -4.0, -7.0, 0.0], [0.0, 6.0, -7.0, 4.0, -3.0], [-2.0, -4.0, -5.0, 2.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[22.0, -14.0], [44.0, 43.0], [-56.0, -36.0], [-78.0, -65.0], [68.0, 63.0]], [[97.0, 90.0], [-31.0, -28.0], [12.0, 3.0], [64.0, 76.0], [-36.0, -40.0]], [[-19.0, -44.0], [-9.0, 19.0], [68.0, 99.0], [-74.0, -73.0], [-44.0, -17.0]], [[-13.0, -23.0], [-62.0, -65.0], [26.0, 29.0], [0.0, -23.0], [45.0, 60.0]]])+(1.-msk_ref)*numpy.array([[[-8.0, -8.0], [-11.0, 36.0], [19.0, -46.0], [21.0, -53.0], [21.0, 16.0]], [[32.0, -17.0], [-40.0, 83.0], [-34.0, 22.0], [33.0, -7.0], [-15.0, -25.0]], [[-46.0, 26.0], [69.0, -40.0], [69.0, -29.0], [-35.0, 1.0], [-8.0, -19.0]], [[-44.0, 49.0], [-58.0, 45.0], [27.0, -41.0], [-38.0, 47.0], [59.0, -24.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, 2.0, -5.0, 0.0, 0.0], [1.0, -2.0, -5.0, 0.0, -1.0], [5.0, 0.0, -5.0, -4.0, -5.0], [-1.0, 0.0, 2.0, -3.0, -1.0]], [[-3.0, 3.0, 3.0, -7.0, -5.0], [-7.0, -1.0, -1.0, 5.0, -1.0], [0.0, 6.0, -1.0, 7.0, 0.0], [-7.0, 6.0, 5.0, -4.0, 7.0]], [[6.0, 3.0, -7.0, 5.0, -7.0], [0.0, 7.0, -7.0, 2.0, -6.0], [0.0, -4.0, 0.0, 7.0, -5.0], [-3.0, 3.0, -6.0, 1.0, 5.0]]], [[[0.0, 7.0, -2.0, 2.0, 1.0], [0.0, 0.0, -3.0, 1.0, -7.0], [6.0, 0.0, 4.0, 0.0, 1.0], [-1.0, -6.0, -1.0, 0.0, 4.0]], [[2.0, 7.0, 6.0, 5.0, 0.0], [3.0, 1.0, -1.0, -1.0, 7.0], [5.0, 4.0, -4.0, 0.0, 6.0], [2.0, 1.0, 6.0, -6.0, -5.0]], [[3.0, 5.0, 0.0, 6.0, -5.0], [-2.0, -6.0, 1.0, 4.0, 0.0], [1.0, 0.0, -1.0, 2.0, 1.0], [-5.0, -7.0, -1.0, -7.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, 0.0, 1.0, 1.0, 0.0], [1.0, -5.0, -6.0, -7.0, 1.0], [0.0, -7.0, 0.0, -2.0, 7.0], [-4.0, -3.0, 6.0, 7.0, -6.0]], [[-2.0, -1.0, 5.0, 0.0, -6.0], [0.0, 5.0, 2.0, 0.0, 0.0], [-5.0, 7.0, 1.0, -7.0, 0.0], [7.0, 3.0, 0.0, -2.0, -7.0]], [[7.0, 2.0, -4.0, 5.0, 6.0], [4.0, 2.0, 6.0, 2.0, 6.0], [4.0, 0.0, 4.0, -5.0, 5.0], [-5.0, 4.0, -7.0, -3.0, -4.0]]], [[[0.0, -5.0, 2.0, -7.0, -3.0], [1.0, -4.0, 4.0, 5.0, 7.0], [0.0, 4.0, 4.0, -7.0, 0.0], [7.0, 6.0, 0.0, 2.0, 3.0]], [[1.0, -3.0, -5.0, -1.0, -6.0], [-1.0, -3.0, 1.0, 5.0, 3.0], [5.0, -4.0, -4.0, 3.0, 3.0], [2.0, 0.0, 0.0, -3.0, -6.0]], [[-3.0, -5.0, 0.0, 2.0, 0.0], [6.0, 5.0, 5.0, 2.0, 4.0], [-7.0, -6.0, -1.0, -5.0, -1.0], [6.0, -1.0, -1.0, 0.0, 7.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      s0=numpy.array([[[[3.0, -2.0, -3.0, -5.0, -6.0], [5.0, 0.0, 4.0, 1.0, 0.0], [-2.0, 7.0, -6.0, 0.0, 0.0], [-6.0, -6.0, -1.0, 3.0, -3.0]], [[-2.0, -5.0, 1.0, 3.0, 0.0], [-2.0, 4.0, 5.0, -3.0, -3.0], [2.0, -3.0, 0.0, -4.0, -5.0], [-7.0, 1.0, -4.0, -6.0, 3.0]], [[3.0, -6.0, -5.0, 1.0, 6.0], [0.0, -2.0, 0.0, -7.0, -1.0], [4.0, 4.0, 7.0, -2.0, 5.0], [5.0, -3.0, 6.0, 0.0, 0.0]]], [[[-3.0, 0.0, 3.0, 5.0, -1.0], [0.0, -1.0, 4.0, 1.0, 1.0], [-7.0, 2.0, 2.0, -6.0, 4.0], [-1.0, 6.0, 0.0, 0.0, 1.0]], [[0.0, 1.0, 2.0, 3.0, -5.0], [-6.0, -2.0, 7.0, -4.0, 0.0], [-2.0, -5.0, 4.0, 3.0, 1.0], [2.0, -5.0, -4.0, -5.0, 3.0]], [[-1.0, -7.0, 2.0, -4.0, 3.0], [0.0, 6.0, -1.0, 0.0, 0.0], [-1.0, -7.0, 4.0, -2.0, 0.0], [-4.0, 5.0, -5.0, -1.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[27.0, -17.0, -36.0, 17.0, -9.0], [19.0, 23.0, -26.0, -11.0, 2.0], [-4.0, -24.0, -24.0, -7.0, -34.0], [10.0, 22.0, -18.0, 9.0, -17.0]], [[-44.0, -65.0, 43.0, -32.0, 102.0], [50.0, 10.0, 49.0, -66.0, 50.0], [-12.0, -2.0, 18.0, -83.0, 39.0], [92.0, 2.0, 20.0, 63.0, -89.0]], [[-29.0, 27.0, 59.0, -4.0, 23.0], [-8.0, -40.0, 40.0, 4.0, 25.0], [15.0, 34.0, 16.0, -12.0, 57.0], [2.0, -39.0, 36.0, -26.0, -7.0]], [[-19.0, 38.0, 35.0, -15.0, 3.0], [-9.0, 41.0, -7.0, 3.0, -18.0], [16.0, 26.0, 34.0, 40.0, 39.0], [2.0, 22.0, 16.0, 14.0, 24.0]], [[23.0, -21.0, -40.0, 21.0, -58.0], [-27.0, 31.0, -1.0, 28.0, -58.0], [-58.0, -44.0, 43.0, 72.0, -28.0], [-36.0, -2.0, -80.0, 33.0, 66.0]]], [[[4.0, -38.0, -67.0, -16.0, 10.0], [1.0, -14.0, -17.0, -4.0, -45.0], [-5.0, -36.0, 1.0, -34.0, -61.0], [-3.0, -18.0, -36.0, 29.0, 11.0]], [[-10.0, 15.0, 16.0, -14.0, -37.0], [-46.0, -56.0, 21.0, 41.0, 1.0], [-10.0, 24.0, -6.0, 26.0, 3.0], [-55.0, -20.0, 15.0, -48.0, 42.0]], [[4.0, 95.0, 29.0, 2.0, -16.0], [-8.0, 0.0, -45.0, 18.0, 12.0], [78.0, 58.0, -36.0, 17.0, 25.0], [-24.0, 20.0, 72.0, -67.0, 9.0]], [[-39.0, -49.0, 9.0, -32.0, 65.0], [10.0, -52.0, 48.0, -24.0, 9.0], [-9.0, -6.0, 18.0, -74.0, 7.0], [32.0, -49.0, 4.0, 26.0, -33.0]], [[3.0, -5.0, -4.0, 18.0, 23.0], [21.0, -4.0, 7.0, -16.0, 2.0], [6.0, -14.0, 7.0, -28.0, 6.0], [23.0, -27.0, -10.0, 11.0, -22.0]]], [[[7.0, -54.0, -10.0, -24.0, -40.0], [-20.0, 34.0, 2.0, 9.0, 11.0], [-63.0, -12.0, -11.0, 48.0, -30.0], [-16.0, 71.0, -22.0, 21.0, 15.0]], [[16.0, -39.0, -106.0, -22.0, 24.0], [27.0, 54.0, -68.0, -28.0, -77.0], [15.0, -54.0, 3.0, -35.0, -90.0], [25.0, 26.0, -50.0, 74.0, 4.0]], [[50.0, 71.0, 1.0, 83.0, -67.0], [-2.0, 41.0, -25.0, 28.0, -22.0], [6.0, -12.0, 18.0, 81.0, 25.0], [-29.0, -15.0, -36.0, -27.0, 41.0]], [[0.0, -49.0, 32.0, 9.0, 38.0], [41.0, 5.0, 31.0, -41.0, 79.0], [-23.0, -4.0, -30.0, -46.0, 20.0], [56.0, 23.0, 18.0, 10.0, -83.0]], [[47.0, 35.0, -52.0, 73.0, -6.0], [38.0, 41.0, -43.0, -12.0, -46.0], [29.0, -46.0, 17.0, 0.0, -15.0], [18.0, -38.0, -53.0, 19.0, 1.0]]], [[[31.0, -31.0, -12.0, 58.0, 19.0], [57.0, 80.0, -1.0, -44.0, 4.0], [-30.0, -54.0, 29.0, 2.0, 12.0], [65.0, 9.0, -60.0, 67.0, -44.0]], [[-28.0, 14.0, 12.0, -5.0, -3.0], [-38.0, -45.0, 42.0, 30.0, -54.0], [-14.0, -2.0, 68.0, 20.0, 26.0], [-33.0, -79.0, -30.0, 6.0, 62.0]], [[23.0, -49.0, -73.0, 8.0, 3.0], [25.0, 74.0, -34.0, -24.0, -59.0], [-30.0, -64.0, 30.0, 8.0, -54.0], [28.0, 25.0, -77.0, 84.0, 8.0]], [[11.0, -52.0, -63.0, 11.0, 35.0], [32.0, 1.0, -5.0, -29.0, -32.0], [-11.0, -56.0, 12.0, -56.0, -46.0], [34.0, -34.0, -53.0, 52.0, -23.0]], [[6.0, 56.0, 40.0, 26.0, -39.0], [-25.0, -24.0, 11.0, 33.0, 14.0], [11.0, 30.0, -1.0, 43.0, 39.0], [-38.0, -20.0, 21.0, -56.0, 28.0]]]])+(1.-msk_ref)*numpy.array([[[[43.0, 28.0, -25.0, 37.0, 39.0], [6.0, -12.0, -21.0, -32.0, -4.0], [29.0, -41.0, -1.0, 19.0, 37.0], [-68.0, -20.0, -2.0, 10.0, -32.0]], [[-20.0, 25.0, -8.0, -47.0, -12.0], [-69.0, -65.0, -68.0, -7.0, -63.0], [55.0, 17.0, -26.0, 107.0, -34.0], [-37.0, -26.0, 37.0, 11.0, 16.0]], [[-56.0, -42.0, 18.0, -47.0, -57.0], [-10.0, 2.0, 14.0, 40.0, 2.0], [-29.0, 20.0, -17.0, -1.0, -42.0], [81.0, 8.0, 15.0, -8.0, 42.0]], [[-9.0, -15.0, 1.0, -46.0, -45.0], [-23.0, -7.0, 45.0, 69.0, 29.0], [32.0, 88.0, 19.0, -22.0, -17.0], [53.0, 62.0, -33.0, -43.0, -26.0]], [[-2.0, 17.0, -7.0, 42.0, 69.0], [40.0, 76.0, 78.0, 30.0, 20.0], [-22.0, 40.0, 37.0, -41.0, -30.0], [-5.0, 33.0, -81.0, -47.0, 60.0]]], [[[23.0, 20.0, 25.0, 11.0, 48.0], [11.0, -17.0, -40.0, -65.0, -13.0], [-20.0, -25.0, 22.0, -14.0, 17.0], [-46.0, -21.0, 30.0, 57.0, 20.0]], [[-42.0, -27.0, 36.0, 11.0, -21.0], [29.0, 56.0, 20.0, -7.0, -1.0], [-80.0, -4.0, -6.0, -47.0, -22.0], [63.0, -8.0, 8.0, 2.0, 31.0]], [[20.0, -41.0, 2.0, -33.0, -84.0], [-5.0, -37.0, 4.0, 25.0, 49.0], [17.0, 1.0, -6.0, -45.0, 50.0], [55.0, 28.0, 25.0, 5.0, -96.0]], [[-42.0, -4.0, 36.0, -37.0, -3.0], [-22.0, -26.0, -54.0, -36.0, -46.0], [-33.0, -8.0, -11.0, 35.0, -40.0], [9.0, -34.0, 55.0, 48.0, 70.0]], [[-1.0, -4.0, -9.0, -12.0, 9.0], [-3.0, -21.0, -8.0, 3.0, 1.0], [11.0, -17.0, -3.0, 19.0, -5.0], [-9.0, -7.0, 7.0, 11.0, 28.0]]], [[[15.0, 52.0, -12.0, 67.0, 45.0], [3.0, 57.0, 5.0, -25.0, -37.0], [3.0, 14.0, -1.0, 18.0, 1.0], [-57.0, -13.0, -39.0, -38.0, -34.0]], [[85.0, 51.0, 5.0, 4.0, 66.0], [-12.0, -70.0, -56.0, -70.0, 2.0], [55.0, 0.0, 48.0, -7.0, 61.0], [-107.0, 5.0, 21.0, 62.0, -50.0]], [[11.0, -28.0, -50.0, 19.0, 12.0], [44.0, 44.0, 110.0, 94.0, 78.0], [20.0, 10.0, 16.0, -45.0, 1.0], [35.0, 54.0, -89.0, -71.0, 18.0]], [[3.0, 31.0, -39.0, 25.0, 12.0], [-29.0, -19.0, -51.0, -23.0, -53.0], [41.0, -52.0, -46.0, 99.0, 1.0], [-66.0, -54.0, 16.0, -7.0, -14.0]], [[46.0, -8.0, -42.0, -4.0, 42.0], [23.0, -34.0, 37.0, 35.0, 61.0], [50.0, -23.0, 27.0, -15.0, 28.0], [-30.0, 29.0, -35.0, 0.0, 21.0]]], [[[33.0, 36.0, -73.0, 16.0, 63.0], [-13.0, -17.0, 30.0, 49.0, 7.0], [93.0, 5.0, 5.0, 69.0, -7.0], [-77.0, 15.0, -67.0, -51.0, 22.0]], [[-73.0, -47.0, 48.0, -48.0, -12.0], [23.0, 45.0, 64.0, 51.0, 23.0], [-77.0, 63.0, 28.0, -62.0, -77.0], [108.0, 40.0, -20.0, -8.0, 124.0]], [[56.0, 53.0, -25.0, 23.0, 84.0], [-3.0, -16.0, 5.0, -11.0, 3.0], [59.0, 25.0, 41.0, 13.0, 16.0], [-92.0, 20.0, -43.0, -5.0, -1.0]], [[25.0, 26.0, -2.0, 6.0, 66.0], [2.0, -35.0, -40.0, -48.0, -16.0], [12.0, -37.0, 15.0, 26.0, 7.0], [-70.0, -26.0, 19.0, 48.0, 47.0]], [[-33.0, -42.0, -1.0, -3.0, -39.0], [25.0, 42.0, 56.0, 51.0, 33.0], [-35.0, 4.0, -10.0, -38.0, -17.0], [76.0, 19.0, -23.0, -34.0, 17.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[3.0, -4.0], [-2.0, 5.0], [0.0, -7.0]]),self.functionspace)
      arg1=numpy.array([-1.0, 6.0, -3.0])
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([-15.0, 55.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[3.0, -3.0], [0.0, -6.0], [-2.0, 2.0]]),self.functionspace)
      arg1=numpy.array([[5.0, -1.0], [2.0, 0.0], [7.0, 6.0]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[1.0, -15.0], [-13.0, 15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[0.0, -2.0], [3.0, 3.0], [7.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([-1.0, 1.0, 7.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([52.0, -16.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[-1.0, 6.0], [-6.0, -2.0], [-3.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[-3.0, -1.0], [-7.0, -6.0], [2.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[39.0, 46.0], [-4.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[0.0, -4.0], [3.0, -2.0], [7.0, -7.0]]),self.functionspace)
      arg1=Data(numpy.array([-3.0, -1.0, 5.0]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([32.0, -21.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[6.0, 3.0], [1.0, 4.0], [2.0, 5.0]]),self.functionspace)
      arg1=Data(numpy.array([[-6.0, 0.0], [4.0, 0.0], [1.0, -3.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-30.0, -6.0], [3.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[5.0, -3.0], [0.0, -6.0], [4.0, -4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, 4.0, 6.0])+(1.-msk_arg1)*numpy.array([-4.0, 7.0, -7.0])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([44.0, -60.0])+(1.-msk_ref)*numpy.array([-48.0, -2.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[7.0, 6.0], [4.0, -5.0], [7.0, -4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 4.0], [-4.0, -5.0], [2.0, 7.0]])+(1.-msk_arg1)*numpy.array([[-3.0, 6.0], [-5.0, 1.0], [-5.0, 2.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 57.0], [18.0, 21.0]])+(1.-msk_ref)*numpy.array([[-76.0, 60.0], [27.0, 23.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[7.0, 2.0, 4.0, 6.0, -4.0], [3.0, -4.0, 2.0, 7.0, 6.0], [7.0, 0.0, -5.0, 0.0, 6.0], [7.0, -4.0, 6.0, -1.0, 5.0]], [[1.0, -5.0, -3.0, 0.0, 0.0], [-7.0, -7.0, 0.0, 5.0, 0.0], [-5.0, 7.0, -1.0, 6.0, -6.0], [1.0, 3.0, -5.0, 7.0, 0.0]], [[0.0, 4.0, -1.0, 1.0, 6.0], [-7.0, 7.0, 7.0, 4.0, -2.0], [-7.0, -3.0, 0.0, 4.0, -4.0], [6.0, 1.0, 6.0, 2.0, -4.0]]], [[[-4.0, -4.0, -4.0, 6.0, -7.0], [-4.0, 2.0, -6.0, -6.0, -4.0], [-2.0, -7.0, 4.0, -1.0, 7.0], [0.0, -1.0, 2.0, -4.0, 4.0]], [[-5.0, 2.0, -7.0, 1.0, 5.0], [1.0, -2.0, 0.0, -2.0, 5.0], [-6.0, 0.0, -6.0, -2.0, 2.0], [-2.0, 1.0, 0.0, 1.0, 2.0]], [[3.0, -6.0, 0.0, 0.0, 7.0], [-5.0, 3.0, 1.0, 4.0, 0.0], [-3.0, -1.0, -4.0, 7.0, 1.0], [7.0, -3.0, 2.0, 6.0, 1.0]]]]),self.functionspace)
      arg1=numpy.array([[-1.0, 4.0, 0.0], [-1.0, 6.0, -4.0]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[-41.0, 18.0, -54.0, -6.0, 13.0], [-1.0, -50.0, 0.0, -9.0, 28.0], [-49.0, 39.0, -23.0, -15.0, -29.0], [-43.0, 35.0, -36.0, 15.0, -1.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[1.0, -5.0, -2.0, 0.0, -3.0], [-5.0, 0.0, 3.0, 0.0, 2.0], [-5.0, -4.0, 4.0, 0.0, 5.0], [4.0, 5.0, -1.0, 0.0, 5.0]], [[-7.0, 7.0, 7.0, 4.0, -2.0], [4.0, -7.0, -6.0, -6.0, -4.0], [-2.0, 3.0, 2.0, -4.0, -6.0], [5.0, 0.0, -4.0, 4.0, -5.0]], [[-6.0, -1.0, -1.0, -7.0, 0.0], [-7.0, 3.0, -7.0, 6.0, 5.0], [6.0, -7.0, 0.0, -4.0, 4.0], [-1.0, -2.0, 1.0, 1.0, -5.0]]], [[[0.0, 3.0, 0.0, 7.0, -4.0], [-1.0, 0.0, 5.0, 3.0, -6.0], [-5.0, -3.0, 0.0, 6.0, 0.0], [-4.0, -4.0, 5.0, 6.0, 0.0]], [[-1.0, -2.0, 4.0, -6.0, -2.0], [6.0, -5.0, 6.0, 3.0, 2.0], [5.0, 4.0, 7.0, -1.0, -1.0], [-5.0, 0.0, -5.0, -6.0, -5.0]], [[2.0, -7.0, 0.0, -2.0, -4.0], [7.0, 3.0, 3.0, 0.0, 0.0], [-3.0, 6.0, -7.0, 1.0, -1.0], [1.0, -4.0, -3.0, 0.0, -4.0]]]]),self.functionspace)
      arg1=numpy.array([[[-5.0, -2.0], [2.0, 7.0], [-2.0, 1.0]], [[2.0, -4.0], [-1.0, 7.0], [-5.0, -2.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[-16.0, -68.0], [84.0, 46.0], [22.0, 80.0], [52.0, -45.0], [25.0, 2.0]], [[4.0, 63.0], [-30.0, -87.0], [-24.0, -39.0], [-21.0, -27.0], [-42.0, 11.0]], [[9.0, 63.0], [0.0, 50.0], [12.0, 69.0], [8.0, -65.0], [-39.0, -53.0]], [[-16.0, 5.0], [-9.0, 12.0], [25.0, -74.0], [24.0, -37.0], [0.0, -77.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[4.0, 3.0, -4.0, -2.0, 6.0], [-4.0, -7.0, 0.0, -4.0, 2.0], [-6.0, 7.0, 0.0, -7.0, -7.0], [2.0, 1.0, -6.0, -4.0, -4.0]], [[-2.0, -6.0, 1.0, 4.0, 0.0], [0.0, 3.0, 7.0, 0.0, 5.0], [-4.0, 3.0, 6.0, -7.0, 1.0], [-6.0, 6.0, 4.0, 0.0, -6.0]], [[-1.0, 6.0, 2.0, 4.0, 2.0], [-1.0, -3.0, 1.0, 0.0, 1.0], [-7.0, -2.0, -6.0, 0.0, 3.0], [7.0, 3.0, -6.0, -7.0, 5.0]]], [[[-4.0, -6.0, -3.0, 0.0, -3.0], [-5.0, -3.0, -3.0, -7.0, 6.0], [0.0, 6.0, -3.0, -7.0, 0.0], [-1.0, -6.0, 0.0, 5.0, 7.0]], [[-6.0, 0.0, 3.0, 5.0, 2.0], [1.0, 0.0, -3.0, 3.0, -2.0], [-6.0, 3.0, 4.0, 5.0, -1.0], [5.0, -7.0, 6.0, 5.0, -7.0]], [[-3.0, 6.0, -2.0, 0.0, -1.0], [-1.0, 0.0, -6.0, -4.0, -2.0], [2.0, 7.0, 4.0, 2.0, 7.0], [2.0, 2.0, 2.0, 6.0, -2.0]]]]),self.functionspace)
      arg1=numpy.array([[[[-6.0, 6.0, -5.0, 0.0, -6.0], [6.0, 7.0, -5.0, 5.0, -7.0], [-1.0, 3.0, -5.0, 0.0, 2.0], [2.0, -6.0, -6.0, 5.0, 2.0]], [[2.0, -1.0, 2.0, -4.0, 5.0], [-3.0, -3.0, 6.0, 0.0, -5.0], [-1.0, -2.0, 2.0, 3.0, 0.0], [-2.0, 2.0, 0.0, 3.0, -6.0]], [[-3.0, -5.0, 3.0, 5.0, 2.0], [-5.0, 3.0, 1.0, -2.0, 5.0], [0.0, -3.0, -5.0, 7.0, 4.0], [6.0, -1.0, -1.0, 2.0, 5.0]]], [[[-7.0, -1.0, -3.0, -2.0, 1.0], [2.0, 0.0, 0.0, 3.0, -2.0], [7.0, -1.0, 6.0, 6.0, -3.0], [0.0, -2.0, 0.0, 6.0, 7.0]], [[0.0, 6.0, 5.0, 4.0, 1.0], [-6.0, -4.0, -2.0, 0.0, -1.0], [-4.0, -6.0, -4.0, 4.0, -2.0], [7.0, 4.0, -2.0, -6.0, -5.0]], [[6.0, -1.0, 7.0, 1.0, 7.0], [-4.0, 6.0, -7.0, -3.0, 3.0], [-6.0, -1.0, -3.0, 0.0, 7.0], [-4.0, -3.0, -6.0, 3.0, -6.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-15.0, 2.0, -66.0, -16.0, -67.0], [75.0, 37.0, 0.0, 19.0, -18.0], [12.0, 62.0, -10.0, -61.0, 7.0], [-24.0, -34.0, 7.0, 15.0, 35.0]], [[30.0, -6.0, 51.0, 72.0, 0.0], [-30.0, 93.0, -87.0, -33.0, 69.0], [-75.0, 3.0, -111.0, -12.0, 90.0], [30.0, -42.0, -60.0, -9.0, -6.0]], [[29.0, -12.0, 38.0, 22.0, 19.0], [-53.0, -49.0, 36.0, -27.0, 30.0], [-18.0, -33.0, -12.0, 11.0, -11.0], [31.0, 48.0, 28.0, -55.0, -28.0]], [[8.0, -6.0, 55.0, 24.0, 45.0], [-74.0, -34.0, 28.0, -18.0, 9.0], [-22.0, -56.0, -22.0, 60.0, 2.0], [47.0, 36.0, -2.0, -20.0, -33.0]], [[-27.0, 42.0, -12.0, 23.0, -40.0], [12.0, 34.0, -25.0, 20.0, -31.0], [-29.0, 4.0, -63.0, 4.0, 18.0], [42.0, -21.0, -36.0, 1.0, -3.0]]], [[[56.0, -7.0, 30.0, 8.0, 11.0], [-31.0, -41.0, 24.0, -30.0, 29.0], [-29.0, -9.0, -6.0, -33.0, -6.0], [-3.0, 42.0, 29.0, -61.0, -47.0]], [[78.0, -27.0, 41.0, -21.0, 48.0], [-42.0, -67.0, 50.0, -38.0, 25.0], [-17.0, -15.0, 38.0, -30.0, -17.0], [-38.0, 57.0, 45.0, -50.0, -68.0]], [[-4.0, -21.0, -31.0, -35.0, -11.0], [10.0, -42.0, 91.0, 7.0, -39.0], [20.0, 10.0, 21.0, -2.0, -23.0], [-5.0, 25.0, 41.0, 5.0, -7.0]], [[49.0, 5.0, 28.0, 22.0, -8.0], [-40.0, -64.0, 42.0, -29.0, 27.0], [-33.0, -19.0, -22.0, -30.0, -21.0], [29.0, 62.0, 42.0, -92.0, -48.0]], [[-59.0, -14.0, -39.0, -37.0, 5.0], [24.0, -2.0, 39.0, 32.0, -50.0], [55.0, 1.0, 45.0, 50.0, -20.0], [-6.0, -17.0, 3.0, 69.0, 43.0]]], [[[61.0, -35.0, -15.0, -41.0, 10.0], [39.0, -15.0, -3.0, -22.0, 39.0], [22.0, 45.0, 75.0, -85.0, -14.0], [-96.0, 5.0, 43.0, -14.0, -5.0]], [[-30.0, 54.0, 11.0, -15.0, 27.0], [9.0, 64.0, -74.0, 36.0, -68.0], [-22.0, -10.0, -16.0, 43.0, 31.0], [-11.0, -55.0, -88.0, 79.0, -29.0]], [[75.0, 47.0, 51.0, -28.0, 47.0], [-34.0, -28.0, -6.0, -9.0, -46.0], [-67.0, -19.0, -4.0, -26.0, 5.0], [-36.0, 28.0, -26.0, -24.0, -131.0]], [[89.0, 0.0, 81.0, 64.0, 19.0], [-73.0, -36.0, -31.0, -62.0, 99.0], [-67.0, -32.0, -47.0, -43.0, 11.0], [27.0, 56.0, 20.0, -122.0, -58.0]], [[77.0, -71.0, 90.0, 14.0, 101.0], [-82.0, 3.0, -3.0, -62.0, 81.0], [-32.0, -33.0, 5.0, 20.0, 49.0], [-33.0, 16.0, -1.0, 1.0, -42.0]]], [[[-26.0, 12.0, 41.0, 83.0, -10.0], [-45.0, 45.0, -63.0, -13.0, 54.0], [-35.0, -34.0, -89.0, 45.0, 39.0], [85.0, -15.0, -41.0, -24.0, 31.0]], [[51.0, -53.0, 13.0, -23.0, 31.0], [-5.0, 38.0, 34.0, -25.0, 3.0], [-33.0, 28.0, -22.0, -25.0, 60.0], [-49.0, -19.0, -7.0, 41.0, -38.0]], [[74.0, 24.0, 64.0, -20.0, 64.0], [-62.0, -84.0, 22.0, -24.0, -8.0], [-34.0, -46.0, 38.0, -6.0, -34.0], [-22.0, 68.0, 18.0, -60.0, -108.0]], [[46.0, 30.0, 51.0, -19.0, 62.0], [-33.0, -33.0, -39.0, -9.0, -4.0], [-17.0, -32.0, 47.0, 1.0, -19.0], [-39.0, 23.0, -15.0, -16.0, -69.0]], [[-64.0, -90.0, -47.0, 5.0, -10.0], [33.0, 21.0, 17.0, -3.0, 70.0], [99.0, 22.0, 59.0, 31.0, -9.0], [-7.0, -29.0, 45.0, 50.0, 149.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[1.0, 6.0, 0.0, -3.0, 0.0], [2.0, -5.0, -1.0, -4.0, -6.0], [3.0, 0.0, -6.0, -2.0, 6.0], [-3.0, 0.0, 0.0, 4.0, 4.0]], [[1.0, 0.0, 5.0, -2.0, 3.0], [6.0, -1.0, -3.0, -2.0, 2.0], [-4.0, -4.0, 6.0, -5.0, 3.0], [-5.0, 0.0, 5.0, 5.0, 6.0]], [[-4.0, 1.0, 0.0, -7.0, 0.0], [-3.0, 0.0, 4.0, 0.0, 7.0], [3.0, 2.0, 4.0, -6.0, -4.0], [-6.0, 3.0, -1.0, 4.0, 3.0]]], [[[4.0, -7.0, -7.0, -1.0, 2.0], [7.0, -6.0, 0.0, 0.0, 7.0], [6.0, -3.0, 4.0, -3.0, -1.0], [-2.0, -4.0, 3.0, 1.0, -6.0]], [[5.0, 3.0, 3.0, -7.0, -1.0], [0.0, 0.0, 0.0, -1.0, -6.0], [0.0, 0.0, 3.0, -1.0, 1.0], [3.0, 5.0, 7.0, 4.0, 2.0]], [[4.0, 3.0, 0.0, 5.0, -3.0], [-6.0, 6.0, -7.0, 3.0, 1.0], [-7.0, 0.0, 0.0, -7.0, -4.0], [0.0, 6.0, -4.0, -1.0, -6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[0.0, -1.0, -5.0], [-1.0, 1.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[48.0, 26.0, 5.0, 66.0, -27.0], [-40.0, 49.0, -66.0, 22.0, -43.0], [-66.0, -3.0, -27.0, -12.0, -9.0], [40.0, 36.0, -24.0, -29.0, -55.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[6.0, -3.0, 0.0, -4.0, 2.0], [-3.0, -3.0, -2.0, 0.0, 2.0], [-5.0, 5.0, -4.0, -4.0, 7.0], [-3.0, 3.0, -3.0, -6.0, -6.0]], [[0.0, 6.0, 4.0, -6.0, 7.0], [-6.0, 1.0, 1.0, -1.0, -4.0], [6.0, 4.0, -5.0, 4.0, 3.0], [3.0, -6.0, 6.0, -1.0, 5.0]], [[-2.0, -7.0, -2.0, -7.0, 6.0], [-1.0, -2.0, -1.0, 6.0, -3.0], [2.0, 5.0, -3.0, 3.0, 0.0], [-3.0, 7.0, 0.0, -4.0, 0.0]]], [[[0.0, -2.0, 6.0, -7.0, -5.0], [1.0, -4.0, -7.0, 3.0, -3.0], [3.0, 4.0, -6.0, -4.0, -7.0], [-7.0, -2.0, 4.0, 1.0, -4.0]], [[6.0, 6.0, 3.0, 0.0, -4.0], [6.0, -1.0, 2.0, -4.0, 4.0], [3.0, 2.0, -6.0, 6.0, 6.0], [-4.0, 3.0, -6.0, 5.0, -3.0]], [[-6.0, 3.0, 0.0, -7.0, -3.0], [0.0, 4.0, -7.0, -5.0, -5.0], [0.0, -2.0, -4.0, -7.0, -7.0], [7.0, -7.0, -6.0, -5.0, 4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[-7.0, -3.0], [-2.0, 5.0], [6.0, 5.0]], [[-1.0, 5.0], [-3.0, -5.0], [1.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-78.0, -64.0], [-46.0, -33.0], [-35.0, 25.0], [-2.0, -95.0], [22.0, 51.0]], [[8.0, -51.0], [18.0, -7.0], [0.0, -46.0], [42.0, 55.0], [-38.0, -81.0]], [[23.0, 55.0], [-25.0, 38.0], [40.0, -32.0], [17.0, -10.0], [-73.0, -78.0]], [[23.0, 1.0], [19.0, -36.0], [17.0, 83.0], [-1.0, -32.0], [49.0, 42.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[4.0, -1.0, 0.0, 7.0, -4.0], [5.0, 0.0, -3.0, -5.0, -1.0], [0.0, -4.0, -1.0, -1.0, -2.0], [-6.0, 0.0, 2.0, 0.0, 0.0]], [[2.0, -5.0, 6.0, 1.0, -5.0], [5.0, 1.0, -7.0, 0.0, -6.0], [2.0, -4.0, 4.0, -6.0, -2.0], [7.0, -2.0, 7.0, 0.0, 1.0]], [[0.0, 5.0, 0.0, 0.0, 1.0], [-6.0, 3.0, 5.0, 3.0, -1.0], [-6.0, 4.0, 0.0, 0.0, 3.0], [-6.0, 1.0, -1.0, -3.0, 5.0]]], [[[-3.0, -7.0, 1.0, -4.0, 5.0], [5.0, -7.0, 3.0, -3.0, -6.0], [-2.0, -3.0, -4.0, -3.0, 1.0], [6.0, -1.0, 7.0, -1.0, -1.0]], [[1.0, 3.0, -4.0, -3.0, 5.0], [0.0, 7.0, 3.0, 7.0, 4.0], [4.0, 5.0, 0.0, -2.0, 3.0], [-6.0, 3.0, -4.0, -2.0, 4.0]], [[6.0, 2.0, 1.0, -4.0, 5.0], [0.0, 0.0, 0.0, -7.0, 4.0], [-4.0, 5.0, 3.0, 6.0, 3.0], [7.0, 3.0, -6.0, -3.0, 6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[[0.0, -4.0, 0.0, 5.0, -3.0], [0.0, -7.0, 0.0, 5.0, 7.0], [-1.0, 6.0, -4.0, -1.0, 5.0], [-7.0, 2.0, 3.0, -5.0, 2.0]], [[-2.0, -2.0, 6.0, -1.0, 3.0], [-1.0, -5.0, -6.0, -6.0, -1.0], [-5.0, 0.0, -4.0, -4.0, 3.0], [4.0, -4.0, -1.0, 0.0, 2.0]], [[3.0, 2.0, 3.0, 5.0, -5.0], [-2.0, -2.0, -1.0, 7.0, 2.0], [-4.0, 4.0, -3.0, 6.0, 0.0], [5.0, 0.0, -4.0, 2.0, -7.0]]], [[[-1.0, -2.0, 3.0, 1.0, 7.0], [6.0, 4.0, 5.0, 6.0, 0.0], [7.0, -2.0, 4.0, 7.0, -6.0], [5.0, 2.0, 3.0, -5.0, 0.0]], [[-2.0, -7.0, -2.0, 5.0, -4.0], [-2.0, 0.0, -4.0, 2.0, -6.0], [-4.0, 6.0, 3.0, 4.0, 2.0], [6.0, 0.0, 0.0, -7.0, 1.0]], [[-5.0, -1.0, 3.0, -5.0, -4.0], [-4.0, 3.0, 0.0, -1.0, 2.0], [3.0, -2.0, 2.0, -7.0, -1.0], [-5.0, 4.0, 2.0, -5.0, -5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-33.0, -27.0, 19.0, -10.0, -55.0], [-46.0, -32.0, -31.0, -14.0, 32.0], [-21.0, 24.0, -21.0, -71.0, 40.0], [-59.0, 18.0, 13.0, -42.0, -17.0]], [[16.0, 15.0, -36.0, 23.0, -106.0], [-61.0, 0.0, -22.0, 22.0, -6.0], [-49.0, 42.0, -6.0, 0.0, 26.0], [-15.0, 12.0, -35.0, 19.0, -54.0]], [[-10.0, 13.0, 50.0, -30.0, 37.0], [4.0, -23.0, -15.0, -39.0, 20.0], [-4.0, -28.0, -30.0, -40.0, 3.0], [0.0, -18.0, -1.0, 18.0, 3.0]], [[28.0, 3.0, -12.0, 35.0, -18.0], [-3.0, -82.0, -14.0, 3.0, 58.0], [-40.0, 40.0, -65.0, -23.0, 60.0], [-63.0, -14.0, 0.0, 26.0, 33.0]], [[-27.0, -22.0, -7.0, -5.0, -13.0], [3.0, 86.0, 34.0, 52.0, -41.0], [55.0, -10.0, 78.0, 50.0, -60.0], [43.0, 42.0, 14.0, -63.0, -45.0]]], [[[-33.0, -52.0, 27.0, -5.0, 65.0], [37.0, -28.0, 1.0, -17.0, 18.0], [29.0, -4.0, -2.0, -26.0, 10.0], [-20.0, 0.0, 49.0, -62.0, 62.0]], [[0.0, -31.0, -20.0, 42.0, -89.0], [-63.0, -39.0, -72.0, -13.0, -37.0], [-94.0, 68.0, -20.0, -7.0, 59.0], [26.0, -18.0, -34.0, -8.0, -12.0]], [[20.0, 9.0, -24.0, 35.0, -28.0], [9.0, 58.0, 40.0, 86.0, -22.0], [27.0, 14.0, 46.0, 94.0, -48.0], [51.0, 28.0, -13.0, -11.0, -52.0]], [[33.0, -10.0, -35.0, 57.0, -21.0], [-10.0, -4.0, -46.0, -1.0, -85.0], [-77.0, 44.0, 6.0, 79.0, 14.0], [112.0, -44.0, -50.0, 32.0, 11.0]], [[-13.0, -6.0, -53.0, -10.0, -84.0], [-52.0, 27.0, -9.0, -8.0, -19.0], [-11.0, 18.0, 27.0, -35.0, 17.0], [-48.0, 26.0, -3.0, -15.0, -23.0]]], [[[-8.0, -36.0, -32.0, 6.0, 22.0], [6.0, -18.0, -32.0, -54.0, -46.0], [-28.0, 12.0, 6.0, -14.0, 30.0], [12.0, -28.0, 8.0, -10.0, 70.0]], [[-12.0, -2.0, -16.0, 1.0, -81.0], [-52.0, 43.0, -15.0, 19.0, -36.0], [-18.0, 18.0, 33.0, 8.0, -9.0], [22.0, 22.0, -23.0, -17.0, -64.0]], [[-19.0, 1.0, 21.0, -28.0, -25.0], [-40.0, -20.0, -44.0, -56.0, -5.0], [-38.0, -4.0, -22.0, -64.0, 28.0], [-12.0, -14.0, -13.0, 10.0, -9.0]], [[-11.0, 30.0, -23.0, -42.0, -52.0], [-32.0, 43.0, 29.0, 3.0, 23.0], [36.0, -24.0, 22.0, -46.0, -15.0], [-74.0, 40.0, 6.0, 4.0, -46.0]], [[-9.0, -8.0, 3.0, 8.0, -32.0], [-16.0, 31.0, 2.0, 32.0, -18.0], [4.0, 10.0, 26.0, 26.0, -19.0], [29.0, 18.0, -7.0, -25.0, -41.0]]], [[[-61.0, 21.0, 75.0, -126.0, 107.0], [25.0, 64.0, 18.0, -97.0, -11.0], [82.0, -122.0, 34.0, -89.0, -64.0], [-1.0, 0.0, 31.0, -5.0, 3.0]], [[-13.0, -16.0, -9.0, 6.0, -42.0], [-24.0, 13.0, -6.0, 16.0, -8.0], [-4.0, 18.0, 16.0, -2.0, 3.0], [-5.0, 18.0, 1.0, -29.0, -23.0]], [[14.0, -4.0, 50.0, 15.0, 109.0], [69.0, -37.0, 10.0, 1.0, 17.0], [14.0, -18.0, -29.0, 39.0, -13.0], [50.0, -34.0, 12.0, 11.0, 51.0]], [[11.0, 13.0, -17.0, -11.0, 28.0], [16.0, -7.0, 6.0, -28.0, 0.0], [4.0, -16.0, -7.0, -12.0, 5.0], [-17.0, -14.0, 3.0, 28.0, 34.0]], [[-24.0, -24.0, 28.0, 13.0, -69.0], [-49.0, -1.0, -32.0, 25.0, -3.0], [-30.0, 34.0, 1.0, -7.0, 11.0], [18.0, 18.0, -12.0, -43.0, -59.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[-3.0, 0.0, -6.0, 5.0, 4.0], [-7.0, -4.0, 0.0, -7.0, -4.0], [0.0, 6.0, -6.0, -3.0, 4.0], [-1.0, 5.0, -7.0, 6.0, -5.0]], [[-6.0, 3.0, 5.0, 3.0, 1.0], [0.0, 3.0, 6.0, 0.0, -4.0], [0.0, 0.0, -7.0, 1.0, 1.0], [-5.0, -6.0, 6.0, 0.0, -2.0]], [[-6.0, 7.0, 7.0, 0.0, 6.0], [-1.0, -3.0, 4.0, 2.0, 0.0], [0.0, -4.0, 6.0, 1.0, -7.0], [0.0, 5.0, 7.0, -5.0, -7.0]]], [[[2.0, 2.0, -7.0, 7.0, 5.0], [-4.0, -4.0, -2.0, 3.0, 4.0], [-4.0, -3.0, 7.0, -6.0, -7.0], [-4.0, 0.0, -4.0, -2.0, 7.0]], [[-3.0, -1.0, 0.0, 6.0, -5.0], [-2.0, -1.0, 4.0, 5.0, -3.0], [-7.0, -3.0, -5.0, -6.0, -2.0], [-5.0, 3.0, 4.0, 5.0, -1.0]], [[-4.0, -6.0, -4.0, 4.0, -4.0], [4.0, 6.0, -3.0, 3.0, 4.0], [-6.0, -2.0, 1.0, 7.0, 6.0], [3.0, -6.0, 5.0, 5.0, -3.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[4.0, 3.0, 3.0], [4.0, -6.0, -7.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[6.0, 86.0, 12.0, -7.0, 115.0], [-63.0, -68.0, 19.0, -61.0, -22.0], [68.0, 32.0, 24.0, -43.0, -60.0], [-26.0, 41.0, -64.0, -64.0, 8.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[-7.0, 0.0, -4.0, 1.0, 6.0], [3.0, -3.0, 3.0, 1.0, 0.0], [-3.0, 3.0, 5.0, 0.0, 6.0], [5.0, -7.0, 1.0, 0.0, -6.0]], [[1.0, -3.0, -7.0, 6.0, -5.0], [-3.0, -5.0, -5.0, -3.0, 3.0], [7.0, -7.0, -5.0, -7.0, 0.0], [5.0, 0.0, 1.0, 6.0, 2.0]], [[3.0, -7.0, -3.0, 0.0, 6.0], [7.0, 2.0, -1.0, 5.0, -1.0], [-1.0, 7.0, -1.0, 0.0, 0.0], [-2.0, -5.0, 4.0, 0.0, -3.0]]], [[[4.0, -2.0, -2.0, -7.0, -6.0], [6.0, -6.0, 7.0, -6.0, 5.0], [-4.0, 7.0, 3.0, 6.0, 0.0], [7.0, 7.0, 3.0, -5.0, 2.0]], [[-3.0, -1.0, 5.0, 6.0, 2.0], [-6.0, 7.0, -1.0, -4.0, 2.0], [-6.0, -1.0, 0.0, 6.0, -2.0], [6.0, -4.0, 7.0, -2.0, 1.0]], [[2.0, -5.0, -6.0, 3.0, 2.0], [3.0, -4.0, -5.0, 2.0, -5.0], [2.0, -6.0, -1.0, -1.0, 3.0], [2.0, -2.0, -2.0, -2.0, -4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[0.0, 7.0], [3.0, -2.0], [1.0, -3.0]], [[0.0, 7.0], [-2.0, -6.0], [2.0, 0.0]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[16.0, -14.0], [-24.0, 19.0], [-46.0, -49.0], [12.0, -90.0], [-9.0, -20.0]], [[16.0, 84.0], [-35.0, -101.0], [-24.0, 89.0], [8.0, -20.0], [-6.0, 20.0]], [[36.0, -24.0], [-24.0, 69.0], [-18.0, 69.0], [-35.0, 20.0], [10.0, 54.0]], [[5.0, 44.0], [-1.0, 39.0], [-11.0, -28.0], [18.0, -35.0], [-7.0, -29.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-2.0, -2.0, -4.0, -1.0, -5.0], [5.0, -5.0, -1.0, 1.0, 7.0], [-3.0, 7.0, 0.0, 4.0, 0.0], [5.0, -5.0, 2.0, 3.0, -4.0]], [[-5.0, 2.0, 7.0, 6.0, -2.0], [-2.0, -3.0, -7.0, 3.0, 3.0], [-5.0, -7.0, -1.0, -2.0, 1.0], [-7.0, -1.0, 3.0, -7.0, 0.0]], [[6.0, -5.0, -2.0, -5.0, 5.0], [0.0, -6.0, -7.0, 0.0, -2.0], [-6.0, 7.0, -1.0, 1.0, -1.0], [7.0, 6.0, 1.0, 2.0, -5.0]]], [[[-6.0, 4.0, 0.0, 1.0, -6.0], [1.0, -3.0, -3.0, 4.0, 6.0], [-3.0, 1.0, -1.0, -2.0, -5.0], [-3.0, -5.0, 3.0, -5.0, -4.0]], [[0.0, -2.0, -6.0, 6.0, 5.0], [0.0, 1.0, 2.0, 4.0, 2.0], [-1.0, -7.0, -2.0, -3.0, 6.0], [2.0, -6.0, 5.0, -3.0, -1.0]], [[0.0, 2.0, -3.0, 2.0, 6.0], [3.0, -2.0, -3.0, -5.0, 7.0], [-1.0, -3.0, -4.0, 2.0, 0.0], [5.0, 7.0, 1.0, 0.0, -4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-6.0, 1.0, 7.0, -7.0, 6.0], [5.0, -7.0, -4.0, -2.0, 0.0], [-4.0, -4.0, -1.0, 0.0, -1.0], [5.0, 0.0, -7.0, -4.0, 4.0]], [[6.0, 6.0, 1.0, -4.0, -6.0], [2.0, -3.0, -2.0, 6.0, 0.0], [3.0, 5.0, -3.0, -6.0, 2.0], [0.0, -7.0, -2.0, -5.0, -4.0]], [[-3.0, -2.0, 7.0, 2.0, -3.0], [-4.0, 0.0, 7.0, 0.0, 7.0], [7.0, -2.0, -4.0, 6.0, -6.0], [7.0, -6.0, 7.0, -3.0, -4.0]]], [[[3.0, -1.0, -7.0, -4.0, 7.0], [0.0, 6.0, -3.0, 0.0, -5.0], [-1.0, 7.0, -4.0, 0.0, 0.0], [-4.0, 3.0, -7.0, 4.0, -3.0]], [[-6.0, -7.0, 0.0, -7.0, 2.0], [-2.0, 0.0, -5.0, -1.0, 6.0], [-2.0, 7.0, 0.0, 7.0, -1.0], [2.0, -4.0, 3.0, 2.0, -4.0]], [[-4.0, -4.0, 3.0, 5.0, -5.0], [-4.0, -1.0, 7.0, 4.0, -4.0], [-1.0, 1.0, 1.0, -3.0, 1.0], [4.0, -5.0, 6.0, 1.0, 6.0]]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-54.0, -38.0, 65.0, 70.0, -42.0], [-44.0, -7.0, 78.0, -26.0, 72.0], [41.0, -71.0, 17.0, 66.0, -44.0], [56.0, -19.0, 108.0, -9.0, 6.0]], [[55.0, 22.0, -69.0, 4.0, 5.0], [10.0, 30.0, -19.0, 26.0, -75.0], [-23.0, 44.0, 2.0, -62.0, 40.0], [-57.0, 26.0, -47.0, 27.0, 12.0]], [[120.0, 96.0, -44.0, 23.0, -57.0], [26.0, 10.0, -3.0, 44.0, -38.0], [38.0, 10.0, -12.0, -87.0, 33.0], [-58.0, 2.0, -36.0, -28.0, -30.0]], [[16.0, -6.0, -37.0, -63.0, -18.0], [7.0, -7.0, -62.0, 40.0, -12.0], [-28.0, 95.0, 1.0, -30.0, 39.0], [-24.0, -43.0, -17.0, 7.0, -23.0]], [[-69.0, -80.0, 58.0, 72.0, -95.0], [-83.0, -1.0, 94.0, 17.0, 71.0], [39.0, -1.0, 21.0, 59.0, -28.0], [68.0, -84.0, 167.0, 7.0, 2.0]]], [[[-51.0, -20.0, 35.0, -16.0, 34.0], [9.0, -26.0, 2.0, -10.0, -17.0], [-30.0, -20.0, 0.0, 3.0, -6.0], [33.0, 2.0, -20.0, -3.0, 43.0]], [[23.0, -7.0, -65.0, 30.0, -3.0], [-1.0, 28.0, -26.0, -17.0, -13.0], [-28.0, 1.0, 48.0, -5.0, 32.0], [-61.0, 54.0, 11.0, 41.0, 9.0]], [[-24.0, -28.0, -51.0, 4.0, 55.0], [17.0, 13.0, -53.0, -54.0, -10.0], [-64.0, -27.0, 59.0, 23.0, 24.0], [-50.0, 89.0, -19.0, 49.0, 35.0]], [[20.0, 7.0, -33.0, -88.0, 49.0], [23.0, 13.0, -77.0, -8.0, 24.0], [-2.0, 62.0, -31.0, 25.0, -4.0], [-23.0, 0.0, -59.0, 0.0, -66.0]], [[-40.0, -19.0, 17.0, -68.0, 41.0], [17.0, -29.0, -27.0, 30.0, -60.0], [-50.0, 54.0, -25.0, -37.0, 16.0], [29.0, -34.0, -63.0, -2.0, 40.0]]], [[[7.0, -7.0, -50.0, 43.0, 12.0], [5.0, 19.0, -13.0, -27.0, -29.0], [-39.0, -30.0, 53.0, -10.0, 29.0], [-51.0, 71.0, 1.0, 40.0, 39.0]], [[-48.0, 11.0, 75.0, 23.0, 71.0], [19.0, -19.0, 46.0, -61.0, 14.0], [16.0, -122.0, -21.0, 44.0, -59.0], [54.0, 53.0, -32.0, -27.0, 35.0]], [[22.0, 27.0, -13.0, 0.0, 18.0], [22.0, 1.0, -20.0, -20.0, 2.0], [-1.0, -28.0, 7.0, -2.0, 2.0], [-23.0, 38.0, -28.0, -4.0, -5.0]], [[-35.0, 5.0, 53.0, 21.0, 3.0], [10.0, -36.0, 30.0, -9.0, -9.0], [-9.0, -61.0, 8.0, -9.0, -9.0], [37.0, 4.0, 0.0, -21.0, 50.0]], [[-42.0, -29.0, 29.0, -28.0, -26.0], [-6.0, -33.0, -24.0, 0.0, 54.0], [-11.0, 14.0, 21.0, 30.0, 2.0], [25.0, -40.0, 44.0, -10.0, -9.0]]], [[[-134.0, -82.0, 113.0, 30.0, 9.0], [-41.0, -37.0, 77.0, -34.0, 56.0], [2.0, -71.0, 5.0, 83.0, -58.0], [110.0, -35.0, 85.0, -9.0, 51.0]], [[-1.0, -4.0, 62.0, 148.0, -124.0], [-67.0, 1.0, 158.0, 38.0, 3.0], [69.0, -67.0, 11.0, -21.0, -20.0], [53.0, -55.0, 138.0, -18.0, 41.0]], [[-22.0, -24.0, 6.0, -66.0, 17.0], [-2.0, -6.0, -34.0, 13.0, 18.0], [-6.0, 62.0, -26.0, 20.0, -6.0], [19.0, -43.0, -13.0, -3.0, -31.0]], [[-63.0, -17.0, 63.0, 52.0, 13.0], [-1.0, -30.0, 46.0, -45.0, 21.0], [-8.0, -107.0, 30.0, 33.0, -26.0], [43.0, 34.0, 33.0, -9.0, 59.0]], [[49.0, 33.0, -47.0, 21.0, -19.0], [18.0, 8.0, -30.0, -7.0, -5.0], [-9.0, -13.0, 36.0, -25.0, 31.0], [-57.0, 42.0, -6.0, 9.0, -4.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[0.0, 6.0, 3.0, -6.0, 2.0], [-4.0, -3.0, -7.0, -7.0, -2.0], [-4.0, 1.0, 5.0, -3.0, 6.0], [0.0, 3.0, 0.0, -2.0, 3.0]], [[7.0, 4.0, 5.0, -4.0, 1.0], [-7.0, -7.0, -3.0, 4.0, -1.0], [-2.0, 0.0, -6.0, -7.0, -7.0], [6.0, -1.0, 7.0, -4.0, 4.0]], [[-1.0, 7.0, -7.0, 2.0, -5.0], [7.0, -4.0, 7.0, 0.0, -7.0], [0.0, -6.0, -4.0, -4.0, 5.0], [3.0, -2.0, 0.0, -3.0, 0.0]]], [[[1.0, -2.0, -3.0, -6.0, 1.0], [1.0, 7.0, 3.0, 7.0, -4.0], [0.0, 6.0, 4.0, 0.0, -5.0], [6.0, -4.0, 3.0, 3.0, 6.0]], [[-5.0, 5.0, 0.0, 5.0, 4.0], [6.0, -5.0, 0.0, 3.0, 4.0], [-7.0, 2.0, -1.0, 0.0, -6.0], [-4.0, 4.0, 1.0, 4.0, 1.0]], [[7.0, -2.0, 5.0, 6.0, 4.0], [-7.0, -6.0, 1.0, 0.0, -2.0], [7.0, 3.0, 3.0, 4.0, 2.0], [-2.0, 1.0, 7.0, -3.0, -2.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, -2.0, 2.0], [-4.0, 0.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -3.0, 1.0], [-2.0, -3.0, 3.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-48.0, 22.0, -32.0, 12.0, -32.0], [52.0, 2.0, 4.0, -36.0, 12.0], [-24.0, -48.0, -24.0, -10.0, 36.0], [-22.0, 10.0, -54.0, 2.0, -24.0]])+(1.-msk_ref)*numpy.array([[12.0, -46.0, -13.0, 53.0, -18.0], [3.0, 12.0, 41.0, -7.0, -6.0], [64.0, -19.0, -2.0, 41.0, 36.0], [-21.0, -12.0, -9.0, -10.0, -45.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[3.0, -5.0, -6.0, -6.0, -5.0], [-4.0, -3.0, -5.0, -1.0, 6.0], [-6.0, -1.0, 1.0, -3.0, 2.0], [3.0, 3.0, -7.0, -7.0, 2.0]], [[-2.0, 2.0, 4.0, 0.0, 6.0], [-5.0, 7.0, 5.0, 4.0, 3.0], [-3.0, 6.0, -5.0, 4.0, -5.0], [1.0, 7.0, 0.0, 6.0, -3.0]], [[-1.0, -2.0, 3.0, 3.0, 1.0], [0.0, -1.0, -1.0, -3.0, -2.0], [-6.0, 0.0, 0.0, -7.0, 5.0], [1.0, 7.0, 6.0, 2.0, -6.0]]], [[[-2.0, -2.0, -1.0, 2.0, 7.0], [-2.0, -1.0, 0.0, -4.0, -6.0], [3.0, 6.0, -4.0, 1.0, 1.0], [3.0, -3.0, -4.0, 1.0, -6.0]], [[-1.0, 1.0, -4.0, -3.0, -1.0], [2.0, -4.0, -5.0, -6.0, -1.0], [2.0, 3.0, -7.0, -7.0, 1.0], [4.0, -1.0, -5.0, 7.0, 2.0]], [[-6.0, 1.0, -5.0, 5.0, 4.0], [4.0, 3.0, 1.0, 7.0, 4.0], [2.0, 0.0, 1.0, 0.0, 1.0], [-5.0, 6.0, -7.0, -6.0, 6.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 2.0], [-4.0, 6.0], [6.0, 2.0]], [[3.0, 3.0], [0.0, -3.0], [-2.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 0.0], [-2.0, -4.0], [0.0, 0.0]], [[-1.0, 3.0], [-5.0, -5.0], [0.0, 0.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, 7.0], [-28.0, -14.0], [9.0, 42.0], [14.0, -6.0], [-5.0, 40.0]], [[6.0, -62.0], [-43.0, 34.0], [-28.0, 30.0], [-60.0, 1.0], [-50.0, -1.0]], [[-19.0, -45.0], [-6.0, 43.0], [6.0, -22.0], [-55.0, 28.0], [51.0, -19.0]], [[21.0, 26.0], [-7.0, 38.0], [38.0, 22.0], [3.0, 26.0], [-54.0, -68.0]]])+(1.-msk_ref)*numpy.array([[[20.0, 7.0], [-22.0, -19.0], [-5.0, 1.0], [-5.0, 21.0], [-29.0, 2.0]], [[-10.0, 4.0], [-2.0, -11.0], [0.0, 5.0], [23.0, 2.0], [23.0, -25.0]], [[-25.0, 11.0], [-36.0, -21.0], [52.0, 43.0], [17.0, 22.0], [10.0, 18.0]], [[-16.0, -15.0], [3.0, -32.0], [8.0, 13.0], [-69.0, -56.0], [8.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-1.0, -3.0, 1.0, 2.0, 3.0], [4.0, 3.0, -3.0, 3.0, -2.0], [7.0, -5.0, 5.0, -7.0, -5.0], [0.0, 0.0, 3.0, 2.0, -6.0]], [[-7.0, 7.0, -1.0, -2.0, -6.0], [0.0, -3.0, -2.0, -1.0, 3.0], [4.0, 0.0, 3.0, -6.0, -5.0], [-3.0, 3.0, -3.0, 0.0, -1.0]], [[-1.0, 3.0, 6.0, -5.0, 5.0], [-7.0, -4.0, -2.0, 3.0, -7.0], [7.0, -2.0, 3.0, -2.0, 0.0], [-4.0, 0.0, -3.0, -4.0, 3.0]]], [[[1.0, -2.0, -5.0, 0.0, 5.0], [0.0, -4.0, 1.0, 1.0, -6.0], [5.0, 0.0, 1.0, 2.0, -6.0], [0.0, 0.0, 7.0, -6.0, 1.0]], [[-2.0, -7.0, 4.0, 4.0, -7.0], [1.0, -1.0, -4.0, 6.0, 1.0], [0.0, 4.0, 6.0, 0.0, -2.0], [4.0, 5.0, 5.0, 3.0, 1.0]], [[-6.0, 4.0, 5.0, 0.0, -4.0], [1.0, -2.0, -6.0, -4.0, 0.0], [-6.0, 6.0, 1.0, 0.0, -4.0], [7.0, 6.0, 2.0, -3.0, 5.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[4.0, -4.0, 3.0, 2.0, 3.0], [-6.0, 3.0, 0.0, 0.0, 1.0], [0.0, 4.0, -5.0, -5.0, 0.0], [-4.0, 4.0, -6.0, -5.0, -7.0]], [[-5.0, 4.0, 0.0, 6.0, -6.0], [-3.0, -2.0, 1.0, -2.0, 7.0], [7.0, -1.0, 2.0, 4.0, 0.0], [-6.0, 3.0, 5.0, -6.0, 5.0]], [[0.0, -5.0, -1.0, 7.0, -4.0], [-6.0, -7.0, -7.0, 4.0, 0.0], [0.0, 7.0, 2.0, 6.0, -6.0], [6.0, 0.0, 6.0, -4.0, -1.0]]], [[[3.0, 1.0, 2.0, 5.0, 0.0], [1.0, 4.0, -3.0, -5.0, -4.0], [-5.0, 4.0, -3.0, 1.0, -1.0], [4.0, -4.0, 3.0, -1.0, -6.0]], [[0.0, 6.0, -6.0, 1.0, 2.0], [-5.0, -4.0, -1.0, 0.0, 1.0], [-7.0, -6.0, 2.0, 0.0, -7.0], [-2.0, 0.0, -7.0, -5.0, -4.0]], [[-1.0, 6.0, 6.0, -5.0, 2.0], [0.0, 1.0, 6.0, -7.0, -5.0], [7.0, -4.0, 6.0, 1.0, -7.0], [5.0, 6.0, 3.0, 4.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -5.0, -1.0, 2.0, 3.0], [-6.0, 5.0, -1.0, 3.0, 7.0], [-6.0, 0.0, 0.0, -6.0, 3.0], [-7.0, -3.0, 6.0, 6.0, -4.0]], [[-1.0, 0.0, -3.0, -5.0, 0.0], [2.0, -4.0, 2.0, 0.0, -3.0], [5.0, 5.0, -6.0, -3.0, 6.0], [3.0, -7.0, 2.0, -5.0, 5.0]], [[1.0, -1.0, 6.0, 3.0, -1.0], [7.0, 6.0, 5.0, 3.0, -4.0], [6.0, -2.0, 0.0, -6.0, 7.0], [3.0, 0.0, 1.0, -2.0, -2.0]]], [[[7.0, 6.0, 2.0, 4.0, 4.0], [-7.0, -7.0, 6.0, 2.0, 7.0], [5.0, -3.0, -1.0, -3.0, 2.0], [-1.0, -4.0, 3.0, 5.0, 6.0]], [[5.0, 7.0, -2.0, -3.0, 3.0], [-5.0, 2.0, 1.0, 6.0, 2.0], [6.0, 4.0, 2.0, 0.0, 0.0], [-4.0, 0.0, -7.0, 1.0, 0.0]], [[5.0, 0.0, 0.0, -4.0, -6.0], [-3.0, -6.0, 0.0, 0.0, -2.0], [1.0, 4.0, -4.0, 5.0, 1.0], [3.0, -6.0, 0.0, 4.0, -5.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[40.0, -66.0, -24.0, -18.0, 27.0], [44.0, 24.0, -37.0, 47.0, -26.0], [-82.0, 36.0, -54.0, -34.0, 61.0], [18.0, -65.0, -36.0, 36.0, -49.0]], [[-57.0, 5.0, 50.0, 20.0, -69.0], [12.0, -20.0, 23.0, -20.0, 27.0], [136.0, 20.0, 51.0, 63.0, 5.0], [14.0, 41.0, 126.0, 14.0, 109.0]], [[-11.0, 11.0, -7.0, -8.0, 3.0], [-64.0, -68.0, -2.0, 16.0, -7.0], [25.0, -17.0, 58.0, 27.0, -94.0], [35.0, 51.0, -3.0, -18.0, 16.0]], [[18.0, 33.0, -13.0, -39.0, 46.0], [4.0, 29.0, 29.0, -16.0, -8.0], [-42.0, -49.0, -16.0, -48.0, 2.0], [-34.0, 2.0, -80.0, 2.0, -35.0]], [[61.0, -122.0, 32.0, 43.0, 3.0], [10.0, 30.0, -73.0, 35.0, -46.0], [-46.0, 131.0, -70.0, -8.0, 42.0], [68.0, -50.0, 34.0, 15.0, -74.0]]], [[[15.0, 31.0, 19.0, -45.0, 44.0], [13.0, 58.0, 54.0, -35.0, 0.0], [0.0, -43.0, -26.0, -61.0, 28.0], [-55.0, 22.0, -70.0, 7.0, -21.0]], [[17.0, -26.0, -1.0, -51.0, 37.0], [16.0, 29.0, 26.0, 24.0, 7.0], [-8.0, -15.0, -31.0, -57.0, 49.0], [-42.0, 7.0, -68.0, 20.0, -12.0]], [[7.0, -45.0, -17.0, -1.0, -9.0], [57.0, 23.0, -23.0, 33.0, 5.0], [-33.0, 28.0, -40.0, -10.0, 81.0], [-6.0, -58.0, 9.0, 30.0, -1.0]], [[24.0, -18.0, -52.0, 52.0, 7.0], [-62.0, -34.0, -55.0, 37.0, 18.0], [-82.0, 18.0, -26.0, -4.0, -33.0], [-16.0, -19.0, -56.0, -68.0, -75.0]], [[-41.0, 55.0, -17.0, -64.0, 6.0], [34.0, 9.0, 69.0, -4.0, 44.0], [44.0, -90.0, 22.0, -26.0, 41.0], [-78.0, 25.0, -40.0, 21.0, 68.0]]], [[[29.0, -78.0, -12.0, 142.0, -43.0], [-91.0, -22.0, -96.0, 37.0, 45.0], [-39.0, 117.0, -64.0, 22.0, -5.0], [-20.0, -16.0, 17.0, -116.0, -90.0]], [[-26.0, 90.0, -1.0, -50.0, 13.0], [22.0, -11.0, 46.0, -50.0, -31.0], [14.0, -82.0, 65.0, 19.0, -58.0], [30.0, 16.0, 8.0, 37.0, 45.0]], [[7.0, 20.0, -16.0, 55.0, -1.0], [-86.0, -31.0, -21.0, -6.0, 23.0], [-19.0, 2.0, 2.0, 7.0, -68.0], [-23.0, 31.0, -33.0, -82.0, -49.0]], [[8.0, 16.0, -15.0, -54.0, 23.0], [74.0, 13.0, 2.0, -6.0, -57.0], [-52.0, -28.0, 13.0, 1.0, 10.0], [60.0, -54.0, 6.0, 77.0, 9.0]], [[-9.0, -42.0, -39.0, -52.0, 3.0], [49.0, -25.0, -9.0, 68.0, 2.0], [-19.0, -11.0, 5.0, -5.0, 48.0], [10.0, -35.0, -11.0, 55.0, 38.0]]], [[[8.0, 74.0, 22.0, -77.0, 56.0], [13.0, 25.0, 63.0, -59.0, -52.0], [0.0, -77.0, 36.0, -29.0, -53.0], [21.0, 33.0, -46.0, 42.0, 1.0]], [[-21.0, 78.0, 6.0, -7.0, 4.0], [-34.0, -20.0, 34.0, -48.0, -4.0], [28.0, -57.0, 52.0, 18.0, -77.0], [2.0, 45.0, -2.0, -19.0, 19.0]], [[46.0, 40.0, 8.0, -3.0, 53.0], [-9.0, 46.0, 4.0, -55.0, -51.0], [-77.0, -16.0, -26.0, -36.0, -38.0], [16.0, -13.0, -59.0, -9.0, -87.0]], [[-7.0, 6.0, -38.0, -36.0, 22.0], [-9.0, -5.0, 25.0, 35.0, 44.0], [-12.0, -50.0, -12.0, -43.0, 30.0], [-77.0, 14.0, -84.0, -15.0, 2.0]], [[-21.0, 42.0, 5.0, -16.0, -12.0], [17.0, -32.0, 4.0, -26.0, -41.0], [16.0, -24.0, 63.0, 50.0, -61.0], [75.0, -1.0, 60.0, 38.0, 44.0]]]])+(1.-msk_ref)*numpy.array([[[[-33.0, -2.0, 22.0, 64.0, 32.0], [6.0, 42.0, -14.0, -16.0, 33.0], [-48.0, -68.0, 61.0, 0.0, -56.0], [-28.0, 84.0, -4.0, 10.0, 7.0]], [[-51.0, -49.0, 10.0, -35.0, -65.0], [90.0, -49.0, 13.0, -46.0, -90.0], [23.0, 23.0, -70.0, 5.0, 54.0], [93.0, -56.0, 42.0, -60.0, 9.0]], [[23.0, -13.0, 20.0, -27.0, -41.0], [34.0, 58.0, 1.0, 35.0, -51.0], [29.0, 34.0, -1.0, 1.0, 34.0], [12.0, -6.0, -33.0, -2.0, -76.0]], [[29.0, 23.0, -34.0, -13.0, 23.0], [-71.0, -4.0, -27.0, 15.0, 48.0], [-28.0, 16.0, 20.0, 24.0, -41.0], [-51.0, 8.0, -25.0, 36.0, -8.0]], [[9.0, -39.0, 69.0, 108.0, 27.0], [17.0, 44.0, 33.0, -8.0, 48.0], [-39.0, -99.0, 33.0, -65.0, 14.0], [-13.0, 37.0, 75.0, 40.0, -2.0]]], [[[27.0, -6.0, -48.0, -20.0, 16.0], [-81.0, -26.0, -38.0, -3.0, 56.0], [-59.0, 22.0, -2.0, 23.0, -36.0], [-50.0, -18.0, 10.0, 43.0, -7.0]], [[-26.0, -42.0, -24.0, 4.0, 6.0], [-13.0, 41.0, -54.0, -17.0, 20.0], [-85.0, -7.0, 28.0, 17.0, -47.0], [-40.0, 40.0, 3.0, 12.0, -33.0]], [[-61.0, -5.0, 7.0, 38.0, 21.0], [31.0, 2.0, -9.0, -37.0, 4.0], [-29.0, -49.0, 27.0, 3.0, -39.0], [6.0, 55.0, 7.0, -27.0, 42.0]], [[39.0, 30.0, 8.0, 22.0, 52.0], [-24.0, 66.0, 22.0, 56.0, 39.0], [32.0, -6.0, 33.0, -56.0, 22.0], [-52.0, 18.0, -20.0, 12.0, 3.0]], [[-59.0, -12.0, -63.0, -67.0, -20.0], [6.0, -20.0, -62.0, -33.0, -35.0], [-39.0, 51.0, -10.0, 63.0, -49.0], [4.0, 9.0, -38.0, -42.0, 1.0]]], [[[50.0, -12.0, 33.0, 59.0, 70.0], [-2.0, 62.0, 66.0, 52.0, 56.0], [39.0, -33.0, -5.0, -141.0, 98.0], [-39.0, -33.0, 72.0, 9.0, 38.0]], [[18.0, 55.0, -15.0, -52.0, -37.0], [-22.0, -65.0, -1.0, 3.0, -31.0], [48.0, 44.0, -16.0, 72.0, -23.0], [31.0, -21.0, -60.0, 2.0, -6.0]], [[72.0, 20.0, -6.0, -14.0, 28.0], [-43.0, 30.0, 28.0, 62.0, 31.0], [45.0, 34.0, -11.0, -55.0, 57.0], [-39.0, -46.0, 0.0, 24.0, -10.0]], [[-24.0, 49.0, 17.0, 18.0, -11.0], [2.0, -37.0, -3.0, -23.0, -9.0], [10.0, -32.0, 34.0, 66.0, -67.0], [23.0, 55.0, -50.0, 2.0, 14.0]], [[-97.0, -25.0, 12.0, 13.0, -21.0], [84.0, 57.0, -43.0, -39.0, -58.0], [-41.0, -31.0, 48.0, 43.0, -61.0], [22.0, 98.0, -44.0, -53.0, -21.0]]], [[[54.0, 32.0, -23.0, -37.0, -26.0], [-75.0, -46.0, -22.0, 12.0, 19.0], [-8.0, 37.0, -2.0, 68.0, -39.0], [-16.0, -21.0, -38.0, 55.0, -42.0]], [[52.0, 35.0, -19.0, -54.0, -21.0], [-37.0, -38.0, 11.0, 30.0, -11.0], [51.0, 59.0, -32.0, 21.0, 24.0], [7.0, -57.0, -29.0, 14.0, -15.0]], [[102.0, 65.0, -8.0, 17.0, 43.0], [-125.0, -42.0, 23.0, 44.0, 97.0], [16.0, -2.0, 13.0, -2.0, -14.0], [-60.0, -28.0, -5.0, 87.0, 11.0]], [[-34.0, -21.0, -44.0, -29.0, 13.0], [-4.0, 52.0, -55.0, 0.0, 0.0], [-51.0, 26.0, 24.0, 15.0, -37.0], [-41.0, 36.0, -31.0, -19.0, -21.0]], [[5.0, 40.0, 27.0, -17.0, -44.0], [28.0, -43.0, 26.0, -1.0, -52.0], [65.0, 10.0, -13.0, 43.0, 4.0], [58.0, -9.0, -39.0, -11.0, -6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, 0.0], [-1.0, -7.0], [1.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-6.0, 7.0], [5.0, -7.0], [-1.0, 4.0]])
      arg1=numpy.array([-5.0, 1.0, -4.0])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-10.0, -35.0])+(1.-msk_ref)*numpy.array([39.0, -58.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, 1.0], [-2.0, 7.0], [6.0, 0.0]])+(1.-msk_arg0)*numpy.array([[5.0, -7.0], [-3.0, 6.0], [1.0, 0.0]])
      arg1=numpy.array([[-3.0, -6.0], [-7.0, 5.0], [-1.0, 3.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[11.0, 14.0], [-52.0, 29.0]])+(1.-msk_ref)*numpy.array([[5.0, -42.0], [-21.0, 72.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 4.0], [5.0, -1.0], [-3.0, -6.0]])+(1.-msk_arg0)*numpy.array([[3.0, -3.0], [7.0, 5.0], [5.0, 6.0]])
      arg1=Symbol(shape=(3,))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([-2.0, 3.0, 2.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-3.0, -23.0])+(1.-msk_ref)*numpy.array([25.0, 33.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 4.0], [-7.0, -7.0], [0.0, -3.0]])+(1.-msk_arg0)*numpy.array([[6.0, 6.0], [1.0, -6.0], [6.0, 6.0]])
      arg1=Symbol(shape=(3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[-2.0, 3.0], [1.0, 0.0], [-3.0, 3.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-1.0, -9.0], [-6.0, 3.0]])+(1.-msk_ref)*numpy.array([[-29.0, 36.0], [-36.0, 36.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, 4.0], [0.0, 0.0], [-5.0, -2.0]])+(1.-msk_arg0)*numpy.array([[2.0, -3.0], [-5.0, -5.0], [-4.0, 5.0]])
      arg1=Data(numpy.array([0.0, 4.0, -4.0]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([20.0, 8.0])+(1.-msk_ref)*numpy.array([-4.0, -40.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 2.0], [4.0, -6.0], [4.0, 1.0]])+(1.-msk_arg0)*numpy.array([[0.0, 6.0], [2.0, 4.0], [2.0, -7.0]])
      arg1=Data(numpy.array([[7.0, -2.0], [4.0, 2.0], [7.0, -7.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[2.0, -8.0], [-3.0, -23.0]])+(1.-msk_ref)*numpy.array([[22.0, -10.0], [9.0, 45.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 2.0], [0.0, 2.0], [6.0, -5.0]])+(1.-msk_arg0)*numpy.array([[-3.0, -4.0], [-6.0, 3.0], [-3.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, -2.0, 7.0])+(1.-msk_arg1)*numpy.array([0.0, 5.0, 1.0])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([46.0, -37.0])+(1.-msk_ref)*numpy.array([-33.0, 17.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 0.0], [-4.0, 4.0], [-7.0, 1.0]])+(1.-msk_arg0)*numpy.array([[1.0, 3.0], [-6.0, 6.0], [-7.0, -1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 1.0], [-3.0, 0.0], [2.0, 0.0]])+(1.-msk_arg1)*numpy.array([[4.0, -3.0], [-3.0, -3.0], [2.0, 0.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-14.0, 3.0], [-10.0, 0.0]])+(1.-msk_ref)*numpy.array([[8.0, 15.0], [-8.0, -27.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 1.0, -6.0, 0.0, -3.0], [5.0, -5.0, 5.0, -7.0, 0.0], [2.0, 5.0, -6.0, -1.0, 7.0], [-3.0, 0.0, 0.0, 4.0, -1.0]], [[0.0, -5.0, -7.0, -2.0, -7.0], [5.0, 0.0, 0.0, -2.0, -4.0], [-6.0, 6.0, 3.0, 0.0, 0.0], [2.0, 2.0, 6.0, 0.0, -7.0]], [[-3.0, 7.0, 2.0, 0.0, -3.0], [-5.0, -3.0, -1.0, -4.0, 5.0], [4.0, -3.0, -3.0, -3.0, 1.0], [-7.0, 3.0, 0.0, -1.0, 1.0]]], [[[5.0, -1.0, 6.0, -3.0, 1.0], [1.0, 3.0, -5.0, 6.0, 5.0], [1.0, 0.0, -2.0, -3.0, 0.0], [-1.0, 5.0, 6.0, 2.0, 4.0]], [[7.0, 5.0, -3.0, -5.0, 6.0], [-4.0, 5.0, 6.0, -4.0, 0.0], [-5.0, -5.0, -3.0, -1.0, -2.0], [0.0, 7.0, -4.0, -4.0, 0.0]], [[-7.0, 7.0, -4.0, 0.0, 0.0], [-2.0, -3.0, 3.0, 3.0, -1.0], [-2.0, 1.0, 6.0, 5.0, -2.0], [-3.0, 2.0, -6.0, 6.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 4.0, 0.0, -3.0, 6.0], [6.0, 2.0, 3.0, -2.0, -2.0], [-3.0, -1.0, 3.0, -4.0, -3.0], [3.0, -4.0, -7.0, -5.0, 6.0]], [[-5.0, -6.0, 4.0, -7.0, -2.0], [-4.0, 4.0, 1.0, 7.0, 2.0], [7.0, -3.0, -4.0, -1.0, -5.0], [1.0, 4.0, -2.0, -6.0, 4.0]], [[-4.0, 2.0, 4.0, 0.0, -1.0], [-1.0, 3.0, 0.0, -3.0, -7.0], [-2.0, -1.0, 6.0, -7.0, -4.0], [-2.0, 5.0, 7.0, -5.0, -4.0]]], [[[7.0, -1.0, 4.0, -4.0, -1.0], [2.0, -1.0, -5.0, 7.0, 0.0], [5.0, 0.0, -7.0, 3.0, 3.0], [4.0, -2.0, 4.0, -3.0, 2.0]], [[6.0, 0.0, -3.0, -7.0, 1.0], [-4.0, -6.0, -7.0, 6.0, -3.0], [-3.0, -3.0, -3.0, 1.0, 5.0], [-3.0, 7.0, -3.0, -4.0, -3.0]], [[6.0, 0.0, 6.0, -6.0, -5.0], [4.0, 7.0, 1.0, -3.0, 5.0], [4.0, -7.0, 0.0, 1.0, 0.0], [-4.0, 5.0, 4.0, -3.0, 2.0]]]])
      arg1=numpy.array([[-1.0, -3.0, -7.0], [7.0, 5.0, -3.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[109.0, -38.0, 52.0, -40.0, 82.0], [8.0, 81.0, -12.0, 54.0, 15.0], [-24.0, -30.0, -29.0, -19.0, -18.0], [48.0, 37.0, 22.0, -21.0, 31.0]])+(1.-msk_ref)*numpy.array([[98.0, -7.0, -45.0, -21.0, 20.0], [-5.0, -93.0, -79.0, 90.0, 15.0], [4.0, 23.0, -97.0, 79.0, 92.0], [33.0, -37.0, -35.0, 26.0, 3.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_array_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, -3.0, 6.0, -2.0, -1.0], [7.0, 3.0, 3.0, -1.0, 0.0], [0.0, 0.0, -7.0, 0.0, 3.0], [-5.0, 2.0, 6.0, 5.0, -7.0]], [[0.0, -4.0, 3.0, 1.0, 3.0], [0.0, 5.0, 0.0, 0.0, 2.0], [-4.0, -5.0, -3.0, 0.0, 4.0], [-5.0, 7.0, 1.0, -7.0, 0.0]], [[-4.0, -7.0, -3.0, -4.0, 5.0], [-6.0, -3.0, -3.0, -4.0, 0.0], [-2.0, 6.0, 0.0, -6.0, 6.0], [0.0, 0.0, -6.0, -6.0, 3.0]]], [[[4.0, 0.0, 0.0, 2.0, 3.0], [4.0, -2.0, -2.0, -4.0, 1.0], [0.0, -1.0, -3.0, 6.0, 3.0], [-3.0, -3.0, 2.0, -5.0, -1.0]], [[5.0, -2.0, 0.0, 1.0, 2.0], [-1.0, 3.0, 4.0, -2.0, -7.0], [4.0, -4.0, -1.0, -6.0, 2.0], [-2.0, 4.0, -2.0, 3.0, 7.0]], [[5.0, -5.0, -3.0, 7.0, -3.0], [-5.0, -4.0, -4.0, 7.0, 7.0], [3.0, 1.0, 0.0, 2.0, 4.0], [0.0, -4.0, 4.0, -1.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 1.0, 1.0, -5.0, -6.0], [-5.0, 1.0, -3.0, -2.0, 0.0], [-1.0, -1.0, 0.0, 5.0, 4.0], [1.0, 0.0, 2.0, -6.0, -7.0]], [[7.0, 6.0, -7.0, 4.0, -2.0], [-6.0, 5.0, -4.0, -3.0, -6.0], [0.0, -3.0, -7.0, -2.0, -2.0], [7.0, 4.0, 5.0, 0.0, 4.0]], [[3.0, 5.0, 7.0, -7.0, -4.0], [1.0, 2.0, 3.0, 6.0, -3.0], [2.0, 5.0, -5.0, 5.0, 7.0], [-7.0, 3.0, 0.0, 1.0, -6.0]]], [[[5.0, 7.0, 4.0, 7.0, -1.0], [-7.0, 1.0, 4.0, 0.0, -2.0], [1.0, 2.0, 2.0, 0.0, -2.0], [-7.0, -1.0, 0.0, -2.0, -4.0]], [[6.0, -4.0, -7.0, 7.0, 6.0], [-5.0, -1.0, 0.0, -3.0, 4.0], [3.0, -3.0, 0.0, 4.0, -1.0], [-6.0, -1.0, 0.0, 2.0, -1.0]], [[-3.0, 7.0, 2.0, 3.0, -7.0], [6.0, 7.0, 4.0, 3.0, 5.0], [-3.0, 5.0, 5.0, -7.0, -1.0], [-6.0, 3.0, -4.0, -7.0, 1.0]]]])
      arg1=numpy.array([[[-2.0, 1.0], [0.0, -2.0], [-6.0, -7.0]], [[4.0, 4.0], [-6.0, 6.0], [-7.0, -4.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-31.0, 57.0], [95.0, 62.0], [27.0, 33.0], [-19.0, 10.0], [-7.0, -6.0]], [[79.0, 79.0], [14.0, 40.0], [8.0, 56.0], [-27.0, -29.0], [-3.0, -70.0]], [[-33.0, 34.0], [-23.0, -64.0], [8.0, -19.0], [82.0, 22.0], [-70.0, -39.0]], [[10.0, -19.0], [-12.0, 16.0], [16.0, 26.0], [-5.0, 63.0], [-71.0, -2.0]]])+(1.-msk_ref)*numpy.array([[[-13.0, 33.0], [-29.0, -70.0], [0.0, -68.0], [17.0, 94.0], [45.0, 86.0]], [[-36.0, -82.0], [-53.0, -53.0], [-24.0, -16.0], [-35.0, -68.0], [-49.0, 29.0]], [[-3.0, 19.0], [-37.0, -60.0], [3.0, 37.0], [-15.0, 26.0], [-45.0, -51.0]], [[90.0, -4.0], [-37.0, -51.0], [24.0, 8.0], [35.0, 19.0], [33.0, 1.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_array_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, 4.0, -4.0, 7.0, -7.0], [-4.0, -6.0, 7.0, 0.0, -5.0], [6.0, 3.0, -7.0, 0.0, 6.0], [3.0, 7.0, 2.0, -5.0, -4.0]], [[-6.0, 0.0, 1.0, -3.0, -2.0], [0.0, 7.0, 1.0, 5.0, 7.0], [1.0, 7.0, 1.0, 2.0, 4.0], [-1.0, 2.0, -5.0, -5.0, 3.0]], [[1.0, 3.0, 0.0, 1.0, -3.0], [6.0, -4.0, 2.0, 0.0, 7.0], [6.0, 2.0, 0.0, -5.0, -4.0], [5.0, -4.0, 2.0, -5.0, 0.0]]], [[[5.0, 4.0, 2.0, -2.0, 2.0], [-6.0, -3.0, -6.0, 7.0, -5.0], [6.0, -7.0, 4.0, -7.0, -3.0], [-2.0, 6.0, 5.0, -5.0, -3.0]], [[6.0, -4.0, -7.0, 1.0, 0.0], [-3.0, 0.0, 7.0, 0.0, 5.0], [2.0, 4.0, -2.0, 7.0, 2.0], [1.0, -4.0, 5.0, 0.0, -2.0]], [[2.0, -7.0, 5.0, 0.0, 5.0], [-5.0, 3.0, -6.0, -2.0, 0.0], [0.0, 7.0, -5.0, -2.0, 0.0], [0.0, -6.0, -5.0, -1.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, 7.0, 6.0, 6.0, -3.0], [7.0, 7.0, -6.0, -1.0, -1.0], [-5.0, -4.0, 6.0, 1.0, -3.0], [-4.0, -7.0, -3.0, 1.0, -4.0]], [[-7.0, 7.0, 0.0, -1.0, -5.0], [-7.0, 2.0, -2.0, 0.0, -5.0], [5.0, -1.0, 6.0, 0.0, 1.0], [-1.0, 6.0, 1.0, -5.0, 5.0]], [[5.0, -1.0, -2.0, 0.0, 5.0], [-1.0, -4.0, -4.0, -7.0, 0.0], [0.0, 5.0, 1.0, -1.0, 1.0], [0.0, 0.0, 5.0, 6.0, 3.0]]], [[[-7.0, -2.0, -7.0, -7.0, -2.0], [-6.0, 1.0, -1.0, -4.0, -2.0], [1.0, -7.0, -4.0, 6.0, -5.0], [-2.0, -5.0, -3.0, 0.0, -6.0]], [[7.0, 5.0, 4.0, 3.0, 2.0], [-4.0, -1.0, 4.0, 5.0, -2.0], [-4.0, -2.0, 6.0, -7.0, -2.0], [7.0, 2.0, -6.0, 4.0, -2.0]], [[5.0, 6.0, -2.0, 0.0, -4.0], [-1.0, 6.0, 1.0, 3.0, -5.0], [-4.0, -7.0, -7.0, 3.0, 3.0], [-1.0, -7.0, 2.0, -6.0, 3.0]]]])
      arg1=numpy.array([[[[-6.0, -4.0, -6.0, 0.0, -4.0], [2.0, 0.0, 6.0, 4.0, -4.0], [0.0, 4.0, -7.0, -6.0, 1.0], [1.0, 3.0, -6.0, -3.0, 5.0]], [[0.0, -1.0, -4.0, -2.0, 5.0], [-4.0, -5.0, 5.0, 0.0, -4.0], [-1.0, 0.0, -1.0, 5.0, -1.0], [4.0, -6.0, 7.0, 4.0, 2.0]], [[-7.0, 0.0, 0.0, -4.0, 4.0], [4.0, 2.0, -6.0, 5.0, 0.0], [-6.0, -7.0, -1.0, 0.0, 5.0], [0.0, 0.0, 5.0, 7.0, -3.0]]], [[[-3.0, -3.0, 0.0, 3.0, 3.0], [-1.0, 7.0, -5.0, -1.0, -4.0], [1.0, 4.0, 1.0, 3.0, 1.0], [7.0, 7.0, 0.0, -3.0, 6.0]], [[-2.0, 2.0, -6.0, 7.0, -3.0], [0.0, -2.0, 1.0, -7.0, 4.0], [-2.0, -4.0, 0.0, -2.0, -6.0], [3.0, 2.0, -2.0, 5.0, -2.0]], [[-1.0, 5.0, -1.0, 4.0, 3.0], [4.0, 0.0, 4.0, -1.0, -3.0], [-4.0, 0.0, 6.0, -4.0, -7.0], [-5.0, -3.0, -7.0, 4.0, 0.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 37.0, 22.0, 73.0, 1.0], [19.0, 55.0, -83.0, -68.0, 46.0], [-15.0, -35.0, 64.0, 1.0, -40.0], [13.0, 59.0, -27.0, 24.0, -27.0]], [[-42.0, -71.0, 7.0, -56.0, -1.0], [-12.0, 42.0, -46.0, 62.0, -27.0], [22.0, 27.0, -69.0, 24.0, 96.0], [55.0, 53.0, 48.0, -51.0, 43.0]], [[27.0, 20.0, 57.0, -25.0, 63.0], [6.0, 23.0, -16.0, 26.0, -39.0], [-5.0, 20.0, 59.0, 29.0, 4.0], [-32.0, -33.0, 10.0, -5.0, 8.0]], [[-45.0, -17.0, -36.0, 3.0, -48.0], [32.0, 1.0, 32.0, 28.0, -4.0], [-7.0, 9.0, -49.0, -65.0, 7.0], [-16.0, 27.0, -60.0, -15.0, 12.0]], [[52.0, 49.0, 45.0, 42.0, 27.0], [0.0, 18.0, -24.0, -50.0, 13.0], [2.0, 1.0, 86.0, 18.0, -53.0], [-26.0, -10.0, -22.0, 6.0, -18.0]]], [[[11.0, 3.0, 47.0, -83.0, 16.0], [2.0, -24.0, -53.0, 46.0, 43.0], [-16.0, -70.0, -14.0, 32.0, 73.0], [-30.0, -45.0, 95.0, 37.0, -68.0]], [[70.0, 41.0, 5.0, 5.0, 43.0], [-41.0, -64.0, 50.0, -44.0, -1.0], [2.0, -8.0, 54.0, 50.0, -57.0], [-14.0, -90.0, 44.0, 39.0, -22.0]], [[-46.0, -27.0, -82.0, -3.0, -72.0], [0.0, -57.0, 48.0, 1.0, 38.0], [-9.0, -38.0, -94.0, -45.0, 10.0], [20.0, 5.0, 3.0, 26.0, -19.0]], [[-19.0, -36.0, -18.0, 3.0, 40.0], [-35.0, 24.0, -18.0, -5.0, -42.0], [10.0, 28.0, -10.0, 54.0, 16.0], [79.0, 25.0, 49.0, -9.0, 52.0]], [[-14.0, 38.0, -28.0, -22.0, 53.0], [-5.0, -66.0, -7.0, -15.0, 32.0], [-64.0, -109.0, 16.0, 40.0, -12.0], [3.0, -82.0, 104.0, 132.0, -72.0]]], [[[-100.0, -39.0, -52.0, 6.0, 17.0], [26.0, 45.0, -23.0, 34.0, -44.0], [-35.0, -2.0, -43.0, -17.0, 29.0], [58.0, 58.0, -3.0, 20.0, 46.0]], [[-26.0, 45.0, -77.0, 13.0, 19.0], [21.0, -88.0, 108.0, -6.0, -17.0], [-62.0, -46.0, 5.0, -40.0, -74.0], [-41.0, -95.0, -16.0, 102.0, -27.0]], [[39.0, -14.0, 55.0, -24.0, 36.0], [-42.0, 27.0, -79.0, -13.0, 15.0], [27.0, -4.0, 22.0, 83.0, 43.0], [44.0, 12.0, 88.0, -17.0, -5.0]], [[44.0, 23.0, -48.0, 36.0, -58.0], [-29.0, -83.0, 74.0, -65.0, 54.0], [15.0, -21.0, -16.0, -17.0, -62.0], [-10.0, -41.0, -11.0, 21.0, -37.0]], [[-3.0, -15.0, -64.0, 13.0, -35.0], [-17.0, -53.0, 97.0, -7.0, -20.0], [13.0, 32.0, -45.0, -29.0, -33.0], [7.0, -23.0, -32.0, -11.0, 28.0]]], [[[-49.0, -3.0, -20.0, -17.0, -6.0], [32.0, -1.0, -6.0, 32.0, 4.0], [-33.0, -35.0, -27.0, -31.0, 21.0], [-12.0, 3.0, -2.0, 33.0, -16.0]], [[-18.0, -86.0, -20.0, -22.0, -22.0], [-40.0, 32.0, 18.0, 36.0, -58.0], [60.0, 96.0, -77.0, 18.0, 57.0], [75.0, 61.0, 2.0, -103.0, 95.0]], [[-46.0, -33.0, -17.0, 32.0, -40.0], [7.0, 54.0, -65.0, -17.0, 27.0], [8.0, -6.0, -36.0, -12.0, 27.0], [57.0, 96.0, -12.0, -22.0, 14.0]], [[81.0, 35.0, 51.0, 11.0, -43.0], [-9.0, -20.0, -4.0, -39.0, 63.0], [34.0, -5.0, 34.0, -6.0, -23.0], [-55.0, -17.0, -23.0, -29.0, -50.0]], [[32.0, 43.0, 19.0, -9.0, 43.0], [3.0, -32.0, 24.0, -4.0, -7.0], [-22.0, -20.0, 52.0, 14.0, -33.0], [-44.0, -70.0, 14.0, 43.0, -28.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, 79.0, -1.0, 42.0, -30.0], [69.0, -18.0, -21.0, -34.0, 81.0], [-64.0, -103.0, 46.0, -72.0, -55.0], [-84.0, -17.0, -55.0, 92.0, -100.0]], [[-45.0, 11.0, -106.0, 43.0, 0.0], [8.0, -61.0, 122.0, -16.0, -46.0], [-37.0, 7.0, -21.0, -47.0, -79.0], [6.0, -43.0, -50.0, 55.0, 30.0]], [[-7.0, -5.0, -58.0, 7.0, -71.0], [3.0, -61.0, 79.0, -5.0, 26.0], [5.0, -6.0, -59.0, -57.0, -21.0], [-21.0, -17.0, -40.0, 1.0, -14.0]], [[-21.0, 4.0, -50.0, 2.0, -59.0], [23.0, -50.0, 69.0, 10.0, 20.0], [-12.0, -16.0, -48.0, -68.0, -18.0], [-38.0, -19.0, -49.0, 14.0, -20.0]], [[-11.0, 7.0, 30.0, -18.0, -17.0], [20.0, 17.0, -77.0, 5.0, 60.0], [-15.0, -63.0, -5.0, -1.0, 41.0], [-11.0, 23.0, 32.0, 24.0, -56.0]]], [[[-8.0, -16.0, 11.0, -32.0, -76.0], [40.0, -1.0, 35.0, 58.0, 11.0], [19.0, 27.0, -53.0, -83.0, 34.0], [-70.0, 16.0, -81.0, -62.0, -4.0]], [[-21.0, -5.0, -50.0, 32.0, -10.0], [13.0, -9.0, 94.0, 8.0, -62.0], [1.0, 64.0, -10.0, -51.0, -50.0], [-11.0, -4.0, -88.0, -25.0, 59.0]], [[58.0, 42.0, 19.0, 49.0, -14.0], [-15.0, -13.0, -9.0, -72.0, 49.0], [13.0, -16.0, 53.0, 11.0, -56.0], [-14.0, -8.0, -13.0, 9.0, -36.0]], [[54.0, 41.0, -27.0, 63.0, -42.0], [-14.0, -52.0, 73.0, -73.0, 31.0], [16.0, 9.0, 28.0, -28.0, -91.0], [-29.0, -30.0, -60.0, 3.0, -18.0]], [[21.0, -14.0, 43.0, -30.0, -36.0], [0.0, 15.0, -43.0, 17.0, 39.0], [27.0, -4.0, -20.0, -1.0, 49.0], [-16.0, 24.0, 10.0, -41.0, -23.0]]], [[[39.0, -16.0, 38.0, -51.0, 48.0], [-47.0, -10.0, -30.0, 11.0, -8.0], [20.0, 0.0, 7.0, 82.0, 43.0], [30.0, -34.0, 101.0, -4.0, -1.0]], [[21.0, -1.0, 47.0, -81.0, -5.0], [-5.0, -30.0, -54.0, 37.0, 61.0], [-4.0, -71.0, -25.0, 30.0, 76.0], [-28.0, -38.0, 95.0, 26.0, -75.0]], [[-36.0, -41.0, -89.0, -14.0, -41.0], [-32.0, -68.0, 58.0, -2.0, 13.0], [0.0, -23.0, -95.0, -2.0, 14.0], [55.0, -13.0, 48.0, 27.0, 3.0]], [[-6.0, -21.0, 33.0, -15.0, 40.0], [4.0, 54.0, -13.0, 39.0, -65.0], [14.0, 63.0, 18.0, 14.0, 23.0], [7.0, 22.0, -18.0, -51.0, 58.0]], [[27.0, 37.0, 23.0, -23.0, 21.0], [11.0, -34.0, 16.0, 9.0, 11.0], [-20.0, -31.0, 32.0, 0.0, -13.0], [-55.0, -63.0, 13.0, 37.0, -42.0]]], [[[17.0, 32.0, -13.0, 41.0, -19.0], [-6.0, -23.0, -16.0, -62.0, 59.0], [-11.0, -52.0, 21.0, 3.0, -40.0], [4.0, -3.0, 10.0, 45.0, -48.0]], [[60.0, 6.0, 13.0, -41.0, 16.0], [-61.0, -69.0, -13.0, -30.0, 53.0], [13.0, -56.0, -4.0, 81.0, 19.0], [23.0, -67.0, 129.0, 42.0, -57.0]], [[2.0, 18.0, 48.0, -65.0, 52.0], [21.0, -4.0, -26.0, 56.0, -10.0], [-30.0, -35.0, 24.0, 18.0, 40.0], [-48.0, -54.0, 48.0, 35.0, -34.0]], [[-50.0, -21.0, -4.0, -10.0, -35.0], [22.0, 29.0, -75.0, 12.0, 50.0], [-15.0, -54.0, -44.0, -15.0, 54.0], [23.0, 59.0, 23.0, 15.0, -31.0]], [[22.0, 40.0, 13.0, -42.0, 50.0], [2.0, -57.0, 23.0, 16.0, 3.0], [-37.0, -53.0, 32.0, 23.0, -9.0], [-47.0, -97.0, 57.0, 73.0, -51.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 2.0, -3.0, -3.0, 0.0], [-1.0, 5.0, 2.0, 0.0, 4.0], [0.0, 6.0, 1.0, -5.0, 4.0], [1.0, -2.0, -2.0, -2.0, 1.0]], [[-1.0, 6.0, -5.0, -2.0, -6.0], [0.0, 5.0, 7.0, -3.0, -2.0], [2.0, 4.0, -5.0, 2.0, -1.0], [-2.0, 0.0, -2.0, -2.0, -5.0]], [[3.0, 4.0, -1.0, 0.0, -7.0], [-6.0, -5.0, 5.0, -4.0, -5.0], [-5.0, 2.0, -7.0, -1.0, 0.0], [-6.0, -7.0, -2.0, 0.0, 5.0]]], [[[7.0, 3.0, -2.0, -2.0, 0.0], [5.0, -1.0, 1.0, -4.0, 0.0], [2.0, 1.0, -5.0, 5.0, -2.0], [5.0, 3.0, 0.0, 3.0, 1.0]], [[-5.0, 0.0, 1.0, -7.0, 3.0], [0.0, -7.0, -2.0, 7.0, -2.0], [0.0, 4.0, -7.0, -7.0, 0.0], [-3.0, -3.0, -7.0, -7.0, 6.0]], [[0.0, 4.0, 5.0, 7.0, -2.0], [-5.0, -7.0, 6.0, 0.0, -2.0], [-7.0, -7.0, 3.0, -2.0, 6.0], [1.0, 7.0, -1.0, -2.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 0.0, -3.0, 3.0, 6.0], [-2.0, 0.0, 5.0, 2.0, 0.0], [7.0, -7.0, 1.0, -5.0, 6.0], [7.0, -3.0, 0.0, 0.0, -1.0]], [[3.0, -6.0, 7.0, 3.0, -7.0], [-3.0, 3.0, 1.0, 6.0, 3.0], [3.0, 3.0, -3.0, 0.0, -2.0], [-2.0, 6.0, -3.0, -4.0, 0.0]], [[-2.0, 4.0, 3.0, -2.0, -5.0], [-4.0, -5.0, -3.0, 4.0, 3.0], [6.0, 4.0, -2.0, 0.0, -3.0], [-4.0, 1.0, -6.0, -5.0, 0.0]]], [[[0.0, -5.0, -3.0, -1.0, 3.0], [-3.0, 5.0, -7.0, 0.0, -1.0], [7.0, -3.0, 0.0, -5.0, 3.0], [1.0, -3.0, -5.0, 5.0, -4.0]], [[-7.0, -4.0, 4.0, -5.0, 5.0], [-6.0, -4.0, -4.0, 0.0, 2.0], [0.0, -1.0, 1.0, -4.0, -4.0], [0.0, 4.0, 3.0, 2.0, 4.0]], [[-6.0, 2.0, 1.0, 6.0, -5.0], [-3.0, 6.0, -4.0, 2.0, -4.0], [3.0, -3.0, -3.0, 4.0, 6.0], [3.0, 2.0, 5.0, 2.0, 1.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[2.0, 5.0, -5.0], [0.0, 7.0, -2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-49.0, 6.0, -29.0, -79.0, 30.0], [38.0, 25.0, -12.0, 54.0, 13.0], [49.0, 64.0, -43.0, -40.0, -9.0], [-1.0, -4.0, -51.0, -59.0, -20.0]])+(1.-msk_ref)*numpy.array([[0.0, -82.0, 40.0, -16.0, 47.0], [-35.0, 0.0, 10.0, 10.0, 22.0], [-7.0, -20.0, 10.0, -46.0, -23.0], [18.0, 43.0, 26.0, 15.0, 24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, -3.0, 5.0, 4.0, 1.0], [1.0, 4.0, -6.0, 7.0, -4.0], [6.0, 0.0, 3.0, 4.0, 4.0], [7.0, 0.0, -6.0, 3.0, -5.0]], [[0.0, 7.0, 4.0, 6.0, -6.0], [-2.0, 0.0, 0.0, 0.0, -4.0], [2.0, 0.0, 0.0, 1.0, 0.0], [4.0, 5.0, -4.0, -2.0, -5.0]], [[-5.0, 1.0, -1.0, -2.0, -5.0], [-5.0, 5.0, -7.0, 1.0, -7.0], [7.0, 1.0, 5.0, 1.0, -4.0], [4.0, 6.0, 0.0, 4.0, 6.0]]], [[[0.0, 0.0, 5.0, 5.0, 2.0], [-5.0, 5.0, 6.0, -1.0, 0.0], [5.0, -4.0, 5.0, -4.0, 4.0], [0.0, -4.0, -3.0, 0.0, 7.0]], [[5.0, -3.0, 5.0, -3.0, 1.0], [4.0, 7.0, 5.0, 3.0, 0.0], [5.0, 1.0, -3.0, 7.0, 2.0], [0.0, 4.0, 0.0, -3.0, -5.0]], [[0.0, -5.0, 6.0, -5.0, 0.0], [1.0, -1.0, -6.0, -5.0, 6.0], [5.0, 3.0, -5.0, -4.0, 5.0], [-7.0, 2.0, -5.0, -6.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -1.0, 0.0, -7.0, 1.0], [4.0, -7.0, 3.0, -1.0, 5.0], [7.0, 0.0, -5.0, -1.0, -2.0], [7.0, 0.0, 0.0, 0.0, -5.0]], [[0.0, -6.0, -7.0, -4.0, 0.0], [4.0, 3.0, 2.0, 1.0, -4.0], [2.0, -1.0, 2.0, 0.0, 0.0], [0.0, 4.0, -5.0, 7.0, 6.0]], [[0.0, 1.0, 1.0, 7.0, 7.0], [1.0, -2.0, -7.0, 0.0, 3.0], [-4.0, -3.0, 5.0, 2.0, 6.0], [-7.0, 6.0, 3.0, -6.0, -7.0]]], [[[6.0, -7.0, 0.0, 6.0, -7.0], [0.0, 4.0, 6.0, -2.0, 6.0], [-7.0, -1.0, -6.0, 0.0, 1.0], [-1.0, -5.0, 3.0, 6.0, -2.0]], [[-4.0, -1.0, 2.0, 0.0, 0.0], [6.0, 3.0, 2.0, 4.0, -7.0], [-7.0, -5.0, -3.0, -6.0, 4.0], [5.0, -7.0, 0.0, 0.0, -4.0]], [[-5.0, 0.0, -4.0, -5.0, -6.0], [6.0, 3.0, 0.0, 3.0, -6.0], [0.0, -6.0, 6.0, 4.0, 0.0], [-7.0, 6.0, -2.0, -5.0, -6.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[-7.0, -3.0], [-5.0, 7.0], [5.0, 3.0]], [[2.0, 4.0], [1.0, -2.0], [-3.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-69.0, -46.0], [3.0, 42.0], [-63.0, 50.0], [-46.0, 25.0], [3.0, -54.0]], [[-31.0, -55.0], [17.0, 4.0], [42.0, -19.0], [-28.0, -53.0], [-5.0, -7.0]], [[-17.0, 52.0], [-11.0, 0.0], [26.0, 7.0], [-17.0, -52.0], [-53.0, 13.0]], [[-28.0, -16.0], [-5.0, 39.0], [71.0, -47.0], [24.0, -35.0], [81.0, 66.0]]])+(1.-msk_ref)*numpy.array([[[23.0, 7.0], [27.0, -62.0], [54.0, -70.0], [131.0, 13.0], [32.0, -40.0]], [[-55.0, 37.0], [26.0, 61.0], [-52.0, 4.0], [-7.0, 9.0], [23.0, -26.0]], [[-100.0, -33.0], [1.0, -40.0], [17.0, 56.0], [-1.0, 41.0], [50.0, 20.0]], [[-60.0, -91.0], [-25.0, 70.0], [52.0, -24.0], [-38.0, 30.0], [-20.0, 6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -6.0, -2.0, -3.0, 5.0], [-3.0, -4.0, -1.0, 0.0, -3.0], [0.0, -3.0, -3.0, 4.0, -1.0], [6.0, 1.0, -4.0, -2.0, 7.0]], [[7.0, 1.0, 7.0, 1.0, -3.0], [6.0, -4.0, 1.0, 5.0, 7.0], [-4.0, 6.0, -2.0, 4.0, -6.0], [0.0, 0.0, -1.0, 3.0, 1.0]], [[2.0, 4.0, -7.0, 1.0, 3.0], [-6.0, 6.0, 7.0, 5.0, -3.0], [0.0, 0.0, 1.0, -6.0, -4.0], [2.0, -4.0, -1.0, 6.0, 2.0]]], [[[-4.0, 5.0, 0.0, -5.0, 3.0], [-1.0, -2.0, 5.0, 2.0, 4.0], [-7.0, 6.0, 7.0, 4.0, 4.0], [-2.0, 1.0, -7.0, -3.0, -7.0]], [[-2.0, 1.0, 5.0, -4.0, 3.0], [-4.0, 0.0, 1.0, 2.0, -5.0], [7.0, -7.0, 3.0, -2.0, 7.0], [0.0, -5.0, -4.0, 0.0, -1.0]], [[-3.0, 2.0, 3.0, -3.0, 5.0], [6.0, 0.0, -4.0, -5.0, -3.0], [-5.0, -7.0, -2.0, 1.0, 2.0], [0.0, 1.0, 0.0, 4.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, -5.0, 7.0, 6.0, -4.0], [-3.0, 2.0, 6.0, -3.0, -5.0], [-3.0, -3.0, -2.0, 2.0, 0.0], [-2.0, -3.0, -4.0, -1.0, 1.0]], [[-3.0, -3.0, 1.0, -5.0, -1.0], [4.0, -6.0, -6.0, -4.0, 7.0], [-7.0, -3.0, -4.0, 3.0, -6.0], [-4.0, 1.0, 4.0, -3.0, 4.0]], [[-5.0, 0.0, -3.0, 1.0, -3.0], [0.0, 0.0, 2.0, 0.0, 5.0], [-2.0, 6.0, 7.0, 3.0, 4.0], [-5.0, 5.0, 5.0, 5.0, 5.0]]], [[[0.0, -1.0, 4.0, 0.0, 0.0], [4.0, -1.0, -7.0, -3.0, -6.0], [0.0, -3.0, -2.0, -7.0, 0.0], [2.0, 6.0, -5.0, 4.0, -4.0]], [[0.0, -5.0, -2.0, 3.0, -7.0], [2.0, -3.0, 6.0, 2.0, 4.0], [0.0, 0.0, -2.0, 2.0, 5.0], [5.0, 6.0, 6.0, 0.0, 1.0]], [[2.0, 1.0, 4.0, 6.0, -4.0], [1.0, 7.0, -6.0, -2.0, 6.0], [6.0, 5.0, 2.0, -7.0, -7.0], [-3.0, -5.0, 6.0, 1.0, -1.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=transposed_tensor_mult(arg0,arg1)
      s1=numpy.array([[[[-7.0, -7.0, 0.0, -7.0, 7.0], [0.0, -3.0, 2.0, 7.0, 7.0], [7.0, -3.0, 7.0, -5.0, 2.0], [-5.0, 3.0, 1.0, -7.0, -3.0]], [[0.0, 1.0, 4.0, -4.0, 2.0], [-3.0, 7.0, -2.0, 0.0, 1.0], [-6.0, 4.0, -1.0, 4.0, 6.0], [7.0, -5.0, -4.0, 4.0, -2.0]], [[-4.0, 4.0, 3.0, 4.0, 3.0], [-7.0, 7.0, 3.0, 6.0, -4.0], [7.0, 3.0, 5.0, 7.0, 0.0], [-4.0, 0.0, 3.0, -3.0, -3.0]]], [[[7.0, -5.0, 5.0, -3.0, 3.0], [3.0, 4.0, 3.0, 6.0, 4.0], [5.0, 3.0, -2.0, 7.0, 5.0], [0.0, -1.0, -3.0, -6.0, 0.0]], [[3.0, 3.0, -7.0, -3.0, -6.0], [0.0, 3.0, 0.0, 5.0, 1.0], [7.0, 7.0, -4.0, 6.0, 0.0], [-1.0, -6.0, 5.0, 2.0, 4.0]], [[0.0, 6.0, 0.0, 4.0, 4.0], [-3.0, 5.0, -2.0, -4.0, -6.0], [-2.0, 0.0, -3.0, 3.0, 2.0], [1.0, 0.0, 6.0, -1.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 53.0, 28.0, 28.0, -34.0], [-38.0, 44.0, -26.0, -52.0, -43.0], [-98.0, 26.0, -14.0, 23.0, 4.0], [70.0, -37.0, -44.0, 87.0, -25.0]], [[64.0, 49.0, 34.0, 44.0, -11.0], [-22.0, 86.0, 9.0, 9.0, -48.0], [8.0, 56.0, -43.0, 109.0, 23.0], [22.0, -34.0, 4.0, 4.0, 18.0]], [[57.0, 26.0, -28.0, -45.0, -39.0], [19.0, 36.0, -45.0, -43.0, 8.0], [-76.0, 48.0, -85.0, 28.0, 44.0], [85.0, -71.0, -8.0, 70.0, 48.0]], [[-30.0, 21.0, 10.0, 36.0, -19.0], [-16.0, -24.0, -14.0, -53.0, -30.0], [-67.0, -27.0, 18.0, -42.0, -31.0], [19.0, 15.0, -27.0, 47.0, -27.0]], [[-17.0, -2.0, -9.0, -9.0, 49.0], [-18.0, 31.0, 24.0, 66.0, 5.0], [100.0, 12.0, 20.0, 38.0, 17.0], [-56.0, 9.0, 62.0, -73.0, 19.0]]], [[[26.0, 32.0, 29.0, 12.0, 18.0], [3.0, 23.0, -51.0, -107.0, -35.0], [-144.0, -16.0, -57.0, -16.0, 37.0], [91.0, -14.0, -26.0, 55.0, 29.0]], [[-10.0, 58.0, -8.0, 74.0, -24.0], [-36.0, 18.0, 12.0, -4.0, -64.0], [28.0, 8.0, 10.0, 32.0, -42.0], [-32.0, 10.0, 36.0, 6.0, 2.0]], [[17.0, -10.0, 43.0, -3.0, 9.0], [-25.0, 62.0, 40.0, 86.0, 11.0], [76.0, 50.0, 25.0, 87.0, 21.0], [-21.0, -19.0, -18.0, -34.0, -36.0]], [[0.0, -9.0, 31.0, -32.0, -1.0], [-29.0, 59.0, 21.0, 72.0, 25.0], [39.0, 55.0, 23.0, 66.0, 30.0], [8.0, -39.0, -31.0, 2.0, -42.0]], [[46.0, -37.0, 74.0, -28.0, 14.0], [21.0, 23.0, -11.0, -28.0, 27.0], [-93.0, 5.0, -22.0, 11.0, 50.0], [78.0, -18.0, -95.0, 27.0, -31.0]]], [[[-28.0, 22.0, -100.0, -4.0, -91.0], [6.0, -60.0, -3.0, 13.0, 5.0], [48.0, 12.0, 5.0, -38.0, -69.0], [-40.0, -15.0, 42.0, 45.0, 11.0]], [[42.0, -66.0, 103.0, -28.0, 23.0], [21.0, 19.0, 14.0, 8.0, 44.0], [-62.0, 2.0, 10.0, 18.0, 46.0], [57.0, -3.0, -122.0, 2.0, -66.0]], [[75.0, -15.0, 9.0, -5.0, -27.0], [26.0, 29.0, 26.0, 50.0, 16.0], [58.0, 46.0, -34.0, 75.0, 13.0], [-8.0, -24.0, -10.0, -24.0, 12.0]], [[18.0, -68.0, 32.0, -70.0, 46.0], [39.0, -11.0, -8.0, 2.0, 64.0], [-34.0, -16.0, -9.0, -27.0, 54.0], [35.0, 0.0, -46.0, -23.0, -5.0]], [[72.0, -2.0, -65.0, -10.0, -53.0], [52.0, -20.0, 6.0, 20.0, 14.0], [66.0, 28.0, -63.0, 29.0, -14.0], [-26.0, -19.0, 46.0, -17.0, 65.0]]], [[[-64.0, -24.0, -4.0, -28.0, 42.0], [-20.0, -12.0, 12.0, 42.0, 26.0], [46.0, -18.0, 56.0, -30.0, 2.0], [-38.0, 20.0, 18.0, -36.0, -24.0]], [[1.0, -37.0, 28.0, -7.0, 32.0], [28.0, -37.0, -9.0, -40.0, 16.0], [-53.0, -47.0, 2.0, -53.0, 9.0], [17.0, 32.0, -33.0, -12.0, -6.0]], [[-29.0, 46.0, -14.0, 61.0, -30.0], [-11.0, -42.0, -30.0, -96.0, -57.0], [-92.0, -44.0, -2.0, -64.0, -49.0], [21.0, 24.0, -2.0, 61.0, 1.0]], [[-31.0, 80.0, 15.0, 51.0, 17.0], [-72.0, 77.0, -9.0, -12.0, -71.0], [-13.0, 27.0, 7.0, 55.0, 7.0], [11.0, -18.0, 37.0, 22.0, 2.0]], [[-109.0, 4.0, -18.0, -13.0, 50.0], [-44.0, -21.0, -7.0, 6.0, 1.0], [11.0, -39.0, 70.0, -66.0, -11.0], [-33.0, 29.0, 37.0, -13.0, -23.0]]]])+(1.-msk_ref)*numpy.array([[[[41.0, 10.0, -27.0, 21.0, -34.0], [38.0, -37.0, -19.0, -59.0, -16.0], [-42.0, -18.0, -49.0, -26.0, -20.0], [16.0, 6.0, 6.0, 22.0, 40.0]], [[13.0, 28.0, 18.0, 69.0, -10.0], [3.0, -20.0, -9.0, -70.0, -53.0], [-59.0, -35.0, -13.0, -21.0, -31.0], [10.0, 31.0, -9.0, 18.0, 6.0]], [[-15.0, -62.0, 29.0, -55.0, 82.0], [18.0, -5.0, 7.0, 29.0, 52.0], [20.0, -28.0, 21.0, -24.0, 48.0], [-10.0, 24.0, -4.0, -68.0, -2.0]], [[-37.0, 2.0, -38.0, -3.0, 41.0], [-10.0, -7.0, 13.0, 39.0, 0.0], [88.0, -14.0, 22.0, -7.0, -6.0], [-66.0, 25.0, 80.0, -65.0, 31.0]], [[19.0, -30.0, 36.0, 25.0, -13.0], [36.0, -57.0, -7.0, -65.0, 0.0], [-84.0, -50.0, -2.0, -59.0, -22.0], [28.0, 35.0, -68.0, 23.0, -25.0]]], [[[55.0, 17.0, 22.0, -9.0, -9.0], [-3.0, 64.0, -4.0, 9.0, -5.0], [-13.0, 51.0, -44.0, 74.0, 40.0], [42.0, -45.0, -15.0, 16.0, 14.0]], [[-30.0, 18.0, -8.0, 50.0, 45.0], [-6.0, -26.0, -1.0, -35.0, -41.0], [10.0, -54.0, 13.0, -38.0, -23.0], [-42.0, 55.0, 56.0, -45.0, 29.0]], [[-81.0, -23.0, -95.0, -31.0, -45.0], [1.0, -86.0, 21.0, 66.0, 42.0], [111.0, -15.0, 66.0, -71.0, -71.0], [-92.0, 19.0, 51.0, -12.0, -18.0]], [[6.0, 26.0, -45.0, 32.0, -58.0], [9.0, -35.0, -3.0, -21.0, -23.0], [6.0, -2.0, -13.0, -16.0, -49.0], [-17.0, 2.0, 20.0, 29.0, 15.0]], [[-15.0, 140.0, -15.0, 57.0, -24.0], [-92.0, 117.0, -39.0, -45.0, -104.0], [-56.0, 68.0, -39.0, 88.0, 14.0], [56.0, -68.0, 56.0, 86.0, 32.0]]], [[[29.0, 42.0, -34.0, 65.0, -17.0], [17.0, -24.0, -10.0, -57.0, -56.0], [-5.0, -25.0, -42.0, -9.0, -36.0], [-20.0, 26.0, 55.0, -7.0, 59.0]], [[-24.0, 87.0, -9.0, 86.0, 2.0], [-57.0, 43.0, -1.0, -23.0, -90.0], [14.0, 6.0, 3.0, 39.0, -29.0], [-25.0, 9.0, 66.0, 4.0, 22.0]], [[-34.0, 54.0, 9.0, 78.0, 13.0], [-49.0, 23.0, 15.0, -2.0, -68.0], [31.0, -9.0, 31.0, 23.0, -34.0], [-42.0, 28.0, 43.0, -17.0, -5.0]], [[-69.0, 0.0, -28.0, -27.0, -32.0], [-30.0, -21.0, 0.0, 28.0, 21.0], [10.0, 8.0, 53.0, -35.0, -27.0], [-10.0, -14.0, -12.0, 42.0, -48.0]], [[-1.0, -17.0, -47.0, -3.0, -58.0], [11.0, -34.0, 38.0, 77.0, 25.0], [113.0, 23.0, 27.0, 13.0, -50.0], [-70.0, 0.0, 19.0, -19.0, -15.0]]], [[[63.0, -23.0, -56.0, -23.0, -73.0], [62.0, -49.0, 1.0, 5.0, 33.0], [26.0, 16.0, -50.0, -6.0, -24.0], [-6.0, -18.0, 0.0, 14.0, 34.0]], [[61.0, 0.0, 7.0, -19.0, -42.0], [-5.0, 68.0, 35.0, 95.0, 20.0], [90.0, 88.0, -18.0, 117.0, 20.0], [-9.0, -56.0, -10.0, -9.0, -9.0]], [[-9.0, 131.0, -36.0, 53.0, -32.0], [-80.0, 103.0, -28.0, -22.0, -94.0], [-12.0, 70.0, -39.0, 90.0, 3.0], [28.0, -63.0, 76.0, 65.0, 43.0]], [[15.0, 10.0, 23.0, 31.0, 18.0], [-17.0, 38.0, 29.0, 43.0, -20.0], [64.0, 18.0, 10.0, 59.0, 2.0], [-35.0, 8.0, 20.0, -45.0, -1.0]], [[-52.0, 34.0, 4.0, 2.0, 8.0], [-56.0, 42.0, -1.0, 22.0, -18.0], [7.0, 23.0, 35.0, 21.0, 4.0], [1.0, -19.0, 11.0, 21.0, -27.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, -4.0, -4.0, -2.0, -3.0], [3.0, 4.0, -3.0, 0.0, -7.0], [0.0, 3.0, -4.0, -1.0, 3.0], [0.0, 4.0, 0.0, 0.0, 6.0]], [[-1.0, 4.0, -5.0, 5.0, 6.0], [3.0, -3.0, -6.0, 0.0, 7.0], [-6.0, 0.0, -4.0, -1.0, -6.0], [1.0, -2.0, 0.0, -1.0, 4.0]], [[5.0, 5.0, 3.0, -3.0, 1.0], [7.0, 4.0, 4.0, 5.0, -4.0], [4.0, -5.0, -1.0, -1.0, -6.0], [5.0, 0.0, -6.0, -2.0, -7.0]]], [[[1.0, 0.0, -4.0, -3.0, -3.0], [0.0, -3.0, -2.0, 0.0, -4.0], [2.0, -2.0, -5.0, -5.0, 5.0], [4.0, -6.0, -1.0, 0.0, -4.0]], [[5.0, -6.0, -7.0, -4.0, 2.0], [0.0, -5.0, 2.0, 5.0, 0.0], [4.0, 0.0, 6.0, 5.0, 2.0], [-1.0, 0.0, 0.0, 3.0, -1.0]], [[5.0, -4.0, 5.0, -2.0, -6.0], [5.0, 4.0, 0.0, 0.0, -7.0], [-7.0, 4.0, -7.0, 1.0, -1.0], [-7.0, -3.0, 7.0, 0.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, -5.0, 0.0, 4.0, 2.0], [0.0, -6.0, -3.0, 2.0, 7.0], [6.0, -2.0, -3.0, 5.0, 3.0], [0.0, -4.0, 7.0, 2.0, 4.0]], [[-7.0, -7.0, 2.0, 4.0, 7.0], [0.0, -4.0, -2.0, 7.0, 6.0], [7.0, 2.0, -4.0, 2.0, 3.0], [4.0, 7.0, -6.0, 4.0, 4.0]], [[1.0, -2.0, 2.0, -7.0, 5.0], [6.0, -5.0, -7.0, 0.0, -1.0], [7.0, -4.0, 6.0, -1.0, 0.0], [-5.0, 0.0, 6.0, -1.0, 1.0]]], [[[-2.0, 6.0, -1.0, -5.0, 4.0], [-1.0, 1.0, 3.0, -6.0, -3.0], [-6.0, 6.0, 3.0, 2.0, -2.0], [5.0, -7.0, 4.0, -2.0, 0.0]], [[-6.0, 5.0, 2.0, -5.0, 7.0], [4.0, -4.0, -4.0, -2.0, 5.0], [4.0, -6.0, 2.0, -6.0, 4.0], [-2.0, 1.0, 2.0, -2.0, -6.0]], [[-6.0, -2.0, 0.0, 4.0, -4.0], [-7.0, -5.0, -3.0, -1.0, 1.0], [-3.0, -2.0, -4.0, 6.0, 3.0], [1.0, 0.0, -2.0, 3.0, 4.0]]]])
      arg1=Data(numpy.array([[-2.0, 5.0, -2.0], [-7.0, 0.0, 7.0]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[19.0, -10.0, 40.0, 42.0, 13.0], [30.0, 18.0, -18.0, -10.0, 36.0], [-101.0, 46.0, -24.0, 41.0, -66.0], [-82.0, 3.0, 68.0, -1.0, 92.0]])+(1.-msk_ref)*numpy.array([[-63.0, -77.0, 13.0, 89.0, -35.0], [-54.0, -40.0, -32.0, 66.0, 46.0], [30.0, -34.0, -75.0, 30.0, 44.0], [2.0, 92.0, -98.0, 53.0, 38.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_constData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, -2.0, 5.0, -3.0, -5.0], [-4.0, 7.0, 0.0, 4.0, 0.0], [-6.0, -4.0, 6.0, -5.0, -3.0], [-1.0, 7.0, 4.0, -3.0, 4.0]], [[-6.0, -4.0, -6.0, 6.0, -1.0], [1.0, 7.0, -6.0, 5.0, -2.0], [-5.0, -1.0, 0.0, 4.0, 0.0], [-7.0, -2.0, 3.0, -2.0, -2.0]], [[1.0, -2.0, 4.0, 4.0, -2.0], [1.0, 0.0, -7.0, 3.0, -7.0], [5.0, -4.0, -1.0, 3.0, -3.0], [-7.0, -1.0, -2.0, -1.0, -3.0]]], [[[0.0, 5.0, 3.0, 0.0, -2.0], [5.0, -6.0, 1.0, -3.0, -6.0], [0.0, 3.0, 7.0, 0.0, -7.0], [-6.0, -2.0, 7.0, -2.0, -5.0]], [[6.0, -5.0, -2.0, 4.0, 3.0], [3.0, 5.0, 2.0, -2.0, 4.0], [0.0, 7.0, -6.0, 4.0, 0.0], [0.0, 6.0, 0.0, -7.0, 1.0]], [[2.0, 3.0, -7.0, -2.0, 0.0], [7.0, 6.0, 7.0, -1.0, 6.0], [7.0, -1.0, -4.0, 3.0, 0.0], [1.0, 3.0, 3.0, 7.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 3.0, 3.0, 0.0, 7.0], [6.0, 7.0, -1.0, 2.0, -6.0], [5.0, -4.0, 0.0, -2.0, 4.0], [-7.0, 6.0, -5.0, 2.0, -7.0]], [[-4.0, -6.0, 4.0, -1.0, 7.0], [0.0, -4.0, 2.0, -6.0, -3.0], [3.0, 0.0, 3.0, -6.0, -1.0], [-6.0, 6.0, 5.0, -3.0, -4.0]], [[-7.0, 6.0, 6.0, 5.0, 2.0], [-6.0, 5.0, 3.0, -5.0, 0.0], [-5.0, -4.0, 2.0, -7.0, 3.0], [4.0, 2.0, 4.0, 4.0, -4.0]]], [[[0.0, -4.0, -4.0, -2.0, 0.0], [-2.0, -6.0, -2.0, 5.0, -3.0], [4.0, 3.0, -3.0, -2.0, 3.0], [6.0, -2.0, -6.0, 0.0, 3.0]], [[6.0, -7.0, 0.0, 6.0, -7.0], [4.0, 2.0, 0.0, 6.0, -5.0], [-5.0, 3.0, -3.0, 5.0, 0.0], [-1.0, 0.0, 6.0, 5.0, -2.0]], [[-6.0, -3.0, 0.0, -1.0, -5.0], [0.0, -7.0, -3.0, 4.0, -5.0], [0.0, 5.0, 5.0, -3.0, -2.0], [-1.0, -3.0, -1.0, -1.0, -7.0]]]])
      arg1=Data(numpy.array([[[-4.0, -7.0], [3.0, 5.0], [0.0, -1.0]], [[-1.0, 4.0], [-6.0, -4.0], [2.0, -6.0]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-58.0, -81.0], [27.0, 18.0], [-43.0, -7.0], [2.0, 43.0], [1.0, 12.0]], [[10.0, -2.0], [-19.0, -94.0], [-17.0, -69.0], [12.0, -4.0], [-12.0, -79.0]], [[23.0, -30.0], [-34.0, 17.0], [-3.0, 35.0], [14.0, 18.0], [19.0, -4.0]], [[-9.0, -51.0], [-62.0, -108.0], [-8.0, -1.0], [64.0, -10.0], [-31.0, -35.0]]])+(1.-msk_ref)*numpy.array([[[-44.0, 27.0], [10.0, -27.0], [4.0, -23.0], [-39.0, -36.0], [25.0, 42.0]], [[-46.0, -60.0], [-60.0, -64.0], [6.0, 24.0], [-59.0, -67.0], [38.0, 65.0]], [[15.0, 21.0], [5.0, 2.0], [40.0, -17.0], [-44.0, -19.0], [-26.0, -12.0]], [[8.0, 49.0], [-10.0, -4.0], [3.0, 14.0], [-49.0, -47.0], [11.0, 95.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_constData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, 0.0, 2.0, 3.0, 7.0], [4.0, 0.0, 3.0, 0.0, -4.0], [2.0, -7.0, 3.0, 0.0, 7.0], [5.0, 1.0, -4.0, 5.0, 2.0]], [[5.0, -3.0, -2.0, 3.0, -4.0], [7.0, -1.0, 5.0, -2.0, 0.0], [4.0, 7.0, -5.0, -5.0, -4.0], [-3.0, -6.0, 6.0, -4.0, -6.0]], [[-1.0, -1.0, 0.0, 6.0, 2.0], [-2.0, 2.0, -5.0, 0.0, -7.0], [-1.0, -4.0, -1.0, -3.0, 4.0], [-7.0, -3.0, 7.0, -1.0, 3.0]]], [[[1.0, -4.0, 2.0, -5.0, -5.0], [7.0, -7.0, -2.0, 7.0, -5.0], [-7.0, 2.0, -5.0, 2.0, -6.0], [-4.0, 6.0, 3.0, 1.0, -7.0]], [[1.0, 1.0, -6.0, 2.0, -5.0], [3.0, 2.0, -2.0, 4.0, 6.0], [-7.0, -2.0, -4.0, 1.0, 2.0], [7.0, -3.0, 0.0, 1.0, -5.0]], [[6.0, 5.0, 0.0, -1.0, 1.0], [-7.0, 1.0, 2.0, -5.0, 5.0], [-3.0, 0.0, -7.0, -3.0, -7.0], [4.0, 6.0, 5.0, -6.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, -4.0, 5.0, -7.0, -4.0], [2.0, -5.0, 7.0, -3.0, -6.0], [5.0, 3.0, -1.0, 2.0, -2.0], [-6.0, -1.0, 3.0, -2.0, 2.0]], [[-6.0, -3.0, -7.0, 2.0, -7.0], [-5.0, 7.0, 7.0, 7.0, -3.0], [-1.0, 1.0, 0.0, -6.0, -5.0], [7.0, 4.0, -7.0, 3.0, -4.0]], [[5.0, 5.0, 6.0, 6.0, 6.0], [-4.0, -7.0, -7.0, -1.0, -2.0], [4.0, -4.0, 1.0, 7.0, -7.0], [4.0, -4.0, -1.0, 0.0, 0.0]]], [[[0.0, 5.0, 7.0, -4.0, 1.0], [-1.0, -1.0, -7.0, 4.0, 0.0], [7.0, 2.0, -1.0, 1.0, -2.0], [-4.0, -4.0, -3.0, 0.0, -3.0]], [[-7.0, -3.0, 3.0, 5.0, 4.0], [2.0, -7.0, -6.0, 6.0, 5.0], [0.0, 4.0, -7.0, -1.0, 4.0], [-1.0, 0.0, -6.0, 0.0, 7.0]], [[-7.0, -1.0, -5.0, -1.0, -7.0], [6.0, 5.0, 6.0, 2.0, 2.0], [1.0, -4.0, -1.0, 1.0, 2.0], [6.0, -3.0, 2.0, -3.0, 2.0]]]])
      arg1=Data(numpy.array([[[[-3.0, -3.0, -6.0, -5.0, 0.0], [4.0, 6.0, -6.0, 0.0, 1.0], [-5.0, 5.0, -4.0, 3.0, -6.0], [5.0, -3.0, 6.0, -6.0, -4.0]], [[6.0, 2.0, -4.0, 0.0, 7.0], [4.0, -4.0, 3.0, 4.0, -4.0], [-2.0, 4.0, 0.0, 2.0, 7.0], [1.0, -6.0, 7.0, -6.0, 3.0]], [[3.0, 3.0, 3.0, 3.0, -3.0], [-3.0, 0.0, 7.0, 6.0, 0.0], [6.0, 5.0, -2.0, 0.0, -2.0], [-1.0, -1.0, 5.0, -6.0, 6.0]]], [[[5.0, -5.0, 4.0, 7.0, -1.0], [4.0, -6.0, -2.0, -3.0, -6.0], [6.0, 7.0, -2.0, 1.0, 6.0], [-6.0, -7.0, 0.0, 4.0, 1.0]], [[5.0, 3.0, -6.0, 0.0, 4.0], [3.0, -4.0, 7.0, 5.0, -1.0], [6.0, 0.0, -4.0, 6.0, 6.0], [-6.0, -4.0, 0.0, -5.0, 5.0]], [[1.0, 4.0, -3.0, 4.0, 6.0], [2.0, -1.0, 4.0, 4.0, 4.0], [-7.0, 0.0, -4.0, 0.0, -1.0], [6.0, -1.0, 3.0, -6.0, 5.0]]]]),self.functionspace)
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[46.0, 32.0, -37.0, 33.0, 77.0], [38.0, -42.0, 43.0, 40.0, -4.0], [-41.0, 17.0, -24.0, 14.0, 49.0], [25.0, -43.0, 42.0, -55.0, 49.0]], [[-31.0, 34.0, -28.0, -11.0, 20.0], [-12.0, 27.0, 19.0, 19.0, 55.0], [-53.0, -45.0, -14.0, -4.0, -42.0], [46.0, 38.0, -11.0, -27.0, 11.0]], [[-38.0, -38.0, 40.0, 4.0, -40.0], [-10.0, 32.0, -64.0, -44.0, 4.0], [-30.0, 16.0, 12.0, -32.0, -50.0], [32.0, 16.0, -2.0, 38.0, -42.0]], [[11.0, 42.0, -41.0, -36.0, 10.0], [-10.0, 29.0, 53.0, 69.0, 15.0], [4.0, 22.0, -18.0, 22.0, -26.0], [24.0, -5.0, 66.0, -96.0, 33.0]], [[-88.0, -9.0, -13.0, -60.0, -43.0], [-27.0, 107.0, -61.0, -10.0, 62.0], [-82.0, -6.0, -6.0, -22.0, -135.0], [95.0, 55.0, 27.0, -31.0, -53.0]]], [[[67.0, -58.0, -27.0, -5.0, 18.0], [73.0, -51.0, -38.0, -18.0, -97.0], [63.0, 87.0, -10.0, 51.0, 96.0], [-73.0, -106.0, 42.0, 1.0, -20.0]], [[-24.0, 49.0, -33.0, -39.0, 8.0], [-30.0, 37.0, 43.0, 43.0, 48.0], [-23.0, -43.0, -2.0, 3.0, -42.0], [33.0, 44.0, 6.0, -50.0, 17.0]], [[-12.0, -2.0, -55.0, -36.0, 56.0], [37.0, 16.0, -40.0, -6.0, 5.0], [-93.0, -4.0, 2.0, 5.0, 1.0], [61.0, -14.0, 34.0, -28.0, -29.0]], [[38.0, -47.0, 27.0, 29.0, -35.0], [22.0, -45.0, -12.0, -29.0, -58.0], [105.0, 41.0, -10.0, 27.0, 57.0], [-98.0, -48.0, -29.0, 50.0, -4.0]], [[1.0, 54.0, -68.0, -16.0, 80.0], [13.0, -23.0, 47.0, 23.0, 40.0], [-51.0, -90.0, -4.0, 19.0, 39.0], [11.0, 25.0, -44.0, -14.0, 24.0]]], [[[-58.0, 1.0, -8.0, -74.0, -8.0], [-28.0, 69.0, -54.0, -16.0, 23.0], [-87.0, -28.0, 48.0, -35.0, -63.0], [81.0, 51.0, 26.0, -5.0, -59.0]], [[51.0, 7.0, 22.0, 37.0, 51.0], [14.0, -74.0, 17.0, -12.0, -45.0], [-3.0, -13.0, 40.0, -17.0, 99.0], [-24.0, -23.0, -13.0, 42.0, 17.0]], [[-94.0, -37.0, 24.0, -81.0, -85.0], [-51.0, 91.0, -86.0, -59.0, 29.0], [-16.0, -45.0, 44.0, -30.0, -98.0], [23.0, 80.0, -43.0, 60.0, -93.0]], [[-27.0, -38.0, 22.0, -7.0, -42.0], [-6.0, 7.0, -45.0, -51.0, -5.0], [31.0, -21.0, 10.0, -2.0, -8.0], [-38.0, 18.0, -59.0, 69.0, -41.0]], [[-60.0, -9.0, -29.0, -93.0, -68.0], [-32.0, 93.0, -28.0, 8.0, 29.0], [22.0, -3.0, -4.0, 19.0, -95.0], [9.0, 40.0, 13.0, -34.0, -47.0]]], [[[-35.0, 15.0, -109.0, -58.0, 56.0], [42.0, 34.0, -15.0, 9.0, 50.0], [-71.0, -50.0, -42.0, 47.0, -23.0], [35.0, 6.0, -14.0, -45.0, -20.0]], [[-27.0, -39.0, 33.0, 52.0, -15.0], [16.0, 0.0, -54.0, -51.0, 16.0], [-35.0, 8.0, -22.0, -21.0, -30.0], [20.0, 0.0, -33.0, 51.0, -19.0]], [[89.0, 50.0, 18.0, 82.0, 48.0], [9.0, -71.0, 105.0, 77.0, -26.0], [33.0, 60.0, -24.0, 3.0, 65.0], [-9.0, -57.0, 68.0, -72.0, 104.0]], [[-38.0, -52.0, -1.0, -45.0, -58.0], [2.0, 42.0, -68.0, -44.0, -10.0], [31.0, 11.0, 0.0, 14.0, -38.0], [-26.0, 5.0, -21.0, 35.0, -62.0]], [[-98.0, -9.0, 38.0, -70.0, -94.0], [-78.0, 103.0, -50.0, -30.0, 53.0], [-17.0, -48.0, 40.0, -43.0, -127.0], [43.0, 101.0, -30.0, 33.0, -65.0]]]])+(1.-msk_ref)*numpy.array([[[[-66.0, -49.0, 96.0, -18.0, -127.0], [-70.0, 65.0, -66.0, -57.0, 4.0], [44.0, 6.0, 42.0, -51.0, -93.0], [-6.0, 63.0, -32.0, 77.0, -62.0]], [[18.0, -17.0, 92.0, 66.0, -59.0], [-34.0, -29.0, 15.0, -16.0, -23.0], [75.0, 28.0, 12.0, -31.0, 6.0], [-46.0, 3.0, -23.0, 53.0, 22.0]], [[6.0, -57.0, 41.0, 22.0, -92.0], [1.0, 9.0, -22.0, -18.0, -32.0], [120.0, 76.0, -38.0, 26.0, -26.0], [-78.0, -35.0, -4.0, 19.0, -8.0]], [[55.0, 74.0, 9.0, 21.0, 14.0], [-41.0, -45.0, 129.0, 77.0, 0.0], [80.0, -25.0, 8.0, 9.0, 51.0], [-51.0, 12.0, -1.0, -41.0, 86.0]], [[6.0, -5.0, 71.0, 17.0, -94.0], [-60.0, -11.0, 43.0, -3.0, -14.0], [149.0, -11.0, 14.0, -1.0, 0.0], [-105.0, 32.0, -64.0, 56.0, 17.0]]], [[[-37.0, 7.0, -38.0, -5.0, 22.0], [14.0, 24.0, -15.0, -7.0, 50.0], [-60.0, -37.0, -30.0, 7.0, -39.0], [39.0, 21.0, -25.0, -8.0, -8.0]], [[1.0, 12.0, 4.0, 17.0, 73.0], [14.0, -29.0, -25.0, -26.0, 0.0], [-114.0, -39.0, 44.0, -44.0, 40.0], [67.0, 10.0, -1.0, 31.0, -12.0]], [[-59.0, 13.0, -101.0, -81.0, 89.0], [43.0, 74.0, -74.0, 1.0, 51.0], [-211.0, -21.0, 0.0, -8.0, -63.0], [163.0, 11.0, 74.0, -76.0, -56.0]], [[100.0, 26.0, -39.0, 48.0, 84.0], [57.0, -96.0, 74.0, 48.0, -53.0], [41.0, 36.0, -26.0, 45.0, 127.0], [-55.0, -86.0, 32.0, -44.0, 71.0]], [[21.0, 29.0, 6.0, 32.0, 17.0], [-11.0, -46.0, 56.0, 9.0, 9.0], [40.0, -52.0, 0.0, 6.0, 47.0], [-49.0, 16.0, -61.0, 29.0, 38.0]]], [[[27.0, -36.0, 11.0, 40.0, -20.0], [34.0, -9.0, -15.0, 3.0, -29.0], [36.0, 90.0, -46.0, 20.0, -4.0], [-16.0, -63.0, 46.0, -26.0, 13.0]], [[11.0, -33.0, -38.0, -29.0, 9.0], [40.0, -10.0, -35.0, -22.0, -33.0], [23.0, 13.0, -8.0, 37.0, 37.0], [-40.0, -37.0, -7.0, 12.0, -31.0]], [[-35.0, -14.0, 50.0, -3.0, -36.0], [-34.0, 29.0, -38.0, -30.0, 8.0], [-30.0, -7.0, 36.0, -46.0, -43.0], [36.0, 38.0, -4.0, 37.0, -31.0]], [[-20.0, -1.0, 40.0, 22.0, -62.0], [-34.0, 33.0, 14.0, 14.0, 25.0], [37.0, 28.0, -24.0, -11.0, -69.0], [3.0, 19.0, 8.0, -15.0, 17.0]], [[-33.0, 5.0, -27.0, -17.0, 16.0], [1.0, 2.0, -12.0, -28.0, 34.0], [-24.0, -79.0, 2.0, 6.0, 1.0], [-8.0, 39.0, -76.0, 44.0, -21.0]]], [[[53.0, 85.0, -8.0, 38.0, 73.0], [-15.0, -42.0, 110.0, 83.0, 15.0], [-32.0, -10.0, 4.0, -14.0, 41.0], [39.0, -2.0, 51.0, -77.0, 90.0]], [[-8.0, 7.0, -29.0, -47.0, 26.0], [2.0, 5.0, -14.0, -8.0, -5.0], [-30.0, -37.0, 32.0, 1.0, 21.0], [9.0, 14.0, -7.0, 8.0, -27.0]], [[-97.0, -21.0, 25.0, -31.0, -55.0], [-39.0, 86.0, -74.0, -47.0, 63.0], [-75.0, -39.0, 12.0, -44.0, -121.0], [75.0, 77.0, -30.0, 36.0, -62.0]], [[21.0, 0.0, 9.0, -2.0, 3.0], [-2.0, -21.0, 9.0, 0.0, -26.0], [25.0, 2.0, 20.0, 0.0, 36.0], [-25.0, -9.0, 0.0, 12.0, 2.0]], [[-8.0, 30.0, -56.0, -23.0, 15.0], [5.0, 16.0, 39.0, 36.0, 37.0], [8.0, -27.0, -38.0, 37.0, -18.0], [-6.0, 9.0, -10.0, -47.0, 22.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 1.0, 4.0, 2.0, 0.0], [5.0, -7.0, 5.0, -6.0, 3.0], [0.0, 3.0, 4.0, 3.0, 0.0], [5.0, 6.0, 2.0, 1.0, 5.0]], [[6.0, -6.0, 0.0, -4.0, 1.0], [3.0, -6.0, 0.0, 2.0, 5.0], [0.0, 2.0, 6.0, 0.0, -4.0], [0.0, -2.0, -7.0, -5.0, 1.0]], [[0.0, 6.0, -4.0, 0.0, -2.0], [-6.0, 2.0, -3.0, 7.0, 0.0], [-3.0, 7.0, -2.0, 5.0, 0.0], [0.0, 6.0, -4.0, 0.0, -7.0]]], [[[-6.0, 5.0, 5.0, -3.0, 0.0], [0.0, 1.0, -3.0, 7.0, -1.0], [-1.0, 0.0, 1.0, 7.0, 6.0], [-7.0, -2.0, 6.0, 1.0, 7.0]], [[-3.0, 0.0, -2.0, 6.0, 4.0], [-6.0, -4.0, -5.0, -6.0, -1.0], [-2.0, 2.0, -4.0, 4.0, -2.0], [6.0, -4.0, 5.0, -2.0, -3.0]], [[-4.0, 1.0, -7.0, -4.0, -6.0], [-1.0, 1.0, 3.0, -3.0, -7.0], [6.0, 3.0, 7.0, -5.0, -6.0], [0.0, -2.0, 0.0, -6.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, -6.0, -6.0, 0.0, 6.0], [5.0, 5.0, -7.0, -6.0, 6.0], [-4.0, -3.0, -5.0, 7.0, 6.0], [5.0, 3.0, 7.0, -1.0, -6.0]], [[-4.0, 7.0, 6.0, -1.0, 7.0], [5.0, -1.0, 3.0, 3.0, -3.0], [0.0, 0.0, 7.0, 7.0, 1.0], [3.0, -2.0, 0.0, -7.0, 4.0]], [[1.0, 3.0, 0.0, 4.0, 7.0], [-3.0, 0.0, 1.0, -5.0, -6.0], [7.0, -7.0, 4.0, 7.0, -1.0], [-1.0, 0.0, -4.0, -7.0, 5.0]]], [[[-5.0, 7.0, 0.0, -6.0, 0.0], [4.0, -5.0, -2.0, -6.0, 0.0], [1.0, -2.0, 7.0, 4.0, 5.0], [-6.0, -6.0, -7.0, -3.0, -4.0]], [[-1.0, -2.0, 0.0, -5.0, 0.0], [-3.0, 0.0, 0.0, 2.0, 1.0], [-4.0, 0.0, 3.0, 0.0, 2.0], [-5.0, -3.0, 3.0, 0.0, -5.0]], [[4.0, -7.0, -3.0, 2.0, -5.0], [2.0, 3.0, -2.0, 3.0, -4.0], [-3.0, 4.0, -4.0, -4.0, -3.0], [7.0, 2.0, -5.0, 0.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, -1.0, 5.0], [-5.0, 4.0, 0.0]])+(1.-msk_arg1)*numpy.array([[5.0, 2.0, -7.0], [5.0, 7.0, 6.0]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[61.0, 4.0, -81.0, 29.0, 5.0], [-92.0, 44.0, -55.0, 16.0, -25.0], [-18.0, 20.0, -65.0, -15.0, -34.0], [24.0, -16.0, -37.0, -15.0, -118.0]])+(1.-msk_ref)*numpy.array([[-38.0, -58.0, -36.0, -83.0, -35.0], [67.0, 16.0, -58.0, 13.0, 49.0], [-110.0, 48.0, -7.0, -4.0, 60.0], [15.0, -28.0, 19.0, 15.0, -124.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 2.0, -4.0, -4.0, 0.0], [4.0, 3.0, 0.0, -3.0, -5.0], [-7.0, 5.0, 4.0, 3.0, -2.0], [-3.0, -3.0, -4.0, -5.0, 6.0]], [[6.0, 7.0, -7.0, 2.0, 6.0], [-4.0, 6.0, 1.0, 6.0, -5.0], [-3.0, 4.0, 0.0, 4.0, 3.0], [-5.0, 2.0, 1.0, -5.0, -7.0]], [[-5.0, -7.0, -2.0, -4.0, 3.0], [2.0, 3.0, 0.0, -4.0, 4.0], [-4.0, -6.0, -3.0, -3.0, 7.0], [1.0, -4.0, 6.0, -5.0, -6.0]]], [[[-7.0, 0.0, 7.0, -1.0, -6.0], [-3.0, -7.0, -6.0, 0.0, -2.0], [1.0, -1.0, -3.0, 1.0, 1.0], [4.0, 3.0, 0.0, -6.0, -1.0]], [[7.0, -3.0, 0.0, -5.0, -4.0], [5.0, -3.0, -5.0, -7.0, -5.0], [6.0, 1.0, -5.0, 0.0, 3.0], [3.0, 3.0, 0.0, 2.0, -4.0]], [[4.0, 4.0, -5.0, -2.0, 6.0], [6.0, 7.0, 0.0, 3.0, -6.0], [5.0, 1.0, 3.0, -1.0, -2.0], [-2.0, -3.0, 3.0, -4.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, -4.0, -7.0, -6.0, 2.0], [-1.0, 1.0, 3.0, -2.0, -5.0], [-3.0, -3.0, -3.0, 3.0, 0.0], [-6.0, -1.0, -2.0, 7.0, 3.0]], [[5.0, -7.0, 3.0, -6.0, 4.0], [3.0, 7.0, 0.0, 0.0, -2.0], [3.0, -5.0, 5.0, -4.0, 2.0], [1.0, 4.0, -2.0, -6.0, 0.0]], [[1.0, 0.0, -5.0, -6.0, 7.0], [4.0, 4.0, -1.0, 5.0, 1.0], [0.0, -3.0, 6.0, -3.0, -3.0], [4.0, -1.0, 3.0, 0.0, 5.0]]], [[[-6.0, -6.0, 1.0, -1.0, 5.0], [-7.0, 2.0, -1.0, -2.0, -2.0], [-7.0, 1.0, -6.0, -4.0, 5.0], [2.0, -5.0, -3.0, 0.0, -7.0]], [[4.0, 1.0, 1.0, 0.0, 6.0], [-6.0, 4.0, 0.0, 3.0, -5.0], [4.0, -1.0, 5.0, -2.0, -1.0], [4.0, -7.0, 2.0, 1.0, 4.0]], [[7.0, 5.0, 5.0, 5.0, -7.0], [-2.0, 5.0, -6.0, -5.0, -7.0], [-3.0, -4.0, -3.0, 3.0, 4.0], [6.0, 2.0, 2.0, -6.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, 4.0], [-6.0, 5.0], [2.0, 1.0]], [[6.0, 0.0], [2.0, -4.0], [0.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 6.0], [6.0, -4.0], [2.0, 5.0]], [[-4.0, 7.0], [-3.0, -7.0], [-5.0, -7.0]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-60.0, 9.0], [-58.0, 32.0], [72.0, -33.0], [-44.0, 18.0], [-74.0, 25.0]], [[28.0, -46.0], [-72.0, 29.0], [-52.0, 25.0], [-64.0, 30.0], [6.0, 3.0]], [[14.0, -91.0], [-30.0, 26.0], [-26.0, 21.0], [-18.0, 33.0], [4.0, 10.0]], [[56.0, -40.0], [-2.0, -6.0], [-2.0, -17.0], [-22.0, -42.0], [28.0, -1.0]]])+(1.-msk_ref)*numpy.array([[[9.0, -104.0], [-46.0, -80.0], [-24.0, -114.0], [-69.0, -84.0], [35.0, 73.0]], [[82.0, 9.0], [5.0, -51.0], [32.0, 48.0], [34.0, 13.0], [48.0, 53.0]], [[49.0, -86.0], [-17.0, 29.0], [66.0, -64.0], [-23.0, -16.0], [-31.0, -9.0]], [[-36.0, -76.0], [53.0, -27.0], [-10.0, -38.0], [-9.0, 101.0], [1.0, -69.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_transposed_tensor_mult_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 0.0, 3.0, 2.0, -2.0], [0.0, 7.0, 7.0, -1.0, -4.0], [2.0, 3.0, 0.0, 7.0, 4.0], [-5.0, 4.0, 0.0, 7.0, 2.0]], [[-3.0, -4.0, -7.0, 0.0, -4.0], [3.0, -1.0, -7.0, 4.0, 1.0], [1.0, 0.0, -5.0, 0.0, 5.0], [0.0, 2.0, 7.0, 6.0, -3.0]], [[6.0, 5.0, -2.0, 5.0, 0.0], [0.0, -3.0, 3.0, -4.0, 0.0], [-4.0, 2.0, -4.0, -4.0, -6.0], [4.0, 3.0, -2.0, -1.0, 3.0]]], [[[-3.0, -6.0, 2.0, 5.0, 1.0], [1.0, 0.0, -7.0, 4.0, 5.0], [-5.0, 4.0, 5.0, -3.0, -4.0], [-1.0, 5.0, 2.0, -5.0, -4.0]], [[1.0, 6.0, -4.0, 5.0, 2.0], [-2.0, 3.0, 6.0, 0.0, -7.0], [-4.0, -5.0, 6.0, 1.0, -5.0], [-7.0, 1.0, -4.0, -5.0, 0.0]], [[6.0, 4.0, -6.0, -4.0, -4.0], [-6.0, 6.0, -3.0, 3.0, 6.0], [2.0, 4.0, -2.0, 7.0, -5.0], [-4.0, 7.0, -2.0, -1.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 7.0, 5.0, 3.0, -3.0], [5.0, 3.0, -2.0, 6.0, 0.0], [5.0, 0.0, -1.0, -4.0, -5.0], [2.0, -6.0, -5.0, -3.0, 7.0]], [[-2.0, -4.0, 3.0, 7.0, 0.0], [7.0, 0.0, 4.0, 0.0, 1.0], [1.0, 0.0, -3.0, -4.0, 5.0], [-6.0, 6.0, 3.0, -5.0, -5.0]], [[4.0, 6.0, -7.0, -3.0, 3.0], [-2.0, -2.0, 2.0, -5.0, 2.0], [4.0, 0.0, 4.0, -7.0, -4.0], [-2.0, -4.0, 4.0, -4.0, -3.0]]], [[[-3.0, 3.0, -6.0, 1.0, -2.0], [-4.0, 6.0, -3.0, -5.0, 1.0], [2.0, -2.0, 0.0, -7.0, -4.0], [5.0, 6.0, 5.0, -2.0, -6.0]], [[-4.0, 0.0, 2.0, -2.0, -5.0], [4.0, 6.0, 6.0, -6.0, -7.0], [-7.0, 5.0, 2.0, -6.0, -4.0], [4.0, 6.0, -4.0, -1.0, 3.0]], [[1.0, 2.0, 4.0, 7.0, 0.0], [-6.0, 0.0, 6.0, -7.0, -3.0], [-4.0, 5.0, 1.0, -4.0, 5.0], [3.0, 1.0, -4.0, -4.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, 2.0, -4.0, -2.0, 0.0], [-1.0, 0.0, -2.0, 5.0, -6.0], [1.0, 4.0, 6.0, -4.0, 0.0], [7.0, 6.0, -6.0, 3.0, 6.0]], [[-1.0, 7.0, -2.0, 7.0, 2.0], [-6.0, 6.0, -6.0, 2.0, -3.0], [1.0, -6.0, 4.0, -2.0, 0.0], [-5.0, 0.0, 0.0, 7.0, -6.0]], [[0.0, -4.0, -1.0, 5.0, 0.0], [-4.0, 2.0, 2.0, -3.0, 0.0], [5.0, -7.0, 0.0, -7.0, -7.0], [7.0, 3.0, -7.0, -6.0, -1.0]]], [[[-1.0, -5.0, -5.0, -5.0, 1.0], [4.0, -3.0, 1.0, -4.0, -5.0], [0.0, -7.0, 0.0, 3.0, 2.0], [-2.0, -1.0, 0.0, -5.0, -3.0]], [[-1.0, 1.0, 3.0, -4.0, -5.0], [-4.0, 5.0, -3.0, 0.0, 5.0], [7.0, -2.0, -2.0, -1.0, -7.0], [-5.0, -2.0, -2.0, 0.0, 6.0]], [[-4.0, 5.0, -3.0, -1.0, -2.0], [2.0, -4.0, -4.0, 5.0, 2.0], [5.0, 3.0, -2.0, 4.0, -3.0], [-7.0, -2.0, 5.0, -4.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[2.0, -6.0, 0.0, 2.0, -7.0], [-4.0, -6.0, -2.0, 5.0, -7.0], [0.0, -7.0, -6.0, -7.0, 1.0], [0.0, 2.0, 0.0, -1.0, 5.0]], [[1.0, 7.0, 2.0, 5.0, 3.0], [6.0, 2.0, -2.0, 0.0, -2.0], [4.0, 0.0, 5.0, 4.0, 6.0], [-4.0, 4.0, 5.0, 5.0, -2.0]], [[-2.0, 2.0, 5.0, -5.0, 4.0], [4.0, 1.0, -7.0, -6.0, -6.0], [2.0, 4.0, -5.0, 7.0, 7.0], [-2.0, -4.0, -2.0, -4.0, 2.0]]], [[[0.0, -6.0, 1.0, 5.0, 1.0], [-2.0, 0.0, -1.0, 6.0, 3.0], [4.0, 3.0, 6.0, 0.0, 0.0], [7.0, 2.0, 2.0, -7.0, -6.0]], [[-5.0, -1.0, 0.0, -2.0, -2.0], [-3.0, 4.0, -6.0, -3.0, 2.0], [6.0, 6.0, 3.0, -5.0, 7.0], [-3.0, -1.0, 0.0, -6.0, -6.0]], [[0.0, 0.0, -4.0, 3.0, 5.0], [3.0, -1.0, 2.0, 7.0, 6.0], [4.0, -5.0, 3.0, 6.0, 0.0], [4.0, 6.0, -4.0, 7.0, 2.0]]]])
      res=transposed_tensor_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[30.0, 15.0, -28.0, 0.0, -26.0], [-17.0, -16.0, -14.0, 53.0, -1.0], [71.0, 41.0, 16.0, -50.0, -73.0], [65.0, 49.0, -56.0, -45.0, 105.0]], [[-12.0, 8.0, 39.0, -1.0, -52.0], [-36.0, 18.0, -6.0, 21.0, 80.0], [83.0, 31.0, -36.0, -35.0, -101.0], [9.0, 1.0, -27.0, -44.0, 97.0]], [[54.0, -79.0, 0.0, -53.0, 20.0], [59.0, -48.0, 70.0, -31.0, -39.0], [-72.0, 44.0, 10.0, 2.0, 64.0], [100.0, 30.0, -26.0, -14.0, -4.0]], [[20.0, -56.0, -11.0, -20.0, -12.0], [-30.0, 36.0, 12.0, -45.0, -20.0], [42.0, -84.0, 10.0, -49.0, -48.0], [42.0, 20.0, -77.0, -33.0, -2.0]], [[3.0, -55.0, 29.0, -33.0, -9.0], [14.0, -1.0, 39.0, -42.0, 21.0], [-12.0, -7.0, -24.0, 1.0, 0.0], [22.0, -9.0, -12.0, -23.0, -3.0]]], [[[22.0, -16.0, 1.0, 30.0, 29.0], [-18.0, 29.0, 13.0, -28.0, -36.0], [-41.0, -39.0, 28.0, -25.0, 34.0], [35.0, 15.0, -26.0, 40.0, -69.0]], [[23.0, 52.0, -32.0, -54.0, -29.0], [11.0, -21.0, -47.0, 72.0, -12.0], [42.0, 67.0, 20.0, 16.0, -18.0], [-24.0, 15.0, 3.0, 8.0, 105.0]], [[69.0, -21.0, 45.0, -34.0, -45.0], [-35.0, 27.0, 21.0, 25.0, 38.0], [42.0, 77.0, 8.0, -74.0, -68.0], [110.0, 52.0, -90.0, 1.0, 120.0]], [[-27.0, 37.0, -29.0, -13.0, 6.0], [15.0, -8.0, -38.0, 14.0, -20.0], [-2.0, -19.0, 4.0, 48.0, 27.0], [-84.0, -28.0, 49.0, 17.0, -20.0]], [[-51.0, -3.0, -50.0, 12.0, 30.0], [58.0, -68.0, 4.0, -8.0, -27.0], [-22.0, -25.0, -18.0, 60.0, 41.0], [-50.0, -27.0, 68.0, -54.0, -51.0]]], [[[14.0, 58.0, 1.0, 22.0, 13.0], [8.0, -15.0, -19.0, 54.0, -6.0], [-35.0, 79.0, 20.0, 15.0, 40.0], [-3.0, 9.0, 34.0, 54.0, 13.0]], [[6.0, -7.0, -61.0, 0.0, 21.0], [33.0, -49.0, 1.0, 13.0, -55.0], [-2.0, -8.0, 20.0, 7.0, 17.0], [24.0, 22.0, -2.0, -39.0, -2.0]], [[2.0, -48.0, 13.0, -102.0, -31.0], [38.0, -15.0, 17.0, -28.0, 16.0], [7.0, 5.0, -28.0, 39.0, 2.0], [-29.0, -25.0, 6.0, -28.0, 43.0]], [[23.0, 81.0, -27.0, -30.0, -22.0], [7.0, -22.0, -56.0, 94.0, -8.0], [29.0, 96.0, 26.0, 18.0, -6.0], [-27.0, 17.0, 19.0, 32.0, 103.0]], [[52.0, 57.0, 0.0, 42.0, 41.0], [-16.0, 25.0, -19.0, 39.0, -54.0], [-81.0, 51.0, 64.0, -11.0, 84.0], [29.0, 30.0, 3.0, 123.0, -48.0]]], [[[-11.0, -48.0, 12.0, 67.0, 42.0], [5.0, -8.0, 54.0, -53.0, -8.0], [-54.0, -39.0, -8.0, -20.0, 31.0], [58.0, 5.0, -4.0, -18.0, -97.0]], [[-8.0, 21.0, -66.0, -15.0, -10.0], [2.0, -20.0, -40.0, 30.0, -36.0], [63.0, -33.0, 16.0, 1.0, -39.0], [-25.0, 12.0, -12.0, -45.0, 42.0]], [[3.0, 33.0, -28.0, 47.0, 40.0], [-14.0, 20.0, -24.0, 2.0, -55.0], [-41.0, -40.0, 40.0, 2.0, 52.0], [-19.0, 4.0, 12.0, 59.0, -82.0]], [[57.0, 75.0, -26.0, 69.0, 34.0], [-41.0, 28.0, -38.0, 65.0, -62.0], [-32.0, 41.0, 78.0, -47.0, 35.0], [54.0, 56.0, -30.0, 98.0, -14.0]], [[5.0, 11.0, 3.0, 6.0, -18.0], [-4.0, -16.0, 0.0, 31.0, 25.0], [34.0, 45.0, -8.0, -19.0, -41.0], [30.0, 17.0, -13.0, -29.0, 63.0]]]])+(1.-msk_ref)*numpy.array([[[[-4.0, 58.0, 9.0, -48.0, 69.0], [53.0, 25.0, 19.0, -58.0, 18.0], [-32.0, 27.0, -15.0, 95.0, -19.0], [-5.0, -34.0, -28.0, 33.0, 21.0]], [[-2.0, -76.0, 17.0, -15.0, -24.0], [-28.0, -46.0, -47.0, 31.0, -56.0], [16.0, -26.0, -68.0, -11.0, 25.0], [33.0, -8.0, -34.0, -58.0, 41.0]], [[17.0, 11.0, -51.0, 38.0, -44.0], [-12.0, -27.0, 35.0, 53.0, 11.0], [2.0, -89.0, 2.0, -58.0, -12.0], [-30.0, 60.0, 1.0, 96.0, 37.0]], [[29.0, 21.0, -28.0, 86.0, 28.0], [43.0, -22.0, 26.0, 94.0, 24.0], [42.0, -77.0, 53.0, 38.0, 10.0], [19.0, 92.0, 15.0, 98.0, 15.0]], [[13.0, 41.0, 13.0, -21.0, 41.0], [43.0, 1.0, 17.0, -30.0, -13.0], [-32.0, -3.0, -24.0, 67.0, -17.0], [-5.0, -17.0, -10.0, 35.0, 33.0]]], [[[1.0, 35.0, 24.0, 9.0, -64.0], [-8.0, 4.0, -42.0, -41.0, -77.0], [8.0, -1.0, -15.0, -77.0, 61.0], [-88.0, -2.0, 55.0, 0.0, -5.0]], [[-20.0, -64.0, -4.0, 34.0, -35.0], [-50.0, 4.0, -34.0, 45.0, 21.0], [56.0, 25.0, 46.0, -65.0, 31.0], [28.0, 20.0, 16.0, -73.0, -61.0]], [[-34.0, 56.0, -9.0, -3.0, 49.0], [46.0, 40.0, -39.0, -16.0, 33.0], [68.0, 19.0, 40.0, 50.0, 78.0], [-35.0, 28.0, -14.0, 41.0, -20.0]], [[52.0, -10.0, -2.0, 3.0, -90.0], [-37.0, -58.0, 50.0, -1.0, -81.0], [-94.0, -78.0, -80.0, -89.0, -71.0], [-35.0, -14.0, 28.0, 36.0, 72.0]], [[32.0, 12.0, 25.0, 5.0, 11.0], [24.0, -21.0, 19.0, -6.0, -43.0], [-42.0, -16.0, -29.0, 35.0, -29.0], [8.0, -13.0, 15.0, 11.0, 32.0]]], [[[38.0, -20.0, 40.0, 7.0, -20.0], [7.0, -48.0, -8.0, 6.0, -93.0], [-38.0, -35.0, -66.0, 8.0, -10.0], [7.0, -15.0, 17.0, -16.0, 53.0]], [[-25.0, 7.0, -22.0, -5.0, 13.0], [4.0, 15.0, -18.0, 8.0, 34.0], [42.0, -1.0, 18.0, 5.0, 35.0], [-9.0, 21.0, -24.0, 19.0, -8.0]], [[-23.0, -9.0, 10.0, -38.0, 15.0], [-1.0, 11.0, -30.0, -28.0, -1.0], [12.0, 30.0, -20.0, 19.0, 23.0], [2.0, -26.0, -27.0, -35.0, -1.0]], [[32.0, 30.0, -34.0, -28.0, -27.0], [-16.0, -11.0, 100.0, -30.0, 21.0], [-110.0, -37.0, -33.0, -31.0, -119.0], [-17.0, -28.0, -4.0, 69.0, 44.0]], [[23.0, 85.0, -34.0, 38.0, 63.0], [69.0, 15.0, 66.0, 22.0, 59.0], [-8.0, -42.0, 54.0, 77.0, -31.0], [-8.0, 52.0, 5.0, 133.0, 15.0]]], [[[-18.0, -92.0, -29.0, 10.0, -28.0], [-65.0, -13.0, -1.0, 61.0, 51.0], [28.0, 2.0, 19.0, -54.0, -20.0], [63.0, 12.0, -28.0, -62.0, -30.0]], [[-28.0, 28.0, -6.0, 59.0, 43.0], [17.0, 67.0, -12.0, 19.0, 90.0], [80.0, 75.0, 143.0, 14.0, 44.0], [12.0, 40.0, 46.0, -19.0, -120.0]], [[5.0, 33.0, 47.0, 6.0, 53.0], [44.0, 28.0, -13.0, -35.0, -12.0], [0.0, 62.0, 31.0, 71.0, 13.0], [11.0, -24.0, 33.0, -35.0, -37.0]], [[2.0, -12.0, -16.0, -31.0, -30.0], [-39.0, 4.0, 44.0, -28.0, 23.0], [-58.0, 13.0, -14.0, -46.0, -68.0], [1.0, -37.0, -5.0, -14.0, -3.0]], [[0.0, -50.0, -27.0, -35.0, -93.0], [-70.0, -42.0, 3.0, 1.0, -39.0], [-36.0, -56.0, -82.0, -111.0, -23.0], [-29.0, -15.0, -27.0, -3.0, 55.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_float_rank0_offset0(self):
      arg0=-3.0
      arg1=1.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=-3.0
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_array_rank0_offset0(self):
      arg0=-3.0
      arg1=numpy.array(-2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(6.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_array_rank1_offset0(self):
      arg0=4.0
      arg1=numpy.array([-1.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-4.0, 0.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_array_rank2_offset0(self):
      arg0=-6.0
      arg1=numpy.array([[-3.0, 3.0, 0.0, -4.0, 4.0], [-7.0, 6.0, -5.0, -1.0, -1.0], [1.0, 0.0, 0.0, -2.0, 5.0], [-7.0, 6.0, 3.0, -1.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[18.0, -18.0, -0.0, 24.0, -24.0], [42.0, -36.0, 30.0, 6.0, 6.0], [-6.0, -0.0, -0.0, 12.0, -30.0], [42.0, -36.0, -18.0, 6.0, -6.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_array_rank3_offset0(self):
      arg0=0.0
      arg1=numpy.array([[[-4.0, 6.0], [6.0, -4.0]], [[0.0, 7.0], [6.0, -3.0]], [[3.0, -7.0], [-3.0, -2.0]], [[-6.0, -6.0], [-7.0, 5.0]], [[-5.0, -4.0], [-6.0, 2.0]], [[0.0, 7.0], [2.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-0.0, 0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, -0.0]], [[0.0, -0.0], [-0.0, -0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_array_rank4_offset0(self):
      arg0=-6.0
      arg1=numpy.array([[[[-1.0, 3.0, 7.0, -6.0], [-4.0, 5.0, -4.0, -7.0], [2.0, -4.0, 3.0, 5.0]], [[-1.0, 6.0, -4.0, 5.0], [-3.0, 0.0, -3.0, -1.0], [-1.0, -4.0, -1.0, 7.0]]], [[[6.0, -2.0, 0.0, -7.0], [-2.0, -4.0, -2.0, 7.0], [-4.0, -2.0, 5.0, -5.0]], [[2.0, 4.0, 7.0, 0.0], [6.0, 0.0, 3.0, 7.0], [-6.0, -6.0, -7.0, -4.0]]], [[[-3.0, 5.0, 1.0, 4.0], [-1.0, 0.0, -6.0, -5.0], [-5.0, 6.0, 6.0, 0.0]], [[6.0, 3.0, -7.0, -3.0], [3.0, -5.0, -6.0, -2.0], [7.0, -5.0, 7.0, 5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[6.0, -18.0, -42.0, 36.0], [24.0, -30.0, 24.0, 42.0], [-12.0, 24.0, -18.0, -30.0]], [[6.0, -36.0, 24.0, -30.0], [18.0, -0.0, 18.0, 6.0], [6.0, 24.0, 6.0, -42.0]]], [[[-36.0, 12.0, -0.0, 42.0], [12.0, 24.0, 12.0, -42.0], [24.0, 12.0, -30.0, 30.0]], [[-12.0, -24.0, -42.0, -0.0], [-36.0, -0.0, -18.0, -42.0], [36.0, 36.0, 42.0, 24.0]]], [[[18.0, -30.0, -6.0, -24.0], [6.0, -0.0, 36.0, 30.0], [30.0, -36.0, -36.0, -0.0]], [[-36.0, -18.0, 42.0, 18.0], [-18.0, 30.0, 36.0, 12.0], [-42.0, 30.0, -42.0, -30.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_Symbol_rank0_offset0(self):
      arg0=2.0
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(2.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(4.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_Symbol_rank1_offset0(self):
      arg0=-1.0
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-2.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([2.0, -4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_Symbol_rank2_offset0(self):
      arg0=1.0
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-7.0, 0.0, 5.0, 5.0, 4.0], [1.0, -3.0, -5.0, 2.0, 2.0], [0.0, 5.0, -4.0, 0.0, -4.0], [-4.0, 7.0, -5.0, 0.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-7.0, 0.0, 5.0, 5.0, 4.0], [1.0, -3.0, -5.0, 2.0, 2.0], [0.0, 5.0, -4.0, 0.0, -4.0], [-4.0, 7.0, -5.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_Symbol_rank3_offset0(self):
      arg0=2.0
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-6.0, 0.0], [-6.0, -6.0]], [[4.0, -7.0], [0.0, -7.0]], [[-7.0, 4.0], [7.0, -5.0]], [[-7.0, -3.0], [0.0, -3.0]], [[5.0, 7.0], [-7.0, -2.0]], [[4.0, -1.0], [-6.0, -7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-12.0, 0.0], [-12.0, -12.0]], [[8.0, -14.0], [0.0, -14.0]], [[-14.0, 8.0], [14.0, -10.0]], [[-14.0, -6.0], [0.0, -6.0]], [[10.0, 14.0], [-14.0, -4.0]], [[8.0, -2.0], [-12.0, -14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_Symbol_rank4_offset0(self):
      arg0=3.0
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-6.0, 3.0, -1.0, 1.0], [1.0, 1.0, 1.0, 1.0], [-7.0, -5.0, 3.0, 7.0]], [[-3.0, 2.0, -3.0, -2.0], [5.0, -1.0, 3.0, 0.0], [3.0, 7.0, 6.0, 7.0]]], [[[-6.0, 7.0, 7.0, 7.0], [6.0, -4.0, 3.0, 5.0], [-2.0, 4.0, 5.0, 3.0]], [[-6.0, 5.0, -6.0, 7.0], [-7.0, 0.0, 0.0, 6.0], [1.0, 2.0, 1.0, -6.0]]], [[[5.0, 7.0, -4.0, 6.0], [-3.0, 1.0, 0.0, 2.0], [-7.0, 6.0, 5.0, -6.0]], [[-2.0, -4.0, 3.0, 7.0], [-5.0, 7.0, -6.0, 4.0], [-1.0, 0.0, -7.0, 1.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-18.0, 9.0, -3.0, 3.0], [3.0, 3.0, 3.0, 3.0], [-21.0, -15.0, 9.0, 21.0]], [[-9.0, 6.0, -9.0, -6.0], [15.0, -3.0, 9.0, 0.0], [9.0, 21.0, 18.0, 21.0]]], [[[-18.0, 21.0, 21.0, 21.0], [18.0, -12.0, 9.0, 15.0], [-6.0, 12.0, 15.0, 9.0]], [[-18.0, 15.0, -18.0, 21.0], [-21.0, 0.0, 0.0, 18.0], [3.0, 6.0, 3.0, -18.0]]], [[[15.0, 21.0, -12.0, 18.0], [-9.0, 3.0, 0.0, 6.0], [-21.0, 18.0, 15.0, -18.0]], [[-6.0, -12.0, 9.0, 21.0], [-15.0, 21.0, -18.0, 12.0], [-3.0, 0.0, -21.0, 3.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_constData_rank0_offset0(self):
      arg0=6.0
      arg1=Data(-3.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-18.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_constData_rank1_offset0(self):
      arg0=-7.0
      arg1=Data(numpy.array([-3.0, -3.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([21.0, 21.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_constData_rank2_offset0(self):
      arg0=-2.0
      arg1=Data(numpy.array([[-2.0, -7.0, 2.0, 5.0, 0.0], [0.0, -3.0, -7.0, -2.0, -3.0], [-6.0, -3.0, -1.0, 3.0, 4.0], [0.0, 7.0, 0.0, 7.0, 4.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[4.0, 14.0, -4.0, -10.0, -0.0], [-0.0, 6.0, 14.0, 4.0, 6.0], [12.0, 6.0, 2.0, -6.0, -8.0], [-0.0, -14.0, -0.0, -14.0, -8.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_constData_rank3_offset0(self):
      arg0=2.0
      arg1=Data(numpy.array([[[-7.0, -2.0], [5.0, -6.0]], [[7.0, -4.0], [6.0, 1.0]], [[7.0, -7.0], [1.0, 4.0]], [[-7.0, -2.0], [0.0, 1.0]], [[-1.0, 7.0], [-1.0, -7.0]], [[1.0, 0.0], [-6.0, -6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-14.0, -4.0], [10.0, -12.0]], [[14.0, -8.0], [12.0, 2.0]], [[14.0, -14.0], [2.0, 8.0]], [[-14.0, -4.0], [0.0, 2.0]], [[-2.0, 14.0], [-2.0, -14.0]], [[2.0, 0.0], [-12.0, -12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_constData_rank4_offset0(self):
      arg0=2.0
      arg1=Data(numpy.array([[[[-4.0, -6.0, -6.0, 5.0], [5.0, -6.0, 1.0, 0.0], [2.0, -7.0, -6.0, 3.0]], [[5.0, -5.0, 6.0, -1.0], [-1.0, 4.0, -1.0, 3.0], [7.0, 3.0, 2.0, 0.0]]], [[[-4.0, 1.0, 5.0, 3.0], [-5.0, 1.0, -4.0, -4.0], [-5.0, -3.0, 0.0, -2.0]], [[4.0, 3.0, -5.0, 2.0], [-2.0, -6.0, 4.0, -1.0], [0.0, -5.0, -6.0, -4.0]]], [[[5.0, -3.0, 4.0, -6.0], [1.0, 0.0, -4.0, -1.0], [-4.0, -7.0, 1.0, -1.0]], [[-6.0, -3.0, 0.0, -2.0], [-1.0, 0.0, 1.0, -6.0], [-2.0, 4.0, -6.0, -1.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-8.0, -12.0, -12.0, 10.0], [10.0, -12.0, 2.0, 0.0], [4.0, -14.0, -12.0, 6.0]], [[10.0, -10.0, 12.0, -2.0], [-2.0, 8.0, -2.0, 6.0], [14.0, 6.0, 4.0, 0.0]]], [[[-8.0, 2.0, 10.0, 6.0], [-10.0, 2.0, -8.0, -8.0], [-10.0, -6.0, 0.0, -4.0]], [[8.0, 6.0, -10.0, 4.0], [-4.0, -12.0, 8.0, -2.0], [0.0, -10.0, -12.0, -8.0]]], [[[10.0, -6.0, 8.0, -12.0], [2.0, 0.0, -8.0, -2.0], [-8.0, -14.0, 2.0, -2.0]], [[-12.0, -6.0, 0.0, -4.0], [-2.0, 0.0, 2.0, -12.0], [-4.0, 8.0, -12.0, -2.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_expandedData_rank0_offset0(self):
      arg0=-2.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(6.0)+(1-msk_arg1)*(2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-12.0)+(1.-msk_ref)*numpy.array(-4.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_expandedData_rank1_offset0(self):
      arg0=4.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, -2.0])+(1.-msk_arg1)*numpy.array([-4.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, -8.0])+(1.-msk_ref)*numpy.array([-16.0, 4.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_expandedData_rank2_offset0(self):
      arg0=-7.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, 6.0, -6.0, 3.0, 5.0], [6.0, -7.0, -4.0, -6.0, 6.0], [-2.0, 5.0, -7.0, -5.0, 3.0], [3.0, 0.0, 1.0, 5.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-6.0, -6.0, 3.0, 2.0, -1.0], [-4.0, -6.0, -1.0, 0.0, 6.0], [-7.0, 6.0, -6.0, 7.0, 0.0], [4.0, -6.0, 5.0, 1.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-49.0, -42.0, 42.0, -21.0, -35.0], [-42.0, 49.0, 28.0, 42.0, -42.0], [14.0, -35.0, 49.0, 35.0, -21.0], [-21.0, -0.0, -7.0, -35.0, -35.0]])+(1.-msk_ref)*numpy.array([[42.0, 42.0, -21.0, -14.0, 7.0], [28.0, 42.0, 7.0, -0.0, -42.0], [49.0, -42.0, 42.0, -49.0, -0.0], [-28.0, 42.0, -35.0, -7.0, -28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_expandedData_rank3_offset0(self):
      arg0=1.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -4.0], [1.0, 2.0]], [[1.0, -6.0], [-6.0, -7.0]], [[-5.0, 5.0], [0.0, -6.0]], [[-5.0, -2.0], [-7.0, 7.0]], [[0.0, 6.0], [2.0, -6.0]], [[6.0, -5.0], [-3.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-4.0, -1.0], [-1.0, -7.0]], [[-6.0, 1.0], [6.0, 0.0]], [[-3.0, -7.0], [-4.0, -7.0]], [[-5.0, -4.0], [6.0, -2.0]], [[-3.0, -3.0], [-7.0, 0.0]], [[4.0, 5.0], [0.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -4.0], [1.0, 2.0]], [[1.0, -6.0], [-6.0, -7.0]], [[-5.0, 5.0], [0.0, -6.0]], [[-5.0, -2.0], [-7.0, 7.0]], [[0.0, 6.0], [2.0, -6.0]], [[6.0, -5.0], [-3.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-4.0, -1.0], [-1.0, -7.0]], [[-6.0, 1.0], [6.0, 0.0]], [[-3.0, -7.0], [-4.0, -7.0]], [[-5.0, -4.0], [6.0, -2.0]], [[-3.0, -3.0], [-7.0, 0.0]], [[4.0, 5.0], [0.0, 4.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_float_rank0_expandedData_rank4_offset0(self):
      arg0=-5.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[4.0, -6.0, 7.0, 4.0], [4.0, 2.0, 3.0, -2.0], [4.0, 7.0, 4.0, 0.0]], [[1.0, -4.0, 6.0, -6.0], [-7.0, -2.0, 3.0, -4.0], [6.0, -4.0, -6.0, 2.0]]], [[[-6.0, -7.0, 5.0, -3.0], [0.0, -1.0, -2.0, 3.0], [5.0, 5.0, 7.0, -6.0]], [[-1.0, 0.0, -2.0, -4.0], [-3.0, -1.0, 3.0, -3.0], [-1.0, 1.0, 1.0, 2.0]]], [[[2.0, -7.0, 0.0, 0.0], [1.0, 4.0, 3.0, -4.0], [-3.0, 7.0, 6.0, -4.0]], [[0.0, 6.0, -5.0, 0.0], [4.0, 3.0, 4.0, -1.0], [-7.0, -2.0, 3.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-3.0, -4.0, 3.0, -6.0], [-7.0, -2.0, -2.0, 3.0], [7.0, 1.0, 2.0, -7.0]], [[0.0, -2.0, -4.0, 6.0], [-5.0, 6.0, 1.0, 3.0], [3.0, 7.0, -2.0, -1.0]]], [[[4.0, -7.0, 6.0, 2.0], [-4.0, -7.0, 5.0, 1.0], [2.0, 5.0, 3.0, 0.0]], [[-5.0, 4.0, -1.0, 0.0], [3.0, -3.0, 0.0, 1.0], [1.0, 2.0, 1.0, -3.0]]], [[[-1.0, -5.0, 5.0, 0.0], [5.0, 7.0, -3.0, 5.0], [0.0, -4.0, -4.0, -7.0]], [[-4.0, 1.0, 3.0, -2.0], [-7.0, 3.0, 6.0, 0.0], [3.0, -5.0, -6.0, -4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-20.0, 30.0, -35.0, -20.0], [-20.0, -10.0, -15.0, 10.0], [-20.0, -35.0, -20.0, -0.0]], [[-5.0, 20.0, -30.0, 30.0], [35.0, 10.0, -15.0, 20.0], [-30.0, 20.0, 30.0, -10.0]]], [[[30.0, 35.0, -25.0, 15.0], [-0.0, 5.0, 10.0, -15.0], [-25.0, -25.0, -35.0, 30.0]], [[5.0, -0.0, 10.0, 20.0], [15.0, 5.0, -15.0, 15.0], [5.0, -5.0, -5.0, -10.0]]], [[[-10.0, 35.0, -0.0, -0.0], [-5.0, -20.0, -15.0, 20.0], [15.0, -35.0, -30.0, 20.0]], [[-0.0, -30.0, 25.0, -0.0], [-20.0, -15.0, -20.0, 5.0], [35.0, 10.0, -15.0, 25.0]]]])+(1.-msk_ref)*numpy.array([[[[15.0, 20.0, -15.0, 30.0], [35.0, 10.0, 10.0, -15.0], [-35.0, -5.0, -10.0, 35.0]], [[-0.0, 10.0, 20.0, -30.0], [25.0, -30.0, -5.0, -15.0], [-15.0, -35.0, 10.0, 5.0]]], [[[-20.0, 35.0, -30.0, -10.0], [20.0, 35.0, -25.0, -5.0], [-10.0, -25.0, -15.0, -0.0]], [[25.0, -20.0, 5.0, -0.0], [-15.0, 15.0, -0.0, -5.0], [-5.0, -10.0, -5.0, 15.0]]], [[[5.0, 25.0, -25.0, -0.0], [-25.0, -35.0, 15.0, -25.0], [-0.0, 20.0, 20.0, 35.0]], [[20.0, -5.0, -15.0, 10.0], [35.0, -15.0, -30.0, -0.0], [-15.0, 25.0, 30.0, 20.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_float_rank0_offset0(self):
      arg0=numpy.array(-1.0)
      arg1=5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-5.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_array_rank0_offset0(self):
      arg0=numpy.array(-1.0)
      arg1=numpy.array(1.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-1.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank1_offset1(self):
      arg0=numpy.array([-7.0, 1.0, 3.0])
      arg1=numpy.array([-1.0, -3.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array(4.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank2_offset2(self):
      arg0=numpy.array([[5.0, 0.0, 0.0], [3.0, 0.0, -5.0]])
      arg1=numpy.array([[-4.0, 0.0, 6.0], [4.0, 2.0, 3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array(-23.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank3_offset3(self):
      arg0=numpy.array([[[4.0, -6.0, 1.0], [-1.0, -5.0, 7.0], [-4.0, 0.0, 4.0], [-2.0, 4.0, 7.0]], [[5.0, 4.0, 5.0], [-6.0, -7.0, -5.0], [-7.0, 3.0, -3.0], [2.0, 0.0, 0.0]]])
      arg1=numpy.array([[[-4.0, 1.0, -6.0], [-5.0, -3.0, -4.0], [3.0, -5.0, 0.0], [4.0, -5.0, 4.0]], [[5.0, 7.0, 6.0], [0.0, 2.0, -7.0], [0.0, 3.0, 7.0], [-1.0, 3.0, 5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array(42.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank4_offset4(self):
      arg0=numpy.array([[[[-7.0, -2.0], [1.0, -7.0], [3.0, -2.0]], [[4.0, -3.0], [1.0, -6.0], [-5.0, 2.0]]], [[[-1.0, 1.0], [5.0, 5.0], [0.0, 4.0]], [[7.0, 4.0], [-6.0, 2.0], [2.0, 4.0]]], [[[7.0, 4.0], [0.0, 2.0], [-3.0, 5.0]], [[-1.0, 1.0], [-3.0, 1.0], [4.0, -4.0]]], [[[3.0, 0.0], [-3.0, 5.0], [-7.0, -1.0]], [[-1.0, 3.0], [0.0, -6.0], [1.0, -4.0]]]])
      arg1=numpy.array([[[[-7.0, 4.0], [6.0, 3.0], [0.0, 6.0]], [[6.0, 3.0], [-5.0, 1.0], [-7.0, 6.0]]], [[[7.0, 5.0], [-6.0, 5.0], [4.0, 5.0]], [[6.0, 6.0], [5.0, 4.0], [5.0, -1.0]]], [[[4.0, -6.0], [5.0, 6.0], [6.0, -6.0]], [[-2.0, 7.0], [-4.0, 7.0], [-2.0, 0.0]]], [[[0.0, 1.0], [-2.0, 4.0], [0.0, -2.0]], [[2.0, -2.0], [-2.0, -6.0], [7.0, 5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      ref=numpy.array(159.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_array_rank1_offset0(self):
      arg0=numpy.array(-3.0)
      arg1=numpy.array([-4.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([12.0, -18.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank2_offset1(self):
      arg0=numpy.array([-7.0, 6.0, 2.0])
      arg1=numpy.array([[6.0, 1.0], [-1.0, -7.0], [3.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([-42.0, -57.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank3_offset2(self):
      arg0=numpy.array([[6.0, 0.0, -6.0], [2.0, -2.0, -6.0]])
      arg1=numpy.array([[[-1.0, 4.0], [6.0, -1.0], [1.0, -3.0]], [[-5.0, 7.0], [0.0, -7.0], [-2.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([-10.0, 46.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank4_offset3(self):
      arg0=numpy.array([[[1.0, -2.0, 0.0], [7.0, 7.0, -6.0], [-7.0, -1.0, 0.0], [-3.0, 0.0, -4.0]], [[-1.0, -5.0, 1.0], [6.0, 7.0, 5.0], [6.0, -1.0, -7.0], [5.0, 7.0, 3.0]]])
      arg1=numpy.array([[[[1.0, -4.0], [2.0, -2.0], [-1.0, 4.0]], [[-5.0, -7.0], [-6.0, 4.0], [0.0, 1.0]], [[-4.0, -1.0], [0.0, 7.0], [5.0, 1.0]], [[-5.0, -5.0], [-6.0, 7.0], [-3.0, 6.0]]], [[[7.0, -1.0], [-2.0, 1.0], [3.0, -6.0]], [[6.0, -5.0], [2.0, 2.0], [1.0, 2.0]], [[-2.0, 7.0], [-7.0, -5.0], [7.0, -4.0]], [[7.0, -4.0], [-4.0, 2.0], [-2.0, -3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([-17.0, 8.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_array_rank2_offset0(self):
      arg0=numpy.array(1.0)
      arg1=numpy.array([[7.0, 4.0, -6.0, -7.0, 6.0], [-1.0, 4.0, -3.0, -7.0, 1.0], [-5.0, -4.0, -4.0, 7.0, -4.0], [0.0, 5.0, 7.0, 0.0, -1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[7.0, 4.0, -6.0, -7.0, 6.0], [-1.0, 4.0, -3.0, -7.0, 1.0], [-5.0, -4.0, -4.0, 7.0, -4.0], [0.0, 5.0, 7.0, 0.0, -1.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank3_offset1(self):
      arg0=numpy.array([2.0, 0.0, 4.0])
      arg1=numpy.array([[[-7.0, 5.0, -6.0, -5.0, -2.0], [1.0, 1.0, 4.0, 0.0, 0.0], [-6.0, 1.0, 1.0, -7.0, 5.0], [0.0, 4.0, 7.0, 0.0, -5.0]], [[-3.0, -2.0, 5.0, 0.0, -5.0], [0.0, 1.0, 7.0, 7.0, 6.0], [-5.0, -5.0, -4.0, -6.0, -3.0], [4.0, 0.0, 7.0, -1.0, -4.0]], [[3.0, 0.0, -5.0, -6.0, -7.0], [5.0, 6.0, -4.0, 5.0, -7.0], [-6.0, -2.0, 4.0, 0.0, -4.0], [0.0, -7.0, 2.0, -2.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[-2.0, 10.0, -32.0, -34.0, -32.0], [22.0, 26.0, -8.0, 20.0, -28.0], [-36.0, -6.0, 18.0, -14.0, -6.0], [0.0, -20.0, 22.0, -8.0, -30.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank4_offset2(self):
      arg0=numpy.array([[-3.0, 5.0, -7.0], [-1.0, -6.0, -1.0]])
      arg1=numpy.array([[[[-4.0, 4.0, 3.0, -7.0, 0.0], [-1.0, -7.0, 0.0, 4.0, -1.0], [0.0, -6.0, 3.0, 3.0, 1.0], [-3.0, -5.0, -1.0, 3.0, -3.0]], [[-5.0, 6.0, 0.0, 5.0, 6.0], [0.0, 0.0, 3.0, 4.0, -3.0], [-7.0, 7.0, -7.0, -7.0, -3.0], [-3.0, 0.0, -1.0, -6.0, -4.0]], [[-7.0, 3.0, 1.0, -2.0, 5.0], [-2.0, -2.0, -2.0, -4.0, -7.0], [3.0, 4.0, 5.0, 4.0, -6.0], [5.0, 0.0, 6.0, 2.0, -1.0]]], [[[-3.0, 4.0, -2.0, 0.0, -1.0], [2.0, 0.0, -5.0, 7.0, -3.0], [2.0, -1.0, -1.0, 5.0, -6.0], [-7.0, 1.0, 2.0, -7.0, -6.0]], [[4.0, -2.0, 6.0, 7.0, -2.0], [0.0, 2.0, -2.0, -4.0, -2.0], [-3.0, 0.0, 0.0, -2.0, -3.0], [-7.0, -1.0, -1.0, 5.0, 0.0]], [[3.0, 7.0, -6.0, 4.0, 5.0], [-1.0, -7.0, -7.0, -4.0, 2.0], [5.0, 1.0, -3.0, -5.0, -3.0], [5.0, -1.0, 0.0, 1.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[12.0, -2.0, -44.0, 14.0, 3.0], [16.0, 30.0, 53.0, 57.0, 50.0], [-45.0, 25.0, -75.0, -60.0, 51.0], [3.0, 21.0, -40.0, -77.0, 3.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_array_rank3_offset0(self):
      arg0=numpy.array(0.0)
      arg1=numpy.array([[[-1.0, -1.0], [-1.0, 4.0]], [[-3.0, -5.0], [4.0, 5.0]], [[-1.0, -4.0], [0.0, -2.0]], [[-5.0, 3.0], [2.0, -4.0]], [[-2.0, 0.0], [3.0, 0.0]], [[0.0, -4.0], [-5.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-0.0, -0.0], [-0.0, 0.0]], [[-0.0, -0.0], [0.0, 0.0]], [[-0.0, -0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[0.0, -0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank4_offset1(self):
      arg0=numpy.array([2.0, -4.0, 5.0])
      arg1=numpy.array([[[[-7.0, 7.0], [2.0, 4.0]], [[4.0, -1.0], [3.0, 6.0]], [[-3.0, -3.0], [3.0, 6.0]], [[2.0, -4.0], [-1.0, 0.0]], [[-2.0, 7.0], [4.0, 6.0]], [[-4.0, 2.0], [4.0, -5.0]]], [[[5.0, 3.0], [0.0, 5.0]], [[-4.0, -5.0], [-2.0, -2.0]], [[-6.0, -7.0], [1.0, -5.0]], [[0.0, -3.0], [-6.0, -1.0]], [[-1.0, 5.0], [5.0, -5.0]], [[3.0, 4.0], [-3.0, -2.0]]], [[[-6.0, 6.0], [6.0, -3.0]], [[-2.0, -2.0], [-3.0, 0.0]], [[4.0, 7.0], [-2.0, 5.0]], [[-2.0, 3.0], [0.0, 1.0]], [[-1.0, 0.0], [6.0, 6.0]], [[2.0, -3.0], [-5.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-64.0, 32.0], [34.0, -27.0]], [[14.0, 8.0], [-1.0, 20.0]], [[38.0, 57.0], [-8.0, 57.0]], [[-6.0, 19.0], [22.0, 9.0]], [[-5.0, -6.0], [18.0, 62.0]], [[-10.0, -27.0], [-5.0, -27.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_array_rank4_offset0(self):
      arg0=numpy.array(6.0)
      arg1=numpy.array([[[[-6.0, 6.0, 0.0, -7.0], [-3.0, 1.0, 0.0, 3.0], [3.0, -2.0, -2.0, -3.0]], [[0.0, 2.0, -7.0, -1.0], [-1.0, -7.0, -3.0, 5.0], [2.0, 6.0, -1.0, 7.0]]], [[[-6.0, -5.0, 5.0, 1.0], [6.0, 7.0, 6.0, 4.0], [6.0, 0.0, 5.0, 0.0]], [[-5.0, -3.0, -2.0, -5.0], [0.0, 4.0, -7.0, 0.0], [-7.0, -7.0, 2.0, -2.0]]], [[[-5.0, -3.0, 6.0, -3.0], [0.0, 2.0, -5.0, 3.0], [0.0, -6.0, 3.0, 4.0]], [[1.0, 1.0, -4.0, 0.0], [6.0, -1.0, -6.0, -6.0], [-6.0, 1.0, 1.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-36.0, 36.0, 0.0, -42.0], [-18.0, 6.0, 0.0, 18.0], [18.0, -12.0, -12.0, -18.0]], [[0.0, 12.0, -42.0, -6.0], [-6.0, -42.0, -18.0, 30.0], [12.0, 36.0, -6.0, 42.0]]], [[[-36.0, -30.0, 30.0, 6.0], [36.0, 42.0, 36.0, 24.0], [36.0, 0.0, 30.0, 0.0]], [[-30.0, -18.0, -12.0, -30.0], [0.0, 24.0, -42.0, 0.0], [-42.0, -42.0, 12.0, -12.0]]], [[[-30.0, -18.0, 36.0, -18.0], [0.0, 12.0, -30.0, 18.0], [0.0, -36.0, 18.0, 24.0]], [[6.0, 6.0, -24.0, 0.0], [36.0, -6.0, -36.0, -36.0], [-36.0, 6.0, 6.0, -30.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_Symbol_rank0_offset0(self):
      arg0=numpy.array(1.0)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-1.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-1.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank1_offset1(self):
      arg0=numpy.array([0.0, -2.0, -6.0])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([5.0, 6.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-12.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank2_offset2(self):
      arg0=numpy.array([[-5.0, 0.0, -3.0], [0.0, -1.0, 6.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[1.0, -2.0, 7.0], [7.0, 2.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-64.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[-1.0, -4.0, -1.0], [-2.0, 7.0, 1.0], [0.0, 4.0, -4.0], [0.0, 1.0, 0.0]], [[-3.0, 7.0, 0.0], [2.0, -4.0, 4.0], [0.0, 2.0, -4.0], [-5.0, 7.0, 0.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[6.0, -5.0, -4.0], [1.0, 7.0, 2.0], [3.0, 4.0, 6.0], [-6.0, -4.0, 5.0]], [[-7.0, 2.0, -2.0], [-4.0, 7.0, -6.0], [0.0, -5.0, 5.0], [-1.0, 0.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(5.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank4_offset4(self):
      arg0=numpy.array([[[[-5.0, 5.0], [-6.0, 7.0], [-2.0, -4.0]], [[7.0, 7.0], [7.0, -5.0], [-5.0, 3.0]]], [[[-6.0, 0.0], [-2.0, 0.0], [0.0, 1.0]], [[-3.0, -2.0], [7.0, 2.0], [-1.0, 5.0]]], [[[3.0, -5.0], [5.0, -6.0], [-3.0, -4.0]], [[6.0, -4.0], [0.0, -5.0], [4.0, -2.0]]], [[[0.0, 0.0], [-4.0, -2.0], [-1.0, -3.0]], [[0.0, -7.0], [1.0, -2.0], [-1.0, 3.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[-3.0, -5.0], [-1.0, -7.0], [-5.0, 1.0]], [[4.0, 4.0], [0.0, 2.0], [2.0, -7.0]]], [[[0.0, 1.0], [4.0, 0.0], [7.0, 5.0]], [[4.0, 3.0], [2.0, 6.0], [2.0, -7.0]]], [[[0.0, 7.0], [1.0, 5.0], [-1.0, 0.0]], [[1.0, 4.0], [-7.0, 1.0], [-5.0, 7.0]]], [[[3.0, 4.0], [2.0, 5.0], [-1.0, 0.0]], [[-2.0, 7.0], [0.0, 3.0], [6.0, -1.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-251.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_Symbol_rank1_offset0(self):
      arg0=numpy.array(2.0)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([3.0, -4.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([6.0, -8.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank2_offset1(self):
      arg0=numpy.array([-4.0, 3.0, 1.0])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-4.0, 0.0], [-2.0, -6.0], [2.0, 3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([12.0, -15.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank3_offset2(self):
      arg0=numpy.array([[-7.0, -7.0, 3.0], [1.0, 1.0, 5.0]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[7.0, 2.0], [2.0, -4.0], [0.0, 5.0]], [[5.0, 4.0], [7.0, -4.0], [-2.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-61.0, 29.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[-2.0, 5.0, -4.0], [0.0, 7.0, -7.0], [-5.0, 4.0, -7.0], [3.0, 3.0, 0.0]], [[-4.0, 1.0, 6.0], [-4.0, 1.0, 0.0], [-5.0, -7.0, -7.0], [0.0, -4.0, 3.0]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-2.0, -3.0], [7.0, 7.0], [-4.0, 0.0]], [[1.0, -4.0], [-6.0, -5.0], [-5.0, -5.0]], [[2.0, -7.0], [-4.0, 5.0], [-3.0, 0.0]], [[5.0, 7.0], [4.0, -6.0], [2.0, 1.0]]], [[[0.0, 3.0], [3.0, 3.0], [-5.0, -6.0]], [[2.0, 2.0], [-6.0, -3.0], [1.0, 4.0]], [[-7.0, -3.0], [2.0, 0.0], [-5.0, 5.0]], [[-3.0, 0.0], [7.0, 0.0], [-6.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([39.0, 44.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_Symbol_rank2_offset0(self):
      arg0=numpy.array(3.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[7.0, 4.0, -3.0, 2.0, 4.0], [0.0, 7.0, -2.0, -4.0, 4.0], [7.0, 2.0, -6.0, -2.0, 5.0], [-3.0, -5.0, -4.0, -6.0, 2.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[21.0, 12.0, -9.0, 6.0, 12.0], [0.0, 21.0, -6.0, -12.0, 12.0], [21.0, 6.0, -18.0, -6.0, 15.0], [-9.0, -15.0, -12.0, -18.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank3_offset1(self):
      arg0=numpy.array([2.0, 2.0, 0.0])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[3.0, 4.0, 2.0, 5.0, -1.0], [-6.0, 3.0, 1.0, 3.0, 5.0], [4.0, -5.0, -1.0, 0.0, -4.0], [1.0, 4.0, 2.0, -3.0, 4.0]], [[-6.0, 0.0, -5.0, 3.0, -3.0], [0.0, -7.0, 1.0, 1.0, -3.0], [7.0, 1.0, -1.0, 0.0, 3.0], [-1.0, -1.0, -3.0, 4.0, -6.0]], [[-3.0, 5.0, 6.0, 0.0, 0.0], [1.0, -3.0, -4.0, 3.0, -7.0], [7.0, 6.0, -1.0, 6.0, 4.0], [-7.0, 0.0, -3.0, 6.0, -2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-6.0, 8.0, -6.0, 16.0, -8.0], [-12.0, -8.0, 4.0, 8.0, 4.0], [22.0, -8.0, -4.0, 0.0, -2.0], [0.0, 6.0, -2.0, 2.0, -4.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank4_offset2(self):
      arg0=numpy.array([[0.0, 2.0, 5.0], [5.0, 4.0, -6.0]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[6.0, -5.0, 1.0, 4.0, 6.0], [6.0, 4.0, 1.0, -7.0, -4.0], [-7.0, -6.0, 2.0, 4.0, 2.0], [0.0, 6.0, 5.0, 0.0, 5.0]], [[-4.0, -2.0, 6.0, 4.0, 7.0], [2.0, 1.0, 0.0, 6.0, 1.0], [6.0, 0.0, -3.0, 2.0, 5.0], [-1.0, 6.0, 3.0, -5.0, 5.0]], [[-4.0, 1.0, 0.0, 1.0, -7.0], [5.0, -3.0, 2.0, 6.0, 0.0], [-4.0, -4.0, 4.0, -1.0, 4.0], [5.0, 0.0, 6.0, 7.0, 2.0]]], [[[4.0, -5.0, -5.0, 7.0, -5.0], [-1.0, -2.0, 4.0, 6.0, 5.0], [-4.0, -4.0, -6.0, 5.0, 6.0], [0.0, -5.0, 7.0, 0.0, -6.0]], [[3.0, 5.0, 5.0, -4.0, -5.0], [2.0, -4.0, 0.0, -2.0, -3.0], [6.0, -1.0, -1.0, -7.0, -3.0], [-5.0, 7.0, 7.0, -4.0, 7.0]], [[5.0, -2.0, -3.0, -4.0, 6.0], [4.0, 2.0, 6.0, 2.0, -1.0], [1.0, 0.0, 3.0, -2.0, 0.0], [-2.0, 2.0, -3.0, 1.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-26.0, 8.0, 25.0, 56.0, -102.0], [8.0, -51.0, -6.0, 52.0, 21.0], [-10.0, -44.0, -38.0, 8.0, 48.0], [15.0, 3.0, 117.0, 3.0, 18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_Symbol_rank3_offset0(self):
      arg0=numpy.array(7.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[3.0, 7.0], [2.0, -6.0]], [[6.0, 2.0], [-6.0, -3.0]], [[4.0, 3.0], [-3.0, -1.0]], [[6.0, -5.0], [1.0, -4.0]], [[2.0, 4.0], [-4.0, 0.0]], [[-6.0, -5.0], [6.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[21.0, 49.0], [14.0, -42.0]], [[42.0, 14.0], [-42.0, -21.0]], [[28.0, 21.0], [-21.0, -7.0]], [[42.0, -35.0], [7.0, -28.0]], [[14.0, 28.0], [-28.0, 0.0]], [[-42.0, -35.0], [42.0, 7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank4_offset1(self):
      arg0=numpy.array([-6.0, 2.0, 7.0])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[7.0, -4.0], [0.0, 0.0]], [[-1.0, -4.0], [1.0, -6.0]], [[2.0, 0.0], [-5.0, -7.0]], [[2.0, -1.0], [-7.0, -7.0]], [[-3.0, -3.0], [5.0, -5.0]], [[-1.0, 0.0], [3.0, -2.0]]], [[[4.0, 5.0], [6.0, -1.0]], [[5.0, -7.0], [4.0, 7.0]], [[3.0, 2.0], [4.0, 6.0]], [[0.0, -4.0], [-1.0, -2.0]], [[-2.0, 0.0], [2.0, -7.0]], [[-2.0, -3.0], [0.0, 0.0]]], [[[-4.0, 5.0], [1.0, 7.0]], [[-5.0, 0.0], [3.0, 1.0]], [[-6.0, -4.0], [0.0, 0.0]], [[-7.0, 3.0], [5.0, -2.0]], [[4.0, -5.0], [0.0, 0.0]], [[1.0, -4.0], [3.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-62.0, 69.0], [19.0, 47.0]], [[-19.0, 10.0], [23.0, 57.0]], [[-48.0, -24.0], [38.0, 54.0]], [[-61.0, 19.0], [75.0, 24.0]], [[42.0, -17.0], [-26.0, 16.0]], [[9.0, -34.0], [3.0, 61.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_Symbol_rank4_offset0(self):
      arg0=numpy.array(-5.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-3.0, -3.0, 2.0, 0.0], [5.0, -4.0, -6.0, -4.0], [-6.0, 5.0, 3.0, 7.0]], [[-5.0, 0.0, -5.0, -4.0], [1.0, -6.0, 4.0, 7.0], [1.0, 5.0, 7.0, -1.0]]], [[[-2.0, 0.0, -3.0, 4.0], [3.0, -2.0, -2.0, 4.0], [-3.0, 0.0, 3.0, -6.0]], [[7.0, -7.0, 1.0, -2.0], [-5.0, 4.0, -5.0, 0.0], [5.0, 0.0, 4.0, 2.0]]], [[[3.0, -5.0, 3.0, -1.0], [6.0, -1.0, 7.0, 0.0], [-6.0, 5.0, -3.0, 4.0]], [[2.0, 4.0, 0.0, 4.0], [7.0, 0.0, 2.0, 3.0], [-1.0, -4.0, -7.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[15.0, 15.0, -10.0, -0.0], [-25.0, 20.0, 30.0, 20.0], [30.0, -25.0, -15.0, -35.0]], [[25.0, -0.0, 25.0, 20.0], [-5.0, 30.0, -20.0, -35.0], [-5.0, -25.0, -35.0, 5.0]]], [[[10.0, -0.0, 15.0, -20.0], [-15.0, 10.0, 10.0, -20.0], [15.0, -0.0, -15.0, 30.0]], [[-35.0, 35.0, -5.0, 10.0], [25.0, -20.0, 25.0, -0.0], [-25.0, -0.0, -20.0, -10.0]]], [[[-15.0, 25.0, -15.0, 5.0], [-30.0, 5.0, -35.0, -0.0], [30.0, -25.0, 15.0, -20.0]], [[-10.0, -20.0, -0.0, -20.0], [-35.0, -0.0, -10.0, -15.0], [5.0, 20.0, 35.0, 20.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_constData_rank0_offset0(self):
      arg0=numpy.array(4.0)
      arg1=Data(7.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(28.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank1_offset1(self):
      arg0=numpy.array([-5.0, -6.0, -5.0])
      arg1=Data(numpy.array([1.0, -1.0, 7.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(-34.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank2_offset2(self):
      arg0=numpy.array([[6.0, 0.0, 1.0], [-6.0, 6.0, -5.0]])
      arg1=Data(numpy.array([[7.0, 0.0, 4.0], [6.0, -1.0, 4.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(-16.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank3_offset3(self):
      arg0=numpy.array([[[0.0, -7.0, 7.0], [-1.0, -3.0, -6.0], [0.0, -5.0, 2.0], [-5.0, 0.0, 5.0]], [[-2.0, -7.0, -4.0], [4.0, -1.0, 5.0], [2.0, 0.0, 5.0], [7.0, 7.0, 0.0]]])
      arg1=Data(numpy.array([[[4.0, -2.0, 5.0], [-2.0, 7.0, 7.0], [7.0, -7.0, 4.0], [2.0, 0.0, 6.0]], [[-1.0, 1.0, 4.0], [7.0, 3.0, -1.0], [3.0, -1.0, -1.0], [-3.0, -7.0, -6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(-19.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank4_offset4(self):
      arg0=numpy.array([[[[1.0, -7.0], [-1.0, 3.0], [6.0, -1.0]], [[-6.0, 4.0], [0.0, 3.0], [-2.0, 2.0]]], [[[-3.0, -2.0], [-4.0, 6.0], [-3.0, 6.0]], [[0.0, 2.0], [-7.0, 0.0], [-7.0, 2.0]]], [[[2.0, 4.0], [7.0, 4.0], [1.0, 0.0]], [[6.0, 6.0], [-3.0, -7.0], [-4.0, 7.0]]], [[[6.0, 2.0], [-3.0, 0.0], [-3.0, -5.0]], [[-3.0, 0.0], [0.0, 7.0], [-4.0, -1.0]]]])
      arg1=Data(numpy.array([[[[5.0, 3.0], [3.0, 1.0], [-2.0, 5.0]], [[-7.0, 0.0], [-5.0, -1.0], [7.0, 0.0]]], [[[3.0, -1.0], [6.0, -1.0], [7.0, -2.0]], [[2.0, 5.0], [4.0, -6.0], [0.0, -5.0]]], [[[7.0, -1.0], [-3.0, 3.0], [-1.0, 7.0]], [[-2.0, -6.0], [-7.0, 2.0], [2.0, 5.0]]], [[[-7.0, 0.0], [5.0, 1.0], [-6.0, 6.0]], [[-6.0, 0.0], [-7.0, 0.0], [-5.0, -1.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(-150.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_constData_rank1_offset0(self):
      arg0=numpy.array(5.0)
      arg1=Data(numpy.array([-5.0, -6.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-25.0, -30.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank2_offset1(self):
      arg0=numpy.array([-6.0, 7.0, 6.0])
      arg1=Data(numpy.array([[6.0, -6.0], [-6.0, 6.0], [-4.0, -6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-102.0, 42.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank3_offset2(self):
      arg0=numpy.array([[7.0, -6.0, 6.0], [5.0, 0.0, -6.0]])
      arg1=Data(numpy.array([[[-7.0, -1.0], [4.0, 4.0], [3.0, -2.0]], [[-7.0, 3.0], [-6.0, -1.0], [-6.0, 4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-54.0, -52.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank4_offset3(self):
      arg0=numpy.array([[[-3.0, 1.0, 3.0], [2.0, 5.0, -4.0], [7.0, -3.0, 2.0], [-3.0, -6.0, -4.0]], [[-5.0, 0.0, 4.0], [-6.0, 3.0, 1.0], [1.0, 0.0, -2.0], [-5.0, -4.0, 4.0]]])
      arg1=Data(numpy.array([[[[-4.0, 3.0], [3.0, -6.0], [-4.0, 4.0]], [[-1.0, -4.0], [3.0, -2.0], [6.0, -3.0]], [[0.0, -1.0], [7.0, 0.0], [-4.0, 0.0]], [[-2.0, -5.0], [3.0, -3.0], [-6.0, 2.0]]], [[[4.0, 0.0], [0.0, -4.0], [-3.0, 5.0]], [[-3.0, 0.0], [6.0, -5.0], [-5.0, -2.0]], [[2.0, 2.0], [3.0, 0.0], [0.0, 3.0]], [[1.0, -7.0], [3.0, -3.0], [4.0, -1.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-25.0, 51.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_constData_rank2_offset0(self):
      arg0=numpy.array(-3.0)
      arg1=Data(numpy.array([[-6.0, 0.0, 2.0, -5.0, -6.0], [2.0, 2.0, 7.0, 4.0, 0.0], [-7.0, 4.0, 1.0, 6.0, -1.0], [7.0, 1.0, -5.0, -7.0, 2.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[18.0, -0.0, -6.0, 15.0, 18.0], [-6.0, -6.0, -21.0, -12.0, -0.0], [21.0, -12.0, -3.0, -18.0, 3.0], [-21.0, -3.0, 15.0, 21.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank3_offset1(self):
      arg0=numpy.array([2.0, 7.0, 1.0])
      arg1=Data(numpy.array([[[-5.0, 3.0, -4.0, 1.0, 1.0], [7.0, -6.0, 2.0, 4.0, 2.0], [5.0, -1.0, 1.0, 2.0, 0.0], [6.0, 7.0, 3.0, -4.0, 5.0]], [[1.0, 0.0, -2.0, -6.0, -3.0], [0.0, 0.0, 1.0, -5.0, -7.0], [1.0, -2.0, -6.0, 6.0, 6.0], [-3.0, -4.0, 3.0, -6.0, -7.0]], [[-3.0, -1.0, -4.0, 7.0, 2.0], [0.0, 5.0, 2.0, -5.0, 6.0], [-4.0, 7.0, 0.0, 0.0, -6.0], [-6.0, 0.0, -2.0, 6.0, 3.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-6.0, 5.0, -26.0, -33.0, -17.0], [14.0, -7.0, 13.0, -32.0, -39.0], [13.0, -9.0, -40.0, 46.0, 36.0], [-15.0, -14.0, 25.0, -44.0, -36.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank4_offset2(self):
      arg0=numpy.array([[-6.0, -3.0, -4.0], [-7.0, -1.0, 2.0]])
      arg1=Data(numpy.array([[[[-5.0, 4.0, -1.0, 1.0, -6.0], [1.0, 0.0, -4.0, -2.0, 3.0], [-5.0, -3.0, 7.0, 4.0, 0.0], [-2.0, 4.0, -2.0, 1.0, 6.0]], [[-1.0, -4.0, 7.0, 5.0, -4.0], [-4.0, 7.0, 2.0, 5.0, -3.0], [5.0, -5.0, -6.0, 1.0, 3.0], [0.0, -5.0, 0.0, 2.0, 1.0]], [[-6.0, 1.0, 0.0, 6.0, 0.0], [-6.0, 0.0, 2.0, 0.0, 6.0], [2.0, -3.0, 4.0, -4.0, 2.0], [0.0, 3.0, -7.0, 5.0, 0.0]]], [[[-7.0, -7.0, -2.0, 3.0, 5.0], [-1.0, 0.0, -1.0, 7.0, -6.0], [-1.0, 3.0, 5.0, 3.0, -1.0], [2.0, 7.0, -4.0, 6.0, 7.0]], [[-1.0, 7.0, 5.0, -7.0, 7.0], [0.0, -1.0, 2.0, -4.0, 6.0], [-4.0, -4.0, 4.0, -7.0, -5.0], [5.0, -2.0, -5.0, -3.0, 0.0]], [[-2.0, 4.0, 7.0, -6.0, 3.0], [7.0, 2.0, -2.0, 4.0, -1.0], [-3.0, 5.0, 1.0, 4.0, -6.0], [3.0, -3.0, 7.0, 3.0, 4.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[103.0, 34.0, 8.0, -71.0, 12.0], [51.0, -16.0, 11.0, -40.0, 1.0], [12.0, 38.0, -77.0, -17.0, -17.0], [-1.0, -74.0, 87.0, -65.0, -80.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_constData_rank3_offset0(self):
      arg0=numpy.array(1.0)
      arg1=Data(numpy.array([[[-2.0, 0.0], [-4.0, -7.0]], [[-6.0, -2.0], [3.0, 4.0]], [[0.0, 4.0], [-6.0, 0.0]], [[0.0, -6.0], [5.0, -1.0]], [[-4.0, 1.0], [-7.0, 5.0]], [[2.0, -5.0], [4.0, -4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-2.0, 0.0], [-4.0, -7.0]], [[-6.0, -2.0], [3.0, 4.0]], [[0.0, 4.0], [-6.0, 0.0]], [[0.0, -6.0], [5.0, -1.0]], [[-4.0, 1.0], [-7.0, 5.0]], [[2.0, -5.0], [4.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank4_offset1(self):
      arg0=numpy.array([-5.0, -6.0, 2.0])
      arg1=Data(numpy.array([[[[0.0, 4.0], [0.0, 7.0]], [[7.0, -6.0], [-1.0, -4.0]], [[-3.0, -6.0], [-1.0, 5.0]], [[7.0, 4.0], [-6.0, 0.0]], [[3.0, -3.0], [-4.0, -3.0]], [[0.0, -6.0], [1.0, 6.0]]], [[[-1.0, 0.0], [0.0, -4.0]], [[7.0, -4.0], [1.0, 2.0]], [[-1.0, 0.0], [-6.0, -5.0]], [[1.0, -1.0], [2.0, -4.0]], [[-3.0, -3.0], [3.0, -3.0]], [[-1.0, -5.0], [-1.0, -2.0]]], [[[-6.0, 1.0], [-7.0, 6.0]], [[-7.0, -5.0], [7.0, 4.0]], [[-4.0, -5.0], [-6.0, 0.0]], [[4.0, 2.0], [-6.0, -6.0]], [[3.0, 7.0], [-4.0, 1.0]], [[-2.0, 0.0], [6.0, 7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-6.0, -18.0], [-14.0, 1.0]], [[-91.0, 44.0], [13.0, 16.0]], [[13.0, 20.0], [29.0, 5.0]], [[-33.0, -10.0], [6.0, 12.0]], [[9.0, 47.0], [-6.0, 35.0]], [[2.0, 60.0], [13.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_constData_rank4_offset0(self):
      arg0=numpy.array(-3.0)
      arg1=Data(numpy.array([[[[1.0, 0.0, -6.0, 0.0], [4.0, -2.0, -5.0, -5.0], [3.0, 7.0, 5.0, -4.0]], [[4.0, 0.0, 4.0, 4.0], [6.0, 0.0, -1.0, 7.0], [7.0, -2.0, -5.0, -1.0]]], [[[-4.0, 0.0, 2.0, 6.0], [0.0, 0.0, 3.0, 0.0], [-2.0, 5.0, -3.0, 4.0]], [[-5.0, -1.0, -7.0, 0.0], [1.0, 4.0, 4.0, 3.0], [0.0, -5.0, -5.0, -7.0]]], [[[-1.0, 5.0, -5.0, 0.0], [5.0, -3.0, -4.0, -3.0], [-4.0, 1.0, 0.0, -5.0]], [[3.0, -6.0, 1.0, 4.0], [-6.0, -4.0, 3.0, -7.0], [0.0, -3.0, -4.0, -1.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-3.0, -0.0, 18.0, -0.0], [-12.0, 6.0, 15.0, 15.0], [-9.0, -21.0, -15.0, 12.0]], [[-12.0, -0.0, -12.0, -12.0], [-18.0, -0.0, 3.0, -21.0], [-21.0, 6.0, 15.0, 3.0]]], [[[12.0, -0.0, -6.0, -18.0], [-0.0, -0.0, -9.0, -0.0], [6.0, -15.0, 9.0, -12.0]], [[15.0, 3.0, 21.0, -0.0], [-3.0, -12.0, -12.0, -9.0], [-0.0, 15.0, 15.0, 21.0]]], [[[3.0, -15.0, 15.0, -0.0], [-15.0, 9.0, 12.0, 9.0], [12.0, -3.0, -0.0, 15.0]], [[-9.0, 18.0, -3.0, -12.0], [18.0, 12.0, -9.0, 21.0], [-0.0, 9.0, 12.0, 3.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_expandedData_rank0_offset0(self):
      arg0=numpy.array(-4.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-0.0)+(1.-msk_ref)*numpy.array(20.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank1_offset1(self):
      arg0=numpy.array([7.0, 4.0, -1.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, 1.0, 0.0])+(1.-msk_arg1)*numpy.array([-5.0, -7.0, -2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(32.0)+(1.-msk_ref)*numpy.array(-61.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank2_offset2(self):
      arg0=numpy.array([[-5.0, 4.0, -2.0], [6.0, 4.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 1.0, 0.0], [5.0, -6.0, 1.0]])+(1.-msk_arg1)*numpy.array([[1.0, -4.0, -7.0], [-5.0, 3.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(7.0)+(1.-msk_ref)*numpy.array(17.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[5.0, -7.0, 1.0], [0.0, 1.0, 1.0], [-5.0, -2.0, -5.0], [-2.0, 1.0, 6.0]], [[-1.0, -5.0, 7.0], [-3.0, 6.0, -4.0], [2.0, -7.0, 7.0], [-7.0, 1.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, 3.0, -5.0], [-3.0, 4.0, 1.0], [-5.0, 2.0, 6.0], [-7.0, -4.0, 3.0]], [[0.0, 2.0, -3.0], [6.0, -7.0, -1.0], [0.0, 2.0, 3.0], [4.0, 0.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 6.0, -5.0], [-5.0, -6.0, -4.0], [3.0, 3.0, -6.0], [0.0, -4.0, -4.0]], [[3.0, 2.0, 2.0], [3.0, 6.0, 5.0], [0.0, -1.0, 7.0], [5.0, 6.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-176.0)+(1.-msk_ref)*numpy.array(-50.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank4_offset4(self):
      arg0=numpy.array([[[[-1.0, 6.0], [0.0, 0.0], [-4.0, -5.0]], [[-7.0, 0.0], [-6.0, -2.0], [3.0, 4.0]]], [[[2.0, 0.0], [5.0, -3.0], [-1.0, 7.0]], [[4.0, 1.0], [6.0, 0.0], [7.0, 7.0]]], [[[4.0, 2.0], [1.0, 2.0], [-7.0, -3.0]], [[-1.0, 0.0], [7.0, 0.0], [-5.0, 3.0]]], [[[0.0, -3.0], [2.0, 6.0], [0.0, 1.0]], [[5.0, -4.0], [5.0, -6.0], [-1.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, 5.0], [3.0, 2.0], [7.0, 7.0]], [[-2.0, 7.0], [-1.0, -7.0], [0.0, -6.0]]], [[[-7.0, 1.0], [5.0, 1.0], [-4.0, 5.0]], [[-6.0, 7.0], [1.0, -1.0], [1.0, -3.0]]], [[[6.0, -4.0], [-6.0, 6.0], [0.0, -4.0]], [[-6.0, -5.0], [7.0, 0.0], [-1.0, 0.0]]], [[[6.0, 6.0], [-2.0, -3.0], [-6.0, -2.0]], [[-2.0, 2.0], [-5.0, -4.0], [6.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 1.0], [2.0, -2.0], [-4.0, 7.0]], [[0.0, -2.0], [-5.0, -2.0], [-1.0, -5.0]]], [[[5.0, 2.0], [-1.0, -3.0], [-1.0, 0.0]], [[1.0, -3.0], [0.0, -7.0], [-5.0, -2.0]]], [[[-6.0, -4.0], [-6.0, 5.0], [4.0, -1.0]], [[-1.0, 6.0], [-7.0, 1.0], [0.0, 3.0]]], [[[1.0, -2.0], [-3.0, 0.0], [3.0, 2.0]], [[1.0, 1.0], [-2.0, -5.0], [-2.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(67.0)+(1.-msk_ref)*numpy.array(-100.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_expandedData_rank1_offset0(self):
      arg0=numpy.array(3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, 0.0])+(1.-msk_arg1)*numpy.array([2.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, 0.0])+(1.-msk_ref)*numpy.array([6.0, 18.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank2_offset1(self):
      arg0=numpy.array([0.0, 1.0, -7.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 2.0], [-2.0, 0.0], [6.0, 2.0]])+(1.-msk_arg1)*numpy.array([[0.0, -5.0], [0.0, -7.0], [-5.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-44.0, -14.0])+(1.-msk_ref)*numpy.array([35.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank3_offset2(self):
      arg0=numpy.array([[-7.0, -6.0, -4.0], [-6.0, 0.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 2.0], [0.0, -4.0], [0.0, -2.0]], [[0.0, -5.0], [5.0, -7.0], [7.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 5.0], [-3.0, 0.0], [-1.0, -3.0]], [[-4.0, 0.0], [3.0, -1.0], [-7.0, -6.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-14.0, 58.0])+(1.-msk_ref)*numpy.array([4.0, -53.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[-2.0, 5.0, 1.0], [0.0, 4.0, 1.0], [0.0, 2.0, -1.0], [-4.0, 3.0, -4.0]], [[-7.0, -3.0, -5.0], [-6.0, -5.0, -5.0], [5.0, -3.0, 5.0], [5.0, -7.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, 4.0], [0.0, -5.0], [-5.0, -7.0]], [[6.0, 5.0], [-5.0, 1.0], [4.0, -2.0]], [[0.0, -5.0], [2.0, 0.0], [-1.0, -6.0]], [[7.0, -1.0], [1.0, -7.0], [7.0, 4.0]]], [[[0.0, -1.0], [-4.0, -6.0], [-3.0, -5.0]], [[6.0, 3.0], [1.0, 2.0], [3.0, -5.0]], [[2.0, 2.0], [-5.0, 0.0], [1.0, -1.0]], [[0.0, 2.0], [-5.0, -1.0], [5.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, 4.0], [4.0, 1.0], [0.0, 1.0]], [[-2.0, 7.0], [-7.0, 3.0], [-4.0, 7.0]], [[-4.0, 2.0], [-6.0, -4.0], [5.0, -1.0]], [[-3.0, 1.0], [2.0, -4.0], [-1.0, 5.0]]], [[[4.0, -2.0], [5.0, 0.0], [6.0, 3.0]], [[-5.0, -2.0], [0.0, 4.0], [6.0, -1.0]], [[-1.0, 0.0], [3.0, 7.0], [1.0, -2.0]], [[-2.0, 0.0], [0.0, -7.0], [6.0, 0.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-35.0, 4.0])+(1.-msk_ref)*numpy.array([-111.0, -12.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_expandedData_rank2_offset0(self):
      arg0=numpy.array(1.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 0.0, 6.0, -5.0, 5.0], [-6.0, 2.0, -1.0, -1.0, 1.0], [1.0, 2.0, 2.0, -7.0, 1.0], [3.0, 7.0, 0.0, -7.0, 7.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -1.0, 2.0, -6.0, 0.0], [-7.0, -3.0, -6.0, -6.0, 0.0], [-4.0, 6.0, 2.0, 7.0, 6.0], [0.0, 4.0, 1.0, 4.0, -2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 0.0, 6.0, -5.0, 5.0], [-6.0, 2.0, -1.0, -1.0, 1.0], [1.0, 2.0, 2.0, -7.0, 1.0], [3.0, 7.0, 0.0, -7.0, 7.0]])+(1.-msk_ref)*numpy.array([[-7.0, -1.0, 2.0, -6.0, 0.0], [-7.0, -3.0, -6.0, -6.0, 0.0], [-4.0, 6.0, 2.0, 7.0, 6.0], [0.0, 4.0, 1.0, 4.0, -2.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank3_offset1(self):
      arg0=numpy.array([-4.0, -6.0, 1.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-7.0, -4.0, 0.0, -5.0, 3.0], [-2.0, -6.0, -1.0, -5.0, 0.0], [-1.0, 6.0, 5.0, 0.0, -3.0], [7.0, 5.0, 1.0, -3.0, -7.0]], [[6.0, -6.0, -3.0, -7.0, 5.0], [-5.0, 3.0, 1.0, -1.0, 1.0], [1.0, -3.0, -6.0, 3.0, 0.0], [6.0, -1.0, -5.0, -3.0, -6.0]], [[-2.0, -3.0, 6.0, -2.0, 0.0], [-1.0, 1.0, -6.0, -5.0, 4.0], [6.0, -7.0, -4.0, 2.0, -1.0], [-3.0, -2.0, -5.0, -4.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 1.0, 0.0, 0.0, 1.0], [-2.0, 4.0, 6.0, 5.0, 0.0], [-2.0, 5.0, 0.0, 2.0, 0.0], [3.0, -7.0, -1.0, 5.0, -2.0]], [[-1.0, -7.0, -4.0, -3.0, -2.0], [4.0, 4.0, -6.0, 4.0, 0.0], [0.0, 2.0, -2.0, -2.0, 7.0], [7.0, -5.0, -1.0, 3.0, -7.0]], [[-5.0, -2.0, 0.0, -3.0, -7.0], [6.0, 3.0, 4.0, -5.0, -3.0], [-2.0, -7.0, -6.0, 3.0, 0.0], [2.0, -5.0, 7.0, 1.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, 49.0, 24.0, 60.0, -42.0], [37.0, 7.0, -8.0, 21.0, -2.0], [4.0, -13.0, 12.0, -16.0, 11.0], [-67.0, -16.0, 21.0, 26.0, 57.0]])+(1.-msk_ref)*numpy.array([[1.0, 36.0, 24.0, 15.0, 1.0], [-10.0, -37.0, 16.0, -49.0, -3.0], [6.0, -39.0, 6.0, 7.0, -42.0], [-52.0, 53.0, 17.0, -37.0, 52.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank4_offset2(self):
      arg0=numpy.array([[-1.0, -7.0, -5.0], [1.0, 0.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, 5.0, -5.0, -3.0, 5.0], [6.0, -7.0, 4.0, -6.0, 3.0], [-2.0, -6.0, 4.0, 2.0, -4.0], [-6.0, -5.0, 2.0, 7.0, -7.0]], [[-1.0, -3.0, -1.0, -6.0, -2.0], [-1.0, 1.0, 6.0, -2.0, -4.0], [5.0, 0.0, 7.0, -6.0, -7.0], [7.0, 0.0, -2.0, 1.0, 5.0]], [[-6.0, -5.0, 0.0, 0.0, -6.0], [-7.0, 5.0, 4.0, -1.0, 6.0], [-4.0, 0.0, 0.0, 0.0, -6.0], [-5.0, -5.0, 4.0, -2.0, 4.0]]], [[[4.0, -6.0, -4.0, 4.0, 6.0], [-1.0, 2.0, 0.0, 5.0, 0.0], [7.0, 7.0, -6.0, 5.0, -5.0], [0.0, -2.0, 2.0, -5.0, -3.0]], [[-7.0, 0.0, 3.0, 5.0, 3.0], [1.0, -5.0, -2.0, -3.0, 5.0], [4.0, -1.0, -6.0, 4.0, 1.0], [-3.0, 0.0, -4.0, 7.0, 3.0]], [[-1.0, 7.0, -3.0, -5.0, -3.0], [5.0, -3.0, -2.0, 7.0, 0.0], [3.0, -3.0, -5.0, 1.0, -7.0], [3.0, 1.0, 4.0, -4.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -7.0, 0.0, 5.0, -5.0], [2.0, 1.0, 5.0, 7.0, 4.0], [2.0, -7.0, -4.0, -1.0, -2.0], [5.0, 2.0, -3.0, -3.0, -6.0]], [[-2.0, 0.0, 0.0, -5.0, -2.0], [7.0, 5.0, 0.0, 0.0, 3.0], [0.0, 7.0, 5.0, 5.0, -1.0], [4.0, 4.0, -7.0, 0.0, 1.0]], [[1.0, -4.0, 3.0, 3.0, 5.0], [-7.0, -4.0, 1.0, 2.0, -3.0], [-4.0, -5.0, 6.0, -2.0, -7.0], [-7.0, 3.0, -6.0, 5.0, -5.0]]], [[[-6.0, -4.0, 6.0, -3.0, 4.0], [6.0, 5.0, 0.0, 3.0, 1.0], [-1.0, -2.0, -4.0, 6.0, 1.0], [-4.0, -4.0, 5.0, 1.0, 0.0]], [[-3.0, 2.0, 0.0, 7.0, -4.0], [3.0, 3.0, -1.0, 1.0, -5.0], [4.0, 0.0, -3.0, 2.0, -6.0], [3.0, 5.0, 2.0, 0.0, 5.0]], [[3.0, -7.0, -1.0, 2.0, 6.0], [-2.0, -1.0, -5.0, 3.0, -2.0], [0.0, -1.0, 0.0, -7.0, 1.0], [7.0, 2.0, 0.0, 4.0, 3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[34.0, 42.0, 5.0, 44.0, 42.0], [40.0, -26.0, -68.0, 37.0, -5.0], [-3.0, 10.0, -64.0, 46.0, 71.0], [-15.0, 29.0, -2.0, -13.0, -58.0]])+(1.-msk_ref)*numpy.array([[0.0, 16.0, -10.0, 14.0, 4.0], [-12.0, -12.0, -15.0, -11.0, -11.0], [17.0, -20.0, -65.0, -25.0, 46.0], [5.0, -47.0, 87.0, -17.0, 27.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_expandedData_rank3_offset0(self):
      arg0=numpy.array(3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 5.0], [-5.0, 4.0]], [[0.0, 4.0], [3.0, 4.0]], [[-2.0, 0.0], [3.0, -1.0]], [[-1.0, 5.0], [-5.0, 0.0]], [[-7.0, 5.0], [-7.0, -2.0]], [[-1.0, 3.0], [-3.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -3.0], [5.0, -2.0]], [[-2.0, -2.0], [0.0, -1.0]], [[-7.0, -4.0], [-4.0, -2.0]], [[-3.0, -7.0], [6.0, -6.0]], [[1.0, -7.0], [-1.0, -5.0]], [[7.0, -6.0], [6.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 15.0], [-15.0, 12.0]], [[0.0, 12.0], [9.0, 12.0]], [[-6.0, 0.0], [9.0, -3.0]], [[-3.0, 15.0], [-15.0, 0.0]], [[-21.0, 15.0], [-21.0, -6.0]], [[-3.0, 9.0], [-9.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[6.0, -9.0], [15.0, -6.0]], [[-6.0, -6.0], [0.0, -3.0]], [[-21.0, -12.0], [-12.0, -6.0]], [[-9.0, -21.0], [18.0, -18.0]], [[3.0, -21.0], [-3.0, -15.0]], [[21.0, -18.0], [18.0, 3.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank4_offset1(self):
      arg0=numpy.array([3.0, 7.0, -6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -5.0], [2.0, -4.0]], [[3.0, 1.0], [7.0, -6.0]], [[-5.0, -3.0], [-6.0, 6.0]], [[-3.0, -5.0], [-6.0, -2.0]], [[4.0, 7.0], [7.0, 0.0]], [[0.0, 0.0], [7.0, 7.0]]], [[[7.0, 3.0], [-3.0, 6.0]], [[-1.0, 7.0], [-1.0, -6.0]], [[6.0, 6.0], [-2.0, -2.0]], [[-3.0, -7.0], [5.0, 7.0]], [[6.0, 0.0], [4.0, 4.0]], [[3.0, 6.0], [1.0, 0.0]]], [[[-5.0, -6.0], [-2.0, 0.0]], [[1.0, -3.0], [2.0, -6.0]], [[-6.0, -6.0], [3.0, 1.0]], [[-6.0, 4.0], [-7.0, 2.0]], [[5.0, -2.0], [0.0, -7.0]], [[-2.0, 1.0], [-1.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, -2.0], [6.0, 6.0]], [[-6.0, -4.0], [-2.0, -7.0]], [[2.0, 3.0], [-2.0, 1.0]], [[4.0, 0.0], [-7.0, 0.0]], [[0.0, -7.0], [0.0, 5.0]], [[-1.0, 2.0], [-5.0, 6.0]]], [[[-2.0, 7.0], [-2.0, 7.0]], [[-1.0, -5.0], [-5.0, 0.0]], [[-6.0, 6.0], [4.0, 3.0]], [[5.0, 7.0], [5.0, -3.0]], [[5.0, 5.0], [7.0, -6.0]], [[1.0, 5.0], [-6.0, -6.0]]], [[[2.0, -4.0], [5.0, 0.0]], [[-1.0, 0.0], [-4.0, 0.0]], [[-5.0, 6.0], [5.0, 0.0]], [[1.0, -1.0], [-2.0, 0.0]], [[-6.0, -5.0], [-5.0, 7.0]], [[1.0, 4.0], [3.0, 0.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[79.0, 42.0], [-3.0, 30.0]], [[-4.0, 70.0], [2.0, -24.0]], [[63.0, 69.0], [-50.0, -2.0]], [[6.0, -88.0], [59.0, 31.0]], [[24.0, 33.0], [49.0, 70.0]], [[33.0, 36.0], [34.0, -9.0]]])+(1.-msk_ref)*numpy.array([[[-23.0, 67.0], [-26.0, 67.0]], [[-19.0, -47.0], [-17.0, -21.0]], [[-6.0, 15.0], [-8.0, 24.0]], [[41.0, 55.0], [26.0, -21.0]], [[71.0, 44.0], [79.0, -69.0]], [[-2.0, 17.0], [-75.0, -24.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank0_expandedData_rank4_offset0(self):
      arg0=numpy.array(7.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, 1.0, -7.0, 4.0], [-6.0, 4.0, 2.0, 4.0], [-6.0, 4.0, -4.0, -3.0]], [[-4.0, -5.0, 0.0, 0.0], [-6.0, -5.0, 7.0, 2.0], [-6.0, 6.0, -4.0, 1.0]]], [[[6.0, 2.0, 4.0, 6.0], [-4.0, 3.0, -3.0, 5.0], [7.0, 0.0, 3.0, 6.0]], [[2.0, 7.0, -7.0, -2.0], [-6.0, -2.0, 0.0, 3.0], [-4.0, 6.0, 0.0, -6.0]]], [[[6.0, -1.0, -2.0, 5.0], [1.0, -5.0, -6.0, 1.0], [6.0, -4.0, 0.0, -5.0]], [[-1.0, 4.0, -3.0, 3.0], [4.0, -6.0, -3.0, -3.0], [4.0, -2.0, 1.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, 7.0, -1.0, -5.0], [4.0, -6.0, 5.0, 7.0], [5.0, -1.0, -5.0, 6.0]], [[7.0, 4.0, 5.0, 2.0], [7.0, -3.0, 0.0, 5.0], [4.0, 3.0, 1.0, 5.0]]], [[[4.0, -7.0, -6.0, 2.0], [1.0, 0.0, 5.0, 4.0], [2.0, -4.0, 0.0, 1.0]], [[-4.0, -5.0, 0.0, 7.0], [5.0, -1.0, -5.0, 0.0], [7.0, 7.0, 0.0, -7.0]]], [[[-3.0, 5.0, 0.0, -7.0], [7.0, 7.0, -2.0, 7.0], [-3.0, -3.0, -1.0, 5.0]], [[-5.0, -4.0, -1.0, 1.0], [5.0, -2.0, 4.0, -4.0], [0.0, 7.0, 5.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 7.0, -49.0, 28.0], [-42.0, 28.0, 14.0, 28.0], [-42.0, 28.0, -28.0, -21.0]], [[-28.0, -35.0, 0.0, 0.0], [-42.0, -35.0, 49.0, 14.0], [-42.0, 42.0, -28.0, 7.0]]], [[[42.0, 14.0, 28.0, 42.0], [-28.0, 21.0, -21.0, 35.0], [49.0, 0.0, 21.0, 42.0]], [[14.0, 49.0, -49.0, -14.0], [-42.0, -14.0, 0.0, 21.0], [-28.0, 42.0, 0.0, -42.0]]], [[[42.0, -7.0, -14.0, 35.0], [7.0, -35.0, -42.0, 7.0], [42.0, -28.0, 0.0, -35.0]], [[-7.0, 28.0, -21.0, 21.0], [28.0, -42.0, -21.0, -21.0], [28.0, -14.0, 7.0, 28.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 49.0, -7.0, -35.0], [28.0, -42.0, 35.0, 49.0], [35.0, -7.0, -35.0, 42.0]], [[49.0, 28.0, 35.0, 14.0], [49.0, -21.0, 0.0, 35.0], [28.0, 21.0, 7.0, 35.0]]], [[[28.0, -49.0, -42.0, 14.0], [7.0, 0.0, 35.0, 28.0], [14.0, -28.0, 0.0, 7.0]], [[-28.0, -35.0, 0.0, 49.0], [35.0, -7.0, -35.0, 0.0], [49.0, 49.0, 0.0, -49.0]]], [[[-21.0, 35.0, 0.0, -49.0], [49.0, 49.0, -14.0, 49.0], [-21.0, -21.0, -7.0, 35.0]], [[-35.0, -28.0, -7.0, 7.0], [35.0, -14.0, 28.0, -28.0], [0.0, 49.0, 35.0, 14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_float_rank0_offset0(self):
      arg0=numpy.array([-5.0, -3.0])
      arg1=-5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([25.0, 15.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank0_offset0(self):
      arg0=numpy.array([6.0, -7.0])
      arg1=numpy.array(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-18.0, 21.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank1_offset1(self):
      arg0=numpy.array([[-4.0, -6.0], [-4.0, -6.0], [-7.0, -7.0]])
      arg1=numpy.array([4.0, -1.0, 4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([-40.0, -46.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank2_offset2(self):
      arg0=numpy.array([[[3.0, 4.0], [0.0, 7.0], [-4.0, -6.0]], [[3.0, 0.0], [5.0, 0.0], [6.0, 5.0]]])
      arg1=numpy.array([[-4.0, 1.0, 3.0], [-3.0, 4.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([29.0, 8.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank3_offset3(self):
      arg0=numpy.array([[[[0.0, -1.0], [-7.0, -5.0], [-5.0, 2.0]], [[0.0, -1.0], [4.0, 4.0], [1.0, 4.0]], [[0.0, 2.0], [-4.0, -2.0], [5.0, -1.0]], [[5.0, 2.0], [-7.0, -4.0], [-6.0, 6.0]]], [[[7.0, 3.0], [0.0, -3.0], [4.0, 4.0]], [[1.0, 0.0], [6.0, -4.0], [0.0, -6.0]], [[4.0, -6.0], [1.0, -6.0], [5.0, 0.0]], [[-3.0, 0.0], [-4.0, 4.0], [5.0, -1.0]]]])
      arg1=numpy.array([[[2.0, 4.0, 0.0], [-5.0, -2.0, 0.0], [7.0, -7.0, -3.0], [4.0, 4.0, 0.0]], [[0.0, 1.0, 6.0], [7.0, 4.0, -6.0], [2.0, 7.0, 4.0], [2.0, 7.0, 6.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([55.0, 7.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank1_offset0(self):
      arg0=numpy.array([-3.0, -6.0])
      arg1=numpy.array([5.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-15.0, -3.0], [-30.0, -6.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank2_offset1(self):
      arg0=numpy.array([[-4.0, -5.0], [-7.0, 5.0], [-3.0, -1.0]])
      arg1=numpy.array([[3.0, -4.0], [-6.0, -3.0], [0.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[30.0, 37.0], [-45.0, 5.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank3_offset2(self):
      arg0=numpy.array([[[3.0, -5.0], [0.0, -4.0], [-6.0, -7.0]], [[-7.0, -6.0], [2.0, 1.0], [3.0, -2.0]]])
      arg1=numpy.array([[[-2.0, 4.0], [-6.0, -5.0], [6.0, 5.0]], [[3.0, -6.0], [-6.0, -3.0], [-6.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[-93.0, 12.0], [-20.0, 2.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank4_offset3(self):
      arg0=numpy.array([[[[-1.0, 6.0], [-6.0, 0.0], [-1.0, -7.0]], [[1.0, 0.0], [5.0, -1.0], [-4.0, -7.0]], [[-6.0, -7.0], [-4.0, -7.0], [-7.0, 0.0]], [[-6.0, 0.0], [-1.0, 1.0], [6.0, 6.0]]], [[[3.0, 0.0], [4.0, -1.0], [-7.0, 0.0]], [[6.0, 1.0], [0.0, -6.0], [3.0, -3.0]], [[-7.0, 2.0], [4.0, 5.0], [6.0, 4.0]], [[-3.0, 2.0], [4.0, 6.0], [2.0, -2.0]]]])
      arg1=numpy.array([[[[1.0, 0.0], [0.0, 7.0], [1.0, 1.0]], [[5.0, -1.0], [-6.0, 0.0], [0.0, 6.0]], [[0.0, -3.0], [-1.0, 5.0], [7.0, -3.0]], [[-2.0, 1.0], [0.0, -2.0], [0.0, 0.0]]], [[[-1.0, -4.0], [-2.0, 1.0], [-5.0, -7.0]], [[2.0, 4.0], [4.0, -5.0], [4.0, 1.0]], [[4.0, 4.0], [2.0, 1.0], [2.0, 2.0]], [[-3.0, 6.0], [-5.0, 6.0], [-3.0, 1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([[-37.0, 11.0], [-24.0, 32.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank2_offset0(self):
      arg0=numpy.array([3.0, -5.0])
      arg1=numpy.array([[3.0, 6.0, -3.0, -4.0, 2.0], [-6.0, -1.0, 3.0, 0.0, 2.0], [4.0, 0.0, 4.0, 6.0, -3.0], [0.0, 3.0, -4.0, -2.0, -6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[9.0, 18.0, -9.0, -12.0, 6.0], [-18.0, -3.0, 9.0, 0.0, 6.0], [12.0, 0.0, 12.0, 18.0, -9.0], [0.0, 9.0, -12.0, -6.0, -18.0]], [[-15.0, -30.0, 15.0, 20.0, -10.0], [30.0, 5.0, -15.0, 0.0, -10.0], [-20.0, 0.0, -20.0, -30.0, 15.0], [0.0, -15.0, 20.0, 10.0, 30.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank3_offset1(self):
      arg0=numpy.array([[3.0, -7.0], [-2.0, -1.0], [-4.0, -1.0]])
      arg1=numpy.array([[[-7.0, -4.0, 5.0, 7.0, 3.0], [0.0, -4.0, 6.0, 5.0, 6.0], [1.0, 5.0, 5.0, 2.0, 3.0], [-6.0, 4.0, 4.0, 2.0, -1.0]], [[-7.0, 1.0, 2.0, -4.0, -2.0], [7.0, -4.0, 0.0, 0.0, 2.0], [5.0, -7.0, -6.0, 7.0, 0.0], [-1.0, -2.0, 4.0, -7.0, -3.0]], [[-1.0, -4.0, 5.0, -6.0, 1.0], [3.0, 5.0, 3.0, 2.0, 3.0], [-2.0, -5.0, -5.0, 0.0, -1.0], [0.0, 0.0, 1.0, 1.0, -7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-3.0, 2.0, -9.0, 53.0, 9.0], [-26.0, -24.0, 6.0, 7.0, 2.0], [1.0, 49.0, 47.0, -8.0, 13.0], [-16.0, 16.0, 0.0, 16.0, 31.0]], [[57.0, 31.0, -42.0, -39.0, -20.0], [-10.0, 27.0, -45.0, -37.0, -47.0], [-10.0, -23.0, -24.0, -21.0, -20.0], [43.0, -26.0, -33.0, -8.0, 17.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank4_offset2(self):
      arg0=numpy.array([[[0.0, 7.0], [0.0, 3.0], [-5.0, 3.0]], [[3.0, -2.0], [0.0, 5.0], [0.0, -1.0]]])
      arg1=numpy.array([[[[0.0, 2.0, -2.0, 7.0, 5.0], [6.0, 2.0, 3.0, 0.0, 0.0], [4.0, 5.0, -1.0, 0.0, -4.0], [2.0, -6.0, 7.0, 3.0, 0.0]], [[-6.0, 0.0, -7.0, -4.0, 6.0], [6.0, 3.0, 1.0, 2.0, 7.0], [-6.0, 6.0, 3.0, -6.0, 1.0], [-7.0, -6.0, 7.0, -3.0, -1.0]], [[2.0, -7.0, 6.0, -2.0, -3.0], [5.0, -6.0, -5.0, -2.0, 5.0], [0.0, 3.0, 6.0, 5.0, 1.0], [-7.0, -3.0, 5.0, 7.0, 3.0]]], [[[3.0, -7.0, -5.0, 0.0, 6.0], [0.0, -6.0, -1.0, 6.0, 2.0], [-4.0, -3.0, 2.0, 0.0, -5.0], [1.0, 5.0, 1.0, 4.0, -5.0]], [[3.0, 5.0, 0.0, 0.0, 3.0], [0.0, 7.0, -1.0, 0.0, 4.0], [-4.0, 6.0, 1.0, -2.0, 7.0], [1.0, 1.0, -1.0, 0.0, 5.0]], [[-7.0, 2.0, -4.0, 4.0, 1.0], [5.0, -3.0, 6.0, 1.0, -3.0], [0.0, 4.0, 5.0, 3.0, 4.0], [7.0, 0.0, -3.0, 4.0, -3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[-1.0, 14.0, -45.0, 10.0, 33.0], [-25.0, 12.0, 22.0, 28.0, -19.0], [-12.0, -24.0, -24.0, -25.0, -20.0], [38.0, 30.0, -22.0, -23.0, -30.0]], [[4.0, 30.0, -3.0, 27.0, 46.0], [70.0, 55.0, 0.0, -13.0, 55.0], [-2.0, 94.0, 16.0, -16.0, 19.0], [-32.0, -74.0, 81.0, 21.0, 44.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_array_rank3_offset0(self):
      arg0=numpy.array([-3.0, 5.0])
      arg1=numpy.array([[[-6.0, 0.0], [-5.0, 5.0]], [[6.0, 4.0], [0.0, 1.0]], [[-1.0, -7.0], [-1.0, 7.0]], [[-4.0, -6.0], [6.0, 0.0]], [[-1.0, 0.0], [2.0, -4.0]], [[-6.0, -3.0], [7.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[18.0, 0.0], [15.0, -15.0]], [[-18.0, -12.0], [0.0, -3.0]], [[3.0, 21.0], [3.0, -21.0]], [[12.0, 18.0], [-18.0, 0.0]], [[3.0, 0.0], [-6.0, 12.0]], [[18.0, 9.0], [-21.0, -3.0]]], [[[-30.0, 0.0], [-25.0, 25.0]], [[30.0, 20.0], [0.0, 5.0]], [[-5.0, -35.0], [-5.0, 35.0]], [[-20.0, -30.0], [30.0, 0.0]], [[-5.0, 0.0], [10.0, -20.0]], [[-30.0, -15.0], [35.0, 5.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank4_offset1(self):
      arg0=numpy.array([[1.0, -1.0], [-5.0, 0.0], [1.0, -7.0]])
      arg1=numpy.array([[[[2.0, -2.0], [1.0, -6.0]], [[0.0, -7.0], [-3.0, -3.0]], [[-2.0, 7.0], [2.0, 4.0]], [[-6.0, 3.0], [4.0, 4.0]], [[0.0, 4.0], [3.0, -7.0]], [[-6.0, 2.0], [3.0, -3.0]]], [[[0.0, -2.0], [-3.0, 5.0]], [[-6.0, 1.0], [2.0, -3.0]], [[-6.0, -5.0], [0.0, -2.0]], [[0.0, -4.0], [3.0, -4.0]], [[7.0, -1.0], [-7.0, 0.0]], [[-4.0, -5.0], [-2.0, 0.0]]], [[[-2.0, -2.0], [-6.0, -2.0]], [[7.0, 7.0], [-6.0, -4.0]], [[-5.0, 0.0], [-2.0, 4.0]], [[-2.0, 7.0], [-2.0, 7.0]], [[-5.0, -2.0], [1.0, -7.0]], [[-3.0, -5.0], [-5.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[0.0, 6.0], [10.0, -33.0]], [[37.0, -5.0], [-19.0, 8.0]], [[23.0, 32.0], [0.0, 18.0]], [[-8.0, 30.0], [-13.0, 31.0]], [[-40.0, 7.0], [39.0, -14.0]], [[11.0, 22.0], [8.0, 3.0]]], [[[12.0, 16.0], [41.0, 20.0]], [[-49.0, -42.0], [45.0, 31.0]], [[37.0, -7.0], [12.0, -32.0]], [[20.0, -52.0], [10.0, -53.0]], [[35.0, 10.0], [-10.0, 56.0]], [[27.0, 33.0], [32.0, -39.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank0_offset0(self):
      arg0=numpy.array([1.0, 0.0])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-3.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-3.0, -0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank1_offset1(self):
      arg0=numpy.array([[-1.0, 2.0], [0.0, -7.0], [6.0, -6.0]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-2.0, 5.0, -6.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-34.0, -3.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[7.0, -7.0], [-5.0, 7.0], [7.0, 5.0]], [[-5.0, -5.0], [-5.0, 6.0], [-2.0, 5.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-5.0, -5.0, 0.0], [-1.0, -6.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([11.0, 4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[[7.0, 6.0], [1.0, 1.0], [0.0, -2.0]], [[-3.0, 1.0], [4.0, 7.0], [5.0, -6.0]], [[-3.0, 7.0], [4.0, -4.0], [-7.0, 1.0]], [[-1.0, -4.0], [-1.0, 0.0], [-3.0, 1.0]]], [[[4.0, -3.0], [-6.0, 7.0], [5.0, -3.0]], [[-5.0, -7.0], [-6.0, -3.0], [3.0, 4.0]], [[2.0, -1.0], [-3.0, 0.0], [-6.0, -4.0]], [[-6.0, 5.0], [1.0, -7.0], [5.0, 5.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-7.0, 7.0, -1.0], [3.0, 2.0, 0.0], [-2.0, 3.0, -5.0], [-3.0, -2.0, 0.0]], [[-1.0, 1.0, 0.0], [4.0, -6.0, 1.0], [1.0, 3.0, -7.0], [7.0, 7.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([59.0, 17.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank1_offset0(self):
      arg0=numpy.array([2.0, -1.0])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([5.0, -5.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[10.0, -10.0], [-5.0, 5.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank2_offset1(self):
      arg0=numpy.array([[3.0, 1.0], [7.0, 5.0], [-1.0, -6.0]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[0.0, -2.0], [-3.0, -5.0], [3.0, -5.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-24.0, -36.0], [-33.0, 3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[0.0, 2.0], [5.0, -4.0], [5.0, -5.0]], [[-2.0, 3.0], [0.0, 3.0], [2.0, -5.0]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[5.0, 3.0], [2.0, 3.0], [-4.0, 4.0]], [[1.0, -1.0], [1.0, -3.0], [5.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-2.0, 25.0], [3.0, -8.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[[5.0, 2.0], [-3.0, -4.0], [7.0, 0.0]], [[6.0, 3.0], [-4.0, -4.0], [-7.0, 6.0]], [[3.0, 1.0], [2.0, -2.0], [-4.0, -5.0]], [[0.0, 1.0], [1.0, 0.0], [1.0, -6.0]]], [[[0.0, -5.0], [-6.0, 0.0], [3.0, -5.0]], [[4.0, -3.0], [2.0, -7.0], [-5.0, 5.0]], [[2.0, -1.0], [-4.0, 1.0], [4.0, 5.0]], [[2.0, -4.0], [-4.0, 7.0], [-1.0, 3.0]]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[7.0, -7.0], [3.0, 6.0], [-4.0, -5.0]], [[0.0, 0.0], [3.0, 1.0], [0.0, -5.0]], [[7.0, 0.0], [-1.0, 2.0], [-2.0, -6.0]], [[-3.0, 4.0], [-1.0, 5.0], [-4.0, 6.0]]], [[[-2.0, 1.0], [0.0, 7.0], [7.0, 0.0]], [[5.0, 0.0], [-1.0, 0.0], [2.0, 0.0]], [[-3.0, -3.0], [-1.0, 5.0], [3.0, -2.0]], [[6.0, -3.0], [1.0, 2.0], [0.0, 1.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[55.0, -109.0], [7.0, -56.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank2_offset0(self):
      arg0=numpy.array([-7.0, 6.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[1.0, 6.0, 4.0, 5.0, -4.0], [7.0, -5.0, -5.0, -6.0, 7.0], [-7.0, 3.0, -7.0, -6.0, 0.0], [2.0, -3.0, 0.0, 5.0, -5.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-7.0, -42.0, -28.0, -35.0, 28.0], [-49.0, 35.0, 35.0, 42.0, -49.0], [49.0, -21.0, 49.0, 42.0, 0.0], [-14.0, 21.0, 0.0, -35.0, 35.0]], [[6.0, 36.0, 24.0, 30.0, -24.0], [42.0, -30.0, -30.0, -36.0, 42.0], [-42.0, 18.0, -42.0, -36.0, 0.0], [12.0, -18.0, 0.0, 30.0, -30.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank3_offset1(self):
      arg0=numpy.array([[-2.0, 4.0], [3.0, -3.0], [5.0, 4.0]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-2.0, 3.0, 0.0, 4.0, 1.0], [-2.0, 0.0, 3.0, -5.0, -7.0], [4.0, 1.0, -2.0, 6.0, -1.0], [-2.0, 3.0, 5.0, -7.0, 3.0]], [[-1.0, 1.0, 5.0, -4.0, -3.0], [4.0, -2.0, 0.0, 5.0, -7.0], [-2.0, -1.0, 0.0, -2.0, -2.0], [2.0, 0.0, -5.0, 0.0, 3.0]], [[0.0, -6.0, 2.0, -5.0, 6.0], [-7.0, 1.0, -7.0, 1.0, -2.0], [-6.0, -6.0, 4.0, 7.0, 0.0], [-3.0, 5.0, -1.0, 7.0, -3.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.0, -33.0, 25.0, -45.0, 19.0], [-19.0, -1.0, -41.0, 30.0, -17.0], [-44.0, -35.0, 24.0, 17.0, -4.0], [-5.0, 19.0, -30.0, 49.0, -12.0]], [[-5.0, -15.0, -7.0, 8.0, 37.0], [-48.0, 10.0, -16.0, -31.0, -15.0], [-2.0, -17.0, 8.0, 58.0, 2.0], [-26.0, 32.0, 31.0, 0.0, -9.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[-3.0, -3.0], [-1.0, 3.0], [4.0, 4.0]], [[-2.0, -3.0], [5.0, 7.0], [7.0, 3.0]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-3.0, 2.0, 2.0, -7.0, 4.0], [-3.0, 0.0, -2.0, -6.0, 7.0], [6.0, -4.0, 4.0, 4.0, 1.0], [1.0, 5.0, 0.0, -4.0, -4.0]], [[-4.0, -6.0, 5.0, 0.0, -3.0], [-1.0, -6.0, -2.0, 4.0, 7.0], [6.0, 3.0, 3.0, -7.0, -4.0], [-7.0, 7.0, 5.0, 0.0, -2.0]], [[-5.0, 7.0, -2.0, 5.0, -1.0], [-4.0, -6.0, -3.0, 0.0, 0.0], [5.0, 2.0, 1.0, 1.0, 1.0], [7.0, -2.0, -4.0, 6.0, 5.0]]], [[[6.0, 0.0, 7.0, 3.0, 7.0], [-1.0, 7.0, -6.0, 7.0, 2.0], [6.0, 0.0, -3.0, 4.0, -6.0], [0.0, 0.0, 5.0, 1.0, -5.0]], [[3.0, -6.0, 0.0, -2.0, 6.0], [7.0, 3.0, -7.0, -7.0, -4.0], [3.0, -1.0, 1.0, 1.0, -5.0], [-2.0, -4.0, 5.0, 0.0, 1.0]], [[-6.0, 3.0, 6.0, 4.0, 0.0], [-7.0, -4.0, 7.0, -6.0, -1.0], [-5.0, 7.0, -4.0, 3.0, 2.0], [3.0, -4.0, 1.0, -4.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-46.0, 19.0, 9.0, 53.0, 3.0], [-18.0, -45.0, 22.0, -77.0, -59.0], [-36.0, 61.0, -28.0, 17.0, 6.0], [43.0, -78.0, 1.0, 6.0, 21.0]], [[-38.0, -29.0, -2.0, 30.0, -4.0], [21.0, -54.0, -22.0, -58.0, -37.0], [8.0, 43.0, 5.0, -25.0, -22.0], [-1.0, -42.0, 22.0, 21.0, 36.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_Symbol_rank3_offset0(self):
      arg0=numpy.array([0.0, 5.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[6.0, -4.0], [7.0, 1.0]], [[-4.0, 4.0], [-2.0, 0.0]], [[-5.0, -1.0], [0.0, 6.0]], [[-6.0, -5.0], [-4.0, 1.0]], [[-6.0, 2.0], [3.0, -4.0]], [[-6.0, 7.0], [-4.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[30.0, -20.0], [35.0, 5.0]], [[-20.0, 20.0], [-10.0, 0.0]], [[-25.0, -5.0], [0.0, 30.0]], [[-30.0, -25.0], [-20.0, 5.0]], [[-30.0, 10.0], [15.0, -20.0]], [[-30.0, 35.0], [-20.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank4_offset1(self):
      arg0=numpy.array([[4.0, 1.0], [5.0, -3.0], [3.0, -4.0]])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[3.0, -7.0], [2.0, -5.0]], [[3.0, -4.0], [-1.0, 1.0]], [[0.0, 7.0], [4.0, 5.0]], [[6.0, -3.0], [4.0, 1.0]], [[7.0, -1.0], [5.0, -7.0]], [[-6.0, 0.0], [4.0, 3.0]]], [[[0.0, -6.0], [-1.0, 1.0]], [[2.0, 6.0], [-3.0, 6.0]], [[7.0, -7.0], [-3.0, -5.0]], [[-4.0, 2.0], [-5.0, 4.0]], [[1.0, 3.0], [6.0, 0.0]], [[-2.0, -7.0], [-3.0, -6.0]]], [[[2.0, 6.0], [-2.0, -2.0]], [[2.0, -7.0], [7.0, -4.0]], [[4.0, 0.0], [-2.0, -3.0]], [[6.0, 4.0], [-1.0, 2.0]], [[-2.0, -2.0], [-4.0, -7.0]], [[3.0, -5.0], [-5.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[18.0, -40.0], [-3.0, -21.0]], [[28.0, -7.0], [2.0, 22.0]], [[47.0, -7.0], [-5.0, -14.0]], [[22.0, 10.0], [-12.0, 30.0]], [[27.0, 5.0], [38.0, -49.0]], [[-25.0, -50.0], [-14.0, -6.0]]], [[[-5.0, -13.0], [13.0, 0.0]], [[-11.0, 6.0], [-20.0, -1.0]], [[-37.0, 28.0], [21.0, 32.0]], [[-6.0, -25.0], [23.0, -19.0]], [[12.0, -2.0], [3.0, 21.0]], [[-12.0, 41.0], [33.0, 5.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank0_offset0(self):
      arg0=numpy.array([-1.0, 3.0])
      arg1=Data(0.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-0.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank1_offset1(self):
      arg0=numpy.array([[1.0, 3.0], [1.0, -2.0], [5.0, 0.0]])
      arg1=Data(numpy.array([2.0, -1.0, 4.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([21.0, 8.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank2_offset2(self):
      arg0=numpy.array([[[-2.0, 0.0], [-5.0, -1.0], [-3.0, -4.0]], [[5.0, -5.0], [7.0, 7.0], [4.0, -6.0]]])
      arg1=Data(numpy.array([[-7.0, -3.0, -5.0], [-4.0, -5.0, -5.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-31.0, 38.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank3_offset3(self):
      arg0=numpy.array([[[[-7.0, 2.0], [5.0, -2.0], [6.0, -6.0]], [[-7.0, -2.0], [-6.0, 7.0], [5.0, 7.0]], [[1.0, 4.0], [4.0, -3.0], [-7.0, -2.0]], [[-3.0, 1.0], [-7.0, 3.0], [4.0, -5.0]]], [[[4.0, -1.0], [2.0, -5.0], [1.0, -7.0]], [[-6.0, 0.0], [-3.0, 4.0], [7.0, -4.0]], [[7.0, -3.0], [-7.0, -1.0], [7.0, 1.0]], [[0.0, -3.0], [5.0, -5.0], [-1.0, 3.0]]]])
      arg1=Data(numpy.array([[[2.0, 7.0, -6.0], [-2.0, -3.0, 6.0], [0.0, -2.0, -4.0], [-2.0, 7.0, 6.0]], [[-5.0, -2.0, -7.0], [-3.0, 0.0, 5.0], [-1.0, 1.0, -1.0], [-2.0, -3.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([34.0, 120.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank1_offset0(self):
      arg0=numpy.array([-4.0, -4.0])
      arg1=Data(numpy.array([-6.0, 2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[24.0, -8.0], [24.0, -8.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank2_offset1(self):
      arg0=numpy.array([[-6.0, 1.0], [0.0, -2.0], [-7.0, -5.0]])
      arg1=Data(numpy.array([[-7.0, -2.0], [-6.0, 1.0], [-7.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[91.0, 12.0], [40.0, -4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank3_offset2(self):
      arg0=numpy.array([[[0.0, 2.0], [2.0, 5.0], [3.0, 1.0]], [[-2.0, -3.0], [-3.0, 0.0], [-2.0, -1.0]]])
      arg1=Data(numpy.array([[[1.0, 6.0], [-7.0, -3.0], [-7.0, 2.0]], [[-1.0, 7.0], [1.0, -2.0], [7.0, -6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-50.0, 4.0], [-44.0, -16.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank4_offset3(self):
      arg0=numpy.array([[[[0.0, -1.0], [-6.0, 1.0], [0.0, 2.0]], [[-5.0, 0.0], [-2.0, 5.0], [2.0, -2.0]], [[-4.0, -1.0], [-3.0, 1.0], [6.0, 1.0]], [[-1.0, 3.0], [2.0, 0.0], [0.0, 5.0]]], [[[0.0, 6.0], [-3.0, -1.0], [-7.0, -7.0]], [[-3.0, 0.0], [-1.0, -6.0], [2.0, -6.0]], [[-5.0, 3.0], [-4.0, -3.0], [-2.0, -2.0]], [[3.0, 0.0], [4.0, -1.0], [-3.0, 0.0]]]])
      arg1=Data(numpy.array([[[[3.0, 7.0], [-2.0, 2.0], [2.0, -1.0]], [[0.0, 1.0], [4.0, 5.0], [-6.0, -5.0]], [[1.0, -2.0], [3.0, -6.0], [-2.0, 6.0]], [[7.0, -1.0], [7.0, 1.0], [-3.0, -3.0]]], [[[3.0, -1.0], [2.0, -3.0], [2.0, -4.0]], [[5.0, 3.0], [0.0, -5.0], [-4.0, -6.0]], [[0.0, 0.0], [-5.0, -5.0], [4.0, 1.0]], [[-4.0, -2.0], [-2.0, -2.0], [-2.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[-71.0, 53.0], [72.0, 118.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank2_offset0(self):
      arg0=numpy.array([5.0, -6.0])
      arg1=Data(numpy.array([[0.0, 6.0, -4.0, -6.0, -5.0], [4.0, 2.0, 7.0, 0.0, 0.0], [-1.0, -1.0, -6.0, -1.0, 1.0], [1.0, -7.0, 6.0, -1.0, -3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 30.0, -20.0, -30.0, -25.0], [20.0, 10.0, 35.0, 0.0, 0.0], [-5.0, -5.0, -30.0, -5.0, 5.0], [5.0, -35.0, 30.0, -5.0, -15.0]], [[0.0, -36.0, 24.0, 36.0, 30.0], [-24.0, -12.0, -42.0, 0.0, 0.0], [6.0, 6.0, 36.0, 6.0, -6.0], [-6.0, 42.0, -36.0, 6.0, 18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank3_offset1(self):
      arg0=numpy.array([[-6.0, 7.0], [3.0, 4.0], [-7.0, -3.0]])
      arg1=Data(numpy.array([[[5.0, 3.0, 4.0, 3.0, 4.0], [4.0, 3.0, -4.0, -5.0, -6.0], [2.0, -6.0, -6.0, 2.0, -1.0], [0.0, -6.0, 1.0, -1.0, -1.0]], [[0.0, -3.0, 0.0, -1.0, 6.0], [7.0, -3.0, -2.0, -3.0, 2.0], [-1.0, 1.0, 7.0, -5.0, -5.0], [-1.0, 6.0, 6.0, 2.0, -5.0]], [[1.0, 4.0, -3.0, 6.0, -1.0], [5.0, -2.0, -6.0, 1.0, 2.0], [-5.0, 1.0, -7.0, 7.0, -3.0], [5.0, 1.0, 5.0, 7.0, -4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-37.0, -55.0, -3.0, -63.0, 1.0], [-38.0, -13.0, 60.0, 14.0, 28.0], [20.0, 32.0, 106.0, -76.0, 12.0], [-38.0, 47.0, -23.0, -37.0, 19.0]], [[32.0, -3.0, 37.0, -1.0, 55.0], [41.0, 15.0, -18.0, -50.0, -40.0], [25.0, -41.0, 7.0, -27.0, -18.0], [-19.0, -21.0, 16.0, -20.0, -15.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank4_offset2(self):
      arg0=numpy.array([[[4.0, -3.0], [-3.0, 6.0], [-4.0, 5.0]], [[1.0, 4.0], [1.0, -1.0], [6.0, -7.0]]])
      arg1=Data(numpy.array([[[[7.0, 1.0, -4.0, -6.0, -7.0], [0.0, -2.0, -4.0, -2.0, 1.0], [-3.0, 1.0, -6.0, -3.0, -7.0], [-2.0, 7.0, 6.0, 7.0, 5.0]], [[7.0, -3.0, -4.0, 3.0, 0.0], [-4.0, 4.0, 0.0, 0.0, 4.0], [1.0, -3.0, 3.0, -6.0, -2.0], [-4.0, 4.0, 5.0, -2.0, -2.0]], [[-3.0, -2.0, 5.0, 7.0, -4.0], [2.0, -5.0, 1.0, -1.0, 1.0], [-3.0, 2.0, -6.0, 0.0, -1.0], [0.0, -1.0, -5.0, -2.0, 4.0]]], [[[5.0, 4.0, 5.0, 0.0, -5.0], [1.0, 5.0, -2.0, 5.0, 5.0], [1.0, 1.0, -4.0, -1.0, 5.0], [4.0, -6.0, 6.0, -6.0, 3.0]], [[0.0, 0.0, 2.0, 7.0, 6.0], [3.0, -4.0, 7.0, -6.0, -4.0], [-5.0, 0.0, 3.0, -3.0, -7.0], [1.0, -6.0, -2.0, 1.0, 3.0]], [[7.0, 6.0, 0.0, 0.0, -2.0], [0.0, -3.0, -1.0, -5.0, -6.0], [0.0, -4.0, 6.0, -7.0, -5.0], [5.0, 6.0, -3.0, -4.0, 7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[66.0, 61.0, -17.0, -54.0, -23.0], [8.0, -17.0, -21.0, -35.0, -47.0], [-7.0, -18.0, 26.0, -40.0, -50.0], [39.0, 44.0, 15.0, 13.0, 58.0]], [[-23.0, -57.0, 31.0, 64.0, -11.0], [-13.0, 50.0, 9.0, 62.0, 92.0], [9.0, 21.0, -55.0, 21.0, 66.0], [-38.0, -62.0, 34.0, -40.0, -47.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_constData_rank3_offset0(self):
      arg0=numpy.array([-1.0, 6.0])
      arg1=Data(numpy.array([[[2.0, -6.0], [7.0, -5.0]], [[2.0, 7.0], [-6.0, -1.0]], [[6.0, 4.0], [-2.0, 2.0]], [[-4.0, 0.0], [2.0, 0.0]], [[4.0, -5.0], [-1.0, 6.0]], [[0.0, -1.0], [-1.0, -7.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-2.0, 6.0], [-7.0, 5.0]], [[-2.0, -7.0], [6.0, 1.0]], [[-6.0, -4.0], [2.0, -2.0]], [[4.0, 0.0], [-2.0, 0.0]], [[-4.0, 5.0], [1.0, -6.0]], [[0.0, 1.0], [1.0, 7.0]]], [[[12.0, -36.0], [42.0, -30.0]], [[12.0, 42.0], [-36.0, -6.0]], [[36.0, 24.0], [-12.0, 12.0]], [[-24.0, 0.0], [12.0, 0.0]], [[24.0, -30.0], [-6.0, 36.0]], [[0.0, -6.0], [-6.0, -42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank4_offset1(self):
      arg0=numpy.array([[-3.0, -2.0], [4.0, 1.0], [6.0, 0.0]])
      arg1=Data(numpy.array([[[[-3.0, -7.0], [7.0, -6.0]], [[-5.0, -5.0], [6.0, -2.0]], [[7.0, 0.0], [-2.0, -1.0]], [[-1.0, -4.0], [1.0, -3.0]], [[3.0, 7.0], [-7.0, -1.0]], [[-6.0, 0.0], [5.0, 2.0]]], [[[-6.0, 3.0], [5.0, -3.0]], [[1.0, 1.0], [-4.0, -7.0]], [[2.0, -6.0], [3.0, 7.0]], [[2.0, 1.0], [-3.0, -1.0]], [[6.0, 7.0], [-4.0, 6.0]], [[-1.0, -5.0], [-4.0, -7.0]]], [[[-2.0, -5.0], [1.0, -1.0]], [[-2.0, 3.0], [3.0, -7.0]], [[5.0, 1.0], [0.0, -1.0]], [[-4.0, -3.0], [-7.0, -4.0]], [[-7.0, 6.0], [7.0, 3.0]], [[3.0, -6.0], [5.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-27.0, 3.0], [5.0, 0.0]], [[7.0, 37.0], [-16.0, -64.0]], [[17.0, -18.0], [18.0, 25.0]], [[-13.0, -2.0], [-57.0, -19.0]], [[-27.0, 43.0], [47.0, 45.0]], [[32.0, -56.0], [-1.0, -34.0]]], [[[0.0, 17.0], [-9.0, 9.0]], [[11.0, 11.0], [-16.0, -3.0]], [[-12.0, -6.0], [7.0, 9.0]], [[4.0, 9.0], [-5.0, 5.0]], [[0.0, -7.0], [10.0, 8.0]], [[11.0, -5.0], [-14.0, -11.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank0_offset0(self):
      arg0=numpy.array([6.0, 5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-2.0)+(1-msk_arg1)*(-2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-12.0, -10.0])+(1.-msk_ref)*numpy.array([-12.0, -10.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank1_offset1(self):
      arg0=numpy.array([[-2.0, 0.0], [7.0, 6.0], [3.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, -7.0, 1.0])+(1.-msk_arg1)*numpy.array([0.0, 6.0, -3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-54.0, -39.0])+(1.-msk_ref)*numpy.array([33.0, 27.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[5.0, 1.0], [0.0, 0.0], [-4.0, -6.0]], [[7.0, -4.0], [-3.0, -7.0], [6.0, 3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, -5.0, -4.0], [-5.0, 3.0, -4.0]])+(1.-msk_arg1)*numpy.array([[5.0, 0.0, 0.0], [6.0, 7.0, -2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-22.0, 17.0])+(1.-msk_ref)*numpy.array([34.0, -74.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[[-5.0, -1.0], [-3.0, 1.0], [-5.0, 4.0]], [[2.0, -6.0], [4.0, 6.0], [3.0, -1.0]], [[3.0, -4.0], [1.0, 7.0], [-3.0, -5.0]], [[-3.0, -6.0], [4.0, -6.0], [-5.0, 1.0]]], [[[0.0, 0.0], [1.0, 1.0], [-3.0, 6.0]], [[7.0, -1.0], [0.0, -4.0], [4.0, 2.0]], [[4.0, 6.0], [3.0, 5.0], [-7.0, -2.0]], [[1.0, 1.0], [0.0, -6.0], [3.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 5.0, 1.0], [3.0, 0.0, -2.0], [0.0, 1.0, -7.0], [3.0, -3.0, -3.0]], [[5.0, 6.0, 0.0], [6.0, -4.0, -6.0], [5.0, 3.0, -3.0], [-1.0, -6.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[5.0, -5.0, 0.0], [-7.0, 7.0, 7.0], [-2.0, -4.0, 1.0], [0.0, 3.0, -1.0]], [[7.0, 3.0, 2.0], [4.0, 4.0, -6.0], [7.0, -7.0, -1.0], [-1.0, 0.0, -7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([72.0, 101.0])+(1.-msk_ref)*numpy.array([22.0, 35.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank1_offset0(self):
      arg0=numpy.array([0.0, -3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 4.0])+(1.-msk_arg1)*numpy.array([0.0, 4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 0.0], [-21.0, -12.0]])+(1.-msk_ref)*numpy.array([[0.0, 0.0], [0.0, -12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank2_offset1(self):
      arg0=numpy.array([[3.0, -7.0], [0.0, -6.0], [2.0, -2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, -3.0], [-2.0, 0.0], [-6.0, -2.0]])+(1.-msk_arg1)*numpy.array([[-3.0, -7.0], [-5.0, 4.0], [5.0, -2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, -13.0], [52.0, 25.0]])+(1.-msk_ref)*numpy.array([[1.0, -25.0], [41.0, 29.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[3.0, 3.0], [6.0, 0.0], [0.0, -6.0]], [[-6.0, 2.0], [6.0, -4.0], [7.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, -7.0], [-4.0, -5.0], [-1.0, 6.0]], [[6.0, -2.0], [5.0, -4.0], [-5.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 1.0], [7.0, 5.0], [-5.0, -2.0]], [[2.0, -4.0], [4.0, 0.0], [0.0, 7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-77.0, -63.0], [16.0, -45.0]])+(1.-msk_ref)*numpy.array([[45.0, 106.0], [9.0, -35.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[[5.0, -7.0], [7.0, 5.0], [7.0, 7.0]], [[-5.0, -1.0], [-5.0, -4.0], [-3.0, -7.0]], [[-6.0, 3.0], [1.0, -3.0], [5.0, 3.0]], [[3.0, 7.0], [-6.0, 2.0], [-5.0, 5.0]]], [[[0.0, 0.0], [-6.0, 3.0], [-3.0, 1.0]], [[0.0, -7.0], [5.0, -3.0], [0.0, 3.0]], [[0.0, 2.0], [-3.0, 0.0], [-7.0, 1.0]], [[2.0, 0.0], [-2.0, 3.0], [-6.0, 4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, 6.0], [-6.0, 0.0], [1.0, 0.0]], [[3.0, -4.0], [-4.0, 7.0], [0.0, 1.0]], [[-1.0, 3.0], [7.0, -5.0], [3.0, 6.0]], [[2.0, -4.0], [-5.0, -3.0], [0.0, 1.0]]], [[[-2.0, 2.0], [0.0, -6.0], [0.0, -4.0]], [[5.0, 6.0], [-5.0, -4.0], [0.0, -2.0]], [[-1.0, -1.0], [3.0, -6.0], [7.0, -5.0]], [[1.0, -6.0], [4.0, 3.0], [-5.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 1.0], [-5.0, 4.0], [6.0, -6.0]], [[1.0, 1.0], [0.0, -3.0], [1.0, 7.0]], [[-7.0, 0.0], [3.0, 2.0], [-1.0, 3.0]], [[1.0, 5.0], [4.0, 6.0], [7.0, 6.0]]], [[[5.0, -4.0], [-4.0, 1.0], [-4.0, 6.0]], [[-1.0, -5.0], [-6.0, 2.0], [7.0, -2.0]], [[-5.0, -4.0], [2.0, -4.0], [-6.0, 4.0]], [[-5.0, -2.0], [-3.0, 1.0], [6.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-55.0, 41.0], [-2.0, -88.0]])+(1.-msk_ref)*numpy.array([[-25.0, -84.0], [69.0, 40.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank2_offset0(self):
      arg0=numpy.array([-4.0, -3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 3.0, -3.0, -7.0, 4.0], [6.0, 0.0, -4.0, -2.0, -2.0], [-6.0, 2.0, 2.0, 7.0, -5.0], [-3.0, 2.0, 0.0, 1.0, -7.0]])+(1.-msk_arg1)*numpy.array([[0.0, 5.0, 2.0, -6.0, 7.0], [-2.0, -5.0, 7.0, -2.0, 6.0], [-4.0, 3.0, 7.0, -7.0, -7.0], [-5.0, -5.0, 0.0, -2.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[16.0, -12.0, 12.0, 28.0, -16.0], [-24.0, 0.0, 16.0, 8.0, 8.0], [24.0, -8.0, -8.0, -28.0, 20.0], [12.0, -8.0, 0.0, -4.0, 28.0]], [[12.0, -9.0, 9.0, 21.0, -12.0], [-18.0, 0.0, 12.0, 6.0, 6.0], [18.0, -6.0, -6.0, -21.0, 15.0], [9.0, -6.0, 0.0, -3.0, 21.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -20.0, -8.0, 24.0, -28.0], [8.0, 20.0, -28.0, 8.0, -24.0], [16.0, -12.0, -28.0, 28.0, 28.0], [20.0, 20.0, 0.0, 8.0, -16.0]], [[0.0, -15.0, -6.0, 18.0, -21.0], [6.0, 15.0, -21.0, 6.0, -18.0], [12.0, -9.0, -21.0, 21.0, 21.0], [15.0, 15.0, 0.0, 6.0, -12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank3_offset1(self):
      arg0=numpy.array([[-1.0, 4.0], [-4.0, 1.0], [-6.0, -1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, -2.0, 3.0, -7.0, 1.0], [4.0, -6.0, 4.0, -1.0, -1.0], [2.0, -7.0, -3.0, -3.0, -4.0], [7.0, 1.0, 3.0, 0.0, 4.0]], [[0.0, 6.0, -5.0, 6.0, -3.0], [1.0, -6.0, -1.0, -6.0, 4.0], [-5.0, 4.0, 1.0, 7.0, 4.0], [-2.0, -5.0, 1.0, 3.0, 0.0]], [[3.0, 3.0, 0.0, 3.0, 4.0], [-6.0, 0.0, 6.0, 4.0, -5.0], [-3.0, -1.0, -1.0, -3.0, 2.0], [4.0, 6.0, -6.0, -2.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 7.0, 6.0, 6.0, -1.0], [2.0, -5.0, -1.0, -3.0, -1.0], [1.0, 1.0, -7.0, -5.0, 5.0], [1.0, -1.0, 4.0, 6.0, -4.0]], [[0.0, -6.0, -5.0, -2.0, -3.0], [-6.0, 6.0, 0.0, 7.0, -6.0], [1.0, 0.0, -5.0, 5.0, 1.0], [-3.0, -1.0, 2.0, -6.0, 7.0]], [[6.0, 2.0, -4.0, -6.0, 1.0], [7.0, -1.0, -6.0, -5.0, -7.0], [-1.0, 0.0, 7.0, 3.0, -1.0], [-2.0, -2.0, 0.0, 2.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -40.0, 17.0, -35.0, -13.0], [28.0, 30.0, -36.0, 1.0, 15.0], [36.0, -3.0, 5.0, -7.0, -24.0], [-23.0, -17.0, 29.0, 0.0, -46.0]], [[-27.0, -5.0, 7.0, -25.0, -3.0], [23.0, -30.0, 9.0, -14.0, 5.0], [6.0, -23.0, -10.0, -2.0, -14.0], [22.0, -7.0, 19.0, 5.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, 5.0, 38.0, 38.0, 7.0], [-20.0, -13.0, 37.0, 5.0, 67.0], [1.0, -1.0, -15.0, -33.0, -3.0], [23.0, 17.0, -12.0, 6.0, -42.0]], [[-30.0, 20.0, 23.0, 28.0, -8.0], [-5.0, -13.0, 2.0, 0.0, -3.0], [6.0, 4.0, -40.0, -18.0, 22.0], [3.0, -3.0, 18.0, 16.0, -12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[-7.0, -1.0], [-7.0, 0.0], [-1.0, 2.0]], [[-1.0, -3.0], [6.0, 0.0], [1.0, -5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, 2.0, -1.0, -5.0, -1.0], [-7.0, -3.0, -5.0, 3.0, 5.0], [-2.0, -3.0, -4.0, 0.0, 1.0], [-5.0, 0.0, 3.0, 1.0, 3.0]], [[2.0, 3.0, 7.0, -4.0, -4.0], [-7.0, 2.0, 1.0, -7.0, 2.0], [3.0, -2.0, 7.0, 6.0, -1.0], [-2.0, 1.0, 5.0, -5.0, -1.0]], [[0.0, -5.0, -7.0, -6.0, 6.0], [-1.0, 0.0, -3.0, -4.0, 0.0], [0.0, -4.0, -3.0, 4.0, -3.0], [-2.0, -1.0, -5.0, 1.0, 5.0]]], [[[4.0, -3.0, -6.0, 5.0, 0.0], [7.0, -2.0, -6.0, -5.0, 0.0], [-2.0, -7.0, 1.0, -7.0, -4.0], [-7.0, 7.0, 2.0, -4.0, -3.0]], [[0.0, -4.0, 0.0, -4.0, 7.0], [7.0, 4.0, -1.0, 0.0, -7.0], [4.0, 7.0, 7.0, 1.0, -3.0], [1.0, 5.0, 5.0, -6.0, -2.0]], [[-5.0, 2.0, 3.0, -7.0, 2.0], [-6.0, 2.0, 1.0, -1.0, 5.0], [0.0, -5.0, 1.0, -7.0, -5.0], [6.0, 6.0, -2.0, -1.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, 0.0, -7.0, 4.0, -4.0], [0.0, -5.0, 0.0, -1.0, 0.0], [-3.0, -3.0, 0.0, 7.0, -6.0], [-2.0, 1.0, -1.0, -3.0, -5.0]], [[-3.0, -1.0, -1.0, 0.0, 0.0], [-5.0, -5.0, 4.0, -7.0, 0.0], [4.0, 5.0, -4.0, -7.0, -6.0], [5.0, 7.0, -1.0, 7.0, 3.0]], [[0.0, -1.0, -2.0, -1.0, 2.0], [6.0, -3.0, -5.0, -4.0, -1.0], [3.0, 0.0, -2.0, -4.0, -4.0], [0.0, -6.0, 0.0, 3.0, 3.0]]], [[[-2.0, 3.0, -4.0, -4.0, 6.0], [-2.0, 5.0, 2.0, 7.0, 5.0], [-4.0, 6.0, 3.0, -2.0, 7.0], [-3.0, -7.0, -7.0, -3.0, -7.0]], [[-4.0, -3.0, -3.0, 7.0, 0.0], [1.0, 4.0, -2.0, -6.0, -4.0], [1.0, -5.0, -7.0, -6.0, 7.0], [-2.0, 2.0, -4.0, -3.0, 6.0]], [[5.0, -3.0, -5.0, -3.0, 0.0], [3.0, -2.0, -6.0, 0.0, -3.0], [3.0, -7.0, 5.0, 6.0, 0.0], [-7.0, 7.0, 0.0, -1.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, -49.0, -26.0, 33.0, 73.0], [128.0, 35.0, 32.0, 36.0, -86.0], [19.0, 83.0, 24.0, -40.0, -16.0], [70.0, 23.0, -25.0, -6.0, -28.0]], [[15.0, -13.0, -10.0, 13.0, 3.0], [14.0, -1.0, 12.0, 9.0, -30.0], [8.0, 41.0, -10.0, 64.0, 30.0], [-8.0, -53.0, -9.0, 18.0, 16.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, -16.0, 39.0, 16.0, 20.0], [40.0, 90.0, -43.0, 17.0, -31.0], [3.0, -57.0, -10.0, -24.0, 123.0], [-37.0, -24.0, -3.0, -47.0, 49.0]], [[-23.0, 4.0, 40.0, 21.0, -10.0], [3.0, -6.0, 14.0, -28.0, -2.0], [6.0, 20.0, -38.0, -39.0, -23.0], [46.0, -27.0, 22.0, 23.0, 57.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank1_expandedData_rank3_offset0(self):
      arg0=numpy.array([0.0, -3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, 0.0], [0.0, 5.0]], [[7.0, -3.0], [5.0, -5.0]], [[-6.0, 0.0], [-6.0, 0.0]], [[-1.0, 1.0], [-5.0, 6.0]], [[-6.0, -4.0], [6.0, 6.0]], [[3.0, 6.0], [2.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, -5.0], [-5.0, -7.0]], [[5.0, 0.0], [6.0, -3.0]], [[0.0, 5.0], [2.0, -5.0]], [[5.0, -6.0], [0.0, -3.0]], [[-3.0, 7.0], [6.0, -6.0]], [[4.0, 3.0], [-6.0, -7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[-18.0, 0.0], [0.0, -15.0]], [[-21.0, 9.0], [-15.0, 15.0]], [[18.0, 0.0], [18.0, 0.0]], [[3.0, -3.0], [15.0, -18.0]], [[18.0, 12.0], [-18.0, -18.0]], [[-9.0, -18.0], [-6.0, 21.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[-3.0, 15.0], [15.0, 21.0]], [[-15.0, 0.0], [-18.0, 9.0]], [[0.0, -15.0], [-6.0, 15.0]], [[-15.0, 18.0], [0.0, 9.0]], [[9.0, -21.0], [-18.0, 18.0]], [[-12.0, -9.0], [18.0, 21.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank4_offset1(self):
      arg0=numpy.array([[-1.0, 3.0], [-2.0, 4.0], [-4.0, -3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -3.0], [7.0, 1.0]], [[3.0, -7.0], [-3.0, 1.0]], [[2.0, 7.0], [1.0, -1.0]], [[-4.0, 4.0], [0.0, -2.0]], [[3.0, 0.0], [6.0, -7.0]], [[-4.0, -4.0], [0.0, 4.0]]], [[[-3.0, 2.0], [3.0, 5.0]], [[1.0, -3.0], [0.0, -1.0]], [[-1.0, -1.0], [5.0, 6.0]], [[0.0, 1.0], [-1.0, 5.0]], [[-7.0, -7.0], [0.0, -6.0]], [[7.0, -5.0], [4.0, 2.0]]], [[[0.0, 7.0], [3.0, -4.0]], [[0.0, 4.0], [5.0, 7.0]], [[-2.0, -3.0], [-1.0, 4.0]], [[-6.0, 0.0], [0.0, -4.0]], [[7.0, -5.0], [-1.0, -6.0]], [[-7.0, -4.0], [0.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 6.0], [3.0, 5.0]], [[7.0, 6.0], [-4.0, 5.0]], [[-4.0, 0.0], [-2.0, -1.0]], [[1.0, -1.0], [-2.0, 3.0]], [[1.0, 0.0], [-6.0, -5.0]], [[2.0, 2.0], [-6.0, -6.0]]], [[[-1.0, -4.0], [2.0, 5.0]], [[-6.0, 2.0], [0.0, 2.0]], [[-7.0, 1.0], [-6.0, -3.0]], [[6.0, -3.0], [7.0, -5.0]], [[-3.0, 0.0], [-3.0, -5.0]], [[0.0, -4.0], [0.0, 2.0]]], [[[2.0, 0.0], [0.0, 4.0]], [[-4.0, -3.0], [0.0, 5.0]], [[3.0, -2.0], [7.0, 0.0]], [[-3.0, -2.0], [-3.0, 5.0]], [[-2.0, 4.0], [-3.0, 0.0]], [[1.0, 2.0], [5.0, 0.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, -29.0], [-25.0, 5.0]], [[-5.0, -3.0], [-17.0, -27.0]], [[8.0, 7.0], [-7.0, -27.0]], [[28.0, -6.0], [2.0, 8.0]], [[-17.0, 34.0], [-2.0, 43.0]], [[18.0, 30.0], [-8.0, 4.0]]], [[[-18.0, -22.0], [24.0, 35.0]], [[13.0, -45.0], [-24.0, -22.0]], [[8.0, 26.0], [26.0, 9.0]], [[6.0, 16.0], [-4.0, 26.0]], [[-40.0, -13.0], [21.0, -27.0]], [[37.0, -20.0], [16.0, 29.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, 2.0], [-7.0, -31.0]], [[21.0, 2.0], [4.0, -29.0]], [[6.0, 6.0], [-14.0, 7.0]], [[-1.0, 15.0], [0.0, -13.0]], [[13.0, -16.0], [24.0, 15.0]], [[-6.0, -2.0], [-14.0, 2.0]]], [[[-10.0, 2.0], [17.0, 23.0]], [[9.0, 35.0], [-12.0, 8.0]], [[-49.0, 10.0], [-51.0, -15.0]], [[36.0, -9.0], [31.0, -26.0]], [[-3.0, -12.0], [-21.0, -35.0]], [[3.0, -16.0], [-33.0, -10.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_float_rank0_offset0(self):
      arg0=numpy.array([[-2.0, -5.0, 1.0, -3.0, -2.0], [3.0, 1.0, -5.0, 4.0, -5.0], [-1.0, -2.0, -7.0, 4.0, 3.0], [0.0, 5.0, 3.0, 3.0, 5.0]])
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-0.0, -0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank0_offset0(self):
      arg0=numpy.array([[4.0, 0.0, -1.0, -6.0, -4.0], [-7.0, 0.0, -4.0, -4.0, 4.0], [-5.0, 0.0, 3.0, -6.0, 2.0], [1.0, 7.0, -2.0, 6.0, 4.0]])
      arg1=numpy.array(-5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-20.0, -0.0, 5.0, 30.0, 20.0], [35.0, -0.0, 20.0, 20.0, -20.0], [25.0, -0.0, -15.0, 30.0, -10.0], [-5.0, -35.0, 10.0, -30.0, -20.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank1_offset1(self):
      arg0=numpy.array([[[-4.0, -6.0, 0.0, 0.0, -5.0], [2.0, 5.0, 7.0, -2.0, 7.0], [-1.0, 7.0, 1.0, -5.0, -4.0], [-2.0, 4.0, 5.0, -3.0, 5.0]], [[2.0, 0.0, -6.0, -7.0, -2.0], [-6.0, 5.0, 4.0, -2.0, 4.0], [-1.0, 6.0, -3.0, -2.0, 6.0], [-1.0, 2.0, 6.0, 1.0, 1.0]], [[7.0, -6.0, -3.0, 5.0, -6.0], [-5.0, 6.0, -7.0, -6.0, 4.0], [-1.0, 7.0, -7.0, -4.0, 4.0], [-7.0, -3.0, 0.0, 6.0, 0.0]]])
      arg1=numpy.array([-7.0, 0.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[28.0, 42.0, 0.0, 0.0, 35.0], [-14.0, -35.0, -49.0, 14.0, -49.0], [7.0, -49.0, -7.0, 35.0, 28.0], [14.0, -28.0, -35.0, 21.0, -35.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank2_offset2(self):
      arg0=numpy.array([[[[-7.0, 3.0, 3.0, -2.0, -5.0], [0.0, 7.0, -6.0, 0.0, 4.0], [-2.0, -3.0, -7.0, 0.0, 2.0], [-3.0, -2.0, 6.0, 4.0, -1.0]], [[-5.0, -5.0, -2.0, -1.0, 3.0], [1.0, 4.0, 0.0, -7.0, -4.0], [0.0, -5.0, 7.0, 0.0, -4.0], [-1.0, 4.0, 0.0, -2.0, -3.0]], [[-4.0, 5.0, -2.0, 6.0, 0.0], [-2.0, 1.0, 7.0, -5.0, -6.0], [4.0, 0.0, 3.0, 0.0, -5.0], [5.0, -7.0, -5.0, 5.0, 0.0]]], [[[-7.0, 6.0, -5.0, -1.0, -5.0], [2.0, -6.0, 7.0, -2.0, 3.0], [-3.0, 1.0, -2.0, -2.0, 6.0], [-6.0, 6.0, -5.0, 5.0, -7.0]], [[7.0, 5.0, -7.0, 0.0, -7.0], [-7.0, 0.0, 4.0, -7.0, 0.0], [-3.0, 1.0, -5.0, -3.0, -6.0], [6.0, 2.0, 1.0, 5.0, -5.0]], [[-1.0, 2.0, 7.0, -4.0, 0.0], [3.0, 5.0, -4.0, -2.0, -4.0], [-2.0, 0.0, 0.0, 0.0, 0.0], [0.0, -2.0, -1.0, 2.0, 5.0]]]])
      arg1=numpy.array([[-1.0, 6.0, -5.0], [-1.0, -5.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[-31.0, -89.0, 35.0, -33.0, 63.0], [49.0, 18.0, -56.0, 20.0, -1.0], [0.0, -33.0, 61.0, 17.0, 23.0], [-52.0, 45.0, 19.0, -71.0, 15.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank1_offset0(self):
      arg0=numpy.array([[3.0, -5.0, -2.0, 1.0, -6.0], [3.0, 0.0, 4.0, 6.0, -2.0], [1.0, -5.0, -4.0, 0.0, 5.0], [7.0, 0.0, -2.0, -2.0, 0.0]])
      arg1=numpy.array([-2.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-6.0, 0.0], [10.0, 0.0], [4.0, 0.0], [-2.0, 0.0], [12.0, 0.0]], [[-6.0, 0.0], [0.0, 0.0], [-8.0, 0.0], [-12.0, 0.0], [4.0, 0.0]], [[-2.0, 0.0], [10.0, 0.0], [8.0, 0.0], [0.0, 0.0], [-10.0, 0.0]], [[-14.0, 0.0], [0.0, 0.0], [4.0, 0.0], [4.0, 0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank2_offset1(self):
      arg0=numpy.array([[[-3.0, -5.0, 0.0, 5.0, 4.0], [2.0, 4.0, -3.0, -6.0, 0.0], [-1.0, -4.0, 3.0, 3.0, 4.0], [5.0, 4.0, -3.0, 5.0, -2.0]], [[6.0, -5.0, 5.0, -3.0, 2.0], [4.0, 6.0, 0.0, 4.0, -1.0], [-4.0, 6.0, 3.0, 3.0, -2.0], [1.0, -4.0, -2.0, 6.0, -6.0]], [[-5.0, 7.0, -7.0, 2.0, 0.0], [-7.0, -7.0, -3.0, -4.0, 4.0], [5.0, -7.0, -2.0, -5.0, 0.0], [-1.0, 7.0, 0.0, 5.0, -3.0]]])
      arg1=numpy.array([[3.0, 2.0], [2.0, 0.0], [1.0, -3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-2.0, 9.0], [-18.0, -31.0], [3.0, 21.0], [11.0, 4.0], [16.0, 8.0]], [[7.0, 25.0], [17.0, 29.0], [-12.0, 3.0], [-14.0, 0.0], [2.0, -12.0]], [[-6.0, -17.0], [-7.0, 13.0], [13.0, 12.0], [10.0, 21.0], [8.0, 8.0]], [[16.0, 13.0], [11.0, -13.0], [-13.0, -6.0], [32.0, -5.0], [-21.0, 5.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank3_offset2(self):
      arg0=numpy.array([[[[2.0, -4.0, 2.0, -6.0, 4.0], [-6.0, 4.0, 7.0, 0.0, 1.0], [-6.0, 3.0, 2.0, -2.0, 2.0], [5.0, 1.0, 1.0, 0.0, 7.0]], [[3.0, 0.0, -4.0, -3.0, -3.0], [0.0, 0.0, 5.0, 0.0, -5.0], [-6.0, 1.0, 7.0, -2.0, 2.0], [-3.0, -3.0, 0.0, 2.0, -3.0]], [[4.0, 0.0, 3.0, 2.0, -4.0], [-1.0, -3.0, 5.0, -5.0, -4.0], [6.0, 6.0, -7.0, -4.0, 7.0], [-4.0, 5.0, 3.0, -1.0, 5.0]]], [[[-1.0, 7.0, 2.0, -7.0, 0.0], [2.0, -4.0, -2.0, 1.0, -3.0], [-4.0, 6.0, 3.0, 6.0, 0.0], [-1.0, -7.0, -3.0, 2.0, 6.0]], [[5.0, 7.0, -3.0, 0.0, 4.0], [-1.0, 3.0, -1.0, 2.0, -7.0], [-5.0, 4.0, 4.0, -2.0, 6.0], [5.0, -4.0, 4.0, 0.0, -4.0]], [[-6.0, 3.0, 4.0, -2.0, 0.0], [-2.0, 4.0, -2.0, 7.0, 2.0], [-3.0, 0.0, 0.0, 0.0, -2.0], [3.0, -2.0, 0.0, 0.0, -7.0]]]])
      arg1=numpy.array([[[1.0, 6.0], [-3.0, -2.0], [3.0, 7.0]], [[-7.0, 0.0], [1.0, -1.0], [-5.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[47.0, 17.0], [-61.0, -25.0], [-14.0, 52.0], [68.0, -20.0], [5.0, -2.0]], [[-14.0, -46.0], [6.0, 8.0], [30.0, 64.0], [-55.0, -23.0], [8.0, -1.0]], [[68.0, 17.0], [-20.0, 54.0], [-57.0, -55.0], [-52.0, -34.0], [33.0, 47.0]], [[-1.0, 9.0], [80.0, 47.0], [35.0, 23.0], [-23.0, -11.0], [20.0, 73.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_array_rank2_offset0(self):
      arg0=numpy.array([[4.0, -6.0, -1.0, -2.0, -5.0], [-2.0, -3.0, -7.0, 7.0, -5.0], [0.0, -5.0, -1.0, 2.0, 5.0], [4.0, 0.0, -3.0, -1.0, 6.0]])
      arg1=numpy.array([[0.0, 0.0, -4.0, -5.0, -5.0], [-5.0, 1.0, 1.0, -4.0, 3.0], [-4.0, -7.0, -1.0, -3.0, 5.0], [6.0, -3.0, -2.0, 6.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[0.0, 0.0, -16.0, -20.0, -20.0], [-20.0, 4.0, 4.0, -16.0, 12.0], [-16.0, -28.0, -4.0, -12.0, 20.0], [24.0, -12.0, -8.0, 24.0, 4.0]], [[0.0, 0.0, 24.0, 30.0, 30.0], [30.0, -6.0, -6.0, 24.0, -18.0], [24.0, 42.0, 6.0, 18.0, -30.0], [-36.0, 18.0, 12.0, -36.0, -6.0]], [[0.0, 0.0, 4.0, 5.0, 5.0], [5.0, -1.0, -1.0, 4.0, -3.0], [4.0, 7.0, 1.0, 3.0, -5.0], [-6.0, 3.0, 2.0, -6.0, -1.0]], [[0.0, 0.0, 8.0, 10.0, 10.0], [10.0, -2.0, -2.0, 8.0, -6.0], [8.0, 14.0, 2.0, 6.0, -10.0], [-12.0, 6.0, 4.0, -12.0, -2.0]], [[0.0, 0.0, 20.0, 25.0, 25.0], [25.0, -5.0, -5.0, 20.0, -15.0], [20.0, 35.0, 5.0, 15.0, -25.0], [-30.0, 15.0, 10.0, -30.0, -5.0]]], [[[0.0, 0.0, 8.0, 10.0, 10.0], [10.0, -2.0, -2.0, 8.0, -6.0], [8.0, 14.0, 2.0, 6.0, -10.0], [-12.0, 6.0, 4.0, -12.0, -2.0]], [[0.0, 0.0, 12.0, 15.0, 15.0], [15.0, -3.0, -3.0, 12.0, -9.0], [12.0, 21.0, 3.0, 9.0, -15.0], [-18.0, 9.0, 6.0, -18.0, -3.0]], [[0.0, 0.0, 28.0, 35.0, 35.0], [35.0, -7.0, -7.0, 28.0, -21.0], [28.0, 49.0, 7.0, 21.0, -35.0], [-42.0, 21.0, 14.0, -42.0, -7.0]], [[0.0, 0.0, -28.0, -35.0, -35.0], [-35.0, 7.0, 7.0, -28.0, 21.0], [-28.0, -49.0, -7.0, -21.0, 35.0], [42.0, -21.0, -14.0, 42.0, 7.0]], [[0.0, 0.0, 20.0, 25.0, 25.0], [25.0, -5.0, -5.0, 20.0, -15.0], [20.0, 35.0, 5.0, 15.0, -25.0], [-30.0, 15.0, 10.0, -30.0, -5.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 20.0, 25.0, 25.0], [25.0, -5.0, -5.0, 20.0, -15.0], [20.0, 35.0, 5.0, 15.0, -25.0], [-30.0, 15.0, 10.0, -30.0, -5.0]], [[0.0, 0.0, 4.0, 5.0, 5.0], [5.0, -1.0, -1.0, 4.0, -3.0], [4.0, 7.0, 1.0, 3.0, -5.0], [-6.0, 3.0, 2.0, -6.0, -1.0]], [[0.0, 0.0, -8.0, -10.0, -10.0], [-10.0, 2.0, 2.0, -8.0, 6.0], [-8.0, -14.0, -2.0, -6.0, 10.0], [12.0, -6.0, -4.0, 12.0, 2.0]], [[0.0, 0.0, -20.0, -25.0, -25.0], [-25.0, 5.0, 5.0, -20.0, 15.0], [-20.0, -35.0, -5.0, -15.0, 25.0], [30.0, -15.0, -10.0, 30.0, 5.0]]], [[[0.0, 0.0, -16.0, -20.0, -20.0], [-20.0, 4.0, 4.0, -16.0, 12.0], [-16.0, -28.0, -4.0, -12.0, 20.0], [24.0, -12.0, -8.0, 24.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 12.0, 15.0, 15.0], [15.0, -3.0, -3.0, 12.0, -9.0], [12.0, 21.0, 3.0, 9.0, -15.0], [-18.0, 9.0, 6.0, -18.0, -3.0]], [[0.0, 0.0, 4.0, 5.0, 5.0], [5.0, -1.0, -1.0, 4.0, -3.0], [4.0, 7.0, 1.0, 3.0, -5.0], [-6.0, 3.0, 2.0, -6.0, -1.0]], [[0.0, 0.0, -24.0, -30.0, -30.0], [-30.0, 6.0, 6.0, -24.0, 18.0], [-24.0, -42.0, -6.0, -18.0, 30.0], [36.0, -18.0, -12.0, 36.0, 6.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank3_offset1(self):
      arg0=numpy.array([[[7.0, -2.0, -2.0, -4.0, 7.0], [-7.0, 0.0, -7.0, 5.0, 3.0], [-5.0, -6.0, -4.0, -3.0, 4.0], [-4.0, -6.0, -7.0, 0.0, 0.0]], [[2.0, -7.0, -2.0, 0.0, 5.0], [-3.0, 7.0, -2.0, 7.0, 1.0], [3.0, 7.0, -2.0, 5.0, -7.0], [0.0, 5.0, 0.0, -1.0, -5.0]], [[2.0, 1.0, -1.0, -5.0, -7.0], [4.0, 6.0, 4.0, 1.0, 2.0], [2.0, -6.0, -1.0, 4.0, -7.0], [6.0, -6.0, -2.0, -3.0, 3.0]]])
      arg1=numpy.array([[[2.0, 5.0, -7.0, -6.0, -2.0], [4.0, 1.0, -1.0, 4.0, 3.0], [7.0, -3.0, -5.0, -5.0, 5.0], [-3.0, 5.0, 2.0, -5.0, -7.0]], [[0.0, 5.0, -3.0, -7.0, 6.0], [0.0, 4.0, 6.0, -6.0, 0.0], [6.0, 4.0, 0.0, -2.0, -6.0], [5.0, 2.0, 0.0, -3.0, 4.0]], [[-3.0, -7.0, -6.0, 6.0, 4.0], [-1.0, -6.0, 7.0, -5.0, 1.0], [-1.0, 0.0, 1.0, 0.0, 0.0], [2.0, 5.0, 4.0, 7.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[8.0, 31.0, -67.0, -44.0, 6.0], [26.0, 3.0, 19.0, 6.0, 23.0], [59.0, -13.0, -33.0, -39.0, 23.0], [-7.0, 49.0, 22.0, -27.0, -35.0]], [[-7.0, -52.0, 29.0, 67.0, -34.0], [-9.0, -36.0, -33.0, 29.0, -5.0], [-57.0, -22.0, 11.0, 24.0, 32.0], [-27.0, -19.0, 0.0, 38.0, -11.0]], [[-1.0, -13.0, 26.0, 20.0, -12.0], [-7.0, -4.0, -17.0, 9.0, -7.0], [-25.0, -2.0, 9.0, 14.0, 2.0], [-6.0, -19.0, -8.0, 9.0, 3.0]], [[7.0, 15.0, 58.0, -6.0, -12.0], [-11.0, 26.0, -31.0, 9.0, -17.0], [-23.0, 12.0, 15.0, 20.0, -20.0], [2.0, -45.0, -28.0, -15.0, 13.0]], [[35.0, 109.0, -22.0, -119.0, -12.0], [35.0, 69.0, -26.0, 33.0, 14.0], [86.0, -1.0, -42.0, -45.0, 5.0], [-10.0, 10.0, -14.0, -99.0, -50.0]]], [[[-26.0, -78.0, 34.0, 87.0, 12.0], [-32.0, -43.0, 17.0, -30.0, -17.0], [-71.0, 9.0, 39.0, 41.0, -17.0], [14.0, -21.0, 2.0, 72.0, 49.0]], [[-18.0, -7.0, -57.0, -13.0, 66.0], [-6.0, -8.0, 84.0, -72.0, 6.0], [36.0, 28.0, 6.0, -14.0, -42.0], [47.0, 44.0, 24.0, 21.0, 46.0]], [[-26.0, -73.0, 31.0, 80.0, 18.0], [-32.0, -39.0, 23.0, -36.0, -17.0], [-65.0, 13.0, 39.0, 39.0, -23.0], [19.0, -19.0, 2.0, 69.0, 53.0]], [[7.0, 53.0, -62.0, -73.0, 36.0], [19.0, 27.0, 44.0, -27.0, 16.0], [76.0, 13.0, -24.0, -39.0, -17.0], [22.0, 44.0, 14.0, -39.0, -4.0]], [[0.0, 6.0, -36.0, -13.0, 8.0], [10.0, -5.0, 17.0, -4.0, 11.0], [25.0, -5.0, -13.0, -17.0, 9.0], [0.0, 27.0, 14.0, -4.0, -11.0]]], [[[-16.0, -24.0, 14.0, 21.0, 36.0], [-22.0, -5.0, 37.0, -48.0, -13.0], [-19.0, 27.0, 27.0, 19.0, -43.0], [34.0, -9.0, -2.0, 30.0, 53.0]], [[6.0, 47.0, 57.0, -49.0, 30.0], [-18.0, 58.0, 6.0, -36.0, -24.0], [6.0, 46.0, 24.0, 16.0, -72.0], [41.0, -46.0, -36.0, -33.0, 52.0]], [[-5.0, -23.0, 40.0, 32.0, -8.0], [-15.0, -6.0, -15.0, 1.0, -13.0], [-39.0, 4.0, 19.0, 24.0, -8.0], [0.0, -29.0, -12.0, 19.0, 17.0]], [[-18.0, -18.0, -18.0, 7.0, 52.0], [-16.0, -7.0, 61.0, -62.0, -5.0], [5.0, 29.0, 19.0, 5.0, -45.0], [42.0, 15.0, 10.0, 28.0, 53.0]], [[29.0, 34.0, 35.0, -17.0, -78.0], [23.0, 18.0, -95.0, 93.0, 5.0], [-7.0, -40.0, -27.0, -6.0, 62.0], [-61.0, -29.0, -20.0, -48.0, -77.0]]], [[[-26.0, -62.0, -8.0, 60.0, 32.0], [-22.0, -40.0, 46.0, -46.0, -6.0], [-34.0, 12.0, 26.0, 20.0, -20.0], [24.0, 10.0, 16.0, 62.0, 46.0]], [[6.0, 37.0, 63.0, -35.0, 18.0], [-18.0, 50.0, -6.0, -24.0, -24.0], [-6.0, 38.0, 24.0, 20.0, -60.0], [31.0, -50.0, -36.0, -27.0, 44.0]], [[-8.0, -21.0, 61.0, 30.0, 6.0], [-26.0, 5.0, -7.0, -18.0, -23.0], [-47.0, 21.0, 33.0, 35.0, -35.0], [17.0, -45.0, -22.0, 21.0, 43.0]], [[9.0, 16.0, 21.0, -11.0, -18.0], [3.0, 14.0, -27.0, 21.0, -3.0], [-3.0, -4.0, -3.0, 2.0, 6.0], [-11.0, -17.0, -12.0, -18.0, -13.0]], [[-9.0, -46.0, -3.0, 53.0, -18.0], [-3.0, -38.0, -9.0, 15.0, 3.0], [-33.0, -20.0, 3.0, 10.0, 30.0], [-19.0, 5.0, 12.0, 36.0, -11.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank4_offset2(self):
      arg0=numpy.array([[[[0.0, 7.0, 2.0, 4.0, 5.0], [-6.0, 6.0, 5.0, -3.0, -7.0], [7.0, 0.0, -4.0, -4.0, 7.0], [3.0, -6.0, 7.0, 0.0, -5.0]], [[5.0, 0.0, 3.0, -5.0, 1.0], [2.0, 3.0, 7.0, 0.0, 7.0], [-5.0, 5.0, 7.0, 5.0, 0.0], [6.0, -6.0, 4.0, -6.0, -3.0]], [[-3.0, 3.0, -3.0, 0.0, 5.0], [0.0, -5.0, -2.0, 4.0, -4.0], [-1.0, -4.0, 7.0, -7.0, -6.0], [-1.0, 6.0, 7.0, 2.0, 0.0]]], [[[4.0, 5.0, 5.0, -5.0, 2.0], [-3.0, -4.0, 0.0, -7.0, 0.0], [-7.0, 6.0, -4.0, -6.0, -7.0], [6.0, 3.0, -5.0, 7.0, 4.0]], [[-7.0, 7.0, -3.0, -5.0, -5.0], [4.0, 5.0, 6.0, 4.0, -2.0], [5.0, -6.0, -3.0, -7.0, -4.0], [-4.0, -2.0, -3.0, 1.0, 3.0]], [[-6.0, 1.0, 4.0, 0.0, -3.0], [-1.0, 2.0, -6.0, 2.0, -5.0], [-5.0, -7.0, 5.0, 4.0, 7.0], [3.0, 2.0, -2.0, -7.0, 0.0]]]])
      arg1=numpy.array([[[[7.0, 1.0, -2.0, -6.0, -5.0], [0.0, 0.0, -6.0, -5.0, -5.0], [5.0, -4.0, 0.0, 7.0, -6.0], [-2.0, 7.0, -1.0, 0.0, -1.0]], [[-4.0, -1.0, -5.0, 0.0, 0.0], [-2.0, 0.0, 1.0, 3.0, 2.0], [-3.0, -7.0, -1.0, 7.0, -1.0], [-7.0, 3.0, 5.0, -5.0, 7.0]], [[4.0, 5.0, -1.0, -5.0, 0.0], [-4.0, 5.0, 0.0, 3.0, -1.0], [5.0, 0.0, 0.0, 3.0, 2.0], [-4.0, 3.0, 0.0, 2.0, 3.0]]], [[[-1.0, 3.0, -2.0, -4.0, 5.0], [-6.0, -3.0, 3.0, 4.0, -6.0], [2.0, 3.0, -6.0, -1.0, -6.0], [5.0, 5.0, -4.0, -5.0, -6.0]], [[-2.0, -6.0, -7.0, -2.0, 5.0], [4.0, -4.0, -4.0, -5.0, -1.0], [-5.0, 0.0, -7.0, 2.0, 6.0], [0.0, 0.0, 0.0, -4.0, -2.0]], [[2.0, -7.0, 0.0, -3.0, 4.0], [-7.0, -6.0, -6.0, -6.0, 4.0], [6.0, -5.0, 0.0, 2.0, 7.0], [-4.0, 4.0, 0.0, -5.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[[-34.0, 76.0, 19.0, 31.0, -39.0], [-8.0, 37.0, 81.0, 93.0, -28.0], [-23.0, 7.0, 20.0, -4.0, -119.0], [21.0, 2.0, 9.0, 7.0, 58.0]], [[44.0, -12.0, -76.0, -94.0, 29.0], [-21.0, -34.0, -61.0, -47.0, -71.0], [31.0, -18.0, -79.0, 69.0, -17.0], [-5.0, 87.0, -27.0, -52.0, -49.0]], [[-1.0, -11.0, -5.0, -23.0, 16.0], [-64.0, -42.0, -6.0, 1.0, -12.0], [35.0, -34.0, -12.0, 23.0, -41.0], [-4.0, 55.0, -7.0, -54.0, -42.0]], [[63.0, 24.0, 62.0, 6.0, -70.0], [20.0, 35.0, -24.0, -30.0, 5.0], [50.0, 4.0, 70.0, -12.0, -19.0], [2.0, -12.0, -9.0, 70.0, 1.0]], [[53.0, 86.0, 11.0, -44.0, -52.0], [-33.0, 57.0, 15.0, 44.0, -47.0], [58.0, -6.0, 22.0, 39.0, -84.0], [-15.0, 51.0, -8.0, 30.0, 36.0]]], [[[-57.0, -34.0, -20.0, 43.0, 31.0], [37.0, -1.0, 19.0, 10.0, 44.0], [-68.0, 6.0, -12.0, -19.0, 69.0], [-13.0, -55.0, 28.0, -6.0, 37.0]], [[8.0, -78.0, -49.0, -11.0, -17.0], [44.0, -45.0, -77.0, -89.0, 8.0], [-25.0, -67.0, -14.0, 66.0, 19.0], [-41.0, 24.0, 25.0, -35.0, 0.0]], [[-25.0, -6.0, -85.0, -14.0, -19.0], [60.0, 2.0, -11.0, -4.0, -39.0], [-72.0, -39.0, -49.0, 78.0, -47.0], [-27.0, 26.0, 30.0, -33.0, 68.0]], [[-2.0, -42.0, -12.0, 12.0, 8.0], [28.0, 13.0, -31.0, -33.0, 57.0], [-17.0, -19.0, 14.0, 10.0, 106.0], [-53.0, -36.0, 31.0, 17.0, 35.0]], [[-99.0, 13.0, -3.0, 81.0, 5.0], [29.0, 18.0, 87.0, 84.0, 35.0], [-96.0, 4.0, 7.0, -26.0, -20.0], [1.0, -60.0, 42.0, -10.0, 83.0]]], [[[52.0, -9.0, -9.0, -4.0, -65.0], [111.0, 26.0, -58.0, -76.0, -27.0], [-24.0, 11.0, 12.0, 18.0, -2.0], [10.0, -24.0, -4.0, 63.0, 22.0]], [[-44.0, 78.0, 9.0, 29.0, -28.0], [-5.0, 28.0, 89.0, 99.0, -44.0], [-35.0, 18.0, 1.0, -9.0, -134.0], [39.0, 5.0, 1.0, -4.0, 48.0]], [[-8.0, -5.0, -5.0, -4.0, 5.0], [-65.0, 29.0, 1.0, 31.0, 74.0], [31.0, -70.0, 38.0, 50.0, 72.0], [-109.0, 14.0, 55.0, -14.0, 69.0]], [[-48.0, -48.0, 51.0, 85.0, -29.0], [-2.0, -13.0, 15.0, 1.0, 96.0], [-23.0, -57.0, 80.0, -14.0, 27.0], [-45.0, -48.0, 53.0, -1.0, 40.0]], [[54.0, -69.0, 34.0, 3.0, -62.0], [1.0, -35.0, -89.0, -103.0, 45.0], [53.0, -84.0, 70.0, 44.0, 13.0], [-53.0, 24.0, 21.0, 4.0, -24.0]]], [[[1.0, 13.0, -19.0, -38.0, 7.0], [-81.0, -25.0, 4.0, 26.0, -22.0], [42.0, -51.0, -14.0, 52.0, -65.0], [-26.0, 78.0, 3.0, -61.0, -13.0]], [[11.0, 37.0, 44.0, -8.0, 43.0], [-52.0, 17.0, 35.0, 40.0, 4.0], [46.0, 65.0, 2.0, -69.0, 38.0], [37.0, -19.0, -36.0, 25.0, -46.0]], [[68.0, 55.0, -10.0, -45.0, -83.0], [-4.0, 74.0, -29.0, 5.0, -9.0], [51.0, -61.0, 47.0, 93.0, -34.0], [-87.0, 49.0, 33.0, 41.0, 92.0]], [[9.0, 80.0, 7.0, -19.0, 12.0], [15.0, 27.0, 53.0, 53.0, -85.0], [-5.0, 98.0, -43.0, -55.0, -75.0], [97.0, -5.0, -58.0, 30.0, -31.0]], [[-33.0, -8.0, -4.0, 8.0, 60.0], [-6.0, -24.0, 27.0, 17.0, -8.0], [-23.0, 53.0, -42.0, -54.0, 27.0], [51.0, -24.0, -26.0, -17.0, -46.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank0_offset0(self):
      arg0=numpy.array([[-4.0, 3.0, 0.0, -1.0, 5.0], [4.0, -3.0, 3.0, -4.0, 2.0], [0.0, -1.0, -6.0, 2.0, 1.0], [0.0, 5.0, 3.0, 6.0, 0.0]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-5.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[20.0, -15.0, -0.0, 5.0, -25.0], [-20.0, 15.0, -15.0, 20.0, -10.0], [-0.0, 5.0, 30.0, -10.0, -5.0], [-0.0, -25.0, -15.0, -30.0, -0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[4.0, 3.0, -6.0, 5.0, -5.0], [-7.0, -3.0, 2.0, 1.0, 7.0], [0.0, 0.0, 1.0, 5.0, 6.0], [-2.0, -4.0, 6.0, 5.0, -3.0]], [[-2.0, 6.0, 2.0, 0.0, -5.0], [1.0, -5.0, -1.0, 2.0, 6.0], [7.0, 0.0, -1.0, 1.0, 0.0], [-6.0, 7.0, 0.0, -6.0, -2.0]], [[1.0, 3.0, -5.0, -5.0, 6.0], [7.0, -4.0, 1.0, 2.0, 7.0], [5.0, -2.0, -5.0, 0.0, -1.0], [-3.0, 3.0, 1.0, -3.0, -5.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([0.0, -5.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[16.0, -12.0, -40.0, -30.0, 61.0], [37.0, 1.0, 11.0, 2.0, 12.0], [-5.0, -12.0, -25.0, -5.0, -6.0], [12.0, -17.0, 6.0, 12.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[[-6.0, 4.0, 0.0, -1.0, -6.0], [-1.0, -3.0, -1.0, -7.0, -3.0], [5.0, -2.0, 5.0, 7.0, -7.0], [-5.0, 0.0, -1.0, 2.0, -5.0]], [[6.0, 7.0, 1.0, 2.0, -5.0], [4.0, -3.0, 2.0, 4.0, -6.0], [6.0, -4.0, 2.0, 4.0, 7.0], [5.0, 0.0, 1.0, -7.0, 6.0]], [[7.0, -7.0, 7.0, 4.0, -4.0], [-1.0, -6.0, 0.0, 0.0, -6.0], [4.0, 7.0, -1.0, 6.0, -2.0], [-7.0, 0.0, 4.0, -4.0, -4.0]]], [[[1.0, 7.0, 5.0, -1.0, 3.0], [2.0, 0.0, -3.0, 7.0, 6.0], [-5.0, -2.0, 7.0, 7.0, -4.0], [1.0, -7.0, -7.0, -4.0, -6.0]], [[-3.0, -7.0, 2.0, -6.0, -6.0], [2.0, 5.0, 2.0, 6.0, -3.0], [5.0, 0.0, 0.0, 2.0, -1.0], [6.0, 3.0, -1.0, -6.0, 6.0]], [[7.0, 6.0, -4.0, -2.0, 5.0], [-3.0, -5.0, 7.0, 3.0, 0.0], [-2.0, -4.0, -3.0, 3.0, -5.0], [0.0, 6.0, 5.0, 6.0, -7.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[3.0, -3.0, 4.0], [-3.0, 6.0, 3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-8.0, -82.0, 10.0, -32.0, -49.0], [-22.0, -9.0, 33.0, -9.0, -51.0], [52.0, 28.0, -25.0, 33.0, -59.0], [-25.0, 57.0, 40.0, 5.0, -16.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank1_offset0(self):
      arg0=numpy.array([[-3.0, -5.0, 6.0, -6.0, -5.0], [-6.0, 2.0, -2.0, -7.0, 0.0], [-4.0, -3.0, -7.0, 0.0, -2.0], [5.0, -4.0, 6.0, -3.0, -5.0]])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-4.0, -3.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[12.0, 9.0], [20.0, 15.0], [-24.0, -18.0], [24.0, 18.0], [20.0, 15.0]], [[24.0, 18.0], [-8.0, -6.0], [8.0, 6.0], [28.0, 21.0], [0.0, 0.0]], [[16.0, 12.0], [12.0, 9.0], [28.0, 21.0], [0.0, 0.0], [8.0, 6.0]], [[-20.0, -15.0], [16.0, 12.0], [-24.0, -18.0], [12.0, 9.0], [20.0, 15.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[-5.0, -4.0, 6.0, -6.0, -7.0], [2.0, 2.0, -6.0, -2.0, 6.0], [-7.0, 1.0, -5.0, -4.0, 3.0], [-7.0, 2.0, 2.0, -4.0, -7.0]], [[2.0, -1.0, 2.0, 0.0, 0.0], [2.0, 7.0, -7.0, 1.0, 4.0], [-6.0, 0.0, 0.0, -4.0, 0.0], [0.0, -4.0, 5.0, 7.0, -2.0]], [[0.0, -2.0, 7.0, 6.0, 1.0], [-4.0, 6.0, 5.0, 2.0, -4.0], [0.0, 1.0, -2.0, 3.0, 2.0], [7.0, 4.0, -5.0, 4.0, 1.0]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[6.0, 3.0], [-7.0, -6.0], [-6.0, -1.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-44.0, -27.0], [-5.0, -4.0], [-20.0, -1.0], [-72.0, -24.0], [-48.0, -22.0]], [[22.0, -2.0], [-73.0, -42.0], [-17.0, 19.0], [-31.0, -14.0], [32.0, -2.0]], [[0.0, 15.0], [0.0, 2.0], [-18.0, -13.0], [-14.0, 9.0], [6.0, 7.0]], [[-84.0, -28.0], [16.0, 26.0], [7.0, -19.0], [-97.0, -58.0], [-34.0, -10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[[-5.0, 1.0, -2.0, 0.0, 6.0], [-6.0, 0.0, -6.0, 3.0, 6.0], [0.0, -2.0, 2.0, 0.0, -2.0], [0.0, 6.0, 5.0, 5.0, -5.0]], [[7.0, -5.0, -2.0, 4.0, -1.0], [4.0, 1.0, -4.0, 5.0, 4.0], [6.0, -7.0, -7.0, -5.0, -5.0], [-5.0, 0.0, -1.0, 5.0, -3.0]], [[4.0, 1.0, -4.0, 0.0, -1.0], [-6.0, -2.0, -2.0, 7.0, 7.0], [-6.0, -1.0, 0.0, 6.0, 0.0], [-3.0, -4.0, 0.0, 7.0, 3.0]]], [[[-5.0, 0.0, 3.0, -5.0, -2.0], [4.0, -5.0, -5.0, 4.0, 0.0], [-2.0, -3.0, 2.0, 0.0, -3.0], [-3.0, 0.0, 6.0, -1.0, 3.0]], [[-7.0, -3.0, -3.0, -6.0, 6.0], [-4.0, -6.0, 1.0, -2.0, 2.0], [5.0, -4.0, 4.0, -4.0, 1.0], [2.0, -1.0, -7.0, -7.0, 0.0]], [[0.0, 3.0, -2.0, 6.0, -3.0], [-6.0, -7.0, 3.0, -3.0, -5.0], [-3.0, 2.0, 7.0, 6.0, 1.0], [0.0, 4.0, 4.0, 1.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[5.0, 0.0], [-2.0, 4.0], [-3.0, -7.0]], [[-5.0, -5.0], [-6.0, -3.0], [6.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[16.0, 46.0], [48.0, -36.0], [-3.0, 26.0], [89.0, 23.0], [-9.0, 13.0]], [[-52.0, 86.0], [23.0, 103.0], [21.0, 2.0], [-42.0, -25.0], [-41.0, -9.0]], [[-32.0, 79.0], [58.0, -6.0], [32.0, -92.0], [52.0, -86.0], [15.0, -14.0]], [[22.0, 10.0], [72.0, 7.0], [63.0, -37.0], [47.0, -9.0], [-43.0, -48.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_Symbol_rank2_offset0(self):
      arg0=numpy.array([[0.0, -7.0, -6.0, 1.0, 0.0], [-3.0, -5.0, 5.0, -4.0, 0.0], [-3.0, -6.0, 7.0, 5.0, 2.0], [-4.0, 5.0, -3.0, -2.0, 7.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[5.0, 4.0, -1.0, -4.0, 3.0], [5.0, -3.0, -4.0, -6.0, 5.0], [7.0, 0.0, -7.0, 0.0, -4.0], [6.0, 2.0, -1.0, 6.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-35.0, -28.0, 7.0, 28.0, -21.0], [-35.0, 21.0, 28.0, 42.0, -35.0], [-49.0, 0.0, 49.0, 0.0, 28.0], [-42.0, -14.0, 7.0, -42.0, 21.0]], [[-30.0, -24.0, 6.0, 24.0, -18.0], [-30.0, 18.0, 24.0, 36.0, -30.0], [-42.0, 0.0, 42.0, 0.0, 24.0], [-36.0, -12.0, 6.0, -36.0, 18.0]], [[5.0, 4.0, -1.0, -4.0, 3.0], [5.0, -3.0, -4.0, -6.0, 5.0], [7.0, 0.0, -7.0, 0.0, -4.0], [6.0, 2.0, -1.0, 6.0, -3.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-15.0, -12.0, 3.0, 12.0, -9.0], [-15.0, 9.0, 12.0, 18.0, -15.0], [-21.0, 0.0, 21.0, 0.0, 12.0], [-18.0, -6.0, 3.0, -18.0, 9.0]], [[-25.0, -20.0, 5.0, 20.0, -15.0], [-25.0, 15.0, 20.0, 30.0, -25.0], [-35.0, 0.0, 35.0, 0.0, 20.0], [-30.0, -10.0, 5.0, -30.0, 15.0]], [[25.0, 20.0, -5.0, -20.0, 15.0], [25.0, -15.0, -20.0, -30.0, 25.0], [35.0, 0.0, -35.0, 0.0, -20.0], [30.0, 10.0, -5.0, 30.0, -15.0]], [[-20.0, -16.0, 4.0, 16.0, -12.0], [-20.0, 12.0, 16.0, 24.0, -20.0], [-28.0, 0.0, 28.0, 0.0, 16.0], [-24.0, -8.0, 4.0, -24.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-15.0, -12.0, 3.0, 12.0, -9.0], [-15.0, 9.0, 12.0, 18.0, -15.0], [-21.0, 0.0, 21.0, 0.0, 12.0], [-18.0, -6.0, 3.0, -18.0, 9.0]], [[-30.0, -24.0, 6.0, 24.0, -18.0], [-30.0, 18.0, 24.0, 36.0, -30.0], [-42.0, 0.0, 42.0, 0.0, 24.0], [-36.0, -12.0, 6.0, -36.0, 18.0]], [[35.0, 28.0, -7.0, -28.0, 21.0], [35.0, -21.0, -28.0, -42.0, 35.0], [49.0, 0.0, -49.0, 0.0, -28.0], [42.0, 14.0, -7.0, 42.0, -21.0]], [[25.0, 20.0, -5.0, -20.0, 15.0], [25.0, -15.0, -20.0, -30.0, 25.0], [35.0, 0.0, -35.0, 0.0, -20.0], [30.0, 10.0, -5.0, 30.0, -15.0]], [[10.0, 8.0, -2.0, -8.0, 6.0], [10.0, -6.0, -8.0, -12.0, 10.0], [14.0, 0.0, -14.0, 0.0, -8.0], [12.0, 4.0, -2.0, 12.0, -6.0]]], [[[-20.0, -16.0, 4.0, 16.0, -12.0], [-20.0, 12.0, 16.0, 24.0, -20.0], [-28.0, 0.0, 28.0, 0.0, 16.0], [-24.0, -8.0, 4.0, -24.0, 12.0]], [[25.0, 20.0, -5.0, -20.0, 15.0], [25.0, -15.0, -20.0, -30.0, 25.0], [35.0, 0.0, -35.0, 0.0, -20.0], [30.0, 10.0, -5.0, 30.0, -15.0]], [[-15.0, -12.0, 3.0, 12.0, -9.0], [-15.0, 9.0, 12.0, 18.0, -15.0], [-21.0, 0.0, 21.0, 0.0, 12.0], [-18.0, -6.0, 3.0, -18.0, 9.0]], [[-10.0, -8.0, 2.0, 8.0, -6.0], [-10.0, 6.0, 8.0, 12.0, -10.0], [-14.0, 0.0, 14.0, 0.0, 8.0], [-12.0, -4.0, 2.0, -12.0, 6.0]], [[35.0, 28.0, -7.0, -28.0, 21.0], [35.0, -21.0, -28.0, -42.0, 35.0], [49.0, 0.0, -49.0, 0.0, -28.0], [42.0, 14.0, -7.0, 42.0, -21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank3_offset1(self):
      arg0=numpy.array([[[2.0, 1.0, 7.0, 6.0, -7.0], [0.0, -2.0, 5.0, -7.0, 5.0], [2.0, -2.0, 4.0, 1.0, -7.0], [5.0, -1.0, 1.0, 2.0, 5.0]], [[-3.0, 4.0, -7.0, 0.0, 7.0], [3.0, -3.0, 4.0, 0.0, -6.0], [-3.0, -5.0, 4.0, -6.0, -4.0], [3.0, -5.0, -4.0, -7.0, -5.0]], [[-6.0, -2.0, 3.0, -1.0, 7.0], [4.0, 6.0, -5.0, 6.0, -5.0], [-5.0, -3.0, 0.0, 4.0, -5.0], [-5.0, -2.0, 0.0, 4.0, -3.0]]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-6.0, -3.0, 2.0, 0.0, 2.0], [0.0, 2.0, 3.0, -5.0, -6.0], [-3.0, -1.0, 1.0, -4.0, 6.0], [-5.0, 1.0, 0.0, 6.0, 1.0]], [[-4.0, 3.0, 0.0, 0.0, 6.0], [-2.0, 6.0, 3.0, 0.0, 5.0], [4.0, 0.0, 1.0, -3.0, -6.0], [7.0, -6.0, 0.0, 2.0, -5.0]], [[1.0, 4.0, -3.0, 7.0, 7.0], [2.0, 3.0, 0.0, -5.0, -5.0], [7.0, -3.0, -7.0, 6.0, -7.0], [3.0, 0.0, -5.0, 0.0, 4.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-6.0, -39.0, 22.0, -42.0, -56.0], [-6.0, -32.0, -3.0, 20.0, 3.0], [-60.0, 16.0, 41.0, -35.0, 72.0], [-49.0, 20.0, 30.0, 6.0, -7.0]], [[-24.0, 1.0, 8.0, -14.0, 12.0], [-12.0, 20.0, 15.0, 5.0, 24.0], [-1.0, 5.0, 19.0, -28.0, -4.0], [17.0, -23.0, 10.0, 14.0, -27.0]], [[-11.0, -30.0, 5.0, 21.0, -7.0], [20.0, -19.0, 0.0, -50.0, -92.0], [-28.0, -16.0, -21.0, 11.0, 63.0], [-75.0, 49.0, -15.0, 28.0, 54.0]], [[-37.0, -22.0, 15.0, -7.0, 5.0], [-2.0, 9.0, 18.0, -25.0, -31.0], [-25.0, -3.0, 13.0, -30.0, 43.0], [-33.0, 6.0, 5.0, 36.0, 2.0]], [[21.0, 70.0, -35.0, 49.0, 77.0], [0.0, 49.0, 0.0, 0.0, 42.0], [98.0, -14.0, -49.0, 49.0, -133.0], [105.0, -49.0, -35.0, -28.0, -14.0]]], [[[-8.0, 25.0, -12.0, 28.0, 46.0], [2.0, 30.0, 9.0, -20.0, -5.0], [40.0, -12.0, -25.0, 15.0, -46.0], [33.0, -18.0, -20.0, 6.0, 1.0]], [[30.0, 21.0, -22.0, 42.0, 20.0], [18.0, -4.0, -15.0, -20.0, -33.0], [36.0, -16.0, -47.0, 53.0, -36.0], [7.0, 16.0, -30.0, -18.0, 37.0]], [[-51.0, -23.0, 25.0, -35.0, -1.0], [-18.0, 19.0, 27.0, 0.0, 15.0], [-34.0, 10.0, 44.0, -62.0, 41.0], [-12.0, -19.0, 25.0, 38.0, -35.0]], [[48.0, 45.0, -32.0, 42.0, 28.0], [12.0, 4.0, -21.0, 5.0, 12.0], [63.0, -11.0, -49.0, 64.0, -84.0], [53.0, -7.0, -30.0, -42.0, 17.0]], [[-11.0, -53.0, 25.0, -35.0, -61.0], [2.0, -41.0, -3.0, 0.0, -35.0], [-74.0, 10.0, 34.0, -32.0, 101.0], [-82.0, 41.0, 25.0, 18.0, 15.0]]], [[[-5.0, -35.0, 19.0, -35.0, -49.0], [-4.0, -29.0, -3.0, 15.0, -2.0], [-53.0, 13.0, 34.0, -29.0, 65.0], [-46.0, 20.0, 25.0, 6.0, -3.0]], [[29.0, -21.0, 5.0, -21.0, -55.0], [4.0, -43.0, -21.0, 25.0, 2.0], [-35.0, 11.0, 14.0, 5.0, 39.0], [-34.0, 28.0, 15.0, -22.0, 11.0]], [[-40.0, 0.0, 8.0, 0.0, 32.0], [-8.0, 32.0, 24.0, -20.0, -4.0], [4.0, -4.0, 8.0, -28.0, 0.0], [8.0, -20.0, 0.0, 32.0, -16.0]], [[22.0, -5.0, -10.0, 28.0, -6.0], [20.0, -22.0, -15.0, -25.0, -56.0], [1.0, -13.0, -33.0, 38.0, 14.0], [-35.0, 37.0, -20.0, -6.0, 47.0]], [[53.0, -11.0, 1.0, -35.0, -73.0], [-2.0, -53.0, -33.0, 60.0, 47.0], [-30.0, 22.0, 24.0, 10.0, 17.0], [-8.0, 17.0, 25.0, -50.0, -7.0]]], [[[-47.0, -26.0, 25.0, -35.0, -7.0], [-16.0, 13.0, 24.0, 0.0, 10.0], [-38.0, 10.0, 43.0, -59.0, 47.0], [-19.0, -13.0, 25.0, 36.0, -30.0]], [[24.0, -20.0, 4.0, -14.0, -46.0], [6.0, -38.0, -18.0, 15.0, -9.0], [-31.0, 7.0, 8.0, 7.0, 38.0], [-36.0, 29.0, 10.0, -16.0, 16.0]], [[10.0, -15.0, 2.0, 0.0, -22.0], [8.0, -22.0, -9.0, -5.0, -26.0], [-19.0, -1.0, -3.0, 8.0, 30.0], [-33.0, 25.0, 0.0, -2.0, 21.0]], [[20.0, -11.0, -8.0, 28.0, -10.0], [22.0, -26.0, -15.0, -30.0, -67.0], [-6.0, -14.0, -33.0, 37.0, 26.0], [-47.0, 44.0, -20.0, -2.0, 53.0]], [[-13.0, -42.0, 19.0, -21.0, -41.0], [4.0, -29.0, 0.0, -10.0, -40.0], [-56.0, 4.0, 21.0, -23.0, 81.0], [-69.0, 35.0, 15.0, 20.0, 18.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[[-1.0, -4.0, -5.0, 4.0, 6.0], [-2.0, 2.0, -7.0, -2.0, -2.0], [1.0, 2.0, -3.0, 4.0, 3.0], [2.0, 0.0, -4.0, 3.0, -1.0]], [[0.0, -1.0, 0.0, -6.0, 0.0], [1.0, -5.0, -6.0, -1.0, 0.0], [6.0, -3.0, -4.0, -7.0, -2.0], [5.0, 0.0, 3.0, 3.0, -7.0]], [[2.0, -7.0, -7.0, 2.0, -6.0], [-2.0, -6.0, 2.0, 3.0, 4.0], [-3.0, -5.0, -6.0, 5.0, -1.0], [-6.0, 5.0, 0.0, 4.0, -5.0]]], [[[4.0, 3.0, 1.0, -3.0, 2.0], [6.0, -5.0, -2.0, 0.0, 6.0], [-6.0, 5.0, 0.0, -7.0, 3.0], [0.0, 5.0, 5.0, 1.0, -7.0]], [[-6.0, -3.0, -6.0, 5.0, 3.0], [4.0, -7.0, 4.0, -4.0, -6.0], [-7.0, 6.0, -3.0, 4.0, -6.0], [-7.0, -3.0, 7.0, -4.0, 2.0]], [[0.0, 0.0, 5.0, 7.0, -3.0], [7.0, -1.0, -5.0, 0.0, -4.0], [7.0, -7.0, 0.0, 3.0, 3.0], [1.0, 1.0, -7.0, 1.0, -2.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-3.0, 0.0, 5.0, 0.0, -4.0], [-5.0, 1.0, 7.0, 5.0, 1.0], [6.0, -1.0, -4.0, 7.0, 5.0], [3.0, -5.0, -3.0, -2.0, 7.0]], [[-2.0, 2.0, -7.0, -4.0, -3.0], [-3.0, -3.0, 7.0, -4.0, 7.0], [-4.0, 3.0, 1.0, -4.0, -1.0], [0.0, 7.0, -4.0, 5.0, 4.0]], [[6.0, -3.0, -3.0, -2.0, 4.0], [2.0, 2.0, -4.0, 1.0, 0.0], [6.0, 2.0, 1.0, -2.0, 5.0], [2.0, -4.0, 5.0, 3.0, -5.0]]], [[[-3.0, -3.0, -5.0, -7.0, -2.0], [-1.0, -6.0, 2.0, -2.0, 3.0], [0.0, 4.0, 2.0, 2.0, -1.0], [0.0, 5.0, -1.0, -6.0, 2.0]], [[-1.0, 0.0, -2.0, 6.0, -4.0], [-6.0, -5.0, 2.0, 4.0, -3.0], [-1.0, -6.0, 4.0, 4.0, 4.0], [-5.0, 4.0, -6.0, -5.0, -3.0]], [[0.0, -2.0, -2.0, -1.0, -1.0], [-6.0, 3.0, 3.0, 6.0, 0.0], [-3.0, -4.0, -2.0, -5.0, -1.0], [6.0, 6.0, -2.0, -6.0, -1.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[9.0, -18.0, -19.0, -68.0, 28.0], [41.0, 9.0, -19.0, -35.0, 29.0], [12.0, 57.0, -10.0, -27.0, -23.0], [31.0, -7.0, 45.0, 14.0, 9.0]], [[-34.0, 10.0, -1.0, -21.0, -3.0], [24.0, -18.0, -7.0, -41.0, 7.0], [-59.0, 17.0, 2.0, -16.0, -69.0], [-11.0, 44.0, -4.0, -21.0, 18.0]], [[-24.0, 8.0, -7.0, -34.0, 9.0], [16.0, 20.0, -2.0, -28.0, 16.0], [-81.0, 11.0, -19.0, -68.0, -90.0], [31.0, 64.0, 5.0, -17.0, 15.0]], [[16.0, -23.0, 47.0, 64.0, -11.0], [-67.0, 40.0, 3.0, 114.0, -62.0], [34.0, -88.0, -20.0, 27.0, 52.0], [33.0, -23.0, -19.0, -81.0, -34.0]], [[-63.0, 18.0, 38.0, 19.0, -61.0], [-44.0, -42.0, 67.0, 14.0, 3.0], [6.0, -16.0, -8.0, 85.0, 13.0], [-27.0, -2.0, -62.0, -39.0, 70.0]]], [[[-30.0, -24.0, -63.0, -25.0, -38.0], [-69.0, -44.0, 42.0, 30.0, 11.0], [-53.0, -27.0, 21.0, -21.0, -18.0], [12.0, 113.0, -52.0, -95.0, -7.0]], [[-10.0, 25.0, 104.0, 26.0, 22.0], [46.0, 67.0, -24.0, 0.0, -27.0], [6.0, -3.0, -55.0, 13.0, -37.0], [23.0, -80.0, 33.0, 24.0, 36.0]], [[47.0, -2.0, 13.0, 63.0, 47.0], [65.0, -8.0, -110.0, -19.0, -67.0], [5.0, -19.0, 46.0, 8.0, 4.0], [-67.0, -39.0, 43.0, 12.0, -94.0]], [[30.0, -11.0, -4.0, -26.0, 39.0], [43.0, 27.0, -41.0, -19.0, 3.0], [14.0, 29.0, -6.0, -32.0, -10.0], [20.0, -25.0, 49.0, 28.0, -21.0]], [[18.0, -22.0, -32.0, -82.0, 40.0], [72.0, -12.0, -42.0, -66.0, 34.0], [30.0, 86.0, 8.0, -14.0, -16.0], [8.0, -24.0, 64.0, 34.0, 0.0]]], [[[-8.0, 25.0, 2.0, -25.0, -1.0], [-23.0, 69.0, 56.0, 4.0, 46.0], [-50.0, 1.0, -55.0, -86.0, -45.0], [74.0, 33.0, -8.0, 48.0, 48.0]], [[-51.0, 8.0, 23.0, 30.0, -46.0], [-10.0, -80.0, 14.0, -11.0, -22.0], [9.0, -9.0, 32.0, 105.0, 14.0], [-76.0, -4.0, -46.0, -52.0, 26.0]], [[-16.0, 10.0, 37.0, 10.0, 12.0], [33.0, 12.0, -31.0, -17.0, -22.0], [-35.0, -3.0, -10.0, -5.0, -53.0], [-6.0, -1.0, 13.0, -17.0, 2.0]], [[49.0, -14.0, 75.0, 88.0, 20.0], [-24.0, 66.0, -38.0, 101.0, -78.0], [69.0, -79.0, -22.0, 33.0, 72.0], [20.0, -90.0, 18.0, -24.0, -54.0]], [[-14.0, -16.0, 23.0, -50.0, 5.0], [4.0, 28.0, 14.0, 10.0, 16.0], [17.0, 25.0, -39.0, -2.0, -18.0], [55.0, -16.0, 21.0, -25.0, 39.0]]], [[[-45.0, 26.0, 5.0, -51.0, -20.0], [-1.0, 13.0, 62.0, -38.0, 58.0], [-40.0, 39.0, -39.0, -27.0, -54.0], [35.0, 27.0, -16.0, 32.0, 84.0]], [[18.0, -32.0, -36.0, -64.0, 21.0], [17.0, -2.0, -13.0, -11.0, 24.0], [30.0, 44.0, 1.0, -17.0, 7.0], [31.0, -1.0, 36.0, -6.0, -7.0]], [[-16.0, 5.0, -66.0, 2.0, -24.0], [6.0, -99.0, -4.0, -56.0, 11.0], [-22.0, 19.0, 71.0, 33.0, 7.0], [-89.0, 52.0, -33.0, 0.0, -20.0]], [[10.0, -11.0, -17.0, -52.0, 8.0], [1.0, 19.0, 23.0, -5.0, 39.0], [31.0, 38.0, -21.0, -18.0, 14.0], [43.0, -15.0, 20.0, 29.0, 26.0]], [[6.0, 26.0, 94.0, 101.0, 13.0], [23.0, 36.0, -52.0, 28.0, -77.0], [-4.0, -62.0, -10.0, 35.0, -6.0], [-35.0, -63.0, 5.0, -4.0, -28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank0_offset0(self):
      arg0=numpy.array([[7.0, 6.0, 5.0, -5.0, 2.0], [-2.0, -7.0, 6.0, -4.0, 3.0], [-4.0, -2.0, -3.0, 0.0, 0.0], [-7.0, -6.0, 3.0, 5.0, 0.0]])
      arg1=Data(1.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[7.0, 6.0, 5.0, -5.0, 2.0], [-2.0, -7.0, 6.0, -4.0, 3.0], [-4.0, -2.0, -3.0, 0.0, 0.0], [-7.0, -6.0, 3.0, 5.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank1_offset1(self):
      arg0=numpy.array([[[-4.0, 7.0, 3.0, 3.0, 0.0], [5.0, 0.0, -1.0, 5.0, 5.0], [-3.0, 7.0, 5.0, -2.0, -2.0], [-3.0, 4.0, 5.0, 6.0, 1.0]], [[0.0, 2.0, -6.0, 1.0, 3.0], [3.0, -7.0, -7.0, -2.0, -6.0], [0.0, -6.0, -1.0, -1.0, 6.0], [-5.0, -4.0, 1.0, -6.0, 0.0]], [[5.0, -4.0, -6.0, -6.0, -5.0], [4.0, -6.0, -2.0, -1.0, 6.0], [3.0, -7.0, 1.0, 0.0, -4.0], [-6.0, -3.0, -1.0, -4.0, 0.0]]])
      arg1=Data(numpy.array([-6.0, 7.0, 7.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[59.0, -56.0, -102.0, -53.0, -14.0], [19.0, -91.0, -57.0, -51.0, -30.0], [39.0, -133.0, -30.0, 5.0, 26.0], [-59.0, -73.0, -30.0, -106.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank2_offset2(self):
      arg0=numpy.array([[[[3.0, -7.0, 7.0, -6.0, 1.0], [0.0, 4.0, 6.0, 0.0, -7.0], [-7.0, -1.0, 2.0, -7.0, 6.0], [-7.0, 1.0, 2.0, 0.0, -1.0]], [[2.0, 3.0, 2.0, -2.0, -3.0], [6.0, -1.0, 0.0, -6.0, 6.0], [-5.0, -1.0, -1.0, -3.0, -4.0], [-6.0, 1.0, -5.0, -1.0, -5.0]], [[4.0, -4.0, -5.0, 3.0, 7.0], [2.0, -4.0, -1.0, -7.0, -4.0], [-5.0, 0.0, -3.0, -5.0, 0.0], [-4.0, 4.0, 6.0, 3.0, 0.0]]], [[[5.0, -4.0, 6.0, 4.0, 0.0], [-3.0, 1.0, -6.0, -3.0, -6.0], [-2.0, 7.0, 0.0, -6.0, 4.0], [4.0, -5.0, 0.0, 2.0, -1.0]], [[0.0, -3.0, 6.0, -3.0, 1.0], [-4.0, 6.0, 1.0, 3.0, 1.0], [-4.0, 4.0, 6.0, 0.0, 2.0], [7.0, -5.0, -3.0, 0.0, 4.0]], [[4.0, 2.0, 6.0, -7.0, -1.0], [6.0, 0.0, 2.0, 4.0, 2.0], [-1.0, 0.0, 2.0, -1.0, 5.0], [-3.0, 0.0, -1.0, -3.0, 3.0]]]])
      arg1=Data(numpy.array([[6.0, 0.0, 6.0], [-7.0, 4.0, 4.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[23.0, -42.0, 18.0, -86.0, 48.0], [41.0, 17.0, 84.0, 7.0, -12.0], [-78.0, -39.0, 26.0, -34.0, 36.0], [-78.0, 45.0, 32.0, -8.0, 29.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank1_offset0(self):
      arg0=numpy.array([[0.0, 7.0, 2.0, 7.0, 5.0], [6.0, -5.0, -4.0, -1.0, 3.0], [0.0, 7.0, -2.0, 2.0, 6.0], [-1.0, -4.0, 4.0, 1.0, 7.0]])
      arg1=Data(numpy.array([5.0, 5.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 0.0], [35.0, 35.0], [10.0, 10.0], [35.0, 35.0], [25.0, 25.0]], [[30.0, 30.0], [-25.0, -25.0], [-20.0, -20.0], [-5.0, -5.0], [15.0, 15.0]], [[0.0, 0.0], [35.0, 35.0], [-10.0, -10.0], [10.0, 10.0], [30.0, 30.0]], [[-5.0, -5.0], [-20.0, -20.0], [20.0, 20.0], [5.0, 5.0], [35.0, 35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank2_offset1(self):
      arg0=numpy.array([[[-6.0, 3.0, -4.0, 0.0, 1.0], [2.0, -7.0, 1.0, -4.0, 0.0], [7.0, -1.0, -3.0, 3.0, 2.0], [4.0, 2.0, 0.0, 0.0, -4.0]], [[0.0, 3.0, 0.0, 5.0, -4.0], [-1.0, -4.0, -5.0, 3.0, -2.0], [0.0, 6.0, -6.0, 1.0, -7.0], [-1.0, 1.0, 3.0, 0.0, -1.0]], [[-5.0, -4.0, 0.0, 3.0, 4.0], [5.0, -2.0, 0.0, -1.0, 3.0], [-6.0, 7.0, 7.0, -2.0, -1.0], [-5.0, 4.0, -3.0, -3.0, -5.0]]])
      arg1=Data(numpy.array([[0.0, 5.0], [0.0, 1.0], [4.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-20.0, -30.0], [-16.0, 18.0], [0.0, -20.0], [12.0, 5.0], [16.0, 1.0]], [[20.0, 9.0], [-8.0, -39.0], [0.0, 0.0], [-4.0, -17.0], [12.0, -2.0]], [[-24.0, 35.0], [28.0, 1.0], [28.0, -21.0], [-8.0, 16.0], [-4.0, 3.0]], [[-20.0, 19.0], [16.0, 11.0], [-12.0, 3.0], [-12.0, 0.0], [-20.0, -21.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank3_offset2(self):
      arg0=numpy.array([[[[0.0, 4.0, -6.0, 5.0, -4.0], [-6.0, -3.0, 2.0, -4.0, -3.0], [-3.0, -7.0, -3.0, 0.0, -6.0], [6.0, -5.0, 1.0, -4.0, 1.0]], [[2.0, -5.0, -3.0, 5.0, 3.0], [1.0, -6.0, 0.0, 4.0, -6.0], [-2.0, 4.0, -6.0, -5.0, -4.0], [-2.0, 3.0, 5.0, 0.0, -3.0]], [[-7.0, -1.0, -1.0, -7.0, 2.0], [-2.0, 5.0, -3.0, 1.0, 7.0], [-3.0, -2.0, 2.0, -4.0, -7.0], [0.0, 3.0, -2.0, 4.0, 1.0]]], [[[-7.0, 7.0, 6.0, -3.0, 0.0], [2.0, -7.0, 1.0, -5.0, -5.0], [-2.0, 5.0, 1.0, 3.0, -2.0], [-4.0, 6.0, 4.0, -2.0, -5.0]], [[-2.0, -7.0, 1.0, -1.0, 7.0], [2.0, -2.0, -4.0, -6.0, -6.0], [-1.0, 6.0, 3.0, 2.0, 7.0], [-7.0, -5.0, -7.0, -3.0, -6.0]], [[-6.0, 5.0, -6.0, -7.0, 4.0], [-6.0, -4.0, 7.0, -3.0, 0.0], [7.0, 4.0, 7.0, 2.0, -4.0], [0.0, -1.0, 2.0, -6.0, 1.0]]]])
      arg1=Data(numpy.array([[[-5.0, 1.0], [-2.0, 7.0], [6.0, 0.0]], [[-6.0, -6.0], [-4.0, -3.0], [3.0, -1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-14.0, 68.0], [-15.0, -57.0], [-28.0, -60.0], [-76.0, 68.0], [10.0, -8.0]], [[-22.0, -11.0], [95.0, 7.0], [3.0, 1.0], [63.0, 75.0], [123.0, 3.0]], [[38.0, -9.0], [-27.0, -31.0], [42.0, -67.0], [-34.0, -61.0], [-32.0, -39.0]], [[26.0, 37.0], [18.0, -4.0], [-17.0, 31.0], [50.0, 23.0], [64.0, 27.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_constData_rank2_offset0(self):
      arg0=numpy.array([[6.0, -7.0, -5.0, -3.0, -3.0], [7.0, -6.0, -7.0, 6.0, -7.0], [3.0, 5.0, 1.0, -5.0, -3.0], [-3.0, -1.0, 2.0, 2.0, -2.0]])
      arg1=Data(numpy.array([[2.0, -1.0, -6.0, -7.0, 3.0], [-5.0, 7.0, -2.0, -2.0, 0.0], [-7.0, 7.0, 5.0, 3.0, 4.0], [-5.0, 4.0, -5.0, 2.0, 1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[12.0, -6.0, -36.0, -42.0, 18.0], [-30.0, 42.0, -12.0, -12.0, 0.0], [-42.0, 42.0, 30.0, 18.0, 24.0], [-30.0, 24.0, -30.0, 12.0, 6.0]], [[-14.0, 7.0, 42.0, 49.0, -21.0], [35.0, -49.0, 14.0, 14.0, 0.0], [49.0, -49.0, -35.0, -21.0, -28.0], [35.0, -28.0, 35.0, -14.0, -7.0]], [[-10.0, 5.0, 30.0, 35.0, -15.0], [25.0, -35.0, 10.0, 10.0, 0.0], [35.0, -35.0, -25.0, -15.0, -20.0], [25.0, -20.0, 25.0, -10.0, -5.0]], [[-6.0, 3.0, 18.0, 21.0, -9.0], [15.0, -21.0, 6.0, 6.0, 0.0], [21.0, -21.0, -15.0, -9.0, -12.0], [15.0, -12.0, 15.0, -6.0, -3.0]], [[-6.0, 3.0, 18.0, 21.0, -9.0], [15.0, -21.0, 6.0, 6.0, 0.0], [21.0, -21.0, -15.0, -9.0, -12.0], [15.0, -12.0, 15.0, -6.0, -3.0]]], [[[14.0, -7.0, -42.0, -49.0, 21.0], [-35.0, 49.0, -14.0, -14.0, 0.0], [-49.0, 49.0, 35.0, 21.0, 28.0], [-35.0, 28.0, -35.0, 14.0, 7.0]], [[-12.0, 6.0, 36.0, 42.0, -18.0], [30.0, -42.0, 12.0, 12.0, 0.0], [42.0, -42.0, -30.0, -18.0, -24.0], [30.0, -24.0, 30.0, -12.0, -6.0]], [[-14.0, 7.0, 42.0, 49.0, -21.0], [35.0, -49.0, 14.0, 14.0, 0.0], [49.0, -49.0, -35.0, -21.0, -28.0], [35.0, -28.0, 35.0, -14.0, -7.0]], [[12.0, -6.0, -36.0, -42.0, 18.0], [-30.0, 42.0, -12.0, -12.0, 0.0], [-42.0, 42.0, 30.0, 18.0, 24.0], [-30.0, 24.0, -30.0, 12.0, 6.0]], [[-14.0, 7.0, 42.0, 49.0, -21.0], [35.0, -49.0, 14.0, 14.0, 0.0], [49.0, -49.0, -35.0, -21.0, -28.0], [35.0, -28.0, 35.0, -14.0, -7.0]]], [[[6.0, -3.0, -18.0, -21.0, 9.0], [-15.0, 21.0, -6.0, -6.0, 0.0], [-21.0, 21.0, 15.0, 9.0, 12.0], [-15.0, 12.0, -15.0, 6.0, 3.0]], [[10.0, -5.0, -30.0, -35.0, 15.0], [-25.0, 35.0, -10.0, -10.0, 0.0], [-35.0, 35.0, 25.0, 15.0, 20.0], [-25.0, 20.0, -25.0, 10.0, 5.0]], [[2.0, -1.0, -6.0, -7.0, 3.0], [-5.0, 7.0, -2.0, -2.0, 0.0], [-7.0, 7.0, 5.0, 3.0, 4.0], [-5.0, 4.0, -5.0, 2.0, 1.0]], [[-10.0, 5.0, 30.0, 35.0, -15.0], [25.0, -35.0, 10.0, 10.0, 0.0], [35.0, -35.0, -25.0, -15.0, -20.0], [25.0, -20.0, 25.0, -10.0, -5.0]], [[-6.0, 3.0, 18.0, 21.0, -9.0], [15.0, -21.0, 6.0, 6.0, 0.0], [21.0, -21.0, -15.0, -9.0, -12.0], [15.0, -12.0, 15.0, -6.0, -3.0]]], [[[-6.0, 3.0, 18.0, 21.0, -9.0], [15.0, -21.0, 6.0, 6.0, 0.0], [21.0, -21.0, -15.0, -9.0, -12.0], [15.0, -12.0, 15.0, -6.0, -3.0]], [[-2.0, 1.0, 6.0, 7.0, -3.0], [5.0, -7.0, 2.0, 2.0, 0.0], [7.0, -7.0, -5.0, -3.0, -4.0], [5.0, -4.0, 5.0, -2.0, -1.0]], [[4.0, -2.0, -12.0, -14.0, 6.0], [-10.0, 14.0, -4.0, -4.0, 0.0], [-14.0, 14.0, 10.0, 6.0, 8.0], [-10.0, 8.0, -10.0, 4.0, 2.0]], [[4.0, -2.0, -12.0, -14.0, 6.0], [-10.0, 14.0, -4.0, -4.0, 0.0], [-14.0, 14.0, 10.0, 6.0, 8.0], [-10.0, 8.0, -10.0, 4.0, 2.0]], [[-4.0, 2.0, 12.0, 14.0, -6.0], [10.0, -14.0, 4.0, 4.0, 0.0], [14.0, -14.0, -10.0, -6.0, -8.0], [10.0, -8.0, 10.0, -4.0, -2.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank3_offset1(self):
      arg0=numpy.array([[[6.0, -3.0, -7.0, 4.0, 5.0], [2.0, -7.0, -2.0, 5.0, 5.0], [-2.0, 6.0, -6.0, -6.0, -1.0], [-5.0, 0.0, 4.0, -7.0, -3.0]], [[4.0, 1.0, 2.0, -6.0, -4.0], [-7.0, 1.0, -1.0, -1.0, -7.0], [-6.0, 6.0, -3.0, -6.0, 6.0], [-1.0, 0.0, -5.0, -5.0, 7.0]], [[-3.0, 4.0, -7.0, -7.0, 4.0], [3.0, -4.0, 4.0, -5.0, -1.0], [6.0, 5.0, 3.0, 0.0, -2.0], [-3.0, 7.0, 4.0, 3.0, 0.0]]])
      arg1=Data(numpy.array([[[5.0, 4.0, 7.0, -3.0, 7.0], [4.0, -7.0, 1.0, 6.0, -3.0], [0.0, -4.0, -6.0, -7.0, -5.0], [-4.0, -1.0, 2.0, 6.0, 6.0]], [[0.0, -4.0, 5.0, -1.0, 4.0], [-7.0, 0.0, -2.0, -6.0, 7.0], [-3.0, 1.0, 4.0, -3.0, 5.0], [0.0, -5.0, 5.0, 0.0, 4.0]], [[6.0, 5.0, -5.0, -2.0, 2.0], [-6.0, -2.0, 0.0, 7.0, -5.0], [-4.0, 1.0, -7.0, 4.0, -5.0], [-2.0, -2.0, 5.0, 3.0, 3.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[12.0, -7.0, 77.0, -16.0, 52.0], [14.0, -36.0, -2.0, -9.0, 25.0], [0.0, -23.0, 1.0, -66.0, 5.0], [-18.0, -20.0, 17.0, 27.0, 43.0]], [[9.0, 4.0, -36.0, 0.0, -9.0], [-43.0, 13.0, -5.0, 4.0, -4.0], [-19.0, 17.0, -6.0, 34.0, 0.0], [4.0, -10.0, 19.0, -6.0, -2.0]], [[-77.0, -71.0, -4.0, 33.0, -55.0], [0.0, 63.0, -11.0, -103.0, 70.0], [22.0, 23.0, 99.0, 15.0, 80.0], [42.0, 11.0, -39.0, -63.0, -55.0]], [[-22.0, 5.0, 33.0, 8.0, -10.0], [100.0, -14.0, 16.0, 11.0, -19.0], [46.0, -29.0, 1.0, -38.0, -15.0], [-2.0, 40.0, -57.0, 3.0, -21.0]], [[49.0, 56.0, -5.0, -19.0, 27.0], [24.0, -43.0, 13.0, 82.0, -63.0], [-4.0, -20.0, -74.0, -7.0, -65.0], [-28.0, 7.0, 10.0, 42.0, 26.0]]], [[[28.0, 51.0, -36.0, -5.0, -8.0], [39.0, -20.0, 16.0, 75.0, -70.0], [9.0, -12.0, -61.0, 19.0, -60.0], [-14.0, 27.0, -16.0, 21.0, -7.0]], [[-59.0, -52.0, -24.0, 28.0, -53.0], [-11.0, 57.0, -9.0, -76.0, 48.0], [13.0, 25.0, 74.0, 30.0, 60.0], [36.0, 10.0, -29.0, -54.0, -50.0]], [[14.0, 16.0, -39.0, -1.0, -10.0], [-25.0, 6.0, 0.0, 22.0, -21.0], [-13.0, 11.0, -20.0, 33.0, -15.0], [0.0, -1.0, 11.0, 0.0, -4.0]], [[-5.0, -1.0, 55.0, -4.0, 21.0], [57.0, -25.0, 7.0, 1.0, 3.0], [23.0, -26.0, 1.0, -52.0, -5.0], [-10.0, 10.0, -20.0, 15.0, 11.0]], [[19.0, 43.0, 5.0, -6.0, 5.0], [75.0, -33.0, 19.0, 65.0, -59.0], [25.0, -28.0, -51.0, -18.0, -55.0], [-18.0, 32.0, -30.0, 27.0, -1.0]]], [[[26.0, 46.0, -74.0, 0.0, -26.0], [-2.0, 2.0, 10.0, 66.0, -66.0], [-6.0, 8.0, -54.0, 56.0, -50.0], [-4.0, 20.0, -4.0, 6.0, -18.0]], [[60.0, 25.0, 47.0, -34.0, 76.0], [-48.0, -52.0, -6.0, 35.0, -1.0], [-38.0, -13.0, -47.0, -40.0, -25.0], [-34.0, -46.0, 67.0, 51.0, 75.0]], [[-12.0, 3.0, -72.0, 15.0, -48.0], [-21.0, 36.0, 0.0, 3.0, -18.0], [-3.0, 24.0, 3.0, 63.0, 0.0], [18.0, 15.0, -12.0, -27.0, -39.0]], [[-30.0, 0.0, -72.0, 24.0, -66.0], [18.0, 42.0, 6.0, 0.0, -24.0], [18.0, 18.0, 12.0, 60.0, 0.0], [24.0, 36.0, -42.0, -36.0, -60.0]], [[-17.0, -38.0, 33.0, 1.0, 13.0], [-34.0, 11.0, -13.0, -56.0, 55.0], [-10.0, 8.0, 44.0, -19.0, 45.0], [8.0, -25.0, 18.0, -12.0, 12.0]]], [[[-43.0, -31.0, -25.0, 22.0, -45.0], [5.0, 41.0, -3.0, -45.0, 23.0], [15.0, 16.0, 47.0, 26.0, 35.0], [26.0, 16.0, -30.0, -39.0, -43.0]], [[42.0, 35.0, -35.0, -14.0, 14.0], [-42.0, -14.0, 0.0, 49.0, -35.0], [-28.0, 7.0, -49.0, 28.0, -35.0], [-14.0, -14.0, 35.0, 21.0, 21.0]], [[44.0, 56.0, -17.0, -15.0, 16.0], [27.0, -36.0, 14.0, 82.0, -67.0], [-1.0, -17.0, -72.0, 3.0, -65.0], [-24.0, 13.0, 3.0, 36.0, 16.0]], [[-17.0, 7.0, -89.0, 20.0, -63.0], [-11.0, 43.0, 3.0, 9.0, -29.0], [3.0, 26.0, 1.0, 76.0, -5.0], [22.0, 26.0, -24.0, -33.0, -53.0]], [[-15.0, -40.0, 14.0, 2.0, 7.0], [-61.0, 21.0, -17.0, -60.0, 58.0], [-21.0, 19.0, 46.0, 0.0, 50.0], [12.0, -32.0, 29.0, -18.0, 10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank4_offset2(self):
      arg0=numpy.array([[[[5.0, -6.0, -4.0, 6.0, 2.0], [4.0, -3.0, -3.0, 1.0, 3.0], [2.0, 7.0, 3.0, 7.0, 3.0], [6.0, 7.0, -1.0, -2.0, 0.0]], [[-5.0, 4.0, -2.0, 3.0, 2.0], [5.0, -1.0, -4.0, 2.0, -1.0], [6.0, -2.0, 4.0, -5.0, 5.0], [0.0, 7.0, -4.0, 4.0, 0.0]], [[0.0, -5.0, 0.0, 1.0, 7.0], [1.0, 0.0, 6.0, -2.0, -2.0], [2.0, 0.0, 0.0, 7.0, 4.0], [-5.0, 3.0, 3.0, 5.0, 5.0]]], [[[5.0, 5.0, -5.0, -7.0, -2.0], [-3.0, -7.0, -1.0, -1.0, -1.0], [0.0, -2.0, 5.0, 6.0, -3.0], [6.0, -6.0, -3.0, 1.0, 1.0]], [[1.0, 4.0, -6.0, 4.0, 6.0], [2.0, 1.0, 5.0, -6.0, 2.0], [-4.0, -3.0, -4.0, -3.0, -7.0], [-7.0, 0.0, 1.0, 3.0, 7.0]], [[0.0, 2.0, 7.0, 2.0, -5.0], [-6.0, -2.0, 6.0, 0.0, 0.0], [-1.0, 0.0, -6.0, 4.0, 4.0], [0.0, -3.0, 6.0, 2.0, -2.0]]]])
      arg1=Data(numpy.array([[[[0.0, -6.0, 6.0, 4.0, -3.0], [3.0, 2.0, 4.0, 5.0, -5.0], [5.0, -4.0, -4.0, 0.0, -5.0], [3.0, 0.0, 0.0, -1.0, -2.0]], [[3.0, -4.0, 0.0, 7.0, 0.0], [0.0, 2.0, 4.0, 0.0, 4.0], [-6.0, 0.0, -3.0, 1.0, 3.0], [0.0, 1.0, 0.0, 6.0, 3.0]], [[-7.0, -3.0, -1.0, 5.0, 0.0], [0.0, -1.0, -4.0, 0.0, -6.0], [1.0, -4.0, -1.0, -4.0, 7.0], [-2.0, -3.0, 5.0, 0.0, 5.0]]], [[[0.0, 2.0, -2.0, -6.0, 6.0], [-2.0, 0.0, 1.0, 6.0, -6.0], [3.0, 0.0, 0.0, 5.0, 6.0], [2.0, 1.0, 3.0, -7.0, 3.0]], [[7.0, 4.0, -1.0, 4.0, 0.0], [-1.0, 1.0, -4.0, 0.0, 5.0], [6.0, -4.0, 5.0, 1.0, 5.0], [0.0, 0.0, 6.0, 4.0, 1.0]], [[0.0, 4.0, -1.0, 3.0, -7.0], [4.0, -5.0, 7.0, 7.0, 1.0], [0.0, -4.0, 0.0, -2.0, 3.0], [-5.0, -6.0, 5.0, 5.0, -3.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-8.0, 4.0, 19.0, -41.0, 15.0], [4.0, 1.0, 1.0, 55.0, -70.0], [76.0, -24.0, 0.0, 21.0, -5.0], [25.0, 0.0, 21.0, -66.0, -9.0]], [[75.0, 69.0, -47.0, -29.0, 34.0], [-24.0, -5.0, 15.0, 14.0, 68.0], [-20.0, 20.0, 37.0, 49.0, 63.0], [-8.0, 12.0, 24.0, 21.0, 12.0]], [[-48.0, 26.0, -15.0, -3.0, -67.0], [32.0, -53.0, 44.0, -1.0, 19.0], [-59.0, 12.0, -8.0, -47.0, -25.0], [-57.0, -49.0, -16.0, 38.0, -40.0]], [[30.0, -41.0, 43.0, 114.0, -74.0], [36.0, 11.0, 23.0, 2.0, 40.0], [16.0, -52.0, -14.0, -36.0, -30.0], [-8.0, -19.0, 18.0, 87.0, -21.0]], [[-1.0, -41.0, 8.0, 78.0, 17.0], [-16.0, 32.0, -73.0, -37.0, -7.0], [35.0, -40.0, 9.0, -20.0, 48.0], [13.0, 9.0, 40.0, 23.0, 52.0]]], [[[22.0, -69.0, 33.0, 64.0, 12.0], [-8.0, 49.0, -21.0, -40.0, 16.0], [-6.0, -4.0, -22.0, 0.0, -24.0], [34.0, 35.0, -22.0, 25.0, 23.0]], [[4.0, 4.0, -3.0, 21.0, -19.0], [-4.0, 3.0, -41.0, -71.0, 56.0], [-24.0, 16.0, 20.0, -31.0, -31.0], [-13.0, 4.0, -25.0, 40.0, -11.0]], [[-19.0, 58.0, -33.0, 34.0, -39.0], [12.0, -45.0, -31.0, 21.0, 0.0], [42.0, -56.0, 43.0, -40.0, 82.0], [-53.0, -59.0, 87.0, 36.0, 8.0]], [[-22.0, -34.0, 16.0, -10.0, -9.0], [11.0, 2.0, 43.0, -1.0, -9.0], [-48.0, 28.0, -38.0, -1.0, -49.0], [5.0, 7.0, -49.0, -6.0, -15.0]], [[25.0, -2.0, 20.0, 9.0, -15.0], [9.0, 8.0, 7.0, 9.0, 9.0], [28.0, -12.0, 3.0, 4.0, -28.0], [11.0, 4.0, -1.0, 6.0, -20.0]]], [[[-24.0, -62.0, 15.0, 41.0, 1.0], [6.0, 15.0, 33.0, 3.0, -19.0], [-48.0, 4.0, -48.0, -4.0, -1.0], [7.0, 6.0, -19.0, 13.0, 23.0]], [[-27.0, -50.0, 49.0, 14.0, -33.0], [28.0, 7.0, 30.0, 23.0, -46.0], [23.0, -16.0, -37.0, -15.0, -68.0], [17.0, -4.0, -24.0, -17.0, -29.0]], [[-16.0, -64.0, 18.0, -24.0, 63.0], [-21.0, 40.0, 7.0, 3.0, -55.0], [-18.0, 28.0, -44.0, 37.0, -11.0], [49.0, 45.0, -39.0, -60.0, 35.0]], [[-85.0, -27.0, 22.0, -8.0, -13.0], [28.0, -26.0, 26.0, 99.0, -144.0], [72.0, -60.0, -35.0, -14.0, 32.0], [-1.0, -44.0, 55.0, -71.0, 9.0]], [[-62.0, -68.0, 23.0, 69.0, -55.0], [38.0, -15.0, 69.0, 25.0, -32.0], [-62.0, -16.0, -66.0, -41.0, -13.0], [-25.0, -34.0, -11.0, 40.0, 1.0]]], [[[-14.0, -37.0, 36.0, -65.0, 18.0], [13.0, 10.0, 78.0, 66.0, -71.0], [1.0, 24.0, -54.0, 43.0, -64.0], [40.0, 21.0, -49.0, -76.0, -26.0]], [[0.0, -103.0, 54.0, 119.0, -36.0], [21.0, 40.0, 17.0, -22.0, 8.0], [-22.0, -28.0, -52.0, -29.0, -38.0], [18.0, 10.0, -18.0, 62.0, 13.0]], [[-26.0, 35.0, -10.0, 23.0, -57.0], [26.0, -42.0, 3.0, 19.0, 0.0], [19.0, -36.0, 18.0, -42.0, 19.0], [-45.0, -52.0, 42.0, 32.0, -21.0]], [[-2.0, 3.0, -24.0, 57.0, -2.0], [-3.0, -8.0, -9.0, 10.0, 7.0], [-8.0, -32.0, 6.0, -12.0, 84.0], [-24.0, -22.0, 56.0, 41.0, 41.0]], [[14.0, 7.0, -12.0, 41.0, 20.0], [-17.0, 12.0, -61.0, -8.0, -3.0], [50.0, -40.0, 30.0, -4.0, 70.0], [2.0, -2.0, 60.0, 11.0, 41.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank0_offset0(self):
      arg0=numpy.array([[7.0, -2.0, 1.0, -3.0, 1.0], [-2.0, 0.0, 1.0, 4.0, 2.0], [1.0, -4.0, -5.0, -5.0, 5.0], [5.0, -6.0, 6.0, -2.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-4.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, -0.0, -0.0]])+(1.-msk_ref)*numpy.array([[-28.0, 8.0, -4.0, 12.0, -4.0], [8.0, -0.0, -4.0, -16.0, -8.0], [-4.0, 16.0, 20.0, 20.0, -20.0], [-20.0, 24.0, -24.0, 8.0, 28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[6.0, -5.0, -5.0, 2.0, -1.0], [1.0, 0.0, 7.0, -1.0, -4.0], [7.0, 6.0, -7.0, -2.0, 7.0], [2.0, -5.0, 1.0, -1.0, 2.0]], [[0.0, 2.0, -6.0, 0.0, -2.0], [-6.0, 3.0, 3.0, 3.0, -1.0], [-2.0, 4.0, 4.0, 7.0, 0.0], [-3.0, 7.0, 1.0, 7.0, 0.0]], [[1.0, -2.0, 5.0, 4.0, -3.0], [7.0, -4.0, 5.0, 5.0, -4.0], [0.0, 7.0, 6.0, -4.0, 4.0], [-2.0, 0.0, -3.0, 3.0, -7.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, -3.0, 2.0])+(1.-msk_arg1)*numpy.array([2.0, 1.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-28.0, 15.0, 53.0, -2.0, 5.0], [27.0, -17.0, -34.0, 6.0, 15.0], [-29.0, -28.0, 35.0, -19.0, -27.0], [-5.0, 4.0, -14.0, -10.0, -24.0]])+(1.-msk_ref)*numpy.array([[12.0, -8.0, -16.0, 4.0, -4.0], [-4.0, 3.0, 17.0, 1.0, -9.0], [12.0, 16.0, -10.0, 3.0, 14.0], [1.0, -3.0, 3.0, 5.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[[-2.0, 3.0, 7.0, -4.0, -2.0], [0.0, -2.0, 0.0, -4.0, 0.0], [3.0, 0.0, -1.0, 6.0, -1.0], [6.0, 1.0, 7.0, 7.0, -1.0]], [[-1.0, 4.0, -6.0, 5.0, 7.0], [4.0, 0.0, -5.0, -4.0, -5.0], [6.0, 6.0, -1.0, 1.0, 2.0], [-7.0, 5.0, -6.0, 7.0, -7.0]], [[7.0, 0.0, 4.0, 6.0, 0.0], [-1.0, -3.0, 5.0, 7.0, 6.0], [1.0, -3.0, 6.0, 5.0, 0.0], [-2.0, -4.0, 0.0, 7.0, 0.0]]], [[[1.0, 4.0, 0.0, 5.0, 5.0], [0.0, 6.0, 3.0, 5.0, -6.0], [-7.0, -4.0, -5.0, 0.0, 1.0], [5.0, -7.0, -4.0, 1.0, 1.0]], [[3.0, 1.0, -5.0, -7.0, -6.0], [4.0, -2.0, 0.0, -3.0, 3.0], [-5.0, 0.0, 1.0, 0.0, 2.0], [-2.0, -7.0, -2.0, 0.0, 3.0]], [[-5.0, 3.0, 0.0, -1.0, 4.0], [4.0, 3.0, 3.0, -5.0, 0.0], [6.0, 5.0, 2.0, 0.0, 0.0], [-7.0, 1.0, -1.0, 2.0, -4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -1.0, 7.0], [1.0, -6.0, -7.0]])+(1.-msk_arg1)*numpy.array([[0.0, -5.0, 0.0], [-3.0, 0.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[66.0, -24.0, 71.0, 87.0, 4.0], [-63.0, -26.0, 22.0, 107.0, 23.0], [-15.0, -66.0, 17.0, 40.0, -14.0], [65.0, -4.0, 28.0, 36.0, 17.0]])+(1.-msk_ref)*numpy.array([[2.0, -32.0, 30.0, -40.0, -50.0], [-20.0, -18.0, 16.0, 5.0, 43.0], [-9.0, -18.0, 20.0, -5.0, -13.0], [20.0, -4.0, 42.0, -38.0, 32.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank1_offset0(self):
      arg0=numpy.array([[-4.0, -3.0, 5.0, 0.0, 7.0], [-5.0, 5.0, 0.0, 2.0, -6.0], [-7.0, 7.0, -6.0, -4.0, -4.0], [-3.0, 0.0, 0.0, 5.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -2.0])+(1.-msk_arg1)*numpy.array([-3.0, 3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-24.0, 8.0], [-18.0, 6.0], [30.0, -10.0], [0.0, 0.0], [42.0, -14.0]], [[-30.0, 10.0], [30.0, -10.0], [0.0, 0.0], [12.0, -4.0], [-36.0, 12.0]], [[-42.0, 14.0], [42.0, -14.0], [-36.0, 12.0], [-24.0, 8.0], [-24.0, 8.0]], [[-18.0, 6.0], [0.0, 0.0], [0.0, 0.0], [30.0, -10.0], [12.0, -4.0]]])+(1.-msk_ref)*numpy.array([[[12.0, -12.0], [9.0, -9.0], [-15.0, 15.0], [0.0, 0.0], [-21.0, 21.0]], [[15.0, -15.0], [-15.0, 15.0], [0.0, 0.0], [-6.0, 6.0], [18.0, -18.0]], [[21.0, -21.0], [-21.0, 21.0], [18.0, -18.0], [12.0, -12.0], [12.0, -12.0]], [[9.0, -9.0], [0.0, 0.0], [0.0, 0.0], [-15.0, 15.0], [-6.0, 6.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[-3.0, -7.0, 6.0, -7.0, 6.0], [6.0, -7.0, 7.0, 3.0, 0.0], [-4.0, 0.0, -7.0, -6.0, -5.0], [0.0, 5.0, -6.0, -3.0, 0.0]], [[6.0, 0.0, -6.0, 6.0, -6.0], [-2.0, -3.0, -5.0, -3.0, 0.0], [-5.0, -3.0, -6.0, 6.0, 6.0], [2.0, -6.0, 0.0, 1.0, 0.0]], [[0.0, -2.0, 5.0, -5.0, 6.0], [3.0, 5.0, -5.0, 0.0, -3.0], [7.0, 6.0, -5.0, 5.0, -5.0], [4.0, -6.0, -1.0, 3.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -6.0], [-3.0, -7.0], [4.0, -7.0]])+(1.-msk_arg1)*numpy.array([[7.0, 1.0], [5.0, 0.0], [0.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-3.0, -24.0], [27.0, 56.0], [8.0, -29.0], [-3.0, 35.0], [12.0, -36.0]], [[-12.0, -43.0], [64.0, 28.0], [-40.0, 28.0], [-6.0, 3.0], [-12.0, 21.0]], [[63.0, 10.0], [33.0, -21.0], [33.0, 119.0], [32.0, -41.0], [-13.0, 23.0]], [[10.0, -42.0], [-31.0, 54.0], [26.0, 43.0], [24.0, -10.0], [-24.0, 42.0]]])+(1.-msk_ref)*numpy.array([[[9.0, -3.0], [-49.0, -9.0], [12.0, 11.0], [-19.0, -12.0], [12.0, 12.0]], [[32.0, 9.0], [-64.0, -2.0], [24.0, 2.0], [6.0, 3.0], [0.0, -3.0]], [[-53.0, 3.0], [-15.0, 6.0], [-79.0, -12.0], [-12.0, -1.0], [-5.0, -10.0]], [[10.0, 4.0], [5.0, -1.0], [-42.0, -7.0], [-16.0, 0.0], [0.0, -6.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[[-3.0, -3.0, 4.0, -6.0, 3.0], [1.0, 3.0, -1.0, 2.0, -5.0], [-2.0, -7.0, 7.0, -1.0, -6.0], [5.0, 6.0, -2.0, 6.0, -2.0]], [[4.0, 3.0, 2.0, -6.0, -3.0], [-2.0, -5.0, -1.0, 2.0, 0.0], [4.0, -2.0, 0.0, 3.0, -7.0], [6.0, 1.0, -2.0, 0.0, 6.0]], [[4.0, 6.0, -4.0, -4.0, -5.0], [-5.0, -6.0, 7.0, -7.0, 6.0], [-4.0, 1.0, -7.0, 4.0, -3.0], [6.0, 0.0, 1.0, 3.0, -4.0]]], [[[-2.0, -6.0, 1.0, -7.0, 7.0], [1.0, -4.0, -5.0, 6.0, -5.0], [0.0, -5.0, -4.0, -6.0, 1.0], [-6.0, 7.0, 7.0, 2.0, 2.0]], [[1.0, -1.0, -5.0, -6.0, 1.0], [-2.0, 6.0, -4.0, 0.0, 5.0], [5.0, -1.0, 5.0, -1.0, -1.0], [-7.0, -5.0, -4.0, -5.0, 2.0]], [[-6.0, 7.0, -7.0, -7.0, -5.0], [2.0, 3.0, 0.0, -3.0, 7.0], [-4.0, 3.0, -7.0, -3.0, -1.0], [0.0, -4.0, 5.0, -4.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, -3.0], [2.0, -1.0], [2.0, -7.0]], [[-2.0, 2.0], [7.0, -4.0], [-7.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 2.0], [0.0, -1.0], [0.0, -5.0]], [[1.0, -6.0], [3.0, -3.0], [3.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[54.0, -49.0], [-41.0, -23.0], [28.0, 15.0], [-29.0, 41.0], [27.0, 24.0]], [[-39.0, 50.0], [22.0, 15.0], [-11.0, -39.0], [9.0, 44.0], [-17.0, -36.0]], [[53.0, -2.0], [-55.0, 19.0], [113.0, -21.0], [35.0, -45.0], [-52.0, 49.0]], [[12.0, -47.0], [11.0, 3.0], [-89.0, 46.0], [25.0, -27.0], [18.0, 18.0]]])+(1.-msk_ref)*numpy.array([[[-8.0, -21.0], [21.0, 0.0], [-47.0, 35.0], [-28.0, 74.0], [-14.0, -11.0]], [[-2.0, 29.0], [14.0, 47.0], [-14.0, 6.0], [-9.0, 1.0], [46.0, -25.0]], [[9.0, -3.0], [22.0, 16.0], [-31.0, 58.0], [-15.0, 14.0], [13.0, 7.0]], [[-42.0, 31.0], [-38.0, -16.0], [16.0, -37.0], [-43.0, 0.0], [8.0, -8.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank2_expandedData_rank2_offset0(self):
      arg0=numpy.array([[4.0, 0.0, 0.0, 3.0, 6.0], [-1.0, 3.0, 3.0, 5.0, 2.0], [5.0, 7.0, 6.0, -6.0, 0.0], [-7.0, 4.0, -5.0, -6.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -6.0, 0.0, 1.0, 2.0], [5.0, 7.0, 3.0, -4.0, 2.0], [0.0, -7.0, 4.0, -1.0, -7.0], [5.0, 4.0, 5.0, -6.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-7.0, 0.0, -5.0, -6.0, 5.0], [-4.0, 4.0, 0.0, 0.0, 2.0], [2.0, -6.0, 2.0, 7.0, -2.0], [7.0, -2.0, 0.0, -3.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-20.0, -24.0, 0.0, 4.0, 8.0], [20.0, 28.0, 12.0, -16.0, 8.0], [0.0, -28.0, 16.0, -4.0, -28.0], [20.0, 16.0, 20.0, -24.0, -16.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-15.0, -18.0, 0.0, 3.0, 6.0], [15.0, 21.0, 9.0, -12.0, 6.0], [0.0, -21.0, 12.0, -3.0, -21.0], [15.0, 12.0, 15.0, -18.0, -12.0]], [[-30.0, -36.0, 0.0, 6.0, 12.0], [30.0, 42.0, 18.0, -24.0, 12.0], [0.0, -42.0, 24.0, -6.0, -42.0], [30.0, 24.0, 30.0, -36.0, -24.0]]], [[[5.0, 6.0, 0.0, -1.0, -2.0], [-5.0, -7.0, -3.0, 4.0, -2.0], [0.0, 7.0, -4.0, 1.0, 7.0], [-5.0, -4.0, -5.0, 6.0, 4.0]], [[-15.0, -18.0, 0.0, 3.0, 6.0], [15.0, 21.0, 9.0, -12.0, 6.0], [0.0, -21.0, 12.0, -3.0, -21.0], [15.0, 12.0, 15.0, -18.0, -12.0]], [[-15.0, -18.0, 0.0, 3.0, 6.0], [15.0, 21.0, 9.0, -12.0, 6.0], [0.0, -21.0, 12.0, -3.0, -21.0], [15.0, 12.0, 15.0, -18.0, -12.0]], [[-25.0, -30.0, 0.0, 5.0, 10.0], [25.0, 35.0, 15.0, -20.0, 10.0], [0.0, -35.0, 20.0, -5.0, -35.0], [25.0, 20.0, 25.0, -30.0, -20.0]], [[-10.0, -12.0, 0.0, 2.0, 4.0], [10.0, 14.0, 6.0, -8.0, 4.0], [0.0, -14.0, 8.0, -2.0, -14.0], [10.0, 8.0, 10.0, -12.0, -8.0]]], [[[-25.0, -30.0, 0.0, 5.0, 10.0], [25.0, 35.0, 15.0, -20.0, 10.0], [0.0, -35.0, 20.0, -5.0, -35.0], [25.0, 20.0, 25.0, -30.0, -20.0]], [[-35.0, -42.0, 0.0, 7.0, 14.0], [35.0, 49.0, 21.0, -28.0, 14.0], [0.0, -49.0, 28.0, -7.0, -49.0], [35.0, 28.0, 35.0, -42.0, -28.0]], [[-30.0, -36.0, 0.0, 6.0, 12.0], [30.0, 42.0, 18.0, -24.0, 12.0], [0.0, -42.0, 24.0, -6.0, -42.0], [30.0, 24.0, 30.0, -36.0, -24.0]], [[30.0, 36.0, 0.0, -6.0, -12.0], [-30.0, -42.0, -18.0, 24.0, -12.0], [0.0, 42.0, -24.0, 6.0, 42.0], [-30.0, -24.0, -30.0, 36.0, 24.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[35.0, 42.0, 0.0, -7.0, -14.0], [-35.0, -49.0, -21.0, 28.0, -14.0], [0.0, 49.0, -28.0, 7.0, 49.0], [-35.0, -28.0, -35.0, 42.0, 28.0]], [[-20.0, -24.0, 0.0, 4.0, 8.0], [20.0, 28.0, 12.0, -16.0, 8.0], [0.0, -28.0, 16.0, -4.0, -28.0], [20.0, 16.0, 20.0, -24.0, -16.0]], [[25.0, 30.0, 0.0, -5.0, -10.0], [-25.0, -35.0, -15.0, 20.0, -10.0], [0.0, 35.0, -20.0, 5.0, 35.0], [-25.0, -20.0, -25.0, 30.0, 20.0]], [[30.0, 36.0, 0.0, -6.0, -12.0], [-30.0, -42.0, -18.0, 24.0, -12.0], [0.0, 42.0, -24.0, 6.0, 42.0], [-30.0, -24.0, -30.0, 36.0, 24.0]], [[-5.0, -6.0, 0.0, 1.0, 2.0], [5.0, 7.0, 3.0, -4.0, 2.0], [0.0, -7.0, 4.0, -1.0, -7.0], [5.0, 4.0, 5.0, -6.0, -4.0]]]])+(1.-msk_ref)*numpy.array([[[[-28.0, 0.0, -20.0, -24.0, 20.0], [-16.0, 16.0, 0.0, 0.0, 8.0], [8.0, -24.0, 8.0, 28.0, -8.0], [28.0, -8.0, 0.0, -12.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-21.0, 0.0, -15.0, -18.0, 15.0], [-12.0, 12.0, 0.0, 0.0, 6.0], [6.0, -18.0, 6.0, 21.0, -6.0], [21.0, -6.0, 0.0, -9.0, 3.0]], [[-42.0, 0.0, -30.0, -36.0, 30.0], [-24.0, 24.0, 0.0, 0.0, 12.0], [12.0, -36.0, 12.0, 42.0, -12.0], [42.0, -12.0, 0.0, -18.0, 6.0]]], [[[7.0, 0.0, 5.0, 6.0, -5.0], [4.0, -4.0, 0.0, 0.0, -2.0], [-2.0, 6.0, -2.0, -7.0, 2.0], [-7.0, 2.0, 0.0, 3.0, -1.0]], [[-21.0, 0.0, -15.0, -18.0, 15.0], [-12.0, 12.0, 0.0, 0.0, 6.0], [6.0, -18.0, 6.0, 21.0, -6.0], [21.0, -6.0, 0.0, -9.0, 3.0]], [[-21.0, 0.0, -15.0, -18.0, 15.0], [-12.0, 12.0, 0.0, 0.0, 6.0], [6.0, -18.0, 6.0, 21.0, -6.0], [21.0, -6.0, 0.0, -9.0, 3.0]], [[-35.0, 0.0, -25.0, -30.0, 25.0], [-20.0, 20.0, 0.0, 0.0, 10.0], [10.0, -30.0, 10.0, 35.0, -10.0], [35.0, -10.0, 0.0, -15.0, 5.0]], [[-14.0, 0.0, -10.0, -12.0, 10.0], [-8.0, 8.0, 0.0, 0.0, 4.0], [4.0, -12.0, 4.0, 14.0, -4.0], [14.0, -4.0, 0.0, -6.0, 2.0]]], [[[-35.0, 0.0, -25.0, -30.0, 25.0], [-20.0, 20.0, 0.0, 0.0, 10.0], [10.0, -30.0, 10.0, 35.0, -10.0], [35.0, -10.0, 0.0, -15.0, 5.0]], [[-49.0, 0.0, -35.0, -42.0, 35.0], [-28.0, 28.0, 0.0, 0.0, 14.0], [14.0, -42.0, 14.0, 49.0, -14.0], [49.0, -14.0, 0.0, -21.0, 7.0]], [[-42.0, 0.0, -30.0, -36.0, 30.0], [-24.0, 24.0, 0.0, 0.0, 12.0], [12.0, -36.0, 12.0, 42.0, -12.0], [42.0, -12.0, 0.0, -18.0, 6.0]], [[42.0, 0.0, 30.0, 36.0, -30.0], [24.0, -24.0, 0.0, 0.0, -12.0], [-12.0, 36.0, -12.0, -42.0, 12.0], [-42.0, 12.0, 0.0, 18.0, -6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[49.0, 0.0, 35.0, 42.0, -35.0], [28.0, -28.0, 0.0, 0.0, -14.0], [-14.0, 42.0, -14.0, -49.0, 14.0], [-49.0, 14.0, 0.0, 21.0, -7.0]], [[-28.0, 0.0, -20.0, -24.0, 20.0], [-16.0, 16.0, 0.0, 0.0, 8.0], [8.0, -24.0, 8.0, 28.0, -8.0], [28.0, -8.0, 0.0, -12.0, 4.0]], [[35.0, 0.0, 25.0, 30.0, -25.0], [20.0, -20.0, 0.0, 0.0, -10.0], [-10.0, 30.0, -10.0, -35.0, 10.0], [-35.0, 10.0, 0.0, 15.0, -5.0]], [[42.0, 0.0, 30.0, 36.0, -30.0], [24.0, -24.0, 0.0, 0.0, -12.0], [-12.0, 36.0, -12.0, -42.0, 12.0], [-42.0, 12.0, 0.0, 18.0, -6.0]], [[-7.0, 0.0, -5.0, -6.0, 5.0], [-4.0, 4.0, 0.0, 0.0, 2.0], [2.0, -6.0, 2.0, 7.0, -2.0], [7.0, -2.0, 0.0, -3.0, 1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank3_offset1(self):
      arg0=numpy.array([[[3.0, -6.0, 2.0, -3.0, -5.0], [-6.0, 4.0, 0.0, 1.0, 6.0], [-3.0, -4.0, 3.0, 5.0, 0.0], [-6.0, 4.0, -4.0, 0.0, -5.0]], [[-4.0, -5.0, 1.0, 1.0, 4.0], [2.0, 0.0, 3.0, 4.0, 0.0], [-3.0, -5.0, 1.0, 6.0, 0.0], [7.0, 1.0, 2.0, -6.0, -1.0]], [[-7.0, -3.0, 0.0, -1.0, 0.0], [4.0, 1.0, 1.0, 0.0, -6.0], [4.0, -1.0, 0.0, -6.0, -4.0], [2.0, -6.0, -6.0, -2.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, 4.0, 6.0, 6.0, -6.0], [-6.0, -2.0, -7.0, 7.0, -1.0], [-1.0, 0.0, 2.0, -7.0, 3.0], [4.0, 1.0, -7.0, 2.0, -6.0]], [[-6.0, -5.0, 7.0, -3.0, -4.0], [7.0, -5.0, 3.0, 4.0, -6.0], [5.0, -2.0, -6.0, 7.0, -3.0], [7.0, 6.0, 1.0, -2.0, 5.0]], [[-1.0, 2.0, 3.0, 6.0, -4.0], [4.0, -5.0, 2.0, -4.0, -1.0], [7.0, 6.0, -1.0, 4.0, 2.0], [5.0, -2.0, -1.0, 4.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, 7.0, -5.0, -2.0, 3.0], [1.0, 0.0, 0.0, 2.0, 0.0], [6.0, 7.0, 2.0, -6.0, 0.0], [7.0, 7.0, -3.0, -1.0, -3.0]], [[1.0, -4.0, -4.0, 1.0, -4.0], [5.0, -1.0, 7.0, -5.0, -7.0], [5.0, 3.0, -1.0, -2.0, -2.0], [1.0, 6.0, -5.0, 3.0, 1.0]], [[-4.0, -7.0, -5.0, 0.0, 4.0], [2.0, -2.0, 5.0, -1.0, 5.0], [-6.0, 1.0, 7.0, -4.0, -3.0], [-1.0, -7.0, 0.0, 1.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[37.0, 18.0, -31.0, -12.0, 26.0], [-74.0, 49.0, -47.0, 33.0, 28.0], [-72.0, -34.0, 37.0, -77.0, 7.0], [-51.0, -7.0, -18.0, -14.0, -87.0]], [[21.0, -5.0, -80.0, -39.0, 68.0], [-11.0, 52.0, 21.0, -50.0, 39.0], [-40.0, -8.0, 21.0, -5.0, -9.0], [-74.0, -30.0, 40.0, -14.0, -10.0]], [[-2.0, 3.0, 19.0, 9.0, -16.0], [-5.0, -9.0, -11.0, 18.0, -8.0], [3.0, -2.0, -2.0, -7.0, 3.0], [15.0, 8.0, -13.0, 2.0, -7.0]], [[-11.0, -19.0, -14.0, -27.0, 18.0], [21.0, 6.0, 22.0, -13.0, -2.0], [1.0, -8.0, -11.0, 24.0, -14.0], [-10.0, 5.0, 23.0, -12.0, 16.0]], [[-34.0, -40.0, -2.0, -42.0, 14.0], [58.0, -10.0, 47.0, -19.0, -19.0], [25.0, -8.0, -34.0, 63.0, -27.0], [8.0, 19.0, 39.0, -18.0, 50.0]]], [[[-28.0, -26.0, -10.0, -18.0, 12.0], [66.0, -18.0, 56.0, -50.0, -10.0], [44.0, 20.0, -28.0, 72.0, -16.0], [10.0, -2.0, 40.0, 0.0, 74.0]], [[7.0, 18.0, 27.0, 30.0, -28.0], [-20.0, -13.0, -26.0, 24.0, -5.0], [3.0, 6.0, 7.0, -24.0, 14.0], [21.0, 2.0, -29.0, 12.0, -17.0]], [[-19.0, -13.0, 24.0, -3.0, -16.0], [25.0, -20.0, 11.0, 8.0, -19.0], [22.0, 0.0, -19.0, 25.0, -7.0], [26.0, 16.0, 2.0, -2.0, 22.0]], [[-22.0, -16.0, 34.0, -6.0, -22.0], [22.0, -22.0, 5.0, 23.0, -25.0], [19.0, -8.0, -22.0, 21.0, -9.0], [32.0, 25.0, -3.0, -6.0, 14.0]], [[18.0, 12.0, 18.0, 0.0, -12.0], [-60.0, 18.0, -54.0, 66.0, 0.0], [-48.0, -36.0, 18.0, -66.0, 6.0], [-6.0, 18.0, -36.0, -12.0, -78.0]]], [[[8.0, 11.0, -27.0, 15.0, 14.0], [13.0, 1.0, 20.0, -49.0, 17.0], [16.0, 30.0, 8.0, 16.0, 8.0], [-13.0, -29.0, 14.0, 16.0, 31.0]], [[23.0, 7.0, -62.0, -15.0, 48.0], [-15.0, 38.0, 11.0, -44.0, 35.0], [-28.0, 4.0, 23.0, -11.0, 1.0], [-56.0, -32.0, 24.0, -2.0, -8.0]], [[0.0, 7.0, 25.0, 15.0, -22.0], [-11.0, -11.0, -18.0, 25.0, -9.0], [2.0, -2.0, 0.0, -14.0, 6.0], [19.0, 9.0, -20.0, 4.0, -13.0]], [[-20.0, -22.0, 54.0, -24.0, -30.0], [-12.0, -10.0, -29.0, 83.0, -35.0], [-17.0, -48.0, -20.0, -17.0, -15.0], [32.0, 53.0, -23.0, -26.0, -42.0]], [[4.0, -8.0, -12.0, -24.0, 16.0], [-16.0, 20.0, -8.0, 16.0, 4.0], [-28.0, -24.0, 4.0, -16.0, -8.0], [-20.0, 8.0, 4.0, -16.0, -28.0]]], [[[-56.0, -55.0, 19.0, -45.0, 0.0], [93.0, -33.0, 67.0, -22.0, -38.0], [55.0, -2.0, -56.0, 99.0, -35.0], [35.0, 32.0, 47.0, -18.0, 85.0]], [[8.0, -1.0, 13.0, -15.0, -4.0], [-41.0, 17.0, -37.0, 56.0, -4.0], [-41.0, -38.0, 8.0, -45.0, -3.0], [-7.0, 22.0, -21.0, -18.0, -61.0]], [[-14.0, -38.0, -28.0, -66.0, 40.0], [14.0, 28.0, 22.0, 4.0, -2.0], [-28.0, -40.0, -14.0, 18.0, -30.0], [-32.0, 20.0, 36.0, -36.0, -8.0]], [[38.0, 26.0, -48.0, 6.0, 32.0], [-50.0, 40.0, -22.0, -16.0, 38.0], [-44.0, 0.0, 38.0, -50.0, 14.0], [-52.0, -32.0, -4.0, 4.0, -44.0]], [[2.0, -27.0, -55.0, -63.0, 58.0], [-1.0, 45.0, 20.0, -15.0, 17.0], [-42.0, -34.0, 2.0, 4.0, -24.0], [-57.0, 1.0, 40.0, -32.0, -17.0]]]])+(1.-msk_ref)*numpy.array([[[[45.0, 86.0, 36.0, -10.0, -3.0], [-31.0, 18.0, -63.0, 33.0, -7.0], [40.0, 2.0, -39.0, 18.0, 29.0], [24.0, 46.0, 11.0, -22.0, -13.0]], [[-35.0, -1.0, 65.0, 7.0, -10.0], [-37.0, 11.0, -50.0, 16.0, 20.0], [-43.0, -60.0, -28.0, 58.0, 19.0], [-44.0, -51.0, 43.0, -12.0, 13.0]], [[15.0, 10.0, -14.0, -3.0, 2.0], [7.0, -1.0, 7.0, -1.0, -7.0], [17.0, 17.0, 3.0, -14.0, -2.0], [15.0, 20.0, -11.0, 1.0, -5.0]], [[-16.0, -18.0, 16.0, 7.0, -17.0], [0.0, 1.0, 2.0, -10.0, -12.0], [-7.0, -19.0, -14.0, 20.0, 1.0], [-19.0, -8.0, 4.0, 5.0, 10.0]], [[-31.0, -51.0, 9.0, 14.0, -31.0], [15.0, -4.0, 28.0, -30.0, -28.0], [-10.0, -23.0, -14.0, 22.0, -8.0], [-31.0, -11.0, -5.0, 17.0, 19.0]]], [[[-56.0, -78.0, 2.0, 14.0, -10.0], [12.0, -10.0, 34.0, -26.0, 6.0], [-50.0, -32.0, 14.0, 16.0, -16.0], [-44.0, -58.0, 8.0, 16.0, 20.0]], [[24.0, 21.0, -25.0, -8.0, 16.0], [6.0, -2.0, 5.0, 7.0, 5.0], [18.0, 29.0, 15.0, -28.0, -3.0], [27.0, 21.0, -12.0, -3.0, -12.0]], [[-1.0, -19.0, -17.0, 3.0, -8.0], [17.0, -5.0, 26.0, -16.0, -16.0], [9.0, 10.0, 4.0, -10.0, -9.0], [2.0, 11.0, -15.0, 10.0, 3.0]], [[11.0, -9.0, -21.0, 2.0, -13.0], [21.0, -4.0, 28.0, -18.0, -28.0], [26.0, 19.0, -2.0, -14.0, -8.0], [11.0, 31.0, -23.0, 11.0, 1.0]], [[66.0, 84.0, 0.0, -12.0, -6.0], [-6.0, 12.0, -30.0, 18.0, -30.0], [72.0, 36.0, -30.0, -12.0, 18.0], [48.0, 84.0, -18.0, -12.0, -18.0]]], [[[-40.0, -37.0, 7.0, 3.0, 19.0], [-10.0, -5.0, -1.0, 5.0, 41.0], [-57.0, -26.0, 25.0, 8.0, -6.0], [-28.0, -67.0, 24.0, -2.0, 6.0]], [[-29.0, -1.0, 45.0, 3.0, 4.0], [-31.0, 7.0, -40.0, 18.0, 30.0], [-43.0, -44.0, -10.0, 38.0, 13.0], [-32.0, -51.0, 37.0, -12.0, 7.0]], [[22.0, 17.0, -19.0, -5.0, 5.0], [8.0, -1.0, 7.0, 1.0, -7.0], [23.0, 24.0, 5.0, -20.0, -2.0], [22.0, 27.0, -14.0, 0.0, -8.0]], [[65.0, 53.0, -19.0, -4.0, -33.0], [23.0, 6.0, 12.0, -14.0, -72.0], [96.0, 47.0, -38.0, -18.0, 6.0], [47.0, 113.0, -45.0, 7.0, -9.0]], [[16.0, 28.0, 20.0, 0.0, -16.0], [-8.0, 8.0, -20.0, 4.0, -20.0], [24.0, -4.0, -28.0, 16.0, 12.0], [4.0, 28.0, 0.0, -4.0, 0.0]]], [[[-43.0, -84.0, -8.0, 19.0, -38.0], [33.0, -11.0, 59.0, -49.0, -39.0], [-13.0, -19.0, -5.0, 14.0, -20.0], [-37.0, -14.0, -17.0, 29.0, 25.0]], [[53.0, 66.0, 6.0, -7.0, -16.0], [-3.0, 11.0, -23.0, 9.0, -37.0], [65.0, 25.0, -35.0, -2.0, 16.0], [35.0, 76.0, -17.0, -7.0, -11.0]], [[-2.0, 6.0, 42.0, 10.0, -44.0], [-6.0, 10.0, -16.0, -12.0, -44.0], [22.0, -28.0, -52.0, 44.0, 14.0], [-20.0, 26.0, 2.0, 4.0, 14.0]], [[2.0, 38.0, 34.0, -6.0, 16.0], [-34.0, 10.0, -52.0, 32.0, 32.0], [-18.0, -20.0, -8.0, 20.0, 18.0], [-4.0, -22.0, 30.0, -20.0, -6.0]], [[-12.0, 11.0, 59.0, 9.0, -35.0], [-22.0, 13.0, -37.0, 1.0, -23.0], [1.0, -44.0, -51.0, 56.0, 20.0], [-30.0, 1.0, 20.0, -4.0, 14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[[-6.0, -1.0, -3.0, -2.0, -7.0], [-1.0, -3.0, -6.0, 1.0, 3.0], [0.0, -1.0, -2.0, -5.0, 3.0], [-4.0, -5.0, 0.0, 1.0, -7.0]], [[-6.0, 2.0, 5.0, 3.0, -4.0], [-3.0, -3.0, -1.0, 1.0, 3.0], [-4.0, -3.0, -7.0, -5.0, -6.0], [-3.0, -7.0, 3.0, 1.0, -5.0]], [[0.0, -1.0, 2.0, 1.0, -5.0], [4.0, -7.0, 6.0, 4.0, -4.0], [-1.0, 1.0, -1.0, 6.0, -4.0], [-6.0, -1.0, 0.0, 5.0, 6.0]]], [[[0.0, -4.0, 0.0, 4.0, -5.0], [-2.0, -4.0, 0.0, 3.0, 4.0], [2.0, 0.0, 6.0, -3.0, 3.0], [0.0, 2.0, -3.0, -3.0, -2.0]], [[1.0, 2.0, 3.0, 6.0, -4.0], [0.0, 3.0, -1.0, 7.0, -4.0], [-2.0, -3.0, -6.0, 6.0, 2.0], [-2.0, 1.0, -5.0, -3.0, -5.0]], [[7.0, -1.0, 0.0, 2.0, -7.0], [-2.0, 2.0, -7.0, -5.0, 3.0], [5.0, 6.0, 0.0, 4.0, 5.0], [4.0, 6.0, -5.0, 2.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -3.0, 7.0, 4.0, -5.0], [7.0, -4.0, -1.0, 0.0, -6.0], [-3.0, -7.0, 2.0, -1.0, -5.0], [3.0, -2.0, -2.0, 5.0, 4.0]], [[-7.0, 7.0, 1.0, -3.0, 6.0], [2.0, -7.0, 0.0, 4.0, -5.0], [6.0, 0.0, 2.0, 0.0, 0.0], [-4.0, -1.0, 2.0, -6.0, 0.0]], [[-4.0, -3.0, 0.0, -5.0, -6.0], [2.0, 0.0, -7.0, -3.0, 3.0], [-2.0, -6.0, 3.0, 3.0, -3.0], [4.0, 0.0, -6.0, 1.0, 4.0]]], [[[3.0, 1.0, 6.0, -7.0, 0.0], [-7.0, 0.0, -7.0, 0.0, -7.0], [-7.0, 6.0, 4.0, -2.0, -6.0], [-5.0, -3.0, 6.0, -2.0, 7.0]], [[0.0, -3.0, -3.0, -4.0, 5.0], [-5.0, 5.0, -3.0, -6.0, 0.0], [-3.0, 1.0, 0.0, 0.0, -2.0], [-3.0, 7.0, 4.0, -7.0, -3.0]], [[-1.0, 2.0, -4.0, 6.0, -4.0], [5.0, 0.0, 0.0, -7.0, 0.0], [-1.0, -6.0, 2.0, 0.0, -3.0], [7.0, -1.0, -1.0, -1.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, -1.0, -2.0, 1.0, 5.0], [7.0, 0.0, 0.0, -1.0, 0.0], [-6.0, -6.0, 5.0, -7.0, 0.0], [4.0, 1.0, 7.0, 5.0, 5.0]], [[0.0, 6.0, -2.0, 0.0, 3.0], [-2.0, 3.0, -2.0, -2.0, -5.0], [0.0, 0.0, -6.0, 4.0, 0.0], [-4.0, 0.0, 2.0, 4.0, 4.0]], [[3.0, 5.0, 0.0, -3.0, 7.0], [4.0, 1.0, -1.0, -6.0, -1.0], [-6.0, 3.0, -5.0, -2.0, -7.0], [5.0, 0.0, -6.0, -4.0, -1.0]]], [[[-6.0, 0.0, 6.0, 1.0, 5.0], [0.0, -1.0, -6.0, -3.0, -5.0], [0.0, -5.0, -2.0, -7.0, 6.0], [3.0, -3.0, 2.0, -5.0, 3.0]], [[-6.0, 3.0, 2.0, 4.0, 0.0], [7.0, 2.0, 4.0, -4.0, 0.0], [4.0, 0.0, 5.0, -7.0, -7.0], [-7.0, -6.0, 0.0, 4.0, 7.0]], [[7.0, 0.0, 3.0, -4.0, 4.0], [-5.0, 5.0, 0.0, -4.0, -2.0], [-5.0, -3.0, 6.0, -1.0, -6.0], [1.0, 5.0, 5.0, 1.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[65.0, -13.0, -79.0, 32.0, -29.0], [-24.0, 71.0, 3.0, -79.0, 66.0], [-28.0, 1.0, -10.0, 6.0, 7.0], [52.0, 18.0, -3.0, -8.0, -69.0]], [[-16.0, 8.0, -31.0, 9.0, 37.0], [8.0, 0.0, 30.0, 6.0, 21.0], [40.0, -3.0, -19.0, 6.0, 31.0], [-8.0, 27.0, -3.0, -23.0, -36.0]], [[-28.0, 29.0, -25.0, -49.0, 48.0], [-22.0, -8.0, -20.0, -4.0, -1.0], [26.0, 12.0, 10.0, 9.0, 3.0], [-30.0, 22.0, 16.0, -64.0, -13.0]], [[-5.0, 14.0, -13.0, -62.0, 44.0], [-54.0, 17.0, -51.0, -41.0, -28.0], [-26.0, 26.0, 25.0, -3.0, -35.0], [-38.0, 29.0, 50.0, -79.0, -6.0]], [[75.0, 1.0, -43.0, 18.0, 49.0], [-47.0, 36.0, 89.0, 72.0, 82.0], [61.0, 87.0, -71.0, 2.0, 109.0], [-37.0, 12.0, -3.0, 29.0, -29.0]]], [[[6.0, -36.0, -14.0, -13.0, -29.0], [-1.0, 25.0, -13.0, -10.0, 47.0], [-7.0, -17.0, -8.0, 17.0, 11.0], [21.0, 13.0, -38.0, 23.0, 10.0]], [[50.0, 0.0, -65.0, 60.0, 46.0], [-18.0, 48.0, 71.0, -23.0, 40.0], [22.0, 30.0, -45.0, -10.0, 48.0], [0.0, 40.0, 28.0, -19.0, -89.0]], [[20.0, -18.0, -12.0, -89.0, 11.0], [-62.0, 26.0, -33.0, 33.0, 59.0], [10.0, 47.0, -10.0, 24.0, 35.0], [-36.0, 13.0, -23.0, -4.0, 45.0]], [[-14.0, -36.0, 25.0, -98.0, 32.0], [-64.0, 24.0, -71.0, -15.0, -20.0], [-42.0, 24.0, 18.0, 5.0, -34.0], [-56.0, 42.0, 27.0, -47.0, 50.0]], [[-11.0, 46.0, 48.0, 29.0, -5.0], [26.0, -53.0, 9.0, 27.0, -73.0], [-2.0, 5.0, 22.0, -23.0, -28.0], [-6.0, -52.0, 29.0, 10.0, 18.0]]], [[[33.0, -7.0, -6.0, 41.0, -48.0], [11.0, 18.0, -1.0, -36.0, 3.0], [-35.0, -14.0, 7.0, -7.0, -20.0], [43.0, -21.0, -3.0, 28.0, -14.0]], [[16.0, 0.0, -25.0, 48.0, -58.0], [34.0, 10.0, 3.0, -39.0, 24.0], [-14.0, -38.0, 7.0, 4.0, -10.0], [64.0, -22.0, -28.0, 29.0, -27.0]], [[81.0, -16.0, 33.0, 0.0, -56.0], [-42.0, 27.0, -15.0, 11.0, 2.0], [-58.0, 50.0, 3.0, -13.0, -11.0], [6.0, -49.0, 8.0, 61.0, 48.0]], [[23.0, -51.0, -92.0, -14.0, -27.0], [-22.0, 85.0, -34.0, -102.0, 94.0], [-28.0, -37.0, -6.0, 29.0, 1.0], [54.0, 62.0, -34.0, -29.0, -59.0]], [[47.0, -32.0, 7.0, 51.0, -37.0], [-5.0, 40.0, -2.0, -59.0, -21.0], [-69.0, -7.0, 4.0, -21.0, -40.0], [31.0, 0.0, 27.0, 22.0, -19.0]]], [[[61.0, 23.0, -41.0, 55.0, 12.0], [-16.0, 27.0, 52.0, -10.0, 21.0], [8.0, 38.0, -24.0, -14.0, 30.0], [10.0, -7.0, 26.0, 2.0, -58.0]], [[78.0, -20.0, -57.0, 24.0, -30.0], [-40.0, 74.0, -5.0, -73.0, 48.0], [-48.0, 18.0, -7.0, -2.0, -4.0], [38.0, 12.0, 12.0, -1.0, -49.0]], [[-25.0, 23.0, 20.0, 2.0, 13.0], [27.0, -46.0, 36.0, 77.0, 6.0], [59.0, 7.0, -16.0, 6.0, 43.0], [-17.0, -24.0, -27.0, 28.0, 24.0]], [[-43.0, -1.0, -9.0, 21.0, -52.0], [65.0, -26.0, -6.0, -7.0, 25.0], [21.0, -70.0, 11.0, 20.0, -2.0], [57.0, -17.0, -62.0, 29.0, 0.0]], [[41.0, -21.0, -47.0, -15.0, -52.0], [-13.0, 38.0, -6.0, -1.0, 99.0], [9.0, 2.0, -16.0, 29.0, 42.0], [41.0, -9.0, -63.0, 41.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[73.0, -27.0, 47.0, -30.0, -20.0], [-58.0, 19.0, 16.0, -14.0, 16.0], [5.0, 15.0, 53.0, 4.0, -49.0], [0.0, 23.0, -19.0, -43.0, -82.0]], [[7.0, 14.0, -25.0, 10.0, -30.0], [4.0, 8.0, 29.0, 11.0, 13.0], [25.0, 26.0, 0.0, 32.0, -25.0], [-44.0, -6.0, -10.0, 34.0, 11.0]], [[3.0, 52.0, 2.0, 3.0, 14.0], [-2.0, 23.0, 0.0, -31.0, -27.0], [18.0, 24.0, -40.0, 16.0, -35.0], [-43.0, -21.0, -23.0, 9.0, 24.0]], [[-33.0, 43.0, 40.0, 15.0, 34.0], [16.0, 28.0, -7.0, -54.0, -40.0], [20.0, -11.0, 1.0, -48.0, -37.0], [-43.0, -40.0, 4.0, 4.0, 45.0]], [[25.0, -54.0, -37.0, 15.0, -135.0], [-54.0, -55.0, 27.0, 104.0, 64.0], [91.0, 73.0, -38.0, 113.0, 75.0], [-31.0, -3.0, -72.0, -29.0, -54.0]]], [[[15.0, 3.0, -10.0, -7.0, -4.0], [25.0, -13.0, 14.0, -3.0, 25.0], [-8.0, 34.0, -15.0, 3.0, -28.0], [20.0, -5.0, -51.0, -25.0, -17.0]], [[14.0, -41.0, 0.0, 18.0, -85.0], [-32.0, 4.0, 49.0, 43.0, 38.0], [62.0, 11.0, 73.0, 28.0, -8.0], [-66.0, 1.0, 17.0, 35.0, -21.0]], [[5.0, 27.0, -9.0, 0.0, -19.0], [12.0, -34.0, -8.0, 4.0, 13.0], [31.0, 75.0, -101.0, 40.0, 7.0], [10.0, -35.0, -115.0, -69.0, -12.0]], [[-88.0, 46.0, 13.0, 40.0, 31.0], [95.0, -7.0, 4.0, -44.0, -14.0], [23.0, 6.0, -22.0, -76.0, -29.0], [-25.0, -75.0, -34.0, 1.0, 88.0]], [[-6.0, -17.0, 13.0, -9.0, 28.0], [-44.0, 8.0, -42.0, 7.0, -37.0], [-25.0, -59.0, 7.0, -4.0, 62.0], [23.0, 30.0, 74.0, 10.0, 0.0]]], [[[32.0, -35.0, 31.0, -23.0, 11.0], [-35.0, 6.0, -11.0, -4.0, 1.0], [-27.0, -28.0, 45.0, -19.0, 3.0], [36.0, 31.0, 27.0, -25.0, -48.0]], [[68.0, -21.0, 20.0, -40.0, 17.0], [-48.0, 16.0, -7.0, -11.0, 2.0], [-42.0, -9.0, 29.0, 8.0, -22.0], [40.0, 47.0, 11.0, -27.0, -69.0]], [[7.0, -63.0, 42.0, -17.0, -8.0], [-46.0, -40.0, -45.0, 28.0, 6.0], [-6.0, -21.0, -5.0, -12.0, 85.0], [75.0, 16.0, -10.0, -88.0, -61.0]], [[53.0, 23.0, 26.0, -18.0, 3.0], [21.0, 26.0, 46.0, -52.0, 26.0], [-2.0, 51.0, 35.0, -22.0, -126.0], [-17.0, -12.0, -67.0, -26.0, -38.0]], [[-22.0, -53.0, 43.0, 6.0, 4.0], [6.0, 4.0, 6.0, -4.0, 9.0], [-11.0, -60.0, 105.0, -77.0, 2.0], [16.0, 7.0, 64.0, 5.0, -7.0]]], [[[42.0, -50.0, 22.0, -10.0, -55.0], [-80.0, 1.0, 4.0, 38.0, 13.0], [32.0, -6.0, 42.0, 38.0, 32.0], [-16.0, 28.0, 22.0, -12.0, -60.0]], [[46.0, -39.0, 56.0, -20.0, -19.0], [-48.0, 8.0, 7.0, -9.0, 14.0], [10.0, -1.0, 59.0, -18.0, -24.0], [8.0, 13.0, -9.0, -49.0, -69.0]], [[13.0, 3.0, -49.0, -3.0, -26.0], [-16.0, -23.0, -8.0, 43.0, 10.0], [5.0, 30.0, -67.0, 73.0, 47.0], [9.0, 14.0, -25.0, 2.0, -7.0]], [[60.0, 21.0, -22.0, -37.0, 36.0], [-6.0, 15.0, -1.0, -20.0, 1.0], [-58.0, 18.0, -23.0, 27.0, -44.0], [39.0, 38.0, -17.0, -6.0, -36.0]], [[88.0, -8.0, -1.0, -43.0, -22.0], [-45.0, -22.0, -4.0, 11.0, 31.0], [-9.0, 73.0, -62.0, 67.0, -13.0], [50.0, 24.0, -104.0, -90.0, -97.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_float_rank0_offset0(self):
      arg0=numpy.array([[[-6.0, -7.0], [-7.0, -1.0]], [[1.0, -3.0], [7.0, -6.0]], [[-2.0, 7.0], [6.0, 7.0]], [[-3.0, -3.0], [-2.0, 4.0]], [[-3.0, 5.0], [-3.0, -7.0]], [[-7.0, -1.0], [3.0, 2.0]]])
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-0.0, -0.0], [-0.0, -0.0]], [[0.0, -0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[-0.0, -0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank0_offset0(self):
      arg0=numpy.array([[[3.0, -4.0], [-1.0, 6.0]], [[-4.0, -4.0], [-5.0, 7.0]], [[2.0, 4.0], [0.0, 0.0]], [[-4.0, -1.0], [6.0, 7.0]], [[6.0, 5.0], [1.0, 0.0]], [[2.0, -1.0], [6.0, 4.0]]])
      arg1=numpy.array(0.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[0.0, -0.0], [-0.0, 0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[-0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, -0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank1_offset1(self):
      arg0=numpy.array([[[[6.0, -6.0], [-1.0, -5.0]], [[0.0, 4.0], [3.0, 5.0]], [[5.0, -3.0], [6.0, 6.0]], [[7.0, -5.0], [-1.0, 4.0]], [[0.0, -4.0], [2.0, 0.0]], [[6.0, -2.0], [2.0, -4.0]]], [[[6.0, -4.0], [-5.0, 0.0]], [[-2.0, 0.0], [6.0, 4.0]], [[5.0, -5.0], [-6.0, 0.0]], [[4.0, -1.0], [5.0, 3.0]], [[7.0, 1.0], [3.0, -6.0]], [[-7.0, -6.0], [-1.0, -4.0]]], [[[5.0, 3.0], [-2.0, 6.0]], [[-2.0, 3.0], [-7.0, 7.0]], [[2.0, 7.0], [2.0, 3.0]], [[-6.0, 7.0], [3.0, -6.0]], [[0.0, 2.0], [0.0, -3.0]], [[-3.0, 3.0], [0.0, -4.0]]]])
      arg1=numpy.array([-7.0, 6.0, -7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-41.0, -3.0], [-9.0, -7.0]], [[2.0, -49.0], [64.0, -60.0]], [[-19.0, -58.0], [-92.0, -63.0]], [[17.0, -20.0], [16.0, 32.0]], [[42.0, 20.0], [4.0, -15.0]], [[-63.0, -43.0], [-20.0, 32.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_array_rank1_offset0(self):
      arg0=numpy.array([[[-7.0, 1.0], [5.0, -6.0]], [[1.0, -6.0], [2.0, 7.0]], [[1.0, 7.0], [-3.0, -4.0]], [[2.0, 1.0], [-3.0, 0.0]], [[1.0, 2.0], [-1.0, 0.0]], [[-3.0, 1.0], [0.0, 0.0]]])
      arg1=numpy.array([4.0, 5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-28.0, -35.0], [4.0, 5.0]], [[20.0, 25.0], [-24.0, -30.0]]], [[[4.0, 5.0], [-24.0, -30.0]], [[8.0, 10.0], [28.0, 35.0]]], [[[4.0, 5.0], [28.0, 35.0]], [[-12.0, -15.0], [-16.0, -20.0]]], [[[8.0, 10.0], [4.0, 5.0]], [[-12.0, -15.0], [0.0, 0.0]]], [[[4.0, 5.0], [8.0, 10.0]], [[-4.0, -5.0], [0.0, 0.0]]], [[[-12.0, -15.0], [4.0, 5.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank2_offset1(self):
      arg0=numpy.array([[[[-6.0, -6.0], [-6.0, 7.0]], [[5.0, 7.0], [5.0, 4.0]], [[-7.0, 5.0], [-1.0, -2.0]], [[2.0, -2.0], [-7.0, -5.0]], [[4.0, 5.0], [4.0, -7.0]], [[4.0, -2.0], [7.0, -1.0]]], [[[4.0, -4.0], [7.0, 4.0]], [[6.0, 0.0], [-1.0, -2.0]], [[6.0, 4.0], [7.0, -5.0]], [[2.0, -4.0], [3.0, -7.0]], [[-7.0, 0.0], [-3.0, 1.0]], [[-6.0, 0.0], [-4.0, 7.0]]], [[[-5.0, -7.0], [-5.0, -6.0]], [[4.0, 6.0], [0.0, -6.0]], [[5.0, -1.0], [-7.0, -3.0]], [[-1.0, 0.0], [1.0, 0.0]], [[3.0, 1.0], [1.0, -7.0]], [[6.0, 6.0], [7.0, 0.0]]]])
      arg1=numpy.array([[0.0, -3.0], [-1.0, -7.0], [1.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[-9.0, 10.0], [-3.0, 74.0]], [[-12.0, -11.0], [-10.0, -25.0]]], [[[-2.0, -73.0], [6.0, -45.0]], [[1.0, -8.0], [-4.0, 26.0]]], [[[-1.0, -41.0], [-5.0, -39.0]], [[-14.0, -18.0], [2.0, 53.0]]], [[[-3.0, -16.0], [4.0, 34.0]], [[-2.0, -4.0], [7.0, 64.0]]], [[[10.0, 25.0], [1.0, -19.0]], [[4.0, 5.0], [-8.0, 42.0]]], [[[12.0, 6.0], [6.0, -18.0]], [[11.0, -21.0], [-7.0, -46.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[6.0, 4.0], [0.0, 2.0]], [[3.0, 0.0], [1.0, 0.0]], [[3.0, 6.0], [-6.0, 6.0]], [[-6.0, 4.0], [-1.0, 0.0]], [[7.0, -7.0], [2.0, 0.0]], [[-4.0, 1.0], [4.0, 0.0]]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[18.0, 12.0], [0.0, 6.0]], [[9.0, 0.0], [3.0, 0.0]], [[9.0, 18.0], [-18.0, 18.0]], [[-18.0, 12.0], [-3.0, 0.0]], [[21.0, -21.0], [6.0, 0.0]], [[-12.0, 3.0], [12.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[[1.0, -5.0], [6.0, 0.0]], [[-5.0, -7.0], [-6.0, 1.0]], [[7.0, -5.0], [-3.0, -2.0]], [[1.0, -4.0], [-3.0, 0.0]], [[-3.0, -7.0], [6.0, -2.0]], [[5.0, -5.0], [4.0, 5.0]]], [[[4.0, 0.0], [6.0, -1.0]], [[2.0, 0.0], [-2.0, 4.0]], [[-3.0, -3.0], [4.0, 6.0]], [[-4.0, 3.0], [-6.0, -1.0]], [[-3.0, 0.0], [5.0, 4.0]], [[4.0, -2.0], [-1.0, 1.0]]], [[[5.0, 0.0], [0.0, -7.0]], [[1.0, 3.0], [-7.0, -1.0]], [[7.0, 4.0], [0.0, 0.0]], [[7.0, -3.0], [7.0, 5.0]], [[-3.0, 5.0], [0.0, 0.0]], [[5.0, 0.0], [-5.0, 0.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([1.0, 6.0, 5.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[50.0, -5.0], [42.0, -41.0]], [[12.0, 8.0], [-53.0, 20.0]], [[24.0, -3.0], [21.0, 34.0]], [[12.0, -1.0], [-4.0, 19.0]], [[-36.0, 18.0], [36.0, 22.0]], [[54.0, -17.0], [-27.0, 11.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_Symbol_rank1_offset0(self):
      arg0=numpy.array([[[-6.0, 2.0], [0.0, -4.0]], [[3.0, 3.0], [-7.0, 0.0]], [[-2.0, 0.0], [-7.0, -5.0]], [[-5.0, 7.0], [3.0, 6.0]], [[1.0, 7.0], [3.0, 1.0]], [[2.0, 2.0], [6.0, 0.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-5.0, 3.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[30.0, -18.0], [-10.0, 6.0]], [[0.0, 0.0], [20.0, -12.0]]], [[[-15.0, 9.0], [-15.0, 9.0]], [[35.0, -21.0], [0.0, 0.0]]], [[[10.0, -6.0], [0.0, 0.0]], [[35.0, -21.0], [25.0, -15.0]]], [[[25.0, -15.0], [-35.0, 21.0]], [[-15.0, 9.0], [-30.0, 18.0]]], [[[-5.0, 3.0], [-35.0, 21.0]], [[-15.0, 9.0], [-5.0, 3.0]]], [[[-10.0, 6.0], [-10.0, 6.0]], [[-30.0, 18.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[[2.0, -3.0], [-7.0, 6.0]], [[1.0, 0.0], [4.0, -1.0]], [[-6.0, -5.0], [-5.0, -6.0]], [[0.0, 0.0], [5.0, 0.0]], [[-3.0, 2.0], [-4.0, 5.0]], [[2.0, -3.0], [5.0, 0.0]]], [[[5.0, -5.0], [-1.0, 0.0]], [[-7.0, 4.0], [-4.0, 1.0]], [[-1.0, -4.0], [-4.0, 0.0]], [[4.0, 2.0], [-5.0, 5.0]], [[3.0, -4.0], [0.0, -7.0]], [[-7.0, -2.0], [-1.0, -6.0]]], [[[4.0, 3.0], [-3.0, 7.0]], [[-7.0, -3.0], [0.0, 3.0]], [[0.0, -2.0], [4.0, -6.0]], [[-3.0, -7.0], [-5.0, -2.0]], [[0.0, 7.0], [-6.0, -6.0]], [[-2.0, 0.0], [-3.0, 5.0]]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[0.0, 1.0], [-7.0, 2.0], [-5.0, 4.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-55.0, 28.0], [20.0, -1.0]], [[22.0, -21.0], [-35.0, 34.0]]], [[[84.0, -41.0], [-13.0, -4.0]], [[28.0, -4.0], [-22.0, 13.0]]], [[[7.0, -8.0], [38.0, -21.0]], [[8.0, 3.0], [30.0, -30.0]]], [[[-13.0, -4.0], [21.0, -24.0]], [[60.0, -25.0], [-25.0, 2.0]]], [[[-21.0, 3.0], [-7.0, 22.0]], [[30.0, -28.0], [79.0, -33.0]]], [[[59.0, -20.0], [14.0, -7.0]], [[22.0, -9.0], [17.0, 8.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank0_offset0(self):
      arg0=numpy.array([[[-6.0, 7.0], [-1.0, -1.0]], [[-4.0, 3.0], [0.0, 3.0]], [[-4.0, 0.0], [1.0, 6.0]], [[0.0, 4.0], [4.0, 0.0]], [[5.0, -7.0], [4.0, 5.0]], [[-4.0, -2.0], [-4.0, 5.0]]])
      arg1=Data(5.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-30.0, 35.0], [-5.0, -5.0]], [[-20.0, 15.0], [0.0, 15.0]], [[-20.0, 0.0], [5.0, 30.0]], [[0.0, 20.0], [20.0, 0.0]], [[25.0, -35.0], [20.0, 25.0]], [[-20.0, -10.0], [-20.0, 25.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank1_offset1(self):
      arg0=numpy.array([[[[2.0, -4.0], [4.0, 2.0]], [[5.0, 0.0], [5.0, 5.0]], [[-4.0, 4.0], [-1.0, -4.0]], [[-1.0, 2.0], [6.0, 7.0]], [[-4.0, -5.0], [3.0, -4.0]], [[6.0, 1.0], [-3.0, 7.0]]], [[[-1.0, 1.0], [-2.0, -1.0]], [[3.0, 5.0], [-7.0, 0.0]], [[-5.0, -6.0], [-6.0, 5.0]], [[2.0, -1.0], [7.0, 0.0]], [[1.0, 3.0], [-4.0, -1.0]], [[-3.0, -7.0], [7.0, -5.0]]], [[[3.0, 0.0], [-4.0, 6.0]], [[0.0, -1.0], [-3.0, 4.0]], [[6.0, 5.0], [5.0, 2.0]], [[5.0, 5.0], [0.0, -5.0]], [[7.0, -2.0], [6.0, 0.0]], [[-1.0, -6.0], [4.0, 2.0]]]])
      arg1=Data(numpy.array([7.0, -3.0, 3.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[26.0, -31.0], [22.0, 35.0]], [[26.0, -18.0], [47.0, 47.0]], [[5.0, 61.0], [26.0, -37.0]], [[2.0, 32.0], [21.0, 34.0]], [[-10.0, -50.0], [51.0, -25.0]], [[48.0, 10.0], [-30.0, 70.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_constData_rank1_offset0(self):
      arg0=numpy.array([[[1.0, -2.0], [-4.0, 2.0]], [[6.0, 2.0], [-5.0, 7.0]], [[5.0, 6.0], [4.0, 2.0]], [[5.0, -7.0], [-7.0, -1.0]], [[1.0, 6.0], [7.0, -6.0]], [[-6.0, 0.0], [7.0, 0.0]]])
      arg1=Data(numpy.array([2.0, -5.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[2.0, -5.0], [-4.0, 10.0]], [[-8.0, 20.0], [4.0, -10.0]]], [[[12.0, -30.0], [4.0, -10.0]], [[-10.0, 25.0], [14.0, -35.0]]], [[[10.0, -25.0], [12.0, -30.0]], [[8.0, -20.0], [4.0, -10.0]]], [[[10.0, -25.0], [-14.0, 35.0]], [[-14.0, 35.0], [-2.0, 5.0]]], [[[2.0, -5.0], [12.0, -30.0]], [[14.0, -35.0], [-12.0, 30.0]]], [[[-12.0, 30.0], [0.0, 0.0]], [[14.0, -35.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank2_offset1(self):
      arg0=numpy.array([[[[-6.0, 4.0], [5.0, -5.0]], [[-3.0, 5.0], [3.0, 1.0]], [[3.0, 4.0], [-7.0, 3.0]], [[-2.0, -1.0], [3.0, 4.0]], [[7.0, 1.0], [-7.0, 1.0]], [[-4.0, 3.0], [-2.0, -3.0]]], [[[-5.0, 5.0], [0.0, 4.0]], [[-2.0, 0.0], [-2.0, 7.0]], [[-6.0, 1.0], [-3.0, -3.0]], [[5.0, -3.0], [0.0, -4.0]], [[-2.0, 7.0], [3.0, -6.0]], [[2.0, -6.0], [-7.0, 1.0]]], [[[7.0, -1.0], [0.0, 1.0]], [[0.0, 5.0], [2.0, -5.0]], [[2.0, 0.0], [0.0, 0.0]], [[-7.0, 2.0], [2.0, 0.0]], [[-4.0, -1.0], [-2.0, -6.0]], [[3.0, -5.0], [1.0, -5.0]]]])
      arg1=Data(numpy.array([[-2.0, 5.0], [-1.0, 3.0], [-5.0, -4.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-18.0, -73.0], [-8.0, 39.0]], [[-10.0, 25.0], [1.0, -17.0]]], [[[8.0, -21.0], [-35.0, 5.0]], [[-14.0, 1.0], [16.0, 46.0]]], [[[-10.0, -11.0], [-9.0, 23.0]], [[17.0, -44.0], [-3.0, 6.0]]], [[[34.0, 33.0], [-5.0, -22.0]], [[-16.0, 7.0], [-4.0, 8.0]]], [[[8.0, 45.0], [-4.0, 30.0]], [[21.0, -18.0], [34.0, 11.0]]], [[[-9.0, -26.0], [25.0, 17.0]], [[6.0, -35.0], [30.0, 8.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[0.0, -3.0], [1.0, 4.0]], [[-7.0, -2.0], [2.0, 1.0]], [[0.0, 0.0], [3.0, 5.0]], [[2.0, 1.0], [-4.0, 3.0]], [[0.0, 5.0], [-6.0, -1.0]], [[1.0, -2.0], [-3.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(7.0)+(1-msk_arg1)*(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -21.0], [7.0, 28.0]], [[-49.0, -14.0], [14.0, 7.0]], [[0.0, 0.0], [21.0, 35.0]], [[14.0, 7.0], [-28.0, 21.0]], [[0.0, 35.0], [-42.0, -7.0]], [[7.0, -14.0], [-21.0, -42.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, 9.0], [-3.0, -12.0]], [[21.0, 6.0], [-6.0, -3.0]], [[-0.0, -0.0], [-9.0, -15.0]], [[-6.0, -3.0], [12.0, -9.0]], [[-0.0, -15.0], [18.0, 3.0]], [[-3.0, 6.0], [9.0, 18.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[[5.0, 1.0], [0.0, 3.0]], [[0.0, 7.0], [2.0, -3.0]], [[0.0, 5.0], [-3.0, 1.0]], [[-2.0, 1.0], [-7.0, -5.0]], [[6.0, -2.0], [-7.0, 2.0]], [[2.0, -4.0], [-3.0, -7.0]]], [[[-3.0, -4.0], [-5.0, -5.0]], [[-4.0, -2.0], [-6.0, 1.0]], [[-6.0, -3.0], [0.0, 7.0]], [[3.0, 2.0], [3.0, -4.0]], [[1.0, 2.0], [-4.0, 7.0]], [[0.0, -2.0], [-3.0, -3.0]]], [[[1.0, 6.0], [-6.0, 0.0]], [[-2.0, -5.0], [-7.0, -6.0]], [[0.0, -7.0], [-2.0, -2.0]], [[-5.0, 1.0], [0.0, 6.0]], [[-4.0, -7.0], [-1.0, 3.0]], [[-2.0, -3.0], [-5.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, -5.0, 1.0])+(1.-msk_arg1)*numpy.array([-2.0, -4.0, 7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[11.0, 25.0], [19.0, 22.0]], [[18.0, -2.0], [21.0, -8.0]], [[30.0, 3.0], [1.0, -38.0]], [[-18.0, -10.0], [-8.0, 31.0]], [[-15.0, -15.0], [26.0, -34.0]], [[-4.0, 11.0], [13.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[9.0, 56.0], [-22.0, 14.0]], [[2.0, -41.0], [-29.0, -40.0]], [[24.0, -47.0], [-8.0, -44.0]], [[-43.0, -3.0], [2.0, 68.0]], [[-44.0, -53.0], [23.0, -11.0]], [[-18.0, -5.0], [-17.0, 12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank3_expandedData_rank1_offset0(self):
      arg0=numpy.array([[[1.0, 0.0], [5.0, 0.0]], [[-6.0, -5.0], [4.0, 1.0]], [[5.0, -7.0], [-7.0, -2.0]], [[-7.0, -6.0], [7.0, -7.0]], [[6.0, -1.0], [-4.0, -1.0]], [[-5.0, -6.0], [-3.0, 3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -2.0])+(1.-msk_arg1)*numpy.array([-3.0, 2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, -2.0], [0.0, 0.0]], [[0.0, -10.0], [0.0, 0.0]]], [[[0.0, 12.0], [0.0, 10.0]], [[0.0, -8.0], [0.0, -2.0]]], [[[0.0, -10.0], [0.0, 14.0]], [[0.0, 14.0], [0.0, 4.0]]], [[[0.0, 14.0], [0.0, 12.0]], [[0.0, -14.0], [0.0, 14.0]]], [[[0.0, -12.0], [0.0, 2.0]], [[0.0, 8.0], [0.0, 2.0]]], [[[0.0, 10.0], [0.0, 12.0]], [[0.0, 6.0], [0.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, 2.0], [0.0, 0.0]], [[-15.0, 10.0], [0.0, 0.0]]], [[[18.0, -12.0], [15.0, -10.0]], [[-12.0, 8.0], [-3.0, 2.0]]], [[[-15.0, 10.0], [21.0, -14.0]], [[21.0, -14.0], [6.0, -4.0]]], [[[21.0, -14.0], [18.0, -12.0]], [[-21.0, 14.0], [21.0, -14.0]]], [[[-18.0, 12.0], [3.0, -2.0]], [[12.0, -8.0], [3.0, -2.0]]], [[[15.0, -10.0], [18.0, -12.0]], [[9.0, -6.0], [-9.0, 6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[[-5.0, -1.0], [-6.0, 5.0]], [[5.0, 5.0], [7.0, -2.0]], [[4.0, -2.0], [-3.0, 0.0]], [[-6.0, 3.0], [5.0, 1.0]], [[-3.0, 3.0], [0.0, 1.0]], [[6.0, 3.0], [-2.0, -6.0]]], [[[3.0, -1.0], [2.0, 1.0]], [[4.0, 1.0], [7.0, 5.0]], [[7.0, 3.0], [0.0, 0.0]], [[7.0, -2.0], [-5.0, -1.0]], [[0.0, 7.0], [-3.0, 7.0]], [[6.0, 2.0], [-6.0, 1.0]]], [[[6.0, 6.0], [-6.0, -3.0]], [[2.0, 2.0], [-6.0, -3.0]], [[0.0, 7.0], [-5.0, 4.0]], [[4.0, -4.0], [7.0, 6.0]], [[-6.0, -3.0], [3.0, 0.0]], [[4.0, -4.0], [-7.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 0.0], [-3.0, 0.0], [-7.0, 1.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 1.0], [-7.0, -7.0], [-6.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-16.0, 6.0], [-32.0, 6.0]], [[78.0, -6.0], [-17.0, -3.0]]], [[[-61.0, 2.0], [-52.0, 2.0]], [[-28.0, -6.0], [20.0, -3.0]]], [[[-49.0, 0.0], [-44.0, 7.0]], [[56.0, -5.0], [-28.0, 4.0]]], [[[-7.0, 4.0], [13.0, -4.0]], [[-69.0, 7.0], [-46.0, 6.0]]], [[[63.0, -6.0], [-21.0, -3.0]], [[-12.0, 3.0], [-28.0, 0.0]]], [[[-88.0, 4.0], [1.0, -4.0]], [[81.0, -7.0], [4.0, 5.0]]]])+(1.-msk_ref)*numpy.array([[[[-37.0, -50.0], [-25.0, -18.0]], [[46.0, 4.0], [-9.0, 10.0]]], [[[-60.0, -31.0], [-39.0, -10.0]], [[-41.0, -18.0], [-9.0, -25.0]]], [[[-65.0, -45.0], [-55.0, -51.0]], [[42.0, 17.0], [-24.0, -16.0]]], [[[-49.0, -71.0], [26.0, 33.0]], [[-27.0, 12.0], [-33.0, -16.0]]], [[[48.0, 21.0], [-43.0, -34.0]], [[3.0, 9.0], [-53.0, -48.0]]], [[[-90.0, -52.0], [-2.0, 5.0]], [[92.0, 68.0], [-13.0, -33.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_float_rank0_offset0(self):
      arg0=numpy.array([[[[5.0, 6.0, 3.0, 5.0], [0.0, 2.0, 3.0, -4.0], [-1.0, 6.0, 6.0, -7.0]], [[-7.0, 0.0, 7.0, -7.0], [4.0, 7.0, -5.0, 0.0], [-4.0, -6.0, -2.0, -3.0]]], [[[-6.0, -3.0, -4.0, 5.0], [1.0, 0.0, -7.0, 6.0], [2.0, -1.0, -1.0, 7.0]], [[2.0, 0.0, 4.0, 3.0], [-4.0, -6.0, 0.0, 0.0], [-2.0, 7.0, -2.0, -3.0]]], [[[-3.0, -5.0, -6.0, -1.0], [-1.0, 0.0, -6.0, 1.0], [5.0, -3.0, 0.0, -6.0]], [[-1.0, 1.0, 7.0, -7.0], [-6.0, 7.0, 5.0, 5.0], [3.0, 7.0, 3.0, -7.0]]]])
      arg1=7.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[35.0, 42.0, 21.0, 35.0], [0.0, 14.0, 21.0, -28.0], [-7.0, 42.0, 42.0, -49.0]], [[-49.0, 0.0, 49.0, -49.0], [28.0, 49.0, -35.0, 0.0], [-28.0, -42.0, -14.0, -21.0]]], [[[-42.0, -21.0, -28.0, 35.0], [7.0, 0.0, -49.0, 42.0], [14.0, -7.0, -7.0, 49.0]], [[14.0, 0.0, 28.0, 21.0], [-28.0, -42.0, 0.0, 0.0], [-14.0, 49.0, -14.0, -21.0]]], [[[-21.0, -35.0, -42.0, -7.0], [-7.0, 0.0, -42.0, 7.0], [35.0, -21.0, 0.0, -42.0]], [[-7.0, 7.0, 49.0, -49.0], [-42.0, 49.0, 35.0, 35.0], [21.0, 49.0, 21.0, -49.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_array_rank0_offset0(self):
      arg0=numpy.array([[[[4.0, -5.0, -7.0, 5.0], [0.0, -1.0, 0.0, 2.0], [2.0, -4.0, 0.0, 2.0]], [[-1.0, 3.0, -4.0, -4.0], [0.0, 6.0, -7.0, 5.0], [0.0, 7.0, -1.0, -4.0]]], [[[5.0, 5.0, 7.0, -7.0], [0.0, 7.0, 0.0, -4.0], [0.0, 2.0, 6.0, 5.0]], [[0.0, 1.0, -4.0, 6.0], [0.0, -3.0, -6.0, 4.0], [-4.0, 6.0, -4.0, 2.0]]], [[[-6.0, 1.0, 0.0, -3.0], [1.0, -7.0, 6.0, -4.0], [1.0, -1.0, -1.0, 0.0]], [[-7.0, 6.0, 0.0, 4.0], [-7.0, -2.0, -4.0, -6.0], [-3.0, 4.0, 2.0, -6.0]]]])
      arg1=numpy.array(-4.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-16.0, 20.0, 28.0, -20.0], [-0.0, 4.0, -0.0, -8.0], [-8.0, 16.0, -0.0, -8.0]], [[4.0, -12.0, 16.0, 16.0], [-0.0, -24.0, 28.0, -20.0], [-0.0, -28.0, 4.0, 16.0]]], [[[-20.0, -20.0, -28.0, 28.0], [-0.0, -28.0, -0.0, 16.0], [-0.0, -8.0, -24.0, -20.0]], [[-0.0, -4.0, 16.0, -24.0], [-0.0, 12.0, 24.0, -16.0], [16.0, -24.0, 16.0, -8.0]]], [[[24.0, -4.0, -0.0, 12.0], [-4.0, 28.0, -24.0, 16.0], [-4.0, 4.0, 4.0, -0.0]], [[28.0, -24.0, -0.0, -16.0], [28.0, 8.0, 16.0, 24.0], [12.0, -16.0, -8.0, 24.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[[7.0, 2.0, -7.0, 2.0], [7.0, 5.0, 4.0, -7.0], [-6.0, 0.0, 7.0, 2.0]], [[1.0, -3.0, -2.0, 7.0], [0.0, -6.0, 7.0, -2.0], [-6.0, 4.0, -5.0, 3.0]]], [[[2.0, -4.0, -6.0, 3.0], [-4.0, -4.0, -7.0, 4.0], [-5.0, -1.0, 5.0, 3.0]], [[5.0, 5.0, -2.0, -7.0], [0.0, -6.0, 2.0, -5.0], [7.0, 4.0, 6.0, 0.0]]], [[[6.0, 2.0, 5.0, -6.0], [7.0, 7.0, 6.0, 0.0], [-2.0, 0.0, 2.0, -1.0]], [[-5.0, 3.0, 5.0, 4.0], [-2.0, 2.0, -3.0, 2.0], [1.0, -5.0, -6.0, -2.0]]]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-4.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-28.0, -8.0, 28.0, -8.0], [-28.0, -20.0, -16.0, 28.0], [24.0, -0.0, -28.0, -8.0]], [[-4.0, 12.0, 8.0, -28.0], [-0.0, 24.0, -28.0, 8.0], [24.0, -16.0, 20.0, -12.0]]], [[[-8.0, 16.0, 24.0, -12.0], [16.0, 16.0, 28.0, -16.0], [20.0, 4.0, -20.0, -12.0]], [[-20.0, -20.0, 8.0, 28.0], [-0.0, 24.0, -8.0, 20.0], [-28.0, -16.0, -24.0, -0.0]]], [[[-24.0, -8.0, -20.0, 24.0], [-28.0, -28.0, -24.0, -0.0], [8.0, -0.0, -8.0, 4.0]], [[20.0, -12.0, -20.0, -16.0], [8.0, -8.0, 12.0, -8.0], [-4.0, 20.0, 24.0, 8.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_constData_rank0_offset0(self):
      arg0=numpy.array([[[[-4.0, -6.0, 1.0, 3.0], [0.0, -7.0, -3.0, 6.0], [-5.0, 2.0, 1.0, 6.0]], [[-7.0, -7.0, 4.0, 2.0], [3.0, 1.0, 3.0, 1.0], [0.0, 0.0, 5.0, -2.0]]], [[[-1.0, 1.0, -1.0, -1.0], [7.0, 0.0, -4.0, -1.0], [5.0, 0.0, 7.0, -5.0]], [[4.0, 3.0, -7.0, 7.0], [2.0, 0.0, 4.0, 3.0], [-1.0, 3.0, -2.0, -5.0]]], [[[-5.0, 3.0, -5.0, -3.0], [0.0, -1.0, 3.0, 6.0], [-5.0, -3.0, -5.0, 1.0]], [[-1.0, 0.0, 1.0, -5.0], [1.0, -4.0, -5.0, -4.0], [-5.0, 4.0, 0.0, 3.0]]]])
      arg1=Data(7.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-28.0, -42.0, 7.0, 21.0], [0.0, -49.0, -21.0, 42.0], [-35.0, 14.0, 7.0, 42.0]], [[-49.0, -49.0, 28.0, 14.0], [21.0, 7.0, 21.0, 7.0], [0.0, 0.0, 35.0, -14.0]]], [[[-7.0, 7.0, -7.0, -7.0], [49.0, 0.0, -28.0, -7.0], [35.0, 0.0, 49.0, -35.0]], [[28.0, 21.0, -49.0, 49.0], [14.0, 0.0, 28.0, 21.0], [-7.0, 21.0, -14.0, -35.0]]], [[[-35.0, 21.0, -35.0, -21.0], [0.0, -7.0, 21.0, 42.0], [-35.0, -21.0, -35.0, 7.0]], [[-7.0, 0.0, 7.0, -35.0], [7.0, -28.0, -35.0, -28.0], [-35.0, 28.0, 0.0, 21.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_array_rank4_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[[3.0, 5.0, 6.0, 1.0], [3.0, 0.0, 6.0, -5.0], [4.0, 6.0, 0.0, -4.0]], [[-7.0, 0.0, 0.0, -5.0], [6.0, 5.0, -7.0, -2.0], [-5.0, -6.0, -5.0, -6.0]]], [[[0.0, 6.0, 0.0, 3.0], [-4.0, 2.0, 3.0, 0.0], [-4.0, -3.0, 3.0, 5.0]], [[4.0, 7.0, -1.0, -5.0], [0.0, 7.0, -5.0, -5.0], [-3.0, -1.0, 6.0, 6.0]]], [[[-7.0, 4.0, -2.0, 2.0], [3.0, 5.0, 7.0, -4.0], [5.0, 3.0, -5.0, -2.0]], [[4.0, -2.0, 3.0, 0.0], [-1.0, -5.0, -3.0, -4.0], [-6.0, 4.0, 7.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(2.0)+(1-msk_arg1)*(-5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 10.0, 12.0, 2.0], [6.0, 0.0, 12.0, -10.0], [8.0, 12.0, 0.0, -8.0]], [[-14.0, 0.0, 0.0, -10.0], [12.0, 10.0, -14.0, -4.0], [-10.0, -12.0, -10.0, -12.0]]], [[[0.0, 12.0, 0.0, 6.0], [-8.0, 4.0, 6.0, 0.0], [-8.0, -6.0, 6.0, 10.0]], [[8.0, 14.0, -2.0, -10.0], [0.0, 14.0, -10.0, -10.0], [-6.0, -2.0, 12.0, 12.0]]], [[[-14.0, 8.0, -4.0, 4.0], [6.0, 10.0, 14.0, -8.0], [10.0, 6.0, -10.0, -4.0]], [[8.0, -4.0, 6.0, 0.0], [-2.0, -10.0, -6.0, -8.0], [-12.0, 8.0, 14.0, -10.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, -25.0, -30.0, -5.0], [-15.0, -0.0, -30.0, 25.0], [-20.0, -30.0, -0.0, 20.0]], [[35.0, -0.0, -0.0, 25.0], [-30.0, -25.0, 35.0, 10.0], [25.0, 30.0, 25.0, 30.0]]], [[[-0.0, -30.0, -0.0, -15.0], [20.0, -10.0, -15.0, -0.0], [20.0, 15.0, -15.0, -25.0]], [[-20.0, -35.0, 5.0, 25.0], [-0.0, -35.0, 25.0, 25.0], [15.0, 5.0, -30.0, -30.0]]], [[[35.0, -20.0, 10.0, -10.0], [-15.0, -25.0, -35.0, 20.0], [-25.0, -15.0, 25.0, 10.0]], [[-20.0, 10.0, -15.0, -0.0], [5.0, 25.0, 15.0, 20.0], [30.0, -20.0, -35.0, 25.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_float_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=-5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(2.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-10.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_array_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-49.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([0.0, 6.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([6.0, 2.0, 6.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(18.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[1.0, -6.0, -4.0], [-3.0, -3.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-2.0, -2.0, 1.0], [0.0, 6.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-36.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[-4.0, 0.0, -6.0], [0.0, 6.0, 5.0], [-7.0, -3.0, 3.0], [2.0, -2.0, -4.0]], [[3.0, -4.0, 2.0], [3.0, 7.0, 4.0], [-6.0, 4.0, 7.0], [-7.0, 5.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-2.0, -6.0, -7.0], [-3.0, 0.0, -6.0], [2.0, 0.0, -3.0], [1.0, -5.0, 2.0]], [[-5.0, -3.0, -3.0], [-2.0, 0.0, 2.0], [0.0, 4.0, -4.0], [7.0, 0.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-63.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=numpy.array([[[[1.0, -4.0], [-5.0, 0.0], [5.0, 5.0]], [[7.0, -2.0], [-3.0, 7.0], [0.0, -2.0]]], [[[-5.0, 5.0], [2.0, -3.0], [-2.0, -5.0]], [[-4.0, -1.0], [-4.0, -6.0], [3.0, 7.0]]], [[[-7.0, 4.0], [1.0, 4.0], [-6.0, 1.0]], [[2.0, -5.0], [-6.0, -4.0], [2.0, 2.0]]], [[[-5.0, 2.0], [-6.0, 1.0], [-6.0, -4.0]], [[-5.0, 7.0], [-6.0, 1.0], [0.0, -3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[0.0, 2.0], [-6.0, -5.0], [3.0, -6.0]], [[3.0, -1.0], [-4.0, 3.0], [4.0, -6.0]]], [[[5.0, 0.0], [0.0, 0.0], [-4.0, 5.0]], [[2.0, 3.0], [0.0, -5.0], [2.0, 0.0]]], [[[7.0, 5.0], [-6.0, 4.0], [2.0, -2.0]], [[0.0, -4.0], [5.0, 3.0], [3.0, 0.0]]], [[[-5.0, 4.0], [0.0, 6.0], [4.0, -2.0]], [[-4.0, -4.0], [3.0, 1.0], [4.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(4.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_array_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([-3.0, 5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([21.0, -35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[5.0, -2.0], [-4.0, 5.0], [7.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([4.0, 6.0, -2.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-18.0, 22.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[7.0, -1.0], [6.0, 4.0], [3.0, 3.0]], [[0.0, 5.0], [-3.0, 5.0], [7.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[0.0, 3.0, 2.0], [0.0, 3.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([8.0, 38.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[[6.0, 5.0], [0.0, 4.0], [0.0, 2.0]], [[0.0, -5.0], [6.0, 0.0], [-6.0, 6.0]], [[6.0, 5.0], [0.0, 7.0], [-4.0, -3.0]], [[0.0, 6.0], [-3.0, 6.0], [6.0, -5.0]]], [[[3.0, -3.0], [-1.0, 7.0], [0.0, 2.0]], [[-5.0, 1.0], [-4.0, 7.0], [-1.0, 6.0]], [[-6.0, -2.0], [0.0, 5.0], [5.0, 5.0]], [[-2.0, -4.0], [-3.0, -1.0], [3.0, 7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[7.0, -2.0, 0.0], [5.0, 4.0, -2.0], [0.0, 5.0, -4.0], [7.0, -3.0, 7.0]], [[1.0, 3.0, -2.0], [0.0, 3.0, 4.0], [-3.0, 7.0, 7.0], [-5.0, 3.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([183.0, 178.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_array_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-6.0, 7.0, -5.0, 3.0, 6.0], [7.0, 0.0, -3.0, -4.0, -6.0], [-6.0, 0.0, 6.0, -5.0, 2.0], [-4.0, 3.0, -5.0, 4.0, -3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(6.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-36.0, 42.0, -30.0, 18.0, 36.0], [42.0, 0.0, -18.0, -24.0, -36.0], [-36.0, 0.0, 36.0, -30.0, 12.0], [-24.0, 18.0, -30.0, 24.0, -18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[0.0, -3.0, 2.0, -4.0, -3.0], [7.0, -2.0, -5.0, -7.0, 4.0], [-6.0, -4.0, -6.0, -5.0, -1.0], [2.0, 3.0, 1.0, -6.0, -5.0]], [[0.0, -5.0, 7.0, -5.0, -1.0], [-7.0, 3.0, -7.0, 0.0, 5.0], [0.0, -5.0, 4.0, -3.0, -5.0], [4.0, 0.0, -3.0, 5.0, 0.0]], [[-6.0, -7.0, -4.0, -7.0, 7.0], [-7.0, 5.0, 4.0, -3.0, 7.0], [3.0, 2.0, 0.0, 6.0, -2.0], [4.0, 3.0, -7.0, 5.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 7.0, 2.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-12.0, -34.0, 31.0, -29.0, 22.0], [-98.0, 41.0, -16.0, 29.0, 29.0], [36.0, -11.0, 58.0, 16.0, -34.0], [26.0, -9.0, -40.0, 75.0, 21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[[-2.0, 4.0, 4.0, -2.0, 0.0], [2.0, -6.0, -7.0, -5.0, -5.0], [-4.0, 1.0, -1.0, -7.0, 2.0], [1.0, -7.0, -6.0, -1.0, -5.0]], [[0.0, 0.0, -2.0, 7.0, 3.0], [6.0, -5.0, -2.0, -1.0, -3.0], [-1.0, 2.0, -4.0, 2.0, 4.0], [3.0, 2.0, -5.0, -6.0, 2.0]], [[4.0, -5.0, -6.0, 7.0, -4.0], [-2.0, 6.0, 2.0, 2.0, -7.0], [-3.0, 1.0, 0.0, -6.0, 5.0], [1.0, 2.0, -6.0, 2.0, -7.0]]], [[[-2.0, -3.0, -3.0, -5.0, -6.0], [0.0, 6.0, 6.0, -4.0, 7.0], [-4.0, 2.0, 7.0, 3.0, 7.0], [-6.0, -4.0, -6.0, -7.0, 0.0]], [[7.0, -1.0, 3.0, 7.0, 7.0], [5.0, -1.0, -6.0, 7.0, -7.0], [0.0, 6.0, -5.0, -1.0, 0.0], [1.0, 2.0, 3.0, 2.0, 4.0]], [[-7.0, -1.0, -4.0, 0.0, 1.0], [3.0, -2.0, -7.0, -2.0, 7.0], [-3.0, -3.0, -4.0, 2.0, 2.0], [0.0, 6.0, 1.0, 7.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[6.0, 1.0, 7.0], [-7.0, 1.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-12.0, 2.0, -24.0, 86.0, 31.0], [30.0, -56.0, -127.0, 4.0, -89.0], [-39.0, -14.0, -92.0, -90.0, 16.0], [59.0, 46.0, -31.0, 102.0, -80.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_array_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[-3.0, 0.0], [-6.0, 2.0]], [[-5.0, 1.0], [-7.0, 4.0]], [[-1.0, -5.0], [-7.0, -6.0]], [[-2.0, -6.0], [3.0, 3.0]], [[3.0, 3.0], [1.0, 7.0]], [[-3.0, 3.0], [7.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-2.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[6.0, -0.0], [12.0, -4.0]], [[10.0, -2.0], [14.0, -8.0]], [[2.0, 10.0], [14.0, 12.0]], [[4.0, 12.0], [-6.0, -6.0]], [[-6.0, -6.0], [-2.0, -14.0]], [[6.0, -6.0], [-14.0, -8.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[[0.0, -6.0], [0.0, 2.0]], [[-2.0, -1.0], [-7.0, -1.0]], [[6.0, -3.0], [-7.0, 1.0]], [[0.0, 7.0], [-7.0, -2.0]], [[3.0, 1.0], [-5.0, 4.0]], [[-3.0, 1.0], [-6.0, -5.0]]], [[[2.0, -5.0], [0.0, -1.0]], [[-1.0, 0.0], [5.0, -1.0]], [[0.0, -7.0], [-1.0, 1.0]], [[-3.0, 5.0], [-6.0, 7.0]], [[0.0, -7.0], [-5.0, 1.0]], [[2.0, -7.0], [-7.0, 5.0]]], [[[-3.0, -4.0], [4.0, 5.0]], [[-7.0, 4.0], [4.0, -3.0]], [[5.0, -4.0], [-7.0, 5.0]], [[0.0, -7.0], [-4.0, -3.0]], [[0.0, -5.0], [6.0, 0.0]], [[7.0, 7.0], [-6.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-6.0, 1.0, 4.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-10.0, 15.0], [16.0, 7.0]], [[-17.0, 22.0], [63.0, -7.0]], [[-16.0, -5.0], [13.0, 15.0]], [[-3.0, -65.0], [20.0, 7.0]], [[-18.0, -33.0], [49.0, -23.0]], [[48.0, 15.0], [5.0, 7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_array_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[1.0, 0.0, -6.0, -5.0], [5.0, -2.0, -6.0, 1.0], [-6.0, 6.0, 0.0, 3.0]], [[1.0, -5.0, -2.0, -2.0], [0.0, 1.0, 7.0, 5.0], [-2.0, 6.0, -1.0, 2.0]]], [[[0.0, -6.0, 5.0, 6.0], [6.0, 2.0, 7.0, -6.0], [2.0, -3.0, -2.0, -1.0]], [[-6.0, 3.0, -4.0, -1.0], [0.0, 3.0, 4.0, 2.0], [-6.0, 3.0, 0.0, 7.0]]], [[[7.0, -5.0, -1.0, -2.0], [3.0, -2.0, -4.0, -7.0], [-4.0, -2.0, 3.0, -7.0]], [[-3.0, 0.0, 7.0, 3.0], [-4.0, 5.0, 6.0, 1.0], [-4.0, 1.0, -3.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[0.0, 0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]], [[0.0, -0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0]]], [[[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0]], [[-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]]], [[[0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, -0.0]], [[-0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(2.0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(14.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([7.0, -5.0, -1.0])
      s1=numpy.array([5.0, 5.0, 4.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(6.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[3.0, 0.0, -7.0], [6.0, 3.0, -1.0]])
      s1=numpy.array([[1.0, 3.0, 4.0], [4.0, 1.0, 0.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(2.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[7.0, 2.0, -1.0], [-2.0, 2.0, 7.0], [-6.0, -1.0, 2.0], [-3.0, -5.0, 3.0]], [[-2.0, 0.0, 2.0], [-1.0, -6.0, 0.0], [-2.0, -6.0, -1.0], [0.0, -7.0, 0.0]]])
      s1=numpy.array([[[1.0, 0.0, 3.0], [-2.0, 6.0, 4.0], [2.0, 2.0, -7.0], [5.0, -7.0, 4.0]], [[-5.0, 3.0, -7.0], [-7.0, -5.0, 1.0], [0.0, 2.0, 3.0], [0.0, -5.0, 4.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(105.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[7.0, 2.0], [-4.0, 0.0], [0.0, -5.0]], [[-1.0, 5.0], [3.0, -5.0], [-6.0, 0.0]]], [[[2.0, 7.0], [2.0, 5.0], [1.0, -1.0]], [[-5.0, -4.0], [-1.0, 2.0], [-5.0, 0.0]]], [[[-1.0, 0.0], [-5.0, -3.0], [-6.0, -4.0]], [[-4.0, -6.0], [6.0, -6.0], [-3.0, 7.0]]], [[[7.0, -7.0], [0.0, 4.0], [-2.0, 0.0]], [[2.0, 4.0], [7.0, 1.0], [-4.0, 5.0]]]])
      s1=numpy.array([[[[2.0, -2.0], [-5.0, 6.0], [4.0, -3.0]], [[-6.0, -3.0], [3.0, 0.0], [2.0, 0.0]]], [[[0.0, 0.0], [0.0, -4.0], [5.0, 4.0]], [[-3.0, -6.0], [-6.0, 1.0], [4.0, -5.0]]], [[[3.0, 0.0], [7.0, 0.0], [5.0, 6.0]], [[-6.0, -7.0], [0.0, -2.0], [-3.0, 0.0]]], [[[-4.0, -4.0], [0.0, -3.0], [1.0, 5.0]], [[0.0, -5.0], [-3.0, 3.0], [7.0, -6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-74.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      s1=numpy.array([-6.0, 6.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-0.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([1.0, -1.0, 5.0])
      s1=numpy.array([[-3.0, 4.0], [2.0, -1.0], [4.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([15.0, -30.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[1.0, 3.0, 0.0], [-5.0, -3.0, -6.0]])
      s1=numpy.array([[[-3.0, -7.0], [-3.0, 2.0], [-2.0, -4.0]], [[-5.0, -4.0], [5.0, -4.0], [5.0, 0.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-32.0, 31.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[5.0, -4.0, -1.0], [-1.0, 4.0, 4.0], [0.0, 0.0, -7.0], [4.0, -5.0, 2.0]], [[-7.0, 0.0, 0.0], [7.0, 0.0, -1.0], [-2.0, 0.0, -2.0], [-4.0, 0.0, 7.0]]])
      s1=numpy.array([[[[0.0, 7.0], [-4.0, -5.0], [7.0, 6.0]], [[-4.0, 6.0], [1.0, 0.0], [3.0, 5.0]], [[1.0, -2.0], [7.0, -6.0], [-4.0, 4.0]], [[6.0, -3.0], [-6.0, 1.0], [-6.0, -1.0]]], [[[-2.0, 7.0], [0.0, 4.0], [0.0, 1.0]], [[7.0, -5.0], [2.0, 6.0], [4.0, 0.0]], [[-7.0, 2.0], [7.0, 6.0], [6.0, 3.0]], [[-3.0, -1.0], [7.0, -6.0], [-6.0, -7.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([130.0, -123.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(1.0)
      s1=numpy.array([[-3.0, 7.0, 3.0, 5.0, -2.0], [4.0, 6.0, -2.0, 7.0, -5.0], [-7.0, 7.0, 4.0, -3.0, -3.0], [1.0, 5.0, -5.0, 0.0, 6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-3.0, 7.0, 3.0, 5.0, -2.0], [4.0, 6.0, -2.0, 7.0, -5.0], [-7.0, 7.0, 4.0, -3.0, -3.0], [1.0, 5.0, -5.0, 0.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, 4.0, -4.0])
      s1=numpy.array([[[-2.0, 4.0, 4.0, 1.0, 3.0], [4.0, 1.0, 7.0, 5.0, -7.0], [5.0, 4.0, 0.0, -4.0, 4.0], [5.0, 1.0, -6.0, 3.0, -2.0]], [[-4.0, -7.0, 0.0, -4.0, -5.0], [1.0, 3.0, 7.0, 3.0, -3.0], [-5.0, 4.0, 3.0, -1.0, 6.0], [0.0, -3.0, 7.0, 2.0, 2.0]], [[3.0, 7.0, -4.0, 3.0, 0.0], [4.0, 4.0, 7.0, 2.0, -2.0], [2.0, 4.0, 6.0, -1.0, 6.0], [2.0, -7.0, 5.0, -7.0, -4.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-22.0, -68.0, 4.0, -31.0, -29.0], [-24.0, -7.0, -21.0, -11.0, 17.0], [-43.0, -12.0, -12.0, 12.0, -12.0], [-23.0, 13.0, 26.0, 27.0, 30.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[0.0, 5.0, -2.0], [-7.0, -4.0, -7.0]])
      s1=numpy.array([[[[1.0, 0.0, 0.0, 3.0, -3.0], [-2.0, 0.0, 7.0, -2.0, 0.0], [7.0, -7.0, 7.0, 1.0, 1.0], [-2.0, -4.0, -5.0, 1.0, 1.0]], [[2.0, 1.0, 5.0, -1.0, -3.0], [-4.0, 1.0, 2.0, 0.0, -1.0], [-3.0, 7.0, 6.0, -4.0, -5.0], [6.0, 5.0, 4.0, -1.0, 0.0]], [[6.0, 2.0, 4.0, -5.0, -1.0], [-3.0, -4.0, -6.0, 7.0, 0.0], [-6.0, 6.0, 5.0, 3.0, -1.0], [7.0, 7.0, -7.0, 7.0, 0.0]]], [[[-4.0, -1.0, 7.0, 1.0, -7.0], [-7.0, -7.0, -4.0, 3.0, 0.0], [-6.0, 5.0, -5.0, -1.0, 0.0], [2.0, 7.0, -6.0, -6.0, -2.0]], [[3.0, 3.0, 3.0, 2.0, 0.0], [7.0, -2.0, -3.0, -1.0, 7.0], [2.0, -1.0, 7.0, -4.0, -3.0], [0.0, 4.0, -6.0, 4.0, -4.0]], [[-4.0, 3.0, 0.0, 2.0, 1.0], [-7.0, 2.0, 5.0, 0.0, 6.0], [-4.0, -6.0, 0.0, -1.0, 1.0], [6.0, 1.0, 6.0, -7.0, 3.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[42.0, -25.0, -44.0, -24.0, 29.0], [56.0, 56.0, 27.0, -31.0, -75.0], [59.0, 34.0, 27.0, 4.0, -18.0], [-40.0, -61.0, 58.0, 56.0, 9.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(4.0)
      s1=numpy.array([[[3.0, 7.0], [5.0, 6.0]], [[-2.0, 4.0], [0.0, -3.0]], [[-1.0, -4.0], [-1.0, -5.0]], [[-3.0, 7.0], [0.0, 3.0]], [[-1.0, 0.0], [7.0, 1.0]], [[-6.0, 0.0], [6.0, 3.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[12.0, 28.0], [20.0, 24.0]], [[-8.0, 16.0], [0.0, -12.0]], [[-4.0, -16.0], [-4.0, -20.0]], [[-12.0, 28.0], [0.0, 12.0]], [[-4.0, 0.0], [28.0, 4.0]], [[-24.0, 0.0], [24.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 0.0, -3.0])
      s1=numpy.array([[[[5.0, 2.0], [-6.0, 6.0]], [[2.0, -1.0], [5.0, 3.0]], [[7.0, 0.0], [5.0, -5.0]], [[-4.0, 6.0], [6.0, 7.0]], [[3.0, 0.0], [6.0, -3.0]], [[-4.0, 7.0], [-6.0, -4.0]]], [[[6.0, -7.0], [2.0, 6.0]], [[1.0, -4.0], [-4.0, 5.0]], [[6.0, 5.0], [-5.0, -5.0]], [[-4.0, -5.0], [4.0, 3.0]], [[-4.0, -3.0], [-1.0, -3.0]], [[-6.0, -7.0], [-6.0, 3.0]]], [[[-4.0, 6.0], [7.0, 4.0]], [[0.0, -1.0], [1.0, -4.0]], [[-1.0, -4.0], [-1.0, 6.0]], [[0.0, 0.0], [-6.0, -2.0]], [[-3.0, -4.0], [-5.0, -1.0]], [[-7.0, -2.0], [2.0, 5.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-13.0, -28.0], [9.0, -42.0]], [[-10.0, 8.0], [-28.0, -3.0]], [[-32.0, 12.0], [-22.0, 7.0]], [[20.0, -30.0], [-12.0, -29.0]], [[-6.0, 12.0], [-15.0, 18.0]], [[41.0, -29.0], [24.0, 5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_Symbol_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-2.0)
      s1=numpy.array([[[[0.0, -5.0, 0.0, 0.0], [3.0, -1.0, 2.0, 7.0], [0.0, -4.0, -7.0, -1.0]], [[-4.0, -5.0, -7.0, -1.0], [0.0, -6.0, -3.0, 2.0], [0.0, 0.0, -7.0, 0.0]]], [[[-2.0, 0.0, 0.0, 4.0], [-4.0, -3.0, -5.0, 0.0], [3.0, -5.0, 7.0, 1.0]], [[-2.0, -2.0, -4.0, -4.0], [2.0, -3.0, -7.0, 1.0], [4.0, -6.0, 3.0, 1.0]]], [[[0.0, 0.0, 5.0, 7.0], [0.0, -7.0, -2.0, 5.0], [5.0, 4.0, 0.0, -5.0]], [[-3.0, 6.0, 6.0, 6.0], [-7.0, -3.0, 0.0, -5.0], [-5.0, -3.0, -3.0, 2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.0, 10.0, -0.0, -0.0], [-6.0, 2.0, -4.0, -14.0], [-0.0, 8.0, 14.0, 2.0]], [[8.0, 10.0, 14.0, 2.0], [-0.0, 12.0, 6.0, -4.0], [-0.0, -0.0, 14.0, -0.0]]], [[[4.0, -0.0, -0.0, -8.0], [8.0, 6.0, 10.0, -0.0], [-6.0, 10.0, -14.0, -2.0]], [[4.0, 4.0, 8.0, 8.0], [-4.0, 6.0, 14.0, -2.0], [-8.0, 12.0, -6.0, -2.0]]], [[[-0.0, -0.0, -10.0, -14.0], [-0.0, 14.0, 4.0, -10.0], [-10.0, -8.0, -0.0, 10.0]], [[6.0, -12.0, -12.0, -12.0], [14.0, 6.0, -0.0, 10.0], [10.0, 6.0, 6.0, -4.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_constData_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(-7.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-1.0)
      sub=res.substitute({arg0:s0})
      ref=Data(7.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([6.0, -5.0, -2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-6.0, 0.0, -3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(-30.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[-3.0, -2.0, -5.0], [-5.0, -7.0, 1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-5.0, -5.0, 7.0], [0.0, 3.0, 1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(-30.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[0.0, -5.0, -2.0], [-2.0, -2.0, 0.0], [7.0, 6.0, -3.0], [-6.0, 6.0, 2.0]], [[-5.0, -6.0, -3.0], [7.0, 0.0, 0.0], [0.0, 7.0, -1.0], [-4.0, -1.0, -4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[3.0, 3.0, -7.0], [-1.0, -5.0, 4.0], [5.0, -4.0, 1.0], [6.0, 2.0, 0.0]], [[0.0, 0.0, 0.0], [0.0, -2.0, 5.0], [-3.0, 1.0, 3.0], [6.0, -4.0, 3.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-33.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Data(numpy.array([[[[2.0, 0.0], [1.0, -2.0], [7.0, 5.0]], [[-6.0, 6.0], [0.0, -5.0], [-6.0, 0.0]]], [[[-4.0, 4.0], [-2.0, 6.0], [0.0, -5.0]], [[7.0, 6.0], [-4.0, 2.0], [-6.0, -2.0]]], [[[0.0, 5.0], [6.0, 6.0], [1.0, -6.0]], [[-7.0, 0.0], [-3.0, -7.0], [2.0, -1.0]]], [[[3.0, 3.0], [3.0, -2.0], [2.0, -6.0]], [[-3.0, 3.0], [-2.0, 1.0], [5.0, -2.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[0.0, 2.0], [2.0, -7.0], [5.0, -7.0]], [[0.0, 3.0], [7.0, -2.0], [7.0, -5.0]]], [[[0.0, -1.0], [-6.0, 6.0], [-1.0, -1.0]], [[1.0, -3.0], [1.0, 6.0], [-6.0, -3.0]]], [[[7.0, 6.0], [7.0, 6.0], [-7.0, -2.0]], [[-3.0, -7.0], [3.0, 0.0], [-2.0, 0.0]]], [[[1.0, 0.0], [7.0, -1.0], [1.0, 0.0]], [[0.0, 7.0], [5.0, -3.0], [4.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(267.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_constData_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-3.0, 5.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-1.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([3.0, -5.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[0.0, 1.0], [0.0, -3.0], [0.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, 7.0, -2.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([0.0, -24.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[0.0, -6.0], [4.0, -3.0], [3.0, 5.0]], [[-1.0, 3.0], [-1.0, 6.0], [0.0, -1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[5.0, -6.0, 6.0], [5.0, 5.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-16.0, 63.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[[5.0, -2.0], [0.0, 4.0], [7.0, 0.0]], [[0.0, -6.0], [5.0, 0.0], [6.0, -1.0]], [[2.0, -1.0], [1.0, 1.0], [-6.0, 0.0]], [[-2.0, 7.0], [7.0, -5.0], [3.0, -4.0]]], [[[-5.0, 2.0], [-7.0, 3.0], [4.0, 0.0]], [[3.0, 7.0], [-3.0, 2.0], [-6.0, 3.0]], [[-6.0, 3.0], [0.0, -6.0], [-1.0, 1.0]], [[7.0, -1.0], [-4.0, 5.0], [-6.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-2.0, -7.0, 6.0], [-3.0, -5.0, -7.0], [-3.0, -2.0, -3.0], [-2.0, -3.0, 6.0]], [[-2.0, 0.0, -3.0], [4.0, -4.0, 0.0], [-3.0, 1.0, -2.0], [-4.0, -6.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([44.0, -48.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_constData_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[0.0, 2.0, -3.0, 0.0, 0.0], [-1.0, 0.0, 3.0, -3.0, -4.0], [6.0, 5.0, -1.0, 2.0, 4.0], [6.0, -2.0, 0.0, 3.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(6.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[0.0, 12.0, -18.0, 0.0, 0.0], [-6.0, 0.0, 18.0, -18.0, -24.0], [36.0, 30.0, -6.0, 12.0, 24.0], [36.0, -12.0, 0.0, 18.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[3.0, 3.0, 2.0, 2.0, -1.0], [-6.0, 5.0, -6.0, -2.0, 0.0], [4.0, -4.0, 7.0, 0.0, -7.0], [-7.0, -2.0, 3.0, -7.0, -5.0]], [[-6.0, 3.0, -2.0, 4.0, -7.0], [-7.0, -3.0, -7.0, 3.0, 7.0], [0.0, 0.0, -3.0, 6.0, 5.0], [7.0, 2.0, 0.0, 0.0, 7.0]], [[0.0, 7.0, 5.0, 6.0, 6.0], [0.0, 6.0, 3.0, 6.0, -2.0], [-1.0, 7.0, -2.0, -1.0, -5.0], [0.0, 5.0, 4.0, -6.0, -1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([3.0, 4.0, -1.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-15.0, 14.0, -7.0, 16.0, -37.0], [-46.0, -3.0, -49.0, 0.0, 30.0], [13.0, -19.0, 11.0, 25.0, 4.0], [7.0, -3.0, 5.0, -15.0, 14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[[-2.0, 4.0, 0.0, 3.0, 4.0], [-5.0, -6.0, 5.0, 6.0, -4.0], [-7.0, -3.0, 1.0, 0.0, 5.0], [0.0, 4.0, -6.0, 1.0, 5.0]], [[2.0, 3.0, 6.0, -2.0, -3.0], [1.0, 7.0, -4.0, 3.0, 3.0], [6.0, -2.0, -4.0, -4.0, -3.0], [-1.0, 2.0, -6.0, -2.0, 1.0]], [[5.0, 2.0, -7.0, -1.0, 0.0], [-7.0, 5.0, 3.0, 0.0, 6.0], [3.0, -2.0, -3.0, 2.0, -6.0], [1.0, 0.0, -5.0, 6.0, 2.0]]], [[[0.0, 0.0, 3.0, -3.0, -1.0], [-6.0, -4.0, -2.0, 4.0, -6.0], [-4.0, 3.0, 6.0, -6.0, 5.0], [2.0, 5.0, -1.0, 6.0, 3.0]], [[0.0, -2.0, -4.0, 2.0, 7.0], [2.0, 7.0, -4.0, 7.0, -2.0], [-6.0, -5.0, 7.0, 5.0, 3.0], [-3.0, 0.0, -6.0, -1.0, 0.0]], [[3.0, 2.0, -3.0, 1.0, -3.0], [6.0, 7.0, -3.0, 4.0, 6.0], [-2.0, 5.0, -4.0, -3.0, 1.0], [-7.0, 0.0, -5.0, 5.0, -4.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[3.0, -5.0, 4.0], [-3.0, 7.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-17.0, -23.0, -74.0, 31.0, 100.0], [-58.0, -21.0, 46.0, 12.0, -41.0], [-55.0, -86.0, 70.0, 102.0, 5.0], [31.0, -13.0, -12.0, -23.0, 37.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_constData_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[-2.0, 5.0], [0.0, 1.0]], [[-3.0, -4.0], [1.0, -2.0]], [[0.0, 1.0], [3.0, -3.0]], [[0.0, 7.0], [5.0, 5.0]], [[-5.0, 0.0], [0.0, -2.0]], [[4.0, -7.0], [3.0, -1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-1.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[2.0, -5.0], [-0.0, -1.0]], [[3.0, 4.0], [-1.0, 2.0]], [[-0.0, -1.0], [-3.0, 3.0]], [[-0.0, -7.0], [-5.0, -5.0]], [[5.0, -0.0], [-0.0, 2.0]], [[-4.0, 7.0], [-3.0, 1.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[[2.0, -4.0], [-5.0, 0.0]], [[2.0, 0.0], [-5.0, -5.0]], [[-3.0, -2.0], [4.0, 2.0]], [[-1.0, -3.0], [3.0, 0.0]], [[7.0, -6.0], [-7.0, -4.0]], [[1.0, 5.0], [0.0, 1.0]]], [[[4.0, 0.0], [5.0, 1.0]], [[-4.0, 0.0], [0.0, 6.0]], [[2.0, 6.0], [-2.0, -1.0]], [[-3.0, 5.0], [1.0, 0.0]], [[1.0, 0.0], [0.0, -5.0]], [[-3.0, 3.0], [-5.0, 2.0]]], [[[4.0, 2.0], [-7.0, 7.0]], [[-7.0, 3.0], [-6.0, -6.0]], [[-2.0, 2.0], [3.0, -2.0]], [[-2.0, -3.0], [-1.0, 3.0]], [[0.0, -2.0], [-6.0, -4.0]], [[4.0, 3.0], [7.0, 6.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, 5.0, 1.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[18.0, 14.0], [33.0, 12.0]], [[-33.0, 3.0], [9.0, 39.0]], [[17.0, 38.0], [-19.0, -13.0]], [[-14.0, 31.0], [-5.0, 3.0]], [[-16.0, 16.0], [15.0, -17.0]], [[-14.0, 3.0], [-18.0, 13.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_constData_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[1.0, -3.0, 5.0, -2.0], [-6.0, 0.0, -4.0, 3.0], [-4.0, -6.0, 0.0, 1.0]], [[0.0, -7.0, -6.0, -1.0], [-5.0, 2.0, 6.0, 5.0], [-6.0, -4.0, 7.0, -3.0]]], [[[-7.0, -3.0, 0.0, -3.0], [1.0, 5.0, 0.0, 2.0], [0.0, -6.0, -7.0, 4.0]], [[2.0, 7.0, 4.0, -2.0], [-1.0, -7.0, -5.0, 7.0], [-7.0, 2.0, -6.0, -4.0]]], [[[-2.0, 2.0, -2.0, -6.0], [-2.0, 1.0, 0.0, 4.0], [0.0, -4.0, 0.0, 4.0]], [[2.0, -4.0, 3.0, 2.0], [6.0, 6.0, 5.0, -7.0], [5.0, 3.0, -4.0, 3.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-1.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-1.0, 3.0, -5.0, 2.0], [6.0, -0.0, 4.0, -3.0], [4.0, 6.0, -0.0, -1.0]], [[-0.0, 7.0, 6.0, 1.0], [5.0, -2.0, -6.0, -5.0], [6.0, 4.0, -7.0, 3.0]]], [[[7.0, 3.0, -0.0, 3.0], [-1.0, -5.0, -0.0, -2.0], [-0.0, 6.0, 7.0, -4.0]], [[-2.0, -7.0, -4.0, 2.0], [1.0, 7.0, 5.0, -7.0], [7.0, -2.0, 6.0, 4.0]]], [[[2.0, -2.0, 2.0, 6.0], [2.0, -1.0, -0.0, -4.0], [-0.0, 4.0, -0.0, -4.0]], [[-2.0, 4.0, -3.0, -2.0], [-6.0, -6.0, -5.0, 7.0], [-5.0, -3.0, 4.0, -3.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(4.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(0.0)+(1.-msk_ref)*numpy.array(28.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, -6.0, -6.0])+(1.-msk_arg1)*numpy.array([-3.0, -3.0, -2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([1.0, 4.0, -4.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-5.0)+(1.-msk_ref)*numpy.array(-7.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 1.0, 7.0], [6.0, -6.0, 0.0]])+(1.-msk_arg1)*numpy.array([[7.0, 4.0, 2.0], [-7.0, 6.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[5.0, -6.0, -5.0], [0.0, 5.0, 6.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-106.0)+(1.-msk_ref)*numpy.array(-11.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 2.0, 0.0], [7.0, 6.0, 0.0], [5.0, -3.0, 0.0], [-6.0, -1.0, -6.0]], [[0.0, -3.0, -5.0], [-7.0, 2.0, 0.0], [-6.0, 3.0, 5.0], [2.0, -2.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 0.0, 7.0], [2.0, -4.0, 3.0], [7.0, -7.0, -1.0], [-2.0, -5.0, -7.0]], [[-1.0, 7.0, 0.0], [1.0, -2.0, -3.0], [-4.0, -7.0, 5.0], [7.0, -4.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-4.0, 3.0, -3.0], [-3.0, -3.0, 6.0], [-2.0, 4.0, -1.0], [1.0, 5.0, 3.0]], [[-6.0, -3.0, 3.0], [-6.0, 2.0, 3.0], [-7.0, 7.0, -4.0], [4.0, -4.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-4.0)+(1.-msk_ref)*numpy.array(-131.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -3.0], [-6.0, 0.0], [1.0, -6.0]], [[1.0, 3.0], [7.0, -6.0], [1.0, 1.0]]], [[[-7.0, -3.0], [-4.0, 6.0], [0.0, -2.0]], [[0.0, -6.0], [6.0, -3.0], [2.0, -1.0]]], [[[-6.0, 7.0], [-1.0, -3.0], [-4.0, 6.0]], [[-7.0, -7.0], [-1.0, -1.0], [-3.0, 3.0]]], [[[4.0, 4.0], [-4.0, 1.0], [-2.0, -7.0]], [[-3.0, 5.0], [-6.0, -5.0], [-3.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 1.0], [1.0, -4.0], [-4.0, 7.0]], [[0.0, -3.0], [1.0, -7.0], [5.0, -2.0]]], [[[-1.0, 4.0], [4.0, 7.0], [4.0, 6.0]], [[0.0, -7.0], [-6.0, -4.0], [0.0, -3.0]]], [[[-1.0, 0.0], [0.0, -3.0], [-5.0, 0.0]], [[-1.0, 3.0], [-7.0, -3.0], [0.0, -6.0]]], [[[2.0, 6.0], [6.0, -5.0], [-6.0, 4.0]], [[7.0, 7.0], [0.0, -4.0], [5.0, -4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[-5.0, -4.0], [-7.0, -2.0], [4.0, -5.0]], [[-6.0, 3.0], [-2.0, 4.0], [5.0, 4.0]]], [[[6.0, 0.0], [7.0, -7.0], [0.0, 1.0]], [[-1.0, 1.0], [0.0, 7.0], [7.0, 6.0]]], [[[4.0, 6.0], [0.0, -7.0], [-7.0, -2.0]], [[-5.0, 3.0], [0.0, -6.0], [4.0, -2.0]]], [[[-4.0, -4.0], [3.0, 3.0], [7.0, 6.0]], [[5.0, -2.0], [-7.0, 0.0], [7.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-140.0)+(1.-msk_ref)*numpy.array(-15.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 6.0])+(1.-msk_arg1)*numpy.array([0.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(6.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, 36.0])+(1.-msk_ref)*numpy.array([0.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 5.0], [-2.0, -5.0], [-5.0, -5.0]])+(1.-msk_arg1)*numpy.array([[-1.0, 2.0], [7.0, -7.0], [6.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([0.0, 7.0, 0.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-14.0, -35.0])+(1.-msk_ref)*numpy.array([49.0, -49.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 5.0], [1.0, -2.0], [-7.0, -4.0]], [[0.0, 5.0], [-2.0, -5.0], [-6.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, 7.0], [2.0, 7.0], [6.0, 0.0]], [[-6.0, 4.0], [1.0, 0.0], [-1.0, 6.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[3.0, -3.0, 0.0], [3.0, 1.0, 2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-5.0, 39.0])+(1.-msk_ref)*numpy.array([-28.0, 24.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -6.0], [0.0, 7.0], [-5.0, 1.0]], [[5.0, 3.0], [2.0, 5.0], [4.0, 1.0]], [[5.0, 1.0], [2.0, -4.0], [-3.0, -4.0]], [[0.0, 1.0], [-2.0, 4.0], [-1.0, 7.0]]], [[[-3.0, 6.0], [-5.0, 3.0], [3.0, -7.0]], [[7.0, 1.0], [5.0, 0.0], [4.0, -3.0]], [[1.0, 4.0], [-3.0, 2.0], [-2.0, 7.0]], [[-5.0, 5.0], [-2.0, 2.0], [-1.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 3.0], [-2.0, 4.0], [-3.0, 7.0]], [[7.0, 5.0], [0.0, -7.0], [-3.0, 0.0]], [[-4.0, -6.0], [-6.0, 1.0], [0.0, -3.0]], [[2.0, 2.0], [3.0, 4.0], [-4.0, 5.0]]], [[[0.0, -1.0], [2.0, -2.0], [6.0, -1.0]], [[-5.0, -6.0], [1.0, -4.0], [1.0, -5.0]], [[-6.0, -7.0], [7.0, -2.0], [-4.0, -1.0]], [[6.0, 0.0], [2.0, -4.0], [-6.0, -6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-1.0, 2.0, 1.0], [0.0, -2.0, -1.0], [-2.0, 6.0, 1.0], [-2.0, -4.0, 4.0]], [[-3.0, 0.0, 7.0], [-5.0, -4.0, -2.0], [-2.0, 4.0, -7.0], [-2.0, -4.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-18.0, -158.0])+(1.-msk_ref)*numpy.array([75.0, 152.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -3.0, -1.0, -6.0, 4.0], [-6.0, 6.0, 6.0, 3.0, 1.0], [-3.0, 6.0, 6.0, 4.0, -5.0], [-2.0, 0.0, -2.0, 6.0, -2.0]])+(1.-msk_arg1)*numpy.array([[5.0, 6.0, -7.0, 7.0, -5.0], [5.0, 3.0, 0.0, 1.0, 7.0], [-1.0, 3.0, 3.0, 6.0, 0.0], [-1.0, 0.0, -1.0, 4.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(3.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, -9.0, -3.0, -18.0, 12.0], [-18.0, 18.0, 18.0, 9.0, 3.0], [-9.0, 18.0, 18.0, 12.0, -15.0], [-6.0, 0.0, -6.0, 18.0, -6.0]])+(1.-msk_ref)*numpy.array([[15.0, 18.0, -21.0, 21.0, -15.0], [15.0, 9.0, 0.0, 3.0, 21.0], [-3.0, 9.0, 9.0, 18.0, 0.0], [-3.0, 0.0, -3.0, 12.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, -3.0, -6.0, -3.0, 7.0], [4.0, -5.0, 0.0, 7.0, 0.0], [2.0, -7.0, 7.0, -7.0, 2.0], [3.0, 5.0, 2.0, 6.0, 3.0]], [[2.0, 2.0, -6.0, 6.0, 0.0], [0.0, 3.0, 0.0, -6.0, 7.0], [3.0, -2.0, 3.0, 6.0, 0.0], [-1.0, 0.0, 7.0, -2.0, 5.0]], [[1.0, -6.0, -6.0, 6.0, -4.0], [5.0, 1.0, 7.0, 3.0, 0.0], [-3.0, 6.0, 7.0, 5.0, 0.0], [3.0, -2.0, 0.0, 3.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 1.0, -1.0, -4.0, 7.0], [3.0, 0.0, -3.0, 4.0, -4.0], [3.0, 7.0, 0.0, -3.0, -4.0], [-7.0, -2.0, -7.0, 1.0, 3.0]], [[7.0, -3.0, 4.0, 3.0, -6.0], [3.0, 1.0, -7.0, 1.0, -3.0], [-3.0, -3.0, 5.0, -7.0, -2.0], [-2.0, 2.0, 0.0, -7.0, 0.0]], [[6.0, -7.0, 3.0, 0.0, 0.0], [-4.0, -5.0, 5.0, -3.0, -7.0], [2.0, 0.0, 6.0, -3.0, -1.0], [0.0, 0.0, 4.0, 3.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-2.0, 5.0, -2.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 28.0, -6.0, 24.0, -6.0], [-18.0, 23.0, -14.0, -50.0, 35.0], [17.0, -8.0, -13.0, 34.0, -4.0], [-17.0, -6.0, 31.0, -28.0, 11.0]])+(1.-msk_ref)*numpy.array([[23.0, -3.0, 16.0, 23.0, -44.0], [17.0, 15.0, -39.0, 3.0, 7.0], [-25.0, -29.0, 13.0, -23.0, 0.0], [4.0, 14.0, 6.0, -43.0, -2.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -6.0, 5.0, -2.0, 6.0], [3.0, -2.0, -2.0, -5.0, 4.0], [0.0, 3.0, -3.0, -7.0, 6.0], [-3.0, 4.0, 4.0, -6.0, 4.0]], [[3.0, 5.0, 4.0, 2.0, 1.0], [-4.0, 5.0, 0.0, -3.0, -2.0], [-6.0, 2.0, -3.0, 4.0, -6.0], [2.0, 7.0, 2.0, 6.0, 6.0]], [[1.0, -3.0, 0.0, 3.0, 7.0], [4.0, -7.0, 6.0, 1.0, -3.0], [-1.0, 0.0, 7.0, 0.0, -5.0], [-7.0, -6.0, -5.0, -5.0, 5.0]]], [[[-1.0, -7.0, 4.0, -4.0, -3.0], [5.0, -2.0, -1.0, 4.0, 5.0], [3.0, -2.0, 2.0, 0.0, 3.0], [-4.0, 7.0, 6.0, 6.0, -7.0]], [[-6.0, 5.0, 1.0, -7.0, 0.0], [0.0, -3.0, 2.0, -5.0, 1.0], [5.0, -6.0, -4.0, 3.0, -4.0], [6.0, -2.0, -4.0, -1.0, 6.0]], [[4.0, 2.0, -5.0, 0.0, 7.0], [3.0, -4.0, -7.0, -5.0, 5.0], [0.0, -2.0, 0.0, 5.0, 6.0], [-7.0, 7.0, 1.0, 0.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 0.0, 6.0, 6.0, 4.0], [0.0, 1.0, 6.0, 2.0, 1.0], [-7.0, 1.0, 1.0, -4.0, -4.0], [2.0, 7.0, 1.0, 0.0, 3.0]], [[-1.0, -6.0, 2.0, -1.0, 1.0], [-1.0, 5.0, 7.0, 4.0, 2.0], [1.0, 5.0, 0.0, 0.0, 0.0], [-4.0, 4.0, 0.0, -2.0, 0.0]], [[7.0, 0.0, 2.0, 7.0, -1.0], [3.0, 1.0, 4.0, -4.0, -6.0], [4.0, 6.0, 0.0, 2.0, 0.0], [0.0, -4.0, -2.0, 4.0, -6.0]]], [[[-1.0, 0.0, 7.0, 1.0, -4.0], [-1.0, -5.0, 3.0, 2.0, -2.0], [3.0, 3.0, 6.0, 2.0, -7.0], [-3.0, 0.0, 6.0, 0.0, -6.0]], [[-5.0, 7.0, 0.0, 1.0, 6.0], [5.0, -5.0, -5.0, -1.0, 7.0], [-5.0, 3.0, 4.0, 4.0, 4.0], [6.0, 2.0, -5.0, -3.0, 2.0]], [[4.0, -3.0, -4.0, 1.0, 4.0], [4.0, 7.0, 0.0, -2.0, 0.0], [-5.0, 2.0, 7.0, -4.0, -1.0], [-2.0, 2.0, -6.0, -2.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-2.0, 7.0, 7.0], [-2.0, 2.0, 5.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[36.0, 60.0, -13.0, 33.0, 85.0], [-1.0, -32.0, 17.0, -47.0, -26.0], [-45.0, -10.0, 22.0, 73.0, -73.0], [-44.0, 16.0, -44.0, 5.0, 80.0]])+(1.-msk_ref)*numpy.array([[54.0, -43.0, -18.0, 35.0, 32.0], [46.0, 75.0, 49.0, -20.0, -12.0], [8.0, 85.0, 29.0, 6.0, 25.0], [-24.0, 0.0, -68.0, -2.0, -2.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 7.0], [2.0, 4.0]], [[0.0, 0.0], [-3.0, -3.0]], [[-3.0, 5.0], [5.0, -5.0]], [[6.0, 7.0], [3.0, -2.0]], [[-3.0, -3.0], [-1.0, 4.0]], [[-3.0, -1.0], [7.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 7.0], [0.0, -2.0]], [[0.0, -6.0], [3.0, -7.0]], [[-6.0, 0.0], [3.0, 0.0]], [[-1.0, -5.0], [4.0, 2.0]], [[2.0, -6.0], [-3.0, -5.0]], [[-3.0, 7.0], [-4.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(3.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, 21.0], [6.0, 12.0]], [[0.0, 0.0], [-9.0, -9.0]], [[-9.0, 15.0], [15.0, -15.0]], [[18.0, 21.0], [9.0, -6.0]], [[-9.0, -9.0], [-3.0, 12.0]], [[-9.0, -3.0], [21.0, 18.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, 21.0], [0.0, -6.0]], [[0.0, -18.0], [9.0, -21.0]], [[-18.0, 0.0], [9.0, 0.0]], [[-3.0, -15.0], [12.0, 6.0]], [[6.0, -18.0], [-9.0, -15.0]], [[-9.0, 21.0], [-12.0, 3.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, 2.0], [-2.0, -6.0]], [[-1.0, 6.0], [0.0, -5.0]], [[5.0, -5.0], [-7.0, -5.0]], [[-2.0, 3.0], [-5.0, -4.0]], [[-6.0, 6.0], [6.0, -2.0]], [[3.0, 3.0], [-5.0, -2.0]]], [[[0.0, -1.0], [0.0, 7.0]], [[0.0, 7.0], [5.0, 4.0]], [[-7.0, 3.0], [-4.0, 4.0]], [[-6.0, -6.0], [7.0, -5.0]], [[-2.0, -7.0], [4.0, -3.0]], [[-5.0, -5.0], [-7.0, -1.0]]], [[[-6.0, -7.0], [-3.0, 2.0]], [[-3.0, -3.0], [5.0, 4.0]], [[-7.0, 2.0], [-1.0, 0.0]], [[-6.0, 4.0], [-1.0, -4.0]], [[1.0, 1.0], [3.0, -6.0]], [[2.0, -3.0], [-4.0, 2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, -3.0], [2.0, 5.0]], [[-5.0, 0.0], [-4.0, -7.0]], [[-5.0, 7.0], [-6.0, 3.0]], [[-1.0, -4.0], [3.0, -3.0]], [[3.0, -2.0], [-3.0, -2.0]], [[-4.0, 4.0], [3.0, -6.0]]], [[[5.0, 5.0], [0.0, 0.0]], [[-7.0, 6.0], [-3.0, 0.0]], [[6.0, -1.0], [1.0, -2.0]], [[4.0, -6.0], [4.0, -1.0]], [[0.0, 4.0], [1.0, -6.0]], [[5.0, 6.0], [-6.0, 4.0]]], [[[-1.0, 3.0], [-4.0, 0.0]], [[0.0, -7.0], [6.0, 4.0]], [[-5.0, -5.0], [-3.0, -2.0]], [[7.0, 0.0], [-2.0, 7.0]], [[0.0, -7.0], [-2.0, 6.0]], [[-3.0, 1.0], [1.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-7.0, -3.0, -4.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-11.0, 17.0], [26.0, 13.0]], [[19.0, -51.0], [-35.0, 7.0]], [[14.0, 18.0], [65.0, 23.0]], [[56.0, -19.0], [18.0, 59.0]], [[44.0, -25.0], [-66.0, 47.0]], [[-14.0, 6.0], [72.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[24.0, -6.0], [2.0, -35.0]], [[56.0, 10.0], [13.0, 33.0]], [[37.0, -26.0], [51.0, -7.0]], [[-33.0, 46.0], [-25.0, -4.0]], [[-21.0, 30.0], [26.0, 8.0]], [[25.0, -50.0], [-7.0, 50.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank0_expandedData_rank4_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, -4.0, 1.0, 0.0], [2.0, 0.0, 0.0, 0.0], [6.0, 0.0, -1.0, -4.0]], [[-3.0, -7.0, -2.0, 0.0], [0.0, -3.0, -1.0, -7.0], [-2.0, -1.0, 6.0, 0.0]]], [[[-6.0, -1.0, 0.0, -1.0], [-2.0, 2.0, 0.0, 0.0], [4.0, 4.0, 5.0, -6.0]], [[0.0, 0.0, 6.0, 0.0], [4.0, -2.0, 6.0, 2.0], [-4.0, 7.0, 5.0, 1.0]]], [[[5.0, -3.0, 4.0, 7.0], [0.0, -1.0, 5.0, -5.0], [-5.0, -3.0, 4.0, 0.0]], [[5.0, -7.0, 2.0, 1.0], [0.0, -7.0, 4.0, -7.0], [4.0, -1.0, -6.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -2.0, 7.0, -6.0], [2.0, 0.0, -6.0, 0.0], [7.0, 7.0, 4.0, -2.0]], [[6.0, -4.0, 0.0, -4.0], [4.0, 4.0, 0.0, 0.0], [-2.0, 6.0, -2.0, 6.0]]], [[[6.0, 5.0, 6.0, 1.0], [7.0, -1.0, 3.0, -7.0], [0.0, -4.0, -2.0, 0.0]], [[5.0, 1.0, 3.0, -6.0], [0.0, 0.0, -1.0, 2.0], [-4.0, -6.0, 5.0, 6.0]]], [[[-1.0, -3.0, 0.0, 7.0], [2.0, 5.0, -1.0, 1.0], [-1.0, 7.0, 0.0, 2.0]], [[-5.0, 7.0, -6.0, 2.0], [4.0, 0.0, -6.0, 1.0], [-3.0, -2.0, -4.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(6.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[36.0, -24.0, 6.0, 0.0], [12.0, 0.0, 0.0, 0.0], [36.0, 0.0, -6.0, -24.0]], [[-18.0, -42.0, -12.0, 0.0], [0.0, -18.0, -6.0, -42.0], [-12.0, -6.0, 36.0, 0.0]]], [[[-36.0, -6.0, 0.0, -6.0], [-12.0, 12.0, 0.0, 0.0], [24.0, 24.0, 30.0, -36.0]], [[0.0, 0.0, 36.0, 0.0], [24.0, -12.0, 36.0, 12.0], [-24.0, 42.0, 30.0, 6.0]]], [[[30.0, -18.0, 24.0, 42.0], [0.0, -6.0, 30.0, -30.0], [-30.0, -18.0, 24.0, 0.0]], [[30.0, -42.0, 12.0, 6.0], [0.0, -42.0, 24.0, -42.0], [24.0, -6.0, -36.0, -18.0]]]])+(1.-msk_ref)*numpy.array([[[[24.0, -12.0, 42.0, -36.0], [12.0, 0.0, -36.0, 0.0], [42.0, 42.0, 24.0, -12.0]], [[36.0, -24.0, 0.0, -24.0], [24.0, 24.0, 0.0, 0.0], [-12.0, 36.0, -12.0, 36.0]]], [[[36.0, 30.0, 36.0, 6.0], [42.0, -6.0, 18.0, -42.0], [0.0, -24.0, -12.0, 0.0]], [[30.0, 6.0, 18.0, -36.0], [0.0, 0.0, -6.0, 12.0], [-24.0, -36.0, 30.0, 36.0]]], [[[-6.0, -18.0, 0.0, 42.0], [12.0, 30.0, -6.0, 6.0], [-6.0, 42.0, 0.0, 12.0]], [[-30.0, 42.0, -36.0, 12.0], [24.0, 0.0, -36.0, 6.0], [-18.0, -12.0, -24.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_float_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, -7.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-35.0, -35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(-1.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([6.0, -5.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-6.0, 5.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank1_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([0.0, 2.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[1.0, 0.0], [-7.0, 0.0], [-6.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-50.0, 42.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=numpy.array([[2.0, 7.0, -4.0], [-1.0, -5.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-2.0, -3.0], [-1.0, -1.0], [-6.0, -4.0]], [[2.0, 6.0], [0.0, 3.0], [1.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([4.0, -25.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=numpy.array([[[0.0, 7.0, -6.0], [7.0, -5.0, -4.0], [-3.0, 6.0, -2.0], [6.0, 0.0, 0.0]], [[4.0, 2.0, -4.0], [-3.0, 0.0, -6.0], [-2.0, -2.0, -6.0], [6.0, 0.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[1.0, -5.0], [2.0, 4.0], [1.0, -4.0]], [[-5.0, 6.0], [-1.0, 0.0], [-2.0, 6.0]], [[-3.0, 2.0], [-7.0, 7.0], [2.0, 1.0]], [[-5.0, 1.0], [6.0, -2.0], [0.0, -4.0]]], [[[5.0, 5.0], [-4.0, -5.0], [-6.0, -5.0]], [[-4.0, 6.0], [6.0, -5.0], [-1.0, 7.0]], [[-7.0, -7.0], [-6.0, 6.0], [7.0, 0.0]], [[7.0, -7.0], [0.0, -3.0], [-6.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-1.0, 40.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([4.0, 5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-6.0, 0.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-24.0, -30.0], [0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank2_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([[-1.0, 2.0], [1.0, -4.0], [0.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-1.0, 2.0], [-5.0, -5.0], [-2.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-4.0, 10.0], [-7.0, 8.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=numpy.array([[[0.0, -7.0], [1.0, -1.0], [6.0, -3.0]], [[4.0, 3.0], [2.0, 0.0], [0.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[0.0, 1.0], [-6.0, 3.0], [-3.0, -4.0]], [[-7.0, -4.0], [-3.0, -7.0], [1.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-58.0, -8.0], [-51.0, -10.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=numpy.array([[[[6.0, 0.0], [5.0, 7.0], [-5.0, 7.0]], [[-6.0, 2.0], [-5.0, 0.0], [-2.0, 4.0]], [[-1.0, 2.0], [-5.0, 0.0], [-1.0, -2.0]], [[4.0, 4.0], [0.0, 7.0], [0.0, 6.0]]], [[[5.0, 0.0], [1.0, 1.0], [-6.0, -1.0]], [[2.0, 0.0], [4.0, 6.0], [6.0, -3.0]], [[5.0, -5.0], [-3.0, -6.0], [-7.0, -1.0]], [[-5.0, 5.0], [4.0, -4.0], [-5.0, -2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[4.0, -4.0], [-7.0, 0.0], [-2.0, -6.0]], [[6.0, 0.0], [3.0, 1.0], [6.0, 2.0]], [[4.0, -7.0], [-7.0, -2.0], [1.0, 4.0]], [[1.0, 0.0], [0.0, 0.0], [-1.0, 3.0]]], [[[4.0, 4.0], [6.0, 7.0], [4.0, -6.0]], [[-4.0, -7.0], [-2.0, -1.0], [4.0, -3.0]], [[-5.0, 6.0], [-2.0, -6.0], [-1.0, -1.0]], [[-7.0, -1.0], [0.0, -6.0], [-4.0, -4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[23.0, -34.0], [93.0, 12.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[6.0, 5.0, 0.0, -3.0, 6.0], [3.0, -3.0, 4.0, 3.0, -6.0], [2.0, 3.0, -5.0, -3.0, -3.0], [0.0, -7.0, 6.0, -6.0, -2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([2.0, -1.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[12.0, 10.0, 0.0, -6.0, 12.0], [6.0, -6.0, 8.0, 6.0, -12.0], [4.0, 6.0, -10.0, -6.0, -6.0], [0.0, -14.0, 12.0, -12.0, -4.0]], [[-6.0, -5.0, 0.0, 3.0, -6.0], [-3.0, 3.0, -4.0, -3.0, 6.0], [-2.0, -3.0, 5.0, 3.0, 3.0], [0.0, 7.0, -6.0, 6.0, 2.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank3_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([[[4.0, -6.0, -2.0, -3.0, -1.0], [1.0, 7.0, -5.0, -1.0, 3.0], [-7.0, -6.0, -7.0, 0.0, 5.0], [4.0, 3.0, 1.0, 1.0, -2.0]], [[-5.0, -3.0, 2.0, -2.0, -6.0], [7.0, 6.0, -1.0, -4.0, 5.0], [6.0, -5.0, 1.0, 0.0, 3.0], [3.0, 6.0, 5.0, 0.0, -6.0]], [[6.0, 7.0, 1.0, -4.0, -1.0], [-1.0, -7.0, -3.0, 3.0, 0.0], [-1.0, -3.0, 2.0, -7.0, -2.0], [0.0, -3.0, 2.0, 6.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-2.0, -5.0], [1.0, 5.0], [-3.0, 4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-31.0, -12.0, 3.0, 16.0, -1.0], [8.0, 13.0, 18.0, -11.0, -1.0], [23.0, 16.0, 9.0, 21.0, -1.0], [-5.0, 9.0, -3.0, -20.0, -8.0]], [[-21.0, 43.0, 24.0, -11.0, -29.0], [26.0, -33.0, 8.0, -3.0, 10.0], [61.0, -7.0, 48.0, -28.0, -18.0], [-5.0, 3.0, 28.0, 19.0, -12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=numpy.array([[[[4.0, -3.0, 6.0, -5.0, -4.0], [7.0, -3.0, 2.0, 0.0, -7.0], [5.0, 6.0, 0.0, -3.0, -7.0], [3.0, 2.0, -7.0, 6.0, -7.0]], [[0.0, 1.0, -1.0, 0.0, 1.0], [-6.0, 0.0, -4.0, 0.0, 1.0], [2.0, -4.0, -5.0, 7.0, -4.0], [-1.0, -3.0, -5.0, 5.0, 5.0]], [[0.0, 3.0, 2.0, -7.0, -7.0], [3.0, -7.0, -3.0, -2.0, 2.0], [0.0, 1.0, 6.0, -2.0, 7.0], [-6.0, -6.0, -3.0, -7.0, -3.0]]], [[[6.0, 5.0, 2.0, 7.0, -3.0], [1.0, 6.0, 6.0, -5.0, -4.0], [4.0, 0.0, -6.0, -5.0, -3.0], [2.0, 2.0, -2.0, 0.0, -3.0]], [[2.0, -5.0, -2.0, 4.0, -7.0], [2.0, 4.0, -4.0, 3.0, -5.0], [-2.0, -6.0, 0.0, -7.0, 2.0], [-6.0, 4.0, 1.0, -6.0, -1.0]], [[-7.0, 1.0, -3.0, -6.0, -6.0], [5.0, 0.0, 0.0, 2.0, 0.0], [1.0, -4.0, 5.0, 1.0, 1.0], [-2.0, 4.0, -6.0, -7.0, -3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[0.0, 0.0], [-7.0, -5.0], [-6.0, 4.0]], [[-2.0, 7.0], [0.0, -2.0], [3.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-33.0, -32.0, -18.0, 10.0, 23.0], [37.0, 30.0, 34.0, 28.0, -11.0], [-19.0, 10.0, 26.0, -24.0, -5.0], [33.0, 65.0, 39.0, -14.0, -20.0]], [[45.0, 51.0, 34.0, 19.0, -34.0], [40.0, 6.0, 58.0, -51.0, -15.0], [21.0, 40.0, 2.0, -65.0, 22.0], [9.0, -7.0, 3.0, -34.0, -53.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_array_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[-2.0, -5.0], [0.0, -2.0]], [[-5.0, 4.0], [6.0, 2.0]], [[0.0, -5.0], [-1.0, -6.0]], [[-3.0, 1.0], [-1.0, 7.0]], [[6.0, 0.0], [-6.0, -3.0]], [[0.0, 3.0], [0.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([3.0, 7.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-6.0, -15.0], [0.0, -6.0]], [[-15.0, 12.0], [18.0, 6.0]], [[0.0, -15.0], [-3.0, -18.0]], [[-9.0, 3.0], [-3.0, 21.0]], [[18.0, 0.0], [-18.0, -9.0]], [[0.0, 9.0], [0.0, 9.0]]], [[[-14.0, -35.0], [0.0, -14.0]], [[-35.0, 28.0], [42.0, 14.0]], [[0.0, -35.0], [-7.0, -42.0]], [[-21.0, 7.0], [-7.0, 49.0]], [[42.0, 0.0], [-42.0, -21.0]], [[0.0, 21.0], [0.0, 21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank4_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=numpy.array([[[[-6.0, 2.0], [0.0, 6.0]], [[7.0, -6.0], [6.0, 5.0]], [[0.0, 0.0], [5.0, 4.0]], [[0.0, 6.0], [0.0, 5.0]], [[-2.0, 3.0], [5.0, -6.0]], [[-7.0, 1.0], [-1.0, 4.0]]], [[[2.0, 7.0], [-4.0, 0.0]], [[-1.0, 2.0], [-6.0, -3.0]], [[0.0, -5.0], [-3.0, -4.0]], [[-4.0, 5.0], [5.0, -7.0]], [[2.0, 7.0], [6.0, -2.0]], [[1.0, 3.0], [0.0, -4.0]]], [[[-1.0, -4.0], [4.0, -3.0]], [[-3.0, -1.0], [-6.0, -6.0]], [[-1.0, -4.0], [2.0, -7.0]], [[-6.0, -1.0], [-1.0, -7.0]], [[1.0, 5.0], [5.0, 3.0]], [[-2.0, -4.0], [-7.0, 3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[7.0, -7.0], [1.0, -3.0], [-4.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-36.0, 37.0], [-20.0, 54.0]], [[60.0, -36.0], [60.0, 56.0]], [[4.0, 11.0], [24.0, 52.0]], [[20.0, 51.0], [9.0, 56.0]], [[-16.0, 8.0], [21.0, -56.0]], [[-40.0, 26.0], [21.0, 12.0]]], [[[40.0, -19.0], [-4.0, -30.0]], [[-34.0, 40.0], [0.0, -2.0]], [[4.0, 31.0], [-34.0, 12.0]], [[36.0, -53.0], [-11.0, 14.0]], [[4.0, -62.0], [-73.0, 36.0]], [[54.0, 0.0], [35.0, -28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-1.0, 5.0])
      s1=numpy.array(-3.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([3.0, -15.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[7.0, -4.0], [6.0, 2.0], [-2.0, 4.0]])
      s1=numpy.array([0.0, 3.0, -1.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([20.0, 2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-3.0, -4.0], [0.0, 1.0], [1.0, -6.0]], [[-5.0, 2.0], [7.0, -2.0], [-6.0, -2.0]]])
      s1=numpy.array([[7.0, 4.0, 6.0], [0.0, -6.0, -6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-21.0, -36.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[0.0, 4.0], [-7.0, 0.0], [-1.0, -4.0]], [[-4.0, -7.0], [7.0, -2.0], [2.0, -6.0]], [[-6.0, 3.0], [-1.0, -4.0], [0.0, -3.0]], [[-6.0, -7.0], [-2.0, 7.0], [6.0, -7.0]]], [[[7.0, 4.0], [-6.0, 1.0], [-4.0, -6.0]], [[5.0, 5.0], [-2.0, -6.0], [-5.0, -6.0]], [[-5.0, 3.0], [1.0, -4.0], [-6.0, 7.0]], [[0.0, 5.0], [2.0, 6.0], [-5.0, -1.0]]]])
      s1=numpy.array([[[6.0, 4.0, 0.0], [5.0, -6.0, -7.0], [0.0, -7.0, -3.0], [-3.0, -3.0, 3.0]], [[4.0, 4.0, 0.0], [-2.0, 3.0, 0.0], [4.0, -6.0, 6.0], [-6.0, 4.0, 5.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-146.0, 118.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, -3.0])
      s1=numpy.array([-2.0, 0.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.0, 0.0], [6.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[3.0, 5.0], [-4.0, -3.0], [0.0, 6.0]])
      s1=numpy.array([[-7.0, 5.0], [-3.0, -4.0], [-7.0, 1.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-9.0, 31.0], [-68.0, 43.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[1.0, 3.0], [0.0, 6.0], [-5.0, 2.0]], [[2.0, 4.0], [-7.0, 5.0], [-5.0, 7.0]]])
      s1=numpy.array([[[-7.0, 1.0], [-3.0, 1.0], [1.0, -5.0]], [[-1.0, 5.0], [0.0, 1.0], [-7.0, 1.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[21.0, 24.0], [-90.0, 31.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-6.0, 1.0], [1.0, 0.0], [4.0, 7.0]], [[-3.0, -4.0], [-3.0, -1.0], [7.0, 4.0]], [[-6.0, 5.0], [7.0, -3.0], [1.0, -5.0]], [[-5.0, -2.0], [-7.0, 2.0], [-3.0, 0.0]]], [[[6.0, -1.0], [0.0, -2.0], [0.0, 2.0]], [[-1.0, 6.0], [0.0, 1.0], [-2.0, 4.0]], [[-2.0, -7.0], [0.0, -1.0], [-1.0, -3.0]], [[0.0, 3.0], [3.0, -3.0], [1.0, -1.0]]]])
      s1=numpy.array([[[[-1.0, 5.0], [-2.0, 1.0], [6.0, -6.0]], [[2.0, 0.0], [-2.0, -6.0], [0.0, 0.0]], [[7.0, 1.0], [5.0, -2.0], [-3.0, -5.0]], [[-1.0, -6.0], [-5.0, 4.0], [-7.0, -4.0]]], [[[-5.0, 7.0], [5.0, 3.0], [7.0, 4.0]], [[-3.0, -2.0], [-2.0, 5.0], [5.0, -3.0]], [[-6.0, -1.0], [7.0, 2.0], [0.0, -7.0]], [[1.0, -5.0], [5.0, 2.0], [0.0, -2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[69.0, 17.0], [94.0, 8.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, -1.0])
      s1=numpy.array([[-3.0, 2.0, 6.0, 5.0, -5.0], [2.0, -3.0, -2.0, -3.0, -7.0], [-7.0, -6.0, -2.0, 7.0, 4.0], [0.0, -1.0, -2.0, -2.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[21.0, -14.0, -42.0, -35.0, 35.0], [-14.0, 21.0, 14.0, 21.0, 49.0], [49.0, 42.0, 14.0, -49.0, -28.0], [0.0, 7.0, 14.0, 14.0, 49.0]], [[3.0, -2.0, -6.0, -5.0, 5.0], [-2.0, 3.0, 2.0, 3.0, 7.0], [7.0, 6.0, 2.0, -7.0, -4.0], [0.0, 1.0, 2.0, 2.0, 7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-3.0, 0.0], [-3.0, 3.0], [-7.0, 2.0]])
      s1=numpy.array([[[-5.0, -7.0, 7.0, 2.0, -5.0], [0.0, -5.0, 3.0, 3.0, -2.0], [-3.0, 3.0, -5.0, 2.0, -4.0], [-7.0, 7.0, 0.0, -6.0, 5.0]], [[-2.0, -3.0, -3.0, 4.0, 2.0], [0.0, -2.0, 3.0, -1.0, 2.0], [-7.0, -1.0, -1.0, 0.0, 1.0], [-7.0, 3.0, 5.0, 0.0, 1.0]], [[-5.0, 2.0, -3.0, -5.0, 3.0], [3.0, 3.0, -1.0, 7.0, -1.0], [-7.0, -3.0, 7.0, -6.0, -2.0], [-1.0, 0.0, -7.0, 7.0, 0.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[56.0, 16.0, 9.0, 17.0, -12.0], [-21.0, 0.0, -11.0, -55.0, 7.0], [79.0, 15.0, -31.0, 36.0, 23.0], [49.0, -30.0, 34.0, -31.0, -18.0]], [[-16.0, -5.0, -15.0, 2.0, 12.0], [6.0, 0.0, 7.0, 11.0, 4.0], [-35.0, -9.0, 11.0, -12.0, -1.0], [-23.0, 9.0, 1.0, 14.0, 3.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-7.0, 7.0], [5.0, 0.0], [-1.0, -5.0]], [[-3.0, 3.0], [2.0, -4.0], [7.0, -4.0]]])
      s1=numpy.array([[[[-1.0, 0.0, -7.0, 3.0, 0.0], [4.0, -1.0, 1.0, -6.0, 2.0], [-5.0, 0.0, 4.0, -2.0, -2.0], [2.0, 2.0, 6.0, 1.0, 3.0]], [[-1.0, 2.0, 2.0, -7.0, -4.0], [7.0, -5.0, 2.0, 7.0, -3.0], [-7.0, -6.0, 6.0, 1.0, 0.0], [-5.0, -2.0, 4.0, 1.0, -4.0]], [[-7.0, -3.0, 5.0, 2.0, -3.0], [7.0, 3.0, 7.0, 4.0, 6.0], [-7.0, -2.0, -1.0, -4.0, 0.0], [0.0, -7.0, 6.0, -1.0, 4.0]]], [[[2.0, 2.0, -4.0, 2.0, 2.0], [-6.0, -5.0, 1.0, 4.0, 0.0], [6.0, -6.0, 2.0, 7.0, 3.0], [-4.0, -2.0, 0.0, -2.0, 7.0]], [[2.0, 1.0, 7.0, -3.0, 2.0], [-2.0, 5.0, 2.0, 7.0, 6.0], [2.0, -2.0, 3.0, -2.0, -6.0], [-3.0, -7.0, -1.0, -7.0, -1.0]], [[5.0, 0.0, 7.0, -6.0, 0.0], [6.0, -1.0, -7.0, -3.0, -2.0], [4.0, -6.0, -2.0, -1.0, 6.0], [6.0, 3.0, -6.0, -7.0, 4.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[42.0, 9.0, 129.0, -112.0, -19.0], [56.0, -3.0, -52.0, 54.0, -37.0], [21.0, -56.0, -11.0, -9.0, 35.0], [9.0, -4.0, -72.0, -58.0, -40.0]], [[6.0, 17.0, -142.0, 53.0, 13.0], [-41.0, -53.0, -5.0, -66.0, -32.0], [-6.0, 24.0, 35.0, 39.0, -5.0], [-10.0, 59.0, 40.0, 62.0, 10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, 4.0])
      s1=numpy.array([[[-1.0, 0.0], [1.0, 0.0]], [[1.0, -3.0], [-5.0, -5.0]], [[6.0, 0.0], [5.0, -2.0]], [[0.0, 5.0], [3.0, 3.0]], [[4.0, 6.0], [7.0, 1.0]], [[0.0, 2.0], [-7.0, -6.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-4.0, 0.0], [4.0, 0.0]], [[4.0, -12.0], [-20.0, -20.0]], [[24.0, 0.0], [20.0, -8.0]], [[0.0, 20.0], [12.0, 12.0]], [[16.0, 24.0], [28.0, 4.0]], [[0.0, 8.0], [-28.0, -24.0]]], [[[-4.0, 0.0], [4.0, 0.0]], [[4.0, -12.0], [-20.0, -20.0]], [[24.0, 0.0], [20.0, -8.0]], [[0.0, 20.0], [12.0, 12.0]], [[16.0, 24.0], [28.0, 4.0]], [[0.0, 8.0], [-28.0, -24.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[1.0, -7.0], [-3.0, -2.0], [5.0, 4.0]])
      s1=numpy.array([[[[-6.0, 4.0], [-3.0, 1.0]], [[2.0, -6.0], [2.0, 5.0]], [[5.0, -1.0], [-3.0, -1.0]], [[-7.0, 4.0], [-3.0, -2.0]], [[-3.0, 5.0], [-1.0, -4.0]], [[-2.0, -6.0], [-1.0, 5.0]]], [[[-1.0, 0.0], [7.0, -5.0]], [[0.0, 5.0], [-2.0, 3.0]], [[2.0, -5.0], [-6.0, -4.0]], [[-3.0, -5.0], [-2.0, -5.0]], [[-5.0, 6.0], [-6.0, -7.0]], [[6.0, -5.0], [4.0, 3.0]]], [[[6.0, -1.0], [4.0, 0.0]], [[-4.0, 4.0], [-4.0, -7.0]], [[6.0, 2.0], [1.0, 0.0]], [[-5.0, 3.0], [7.0, -5.0]], [[6.0, -6.0], [4.0, 7.0]], [[4.0, -1.0], [4.0, 6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[27.0, -1.0], [-4.0, 16.0]], [[-18.0, -1.0], [-12.0, -39.0]], [[29.0, 24.0], [20.0, 11.0]], [[-23.0, 34.0], [38.0, -12.0]], [[42.0, -43.0], [37.0, 52.0]], [[0.0, 4.0], [7.0, 26.0]]], [[[68.0, -32.0], [23.0, 3.0]], [[-30.0, 48.0], [-26.0, -69.0]], [[-15.0, 25.0], [37.0, 15.0]], [[35.0, -6.0], [53.0, 4.0]], [[55.0, -71.0], [35.0, 70.0]], [[18.0, 48.0], [15.0, -17.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(-3.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([6.0, -6.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-18.0, 18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([1.0, 6.0, -1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-6.0, 1.0], [-3.0, 2.0], [0.0, 3.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-24.0, 10.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Data(numpy.array([[1.0, 4.0, 1.0], [-2.0, 6.0, 2.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[2.0, -1.0], [-4.0, 2.0], [7.0, 0.0]], [[3.0, -5.0], [-3.0, 5.0], [-5.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-41.0, 51.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=Data(numpy.array([[[2.0, 3.0, 0.0], [-7.0, 6.0, 6.0], [-5.0, 6.0, 0.0], [2.0, -3.0, 2.0]], [[6.0, 7.0, 7.0], [3.0, 4.0, 0.0], [-3.0, 1.0, -1.0], [-5.0, 0.0, -5.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-1.0, -2.0], [-7.0, -7.0], [3.0, -3.0]], [[6.0, -5.0], [-5.0, -6.0], [-1.0, -1.0]], [[-2.0, 2.0], [-5.0, 1.0], [3.0, 6.0]], [[3.0, 7.0], [0.0, 7.0], [-1.0, -2.0]]], [[[-2.0, 6.0], [-3.0, 2.0], [7.0, -1.0]], [[-3.0, 0.0], [0.0, -6.0], [-6.0, -3.0]], [[4.0, -3.0], [-1.0, 2.0], [-6.0, 4.0]], [[0.0, -1.0], [-6.0, -2.0], [2.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-127.0, -16.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-3.0, -1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-5.0, 2.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[15.0, 5.0], [-6.0, -2.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([[4.0, -5.0], [-7.0, -2.0], [2.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[4.0, 1.0], [3.0, -1.0], [5.0, 5.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[5.0, -26.0], [21.0, -3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Data(numpy.array([[[5.0, 4.0], [-6.0, -5.0], [5.0, 4.0]], [[5.0, -7.0], [-5.0, 3.0], [-2.0, -7.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-4.0, -5.0], [-1.0, -3.0], [1.0, 0.0]], [[6.0, -3.0], [-2.0, 5.0], [-2.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[35.0, -41.0], [-61.0, -18.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      arg1=Data(numpy.array([[[[-4.0, -3.0], [-7.0, 7.0], [1.0, -7.0]], [[-6.0, 4.0], [-7.0, 5.0], [3.0, 7.0]], [[-2.0, 7.0], [0.0, 3.0], [0.0, -6.0]], [[7.0, -7.0], [0.0, -4.0], [1.0, 0.0]]], [[[-7.0, -2.0], [-6.0, -4.0], [-7.0, 0.0]], [[-4.0, -6.0], [-6.0, -1.0], [-6.0, 3.0]], [[-6.0, -2.0], [-1.0, -1.0], [-1.0, 0.0]], [[2.0, 0.0], [0.0, -3.0], [5.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-4.0, 0.0], [5.0, -1.0], [-6.0, -4.0]], [[-4.0, 0.0], [6.0, -6.0], [-3.0, 6.0]], [[-2.0, 4.0], [-3.0, 1.0], [1.0, 0.0]], [[0.0, -4.0], [0.0, -3.0], [-4.0, -6.0]]], [[[0.0, 3.0], [-4.0, 2.0], [5.0, -4.0]], [[-1.0, 7.0], [6.0, -3.0], [3.0, 4.0]], [[4.0, 7.0], [-1.0, 1.0], [5.0, -1.0]], [[5.0, 1.0], [-4.0, 0.0], [1.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-126.0, 83.0], [-48.0, 48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[5.0, 0.0, -7.0, -6.0, 3.0], [-2.0, 1.0, -3.0, 6.0, -3.0], [-5.0, -6.0, -5.0, -4.0, -4.0], [-4.0, -1.0, -6.0, -3.0, -5.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, 3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[15.0, 0.0, -21.0, -18.0, 9.0], [-6.0, 3.0, -9.0, 18.0, -9.0], [-15.0, -18.0, -15.0, -12.0, -12.0], [-12.0, -3.0, -18.0, -9.0, -15.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank3_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([[[2.0, -4.0, -5.0, -3.0, 5.0], [7.0, 2.0, -2.0, 2.0, -2.0], [1.0, -6.0, -1.0, -6.0, 7.0], [-1.0, 0.0, 0.0, 7.0, 0.0]], [[6.0, -2.0, 2.0, 0.0, 2.0], [0.0, -3.0, 2.0, -1.0, 6.0], [4.0, -1.0, 2.0, -6.0, -5.0], [-4.0, 0.0, 7.0, 1.0, 2.0]], [[-4.0, -3.0, 3.0, 6.0, -4.0], [3.0, 0.0, 1.0, 3.0, 7.0], [-4.0, -7.0, -4.0, 3.0, -4.0], [-4.0, 1.0, 0.0, 2.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-4.0, -1.0], [-3.0, -7.0], [-1.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-22.0, 25.0, 11.0, 6.0, -22.0], [-31.0, 1.0, 1.0, -8.0, -17.0], [-12.0, 34.0, 2.0, 39.0, -9.0], [20.0, -1.0, -21.0, -33.0, -6.0]], [[-16.0, 39.0, -30.0, -39.0, 9.0], [-28.0, 19.0, -19.0, -16.0, -89.0], [-1.0, 62.0, 15.0, 27.0, 56.0], [57.0, -7.0, -49.0, -28.0, -14.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      arg1=Data(numpy.array([[[[0.0, -7.0, -6.0, 0.0, 0.0], [-3.0, 0.0, 0.0, -5.0, 2.0], [0.0, -7.0, 4.0, 3.0, -1.0], [-1.0, -1.0, 4.0, 0.0, 2.0]], [[-1.0, 2.0, 3.0, 7.0, -7.0], [-6.0, 7.0, -1.0, -1.0, 5.0], [-5.0, 4.0, -4.0, 3.0, 6.0], [5.0, 2.0, 4.0, 4.0, -1.0]], [[2.0, -4.0, -4.0, -4.0, -3.0], [1.0, 7.0, -6.0, 2.0, -4.0], [-1.0, -2.0, 5.0, 2.0, 0.0], [7.0, 7.0, -6.0, 4.0, 0.0]]], [[[-2.0, -7.0, 5.0, -5.0, 5.0], [-2.0, 2.0, 5.0, 6.0, 7.0], [0.0, -2.0, -2.0, -6.0, 3.0], [-5.0, -4.0, -6.0, -5.0, 7.0]], [[0.0, -4.0, 0.0, -2.0, 4.0], [-3.0, -2.0, 2.0, -3.0, 1.0], [1.0, 5.0, 1.0, 4.0, -1.0], [3.0, -3.0, -3.0, -3.0, -1.0]], [[-2.0, -4.0, 3.0, 6.0, -5.0], [2.0, 2.0, -6.0, 2.0, -7.0], [-4.0, -4.0, 0.0, 0.0, -4.0], [-5.0, -1.0, -3.0, -6.0, 3.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[1.0, -5.0], [3.0, 1.0], [-1.0, 2.0]], [[2.0, -6.0], [6.0, 6.0], [-6.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[3.0, -11.0, -1.0, -33.0, 46.0], [-56.0, -6.0, 61.0, -28.0, 83.0], [16.0, 57.0, -11.0, 22.0, 41.0], [45.0, -22.0, 10.0, 16.0, -11.0]], [[19.0, 55.0, -11.0, 5.0, -9.0], [1.0, -7.0, -19.0, -30.0, -35.0], [7.0, 85.0, 4.0, 52.0, -5.0], [82.0, 29.0, -4.0, 36.0, -65.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_constData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[6.0, 4.0], [-2.0, 7.0]], [[6.0, 5.0], [-4.0, -6.0]], [[2.0, 2.0], [1.0, 2.0]], [[-1.0, -5.0], [0.0, 2.0]], [[-4.0, 2.0], [0.0, -2.0]], [[0.0, 0.0], [-1.0, 2.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, 1.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[24.0, 16.0], [-8.0, 28.0]], [[24.0, 20.0], [-16.0, -24.0]], [[8.0, 8.0], [4.0, 8.0]], [[-4.0, -20.0], [0.0, 8.0]], [[-16.0, 8.0], [0.0, -8.0]], [[0.0, 0.0], [-4.0, 8.0]]], [[[6.0, 4.0], [-2.0, 7.0]], [[6.0, 5.0], [-4.0, -6.0]], [[2.0, 2.0], [1.0, 2.0]], [[-1.0, -5.0], [0.0, 2.0]], [[-4.0, 2.0], [0.0, -2.0]], [[0.0, 0.0], [-1.0, 2.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank4_offset1(self):
      arg0=Symbol(shape=(3, 2))
      arg1=Data(numpy.array([[[[3.0, -4.0], [2.0, 3.0]], [[-1.0, 4.0], [0.0, 6.0]], [[1.0, -6.0], [2.0, 1.0]], [[-5.0, 4.0], [-1.0, -1.0]], [[1.0, 7.0], [0.0, -1.0]], [[0.0, -5.0], [-4.0, 7.0]]], [[[-3.0, -2.0], [0.0, 6.0]], [[0.0, 6.0], [-7.0, 2.0]], [[3.0, 3.0], [6.0, 0.0]], [[3.0, -2.0], [-7.0, 0.0]], [[-1.0, -4.0], [-3.0, -1.0]], [[-5.0, -3.0], [7.0, 5.0]]], [[[0.0, 0.0], [0.0, -6.0]], [[-6.0, -3.0], [-7.0, 1.0]], [[6.0, -7.0], [-5.0, 1.0]], [[4.0, 5.0], [-6.0, 4.0]], [[-1.0, -7.0], [1.0, -5.0]], [[0.0, 6.0], [4.0, -1.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-4.0, 1.0], [-6.0, 5.0], [4.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[6.0, 28.0], [-8.0, -72.0]], [[-20.0, -64.0], [14.0, -32.0]], [[2.0, -22.0], [-64.0, 0.0]], [[18.0, 16.0], [22.0, 20.0]], [[-2.0, -32.0], [22.0, -10.0]], [[30.0, 62.0], [-10.0, -62.0]]], [[[-12.0, -14.0], [2.0, 75.0]], [[41.0, 55.0], [14.0, 9.0]], [[-26.0, 58.0], [67.0, -6.0]], [[-18.0, -41.0], [6.0, -29.0]], [[3.0, 36.0], [-22.0, 29.0]], [[-25.0, -62.0], [3.0, 39.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(6.0)+(1-msk_arg1)*(2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([5.0, -5.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([30.0, -30.0])+(1.-msk_ref)*numpy.array([10.0, -10.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 0.0, -2.0])+(1.-msk_arg1)*numpy.array([4.0, 3.0, -6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-3.0, 7.0], [0.0, -6.0], [0.0, -2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, -31.0])+(1.-msk_ref)*numpy.array([-12.0, 22.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 7.0, 0.0], [-2.0, 1.0, 2.0]])+(1.-msk_arg1)*numpy.array([[7.0, -1.0, 6.0], [0.0, -5.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-3.0, 6.0], [0.0, 5.0], [-7.0, 3.0]], [[0.0, 5.0], [6.0, 2.0], [3.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([27.0, 9.0])+(1.-msk_ref)*numpy.array([-81.0, 69.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, -6.0, 2.0], [-3.0, -1.0, 6.0], [-4.0, 3.0, 0.0], [1.0, -6.0, -1.0]], [[-3.0, -6.0, -2.0], [-6.0, -6.0, -6.0], [4.0, 2.0, -3.0], [-6.0, -5.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 2.0, -5.0], [1.0, -1.0, 2.0], [0.0, -7.0, -4.0], [2.0, 3.0, -5.0]], [[-6.0, 6.0, -1.0], [2.0, 1.0, -7.0], [2.0, 1.0, 0.0], [-4.0, 4.0, -1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[7.0, -7.0], [-6.0, 5.0], [4.0, -7.0]], [[4.0, -1.0], [-1.0, -2.0], [-6.0, -3.0]], [[4.0, 7.0], [-3.0, -3.0], [-2.0, -7.0]], [[-6.0, 0.0], [-1.0, -4.0], [6.0, 5.0]]], [[[0.0, -7.0], [-6.0, 7.0], [0.0, 2.0]], [[0.0, -5.0], [-7.0, -5.0], [-1.0, 0.0]], [[-7.0, -2.0], [2.0, 5.0], [-3.0, -3.0]], [[-7.0, 4.0], [2.0, 6.0], [-2.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([40.0, -50.0])+(1.-msk_ref)*numpy.array([-44.0, 103.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 4.0])+(1.-msk_arg1)*numpy.array([7.0, -7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, -6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-20.0, 16.0], [30.0, -24.0]])+(1.-msk_ref)*numpy.array([[28.0, -28.0], [-42.0, 42.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 0.0], [0.0, -4.0], [4.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-3.0, 1.0], [0.0, 2.0], [0.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-5.0, -7.0], [-3.0, -5.0], [0.0, -1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, 12.0], [-18.0, 20.0]])+(1.-msk_ref)*numpy.array([[15.0, -11.0], [21.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 7.0], [-3.0, -4.0], [3.0, 7.0]], [[4.0, -2.0], [-7.0, -4.0], [-5.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -4.0], [-1.0, -6.0], [-1.0, 1.0]], [[1.0, -3.0], [3.0, 0.0], [5.0, -4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[7.0, 7.0], [-4.0, 4.0], [5.0, 0.0]], [[-7.0, -5.0], [1.0, 2.0], [6.0, -4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, 74.0], [2.0, 59.0]])+(1.-msk_ref)*numpy.array([[39.0, -2.0], [-9.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -2.0], [3.0, 5.0], [-4.0, 7.0]], [[7.0, -7.0], [-2.0, 3.0], [5.0, -1.0]], [[-1.0, 5.0], [-2.0, 1.0], [0.0, 1.0]], [[0.0, 3.0], [0.0, -2.0], [-5.0, 4.0]]], [[[-6.0, -2.0], [-2.0, -1.0], [7.0, -4.0]], [[-1.0, 1.0], [3.0, -6.0], [7.0, 4.0]], [[6.0, -5.0], [0.0, -1.0], [-3.0, 7.0]], [[0.0, 4.0], [-1.0, -4.0], [-5.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 5.0], [4.0, 4.0], [-2.0, 1.0]], [[7.0, 0.0], [5.0, 2.0], [5.0, -2.0]], [[3.0, -1.0], [5.0, 2.0], [0.0, -4.0]], [[-4.0, 4.0], [3.0, -2.0], [7.0, 6.0]]], [[[-2.0, 5.0], [-4.0, -1.0], [-3.0, -5.0]], [[3.0, -3.0], [-5.0, -7.0], [4.0, 0.0]], [[1.0, -6.0], [1.0, 5.0], [0.0, 5.0]], [[-4.0, -3.0], [1.0, 7.0], [-5.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[4.0, -5.0], [-7.0, 0.0], [-7.0, 4.0]], [[-5.0, -3.0], [1.0, 5.0], [-5.0, -7.0]], [[-2.0, -2.0], [5.0, 0.0], [-7.0, -7.0]], [[2.0, 6.0], [2.0, -3.0], [5.0, 5.0]]], [[[6.0, 5.0], [0.0, -1.0], [0.0, 0.0]], [[-7.0, 2.0], [-5.0, 7.0], [-1.0, 5.0]], [[6.0, 6.0], [-5.0, 6.0], [-7.0, 4.0]], [[6.0, 1.0], [0.0, -4.0], [5.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-127.0, -102.0], [-16.0, 99.0]])+(1.-msk_ref)*numpy.array([[-85.0, 63.0], [-34.0, 39.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, -3.0, 2.0, -6.0, 2.0], [-5.0, -3.0, 1.0, -4.0, 1.0], [5.0, -1.0, 6.0, -3.0, -2.0], [-7.0, 7.0, -7.0, -4.0, 0.0]])+(1.-msk_arg1)*numpy.array([[5.0, -4.0, 5.0, -3.0, 6.0], [3.0, -4.0, 2.0, 6.0, -2.0], [2.0, 7.0, -1.0, -6.0, 1.0], [-7.0, -1.0, -2.0, 7.0, -6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, -1.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-14.0, 21.0, -14.0, 42.0, -14.0], [35.0, 21.0, -7.0, 28.0, -7.0], [-35.0, 7.0, -42.0, 21.0, 14.0], [49.0, -49.0, 49.0, 28.0, 0.0]], [[-2.0, 3.0, -2.0, 6.0, -2.0], [5.0, 3.0, -1.0, 4.0, -1.0], [-5.0, 1.0, -6.0, 3.0, 2.0], [7.0, -7.0, 7.0, 4.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, 28.0, -35.0, 21.0, -42.0], [-21.0, 28.0, -14.0, -42.0, 14.0], [-14.0, -49.0, 7.0, 42.0, -7.0], [49.0, 7.0, 14.0, -49.0, 42.0]], [[-5.0, 4.0, -5.0, 3.0, -6.0], [-3.0, 4.0, -2.0, -6.0, 2.0], [-2.0, -7.0, 1.0, 6.0, -1.0], [7.0, 1.0, 2.0, -7.0, 6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 4.0, -7.0, -1.0, 0.0], [4.0, -7.0, 0.0, -5.0, 5.0], [3.0, -1.0, 1.0, 0.0, -6.0], [-6.0, 1.0, 0.0, -4.0, 0.0]], [[5.0, 3.0, 0.0, -5.0, 2.0], [-4.0, 1.0, -6.0, 3.0, -1.0], [0.0, 2.0, 2.0, 3.0, -3.0], [5.0, 2.0, -7.0, 4.0, 5.0]], [[-4.0, 6.0, -2.0, 0.0, -2.0], [-1.0, -5.0, -1.0, 6.0, -7.0], [2.0, -7.0, 7.0, 7.0, -6.0], [2.0, 4.0, 6.0, -6.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 4.0, -5.0, -4.0, -1.0], [-3.0, -7.0, 5.0, 6.0, 7.0], [7.0, -4.0, -7.0, 1.0, -2.0], [0.0, 3.0, 1.0, 5.0, -4.0]], [[-2.0, -7.0, -1.0, 0.0, 0.0], [2.0, -6.0, -2.0, 2.0, 5.0], [-6.0, 7.0, -6.0, 3.0, 0.0], [0.0, -1.0, 0.0, -3.0, 1.0]], [[-7.0, 0.0, -3.0, 3.0, 0.0], [4.0, -6.0, 2.0, -3.0, -3.0], [3.0, 7.0, -4.0, 3.0, -7.0], [2.0, 0.0, 2.0, 4.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, 3.0], [1.0, -5.0], [-7.0, 3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-16.0, -67.0, 63.0, 2.0, 16.0], [-25.0, 85.0, 1.0, -4.0, 13.0], [-35.0, 58.0, -54.0, -46.0, 81.0], [33.0, -33.0, -49.0, 74.0, -37.0]], [[-16.0, 15.0, -27.0, 22.0, -16.0], [29.0, -41.0, 27.0, -12.0, -1.0], [15.0, -34.0, 14.0, 6.0, -21.0], [-37.0, 5.0, 53.0, -50.0, -7.0]]])+(1.-msk_ref)*numpy.array([[[89.0, -35.0, 55.0, 7.0, 7.0], [-5.0, 85.0, -51.0, -19.0, -23.0], [-76.0, -14.0, 71.0, -25.0, 63.0], [-14.0, -22.0, -21.0, -66.0, 15.0]], [[-29.0, 47.0, -19.0, -3.0, -3.0], [-7.0, -9.0, 31.0, -1.0, -13.0], [60.0, -26.0, -3.0, -3.0, -27.0], [6.0, 14.0, 9.0, 42.0, -11.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 2.0, -2.0, 0.0, 7.0], [0.0, -6.0, 4.0, -7.0, -4.0], [-4.0, 6.0, -5.0, 6.0, 2.0], [-5.0, 0.0, -1.0, 4.0, 3.0]], [[0.0, 3.0, -4.0, 5.0, 3.0], [-2.0, -4.0, -5.0, 1.0, 6.0], [1.0, -6.0, 6.0, 6.0, -3.0], [-4.0, -3.0, -4.0, 5.0, -1.0]], [[5.0, -2.0, 1.0, 7.0, 1.0], [-6.0, 7.0, -3.0, 0.0, -6.0], [5.0, -6.0, -5.0, 6.0, -4.0], [0.0, 4.0, 1.0, 4.0, 0.0]]], [[[-6.0, 4.0, -3.0, 2.0, 0.0], [3.0, 7.0, -4.0, -4.0, 1.0], [-6.0, -6.0, 6.0, 2.0, -1.0], [-4.0, 7.0, -6.0, 4.0, -3.0]], [[-5.0, 0.0, -4.0, 6.0, -7.0], [-1.0, -7.0, 2.0, 5.0, 6.0], [-2.0, -5.0, 7.0, -6.0, -6.0], [7.0, 3.0, 2.0, -5.0, 6.0]], [[-1.0, 6.0, 4.0, 3.0, 3.0], [-5.0, 7.0, 2.0, -5.0, 4.0], [-6.0, 5.0, 7.0, 2.0, 2.0], [4.0, -3.0, -7.0, 0.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, 0.0, 7.0, 3.0, -1.0], [-3.0, 3.0, -7.0, -4.0, 0.0], [-7.0, -6.0, 0.0, -7.0, -6.0], [0.0, -7.0, 7.0, 7.0, 7.0]], [[7.0, -6.0, 0.0, 2.0, 5.0], [-5.0, 2.0, -1.0, -4.0, 2.0], [-5.0, -3.0, 0.0, -6.0, -5.0], [1.0, -4.0, -5.0, -6.0, -7.0]], [[0.0, -1.0, 1.0, 6.0, -2.0], [6.0, -5.0, -1.0, -5.0, 4.0], [-2.0, -6.0, -6.0, 2.0, 1.0], [-3.0, 0.0, 4.0, -5.0, 2.0]]], [[[-2.0, 5.0, 3.0, 5.0, -5.0], [2.0, 7.0, 1.0, -1.0, 5.0], [-4.0, 0.0, 4.0, 1.0, -5.0], [-4.0, 6.0, 2.0, -7.0, 0.0]], [[3.0, 5.0, -4.0, 5.0, -1.0], [5.0, -7.0, 6.0, 6.0, 6.0], [2.0, 3.0, -6.0, 3.0, 0.0], [3.0, -1.0, -2.0, -2.0, -1.0]], [[4.0, 4.0, -2.0, -2.0, -1.0], [4.0, -5.0, 7.0, 0.0, -1.0], [-4.0, 6.0, 0.0, -3.0, 6.0], [-3.0, -3.0, 0.0, -3.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[5.0, 4.0], [-2.0, 0.0], [-7.0, 4.0]], [[-5.0, -1.0], [-1.0, -1.0], [7.0, -4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-7.0, 40.0, 38.0, -54.0, 50.0], [-3.0, -50.0, 83.0, -57.0, 27.0], [-67.0, 154.0, 10.0, -14.0, 69.0], [24.0, -81.0, -25.0, -33.0, 26.0]], [[35.0, -28.0, -13.0, 8.0, 27.0], [-6.0, -24.0, -2.0, -9.0, -63.0], [36.0, -9.0, -81.0, 44.0, -9.0], [-39.0, 18.0, 32.0, 33.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[41.0, 17.0, 3.0, -75.0, 18.0], [-34.0, -17.0, 12.0, 22.0, -70.0], [-21.0, 57.0, 28.0, -66.0, 40.0], [15.0, -77.0, 9.0, 98.0, 50.0]], [[-1.0, -30.0, 41.0, 34.0, -2.0], [-11.0, 12.0, -67.0, -41.0, 9.0], [-18.0, -75.0, -22.0, -12.0, -39.0], [1.0, -21.0, 44.0, 29.0, 29.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank1_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 2.0], [-2.0, -1.0]], [[4.0, 7.0], [6.0, -2.0]], [[0.0, 4.0], [-3.0, -1.0]], [[2.0, -7.0], [-5.0, -2.0]], [[3.0, 2.0], [-3.0, 7.0]], [[-1.0, -3.0], [2.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, -6.0], [-3.0, 4.0]], [[-5.0, -2.0], [6.0, 4.0]], [[0.0, -3.0], [-5.0, -1.0]], [[0.0, -1.0], [6.0, -5.0]], [[-5.0, -1.0], [0.0, -6.0]], [[1.0, 3.0], [0.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([3.0, -6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0], [-6.0, -3.0]], [[12.0, 21.0], [18.0, -6.0]], [[0.0, 12.0], [-9.0, -3.0]], [[6.0, -21.0], [-15.0, -6.0]], [[9.0, 6.0], [-9.0, 21.0]], [[-3.0, -9.0], [6.0, -18.0]]], [[[-6.0, -12.0], [12.0, 6.0]], [[-24.0, -42.0], [-36.0, 12.0]], [[0.0, -24.0], [18.0, 6.0]], [[-12.0, 42.0], [30.0, 12.0]], [[-18.0, -12.0], [18.0, -42.0]], [[6.0, 18.0], [-12.0, 36.0]]]])+(1.-msk_ref)*numpy.array([[[[-3.0, -18.0], [-9.0, 12.0]], [[-15.0, -6.0], [18.0, 12.0]], [[0.0, -9.0], [-15.0, -3.0]], [[0.0, -3.0], [18.0, -15.0]], [[-15.0, -3.0], [0.0, -18.0]], [[3.0, 9.0], [0.0, -15.0]]], [[[6.0, 36.0], [18.0, -24.0]], [[30.0, 12.0], [-36.0, -24.0]], [[0.0, 18.0], [30.0, 6.0]], [[0.0, 6.0], [-36.0, 30.0]], [[30.0, 6.0], [0.0, 36.0]], [[-6.0, -18.0], [0.0, 30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -6.0], [0.0, -3.0]], [[0.0, 0.0], [-6.0, 4.0]], [[7.0, 5.0], [-3.0, -1.0]], [[-1.0, 0.0], [-2.0, 0.0]], [[-3.0, -3.0], [4.0, 2.0]], [[-1.0, 0.0], [6.0, -4.0]]], [[[-2.0, 0.0], [4.0, 3.0]], [[6.0, 3.0], [2.0, -4.0]], [[-4.0, -1.0], [7.0, 3.0]], [[0.0, -5.0], [-3.0, 6.0]], [[-5.0, -3.0], [-6.0, 1.0]], [[2.0, 3.0], [-6.0, -5.0]]], [[[0.0, -7.0], [-4.0, -7.0]], [[3.0, 0.0], [-7.0, -6.0]], [[-4.0, 3.0], [2.0, 4.0]], [[3.0, -4.0], [-1.0, 1.0]], [[-4.0, 5.0], [0.0, -7.0]], [[1.0, 6.0], [7.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, -1.0], [0.0, 0.0]], [[2.0, 1.0], [-4.0, 0.0]], [[-4.0, -6.0], [3.0, 4.0]], [[-5.0, 5.0], [-2.0, 1.0]], [[4.0, -4.0], [-6.0, 3.0]], [[-7.0, 2.0], [6.0, 7.0]]], [[[-7.0, -2.0], [3.0, 5.0]], [[7.0, 0.0], [-7.0, 4.0]], [[5.0, 0.0], [1.0, 4.0]], [[5.0, 5.0], [-1.0, 0.0]], [[2.0, 3.0], [-4.0, 2.0]], [[-1.0, 5.0], [1.0, -4.0]]], [[[-7.0, -6.0], [-7.0, 5.0]], [[-6.0, 0.0], [0.0, 2.0]], [[0.0, 3.0], [0.0, -1.0]], [[2.0, 0.0], [-5.0, -5.0]], [[6.0, 2.0], [-7.0, -2.0]], [[0.0, 5.0], [-7.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[5.0, 3.0], [1.0, 0.0], [3.0, 2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, -51.0], [-8.0, -33.0]], [[15.0, 3.0], [-49.0, -2.0]], [[19.0, 33.0], [-2.0, 10.0]], [[4.0, -17.0], [-16.0, 9.0]], [[-32.0, -3.0], [14.0, -10.0]], [[0.0, 21.0], [45.0, -37.0]]], [[[3.0, -32.0], [-8.0, -23.0]], [[6.0, 0.0], [-32.0, 0.0]], [[13.0, 21.0], [-5.0, 5.0]], [[3.0, -8.0], [-8.0, 2.0]], [[-17.0, 1.0], [12.0, -8.0]], [[-1.0, 12.0], [32.0, -20.0]]]])+(1.-msk_ref)*numpy.array([[[[-38.0, -25.0], [-18.0, 20.0]], [[-1.0, 5.0], [-27.0, 10.0]], [[-15.0, -21.0], [16.0, 21.0]], [[-14.0, 30.0], [-26.0, -10.0]], [[40.0, -11.0], [-55.0, 11.0]], [[-36.0, 30.0], [10.0, 10.0]]], [[[-20.0, -15.0], [-14.0, 10.0]], [[-6.0, 3.0], [-12.0, 4.0]], [[-12.0, -12.0], [9.0, 10.0]], [[-11.0, 15.0], [-16.0, -7.0]], [[24.0, -8.0], [-32.0, 5.0]], [[-21.0, 16.0], [4.0, 7.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_float_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[0.0, 2.0, -1.0, -2.0, 4.0], [-3.0, 3.0, 7.0, 3.0, -4.0], [4.0, -2.0, 1.0, 7.0, -1.0], [2.0, 1.0, 5.0, -1.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.0, 10.0, -5.0, -10.0, 20.0], [-15.0, 15.0, 35.0, 15.0, -20.0], [20.0, -10.0, 5.0, 35.0, -5.0], [10.0, 5.0, 25.0, -5.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[6.0, -4.0, 0.0, 7.0, 3.0], [5.0, 5.0, -7.0, 6.0, 0.0], [4.0, -6.0, 4.0, 7.0, -3.0], [-1.0, 4.0, 0.0, 0.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-18.0, 12.0, -0.0, -21.0, -9.0], [-15.0, -15.0, 21.0, -18.0, -0.0], [-12.0, 18.0, -12.0, -21.0, 9.0], [3.0, -12.0, -0.0, -0.0, 21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank1_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=numpy.array([5.0, 5.0, -6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, 0.0, 6.0, -2.0, -5.0], [-6.0, 6.0, 1.0, -3.0, 6.0], [-5.0, 5.0, -1.0, 7.0, -4.0], [1.0, 4.0, 5.0, 7.0, -2.0]], [[-4.0, -5.0, 0.0, 0.0, -5.0], [-1.0, -1.0, 5.0, 6.0, 0.0], [7.0, 0.0, -2.0, -7.0, 3.0], [1.0, -7.0, -1.0, 3.0, -1.0]], [[0.0, 3.0, 4.0, 0.0, -1.0], [-6.0, -6.0, 3.0, 7.0, 6.0], [-4.0, 6.0, 0.0, 0.0, -2.0], [-6.0, 5.0, 4.0, 1.0, -6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-10.0, -43.0, 6.0, -10.0, -44.0], [1.0, 61.0, 12.0, -27.0, -6.0], [34.0, -11.0, -15.0, 0.0, 7.0], [46.0, -45.0, -4.0, 44.0, 21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[-3.0, 0.0, -4.0], [-2.0, 2.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, 6.0, -1.0, 3.0, -3.0], [1.0, 4.0, -4.0, -4.0, -7.0], [2.0, -5.0, -4.0, 2.0, -2.0], [-6.0, 5.0, 1.0, -4.0, 7.0]], [[3.0, 1.0, -6.0, -7.0, -1.0], [-5.0, 3.0, 6.0, 2.0, -2.0], [-7.0, -1.0, -1.0, -7.0, 7.0], [-6.0, -1.0, 1.0, 3.0, 2.0]], [[-2.0, 1.0, -2.0, -5.0, 0.0], [-4.0, -6.0, -3.0, 3.0, 0.0], [4.0, 0.0, 2.0, 3.0, -6.0], [7.0, 7.0, 2.0, 4.0, 5.0]]], [[[0.0, -1.0, -2.0, 1.0, 2.0], [0.0, 4.0, -6.0, 7.0, 3.0], [2.0, 3.0, 7.0, 1.0, 2.0], [0.0, 6.0, -2.0, -1.0, -3.0]], [[-3.0, 2.0, -1.0, 7.0, -2.0], [-6.0, 5.0, 0.0, 5.0, -3.0], [-1.0, -6.0, 1.0, 5.0, -3.0], [7.0, 6.0, 0.0, 3.0, -7.0]], [[6.0, -7.0, -5.0, 7.0, -1.0], [0.0, -6.0, -1.0, 5.0, 2.0], [-7.0, 5.0, 5.0, 5.0, -7.0], [-5.0, -7.0, -3.0, 6.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[59.0, -65.0, -22.0, 72.0, -6.0], [1.0, -28.0, 29.0, 31.0, 23.0], [-77.0, 32.0, 27.0, 25.0, -29.0], [-31.0, -92.0, -28.0, 46.0, -98.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([6.0, 4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[7.0, 3.0, -6.0, -3.0, -1.0], [7.0, 0.0, -3.0, 6.0, -7.0], [7.0, 0.0, 0.0, 0.0, 6.0], [6.0, -7.0, -4.0, 0.0, 5.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[42.0, 28.0], [18.0, 12.0], [-36.0, -24.0], [-18.0, -12.0], [-6.0, -4.0]], [[42.0, 28.0], [0.0, 0.0], [-18.0, -12.0], [36.0, 24.0], [-42.0, -28.0]], [[42.0, 28.0], [0.0, 0.0], [0.0, 0.0], [0.0, 0.0], [36.0, 24.0]], [[36.0, 24.0], [-42.0, -28.0], [-24.0, -16.0], [0.0, 0.0], [30.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank2_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=numpy.array([[-1.0, -3.0], [4.0, 3.0], [1.0, 3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, -2.0, 7.0, -4.0, -7.0], [-5.0, 1.0, 4.0, 0.0, -4.0], [2.0, 0.0, 6.0, -5.0, 3.0], [0.0, -2.0, 0.0, -6.0, 5.0]], [[5.0, 0.0, 4.0, 5.0, 0.0], [1.0, -6.0, 7.0, 0.0, 7.0], [-6.0, 5.0, 6.0, 5.0, -2.0], [7.0, -3.0, 0.0, -4.0, 1.0]], [[0.0, -2.0, -5.0, -2.0, 5.0], [4.0, 0.0, -3.0, 5.0, -1.0], [1.0, -5.0, -3.0, 6.0, 1.0], [-2.0, -3.0, -7.0, 0.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[15.0, 0.0], [0.0, 0.0], [4.0, -24.0], [22.0, 21.0], [12.0, 36.0]], [[13.0, 30.0], [-25.0, -21.0], [21.0, 0.0], [5.0, 15.0], [31.0, 30.0]], [[-25.0, -21.0], [15.0, 0.0], [15.0, -9.0], [31.0, 48.0], [-10.0, -12.0]], [[26.0, 15.0], [-13.0, -12.0], [-7.0, -21.0], [-10.0, 6.0], [3.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[[5.0, 0.0], [3.0, 0.0], [4.0, -4.0]], [[4.0, 5.0], [7.0, 4.0], [-7.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-3.0, -1.0, -3.0, 5.0, 2.0], [-4.0, 3.0, -5.0, 6.0, 1.0], [4.0, -3.0, 4.0, 5.0, 1.0], [3.0, 7.0, 3.0, 5.0, -7.0]], [[6.0, -1.0, 5.0, -7.0, -2.0], [0.0, 5.0, -1.0, -7.0, -6.0], [-7.0, -7.0, -7.0, -7.0, 6.0], [4.0, -6.0, -2.0, -2.0, 5.0]], [[0.0, -3.0, 7.0, -4.0, -5.0], [7.0, 5.0, 6.0, 4.0, 7.0], [3.0, 1.0, 0.0, 2.0, -3.0], [6.0, 3.0, 7.0, 1.0, -5.0]]], [[[0.0, 3.0, 1.0, 5.0, -1.0], [6.0, 3.0, 2.0, -5.0, -4.0], [7.0, -7.0, -6.0, 6.0, 5.0], [-3.0, 3.0, -2.0, -5.0, 1.0]], [[6.0, 0.0, 6.0, -3.0, 0.0], [3.0, -5.0, 6.0, 0.0, 5.0], [0.0, 3.0, 2.0, -3.0, 0.0], [2.0, 5.0, 1.0, -1.0, 7.0]], [[1.0, -7.0, -2.0, 6.0, -5.0], [-3.0, 1.0, 7.0, 5.0, -6.0], [3.0, -3.0, -2.0, -4.0, 6.0], [-7.0, 2.0, -1.0, 7.0, 3.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[38.0, 26.0], [41.0, 13.0], [88.0, -3.0], [-55.0, 41.0], [15.0, 5.0]], [[74.0, 8.0], [20.0, -23.0], [-3.0, 24.0], [-30.0, -31.0], [76.0, -40.0]], [[18.0, 29.0], [-18.0, -33.0], [3.0, -26.0], [43.0, 2.0], [-11.0, 49.0]], [[102.0, -45.0], [62.0, 27.0], [43.0, -36.0], [-53.0, -19.0], [-8.0, 59.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_array_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[3.0, -3.0, 0.0, -7.0, 0.0], [5.0, 4.0, 7.0, 1.0, -6.0], [-2.0, 1.0, 1.0, -1.0, -1.0], [-1.0, -3.0, 0.0, 6.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-1.0, 2.0, 6.0, -1.0, 1.0], [-6.0, 6.0, 6.0, 0.0, 3.0], [-2.0, 1.0, 0.0, 1.0, 0.0], [0.0, -1.0, 5.0, 6.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-3.0, 3.0, 0.0, 7.0, 0.0], [-5.0, -4.0, -7.0, -1.0, 6.0], [2.0, -1.0, -1.0, 1.0, 1.0], [1.0, 3.0, 0.0, -6.0, -7.0]], [[6.0, -6.0, 0.0, -14.0, 0.0], [10.0, 8.0, 14.0, 2.0, -12.0], [-4.0, 2.0, 2.0, -2.0, -2.0], [-2.0, -6.0, 0.0, 12.0, 14.0]], [[18.0, -18.0, 0.0, -42.0, 0.0], [30.0, 24.0, 42.0, 6.0, -36.0], [-12.0, 6.0, 6.0, -6.0, -6.0], [-6.0, -18.0, 0.0, 36.0, 42.0]], [[-3.0, 3.0, 0.0, 7.0, 0.0], [-5.0, -4.0, -7.0, -1.0, 6.0], [2.0, -1.0, -1.0, 1.0, 1.0], [1.0, 3.0, 0.0, -6.0, -7.0]], [[3.0, -3.0, 0.0, -7.0, 0.0], [5.0, 4.0, 7.0, 1.0, -6.0], [-2.0, 1.0, 1.0, -1.0, -1.0], [-1.0, -3.0, 0.0, 6.0, 7.0]]], [[[-18.0, 18.0, 0.0, 42.0, 0.0], [-30.0, -24.0, -42.0, -6.0, 36.0], [12.0, -6.0, -6.0, 6.0, 6.0], [6.0, 18.0, 0.0, -36.0, -42.0]], [[18.0, -18.0, 0.0, -42.0, 0.0], [30.0, 24.0, 42.0, 6.0, -36.0], [-12.0, 6.0, 6.0, -6.0, -6.0], [-6.0, -18.0, 0.0, 36.0, 42.0]], [[18.0, -18.0, 0.0, -42.0, 0.0], [30.0, 24.0, 42.0, 6.0, -36.0], [-12.0, 6.0, 6.0, -6.0, -6.0], [-6.0, -18.0, 0.0, 36.0, 42.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[9.0, -9.0, 0.0, -21.0, 0.0], [15.0, 12.0, 21.0, 3.0, -18.0], [-6.0, 3.0, 3.0, -3.0, -3.0], [-3.0, -9.0, 0.0, 18.0, 21.0]]], [[[-6.0, 6.0, 0.0, 14.0, 0.0], [-10.0, -8.0, -14.0, -2.0, 12.0], [4.0, -2.0, -2.0, 2.0, 2.0], [2.0, 6.0, 0.0, -12.0, -14.0]], [[3.0, -3.0, 0.0, -7.0, 0.0], [5.0, 4.0, 7.0, 1.0, -6.0], [-2.0, 1.0, 1.0, -1.0, -1.0], [-1.0, -3.0, 0.0, 6.0, 7.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[3.0, -3.0, 0.0, -7.0, 0.0], [5.0, 4.0, 7.0, 1.0, -6.0], [-2.0, 1.0, 1.0, -1.0, -1.0], [-1.0, -3.0, 0.0, 6.0, 7.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-3.0, 3.0, 0.0, 7.0, 0.0], [-5.0, -4.0, -7.0, -1.0, 6.0], [2.0, -1.0, -1.0, 1.0, 1.0], [1.0, 3.0, 0.0, -6.0, -7.0]], [[15.0, -15.0, 0.0, -35.0, 0.0], [25.0, 20.0, 35.0, 5.0, -30.0], [-10.0, 5.0, 5.0, -5.0, -5.0], [-5.0, -15.0, 0.0, 30.0, 35.0]], [[18.0, -18.0, 0.0, -42.0, 0.0], [30.0, 24.0, 42.0, 6.0, -36.0], [-12.0, 6.0, 6.0, -6.0, -6.0], [-6.0, -18.0, 0.0, 36.0, 42.0]], [[-18.0, 18.0, 0.0, 42.0, 0.0], [-30.0, -24.0, -42.0, -6.0, 36.0], [12.0, -6.0, -6.0, 6.0, 6.0], [6.0, 18.0, 0.0, -36.0, -42.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank3_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=numpy.array([[[6.0, -7.0, 6.0, 7.0, 7.0], [-2.0, 0.0, -6.0, 0.0, 3.0], [0.0, -6.0, 4.0, -5.0, 7.0], [0.0, -6.0, 6.0, -3.0, 7.0]], [[-2.0, 0.0, -7.0, 3.0, -3.0], [6.0, 0.0, -6.0, -1.0, 7.0], [1.0, 4.0, 0.0, 7.0, 2.0], [4.0, 7.0, -7.0, -4.0, 2.0]], [[-2.0, -5.0, -3.0, 2.0, -5.0], [3.0, 1.0, 7.0, -5.0, -4.0], [2.0, 1.0, -3.0, -1.0, -3.0], [0.0, 1.0, -1.0, -2.0, -1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[7.0, 6.0, -1.0, 3.0, -6.0], [-6.0, -1.0, 0.0, 0.0, 7.0], [-3.0, -5.0, -4.0, -5.0, -2.0], [-7.0, -5.0, 0.0, -3.0, -7.0]], [[-4.0, 0.0, 7.0, -1.0, 1.0], [-7.0, 2.0, 7.0, -3.0, -2.0], [-1.0, -4.0, -4.0, -2.0, -1.0], [-7.0, -6.0, 0.0, -6.0, 3.0]], [[6.0, -3.0, 0.0, 3.0, 0.0], [5.0, -5.0, 3.0, 0.0, -1.0], [0.0, -5.0, -1.0, -3.0, -3.0], [-7.0, 5.0, -4.0, 7.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[38.0, -79.0, 52.0, 49.0, 31.0], [-20.0, 6.0, 24.0, -26.0, -31.0], [8.0, -52.0, 10.0, -69.0, 23.0], [-16.0, -64.0, 64.0, -17.0, 35.0]], [[42.0, -27.0, 45.0, 36.0, 57.0], [-21.0, -3.0, -57.0, 15.0, 30.0], [-6.0, -39.0, 33.0, -27.0, 51.0], [0.0, -39.0, 39.0, -12.0, 45.0]], [[-20.0, 7.0, -55.0, 14.0, -28.0], [44.0, 0.0, -36.0, -7.0, 46.0], [7.0, 34.0, -4.0, 54.0, 7.0], [28.0, 55.0, -55.0, -25.0, 7.0]], [[14.0, -36.0, 16.0, 24.0, 9.0], [-3.0, 3.0, 9.0, -14.0, -10.0], [5.0, -19.0, 3.0, -25.0, 10.0], [-4.0, -22.0, 22.0, -11.0, 16.0]], [[-38.0, 42.0, -43.0, -39.0, -45.0], [18.0, 0.0, 30.0, -1.0, -11.0], [1.0, 40.0, -24.0, 37.0, -40.0], [4.0, 43.0, -43.0, 14.0, -40.0]]], [[[-32.0, 17.0, -2.0, -53.0, -46.0], [-15.0, 5.0, 113.0, -18.0, -87.0], [3.0, 13.0, -39.0, -24.0, -71.0], [-28.0, -8.0, 8.0, 36.0, -61.0]], [[0.0, 32.0, -5.0, -11.0, 12.0], [-1.0, -5.0, -41.0, 23.0, 31.0], [-8.0, 9.0, 11.0, 24.0, 12.0], [8.0, 15.0, -15.0, 5.0, 2.0]], [[-20.0, -15.0, -58.0, 27.0, -36.0], [51.0, 3.0, -21.0, -22.0, 37.0], [13.0, 31.0, -9.0, 46.0, 5.0], [28.0, 52.0, -52.0, -34.0, 11.0]], [[6.0, 0.0, 21.0, -9.0, 9.0], [-18.0, 0.0, 18.0, 3.0, -21.0], [-3.0, -12.0, 0.0, -21.0, -6.0], [-12.0, -21.0, 21.0, 12.0, -6.0]], [[48.0, -44.0, 59.0, 41.0, 60.0], [-29.0, -1.0, -37.0, 7.0, 11.0], [-4.0, -51.0, 31.0, -48.0, 48.0], [-8.0, -57.0, 57.0, -11.0, 46.0]]], [[[-16.0, 21.0, -11.0, -24.0, -18.0], [0.0, 0.0, 24.0, 1.0, -16.0], [-1.0, 14.0, -12.0, 8.0, -23.0], [-4.0, 11.0, -11.0, 13.0, -23.0]], [[-12.0, 60.0, 13.0, -57.0, 2.0], [-29.0, -5.0, 19.0, 29.0, -23.0], [-14.0, 9.0, -5.0, 2.0, -28.0], [-16.0, -3.0, 3.0, 41.0, -38.0]], [[-14.0, 33.0, 7.0, -42.0, -11.0], [-19.0, -1.0, 41.0, 9.0, -36.0], [-6.0, 7.0, -13.0, -7.0, -33.0], [-16.0, -5.0, 5.0, 30.0, -35.0]], [[-20.0, 50.0, -7.0, -47.0, -14.0], [-11.0, -3.0, 21.0, 17.0, -17.0], [-8.0, 19.0, -11.0, 14.0, -30.0], [-8.0, 13.0, -13.0, 29.0, -36.0]], [[-4.0, 29.0, 4.0, -23.0, 4.0], [-11.0, -3.0, -3.0, 16.0, -1.0], [-7.0, 5.0, 1.0, 6.0, -7.0], [-4.0, 2.0, -2.0, 16.0, -13.0]]], [[[-14.0, 84.0, 28.0, -84.0, 7.0], [-49.0, -7.0, 35.0, 42.0, -42.0], [-21.0, 7.0, -7.0, -7.0, -42.0], [-28.0, -14.0, 14.0, 63.0, -56.0]], [[-28.0, 10.0, -3.0, -43.0, -42.0], [-11.0, 5.0, 101.0, -19.0, -77.0], [4.0, 11.0, -35.0, -22.0, -62.0], [-24.0, -7.0, 7.0, 29.0, -52.0]], [[8.0, 20.0, 12.0, -8.0, 20.0], [-12.0, -4.0, -28.0, 20.0, 16.0], [-8.0, -4.0, 12.0, 4.0, 12.0], [0.0, -4.0, 4.0, 8.0, 4.0]], [[-20.0, -14.0, 3.0, -25.0, -38.0], [-9.0, 7.0, 103.0, -29.0, -79.0], [8.0, 1.0, -33.0, -34.0, -54.0], [-24.0, -17.0, 17.0, 19.0, -40.0]], [[-50.0, 44.0, -66.0, -38.0, -63.0], [35.0, 1.0, 31.0, -8.0, -4.0], [5.0, 55.0, -31.0, 55.0, -46.0], [12.0, 64.0, -64.0, 7.0, -44.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=numpy.array([[[[2.0, -3.0, -2.0, -2.0, 6.0], [-7.0, 3.0, 6.0, -7.0, 2.0], [3.0, 3.0, 3.0, 4.0, -4.0], [-1.0, 4.0, 6.0, -2.0, 6.0]], [[3.0, -6.0, -4.0, -7.0, -6.0], [-4.0, 2.0, 3.0, 3.0, -1.0], [5.0, 5.0, 0.0, 7.0, -2.0], [5.0, -5.0, -3.0, -3.0, 0.0]], [[-6.0, 7.0, -7.0, -2.0, 0.0], [-2.0, 6.0, 4.0, -7.0, -1.0], [1.0, 2.0, -6.0, -2.0, 3.0], [7.0, 4.0, 0.0, 4.0, -4.0]]], [[[0.0, 1.0, -5.0, -4.0, 2.0], [4.0, 4.0, 6.0, 7.0, -1.0], [5.0, -2.0, -3.0, 0.0, -3.0], [0.0, 0.0, -6.0, -7.0, 4.0]], [[-3.0, -3.0, 7.0, 4.0, 4.0], [1.0, -6.0, 3.0, -4.0, -1.0], [-1.0, 6.0, 7.0, -2.0, 5.0], [-6.0, -4.0, 6.0, -1.0, 3.0]], [[-2.0, 5.0, 1.0, 3.0, -1.0], [0.0, 7.0, 5.0, -2.0, 7.0], [-3.0, 6.0, -2.0, 3.0, 6.0], [0.0, 1.0, -6.0, 1.0, 1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, -1.0, 3.0, -5.0, -3.0], [-1.0, -6.0, 2.0, -3.0, -7.0], [4.0, -4.0, 5.0, 7.0, -2.0], [4.0, 0.0, 5.0, 3.0, 1.0]], [[-7.0, 5.0, 6.0, -7.0, 7.0], [-6.0, -3.0, 3.0, -1.0, -7.0], [0.0, 7.0, -3.0, 0.0, 0.0], [-4.0, 5.0, -2.0, -4.0, 0.0]], [[2.0, 7.0, -3.0, -1.0, 6.0], [-3.0, 0.0, 3.0, 5.0, 6.0], [0.0, -3.0, 0.0, 1.0, 6.0], [-3.0, -1.0, 7.0, -5.0, -6.0]]], [[[-4.0, 0.0, 4.0, 1.0, 4.0], [-3.0, 2.0, -1.0, -2.0, 7.0], [2.0, -7.0, 7.0, 7.0, -6.0], [5.0, -7.0, 6.0, -5.0, -4.0]], [[0.0, 0.0, -5.0, 4.0, -6.0], [-6.0, -3.0, -5.0, -1.0, -3.0], [1.0, -7.0, -1.0, 0.0, -1.0], [-7.0, -2.0, 1.0, -6.0, 0.0]], [[5.0, -5.0, 0.0, 6.0, 2.0], [6.0, 0.0, -1.0, 4.0, 2.0], [-1.0, -4.0, 1.0, -2.0, 3.0], [-3.0, 1.0, 0.0, 0.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-53.0, 92.0, 49.0, 86.0, -1.0], [43.0, 2.0, -42.0, -38.0, 34.0], [-83.0, -8.0, -25.0, -58.0, 82.0], [-16.0, 28.0, -15.0, 72.0, -49.0]], [[-19.0, -3.0, -72.0, -62.0, -31.0], [-27.0, 14.0, 12.0, -17.0, -49.0], [44.0, 6.0, -35.0, 2.0, -15.0], [75.0, -6.0, 9.0, 10.0, -39.0]], [[57.0, -47.0, -64.0, -78.0, -30.0], [-28.0, 49.0, 33.0, 66.0, 4.0], [61.0, -5.0, -20.0, 70.0, -70.0], [36.0, -10.0, -54.0, -59.0, 31.0]], [[-49.0, 69.0, 74.0, 91.0, 24.0], [73.0, -13.0, -7.0, 0.0, 35.0], [-68.0, 6.0, 4.0, -57.0, 84.0], [-61.0, 1.0, -27.0, 22.0, -4.0]], [[-7.0, 41.0, -124.0, -89.0, -78.0], [-9.0, 107.0, 43.0, 48.0, -3.0], [52.0, 6.0, -103.0, 43.0, -14.0], [116.0, 3.0, -111.0, -11.0, -42.0]]], [[[4.0, 63.0, 26.0, 56.0, -6.0], [19.0, 33.0, -42.0, 1.0, 58.0], [-63.0, -33.0, -30.0, -10.0, 22.0], [-14.0, 44.0, -42.0, 41.0, -18.0]], [[-12.0, 47.0, -7.0, 13.0, -26.0], [59.0, 2.0, -42.0, 59.0, -8.0], [-20.0, -55.0, -45.0, -39.0, 9.0], [9.0, 3.0, -57.0, 10.0, -37.0]], [[12.0, 6.0, -68.0, -50.0, -27.0], [-41.0, 49.0, 7.0, -11.0, -3.0], [27.0, -7.0, -42.0, 30.0, -33.0], [64.0, 24.0, -15.0, 10.0, -20.0]], [[-44.0, 71.0, -18.0, 19.0, -24.0], [6.0, 45.0, 4.0, -35.0, 21.0], [-30.0, 18.0, -48.0, -15.0, 54.0], [39.0, 21.0, -33.0, 48.0, -45.0]], [[-66.0, 131.0, -54.0, 17.0, 0.0], [90.0, 61.0, 4.0, 43.0, -3.0], [-18.0, -64.0, -103.0, -77.0, 36.0], [32.0, 45.0, -93.0, 15.0, -45.0]]], [[[7.0, -18.0, -12.0, -15.0, 33.0], [-19.0, 7.0, 34.0, -16.0, -2.0], [24.0, 8.0, 15.0, 11.0, -23.0], [-10.0, 11.0, 24.0, -24.0, 34.0]], [[60.0, -57.0, -17.0, -47.0, -104.0], [-29.0, -30.0, -98.0, 57.0, -26.0], [4.0, -35.0, -14.0, 41.0, -45.0], [60.0, -39.0, -21.0, 27.0, -65.0]], [[2.0, 18.0, -39.0, -18.0, 57.0], [4.0, 50.0, 65.0, 7.0, 14.0], [33.0, -14.0, -15.0, 4.0, -34.0], [-14.0, 40.0, -15.0, -48.0, 56.0]], [[12.0, -17.0, -58.0, -50.0, 58.0], [-23.0, 41.0, 78.0, -3.0, -8.0], [63.0, -3.0, -2.0, 20.0, -58.0], [0.0, 30.0, 12.0, -61.0, 64.0]], [[-43.0, 60.0, -12.0, 21.0, -31.0], [-23.0, 33.0, -12.0, -72.0, 18.0], [-38.0, 30.0, -37.0, -9.0, 57.0], [50.0, 23.0, 0.0, 74.0, -60.0]]], [[[41.0, 2.0, -48.0, -31.0, 33.0], [7.0, 27.0, -6.0, 50.0, -4.0], [30.0, -84.0, -28.0, -1.0, -85.0], [-3.0, 49.0, -18.0, -39.0, 32.0]], [[25.0, -33.0, 9.0, -10.0, -53.0], [-48.0, -5.0, -32.0, -21.0, 12.0], [-12.0, 31.0, 11.0, 44.0, 4.0], [30.0, -20.0, 9.0, 33.0, -29.0]], [[-41.0, 49.0, -74.0, -30.0, 58.0], [-16.0, 71.0, 91.0, -52.0, -2.0], [41.0, 13.0, -38.0, -10.0, -8.0], [28.0, 54.0, 6.0, -19.0, 29.0]], [[42.0, -7.0, 28.0, 28.0, 8.0], [-21.0, -13.0, -62.0, -9.0, 26.0], [-35.0, -47.0, 12.0, 6.0, -34.0], [-22.0, 36.0, 24.0, 27.0, 0.0]], [[38.0, -49.0, 60.0, 26.0, -2.0], [-11.0, -49.0, -42.0, 7.0, 12.0], [-23.0, -1.0, 51.0, 16.0, -10.0], [-43.0, -20.0, 30.0, 2.0, 14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-7.0, 0.0, 5.0, 4.0, -6.0], [-4.0, 0.0, 3.0, 0.0, -4.0], [-7.0, 7.0, 0.0, -6.0, 0.0], [4.0, 0.0, -2.0, 0.0, 0.0]])
      s1=numpy.array(-3.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[21.0, -0.0, -15.0, -12.0, 18.0], [12.0, -0.0, -9.0, -0.0, 12.0], [21.0, -21.0, -0.0, 18.0, -0.0], [-12.0, -0.0, 6.0, -0.0, -0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[0.0, -1.0, -1.0, -5.0, -2.0], [-2.0, 2.0, 7.0, -4.0, -1.0], [-1.0, 0.0, -5.0, 2.0, -7.0], [-7.0, -2.0, 2.0, -4.0, -4.0]], [[2.0, 1.0, -1.0, 0.0, -4.0], [-6.0, 7.0, -1.0, 4.0, 2.0], [0.0, 4.0, 6.0, -7.0, -1.0], [-4.0, 0.0, 1.0, -7.0, 1.0]], [[3.0, 3.0, -3.0, 6.0, 2.0], [0.0, -1.0, 4.0, -4.0, 3.0], [3.0, 3.0, -5.0, -2.0, -3.0], [-1.0, -7.0, -7.0, 0.0, 4.0]]])
      s1=numpy.array([-5.0, -6.0, -2.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-18.0, -7.0, 17.0, 13.0, 30.0], [46.0, -50.0, -37.0, 4.0, -13.0], [-1.0, -30.0, -1.0, 36.0, 47.0], [61.0, 24.0, -2.0, 62.0, 6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[2.0, 0.0, -5.0, 5.0, 6.0], [-6.0, 4.0, 5.0, 2.0, 7.0], [5.0, 2.0, -3.0, -3.0, 0.0], [5.0, -6.0, 7.0, 2.0, -2.0]], [[4.0, -6.0, 4.0, -3.0, 5.0], [6.0, -7.0, 7.0, 6.0, 0.0], [6.0, -3.0, 3.0, -2.0, -3.0], [-7.0, 3.0, 7.0, 2.0, -1.0]], [[2.0, 1.0, -6.0, 1.0, 1.0], [3.0, 7.0, 3.0, 7.0, 6.0], [0.0, -7.0, 2.0, -1.0, 3.0], [0.0, 0.0, -3.0, 4.0, -2.0]]], [[[0.0, 5.0, 2.0, 7.0, 3.0], [-5.0, 3.0, -1.0, -2.0, -1.0], [-6.0, 0.0, -2.0, -6.0, -2.0], [0.0, 0.0, -7.0, -3.0, 7.0]], [[1.0, 7.0, -3.0, 0.0, 2.0], [6.0, -7.0, -7.0, 1.0, 4.0], [-2.0, -2.0, -6.0, 3.0, -3.0], [3.0, 7.0, -2.0, 6.0, 5.0]], [[0.0, 3.0, -7.0, -3.0, 7.0], [-6.0, -1.0, 4.0, 0.0, 1.0], [5.0, -2.0, 4.0, 4.0, 2.0], [-7.0, 7.0, 5.0, -7.0, 1.0]]]])
      s1=numpy.array([[3.0, -3.0, -7.0], [-5.0, 0.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-20.0, -35.0, 54.0, 3.0, -68.0], [10.0, -24.0, -50.0, -51.0, -23.0], [-8.0, 78.0, -50.0, 6.0, -16.0], [85.0, -76.0, 21.0, 36.0, -31.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[0.0, 0.0, -7.0, 2.0, -2.0], [0.0, -4.0, 4.0, 0.0, -4.0], [1.0, 6.0, -6.0, 4.0, 6.0], [3.0, -3.0, 2.0, 0.0, 1.0]])
      s1=numpy.array([-2.0, -7.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[0.0, 0.0], [0.0, 0.0], [14.0, 49.0], [-4.0, -14.0], [4.0, 14.0]], [[0.0, 0.0], [8.0, 28.0], [-8.0, -28.0], [0.0, 0.0], [8.0, 28.0]], [[-2.0, -7.0], [-12.0, -42.0], [12.0, 42.0], [-8.0, -28.0], [-12.0, -42.0]], [[-6.0, -21.0], [6.0, 21.0], [-4.0, -14.0], [0.0, 0.0], [-2.0, -7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, 4.0, 2.0, 1.0, 6.0], [-7.0, 2.0, -7.0, 0.0, 0.0], [-7.0, -3.0, 0.0, -7.0, 6.0], [-6.0, -5.0, 1.0, -1.0, -4.0]], [[2.0, 2.0, -4.0, 3.0, -7.0], [0.0, -3.0, 3.0, 2.0, -1.0], [6.0, 0.0, -5.0, 5.0, 0.0], [-5.0, 1.0, 5.0, 5.0, 3.0]], [[3.0, 1.0, 1.0, -5.0, 2.0], [-7.0, 0.0, 7.0, -5.0, -6.0], [-3.0, -1.0, 3.0, 5.0, -4.0], [-5.0, -1.0, 0.0, 0.0, 1.0]]])
      s1=numpy.array([[-7.0, 2.0], [0.0, -1.0], [6.0, 6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[4.0, 20.0], [-22.0, 12.0], [-8.0, 14.0], [-37.0, -31.0], [-30.0, 31.0]], [[7.0, -56.0], [-14.0, 7.0], [91.0, 25.0], [-30.0, -32.0], [-36.0, -35.0]], [[31.0, -38.0], [15.0, -12.0], [18.0, 23.0], [79.0, 11.0], [-66.0, -12.0]], [[12.0, -37.0], [29.0, -17.0], [-7.0, -3.0], [7.0, -7.0], [34.0, -5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, -7.0, 1.0, 4.0, 4.0], [4.0, -1.0, -6.0, 6.0, 4.0], [-4.0, -1.0, 0.0, 2.0, -3.0], [2.0, -6.0, 5.0, 2.0, 5.0]], [[-3.0, -6.0, -1.0, 0.0, 7.0], [-3.0, -5.0, -4.0, 0.0, 1.0], [3.0, -7.0, 5.0, 0.0, 2.0], [-6.0, 6.0, 4.0, 7.0, 5.0]], [[1.0, -6.0, 1.0, -4.0, -5.0], [-5.0, 2.0, 3.0, 3.0, 5.0], [-3.0, -4.0, 6.0, -6.0, 4.0], [0.0, 3.0, 0.0, -4.0, 0.0]]], [[[3.0, 6.0, -1.0, 4.0, 5.0], [0.0, 0.0, -3.0, -7.0, 1.0], [3.0, 0.0, -1.0, -2.0, -2.0], [4.0, 3.0, -1.0, 0.0, -4.0]], [[6.0, -3.0, 4.0, -7.0, -2.0], [-2.0, -5.0, 4.0, -4.0, -6.0], [2.0, 4.0, 4.0, 2.0, 2.0], [4.0, 1.0, 4.0, -4.0, 0.0]], [[6.0, -3.0, -1.0, 1.0, -7.0], [-4.0, -1.0, -4.0, 1.0, 7.0], [2.0, -4.0, 1.0, -6.0, -2.0], [0.0, 6.0, 6.0, 2.0, -7.0]]]])
      s1=numpy.array([[[-2.0, 2.0], [-3.0, 3.0], [2.0, 3.0]], [[0.0, 0.0], [0.0, -6.0], [-2.0, 2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[9.0, -40.0], [26.0, -38.0], [5.0, -24.0], [-18.0, 40.0], [-25.0, 12.0]], [[-1.0, -12.0], [23.0, 17.0], [38.0, -47.0], [-8.0, 47.0], [-15.0, 76.0]], [[-11.0, -16.0], [23.0, -67.0], [-5.0, 11.0], [-4.0, -38.0], [12.0, -4.0]], [[14.0, -38.0], [-12.0, 21.0], [-34.0, 10.0], [-37.0, 41.0], [-11.0, 11.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-2.0, 0.0, 4.0, -2.0, 0.0], [4.0, 3.0, 0.0, 1.0, 0.0], [2.0, 2.0, -4.0, 6.0, 2.0], [3.0, -6.0, -5.0, -1.0, 3.0]])
      s1=numpy.array([[5.0, 0.0, 4.0, -2.0, 0.0], [1.0, -4.0, 4.0, 7.0, -5.0], [0.0, -1.0, -2.0, -2.0, -4.0], [1.0, -2.0, -6.0, -1.0, 4.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-10.0, 0.0, -8.0, 4.0, 0.0], [-2.0, 8.0, -8.0, -14.0, 10.0], [0.0, 2.0, 4.0, 4.0, 8.0], [-2.0, 4.0, 12.0, 2.0, -8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[20.0, 0.0, 16.0, -8.0, 0.0], [4.0, -16.0, 16.0, 28.0, -20.0], [0.0, -4.0, -8.0, -8.0, -16.0], [4.0, -8.0, -24.0, -4.0, 16.0]], [[-10.0, 0.0, -8.0, 4.0, 0.0], [-2.0, 8.0, -8.0, -14.0, 10.0], [0.0, 2.0, 4.0, 4.0, 8.0], [-2.0, 4.0, 12.0, 2.0, -8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[20.0, 0.0, 16.0, -8.0, 0.0], [4.0, -16.0, 16.0, 28.0, -20.0], [0.0, -4.0, -8.0, -8.0, -16.0], [4.0, -8.0, -24.0, -4.0, 16.0]], [[15.0, 0.0, 12.0, -6.0, 0.0], [3.0, -12.0, 12.0, 21.0, -15.0], [0.0, -3.0, -6.0, -6.0, -12.0], [3.0, -6.0, -18.0, -3.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[5.0, 0.0, 4.0, -2.0, 0.0], [1.0, -4.0, 4.0, 7.0, -5.0], [0.0, -1.0, -2.0, -2.0, -4.0], [1.0, -2.0, -6.0, -1.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[10.0, 0.0, 8.0, -4.0, 0.0], [2.0, -8.0, 8.0, 14.0, -10.0], [0.0, -2.0, -4.0, -4.0, -8.0], [2.0, -4.0, -12.0, -2.0, 8.0]], [[10.0, 0.0, 8.0, -4.0, 0.0], [2.0, -8.0, 8.0, 14.0, -10.0], [0.0, -2.0, -4.0, -4.0, -8.0], [2.0, -4.0, -12.0, -2.0, 8.0]], [[-20.0, 0.0, -16.0, 8.0, 0.0], [-4.0, 16.0, -16.0, -28.0, 20.0], [0.0, 4.0, 8.0, 8.0, 16.0], [-4.0, 8.0, 24.0, 4.0, -16.0]], [[30.0, 0.0, 24.0, -12.0, 0.0], [6.0, -24.0, 24.0, 42.0, -30.0], [0.0, -6.0, -12.0, -12.0, -24.0], [6.0, -12.0, -36.0, -6.0, 24.0]], [[10.0, 0.0, 8.0, -4.0, 0.0], [2.0, -8.0, 8.0, 14.0, -10.0], [0.0, -2.0, -4.0, -4.0, -8.0], [2.0, -4.0, -12.0, -2.0, 8.0]]], [[[15.0, 0.0, 12.0, -6.0, 0.0], [3.0, -12.0, 12.0, 21.0, -15.0], [0.0, -3.0, -6.0, -6.0, -12.0], [3.0, -6.0, -18.0, -3.0, 12.0]], [[-30.0, 0.0, -24.0, 12.0, 0.0], [-6.0, 24.0, -24.0, -42.0, 30.0], [0.0, 6.0, 12.0, 12.0, 24.0], [-6.0, 12.0, 36.0, 6.0, -24.0]], [[-25.0, 0.0, -20.0, 10.0, 0.0], [-5.0, 20.0, -20.0, -35.0, 25.0], [0.0, 5.0, 10.0, 10.0, 20.0], [-5.0, 10.0, 30.0, 5.0, -20.0]], [[-5.0, 0.0, -4.0, 2.0, 0.0], [-1.0, 4.0, -4.0, -7.0, 5.0], [0.0, 1.0, 2.0, 2.0, 4.0], [-1.0, 2.0, 6.0, 1.0, -4.0]], [[15.0, 0.0, 12.0, -6.0, 0.0], [3.0, -12.0, 12.0, 21.0, -15.0], [0.0, -3.0, -6.0, -6.0, -12.0], [3.0, -6.0, -18.0, -3.0, 12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[6.0, -6.0, -1.0, 0.0, 5.0], [5.0, -3.0, -7.0, -3.0, -4.0], [0.0, -4.0, 0.0, 0.0, -7.0], [-1.0, 3.0, -3.0, 7.0, 0.0]], [[-3.0, 6.0, 5.0, -7.0, 6.0], [0.0, -6.0, 3.0, -1.0, -7.0], [-3.0, 3.0, 3.0, 7.0, 6.0], [-5.0, 0.0, -4.0, 3.0, 5.0]], [[-4.0, 1.0, -7.0, -4.0, 2.0], [-3.0, -1.0, 3.0, -7.0, -5.0], [0.0, 6.0, 6.0, 1.0, 2.0], [-1.0, -1.0, 5.0, 4.0, -3.0]]])
      s1=numpy.array([[[-5.0, 3.0, -3.0, -3.0, 7.0], [3.0, -6.0, 5.0, -4.0, -4.0], [-3.0, -5.0, -7.0, -3.0, -1.0], [0.0, -2.0, -6.0, 2.0, -7.0]], [[4.0, -6.0, 0.0, 7.0, 7.0], [0.0, -7.0, 5.0, 2.0, 3.0], [2.0, -6.0, 2.0, -2.0, 4.0], [-3.0, 3.0, -4.0, -2.0, 1.0]], [[0.0, 4.0, -1.0, -1.0, -2.0], [-2.0, 6.0, 1.0, 0.0, -1.0], [-6.0, 7.0, -4.0, 7.0, 0.0], [-1.0, 2.0, 1.0, -3.0, -3.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-42.0, 20.0, -14.0, -35.0, 29.0], [26.0, -39.0, 11.0, -30.0, -29.0], [0.0, -40.0, -32.0, -40.0, -18.0], [13.0, -29.0, -28.0, 30.0, -33.0]], [[54.0, -50.0, 17.0, 59.0, -2.0], [-20.0, 0.0, 1.0, 36.0, 41.0], [24.0, 1.0, 50.0, 13.0, 30.0], [-19.0, 32.0, 13.0, -27.0, 45.0]], [[25.0, -61.0, 10.0, 45.0, 42.0], [11.0, -71.0, 13.0, 14.0, 26.0], [55.0, -74.0, 45.0, -56.0, 21.0], [-8.0, 3.0, -21.0, 9.0, 33.0]], [[-28.0, 26.0, 4.0, -45.0, -41.0], [8.0, 25.0, -39.0, -14.0, -17.0], [10.0, 14.0, 2.0, -14.0, -28.0], [25.0, -29.0, 24.0, 26.0, 5.0]], [[-1.0, -13.0, -17.0, 25.0, 73.0], [11.0, -60.0, 57.0, -8.0, -4.0], [-15.0, -47.0, -31.0, -13.0, 19.0], [-20.0, 12.0, -52.0, -8.0, -35.0]]], [[[-25.0, 3.0, -12.0, -12.0, 41.0], [21.0, -48.0, 22.0, -20.0, -17.0], [3.0, -46.0, -23.0, -36.0, -5.0], [3.0, -16.0, -33.0, 19.0, -26.0]], [[-9.0, 23.0, 10.0, -32.0, -61.0], [-7.0, 54.0, -46.0, 0.0, -5.0], [3.0, 44.0, 13.0, 14.0, -21.0], [19.0, -14.0, 41.0, 9.0, 18.0]], [[47.0, -27.0, 18.0, 39.0, -34.0], [-27.0, 39.0, -17.0, 34.0, 34.0], [9.0, 38.0, 43.0, 36.0, 19.0], [-12.0, 29.0, 33.0, -29.0, 43.0]], [[11.0, -31.0, 16.0, 9.0, -14.0], [5.0, -17.0, -27.0, 10.0, 16.0], [49.0, -28.0, 47.0, -38.0, -1.0], [10.0, -11.0, 15.0, 17.0, 41.0]], [[-8.0, 10.0, 17.0, -32.0, -67.0], [-2.0, 43.0, -60.0, 2.0, 0.0], [28.0, 27.0, 34.0, -9.0, -24.0], [26.0, -23.0, 47.0, 21.0, 36.0]]], [[[-12.0, 18.0, 0.0, -21.0, -21.0], [0.0, 21.0, -15.0, -6.0, -9.0], [-6.0, 18.0, -6.0, 6.0, -12.0], [9.0, -9.0, 12.0, 6.0, -3.0]], [[32.0, -6.0, 6.0, 27.0, -19.0], [-24.0, 39.0, 1.0, 22.0, 19.0], [-18.0, 44.0, 10.0, 48.0, 16.0], [-15.0, 29.0, 18.0, -32.0, 13.0]], [[12.0, 6.0, -6.0, 15.0, 9.0], [-12.0, 15.0, 21.0, 6.0, 3.0], [-30.0, 24.0, -18.0, 36.0, 12.0], [-15.0, 21.0, -6.0, -24.0, -15.0]], [[28.0, -38.0, -1.0, 48.0, 47.0], [-2.0, -43.0, 36.0, 14.0, 20.0], [8.0, -35.0, 10.0, -7.0, 28.0], [-22.0, 23.0, -27.0, -17.0, 4.0]], [[59.0, -49.0, 19.0, 61.0, -11.0], [-25.0, 12.0, -3.0, 40.0, 44.0], [21.0, 13.0, 53.0, 23.0, 31.0], [-20.0, 36.0, 20.0, -32.0, 49.0]]], [[[-15.0, 23.0, 4.0, -31.0, -40.0], [-1.0, 35.0, -31.0, -6.0, -10.0], [-1.0, 28.0, 1.0, 6.0, -19.0], [16.0, -15.0, 25.0, 11.0, 5.0]], [[-15.0, 5.0, -8.0, -8.0, 23.0], [11.0, -24.0, 14.0, -12.0, -11.0], [-3.0, -22.0, -17.0, -16.0, -3.0], [1.0, -8.0, -19.0, 9.0, -18.0]], [[-1.0, 35.0, 4.0, -24.0, -59.0], [-19.0, 76.0, -30.0, 4.0, -5.0], [-29.0, 74.0, -7.0, 52.0, -13.0], [7.0, 4.0, 39.0, -13.0, 2.0]], [[-23.0, 19.0, -25.0, -4.0, 62.0], [13.0, -39.0, 54.0, -22.0, -23.0], [-39.0, -25.0, -59.0, 1.0, 5.0], [-13.0, 3.0, -50.0, -4.0, -58.0]], [[20.0, -42.0, 3.0, 38.0, 41.0], [6.0, -53.0, 22.0, 10.0, 18.0], [28.0, -51.0, 22.0, -31.0, 20.0], [-12.0, 9.0, -23.0, -1.0, 14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-2.0, -6.0, 0.0, -7.0, -3.0], [3.0, 0.0, -4.0, -5.0, -6.0], [0.0, 0.0, 0.0, 4.0, 4.0], [-3.0, 0.0, -7.0, -2.0, 3.0]], [[7.0, 1.0, 2.0, 3.0, -5.0], [0.0, -5.0, -6.0, 5.0, 5.0], [3.0, 5.0, 6.0, 5.0, 4.0], [5.0, -5.0, 7.0, 0.0, -5.0]], [[-2.0, -3.0, 0.0, 7.0, -2.0], [-6.0, 3.0, -3.0, 4.0, 7.0], [0.0, 0.0, -5.0, -4.0, -7.0], [-7.0, -6.0, -4.0, -1.0, 7.0]]], [[[2.0, 6.0, 0.0, -5.0, 7.0], [1.0, 4.0, -3.0, 5.0, 5.0], [-7.0, 4.0, 1.0, 4.0, 5.0], [5.0, 0.0, 3.0, -3.0, -3.0]], [[6.0, -5.0, 0.0, 2.0, 1.0], [0.0, -2.0, -7.0, -6.0, -5.0], [-7.0, -5.0, 5.0, 0.0, -7.0], [-4.0, -3.0, 5.0, -5.0, -3.0]], [[-1.0, -1.0, 1.0, 2.0, -1.0], [-7.0, -1.0, 4.0, -5.0, -5.0], [-6.0, 1.0, 0.0, 1.0, -5.0], [-1.0, -1.0, 4.0, -3.0, -6.0]]]])
      s1=numpy.array([[[[-1.0, 5.0, 6.0, -4.0, 7.0], [-5.0, -4.0, 0.0, 4.0, 6.0], [-2.0, -2.0, -1.0, -1.0, 0.0], [-1.0, 0.0, 5.0, -5.0, 2.0]], [[1.0, -5.0, 0.0, -1.0, -4.0], [0.0, -5.0, 3.0, 7.0, 2.0], [-4.0, -2.0, 4.0, -5.0, 4.0], [-5.0, -2.0, 2.0, 5.0, -7.0]], [[0.0, -1.0, -3.0, 6.0, 3.0], [-6.0, -4.0, -3.0, -1.0, 6.0], [-6.0, 5.0, 6.0, -5.0, 5.0], [3.0, 0.0, 0.0, 0.0, -5.0]]], [[[-7.0, 1.0, 2.0, 2.0, 4.0], [-6.0, -7.0, 0.0, 0.0, -1.0], [-5.0, 5.0, -3.0, -4.0, -3.0], [1.0, -4.0, -6.0, -6.0, -4.0]], [[6.0, 4.0, 6.0, -5.0, 3.0], [3.0, -7.0, 0.0, 4.0, -5.0], [0.0, 7.0, -6.0, -4.0, 4.0], [7.0, -5.0, -7.0, -7.0, -7.0]], [[1.0, -4.0, 6.0, 4.0, -6.0], [1.0, 0.0, -5.0, 0.0, 4.0], [-4.0, 5.0, -4.0, 3.0, 0.0], [0.0, -7.0, -4.0, 2.0, -5.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[30.0, -13.0, 28.0, -41.0, -16.0], [27.0, -75.0, 32.0, 67.0, -46.0], [-18.0, 27.0, -20.0, -58.0, 36.0], [5.0, -45.0, -46.0, -11.0, -88.0]], [[-66.0, -42.0, -51.0, 38.0, -40.0], [-4.0, 24.0, 17.0, -34.0, -37.0], [0.0, -15.0, 8.0, 9.0, -49.0], [-37.0, 6.0, -25.0, 32.0, 12.0]], [[3.0, -14.0, 6.0, 2.0, -14.0], [1.0, -10.0, 1.0, 14.0, 8.0], [-12.0, 1.0, 4.0, -7.0, 8.0], [-10.0, -11.0, 0.0, 12.0, -19.0]], [[59.0, -62.0, -49.0, 55.0, -66.0], [31.0, 6.0, -22.0, -6.0, 9.0], [-23.0, 42.0, 56.0, -25.0, 70.0], [22.0, -10.0, -21.0, 70.0, -74.0]], [[-46.0, 27.0, 2.0, 10.0, 30.0], [-13.0, -11.0, -4.0, -41.0, -56.0], [7.0, 43.0, -52.0, 3.0, -47.0], [36.0, -16.0, -70.0, -61.0, 9.0]]], [[[-17.0, 50.0, -4.0, -74.0, 49.0], [8.0, 5.0, 53.0, 18.0, -47.0], [53.0, -66.0, -14.0, 2.0, -33.0], [-20.0, 45.0, 37.0, -35.0, 67.0]], [[-46.0, 22.0, -19.0, 37.0, 45.0], [-49.0, -1.0, -19.0, -46.0, 10.0], [-14.0, 26.0, 2.0, -1.0, -25.0], [24.0, 11.0, -16.0, -37.0, 23.0]], [[-19.0, -34.0, -39.0, 49.0, -70.0], [39.0, 128.0, -29.0, -83.0, 0.0], [49.0, -39.0, -3.0, 101.0, -58.0], [-27.0, 31.0, 19.0, 65.0, 90.0]], [[-66.0, -53.0, -98.0, 59.0, -11.0], [-52.0, -14.0, 28.0, -13.0, 9.0], [-39.0, -22.0, 90.0, -51.0, 1.0], [-45.0, 35.0, 17.0, 52.0, -18.0]], [[-59.0, -57.0, -107.0, 76.0, -6.0], [-62.0, -29.0, 19.0, -16.0, 16.0], [-55.0, 2.0, 103.0, -69.0, 20.0], [-28.0, 30.0, 5.0, 50.0, -42.0]]], [[[4.0, -26.0, -92.0, -6.0, -25.0], [15.0, 83.0, 39.0, -7.0, 24.0], [47.0, -120.0, 99.0, 23.0, 5.0], [-71.0, 99.0, 121.0, 94.0, 86.0]], [[-52.0, -45.0, -16.0, 32.0, -25.0], [-38.0, -18.0, 10.0, 15.0, 35.0], [-44.0, -20.0, 34.0, -18.0, -12.0], [-56.0, -8.0, 17.0, 38.0, -21.0]], [[29.0, -4.0, 47.0, -59.0, -20.0], [39.0, -52.0, 33.0, 67.0, -44.0], [1.0, 3.0, -39.0, -29.0, 16.0], [-9.0, -41.0, -29.0, -11.0, -56.0]], [[-26.0, -1.0, 50.0, -33.0, 6.0], [-19.0, -53.0, 22.0, 55.0, 10.0], [-28.0, -13.0, -24.0, -22.0, -12.0], [-37.0, -33.0, 2.0, -17.0, -28.0]], [[-82.0, 4.0, -17.0, -37.0, 20.0], [-34.0, 6.0, 58.0, 23.0, 0.0], [13.0, -100.0, 17.0, 4.0, -62.0], [-89.0, 42.0, 67.0, 9.0, 69.0]]], [[[-52.0, -40.0, -17.0, -9.0, -48.0], [14.0, 8.0, 41.0, 14.0, -39.0], [7.0, -47.0, -6.0, 6.0, -46.0], [-66.0, -3.0, -3.0, 36.0, 7.0]], [[-24.0, 23.0, -6.0, -20.0, -1.0], [26.0, 70.0, 8.0, -41.0, -35.0], [60.0, -46.0, -34.0, 64.0, -62.0], [-14.0, 32.0, 15.0, -6.0, 91.0]], [[27.0, -59.0, 30.0, -6.0, -86.0], [60.0, -47.0, 13.0, 45.0, -64.0], [-21.0, 50.0, -44.0, -28.0, 19.0], [-2.0, -79.0, -90.0, 25.0, -110.0]], [[-10.0, -20.0, -63.0, 9.0, -26.0], [16.0, 68.0, 18.0, -27.0, -2.0], [37.0, -66.0, 47.0, 30.0, -16.0], [-39.0, 58.0, 55.0, 57.0, 63.0]], [[-11.0, 42.0, -63.0, 20.0, 77.0], [-54.0, 27.0, -6.0, -42.0, 44.0], [11.0, -27.0, 70.0, -7.0, 12.0], [19.0, 79.0, 68.0, -13.0, 69.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(5.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-4.0, 2.0, -3.0, 0.0, 6.0], [-3.0, -2.0, -5.0, -3.0, 6.0], [6.0, 5.0, 2.0, -3.0, 2.0], [-1.0, 5.0, -1.0, 0.0, 1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-20.0, 10.0, -15.0, 0.0, 30.0], [-15.0, -10.0, -25.0, -15.0, 30.0], [30.0, 25.0, 10.0, -15.0, 10.0], [-5.0, 25.0, -5.0, 0.0, 5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Data(numpy.array([1.0, 7.0, -2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-2.0, 6.0, -1.0, -6.0, 2.0], [0.0, 4.0, 0.0, 2.0, -6.0], [-5.0, -3.0, -7.0, 1.0, -1.0], [0.0, 0.0, 4.0, -6.0, -6.0]], [[-5.0, -3.0, 4.0, -1.0, 0.0], [4.0, 7.0, -7.0, -1.0, -1.0], [6.0, 0.0, -7.0, 2.0, -3.0], [6.0, 6.0, -4.0, -5.0, 4.0]], [[-3.0, -6.0, 6.0, 1.0, 1.0], [-5.0, -2.0, -3.0, 7.0, -4.0], [-4.0, 4.0, -6.0, 2.0, -6.0], [3.0, 7.0, -2.0, -4.0, -3.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-31.0, -3.0, 15.0, -15.0, 0.0], [38.0, 57.0, -43.0, -19.0, -5.0], [45.0, -11.0, -44.0, 11.0, -10.0], [36.0, 28.0, -20.0, -33.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[-6.0, -6.0, 0.0], [-4.0, -4.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[7.0, -6.0, -7.0, -2.0, -5.0], [7.0, -6.0, 3.0, -5.0, -2.0], [-3.0, 3.0, -5.0, 7.0, 0.0], [-6.0, 4.0, 3.0, -4.0, 0.0]], [[-6.0, 7.0, 2.0, 0.0, -1.0], [3.0, 0.0, 0.0, -5.0, 0.0], [-7.0, 1.0, 2.0, 4.0, -6.0], [-7.0, 0.0, 0.0, 1.0, 0.0]], [[-3.0, 7.0, -5.0, 2.0, -6.0], [3.0, -5.0, 6.0, 2.0, -7.0], [3.0, -6.0, 5.0, 0.0, 0.0], [-1.0, 7.0, -4.0, 0.0, 5.0]]], [[[-4.0, 4.0, 7.0, 6.0, -1.0], [5.0, -4.0, 0.0, -7.0, 0.0], [-1.0, -7.0, 4.0, 5.0, 6.0], [-4.0, 4.0, -2.0, 4.0, -3.0]], [[6.0, -7.0, -5.0, -4.0, -2.0], [2.0, -2.0, 0.0, -6.0, -5.0], [7.0, 5.0, -5.0, 3.0, 1.0], [-5.0, 0.0, 7.0, 0.0, -3.0]], [[7.0, 0.0, 6.0, -4.0, -7.0], [-1.0, 3.0, 1.0, -6.0, 3.0], [2.0, 3.0, -1.0, -6.0, -7.0], [4.0, -2.0, -1.0, 5.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-14.0, 6.0, 22.0, 4.0, 48.0], [-88.0, 60.0, -18.0, 112.0, 32.0], [36.0, -16.0, 22.0, -98.0, 8.0], [114.0, -40.0, -38.0, 2.0, 24.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([-1.0, 0.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-5.0, -1.0, 0.0, 1.0, -1.0], [6.0, -6.0, 0.0, -7.0, -6.0], [-3.0, -6.0, -2.0, 0.0, 1.0], [-3.0, -6.0, -5.0, 7.0, -3.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[5.0, 0.0], [1.0, 0.0], [0.0, 0.0], [-1.0, 0.0], [1.0, 0.0]], [[-6.0, 0.0], [6.0, 0.0], [0.0, 0.0], [7.0, 0.0], [6.0, 0.0]], [[3.0, 0.0], [6.0, 0.0], [2.0, 0.0], [0.0, 0.0], [-1.0, 0.0]], [[3.0, 0.0], [6.0, 0.0], [5.0, 0.0], [-7.0, 0.0], [3.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Data(numpy.array([[0.0, -1.0], [-5.0, 7.0], [-6.0, -7.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-3.0, 0.0, 1.0, -3.0, -7.0], [1.0, -7.0, -6.0, 3.0, -1.0], [-4.0, -4.0, -3.0, -7.0, 6.0], [0.0, 4.0, -5.0, -5.0, 3.0]], [[0.0, 6.0, 4.0, 7.0, 0.0], [4.0, -5.0, 6.0, -4.0, -3.0], [0.0, 2.0, 3.0, 3.0, 6.0], [-2.0, 2.0, -2.0, 4.0, 4.0]], [[6.0, 0.0, -1.0, -1.0, 7.0], [7.0, 6.0, 1.0, 6.0, -3.0], [1.0, 0.0, 1.0, 4.0, 1.0], [-3.0, -1.0, 2.0, 3.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-36.0, -39.0], [-30.0, 42.0], [-14.0, 34.0], [-29.0, 59.0], [-42.0, -42.0]], [[-62.0, -22.0], [-11.0, -70.0], [-36.0, 41.0], [-16.0, -73.0], [33.0, 1.0]], [[-6.0, -3.0], [-10.0, 18.0], [-21.0, 17.0], [-39.0, 0.0], [-36.0, 29.0]], [[28.0, 7.0], [-4.0, 17.0], [-2.0, -23.0], [-38.0, 12.0], [-62.0, -24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[[-2.0, -6.0], [1.0, 0.0], [1.0, 0.0]], [[-2.0, -5.0], [-2.0, -3.0], [-1.0, 4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-1.0, -1.0, -2.0, -3.0, -6.0], [-4.0, -2.0, 7.0, -5.0, 0.0], [3.0, -4.0, -2.0, 6.0, -3.0], [5.0, -5.0, 3.0, 0.0, 6.0]], [[-6.0, 3.0, -4.0, 2.0, 1.0], [5.0, -7.0, 7.0, -2.0, 7.0], [-6.0, -5.0, 2.0, 0.0, 0.0], [1.0, 1.0, 1.0, 5.0, 5.0]], [[-3.0, 2.0, 0.0, -1.0, -1.0], [-7.0, 0.0, -3.0, 7.0, 2.0], [6.0, -7.0, -5.0, 5.0, 0.0], [7.0, 5.0, 0.0, -7.0, 4.0]]], [[[2.0, 7.0, -6.0, 0.0, 1.0], [6.0, -2.0, 2.0, 1.0, 3.0], [0.0, 5.0, -6.0, 1.0, -4.0], [-6.0, -3.0, -7.0, 4.0, -3.0]], [[-1.0, -1.0, 1.0, -3.0, 5.0], [0.0, -4.0, 4.0, 7.0, -2.0], [-1.0, 3.0, -3.0, -1.0, 0.0], [0.0, -3.0, -1.0, 0.0, -5.0]], [[4.0, -6.0, -4.0, -7.0, 6.0], [-5.0, -3.0, 4.0, 0.0, 2.0], [-1.0, 7.0, -7.0, -2.0, 1.0], [1.0, 3.0, -7.0, 3.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-13.0, 15.0], [1.0, -50.0], [14.0, 23.0], [20.0, -1.0], [-6.0, 40.0]], [[-1.0, -26.0], [12.0, 22.0], [-26.0, -48.0], [-1.0, 4.0], [5.0, -1.0]], [[-3.0, -19.0], [-27.0, 18.0], [26.0, 23.0], [-5.0, -46.0], [13.0, 42.0]], [[9.0, 4.0], [25.0, 66.0], [18.0, -8.0], [-13.0, -8.0], [18.0, -26.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_constData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[-3.0, 0.0, -3.0, -3.0, 2.0], [6.0, 6.0, 1.0, 4.0, -1.0], [0.0, 6.0, 7.0, 1.0, -1.0], [4.0, -4.0, 1.0, -1.0, 1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[0.0, -1.0, -6.0, -4.0, -4.0], [6.0, 7.0, 1.0, -1.0, -4.0], [-4.0, 2.0, 1.0, 6.0, 0.0], [1.0, -5.0, 2.0, 7.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[3.0, 0.0, 3.0, 3.0, -2.0], [-6.0, -6.0, -1.0, -4.0, 1.0], [0.0, -6.0, -7.0, -1.0, 1.0], [-4.0, 4.0, -1.0, 1.0, -1.0]], [[18.0, 0.0, 18.0, 18.0, -12.0], [-36.0, -36.0, -6.0, -24.0, 6.0], [0.0, -36.0, -42.0, -6.0, 6.0], [-24.0, 24.0, -6.0, 6.0, -6.0]], [[12.0, 0.0, 12.0, 12.0, -8.0], [-24.0, -24.0, -4.0, -16.0, 4.0], [0.0, -24.0, -28.0, -4.0, 4.0], [-16.0, 16.0, -4.0, 4.0, -4.0]], [[12.0, 0.0, 12.0, 12.0, -8.0], [-24.0, -24.0, -4.0, -16.0, 4.0], [0.0, -24.0, -28.0, -4.0, 4.0], [-16.0, 16.0, -4.0, 4.0, -4.0]]], [[[-18.0, 0.0, -18.0, -18.0, 12.0], [36.0, 36.0, 6.0, 24.0, -6.0], [0.0, 36.0, 42.0, 6.0, -6.0], [24.0, -24.0, 6.0, -6.0, 6.0]], [[-21.0, 0.0, -21.0, -21.0, 14.0], [42.0, 42.0, 7.0, 28.0, -7.0], [0.0, 42.0, 49.0, 7.0, -7.0], [28.0, -28.0, 7.0, -7.0, 7.0]], [[-3.0, 0.0, -3.0, -3.0, 2.0], [6.0, 6.0, 1.0, 4.0, -1.0], [0.0, 6.0, 7.0, 1.0, -1.0], [4.0, -4.0, 1.0, -1.0, 1.0]], [[3.0, 0.0, 3.0, 3.0, -2.0], [-6.0, -6.0, -1.0, -4.0, 1.0], [0.0, -6.0, -7.0, -1.0, 1.0], [-4.0, 4.0, -1.0, 1.0, -1.0]], [[12.0, 0.0, 12.0, 12.0, -8.0], [-24.0, -24.0, -4.0, -16.0, 4.0], [0.0, -24.0, -28.0, -4.0, 4.0], [-16.0, 16.0, -4.0, 4.0, -4.0]]], [[[12.0, 0.0, 12.0, 12.0, -8.0], [-24.0, -24.0, -4.0, -16.0, 4.0], [0.0, -24.0, -28.0, -4.0, 4.0], [-16.0, 16.0, -4.0, 4.0, -4.0]], [[-6.0, 0.0, -6.0, -6.0, 4.0], [12.0, 12.0, 2.0, 8.0, -2.0], [0.0, 12.0, 14.0, 2.0, -2.0], [8.0, -8.0, 2.0, -2.0, 2.0]], [[-3.0, 0.0, -3.0, -3.0, 2.0], [6.0, 6.0, 1.0, 4.0, -1.0], [0.0, 6.0, 7.0, 1.0, -1.0], [4.0, -4.0, 1.0, -1.0, 1.0]], [[-18.0, 0.0, -18.0, -18.0, 12.0], [36.0, 36.0, 6.0, 24.0, -6.0], [0.0, 36.0, 42.0, 6.0, -6.0], [24.0, -24.0, 6.0, -6.0, 6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-3.0, 0.0, -3.0, -3.0, 2.0], [6.0, 6.0, 1.0, 4.0, -1.0], [0.0, 6.0, 7.0, 1.0, -1.0], [4.0, -4.0, 1.0, -1.0, 1.0]], [[15.0, 0.0, 15.0, 15.0, -10.0], [-30.0, -30.0, -5.0, -20.0, 5.0], [0.0, -30.0, -35.0, -5.0, 5.0], [-20.0, 20.0, -5.0, 5.0, -5.0]], [[-6.0, 0.0, -6.0, -6.0, 4.0], [12.0, 12.0, 2.0, 8.0, -2.0], [0.0, 12.0, 14.0, 2.0, -2.0], [8.0, -8.0, 2.0, -2.0, 2.0]], [[-21.0, 0.0, -21.0, -21.0, 14.0], [42.0, 42.0, 7.0, 28.0, -7.0], [0.0, 42.0, 49.0, 7.0, -7.0], [28.0, -28.0, 7.0, -7.0, 7.0]], [[12.0, 0.0, 12.0, 12.0, -8.0], [-24.0, -24.0, -4.0, -16.0, 4.0], [0.0, -24.0, -28.0, -4.0, 4.0], [-16.0, 16.0, -4.0, 4.0, -4.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank3_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      arg1=Data(numpy.array([[[-5.0, -7.0, -1.0, 7.0, 4.0], [1.0, -4.0, 6.0, 2.0, 2.0], [7.0, 7.0, 0.0, 5.0, 0.0], [0.0, -7.0, 5.0, -6.0, 0.0]], [[-4.0, 3.0, 6.0, 1.0, 7.0], [-3.0, 1.0, -6.0, 6.0, 1.0], [5.0, 4.0, -5.0, 2.0, 5.0], [-6.0, 5.0, 1.0, 1.0, 7.0]], [[-6.0, -7.0, -6.0, 1.0, 4.0], [0.0, 4.0, 3.0, 2.0, -7.0], [3.0, -5.0, 7.0, -4.0, -5.0], [-2.0, 3.0, 0.0, 0.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[0.0, -7.0, 6.0, -5.0, -3.0], [7.0, 2.0, -2.0, 7.0, -3.0], [-7.0, -2.0, 1.0, 5.0, -2.0], [0.0, -2.0, -7.0, 5.0, 4.0]], [[0.0, -4.0, 2.0, 2.0, 2.0], [-4.0, 4.0, 2.0, 6.0, 1.0], [-4.0, 3.0, -7.0, -7.0, 5.0], [-7.0, 5.0, 6.0, 3.0, 0.0]], [[2.0, 2.0, -1.0, 1.0, -7.0], [-2.0, 0.0, -5.0, 0.0, 4.0], [-4.0, -4.0, 4.0, 7.0, 1.0], [2.0, 3.0, 3.0, -5.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-12.0, -14.0, -12.0, 2.0, 8.0], [0.0, 8.0, 6.0, 4.0, -14.0], [6.0, -10.0, 14.0, -8.0, -10.0], [-4.0, 6.0, 0.0, 0.0, 0.0]], [[39.0, 23.0, -29.0, -51.0, -48.0], [5.0, 32.0, -12.0, -34.0, -32.0], [-63.0, -75.0, 34.0, -51.0, -30.0], [20.0, 35.0, -39.0, 38.0, -28.0]], [[-32.0, -29.0, 12.0, 43.0, 34.0], [0.0, -26.0, 21.0, 22.0, 21.0], [49.0, 55.0, -17.0, 38.0, 15.0], [-10.0, -35.0, 32.0, -34.0, 14.0]], [[11.0, 34.0, 11.0, -32.0, -2.0], [-11.0, 26.0, -39.0, 4.0, -15.0], [-22.0, -32.0, -3.0, -25.0, 5.0], [-14.0, 48.0, -23.0, 32.0, 14.0]], [[49.0, 76.0, 57.0, -26.0, -26.0], [-9.0, -14.0, -51.0, -8.0, 45.0], [-32.0, 22.0, -59.0, 17.0, 45.0], [2.0, 10.0, -13.0, 20.0, 14.0]]], [[[-7.0, -47.0, -19.0, 43.0, -8.0], [19.0, -40.0, 60.0, -14.0, 24.0], [23.0, 43.0, 6.0, 35.0, -10.0], [28.0, -75.0, 31.0, -46.0, -28.0]], [[-26.0, -2.0, 22.0, 18.0, 36.0], [-10.0, -4.0, -12.0, 28.0, 8.0], [34.0, 30.0, -20.0, 18.0, 20.0], [-24.0, 6.0, 14.0, -8.0, 28.0]], [[32.0, 55.0, 44.0, -17.0, -14.0], [-8.0, -10.0, -39.0, -2.0, 33.0], [-19.0, 19.0, -45.0, 14.0, 35.0], [-2.0, 9.0, -8.0, 14.0, 14.0]], [[-59.0, -31.0, 29.0, 55.0, 70.0], [-11.0, -22.0, 6.0, 50.0, 20.0], [79.0, 73.0, -30.0, 47.0, 30.0], [-36.0, -19.0, 41.0, -36.0, 42.0]], [[-13.0, -4.0, -15.0, -16.0, 11.0], [-6.0, 29.0, -12.0, 8.0, -33.0], [-4.0, -37.0, 23.0, -29.0, -15.0], [-14.0, 38.0, -14.0, 19.0, 7.0]]], [[[75.0, 65.0, 7.0, -57.0, -72.0], [5.0, 8.0, -30.0, -46.0, 10.0], [-81.0, -45.0, -8.0, -27.0, 0.0], [32.0, 17.0, -39.0, 38.0, -28.0]], [[22.0, 51.0, 44.0, -15.0, -3.0], [-11.0, -5.0, -42.0, 6.0, 27.0], [-11.0, 18.0, -43.0, 12.0, 35.0], [-10.0, 17.0, -7.0, 15.0, 21.0]], [[-1.0, -56.0, -67.0, 4.0, -29.0], [22.0, 5.0, 60.0, -32.0, -33.0], [-16.0, -41.0, 63.0, -25.0, -55.0], [34.0, -30.0, -2.0, -13.0, -49.0]], [[-39.0, -105.0, -89.0, 35.0, -1.0], [26.0, 1.0, 93.0, -18.0, -46.0], [21.0, -28.0, 84.0, -17.0, -70.0], [28.0, -49.0, 18.0, -37.0, -49.0]], [[-16.0, 22.0, 26.0, -8.0, 31.0], [-17.0, 17.0, -39.0, 28.0, -6.0], [14.0, 1.0, -18.0, -4.0, 20.0], [-32.0, 42.0, -5.0, 17.0, 35.0]]], [[[16.0, -35.0, -54.0, -5.0, -41.0], [21.0, 1.0, 48.0, -38.0, -21.0], [-29.0, -38.0, 49.0, -22.0, -45.0], [38.0, -29.0, -7.0, -7.0, -49.0]], [[-28.0, 8.0, 14.0, -6.0, 39.0], [-17.0, 25.0, -33.0, 32.0, -20.0], [20.0, -9.0, -4.0, -12.0, 10.0], [-36.0, 48.0, -5.0, 17.0, 35.0]], [[-7.0, 46.0, 25.0, -40.0, 26.0], [-25.0, 46.0, -69.0, 28.0, -29.0], [-10.0, -40.0, -9.0, -35.0, 15.0], [-42.0, 88.0, -29.0, 48.0, 42.0]], [[-7.0, 9.0, 43.0, 33.0, 21.0], [-4.0, -37.0, -3.0, 18.0, 48.0], [35.0, 72.0, -50.0, 51.0, 40.0], [-8.0, -35.0, 28.0, -27.0, 21.0]], [[-32.0, -42.0, -16.0, 30.0, 24.0], [4.0, -8.0, 30.0, 12.0, -6.0], [34.0, 18.0, 14.0, 12.0, -10.0], [-4.0, -22.0, 20.0, -24.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      arg1=Data(numpy.array([[[[1.0, -7.0, 5.0, 2.0, 0.0], [-7.0, -5.0, 7.0, 0.0, 4.0], [1.0, 1.0, 7.0, 1.0, -7.0], [6.0, 7.0, -5.0, 7.0, 5.0]], [[-2.0, -2.0, 3.0, 0.0, -4.0], [-2.0, -2.0, -6.0, 0.0, 6.0], [2.0, -5.0, -5.0, -1.0, -4.0], [-1.0, -5.0, -1.0, 4.0, -4.0]], [[6.0, 7.0, -6.0, -6.0, 0.0], [-3.0, 6.0, 0.0, -4.0, 4.0], [6.0, -3.0, -2.0, -5.0, 4.0], [1.0, 6.0, -1.0, -6.0, -1.0]]], [[[-5.0, -2.0, -5.0, 5.0, -1.0], [5.0, -1.0, 5.0, -5.0, -4.0], [-6.0, 6.0, 0.0, 1.0, -5.0], [-1.0, -4.0, 0.0, 6.0, -3.0]], [[0.0, -7.0, 1.0, 2.0, 6.0], [-5.0, -5.0, 1.0, 2.0, 6.0], [-7.0, 6.0, 3.0, 7.0, 0.0], [-3.0, 3.0, -5.0, -7.0, 6.0]], [[0.0, -7.0, -4.0, 3.0, -5.0], [-2.0, 0.0, 1.0, 2.0, 4.0], [3.0, -2.0, -7.0, 7.0, 5.0], [-2.0, 7.0, 3.0, -7.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[0.0, 5.0, -3.0, -6.0, 7.0], [-3.0, 3.0, 2.0, -4.0, -5.0], [2.0, -5.0, -4.0, -4.0, 7.0], [-3.0, 6.0, 6.0, -5.0, 0.0]], [[-3.0, 7.0, 3.0, 1.0, 0.0], [-7.0, -6.0, 6.0, -4.0, 5.0], [4.0, -6.0, 6.0, -1.0, 3.0], [-1.0, -6.0, -3.0, 4.0, 0.0]], [[0.0, -6.0, 7.0, 0.0, 1.0], [-4.0, 0.0, 0.0, 4.0, -1.0], [-4.0, -2.0, -1.0, 6.0, 0.0], [-1.0, 0.0, 0.0, -2.0, 7.0]]], [[[3.0, -5.0, 3.0, 3.0, -6.0], [-1.0, 7.0, -5.0, -1.0, 0.0], [3.0, 1.0, -5.0, 5.0, 5.0], [2.0, 4.0, 6.0, 2.0, 1.0]], [[0.0, -3.0, -3.0, 3.0, 1.0], [3.0, 5.0, -4.0, 6.0, 3.0], [5.0, -6.0, 0.0, 2.0, 2.0], [-1.0, 3.0, 7.0, 4.0, -3.0]], [[6.0, -4.0, 1.0, 4.0, -5.0], [-4.0, 5.0, 1.0, 3.0, 5.0], [-2.0, 2.0, 4.0, 1.0, 1.0], [0.0, 4.0, -6.0, 4.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-9.0, -42.0, -48.0, 33.0, -21.0], [9.0, 3.0, 39.0, -3.0, -6.0], [-6.0, 21.0, -27.0, 48.0, 27.0], [-12.0, 45.0, 21.0, -36.0, 3.0]], [[-20.0, -32.0, 120.0, 3.0, -21.0], [-33.0, -55.0, -39.0, 35.0, 24.0], [22.0, -52.0, 31.0, -26.0, -82.0], [39.0, -53.0, -23.0, 118.0, 0.0]], [[18.0, 72.0, -70.0, -36.0, -38.0], [22.0, 63.0, -26.0, -47.0, 8.0], [51.0, -41.0, -66.0, -52.0, 27.0], [-10.0, -8.0, 23.0, -19.0, -61.0]], [[-23.0, -15.0, -55.0, 21.0, -9.0], [32.0, 10.0, -26.0, -1.0, 4.0], [-31.0, 17.0, -66.0, 45.0, 43.0], [-57.0, -22.0, 26.0, -69.0, -25.0]], [[43.0, -2.0, 80.0, -35.0, 37.0], [-77.0, -28.0, 15.0, 18.0, 42.0], [27.0, -16.0, 85.0, -32.0, -40.0], [56.0, 47.0, -56.0, 35.0, 58.0]]], [[[-8.0, 16.0, 12.0, 7.0, 67.0], [35.0, -9.0, 15.0, 19.0, -64.0], [-68.0, 64.0, 59.0, 16.0, 18.0], [-15.0, -25.0, -1.0, -24.0, 38.0]], [[-20.0, -93.0, -53.0, 66.0, 22.0], [-9.0, -35.0, 102.0, -15.0, -2.0], [-71.0, 95.0, 31.0, 86.0, -7.0], [-8.0, 73.0, -19.0, -31.0, 48.0]], [[15.0, 5.0, 45.0, -26.0, -48.0], [-33.0, 3.0, -50.0, 19.0, 44.0], [75.0, -84.0, -35.0, -30.0, -8.0], [21.0, -1.0, 7.0, 29.0, -23.0]], [[33.0, 3.0, -57.0, -16.0, 38.0], [-17.0, 23.0, 0.0, 7.0, 28.0], [-15.0, 28.0, -19.0, 42.0, 80.0], [-39.0, 59.0, -1.0, -137.0, 31.0]], [[-21.0, -38.0, -21.0, 17.0, -27.0], [3.0, -6.0, -57.0, 20.0, 44.0], [-7.0, -19.0, -84.0, 51.0, 36.0], [-55.0, -22.0, 21.0, -65.0, -26.0]]], [[[-45.0, -77.0, 44.0, 47.0, 21.0], [-16.0, -70.0, 8.0, 7.0, 26.0], [-73.0, 46.0, 31.0, 42.0, -71.0], [-10.0, -41.0, -41.0, 51.0, 19.0]], [[-10.0, 59.0, -50.0, 1.0, -23.0], [84.0, 54.0, 2.0, -5.0, -96.0], [13.0, -3.0, -33.0, -16.0, 56.0], [-13.0, -25.0, 69.0, -13.0, -38.0]], [[3.0, -9.0, 13.0, -15.0, -39.0], [-14.0, 7.0, -85.0, 37.0, 52.0], [44.0, -69.0, -84.0, 18.0, 45.0], [-34.0, -16.0, 27.0, -56.0, -28.0]], [[9.0, 41.0, -86.0, -12.0, 6.0], [25.0, 43.0, 6.0, -43.0, -2.0], [-11.0, 23.0, -36.0, -7.0, 36.0], [-30.0, 6.0, 8.0, -59.0, -25.0]], [[-24.0, -86.0, 17.0, 46.0, -10.0], [-42.0, -56.0, 59.0, -19.0, 42.0], [-28.0, 32.0, 33.0, 30.0, -81.0], [26.0, 27.0, -45.0, 70.0, 20.0]]], [[[-17.0, 19.0, -23.0, 8.0, -4.0], [41.0, 14.0, -6.0, -8.0, -36.0], [-16.0, 11.0, -17.0, -2.0, 11.0], [-17.0, -33.0, 22.0, 0.0, -22.0]], [[-2.0, -87.0, -21.0, 50.0, 18.0], [-33.0, -37.0, 105.0, -6.0, 6.0], [-39.0, 70.0, 53.0, 65.0, -18.0], [21.0, 93.0, -27.0, -7.0, 60.0]], [[-18.0, -55.0, 22.0, 38.0, 78.0], [-29.0, -65.0, 91.0, -28.0, 0.0], [-103.0, 111.0, 120.0, 22.0, -90.0], [24.0, 12.0, -80.0, 59.0, 66.0]], [[-35.0, -47.0, -23.0, 32.0, -14.0], [15.0, -17.0, -41.0, 14.0, 28.0], [-37.0, 9.0, -67.0, 59.0, 21.0], [-58.0, -35.0, 15.0, -51.0, -21.0]], [[37.0, 110.0, -26.0, -61.0, 11.0], [11.0, 56.0, -4.0, -51.0, -18.0], [39.0, -21.0, 19.0, -97.0, -7.0], [27.0, -13.0, -10.0, 27.0, -28.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[1.0, 1.0, 2.0, -2.0, 3.0], [7.0, -1.0, 3.0, -4.0, -4.0], [1.0, 5.0, 7.0, -3.0, 0.0], [7.0, 7.0, 4.0, 2.0, 2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 1.0, 2.0, -2.0, 3.0], [7.0, -1.0, 3.0, -4.0, -4.0], [1.0, 5.0, 7.0, -3.0, 0.0], [7.0, 7.0, 4.0, 2.0, 2.0]])+(1.-msk_ref)*numpy.array([[-3.0, -3.0, -6.0, 6.0, -9.0], [-21.0, 3.0, -9.0, 12.0, 12.0], [-3.0, -15.0, -21.0, 9.0, -0.0], [-21.0, -21.0, -12.0, -6.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, 6.0, 7.0])+(1.-msk_arg1)*numpy.array([-4.0, 3.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, -3.0, 0.0, 6.0, -3.0], [5.0, 3.0, 2.0, -3.0, 0.0], [1.0, 3.0, -5.0, 6.0, 7.0], [0.0, -1.0, 3.0, -3.0, 6.0]], [[0.0, -3.0, 2.0, -1.0, -7.0], [-2.0, 5.0, 0.0, 5.0, 7.0], [0.0, -3.0, 5.0, 0.0, 0.0], [-3.0, 0.0, 1.0, -7.0, -3.0]], [[2.0, -4.0, 4.0, -2.0, -3.0], [1.0, -2.0, -2.0, 1.0, -6.0], [2.0, -2.0, 3.0, 0.0, 6.0], [-2.0, -1.0, -7.0, -4.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[20.0, -55.0, 40.0, -2.0, -72.0], [10.0, 25.0, -8.0, 28.0, 0.0], [17.0, -23.0, 36.0, 18.0, 63.0], [-32.0, -10.0, -34.0, -79.0, -35.0]])+(1.-msk_ref)*numpy.array([[-18.0, 23.0, -14.0, -17.0, 6.0], [-31.0, 13.0, 2.0, 22.0, 51.0], [-14.0, -11.0, 20.0, -24.0, -58.0], [1.0, 9.0, 26.0, 11.0, -8.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, 7.0, 5.0], [1.0, -5.0, -5.0]])+(1.-msk_arg1)*numpy.array([[2.0, 1.0, -7.0], [-5.0, 3.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-4.0, 4.0, -4.0, 3.0, 6.0], [7.0, -6.0, -4.0, 3.0, -4.0], [4.0, -7.0, 1.0, -6.0, 4.0], [-4.0, 4.0, -7.0, 1.0, 4.0]], [[1.0, 6.0, 7.0, 4.0, 4.0], [7.0, 4.0, -2.0, -4.0, 0.0], [-7.0, -5.0, -3.0, -7.0, 0.0], [-2.0, 5.0, 7.0, -7.0, 0.0]], [[-7.0, 5.0, -2.0, -5.0, -7.0], [-3.0, -6.0, 4.0, -1.0, 4.0], [-5.0, 2.0, 1.0, -7.0, -3.0], [6.0, 4.0, -4.0, 5.0, 5.0]]], [[[-2.0, 4.0, 6.0, 7.0, 1.0], [0.0, -2.0, -7.0, 4.0, -6.0], [4.0, 7.0, 0.0, 4.0, -7.0], [0.0, 1.0, -3.0, 6.0, -3.0]], [[-2.0, 2.0, -1.0, 3.0, 2.0], [3.0, -6.0, 6.0, 1.0, -6.0], [6.0, 0.0, 4.0, 7.0, -1.0], [-4.0, 6.0, -6.0, 2.0, -3.0]], [[7.0, 7.0, 1.0, 0.0, 4.0], [0.0, -3.0, 5.0, -6.0, 6.0], [-7.0, 7.0, 7.0, -3.0, 6.0], [0.0, 7.0, 0.0, -7.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-83.0, 54.0, 17.0, 16.0, 6.0], [68.0, -1.0, -84.0, 17.0, -14.0], [-37.0, -102.0, -64.0, -142.0, -19.0], [8.0, 19.0, 7.0, 14.0, 60.0]])+(1.-msk_ref)*numpy.array([[46.0, -35.0, -20.0, 19.0, 66.0], [51.0, 26.0, 15.0, -8.0, -24.0], [34.0, -68.0, 4.0, 31.0, 61.0], [-64.0, -2.0, 18.0, -64.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, -6.0])+(1.-msk_arg1)*numpy.array([5.0, 7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[1.0, 5.0, 0.0, 6.0, 0.0], [-7.0, 2.0, 6.0, -4.0, -5.0], [6.0, 1.0, 3.0, 6.0, 1.0], [4.0, 7.0, 1.0, 6.0, 4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-5.0, -6.0], [-25.0, -30.0], [0.0, 0.0], [-30.0, -36.0], [0.0, 0.0]], [[35.0, 42.0], [-10.0, -12.0], [-30.0, -36.0], [20.0, 24.0], [25.0, 30.0]], [[-30.0, -36.0], [-5.0, -6.0], [-15.0, -18.0], [-30.0, -36.0], [-5.0, -6.0]], [[-20.0, -24.0], [-35.0, -42.0], [-5.0, -6.0], [-30.0, -36.0], [-20.0, -24.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 7.0], [25.0, 35.0], [0.0, 0.0], [30.0, 42.0], [0.0, 0.0]], [[-35.0, -49.0], [10.0, 14.0], [30.0, 42.0], [-20.0, -28.0], [-25.0, -35.0]], [[30.0, 42.0], [5.0, 7.0], [15.0, 21.0], [30.0, 42.0], [5.0, 7.0]], [[20.0, 28.0], [35.0, 49.0], [5.0, 7.0], [30.0, 42.0], [20.0, 28.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, -3.0], [-6.0, -4.0], [-5.0, 0.0]])+(1.-msk_arg1)*numpy.array([[3.0, 6.0], [2.0, -3.0], [-1.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-4.0, -1.0, 4.0, -4.0, 4.0], [-7.0, 1.0, -6.0, -6.0, -4.0], [-2.0, -4.0, 3.0, 6.0, -5.0], [3.0, 2.0, -3.0, -6.0, -4.0]], [[-3.0, -1.0, -2.0, -3.0, 1.0], [5.0, 5.0, -2.0, -5.0, -2.0], [-5.0, 4.0, -1.0, 1.0, 6.0], [3.0, 5.0, 0.0, 5.0, -2.0]], [[0.0, 6.0, 1.0, -3.0, -5.0], [6.0, -4.0, 6.0, -1.0, 2.0], [1.0, -7.0, -2.0, -7.0, 0.0], [-2.0, -6.0, 6.0, 6.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[38.0, 24.0], [-19.0, 7.0], [-13.0, -4.0], [53.0, 24.0], [-1.0, -16.0]], [[-25.0, 1.0], [-15.0, -23.0], [12.0, 26.0], [65.0, 38.0], [22.0, 20.0]], [[35.0, 26.0], [31.0, -4.0], [1.0, -5.0], [-1.0, -22.0], [-11.0, -9.0]], [[-23.0, -21.0], [-10.0, -26.0], [-15.0, 9.0], [-30.0, -2.0], [12.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, -15.0], [-11.0, -27.0], [7.0, 26.0], [-15.0, -3.0], [19.0, 41.0]], [[-17.0, -81.0], [17.0, 7.0], [-28.0, -54.0], [-27.0, -17.0], [-18.0, -26.0]], [[-17.0, -1.0], [3.0, -8.0], [9.0, 29.0], [27.0, 61.0], [-3.0, -48.0]], [[17.0, 17.0], [22.0, 21.0], [-15.0, -42.0], [-14.0, -75.0], [-20.0, -34.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-7.0, -2.0], [-3.0, -1.0], [0.0, -5.0]], [[7.0, 6.0], [-3.0, 0.0], [3.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 1.0], [-5.0, 5.0], [1.0, -7.0]], [[3.0, -1.0], [-3.0, 7.0], [-4.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-6.0, -3.0, -6.0, -7.0, -1.0], [-1.0, 2.0, -6.0, -6.0, 3.0], [4.0, -7.0, -1.0, 4.0, -4.0], [4.0, 4.0, 5.0, 0.0, -5.0]], [[5.0, 5.0, -7.0, 2.0, 5.0], [-1.0, -5.0, 5.0, -4.0, -1.0], [-4.0, -7.0, 3.0, 0.0, -6.0], [0.0, 3.0, 5.0, 7.0, 0.0]], [[2.0, -4.0, 1.0, -4.0, -1.0], [-1.0, 7.0, -4.0, 4.0, 0.0], [-2.0, -2.0, -4.0, 2.0, -7.0], [3.0, 6.0, 1.0, 1.0, -7.0]]], [[[3.0, -7.0, 6.0, -5.0, -3.0], [-4.0, 5.0, -1.0, 0.0, 0.0], [-5.0, -6.0, 1.0, -4.0, 7.0], [-3.0, -2.0, -6.0, 7.0, 7.0]], [[-7.0, -1.0, -6.0, 7.0, 5.0], [4.0, 1.0, 3.0, 7.0, 6.0], [4.0, -1.0, 4.0, -4.0, 1.0], [0.0, 1.0, 0.0, 7.0, 7.0]], [[4.0, 0.0, 5.0, 3.0, 5.0], [0.0, 2.0, -4.0, 0.0, 5.0], [3.0, 3.0, 1.0, 3.0, 7.0], [-6.0, 4.0, 4.0, 3.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[81.0, 35.0], [-40.0, -21.0], [138.0, 75.0], [-4.0, 17.0], [-29.0, 9.0]], [[-30.0, -16.0], [39.0, 6.0], [-1.0, 1.0], [33.0, -4.0], [-21.0, 20.0]], [[-54.0, -9.0], [40.0, 10.0], [-4.0, 30.0], [-35.0, -27.0], [113.0, 126.0]], [[-67.0, -71.0], [-42.0, -33.0], [-80.0, -36.0], [16.0, 45.0], [81.0, 117.0]]])+(1.-msk_ref)*numpy.array([[[3.0, -47.0], [-41.0, 50.0], [64.0, -96.0], [-48.0, 85.0], [-68.0, 69.0]], [[-18.0, 33.0], [32.0, -70.0], [-13.0, 69.0], [15.0, -5.0], [-39.0, 40.0]], [[-29.0, 31.0], [20.0, -29.0], [-30.0, 69.0], [-18.0, -34.0], [21.0, 15.0]], [[10.0, -14.0], [-42.0, -14.0], [-68.0, 29.0], [-46.0, 70.0], [-21.0, 86.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank2_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 0.0, 1.0, -6.0, -6.0], [5.0, -6.0, -2.0, -4.0, -5.0], [-3.0, 0.0, 4.0, 0.0, 7.0], [3.0, 6.0, -5.0, 3.0, -4.0]])+(1.-msk_arg1)*numpy.array([[3.0, 4.0, -4.0, -5.0, 1.0], [0.0, 5.0, 6.0, 3.0, 3.0], [0.0, 0.0, 1.0, -1.0, 6.0], [0.0, -2.0, 4.0, -5.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[4.0, -2.0, 0.0, -4.0, -5.0], [-4.0, 7.0, 3.0, -7.0, 1.0], [-3.0, 7.0, 3.0, 5.0, 1.0], [2.0, 5.0, -5.0, 4.0, 7.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-24.0, 0.0, 4.0, -24.0, -24.0], [20.0, -24.0, -8.0, -16.0, -20.0], [-12.0, 0.0, 16.0, 0.0, 28.0], [12.0, 24.0, -20.0, 12.0, -16.0]], [[12.0, 0.0, -2.0, 12.0, 12.0], [-10.0, 12.0, 4.0, 8.0, 10.0], [6.0, 0.0, -8.0, 0.0, -14.0], [-6.0, -12.0, 10.0, -6.0, 8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[24.0, 0.0, -4.0, 24.0, 24.0], [-20.0, 24.0, 8.0, 16.0, 20.0], [12.0, 0.0, -16.0, 0.0, -28.0], [-12.0, -24.0, 20.0, -12.0, 16.0]], [[30.0, 0.0, -5.0, 30.0, 30.0], [-25.0, 30.0, 10.0, 20.0, 25.0], [15.0, 0.0, -20.0, 0.0, -35.0], [-15.0, -30.0, 25.0, -15.0, 20.0]]], [[[24.0, 0.0, -4.0, 24.0, 24.0], [-20.0, 24.0, 8.0, 16.0, 20.0], [12.0, 0.0, -16.0, 0.0, -28.0], [-12.0, -24.0, 20.0, -12.0, 16.0]], [[-42.0, 0.0, 7.0, -42.0, -42.0], [35.0, -42.0, -14.0, -28.0, -35.0], [-21.0, 0.0, 28.0, 0.0, 49.0], [21.0, 42.0, -35.0, 21.0, -28.0]], [[-18.0, 0.0, 3.0, -18.0, -18.0], [15.0, -18.0, -6.0, -12.0, -15.0], [-9.0, 0.0, 12.0, 0.0, 21.0], [9.0, 18.0, -15.0, 9.0, -12.0]], [[42.0, 0.0, -7.0, 42.0, 42.0], [-35.0, 42.0, 14.0, 28.0, 35.0], [21.0, 0.0, -28.0, 0.0, -49.0], [-21.0, -42.0, 35.0, -21.0, 28.0]], [[-6.0, 0.0, 1.0, -6.0, -6.0], [5.0, -6.0, -2.0, -4.0, -5.0], [-3.0, 0.0, 4.0, 0.0, 7.0], [3.0, 6.0, -5.0, 3.0, -4.0]]], [[[18.0, 0.0, -3.0, 18.0, 18.0], [-15.0, 18.0, 6.0, 12.0, 15.0], [9.0, 0.0, -12.0, 0.0, -21.0], [-9.0, -18.0, 15.0, -9.0, 12.0]], [[-42.0, 0.0, 7.0, -42.0, -42.0], [35.0, -42.0, -14.0, -28.0, -35.0], [-21.0, 0.0, 28.0, 0.0, 49.0], [21.0, 42.0, -35.0, 21.0, -28.0]], [[-18.0, 0.0, 3.0, -18.0, -18.0], [15.0, -18.0, -6.0, -12.0, -15.0], [-9.0, 0.0, 12.0, 0.0, 21.0], [9.0, 18.0, -15.0, 9.0, -12.0]], [[-30.0, 0.0, 5.0, -30.0, -30.0], [25.0, -30.0, -10.0, -20.0, -25.0], [-15.0, 0.0, 20.0, 0.0, 35.0], [15.0, 30.0, -25.0, 15.0, -20.0]], [[-6.0, 0.0, 1.0, -6.0, -6.0], [5.0, -6.0, -2.0, -4.0, -5.0], [-3.0, 0.0, 4.0, 0.0, 7.0], [3.0, 6.0, -5.0, 3.0, -4.0]]], [[[-12.0, 0.0, 2.0, -12.0, -12.0], [10.0, -12.0, -4.0, -8.0, -10.0], [-6.0, 0.0, 8.0, 0.0, 14.0], [6.0, 12.0, -10.0, 6.0, -8.0]], [[-30.0, 0.0, 5.0, -30.0, -30.0], [25.0, -30.0, -10.0, -20.0, -25.0], [-15.0, 0.0, 20.0, 0.0, 35.0], [15.0, 30.0, -25.0, 15.0, -20.0]], [[30.0, 0.0, -5.0, 30.0, 30.0], [-25.0, 30.0, 10.0, 20.0, 25.0], [15.0, 0.0, -20.0, 0.0, -35.0], [-15.0, -30.0, 25.0, -15.0, 20.0]], [[-24.0, 0.0, 4.0, -24.0, -24.0], [20.0, -24.0, -8.0, -16.0, -20.0], [-12.0, 0.0, 16.0, 0.0, 28.0], [12.0, 24.0, -20.0, 12.0, -16.0]], [[-42.0, 0.0, 7.0, -42.0, -42.0], [35.0, -42.0, -14.0, -28.0, -35.0], [-21.0, 0.0, 28.0, 0.0, 49.0], [21.0, 42.0, -35.0, 21.0, -28.0]]]])+(1.-msk_ref)*numpy.array([[[[12.0, 16.0, -16.0, -20.0, 4.0], [0.0, 20.0, 24.0, 12.0, 12.0], [0.0, 0.0, 4.0, -4.0, 24.0], [0.0, -8.0, 16.0, -20.0, -28.0]], [[-6.0, -8.0, 8.0, 10.0, -2.0], [0.0, -10.0, -12.0, -6.0, -6.0], [0.0, 0.0, -2.0, 2.0, -12.0], [0.0, 4.0, -8.0, 10.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, -16.0, 16.0, 20.0, -4.0], [0.0, -20.0, -24.0, -12.0, -12.0], [0.0, 0.0, -4.0, 4.0, -24.0], [0.0, 8.0, -16.0, 20.0, 28.0]], [[-15.0, -20.0, 20.0, 25.0, -5.0], [0.0, -25.0, -30.0, -15.0, -15.0], [0.0, 0.0, -5.0, 5.0, -30.0], [0.0, 10.0, -20.0, 25.0, 35.0]]], [[[-12.0, -16.0, 16.0, 20.0, -4.0], [0.0, -20.0, -24.0, -12.0, -12.0], [0.0, 0.0, -4.0, 4.0, -24.0], [0.0, 8.0, -16.0, 20.0, 28.0]], [[21.0, 28.0, -28.0, -35.0, 7.0], [0.0, 35.0, 42.0, 21.0, 21.0], [0.0, 0.0, 7.0, -7.0, 42.0], [0.0, -14.0, 28.0, -35.0, -49.0]], [[9.0, 12.0, -12.0, -15.0, 3.0], [0.0, 15.0, 18.0, 9.0, 9.0], [0.0, 0.0, 3.0, -3.0, 18.0], [0.0, -6.0, 12.0, -15.0, -21.0]], [[-21.0, -28.0, 28.0, 35.0, -7.0], [0.0, -35.0, -42.0, -21.0, -21.0], [0.0, 0.0, -7.0, 7.0, -42.0], [0.0, 14.0, -28.0, 35.0, 49.0]], [[3.0, 4.0, -4.0, -5.0, 1.0], [0.0, 5.0, 6.0, 3.0, 3.0], [0.0, 0.0, 1.0, -1.0, 6.0], [0.0, -2.0, 4.0, -5.0, -7.0]]], [[[-9.0, -12.0, 12.0, 15.0, -3.0], [0.0, -15.0, -18.0, -9.0, -9.0], [0.0, 0.0, -3.0, 3.0, -18.0], [0.0, 6.0, -12.0, 15.0, 21.0]], [[21.0, 28.0, -28.0, -35.0, 7.0], [0.0, 35.0, 42.0, 21.0, 21.0], [0.0, 0.0, 7.0, -7.0, 42.0], [0.0, -14.0, 28.0, -35.0, -49.0]], [[9.0, 12.0, -12.0, -15.0, 3.0], [0.0, 15.0, 18.0, 9.0, 9.0], [0.0, 0.0, 3.0, -3.0, 18.0], [0.0, -6.0, 12.0, -15.0, -21.0]], [[15.0, 20.0, -20.0, -25.0, 5.0], [0.0, 25.0, 30.0, 15.0, 15.0], [0.0, 0.0, 5.0, -5.0, 30.0], [0.0, -10.0, 20.0, -25.0, -35.0]], [[3.0, 4.0, -4.0, -5.0, 1.0], [0.0, 5.0, 6.0, 3.0, 3.0], [0.0, 0.0, 1.0, -1.0, 6.0], [0.0, -2.0, 4.0, -5.0, -7.0]]], [[[6.0, 8.0, -8.0, -10.0, 2.0], [0.0, 10.0, 12.0, 6.0, 6.0], [0.0, 0.0, 2.0, -2.0, 12.0], [0.0, -4.0, 8.0, -10.0, -14.0]], [[15.0, 20.0, -20.0, -25.0, 5.0], [0.0, 25.0, 30.0, 15.0, 15.0], [0.0, 0.0, 5.0, -5.0, 30.0], [0.0, -10.0, 20.0, -25.0, -35.0]], [[-15.0, -20.0, 20.0, 25.0, -5.0], [0.0, -25.0, -30.0, -15.0, -15.0], [0.0, 0.0, -5.0, 5.0, -30.0], [0.0, 10.0, -20.0, 25.0, 35.0]], [[12.0, 16.0, -16.0, -20.0, 4.0], [0.0, 20.0, 24.0, 12.0, 12.0], [0.0, 0.0, 4.0, -4.0, 24.0], [0.0, -8.0, 16.0, -20.0, -28.0]], [[21.0, 28.0, -28.0, -35.0, 7.0], [0.0, 35.0, 42.0, 21.0, 21.0], [0.0, 0.0, 7.0, -7.0, 42.0], [0.0, -14.0, 28.0, -35.0, -49.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 5.0, -5.0, 7.0, -6.0], [1.0, 0.0, 7.0, 7.0, -6.0], [1.0, -3.0, -4.0, 5.0, 0.0], [-5.0, 5.0, 7.0, -4.0, 3.0]], [[0.0, -4.0, 6.0, 0.0, -2.0], [0.0, 5.0, -2.0, -5.0, 0.0], [-1.0, 0.0, 0.0, -7.0, 6.0], [2.0, 7.0, 1.0, 3.0, -6.0]], [[-3.0, 2.0, 3.0, -5.0, 7.0], [3.0, -4.0, 0.0, 6.0, 7.0], [2.0, -6.0, 2.0, 7.0, 5.0], [0.0, -6.0, 0.0, -6.0, -1.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, -7.0, -7.0, -1.0, -5.0], [-7.0, 0.0, 2.0, 0.0, 0.0], [4.0, -1.0, 1.0, -5.0, -1.0], [-4.0, 1.0, 4.0, 4.0, -2.0]], [[-3.0, 6.0, 3.0, 1.0, 2.0], [4.0, -4.0, 2.0, 4.0, 1.0], [2.0, 1.0, -2.0, -6.0, -2.0], [-3.0, 2.0, 7.0, -4.0, -2.0]], [[-6.0, -7.0, 1.0, -2.0, -5.0], [1.0, 1.0, 4.0, 3.0, 0.0], [6.0, 0.0, -3.0, -4.0, -5.0], [-6.0, 5.0, -2.0, 3.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[1.0, -1.0, -2.0, -3.0, -1.0], [7.0, -3.0, 0.0, 0.0, 0.0], [1.0, 3.0, 2.0, 4.0, 6.0], [0.0, 3.0, 2.0, 2.0, 4.0]], [[0.0, 4.0, -4.0, 6.0, 4.0], [-6.0, -7.0, -1.0, -1.0, 4.0], [2.0, -3.0, -2.0, 5.0, 2.0], [-1.0, -1.0, -7.0, -1.0, 5.0]], [[-5.0, 3.0, 0.0, 2.0, 6.0], [-6.0, -1.0, -7.0, 6.0, 2.0], [4.0, -2.0, -6.0, 7.0, -3.0], [-1.0, 7.0, -2.0, 7.0, -4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[15.0, -5.0, -20.0, 32.0, -41.0], [-14.0, 20.0, 7.0, -23.0, -41.0], [-9.0, 27.0, -14.0, -30.0, -25.0], [-5.0, 35.0, 7.0, 26.0, 8.0]], [[-9.0, -15.0, 38.0, -22.0, 19.0], [8.0, 8.0, -15.0, -9.0, 27.0], [1.0, -15.0, 10.0, -12.0, 39.0], [13.0, 5.0, -3.0, -2.0, -30.0]], [[0.0, 6.0, -14.0, -14.0, 20.0], [-2.0, -20.0, -6.0, 6.0, 12.0], [2.0, 6.0, 8.0, 18.0, -24.0], [2.0, -38.0, -18.0, -4.0, 18.0]], [[-6.0, -35.0, 57.0, -31.0, 20.0], [3.0, 22.0, -33.0, -39.0, 32.0], [-5.0, -3.0, 16.0, -43.0, 46.0], [27.0, 15.0, -15.0, 18.0, -47.0]], [[-18.0, -9.0, 47.0, -37.0, 40.0], [17.0, -4.0, -15.0, 9.0, 48.0], [7.0, -33.0, 16.0, 9.0, 54.0], [13.0, -13.0, -3.0, -20.0, -33.0]]], [[[18.0, 47.0, -89.0, 79.0, -72.0], [-11.0, -6.0, 61.0, 43.0, -84.0], [1.0, 15.0, -40.0, 35.0, -66.0], [-47.0, 29.0, 43.0, -10.0, 63.0]], [[3.0, 11.0, -30.0, -16.0, 25.0], [-6.0, -31.0, -7.0, 8.0, 11.0], [2.0, 15.0, 10.0, 27.0, -47.0], [1.0, -58.0, -28.0, -3.0, 34.0]], [[21.0, -10.0, -27.0, 35.0, -47.0], [-21.0, 23.0, 2.0, -37.0, -49.0], [-13.0, 42.0, -14.0, -42.0, -41.0], [-2.0, 35.0, -1.0, 39.0, 13.0]], [[-18.0, 16.0, 12.0, -30.0, 44.0], [18.0, -29.0, 2.0, 41.0, 42.0], [13.0, -36.0, 12.0, 49.0, 24.0], [-2.0, -43.0, -1.0, -39.0, 0.0]], [[-6.0, -12.0, 30.0, -10.0, 6.0], [6.0, 12.0, -8.0, -8.0, 14.0], [0.0, -12.0, 4.0, -14.0, 34.0], [8.0, 16.0, 4.0, 0.0, -26.0]]], [[[-12.0, 5.0, 19.0, -13.0, 18.0], [13.0, -6.0, 3.0, 21.0, 22.0], [7.0, -27.0, 4.0, 19.0, 32.0], [-1.0, -5.0, 9.0, -22.0, -13.0]], [[6.0, 23.0, -39.0, 31.0, -26.0], [-3.0, -7.0, 27.0, 24.0, -32.0], [2.0, 3.0, -16.0, 22.0, -28.0], [-21.0, 6.0, 18.0, -9.0, 29.0]], [[18.0, 6.0, -40.0, 44.0, -50.0], [-16.0, 14.0, 18.0, -12.0, -54.0], [-8.0, 30.0, -20.0, -18.0, -42.0], [-14.0, 32.0, 12.0, 22.0, 24.0]], [[-21.0, 14.0, 31.0, -7.0, 15.0], [25.0, -3.0, 18.0, 45.0, 25.0], [13.0, -54.0, -2.0, 34.0, 65.0], [-10.0, 13.0, 33.0, -43.0, -25.0]], [[9.0, 16.0, -27.0, 57.0, -61.0], [-3.0, 22.0, 38.0, 14.0, -57.0], [-2.0, 0.0, -30.0, -5.0, -3.0], [-26.0, 62.0, 44.0, 0.0, 9.0]]], [[[3.0, 2.0, -9.0, 5.0, -5.0], [-3.0, -1.0, 2.0, -1.0, -7.0], [-1.0, 6.0, -2.0, 0.0, -11.0], [-2.0, -1.0, -1.0, 3.0, 7.0]], [[-21.0, 33.0, 0.0, -14.0, 33.0], [24.0, -33.0, 23.0, 68.0, 31.0], [18.0, -51.0, 2.0, 71.0, 29.0], [-17.0, -34.0, 20.0, -57.0, 8.0]], [[6.0, 34.0, -58.0, 24.0, -12.0], [-4.0, -27.0, 28.0, 37.0, -26.0], [5.0, 6.0, -12.0, 45.0, -52.0], [-24.0, -27.0, 7.0, -17.0, 50.0]], [[-21.0, 28.0, 5.0, -21.0, 39.0], [23.0, -33.0, 16.0, 61.0, 37.0], [17.0, -48.0, 6.0, 66.0, 29.0], [-12.0, -39.0, 13.0, -53.0, 5.0]], [[12.0, -8.0, -2.0, 48.0, -62.0], [-8.0, 41.0, 18.0, -21.0, -52.0], [-9.0, 12.0, -24.0, -43.0, 10.0], [-10.0, 79.0, 33.0, 23.0, -14.0]]]])+(1.-msk_ref)*numpy.array([[[[27.0, 28.0, -12.0, 9.0, 20.0], [-12.0, -5.0, -18.0, -15.0, 0.0], [-26.0, -1.0, 16.0, 15.0, 24.0], [26.0, -24.0, 14.0, -11.0, -17.0]], [[-27.0, 10.0, 22.0, -1.0, -2.0], [26.0, -13.0, 18.0, 25.0, 4.0], [22.0, 5.0, -18.0, -31.0, -22.0], [-26.0, 22.0, 18.0, -11.0, 3.0]], [[18.0, -10.0, 2.0, -2.0, 2.0], [-2.0, 16.0, -12.0, -16.0, -4.0], [-16.0, -2.0, 6.0, 34.0, 10.0], [20.0, -10.0, -36.0, 8.0, 12.0]], [[-21.0, 43.0, 41.0, 5.0, 17.0], [47.0, -22.0, 14.0, 30.0, 6.0], [12.0, 9.0, -21.0, -29.0, -19.0], [-18.0, 19.0, 26.0, -30.0, 0.0]], [[-45.0, -11.0, 25.0, -7.0, -17.0], [29.0, -10.0, 30.0, 34.0, 4.0], [40.0, 5.0, -27.0, -43.0, -37.0], [-44.0, 37.0, 12.0, -2.0, 12.0]]], [[[33.0, -43.0, -73.0, -1.0, -17.0], [-79.0, 18.0, -22.0, -42.0, -6.0], [-20.0, -13.0, 37.0, 25.0, 35.0], [26.0, -35.0, -2.0, 34.0, -20.0]], [[36.0, -14.0, -1.0, -2.0, 6.0], [-8.0, 27.0, -24.0, -31.0, -7.0], [-32.0, -4.0, 14.0, 61.0, 22.0], [39.0, -22.0, -59.0, 13.0, 17.0]], [[45.0, 43.0, -10.0, 13.0, 33.0], [-11.0, -3.0, -30.0, -25.0, -1.0], [-44.0, -1.0, 23.0, 34.0, 37.0], [45.0, -37.0, 7.0, -17.0, -19.0]], [[-33.0, -48.0, 3.0, -13.0, -32.0], [2.0, 10.0, 22.0, 14.0, -1.0], [34.0, -1.0, -16.0, -18.0, -28.0], [-33.0, 28.0, -19.0, 22.0, 20.0]], [[-24.0, 10.0, 14.0, 0.0, -2.0], [18.0, -14.0, 16.0, 22.0, 4.0], [20.0, 4.0, -14.0, -32.0, -18.0], [-24.0, 18.0, 24.0, -10.0, -2.0]]], [[[-33.0, -23.0, 3.0, -7.0, -21.0], [5.0, -4.0, 22.0, 20.0, 2.0], [32.0, 1.0, -15.0, -33.0, -25.0], [-34.0, 25.0, 10.0, 8.0, 6.0]], [[12.0, -25.0, -32.0, -2.0, -11.0], [-35.0, 10.0, -8.0, -18.0, -3.0], [-6.0, -6.0, 15.0, 11.0, 13.0], [9.0, -13.0, -5.0, 18.0, -6.0]], [[36.0, 16.0, -26.0, 8.0, 16.0], [-28.0, 2.0, -24.0, -26.0, -2.0], [-32.0, -4.0, 24.0, 26.0, 32.0], [34.0, -32.0, 6.0, -2.0, -18.0]], [[-69.0, -47.0, -6.0, -13.0, -45.0], [-1.0, -13.0, 46.0, 41.0, 5.0], [68.0, 1.0, -27.0, -78.0, -49.0], [-73.0, 49.0, 37.0, 17.0, 3.0]], [[-6.0, -9.0, -39.0, 2.0, -11.0], [-37.0, -11.0, 4.0, -1.0, 2.0], [10.0, -4.0, 11.0, -30.0, 5.0], [-12.0, -5.0, 44.0, 7.0, -25.0]]], [[[9.0, 1.0, -4.0, 1.0, 3.0], [-5.0, 3.0, -6.0, -7.0, -1.0], [-8.0, -1.0, 5.0, 10.0, 7.0], [9.0, -7.0, -5.0, 1.0, -1.0]], [[-48.0, -76.0, -17.0, -18.0, -52.0], [-18.0, 11.0, 32.0, 17.0, -1.0], [52.0, -4.0, -16.0, -37.0, -36.0], [-51.0, 36.0, -9.0, 37.0, 17.0]], [[27.0, -42.0, -37.0, -5.0, -14.0], [-44.0, 26.0, -18.0, -34.0, -7.0], [-18.0, -9.0, 22.0, 40.0, 22.0], [25.0, -22.0, -37.0, 30.0, 4.0]], [[-45.0, -69.0, -10.0, -17.0, -47.0], [-11.0, 11.0, 30.0, 17.0, -1.0], [48.0, -3.0, -17.0, -32.0, -35.0], [-47.0, 35.0, -13.0, 33.0, 19.0]], [[-3.0, 30.0, -17.0, 9.0, 10.0], [-12.0, -24.0, 2.0, 8.0, 5.0], [2.0, 1.0, 6.0, -34.0, 6.0], [-7.0, -6.0, 59.0, -16.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3, 4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, 3.0, -2.0, 0.0, 3.0], [0.0, -2.0, -4.0, 1.0, -3.0], [2.0, 0.0, 4.0, -5.0, 0.0], [1.0, 0.0, -1.0, -2.0, -2.0]], [[5.0, 4.0, 5.0, 6.0, 5.0], [0.0, 2.0, 0.0, -6.0, -1.0], [6.0, -7.0, -2.0, -5.0, -3.0], [2.0, -4.0, -5.0, -5.0, 3.0]], [[0.0, 1.0, -4.0, -1.0, -7.0], [0.0, 0.0, -1.0, 7.0, 3.0], [6.0, 1.0, 7.0, 1.0, 0.0], [-6.0, -7.0, 7.0, -1.0, -4.0]]], [[[7.0, -5.0, 2.0, -6.0, 2.0], [-1.0, -2.0, -3.0, -5.0, 0.0], [-4.0, 4.0, -4.0, -6.0, 6.0], [-1.0, -3.0, -4.0, -7.0, -1.0]], [[-6.0, 4.0, 1.0, 4.0, -6.0], [0.0, 2.0, 1.0, -4.0, 7.0], [-3.0, -3.0, 1.0, -7.0, -1.0], [-4.0, 6.0, -3.0, -3.0, 6.0]], [[0.0, -4.0, 1.0, 4.0, -1.0], [7.0, 3.0, -1.0, -3.0, 1.0], [0.0, 7.0, -3.0, 6.0, 7.0], [-2.0, 2.0, -4.0, -1.0, 2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, -4.0, 6.0, -7.0, -5.0], [-2.0, 0.0, -4.0, 2.0, 7.0], [5.0, 3.0, -6.0, -6.0, -1.0], [-4.0, 4.0, 4.0, 0.0, -7.0]], [[1.0, 6.0, 1.0, 4.0, 3.0], [2.0, 2.0, 6.0, 0.0, -3.0], [5.0, 0.0, -2.0, 0.0, -5.0], [-3.0, -7.0, 3.0, -3.0, 0.0]], [[3.0, -5.0, 2.0, 2.0, -7.0], [5.0, -7.0, -1.0, -1.0, 7.0], [-6.0, 4.0, 0.0, 2.0, 5.0], [-1.0, 5.0, 0.0, 0.0, -1.0]]], [[[0.0, 4.0, -5.0, -6.0, 6.0], [0.0, 6.0, 1.0, 7.0, -7.0], [-1.0, 4.0, 4.0, 1.0, -3.0], [-7.0, 3.0, 6.0, -4.0, 2.0]], [[6.0, -3.0, 4.0, -2.0, 5.0], [3.0, 3.0, -1.0, -7.0, 2.0], [-6.0, -2.0, -6.0, -6.0, 2.0], [-3.0, -4.0, 5.0, -2.0, 6.0]], [[0.0, -7.0, 3.0, -2.0, -1.0], [-3.0, -6.0, -4.0, -4.0, -6.0], [-4.0, 2.0, 0.0, 4.0, 0.0], [-1.0, 7.0, 0.0, 0.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-1.0, 7.0, 7.0, 3.0, 3.0], [0.0, -4.0, -6.0, -1.0, -5.0], [2.0, 7.0, -2.0, -6.0, 2.0], [7.0, 3.0, -7.0, 5.0, -7.0]], [[-6.0, -2.0, -4.0, 2.0, -7.0], [3.0, -1.0, 6.0, 7.0, 4.0], [1.0, 2.0, 6.0, 5.0, 1.0], [-4.0, -6.0, -3.0, 6.0, -1.0]], [[5.0, -7.0, -1.0, -5.0, -7.0], [0.0, -2.0, -2.0, -2.0, -4.0], [-3.0, 6.0, 5.0, 6.0, 3.0], [-2.0, -3.0, -2.0, 1.0, 0.0]]], [[[0.0, 2.0, -2.0, -3.0, 0.0], [-2.0, -5.0, -6.0, 2.0, -4.0], [3.0, 0.0, -6.0, -1.0, -4.0], [5.0, 2.0, 0.0, -7.0, 6.0]], [[3.0, 0.0, -5.0, -3.0, -7.0], [7.0, -5.0, 7.0, 1.0, -7.0], [1.0, -5.0, -6.0, 5.0, 0.0], [6.0, 6.0, 6.0, -6.0, -3.0]], [[1.0, 0.0, 3.0, 4.0, 5.0], [-1.0, 1.0, -1.0, -3.0, -1.0], [2.0, -7.0, -3.0, -3.0, -5.0], [7.0, 0.0, 0.0, -6.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-43.0, -14.0, -44.0, -25.0, -87.0], [7.0, -1.0, 1.0, 55.0, 46.0], [-17.0, 45.0, 43.0, 25.0, 22.0], [-57.0, 9.0, 53.0, 17.0, -16.0]], [[-31.0, -4.0, 8.0, -17.0, 64.0], [-2.0, -22.0, -27.0, -40.0, -40.0], [-48.0, 15.0, -25.0, -44.0, 18.0], [43.0, 51.0, -54.0, -11.0, 6.0]], [[-39.0, -18.0, -36.0, -19.0, 31.0], [23.0, -19.0, -29.0, 45.0, -52.0], [7.0, 55.0, 23.0, 49.0, 26.0], [21.0, 5.0, 17.0, 33.0, -44.0]], [[-8.0, -1.0, 19.0, 39.0, 62.0], [31.0, 10.0, -5.0, -29.0, -43.0], [9.0, 6.0, -30.0, 33.0, 7.0], [44.0, 26.0, -43.0, 15.0, 13.0]], [[-8.0, -74.0, -15.0, -43.0, 60.0], [35.0, -19.0, -17.0, 9.0, -67.0], [-57.0, 98.0, -45.0, 92.0, 63.0], [49.0, 45.0, -16.0, 52.0, -31.0]]], [[[-41.0, 54.0, 17.0, 54.0, -30.0], [-5.0, 21.0, 14.0, -33.0, 45.0], [5.0, -57.0, 12.0, -58.0, -35.0], [-18.0, 34.0, -24.0, -21.0, 51.0]], [[10.0, -17.0, -3.0, 10.0, 16.0], [12.0, 9.0, 27.0, 30.0, -27.0], [9.0, 7.0, -16.0, 94.0, -15.0], [29.0, 5.0, 26.0, 64.0, -10.0]], [[-24.0, 66.0, 44.0, 98.0, -27.0], [-1.0, 47.0, 52.0, -51.0, 54.0], [15.0, -96.0, -16.0, -21.0, -68.0], [-2.0, 48.0, -31.0, 6.0, 84.0]], [[48.0, 29.0, 47.0, 24.0, 47.0], [-23.0, 5.0, 4.0, -62.0, -6.0], [17.0, -67.0, -30.0, -69.0, -31.0], [25.0, -20.0, -47.0, -45.0, 29.0]], [[59.0, -7.0, 30.0, 20.0, 68.0], [-3.0, 9.0, 30.0, -6.0, -51.0], [27.0, -34.0, -44.0, 68.0, -36.0], [61.0, -20.0, -2.0, 44.0, -2.0]]], [[[10.0, -12.0, 22.0, 3.0, 30.0], [11.0, 0.0, -15.0, -50.0, -7.0], [-23.0, 13.0, -32.0, -31.0, 28.0], [11.0, 18.0, -51.0, -32.0, 18.0]], [[5.0, 43.0, -40.0, -42.0, 26.0], [-49.0, -41.0, -32.0, 78.0, -47.0], [77.0, -42.0, 82.0, -46.0, -50.0], [9.0, -94.0, 68.0, -8.0, -76.0]], [[34.0, 41.0, -7.0, 31.0, 16.0], [-15.0, 7.0, 18.0, 60.0, -30.0], [104.0, -64.0, 42.0, 45.0, -69.0], [16.0, -83.0, 61.0, 32.0, -34.0]], [[18.0, 45.0, 13.0, 38.0, -64.0], [-20.0, 25.0, 29.0, 0.0, 63.0], [43.0, -69.0, 26.0, -36.0, -47.0], [-45.0, -35.0, 24.0, -24.0, 28.0]], [[-33.0, 53.0, -24.0, 7.0, -13.0], [-31.0, -9.0, 6.0, 52.0, -3.0], [44.0, -55.0, 58.0, -18.0, -62.0], [0.0, -23.0, 50.0, 21.0, -19.0]]], [[[-56.0, -26.0, -3.0, 0.0, -18.0], [44.0, 1.0, -42.0, -53.0, 26.0], [-60.0, 77.0, -13.0, -47.0, 85.0], [-32.0, 65.0, -67.0, -52.0, 27.0]], [[-67.0, -4.0, -14.0, -21.0, -32.0], [-2.0, -10.0, -9.0, -16.0, 30.0], [-74.0, 29.0, 1.0, -42.0, 24.0], [-17.0, 75.0, -20.0, -5.0, 22.0]], [[-16.0, -11.0, 13.0, 8.0, -58.0], [0.0, 20.0, 36.0, -27.0, 60.0], [-62.0, 1.0, -30.0, 6.0, 3.0], [-25.0, 62.0, -10.0, 13.0, 49.0]], [[-8.0, 75.0, -10.0, 29.0, 66.0], [-35.0, -14.0, 0.0, 53.0, -66.0], [98.0, -93.0, 55.0, -6.0, -96.0], [54.0, -58.0, 42.0, 32.0, -37.0]], [[90.0, -47.0, 13.0, -74.0, 9.0], [-41.0, -21.0, 12.0, -4.0, -4.0], [-35.0, 5.0, -38.0, -5.0, 7.0], [7.0, -42.0, 17.0, -9.0, -23.0]]]])+(1.-msk_ref)*numpy.array([[[[28.0, -73.0, 13.0, -15.0, -34.0], [21.0, -44.0, -44.0, -32.0, 46.0], [-87.0, 13.0, 0.0, 2.0, 62.0], [7.0, 58.0, -7.0, 12.0, 22.0]], [[-30.0, 3.0, 16.0, -83.0, 20.0], [-53.0, 57.0, -31.0, 35.0, -8.0], [65.0, 1.0, -30.0, -54.0, -38.0], [-29.0, 13.0, 34.0, -2.0, -38.0]], [[-44.0, -61.0, 35.0, -51.0, -80.0], [-51.0, -46.0, -60.0, 24.0, 40.0], [41.0, 25.0, -12.0, -4.0, 4.0], [11.0, 86.0, -21.0, 30.0, -76.0]], [[-34.0, -6.0, 25.0, -7.0, -11.0], [-48.0, -12.0, -11.0, -5.0, -29.0], [60.0, -9.0, -16.0, 3.0, -35.0], [13.0, 4.0, -15.0, 12.0, -32.0]], [[-73.0, -33.0, -16.0, -59.0, -27.0], [-91.0, -16.0, -60.0, 42.0, -51.0], [44.0, 5.0, 38.0, 30.0, -17.0], [32.0, 89.0, -44.0, 35.0, -46.0]]], [[[45.0, -4.0, 38.0, 12.0, 33.0], [30.0, 21.0, 13.0, -59.0, 25.0], [-21.0, -24.0, -56.0, -48.0, 5.0], [-15.0, -62.0, 32.0, -15.0, 36.0]], [[-33.0, 8.0, -21.0, 58.0, -25.0], [-22.0, -39.0, 8.0, -10.0, -20.0], [18.0, -28.0, 36.0, 49.0, 4.0], [70.0, -7.0, -74.0, 33.0, -8.0]], [[48.0, 32.0, 21.0, 86.0, 62.0], [38.0, 17.0, 53.0, -97.0, -12.0], [-20.0, -66.0, -42.0, -20.0, -2.0], [30.0, -129.0, -7.0, -8.0, 72.0]], [[8.0, 82.0, -18.0, 23.0, 60.0], [18.0, 61.0, 61.0, 19.0, -36.0], [46.0, -11.0, -6.0, -14.0, -48.0], [-29.0, -82.0, 34.0, -31.0, 13.0]], [[-45.0, 76.0, -45.0, 83.0, 7.0], [-20.0, -3.0, 55.0, 19.0, -55.0], [69.0, -35.0, 48.0, 56.0, -37.0], [62.0, -59.0, -67.0, 18.0, -13.0]]], [[[-4.0, 8.0, 2.0, -40.0, 35.0], [-20.0, 32.0, -5.0, 13.0, -41.0], [16.0, 8.0, -8.0, -13.0, -29.0], [-34.0, 5.0, 34.0, -17.0, 5.0]], [[-17.0, 18.0, 15.0, -5.0, -89.0], [26.0, -11.0, 11.0, 71.0, 117.0], [67.0, 41.0, -16.0, -28.0, 3.0], [-18.0, 15.0, 9.0, 4.0, -99.0]], [[-13.0, 34.0, 1.0, 102.0, -70.0], [32.0, -59.0, 51.0, 3.0, 51.0], [44.0, -4.0, 12.0, 40.0, 3.0], [48.0, -40.0, -56.0, 18.0, -45.0]], [[59.0, 26.0, -3.0, 76.0, 25.0], [76.0, -5.0, 54.0, -48.0, 20.0], [-58.0, -14.0, -8.0, 5.0, 24.0], [-2.0, -73.0, 10.0, -21.0, 58.0]], [[8.0, 2.0, 24.0, 30.0, -47.0], [28.0, -13.0, 11.0, -7.0, 90.0], [21.0, -8.0, -30.0, -30.0, 20.0], [19.0, -31.0, -13.0, 13.0, -35.0]]], [[[19.0, -89.0, 54.0, -125.0, 20.0], [-35.0, 12.0, -79.0, -19.0, -18.0], [-42.0, 35.0, -50.0, -49.0, 0.0], [-74.0, 86.0, 76.0, -20.0, 13.0]], [[18.0, -43.0, 20.0, -75.0, 30.0], [-15.0, 39.0, -49.0, -19.0, 16.0], [-35.0, -7.0, -34.0, -58.0, 18.0], [-23.0, 21.0, 36.0, -2.0, 22.0]], [[34.0, 2.0, -25.0, 21.0, 70.0], [16.0, 26.0, 6.0, -54.0, -42.0], [-74.0, -41.0, 12.0, 2.0, 24.0], [21.0, -41.0, -7.0, -3.0, 87.0]], [[-32.0, 43.0, 31.0, 57.0, -80.0], [7.0, -19.0, 38.0, 26.0, 97.0], [116.0, -9.0, -34.0, -23.0, -21.0], [34.0, -56.0, -34.0, 22.0, -98.0]], [[-12.0, 90.0, -100.0, 25.0, 58.0], [18.0, 55.0, 51.0, 69.0, -64.0], [-8.0, -1.0, 86.0, 46.0, -12.0], [3.0, -26.0, -10.0, -15.0, 33.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_float_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=4.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[0.0, 6.0], [-6.0, -7.0]], [[4.0, 7.0], [5.0, -5.0]], [[-1.0, -3.0], [7.0, 3.0]], [[3.0, 0.0], [-7.0, -1.0]], [[2.0, 5.0], [4.0, 0.0]], [[6.0, 1.0], [-1.0, 1.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.0, 24.0], [-24.0, -28.0]], [[16.0, 28.0], [20.0, -20.0]], [[-4.0, -12.0], [28.0, 12.0]], [[12.0, 0.0], [-28.0, -4.0]], [[8.0, 20.0], [16.0, 0.0]], [[24.0, 4.0], [-4.0, 4.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[3.0, 1.0], [5.0, 2.0]], [[-6.0, -4.0], [4.0, 2.0]], [[-6.0, 3.0], [-3.0, -4.0]], [[-1.0, -6.0], [-6.0, -7.0]], [[5.0, 4.0], [6.0, -4.0]], [[-3.0, 0.0], [5.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[6.0, 2.0], [10.0, 4.0]], [[-12.0, -8.0], [8.0, 4.0]], [[-12.0, 6.0], [-6.0, -8.0]], [[-2.0, -12.0], [-12.0, -14.0]], [[10.0, 8.0], [12.0, -8.0]], [[-6.0, 0.0], [10.0, -2.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank1_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=numpy.array([-5.0, 1.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[7.0, -1.0], [-7.0, 6.0]], [[5.0, -3.0], [2.0, 7.0]], [[-7.0, 0.0], [4.0, 5.0]], [[-6.0, -2.0], [4.0, 6.0]], [[-1.0, 0.0], [-6.0, 6.0]], [[2.0, 0.0], [-4.0, -1.0]]], [[[-7.0, 4.0], [-6.0, -7.0]], [[-1.0, -3.0], [-4.0, -6.0]], [[7.0, 6.0], [-3.0, 0.0]], [[-7.0, -3.0], [-5.0, -7.0]], [[0.0, -7.0], [5.0, 2.0]], [[-6.0, 2.0], [-2.0, 5.0]]], [[[-2.0, -3.0], [6.0, 5.0]], [[0.0, 3.0], [7.0, 2.0]], [[0.0, 5.0], [-4.0, 3.0]], [[-1.0, -5.0], [6.0, 7.0]], [[-5.0, 0.0], [6.0, -4.0]], [[6.0, -1.0], [-3.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-32.0, 24.0], [-1.0, -62.0]], [[-26.0, -3.0], [-49.0, -51.0]], [[42.0, -19.0], [-3.0, -40.0]], [[28.0, 32.0], [-55.0, -72.0]], [[30.0, -7.0], [5.0, -8.0]], [[-46.0, 7.0], [33.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_array_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([2.0, 4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-2.0, -3.0], [-1.0, 5.0]], [[0.0, 0.0], [-6.0, 3.0]], [[3.0, 2.0], [5.0, 4.0]], [[0.0, 7.0], [6.0, -4.0]], [[-2.0, 6.0], [5.0, 0.0]], [[-3.0, 3.0], [-6.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-4.0, -8.0], [-6.0, -12.0]], [[-2.0, -4.0], [10.0, 20.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[-12.0, -24.0], [6.0, 12.0]]], [[[6.0, 12.0], [4.0, 8.0]], [[10.0, 20.0], [8.0, 16.0]]], [[[0.0, 0.0], [14.0, 28.0]], [[12.0, 24.0], [-8.0, -16.0]]], [[[-4.0, -8.0], [12.0, 24.0]], [[10.0, 20.0], [0.0, 0.0]]], [[[-6.0, -12.0], [6.0, 12.0]], [[-12.0, -24.0], [-4.0, -8.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank2_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=numpy.array([[1.0, 3.0], [-1.0, -1.0], [-7.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[-6.0, -2.0], [-7.0, -7.0]], [[7.0, 1.0], [7.0, -6.0]], [[-1.0, 1.0], [-2.0, 4.0]], [[5.0, 1.0], [-2.0, -2.0]], [[-3.0, 5.0], [5.0, 4.0]], [[-5.0, 6.0], [-2.0, 6.0]]], [[[4.0, 3.0], [1.0, -6.0]], [[7.0, -6.0], [7.0, 7.0]], [[4.0, 7.0], [-7.0, 0.0]], [[-3.0, 0.0], [7.0, -4.0]], [[-1.0, -2.0], [-2.0, -3.0]], [[-7.0, 4.0], [7.0, -1.0]]], [[[-4.0, -3.0], [2.0, -3.0]], [[4.0, -1.0], [-1.0, -4.0]], [[6.0, -6.0], [1.0, 4.0]], [[-7.0, -2.0], [1.0, 7.0]], [[-7.0, 5.0], [-3.0, -1.0]], [[-4.0, -6.0], [1.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[18.0, 6.0], [16.0, 12.0]], [[-22.0, -36.0], [20.0, 6.0]]], [[[-28.0, -14.0], [14.0, 16.0]], [[7.0, 21.0], [15.0, 3.0]]], [[[-47.0, -49.0], [36.0, 38.0]], [[-2.0, -6.0], [-24.0, -16.0]]], [[[57.0, 67.0], [15.0, 17.0]], [[-16.0, -20.0], [-47.0, -51.0]]], [[[47.0, 41.0], [-28.0, -18.0]], [[28.0, 38.0], [14.0, 22.0]]], [[[30.0, 20.0], [44.0, 56.0]], [[-16.0, -20.0], [0.0, 12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[6.0, 5.0], [-6.0, -6.0]], [[3.0, 3.0], [0.0, -4.0]], [[0.0, 6.0], [-3.0, -3.0]], [[-7.0, -6.0], [0.0, 6.0]], [[-1.0, 0.0], [1.0, -7.0]], [[1.0, 2.0], [5.0, 1.0]]])
      s1=numpy.array(5.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[30.0, 25.0], [-30.0, -30.0]], [[15.0, 15.0], [0.0, -20.0]], [[0.0, 30.0], [-15.0, -15.0]], [[-35.0, -30.0], [0.0, 30.0]], [[-5.0, 0.0], [5.0, -35.0]], [[5.0, 10.0], [25.0, 5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[1.0, 3.0], [-3.0, -3.0]], [[-1.0, 2.0], [-1.0, 0.0]], [[-2.0, -3.0], [0.0, 2.0]], [[0.0, 4.0], [5.0, 5.0]], [[2.0, -3.0], [0.0, -6.0]], [[0.0, -5.0], [2.0, 3.0]]], [[[1.0, 6.0], [4.0, 3.0]], [[-6.0, 4.0], [-6.0, -3.0]], [[6.0, -7.0], [0.0, 5.0]], [[0.0, -6.0], [1.0, 6.0]], [[0.0, 7.0], [4.0, 5.0]], [[4.0, 7.0], [1.0, 5.0]]], [[[6.0, 2.0], [-3.0, -4.0]], [[4.0, -5.0], [-3.0, -3.0]], [[0.0, 5.0], [2.0, -1.0]], [[4.0, 2.0], [-1.0, -2.0]], [[0.0, -6.0], [3.0, -5.0]], [[6.0, 2.0], [-2.0, 7.0]]]])
      s1=numpy.array([1.0, 3.0, -1.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-2.0, 19.0], [12.0, 10.0]], [[-23.0, 19.0], [-16.0, -6.0]], [[16.0, -29.0], [-2.0, 18.0]], [[-4.0, -16.0], [9.0, 25.0]], [[2.0, 24.0], [9.0, 14.0]], [[6.0, 14.0], [7.0, 11.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-1.0, 2.0], [-1.0, -1.0]], [[7.0, 2.0], [7.0, 5.0]], [[0.0, 6.0], [4.0, 2.0]], [[7.0, 1.0], [0.0, 2.0]], [[-6.0, 2.0], [-7.0, 2.0]], [[-1.0, 3.0], [-5.0, 0.0]]])
      s1=numpy.array([-1.0, 3.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[1.0, -3.0], [-2.0, 6.0]], [[1.0, -3.0], [1.0, -3.0]]], [[[-7.0, 21.0], [-2.0, 6.0]], [[-7.0, 21.0], [-5.0, 15.0]]], [[[0.0, 0.0], [-6.0, 18.0]], [[-4.0, 12.0], [-2.0, 6.0]]], [[[-7.0, 21.0], [-1.0, 3.0]], [[0.0, 0.0], [-2.0, 6.0]]], [[[6.0, -18.0], [-2.0, 6.0]], [[7.0, -21.0], [-2.0, 6.0]]], [[[1.0, -3.0], [-3.0, 9.0]], [[5.0, -15.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[6.0, 1.0], [4.0, -6.0]], [[0.0, 2.0], [0.0, 5.0]], [[3.0, 3.0], [5.0, -4.0]], [[-5.0, -6.0], [1.0, 4.0]], [[-5.0, -3.0], [6.0, -1.0]], [[-1.0, 3.0], [2.0, -6.0]]], [[[1.0, 7.0], [-7.0, -3.0]], [[-6.0, -7.0], [-5.0, -4.0]], [[-4.0, -5.0], [4.0, 6.0]], [[-7.0, 0.0], [-4.0, -2.0]], [[3.0, 0.0], [3.0, -3.0]], [[7.0, -2.0], [-2.0, 4.0]]], [[[0.0, 5.0], [6.0, -4.0]], [[1.0, 0.0], [3.0, 4.0]], [[6.0, -7.0], [-2.0, 4.0]], [[7.0, 6.0], [5.0, 5.0]], [[7.0, -1.0], [0.0, -1.0]], [[-7.0, 1.0], [-6.0, -4.0]]]])
      s1=numpy.array([[0.0, -2.0], [6.0, 7.0], [-3.0, -1.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[6.0, -5.0], [27.0, 42.0]], [[-60.0, -63.0], [-6.0, -5.0]]], [[[-39.0, -43.0], [-42.0, -53.0]], [[-39.0, -38.0], [-36.0, -42.0]]], [[[-42.0, -40.0], [-9.0, -34.0]], [[30.0, 20.0], [24.0, 46.0]]], [[[-63.0, -46.0], [-18.0, 6.0]], [[-39.0, -35.0], [-27.0, -27.0]]], [[[-3.0, 24.0], [3.0, 7.0]], [[18.0, 9.0], [-15.0, -18.0]]], [[[63.0, 58.0], [-15.0, -21.0]], [[6.0, -12.0], [36.0, 44.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(-2.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-7.0, 0.0], [-6.0, 0.0]], [[7.0, 3.0], [-6.0, -2.0]], [[2.0, -7.0], [-5.0, 7.0]], [[0.0, 3.0], [3.0, -2.0]], [[1.0, 5.0], [6.0, 5.0]], [[-5.0, -5.0], [-5.0, -7.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[14.0, -0.0], [12.0, -0.0]], [[-14.0, -6.0], [12.0, 4.0]], [[-4.0, 14.0], [10.0, -14.0]], [[-0.0, -6.0], [-6.0, 4.0]], [[-2.0, -10.0], [-12.0, -10.0]], [[10.0, 10.0], [10.0, 14.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=Data(numpy.array([0.0, -6.0, -2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[-7.0, -7.0], [2.0, 0.0]], [[1.0, -6.0], [-4.0, -7.0]], [[1.0, 0.0], [3.0, 2.0]], [[-2.0, 5.0], [0.0, 1.0]], [[-7.0, -1.0], [-4.0, 2.0]], [[-6.0, -4.0], [0.0, 3.0]]], [[[2.0, -6.0], [-7.0, 3.0]], [[7.0, -7.0], [0.0, 2.0]], [[-2.0, 5.0], [2.0, 3.0]], [[-2.0, 0.0], [1.0, -6.0]], [[-4.0, 5.0], [6.0, -3.0]], [[-1.0, -3.0], [5.0, -5.0]]], [[[5.0, -1.0], [5.0, 5.0]], [[0.0, -1.0], [1.0, -4.0]], [[-3.0, 5.0], [6.0, -6.0]], [[4.0, -4.0], [-6.0, -6.0]], [[4.0, -5.0], [6.0, -4.0]], [[0.0, -5.0], [5.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-22.0, 38.0], [32.0, -28.0]], [[-42.0, 44.0], [-2.0, -4.0]], [[18.0, -40.0], [-24.0, -6.0]], [[4.0, 8.0], [6.0, 48.0]], [[16.0, -20.0], [-48.0, 26.0]], [[6.0, 28.0], [-40.0, 34.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_constData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([-1.0, -6.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-5.0, -1.0], [-1.0, 6.0]], [[0.0, 7.0], [1.0, 6.0]], [[-2.0, 3.0], [1.0, -6.0]], [[7.0, 2.0], [-6.0, -5.0]], [[-7.0, -6.0], [6.0, -5.0]], [[6.0, 1.0], [-6.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[5.0, 30.0], [1.0, 6.0]], [[1.0, 6.0], [-6.0, -36.0]]], [[[0.0, 0.0], [-7.0, -42.0]], [[-1.0, -6.0], [-6.0, -36.0]]], [[[2.0, 12.0], [-3.0, -18.0]], [[-1.0, -6.0], [6.0, 36.0]]], [[[-7.0, -42.0], [-2.0, -12.0]], [[6.0, 36.0], [5.0, 30.0]]], [[[7.0, 42.0], [6.0, 36.0]], [[-6.0, -36.0], [5.0, 30.0]]], [[[-6.0, -36.0], [-1.0, -6.0]], [[6.0, 36.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      arg1=Data(numpy.array([[6.0, 0.0], [-6.0, 0.0], [1.0, -3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[2.0, -1.0], [-5.0, -2.0]], [[-7.0, -4.0], [0.0, -6.0]], [[5.0, -1.0], [-5.0, -1.0]], [[-3.0, 7.0], [-2.0, 4.0]], [[3.0, 6.0], [-1.0, 1.0]], [[-4.0, 2.0], [0.0, -2.0]]], [[[1.0, 2.0], [5.0, -5.0]], [[-3.0, 4.0], [6.0, 7.0]], [[7.0, 0.0], [1.0, 2.0]], [[-5.0, -7.0], [3.0, -4.0]], [[5.0, 1.0], [0.0, 0.0]], [[6.0, -6.0], [6.0, -3.0]]], [[[-6.0, 0.0], [-4.0, 4.0]], [[-5.0, 5.0], [4.0, 2.0]], [[2.0, -1.0], [-5.0, 3.0]], [[-2.0, -2.0], [-1.0, 0.0]], [[-6.0, -3.0], [-2.0, 6.0]], [[4.0, 5.0], [-5.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.0, 18.0], [-18.0, 0.0]], [[-64.0, 12.0], [22.0, -12.0]]], [[[-29.0, 15.0], [-43.0, -15.0]], [[-32.0, -12.0], [-76.0, -6.0]]], [[[-10.0, -6.0], [-7.0, 3.0]], [[-41.0, 15.0], [-15.0, -9.0]]], [[[10.0, 6.0], [82.0, 6.0]], [[-31.0, 3.0], [48.0, 0.0]]], [[[-18.0, 18.0], [27.0, 9.0]], [[-8.0, 6.0], [12.0, -18.0]]], [[[-56.0, -12.0], [53.0, -15.0]], [[-41.0, 15.0], [11.0, -15.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-7.0)+(1-msk_arg1)*(0.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[6.0, -7.0], [4.0, 7.0]], [[3.0, 6.0], [-2.0, -4.0]], [[-4.0, -2.0], [-6.0, -4.0]], [[4.0, 1.0], [-2.0, 6.0]], [[0.0, 7.0], [7.0, 3.0]], [[0.0, 6.0], [-3.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-42.0, 49.0], [-28.0, -49.0]], [[-21.0, -42.0], [14.0, 28.0]], [[28.0, 14.0], [42.0, 28.0]], [[-28.0, -7.0], [14.0, -42.0]], [[-0.0, -49.0], [-49.0, -21.0]], [[-0.0, -42.0], [21.0, -42.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, -0.0]], [[-0.0, -0.0], [-0.0, -0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-2.0, -7.0, -2.0])+(1.-msk_arg1)*numpy.array([-4.0, -4.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[0.0, -4.0], [-1.0, 7.0]], [[4.0, 2.0], [1.0, 1.0]], [[-5.0, 4.0], [5.0, 1.0]], [[-7.0, 0.0], [0.0, -1.0]], [[7.0, 2.0], [-3.0, 4.0]], [[3.0, -5.0], [-4.0, 0.0]]], [[[-7.0, 0.0], [-2.0, -1.0]], [[-2.0, 1.0], [5.0, -3.0]], [[0.0, 1.0], [-1.0, 0.0]], [[5.0, -4.0], [-5.0, -2.0]], [[-6.0, 0.0], [-1.0, -4.0]], [[-4.0, 3.0], [-7.0, 3.0]]], [[[-1.0, 6.0], [-7.0, 0.0]], [[-1.0, 6.0], [3.0, 2.0]], [[3.0, -7.0], [1.0, 2.0]], [[5.0, -3.0], [7.0, 1.0]], [[-5.0, 0.0], [6.0, -2.0]], [[-2.0, 0.0], [-6.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[51.0, -4.0], [30.0, -7.0]], [[8.0, -23.0], [-43.0, 15.0]], [[4.0, -1.0], [-5.0, -6.0]], [[-31.0, 34.0], [21.0, 14.0]], [[38.0, -4.0], [1.0, 24.0]], [[26.0, -11.0], [69.0, -7.0]]])+(1.-msk_ref)*numpy.array([[[27.0, 22.0], [5.0, -24.0]], [[-9.0, -6.0], [-21.0, 10.0]], [[23.0, -27.0], [-15.0, -2.0]], [[13.0, 13.0], [27.0, 13.0]], [[-9.0, -8.0], [22.0, -2.0]], [[2.0, 8.0], [38.0, -19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank3_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, -3.0])+(1.-msk_arg1)*numpy.array([1.0, -3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-2.0, -6.0], [5.0, 5.0]], [[-5.0, -3.0], [3.0, -7.0]], [[7.0, 0.0], [-1.0, 0.0]], [[-1.0, -5.0], [1.0, -3.0]], [[7.0, -2.0], [5.0, -1.0]], [[0.0, -2.0], [-3.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 6.0], [24.0, 18.0]], [[-20.0, -15.0], [-20.0, -15.0]]], [[[20.0, 15.0], [12.0, 9.0]], [[-12.0, -9.0], [28.0, 21.0]]], [[[-28.0, -21.0], [0.0, 0.0]], [[4.0, 3.0], [0.0, 0.0]]], [[[4.0, 3.0], [20.0, 15.0]], [[-4.0, -3.0], [12.0, 9.0]]], [[[-28.0, -21.0], [8.0, 6.0]], [[-20.0, -15.0], [4.0, 3.0]]], [[[0.0, 0.0], [8.0, 6.0]], [[12.0, 9.0], [0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-2.0, 6.0], [-6.0, 18.0]], [[5.0, -15.0], [5.0, -15.0]]], [[[-5.0, 15.0], [-3.0, 9.0]], [[3.0, -9.0], [-7.0, 21.0]]], [[[7.0, -21.0], [0.0, 0.0]], [[-1.0, 3.0], [0.0, 0.0]]], [[[-1.0, 3.0], [-5.0, 15.0]], [[1.0, -3.0], [-3.0, 9.0]]], [[[7.0, -21.0], [-2.0, 6.0]], [[5.0, -15.0], [-1.0, 3.0]]], [[[0.0, 0.0], [-2.0, 6.0]], [[-3.0, 9.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3, 6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-1.0, -5.0], [0.0, -2.0], [4.0, -7.0]])+(1.-msk_arg1)*numpy.array([[0.0, -5.0], [3.0, -3.0], [0.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[7.0, -4.0], [-3.0, 4.0]], [[1.0, -6.0], [3.0, -1.0]], [[-4.0, 0.0], [-7.0, 4.0]], [[2.0, 2.0], [3.0, 0.0]], [[2.0, -1.0], [3.0, 0.0]], [[-2.0, -3.0], [-7.0, 0.0]]], [[[-4.0, 0.0], [-6.0, 4.0]], [[4.0, -7.0], [0.0, 1.0]], [[-7.0, 0.0], [6.0, 0.0]], [[-7.0, 1.0], [3.0, 4.0]], [[-6.0, 7.0], [-7.0, -7.0]], [[2.0, 0.0], [2.0, 7.0]]], [[[-1.0, -1.0], [-3.0, 4.0]], [[-1.0, 1.0], [7.0, 4.0]], [[4.0, 4.0], [7.0, 4.0]], [[-2.0, 4.0], [3.0, -5.0]], [[-7.0, -2.0], [7.0, 6.0]], [[-2.0, -1.0], [7.0, 3.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-11.0, -20.0], [0.0, 27.0]], [[-9.0, 48.0], [12.0, -56.0]]], [[[-5.0, -6.0], [10.0, 37.0]], [[25.0, -64.0], [17.0, -25.0]]], [[[20.0, 6.0], [16.0, -28.0]], [[35.0, -26.0], [12.0, -48.0]]], [[[-10.0, 18.0], [14.0, -40.0]], [[9.0, -42.0], [-20.0, 27.0]]], [[[-30.0, 51.0], [-7.0, 5.0]], [[25.0, -50.0], [24.0, -28.0]]], [[[-6.0, 20.0], [-1.0, 22.0]], [[35.0, -18.0], [12.0, -35.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, -16.0], [0.0, 27.0]], [[-18.0, 54.0], [12.0, -60.0]]], [[[12.0, -10.0], [-21.0, 44.0]], [[0.0, -64.0], [3.0, -26.0]]], [[[-21.0, 13.0], [0.0, -28.0]], [[18.0, -32.0], [0.0, -48.0]]], [[[-21.0, 25.0], [3.0, -41.0]], [[9.0, -45.0], [12.0, 23.0]]], [[[-18.0, 57.0], [21.0, -2.0]], [[-21.0, -43.0], [-21.0, -21.0]]], [[[6.0, 18.0], [0.0, 22.0]], [[6.0, -20.0], [21.0, -42.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_float_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=6.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[6.0, 5.0, 5.0, 2.0], [-3.0, -1.0, 1.0, 3.0], [0.0, 7.0, 1.0, -5.0]], [[5.0, 4.0, -2.0, 4.0], [0.0, 5.0, 7.0, 6.0], [-5.0, 2.0, -6.0, 4.0]]], [[[-2.0, 7.0, 4.0, 4.0], [2.0, -4.0, -7.0, -6.0], [-1.0, 3.0, -6.0, 0.0]], [[7.0, -6.0, 6.0, -4.0], [-6.0, -4.0, 4.0, -7.0], [0.0, -1.0, 7.0, 1.0]]], [[[-1.0, 7.0, 0.0, 0.0], [-7.0, -6.0, 2.0, 7.0], [-2.0, -1.0, 1.0, 0.0]], [[-5.0, -2.0, -7.0, -6.0], [6.0, -2.0, 2.0, -7.0], [0.0, -6.0, 5.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[36.0, 30.0, 30.0, 12.0], [-18.0, -6.0, 6.0, 18.0], [0.0, 42.0, 6.0, -30.0]], [[30.0, 24.0, -12.0, 24.0], [0.0, 30.0, 42.0, 36.0], [-30.0, 12.0, -36.0, 24.0]]], [[[-12.0, 42.0, 24.0, 24.0], [12.0, -24.0, -42.0, -36.0], [-6.0, 18.0, -36.0, 0.0]], [[42.0, -36.0, 36.0, -24.0], [-36.0, -24.0, 24.0, -42.0], [0.0, -6.0, 42.0, 6.0]]], [[[-6.0, 42.0, 0.0, 0.0], [-42.0, -36.0, 12.0, 42.0], [-12.0, -6.0, 6.0, 0.0]], [[-30.0, -12.0, -42.0, -36.0], [36.0, -12.0, 12.0, -42.0], [0.0, -36.0, 30.0, 36.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_array_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(0.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-7.0, -3.0, 3.0, 0.0], [-5.0, 1.0, -1.0, 5.0], [-6.0, 6.0, -2.0, -6.0]], [[-3.0, 6.0, 1.0, -2.0], [2.0, -7.0, 6.0, -4.0], [7.0, 3.0, -1.0, -3.0]]], [[[1.0, 7.0, -7.0, -1.0], [7.0, 0.0, 3.0, 5.0], [3.0, -4.0, -4.0, 2.0]], [[4.0, -5.0, -6.0, 2.0], [-2.0, 0.0, 6.0, 6.0], [6.0, 7.0, -5.0, -6.0]]], [[[3.0, 2.0, -5.0, -3.0], [-7.0, -6.0, -2.0, 1.0], [-4.0, 4.0, 3.0, 0.0]], [[-2.0, 6.0, 7.0, -4.0], [3.0, -4.0, 0.0, 1.0], [-3.0, 0.0, 6.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]], [[-0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [0.0, 0.0, -0.0, -0.0]]], [[[0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0]]], [[[0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[3.0, 1.0, 3.0, 6.0], [7.0, 2.0, -1.0, -3.0], [-4.0, 3.0, -1.0, -5.0]], [[3.0, 2.0, -3.0, 6.0], [-4.0, -7.0, 5.0, -3.0], [-6.0, 0.0, 4.0, -7.0]]], [[[2.0, 4.0, -2.0, -6.0], [-3.0, -5.0, 0.0, 5.0], [-6.0, 5.0, 2.0, -3.0]], [[0.0, 1.0, 7.0, 7.0], [-7.0, 0.0, 7.0, -3.0], [-1.0, 2.0, 3.0, -4.0]]], [[[5.0, -6.0, 3.0, -7.0], [1.0, 5.0, 1.0, -7.0], [-5.0, 0.0, 7.0, -4.0]], [[3.0, 4.0, 4.0, -7.0], [0.0, 2.0, 0.0, 7.0], [-2.0, 4.0, 0.0, -7.0]]]])
      s1=numpy.array(-3.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-9.0, -3.0, -9.0, -18.0], [-21.0, -6.0, 3.0, 9.0], [12.0, -9.0, 3.0, 15.0]], [[-9.0, -6.0, 9.0, -18.0], [12.0, 21.0, -15.0, 9.0], [18.0, -0.0, -12.0, 21.0]]], [[[-6.0, -12.0, 6.0, 18.0], [9.0, 15.0, -0.0, -15.0], [18.0, -15.0, -6.0, 9.0]], [[-0.0, -3.0, -21.0, -21.0], [21.0, -0.0, -21.0, 9.0], [3.0, -6.0, -9.0, 12.0]]], [[[-15.0, 18.0, -9.0, 21.0], [-3.0, -15.0, -3.0, 21.0], [15.0, -0.0, -21.0, 12.0]], [[-9.0, -12.0, -12.0, 21.0], [-0.0, -6.0, -0.0, -21.0], [6.0, -12.0, -0.0, 21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_constData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(0.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-2.0, 0.0, -4.0, -4.0], [-5.0, -7.0, 7.0, -6.0], [-1.0, -6.0, -5.0, 7.0]], [[0.0, -1.0, 4.0, 7.0], [-1.0, -2.0, -1.0, 0.0], [4.0, -4.0, -3.0, 7.0]]], [[[4.0, 4.0, 6.0, -1.0], [3.0, -3.0, 7.0, -3.0], [5.0, 2.0, -3.0, 5.0]], [[-3.0, -3.0, -3.0, 7.0], [-4.0, 7.0, -4.0, 1.0], [5.0, 6.0, -7.0, 6.0]]], [[[-7.0, 0.0, 4.0, 4.0], [5.0, -1.0, 6.0, -1.0], [0.0, -5.0, 5.0, 3.0]], [[-3.0, 0.0, 0.0, -5.0], [-1.0, 3.0, 6.0, -6.0], [-5.0, 4.0, 4.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0]], [[0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0]]], [[[0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [0.0, 0.0, -0.0, 0.0]], [[-0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, 0.0]]], [[[-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_Symbol_rank4_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-2.0, -4.0, 5.0, -7.0], [5.0, 2.0, -1.0, 2.0], [4.0, 5.0, -1.0, 7.0]], [[5.0, -1.0, 4.0, -6.0], [0.0, -1.0, 4.0, -6.0], [-1.0, 2.0, 5.0, -6.0]]], [[[-4.0, -4.0, -5.0, -5.0], [7.0, -6.0, 1.0, 7.0], [-1.0, 5.0, 6.0, 2.0]], [[-6.0, 4.0, 1.0, 5.0], [3.0, 0.0, -4.0, 7.0], [3.0, 4.0, -1.0, -2.0]]], [[[-2.0, -6.0, 1.0, -5.0], [0.0, 0.0, 2.0, 7.0], [-7.0, 0.0, 3.0, 4.0]], [[4.0, -5.0, 7.0, -2.0], [-2.0, 0.0, 6.0, 0.0], [1.0, 2.0, 0.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[8.0, 16.0, -20.0, 28.0], [-20.0, -8.0, 4.0, -8.0], [-16.0, -20.0, 4.0, -28.0]], [[-20.0, 4.0, -16.0, 24.0], [-0.0, 4.0, -16.0, 24.0], [4.0, -8.0, -20.0, 24.0]]], [[[16.0, 16.0, 20.0, 20.0], [-28.0, 24.0, -4.0, -28.0], [4.0, -20.0, -24.0, -8.0]], [[24.0, -16.0, -4.0, -20.0], [-12.0, -0.0, 16.0, -28.0], [-12.0, -16.0, 4.0, 8.0]]], [[[8.0, 24.0, -4.0, 20.0], [-0.0, -0.0, -8.0, -28.0], [28.0, -0.0, -12.0, -16.0]], [[-16.0, 20.0, -28.0, 8.0], [8.0, -0.0, -24.0, -0.0], [-4.0, -8.0, -0.0, 24.0]]]])+(1.-msk_ref)*numpy.array([[[[-10.0, -20.0, 25.0, -35.0], [25.0, 10.0, -5.0, 10.0], [20.0, 25.0, -5.0, 35.0]], [[25.0, -5.0, 20.0, -30.0], [0.0, -5.0, 20.0, -30.0], [-5.0, 10.0, 25.0, -30.0]]], [[[-20.0, -20.0, -25.0, -25.0], [35.0, -30.0, 5.0, 35.0], [-5.0, 25.0, 30.0, 10.0]], [[-30.0, 20.0, 5.0, 25.0], [15.0, 0.0, -20.0, 35.0], [15.0, 20.0, -5.0, -10.0]]], [[[-10.0, -30.0, 5.0, -25.0], [0.0, 0.0, 10.0, 35.0], [-35.0, 0.0, 15.0, 20.0]], [[20.0, -25.0, 35.0, -10.0], [-10.0, 0.0, 30.0, 0.0], [5.0, 10.0, 0.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_float_rank0_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      arg1=5.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-5.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_array_rank0_offset0(self):
      arg0=Data(2.0,self.functionspace)
      arg1=numpy.array(-4.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(-8.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank1_offset1(self):
      arg0=Data(numpy.array([3.0, -2.0, 6.0]),self.functionspace)
      arg1=numpy.array([-3.0, -7.0, -2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(-7.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank2_offset2(self):
      arg0=Data(numpy.array([[-7.0, -2.0, 0.0], [-5.0, 0.0, -3.0]]),self.functionspace)
      arg1=numpy.array([[5.0, -5.0, 0.0], [-1.0, 0.0, -1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(-17.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[-3.0, 4.0, -1.0], [-3.0, 7.0, 1.0], [-6.0, 6.0, -7.0], [6.0, 7.0, -5.0]], [[-6.0, 7.0, 1.0], [3.0, -7.0, -1.0], [6.0, 4.0, -1.0], [-4.0, 7.0, 6.0]]]),self.functionspace)
      arg1=numpy.array([[[-5.0, -4.0, -5.0], [7.0, 0.0, -5.0], [-6.0, -3.0, -5.0], [-7.0, -3.0, 4.0]], [[-4.0, 1.0, 4.0], [3.0, 3.0, -5.0], [2.0, 0.0, 6.0], [-4.0, 5.0, -6.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(-3.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-7.0, -5.0], [-3.0, -4.0], [-1.0, 3.0]], [[0.0, -7.0], [-1.0, 0.0], [-5.0, 0.0]]], [[[0.0, 5.0], [5.0, 1.0], [-5.0, 7.0]], [[3.0, -7.0], [-7.0, 5.0], [-6.0, 0.0]]], [[[-6.0, -3.0], [-4.0, 2.0], [5.0, 0.0]], [[-1.0, -5.0], [-2.0, 6.0], [6.0, 6.0]]], [[[-5.0, 5.0], [5.0, -2.0], [3.0, -7.0]], [[-4.0, -7.0], [1.0, 5.0], [0.0, -4.0]]]]),self.functionspace)
      arg1=numpy.array([[[[4.0, 6.0], [-1.0, 0.0], [-6.0, -5.0]], [[5.0, 6.0], [5.0, 6.0], [4.0, -7.0]]], [[[-2.0, -6.0], [-2.0, 6.0], [-1.0, -3.0]], [[-4.0, 4.0], [3.0, 0.0], [1.0, 0.0]]], [[[-5.0, 0.0], [7.0, 1.0], [-7.0, 0.0]], [[-7.0, -3.0], [-2.0, 0.0], [6.0, -4.0]]], [[[2.0, -6.0], [-3.0, -7.0], [7.0, -5.0]], [[4.0, -7.0], [6.0, 0.0], [4.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(-159.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_array_rank1_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      arg1=numpy.array([-7.0, -6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([7.0, 6.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank2_offset1(self):
      arg0=Data(numpy.array([-3.0, 3.0, 7.0]),self.functionspace)
      arg1=numpy.array([[0.0, -1.0], [-7.0, 7.0], [-6.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-63.0, 66.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank3_offset2(self):
      arg0=Data(numpy.array([[-5.0, 6.0, 6.0], [0.0, 0.0, -4.0]]),self.functionspace)
      arg1=numpy.array([[[-2.0, 0.0], [7.0, 6.0], [6.0, -1.0]], [[-2.0, 2.0], [-3.0, -3.0], [-2.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([96.0, 38.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[1.0, -5.0, 6.0], [-3.0, -3.0, 5.0], [-1.0, -1.0, -4.0], [7.0, -4.0, 3.0]], [[-7.0, 0.0, 0.0], [7.0, 5.0, 3.0], [1.0, -6.0, -5.0], [6.0, -3.0, -3.0]]]),self.functionspace)
      arg1=numpy.array([[[[4.0, 0.0], [-3.0, -5.0], [-3.0, -5.0]], [[5.0, 0.0], [3.0, -3.0], [6.0, -7.0]], [[2.0, 0.0], [-5.0, 6.0], [0.0, -5.0]], [[0.0, 2.0], [-5.0, 4.0], [1.0, 7.0]]], [[[1.0, 0.0], [0.0, 0.0], [-6.0, 6.0]], [[6.0, 0.0], [6.0, 6.0], [-2.0, 7.0]], [[6.0, -4.0], [0.0, 3.0], [-2.0, 1.0]], [[-2.0, -3.0], [1.0, -6.0], [6.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([75.0, 20.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_array_rank2_offset0(self):
      arg0=Data(5.0,self.functionspace)
      arg1=numpy.array([[-4.0, -1.0, 2.0, 6.0, 2.0], [-5.0, 4.0, 1.0, -4.0, 6.0], [7.0, 6.0, 7.0, -3.0, -2.0], [-6.0, 0.0, 6.0, 3.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-20.0, -5.0, 10.0, 30.0, 10.0], [-25.0, 20.0, 5.0, -20.0, 30.0], [35.0, 30.0, 35.0, -15.0, -10.0], [-30.0, 0.0, 30.0, 15.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank3_offset1(self):
      arg0=Data(numpy.array([1.0, -4.0, 4.0]),self.functionspace)
      arg1=numpy.array([[[-7.0, -7.0, -3.0, 3.0, 0.0], [4.0, 2.0, -2.0, -3.0, -1.0], [-5.0, 4.0, -2.0, 7.0, 5.0], [0.0, -2.0, -1.0, -1.0, 1.0]], [[-7.0, 3.0, 0.0, -3.0, 7.0], [0.0, 2.0, 0.0, -3.0, -6.0], [4.0, 1.0, 6.0, -2.0, -4.0], [-3.0, -7.0, -4.0, -1.0, 1.0]], [[-2.0, 0.0, -5.0, 4.0, -1.0], [-7.0, 2.0, 0.0, -2.0, 7.0], [-2.0, 4.0, 5.0, 6.0, 5.0], [-7.0, 0.0, 4.0, 0.0, -3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[13.0, -19.0, -23.0, 31.0, -32.0], [-24.0, 2.0, -2.0, 1.0, 51.0], [-29.0, 16.0, -6.0, 39.0, 41.0], [-16.0, 26.0, 31.0, 3.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank4_offset2(self):
      arg0=Data(numpy.array([[-2.0, 5.0, -2.0], [1.0, 6.0, 0.0]]),self.functionspace)
      arg1=numpy.array([[[[2.0, 0.0, 2.0, -5.0, -2.0], [4.0, 0.0, -2.0, 1.0, 6.0], [-5.0, 7.0, -5.0, -3.0, -7.0], [-7.0, 3.0, 6.0, 3.0, -3.0]], [[0.0, -4.0, -3.0, -5.0, -5.0], [6.0, 3.0, 1.0, 6.0, 5.0], [-3.0, -2.0, 1.0, -3.0, 0.0], [3.0, 3.0, 4.0, -4.0, 2.0]], [[-1.0, -1.0, -6.0, 4.0, -5.0], [-1.0, 1.0, 3.0, -4.0, -1.0], [0.0, -3.0, -4.0, 6.0, 7.0], [-5.0, 1.0, -4.0, -3.0, 7.0]]], [[[-5.0, 7.0, -3.0, -2.0, -7.0], [-7.0, 3.0, -7.0, 4.0, -1.0], [4.0, 0.0, 7.0, 4.0, -4.0], [6.0, 4.0, -5.0, 7.0, -6.0]], [[3.0, -2.0, -3.0, 0.0, -5.0], [0.0, 3.0, 7.0, -1.0, -7.0], [0.0, -6.0, -5.0, -5.0, -2.0], [5.0, 3.0, 4.0, 1.0, 0.0]], [[1.0, 1.0, 0.0, 7.0, 7.0], [-1.0, 2.0, 5.0, 2.0, -3.0], [2.0, 3.0, -1.0, -7.0, 2.0], [7.0, -7.0, 7.0, 0.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[11.0, -23.0, -28.0, -25.0, -48.0], [17.0, 34.0, 38.0, 34.0, -28.0], [-1.0, -54.0, 0.0, -47.0, -16.0], [75.0, 29.0, 35.0, -7.0, -4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_array_rank3_offset0(self):
      arg0=Data(6.0,self.functionspace)
      arg1=numpy.array([[[5.0, 0.0], [3.0, 5.0]], [[-5.0, 0.0], [-2.0, 7.0]], [[7.0, -4.0], [0.0, 0.0]], [[-1.0, 0.0], [-1.0, -4.0]], [[1.0, -4.0], [1.0, 1.0]], [[4.0, 5.0], [-1.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[30.0, 0.0], [18.0, 30.0]], [[-30.0, 0.0], [-12.0, 42.0]], [[42.0, -24.0], [0.0, 0.0]], [[-6.0, 0.0], [-6.0, -24.0]], [[6.0, -24.0], [6.0, 6.0]], [[24.0, 30.0], [-6.0, 12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank4_offset1(self):
      arg0=Data(numpy.array([3.0, 1.0, 0.0]),self.functionspace)
      arg1=numpy.array([[[[-4.0, 3.0], [-5.0, -6.0]], [[5.0, 5.0], [0.0, -7.0]], [[-2.0, -4.0], [0.0, 6.0]], [[0.0, 6.0], [1.0, -7.0]], [[-4.0, 0.0], [3.0, 0.0]], [[0.0, -6.0], [6.0, 3.0]]], [[[7.0, 2.0], [2.0, 1.0]], [[-6.0, 2.0], [6.0, 2.0]], [[3.0, 5.0], [0.0, -3.0]], [[4.0, -2.0], [-3.0, -5.0]], [[4.0, 0.0], [1.0, 5.0]], [[0.0, 3.0], [3.0, -5.0]]], [[[-6.0, 4.0], [-6.0, 5.0]], [[-5.0, -6.0], [7.0, 3.0]], [[-4.0, -5.0], [2.0, -2.0]], [[6.0, 6.0], [-5.0, -7.0]], [[-4.0, -4.0], [3.0, 1.0]], [[0.0, -3.0], [3.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-5.0, 11.0], [-13.0, -17.0]], [[9.0, 17.0], [6.0, -19.0]], [[-3.0, -7.0], [0.0, 15.0]], [[4.0, 16.0], [0.0, -26.0]], [[-8.0, 0.0], [10.0, 5.0]], [[0.0, -15.0], [21.0, 4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_array_rank4_offset0(self):
      arg0=Data(0.0,self.functionspace)
      arg1=numpy.array([[[[-7.0, 1.0, 0.0, -6.0], [0.0, 2.0, 4.0, 4.0], [-7.0, -1.0, -1.0, 0.0]], [[1.0, -2.0, 2.0, 0.0], [2.0, 4.0, -1.0, 5.0], [0.0, 2.0, 6.0, -1.0]]], [[[-1.0, 5.0, 1.0, -5.0], [-7.0, 7.0, -1.0, -6.0], [-2.0, -6.0, -6.0, -4.0]], [[5.0, 1.0, 2.0, 4.0], [-2.0, -4.0, -2.0, -3.0], [1.0, 3.0, 4.0, -1.0]]], [[[6.0, -3.0, 1.0, -4.0], [5.0, 6.0, -5.0, -3.0], [4.0, 4.0, 2.0, -3.0]], [[-7.0, 3.0, -5.0, 3.0], [-4.0, 7.0, -3.0, -7.0], [-3.0, -3.0, 4.0, 3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, 0.0]], [[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]]], [[[-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, -0.0]], [[0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, -0.0], [0.0, 0.0, 0.0, -0.0]]], [[[0.0, -0.0, 0.0, -0.0], [0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, -0.0]], [[-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_Symbol_rank0_offset0(self):
      arg0=Data(1.0,self.functionspace)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-7.0)
      sub=res.substitute({arg1:s1})
      ref=Data(-7.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([5.0, 1.0, 6.0]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-4.0, -5.0, 5.0])
      sub=res.substitute({arg1:s1})
      ref=Data(5.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[1.0, -5.0, 5.0], [5.0, -3.0, -2.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[0.0, -1.0, -6.0], [0.0, 5.0, 3.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(-46.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[1.0, 3.0, 5.0], [4.0, 0.0, 0.0], [2.0, 2.0, 0.0], [0.0, -5.0, 1.0]], [[-5.0, -2.0, 2.0], [-3.0, 0.0, -4.0], [3.0, 3.0, -7.0], [3.0, -5.0, -2.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[0.0, 7.0, 1.0], [0.0, -5.0, 5.0], [7.0, 3.0, 6.0], [4.0, 3.0, 7.0]], [[1.0, -2.0, 4.0], [7.0, 3.0, 5.0], [-6.0, 5.0, 2.0], [2.0, -7.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(22.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-6.0, -2.0], [7.0, 1.0], [0.0, -7.0]], [[-4.0, -6.0], [6.0, 7.0], [-4.0, -4.0]]], [[[-2.0, 3.0], [-4.0, 1.0], [-5.0, 1.0]], [[-1.0, 0.0], [-5.0, 2.0], [-4.0, -7.0]]], [[[-1.0, 4.0], [-1.0, -1.0], [-2.0, 3.0]], [[-1.0, 4.0], [7.0, 1.0], [1.0, 3.0]]], [[[-3.0, 0.0], [7.0, 0.0], [1.0, -3.0]], [[0.0, 6.0], [4.0, -7.0], [-3.0, -6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[5.0, -3.0], [4.0, -5.0], [3.0, 6.0]], [[6.0, 2.0], [-4.0, 0.0], [-5.0, 2.0]]], [[[-2.0, 7.0], [3.0, 0.0], [-3.0, 5.0]], [[3.0, 6.0], [0.0, -1.0], [-2.0, 6.0]]], [[[4.0, 0.0], [2.0, 2.0], [1.0, 1.0]], [[-2.0, 1.0], [0.0, 0.0], [7.0, 7.0]]], [[[5.0, 0.0], [-6.0, -2.0], [-6.0, -3.0]], [[-2.0, -3.0], [-6.0, 6.0], [-5.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(-175.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_Symbol_rank1_offset0(self):
      arg0=Data(2.0,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-1.0, 1.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-2.0, 2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([1.0, -2.0, -2.0]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[0.0, 0.0], [-6.0, 3.0], [3.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([6.0, 2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[-5.0, 5.0, -5.0], [0.0, -5.0, 6.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, 0.0], [2.0, 4.0], [-2.0, -6.0]], [[0.0, -5.0], [1.0, 2.0], [-5.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-15.0, 40.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[5.0, -5.0, 5.0], [-2.0, 0.0, 1.0], [-2.0, 1.0, 4.0], [3.0, 5.0, 1.0]], [[-7.0, 1.0, -6.0], [4.0, -6.0, -5.0], [4.0, 0.0, 7.0], [-2.0, 0.0, -7.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-6.0, -5.0], [-5.0, -1.0], [-7.0, 5.0]], [[-5.0, 3.0], [7.0, 6.0], [6.0, -3.0]], [[3.0, 5.0], [-7.0, 5.0], [2.0, 1.0]], [[-4.0, 6.0], [1.0, 7.0], [0.0, -3.0]]], [[[0.0, -6.0], [0.0, 3.0], [-5.0, 2.0]], [[0.0, 2.0], [-7.0, -4.0], [0.0, -5.0]], [[-2.0, 7.0], [-6.0, -4.0], [-4.0, -5.0]], [[-4.0, 0.0], [-1.0, -7.0], [-4.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([36.0, 79.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_Symbol_rank2_offset0(self):
      arg0=Data(-2.0,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-2.0, -5.0, -3.0, -7.0, 0.0], [-3.0, -6.0, -4.0, -5.0, -6.0], [0.0, 7.0, 0.0, -7.0, -6.0], [4.0, -1.0, -7.0, -6.0, 2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[4.0, 10.0, 6.0, 14.0, -0.0], [6.0, 12.0, 8.0, 10.0, 12.0], [-0.0, -14.0, -0.0, 14.0, 12.0], [-8.0, 2.0, 14.0, 12.0, -4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([4.0, 0.0, -6.0]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[0.0, 5.0, 0.0, -3.0, 0.0], [-4.0, -3.0, 3.0, -7.0, 3.0], [0.0, -7.0, -5.0, 7.0, 1.0], [-4.0, -3.0, 0.0, 5.0, 6.0]], [[-4.0, 0.0, -3.0, 2.0, -7.0], [0.0, -3.0, 7.0, 6.0, 6.0], [-3.0, -5.0, -4.0, -3.0, 7.0], [-2.0, 7.0, -2.0, -6.0, 5.0]], [[0.0, -7.0, -4.0, 2.0, 0.0], [7.0, 5.0, 3.0, 6.0, -3.0], [-7.0, -4.0, -3.0, -1.0, -1.0], [-5.0, 6.0, -3.0, 3.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.0, 62.0, 24.0, -24.0, 0.0], [-58.0, -42.0, -6.0, -64.0, 30.0], [42.0, -4.0, -2.0, 34.0, 10.0], [14.0, -48.0, 18.0, 2.0, 60.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[2.0, -1.0, -4.0], [2.0, 0.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[0.0, 7.0, 1.0, 0.0, 3.0], [-4.0, -3.0, 0.0, -4.0, 2.0], [3.0, 6.0, -2.0, -3.0, -5.0], [3.0, 0.0, 2.0, -4.0, -5.0]], [[-3.0, 5.0, 2.0, -7.0, 4.0], [-1.0, -6.0, 1.0, 4.0, 5.0], [-6.0, 4.0, -4.0, 1.0, -5.0], [-5.0, 0.0, 4.0, 7.0, -7.0]], [[0.0, 2.0, 0.0, -3.0, -4.0], [-5.0, -7.0, 0.0, -7.0, 6.0], [-7.0, -2.0, 2.0, -7.0, -7.0], [2.0, 2.0, 0.0, -4.0, -5.0]]], [[[-1.0, 3.0, -7.0, 6.0, 1.0], [7.0, 6.0, -3.0, 0.0, 3.0], [0.0, -6.0, 0.0, -6.0, -6.0], [-1.0, 0.0, 5.0, -7.0, 1.0]], [[-6.0, 5.0, 2.0, 6.0, -7.0], [-4.0, 6.0, 0.0, -6.0, 3.0], [3.0, -4.0, -1.0, 4.0, -4.0], [4.0, -4.0, -6.0, -5.0, -4.0]], [[-3.0, 1.0, 4.0, 3.0, 4.0], [3.0, 2.0, -3.0, 3.0, 7.0], [4.0, -7.0, 6.0, 2.0, 3.0], [-7.0, 6.0, 0.0, 3.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[1.0, 7.0, -14.0, 31.0, 20.0], [27.0, 40.0, -7.0, 16.0, -19.0], [40.0, 4.0, -8.0, 9.0, 11.0], [1.0, -8.0, 10.0, -13.0, 19.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_Symbol_rank3_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[2.0, 3.0], [4.0, 0.0]], [[0.0, 3.0], [1.0, -7.0]], [[4.0, -6.0], [4.0, 0.0]], [[-4.0, -5.0], [-6.0, -4.0]], [[-4.0, -2.0], [1.0, 2.0]], [[0.0, -5.0], [6.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-14.0, -21.0], [-28.0, -0.0]], [[-0.0, -21.0], [-7.0, 49.0]], [[-28.0, 42.0], [-28.0, -0.0]], [[28.0, 35.0], [42.0, 28.0]], [[28.0, 14.0], [-7.0, -14.0]], [[-0.0, 35.0], [-42.0, -35.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([-1.0, 0.0, 0.0]),self.functionspace)
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[7.0, -6.0], [-6.0, 2.0]], [[7.0, 6.0], [-5.0, 4.0]], [[-6.0, -5.0], [3.0, -3.0]], [[6.0, -2.0], [1.0, 3.0]], [[6.0, -5.0], [4.0, -1.0]], [[0.0, -1.0], [-5.0, 0.0]]], [[[7.0, 4.0], [-4.0, 0.0]], [[-2.0, -3.0], [0.0, -5.0]], [[-1.0, -5.0], [-1.0, 4.0]], [[3.0, -3.0], [-6.0, 0.0]], [[-3.0, -5.0], [6.0, -6.0]], [[1.0, -5.0], [1.0, -2.0]]], [[[-7.0, -7.0], [1.0, 6.0]], [[-6.0, -1.0], [0.0, -1.0]], [[0.0, -3.0], [1.0, 7.0]], [[4.0, -5.0], [-3.0, 5.0]], [[7.0, 1.0], [-6.0, -2.0]], [[6.0, -1.0], [-6.0, -6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-7.0, 6.0], [6.0, -2.0]], [[-7.0, -6.0], [5.0, -4.0]], [[6.0, 5.0], [-3.0, 3.0]], [[-6.0, 2.0], [-1.0, -3.0]], [[-6.0, 5.0], [-4.0, 1.0]], [[0.0, 1.0], [5.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_Symbol_rank4_offset0(self):
      arg0=Data(4.0,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-4.0, 0.0, 0.0, 4.0], [-7.0, -7.0, -2.0, 0.0], [-7.0, -5.0, 7.0, 5.0]], [[-2.0, 2.0, 1.0, -4.0], [-1.0, -4.0, -7.0, -6.0], [-6.0, 6.0, -6.0, -3.0]]], [[[-1.0, 4.0, 3.0, -2.0], [2.0, 0.0, -3.0, 6.0], [-3.0, -4.0, 0.0, -5.0]], [[-7.0, 4.0, 1.0, -1.0], [-7.0, -5.0, 7.0, -4.0], [6.0, 0.0, -7.0, -3.0]]], [[[-2.0, -2.0, 6.0, 6.0], [5.0, 6.0, 5.0, 5.0], [-1.0, 0.0, -6.0, 1.0]], [[0.0, -3.0, -2.0, -4.0], [-7.0, 1.0, -2.0, -4.0], [-4.0, 3.0, 3.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-16.0, 0.0, 0.0, 16.0], [-28.0, -28.0, -8.0, 0.0], [-28.0, -20.0, 28.0, 20.0]], [[-8.0, 8.0, 4.0, -16.0], [-4.0, -16.0, -28.0, -24.0], [-24.0, 24.0, -24.0, -12.0]]], [[[-4.0, 16.0, 12.0, -8.0], [8.0, 0.0, -12.0, 24.0], [-12.0, -16.0, 0.0, -20.0]], [[-28.0, 16.0, 4.0, -4.0], [-28.0, -20.0, 28.0, -16.0], [24.0, 0.0, -28.0, -12.0]]], [[[-8.0, -8.0, 24.0, 24.0], [20.0, 24.0, 20.0, 20.0], [-4.0, 0.0, -24.0, 4.0]], [[0.0, -12.0, -8.0, -16.0], [-28.0, 4.0, -8.0, -16.0], [-16.0, 12.0, 12.0, -16.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_constData_rank0_offset0(self):
      arg0=Data(5.0,self.functionspace)
      arg1=Data(1.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(5.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank1_offset1(self):
      arg0=Data(numpy.array([4.0, -2.0, 4.0]),self.functionspace)
      arg1=Data(numpy.array([5.0, 6.0, 6.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(32.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[-1.0, 2.0, -7.0], [0.0, 6.0, -6.0]]),self.functionspace)
      arg1=Data(numpy.array([[4.0, 4.0, 4.0], [-4.0, 7.0, 7.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(-24.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[5.0, 6.0, -4.0], [6.0, 1.0, 0.0], [0.0, -2.0, -6.0], [-5.0, -3.0, -1.0]], [[0.0, -2.0, 0.0], [6.0, 3.0, 0.0], [-3.0, -2.0, 1.0], [0.0, 6.0, -7.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[-4.0, 6.0, 6.0], [-7.0, -4.0, -2.0], [0.0, 6.0, -6.0], [4.0, -3.0, 6.0]], [[3.0, -6.0, 2.0], [1.0, -4.0, 7.0], [-4.0, -6.0, 0.0], [-2.0, 0.0, -1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(-10.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[5.0, -1.0], [7.0, -7.0], [3.0, -2.0]], [[1.0, -3.0], [-1.0, -3.0], [3.0, 2.0]]], [[[0.0, -3.0], [1.0, -7.0], [-5.0, -7.0]], [[7.0, 5.0], [3.0, 6.0], [0.0, 2.0]]], [[[-2.0, 5.0], [-5.0, -5.0], [-6.0, -2.0]], [[6.0, 0.0], [-7.0, 7.0], [7.0, -4.0]]], [[[4.0, -1.0], [2.0, 5.0], [2.0, 5.0]], [[3.0, -5.0], [3.0, -5.0], [-7.0, 7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[6.0, 3.0], [2.0, 0.0], [-5.0, 6.0]], [[-1.0, -3.0], [5.0, -1.0], [5.0, 0.0]]], [[[-5.0, 4.0], [2.0, -7.0], [4.0, 0.0]], [[2.0, -1.0], [-5.0, 0.0], [2.0, -1.0]]], [[[-4.0, 7.0], [-3.0, 7.0], [-6.0, 0.0]], [[2.0, 3.0], [2.0, -6.0], [4.0, 7.0]]], [[[6.0, -7.0], [-5.0, 7.0], [7.0, 0.0]], [[-7.0, 7.0], [2.0, -3.0], [-6.0, -2.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      ref=Data(124.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_constData_rank1_offset0(self):
      arg0=Data(-2.0,self.functionspace)
      arg1=Data(numpy.array([4.0, -3.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-8.0, 6.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank2_offset1(self):
      arg0=Data(numpy.array([0.0, -1.0, -3.0]),self.functionspace)
      arg1=Data(numpy.array([[0.0, 0.0], [4.0, 5.0], [6.0, 1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-22.0, -8.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[0.0, -1.0, -5.0], [-3.0, 4.0, 3.0]]),self.functionspace)
      arg1=Data(numpy.array([[[-1.0, 4.0], [6.0, -5.0], [4.0, -4.0]], [[3.0, -1.0], [6.0, 3.0], [0.0, 1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-11.0, 43.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[-1.0, 5.0, 3.0], [-1.0, 0.0, 6.0], [-5.0, 1.0, 3.0], [7.0, -5.0, 2.0]], [[-6.0, -5.0, -3.0], [4.0, 2.0, -6.0], [0.0, 2.0, 6.0], [-4.0, 2.0, 3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-2.0, 4.0], [-6.0, 3.0], [0.0, 4.0]], [[4.0, 2.0], [1.0, 0.0], [-3.0, -4.0]], [[1.0, 5.0], [4.0, 1.0], [0.0, -6.0]], [[1.0, -3.0], [-6.0, -1.0], [-4.0, 2.0]]], [[[1.0, 1.0], [0.0, -4.0], [-4.0, 0.0]], [[-5.0, -5.0], [2.0, -1.0], [3.0, -5.0]], [[-2.0, 5.0], [-6.0, -7.0], [-6.0, 3.0]], [[4.0, 1.0], [-6.0, -1.0], [6.0, -7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-108.0, -58.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_constData_rank2_offset0(self):
      arg0=Data(-5.0,self.functionspace)
      arg1=Data(numpy.array([[3.0, -5.0, 7.0, 1.0, -2.0], [1.0, -7.0, 4.0, 3.0, -4.0], [-7.0, 7.0, -3.0, -1.0, 0.0], [-1.0, 2.0, 5.0, -3.0, -1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-15.0, 25.0, -35.0, -5.0, 10.0], [-5.0, 35.0, -20.0, -15.0, 20.0], [35.0, -35.0, 15.0, 5.0, -0.0], [5.0, -10.0, -25.0, 15.0, 5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank3_offset1(self):
      arg0=Data(numpy.array([-5.0, -2.0, -1.0]),self.functionspace)
      arg1=Data(numpy.array([[[3.0, 7.0, 0.0, -1.0, -6.0], [5.0, -7.0, -3.0, -1.0, 3.0], [-7.0, 7.0, 0.0, -5.0, 3.0], [4.0, -2.0, 0.0, 2.0, -7.0]], [[-2.0, 1.0, 4.0, 4.0, 4.0], [4.0, 1.0, -7.0, -4.0, -1.0], [-3.0, -4.0, -5.0, 6.0, 4.0], [-1.0, 2.0, 4.0, -7.0, 7.0]], [[4.0, 5.0, -3.0, -6.0, -5.0], [-4.0, 4.0, 1.0, 6.0, -2.0], [-1.0, 3.0, -4.0, 0.0, -6.0], [-2.0, -6.0, -1.0, -4.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-15.0, -42.0, -5.0, 3.0, 27.0], [-29.0, 29.0, 28.0, 7.0, -11.0], [42.0, -30.0, 14.0, 13.0, -17.0], [-16.0, 12.0, -7.0, 8.0, 21.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[0.0, -4.0, -5.0], [-2.0, 4.0, 1.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[-5.0, -7.0, 7.0, 5.0, 7.0], [-1.0, 7.0, 3.0, 6.0, 4.0], [-4.0, -4.0, -3.0, 0.0, -2.0], [4.0, 5.0, 3.0, -4.0, 7.0]], [[6.0, 0.0, -2.0, -7.0, 0.0], [4.0, -4.0, -4.0, -6.0, 1.0], [7.0, 3.0, -3.0, 2.0, 2.0], [-7.0, 3.0, -1.0, 0.0, 2.0]], [[3.0, -2.0, 0.0, -4.0, 4.0], [0.0, 4.0, -1.0, 3.0, 5.0], [0.0, -7.0, 2.0, -7.0, -4.0], [0.0, -1.0, 4.0, -5.0, 0.0]]], [[[-1.0, 2.0, -7.0, 7.0, 5.0], [-6.0, 5.0, 2.0, 0.0, -5.0], [-3.0, 5.0, -1.0, 3.0, 2.0], [-1.0, 0.0, -5.0, 4.0, 0.0]], [[3.0, 4.0, 2.0, -6.0, -2.0], [3.0, -2.0, -3.0, 0.0, 4.0], [2.0, -7.0, 2.0, 7.0, -1.0], [-6.0, 2.0, 3.0, 3.0, -3.0]], [[5.0, -6.0, 0.0, 2.0, 4.0], [-2.0, 4.0, 0.0, 7.0, 2.0], [4.0, 0.0, 4.0, -2.0, 6.0], [-2.0, 0.0, 5.0, -6.0, 5.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-20.0, 16.0, 30.0, 12.0, -34.0], [6.0, -18.0, 5.0, 16.0, -1.0], [-10.0, -15.0, 16.0, 47.0, 10.0], [4.0, 1.0, 11.0, 23.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_constData_rank3_offset0(self):
      arg0=Data(5.0,self.functionspace)
      arg1=Data(numpy.array([[[3.0, -7.0], [-7.0, -2.0]], [[-4.0, 4.0], [6.0, -5.0]], [[6.0, 6.0], [-3.0, -4.0]], [[-3.0, -6.0], [-6.0, -3.0]], [[3.0, 1.0], [5.0, 1.0]], [[1.0, -3.0], [5.0, -6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[15.0, -35.0], [-35.0, -10.0]], [[-20.0, 20.0], [30.0, -25.0]], [[30.0, 30.0], [-15.0, -20.0]], [[-15.0, -30.0], [-30.0, -15.0]], [[15.0, 5.0], [25.0, 5.0]], [[5.0, -15.0], [25.0, -30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank4_offset1(self):
      arg0=Data(numpy.array([4.0, 4.0, -5.0]),self.functionspace)
      arg1=Data(numpy.array([[[[5.0, -5.0], [2.0, -1.0]], [[-1.0, 2.0], [-1.0, 5.0]], [[-6.0, 0.0], [6.0, 0.0]], [[2.0, -1.0], [1.0, 1.0]], [[4.0, -7.0], [5.0, 0.0]], [[-6.0, -5.0], [4.0, 2.0]]], [[[1.0, -1.0], [-5.0, 5.0]], [[4.0, 3.0], [-1.0, 3.0]], [[7.0, 2.0], [-2.0, -3.0]], [[6.0, 3.0], [-2.0, 0.0]], [[2.0, -7.0], [0.0, -6.0]], [[-4.0, 2.0], [-2.0, -5.0]]], [[[-5.0, 0.0], [2.0, 5.0]], [[6.0, 5.0], [-6.0, 3.0]], [[-1.0, 4.0], [-6.0, -6.0]], [[4.0, 4.0], [-3.0, -6.0]], [[-1.0, 0.0], [1.0, 4.0]], [[-7.0, 7.0], [2.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[49.0, -24.0], [-22.0, -9.0]], [[-18.0, -5.0], [22.0, 17.0]], [[9.0, -12.0], [46.0, 18.0]], [[12.0, -12.0], [11.0, 34.0]], [[29.0, -56.0], [15.0, -44.0]], [[-5.0, -47.0], [-2.0, -12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_constData_rank4_offset0(self):
      arg0=Data(3.0,self.functionspace)
      arg1=Data(numpy.array([[[[3.0, 3.0, 2.0, 1.0], [-1.0, 7.0, -2.0, 0.0], [4.0, 1.0, 7.0, -5.0]], [[-4.0, -7.0, -6.0, -6.0], [3.0, -2.0, -7.0, 2.0], [2.0, 4.0, -7.0, -2.0]]], [[[1.0, 2.0, 3.0, 4.0], [7.0, 1.0, -1.0, 4.0], [5.0, -7.0, -1.0, -3.0]], [[-2.0, -2.0, -5.0, 4.0], [0.0, -6.0, -7.0, -6.0], [-2.0, -3.0, 3.0, -5.0]]], [[[0.0, -1.0, 0.0, -6.0], [-5.0, 7.0, 4.0, -2.0], [-6.0, -3.0, 7.0, 6.0]], [[0.0, 2.0, 7.0, -7.0], [-1.0, -1.0, 1.0, -6.0], [-4.0, -5.0, 7.0, -6.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[9.0, 9.0, 6.0, 3.0], [-3.0, 21.0, -6.0, 0.0], [12.0, 3.0, 21.0, -15.0]], [[-12.0, -21.0, -18.0, -18.0], [9.0, -6.0, -21.0, 6.0], [6.0, 12.0, -21.0, -6.0]]], [[[3.0, 6.0, 9.0, 12.0], [21.0, 3.0, -3.0, 12.0], [15.0, -21.0, -3.0, -9.0]], [[-6.0, -6.0, -15.0, 12.0], [0.0, -18.0, -21.0, -18.0], [-6.0, -9.0, 9.0, -15.0]]], [[[0.0, -3.0, 0.0, -18.0], [-15.0, 21.0, 12.0, -6.0], [-18.0, -9.0, 21.0, 18.0]], [[0.0, 6.0, 21.0, -21.0], [-3.0, -3.0, 3.0, -18.0], [-12.0, -15.0, 21.0, -18.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_expandedData_rank0_offset0(self):
      arg0=Data(4.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(0.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-16.0)+(1.-msk_ref)*numpy.array(0.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([7.0, 2.0, 7.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, 7.0, 2.0])+(1.-msk_arg1)*numpy.array([-2.0, 2.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(-45.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[-6.0, 7.0, 0.0], [4.0, -4.0, 2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -3.0, 0.0], [1.0, 0.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -2.0, -4.0], [6.0, 0.0, 5.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-35.0)+(1.-msk_ref)*numpy.array(62.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[2.0, -4.0, 0.0], [7.0, -2.0, 5.0], [0.0, -6.0, 7.0], [0.0, 0.0, -1.0]], [[-4.0, 2.0, 4.0], [1.0, 2.0, 3.0], [-4.0, 4.0, -4.0], [-3.0, -3.0, -5.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 7.0, 0.0], [5.0, 3.0, -3.0], [7.0, -4.0, -5.0], [-5.0, -5.0, 7.0]], [[2.0, -5.0, 0.0], [1.0, -3.0, 5.0], [0.0, -2.0, 2.0], [6.0, 7.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, 1.0, 0.0], [1.0, -3.0, 5.0], [-3.0, -7.0, -4.0], [-2.0, -6.0, -4.0]], [[0.0, 2.0, -1.0], [3.0, 3.0, 0.0], [-4.0, -1.0, 3.0], [-3.0, 1.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-97.0)+(1.-msk_ref)*numpy.array(79.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-2.0, 5.0], [-5.0, -3.0], [-6.0, 7.0]], [[-2.0, 6.0], [0.0, 2.0], [7.0, 0.0]]], [[[-3.0, 1.0], [-2.0, -6.0], [-1.0, 1.0]], [[3.0, 6.0], [-3.0, 3.0], [3.0, 4.0]]], [[[3.0, -7.0], [-1.0, 1.0], [0.0, 2.0]], [[1.0, -4.0], [1.0, -4.0], [-5.0, 1.0]]], [[[-6.0, 0.0], [6.0, 0.0], [-1.0, -4.0]], [[1.0, -3.0], [6.0, -7.0], [0.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -7.0], [2.0, 2.0], [4.0, 0.0]], [[-1.0, -7.0], [0.0, 7.0], [-1.0, 5.0]]], [[[-4.0, -1.0], [4.0, 0.0], [2.0, -5.0]], [[4.0, -4.0], [-6.0, -5.0], [2.0, 4.0]]], [[[0.0, -4.0], [-1.0, 2.0], [-6.0, 2.0]], [[-5.0, 3.0], [4.0, 0.0], [-4.0, -7.0]]], [[[4.0, -2.0], [1.0, -3.0], [1.0, -1.0]], [[5.0, -5.0], [0.0, -5.0], [0.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, 6.0], [-6.0, 7.0], [0.0, 0.0]], [[-4.0, 1.0], [3.0, -2.0], [6.0, 7.0]]], [[[7.0, 0.0], [0.0, -7.0], [-1.0, -5.0]], [[-2.0, 5.0], [-7.0, 7.0], [3.0, 7.0]]], [[[7.0, 5.0], [2.0, -2.0], [4.0, -3.0]], [[0.0, 3.0], [5.0, 7.0], [-1.0, 6.0]]], [[[3.0, 4.0], [-1.0, -6.0], [0.0, 0.0]], [[6.0, -4.0], [2.0, 0.0], [1.0, -6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-26.0)+(1.-msk_ref)*numpy.array(177.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_expandedData_rank1_offset0(self):
      arg0=Data(6.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 6.0])+(1.-msk_arg1)*numpy.array([-6.0, -3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([0.0, 36.0])+(1.-msk_ref)*numpy.array([-36.0, -18.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([-2.0, -7.0, -1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 2.0], [0.0, 2.0], [-2.0, 3.0]])+(1.-msk_arg1)*numpy.array([[7.0, 3.0], [2.0, 5.0], [-3.0, 5.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-4.0, -21.0])+(1.-msk_ref)*numpy.array([-25.0, -46.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[2.0, 7.0, 0.0], [4.0, 5.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -4.0], [2.0, 2.0], [-6.0, 4.0]], [[-2.0, -2.0], [2.0, 3.0], [-2.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 1.0], [6.0, 4.0], [-6.0, 2.0]], [[0.0, 7.0], [3.0, 0.0], [-1.0, 5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([16.0, 13.0])+(1.-msk_ref)*numpy.array([63.0, 58.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[-7.0, 3.0, -3.0], [3.0, 5.0, 0.0], [2.0, -1.0, -4.0], [7.0, 3.0, 6.0]], [[3.0, 6.0, 7.0], [-1.0, 0.0, 6.0], [4.0, 0.0, 4.0], [0.0, 0.0, -7.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, 6.0], [3.0, -4.0], [-7.0, 4.0]], [[-7.0, -5.0], [3.0, 3.0], [3.0, 3.0]], [[-3.0, -7.0], [-4.0, -4.0], [2.0, -3.0]], [[7.0, 5.0], [-7.0, 2.0], [-5.0, -1.0]]], [[[2.0, 3.0], [-1.0, -7.0], [5.0, -3.0]], [[0.0, 2.0], [4.0, 0.0], [5.0, 3.0]], [[1.0, -4.0], [-1.0, -3.0], [5.0, 1.0]], [[0.0, 0.0], [-2.0, -1.0], [-2.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 7.0], [2.0, -5.0], [-2.0, 7.0]], [[2.0, 1.0], [3.0, 2.0], [-7.0, 5.0]], [[6.0, 1.0], [-3.0, 1.0], [1.0, -7.0]], [[-5.0, 0.0], [0.0, 0.0], [-6.0, 0.0]]], [[[-4.0, 1.0], [1.0, -5.0], [0.0, 7.0]], [[-1.0, 4.0], [-1.0, 0.0], [-4.0, 3.0]], [[-5.0, -6.0], [3.0, -2.0], [3.0, 4.0]], [[6.0, 3.0], [6.0, -5.0], [-7.0, 5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([157.0, -44.0])+(1.-msk_ref)*numpy.array([-36.0, -50.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_expandedData_rank2_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 6.0, 6.0, 2.0, 1.0], [4.0, 7.0, 7.0, 2.0, 6.0], [-5.0, 4.0, 3.0, -5.0, -7.0], [5.0, -6.0, 4.0, 3.0, -1.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -2.0, 7.0, 1.0, -5.0], [-2.0, 2.0, -5.0, 2.0, 7.0], [-5.0, 3.0, -2.0, -7.0, 5.0], [4.0, 5.0, 3.0, 6.0, 2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, -6.0, -6.0, -2.0, -1.0], [-4.0, -7.0, -7.0, -2.0, -6.0], [5.0, -4.0, -3.0, 5.0, 7.0], [-5.0, 6.0, -4.0, -3.0, 1.0]])+(1.-msk_ref)*numpy.array([[4.0, 2.0, -7.0, -1.0, 5.0], [2.0, -2.0, 5.0, -2.0, -7.0], [5.0, -3.0, 2.0, 7.0, -5.0], [-4.0, -5.0, -3.0, -6.0, -2.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([-5.0, 6.0, -1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, -5.0, 3.0, -3.0, 2.0], [-3.0, 2.0, -3.0, -1.0, 5.0], [2.0, 0.0, 5.0, 4.0, 4.0], [-5.0, -5.0, 1.0, 4.0, 4.0]], [[4.0, 0.0, 5.0, -5.0, 2.0], [5.0, 3.0, -3.0, 0.0, 4.0], [4.0, 2.0, -5.0, 0.0, 1.0], [3.0, 4.0, 7.0, 5.0, 1.0]], [[2.0, -6.0, 5.0, -7.0, 2.0], [-3.0, -5.0, 7.0, 5.0, -6.0], [-5.0, 5.0, 1.0, -6.0, 7.0], [0.0, -4.0, 6.0, 6.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 3.0, 5.0, -5.0, 0.0], [-7.0, 3.0, 0.0, -4.0, 3.0], [3.0, 1.0, 2.0, 7.0, 0.0], [-3.0, 5.0, -6.0, -1.0, 1.0]], [[6.0, 6.0, 0.0, 1.0, 0.0], [-1.0, 2.0, 4.0, -7.0, 2.0], [1.0, 1.0, 4.0, -5.0, 0.0], [0.0, -6.0, -4.0, 0.0, -7.0]], [[-7.0, 4.0, -5.0, -3.0, -4.0], [0.0, 7.0, -7.0, -5.0, -2.0], [4.0, 0.0, -5.0, -5.0, 0.0], [5.0, 6.0, 4.0, -5.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[37.0, 31.0, 10.0, -8.0, 0.0], [48.0, 13.0, -10.0, 0.0, 5.0], [19.0, 7.0, -56.0, -14.0, -21.0], [43.0, 53.0, 31.0, 4.0, -21.0]])+(1.-msk_ref)*numpy.array([[73.0, 17.0, -20.0, 34.0, 4.0], [29.0, -10.0, 31.0, -17.0, -1.0], [-13.0, 1.0, 19.0, -60.0, 0.0], [10.0, -67.0, 2.0, 10.0, -48.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[0.0, 7.0, 6.0], [1.0, -1.0, 7.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, -2.0, -2.0, -6.0, 2.0], [2.0, 5.0, -3.0, -3.0, 5.0], [3.0, 5.0, 1.0, 0.0, 7.0], [-5.0, -3.0, 0.0, -4.0, 4.0]], [[-6.0, -5.0, 6.0, 5.0, 1.0], [7.0, -5.0, -6.0, -7.0, 0.0], [-3.0, -3.0, 4.0, 3.0, -1.0], [2.0, 5.0, -3.0, -4.0, 0.0]], [[6.0, -7.0, -7.0, 5.0, -7.0], [-5.0, -4.0, -4.0, 5.0, -6.0], [6.0, 2.0, 0.0, 4.0, 5.0], [5.0, -2.0, -6.0, 7.0, -3.0]]], [[[2.0, 7.0, 0.0, -5.0, -2.0], [-2.0, -3.0, 7.0, -7.0, 1.0], [-3.0, -5.0, -2.0, 6.0, -7.0], [-3.0, 5.0, 0.0, 0.0, 1.0]], [[2.0, -6.0, 4.0, 0.0, -7.0], [2.0, 5.0, -7.0, -4.0, 6.0], [-5.0, 0.0, -6.0, 3.0, -1.0], [-5.0, -6.0, 2.0, 1.0, -4.0]], [[-2.0, -7.0, -5.0, -5.0, -6.0], [-7.0, 4.0, 3.0, 5.0, 0.0], [0.0, -6.0, 7.0, 4.0, 3.0], [-6.0, 5.0, 1.0, 5.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -1.0, -1.0, 5.0, 7.0], [-1.0, -4.0, -5.0, 5.0, 1.0], [2.0, -1.0, 1.0, 3.0, 2.0], [7.0, 7.0, 0.0, 1.0, 0.0]], [[-7.0, 1.0, 2.0, -3.0, 0.0], [1.0, -5.0, -7.0, -3.0, -3.0], [-6.0, 4.0, -2.0, 3.0, 5.0], [-4.0, -3.0, -2.0, 0.0, 3.0]], [[2.0, 3.0, 6.0, 5.0, 6.0], [4.0, -5.0, -7.0, 1.0, -5.0], [7.0, 5.0, 0.0, 0.0, 1.0], [-7.0, 5.0, 3.0, 3.0, -6.0]]], [[[-3.0, 0.0, 7.0, -1.0, 1.0], [4.0, -1.0, -1.0, 6.0, 1.0], [-4.0, 0.0, 2.0, 0.0, -6.0], [3.0, -4.0, 0.0, -4.0, 0.0]], [[-3.0, 4.0, 5.0, 0.0, -2.0], [0.0, 0.0, -3.0, 0.0, -4.0], [-2.0, -3.0, -7.0, 2.0, -1.0], [-7.0, 0.0, -3.0, 1.0, 4.0]], [[-1.0, 7.0, 3.0, 5.0, 0.0], [1.0, 6.0, 0.0, 0.0, -5.0], [2.0, 2.0, -6.0, 3.0, 5.0], [-4.0, 6.0, -6.0, 0.0, -4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-20.0, -113.0, -39.0, 25.0, -72.0], [-34.0, -39.0, -31.0, 13.0, -41.0], [17.0, -56.0, 81.0, 76.0, 38.0], [4.0, 69.0, -52.0, 48.0, 22.0]])+(1.-msk_ref)*numpy.array([[-44.0, 70.0, 73.0, 43.0, 39.0], [42.0, -24.0, -89.0, -9.0, -81.0], [12.0, 75.0, -47.0, 40.0, 71.0], [-88.0, 47.0, -35.0, 13.0, -47.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_expandedData_rank3_offset0(self):
      arg0=Data(0.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, -5.0], [-6.0, -2.0]], [[-2.0, 3.0], [1.0, 2.0]], [[-7.0, -6.0], [3.0, -7.0]], [[0.0, -5.0], [-5.0, 1.0]], [[3.0, 6.0], [-1.0, 5.0]], [[0.0, 7.0], [6.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[5.0, 2.0], [3.0, 0.0]], [[7.0, -2.0], [-3.0, -1.0]], [[-3.0, -6.0], [-5.0, 3.0]], [[1.0, -7.0], [-2.0, 0.0]], [[1.0, -2.0], [5.0, 1.0]], [[1.0, 6.0], [-1.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -0.0], [-0.0, -0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[-0.0, -0.0], [0.0, -0.0]], [[0.0, -0.0], [-0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0], [0.0, 0.0]], [[0.0, -0.0], [-0.0, -0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[0.0, -0.0], [-0.0, 0.0]], [[0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([-1.0, 5.0, 1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, 3.0], [0.0, -7.0]], [[0.0, 5.0], [6.0, -3.0]], [[0.0, 0.0], [-1.0, 2.0]], [[-5.0, -2.0], [2.0, 4.0]], [[7.0, -3.0], [2.0, 0.0]], [[7.0, 6.0], [1.0, -7.0]]], [[[1.0, 2.0], [6.0, 7.0]], [[5.0, -2.0], [1.0, -5.0]], [[2.0, 2.0], [2.0, 4.0]], [[-5.0, 2.0], [-5.0, 0.0]], [[6.0, -2.0], [3.0, 7.0]], [[5.0, -2.0], [-1.0, 4.0]]], [[[-6.0, 5.0], [-6.0, 0.0]], [[7.0, -3.0], [0.0, 0.0]], [[0.0, 7.0], [-4.0, -7.0]], [[-2.0, -6.0], [-4.0, 4.0]], [[4.0, 0.0], [-3.0, -6.0]], [[-3.0, -1.0], [-2.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, -7.0], [-3.0, -6.0]], [[3.0, -2.0], [5.0, 1.0]], [[-5.0, -2.0], [4.0, -7.0]], [[2.0, 1.0], [-5.0, -6.0]], [[4.0, -6.0], [7.0, 5.0]], [[0.0, -4.0], [4.0, 7.0]]], [[[-1.0, -5.0], [-6.0, -3.0]], [[0.0, 7.0], [4.0, 0.0]], [[-2.0, 0.0], [4.0, 0.0]], [[5.0, 7.0], [0.0, 6.0]], [[3.0, 3.0], [-4.0, -7.0]], [[4.0, 5.0], [0.0, -4.0]]], [[[7.0, 0.0], [0.0, -1.0]], [[0.0, -1.0], [-6.0, 4.0]], [[-2.0, 1.0], [-2.0, 0.0]], [[-3.0, -1.0], [-1.0, -5.0]], [[-1.0, 1.0], [-5.0, -6.0]], [[-4.0, -1.0], [-3.0, -6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 12.0], [24.0, 42.0]], [[32.0, -18.0], [-1.0, -22.0]], [[10.0, 17.0], [7.0, 11.0]], [[-22.0, 6.0], [-31.0, 0.0]], [[27.0, -7.0], [10.0, 29.0]], [[15.0, -17.0], [-8.0, 22.0]]])+(1.-msk_ref)*numpy.array([[[-4.0, -18.0], [-27.0, -10.0]], [[-3.0, 36.0], [9.0, 3.0]], [[-7.0, 3.0], [14.0, 7.0]], [[20.0, 33.0], [4.0, 31.0]], [[10.0, 22.0], [-32.0, -46.0]], [[16.0, 28.0], [-7.0, -33.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank0_expandedData_rank4_offset0(self):
      arg0=Data(5.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -5.0, 2.0, 5.0], [-4.0, -5.0, 0.0, 1.0], [6.0, 4.0, -2.0, 7.0]], [[-2.0, -2.0, 6.0, -2.0], [0.0, -5.0, 7.0, -4.0], [0.0, 1.0, 7.0, 1.0]]], [[[-7.0, -7.0, 6.0, 1.0], [1.0, -4.0, 1.0, 3.0], [3.0, -3.0, -6.0, -4.0]], [[-7.0, 7.0, 5.0, -3.0], [-7.0, 2.0, 4.0, 3.0], [3.0, 0.0, 0.0, -7.0]]], [[[0.0, 3.0, 5.0, -6.0], [5.0, 6.0, -3.0, -3.0], [-1.0, -1.0, 1.0, 0.0]], [[-3.0, -6.0, -2.0, -1.0], [-5.0, -4.0, 6.0, 3.0], [-1.0, 1.0, -7.0, -2.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, -6.0, 0.0, 0.0], [-2.0, 0.0, 1.0, 2.0], [-6.0, -6.0, -5.0, 5.0]], [[-2.0, -7.0, 5.0, -7.0], [-3.0, 0.0, -5.0, -7.0], [3.0, -5.0, -5.0, -2.0]]], [[[-2.0, 0.0, 5.0, -7.0], [2.0, 3.0, 3.0, -5.0], [-5.0, 7.0, -1.0, -5.0]], [[6.0, -3.0, -4.0, 2.0], [-5.0, -4.0, 7.0, -7.0], [-7.0, -1.0, 4.0, 5.0]]], [[[-7.0, 3.0, 5.0, 1.0], [1.0, 5.0, -6.0, -5.0], [4.0, -4.0, 0.0, -5.0]], [[5.0, -2.0, -1.0, 0.0], [-6.0, 5.0, -3.0, 0.0], [-2.0, 3.0, -5.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-25.0, -25.0, 10.0, 25.0], [-20.0, -25.0, 0.0, 5.0], [30.0, 20.0, -10.0, 35.0]], [[-10.0, -10.0, 30.0, -10.0], [0.0, -25.0, 35.0, -20.0], [0.0, 5.0, 35.0, 5.0]]], [[[-35.0, -35.0, 30.0, 5.0], [5.0, -20.0, 5.0, 15.0], [15.0, -15.0, -30.0, -20.0]], [[-35.0, 35.0, 25.0, -15.0], [-35.0, 10.0, 20.0, 15.0], [15.0, 0.0, 0.0, -35.0]]], [[[0.0, 15.0, 25.0, -30.0], [25.0, 30.0, -15.0, -15.0], [-5.0, -5.0, 5.0, 0.0]], [[-15.0, -30.0, -10.0, -5.0], [-25.0, -20.0, 30.0, 15.0], [-5.0, 5.0, -35.0, -10.0]]]])+(1.-msk_ref)*numpy.array([[[[-20.0, -30.0, 0.0, 0.0], [-10.0, 0.0, 5.0, 10.0], [-30.0, -30.0, -25.0, 25.0]], [[-10.0, -35.0, 25.0, -35.0], [-15.0, 0.0, -25.0, -35.0], [15.0, -25.0, -25.0, -10.0]]], [[[-10.0, 0.0, 25.0, -35.0], [10.0, 15.0, 15.0, -25.0], [-25.0, 35.0, -5.0, -25.0]], [[30.0, -15.0, -20.0, 10.0], [-25.0, -20.0, 35.0, -35.0], [-35.0, -5.0, 20.0, 25.0]]], [[[-35.0, 15.0, 25.0, 5.0], [5.0, 25.0, -30.0, -25.0], [20.0, -20.0, 0.0, -25.0]], [[25.0, -10.0, -5.0, 0.0], [-30.0, 25.0, -15.0, 0.0], [-10.0, 15.0, -25.0, 10.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_float_rank0_offset0(self):
      arg0=Data(numpy.array([2.0, 2.0]),self.functionspace)
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([0.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank0_offset0(self):
      arg0=Data(numpy.array([0.0, -1.0]),self.functionspace)
      arg1=numpy.array(4.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([0.0, -4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank1_offset1(self):
      arg0=Data(numpy.array([[-4.0, 1.0], [-4.0, 7.0], [5.0, 0.0]]),self.functionspace)
      arg1=numpy.array([-2.0, 0.0, 5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([33.0, -2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[1.0, 6.0], [3.0, -3.0], [3.0, 5.0]], [[-1.0, 0.0], [2.0, -3.0], [-2.0, -1.0]]]),self.functionspace)
      arg1=numpy.array([[6.0, 5.0, -6.0], [-6.0, 4.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([31.0, -14.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[[3.0, -6.0], [0.0, -5.0], [-3.0, -5.0]], [[0.0, 4.0], [-3.0, 1.0], [-6.0, -7.0]], [[0.0, 5.0], [-4.0, 0.0], [-1.0, -4.0]], [[4.0, -3.0], [7.0, 1.0], [0.0, -7.0]]], [[[-3.0, 0.0], [6.0, -5.0], [0.0, -4.0]], [[6.0, -1.0], [6.0, 2.0], [7.0, -2.0]], [[0.0, -6.0], [5.0, 6.0], [2.0, -6.0]], [[-2.0, -1.0], [-5.0, 0.0], [-4.0, 1.0]]]]),self.functionspace)
      arg1=numpy.array([[[-7.0, 5.0, -7.0], [7.0, 0.0, 0.0], [3.0, 0.0, 0.0], [-4.0, 0.0, -4.0]], [[6.0, 4.0, 0.0], [0.0, -3.0, 3.0], [-2.0, 7.0, 1.0], [5.0, 0.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([16.0, 147.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank1_offset0(self):
      arg0=Data(numpy.array([-7.0, -3.0]),self.functionspace)
      arg1=numpy.array([6.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-42.0, -7.0], [-18.0, -3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank2_offset1(self):
      arg0=Data(numpy.array([[5.0, 6.0], [-6.0, 5.0], [0.0, 1.0]]),self.functionspace)
      arg1=numpy.array([[3.0, -7.0], [4.0, 6.0], [1.0, -3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-9.0, -71.0], [39.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[-4.0, 4.0], [-6.0, -2.0], [-7.0, 0.0]], [[-7.0, 0.0], [-4.0, -4.0], [2.0, 2.0]]]),self.functionspace)
      arg1=numpy.array([[[7.0, 1.0], [-6.0, 7.0], [-6.0, 5.0]], [[0.0, 5.0], [-4.0, -4.0], [-1.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[64.0, -96.0], [54.0, 10.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-5.0, -4.0], [-2.0, -1.0], [2.0, 3.0]], [[4.0, -7.0], [0.0, 7.0], [-3.0, -4.0]], [[0.0, -3.0], [-4.0, -6.0], [6.0, 0.0]], [[-1.0, 4.0], [0.0, 3.0], [-4.0, -5.0]]], [[[5.0, -4.0], [-3.0, 4.0], [6.0, 0.0]], [[2.0, 7.0], [-4.0, -6.0], [7.0, -2.0]], [[-7.0, -1.0], [6.0, 4.0], [-3.0, 0.0]], [[6.0, 1.0], [-6.0, -2.0], [2.0, 5.0]]]]),self.functionspace)
      arg1=numpy.array([[[[-4.0, 2.0], [-3.0, -7.0], [7.0, -7.0]], [[-2.0, 6.0], [-3.0, 5.0], [3.0, 7.0]], [[7.0, 0.0], [-1.0, -3.0], [0.0, 7.0]], [[-6.0, -4.0], [0.0, -3.0], [-5.0, 7.0]]], [[[7.0, -2.0], [7.0, 7.0], [-7.0, -4.0]], [[-4.0, -5.0], [0.0, 4.0], [5.0, -1.0]], [[1.0, -1.0], [-7.0, 3.0], [-4.0, 2.0]], [[-2.0, -3.0], [-4.0, 5.0], [-5.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[17.0, -90.0], [-79.0, -110.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank2_offset0(self):
      arg0=Data(numpy.array([-3.0, -7.0]),self.functionspace)
      arg1=numpy.array([[2.0, 7.0, -1.0, 6.0, 5.0], [6.0, -5.0, -2.0, 3.0, 5.0], [0.0, 3.0, -1.0, -2.0, 1.0], [-3.0, -2.0, -1.0, 6.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-6.0, -21.0, 3.0, -18.0, -15.0], [-18.0, 15.0, 6.0, -9.0, -15.0], [0.0, -9.0, 3.0, 6.0, -3.0], [9.0, 6.0, 3.0, -18.0, -18.0]], [[-14.0, -49.0, 7.0, -42.0, -35.0], [-42.0, 35.0, 14.0, -21.0, -35.0], [0.0, -21.0, 7.0, 14.0, -7.0], [21.0, 14.0, 7.0, -42.0, -42.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank3_offset1(self):
      arg0=Data(numpy.array([[-6.0, 5.0], [-3.0, -4.0], [-5.0, 3.0]]),self.functionspace)
      arg1=numpy.array([[[-4.0, 1.0, 2.0, 2.0, 1.0], [-7.0, 2.0, -2.0, 7.0, -6.0], [3.0, 0.0, -2.0, 0.0, 0.0], [3.0, -2.0, 7.0, 1.0, -3.0]], [[-5.0, 0.0, 7.0, 2.0, 2.0], [-7.0, 7.0, -1.0, 3.0, 0.0], [3.0, -2.0, 2.0, -4.0, 7.0], [2.0, 7.0, -4.0, 4.0, -4.0]], [[2.0, -1.0, -3.0, -1.0, 5.0], [3.0, -7.0, -6.0, -7.0, 3.0], [-7.0, 0.0, -5.0, -1.0, 2.0], [4.0, 1.0, -2.0, -1.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[29.0, -1.0, -18.0, -13.0, -37.0], [48.0, 2.0, 45.0, -16.0, 21.0], [8.0, 6.0, 31.0, 17.0, -31.0], [-44.0, -14.0, -20.0, -13.0, 15.0]], [[6.0, 2.0, -27.0, -1.0, 12.0], [2.0, -39.0, -24.0, 2.0, -21.0], [-18.0, 8.0, -33.0, 13.0, -22.0], [19.0, -35.0, 45.0, -14.0, 10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[5.0, -7.0], [-2.0, -3.0], [7.0, 2.0]], [[-4.0, 5.0], [-3.0, -6.0], [5.0, -1.0]]]),self.functionspace)
      arg1=numpy.array([[[[-7.0, 4.0, 0.0, 4.0, 2.0], [-1.0, -5.0, -6.0, -3.0, 1.0], [3.0, -5.0, -6.0, 0.0, 1.0], [4.0, -5.0, 1.0, 3.0, -7.0]], [[1.0, 0.0, -5.0, 3.0, -5.0], [4.0, 0.0, 3.0, 4.0, 0.0], [2.0, -2.0, 5.0, 6.0, -7.0], [6.0, -3.0, 1.0, 3.0, -7.0]], [[1.0, -3.0, -1.0, 6.0, -3.0], [-1.0, 7.0, 6.0, 4.0, 6.0], [3.0, 0.0, -5.0, 5.0, -2.0], [5.0, 1.0, 6.0, 6.0, 3.0]]], [[[-4.0, -2.0, -6.0, -1.0, -4.0], [3.0, 1.0, -5.0, 6.0, 4.0], [3.0, 7.0, 7.0, -5.0, 1.0], [-4.0, -6.0, 0.0, 0.0, 0.0]], [[2.0, -1.0, -6.0, 0.0, 5.0], [7.0, 6.0, -3.0, 3.0, 0.0], [-7.0, 3.0, 0.0, -3.0, -2.0], [1.0, -7.0, 1.0, -4.0, -1.0]], [[-3.0, -5.0, -5.0, 2.0, -7.0], [4.0, 5.0, 3.0, -5.0, 6.0], [-3.0, 2.0, 0.0, -1.0, 6.0], [-1.0, -3.0, 0.0, 3.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-35.0, -15.0, 20.0, 70.0, -35.0], [-33.0, 27.0, 50.0, -53.0, 61.0], [26.0, -48.0, -103.0, 47.0, 37.0], [51.0, 18.0, 42.0, 78.0, 13.0]], [[19.0, -33.0, 24.0, -32.0, -48.0], [-38.0, 13.0, 35.0, 34.0, 19.0], [39.0, 56.0, 52.0, -14.0, 21.0], [-61.0, 61.0, -4.0, 3.0, 80.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_array_rank3_offset0(self):
      arg0=Data(numpy.array([7.0, 1.0]),self.functionspace)
      arg1=numpy.array([[[1.0, 0.0], [5.0, 0.0]], [[0.0, 6.0], [-2.0, -1.0]], [[1.0, 2.0], [-6.0, -2.0]], [[-6.0, -5.0], [-5.0, 5.0]], [[0.0, 1.0], [3.0, 6.0]], [[2.0, 2.0], [7.0, 7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[7.0, 0.0], [35.0, 0.0]], [[0.0, 42.0], [-14.0, -7.0]], [[7.0, 14.0], [-42.0, -14.0]], [[-42.0, -35.0], [-35.0, 35.0]], [[0.0, 7.0], [21.0, 42.0]], [[14.0, 14.0], [49.0, 49.0]]], [[[1.0, 0.0], [5.0, 0.0]], [[0.0, 6.0], [-2.0, -1.0]], [[1.0, 2.0], [-6.0, -2.0]], [[-6.0, -5.0], [-5.0, 5.0]], [[0.0, 1.0], [3.0, 6.0]], [[2.0, 2.0], [7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank4_offset1(self):
      arg0=Data(numpy.array([[5.0, -7.0], [0.0, -4.0], [5.0, -1.0]]),self.functionspace)
      arg1=numpy.array([[[[2.0, 2.0], [-6.0, 0.0]], [[-6.0, -5.0], [-1.0, 6.0]], [[2.0, -2.0], [-3.0, 1.0]], [[-1.0, 6.0], [1.0, -7.0]], [[0.0, 0.0], [-5.0, -6.0]], [[3.0, -3.0], [3.0, -7.0]]], [[[-2.0, 5.0], [2.0, -6.0]], [[1.0, 0.0], [-4.0, -4.0]], [[-7.0, -5.0], [1.0, -2.0]], [[-1.0, 3.0], [-6.0, -1.0]], [[0.0, -1.0], [3.0, 5.0]], [[-2.0, 6.0], [-2.0, -7.0]]], [[[-1.0, 6.0], [-2.0, 0.0]], [[-2.0, 7.0], [1.0, 3.0]], [[3.0, 6.0], [1.0, -6.0]], [[0.0, -5.0], [-6.0, 7.0]], [[0.0, -5.0], [1.0, 5.0]], [[0.0, -5.0], [-7.0, -4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[5.0, 40.0], [-40.0, 0.0]], [[-40.0, 10.0], [0.0, 45.0]], [[25.0, 20.0], [-10.0, -25.0]], [[-5.0, 5.0], [-25.0, 0.0]], [[0.0, -25.0], [-20.0, -5.0]], [[15.0, -40.0], [-20.0, -55.0]]], [[[-5.0, -40.0], [36.0, 24.0]], [[40.0, 28.0], [22.0, -29.0]], [[11.0, 28.0], [16.0, 7.0]], [[11.0, -49.0], [23.0, 46.0]], [[0.0, 9.0], [22.0, 17.0]], [[-13.0, 2.0], [-6.0, 81.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([5.0, 7.0]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(1.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([5.0, 7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[-7.0, 0.0], [2.0, 3.0], [-3.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-1.0, 6.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([1.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[5.0, -4.0], [0.0, 7.0], [-4.0, -4.0]], [[3.0, 0.0], [-2.0, 0.0], [-3.0, 6.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-5.0, 6.0, -3.0], [-5.0, 6.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-28.0, 50.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[[3.0, 0.0], [3.0, 6.0], [3.0, 5.0]], [[-3.0, -3.0], [0.0, -3.0], [-6.0, 3.0]], [[-5.0, 1.0], [-7.0, 5.0], [4.0, -5.0]], [[1.0, -4.0], [3.0, -2.0], [7.0, -4.0]]], [[[-6.0, -7.0], [5.0, -4.0], [3.0, -5.0]], [[1.0, 4.0], [-6.0, 2.0], [3.0, -1.0]], [[-4.0, 1.0], [0.0, 6.0], [1.0, 2.0]], [[6.0, -3.0], [5.0, 5.0], [0.0, 1.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-6.0, 3.0, 0.0], [6.0, 1.0, 1.0], [4.0, -6.0, 3.0], [3.0, 6.0, -6.0]], [[-1.0, 3.0, -2.0], [-4.0, 0.0, 5.0], [0.0, 3.0, 3.0], [-2.0, 3.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([12.0, -18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([0.0, -3.0]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-3.0, 7.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.0, 0.0], [9.0, -21.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[-7.0, -6.0], [6.0, 3.0], [3.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-2.0, -2.0], [-4.0, -2.0], [6.0, 5.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[8.0, 17.0], [-18.0, -9.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[7.0, 4.0], [2.0, 4.0], [4.0, 0.0]], [[1.0, -1.0], [-1.0, 2.0], [3.0, 5.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-1.0, 2.0], [-3.0, -5.0], [0.0, -4.0]], [[3.0, -1.0], [-7.0, -4.0], [2.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[3.0, 6.0], [-23.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-6.0, 2.0], [0.0, -1.0], [-1.0, -3.0]], [[0.0, -5.0], [6.0, -6.0], [-2.0, 0.0]], [[0.0, -4.0], [7.0, -5.0], [-5.0, -4.0]], [[-6.0, -5.0], [-5.0, 0.0], [4.0, 0.0]]], [[[-7.0, -7.0], [-4.0, 5.0], [-7.0, -3.0]], [[2.0, 1.0], [-4.0, 5.0], [-2.0, 4.0]], [[6.0, -6.0], [4.0, 6.0], [-4.0, 7.0]], [[-3.0, 0.0], [0.0, -7.0], [2.0, 6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[1.0, 2.0], [-3.0, 7.0], [5.0, 0.0]], [[1.0, -4.0], [-7.0, -5.0], [-2.0, -1.0]], [[-2.0, 0.0], [2.0, 3.0], [-4.0, 6.0]], [[-1.0, 0.0], [6.0, -1.0], [6.0, 7.0]]], [[[6.0, -1.0], [4.0, -5.0], [1.0, 3.0]], [[-5.0, 0.0], [-3.0, 2.0], [-5.0, 5.0]], [[-3.0, -6.0], [-1.0, 0.0], [0.0, -1.0]], [[7.0, 0.0], [6.0, 0.0], [-5.0, -2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-121.0, -64.0], [-79.0, 28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([-2.0, 0.0]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-7.0, -3.0, 4.0, 4.0, 7.0], [-5.0, 0.0, -2.0, -4.0, -4.0], [0.0, 4.0, -5.0, -7.0, 4.0], [6.0, -2.0, -4.0, 0.0, 2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[14.0, 6.0, -8.0, -8.0, -14.0], [10.0, 0.0, 4.0, 8.0, 8.0], [0.0, -8.0, 10.0, 14.0, -8.0], [-12.0, 4.0, 8.0, 0.0, -4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[0.0, -5.0], [-7.0, 0.0], [-5.0, -2.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[4.0, -4.0, -4.0, 2.0, -3.0], [0.0, 6.0, 5.0, -1.0, 0.0], [5.0, 7.0, 0.0, 0.0, -1.0], [-1.0, 0.0, 0.0, 6.0, -2.0]], [[4.0, -5.0, 5.0, 5.0, -7.0], [1.0, 7.0, 5.0, 5.0, -7.0], [-7.0, 4.0, -4.0, -7.0, 4.0], [5.0, 6.0, -6.0, 0.0, -2.0]], [[7.0, 6.0, 4.0, -6.0, -7.0], [3.0, 3.0, -1.0, -1.0, 5.0], [1.0, 2.0, 1.0, 0.0, 1.0], [4.0, 4.0, 5.0, 2.0, -1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-63.0, 5.0, -55.0, -5.0, 84.0], [-22.0, -64.0, -30.0, -30.0, 24.0], [44.0, -38.0, 23.0, 49.0, -33.0], [-55.0, -62.0, 17.0, -10.0, 19.0]], [[-34.0, 8.0, 12.0, 2.0, 29.0], [-6.0, -36.0, -23.0, 7.0, -10.0], [-27.0, -39.0, -2.0, 0.0, 3.0], [-3.0, -8.0, -10.0, -34.0, 12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[2.0, 1.0], [4.0, 6.0], [6.0, -1.0]], [[-5.0, -2.0], [0.0, 3.0], [0.0, 6.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-7.0, 0.0, 7.0, -6.0, 3.0], [-5.0, -5.0, 5.0, -6.0, 7.0], [3.0, -4.0, -5.0, -2.0, -7.0], [-1.0, 0.0, -3.0, 2.0, -3.0]], [[2.0, 7.0, 3.0, 1.0, -3.0], [0.0, 0.0, 0.0, 2.0, -1.0], [-5.0, -1.0, 1.0, 2.0, -5.0], [7.0, 0.0, 5.0, 4.0, -4.0]], [[1.0, 5.0, 1.0, 7.0, 1.0], [5.0, 0.0, 2.0, 7.0, -5.0], [-5.0, 7.0, -3.0, -5.0, -3.0], [-1.0, -5.0, -6.0, -2.0, 0.0]]], [[[3.0, 2.0, -5.0, -6.0, 6.0], [0.0, 6.0, 5.0, -4.0, 6.0], [0.0, 1.0, 1.0, 2.0, 1.0], [-1.0, -6.0, 7.0, 6.0, 6.0]], [[1.0, 1.0, 0.0, 5.0, -3.0], [3.0, 4.0, 3.0, 5.0, 6.0], [3.0, -4.0, 4.0, 7.0, 6.0], [1.0, -4.0, 7.0, -6.0, 3.0]], [[0.0, -4.0, -2.0, 7.0, -4.0], [0.0, 0.0, -6.0, -4.0, 0.0], [-3.0, 1.0, -6.0, 7.0, 0.0], [4.0, -2.0, -3.0, 3.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-15.0, 48.0, 57.0, 64.0, -30.0], [20.0, -40.0, -3.0, 58.0, -50.0], [-44.0, 25.0, -29.0, -36.0, -57.0], [25.0, 0.0, -57.0, -22.0, -52.0]], [[1.0, 12.0, 22.0, 62.0, -61.0], [-1.0, -5.0, -34.0, -2.0, 12.0], [-31.0, -25.0, -22.0, 74.0, -18.0], [71.0, -7.0, 22.0, 16.0, -30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_Symbol_rank3_offset0(self):
      arg0=Data(numpy.array([-2.0, 2.0]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-5.0, 4.0], [7.0, -3.0]], [[-5.0, -3.0], [2.0, -6.0]], [[3.0, -5.0], [4.0, 6.0]], [[0.0, -5.0], [0.0, -4.0]], [[1.0, -1.0], [6.0, 2.0]], [[-3.0, -5.0], [4.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[10.0, -8.0], [-14.0, 6.0]], [[10.0, 6.0], [-4.0, 12.0]], [[-6.0, 10.0], [-8.0, -12.0]], [[0.0, 10.0], [0.0, 8.0]], [[-2.0, 2.0], [-12.0, -4.0]], [[6.0, 10.0], [-8.0, -4.0]]], [[[-10.0, 8.0], [14.0, -6.0]], [[-10.0, -6.0], [4.0, -12.0]], [[6.0, -10.0], [8.0, 12.0]], [[0.0, -10.0], [0.0, -8.0]], [[2.0, -2.0], [12.0, 4.0]], [[-6.0, -10.0], [8.0, 4.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([[6.0, 0.0], [0.0, 6.0], [7.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[4.0, 6.0], [-6.0, 0.0]], [[4.0, 7.0], [6.0, -5.0]], [[1.0, 3.0], [-4.0, 0.0]], [[3.0, -3.0], [-7.0, -2.0]], [[-3.0, -1.0], [-4.0, 4.0]], [[5.0, -6.0], [1.0, -5.0]]], [[[-3.0, -6.0], [-4.0, -7.0]], [[1.0, 0.0], [6.0, 3.0]], [[-4.0, -1.0], [0.0, 6.0]], [[5.0, 5.0], [1.0, -3.0]], [[0.0, -3.0], [0.0, 0.0]], [[-1.0, 1.0], [6.0, 2.0]]], [[[4.0, 4.0], [-4.0, -4.0]], [[6.0, 6.0], [-4.0, -3.0]], [[7.0, 0.0], [0.0, -7.0]], [[-1.0, 3.0], [7.0, -1.0]], [[0.0, -4.0], [-5.0, -4.0]], [[-5.0, -7.0], [-7.0, -2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[52.0, 64.0], [-64.0, -28.0]], [[66.0, 84.0], [8.0, -51.0]], [[55.0, 18.0], [-24.0, -49.0]], [[11.0, 3.0], [7.0, -19.0]], [[-18.0, -34.0], [-59.0, -4.0]], [[-5.0, -85.0], [-43.0, -44.0]]], [[[-30.0, -48.0], [-12.0, -30.0]], [[-12.0, -18.0], [48.0, 27.0]], [[-45.0, -6.0], [0.0, 57.0]], [[33.0, 21.0], [-15.0, -15.0]], [[0.0, -6.0], [15.0, 12.0]], [[9.0, 27.0], [57.0, 18.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank0_offset0(self):
      arg0=Data(numpy.array([6.0, 3.0]),self.functionspace)
      arg1=Data(-3.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-18.0, -9.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[1.0, 1.0], [3.0, -1.0], [-4.0, -2.0]]),self.functionspace)
      arg1=Data(numpy.array([7.0, 0.0, -1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([11.0, 9.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[1.0, 7.0], [3.0, 0.0], [7.0, -5.0]], [[1.0, -1.0], [-2.0, -1.0], [-6.0, 7.0]]]),self.functionspace)
      arg1=Data(numpy.array([[-1.0, 0.0, 6.0], [0.0, -5.0, -5.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([81.0, -67.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[7.0, 1.0], [-5.0, -7.0], [3.0, -2.0]], [[7.0, 0.0], [-3.0, 0.0], [5.0, -1.0]], [[6.0, 0.0], [0.0, 5.0], [1.0, 7.0]], [[5.0, 1.0], [7.0, 2.0], [-1.0, 7.0]]], [[[-1.0, 2.0], [5.0, -2.0], [4.0, -1.0]], [[4.0, -3.0], [-1.0, 7.0], [-5.0, -7.0]], [[-1.0, 0.0], [-4.0, 6.0], [-2.0, 5.0]], [[1.0, -1.0], [-4.0, -1.0], [-6.0, 7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[1.0, 6.0, -3.0], [4.0, 0.0, -2.0], [5.0, 7.0, 0.0], [1.0, -3.0, -3.0]], [[4.0, 4.0, -1.0], [0.0, -1.0, -2.0], [-7.0, -1.0, -5.0], [-7.0, 0.0, -7.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([82.0, -89.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank1_offset0(self):
      arg0=Data(numpy.array([0.0, 5.0]),self.functionspace)
      arg1=Data(numpy.array([-3.0, 6.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[0.0, 0.0], [-15.0, 30.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[7.0, 4.0], [6.0, -1.0], [-5.0, -4.0]]),self.functionspace)
      arg1=Data(numpy.array([[3.0, -7.0], [-7.0, 1.0], [6.0, -1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-51.0, -38.0], [-5.0, -25.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[5.0, 4.0], [-4.0, 7.0], [-4.0, -7.0]], [[0.0, 6.0], [3.0, -3.0], [-5.0, 0.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 7.0], [2.0, 5.0], [7.0, -7.0]], [[-4.0, -4.0], [6.0, -6.0], [-2.0, -2.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-18.0, 35.0], [-85.0, 106.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[7.0, 1.0], [-4.0, 2.0], [-5.0, 0.0]], [[-4.0, -7.0], [5.0, 4.0], [2.0, -2.0]], [[7.0, 7.0], [5.0, 0.0], [-1.0, 5.0]], [[-5.0, 7.0], [3.0, 0.0], [4.0, 0.0]]], [[[3.0, -2.0], [7.0, 0.0], [-3.0, 4.0]], [[4.0, -5.0], [-6.0, -5.0], [-6.0, 6.0]], [[5.0, 5.0], [-1.0, -7.0], [-6.0, 3.0]], [[-4.0, 0.0], [-3.0, 3.0], [3.0, 7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[3.0, 1.0], [5.0, 7.0], [-5.0, 3.0]], [[6.0, 5.0], [7.0, 2.0], [-5.0, 5.0]], [[-5.0, 0.0], [-4.0, -7.0], [-1.0, 2.0]], [[-7.0, 2.0], [-3.0, -6.0], [1.0, 5.0]]], [[[4.0, -3.0], [-7.0, -6.0], [5.0, -2.0]], [[7.0, -5.0], [2.0, 4.0], [6.0, 0.0]], [[-3.0, 5.0], [-7.0, 3.0], [-2.0, 0.0]], [[0.0, 7.0], [-5.0, 6.0], [1.0, -2.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[-47.0, -200.0], [-57.0, 13.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank2_offset0(self):
      arg0=Data(numpy.array([0.0, -3.0]),self.functionspace)
      arg1=Data(numpy.array([[3.0, -2.0, 2.0, -7.0, 1.0], [-7.0, 2.0, 2.0, -4.0, -4.0], [3.0, -7.0, -3.0, 1.0, 0.0], [-3.0, 6.0, 3.0, 2.0, 6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-9.0, 6.0, -6.0, 21.0, -3.0], [21.0, -6.0, -6.0, 12.0, 12.0], [-9.0, 21.0, 9.0, -3.0, 0.0], [9.0, -18.0, -9.0, -6.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[-5.0, -4.0], [0.0, -2.0], [4.0, -4.0]]),self.functionspace)
      arg1=Data(numpy.array([[[2.0, -6.0, 6.0, -4.0, 0.0], [6.0, 3.0, -4.0, 2.0, -2.0], [-1.0, 5.0, 4.0, 0.0, -7.0], [3.0, -5.0, -1.0, -3.0, -4.0]], [[-5.0, -3.0, 2.0, -4.0, -3.0], [-5.0, 0.0, 1.0, -6.0, -6.0], [-5.0, 4.0, -5.0, -4.0, 5.0], [-7.0, 1.0, -5.0, -4.0, 0.0]], [[2.0, 0.0, -5.0, -2.0, 6.0], [3.0, 0.0, 2.0, 2.0, 6.0], [-1.0, 6.0, 5.0, 1.0, 0.0], [-1.0, 6.0, -1.0, 0.0, -2.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-2.0, 30.0, -50.0, 12.0, 24.0], [-18.0, -15.0, 28.0, -2.0, 34.0], [1.0, -1.0, 0.0, 4.0, 35.0], [-19.0, 49.0, 1.0, 15.0, 12.0]], [[-6.0, 30.0, -8.0, 32.0, -18.0], [-26.0, -12.0, 6.0, -4.0, -4.0], [18.0, -52.0, -26.0, 4.0, 18.0], [6.0, -6.0, 18.0, 20.0, 24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[5.0, 3.0], [3.0, 5.0], [0.0, 5.0]], [[-4.0, 7.0], [3.0, 3.0], [4.0, 1.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-4.0, -5.0, -3.0, 1.0, -1.0], [-6.0, 0.0, -5.0, -7.0, -4.0], [7.0, -2.0, -5.0, -6.0, 4.0], [7.0, -4.0, 4.0, 7.0, 1.0]], [[-7.0, 5.0, 3.0, 2.0, 2.0], [-7.0, -2.0, -4.0, -6.0, -7.0], [0.0, -3.0, 3.0, 3.0, -6.0], [0.0, -6.0, 0.0, -7.0, 0.0]], [[-7.0, -1.0, 4.0, 5.0, 7.0], [2.0, -6.0, -3.0, 5.0, -6.0], [7.0, -6.0, 0.0, 1.0, -6.0], [-1.0, 1.0, 5.0, -5.0, 6.0]]], [[[0.0, 7.0, 6.0, -1.0, 3.0], [3.0, 2.0, 2.0, 1.0, 1.0], [7.0, 0.0, 0.0, -1.0, 2.0], [-3.0, 4.0, -5.0, 6.0, 4.0]], [[7.0, 7.0, 5.0, 0.0, 0.0], [7.0, 0.0, 3.0, 6.0, 0.0], [0.0, 2.0, 5.0, 7.0, -6.0], [6.0, -3.0, 7.0, 0.0, -6.0]], [[0.0, -1.0, -6.0, -3.0, 4.0], [5.0, -3.0, -7.0, 6.0, -3.0], [-3.0, 4.0, 2.0, 4.0, -4.0], [-7.0, -7.0, -5.0, 6.0, -6.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-20.0, -21.0, -39.0, 3.0, 5.0], [-22.0, -26.0, -64.0, -15.0, -57.0], [-5.0, 3.0, 7.0, 20.0, -40.0], [37.0, -91.0, 41.0, 14.0, -53.0]], [[-61.0, 74.0, 77.0, 28.0, 67.0], [4.0, -29.0, -34.0, 5.0, -73.0], [102.0, -41.0, 17.0, 20.0, -56.0], [6.0, -25.0, 18.0, 9.0, 37.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_constData_rank3_offset0(self):
      arg0=Data(numpy.array([2.0, -6.0]),self.functionspace)
      arg1=Data(numpy.array([[[3.0, -2.0], [-6.0, -2.0]], [[-5.0, 6.0], [7.0, 5.0]], [[0.0, -4.0], [-1.0, 7.0]], [[7.0, -4.0], [-7.0, 4.0]], [[0.0, -5.0], [7.0, -5.0]], [[7.0, -4.0], [-4.0, 6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[6.0, -4.0], [-12.0, -4.0]], [[-10.0, 12.0], [14.0, 10.0]], [[0.0, -8.0], [-2.0, 14.0]], [[14.0, -8.0], [-14.0, 8.0]], [[0.0, -10.0], [14.0, -10.0]], [[14.0, -8.0], [-8.0, 12.0]]], [[[-18.0, 12.0], [36.0, 12.0]], [[30.0, -36.0], [-42.0, -30.0]], [[0.0, 24.0], [6.0, -42.0]], [[-42.0, 24.0], [42.0, -24.0]], [[0.0, 30.0], [-42.0, 30.0]], [[-42.0, 24.0], [24.0, -36.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank4_offset1(self):
      arg0=Data(numpy.array([[4.0, 2.0], [5.0, -2.0], [-4.0, -7.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[3.0, 6.0], [-1.0, -6.0]], [[3.0, -6.0], [-4.0, 3.0]], [[2.0, 0.0], [3.0, -6.0]], [[-5.0, 4.0], [-2.0, -2.0]], [[7.0, -6.0], [-3.0, -2.0]], [[-5.0, 6.0], [4.0, 4.0]]], [[[-7.0, 2.0], [5.0, 3.0]], [[4.0, -6.0], [-6.0, 4.0]], [[1.0, -3.0], [4.0, 5.0]], [[5.0, -3.0], [5.0, -1.0]], [[7.0, 4.0], [-4.0, 3.0]], [[4.0, 4.0], [-1.0, 7.0]]], [[[1.0, 7.0], [2.0, -6.0]], [[-4.0, -6.0], [1.0, 3.0]], [[3.0, -4.0], [-7.0, -4.0]], [[-6.0, 7.0], [0.0, -6.0]], [[-6.0, 0.0], [-1.0, 5.0]], [[0.0, -3.0], [-2.0, 4.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-27.0, 6.0], [13.0, 15.0]], [[48.0, -30.0], [-50.0, 20.0]], [[1.0, 1.0], [60.0, 17.0]], [[29.0, -27.0], [17.0, 11.0]], [[87.0, -4.0], [-28.0, -13.0]], [[0.0, 56.0], [19.0, 35.0]]], [[[13.0, -41.0], [-26.0, 24.0]], [[26.0, 42.0], [-3.0, -23.0]], [[-19.0, 34.0], [47.0, 6.0]], [[22.0, -35.0], [-14.0, 40.0]], [[42.0, -20.0], [9.0, -45.0]], [[-18.0, 25.0], [24.0, -34.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([0.0, 4.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(3.0)+(1-msk_arg1)*(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([0.0, 12.0])+(1.-msk_ref)*numpy.array([-0.0, -12.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[-2.0, -3.0], [1.0, 0.0], [0.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([1.0, -4.0, -5.0])+(1.-msk_arg1)*numpy.array([-3.0, -1.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-6.0, -3.0])+(1.-msk_ref)*numpy.array([5.0, 9.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[-4.0, 0.0], [2.0, 6.0], [-6.0, -5.0]], [[-5.0, 0.0], [-5.0, 2.0], [-4.0, -7.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, -2.0, -2.0], [0.0, 5.0, 6.0]])+(1.-msk_arg1)*numpy.array([[1.0, -2.0, 1.0], [3.0, 0.0, 3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-61.0, -34.0])+(1.-msk_ref)*numpy.array([-41.0, -38.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[-4.0, -4.0], [0.0, 5.0], [6.0, -2.0]], [[0.0, 0.0], [1.0, 3.0], [7.0, -4.0]], [[-4.0, -2.0], [-3.0, 6.0], [-7.0, -5.0]], [[6.0, -5.0], [7.0, 3.0], [-6.0, 7.0]]], [[[-6.0, 4.0], [7.0, 4.0], [0.0, 0.0]], [[7.0, 1.0], [0.0, 1.0], [3.0, 2.0]], [[-1.0, 7.0], [-4.0, 7.0], [4.0, 1.0]], [[4.0, 2.0], [0.0, 1.0], [5.0, 3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 4.0, 4.0], [-4.0, 7.0, -5.0], [-4.0, -4.0, 6.0], [-4.0, 0.0, -7.0]], [[-4.0, 7.0, -1.0], [-1.0, 5.0, -4.0], [-2.0, 1.0, -6.0], [-3.0, 0.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 5.0, -1.0], [-3.0, 0.0, -6.0], [7.0, -1.0, 2.0], [-5.0, 7.0, 5.0]], [[4.0, -4.0, 4.0], [3.0, 1.0, 0.0], [-4.0, 3.0, 3.0], [-2.0, 2.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, -40.0])+(1.-msk_ref)*numpy.array([-115.0, 114.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([1.0, 3.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -1.0])+(1.-msk_arg1)*numpy.array([-7.0, -6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -1.0], [0.0, -3.0]])+(1.-msk_ref)*numpy.array([[-7.0, -6.0], [-21.0, -18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[-4.0, -7.0], [4.0, 3.0], [5.0, 1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 1.0], [6.0, -5.0], [-3.0, -4.0]])+(1.-msk_arg1)*numpy.array([[6.0, -3.0], [0.0, -3.0], [5.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-3.0, -44.0], [-6.0, -26.0]])+(1.-msk_ref)*numpy.array([[1.0, 0.0], [-37.0, 12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[7.0, 1.0], [-2.0, 0.0], [-1.0, -4.0]], [[6.0, 4.0], [0.0, -1.0], [-4.0, 0.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -5.0], [-3.0, 6.0], [4.0, 6.0]], [[-3.0, -3.0], [2.0, -1.0], [2.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 3.0], [4.0, -3.0], [0.0, -6.0]], [[1.0, 0.0], [-4.0, 0.0], [-5.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, -51.0], [-30.0, -40.0]])+(1.-msk_ref)*numpy.array([[39.0, 33.0], [11.0, 27.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-4.0, 3.0], [-5.0, 3.0], [2.0, -3.0]], [[0.0, 0.0], [0.0, 4.0], [3.0, -5.0]], [[-5.0, 2.0], [-7.0, -7.0], [-6.0, 2.0]], [[0.0, 1.0], [5.0, 5.0], [-3.0, -1.0]]], [[[0.0, -6.0], [6.0, 4.0], [-5.0, 5.0]], [[7.0, 5.0], [-5.0, 7.0], [7.0, 5.0]], [[1.0, 1.0], [1.0, 2.0], [-3.0, 6.0]], [[6.0, -6.0], [0.0, -6.0], [-3.0, -5.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -5.0], [-7.0, 2.0], [2.0, -7.0]], [[0.0, -6.0], [4.0, -1.0], [-2.0, -6.0]], [[4.0, 0.0], [-7.0, 2.0], [7.0, -2.0]], [[5.0, 3.0], [-6.0, 0.0], [-6.0, -3.0]]], [[[4.0, 2.0], [2.0, 5.0], [7.0, 6.0]], [[2.0, 7.0], [1.0, -3.0], [3.0, -5.0]], [[7.0, -1.0], [3.0, 7.0], [3.0, 6.0]], [[7.0, 0.0], [-4.0, 3.0], [-6.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, 4.0], [3.0, 2.0], [2.0, -3.0]], [[3.0, 2.0], [4.0, -2.0], [5.0, 3.0]], [[1.0, -6.0], [-2.0, 0.0], [-2.0, -7.0]], [[-2.0, -6.0], [3.0, -1.0], [-3.0, 7.0]]], [[[-7.0, 6.0], [5.0, 2.0], [-1.0, -3.0]], [[6.0, 0.0], [0.0, -7.0], [4.0, 0.0]], [[-1.0, 3.0], [-2.0, -5.0], [0.0, -6.0]], [[2.0, 6.0], [6.0, -4.0], [-1.0, 5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[76.0, 2.0], [145.0, 84.0]])+(1.-msk_ref)*numpy.array([[190.0, 122.0], [63.0, -212.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([1.0, 0.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 2.0, -5.0, 6.0, 5.0], [-1.0, 7.0, 4.0, 6.0, 5.0], [-3.0, 1.0, -4.0, 3.0, 0.0], [7.0, 3.0, -6.0, -7.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-3.0, -3.0, 5.0, -6.0, 6.0], [5.0, -4.0, -6.0, -7.0, 3.0], [6.0, 3.0, -7.0, 7.0, -3.0], [-2.0, -1.0, -7.0, 1.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-7.0, 2.0, -5.0, 6.0, 5.0], [-1.0, 7.0, 4.0, 6.0, 5.0], [-3.0, 1.0, -4.0, 3.0, 0.0], [7.0, 3.0, -6.0, -7.0, -6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-3.0, -3.0, 5.0, -6.0, 6.0], [5.0, -4.0, -6.0, -7.0, 3.0], [6.0, 3.0, -7.0, 7.0, -3.0], [-2.0, -1.0, -7.0, 1.0, 6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[-5.0, 6.0], [1.0, -5.0], [0.0, 2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 0.0, -7.0, 0.0, -7.0], [7.0, -2.0, -2.0, -2.0, 0.0], [6.0, -1.0, 0.0, -6.0, 4.0], [-5.0, 6.0, 2.0, -5.0, 2.0]], [[-6.0, -1.0, -5.0, -5.0, 0.0], [0.0, -3.0, 4.0, -3.0, 2.0], [-3.0, -5.0, 6.0, -4.0, 5.0], [6.0, -4.0, -4.0, -7.0, 1.0]], [[2.0, -4.0, -4.0, -2.0, 0.0], [2.0, 0.0, -5.0, 0.0, 7.0], [4.0, 2.0, 7.0, -2.0, -4.0], [6.0, 7.0, 7.0, 1.0, -1.0]]])+(1.-msk_arg1)*numpy.array([[[-4.0, 1.0, -7.0, 0.0, -7.0], [0.0, -1.0, -3.0, -3.0, 6.0], [2.0, 6.0, 0.0, 0.0, 7.0], [7.0, 0.0, 0.0, 5.0, 0.0]], [[-4.0, -4.0, 5.0, -4.0, -7.0], [3.0, 7.0, -7.0, 0.0, 5.0], [-2.0, 4.0, -4.0, -6.0, 5.0], [-6.0, -3.0, 3.0, 0.0, 0.0]], [[-4.0, -5.0, 5.0, 5.0, -6.0], [-7.0, 1.0, 0.0, -6.0, 7.0], [-5.0, -1.0, 5.0, -7.0, 5.0], [-4.0, 6.0, -5.0, -7.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, -1.0, 30.0, -5.0, 35.0], [-35.0, 7.0, 14.0, 7.0, 2.0], [-33.0, 0.0, 6.0, 26.0, -15.0], [31.0, -34.0, -14.0, 18.0, -9.0]], [[34.0, -3.0, -25.0, 21.0, -42.0], [46.0, 3.0, -42.0, 3.0, 4.0], [59.0, 23.0, -16.0, -20.0, -9.0], [-48.0, 70.0, 46.0, 7.0, 5.0]]])+(1.-msk_ref)*numpy.array([[[16.0, -9.0, 40.0, -4.0, 28.0], [3.0, 12.0, 8.0, 15.0, -25.0], [-12.0, -26.0, -4.0, -6.0, -30.0], [-41.0, -3.0, 3.0, -25.0, 0.0]], [[-12.0, 16.0, -57.0, 30.0, -19.0], [-29.0, -39.0, 17.0, -30.0, 25.0], [12.0, 14.0, 30.0, 16.0, 27.0], [64.0, 27.0, -25.0, 16.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[0.0, -6.0], [0.0, -7.0], [4.0, 3.0]], [[-2.0, -5.0], [-3.0, 1.0], [-6.0, 0.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 0.0, -1.0, -6.0, 1.0], [-5.0, 0.0, -7.0, -2.0, -4.0], [6.0, 5.0, 0.0, 0.0, -6.0], [-4.0, -3.0, 6.0, 0.0, -4.0]], [[-6.0, 2.0, -7.0, 2.0, 0.0], [6.0, -5.0, -1.0, 3.0, -2.0], [6.0, 6.0, -5.0, -4.0, -3.0], [6.0, -5.0, 0.0, 3.0, 5.0]], [[3.0, -6.0, -3.0, 0.0, -1.0], [-2.0, -3.0, 5.0, -7.0, 2.0], [5.0, -4.0, 1.0, 0.0, -1.0], [5.0, -4.0, 5.0, 3.0, -2.0]]], [[[6.0, -5.0, -5.0, -6.0, 0.0], [-6.0, 3.0, 6.0, -5.0, 6.0], [-5.0, 7.0, 2.0, 5.0, -2.0], [1.0, -3.0, -2.0, 4.0, -7.0]], [[7.0, 5.0, 1.0, -4.0, -3.0], [-5.0, 5.0, 0.0, -6.0, 0.0], [0.0, 0.0, -6.0, -6.0, 2.0], [-2.0, -5.0, -5.0, 1.0, 2.0]], [[2.0, -6.0, -6.0, 1.0, 6.0], [7.0, 3.0, -4.0, -4.0, -1.0], [-6.0, -4.0, 4.0, -2.0, -7.0], [-1.0, 1.0, 3.0, -5.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, 3.0, 3.0, 1.0, -4.0], [3.0, 6.0, 5.0, 0.0, 2.0], [2.0, -4.0, -3.0, -6.0, -2.0], [-1.0, 4.0, -4.0, 0.0, 0.0]], [[3.0, 7.0, 0.0, 4.0, 0.0], [6.0, -1.0, -3.0, -6.0, 7.0], [1.0, 0.0, -6.0, -7.0, 4.0], [5.0, 5.0, 3.0, -5.0, -5.0]], [[3.0, 1.0, 1.0, 2.0, -7.0], [6.0, 1.0, -5.0, 5.0, 4.0], [6.0, -5.0, -4.0, 3.0, -4.0], [5.0, 3.0, -1.0, 1.0, 0.0]]], [[[-2.0, -6.0, 2.0, 2.0, 5.0], [-2.0, 0.0, 0.0, -4.0, 5.0], [5.0, -7.0, 6.0, -3.0, -2.0], [3.0, 4.0, 3.0, 2.0, 7.0]], [[-7.0, -6.0, 3.0, 4.0, -3.0], [-6.0, 3.0, -5.0, 6.0, 4.0], [6.0, -1.0, 3.0, 0.0, -2.0], [-6.0, 0.0, 6.0, 6.0, 6.0]], [[4.0, -4.0, -7.0, 3.0, 0.0], [-1.0, -6.0, -7.0, 5.0, 6.0], [-7.0, 4.0, 0.0, 4.0, -6.0], [-1.0, -5.0, -7.0, 3.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-33.0, 7.0, 31.0, 18.0, -31.0], [-23.0, -51.0, 32.0, 24.0, 2.0], [66.0, -6.0, -6.0, 20.0, 36.0], [30.0, -1.0, 21.0, 31.0, 30.0]], [[28.0, -2.0, 72.0, 48.0, -12.0], [7.0, 16.0, 34.0, -11.0, 14.0], [-38.0, -119.0, 22.0, -3.0, 66.0], [-10.0, 51.0, -16.0, -31.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[13.0, 58.0, 33.0, -26.0, -29.0], [52.0, 31.0, 37.0, -20.0, -42.0], [38.0, -27.0, -37.0, -6.0, 30.0], [38.0, 34.0, 14.0, -36.0, 10.0]], [[21.0, -40.0, -22.0, -34.0, -25.0], [-38.0, -23.0, -29.0, 83.0, -70.0], [-20.0, 43.0, 21.0, 109.0, -20.0], [-35.0, -70.0, -9.0, 34.0, 6.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank1_expandedData_rank3_offset0(self):
      arg0=Data(numpy.array([0.0, -6.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, -4.0], [-1.0, -2.0]], [[6.0, 0.0], [0.0, 4.0]], [[-2.0, -6.0], [-1.0, -5.0]], [[-4.0, 3.0], [0.0, 7.0]], [[3.0, 1.0], [0.0, 0.0]], [[-7.0, -2.0], [7.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, 4.0], [5.0, -7.0]], [[1.0, -7.0], [4.0, -7.0]], [[-3.0, -1.0], [-5.0, -2.0]], [[-4.0, 6.0], [-2.0, 4.0]], [[0.0, 2.0], [-2.0, 1.0]], [[7.0, 5.0], [-7.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[30.0, 24.0], [6.0, 12.0]], [[-36.0, 0.0], [0.0, -24.0]], [[12.0, 36.0], [6.0, 30.0]], [[24.0, -18.0], [0.0, -42.0]], [[-18.0, -6.0], [0.0, 0.0]], [[42.0, 12.0], [-42.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[42.0, -24.0], [-30.0, 42.0]], [[-6.0, 42.0], [-24.0, 42.0]], [[18.0, 6.0], [30.0, 12.0]], [[24.0, -36.0], [12.0, -24.0]], [[0.0, -12.0], [12.0, -6.0]], [[-42.0, -30.0], [42.0, -24.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([[0.0, 3.0], [-7.0, 7.0], [0.0, 4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, 6.0], [2.0, 1.0]], [[-5.0, 4.0], [0.0, -7.0]], [[-2.0, 3.0], [-6.0, 2.0]], [[-1.0, -5.0], [6.0, 6.0]], [[1.0, 1.0], [-2.0, -3.0]], [[-7.0, -5.0], [-6.0, -6.0]]], [[[1.0, 3.0], [-2.0, 2.0]], [[-6.0, 0.0], [-1.0, -1.0]], [[-5.0, -4.0], [-2.0, 3.0]], [[-6.0, 4.0], [5.0, -7.0]], [[-4.0, 5.0], [-4.0, 5.0]], [[-6.0, -6.0], [-3.0, -1.0]]], [[[-2.0, -5.0], [7.0, 2.0]], [[4.0, 0.0], [0.0, 5.0]], [[4.0, 7.0], [-3.0, 1.0]], [[5.0, -3.0], [-3.0, -6.0]], [[6.0, 5.0], [4.0, 5.0]], [[4.0, -7.0], [-1.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 1.0], [-7.0, 5.0]], [[7.0, 0.0], [-7.0, 4.0]], [[1.0, 4.0], [0.0, 7.0]], [[1.0, -6.0], [1.0, -6.0]], [[0.0, -7.0], [-5.0, 1.0]], [[-2.0, -1.0], [-6.0, 1.0]]], [[[-5.0, 5.0], [2.0, 7.0]], [[2.0, 1.0], [4.0, -1.0]], [[-1.0, 5.0], [0.0, 7.0]], [[-3.0, -1.0], [4.0, 0.0]], [[7.0, -6.0], [-7.0, 7.0]], [[2.0, 1.0], [3.0, -2.0]]], [[[-7.0, 4.0], [7.0, 5.0]], [[-7.0, -3.0], [-2.0, -6.0]], [[0.0, 2.0], [-3.0, -6.0]], [[7.0, 5.0], [0.0, 5.0]], [[3.0, 1.0], [4.0, -4.0]], [[4.0, -2.0], [-4.0, 1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-7.0, -21.0], [14.0, -14.0]], [[42.0, 0.0], [7.0, 7.0]], [[35.0, 28.0], [14.0, -21.0]], [[42.0, -28.0], [-35.0, 49.0]], [[28.0, -35.0], [28.0, -35.0]], [[42.0, 42.0], [21.0, 7.0]]], [[[17.0, 19.0], [20.0, 25.0]], [[-41.0, 12.0], [-7.0, -8.0]], [[-25.0, 9.0], [-44.0, 31.0]], [[-25.0, 1.0], [41.0, -55.0]], [[-1.0, 58.0], [-18.0, 46.0]], [[-47.0, -85.0], [-43.0, -25.0]]]])+(1.-msk_ref)*numpy.array([[[[35.0, -35.0], [-14.0, -49.0]], [[-14.0, -7.0], [-28.0, 7.0]], [[7.0, -35.0], [0.0, -49.0]], [[21.0, 7.0], [-28.0, 0.0]], [[-49.0, 42.0], [49.0, -49.0]], [[-14.0, -7.0], [-21.0, 14.0]]], [[[-54.0, 54.0], [21.0, 84.0]], [[7.0, -5.0], [-1.0, -19.0]], [[-4.0, 55.0], [-12.0, 46.0]], [[10.0, -5.0], [31.0, 2.0]], [[61.0, -59.0], [-48.0, 36.0]], [[24.0, -4.0], [-13.0, -7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_float_rank0_offset0(self):
      arg0=Data(numpy.array([[-3.0, 7.0, 6.0, 6.0, 2.0], [2.0, -6.0, -6.0, -3.0, 2.0], [7.0, -7.0, 4.0, 3.0, 5.0], [4.0, -5.0, 0.0, 6.0, 0.0]]),self.functionspace)
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-0.0, 0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank0_offset0(self):
      arg0=Data(numpy.array([[-7.0, -5.0, 2.0, -4.0, -7.0], [-4.0, 6.0, -2.0, 7.0, 0.0], [6.0, 1.0, 7.0, 3.0, 7.0], [2.0, -2.0, 1.0, 3.0, -3.0]]),self.functionspace)
      arg1=numpy.array(5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-35.0, -25.0, 10.0, -20.0, -35.0], [-20.0, 30.0, -10.0, 35.0, 0.0], [30.0, 5.0, 35.0, 15.0, 35.0], [10.0, -10.0, 5.0, 15.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[-2.0, 4.0, -4.0, 7.0, 2.0], [-2.0, 0.0, 3.0, 2.0, -7.0], [4.0, 6.0, 6.0, 0.0, -1.0], [5.0, -7.0, 7.0, 6.0, 0.0]], [[-7.0, 0.0, 0.0, 6.0, -2.0], [0.0, -7.0, 4.0, 0.0, -7.0], [3.0, -2.0, 0.0, 1.0, 3.0], [7.0, -1.0, 1.0, -2.0, 0.0]], [[4.0, 3.0, 3.0, -1.0, 3.0], [2.0, 0.0, 2.0, 1.0, 0.0], [-2.0, 6.0, -4.0, 5.0, 1.0], [3.0, 0.0, 3.0, -5.0, -1.0]]]),self.functionspace)
      arg1=numpy.array([-3.0, 2.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-4.0, -9.0, 15.0, -10.0, -7.0], [8.0, -14.0, 1.0, -5.0, 7.0], [-8.0, -16.0, -22.0, 7.0, 10.0], [2.0, 19.0, -16.0, -27.0, -1.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[[-3.0, 7.0, -6.0, 5.0, 3.0], [-2.0, -2.0, 6.0, 3.0, 7.0], [4.0, -5.0, 4.0, 3.0, 0.0], [-6.0, 0.0, -5.0, 4.0, -6.0]], [[0.0, 2.0, -5.0, 3.0, 6.0], [-5.0, -2.0, -6.0, -6.0, 5.0], [4.0, -6.0, 5.0, -1.0, 1.0], [7.0, -1.0, 1.0, -3.0, 5.0]], [[-4.0, -1.0, -7.0, 4.0, 0.0], [-6.0, -5.0, -4.0, -3.0, -6.0], [-6.0, 0.0, -1.0, -3.0, -3.0], [5.0, -4.0, 4.0, 2.0, -4.0]]], [[[-6.0, 5.0, 6.0, 1.0, -2.0], [4.0, -3.0, -2.0, 2.0, -5.0], [-2.0, 1.0, -6.0, -4.0, -2.0], [-1.0, 7.0, 3.0, 0.0, 5.0]], [[4.0, 3.0, 7.0, 7.0, 6.0], [-6.0, -5.0, -5.0, -3.0, -2.0], [0.0, -1.0, 6.0, -5.0, 7.0], [0.0, 2.0, -7.0, -6.0, 2.0]], [[-1.0, -6.0, 7.0, 6.0, 6.0], [0.0, -3.0, 0.0, 6.0, -2.0], [-5.0, -2.0, 7.0, -1.0, 1.0], [-1.0, 0.0, -7.0, -7.0, -6.0]]]]),self.functionspace)
      arg1=numpy.array([[0.0, -5.0, 6.0], [3.0, -3.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-50.0, 14.0, -48.0, -33.0, -78.0], [19.0, -2.0, 15.0, 3.0, -62.0], [-42.0, 44.0, -95.0, -6.0, -54.0], [-4.0, -4.0, 77.0, 73.0, -16.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank1_offset0(self):
      arg0=Data(numpy.array([[-5.0, -2.0, 3.0, 1.0, -6.0], [-6.0, 0.0, 4.0, 3.0, 0.0], [-4.0, 5.0, 3.0, 7.0, 1.0], [-7.0, -5.0, 0.0, -5.0, -6.0]]),self.functionspace)
      arg1=numpy.array([-7.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[35.0, 25.0], [14.0, 10.0], [-21.0, -15.0], [-7.0, -5.0], [42.0, 30.0]], [[42.0, 30.0], [0.0, 0.0], [-28.0, -20.0], [-21.0, -15.0], [0.0, 0.0]], [[28.0, 20.0], [-35.0, -25.0], [-21.0, -15.0], [-49.0, -35.0], [-7.0, -5.0]], [[49.0, 35.0], [35.0, 25.0], [0.0, 0.0], [35.0, 25.0], [42.0, 30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[0.0, 6.0, 4.0, 5.0, 2.0], [6.0, 4.0, 0.0, -4.0, 3.0], [-7.0, 6.0, -5.0, 6.0, -7.0], [7.0, -5.0, 4.0, -3.0, -4.0]], [[1.0, 7.0, -7.0, -2.0, 1.0], [0.0, -5.0, -1.0, -1.0, -3.0], [-6.0, -3.0, 0.0, -2.0, 0.0], [3.0, -6.0, 0.0, -3.0, 1.0]], [[0.0, 0.0, -7.0, 0.0, 6.0], [4.0, 2.0, -3.0, 6.0, -2.0], [0.0, -7.0, 0.0, 2.0, 0.0], [5.0, -1.0, 3.0, 7.0, 6.0]]]),self.functionspace)
      arg1=numpy.array([[5.0, 3.0], [-6.0, 5.0], [-7.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-6.0, 5.0], [-12.0, 53.0], [111.0, -30.0], [37.0, 5.0], [-38.0, 17.0]], [[2.0, 22.0], [36.0, -11.0], [27.0, -8.0], [-56.0, -11.0], [47.0, -8.0]], [[1.0, -51.0], [97.0, -4.0], [-25.0, -15.0], [28.0, 10.0], [-35.0, -21.0]], [[-18.0, 41.0], [18.0, -46.0], [-1.0, 15.0], [-46.0, -17.0], [-68.0, -1.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[[3.0, 6.0, 4.0, 0.0, 6.0], [1.0, -4.0, 0.0, 0.0, 3.0], [0.0, 1.0, 7.0, -5.0, 0.0], [7.0, 7.0, -4.0, 7.0, 6.0]], [[-6.0, -1.0, 7.0, -1.0, -4.0], [-1.0, -4.0, 4.0, -4.0, 1.0], [7.0, 0.0, -6.0, -7.0, -2.0], [-5.0, -3.0, 1.0, 0.0, -1.0]], [[-3.0, -1.0, 2.0, 2.0, -6.0], [2.0, 4.0, 1.0, 6.0, 6.0], [-2.0, -7.0, 5.0, -1.0, -4.0], [-5.0, 3.0, -3.0, -6.0, 1.0]]], [[[5.0, 5.0, 5.0, 3.0, 0.0], [-1.0, 6.0, 0.0, 0.0, 1.0], [4.0, 0.0, 4.0, -3.0, 5.0], [6.0, 0.0, 4.0, 5.0, -7.0]], [[0.0, 7.0, -6.0, 4.0, -6.0], [-1.0, 6.0, 3.0, 6.0, 5.0], [-6.0, 0.0, -7.0, 7.0, -7.0], [3.0, 1.0, 1.0, -5.0, 6.0]], [[4.0, -6.0, -6.0, 0.0, -4.0], [2.0, -2.0, 5.0, -5.0, -1.0], [6.0, 4.0, -4.0, -7.0, 6.0], [3.0, 2.0, -2.0, 5.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[[0.0, 0.0], [7.0, 5.0], [7.0, -5.0]], [[0.0, -6.0], [4.0, -7.0], [2.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-55.0, -45.0], [2.0, -79.0], [27.0, 37.0], [23.0, -61.0], [-102.0, 52.0]], [[7.0, -2.0], [20.0, -118.0], [57.0, -6.0], [28.0, -92.0], [67.0, -66.0]], [[23.0, 63.0], [-41.0, 35.0], [-43.0, -30.0], [-42.0, -61.0], [-58.0, 29.0]], [[-52.0, -57.0], [8.0, -37.0], [-14.0, -11.0], [-52.0, 35.0], [24.0, -10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_array_rank2_offset0(self):
      arg0=Data(numpy.array([[0.0, -2.0, 1.0, -2.0, 6.0], [-3.0, -4.0, 2.0, 0.0, -6.0], [7.0, -1.0, -7.0, 2.0, 1.0], [-3.0, 0.0, -4.0, -5.0, -7.0]]),self.functionspace)
      arg1=numpy.array([[0.0, 5.0, -5.0, -2.0, -2.0], [3.0, -7.0, 0.0, 5.0, -4.0], [5.0, 0.0, 4.0, 1.0, 3.0], [3.0, 2.0, 5.0, 4.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, -10.0, 10.0, 4.0, 4.0], [-6.0, 14.0, 0.0, -10.0, 8.0], [-10.0, 0.0, -8.0, -2.0, -6.0], [-6.0, -4.0, -10.0, -8.0, 14.0]], [[0.0, 5.0, -5.0, -2.0, -2.0], [3.0, -7.0, 0.0, 5.0, -4.0], [5.0, 0.0, 4.0, 1.0, 3.0], [3.0, 2.0, 5.0, 4.0, -7.0]], [[0.0, -10.0, 10.0, 4.0, 4.0], [-6.0, 14.0, 0.0, -10.0, 8.0], [-10.0, 0.0, -8.0, -2.0, -6.0], [-6.0, -4.0, -10.0, -8.0, 14.0]], [[0.0, 30.0, -30.0, -12.0, -12.0], [18.0, -42.0, 0.0, 30.0, -24.0], [30.0, 0.0, 24.0, 6.0, 18.0], [18.0, 12.0, 30.0, 24.0, -42.0]]], [[[0.0, -15.0, 15.0, 6.0, 6.0], [-9.0, 21.0, 0.0, -15.0, 12.0], [-15.0, 0.0, -12.0, -3.0, -9.0], [-9.0, -6.0, -15.0, -12.0, 21.0]], [[0.0, -20.0, 20.0, 8.0, 8.0], [-12.0, 28.0, 0.0, -20.0, 16.0], [-20.0, 0.0, -16.0, -4.0, -12.0], [-12.0, -8.0, -20.0, -16.0, 28.0]], [[0.0, 10.0, -10.0, -4.0, -4.0], [6.0, -14.0, 0.0, 10.0, -8.0], [10.0, 0.0, 8.0, 2.0, 6.0], [6.0, 4.0, 10.0, 8.0, -14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, -30.0, 30.0, 12.0, 12.0], [-18.0, 42.0, 0.0, -30.0, 24.0], [-30.0, 0.0, -24.0, -6.0, -18.0], [-18.0, -12.0, -30.0, -24.0, 42.0]]], [[[0.0, 35.0, -35.0, -14.0, -14.0], [21.0, -49.0, 0.0, 35.0, -28.0], [35.0, 0.0, 28.0, 7.0, 21.0], [21.0, 14.0, 35.0, 28.0, -49.0]], [[0.0, -5.0, 5.0, 2.0, 2.0], [-3.0, 7.0, 0.0, -5.0, 4.0], [-5.0, 0.0, -4.0, -1.0, -3.0], [-3.0, -2.0, -5.0, -4.0, 7.0]], [[0.0, -35.0, 35.0, 14.0, 14.0], [-21.0, 49.0, 0.0, -35.0, 28.0], [-35.0, 0.0, -28.0, -7.0, -21.0], [-21.0, -14.0, -35.0, -28.0, 49.0]], [[0.0, 10.0, -10.0, -4.0, -4.0], [6.0, -14.0, 0.0, 10.0, -8.0], [10.0, 0.0, 8.0, 2.0, 6.0], [6.0, 4.0, 10.0, 8.0, -14.0]], [[0.0, 5.0, -5.0, -2.0, -2.0], [3.0, -7.0, 0.0, 5.0, -4.0], [5.0, 0.0, 4.0, 1.0, 3.0], [3.0, 2.0, 5.0, 4.0, -7.0]]], [[[0.0, -15.0, 15.0, 6.0, 6.0], [-9.0, 21.0, 0.0, -15.0, 12.0], [-15.0, 0.0, -12.0, -3.0, -9.0], [-9.0, -6.0, -15.0, -12.0, 21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, -20.0, 20.0, 8.0, 8.0], [-12.0, 28.0, 0.0, -20.0, 16.0], [-20.0, 0.0, -16.0, -4.0, -12.0], [-12.0, -8.0, -20.0, -16.0, 28.0]], [[0.0, -25.0, 25.0, 10.0, 10.0], [-15.0, 35.0, 0.0, -25.0, 20.0], [-25.0, 0.0, -20.0, -5.0, -15.0], [-15.0, -10.0, -25.0, -20.0, 35.0]], [[0.0, -35.0, 35.0, 14.0, 14.0], [-21.0, 49.0, 0.0, -35.0, 28.0], [-35.0, 0.0, -28.0, -7.0, -21.0], [-21.0, -14.0, -35.0, -28.0, 49.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank3_offset1(self):
      arg0=Data(numpy.array([[[5.0, 5.0, 4.0, -7.0, 6.0], [-3.0, -5.0, -4.0, 0.0, 7.0], [7.0, 3.0, 6.0, 4.0, 1.0], [2.0, -3.0, -5.0, -7.0, -1.0]], [[6.0, -3.0, -4.0, 1.0, -7.0], [-1.0, 1.0, -7.0, -2.0, 5.0], [-6.0, -1.0, 4.0, -4.0, 7.0], [0.0, -2.0, 5.0, -3.0, 4.0]], [[5.0, -7.0, -5.0, 3.0, 6.0], [-2.0, -6.0, -1.0, 1.0, -7.0], [4.0, -3.0, -3.0, 2.0, 1.0], [-6.0, -3.0, 7.0, 0.0, 3.0]]]),self.functionspace)
      arg1=numpy.array([[[-6.0, -1.0, 0.0, 7.0, -4.0], [0.0, 2.0, -4.0, -1.0, -1.0], [1.0, -1.0, 7.0, 0.0, 3.0], [-2.0, -5.0, -2.0, 0.0, 4.0]], [[3.0, -2.0, 1.0, -3.0, 2.0], [6.0, -6.0, 2.0, -5.0, 5.0], [2.0, 1.0, 7.0, 2.0, 3.0], [7.0, -6.0, -6.0, 6.0, 0.0]], [[-3.0, -4.0, 0.0, 6.0, 6.0], [-7.0, 5.0, 0.0, -3.0, -5.0], [-6.0, 5.0, -6.0, 4.0, 5.0], [-5.0, 0.0, 0.0, -4.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-27.0, -37.0, 6.0, 47.0, 22.0], [1.0, -1.0, -8.0, -50.0, 0.0], [-13.0, 26.0, 47.0, 32.0, 58.0], [7.0, -61.0, -46.0, 16.0, -5.0]], [[-18.0, 29.0, -3.0, 2.0, -68.0], [31.0, -7.0, -26.0, 31.0, 15.0], [41.0, -43.0, 56.0, -34.0, -29.0], [4.0, -7.0, 8.0, 10.0, 55.0]], [[-21.0, 24.0, -4.0, 10.0, -54.0], [11.0, 7.0, -24.0, 31.0, 1.0], [26.0, -33.0, 30.0, -28.0, -25.0], [-11.0, 4.0, 16.0, -4.0, 41.0]], [[36.0, -7.0, 1.0, -34.0, 48.0], [-15.0, -5.0, 30.0, -7.0, -3.0], [-23.0, 23.0, -60.0, 14.0, -3.0], [6.0, 29.0, 8.0, -6.0, -43.0]], [[-75.0, -16.0, -7.0, 99.0, -2.0], [-84.0, 84.0, -38.0, 11.0, -71.0], [-44.0, 17.0, -43.0, 10.0, 27.0], [-91.0, 12.0, 30.0, -66.0, -6.0]]], [[[21.0, 13.0, -1.0, -30.0, -2.0], [8.0, -10.0, 10.0, 14.0, 8.0], [7.0, -8.0, -16.0, -10.0, -22.0], [9.0, 21.0, 12.0, 2.0, -2.0]], [[51.0, 27.0, 1.0, -74.0, -14.0], [48.0, -46.0, 22.0, 18.0, 40.0], [33.0, -24.0, 8.0, -22.0, -42.0], [47.0, 19.0, 4.0, 30.0, 10.0]], [[6.0, 22.0, -7.0, -13.0, -4.0], [-35.0, 29.0, 2.0, 42.0, -26.0], [-12.0, -8.0, -71.0, -18.0, -38.0], [-36.0, 62.0, 50.0, -38.0, -11.0]], [[-9.0, 0.0, -2.0, 12.0, 2.0], [-19.0, 17.0, -4.0, 7.0, -15.0], [-10.0, 3.0, -20.0, 0.0, -1.0], [-19.0, 12.0, 12.0, -16.0, -5.0]], [[-6.0, 11.0, 5.0, -8.0, -60.0], [79.0, -51.0, -18.0, -11.0, 53.0], [59.0, -37.0, 126.0, -18.0, 1.0], [56.0, -65.0, -44.0, 58.0, 63.0]]], [[[-72.0, -11.0, -6.0, 91.0, -16.0], [-64.0, 70.0, -40.0, 11.0, -57.0], [-29.0, 7.0, -17.0, 4.0, 23.0], [-76.0, 1.0, 22.0, -52.0, 8.0]], [[-12.0, 11.0, -1.0, 6.0, -32.0], [15.0, -3.0, -14.0, 11.0, 7.0], [19.0, -19.0, 32.0, -14.0, -9.0], [2.0, -9.0, 0.0, 6.0, 27.0]], [[-15.0, -2.0, 4.0, 12.0, -34.0], [45.0, -27.0, -16.0, -17.0, 29.0], [32.0, -17.0, 88.0, -4.0, 15.0], [31.0, -54.0, -36.0, 36.0, 39.0]], [[-42.0, -4.0, -4.0, 52.0, -12.0], [-38.0, 42.0, -24.0, 10.0, -34.0], [-16.0, 2.0, -12.0, 0.0, 10.0], [-46.0, 4.0, 16.0, -32.0, 6.0]], [[12.0, -19.0, 7.0, -8.0, 16.0], [35.0, -35.0, 10.0, -39.0, 29.0], [9.0, 11.0, 50.0, 18.0, 29.0], [42.0, -47.0, -44.0, 38.0, -1.0]]], [[[6.0, 22.0, 0.0, -22.0, -44.0], [42.0, -26.0, -8.0, 16.0, 28.0], [38.0, -32.0, 50.0, -24.0, -24.0], [26.0, -10.0, -4.0, 24.0, 38.0]], [[21.0, 19.0, -2.0, -33.0, -10.0], [9.0, -9.0, 8.0, 22.0, 8.0], [11.0, -14.0, -17.0, -16.0, -30.0], [7.0, 27.0, 18.0, 0.0, 3.0]], [[24.0, -33.0, 5.0, -8.0, 72.0], [-19.0, -5.0, 30.0, -41.0, -5.0], [-37.0, 45.0, -42.0, 38.0, 35.0], [10.0, -5.0, -20.0, 2.0, -55.0]], [[33.0, 13.0, -3.0, -40.0, 22.0], [-18.0, 4.0, 22.0, 22.0, -8.0], [-13.0, 4.0, -70.0, -6.0, -30.0], [-7.0, 53.0, 32.0, -18.0, -28.0]], [[9.0, -19.0, 4.0, -1.0, 30.0], [3.0, -11.0, 12.0, -28.0, 6.0], [-11.0, 20.0, 3.0, 20.0, 24.0], [15.0, -19.0, -22.0, 12.0, -19.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[[7.0, -6.0, -4.0, 2.0, -6.0], [-5.0, 5.0, 2.0, -1.0, -2.0], [0.0, 6.0, 3.0, -1.0, 7.0], [7.0, 4.0, -1.0, 3.0, -3.0]], [[0.0, 5.0, -7.0, -3.0, -1.0], [6.0, 7.0, 0.0, 2.0, -3.0], [-4.0, -3.0, -6.0, 0.0, 1.0], [6.0, -7.0, -7.0, 5.0, -3.0]], [[7.0, -2.0, 0.0, 4.0, 3.0], [6.0, 2.0, 4.0, 3.0, 5.0], [0.0, -3.0, 0.0, -7.0, -1.0], [-4.0, 4.0, -5.0, -4.0, -3.0]]], [[[-5.0, 5.0, -5.0, 1.0, -7.0], [-6.0, 0.0, 1.0, -2.0, 4.0], [2.0, -4.0, 6.0, 5.0, -7.0], [1.0, 0.0, -7.0, -4.0, 1.0]], [[0.0, 0.0, 1.0, 7.0, 6.0], [-2.0, 0.0, -1.0, -4.0, -6.0], [2.0, 5.0, 6.0, 2.0, 6.0], [-3.0, 6.0, -3.0, 0.0, -7.0]], [[-7.0, 7.0, -2.0, 1.0, 7.0], [-1.0, 2.0, 4.0, 2.0, 4.0], [6.0, 5.0, 0.0, -6.0, -2.0], [-1.0, 7.0, 4.0, -6.0, -7.0]]]]),self.functionspace)
      arg1=numpy.array([[[[3.0, -6.0, -6.0, -6.0, 4.0], [-5.0, -6.0, -2.0, 1.0, 5.0], [6.0, 6.0, 0.0, -5.0, 4.0], [-5.0, 0.0, -7.0, -2.0, 6.0]], [[-3.0, 7.0, 3.0, -1.0, -5.0], [6.0, 0.0, 0.0, -1.0, -3.0], [-3.0, 0.0, -2.0, -7.0, -4.0], [4.0, 6.0, -4.0, -6.0, 5.0]], [[5.0, -7.0, -3.0, 6.0, 7.0], [-5.0, 5.0, 0.0, 3.0, -2.0], [0.0, 7.0, 1.0, 1.0, -2.0], [-5.0, -4.0, 3.0, -4.0, 6.0]]], [[[-5.0, 0.0, 4.0, 0.0, -4.0], [-4.0, 5.0, 2.0, -5.0, -4.0], [-4.0, 7.0, 4.0, 0.0, -5.0], [3.0, 2.0, -1.0, 7.0, -5.0]], [[3.0, 4.0, -1.0, -4.0, 5.0], [-3.0, 5.0, 4.0, 2.0, -6.0], [6.0, -4.0, -4.0, 7.0, 2.0], [6.0, 5.0, -7.0, -3.0, -1.0]], [[5.0, -5.0, 6.0, 6.0, 6.0], [5.0, 4.0, -2.0, 6.0, 1.0], [4.0, 7.0, -2.0, 0.0, 4.0], [3.0, 7.0, -3.0, -4.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[46.0, -56.0, -125.0, -42.0, 55.0], [-85.0, -60.0, -10.0, 11.0, 34.0], [34.0, 7.0, 1.0, -28.0, 11.0], [-106.0, -87.0, -2.0, -49.0, 67.0]], [[-33.0, 50.0, 119.0, 61.0, -41.0], [85.0, 79.0, 8.0, 0.0, -54.0], [-43.0, 34.0, -6.0, -7.0, -37.0], [96.0, 97.0, -10.0, -3.0, -6.0]], [[27.0, -11.0, -30.0, 15.0, 32.0], [-15.0, -4.0, 6.0, 18.0, 13.0], [15.0, -77.0, -6.0, 76.0, 31.0], [-23.0, -61.0, 60.0, 20.0, -47.0]], [[56.0, -38.0, -30.0, -7.0, 88.0], [-68.0, 52.0, 24.0, 32.0, -34.0], [63.0, 26.0, -16.0, 64.0, 25.0], [6.0, 10.0, -43.0, -20.0, 15.0]], [[88.0, -3.0, 32.0, 73.0, 102.0], [54.0, 74.0, 8.0, 93.0, -34.0], [59.0, -39.0, -61.0, 82.0, 49.0], [47.0, 47.0, -1.0, -89.0, 48.0]]], [[[16.0, 27.0, 2.0, 62.0, 0.0], [56.0, 16.0, -8.0, 27.0, -20.0], [-40.0, -29.0, -20.0, -25.0, -34.0], [-14.0, -17.0, 52.0, -82.0, 62.0]], [[14.0, -5.0, -3.0, -13.0, 11.0], [17.0, -12.0, -14.0, 16.0, 2.0], [17.0, 58.0, -16.0, -72.0, -4.0], [-1.0, 48.0, -63.0, -68.0, 89.0]], [[38.0, -64.0, 5.0, 40.0, 51.0], [-11.0, 24.0, -14.0, 31.0, 8.0], [18.0, 79.0, 4.0, -13.0, 9.0], [-21.0, 9.0, -8.0, -26.0, 56.0]], [[14.0, -27.0, 11.0, 50.0, 7.0], [32.0, -1.0, -22.0, 20.0, 17.0], [-20.0, 31.0, 3.0, -34.0, -8.0], [-26.0, -10.0, 32.0, -32.0, 48.0]], [[10.0, -88.0, 34.0, 93.0, 20.0], [-11.0, 43.0, -20.0, 8.0, 13.0], [-39.0, 103.0, 43.0, -6.0, -22.0], [-39.0, -32.0, 67.0, 32.0, 13.0]]], [[[38.0, -50.0, 30.0, 32.0, 58.0], [-8.0, 44.0, 0.0, 34.0, -2.0], [40.0, 48.0, -4.0, 42.0, 34.0], [20.0, 32.0, -18.0, 8.0, 4.0]], [[72.0, -41.0, -27.0, -41.0, 89.0], [-7.0, -26.0, -10.0, 60.0, 36.0], [111.0, 2.0, -43.0, 23.0, 92.0], [6.0, 46.0, -85.0, -45.0, 48.0]], [[15.0, -36.0, -18.0, -36.0, 48.0], [-93.0, 42.0, 30.0, -9.0, -27.0], [48.0, 36.0, 12.0, 69.0, 18.0], [15.0, 6.0, -45.0, 54.0, -48.0]], [[-87.0, 93.0, 9.0, -80.0, -99.0], [-16.0, -18.0, 32.0, -79.0, -29.0], [-38.0, -70.0, 17.0, 12.0, -35.0], [49.0, 6.0, -15.0, 83.0, -111.0]], [[56.0, 6.0, -82.0, -85.0, 62.0], [-24.0, -60.0, 0.0, 38.0, 24.0], [95.0, -52.0, -51.0, -1.0, 65.0], [-17.0, 12.0, -85.0, -75.0, 58.0]]], [[[-36.0, 21.0, -11.0, -66.0, -55.0], [21.0, -76.0, -22.0, -28.0, 38.0], [-2.0, 26.0, 2.0, -102.0, -3.0], [-9.0, 32.0, -62.0, -14.0, 40.0]], [[106.0, -112.0, -21.0, 25.0, 151.0], [-65.0, 54.0, 2.0, 77.0, 4.0], [109.0, 77.0, -20.0, 75.0, 76.0], [-11.0, 21.0, -51.0, -28.0, 49.0]], [[39.0, -40.0, -1.0, 19.0, 33.0], [45.0, -53.0, -32.0, 44.0, 76.0], [41.0, -50.0, -15.0, 28.0, 79.0], [-25.0, -23.0, 36.0, 8.0, -9.0]], [[-36.0, 75.0, -43.0, -83.0, -61.0], [21.0, -82.0, -2.0, -30.0, 18.0], [-5.0, -80.0, -18.0, -54.0, -4.0], [-5.0, -4.0, -31.0, -24.0, 3.0]], [[-76.0, 25.0, -13.0, -11.0, -99.0], [-6.0, -55.0, -6.0, -70.0, 31.0], [-83.0, -53.0, 49.0, -16.0, -41.0], [-42.0, -88.0, 93.0, 92.0, -91.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[4.0, -2.0, 5.0, 0.0, -6.0], [-6.0, 2.0, 7.0, -6.0, -6.0], [7.0, -6.0, -4.0, 1.0, 4.0], [-7.0, 3.0, -2.0, 1.0, -4.0]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(6.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[24.0, -12.0, 30.0, 0.0, -36.0], [-36.0, 12.0, 42.0, -36.0, -36.0], [42.0, -36.0, -24.0, 6.0, 24.0], [-42.0, 18.0, -12.0, 6.0, -24.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[-4.0, -2.0, 7.0, 0.0, -6.0], [0.0, 1.0, 4.0, -7.0, 2.0], [2.0, 0.0, -2.0, 2.0, -1.0], [7.0, -2.0, 6.0, 1.0, 6.0]], [[-4.0, -6.0, 0.0, -7.0, 1.0], [-7.0, 6.0, -6.0, 3.0, -2.0], [-1.0, 3.0, -3.0, -1.0, 1.0], [-6.0, 5.0, -7.0, -6.0, -7.0]], [[-1.0, 0.0, -4.0, 5.0, -1.0], [-6.0, 0.0, 5.0, 4.0, -2.0], [-1.0, 1.0, 0.0, 6.0, 5.0], [6.0, 1.0, -6.0, -5.0, 7.0]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-5.0, 2.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[12.0, -2.0, -35.0, -14.0, 32.0], [-14.0, 7.0, -32.0, 41.0, -14.0], [-12.0, 6.0, 4.0, -12.0, 7.0], [-47.0, 20.0, -44.0, -17.0, -44.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[[3.0, 2.0, 7.0, 4.0, -7.0], [5.0, 2.0, -6.0, -4.0, -3.0], [1.0, 5.0, 7.0, 7.0, -3.0], [-7.0, 4.0, 5.0, 5.0, 2.0]], [[7.0, -4.0, -5.0, -1.0, -4.0], [5.0, -5.0, -5.0, 4.0, -3.0], [-7.0, -5.0, 0.0, 2.0, -3.0], [2.0, -5.0, -1.0, -5.0, 7.0]], [[-4.0, 5.0, -1.0, 2.0, -6.0], [7.0, 2.0, 4.0, 7.0, -7.0], [0.0, -3.0, -5.0, 3.0, 5.0], [-4.0, 4.0, -6.0, 5.0, 0.0]]], [[[-3.0, 5.0, 5.0, 1.0, 0.0], [-3.0, 3.0, -2.0, -2.0, -7.0], [7.0, -2.0, 0.0, -7.0, -3.0], [3.0, -6.0, -1.0, 4.0, -6.0]], [[-2.0, -7.0, -3.0, 3.0, -5.0], [-7.0, 5.0, 0.0, -1.0, -7.0], [6.0, -1.0, 0.0, -2.0, -4.0], [4.0, -7.0, -7.0, -5.0, -7.0]], [[-5.0, -7.0, 6.0, 2.0, -2.0], [7.0, -5.0, 2.0, -4.0, -3.0], [6.0, -4.0, 5.0, -1.0, 5.0], [-3.0, -1.0, -6.0, 6.0, -3.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-7.0, 7.0, -3.0], [-2.0, -3.0, 5.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[27.0, -81.0, -52.0, -42.0, 44.0], [41.0, -101.0, 9.0, 22.0, 41.0], [-58.0, -74.0, -9.0, -29.0, 28.0], [42.0, -47.0, -31.0, -48.0, 53.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[-7.0, 3.0, -1.0, 4.0, -2.0], [4.0, -3.0, -1.0, 0.0, 4.0], [4.0, 0.0, 2.0, -1.0, 2.0], [2.0, -5.0, 0.0, 3.0, 4.0]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-1.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[7.0, 49.0], [-3.0, -21.0], [1.0, 7.0], [-4.0, -28.0], [2.0, 14.0]], [[-4.0, -28.0], [3.0, 21.0], [1.0, 7.0], [0.0, 0.0], [-4.0, -28.0]], [[-4.0, -28.0], [0.0, 0.0], [-2.0, -14.0], [1.0, 7.0], [-2.0, -14.0]], [[-2.0, -14.0], [5.0, 35.0], [0.0, 0.0], [-3.0, -21.0], [-4.0, -28.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[3.0, -2.0, 2.0, 0.0, -5.0], [-7.0, -2.0, 6.0, -1.0, 4.0], [2.0, 0.0, 4.0, 6.0, 0.0], [3.0, -4.0, -5.0, -2.0, -6.0]], [[2.0, 5.0, 5.0, 5.0, 0.0], [3.0, 2.0, -3.0, -3.0, 4.0], [-6.0, -1.0, 0.0, 0.0, 6.0], [0.0, 2.0, 2.0, -5.0, 3.0]], [[4.0, 6.0, -1.0, 0.0, -3.0], [-4.0, -4.0, 5.0, -2.0, -5.0], [-7.0, -1.0, -2.0, 5.0, -1.0], [7.0, 5.0, -6.0, 0.0, -3.0]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-2.0, 6.0], [0.0, 7.0], [7.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[22.0, 8.0], [46.0, -13.0], [-11.0, 53.0], [0.0, 35.0], [-11.0, -12.0]], [[-14.0, 3.0], [-24.0, 26.0], [23.0, -15.0], [-12.0, -15.0], [-43.0, 82.0]], [[-53.0, 12.0], [-7.0, -1.0], [-22.0, 36.0], [23.0, 6.0], [-7.0, 48.0]], [[43.0, -24.0], [43.0, -40.0], [-32.0, 20.0], [4.0, -47.0], [-9.0, 3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[[-4.0, -4.0, 1.0, -5.0, 0.0], [2.0, 0.0, -3.0, 3.0, 7.0], [6.0, -4.0, -1.0, -3.0, -4.0], [-6.0, -1.0, 5.0, 7.0, 6.0]], [[-7.0, 4.0, 1.0, -4.0, -6.0], [5.0, 4.0, 4.0, 2.0, 6.0], [-4.0, 0.0, 3.0, -3.0, -4.0], [5.0, -7.0, 0.0, 0.0, -6.0]], [[-3.0, -5.0, 5.0, 3.0, -6.0], [7.0, -6.0, -6.0, -6.0, 5.0], [-7.0, -2.0, 0.0, 2.0, 3.0], [4.0, -5.0, 6.0, 2.0, 1.0]]], [[[5.0, 1.0, 6.0, -1.0, 2.0], [0.0, 5.0, 7.0, 1.0, 6.0], [-2.0, 7.0, -6.0, 4.0, 3.0], [-7.0, 2.0, -7.0, 2.0, 0.0]], [[4.0, -7.0, -1.0, 4.0, -5.0], [6.0, -2.0, 4.0, 2.0, 5.0], [1.0, 0.0, 0.0, -3.0, -3.0], [-3.0, -6.0, 3.0, 0.0, 4.0]], [[-4.0, -2.0, 1.0, -4.0, 0.0], [-1.0, 4.0, 1.0, -7.0, -4.0], [-4.0, 0.0, -2.0, -3.0, 2.0], [-4.0, 1.0, 7.0, -1.0, 5.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, -4.0], [-5.0, 3.0], [1.0, 4.0]], [[2.0, 3.0], [-6.0, 6.0], [4.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[2.0, 2.0], [11.0, -41.0], [22.0, 36.0], [-19.0, 21.0], [58.0, -66.0]], [[-58.0, 66.0], [12.0, 11.0], [-32.0, 50.0], [-54.0, -50.0], [-59.0, 38.0]], [[-13.0, -84.0], [12.0, 29.0], [-35.0, -15.0], [31.0, -10.0], [55.0, 17.0]], [[-33.0, -4.0], [74.0, -62.0], [2.0, 36.0], [2.0, -19.0], [27.0, 11.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([[-6.0, 0.0, -1.0, 6.0, 2.0], [-4.0, 0.0, 6.0, -7.0, -6.0], [1.0, -4.0, 1.0, 0.0, -3.0], [-4.0, -1.0, 0.0, 1.0, 7.0]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-6.0, -4.0, -7.0, -7.0, 2.0], [5.0, -7.0, 3.0, 0.0, 2.0], [2.0, -7.0, -2.0, 1.0, -7.0], [2.0, 0.0, 3.0, -5.0, -1.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[36.0, 24.0, 42.0, 42.0, -12.0], [-30.0, 42.0, -18.0, 0.0, -12.0], [-12.0, 42.0, 12.0, -6.0, 42.0], [-12.0, 0.0, -18.0, 30.0, 6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[6.0, 4.0, 7.0, 7.0, -2.0], [-5.0, 7.0, -3.0, 0.0, -2.0], [-2.0, 7.0, 2.0, -1.0, 7.0], [-2.0, 0.0, -3.0, 5.0, 1.0]], [[-36.0, -24.0, -42.0, -42.0, 12.0], [30.0, -42.0, 18.0, 0.0, 12.0], [12.0, -42.0, -12.0, 6.0, -42.0], [12.0, 0.0, 18.0, -30.0, -6.0]], [[-12.0, -8.0, -14.0, -14.0, 4.0], [10.0, -14.0, 6.0, 0.0, 4.0], [4.0, -14.0, -4.0, 2.0, -14.0], [4.0, 0.0, 6.0, -10.0, -2.0]]], [[[24.0, 16.0, 28.0, 28.0, -8.0], [-20.0, 28.0, -12.0, 0.0, -8.0], [-8.0, 28.0, 8.0, -4.0, 28.0], [-8.0, 0.0, -12.0, 20.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-36.0, -24.0, -42.0, -42.0, 12.0], [30.0, -42.0, 18.0, 0.0, 12.0], [12.0, -42.0, -12.0, 6.0, -42.0], [12.0, 0.0, 18.0, -30.0, -6.0]], [[42.0, 28.0, 49.0, 49.0, -14.0], [-35.0, 49.0, -21.0, 0.0, -14.0], [-14.0, 49.0, 14.0, -7.0, 49.0], [-14.0, 0.0, -21.0, 35.0, 7.0]], [[36.0, 24.0, 42.0, 42.0, -12.0], [-30.0, 42.0, -18.0, 0.0, -12.0], [-12.0, 42.0, 12.0, -6.0, 42.0], [-12.0, 0.0, -18.0, 30.0, 6.0]]], [[[-6.0, -4.0, -7.0, -7.0, 2.0], [5.0, -7.0, 3.0, 0.0, 2.0], [2.0, -7.0, -2.0, 1.0, -7.0], [2.0, 0.0, 3.0, -5.0, -1.0]], [[24.0, 16.0, 28.0, 28.0, -8.0], [-20.0, 28.0, -12.0, 0.0, -8.0], [-8.0, 28.0, 8.0, -4.0, 28.0], [-8.0, 0.0, -12.0, 20.0, 4.0]], [[-6.0, -4.0, -7.0, -7.0, 2.0], [5.0, -7.0, 3.0, 0.0, 2.0], [2.0, -7.0, -2.0, 1.0, -7.0], [2.0, 0.0, 3.0, -5.0, -1.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[18.0, 12.0, 21.0, 21.0, -6.0], [-15.0, 21.0, -9.0, 0.0, -6.0], [-6.0, 21.0, 6.0, -3.0, 21.0], [-6.0, 0.0, -9.0, 15.0, 3.0]]], [[[24.0, 16.0, 28.0, 28.0, -8.0], [-20.0, 28.0, -12.0, 0.0, -8.0], [-8.0, 28.0, 8.0, -4.0, 28.0], [-8.0, 0.0, -12.0, 20.0, 4.0]], [[6.0, 4.0, 7.0, 7.0, -2.0], [-5.0, 7.0, -3.0, 0.0, -2.0], [-2.0, 7.0, 2.0, -1.0, 7.0], [-2.0, 0.0, -3.0, 5.0, 1.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, -4.0, -7.0, -7.0, 2.0], [5.0, -7.0, 3.0, 0.0, 2.0], [2.0, -7.0, -2.0, 1.0, -7.0], [2.0, 0.0, 3.0, -5.0, -1.0]], [[-42.0, -28.0, -49.0, -49.0, 14.0], [35.0, -49.0, 21.0, 0.0, 14.0], [14.0, -49.0, -14.0, 7.0, -49.0], [14.0, 0.0, 21.0, -35.0, -7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[[-3.0, 0.0, 7.0, 4.0, -7.0], [6.0, -5.0, 7.0, 6.0, -3.0], [7.0, 5.0, 0.0, 7.0, 1.0], [-3.0, -2.0, 2.0, -1.0, 3.0]], [[-1.0, -2.0, -3.0, 4.0, 6.0], [0.0, 1.0, 5.0, -5.0, 7.0], [-2.0, -1.0, 3.0, 1.0, -2.0], [-7.0, 7.0, 7.0, -4.0, 1.0]], [[-2.0, -5.0, -1.0, -2.0, -3.0], [0.0, 7.0, -6.0, 1.0, 6.0], [-2.0, 7.0, 3.0, 2.0, -5.0], [-2.0, 2.0, -3.0, 3.0, 2.0]]]),self.functionspace)
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-3.0, 4.0, -5.0, -5.0, 2.0], [-4.0, 0.0, 2.0, 7.0, -4.0], [0.0, -6.0, 1.0, 3.0, -4.0], [-3.0, 1.0, 0.0, 5.0, -2.0]], [[-6.0, 5.0, 1.0, -6.0, -6.0], [1.0, 7.0, -3.0, -1.0, 6.0], [-5.0, 3.0, 3.0, -3.0, -3.0], [7.0, -1.0, 6.0, 4.0, 7.0]], [[-5.0, 2.0, 6.0, 6.0, -4.0], [1.0, -7.0, 4.0, -6.0, -5.0], [1.0, 0.0, -1.0, 7.0, -3.0], [-1.0, 0.0, 1.0, -4.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[25.0, -21.0, 2.0, 9.0, 8.0], [9.0, 7.0, -11.0, -8.0, 16.0], [3.0, 15.0, -4.0, -20.0, 21.0], [4.0, -2.0, -8.0, -11.0, 9.0]], [[37.0, -20.0, -32.0, -18.0, 32.0], [-7.0, 21.0, -14.0, 32.0, 13.0], [5.0, -6.0, -1.0, -29.0, 21.0], [-9.0, 2.0, -17.0, 12.0, 11.0]], [[2.0, 11.0, -44.0, -23.0, 36.0], [-32.0, -14.0, 19.0, 58.0, -41.0], [14.0, -51.0, -1.0, 23.0, -16.0], [-41.0, 10.0, -19.0, 27.0, -30.0]], [[-26.0, 32.0, -28.0, -56.0, -8.0], [-14.0, 42.0, -12.0, 36.0, 18.0], [-22.0, -12.0, 18.0, -14.0, -22.0], [18.0, 0.0, 22.0, 44.0, 30.0]], [[0.0, -4.0, 23.0, -19.0, -38.0], [31.0, 63.0, -44.0, -37.0, 79.0], [-33.0, 60.0, 14.0, -60.0, 19.0], [66.0, -13.0, 33.0, 1.0, 71.0]]], [[[-18.0, 24.0, -30.0, -30.0, 12.0], [-24.0, 0.0, 12.0, 42.0, -24.0], [0.0, -36.0, 6.0, 18.0, -24.0], [-18.0, 6.0, 0.0, 30.0, -12.0]], [[-26.0, -1.0, 68.0, 61.0, -44.0], [28.0, -42.0, 15.0, -78.0, -9.0], [2.0, 33.0, -9.0, 31.0, -4.0], [15.0, -6.0, 13.0, -49.0, -18.0]], [[-21.0, 41.0, -66.0, -101.0, 8.0], [-29.0, 77.0, -25.0, 80.0, 32.0], [-31.0, -27.0, 28.0, -36.0, -25.0], [20.0, 2.0, 24.0, 79.0, 51.0]], [[7.0, 1.0, -29.0, 6.0, 38.0], [-28.0, -42.0, 31.0, 41.0, -59.0], [26.0, -51.0, -10.0, 40.0, -12.0], [-54.0, 11.0, -29.0, 6.0, -52.0]], [[-63.0, 35.0, 58.0, 9.0, -72.0], [25.0, 7.0, -3.0, -64.0, 24.0], [-29.0, 39.0, 12.0, 12.0, -27.0], [52.0, -10.0, 48.0, -11.0, 25.0]]], [[[1.0, 14.0, -49.0, -35.0, 34.0], [-32.0, 0.0, 12.0, 63.0, -30.0], [8.0, -48.0, 3.0, 13.0, -16.0], [-33.0, 9.0, -14.0, 35.0, -18.0]], [[-44.0, 29.0, 16.0, 23.0, -12.0], [-14.0, -56.0, 41.0, -6.0, -61.0], [12.0, -33.0, -5.0, 67.0, -38.0], [-29.0, 6.0, 1.0, -7.0, -52.0]], [[-33.0, 21.0, 21.0, 0.0, -30.0], [6.0, 0.0, 3.0, -21.0, 3.0], [-12.0, 9.0, 6.0, 12.0, -18.0], [18.0, -3.0, 21.0, 0.0, 6.0]], [[-37.0, 37.0, -22.0, -29.0, 0.0], [-25.0, -7.0, 19.0, 36.0, -32.0], [-3.0, -39.0, 8.0, 32.0, -37.0], [-16.0, 6.0, 8.0, 31.0, -17.0]], [[34.0, -16.0, -37.0, -23.0, 34.0], [-11.0, 21.0, -12.0, 39.0, 9.0], [5.0, -12.0, 0.0, -26.0, 17.0], [-12.0, 3.0, -17.0, 17.0, 9.0]]], [[[61.0, -51.0, -4.0, 45.0, 44.0], [3.0, -35.0, 7.0, -2.0, -20.0], [33.0, -3.0, -22.0, -2.0, 39.0], [-38.0, 4.0, -44.0, -35.0, -33.0]], [[-46.0, 31.0, 29.0, -20.0, -54.0], [17.0, 35.0, -17.0, -33.0, 40.0], [-33.0, 33.0, 17.0, -13.0, -19.0], [53.0, -9.0, 44.0, 10.0, 43.0]], [[-33.0, 37.0, -21.0, -70.0, -26.0], [-4.0, 70.0, -29.0, 25.0, 49.0], [-38.0, 9.0, 26.0, -36.0, -20.0], [46.0, -5.0, 39.0, 50.0, 60.0]], [[12.0, -18.0, 19.0, 47.0, 10.0], [3.0, -49.0, 22.0, -21.0, -35.0], [23.0, -6.0, -16.0, 30.0, 7.0], [-28.0, 3.0, -21.0, -33.0, -41.0]], [[-25.0, 21.0, -2.0, -9.0, -8.0], [-9.0, -7.0, 11.0, 8.0, -16.0], [-3.0, -15.0, 4.0, 20.0, -21.0], [-4.0, 2.0, 8.0, 11.0, -9.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-1.0, 2.0, 5.0, -5.0, -7.0], [3.0, 6.0, -2.0, -3.0, 0.0], [-5.0, 7.0, -2.0, -6.0, 6.0], [-7.0, 6.0, 5.0, 2.0, 0.0]], [[3.0, 0.0, -2.0, 4.0, 1.0], [-4.0, -1.0, 2.0, -6.0, -3.0], [-5.0, -4.0, 0.0, -3.0, -1.0], [-2.0, 6.0, -3.0, 2.0, -7.0]], [[0.0, 0.0, -2.0, -2.0, 5.0], [2.0, 5.0, -5.0, 7.0, -3.0], [-1.0, -5.0, 6.0, 7.0, -4.0], [1.0, 4.0, 4.0, 0.0, 4.0]]], [[[-1.0, 3.0, -6.0, 7.0, -1.0], [-4.0, 4.0, -5.0, 0.0, 5.0], [0.0, 7.0, 2.0, -7.0, 2.0], [4.0, -7.0, 6.0, 1.0, 5.0]], [[-1.0, 1.0, -5.0, 4.0, 7.0], [-6.0, -3.0, 1.0, -7.0, -5.0], [-3.0, 6.0, -3.0, -5.0, 4.0], [-2.0, -3.0, 0.0, -7.0, 7.0]], [[-1.0, 0.0, 2.0, 5.0, -4.0], [-5.0, -2.0, 1.0, -3.0, 2.0], [4.0, 0.0, -6.0, 6.0, 0.0], [7.0, -5.0, 0.0, 7.0, -3.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[1.0, 3.0, 1.0, 0.0, 4.0], [6.0, 7.0, -5.0, 7.0, 6.0], [3.0, 0.0, -2.0, -2.0, 2.0], [0.0, -6.0, -1.0, -6.0, -7.0]], [[-4.0, -2.0, 7.0, 4.0, 2.0], [6.0, -2.0, 0.0, -4.0, 6.0], [0.0, -1.0, -3.0, -4.0, 7.0], [0.0, -7.0, -1.0, 2.0, 2.0]], [[2.0, -6.0, 2.0, -3.0, 2.0], [2.0, -7.0, -5.0, 5.0, -3.0], [5.0, 0.0, 1.0, 1.0, -1.0], [-2.0, -4.0, 6.0, 0.0, 5.0]]], [[[-5.0, 7.0, 1.0, -7.0, -4.0], [7.0, -1.0, 2.0, -5.0, -2.0], [6.0, -3.0, -3.0, 0.0, -7.0], [-7.0, -4.0, -4.0, 0.0, -2.0]], [[-4.0, 7.0, 4.0, 4.0, 7.0], [-5.0, 2.0, -5.0, -2.0, 2.0], [-3.0, 3.0, -4.0, 6.0, 6.0], [1.0, 3.0, -2.0, -5.0, -5.0]], [[6.0, -2.0, 2.0, -7.0, 0.0], [3.0, -3.0, -6.0, 3.0, -1.0], [-5.0, -1.0, 0.0, -7.0, -6.0], [-6.0, 0.0, -5.0, 3.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-10.0, -21.0, 13.0, 22.0, -1.0], [7.0, -11.0, 14.0, -15.0, 13.0], [-1.0, -2.0, 0.0, -9.0, 26.0], [12.0, -14.0, 9.0, 14.0, 16.0]], [[-17.0, 34.0, 9.0, -17.0, 3.0], [28.0, 13.0, -9.0, -3.0, 8.0], [21.0, -6.0, -17.0, 2.0, -11.0], [-20.0, -21.0, -16.0, -17.0, -25.0]], [[71.0, -50.0, -35.0, 6.0, 1.0], [3.0, 43.0, -14.0, 79.0, 24.0], [-26.0, 3.0, 32.0, -48.0, -2.0], [29.0, 1.0, 9.0, -3.0, -4.0]], [[-46.0, 56.0, 52.0, -46.0, -16.0], [34.0, -43.0, -1.0, -89.0, -11.0], [-20.0, -18.0, -41.0, -19.0, -35.0], [-71.0, -6.0, -72.0, 33.0, 19.0]], [[-48.0, -3.0, 29.0, 52.0, 37.0], [-80.0, -59.0, -3.0, -49.0, -31.0], [-3.0, 27.0, -9.0, 85.0, 61.0], [28.0, 40.0, 46.0, -3.0, 27.0]]], [[[37.0, -55.0, -59.0, 17.0, -18.0], [-15.0, 22.0, 27.0, 64.0, -11.0], [38.0, 3.0, 44.0, 11.0, -2.0], [48.0, 0.0, 66.0, -11.0, -1.0]], [[0.0, 1.0, -3.0, -45.0, -5.0], [77.0, 5.0, -20.0, 51.0, 3.0], [86.0, -18.0, -4.0, -7.0, -34.0], [-29.0, -74.0, 25.0, -29.0, -20.0]], [[7.0, -10.0, 3.0, 55.0, 13.0], [-47.0, 21.0, 14.0, -21.0, 26.0], [-69.0, 15.0, 4.0, -10.0, 50.0], [40.0, 41.0, -17.0, 14.0, 2.0]], [[45.0, -82.0, -65.0, -52.0, -59.0], [-14.0, -63.0, 33.0, 43.0, -86.0], [62.0, -12.0, 59.0, 16.0, -79.0], [-3.0, 11.0, 80.0, 32.0, 67.0]], [[13.0, 20.0, -38.0, -72.0, -67.0], [42.0, 6.0, 38.0, -12.0, -31.0], [20.0, -29.0, 11.0, -35.0, -95.0], [-46.0, -2.0, -35.0, 25.0, 2.0]]], [[[49.0, -28.0, -46.0, -57.0, -53.0], [-35.0, -36.0, 21.0, -2.0, -67.0], [-31.0, -8.0, 36.0, -17.0, -86.0], [-25.0, 60.0, -10.0, 47.0, 51.0]], [[-46.0, 150.0, 0.0, -26.0, 24.0], [27.0, 97.0, -26.0, -7.0, 31.0], [20.0, 1.0, -52.0, 33.0, -22.0], [-33.0, -4.0, -73.0, -80.0, -126.0]], [[-24.0, -37.0, -12.0, -2.0, -25.0], [11.0, -46.0, 35.0, -6.0, -34.0], [75.0, -9.0, 16.0, 34.0, -6.0], [7.0, -29.0, 66.0, 9.0, 31.0]], [[111.0, -150.0, -28.0, -46.0, -23.0], [-46.0, -106.0, -30.0, 68.0, -77.0], [-40.0, 3.0, 69.0, -41.0, -57.0], [-6.0, 42.0, 59.0, 73.0, 134.0]], [[-24.0, 86.0, 9.0, 10.0, 34.0], [16.0, 78.0, -26.0, 8.0, 46.0], [-2.0, 7.0, -35.0, 12.0, 19.0], [-2.0, -9.0, -45.0, -58.0, -88.0]]], [[[33.0, -23.0, -9.0, -96.0, -60.0], [7.0, -81.0, 6.0, -31.0, -76.0], [-21.0, -23.0, 17.0, -38.0, -111.0], [-74.0, 30.0, -32.0, 69.0, 80.0]], [[7.0, -78.0, 27.0, 84.0, 51.0], [31.0, 18.0, -19.0, 64.0, 73.0], [30.0, 11.0, 7.0, -15.0, 111.0], [68.0, -75.0, 71.0, -24.0, -1.0]], [[-5.0, 39.0, -2.0, -66.0, -2.0], [62.0, 7.0, -33.0, 37.0, -12.0], [71.0, -15.0, -15.0, 6.0, -57.0], [-50.0, -49.0, -2.0, -36.0, -33.0]], [[59.0, -54.0, 3.0, -76.0, -41.0], [87.0, -26.0, -15.0, 36.0, 1.0], [-2.0, -33.0, 15.0, -103.0, -73.0], [-56.0, -51.0, -29.0, 48.0, 51.0]], [[-35.0, 80.0, -14.0, -26.0, 23.0], [-43.0, 4.0, -27.0, 0.0, -47.0], [44.0, 16.0, -18.0, 95.0, -28.0], [-18.0, 34.0, 12.0, -58.0, -51.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[0.0, 6.0, -2.0, 5.0, -4.0], [5.0, -5.0, 3.0, 7.0, -3.0], [-6.0, -5.0, -5.0, 0.0, -4.0], [4.0, 6.0, -3.0, 4.0, 3.0]]),self.functionspace)
      arg1=Data(1.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[0.0, 6.0, -2.0, 5.0, -4.0], [5.0, -5.0, 3.0, 7.0, -3.0], [-6.0, -5.0, -5.0, 0.0, -4.0], [4.0, 6.0, -3.0, 4.0, 3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[2.0, 4.0, 6.0, -7.0, -5.0], [-6.0, -6.0, -1.0, -5.0, 0.0], [3.0, 7.0, 3.0, -5.0, -3.0], [-7.0, 7.0, 2.0, 7.0, -7.0]], [[0.0, 1.0, -4.0, 1.0, 5.0], [-4.0, 6.0, 4.0, -1.0, 4.0], [0.0, 6.0, 7.0, -6.0, 5.0], [0.0, 2.0, 5.0, 7.0, -6.0]], [[0.0, -4.0, -6.0, 0.0, -3.0], [4.0, 0.0, -6.0, 4.0, 5.0], [5.0, -3.0, -1.0, -4.0, -6.0], [-4.0, -2.0, 3.0, -2.0, 2.0]]]),self.functionspace)
      arg1=Data(numpy.array([1.0, 6.0, 1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[2.0, 6.0, -24.0, -1.0, 22.0], [-26.0, 30.0, 17.0, -7.0, 29.0], [8.0, 40.0, 44.0, -45.0, 21.0], [-11.0, 17.0, 35.0, 47.0, -41.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[1.0, -4.0, -7.0, 5.0, 6.0], [2.0, 3.0, -3.0, -4.0, 0.0], [0.0, -1.0, 0.0, -3.0, -5.0], [-7.0, 7.0, -7.0, -1.0, 5.0]], [[-3.0, 0.0, 4.0, -2.0, -6.0], [5.0, -3.0, 5.0, 0.0, 4.0], [-4.0, 3.0, 5.0, 7.0, -6.0], [0.0, 4.0, 3.0, 7.0, -2.0]], [[2.0, -2.0, -7.0, 0.0, 0.0], [-3.0, 5.0, 6.0, -5.0, -1.0], [-4.0, 7.0, -6.0, 3.0, 5.0], [3.0, -5.0, -1.0, -4.0, -3.0]]], [[[3.0, 1.0, 7.0, 7.0, 0.0], [0.0, -2.0, 2.0, 7.0, -3.0], [-4.0, 0.0, 7.0, -4.0, 6.0], [3.0, -6.0, -3.0, 4.0, -1.0]], [[-3.0, 6.0, -4.0, -5.0, -7.0], [-6.0, -3.0, 6.0, 2.0, -4.0], [0.0, -6.0, 0.0, -5.0, 1.0], [-1.0, 3.0, 0.0, 1.0, -1.0]], [[3.0, 2.0, 1.0, -2.0, 2.0], [7.0, -5.0, 1.0, 2.0, -7.0], [5.0, 2.0, -4.0, 4.0, 0.0], [4.0, -4.0, -7.0, -3.0, 0.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[-1.0, 5.0, -4.0], [1.0, 6.0, -6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-57.0, 37.0, 32.0, -26.0, -90.0], [-43.0, -28.0, 36.0, 31.0, 39.0], [-38.0, -60.0, 80.0, -32.0, -33.0], [-32.0, 69.0, 65.0, 80.0, -10.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[4.0, -2.0, 1.0, 3.0, -5.0], [0.0, 7.0, -1.0, 0.0, 6.0], [1.0, -5.0, 5.0, 2.0, 7.0], [1.0, -2.0, -6.0, 7.0, 1.0]]),self.functionspace)
      arg1=Data(numpy.array([-2.0, 1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-8.0, 4.0], [4.0, -2.0], [-2.0, 1.0], [-6.0, 3.0], [10.0, -5.0]], [[0.0, 0.0], [-14.0, 7.0], [2.0, -1.0], [0.0, 0.0], [-12.0, 6.0]], [[-2.0, 1.0], [10.0, -5.0], [-10.0, 5.0], [-4.0, 2.0], [-14.0, 7.0]], [[-2.0, 1.0], [4.0, -2.0], [12.0, -6.0], [-14.0, 7.0], [-2.0, 1.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[6.0, 3.0, 3.0, 7.0, 6.0], [-7.0, 6.0, -4.0, 4.0, -7.0], [7.0, 7.0, 1.0, 6.0, 5.0], [-1.0, 1.0, -6.0, -5.0, -2.0]], [[-7.0, 3.0, 0.0, -3.0, 2.0], [1.0, 4.0, -7.0, 5.0, -5.0], [-7.0, 2.0, 0.0, 0.0, 1.0], [6.0, 0.0, -1.0, -1.0, 0.0]], [[0.0, -5.0, 4.0, 4.0, 4.0], [6.0, 0.0, 3.0, -1.0, -5.0], [-1.0, 4.0, 7.0, 3.0, -5.0], [-2.0, -5.0, -1.0, 2.0, 3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[-3.0, 0.0], [5.0, 6.0], [0.0, -7.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-53.0, -42.0], [6.0, 53.0], [-9.0, -28.0], [-36.0, -46.0], [-8.0, -16.0]], [[26.0, -36.0], [2.0, 24.0], [-23.0, -63.0], [13.0, 37.0], [-4.0, 5.0]], [[-56.0, -35.0], [-11.0, -16.0], [-3.0, -49.0], [-18.0, -21.0], [-10.0, 41.0]], [[33.0, 50.0], [-3.0, 35.0], [13.0, 1.0], [10.0, -20.0], [6.0, -21.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[3.0, 7.0, 7.0, 1.0, 6.0], [-4.0, 6.0, -2.0, 1.0, -2.0], [7.0, 2.0, 7.0, 1.0, -6.0], [-1.0, 6.0, 4.0, -7.0, -6.0]], [[4.0, -2.0, 1.0, 2.0, 7.0], [3.0, -2.0, 5.0, -3.0, 4.0], [-1.0, 1.0, 2.0, 4.0, -4.0], [7.0, -2.0, 0.0, 0.0, -6.0]], [[3.0, 7.0, 2.0, 2.0, -2.0], [-7.0, 6.0, 4.0, -1.0, -3.0], [-4.0, 0.0, 7.0, 7.0, 5.0], [-4.0, -1.0, -1.0, 6.0, -5.0]]], [[[5.0, 7.0, -1.0, -1.0, 0.0], [-6.0, 0.0, 3.0, 5.0, -7.0], [3.0, -4.0, 1.0, -5.0, -5.0], [-4.0, -5.0, 5.0, 7.0, 4.0]], [[2.0, -7.0, 1.0, 7.0, 4.0], [5.0, -7.0, -5.0, 0.0, 6.0], [3.0, 0.0, 4.0, -6.0, 0.0], [-5.0, -7.0, 1.0, -6.0, 1.0]], [[7.0, 3.0, 0.0, 7.0, -3.0], [-7.0, -1.0, -2.0, 5.0, 6.0], [-2.0, -3.0, 7.0, -2.0, 3.0], [-1.0, -2.0, -7.0, -7.0, -3.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[-6.0, 0.0], [4.0, -3.0], [3.0, 0.0]], [[7.0, 1.0], [-3.0, -2.0], [5.0, 1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[71.0, -4.0], [56.0, 30.0], [-42.0, -6.0], [15.0, -14.0], [-41.0, -32.0]], [[-77.0, -32.0], [-10.0, 19.0], [70.0, -4.0], [39.0, 19.0], [-18.0, -25.0]], [[-56.0, -2.0], [-51.0, -10.0], [17.0, -6.0], [4.0, -7.0], [15.0, 10.0]], [[4.0, -16.0], [-71.0, 13.0], [-30.0, -4.0], [92.0, 12.0], [7.0, 17.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_constData_rank2_offset0(self):
      arg0=Data(numpy.array([[-5.0, 1.0, 6.0, -7.0, -6.0], [5.0, 7.0, 5.0, -6.0, -4.0], [-5.0, -7.0, 0.0, 5.0, -4.0], [-3.0, 6.0, 0.0, 7.0, 7.0]]),self.functionspace)
      arg1=Data(numpy.array([[4.0, 2.0, -6.0, -2.0, 0.0], [0.0, -7.0, 0.0, -7.0, -4.0], [5.0, -7.0, 2.0, -5.0, 7.0], [0.0, 5.0, 2.0, -5.0, -6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-20.0, -10.0, 30.0, 10.0, 0.0], [0.0, 35.0, 0.0, 35.0, 20.0], [-25.0, 35.0, -10.0, 25.0, -35.0], [0.0, -25.0, -10.0, 25.0, 30.0]], [[4.0, 2.0, -6.0, -2.0, 0.0], [0.0, -7.0, 0.0, -7.0, -4.0], [5.0, -7.0, 2.0, -5.0, 7.0], [0.0, 5.0, 2.0, -5.0, -6.0]], [[24.0, 12.0, -36.0, -12.0, 0.0], [0.0, -42.0, 0.0, -42.0, -24.0], [30.0, -42.0, 12.0, -30.0, 42.0], [0.0, 30.0, 12.0, -30.0, -36.0]], [[-28.0, -14.0, 42.0, 14.0, 0.0], [0.0, 49.0, 0.0, 49.0, 28.0], [-35.0, 49.0, -14.0, 35.0, -49.0], [0.0, -35.0, -14.0, 35.0, 42.0]], [[-24.0, -12.0, 36.0, 12.0, 0.0], [0.0, 42.0, 0.0, 42.0, 24.0], [-30.0, 42.0, -12.0, 30.0, -42.0], [0.0, -30.0, -12.0, 30.0, 36.0]]], [[[20.0, 10.0, -30.0, -10.0, 0.0], [0.0, -35.0, 0.0, -35.0, -20.0], [25.0, -35.0, 10.0, -25.0, 35.0], [0.0, 25.0, 10.0, -25.0, -30.0]], [[28.0, 14.0, -42.0, -14.0, 0.0], [0.0, -49.0, 0.0, -49.0, -28.0], [35.0, -49.0, 14.0, -35.0, 49.0], [0.0, 35.0, 14.0, -35.0, -42.0]], [[20.0, 10.0, -30.0, -10.0, 0.0], [0.0, -35.0, 0.0, -35.0, -20.0], [25.0, -35.0, 10.0, -25.0, 35.0], [0.0, 25.0, 10.0, -25.0, -30.0]], [[-24.0, -12.0, 36.0, 12.0, 0.0], [0.0, 42.0, 0.0, 42.0, 24.0], [-30.0, 42.0, -12.0, 30.0, -42.0], [0.0, -30.0, -12.0, 30.0, 36.0]], [[-16.0, -8.0, 24.0, 8.0, 0.0], [0.0, 28.0, 0.0, 28.0, 16.0], [-20.0, 28.0, -8.0, 20.0, -28.0], [0.0, -20.0, -8.0, 20.0, 24.0]]], [[[-20.0, -10.0, 30.0, 10.0, 0.0], [0.0, 35.0, 0.0, 35.0, 20.0], [-25.0, 35.0, -10.0, 25.0, -35.0], [0.0, -25.0, -10.0, 25.0, 30.0]], [[-28.0, -14.0, 42.0, 14.0, 0.0], [0.0, 49.0, 0.0, 49.0, 28.0], [-35.0, 49.0, -14.0, 35.0, -49.0], [0.0, -35.0, -14.0, 35.0, 42.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[20.0, 10.0, -30.0, -10.0, 0.0], [0.0, -35.0, 0.0, -35.0, -20.0], [25.0, -35.0, 10.0, -25.0, 35.0], [0.0, 25.0, 10.0, -25.0, -30.0]], [[-16.0, -8.0, 24.0, 8.0, 0.0], [0.0, 28.0, 0.0, 28.0, 16.0], [-20.0, 28.0, -8.0, 20.0, -28.0], [0.0, -20.0, -8.0, 20.0, 24.0]]], [[[-12.0, -6.0, 18.0, 6.0, 0.0], [0.0, 21.0, 0.0, 21.0, 12.0], [-15.0, 21.0, -6.0, 15.0, -21.0], [0.0, -15.0, -6.0, 15.0, 18.0]], [[24.0, 12.0, -36.0, -12.0, 0.0], [0.0, -42.0, 0.0, -42.0, -24.0], [30.0, -42.0, 12.0, -30.0, 42.0], [0.0, 30.0, 12.0, -30.0, -36.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[28.0, 14.0, -42.0, -14.0, 0.0], [0.0, -49.0, 0.0, -49.0, -28.0], [35.0, -49.0, 14.0, -35.0, 49.0], [0.0, 35.0, 14.0, -35.0, -42.0]], [[28.0, 14.0, -42.0, -14.0, 0.0], [0.0, -49.0, 0.0, -49.0, -28.0], [35.0, -49.0, 14.0, -35.0, 49.0], [0.0, 35.0, 14.0, -35.0, -42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[[-4.0, -6.0, 5.0, -3.0, 0.0], [5.0, 5.0, 4.0, 0.0, 2.0], [-2.0, 1.0, -3.0, 5.0, 7.0], [-4.0, 5.0, 2.0, 3.0, -2.0]], [[-2.0, -5.0, 5.0, -1.0, -1.0], [-3.0, 0.0, -2.0, 1.0, 5.0], [0.0, 0.0, 2.0, 7.0, -4.0], [2.0, 0.0, 4.0, 7.0, 1.0]], [[5.0, 1.0, 2.0, 0.0, 4.0], [-5.0, 2.0, -5.0, -1.0, 0.0], [0.0, -2.0, 6.0, -3.0, 0.0], [-6.0, -5.0, 2.0, 2.0, 0.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[5.0, -5.0, 3.0, 1.0, -1.0], [2.0, -1.0, -2.0, 6.0, 4.0], [7.0, 4.0, -2.0, -4.0, 4.0], [7.0, 4.0, -2.0, -1.0, -6.0]], [[0.0, 6.0, -7.0, 6.0, 3.0], [7.0, -3.0, -2.0, 6.0, 6.0], [0.0, -7.0, 6.0, -6.0, -6.0], [-7.0, 4.0, 5.0, 0.0, 7.0]], [[7.0, -3.0, 7.0, 5.0, 4.0], [-1.0, 2.0, -1.0, 6.0, -2.0], [-1.0, 0.0, -2.0, 5.0, -2.0], [-2.0, -4.0, 5.0, 4.0, 5.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[15.0, -7.0, 37.0, 9.0, 18.0], [-27.0, 20.0, 7.0, -6.0, -38.0], [-33.0, -2.0, -14.0, 53.0, -14.0], [-24.0, -44.0, 23.0, 24.0, 35.0]], [[-23.0, -3.0, 24.0, -31.0, -5.0], [-48.0, 23.0, 21.0, -60.0, -56.0], [-43.0, 11.0, -20.0, 59.0, 4.0], [-9.0, -48.0, -8.0, 10.0, 6.0]], [[39.0, -1.0, -6.0, 45.0, 18.0], [43.0, -16.0, -22.0, 72.0, 46.0], [33.0, -15.0, 16.0, -40.0, -14.0], [-4.0, 32.0, 25.0, 3.0, 15.0]], [[-15.0, 9.0, -2.0, -9.0, 0.0], [-13.0, 6.0, 8.0, -24.0, -18.0], [-21.0, -5.0, 0.0, 18.0, -6.0], [-14.0, -16.0, 1.0, 3.0, 11.0]], [[28.0, -18.0, 35.0, 14.0, 13.0], [-11.0, 11.0, -2.0, 18.0, -14.0], [-4.0, 7.0, -14.0, 26.0, -2.0], [-1.0, -20.0, 15.0, 16.0, 13.0]]], [[[-10.0, -28.0, 1.0, -38.0, -34.0], [-6.0, -6.0, 1.0, -18.0, 12.0], [40.0, 41.0, -18.0, -27.0, 48.0], [66.0, 28.0, -50.0, -25.0, -76.0]], [[39.0, -31.0, 29.0, 15.0, 3.0], [8.0, -1.0, -12.0, 42.0, 16.0], [33.0, 20.0, -14.0, -10.0, 16.0], [31.0, 12.0, 0.0, 3.0, -20.0]], [[-15.0, -17.0, -9.0, -33.0, -30.0], [-1.0, -8.0, 1.0, -18.0, 14.0], [33.0, 30.0, -10.0, -29.0, 38.0], [52.0, 28.0, -43.0, -24.0, -63.0]], [[-7.0, 9.0, -14.0, 1.0, -1.0], [8.0, -5.0, -1.0, 0.0, 8.0], [1.0, -7.0, 8.0, -11.0, -4.0], [-5.0, 8.0, 0.0, -4.0, 2.0]], [[10.0, 20.0, -29.0, 32.0, 13.0], [39.0, -17.0, -14.0, 42.0, 38.0], [14.0, -27.0, 26.0, -38.0, -22.0], [-21.0, 28.0, 21.0, -2.0, 23.0]]], [[[-10.0, 10.0, -6.0, -2.0, 2.0], [-4.0, 2.0, 4.0, -12.0, -8.0], [-14.0, -8.0, 4.0, 8.0, -8.0], [-14.0, -8.0, 4.0, 2.0, 12.0]], [[-9.0, 1.0, -11.0, -9.0, -9.0], [4.0, -5.0, 0.0, -6.0, 8.0], [9.0, 4.0, 2.0, -14.0, 8.0], [11.0, 12.0, -12.0, -9.0, -16.0]], [[27.0, 9.0, 19.0, 39.0, 33.0], [2.0, 9.0, -4.0, 30.0, -12.0], [-27.0, -26.0, 6.0, 30.0, -36.0], [-47.0, -28.0, 46.0, 27.0, 62.0]], [[4.0, 26.0, -55.0, 32.0, 4.0], [62.0, -32.0, -21.0, 54.0, 68.0], [38.0, -29.0, 38.0, -77.0, -16.0], [-8.0, 60.0, 10.0, -17.0, 4.0]], [[35.0, -59.0, 49.0, -17.0, -19.0], [-14.0, 5.0, -6.0, 18.0, 4.0], [49.0, 56.0, -38.0, -4.0, 52.0], [77.0, 12.0, -34.0, -7.0, -70.0]]], [[[-62.0, 50.0, -68.0, -22.0, -14.0], [12.0, -14.0, 10.0, -48.0, 8.0], [-22.0, -30.0, 32.0, -26.0, -16.0], [-30.0, 16.0, -12.0, -20.0, 8.0]], [[-10.0, -10.0, -20.0, -20.0, -25.0], [15.0, -15.0, -5.0, 0.0, 30.0], [40.0, 20.0, 0.0, -45.0, 30.0], [45.0, 40.0, -35.0, -25.0, -55.0]], [[24.0, 8.0, -8.0, 36.0, 18.0], [30.0, -10.0, -14.0, 48.0, 28.0], [12.0, -20.0, 16.0, -22.0, -20.0], [-18.0, 16.0, 26.0, 6.0, 26.0]], [[29.0, 21.0, -26.0, 55.0, 26.0], [53.0, -20.0, -22.0, 72.0, 50.0], [19.0, -37.0, 32.0, -44.0, -34.0], [-32.0, 32.0, 39.0, 5.0, 41.0]], [[-10.0, 16.0, -13.0, 4.0, 5.0], [3.0, -1.0, 2.0, -6.0, -2.0], [-14.0, -15.0, 10.0, 2.0, -14.0], [-21.0, -4.0, 9.0, 2.0, 19.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-2.0, 3.0, -3.0, 5.0, -2.0], [7.0, 5.0, 1.0, 6.0, -6.0], [2.0, -7.0, 7.0, -6.0, 7.0], [7.0, 1.0, -1.0, -1.0, -1.0]], [[0.0, -7.0, 5.0, 5.0, -1.0], [4.0, 4.0, 6.0, -7.0, -4.0], [0.0, 0.0, 0.0, -3.0, 2.0], [4.0, -3.0, 5.0, 5.0, 0.0]], [[7.0, -3.0, 1.0, 5.0, -5.0], [-7.0, -5.0, -3.0, 1.0, 5.0], [-1.0, -3.0, 3.0, 7.0, -7.0], [-2.0, 0.0, 7.0, -6.0, 5.0]]], [[[3.0, 0.0, -7.0, -4.0, 7.0], [4.0, 7.0, -2.0, 2.0, -3.0], [-4.0, 6.0, -2.0, 0.0, 4.0], [-5.0, -6.0, 6.0, -6.0, -4.0]], [[-4.0, -4.0, -3.0, -1.0, 5.0], [-4.0, -6.0, 2.0, -5.0, 4.0], [4.0, -2.0, 2.0, 5.0, 3.0], [-7.0, 0.0, 5.0, 0.0, -5.0]], [[0.0, -5.0, 0.0, -3.0, -2.0], [-1.0, -2.0, -6.0, -5.0, -4.0], [4.0, -3.0, -1.0, 4.0, 3.0], [-6.0, 3.0, -5.0, 3.0, 5.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-7.0, -7.0, -5.0, -3.0, 0.0], [-2.0, 3.0, 1.0, -2.0, 7.0], [1.0, 4.0, -1.0, 4.0, -6.0], [7.0, 3.0, 5.0, -1.0, 5.0]], [[0.0, 0.0, -5.0, 1.0, 6.0], [2.0, -5.0, -4.0, -2.0, 5.0], [-3.0, 0.0, -2.0, 2.0, -5.0], [5.0, 5.0, 2.0, -4.0, 0.0]], [[0.0, -2.0, 4.0, -5.0, -6.0], [-7.0, 3.0, 0.0, 3.0, 4.0], [2.0, 3.0, -1.0, 6.0, -5.0], [-6.0, 1.0, -6.0, -1.0, -6.0]]], [[[-7.0, -1.0, 1.0, 6.0, -2.0], [5.0, 0.0, -2.0, 1.0, -2.0], [-6.0, -6.0, 0.0, -3.0, 7.0], [0.0, -6.0, 1.0, -2.0, -2.0]], [[6.0, -1.0, -5.0, 2.0, 0.0], [-3.0, 6.0, 5.0, 0.0, 7.0], [-1.0, -7.0, 6.0, 0.0, -5.0], [3.0, -4.0, -6.0, -4.0, 5.0]], [[6.0, 1.0, -4.0, -5.0, -2.0], [-6.0, 4.0, 3.0, -4.0, -7.0], [6.0, 0.0, 7.0, 3.0, -5.0], [-1.0, 2.0, -1.0, -5.0, -3.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-31.0, 1.0, 61.0, -19.0, -48.0], [-18.0, -9.0, -28.0, 28.0, -20.0], [-2.0, 23.0, -29.0, 25.0, 18.0], [-68.0, -1.0, -25.0, 5.0, -78.0]], [[-75.0, -16.0, 48.0, 16.0, -14.0], [43.0, -9.0, -4.0, 19.0, -19.0], [-8.0, 31.0, -45.0, -35.0, 77.0], [-3.0, -23.0, 48.0, 69.0, 28.0]], [[52.0, 29.0, 2.0, -39.0, 38.0], [-17.0, -49.0, -24.0, -8.0, 1.0], [29.0, 54.0, -26.0, 25.0, -46.0], [-11.0, 71.0, 0.0, 8.0, -22.0]], [[-31.0, -43.0, -17.0, -46.0, 14.0], [-34.0, -13.0, -21.0, 3.0, 102.0], [7.0, 66.0, -47.0, 63.0, -88.0], [30.0, 67.0, 10.0, -3.0, 7.0]], [[-17.0, 10.0, -15.0, 92.0, 14.0], [69.0, 6.0, 7.0, 6.0, -4.0], [-68.0, -100.0, 25.0, -67.0, 76.0], [28.0, -82.0, -3.0, -13.0, 37.0]]], [[[-107.0, -36.0, -55.0, 39.0, 60.0], [81.0, -48.0, -40.0, -35.0, 12.0], [-45.0, 11.0, -39.0, -21.0, 26.0], [100.0, 24.0, 114.0, -3.0, 52.0]], [[-132.0, -28.0, -20.0, 54.0, 44.0], [98.0, -64.0, -61.0, -18.0, -7.0], [-65.0, 5.0, -58.0, -29.0, 64.0], [69.0, 8.0, 108.0, 4.0, 17.0]], [[-17.0, -7.0, -35.0, 40.0, 70.0], [51.0, -48.0, -27.0, -1.0, 85.0], [-49.0, -7.0, -40.0, -14.0, -15.0], [67.0, 22.0, 27.0, 4.0, 55.0]], [[-116.0, -46.0, 56.0, -3.0, -42.0], [22.0, 6.0, -10.0, 27.0, 7.0], [-8.0, 50.0, -58.0, -5.0, 58.0], [-9.0, -18.0, 47.0, 62.0, 10.0]], [[63.0, 27.0, 63.0, -1.0, -40.0], [-34.0, 25.0, 24.0, 48.0, 20.0], [6.0, -19.0, 5.0, -5.0, 10.0], [-76.0, -39.0, -91.0, 27.0, -22.0]]], [[[62.0, -8.0, -54.0, -37.0, 6.0], [-53.0, 43.0, 42.0, -27.0, 18.0], [44.0, 1.0, 51.0, 26.0, -75.0], [28.0, 21.0, -16.0, -29.0, 32.0]], [[-23.0, 48.0, 51.0, 83.0, 12.0], [89.0, -54.0, -38.0, 23.0, -66.0], [-65.0, -59.0, -23.0, -73.0, 124.0], [-34.0, -58.0, 4.0, 21.0, -30.0]], [[-29.0, -56.0, -31.0, -39.0, -12.0], [-45.0, 38.0, 18.0, -3.0, 86.0], [17.0, 35.0, -5.0, 49.0, -76.0], [38.0, 26.0, 4.0, -9.0, 34.0]], [[96.0, 27.0, 32.0, -30.0, -68.0], [-82.0, 64.0, 43.0, 23.0, -22.0], [36.0, -38.0, 63.0, 24.0, -29.0], [-88.0, -38.0, -112.0, -29.0, -59.0]], [[-41.0, -39.0, -96.0, 31.0, 40.0], [32.0, 20.0, 15.0, -47.0, 23.0], [-22.0, -38.0, 35.0, -13.0, -19.0], [107.0, -6.0, 64.0, -43.0, 75.0]]], [[[-92.0, -39.0, -9.0, -21.0, 58.0], [40.0, -71.0, -52.0, -9.0, 64.0], [-8.0, 101.0, -97.0, 21.0, -22.0], [66.0, 85.0, 98.0, 47.0, 40.0]], [[53.0, 2.0, -8.0, -57.0, -12.0], [-56.0, 30.0, 34.0, -14.0, -17.0], [64.0, 40.0, 26.0, 25.0, -48.0], [-11.0, 30.0, -10.0, 8.0, 8.0]], [[-35.0, -23.0, 9.0, 44.0, -14.0], [8.0, 3.0, -23.0, 39.0, 104.0], [-73.0, -54.0, -21.0, 15.0, -12.0], [-4.0, -37.0, -56.0, -33.0, -19.0]], [[67.0, 28.0, -62.0, -13.0, 72.0], [6.0, -34.0, 0.0, -44.0, -15.0], [26.0, 14.0, 18.0, -3.0, -46.0], [51.0, 58.0, 32.0, -16.0, 34.0]], [[35.0, 11.0, 26.0, -81.0, -32.0], [-68.0, 2.0, -3.0, -7.0, -49.0], [68.0, 70.0, 1.0, 53.0, -47.0], [-57.0, 56.0, -14.0, -1.0, -67.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[-3.0, 4.0, -7.0, 1.0, 2.0], [4.0, 4.0, 4.0, 4.0, 3.0], [-2.0, 1.0, 0.0, -1.0, 1.0], [2.0, 3.0, -3.0, -4.0, 4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-0.0, 0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, -0.0, 0.0]])+(1.-msk_ref)*numpy.array([[6.0, -8.0, 14.0, -2.0, -4.0], [-8.0, -8.0, -8.0, -8.0, -6.0], [4.0, -2.0, -0.0, 2.0, -2.0], [-4.0, -6.0, 6.0, 8.0, -8.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[-6.0, 1.0, -4.0, -7.0, 7.0], [1.0, 2.0, 3.0, -3.0, 1.0], [7.0, 0.0, 0.0, 4.0, -5.0], [2.0, 6.0, -4.0, 1.0, -2.0]], [[4.0, 3.0, -6.0, 3.0, -3.0], [4.0, 7.0, -1.0, 7.0, 0.0], [-5.0, 3.0, 7.0, 0.0, 5.0], [0.0, 6.0, -1.0, 1.0, -1.0]], [[4.0, 6.0, -2.0, -5.0, -1.0], [6.0, -1.0, 2.0, -4.0, 2.0], [3.0, -7.0, 0.0, 2.0, -5.0], [1.0, 3.0, 6.0, 2.0, 6.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 7.0, -7.0])+(1.-msk_arg1)*numpy.array([0.0, -5.0, 7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-12.0, -19.0, -36.0, 42.0, 0.0], [-12.0, 60.0, -15.0, 71.0, -12.0], [-42.0, 70.0, 49.0, -6.0, 60.0], [-3.0, 33.0, -57.0, -5.0, -53.0]])+(1.-msk_ref)*numpy.array([[8.0, 27.0, 16.0, -50.0, 8.0], [22.0, -42.0, 19.0, -63.0, 14.0], [46.0, -64.0, -35.0, 14.0, -60.0], [7.0, -9.0, 47.0, 9.0, 47.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[2.0, 1.0, -1.0, 5.0, 7.0], [-6.0, -3.0, 6.0, -3.0, -2.0], [5.0, -2.0, -2.0, -7.0, -7.0], [-3.0, -5.0, -7.0, 6.0, 3.0]], [[1.0, 4.0, 7.0, -4.0, -5.0], [-6.0, -2.0, -3.0, -1.0, -6.0], [6.0, 7.0, 6.0, -4.0, 6.0], [-7.0, 6.0, 5.0, -6.0, 2.0]], [[-2.0, -3.0, -4.0, 3.0, -5.0], [-7.0, -2.0, 1.0, -4.0, -2.0], [4.0, 1.0, 3.0, -2.0, 0.0], [-6.0, -1.0, 6.0, 2.0, -1.0]]], [[[7.0, 1.0, -3.0, 0.0, 6.0], [-2.0, 3.0, -1.0, 5.0, 7.0], [2.0, 0.0, 5.0, -3.0, 6.0], [3.0, 6.0, 2.0, -1.0, 3.0]], [[-5.0, 1.0, 6.0, 2.0, 4.0], [-6.0, -7.0, 1.0, -1.0, 0.0], [-2.0, 2.0, 1.0, -2.0, -7.0], [3.0, 4.0, 3.0, 3.0, -3.0]], [[-1.0, 4.0, -7.0, 6.0, -3.0], [6.0, -7.0, 0.0, 4.0, -2.0], [-2.0, -1.0, -1.0, 3.0, -1.0], [-1.0, 1.0, -2.0, -1.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 0.0, 0.0], [-5.0, 2.0, 3.0]])+(1.-msk_arg1)*numpy.array([[3.0, -7.0, 5.0], [-1.0, -1.0, -1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-46.0, 10.0, 5.0, 27.0, -24.0], [10.0, -53.0, 13.0, -18.0, -43.0], [-15.0, -1.0, -28.0, 13.0, -54.0], [-15.0, -24.0, -17.0, 14.0, -18.0]])+(1.-msk_ref)*numpy.array([[-12.0, -46.0, -68.0, 50.0, 24.0], [-9.0, 6.0, 44.0, -30.0, 21.0], [-5.0, -51.0, -38.0, -1.0, -61.0], [5.0, -73.0, -29.0, 69.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[5.0, -6.0, -1.0, -6.0, -1.0], [-7.0, 2.0, 7.0, -7.0, 5.0], [-3.0, -7.0, -2.0, -3.0, 0.0], [2.0, -4.0, -3.0, -4.0, -5.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 3.0])+(1.-msk_arg1)*numpy.array([6.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-25.0, 15.0], [30.0, -18.0], [5.0, -3.0], [30.0, -18.0], [5.0, -3.0]], [[35.0, -21.0], [-10.0, 6.0], [-35.0, 21.0], [35.0, -21.0], [-25.0, 15.0]], [[15.0, -9.0], [35.0, -21.0], [10.0, -6.0], [15.0, -9.0], [0.0, 0.0]], [[-10.0, 6.0], [20.0, -12.0], [15.0, -9.0], [20.0, -12.0], [25.0, -15.0]]])+(1.-msk_ref)*numpy.array([[[30.0, 0.0], [-36.0, 0.0], [-6.0, 0.0], [-36.0, 0.0], [-6.0, 0.0]], [[-42.0, 0.0], [12.0, 0.0], [42.0, 0.0], [-42.0, 0.0], [30.0, 0.0]], [[-18.0, 0.0], [-42.0, 0.0], [-12.0, 0.0], [-18.0, 0.0], [0.0, 0.0]], [[12.0, 0.0], [-24.0, 0.0], [-18.0, 0.0], [-24.0, 0.0], [-30.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[-6.0, -4.0, 5.0, 6.0, -1.0], [-4.0, 6.0, 3.0, -7.0, 6.0], [-1.0, 0.0, -4.0, 2.0, 6.0], [5.0, 6.0, -3.0, -6.0, 3.0]], [[7.0, -4.0, 3.0, 0.0, 0.0], [5.0, 0.0, -6.0, -5.0, 3.0], [-1.0, -1.0, 0.0, -7.0, -2.0], [-5.0, -3.0, 7.0, 5.0, -2.0]], [[1.0, -1.0, 3.0, 1.0, -7.0], [2.0, -5.0, 4.0, -4.0, 7.0], [5.0, 3.0, 4.0, 0.0, 4.0], [-4.0, -3.0, 7.0, 1.0, 2.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 3.0], [-6.0, 5.0], [4.0, 5.0]])+(1.-msk_arg1)*numpy.array([[1.0, -6.0], [-5.0, 5.0], [-1.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-26.0, 22.0], [28.0, -37.0], [-16.0, 45.0], [-8.0, 23.0], [-26.0, -38.0]], [[-14.0, 23.0], [-32.0, -7.0], [46.0, -1.0], [28.0, -66.0], [-2.0, 68.0]], [[28.0, 17.0], [18.0, 10.0], [24.0, 8.0], [38.0, -29.0], [16.0, 28.0]], [[4.0, -30.0], [-6.0, -12.0], [-8.0, 61.0], [-14.0, 12.0], [14.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[-42.0, 64.0], [17.0, 11.0], [-13.0, -36.0], [5.0, -43.0], [6.0, 55.0]], [[-31.0, 35.0], [11.0, -1.0], [29.0, -76.0], [22.0, 45.0], [-16.0, -70.0]], [[-1.0, -34.0], [2.0, -26.0], [-8.0, -4.0], [37.0, -47.0], [12.0, -74.0]], [[34.0, -27.0], [24.0, -30.0], [-45.0, 4.0], [-32.0, 54.0], [11.0, -42.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[5.0, 3.0, -2.0, 0.0, 7.0], [2.0, 0.0, -5.0, -6.0, 5.0], [-4.0, -1.0, -2.0, -7.0, 0.0], [5.0, 0.0, -4.0, 7.0, 0.0]], [[5.0, 4.0, 6.0, 3.0, 7.0], [7.0, 7.0, -3.0, -4.0, 0.0], [6.0, -4.0, 4.0, -3.0, 6.0], [-4.0, -6.0, -3.0, -4.0, 6.0]], [[-5.0, 6.0, 2.0, 0.0, 2.0], [3.0, 0.0, 2.0, -5.0, -4.0], [-6.0, 5.0, -7.0, 1.0, 5.0], [-6.0, 2.0, -3.0, -6.0, -3.0]]], [[[0.0, -6.0, -2.0, -7.0, -6.0], [0.0, -5.0, -2.0, -5.0, 2.0], [6.0, 3.0, 3.0, 6.0, -2.0], [6.0, -6.0, -5.0, -6.0, 0.0]], [[0.0, -5.0, 4.0, 1.0, -5.0], [-4.0, 6.0, -4.0, 0.0, -1.0], [3.0, 0.0, -1.0, 7.0, -3.0], [2.0, -3.0, -4.0, 5.0, 3.0]], [[5.0, -5.0, 4.0, -6.0, 3.0], [1.0, 4.0, 6.0, 7.0, 7.0], [-7.0, -3.0, 5.0, 3.0, -2.0], [2.0, 0.0, 3.0, 0.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -6.0], [-3.0, 5.0], [0.0, 0.0]], [[6.0, 2.0], [4.0, 7.0], [-6.0, -1.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 1.0], [7.0, 7.0], [-7.0, 3.0]], [[4.0, -7.0], [1.0, -7.0], [4.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-45.0, -10.0], [-38.0, -40.0], [-38.0, 62.0], [-11.0, 14.0], [-95.0, -57.0]], [[-43.0, -6.0], [-51.0, 63.0], [-55.0, -23.0], [-60.0, -1.0], [-34.0, -40.0]], [[72.0, 94.0], [48.0, -5.0], [-28.0, 26.0], [55.0, 85.0], [-30.0, 7.0]], [[44.0, -26.0], [-30.0, -63.0], [-55.0, -32.0], [-4.0, -39.0], [-6.0, 51.0]]])+(1.-msk_ref)*numpy.array([[[75.0, 0.0], [-72.0, 151.0], [46.0, 12.0], [-30.0, 93.0], [-3.0, 124.0]], [[22.0, 83.0], [51.0, 22.0], [-8.0, -8.0], [33.0, -49.0], [48.0, -49.0]], [[95.0, -8.0], [-60.0, -20.0], [114.0, -34.0], [36.0, -131.0], [-12.0, 102.0]], [[33.0, -107.0], [-83.0, 27.0], [0.0, 14.0], [-26.0, -32.0], [66.0, 12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank2_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([[0.0, 3.0, 7.0, 4.0, 6.0], [0.0, -6.0, -4.0, -3.0, -4.0], [-2.0, 1.0, 1.0, -3.0, -6.0], [1.0, 0.0, -3.0, 1.0, 3.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 0.0, 2.0, -3.0, 5.0], [-2.0, 5.0, -5.0, -3.0, -7.0], [-6.0, -5.0, 0.0, -5.0, 6.0], [-6.0, 0.0, -1.0, -6.0, -2.0]])+(1.-msk_arg1)*numpy.array([[7.0, -7.0, 7.0, -5.0, -1.0], [-7.0, -5.0, 2.0, -6.0, 4.0], [6.0, -2.0, 1.0, 2.0, 0.0], [0.0, 5.0, -4.0, 6.0, 5.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-15.0, 0.0, 6.0, -9.0, 15.0], [-6.0, 15.0, -15.0, -9.0, -21.0], [-18.0, -15.0, 0.0, -15.0, 18.0], [-18.0, 0.0, -3.0, -18.0, -6.0]], [[-35.0, 0.0, 14.0, -21.0, 35.0], [-14.0, 35.0, -35.0, -21.0, -49.0], [-42.0, -35.0, 0.0, -35.0, 42.0], [-42.0, 0.0, -7.0, -42.0, -14.0]], [[-20.0, 0.0, 8.0, -12.0, 20.0], [-8.0, 20.0, -20.0, -12.0, -28.0], [-24.0, -20.0, 0.0, -20.0, 24.0], [-24.0, 0.0, -4.0, -24.0, -8.0]], [[-30.0, 0.0, 12.0, -18.0, 30.0], [-12.0, 30.0, -30.0, -18.0, -42.0], [-36.0, -30.0, 0.0, -30.0, 36.0], [-36.0, 0.0, -6.0, -36.0, -12.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[30.0, 0.0, -12.0, 18.0, -30.0], [12.0, -30.0, 30.0, 18.0, 42.0], [36.0, 30.0, 0.0, 30.0, -36.0], [36.0, 0.0, 6.0, 36.0, 12.0]], [[20.0, 0.0, -8.0, 12.0, -20.0], [8.0, -20.0, 20.0, 12.0, 28.0], [24.0, 20.0, 0.0, 20.0, -24.0], [24.0, 0.0, 4.0, 24.0, 8.0]], [[15.0, 0.0, -6.0, 9.0, -15.0], [6.0, -15.0, 15.0, 9.0, 21.0], [18.0, 15.0, 0.0, 15.0, -18.0], [18.0, 0.0, 3.0, 18.0, 6.0]], [[20.0, 0.0, -8.0, 12.0, -20.0], [8.0, -20.0, 20.0, 12.0, 28.0], [24.0, 20.0, 0.0, 20.0, -24.0], [24.0, 0.0, 4.0, 24.0, 8.0]]], [[[10.0, 0.0, -4.0, 6.0, -10.0], [4.0, -10.0, 10.0, 6.0, 14.0], [12.0, 10.0, 0.0, 10.0, -12.0], [12.0, 0.0, 2.0, 12.0, 4.0]], [[-5.0, 0.0, 2.0, -3.0, 5.0], [-2.0, 5.0, -5.0, -3.0, -7.0], [-6.0, -5.0, 0.0, -5.0, 6.0], [-6.0, 0.0, -1.0, -6.0, -2.0]], [[-5.0, 0.0, 2.0, -3.0, 5.0], [-2.0, 5.0, -5.0, -3.0, -7.0], [-6.0, -5.0, 0.0, -5.0, 6.0], [-6.0, 0.0, -1.0, -6.0, -2.0]], [[15.0, 0.0, -6.0, 9.0, -15.0], [6.0, -15.0, 15.0, 9.0, 21.0], [18.0, 15.0, 0.0, 15.0, -18.0], [18.0, 0.0, 3.0, 18.0, 6.0]], [[30.0, 0.0, -12.0, 18.0, -30.0], [12.0, -30.0, 30.0, 18.0, 42.0], [36.0, 30.0, 0.0, 30.0, -36.0], [36.0, 0.0, 6.0, 36.0, 12.0]]], [[[-5.0, 0.0, 2.0, -3.0, 5.0], [-2.0, 5.0, -5.0, -3.0, -7.0], [-6.0, -5.0, 0.0, -5.0, 6.0], [-6.0, 0.0, -1.0, -6.0, -2.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[15.0, 0.0, -6.0, 9.0, -15.0], [6.0, -15.0, 15.0, 9.0, 21.0], [18.0, 15.0, 0.0, 15.0, -18.0], [18.0, 0.0, 3.0, 18.0, 6.0]], [[-5.0, 0.0, 2.0, -3.0, 5.0], [-2.0, 5.0, -5.0, -3.0, -7.0], [-6.0, -5.0, 0.0, -5.0, 6.0], [-6.0, 0.0, -1.0, -6.0, -2.0]], [[-15.0, 0.0, 6.0, -9.0, 15.0], [-6.0, 15.0, -15.0, -9.0, -21.0], [-18.0, -15.0, 0.0, -15.0, 18.0], [-18.0, 0.0, -3.0, -18.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[21.0, -21.0, 21.0, -15.0, -3.0], [-21.0, -15.0, 6.0, -18.0, 12.0], [18.0, -6.0, 3.0, 6.0, 0.0], [0.0, 15.0, -12.0, 18.0, 15.0]], [[49.0, -49.0, 49.0, -35.0, -7.0], [-49.0, -35.0, 14.0, -42.0, 28.0], [42.0, -14.0, 7.0, 14.0, 0.0], [0.0, 35.0, -28.0, 42.0, 35.0]], [[28.0, -28.0, 28.0, -20.0, -4.0], [-28.0, -20.0, 8.0, -24.0, 16.0], [24.0, -8.0, 4.0, 8.0, 0.0], [0.0, 20.0, -16.0, 24.0, 20.0]], [[42.0, -42.0, 42.0, -30.0, -6.0], [-42.0, -30.0, 12.0, -36.0, 24.0], [36.0, -12.0, 6.0, 12.0, 0.0], [0.0, 30.0, -24.0, 36.0, 30.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-42.0, 42.0, -42.0, 30.0, 6.0], [42.0, 30.0, -12.0, 36.0, -24.0], [-36.0, 12.0, -6.0, -12.0, 0.0], [0.0, -30.0, 24.0, -36.0, -30.0]], [[-28.0, 28.0, -28.0, 20.0, 4.0], [28.0, 20.0, -8.0, 24.0, -16.0], [-24.0, 8.0, -4.0, -8.0, 0.0], [0.0, -20.0, 16.0, -24.0, -20.0]], [[-21.0, 21.0, -21.0, 15.0, 3.0], [21.0, 15.0, -6.0, 18.0, -12.0], [-18.0, 6.0, -3.0, -6.0, 0.0], [0.0, -15.0, 12.0, -18.0, -15.0]], [[-28.0, 28.0, -28.0, 20.0, 4.0], [28.0, 20.0, -8.0, 24.0, -16.0], [-24.0, 8.0, -4.0, -8.0, 0.0], [0.0, -20.0, 16.0, -24.0, -20.0]]], [[[-14.0, 14.0, -14.0, 10.0, 2.0], [14.0, 10.0, -4.0, 12.0, -8.0], [-12.0, 4.0, -2.0, -4.0, 0.0], [0.0, -10.0, 8.0, -12.0, -10.0]], [[7.0, -7.0, 7.0, -5.0, -1.0], [-7.0, -5.0, 2.0, -6.0, 4.0], [6.0, -2.0, 1.0, 2.0, 0.0], [0.0, 5.0, -4.0, 6.0, 5.0]], [[7.0, -7.0, 7.0, -5.0, -1.0], [-7.0, -5.0, 2.0, -6.0, 4.0], [6.0, -2.0, 1.0, 2.0, 0.0], [0.0, 5.0, -4.0, 6.0, 5.0]], [[-21.0, 21.0, -21.0, 15.0, 3.0], [21.0, 15.0, -6.0, 18.0, -12.0], [-18.0, 6.0, -3.0, -6.0, 0.0], [0.0, -15.0, 12.0, -18.0, -15.0]], [[-42.0, 42.0, -42.0, 30.0, 6.0], [42.0, 30.0, -12.0, 36.0, -24.0], [-36.0, 12.0, -6.0, -12.0, 0.0], [0.0, -30.0, 24.0, -36.0, -30.0]]], [[[7.0, -7.0, 7.0, -5.0, -1.0], [-7.0, -5.0, 2.0, -6.0, 4.0], [6.0, -2.0, 1.0, 2.0, 0.0], [0.0, 5.0, -4.0, 6.0, 5.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-21.0, 21.0, -21.0, 15.0, 3.0], [21.0, 15.0, -6.0, 18.0, -12.0], [-18.0, 6.0, -3.0, -6.0, 0.0], [0.0, -15.0, 12.0, -18.0, -15.0]], [[7.0, -7.0, 7.0, -5.0, -1.0], [-7.0, -5.0, 2.0, -6.0, 4.0], [6.0, -2.0, 1.0, 2.0, 0.0], [0.0, 5.0, -4.0, 6.0, 5.0]], [[21.0, -21.0, 21.0, -15.0, -3.0], [-21.0, -15.0, 6.0, -18.0, 12.0], [18.0, -6.0, 3.0, 6.0, 0.0], [0.0, 15.0, -12.0, 18.0, 15.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[[5.0, -6.0, 2.0, 4.0, -4.0], [-5.0, -3.0, 2.0, -5.0, 4.0], [0.0, -7.0, 4.0, -2.0, 7.0], [1.0, -7.0, -6.0, 5.0, -3.0]], [[6.0, -4.0, -6.0, 4.0, 5.0], [-2.0, 4.0, 6.0, 1.0, 7.0], [2.0, 7.0, 3.0, -6.0, 4.0], [3.0, -6.0, 0.0, 5.0, -1.0]], [[-3.0, -3.0, 7.0, 2.0, 0.0], [1.0, 0.0, -3.0, 1.0, 3.0], [-7.0, -5.0, -2.0, 3.0, 7.0], [6.0, 3.0, 3.0, 3.0, 2.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, -2.0, -2.0, 3.0, 6.0], [-2.0, -1.0, -4.0, 4.0, 1.0], [-2.0, 7.0, -4.0, 4.0, -1.0], [5.0, 4.0, 0.0, 4.0, -6.0]], [[-2.0, 4.0, 1.0, -3.0, 0.0], [4.0, -7.0, 1.0, 3.0, 2.0], [-1.0, 0.0, -4.0, -7.0, 0.0], [-6.0, 1.0, 3.0, 3.0, 0.0]], [[-6.0, 0.0, -5.0, 0.0, 5.0], [-3.0, 5.0, 1.0, 4.0, -7.0], [-7.0, -1.0, 7.0, -4.0, -6.0], [4.0, 0.0, 0.0, -6.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, -3.0, 5.0, -5.0, -7.0], [0.0, -4.0, 2.0, -1.0, 3.0], [-5.0, -3.0, 3.0, -4.0, 7.0], [-1.0, 5.0, 1.0, 5.0, 0.0]], [[5.0, 0.0, 0.0, 7.0, -2.0], [-5.0, 7.0, 5.0, 7.0, -1.0], [1.0, -6.0, -1.0, -3.0, -6.0], [0.0, -4.0, 2.0, 5.0, 4.0]], [[3.0, -5.0, 1.0, -3.0, -1.0], [-5.0, -4.0, -3.0, 6.0, 3.0], [-4.0, 4.0, -4.0, -1.0, 1.0], [6.0, 0.0, -3.0, -5.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[26.0, 14.0, 11.0, -3.0, 15.0], [23.0, -62.0, -17.0, 26.0, 38.0], [5.0, 38.0, -65.0, -10.0, 13.0], [-23.0, 26.0, 18.0, 56.0, -18.0]], [[2.0, -4.0, 23.0, -6.0, -51.0], [5.0, 19.0, 17.0, -48.0, 7.0], [37.0, -39.0, 19.0, 16.0, 24.0], [-18.0, -28.0, -12.0, -18.0, 48.0]], [[-22.0, -28.0, -45.0, 24.0, 47.0], [-49.0, 75.0, -7.0, 18.0, -59.0], [-47.0, 7.0, 65.0, 22.0, -44.0], [74.0, 2.0, -18.0, -52.0, -40.0]], [[-4.0, 8.0, -14.0, 0.0, 34.0], [2.0, -22.0, -10.0, 36.0, -2.0], [-26.0, 26.0, -18.0, -20.0, -16.0], [4.0, 20.0, 12.0, 16.0, -32.0]], [[-26.0, 28.0, 13.0, -27.0, -24.0], [28.0, -31.0, 21.0, -1.0, 6.0], [3.0, -28.0, -4.0, -51.0, 4.0], [-50.0, -11.0, 15.0, -1.0, 24.0]]], [[[-22.0, 2.0, 3.0, -9.0, -25.0], [-1.0, 24.0, 19.0, -22.0, -16.0], [5.0, -36.0, 35.0, -10.0, -1.0], [-9.0, -22.0, -6.0, -32.0, 26.0]], [[-20.0, 22.0, 10.0, -21.0, -18.0], [22.0, -25.0, 16.0, 0.0, 5.0], [2.0, -21.0, -4.0, -40.0, 3.0], [-39.0, -8.0, 12.0, 0.0, 18.0]], [[14.0, 20.0, 17.0, -12.0, -3.0], [29.0, -59.0, -5.0, 14.0, 35.0], [11.0, 17.0, -53.0, -22.0, 16.0], [-38.0, 14.0, 18.0, 44.0, 0.0]], [[-28.0, 14.0, 6.0, -18.0, -25.0], [11.0, 3.0, 22.0, -13.0, -10.0], [2.0, -36.0, 23.0, -31.0, -1.0], [-27.0, -19.0, 3.0, -23.0, 26.0]], [[-16.0, 20.0, -16.0, -9.0, 39.0], [11.0, -38.0, -6.0, 49.0, -3.0], [-36.0, 25.0, -23.0, -45.0, -22.0], [-10.0, 23.0, 21.0, 19.0, -36.0]]], [[[38.0, 8.0, 37.0, -6.0, -35.0], [29.0, -49.0, -5.0, -22.0, 53.0], [47.0, 7.0, -57.0, 14.0, 42.0], [-40.0, 2.0, 6.0, 48.0, 28.0]], [[-12.0, 42.0, 46.0, -42.0, -67.0], [57.0, -67.0, 30.0, -27.0, 42.0], [42.0, -44.0, -35.0, -57.0, 37.0], [-97.0, -21.0, 21.0, 23.0, 62.0]], [[22.0, 4.0, 5.0, 3.0, 14.0], [10.0, -35.0, -15.0, 17.0, 24.0], [3.0, 30.0, -42.0, 3.0, 8.0], [-6.0, 19.0, 9.0, 37.0, -16.0]], [[-14.0, -20.0, -17.0, 12.0, 3.0], [-29.0, 59.0, 5.0, -14.0, -35.0], [-11.0, -17.0, 53.0, 22.0, -16.0], [38.0, -14.0, -18.0, -44.0, 0.0]], [[-22.0, 2.0, -45.0, 9.0, 77.0], [-19.0, 0.0, -17.0, 68.0, -34.0], [-67.0, 42.0, 5.0, -28.0, -49.0], [39.0, 32.0, 12.0, -2.0, -70.0]]], [[[-38.0, 10.0, -29.0, -6.0, 36.0], [-8.0, 8.0, 5.0, 37.0, -35.0], [-47.0, 1.0, 26.0, -41.0, -37.0], [11.0, 7.0, 9.0, -23.0, -30.0]], [[-34.0, -10.0, -7.0, -3.0, -27.0], [-19.0, 64.0, 25.0, -34.0, -40.0], [-1.0, -52.0, 73.0, 2.0, -11.0], [13.0, -34.0, -18.0, -64.0, 30.0]], [[-42.0, 12.0, -3.0, -18.0, -21.0], [3.0, 21.0, 27.0, -12.0, -27.0], [-9.0, -45.0, 45.0, -36.0, -12.0], [-18.0, -24.0, 0.0, -42.0, 24.0]], [[-8.0, 10.0, -20.0, 0.0, 45.0], [1.0, -25.0, -12.0, 47.0, -6.0], [-36.0, 32.0, -19.0, -27.0, -23.0], [7.0, 25.0, 15.0, 17.0, -42.0]], [[-22.0, 2.0, -5.0, -6.0, -8.0], [-4.0, 20.0, 13.0, -7.0, -19.0], [-7.0, -23.0, 30.0, -13.0, -9.0], [-1.0, -13.0, -3.0, -27.0, 10.0]]]])+(1.-msk_ref)*numpy.array([[[[51.0, 0.0, 22.0, 26.0, -44.0], [-15.0, 34.0, 49.0, 19.0, 0.0], [-7.0, -63.0, 21.0, -35.0, -4.0], [-23.0, 1.0, 26.0, 70.0, 18.0]], [[-65.0, 33.0, -33.0, 11.0, 53.0], [35.0, 8.0, -23.0, -40.0, -23.0], [38.0, 30.0, -2.0, 39.0, -21.0], [-12.0, -14.0, -5.0, -35.0, -22.0]], [[3.0, -41.0, 17.0, -73.0, -9.0], [-5.0, -78.0, -47.0, -2.0, 33.0], [-44.0, 58.0, -16.0, 3.0, 57.0], [40.0, 34.0, -31.0, -55.0, -10.0]], [[50.0, -22.0, 22.0, 2.0, -38.0], [-30.0, 4.0, 22.0, 36.0, 14.0], [-24.0, -28.0, 0.0, -30.0, 6.0], [8.0, 4.0, 6.0, 30.0, 20.0]], [[1.0, 12.0, -20.0, 55.0, 18.0], [-25.0, 51.0, 17.0, 39.0, -17.0], [25.0, -18.0, -17.0, 1.0, -58.0], [4.0, -40.0, 6.0, 5.0, 20.0]]], [[[-37.0, 10.0, -24.0, 8.0, 38.0], [5.0, 2.0, -23.0, -3.0, -10.0], [19.0, 31.0, -17.0, 25.0, -22.0], [11.0, -17.0, -12.0, -40.0, -6.0]], [[2.0, 9.0, -15.0, 43.0, 13.0], [-20.0, 40.0, 14.0, 31.0, -13.0], [19.0, -15.0, -13.0, 0.0, -45.0], [3.0, -31.0, 5.0, 5.0, 16.0]], [[33.0, 9.0, 7.0, 41.0, -23.0], [-15.0, 46.0, 43.0, 22.0, -9.0], [8.0, -54.0, 12.0, -23.0, -25.0], [-20.0, -14.0, 23.0, 55.0, 18.0]], [[-22.0, 10.0, -24.0, 29.0, 32.0], [-10.0, 23.0, -8.0, 18.0, -13.0], [22.0, 13.0, -20.0, 16.0, -40.0], [11.0, -29.0, -6.0, -25.0, 6.0]], [[68.0, -27.0, 23.0, 20.0, -45.0], [-50.0, 21.0, 34.0, 63.0, 14.0], [-25.0, -42.0, -7.0, -40.0, -11.0], [14.0, -8.0, 9.0, 40.0, 34.0]]], [[[-11.0, 35.0, -7.0, 35.0, 3.0], [25.0, 42.0, 31.0, -28.0, -23.0], [30.0, -40.0, 26.0, 1.0, -19.0], [-42.0, -8.0, 25.0, 45.0, -6.0]], [[-22.0, 46.0, -40.0, 99.0, 40.0], [-10.0, 97.0, 36.0, 26.0, -43.0], [62.0, -41.0, -8.0, 12.0, -96.0], [-23.0, -63.0, 22.0, 25.0, 18.0]], [[33.0, -2.0, 18.0, 7.0, -32.0], [-5.0, 13.0, 29.0, 5.0, 3.0], [-9.0, -38.0, 17.0, -23.0, 8.0], [-16.0, 8.0, 16.0, 45.0, 8.0]], [[-33.0, -9.0, -7.0, -41.0, 23.0], [15.0, -46.0, -43.0, -22.0, 9.0], [-8.0, 54.0, -12.0, 23.0, 25.0], [20.0, 14.0, -23.0, -55.0, -18.0]], [[83.0, -56.0, 42.0, -28.0, -64.0], [-55.0, -28.0, 13.0, 63.0, 38.0], [-59.0, -17.0, -11.0, -47.0, 32.0], [35.0, 19.0, -6.0, 20.0, 30.0]]], [[[39.0, -33.0, 11.0, -2.0, -19.0], [-45.0, -7.0, -1.0, 56.0, 18.0], [-26.0, 3.0, -24.0, -19.0, -5.0], [35.0, -7.0, -11.0, -10.0, 24.0]], [[-63.0, 6.0, -32.0, -16.0, 58.0], [15.0, -26.0, -53.0, -17.0, -6.0], [17.0, 69.0, -27.0, 43.0, -10.0], [25.0, -11.0, -28.0, -80.0, -18.0]], [[-27.0, 3.0, -27.0, 21.0, 39.0], [-15.0, 12.0, -21.0, 24.0, -9.0], [18.0, 30.0, -30.0, 21.0, -39.0], [24.0, -30.0, -15.0, -45.0, 6.0]], [[64.0, -30.0, 28.0, 1.0, -48.0], [-40.0, 3.0, 26.0, 48.0, 19.0], [-32.0, -33.0, -2.0, -38.0, 8.0], [13.0, 5.0, 6.0, 35.0, 26.0]], [[-17.0, -1.0, -13.0, 2.0, 21.0], [-5.0, -3.0, -17.0, 8.0, -2.0], [6.0, 23.0, -16.0, 13.0, -13.0], [15.0, -11.0, -11.0, -30.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[6.0, -3.0, 3.0, -4.0, 6.0], [-2.0, -1.0, -7.0, 1.0, -2.0], [-5.0, 3.0, 3.0, 4.0, 7.0], [-2.0, 4.0, -3.0, -5.0, 6.0]], [[-5.0, 4.0, -2.0, 3.0, 3.0], [-6.0, 5.0, 6.0, -1.0, 5.0], [-1.0, -1.0, -3.0, -7.0, 0.0], [-5.0, 2.0, 0.0, 2.0, -6.0]], [[-1.0, -1.0, 1.0, -1.0, 1.0], [-6.0, 7.0, 6.0, 4.0, 6.0], [-1.0, -5.0, -3.0, -6.0, 2.0], [2.0, -3.0, 0.0, -7.0, -4.0]]], [[[-5.0, 0.0, 3.0, -1.0, -1.0], [-5.0, -3.0, -6.0, -6.0, 2.0], [5.0, 7.0, 1.0, 0.0, 0.0], [-3.0, 7.0, -4.0, 0.0, 0.0]], [[1.0, 2.0, 6.0, 6.0, 1.0], [5.0, 0.0, -3.0, 0.0, 3.0], [0.0, 3.0, 2.0, -6.0, 0.0], [-1.0, 0.0, -2.0, -3.0, 2.0]], [[7.0, -2.0, 0.0, -3.0, 4.0], [-2.0, 4.0, 6.0, 5.0, 1.0], [0.0, -3.0, -6.0, 1.0, -5.0], [2.0, -5.0, 0.0, 7.0, -4.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -2.0, -4.0, 3.0, -6.0], [-3.0, -1.0, -1.0, -1.0, -4.0], [7.0, -7.0, 3.0, 4.0, 0.0], [0.0, -4.0, -1.0, -6.0, 0.0]], [[-2.0, 2.0, 2.0, 0.0, -3.0], [3.0, -2.0, -7.0, -3.0, -6.0], [5.0, 1.0, 4.0, 7.0, 0.0], [-3.0, 3.0, -3.0, 5.0, -6.0]], [[1.0, 1.0, 0.0, -5.0, -1.0], [1.0, -3.0, -4.0, 0.0, 3.0], [3.0, -7.0, 0.0, -7.0, -6.0], [-6.0, -2.0, 2.0, 0.0, -3.0]]], [[[-5.0, -3.0, -6.0, -6.0, 0.0], [-4.0, -6.0, -7.0, 2.0, -1.0], [-5.0, 7.0, -7.0, 5.0, -7.0], [4.0, 6.0, -4.0, 4.0, 6.0]], [[3.0, -1.0, 0.0, 4.0, -6.0], [3.0, 0.0, -7.0, 5.0, 0.0], [-1.0, -5.0, 6.0, -1.0, -6.0], [-4.0, -1.0, -6.0, 1.0, 2.0]], [[3.0, 6.0, -3.0, 2.0, -6.0], [6.0, -2.0, -4.0, -3.0, -7.0], [-3.0, 7.0, 3.0, -3.0, -4.0], [4.0, -3.0, 1.0, 2.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, -5.0, -5.0, -5.0, 3.0], [-7.0, 0.0, 2.0, -3.0, -2.0], [3.0, -7.0, 2.0, 3.0, 5.0], [3.0, -2.0, -2.0, -1.0, -3.0]], [[5.0, 0.0, -7.0, 1.0, -3.0], [-7.0, 0.0, 6.0, -1.0, 4.0], [6.0, 6.0, -2.0, -6.0, -3.0], [-7.0, -3.0, 6.0, 0.0, 3.0]], [[6.0, 6.0, -2.0, 7.0, 7.0], [-1.0, -4.0, 4.0, 3.0, -4.0], [0.0, 3.0, -3.0, -6.0, -5.0], [-6.0, 4.0, -4.0, -7.0, -3.0]]], [[[-6.0, 2.0, -5.0, -6.0, -5.0], [0.0, 3.0, -1.0, 7.0, -7.0], [-6.0, 3.0, -4.0, 7.0, -1.0], [0.0, 7.0, 7.0, 7.0, -1.0]], [[4.0, 2.0, -1.0, -2.0, -1.0], [-1.0, -6.0, 4.0, 7.0, -1.0], [1.0, -1.0, -7.0, 1.0, 4.0], [-1.0, 1.0, 3.0, -5.0, -6.0]], [[-2.0, 6.0, 6.0, 1.0, 4.0], [1.0, -5.0, -6.0, -4.0, -4.0], [-2.0, -4.0, 1.0, 7.0, -4.0], [1.0, -7.0, 2.0, -1.0, 7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[64.0, 33.0, -25.0, 71.0, -68.0], [31.0, 23.0, 33.0, -17.0, -41.0], [17.0, -31.0, 60.0, -51.0, 7.0], [25.0, -89.0, 28.0, -66.0, 40.0]], [[-12.0, -1.0, 26.0, 0.0, 7.0], [14.0, 2.0, -27.0, 7.0, -1.0], [0.0, 8.0, 13.0, 27.0, 2.0], [-22.0, 30.0, -25.0, 36.0, -27.0]], [[11.0, -24.0, -34.0, 10.0, -49.0], [-8.0, -20.0, -56.0, 39.0, 0.0], [-7.0, -39.0, 16.0, 0.0, -63.0], [-12.0, -8.0, -43.0, -10.0, 39.0]], [[3.0, -8.0, 37.0, 17.0, -2.0], [24.0, 13.0, -36.0, 32.0, 17.0], [-8.0, -20.0, 34.0, 10.0, -11.0], [-43.0, 24.0, -42.0, 35.0, -24.0]], [[21.0, 21.0, -24.0, 31.0, -76.0], [23.0, -17.0, -47.0, -24.0, -66.0], [52.0, -30.0, 55.0, 20.0, -21.0], [-7.0, -36.0, -11.0, -16.0, -5.0]]], [[[38.0, -16.0, 32.0, 70.0, 18.0], [5.0, 66.0, 76.0, 41.0, 45.0], [-36.0, -24.0, 29.0, -32.0, 49.0], [6.0, -27.0, -4.0, -37.0, 24.0]], [[23.0, 52.0, 20.0, -12.0, -40.0], [61.0, -20.0, -57.0, -32.0, -30.0], [42.0, -30.0, 50.0, -45.0, -37.0], [-53.0, -25.0, 16.0, 27.0, -49.0]], [[26.0, 89.0, 58.0, -15.0, 0.0], [96.0, 1.0, -20.0, -56.0, -26.0], [14.0, 28.0, 45.0, -73.0, 0.0], [-42.0, -17.0, 49.0, 57.0, -66.0]], [[52.0, 48.0, 15.0, 29.0, -37.0], [52.0, 15.0, 12.0, -25.0, -15.0], [29.0, -43.0, 56.0, -76.0, -2.0], [-25.0, -66.0, 39.0, -25.0, -17.0]], [[-4.0, 17.0, 3.0, -34.0, -33.0], [34.0, -40.0, -96.0, 3.0, -13.0], [13.0, -17.0, 21.0, -11.0, -72.0], [-51.0, 17.0, -26.0, 50.0, -25.0]]], [[[-29.0, -8.0, -12.0, -40.0, 34.0], [-9.0, -20.0, -19.0, 18.0, 18.0], [-68.0, 76.0, -54.0, 5.0, -29.0], [29.0, 49.0, -14.0, 45.0, 39.0]], [[-35.0, -55.0, -47.0, -2.0, -10.0], [-54.0, -22.0, -34.0, 38.0, -7.0], [-28.0, 26.0, -35.0, 81.0, -25.0], [37.0, 43.0, -59.0, 2.0, 54.0]], [[-11.0, -56.0, -6.0, 14.0, 18.0], [-55.0, 18.0, 33.0, 36.0, 38.0], [8.0, -48.0, -16.0, 33.0, 23.0], [-1.0, 7.0, -22.0, -39.0, 7.0]], [[-3.0, -16.0, -33.0, 20.0, 33.0], [-51.0, 26.0, 107.0, -16.0, 1.0], [-22.0, 44.0, -49.0, 12.0, 68.0], [85.0, -22.0, 42.0, -63.0, 53.0]], [[-6.0, -42.0, -13.0, 1.0, -14.0], [-49.0, -3.0, 5.0, 8.0, 13.0], [70.0, -98.0, 6.0, 29.0, 8.0], [-32.0, -17.0, -8.0, -52.0, -31.0]]], [[[28.0, 18.0, 10.0, 2.0, 19.0], [14.0, 20.0, 49.0, 0.0, 33.0], [-23.0, -7.0, -5.0, -77.0, 7.0], [3.0, -34.0, 41.0, -22.0, 14.0]], [[-53.0, -58.0, -39.0, -25.0, 3.0], [-67.0, -31.0, -35.0, 19.0, -9.0], [9.0, 9.0, -44.0, 101.0, -11.0], [20.0, 53.0, -49.0, 4.0, 14.0]], [[11.0, 20.0, 36.0, 7.0, 30.0], [19.0, 27.0, 45.0, -15.0, 16.0], [1.0, 3.0, 7.0, -30.0, 40.0], [-8.0, -10.0, 31.0, 0.0, -28.0]], [[-4.0, 52.0, 3.0, 22.0, 7.0], [47.0, 8.0, 12.0, -37.0, -62.0], [-64.0, 150.0, -4.0, 25.0, 32.0], [76.0, 22.0, 10.0, 51.0, 38.0]], [[8.0, -54.0, -24.0, 38.0, -2.0], [-58.0, 26.0, 54.0, 34.0, 28.0], [10.0, -58.0, -6.0, 20.0, 28.0], [18.0, -24.0, -12.0, -72.0, 32.0]]]])+(1.-msk_ref)*numpy.array([[[[19.0, -2.0, 73.0, -7.0, 78.0], [0.0, -52.0, -55.0, -72.0, -22.0], [5.0, -119.0, 45.0, 69.0, 31.0], [65.0, -84.0, -56.0, -46.0, 18.0]], [[11.0, 1.0, -25.0, 6.0, -38.0], [-10.0, 2.0, 34.0, 24.0, 32.0], [21.0, 48.0, -27.0, -39.0, -6.0], [-35.0, 6.0, 36.0, 2.0, -2.0]], [[17.0, 9.0, -24.0, -40.0, 1.0], [-14.0, -31.0, 19.0, 59.0, -45.0], [-15.0, -27.0, -47.0, 42.0, 37.0], [11.0, 31.0, 17.0, -19.0, -57.0]], [[25.0, 6.0, -18.0, 7.0, -41.0], [-1.0, -20.0, 49.0, 53.0, 37.0], [24.0, 46.0, -52.0, -46.0, 13.0], [-36.0, 15.0, 35.0, -23.0, -32.0]], [[53.0, 0.0, -25.0, -12.0, 36.0], [-61.0, -33.0, 15.0, -34.0, -14.0], [35.0, -41.0, 4.0, 16.0, 5.0], [-6.0, -51.0, 6.0, -29.0, 11.0]]], [[[-22.0, -38.0, 72.0, -20.0, -18.0], [55.0, -11.0, -27.0, 2.0, 42.0], [-3.0, -52.0, 9.0, 22.0, 71.0], [65.0, -18.0, -32.0, -14.0, -33.0]], [[72.0, 65.0, -5.0, 81.0, 62.0], [-31.0, -57.0, 35.0, -18.0, -1.0], [37.0, 33.0, -17.0, -68.0, -68.0], [-76.0, -34.0, -9.0, -73.0, 28.0]], [[43.0, 89.0, 50.0, 131.0, 60.0], [10.0, -54.0, 4.0, -54.0, 35.0], [36.0, 64.0, 7.0, -96.0, -113.0], [-90.0, -67.0, -13.0, -68.0, 87.0]], [[50.0, 37.0, 54.0, 63.0, 84.0], [1.0, -59.0, -12.0, -52.0, 0.0], [23.0, -39.0, 21.0, -22.0, -26.0], [-9.0, -60.0, -56.0, -76.0, 23.0]], [[49.0, 62.0, -44.0, 40.0, 12.0], [-29.0, -41.0, 54.0, 50.0, -21.0], [13.0, 61.0, -60.0, -48.0, -49.0], [-79.0, 23.0, 35.0, -42.0, -10.0]]], [[[-66.0, 29.0, 9.0, -13.0, -44.0], [43.0, 19.0, -25.0, 48.0, -25.0], [-51.0, 41.0, -25.0, 32.0, -22.0], [-2.0, 44.0, 43.0, 47.0, 10.0]], [[-44.0, -43.0, -54.0, -102.0, -73.0], [-15.0, 38.0, 3.0, 59.0, -30.0], [-30.0, -12.0, -29.0, 76.0, 60.0], [40.0, 50.0, 60.0, 69.0, -43.0]], [[-4.0, -63.0, -31.0, -55.0, -34.0], [-5.0, 33.0, 19.0, 30.0, 9.0], [-1.0, -23.0, -3.0, 12.0, 70.0], [40.0, 42.0, -11.0, 21.0, -64.0]], [[-77.0, -62.0, 53.0, -56.0, 1.0], [34.0, 55.0, -88.0, -69.0, -10.0], [-38.0, -86.0, 83.0, 91.0, 43.0], [104.0, -24.0, -42.0, 67.0, 28.0]], [[57.0, -53.0, -69.0, -26.0, 15.0], [-56.0, 17.0, 52.0, 5.0, -2.0], [31.0, -23.0, 3.0, -26.0, 45.0], [4.0, 29.0, -32.0, -16.0, -62.0]]], [[[-13.0, 26.0, 69.0, 41.0, 47.0], [50.0, -21.0, -39.0, -19.0, -10.0], [-23.0, -26.0, 21.0, 4.0, -14.0], [20.0, -9.0, -54.0, -30.0, 8.0]], [[-20.0, -54.0, -93.0, -86.0, -70.0], [-44.0, 58.0, 31.0, 46.0, -17.0], [-8.0, 16.0, -20.0, 32.0, 42.0], [11.0, 58.0, 55.0, 71.0, -39.0]], [[1.0, 3.0, 37.0, 43.0, 13.0], [23.0, 0.0, -10.0, -33.0, 36.0], [13.0, 11.0, 24.0, -39.0, -19.0], [-7.0, -24.0, -28.0, -15.0, 25.0]], [[-83.0, 19.0, 70.0, -9.0, -39.0], [38.0, 11.0, -80.0, -57.0, 21.0], [-20.0, 1.0, 35.0, 61.0, -36.0], [23.0, -76.0, 55.0, 62.0, 109.0]], [[-8.0, -74.0, -6.0, -72.0, -10.0], [-2.0, 24.0, -8.0, 6.0, -6.0], [-8.0, -76.0, 18.0, 52.0, 92.0], [78.0, 20.0, -34.0, 16.0, -64.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[1.0, -3.0], [7.0, 5.0]], [[-2.0, -1.0], [0.0, 7.0]], [[4.0, 1.0], [3.0, 7.0]], [[-6.0, 7.0], [6.0, 2.0]], [[6.0, -2.0], [-6.0, 2.0]], [[1.0, 6.0], [4.0, -3.0]]]),self.functionspace)
      arg1=6.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[6.0, -18.0], [42.0, 30.0]], [[-12.0, -6.0], [0.0, 42.0]], [[24.0, 6.0], [18.0, 42.0]], [[-36.0, 42.0], [36.0, 12.0]], [[36.0, -12.0], [-36.0, 12.0]], [[6.0, 36.0], [24.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[7.0, 2.0], [7.0, -7.0]], [[-2.0, 6.0], [5.0, 5.0]], [[5.0, 4.0], [0.0, 5.0]], [[7.0, -2.0], [4.0, 5.0]], [[2.0, -3.0], [-3.0, 3.0]], [[6.0, -2.0], [0.0, 0.0]]]),self.functionspace)
      arg1=numpy.array(5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[35.0, 10.0], [35.0, -35.0]], [[-10.0, 30.0], [25.0, 25.0]], [[25.0, 20.0], [0.0, 25.0]], [[35.0, -10.0], [20.0, 25.0]], [[10.0, -15.0], [-15.0, 15.0]], [[30.0, -10.0], [0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[[-2.0, 0.0], [-6.0, -6.0]], [[5.0, -2.0], [-2.0, -6.0]], [[6.0, -1.0], [2.0, -3.0]], [[5.0, -3.0], [-3.0, -3.0]], [[4.0, -2.0], [7.0, 2.0]], [[1.0, 4.0], [0.0, 2.0]]], [[[0.0, -1.0], [7.0, -5.0]], [[-2.0, 1.0], [-3.0, -3.0]], [[-4.0, -1.0], [-5.0, 0.0]], [[2.0, -2.0], [4.0, 5.0]], [[4.0, 6.0], [7.0, -6.0]], [[1.0, 4.0], [-4.0, 5.0]]], [[[-3.0, 1.0], [1.0, 1.0]], [[-2.0, -3.0], [1.0, 2.0]], [[-2.0, 3.0], [4.0, -6.0]], [[-6.0, -5.0], [0.0, -6.0]], [[1.0, 6.0], [-1.0, 1.0]], [[-3.0, -5.0], [0.0, -6.0]]]]),self.functionspace)
      arg1=numpy.array([0.0, -3.0, 2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-6.0, 5.0], [-19.0, 17.0]], [[2.0, -9.0], [11.0, 13.0]], [[8.0, 9.0], [23.0, -12.0]], [[-18.0, -4.0], [-12.0, -27.0]], [[-10.0, -6.0], [-23.0, 20.0]], [[-9.0, -22.0], [12.0, -27.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_array_rank1_offset0(self):
      arg0=Data(numpy.array([[[-4.0, -3.0], [-5.0, 5.0]], [[3.0, 0.0], [-3.0, -5.0]], [[-6.0, -7.0], [1.0, 2.0]], [[5.0, -7.0], [3.0, -6.0]], [[-6.0, -1.0], [5.0, 3.0]], [[1.0, 7.0], [7.0, 6.0]]]),self.functionspace)
      arg1=numpy.array([6.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-24.0, 20.0], [-18.0, 15.0]], [[-30.0, 25.0], [30.0, -25.0]]], [[[18.0, -15.0], [0.0, 0.0]], [[-18.0, 15.0], [-30.0, 25.0]]], [[[-36.0, 30.0], [-42.0, 35.0]], [[6.0, -5.0], [12.0, -10.0]]], [[[30.0, -25.0], [-42.0, 35.0]], [[18.0, -15.0], [-36.0, 30.0]]], [[[-36.0, 30.0], [-6.0, 5.0]], [[30.0, -25.0], [18.0, -15.0]]], [[[6.0, -5.0], [42.0, -35.0]], [[42.0, -35.0], [36.0, -30.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[[4.0, 7.0], [7.0, -4.0]], [[0.0, 3.0], [-5.0, -5.0]], [[2.0, 6.0], [-6.0, -7.0]], [[4.0, -4.0], [2.0, 0.0]], [[-6.0, -7.0], [0.0, 5.0]], [[4.0, 6.0], [-2.0, 5.0]]], [[[-3.0, 3.0], [-4.0, 6.0]], [[4.0, 0.0], [-5.0, 1.0]], [[-3.0, 5.0], [1.0, -1.0]], [[-7.0, 2.0], [2.0, 7.0]], [[5.0, -5.0], [0.0, 2.0]], [[-6.0, 6.0], [7.0, -7.0]]], [[[-5.0, 0.0], [2.0, 5.0]], [[6.0, 3.0], [-7.0, -7.0]], [[-5.0, -1.0], [1.0, -6.0]], [[4.0, 5.0], [0.0, -7.0]], [[0.0, 5.0], [0.0, 0.0]], [[-4.0, -4.0], [4.0, 0.0]]]]),self.functionspace)
      arg1=numpy.array([[-2.0, -3.0], [-3.0, 5.0], [4.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-19.0, -57.0], [-23.0, -6.0]], [[6.0, -29.0], [10.0, 72.0]]], [[[12.0, 56.0], [6.0, 9.0]], [[-3.0, -52.0], [-21.0, -22.0]]], [[[-15.0, -51.0], [-31.0, 1.0]], [[13.0, 29.0], [-7.0, -20.0]]], [[[29.0, -23.0], [22.0, 52.0]], [[-10.0, 4.0], [-49.0, -7.0]]], [[[-3.0, 43.0], [49.0, 26.0]], [[0.0, 0.0], [-16.0, -5.0]]], [[[-6.0, -66.0], [-46.0, -12.0]], [[-1.0, 65.0], [11.0, -50.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[-5.0, 0.0], [5.0, -7.0]], [[3.0, -7.0], [-5.0, -6.0]], [[6.0, 1.0], [-6.0, 0.0]], [[6.0, 7.0], [-3.0, -2.0]], [[2.0, -4.0], [0.0, -3.0]], [[-5.0, -2.0], [-7.0, -2.0]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-5.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[25.0, -0.0], [-25.0, 35.0]], [[-15.0, 35.0], [25.0, 30.0]], [[-30.0, -5.0], [30.0, -0.0]], [[-30.0, -35.0], [15.0, 10.0]], [[-10.0, 20.0], [-0.0, 15.0]], [[25.0, 10.0], [35.0, 10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[[0.0, 4.0], [-2.0, 3.0]], [[0.0, -4.0], [-4.0, 3.0]], [[3.0, -2.0], [-6.0, 5.0]], [[7.0, -4.0], [4.0, -7.0]], [[3.0, 2.0], [0.0, -6.0]], [[-3.0, -6.0], [6.0, 7.0]]], [[[-1.0, -1.0], [4.0, -6.0]], [[5.0, 6.0], [0.0, -2.0]], [[4.0, 5.0], [-4.0, 4.0]], [[2.0, -5.0], [-7.0, -6.0]], [[-3.0, 6.0], [7.0, 0.0]], [[4.0, 0.0], [2.0, 1.0]]], [[[-3.0, -4.0], [-6.0, 5.0]], [[-6.0, 7.0], [-6.0, 5.0]], [[6.0, -2.0], [-5.0, 1.0]], [[-4.0, 1.0], [6.0, 1.0]], [[7.0, 3.0], [5.0, 1.0]], [[4.0, 0.0], [0.0, 7.0]]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([6.0, 0.0, -7.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[21.0, 52.0], [30.0, -17.0]], [[42.0, -73.0], [18.0, -17.0]], [[-24.0, 2.0], [-1.0, 23.0]], [[70.0, -31.0], [-18.0, -49.0]], [[-31.0, -9.0], [-35.0, -43.0]], [[-46.0, -36.0], [36.0, -7.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[[5.0, 4.0], [4.0, 7.0]], [[-5.0, 0.0], [3.0, -5.0]], [[6.0, 1.0], [5.0, 0.0]], [[7.0, 4.0], [7.0, -1.0]], [[6.0, 5.0], [-3.0, 2.0]], [[0.0, 5.0], [6.0, 6.0]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([1.0, 7.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[5.0, 35.0], [4.0, 28.0]], [[4.0, 28.0], [7.0, 49.0]]], [[[-5.0, -35.0], [0.0, 0.0]], [[3.0, 21.0], [-5.0, -35.0]]], [[[6.0, 42.0], [1.0, 7.0]], [[5.0, 35.0], [0.0, 0.0]]], [[[7.0, 49.0], [4.0, 28.0]], [[7.0, 49.0], [-1.0, -7.0]]], [[[6.0, 42.0], [5.0, 35.0]], [[-3.0, -21.0], [2.0, 14.0]]], [[[0.0, 0.0], [5.0, 35.0]], [[6.0, 42.0], [6.0, 42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-3.0, 0.0], [4.0, 6.0]], [[1.0, 5.0], [2.0, 0.0]], [[7.0, 7.0], [-7.0, 4.0]], [[-6.0, -2.0], [-1.0, 3.0]], [[-1.0, -2.0], [-6.0, -5.0]], [[4.0, -5.0], [-7.0, -7.0]]], [[[2.0, 6.0], [-1.0, 0.0]], [[3.0, 0.0], [0.0, 5.0]], [[3.0, 6.0], [-3.0, -5.0]], [[2.0, 4.0], [7.0, 5.0]], [[3.0, -5.0], [0.0, 2.0]], [[-3.0, 5.0], [5.0, 0.0]]], [[[-6.0, 0.0], [-2.0, 5.0]], [[0.0, -7.0], [6.0, 3.0]], [[-4.0, 1.0], [-7.0, -7.0]], [[-5.0, 5.0], [1.0, 5.0]], [[-5.0, -7.0], [3.0, -4.0]], [[-2.0, 6.0], [0.0, 3.0]]]]),self.functionspace)
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[4.0, -1.0], [-1.0, 4.0], [-5.0, -5.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[16.0, 41.0], [-6.0, 24.0]], [[27.0, 2.0], [-1.0, -31.0]]], [[[1.0, 11.0], [55.0, 30.0]], [[-22.0, -32.0], [-20.0, 5.0]]], [[[45.0, 25.0], [17.0, 12.0]], [[10.0, 30.0], [56.0, 11.0]]], [[[-1.0, 39.0], [-37.0, -7.0]], [[-16.0, 24.0], [-18.0, -8.0]]], [[[18.0, 38.0], [32.0, 17.0]], [[-39.0, -9.0], [-2.0, 33.0]]], [[[29.0, -6.0], [-55.0, -5.0]], [[-33.0, 27.0], [-43.0, -8.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[5.0, 3.0], [4.0, 6.0]], [[-2.0, 6.0], [-6.0, -1.0]], [[6.0, -2.0], [-4.0, 4.0]], [[0.0, 3.0], [5.0, 6.0]], [[0.0, 0.0], [-6.0, 0.0]], [[-3.0, -5.0], [-7.0, 2.0]]]),self.functionspace)
      arg1=Data(-2.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-10.0, -6.0], [-8.0, -12.0]], [[4.0, -12.0], [12.0, 2.0]], [[-12.0, 4.0], [8.0, -8.0]], [[-0.0, -6.0], [-10.0, -12.0]], [[-0.0, -0.0], [12.0, -0.0]], [[6.0, 10.0], [14.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[0.0, 7.0], [-1.0, -4.0]], [[-7.0, -5.0], [-7.0, 6.0]], [[0.0, -7.0], [-2.0, -7.0]], [[-1.0, 5.0], [-7.0, 4.0]], [[4.0, 0.0], [7.0, -4.0]], [[-2.0, -2.0], [-3.0, 2.0]]], [[[0.0, -6.0], [-2.0, -2.0]], [[-7.0, 5.0], [4.0, 6.0]], [[-6.0, -6.0], [0.0, -6.0]], [[6.0, 0.0], [4.0, 7.0]], [[-2.0, -2.0], [-3.0, 0.0]], [[-6.0, -1.0], [5.0, -1.0]]], [[[7.0, 7.0], [5.0, 4.0]], [[5.0, -2.0], [3.0, 5.0]], [[4.0, -2.0], [0.0, -2.0]], [[-5.0, -1.0], [-7.0, 0.0]], [[4.0, -5.0], [5.0, -6.0]], [[2.0, 7.0], [0.0, -6.0]]]]),self.functionspace)
      arg1=Data(numpy.array([7.0, 0.0, 0.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[0.0, 49.0], [-7.0, -28.0]], [[-49.0, -35.0], [-49.0, 42.0]], [[0.0, -49.0], [-14.0, -49.0]], [[-7.0, 35.0], [-49.0, 28.0]], [[28.0, 0.0], [49.0, -28.0]], [[-14.0, -14.0], [-21.0, 14.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[[3.0, 5.0], [0.0, 4.0]], [[0.0, -6.0], [-5.0, -1.0]], [[-3.0, 2.0], [-1.0, 4.0]], [[-5.0, 7.0], [3.0, 1.0]], [[4.0, -4.0], [-2.0, 5.0]], [[-5.0, 2.0], [1.0, 3.0]]]),self.functionspace)
      arg1=Data(numpy.array([-1.0, -4.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-3.0, -12.0], [-5.0, -20.0]], [[0.0, 0.0], [-4.0, -16.0]]], [[[0.0, 0.0], [6.0, 24.0]], [[5.0, 20.0], [1.0, 4.0]]], [[[3.0, 12.0], [-2.0, -8.0]], [[1.0, 4.0], [-4.0, -16.0]]], [[[5.0, 20.0], [-7.0, -28.0]], [[-3.0, -12.0], [-1.0, -4.0]]], [[[-4.0, -16.0], [4.0, 16.0]], [[2.0, 8.0], [-5.0, -20.0]]], [[[5.0, 20.0], [-2.0, -8.0]], [[-1.0, -4.0], [-3.0, -12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-5.0, 1.0], [3.0, -1.0]], [[2.0, -3.0], [6.0, 7.0]], [[0.0, -1.0], [-2.0, -3.0]], [[2.0, -2.0], [-5.0, -3.0]], [[7.0, 4.0], [-1.0, -2.0]], [[-3.0, -5.0], [4.0, -7.0]]], [[[1.0, -3.0], [1.0, 2.0]], [[-5.0, -3.0], [-2.0, -1.0]], [[0.0, -4.0], [-7.0, 3.0]], [[-4.0, -7.0], [-3.0, -5.0]], [[-7.0, 7.0], [0.0, 2.0]], [[-5.0, 4.0], [7.0, -7.0]]], [[[0.0, 1.0], [4.0, -2.0]], [[6.0, 6.0], [-4.0, -6.0]], [[-6.0, -6.0], [-3.0, 3.0]], [[-7.0, -4.0], [-3.0, 0.0]], [[4.0, -2.0], [-5.0, 3.0]], [[-4.0, 4.0], [-3.0, 4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[6.0, 0.0], [1.0, 0.0], [2.0, -6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-29.0, 0.0], [5.0, -6.0]], [[27.0, -24.0], [-8.0, 12.0]]], [[[19.0, -36.0], [-9.0, -36.0]], [[26.0, 24.0], [29.0, 36.0]]], [[[-12.0, 36.0], [-22.0, 36.0]], [[-25.0, 18.0], [-9.0, -18.0]]], [[[-6.0, 42.0], [-27.0, 24.0]], [[-39.0, 18.0], [-23.0, 0.0]]], [[[43.0, -24.0], [27.0, 12.0]], [[-16.0, 30.0], [-4.0, -18.0]]], [[[-31.0, 24.0], [-18.0, -24.0]], [[25.0, 18.0], [-41.0, -24.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[-3.0, 2.0], [-5.0, -1.0]], [[3.0, 3.0], [5.0, 0.0]], [[3.0, -5.0], [-1.0, 6.0]], [[0.0, 0.0], [2.0, 7.0]], [[2.0, 1.0], [7.0, 6.0]], [[-6.0, -7.0], [-7.0, 4.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, -6.0], [15.0, 3.0]], [[-9.0, -9.0], [-15.0, -0.0]], [[-9.0, 15.0], [3.0, -18.0]], [[-0.0, -0.0], [-6.0, -21.0]], [[-6.0, -3.0], [-21.0, -18.0]], [[18.0, 21.0], [21.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[6.0, -4.0], [10.0, 2.0]], [[-6.0, -6.0], [-10.0, -0.0]], [[-6.0, 10.0], [2.0, -12.0]], [[-0.0, -0.0], [-4.0, -14.0]], [[-4.0, -2.0], [-14.0, -12.0]], [[12.0, 14.0], [14.0, -8.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[0.0, 1.0], [-5.0, -5.0]], [[-7.0, 5.0], [-3.0, 7.0]], [[0.0, -6.0], [6.0, 2.0]], [[-6.0, 3.0], [-7.0, 4.0]], [[-3.0, 3.0], [-5.0, 6.0]], [[-5.0, 2.0], [2.0, -5.0]]], [[[0.0, 1.0], [0.0, -1.0]], [[-2.0, 2.0], [-7.0, 5.0]], [[-2.0, 4.0], [0.0, 2.0]], [[-4.0, 0.0], [1.0, 4.0]], [[3.0, -7.0], [4.0, 3.0]], [[7.0, -4.0], [-3.0, -2.0]]], [[[0.0, -2.0], [-2.0, 3.0]], [[0.0, 0.0], [1.0, -2.0]], [[6.0, 1.0], [-2.0, 7.0]], [[0.0, 0.0], [3.0, -5.0]], [[-3.0, 1.0], [-2.0, -2.0]], [[4.0, 2.0], [-1.0, 1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, 1.0, -3.0])+(1.-msk_arg1)*numpy.array([-3.0, -5.0, -4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 1.0], [36.0, 20.0]], [[40.0, -28.0], [8.0, -31.0]], [[-20.0, 37.0], [-30.0, -31.0]], [[32.0, -18.0], [34.0, -5.0]], [[30.0, -28.0], [40.0, -27.0]], [[25.0, -22.0], [-12.0, 25.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0], [23.0, 8.0]], [[31.0, -25.0], [40.0, -38.0]], [[-14.0, -6.0], [-10.0, -44.0]], [[38.0, -9.0], [4.0, -12.0]], [[6.0, 22.0], [3.0, -25.0]], [[-36.0, 6.0], [13.0, 21.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank3_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[[2.0, -5.0], [-2.0, 4.0]], [[3.0, -7.0], [-4.0, -7.0]], [[-7.0, -5.0], [3.0, 3.0]], [[0.0, -6.0], [4.0, 4.0]], [[-6.0, 3.0], [-1.0, -4.0]], [[5.0, -6.0], [3.0, 4.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 4.0])+(1.-msk_arg1)*numpy.array([2.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 8.0], [-10.0, -20.0]], [[-4.0, -8.0], [8.0, 16.0]]], [[[6.0, 12.0], [-14.0, -28.0]], [[-8.0, -16.0], [-14.0, -28.0]]], [[[-14.0, -28.0], [-10.0, -20.0]], [[6.0, 12.0], [6.0, 12.0]]], [[[0.0, 0.0], [-12.0, -24.0]], [[8.0, 16.0], [8.0, 16.0]]], [[[-12.0, -24.0], [6.0, 12.0]], [[-2.0, -4.0], [-8.0, -16.0]]], [[[10.0, 20.0], [-12.0, -24.0]], [[6.0, 12.0], [8.0, 16.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 12.0], [-10.0, -30.0]], [[-4.0, -12.0], [8.0, 24.0]]], [[[6.0, 18.0], [-14.0, -42.0]], [[-8.0, -24.0], [-14.0, -42.0]]], [[[-14.0, -42.0], [-10.0, -30.0]], [[6.0, 18.0], [6.0, 18.0]]], [[[0.0, 0.0], [-12.0, -36.0]], [[8.0, 24.0], [8.0, 24.0]]], [[[-12.0, -36.0], [6.0, 18.0]], [[-2.0, -6.0], [-8.0, -24.0]]], [[[10.0, 30.0], [-12.0, -36.0]], [[6.0, 18.0], [8.0, 24.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-2.0, -1.0], [4.0, 3.0]], [[3.0, 6.0], [0.0, -7.0]], [[-4.0, -3.0], [-5.0, 5.0]], [[6.0, -4.0], [0.0, -5.0]], [[-1.0, -6.0], [5.0, -7.0]], [[0.0, -3.0], [2.0, 7.0]]], [[[-2.0, 6.0], [1.0, -5.0]], [[-1.0, 3.0], [2.0, 0.0]], [[-4.0, -3.0], [-3.0, -3.0]], [[4.0, 0.0], [7.0, -2.0]], [[4.0, 7.0], [7.0, -6.0]], [[1.0, -2.0], [1.0, -4.0]]], [[[-1.0, -4.0], [6.0, 1.0]], [[2.0, -5.0], [3.0, -1.0]], [[0.0, -2.0], [0.0, 0.0]], [[-2.0, 1.0], [3.0, 0.0]], [[0.0, 2.0], [-2.0, 2.0]], [[-1.0, 5.0], [-4.0, 1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 1.0], [-1.0, 4.0], [-4.0, 4.0]])+(1.-msk_arg1)*numpy.array([[3.0, 4.0], [-1.0, 5.0], [-4.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, -14.0], [10.0, 7.0]], [[-25.0, 32.0], [1.0, -13.0]]], [[[-7.0, 7.0], [17.0, -2.0]], [[-14.0, 20.0], [4.0, -11.0]]], [[[4.0, -20.0], [11.0, -23.0]], [[3.0, -17.0], [3.0, -7.0]]], [[[4.0, 14.0], [-4.0, 0.0]], [[-19.0, 40.0], [2.0, -13.0]]], [[[-4.0, 15.0], [-15.0, 30.0]], [[1.0, 25.0], [-2.0, -23.0]]], [[[3.0, 0.0], [-18.0, 9.0]], [[15.0, -10.0], [0.0, -5.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -22.0], [7.0, 10.0]], [[-13.0, 45.0], [10.0, -9.0]]], [[[2.0, 15.0], [35.0, 19.0]], [[-14.0, 22.0], [-17.0, -32.0]]], [[[-8.0, -36.0], [2.0, -35.0]], [[-12.0, -35.0], [18.0, 5.0]]], [[[22.0, 36.0], [-16.0, -12.0]], [[-19.0, 47.0], [-13.0, -30.0]]], [[[-7.0, 16.0], [-33.0, 19.0]], [[16.0, 47.0], [-23.0, -50.0]]], [[[3.0, 1.0], [-27.0, -2.0]], [[21.0, -3.0], [21.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-3.0, 2.0, 5.0, -6.0], [5.0, -2.0, 7.0, -2.0], [4.0, -4.0, 4.0, 0.0]], [[4.0, -1.0, 2.0, -3.0], [-6.0, 2.0, 0.0, 5.0], [-7.0, 1.0, 3.0, -1.0]]], [[[-1.0, 7.0, 0.0, 5.0], [1.0, 6.0, 3.0, 0.0], [5.0, 0.0, -5.0, -4.0]], [[7.0, -5.0, -3.0, 0.0], [3.0, -2.0, 0.0, 7.0], [-6.0, 2.0, -2.0, 1.0]]], [[[0.0, -2.0, 0.0, -5.0], [-2.0, 1.0, 4.0, -5.0], [0.0, 4.0, -5.0, 0.0]], [[0.0, 0.0, 6.0, 2.0], [-6.0, -5.0, 3.0, 2.0], [3.0, 6.0, 4.0, -4.0]]]]),self.functionspace)
      arg1=-4.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[12.0, -8.0, -20.0, 24.0], [-20.0, 8.0, -28.0, 8.0], [-16.0, 16.0, -16.0, -0.0]], [[-16.0, 4.0, -8.0, 12.0], [24.0, -8.0, -0.0, -20.0], [28.0, -4.0, -12.0, 4.0]]], [[[4.0, -28.0, -0.0, -20.0], [-4.0, -24.0, -12.0, -0.0], [-20.0, -0.0, 20.0, 16.0]], [[-28.0, 20.0, 12.0, -0.0], [-12.0, 8.0, -0.0, -28.0], [24.0, -8.0, 8.0, -4.0]]], [[[-0.0, 8.0, -0.0, 20.0], [8.0, -4.0, -16.0, 20.0], [-0.0, -16.0, 20.0, -0.0]], [[-0.0, -0.0, -24.0, -8.0], [24.0, 20.0, -12.0, -8.0], [-12.0, -24.0, -16.0, 16.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[[7.0, 0.0, 2.0, 4.0], [-4.0, 6.0, -3.0, -7.0], [1.0, 0.0, -6.0, -2.0]], [[-6.0, 1.0, 7.0, 4.0], [4.0, -7.0, -2.0, -7.0], [0.0, 2.0, 6.0, -3.0]]], [[[1.0, 7.0, 7.0, -6.0], [6.0, -4.0, 2.0, 4.0], [-7.0, -3.0, -6.0, -7.0]], [[2.0, 5.0, -5.0, 0.0], [3.0, 7.0, 0.0, 1.0], [-5.0, 1.0, 4.0, 6.0]]], [[[-1.0, 4.0, 1.0, 3.0], [-6.0, -1.0, -4.0, 0.0], [7.0, -3.0, 4.0, -4.0]], [[6.0, 7.0, 0.0, 4.0], [-4.0, 7.0, 6.0, -3.0], [-1.0, -2.0, -2.0, -7.0]]]]),self.functionspace)
      arg1=numpy.array(-4.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-28.0, -0.0, -8.0, -16.0], [16.0, -24.0, 12.0, 28.0], [-4.0, -0.0, 24.0, 8.0]], [[24.0, -4.0, -28.0, -16.0], [-16.0, 28.0, 8.0, 28.0], [-0.0, -8.0, -24.0, 12.0]]], [[[-4.0, -28.0, -28.0, 24.0], [-24.0, 16.0, -8.0, -16.0], [28.0, 12.0, 24.0, 28.0]], [[-8.0, -20.0, 20.0, -0.0], [-12.0, -28.0, -0.0, -4.0], [20.0, -4.0, -16.0, -24.0]]], [[[4.0, -16.0, -4.0, -12.0], [24.0, 4.0, 16.0, -0.0], [-28.0, 12.0, -16.0, 16.0]], [[-24.0, -28.0, -0.0, -16.0], [16.0, -28.0, -24.0, 12.0], [4.0, 8.0, 8.0, 28.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-3.0, -3.0, -6.0, 0.0], [-4.0, -2.0, 1.0, -6.0], [3.0, 0.0, -2.0, -3.0]], [[2.0, -7.0, -3.0, 2.0], [-1.0, 3.0, -4.0, -4.0], [-6.0, -3.0, 5.0, -4.0]]], [[[6.0, -2.0, -6.0, 7.0], [2.0, 0.0, 1.0, -5.0], [-1.0, -4.0, 5.0, 2.0]], [[0.0, 0.0, 1.0, -6.0], [1.0, -2.0, 0.0, -7.0], [6.0, -6.0, 7.0, 0.0]]], [[[0.0, -2.0, -4.0, 0.0], [1.0, 7.0, -6.0, 1.0], [2.0, 0.0, 1.0, 7.0]], [[3.0, -3.0, 4.0, 4.0], [1.0, 1.0, -6.0, -7.0], [5.0, 0.0, 1.0, 0.0]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-2.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[6.0, 6.0, 12.0, -0.0], [8.0, 4.0, -2.0, 12.0], [-6.0, -0.0, 4.0, 6.0]], [[-4.0, 14.0, 6.0, -4.0], [2.0, -6.0, 8.0, 8.0], [12.0, 6.0, -10.0, 8.0]]], [[[-12.0, 4.0, 12.0, -14.0], [-4.0, -0.0, -2.0, 10.0], [2.0, 8.0, -10.0, -4.0]], [[-0.0, -0.0, -2.0, 12.0], [-2.0, 4.0, -0.0, 14.0], [-12.0, 12.0, -14.0, -0.0]]], [[[-0.0, 4.0, 8.0, -0.0], [-2.0, -14.0, 12.0, -2.0], [-4.0, -0.0, -2.0, -14.0]], [[-6.0, 6.0, -8.0, -8.0], [-2.0, -2.0, 12.0, 14.0], [-10.0, -0.0, -2.0, -0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[7.0, -7.0, 7.0, 4.0], [-2.0, 7.0, -3.0, -7.0], [3.0, 4.0, 7.0, -4.0]], [[7.0, -1.0, 4.0, -4.0], [-3.0, -2.0, 4.0, -7.0], [-6.0, -7.0, -5.0, -1.0]]], [[[6.0, 7.0, -2.0, -6.0], [0.0, 6.0, -6.0, 3.0], [5.0, 1.0, 3.0, -2.0]], [[-2.0, 3.0, 6.0, 0.0], [7.0, -6.0, -6.0, 4.0], [-5.0, 4.0, -5.0, 7.0]]], [[[-2.0, 2.0, -6.0, 4.0], [0.0, -6.0, 0.0, 0.0], [0.0, -7.0, 1.0, 0.0]], [[1.0, 3.0, -6.0, -2.0], [-3.0, -1.0, 6.0, -4.0], [-6.0, -3.0, 0.0, 4.0]]]]),self.functionspace)
      arg1=Data(0.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, -0.0]], [[0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, -0.0]]], [[[0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, 0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_constData_rank4_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-6.0, -1.0, -7.0, 0.0], [-3.0, -6.0, -1.0, -2.0], [-3.0, -6.0, 0.0, 0.0]], [[-2.0, -7.0, 4.0, -4.0], [-6.0, 0.0, 0.0, 6.0], [4.0, -5.0, 1.0, -7.0]]], [[[-2.0, -6.0, -4.0, 0.0], [-4.0, -3.0, -5.0, 2.0], [-1.0, -1.0, 0.0, -2.0]], [[2.0, -1.0, -7.0, 3.0], [0.0, 5.0, -2.0, 0.0], [3.0, 1.0, 2.0, -1.0]]], [[[5.0, 2.0, -3.0, 6.0], [0.0, 1.0, 2.0, 5.0], [-3.0, -7.0, -2.0, -6.0]], [[7.0, -4.0, -6.0, 3.0], [3.0, 3.0, 5.0, 5.0], [1.0, -5.0, -5.0, 3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, 4.0, 28.0, -0.0], [12.0, 24.0, 4.0, 8.0], [12.0, 24.0, -0.0, -0.0]], [[8.0, 28.0, -16.0, 16.0], [24.0, -0.0, -0.0, -24.0], [-16.0, 20.0, -4.0, 28.0]]], [[[8.0, 24.0, 16.0, -0.0], [16.0, 12.0, 20.0, -8.0], [4.0, 4.0, -0.0, 8.0]], [[-8.0, 4.0, 28.0, -12.0], [-0.0, -20.0, 8.0, -0.0], [-12.0, -4.0, -8.0, 4.0]]], [[[-20.0, -8.0, 12.0, -24.0], [-0.0, -4.0, -8.0, -20.0], [12.0, 28.0, 8.0, 24.0]], [[-28.0, 16.0, 24.0, -12.0], [-12.0, -12.0, -20.0, -20.0], [-4.0, 20.0, 20.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[-42.0, -7.0, -49.0, 0.0], [-21.0, -42.0, -7.0, -14.0], [-21.0, -42.0, 0.0, 0.0]], [[-14.0, -49.0, 28.0, -28.0], [-42.0, 0.0, 0.0, 42.0], [28.0, -35.0, 7.0, -49.0]]], [[[-14.0, -42.0, -28.0, 0.0], [-28.0, -21.0, -35.0, 14.0], [-7.0, -7.0, 0.0, -14.0]], [[14.0, -7.0, -49.0, 21.0], [0.0, 35.0, -14.0, 0.0], [21.0, 7.0, 14.0, -7.0]]], [[[35.0, 14.0, -21.0, 42.0], [0.0, 7.0, 14.0, 35.0], [-21.0, -49.0, -14.0, -42.0]], [[49.0, -28.0, -42.0, 21.0], [21.0, 21.0, 35.0, 35.0], [7.0, -35.0, -35.0, 21.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(6.0)
      arg1=6.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(36.0)+(1.-msk_ref)*numpy.array(36.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(2.0)
      arg1=numpy.array(-7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(-14.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, 4.0, -1.0])+(1.-msk_arg0)*numpy.array([-1.0, -3.0, -3.0])
      arg1=numpy.array([7.0, 1.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-51.0)+(1.-msk_ref)*numpy.array(-28.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, -7.0, 7.0], [-4.0, -2.0, 3.0]])+(1.-msk_arg0)*numpy.array([[6.0, 4.0, 4.0], [-4.0, 4.0, -4.0]])
      arg1=numpy.array([[5.0, -3.0, 2.0], [-6.0, -3.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(121.0)+(1.-msk_ref)*numpy.array(10.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 6.0, 0.0], [-5.0, -7.0, 5.0], [0.0, 6.0, 5.0], [6.0, 0.0, -7.0]], [[5.0, 1.0, 0.0], [-3.0, 1.0, -5.0], [6.0, -2.0, 6.0], [4.0, 0.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 5.0, -6.0], [0.0, 1.0, -3.0], [7.0, 2.0, 7.0], [-5.0, -7.0, 7.0]], [[1.0, -6.0, -5.0], [-4.0, -4.0, -4.0], [-4.0, 7.0, 7.0], [7.0, -7.0, -4.0]]])
      arg1=numpy.array([[[-7.0, 1.0, -6.0], [-4.0, -2.0, 5.0], [1.0, -7.0, 0.0], [-1.0, -5.0, 0.0]], [[-5.0, 4.0, -6.0], [7.0, -2.0, 4.0], [-2.0, 6.0, -5.0], [1.0, -1.0, -7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-97.0)+(1.-msk_ref)*numpy.array(65.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, -6.0], [7.0, -6.0], [0.0, -2.0]], [[-5.0, -3.0], [7.0, 3.0], [-4.0, 2.0]]], [[[-5.0, 6.0], [-3.0, 5.0], [1.0, -3.0]], [[-1.0, -5.0], [6.0, 0.0], [2.0, -4.0]]], [[[-5.0, 3.0], [-1.0, 1.0], [3.0, 2.0]], [[7.0, 4.0], [2.0, -6.0], [4.0, 1.0]]], [[[2.0, -7.0], [-3.0, -6.0], [-1.0, -5.0]], [[6.0, -3.0], [-1.0, -7.0], [3.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, -1.0], [-7.0, 3.0], [4.0, 4.0]], [[6.0, 6.0], [3.0, 0.0], [5.0, 0.0]]], [[[5.0, -5.0], [-3.0, 1.0], [3.0, -6.0]], [[4.0, 2.0], [-2.0, 7.0], [7.0, 7.0]]], [[[-7.0, 2.0], [6.0, -3.0], [0.0, -7.0]], [[0.0, 4.0], [1.0, -7.0], [-4.0, -7.0]]], [[[7.0, -3.0], [-5.0, -2.0], [-2.0, -4.0]], [[1.0, 0.0], [0.0, -4.0], [0.0, 4.0]]]])
      arg1=numpy.array([[[[3.0, 3.0], [3.0, -6.0], [0.0, -7.0]], [[-4.0, -3.0], [-3.0, 2.0], [-1.0, 0.0]]], [[[1.0, 0.0], [3.0, 2.0], [-5.0, 1.0]], [[2.0, 0.0], [3.0, -6.0], [4.0, -5.0]]], [[[5.0, 0.0], [6.0, 2.0], [-7.0, 0.0]], [[-6.0, 6.0], [-5.0, 0.0], [3.0, -7.0]]], [[[-2.0, 5.0], [3.0, -4.0], [0.0, -3.0]], [[7.0, 3.0], [3.0, -1.0], [-2.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(79.0)+(1.-msk_ref)*numpy.array(-130.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(-5.0)
      arg1=numpy.array([3.0, 4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-12.0, -16.0])+(1.-msk_ref)*numpy.array([-15.0, -20.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, 2.0, 0.0])+(1.-msk_arg0)*numpy.array([7.0, -5.0, -1.0])
      arg1=numpy.array([[2.0, 4.0], [-2.0, 0.0], [-4.0, -1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-14.0, -20.0])+(1.-msk_ref)*numpy.array([28.0, 29.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -6.0, -3.0], [7.0, -6.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-6.0, -2.0, -4.0], [7.0, 0.0, 6.0]])
      arg1=numpy.array([[[5.0, -6.0], [-7.0, 5.0], [3.0, -1.0]], [[-1.0, -3.0], [7.0, -5.0], [0.0, 7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, -78.0])+(1.-msk_ref)*numpy.array([-35.0, 51.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-6.0, -7.0, -3.0], [-3.0, 3.0, 6.0], [1.0, 5.0, -2.0], [6.0, -7.0, -7.0]], [[6.0, -2.0, 4.0], [2.0, 7.0, -2.0], [6.0, 4.0, 6.0], [3.0, 2.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-6.0, 2.0, 7.0], [-3.0, 7.0, 5.0], [2.0, 0.0, 6.0], [5.0, 5.0, -6.0]], [[4.0, -5.0, 7.0], [4.0, -3.0, 2.0], [2.0, 5.0, 0.0], [2.0, 1.0, 6.0]]])
      arg1=numpy.array([[[[7.0, 5.0], [-7.0, 2.0], [-3.0, -6.0]], [[-1.0, 7.0], [-3.0, 3.0], [7.0, 6.0]], [[0.0, -2.0], [-7.0, 3.0], [6.0, 0.0]], [[0.0, -3.0], [-3.0, 3.0], [0.0, 0.0]]], [[[-3.0, -6.0], [-5.0, 0.0], [-1.0, -3.0]], [[2.0, 0.0], [6.0, 5.0], [3.0, -2.0]], [[-2.0, 5.0], [-2.0, 3.0], [-5.0, -6.0]], [[-2.0, 7.0], [0.0, 6.0], [-2.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-16.0, -5.0])+(1.-msk_ref)*numpy.array([-67.0, -67.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(5.0)
      arg1=numpy.array([[-5.0, -4.0, 0.0, -1.0, 0.0], [0.0, 1.0, 7.0, 6.0, 5.0], [2.0, -3.0, 4.0, 4.0, 5.0], [0.0, -5.0, 3.0, 4.0, -5.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[25.0, 20.0, -0.0, 5.0, -0.0], [-0.0, -5.0, -35.0, -30.0, -25.0], [-10.0, 15.0, -20.0, -20.0, -25.0], [-0.0, 25.0, -15.0, -20.0, 25.0]])+(1.-msk_ref)*numpy.array([[-25.0, -20.0, 0.0, -5.0, 0.0], [0.0, 5.0, 35.0, 30.0, 25.0], [10.0, -15.0, 20.0, 20.0, 25.0], [0.0, -25.0, 15.0, 20.0, -25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, 5.0, -1.0])+(1.-msk_arg0)*numpy.array([-6.0, -2.0, -1.0])
      arg1=numpy.array([[[-7.0, 3.0, 1.0, 3.0, 0.0], [-1.0, -7.0, 4.0, -3.0, 0.0], [0.0, 0.0, 0.0, 6.0, -7.0], [5.0, -2.0, -5.0, 1.0, -7.0]], [[-3.0, -6.0, -7.0, -5.0, -2.0], [0.0, 7.0, -2.0, -2.0, -2.0], [0.0, 4.0, 4.0, 6.0, -1.0], [3.0, 4.0, -3.0, 7.0, 7.0]], [[-6.0, 4.0, -2.0, -1.0, 7.0], [2.0, -1.0, 0.0, -2.0, -6.0], [2.0, -7.0, 4.0, 0.0, 2.0], [6.0, -5.0, 4.0, 3.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[26.0, -49.0, -38.0, -39.0, -17.0], [3.0, 71.0, -30.0, 7.0, -4.0], [-2.0, 27.0, 16.0, 0.0, 28.0], [-16.0, 35.0, 6.0, 27.0, 75.0]])+(1.-msk_ref)*numpy.array([[54.0, -10.0, 10.0, -7.0, -3.0], [4.0, 29.0, -20.0, 24.0, 10.0], [-2.0, -1.0, -12.0, -48.0, 42.0], [-42.0, 9.0, 32.0, -23.0, 33.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 2.0, -6.0], [6.0, -3.0, 3.0]])+(1.-msk_arg0)*numpy.array([[6.0, 1.0, 7.0], [-5.0, 2.0, -4.0]])
      arg1=numpy.array([[[[5.0, -6.0, -4.0, -6.0, -5.0], [-7.0, -5.0, -1.0, -1.0, 6.0], [-2.0, 6.0, -5.0, -2.0, 7.0], [-6.0, -4.0, 5.0, 7.0, -5.0]], [[2.0, -4.0, -5.0, -4.0, 7.0], [1.0, 1.0, -4.0, 1.0, 1.0], [-7.0, 0.0, -3.0, -6.0, 3.0], [2.0, -6.0, -6.0, 1.0, 0.0]], [[2.0, 1.0, 0.0, 4.0, -3.0], [6.0, 7.0, -1.0, -2.0, 3.0], [-5.0, -5.0, -2.0, -5.0, -2.0], [3.0, -2.0, 0.0, 2.0, 5.0]]], [[[-7.0, 6.0, 4.0, 7.0, 7.0], [-5.0, 0.0, -2.0, -1.0, 3.0], [-1.0, 0.0, 1.0, 1.0, 6.0], [5.0, -2.0, 5.0, -6.0, 1.0]], [[-3.0, 3.0, -6.0, -5.0, 5.0], [4.0, -7.0, 6.0, -3.0, 4.0], [6.0, 3.0, -4.0, 0.0, 7.0], [-6.0, 5.0, 4.0, 7.0, -3.0]], [[-5.0, -5.0, 2.0, -1.0, 7.0], [4.0, -3.0, 3.0, 3.0, 7.0], [5.0, 2.0, -6.0, -4.0, 4.0], [7.0, 5.0, 7.0, -6.0, 0.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-46.0, -14.0, 30.0, 10.0, 70.0], [-78.0, -38.0, -25.0, 24.0, 23.0], [3.0, 39.0, -4.0, 8.0, 59.0], [43.0, -20.0, 37.0, -71.0, -25.0]])+(1.-msk_ref)*numpy.array([[95.0, -37.0, -69.0, -53.0, -97.0], [18.0, 18.0, -7.0, -32.0, 23.0], [-57.0, -1.0, -36.0, -42.0, -1.0], [-78.0, -44.0, -21.0, 125.0, -6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-2.0)+(1-msk_arg0)*(-6.0)
      arg1=numpy.array([[[6.0, 7.0], [0.0, 4.0]], [[0.0, -5.0], [5.0, -6.0]], [[-7.0, -5.0], [6.0, -5.0]], [[7.0, -7.0], [5.0, -5.0]], [[0.0, -5.0], [0.0, -3.0]], [[-6.0, 3.0], [1.0, 5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -14.0], [-0.0, -8.0]], [[-0.0, 10.0], [-10.0, 12.0]], [[14.0, 10.0], [-12.0, 10.0]], [[-14.0, 14.0], [-10.0, 10.0]], [[-0.0, 10.0], [-0.0, 6.0]], [[12.0, -6.0], [-2.0, -10.0]]])+(1.-msk_ref)*numpy.array([[[-36.0, -42.0], [-0.0, -24.0]], [[-0.0, 30.0], [-30.0, 36.0]], [[42.0, 30.0], [-36.0, 30.0]], [[-42.0, 42.0], [-30.0, 30.0]], [[-0.0, 30.0], [-0.0, 18.0]], [[36.0, -18.0], [-6.0, -30.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, -4.0, -1.0])+(1.-msk_arg0)*numpy.array([7.0, 5.0, 6.0])
      arg1=numpy.array([[[[2.0, 1.0], [3.0, -3.0]], [[2.0, -6.0], [-7.0, 6.0]], [[0.0, 6.0], [1.0, -4.0]], [[7.0, -7.0], [7.0, 4.0]], [[3.0, 0.0], [7.0, 7.0]], [[6.0, -7.0], [0.0, 7.0]]], [[[5.0, 6.0], [2.0, -5.0]], [[1.0, -6.0], [0.0, 5.0]], [[1.0, -6.0], [-6.0, -5.0]], [[-3.0, 2.0], [-5.0, -3.0]], [[-3.0, 2.0], [1.0, 6.0]], [[-5.0, -5.0], [2.0, -7.0]]], [[[-5.0, -7.0], [-2.0, 7.0]], [[-2.0, 0.0], [2.0, 5.0]], [[-2.0, -5.0], [5.0, -7.0]], [[6.0, -5.0], [-4.0, -4.0]], [[0.0, -6.0], [-6.0, -3.0]], [[-5.0, -3.0], [-6.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-25.0, -22.0], [-21.0, 28.0]], [[-12.0, 54.0], [33.0, -55.0]], [[-2.0, -1.0], [14.0, 47.0]], [[-29.0, 32.0], [-11.0, -4.0]], [[-3.0, -2.0], [-33.0, -56.0]], [[-5.0, 58.0], [-2.0, -11.0]]])+(1.-msk_ref)*numpy.array([[[9.0, -5.0], [19.0, -4.0]], [[7.0, -72.0], [-37.0, 97.0]], [[-7.0, -18.0], [7.0, -95.0]], [[70.0, -69.0], [0.0, -11.0]], [[6.0, -26.0], [18.0, 61.0]], [[-13.0, -92.0], [-26.0, 38.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_array_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(-7.0)
      arg1=numpy.array([[[[-1.0, 2.0, -3.0, 4.0], [2.0, -7.0, -3.0, 5.0], [3.0, -6.0, 2.0, -3.0]], [[-3.0, -6.0, 0.0, -4.0], [2.0, -4.0, 3.0, 5.0], [5.0, 4.0, 1.0, 2.0]]], [[[-4.0, -5.0, -3.0, 6.0], [2.0, -3.0, -3.0, -7.0], [-3.0, -3.0, -2.0, -5.0]], [[1.0, -4.0, -7.0, -7.0], [-7.0, -3.0, 2.0, -7.0], [-2.0, 1.0, 3.0, -3.0]]], [[[2.0, -2.0, 4.0, 4.0], [6.0, 4.0, 4.0, 1.0], [-3.0, 5.0, 7.0, 3.0]], [[1.0, -4.0, -6.0, 2.0], [-7.0, -6.0, -2.0, 1.0], [-5.0, -5.0, -5.0, 7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-6.0, 12.0, -18.0, 24.0], [12.0, -42.0, -18.0, 30.0], [18.0, -36.0, 12.0, -18.0]], [[-18.0, -36.0, 0.0, -24.0], [12.0, -24.0, 18.0, 30.0], [30.0, 24.0, 6.0, 12.0]]], [[[-24.0, -30.0, -18.0, 36.0], [12.0, -18.0, -18.0, -42.0], [-18.0, -18.0, -12.0, -30.0]], [[6.0, -24.0, -42.0, -42.0], [-42.0, -18.0, 12.0, -42.0], [-12.0, 6.0, 18.0, -18.0]]], [[[12.0, -12.0, 24.0, 24.0], [36.0, 24.0, 24.0, 6.0], [-18.0, 30.0, 42.0, 18.0]], [[6.0, -24.0, -36.0, 12.0], [-42.0, -36.0, -12.0, 6.0], [-30.0, -30.0, -30.0, 42.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, -14.0, 21.0, -28.0], [-14.0, 49.0, 21.0, -35.0], [-21.0, 42.0, -14.0, 21.0]], [[21.0, 42.0, -0.0, 28.0], [-14.0, 28.0, -21.0, -35.0], [-35.0, -28.0, -7.0, -14.0]]], [[[28.0, 35.0, 21.0, -42.0], [-14.0, 21.0, 21.0, 49.0], [21.0, 21.0, 14.0, 35.0]], [[-7.0, 28.0, 49.0, 49.0], [49.0, 21.0, -14.0, 49.0], [14.0, -7.0, -21.0, 21.0]]], [[[-14.0, 14.0, -28.0, -28.0], [-42.0, -28.0, -28.0, -7.0], [21.0, -35.0, -49.0, -21.0]], [[-7.0, 28.0, 42.0, -14.0], [49.0, 42.0, 14.0, -7.0], [35.0, 35.0, 35.0, -49.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(4.0)+(1-msk_arg0)*(2.0)
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-1.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-4.0)+(1.-msk_ref)*numpy.array(-2.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, 1.0, 1.0])+(1.-msk_arg0)*numpy.array([-5.0, 7.0, 1.0])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([5.0, 6.0, 4.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(35.0)+(1.-msk_ref)*numpy.array(21.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 1.0, -4.0], [-7.0, -5.0, -7.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 0.0, 5.0], [-7.0, -3.0, -2.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[4.0, -4.0, -3.0], [3.0, -2.0, 0.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(21.0)+(1.-msk_ref)*numpy.array(-46.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, 2.0, -2.0], [3.0, 6.0, 0.0], [-7.0, -5.0, 0.0], [-7.0, -4.0, 1.0]], [[-1.0, 7.0, 4.0], [6.0, 4.0, -5.0], [2.0, 4.0, 0.0], [0.0, 0.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 7.0, -4.0], [3.0, -1.0, -5.0], [5.0, -1.0, 0.0], [2.0, -4.0, 6.0]], [[0.0, -3.0, -3.0], [7.0, 2.0, 6.0], [6.0, 1.0, 0.0], [-2.0, 0.0, 5.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[4.0, 2.0, 5.0], [-6.0, -2.0, 4.0], [0.0, 1.0, 5.0], [0.0, -4.0, 3.0]], [[-4.0, -7.0, -3.0], [2.0, -3.0, 2.0], [-3.0, -2.0, 1.0], [-7.0, 3.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-77.0)+(1.-msk_ref)*numpy.array(78.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, 1.0], [-3.0, 7.0], [4.0, 4.0]], [[-5.0, 1.0], [-7.0, 5.0], [3.0, 6.0]]], [[[2.0, -6.0], [-2.0, 3.0], [0.0, -2.0]], [[4.0, -7.0], [-5.0, 3.0], [-6.0, 0.0]]], [[[3.0, -7.0], [-3.0, 0.0], [6.0, 5.0]], [[-7.0, -3.0], [5.0, 6.0], [6.0, 2.0]]], [[[-3.0, 1.0], [-6.0, -3.0], [-3.0, -3.0]], [[0.0, -7.0], [-1.0, 0.0], [1.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, 1.0], [-6.0, -5.0], [-2.0, 5.0]], [[-6.0, -5.0], [-1.0, -1.0], [-7.0, 2.0]]], [[[1.0, 1.0], [7.0, -2.0], [-7.0, 0.0]], [[0.0, 5.0], [3.0, 7.0], [0.0, -3.0]]], [[[-4.0, -4.0], [2.0, -6.0], [7.0, 1.0]], [[2.0, 5.0], [-1.0, -2.0], [-3.0, 3.0]]], [[[-1.0, -4.0], [-1.0, -3.0], [5.0, -4.0]], [[-1.0, 6.0], [-5.0, -4.0], [-4.0, 0.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[-5.0, 0.0], [-2.0, -3.0], [-3.0, -5.0]], [[-1.0, 3.0], [-7.0, -6.0], [3.0, -6.0]]], [[[0.0, -3.0], [-7.0, -3.0], [7.0, -5.0]], [[2.0, -6.0], [-2.0, 7.0], [-1.0, 5.0]]], [[[-4.0, -4.0], [4.0, -6.0], [-4.0, 1.0]], [[-7.0, 3.0], [7.0, -5.0], [4.0, 7.0]]], [[[-6.0, -1.0], [6.0, 5.0], [2.0, 2.0]], [[-2.0, 0.0], [-1.0, 0.0], [1.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(118.0)+(1.-msk_ref)*numpy.array(-82.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(4.0)
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-6.0, 0.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([24.0, -0.0])+(1.-msk_ref)*numpy.array([-24.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, -3.0, 3.0])+(1.-msk_arg0)*numpy.array([0.0, -6.0, 5.0])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-1.0, -7.0], [-2.0, 3.0], [6.0, 0.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, -16.0])+(1.-msk_ref)*numpy.array([42.0, -18.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -4.0, -5.0], [5.0, 3.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-4.0, -4.0, -2.0], [-6.0, -6.0, 0.0]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-7.0, -6.0], [-4.0, 1.0], [6.0, 7.0]], [[-6.0, 5.0], [-6.0, 4.0], [-2.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-104.0, -12.0])+(1.-msk_ref)*numpy.array([104.0, -48.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, -2.0, 2.0], [5.0, 1.0, -1.0], [-1.0, 7.0, -3.0], [5.0, -6.0, 4.0]], [[5.0, -1.0, 2.0], [7.0, 0.0, 6.0], [3.0, -1.0, -1.0], [2.0, -5.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, -5.0, -3.0], [7.0, -4.0, -3.0], [-6.0, -4.0, 0.0], [-7.0, 1.0, 3.0]], [[-6.0, 3.0, 0.0], [6.0, -4.0, -3.0], [7.0, 0.0, 4.0], [5.0, -4.0, 5.0]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[0.0, 5.0], [0.0, 7.0], [0.0, -3.0]], [[3.0, 4.0], [0.0, -2.0], [6.0, 6.0]], [[0.0, 7.0], [-6.0, 7.0], [7.0, 2.0]], [[3.0, 5.0], [3.0, 0.0], [-5.0, 5.0]]], [[[-4.0, -2.0], [-2.0, 2.0], [6.0, -7.0]], [[4.0, 5.0], [-7.0, 6.0], [-4.0, -6.0]], [[6.0, 1.0], [6.0, -6.0], [-3.0, 6.0]], [[5.0, -3.0], [7.0, -6.0], [2.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-75.0, 68.0])+(1.-msk_ref)*numpy.array([113.0, -51.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(2.0)+(1-msk_arg0)*(-6.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[2.0, -7.0, 5.0, 6.0, -1.0], [6.0, -1.0, 0.0, 5.0, 5.0], [-7.0, 0.0, 1.0, 3.0, 2.0], [-7.0, -4.0, 6.0, 7.0, 7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, -14.0, 10.0, 12.0, -2.0], [12.0, -2.0, 0.0, 10.0, 10.0], [-14.0, 0.0, 2.0, 6.0, 4.0], [-14.0, -8.0, 12.0, 14.0, 14.0]])+(1.-msk_ref)*numpy.array([[-12.0, 42.0, -30.0, -36.0, 6.0], [-36.0, 6.0, -0.0, -30.0, -30.0], [42.0, -0.0, -6.0, -18.0, -12.0], [42.0, 24.0, -36.0, -42.0, -42.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, 7.0, 5.0])+(1.-msk_arg0)*numpy.array([-5.0, -5.0, 0.0])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-5.0, -6.0, 2.0, -4.0, -2.0], [0.0, 7.0, -5.0, -2.0, -3.0], [-2.0, -2.0, 0.0, -1.0, 1.0], [-4.0, 7.0, -6.0, 0.0, 7.0]], [[1.0, -3.0, -3.0, 5.0, -4.0], [-4.0, 3.0, -3.0, 4.0, -6.0], [6.0, -2.0, 0.0, 6.0, -7.0], [2.0, -6.0, 2.0, 3.0, -7.0]], [[1.0, 0.0, 0.0, 5.0, -2.0], [-3.0, -1.0, -2.0, -5.0, 0.0], [4.0, 6.0, 0.0, 4.0, 0.0], [2.0, 6.0, -4.0, 3.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[47.0, 21.0, -35.0, 88.0, -24.0], [-43.0, -33.0, 4.0, 17.0, -21.0], [76.0, 30.0, 0.0, 69.0, -56.0], [52.0, -61.0, 36.0, 36.0, -128.0]])+(1.-msk_ref)*numpy.array([[20.0, 45.0, 5.0, -5.0, 30.0], [20.0, -50.0, 40.0, -10.0, 45.0], [-20.0, 20.0, 0.0, -25.0, 30.0], [10.0, -5.0, 20.0, -15.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 1.0, 0.0], [0.0, 3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -3.0, -1.0], [-7.0, 2.0, 0.0]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[0.0, -6.0, -1.0, 2.0, -5.0], [0.0, 4.0, -3.0, -6.0, 4.0], [-2.0, -2.0, 3.0, 1.0, -3.0], [0.0, 0.0, 1.0, -2.0, -6.0]], [[-5.0, 7.0, -1.0, 0.0, -3.0], [1.0, -1.0, 2.0, -3.0, -1.0], [-7.0, -3.0, 1.0, 0.0, -2.0], [4.0, 5.0, -7.0, -7.0, -7.0]], [[-2.0, 7.0, 4.0, 4.0, -3.0], [-3.0, 5.0, -5.0, 0.0, -5.0], [-2.0, 1.0, 4.0, -2.0, -4.0], [-4.0, 1.0, 1.0, 3.0, 6.0]]], [[[4.0, -5.0, 1.0, -3.0, 0.0], [0.0, 2.0, -1.0, -4.0, -4.0], [4.0, 3.0, -6.0, 5.0, -3.0], [-7.0, -2.0, 0.0, 5.0, -4.0]], [[-3.0, -4.0, 7.0, -4.0, -5.0], [-5.0, -5.0, 4.0, 2.0, 5.0], [5.0, -6.0, -3.0, 7.0, 3.0], [6.0, -5.0, -1.0, 5.0, -6.0]], [[-4.0, 1.0, -1.0, -6.0, -2.0], [5.0, -2.0, 3.0, -7.0, 0.0], [0.0, -1.0, 6.0, -5.0, -5.0], [4.0, 2.0, -1.0, -5.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-14.0, -35.0, 15.0, -2.0, -43.0], [-14.0, 4.0, -1.0, -27.0, 34.0], [-2.0, -31.0, 7.0, 26.0, -8.0], [22.0, -10.0, -5.0, -2.0, -55.0]])+(1.-msk_ref)*numpy.array([[-17.0, 29.0, 11.0, -1.0, 27.0], [-10.0, -46.0, 29.0, 71.0, 26.0], [15.0, -15.0, 14.0, -24.0, 52.0], [53.0, -12.0, 13.0, 3.0, 61.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-1.0)+(1-msk_arg0)*(4.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-5.0, 5.0], [6.0, 3.0]], [[7.0, 0.0], [-6.0, -6.0]], [[-1.0, -6.0], [1.0, -1.0]], [[6.0, -7.0], [-3.0, -3.0]], [[1.0, -3.0], [7.0, -1.0]], [[-4.0, -4.0], [-5.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, -5.0], [-6.0, -3.0]], [[-7.0, -0.0], [6.0, 6.0]], [[1.0, 6.0], [-1.0, 1.0]], [[-6.0, 7.0], [3.0, 3.0]], [[-1.0, 3.0], [-7.0, 1.0]], [[4.0, 4.0], [5.0, -2.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, 20.0], [24.0, 12.0]], [[28.0, 0.0], [-24.0, -24.0]], [[-4.0, -24.0], [4.0, -4.0]], [[24.0, -28.0], [-12.0, -12.0]], [[4.0, -12.0], [28.0, -4.0]], [[-16.0, -16.0], [-20.0, 8.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, 4.0, 5.0])+(1.-msk_arg0)*numpy.array([7.0, -3.0, -5.0])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[6.0, -2.0], [-4.0, -6.0]], [[-3.0, -2.0], [3.0, -1.0]], [[0.0, -5.0], [-5.0, 2.0]], [[4.0, -4.0], [-6.0, -7.0]], [[7.0, -3.0], [5.0, -6.0]], [[0.0, 7.0], [0.0, 0.0]]], [[[3.0, -3.0], [-7.0, 1.0]], [[-2.0, 2.0], [-4.0, 2.0]], [[-5.0, 0.0], [6.0, -5.0]], [[-1.0, -2.0], [-5.0, -1.0]], [[0.0, -4.0], [-1.0, -5.0]], [[-7.0, 4.0], [-1.0, -5.0]]], [[[5.0, 5.0], [5.0, -2.0]], [[-6.0, -1.0], [-5.0, -6.0]], [[-5.0, 6.0], [-1.0, 5.0]], [[0.0, 3.0], [-7.0, -3.0]], [[6.0, 5.0], [-2.0, 0.0]], [[4.0, -1.0], [2.0, 2.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[67.0, 3.0], [-23.0, -36.0]], [[-53.0, -7.0], [-26.0, -27.0]], [[-45.0, 5.0], [-6.0, 15.0]], [[16.0, -13.0], [-85.0, -54.0]], [[65.0, -6.0], [11.0, -50.0]], [[-8.0, 46.0], [6.0, -10.0]]])+(1.-msk_ref)*numpy.array([[[8.0, -30.0], [-32.0, -35.0]], [[15.0, -15.0], [58.0, 17.0]], [[40.0, -65.0], [-48.0, 4.0]], [[31.0, -37.0], [8.0, -31.0]], [[19.0, -34.0], [48.0, -27.0]], [[1.0, 42.0], [-7.0, 5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_Symbol_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(3.0)+(1-msk_arg0)*(1.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[6.0, -5.0, 2.0, 0.0], [7.0, -3.0, 0.0, 0.0], [4.0, -5.0, 3.0, 1.0]], [[3.0, 7.0, 7.0, -5.0], [3.0, 3.0, 0.0, 3.0], [1.0, 6.0, -1.0, 1.0]]], [[[5.0, 0.0, -1.0, 6.0], [4.0, 1.0, 2.0, -6.0], [-7.0, 0.0, 6.0, -3.0]], [[0.0, 0.0, 3.0, 2.0], [7.0, 7.0, -7.0, 7.0], [-7.0, -4.0, 3.0, 5.0]]], [[[-4.0, -4.0, 4.0, 7.0], [-2.0, 7.0, -4.0, 2.0], [3.0, 6.0, -7.0, 0.0]], [[-7.0, 6.0, -4.0, 4.0], [5.0, 6.0, 4.0, 4.0], [7.0, 6.0, -3.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[18.0, -15.0, 6.0, 0.0], [21.0, -9.0, 0.0, 0.0], [12.0, -15.0, 9.0, 3.0]], [[9.0, 21.0, 21.0, -15.0], [9.0, 9.0, 0.0, 9.0], [3.0, 18.0, -3.0, 3.0]]], [[[15.0, 0.0, -3.0, 18.0], [12.0, 3.0, 6.0, -18.0], [-21.0, 0.0, 18.0, -9.0]], [[0.0, 0.0, 9.0, 6.0], [21.0, 21.0, -21.0, 21.0], [-21.0, -12.0, 9.0, 15.0]]], [[[-12.0, -12.0, 12.0, 21.0], [-6.0, 21.0, -12.0, 6.0], [9.0, 18.0, -21.0, 0.0]], [[-21.0, 18.0, -12.0, 12.0], [15.0, 18.0, 12.0, 12.0], [21.0, 18.0, -9.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, -5.0, 2.0, 0.0], [7.0, -3.0, 0.0, 0.0], [4.0, -5.0, 3.0, 1.0]], [[3.0, 7.0, 7.0, -5.0], [3.0, 3.0, 0.0, 3.0], [1.0, 6.0, -1.0, 1.0]]], [[[5.0, 0.0, -1.0, 6.0], [4.0, 1.0, 2.0, -6.0], [-7.0, 0.0, 6.0, -3.0]], [[0.0, 0.0, 3.0, 2.0], [7.0, 7.0, -7.0, 7.0], [-7.0, -4.0, 3.0, 5.0]]], [[[-4.0, -4.0, 4.0, 7.0], [-2.0, 7.0, -4.0, 2.0], [3.0, 6.0, -7.0, 0.0]], [[-7.0, 6.0, -4.0, 4.0], [5.0, 6.0, 4.0, 4.0], [7.0, 6.0, -3.0, -4.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(2.0)
      arg1=Data(-1.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(4.0)+(1.-msk_ref)*numpy.array(-2.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, 3.0, -7.0])+(1.-msk_arg0)*numpy.array([7.0, -7.0, 7.0])
      arg1=Data(numpy.array([-3.0, -1.0, 4.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-25.0)+(1.-msk_ref)*numpy.array(14.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 7.0, 0.0], [5.0, -1.0, -7.0]])+(1.-msk_arg0)*numpy.array([[7.0, 3.0, 2.0], [-6.0, -5.0, 7.0]])
      arg1=Data(numpy.array([[-7.0, -5.0, 1.0], [5.0, 7.0, -1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(4.0)+(1.-msk_ref)*numpy.array(-134.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 5.0, -7.0], [7.0, 2.0, 5.0], [-2.0, -6.0, 3.0], [-1.0, -5.0, -2.0]], [[-2.0, 1.0, -3.0], [-1.0, -5.0, 6.0], [-4.0, -2.0, 0.0], [-4.0, 0.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, -1.0, -4.0], [4.0, -4.0, -1.0], [-5.0, 3.0, 1.0], [-1.0, 0.0, -4.0]], [[6.0, -4.0, 0.0], [-7.0, 5.0, 0.0], [-6.0, -4.0, -6.0], [7.0, -2.0, 1.0]]])
      arg1=Data(numpy.array([[[6.0, 7.0, 3.0], [7.0, -2.0, -6.0], [-7.0, 4.0, -3.0], [0.0, -7.0, 3.0]], [[-1.0, -5.0, -4.0], [7.0, 2.0, 0.0], [5.0, -2.0, 5.0], [0.0, 0.0, -2.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(23.0)+(1.-msk_ref)*numpy.array(-30.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, -4.0], [-5.0, 2.0], [7.0, 1.0]], [[-6.0, 3.0], [-7.0, 0.0], [-3.0, -6.0]]], [[[5.0, 4.0], [6.0, -3.0], [7.0, 3.0]], [[2.0, -7.0], [7.0, 5.0], [-5.0, -5.0]]], [[[-4.0, -2.0], [-1.0, 0.0], [4.0, -6.0]], [[1.0, -1.0], [4.0, 3.0], [3.0, 0.0]]], [[[2.0, -4.0], [2.0, -7.0], [-1.0, 3.0]], [[0.0, -2.0], [-3.0, -1.0], [-5.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, -7.0], [-3.0, -4.0], [0.0, 2.0]], [[-2.0, -3.0], [5.0, -7.0], [5.0, -7.0]]], [[[4.0, 0.0], [-6.0, -5.0], [3.0, -5.0]], [[-7.0, -7.0], [-4.0, 7.0], [0.0, 3.0]]], [[[6.0, -7.0], [-2.0, 3.0], [0.0, -2.0]], [[-2.0, 4.0], [-6.0, 6.0], [6.0, 6.0]]], [[[5.0, -2.0], [-7.0, 4.0], [5.0, 1.0]], [[3.0, -6.0], [7.0, 1.0], [7.0, -6.0]]]])
      arg1=Data(numpy.array([[[[-1.0, -1.0], [-4.0, -7.0], [5.0, -1.0]], [[4.0, 5.0], [5.0, 6.0], [-7.0, 0.0]]], [[[2.0, -2.0], [0.0, 3.0], [-1.0, 5.0]], [[-6.0, 0.0], [4.0, 6.0], [0.0, 5.0]]], [[[6.0, -5.0], [7.0, -4.0], [6.0, -7.0]], [[-4.0, 0.0], [-5.0, -1.0], [-4.0, 5.0]]], [[[7.0, 0.0], [3.0, 2.0], [-5.0, 3.0]], [[5.0, -6.0], [0.0, -2.0], [2.0, 6.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(108.0)+(1.-msk_ref)*numpy.array(149.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(-5.0)
      arg1=Data(numpy.array([-3.0, 4.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, -16.0])+(1.-msk_ref)*numpy.array([15.0, -20.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, 2.0, -2.0])+(1.-msk_arg0)*numpy.array([-7.0, 3.0, -5.0])
      arg1=Data(numpy.array([[2.0, 0.0], [3.0, 2.0], [1.0, 3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-10.0, -2.0])+(1.-msk_ref)*numpy.array([-10.0, -9.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 5.0, -6.0], [2.0, -1.0, 1.0]])+(1.-msk_arg0)*numpy.array([[-3.0, 6.0, -1.0], [4.0, -1.0, 5.0]])
      arg1=Data(numpy.array([[[3.0, -7.0], [-5.0, 0.0], [4.0, 5.0]], [[-4.0, -2.0], [-3.0, 7.0], [2.0, 7.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-37.0, -69.0])+(1.-msk_ref)*numpy.array([-46.0, 36.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 2.0, 4.0], [7.0, -5.0, 2.0], [0.0, 4.0, 1.0], [4.0, 7.0, 4.0]], [[6.0, 0.0, 2.0], [-1.0, 0.0, -6.0], [-5.0, 1.0, 6.0], [3.0, -6.0, -2.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 2.0, 3.0], [0.0, -2.0, -6.0], [-4.0, -5.0, 2.0], [7.0, -6.0, -5.0]], [[4.0, -7.0, -5.0], [-4.0, 2.0, -2.0], [0.0, -7.0, 2.0], [2.0, 0.0, -1.0]]])
      arg1=Data(numpy.array([[[[2.0, 6.0], [0.0, 3.0], [-3.0, 3.0]], [[-1.0, 2.0], [-4.0, -5.0], [2.0, -2.0]], [[-7.0, -2.0], [3.0, -2.0], [0.0, -7.0]], [[-4.0, -3.0], [-3.0, -1.0], [-5.0, -6.0]]], [[[6.0, 0.0], [5.0, 4.0], [-2.0, -6.0]], [[1.0, 1.0], [0.0, 2.0], [-2.0, 0.0]], [[0.0, 0.0], [2.0, 5.0], [-1.0, -3.0]], [[-1.0, -7.0], [-5.0, 3.0], [3.0, 0.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([16.0, -82.0])+(1.-msk_ref)*numpy.array([-7.0, 3.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(5.0)+(1-msk_arg0)*(5.0)
      arg1=Data(numpy.array([[6.0, -1.0, 4.0, 5.0, 4.0], [2.0, 4.0, 3.0, 0.0, -1.0], [5.0, 0.0, -3.0, 1.0, -3.0], [6.0, -4.0, -5.0, 6.0, 3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, -5.0, 20.0, 25.0, 20.0], [10.0, 20.0, 15.0, 0.0, -5.0], [25.0, 0.0, -15.0, 5.0, -15.0], [30.0, -20.0, -25.0, 30.0, 15.0]])+(1.-msk_ref)*numpy.array([[30.0, -5.0, 20.0, 25.0, 20.0], [10.0, 20.0, 15.0, 0.0, -5.0], [25.0, 0.0, -15.0, 5.0, -15.0], [30.0, -20.0, -25.0, 30.0, 15.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, 7.0, -6.0])+(1.-msk_arg0)*numpy.array([-5.0, -6.0, 0.0])
      arg1=Data(numpy.array([[[-7.0, -3.0, 1.0, 7.0, 7.0], [1.0, 2.0, 2.0, -7.0, 3.0], [2.0, -2.0, 5.0, 5.0, -7.0], [3.0, 7.0, 6.0, 2.0, 2.0]], [[5.0, -5.0, -2.0, 7.0, 7.0], [-5.0, 6.0, 4.0, 4.0, -7.0], [-6.0, 4.0, -7.0, -3.0, -2.0], [4.0, 0.0, 5.0, -2.0, 0.0]], [[-5.0, -3.0, -7.0, 6.0, 0.0], [6.0, 0.0, 2.0, 2.0, 5.0], [0.0, 4.0, -5.0, -6.0, 1.0], [-3.0, 0.0, -1.0, -7.0, -3.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[58.0, -20.0, 29.0, 20.0, 56.0], [-70.0, 44.0, 18.0, 9.0, -76.0], [-40.0, 2.0, -14.0, 20.0, -27.0], [49.0, 7.0, 47.0, 30.0, 20.0]])+(1.-msk_ref)*numpy.array([[5.0, 45.0, 7.0, -77.0, -77.0], [25.0, -46.0, -34.0, 11.0, 27.0], [26.0, -14.0, 17.0, -7.0, 47.0], [-39.0, -35.0, -60.0, 2.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -7.0, -3.0], [-7.0, 0.0, 7.0]])+(1.-msk_arg0)*numpy.array([[6.0, 1.0, 2.0], [0.0, -4.0, 4.0]])
      arg1=Data(numpy.array([[[[-4.0, -1.0, -2.0, -3.0, 0.0], [-6.0, 1.0, 5.0, 1.0, 3.0], [2.0, -3.0, 0.0, -1.0, 4.0], [6.0, 6.0, 5.0, -3.0, -7.0]], [[3.0, -3.0, -1.0, 6.0, 1.0], [5.0, -5.0, 4.0, -2.0, 6.0], [5.0, 4.0, -7.0, 0.0, 0.0], [-6.0, -1.0, 7.0, 5.0, 7.0]], [[0.0, -5.0, -2.0, -1.0, 6.0], [6.0, -3.0, 1.0, -7.0, -5.0], [2.0, -2.0, 0.0, 0.0, 5.0], [4.0, 1.0, 4.0, -4.0, 0.0]]], [[[-5.0, -4.0, -2.0, 4.0, 0.0], [-2.0, 3.0, -3.0, 2.0, -2.0], [4.0, 2.0, -1.0, 1.0, 7.0], [-6.0, -2.0, 1.0, -7.0, -4.0]], [[4.0, -7.0, -5.0, -7.0, 0.0], [-1.0, -5.0, -4.0, 4.0, -5.0], [0.0, -4.0, -7.0, 0.0, -7.0], [6.0, 4.0, -5.0, 3.0, -3.0]], [[1.0, -7.0, -2.0, -3.0, 5.0], [-7.0, 0.0, 5.0, -4.0, -3.0], [6.0, 7.0, 2.0, -6.0, 0.0], [-4.0, 0.0, -3.0, -5.0, 7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 11.0, 5.0, -100.0, 10.0], [-112.0, 27.0, 45.0, -3.0, -22.0], [-19.0, 1.0, 70.0, -53.0, -48.0], [68.0, 42.0, -69.0, -21.0, 0.0]])+(1.-msk_ref)*numpy.array([[-33.0, -19.0, -5.0, 2.0, 33.0], [-43.0, 15.0, 72.0, -42.0, 22.0], [45.0, 26.0, 29.0, -30.0, 62.0], [-2.0, 21.0, 53.0, -53.0, 5.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(2.0)
      arg1=Data(numpy.array([[[6.0, 5.0], [5.0, 3.0]], [[2.0, 1.0], [-4.0, 7.0]], [[1.0, 3.0], [5.0, -6.0]], [[-2.0, 6.0], [2.0, -2.0]], [[-1.0, 2.0], [2.0, 3.0]], [[0.0, 4.0], [-5.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[12.0, 10.0], [10.0, 6.0]], [[4.0, 2.0], [-8.0, 14.0]], [[2.0, 6.0], [10.0, -12.0]], [[-4.0, 12.0], [4.0, -4.0]], [[-2.0, 4.0], [4.0, 6.0]], [[0.0, 8.0], [-10.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, -3.0, 3.0])+(1.-msk_arg0)*numpy.array([6.0, 0.0, -7.0])
      arg1=Data(numpy.array([[[[0.0, 5.0], [6.0, -6.0]], [[-1.0, 6.0], [-1.0, -6.0]], [[4.0, -4.0], [-1.0, -7.0]], [[-1.0, 1.0], [3.0, 4.0]], [[-6.0, 1.0], [5.0, -5.0]], [[-4.0, -3.0], [0.0, -3.0]]], [[[-2.0, -4.0], [4.0, -1.0]], [[0.0, 0.0], [0.0, 2.0]], [[1.0, -6.0], [3.0, -7.0]], [[3.0, 1.0], [4.0, -2.0]], [[0.0, 2.0], [0.0, 4.0]], [[0.0, 0.0], [-4.0, 0.0]]], [[[4.0, -7.0], [-4.0, -5.0]], [[6.0, -3.0], [-2.0, -2.0]], [[2.0, -4.0], [-6.0, -5.0]], [[0.0, -5.0], [0.0, 4.0]], [[-7.0, -2.0], [5.0, 0.0]], [[2.0, 4.0], [-7.0, 5.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[18.0, -9.0], [-24.0, -12.0]], [[18.0, -9.0], [-6.0, -12.0]], [[3.0, 6.0], [-27.0, 6.0]], [[-9.0, -18.0], [-12.0, 18.0]], [[-21.0, -12.0], [15.0, -12.0]], [[6.0, 12.0], [-9.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-28.0, 79.0], [64.0, -1.0]], [[-48.0, 57.0], [8.0, -22.0]], [[10.0, 4.0], [36.0, -7.0]], [[-6.0, 41.0], [18.0, -4.0]], [[13.0, 20.0], [-5.0, -30.0]], [[-38.0, -46.0], [49.0, -53.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_constData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-6.0)+(1-msk_arg0)*(-5.0)
      arg1=Data(numpy.array([[[[-2.0, -4.0, -2.0, -3.0], [2.0, 4.0, -2.0, 0.0], [2.0, 4.0, -1.0, 0.0]], [[4.0, 3.0, 0.0, 5.0], [-2.0, -1.0, -2.0, 4.0], [-3.0, 0.0, -5.0, -7.0]]], [[[-7.0, 1.0, 4.0, 0.0], [0.0, -6.0, 1.0, -4.0], [0.0, -5.0, 6.0, -2.0]], [[-1.0, -5.0, 6.0, -7.0], [4.0, 1.0, -7.0, 0.0], [7.0, 6.0, 5.0, -5.0]]], [[[-1.0, 3.0, -5.0, 1.0], [-3.0, 1.0, 3.0, 7.0], [-1.0, -3.0, 6.0, -3.0]], [[7.0, -7.0, 0.0, -7.0], [1.0, -6.0, -6.0, 7.0], [2.0, -1.0, -1.0, 4.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[12.0, 24.0, 12.0, 18.0], [-12.0, -24.0, 12.0, -0.0], [-12.0, -24.0, 6.0, -0.0]], [[-24.0, -18.0, -0.0, -30.0], [12.0, 6.0, 12.0, -24.0], [18.0, -0.0, 30.0, 42.0]]], [[[42.0, -6.0, -24.0, -0.0], [-0.0, 36.0, -6.0, 24.0], [-0.0, 30.0, -36.0, 12.0]], [[6.0, 30.0, -36.0, 42.0], [-24.0, -6.0, 42.0, -0.0], [-42.0, -36.0, -30.0, 30.0]]], [[[6.0, -18.0, 30.0, -6.0], [18.0, -6.0, -18.0, -42.0], [6.0, 18.0, -36.0, 18.0]], [[-42.0, 42.0, -0.0, 42.0], [-6.0, 36.0, 36.0, -42.0], [-12.0, 6.0, 6.0, -24.0]]]])+(1.-msk_ref)*numpy.array([[[[10.0, 20.0, 10.0, 15.0], [-10.0, -20.0, 10.0, -0.0], [-10.0, -20.0, 5.0, -0.0]], [[-20.0, -15.0, -0.0, -25.0], [10.0, 5.0, 10.0, -20.0], [15.0, -0.0, 25.0, 35.0]]], [[[35.0, -5.0, -20.0, -0.0], [-0.0, 30.0, -5.0, 20.0], [-0.0, 25.0, -30.0, 10.0]], [[5.0, 25.0, -30.0, 35.0], [-20.0, -5.0, 35.0, -0.0], [-35.0, -30.0, -25.0, 25.0]]], [[[5.0, -15.0, 25.0, -5.0], [15.0, -5.0, -15.0, -35.0], [5.0, 15.0, -30.0, 15.0]], [[-35.0, 35.0, -0.0, 35.0], [-5.0, 30.0, 30.0, -35.0], [-10.0, 5.0, 5.0, -20.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(4.0)+(1-msk_arg0)*(2.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-5.0)+(1-msk_arg1)*(2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-20.0)+(1.-msk_ref)*numpy.array(4.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, 5.0, -6.0])+(1.-msk_arg0)*numpy.array([-3.0, -6.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -7.0, 6.0])+(1.-msk_arg1)*numpy.array([1.0, -3.0, 3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-71.0)+(1.-msk_ref)*numpy.array(15.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 3.0, 4.0], [2.0, 7.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-4.0, -3.0, 2.0], [0.0, 5.0, -6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, -6.0, -1.0], [-2.0, -5.0, -5.0]])+(1.-msk_arg1)*numpy.array([[6.0, 6.0, 4.0], [-3.0, 4.0, -7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-75.0)+(1.-msk_ref)*numpy.array(28.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 1.0, 4.0], [1.0, -1.0, -5.0], [1.0, 0.0, -4.0], [4.0, 6.0, 1.0]], [[-6.0, -7.0, 0.0], [-3.0, -7.0, 2.0], [6.0, -5.0, 6.0], [1.0, 7.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[6.0, 3.0, 1.0], [-1.0, -4.0, -6.0], [-5.0, 1.0, 1.0], [-7.0, -2.0, -1.0]], [[-4.0, -5.0, 5.0], [1.0, 7.0, 6.0], [-2.0, -2.0, 6.0], [4.0, 2.0, 1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, 3.0, -7.0], [2.0, 3.0, 0.0], [-7.0, -3.0, 5.0], [4.0, -3.0, 1.0]], [[5.0, 4.0, 5.0], [-3.0, -7.0, -1.0], [-4.0, 4.0, 2.0], [5.0, -6.0, -2.0]]])+(1.-msk_arg1)*numpy.array([[[4.0, -3.0, 4.0], [2.0, -3.0, -5.0], [5.0, -1.0, 7.0], [-6.0, 0.0, -3.0]], [[3.0, -7.0, -7.0], [0.0, 2.0, -7.0], [2.0, 0.0, -6.0], [-7.0, 0.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-137.0)+(1.-msk_ref)*numpy.array(-21.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -4.0], [4.0, 1.0], [7.0, -6.0]], [[0.0, 6.0], [3.0, 6.0], [3.0, 5.0]]], [[[-2.0, -4.0], [6.0, 0.0], [-7.0, 4.0]], [[2.0, 0.0], [1.0, -6.0], [-2.0, 4.0]]], [[[4.0, -2.0], [7.0, 5.0], [2.0, -7.0]], [[-4.0, 5.0], [-1.0, 0.0], [-6.0, -7.0]]], [[[-6.0, 4.0], [7.0, 7.0], [-6.0, -5.0]], [[-7.0, -6.0], [4.0, -5.0], [-1.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, 0.0], [-3.0, 5.0], [-4.0, 3.0]], [[-7.0, 7.0], [1.0, 5.0], [-1.0, 1.0]]], [[[0.0, 3.0], [-7.0, -5.0], [-7.0, 5.0]], [[3.0, -5.0], [-6.0, 3.0], [3.0, -5.0]]], [[[-3.0, 6.0], [-1.0, 2.0], [-2.0, 4.0]], [[5.0, 0.0], [-2.0, 7.0], [0.0, -4.0]]], [[[5.0, -3.0], [0.0, 6.0], [-2.0, 5.0]], [[2.0, -1.0], [-3.0, -6.0], [3.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -1.0], [-7.0, 4.0], [1.0, -1.0]], [[-6.0, 0.0], [-7.0, 6.0], [-3.0, 5.0]]], [[[-1.0, -4.0], [-5.0, -1.0], [-1.0, -1.0]], [[5.0, -5.0], [3.0, -3.0], [-5.0, 4.0]]], [[[-1.0, -4.0], [0.0, -1.0], [0.0, 6.0]], [[0.0, 3.0], [-4.0, -1.0], [7.0, -2.0]]], [[[-2.0, -3.0], [-2.0, 0.0], [-5.0, -6.0]], [[0.0, -4.0], [4.0, 0.0], [-7.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -4.0], [6.0, 0.0], [7.0, 1.0]], [[2.0, -3.0], [5.0, -1.0], [-6.0, 0.0]]], [[[3.0, 4.0], [-5.0, 4.0], [4.0, 4.0]], [[2.0, -6.0], [2.0, -4.0], [2.0, 1.0]]], [[[2.0, 4.0], [0.0, 6.0], [0.0, 1.0]], [[5.0, -7.0], [7.0, 2.0], [-6.0, 7.0]]], [[[6.0, 6.0], [-2.0, 2.0], [3.0, 7.0]], [[0.0, 1.0], [7.0, 0.0], [7.0, 7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(71.0)+(1.-msk_ref)*numpy.array(78.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(5.0)+(1-msk_arg0)*(-4.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-7.0, 2.0])+(1.-msk_arg1)*numpy.array([-2.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-35.0, 10.0])+(1.-msk_ref)*numpy.array([8.0, -4.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, -6.0, 1.0])+(1.-msk_arg0)*numpy.array([6.0, 6.0, -4.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 0.0], [0.0, -3.0], [1.0, -7.0]])+(1.-msk_arg1)*numpy.array([[-3.0, 1.0], [-5.0, 1.0], [6.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([1.0, 11.0])+(1.-msk_ref)*numpy.array([-72.0, 12.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, -4.0, 0.0], [0.0, -4.0, -2.0]])+(1.-msk_arg0)*numpy.array([[2.0, 6.0, -5.0], [-1.0, 5.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, 5.0], [-5.0, 0.0], [0.0, -6.0]], [[-5.0, -4.0], [-1.0, -4.0], [0.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 4.0], [3.0, -7.0], [-7.0, -1.0]], [[6.0, 3.0], [-3.0, 6.0], [0.0, 7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([30.0, 21.0])+(1.-msk_ref)*numpy.array([34.0, 19.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 0.0, -6.0], [1.0, 4.0, -1.0], [3.0, -7.0, 3.0], [-4.0, 3.0, -2.0]], [[-3.0, -4.0, -4.0], [4.0, 6.0, -4.0], [-3.0, 0.0, -3.0], [-2.0, 0.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, 2.0, 2.0], [3.0, -5.0, -1.0], [1.0, 2.0, -5.0], [-6.0, -5.0, 0.0]], [[7.0, -7.0, -7.0], [0.0, -7.0, -2.0], [0.0, 6.0, 4.0], [-2.0, -4.0, -4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 1.0], [4.0, -4.0], [2.0, -1.0]], [[4.0, 1.0], [1.0, -6.0], [2.0, 2.0]], [[5.0, 3.0], [-3.0, -2.0], [-3.0, -5.0]], [[3.0, 2.0], [4.0, 5.0], [-2.0, 0.0]]], [[[7.0, 5.0], [-1.0, 2.0], [6.0, 0.0]], [[-5.0, 7.0], [2.0, 0.0], [2.0, -3.0]], [[1.0, -6.0], [3.0, 5.0], [-7.0, 7.0]], [[4.0, 5.0], [-7.0, -3.0], [-7.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 1.0], [4.0, -5.0], [5.0, 5.0]], [[5.0, -3.0], [7.0, 7.0], [5.0, 7.0]], [[2.0, 0.0], [-7.0, -1.0], [4.0, -6.0]], [[-1.0, 2.0], [0.0, 1.0], [-5.0, 0.0]]], [[[-6.0, 0.0], [0.0, 1.0], [0.0, -4.0]], [[-4.0, 0.0], [-5.0, 2.0], [0.0, 6.0]], [[-5.0, -1.0], [7.0, 4.0], [-3.0, -3.0]], [[-4.0, -3.0], [-5.0, -4.0], [0.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 24.0])+(1.-msk_ref)*numpy.array([18.0, -38.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(4.0)+(1-msk_arg0)*(-7.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -2.0, 1.0, 0.0, -4.0], [4.0, -4.0, 3.0, 1.0, 2.0], [6.0, -5.0, -1.0, 7.0, -7.0], [0.0, 6.0, 0.0, -3.0, -1.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -7.0, -4.0, 3.0, -2.0], [-3.0, -4.0, -7.0, 0.0, 4.0], [0.0, -4.0, -7.0, 0.0, -2.0], [-6.0, -5.0, 0.0, 5.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, -8.0, 4.0, 0.0, -16.0], [16.0, -16.0, 12.0, 4.0, 8.0], [24.0, -20.0, -4.0, 28.0, -28.0], [0.0, 24.0, 0.0, -12.0, -4.0]])+(1.-msk_ref)*numpy.array([[49.0, 49.0, 28.0, -21.0, 14.0], [21.0, 28.0, 49.0, -0.0, -28.0], [-0.0, 28.0, 49.0, -0.0, 14.0], [42.0, 35.0, -0.0, -35.0, -28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, -7.0, 5.0])+(1.-msk_arg0)*numpy.array([0.0, -5.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, 0.0, -2.0, 2.0, -4.0], [-3.0, -4.0, -5.0, -1.0, 0.0], [0.0, -2.0, -3.0, -4.0, 4.0], [-1.0, -2.0, -7.0, -7.0, 7.0]], [[6.0, -5.0, 4.0, -5.0, 4.0], [7.0, 3.0, -2.0, -3.0, 7.0], [-6.0, 0.0, -4.0, -3.0, -7.0], [0.0, 3.0, -4.0, -2.0, -1.0]], [[0.0, -5.0, 5.0, 5.0, 2.0], [-6.0, 2.0, 7.0, 4.0, -2.0], [4.0, 1.0, -7.0, 0.0, -7.0], [6.0, 5.0, 0.0, -2.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-4.0, -1.0, -7.0, 0.0, -5.0], [-5.0, -6.0, 6.0, -3.0, 0.0], [4.0, -4.0, -1.0, 6.0, 0.0], [7.0, 5.0, -2.0, 6.0, 4.0]], [[-4.0, -3.0, -2.0, -4.0, -5.0], [-7.0, -4.0, -2.0, 5.0, -1.0], [0.0, -5.0, -7.0, -4.0, -1.0], [2.0, 1.0, -1.0, 7.0, -4.0]], [[7.0, 5.0, 0.0, -4.0, -2.0], [-1.0, -3.0, 6.0, 2.0, 0.0], [0.0, 0.0, 4.0, 0.0, 0.0], [0.0, 5.0, -5.0, -2.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-46.0, 10.0, -5.0, 62.0, -22.0], [-82.0, -15.0, 44.0, 40.0, -59.0], [62.0, 3.0, -10.0, 17.0, 18.0], [29.0, 2.0, 21.0, -3.0, -11.0]])+(1.-msk_ref)*numpy.array([[20.0, 15.0, 10.0, 20.0, 25.0], [35.0, 20.0, 10.0, -25.0, 5.0], [0.0, 25.0, 35.0, 20.0, 5.0], [-10.0, -5.0, 5.0, -35.0, 20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -7.0, 4.0], [-4.0, 4.0, -1.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0, -6.0], [3.0, 6.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, -2.0, -2.0, 0.0, -6.0], [6.0, 6.0, 2.0, -3.0, -1.0], [-1.0, 0.0, 2.0, -6.0, -4.0], [-1.0, 0.0, -1.0, -3.0, -3.0]], [[-7.0, -1.0, 2.0, -6.0, 4.0], [-3.0, 1.0, 4.0, 3.0, 2.0], [7.0, 3.0, 7.0, 5.0, 1.0], [-6.0, 0.0, 5.0, 5.0, -4.0]], [[0.0, -3.0, 2.0, 5.0, 4.0], [-7.0, 0.0, 7.0, 6.0, 4.0], [-1.0, 6.0, -3.0, -4.0, 7.0], [5.0, -3.0, -6.0, -2.0, 6.0]]], [[[-4.0, 7.0, -1.0, -3.0, -3.0], [6.0, 2.0, -7.0, -6.0, -6.0], [1.0, -7.0, -2.0, 0.0, 3.0], [-2.0, 5.0, 6.0, 3.0, 6.0]], [[3.0, -2.0, 0.0, -2.0, -1.0], [6.0, 7.0, 5.0, 1.0, -7.0], [0.0, -4.0, -1.0, 6.0, -2.0], [2.0, 2.0, -6.0, -1.0, -7.0]], [[4.0, -4.0, -3.0, 3.0, -6.0], [4.0, -7.0, -2.0, -3.0, 4.0], [-1.0, -4.0, -6.0, 1.0, -1.0], [3.0, -5.0, -3.0, 4.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -4.0, -1.0, -1.0, -1.0], [-6.0, -4.0, -5.0, -6.0, 0.0], [4.0, -3.0, 2.0, 0.0, -3.0], [-5.0, 2.0, -1.0, -4.0, -4.0]], [[-1.0, 0.0, 6.0, 6.0, -2.0], [-4.0, 7.0, 2.0, 7.0, 0.0], [3.0, 0.0, -2.0, 0.0, 4.0], [-3.0, 1.0, 1.0, -6.0, 7.0]], [[3.0, 1.0, 0.0, 5.0, 4.0], [-4.0, 4.0, 3.0, -3.0, 2.0], [3.0, 6.0, -2.0, 6.0, -1.0], [-6.0, 4.0, 0.0, 6.0, -3.0]]], [[[0.0, -2.0, 7.0, -6.0, -4.0], [4.0, -6.0, -1.0, -6.0, 0.0], [7.0, 1.0, 0.0, -7.0, 7.0], [-4.0, 7.0, -3.0, -5.0, 6.0]], [[5.0, -6.0, -3.0, 1.0, 3.0], [2.0, 2.0, -5.0, -3.0, -7.0], [-2.0, 6.0, 2.0, 6.0, -5.0], [-1.0, -4.0, 6.0, -4.0, -7.0]], [[6.0, 1.0, -4.0, -4.0, 6.0], [-1.0, 7.0, 6.0, 1.0, 0.0], [0.0, -4.0, -2.0, 5.0, -5.0], [0.0, 0.0, 7.0, 2.0, -5.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[97.0, -25.0, 13.0, 63.0, 38.0], [-47.0, -16.0, 38.0, 52.0, 0.0], [-50.0, 19.0, -63.0, 8.0, 26.0], [81.0, -19.0, -98.0, -45.0, 14.0]])+(1.-msk_ref)*numpy.array([[53.0, -41.0, -19.0, -64.0, 22.0], [37.0, 26.0, -7.0, -4.0, -54.0], [-6.0, -25.0, 8.0, 14.0, -34.0], [15.0, -26.0, 77.0, -67.0, -34.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(2.0)+(1-msk_arg0)*(3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, -4.0], [-2.0, -4.0]], [[2.0, 6.0], [-6.0, -2.0]], [[-7.0, 1.0], [3.0, -3.0]], [[-7.0, -3.0], [-5.0, -5.0]], [[-4.0, -4.0], [-7.0, 1.0]], [[-6.0, -7.0], [0.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, 1.0], [4.0, 4.0]], [[0.0, 3.0], [5.0, -3.0]], [[1.0, -2.0], [-5.0, 0.0]], [[-2.0, 1.0], [4.0, -3.0]], [[3.0, -4.0], [0.0, -6.0]], [[-1.0, 3.0], [-4.0, 3.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[8.0, -8.0], [-4.0, -8.0]], [[4.0, 12.0], [-12.0, -4.0]], [[-14.0, 2.0], [6.0, -6.0]], [[-14.0, -6.0], [-10.0, -10.0]], [[-8.0, -8.0], [-14.0, 2.0]], [[-12.0, -14.0], [0.0, -14.0]]])+(1.-msk_ref)*numpy.array([[[18.0, 3.0], [12.0, 12.0]], [[0.0, 9.0], [15.0, -9.0]], [[3.0, -6.0], [-15.0, 0.0]], [[-6.0, 3.0], [12.0, -9.0]], [[9.0, -12.0], [0.0, -18.0]], [[-3.0, 9.0], [-12.0, 9.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([4.0, 3.0, 2.0])+(1.-msk_arg0)*numpy.array([5.0, 0.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -5.0], [7.0, 3.0]], [[-3.0, 0.0], [-3.0, 6.0]], [[4.0, -2.0], [2.0, 5.0]], [[0.0, -7.0], [-1.0, 2.0]], [[3.0, 0.0], [-3.0, 0.0]], [[6.0, -7.0], [7.0, 0.0]]], [[[-3.0, 6.0], [2.0, -4.0]], [[3.0, 6.0], [0.0, 3.0]], [[-4.0, -5.0], [7.0, 6.0]], [[-1.0, -5.0], [6.0, -7.0]], [[-2.0, 1.0], [2.0, -4.0]], [[7.0, 1.0], [7.0, -4.0]]], [[[-3.0, 5.0], [2.0, 0.0]], [[-3.0, 0.0], [4.0, 3.0]], [[-4.0, -6.0], [-2.0, -5.0]], [[-7.0, -4.0], [-2.0, -1.0]], [[4.0, -1.0], [-2.0, -4.0]], [[-2.0, 4.0], [6.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -6.0], [2.0, 2.0]], [[-2.0, 3.0], [-3.0, 5.0]], [[6.0, -4.0], [0.0, 2.0]], [[2.0, -3.0], [5.0, 5.0]], [[-7.0, 6.0], [5.0, 4.0]], [[0.0, -7.0], [1.0, -5.0]]], [[[1.0, 0.0], [-1.0, -1.0]], [[1.0, 0.0], [-1.0, 7.0]], [[-5.0, 0.0], [0.0, -5.0]], [[3.0, 6.0], [0.0, -1.0]], [[5.0, 4.0], [-1.0, 6.0]], [[-7.0, 3.0], [5.0, -6.0]]], [[[7.0, 5.0], [-3.0, -6.0]], [[-3.0, 0.0], [2.0, -2.0]], [[5.0, 6.0], [4.0, -4.0]], [[0.0, 4.0], [1.0, -5.0]], [[-2.0, -6.0], [-2.0, -6.0]], [[1.0, -3.0], [7.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-15.0, 8.0], [38.0, 0.0]], [[-9.0, 18.0], [-4.0, 39.0]], [[-4.0, -35.0], [25.0, 28.0]], [[-17.0, -51.0], [10.0, -15.0]], [[14.0, 1.0], [-10.0, -20.0]], [[41.0, -17.0], [61.0, -24.0]]])+(1.-msk_ref)*numpy.array([[[20.0, -30.0], [10.0, 10.0]], [[-10.0, 15.0], [-15.0, 25.0]], [[30.0, -20.0], [0.0, 10.0]], [[10.0, -15.0], [25.0, 25.0]], [[-35.0, 30.0], [25.0, 20.0]], [[0.0, -35.0], [5.0, -25.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank0_expandedData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(-2.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[7.0, -5.0, -3.0, -5.0], [-7.0, -5.0, 0.0, 2.0], [-1.0, -5.0, -4.0, -1.0]], [[0.0, -2.0, 0.0, 2.0], [-7.0, 1.0, -1.0, 0.0], [-6.0, 1.0, 2.0, 1.0]]], [[[7.0, 4.0, 5.0, -1.0], [-7.0, 6.0, 3.0, 2.0], [0.0, -1.0, 1.0, 0.0]], [[-5.0, 3.0, 1.0, -4.0], [1.0, 6.0, -6.0, 5.0], [6.0, -5.0, 3.0, 0.0]]], [[[-1.0, 0.0, 3.0, 6.0], [1.0, 7.0, 4.0, -2.0], [2.0, -1.0, 4.0, 5.0]], [[5.0, -6.0, 0.0, 6.0], [3.0, 4.0, 0.0, 7.0], [4.0, 2.0, -1.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, 3.0, -5.0, 5.0], [-6.0, -1.0, 7.0, 2.0], [0.0, -6.0, 2.0, -5.0]], [[0.0, -3.0, 7.0, -6.0], [1.0, 3.0, -2.0, -1.0], [6.0, 3.0, 0.0, -7.0]]], [[[-2.0, -2.0, 3.0, -1.0], [-2.0, 5.0, -3.0, -2.0], [0.0, -6.0, -4.0, -3.0]], [[4.0, -3.0, 1.0, 6.0], [3.0, 4.0, 5.0, -3.0], [-3.0, -4.0, 4.0, 4.0]]], [[[0.0, -1.0, 2.0, 2.0], [1.0, -2.0, 6.0, 3.0], [0.0, 7.0, 1.0, 4.0]], [[7.0, 7.0, 2.0, -5.0], [-2.0, 3.0, -5.0, -2.0], [-7.0, 3.0, 2.0, 0.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, -0.0, -0.0, -0.0], [-0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, -0.0]], [[0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]]], [[[-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0]], [[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-2.0, -6.0, 10.0, -10.0], [12.0, 2.0, -14.0, -4.0], [-0.0, 12.0, -4.0, 10.0]], [[-0.0, 6.0, -14.0, 12.0], [-2.0, -6.0, 4.0, 2.0], [-12.0, -6.0, -0.0, 14.0]]], [[[4.0, 4.0, -6.0, 2.0], [4.0, -10.0, 6.0, 4.0], [-0.0, 12.0, 8.0, 6.0]], [[-8.0, 6.0, -2.0, -12.0], [-6.0, -8.0, -10.0, 6.0], [6.0, 8.0, -8.0, -8.0]]], [[[-0.0, 2.0, -4.0, -4.0], [-2.0, 4.0, -12.0, -6.0], [-0.0, -14.0, -2.0, -8.0]], [[-14.0, -14.0, -4.0, 10.0], [4.0, -6.0, 10.0, 4.0], [14.0, -6.0, -4.0, -0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([4.0, 4.0])+(1.-msk_arg0)*numpy.array([2.0, -4.0])
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([0.0, 0.0])+(1.-msk_ref)*numpy.array([0.0, -0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, 6.0])+(1.-msk_arg0)*numpy.array([-4.0, 0.0])
      arg1=numpy.array(-3.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([9.0, -18.0])+(1.-msk_ref)*numpy.array([12.0, -0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, -6.0], [7.0, -2.0], [-5.0, -1.0]])+(1.-msk_arg0)*numpy.array([[6.0, 2.0], [-5.0, -4.0], [4.0, 2.0]])
      arg1=numpy.array([3.0, 6.0, 7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([22.0, -37.0])+(1.-msk_ref)*numpy.array([16.0, -4.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, -3.0], [5.0, -3.0], [-4.0, -5.0]], [[3.0, -3.0], [-3.0, -2.0], [0.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, -3.0], [0.0, -1.0], [-4.0, -2.0]], [[-5.0, 5.0], [-4.0, 0.0], [4.0, -3.0]]])
      arg1=numpy.array([[1.0, -3.0, 0.0], [6.0, -6.0, -3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, 9.0])+(1.-msk_ref)*numpy.array([-22.0, 39.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 6.0], [0.0, -1.0], [6.0, -3.0]], [[4.0, 5.0], [6.0, -4.0], [4.0, 1.0]], [[6.0, 4.0], [0.0, 6.0], [0.0, 0.0]], [[0.0, 1.0], [2.0, -4.0], [-2.0, 0.0]]], [[[-4.0, -3.0], [-3.0, 3.0], [-3.0, -4.0]], [[7.0, 3.0], [1.0, 0.0], [-7.0, -5.0]], [[-1.0, 4.0], [-6.0, -5.0], [3.0, -2.0]], [[-7.0, -2.0], [0.0, -7.0], [7.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-2.0, 5.0], [-7.0, -4.0], [2.0, 0.0]], [[-4.0, 6.0], [-2.0, -6.0], [7.0, 0.0]], [[-1.0, 0.0], [7.0, -4.0], [-4.0, 0.0]], [[-5.0, -3.0], [-7.0, -5.0], [3.0, 3.0]]], [[[6.0, 5.0], [2.0, 7.0], [0.0, 7.0]], [[-3.0, 2.0], [0.0, 2.0], [-3.0, 2.0]], [[6.0, -5.0], [0.0, 4.0], [-6.0, -5.0]], [[6.0, 7.0], [2.0, 6.0], [1.0, 4.0]]]])
      arg1=numpy.array([[[-5.0, 6.0, -7.0], [7.0, 4.0, 7.0], [-1.0, -5.0, 1.0], [1.0, 2.0, 4.0]], [[7.0, 0.0, 5.0], [-2.0, 6.0, 0.0], [5.0, -1.0, -7.0], [-4.0, 4.0, 2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, -68.0])+(1.-msk_ref)*numpy.array([28.0, 76.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, -5.0])+(1.-msk_arg0)*numpy.array([3.0, -7.0])
      arg1=numpy.array([2.0, -4.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[14.0, -28.0], [-10.0, 20.0]])+(1.-msk_ref)*numpy.array([[6.0, -12.0], [-14.0, 28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, 6.0], [6.0, 6.0], [0.0, 4.0]])+(1.-msk_arg0)*numpy.array([[5.0, 6.0], [3.0, 3.0], [7.0, -7.0]])
      arg1=numpy.array([[7.0, -4.0], [7.0, -6.0], [1.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[49.0, -40.0], [88.0, -36.0]])+(1.-msk_ref)*numpy.array([[63.0, 4.0], [56.0, -84.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 3.0], [2.0, 6.0], [-5.0, -6.0]], [[5.0, -1.0], [2.0, 7.0], [0.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 0.0], [-4.0, -6.0], [1.0, 0.0]], [[-5.0, -6.0], [4.0, 0.0], [-5.0, 1.0]]])
      arg1=numpy.array([[[1.0, 5.0], [3.0, 2.0], [-4.0, -2.0]], [[7.0, -4.0], [-2.0, -7.0], [2.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[58.0, -15.0], [18.0, 9.0]])+(1.-msk_ref)*numpy.array([[-69.0, 7.0], [-58.0, 7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 6.0], [5.0, 5.0], [0.0, 3.0]], [[1.0, 6.0], [-5.0, 0.0], [-5.0, -1.0]], [[-4.0, 0.0], [0.0, 6.0], [7.0, 4.0]], [[-5.0, 4.0], [5.0, -7.0], [-6.0, -1.0]]], [[[1.0, 0.0], [-4.0, -6.0], [2.0, -6.0]], [[5.0, -2.0], [3.0, -1.0], [-3.0, -2.0]], [[0.0, -3.0], [0.0, 2.0], [0.0, 0.0]], [[-2.0, 1.0], [1.0, -5.0], [-4.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 3.0], [6.0, 1.0], [5.0, 2.0]], [[-5.0, 2.0], [-3.0, 7.0], [0.0, 0.0]], [[-7.0, 2.0], [-5.0, 0.0], [1.0, -2.0]], [[3.0, 2.0], [0.0, -6.0], [-3.0, -3.0]]], [[[-6.0, -5.0], [5.0, 0.0], [-1.0, 1.0]], [[-3.0, -7.0], [6.0, -7.0], [0.0, -6.0]], [[5.0, 4.0], [-2.0, -5.0], [-4.0, 0.0]], [[0.0, 6.0], [1.0, 3.0], [0.0, -6.0]]]])
      arg1=numpy.array([[[[0.0, -5.0], [-5.0, -5.0], [-4.0, 0.0]], [[0.0, 0.0], [4.0, -2.0], [-2.0, 6.0]], [[-7.0, -7.0], [7.0, -2.0], [-3.0, 3.0]], [[1.0, 0.0], [3.0, 5.0], [-5.0, 3.0]]], [[[-3.0, 6.0], [-7.0, 3.0], [7.0, 6.0]], [[4.0, -5.0], [4.0, -2.0], [0.0, 3.0]], [[-3.0, -6.0], [0.0, -7.0], [-4.0, 0.0]], [[-7.0, 7.0], [0.0, 1.0], [3.0, -6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[85.0, -12.0], [-30.0, -135.0]])+(1.-msk_ref)*numpy.array([[-44.0, 20.0], [-100.0, 6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, 7.0])+(1.-msk_arg0)*numpy.array([4.0, -2.0])
      arg1=numpy.array([[-3.0, -3.0, 0.0, -5.0, -7.0], [5.0, -7.0, 3.0, -5.0, 3.0], [3.0, 5.0, 6.0, 5.0, -5.0], [3.0, -4.0, -3.0, 7.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, -9.0, 0.0, -15.0, -21.0], [15.0, -21.0, 9.0, -15.0, 9.0], [9.0, 15.0, 18.0, 15.0, -15.0], [9.0, -12.0, -9.0, 21.0, 21.0]], [[-21.0, -21.0, 0.0, -35.0, -49.0], [35.0, -49.0, 21.0, -35.0, 21.0], [21.0, 35.0, 42.0, 35.0, -35.0], [21.0, -28.0, -21.0, 49.0, 49.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -12.0, 0.0, -20.0, -28.0], [20.0, -28.0, 12.0, -20.0, 12.0], [12.0, 20.0, 24.0, 20.0, -20.0], [12.0, -16.0, -12.0, 28.0, 28.0]], [[6.0, 6.0, 0.0, 10.0, 14.0], [-10.0, 14.0, -6.0, 10.0, -6.0], [-6.0, -10.0, -12.0, -10.0, 10.0], [-6.0, 8.0, 6.0, -14.0, -14.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -1.0], [-7.0, 6.0], [5.0, -4.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 3.0], [3.0, 3.0], [4.0, -7.0]])
      arg1=numpy.array([[[-1.0, -7.0, 0.0, -6.0, 2.0], [-1.0, -1.0, 5.0, 3.0, -7.0], [-4.0, 2.0, -7.0, -3.0, -6.0], [-4.0, 7.0, -6.0, -5.0, -7.0]], [[-5.0, 0.0, 0.0, 6.0, -2.0], [-6.0, -4.0, -5.0, 0.0, 0.0], [7.0, -7.0, 1.0, 1.0, 2.0], [-7.0, 0.0, -1.0, -4.0, -3.0]], [[-6.0, -2.0, -6.0, 0.0, 5.0], [6.0, -7.0, 0.0, 2.0, -7.0], [7.0, 4.0, -1.0, -4.0, -6.0], [-3.0, -7.0, -5.0, 3.0, -4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, -31.0, -30.0, -60.0, 45.0], [69.0, -10.0, 50.0, 19.0, -56.0], [-26.0, 75.0, -33.0, -36.0, -62.0], [22.0, -14.0, -36.0, 28.0, -20.0]], [[-5.0, 15.0, 24.0, 42.0, -34.0], [-59.0, 5.0, -35.0, -11.0, 35.0], [18.0, -60.0, 17.0, 25.0, 42.0], [-26.0, 21.0, 20.0, -31.0, 5.0]]])+(1.-msk_ref)*numpy.array([[[-35.0, 20.0, -24.0, 42.0, 6.0], [10.0, -36.0, -35.0, -4.0, 0.0], [65.0, -13.0, 27.0, -1.0, 6.0], [-17.0, -56.0, 1.0, 20.0, 3.0]], [[24.0, -7.0, 42.0, 0.0, -35.0], [-63.0, 34.0, 0.0, -5.0, 28.0], [-40.0, -43.0, -11.0, 22.0, 30.0], [-12.0, 70.0, 14.0, -48.0, -2.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 5.0], [3.0, -6.0], [7.0, -5.0]], [[5.0, 6.0], [-6.0, 2.0], [7.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -5.0], [0.0, 7.0], [3.0, -1.0]], [[0.0, -6.0], [-4.0, 4.0], [-1.0, -3.0]]])
      arg1=numpy.array([[[[2.0, -5.0, 3.0, 4.0, 0.0], [-1.0, 4.0, 4.0, 0.0, -3.0], [7.0, 6.0, 7.0, 3.0, 0.0], [0.0, 3.0, 0.0, -4.0, -6.0]], [[-3.0, -5.0, 2.0, 4.0, -7.0], [-7.0, -7.0, -2.0, 5.0, 0.0], [-5.0, 0.0, 4.0, 3.0, -5.0], [1.0, 6.0, -1.0, 5.0, -5.0]], [[-6.0, -2.0, -3.0, -6.0, -4.0], [-5.0, 4.0, 6.0, -2.0, 5.0], [0.0, -3.0, 7.0, -4.0, 0.0], [3.0, 3.0, 1.0, 7.0, 0.0]]], [[[-7.0, -6.0, 6.0, 0.0, -2.0], [1.0, -5.0, -2.0, -4.0, -5.0], [3.0, -7.0, -6.0, -5.0, 5.0], [-3.0, -5.0, 3.0, 0.0, 6.0]], [[3.0, 2.0, -2.0, -6.0, -1.0], [-5.0, -5.0, -1.0, 4.0, 0.0], [0.0, -2.0, 3.0, 5.0, 5.0], [-4.0, -4.0, 0.0, -2.0, 0.0]], [[-6.0, 0.0, -4.0, -7.0, 2.0], [0.0, 4.0, 0.0, 5.0, -7.0], [-3.0, 0.0, -7.0, -2.0, -5.0], [5.0, 7.0, 3.0, 5.0, 6.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-140.0, -86.0, 8.0, -31.0, -39.0], [-24.0, 52.0, 44.0, -8.0, -48.0], [0.0, -26.0, -15.0, -79.0, -55.0], [68.0, 96.0, 40.0, 99.0, 39.0]], [[40.0, -17.0, 62.0, 35.0, 42.0], [58.0, -10.0, -12.0, -51.0, -49.0], [92.0, -1.0, -33.0, 3.0, 85.0], [-62.0, -95.0, 10.0, -104.0, 18.0]]])+(1.-msk_ref)*numpy.array([[[-24.0, -14.0, 3.0, 13.0, -10.0], [5.0, 28.0, 22.0, -27.0, 22.0], [3.0, -1.0, 16.0, -30.0, -15.0], [20.0, 18.0, 0.0, 24.0, -6.0]], [[47.0, 36.0, -30.0, 11.0, -43.0], [-65.0, -75.0, -32.0, 62.0, 61.0], [-79.0, 7.0, 55.0, 66.0, -30.0], [-9.0, 17.0, -35.0, 25.0, -59.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, 7.0])+(1.-msk_arg0)*numpy.array([2.0, 5.0])
      arg1=numpy.array([[[2.0, 0.0], [-3.0, 7.0]], [[-3.0, 1.0], [2.0, 3.0]], [[-4.0, 0.0], [3.0, -7.0]], [[-2.0, 0.0], [-3.0, 6.0]], [[-3.0, -2.0], [5.0, -5.0]], [[0.0, -3.0], [3.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-12.0, 0.0], [18.0, -42.0]], [[18.0, -6.0], [-12.0, -18.0]], [[24.0, 0.0], [-18.0, 42.0]], [[12.0, 0.0], [18.0, -36.0]], [[18.0, 12.0], [-30.0, 30.0]], [[0.0, 18.0], [-18.0, 0.0]]], [[[14.0, 0.0], [-21.0, 49.0]], [[-21.0, 7.0], [14.0, 21.0]], [[-28.0, 0.0], [21.0, -49.0]], [[-14.0, 0.0], [-21.0, 42.0]], [[-21.0, -14.0], [35.0, -35.0]], [[0.0, -21.0], [21.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 0.0], [-6.0, 14.0]], [[-6.0, 2.0], [4.0, 6.0]], [[-8.0, 0.0], [6.0, -14.0]], [[-4.0, 0.0], [-6.0, 12.0]], [[-6.0, -4.0], [10.0, -10.0]], [[0.0, -6.0], [6.0, 0.0]]], [[[10.0, 0.0], [-15.0, 35.0]], [[-15.0, 5.0], [10.0, 15.0]], [[-20.0, 0.0], [15.0, -35.0]], [[-10.0, 0.0], [-15.0, 30.0]], [[-15.0, -10.0], [25.0, -25.0]], [[0.0, -15.0], [15.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -1.0], [2.0, 6.0], [-6.0, -3.0]])+(1.-msk_arg0)*numpy.array([[-6.0, -7.0], [6.0, -1.0], [-3.0, -1.0]])
      arg1=numpy.array([[[[-4.0, -7.0], [4.0, -6.0]], [[-2.0, -4.0], [1.0, -6.0]], [[-7.0, -3.0], [-1.0, 3.0]], [[0.0, -7.0], [-7.0, 1.0]], [[-1.0, 3.0], [-6.0, 0.0]], [[2.0, 5.0], [-6.0, -4.0]]], [[[-6.0, -2.0], [2.0, -1.0]], [[-4.0, 0.0], [-1.0, 3.0]], [[4.0, 1.0], [-6.0, 3.0]], [[-3.0, 6.0], [-1.0, 7.0]], [[0.0, -4.0], [7.0, 4.0]], [[-1.0, 4.0], [7.0, -6.0]]], [[[0.0, 2.0], [4.0, 4.0]], [[7.0, 4.0], [-7.0, -5.0]], [[4.0, -3.0], [2.0, 6.0]], [[-4.0, 6.0], [2.0, 7.0]], [[-2.0, -3.0], [-7.0, 1.0]], [[0.0, 0.0], [4.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-28.0, -44.0], [-4.0, -50.0]], [[-58.0, -40.0], [44.0, 12.0]], [[-44.0, 8.0], [-28.0, -18.0]], [[18.0, -52.0], [-42.0, -24.0]], [[8.0, 22.0], [32.0, 2.0]], [[6.0, 28.0], [-34.0, -52.0]]], [[[-32.0, -11.0], [-4.0, -12.0]], [[-43.0, -8.0], [14.0, 39.0]], [[19.0, 18.0], [-41.0, -3.0]], [[-6.0, 25.0], [-5.0, 20.0]], [[7.0, -18.0], [69.0, 21.0]], [[-8.0, 19.0], [36.0, -44.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, 24.0], [-24.0, 18.0]], [[-33.0, 12.0], [9.0, 69.0]], [[54.0, 33.0], [-36.0, -18.0]], [[-6.0, 60.0], [30.0, 15.0]], [[12.0, -33.0], [99.0, 21.0]], [[-18.0, -6.0], [66.0, -24.0]]], [[[34.0, 49.0], [-34.0, 39.0]], [[11.0, 24.0], [1.0, 44.0]], [[41.0, 23.0], [11.0, -30.0]], [[7.0, 37.0], [48.0, -21.0]], [[9.0, -14.0], [42.0, -5.0]], [[-13.0, -39.0], [31.0, 30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, -7.0])+(1.-msk_arg0)*numpy.array([0.0, 7.0])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(5.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([25.0, -35.0])+(1.-msk_ref)*numpy.array([0.0, 35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 3.0], [-5.0, 5.0], [-3.0, -4.0]])+(1.-msk_arg0)*numpy.array([[7.0, 6.0], [4.0, -7.0], [-3.0, 5.0]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([0.0, 2.0, -7.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([11.0, 38.0])+(1.-msk_ref)*numpy.array([29.0, -49.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, 1.0], [-3.0, -5.0], [0.0, -4.0]], [[7.0, 0.0], [-4.0, -6.0], [-6.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -2.0], [-1.0, 0.0], [0.0, 3.0]], [[-6.0, 2.0], [7.0, 6.0], [-7.0, 4.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-2.0, 5.0, 1.0], [6.0, 0.0, 4.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-9.0, -55.0])+(1.-msk_ref)*numpy.array([-73.0, 35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -2.0], [-4.0, 6.0], [-1.0, 2.0]], [[2.0, -5.0], [-3.0, 5.0], [0.0, 3.0]], [[-2.0, -6.0], [0.0, 4.0], [-1.0, -5.0]], [[-2.0, -2.0], [-5.0, -1.0], [-4.0, -2.0]]], [[[-5.0, -1.0], [1.0, -1.0], [2.0, -6.0]], [[-2.0, -2.0], [-6.0, 3.0], [7.0, 3.0]], [[6.0, 0.0], [-7.0, 3.0], [7.0, 3.0]], [[-7.0, 0.0], [-5.0, 4.0], [-2.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, -1.0], [6.0, -4.0], [-1.0, 7.0]], [[3.0, 4.0], [0.0, 4.0], [-1.0, 1.0]], [[2.0, 0.0], [0.0, 0.0], [-1.0, 6.0]], [[0.0, -6.0], [-1.0, -5.0], [7.0, 5.0]]], [[[0.0, -7.0], [-3.0, -7.0], [6.0, -6.0]], [[3.0, 5.0], [-7.0, -7.0], [1.0, 2.0]], [[5.0, -1.0], [0.0, -2.0], [5.0, 4.0]], [[-3.0, -5.0], [0.0, -6.0], [4.0, 3.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[2.0, -1.0, 2.0], [4.0, 0.0, -2.0], [2.0, 5.0, 4.0], [-5.0, -1.0, -3.0]], [[-1.0, -3.0, 5.0], [0.0, -3.0, 3.0], [-2.0, 1.0, -2.0], [-1.0, 2.0, 6.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([32.0, -54.0])+(1.-msk_ref)*numpy.array([66.0, 102.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, 0.0])+(1.-msk_arg0)*numpy.array([4.0, 3.0])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([4.0, 4.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[20.0, 20.0], [0.0, 0.0]])+(1.-msk_ref)*numpy.array([[16.0, 16.0], [12.0, 12.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -1.0], [0.0, 0.0], [-6.0, -5.0]])+(1.-msk_arg0)*numpy.array([[0.0, 3.0], [-7.0, -2.0], [-6.0, -6.0]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-7.0, -7.0], [-5.0, -3.0], [-1.0, 1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, -6.0], [12.0, 2.0]])+(1.-msk_ref)*numpy.array([[41.0, 15.0], [-5.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 1.0], [5.0, -7.0], [-3.0, -1.0]], [[-5.0, 4.0], [6.0, 2.0], [1.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, 7.0], [3.0, 4.0], [-4.0, 4.0]], [[6.0, -7.0], [-4.0, -1.0], [-1.0, 0.0]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, 0.0], [-6.0, 4.0], [4.0, 3.0]], [[0.0, -2.0], [4.0, -3.0], [-6.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, 3.0], [82.0, -45.0]])+(1.-msk_ref)*numpy.array([[-44.0, 0.0], [-12.0, 45.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 1.0], [0.0, 0.0], [-6.0, 2.0]], [[3.0, -3.0], [-7.0, 2.0], [-1.0, -7.0]], [[0.0, -7.0], [2.0, 7.0], [-4.0, 2.0]], [[-1.0, -6.0], [5.0, -4.0], [-1.0, 4.0]]], [[[-6.0, -4.0], [-3.0, -7.0], [0.0, -3.0]], [[5.0, 0.0], [4.0, 4.0], [-7.0, 2.0]], [[0.0, 1.0], [6.0, -6.0], [0.0, -6.0]], [[2.0, -7.0], [-7.0, -2.0], [-5.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -7.0], [-2.0, -6.0], [4.0, -5.0]], [[3.0, 4.0], [0.0, -2.0], [-5.0, 4.0]], [[6.0, 3.0], [6.0, -2.0], [4.0, 2.0]], [[1.0, -4.0], [0.0, 5.0], [3.0, 7.0]]], [[[-5.0, 6.0], [-6.0, 6.0], [7.0, -4.0]], [[-2.0, -1.0], [3.0, 5.0], [-4.0, -1.0]], [[-1.0, -1.0], [-2.0, 0.0], [5.0, 3.0]], [[0.0, -2.0], [-5.0, -7.0], [1.0, -2.0]]]])
      arg1=Symbol(shape=(2, 4, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-7.0, -7.0], [-7.0, 3.0], [-5.0, -1.0]], [[0.0, 3.0], [6.0, -5.0], [-4.0, 0.0]], [[-1.0, 0.0], [4.0, -5.0], [7.0, -2.0]], [[-7.0, 2.0], [-1.0, -1.0], [2.0, -4.0]]], [[[-3.0, 1.0], [5.0, 2.0], [-1.0, 5.0]], [[4.0, 2.0], [2.0, 0.0], [-1.0, 0.0]], [[-3.0, -1.0], [0.0, -4.0], [-7.0, 2.0]], [[5.0, 3.0], [3.0, -4.0], [-3.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[63.0, 117.0], [122.0, -114.0]])+(1.-msk_ref)*numpy.array([[-11.0, 1.0], [108.0, 54.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, 1.0])+(1.-msk_arg0)*numpy.array([4.0, 7.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-5.0, -6.0, 3.0, 4.0, 7.0], [-4.0, 7.0, -7.0, -2.0, -6.0], [0.0, -1.0, 4.0, 3.0, 5.0], [-3.0, -6.0, 6.0, -7.0, -2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-35.0, -42.0, 21.0, 28.0, 49.0], [-28.0, 49.0, -49.0, -14.0, -42.0], [0.0, -7.0, 28.0, 21.0, 35.0], [-21.0, -42.0, 42.0, -49.0, -14.0]], [[-5.0, -6.0, 3.0, 4.0, 7.0], [-4.0, 7.0, -7.0, -2.0, -6.0], [0.0, -1.0, 4.0, 3.0, 5.0], [-3.0, -6.0, 6.0, -7.0, -2.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, -24.0, 12.0, 16.0, 28.0], [-16.0, 28.0, -28.0, -8.0, -24.0], [0.0, -4.0, 16.0, 12.0, 20.0], [-12.0, -24.0, 24.0, -28.0, -8.0]], [[-35.0, -42.0, 21.0, 28.0, 49.0], [-28.0, 49.0, -49.0, -14.0, -42.0], [0.0, -7.0, 28.0, 21.0, 35.0], [-21.0, -42.0, 42.0, -49.0, -14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, 5.0], [-2.0, 0.0], [2.0, -7.0]])+(1.-msk_arg0)*numpy.array([[5.0, 2.0], [-1.0, 4.0], [0.0, -7.0]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-2.0, 3.0, 4.0, -1.0, 1.0], [-1.0, 6.0, 0.0, 1.0, 5.0], [7.0, -7.0, -6.0, 5.0, -2.0], [3.0, -3.0, -5.0, 0.0, -7.0]], [[-1.0, -3.0, 5.0, 7.0, 5.0], [7.0, 0.0, 5.0, -5.0, -2.0], [-6.0, 1.0, 0.0, 0.0, 1.0], [3.0, 6.0, 0.0, 6.0, 0.0]], [[-7.0, -2.0, -6.0, 6.0, 2.0], [4.0, 7.0, 5.0, -1.0, -4.0], [-3.0, 6.0, 2.0, -4.0, 3.0], [3.0, -6.0, -7.0, 7.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-26.0, 23.0, 6.0, -9.0, 1.0], [-13.0, 56.0, 0.0, 15.0, 31.0], [55.0, -39.0, -38.0, 27.0, -10.0], [21.0, -45.0, -49.0, 2.0, -57.0]], [[39.0, 29.0, 62.0, -47.0, -9.0], [-33.0, -19.0, -35.0, 12.0, 53.0], [56.0, -77.0, -44.0, 53.0, -31.0], [-6.0, 27.0, 24.0, -49.0, -7.0]]])+(1.-msk_ref)*numpy.array([[[-9.0, 18.0, 15.0, -12.0, 0.0], [-12.0, 30.0, -5.0, 10.0, 27.0], [41.0, -36.0, -30.0, 25.0, -11.0], [12.0, -21.0, -25.0, -6.0, -35.0]], [[41.0, 8.0, 70.0, -16.0, 8.0], [-2.0, -37.0, -15.0, -11.0, 30.0], [11.0, -52.0, -26.0, 38.0, -21.0], [-3.0, 60.0, 39.0, -25.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 1.0], [6.0, 5.0], [-7.0, 7.0]], [[2.0, 6.0], [6.0, -3.0], [5.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, -5.0], [1.0, 2.0], [-1.0, -5.0]], [[-6.0, -1.0], [2.0, 4.0], [4.0, -4.0]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[5.0, 4.0, 0.0, 2.0, 1.0], [2.0, -2.0, 1.0, -6.0, 2.0], [3.0, 2.0, 4.0, -6.0, 7.0], [1.0, 0.0, 1.0, -2.0, -3.0]], [[-4.0, 1.0, -3.0, 1.0, -2.0], [-1.0, 0.0, 5.0, 2.0, 3.0], [3.0, 3.0, 1.0, 4.0, -1.0], [5.0, 4.0, 1.0, 0.0, -2.0]], [[-5.0, -6.0, 3.0, 3.0, 3.0], [3.0, -2.0, -3.0, -4.0, 4.0], [-2.0, -2.0, 5.0, -2.0, 1.0], [-3.0, 2.0, 7.0, -1.0, 5.0]]], [[[4.0, 5.0, -7.0, -6.0, 2.0], [-3.0, 1.0, 2.0, -6.0, 3.0], [1.0, -3.0, 1.0, -6.0, -7.0], [3.0, -5.0, -3.0, 4.0, -2.0]], [[7.0, -5.0, -6.0, 7.0, 1.0], [-7.0, 0.0, 0.0, 4.0, -3.0], [-1.0, 2.0, -4.0, -6.0, -3.0], [-1.0, 4.0, 4.0, 4.0, -6.0]], [[-4.0, -1.0, 5.0, 0.0, 0.0], [4.0, 2.0, 0.0, 2.0, 7.0], [-7.0, 7.0, 4.0, 4.0, 7.0], [5.0, -7.0, 6.0, -6.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[31.0, 15.0, -64.0, 11.0, -25.0], [-59.0, 30.0, 53.0, 74.0, 9.0], [-13.0, 69.0, -39.0, 22.0, -24.0], [74.0, -11.0, 3.0, 13.0, -81.0]], [[-59.0, 9.0, -3.0, -29.0, 21.0], [33.0, -4.0, 17.0, -66.0, 93.0], [-8.0, 0.0, 74.0, -6.0, -3.0], [41.0, -29.0, 43.0, -15.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[-45.0, -53.0, 44.0, 40.0, -19.0], [8.0, 12.0, -8.0, 82.0, -5.0], [-43.0, 47.0, -18.0, 70.0, 34.0], [4.0, 12.0, 40.0, -31.0, 5.0]], [[32.0, -9.0, -58.0, 11.0, -22.0], [-68.0, 11.0, 18.0, 68.0, -67.0], [24.0, -11.0, -76.0, 14.0, -75.0], [-7.0, 47.0, -43.0, 51.0, -36.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, 3.0])+(1.-msk_arg0)*numpy.array([2.0, 3.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-7.0, -3.0], [3.0, -1.0]], [[6.0, -6.0], [-1.0, -4.0]], [[0.0, 2.0], [5.0, 4.0]], [[-5.0, 2.0], [4.0, -1.0]], [[7.0, 2.0], [0.0, 6.0]], [[-4.0, 0.0], [7.0, -3.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-21.0, -9.0], [9.0, -3.0]], [[18.0, -18.0], [-3.0, -12.0]], [[0.0, 6.0], [15.0, 12.0]], [[-15.0, 6.0], [12.0, -3.0]], [[21.0, 6.0], [0.0, 18.0]], [[-12.0, 0.0], [21.0, -9.0]]], [[[-21.0, -9.0], [9.0, -3.0]], [[18.0, -18.0], [-3.0, -12.0]], [[0.0, 6.0], [15.0, 12.0]], [[-15.0, 6.0], [12.0, -3.0]], [[21.0, 6.0], [0.0, 18.0]], [[-12.0, 0.0], [21.0, -9.0]]]])+(1.-msk_ref)*numpy.array([[[[-14.0, -6.0], [6.0, -2.0]], [[12.0, -12.0], [-2.0, -8.0]], [[0.0, 4.0], [10.0, 8.0]], [[-10.0, 4.0], [8.0, -2.0]], [[14.0, 4.0], [0.0, 12.0]], [[-8.0, 0.0], [14.0, -6.0]]], [[[-21.0, -9.0], [9.0, -3.0]], [[18.0, -18.0], [-3.0, -12.0]], [[0.0, 6.0], [15.0, 12.0]], [[-15.0, 6.0], [12.0, -3.0]], [[21.0, 6.0], [0.0, 18.0]], [[-12.0, 0.0], [21.0, -9.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 3.0], [5.0, 0.0], [-7.0, 1.0]])+(1.-msk_arg0)*numpy.array([[-1.0, -4.0], [5.0, 7.0], [-4.0, 5.0]])
      arg1=Symbol(shape=(3, 6, 2, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[2.0, 6.0], [-4.0, 1.0]], [[5.0, 1.0], [2.0, -2.0]], [[4.0, 1.0], [-7.0, -1.0]], [[-3.0, 0.0], [-3.0, 7.0]], [[5.0, -4.0], [6.0, -6.0]], [[1.0, 0.0], [0.0, 0.0]]], [[[-7.0, 5.0], [-5.0, 2.0]], [[4.0, 1.0], [0.0, 2.0]], [[2.0, -3.0], [-1.0, 4.0]], [[4.0, 0.0], [4.0, 4.0]], [[-2.0, -1.0], [-1.0, -6.0]], [[0.0, 6.0], [0.0, -5.0]]], [[[-2.0, 0.0], [7.0, 4.0]], [[5.0, 7.0], [-6.0, -3.0]], [[-7.0, -2.0], [-1.0, -7.0]], [[-7.0, 0.0], [0.0, 0.0]], [[4.0, -3.0], [2.0, -5.0]], [[-7.0, 2.0], [2.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-29.0, 1.0], [-58.0, -22.0]], [[-35.0, -48.0], [34.0, 39.0]], [[43.0, -5.0], [30.0, 73.0]], [[81.0, 0.0], [32.0, -8.0]], [[-58.0, 32.0], [-43.0, 29.0]], [[45.0, 16.0], [-14.0, -25.0]]], [[[4.0, 18.0], [-5.0, 7.0]], [[20.0, 10.0], [0.0, -9.0]], [[5.0, 1.0], [-22.0, -10.0]], [[-16.0, 0.0], [-9.0, 21.0]], [[19.0, -15.0], [20.0, -23.0]], [[-4.0, 2.0], [2.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-29.0, 19.0], [-49.0, -7.0]], [[-5.0, -24.0], [22.0, 24.0]], [[34.0, -8.0], [6.0, 49.0]], [[51.0, 0.0], [23.0, 13.0]], [[-31.0, 11.0], [-19.0, -4.0]], [[27.0, 22.0], [-8.0, -25.0]]], [[[-67.0, 11.0], [16.0, 30.0]], [[33.0, 38.0], [-38.0, 7.0]], [[-37.0, -35.0], [16.0, -3.0]], [[5.0, 0.0], [40.0, 0.0]], [[-14.0, -6.0], [-21.0, -43.0]], [[-39.0, 52.0], [10.0, -35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([2.0, 1.0])+(1.-msk_arg0)*numpy.array([1.0, 6.0])
      arg1=Data(7.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([14.0, 7.0])+(1.-msk_ref)*numpy.array([7.0, 42.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 7.0], [-1.0, 7.0], [-7.0, 1.0]])+(1.-msk_arg0)*numpy.array([[-7.0, 4.0], [6.0, -1.0], [4.0, 7.0]])
      arg1=Data(numpy.array([-2.0, 2.0, 2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-22.0, 2.0])+(1.-msk_ref)*numpy.array([34.0, 4.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -6.0], [7.0, 5.0], [5.0, 1.0]], [[0.0, 3.0], [4.0, -2.0], [1.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, 2.0], [5.0, -6.0], [-7.0, 4.0]], [[7.0, -2.0], [-1.0, 0.0], [-7.0, 7.0]]])
      arg1=Data(numpy.array([[-5.0, -5.0, -2.0], [0.0, 0.0, 3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-42.0, 3.0])+(1.-msk_ref)*numpy.array([-12.0, 33.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -5.0], [4.0, -1.0], [4.0, -5.0]], [[0.0, -7.0], [5.0, 0.0], [0.0, -5.0]], [[2.0, 2.0], [-7.0, 7.0], [-4.0, -4.0]], [[0.0, 4.0], [0.0, 5.0], [0.0, -2.0]]], [[[1.0, -6.0], [-4.0, -1.0], [5.0, -7.0]], [[-7.0, 4.0], [2.0, 0.0], [-3.0, 3.0]], [[0.0, 0.0], [-5.0, 4.0], [7.0, 6.0]], [[-4.0, -2.0], [4.0, -2.0], [5.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, -2.0], [-3.0, -2.0], [-4.0, 0.0]], [[-5.0, 5.0], [2.0, -4.0], [3.0, -7.0]], [[3.0, 5.0], [-5.0, -2.0], [-5.0, -2.0]], [[-6.0, 3.0], [5.0, 5.0], [-2.0, -1.0]]], [[[-1.0, -1.0], [0.0, -4.0], [-3.0, 5.0]], [[-2.0, 1.0], [6.0, -7.0], [-2.0, 2.0]], [[2.0, 7.0], [0.0, 0.0], [-1.0, -3.0]], [[0.0, 0.0], [5.0, 1.0], [2.0, -4.0]]]])
      arg1=Data(numpy.array([[[0.0, 2.0, -4.0], [7.0, -2.0, 4.0], [-7.0, 6.0, 2.0], [1.0, 3.0, -5.0]], [[-1.0, 4.0, 4.0], [4.0, -5.0, -3.0], [2.0, -4.0, -3.0], [3.0, -2.0, -6.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-159.0, -87.0])+(1.-msk_ref)*numpy.array([-117.0, 66.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([2.0, -4.0])+(1.-msk_arg0)*numpy.array([2.0, -3.0])
      arg1=Data(numpy.array([2.0, 0.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 0.0], [-8.0, 0.0]])+(1.-msk_ref)*numpy.array([[4.0, 0.0], [-6.0, 0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, -2.0], [6.0, 0.0], [3.0, 4.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 4.0], [-3.0, -7.0], [0.0, -6.0]])
      arg1=Data(numpy.array([[-7.0, 6.0], [4.0, 6.0], [0.0, 0.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-18.0, 72.0], [14.0, -12.0]])+(1.-msk_ref)*numpy.array([[2.0, -30.0], [-56.0, -18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 0.0], [2.0, 7.0], [-7.0, -4.0]], [[0.0, 2.0], [6.0, 7.0], [0.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -7.0], [-5.0, 0.0], [1.0, 4.0]], [[6.0, 7.0], [3.0, -1.0], [-7.0, -2.0]]])
      arg1=Data(numpy.array([[[1.0, 2.0], [4.0, 2.0], [-3.0, -1.0]], [[5.0, 6.0], [-6.0, 7.0], [4.0, 0.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-12.0, 43.0], [16.0, 79.0]])+(1.-msk_ref)*numpy.array([[-37.0, 50.0], [14.0, 17.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 7.0], [4.0, 2.0], [-1.0, 4.0]], [[-5.0, 0.0], [5.0, -3.0], [7.0, -4.0]], [[2.0, -7.0], [-5.0, -2.0], [-4.0, -1.0]], [[-1.0, 2.0], [-5.0, 7.0], [2.0, -5.0]]], [[[-7.0, 0.0], [-7.0, -4.0], [-2.0, 5.0]], [[1.0, 1.0], [-6.0, -4.0], [-3.0, 1.0]], [[2.0, -6.0], [0.0, -3.0], [-7.0, -2.0]], [[2.0, -1.0], [4.0, 2.0], [-6.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 5.0], [-4.0, -6.0], [4.0, 2.0]], [[-5.0, -3.0], [-1.0, 1.0], [5.0, 5.0]], [[-7.0, -1.0], [5.0, -7.0], [-7.0, 7.0]], [[6.0, -2.0], [-6.0, -5.0], [5.0, 7.0]]], [[[4.0, -1.0], [6.0, 0.0], [5.0, 7.0]], [[-4.0, -7.0], [0.0, -1.0], [0.0, -1.0]], [[-1.0, 5.0], [0.0, 3.0], [7.0, 7.0]], [[6.0, -5.0], [-1.0, 7.0], [2.0, -3.0]]]])
      arg1=Data(numpy.array([[[[5.0, 0.0], [5.0, -2.0], [3.0, 3.0]], [[5.0, 5.0], [4.0, 0.0], [-7.0, -3.0]], [[-3.0, -4.0], [6.0, 3.0], [-4.0, -3.0]], [[-6.0, 4.0], [-5.0, -3.0], [7.0, 7.0]]], [[[7.0, 4.0], [3.0, 6.0], [1.0, 1.0]], [[1.0, 0.0], [-4.0, -6.0], [1.0, -6.0]], [[-3.0, 6.0], [6.0, -2.0], [2.0, 3.0]], [[3.0, 2.0], [-6.0, -2.0], [3.0, 7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-118.0, -116.0], [2.0, -32.0]])+(1.-msk_ref)*numpy.array([[110.0, 221.0], [-79.0, 21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, 5.0])+(1.-msk_arg0)*numpy.array([6.0, 4.0])
      arg1=Data(numpy.array([[-3.0, 6.0, 2.0, -7.0, -3.0], [-4.0, 2.0, -5.0, 5.0, 0.0], [3.0, -4.0, 4.0, 7.0, -3.0], [0.0, 2.0, 2.0, 7.0, -5.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, -18.0, -6.0, 21.0, 9.0], [12.0, -6.0, 15.0, -15.0, 0.0], [-9.0, 12.0, -12.0, -21.0, 9.0], [0.0, -6.0, -6.0, -21.0, 15.0]], [[-15.0, 30.0, 10.0, -35.0, -15.0], [-20.0, 10.0, -25.0, 25.0, 0.0], [15.0, -20.0, 20.0, 35.0, -15.0], [0.0, 10.0, 10.0, 35.0, -25.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, 36.0, 12.0, -42.0, -18.0], [-24.0, 12.0, -30.0, 30.0, 0.0], [18.0, -24.0, 24.0, 42.0, -18.0], [0.0, 12.0, 12.0, 42.0, -30.0]], [[-12.0, 24.0, 8.0, -28.0, -12.0], [-16.0, 8.0, -20.0, 20.0, 0.0], [12.0, -16.0, 16.0, 28.0, -12.0], [0.0, 8.0, 8.0, 28.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -4.0], [2.0, 3.0], [-3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 1.0], [0.0, -6.0], [-7.0, 0.0]])
      arg1=Data(numpy.array([[[1.0, 0.0, 0.0, 2.0, -1.0], [6.0, -3.0, 0.0, 3.0, -3.0], [-4.0, 3.0, -7.0, -4.0, -5.0], [0.0, 2.0, 5.0, -5.0, -7.0]], [[0.0, 4.0, 2.0, -1.0, 4.0], [-7.0, -5.0, 3.0, -6.0, 3.0], [7.0, -5.0, -5.0, -6.0, 4.0], [-2.0, -6.0, -4.0, 2.0, 1.0]], [[-7.0, 6.0, -2.0, 7.0, -3.0], [4.0, -3.0, -3.0, 3.0, 7.0], [7.0, 0.0, 2.0, 6.0, 6.0], [-6.0, -6.0, 0.0, -4.0, 1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[24.0, -10.0, 10.0, -17.0, 14.0], [-8.0, -10.0, 15.0, -12.0, -24.0], [-19.0, -1.0, -37.0, -42.0, -25.0], [14.0, 12.0, 7.0, 1.0, -22.0]], [[-4.0, 12.0, 6.0, -11.0, 16.0], [-45.0, -3.0, 9.0, -30.0, 21.0], [37.0, -27.0, 13.0, -2.0, 32.0], [-6.0, -26.0, -32.0, 26.0, 31.0]]])+(1.-msk_ref)*numpy.array([[[48.0, -42.0, 14.0, -51.0, 22.0], [-34.0, 24.0, 21.0, -24.0, -46.0], [-45.0, -3.0, -7.0, -38.0, -37.0], [42.0, 40.0, -5.0, 33.0, 0.0]], [[1.0, -24.0, -12.0, 8.0, -25.0], [48.0, 27.0, -18.0, 39.0, -21.0], [-46.0, 33.0, 23.0, 32.0, -29.0], [12.0, 38.0, 29.0, -17.0, -13.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, -2.0], [5.0, 3.0], [4.0, 4.0]], [[0.0, -4.0], [-6.0, 5.0], [-2.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 1.0], [0.0, 5.0], [7.0, -3.0]], [[-3.0, -4.0], [6.0, 5.0], [0.0, 1.0]]])
      arg1=Data(numpy.array([[[[-5.0, 2.0, 2.0, 6.0, 5.0], [5.0, -4.0, 5.0, -7.0, 6.0], [5.0, -5.0, 5.0, -1.0, 2.0], [1.0, 4.0, 6.0, 4.0, -5.0]], [[-2.0, -2.0, -1.0, 4.0, 0.0], [-4.0, 6.0, 2.0, 7.0, -4.0], [4.0, 6.0, 1.0, -3.0, 3.0], [2.0, -4.0, 3.0, -6.0, -5.0]], [[5.0, -5.0, -6.0, 0.0, 0.0], [-2.0, -1.0, 7.0, 7.0, 7.0], [-4.0, 6.0, 1.0, -3.0, -5.0], [-1.0, -6.0, -5.0, 1.0, 2.0]]], [[[-7.0, 5.0, -3.0, 6.0, 7.0], [5.0, -1.0, -7.0, 0.0, 1.0], [2.0, -5.0, 3.0, 1.0, -1.0], [5.0, 2.0, 3.0, -5.0, 7.0]], [[-3.0, -5.0, 2.0, -4.0, -6.0], [0.0, 5.0, 4.0, 3.0, 4.0], [-2.0, 6.0, -6.0, -5.0, -3.0], [2.0, 5.0, 4.0, 5.0, -1.0]], [[4.0, 3.0, -2.0, -3.0, 0.0], [6.0, -7.0, 0.0, 1.0, 4.0], [-4.0, 6.0, 6.0, 2.0, -7.0], [-5.0, -2.0, -1.0, 2.0, -6.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[25.0, -8.0, -39.0, 44.0, 31.0], [-45.0, 14.0, 9.0, 50.0, -30.0], [19.0, 11.0, 28.0, 0.0, 25.0], [3.0, -74.0, -33.0, -64.0, 6.0]], [[49.0, -66.0, -15.0, -53.0, -68.0], [-32.0, 30.0, 72.0, 81.0, 32.0], [-44.0, 120.0, -27.0, -42.0, -47.0], [-25.0, -33.0, -18.0, 29.0, -48.0]]])+(1.-msk_ref)*numpy.array([[[23.0, -74.0, -15.0, -24.0, -42.0], [-14.0, 14.0, 109.0, 46.0, 88.0], [-31.0, 78.0, -23.0, -57.0, -44.0], [-7.0, -6.0, -2.0, 64.0, -28.0]], [[-13.0, -35.0, 35.0, -21.0, -53.0], [-23.0, 51.0, 42.0, 23.0, -15.0], [15.0, 63.0, -29.0, -34.0, 14.0], [-1.0, 17.0, 43.0, 18.0, -75.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, -6.0])+(1.-msk_arg0)*numpy.array([-7.0, 5.0])
      arg1=Data(numpy.array([[[-1.0, 6.0], [-4.0, 7.0]], [[6.0, -6.0], [-6.0, 4.0]], [[5.0, 3.0], [3.0, 1.0]], [[-4.0, 4.0], [-3.0, -1.0]], [[-2.0, 2.0], [2.0, 6.0]], [[7.0, -1.0], [-1.0, -3.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-5.0, 30.0], [-20.0, 35.0]], [[30.0, -30.0], [-30.0, 20.0]], [[25.0, 15.0], [15.0, 5.0]], [[-20.0, 20.0], [-15.0, -5.0]], [[-10.0, 10.0], [10.0, 30.0]], [[35.0, -5.0], [-5.0, -15.0]]], [[[6.0, -36.0], [24.0, -42.0]], [[-36.0, 36.0], [36.0, -24.0]], [[-30.0, -18.0], [-18.0, -6.0]], [[24.0, -24.0], [18.0, 6.0]], [[12.0, -12.0], [-12.0, -36.0]], [[-42.0, 6.0], [6.0, 18.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, -42.0], [28.0, -49.0]], [[-42.0, 42.0], [42.0, -28.0]], [[-35.0, -21.0], [-21.0, -7.0]], [[28.0, -28.0], [21.0, 7.0]], [[14.0, -14.0], [-14.0, -42.0]], [[-49.0, 7.0], [7.0, 21.0]]], [[[-5.0, 30.0], [-20.0, 35.0]], [[30.0, -30.0], [-30.0, 20.0]], [[25.0, 15.0], [15.0, 5.0]], [[-20.0, 20.0], [-15.0, -5.0]], [[-10.0, 10.0], [10.0, 30.0]], [[35.0, -5.0], [-5.0, -15.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -3.0], [2.0, 0.0], [-5.0, 2.0]])+(1.-msk_arg0)*numpy.array([[1.0, -2.0], [-4.0, 7.0], [1.0, 0.0]])
      arg1=Data(numpy.array([[[[1.0, 3.0], [-3.0, 0.0]], [[0.0, 6.0], [0.0, 0.0]], [[-2.0, -3.0], [-5.0, -3.0]], [[-1.0, 7.0], [1.0, -5.0]], [[-6.0, -3.0], [-1.0, 0.0]], [[-2.0, 0.0], [-3.0, 6.0]]], [[[0.0, -6.0], [-7.0, 4.0]], [[2.0, 1.0], [-1.0, -7.0]], [[1.0, -2.0], [-5.0, 5.0]], [[2.0, 0.0], [-1.0, 1.0]], [[7.0, 3.0], [3.0, 4.0]], [[-4.0, -2.0], [-2.0, -7.0]]], [[[6.0, 1.0], [-7.0, -2.0]], [[6.0, 0.0], [-7.0, -7.0]], [[6.0, -3.0], [-6.0, -3.0]], [[-4.0, -2.0], [6.0, -2.0]], [[7.0, -4.0], [7.0, -7.0]], [[7.0, 0.0], [6.0, 3.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-36.0, -35.0], [39.0, 18.0]], [[-26.0, -34.0], [33.0, 21.0]], [[-16.0, 29.0], [50.0, 43.0]], [[30.0, -32.0], [-38.0, 42.0]], [[15.0, 44.0], [-23.0, 43.0]], [[-31.0, -4.0], [-16.0, -65.0]]], [[[9.0, -7.0], [-5.0, -4.0]], [[12.0, -18.0], [-14.0, -14.0]], [[18.0, 3.0], [3.0, 3.0]], [[-5.0, -25.0], [9.0, 11.0]], [[32.0, 1.0], [17.0, -14.0]], [[20.0, 0.0], [21.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, 28.0], [18.0, -18.0]], [[-2.0, 2.0], [-3.0, 21.0]], [[0.0, 2.0], [9.0, -26.0]], [[-13.0, 5.0], [11.0, -11.0]], [[-27.0, -19.0], [-6.0, -23.0]], [[21.0, 8.0], [11.0, 37.0]]], [[[-2.0, -48.0], [-43.0, 28.0]], [[14.0, -5.0], [-7.0, -49.0]], [[11.0, -8.0], [-25.0, 41.0]], [[16.0, -14.0], [-9.0, 17.0]], [[61.0, 27.0], [23.0, 28.0]], [[-24.0, -14.0], [-8.0, -61.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, -5.0])+(1.-msk_arg0)*numpy.array([-2.0, -2.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(6.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([0.0, -0.0])+(1.-msk_ref)*numpy.array([-12.0, -12.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, 4.0], [0.0, -5.0], [-7.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-5.0, 6.0], [-4.0, 3.0], [-6.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, 1.0, 3.0])+(1.-msk_arg1)*numpy.array([1.0, 1.0, 5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([9.0, -29.0])+(1.-msk_ref)*numpy.array([-39.0, 19.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, -2.0], [-6.0, 1.0], [-2.0, -5.0]], [[4.0, 4.0], [-4.0, 0.0], [2.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 7.0], [2.0, 6.0], [3.0, -6.0]], [[2.0, 7.0], [-7.0, 0.0], [7.0, -1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, -1.0, 3.0], [1.0, 6.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -5.0, 0.0], [5.0, -2.0, 0.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-35.0, -12.0])+(1.-msk_ref)*numpy.array([42.0, -23.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 3.0], [-7.0, 4.0], [-6.0, 6.0]], [[-7.0, -4.0], [5.0, -7.0], [-1.0, -5.0]], [[-5.0, 0.0], [-7.0, -4.0], [-5.0, 3.0]], [[-2.0, 5.0], [-1.0, 6.0], [0.0, 0.0]]], [[[6.0, -7.0], [-5.0, -4.0], [-6.0, -1.0]], [[1.0, 2.0], [-5.0, -6.0], [4.0, 4.0]], [[0.0, -2.0], [0.0, 2.0], [-2.0, -6.0]], [[-4.0, 2.0], [4.0, 0.0], [-4.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-7.0, 5.0], [5.0, 7.0], [-6.0, -2.0]], [[1.0, 1.0], [0.0, 2.0], [2.0, 5.0]], [[1.0, 4.0], [-1.0, -2.0], [2.0, 2.0]], [[3.0, 4.0], [2.0, -5.0], [-6.0, 0.0]]], [[[5.0, 2.0], [-2.0, 0.0], [0.0, -6.0]], [[6.0, -2.0], [0.0, 6.0], [6.0, 5.0]], [[-2.0, -1.0], [-1.0, 6.0], [-6.0, 2.0]], [[2.0, -7.0], [-3.0, 4.0], [4.0, 6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, -6.0, 0.0], [3.0, -3.0, 6.0], [3.0, 6.0, 0.0], [-5.0, -7.0, -1.0]], [[4.0, -1.0, 0.0], [4.0, 6.0, -4.0], [-1.0, -1.0, 3.0], [2.0, -3.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 0.0, -5.0], [-7.0, 0.0, 0.0], [-1.0, -1.0, 6.0], [-2.0, -1.0, -3.0]], [[2.0, -6.0, 0.0], [7.0, 4.0, -5.0], [-4.0, 3.0, 1.0], [7.0, 6.0, 4.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-100.0, -209.0])+(1.-msk_ref)*numpy.array([83.0, 27.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-6.0, 3.0])+(1.-msk_arg0)*numpy.array([4.0, -6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-7.0, -2.0])+(1.-msk_arg1)*numpy.array([-6.0, -1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[42.0, 12.0], [-21.0, -6.0]])+(1.-msk_ref)*numpy.array([[-24.0, -4.0], [36.0, 6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, 2.0], [-5.0, -1.0], [-2.0, 6.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 4.0], [3.0, 5.0], [-2.0, -5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, 0.0], [6.0, 6.0], [-2.0, 7.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 3.0], [0.0, -2.0], [-2.0, -1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-32.0, -44.0], [-6.0, 36.0]])+(1.-msk_ref)*numpy.array([[8.0, -10.0], [2.0, 7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -5.0], [-6.0, -7.0], [0.0, -4.0]], [[1.0, 2.0], [6.0, 6.0], [-4.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[4.0, 1.0], [-6.0, 4.0], [-3.0, 7.0]], [[-2.0, 6.0], [4.0, 7.0], [7.0, -4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, -6.0], [-6.0, 5.0], [0.0, 4.0]], [[-4.0, -4.0], [-3.0, 7.0], [-4.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, -3.0], [-6.0, 6.0], [0.0, 0.0]], [[-5.0, 0.0], [0.0, 0.0], [0.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, 36.0], [27.0, 41.0]])+(1.-msk_ref)*numpy.array([[18.0, -48.0], [-61.0, 21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, 5.0], [-3.0, -3.0], [-4.0, 5.0]], [[7.0, -4.0], [-1.0, 1.0], [-7.0, -6.0]], [[-6.0, -4.0], [-3.0, -7.0], [0.0, 5.0]], [[-1.0, -5.0], [-4.0, 5.0], [0.0, -3.0]]], [[[1.0, -2.0], [-4.0, -1.0], [-4.0, 0.0]], [[-1.0, 2.0], [-4.0, -6.0], [1.0, -1.0]], [[0.0, 2.0], [-7.0, -5.0], [1.0, -4.0]], [[7.0, 6.0], [1.0, 0.0], [-6.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -6.0], [-2.0, 0.0], [0.0, 4.0]], [[6.0, 2.0], [4.0, -6.0], [4.0, 1.0]], [[6.0, 5.0], [0.0, 6.0], [-6.0, 0.0]], [[-6.0, 0.0], [4.0, 1.0], [-4.0, 0.0]]], [[[4.0, -7.0], [-7.0, 0.0], [2.0, -4.0]], [[4.0, 6.0], [1.0, 7.0], [3.0, 7.0]], [[-2.0, -6.0], [-2.0, -2.0], [-6.0, 7.0]], [[1.0, 7.0], [1.0, 7.0], [-7.0, 2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, 0.0], [6.0, 5.0], [-6.0, 7.0]], [[6.0, -1.0], [1.0, -6.0], [6.0, -3.0]], [[7.0, -3.0], [-5.0, -1.0], [4.0, -5.0]], [[-5.0, -1.0], [0.0, 5.0], [1.0, 6.0]]], [[[4.0, 7.0], [-4.0, 2.0], [-1.0, -1.0]], [[0.0, -6.0], [5.0, 6.0], [7.0, 0.0]], [[5.0, 5.0], [6.0, -6.0], [2.0, 0.0]], [[-7.0, 2.0], [-3.0, -1.0], [-2.0, 1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, 5.0], [-2.0, 3.0], [-7.0, 0.0]], [[-5.0, 5.0], [-3.0, 4.0], [-4.0, -3.0]], [[1.0, -4.0], [-6.0, 6.0], [6.0, 4.0]], [[3.0, 6.0], [0.0, -2.0], [4.0, 2.0]]], [[[-6.0, -4.0], [1.0, 0.0], [0.0, -7.0]], [[-6.0, -6.0], [-7.0, 4.0], [0.0, 0.0]], [[0.0, 1.0], [-5.0, -7.0], [-1.0, -2.0]], [[-6.0, 1.0], [0.0, 6.0], [0.0, 2.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-87.0, 13.0], [-164.0, 30.0]])+(1.-msk_ref)*numpy.array([[-172.0, -95.0], [-131.0, 62.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, -2.0])+(1.-msk_arg0)*numpy.array([4.0, 5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 0.0, 5.0, -7.0, 5.0], [1.0, 2.0, 3.0, 3.0, 7.0], [4.0, 1.0, 0.0, 7.0, -2.0], [-6.0, -1.0, -4.0, 4.0, 2.0]])+(1.-msk_arg1)*numpy.array([[5.0, 0.0, 0.0, -1.0, -6.0], [0.0, 0.0, -6.0, 1.0, 0.0], [1.0, -6.0, -4.0, 0.0, -3.0], [1.0, -7.0, 0.0, -2.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-3.0, 0.0, 5.0, -7.0, 5.0], [1.0, 2.0, 3.0, 3.0, 7.0], [4.0, 1.0, 0.0, 7.0, -2.0], [-6.0, -1.0, -4.0, 4.0, 2.0]], [[6.0, 0.0, -10.0, 14.0, -10.0], [-2.0, -4.0, -6.0, -6.0, -14.0], [-8.0, -2.0, 0.0, -14.0, 4.0], [12.0, 2.0, 8.0, -8.0, -4.0]]])+(1.-msk_ref)*numpy.array([[[20.0, 0.0, 0.0, -4.0, -24.0], [0.0, 0.0, -24.0, 4.0, 0.0], [4.0, -24.0, -16.0, 0.0, -12.0], [4.0, -28.0, 0.0, -8.0, -16.0]], [[25.0, 0.0, 0.0, -5.0, -30.0], [0.0, 0.0, -30.0, 5.0, 0.0], [5.0, -30.0, -20.0, 0.0, -15.0], [5.0, -35.0, 0.0, -10.0, -20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -2.0], [1.0, -4.0], [1.0, -3.0]])+(1.-msk_arg0)*numpy.array([[4.0, -5.0], [0.0, 3.0], [-5.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -5.0, 0.0, -1.0, 0.0], [-5.0, 0.0, 0.0, 5.0, -3.0], [-7.0, 2.0, 5.0, 6.0, -1.0], [-5.0, 7.0, -4.0, 5.0, 6.0]], [[-7.0, 1.0, -6.0, 6.0, 1.0], [-2.0, -4.0, -4.0, 2.0, -5.0], [-1.0, -4.0, -5.0, -4.0, 7.0], [-1.0, 0.0, -1.0, 6.0, -1.0]], [[-4.0, 6.0, 7.0, -2.0, 2.0], [1.0, 1.0, -4.0, 1.0, -2.0], [1.0, -4.0, -5.0, 3.0, 7.0], [-1.0, 2.0, -3.0, 7.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[5.0, 6.0, -4.0, -4.0, -1.0], [0.0, 6.0, -1.0, 0.0, -4.0], [7.0, 2.0, 3.0, 0.0, 3.0], [3.0, -2.0, 0.0, 1.0, -6.0]], [[4.0, -1.0, -6.0, 0.0, 0.0], [5.0, -6.0, 4.0, -4.0, -2.0], [5.0, 1.0, -4.0, -2.0, 1.0], [4.0, -6.0, 3.0, 2.0, -3.0]], [[6.0, -1.0, 2.0, 7.0, -2.0], [-6.0, 0.0, 4.0, -5.0, -5.0], [6.0, 0.0, 5.0, 0.0, 0.0], [7.0, -5.0, 7.0, 0.0, 7.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-11.0, 7.0, 1.0, 4.0, 3.0], [-1.0, -3.0, -8.0, 3.0, -7.0], [0.0, -8.0, -10.0, -1.0, 14.0], [-2.0, 2.0, -4.0, 13.0, 5.0]], [[40.0, -12.0, 3.0, -16.0, -10.0], [15.0, 13.0, 28.0, -21.0, 32.0], [15.0, 24.0, 25.0, -5.0, -47.0], [17.0, -20.0, 21.0, -55.0, -26.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 29.0, -26.0, -51.0, 6.0], [30.0, 24.0, -24.0, 25.0, 9.0], [-2.0, 8.0, -13.0, 0.0, 12.0], [-23.0, 17.0, -35.0, 4.0, -59.0]], [[-55.0, -26.0, -12.0, -29.0, 19.0], [57.0, -48.0, -11.0, 23.0, 49.0], [-62.0, -7.0, -62.0, -6.0, -12.0], [-52.0, 27.0, -40.0, 1.0, -28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, -4.0], [6.0, 0.0], [0.0, -6.0]], [[4.0, 4.0], [3.0, 5.0], [-6.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 7.0], [-3.0, 2.0], [-7.0, -4.0]], [[5.0, -5.0], [-5.0, -5.0], [-1.0, 4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -3.0, -1.0, -5.0, -6.0], [5.0, 1.0, -6.0, 0.0, 3.0], [-2.0, -6.0, -3.0, 3.0, 5.0], [3.0, 4.0, 3.0, 0.0, 7.0]], [[-5.0, 3.0, 1.0, 1.0, -5.0], [4.0, 2.0, -7.0, 5.0, 5.0], [0.0, -6.0, -7.0, -5.0, 7.0], [-5.0, -6.0, 2.0, -7.0, 0.0]], [[-2.0, 3.0, -6.0, -4.0, -3.0], [2.0, -3.0, -7.0, -5.0, 2.0], [1.0, 3.0, 5.0, -6.0, 0.0], [4.0, 7.0, 5.0, -1.0, -3.0]]], [[[-7.0, 5.0, -5.0, -2.0, 4.0], [6.0, -5.0, 3.0, 5.0, 4.0], [2.0, -2.0, -6.0, -4.0, -4.0], [1.0, 0.0, -1.0, -3.0, 4.0]], [[-6.0, 6.0, -3.0, 0.0, -1.0], [0.0, 4.0, -1.0, -6.0, -3.0], [-7.0, -3.0, -4.0, -6.0, 0.0], [-4.0, 0.0, 0.0, -2.0, 0.0]], [[0.0, -7.0, -2.0, 5.0, -7.0], [0.0, 7.0, 2.0, 4.0, 0.0], [4.0, -7.0, 6.0, -6.0, -2.0], [5.0, 7.0, -3.0, 6.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, 3.0, -4.0, -5.0, 0.0], [0.0, -5.0, 0.0, -6.0, 7.0], [3.0, -5.0, -6.0, -1.0, 0.0], [-4.0, 2.0, -1.0, -1.0, 0.0]], [[-2.0, 1.0, -2.0, -7.0, -4.0], [1.0, -2.0, 0.0, 3.0, 1.0], [2.0, 6.0, 1.0, 7.0, 7.0], [0.0, 5.0, 0.0, -5.0, 0.0]], [[2.0, 6.0, 2.0, -4.0, 0.0], [2.0, -2.0, -2.0, -5.0, 2.0], [0.0, -3.0, -5.0, 1.0, 5.0], [-7.0, -5.0, -6.0, 1.0, -3.0]]], [[[4.0, 7.0, -5.0, 7.0, 0.0], [0.0, 2.0, 5.0, -2.0, 0.0], [1.0, 4.0, -6.0, -2.0, 0.0], [4.0, 0.0, 5.0, 5.0, 0.0]], [[2.0, 1.0, 1.0, 1.0, -6.0], [4.0, -6.0, -3.0, 1.0, 1.0], [-7.0, -4.0, 6.0, -4.0, 4.0], [-5.0, 6.0, 7.0, -3.0, 1.0]], [[-5.0, 6.0, -7.0, 5.0, -3.0], [0.0, -2.0, -4.0, -7.0, 5.0], [-1.0, -5.0, -5.0, 0.0, -1.0], [0.0, 1.0, 7.0, -2.0, -7.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-86.0, 92.0, -13.0, -42.0, 13.0], [58.0, -36.0, -57.0, 8.0, 43.0], [-41.0, -23.0, -120.0, -22.0, 48.0], [-62.0, -70.0, 32.0, -96.0, 72.0]], [[-26.0, 72.0, 13.0, 16.0, 81.0], [-8.0, -14.0, 65.0, 4.0, -23.0], [-41.0, 11.0, -86.0, 2.0, -28.0], [-72.0, -86.0, -34.0, -40.0, 34.0]]])+(1.-msk_ref)*numpy.array([[[-7.0, -15.0, -39.0, 64.0, 45.0], [-37.0, 52.0, 58.0, 6.0, -13.0], [41.0, 38.0, -35.0, -20.0, -75.0], [86.0, -7.0, 23.0, 48.0, 23.0]], [[-111.0, -17.0, -48.0, -53.0, 10.0], [-26.0, -19.0, -18.0, -39.0, 58.0], [51.0, -31.0, -40.0, 33.0, -30.0], [5.0, 18.0, -15.0, -39.0, -21.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank1_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([2.0, -7.0])+(1.-msk_arg0)*numpy.array([7.0, 6.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 1.0], [-1.0, 2.0]], [[3.0, 6.0], [1.0, 3.0]], [[5.0, -3.0], [2.0, -7.0]], [[0.0, -4.0], [-5.0, 1.0]], [[0.0, 7.0], [-5.0, -4.0]], [[-2.0, -6.0], [-3.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 3.0], [-1.0, 2.0]], [[0.0, -2.0], [3.0, -7.0]], [[-7.0, -4.0], [4.0, -7.0]], [[1.0, -5.0], [0.0, -1.0]], [[6.0, 6.0], [-1.0, -4.0]], [[-7.0, 3.0], [-1.0, -6.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 2.0], [-2.0, 4.0]], [[6.0, 12.0], [2.0, 6.0]], [[10.0, -6.0], [4.0, -14.0]], [[0.0, -8.0], [-10.0, 2.0]], [[0.0, 14.0], [-10.0, -8.0]], [[-4.0, -12.0], [-6.0, 14.0]]], [[[0.0, -7.0], [7.0, -14.0]], [[-21.0, -42.0], [-7.0, -21.0]], [[-35.0, 21.0], [-14.0, 49.0]], [[0.0, 28.0], [35.0, -7.0]], [[0.0, -49.0], [35.0, 28.0]], [[14.0, 42.0], [21.0, -49.0]]]])+(1.-msk_ref)*numpy.array([[[[-14.0, 21.0], [-7.0, 14.0]], [[0.0, -14.0], [21.0, -49.0]], [[-49.0, -28.0], [28.0, -49.0]], [[7.0, -35.0], [0.0, -7.0]], [[42.0, 42.0], [-7.0, -28.0]], [[-49.0, 21.0], [-7.0, -42.0]]], [[[-12.0, 18.0], [-6.0, 12.0]], [[0.0, -12.0], [18.0, -42.0]], [[-42.0, -24.0], [24.0, -42.0]], [[6.0, -30.0], [0.0, -6.0]], [[36.0, 36.0], [-6.0, -24.0]], [[-42.0, 18.0], [-6.0, -36.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, 5.0], [-7.0, 4.0], [0.0, -1.0]])+(1.-msk_arg0)*numpy.array([[0.0, -6.0], [7.0, -7.0], [-2.0, 6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -7.0], [3.0, 2.0]], [[3.0, -6.0], [7.0, 6.0]], [[-3.0, 1.0], [-6.0, -5.0]], [[3.0, -2.0], [7.0, 3.0]], [[-3.0, -6.0], [4.0, -5.0]], [[4.0, 0.0], [3.0, 3.0]]], [[[3.0, 0.0], [7.0, 2.0]], [[-3.0, 2.0], [-4.0, 5.0]], [[1.0, 3.0], [-7.0, -3.0]], [[5.0, 2.0], [-2.0, -4.0]], [[3.0, 1.0], [0.0, 3.0]], [[-3.0, 3.0], [5.0, -6.0]]], [[[-1.0, 4.0], [4.0, 2.0]], [[3.0, -4.0], [3.0, -5.0]], [[-1.0, 5.0], [5.0, 0.0]], [[1.0, -6.0], [2.0, -3.0]], [[-7.0, -6.0], [5.0, -7.0]], [[-7.0, -3.0], [-6.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, -2.0], [-5.0, 4.0]], [[-5.0, 3.0], [0.0, 2.0]], [[0.0, 0.0], [-4.0, 4.0]], [[4.0, 0.0], [-4.0, 3.0]], [[6.0, -3.0], [7.0, -7.0]], [[7.0, -1.0], [-7.0, 3.0]]], [[[5.0, 0.0], [-2.0, -5.0]], [[-4.0, -5.0], [7.0, 6.0]], [[7.0, 0.0], [-5.0, 4.0]], [[0.0, -7.0], [6.0, 3.0]], [[-5.0, -5.0], [2.0, -6.0]], [[1.0, -7.0], [0.0, 0.0]]], [[[-5.0, -4.0], [2.0, 5.0]], [[7.0, 3.0], [-2.0, -2.0]], [[1.0, 6.0], [0.0, -7.0]], [[7.0, 5.0], [0.0, 2.0]], [[-7.0, 7.0], [7.0, -6.0]], [[5.0, -7.0], [-5.0, 3.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-11.0, 35.0], [-64.0, -24.0]], [[6.0, 16.0], [-7.0, -65.0]], [[8.0, -26.0], [79.0, 46.0]], [[-50.0, -4.0], [-21.0, 13.0]], [[-6.0, 23.0], [-20.0, 4.0]], [[1.0, -21.0], [-50.0, 27.0]]], [[[3.0, -39.0], [39.0, 16.0]], [[0.0, -18.0], [16.0, 55.0]], [[-10.0, 12.0], [-63.0, -37.0]], [[34.0, 4.0], [25.0, 2.0]], [[4.0, -20.0], [15.0, -6.0]], [[15.0, 15.0], [41.0, -16.0]]]])+(1.-msk_ref)*numpy.array([[[[45.0, 8.0], [-18.0, -45.0]], [[-42.0, -41.0], [53.0, 46.0]], [[47.0, -12.0], [-35.0, 42.0]], [[-14.0, -59.0], [42.0, 17.0]], [[-21.0, -49.0], [0.0, -30.0]], [[-3.0, -35.0], [10.0, -6.0]]], [[[-95.0, -12.0], [56.0, 41.0]], [[100.0, 35.0], [-61.0, -66.0]], [[-43.0, 36.0], [59.0, -94.0]], [[18.0, 79.0], [-18.0, -27.0]], [[-43.0, 95.0], [-14.0, 48.0]], [[-19.0, 13.0], [12.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, -2.0, -1.0, -5.0, -2.0], [-5.0, 0.0, -5.0, -5.0, 7.0], [-6.0, -5.0, 7.0, -2.0, 4.0], [-3.0, 7.0, -2.0, 0.0, -6.0]])+(1.-msk_arg0)*numpy.array([[0.0, 4.0, -1.0, 6.0, -4.0], [-5.0, -3.0, 4.0, -5.0, 2.0], [-1.0, -1.0, -5.0, -2.0, 6.0], [-5.0, -5.0, -4.0, 5.0, 4.0]])
      arg1=6.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, -12.0, -6.0, -30.0, -12.0], [-30.0, 0.0, -30.0, -30.0, 42.0], [-36.0, -30.0, 42.0, -12.0, 24.0], [-18.0, 42.0, -12.0, 0.0, -36.0]])+(1.-msk_ref)*numpy.array([[0.0, 24.0, -6.0, 36.0, -24.0], [-30.0, -18.0, 24.0, -30.0, 12.0], [-6.0, -6.0, -30.0, -12.0, 36.0], [-30.0, -30.0, -24.0, 30.0, 24.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 4.0, -6.0, -2.0, -6.0], [1.0, 1.0, -4.0, 2.0, 1.0], [-4.0, -7.0, -5.0, -7.0, -6.0], [6.0, -7.0, 6.0, 3.0, -2.0]])+(1.-msk_arg0)*numpy.array([[0.0, -3.0, 6.0, 6.0, 5.0], [0.0, -3.0, 5.0, 6.0, -4.0], [0.0, -4.0, 6.0, 6.0, 6.0], [4.0, -4.0, -3.0, 4.0, 6.0]])
      arg1=numpy.array(2.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 8.0, -12.0, -4.0, -12.0], [2.0, 2.0, -8.0, 4.0, 2.0], [-8.0, -14.0, -10.0, -14.0, -12.0], [12.0, -14.0, 12.0, 6.0, -4.0]])+(1.-msk_ref)*numpy.array([[0.0, -6.0, 12.0, 12.0, 10.0], [0.0, -6.0, 10.0, 12.0, -8.0], [0.0, -8.0, 12.0, 12.0, 12.0], [8.0, -8.0, -6.0, 8.0, 12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[5.0, 2.0, 4.0, -5.0, 3.0], [-4.0, 4.0, 6.0, 5.0, 7.0], [5.0, -2.0, 4.0, 7.0, 4.0], [-7.0, 5.0, -2.0, -1.0, -7.0]], [[5.0, 3.0, 2.0, 0.0, 1.0], [1.0, -7.0, 4.0, -1.0, 1.0], [3.0, 1.0, -4.0, 1.0, 6.0], [0.0, 4.0, -2.0, 4.0, -3.0]], [[-1.0, -5.0, -5.0, -2.0, 0.0], [-3.0, -3.0, 1.0, -6.0, 7.0], [-3.0, -6.0, -6.0, -2.0, 0.0], [4.0, -5.0, 1.0, 1.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[-5.0, 3.0, 0.0, 2.0, 0.0], [3.0, -7.0, 6.0, 2.0, -1.0], [1.0, -2.0, -4.0, 2.0, 2.0], [-1.0, 3.0, 0.0, -6.0, 0.0]], [[5.0, 3.0, -3.0, 7.0, 7.0], [5.0, -2.0, -7.0, 0.0, -2.0], [2.0, 1.0, 6.0, 4.0, 0.0], [-7.0, 1.0, -4.0, -3.0, 1.0]], [[-4.0, -2.0, 5.0, 6.0, -2.0], [-6.0, 0.0, -2.0, -2.0, 6.0], [0.0, 0.0, 0.0, 2.0, 4.0], [2.0, 5.0, -6.0, 3.0, 5.0]]])
      arg1=numpy.array([4.0, -5.0, 3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, -22.0, -9.0, -26.0, 7.0], [-30.0, 42.0, 7.0, 7.0, 44.0], [-4.0, -31.0, 18.0, 17.0, -14.0], [-16.0, -15.0, 5.0, -21.0, -7.0]])+(1.-msk_ref)*numpy.array([[-57.0, -9.0, 30.0, -9.0, -41.0], [-31.0, -18.0, 53.0, 2.0, 24.0], [-6.0, -13.0, -46.0, -6.0, 20.0], [37.0, 22.0, 2.0, 0.0, 10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, -5.0, 5.0, -5.0, 5.0], [6.0, -3.0, 5.0, 0.0, -7.0], [0.0, -2.0, 3.0, 4.0, 6.0], [0.0, 1.0, -2.0, -6.0, -3.0]], [[3.0, 1.0, 1.0, 4.0, -3.0], [-7.0, -2.0, -7.0, -5.0, -2.0], [0.0, -4.0, 1.0, 5.0, 5.0], [-7.0, -6.0, -5.0, 2.0, 4.0]], [[0.0, -3.0, 7.0, 1.0, -5.0], [0.0, -6.0, -6.0, -6.0, 0.0], [7.0, -5.0, -1.0, 2.0, -1.0], [6.0, 0.0, 4.0, 1.0, -1.0]]], [[[4.0, 4.0, 4.0, -1.0, -7.0], [0.0, 0.0, -4.0, -4.0, 3.0], [7.0, -2.0, 6.0, 4.0, 0.0], [-6.0, -2.0, -1.0, -7.0, 7.0]], [[1.0, 5.0, -5.0, 3.0, 0.0], [0.0, -4.0, 1.0, 0.0, -4.0], [-2.0, 6.0, -6.0, 4.0, -3.0], [-1.0, -7.0, 7.0, 5.0, -3.0]], [[-2.0, -1.0, -7.0, 4.0, -6.0], [-3.0, 2.0, 0.0, 0.0, 0.0], [6.0, 7.0, 5.0, 7.0, -3.0], [-6.0, -1.0, -3.0, -1.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -2.0, 5.0, -4.0, 3.0], [0.0, 2.0, 3.0, 4.0, 2.0], [-1.0, 3.0, 3.0, 6.0, 6.0], [-1.0, -5.0, 5.0, 6.0, 0.0]], [[1.0, 7.0, 2.0, -3.0, 1.0], [0.0, -5.0, -6.0, 7.0, 0.0], [0.0, -2.0, -1.0, 7.0, 1.0], [-7.0, 0.0, -2.0, 2.0, 2.0]], [[-5.0, -1.0, -1.0, 6.0, 2.0], [1.0, 6.0, 0.0, -7.0, -4.0], [6.0, -5.0, -1.0, -1.0, 1.0], [-6.0, 2.0, -3.0, 0.0, -2.0]]], [[[1.0, 2.0, 7.0, 2.0, 0.0], [-5.0, 6.0, -1.0, 2.0, -5.0], [1.0, 0.0, -7.0, 2.0, -5.0], [2.0, -2.0, 5.0, 5.0, -3.0]], [[-5.0, -6.0, 1.0, 1.0, 7.0], [3.0, 0.0, -4.0, 6.0, 4.0], [7.0, -4.0, -2.0, 7.0, -1.0], [0.0, 5.0, 6.0, -4.0, 7.0]], [[0.0, 4.0, -4.0, 6.0, -4.0], [-6.0, 2.0, 5.0, 0.0, -7.0], [0.0, -2.0, 0.0, 5.0, 0.0], [-7.0, -3.0, -2.0, 2.0, 5.0]]]])
      arg1=numpy.array([[7.0, 3.0, -3.0], [-7.0, -3.0, -5.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-40.0, -61.0, 39.0, -48.0, 120.0], [36.0, -7.0, 57.0, 31.0, -64.0], [-94.0, -50.0, -22.0, -38.0, 84.0], [36.0, 29.0, -40.0, 0.0, -36.0]])+(1.-msk_ref)*numpy.array([[26.0, -6.0, 12.0, -102.0, 17.0], [53.0, -71.0, -3.0, 38.0, 84.0], [-53.0, 52.0, 76.0, 6.0, 80.0], [11.0, -27.0, -5.0, 15.0, -13.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, -6.0, -5.0, 3.0, -7.0], [3.0, -3.0, 1.0, 6.0, -1.0], [4.0, 0.0, -3.0, -5.0, -5.0], [0.0, -4.0, -6.0, 7.0, 4.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 5.0, 4.0, 0.0, -1.0], [2.0, 0.0, 0.0, 2.0, 3.0], [-3.0, 7.0, -2.0, -2.0, -4.0], [7.0, -7.0, 2.0, 5.0, 0.0]])
      arg1=numpy.array([-1.0, -2.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[5.0, 10.0], [6.0, 12.0], [5.0, 10.0], [-3.0, -6.0], [7.0, 14.0]], [[-3.0, -6.0], [3.0, 6.0], [-1.0, -2.0], [-6.0, -12.0], [1.0, 2.0]], [[-4.0, -8.0], [0.0, 0.0], [3.0, 6.0], [5.0, 10.0], [5.0, 10.0]], [[0.0, 0.0], [4.0, 8.0], [6.0, 12.0], [-7.0, -14.0], [-4.0, -8.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 8.0], [-5.0, -10.0], [-4.0, -8.0], [0.0, 0.0], [1.0, 2.0]], [[-2.0, -4.0], [0.0, 0.0], [0.0, 0.0], [-2.0, -4.0], [-3.0, -6.0]], [[3.0, 6.0], [-7.0, -14.0], [2.0, 4.0], [2.0, 4.0], [4.0, 8.0]], [[-7.0, -14.0], [7.0, 14.0], [-2.0, -4.0], [-5.0, -10.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, -3.0, 0.0, -7.0, -6.0], [-3.0, 6.0, -3.0, -1.0, 3.0], [-5.0, 6.0, -6.0, 6.0, 2.0], [3.0, 2.0, -5.0, 4.0, 4.0]], [[-6.0, 7.0, 4.0, 0.0, -3.0], [6.0, -7.0, 3.0, -5.0, 6.0], [-1.0, 3.0, 6.0, 3.0, -6.0], [-2.0, 2.0, -7.0, 3.0, -4.0]], [[4.0, -6.0, 0.0, 6.0, 2.0], [-7.0, 7.0, -4.0, 0.0, 5.0], [-1.0, -6.0, 0.0, 4.0, 4.0], [6.0, -4.0, 0.0, -3.0, -1.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 4.0, 0.0, 0.0, -5.0], [4.0, 2.0, 0.0, 2.0, -3.0], [5.0, -7.0, 0.0, -5.0, 2.0], [0.0, 0.0, 3.0, 5.0, -2.0]], [[1.0, -5.0, 1.0, 0.0, 5.0], [-1.0, -1.0, -5.0, -5.0, 2.0], [2.0, -2.0, -3.0, 0.0, -2.0], [-7.0, 3.0, 7.0, 0.0, 0.0]], [[0.0, -2.0, 0.0, 6.0, -5.0], [2.0, 4.0, -1.0, 3.0, 4.0], [7.0, -1.0, -6.0, 4.0, 2.0], [3.0, 5.0, -2.0, 0.0, 6.0]]])
      arg1=numpy.array([[7.0, 0.0], [1.0, -2.0], [7.0, 3.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[29.0, 24.0], [-56.0, -32.0], [4.0, -8.0], [-7.0, 18.0], [-31.0, 12.0]], [[-64.0, -33.0], [84.0, 35.0], [-46.0, -18.0], [-12.0, 10.0], [62.0, 3.0]], [[-43.0, -1.0], [3.0, -24.0], [-36.0, -12.0], [73.0, 6.0], [36.0, 24.0]], [[61.0, 22.0], [-12.0, -16.0], [-42.0, 14.0], [10.0, -15.0], [17.0, 5.0]]])+(1.-msk_ref)*numpy.array([[[22.0, -2.0], [9.0, 4.0], [1.0, -2.0], [42.0, 18.0], [-65.0, -25.0]], [[41.0, 8.0], [41.0, 14.0], [-12.0, 7.0], [30.0, 19.0], [9.0, 8.0]], [[86.0, 17.0], [-58.0, 1.0], [-45.0, -12.0], [-7.0, 12.0], [26.0, 10.0]], [[14.0, 23.0], [38.0, 9.0], [14.0, -20.0], [35.0, 0.0], [28.0, 18.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 7.0, -4.0, -1.0, -4.0], [-1.0, -2.0, 7.0, 0.0, -5.0], [-5.0, -7.0, 0.0, 4.0, -6.0], [0.0, 3.0, -5.0, 7.0, -7.0]], [[-5.0, 6.0, 5.0, -5.0, 4.0], [-2.0, -5.0, 6.0, -4.0, -2.0], [-1.0, 5.0, -1.0, -7.0, -4.0], [-5.0, 7.0, -5.0, 7.0, -5.0]], [[-1.0, -2.0, 6.0, -2.0, 1.0], [-1.0, -1.0, 7.0, -6.0, 0.0], [-3.0, -5.0, -4.0, 0.0, -3.0], [-7.0, -4.0, 0.0, 7.0, 4.0]]], [[[-5.0, -6.0, 3.0, 7.0, -2.0], [6.0, 0.0, -3.0, -2.0, -1.0], [0.0, 6.0, -3.0, 0.0, 1.0], [-7.0, 2.0, 6.0, -6.0, -5.0]], [[3.0, 6.0, 4.0, 2.0, 0.0], [4.0, 6.0, -6.0, 7.0, 2.0], [-1.0, 2.0, -2.0, -4.0, 7.0], [-2.0, -1.0, -5.0, -5.0, -5.0]], [[-3.0, 1.0, 7.0, -7.0, 0.0], [0.0, 0.0, -1.0, 6.0, 6.0], [6.0, 7.0, 2.0, 4.0, -3.0], [7.0, 1.0, -1.0, -7.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, 6.0, 7.0, -1.0, 1.0], [3.0, -6.0, -3.0, 1.0, 2.0], [-3.0, -3.0, -6.0, -2.0, 5.0], [1.0, 0.0, 2.0, -6.0, 0.0]], [[-1.0, -7.0, 5.0, 1.0, -7.0], [7.0, 3.0, 0.0, 6.0, 0.0], [-6.0, 5.0, -6.0, 2.0, 1.0], [-4.0, 0.0, -6.0, 1.0, 1.0]], [[-3.0, 5.0, 0.0, 1.0, -3.0], [-7.0, 2.0, -2.0, -6.0, 2.0], [-3.0, -3.0, -3.0, -6.0, -6.0], [-4.0, 4.0, 6.0, -4.0, -1.0]]], [[[-2.0, -2.0, -4.0, 1.0, 0.0], [0.0, -7.0, -7.0, 2.0, 2.0], [6.0, -4.0, -7.0, -7.0, 2.0], [3.0, -4.0, -1.0, -3.0, 2.0]], [[7.0, 2.0, 1.0, 2.0, -3.0], [-6.0, -7.0, -2.0, 3.0, 1.0], [-6.0, -5.0, -4.0, 3.0, 0.0], [-6.0, -4.0, 4.0, 0.0, -7.0]], [[-2.0, 0.0, 5.0, 3.0, 5.0], [0.0, -6.0, -7.0, 6.0, 7.0], [3.0, -3.0, 3.0, 7.0, 2.0], [-4.0, -3.0, 4.0, -2.0, 0.0]]]])
      arg1=numpy.array([[[3.0, 1.0], [7.0, -1.0], [-1.0, -2.0]], [[3.0, 6.0], [-3.0, 0.0], [-4.0, -5.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-31.0, -3.0], [25.0, -36.0], [-14.0, -38.0], [7.0, 85.0], [9.0, -22.0]], [[-10.0, 39.0], [-58.0, 5.0], [69.0, -26.0], [-73.0, -26.0], [-62.0, -39.0]], [[-40.0, -28.0], [3.0, -1.0], [-14.0, -19.0], [-41.0, -9.0], [-49.0, 25.0]], [[-71.0, -58.0], [67.0, 11.0], [-13.0, 41.0], [88.0, -15.0], [-60.0, -40.0]]])+(1.-msk_ref)*numpy.array([[[-26.0, 4.0], [-48.0, -9.0], [21.0, -47.0], [-12.0, -13.0], [-54.0, -11.0]], [[83.0, 10.0], [25.0, -25.0], [6.0, -6.0], [24.0, -11.0], [-21.0, -25.0]], [[-24.0, 30.0], [44.0, -11.0], [-78.0, -51.0], [-44.0, -69.0], [26.0, 18.0]], [[22.0, 51.0], [8.0, -17.0], [-73.0, -30.0], [-8.0, -7.0], [35.0, 13.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -7.0, -3.0, -2.0, 3.0], [2.0, 3.0, -2.0, 5.0, 3.0], [-3.0, 7.0, 1.0, 1.0, 1.0], [0.0, 6.0, -2.0, 6.0, -3.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0, -6.0, 1.0, 0.0], [4.0, -3.0, -4.0, -3.0, -3.0], [6.0, 4.0, 3.0, -3.0, -5.0], [7.0, -7.0, -2.0, 3.0, 3.0]])
      arg1=numpy.array([[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]], [[-35.0, 21.0, 49.0, -21.0, 35.0], [-7.0, 28.0, -42.0, 0.0, 14.0], [-49.0, 42.0, -21.0, 28.0, -49.0], [-21.0, 35.0, -42.0, 0.0, -28.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]], [[-10.0, 6.0, 14.0, -6.0, 10.0], [-2.0, 8.0, -12.0, 0.0, 4.0], [-14.0, 12.0, -6.0, 8.0, -14.0], [-6.0, 10.0, -12.0, 0.0, -8.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]]], [[[10.0, -6.0, -14.0, 6.0, -10.0], [2.0, -8.0, 12.0, 0.0, -4.0], [14.0, -12.0, 6.0, -8.0, 14.0], [6.0, -10.0, 12.0, 0.0, 8.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]], [[-10.0, 6.0, 14.0, -6.0, 10.0], [-2.0, 8.0, -12.0, 0.0, 4.0], [-14.0, 12.0, -6.0, 8.0, -14.0], [-6.0, 10.0, -12.0, 0.0, -8.0]], [[25.0, -15.0, -35.0, 15.0, -25.0], [5.0, -20.0, 30.0, 0.0, -10.0], [35.0, -30.0, 15.0, -20.0, 35.0], [15.0, -25.0, 30.0, 0.0, 20.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]]], [[[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]], [[35.0, -21.0, -49.0, 21.0, -35.0], [7.0, -28.0, 42.0, 0.0, -14.0], [49.0, -42.0, 21.0, -28.0, 49.0], [21.0, -35.0, 42.0, 0.0, 28.0]], [[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]], [[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]], [[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[30.0, -18.0, -42.0, 18.0, -30.0], [6.0, -24.0, 36.0, 0.0, -12.0], [42.0, -36.0, 18.0, -24.0, 42.0], [18.0, -30.0, 36.0, 0.0, 24.0]], [[-10.0, 6.0, 14.0, -6.0, 10.0], [-2.0, 8.0, -12.0, 0.0, 4.0], [-14.0, 12.0, -6.0, 8.0, -14.0], [-6.0, 10.0, -12.0, 0.0, -8.0]], [[30.0, -18.0, -42.0, 18.0, -30.0], [6.0, -24.0, 36.0, 0.0, -12.0], [42.0, -36.0, 18.0, -24.0, 42.0], [18.0, -30.0, 36.0, 0.0, 24.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]], [[-30.0, 18.0, 42.0, -18.0, 30.0], [-6.0, 24.0, -36.0, 0.0, 12.0], [-42.0, 36.0, -18.0, 24.0, -42.0], [-18.0, 30.0, -36.0, 0.0, -24.0]], [[5.0, -3.0, -7.0, 3.0, -5.0], [1.0, -4.0, 6.0, 0.0, -2.0], [7.0, -6.0, 3.0, -4.0, 7.0], [3.0, -5.0, 6.0, 0.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[20.0, -12.0, -28.0, 12.0, -20.0], [4.0, -16.0, 24.0, 0.0, -8.0], [28.0, -24.0, 12.0, -16.0, 28.0], [12.0, -20.0, 24.0, 0.0, 16.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]], [[-20.0, 12.0, 28.0, -12.0, 20.0], [-4.0, 16.0, -24.0, 0.0, 8.0], [-28.0, 24.0, -12.0, 16.0, -28.0], [-12.0, 20.0, -24.0, 0.0, -16.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]]], [[[30.0, -18.0, -42.0, 18.0, -30.0], [6.0, -24.0, 36.0, 0.0, -12.0], [42.0, -36.0, 18.0, -24.0, 42.0], [18.0, -30.0, 36.0, 0.0, 24.0]], [[20.0, -12.0, -28.0, 12.0, -20.0], [4.0, -16.0, 24.0, 0.0, -8.0], [28.0, -24.0, 12.0, -16.0, 28.0], [12.0, -20.0, 24.0, 0.0, 16.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]], [[-15.0, 9.0, 21.0, -9.0, 15.0], [-3.0, 12.0, -18.0, 0.0, 6.0], [-21.0, 18.0, -9.0, 12.0, -21.0], [-9.0, 15.0, -18.0, 0.0, -12.0]], [[-25.0, 15.0, 35.0, -15.0, 25.0], [-5.0, 20.0, -30.0, 0.0, 10.0], [-35.0, 30.0, -15.0, 20.0, -35.0], [-15.0, 25.0, -30.0, 0.0, -20.0]]], [[[35.0, -21.0, -49.0, 21.0, -35.0], [7.0, -28.0, 42.0, 0.0, -14.0], [49.0, -42.0, 21.0, -28.0, 49.0], [21.0, -35.0, 42.0, 0.0, 28.0]], [[-35.0, 21.0, 49.0, -21.0, 35.0], [-7.0, 28.0, -42.0, 0.0, 14.0], [-49.0, 42.0, -21.0, 28.0, -49.0], [-21.0, 35.0, -42.0, 0.0, -28.0]], [[-10.0, 6.0, 14.0, -6.0, 10.0], [-2.0, 8.0, -12.0, 0.0, 4.0], [-14.0, 12.0, -6.0, 8.0, -14.0], [-6.0, 10.0, -12.0, 0.0, -8.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]], [[15.0, -9.0, -21.0, 9.0, -15.0], [3.0, -12.0, 18.0, 0.0, -6.0], [21.0, -18.0, 9.0, -12.0, 21.0], [9.0, -15.0, 18.0, 0.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, -3.0, -2.0, -5.0, -6.0], [7.0, 4.0, 6.0, 1.0, -7.0], [1.0, 0.0, 4.0, 0.0, -5.0], [-6.0, -7.0, 7.0, 2.0, 2.0]], [[-3.0, -6.0, -1.0, -5.0, 3.0], [6.0, -3.0, 0.0, 0.0, 7.0], [-7.0, -6.0, 0.0, -2.0, 2.0], [0.0, -4.0, 6.0, 6.0, -6.0]], [[-1.0, -4.0, 2.0, -6.0, 5.0], [6.0, -5.0, -2.0, 6.0, 5.0], [4.0, 0.0, 4.0, 6.0, -7.0], [5.0, 6.0, 1.0, 2.0, 4.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, -7.0, -1.0, 0.0, 7.0], [-6.0, 3.0, 7.0, -6.0, -2.0], [2.0, 4.0, -2.0, -3.0, -2.0], [-6.0, -7.0, 4.0, -2.0, -2.0]], [[-5.0, -2.0, -1.0, 3.0, 7.0], [7.0, 0.0, -4.0, 4.0, -5.0], [-4.0, -3.0, -2.0, 5.0, 5.0], [7.0, -7.0, -3.0, -6.0, 3.0]], [[6.0, 4.0, 5.0, 0.0, 1.0], [0.0, 0.0, -6.0, 7.0, 5.0], [-3.0, 7.0, -4.0, -3.0, -3.0], [2.0, -4.0, 7.0, -1.0, 5.0]]])
      arg1=numpy.array([[[-4.0, -6.0, 7.0, 5.0, 5.0], [6.0, -5.0, 1.0, -7.0, 5.0], [6.0, -4.0, 6.0, -3.0, 4.0], [6.0, 0.0, -5.0, 1.0, 2.0]], [[-2.0, 4.0, 3.0, 2.0, 6.0], [0.0, -6.0, -3.0, -4.0, -7.0], [5.0, 0.0, 1.0, -5.0, 7.0], [-7.0, -4.0, 6.0, 0.0, 7.0]], [[2.0, 5.0, -1.0, 6.0, 4.0], [4.0, -4.0, -4.0, -7.0, 7.0], [4.0, 1.0, -6.0, 4.0, -2.0], [5.0, 7.0, 1.0, 0.0, 1.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[32.0, 25.0, -57.0, -47.0, -57.0], [-46.0, 57.0, 6.0, 68.0, -21.0], [-61.0, 27.0, -39.0, 32.0, -47.0], [-26.0, 5.0, 16.0, -7.0, -36.0]], [[16.0, -26.0, -35.0, -51.0, -67.0], [-34.0, 67.0, 31.0, 73.0, -1.0], [-64.0, 8.0, 0.0, 23.0, -46.0], [4.0, -4.0, -25.0, -3.0, -52.0]], [[14.0, 18.0, -19.0, 0.0, -8.0], [-4.0, 8.0, -7.0, 4.0, 11.0], [-9.0, 10.0, -25.0, 19.0, -19.0], [5.0, 18.0, 6.0, -2.0, -9.0]], [[18.0, -20.0, -44.0, -71.0, -79.0], [-54.0, 79.0, 34.0, 97.0, -32.0], [-79.0, 14.0, 1.0, 16.0, -43.0], [-25.0, -22.0, -11.0, -5.0, -51.0]], [[28.0, 73.0, -38.0, 6.0, 8.0], [-16.0, -8.0, -35.0, -5.0, -16.0], [-1.0, 29.0, -63.0, 23.0, -13.0], [-32.0, 23.0, 53.0, -6.0, 14.0]]], [[[-28.0, 12.0, 61.0, 83.0, 95.0], [66.0, -95.0, -35.0, -115.0, 35.0], [96.0, -22.0, 12.0, -27.0, 58.0], [30.0, 18.0, 7.0, 7.0, 62.0]], [[-20.0, -61.0, 24.0, -16.0, -18.0], [4.0, 18.0, 33.0, 19.0, 6.0], [-11.0, -21.0, 51.0, -17.0, 5.0], [20.0, -23.0, -43.0, 4.0, -18.0]], [[-28.0, -46.0, 44.0, 18.0, 22.0], [28.0, -22.0, 14.0, -28.0, 16.0], [28.0, -26.0, 48.0, -26.0, 28.0], [26.0, -14.0, -32.0, 6.0, 10.0]], [[8.0, 24.0, 1.0, 41.0, 29.0], [30.0, -29.0, -23.0, -49.0, 47.0], [30.0, 2.0, -30.0, 21.0, -8.0], [36.0, 42.0, 1.0, 1.0, 8.0]], [[24.0, 95.0, -33.0, 9.0, 27.0], [-22.0, -27.0, -48.0, -14.0, -49.0], [13.0, 33.0, -65.0, 6.0, 11.0], [-66.0, 7.0, 82.0, -7.0, 40.0]]], [[[18.0, -14.0, -18.0, 15.0, -21.0], [22.0, 21.0, 6.0, -7.0, 82.0], [-13.0, 0.0, -25.0, 48.0, -53.0], [75.0, 56.0, -43.0, 1.0, -43.0]], [[12.0, -24.0, -18.0, -12.0, -36.0], [0.0, 36.0, 18.0, 24.0, 42.0], [-30.0, 0.0, -6.0, 30.0, -42.0], [42.0, 24.0, -36.0, 0.0, -42.0]], [[-8.0, -4.0, 24.0, 44.0, 36.0], [40.0, -36.0, -12.0, -56.0, 48.0], [40.0, -12.0, 0.0, 4.0, 8.0], [44.0, 28.0, -16.0, 4.0, 12.0]], [[16.0, 22.0, -12.0, 32.0, 12.0], [24.0, -12.0, -18.0, -34.0, 56.0], [14.0, 6.0, -38.0, 34.0, -26.0], [44.0, 50.0, -6.0, 0.0, -8.0]], [[2.0, 3.0, -22.0, -63.0, -41.0], [-58.0, 41.0, 17.0, 76.0, -88.0], [-48.0, 13.0, 14.0, -23.0, 8.0], [-79.0, -57.0, 30.0, -5.0, -3.0]]], [[[34.0, 61.0, -47.0, 0.0, -10.0], [-16.0, 10.0, -26.0, 7.0, 5.0], [-16.0, 29.0, -66.0, 38.0, -34.0], [-11.0, 35.0, 35.0, -6.0, -7.0]], [[48.0, 56.0, -67.0, -7.0, -35.0], [-18.0, 35.0, -19.0, 23.0, 35.0], [-38.0, 34.0, -82.0, 65.0, -68.0], [16.0, 58.0, 17.0, -7.0, -36.0]], [[-38.0, -13.0, 66.0, 53.0, 75.0], [46.0, -75.0, -15.0, -80.0, 0.0], [76.0, -27.0, 42.0, -47.0, 68.0], [5.0, -17.0, 2.0, 7.0, 57.0]], [[-16.0, 22.0, 30.0, 34.0, 54.0], [20.0, -54.0, -24.0, -52.0, -18.0], [50.0, -6.0, 6.0, -28.0, 46.0], [-20.0, -10.0, 28.0, 2.0, 48.0]], [[12.0, -16.0, -8.0, 22.0, -10.0], [28.0, 10.0, 4.0, -18.0, 80.0], [-2.0, -4.0, -18.0, 40.0, -42.0], [74.0, 52.0, -42.0, 2.0, -34.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -32.0, 28.0, 61.0, 29.0], [66.0, -29.0, -2.0, -71.0, 112.0], [41.0, -22.0, 1.0, 28.0, -19.0], [107.0, 62.0, -59.0, 7.0, -15.0]], [[40.0, 54.0, -59.0, -15.0, -31.0], [-26.0, 31.0, -17.0, 29.0, 7.0], [-36.0, 32.0, -68.0, 47.0, -50.0], [-8.0, 36.0, 27.0, -7.0, -24.0]], [[16.0, 27.0, -15.0, 23.0, 9.0], [14.0, -9.0, -18.0, -24.0, 37.0], [9.0, 9.0, -37.0, 28.0, -21.0], [26.0, 39.0, 4.0, -1.0, -4.0]], [[-6.0, 12.0, 9.0, 6.0, 18.0], [0.0, -18.0, -9.0, -12.0, -21.0], [15.0, 0.0, 3.0, -15.0, 21.0], [-21.0, -12.0, 18.0, 0.0, 21.0]], [[-40.0, -9.0, 69.0, 55.0, 81.0], [46.0, -81.0, -18.0, -84.0, -7.0], [81.0, -27.0, 43.0, -52.0, 75.0], [-2.0, -21.0, 8.0, 7.0, 64.0]]], [[[10.0, 64.0, -21.0, -16.0, 12.0], [-36.0, -12.0, -27.0, 14.0, -79.0], [-1.0, 24.0, -29.0, -17.0, 25.0], [-85.0, -28.0, 72.0, -6.0, 37.0]], [[-12.0, -18.0, 21.0, 15.0, 15.0], [18.0, -15.0, 3.0, -21.0, 15.0], [18.0, -12.0, 18.0, -9.0, 12.0], [18.0, 0.0, -15.0, 3.0, 6.0]], [[-32.0, -88.0, 43.0, -9.0, -13.0], [18.0, 13.0, 43.0, 9.0, 21.0], [-2.0, -34.0, 74.0, -25.0, 12.0], [40.0, -26.0, -65.0, 7.0, -20.0]], [[30.0, 87.0, -37.0, 20.0, 22.0], [-8.0, -22.0, -46.0, -23.0, -9.0], [12.0, 31.0, -74.0, 26.0, -10.0], [-29.0, 33.0, 61.0, -6.0, 23.0]], [[28.0, 17.0, -34.0, 10.0, -20.0], [8.0, 20.0, -7.0, -1.0, 60.0], [-17.0, 13.0, -47.0, 51.0, -53.0], [48.0, 55.0, -15.0, -2.0, -34.0]]], [[[-6.0, -43.0, 5.0, -16.0, -26.0], [0.0, 26.0, 26.0, 23.0, 17.0], [-20.0, -11.0, 26.0, 2.0, -14.0], [25.0, -5.0, -37.0, 2.0, -27.0]], [[4.0, -1.0, 12.0, 56.0, 30.0], [52.0, -30.0, -15.0, -65.0, 90.0], [37.0, -9.0, -21.0, 31.0, -19.0], [80.0, 61.0, -31.0, 4.0, -6.0]], [[4.0, -16.0, -16.0, -38.0, -38.0], [-28.0, 38.0, 20.0, 50.0, -24.0], [-38.0, 4.0, 10.0, 0.0, -14.0], [-18.0, -20.0, -6.0, -2.0, -22.0]], [[-4.0, 23.0, -3.0, -23.0, 3.0], [-30.0, -3.0, -6.0, 22.0, -71.0], [-5.0, 9.0, 5.0, -28.0, 29.0], [-68.0, -41.0, 42.0, -3.0, 26.0]], [[-8.0, 17.0, 4.0, -18.0, 8.0], [-24.0, -8.0, -5.0, 15.0, -66.0], [1.0, 5.0, 11.0, -31.0, 33.0], [-62.0, -41.0, 37.0, -2.0, 28.0]]], [[[14.0, 74.0, -23.0, -4.0, 20.0], [-28.0, -20.0, -35.0, 0.0, -65.0], [7.0, 26.0, -41.0, -9.0, 21.0], [-75.0, -14.0, 74.0, -6.0, 39.0]], [[34.0, -6.0, -66.0, -73.0, -93.0], [-58.0, 93.0, 30.0, 105.0, -14.0], [-93.0, 24.0, -25.0, 40.0, -69.0], [-13.0, 0.0, -11.0, -7.0, -67.0]], [[4.0, -1.0, 12.0, 56.0, 30.0], [52.0, -30.0, -15.0, -65.0, 90.0], [37.0, -9.0, -21.0, 31.0, -19.0], [80.0, 61.0, -31.0, 4.0, -6.0]], [[18.0, -17.0, -31.0, -28.0, -50.0], [-16.0, 50.0, 20.0, 45.0, 25.0], [-46.0, 7.0, -12.0, 32.0, -48.0], [25.0, 17.0, -27.0, -2.0, -47.0]], [[12.0, 49.0, -10.0, 26.0, 28.0], [8.0, -28.0, -31.0, -33.0, 4.0], [23.0, 13.0, -39.0, 11.0, 3.0], [-8.0, 23.0, 33.0, -2.0, 22.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 3.0, -2.0, 1.0, 0.0], [6.0, 6.0, -1.0, 3.0, -3.0], [-4.0, -1.0, 0.0, -4.0, 7.0], [-2.0, 6.0, -3.0, 5.0, 0.0]], [[-2.0, -5.0, -1.0, 7.0, -6.0], [0.0, 1.0, 0.0, -5.0, 5.0], [5.0, -2.0, -5.0, -1.0, -6.0], [-7.0, -1.0, -6.0, 6.0, 4.0]], [[-4.0, 7.0, 0.0, 5.0, 7.0], [4.0, 2.0, -6.0, 4.0, 5.0], [-2.0, -1.0, -1.0, 1.0, 7.0], [-3.0, 2.0, 1.0, 6.0, 1.0]]], [[[-3.0, -3.0, -6.0, 0.0, 6.0], [2.0, 0.0, 3.0, -1.0, -7.0], [-2.0, -4.0, 0.0, -4.0, 3.0], [5.0, 0.0, 0.0, 0.0, 3.0]], [[-6.0, -7.0, 6.0, 0.0, 2.0], [0.0, 3.0, 6.0, -7.0, 4.0], [7.0, 5.0, -1.0, 6.0, -3.0], [5.0, 1.0, -2.0, -5.0, 1.0]], [[-3.0, 4.0, -5.0, 3.0, -3.0], [3.0, 6.0, 7.0, 7.0, 2.0], [-4.0, -1.0, -5.0, -5.0, -3.0], [1.0, -4.0, 0.0, 0.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, 4.0, 1.0, 5.0, 4.0], [-1.0, 1.0, 2.0, -5.0, 0.0], [-6.0, 5.0, 0.0, 0.0, -1.0], [-3.0, -3.0, -6.0, -1.0, 0.0]], [[-1.0, -1.0, 2.0, -3.0, 0.0], [-6.0, 1.0, 7.0, -3.0, 2.0], [-4.0, 0.0, 0.0, 2.0, -7.0], [1.0, -2.0, -2.0, 1.0, 0.0]], [[-3.0, -4.0, 7.0, -4.0, -4.0], [5.0, 7.0, 5.0, -7.0, 4.0], [-2.0, 0.0, 3.0, 4.0, -2.0], [-1.0, -4.0, 7.0, 1.0, -2.0]]], [[[-4.0, -5.0, 2.0, 1.0, 6.0], [0.0, 5.0, 2.0, -5.0, 0.0], [0.0, -7.0, -1.0, 4.0, -2.0], [2.0, 1.0, 1.0, 2.0, 6.0]], [[-7.0, -5.0, -3.0, 6.0, 4.0], [7.0, 6.0, 3.0, -5.0, -4.0], [0.0, -6.0, -6.0, -4.0, -7.0], [5.0, 2.0, -1.0, -4.0, -3.0]], [[4.0, 1.0, 2.0, 5.0, 0.0], [-7.0, -6.0, 4.0, 0.0, 4.0], [-5.0, -1.0, 0.0, 2.0, 2.0], [5.0, 4.0, -1.0, -3.0, 4.0]]]])
      arg1=numpy.array([[[[3.0, -6.0, 5.0, -5.0, -5.0], [-4.0, -7.0, -1.0, 2.0, -4.0], [-1.0, 1.0, -4.0, 0.0, 1.0], [-7.0, -6.0, -4.0, -1.0, -1.0]], [[1.0, 3.0, -4.0, -4.0, 4.0], [6.0, -7.0, -1.0, 0.0, -7.0], [-4.0, 4.0, -7.0, -3.0, 4.0], [6.0, -4.0, 0.0, 7.0, -2.0]], [[-2.0, 0.0, 1.0, 6.0, 0.0], [-3.0, -3.0, -6.0, 4.0, 7.0], [6.0, 6.0, 2.0, 4.0, 0.0], [2.0, -1.0, 0.0, -3.0, -5.0]]], [[[0.0, 7.0, -5.0, 2.0, 0.0], [1.0, 3.0, 0.0, -6.0, 6.0], [4.0, -6.0, 6.0, 0.0, -2.0], [-1.0, -3.0, -1.0, -3.0, 2.0]], [[1.0, 3.0, 5.0, 3.0, 2.0], [6.0, 7.0, 7.0, 1.0, 6.0], [2.0, 0.0, 3.0, -2.0, 7.0], [6.0, 1.0, -2.0, 6.0, 2.0]], [[4.0, 2.0, 1.0, 0.0, 7.0], [-3.0, 6.0, 3.0, -3.0, 7.0], [-7.0, 4.0, -2.0, -4.0, 3.0], [7.0, 4.0, 4.0, -4.0, -1.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, -81.0, 11.0, -65.0, -66.0], [-50.0, -78.0, -30.0, 15.0, -109.0], [-24.0, -21.0, -44.0, 14.0, -48.0], [-109.0, -27.0, -17.0, -22.0, 4.0]], [[-1.0, -67.0, 26.0, 20.0, -21.0], [-120.0, -41.0, -77.0, 33.0, 40.0], [5.0, 59.0, -10.0, 41.0, -48.0], [-48.0, 13.0, 21.0, -108.0, -52.0]], [[-21.0, -25.0, 49.0, 20.0, -17.0], [47.0, 15.0, 30.0, 53.0, -20.0], [29.0, 10.0, 7.0, 11.0, 33.0], [15.0, 20.0, -18.0, 69.0, 9.0]], [[12.0, 21.0, -15.0, -3.0, 44.0], [14.0, -53.0, -29.0, 13.0, 3.0], [-20.0, 71.0, -49.0, -13.0, 38.0], [66.0, -27.0, 8.0, 21.0, -43.0]], [[-30.0, 24.0, 8.0, 84.0, -41.0], [-30.0, 35.0, -31.0, 3.0, 118.0], [115.0, -30.0, 104.0, 54.0, -31.0], [-37.0, -11.0, -22.0, -57.0, -4.0]]], [[[22.0, -16.0, 27.0, -2.0, -9.0], [-43.0, -30.0, -21.0, 7.0, 37.0], [5.0, 30.0, -10.0, 4.0, 11.0], [-15.0, -34.0, -14.0, -36.0, -25.0]], [[42.0, -12.0, 49.0, -13.0, 22.0], [-24.0, 2.0, 20.0, 5.0, 43.0], [-34.0, 46.0, -30.0, -25.0, 49.0], [28.0, -15.0, -6.0, -11.0, -18.0]], [[43.0, 59.0, 11.0, -7.0, 66.0], [40.0, 118.0, 100.0, -59.0, 65.0], [-60.0, -27.0, 14.0, -64.0, 56.0], [77.0, 37.0, 17.0, 18.0, 42.0]], [[17.0, -47.0, 16.0, 6.0, 0.0], [-118.0, -8.0, -50.0, 0.0, 52.0], [-26.0, 41.0, -10.0, 17.0, -43.0], [-35.0, 22.0, 31.0, -117.0, -36.0]], [[-2.0, 0.0, 27.0, 23.0, 57.0], [38.0, -10.0, 2.0, 54.0, 8.0], [-21.0, 97.0, -47.0, -11.0, 65.0], [106.0, 26.0, 19.0, 60.0, -40.0]]], [[[-12.0, 38.0, -1.0, 5.0, 26.0], [104.0, 18.0, 48.0, 15.0, -31.0], [6.0, 0.0, -6.0, -21.0, 57.0], [70.0, 3.0, -12.0, 109.0, 18.0]], [[-2.0, -15.0, 46.0, 14.0, 0.0], [24.0, 41.0, 41.0, 26.0, 10.0], [4.0, 5.0, 9.0, -4.0, 31.0], [20.0, 28.0, -6.0, 36.0, 13.0]], [[-24.0, -28.0, 9.0, 11.0, -57.0], [-18.0, 1.0, -11.0, 10.0, -13.0], [47.0, -46.0, 40.0, 33.0, -42.0], [-73.0, 0.0, -18.0, -18.0, 18.0]], [[-29.0, 1.0, 30.0, 40.0, -7.0], [54.0, 32.0, 26.0, 41.0, 7.0], [45.0, 2.0, 29.0, 15.0, 27.0], [29.0, 25.0, -12.0, 62.0, 10.0]], [[-14.0, -54.0, 33.0, 28.0, -86.0], [-91.0, -58.0, -73.0, 30.0, 42.0], [86.0, -5.0, 43.0, 64.0, -53.0], [-113.0, -49.0, -37.0, -85.0, -27.0]]], [[[2.0, 43.0, 16.0, 45.0, -1.0], [7.0, 128.0, 65.0, -44.0, 103.0], [35.0, -74.0, 94.0, -5.0, -2.0], [-2.0, 37.0, -3.0, -27.0, 50.0]], [[-2.0, -44.0, 37.0, -11.0, -60.0], [-18.0, -58.0, -22.0, 33.0, -25.0], [40.0, -2.0, -2.0, 25.0, -3.0], [-66.0, -49.0, -42.0, 3.0, -8.0]], [[-19.0, -6.0, 0.0, 39.0, -13.0], [-39.0, 46.0, -11.0, -4.0, 49.0], [29.0, -21.0, 50.0, 26.0, -41.0], [-25.0, 39.0, 16.0, -54.0, 6.0]], [[4.0, -27.0, -18.0, -28.0, -11.0], [-32.0, -130.0, -82.0, 29.0, -50.0], [-3.0, 65.0, -65.0, 16.0, -6.0], [-17.0, -65.0, -10.0, -11.0, -57.0]], [[-5.0, 32.0, -27.0, -1.0, 4.0], [36.0, -27.0, -9.0, -7.0, -11.0], [18.0, -4.0, -1.0, -2.0, 11.0], [15.0, -33.0, -13.0, 30.0, -3.0]]]])+(1.-msk_ref)*numpy.array([[[[26.0, -68.0, 10.0, -63.0, -10.0], [-71.0, -49.0, -22.0, 1.0, -68.0], [-76.0, 22.0, -68.0, -11.0, -29.0], [-50.0, 4.0, 18.0, -48.0, -13.0]], [[18.0, -75.0, 21.0, -65.0, -27.0], [-48.0, -53.0, -11.0, 14.0, -90.0], [-61.0, 10.0, -64.0, -7.0, -22.0], [-60.0, -2.0, 3.0, -18.0, -3.0]], [[-4.0, 9.0, -19.0, 24.0, 11.0], [-35.0, -45.0, -60.0, 9.0, 39.0], [21.0, 47.0, -5.0, 20.0, -10.0], [13.0, -22.0, 8.0, -40.0, -44.0]], [[46.0, -4.0, 63.0, -17.0, 10.0], [-4.0, 73.0, 79.0, -21.0, 50.0], [-36.0, -17.0, 7.0, -39.0, 48.0], [9.0, 9.0, -13.0, -1.0, 30.0]], [[24.0, 30.0, 6.0, -20.0, -12.0], [26.0, 30.0, 48.0, -40.0, 16.0], [4.0, -56.0, 24.0, -24.0, 20.0], [-18.0, -34.0, -30.0, 14.0, 36.0]]], [[[-40.0, -5.0, 52.0, 80.0, -54.0], [16.0, 41.0, 5.0, 46.0, 74.0], [118.0, -23.0, 91.0, 52.0, 3.0], [-26.0, 4.0, -38.0, 14.0, 9.0]], [[-28.0, 38.0, 7.0, 61.0, -31.0], [40.0, -14.0, -20.0, 24.0, 62.0], [111.0, -7.0, 63.0, 37.0, 19.0], [2.0, -50.0, -45.0, 30.0, -10.0]], [[22.0, 40.0, -4.0, 5.0, 52.0], [27.0, -27.0, -6.0, 3.0, 36.0], [-14.0, 64.0, -34.0, -23.0, 59.0], [82.0, -32.0, 0.0, 28.0, -35.0]], [[-9.0, -29.0, -20.0, -30.0, 3.0], [-12.0, 27.0, 15.0, -13.0, -68.0], [-55.0, -29.0, -18.0, -9.0, -42.0], [-22.0, 59.0, 35.0, -10.0, 26.0]], [[6.0, 2.0, -20.0, 4.0, 28.0], [-36.0, -30.0, -42.0, 0.0, 18.0], [-20.0, 48.0, -26.0, 2.0, -8.0], [24.0, 0.0, 24.0, -38.0, -36.0]]], [[[-38.0, 14.0, -21.0, 34.0, -21.0], [21.0, 46.0, 7.0, -5.0, 3.0], [45.0, -54.0, 58.0, 24.0, -37.0], [-21.0, 34.0, 4.0, 4.0, 29.0]], [[5.0, -99.0, 29.0, -57.0, -44.0], [-60.0, -104.0, -50.0, 49.0, -105.0], [-38.0, 43.0, -78.0, 16.0, -26.0], [-71.0, -19.0, -5.0, -16.0, -30.0]], [[-12.0, -25.0, -22.0, -2.0, -12.0], [-46.0, -54.0, -60.0, 12.0, -21.0], [2.0, 24.0, -18.0, 24.0, -40.0], [-29.0, -6.0, 13.0, -42.0, -29.0]], [[-2.0, 26.0, -42.0, 12.0, 14.0], [-26.0, -30.0, -48.0, -18.0, 28.0], [10.0, 16.0, 2.0, 10.0, -22.0], [6.0, -20.0, 12.0, -42.0, -26.0]], [[-5.0, -46.0, -2.0, -4.0, -23.0], [-82.0, 19.0, -23.0, -11.0, -1.0], [-19.0, -21.0, 12.0, 19.0, -68.0], [-65.0, 43.0, 28.0, -86.0, 5.0]]], [[[19.0, 60.0, 0.0, 24.0, 64.0], [38.0, 88.0, 58.0, -32.0, 75.0], [-24.0, 3.0, 20.0, -37.0, 47.0], [88.0, 34.0, 20.0, 17.0, 15.0]], [[15.0, 33.0, -2.0, 7.0, 39.0], [13.0, 88.0, 55.0, -38.0, 44.0], [-33.0, -25.0, 22.0, -30.0, 13.0], [40.0, 45.0, 23.0, -6.0, 29.0]], [[-39.0, 32.0, -26.0, 79.0, 13.0], [-11.0, 25.0, -44.0, 12.0, 80.0], [65.0, 18.0, 57.0, 40.0, -26.0], [30.0, 29.0, 21.0, -34.0, -24.0]], [[-20.0, 5.0, -41.0, -1.0, -20.0], [-6.0, -43.0, -43.0, -5.0, -29.0], [24.0, -15.0, 5.0, 21.0, -38.0], [-32.0, -21.0, -2.0, -13.0, -7.0]], [[17.0, 41.0, -43.0, -9.0, 22.0], [-18.0, 27.0, 3.0, -59.0, 32.0], [-22.0, -32.0, 15.0, -18.0, -21.0], [0.0, -3.0, 16.0, -46.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 3.0, 0.0, 5.0, -4.0], [0.0, -6.0, -5.0, -2.0, 5.0], [3.0, 0.0, 3.0, 6.0, 4.0], [-4.0, -7.0, 0.0, -6.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-2.0, -6.0, 0.0, -2.0, -7.0], [5.0, 0.0, 5.0, 4.0, 5.0], [-7.0, 1.0, -5.0, -6.0, -1.0], [-1.0, -5.0, 0.0, 7.0, 4.0]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(1.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 3.0, 0.0, 5.0, -4.0], [0.0, -6.0, -5.0, -2.0, 5.0], [3.0, 0.0, 3.0, 6.0, 4.0], [-4.0, -7.0, 0.0, -6.0, 5.0]])+(1.-msk_ref)*numpy.array([[-2.0, -6.0, 0.0, -2.0, -7.0], [5.0, 0.0, 5.0, 4.0, 5.0], [-7.0, 1.0, -5.0, -6.0, -1.0], [-1.0, -5.0, 0.0, 7.0, 4.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, -7.0, -2.0, 6.0, -4.0], [0.0, 3.0, 3.0, -7.0, 0.0], [-4.0, -6.0, -6.0, 0.0, 4.0], [4.0, -7.0, 4.0, -1.0, -6.0]], [[0.0, 0.0, 6.0, 1.0, -5.0], [6.0, 3.0, -2.0, 2.0, 3.0], [6.0, 6.0, 4.0, 0.0, 3.0], [3.0, -4.0, 0.0, 1.0, -1.0]], [[2.0, 3.0, 4.0, -6.0, -5.0], [3.0, -5.0, -1.0, 5.0, 4.0], [-7.0, -4.0, 0.0, 7.0, 5.0], [5.0, 1.0, -3.0, 2.0, 4.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 5.0, 4.0, 5.0, -2.0], [6.0, 0.0, 1.0, 5.0, 5.0], [1.0, 5.0, 0.0, 4.0, -7.0], [1.0, -3.0, 1.0, -6.0, -1.0]], [[-1.0, 5.0, -7.0, 6.0, -5.0], [7.0, -3.0, 3.0, 6.0, 5.0], [4.0, 6.0, 0.0, -4.0, 6.0], [7.0, -4.0, 6.0, -4.0, 5.0]], [[4.0, -2.0, 3.0, -7.0, 4.0], [6.0, -2.0, -2.0, 7.0, -7.0], [-3.0, -6.0, 5.0, 4.0, 0.0], [-4.0, 4.0, -1.0, -6.0, 6.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-1.0, -5.0, -1.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 4.0, -32.0, -5.0, 34.0], [-33.0, -13.0, 8.0, -8.0, -19.0], [-19.0, -20.0, -14.0, -7.0, -24.0], [-24.0, 26.0, -1.0, -6.0, 7.0]])+(1.-msk_ref)*numpy.array([[-1.0, -28.0, 28.0, -28.0, 23.0], [-47.0, 17.0, -14.0, -42.0, -23.0], [-18.0, -29.0, -5.0, 12.0, -23.0], [-32.0, 19.0, -30.0, 32.0, -30.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, 1.0, -3.0, 0.0, 4.0], [0.0, 0.0, -1.0, -2.0, 1.0], [2.0, 0.0, 5.0, 4.0, 1.0], [7.0, 4.0, 6.0, 0.0, 0.0]], [[-6.0, -7.0, 7.0, 7.0, -6.0], [-1.0, 0.0, -2.0, 0.0, -3.0], [-6.0, 0.0, -6.0, 0.0, 0.0], [5.0, -6.0, 0.0, 2.0, 1.0]], [[-6.0, 4.0, -1.0, 7.0, 2.0], [6.0, 0.0, 5.0, -3.0, 0.0], [0.0, 5.0, -4.0, -2.0, 2.0], [5.0, 1.0, -6.0, 0.0, 4.0]]], [[[-1.0, 3.0, 6.0, 3.0, 3.0], [-3.0, 2.0, 0.0, -5.0, -5.0], [3.0, 6.0, 5.0, 3.0, 1.0], [-3.0, 7.0, -3.0, -7.0, 0.0]], [[-5.0, -4.0, -2.0, 7.0, -3.0], [6.0, -3.0, 7.0, -5.0, 5.0], [-2.0, -5.0, 4.0, 3.0, -7.0], [-2.0, 0.0, 4.0, 5.0, -4.0]], [[5.0, -2.0, -1.0, 2.0, 5.0], [4.0, 0.0, 0.0, -7.0, 6.0], [-2.0, 0.0, 0.0, 5.0, -7.0], [3.0, 6.0, 0.0, -6.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -5.0, 7.0, 0.0, -6.0], [0.0, -3.0, 7.0, -6.0, -5.0], [6.0, -7.0, -6.0, 5.0, -3.0], [-5.0, 7.0, -4.0, -5.0, 0.0]], [[4.0, -1.0, 0.0, 6.0, -4.0], [-5.0, 1.0, -4.0, 0.0, -3.0], [-1.0, 0.0, 6.0, -5.0, -7.0], [-6.0, 0.0, -2.0, 7.0, -2.0]], [[-1.0, -3.0, -3.0, -2.0, 1.0], [6.0, 5.0, 0.0, 1.0, 3.0], [-7.0, 0.0, -4.0, -5.0, 5.0], [4.0, -1.0, -7.0, 4.0, 4.0]]], [[[-2.0, 3.0, 2.0, 5.0, -5.0], [-1.0, -3.0, -1.0, 3.0, 4.0], [6.0, 0.0, -2.0, 5.0, 2.0], [-5.0, 4.0, 3.0, -6.0, 4.0]], [[-2.0, 3.0, -4.0, 6.0, 0.0], [0.0, 5.0, -3.0, 6.0, 3.0], [-7.0, -5.0, -3.0, 2.0, 1.0], [0.0, -1.0, -5.0, 2.0, 4.0]], [[-5.0, -7.0, 0.0, -1.0, -2.0], [-1.0, 6.0, -5.0, 6.0, -5.0], [-1.0, 2.0, -5.0, 5.0, 2.0], [6.0, 3.0, -1.0, -4.0, -7.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[5.0, -3.0, 0.0], [-3.0, -1.0, -7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-4.0, 35.0, -45.0, -51.0, -3.0], [-22.0, -3.0, -6.0, 59.0, -18.0], [35.0, -13.0, 24.0, -27.0, 58.0], [10.0, -25.0, 35.0, 52.0, -48.0]])+(1.-msk_ref)*numpy.array([[11.0, 15.0, 33.0, -32.0, 11.0], [25.0, -56.0, 88.0, -87.0, 4.0], [29.0, -44.0, -4.0, -12.0, -15.0], [-34.0, 3.0, -11.0, -2.0, 39.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 0.0, -5.0, 1.0, -4.0], [2.0, -3.0, -2.0, 7.0, 3.0], [-5.0, -2.0, 2.0, 4.0, -5.0], [-7.0, -7.0, -4.0, 5.0, 2.0]])+(1.-msk_arg0)*numpy.array([[0.0, 3.0, -6.0, 0.0, -4.0], [-7.0, -2.0, 2.0, -7.0, 6.0], [-6.0, 7.0, 3.0, -5.0, -6.0], [-5.0, -7.0, 0.0, 5.0, -5.0]])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-7.0, 0.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[14.0, 0.0], [0.0, 0.0], [35.0, 0.0], [-7.0, 0.0], [28.0, 0.0]], [[-14.0, 0.0], [21.0, 0.0], [14.0, 0.0], [-49.0, 0.0], [-21.0, 0.0]], [[35.0, 0.0], [14.0, 0.0], [-14.0, 0.0], [-28.0, 0.0], [35.0, 0.0]], [[49.0, 0.0], [49.0, 0.0], [28.0, 0.0], [-35.0, 0.0], [-14.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 0.0], [-21.0, 0.0], [42.0, 0.0], [0.0, 0.0], [28.0, 0.0]], [[49.0, 0.0], [14.0, 0.0], [-14.0, 0.0], [49.0, 0.0], [-42.0, 0.0]], [[42.0, 0.0], [-49.0, 0.0], [-21.0, 0.0], [35.0, 0.0], [42.0, 0.0]], [[35.0, 0.0], [49.0, 0.0], [0.0, 0.0], [-35.0, 0.0], [35.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[7.0, -4.0, -2.0, 5.0, 2.0], [4.0, -1.0, -4.0, 0.0, 6.0], [6.0, 0.0, -6.0, -4.0, 0.0], [6.0, 3.0, 2.0, 4.0, 0.0]], [[-3.0, 4.0, 0.0, 5.0, 0.0], [-5.0, 0.0, 4.0, -1.0, 7.0], [-4.0, -1.0, -4.0, -6.0, -1.0], [7.0, -3.0, 2.0, 0.0, 0.0]], [[0.0, -3.0, 0.0, -5.0, 4.0], [0.0, 6.0, -7.0, 6.0, 2.0], [-4.0, -4.0, -1.0, 2.0, 7.0], [0.0, 3.0, 5.0, 0.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 1.0, 4.0, 6.0, -7.0], [-2.0, 5.0, 0.0, -2.0, 7.0], [3.0, 3.0, -3.0, 6.0, 5.0], [0.0, 0.0, -6.0, -5.0, 7.0]], [[-2.0, -3.0, 0.0, -3.0, 0.0], [3.0, -1.0, 2.0, 6.0, -4.0], [-5.0, -5.0, 0.0, 1.0, 5.0], [0.0, 4.0, -4.0, 3.0, -5.0]], [[4.0, 6.0, -7.0, -7.0, 0.0], [0.0, -7.0, 0.0, -6.0, 0.0], [-3.0, 1.0, -3.0, -2.0, 0.0], [3.0, -7.0, 2.0, 6.0, 0.0]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-3.0, 2.0], [-2.0, -4.0], [-6.0, 1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-15.0, 26.0], [22.0, -27.0], [6.0, -4.0], [5.0, -15.0], [-30.0, 8.0]], [[-2.0, 28.0], [-33.0, 4.0], [46.0, -31.0], [-34.0, 10.0], [-44.0, -14.0]], [[14.0, 24.0], [26.0, 0.0], [32.0, 3.0], [12.0, 18.0], [-40.0, 11.0]], [[-32.0, -16.0], [-21.0, 21.0], [-40.0, 1.0], [-12.0, 8.0], [0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-29.0, 18.0], [-33.0, 20.0], [30.0, 1.0], [30.0, 17.0], [21.0, -14.0]], [[0.0, -16.0], [29.0, 7.0], [-4.0, -8.0], [30.0, -34.0], [-13.0, 30.0]], [[19.0, 23.0], [-5.0, 27.0], [27.0, -9.0], [-8.0, 6.0], [-25.0, -10.0]], [[-18.0, 3.0], [34.0, -23.0], [14.0, 6.0], [-27.0, -16.0], [-11.0, 34.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 5.0, 1.0, 1.0, -1.0], [0.0, 2.0, -6.0, -5.0, -4.0], [-7.0, 2.0, -5.0, -4.0, 7.0], [3.0, 2.0, -5.0, -5.0, 7.0]], [[-4.0, 0.0, 3.0, 3.0, -5.0], [-4.0, 1.0, -3.0, -7.0, 5.0], [1.0, 6.0, 3.0, 5.0, 5.0], [-4.0, 4.0, 4.0, -1.0, 4.0]], [[1.0, 1.0, -2.0, -7.0, 6.0], [-5.0, 0.0, 6.0, -1.0, 5.0], [1.0, 7.0, 0.0, -1.0, -4.0], [-7.0, -7.0, 6.0, -6.0, 7.0]]], [[[-1.0, -4.0, 6.0, -4.0, -5.0], [-5.0, 5.0, -4.0, 1.0, 7.0], [1.0, 4.0, 3.0, 7.0, 0.0], [2.0, -3.0, -2.0, 3.0, -7.0]], [[5.0, 7.0, 6.0, -2.0, -5.0], [-4.0, 6.0, -5.0, -1.0, 1.0], [-1.0, -4.0, -5.0, 5.0, 0.0], [6.0, 0.0, -7.0, 4.0, 1.0]], [[-3.0, 6.0, 0.0, -1.0, 4.0], [-1.0, 7.0, 6.0, -5.0, -5.0], [1.0, -5.0, 0.0, -7.0, -7.0], [-4.0, 6.0, -5.0, 0.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -7.0, -1.0, 7.0, 0.0], [1.0, -4.0, 4.0, 1.0, -3.0], [7.0, -3.0, 0.0, -5.0, -5.0], [7.0, -1.0, 3.0, 1.0, 6.0]], [[5.0, 0.0, -4.0, 0.0, -2.0], [7.0, -3.0, -2.0, -5.0, -2.0], [2.0, 0.0, 7.0, 3.0, 1.0], [-5.0, -5.0, 6.0, -6.0, -4.0]], [[-4.0, 4.0, 4.0, 3.0, -1.0], [6.0, -6.0, 2.0, -4.0, 4.0], [-5.0, 5.0, 0.0, 0.0, 7.0], [4.0, -6.0, 1.0, 4.0, 2.0]]], [[[0.0, -1.0, -5.0, -1.0, -7.0], [-7.0, -3.0, -4.0, 0.0, -5.0], [-4.0, -1.0, 5.0, 3.0, 0.0], [4.0, 6.0, 5.0, 6.0, -4.0]], [[-5.0, 3.0, -5.0, 6.0, -4.0], [-3.0, -1.0, -2.0, -1.0, -4.0], [-3.0, -5.0, 0.0, 3.0, 4.0], [3.0, 0.0, -4.0, 5.0, 4.0]], [[7.0, -1.0, 0.0, 0.0, -1.0], [0.0, -5.0, 2.0, 4.0, 5.0], [6.0, 3.0, -1.0, 7.0, -5.0], [5.0, 6.0, -1.0, -6.0, -4.0]]]])
      arg1=Symbol(shape=(2, 3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, 2.0], [5.0, -7.0], [5.0, -4.0]], [[0.0, 0.0], [5.0, 4.0], [-5.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[25.0, 55.0], [10.0, 40.0], [35.0, 13.0], [-25.0, 0.0], [-40.0, -7.0]], [[-60.0, 31.0], [0.0, 28.0], [-40.0, -29.0], [-20.0, 34.0], [80.0, -64.0]], [[0.0, -28.0], [70.0, -87.0], [-10.0, -51.0], [80.0, -26.0], [40.0, -12.0]], [[-5.0, 82.0], [-45.0, 10.0], [40.0, -95.0], [-15.0, 37.0], [75.0, -41.0]]])+(1.-msk_ref)*numpy.array([[[-55.0, -32.0], [40.0, -19.0], [-25.0, -10.0], [45.0, 26.0], [-30.0, 1.0]], [[50.0, -83.0], [-25.0, 28.0], [-20.0, 8.0], [-70.0, 53.0], [-35.0, -19.0]], [[-60.0, 14.0], [-15.0, -43.0], [40.0, -50.0], [-5.0, -12.0], [85.0, -34.0]], [[-15.0, 50.0], [-85.0, 63.0], [20.0, -57.0], [45.0, 42.0], [30.0, 44.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 7.0, -5.0, -7.0, 2.0], [4.0, 6.0, 4.0, -5.0, 0.0], [-7.0, -4.0, 2.0, 5.0, -5.0], [7.0, 7.0, 2.0, 2.0, 3.0]])+(1.-msk_arg0)*numpy.array([[-4.0, 0.0, -1.0, -7.0, 4.0], [3.0, 3.0, 0.0, 0.0, 1.0], [4.0, 5.0, -3.0, 5.0, 4.0], [-2.0, -6.0, -4.0, -1.0, 1.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[3.0, 0.0, 0.0, -1.0, 2.0], [7.0, -5.0, -5.0, 3.0, 1.0], [5.0, -7.0, 1.0, -4.0, -6.0], [0.0, 0.0, 4.0, -3.0, 3.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-12.0, 0.0, 0.0, 4.0, -8.0], [-28.0, 20.0, 20.0, -12.0, -4.0], [-20.0, 28.0, -4.0, 16.0, 24.0], [0.0, 0.0, -16.0, 12.0, -12.0]], [[21.0, 0.0, 0.0, -7.0, 14.0], [49.0, -35.0, -35.0, 21.0, 7.0], [35.0, -49.0, 7.0, -28.0, -42.0], [0.0, 0.0, 28.0, -21.0, 21.0]], [[-15.0, 0.0, 0.0, 5.0, -10.0], [-35.0, 25.0, 25.0, -15.0, -5.0], [-25.0, 35.0, -5.0, 20.0, 30.0], [0.0, 0.0, -20.0, 15.0, -15.0]], [[-21.0, 0.0, 0.0, 7.0, -14.0], [-49.0, 35.0, 35.0, -21.0, -7.0], [-35.0, 49.0, -7.0, 28.0, 42.0], [0.0, 0.0, -28.0, 21.0, -21.0]], [[6.0, 0.0, 0.0, -2.0, 4.0], [14.0, -10.0, -10.0, 6.0, 2.0], [10.0, -14.0, 2.0, -8.0, -12.0], [0.0, 0.0, 8.0, -6.0, 6.0]]], [[[12.0, 0.0, 0.0, -4.0, 8.0], [28.0, -20.0, -20.0, 12.0, 4.0], [20.0, -28.0, 4.0, -16.0, -24.0], [0.0, 0.0, 16.0, -12.0, 12.0]], [[18.0, 0.0, 0.0, -6.0, 12.0], [42.0, -30.0, -30.0, 18.0, 6.0], [30.0, -42.0, 6.0, -24.0, -36.0], [0.0, 0.0, 24.0, -18.0, 18.0]], [[12.0, 0.0, 0.0, -4.0, 8.0], [28.0, -20.0, -20.0, 12.0, 4.0], [20.0, -28.0, 4.0, -16.0, -24.0], [0.0, 0.0, 16.0, -12.0, 12.0]], [[-15.0, 0.0, 0.0, 5.0, -10.0], [-35.0, 25.0, 25.0, -15.0, -5.0], [-25.0, 35.0, -5.0, 20.0, 30.0], [0.0, 0.0, -20.0, 15.0, -15.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-21.0, 0.0, 0.0, 7.0, -14.0], [-49.0, 35.0, 35.0, -21.0, -7.0], [-35.0, 49.0, -7.0, 28.0, 42.0], [0.0, 0.0, -28.0, 21.0, -21.0]], [[-12.0, 0.0, 0.0, 4.0, -8.0], [-28.0, 20.0, 20.0, -12.0, -4.0], [-20.0, 28.0, -4.0, 16.0, 24.0], [0.0, 0.0, -16.0, 12.0, -12.0]], [[6.0, 0.0, 0.0, -2.0, 4.0], [14.0, -10.0, -10.0, 6.0, 2.0], [10.0, -14.0, 2.0, -8.0, -12.0], [0.0, 0.0, 8.0, -6.0, 6.0]], [[15.0, 0.0, 0.0, -5.0, 10.0], [35.0, -25.0, -25.0, 15.0, 5.0], [25.0, -35.0, 5.0, -20.0, -30.0], [0.0, 0.0, 20.0, -15.0, 15.0]], [[-15.0, 0.0, 0.0, 5.0, -10.0], [-35.0, 25.0, 25.0, -15.0, -5.0], [-25.0, 35.0, -5.0, 20.0, 30.0], [0.0, 0.0, -20.0, 15.0, -15.0]]], [[[21.0, 0.0, 0.0, -7.0, 14.0], [49.0, -35.0, -35.0, 21.0, 7.0], [35.0, -49.0, 7.0, -28.0, -42.0], [0.0, 0.0, 28.0, -21.0, 21.0]], [[21.0, 0.0, 0.0, -7.0, 14.0], [49.0, -35.0, -35.0, 21.0, 7.0], [35.0, -49.0, 7.0, -28.0, -42.0], [0.0, 0.0, 28.0, -21.0, 21.0]], [[6.0, 0.0, 0.0, -2.0, 4.0], [14.0, -10.0, -10.0, 6.0, 2.0], [10.0, -14.0, 2.0, -8.0, -12.0], [0.0, 0.0, 8.0, -6.0, 6.0]], [[6.0, 0.0, 0.0, -2.0, 4.0], [14.0, -10.0, -10.0, 6.0, 2.0], [10.0, -14.0, 2.0, -8.0, -12.0], [0.0, 0.0, 8.0, -6.0, 6.0]], [[9.0, 0.0, 0.0, -3.0, 6.0], [21.0, -15.0, -15.0, 9.0, 3.0], [15.0, -21.0, 3.0, -12.0, -18.0], [0.0, 0.0, 12.0, -9.0, 9.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, 0.0, 0.0, 4.0, -8.0], [-28.0, 20.0, 20.0, -12.0, -4.0], [-20.0, 28.0, -4.0, 16.0, 24.0], [0.0, 0.0, -16.0, 12.0, -12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-3.0, 0.0, 0.0, 1.0, -2.0], [-7.0, 5.0, 5.0, -3.0, -1.0], [-5.0, 7.0, -1.0, 4.0, 6.0], [0.0, 0.0, -4.0, 3.0, -3.0]], [[-21.0, 0.0, 0.0, 7.0, -14.0], [-49.0, 35.0, 35.0, -21.0, -7.0], [-35.0, 49.0, -7.0, 28.0, 42.0], [0.0, 0.0, -28.0, 21.0, -21.0]], [[12.0, 0.0, 0.0, -4.0, 8.0], [28.0, -20.0, -20.0, 12.0, 4.0], [20.0, -28.0, 4.0, -16.0, -24.0], [0.0, 0.0, 16.0, -12.0, 12.0]]], [[[9.0, 0.0, 0.0, -3.0, 6.0], [21.0, -15.0, -15.0, 9.0, 3.0], [15.0, -21.0, 3.0, -12.0, -18.0], [0.0, 0.0, 12.0, -9.0, 9.0]], [[9.0, 0.0, 0.0, -3.0, 6.0], [21.0, -15.0, -15.0, 9.0, 3.0], [15.0, -21.0, 3.0, -12.0, -18.0], [0.0, 0.0, 12.0, -9.0, 9.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[3.0, 0.0, 0.0, -1.0, 2.0], [7.0, -5.0, -5.0, 3.0, 1.0], [5.0, -7.0, 1.0, -4.0, -6.0], [0.0, 0.0, 4.0, -3.0, 3.0]]], [[[12.0, 0.0, 0.0, -4.0, 8.0], [28.0, -20.0, -20.0, 12.0, 4.0], [20.0, -28.0, 4.0, -16.0, -24.0], [0.0, 0.0, 16.0, -12.0, 12.0]], [[15.0, 0.0, 0.0, -5.0, 10.0], [35.0, -25.0, -25.0, 15.0, 5.0], [25.0, -35.0, 5.0, -20.0, -30.0], [0.0, 0.0, 20.0, -15.0, 15.0]], [[-9.0, 0.0, 0.0, 3.0, -6.0], [-21.0, 15.0, 15.0, -9.0, -3.0], [-15.0, 21.0, -3.0, 12.0, 18.0], [0.0, 0.0, -12.0, 9.0, -9.0]], [[15.0, 0.0, 0.0, -5.0, 10.0], [35.0, -25.0, -25.0, 15.0, 5.0], [25.0, -35.0, 5.0, -20.0, -30.0], [0.0, 0.0, 20.0, -15.0, 15.0]], [[12.0, 0.0, 0.0, -4.0, 8.0], [28.0, -20.0, -20.0, 12.0, 4.0], [20.0, -28.0, 4.0, -16.0, -24.0], [0.0, 0.0, 16.0, -12.0, 12.0]]], [[[-6.0, 0.0, 0.0, 2.0, -4.0], [-14.0, 10.0, 10.0, -6.0, -2.0], [-10.0, 14.0, -2.0, 8.0, 12.0], [0.0, 0.0, -8.0, 6.0, -6.0]], [[-18.0, 0.0, 0.0, 6.0, -12.0], [-42.0, 30.0, 30.0, -18.0, -6.0], [-30.0, 42.0, -6.0, 24.0, 36.0], [0.0, 0.0, -24.0, 18.0, -18.0]], [[-12.0, 0.0, 0.0, 4.0, -8.0], [-28.0, 20.0, 20.0, -12.0, -4.0], [-20.0, 28.0, -4.0, 16.0, 24.0], [0.0, 0.0, -16.0, 12.0, -12.0]], [[-3.0, 0.0, 0.0, 1.0, -2.0], [-7.0, 5.0, 5.0, -3.0, -1.0], [-5.0, 7.0, -1.0, 4.0, 6.0], [0.0, 0.0, -4.0, 3.0, -3.0]], [[3.0, 0.0, 0.0, -1.0, 2.0], [7.0, -5.0, -5.0, 3.0, 1.0], [5.0, -7.0, 1.0, -4.0, -6.0], [0.0, 0.0, 4.0, -3.0, 3.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[7.0, 5.0, -7.0, -6.0, 7.0], [-6.0, 5.0, 4.0, 1.0, 6.0], [-3.0, 3.0, 3.0, -5.0, -2.0], [0.0, 5.0, 0.0, 4.0, 2.0]], [[4.0, 4.0, -3.0, -1.0, 1.0], [3.0, 7.0, 0.0, 5.0, -5.0], [-7.0, -2.0, 0.0, 6.0, 0.0], [-5.0, -4.0, -7.0, -2.0, -1.0]], [[-1.0, -6.0, 6.0, -5.0, 4.0], [-3.0, 3.0, 1.0, 0.0, -1.0], [-7.0, -7.0, 5.0, -6.0, -1.0], [-3.0, -2.0, -3.0, -5.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 2.0, 0.0, -3.0, 3.0], [1.0, 2.0, 1.0, -1.0, -7.0], [-2.0, -6.0, 0.0, -5.0, 2.0], [7.0, 0.0, 6.0, 4.0, 1.0]], [[6.0, -4.0, -4.0, -3.0, -7.0], [-1.0, -6.0, 0.0, -4.0, 5.0], [1.0, -1.0, -2.0, -2.0, -4.0], [7.0, 3.0, -7.0, 4.0, 0.0]], [[-5.0, 6.0, 0.0, 5.0, -5.0], [0.0, -5.0, 4.0, 6.0, 4.0], [2.0, 3.0, -3.0, 0.0, 2.0], [-3.0, 2.0, -2.0, -7.0, 1.0]]])
      arg1=Symbol(shape=(3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[3.0, 3.0, -4.0, 7.0, 7.0], [-4.0, 3.0, -5.0, -4.0, -1.0], [6.0, 1.0, 0.0, -7.0, -2.0], [7.0, 0.0, -6.0, 0.0, 0.0]], [[-4.0, 2.0, -4.0, -2.0, 0.0], [6.0, 0.0, -6.0, 7.0, 5.0], [-5.0, 5.0, -5.0, 1.0, 0.0], [-6.0, -2.0, 1.0, 3.0, 0.0]], [[0.0, -7.0, 5.0, 4.0, 7.0], [7.0, 5.0, -4.0, 4.0, 7.0], [-1.0, 4.0, 2.0, -6.0, 4.0], [4.0, -3.0, -1.0, 5.0, 5.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 36.0, -49.0, 37.0, 42.0], [-11.0, 16.0, -55.0, -4.0, 6.0], [23.0, 23.0, -22.0, -39.0, -18.0], [21.0, -5.0, -37.0, 7.0, -5.0]], [[-1.0, 65.0, -66.0, 3.0, -7.0], [-38.0, -15.0, -25.0, -16.0, -27.0], [16.0, 1.0, -32.0, 5.0, -34.0], [-13.0, 10.0, -20.0, -18.0, -30.0]], [[-9.0, -69.0, 70.0, -19.0, -7.0], [52.0, 9.0, 29.0, 31.0, 34.0], [-33.0, 2.0, 27.0, 10.0, 38.0], [-7.0, -12.0, 33.0, 21.0, 30.0]], [[-14.0, 15.0, 3.0, -60.0, -77.0], [-17.0, -43.0, 56.0, -3.0, -34.0], [-26.0, -31.0, -5.0, 71.0, -8.0], [-56.0, 17.0, 40.0, -28.0, -25.0]], [[17.0, -5.0, -12.0, 63.0, 77.0], [6.0, 41.0, -57.0, -5.0, 26.0], [33.0, 28.0, 3.0, -72.0, 2.0], [59.0, -14.0, -45.0, 23.0, 20.0]]], [[[-30.0, 9.0, -3.0, -60.0, -63.0], [21.0, -33.0, 24.0, 33.0, 0.0], [-48.0, -3.0, -21.0, 63.0, 0.0], [-72.0, 3.0, 42.0, -6.0, -15.0]], [[-13.0, 8.0, -33.0, 33.0, 56.0], [43.0, 30.0, -79.0, 41.0, 51.0], [-8.0, 52.0, -29.0, -46.0, 2.0], [5.0, -23.0, -26.0, 36.0, 15.0]], [[12.0, 5.0, -11.0, 32.0, 35.0], [-9.0, 17.0, -24.0, -12.0, 3.0], [23.0, 8.0, 2.0, -34.0, -4.0], [32.0, -3.0, -25.0, 5.0, 5.0]], [[-17.0, 13.0, -24.0, -3.0, 7.0], [26.0, 3.0, -35.0, 31.0, 24.0], [-19.0, 26.0, -25.0, -2.0, -2.0], [-23.0, -10.0, -1.0, 15.0, 0.0]], [[38.0, 15.0, -9.0, 48.0, 35.0], [-61.0, 13.0, 4.0, -63.0, -38.0], [62.0, -23.0, 23.0, -41.0, -16.0], [68.0, 13.0, -40.0, -20.0, -5.0]]], [[[19.0, 26.0, 5.0, -35.0, -70.0], [-79.0, -44.0, 85.0, -65.0, -81.0], [24.0, -66.0, 21.0, 56.0, -22.0], [-7.0, 35.0, 18.0, -56.0, -35.0]], [[17.0, 54.0, -39.0, -3.0, -28.0], [-73.0, -26.0, 25.0, -54.0, -62.0], [35.0, -35.0, -4.0, 19.0, -34.0], [5.0, 25.0, -13.0, -41.0, -35.0]], [[9.0, -26.0, 13.0, 41.0, 56.0], [23.0, 34.0, -35.0, 8.0, 32.0], [13.0, 23.0, 10.0, -51.0, 14.0], [41.0, -15.0, -23.0, 25.0, 25.0]], [[-39.0, 39.0, -34.0, -71.0, -77.0], [14.0, -45.0, 13.0, 38.0, -7.0], [-54.0, 1.0, -42.0, 77.0, -14.0], [-95.0, 6.0, 42.0, -12.0, -30.0]], [[-6.0, 1.0, 3.0, -18.0, -21.0], [1.0, -11.0, 14.0, 4.0, -5.0], [-11.0, -6.0, -2.0, 20.0, 0.0], [-18.0, 3.0, 13.0, -5.0, -5.0]]], [[[20.0, 11.0, 5.0, -2.0, -21.0], [-51.0, -15.0, 42.0, -47.0, -46.0], [28.0, -37.0, 19.0, 13.0, -12.0], [18.0, 19.0, -2.0, -30.0, -15.0]], [[31.0, 21.0, -14.0, 35.0, 21.0], [-58.0, 5.0, 7.0, -56.0, -39.0], [52.0, -23.0, 16.0, -27.0, -18.0], [51.0, 14.0, -32.0, -22.0, -10.0]], [[28.0, 7.0, 13.0, 2.0, -21.0], [-63.0, -15.0, 54.0, -61.0, -56.0], [38.0, -47.0, 29.0, 11.0, -12.0], [30.0, 23.0, -4.0, -36.0, -15.0]], [[20.0, 43.0, -33.0, 12.0, -7.0], [-63.0, -13.0, 12.0, -50.0, -49.0], [39.0, -26.0, 0.0, 0.0, -28.0], [20.0, 19.0, -21.0, -31.0, -25.0]], [[10.0, -38.0, 26.0, 40.0, 56.0], [28.0, 36.0, -28.0, 9.0, 35.0], [11.0, 21.0, 17.0, -51.0, 20.0], [44.0, -16.0, -19.0, 27.0, 30.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, 41.0, -41.0, -46.0, -49.0], [9.0, -31.0, -6.0, 30.0, -3.0], [-37.0, 8.0, -40.0, 50.0, -16.0], [-70.0, 3.0, 23.0, -7.0, -25.0]], [[22.0, -44.0, 38.0, 46.0, 56.0], [10.0, 36.0, -10.0, -12.0, 20.0], [26.0, 6.0, 32.0, -54.0, 20.0], [62.0, -10.0, -22.0, 18.0, 30.0]], [[16.0, -8.0, 16.0, 8.0, 0.0], [-24.0, 0.0, 24.0, -28.0, -20.0], [20.0, -20.0, 20.0, -4.0, 0.0], [24.0, 8.0, -4.0, -12.0, 0.0]], [[3.0, -50.0, 49.0, 5.0, 14.0], [29.0, 16.0, 13.0, 11.0, 23.0], [-8.0, 2.0, 25.0, -12.0, 26.0], [17.0, -9.0, 10.0, 16.0, 25.0]], [[37.0, 30.0, -9.0, 15.0, -14.0], [-89.0, -16.0, 47.0, -81.0, -73.0], [58.0, -52.0, 25.0, 2.0, -26.0], [43.0, 29.0, -20.0, -46.0, -25.0]]], [[[7.0, 1.0, 0.0, 9.0, 7.0], [-10.0, 3.0, 1.0, -11.0, -6.0], [11.0, -4.0, 5.0, -8.0, -2.0], [13.0, 2.0, -7.0, -3.0, 0.0]], [[30.0, 29.0, -9.0, 6.0, -21.0], [-79.0, -19.0, 46.0, -70.0, -67.0], [47.0, -48.0, 20.0, 10.0, -24.0], [30.0, 27.0, -13.0, -43.0, -25.0]], [[3.0, -25.0, 16.0, 23.0, 35.0], [24.0, 23.0, -21.0, 12.0, 27.0], [2.0, 17.0, 8.0, -31.0, 14.0], [23.0, -12.0, -10.0, 20.0, 20.0]], [[13.0, -53.0, 50.0, 25.0, 35.0], [22.0, 27.0, 5.0, 0.0, 23.0], [8.0, 3.0, 32.0, -33.0, 26.0], [41.0, -10.0, -4.0, 18.0, 30.0]], [[-41.0, -39.0, 28.0, -43.0, -21.0], [86.0, -1.0, -11.0, 79.0, 60.0], [-71.0, 34.0, -17.0, 30.0, 30.0], [-63.0, -22.0, 43.0, 35.0, 20.0]]], [[[-10.0, -18.0, 14.0, -8.0, 0.0], [28.0, 4.0, -4.0, 23.0, 21.0], [-19.0, 11.0, -1.0, 3.0, 12.0], [-12.0, -8.0, 11.0, 13.0, 10.0]], [[-14.0, -41.0, 43.0, -28.0, -21.0], [39.0, -3.0, 24.0, 29.0, 22.0], [-34.0, 1.0, 11.0, 23.0, 24.0], [-24.0, -7.0, 32.0, 12.0, 15.0]], [[8.0, 17.0, -7.0, -8.0, -21.0], [-33.0, -15.0, 24.0, -26.0, -31.0], [13.0, -22.0, 4.0, 16.0, -12.0], [0.0, 13.0, 1.0, -21.0, -15.0]], [[-7.0, -19.0, 28.0, -31.0, -35.0], [8.0, -15.0, 37.0, 6.0, -5.0], [-20.0, -15.0, 10.0, 33.0, 10.0], [-23.0, 4.0, 28.0, -6.0, 0.0]], [[22.0, -16.0, 18.0, 30.0, 28.0], [-18.0, 16.0, 6.0, -28.0, -8.0], [30.0, -10.0, 24.0, -30.0, 4.0], [46.0, 2.0, -18.0, -2.0, 10.0]]], [[[-7.0, 56.0, -71.0, 23.0, 28.0], [-7.0, 6.0, -65.0, 9.0, 7.0], [10.0, 30.0, -41.0, -24.0, -26.0], [-5.0, -5.0, -32.0, 6.0, -15.0]], [[-12.0, -8.0, -2.0, 2.0, 14.0], [32.0, 10.0, -26.0, 29.0, 29.0], [-17.0, 23.0, -11.0, -9.0, 8.0], [-10.0, -12.0, 1.0, 19.0, 10.0]], [[46.0, 18.0, -6.0, 48.0, 28.0], [-80.0, 8.0, 20.0, -81.0, -55.0], [73.0, -37.0, 31.0, -37.0, -20.0], [76.0, 20.0, -41.0, -31.0, -10.0]], [[-4.0, 69.0, -67.0, -8.0, -21.0], [-41.0, -23.0, -16.0, -16.0, -33.0], [11.0, -4.0, -34.0, 18.0, -36.0], [-24.0, 13.0, -13.0, -23.0, -35.0]], [[3.0, -4.0, 1.0, 11.0, 14.0], [3.0, 8.0, -9.0, 0.0, 6.0], [5.0, 5.0, 2.0, -13.0, 2.0], [11.0, -3.0, -7.0, 5.0, 5.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 1.0, 2.0, -7.0, 3.0], [-1.0, -4.0, 7.0, -2.0, 0.0], [1.0, -5.0, 4.0, 0.0, -6.0], [-7.0, 7.0, -7.0, -6.0, 5.0]], [[-4.0, -3.0, 0.0, 5.0, 2.0], [-3.0, 1.0, 5.0, 0.0, 6.0], [1.0, 5.0, -1.0, 6.0, 3.0], [6.0, 6.0, 1.0, -3.0, -3.0]], [[2.0, -7.0, 4.0, -4.0, 3.0], [-4.0, 3.0, 5.0, 7.0, 0.0], [0.0, -2.0, 1.0, 7.0, 7.0], [0.0, -6.0, 1.0, 0.0, -2.0]]], [[[4.0, 6.0, 3.0, -4.0, 6.0], [-5.0, -2.0, -3.0, 3.0, 0.0], [-6.0, 0.0, 4.0, 4.0, 2.0], [-3.0, -4.0, -1.0, 1.0, -5.0]], [[-2.0, 2.0, -2.0, -3.0, -7.0], [6.0, 7.0, 7.0, -5.0, 1.0], [3.0, -2.0, 6.0, -1.0, -3.0], [0.0, 4.0, -1.0, 7.0, 1.0]], [[0.0, 7.0, 0.0, 5.0, -4.0], [7.0, 0.0, 2.0, -7.0, 0.0], [-2.0, 0.0, 2.0, 1.0, -1.0], [2.0, -6.0, -2.0, 4.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-2.0, -5.0, 0.0, 0.0, -4.0], [1.0, -1.0, 5.0, 0.0, -4.0], [-4.0, 1.0, 0.0, 1.0, 5.0], [4.0, 7.0, 1.0, 7.0, 4.0]], [[-6.0, 0.0, 5.0, -2.0, 2.0], [4.0, 3.0, -4.0, -1.0, 0.0], [0.0, 7.0, 0.0, 6.0, 3.0], [-2.0, 4.0, -3.0, 4.0, 7.0]], [[-1.0, -7.0, 2.0, -7.0, -4.0], [2.0, 1.0, -6.0, 3.0, -2.0], [7.0, 0.0, 0.0, -2.0, -7.0], [1.0, 1.0, 5.0, 6.0, -1.0]]], [[[3.0, 2.0, 3.0, -6.0, 0.0], [1.0, -7.0, -3.0, -7.0, 3.0], [6.0, -5.0, 6.0, 3.0, 5.0], [-7.0, -7.0, 3.0, 3.0, 2.0]], [[2.0, -3.0, 7.0, -5.0, 3.0], [-2.0, -4.0, -5.0, -1.0, -3.0], [-2.0, 3.0, 5.0, -1.0, 0.0], [7.0, -2.0, 0.0, 6.0, -4.0]], [[-6.0, -3.0, -2.0, 0.0, 0.0], [-4.0, -6.0, -1.0, -4.0, 6.0], [-5.0, 7.0, 4.0, 0.0, -1.0], [4.0, 2.0, 4.0, 2.0, 2.0]]]])
      arg1=Symbol(shape=(2, 3, 4, 5))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[6.0, 1.0, 1.0, 4.0, -7.0], [-2.0, -6.0, 2.0, 7.0, -5.0], [0.0, 5.0, -5.0, 1.0, 6.0], [0.0, 5.0, 4.0, -3.0, -2.0]], [[-2.0, -5.0, 5.0, 2.0, -2.0], [-5.0, 1.0, -1.0, -4.0, -3.0], [-3.0, -3.0, -7.0, -4.0, -3.0], [1.0, 5.0, -7.0, 0.0, 6.0]], [[0.0, 0.0, 2.0, -2.0, -5.0], [2.0, -4.0, 1.0, 5.0, -3.0], [3.0, 2.0, -4.0, -2.0, 0.0], [-5.0, -4.0, -7.0, -7.0, -3.0]]], [[[7.0, 0.0, 0.0, -7.0, -3.0], [-1.0, 5.0, -6.0, -3.0, 1.0], [-5.0, 2.0, 0.0, -4.0, 0.0], [1.0, -6.0, 7.0, 0.0, 6.0]], [[-4.0, -2.0, -6.0, -4.0, 4.0], [6.0, -6.0, 2.0, 3.0, -4.0], [5.0, 1.0, 0.0, 0.0, 1.0], [4.0, 7.0, -7.0, -1.0, 0.0]], [[-4.0, -7.0, -2.0, -7.0, 4.0], [3.0, 1.0, -7.0, 1.0, -4.0], [-5.0, -1.0, 0.0, 2.0, 7.0], [0.0, 3.0, -2.0, 1.0, -1.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[74.0, 29.0, 1.0, -12.0, -57.0], [-2.0, -10.0, -12.0, 43.0, -7.0], [-12.0, 47.0, -5.0, 1.0, 40.0], [-18.0, -41.0, 76.0, -27.0, -16.0]], [[18.0, -37.0, -54.0, -87.0, 52.0], [26.0, 44.0, -83.0, -21.0, -5.0], [-67.0, 7.0, 44.0, 17.0, 66.0], [46.0, 17.0, 88.0, 51.0, 30.0]], [[41.0, 6.0, 22.0, -13.0, -51.0], [-11.0, -1.0, -14.0, 19.0, -11.0], [-13.0, 22.0, -26.0, -18.0, 10.0], [-25.0, -38.0, 15.0, -32.0, 2.0]], [[-88.0, -61.0, 18.0, -5.0, 79.0], [-18.0, 66.0, -40.0, -81.0, 20.0], [-47.0, -74.0, 16.0, 7.0, -25.0], [9.0, 24.0, -52.0, 57.0, 27.0]], [[100.0, 35.0, 69.0, 24.0, -102.0], [-70.0, 40.0, -15.0, -15.0, 20.0], [-42.0, 24.0, -41.0, -43.0, -23.0], [-35.0, -84.0, 76.0, -27.0, 37.0]]], [[[-87.0, -47.0, -74.0, -40.0, 100.0], [71.0, -35.0, -10.0, 25.0, -31.0], [17.0, -15.0, 42.0, 53.0, 58.0], [36.0, 89.0, -46.0, 32.0, -41.0]], [[-68.0, -23.0, -35.0, -34.0, 45.0], [53.0, -39.0, 20.0, 10.0, -22.0], [51.0, -14.0, 1.0, -6.0, -20.0], [12.0, 34.0, -107.0, -16.0, -7.0]], [[-25.0, -46.0, -4.0, 7.0, -39.0], [22.0, -112.0, 32.0, 86.0, -104.0], [40.0, 29.0, -90.0, -7.0, 48.0], [5.0, 113.0, -116.0, -61.0, -19.0]], [[57.0, 57.0, 56.0, 26.0, -78.0], [-36.0, 22.0, 24.0, -10.0, 40.0], [16.0, 12.0, -18.0, -42.0, -66.0], [-52.0, -112.0, 13.0, -45.0, 8.0]], [[-16.0, -32.0, 24.0, 8.0, -8.0], [-24.0, 0.0, -4.0, -21.0, -22.0], [-13.0, -17.0, -42.0, -24.0, -17.0], [10.0, 37.0, -49.0, -1.0, 36.0]]], [[[-42.0, 4.0, -8.0, 50.0, 13.0], [11.0, -55.0, 57.0, 28.0, -18.0], [52.0, -5.0, -12.0, 17.0, -8.0], [7.0, 61.0, -62.0, -8.0, -30.0]], [[-32.0, -26.0, 28.0, 2.0, 27.0], [-31.0, 55.0, -21.0, -71.0, 24.0], [-31.0, -46.0, -2.0, -21.0, -47.0], [7.0, -6.0, -27.0, 31.0, 46.0]], [[22.0, -17.0, -39.0, -54.0, -11.0], [37.0, -43.0, -16.0, 45.0, -48.0], [6.0, 37.0, -17.0, -6.0, 47.0], [22.0, 35.0, -2.0, -23.0, 5.0]], [[16.0, -35.0, 48.0, -33.0, -59.0], [-23.0, 5.0, -32.0, -3.0, -35.0], [-27.0, 2.0, -70.0, -52.0, -12.0], [-29.0, -26.0, -58.0, -47.0, 38.0]], [[-12.0, -8.0, 43.0, -27.0, -21.0], [-12.0, 38.0, -19.0, -35.0, 18.0], [-8.0, -23.0, -19.0, -42.0, -55.0], [-42.0, -79.0, -57.0, -29.0, 22.0]]], [[[-83.0, -51.0, 19.0, -9.0, 54.0], [-7.0, 35.0, -16.0, -62.0, 6.0], [-13.0, -61.0, -7.0, -15.0, -46.0], [3.0, 19.0, -95.0, 23.0, 30.0]], [[10.0, 11.0, 13.0, 106.0, -27.0], [-46.0, -62.0, 76.0, 13.0, -31.0], [34.0, 7.0, -53.0, -1.0, -14.0], [48.0, 123.0, -16.0, 11.0, 22.0]], [[-39.0, 4.0, 10.0, -3.0, 33.0], [0.0, 38.0, 4.0, -50.0, 40.0], [10.0, -37.0, 24.0, -13.0, -60.0], [-9.0, -41.0, -38.0, 13.0, 13.0]], [[-67.0, -33.0, -71.0, -93.0, 89.0], [80.0, 0.0, -29.0, -8.0, -4.0], [19.0, -16.0, 51.0, 10.0, 8.0], [26.0, 10.0, -53.0, 15.0, -4.0]], [[-31.0, -31.0, -34.0, 0.0, 28.0], [33.0, -49.0, -6.0, 62.0, -47.0], [-2.0, 14.0, 4.0, 55.0, 89.0], [6.0, 76.0, -1.0, 5.0, -59.0]]]])+(1.-msk_ref)*numpy.array([[[[37.0, 66.0, -34.0, -5.0, 6.0], [23.0, 7.0, 29.0, -4.0, 50.0], [40.0, 20.0, 56.0, 0.0, -34.0], [10.0, -58.0, 60.0, 5.0, -5.0]], [[8.0, 22.0, 5.0, 13.0, 40.0], [-33.0, 83.0, -14.0, -88.0, 72.0], [-31.0, -35.0, 53.0, -5.0, -54.0], [25.0, -39.0, 70.0, 64.0, 46.0]], [[-9.0, -25.0, -9.0, -29.0, -9.0], [12.0, -32.0, 7.0, 0.0, -38.0], [21.0, 4.0, -43.0, -40.0, -22.0], [26.0, 42.0, -73.0, -23.0, 44.0]], [[-18.0, 20.0, 6.0, 72.0, 37.0], [-28.0, 26.0, 21.0, -24.0, 41.0], [-10.0, -25.0, 42.0, 46.0, 1.0], [7.0, 19.0, 56.0, 54.0, -27.0]], [[-40.0, -20.0, -20.0, -16.0, 56.0], [8.0, 24.0, -8.0, -47.0, 14.0], [-3.0, -31.0, 22.0, -4.0, -27.0], [34.0, 27.0, -23.0, 37.0, 32.0]]], [[[29.0, 13.0, 45.0, 37.0, -52.0], [-43.0, 3.0, 18.0, -12.0, 2.0], [-1.0, 1.0, -41.0, -31.0, -36.0], [-13.0, -15.0, -9.0, -19.0, 26.0]], [[-21.0, 34.0, 52.0, 107.0, -23.0], [-46.0, -12.0, 72.0, -11.0, 26.0], [39.0, -24.0, -20.0, 1.0, -61.0], [-25.0, 2.0, -41.0, -6.0, -19.0]], [[41.0, 42.0, 5.0, 72.0, -12.0], [-32.0, 4.0, 23.0, 14.0, 26.0], [-11.0, 15.0, 27.0, 43.0, 30.0], [3.0, 9.0, 106.0, 31.0, -33.0]], [[-27.0, 35.0, 15.0, 73.0, -12.0], [0.0, -46.0, 72.0, 33.0, 7.0], [62.0, -2.0, -5.0, 18.0, -26.0], [-27.0, 6.0, -48.0, -24.0, -53.0]], [[-15.0, -40.0, -2.0, -63.0, 41.0], [1.0, 71.0, -76.0, -50.0, 17.0], [-66.0, -27.0, 28.0, 0.0, 15.0], [1.0, -33.0, 28.0, 35.0, 26.0]]], [[[46.0, 35.0, 32.0, -29.0, -53.0], [-11.0, 33.0, -6.0, -22.0, 33.0], [6.0, 9.0, -8.0, -52.0, -61.0], [-37.0, -113.0, 1.0, -40.0, 28.0]], [[-83.0, -89.0, 4.0, -8.0, 34.0], [7.0, -35.0, -18.0, 10.0, -71.0], [-16.0, -30.0, -54.0, 7.0, 37.0], [14.0, 112.0, -115.0, 1.0, 3.0]], [[6.0, -38.0, -38.0, -90.0, 18.0], [36.0, 4.0, -54.0, 1.0, -30.0], [-25.0, 13.0, 0.0, -16.0, 33.0], [26.0, 11.0, -1.0, -1.0, 32.0]], [[19.0, -27.0, 33.0, 3.0, -22.0], [-45.0, 29.0, -26.0, -39.0, -10.0], [-44.0, -12.0, -39.0, -31.0, -13.0], [15.0, 18.0, 4.0, 12.0, 58.0]], [[63.0, -3.0, 8.0, 12.0, -25.0], [-47.0, 25.0, -23.0, -28.0, -4.0], [-50.0, 13.0, -18.0, -15.0, 14.0], [43.0, 35.0, 85.0, 33.0, 60.0]]], [[[-65.0, -28.0, -54.0, 3.0, 36.0], [65.0, -103.0, 39.0, 87.0, -68.0], [59.0, 17.0, -10.0, 46.0, 65.0], [14.0, 109.0, -83.0, -22.0, -69.0]], [[-15.0, -23.0, 37.0, 77.0, -41.0], [-31.0, -63.0, 35.0, 55.0, -57.0], [6.0, 7.0, -67.0, 21.0, 42.0], [-16.0, 85.0, -46.0, -24.0, -37.0]], [[17.0, -12.0, -12.0, -61.0, -19.0], [32.0, -10.0, -36.0, 39.0, -24.0], [-11.0, 26.0, -4.0, -1.0, 43.0], [-25.0, -36.0, 3.0, -34.0, -21.0]], [[23.0, -39.0, -1.0, -35.0, -64.0], [17.0, -81.0, -4.0, 74.0, -94.0], [11.0, 45.0, -87.0, -29.0, 50.0], [1.0, 61.0, -67.0, -67.0, 8.0]], [[32.0, -37.0, 57.0, 20.0, -51.0], [-65.0, 23.0, -34.0, -21.0, -28.0], [-64.0, -5.0, -65.0, -26.0, 13.0], [-2.0, 25.0, 12.0, 1.0, 47.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 0.0, 7.0, 4.0, -6.0], [5.0, -5.0, 7.0, 3.0, 3.0], [0.0, 4.0, -4.0, 1.0, 1.0], [-3.0, 7.0, -3.0, -3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[4.0, 0.0, 3.0, 4.0, -1.0], [-7.0, 7.0, -7.0, 5.0, 6.0], [-4.0, -1.0, -1.0, -1.0, 7.0], [0.0, -7.0, -2.0, 1.0, -1.0]])
      arg1=Data(0.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0, 0.0]])+(1.-msk_ref)*numpy.array([[0.0, 0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0, 0.0], [-0.0, -0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0, -0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, 5.0, -2.0, -3.0, 3.0], [1.0, 4.0, -2.0, 0.0, 4.0], [5.0, 1.0, -2.0, 0.0, 5.0], [7.0, -7.0, -3.0, -4.0, -3.0]], [[4.0, 1.0, -2.0, 5.0, -4.0], [7.0, 6.0, -1.0, -7.0, -6.0], [-2.0, 5.0, 5.0, 6.0, 6.0], [0.0, 6.0, -6.0, 7.0, 3.0]], [[-5.0, -5.0, -2.0, -3.0, 2.0], [-6.0, -5.0, -1.0, -3.0, -1.0], [-6.0, 6.0, 4.0, 0.0, -1.0], [-7.0, -4.0, 0.0, 3.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[4.0, 1.0, -5.0, 7.0, 2.0], [6.0, 1.0, 1.0, -6.0, 0.0], [-6.0, 1.0, 5.0, 6.0, 0.0], [-4.0, 7.0, 5.0, -6.0, 7.0]], [[0.0, -1.0, -5.0, 0.0, 1.0], [4.0, -3.0, 1.0, 0.0, -5.0], [6.0, 0.0, 3.0, -6.0, 0.0], [3.0, 7.0, -7.0, -5.0, 2.0]], [[6.0, 4.0, 0.0, 4.0, 3.0], [3.0, -5.0, 5.0, -7.0, 5.0], [-1.0, 4.0, -7.0, 6.0, -5.0], [2.0, 0.0, 0.0, 6.0, 0.0]]])
      arg1=Data(numpy.array([0.0, -6.0, -6.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 24.0, 24.0, -12.0, 12.0], [-6.0, -6.0, 12.0, 60.0, 42.0], [48.0, -66.0, -54.0, -36.0, -30.0], [42.0, -12.0, 36.0, -60.0, 12.0]])+(1.-msk_ref)*numpy.array([[-36.0, -18.0, 30.0, -24.0, -24.0], [-42.0, 48.0, -36.0, 42.0, 0.0], [-30.0, -24.0, 24.0, 0.0, 30.0], [-30.0, -42.0, 42.0, -6.0, -12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, 7.0, -5.0, -3.0, 3.0], [-2.0, -7.0, 0.0, -5.0, 0.0], [-6.0, 5.0, 6.0, 0.0, 0.0], [1.0, 0.0, -1.0, 3.0, 1.0]], [[0.0, 5.0, -3.0, 1.0, 5.0], [-3.0, -2.0, 2.0, -4.0, -2.0], [0.0, -7.0, 4.0, 5.0, 6.0], [5.0, 1.0, 0.0, 0.0, 1.0]], [[4.0, 7.0, -4.0, -3.0, 2.0], [-2.0, 6.0, 5.0, -7.0, 3.0], [6.0, 4.0, -4.0, -6.0, 4.0], [-7.0, -2.0, -4.0, -6.0, -5.0]]], [[[-3.0, -4.0, -6.0, -6.0, 4.0], [-6.0, -2.0, -1.0, 1.0, -6.0], [-1.0, 0.0, 7.0, 2.0, 2.0], [-6.0, -7.0, -7.0, 6.0, 0.0]], [[0.0, -5.0, -7.0, -4.0, -4.0], [-2.0, 3.0, -7.0, 5.0, -4.0], [3.0, 2.0, 2.0, 5.0, -3.0], [-4.0, 5.0, -2.0, 3.0, -4.0]], [[-2.0, 3.0, -2.0, -5.0, 7.0], [3.0, 6.0, -5.0, -3.0, -2.0], [-5.0, 7.0, -5.0, 3.0, 3.0], [2.0, 7.0, -6.0, -6.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, 7.0, -5.0, 6.0, -1.0], [-5.0, -4.0, -4.0, 5.0, -1.0], [-4.0, -4.0, 1.0, 6.0, 0.0], [-5.0, 4.0, 5.0, 1.0, -7.0]], [[7.0, 3.0, 1.0, 0.0, -1.0], [-3.0, 0.0, -6.0, 4.0, 0.0], [0.0, -4.0, -4.0, 5.0, -6.0], [-5.0, -3.0, 5.0, 4.0, -7.0]], [[5.0, 0.0, 4.0, -6.0, -1.0], [2.0, 6.0, 0.0, 1.0, -2.0], [-3.0, -4.0, -7.0, -6.0, -1.0], [-4.0, -1.0, -5.0, -5.0, 5.0]]], [[[1.0, 5.0, -6.0, -7.0, -4.0], [-2.0, -2.0, 0.0, -7.0, 5.0], [-2.0, -3.0, -1.0, 6.0, -3.0], [0.0, 0.0, -4.0, -2.0, 6.0]], [[7.0, 2.0, 2.0, -2.0, -3.0], [1.0, 0.0, -3.0, -6.0, -5.0], [4.0, -2.0, 1.0, 7.0, 0.0], [2.0, -2.0, 0.0, 3.0, 3.0]], [[4.0, 6.0, 6.0, 0.0, 4.0], [7.0, 4.0, -4.0, 3.0, 0.0], [4.0, 3.0, 1.0, 4.0, -4.0], [2.0, -2.0, 1.0, 7.0, -2.0]]]])
      arg1=Data(numpy.array([[-6.0, -4.0, 3.0], [0.0, 4.0, -6.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[42.0, -79.0, 14.0, 19.0, -90.0], [-8.0, 44.0, 9.0, 63.0, 13.0], [96.0, -24.0, -26.0, -36.0, -42.0], [-75.0, -32.0, 22.0, 12.0, -83.0]])+(1.-msk_ref)*numpy.array([[-33.0, -82.0, 10.0, -62.0, -29.0], [10.0, 18.0, 60.0, -85.0, -20.0], [7.0, 2.0, -13.0, -70.0, 45.0], [34.0, -11.0, -71.0, -67.0, 109.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 0.0, 5.0, -3.0, 3.0], [3.0, -4.0, -3.0, -6.0, -4.0], [0.0, 1.0, 5.0, -4.0, -1.0], [3.0, 0.0, -7.0, -4.0, -2.0]])+(1.-msk_arg0)*numpy.array([[5.0, 6.0, 3.0, 1.0, 4.0], [-7.0, 1.0, 0.0, 0.0, 5.0], [1.0, 4.0, 0.0, -4.0, -4.0], [-5.0, -4.0, 7.0, -7.0, 1.0]])
      arg1=Data(numpy.array([3.0, -7.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 0.0], [0.0, 0.0], [15.0, -35.0], [-9.0, 21.0], [9.0, -21.0]], [[9.0, -21.0], [-12.0, 28.0], [-9.0, 21.0], [-18.0, 42.0], [-12.0, 28.0]], [[0.0, 0.0], [3.0, -7.0], [15.0, -35.0], [-12.0, 28.0], [-3.0, 7.0]], [[9.0, -21.0], [0.0, 0.0], [-21.0, 49.0], [-12.0, 28.0], [-6.0, 14.0]]])+(1.-msk_ref)*numpy.array([[[15.0, -35.0], [18.0, -42.0], [9.0, -21.0], [3.0, -7.0], [12.0, -28.0]], [[-21.0, 49.0], [3.0, -7.0], [0.0, 0.0], [0.0, 0.0], [15.0, -35.0]], [[3.0, -7.0], [12.0, -28.0], [0.0, 0.0], [-12.0, 28.0], [-12.0, 28.0]], [[-15.0, 35.0], [-12.0, 28.0], [21.0, -49.0], [-21.0, 49.0], [3.0, -7.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, -2.0, 6.0, 0.0, 2.0], [3.0, -2.0, -4.0, -5.0, 7.0], [-4.0, -5.0, -7.0, 0.0, 6.0], [0.0, 1.0, 7.0, 0.0, 5.0]], [[0.0, 7.0, -6.0, 1.0, 4.0], [-5.0, -1.0, 0.0, 4.0, 5.0], [6.0, 3.0, -3.0, -1.0, -2.0], [-2.0, -6.0, 3.0, -1.0, 1.0]], [[-1.0, 3.0, 3.0, 0.0, -3.0], [-6.0, -2.0, -2.0, -5.0, 2.0], [-7.0, 4.0, 5.0, 5.0, -7.0], [-3.0, -4.0, 1.0, -7.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -1.0, -4.0, 3.0, -4.0], [-4.0, 0.0, 0.0, 6.0, 3.0], [-1.0, 7.0, -5.0, 6.0, -6.0], [3.0, -4.0, 0.0, 2.0, 1.0]], [[5.0, -7.0, 2.0, -2.0, 2.0], [4.0, 1.0, 1.0, 0.0, 5.0], [-4.0, -6.0, -4.0, 2.0, 0.0], [6.0, -1.0, 2.0, -7.0, -2.0]], [[2.0, 0.0, 4.0, -7.0, 4.0], [1.0, 7.0, 1.0, -1.0, 0.0], [0.0, 0.0, 3.0, -7.0, -2.0], [-6.0, 0.0, 2.0, -3.0, -4.0]]])
      arg1=Data(numpy.array([[6.0, 0.0], [1.0, 7.0], [-7.0, 5.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[31.0, -5.0], [-26.0, 64.0], [9.0, -27.0], [1.0, 7.0], [37.0, 13.0]], [[55.0, -65.0], [1.0, -17.0], [-10.0, -10.0], [9.0, 3.0], [33.0, 45.0]], [[31.0, 7.0], [-55.0, 41.0], [-80.0, 4.0], [-36.0, 18.0], [83.0, -49.0]], [[19.0, -29.0], [28.0, -62.0], [38.0, 26.0], [48.0, -42.0], [31.0, 7.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 45.0], [-13.0, -49.0], [-50.0, 34.0], [65.0, -49.0], [-50.0, 34.0]], [[-27.0, 33.0], [-48.0, 42.0], [-6.0, 12.0], [43.0, -5.0], [23.0, 35.0]], [[-10.0, -28.0], [36.0, -42.0], [-55.0, -13.0], [87.0, -21.0], [-22.0, -10.0]], [[66.0, 12.0], [-25.0, -7.0], [-12.0, 24.0], [26.0, -64.0], [32.0, -34.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, 3.0, 1.0, -6.0, -3.0], [1.0, -6.0, -3.0, -3.0, -4.0], [2.0, -7.0, 2.0, 0.0, 6.0], [0.0, 4.0, -6.0, -6.0, 6.0]], [[-7.0, 0.0, -4.0, -2.0, 0.0], [2.0, 7.0, 1.0, -2.0, 3.0], [0.0, 7.0, 6.0, 4.0, -3.0], [0.0, 3.0, -3.0, 0.0, -5.0]], [[-1.0, -5.0, -4.0, -1.0, 6.0], [-2.0, -5.0, -2.0, 5.0, 0.0], [-7.0, -2.0, 6.0, -5.0, -2.0], [0.0, -6.0, 0.0, 0.0, -2.0]]], [[[-4.0, -3.0, -4.0, -7.0, 4.0], [-6.0, 6.0, 5.0, -4.0, -3.0], [-6.0, 6.0, 1.0, 5.0, -1.0], [-7.0, 1.0, 7.0, 4.0, 2.0]], [[7.0, -2.0, -1.0, -5.0, -4.0], [4.0, 4.0, -4.0, 5.0, 7.0], [5.0, -4.0, -2.0, 6.0, 6.0], [-2.0, -5.0, 2.0, 6.0, 7.0]], [[-2.0, 1.0, -3.0, 0.0, 0.0], [-1.0, -6.0, 0.0, 3.0, -1.0], [7.0, 4.0, 5.0, 6.0, 5.0], [-2.0, 1.0, 0.0, 2.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-2.0, 5.0, 0.0, 6.0, -6.0], [3.0, 5.0, 0.0, -6.0, -3.0], [-5.0, 0.0, -6.0, 7.0, -6.0], [7.0, -3.0, -6.0, -1.0, 4.0]], [[-7.0, 3.0, 2.0, -6.0, 5.0], [1.0, 0.0, 4.0, 5.0, -1.0], [-6.0, 3.0, -3.0, 5.0, 0.0], [-4.0, -5.0, 7.0, 0.0, -5.0]], [[-4.0, -7.0, 1.0, 5.0, -4.0], [0.0, 0.0, 3.0, 2.0, 2.0], [4.0, -3.0, 1.0, 5.0, 1.0], [7.0, -7.0, 1.0, 2.0, 0.0]]], [[[4.0, 7.0, -7.0, 0.0, -7.0], [6.0, -2.0, -4.0, 0.0, -6.0], [6.0, 5.0, 0.0, -6.0, 3.0], [7.0, -2.0, 2.0, 5.0, -7.0]], [[2.0, -6.0, 5.0, 6.0, -7.0], [0.0, 7.0, -1.0, 2.0, -5.0], [-2.0, -7.0, -1.0, 2.0, 1.0], [-6.0, -4.0, -1.0, -1.0, -6.0]], [[-5.0, 4.0, -1.0, 5.0, -6.0], [7.0, -7.0, -6.0, -7.0, -1.0], [5.0, 0.0, 2.0, 3.0, 0.0], [0.0, 0.0, 2.0, 1.0, 5.0]]]])
      arg1=Data(numpy.array([[[3.0, -3.0], [0.0, 6.0], [-1.0, 4.0]], [[6.0, 3.0], [-6.0, 5.0], [4.0, 1.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-70.0, -28.0], [12.0, -47.0], [-23.0, -63.0], [-29.0, -44.0], [33.0, 25.0]], [[-59.0, 2.0], [-25.0, 72.0], [47.0, 2.0], [-56.0, 33.0], [-76.0, 55.0]], [[-25.0, -20.0], [57.0, 57.0], [38.0, 52.0], [23.0, 55.0], [-2.0, -12.0]], [[-38.0, -33.0], [58.0, -39.0], [12.0, 31.0], [-22.0, 62.0], [-10.0, -15.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -35.0], [116.0, -30.0], [-77.0, 19.0], [-3.0, 1.0], [-38.0, -30.0]], [[73.0, 22.0], [-67.0, 7.0], [-45.0, 13.0], [-60.0, 59.0], [-21.0, -33.0]], [[49.0, 8.0], [75.0, -14.0], [-5.0, 1.0], [-20.0, 24.0], [-7.0, 36.0]], [[92.0, -26.0], [10.0, -75.0], [7.0, 67.0], [35.0, 22.0], [26.0, -88.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 4.0, -6.0, 7.0, -3.0], [3.0, -6.0, 0.0, -6.0, 0.0], [1.0, 2.0, -3.0, -1.0, 6.0], [2.0, -3.0, 7.0, 7.0, 0.0]])+(1.-msk_arg0)*numpy.array([[2.0, 7.0, 7.0, 6.0, -6.0], [-3.0, 6.0, 0.0, -2.0, 1.0], [5.0, 6.0, 0.0, 4.0, -6.0], [-5.0, 5.0, 2.0, 3.0, 2.0]])
      arg1=Data(numpy.array([[-2.0, 4.0, 2.0, 3.0, -7.0], [3.0, 6.0, -2.0, -2.0, -3.0], [-6.0, -1.0, -3.0, 6.0, -4.0], [-2.0, -5.0, -5.0, 4.0, -3.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-8.0, 16.0, 8.0, 12.0, -28.0], [12.0, 24.0, -8.0, -8.0, -12.0], [-24.0, -4.0, -12.0, 24.0, -16.0], [-8.0, -20.0, -20.0, 16.0, -12.0]], [[-8.0, 16.0, 8.0, 12.0, -28.0], [12.0, 24.0, -8.0, -8.0, -12.0], [-24.0, -4.0, -12.0, 24.0, -16.0], [-8.0, -20.0, -20.0, 16.0, -12.0]], [[12.0, -24.0, -12.0, -18.0, 42.0], [-18.0, -36.0, 12.0, 12.0, 18.0], [36.0, 6.0, 18.0, -36.0, 24.0], [12.0, 30.0, 30.0, -24.0, 18.0]], [[-14.0, 28.0, 14.0, 21.0, -49.0], [21.0, 42.0, -14.0, -14.0, -21.0], [-42.0, -7.0, -21.0, 42.0, -28.0], [-14.0, -35.0, -35.0, 28.0, -21.0]], [[6.0, -12.0, -6.0, -9.0, 21.0], [-9.0, -18.0, 6.0, 6.0, 9.0], [18.0, 3.0, 9.0, -18.0, 12.0], [6.0, 15.0, 15.0, -12.0, 9.0]]], [[[-6.0, 12.0, 6.0, 9.0, -21.0], [9.0, 18.0, -6.0, -6.0, -9.0], [-18.0, -3.0, -9.0, 18.0, -12.0], [-6.0, -15.0, -15.0, 12.0, -9.0]], [[12.0, -24.0, -12.0, -18.0, 42.0], [-18.0, -36.0, 12.0, 12.0, 18.0], [36.0, 6.0, 18.0, -36.0, 24.0], [12.0, 30.0, 30.0, -24.0, 18.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[12.0, -24.0, -12.0, -18.0, 42.0], [-18.0, -36.0, 12.0, 12.0, 18.0], [36.0, 6.0, 18.0, -36.0, 24.0], [12.0, 30.0, 30.0, -24.0, 18.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-2.0, 4.0, 2.0, 3.0, -7.0], [3.0, 6.0, -2.0, -2.0, -3.0], [-6.0, -1.0, -3.0, 6.0, -4.0], [-2.0, -5.0, -5.0, 4.0, -3.0]], [[-4.0, 8.0, 4.0, 6.0, -14.0], [6.0, 12.0, -4.0, -4.0, -6.0], [-12.0, -2.0, -6.0, 12.0, -8.0], [-4.0, -10.0, -10.0, 8.0, -6.0]], [[6.0, -12.0, -6.0, -9.0, 21.0], [-9.0, -18.0, 6.0, 6.0, 9.0], [18.0, 3.0, 9.0, -18.0, 12.0], [6.0, 15.0, 15.0, -12.0, 9.0]], [[2.0, -4.0, -2.0, -3.0, 7.0], [-3.0, -6.0, 2.0, 2.0, 3.0], [6.0, 1.0, 3.0, -6.0, 4.0], [2.0, 5.0, 5.0, -4.0, 3.0]], [[-12.0, 24.0, 12.0, 18.0, -42.0], [18.0, 36.0, -12.0, -12.0, -18.0], [-36.0, -6.0, -18.0, 36.0, -24.0], [-12.0, -30.0, -30.0, 24.0, -18.0]]], [[[-4.0, 8.0, 4.0, 6.0, -14.0], [6.0, 12.0, -4.0, -4.0, -6.0], [-12.0, -2.0, -6.0, 12.0, -8.0], [-4.0, -10.0, -10.0, 8.0, -6.0]], [[6.0, -12.0, -6.0, -9.0, 21.0], [-9.0, -18.0, 6.0, 6.0, 9.0], [18.0, 3.0, 9.0, -18.0, 12.0], [6.0, 15.0, 15.0, -12.0, 9.0]], [[-14.0, 28.0, 14.0, 21.0, -49.0], [21.0, 42.0, -14.0, -14.0, -21.0], [-42.0, -7.0, -21.0, 42.0, -28.0], [-14.0, -35.0, -35.0, 28.0, -21.0]], [[-14.0, 28.0, 14.0, 21.0, -49.0], [21.0, 42.0, -14.0, -14.0, -21.0], [-42.0, -7.0, -21.0, 42.0, -28.0], [-14.0, -35.0, -35.0, 28.0, -21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-4.0, 8.0, 4.0, 6.0, -14.0], [6.0, 12.0, -4.0, -4.0, -6.0], [-12.0, -2.0, -6.0, 12.0, -8.0], [-4.0, -10.0, -10.0, 8.0, -6.0]], [[-14.0, 28.0, 14.0, 21.0, -49.0], [21.0, 42.0, -14.0, -14.0, -21.0], [-42.0, -7.0, -21.0, 42.0, -28.0], [-14.0, -35.0, -35.0, 28.0, -21.0]], [[-14.0, 28.0, 14.0, 21.0, -49.0], [21.0, 42.0, -14.0, -14.0, -21.0], [-42.0, -7.0, -21.0, 42.0, -28.0], [-14.0, -35.0, -35.0, 28.0, -21.0]], [[-12.0, 24.0, 12.0, 18.0, -42.0], [18.0, 36.0, -12.0, -12.0, -18.0], [-36.0, -6.0, -18.0, 36.0, -24.0], [-12.0, -30.0, -30.0, 24.0, -18.0]], [[12.0, -24.0, -12.0, -18.0, 42.0], [-18.0, -36.0, 12.0, 12.0, 18.0], [36.0, 6.0, 18.0, -36.0, 24.0], [12.0, 30.0, 30.0, -24.0, 18.0]]], [[[6.0, -12.0, -6.0, -9.0, 21.0], [-9.0, -18.0, 6.0, 6.0, 9.0], [18.0, 3.0, 9.0, -18.0, 12.0], [6.0, 15.0, 15.0, -12.0, 9.0]], [[-12.0, 24.0, 12.0, 18.0, -42.0], [18.0, 36.0, -12.0, -12.0, -18.0], [-36.0, -6.0, -18.0, 36.0, -24.0], [-12.0, -30.0, -30.0, 24.0, -18.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[4.0, -8.0, -4.0, -6.0, 14.0], [-6.0, -12.0, 4.0, 4.0, 6.0], [12.0, 2.0, 6.0, -12.0, 8.0], [4.0, 10.0, 10.0, -8.0, 6.0]], [[-2.0, 4.0, 2.0, 3.0, -7.0], [3.0, 6.0, -2.0, -2.0, -3.0], [-6.0, -1.0, -3.0, 6.0, -4.0], [-2.0, -5.0, -5.0, 4.0, -3.0]]], [[[-10.0, 20.0, 10.0, 15.0, -35.0], [15.0, 30.0, -10.0, -10.0, -15.0], [-30.0, -5.0, -15.0, 30.0, -20.0], [-10.0, -25.0, -25.0, 20.0, -15.0]], [[-12.0, 24.0, 12.0, 18.0, -42.0], [18.0, 36.0, -12.0, -12.0, -18.0], [-36.0, -6.0, -18.0, 36.0, -24.0], [-12.0, -30.0, -30.0, 24.0, -18.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-8.0, 16.0, 8.0, 12.0, -28.0], [12.0, 24.0, -8.0, -8.0, -12.0], [-24.0, -4.0, -12.0, 24.0, -16.0], [-8.0, -20.0, -20.0, 16.0, -12.0]], [[12.0, -24.0, -12.0, -18.0, 42.0], [-18.0, -36.0, 12.0, 12.0, 18.0], [36.0, 6.0, 18.0, -36.0, 24.0], [12.0, 30.0, 30.0, -24.0, 18.0]]], [[[10.0, -20.0, -10.0, -15.0, 35.0], [-15.0, -30.0, 10.0, 10.0, 15.0], [30.0, 5.0, 15.0, -30.0, 20.0], [10.0, 25.0, 25.0, -20.0, 15.0]], [[-10.0, 20.0, 10.0, 15.0, -35.0], [15.0, 30.0, -10.0, -10.0, -15.0], [-30.0, -5.0, -15.0, 30.0, -20.0], [-10.0, -25.0, -25.0, 20.0, -15.0]], [[-4.0, 8.0, 4.0, 6.0, -14.0], [6.0, 12.0, -4.0, -4.0, -6.0], [-12.0, -2.0, -6.0, 12.0, -8.0], [-4.0, -10.0, -10.0, 8.0, -6.0]], [[-6.0, 12.0, 6.0, 9.0, -21.0], [9.0, 18.0, -6.0, -6.0, -9.0], [-18.0, -3.0, -9.0, 18.0, -12.0], [-6.0, -15.0, -15.0, 12.0, -9.0]], [[-4.0, 8.0, 4.0, 6.0, -14.0], [6.0, 12.0, -4.0, -4.0, -6.0], [-12.0, -2.0, -6.0, 12.0, -8.0], [-4.0, -10.0, -10.0, 8.0, -6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 0.0, 1.0, -5.0, -2.0], [-6.0, -4.0, -4.0, -1.0, 0.0], [6.0, 4.0, -5.0, 0.0, 0.0], [-6.0, -6.0, -1.0, -5.0, -3.0]], [[5.0, 4.0, 6.0, 6.0, -3.0], [7.0, -1.0, 2.0, -7.0, -6.0], [-2.0, 0.0, -2.0, 3.0, -4.0], [6.0, -1.0, -2.0, -4.0, 6.0]], [[2.0, -4.0, -5.0, 2.0, -2.0], [0.0, 2.0, -7.0, -5.0, -6.0], [-5.0, -2.0, 1.0, 7.0, 4.0], [-3.0, 4.0, -2.0, -5.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[4.0, 7.0, 1.0, 3.0, -7.0], [2.0, -7.0, -4.0, -5.0, -6.0], [-2.0, -1.0, 5.0, 5.0, -1.0], [-7.0, -6.0, -1.0, -7.0, -7.0]], [[-6.0, -3.0, 1.0, 3.0, 1.0], [6.0, -7.0, 6.0, 6.0, 0.0], [-6.0, -3.0, -7.0, -2.0, 0.0], [0.0, -6.0, 1.0, 0.0, -3.0]], [[4.0, 5.0, -2.0, -4.0, -3.0], [6.0, 1.0, 1.0, 7.0, -3.0], [-1.0, -5.0, 1.0, 4.0, -3.0], [1.0, 4.0, -7.0, 1.0, -1.0]]])
      arg1=Data(numpy.array([[[0.0, -7.0, 3.0, -5.0, 1.0], [0.0, 2.0, 0.0, 4.0, -7.0], [-4.0, 3.0, -4.0, 0.0, 4.0], [-5.0, 6.0, 3.0, 2.0, 3.0]], [[-4.0, -4.0, -5.0, -7.0, 0.0], [6.0, 1.0, 4.0, 2.0, -1.0], [-2.0, -6.0, -5.0, -2.0, -6.0], [-3.0, -6.0, 2.0, 0.0, -6.0]], [[3.0, 7.0, 2.0, 7.0, -1.0], [-7.0, -6.0, 7.0, -2.0, -7.0], [-2.0, -2.0, -4.0, -2.0, -1.0], [-2.0, -6.0, 7.0, -7.0, 4.0]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-14.0, 8.0, -27.0, -11.0, -4.0], [16.0, -11.0, 34.0, -2.0, -5.0], [-6.0, -40.0, -25.0, -14.0, -40.0], [-9.0, -54.0, 18.0, -18.0, -28.0]], [[-28.0, -44.0, -28.0, -56.0, 4.0], [52.0, 28.0, -12.0, 16.0, 24.0], [0.0, -16.0, -4.0, 0.0, -20.0], [-4.0, 0.0, -20.0, 28.0, -40.0]], [[-39.0, -66.0, -37.0, -82.0, 6.0], [71.0, 38.0, -11.0, 26.0, 22.0], [-6.0, -23.0, -14.0, -2.0, -27.0], [-13.0, 0.0, -20.0, 37.0, -53.0]], [[-18.0, 25.0, -41.0, -3.0, -7.0], [22.0, -16.0, 38.0, -12.0, 15.0], [4.0, -55.0, -18.0, -16.0, -58.0], [3.0, -78.0, 11.0, -24.0, -43.0]], [[6.0, 12.0, 5.0, 17.0, 0.0], [-4.0, 5.0, -26.0, -10.0, 31.0], [18.0, 16.0, 31.0, 10.0, 12.0], [23.0, 18.0, -26.0, 10.0, 4.0]]], [[[-28.0, 14.0, -53.0, -19.0, -6.0], [42.0, -5.0, 28.0, -10.0, 35.0], [10.0, -60.0, -11.0, -14.0, -66.0], [9.0, -78.0, -4.0, -12.0, -60.0]], [[10.0, 46.0, -3.0, 41.0, -6.0], [-20.0, -21.0, 10.0, -22.0, 15.0], [14.0, -10.0, 13.0, -2.0, -12.0], [19.0, -30.0, 0.0, -22.0, 2.0]], [[-29.0, -29.0, -36.0, -43.0, 3.0], [61.0, 36.0, -41.0, 2.0, 75.0], [26.0, -10.0, 34.0, 10.0, -21.0], [28.0, 6.0, -57.0, 41.0, -52.0]], [[13.0, 0.0, 22.0, 19.0, 4.0], [-7.0, 21.0, -63.0, -8.0, 49.0], [28.0, 49.0, 59.0, 24.0, 43.0], [36.0, 66.0, -52.0, 33.0, 19.0]], [[6.0, -18.0, 18.0, 0.0, 6.0], [6.0, 30.0, -66.0, 0.0, 48.0], [24.0, 48.0, 54.0, 24.0, 42.0], [30.0, 72.0, -54.0, 42.0, 12.0]]], [[[-7.0, -69.0, 18.0, -51.0, 11.0], [23.0, 40.0, -43.0, 30.0, -5.0], [-10.0, 40.0, 6.0, 14.0, 41.0], [-14.0, 78.0, -21.0, 47.0, 10.0]], [[-6.0, -42.0, 8.0, -34.0, 6.0], [14.0, 20.0, -14.0, 20.0, -14.0], [-12.0, 16.0, -8.0, 4.0, 18.0], [-16.0, 36.0, -2.0, 22.0, 4.0]], [[11.0, 50.0, -3.0, 46.0, -6.0], [-19.0, -18.0, -1.0, -26.0, 30.0], [22.0, -5.0, 26.0, 2.0, -9.0], [29.0, -24.0, -12.0, -17.0, 1.0]], [[9.0, 37.0, -1.0, 28.0, -7.0], [-31.0, -39.0, 61.0, -8.0, -52.0], [-20.0, -32.0, -43.0, -20.0, -25.0], [-23.0, -60.0, 55.0, -49.0, 10.0]], [[28.0, 44.0, 28.0, 56.0, -4.0], [-52.0, -28.0, 12.0, -16.0, -24.0], [0.0, 16.0, 4.0, 0.0, 20.0], [4.0, 0.0, 20.0, -28.0, 40.0]]], [[[-33.0, -3.0, -54.0, -33.0, -3.0], [57.0, 12.0, 3.0, -6.0, 57.0], [18.0, -48.0, 6.0, -6.0, -57.0], [18.0, -54.0, -27.0, 9.0, -66.0]], [[16.0, 74.0, -5.0, 65.0, -10.0], [-34.0, -37.0, 24.0, -34.0, 15.0], [18.0, -20.0, 13.0, -6.0, -22.0], [25.0, -54.0, 8.0, -40.0, 4.0]], [[2.0, 1.0, 3.0, 5.0, 1.0], [2.0, 8.0, -22.0, -4.0, 23.0], [12.0, 13.0, 22.0, 8.0, 10.0], [15.0, 18.0, -21.0, 12.0, 1.0]], [[1.0, 16.0, -5.0, 18.0, 0.0], [11.0, 16.0, -51.0, -18.0, 74.0], [38.0, 19.0, 60.0, 18.0, 9.0], [47.0, 24.0, -58.0, 25.0, -11.0]], [[-33.0, -24.0, -45.0, -48.0, 0.0], [57.0, 18.0, 3.0, 6.0, 36.0], [6.0, -39.0, -6.0, -6.0, -45.0], [3.0, -36.0, -18.0, 15.0, -57.0]]]])+(1.-msk_ref)*numpy.array([[[[36.0, 24.0, 50.0, 50.0, 0.0], [-64.0, -22.0, 4.0, -4.0, -50.0], [-12.0, 40.0, -2.0, 4.0, 48.0], [-10.0, 36.0, 28.0, -20.0, 64.0]], [[27.0, -2.0, 46.0, 21.0, 2.0], [-53.0, -19.0, 23.0, 12.0, -81.0], [-32.0, 29.0, -33.0, -4.0, 41.0], [-36.0, 30.0, 50.0, -21.0, 59.0]], [[-10.0, -25.0, -6.0, -26.0, 3.0], [20.0, 15.0, -10.0, 10.0, 6.0], [-2.0, 1.0, -1.0, 2.0, 0.0], [-4.0, 12.0, -9.0, 16.0, -11.0]], [[-24.0, -61.0, -14.0, -64.0, 7.0], [46.0, 33.0, -16.0, 26.0, 4.0], [-10.0, -1.0, -11.0, 2.0, -2.0], [-16.0, 24.0, -13.0, 34.0, -25.0]], [[-13.0, 24.0, -32.0, 7.0, -4.0], [27.0, 5.0, -17.0, -20.0, 69.0], [32.0, -21.0, 35.0, 4.0, -31.0], [38.0, -30.0, -40.0, 7.0, -39.0]]], [[[-6.0, 4.0, -12.0, -10.0, -4.0], [-6.0, -26.0, 66.0, 8.0, -62.0], [-32.0, -42.0, -62.0, -24.0, -34.0], [-40.0, -60.0, 60.0, -38.0, -6.0]], [[31.0, 84.0, 16.0, 91.0, -8.0], [-49.0, -27.0, -21.0, -44.0, 49.0], [40.0, 19.0, 59.0, 12.0, 13.0], [54.0, -6.0, -28.0, -21.0, 25.0]], [[-21.0, 11.0, -40.0, -15.0, -5.0], [29.0, -8.0, 31.0, -6.0, 15.0], [2.0, -50.0, -18.0, -14.0, -53.0], [0.0, -66.0, 7.0, -15.0, -44.0]], [[-3.0, 60.0, -31.0, 32.0, -12.0], [-13.0, -46.0, 73.0, -22.0, -20.0], [-6.0, -65.0, -38.0, -26.0, -63.0], [-7.0, -108.0, 46.0, -59.0, -23.0]], [[-9.0, 21.0, -24.0, 9.0, -3.0], [21.0, 6.0, -21.0, -18.0, 63.0], [30.0, -12.0, 36.0, 6.0, -21.0], [36.0, -18.0, -39.0, 9.0, -30.0]]], [[[21.0, 31.0, 22.0, 45.0, -1.0], [-29.0, -4.0, -31.0, -18.0, 27.0], [22.0, 32.0, 42.0, 14.0, 29.0], [30.0, 30.0, -25.0, 3.0, 26.0]], [[-3.0, -16.0, 2.0, -9.0, 4.0], [17.0, 25.0, -47.0, 0.0, 45.0], [20.0, 25.0, 39.0, 16.0, 19.0], [24.0, 42.0, -44.0, 33.0, -5.0]], [[31.0, 0.0, 52.0, 31.0, 4.0], [-49.0, -3.0, -21.0, 4.0, -35.0], [-8.0, 55.0, 11.0, 12.0, 61.0], [-6.0, 66.0, 8.0, 3.0, 61.0]], [[20.0, 1.0, 33.0, 17.0, 1.0], [-40.0, -16.0, 20.0, 8.0, -61.0], [-24.0, 19.0, -26.0, -4.0, 28.0], [-27.0, 18.0, 39.0, -18.0, 43.0]], [[-9.0, -14.0, -9.0, -16.0, 2.0], [21.0, 16.0, -21.0, 2.0, 28.0], [10.0, 3.0, 16.0, 6.0, -1.0], [11.0, 12.0, -24.0, 19.0, -15.0]]], [[[3.0, 56.0, -19.0, 42.0, -8.0], [-7.0, -20.0, 7.0, -30.0, 42.0], [26.0, -23.0, 24.0, -2.0, -29.0], [33.0, -48.0, -14.0, -21.0, -17.0]], [[36.0, 94.0, 20.0, 100.0, -10.0], [-64.0, -42.0, 4.0, -44.0, 20.0], [28.0, 10.0, 38.0, 4.0, 8.0], [40.0, -24.0, -2.0, -40.0, 34.0]], [[-25.0, -46.0, -22.0, -51.0, 6.0], [55.0, 41.0, -45.0, 12.0, 55.0], [16.0, 5.0, 27.0, 12.0, -3.0], [16.0, 30.0, -50.0, 47.0, -37.0]], [[3.0, 56.0, -19.0, 42.0, -8.0], [-7.0, -20.0, 7.0, -30.0, 42.0], [26.0, -23.0, 24.0, -2.0, -29.0], [33.0, -48.0, -14.0, -21.0, -17.0]], [[9.0, 54.0, -8.0, 49.0, -6.0], [-11.0, -11.0, -19.0, -32.0, 59.0], [36.0, -1.0, 47.0, 8.0, -9.0], [46.0, -18.0, -34.0, -7.0, -7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 0.0, -1.0, -7.0, -2.0], [5.0, 0.0, -2.0, -7.0, 0.0], [2.0, 0.0, 0.0, -5.0, -2.0], [5.0, -6.0, 1.0, -7.0, -5.0]], [[-3.0, 4.0, 0.0, 7.0, -1.0], [7.0, -2.0, 1.0, 7.0, 0.0], [-3.0, 6.0, 2.0, 1.0, -5.0], [5.0, 2.0, -4.0, -7.0, 1.0]], [[-3.0, -7.0, 0.0, -5.0, -7.0], [5.0, -2.0, -2.0, -1.0, -2.0], [1.0, -5.0, 0.0, 4.0, 7.0], [-5.0, 0.0, -6.0, 1.0, -7.0]]], [[[3.0, 7.0, -6.0, -6.0, 1.0], [1.0, 3.0, 7.0, -1.0, 7.0], [-1.0, 6.0, 7.0, 1.0, 3.0], [-1.0, -5.0, -3.0, 0.0, 2.0]], [[0.0, 0.0, 7.0, 3.0, -5.0], [-7.0, -6.0, -3.0, 3.0, 7.0], [2.0, 3.0, 1.0, 0.0, -3.0], [-5.0, 0.0, 5.0, -6.0, -5.0]], [[0.0, -3.0, -7.0, 3.0, 0.0], [5.0, 0.0, 1.0, 7.0, 1.0], [-3.0, 7.0, 2.0, 4.0, -7.0], [-5.0, 1.0, 2.0, 3.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, -1.0, 2.0, 0.0, -4.0], [-5.0, 4.0, 7.0, 5.0, 7.0], [1.0, 6.0, 2.0, -5.0, -5.0], [1.0, 3.0, 6.0, 2.0, 0.0]], [[7.0, 6.0, -7.0, -1.0, -1.0], [2.0, 1.0, 2.0, 1.0, 1.0], [-2.0, 4.0, 0.0, 2.0, -7.0], [-4.0, -7.0, -3.0, -6.0, 3.0]], [[-4.0, -1.0, -5.0, -2.0, -1.0], [-7.0, 2.0, 6.0, -7.0, -7.0], [2.0, -1.0, -6.0, 6.0, -5.0], [4.0, 0.0, -2.0, 0.0, -1.0]]], [[[1.0, 4.0, 6.0, -2.0, -6.0], [7.0, 3.0, 0.0, 0.0, 0.0], [0.0, -1.0, -1.0, -1.0, -2.0], [-7.0, 2.0, -2.0, -3.0, 5.0]], [[-2.0, 1.0, 6.0, -7.0, 1.0], [5.0, -4.0, -4.0, 0.0, 7.0], [-2.0, 0.0, -6.0, 5.0, -5.0], [5.0, 1.0, -4.0, -1.0, -3.0]], [[7.0, -5.0, 4.0, 4.0, 0.0], [-3.0, -1.0, -6.0, 5.0, -2.0], [-1.0, 0.0, 3.0, 0.0, -2.0], [2.0, 1.0, 6.0, 3.0, 2.0]]]])
      arg1=Data(numpy.array([[[[4.0, 3.0, 7.0, 7.0, 5.0], [-1.0, -7.0, -1.0, 5.0, 7.0], [-6.0, -2.0, 5.0, 5.0, -1.0], [6.0, -4.0, -6.0, -4.0, 6.0]], [[5.0, -4.0, -1.0, 4.0, -2.0], [-6.0, 7.0, 7.0, -5.0, 0.0], [5.0, 3.0, -5.0, 5.0, -5.0], [2.0, 7.0, -2.0, 2.0, 4.0]], [[4.0, 4.0, -4.0, 0.0, -7.0], [-4.0, 3.0, 4.0, 1.0, 2.0], [1.0, -5.0, -6.0, 1.0, 4.0], [4.0, -3.0, -5.0, -7.0, 1.0]]], [[[0.0, -3.0, -3.0, 0.0, 2.0], [0.0, 0.0, -4.0, 5.0, 5.0], [7.0, 2.0, 4.0, 0.0, -7.0], [-6.0, -5.0, 1.0, 6.0, -3.0]], [[7.0, -1.0, 0.0, -5.0, 6.0], [2.0, 3.0, 0.0, -3.0, 4.0], [4.0, 5.0, 1.0, 7.0, 5.0], [-4.0, 1.0, -7.0, -7.0, 0.0]], [[0.0, 0.0, -1.0, -3.0, 1.0], [-6.0, 0.0, -4.0, 1.0, -5.0], [6.0, -3.0, 2.0, -5.0, 0.0], [0.0, 0.0, 0.0, -7.0, -7.0]]]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-7.0, 6.0, 41.0, 23.0, 58.0], [25.0, -65.0, -50.0, 52.0, 44.0], [-27.0, 2.0, 70.0, 7.0, -23.0], [-6.0, -47.0, -6.0, 13.0, 6.0]], [[-8.0, -65.0, 6.0, 25.0, 52.0], [22.0, 7.0, -16.0, 5.0, 36.0], [44.0, 70.0, 44.0, 28.0, -97.0], [-62.0, 14.0, 34.0, 120.0, 9.0]], [[45.0, 8.0, 18.0, -21.0, 18.0], [57.0, 28.0, 53.0, -63.0, 26.0], [-50.0, 46.0, -36.0, 79.0, 78.0], [2.0, 41.0, -49.0, -32.0, 61.0]], [[8.0, -54.0, -21.0, -45.0, -5.0], [-27.0, 92.0, 48.0, -111.0, -92.0], [60.0, 54.0, -55.0, 1.0, 9.0], [-24.0, 125.0, 26.0, -1.0, -22.0]], [[-76.0, -28.0, 12.0, 7.0, 13.0], [26.0, -29.0, -37.0, 8.0, -43.0], [-13.0, 13.0, 36.0, -57.0, -53.0], [-28.0, 12.0, 85.0, 96.0, -26.0]]], [[[26.0, 11.0, 0.0, 83.0, -59.0], [-111.0, 8.0, 40.0, 26.0, -3.0], [19.0, -62.0, -33.0, -9.0, -62.0], [86.0, 2.0, -19.0, -21.0, 25.0]], [[-60.0, -3.0, 1.0, 22.0, -12.0], [8.0, -38.0, -34.0, 41.0, -13.0], [-15.0, -20.0, 28.0, -54.0, -49.0], [-6.0, -29.0, 59.0, 70.0, -19.0]], [[-32.0, -36.0, -29.0, 2.0, -1.0], [-8.0, 6.0, -31.0, 28.0, 0.0], [58.0, 13.0, 24.0, -33.0, -75.0], [-48.0, -17.0, 48.0, 80.0, -38.0]], [[24.0, -53.0, -56.0, -57.0, -19.0], [-67.0, 104.0, 28.0, -78.0, -79.0], [123.0, 32.0, -51.0, -15.0, -10.0], [-38.0, 88.0, 11.0, -27.0, -61.0]], [[41.0, -36.0, -14.0, -38.0, 71.0], [16.0, 15.0, -40.0, 13.0, 54.0], [81.0, 56.0, 49.0, 42.0, -22.0], [-78.0, -22.0, -32.0, 0.0, -30.0]]], [[[11.0, 23.0, 19.0, 1.0, 16.0], [34.0, -26.0, -3.0, 12.0, 34.0], [-43.0, -1.0, 11.0, 25.0, 34.0], [8.0, -25.0, -26.0, -20.0, 25.0]], [[31.0, -65.0, -11.0, -12.0, 60.0], [-52.0, 36.0, -30.0, -7.0, -3.0], [121.0, 49.0, 41.0, 11.0, -77.0], [-56.0, 30.0, -2.0, 13.0, -48.0]], [[17.0, -30.0, -25.0, -3.0, 18.0], [-22.0, 17.0, -22.0, 24.0, 29.0], [75.0, 19.0, 23.0, 7.0, -54.0], [-42.0, -20.0, -4.0, 25.0, -27.0]], [[1.0, -6.0, -59.0, -43.0, -49.0], [-41.0, 54.0, 8.0, -17.0, -42.0], [70.0, -17.0, -42.0, -36.0, 9.0], [-18.0, 10.0, 9.0, -28.0, -53.0]], [[-26.0, 36.0, -39.0, 2.0, -68.0], [40.0, -9.0, 11.0, 39.0, 38.0], [-39.0, -34.0, -32.0, -14.0, 19.0], [0.0, -66.0, 11.0, 37.0, 15.0]]], [[[-10.0, -17.0, 58.0, 95.0, 13.0], [5.0, -30.0, 34.0, 0.0, 25.0], [-67.0, 18.0, 11.0, 35.0, -68.0], [46.0, 30.0, 19.0, 89.0, 83.0]], [[-14.0, -11.0, -30.0, -37.0, -43.0], [-12.0, 56.0, 36.0, -64.0, -72.0], [17.0, 5.0, -58.0, -25.0, 31.0], [-2.0, 63.0, 27.0, -9.0, -20.0]], [[-5.0, -1.0, 42.0, -40.0, 81.0], [45.0, -38.0, -49.0, -9.0, -10.0], [-21.0, 29.0, 58.0, 4.0, 41.0], [-28.0, 6.0, -6.0, -37.0, -21.0]], [[-101.0, 17.0, -49.0, -56.0, -61.0], [15.0, -15.0, -50.0, 22.0, -86.0], [2.0, -51.0, -6.0, -126.0, 16.0], [-28.0, -30.0, 93.0, 28.0, -90.0]], [[-78.0, -48.0, -7.0, 15.0, -11.0], [59.0, 6.0, 4.0, -19.0, -24.0], [-20.0, 48.0, 1.0, -27.0, -67.0], [-48.0, 33.0, 100.0, 167.0, 10.0]]]])+(1.-msk_ref)*numpy.array([[[[33.0, -24.0, 48.0, 66.0, 46.0], [-79.0, -18.0, -6.0, 14.0, 3.0], [30.0, -2.0, 40.0, 17.0, -75.0], [42.0, 26.0, -21.0, -15.0, 14.0]], [[29.0, -44.0, -16.0, 27.0, -1.0], [1.0, 49.0, 43.0, -24.0, 40.0], [37.0, 53.0, -22.0, 56.0, -56.0], [-26.0, 30.0, -4.0, 75.0, 40.0]], [[-5.0, -10.0, 19.0, -56.0, 111.0], [48.0, -60.0, -111.0, 56.0, 38.0], [38.0, 30.0, 113.0, -8.0, 1.0], [-82.0, -66.0, -9.0, -21.0, -67.0]], [[-62.0, 9.0, 11.0, 19.0, -26.0], [-24.0, -34.0, -23.0, 18.0, -62.0], [-25.0, -44.0, 10.0, -76.0, -24.0], [30.0, 2.0, 59.0, 21.0, -28.0]], [[-18.0, 5.0, -5.0, -37.0, -17.0], [16.0, 21.0, 17.0, -49.0, -56.0], [-20.0, 3.0, -32.0, -19.0, 52.0], [2.0, 43.0, 18.0, -22.0, -11.0]]], [[[-3.0, -77.0, -27.0, -43.0, 61.0], [49.0, 43.0, -25.0, -25.0, 21.0], [84.0, 99.0, 34.0, 28.0, -57.0], [-116.0, 25.0, 33.0, 101.0, -29.0]], [[1.0, 11.0, 11.0, 55.0, -15.0], [-20.0, -27.0, 3.0, 43.0, 36.0], [-18.0, -26.0, 9.0, 4.0, -42.0], [32.0, -34.0, -5.0, 25.0, 28.0]], [[34.0, 41.0, 29.0, 95.0, -41.0], [-15.0, -29.0, 55.0, 37.0, 75.0], [-78.0, -40.0, -27.0, 53.0, -13.0], [86.0, -36.0, -48.0, 4.0, 98.0]], [[-3.0, -17.0, 57.0, 24.0, 77.0], [-13.0, -49.0, -46.0, 18.0, -4.0], [-2.0, 13.0, 72.0, -2.0, -38.0], [4.0, 8.0, 3.0, -4.0, -8.0]], [[54.0, -18.0, 78.0, 24.0, 122.0], [41.0, -42.0, -20.0, 0.0, 73.0], [-28.0, 65.0, 75.0, 92.0, -5.0], [-12.0, 7.0, -58.0, -12.0, 53.0]]], [[[-12.0, 21.0, 2.0, 12.0, -18.0], [5.0, -21.0, -3.0, 22.0, 8.0], [-28.0, -25.0, -1.0, -12.0, 7.0], [18.0, -26.0, 2.0, -1.0, 7.0]], [[40.0, 1.0, 45.0, 58.0, 27.0], [-26.0, -17.0, 22.0, 4.0, 35.0], [-24.0, 3.0, 12.0, 49.0, -23.0], [46.0, 12.0, -40.0, -15.0, 54.0]], [[-58.0, -9.0, 38.0, 35.0, 17.0], [-8.0, -50.0, -34.0, 20.0, -42.0], [-31.0, -15.0, 42.0, -53.0, -49.0], [18.0, 9.0, 59.0, 49.0, -12.0]], [[49.0, -1.0, -58.0, -52.0, -43.0], [-21.0, 82.0, 47.0, -49.0, -8.0], [59.0, 9.0, -70.0, 26.0, 51.0], [-16.0, 26.0, -40.0, -59.0, -13.0]], [[-110.0, 4.0, 0.0, -32.0, -12.0], [69.0, -44.0, -48.0, 8.0, -65.0], [-56.0, -9.0, 23.0, -90.0, 9.0], [-32.0, -9.0, 102.0, 78.0, -43.0]]], [[[35.0, 51.0, 14.0, -40.0, 3.0], [5.0, -8.0, 7.0, -19.0, -10.0], [-39.0, -29.0, -18.0, 14.0, 109.0], [36.0, -4.0, -60.0, -131.0, 1.0]], [[-16.0, 30.0, 21.0, -15.0, 40.0], [35.0, -67.0, -64.0, 58.0, 30.0], [-29.0, -21.0, 61.0, -18.0, 23.0], [-12.0, -70.0, -9.0, -28.0, -23.0]], [[-27.0, 32.0, 53.0, 32.0, 28.0], [-24.0, -81.0, -51.0, 51.0, -18.0], [-47.0, -53.0, 57.0, -45.0, -5.0], [50.0, -33.0, 6.0, -42.0, -14.0]], [[-29.0, 40.0, 26.0, -14.0, 13.0], [14.0, -59.0, -44.0, 31.0, -20.0], [-49.0, -42.0, 33.0, -42.0, 44.0], [22.0, -36.0, 4.0, -52.0, -24.0]], [[-10.0, -28.0, -16.0, 21.0, -5.0], [-32.0, 9.0, -11.0, 20.0, 1.0], [49.0, 3.0, 12.0, -17.0, -69.0], [-16.0, -4.0, 25.0, 50.0, -18.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 7.0, 0.0, 5.0, -3.0], [4.0, 6.0, 5.0, 2.0, 0.0], [-6.0, 2.0, 0.0, 4.0, 3.0], [0.0, 0.0, 5.0, 3.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-3.0, -4.0, 0.0, 0.0, -7.0], [-6.0, 2.0, 2.0, 0.0, -7.0], [-7.0, 0.0, 0.0, -7.0, 1.0], [2.0, 0.0, -5.0, -3.0, 6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(-6.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-16.0, -28.0, -0.0, -20.0, 12.0], [-16.0, -24.0, -20.0, -8.0, -0.0], [24.0, -8.0, -0.0, -16.0, -12.0], [-0.0, -0.0, -20.0, -12.0, 24.0]])+(1.-msk_ref)*numpy.array([[18.0, 24.0, -0.0, -0.0, 42.0], [36.0, -12.0, -12.0, -0.0, 42.0], [42.0, -0.0, -0.0, 42.0, -6.0], [-12.0, -0.0, 30.0, 18.0, -36.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, 6.0, 3.0, 7.0, 3.0], [-1.0, -5.0, -1.0, -7.0, 7.0], [-6.0, -6.0, 6.0, 7.0, 7.0], [-1.0, 2.0, -6.0, 7.0, 2.0]], [[5.0, 6.0, -1.0, 7.0, -6.0], [7.0, 2.0, 7.0, -7.0, 5.0], [-2.0, 0.0, -6.0, 7.0, -2.0], [-5.0, -7.0, 7.0, -3.0, 3.0]], [[-6.0, -5.0, -6.0, 3.0, -7.0], [-5.0, 0.0, -6.0, 7.0, -2.0], [-2.0, 7.0, 3.0, 1.0, 2.0], [-2.0, 0.0, 3.0, 0.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, -3.0, -1.0, -6.0, 1.0], [1.0, -7.0, -1.0, -2.0, 7.0], [-3.0, 4.0, 0.0, -4.0, -6.0], [3.0, -6.0, -1.0, -1.0, 2.0]], [[-3.0, -3.0, -4.0, -6.0, 0.0], [-5.0, -7.0, -3.0, 0.0, -5.0], [-2.0, 7.0, 5.0, 0.0, 7.0], [0.0, 0.0, -1.0, -5.0, -7.0]], [[6.0, 2.0, 0.0, 2.0, -2.0], [6.0, 0.0, 2.0, -6.0, 4.0], [0.0, -1.0, 3.0, 4.0, -3.0], [-1.0, 3.0, 2.0, -4.0, -7.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -6.0, 1.0])+(1.-msk_arg1)*numpy.array([3.0, -7.0, -5.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-36.0, -41.0, 0.0, -39.0, 29.0], [-47.0, -12.0, -48.0, 49.0, -32.0], [10.0, 7.0, 39.0, -41.0, 14.0], [28.0, 42.0, -39.0, 18.0, -17.0]])+(1.-msk_ref)*numpy.array([[0.0, 2.0, 25.0, 14.0, 13.0], [8.0, 28.0, 8.0, 24.0, 36.0], [5.0, -32.0, -50.0, -32.0, -52.0], [14.0, -33.0, -6.0, 52.0, 90.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 0.0, -3.0, 5.0, 1.0], [-1.0, 5.0, -3.0, 3.0, 0.0], [4.0, -1.0, -3.0, 0.0, -2.0], [-7.0, -7.0, 6.0, 3.0, 0.0]], [[5.0, 0.0, -1.0, 7.0, 4.0], [5.0, -4.0, 7.0, 1.0, -5.0], [0.0, 4.0, 0.0, 3.0, -2.0], [3.0, 3.0, -2.0, 1.0, 7.0]], [[3.0, 6.0, 5.0, -1.0, 6.0], [-5.0, 0.0, 2.0, -1.0, 0.0], [-7.0, 1.0, -3.0, -2.0, -5.0], [0.0, 0.0, 0.0, 0.0, -1.0]]], [[[-3.0, -2.0, 0.0, 6.0, -3.0], [4.0, 2.0, 3.0, -1.0, -1.0], [1.0, 3.0, -4.0, 6.0, 0.0], [-5.0, 5.0, -1.0, -6.0, -2.0]], [[-1.0, -7.0, -1.0, 1.0, 0.0], [-4.0, 5.0, 0.0, 5.0, -4.0], [-1.0, -4.0, -5.0, -7.0, -5.0], [3.0, 6.0, 3.0, 0.0, 6.0]], [[1.0, 0.0, 3.0, 4.0, -1.0], [0.0, 0.0, -1.0, -3.0, -5.0], [2.0, 1.0, -7.0, 5.0, -7.0], [-4.0, 0.0, -5.0, -4.0, 4.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 2.0, 6.0, -1.0, 0.0], [6.0, -1.0, 1.0, 6.0, -3.0], [-3.0, -1.0, -4.0, 5.0, 2.0], [-2.0, 1.0, -6.0, 0.0, 5.0]], [[1.0, -6.0, -4.0, 6.0, 5.0], [-5.0, 1.0, 4.0, -6.0, 5.0], [6.0, -6.0, -4.0, 2.0, -5.0], [-4.0, 0.0, -4.0, -5.0, -1.0]], [[-7.0, -7.0, -2.0, -3.0, -1.0], [-4.0, -3.0, 0.0, -3.0, -5.0], [2.0, -2.0, 1.0, -5.0, 7.0], [-5.0, 3.0, 0.0, 0.0, -7.0]]], [[[-6.0, -3.0, -3.0, -3.0, -1.0], [-6.0, -2.0, 3.0, -1.0, -1.0], [5.0, -3.0, 1.0, 2.0, 1.0], [7.0, 1.0, 3.0, 5.0, 3.0]], [[0.0, -1.0, -3.0, 2.0, 4.0], [-3.0, -6.0, -7.0, -1.0, 7.0], [0.0, 3.0, -5.0, -2.0, 6.0], [2.0, 2.0, -7.0, 1.0, -4.0]], [[-2.0, 0.0, 2.0, -5.0, -1.0], [2.0, -6.0, -4.0, 1.0, 7.0], [-7.0, 0.0, 0.0, -7.0, 7.0], [-1.0, -3.0, 4.0, -7.0, -6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 5.0, 3.0], [3.0, 7.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 2.0, 3.0], [5.0, 1.0, 6.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[22.0, -37.0, 24.0, 58.0, 22.0], [-3.0, 6.0, 55.0, 13.0, -76.0], [-29.0, 11.0, -75.0, -2.0, -82.0], [26.0, 93.0, -30.0, -38.0, 84.0]])+(1.-msk_ref)*numpy.array([[-61.0, -57.0, -44.0, -36.0, 0.0], [-67.0, -55.0, -12.0, -45.0, 51.0], [13.0, -26.0, 11.0, -65.0, 56.0], [16.0, -6.0, 48.0, -26.0, -68.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, -2.0, 3.0, 5.0, -5.0], [-7.0, 4.0, -6.0, 2.0, 3.0], [-6.0, -6.0, -1.0, -3.0, 2.0], [-3.0, -5.0, 4.0, -2.0, 2.0]])+(1.-msk_arg0)*numpy.array([[7.0, -6.0, -5.0, -2.0, 0.0], [-6.0, -5.0, 0.0, -7.0, 0.0], [-4.0, -5.0, -2.0, 6.0, 0.0], [3.0, 7.0, 7.0, 3.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, -3.0])+(1.-msk_arg1)*numpy.array([0.0, 1.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[35.0, 21.0], [10.0, 6.0], [-15.0, -9.0], [-25.0, -15.0], [25.0, 15.0]], [[35.0, 21.0], [-20.0, -12.0], [30.0, 18.0], [-10.0, -6.0], [-15.0, -9.0]], [[30.0, 18.0], [30.0, 18.0], [5.0, 3.0], [15.0, 9.0], [-10.0, -6.0]], [[15.0, 9.0], [25.0, 15.0], [-20.0, -12.0], [10.0, 6.0], [-10.0, -6.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 7.0], [0.0, -6.0], [0.0, -5.0], [0.0, -2.0], [0.0, 0.0]], [[0.0, -6.0], [0.0, -5.0], [0.0, 0.0], [0.0, -7.0], [0.0, 0.0]], [[0.0, -4.0], [0.0, -5.0], [0.0, -2.0], [0.0, 6.0], [0.0, 0.0]], [[0.0, 3.0], [0.0, 7.0], [0.0, 7.0], [0.0, 3.0], [0.0, -7.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, -6.0, 0.0, 3.0, 6.0], [4.0, 0.0, 0.0, -5.0, 1.0], [2.0, 7.0, 3.0, 4.0, 0.0], [-4.0, -5.0, -6.0, 4.0, -4.0]], [[-1.0, -5.0, 4.0, 7.0, -5.0], [-2.0, 0.0, 4.0, 1.0, 7.0], [3.0, 7.0, 4.0, 7.0, -7.0], [0.0, 7.0, -6.0, 3.0, 6.0]], [[4.0, 7.0, -4.0, -3.0, -6.0], [-4.0, -2.0, -3.0, -5.0, -3.0], [1.0, 5.0, 1.0, 4.0, 3.0], [-7.0, 7.0, 6.0, -5.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 0.0, -7.0, -3.0, -2.0], [-1.0, 0.0, -6.0, -4.0, -1.0], [4.0, 2.0, -5.0, 0.0, -4.0], [-4.0, 2.0, 1.0, 7.0, 0.0]], [[-5.0, 3.0, 4.0, -5.0, 3.0], [3.0, -3.0, -1.0, -6.0, 1.0], [-6.0, -5.0, 6.0, 2.0, 1.0], [-4.0, 5.0, 6.0, 6.0, -4.0]], [[-1.0, -4.0, -1.0, 0.0, -1.0], [-7.0, 2.0, -1.0, -6.0, 0.0], [1.0, 3.0, -1.0, -1.0, -3.0], [-6.0, 5.0, 4.0, -5.0, -7.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-1.0, -2.0], [3.0, 5.0], [5.0, 2.0]])+(1.-msk_arg1)*numpy.array([[7.0, 1.0], [3.0, 5.0], [-5.0, -4.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[15.0, -1.0], [26.0, 1.0], [-8.0, 12.0], [3.0, 23.0], [-51.0, -49.0]], [[-30.0, -26.0], [-10.0, -4.0], [-3.0, 14.0], [-17.0, 5.0], [5.0, 27.0]], [[12.0, 13.0], [39.0, 31.0], [14.0, 16.0], [37.0, 35.0], [-6.0, -29.0]], [[-31.0, -6.0], [61.0, 59.0], [18.0, -6.0], [-20.0, -3.0], [2.0, 30.0]]])+(1.-msk_ref)*numpy.array([[[-17.0, -22.0], [29.0, 31.0], [-32.0, 17.0], [-36.0, -28.0], [0.0, 17.0]], [[37.0, 42.0], [-19.0, -23.0], [-40.0, -7.0], [-16.0, -10.0], [-4.0, 4.0]], [[5.0, -30.0], [-16.0, -35.0], [-12.0, 29.0], [11.0, 14.0], [-10.0, 13.0]], [[-10.0, 0.0], [4.0, 7.0], [5.0, 15.0], [92.0, 57.0], [23.0, 8.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, 0.0, -5.0, 1.0, 1.0], [7.0, 0.0, -5.0, 1.0, 3.0], [0.0, -5.0, 4.0, 0.0, 4.0], [2.0, 1.0, 2.0, -4.0, 5.0]], [[-3.0, -7.0, 0.0, 2.0, 0.0], [-7.0, 0.0, -1.0, -2.0, -7.0], [6.0, -3.0, -6.0, 0.0, 1.0], [5.0, 0.0, 1.0, 2.0, -2.0]], [[5.0, 2.0, -1.0, 1.0, 4.0], [5.0, 3.0, 2.0, -5.0, 3.0], [1.0, 7.0, 4.0, 4.0, -5.0], [-6.0, -5.0, 0.0, -3.0, -7.0]]], [[[1.0, -4.0, 1.0, -3.0, 0.0], [-2.0, 2.0, 1.0, -6.0, 4.0], [3.0, 6.0, 4.0, -4.0, 7.0], [3.0, -3.0, 3.0, 5.0, 4.0]], [[7.0, -7.0, -7.0, 5.0, -2.0], [-5.0, 2.0, -5.0, 5.0, 0.0], [1.0, -5.0, -6.0, 7.0, -4.0], [-7.0, 6.0, -2.0, -7.0, 7.0]], [[2.0, 6.0, 7.0, -1.0, 7.0], [-2.0, 1.0, -1.0, 5.0, 6.0], [0.0, 6.0, -7.0, -6.0, 2.0], [3.0, 3.0, 4.0, 3.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, -4.0, 4.0, -3.0, 4.0], [1.0, -1.0, -2.0, -3.0, 0.0], [-7.0, 1.0, 2.0, -5.0, 1.0], [7.0, 6.0, 3.0, 1.0, 7.0]], [[-1.0, -2.0, -4.0, 2.0, -3.0], [-5.0, -5.0, -6.0, 4.0, 0.0], [0.0, 6.0, 7.0, -3.0, -3.0], [1.0, 7.0, 5.0, 7.0, -5.0]], [[-3.0, -5.0, 7.0, -7.0, -4.0], [-3.0, -6.0, 0.0, 6.0, 0.0], [1.0, 0.0, 1.0, -7.0, 4.0], [-1.0, 2.0, -2.0, 4.0, -6.0]]], [[[7.0, -5.0, -1.0, 0.0, 3.0], [-5.0, -2.0, 3.0, 0.0, -3.0], [-1.0, 0.0, 3.0, 0.0, 2.0], [5.0, 7.0, 4.0, -3.0, 4.0]], [[-7.0, 5.0, 0.0, 0.0, 7.0], [-2.0, -3.0, 1.0, -2.0, -6.0], [-4.0, -4.0, -3.0, -7.0, 0.0], [3.0, 5.0, 6.0, 0.0, -7.0]], [[-2.0, 4.0, 0.0, -4.0, 0.0], [5.0, 1.0, 4.0, 1.0, 4.0], [-7.0, -1.0, -6.0, -5.0, -6.0], [3.0, 7.0, 6.0, 3.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 0.0], [-7.0, -1.0], [-5.0, 2.0]], [[3.0, -3.0], [4.0, -2.0], [-3.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, -7.0], [-2.0, -2.0], [-1.0, -2.0]], [[3.0, 4.0], [4.0, -5.0], [7.0, 0.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, -4.0], [-19.0, 37.0], [-56.0, 9.0], [-2.0, -1.0], [-46.0, 12.0]], [[25.0, 33.0], [-4.0, -4.0], [-32.0, 12.0], [29.0, 0.0], [37.0, 1.0]], [[-34.0, -15.0], [-49.0, 9.0], [43.0, 14.0], [14.0, 6.0], [29.0, -24.0]], [[-27.0, -12.0], [34.0, -13.0], [-12.0, -6.0], [-33.0, -9.0], [107.0, -38.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, 36.0], [46.0, -3.0], [-6.0, -38.0], [-22.0, 31.0], [43.0, -37.0]], [[24.0, -1.0], [6.0, 36.0], [55.0, 33.0], [-12.0, 11.0], [-5.0, 18.0]], [[-62.0, 63.0], [-36.0, 1.0], [-62.0, -3.0], [-45.0, 90.0], [-35.0, -1.0]], [[40.0, -44.0], [68.0, -57.0], [67.0, -41.0], [-7.0, -41.0], [-7.0, 24.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank2_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 4.0, -6.0, -5.0, 0.0], [0.0, -2.0, 0.0, 0.0, -7.0], [0.0, -2.0, -5.0, 0.0, -4.0], [0.0, -4.0, 5.0, 1.0, -5.0]])+(1.-msk_arg0)*numpy.array([[5.0, 5.0, 6.0, 4.0, 2.0], [7.0, 3.0, -3.0, 7.0, 1.0], [4.0, 1.0, 2.0, 1.0, 4.0], [-6.0, -1.0, 4.0, -5.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, -1.0, 4.0, 5.0, 5.0], [4.0, -4.0, -2.0, 1.0, -3.0], [-3.0, 0.0, 4.0, -4.0, 0.0], [2.0, -3.0, 0.0, 4.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -5.0, 0.0, 0.0, -3.0], [0.0, 2.0, -4.0, -1.0, -5.0], [-1.0, -1.0, -3.0, -4.0, -1.0], [-4.0, 5.0, 1.0, 2.0, 1.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-18.0, 6.0, -24.0, -30.0, -30.0], [-24.0, 24.0, 12.0, -6.0, 18.0], [18.0, 0.0, -24.0, 24.0, 0.0], [-12.0, 18.0, 0.0, -24.0, 36.0]], [[12.0, -4.0, 16.0, 20.0, 20.0], [16.0, -16.0, -8.0, 4.0, -12.0], [-12.0, 0.0, 16.0, -16.0, 0.0], [8.0, -12.0, 0.0, 16.0, -24.0]], [[-18.0, 6.0, -24.0, -30.0, -30.0], [-24.0, 24.0, 12.0, -6.0, 18.0], [18.0, 0.0, -24.0, 24.0, 0.0], [-12.0, 18.0, 0.0, -24.0, 36.0]], [[-15.0, 5.0, -20.0, -25.0, -25.0], [-20.0, 20.0, 10.0, -5.0, 15.0], [15.0, 0.0, -20.0, 20.0, 0.0], [-10.0, 15.0, 0.0, -20.0, 30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 2.0, -8.0, -10.0, -10.0], [-8.0, 8.0, 4.0, -2.0, 6.0], [6.0, 0.0, -8.0, 8.0, 0.0], [-4.0, 6.0, 0.0, -8.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-21.0, 7.0, -28.0, -35.0, -35.0], [-28.0, 28.0, 14.0, -7.0, 21.0], [21.0, 0.0, -28.0, 28.0, 0.0], [-14.0, 21.0, 0.0, -28.0, 42.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 2.0, -8.0, -10.0, -10.0], [-8.0, 8.0, 4.0, -2.0, 6.0], [6.0, 0.0, -8.0, 8.0, 0.0], [-4.0, 6.0, 0.0, -8.0, 12.0]], [[-15.0, 5.0, -20.0, -25.0, -25.0], [-20.0, 20.0, 10.0, -5.0, 15.0], [15.0, 0.0, -20.0, 20.0, 0.0], [-10.0, 15.0, 0.0, -20.0, 30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, 4.0, -16.0, -20.0, -20.0], [-16.0, 16.0, 8.0, -4.0, 12.0], [12.0, 0.0, -16.0, 16.0, 0.0], [-8.0, 12.0, 0.0, -16.0, 24.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, 4.0, -16.0, -20.0, -20.0], [-16.0, 16.0, 8.0, -4.0, 12.0], [12.0, 0.0, -16.0, 16.0, 0.0], [-8.0, 12.0, 0.0, -16.0, 24.0]], [[15.0, -5.0, 20.0, 25.0, 25.0], [20.0, -20.0, -10.0, 5.0, -15.0], [-15.0, 0.0, 20.0, -20.0, 0.0], [10.0, -15.0, 0.0, 20.0, -30.0]], [[3.0, -1.0, 4.0, 5.0, 5.0], [4.0, -4.0, -2.0, 1.0, -3.0], [-3.0, 0.0, 4.0, -4.0, 0.0], [2.0, -3.0, 0.0, 4.0, -6.0]], [[-15.0, 5.0, -20.0, -25.0, -25.0], [-20.0, 20.0, 10.0, -5.0, 15.0], [15.0, 0.0, -20.0, 20.0, 0.0], [-10.0, 15.0, 0.0, -20.0, 30.0]]]])+(1.-msk_ref)*numpy.array([[[[-35.0, -25.0, 0.0, 0.0, -15.0], [0.0, 10.0, -20.0, -5.0, -25.0], [-5.0, -5.0, -15.0, -20.0, -5.0], [-20.0, 25.0, 5.0, 10.0, 5.0]], [[-35.0, -25.0, 0.0, 0.0, -15.0], [0.0, 10.0, -20.0, -5.0, -25.0], [-5.0, -5.0, -15.0, -20.0, -5.0], [-20.0, 25.0, 5.0, 10.0, 5.0]], [[-42.0, -30.0, 0.0, 0.0, -18.0], [0.0, 12.0, -24.0, -6.0, -30.0], [-6.0, -6.0, -18.0, -24.0, -6.0], [-24.0, 30.0, 6.0, 12.0, 6.0]], [[-28.0, -20.0, 0.0, 0.0, -12.0], [0.0, 8.0, -16.0, -4.0, -20.0], [-4.0, -4.0, -12.0, -16.0, -4.0], [-16.0, 20.0, 4.0, 8.0, 4.0]], [[-14.0, -10.0, 0.0, 0.0, -6.0], [0.0, 4.0, -8.0, -2.0, -10.0], [-2.0, -2.0, -6.0, -8.0, -2.0], [-8.0, 10.0, 2.0, 4.0, 2.0]]], [[[-49.0, -35.0, 0.0, 0.0, -21.0], [0.0, 14.0, -28.0, -7.0, -35.0], [-7.0, -7.0, -21.0, -28.0, -7.0], [-28.0, 35.0, 7.0, 14.0, 7.0]], [[-21.0, -15.0, 0.0, 0.0, -9.0], [0.0, 6.0, -12.0, -3.0, -15.0], [-3.0, -3.0, -9.0, -12.0, -3.0], [-12.0, 15.0, 3.0, 6.0, 3.0]], [[21.0, 15.0, 0.0, 0.0, 9.0], [0.0, -6.0, 12.0, 3.0, 15.0], [3.0, 3.0, 9.0, 12.0, 3.0], [12.0, -15.0, -3.0, -6.0, -3.0]], [[-49.0, -35.0, 0.0, 0.0, -21.0], [0.0, 14.0, -28.0, -7.0, -35.0], [-7.0, -7.0, -21.0, -28.0, -7.0], [-28.0, 35.0, 7.0, 14.0, 7.0]], [[-7.0, -5.0, 0.0, 0.0, -3.0], [0.0, 2.0, -4.0, -1.0, -5.0], [-1.0, -1.0, -3.0, -4.0, -1.0], [-4.0, 5.0, 1.0, 2.0, 1.0]]], [[[-28.0, -20.0, 0.0, 0.0, -12.0], [0.0, 8.0, -16.0, -4.0, -20.0], [-4.0, -4.0, -12.0, -16.0, -4.0], [-16.0, 20.0, 4.0, 8.0, 4.0]], [[-7.0, -5.0, 0.0, 0.0, -3.0], [0.0, 2.0, -4.0, -1.0, -5.0], [-1.0, -1.0, -3.0, -4.0, -1.0], [-4.0, 5.0, 1.0, 2.0, 1.0]], [[-14.0, -10.0, 0.0, 0.0, -6.0], [0.0, 4.0, -8.0, -2.0, -10.0], [-2.0, -2.0, -6.0, -8.0, -2.0], [-8.0, 10.0, 2.0, 4.0, 2.0]], [[-7.0, -5.0, 0.0, 0.0, -3.0], [0.0, 2.0, -4.0, -1.0, -5.0], [-1.0, -1.0, -3.0, -4.0, -1.0], [-4.0, 5.0, 1.0, 2.0, 1.0]], [[-28.0, -20.0, 0.0, 0.0, -12.0], [0.0, 8.0, -16.0, -4.0, -20.0], [-4.0, -4.0, -12.0, -16.0, -4.0], [-16.0, 20.0, 4.0, 8.0, 4.0]]], [[[42.0, 30.0, 0.0, 0.0, 18.0], [0.0, -12.0, 24.0, 6.0, 30.0], [6.0, 6.0, 18.0, 24.0, 6.0], [24.0, -30.0, -6.0, -12.0, -6.0]], [[7.0, 5.0, 0.0, 0.0, 3.0], [0.0, -2.0, 4.0, 1.0, 5.0], [1.0, 1.0, 3.0, 4.0, 1.0], [4.0, -5.0, -1.0, -2.0, -1.0]], [[-28.0, -20.0, 0.0, 0.0, -12.0], [0.0, 8.0, -16.0, -4.0, -20.0], [-4.0, -4.0, -12.0, -16.0, -4.0], [-16.0, 20.0, 4.0, 8.0, 4.0]], [[35.0, 25.0, 0.0, 0.0, 15.0], [0.0, -10.0, 20.0, 5.0, 25.0], [5.0, 5.0, 15.0, 20.0, 5.0], [20.0, -25.0, -5.0, -10.0, -5.0]], [[-49.0, -35.0, 0.0, 0.0, -21.0], [0.0, 14.0, -28.0, -7.0, -35.0], [-7.0, -7.0, -21.0, -28.0, -7.0], [-28.0, 35.0, 7.0, 14.0, 7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, 7.0, 2.0, 7.0, 1.0], [-3.0, 2.0, -4.0, -5.0, -5.0], [5.0, 7.0, -5.0, -5.0, 0.0], [3.0, 6.0, 0.0, 7.0, -6.0]], [[2.0, -5.0, 0.0, -5.0, 6.0], [6.0, 0.0, -3.0, -4.0, 0.0], [-7.0, 0.0, 5.0, -2.0, 7.0], [-2.0, 7.0, 3.0, 3.0, 4.0]], [[-2.0, -5.0, 6.0, 7.0, 4.0], [-4.0, 5.0, 4.0, -1.0, 7.0], [2.0, 4.0, 1.0, 2.0, -5.0], [3.0, 2.0, -2.0, 4.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 1.0, -6.0, 0.0, 4.0], [0.0, 6.0, -4.0, -1.0, 1.0], [3.0, 2.0, -7.0, 3.0, 0.0], [6.0, -4.0, 6.0, -2.0, 4.0]], [[0.0, 6.0, 5.0, 0.0, -1.0], [5.0, 6.0, 6.0, 2.0, 2.0], [-4.0, -4.0, -2.0, 3.0, 7.0], [6.0, -5.0, 0.0, -7.0, -4.0]], [[-6.0, -4.0, -2.0, -1.0, -1.0], [7.0, 0.0, -5.0, 1.0, -3.0], [2.0, 0.0, -5.0, -3.0, 0.0], [6.0, 1.0, 4.0, -7.0, -4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, -1.0, 1.0, -1.0, -3.0], [-1.0, -7.0, 5.0, -6.0, 1.0], [3.0, 2.0, 4.0, 0.0, -4.0], [4.0, -1.0, 0.0, 7.0, 3.0]], [[2.0, 5.0, 1.0, -3.0, -7.0], [-4.0, -5.0, 3.0, -2.0, 7.0], [-2.0, 0.0, 2.0, 1.0, -1.0], [-6.0, 1.0, -7.0, -7.0, -4.0]], [[1.0, 3.0, 5.0, 6.0, -7.0], [-7.0, -6.0, 4.0, 0.0, -5.0], [0.0, -3.0, 6.0, -5.0, -5.0], [-7.0, -6.0, -5.0, -5.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, -7.0, -2.0, 0.0, -5.0], [-4.0, -2.0, 6.0, 0.0, 1.0], [1.0, -4.0, 7.0, 5.0, 4.0], [-1.0, 2.0, 3.0, 2.0, 4.0]], [[6.0, 0.0, -1.0, -2.0, 1.0], [0.0, 7.0, 4.0, -4.0, 1.0], [5.0, 6.0, 2.0, -6.0, 6.0], [-6.0, 1.0, 1.0, -3.0, -3.0]], [[3.0, 7.0, 5.0, 3.0, 3.0], [7.0, 7.0, -6.0, 6.0, -6.0], [-2.0, 0.0, -5.0, 6.0, 4.0], [-1.0, 0.0, 7.0, -4.0, -2.0]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-14.0, 0.0, -4.0, -22.0, -12.0], [2.0, -26.0, 18.0, -28.0, 28.0], [8.0, 14.0, 8.0, 12.0, -8.0], [18.0, 10.0, -4.0, 24.0, -8.0]], [[-43.0, -47.0, -23.0, -22.0, 49.0], [48.0, 6.0, 0.0, -32.0, -3.0], [31.0, 29.0, -12.0, 20.0, 2.0], [93.0, 18.0, 60.0, 109.0, 11.0]], [[-2.0, 16.0, 32.0, 34.0, -48.0], [-44.0, -50.0, 34.0, -12.0, -28.0], [6.0, -14.0, 44.0, -30.0, -38.0], [-34.0, -38.0, -30.0, -16.0, 42.0]], [[-31.0, -11.0, 37.0, 50.0, -35.0], [-36.0, -66.0, 48.0, -32.0, -63.0], [31.0, -7.0, 60.0, -40.0, -58.0], [9.0, -54.0, 0.0, 49.0, 83.0]], [[12.0, 41.0, 27.0, 5.0, -73.0], [-53.0, -61.0, 39.0, -18.0, 23.0], [-9.0, -10.0, 40.0, -14.0, -30.0], [-60.0, -19.0, -62.0, -55.0, 3.0]]], [[[20.0, 21.0, -17.0, -39.0, -5.0], [7.0, 15.0, -13.0, 6.0, 59.0], [-21.0, 6.0, -24.0, 26.0, 26.0], [-20.0, 33.0, -22.0, -43.0, -57.0]], [[-3.0, 13.0, 27.0, 28.0, -41.0], [-37.0, -44.0, 30.0, -12.0, -23.0], [6.0, -11.0, 38.0, -25.0, -33.0], [-27.0, -32.0, -25.0, -11.0, 36.0]], [[14.0, 1.0, 13.0, 37.0, 5.0], [-12.0, 19.0, -13.0, 30.0, -45.0], [-6.0, -20.0, 2.0, -23.0, -1.0], [-26.0, -23.0, 1.0, -27.0, 24.0]], [[11.0, -18.0, -14.0, 11.0, 50.0], [28.0, 61.0, -41.0, 38.0, -28.0], [-7.0, -7.0, -34.0, 1.0, 29.0], [11.0, 7.0, 33.0, -2.0, -5.0]], [[27.0, 26.0, 30.0, 47.0, -34.0], [-44.0, -7.0, 3.0, 30.0, -40.0], [-15.0, -31.0, 22.0, -35.0, -15.0], [-69.0, -37.0, -35.0, -70.0, 27.0]]], [[[-32.0, -34.0, 8.0, 28.0, 20.0], [9.0, -12.0, 12.0, -16.0, -54.0], [29.0, 4.0, 18.0, -17.0, -23.0], [48.0, -24.0, 39.0, 74.0, 55.0]], [[-24.0, 5.0, 27.0, 17.0, -49.0], [-35.0, -73.0, 51.0, -42.0, -13.0], [21.0, 2.0, 52.0, -20.0, -48.0], [0.0, -31.0, -20.0, 29.0, 45.0]], [[31.0, 33.0, 5.0, -4.0, -27.0], [-22.0, 4.0, -6.0, 20.0, 25.0], [-25.0, -13.0, -4.0, 0.0, 10.0], [-57.0, 4.0, -40.0, -75.0, -29.0]], [[18.0, 1.0, 3.0, 23.0, 15.0], [-1.0, 33.0, -23.0, 34.0, -29.0], [-11.0, -16.0, -12.0, -12.0, 12.0], [-22.0, -9.0, 4.0, -31.0, 5.0]], [[9.0, 20.0, -18.0, -51.0, -14.0], [7.0, -5.0, 1.0, -14.0, 74.0], [-14.0, 15.0, -16.0, 32.0, 18.0], [-7.0, 37.0, -24.0, -24.0, -58.0]]], [[[-13.0, -4.0, 16.0, 21.0, -16.0], [-16.0, -29.0, 21.0, -14.0, -26.0], [13.0, -3.0, 26.0, -17.0, -25.0], [3.0, -23.0, -1.0, 20.0, 35.0]], [[-8.0, 35.0, 23.0, -15.0, -81.0], [-48.0, -89.0, 59.0, -50.0, 45.0], [4.0, 6.0, 50.0, -3.0, -41.0], [-32.0, -11.0, -59.0, -17.0, 2.0]], [[4.0, 9.0, -7.0, -21.0, -7.0], [2.0, -3.0, 1.0, -6.0, 31.0], [-6.0, 6.0, -6.0, 13.0, 7.0], [-4.0, 15.0, -11.0, -11.0, -24.0]], [[-18.0, 20.0, 30.0, 8.0, -70.0], [-47.0, -88.0, 60.0, -48.0, 8.0], [15.0, 2.0, 58.0, -17.0, -51.0], [-18.0, -28.0, -41.0, 8.0, 33.0]], [[32.0, 26.0, -2.0, -6.0, -10.0], [-10.0, 22.0, -18.0, 28.0, 22.0], [-26.0, -12.0, -16.0, 4.0, 20.0], [-48.0, 10.0, -28.0, -70.0, -34.0]]]])+(1.-msk_ref)*numpy.array([[[[-39.0, -91.0, -44.0, -18.0, -53.0], [-70.0, -56.0, 78.0, -36.0, 43.0], [19.0, -28.0, 79.0, -1.0, 4.0], [-1.0, 14.0, -21.0, 38.0, 40.0]], [[21.0, -35.0, -28.0, -24.0, -11.0], [-32.0, 12.0, 54.0, -48.0, 31.0], [39.0, 32.0, 39.0, -55.0, 24.0], [-33.0, 8.0, -19.0, 0.0, -6.0]], [[42.0, 28.0, -3.0, -16.0, 29.0], [10.0, 33.0, -4.0, -32.0, 11.0], [23.0, 54.0, -22.0, -72.0, -2.0], [-22.0, -7.0, -27.0, -19.0, -35.0]], [[-3.0, -7.0, -5.0, -3.0, -3.0], [-7.0, -7.0, 6.0, -6.0, 6.0], [2.0, 0.0, 5.0, -6.0, -4.0], [1.0, 0.0, -7.0, 4.0, 2.0]], [[-21.0, -35.0, -12.0, -1.0, -24.0], [-23.0, -22.0, 26.0, -2.0, 9.0], [1.0, -22.0, 31.0, 20.0, 6.0], [3.0, 7.0, 4.0, 15.0, 21.0]]], [[[51.0, 49.0, 30.0, 11.0, 26.0], [49.0, 84.0, -22.0, 22.0, -37.0], [11.0, 30.0, -25.0, 12.0, 58.0], [-37.0, 5.0, 54.0, -43.0, -29.0]], [[18.0, -42.0, -18.0, -12.0, -24.0], [-24.0, 30.0, 60.0, -24.0, 12.0], [36.0, 12.0, 54.0, -6.0, 60.0], [-42.0, 18.0, 24.0, -6.0, 6.0]], [[33.0, -7.0, -23.0, -27.0, 11.0], [-19.0, 15.0, 30.0, -54.0, 32.0], [36.0, 52.0, 9.0, -86.0, 0.0], [-27.0, -2.0, -41.0, -6.0, -24.0]], [[18.0, 14.0, 5.0, -1.0, 10.0], [11.0, 23.0, -4.0, -2.0, -5.0], [7.0, 16.0, -8.0, -11.0, 12.0], [-12.0, 0.0, 6.0, -12.0, -12.0]], [[0.0, -28.0, -19.0, -13.0, -12.0], [-25.0, -9.0, 32.0, -26.0, 21.0], [17.0, 8.0, 26.0, -25.0, 4.0], [-10.0, 4.0, -16.0, 8.0, 4.0]]], [[[-27.0, -7.0, 8.0, 14.0, -13.0], [2.0, -20.0, -10.0, 28.0, -13.0], [-21.0, -36.0, 3.0, 51.0, -4.0], [19.0, 2.0, 19.0, 10.0, 20.0]], [[-30.0, -14.0, 0.0, 8.0, -14.0], [-8.0, -32.0, -4.0, 16.0, -2.0], [-18.0, -32.0, 6.0, 34.0, -16.0], [22.0, 0.0, 2.0, 16.0, 20.0]], [[-6.0, 14.0, -9.0, -11.0, 18.0], [-7.0, -35.0, -20.0, -22.0, 21.0], [-7.0, 16.0, -28.0, -53.0, -60.0], [24.0, -16.0, -58.0, 12.0, -12.0]], [[0.0, -42.0, -24.0, -15.0, -21.0], [-33.0, -6.0, 48.0, -30.0, 24.0], [24.0, 6.0, 42.0, -21.0, 18.0], [-18.0, 9.0, -9.0, 9.0, 9.0]], [[42.0, 0.0, -7.0, -14.0, 7.0], [0.0, 49.0, 28.0, -28.0, 7.0], [35.0, 42.0, 14.0, -42.0, 42.0], [-42.0, 7.0, 7.0, -21.0, -21.0]]], [[[36.0, 0.0, 12.0, 6.0, -6.0], [18.0, 72.0, 24.0, 12.0, -24.0], [24.0, 12.0, 24.0, 30.0, 84.0], [-48.0, 18.0, 66.0, -30.0, -6.0]], [[-15.0, 35.0, 18.0, 13.0, 18.0], [23.0, -20.0, -50.0, 26.0, -15.0], [-31.0, -14.0, -43.0, 16.0, -42.0], [33.0, -13.0, -10.0, 3.0, -3.0]], [[-6.0, -14.0, 8.0, 12.0, -18.0], [4.0, 16.0, 12.0, 24.0, -18.0], [-2.0, -24.0, 22.0, 54.0, 40.0], [-10.0, 12.0, 46.0, -4.0, 16.0]], [[-57.0, -35.0, -24.0, -7.0, -18.0], [-41.0, -94.0, 2.0, -14.0, 33.0], [-23.0, -34.0, 7.0, -10.0, -78.0], [51.0, -11.0, -62.0, 45.0, 27.0]], [[-48.0, -56.0, -24.0, -4.0, -36.0], [-44.0, -64.0, 32.0, -8.0, 24.0], [-8.0, -40.0, 40.0, 20.0, -24.0], [24.0, 4.0, -20.0, 36.0, 36.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, -6.0, -6.0, 0.0, 2.0], [2.0, -4.0, -4.0, 1.0, -7.0], [-6.0, -5.0, -4.0, 5.0, -3.0], [6.0, 4.0, -5.0, 6.0, -6.0]], [[-1.0, -7.0, 0.0, 3.0, 0.0], [3.0, -2.0, 2.0, 7.0, 7.0], [1.0, 7.0, -5.0, -3.0, -2.0], [-3.0, 4.0, 6.0, -2.0, -2.0]], [[-7.0, -3.0, -6.0, 2.0, -1.0], [3.0, 0.0, 3.0, 2.0, 0.0], [-6.0, -5.0, 0.0, -5.0, 2.0], [4.0, -7.0, -3.0, 1.0, 4.0]]], [[[0.0, -1.0, 3.0, 0.0, 6.0], [0.0, 2.0, -6.0, 5.0, 0.0], [7.0, 6.0, -2.0, -5.0, 0.0], [1.0, 3.0, 5.0, 0.0, 0.0]], [[-4.0, -4.0, -1.0, -4.0, 7.0], [5.0, -2.0, 0.0, 3.0, 0.0], [2.0, -2.0, -6.0, -5.0, -2.0], [4.0, 7.0, -4.0, -4.0, 0.0]], [[1.0, -2.0, 7.0, -2.0, -4.0], [5.0, 0.0, 1.0, 0.0, 3.0], [-6.0, 4.0, 4.0, 7.0, 1.0], [-1.0, 1.0, -3.0, -3.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 4.0, 5.0, -2.0, 0.0], [-1.0, -5.0, -7.0, 4.0, 0.0], [0.0, -7.0, -2.0, -4.0, -7.0], [6.0, 6.0, -1.0, 5.0, -2.0]], [[0.0, -5.0, 2.0, 4.0, -4.0], [-7.0, 6.0, 3.0, -4.0, 5.0], [0.0, -4.0, -7.0, 7.0, -4.0], [7.0, 3.0, -6.0, 0.0, 5.0]], [[-7.0, -7.0, -4.0, 0.0, 5.0], [7.0, -4.0, 6.0, -6.0, 7.0], [0.0, 7.0, -6.0, -6.0, 4.0], [-1.0, 5.0, 0.0, 5.0, -6.0]]], [[[-1.0, 0.0, -5.0, 0.0, 6.0], [-1.0, 5.0, 4.0, 0.0, 1.0], [4.0, -4.0, 0.0, -4.0, -3.0], [2.0, 5.0, 0.0, 2.0, -1.0]], [[5.0, -4.0, -6.0, 5.0, 3.0], [3.0, -7.0, -7.0, 5.0, 0.0], [-4.0, -6.0, -7.0, 3.0, -3.0], [5.0, 3.0, 4.0, 4.0, 3.0]], [[0.0, -6.0, -2.0, -3.0, -7.0], [-4.0, -3.0, -7.0, 0.0, 0.0], [5.0, -4.0, 3.0, -3.0, 3.0], [2.0, 2.0, 0.0, 3.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 4.0, -7.0, -3.0, 2.0], [2.0, -1.0, 2.0, 2.0, -4.0], [-5.0, 0.0, 1.0, -5.0, -1.0], [5.0, 7.0, 1.0, -4.0, 7.0]], [[0.0, 5.0, -2.0, 0.0, 3.0], [-2.0, -3.0, 3.0, 7.0, -3.0], [5.0, 6.0, 5.0, -4.0, 1.0], [-7.0, 2.0, 7.0, 4.0, -3.0]], [[-2.0, -1.0, -4.0, -1.0, 0.0], [-2.0, -2.0, -7.0, 7.0, 4.0], [-5.0, -5.0, -1.0, 3.0, 3.0], [0.0, 3.0, 0.0, -3.0, -3.0]]], [[[2.0, 0.0, -7.0, -6.0, -3.0], [6.0, 3.0, -2.0, -2.0, -3.0], [6.0, 2.0, 2.0, -1.0, 0.0], [7.0, -1.0, 2.0, -4.0, -7.0]], [[5.0, 5.0, -7.0, -5.0, -6.0], [5.0, -6.0, -6.0, -1.0, 1.0], [5.0, 2.0, 4.0, 2.0, 0.0], [-5.0, -3.0, -5.0, 0.0, -5.0]], [[-1.0, -6.0, -2.0, 1.0, 0.0], [4.0, 5.0, -7.0, -1.0, -2.0], [-2.0, 7.0, -5.0, 1.0, 0.0], [1.0, -2.0, -2.0, 1.0, 1.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, -1.0, -1.0, 4.0, 6.0], [6.0, -1.0, -7.0, 3.0, 0.0], [1.0, 0.0, -6.0, 7.0, -2.0], [6.0, -2.0, 5.0, -6.0, -1.0]], [[-3.0, 7.0, -5.0, -4.0, -1.0], [2.0, 1.0, 2.0, 0.0, 7.0], [-2.0, 3.0, -5.0, -5.0, -2.0], [-3.0, -3.0, -5.0, 6.0, 1.0]], [[5.0, -5.0, -2.0, -2.0, -6.0], [0.0, 0.0, 7.0, 2.0, -4.0], [3.0, -5.0, 4.0, -7.0, 1.0], [2.0, 1.0, -6.0, -7.0, -2.0]]], [[[3.0, -3.0, -5.0, -1.0, -6.0], [-2.0, -5.0, 6.0, -3.0, -6.0], [-2.0, 0.0, -2.0, 0.0, 5.0], [3.0, -3.0, 0.0, -5.0, -2.0]], [[-4.0, 0.0, -3.0, 0.0, 5.0], [3.0, 1.0, 0.0, -4.0, 0.0], [1.0, 1.0, 3.0, -6.0, 5.0], [3.0, -5.0, 0.0, 2.0, -5.0]], [[3.0, -3.0, -7.0, 5.0, 3.0], [7.0, -4.0, -3.0, 7.0, -7.0], [6.0, 4.0, -7.0, 5.0, 2.0], [0.0, -5.0, 2.0, 0.0, 4.0]]]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-7.0, -16.0, 42.0, 22.0, 25.0], [4.0, 44.0, 67.0, -49.0, -39.0], [-2.0, 28.0, -17.0, -34.0, -24.0], [38.0, 1.0, 13.0, 10.0, 59.0]], [[-14.0, -64.0, 107.0, 45.0, -6.0], [-26.0, 44.0, 28.0, -74.0, 36.0], [-12.0, -51.0, -46.0, 40.0, -10.0], [30.0, -48.0, -33.0, 7.0, 13.0]], [[6.0, -65.0, 38.0, 18.0, -15.0], [41.0, 68.0, -19.0, -66.0, -24.0], [59.0, 83.0, -33.0, 14.0, -12.0], [3.0, -74.0, -9.0, 37.0, -33.0]], [[-22.0, 5.0, 18.0, 16.0, 33.0], [-38.0, 1.0, 33.0, 41.0, -1.0], [-11.0, -14.0, 7.0, -16.0, 9.0], [-3.0, 28.0, 45.0, 4.0, 3.0]], [[53.0, 68.0, -93.0, -80.0, -56.0], [61.0, -44.0, -15.0, -18.0, -15.0], [74.0, 3.0, 63.0, -9.0, -5.0], [13.0, -8.0, -13.0, -33.0, -64.0]]], [[[14.0, 15.0, -77.0, -29.0, -17.0], [37.0, -22.0, -73.0, 36.0, -10.0], [5.0, 48.0, 9.0, 2.0, 10.0], [-31.0, 4.0, -12.0, 0.0, -24.0]], [[-6.0, -36.0, 32.0, 10.0, -8.0], [-2.0, 28.0, -6.0, -24.0, 14.0], [12.0, -12.0, -18.0, 22.0, 2.0], [18.0, -28.0, -4.0, 0.0, -26.0]], [[-19.0, -15.0, 52.0, 46.0, 16.0], [-50.0, -21.0, -18.0, 38.0, 38.0], [-23.0, -8.0, -14.0, 28.0, 15.0], [-75.0, -11.0, -4.0, 40.0, 0.0]], [[21.0, 52.0, -85.0, -50.0, -10.0], [29.0, -29.0, -19.0, 52.0, -29.0], [65.0, 48.0, 56.0, -26.0, 12.0], [-24.0, 13.0, 45.0, -2.0, -70.0]], [[-3.0, -11.0, 29.0, 24.0, 7.0], [-16.0, 1.0, -14.0, 32.0, 1.0], [64.0, 63.0, 13.0, 10.0, 14.0], [-81.0, -41.0, 36.0, 59.0, -67.0]]], [[[42.0, 33.0, 13.0, -34.0, -42.0], [26.0, -6.0, 49.0, -57.0, -10.0], [129.0, 12.0, 57.0, -1.0, -11.0], [-4.0, -59.0, 17.0, 12.0, -92.0]], [[8.0, -14.0, 5.0, -2.0, 5.0], [28.0, 44.0, 18.0, -10.0, -49.0], [103.0, 103.0, 19.0, -24.0, -3.0], [-18.0, -44.0, 58.0, 43.0, -69.0]], [[-38.0, -95.0, 86.0, 58.0, 19.0], [-24.0, 69.0, -11.0, -37.0, 23.0], [-55.0, -18.0, -77.0, 34.0, -1.0], [35.0, -26.0, -21.0, 8.0, 35.0]], [[-32.0, -57.0, 47.0, 52.0, 46.0], [-1.0, 64.0, 27.0, -38.0, -35.0], [-84.0, 36.0, -70.0, -26.0, -23.0], [43.0, 20.0, -15.0, 10.0, 126.0]], [[-15.0, -40.0, 29.0, 18.0, 0.0], [-12.0, 22.0, -21.0, -5.0, 22.0], [-17.0, -19.0, -28.0, 26.0, 7.0], [10.0, -15.0, -9.0, -1.0, -10.0]]], [[[15.0, 31.0, -85.0, -49.0, -24.0], [32.0, -31.0, -44.0, 14.0, 4.0], [-37.0, -35.0, 10.0, 0.0, 3.0], [37.0, 37.0, -31.0, -53.0, 11.0]], [[54.0, 72.0, -80.0, -57.0, -31.0], [71.0, -30.0, 14.0, -27.0, -60.0], [86.0, 86.0, 60.0, -45.0, -21.0], [-21.0, -11.0, 1.0, 10.0, -18.0]], [[-1.0, 11.0, 34.0, 5.0, 17.0], [-18.0, 17.0, 64.0, 8.0, -23.0], [86.0, 32.0, 37.0, -24.0, 2.0], [-15.0, -19.0, 73.0, 30.0, -62.0]], [[-19.0, 11.0, -8.0, -2.0, 30.0], [-18.0, 7.0, 44.0, 12.0, -12.0], [-59.0, -46.0, -6.0, -30.0, -5.0], [61.0, 59.0, 18.0, -38.0, 62.0]], [[-15.0, -80.0, 16.0, 21.0, -18.0], [12.0, 39.0, -95.0, -5.0, 32.0], [-14.0, 17.0, -55.0, 57.0, 16.0], [-9.0, -48.0, -34.0, 11.0, -41.0]]]])+(1.-msk_ref)*numpy.array([[[[-57.0, 37.0, 3.0, 19.0, 79.0], [23.0, 9.0, -62.0, -28.0, 34.0], [-13.0, 40.0, -17.0, 26.0, 11.0], [4.0, -31.0, 47.0, 58.0, -10.0]], [[-18.0, 14.0, 89.0, 20.0, 33.0], [-40.0, 11.0, -69.0, -28.0, 35.0], [-47.0, -8.0, 3.0, 96.0, -37.0], [13.0, 50.0, 75.0, -13.0, 1.0]], [[-18.0, 50.0, 50.0, 15.0, 46.0], [12.0, 24.0, -83.0, 32.0, 74.0], [-19.0, 12.0, -50.0, 79.0, -77.0], [-17.0, 35.0, 35.0, 23.0, 37.0]], [[-43.0, 39.0, -12.0, -39.0, 0.0], [-10.0, 23.0, 31.0, -47.0, 49.0], [-23.0, 5.0, 28.0, -79.0, 15.0], [-9.0, -18.0, -36.0, 46.0, -31.0]], [[22.0, -50.0, 20.0, -35.0, -68.0], [-60.0, -3.0, 84.0, -69.0, -35.0], [-28.0, -62.0, 86.0, -68.0, 44.0], [49.0, 19.0, -24.0, -83.0, -69.0]]], [[[28.0, -68.0, 46.0, -9.0, -32.0], [-37.0, 18.0, 48.0, -26.0, -43.0], [15.0, -69.0, 108.0, -59.0, 25.0], [35.0, 38.0, -20.0, -74.0, -49.0]], [[-9.0, 61.0, 0.0, -56.0, -86.0], [-70.0, -9.0, 58.0, -31.0, 49.0], [-64.0, 19.0, -26.0, -10.0, -22.0], [-62.0, 23.0, -37.0, 55.0, 32.0]], [[33.0, 7.0, 30.0, -91.0, -161.0], [-114.0, 11.0, 142.0, -42.0, 22.0], [-52.0, -56.0, 71.0, -99.0, -15.0], [-48.0, 69.0, -100.0, -16.0, -3.0]], [[-34.0, -2.0, 13.0, 44.0, 89.0], [31.0, -3.0, -78.0, -20.0, -4.0], [-1.0, 23.0, -13.0, 60.0, 19.0], [39.0, -27.0, 76.0, 4.0, -21.0]], [[23.0, -3.0, -44.0, -35.0, -53.0], [8.0, 0.0, 65.0, 11.0, 1.0], [9.0, -20.0, 1.0, -74.0, 2.0], [2.0, -11.0, -67.0, -24.0, -11.0]]], [[[43.0, -27.0, -43.0, 21.0, -29.0], [15.0, -44.0, 9.0, 39.0, -59.0], [18.0, 16.0, -55.0, 49.0, 10.0], [0.0, -17.0, 10.0, -28.0, 32.0]], [[40.0, -32.0, 79.0, -42.0, -98.0], [-88.0, 33.0, 78.0, 1.0, -4.0], [0.0, -69.0, 108.0, -62.0, -29.0], [-46.0, 95.0, -65.0, -23.0, 11.0]], [[26.0, -26.0, 49.0, 47.0, 5.0], [-26.0, -24.0, -51.0, 31.0, -46.0], [5.0, 14.0, -19.0, 120.0, -17.0], [-24.0, 39.0, 67.0, -2.0, 54.0]], [[-88.0, 104.0, 13.0, -43.0, 35.0], [-14.0, 46.0, -15.0, -45.0, 118.0], [-43.0, 42.0, 3.0, -54.0, -23.0], [-60.0, -7.0, -25.0, 134.0, 4.0]], [[37.0, -41.0, 22.0, -2.0, -50.0], [-32.0, 3.0, 42.0, 29.0, -47.0], [34.0, -23.0, 54.0, -24.0, 2.0], [-40.0, 39.0, -33.0, -1.0, 28.0]]], [[[-28.0, 36.0, -78.0, 6.0, 54.0], [75.0, -12.0, -29.0, 4.0, 27.0], [2.0, 39.0, -78.0, -6.0, 12.0], [34.0, -75.0, 5.0, 13.0, -18.0]], [[31.0, -31.0, -79.0, 7.0, -6.0], [55.0, -33.0, 23.0, 15.0, -43.0], [20.0, -5.0, -46.0, -16.0, 31.0], [61.0, -56.0, -11.0, -72.0, -30.0]], [[1.0, -41.0, 19.0, 20.0, 20.0], [-6.0, -1.0, -5.0, -19.0, -42.0], [15.0, -14.0, 48.0, -1.0, 34.0], [24.0, 0.0, 25.0, -22.0, -25.0]], [[29.0, -45.0, -58.0, 23.0, 17.0], [59.0, -23.0, 3.0, 24.0, -53.0], [38.0, -9.0, -23.0, -9.0, 31.0], [58.0, -46.0, 1.0, -67.0, -27.0]], [[-65.0, 73.0, -8.0, -20.0, 37.0], [2.0, 19.0, -21.0, -34.0, 72.0], [-31.0, 44.0, -19.0, -20.0, -4.0], [-33.0, -24.0, -1.0, 95.0, 2.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-2.0, 3.0], [-3.0, 4.0]], [[4.0, 1.0], [1.0, 4.0]], [[-4.0, 3.0], [6.0, -6.0]], [[-4.0, 4.0], [3.0, -6.0]], [[0.0, 4.0], [7.0, 6.0]], [[7.0, 1.0], [-7.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[-5.0, -2.0], [2.0, 1.0]], [[0.0, -4.0], [-1.0, -1.0]], [[-4.0, 2.0], [4.0, -6.0]], [[-1.0, 6.0], [-5.0, 5.0]], [[-6.0, 0.0], [0.0, -7.0]], [[2.0, -1.0], [-1.0, 4.0]]])
      arg1=0.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [-0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -0.0], [0.0, 0.0]], [[0.0, -0.0], [-0.0, -0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[0.0, -0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[1.0, 6.0], [-4.0, -5.0]], [[-1.0, 0.0], [-7.0, 0.0]], [[7.0, 5.0], [-3.0, 5.0]], [[-2.0, -6.0], [1.0, 3.0]], [[-6.0, -6.0], [-5.0, 1.0]], [[-7.0, 0.0], [5.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[4.0, 7.0], [3.0, -4.0]], [[-1.0, 3.0], [6.0, 0.0]], [[-3.0, -2.0], [2.0, 7.0]], [[-2.0, 0.0], [3.0, -4.0]], [[-1.0, -5.0], [-7.0, 1.0]], [[1.0, 2.0], [4.0, -7.0]]])
      arg1=numpy.array(-5.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-5.0, -30.0], [20.0, 25.0]], [[5.0, -0.0], [35.0, -0.0]], [[-35.0, -25.0], [15.0, -25.0]], [[10.0, 30.0], [-5.0, -15.0]], [[30.0, 30.0], [25.0, -5.0]], [[35.0, -0.0], [-25.0, -25.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, -35.0], [-15.0, 20.0]], [[5.0, -15.0], [-30.0, -0.0]], [[15.0, 10.0], [-10.0, -35.0]], [[10.0, -0.0], [-15.0, 20.0]], [[5.0, 25.0], [35.0, -5.0]], [[-5.0, -10.0], [-20.0, 35.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 2.0], [-2.0, -3.0]], [[0.0, -5.0], [-5.0, 4.0]], [[3.0, -5.0], [4.0, -2.0]], [[7.0, 0.0], [-6.0, 2.0]], [[-5.0, -5.0], [3.0, -1.0]], [[-5.0, 5.0], [-6.0, -5.0]]], [[[-1.0, 4.0], [-1.0, -4.0]], [[-3.0, 0.0], [5.0, 6.0]], [[1.0, 4.0], [-5.0, -7.0]], [[1.0, 6.0], [-2.0, 4.0]], [[-3.0, 7.0], [4.0, -5.0]], [[1.0, 1.0], [0.0, -2.0]]], [[[-2.0, 0.0], [6.0, 1.0]], [[-5.0, 1.0], [3.0, 2.0]], [[-4.0, 5.0], [3.0, 1.0]], [[6.0, -6.0], [3.0, 1.0]], [[-6.0, 7.0], [-3.0, 0.0]], [[-7.0, 2.0], [-4.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -7.0], [-2.0, 4.0]], [[0.0, -1.0], [-6.0, 5.0]], [[0.0, -2.0], [-4.0, 0.0]], [[-3.0, 4.0], [-7.0, -1.0]], [[-7.0, 7.0], [-1.0, 0.0]], [[0.0, 0.0], [3.0, 0.0]]], [[[-1.0, 3.0], [4.0, 3.0]], [[-2.0, 0.0], [-1.0, 2.0]], [[-6.0, -1.0], [-4.0, -4.0]], [[2.0, 1.0], [-5.0, 6.0]], [[0.0, -4.0], [-6.0, 4.0]], [[-6.0, -4.0], [4.0, -3.0]]], [[[-3.0, 7.0], [3.0, -2.0]], [[7.0, -3.0], [0.0, 6.0]], [[-6.0, 4.0], [4.0, -5.0]], [[-6.0, 0.0], [-5.0, 2.0]], [[-3.0, -5.0], [-3.0, -7.0]], [[-1.0, 7.0], [7.0, 7.0]]]])
      arg1=numpy.array([-1.0, 2.0, -3.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 6.0], [-18.0, -8.0]], [[9.0, 2.0], [6.0, 2.0]], [[11.0, -2.0], [-23.0, -15.0]], [[-23.0, 30.0], [-7.0, 3.0]], [[17.0, -2.0], [14.0, -9.0]], [[28.0, -9.0], [18.0, 19.0]]])+(1.-msk_ref)*numpy.array([[[7.0, -8.0], [1.0, 8.0]], [[-25.0, 10.0], [4.0, -19.0]], [[6.0, -12.0], [-16.0, 7.0]], [[25.0, -2.0], [12.0, 7.0]], [[16.0, 0.0], [-2.0, 29.0]], [[-9.0, -29.0], [-16.0, -27.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 2.0], [6.0, -3.0]], [[3.0, 6.0], [-3.0, 7.0]], [[0.0, -4.0], [1.0, -1.0]], [[-2.0, -1.0], [1.0, 6.0]], [[-2.0, -1.0], [-4.0, -7.0]], [[3.0, 2.0], [3.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 7.0], [1.0, 2.0]], [[0.0, 6.0], [-5.0, -5.0]], [[-3.0, -2.0], [-6.0, 2.0]], [[-4.0, -2.0], [7.0, 0.0]], [[-4.0, -3.0], [3.0, -3.0]], [[3.0, -6.0], [5.0, -6.0]]])
      arg1=numpy.array([-3.0, -7.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 7.0], [-6.0, -14.0]], [[-18.0, -42.0], [9.0, 21.0]]], [[[-9.0, -21.0], [-18.0, -42.0]], [[9.0, 21.0], [-21.0, -49.0]]], [[[0.0, 0.0], [12.0, 28.0]], [[-3.0, -7.0], [3.0, 7.0]]], [[[6.0, 14.0], [3.0, 7.0]], [[-3.0, -7.0], [-18.0, -42.0]]], [[[6.0, 14.0], [3.0, 7.0]], [[12.0, 28.0], [21.0, 49.0]]], [[[-9.0, -21.0], [-6.0, -14.0]], [[-9.0, -21.0], [12.0, 28.0]]]])+(1.-msk_ref)*numpy.array([[[[-9.0, -21.0], [-21.0, -49.0]], [[-3.0, -7.0], [-6.0, -14.0]]], [[[0.0, 0.0], [-18.0, -42.0]], [[15.0, 35.0], [15.0, 35.0]]], [[[9.0, 21.0], [6.0, 14.0]], [[18.0, 42.0], [-6.0, -14.0]]], [[[12.0, 28.0], [6.0, 14.0]], [[-21.0, -49.0], [0.0, 0.0]]], [[[12.0, 28.0], [9.0, 21.0]], [[-9.0, -21.0], [9.0, 21.0]]], [[[-9.0, -21.0], [18.0, 42.0]], [[-15.0, -35.0], [18.0, 42.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 0.0], [-4.0, -7.0]], [[-1.0, 5.0], [1.0, -7.0]], [[-2.0, 5.0], [-5.0, 2.0]], [[7.0, -2.0], [-2.0, 4.0]], [[2.0, -7.0], [3.0, 4.0]], [[1.0, 2.0], [0.0, 7.0]]], [[[3.0, 0.0], [5.0, -7.0]], [[1.0, 6.0], [0.0, -6.0]], [[-4.0, -4.0], [1.0, 5.0]], [[-5.0, -1.0], [1.0, 2.0]], [[4.0, 6.0], [-6.0, 0.0]], [[6.0, -6.0], [-4.0, -4.0]]], [[[-4.0, 6.0], [-1.0, -7.0]], [[-2.0, -3.0], [2.0, -3.0]], [[-1.0, -6.0], [-2.0, 0.0]], [[0.0, 0.0], [5.0, 1.0]], [[-6.0, -4.0], [3.0, -3.0]], [[7.0, 6.0], [-2.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -7.0], [4.0, -7.0]], [[-7.0, -6.0], [-7.0, 4.0]], [[-4.0, -5.0], [6.0, -5.0]], [[1.0, -6.0], [1.0, -2.0]], [[6.0, -7.0], [0.0, -5.0]], [[4.0, -2.0], [-4.0, -4.0]]], [[[-4.0, 1.0], [1.0, -2.0]], [[1.0, -7.0], [-2.0, -1.0]], [[-5.0, -7.0], [-5.0, 3.0]], [[1.0, -4.0], [-5.0, 6.0]], [[-2.0, 7.0], [-1.0, -1.0]], [[5.0, -5.0], [-6.0, -5.0]]], [[[6.0, 0.0], [6.0, 0.0]], [[3.0, -5.0], [6.0, -5.0]], [[-2.0, -7.0], [4.0, 0.0]], [[-7.0, 2.0], [-6.0, 5.0]], [[1.0, -4.0], [-1.0, 1.0]], [[1.0, 0.0], [-4.0, 0.0]]]])
      arg1=numpy.array([[6.0, 4.0], [-1.0, -3.0], [5.0, -2.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-23.0, -1.0], [30.0, -12.0]], [[-34.0, -29.0], [-70.0, 7.0]]], [[[-17.0, -3.0], [9.0, 8.0]], [[16.0, 0.0], [-51.0, -4.0]]], [[[-13.0, 6.0], [4.0, 44.0]], [[-41.0, -19.0], [7.0, -7.0]]], [[[47.0, 43.0], [-11.0, -5.0]], [[12.0, -21.0], [27.0, 8.0]]], [[[-22.0, 8.0], [-68.0, -38.0]], [[39.0, 24.0], [9.0, 22.0]]], [[[35.0, -28.0], [48.0, 14.0]], [[-6.0, 16.0], [26.0, 48.0]]]])+(1.-msk_ref)*numpy.array([[[[34.0, 0.0], [-43.0, -31.0]], [[53.0, 1.0], [-40.0, -22.0]]], [[[-28.0, -37.0], [-54.0, 7.0]], [[-10.0, -34.0], [0.0, 29.0]]], [[[-29.0, 3.0], [-58.0, 15.0]], [[61.0, 31.0], [-33.0, -29.0]]], [[[-30.0, 15.0], [-22.0, -16.0]], [[-19.0, 31.0], [7.0, -36.0]]], [[[43.0, 28.0], [-69.0, -41.0]], [[-4.0, 5.0], [-24.0, -19.0]]], [[[24.0, -1.0], [-7.0, 7.0]], [[-38.0, 10.0], [-19.0, -1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, -6.0], [7.0, -2.0]], [[-6.0, 3.0], [-3.0, -6.0]], [[-7.0, 4.0], [4.0, -1.0]], [[6.0, 2.0], [7.0, -7.0]], [[3.0, 6.0], [1.0, -4.0]], [[5.0, 5.0], [-5.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 4.0], [7.0, -1.0]], [[0.0, -6.0], [-3.0, 0.0]], [[0.0, 5.0], [-2.0, -2.0]], [[4.0, 0.0], [-7.0, 6.0]], [[-3.0, -6.0], [5.0, -6.0]], [[-5.0, 3.0], [-3.0, -7.0]]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-15.0, -18.0], [21.0, -6.0]], [[-18.0, 9.0], [-9.0, -18.0]], [[-21.0, 12.0], [12.0, -3.0]], [[18.0, 6.0], [21.0, -21.0]], [[9.0, 18.0], [3.0, -12.0]], [[15.0, 15.0], [-15.0, -18.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, 12.0], [21.0, -3.0]], [[0.0, -18.0], [-9.0, 0.0]], [[0.0, 15.0], [-6.0, -6.0]], [[12.0, 0.0], [-21.0, 18.0]], [[-9.0, -18.0], [15.0, -18.0]], [[-15.0, 9.0], [-9.0, -21.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, 0.0], [1.0, 0.0]], [[1.0, -3.0], [7.0, 6.0]], [[1.0, 5.0], [7.0, 6.0]], [[0.0, 0.0], [6.0, 6.0]], [[0.0, -1.0], [4.0, 1.0]], [[1.0, 1.0], [4.0, 7.0]]], [[[0.0, 1.0], [5.0, 6.0]], [[1.0, 0.0], [-6.0, 7.0]], [[5.0, 1.0], [1.0, 1.0]], [[-1.0, 0.0], [-3.0, 4.0]], [[0.0, 4.0], [6.0, -2.0]], [[0.0, -5.0], [-2.0, 4.0]]], [[[5.0, -5.0], [-5.0, 7.0]], [[-3.0, 2.0], [-3.0, -4.0]], [[-1.0, 5.0], [6.0, 6.0]], [[4.0, -5.0], [0.0, -3.0]], [[-7.0, 2.0], [-7.0, 4.0]], [[2.0, -6.0], [4.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, -7.0], [7.0, -4.0]], [[-7.0, -5.0], [5.0, 3.0]], [[-3.0, 2.0], [2.0, -5.0]], [[-3.0, 2.0], [-1.0, 0.0]], [[1.0, -3.0], [6.0, -5.0]], [[-3.0, -5.0], [-3.0, -5.0]]], [[[2.0, -4.0], [-5.0, -5.0]], [[0.0, 4.0], [4.0, 7.0]], [[-2.0, 1.0], [7.0, 5.0]], [[-6.0, -6.0], [3.0, 6.0]], [[0.0, 3.0], [1.0, -7.0]], [[2.0, 6.0], [-3.0, -1.0]]], [[[0.0, 0.0], [1.0, 2.0]], [[-5.0, 6.0], [-4.0, -3.0]], [[1.0, -1.0], [0.0, -4.0]], [[1.0, 1.0], [-3.0, 7.0]], [[1.0, 0.0], [5.0, -5.0]], [[4.0, 4.0], [-3.0, -3.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-1.0, 5.0, 5.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[28.0, -20.0], [-1.0, 65.0]], [[-11.0, 13.0], [-52.0, 9.0]], [[19.0, 25.0], [28.0, 29.0]], [[15.0, -25.0], [-21.0, -1.0]], [[-35.0, 31.0], [-9.0, 9.0]], [[9.0, -56.0], [6.0, 23.0]]])+(1.-msk_ref)*numpy.array([[[11.0, -13.0], [-27.0, -11.0]], [[-18.0, 55.0], [-5.0, 17.0]], [[-2.0, -2.0], [33.0, 10.0]], [[-22.0, -27.0], [1.0, 65.0]], [[4.0, 18.0], [24.0, -55.0]], [[33.0, 55.0], [-27.0, -15.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, -7.0], [7.0, 7.0]], [[-4.0, 2.0], [-3.0, 0.0]], [[4.0, -3.0], [6.0, 4.0]], [[-5.0, -1.0], [-4.0, -7.0]], [[-7.0, 3.0], [0.0, -7.0]], [[-4.0, -4.0], [1.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 3.0], [-4.0, 4.0]], [[6.0, 0.0], [0.0, -2.0]], [[0.0, -4.0], [-1.0, 3.0]], [[0.0, -7.0], [4.0, 6.0]], [[5.0, -2.0], [-2.0, 7.0]], [[-2.0, 6.0], [-1.0, 1.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-6.0, -2.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-36.0, -12.0], [42.0, 14.0]], [[-42.0, -14.0], [-42.0, -14.0]]], [[[24.0, 8.0], [-12.0, -4.0]], [[18.0, 6.0], [0.0, 0.0]]], [[[-24.0, -8.0], [18.0, 6.0]], [[-36.0, -12.0], [-24.0, -8.0]]], [[[30.0, 10.0], [6.0, 2.0]], [[24.0, 8.0], [42.0, 14.0]]], [[[42.0, 14.0], [-18.0, -6.0]], [[0.0, 0.0], [42.0, 14.0]]], [[[24.0, 8.0], [24.0, 8.0]], [[-6.0, -2.0], [-18.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[42.0, 14.0], [-18.0, -6.0]], [[24.0, 8.0], [-24.0, -8.0]]], [[[-36.0, -12.0], [0.0, 0.0]], [[0.0, 0.0], [12.0, 4.0]]], [[[0.0, 0.0], [24.0, 8.0]], [[6.0, 2.0], [-18.0, -6.0]]], [[[0.0, 0.0], [42.0, 14.0]], [[-24.0, -8.0], [-36.0, -12.0]]], [[[-30.0, -10.0], [12.0, 4.0]], [[12.0, 4.0], [-42.0, -14.0]]], [[[12.0, 4.0], [-36.0, -12.0]], [[6.0, 2.0], [-6.0, -2.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, 4.0], [1.0, -3.0]], [[7.0, -6.0], [-4.0, -5.0]], [[2.0, 5.0], [-4.0, -6.0]], [[-5.0, -3.0], [3.0, 7.0]], [[-6.0, -6.0], [-4.0, -7.0]], [[4.0, 7.0], [-1.0, 2.0]]], [[[2.0, -3.0], [-7.0, -1.0]], [[5.0, 4.0], [-6.0, -4.0]], [[-3.0, 2.0], [-4.0, 7.0]], [[-4.0, -1.0], [-4.0, -1.0]], [[2.0, 0.0], [-1.0, 6.0]], [[-6.0, 4.0], [-6.0, -3.0]]], [[[3.0, 3.0], [-3.0, 2.0]], [[-3.0, -7.0], [-5.0, -5.0]], [[1.0, -5.0], [-4.0, 4.0]], [[4.0, -4.0], [5.0, 6.0]], [[-1.0, -3.0], [-2.0, -3.0]], [[-4.0, 2.0], [-7.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, -4.0], [-6.0, 0.0]], [[-3.0, 6.0], [6.0, 0.0]], [[0.0, -6.0], [3.0, 6.0]], [[-2.0, -4.0], [-6.0, 7.0]], [[-5.0, 0.0], [2.0, 6.0]], [[7.0, 4.0], [5.0, 5.0]]], [[[-6.0, 1.0], [5.0, 6.0]], [[0.0, -6.0], [-2.0, 5.0]], [[5.0, 4.0], [7.0, 6.0]], [[6.0, 0.0], [0.0, -1.0]], [[0.0, 5.0], [-7.0, -5.0]], [[6.0, 4.0], [1.0, -1.0]]], [[[3.0, -5.0], [-3.0, -1.0]], [[0.0, -3.0], [6.0, -4.0]], [[-5.0, -1.0], [-7.0, -1.0]], [[7.0, 6.0], [1.0, 2.0]], [[-6.0, 5.0], [0.0, -6.0]], [[7.0, 1.0], [-4.0, 0.0]]]])
      arg1=Symbol(shape=(3, 2))
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[1.0, 7.0], [-3.0, -1.0], [0.0, 3.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-11.0, -28.0], [13.0, 40.0]], [[22.0, 5.0], [0.0, -14.0]]], [[[-8.0, 35.0], [-18.0, -67.0]], [[14.0, -37.0], [7.0, -46.0]]], [[[11.0, 20.0], [-1.0, 18.0]], [[8.0, -36.0], [-27.0, -37.0]]], [[[7.0, -19.0], [0.0, -32.0]], [[15.0, 40.0], [10.0, 68.0]]], [[[-12.0, -47.0], [-6.0, -51.0]], [[-1.0, -33.0], [-25.0, -64.0]]], [[[22.0, 22.0], [-5.0, 51.0]], [[17.0, -22.0], [11.0, 17.0]]]])+(1.-msk_ref)*numpy.array([[[[24.0, 57.0], [-7.0, -44.0]], [[-21.0, -56.0], [-18.0, -9.0]]], [[[-3.0, -21.0], [24.0, 39.0]], [[12.0, 62.0], [-15.0, -17.0]]], [[[-15.0, -20.0], [-18.0, -49.0]], [[-18.0, -7.0], [-12.0, 33.0]]], [[[-20.0, 1.0], [-4.0, -10.0]], [[-6.0, -39.0], [10.0, 56.0]]], [[[-5.0, -53.0], [-15.0, 10.0]], [[23.0, 21.0], [21.0, 29.0]]], [[[-11.0, 64.0], [-8.0, 27.0]], [[2.0, 22.0], [8.0, 36.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-4.0, -5.0], [0.0, -5.0]], [[-2.0, 0.0], [2.0, 0.0]], [[-5.0, 0.0], [4.0, -4.0]], [[-5.0, 6.0], [6.0, -6.0]], [[-1.0, 5.0], [4.0, 0.0]], [[6.0, -1.0], [-5.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[3.0, 4.0], [4.0, 6.0]], [[4.0, -6.0], [-6.0, -3.0]], [[3.0, 3.0], [-7.0, -4.0]], [[4.0, -1.0], [-4.0, -6.0]], [[-5.0, -1.0], [2.0, -3.0]], [[-3.0, 7.0], [-2.0, -4.0]]])
      arg1=Data(1.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-4.0, -5.0], [0.0, -5.0]], [[-2.0, 0.0], [2.0, 0.0]], [[-5.0, 0.0], [4.0, -4.0]], [[-5.0, 6.0], [6.0, -6.0]], [[-1.0, 5.0], [4.0, 0.0]], [[6.0, -1.0], [-5.0, -3.0]]])+(1.-msk_ref)*numpy.array([[[3.0, 4.0], [4.0, 6.0]], [[4.0, -6.0], [-6.0, -3.0]], [[3.0, 3.0], [-7.0, -4.0]], [[4.0, -1.0], [-4.0, -6.0]], [[-5.0, -1.0], [2.0, -3.0]], [[-3.0, 7.0], [-2.0, -4.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, 0.0], [-4.0, 0.0]], [[-1.0, 0.0], [0.0, -5.0]], [[-6.0, -7.0], [-1.0, 0.0]], [[0.0, -4.0], [0.0, 4.0]], [[-5.0, 5.0], [7.0, -3.0]], [[-6.0, -6.0], [6.0, 3.0]]], [[[-7.0, -1.0], [0.0, 7.0]], [[2.0, 4.0], [6.0, 4.0]], [[-2.0, 0.0], [-3.0, 0.0]], [[5.0, -5.0], [-1.0, -5.0]], [[-5.0, 6.0], [0.0, 0.0]], [[0.0, -4.0], [6.0, -3.0]]], [[[-5.0, -6.0], [4.0, -7.0]], [[-4.0, -6.0], [0.0, -6.0]], [[-3.0, -1.0], [7.0, -6.0]], [[1.0, -5.0], [-4.0, -7.0]], [[0.0, -3.0], [-3.0, -2.0]], [[4.0, -6.0], [-4.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 5.0], [0.0, 3.0]], [[6.0, 0.0], [-3.0, 0.0]], [[-3.0, -7.0], [-1.0, -5.0]], [[-3.0, -6.0], [5.0, -7.0]], [[1.0, 0.0], [-1.0, -7.0]], [[1.0, 6.0], [2.0, 3.0]]], [[[-5.0, -1.0], [1.0, 7.0]], [[6.0, 2.0], [1.0, -2.0]], [[-3.0, -2.0], [-4.0, -1.0]], [[-4.0, -3.0], [6.0, 1.0]], [[-4.0, -1.0], [3.0, -6.0]], [[2.0, 7.0], [-6.0, -5.0]]], [[[-4.0, -5.0], [6.0, 5.0]], [[-1.0, -6.0], [7.0, -3.0]], [[-3.0, -7.0], [-4.0, 6.0]], [[5.0, 4.0], [7.0, -5.0]], [[-3.0, -1.0], [-6.0, 4.0]], [[4.0, 0.0], [-6.0, 4.0]]]])
      arg1=Data(numpy.array([2.0, 6.0, 2.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-58.0, -18.0], [0.0, 28.0]], [[2.0, 12.0], [36.0, 2.0]], [[-30.0, -16.0], [-6.0, -12.0]], [[32.0, -48.0], [-14.0, -36.0]], [[-40.0, 40.0], [8.0, -10.0]], [[-4.0, -48.0], [40.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[-46.0, -6.0], [18.0, 58.0]], [[46.0, 0.0], [14.0, -18.0]], [[-30.0, -40.0], [-34.0, -4.0]], [[-20.0, -22.0], [60.0, -18.0]], [[-28.0, -8.0], [4.0, -42.0]], [[22.0, 54.0], [-44.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-6.0, 3.0], [-3.0, -3.0]], [[1.0, 6.0], [0.0, -4.0]], [[7.0, 4.0], [2.0, 6.0]], [[4.0, 2.0], [-1.0, -4.0]], [[-4.0, 0.0], [4.0, -2.0]], [[4.0, -5.0], [6.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[5.0, -2.0], [-4.0, 5.0]], [[7.0, -2.0], [3.0, 1.0]], [[-4.0, 0.0], [0.0, -1.0]], [[6.0, 5.0], [2.0, 1.0]], [[-2.0, 5.0], [-2.0, -5.0]], [[0.0, 5.0], [1.0, -1.0]]])
      arg1=Data(numpy.array([6.0, -1.0]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-36.0, 6.0], [18.0, -3.0]], [[-18.0, 3.0], [-18.0, 3.0]]], [[[6.0, -1.0], [36.0, -6.0]], [[0.0, 0.0], [-24.0, 4.0]]], [[[42.0, -7.0], [24.0, -4.0]], [[12.0, -2.0], [36.0, -6.0]]], [[[24.0, -4.0], [12.0, -2.0]], [[-6.0, 1.0], [-24.0, 4.0]]], [[[-24.0, 4.0], [0.0, 0.0]], [[24.0, -4.0], [-12.0, 2.0]]], [[[24.0, -4.0], [-30.0, 5.0]], [[36.0, -6.0], [18.0, -3.0]]]])+(1.-msk_ref)*numpy.array([[[[30.0, -5.0], [-12.0, 2.0]], [[-24.0, 4.0], [30.0, -5.0]]], [[[42.0, -7.0], [-12.0, 2.0]], [[18.0, -3.0], [6.0, -1.0]]], [[[-24.0, 4.0], [0.0, 0.0]], [[0.0, 0.0], [-6.0, 1.0]]], [[[36.0, -6.0], [30.0, -5.0]], [[12.0, -2.0], [6.0, -1.0]]], [[[-12.0, 2.0], [30.0, -5.0]], [[-12.0, 2.0], [-30.0, 5.0]]], [[[0.0, 0.0], [30.0, -5.0]], [[6.0, -1.0], [-6.0, 1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -2.0], [4.0, -5.0]], [[-3.0, -2.0], [0.0, 0.0]], [[5.0, -2.0], [0.0, 2.0]], [[5.0, 2.0], [0.0, -4.0]], [[6.0, -1.0], [1.0, 7.0]], [[-5.0, -4.0], [7.0, 0.0]]], [[[6.0, -2.0], [2.0, -3.0]], [[6.0, -4.0], [-3.0, -3.0]], [[-1.0, 7.0], [2.0, 4.0]], [[6.0, -4.0], [-4.0, 6.0]], [[-4.0, -5.0], [-6.0, 1.0]], [[-2.0, -5.0], [4.0, 6.0]]], [[[0.0, 5.0], [4.0, -3.0]], [[7.0, -4.0], [1.0, 4.0]], [[0.0, 2.0], [-7.0, -4.0]], [[7.0, 1.0], [3.0, -4.0]], [[-5.0, 5.0], [4.0, 0.0]], [[4.0, 7.0], [-6.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[7.0, -4.0], [-1.0, 0.0]], [[4.0, -5.0], [7.0, 0.0]], [[-1.0, 3.0], [-1.0, -2.0]], [[4.0, -2.0], [3.0, 5.0]], [[7.0, -2.0], [7.0, 0.0]], [[-7.0, -7.0], [7.0, 4.0]]], [[[-5.0, 4.0], [0.0, 5.0]], [[0.0, 0.0], [-7.0, -7.0]], [[-4.0, -6.0], [0.0, 3.0]], [[-5.0, 6.0], [-6.0, 4.0]], [[5.0, 3.0], [-2.0, 2.0]], [[-1.0, 2.0], [0.0, -2.0]]], [[[1.0, -3.0], [0.0, 0.0]], [[1.0, -3.0], [6.0, 4.0]], [[-5.0, 6.0], [5.0, 4.0]], [[6.0, 0.0], [-7.0, -3.0]], [[-1.0, -3.0], [-2.0, -5.0]], [[-5.0, -1.0], [-2.0, -4.0]]]])
      arg1=Data(numpy.array([[-3.0, 5.0], [6.0, -5.0], [7.0, -1.0]]),self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[18.0, 0.0], [29.0, -5.0]], [[28.0, 6.0], [-24.0, -7.0]]], [[[94.0, -52.0], [-46.0, 14.0]], [[-11.0, 14.0], [10.0, 11.0]]], [[[-21.0, 30.0], [62.0, -47.0]], [[-37.0, -3.0], [-10.0, -6.0]]], [[[70.0, -12.0], [-23.0, 29.0]], [[-3.0, 17.0], [20.0, -46.0]]], [[[-77.0, 55.0], [8.0, 15.0]], [[-11.0, 31.0], [-15.0, 30.0]]], [[[31.0, -19.0], [31.0, -2.0]], [[-39.0, 21.0], [71.0, -35.0]]]])+(1.-msk_ref)*numpy.array([[[[-44.0, 59.0], [15.0, -37.0]], [[3.0, -5.0], [30.0, -25.0]]], [[[-5.0, 19.0], [-6.0, -22.0]], [[-21.0, 64.0], [-14.0, 31.0]]], [[[-56.0, 20.0], [-3.0, 39.0]], [[38.0, -10.0], [52.0, -29.0]]], [[[0.0, 39.0], [42.0, -40.0]], [[-94.0, 52.0], [-12.0, 8.0]]], [[[2.0, 11.0], [3.0, -22.0]], [[-47.0, 47.0], [-23.0, -5.0]]], [[[-20.0, -25.0], [26.0, -44.0]], [[-35.0, 37.0], [-52.0, 34.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[6.0, -6.0], [2.0, -7.0]], [[4.0, 4.0], [7.0, -7.0]], [[-3.0, 7.0], [6.0, -1.0]], [[5.0, -7.0], [7.0, -7.0]], [[5.0, -7.0], [-3.0, -6.0]], [[4.0, -5.0], [-6.0, -3.0]]])+(1.-msk_arg0)*numpy.array([[[5.0, -6.0], [0.0, -2.0]], [[0.0, -1.0], [3.0, -3.0]], [[3.0, -4.0], [4.0, 7.0]], [[0.0, 0.0], [7.0, 4.0]], [[-7.0, 4.0], [3.0, 0.0]], [[1.0, -4.0], [-2.0, 7.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(5.0)+(1-msk_arg1)*(1.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[30.0, -30.0], [10.0, -35.0]], [[20.0, 20.0], [35.0, -35.0]], [[-15.0, 35.0], [30.0, -5.0]], [[25.0, -35.0], [35.0, -35.0]], [[25.0, -35.0], [-15.0, -30.0]], [[20.0, -25.0], [-30.0, -15.0]]])+(1.-msk_ref)*numpy.array([[[5.0, -6.0], [0.0, -2.0]], [[0.0, -1.0], [3.0, -3.0]], [[3.0, -4.0], [4.0, 7.0]], [[0.0, 0.0], [7.0, 4.0]], [[-7.0, 4.0], [3.0, 0.0]], [[1.0, -4.0], [-2.0, 7.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, -6.0], [7.0, 0.0]], [[-1.0, 3.0], [2.0, 3.0]], [[-1.0, -1.0], [4.0, -1.0]], [[-7.0, -3.0], [4.0, 3.0]], [[2.0, -6.0], [7.0, 0.0]], [[-3.0, 3.0], [-2.0, -3.0]]], [[[-3.0, 3.0], [4.0, 4.0]], [[2.0, 0.0], [-7.0, -2.0]], [[-1.0, -2.0], [-2.0, -6.0]], [[-1.0, 1.0], [-4.0, 3.0]], [[-3.0, 0.0], [-5.0, 0.0]], [[-6.0, -1.0], [7.0, 7.0]]], [[[-7.0, 1.0], [-7.0, 2.0]], [[1.0, 4.0], [-5.0, 0.0]], [[5.0, -7.0], [-2.0, -3.0]], [[-7.0, -1.0], [3.0, -6.0]], [[-1.0, 0.0], [4.0, -5.0]], [[-1.0, 5.0], [-5.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 0.0], [6.0, -6.0]], [[-6.0, -7.0], [6.0, -1.0]], [[-1.0, -6.0], [0.0, -3.0]], [[-7.0, -1.0], [-7.0, -5.0]], [[4.0, -3.0], [4.0, -6.0]], [[-1.0, -1.0], [2.0, 0.0]]], [[[-6.0, -3.0], [-1.0, 3.0]], [[-1.0, 1.0], [6.0, 4.0]], [[5.0, -5.0], [-4.0, 6.0]], [[-6.0, 5.0], [1.0, -7.0]], [[7.0, 0.0], [2.0, -6.0]], [[1.0, -6.0], [0.0, -4.0]]], [[[2.0, -1.0], [-7.0, -1.0]], [[-4.0, 2.0], [7.0, -5.0]], [[-7.0, 2.0], [-2.0, -4.0]], [[6.0, -7.0], [-3.0, -5.0]], [[-3.0, -1.0], [-1.0, -6.0]], [[6.0, 4.0], [6.0, -7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 3.0, -6.0])+(1.-msk_arg1)*numpy.array([-2.0, 5.0, 6.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[41.0, 27.0], [26.0, 0.0]], [[4.0, -36.0], [1.0, -18.0]], [[-29.0, 40.0], [-10.0, 4.0]], [[67.0, 21.0], [-46.0, 33.0]], [[-11.0, 24.0], [-67.0, 30.0]], [[0.0, -45.0], [59.0, 63.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -21.0], [-59.0, 21.0]], [[-17.0, 31.0], [60.0, -8.0]], [[-15.0, -1.0], [-32.0, 12.0]], [[20.0, -15.0], [1.0, -55.0]], [[9.0, 0.0], [-4.0, -54.0]], [[43.0, -4.0], [32.0, -62.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank3_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-4.0, -2.0], [0.0, 0.0]], [[1.0, 0.0], [6.0, 4.0]], [[-1.0, 2.0], [0.0, -3.0]], [[-1.0, 6.0], [6.0, 0.0]], [[0.0, -5.0], [-6.0, -5.0]], [[1.0, 0.0], [0.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, -2.0], [-4.0, 6.0]], [[-5.0, -5.0], [1.0, 5.0]], [[7.0, 5.0], [2.0, 3.0]], [[-4.0, 0.0], [-6.0, 3.0]], [[2.0, -2.0], [-4.0, -7.0]], [[3.0, 0.0], [-2.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, 1.0])+(1.-msk_arg1)*numpy.array([3.0, 0.0])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-16.0, -4.0], [-8.0, -2.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[4.0, 1.0], [0.0, 0.0]], [[24.0, 6.0], [16.0, 4.0]]], [[[-4.0, -1.0], [8.0, 2.0]], [[0.0, 0.0], [-12.0, -3.0]]], [[[-4.0, -1.0], [24.0, 6.0]], [[24.0, 6.0], [0.0, 0.0]]], [[[0.0, 0.0], [-20.0, -5.0]], [[-24.0, -6.0], [-20.0, -5.0]]], [[[4.0, 1.0], [0.0, 0.0]], [[0.0, 0.0], [-24.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 0.0], [-6.0, 0.0]], [[-12.0, 0.0], [18.0, 0.0]]], [[[-15.0, 0.0], [-15.0, 0.0]], [[3.0, 0.0], [15.0, 0.0]]], [[[21.0, 0.0], [15.0, 0.0]], [[6.0, 0.0], [9.0, 0.0]]], [[[-12.0, 0.0], [0.0, 0.0]], [[-18.0, 0.0], [9.0, 0.0]]], [[[6.0, 0.0], [-6.0, 0.0]], [[-12.0, 0.0], [-21.0, 0.0]]], [[[9.0, 0.0], [0.0, 0.0]], [[-6.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, -3.0], [6.0, 1.0]], [[6.0, 5.0], [1.0, 0.0]], [[6.0, 6.0], [1.0, -2.0]], [[7.0, 6.0], [-4.0, 2.0]], [[7.0, 5.0], [7.0, -1.0]], [[1.0, 4.0], [3.0, -3.0]]], [[[6.0, -7.0], [-6.0, -5.0]], [[0.0, 7.0], [-4.0, 1.0]], [[-4.0, 5.0], [-6.0, -4.0]], [[-3.0, 5.0], [0.0, 7.0]], [[1.0, -5.0], [4.0, -2.0]], [[-3.0, 7.0], [3.0, 4.0]]], [[[-4.0, -7.0], [6.0, -2.0]], [[7.0, 4.0], [-4.0, -1.0]], [[-3.0, 1.0], [4.0, 3.0]], [[5.0, 0.0], [-5.0, 3.0]], [[7.0, 1.0], [-4.0, 7.0]], [[0.0, -5.0], [3.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 4.0], [-1.0, 3.0]], [[6.0, -3.0], [-6.0, -5.0]], [[-6.0, 1.0], [7.0, -5.0]], [[2.0, 1.0], [-3.0, 7.0]], [[-7.0, -3.0], [3.0, 2.0]], [[7.0, 6.0], [-7.0, 4.0]]], [[[-2.0, -7.0], [4.0, 3.0]], [[2.0, 0.0], [0.0, 7.0]], [[0.0, -6.0], [2.0, -3.0]], [[0.0, 7.0], [-5.0, -3.0]], [[-3.0, 6.0], [0.0, -4.0]], [[-5.0, -3.0], [0.0, 0.0]]], [[[0.0, 4.0], [0.0, 7.0]], [[-5.0, -6.0], [-3.0, 0.0]], [[-2.0, 6.0], [-1.0, -6.0]], [[2.0, -6.0], [-7.0, -7.0]], [[-7.0, 5.0], [-6.0, 0.0]], [[5.0, -6.0], [0.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 2.0], [7.0, -7.0], [1.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-3.0, 6.0], [1.0, -3.0], [2.0, 7.0]])
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[63.0, -52.0], [-41.0, 43.0]], [[-66.0, 54.0], [-42.0, 37.0]]], [[[-23.0, 12.0], [28.0, -39.0]], [[-37.0, 30.0], [6.0, -7.0]]], [[[-61.0, 40.0], [6.0, -23.0]], [[-43.0, 44.0], [-15.0, 24.0]]], [[[-51.0, 35.0], [5.0, -23.0]], [[15.0, -8.0], [42.0, -45.0]]], [[[-21.0, 7.0], [-59.0, 45.0]], [[-11.0, -14.0], [-2.0, 12.0]]], [[[-26.0, 23.0], [24.0, -41.0]], [[9.0, -15.0], [44.0, -34.0]]]])+(1.-msk_ref)*numpy.array([[[[10.0, -18.0], [-11.0, 73.0]], [[7.0, -18.0], [8.0, 58.0]]], [[[-26.0, -5.0], [-3.0, -60.0]], [[12.0, -57.0], [22.0, -51.0]]], [[[14.0, -50.0], [3.0, 66.0]], [[-21.0, 29.0], [0.0, -63.0]]], [[[-2.0, 26.0], [-8.0, -57.0]], [[-10.0, -52.0], [-38.0, 2.0]]], [[[4.0, -82.0], [25.0, -1.0]], [[-21.0, -24.0], [-10.0, 24.0]]], [[[-16.0, 92.0], [-33.0, 3.0]], [[21.0, -42.0], [-12.0, 24.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, 3.0, -2.0, -4.0], [-5.0, -6.0, 1.0, -5.0], [-2.0, 7.0, 4.0, 1.0]], [[0.0, 0.0, 4.0, 0.0], [1.0, -1.0, -3.0, 7.0], [5.0, -4.0, 6.0, -5.0]]], [[[-5.0, 0.0, 2.0, -5.0], [-5.0, -4.0, -2.0, 6.0], [-5.0, 3.0, -4.0, -2.0]], [[3.0, 0.0, 1.0, 2.0], [5.0, 3.0, 2.0, -2.0], [-1.0, -4.0, -3.0, -6.0]]], [[[1.0, 6.0, 5.0, 7.0], [-1.0, -1.0, 0.0, 6.0], [-3.0, 0.0, 0.0, -1.0]], [[-4.0, 5.0, 2.0, 7.0], [1.0, -6.0, 5.0, 3.0], [-3.0, 7.0, 6.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, -2.0, -5.0, 5.0], [0.0, 3.0, 2.0, 2.0], [-6.0, 1.0, -3.0, 6.0]], [[-4.0, 3.0, 3.0, -4.0], [5.0, -4.0, 0.0, 0.0], [-5.0, -2.0, -5.0, -2.0]]], [[[-3.0, 0.0, -1.0, -5.0], [4.0, -6.0, -3.0, 3.0], [1.0, -7.0, 4.0, 0.0]], [[6.0, 0.0, -7.0, 0.0], [-7.0, 7.0, 0.0, 6.0], [2.0, 4.0, 0.0, 1.0]]], [[[0.0, 6.0, -6.0, 6.0], [6.0, 4.0, 5.0, -7.0], [0.0, 0.0, 0.0, 4.0]], [[-1.0, 4.0, -2.0, -1.0], [1.0, -1.0, 0.0, -1.0], [0.0, -3.0, -4.0, -7.0]]]])
      arg1=4.0
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, 12.0, -8.0, -16.0], [-20.0, -24.0, 4.0, -20.0], [-8.0, 28.0, 16.0, 4.0]], [[0.0, 0.0, 16.0, 0.0], [4.0, -4.0, -12.0, 28.0], [20.0, -16.0, 24.0, -20.0]]], [[[-20.0, 0.0, 8.0, -20.0], [-20.0, -16.0, -8.0, 24.0], [-20.0, 12.0, -16.0, -8.0]], [[12.0, 0.0, 4.0, 8.0], [20.0, 12.0, 8.0, -8.0], [-4.0, -16.0, -12.0, -24.0]]], [[[4.0, 24.0, 20.0, 28.0], [-4.0, -4.0, 0.0, 24.0], [-12.0, 0.0, 0.0, -4.0]], [[-16.0, 20.0, 8.0, 28.0], [4.0, -24.0, 20.0, 12.0], [-12.0, 28.0, 24.0, -20.0]]]])+(1.-msk_ref)*numpy.array([[[[8.0, -8.0, -20.0, 20.0], [0.0, 12.0, 8.0, 8.0], [-24.0, 4.0, -12.0, 24.0]], [[-16.0, 12.0, 12.0, -16.0], [20.0, -16.0, 0.0, 0.0], [-20.0, -8.0, -20.0, -8.0]]], [[[-12.0, 0.0, -4.0, -20.0], [16.0, -24.0, -12.0, 12.0], [4.0, -28.0, 16.0, 0.0]], [[24.0, 0.0, -28.0, 0.0], [-28.0, 28.0, 0.0, 24.0], [8.0, 16.0, 0.0, 4.0]]], [[[0.0, 24.0, -24.0, 24.0], [24.0, 16.0, 20.0, -28.0], [0.0, 0.0, 0.0, 16.0]], [[-4.0, 16.0, -8.0, -4.0], [4.0, -4.0, 0.0, -4.0], [0.0, -12.0, -16.0, -28.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, -6.0, 6.0, 0.0], [1.0, 1.0, -6.0, -7.0], [6.0, -3.0, 4.0, -7.0]], [[-2.0, 0.0, 2.0, -5.0], [4.0, -5.0, 5.0, -3.0], [-7.0, -6.0, 4.0, -7.0]]], [[[1.0, -5.0, 6.0, -2.0], [-1.0, -1.0, -5.0, -3.0], [0.0, -4.0, -1.0, -3.0]], [[0.0, 2.0, 5.0, 1.0], [-3.0, 6.0, 7.0, 3.0], [3.0, 4.0, 6.0, -6.0]]], [[[6.0, -2.0, -1.0, -3.0], [0.0, -2.0, 4.0, 0.0], [0.0, -5.0, 4.0, -4.0]], [[-2.0, 2.0, 5.0, -6.0], [-5.0, 7.0, 4.0, 0.0], [6.0, 5.0, 7.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 0.0, -7.0, -2.0], [5.0, 1.0, 1.0, -4.0], [0.0, -4.0, -6.0, -4.0]], [[1.0, 4.0, -3.0, 0.0], [-7.0, -7.0, 4.0, -5.0], [-6.0, 7.0, 7.0, 2.0]]], [[[3.0, 2.0, -7.0, -2.0], [-4.0, -3.0, 0.0, 4.0], [1.0, -6.0, -2.0, 6.0]], [[1.0, 1.0, 3.0, 0.0], [0.0, 4.0, 4.0, 7.0], [7.0, 1.0, 4.0, 0.0]]], [[[6.0, -5.0, -6.0, 1.0], [2.0, 0.0, -1.0, -5.0], [-1.0, -1.0, 0.0, 0.0]], [[5.0, 1.0, 6.0, 0.0], [-7.0, 7.0, -2.0, 0.0], [7.0, 7.0, -7.0, 1.0]]]])
      arg1=numpy.array(7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[21.0, -42.0, 42.0, 0.0], [7.0, 7.0, -42.0, -49.0], [42.0, -21.0, 28.0, -49.0]], [[-14.0, 0.0, 14.0, -35.0], [28.0, -35.0, 35.0, -21.0], [-49.0, -42.0, 28.0, -49.0]]], [[[7.0, -35.0, 42.0, -14.0], [-7.0, -7.0, -35.0, -21.0], [0.0, -28.0, -7.0, -21.0]], [[0.0, 14.0, 35.0, 7.0], [-21.0, 42.0, 49.0, 21.0], [21.0, 28.0, 42.0, -42.0]]], [[[42.0, -14.0, -7.0, -21.0], [0.0, -14.0, 28.0, 0.0], [0.0, -35.0, 28.0, -28.0]], [[-14.0, 14.0, 35.0, -42.0], [-35.0, 49.0, 28.0, 0.0], [42.0, 35.0, 49.0, -7.0]]]])+(1.-msk_ref)*numpy.array([[[[-42.0, 0.0, -49.0, -14.0], [35.0, 7.0, 7.0, -28.0], [0.0, -28.0, -42.0, -28.0]], [[7.0, 28.0, -21.0, 0.0], [-49.0, -49.0, 28.0, -35.0], [-42.0, 49.0, 49.0, 14.0]]], [[[21.0, 14.0, -49.0, -14.0], [-28.0, -21.0, 0.0, 28.0], [7.0, -42.0, -14.0, 42.0]], [[7.0, 7.0, 21.0, 0.0], [0.0, 28.0, 28.0, 49.0], [49.0, 7.0, 28.0, 0.0]]], [[[42.0, -35.0, -42.0, 7.0], [14.0, 0.0, -7.0, -35.0], [-7.0, -7.0, 0.0, 0.0]], [[35.0, 7.0, 42.0, 0.0], [-49.0, 49.0, -14.0, 0.0], [49.0, 49.0, -49.0, 7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, -1.0, -7.0, 3.0], [-1.0, -2.0, -3.0, 7.0], [-7.0, 7.0, 6.0, -2.0]], [[7.0, -5.0, 5.0, -2.0], [3.0, -3.0, -7.0, -7.0], [2.0, -1.0, -6.0, 0.0]]], [[[2.0, -2.0, 6.0, 7.0], [-1.0, 0.0, -1.0, -4.0], [0.0, -7.0, 1.0, -3.0]], [[1.0, 7.0, -4.0, -2.0], [4.0, 2.0, 2.0, 1.0], [0.0, -7.0, -5.0, 5.0]]], [[[7.0, 3.0, 0.0, -3.0], [6.0, 5.0, 5.0, 0.0], [-7.0, -3.0, 3.0, 3.0]], [[5.0, 0.0, -6.0, 1.0], [7.0, 3.0, 5.0, -1.0], [-6.0, 3.0, 0.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, 4.0, 0.0, -3.0], [3.0, -1.0, 0.0, -7.0], [-3.0, 3.0, 0.0, -4.0]], [[0.0, -3.0, -2.0, 0.0], [6.0, 7.0, 6.0, -5.0], [2.0, -3.0, 5.0, -1.0]]], [[[4.0, 1.0, -3.0, -3.0], [0.0, -4.0, -3.0, -7.0], [6.0, -6.0, 0.0, -3.0]], [[3.0, 7.0, -5.0, 5.0], [4.0, -2.0, 5.0, 0.0], [-6.0, -6.0, 1.0, 7.0]]], [[[4.0, 1.0, -3.0, -4.0], [2.0, -4.0, -6.0, 3.0], [-3.0, 1.0, 3.0, -7.0]], [[4.0, 3.0, 1.0, 2.0], [1.0, 3.0, 0.0, -1.0], [-6.0, 0.0, -3.0, -5.0]]]])
      arg1=Symbol(shape=())
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-7.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-35.0, 7.0, 49.0, -21.0], [7.0, 14.0, 21.0, -49.0], [49.0, -49.0, -42.0, 14.0]], [[-49.0, 35.0, -35.0, 14.0], [-21.0, 21.0, 49.0, 49.0], [-14.0, 7.0, 42.0, -0.0]]], [[[-14.0, 14.0, -42.0, -49.0], [7.0, -0.0, 7.0, 28.0], [-0.0, 49.0, -7.0, 21.0]], [[-7.0, -49.0, 28.0, 14.0], [-28.0, -14.0, -14.0, -7.0], [-0.0, 49.0, 35.0, -35.0]]], [[[-49.0, -21.0, -0.0, 21.0], [-42.0, -35.0, -35.0, -0.0], [49.0, 21.0, -21.0, -21.0]], [[-35.0, -0.0, 42.0, -7.0], [-49.0, -21.0, -35.0, 7.0], [42.0, -21.0, -0.0, -42.0]]]])+(1.-msk_ref)*numpy.array([[[[35.0, -28.0, -0.0, 21.0], [-21.0, 7.0, -0.0, 49.0], [21.0, -21.0, -0.0, 28.0]], [[-0.0, 21.0, 14.0, -0.0], [-42.0, -49.0, -42.0, 35.0], [-14.0, 21.0, -35.0, 7.0]]], [[[-28.0, -7.0, 21.0, 21.0], [-0.0, 28.0, 21.0, 49.0], [-42.0, 42.0, -0.0, 21.0]], [[-21.0, -49.0, 35.0, -35.0], [-28.0, 14.0, -35.0, -0.0], [42.0, 42.0, -7.0, -49.0]]], [[[-28.0, -7.0, 21.0, 28.0], [-14.0, 28.0, 42.0, -21.0], [21.0, -7.0, -21.0, 49.0]], [[-28.0, -21.0, -7.0, -14.0], [-7.0, -21.0, -0.0, 7.0], [42.0, -0.0, 21.0, 35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 0.0, 1.0, 0.0], [2.0, -5.0, 4.0, 7.0], [7.0, 2.0, 1.0, 0.0]], [[-3.0, 6.0, 0.0, 7.0], [0.0, 1.0, 0.0, 4.0], [1.0, 7.0, -2.0, 7.0]]], [[[0.0, -5.0, -3.0, 7.0], [-4.0, 6.0, -4.0, 0.0], [1.0, 7.0, -6.0, 0.0]], [[7.0, 2.0, -3.0, 1.0], [7.0, 2.0, -4.0, -1.0], [0.0, 1.0, -6.0, -3.0]]], [[[1.0, 1.0, 4.0, -3.0], [-4.0, 0.0, -3.0, -3.0], [-5.0, 3.0, 6.0, 0.0]], [[7.0, -4.0, 3.0, -2.0], [-4.0, -5.0, -3.0, 4.0], [-4.0, -7.0, -4.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, -2.0, 0.0, -2.0], [-4.0, -6.0, -1.0, -7.0], [3.0, 2.0, 7.0, 1.0]], [[2.0, -6.0, -1.0, 7.0], [0.0, 2.0, -1.0, 4.0], [-2.0, -7.0, -7.0, 6.0]]], [[[-6.0, -1.0, -4.0, 7.0], [1.0, 7.0, -5.0, -2.0], [3.0, 7.0, 3.0, 2.0]], [[-4.0, 7.0, 6.0, 1.0], [3.0, -4.0, 6.0, -4.0], [5.0, -3.0, -1.0, -2.0]]], [[[2.0, 6.0, 7.0, 3.0], [4.0, -1.0, -1.0, 1.0], [4.0, 0.0, 7.0, -2.0]], [[0.0, 0.0, 4.0, -7.0], [1.0, 7.0, 6.0, 6.0], [5.0, 6.0, 1.0, -6.0]]]])
      arg1=Data(-2.0,self.functionspace)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, -0.0, -2.0, -0.0], [-4.0, 10.0, -8.0, -14.0], [-14.0, -4.0, -2.0, -0.0]], [[6.0, -12.0, -0.0, -14.0], [-0.0, -2.0, -0.0, -8.0], [-2.0, -14.0, 4.0, -14.0]]], [[[-0.0, 10.0, 6.0, -14.0], [8.0, -12.0, 8.0, -0.0], [-2.0, -14.0, 12.0, -0.0]], [[-14.0, -4.0, 6.0, -2.0], [-14.0, -4.0, 8.0, 2.0], [-0.0, -2.0, 12.0, 6.0]]], [[[-2.0, -2.0, -8.0, 6.0], [8.0, -0.0, 6.0, 6.0], [10.0, -6.0, -12.0, -0.0]], [[-14.0, 8.0, -6.0, 4.0], [8.0, 10.0, 6.0, -8.0], [8.0, 14.0, 8.0, 10.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, 4.0, -0.0, 4.0], [8.0, 12.0, 2.0, 14.0], [-6.0, -4.0, -14.0, -2.0]], [[-4.0, 12.0, 2.0, -14.0], [-0.0, -4.0, 2.0, -8.0], [4.0, 14.0, 14.0, -12.0]]], [[[12.0, 2.0, 8.0, -14.0], [-2.0, -14.0, 10.0, 4.0], [-6.0, -14.0, -6.0, -4.0]], [[8.0, -14.0, -12.0, -2.0], [-6.0, 8.0, -12.0, 8.0], [-10.0, 6.0, 2.0, 4.0]]], [[[-4.0, -12.0, -14.0, -6.0], [-8.0, 2.0, 2.0, -2.0], [-8.0, -0.0, -14.0, 4.0]], [[-0.0, -0.0, -8.0, 14.0], [-2.0, -14.0, -12.0, -12.0], [-10.0, -12.0, -2.0, 12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTransposedTensorProduct_expandedData_rank4_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -7.0, -1.0, -1.0], [5.0, 5.0, -4.0, 7.0], [6.0, 0.0, -1.0, 6.0]], [[-5.0, -2.0, -7.0, -5.0], [6.0, 5.0, 1.0, -6.0], [-5.0, 0.0, -4.0, -4.0]]], [[[-2.0, -6.0, -7.0, 2.0], [-5.0, -3.0, 0.0, 4.0], [6.0, 5.0, 3.0, 6.0]], [[1.0, 0.0, 0.0, -2.0], [6.0, 6.0, -3.0, 7.0], [-3.0, 3.0, 4.0, -7.0]]], [[[3.0, 5.0, -1.0, -6.0], [-3.0, 6.0, 5.0, -4.0], [-6.0, -6.0, -1.0, 5.0]], [[-1.0, 2.0, 0.0, -1.0], [2.0, 0.0, -6.0, -3.0], [-5.0, -7.0, -7.0, -3.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -6.0, -5.0, -6.0], [-7.0, -7.0, 1.0, 5.0], [0.0, 6.0, 1.0, 6.0]], [[3.0, 6.0, 7.0, 6.0], [7.0, 0.0, -3.0, -5.0], [4.0, -3.0, 5.0, -4.0]]], [[[-7.0, 4.0, 0.0, 1.0], [-6.0, 1.0, 6.0, 4.0], [4.0, -1.0, -7.0, 5.0]], [[-5.0, -3.0, 0.0, 1.0], [4.0, 0.0, 0.0, 4.0], [-3.0, 2.0, 2.0, 0.0]]], [[[-1.0, 2.0, -3.0, 0.0], [-4.0, -4.0, 4.0, -3.0], [0.0, 0.0, -7.0, 7.0]], [[-6.0, 3.0, 6.0, 0.0], [3.0, -4.0, 3.0, 1.0], [-1.0, -3.0, 0.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(7.0)
      res=generalTransposedTensorProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, 21.0, 3.0, 3.0], [-15.0, -15.0, 12.0, -21.0], [-18.0, -0.0, 3.0, -18.0]], [[15.0, 6.0, 21.0, 15.0], [-18.0, -15.0, -3.0, 18.0], [15.0, -0.0, 12.0, 12.0]]], [[[6.0, 18.0, 21.0, -6.0], [15.0, 9.0, -0.0, -12.0], [-18.0, -15.0, -9.0, -18.0]], [[-3.0, -0.0, -0.0, 6.0], [-18.0, -18.0, 9.0, -21.0], [9.0, -9.0, -12.0, 21.0]]], [[[-9.0, -15.0, 3.0, 18.0], [9.0, -18.0, -15.0, 12.0], [18.0, 18.0, 3.0, -15.0]], [[3.0, -6.0, -0.0, 3.0], [-6.0, -0.0, 18.0, 9.0], [15.0, 21.0, 21.0, 9.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, -42.0, -35.0, -42.0], [-49.0, -49.0, 7.0, 35.0], [0.0, 42.0, 7.0, 42.0]], [[21.0, 42.0, 49.0, 42.0], [49.0, 0.0, -21.0, -35.0], [28.0, -21.0, 35.0, -28.0]]], [[[-49.0, 28.0, 0.0, 7.0], [-42.0, 7.0, 42.0, 28.0], [28.0, -7.0, -49.0, 35.0]], [[-35.0, -21.0, 0.0, 7.0], [28.0, 0.0, 0.0, 28.0], [-21.0, 14.0, 14.0, 0.0]]], [[[-7.0, 14.0, -21.0, 0.0], [-28.0, -28.0, 28.0, -21.0], [0.0, 0.0, -49.0, 49.0]], [[-42.0, 21.0, 42.0, 0.0], [21.0, -28.0, 21.0, 7.0], [-7.0, -21.0, 0.0, -35.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[5.0, -3.0, -5.0], [3.0, 1.0, 2.0]])
      arg1=numpy.array([[-5.0, -1.0, 6.0], [7.0, -5.0, 0.0]])
      res=matrix_transposed_mult(arg0,arg1)
      ref=numpy.array([[-52.0, 50.0], [-4.0, 16.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-1.0, -1.0, 6.0], [6.0, 2.0, 5.0]])
      arg1=Symbol(shape=(2, 3))
      res=matrix_transposed_mult(arg0,arg1)
      s1=numpy.array([[-5.0, -5.0, -4.0], [2.0, 6.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-14.0, -44.0], [-60.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[4.0, 1.0, -7.0], [0.0, 2.0, 6.0]])
      arg1=Data(numpy.array([[3.0, 1.0, -3.0], [-7.0, 4.0, -3.0]]),self.functionspace)
      res=matrix_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[34.0, -3.0], [-16.0, -10.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[-4.0, 5.0, -4.0], [-1.0, 2.0, -6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, 7.0, 3.0], [-7.0, 6.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 6.0, -2.0], [0.0, -1.0, -4.0]])
      res=matrix_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[35.0, 58.0], [-1.0, 19.0]])+(1.-msk_ref)*numpy.array([[46.0, 11.0], [26.0, 22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[0.0, 4.0, -4.0], [0.0, 6.0, 6.0]])
      res=matrix_transposed_mult(arg0,arg1)
      s0=numpy.array([[3.0, 6.0, 4.0], [-3.0, -5.0, 4.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[8.0, 60.0], [-36.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=matrix_transposed_mult(arg0,arg1)
      s0=numpy.array([[-6.0, -3.0, -4.0], [-5.0, 3.0, 0.0]])
      s1=numpy.array([[-4.0, -2.0, -1.0], [-2.0, 6.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[34.0, 22.0], [14.0, 28.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[-1.0, 0.0, 4.0], [-1.0, 0.0, -4.0]]),self.functionspace)
      res=matrix_transposed_mult(arg0,arg1)
      s0=numpy.array([[5.0, 1.0, 4.0], [-2.0, -4.0, -6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[11.0, -21.0], [-22.0, 26.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 1.0, 0.0], [6.0, 2.0, -1.0]])+(1.-msk_arg1)*numpy.array([[0.0, -4.0, 1.0], [6.0, 0.0, 5.0]])
      res=matrix_transposed_mult(arg0,arg1)
      s0=numpy.array([[1.0, -4.0, -1.0], [-4.0, 6.0, 3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-2.0, -1.0], [-2.0, -15.0]])+(1.-msk_ref)*numpy.array([[15.0, 1.0], [-21.0, -9.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[3.0, 1.0, -6.0], [5.0, -6.0, -3.0]]),self.functionspace)
      arg1=numpy.array([[0.0, 3.0, 6.0], [-2.0, 6.0, 0.0]])
      res=matrix_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[-33.0, 0.0], [-36.0, -46.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[6.0, -5.0, 7.0], [1.0, -2.0, 4.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=matrix_transposed_mult(arg0,arg1)
      s1=numpy.array([[-1.0, -2.0, -5.0], [3.0, -1.0, 4.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-31.0, 51.0], [-17.0, 21.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[-5.0, 4.0, -2.0], [-7.0, -4.0, 4.0]]),self.functionspace)
      arg1=Data(numpy.array([[-6.0, 2.0, 0.0], [-4.0, 1.0, 0.0]]),self.functionspace)
      res=matrix_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[38.0, 24.0], [34.0, 24.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[0.0, -1.0, 1.0], [-5.0, 2.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 4.0, 1.0], [-1.0, -2.0, 4.0]])+(1.-msk_arg1)*numpy.array([[3.0, 6.0, 0.0], [1.0, 0.0, -2.0]])
      res=matrix_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-3.0, 6.0], [-12.0, 1.0]])+(1.-msk_ref)*numpy.array([[-6.0, -2.0], [-3.0, -5.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, -3.0, -1.0], [5.0, 7.0, -2.0]])+(1.-msk_arg0)*numpy.array([[1.0, -6.0, 1.0], [4.0, -4.0, 2.0]])
      arg1=numpy.array([[4.0, 7.0, 5.0], [0.0, -3.0, -5.0]])
      res=matrix_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-34.0, 14.0], [59.0, -11.0]])+(1.-msk_ref)*numpy.array([[-33.0, 13.0], [-2.0, 2.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, -7.0, -3.0], [4.0, 1.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-7.0, 6.0, 5.0], [-6.0, 6.0, -7.0]])
      arg1=Symbol(shape=(2, 3))
      res=matrix_transposed_mult(arg0,arg1)
      s1=numpy.array([[2.0, -1.0, -2.0], [7.0, -4.0, -2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[13.0, 34.0], [19.0, 36.0]])+(1.-msk_ref)*numpy.array([[-30.0, -83.0], [-4.0, -52.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 0.0, -4.0], [-2.0, 4.0, -5.0]])+(1.-msk_arg0)*numpy.array([[3.0, 7.0, 0.0], [-4.0, -3.0, 1.0]])
      arg1=Data(numpy.array([[5.0, -7.0, 6.0], [3.0, 4.0, -1.0]]),self.functionspace)
      res=matrix_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-39.0, -5.0], [-68.0, 15.0]])+(1.-msk_ref)*numpy.array([[-34.0, 37.0], [7.0, -25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_matrix_transposed_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, -5.0, -6.0], [0.0, 6.0, -3.0]])+(1.-msk_arg0)*numpy.array([[3.0, -3.0, 4.0], [0.0, 1.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, -3.0, 6.0], [2.0, 2.0, 2.0]])+(1.-msk_arg1)*numpy.array([[2.0, -7.0, 4.0], [-1.0, 4.0, 0.0]])
      res=matrix_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-56.0, -32.0], [-36.0, 6.0]])+(1.-msk_ref)*numpy.array([[43.0, -15.0], [9.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_array_rank1(self):
      arg0=numpy.array([[-5.0, -7.0, -5.0], [0.0, -6.0, 3.0]])
      arg1=numpy.array([-3.0, 0.0, 2.0])
      res=tensor_transposed_mult(arg0,arg1)
      ref=numpy.array([5.0, 6.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_array_rank2(self):
      arg0=numpy.array([[1.0, -6.0, -3.0], [-5.0, 0.0, -1.0]])
      arg1=numpy.array([[7.0, 3.0, -7.0], [-7.0, -6.0, 3.0]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=numpy.array([[10.0, 20.0], [-28.0, 32.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_Symbol_rank1(self):
      arg0=numpy.array([[-4.0, -7.0, -1.0], [-6.0, 0.0, 5.0]])
      arg1=Symbol(shape=(3,))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([-2.0, -6.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([50.0, 12.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_Symbol_rank2(self):
      arg0=numpy.array([[-3.0, -6.0, 1.0], [-3.0, 3.0, -6.0]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[7.0, 1.0, 5.0], [-4.0, -5.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-22.0, 42.0], [-48.0, -3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_constData_rank1(self):
      arg0=numpy.array([[-7.0, 5.0, -6.0], [0.0, -6.0, -7.0]])
      arg1=Data(numpy.array([-3.0, -6.0, -6.0]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([27.0, 78.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_constData_rank2(self):
      arg0=numpy.array([[-4.0, 5.0, 7.0], [3.0, 1.0, -6.0]])
      arg1=Data(numpy.array([[0.0, 1.0, -1.0], [7.0, 0.0, -3.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[-2.0, -49.0], [7.0, 39.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_expandedData_rank1(self):
      arg0=numpy.array([[-5.0, -1.0, -6.0], [-5.0, -2.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 6.0, -1.0])+(1.-msk_arg1)*numpy.array([4.0, 2.0, -6.0])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([25.0, 11.0])+(1.-msk_ref)*numpy.array([14.0, -36.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank2_expandedData_rank2(self):
      arg0=numpy.array([[0.0, 4.0, 6.0], [-4.0, -5.0, -5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -7.0, 1.0], [3.0, 5.0, -6.0]])+(1.-msk_arg1)*numpy.array([[6.0, 7.0, -3.0], [7.0, 0.0, -6.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-22.0, -16.0], [26.0, -7.0]])+(1.-msk_ref)*numpy.array([[10.0, -36.0], [-44.0, 2.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_array_rank2(self):
      arg0=numpy.array([[[[-3.0, -3.0, -7.0], [2.0, 0.0, 6.0]], [[7.0, 2.0, -4.0], [7.0, 6.0, -1.0]], [[-3.0, 3.0, 6.0], [7.0, 1.0, 1.0]], [[-4.0, -6.0, 1.0], [-5.0, -7.0, -2.0]], [[3.0, 6.0, -3.0], [4.0, -2.0, 0.0]]], [[[-4.0, 4.0, 4.0], [-6.0, -4.0, 0.0]], [[-6.0, 6.0, -5.0], [-6.0, 7.0, -3.0]], [[-1.0, -2.0, 7.0], [6.0, 0.0, 0.0]], [[4.0, -3.0, 1.0], [-2.0, 0.0, 0.0]], [[6.0, -3.0, -6.0], [2.0, 6.0, -6.0]]], [[[-3.0, 0.0, -7.0], [0.0, -7.0, -6.0]], [[-1.0, -7.0, -7.0], [5.0, 6.0, -1.0]], [[-7.0, 0.0, 2.0], [-1.0, 4.0, -2.0]], [[2.0, -7.0, 2.0], [-7.0, -6.0, 6.0]], [[-7.0, -4.0, 6.0], [3.0, -6.0, 3.0]]], [[[-5.0, -7.0, 0.0], [4.0, -4.0, 4.0]], [[0.0, 7.0, 7.0], [-6.0, -3.0, 0.0]], [[4.0, -4.0, -5.0], [4.0, -7.0, 2.0]], [[-5.0, 2.0, 4.0], [-4.0, 2.0, -6.0]], [[0.0, 2.0, 7.0], [4.0, 0.0, -4.0]]]])
      arg1=numpy.array([[1.0, 1.0, 6.0], [-4.0, 5.0, 5.0]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=numpy.array([[-26.0, -18.0, 18.0, -29.0, -35.0], [28.0, 14.0, 15.0, 15.0, -41.0], [-110.0, -45.0, 19.0, 35.0, -2.0], [-28.0, 58.0, -71.0, 17.0, 8.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_array_rank3(self):
      arg0=numpy.array([[[[3.0, 0.0, 7.0], [-3.0, -5.0, -3.0]], [[6.0, 5.0, 2.0], [2.0, 7.0, 2.0]], [[-6.0, 4.0, 1.0], [-5.0, 1.0, 6.0]], [[-5.0, -5.0, -3.0], [2.0, 1.0, 5.0]], [[1.0, 4.0, 0.0], [5.0, -1.0, 5.0]]], [[[-3.0, -5.0, -3.0], [-5.0, 3.0, -2.0]], [[-7.0, -1.0, 6.0], [7.0, -1.0, 3.0]], [[-6.0, 5.0, 6.0], [-7.0, 6.0, -4.0]], [[-1.0, 0.0, 0.0], [0.0, 4.0, -2.0]], [[-2.0, -1.0, 6.0], [-7.0, -5.0, 7.0]]], [[[7.0, 0.0, -7.0], [0.0, -4.0, -2.0]], [[4.0, 5.0, -5.0], [-1.0, -3.0, -5.0]], [[5.0, 6.0, 0.0], [7.0, 0.0, 2.0]], [[5.0, 5.0, -4.0], [4.0, 0.0, 2.0]], [[1.0, -1.0, 2.0], [-1.0, 0.0, 4.0]]], [[[0.0, -3.0, 4.0], [-5.0, 3.0, -5.0]], [[5.0, 1.0, -3.0], [-7.0, 1.0, 3.0]], [[4.0, 1.0, -1.0], [-3.0, -6.0, 7.0]], [[-7.0, -1.0, -5.0], [6.0, 0.0, 7.0]], [[-7.0, 5.0, 2.0], [0.0, 0.0, 2.0]]]])
      arg1=numpy.array([[[-7.0, 1.0, -5.0], [-2.0, 7.0, 6.0]], [[5.0, 3.0, 4.0], [-1.0, 4.0, 4.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=numpy.array([[[-103.0, 14.0], [10.0, 87.0], [94.0, 19.0], [78.0, -30.0], [10.0, 28.0]], [[50.0, -33.0], [15.0, -13.0], [49.0, 24.0], [23.0, 3.0], [4.0, 26.0]], [[-54.0, -17.0], [-47.0, -16.0], [-31.0, 44.0], [-6.0, 28.0], [8.0, 27.0]], [[-22.0, 4.0], [20.0, 39.0], [-16.0, 26.0], [103.0, -36.0], [56.0, -4.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_array_rank4(self):
      arg0=numpy.array([[[[2.0, -6.0, -5.0], [-1.0, 4.0, 3.0]], [[3.0, 1.0, 4.0], [0.0, -5.0, -3.0]], [[-1.0, 2.0, 3.0], [-1.0, 7.0, 5.0]], [[7.0, -7.0, -3.0], [0.0, 4.0, 6.0]], [[5.0, -5.0, -1.0], [1.0, -5.0, -7.0]]], [[[0.0, 5.0, 4.0], [-2.0, 6.0, 3.0]], [[2.0, 6.0, 3.0], [0.0, 0.0, 6.0]], [[0.0, 5.0, 0.0], [-7.0, 0.0, 0.0]], [[0.0, -6.0, 0.0], [0.0, -4.0, -7.0]], [[-4.0, 6.0, 7.0], [-4.0, -7.0, 5.0]]], [[[0.0, 0.0, -7.0], [5.0, 0.0, 0.0]], [[-5.0, 0.0, -3.0], [5.0, 4.0, -3.0]], [[2.0, -4.0, 7.0], [2.0, -1.0, 5.0]], [[-7.0, -1.0, -6.0], [7.0, -7.0, 7.0]], [[-6.0, 6.0, 5.0], [-5.0, 5.0, -6.0]]], [[[-6.0, -3.0, 6.0], [3.0, 1.0, 1.0]], [[0.0, 5.0, -2.0], [-5.0, -1.0, -2.0]], [[4.0, -1.0, -3.0], [7.0, -7.0, 5.0]], [[-6.0, 6.0, -4.0], [3.0, 0.0, -5.0]], [[-1.0, 7.0, 1.0], [-1.0, -4.0, -4.0]]]])
      arg1=numpy.array([[[[0.0, -2.0, 5.0], [0.0, 7.0, -3.0]], [[-4.0, 4.0, 4.0], [3.0, 0.0, 3.0]], [[5.0, 7.0, 7.0], [3.0, -1.0, -7.0]], [[4.0, -5.0, 3.0], [-5.0, -6.0, 2.0]], [[0.0, 1.0, -4.0], [-3.0, 7.0, 4.0]]], [[[7.0, -1.0, 3.0], [-1.0, -2.0, -4.0]], [[7.0, -7.0, -4.0], [-4.0, -4.0, 1.0]], [[7.0, -5.0, 7.0], [6.0, 4.0, 0.0]], [[-5.0, 4.0, -4.0], [-5.0, -1.0, 6.0]], [[0.0, -7.0, 1.0], [2.0, -5.0, 6.0]]], [[[-4.0, 0.0, 1.0], [6.0, -2.0, 5.0]], [[-5.0, -7.0, 7.0], [2.0, -7.0, -5.0]], [[5.0, 2.0, 0.0], [1.0, -2.0, 0.0]], [[7.0, 0.0, 2.0], [4.0, 0.0, 4.0]], [[-3.0, 5.0, -4.0], [0.0, 4.0, -2.0]]], [[[1.0, 0.0, 3.0], [5.0, 3.0, 1.0]], [[0.0, 1.0, 2.0], [6.0, 2.0, -2.0]], [[0.0, -5.0, -6.0], [3.0, -5.0, -5.0]], [[6.0, 0.0, -2.0], [-2.0, 4.0, 6.0]], [[3.0, 0.0, -3.0], [5.0, -4.0, 0.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=numpy.array([[[[6.0, -46.0, -95.0, 10.0, 57.0], [-14.0, 67.0, 19.0, 5.0, 33.0], [-12.0, -48.0, -11.0, 12.0, -6.0], [-3.0, -20.0, 22.0, 58.0, 0.0]], [[-8.0, -1.0, 76.0, 43.0, -62.0], [54.0, 15.0, 24.0, -40.0, 4.0], [-13.0, 56.0, 27.0, 17.0, -34.0], [-3.0, 5.0, 11.0, -28.0, 17.0]], [[45.0, 36.0, -15.0, -32.0, 62.0], [-33.0, -52.0, 26.0, 29.0, -18.0], [12.0, -64.0, -16.0, 15.0, 19.0], [29.0, 6.0, -91.0, 48.0, -45.0]], [[9.0, -50.0, -81.0, 42.0, 57.0], [15.0, 100.0, 79.0, -19.0, 62.0], [-9.0, -65.0, 13.0, 67.0, -40.0], [16.0, -17.0, 3.0, 100.0, 14.0]], [[-9.0, -62.0, 40.0, 53.0, -67.0], [74.0, 83.0, 39.0, -83.0, 19.0], [-40.0, 75.0, 26.0, 9.0, -42.0], [-15.0, 3.0, 94.0, -32.0, 43.0]]], [[[43.0, 39.0, 30.0, -33.0, 49.0], [-15.0, -64.0, 15.0, 26.0, -47.0], [-5.0, -68.0, -4.0, 12.0, 27.0], [23.0, 7.0, -100.0, 38.0, -46.0]], [[-15.0, 46.0, 31.0, -1.0, 18.0], [-7.0, -34.0, 5.0, 38.0, -3.0], [25.0, -61.0, 22.0, 44.0, 0.0], [17.0, 0.0, -78.0, 42.0, -3.0]], [[-10.0, -1.0, 14.0, 10.0, 26.0], [2.0, -7.0, -67.0, 55.0, -49.0], [-42.0, -49.0, 3.0, -28.0, 25.0], [-35.0, -37.0, -46.0, 14.0, -35.0]], [[5.0, -45.0, 11.0, 40.0, -62.0], [42.0, 51.0, 14.0, -62.0, 20.0], [-27.0, 105.0, -4.0, -28.0, -32.0], [-19.0, 0.0, 85.0, -58.0, 16.0]], [[-41.0, 71.0, 31.0, 47.0, -39.0], [-15.0, -49.0, -61.0, 73.0, 22.0], [38.0, 43.0, 2.0, -10.0, -24.0], [-19.0, -28.0, -74.0, -28.0, -25.0]]], [[[-35.0, -13.0, -34.0, -46.0, 13.0], [-26.0, 8.0, -19.0, 3.0, 3.0], [23.0, -39.0, 5.0, 6.0, 28.0], [4.0, 16.0, 57.0, 4.0, 46.0]], [[22.0, 14.0, -14.0, -84.0, 13.0], [-45.0, -62.0, -10.0, -10.0, -31.0], [24.0, 1.0, -28.0, -33.0, 49.0], [20.0, 38.0, 28.0, -36.0, 3.0]], [[21.0, 25.0, 3.0, 55.0, -25.0], [19.0, 15.0, 91.0, -33.0, 74.0], [38.0, 53.0, 6.0, 56.0, -68.0], [35.0, 10.0, -36.0, 20.0, -1.0]], [[-98.0, 42.0, -105.0, -20.0, -19.0], [-87.0, -11.0, -72.0, 69.0, 92.0], [113.0, 28.0, -16.0, -5.0, -2.0], [-4.0, 1.0, 62.0, -30.0, 60.0]], [[66.0, 35.0, 69.0, -56.0, 12.0], [-14.0, -110.0, -47.0, 18.0, -108.0], [-41.0, 8.0, -33.0, -76.0, 60.0], [-7.0, 8.0, -70.0, -52.0, -78.0]]], [[[40.0, 48.0, -8.0, -10.0, -25.0], [-30.0, -60.0, 37.0, -16.0, 34.0], [51.0, 87.0, -35.0, -14.0, -19.0], [31.0, 27.0, -22.0, -44.0, -25.0]], [[-21.0, -9.0, 21.0, -4.0, 13.0], [4.0, -5.0, -73.0, 42.0, -54.0], [-40.0, -42.0, 7.0, -32.0, 33.0], [-36.0, -27.0, -13.0, -2.0, -15.0]], [[-77.0, 4.0, -15.0, 29.0, -39.0], [7.0, 52.0, 26.0, -10.0, 83.0], [62.0, 4.0, 39.0, 70.0, -43.0], [14.0, 11.0, 54.0, 18.0, 84.0]], [[-17.0, 26.0, 28.0, -91.0, -7.0], [-43.0, -85.0, -82.0, 25.0, -70.0], [13.0, -9.0, -15.0, -58.0, 74.0], [-8.0, 26.0, 28.0, -64.0, 9.0]], [[-25.0, 21.0, 80.0, -15.0, -38.0], [14.0, -44.0, -57.0, 14.0, -54.0], [-13.0, 9.0, 16.0, -25.0, 26.0], [-19.0, 3.0, -4.0, -46.0, 5.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_Symbol_rank2(self):
      arg0=numpy.array([[[[-6.0, 2.0, 5.0], [6.0, 6.0, -3.0]], [[0.0, -2.0, -3.0], [-5.0, 5.0, -1.0]], [[-3.0, -7.0, -7.0], [0.0, -5.0, 2.0]], [[5.0, 1.0, 2.0], [4.0, -6.0, 0.0]], [[-6.0, -1.0, -1.0], [5.0, -6.0, -7.0]]], [[[0.0, 6.0, 3.0], [4.0, 7.0, 7.0]], [[4.0, 3.0, 7.0], [7.0, -7.0, 7.0]], [[4.0, 6.0, 2.0], [0.0, -6.0, 2.0]], [[6.0, -2.0, 4.0], [-7.0, 2.0, -6.0]], [[-2.0, -5.0, -1.0], [5.0, 2.0, 5.0]]], [[[7.0, -6.0, 6.0], [1.0, 6.0, -3.0]], [[1.0, 0.0, -4.0], [1.0, 7.0, 4.0]], [[3.0, 4.0, 7.0], [5.0, -2.0, -2.0]], [[4.0, 4.0, -4.0], [0.0, -1.0, -3.0]], [[0.0, 4.0, 0.0], [0.0, -7.0, -3.0]]], [[[-1.0, 5.0, 5.0], [4.0, 0.0, 2.0]], [[1.0, -7.0, 7.0], [-6.0, 5.0, 1.0]], [[5.0, -1.0, 5.0], [0.0, -4.0, 2.0]], [[-7.0, 1.0, -6.0], [2.0, -2.0, 5.0]], [[-2.0, 3.0, 0.0], [4.0, -2.0, 0.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[-4.0, 4.0, 3.0], [-5.0, 7.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[71.0, 47.0, -80.0, -72.0, -22.0], [34.0, -95.0, -36.0, 53.0, -46.0], [15.0, 12.0, -6.0, -7.0, -21.0], [11.0, 50.0, -45.0, -30.0, -14.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_Symbol_rank3(self):
      arg0=numpy.array([[[[6.0, 6.0, -1.0], [7.0, -1.0, 1.0]], [[4.0, 0.0, 4.0], [-5.0, -5.0, 4.0]], [[5.0, -3.0, 7.0], [3.0, 2.0, -3.0]], [[-2.0, 1.0, -5.0], [-4.0, -5.0, 6.0]], [[7.0, -2.0, 6.0], [1.0, -7.0, 5.0]]], [[[6.0, 1.0, 1.0], [3.0, 2.0, -7.0]], [[2.0, 0.0, 1.0], [-1.0, -5.0, 3.0]], [[1.0, 1.0, -6.0], [0.0, -4.0, -4.0]], [[-6.0, -2.0, 0.0], [-2.0, -6.0, 0.0]], [[4.0, -2.0, -3.0], [-1.0, -6.0, 0.0]]], [[[-6.0, 1.0, -5.0], [-3.0, 0.0, 6.0]], [[-2.0, 3.0, 4.0], [-1.0, 7.0, -5.0]], [[-6.0, -4.0, 3.0], [-7.0, 1.0, 6.0]], [[-5.0, -3.0, 7.0], [1.0, 3.0, -5.0]], [[7.0, -2.0, -3.0], [2.0, -2.0, 2.0]]], [[[5.0, 3.0, 5.0], [-1.0, -7.0, 0.0]], [[-5.0, -1.0, 3.0], [0.0, 1.0, 4.0]], [[4.0, -3.0, 3.0], [3.0, -2.0, -2.0]], [[7.0, 2.0, -6.0], [5.0, 1.0, 2.0]], [[-2.0, -7.0, -5.0], [-4.0, 7.0, 4.0]]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[2.0, -2.0, -2.0], [-1.0, -3.0, 5.0]], [[1.0, 4.0, -7.0], [-6.0, 1.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[3.0, -4.0], [40.0, 9.0], [-22.0, -78.0], [53.0, 68.0], [51.0, -46.0]], [[-36.0, -27.0], [33.0, 2.0], [4.0, 35.0], [12.0, -8.0], [37.0, 17.0]], [[29.0, 63.0], [-63.0, -15.0], [24.0, 12.0], [-53.0, -79.0], [38.0, 10.0]], [[16.0, -19.0], [3.0, -21.0], [1.0, -53.0], [24.0, 32.0], [23.0, 44.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_Symbol_rank4(self):
      arg0=numpy.array([[[[-4.0, -6.0, -7.0], [0.0, 3.0, 7.0]], [[-4.0, 3.0, 0.0], [-3.0, -2.0, -4.0]], [[-3.0, -3.0, 3.0], [4.0, 1.0, 3.0]], [[2.0, 6.0, 5.0], [1.0, 6.0, -5.0]], [[-2.0, 3.0, -5.0], [6.0, -7.0, 7.0]]], [[[-1.0, -4.0, 2.0], [-5.0, -5.0, 3.0]], [[-3.0, 3.0, 5.0], [-3.0, 0.0, -3.0]], [[5.0, -2.0, -2.0], [-7.0, 0.0, 3.0]], [[-1.0, -1.0, -7.0], [-2.0, -2.0, -7.0]], [[0.0, -4.0, 4.0], [5.0, -4.0, -7.0]]], [[[1.0, 3.0, 2.0], [-3.0, -4.0, 5.0]], [[4.0, -6.0, -2.0], [1.0, 5.0, -2.0]], [[5.0, 4.0, -4.0], [0.0, -6.0, 0.0]], [[-6.0, 6.0, 2.0], [1.0, -1.0, 3.0]], [[-2.0, -4.0, 2.0], [6.0, 0.0, 0.0]]], [[[-4.0, 5.0, -6.0], [-5.0, 7.0, 6.0]], [[2.0, 3.0, 5.0], [1.0, 2.0, -5.0]], [[-5.0, 2.0, 2.0], [-1.0, -2.0, 5.0]], [[5.0, -2.0, 3.0], [-7.0, -2.0, -3.0]], [[-2.0, 0.0, 2.0], [6.0, 4.0, 4.0]]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[[-1.0, 2.0, 6.0], [4.0, -2.0, 0.0]], [[0.0, -5.0, -7.0], [-2.0, 0.0, -6.0]], [[-2.0, 1.0, 1.0], [-5.0, -3.0, -3.0]], [[-3.0, 0.0, 0.0], [1.0, 7.0, 4.0]], [[-7.0, 3.0, 6.0], [-6.0, 0.0, -1.0]]], [[[-7.0, -3.0, -1.0], [0.0, 6.0, 0.0]], [[-7.0, 5.0, -7.0], [-3.0, -5.0, 5.0]], [[0.0, -2.0, -7.0], [6.0, -7.0, 5.0]], [[7.0, 2.0, -1.0], [2.0, 3.0, 0.0]], [[-3.0, -7.0, 7.0], [5.0, 4.0, -5.0]]], [[[1.0, -1.0, 6.0], [0.0, -1.0, 4.0]], [[-2.0, 6.0, 7.0], [3.0, 7.0, -6.0]], [[1.0, 2.0, 3.0], [-7.0, 0.0, -6.0]], [[1.0, -5.0, -2.0], [-5.0, 3.0, 7.0]], [[-7.0, -5.0, -3.0], [-6.0, 7.0, 3.0]]], [[[2.0, -2.0, 6.0], [-7.0, 6.0, 6.0]], [[6.0, 4.0, 0.0], [-6.0, -6.0, -6.0]], [[0.0, -7.0, -2.0], [-2.0, 7.0, 4.0]], [[0.0, 2.0, -6.0], [6.0, -5.0, -4.0]], [[-3.0, -4.0, 0.0], [-3.0, 1.0, 7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-56.0, 37.0, -35.0, 61.0, -39.0], [71.0, 67.0, 75.0, -24.0, -18.0], [-15.0, -98.0, -79.0, 98.0, 121.0], [22.0, -108.0, 105.0, -13.0, 88.0]], [[2.0, 15.0, 44.0, -21.0, 59.0], [7.0, 42.0, -30.0, -34.0, -12.0], [-21.0, 27.0, 47.0, -38.0, 5.0], [-29.0, 42.0, -45.0, 14.0, -21.0]], [[29.0, -32.0, -26.0, 32.0, 3.0], [33.0, -17.0, 17.0, -19.0, 60.0], [29.0, 10.0, -46.0, 10.0, 19.0], [14.0, -78.0, 26.0, -17.0, 31.0]], [[32.0, -37.0, -1.0, 17.0, 33.0], [-1.0, -77.0, -108.0, 41.0, 41.0], [0.0, 142.0, 52.0, -60.0, -38.0], [21.0, 24.0, -32.0, -22.0, -62.0]], [[16.0, -34.0, -28.0, -9.0, -50.0], [-32.0, 116.0, 149.0, -12.0, -83.0], [0.0, -86.0, -95.0, -9.0, -50.0], [-82.0, -36.0, -44.0, 79.0, 18.0]]], [[[-5.0, -2.0, 31.0, -25.0, 34.0], [-13.0, 28.0, 14.0, -42.0, -15.0], [32.0, -76.0, 14.0, 46.0, 25.0], [41.0, 20.0, 11.0, -37.0, 50.0]], [[27.0, -26.0, 38.0, -6.0, 81.0], [7.0, -5.0, -74.0, -26.0, 23.0], [12.0, 68.0, 57.0, -34.0, 0.0], [21.0, 30.0, -37.0, -30.0, -15.0]], [[-49.0, 20.0, 12.0, -10.0, -14.0], [-27.0, 5.0, -9.0, 19.0, -65.0], [7.0, -75.0, 26.0, 75.0, 32.0], [69.0, 46.0, 44.0, -46.0, 35.0]], [[-47.0, 100.0, 31.0, -41.0, -19.0], [5.0, 32.0, 18.0, -12.0, -22.0], [-68.0, -31.0, 32.0, -27.0, 10.0], [-82.0, 56.0, -17.0, 66.0, -38.0]], [[44.0, 24.0, 8.0, -51.0, -11.0], [-16.0, -78.0, 3.0, -14.0, 100.0], [4.0, 33.0, 11.0, -74.0, -71.0], [-69.0, 20.0, -46.0, 46.0, -52.0]]], [[[13.0, -53.0, 15.0, -14.0, 27.0], [-42.0, 48.0, 15.0, -7.0, -66.0], [34.0, -37.0, 4.0, 20.0, -23.0], [35.0, 30.0, -27.0, -24.0, 25.0]], [[-34.0, 54.0, -30.0, 16.0, -62.0], [22.0, -82.0, -13.0, 35.0, 51.0], [-15.0, -8.0, -9.0, 34.0, 31.0], [19.0, -24.0, 71.0, -11.0, 0.0]], [[-9.0, 8.0, 8.0, -57.0, -47.0], [-79.0, 43.0, 62.0, 29.0, -95.0], [-17.0, -56.0, 1.0, -25.0, -85.0], [-58.0, 82.0, -62.0, 62.0, -37.0]], [[36.0, -64.0, 9.0, 24.0, 63.0], [16.0, 75.0, 2.0, -33.0, -24.0], [13.0, 40.0, -13.0, -27.0, 2.0], [-7.0, -30.0, -43.0, -1.0, 11.0]], [[30.0, -6.0, -28.0, 12.0, -22.0], [24.0, -38.0, 30.0, -12.0, 78.0], [14.0, 12.0, -46.0, -16.0, -8.0], [-26.0, -64.0, 12.0, 16.0, 4.0]]], [[[-56.0, -9.0, -7.0, 80.0, 31.0], [61.0, 105.0, -17.0, -1.0, -92.0], [-28.0, -6.0, -13.0, 71.0, 118.0], [59.0, -52.0, 60.0, -43.0, 56.0]], [[34.0, -22.0, 8.0, -11.0, 24.0], [-16.0, -72.0, -74.0, 23.0, 46.0], [7.0, 96.0, 46.0, -57.0, -51.0], [3.0, 36.0, -39.0, -8.0, -54.0]], [[21.0, -52.0, 10.0, 20.0, 54.0], [15.0, 69.0, 15.0, -41.0, -23.0], [27.0, -11.0, -18.0, 15.0, 26.0], [23.0, -34.0, -10.0, -24.0, 43.0]], [[-15.0, 21.0, 41.0, -48.0, 22.0], [-44.0, -50.0, -60.0, 8.0, -8.0], [15.0, -18.0, 77.0, 17.0, -15.0], [51.0, 94.0, -4.0, -42.0, -9.0]], [[30.0, -50.0, -48.0, 56.0, -14.0], [36.0, -18.0, 14.0, 8.0, 46.0], [22.0, 40.0, -62.0, 4.0, 12.0], [14.0, -96.0, 28.0, -12.0, 20.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_constData_rank2(self):
      arg0=numpy.array([[[[4.0, -4.0, 1.0], [-3.0, -5.0, 7.0]], [[-6.0, 7.0, 0.0], [7.0, -4.0, 6.0]], [[5.0, -7.0, 5.0], [6.0, -1.0, 6.0]], [[5.0, 0.0, 0.0], [-5.0, -2.0, 7.0]], [[-6.0, 6.0, -4.0], [-5.0, -7.0, -5.0]]], [[[1.0, 4.0, 6.0], [0.0, 0.0, 4.0]], [[-3.0, -2.0, 3.0], [-6.0, 2.0, 5.0]], [[5.0, -5.0, -2.0], [4.0, 6.0, 5.0]], [[2.0, 7.0, -3.0], [2.0, 4.0, 0.0]], [[4.0, 4.0, -3.0], [3.0, 0.0, 0.0]]], [[[3.0, 5.0, -3.0], [-2.0, -7.0, 1.0]], [[6.0, -7.0, 2.0], [-5.0, -1.0, -3.0]], [[3.0, -2.0, 1.0], [7.0, 4.0, 3.0]], [[-2.0, 0.0, -7.0], [5.0, 1.0, -1.0]], [[-6.0, -1.0, 1.0], [-6.0, -7.0, 4.0]]], [[[6.0, -5.0, 3.0], [-1.0, -6.0, 6.0]], [[-2.0, -1.0, -6.0], [0.0, 7.0, 1.0]], [[-7.0, 4.0, 2.0], [-3.0, 6.0, 3.0]], [[-6.0, -1.0, -1.0], [-3.0, 5.0, -3.0]], [[4.0, -6.0, 2.0], [5.0, 6.0, -7.0]]]])
      arg1=Data(numpy.array([[1.0, 5.0, 3.0], [-6.0, -6.0, 3.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[56.0, 29.0, -27.0, 68.0, 69.0], [51.0, 35.0, -71.0, -8.0, -3.0], [76.0, 4.0, -61.0, -62.0, 82.0], [50.0, -64.0, 10.0, -35.0, -107.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_constData_rank3(self):
      arg0=numpy.array([[[[3.0, -1.0, -2.0], [1.0, 0.0, 5.0]], [[7.0, -6.0, 4.0], [-3.0, -5.0, 4.0]], [[3.0, 0.0, 6.0], [7.0, -3.0, 2.0]], [[2.0, -5.0, 6.0], [2.0, 2.0, -4.0]], [[0.0, 6.0, 7.0], [3.0, 5.0, -5.0]]], [[[0.0, 6.0, -6.0], [3.0, 7.0, -3.0]], [[-4.0, 6.0, 5.0], [-3.0, 5.0, -6.0]], [[5.0, 5.0, 2.0], [7.0, -5.0, -2.0]], [[0.0, -5.0, -6.0], [-2.0, 0.0, -3.0]], [[1.0, -3.0, -5.0], [2.0, 5.0, 3.0]]], [[[5.0, 1.0, -1.0], [2.0, 1.0, 4.0]], [[-7.0, 5.0, 1.0], [0.0, 6.0, 1.0]], [[-2.0, 0.0, -2.0], [1.0, 2.0, -6.0]], [[1.0, 7.0, 5.0], [3.0, 1.0, 6.0]], [[-7.0, 6.0, -1.0], [6.0, 7.0, -3.0]]], [[[5.0, 0.0, -1.0], [-2.0, 1.0, 6.0]], [[1.0, 0.0, 0.0], [5.0, -3.0, 2.0]], [[7.0, -3.0, 4.0], [5.0, -2.0, 6.0]], [[5.0, 1.0, 7.0], [2.0, 2.0, 0.0]], [[2.0, 1.0, -1.0], [-2.0, 0.0, 0.0]]]])
      arg1=Data(numpy.array([[[1.0, -3.0, -2.0], [0.0, -6.0, 4.0]], [[4.0, 0.0, 7.0], [5.0, 0.0, -5.0]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[30.0, -22.0], [63.0, 21.0], [17.0, 79.0], [-23.0, 80.0], [-82.0, 89.0]], [[-60.0, -12.0], [-86.0, 34.0], [8.0, 79.0], [15.0, -37.0], [2.0, -36.0]], [[14.0, 3.0], [-56.0, -26.0], [-34.0, 13.0], [-12.0, 24.0], [-77.0, 10.0]], [[25.0, -27.0], [27.0, 19.0], [44.0, 51.0], [-24.0, 79.0], [1.0, -9.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_constData_rank4(self):
      arg0=numpy.array([[[[0.0, 4.0, -1.0], [6.0, 6.0, 6.0]], [[3.0, 2.0, -3.0], [4.0, 1.0, 6.0]], [[-1.0, -3.0, 5.0], [7.0, -5.0, 1.0]], [[4.0, 4.0, -7.0], [-3.0, 7.0, 4.0]], [[3.0, 6.0, -3.0], [0.0, -3.0, 1.0]]], [[[-3.0, -7.0, -5.0], [2.0, -7.0, 1.0]], [[-4.0, 0.0, 5.0], [-2.0, 0.0, 6.0]], [[-3.0, -4.0, 1.0], [4.0, 5.0, -6.0]], [[3.0, 4.0, 0.0], [-5.0, 0.0, -4.0]], [[4.0, 1.0, -4.0], [0.0, 1.0, 5.0]]], [[[7.0, -4.0, 3.0], [7.0, 5.0, -3.0]], [[-3.0, 1.0, 7.0], [0.0, 3.0, 7.0]], [[6.0, 7.0, 7.0], [5.0, -3.0, 7.0]], [[-2.0, -6.0, 1.0], [-6.0, 3.0, -1.0]], [[-7.0, 2.0, 6.0], [6.0, -6.0, 4.0]]], [[[0.0, 3.0, -7.0], [-3.0, 7.0, 7.0]], [[-5.0, -5.0, -1.0], [-4.0, -6.0, 0.0]], [[6.0, 1.0, 7.0], [-4.0, 5.0, 6.0]], [[3.0, -4.0, -2.0], [1.0, 7.0, 7.0]], [[0.0, 4.0, -7.0], [-2.0, 0.0, -7.0]]]])
      arg1=Data(numpy.array([[[[4.0, 0.0, -6.0], [-4.0, -6.0, 1.0]], [[3.0, -2.0, 1.0], [-3.0, 4.0, 3.0]], [[0.0, -1.0, -7.0], [-7.0, -6.0, 0.0]], [[-3.0, -2.0, 5.0], [5.0, 7.0, -6.0]], [[-4.0, -5.0, 0.0], [-4.0, -1.0, -4.0]]], [[[-4.0, 3.0, -2.0], [-7.0, 0.0, 0.0]], [[-7.0, 1.0, 4.0], [6.0, 7.0, 0.0]], [[-5.0, 6.0, 6.0], [-2.0, -4.0, 5.0]], [[-5.0, 3.0, 0.0], [-3.0, 1.0, 6.0]], [[-4.0, 0.0, 0.0], [-5.0, -3.0, 2.0]]], [[[-4.0, 1.0, 0.0], [4.0, 2.0, 4.0]], [[7.0, 0.0, -6.0], [4.0, -1.0, 3.0]], [[2.0, 3.0, -4.0], [-2.0, -2.0, 2.0]], [[0.0, 4.0, -3.0], [4.0, -2.0, 3.0]], [[6.0, 1.0, 2.0], [2.0, -3.0, -3.0]]], [[[2.0, -6.0, -5.0], [6.0, -4.0, -3.0]], [[1.0, 4.0, 0.0], [-3.0, 4.0, 7.0]], [[0.0, 7.0, 0.0], [-2.0, 2.0, -1.0]], [[-7.0, 0.0, -1.0], [-1.0, 1.0, 3.0]], [[4.0, -1.0, 6.0], [4.0, 2.0, 1.0]]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-48.0, 15.0, -75.0, 23.0, -74.0], [-28.0, 78.0, 12.0, 36.0, -36.0], [64.0, 42.0, 4.0, 49.0, -22.0], [-25.0, 64.0, 22.0, 19.0, 32.0]], [[14.0, 12.0, -15.0, -37.0, -63.0], [-28.0, 0.0, -3.0, 16.0, -23.0], [32.0, 72.0, 26.0, 49.0, 1.0], [11.0, 45.0, 2.0, -3.0, 16.0]], [[-31.0, -30.0, -51.0, 28.0, -8.0], [-64.0, 31.0, 28.0, -24.0, -14.0], [23.0, -1.0, -33.0, 14.0, 27.0], [50.0, -47.0, -46.0, -7.0, 48.0]], [[32.0, 46.0, 24.0, -45.0, -47.0], [31.0, -21.0, -40.0, 32.0, -14.0], [6.0, 63.0, 48.0, 23.0, -25.0], [-39.0, 85.0, 44.0, 1.0, -24.0]], [[49.0, -15.0, 33.0, -63.0, -43.0], [12.0, -48.0, 20.0, 6.0, -1.0], [-8.0, 45.0, 44.0, 42.0, 24.0], [-6.0, 22.0, 35.0, -18.0, -17.0]]], [[[53.0, -31.0, 70.0, -47.0, 42.0], [-13.0, -43.0, -28.0, -13.0, 25.0], [3.0, 27.0, 5.0, 12.0, -13.0], [98.0, -58.0, -68.0, 20.0, -40.0]], [[-32.0, 17.0, -21.0, -9.0, 0.0], [20.0, 36.0, 84.0, 62.0, 38.0], [32.0, -48.0, -12.0, -5.0, -36.0], [-63.0, 44.0, -2.0, 43.0, 12.0]], [[-70.0, -10.0, -61.0, 113.0, 35.0], [-30.0, 80.0, -61.0, -40.0, -35.0], [10.0, -34.0, -52.0, -31.0, -9.0], [35.0, -53.0, -20.0, 3.0, 18.0]], [[28.0, 4.0, 31.0, -18.0, 4.0], [35.0, -47.0, -1.0, -12.0, 5.0], [-44.0, -11.0, 20.0, -16.0, 24.0], [-36.0, 6.0, 42.0, -28.0, -16.0]], [[39.0, 25.0, 21.0, -57.0, -42.0], [-5.0, -36.0, -17.0, 14.0, -9.0], [7.0, 66.0, 35.0, 29.0, -1.0], [3.0, 47.0, 4.0, -8.0, -2.0]]], [[[-51.0, 22.0, -96.0, 90.0, -29.0], [-95.0, 36.0, -90.0, -81.0, -84.0], [-6.0, 45.0, -40.0, -16.0, 52.0], [54.0, -31.0, -29.0, -63.0, 85.0]], [[-65.0, 29.0, -68.0, 21.0, -24.0], [1.0, 71.0, 86.0, 63.0, 17.0], [47.0, -45.0, -23.0, -2.0, -33.0], [-80.0, 62.0, 6.0, 38.0, 42.0]], [[-13.0, 5.0, -73.0, -35.0, -104.0], [-52.0, 2.0, 91.0, 15.0, -26.0], [25.0, 44.0, 15.0, 54.0, 55.0], [-44.0, 56.0, 26.0, -36.0, 80.0]], [[-9.0, 34.0, 23.0, 20.0, 63.0], [30.0, -3.0, -25.0, 7.0, 27.0], [-20.0, -50.0, -22.0, -60.0, -34.0], [-18.0, -3.0, -23.0, 19.0, -15.0]], [[-48.0, -49.0, -50.0, 11.0, -16.0], [-20.0, 69.0, 115.0, 41.0, 24.0], [58.0, -43.0, -24.0, 38.0, -10.0], [-8.0, -13.0, -14.0, 43.0, 22.0]]], [[[19.0, 45.0, 25.0, -49.0, -38.0], [44.0, 6.0, -11.0, 67.0, 8.0], [33.0, 44.0, 43.0, 28.0, -59.0], [-50.0, 98.0, 34.0, 38.0, -36.0]], [[38.0, -18.0, 76.0, -42.0, 67.0], [35.0, -40.0, 21.0, 16.0, 58.0], [-13.0, -39.0, -1.0, -21.0, -27.0], [25.0, -37.0, -39.0, 34.0, -49.0]], [[-26.0, 73.0, -52.0, -6.0, -42.0], [-7.0, -2.0, 36.0, 26.0, -7.0], [-5.0, -3.0, -3.0, -25.0, 10.0], [-91.0, 84.0, 19.0, -22.0, 65.0]], [[-15.0, 61.0, -31.0, 1.0, -31.0], [-27.0, 22.0, -46.0, 19.0, -24.0], [30.0, 51.0, 0.0, 1.0, -30.0], [-3.0, 61.0, -23.0, 8.0, 29.0]], [[43.0, -30.0, 59.0, -11.0, 16.0], [40.0, -36.0, -49.0, -24.0, -4.0], [-32.0, 13.0, 30.0, 8.0, 7.0], [20.0, -27.0, 39.0, -12.0, -61.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_expandedData_rank2(self):
      arg0=numpy.array([[[[-7.0, -7.0, 0.0], [1.0, 7.0, -7.0]], [[-1.0, -5.0, 6.0], [-3.0, -1.0, 6.0]], [[4.0, 3.0, -3.0], [-4.0, 3.0, -2.0]], [[3.0, 4.0, -5.0], [5.0, -3.0, 0.0]], [[7.0, -3.0, -3.0], [3.0, -4.0, -7.0]]], [[[-6.0, 2.0, 4.0], [-6.0, 4.0, 0.0]], [[5.0, 2.0, 0.0], [1.0, 4.0, 7.0]], [[-5.0, -6.0, 2.0], [4.0, -7.0, 3.0]], [[0.0, -6.0, 7.0], [2.0, 7.0, -4.0]], [[-3.0, 3.0, 0.0], [-5.0, -1.0, -2.0]]], [[[4.0, 2.0, -7.0], [1.0, -5.0, 0.0]], [[4.0, 5.0, 7.0], [7.0, -5.0, -3.0]], [[5.0, 6.0, 6.0], [-6.0, 0.0, -7.0]], [[4.0, -3.0, -3.0], [2.0, 5.0, 1.0]], [[-2.0, 6.0, -2.0], [-6.0, -5.0, -3.0]]], [[[2.0, 2.0, 0.0], [0.0, 2.0, -1.0]], [[1.0, 2.0, -4.0], [-3.0, -7.0, -4.0]], [[-2.0, 2.0, 4.0], [-3.0, -1.0, 4.0]], [[4.0, -5.0, -2.0], [-4.0, -4.0, -6.0]], [[0.0, 0.0, -7.0], [-7.0, 6.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-3.0, -6.0, -6.0], [-5.0, 0.0, -5.0]])+(1.-msk_arg1)*numpy.array([[4.0, 5.0, -1.0], [3.0, 0.0, 2.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[93.0, -18.0, 18.0, -28.0, 35.0], [12.0, -67.0, 4.0, 4.0, 26.0], [13.0, -104.0, -22.0, 9.0, 27.0], [-13.0, 44.0, -35.0, 80.0, 92.0]])+(1.-msk_ref)*numpy.array([[-74.0, -32.0, 18.0, 52.0, 11.0], [-36.0, 47.0, -34.0, -39.0, -16.0], [36.0, 49.0, 12.0, 12.0, 0.0], [16.0, 1.0, -3.0, -31.0, -20.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_expandedData_rank3(self):
      arg0=numpy.array([[[[7.0, 3.0, -1.0], [6.0, -6.0, -4.0]], [[6.0, -7.0, 5.0], [0.0, 0.0, -7.0]], [[1.0, -5.0, 2.0], [-7.0, 4.0, 1.0]], [[-5.0, 2.0, 6.0], [-1.0, -7.0, 5.0]], [[-5.0, -5.0, -6.0], [2.0, -2.0, -6.0]]], [[[5.0, 5.0, -2.0], [-6.0, 1.0, -2.0]], [[2.0, -1.0, 0.0], [-4.0, 0.0, 6.0]], [[-7.0, -5.0, -3.0], [-5.0, -5.0, 2.0]], [[5.0, -2.0, 2.0], [0.0, 4.0, -4.0]], [[7.0, -7.0, 4.0], [-4.0, -2.0, 2.0]]], [[[4.0, -2.0, 4.0], [2.0, 0.0, -1.0]], [[3.0, 6.0, -4.0], [-4.0, 7.0, 3.0]], [[-2.0, 2.0, -3.0], [0.0, 2.0, -1.0]], [[0.0, 3.0, -1.0], [-6.0, -3.0, 4.0]], [[2.0, -6.0, -3.0], [-1.0, 1.0, 3.0]]], [[[-1.0, -1.0, -1.0], [6.0, 7.0, 4.0]], [[-2.0, 4.0, 0.0], [7.0, 1.0, 0.0]], [[-2.0, 0.0, -1.0], [-6.0, -6.0, -2.0]], [[0.0, 0.0, -3.0], [6.0, -5.0, -1.0]], [[-1.0, -2.0, 4.0], [6.0, 0.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-7.0, -4.0, -7.0], [-1.0, 4.0, -5.0]], [[-6.0, 5.0, 2.0], [-1.0, 0.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 0.0, 6.0], [1.0, -6.0, 0.0]], [[-6.0, -2.0, 3.0], [-1.0, 3.0, -5.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-64.0, -23.0], [-14.0, -40.0], [17.0, -23.0], [-67.0, 38.0], [117.0, 9.0]], [[-21.0, 3.0], [-36.0, -31.0], [65.0, 10.0], [-5.0, -24.0], [-63.0, -71.0]], [[-45.0, -25.0], [0.0, -1.0], [40.0, 19.0], [-31.0, 7.0], [21.0, -56.0]], [[20.0, -19.0], [-5.0, 25.0], [13.0, 22.0], [0.0, -9.0], [-9.0, 4.0]]])+(1.-msk_ref)*numpy.array([[[57.0, -55.0], [48.0, 28.0], [-16.0, 24.0], [62.0, -1.0], [-37.0, 44.0]], [[-9.0, -27.0], [2.0, -36.0], [-14.0, 23.0], [3.0, 12.0], [53.0, -28.0]], [[38.0, -5.0], [-61.0, -32.0], [-36.0, 10.0], [6.0, -32.0], [-19.0, -20.0]], [[-45.0, 0.0], [-5.0, 0.0], [18.0, 7.0], [18.0, -25.0], [27.0, 26.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_array_rank4_expandedData_rank4(self):
      arg0=numpy.array([[[[6.0, -1.0, 6.0], [-5.0, -6.0, 0.0]], [[6.0, -2.0, 0.0], [3.0, -6.0, -7.0]], [[3.0, -4.0, -4.0], [-6.0, -5.0, 6.0]], [[-4.0, -1.0, 3.0], [-4.0, 0.0, -7.0]], [[-6.0, 1.0, -6.0], [4.0, -1.0, -2.0]]], [[[1.0, 2.0, -7.0], [6.0, -4.0, -1.0]], [[-6.0, 0.0, 4.0], [0.0, -4.0, 3.0]], [[0.0, 3.0, 6.0], [5.0, -1.0, -3.0]], [[-7.0, -5.0, 2.0], [-4.0, 6.0, -4.0]], [[-3.0, 3.0, -7.0], [0.0, 6.0, 6.0]]], [[[0.0, -1.0, -6.0], [7.0, 2.0, -4.0]], [[6.0, 0.0, 3.0], [-6.0, 2.0, -3.0]], [[-1.0, 7.0, 0.0], [-2.0, -6.0, 0.0]], [[1.0, 1.0, 3.0], [-1.0, 2.0, 7.0]], [[1.0, 0.0, 0.0], [-5.0, 0.0, 7.0]]], [[[4.0, 3.0, -7.0], [5.0, -6.0, -7.0]], [[-3.0, -2.0, 4.0], [0.0, 7.0, 7.0]], [[7.0, 1.0, 1.0], [7.0, 3.0, 2.0]], [[-3.0, -1.0, -6.0], [-4.0, -5.0, 4.0]], [[-1.0, -5.0, 1.0], [-2.0, -6.0, 1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, -3.0, 6.0], [3.0, 3.0, -6.0]], [[-7.0, 2.0, 0.0], [0.0, -2.0, 7.0]], [[5.0, 0.0, 4.0], [-7.0, 5.0, -6.0]], [[4.0, 4.0, -1.0], [0.0, -3.0, 7.0]], [[-2.0, 3.0, 7.0], [5.0, 0.0, 2.0]]], [[[5.0, 0.0, 7.0], [4.0, 5.0, 5.0]], [[3.0, -4.0, 3.0], [-4.0, 6.0, 0.0]], [[0.0, 6.0, -6.0], [-6.0, -7.0, 0.0]], [[2.0, -1.0, -5.0], [-3.0, 0.0, 2.0]], [[7.0, -5.0, -5.0], [1.0, 0.0, -6.0]]], [[[-1.0, 6.0, -2.0], [-6.0, 0.0, 1.0]], [[-4.0, -7.0, -5.0], [4.0, 4.0, -5.0]], [[5.0, 0.0, -3.0], [1.0, -4.0, 0.0]], [[-1.0, -1.0, -4.0], [4.0, 1.0, -3.0]], [[-1.0, -3.0, 7.0], [7.0, -3.0, 3.0]]], [[[7.0, -2.0, 1.0], [-7.0, -7.0, 0.0]], [[0.0, -1.0, -7.0], [6.0, -5.0, -4.0]], [[-2.0, 4.0, -6.0], [4.0, -4.0, 0.0]], [[3.0, -6.0, -4.0], [-4.0, 7.0, 3.0]], [[-7.0, 6.0, 5.0], [-5.0, -3.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, 6.0, -5.0], [6.0, 6.0, 6.0]], [[3.0, 3.0, 1.0], [3.0, -5.0, -3.0]], [[2.0, 6.0, -2.0], [-6.0, -4.0, -2.0]], [[0.0, -5.0, 6.0], [0.0, 5.0, 0.0]], [[5.0, -5.0, -6.0], [-5.0, 0.0, -4.0]]], [[[0.0, 4.0, -1.0], [2.0, -6.0, 6.0]], [[-6.0, 7.0, 4.0], [0.0, 1.0, 1.0]], [[7.0, -3.0, 6.0], [3.0, -1.0, -5.0]], [[4.0, -3.0, -1.0], [5.0, 4.0, 2.0]], [[2.0, -3.0, -3.0], [-2.0, 0.0, -5.0]]], [[[-2.0, 2.0, -5.0], [-7.0, 0.0, -7.0]], [[-3.0, 3.0, -7.0], [-1.0, 7.0, -7.0]], [[-7.0, 5.0, 0.0], [6.0, 4.0, 0.0]], [[-2.0, -4.0, 7.0], [-5.0, -1.0, -1.0]], [[-1.0, -2.0, -6.0], [4.0, 7.0, 2.0]]], [[[2.0, -7.0, 6.0], [7.0, -2.0, 4.0]], [[7.0, 3.0, -1.0], [4.0, -6.0, -1.0]], [[4.0, -7.0, -2.0], [1.0, -5.0, 1.0]], [[0.0, 7.0, 5.0], [3.0, 0.0, 5.0]], [[4.0, 3.0, 6.0], [-3.0, -5.0, -6.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, -32.0, 59.0, 32.0, 2.0], [22.0, 24.0, 30.0, -2.0, 12.0], [6.0, -91.0, 31.0, -55.0, 22.0], [127.0, -41.0, -48.0, -22.0, 25.0]], [[57.0, -83.0, 21.0, -15.0, -17.0], [-23.0, -22.0, 12.0, -9.0, 97.0], [-43.0, 13.0, 57.0, 23.0, 18.0], [67.0, 78.0, 16.0, -45.0, -72.0]], [[-72.0, 23.0, -20.0, 57.0, -64.0], [-32.0, 7.0, 71.0, 60.0, 19.0], [23.0, -38.0, 41.0, -30.0, -28.0], [102.0, -3.0, -2.0, 56.0, -2.0]], [[39.0, -23.0, 62.0, -72.0, -8.0], [-50.0, 17.0, 0.0, -24.0, 0.0], [9.0, 27.0, -33.0, -2.0, -21.0], [5.0, -16.0, -30.0, -23.0, 36.0]], [[-36.0, 32.0, -75.0, -25.0, -11.0], [-71.0, -62.0, 25.0, 1.0, -1.0], [-2.0, 69.0, -4.0, 50.0, -14.0], [-71.0, 78.0, 72.0, -29.0, -5.0]]], [[[-33.0, -2.0, -79.0, 24.0, -17.0], [-45.0, -74.0, 46.0, 15.0, 44.0], [-12.0, 30.0, 48.0, 48.0, -5.0], [-18.0, 107.0, 88.0, -36.0, -51.0]], [[-24.0, 71.0, -52.0, 5.0, 46.0], [-7.0, -30.0, 4.0, -26.0, -80.0], [1.0, -27.0, -26.0, -23.0, 55.0], [-10.0, -20.0, 4.0, -53.0, 83.0]], [[57.0, -13.0, 2.0, -12.0, 70.0], [42.0, -20.0, -41.0, -54.0, -22.0], [-27.0, -20.0, -9.0, 1.0, 62.0], [-28.0, 2.0, 0.0, -78.0, 17.0]], [[36.0, -1.0, 55.0, -96.0, -15.0], [-27.0, 57.0, -60.0, -15.0, -14.0], [-7.0, 81.0, -69.0, 6.0, -22.0], [-51.0, -47.0, -58.0, 47.0, 19.0]], [[-78.0, 57.0, -49.0, 31.0, -22.0], [-4.0, -6.0, 18.0, 38.0, -37.0], [41.0, 20.0, -18.0, 16.0, -55.0], [-76.0, -8.0, 36.0, 61.0, 4.0]]], [[[18.0, -34.0, -39.0, -32.0, -18.0], [-24.0, -30.0, -26.0, 2.0, 66.0], [-40.0, 93.0, 17.0, 67.0, -8.0], [-67.0, 91.0, 52.0, 4.0, -89.0]], [[42.0, -67.0, 112.0, -6.0, -27.0], [22.0, 63.0, 4.0, 9.0, 39.0], [21.0, -40.0, 7.0, -31.0, -42.0], [73.0, -55.0, -62.0, 35.0, -12.0]], [[-48.0, 33.0, -21.0, 42.0, 13.0], [-43.0, -59.0, 96.0, -3.0, -44.0], [55.0, -77.0, 17.0, -20.0, -16.0], [35.0, 11.0, 46.0, -79.0, 77.0]], [[-21.0, 40.0, -8.0, 48.0, 31.0], [67.0, 24.0, -20.0, 3.0, -56.0], [12.0, -57.0, -13.0, -37.0, 25.0], [1.0, -66.0, -28.0, 24.0, 34.0]], [[-54.0, 42.0, -2.0, 53.0, -13.0], [20.0, 23.0, 30.0, 31.0, -40.0], [36.0, -59.0, 0.0, -42.0, -15.0], [42.0, -58.0, -22.0, 44.0, 39.0]]], [[[0.0, -59.0, -31.0, 4.0, -37.0], [-74.0, -77.0, 72.0, 11.0, 95.0], [-9.0, 29.0, 70.0, 56.0, -30.0], [22.0, 134.0, 90.0, -61.0, -73.0]], [[0.0, 52.0, -6.0, 4.0, 42.0], [83.0, 53.0, -85.0, -10.0, -73.0], [-10.0, -1.0, -55.0, -25.0, 37.0], [-62.0, -89.0, -54.0, 57.0, 29.0]], [[42.0, -39.0, -7.0, 36.0, 35.0], [95.0, 10.0, -63.0, -9.0, 34.0], [-43.0, -10.0, 27.0, 13.0, 43.0], [-22.0, 11.0, 0.0, 10.0, -76.0]], [[-93.0, 57.0, -60.0, 33.0, -51.0], [-78.0, -37.0, 89.0, 45.0, -14.0], [37.0, -7.0, 19.0, -5.0, -37.0], [38.0, 28.0, 42.0, 14.0, 32.0]], [[-12.0, 16.0, -23.0, 0.0, -14.0], [-31.0, -8.0, 18.0, 6.0, 5.0], [-18.0, -3.0, 14.0, -15.0, 30.0], [60.0, 12.0, -8.0, -8.0, 13.0]]]])+(1.-msk_ref)*numpy.array([[[[-132.0, 36.0, 48.0, 11.0, 24.0], [16.0, -25.0, 72.0, -28.0, 7.0], [-9.0, -100.0, -101.0, 65.0, -102.0], [32.0, 49.0, 44.0, 8.0, 102.0]], [[-102.0, 72.0, 20.0, -20.0, 53.0], [-8.0, -63.0, 98.0, 7.0, 47.0], [12.0, -20.0, -58.0, -6.0, -46.0], [31.0, 91.0, 64.0, -40.0, 81.0]], [[-49.0, -18.0, 34.0, -29.0, 65.0], [42.0, -61.0, -34.0, -10.0, 12.0], [6.0, -64.0, -97.0, 11.0, -18.0], [2.0, 13.0, 73.0, -36.0, -17.0]], [[-67.0, -3.0, 18.0, 23.0, 15.0], [-57.0, 22.0, 16.0, -50.0, 29.0], [68.0, 41.0, -1.0, 60.0, -42.0], [-39.0, -43.0, -26.0, -39.0, 53.0]], [[72.0, 2.0, -10.0, -46.0, -11.0], [12.0, 16.0, -58.0, -9.0, 5.0], [30.0, 66.0, 67.0, -51.0, 45.0], [-33.0, -9.0, -12.0, -21.0, -52.0]]], [[[48.0, 43.0, 10.0, -72.0, 11.0], [45.0, -25.0, -14.0, 17.0, 10.0], [2.0, 25.0, 23.0, -84.0, 31.0], [-8.0, 69.0, 29.0, -8.0, -24.0]], [[4.0, -3.0, -10.0, 4.0, -66.0], [38.0, 51.0, -29.0, -38.0, -39.0], [-29.0, -59.0, 26.0, 41.0, -40.0], [32.0, -25.0, -9.0, 35.0, 2.0]], [[-6.0, 44.0, -14.0, 16.0, -64.0], [4.0, 41.0, 58.0, 0.0, -22.0], [-38.0, -24.0, 41.0, 9.0, -35.0], [40.0, 32.0, -26.0, 51.0, 53.0]], [[-17.0, -64.0, -40.0, 67.0, 14.0], [-90.0, 17.0, -20.0, -19.0, 23.0], [50.0, 66.0, 24.0, 66.0, 23.0], [-23.0, -114.0, -35.0, -57.0, -25.0]], [[140.0, -55.0, -10.0, -27.0, -12.0], [19.0, 23.0, -108.0, 22.0, -24.0], [5.0, 67.0, 60.0, -67.0, 93.0], [-57.0, -47.0, -43.0, 16.0, -111.0]]], [[[54.0, 14.0, -36.0, -21.0, 22.0], [-20.0, -33.0, 6.0, 44.0, 27.0], [7.0, 74.0, 45.0, -71.0, 72.0], [0.0, 23.0, 12.0, -36.0, -46.0]], [[-87.0, 2.0, 40.0, 28.0, 54.0], [-45.0, -25.0, 55.0, -7.0, 30.0], [36.0, 2.0, -70.0, 40.0, -40.0], [-28.0, 6.0, -1.0, -18.0, 68.0]], [[-1.0, 42.0, 76.0, -65.0, -30.0], [60.0, 49.0, -28.0, -59.0, -19.0], [30.0, -16.0, 6.0, -10.0, -63.0], [-53.0, 42.0, -25.0, 43.0, 53.0]], [[34.0, -25.0, -14.0, 23.0, -41.0], [29.0, 22.0, -18.0, 15.0, -43.0], [-57.0, -55.0, 0.0, 11.0, 3.0], [30.0, -16.0, -13.0, 54.0, -24.0]], [[7.0, -33.0, 18.0, 0.0, 2.0], [32.0, 1.0, -43.0, -7.0, -23.0], [-16.0, -47.0, -37.0, 16.0, -7.0], [-5.0, -20.0, 6.0, 20.0, -23.0]]], [[[-15.0, 80.0, 48.0, -87.0, 50.0], [23.0, -44.0, 33.0, 1.0, 45.0], [47.0, 48.0, -7.0, -81.0, -4.0], [-36.0, 107.0, 37.0, -34.0, 40.0]], [[67.0, -67.0, -68.0, 69.0, -57.0], [-12.0, 34.0, -33.0, 32.0, -47.0], [-67.0, -25.0, 39.0, 28.0, 46.0], [46.0, -80.0, -34.0, 41.0, -71.0]], [[38.0, 25.0, -40.0, 16.0, -19.0], [11.0, -26.0, 60.0, 75.0, -16.0], [-80.0, -25.0, 10.0, -51.0, 38.0], [64.0, 59.0, 13.0, 43.0, -11.0]], [[9.0, -17.0, 36.0, -56.0, 30.0], [48.0, -14.0, -81.0, -35.0, 3.0], [34.0, -11.0, -28.0, -11.0, -2.0], [-37.0, -8.0, 32.0, -29.0, -38.0]], [[-72.0, 4.0, 0.0, 1.0, 20.0], [17.0, -30.0, 9.0, -22.0, 9.0], [-6.0, -66.0, -54.0, 44.0, -43.0], [41.0, 4.0, 58.0, -31.0, 17.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_array_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([-4.0, 0.0, -1.0])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[-2.0, 6.0, -7.0], [-6.0, 1.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([15.0, 24.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_array_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[5.0, -7.0, -3.0], [0.0, 5.0, -5.0]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[-5.0, 3.0, 3.0], [7.0, 2.0, -5.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-55.0, 0.0], [36.0, 35.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_Symbol_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3,))
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[3.0, 3.0, 2.0], [3.0, 3.0, 3.0]])
      s1=numpy.array([-2.0, -5.0, -4.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-29.0, -33.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_Symbol_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[7.0, -2.0, 0.0], [0.0, -2.0, -6.0]])
      s1=numpy.array([[1.0, -3.0, 5.0], [-2.0, -6.0, 0.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[13.0, -2.0], [-24.0, 12.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_constData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([2.0, 7.0, 2.0]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[5.0, 4.0, 2.0], [4.0, -1.0, 3.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([42.0, 7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_constData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[5.0, -3.0, -5.0], [5.0, 6.0, 6.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[-7.0, 0.0, 4.0], [5.0, -1.0, 6.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-55.0, -11.0], [-2.0, 55.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_expandedData_rank1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, 2.0, 5.0])+(1.-msk_arg1)*numpy.array([6.0, 7.0, 4.0])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[0.0, -2.0, 7.0], [-4.0, 0.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([31.0, 9.0])+(1.-msk_ref)*numpy.array([14.0, -20.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank2_expandedData_rank2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, -1.0, 5.0], [0.0, -4.0, 6.0]])+(1.-msk_arg1)*numpy.array([[-4.0, -6.0, -4.0], [1.0, -7.0, -7.0]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[-2.0, -1.0, 7.0], [-1.0, -3.0, -2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[44.0, 46.0], [-3.0, 0.0]])+(1.-msk_ref)*numpy.array([[-14.0, -44.0], [30.0, 34.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_array_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[-6.0, -3.0, -1.0], [2.0, 5.0, 0.0]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[4.0, -6.0, 0.0], [-4.0, 6.0, 2.0]], [[3.0, 0.0, 2.0], [-3.0, -7.0, -6.0]], [[-3.0, -5.0, 2.0], [-2.0, -3.0, -2.0]], [[0.0, 3.0, 0.0], [-3.0, 1.0, -7.0]], [[-6.0, 1.0, 7.0], [6.0, 0.0, 5.0]]], [[[-6.0, -5.0, 0.0], [6.0, -1.0, -3.0]], [[0.0, 3.0, -7.0], [4.0, -3.0, 3.0]], [[4.0, -2.0, -4.0], [0.0, 3.0, -5.0]], [[-4.0, -6.0, 4.0], [0.0, 3.0, -2.0]], [[2.0, -4.0, -2.0], [-7.0, 3.0, 0.0]]], [[[2.0, 1.0, 7.0], [1.0, -7.0, 0.0]], [[3.0, 4.0, -2.0], [2.0, 0.0, -1.0]], [[0.0, 5.0, 5.0], [1.0, 2.0, -7.0]], [[6.0, -6.0, -1.0], [1.0, -3.0, 1.0]], [[-7.0, -2.0, 2.0], [0.0, -6.0, -1.0]]], [[[-4.0, 2.0, -4.0], [-1.0, -7.0, -5.0]], [[4.0, -1.0, -6.0], [-5.0, 4.0, 3.0]], [[-7.0, 3.0, 0.0], [-5.0, 4.0, 3.0]], [[0.0, -4.0, -2.0], [4.0, -4.0, 5.0]], [[-7.0, -2.0, -6.0], [1.0, 1.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[16.0, -61.0, 12.0, -10.0, 38.0], [58.0, -9.0, 1.0, 53.0, 3.0], [-55.0, -24.0, -8.0, -30.0, 16.0], [-15.0, -5.0, 43.0, 2.0, 61.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_array_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[-6.0, -2.0, -2.0], [-2.0, 2.0, 6.0]], [[4.0, 7.0, -6.0], [1.0, 2.0, -5.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[2.0, 0.0, -1.0], [2.0, 0.0, -7.0]], [[2.0, -1.0, 7.0], [-7.0, 1.0, 1.0]], [[4.0, -4.0, -6.0], [0.0, 4.0, 2.0]], [[2.0, 5.0, 7.0], [-2.0, 6.0, 0.0]], [[-7.0, -6.0, -5.0], [-7.0, -7.0, -1.0]]], [[[6.0, 1.0, 2.0], [-3.0, 3.0, 2.0]], [[7.0, 6.0, -7.0], [-7.0, -7.0, -3.0]], [[-6.0, 6.0, -1.0], [-7.0, -7.0, 3.0]], [[-6.0, 0.0, -2.0], [3.0, 3.0, -4.0]], [[-7.0, -5.0, -2.0], [6.0, -5.0, -1.0]]], [[[-7.0, 5.0, 7.0], [1.0, 5.0, -5.0]], [[0.0, 0.0, -5.0], [-2.0, -3.0, -5.0]], [[4.0, 7.0, 0.0], [-1.0, 0.0, -7.0]], [[-2.0, 7.0, -5.0], [1.0, -2.0, 3.0]], [[1.0, 7.0, -2.0], [-7.0, 5.0, 0.0]]], [[[-3.0, 5.0, 5.0], [7.0, 0.0, -1.0]], [[6.0, -1.0, -1.0], [-7.0, 0.0, 3.0]], [[0.0, -2.0, -1.0], [1.0, -1.0, -4.0]], [[-5.0, 3.0, 5.0], [4.0, 4.0, 3.0]], [[0.0, 1.0, 0.0], [2.0, -7.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-56.0, 51.0], [-2.0, -51.0], [16.0, 22.0], [-20.0, 11.0], [58.0, -56.0]], [[-18.0, 12.0], [-58.0, 106.0], [44.0, -12.0], [16.0, 17.0], [28.0, -50.0]], [[-4.0, 1.0], [-22.0, 47.0], [-78.0, 99.0], [20.0, 53.0], [8.0, 68.0]], [[-22.0, 5.0], [0.0, 1.0], [-22.0, 11.0], [32.0, -32.0], [-26.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_array_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[[1.0, 7.0, -2.0], [-7.0, -5.0, 4.0]], [[3.0, 0.0, -4.0], [-4.0, -5.0, -4.0]], [[0.0, 5.0, 4.0], [0.0, 2.0, -3.0]], [[-6.0, -1.0, 3.0], [-6.0, -3.0, 2.0]], [[4.0, 4.0, 2.0], [0.0, 2.0, 7.0]]], [[[6.0, 7.0, 2.0], [-3.0, 6.0, 2.0]], [[-5.0, 2.0, -4.0], [-7.0, 6.0, 0.0]], [[3.0, -7.0, 4.0], [-1.0, 2.0, 6.0]], [[-4.0, 4.0, -4.0], [7.0, 4.0, -5.0]], [[-3.0, 1.0, 2.0], [-1.0, -7.0, -7.0]]], [[[-2.0, 2.0, -4.0], [-1.0, 4.0, 2.0]], [[-6.0, -2.0, -1.0], [3.0, 3.0, 0.0]], [[7.0, 7.0, 1.0], [-6.0, -2.0, -4.0]], [[0.0, 0.0, 3.0], [2.0, 3.0, -4.0]], [[-5.0, -6.0, 4.0], [1.0, -2.0, 3.0]]], [[[-7.0, 0.0, -6.0], [2.0, 5.0, 2.0]], [[3.0, -2.0, -5.0], [3.0, -3.0, 0.0]], [[4.0, 4.0, -2.0], [1.0, 3.0, -2.0]], [[0.0, 0.0, 0.0], [2.0, -1.0, 4.0]], [[5.0, 2.0, 7.0], [-3.0, 4.0, 0.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[-6.0, 5.0, -7.0], [0.0, 0.0, -4.0]], [[-6.0, -7.0, 7.0], [-3.0, 2.0, 0.0]], [[-2.0, 7.0, -7.0], [7.0, -3.0, 2.0]], [[3.0, -1.0, 5.0], [3.0, -3.0, -5.0]], [[1.0, -1.0, -7.0], [0.0, 1.0, 4.0]]], [[[2.0, -7.0, 0.0], [0.0, 2.0, -5.0]], [[3.0, 0.0, -5.0], [-4.0, -5.0, 3.0]], [[2.0, 0.0, 6.0], [-7.0, 1.0, 0.0]], [[1.0, -5.0, 5.0], [-6.0, 0.0, 3.0]], [[2.0, 2.0, -5.0], [-2.0, -7.0, 6.0]]], [[[-3.0, -3.0, 5.0], [-4.0, 4.0, -7.0]], [[5.0, 6.0, 5.0], [-5.0, 4.0, -1.0]], [[2.0, 2.0, 0.0], [-5.0, -2.0, 0.0]], [[0.0, 2.0, 4.0], [6.0, 0.0, -7.0]], [[0.0, -7.0, 1.0], [6.0, 6.0, 2.0]]], [[[-6.0, -7.0, -4.0], [1.0, -1.0, -1.0]], [[0.0, 0.0, 0.0], [-1.0, -2.0, 5.0]], [[-6.0, -7.0, 3.0], [-1.0, 6.0, -5.0]], [[3.0, -1.0, 7.0], [-3.0, 3.0, -6.0]], [[-4.0, 0.0, -4.0], [6.0, 2.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[27.0, 26.0, 9.0, 2.0, -46.0], [-23.0, 68.0, -105.0, 92.0, 37.0], [42.0, 33.0, 2.0, -5.0, -40.0], [76.0, 7.0, 18.0, -16.0, -69.0]], [[-58.0, -44.0, -3.0, 76.0, -34.0], [-50.0, 21.0, 66.0, -45.0, 14.0], [-19.0, 40.0, -70.0, 21.0, 93.0], [4.0, -54.0, -63.0, -8.0, 22.0]], [[35.0, 1.0, -5.0, -45.0, 14.0], [-12.0, -15.0, -84.0, 91.0, -1.0], [31.0, 17.0, -16.0, -24.0, -41.0], [59.0, 45.0, 28.0, 25.0, -78.0]], [[-40.0, 12.0, 24.0, -21.0, -23.0], [-16.0, -76.0, -3.0, -2.0, 53.0], [-53.0, -21.0, 27.0, 32.0, 5.0], [-70.0, 4.0, 2.0, -11.0, 27.0]], [[19.0, 10.0, -43.0, -21.0, 16.0], [-1.0, 27.0, 8.0, 4.0, -53.0], [36.0, 6.0, -25.0, -34.0, -17.0], [48.0, 37.0, 9.0, 15.0, -42.0]]], [[[-77.0, 16.0, -16.0, -21.0, -51.0], [-35.0, -12.0, 29.0, -3.0, 8.0], [-20.0, 8.0, -19.0, 26.0, 13.0], [-14.0, 14.0, -4.0, -22.0, 4.0]], [[78.0, 58.0, -39.0, 12.0, 13.0], [-4.0, 3.0, 1.0, -55.0, -1.0], [4.0, -40.0, 38.0, -50.0, -20.0], [-18.0, 37.0, -3.0, 9.0, -28.0]], [[34.0, 5.0, 26.0, 45.0, 22.0], [51.0, 21.0, 39.0, -77.0, 6.0], [-17.0, -36.0, 60.0, 7.0, 5.0], [-59.0, -48.0, -8.0, -15.0, 77.0]], [[10.0, -5.0, -14.0, 56.0, 15.0], [5.0, 7.0, 82.0, -101.0, -13.0], [-20.0, -19.0, 1.0, -9.0, 48.0], [-43.0, -30.0, -38.0, 0.0, 48.0]], [[99.0, 45.0, -42.0, 16.0, 34.0], [-8.0, -14.0, -4.0, -52.0, -5.0], [6.0, -38.0, 25.0, -64.0, -12.0], [-11.0, 42.0, -9.0, 27.0, -43.0]]], [[[-54.0, -5.0, 34.0, 34.0, -55.0], [-7.0, 41.0, 2.0, 3.0, 41.0], [-14.0, 19.0, 7.0, 47.0, 20.0], [-11.0, -52.0, -12.0, -40.0, 42.0]], [[48.0, -1.0, 61.0, -5.0, 55.0], [119.0, 26.0, 0.0, -30.0, -15.0], [1.0, -50.0, 108.0, 21.0, -57.0], [-57.0, -49.0, 43.0, -18.0, 103.0]], [[61.0, 36.0, 6.0, 22.0, 12.0], [29.0, 17.0, -7.0, -43.0, 15.0], [-3.0, -37.0, 62.0, -16.0, -23.0], [-34.0, -7.0, 5.0, -8.0, 21.0]], [[-64.0, -12.0, 47.0, -40.0, -33.0], [-10.0, -54.0, -46.0, 69.0, 53.0], [-32.0, 10.0, 10.0, 52.0, -11.0], [-26.0, -6.0, 20.0, -16.0, 14.0]], [[-115.0, -66.0, -25.0, -40.0, 0.0], [-25.0, -24.0, 71.0, 24.0, -67.0], [4.0, 49.0, -104.0, 25.0, 46.0], [40.0, 9.0, -10.0, 14.0, -1.0]]], [[[-53.0, 3.0, -50.0, 26.0, -69.0], [-104.0, 19.0, 6.0, 20.0, 16.0], [7.0, 54.0, -95.0, -9.0, 56.0], [61.0, 22.0, -44.0, -1.0, -79.0]], [[37.0, -6.0, -19.0, 22.0, 31.0], [1.0, -5.0, 27.0, -40.0, -20.0], [3.0, -9.0, -10.0, -28.0, 18.0], [-2.0, 3.0, -17.0, 20.0, -5.0]], [[-104.0, -36.0, 4.0, 30.0, -69.0], [-50.0, 47.0, 26.0, 26.0, 11.0], [1.0, 62.0, -74.0, 45.0, 56.0], [42.0, -40.0, -31.0, -28.0, 4.0]], [[-36.0, 2.0, 47.0, 1.0, -14.0], [40.0, -6.0, 17.0, -23.0, 28.0], [-33.0, -23.0, 57.0, 48.0, -8.0], [-66.0, -42.0, 12.0, -33.0, 83.0]], [[-28.0, -50.0, -27.0, -20.0, 15.0], [-28.0, 6.0, 0.0, 57.0, -51.0], [36.0, 52.0, -92.0, -14.0, 21.0], [84.0, 20.0, -6.0, 30.0, -58.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_Symbol_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[0.0, -1.0, -5.0], [-5.0, 0.0, -3.0]], [[2.0, -1.0, -3.0], [3.0, -1.0, 5.0]], [[3.0, 0.0, 1.0], [2.0, 1.0, -4.0]], [[-1.0, -6.0, 7.0], [-7.0, -2.0, 4.0]], [[-4.0, -1.0, 1.0], [-3.0, -7.0, -7.0]]], [[[-4.0, 6.0, -7.0], [-3.0, -6.0, -2.0]], [[2.0, -6.0, 0.0], [6.0, 6.0, 5.0]], [[0.0, 1.0, -5.0], [-1.0, -7.0, 4.0]], [[-1.0, 4.0, 6.0], [1.0, 3.0, -1.0]], [[4.0, 0.0, -7.0], [-1.0, 0.0, -3.0]]], [[[0.0, -7.0, 4.0], [-6.0, 7.0, -3.0]], [[-3.0, 3.0, -4.0], [-3.0, -1.0, 2.0]], [[7.0, -5.0, 0.0], [3.0, 2.0, -1.0]], [[0.0, -5.0, 4.0], [-4.0, -2.0, -7.0]], [[1.0, 0.0, 7.0], [-3.0, -7.0, 4.0]]], [[[-1.0, -7.0, 7.0], [-2.0, -6.0, 7.0]], [[-7.0, -7.0, -1.0], [-7.0, 0.0, 5.0]], [[7.0, 5.0, 5.0], [5.0, 1.0, 0.0]], [[5.0, 0.0, -2.0], [-4.0, 0.0, 4.0]], [[-6.0, -3.0, -3.0], [3.0, 2.0, 0.0]]]])
      s1=numpy.array([[6.0, 3.0, 6.0], [-3.0, 1.0, 0.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-18.0, -19.0, 19.0, 37.0, -19.0], [-45.0, -18.0, -31.0, 42.0, -15.0], [28.0, -25.0, 20.0, 19.0, 50.0], [15.0, -48.0, 73.0, 30.0, -70.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_Symbol_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[6.0, -3.0, 5.0], [-1.0, -1.0, 7.0]], [[-1.0, 0.0, 0.0], [7.0, -1.0, 7.0]], [[-7.0, -4.0, -7.0], [6.0, 3.0, -1.0]], [[-2.0, 0.0, -1.0], [0.0, 2.0, 4.0]], [[1.0, 1.0, 1.0], [3.0, -3.0, 5.0]]], [[[4.0, -6.0, 7.0], [0.0, -5.0, -4.0]], [[1.0, -1.0, -5.0], [1.0, -5.0, 2.0]], [[4.0, -4.0, -7.0], [3.0, 1.0, 2.0]], [[6.0, -2.0, 7.0], [0.0, 5.0, 5.0]], [[-5.0, 6.0, -4.0], [-7.0, -2.0, -3.0]]], [[[-4.0, -6.0, -3.0], [0.0, -7.0, -6.0]], [[-2.0, 4.0, 6.0], [2.0, -7.0, -3.0]], [[7.0, 2.0, -7.0], [-4.0, 1.0, 3.0]], [[5.0, 2.0, 0.0], [6.0, 0.0, -2.0]], [[-4.0, 3.0, -7.0], [-4.0, 5.0, -3.0]]], [[[-6.0, -1.0, -6.0], [0.0, 2.0, 5.0]], [[1.0, -6.0, -7.0], [-3.0, -4.0, -5.0]], [[2.0, -3.0, 2.0], [5.0, 1.0, 3.0]], [[3.0, -7.0, 6.0], [2.0, 6.0, 2.0]], [[4.0, -6.0, 5.0], [-6.0, 0.0, 7.0]]]])
      s1=numpy.array([[[0.0, 0.0, -1.0], [-3.0, 6.0, 1.0]], [[5.0, -6.0, -4.0], [2.0, -3.0, 7.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-1.0, 78.0], [-20.0, 61.0], [6.0, 13.0], [17.0, 16.0], [-23.0, 45.0]], [[-41.0, 15.0], [-26.0, 62.0], [6.0, 89.0], [28.0, 34.0], [10.0, -74.0]], [[-45.0, 7.0], [-57.0, -54.0], [28.0, 61.0], [-20.0, 11.0], [46.0, -54.0]], [[23.0, 29.0], [-13.0, 40.0], [-8.0, 48.0], [26.0, 33.0], [20.0, 73.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_Symbol_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[-3.0, -6.0, 5.0], [6.0, -5.0, 0.0]], [[4.0, 6.0, -5.0], [-2.0, 6.0, -5.0]], [[4.0, -2.0, 2.0], [3.0, -4.0, -7.0]], [[6.0, -6.0, 3.0], [6.0, -5.0, -5.0]], [[2.0, -4.0, -2.0], [7.0, 6.0, 4.0]]], [[[-2.0, -5.0, 6.0], [6.0, 7.0, -7.0]], [[-3.0, -1.0, 1.0], [3.0, 0.0, 6.0]], [[-6.0, 4.0, 2.0], [6.0, 3.0, 1.0]], [[-6.0, 1.0, -1.0], [-7.0, 0.0, 6.0]], [[-1.0, -7.0, 1.0], [0.0, -5.0, 0.0]]], [[[7.0, -2.0, 1.0], [-5.0, 2.0, -1.0]], [[3.0, -5.0, -1.0], [1.0, 1.0, 5.0]], [[-7.0, 1.0, 1.0], [6.0, -3.0, -4.0]], [[-4.0, -7.0, 2.0], [5.0, -5.0, -5.0]], [[7.0, 2.0, 3.0], [-3.0, 0.0, -2.0]]], [[[4.0, 2.0, 5.0], [5.0, -4.0, 5.0]], [[5.0, -4.0, -5.0], [-3.0, -3.0, -3.0]], [[2.0, 5.0, 3.0], [1.0, 5.0, -3.0]], [[-1.0, 5.0, 1.0], [-1.0, -2.0, 0.0]], [[-4.0, 4.0, -1.0], [6.0, 0.0, 2.0]]]])
      s1=numpy.array([[[[3.0, 2.0, 6.0], [-7.0, 1.0, 0.0]], [[1.0, -6.0, -6.0], [6.0, 0.0, -6.0]], [[-6.0, -2.0, 0.0], [-7.0, -3.0, -1.0]], [[-1.0, -4.0, -6.0], [2.0, 0.0, 4.0]], [[3.0, -1.0, -5.0], [-6.0, 0.0, -7.0]]], [[[0.0, -4.0, 0.0], [3.0, -4.0, 2.0]], [[4.0, -6.0, 7.0], [-5.0, -1.0, 5.0]], [[-2.0, -3.0, -6.0], [0.0, 0.0, -5.0]], [[6.0, 6.0, -5.0], [-2.0, 5.0, -2.0]], [[4.0, -2.0, 6.0], [-7.0, 1.0, 0.0]]], [[[2.0, -4.0, -6.0], [0.0, -1.0, 3.0]], [[-3.0, -4.0, 6.0], [0.0, -1.0, -5.0]], [[5.0, -7.0, 0.0], [6.0, -6.0, -5.0]], [[7.0, -3.0, -5.0], [-7.0, 5.0, 7.0]], [[-3.0, -5.0, -7.0], [-1.0, -1.0, 0.0]]], [[[-4.0, 5.0, 5.0], [3.0, 7.0, 4.0]], [[4.0, -1.0, -6.0], [2.0, -3.0, -5.0]], [[1.0, -2.0, 0.0], [2.0, -1.0, -4.0]], [[3.0, -2.0, 0.0], [-3.0, -1.0, 3.0]], [[0.0, 7.0, -4.0], [-4.0, -3.0, 7.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-38.0, 39.0, 3.0, 9.0, -64.0], [62.0, 34.0, -6.0, -116.0, -17.0], [-7.0, 68.0, 93.0, -95.0, 3.0], [-10.0, -9.0, 26.0, -10.0, -71.0]], [[14.0, 16.0, -35.0, -22.0, 78.0], [-64.0, -76.0, 29.0, 129.0, -6.0], [-7.0, -47.0, -45.0, 44.0, -11.0], [5.0, 43.0, 2.0, -15.0, 17.0]], [[-5.0, 64.0, -22.0, -30.0, 35.0], [19.0, -4.0, 21.0, -10.0, 7.0], [-13.0, 47.0, 111.0, -66.0, -15.0], [-63.0, 59.0, 46.0, -10.0, -71.0]], [[-23.0, 90.0, -46.0, -8.0, 8.0], [52.0, 31.0, 13.0, -42.0, 7.0], [8.0, 54.0, 163.0, -57.0, -10.0], [-76.0, 64.0, 55.0, 2.0, -98.0]], [[-57.0, 56.0, -75.0, 56.0, -50.0], [21.0, -3.0, 0.0, 6.0, -39.0], [38.0, -28.0, 24.0, 45.0, 15.0], [41.0, 0.0, 2.0, -1.0, -38.0]]], [[[-15.0, 70.0, -34.0, -30.0, -18.0], [-4.0, -8.0, 18.0, -35.0, 3.0], [-48.0, 90.0, 54.0, -85.0, -24.0], [52.0, -13.0, 41.0, -42.0, -153.0]], [[-26.0, -21.0, -7.0, 31.0, -73.0], [25.0, 16.0, -27.0, -47.0, -25.0], [10.0, -11.0, -20.0, -2.0, 4.0], [45.0, -41.0, -19.0, 2.0, 19.0]], [[-37.0, -12.0, -24.0, -6.0, -75.0], [-8.0, -62.0, -17.0, -21.0, -59.0], [-40.0, 6.0, -45.0, -84.0, -25.0], [97.0, -42.0, -9.0, -44.0, -6.0]], [[27.0, -84.0, 77.0, 18.0, -14.0], [-13.0, 28.0, -15.0, -23.0, 17.0], [8.0, -22.0, -109.0, 51.0, 27.0], [27.0, -63.0, -46.0, 19.0, 81.0]], [[-16.0, 35.0, 35.0, 23.0, -1.0], [48.0, 50.0, 17.0, -78.0, 11.0], [25.0, 42.0, 74.0, -16.0, 36.0], [-61.0, 12.0, 18.0, 16.0, -38.0]]], [[[60.0, -11.0, -8.0, -19.0, 55.0], [-17.0, 65.0, -9.0, 47.0, 75.0], [11.0, -4.0, 12.0, 88.0, -15.0], [-38.0, 13.0, 3.0, 35.0, -11.0]], [[-13.0, 15.0, -23.0, 45.0, -22.0], [29.0, 54.0, -10.0, -14.0, 10.0], [46.0, -21.0, 25.0, 74.0, 21.0], [-12.0, -3.0, -6.0, 30.0, -3.0]], [[-58.0, 41.0, 11.0, -7.0, -35.0], [18.0, -74.0, 25.0, -60.0, -69.0], [-33.0, 46.0, 32.0, -142.0, 6.0], [19.0, 6.0, 22.0, -50.0, -40.0]], [[-54.0, 86.0, 23.0, 10.0, -10.0], [53.0, -5.0, 42.0, -101.0, -30.0], [-2.0, 82.0, 114.0, -112.0, 33.0], [-49.0, 29.0, 45.0, -23.0, -97.0]], [[64.0, -29.0, -23.0, -47.0, 36.0], [-21.0, 42.0, -28.0, 49.0, 63.0], [-18.0, -1.0, 13.0, 35.0, -49.0], [-20.0, 12.0, 5.0, 20.0, 0.0]]], [[[7.0, -38.0, -56.0, -12.0, -80.0], [33.0, 43.0, -69.0, -29.0, 3.0], [-11.0, -11.0, 35.0, -23.0, -58.0], [26.0, -19.0, -6.0, 12.0, 21.0]], [[-5.0, 59.0, 11.0, 23.0, 83.0], [13.0, 12.0, 47.0, 28.0, 16.0], [50.0, -11.0, 68.0, 57.0, 46.0], [-107.0, 72.0, 22.0, 26.0, -8.0]], [[32.0, -22.0, -41.0, -50.0, 1.0], [-43.0, -26.0, -22.0, 56.0, 14.0], [-48.0, 2.0, -34.0, -19.0, -58.0], [58.0, -13.0, 1.0, -21.0, -17.0]], [[18.0, -43.0, 9.0, -27.0, -7.0], [-15.0, -20.0, -19.0, 11.0, -3.0], [-26.0, -9.0, -34.0, -30.0, -26.0], [17.0, -11.0, -11.0, -8.0, 41.0]], [[-52.0, 2.0, -28.0, 14.0, -61.0], [6.0, -67.0, -8.0, -11.0, -72.0], [-12.0, -20.0, -22.0, -63.0, -7.0], [57.0, -12.0, -8.0, -32.0, 22.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_constData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[5.0, 3.0, -3.0], [-3.0, 7.0, 7.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[5.0, 7.0, -3.0], [1.0, -6.0, -5.0]], [[-5.0, 7.0, -4.0], [6.0, -7.0, 2.0]], [[-4.0, -6.0, -3.0], [-5.0, 5.0, -1.0]], [[-6.0, 4.0, 0.0], [7.0, 4.0, -7.0]], [[0.0, 4.0, 5.0], [1.0, -7.0, 0.0]]], [[[-5.0, 4.0, -7.0], [-4.0, 4.0, 0.0]], [[-7.0, 2.0, 1.0], [0.0, 5.0, 1.0]], [[3.0, 0.0, 3.0], [-5.0, -2.0, -1.0]], [[2.0, 0.0, -5.0], [3.0, 3.0, 4.0]], [[-3.0, 6.0, 5.0], [5.0, 5.0, 7.0]]], [[[4.0, -4.0, -7.0], [2.0, -4.0, 3.0]], [[-5.0, 3.0, -6.0], [-6.0, 6.0, 0.0]], [[7.0, -5.0, -5.0], [-1.0, 1.0, 2.0]], [[-6.0, 0.0, 4.0], [1.0, 4.0, 0.0]], [[0.0, 4.0, 7.0], [5.0, 2.0, 4.0]]], [[[5.0, 5.0, 4.0], [2.0, -2.0, -7.0]], [[0.0, -7.0, -4.0], [-2.0, -5.0, 3.0]], [[2.0, 2.0, -2.0], [2.0, -1.0, 1.0]], [[0.0, 4.0, -6.0], [-4.0, -2.0, 3.0]], [[-3.0, 2.0, -3.0], [0.0, 5.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-25.0, -45.0, 14.0, -60.0, -55.0], [48.0, 10.0, 0.0, 65.0, 57.0], [16.0, 62.0, 59.0, -17.0, 18.0], [-41.0, -17.0, 16.0, 49.0, -7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_constData_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[-7.0, 4.0, 6.0], [6.0, -5.0, 4.0]], [[-7.0, 5.0, 5.0], [-1.0, 4.0, -6.0]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[5.0, 0.0, 3.0], [3.0, 6.0, 2.0]], [[0.0, 0.0, 6.0], [-3.0, -5.0, 0.0]], [[-6.0, -1.0, 5.0], [2.0, -6.0, -1.0]], [[6.0, 6.0, -5.0], [6.0, 4.0, -1.0]], [[4.0, -4.0, -1.0], [1.0, -1.0, -1.0]]], [[[0.0, 0.0, -4.0], [6.0, -3.0, -4.0]], [[2.0, -4.0, 4.0], [6.0, -3.0, 1.0]], [[-6.0, 3.0, 0.0], [-2.0, 3.0, 3.0]], [[0.0, -1.0, -5.0], [-7.0, -2.0, -6.0]], [[1.0, 7.0, 5.0], [-7.0, 7.0, 7.0]]], [[[-2.0, -4.0, 7.0], [-3.0, -6.0, 6.0]], [[-6.0, 0.0, -2.0], [-7.0, -6.0, -2.0]], [[2.0, -3.0, 6.0], [2.0, -2.0, 0.0]], [[-1.0, -5.0, -1.0], [0.0, 1.0, 3.0]], [[5.0, 7.0, 0.0], [4.0, 0.0, -7.0]]], [[[6.0, -1.0, 6.0], [-4.0, 0.0, -2.0]], [[5.0, 1.0, 6.0], [-3.0, 0.0, 3.0]], [[-2.0, 2.0, 0.0], [-4.0, 7.0, 4.0]], [[2.0, -4.0, 3.0], [0.0, -5.0, -4.0]], [[0.0, 3.0, -7.0], [-6.0, -6.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-21.0, -11.0], [43.0, 13.0], [106.0, 42.0], [-36.0, -21.0], [-43.0, -52.0]], [[11.0, -14.0], [49.0, -38.0], [39.0, 53.0], [-90.0, 5.0], [2.0, 46.0]], [[76.0, -28.0], [10.0, 27.0], [32.0, -9.0], [-12.0, -37.0], [-11.0, 38.0]], [[-42.0, -1.0], [-1.0, -15.0], [-21.0, 32.0], [-3.0, -15.0], [-16.0, -68.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_constData_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[[2.0, -4.0, 5.0], [-1.0, -5.0, -2.0]], [[6.0, 3.0, -1.0], [-1.0, 2.0, 3.0]], [[-5.0, -2.0, 3.0], [-7.0, 0.0, -7.0]], [[-7.0, -5.0, 5.0], [-4.0, 1.0, -5.0]], [[7.0, 0.0, 3.0], [-5.0, -4.0, -1.0]]], [[[-7.0, 2.0, 6.0], [5.0, 1.0, 2.0]], [[-7.0, -6.0, 1.0], [0.0, -2.0, -4.0]], [[-2.0, 0.0, 1.0], [-5.0, 7.0, -3.0]], [[2.0, 6.0, -6.0], [1.0, 0.0, 2.0]], [[6.0, 0.0, 7.0], [3.0, 1.0, -1.0]]], [[[2.0, -2.0, -6.0], [0.0, -3.0, 0.0]], [[0.0, 6.0, 6.0], [-3.0, 7.0, -2.0]], [[4.0, -3.0, 5.0], [5.0, 0.0, -2.0]], [[6.0, -7.0, 1.0], [-4.0, 1.0, 5.0]], [[-5.0, -2.0, -1.0], [-7.0, -4.0, 5.0]]], [[[6.0, -1.0, 3.0], [7.0, -3.0, 2.0]], [[0.0, -3.0, 6.0], [7.0, 7.0, 7.0]], [[-1.0, 3.0, 4.0], [-4.0, 5.0, 2.0]], [[-5.0, 7.0, 6.0], [5.0, -5.0, 5.0]], [[-4.0, 1.0, -3.0], [7.0, -1.0, 0.0]]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[6.0, -6.0, -6.0], [0.0, -2.0, -2.0]], [[5.0, 0.0, -2.0], [-2.0, -5.0, -6.0]], [[3.0, -6.0, 7.0], [5.0, 0.0, 2.0]], [[-1.0, 1.0, -3.0], [-1.0, -1.0, 2.0]], [[2.0, 1.0, 6.0], [7.0, -1.0, -1.0]]], [[[0.0, 5.0, 7.0], [-6.0, -6.0, -3.0]], [[-2.0, -2.0, -5.0], [0.0, 7.0, 3.0]], [[1.0, -2.0, 5.0], [3.0, -7.0, -1.0]], [[2.0, -4.0, -6.0], [-2.0, -7.0, -7.0]], [[0.0, 5.0, -4.0], [-7.0, -7.0, -7.0]]], [[[-7.0, 6.0, -1.0], [-5.0, 2.0, 0.0]], [[1.0, 0.0, -4.0], [3.0, 7.0, 0.0]], [[3.0, 4.0, -6.0], [-1.0, 4.0, 4.0]], [[-4.0, 3.0, -7.0], [7.0, -1.0, 1.0]], [[-6.0, -5.0, -6.0], [-4.0, 6.0, -7.0]]], [[[2.0, 0.0, 6.0], [7.0, 0.0, -7.0]], [[2.0, 7.0, 7.0], [-6.0, 4.0, 7.0]], [[3.0, -1.0, -1.0], [4.0, 3.0, 0.0]], [[6.0, 3.0, -6.0], [3.0, 3.0, 7.0]], [[2.0, 2.0, 5.0], [0.0, -4.0, -5.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[20.0, 14.0, -22.0, -34.0, 34.0], [-96.0, 0.0, -26.0, 8.0, -6.0], [66.0, -82.0, 16.0, 60.0, -14.0], [26.0, -46.0, -62.0, -108.0, -10.0]], [[39.0, 6.0, 25.0, -12.0, 65.0], [-74.0, -3.0, -19.0, 8.0, 11.0], [37.0, -29.0, 12.0, 1.0, -19.0], [13.0, -103.0, -42.0, -52.0, -23.0]], [[56.0, -6.0, -31.0, 14.0, 15.0], [38.0, 14.0, -30.0, -63.0, 80.0], [-24.0, -13.0, 86.0, 57.0, -35.0], [84.0, 109.0, -9.0, 20.0, -4.0]], [[-19.0, 5.0, -13.0, -20.0, -9.0], [-11.0, -8.0, -9.0, 25.0, -33.0], [17.0, -20.0, -31.0, -3.0, 27.0], [-16.0, -21.0, -5.0, 4.0, 8.0]], [[30.0, -3.0, -36.0, -13.0, 2.0], [56.0, -8.0, -37.0, -21.0, 75.0], [-31.0, 16.0, 72.0, -23.0, -68.0], [79.0, 68.0, -10.0, 68.0, 25.0]]], [[[57.0, -7.0, 74.0, 43.0, 78.0], [10.0, 1.0, 4.0, -24.0, 28.0], [-34.0, 54.0, -4.0, -25.0, 34.0], [-14.0, -78.0, 31.0, 62.0, -52.0]], [[-62.0, 10.0, -22.0, -9.0, -60.0], [-7.0, -5.0, 39.0, 20.0, -43.0], [9.0, 1.0, -33.0, 19.0, 6.0], [-40.0, 46.0, 17.0, -54.0, 14.0]], [[69.0, -25.0, 0.0, 14.0, 36.0], [25.0, 28.0, -58.0, -39.0, 44.0], [-3.0, -38.0, 52.0, 1.0, -4.0], [63.0, 1.0, -36.0, 56.0, 7.0]], [[41.0, -27.0, 43.0, 12.0, 41.0], [-89.0, 46.0, -28.0, 0.0, -36.0], [69.0, -89.0, -6.0, 0.0, 11.0], [-9.0, -136.0, -79.0, -84.0, -1.0]], [[16.0, -9.0, 76.0, 11.0, 58.0], [-70.0, 8.0, 3.0, 33.0, -49.0], [35.0, -8.0, -56.0, -53.0, 36.0], [-59.0, -186.0, -22.0, -24.0, -25.0]]], [[[-48.0, -14.0, 55.0, 36.0, -35.0], [32.0, 8.0, 52.0, 23.0, -62.0], [-26.0, 59.0, -76.0, -63.0, 51.0], [-92.0, -45.0, 51.0, 36.0, 0.0]], [[-56.0, 21.0, -38.0, -32.0, -48.0], [-9.0, -25.0, 28.0, 29.0, -6.0], [5.0, 16.0, -1.0, -3.0, -50.0], [-6.0, 46.0, 6.0, -49.0, 22.0]], [[-67.0, 57.0, -62.0, -83.0, -12.0], [-42.0, -75.0, 9.0, 73.0, -27.0], [22.0, 11.0, -43.0, 12.0, -6.0], [-15.0, 1.0, 17.0, -28.0, -1.0]], [[-59.0, -14.0, -63.0, -56.0, -81.0], [28.0, 1.0, -44.0, 61.0, -54.0], [31.0, -54.0, -27.0, -76.0, -19.0], [6.0, -2.0, -46.0, 44.0, 90.0]], [[-34.0, -50.0, 99.0, 94.0, -57.0], [-32.0, 82.0, 89.0, -24.0, -77.0], [16.0, 2.0, -45.0, -20.0, 15.0], [-109.0, -56.0, -1.0, -126.0, 3.0]]], [[[41.0, -22.0, 8.0, 23.0, 4.0], [43.0, 20.0, -12.0, -39.0, 82.0], [-32.0, 29.0, 87.0, -45.0, -100.0], [65.0, 36.0, -20.0, 26.0, 23.0]], [[-17.0, 61.0, -10.0, -21.0, 42.0], [30.0, -85.0, 40.0, 12.0, 40.0], [-64.0, 116.0, -22.0, 33.0, 30.0], [-14.0, 56.0, 105.0, 66.0, -68.0]], [[-14.0, 18.0, -44.0, -34.0, -14.0], [-6.0, -22.0, -6.0, 10.0, 26.0], [5.0, -3.0, 30.0, 11.0, -52.0], [35.0, 46.0, -11.0, -23.0, 15.0]], [[-62.0, 75.0, -124.0, -131.0, -10.0], [-40.0, -100.0, -33.0, 83.0, -1.0], [33.0, -20.0, -14.0, 35.0, -28.0], [41.0, 46.0, -4.0, -10.0, 15.0]], [[51.0, -10.0, 36.0, 22.0, 50.0], [6.0, 7.0, -12.0, -24.0, 48.0], [-18.0, 24.0, 37.0, -26.0, -28.0], [27.0, -39.0, -6.0, 29.0, -17.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_expandedData_rank2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 0.0, 5.0], [-3.0, 1.0, 2.0]])+(1.-msk_arg1)*numpy.array([[5.0, 1.0, 1.0], [5.0, 0.0, 6.0]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[0.0, -6.0, 1.0], [0.0, 0.0, 2.0]], [[-1.0, 3.0, 2.0], [-2.0, -4.0, 5.0]], [[-1.0, -5.0, 0.0], [0.0, -7.0, -6.0]], [[0.0, -6.0, -4.0], [-2.0, 4.0, 0.0]], [[1.0, -7.0, -7.0], [-6.0, -2.0, -3.0]]], [[[-5.0, 1.0, -3.0], [-6.0, -3.0, 6.0]], [[4.0, -4.0, 7.0], [2.0, -6.0, -6.0]], [[1.0, -2.0, 7.0], [-7.0, -2.0, -2.0]], [[1.0, 0.0, 0.0], [-3.0, 7.0, 7.0]], [[0.0, 3.0, -1.0], [-3.0, 7.0, 1.0]]], [[[0.0, 2.0, -3.0], [-3.0, 4.0, -4.0]], [[0.0, -2.0, 4.0], [-5.0, 5.0, -5.0]], [[1.0, 5.0, 5.0], [-1.0, 1.0, 1.0]], [[-3.0, -3.0, -3.0], [-1.0, 0.0, -4.0]], [[3.0, 1.0, -1.0], [-2.0, 6.0, 5.0]]], [[[5.0, 0.0, 5.0], [-3.0, 6.0, -6.0]], [[-2.0, -7.0, 1.0], [4.0, 3.0, -1.0]], [[-6.0, -4.0, 2.0], [5.0, -3.0, -2.0]], [[-2.0, -2.0, -7.0], [-6.0, 3.0, -2.0]], [[3.0, 7.0, -4.0], [5.0, -4.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[9.0, 19.0, -22.0, -10.0, -22.0], [-3.0, 23.0, 53.0, 33.0, 13.0], [-10.0, 30.0, 34.0, -29.0, 26.0], [43.0, -12.0, -30.0, -24.0, -26.0]])+(1.-msk_ref)*numpy.array([[7.0, 20.0, -46.0, -20.0, -57.0], [-21.0, -3.0, -37.0, 32.0, -7.0], [-40.0, -53.0, 16.0, -50.0, 35.0], [-21.0, -2.0, -19.0, -61.0, 55.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_expandedData_rank3(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 7.0, 5.0], [-1.0, 5.0, -7.0]], [[3.0, 1.0, -1.0], [-6.0, 6.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, -2.0, -3.0], [0.0, -3.0, -5.0]], [[-1.0, 2.0, -6.0], [-3.0, -3.0, 4.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[0.0, 4.0, -4.0], [6.0, -1.0, 6.0]], [[-7.0, 6.0, 1.0], [0.0, -2.0, -6.0]], [[3.0, 7.0, 2.0], [6.0, -1.0, 3.0]], [[-3.0, -5.0, 0.0], [-6.0, 4.0, 6.0]], [[-4.0, -4.0, 1.0], [-7.0, 4.0, 0.0]]], [[[-7.0, 4.0, 0.0], [2.0, 7.0, 4.0]], [[0.0, -6.0, -2.0], [5.0, 0.0, 5.0]], [[-5.0, -4.0, -3.0], [3.0, 5.0, -5.0]], [[7.0, -3.0, 6.0], [-1.0, 3.0, 5.0]], [[0.0, -6.0, -2.0], [-5.0, 0.0, 0.0]]], [[[-2.0, -3.0, 0.0], [1.0, -1.0, -2.0]], [[0.0, 1.0, -4.0], [-7.0, 0.0, 6.0]], [[-6.0, 4.0, 5.0], [-3.0, -4.0, 6.0]], [[-3.0, 1.0, 5.0], [-6.0, -5.0, -6.0]], [[-6.0, 6.0, -5.0], [-3.0, 4.0, -1.0]]], [[[-6.0, -3.0, -7.0], [1.0, -5.0, 7.0]], [[-7.0, 7.0, 6.0], [2.0, 1.0, -5.0]], [[-7.0, -1.0, 2.0], [-3.0, 5.0, -2.0]], [[-6.0, 1.0, -6.0], [-7.0, 3.0, 1.0]], [[-1.0, 5.0, -1.0], [-7.0, -6.0, -3.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-45.0, -52.0], [86.0, -10.0], [24.0, -37.0], [-48.0, 28.0], [8.0, 49.0]], [[40.0, 1.0], [-92.0, -49.0], [19.0, 11.0], [-17.0, 21.0], [-47.0, 26.0]], [[-11.0, -15.0], [-48.0, 29.0], [0.0, -43.0], [58.0, 11.0], [53.0, 38.0]], [[-125.0, -71.0], [124.0, -11.0], [52.0, 30.0], [-2.0, 46.0], [29.0, 18.0]]])+(1.-msk_ref)*numpy.array([[[-23.0, 41.0], [70.0, -5.0], [-53.0, -4.0], [-11.0, 23.0], [21.0, -1.0]], [[0.0, 4.0], [-7.0, 5.0], [62.0, -29.0], [-95.0, -35.0], [18.0, 15.0]], [[33.0, -12.0], [-20.0, 71.0], [1.0, 29.0], [49.0, -16.0], [38.0, 41.0]], [[49.0, 82.0], [39.0, -44.0], [40.0, -21.0], [44.0, 60.0], [33.0, 44.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_Symbol_rank4_expandedData_rank4(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, -6.0, 7.0], [-6.0, 5.0, -2.0]], [[0.0, 2.0, 5.0], [6.0, 7.0, -3.0]], [[-6.0, -3.0, 7.0], [-6.0, -1.0, 5.0]], [[-6.0, -1.0, -2.0], [-7.0, 5.0, 6.0]], [[3.0, -7.0, 2.0], [7.0, -7.0, 0.0]]], [[[-4.0, 0.0, 7.0], [0.0, -3.0, 7.0]], [[3.0, 2.0, 5.0], [-2.0, -3.0, 0.0]], [[1.0, 6.0, -7.0], [7.0, 0.0, -1.0]], [[4.0, 2.0, 7.0], [-1.0, -7.0, 0.0]], [[-1.0, -3.0, 2.0], [0.0, -5.0, 1.0]]], [[[4.0, 0.0, -4.0], [2.0, 4.0, -5.0]], [[3.0, 5.0, 0.0], [0.0, -7.0, 3.0]], [[-6.0, -5.0, 3.0], [-4.0, 5.0, 3.0]], [[7.0, -3.0, 6.0], [-7.0, 0.0, -3.0]], [[-3.0, 4.0, -7.0], [-1.0, -7.0, -4.0]]], [[[4.0, -2.0, -1.0], [1.0, -1.0, 5.0]], [[7.0, 4.0, -7.0], [1.0, 1.0, 0.0]], [[1.0, -2.0, 0.0], [-5.0, 2.0, -1.0]], [[3.0, -2.0, -3.0], [-3.0, -1.0, -7.0]], [[-7.0, 6.0, 0.0], [0.0, -3.0, 1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 7.0, -3.0], [7.0, 7.0, 4.0]], [[4.0, 1.0, -5.0], [-2.0, 1.0, -7.0]], [[-4.0, 2.0, 3.0], [5.0, 5.0, 4.0]], [[-2.0, 7.0, -7.0], [0.0, 0.0, 6.0]], [[-6.0, -6.0, -7.0], [2.0, 0.0, 7.0]]], [[[2.0, 4.0, 2.0], [7.0, -2.0, 4.0]], [[2.0, 2.0, -6.0], [0.0, 6.0, 1.0]], [[-3.0, -2.0, 6.0], [5.0, 6.0, -4.0]], [[1.0, 3.0, 0.0], [0.0, -3.0, 0.0]], [[0.0, 6.0, 0.0], [2.0, 5.0, -1.0]]], [[[1.0, 2.0, 0.0], [-4.0, 6.0, 4.0]], [[0.0, 2.0, 0.0], [-1.0, 5.0, 7.0]], [[2.0, -1.0, -5.0], [3.0, -3.0, -4.0]], [[-4.0, 2.0, -2.0], [2.0, 0.0, -6.0]], [[0.0, 5.0, -7.0], [7.0, -6.0, 0.0]]], [[[0.0, 6.0, 7.0], [-3.0, 0.0, -6.0]], [[-1.0, 5.0, -1.0], [1.0, -7.0, -7.0]], [[0.0, -3.0, -5.0], [0.0, -3.0, 0.0]], [[1.0, 6.0, 1.0], [0.0, -1.0, -5.0]], [[-4.0, 5.0, 3.0], [4.0, -7.0, 6.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      s0=numpy.array([[[[6.0, -3.0, 6.0], [-2.0, 2.0, -1.0]], [[4.0, 1.0, -4.0], [0.0, 5.0, -1.0]], [[-6.0, -7.0, 7.0], [-3.0, 6.0, 7.0]], [[-6.0, -3.0, -2.0], [7.0, 2.0, -7.0]], [[0.0, -7.0, 3.0], [6.0, 0.0, 7.0]]], [[[1.0, -3.0, -4.0], [-2.0, -5.0, -3.0]], [[-6.0, 0.0, -1.0], [7.0, -7.0, 0.0]], [[3.0, 6.0, -2.0], [0.0, 0.0, 1.0]], [[5.0, 6.0, -6.0], [5.0, 1.0, -1.0]], [[6.0, -5.0, 0.0], [0.0, -4.0, 3.0]]], [[[-2.0, -6.0, 4.0], [2.0, -5.0, 5.0]], [[0.0, 2.0, 3.0], [-6.0, 2.0, 5.0]], [[-3.0, -4.0, 0.0], [-2.0, 7.0, -5.0]], [[4.0, 0.0, -2.0], [-3.0, -3.0, -5.0]], [[4.0, -1.0, -2.0], [0.0, -7.0, 5.0]]], [[[-6.0, 6.0, 0.0], [-1.0, -7.0, 7.0]], [[-3.0, 7.0, 3.0], [2.0, 1.0, 6.0]], [[-2.0, -1.0, 3.0], [-7.0, 4.0, 6.0]], [[3.0, 1.0, 1.0], [-2.0, 6.0, -2.0]], [[-5.0, 5.0, 5.0], [-5.0, -1.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[66.0, 29.0, 20.0, -27.0, 23.0], [5.0, 40.0, -67.0, 48.0, 4.0], [9.0, -14.0, 12.0, 104.0, -80.0], [15.0, -12.0, 27.0, 17.0, -67.0]], [[-19.0, 20.0, -65.0, 2.0, -38.0], [-66.0, -21.0, 39.0, -45.0, -41.0], [57.0, -21.0, -19.0, 4.0, -11.0], [8.0, 65.0, 13.0, 24.0, -38.0]], [[143.0, 24.0, 153.0, 122.0, -18.0], [104.0, -9.0, -125.0, -28.0, 18.0], [-69.0, -74.0, 155.0, 21.0, -126.0], [9.0, -116.0, 28.0, -71.0, -11.0]], [[4.0, 61.0, -48.0, -38.0, 34.0], [-45.0, -54.0, 46.0, -65.0, -6.0], [41.0, -68.0, 6.0, -73.0, 27.0], [-46.0, -31.0, -24.0, 20.0, 11.0]], [[13.0, 16.0, 41.0, 1.0, 97.0], [70.0, -11.0, -28.0, 1.0, 34.0], [-35.0, -14.0, 41.0, -24.0, -83.0], [52.0, -43.0, -23.0, -62.0, -35.0]]], [[[-20.0, -64.0, -23.0, -24.0, 37.0], [-38.0, -4.0, 0.0, 7.0, 22.0], [11.0, 14.0, -29.0, 15.0, 62.0], [2.0, 16.0, 10.0, 53.0, -13.0]], [[-66.0, -12.0, -6.0, -46.0, 78.0], [38.0, -16.0, 50.0, 11.0, 39.0], [-34.0, 31.0, -30.0, -97.0, 67.0], [-9.0, -35.0, -55.0, -29.0, 63.0]], [[-61.0, -1.0, -45.0, -14.0, -37.0], [-19.0, 11.0, 52.0, 10.0, -24.0], [15.0, 42.0, -51.0, -12.0, 25.0], [7.0, 59.0, -10.0, -4.0, 16.0]], [[-116.0, 22.0, -126.0, -60.0, -11.0], [-72.0, -16.0, 119.0, -22.0, -41.0], [63.0, 35.0, -96.0, -51.0, 43.0], [13.0, 107.0, -29.0, 12.0, -3.0]], [[-14.0, -47.0, -2.0, -33.0, 81.0], [9.0, 20.0, -27.0, 42.0, 32.0], [-7.0, 30.0, -22.0, 48.0, -22.0], [53.0, 18.0, 5.0, 11.0, -57.0]]], [[[23.0, -30.0, 76.0, 1.0, 93.0], [86.0, 13.0, -57.0, 41.0, 58.0], [-65.0, 14.0, 36.0, -1.0, -33.0], [32.0, -69.0, -15.0, -42.0, -2.0]], [[45.0, -18.0, 74.0, 74.0, -64.0], [50.0, 25.0, -56.0, 17.0, -5.0], [-41.0, 11.0, 48.0, 39.0, -41.0], [10.0, -17.0, 25.0, -32.0, 11.0]], [[90.0, 44.0, 10.0, 41.0, -44.0], [-44.0, -34.0, -36.0, -67.0, -25.0], [37.0, -93.0, 66.0, 20.0, -34.0], [-38.0, -32.0, 34.0, 33.0, -29.0]], [[-13.0, -34.0, -42.0, -44.0, 8.0], [-56.0, 17.0, 2.0, 26.0, 2.0], [31.0, 18.0, -48.0, 52.0, 46.0], [-7.0, 36.0, 18.0, 65.0, -24.0]], [[-65.0, -76.0, -3.0, -24.0, 64.0], [26.0, 21.0, 7.0, 49.0, 35.0], [-29.0, 71.0, -45.0, 4.0, 27.0], [52.0, 31.0, -13.0, -8.0, -8.0]]], [[[-61.0, -64.0, 66.0, 44.0, -18.0], [94.0, 17.0, 16.0, 38.0, 30.0], [-89.0, 82.0, -4.0, -74.0, 64.0], [5.0, -26.0, -34.0, -69.0, 106.0]], [[-31.0, 30.0, 35.0, 32.0, -45.0], [72.0, 13.0, 26.0, 14.0, -11.0], [-46.0, 37.0, 7.0, -56.0, -17.0], [2.0, -11.0, -31.0, -81.0, 66.0]], [[83.0, -19.0, 104.0, 112.0, -70.0], [59.0, 9.0, -84.0, -10.0, -3.0], [-48.0, -21.0, 92.0, 38.0, -64.0], [10.0, -42.0, 37.0, -38.0, 2.0]], [[38.0, 43.0, -18.0, 11.0, -52.0], [-37.0, 2.0, -10.0, -19.0, -36.0], [38.0, -34.0, 12.0, 44.0, -44.0], [-9.0, 22.0, 25.0, 18.0, -35.0]], [[57.0, 16.0, 51.0, 9.0, -68.0], [16.0, 33.0, -39.0, 37.0, -1.0], [-24.0, -1.0, 17.0, 33.0, 36.0], [-69.0, -56.0, 14.0, 18.0, 62.0]]]])+(1.-msk_ref)*numpy.array([[[[-55.0, 4.0, -16.0, -81.0, -71.0], [-10.0, -19.0, 30.0, -9.0, -11.0], [16.0, -1.0, -23.0, -40.0, -83.0], [36.0, -36.0, -27.0, -3.0, -49.0]], [[42.0, 49.0, -5.0, 21.0, -9.0], [-10.0, 63.0, -4.0, -8.0, 32.0], [32.0, 20.0, 16.0, 0.0, 3.0], [-16.0, -23.0, 2.0, 6.0, -64.0]], [[-9.0, -103.0, 74.0, -44.0, 72.0], [-31.0, -25.0, 67.0, -45.0, -25.0], [56.0, 68.0, -95.0, -52.0, -141.0], [-26.0, -130.0, -32.0, -82.0, -2.0]], [[32.0, 20.0, 29.0, -37.0, 33.0], [-11.0, -1.0, 87.0, -21.0, 13.0], [-56.0, -52.0, 44.0, 78.0, 36.0], [-11.0, 35.0, 13.0, 7.0, -25.0]], [[12.0, -83.0, 53.0, -28.0, 82.0], [48.0, -25.0, 34.0, -21.0, -37.0], [-10.0, 29.0, -18.0, -50.0, -14.0], [-81.0, -81.0, 6.0, -74.0, 40.0]]], [[[-72.0, 41.0, -69.0, -13.0, 15.0], [-34.0, -13.0, -49.0, 7.0, -44.0], [-39.0, -50.0, 46.0, 12.0, 29.0], [-22.0, 42.0, 44.0, -1.0, -22.0]], [[15.0, -40.0, 21.0, 19.0, 57.0], [49.0, -48.0, 5.0, 15.0, -21.0], [-76.0, -42.0, 35.0, 40.0, 98.0], [-28.0, 63.0, 26.0, 0.0, 98.0]], [[46.0, 21.0, -2.0, 56.0, -33.0], [30.0, 31.0, -37.0, 21.0, 35.0], [19.0, 19.0, 6.0, -2.0, 44.0], [16.0, 22.0, -8.0, 32.0, 18.0]], [[88.0, 54.0, 0.0, 68.0, -21.0], [51.0, 63.0, -28.0, 20.0, 52.0], [-1.0, 5.0, 50.0, 20.0, 101.0], [-15.0, 36.0, 9.0, 39.0, -1.0]], [[-63.0, -6.0, -42.0, -29.0, 15.0], [12.0, -19.0, -44.0, 3.0, -53.0], [-16.0, -9.0, 17.0, -52.0, -1.0], [-48.0, -24.0, 27.0, -35.0, -3.0]]], [[[-51.0, -78.0, 13.0, -36.0, 59.0], [24.0, -65.0, 2.0, -5.0, -62.0], [-32.0, -4.0, -17.0, -38.0, -14.0], [-44.0, -30.0, 13.0, -54.0, 63.0]], [[-3.0, -34.0, 13.0, 23.0, -10.0], [-12.0, 3.0, -24.0, 0.0, 5.0], [60.0, 55.0, -61.0, -44.0, -65.0], [21.0, -48.0, -27.0, -12.0, 11.0]], [[-7.0, 30.0, 9.0, -52.0, 3.0], [-70.0, 23.0, 69.0, -36.0, 12.0], [19.0, -6.0, -9.0, 30.0, -76.0], [12.0, -33.0, -9.0, -9.0, -95.0]], [[-64.0, 64.0, -72.0, -24.0, -51.0], [-31.0, -3.0, -37.0, 13.0, -16.0], [-22.0, -47.0, 38.0, 12.0, 11.0], [25.0, 51.0, 19.0, 30.0, -43.0]], [[-38.0, -17.0, -39.0, 29.0, 31.0], [34.0, -19.0, -84.0, 22.0, -46.0], [-20.0, -2.0, 20.0, -44.0, 51.0], [-50.0, 7.0, 34.0, -22.0, 52.0]]], [[[26.0, -72.0, 24.0, 96.0, 47.0], [47.0, -35.0, -69.0, 33.0, -8.0], [-4.0, 27.0, -28.0, -8.0, 65.0], [-3.0, 35.0, 3.0, 2.0, 141.0]], [[91.0, -65.0, 74.0, 70.0, 1.0], [64.0, 2.0, 5.0, 15.0, 45.0], [33.0, 59.0, -49.0, -12.0, 22.0], [21.0, -12.0, -39.0, 11.0, 93.0]], [[-9.0, -48.0, 24.0, 12.0, 25.0], [-35.0, 6.0, -9.0, -17.0, -6.0], [72.0, 67.0, -75.0, -50.0, -99.0], [0.0, -83.0, -24.0, -39.0, -8.0]], [[18.0, 32.0, 5.0, -18.0, -49.0], [-22.0, 36.0, 29.0, -12.0, 34.0], [41.0, 20.0, -16.0, -4.0, -52.0], [31.0, -29.0, -26.0, 14.0, -66.0]], [[-36.0, 11.0, -9.0, -26.0, -87.0], [-37.0, -42.0, 28.0, 13.0, 21.0], [-5.0, -32.0, -28.0, 46.0, -39.0], [116.0, 69.0, -37.0, 61.0, 11.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_array_rank1(self):
      arg0=Data(numpy.array([[1.0, 5.0, 6.0], [6.0, 6.0, -3.0]]),self.functionspace)
      arg1=numpy.array([4.0, 3.0, -4.0])
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([-5.0, 54.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_array_rank2(self):
      arg0=Data(numpy.array([[5.0, 3.0, 4.0], [-7.0, 6.0, 7.0]]),self.functionspace)
      arg1=numpy.array([[5.0, 0.0, 2.0], [-6.0, 7.0, 7.0]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[33.0, 19.0], [-21.0, 133.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_Symbol_rank1(self):
      arg0=Data(numpy.array([[4.0, -3.0, -1.0], [0.0, 3.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([-5.0, 7.0, -1.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-40.0, 21.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_Symbol_rank2(self):
      arg0=Data(numpy.array([[5.0, 2.0, 1.0], [-5.0, 4.0, 5.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[0.0, -5.0, -2.0], [0.0, -5.0, 1.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-12.0, -9.0], [-30.0, -15.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_constData_rank1(self):
      arg0=Data(numpy.array([[-7.0, 4.0, -4.0], [-7.0, -6.0, 0.0]]),self.functionspace)
      arg1=Data(numpy.array([-5.0, 5.0, 0.0]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([55.0, 5.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_constData_rank2(self):
      arg0=Data(numpy.array([[3.0, 6.0, -7.0], [-1.0, 0.0, -1.0]]),self.functionspace)
      arg1=Data(numpy.array([[5.0, -1.0, 0.0], [-7.0, -6.0, 0.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[9.0, -57.0], [-5.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_expandedData_rank1(self):
      arg0=Data(numpy.array([[0.0, 7.0, -7.0], [2.0, -4.0, -5.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 6.0, 2.0])+(1.-msk_arg1)*numpy.array([1.0, 7.0, 0.0])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([28.0, -34.0])+(1.-msk_ref)*numpy.array([49.0, -26.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank2_expandedData_rank2(self):
      arg0=Data(numpy.array([[-3.0, 5.0, 0.0], [-5.0, -1.0, -6.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, -4.0, -1.0], [-6.0, 0.0, 4.0]])+(1.-msk_arg1)*numpy.array([[2.0, -6.0, -6.0], [3.0, -3.0, 6.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-26.0, 18.0], [0.0, 6.0]])+(1.-msk_ref)*numpy.array([[-36.0, -24.0], [32.0, -48.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_array_rank2(self):
      arg0=Data(numpy.array([[[[-6.0, -7.0, 6.0], [1.0, 5.0, 6.0]], [[0.0, -3.0, 4.0], [0.0, 1.0, 1.0]], [[3.0, 7.0, -6.0], [-7.0, -4.0, -5.0]], [[-1.0, -4.0, 7.0], [5.0, -3.0, -5.0]], [[-1.0, -4.0, 0.0], [-5.0, 2.0, -4.0]]], [[[2.0, 4.0, 1.0], [0.0, -1.0, -2.0]], [[2.0, -1.0, 1.0], [7.0, 6.0, 5.0]], [[3.0, 6.0, 6.0], [0.0, -5.0, -3.0]], [[3.0, -1.0, 4.0], [-7.0, 0.0, 4.0]], [[0.0, 0.0, -1.0], [7.0, 6.0, 2.0]]], [[[1.0, 4.0, -3.0], [0.0, -3.0, 2.0]], [[3.0, -6.0, 5.0], [7.0, -1.0, 1.0]], [[-3.0, -3.0, 0.0], [-1.0, 0.0, 0.0]], [[-2.0, 1.0, 6.0], [6.0, 7.0, 0.0]], [[2.0, 5.0, 7.0], [-3.0, -3.0, 3.0]]], [[[4.0, -5.0, 0.0], [-1.0, 1.0, 4.0]], [[1.0, 5.0, -6.0], [-4.0, 2.0, -3.0]], [[-3.0, -1.0, 0.0], [-2.0, 0.0, 1.0]], [[-6.0, 2.0, 4.0], [0.0, -6.0, 0.0]], [[-1.0, 1.0, -6.0], [7.0, 7.0, -1.0]]]]),self.functionspace)
      arg1=numpy.array([[-5.0, -1.0, -7.0], [1.0, -5.0, -7.0]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[-71.0, -37.0, 68.0, 15.0, 22.0], [-2.0, -74.0, -17.0, -77.0, -30.0], [13.0, -39.0, 17.0, -62.0, -73.0], [-49.0, 39.0, 7.0, 30.0, 25.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_array_rank3(self):
      arg0=Data(numpy.array([[[[0.0, -5.0, 0.0], [-5.0, -1.0, 5.0]], [[-7.0, -5.0, 1.0], [-6.0, 3.0, -6.0]], [[7.0, -3.0, 0.0], [-6.0, 1.0, -4.0]], [[5.0, 0.0, 0.0], [5.0, 1.0, 7.0]], [[-2.0, -4.0, 1.0], [0.0, 2.0, -4.0]]], [[[2.0, 1.0, 7.0], [-6.0, 0.0, 6.0]], [[-4.0, 7.0, 3.0], [-7.0, 5.0, 6.0]], [[6.0, 0.0, -5.0], [0.0, -1.0, 0.0]], [[-6.0, 7.0, -6.0], [4.0, -5.0, -3.0]], [[0.0, 0.0, -2.0], [0.0, -1.0, 6.0]]], [[[-6.0, 0.0, -5.0], [-3.0, -1.0, 5.0]], [[-7.0, -4.0, -5.0], [0.0, -2.0, -3.0]], [[1.0, 3.0, 2.0], [0.0, 3.0, 1.0]], [[3.0, 6.0, 0.0], [1.0, 6.0, 3.0]], [[-4.0, -4.0, -6.0], [6.0, 2.0, -5.0]]], [[[-6.0, 6.0, -5.0], [5.0, -2.0, -7.0]], [[-1.0, -4.0, 1.0], [0.0, -5.0, 0.0]], [[-5.0, -6.0, -5.0], [-6.0, 5.0, -3.0]], [[0.0, -3.0, -5.0], [7.0, 5.0, -7.0]], [[-4.0, 5.0, 6.0], [4.0, 4.0, 2.0]]]]),self.functionspace)
      arg1=numpy.array([[[1.0, 7.0, 1.0], [0.0, -1.0, 2.0]], [[4.0, -6.0, -3.0], [-7.0, 3.0, -7.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[-24.0, 27.0], [-56.0, 92.0], [-23.0, 119.0], [18.0, -61.0], [-39.0, 47.0]], [[28.0, -19.0], [55.0, -45.0], [2.0, 36.0], [36.0, -70.0], [11.0, -39.0]], [[0.0, -26.0], [-44.0, 26.0], [23.0, -18.0], [45.0, -34.0], [-50.0, 25.0]], [[19.0, -37.0], [-23.0, 2.0], [-63.0, 109.0], [-45.0, 48.0], [37.0, -94.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_array_rank4(self):
      arg0=Data(numpy.array([[[[-6.0, 2.0, 0.0], [-5.0, -1.0, 5.0]], [[7.0, 7.0, 4.0], [0.0, 6.0, 6.0]], [[2.0, 4.0, -1.0], [-6.0, 0.0, -3.0]], [[-1.0, -2.0, 3.0], [1.0, 6.0, -1.0]], [[6.0, 0.0, -5.0], [-5.0, -4.0, 1.0]]], [[[5.0, 7.0, 7.0], [5.0, -5.0, -7.0]], [[3.0, 3.0, 5.0], [-3.0, -5.0, -4.0]], [[6.0, 0.0, 0.0], [-3.0, 3.0, 7.0]], [[7.0, -4.0, -7.0], [-1.0, 0.0, 2.0]], [[2.0, 0.0, -3.0], [4.0, 0.0, 5.0]]], [[[6.0, 2.0, -1.0], [-2.0, 7.0, 4.0]], [[3.0, 0.0, -5.0], [-6.0, -3.0, 1.0]], [[-1.0, -6.0, -2.0], [-3.0, 2.0, -6.0]], [[-7.0, -2.0, -6.0], [-1.0, 3.0, 0.0]], [[6.0, -2.0, -2.0], [4.0, -1.0, -2.0]]], [[[-2.0, -5.0, -6.0], [-7.0, 7.0, -5.0]], [[5.0, -5.0, 1.0], [-2.0, -4.0, 3.0]], [[2.0, -1.0, -4.0], [0.0, -2.0, -4.0]], [[0.0, 2.0, 4.0], [-6.0, -3.0, 6.0]], [[4.0, -4.0, 5.0], [7.0, 3.0, -3.0]]]]),self.functionspace)
      arg1=numpy.array([[[[-2.0, -4.0, 4.0], [7.0, 4.0, 3.0]], [[0.0, -6.0, 2.0], [-2.0, 0.0, -2.0]], [[3.0, 1.0, -4.0], [4.0, -3.0, -6.0]], [[-4.0, -5.0, 3.0], [5.0, -5.0, 5.0]], [[0.0, 0.0, 0.0], [-2.0, 4.0, -5.0]]], [[[2.0, -1.0, 6.0], [-6.0, -4.0, -1.0]], [[-5.0, 4.0, 3.0], [-6.0, -4.0, 4.0]], [[-6.0, 0.0, 4.0], [-5.0, 2.0, 2.0]], [[1.0, 0.0, 6.0], [-1.0, 1.0, -3.0]], [[-5.0, 6.0, -3.0], [5.0, 1.0, 5.0]]], [[[7.0, 0.0, 2.0], [3.0, 6.0, -4.0]], [[4.0, 0.0, -3.0], [3.0, 0.0, -5.0]], [[-1.0, -7.0, -3.0], [1.0, -5.0, 1.0]], [[5.0, 4.0, 0.0], [3.0, -6.0, -2.0]], [[0.0, -1.0, 6.0], [0.0, 3.0, 4.0]]], [[[4.0, 4.0, -5.0], [-3.0, 2.0, -6.0]], [[-2.0, 1.0, 3.0], [-7.0, -7.0, 4.0]], [[-5.0, 7.0, -4.0], [-2.0, -5.0, 0.0]], [[7.0, -4.0, -2.0], [-2.0, 5.0, -1.0]], [[-4.0, 5.0, 0.0], [-3.0, 5.0, -3.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-20.0, -12.0, -63.0, 19.0, -19.0], [15.0, 92.0, 69.0, -17.0, 41.0], [-83.0, -64.0, -3.0, -41.0, 15.0], [-33.0, 76.0, 59.0, -50.0, 29.0]], [[16.0, -46.0, -42.0, -51.0, -6.0], [1.0, 5.0, -2.0, 19.0, 31.0], [69.0, -14.0, -92.0, 15.0, 59.0], [12.0, -13.0, -32.0, 37.0, 19.0]], [[-75.0, -8.0, 8.0, -76.0, 27.0], [33.0, 27.0, 8.0, 11.0, -28.0], [6.0, 8.0, -36.0, 14.0, -22.0], [65.0, 27.0, 34.0, 15.0, 39.0]], [[50.0, 18.0, -25.0, -7.0, 27.0], [-11.0, -28.0, 23.0, 25.0, -10.0], [42.0, -5.0, -24.0, -44.0, 34.0], [-12.0, -44.0, -53.0, 24.0, 24.0]], [[-80.0, -2.0, 24.0, -39.0, -11.0], [27.0, 5.0, -37.0, -26.0, -39.0], [-11.0, 19.0, 25.0, 37.0, -38.0], [50.0, 40.0, 20.0, 41.0, -32.0]]], [[[-16.0, -24.0, 71.0, -19.0, 5.0], [42.0, -14.0, -51.0, 58.0, -19.0], [62.0, 49.0, -52.0, 112.0, -8.0], [30.0, -10.0, 11.0, -35.0, -4.0]], [[-51.0, 6.0, 19.0, -22.0, 6.0], [75.0, 34.0, -1.0, 43.0, -52.0], [8.0, 8.0, -21.0, 56.0, -4.0], [22.0, 52.0, 17.0, -16.0, -1.0]], [[0.0, -8.0, -45.0, -19.0, -17.0], [11.0, 4.0, -1.0, -9.0, -7.0], [23.0, -20.0, -17.0, -11.0, 37.0], [-3.0, 16.0, -39.0, 56.0, -21.0]], [[-27.0, 8.0, 29.0, -24.0, -8.0], [-20.0, -58.0, -61.0, -40.0, -33.0], [24.0, 36.0, 43.0, 12.0, -30.0], [38.0, -24.0, -33.0, 79.0, -51.0]], [[27.0, -24.0, 4.0, 28.0, -33.0], [-43.0, -23.0, -34.0, -35.0, 44.0], [0.0, 4.0, 16.0, 12.0, 2.0], [-19.0, -21.0, -6.0, 7.0, -35.0]]], [[[2.0, -18.0, -29.0, -62.0, 12.0], [-16.0, -25.0, -8.0, -3.0, 2.0], [60.0, 1.0, -50.0, -18.0, 29.0], [33.0, -32.0, -43.0, 71.0, 15.0]], [[-77.0, 0.0, 8.0, -37.0, -5.0], [23.0, 22.0, -12.0, -27.0, -28.0], [-29.0, 4.0, 22.0, 13.0, -35.0], [43.0, 46.0, 32.0, 27.0, -12.0]], [[-13.0, 50.0, 17.0, -27.0, 44.0], [8.0, -39.0, 5.0, 10.0, -68.0], [16.0, 23.0, 30.0, -38.0, -24.0], [31.0, -27.0, -33.0, 43.0, 11.0]], [[3.0, 2.0, -12.0, 0.0, 14.0], [-54.0, 3.0, 29.0, -39.0, 39.0], [-46.0, -13.0, 23.0, -64.0, -25.0], [3.0, -20.0, 32.0, -12.0, 36.0]], [[6.0, 4.0, 55.0, -5.0, -2.0], [-16.0, -72.0, -70.0, -5.0, -27.0], [52.0, 52.0, 21.0, 44.0, -21.0], [24.0, -49.0, -39.0, 43.0, -45.0]]], [[[-36.0, 42.0, -6.0, -80.0, 67.0], [-16.0, -34.0, 27.0, -9.0, -55.0], [15.0, 14.0, 8.0, -83.0, -30.0], [67.0, -39.0, -22.0, 72.0, 54.0]], [[-7.0, 30.0, -8.0, 33.0, -27.0], [46.0, -2.0, -18.0, 0.0, -57.0], [-5.0, -4.0, 48.0, 17.0, 11.0], [-25.0, 42.0, -40.0, 34.0, -68.0]], [[-36.0, 6.0, 51.0, -25.0, 12.0], [-7.0, -34.0, -40.0, -12.0, -26.0], [10.0, 40.0, 23.0, 26.0, -45.0], [44.0, -19.0, 9.0, 20.0, -11.0]], [[-28.0, -4.0, -65.0, 17.0, -30.0], [64.0, 92.0, 52.0, 9.0, -3.0], [-52.0, -60.0, -11.0, -4.0, 37.0], [-36.0, 101.0, 25.0, -25.0, -5.0]], [[80.0, 26.0, 25.0, 24.0, 13.0], [-9.0, -87.0, -39.0, 39.0, -36.0], [89.0, 37.0, -2.0, 13.0, 31.0], [-22.0, -79.0, -97.0, 38.0, -33.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_Symbol_rank2(self):
      arg0=Data(numpy.array([[[[-7.0, 4.0, -7.0], [6.0, 0.0, 4.0]], [[2.0, -4.0, -6.0], [-7.0, -1.0, 2.0]], [[-7.0, 7.0, -5.0], [1.0, -2.0, -1.0]], [[0.0, 6.0, -5.0], [-4.0, 4.0, 4.0]], [[4.0, -5.0, 7.0], [-5.0, -1.0, -2.0]]], [[[-2.0, -1.0, 6.0], [2.0, 5.0, -2.0]], [[-7.0, 2.0, -3.0], [4.0, -6.0, 4.0]], [[-1.0, -3.0, 0.0], [-4.0, -1.0, 3.0]], [[2.0, 3.0, -4.0], [7.0, 4.0, 0.0]], [[-7.0, -2.0, 2.0], [7.0, 2.0, 0.0]]], [[[7.0, 7.0, 1.0], [-1.0, 3.0, 7.0]], [[1.0, 6.0, -1.0], [2.0, -2.0, 0.0]], [[4.0, -2.0, -4.0], [4.0, 5.0, 0.0]], [[-5.0, 7.0, 2.0], [-7.0, 5.0, -5.0]], [[4.0, -7.0, 1.0], [-1.0, 0.0, 7.0]]], [[[-1.0, 4.0, -7.0], [-7.0, 0.0, -5.0]], [[1.0, 3.0, 0.0], [-1.0, 0.0, 2.0]], [[-2.0, -3.0, -1.0], [-2.0, 2.0, 7.0]], [[-3.0, 3.0, -6.0], [0.0, -6.0, -1.0]], [[-4.0, 2.0, -5.0], [-6.0, 1.0, 1.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[0.0, -2.0, -2.0], [1.0, -1.0, 5.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[32.0, 24.0, -6.0, 10.0, -18.0], [-23.0, 32.0, 18.0, 5.0, 5.0], [15.0, -6.0, 11.0, -55.0, 46.0], [-26.0, 3.0, 39.0, 7.0, 4.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_Symbol_rank3(self):
      arg0=Data(numpy.array([[[[-2.0, 1.0, -2.0], [6.0, -7.0, -6.0]], [[-7.0, -7.0, -2.0], [5.0, -7.0, -7.0]], [[-1.0, -4.0, 0.0], [-4.0, -7.0, 7.0]], [[-5.0, 6.0, 7.0], [1.0, -7.0, 3.0]], [[7.0, 1.0, 1.0], [-4.0, 2.0, -2.0]]], [[[7.0, -6.0, 0.0], [2.0, 0.0, 7.0]], [[-7.0, 6.0, 2.0], [1.0, 4.0, 0.0]], [[7.0, 2.0, 1.0], [-3.0, 2.0, -4.0]], [[7.0, 0.0, 0.0], [-3.0, -4.0, 5.0]], [[-5.0, 6.0, -1.0], [-6.0, 0.0, 1.0]]], [[[5.0, -7.0, -2.0], [6.0, -3.0, 1.0]], [[0.0, 0.0, 0.0], [5.0, 1.0, -7.0]], [[3.0, 7.0, 2.0], [0.0, 6.0, -6.0]], [[-1.0, 2.0, -5.0], [3.0, 3.0, -5.0]], [[6.0, -7.0, 6.0], [7.0, 2.0, -5.0]]], [[[-1.0, -5.0, 0.0], [0.0, -5.0, 3.0]], [[6.0, -4.0, -5.0], [3.0, 5.0, 4.0]], [[6.0, 4.0, -7.0], [7.0, 5.0, 6.0]], [[-3.0, -3.0, 4.0], [-5.0, -7.0, -2.0]], [[-4.0, 0.0, 5.0], [-1.0, -3.0, 7.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[5.0, -1.0, 2.0], [-2.0, 0.0, 3.0]], [[7.0, -3.0, 5.0], [1.0, 4.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-45.0, -19.0], [-63.0, -26.0], [28.0, -62.0], [-10.0, -60.0], [38.0, 65.0]], [[58.0, 34.0], [-39.0, -40.0], [29.0, 73.0], [56.0, 5.0], [-18.0, -69.0]], [[19.0, 35.0], [-31.0, 44.0], [-6.0, 64.0], [-38.0, 2.0], [20.0, 133.0]], [[9.0, -27.0], [30.0, 32.0], [16.0, -8.0], [0.0, -15.0], [13.0, -51.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_Symbol_rank4(self):
      arg0=Data(numpy.array([[[[-4.0, 0.0, -7.0], [6.0, 0.0, -5.0]], [[3.0, 7.0, 2.0], [-7.0, -1.0, -3.0]], [[6.0, 7.0, 1.0], [-4.0, -2.0, -6.0]], [[5.0, -1.0, 2.0], [2.0, 7.0, -4.0]], [[3.0, 3.0, 5.0], [-3.0, 6.0, 5.0]]], [[[4.0, -4.0, 0.0], [7.0, 0.0, 7.0]], [[0.0, 0.0, 5.0], [6.0, 1.0, -6.0]], [[-2.0, -2.0, 5.0], [5.0, -4.0, 7.0]], [[0.0, -6.0, 4.0], [-4.0, 6.0, -5.0]], [[0.0, 4.0, 3.0], [-7.0, 4.0, -1.0]]], [[[7.0, -7.0, 4.0], [-3.0, 2.0, 4.0]], [[-2.0, 4.0, -3.0], [4.0, -3.0, 0.0]], [[-1.0, -6.0, 2.0], [4.0, -2.0, -2.0]], [[0.0, 1.0, -7.0], [-6.0, -1.0, -3.0]], [[-6.0, -7.0, 4.0], [-4.0, -5.0, -7.0]]], [[[3.0, 3.0, -7.0], [7.0, 5.0, -5.0]], [[3.0, 3.0, -2.0], [-4.0, 2.0, -5.0]], [[2.0, -4.0, -2.0], [4.0, -5.0, 1.0]], [[0.0, 5.0, -2.0], [-2.0, 1.0, 4.0]], [[0.0, 1.0, 4.0], [5.0, -7.0, -7.0]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[[0.0, 4.0, -3.0], [-3.0, 0.0, 5.0]], [[5.0, 3.0, -6.0], [3.0, -2.0, -5.0]], [[-7.0, -7.0, 5.0], [-1.0, 5.0, 7.0]], [[-1.0, -3.0, 0.0], [6.0, 6.0, 6.0]], [[-7.0, -4.0, -4.0], [-1.0, -7.0, -4.0]]], [[[2.0, -3.0, 4.0], [-3.0, 7.0, 5.0]], [[-2.0, 6.0, -5.0], [4.0, -7.0, -7.0]], [[-6.0, 5.0, -6.0], [-6.0, 5.0, -7.0]], [[2.0, 6.0, 4.0], [-2.0, 2.0, 0.0]], [[-4.0, -3.0, 6.0], [4.0, 4.0, -7.0]]], [[[4.0, -4.0, -7.0], [3.0, 6.0, -1.0]], [[0.0, -2.0, -7.0], [2.0, 4.0, -6.0]], [[1.0, 6.0, 5.0], [5.0, -1.0, 0.0]], [[0.0, 7.0, 2.0], [1.0, -1.0, 0.0]], [[3.0, 2.0, 2.0], [-1.0, -7.0, 5.0]]], [[[7.0, -6.0, -6.0], [2.0, 6.0, 3.0]], [[1.0, 6.0, 2.0], [-6.0, -3.0, 3.0]], [[-6.0, -3.0, 0.0], [1.0, 4.0, 0.0]], [[-4.0, 0.0, 7.0], [-2.0, 5.0, 2.0]], [[0.0, 3.0, -2.0], [-6.0, -2.0, -5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-22.0, 65.0, -48.0, 10.0, 70.0], [-79.0, 102.0, 65.0, -48.0, 33.0], [56.0, 91.0, -9.0, -8.0, -57.0], [11.0, -69.0, 30.0, -55.0, 3.0]], [[28.0, 20.0, -79.0, -90.0, -31.0], [-8.0, 26.0, 63.0, 68.0, -32.0], [-54.0, -28.0, 21.0, 47.0, 26.0], [-62.0, 85.0, -50.0, 5.0, 76.0]], [[7.0, 67.0, -134.0, -99.0, -32.0], [-37.0, 65.0, 49.0, 62.0, -21.0], [-29.0, -1.0, 35.0, 49.0, 22.0], [-44.0, 62.0, -69.0, -31.0, 77.0]], [[-36.0, 22.0, -13.0, 28.0, -74.0], [44.0, -39.0, 4.0, 22.0, 59.0], [62.0, 44.0, 12.0, -8.0, -54.0], [63.0, -42.0, 3.0, 17.0, -13.0]], [[31.0, -52.0, 51.0, 36.0, -112.0], [93.0, -102.0, -20.0, 62.0, -14.0], [-13.0, -53.0, 25.0, 22.0, 11.0], [18.0, 46.0, -6.0, 69.0, -20.0]]], [[[-2.0, -6.0, 42.0, 92.0, -47.0], [34.0, -53.0, -135.0, -30.0, -25.0], [46.0, -20.0, 15.0, -21.0, 32.0], [87.0, -41.0, -5.0, -16.0, -89.0]], [[-63.0, 16.0, -18.0, 6.0, -9.0], [-21.0, 34.0, -19.0, 10.0, 100.0], [-5.0, 17.0, 54.0, 15.0, -33.0], [-30.0, -47.0, 10.0, 16.0, -18.0]], [[-3.0, -58.0, 77.0, 56.0, -3.0], [14.0, -34.0, -127.0, -14.0, -1.0], [-51.0, -79.0, 40.0, 5.0, 58.0], [-25.0, -1.0, 7.0, 27.0, -73.0]], [[-49.0, -41.0, 61.0, 0.0, -10.0], [63.0, -79.0, 35.0, 0.0, 85.0], [25.0, 30.0, -42.0, -44.0, -67.0], [25.0, -37.0, 38.0, 56.0, 11.0]], [[23.0, -30.0, 7.0, -36.0, -45.0], [44.0, -40.0, 71.0, 58.0, 1.0], [-33.0, -21.0, 0.0, 23.0, -12.0], [-35.0, 57.0, -3.0, 53.0, 45.0]]], [[[-11.0, -43.0, 61.0, 32.0, -64.0], [94.0, -130.0, -101.0, -2.0, -15.0], [27.0, -36.0, -32.0, -46.0, 24.0], [85.0, -3.0, -16.0, 24.0, -35.0]], [[13.0, 38.0, -48.0, -4.0, 27.0], [-61.0, 80.0, 11.0, -6.0, -18.0], [-9.0, 9.0, 30.0, 29.0, 13.0], [-30.0, 1.0, -8.0, -36.0, 0.0]], [[-52.0, -9.0, 31.0, 19.0, 41.0], [-12.0, 0.0, -56.0, -42.0, 56.0], [8.0, 10.0, -5.0, -32.0, -11.0], [7.0, -57.0, 20.0, -4.0, -32.0]], [[28.0, 44.0, -62.0, -63.0, 49.0], [-35.0, 45.0, 99.0, -12.0, -52.0], [24.0, 49.0, -58.0, -12.0, -14.0], [9.0, 22.0, -13.0, -48.0, 70.0]], [[-63.0, -42.0, 41.0, -69.0, 121.0], [-33.0, 18.0, 25.0, -40.0, 82.0], [-59.0, 0.0, -43.0, -40.0, -20.0], [-83.0, -22.0, 33.0, 21.0, 40.0]]], [[[-13.0, 102.0, -94.0, 30.0, -27.0], [-42.0, 75.0, 57.0, -8.0, 20.0], [105.0, 107.0, 16.0, 9.0, -66.0], [74.0, -65.0, 0.0, -60.0, -4.0]], [[5.0, 45.0, -73.0, -54.0, -15.0], [-10.0, 27.0, 78.0, 28.0, -6.0], [19.0, 38.0, -11.0, 11.0, -24.0], [4.0, 20.0, -23.0, -18.0, 58.0]], [[-17.0, 27.0, -18.0, 10.0, 37.0], [-34.0, 26.0, -76.0, -46.0, -19.0], [19.0, 4.0, -7.0, -23.0, 30.0], [31.0, -32.0, -16.0, -53.0, -27.0]], [[52.0, -1.0, -10.0, 3.0, -33.0], [10.0, -3.0, 26.0, 28.0, -59.0], [-10.0, -20.0, 9.0, 28.0, 21.0], [-4.0, 47.0, -13.0, 3.0, 9.0]], [[-58.0, 43.0, -76.0, -57.0, 52.0], [-86.0, 104.0, -35.0, -2.0, 62.0], [-52.0, -6.0, 58.0, 27.0, 19.0], [-83.0, -16.0, -26.0, -31.0, 14.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_constData_rank2(self):
      arg0=Data(numpy.array([[[[1.0, 4.0, 1.0], [-3.0, -3.0, 6.0]], [[-2.0, 6.0, 1.0], [-1.0, -1.0, 0.0]], [[-7.0, -6.0, 6.0], [-1.0, 2.0, -7.0]], [[5.0, 3.0, 0.0], [1.0, 3.0, -4.0]], [[6.0, -3.0, -5.0], [3.0, -1.0, -3.0]]], [[[-1.0, 5.0, 5.0], [5.0, 2.0, 1.0]], [[0.0, 2.0, 1.0], [1.0, 4.0, 3.0]], [[-6.0, 2.0, 1.0], [-3.0, -4.0, -3.0]], [[3.0, -7.0, 0.0], [4.0, 7.0, -4.0]], [[-1.0, 3.0, -2.0], [7.0, -5.0, 2.0]]], [[[5.0, -1.0, -5.0], [4.0, 6.0, 0.0]], [[-1.0, -2.0, -6.0], [1.0, 0.0, 0.0]], [[-6.0, -3.0, 4.0], [0.0, 0.0, 2.0]], [[-7.0, -5.0, 5.0], [6.0, -7.0, 0.0]], [[-5.0, 0.0, 4.0], [-4.0, -1.0, -3.0]]], [[[3.0, -2.0, 1.0], [-1.0, -7.0, 7.0]], [[-6.0, 4.0, 6.0], [-6.0, 0.0, 4.0]], [[4.0, 0.0, 3.0], [6.0, 7.0, -1.0]], [[-5.0, 0.0, 3.0], [4.0, -3.0, 1.0]], [[6.0, -1.0, 5.0], [-7.0, 5.0, 3.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[3.0, 6.0, 6.0], [-5.0, 3.0, -1.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[33.0, 38.0, -3.0, 41.0, -45.0], [37.0, 22.0, 6.0, -28.0, -49.0], [-23.0, -56.0, -14.0, -72.0, 29.0], [-20.0, 68.0, 22.0, -27.0, 89.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_constData_rank3(self):
      arg0=Data(numpy.array([[[[-7.0, -5.0, -3.0], [-5.0, 3.0, -5.0]], [[7.0, 6.0, 1.0], [-3.0, 3.0, -5.0]], [[2.0, -6.0, 5.0], [-1.0, 1.0, -2.0]], [[5.0, -7.0, -1.0], [2.0, -4.0, 1.0]], [[-5.0, -1.0, 7.0], [2.0, 7.0, -5.0]]], [[[-5.0, 4.0, 0.0], [4.0, 4.0, 7.0]], [[0.0, 1.0, -5.0], [0.0, 4.0, -7.0]], [[7.0, 6.0, -2.0], [-1.0, 0.0, 6.0]], [[1.0, -6.0, -4.0], [-1.0, 3.0, -7.0]], [[-1.0, -2.0, -5.0], [-6.0, -2.0, 1.0]]], [[[7.0, 0.0, -5.0], [5.0, 0.0, 0.0]], [[0.0, -6.0, -7.0], [0.0, -2.0, 2.0]], [[-5.0, 0.0, -3.0], [-2.0, 6.0, 6.0]], [[2.0, 1.0, -3.0], [-6.0, 5.0, 3.0]], [[0.0, 0.0, -1.0], [-3.0, 7.0, -7.0]]], [[[-4.0, 6.0, 7.0], [-7.0, 0.0, 7.0]], [[0.0, 2.0, -2.0], [7.0, 6.0, -5.0]], [[0.0, -4.0, -5.0], [7.0, 4.0, 0.0]], [[5.0, 2.0, 7.0], [-1.0, -2.0, -7.0]], [[7.0, 4.0, -6.0], [3.0, 6.0, 3.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[-3.0, -7.0, 2.0], [1.0, 4.0, 3.0]], [[-5.0, -1.0, 0.0], [5.0, 2.0, 3.0]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[42.0, 6.0], [-67.0, -65.0], [43.0, -13.0], [21.0, -13.0], [51.0, 35.0]], [[28.0, 70.0], [-22.0, -14.0], [-50.0, -28.0], [21.0, -19.0], [-4.0, -24.0]], [[-26.0, -10.0], [26.0, 8.0], [49.0, 45.0], [4.0, -22.0], [2.0, -22.0]], [[-2.0, 0.0], [-2.0, 30.0], [41.0, 47.0], [-45.0, -57.0], [-25.0, -3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_constData_rank4(self):
      arg0=Data(numpy.array([[[[-2.0, 0.0, -2.0], [-5.0, 3.0, -2.0]], [[4.0, 2.0, 0.0], [-7.0, -2.0, -7.0]], [[-7.0, 0.0, -2.0], [2.0, -4.0, 1.0]], [[2.0, -3.0, -7.0], [-1.0, -4.0, -3.0]], [[-3.0, 5.0, 2.0], [7.0, 0.0, 7.0]]], [[[-4.0, 4.0, 0.0], [-5.0, 4.0, 7.0]], [[-5.0, 7.0, -7.0], [-2.0, -4.0, -5.0]], [[-6.0, -3.0, 0.0], [7.0, -1.0, -6.0]], [[3.0, -7.0, 4.0], [-1.0, 2.0, -3.0]], [[4.0, -2.0, -2.0], [0.0, -1.0, 0.0]]], [[[-7.0, -6.0, 0.0], [-1.0, 1.0, 0.0]], [[7.0, -2.0, -6.0], [-3.0, 3.0, 0.0]], [[3.0, 6.0, 2.0], [7.0, -3.0, 4.0]], [[-2.0, 7.0, -4.0], [-4.0, -5.0, -4.0]], [[-1.0, 1.0, 5.0], [-7.0, 2.0, -7.0]]], [[[1.0, -2.0, 6.0], [-1.0, 5.0, -4.0]], [[0.0, 0.0, -5.0], [-5.0, -7.0, 5.0]], [[0.0, -5.0, 7.0], [3.0, 7.0, 2.0]], [[3.0, 7.0, 6.0], [-3.0, -5.0, 0.0]], [[7.0, -3.0, -7.0], [-3.0, 0.0, -4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[2.0, -6.0, -3.0], [7.0, 0.0, 4.0]], [[4.0, -5.0, 3.0], [-1.0, -1.0, 0.0]], [[-7.0, 1.0, 0.0], [4.0, 0.0, 6.0]], [[-3.0, -3.0, -4.0], [-6.0, 4.0, 0.0]], [[6.0, 1.0, -7.0], [-6.0, -1.0, 6.0]]], [[[6.0, 2.0, 7.0], [5.0, 1.0, 1.0]], [[4.0, 3.0, 4.0], [3.0, 5.0, -1.0]], [[-3.0, 3.0, -5.0], [-7.0, -4.0, 6.0]], [[-2.0, 7.0, 3.0], [4.0, 5.0, -7.0]], [[-6.0, 0.0, -4.0], [-5.0, 2.0, 6.0]]], [[[3.0, 4.0, 5.0], [-4.0, -7.0, -7.0]], [[-3.0, -1.0, -5.0], [0.0, -6.0, 4.0]], [[-3.0, 3.0, 0.0], [-4.0, 4.0, 1.0]], [[4.0, 2.0, -4.0], [-7.0, -6.0, 4.0]], [[-6.0, 0.0, 0.0], [7.0, -5.0, 1.0]]], [[[0.0, 5.0, -7.0], [3.0, 5.0, -7.0]], [[4.0, 3.0, -6.0], [-5.0, 0.0, 2.0]], [[-5.0, 6.0, -4.0], [-4.0, 0.0, 7.0]], [[-3.0, -4.0, 4.0], [0.0, 6.0, 2.0]], [[3.0, 6.0, -5.0], [6.0, 1.0, 0.0]]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      ref=Data(numpy.array([[[[-41.0, -12.0, -18.0, 56.0, 17.0], [-50.0, -14.0, 27.0, 7.0, 39.0], [-3.0, -10.0, 36.0, 9.0, -40.0], [28.0, 25.0, 24.0, 12.0, -23.0]], [[-81.0, 15.0, -96.0, 16.0, 28.0], [-16.0, -2.0, 9.0, 17.0, -35.0], [111.0, -30.0, 7.0, 53.0, -70.0], [28.0, 43.0, -29.0, -46.0, -20.0]], [[10.0, -32.0, 63.0, 1.0, -30.0], [-49.0, -51.0, 39.0, -11.0, 38.0], [-18.0, 59.0, -2.0, -6.0, 77.0], [-7.0, -24.0, 42.0, -9.0, -3.0]], [[24.0, 7.0, -39.0, 21.0, 50.0], [-55.0, -49.0, 25.0, -49.0, -5.0], [12.0, 44.0, -30.0, 49.0, -2.0], [32.0, 40.0, -17.0, -52.0, 13.0]], [[35.0, -38.0, 96.0, -56.0, -27.0], [48.0, 25.0, 7.0, 26.0, 17.0], [-56.0, 22.0, 3.0, -31.0, 74.0], [-17.0, -30.0, 58.0, 11.0, 53.0]]], [[[-39.0, -35.0, 54.0, 46.0, 48.0], [-30.0, -6.0, 85.0, -13.0, 99.0], [-53.0, 12.0, 67.0, 31.0, -24.0], [-24.0, 35.0, 113.0, 34.0, -14.0]], [[-65.0, -70.0, 4.0, 18.0, 12.0], [-84.0, -48.0, 71.0, 45.0, 30.0], [49.0, 47.0, 23.0, 40.0, 31.0], [93.0, 43.0, 68.0, -75.0, 46.0]], [[31.0, -15.0, 31.0, -19.0, -116.0], [-14.0, -11.0, -72.0, 56.0, -37.0], [-9.0, 3.0, -29.0, -97.0, 84.0], [43.0, -80.0, -58.0, 12.0, 5.0]], [[17.0, 58.0, -50.0, 10.0, -31.0], [26.0, 17.0, -69.0, -16.0, -43.0], [12.0, -46.0, -21.0, -35.0, -38.0], [-35.0, -34.0, -90.0, 41.0, -57.0]], [[26.0, 21.0, -30.0, -2.0, 37.0], [5.0, -3.0, -4.0, -33.0, -18.0], [1.0, 6.0, -22.0, 26.0, -19.0], [-1.0, 22.0, -24.0, -18.0, 9.0]]], [[[15.0, 2.0, 39.0, 49.0, -43.0], [-58.0, -44.0, 6.0, -27.0, 49.0], [-48.0, 21.0, 11.0, -39.0, 30.0], [-28.0, -41.0, 3.0, 51.0, -62.0]], [[23.0, 20.0, -63.0, 39.0, 97.0], [-16.0, 4.0, 12.0, -43.0, 3.0], [-26.0, -7.0, -3.0, 51.0, -78.0], [38.0, 73.0, -11.0, -19.0, 24.0]], [[29.0, -16.0, 37.0, -89.0, -5.0], [80.0, 40.0, -14.0, 27.0, -43.0], [8.0, 9.0, -27.0, 1.0, 50.0], [-6.0, -9.0, 13.0, -35.0, 74.0]], [[-78.0, -46.0, -19.0, 5.0, 28.0], [-55.0, -36.0, 71.0, 28.0, 14.0], [81.0, 33.0, 19.0, 64.0, 5.0], [54.0, 49.0, 56.0, -76.0, 27.0]], [[-100.0, 11.0, -62.0, 30.0, -42.0], [-9.0, 15.0, -20.0, 55.0, -17.0], [89.0, -63.0, 35.0, -13.0, -60.0], [8.0, -10.0, -30.0, 17.0, -62.0]]], [[[-27.0, 28.0, -37.0, 5.0, -61.0], [40.0, 48.0, -76.0, 51.0, -39.0], [22.0, -77.0, 11.0, -63.0, -42.0], [-2.0, -41.0, -65.0, 51.0, -40.0]], [[0.0, -3.0, 10.0, 22.0, 102.0], [-62.0, -75.0, 118.0, -105.0, 61.0], [9.0, 87.0, -3.0, 117.0, 5.0], [-50.0, 65.0, 75.0, -52.0, -12.0]], [[38.0, 36.0, 19.0, -3.0, -67.0], [63.0, 55.0, -87.0, 19.0, -17.0], [-60.0, -64.0, 3.0, -93.0, -12.0], [-44.0, -68.0, -56.0, 94.0, -40.0]], [[-75.0, 3.0, -26.0, -56.0, 6.0], [54.0, 23.0, 23.0, 24.0, -37.0], [114.0, -16.0, 4.0, 53.0, -14.0], [-41.0, 12.0, 15.0, -43.0, -2.0]], [[16.0, 25.0, -88.0, 34.0, 82.0], [-32.0, -14.0, 2.0, -40.0, -23.0], [14.0, 1.0, -22.0, 55.0, -67.0], [53.0, 68.0, -41.0, -45.0, 20.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_expandedData_rank2(self):
      arg0=Data(numpy.array([[[[-4.0, -2.0, -1.0], [-1.0, 1.0, -4.0]], [[6.0, -2.0, 4.0], [1.0, 1.0, 0.0]], [[5.0, -1.0, 6.0], [5.0, 1.0, 6.0]], [[4.0, -3.0, -1.0], [-6.0, -5.0, -2.0]], [[7.0, -6.0, -5.0], [4.0, -3.0, 1.0]]], [[[6.0, 3.0, -3.0], [3.0, 6.0, 5.0]], [[-4.0, 1.0, 5.0], [7.0, 7.0, 5.0]], [[4.0, 1.0, 1.0], [6.0, 6.0, 6.0]], [[1.0, -4.0, -7.0], [6.0, 3.0, 0.0]], [[-7.0, 0.0, 6.0], [1.0, 3.0, -3.0]]], [[[7.0, 2.0, -5.0], [2.0, -1.0, 4.0]], [[1.0, 1.0, -3.0], [1.0, -3.0, 1.0]], [[-7.0, 2.0, -6.0], [3.0, -7.0, -4.0]], [[7.0, 3.0, 7.0], [-1.0, -7.0, 4.0]], [[-5.0, -6.0, -4.0], [0.0, 5.0, 5.0]]], [[[-7.0, -5.0, 4.0], [2.0, -2.0, -3.0]], [[2.0, -2.0, -2.0], [4.0, 3.0, 0.0]], [[-6.0, 4.0, -7.0], [-6.0, -4.0, -5.0]], [[5.0, 0.0, -4.0], [5.0, -4.0, 1.0]], [[0.0, -5.0, 3.0], [-1.0, -1.0, -6.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, -1.0, -4.0], [2.0, 6.0, -6.0]])+(1.-msk_arg1)*numpy.array([[6.0, 5.0, 6.0], [-7.0, 7.0, 6.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[58.0, -42.0, -73.0, -47.0, -32.0], [-15.0, 29.0, -17.0, 56.0, 56.0], [-50.0, -17.0, 52.0, -141.0, 52.0], [41.0, 24.0, 54.0, -34.0, 21.0]])+(1.-msk_ref)*numpy.array([[-50.0, 50.0, 69.0, -2.0, -61.0], [84.0, 41.0, 71.0, -77.0, -10.0], [25.0, -29.0, -162.0, 81.0, -19.0], [-89.0, -17.0, -74.0, -51.0, -43.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_expandedData_rank3(self):
      arg0=Data(numpy.array([[[[-4.0, 5.0, 7.0], [-2.0, -5.0, 4.0]], [[-3.0, -1.0, -3.0], [-6.0, 3.0, 6.0]], [[3.0, -3.0, 3.0], [-2.0, -1.0, -6.0]], [[3.0, -6.0, -1.0], [0.0, -7.0, 1.0]], [[1.0, 6.0, -3.0], [-4.0, 0.0, -7.0]]], [[[-1.0, 5.0, 3.0], [-7.0, -7.0, -5.0]], [[-6.0, 3.0, -4.0], [-6.0, 1.0, -3.0]], [[-4.0, 3.0, 1.0], [5.0, -4.0, -7.0]], [[4.0, 4.0, 1.0], [4.0, 0.0, 7.0]], [[5.0, -2.0, 4.0], [4.0, 4.0, 4.0]]], [[[7.0, 0.0, -6.0], [0.0, 5.0, 6.0]], [[-7.0, 7.0, 1.0], [-6.0, 0.0, -5.0]], [[-4.0, 4.0, -1.0], [-3.0, -5.0, -6.0]], [[-5.0, 3.0, -2.0], [0.0, 6.0, 7.0]], [[-3.0, -6.0, 7.0], [-6.0, 0.0, -6.0]]], [[[2.0, -6.0, 1.0], [-5.0, 1.0, -2.0]], [[-7.0, 3.0, 6.0], [-2.0, 1.0, -1.0]], [[4.0, 6.0, 3.0], [-4.0, -1.0, 6.0]], [[2.0, 3.0, 5.0], [-3.0, -6.0, 7.0]], [[1.0, 6.0, 3.0], [-3.0, 0.0, 2.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, -5.0, -7.0], [-4.0, -3.0, 4.0]], [[7.0, -5.0, 5.0], [7.0, 3.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[4.0, -3.0, 5.0], [7.0, 6.0, 3.0]], [[-6.0, -4.0, -3.0], [-5.0, 1.0, 1.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-43.0, -71.0], [59.0, -100.0], [-13.0, 70.0], [68.0, 19.0], [-19.0, -24.0]], [[-19.0, -57.0], [10.0, -98.0], [-66.0, 27.0], [-7.0, -1.0], [-20.0, 81.0]], [[65.0, -2.0], [-52.0, -91.0], [-18.0, -53.0], [-1.0, -84.0], [-25.0, 38.0]], [[36.0, 29.0], [-70.0, -39.0], [0.0, -54.0], [12.0, -57.0], [-29.0, -41.0]]])+(1.-msk_ref)*numpy.array([[[-28.0, -8.0], [-30.0, 70.0], [-2.0, -12.0], [-14.0, 3.0], [-78.0, -8.0]], [[-110.0, 0.0], [-98.0, 64.0], [-30.0, -27.0], [58.0, -56.0], [110.0, -46.0]], [[46.0, -13.0], [-101.0, 36.0], [-102.0, 15.0], [18.0, 37.0], [-19.0, 45.0]], [[-4.0, 33.0], [-18.0, 22.0], [-3.0, -32.0], [-12.0, -23.0], [-14.0, -22.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_constData_rank4_expandedData_rank4(self):
      arg0=Data(numpy.array([[[[-6.0, 6.0, -1.0], [-1.0, 4.0, 4.0]], [[-3.0, -1.0, -6.0], [1.0, -1.0, -4.0]], [[-7.0, -4.0, -7.0], [-6.0, 0.0, 0.0]], [[1.0, 3.0, 3.0], [-4.0, 7.0, -5.0]], [[-5.0, 2.0, 0.0], [-1.0, 7.0, -5.0]]], [[[1.0, 0.0, 5.0], [-4.0, 1.0, 3.0]], [[1.0, 1.0, -1.0], [-3.0, -6.0, -1.0]], [[0.0, 6.0, 7.0], [-7.0, -7.0, 3.0]], [[4.0, -1.0, -1.0], [2.0, 5.0, 0.0]], [[-6.0, 3.0, 0.0], [-2.0, -1.0, 5.0]]], [[[4.0, 4.0, 2.0], [0.0, 5.0, 0.0]], [[-7.0, -6.0, -7.0], [4.0, 0.0, -4.0]], [[0.0, -3.0, -7.0], [0.0, 6.0, 1.0]], [[0.0, -1.0, 2.0], [-7.0, -7.0, 2.0]], [[3.0, -4.0, 3.0], [-4.0, -1.0, -7.0]]], [[[6.0, -5.0, 5.0], [4.0, -6.0, -3.0]], [[-2.0, -2.0, -1.0], [7.0, -1.0, 6.0]], [[-4.0, -4.0, 6.0], [1.0, 6.0, -4.0]], [[5.0, -1.0, 5.0], [0.0, -2.0, 4.0]], [[-3.0, -5.0, 1.0], [0.0, -1.0, 0.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-3.0, -6.0, 2.0], [6.0, -1.0, -5.0]], [[-3.0, -4.0, 0.0], [-6.0, -1.0, 7.0]], [[-7.0, 0.0, -5.0], [-3.0, 1.0, 0.0]], [[-5.0, 3.0, 0.0], [-3.0, 1.0, -2.0]], [[-5.0, -6.0, 5.0], [-4.0, -6.0, 3.0]]], [[[7.0, -3.0, 0.0], [6.0, 1.0, 7.0]], [[-3.0, 7.0, -3.0], [2.0, -2.0, 5.0]], [[-6.0, 4.0, 1.0], [1.0, 0.0, -4.0]], [[-7.0, 0.0, 4.0], [2.0, -5.0, -1.0]], [[0.0, -5.0, 0.0], [-1.0, -6.0, -2.0]]], [[[-3.0, 0.0, -7.0], [6.0, 5.0, -1.0]], [[2.0, 4.0, -5.0], [6.0, 3.0, 4.0]], [[4.0, -5.0, 4.0], [-4.0, 3.0, 0.0]], [[7.0, 5.0, 0.0], [2.0, 0.0, -3.0]], [[-4.0, 1.0, -5.0], [0.0, -3.0, -3.0]]], [[[1.0, -4.0, 4.0], [2.0, 0.0, 0.0]], [[-4.0, -3.0, 3.0], [5.0, 2.0, 0.0]], [[-7.0, 5.0, -2.0], [0.0, -2.0, -5.0]], [[-2.0, -4.0, 3.0], [-1.0, 6.0, 4.0]], [[-6.0, 6.0, 7.0], [7.0, -6.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, 3.0, 3.0], [2.0, -7.0, -3.0]], [[6.0, 6.0, 5.0], [4.0, 1.0, -1.0]], [[-7.0, 6.0, 2.0], [-7.0, -6.0, -1.0]], [[4.0, 2.0, 3.0], [4.0, 5.0, 3.0]], [[4.0, -3.0, -7.0], [-6.0, -1.0, -6.0]]], [[[-3.0, -2.0, 7.0], [-4.0, -7.0, 0.0]], [[-2.0, 5.0, -7.0], [4.0, 5.0, 0.0]], [[0.0, -6.0, 4.0], [0.0, -6.0, -5.0]], [[2.0, 7.0, 5.0], [-5.0, -4.0, -2.0]], [[6.0, -7.0, 4.0], [4.0, 0.0, 6.0]]], [[[6.0, 3.0, -5.0], [7.0, 2.0, 0.0]], [[3.0, 4.0, 4.0], [0.0, 5.0, 0.0]], [[-7.0, 3.0, -1.0], [0.0, -3.0, -6.0]], [[3.0, 7.0, 1.0], [3.0, -4.0, -3.0]], [[3.0, 7.0, 2.0], [-7.0, 7.0, -2.0]]], [[[7.0, 5.0, 7.0], [-5.0, 6.0, 5.0]], [[7.0, -3.0, 0.0], [6.0, 5.0, 0.0]], [[-7.0, 5.0, -5.0], [-1.0, -1.0, -6.0]], [[2.0, -2.0, 3.0], [7.0, 5.0, -1.0]], [[-4.0, 0.0, -2.0], [7.0, 1.0, 1.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-50.0, 24.0, 54.0, 47.0, -19.0], [-34.0, 73.0, 42.0, 12.0, -61.0], [35.0, 39.0, -42.0, -26.0, 11.0], [-36.0, 6.0, 46.0, 26.0, 6.0]], [[30.0, -20.0, 47.0, 16.0, -19.0], [-41.0, 4.0, 25.0, 8.0, 18.0], [56.0, 7.0, -38.0, -12.0, 56.0], [-21.0, 0.0, 50.0, -31.0, 11.0]], [[-5.0, 73.0, 102.0, 41.0, 48.0], [-73.0, 2.0, 13.0, 9.0, 26.0], [34.0, -31.0, -12.0, -81.0, 59.0], [-31.0, -11.0, 43.0, 15.0, -73.0]], [[-21.0, -33.0, -3.0, 33.0, -49.0], [-54.0, -38.0, 25.0, -33.0, -43.0], [-8.0, -24.0, 38.0, 29.0, -22.0], [-7.0, -10.0, 13.0, 21.0, -2.0]], [[15.0, -29.0, 45.0, 51.0, -40.0], [-75.0, -12.0, 57.0, 3.0, -41.0], [49.0, -7.0, -5.0, -12.0, 16.0], [-15.0, 23.0, 56.0, 25.0, 28.0]]], [[[-33.0, 41.0, -19.0, 2.0, 39.0], [5.0, -13.0, -17.0, -3.0, -8.0], [-60.0, -32.0, 43.0, -10.0, -41.0], [13.0, -7.0, -34.0, 35.0, -26.0]], [[-18.0, 10.0, 1.0, 3.0, 29.0], [-27.0, 8.0, -2.0, 14.0, 36.0], [-43.0, -29.0, -11.0, 9.0, 23.0], [-13.0, -37.0, 17.0, -46.0, 15.0]], [[-72.0, 46.0, -21.0, 26.0, 78.0], [-46.0, 36.0, 12.0, 46.0, 13.0], [-129.0, -62.0, 5.0, 7.0, -17.0], [-10.0, -46.0, 15.0, -26.0, 57.0]], [[-1.0, -25.0, -24.0, -24.0, -57.0], [48.0, -22.0, -27.0, -53.0, -27.0], [32.0, 36.0, 24.0, 27.0, -27.0], [8.0, 4.0, -41.0, 21.0, -53.0]], [[-36.0, 54.0, 47.0, 34.0, 41.0], [-29.0, 62.0, 26.0, 38.0, -17.0], [-4.0, 5.0, -34.0, -46.0, 15.0], [-22.0, 3.0, 34.0, 16.0, 11.0]]], [[[-37.0, -33.0, -33.0, -3.0, -64.0], [21.0, 0.0, -6.0, -45.0, -50.0], [-1.0, 29.0, 19.0, 48.0, -37.0], [-4.0, -12.0, -22.0, 12.0, -16.0]], [[87.0, -7.0, 72.0, 13.0, 8.0], [-35.0, -12.0, 31.0, 33.0, 34.0], [98.0, 5.0, -42.0, -59.0, 69.0], [-3.0, 45.0, 53.0, -3.0, 13.0]], [[-7.0, 13.0, 41.0, -5.0, -50.0], [22.0, -7.0, -23.0, -59.0, -23.0], [78.0, 45.0, 5.0, -18.0, 11.0], [-16.0, 0.0, -18.0, 31.0, -110.0]], [[-35.0, 67.0, 4.0, 7.0, 92.0], [-32.0, -3.0, -17.0, 27.0, 50.0], [-93.0, -69.0, 20.0, -25.0, 4.0], [-2.0, -40.0, -5.0, -17.0, -13.0]], [[33.0, -17.0, -25.0, -2.0, 25.0], [-41.0, -87.0, -7.0, -5.0, 44.0], [-52.0, -80.0, 57.0, 14.0, -7.0], [23.0, -13.0, -10.0, -11.0, 6.0]]], [[[67.0, -37.0, -85.0, -57.0, 36.0], [54.0, -63.0, -35.0, 19.0, 63.0], [-56.0, -39.0, 35.0, 34.0, -27.0], [54.0, 14.0, -50.0, -29.0, 54.0]], [[29.0, 15.0, -3.0, -30.0, 13.0], [75.0, 41.0, -14.0, 23.0, -3.0], [44.0, 56.0, -33.0, -28.0, -4.0], [16.0, 44.0, -22.0, 20.0, 6.0]], [[68.0, -12.0, 1.0, 19.0, 22.0], [-32.0, -64.0, 31.0, 28.0, -9.0], [10.0, -46.0, 42.0, -34.0, -24.0], [38.0, 63.0, 4.0, 61.0, 41.0]], [[-17.0, 19.0, -62.0, -38.0, 30.0], [64.0, -13.0, -45.0, -9.0, 9.0], [-64.0, -9.0, 39.0, 18.0, -52.0], [29.0, -6.0, -66.0, 13.0, -17.0]], [[42.0, 30.0, 15.0, -1.0, 56.0], [-7.0, -27.0, -1.0, 30.0, 31.0], [-3.0, -34.0, 14.0, -46.0, 5.0], [21.0, 28.0, -4.0, 23.0, 1.0]]]])+(1.-msk_ref)*numpy.array([[[[15.0, -9.0, 55.0, 13.0, -57.0], [-25.0, 65.0, -84.0, 6.0, -62.0], [-12.0, 22.0, 25.0, -8.0, 49.0], [30.0, -46.0, 50.0, -18.0, 27.0]], [[21.0, -47.0, 6.0, -45.0, 52.0], [-28.0, 42.0, 8.0, -36.0, -55.0], [14.0, -42.0, 51.0, -3.0, -34.0], [-99.0, -17.0, 70.0, -16.0, 26.0]], [[4.0, -125.0, 53.0, -81.0, 69.0], [4.0, 19.0, -4.0, -47.0, -66.0], [-61.0, -65.0, 44.0, -74.0, -21.0], [-88.0, -73.0, 70.0, -69.0, 0.0]], [[-31.0, 35.0, 8.0, 23.0, 21.0], [-21.0, 11.0, -23.0, 40.0, -49.0], [-14.0, 62.0, 8.0, 2.0, 117.0], [80.0, 9.0, 20.0, 17.0, -36.0]], [[5.0, -10.0, 17.0, 0.0, 3.0], [-34.0, 51.0, -29.0, -9.0, -78.0], [-17.0, 28.0, 50.0, -17.0, 65.0], [-3.0, -12.0, 69.0, 19.0, 15.0]]], [[[-16.0, 13.0, 22.0, 17.0, -26.0], [41.0, -48.0, -1.0, 37.0, 28.0], [-45.0, 28.0, -33.0, -17.0, 42.0], [83.0, -12.0, -47.0, -9.0, -38.0]], [[32.0, -10.0, 55.0, -42.0, 38.0], [42.0, -32.0, 31.0, 45.0, -23.0], [-19.0, -27.0, 21.0, 27.0, -11.0], [-21.0, -44.0, 18.0, -53.0, -30.0]], [[65.0, 33.0, 138.0, -21.0, -36.0], [114.0, -82.0, 19.0, 134.0, -24.0], [-80.0, 17.0, 14.0, 47.0, 50.0], [87.0, -95.0, -9.0, -78.0, -67.0]], [[-65.0, 26.0, -80.0, 44.0, 9.0], [-60.0, 27.0, -28.0, -34.0, 35.0], [50.0, 29.0, -45.0, -10.0, 24.0], [36.0, 68.0, -35.0, 46.0, 5.0]], [[39.0, -32.0, 75.0, -16.0, -50.0], [27.0, 14.0, -37.0, 13.0, -35.0], [-43.0, -11.0, 24.0, -14.0, 0.0], [2.0, -68.0, 30.0, -42.0, 14.0]]], [[[-45.0, 63.0, -30.0, 55.0, -15.0], [-41.0, 23.0, -46.0, 26.0, 4.0], [36.0, 61.0, -33.0, 22.0, 79.0], [92.0, 41.0, -23.0, 31.0, -15.0]], [[30.0, -93.0, -25.0, -57.0, 39.0], [-32.0, 49.0, 28.0, -103.0, -36.0], [3.0, -73.0, 62.0, -46.0, -97.0], [-168.0, -7.0, 74.0, 9.0, 66.0]], [[-75.0, -48.0, -69.0, 6.0, 46.0], [-85.0, 64.0, -51.0, -82.0, -1.0], [38.0, -10.0, -26.0, -55.0, 5.0], [-23.0, 39.0, 8.0, 14.0, 21.0]], [[32.0, -33.0, 87.0, -53.0, 26.0], [93.0, -82.0, 46.0, 62.0, -1.0], [-76.0, -31.0, 4.0, -4.0, -7.0], [12.0, -74.0, -13.0, -78.0, -58.0]], [[-4.0, -1.0, 2.0, -29.0, 70.0], [43.0, -68.0, 77.0, 31.0, 0.0], [-39.0, 0.0, 9.0, -3.0, 22.0], [1.0, 4.0, -9.0, -3.0, -54.0]]], [[[17.0, 44.0, -51.0, 6.0, 4.0], [53.0, -86.0, 101.0, 12.0, 89.0], [12.0, -12.0, -26.0, 33.0, -71.0], [-19.0, 51.0, -72.0, 38.0, -15.0]], [[8.0, -8.0, -49.0, 26.0, -72.0], [-18.0, 24.0, -16.0, -66.0, 62.0], [34.0, -23.0, -24.0, -14.0, -90.0], [-42.0, 29.0, -33.0, 35.0, 64.0]], [[6.0, -4.0, -23.0, 16.0, -34.0], [16.0, -20.0, 32.0, -27.0, 8.0], [-47.0, 26.0, 16.0, -43.0, 15.0], [5.0, 20.0, -5.0, 59.0, 13.0]], [[-21.0, 43.0, -23.0, 35.0, -34.0], [36.0, -60.0, 18.0, 28.0, 81.0], [-2.0, 21.0, -61.0, 9.0, -4.0], [73.0, 28.0, -87.0, 13.0, -28.0]], [[16.0, -44.0, -1.0, -24.0, -3.0], [33.0, -31.0, 40.0, -32.0, 21.0], [-40.0, -30.0, 8.0, -39.0, -49.0], [-45.0, -11.0, -8.0, 2.0, 9.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_array_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, -6.0, -4.0], [-4.0, -3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[6.0, 6.0, 1.0], [-2.0, 3.0, -5.0]])
      arg1=numpy.array([0.0, 7.0, 4.0])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-58.0, -21.0])+(1.-msk_ref)*numpy.array([46.0, 1.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 5.0, 0.0], [-1.0, 5.0, -6.0]])+(1.-msk_arg0)*numpy.array([[5.0, -2.0, -6.0], [0.0, 2.0, -6.0]])
      arg1=numpy.array([[1.0, -2.0, 0.0], [-4.0, 5.0, -2.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, 17.0], [-11.0, 41.0]])+(1.-msk_ref)*numpy.array([[9.0, -18.0], [-4.0, 22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_Symbol_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, 1.0, 4.0], [4.0, 0.0, 3.0]])+(1.-msk_arg0)*numpy.array([[1.0, -6.0, -4.0], [-2.0, 6.0, -3.0]])
      arg1=Symbol(shape=(3,))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([-2.0, -7.0, -4.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-9.0, -20.0])+(1.-msk_ref)*numpy.array([56.0, -26.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 6.0, 0.0], [2.0, -2.0, 4.0]])+(1.-msk_arg0)*numpy.array([[4.0, 5.0, -6.0], [0.0, 5.0, 3.0]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[-7.0, 7.0, -3.0], [-6.0, 3.0, -2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[14.0, -6.0], [-40.0, -26.0]])+(1.-msk_ref)*numpy.array([[25.0, 3.0], [26.0, 9.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_constData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 4.0, -3.0], [4.0, -1.0, -6.0]])+(1.-msk_arg0)*numpy.array([[1.0, 7.0, 3.0], [-1.0, -6.0, -2.0]])
      arg1=Data(numpy.array([6.0, 7.0, 6.0]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([40.0, -19.0])+(1.-msk_ref)*numpy.array([73.0, -60.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 7.0, 0.0], [0.0, -6.0, -6.0]])+(1.-msk_arg0)*numpy.array([[0.0, 6.0, -7.0], [4.0, -5.0, -5.0]])
      arg1=Data(numpy.array([[-5.0, 5.0, 1.0], [3.0, -1.0, 0.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[15.0, 5.0], [-36.0, 6.0]])+(1.-msk_ref)*numpy.array([[23.0, -6.0], [-50.0, 17.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_expandedData_rank1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 0.0, 7.0], [0.0, 2.0, 2.0]])+(1.-msk_arg0)*numpy.array([[1.0, -7.0, -1.0], [-2.0, 4.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([4.0, -2.0, 1.0])+(1.-msk_arg1)*numpy.array([3.0, -1.0, 1.0])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([23.0, -2.0])+(1.-msk_ref)*numpy.array([9.0, -9.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank2_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, 1.0, 4.0], [0.0, 0.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-6.0, 0.0, -6.0], [0.0, 6.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 1.0, 4.0], [-7.0, 1.0, 4.0]])+(1.-msk_arg1)*numpy.array([[2.0, 0.0, -2.0], [-3.0, 6.0, -1.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[32.0, 38.0], [28.0, 28.0]])+(1.-msk_ref)*numpy.array([[0.0, 24.0], [-10.0, 31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_array_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, -7.0, 7.0], [5.0, 6.0, -3.0]], [[-6.0, 2.0, 2.0], [1.0, -4.0, 6.0]], [[0.0, 5.0, 0.0], [6.0, 5.0, -2.0]], [[2.0, -4.0, -7.0], [7.0, 1.0, 7.0]], [[-4.0, -1.0, 2.0], [5.0, 1.0, -3.0]]], [[[-6.0, -3.0, 4.0], [7.0, 5.0, 5.0]], [[-7.0, 7.0, 3.0], [5.0, 4.0, 1.0]], [[-6.0, -7.0, 0.0], [-1.0, -5.0, -3.0]], [[-2.0, -4.0, 2.0], [3.0, 1.0, 6.0]], [[-1.0, -6.0, -7.0], [3.0, 3.0, 0.0]]], [[[-2.0, -2.0, 3.0], [-2.0, 0.0, -5.0]], [[-3.0, -6.0, 6.0], [1.0, -5.0, 3.0]], [[0.0, 0.0, -7.0], [-3.0, 2.0, -5.0]], [[-1.0, 2.0, 7.0], [4.0, -5.0, 6.0]], [[6.0, -3.0, -3.0], [4.0, -4.0, 7.0]]], [[[-1.0, 4.0, 7.0], [6.0, -7.0, -6.0]], [[0.0, 4.0, -1.0], [3.0, 1.0, -6.0]], [[-1.0, 5.0, 2.0], [-5.0, 0.0, 3.0]], [[-7.0, 2.0, -1.0], [-6.0, 1.0, -5.0]], [[-2.0, -2.0, -2.0], [-2.0, -1.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 0.0, 0.0], [1.0, -3.0, 4.0]], [[3.0, -2.0, 5.0], [7.0, 0.0, -2.0]], [[6.0, -6.0, 5.0], [5.0, -6.0, -4.0]], [[6.0, -7.0, -5.0], [-1.0, -1.0, 0.0]], [[-3.0, 5.0, -6.0], [-7.0, 1.0, -3.0]]], [[[-2.0, 0.0, -5.0], [5.0, 5.0, -4.0]], [[4.0, -3.0, -7.0], [1.0, -2.0, -5.0]], [[4.0, -3.0, -7.0], [4.0, 6.0, 5.0]], [[0.0, -2.0, -2.0], [-4.0, 5.0, -4.0]], [[4.0, 3.0, -5.0], [-2.0, 4.0, 0.0]]], [[[-2.0, -6.0, 7.0], [-5.0, 6.0, -1.0]], [[-6.0, 5.0, -5.0], [3.0, -6.0, 3.0]], [[3.0, -2.0, 0.0], [-2.0, -5.0, 5.0]], [[3.0, 0.0, -2.0], [0.0, -1.0, -6.0]], [[4.0, -4.0, 7.0], [3.0, -3.0, 0.0]]], [[[-3.0, -6.0, -6.0], [7.0, -1.0, 6.0]], [[3.0, 4.0, -1.0], [-1.0, -4.0, -7.0]], [[7.0, 6.0, -1.0], [3.0, 3.0, 6.0]], [[1.0, -2.0, 6.0], [-5.0, 0.0, 7.0]], [[-4.0, -4.0, -6.0], [-4.0, 6.0, 6.0]]]])
      arg1=numpy.array([[0.0, -7.0, -2.0], [-4.0, -3.0, 7.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, 32.0, -88.0, 60.0, -41.0], [5.0, -80.0, 47.0, 51.0, 35.0], [-19.0, 62.0, -15.0, 13.0, 72.0], [-87.0, -83.0, 2.0, -26.0, -6.0]])+(1.-msk_ref)*numpy.array([[33.0, -38.0, 2.0, 66.0, -19.0], [-53.0, 2.0, 36.0, -9.0, -15.0], [23.0, 2.0, 72.0, -35.0, 11.0], [71.0, -59.0, -19.0, 71.0, 80.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_array_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, 5.0, -3.0], [-1.0, 0.0, -2.0]], [[2.0, 5.0, -2.0], [3.0, -6.0, -2.0]], [[3.0, -7.0, -2.0], [-3.0, 2.0, -3.0]], [[-2.0, -3.0, -5.0], [-1.0, 5.0, 3.0]], [[6.0, 3.0, 2.0], [-7.0, 1.0, 5.0]]], [[[-2.0, 0.0, 6.0], [5.0, 7.0, 5.0]], [[0.0, 0.0, -2.0], [4.0, 2.0, 1.0]], [[-4.0, -2.0, -6.0], [-4.0, -7.0, 4.0]], [[2.0, 3.0, -1.0], [6.0, 5.0, -6.0]], [[0.0, 3.0, -2.0], [0.0, 3.0, 0.0]]], [[[7.0, 4.0, 6.0], [2.0, 3.0, -6.0]], [[-1.0, -2.0, 0.0], [0.0, -5.0, 3.0]], [[-6.0, -3.0, -5.0], [-2.0, 0.0, 6.0]], [[-5.0, 0.0, 0.0], [5.0, -4.0, -3.0]], [[-4.0, 5.0, 2.0], [3.0, 0.0, 0.0]]], [[[0.0, -2.0, -4.0], [7.0, 5.0, -3.0]], [[-6.0, 3.0, -6.0], [-4.0, -4.0, 7.0]], [[3.0, -5.0, -6.0], [1.0, -5.0, 7.0]], [[-3.0, -6.0, 6.0], [-1.0, -5.0, -2.0]], [[-2.0, -5.0, -2.0], [-6.0, 6.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, -4.0, 7.0], [-2.0, 6.0, 4.0]], [[4.0, -5.0, 0.0], [0.0, 2.0, 0.0]], [[-5.0, 5.0, -6.0], [3.0, 0.0, 6.0]], [[4.0, -7.0, 6.0], [2.0, -5.0, -3.0]], [[-4.0, 6.0, 2.0], [-2.0, 2.0, 6.0]]], [[[-4.0, 1.0, -4.0], [2.0, 0.0, -3.0]], [[0.0, -7.0, -2.0], [6.0, 0.0, -2.0]], [[0.0, 0.0, -6.0], [-3.0, 7.0, 0.0]], [[-6.0, -2.0, -2.0], [-3.0, -5.0, 7.0]], [[-4.0, 5.0, -2.0], [-6.0, 0.0, -6.0]]], [[[0.0, -5.0, -4.0], [4.0, 0.0, 3.0]], [[0.0, 0.0, -2.0], [0.0, -1.0, 0.0]], [[5.0, 5.0, -6.0], [-3.0, 4.0, -3.0]], [[1.0, 0.0, -4.0], [4.0, 2.0, -3.0]], [[2.0, 1.0, -4.0], [1.0, -1.0, 0.0]]], [[[0.0, -3.0, 5.0], [-3.0, -6.0, -2.0]], [[-1.0, -6.0, 6.0], [5.0, -1.0, 5.0]], [[4.0, 1.0, 2.0], [3.0, 4.0, 0.0]], [[5.0, -3.0, 0.0], [1.0, 1.0, -4.0]], [[-5.0, -1.0, 0.0], [4.0, 1.0, 0.0]]]])
      arg1=numpy.array([[[-1.0, 1.0, -1.0], [-5.0, 0.0, -2.0]], [[7.0, -2.0, 1.0], [-5.0, 6.0, 1.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[11.0, 32.0], [-6.0, -51.0], [13.0, 57.0], [3.0, 25.0], [20.0, 84.0]], [[-39.0, 14.0], [-20.0, -9.0], [20.0, -48.0], [-16.0, 1.0], [5.0, 10.0]], [[-7.0, 49.0], [-7.0, -30.0], [6.0, -25.0], [-14.0, -87.0], [-8.0, -51.0]], [[-27.0, -8.0], [21.0, -51.0], [-21.0, -3.0], [0.0, -30.0], [25.0, 62.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 72.0], [-9.0, 50.0], [-11.0, -60.0], [-21.0, 5.0], [6.0, -10.0]], [[5.0, -47.0], [-31.0, -20.0], [21.0, 51.0], [7.0, -48.0], [53.0, -16.0]], [[-27.0, -11.0], [2.0, -8.0], [27.0, 55.0], [-11.0, -8.0], [-2.0, -3.0]], [[11.0, -12.0], [-46.0, -15.0], [-20.0, 37.0], [-5.0, 38.0], [-16.0, -47.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_array_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, -5.0, -1.0], [-3.0, 4.0, -6.0]], [[-5.0, -4.0, -5.0], [3.0, -6.0, -2.0]], [[3.0, 2.0, -2.0], [3.0, 3.0, -1.0]], [[-6.0, -5.0, 0.0], [-3.0, 4.0, -5.0]], [[1.0, 5.0, -6.0], [5.0, -6.0, 6.0]]], [[[-7.0, -2.0, 6.0], [6.0, 3.0, -1.0]], [[-3.0, 5.0, 0.0], [-6.0, -5.0, -3.0]], [[-7.0, 6.0, 6.0], [0.0, -6.0, 1.0]], [[-1.0, 0.0, 7.0], [1.0, -3.0, 0.0]], [[5.0, 0.0, 1.0], [1.0, -7.0, 7.0]]], [[[0.0, 5.0, 3.0], [0.0, 7.0, 1.0]], [[-4.0, 2.0, 0.0], [-3.0, -3.0, -2.0]], [[-7.0, -4.0, -3.0], [-3.0, 1.0, 6.0]], [[-2.0, -6.0, -5.0], [-6.0, -2.0, -4.0]], [[-5.0, -2.0, 5.0], [-6.0, -7.0, -3.0]]], [[[0.0, -4.0, -7.0], [0.0, 2.0, -1.0]], [[0.0, 5.0, 1.0], [-4.0, 1.0, -4.0]], [[2.0, 0.0, -2.0], [0.0, -4.0, -2.0]], [[1.0, -2.0, 7.0], [6.0, -2.0, 6.0]], [[-2.0, 5.0, -6.0], [0.0, 5.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, 1.0, 1.0], [-2.0, 5.0, 4.0]], [[1.0, -1.0, 3.0], [-2.0, -7.0, 1.0]], [[6.0, -2.0, -7.0], [-2.0, 4.0, 0.0]], [[5.0, -3.0, -3.0], [-2.0, 6.0, -7.0]], [[3.0, 4.0, -6.0], [-7.0, 6.0, -6.0]]], [[[7.0, -3.0, 0.0], [-1.0, 1.0, 4.0]], [[3.0, 1.0, -4.0], [7.0, 4.0, 1.0]], [[0.0, 2.0, 0.0], [6.0, 3.0, 1.0]], [[-7.0, -6.0, 4.0], [-2.0, -7.0, 2.0]], [[-4.0, 7.0, -5.0], [0.0, -7.0, -6.0]]], [[[6.0, 7.0, -5.0], [0.0, -5.0, -3.0]], [[-3.0, 7.0, 7.0], [1.0, 3.0, 6.0]], [[-2.0, 4.0, -3.0], [-2.0, -1.0, -7.0]], [[-6.0, 7.0, 3.0], [5.0, -5.0, -3.0]], [[6.0, 3.0, 1.0], [-1.0, -7.0, 6.0]]], [[[-6.0, 5.0, -6.0], [4.0, -4.0, 1.0]], [[7.0, 0.0, -6.0], [-2.0, 1.0, 3.0]], [[3.0, -1.0, -1.0], [-3.0, 6.0, -2.0]], [[3.0, -6.0, 2.0], [0.0, 0.0, -1.0]], [[4.0, 5.0, -6.0], [3.0, -5.0, -4.0]]]])
      arg1=numpy.array([[[[-3.0, 0.0, 7.0], [-6.0, 6.0, 7.0]], [[7.0, 0.0, -6.0], [0.0, 0.0, 0.0]], [[6.0, 1.0, -3.0], [0.0, -4.0, -1.0]], [[-5.0, 7.0, -4.0], [2.0, -7.0, 4.0]], [[-7.0, 3.0, -1.0], [0.0, -2.0, 4.0]]], [[[1.0, 0.0, 3.0], [6.0, -4.0, -3.0]], [[7.0, 3.0, -6.0], [-6.0, 7.0, 5.0]], [[-6.0, -1.0, 7.0], [-5.0, 2.0, 3.0]], [[6.0, 5.0, -3.0], [5.0, -7.0, -3.0]], [[5.0, 0.0, 4.0], [0.0, -1.0, 4.0]]], [[[2.0, -7.0, -2.0], [5.0, -4.0, -1.0]], [[-7.0, 0.0, 1.0], [6.0, 2.0, 4.0]], [[-6.0, 5.0, -4.0], [4.0, 5.0, -3.0]], [[1.0, 6.0, -5.0], [3.0, -1.0, 2.0]], [[-3.0, -1.0, 7.0], [-6.0, 2.0, -2.0]]], [[[6.0, 5.0, -2.0], [-1.0, 5.0, 5.0]], [[6.0, -1.0, 2.0], [2.0, 7.0, -7.0]], [[4.0, 0.0, 1.0], [0.0, 3.0, 7.0]], [[-2.0, -2.0, -5.0], [5.0, -5.0, 0.0]], [[6.0, 6.0, -3.0], [-6.0, -3.0, -4.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-4.0, -1.0, -18.0, -84.0, -39.0], [-20.0, 0.0, 9.0, -53.0, -37.0], [10.0, -28.0, 11.0, -51.0, 39.0], [-36.0, 61.0, -35.0, -18.0, -3.0]], [[-88.0, -5.0, 7.0, 57.0, 32.0], [28.0, -87.0, -34.0, 28.0, -47.0], [69.0, 28.0, 18.0, 7.0, -42.0], [-83.0, -58.0, -57.0, 88.0, -31.0]], [[-30.0, 33.0, 15.0, -12.0, -23.0], [6.0, 37.0, -46.0, 31.0, 0.0], [0.0, -3.0, 30.0, 29.0, -35.0], [39.0, 46.0, 12.0, 0.0, 13.0]], [[25.0, -42.0, -52.0, -59.0, -1.0], [-25.0, -36.0, 49.0, -89.0, -54.0], [-3.0, 12.0, 34.0, -59.0, 59.0], [-63.0, 26.0, -47.0, -13.0, -40.0]], [[-69.0, 43.0, 47.0, 130.0, 50.0], [19.0, 16.0, -72.0, 98.0, 11.0], [22.0, 29.0, 15.0, 94.0, -104.0], [38.0, -85.0, 22.0, 73.0, 18.0]]], [[[38.0, -85.0, -73.0, -16.0, 27.0], [38.0, -111.0, 59.0, -58.0, -18.0], [7.0, 93.0, 50.0, -36.0, 37.0], [-60.0, 12.0, -20.0, 3.0, -113.0]], [[-6.0, -21.0, 10.0, 61.0, 34.0], [-10.0, -20.0, 24.0, 21.0, -22.0], [-48.0, -37.0, 3.0, 8.0, 36.0], [-27.0, -49.0, -48.0, -9.0, 75.0]], [[34.0, -85.0, -31.0, 99.0, 77.0], [32.0, -104.0, 69.0, 9.0, -1.0], [-45.0, 47.0, 15.0, 7.0, 43.0], [-49.0, -85.0, -33.0, 2.0, -10.0]], [[28.0, -49.0, -15.0, 0.0, 6.0], [38.0, -76.0, 44.0, -1.0, 26.0], [1.0, 14.0, -33.0, -30.0, 40.0], [-36.0, -11.0, -6.0, -13.0, -24.0]], [[-7.0, 29.0, 48.0, 50.0, 6.0], [21.0, 9.0, -21.0, 60.0, 64.0], [34.0, -14.0, -86.0, 24.0, -42.0], [27.0, -64.0, 49.0, 25.0, 14.0]]], [[[70.0, -18.0, -33.0, -22.0, 2.0], [-22.0, 51.0, 33.0, -36.0, 9.0], [-70.0, 21.0, 45.0, 10.0, 28.0], [59.0, 43.0, 31.0, -60.0, -4.0]], [[-2.0, -28.0, -8.0, 41.0, 32.0], [-4.0, -35.0, 25.0, -2.0, -25.0], [-23.0, -4.0, 13.0, -2.0, 26.0], [-36.0, -39.0, -39.0, 4.0, 23.0]], [[66.0, -31.0, -47.0, 30.0, 62.0], [-56.0, 12.0, 60.0, -93.0, -24.0], [-5.0, 54.0, 9.0, -14.0, 12.0], [-18.0, -85.0, 14.0, 17.0, -66.0]], [[-33.0, 16.0, 9.0, -26.0, -11.0], [-33.0, 0.0, -3.0, -31.0, -44.0], [30.0, -47.0, -20.0, -37.0, 17.0], [-56.0, -14.0, -47.0, 21.0, 25.0]], [[23.0, -65.0, -16.0, 16.0, 26.0], [11.0, -99.0, 74.0, -27.0, -10.0], [-5.0, -22.0, -50.0, -59.0, 80.0], [-94.0, -58.0, -57.0, -6.0, 12.0]]], [[[-44.0, 42.0, 10.0, -18.0, -13.0], [-26.0, 39.0, -44.0, -10.0, -34.0], [35.0, -7.0, 21.0, 7.0, -39.0], [-1.0, 11.0, -8.0, 33.0, -5.0]], [[9.0, -6.0, 2.0, 0.0, -4.0], [-13.0, 20.0, 12.0, 7.0, -13.0], [-57.0, -37.0, 22.0, 4.0, 36.0], [12.0, 24.0, -24.0, -40.0, 64.0]], [[-58.0, 26.0, 36.0, 18.0, -12.0], [18.0, -12.0, -40.0, 52.0, -2.0], [26.0, -32.0, -18.0, 12.0, -24.0], [-14.0, -6.0, -20.0, 26.0, 38.0]], [[40.0, -35.0, -15.0, 3.0, 8.0], [48.0, -61.0, 29.0, 1.0, 59.0], [34.0, 56.0, -48.0, -14.0, -4.0], [-4.0, -22.0, 47.0, 7.0, -81.0]], [[-41.0, 22.0, -4.0, 14.0, 5.0], [-25.0, 47.0, -40.0, 11.0, -59.0], [-42.0, -2.0, 101.0, 43.0, -21.0], [25.0, 41.0, -34.0, -1.0, 41.0]]]])+(1.-msk_ref)*numpy.array([[[[65.0, 22.0, -2.0, -40.0, -20.0], [-37.0, 92.0, 14.0, -31.0, 35.0], [-35.0, -13.0, -18.0, 2.0, 8.0], [74.0, 28.0, 60.0, -50.0, 8.0]], [[-5.0, -11.0, 23.0, 25.0, 5.0], [23.0, -46.0, 15.0, 28.0, 28.0], [20.0, -26.0, -69.0, -17.0, 15.0], [-33.0, -47.0, -7.0, 10.0, 20.0]], [[-31.0, 84.0, 39.0, -48.0, -49.0], [-43.0, 118.0, -65.0, 9.0, -2.0], [14.0, -53.0, -6.0, 19.0, -45.0], [62.0, 48.0, 29.0, -3.0, 45.0]], [[-37.0, 53.0, 19.0, -108.0, -81.0], [-19.0, 63.0, -47.0, -7.0, -21.0], [10.0, -66.0, 10.0, -24.0, 5.0], [18.0, 114.0, -14.0, -29.0, 43.0]], [[-15.0, 57.0, 22.0, -43.0, -39.0], [-63.0, 123.0, -35.0, -3.0, -39.0], [-63.0, -81.0, 46.0, 18.0, 11.0], [57.0, 72.0, -18.0, -49.0, 108.0]]], [[[19.0, 49.0, 31.0, -49.0, -44.0], [-15.0, 73.0, -20.0, 3.0, 50.0], [22.0, -37.0, -68.0, -7.0, -18.0], [53.0, 22.0, 59.0, -18.0, 11.0]], [[-48.0, 45.0, 14.0, -2.0, -18.0], [14.0, 39.0, -71.0, 39.0, -1.0], [25.0, 29.0, 48.0, 48.0, -74.0], [49.0, 44.0, 27.0, 27.0, -22.0]], [[-11.0, 0.0, -11.0, 9.0, 4.0], [21.0, -4.0, -23.0, 16.0, 1.0], [3.0, 46.0, 46.0, 29.0, -34.0], [24.0, 24.0, 16.0, 11.0, -37.0]], [[33.0, -73.0, -34.0, 30.0, 49.0], [15.0, -118.0, 78.0, -51.0, -4.0], [36.0, 35.0, -53.0, -58.0, 49.0], [-103.0, -95.0, -31.0, 31.0, -65.0]], [[-107.0, 2.0, 32.0, 114.0, 44.0], [27.0, -56.0, -50.0, 93.0, -57.0], [-13.0, -15.0, 62.0, 58.0, -32.0], [-44.0, -48.0, -84.0, 54.0, 78.0]]], [[[-104.0, 72.0, 81.0, 62.0, -18.0], [20.0, 43.0, -97.0, 130.0, 3.0], [-4.0, -69.0, 3.0, 72.0, -64.0], [41.0, 5.0, -17.0, 24.0, 120.0]], [[112.0, -63.0, -50.0, 41.0, 53.0], [-6.0, 3.0, 79.0, -38.0, 34.0], [-82.0, 64.0, 26.0, 16.0, 39.0], [47.0, -30.0, 46.0, -53.0, -36.0]], [[-58.0, 4.0, 12.0, 25.0, 3.0], [2.0, -14.0, -26.0, 35.0, -49.0], [-25.0, -31.0, 52.0, 18.0, 5.0], [-24.0, 16.0, -63.0, 6.0, 64.0]], [[-42.0, -60.0, -15.0, 100.0, 58.0], [62.0, -119.0, 6.0, 59.0, -25.0], [-19.0, 53.0, 63.0, 35.0, -2.0], [-52.0, -41.0, -57.0, 33.0, -6.0]], [[-5.0, 36.0, 58.0, 58.0, 4.0], [13.0, 32.0, -23.0, 74.0, 65.0], [6.0, -37.0, -82.0, 35.0, -34.0], [45.0, -58.0, 46.0, 7.0, 54.0]]], [[[-65.0, -6.0, 2.0, 129.0, 75.0], [13.0, -38.0, -36.0, 52.0, -46.0], [0.0, 56.0, 78.0, 72.0, -63.0], [-18.0, -80.0, -35.0, 72.0, -4.0]], [[-24.0, 85.0, 53.0, -10.0, -33.0], [-36.0, 119.0, -63.0, 34.0, 22.0], [9.0, -53.0, -30.0, 36.0, -55.0], [76.0, 12.0, 46.0, 1.0, 57.0]], [[24.0, 27.0, -2.0, -74.0, -43.0], [-36.0, 74.0, -3.0, -35.0, -3.0], [-22.0, -36.0, 5.0, -17.0, 19.0], [38.0, 67.0, 15.0, -44.0, 23.0]], [[-2.0, 9.0, 7.0, -69.0, -45.0], [12.0, -14.0, -1.0, -15.0, 19.0], [45.0, -23.0, -53.0, -45.0, 13.0], [-21.0, 35.0, 7.0, -4.0, -20.0]], [[-130.0, 64.0, 71.0, 64.0, -13.0], [36.0, 6.0, -108.0, 129.0, -15.0], [24.0, -42.0, 24.0, 70.0, -79.0], [13.0, 6.0, -33.0, 52.0, 85.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_Symbol_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, -3.0, 1.0], [7.0, 2.0, 3.0]], [[-3.0, -2.0, -3.0], [-3.0, 7.0, -2.0]], [[3.0, -5.0, -2.0], [-6.0, 4.0, 2.0]], [[7.0, 2.0, 7.0], [-6.0, 2.0, -5.0]], [[3.0, 0.0, 7.0], [5.0, -4.0, 2.0]]], [[[4.0, 0.0, 5.0], [0.0, -4.0, -3.0]], [[-5.0, 6.0, 0.0], [3.0, -4.0, -3.0]], [[0.0, -3.0, 6.0], [-4.0, 0.0, -2.0]], [[0.0, 0.0, 7.0], [5.0, 7.0, 7.0]], [[0.0, 1.0, 7.0], [-2.0, 7.0, 3.0]]], [[[1.0, -2.0, 3.0], [-6.0, 1.0, -5.0]], [[4.0, 2.0, 4.0], [5.0, -6.0, -1.0]], [[4.0, 4.0, -5.0], [-2.0, -1.0, 0.0]], [[-6.0, -3.0, 6.0], [1.0, 6.0, 3.0]], [[2.0, -5.0, 0.0], [-4.0, -5.0, 7.0]]], [[[0.0, 4.0, 6.0], [1.0, -1.0, -2.0]], [[-3.0, 0.0, 7.0], [0.0, -5.0, -7.0]], [[-1.0, 5.0, 2.0], [0.0, -2.0, -6.0]], [[-4.0, 2.0, 3.0], [-6.0, 5.0, -7.0]], [[5.0, -6.0, 3.0], [-7.0, 4.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, -3.0, -7.0], [-1.0, 6.0, 0.0]], [[5.0, -6.0, -6.0], [0.0, 2.0, 4.0]], [[3.0, 1.0, 4.0], [-4.0, 4.0, 7.0]], [[2.0, -6.0, 0.0], [7.0, -3.0, 1.0]], [[1.0, 4.0, 2.0], [0.0, -3.0, 1.0]]], [[[0.0, 1.0, 1.0], [-4.0, 7.0, -3.0]], [[3.0, 0.0, 7.0], [-2.0, 3.0, 3.0]], [[-5.0, 0.0, 1.0], [-4.0, -1.0, 1.0]], [[-4.0, 4.0, -5.0], [3.0, 0.0, 6.0]], [[-4.0, 2.0, -4.0], [4.0, -2.0, 5.0]]], [[[7.0, 2.0, 0.0], [0.0, -7.0, 0.0]], [[-3.0, -5.0, 1.0], [6.0, -3.0, -5.0]], [[6.0, -5.0, -6.0], [1.0, 6.0, 0.0]], [[0.0, -7.0, 1.0], [-6.0, -7.0, 7.0]], [[5.0, -1.0, -4.0], [2.0, -7.0, 2.0]]], [[[3.0, -1.0, -6.0], [4.0, -5.0, -2.0]], [[-3.0, -5.0, 6.0], [7.0, 0.0, -1.0]], [[7.0, 7.0, 0.0], [7.0, -4.0, 5.0]], [[6.0, 0.0, 1.0], [-3.0, -2.0, 1.0]], [[-3.0, -7.0, 1.0], [-3.0, -4.0, -2.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[1.0, 7.0, -7.0], [-2.0, -6.0, 7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-37.0, -46.0, -16.0, -63.0, -18.0], [-28.0, 34.0, -69.0, -52.0, -59.0], [-63.0, 9.0, 77.0, -86.0, 54.0], [-24.0, -71.0, -10.0, -78.0, -26.0]])+(1.-msk_ref)*numpy.array([[-12.0, 21.0, 15.0, -29.0, 40.0], [-55.0, -39.0, 9.0, 95.0, 77.0], [63.0, -74.0, -25.0, 47.0, 78.0], [46.0, -101.0, 101.0, 24.0, -43.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_Symbol_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, -1.0, 3.0], [-3.0, -2.0, 1.0]], [[0.0, 5.0, -2.0], [2.0, 4.0, -7.0]], [[3.0, -3.0, 1.0], [-5.0, -4.0, 7.0]], [[-4.0, -4.0, 3.0], [6.0, -7.0, -6.0]], [[1.0, 6.0, 4.0], [0.0, -6.0, 6.0]]], [[[0.0, 4.0, -5.0], [5.0, -2.0, 3.0]], [[5.0, -5.0, -6.0], [4.0, 4.0, 4.0]], [[-1.0, 7.0, 3.0], [-2.0, -7.0, -6.0]], [[0.0, -5.0, 4.0], [-2.0, -7.0, 4.0]], [[-3.0, -4.0, 0.0], [6.0, 0.0, 7.0]]], [[[-1.0, 6.0, -2.0], [-2.0, -3.0, -4.0]], [[-6.0, 0.0, -1.0], [-3.0, 2.0, -5.0]], [[0.0, 7.0, -2.0], [-7.0, 3.0, -5.0]], [[0.0, 7.0, -7.0], [-1.0, -4.0, 4.0]], [[0.0, 0.0, 2.0], [0.0, -3.0, -6.0]]], [[[-2.0, 6.0, 7.0], [4.0, -7.0, -2.0]], [[7.0, -2.0, 4.0], [3.0, 4.0, 3.0]], [[-7.0, -1.0, -4.0], [-2.0, 0.0, 3.0]], [[6.0, -5.0, 6.0], [-6.0, -6.0, 7.0]], [[4.0, 1.0, -1.0], [-1.0, -2.0, 2.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, -2.0, -3.0], [0.0, 6.0, 4.0]], [[-1.0, -2.0, 7.0], [-2.0, 0.0, -2.0]], [[-1.0, 1.0, -7.0], [6.0, -3.0, 7.0]], [[-6.0, -6.0, 4.0], [-7.0, -6.0, -6.0]], [[-4.0, 5.0, 4.0], [-1.0, 1.0, 6.0]]], [[[3.0, 3.0, 7.0], [-2.0, 6.0, 6.0]], [[-1.0, -5.0, 6.0], [-3.0, -2.0, 5.0]], [[-5.0, -5.0, -5.0], [4.0, 0.0, 6.0]], [[-5.0, 2.0, 0.0], [4.0, 6.0, 0.0]], [[0.0, 6.0, -3.0], [-1.0, -3.0, 4.0]]], [[[4.0, 7.0, -1.0], [-5.0, 3.0, -7.0]], [[-1.0, -6.0, -4.0], [5.0, -2.0, -3.0]], [[-2.0, 0.0, 0.0], [3.0, -7.0, 4.0]], [[3.0, 0.0, -1.0], [0.0, -1.0, 7.0]], [[-3.0, 0.0, -1.0], [0.0, 3.0, 0.0]]], [[[4.0, 1.0, -2.0], [7.0, -7.0, 0.0]], [[-2.0, 5.0, -6.0], [-2.0, 1.0, -1.0]], [[0.0, 0.0, -1.0], [0.0, -4.0, 1.0]], [[-5.0, -7.0, -4.0], [-5.0, -4.0, -7.0]], [[4.0, -1.0, -4.0], [4.0, 5.0, -4.0]]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[2.0, 1.0, 2.0], [0.0, -7.0, -2.0]], [[2.0, 1.0, 2.0], [7.0, -4.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, -9.0], [-13.0, -50.0], [19.0, 35.0], [55.0, 22.0], [46.0, 82.0]], [[2.0, 58.0], [-43.0, 33.0], [72.0, -17.0], [44.0, 45.0], [-24.0, 81.0]], [[29.0, -30.0], [-18.0, -78.0], [-8.0, -93.0], [13.0, 30.0], [37.0, -26.0]], [[69.0, 58.0], [-14.0, 46.0], [-29.0, -16.0], [47.0, 50.0], [17.0, 22.0]]])+(1.-msk_ref)*numpy.array([[[-64.0, -10.0], [14.0, -18.0], [-8.0, 88.0], [44.0, -77.0], [-14.0, 36.0]], [[-31.0, 27.0], [9.0, 27.0], [-37.0, 45.0], [-50.0, -4.0], [13.0, 33.0]], [[6.0, -83.0], [4.0, 6.0], [37.0, 73.0], [-3.0, 57.0], [-29.0, -20.0]], [[54.0, 82.0], [-16.0, -36.0], [24.0, 21.0], [17.0, -93.0], [-28.0, -21.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_Symbol_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -5.0, -5.0], [-5.0, 5.0, 4.0]], [[1.0, 0.0, 6.0], [0.0, -2.0, -1.0]], [[7.0, 0.0, -4.0], [7.0, 2.0, -1.0]], [[0.0, 0.0, 2.0], [-6.0, 5.0, -3.0]], [[-5.0, -6.0, -6.0], [-1.0, 0.0, 0.0]]], [[[2.0, 2.0, -4.0], [0.0, 0.0, -4.0]], [[2.0, 4.0, 6.0], [3.0, 0.0, -3.0]], [[-1.0, 4.0, 3.0], [5.0, 6.0, 0.0]], [[1.0, 5.0, -1.0], [0.0, -4.0, 3.0]], [[3.0, -3.0, -6.0], [3.0, 1.0, 2.0]]], [[[2.0, -3.0, 5.0], [7.0, -2.0, -7.0]], [[6.0, 5.0, 0.0], [-6.0, 0.0, -7.0]], [[-7.0, 4.0, 4.0], [-4.0, -3.0, -3.0]], [[-7.0, -4.0, 4.0], [-6.0, 6.0, -7.0]], [[0.0, 7.0, -6.0], [3.0, -7.0, 2.0]]], [[[-6.0, 3.0, 2.0], [3.0, -4.0, -2.0]], [[-7.0, 0.0, -1.0], [-1.0, 0.0, 5.0]], [[0.0, -7.0, -2.0], [4.0, 2.0, 0.0]], [[0.0, -4.0, -4.0], [-4.0, 5.0, 4.0]], [[-3.0, 0.0, -1.0], [7.0, -7.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, -4.0, -4.0], [3.0, -4.0, -2.0]], [[-6.0, 3.0, -2.0], [4.0, 4.0, 2.0]], [[-2.0, 1.0, -1.0], [0.0, 4.0, 0.0]], [[-2.0, 4.0, 2.0], [1.0, 2.0, -6.0]], [[7.0, 4.0, 5.0], [0.0, -3.0, 2.0]]], [[[-3.0, 3.0, 1.0], [-3.0, 5.0, 0.0]], [[-4.0, -6.0, -3.0], [5.0, -7.0, 3.0]], [[0.0, -1.0, -6.0], [7.0, -1.0, 7.0]], [[-7.0, -4.0, 4.0], [-3.0, -5.0, -7.0]], [[-2.0, 2.0, 0.0], [-4.0, 5.0, -5.0]]], [[[5.0, -7.0, 1.0], [-7.0, -4.0, -7.0]], [[3.0, -3.0, 3.0], [-3.0, 3.0, -3.0]], [[-4.0, 7.0, -6.0], [2.0, -4.0, 1.0]], [[4.0, 4.0, 6.0], [2.0, -4.0, -1.0]], [[0.0, -2.0, 6.0], [6.0, -5.0, 0.0]]], [[[7.0, -7.0, -4.0], [-4.0, -4.0, 0.0]], [[-2.0, 5.0, 6.0], [-2.0, 6.0, -1.0]], [[6.0, -2.0, 4.0], [-6.0, 4.0, -4.0]], [[0.0, 4.0, 3.0], [1.0, -4.0, 6.0]], [[-3.0, 5.0, 0.0], [2.0, -7.0, -4.0]]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=tensor_transposed_mult(arg0,arg1)
      s1=numpy.array([[[[-1.0, 3.0, 1.0], [-4.0, 2.0, 0.0]], [[3.0, 0.0, -7.0], [-6.0, 4.0, 7.0]], [[-6.0, 0.0, 2.0], [4.0, -1.0, 0.0]], [[-3.0, -3.0, -2.0], [-7.0, -7.0, -2.0]], [[4.0, -5.0, 6.0], [0.0, 0.0, -1.0]]], [[[-5.0, 7.0, -6.0], [-2.0, 0.0, 0.0]], [[1.0, 6.0, -4.0], [1.0, 2.0, 0.0]], [[0.0, -6.0, -4.0], [6.0, 3.0, 2.0]], [[3.0, 3.0, 6.0], [-4.0, 0.0, 5.0]], [[1.0, 6.0, 2.0], [-6.0, 1.0, -1.0]]], [[[-7.0, 7.0, 1.0], [1.0, 0.0, 0.0]], [[6.0, -4.0, 1.0], [-2.0, 0.0, 2.0]], [[-3.0, 7.0, -7.0], [6.0, -3.0, 7.0]], [[3.0, 0.0, 0.0], [-5.0, 0.0, -1.0]], [[-6.0, 0.0, 0.0], [5.0, -4.0, -2.0]]], [[[4.0, -1.0, 1.0], [-2.0, -4.0, 5.0]], [[-5.0, 4.0, -5.0], [5.0, 5.0, 4.0]], [[5.0, 6.0, 1.0], [1.0, 2.0, -4.0]], [[-6.0, 0.0, 0.0], [3.0, -6.0, 1.0]], [[1.0, 7.0, -2.0], [0.0, -6.0, -2.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[16.0, 95.0, 1.0, 35.0, -33.0], [35.0, -11.0, 43.0, -23.0, -15.0], [-3.0, -3.0, 1.0, 3.0, -17.0], [-14.0, 51.0, -76.0, -5.0, -69.0]], [[1.0, -54.0, 8.0, 1.0, 41.0], [-41.0, -27.0, -32.0, 34.0, 12.0], [-1.0, 10.0, -46.0, 4.0, 4.0], [13.0, -49.0, 11.0, 5.0, 3.0]], [[-35.0, 8.0, -24.0, -74.0, 5.0], [-25.0, 34.0, 62.0, -36.0, -40.0], [-46.0, 22.0, 36.0, -13.0, -13.0], [-3.0, 26.0, 46.0, -34.0, 5.0]], [[36.0, 21.0, -25.0, 9.0, 15.0], [0.0, -4.0, -35.0, 21.0, 48.0], [-4.0, 8.0, -86.0, 33.0, -44.0], [-21.0, -27.0, 18.0, -51.0, -28.0]], [[-15.0, 33.0, 14.0, 52.0, -26.0], [21.0, -18.0, 54.0, -65.0, -47.0], [-14.0, -10.0, 9.0, -10.0, 25.0], [-18.0, 26.0, -68.0, 27.0, -35.0]]], [[[0.0, 6.0, -20.0, 4.0, -22.0], [28.0, 30.0, -4.0, -32.0, 10.0], [-4.0, -8.0, 8.0, 10.0, -4.0], [-18.0, 2.0, 34.0, -16.0, 32.0]], [[4.0, -75.0, 12.0, -45.0, 27.0], [-24.0, 5.0, -36.0, 27.0, 23.0], [23.0, -10.0, -23.0, -6.0, 9.0], [-11.0, -21.0, 55.0, -6.0, 24.0]], [[8.0, -30.0, 26.0, -92.0, -6.0], [5.0, 28.0, 12.0, 7.0, 5.0], [43.0, -29.0, 22.0, -28.0, 7.0], [-39.0, 61.0, 39.0, -15.0, -15.0]], [[5.0, 15.0, -4.0, 6.0, -30.0], [36.0, 27.0, -32.0, 27.0, 22.0], [27.0, -9.0, 72.0, 0.0, 4.0], [29.0, 12.0, 14.0, 21.0, 56.0]], [[-28.0, 51.0, -19.0, -20.0, -11.0], [-6.0, 14.0, 67.0, -38.0, -46.0], [-45.0, 22.0, 41.0, -8.0, -11.0], [9.0, 31.0, -12.0, -13.0, -16.0]]], [[[-38.0, -128.0, 28.0, -28.0, 60.0], [-75.0, -33.0, 20.0, -36.0, -43.0], [-23.0, 1.0, -63.0, -22.0, 45.0], [-25.0, -50.0, 28.0, 14.0, -3.0]], [[33.0, 5.0, -60.0, 23.0, 6.0], [17.0, 30.0, -80.0, 22.0, 79.0], [-13.0, 14.0, -68.0, 55.0, -52.0], [-4.0, -68.0, 82.0, -61.0, 55.0]], [[33.0, -58.0, 37.0, 56.0, -21.0], [47.0, -9.0, -79.0, 16.0, 49.0], [77.0, -52.0, -15.0, 2.0, 40.0], [-23.0, -16.0, -5.0, 45.0, 37.0]], [[35.0, -38.0, 20.0, 39.0, 23.0], [-5.0, -41.0, -24.0, -20.0, 26.0], [19.0, -24.0, -138.0, 16.0, 2.0], [-67.0, -29.0, -21.0, -19.0, -65.0]], [[-11.0, 10.0, 7.0, 15.0, -73.0], [79.0, 55.0, -17.0, -17.0, 3.0], [46.0, -36.0, 144.0, -17.0, 39.0], [19.0, 46.0, 17.0, 53.0, 99.0]]], [[[-3.0, -80.0, 56.0, 16.0, -25.0], [33.0, -1.0, -24.0, -19.0, -4.0], [68.0, -56.0, 41.0, -31.0, 71.0], [-25.0, 19.0, -7.0, 67.0, 39.0]], [[10.0, 27.0, 36.0, 20.0, -39.0], [43.0, -4.0, 8.0, 2.0, -8.0], [47.0, -31.0, 57.0, -21.0, 27.0], [-2.0, 55.0, -57.0, 44.0, -15.0]], [[-35.0, -2.0, 10.0, -17.0, 23.0], [-45.0, -26.0, 80.0, -49.0, -68.0], [-47.0, 18.0, -17.0, -20.0, 12.0], [-11.0, 12.0, -36.0, 0.0, -57.0]], [[10.0, 100.0, -29.0, 5.0, -8.0], [4.0, -2.0, 39.0, 0.0, -7.0], [-36.0, 28.0, -11.0, 16.0, -48.0], [8.0, 25.0, -38.0, -38.0, -58.0]], [[-40.0, -107.0, 51.0, 21.0, -13.0], [7.0, -6.0, 15.0, -68.0, -49.0], [27.0, -43.0, 44.0, -39.0, 91.0], [-24.0, 0.0, -3.0, 76.0, 51.0]]]])+(1.-msk_ref)*numpy.array([[[[-31.0, -35.0, 38.0, 46.0, -22.0], [15.0, -18.0, 42.0, -73.0, -57.0], [6.0, -28.0, 31.0, -28.0, 65.0], [-20.0, 16.0, -50.0, 61.0, 3.0]], [[5.0, 2.0, 44.0, -47.0, -53.0], [55.0, 32.0, 30.0, -27.0, -14.0], [65.0, -54.0, 79.0, -40.0, 36.0], [-43.0, 100.0, -10.0, 26.0, -9.0]], [[12.0, 17.0, 6.0, -23.0, -19.0], [23.0, 16.0, 10.0, -9.0, 6.0], [20.0, -17.0, 8.0, -6.0, -4.0], [-26.0, 39.0, 3.0, -12.0, -17.0]], [[16.0, -60.0, 18.0, -19.0, -10.0], [24.0, 19.0, -32.0, -16.0, 28.0], [45.0, -40.0, -22.0, -5.0, 21.0], [-50.0, 7.0, 45.0, -3.0, 22.0]], [[4.0, -12.0, -29.0, -26.0, 36.0], [-37.0, 5.0, -49.0, 73.0, 36.0], [-16.0, 35.0, -5.0, 19.0, -34.0], [51.0, -51.0, 50.0, -22.0, 39.0]]], [[[35.0, 22.0, 3.0, -16.0, -21.0], [36.0, 18.0, -25.0, 18.0, 40.0], [40.0, -23.0, -10.0, 6.0, -17.0], [-28.0, 32.0, 11.0, -21.0, -14.0]], [[-51.0, -28.0, 45.0, 44.0, -7.0], [-14.0, -37.0, 63.0, -53.0, -86.0], [-12.0, -7.0, 63.0, -40.0, 71.0], [20.0, 13.0, -80.0, 84.0, -4.0]], [[-39.0, 45.0, 17.0, -41.0, -38.0], [15.0, 23.0, 83.0, -32.0, -68.0], [-6.0, -2.0, 129.0, -42.0, 25.0], [20.0, 84.0, -35.0, 34.0, -3.0]], [[1.0, -100.0, 43.0, 95.0, 23.0], [-11.0, -60.0, -39.0, -32.0, -3.0], [22.0, -30.0, -87.0, 1.0, 61.0], [-29.0, -69.0, -40.0, 56.0, 1.0]], [[34.0, 3.0, -9.0, 3.0, -13.0], [32.0, 16.0, -31.0, -9.0, 44.0], [24.0, -22.0, -54.0, 19.0, -18.0], [-47.0, 3.0, 28.0, -35.0, -8.0]]], [[[-5.0, -15.0, -52.0, 95.0, 68.0], [-66.0, -56.0, -30.0, -7.0, 10.0], [-90.0, 59.0, -150.0, 57.0, -35.0], [23.0, -141.0, -3.0, -34.0, -8.0]], [[9.0, -3.0, -27.0, 0.0, 48.0], [-48.0, -24.0, -9.0, 15.0, 15.0], [-42.0, 33.0, -99.0, 27.0, -39.0], [-3.0, -54.0, 15.0, -48.0, -36.0]], [[3.0, 9.0, 24.0, 15.0, -88.0], [101.0, 56.0, -16.0, -30.0, 9.0], [73.0, -60.0, 134.0, -23.0, 48.0], [-12.0, 72.0, 6.0, 55.0, 79.0]], [[-2.0, -65.0, 0.0, -20.0, 33.0], [-32.0, -2.0, -50.0, 47.0, 25.0], [8.0, 8.0, -9.0, 3.0, 4.0], [25.0, -48.0, 48.0, 5.0, 46.0]], [[-34.0, -98.0, 41.0, -13.0, 46.0], [-62.0, -40.0, 9.0, 6.0, -41.0], [-2.0, 2.0, -5.0, -30.0, 50.0], [16.0, -33.0, -10.0, 48.0, 4.0]]], [[[-24.0, 57.0, -62.0, 64.0, 39.0], [-52.0, -31.0, 22.0, -8.0, -23.0], [-106.0, 74.0, -54.0, 41.0, -46.0], [55.0, -83.0, -23.0, -30.0, -10.0]], [[43.0, -19.0, 10.0, -47.0, 4.0], [13.0, 14.0, -50.0, 48.0, 59.0], [53.0, -24.0, -38.0, 5.0, -20.0], [-32.0, 16.0, 40.0, -31.0, -13.0]], [[24.0, 14.0, -56.0, 2.0, 62.0], [-56.0, -20.0, -36.0, 40.0, 46.0], [-58.0, 52.0, -136.0, 52.0, -74.0], [6.0, -84.0, 40.0, -82.0, -32.0]], [[3.0, -1.0, 14.0, -9.0, -8.0], [8.0, 5.0, -30.0, 56.0, 14.0], [32.0, -3.0, 67.0, -11.0, 9.0], [43.0, 10.0, -4.0, 33.0, 34.0]], [[-4.0, -77.0, 33.0, 37.0, -33.0], [46.0, 15.0, -47.0, -22.0, 12.0], [58.0, -50.0, 49.0, -15.0, 64.0], [-13.0, -6.0, 19.0, 62.0, 82.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_constData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, -5.0, 1.0], [3.0, -6.0, -2.0]], [[-3.0, -2.0, 7.0], [-4.0, -3.0, 1.0]], [[3.0, -7.0, -2.0], [7.0, 4.0, 5.0]], [[0.0, 3.0, -7.0], [-2.0, 5.0, -1.0]], [[-2.0, 3.0, 7.0], [4.0, 7.0, -1.0]]], [[[-3.0, -4.0, 4.0], [-6.0, 0.0, 3.0]], [[-4.0, 0.0, 3.0], [-4.0, -5.0, -3.0]], [[-2.0, 0.0, -3.0], [-6.0, -4.0, 1.0]], [[0.0, 0.0, 3.0], [6.0, 7.0, 5.0]], [[7.0, -4.0, -6.0], [-5.0, 0.0, 5.0]]], [[[-2.0, -5.0, 1.0], [0.0, 3.0, 4.0]], [[-3.0, 7.0, -4.0], [0.0, 4.0, 0.0]], [[2.0, 0.0, 0.0], [-4.0, 6.0, 4.0]], [[0.0, 3.0, 0.0], [3.0, -2.0, 7.0]], [[-3.0, -6.0, -1.0], [-4.0, 2.0, 6.0]]], [[[-2.0, 7.0, -6.0], [7.0, 3.0, 3.0]], [[-2.0, -2.0, 5.0], [2.0, -4.0, 5.0]], [[1.0, 3.0, -5.0], [1.0, -4.0, -1.0]], [[4.0, -4.0, 4.0], [4.0, 2.0, -3.0]], [[1.0, -3.0, 7.0], [2.0, -6.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, -7.0, 1.0], [6.0, -3.0, 6.0]], [[1.0, -6.0, 6.0], [-5.0, 7.0, 3.0]], [[-6.0, 6.0, 4.0], [-4.0, 7.0, 6.0]], [[-3.0, -5.0, 2.0], [-6.0, -3.0, -6.0]], [[1.0, -6.0, 7.0], [5.0, 0.0, -6.0]]], [[[2.0, 6.0, 3.0], [-1.0, 5.0, 5.0]], [[3.0, 2.0, 4.0], [-7.0, 6.0, -3.0]], [[-3.0, 6.0, 6.0], [0.0, -3.0, -3.0]], [[-2.0, 4.0, -2.0], [-3.0, -7.0, 3.0]], [[6.0, -2.0, 6.0], [3.0, 3.0, -6.0]]], [[[-1.0, -5.0, 6.0], [0.0, 4.0, 1.0]], [[-3.0, -3.0, -3.0], [7.0, -5.0, -3.0]], [[3.0, 1.0, -5.0], [1.0, -5.0, -2.0]], [[1.0, 7.0, 7.0], [-7.0, 3.0, 2.0]], [[-3.0, 6.0, 1.0], [-3.0, 4.0, 7.0]]], [[[4.0, -4.0, 7.0], [5.0, -5.0, -5.0]], [[3.0, 0.0, -6.0], [-6.0, 2.0, 7.0]], [[-2.0, -4.0, 5.0], [6.0, -5.0, -6.0]], [[-1.0, -1.0, 7.0], [4.0, -3.0, 3.0]], [[7.0, 0.0, 7.0], [0.0, -6.0, 6.0]]]])
      arg1=Data(numpy.array([[-2.0, 5.0, 4.0], [7.0, -7.0, 0.0]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[28.0, 17.0, -28.0, -62.0, 26.0], [-40.0, 27.0, -22.0, 5.0, -93.0], [-38.0, -3.0, -74.0, 50.0, -70.0], [43.0, 56.0, 28.0, 2.0, 67.0]])+(1.-msk_ref)*numpy.array([[20.0, -92.0, -19.0, -32.0, 31.0], [-4.0, -71.0, 81.0, 44.0, 2.0], [-27.0, 63.0, 21.0, -9.0, -9.0], [70.0, -86.0, 81.0, 74.0, 56.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_constData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -2.0, 0.0], [7.0, -4.0, -2.0]], [[2.0, -2.0, -1.0], [4.0, -5.0, 2.0]], [[0.0, 0.0, 5.0], [-2.0, 1.0, 0.0]], [[7.0, 6.0, 3.0], [-7.0, 5.0, 7.0]], [[-2.0, 7.0, -2.0], [-5.0, -2.0, 6.0]]], [[[6.0, -7.0, -5.0], [6.0, -7.0, 0.0]], [[2.0, 2.0, 1.0], [7.0, 0.0, -7.0]], [[-7.0, 0.0, -6.0], [0.0, 0.0, -4.0]], [[5.0, 6.0, -4.0], [-5.0, -6.0, 5.0]], [[4.0, -4.0, 1.0], [0.0, -3.0, 0.0]]], [[[5.0, 4.0, -3.0], [3.0, -5.0, -2.0]], [[1.0, 1.0, -1.0], [-5.0, 1.0, 2.0]], [[5.0, -7.0, 6.0], [3.0, -6.0, 1.0]], [[-3.0, 1.0, 2.0], [-6.0, 0.0, 3.0]], [[3.0, -5.0, -3.0], [-7.0, 7.0, 6.0]]], [[[-1.0, 0.0, -4.0], [1.0, 2.0, 1.0]], [[0.0, -7.0, 3.0], [0.0, 0.0, -7.0]], [[0.0, 7.0, -3.0], [-3.0, 2.0, -2.0]], [[-2.0, -5.0, 0.0], [5.0, 6.0, 5.0]], [[7.0, 4.0, 1.0], [1.0, 0.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, -3.0, 5.0], [-1.0, 3.0, -3.0]], [[4.0, 0.0, -6.0], [-4.0, 4.0, -1.0]], [[-6.0, 6.0, 6.0], [1.0, 5.0, 5.0]], [[-1.0, 0.0, 0.0], [-2.0, 0.0, 6.0]], [[-1.0, 3.0, 1.0], [-2.0, -3.0, -5.0]]], [[[0.0, -2.0, 7.0], [2.0, 2.0, 2.0]], [[-2.0, -6.0, 0.0], [-1.0, 1.0, -1.0]], [[0.0, -2.0, 0.0], [-6.0, 4.0, -2.0]], [[4.0, 7.0, -4.0], [0.0, -3.0, -6.0]], [[3.0, 0.0, -7.0], [7.0, -7.0, 1.0]]], [[[3.0, -2.0, -5.0], [-2.0, 4.0, -5.0]], [[-7.0, -3.0, 5.0], [-3.0, -3.0, -6.0]], [[5.0, -4.0, -6.0], [0.0, 4.0, 1.0]], [[5.0, -5.0, 3.0], [-7.0, 4.0, -4.0]], [[-3.0, 5.0, -3.0], [6.0, -7.0, 4.0]]], [[[6.0, 2.0, -7.0], [-3.0, -7.0, 3.0]], [[6.0, -7.0, -3.0], [-3.0, 0.0, -1.0]], [[6.0, -2.0, -1.0], [-2.0, 2.0, -3.0]], [[-2.0, -6.0, 6.0], [1.0, 6.0, 4.0]], [[4.0, -2.0, -2.0], [3.0, 2.0, -1.0]]]])
      arg1=Data(numpy.array([[[-5.0, 4.0, 4.0], [3.0, 1.0, 4.0]], [[-3.0, -7.0, -1.0], [-3.0, -1.0, 5.0]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[31.0, 5.0], [-7.0, 12.0], [15.0, 0.0], [13.0, -15.0], [37.0, 6.0]], [[-67.0, 25.0], [-5.0, -77.0], [-5.0, 7.0], [-18.0, -7.0], [-35.0, 18.0]], [[-25.0, -54.0], [-11.0, 15.0], [-22.0, 30.0], [21.0, 33.0], [-37.0, 73.0]], [[-2.0, 7.0], [-44.0, 11.0], [1.0, -49.0], [31.0, 45.0], [16.0, -18.0]]])+(1.-msk_ref)*numpy.array([[[-34.0, -17.0], [-56.0, -3.0], [106.0, -13.0], [23.0, 39.0], [-8.0, -35.0]], [[36.0, 9.0], [-20.0, 45.0], [-30.0, 18.0], [-35.0, -84.0], [-25.0, -11.0]], [[-65.0, -13.0], [7.0, 19.0], [-57.0, 20.0], [-66.0, 14.0], [50.0, -14.0]], [[-54.0, 6.0], [-83.0, 38.0], [-58.0, -14.0], [35.0, 53.0], [-29.0, -12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_constData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -1.0, 2.0], [-3.0, 3.0, -5.0]], [[3.0, 1.0, 6.0], [-6.0, 6.0, -1.0]], [[-2.0, 0.0, -1.0], [-5.0, -1.0, 4.0]], [[0.0, 4.0, 5.0], [4.0, 0.0, -3.0]], [[-2.0, 0.0, -1.0], [6.0, -5.0, 0.0]]], [[[6.0, 0.0, 5.0], [-2.0, 3.0, 0.0]], [[0.0, -2.0, 6.0], [-3.0, -3.0, -7.0]], [[-4.0, -1.0, -1.0], [5.0, 5.0, 1.0]], [[1.0, -7.0, -3.0], [-4.0, 3.0, 3.0]], [[6.0, 0.0, -3.0], [-5.0, 1.0, -3.0]]], [[[-5.0, 3.0, 7.0], [-6.0, 1.0, -3.0]], [[-7.0, -5.0, 0.0], [-6.0, 6.0, -6.0]], [[5.0, 5.0, -1.0], [0.0, 5.0, 3.0]], [[-4.0, -1.0, -4.0], [-3.0, 3.0, 7.0]], [[0.0, 7.0, -1.0], [0.0, -6.0, -3.0]]], [[[4.0, 3.0, 5.0], [-6.0, -2.0, -3.0]], [[6.0, 0.0, -7.0], [2.0, 0.0, 4.0]], [[-4.0, 4.0, 6.0], [0.0, 4.0, 5.0]], [[4.0, 0.0, 5.0], [4.0, -4.0, 1.0]], [[-2.0, -1.0, -3.0], [-3.0, 0.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-2.0, -6.0, 4.0], [-7.0, -5.0, -3.0]], [[4.0, 2.0, 2.0], [-7.0, -5.0, -6.0]], [[-5.0, -2.0, 3.0], [7.0, 3.0, 7.0]], [[-7.0, 6.0, -1.0], [1.0, 6.0, -5.0]], [[1.0, -3.0, 3.0], [-1.0, -4.0, -3.0]]], [[[-5.0, -6.0, 5.0], [5.0, -2.0, 5.0]], [[-4.0, 6.0, -1.0], [4.0, 4.0, 0.0]], [[-3.0, -5.0, 0.0], [-6.0, -4.0, -5.0]], [[-1.0, 3.0, -2.0], [0.0, 0.0, -7.0]], [[2.0, 0.0, 1.0], [-1.0, -1.0, -1.0]]], [[[7.0, 4.0, 2.0], [1.0, 4.0, -2.0]], [[6.0, -5.0, 0.0], [0.0, 7.0, 7.0]], [[-1.0, 4.0, -5.0], [-4.0, -6.0, -6.0]], [[-3.0, 0.0, -5.0], [-4.0, 5.0, -1.0]], [[5.0, -2.0, -3.0], [0.0, 0.0, 0.0]]], [[[-7.0, -2.0, -2.0], [5.0, -7.0, 6.0]], [[5.0, -7.0, -4.0], [4.0, 6.0, 3.0]], [[3.0, 1.0, -6.0], [-7.0, 2.0, 0.0]], [[6.0, 2.0, -4.0], [7.0, -5.0, 2.0]], [[3.0, -7.0, -6.0], [-6.0, 6.0, -5.0]]]])
      arg1=Data(numpy.array([[[[6.0, 0.0, 3.0], [3.0, -1.0, 3.0]], [[-3.0, -4.0, 7.0], [-7.0, -4.0, 0.0]], [[-1.0, -1.0, 0.0], [-1.0, 0.0, -4.0]], [[-6.0, -1.0, -4.0], [0.0, 4.0, 0.0]], [[1.0, 7.0, -4.0], [-1.0, -5.0, 1.0]]], [[[4.0, -4.0, -7.0], [-5.0, 5.0, -6.0]], [[6.0, 4.0, 0.0], [-5.0, 4.0, 3.0]], [[-6.0, -2.0, -7.0], [-6.0, 4.0, 0.0]], [[3.0, 2.0, -6.0], [7.0, 0.0, 1.0]], [[6.0, -6.0, 0.0], [2.0, 5.0, 2.0]]], [[[-4.0, 0.0, 5.0], [-5.0, 3.0, 3.0]], [[0.0, 7.0, 0.0], [0.0, 4.0, 3.0]], [[-6.0, 0.0, 0.0], [6.0, 2.0, 2.0]], [[7.0, 0.0, -5.0], [-3.0, 7.0, 7.0]], [[-2.0, 0.0, -7.0], [-3.0, -3.0, -4.0]]], [[[3.0, 0.0, 1.0], [5.0, 5.0, 6.0]], [[-7.0, -6.0, 7.0], [2.0, 1.0, -3.0]], [[0.0, 0.0, -2.0], [-4.0, 0.0, 4.0]], [[-6.0, -6.0, 7.0], [1.0, 5.0, -5.0]], [[-4.0, 0.0, 6.0], [-6.0, -3.0, 5.0]]]]),self.functionspace)
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[15.0, 9.0, 18.0, -31.0, -26.0], [74.0, 44.0, -18.0, -22.0, 41.0], [-5.0, -10.0, -58.0, 27.0, -6.0], [-10.0, -10.0, -12.0, 21.0, -28.0]], [[9.0, 47.0, 6.0, -19.0, -39.0], [32.0, 73.0, -2.0, -68.0, 28.0], [63.0, 28.0, -44.0, 44.0, -44.0], [9.0, 12.0, 8.0, 47.0, 37.0]], [[-17.0, 38.0, -9.0, 12.0, 16.0], [-5.0, 21.0, 45.0, -31.0, -19.0], [37.0, 8.0, -12.0, 27.0, 13.0], [-13.0, -16.0, 38.0, -25.0, 55.0]], [[18.0, -9.0, 4.0, -24.0, 1.0], [-53.0, -13.0, -67.0, 3.0, -22.0], [-4.0, 19.0, 18.0, -58.0, -35.0], [7.0, 28.0, -38.0, 30.0, -9.0]], [[8.0, -23.0, -4.0, -4.0, 21.0], [-56.0, -62.0, -37.0, 42.0, -25.0], [-42.0, -20.0, 38.0, -62.0, 8.0], [-2.0, 14.0, -22.0, -14.0, -19.0]]], [[[42.0, 19.0, -4.0, -44.0, -27.0], [14.0, 58.0, -47.0, -26.0, 47.0], [20.0, 12.0, -42.0, 44.0, -50.0], [28.0, -8.0, -2.0, 12.0, 9.0]], [[-9.0, 83.0, 33.0, -34.0, -27.0], [8.0, -26.0, -32.0, -68.0, -23.0], [15.0, -47.0, -38.0, -91.0, 4.0], [-66.0, 66.0, -28.0, 71.0, 28.0]], [[-14.0, -46.0, -4.0, 49.0, -36.0], [-11.0, -30.0, 23.0, 28.0, 19.0], [4.0, 16.0, 66.0, 4.0, -19.0], [43.0, 39.0, -14.0, 48.0, -30.0]], [[-9.0, 20.0, -2.0, 25.0, -44.0], [70.0, 19.0, 65.0, -18.0, 61.0], [19.0, -28.0, -18.0, 76.0, 10.0], [13.0, 0.0, 34.0, 11.0, 8.0]], [[2.0, -8.0, 11.0, -20.0, 15.0], [93.0, 56.0, 19.0, -2.0, 25.0], [-20.0, -5.0, -70.0, 58.0, 33.0], [-23.0, -63.0, 14.0, -42.0, -30.0]]], [[[-37.0, 90.0, 20.0, 3.0, -14.0], [-28.0, 7.0, 15.0, -96.0, -61.0], [79.0, 16.0, -10.0, -66.0, -12.0], [-51.0, 64.0, -2.0, 75.0, 80.0]], [[-84.0, 59.0, 42.0, 71.0, -72.0], [88.0, -26.0, 112.0, -79.0, -6.0], [58.0, -29.0, 6.0, -31.0, 38.0], [-57.0, 91.0, 0.0, 126.0, 16.0]], [[31.0, -62.0, -22.0, -11.0, 22.0], [14.0, 79.0, -13.0, 34.0, 31.0], [-1.0, 64.0, -14.0, 96.0, -30.0], [57.0, -76.0, 14.0, -57.0, -26.0]], [[-27.0, -3.0, -20.0, 53.0, 0.0], [4.0, 20.0, 84.0, -4.0, 5.0], [41.0, 26.0, 26.0, 71.0, 8.0], [26.0, -18.0, 48.0, -21.0, 36.0]], [[-6.0, -11.0, 5.0, -27.0, 80.0], [-33.0, -5.0, -31.0, 17.0, -78.0], [-32.0, 16.0, -18.0, -58.0, 37.0], [-49.0, -46.0, -10.0, -64.0, -3.0]]], [[[14.0, 61.0, 11.0, -55.0, 18.0], [7.0, 49.0, -37.0, -57.0, -22.0], [24.0, 4.0, -70.0, -14.0, -7.0], [-41.0, -16.0, 2.0, -8.0, 41.0]], [[33.0, -81.0, -24.0, -8.0, 36.0], [39.0, 38.0, 1.0, 78.0, 48.0], [-57.0, 12.0, -16.0, 99.0, 15.0], [45.0, -99.0, 22.0, -103.0, -58.0]], [[5.0, 22.0, -20.0, 12.0, -15.0], [-84.0, 23.0, -10.0, -35.0, -18.0], [73.0, 59.0, 42.0, 5.0, -66.0], [44.0, 35.0, 8.0, 37.0, 65.0]], [[58.0, 11.0, -12.0, -60.0, 1.0], [-65.0, -9.0, -99.0, 11.0, 14.0], [-20.0, -13.0, -6.0, -30.0, -47.0], [23.0, 8.0, -22.0, -10.0, 7.0]], [[-12.0, 10.0, -18.0, 25.0, 12.0], [-4.0, 17.0, 53.0, -5.0, 0.0], [26.0, 11.0, 6.0, 52.0, 10.0], [12.0, -25.0, 42.0, -36.0, 38.0]]]])+(1.-msk_ref)*numpy.array([[[[-25.0, 127.0, 27.0, -18.0, -31.0], [16.0, -30.0, 18.0, -94.0, -21.0], [39.0, -71.0, -46.0, -69.0, 24.0], [-80.0, 68.0, 8.0, 59.0, 74.0]], [[-4.0, 63.0, 25.0, -54.0, 36.0], [40.0, 29.0, -20.0, -51.0, -39.0], [-4.0, -24.0, -88.0, -38.0, 38.0], [-82.0, -27.0, 0.0, -24.0, 23.0]], [[18.0, -17.0, -28.0, 32.0, -46.0], [-95.0, -40.0, -17.0, 19.0, 25.0], [30.0, 19.0, 92.0, -1.0, -69.0], [80.0, 64.0, -6.0, 50.0, 22.0]], [[-63.0, -41.0, 20.0, 64.0, 3.0], [10.0, -14.0, 55.0, -1.0, -56.0], [21.0, 51.0, 50.0, -40.0, 20.0], [-17.0, 29.0, -22.0, 55.0, -27.0]], [[7.0, 53.0, 15.0, -31.0, -14.0], [-2.0, -26.0, -31.0, -31.0, -4.0], [-5.0, -46.0, -26.0, -54.0, 4.0], [-37.0, 35.0, -14.0, 27.0, 17.0]]], [[[17.0, 47.0, -14.0, 8.0, -57.0], [-96.0, -72.0, -31.0, -17.0, 16.0], [29.0, -35.0, 66.0, -54.0, -54.0], [35.0, 99.0, -10.0, 71.0, 51.0]], [[-19.0, -63.0, -6.0, 38.0, 18.0], [-33.0, -4.0, 11.0, 34.0, -32.0], [3.0, 58.0, 56.0, -7.0, -9.0], [27.0, -3.0, -14.0, 5.0, -26.0]], [[-47.0, 87.0, 34.0, 7.0, -17.0], [48.0, -39.0, 48.0, -66.0, -30.0], [15.0, -66.0, -36.0, -66.0, 56.0], [-89.0, 50.0, 4.0, 47.0, 35.0]], [[-33.0, -23.0, 26.0, 11.0, 21.0], [40.0, -15.0, 14.0, 8.0, -38.0], [-27.0, 0.0, -8.0, -46.0, 44.0], [-47.0, -4.0, -24.0, 9.0, -43.0]], [[10.0, 12.0, 3.0, -20.0, 3.0], [7.0, 10.0, -17.0, -8.0, 3.0], [-4.0, -7.0, -22.0, -2.0, -1.0], [-9.0, -7.0, -2.0, -6.0, 2.0]]], [[[41.0, -46.0, -4.0, -38.0, 4.0], [25.0, 63.0, -54.0, 22.0, 36.0], [-17.0, 38.0, -32.0, 50.0, -35.0], [36.0, -47.0, -16.0, -21.0, -44.0]], [[50.0, -26.0, -29.0, -3.0, -57.0], [37.0, 65.0, 2.0, 15.0, 115.0], [18.0, 14.0, -8.0, 140.0, -61.0], [95.0, -26.0, 28.0, -6.0, -10.0]], [[-45.0, 4.0, 25.0, -2.0, 75.0], [41.0, -12.0, 33.0, 1.0, -80.0], [-37.0, -14.0, -42.0, -54.0, 91.0], [-94.0, -48.0, 2.0, -57.0, -14.0]], [[-53.0, -18.0, 11.0, 58.0, -5.0], [74.0, 19.0, 97.0, -8.0, -3.0], [19.0, 17.0, 2.0, 44.0, 42.0], [-15.0, -14.0, 22.0, 9.0, -14.0]], [[21.0, -28.0, -3.0, -16.0, 3.0], [49.0, 22.0, -5.0, 29.0, 42.0], [-35.0, -14.0, -30.0, 50.0, 11.0], [12.0, -44.0, 6.0, -39.0, -38.0]]], [[[-8.0, 8.0, -20.0, 24.0, 23.0], [-102.0, -85.0, 2.0, 28.0, -43.0], [-10.0, -24.0, 70.0, -61.0, 10.0], [3.0, 32.0, 8.0, -20.0, 37.0]], [[33.0, -67.0, -14.0, 17.0, -59.0], [68.0, 15.0, 12.0, 56.0, 116.0], [-33.0, -16.0, 12.0, 106.0, -24.0], [79.0, -16.0, 4.0, 3.0, -71.0]], [[-23.0, -14.0, 3.0, 13.0, 31.0], [95.0, 65.0, 72.0, -2.0, 8.0], [-1.0, 15.0, -56.0, 86.0, 51.0], [-22.0, -81.0, 40.0, -63.0, -12.0]], [[56.0, -83.0, -23.0, -42.0, 56.0], [-28.0, -5.0, -74.0, 97.0, 17.0], [-88.0, 0.0, 0.0, 20.0, 2.0], [36.0, -79.0, -12.0, -104.0, -65.0]], [[-39.0, -5.0, 30.0, 37.0, -51.0], [172.0, 29.0, 98.0, -16.0, 68.0], [-9.0, -40.0, -52.0, 76.0, 56.0], [-27.0, -12.0, 16.0, 31.0, -55.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_expandedData_rank2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, 0.0, 1.0], [2.0, 3.0, 3.0]], [[6.0, -4.0, 4.0], [2.0, 3.0, -6.0]], [[-5.0, 5.0, -2.0], [0.0, -2.0, 0.0]], [[-5.0, 1.0, -1.0], [2.0, 5.0, -7.0]], [[0.0, 5.0, -2.0], [7.0, -3.0, -6.0]]], [[[3.0, 4.0, 1.0], [-1.0, 1.0, 3.0]], [[4.0, 0.0, -1.0], [5.0, 0.0, 0.0]], [[3.0, 0.0, 0.0], [6.0, 5.0, 4.0]], [[7.0, -7.0, 4.0], [6.0, -3.0, -2.0]], [[-3.0, -6.0, 4.0], [-3.0, 4.0, 1.0]]], [[[0.0, 7.0, 6.0], [7.0, 5.0, 3.0]], [[-4.0, -5.0, 5.0], [-1.0, 4.0, -4.0]], [[6.0, -5.0, -7.0], [7.0, -4.0, -6.0]], [[-7.0, 7.0, -7.0], [-2.0, 6.0, 3.0]], [[-6.0, -2.0, 3.0], [-6.0, -3.0, 7.0]]], [[[-1.0, 4.0, 6.0], [-6.0, 4.0, 2.0]], [[1.0, 3.0, 2.0], [-5.0, -6.0, 3.0]], [[0.0, 0.0, 4.0], [-6.0, -2.0, 5.0]], [[2.0, -5.0, 0.0], [1.0, -3.0, -1.0]], [[3.0, 1.0, -7.0], [0.0, -5.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[2.0, 5.0, -5.0], [-1.0, -6.0, 0.0]], [[-6.0, 3.0, 7.0], [5.0, -2.0, 0.0]], [[-4.0, 4.0, 2.0], [4.0, -4.0, -1.0]], [[3.0, 0.0, -4.0], [5.0, 6.0, 2.0]], [[6.0, -7.0, -4.0], [-7.0, -1.0, 5.0]]], [[[1.0, -4.0, 3.0], [-5.0, -1.0, 0.0]], [[-5.0, 2.0, -1.0], [2.0, 5.0, -6.0]], [[-4.0, -6.0, 2.0], [0.0, 2.0, -7.0]], [[-5.0, 6.0, 2.0], [7.0, -6.0, 0.0]], [[-3.0, -3.0, -5.0], [-3.0, 4.0, -7.0]]], [[[6.0, 2.0, -2.0], [6.0, -5.0, -6.0]], [[6.0, -2.0, 6.0], [-4.0, -2.0, -4.0]], [[2.0, 6.0, -5.0], [-4.0, -2.0, 2.0]], [[4.0, 2.0, -3.0], [1.0, 6.0, 0.0]], [[2.0, -7.0, -3.0], [6.0, 3.0, 0.0]]], [[[-2.0, 7.0, -5.0], [-5.0, 1.0, -4.0]], [[7.0, -2.0, 3.0], [7.0, 6.0, -1.0]], [[0.0, -4.0, 7.0], [6.0, -3.0, 0.0]], [[-7.0, -1.0, 6.0], [-4.0, 0.0, 5.0]], [[-6.0, 4.0, 5.0], [4.0, 4.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 6.0, 0.0], [3.0, -6.0, -2.0]])+(1.-msk_arg1)*numpy.array([[2.0, 7.0, -5.0], [1.0, -1.0, 2.0]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, -54.0, 67.0, 21.0, 81.0], [-6.0, -5.0, -35.0, -37.0, -56.0], [27.0, -29.0, -3.0, 29.0, 4.0], [-17.0, 28.0, -16.0, -17.0, 21.0]])+(1.-msk_ref)*numpy.array([[69.0, -19.0, 16.0, 29.0, -13.0], [-45.0, -6.0, -76.0, 35.0, -23.0], [35.0, -42.0, 73.0, 32.0, -27.0], [56.0, -16.0, -54.0, -45.0, -19.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_expandedData_rank3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 5.0, -3.0], [2.0, 6.0, -3.0]], [[6.0, 0.0, 5.0], [0.0, 2.0, 1.0]], [[-3.0, 2.0, 4.0], [0.0, 7.0, 0.0]], [[-7.0, -6.0, 5.0], [2.0, 7.0, 5.0]], [[0.0, 2.0, -5.0], [-7.0, 5.0, -5.0]]], [[[-2.0, -5.0, -4.0], [-3.0, 2.0, -3.0]], [[0.0, -2.0, 7.0], [-3.0, -1.0, 3.0]], [[2.0, 5.0, 3.0], [3.0, -5.0, -5.0]], [[7.0, -3.0, -2.0], [0.0, -1.0, -4.0]], [[-3.0, 3.0, 7.0], [-3.0, -5.0, -5.0]]], [[[5.0, 3.0, -6.0], [3.0, -4.0, 0.0]], [[-6.0, -6.0, 6.0], [6.0, 0.0, 5.0]], [[2.0, 2.0, 7.0], [3.0, -6.0, -4.0]], [[5.0, -2.0, 0.0], [1.0, 4.0, 0.0]], [[-6.0, -2.0, -5.0], [-3.0, -1.0, -3.0]]], [[[-6.0, 0.0, 5.0], [-3.0, 4.0, -5.0]], [[5.0, 0.0, -2.0], [5.0, 2.0, 1.0]], [[-2.0, 0.0, -7.0], [-7.0, -7.0, -4.0]], [[-1.0, 3.0, 0.0], [2.0, -5.0, 0.0]], [[4.0, 2.0, -7.0], [5.0, -2.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 3.0, 5.0], [0.0, 2.0, -2.0]], [[-4.0, 0.0, -2.0], [-6.0, -3.0, -2.0]], [[5.0, 5.0, 1.0], [2.0, -7.0, 2.0]], [[0.0, -3.0, -2.0], [-4.0, 1.0, -5.0]], [[7.0, -6.0, -5.0], [4.0, -7.0, 0.0]]], [[[6.0, 1.0, 1.0], [1.0, -7.0, -1.0]], [[5.0, -3.0, 5.0], [-1.0, 2.0, 1.0]], [[3.0, 6.0, -5.0], [6.0, 5.0, 0.0]], [[-3.0, -2.0, -7.0], [-6.0, -1.0, 2.0]], [[1.0, -6.0, 0.0], [-5.0, -3.0, 4.0]]], [[[-7.0, -7.0, -2.0], [0.0, -4.0, 3.0]], [[7.0, 4.0, -6.0], [3.0, 2.0, 7.0]], [[6.0, -3.0, -7.0], [-4.0, -1.0, -6.0]], [[-1.0, 3.0, -4.0], [-7.0, 2.0, -5.0]], [[-1.0, 1.0, -7.0], [-6.0, 7.0, -5.0]]], [[[6.0, 5.0, 0.0], [-3.0, 7.0, 3.0]], [[-1.0, 7.0, 0.0], [-2.0, 2.0, -4.0]], [[-2.0, 7.0, 7.0], [0.0, 3.0, -2.0]], [[5.0, -1.0, 2.0], [6.0, 6.0, -2.0]], [[4.0, -3.0, 1.0], [-4.0, -6.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 0.0, -5.0], [3.0, -7.0, -5.0]], [[7.0, 2.0, 0.0], [-1.0, 5.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, 4.0, -5.0], [1.0, -1.0, -6.0]], [[0.0, 0.0, 1.0], [3.0, -5.0, -2.0]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 17.0], [-26.0, 59.0], [-78.0, 18.0], [-114.0, 7.0], [-6.0, 1.0]], [[6.0, -32.0], [-52.0, 15.0], [60.0, -39.0], [58.0, 10.0], [7.0, -72.0]], [[82.0, 18.0], [-55.0, -25.0], [42.0, -43.0], [-10.0, 50.0], [20.0, -69.0]], [[-55.0, -54.0], [21.0, 47.0], [77.0, -70.0], [38.0, -28.0], [51.0, 52.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -1.0], [47.0, -1.0], [-23.0, 38.0], [23.0, -9.0], [-37.0, 42.0]], [[-29.0, 41.0], [-81.0, -10.0], [29.0, -12.0], [31.0, -24.0], [-57.0, -8.0]], [[17.0, 12.0], [-44.0, -21.0], [14.0, -2.0], [60.0, -25.0], [63.0, -50.0]], [[-50.0, -50.0], [55.0, -8.0], [16.0, -4.0], [-37.0, -6.0], [-31.0, 23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_tensor_transposed_mult_expandedData_rank4_expandedData_rank4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 4.0, 5.0], [6.0, 3.0, -1.0]], [[0.0, 7.0, -3.0], [0.0, 5.0, -2.0]], [[-2.0, 4.0, -5.0], [4.0, 2.0, 4.0]], [[3.0, 5.0, 0.0], [-2.0, -7.0, 3.0]], [[1.0, 6.0, -6.0], [-3.0, 7.0, -5.0]]], [[[-5.0, 4.0, 5.0], [6.0, -6.0, 0.0]], [[-7.0, -4.0, -7.0], [7.0, -6.0, -2.0]], [[-5.0, -6.0, 0.0], [-6.0, -1.0, 0.0]], [[-5.0, 1.0, 0.0], [0.0, -5.0, -2.0]], [[4.0, 1.0, -1.0], [0.0, -7.0, 4.0]]], [[[4.0, -4.0, -5.0], [-6.0, 5.0, 1.0]], [[1.0, -1.0, -6.0], [-3.0, 0.0, 6.0]], [[-1.0, 3.0, 4.0], [2.0, 1.0, 2.0]], [[-5.0, 1.0, 7.0], [-5.0, -5.0, -5.0]], [[-2.0, 6.0, 1.0], [-5.0, -4.0, -5.0]]], [[[0.0, -3.0, 0.0], [-3.0, 5.0, 3.0]], [[-6.0, 6.0, -1.0], [-4.0, -7.0, -3.0]], [[4.0, -3.0, -3.0], [4.0, 3.0, -4.0]], [[4.0, -1.0, -4.0], [6.0, 5.0, -3.0]], [[2.0, 0.0, 7.0], [-5.0, -7.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 5.0, 4.0], [3.0, -5.0, 7.0]], [[3.0, -1.0, 0.0], [6.0, 3.0, -6.0]], [[3.0, -5.0, 7.0], [-4.0, 3.0, 0.0]], [[0.0, 0.0, -7.0], [-6.0, 2.0, 4.0]], [[-7.0, -2.0, 7.0], [-2.0, 3.0, 5.0]]], [[[0.0, -3.0, -4.0], [2.0, 7.0, -3.0]], [[1.0, 1.0, 0.0], [0.0, 7.0, 5.0]], [[-3.0, 3.0, 0.0], [6.0, -6.0, 7.0]], [[1.0, 2.0, -5.0], [-6.0, 2.0, -6.0]], [[0.0, -1.0, 2.0], [7.0, -3.0, 1.0]]], [[[-5.0, -1.0, 7.0], [5.0, -1.0, -5.0]], [[3.0, 1.0, 3.0], [-3.0, 6.0, -4.0]], [[0.0, 0.0, -3.0], [3.0, -7.0, 5.0]], [[-5.0, 5.0, 7.0], [0.0, 3.0, -4.0]], [[-1.0, -7.0, -2.0], [-2.0, 7.0, 4.0]]], [[[0.0, 2.0, 0.0], [2.0, -1.0, -7.0]], [[1.0, 2.0, 0.0], [-6.0, -2.0, -3.0]], [[0.0, -4.0, 1.0], [7.0, -6.0, 2.0]], [[6.0, -1.0, -4.0], [7.0, -2.0, 1.0]], [[-1.0, -7.0, -4.0], [-1.0, 5.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, -4.0, -1.0], [7.0, -3.0, -1.0]], [[-4.0, -4.0, -5.0], [0.0, -2.0, -4.0]], [[-3.0, 6.0, -4.0], [-6.0, 1.0, 0.0]], [[-2.0, 1.0, 1.0], [-6.0, -6.0, 3.0]], [[2.0, -6.0, 6.0], [2.0, -6.0, 5.0]]], [[[-3.0, -3.0, -6.0], [0.0, -3.0, -3.0]], [[0.0, -6.0, 7.0], [-1.0, -5.0, -7.0]], [[1.0, -2.0, 0.0], [-4.0, -1.0, 3.0]], [[4.0, -1.0, 4.0], [3.0, 4.0, -6.0]], [[1.0, 1.0, 7.0], [0.0, -6.0, -2.0]]], [[[-4.0, 7.0, -3.0], [-1.0, 0.0, 7.0]], [[-7.0, 4.0, -3.0], [-7.0, 0.0, 2.0]], [[0.0, 7.0, -1.0], [-3.0, -7.0, -1.0]], [[1.0, 4.0, 5.0], [0.0, 3.0, 0.0]], [[7.0, -6.0, 6.0], [7.0, -3.0, 5.0]]], [[[-6.0, -3.0, -7.0], [1.0, 7.0, -4.0]], [[2.0, 3.0, 5.0], [0.0, -3.0, 5.0]], [[0.0, 7.0, -7.0], [-6.0, 6.0, 2.0]], [[3.0, -7.0, 3.0], [6.0, 5.0, -3.0]], [[5.0, -4.0, -6.0], [3.0, 2.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, 7.0, 0.0], [-5.0, 0.0, -3.0]], [[-5.0, -7.0, -2.0], [6.0, -1.0, -7.0]], [[-6.0, 1.0, 0.0], [5.0, -7.0, -2.0]], [[-1.0, -3.0, -6.0], [0.0, -4.0, 7.0]], [[4.0, -6.0, -1.0], [-2.0, -5.0, 7.0]]], [[[3.0, -4.0, 5.0], [-7.0, 6.0, -3.0]], [[0.0, 1.0, -6.0], [-5.0, 6.0, -2.0]], [[2.0, 0.0, -5.0], [-1.0, -2.0, 5.0]], [[-1.0, 6.0, 6.0], [2.0, 7.0, 1.0]], [[2.0, 4.0, 5.0], [0.0, 7.0, -1.0]]], [[[-6.0, -3.0, -7.0], [2.0, 1.0, -5.0]], [[-6.0, -5.0, 0.0], [6.0, -6.0, 6.0]], [[-1.0, 5.0, -4.0], [-6.0, 7.0, 6.0]], [[3.0, -7.0, 6.0], [-4.0, 1.0, 5.0]], [[-1.0, -1.0, -3.0], [-3.0, 3.0, 5.0]]], [[[0.0, -4.0, -3.0], [-4.0, 1.0, -7.0]], [[-1.0, 5.0, -4.0], [-4.0, -4.0, 1.0]], [[0.0, -1.0, 3.0], [4.0, 2.0, 3.0]], [[-7.0, 0.0, 3.0], [4.0, -1.0, -1.0]], [[-7.0, 0.0, 0.0], [3.0, 3.0, 3.0]]]])
      res=tensor_transposed_mult(arg0,arg1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-22.0, -63.0, -44.0, -58.0, 5.0], [-63.0, -3.0, -33.0, 72.0, 28.0], [-20.0, -78.0, -15.0, 55.0, 69.0], [-46.0, 33.0, -27.0, 56.0, 3.0]], [[-38.0, -15.0, 59.0, -32.0, -100.0], [-12.0, -74.0, -25.0, 13.0, -40.0], [44.0, 33.0, 19.0, 28.0, -85.0], [43.0, -19.0, 96.0, -27.0, 0.0]], [[21.0, -3.0, 28.0, -21.0, -42.0], [6.0, -101.0, -16.0, -36.0, -53.0], [75.0, 25.0, 3.0, -5.0, -26.0], [37.0, -3.0, 59.0, -27.0, 20.0]], [[-37.0, -30.0, 26.0, 62.0, 29.0], [-12.0, -14.0, 17.0, -45.0, 44.0], [46.0, 19.0, 87.0, 2.0, 13.0], [-96.0, 57.0, 11.0, -82.0, -25.0]], [[-62.0, 8.0, 82.0, -41.0, -143.0], [9.0, -75.0, -21.0, 23.0, -67.0], [24.0, 46.0, 13.0, 16.0, -132.0], [84.0, -56.0, 134.0, -25.0, 22.0]]], [[[74.0, -9.0, -23.0, 19.0, 44.0], [-9.0, 35.0, -31.0, -10.0, 70.0], [27.0, -6.0, 47.0, 18.0, 31.0], [-53.0, 45.0, -79.0, -22.0, -65.0]], [[141.0, 99.0, -23.0, -9.0, 8.0], [99.0, 12.0, -27.0, -43.0, -20.0], [0.0, 1.0, 2.0, -76.0, -10.0], [76.0, -53.0, -61.0, 4.0, 32.0]], [[20.0, 46.0, 14.0, 46.0, 20.0], [36.0, 47.0, 32.0, -36.0, -5.0], [-16.0, 53.0, -17.0, -32.0, -38.0], [35.0, -25.0, -12.0, -14.0, -21.0]], [[48.0, 34.0, 16.0, 35.0, 4.0], [33.0, 33.0, -8.0, -29.0, 30.0], [13.0, 35.0, 44.0, -16.0, -36.0], [0.0, -2.0, -27.0, -41.0, -39.0]], [[-14.0, -17.0, -9.0, 46.0, 58.0], [0.0, -6.0, 21.0, -41.0, 32.0], [22.0, -13.0, 53.0, -18.0, 57.0], [-85.0, 47.0, -20.0, -45.0, 8.0]]], [[[-65.0, 11.0, 25.0, -8.0, -35.0], [12.0, -37.0, 34.0, -4.0, -67.0], [-16.0, 15.0, -41.0, -22.0, -30.0], [48.0, -39.0, 75.0, 11.0, 58.0]], [[-24.0, 6.0, 33.0, 27.0, -4.0], [18.0, -75.0, 33.0, -64.0, -54.0], [52.0, 40.0, 2.0, -33.0, -14.0], [12.0, -1.0, 65.0, -44.0, 36.0]], [[0.0, -38.0, -6.0, -3.0, 12.0], [-39.0, -11.0, -10.0, 7.0, 20.0], [25.0, -3.0, 2.0, 34.0, 20.0], [-30.0, 34.0, -9.0, -1.0, -33.0]], [[9.0, 11.0, 18.0, 63.0, 21.0], [0.0, 108.0, 3.0, 2.0, 85.0], [-24.0, 43.0, 55.0, 19.0, -44.0], [-42.0, 18.0, -52.0, -41.0, -96.0]], [[-29.0, 7.0, 64.0, 50.0, -45.0], [9.0, 31.0, -5.0, -11.0, 45.0], [17.0, 60.0, 89.0, 15.0, -92.0], [-26.0, 6.0, 31.0, -80.0, -63.0]]], [[[-27.0, -10.0, 5.0, -6.0, -3.0], [-15.0, -25.0, 22.0, -4.0, -39.0], [3.0, 15.0, -50.0, 3.0, -3.0], [29.0, -9.0, 33.0, 19.0, 13.0]], [[15.0, 31.0, 75.0, 74.0, -35.0], [36.0, 17.0, -4.0, -56.0, 41.0], [52.0, 91.0, 107.0, -8.0, -106.0], [-16.0, 7.0, 25.0, -113.0, -74.0]], [[10.0, 21.0, -39.0, -68.0, -22.0], [18.0, 6.0, -21.0, 55.0, -30.0], [-60.0, -67.0, -47.0, -14.0, 27.0], [47.0, -45.0, -14.0, 75.0, 68.0]], [[10.0, 10.0, -33.0, -88.0, -43.0], [9.0, -32.0, -32.0, 57.0, -49.0], [-38.0, -68.0, -53.0, -5.0, 22.0], [60.0, -45.0, 9.0, 77.0, 76.0]], [[-34.0, -25.0, -11.0, 72.0, 73.0], [-24.0, 96.0, 26.0, -1.0, 95.0], [-31.0, -2.0, 58.0, 16.0, 37.0], [-111.0, 55.0, -63.0, -35.0, -61.0]]]])+(1.-msk_ref)*numpy.array([[[[-1.0, -69.0, 41.0, 30.0, 34.0], [-72.0, -78.0, 22.0, 32.0, -2.0], [-77.0, 65.0, -2.0, 7.0, -6.0], [-98.0, 24.0, 30.0, 22.0, 15.0]], [[-37.0, 67.0, 2.0, -54.0, -51.0], [7.0, -1.0, -36.0, 18.0, 29.0], [30.0, -31.0, -59.0, -35.0, -41.0], [25.0, -50.0, 13.0, 6.0, -12.0]], [[-33.0, -21.0, -64.0, -42.0, 28.0], [110.0, -9.0, -31.0, 22.0, 42.0], [-57.0, -35.0, -11.0, 105.0, 2.0], [18.0, -52.0, 16.0, -19.0, -24.0]], [[18.0, -52.0, -52.0, 62.0, 37.0], [7.0, 76.0, 57.0, -36.0, -25.0], [19.0, -24.0, 102.0, 4.0, 65.0], [19.0, 48.0, -29.0, -51.0, 0.0]], [[23.0, -15.0, -1.0, -6.0, 1.0], [39.0, -26.0, -28.0, 59.0, 29.0], [-27.0, 52.0, 32.0, 71.0, 28.0], [-37.0, -30.0, 36.0, 54.0, 67.0]]], [[[-22.0, 55.0, -36.0, -16.0, -38.0], [29.0, 59.0, -11.0, 8.0, 20.0], [63.0, -33.0, 20.0, -19.0, 15.0], [44.0, -38.0, 4.0, -8.0, 18.0]], [[-14.0, -54.0, -64.0, 3.0, -2.0], [26.0, 33.0, 13.0, 59.0, 50.0], [-27.0, -23.0, 83.0, 28.0, 44.0], [-32.0, -19.0, 28.0, -19.0, 29.0]], [[-12.0, -13.0, 79.0, 67.0, 37.0], [-120.0, -77.0, 35.0, -2.0, -43.0], [-20.0, 117.0, -18.0, -25.0, -1.0], [-91.0, 25.0, 30.0, 44.0, 42.0]], [[56.0, -5.0, -36.0, -27.0, -43.0], [42.0, 86.0, -1.0, -23.0, 5.0], [43.0, -100.0, 43.0, -45.0, 6.0], [75.0, 39.0, -55.0, -42.0, -37.0]], [[-45.0, 41.0, 53.0, 10.0, 12.0], [-56.0, -68.0, -6.0, 0.0, -16.0], [-5.0, 71.0, -70.0, -7.0, -30.0], [-40.0, -28.0, 32.0, 36.0, 15.0]]], [[[13.0, 84.0, 71.0, -65.0, -61.0], [-2.0, -64.0, -73.0, 39.0, 19.0], [18.0, 41.0, -95.0, -12.0, -58.0], [-3.0, -49.0, 25.0, 82.0, 32.0]], [[16.0, -24.0, -66.0, -76.0, -49.0], [89.0, 42.0, -38.0, 53.0, 71.0], [-22.0, -101.0, 26.0, 18.0, -6.0], [33.0, -26.0, -4.0, -26.0, -24.0]], [[-30.0, -4.0, 54.0, 81.0, 67.0], [-93.0, -49.0, 51.0, -56.0, -69.0], [-5.0, 90.0, -25.0, -12.0, 4.0], [-45.0, 33.0, 4.0, 5.0, 3.0]], [[77.0, 1.0, 22.0, -92.0, -100.0], [30.0, -11.0, -71.0, 94.0, 70.0], [-11.0, -37.0, -1.0, -25.0, -32.0], [-10.0, -14.0, 10.0, 57.0, 32.0]], [[-45.0, 11.0, -68.0, 34.0, 37.0], [59.0, 49.0, 16.0, -4.0, 5.0], [24.0, 11.0, 59.0, 69.0, 61.0], [21.0, -42.0, 19.0, -18.0, 34.0]]], [[[25.0, 48.0, 33.0, -51.0, -60.0], [-7.0, 0.0, -35.0, 2.0, 8.0], [32.0, -34.0, -51.0, -58.0, -46.0], [32.0, -1.0, -17.0, 16.0, -18.0]], [[47.0, -32.0, -14.0, -20.0, -7.0], [34.0, 26.0, -3.0, -18.0, -1.0], [-11.0, -58.0, 13.0, -4.0, -6.0], [35.0, 38.0, -39.0, -26.0, -40.0]], [[-69.0, 60.0, 69.0, 44.0, 53.0], [-70.0, -85.0, 10.0, -44.0, -55.0], [3.0, 110.0, -96.0, 10.0, -28.0], [-35.0, -26.0, 29.0, 35.0, 9.0]], [[-81.0, 22.0, 10.0, 36.0, 37.0], [-62.0, -26.0, 34.0, -35.0, -27.0], [2.0, 29.0, -45.0, -24.0, -15.0], [-21.0, -14.0, 16.0, -25.0, -24.0]], [[-35.0, 58.0, -39.0, 19.0, 12.0], [45.0, 54.0, 4.0, -33.0, -14.0], [63.0, -1.0, 17.0, 26.0, 33.0], [56.0, -35.0, -2.0, -13.0, 16.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_float_rank0_offset0(self):
      arg0=-5.0
      arg1=7.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=-35.0
      self.assertTrue(isinstance(res,float),"wrong type of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_array_rank0_offset0(self):
      arg0=0.0
      arg1=numpy.array(-7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-0.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_array_rank1_offset0(self):
      arg0=4.0
      arg1=numpy.array([-2.0, 3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-8.0, 12.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_array_rank2_offset0(self):
      arg0=0.0
      arg1=numpy.array([[1.0, 0.0, 7.0, 4.0, 5.0], [6.0, -3.0, 5.0, 0.0, 2.0], [2.0, 6.0, -6.0, -1.0, 7.0], [0.0, -3.0, 6.0, 1.0, 4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_array_rank3_offset0(self):
      arg0=-6.0
      arg1=numpy.array([[[3.0, 6.0], [1.0, -6.0]], [[-3.0, -3.0], [-4.0, -2.0]], [[4.0, -4.0], [4.0, -2.0]], [[5.0, 1.0], [6.0, 4.0]], [[0.0, 0.0], [2.0, 7.0]], [[-3.0, -1.0], [6.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-18.0, -36.0], [-6.0, 36.0]], [[18.0, 18.0], [24.0, 12.0]], [[-24.0, 24.0], [-24.0, 12.0]], [[-30.0, -6.0], [-36.0, -24.0]], [[-0.0, -0.0], [-12.0, -42.0]], [[18.0, 6.0], [-36.0, -0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_array_rank4_offset0(self):
      arg0=-4.0
      arg1=numpy.array([[[[-1.0, -5.0, -2.0, 2.0], [1.0, 3.0, -3.0, 4.0], [-6.0, 2.0, 4.0, 7.0]], [[4.0, 7.0, -3.0, 0.0], [3.0, -7.0, 1.0, -5.0], [0.0, 2.0, -4.0, 2.0]]], [[[0.0, 0.0, -3.0, 3.0], [0.0, 4.0, 7.0, -4.0], [6.0, 7.0, 0.0, -6.0]], [[1.0, -6.0, -4.0, 4.0], [4.0, 7.0, -5.0, 6.0], [1.0, 3.0, 0.0, -5.0]]], [[[0.0, 7.0, -3.0, 0.0], [1.0, 0.0, 1.0, -4.0], [7.0, 1.0, -4.0, 4.0]], [[-2.0, 5.0, -4.0, -5.0], [-4.0, 2.0, -3.0, 7.0], [-2.0, 1.0, 6.0, -3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[4.0, 20.0, 8.0, -8.0], [-4.0, -12.0, 12.0, -16.0], [24.0, -8.0, -16.0, -28.0]], [[-16.0, -28.0, 12.0, -0.0], [-12.0, 28.0, -4.0, 20.0], [-0.0, -8.0, 16.0, -8.0]]], [[[-0.0, -0.0, 12.0, -12.0], [-0.0, -16.0, -28.0, 16.0], [-24.0, -28.0, -0.0, 24.0]], [[-4.0, 24.0, 16.0, -16.0], [-16.0, -28.0, 20.0, -24.0], [-4.0, -12.0, -0.0, 20.0]]], [[[-0.0, -28.0, 12.0, -0.0], [-4.0, -0.0, -4.0, 16.0], [-28.0, -4.0, 16.0, -16.0]], [[8.0, -20.0, 16.0, 20.0], [16.0, -8.0, 12.0, -28.0], [8.0, -4.0, -24.0, 12.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_Symbol_rank0_offset0(self):
      arg0=-5.0
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(5.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-25.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_Symbol_rank1_offset0(self):
      arg0=-6.0
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([0.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-0.0, -24.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_Symbol_rank2_offset0(self):
      arg0=4.0
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[5.0, 0.0, -5.0, 0.0, -6.0], [0.0, -3.0, -6.0, -2.0, 6.0], [-5.0, -5.0, 1.0, 3.0, 0.0], [0.0, -1.0, -4.0, 7.0, -7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[20.0, 0.0, -20.0, 0.0, -24.0], [0.0, -12.0, -24.0, -8.0, 24.0], [-20.0, -20.0, 4.0, 12.0, 0.0], [0.0, -4.0, -16.0, 28.0, -28.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_Symbol_rank3_offset0(self):
      arg0=6.0
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-3.0, -5.0], [2.0, -3.0]], [[-5.0, 0.0], [-6.0, 5.0]], [[6.0, 0.0], [-7.0, 0.0]], [[-6.0, 3.0], [-3.0, -6.0]], [[-1.0, 0.0], [-2.0, -3.0]], [[0.0, 4.0], [-4.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-18.0, -30.0], [12.0, -18.0]], [[-30.0, 0.0], [-36.0, 30.0]], [[36.0, 0.0], [-42.0, 0.0]], [[-36.0, 18.0], [-18.0, -36.0]], [[-6.0, 0.0], [-12.0, -18.0]], [[0.0, 24.0], [-24.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_Symbol_rank4_offset0(self):
      arg0=-6.0
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-7.0, -1.0, 2.0, 4.0], [-2.0, 6.0, -1.0, 1.0], [6.0, -1.0, 7.0, 6.0]], [[-1.0, -5.0, -1.0, 5.0], [2.0, 2.0, 2.0, 0.0], [1.0, -5.0, 5.0, -6.0]]], [[[-1.0, 7.0, 1.0, 1.0], [-1.0, -6.0, -4.0, 6.0], [-2.0, 2.0, 5.0, 0.0]], [[6.0, -2.0, 6.0, -7.0], [5.0, -4.0, 3.0, -4.0], [-6.0, -6.0, 4.0, -6.0]]], [[[-1.0, 1.0, 2.0, 0.0], [4.0, 1.0, 4.0, -5.0], [0.0, 3.0, -3.0, 6.0]], [[3.0, 3.0, 0.0, -2.0], [-1.0, -4.0, -3.0, 1.0], [0.0, -5.0, -3.0, 1.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[42.0, 6.0, -12.0, -24.0], [12.0, -36.0, 6.0, -6.0], [-36.0, 6.0, -42.0, -36.0]], [[6.0, 30.0, 6.0, -30.0], [-12.0, -12.0, -12.0, -0.0], [-6.0, 30.0, -30.0, 36.0]]], [[[6.0, -42.0, -6.0, -6.0], [6.0, 36.0, 24.0, -36.0], [12.0, -12.0, -30.0, -0.0]], [[-36.0, 12.0, -36.0, 42.0], [-30.0, 24.0, -18.0, 24.0], [36.0, 36.0, -24.0, 36.0]]], [[[6.0, -6.0, -12.0, -0.0], [-24.0, -6.0, -24.0, 30.0], [-0.0, -18.0, 18.0, -36.0]], [[-18.0, -18.0, -0.0, 12.0], [6.0, 24.0, 18.0, -6.0], [-0.0, 30.0, 18.0, -6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_constData_rank0_offset0(self):
      arg0=6.0
      arg1=Data(-1.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(-6.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_constData_rank1_offset0(self):
      arg0=0.0
      arg1=Data(numpy.array([-4.0, 6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-0.0, 0.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_constData_rank2_offset0(self):
      arg0=0.0
      arg1=Data(numpy.array([[-1.0, 0.0, 2.0, 1.0, 3.0], [-3.0, 3.0, 1.0, -4.0, 0.0], [-5.0, 7.0, -6.0, -5.0, 7.0], [6.0, 6.0, -2.0, 7.0, 5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-0.0, 0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, -0.0, -0.0, 0.0], [0.0, 0.0, -0.0, 0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_constData_rank3_offset0(self):
      arg0=2.0
      arg1=Data(numpy.array([[[1.0, -6.0], [0.0, 0.0]], [[1.0, 0.0], [-5.0, 2.0]], [[4.0, 2.0], [-5.0, 7.0]], [[3.0, 0.0], [-6.0, 0.0]], [[2.0, 2.0], [-2.0, -2.0]], [[4.0, -6.0], [0.0, -1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[2.0, -12.0], [0.0, 0.0]], [[2.0, 0.0], [-10.0, 4.0]], [[8.0, 4.0], [-10.0, 14.0]], [[6.0, 0.0], [-12.0, 0.0]], [[4.0, 4.0], [-4.0, -4.0]], [[8.0, -12.0], [0.0, -2.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_constData_rank4_offset0(self):
      arg0=6.0
      arg1=Data(numpy.array([[[[7.0, 4.0, -1.0, -6.0], [1.0, -4.0, 3.0, -6.0], [6.0, -7.0, -6.0, 3.0]], [[-1.0, -2.0, -1.0, -1.0], [4.0, 5.0, 4.0, -4.0], [-2.0, -6.0, 2.0, 5.0]]], [[[3.0, -4.0, 7.0, -4.0], [0.0, -6.0, 1.0, 6.0], [5.0, 4.0, -4.0, -1.0]], [[0.0, 7.0, 0.0, 4.0], [5.0, 5.0, -3.0, 1.0], [4.0, 2.0, -7.0, -1.0]]], [[[-2.0, 1.0, 7.0, -2.0], [-7.0, 6.0, -4.0, 1.0], [-1.0, -6.0, -3.0, 0.0]], [[-7.0, 5.0, 4.0, 5.0], [7.0, -6.0, 7.0, 0.0], [0.0, 0.0, -3.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[42.0, 24.0, -6.0, -36.0], [6.0, -24.0, 18.0, -36.0], [36.0, -42.0, -36.0, 18.0]], [[-6.0, -12.0, -6.0, -6.0], [24.0, 30.0, 24.0, -24.0], [-12.0, -36.0, 12.0, 30.0]]], [[[18.0, -24.0, 42.0, -24.0], [0.0, -36.0, 6.0, 36.0], [30.0, 24.0, -24.0, -6.0]], [[0.0, 42.0, 0.0, 24.0], [30.0, 30.0, -18.0, 6.0], [24.0, 12.0, -42.0, -6.0]]], [[[-12.0, 6.0, 42.0, -12.0], [-42.0, 36.0, -24.0, 6.0], [-6.0, -36.0, -18.0, 0.0]], [[-42.0, 30.0, 24.0, 30.0], [42.0, -36.0, 42.0, 0.0], [0.0, 0.0, -18.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_expandedData_rank0_offset0(self):
      arg0=6.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-6.0)+(1-msk_arg1)*(2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-36.0)+(1.-msk_ref)*numpy.array(12.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_expandedData_rank1_offset0(self):
      arg0=1.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, 4.0])+(1.-msk_arg1)*numpy.array([3.0, 3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, 4.0])+(1.-msk_ref)*numpy.array([3.0, 3.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_expandedData_rank2_offset0(self):
      arg0=7.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 3.0, -7.0, -2.0, -7.0], [-7.0, 0.0, 3.0, -7.0, 3.0], [7.0, 3.0, -6.0, -6.0, 7.0], [0.0, 0.0, 2.0, -2.0, -1.0]])+(1.-msk_arg1)*numpy.array([[0.0, -7.0, 3.0, 1.0, -2.0], [6.0, 4.0, 0.0, -2.0, 7.0], [0.0, 1.0, -7.0, 5.0, 4.0], [2.0, 2.0, -4.0, 6.0, -1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-42.0, 21.0, -49.0, -14.0, -49.0], [-49.0, 0.0, 21.0, -49.0, 21.0], [49.0, 21.0, -42.0, -42.0, 49.0], [0.0, 0.0, 14.0, -14.0, -7.0]])+(1.-msk_ref)*numpy.array([[0.0, -49.0, 21.0, 7.0, -14.0], [42.0, 28.0, 0.0, -14.0, 49.0], [0.0, 7.0, -49.0, 35.0, 28.0], [14.0, 14.0, -28.0, 42.0, -7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_expandedData_rank3_offset0(self):
      arg0=0.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, -4.0], [2.0, -2.0]], [[0.0, 0.0], [6.0, 5.0]], [[-6.0, 4.0], [-3.0, 0.0]], [[2.0, -3.0], [0.0, -5.0]], [[4.0, 3.0], [0.0, 3.0]], [[7.0, 6.0], [2.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, -2.0], [0.0, 3.0]], [[-6.0, -4.0], [-5.0, 5.0]], [[-4.0, 7.0], [1.0, -7.0]], [[4.0, 5.0], [-3.0, 7.0]], [[-4.0, 4.0], [-1.0, -2.0]], [[5.0, -2.0], [-6.0, 5.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, 0.0]], [[-0.0, 0.0], [-0.0, 0.0]], [[0.0, -0.0], [0.0, -0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -0.0], [0.0, 0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[0.0, -0.0], [-0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_float_rank0_expandedData_rank4_offset0(self):
      arg0=3.0
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 0.0, -2.0, 6.0], [-5.0, 6.0, 0.0, -3.0], [-6.0, -7.0, -4.0, -6.0]], [[-3.0, -2.0, 1.0, 5.0], [5.0, -2.0, 6.0, -3.0], [0.0, -5.0, 1.0, -2.0]]], [[[-4.0, -6.0, 4.0, 0.0], [2.0, 2.0, -5.0, -6.0], [7.0, 1.0, 5.0, 3.0]], [[-6.0, 1.0, 0.0, -4.0], [-3.0, 3.0, 4.0, -1.0], [7.0, 6.0, 0.0, 6.0]]], [[[4.0, -6.0, 1.0, -5.0], [0.0, -1.0, 4.0, -4.0], [1.0, -4.0, 0.0, 0.0]], [[2.0, -5.0, 6.0, 4.0], [6.0, 5.0, -1.0, 0.0], [7.0, 1.0, 4.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, -4.0, 7.0, -4.0], [3.0, -4.0, 1.0, 0.0], [-2.0, 3.0, 7.0, 6.0]], [[-5.0, -3.0, 4.0, -7.0], [0.0, 0.0, 4.0, 0.0], [-6.0, -6.0, 0.0, -4.0]]], [[[-1.0, -6.0, -6.0, 0.0], [0.0, 2.0, 7.0, -7.0], [3.0, 3.0, 2.0, -5.0]], [[7.0, 5.0, 7.0, 5.0], [4.0, 4.0, -4.0, 6.0], [0.0, -5.0, 6.0, -7.0]]], [[[0.0, 7.0, -3.0, 2.0], [6.0, -6.0, 7.0, 4.0], [-6.0, -6.0, 6.0, -6.0]], [[0.0, 4.0, 3.0, 4.0], [-7.0, -2.0, 7.0, -3.0], [-5.0, 5.0, -4.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0, -6.0, 18.0], [-15.0, 18.0, 0.0, -9.0], [-18.0, -21.0, -12.0, -18.0]], [[-9.0, -6.0, 3.0, 15.0], [15.0, -6.0, 18.0, -9.0], [0.0, -15.0, 3.0, -6.0]]], [[[-12.0, -18.0, 12.0, 0.0], [6.0, 6.0, -15.0, -18.0], [21.0, 3.0, 15.0, 9.0]], [[-18.0, 3.0, 0.0, -12.0], [-9.0, 9.0, 12.0, -3.0], [21.0, 18.0, 0.0, 18.0]]], [[[12.0, -18.0, 3.0, -15.0], [0.0, -3.0, 12.0, -12.0], [3.0, -12.0, 0.0, 0.0]], [[6.0, -15.0, 18.0, 12.0], [18.0, 15.0, -3.0, 0.0], [21.0, 3.0, 12.0, 9.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -12.0, 21.0, -12.0], [9.0, -12.0, 3.0, 0.0], [-6.0, 9.0, 21.0, 18.0]], [[-15.0, -9.0, 12.0, -21.0], [0.0, 0.0, 12.0, 0.0], [-18.0, -18.0, 0.0, -12.0]]], [[[-3.0, -18.0, -18.0, 0.0], [0.0, 6.0, 21.0, -21.0], [9.0, 9.0, 6.0, -15.0]], [[21.0, 15.0, 21.0, 15.0], [12.0, 12.0, -12.0, 18.0], [0.0, -15.0, 18.0, -21.0]]], [[[0.0, 21.0, -9.0, 6.0], [18.0, -18.0, 21.0, 12.0], [-18.0, -18.0, 18.0, -18.0]], [[0.0, 12.0, 9.0, 12.0], [-21.0, -6.0, 21.0, -9.0], [-15.0, 15.0, -12.0, -12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_float_rank0_offset0(self):
      arg0=numpy.array(4.0)
      arg1=-4.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(-16.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_array_rank0_offset0(self):
      arg0=numpy.array(0.0)
      arg1=numpy.array(7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array(0.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank1_offset1(self):
      arg0=numpy.array([6.0, -2.0, 3.0])
      arg1=numpy.array([3.0, -1.0, 3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array(29.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank2_offset2(self):
      arg0=numpy.array([[5.0, 1.0, 0.0], [-3.0, -7.0, 4.0]])
      arg1=numpy.array([[-5.0, 1.0, 3.0], [-6.0, -4.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array(22.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank3_offset3(self):
      arg0=numpy.array([[[-1.0, 3.0, -1.0], [0.0, 0.0, -6.0], [-5.0, 7.0, 7.0], [1.0, 7.0, -6.0]], [[5.0, -2.0, -3.0], [6.0, 5.0, -3.0], [-2.0, 4.0, 6.0], [-1.0, 2.0, -6.0]]])
      arg1=numpy.array([[[-5.0, -2.0, 6.0], [4.0, 1.0, 6.0], [-3.0, 1.0, 6.0], [-3.0, 4.0, 0.0]], [[0.0, -2.0, -1.0], [0.0, 1.0, 5.0], [-7.0, 0.0, 0.0], [7.0, -5.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array(16.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank4_offset4(self):
      arg0=numpy.array([[[[5.0, -7.0], [-1.0, 1.0], [-4.0, 4.0]], [[-4.0, -4.0], [-3.0, 0.0], [7.0, 3.0]]], [[[0.0, 0.0], [5.0, 7.0], [-5.0, -4.0]], [[1.0, 4.0], [3.0, -2.0], [0.0, -6.0]]], [[[2.0, -3.0], [-3.0, -3.0], [-1.0, 7.0]], [[7.0, -1.0], [-1.0, -7.0], [-7.0, -6.0]]], [[[1.0, -6.0], [0.0, 0.0], [-1.0, 2.0]], [[5.0, 4.0], [-4.0, 6.0], [-6.0, -4.0]]]])
      arg1=numpy.array([[[[-4.0, 7.0], [2.0, 4.0], [-1.0, -4.0]], [[2.0, 2.0], [-4.0, -7.0], [-2.0, 4.0]]], [[[0.0, 0.0], [-5.0, -6.0], [0.0, -1.0]], [[0.0, 3.0], [6.0, -1.0], [-3.0, 5.0]]], [[[5.0, 0.0], [-1.0, -4.0], [-1.0, 3.0]], [[0.0, 1.0], [-6.0, -6.0], [-6.0, -2.0]]], [[[-4.0, 6.0], [7.0, 4.0], [7.0, 7.0]], [[5.0, -6.0], [-6.0, 4.0], [5.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      ref=numpy.array(-8.0)
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_array_rank1_offset0(self):
      arg0=numpy.array(4.0)
      arg1=numpy.array([-3.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-12.0, 28.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank2_offset1(self):
      arg0=numpy.array([-3.0, -4.0, -2.0])
      arg1=numpy.array([[6.0, 4.0, 7.0], [-7.0, 7.0, -2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([-48.0, -3.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank3_offset2(self):
      arg0=numpy.array([[7.0, 1.0, 2.0], [3.0, 3.0, 1.0]])
      arg1=numpy.array([[[5.0, 2.0, -7.0], [-3.0, 5.0, 2.0]], [[-2.0, -3.0, 3.0], [3.0, 1.0, -3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([31.0, -2.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank4_offset3(self):
      arg0=numpy.array([[[-6.0, 1.0, -3.0], [3.0, 5.0, -6.0], [3.0, -3.0, -2.0], [1.0, 5.0, 2.0]], [[3.0, 0.0, 3.0], [7.0, -7.0, 7.0], [-6.0, -6.0, 0.0], [-1.0, 0.0, -3.0]]])
      arg1=numpy.array([[[[3.0, -6.0, 1.0], [-1.0, 0.0, 7.0], [3.0, -6.0, 1.0], [1.0, -2.0, 6.0]], [[-2.0, 5.0, -6.0], [3.0, -7.0, -4.0], [-7.0, 2.0, -4.0], [-7.0, 2.0, -6.0]]], [[[6.0, 4.0, 6.0], [-6.0, -3.0, -1.0], [-3.0, 0.0, -7.0], [6.0, 3.0, -1.0]], [[3.0, -5.0, -3.0], [7.0, 0.0, 1.0], [6.0, -1.0, 0.0], [-7.0, 5.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([29.0, -8.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_array_rank2_offset0(self):
      arg0=numpy.array(-2.0)
      arg1=numpy.array([[3.0, 4.0, -5.0, 3.0, 1.0], [3.0, -4.0, -4.0, 2.0, -2.0], [1.0, 5.0, 4.0, -3.0, 6.0], [-2.0, 0.0, -3.0, 2.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-6.0, -8.0, 10.0, -6.0, -2.0], [-6.0, 8.0, 8.0, -4.0, 4.0], [-2.0, -10.0, -8.0, 6.0, -12.0], [4.0, -0.0, 6.0, -4.0, -12.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank3_offset1(self):
      arg0=numpy.array([5.0, -6.0, 7.0])
      arg1=numpy.array([[[7.0, 3.0, -1.0], [6.0, -6.0, 6.0], [-7.0, -5.0, -1.0], [5.0, -5.0, -4.0], [4.0, -3.0, 1.0]], [[-3.0, 0.0, 5.0], [4.0, 4.0, -4.0], [2.0, 7.0, 7.0], [-1.0, -4.0, 1.0], [4.0, 4.0, 0.0]], [[2.0, -4.0, -1.0], [-2.0, 0.0, 2.0], [0.0, 0.0, 1.0], [7.0, 5.0, 0.0], [0.0, 2.0, -3.0]], [[0.0, -4.0, 2.0], [1.0, -5.0, -7.0], [-6.0, -1.0, -4.0], [-2.0, -7.0, -7.0], [-6.0, 5.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[10.0, 108.0, -12.0, 27.0, 45.0], [20.0, -32.0, 17.0, 26.0, -4.0], [27.0, 4.0, 7.0, 5.0, -33.0], [38.0, -14.0, -52.0, -17.0, -109.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank4_offset2(self):
      arg0=numpy.array([[6.0, -2.0, 1.0], [0.0, -2.0, -3.0]])
      arg1=numpy.array([[[[6.0, -7.0, -5.0], [0.0, 5.0, -6.0]], [[-5.0, -3.0, 0.0], [0.0, 2.0, 1.0]], [[3.0, -4.0, 7.0], [-6.0, 5.0, 5.0]], [[2.0, -7.0, -1.0], [-6.0, -4.0, -2.0]], [[-1.0, -4.0, 7.0], [6.0, 2.0, 0.0]]], [[[-5.0, -5.0, 7.0], [1.0, 7.0, 7.0]], [[1.0, 7.0, 2.0], [5.0, -4.0, -6.0]], [[0.0, -5.0, 2.0], [6.0, -3.0, 3.0]], [[2.0, 7.0, -3.0], [3.0, -2.0, -4.0]], [[-7.0, 7.0, -7.0], [3.0, -5.0, 7.0]]], [[[-2.0, 7.0, -5.0], [4.0, 5.0, 0.0]], [[4.0, 6.0, 0.0], [2.0, 1.0, 3.0]], [[-4.0, -4.0, 4.0], [-6.0, 0.0, -7.0]], [[0.0, -1.0, -2.0], [0.0, 6.0, 0.0]], [[-2.0, 2.0, -3.0], [2.0, -3.0, 3.0]]], [[[-3.0, -5.0, -1.0], [-2.0, 5.0, -2.0]], [[1.0, 1.0, 3.0], [-6.0, 0.0, 1.0]], [[6.0, 2.0, -1.0], [-4.0, 2.0, 3.0]], [[4.0, -6.0, -4.0], [2.0, 5.0, 0.0]], [[-6.0, -7.0, 5.0], [7.0, -5.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[53.0, -31.0, 8.0, 39.0, 5.0], [-48.0, 20.0, 9.0, 11.0, -74.0], [-41.0, 1.0, 9.0, -12.0, -22.0], [-13.0, 4.0, 18.0, 22.0, 14.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_array_rank3_offset0(self):
      arg0=numpy.array(-7.0)
      arg1=numpy.array([[[2.0, -3.0], [-2.0, 0.0]], [[-6.0, -4.0], [2.0, -6.0]], [[1.0, -4.0], [0.0, 1.0]], [[1.0, 5.0], [-7.0, -4.0]], [[-3.0, 3.0], [-4.0, 0.0]], [[5.0, -5.0], [0.0, 7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-14.0, 21.0], [14.0, -0.0]], [[42.0, 28.0], [-14.0, 42.0]], [[-7.0, 28.0], [-0.0, -7.0]], [[-7.0, -35.0], [49.0, 28.0]], [[21.0, -21.0], [28.0, -0.0]], [[-35.0, 35.0], [-0.0, -49.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank4_offset1(self):
      arg0=numpy.array([-2.0, -6.0, -2.0])
      arg1=numpy.array([[[[4.0, 5.0, 2.0], [1.0, 0.0, 2.0]], [[5.0, -1.0, 5.0], [-1.0, -1.0, 7.0]]], [[[4.0, 0.0, 0.0], [-4.0, -1.0, -5.0]], [[4.0, 0.0, -3.0], [-7.0, 2.0, 4.0]]], [[[4.0, 2.0, 0.0], [-1.0, 7.0, 0.0]], [[-5.0, -1.0, 6.0], [5.0, -2.0, 6.0]]], [[[4.0, 0.0, -7.0], [7.0, 2.0, -6.0]], [[4.0, -3.0, -7.0], [-6.0, 4.0, 5.0]]], [[[-3.0, 1.0, 7.0], [1.0, 2.0, 6.0]], [[0.0, -1.0, 6.0], [-5.0, 5.0, 3.0]]], [[[7.0, -3.0, -3.0], [2.0, 4.0, 0.0]], [[4.0, -6.0, -3.0], [-5.0, -5.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-42.0, -6.0], [-14.0, -6.0]], [[-8.0, 24.0], [-2.0, -6.0]], [[-20.0, -40.0], [4.0, -10.0]], [[6.0, -14.0], [24.0, -22.0]], [[-14.0, -26.0], [-6.0, -26.0]], [[10.0, -28.0], [34.0, 50.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_array_rank4_offset0(self):
      arg0=numpy.array(-2.0)
      arg1=numpy.array([[[[-2.0, 4.0, -2.0, -5.0], [3.0, -3.0, -4.0, 7.0], [6.0, 0.0, -7.0, -3.0]], [[1.0, 2.0, -4.0, 7.0], [4.0, 5.0, 3.0, 5.0], [3.0, 4.0, 7.0, 6.0]]], [[[-6.0, -2.0, -5.0, 1.0], [7.0, 4.0, -6.0, -7.0], [5.0, 0.0, 3.0, 4.0]], [[7.0, 0.0, -1.0, 2.0], [1.0, -6.0, 0.0, 0.0], [4.0, -1.0, -2.0, -6.0]]], [[[-3.0, 4.0, 6.0, 7.0], [2.0, -6.0, 3.0, -3.0], [-3.0, 0.0, -3.0, 4.0]], [[7.0, 0.0, -1.0, 0.0], [0.0, -1.0, 6.0, 1.0], [-5.0, 0.0, 5.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[4.0, -8.0, 4.0, 10.0], [-6.0, 6.0, 8.0, -14.0], [-12.0, -0.0, 14.0, 6.0]], [[-2.0, -4.0, 8.0, -14.0], [-8.0, -10.0, -6.0, -10.0], [-6.0, -8.0, -14.0, -12.0]]], [[[12.0, 4.0, 10.0, -2.0], [-14.0, -8.0, 12.0, 14.0], [-10.0, -0.0, -6.0, -8.0]], [[-14.0, -0.0, 2.0, -4.0], [-2.0, 12.0, -0.0, -0.0], [-8.0, 2.0, 4.0, 12.0]]], [[[6.0, -8.0, -12.0, -14.0], [-4.0, 12.0, -6.0, 6.0], [6.0, -0.0, 6.0, -8.0]], [[-14.0, -0.0, 2.0, -0.0], [-0.0, 2.0, -12.0, -2.0], [10.0, -0.0, -10.0, -0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_Symbol_rank0_offset0(self):
      arg0=numpy.array(7.0)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array(21.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank1_offset1(self):
      arg0=numpy.array([-4.0, 0.0, -1.0])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([1.0, -3.0, -2.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-2.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank2_offset2(self):
      arg0=numpy.array([[7.0, 0.0, 2.0], [-2.0, 5.0, -2.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-5.0, -1.0, 3.0], [-6.0, 5.0, 1.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(6.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[6.0, 7.0, -1.0], [4.0, 3.0, 6.0], [-4.0, 4.0, 3.0], [3.0, -5.0, 1.0]], [[-1.0, -2.0, 1.0], [4.0, -4.0, -6.0], [0.0, 0.0, -2.0], [3.0, -5.0, -7.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-2.0, -1.0, -2.0], [-6.0, -3.0, 5.0], [-7.0, -5.0, -3.0], [0.0, -6.0, -2.0]], [[-1.0, 7.0, -7.0], [0.0, -5.0, 6.0], [2.0, -2.0, 6.0], [-3.0, -5.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-25.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank4_offset4(self):
      arg0=numpy.array([[[[1.0, -6.0], [0.0, 6.0], [-1.0, -6.0]], [[-6.0, 3.0], [-3.0, 0.0], [-3.0, 2.0]]], [[[-5.0, 1.0], [2.0, -1.0], [-3.0, -7.0]], [[-2.0, 7.0], [4.0, 2.0], [7.0, 7.0]]], [[[-4.0, 6.0], [5.0, 7.0], [-5.0, 6.0]], [[5.0, -7.0], [1.0, 7.0], [6.0, -6.0]]], [[[0.0, 1.0], [-4.0, 2.0], [5.0, -4.0]], [[7.0, 7.0], [0.0, 2.0], [5.0, 7.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[6.0, 3.0], [-4.0, 6.0], [-3.0, 2.0]], [[-1.0, 6.0], [6.0, 4.0], [-4.0, -4.0]]], [[[-1.0, -5.0], [4.0, 2.0], [-7.0, 0.0]], [[3.0, 2.0], [1.0, -1.0], [7.0, -7.0]]], [[[6.0, -6.0], [3.0, -7.0], [-2.0, -7.0]], [[-2.0, -2.0], [5.0, 3.0], [-1.0, 0.0]]], [[[-2.0, 6.0], [5.0, -3.0], [-4.0, -3.0]], [[5.0, 1.0], [-5.0, -6.0], [-6.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array(-68.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_Symbol_rank1_offset0(self):
      arg0=numpy.array(0.0)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([6.0, -1.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([0.0, -0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank2_offset1(self):
      arg0=numpy.array([-3.0, -2.0, 0.0])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[4.0, 0.0, -4.0], [2.0, -2.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-12.0, -2.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank3_offset2(self):
      arg0=numpy.array([[-7.0, -6.0, -1.0], [-7.0, 5.0, -6.0]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[6.0, 6.0, 3.0], [7.0, -6.0, -4.0]], [[-2.0, 3.0, 7.0], [-6.0, -2.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-136.0, 21.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[0.0, -5.0, 2.0], [0.0, 0.0, 2.0], [-3.0, -5.0, 0.0], [-2.0, 6.0, 4.0]], [[-7.0, -5.0, 5.0], [0.0, 7.0, -3.0], [7.0, -6.0, -1.0], [4.0, 4.0, -5.0]]])
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[7.0, 1.0, -2.0], [-6.0, 0.0, -4.0], [-5.0, -1.0, 3.0], [5.0, -7.0, -2.0]], [[6.0, -5.0, -1.0], [-4.0, 1.0, 6.0], [-7.0, -6.0, 5.0], [0.0, 1.0, 5.0]]], [[[-1.0, 0.0, -1.0], [1.0, 1.0, 0.0], [6.0, 6.0, -5.0], [-1.0, 3.0, 1.0]], [[2.0, -2.0, 0.0], [5.0, 3.0, -6.0], [2.0, 5.0, -7.0], [-7.0, -7.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-129.0, -86.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_Symbol_rank2_offset0(self):
      arg0=numpy.array(6.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[7.0, -6.0, -7.0, -1.0, 1.0], [-3.0, 6.0, 6.0, -7.0, -1.0], [-6.0, -3.0, -5.0, 7.0, -1.0], [-2.0, 7.0, 6.0, -2.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[42.0, -36.0, -42.0, -6.0, 6.0], [-18.0, 36.0, 36.0, -42.0, -6.0], [-36.0, -18.0, -30.0, 42.0, -6.0], [-12.0, 42.0, 36.0, -12.0, -18.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank3_offset1(self):
      arg0=numpy.array([-1.0, 7.0, 0.0])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-5.0, -5.0, -6.0], [0.0, -2.0, 1.0], [-7.0, 0.0, 0.0], [-3.0, -4.0, 2.0], [6.0, -2.0, 0.0]], [[-4.0, -6.0, -7.0], [0.0, 2.0, -1.0], [-2.0, -1.0, 3.0], [-4.0, -3.0, 0.0], [-1.0, -6.0, 6.0]], [[4.0, -2.0, -7.0], [0.0, -3.0, 1.0], [3.0, 6.0, -5.0], [6.0, 0.0, 6.0], [4.0, 6.0, 5.0]], [[7.0, -2.0, -4.0], [0.0, 1.0, -4.0], [0.0, 5.0, -4.0], [3.0, 6.0, -6.0], [7.0, 4.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-30.0, -14.0, 7.0, -25.0, -20.0], [-38.0, 14.0, -5.0, -17.0, -41.0], [-18.0, -21.0, 39.0, -6.0, 38.0], [-21.0, 7.0, 35.0, 39.0, 21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank4_offset2(self):
      arg0=numpy.array([[5.0, 1.0, 5.0], [-2.0, 5.0, 1.0]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[6.0, 3.0, -3.0], [-2.0, -7.0, -6.0]], [[0.0, 3.0, -4.0], [-4.0, 7.0, -4.0]], [[-3.0, 2.0, -5.0], [5.0, -2.0, -3.0]], [[2.0, -6.0, 2.0], [0.0, -3.0, 7.0]], [[0.0, 0.0, 1.0], [-3.0, -5.0, 0.0]]], [[[-2.0, 3.0, 0.0], [7.0, -6.0, 0.0]], [[5.0, -4.0, 7.0], [0.0, -1.0, -4.0]], [[1.0, 1.0, 1.0], [3.0, 0.0, 5.0]], [[0.0, 1.0, -1.0], [0.0, -6.0, 6.0]], [[1.0, -2.0, -6.0], [-1.0, 4.0, -1.0]]], [[[-6.0, 3.0, 0.0], [-1.0, 0.0, 3.0]], [[0.0, -2.0, -3.0], [-7.0, -3.0, 0.0]], [[0.0, 7.0, 5.0], [-7.0, 5.0, 7.0]], [[-1.0, 0.0, -7.0], [-1.0, 6.0, 4.0]], [[3.0, 0.0, 0.0], [7.0, -1.0, 4.0]]], [[[4.0, 6.0, 2.0], [-5.0, 0.0, -1.0]], [[6.0, -3.0, 6.0], [1.0, -1.0, 4.0]], [[2.0, -4.0, 7.0], [5.0, 7.0, -3.0]], [[-7.0, 4.0, 5.0], [5.0, -7.0, -5.0]], [[-5.0, 1.0, -4.0], [-4.0, -4.0, 4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-19.0, 22.0, -61.0, 6.0, -14.0], [-51.0, 47.0, 10.0, -28.0, -6.0], [-22.0, -18.0, 78.0, -4.0, 0.0], [45.0, 54.0, 63.0, -56.0, -52.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_Symbol_rank3_offset0(self):
      arg0=numpy.array(7.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-3.0, -1.0], [-6.0, -2.0]], [[7.0, 1.0], [-6.0, -2.0]], [[-6.0, 2.0], [-3.0, 3.0]], [[7.0, 6.0], [0.0, 7.0]], [[3.0, 6.0], [-6.0, 2.0]], [[1.0, 2.0], [-1.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-21.0, -7.0], [-42.0, -14.0]], [[49.0, 7.0], [-42.0, -14.0]], [[-42.0, 14.0], [-21.0, 21.0]], [[49.0, 42.0], [0.0, 49.0]], [[21.0, 42.0], [-42.0, 14.0]], [[7.0, 14.0], [-7.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank4_offset1(self):
      arg0=numpy.array([-4.0, -4.0, 5.0])
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[1.0, -3.0, -5.0], [-1.0, -6.0, -3.0]], [[-2.0, -3.0, 6.0], [4.0, 7.0, 0.0]]], [[[0.0, -7.0, -2.0], [0.0, -7.0, 0.0]], [[-4.0, -3.0, -4.0], [3.0, 3.0, 7.0]]], [[[-2.0, -5.0, 0.0], [-6.0, 4.0, 3.0]], [[-1.0, -1.0, -4.0], [-7.0, -7.0, 1.0]]], [[[-2.0, -2.0, 6.0], [4.0, -2.0, -6.0]], [[2.0, 2.0, 0.0], [4.0, 3.0, -1.0]]], [[[-6.0, -3.0, -6.0], [-5.0, -5.0, 1.0]], [[-4.0, 5.0, -1.0], [6.0, 2.0, -4.0]]], [[[-6.0, 1.0, 6.0], [3.0, -3.0, -7.0]], [[7.0, 5.0, 0.0], [-3.0, -6.0, -4.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-17.0, 13.0], [50.0, -44.0]], [[18.0, 28.0], [8.0, 11.0]], [[28.0, 23.0], [-12.0, 61.0]], [[46.0, -38.0], [-16.0, -33.0]], [[6.0, 45.0], [-9.0, -52.0]], [[50.0, -35.0], [-48.0, 16.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_Symbol_rank4_offset0(self):
      arg0=numpy.array(-1.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[-1.0, -4.0, -3.0, -3.0], [0.0, 0.0, -2.0, 5.0], [-3.0, -3.0, 1.0, -7.0]], [[2.0, -2.0, 6.0, 0.0], [2.0, 4.0, 3.0, -6.0], [6.0, -6.0, 0.0, 6.0]]], [[[0.0, 5.0, -4.0, 1.0], [-4.0, -2.0, 0.0, 3.0], [0.0, -3.0, -7.0, 7.0]], [[3.0, -2.0, 7.0, -5.0], [7.0, 1.0, -1.0, 5.0], [7.0, -4.0, 1.0, 7.0]]], [[[3.0, 5.0, 0.0, 0.0], [5.0, -6.0, -7.0, -6.0], [-1.0, -2.0, -3.0, 4.0]], [[0.0, -6.0, -1.0, -6.0], [3.0, 4.0, 5.0, 2.0], [2.0, -3.0, 7.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[1.0, 4.0, 3.0, 3.0], [-0.0, -0.0, 2.0, -5.0], [3.0, 3.0, -1.0, 7.0]], [[-2.0, 2.0, -6.0, -0.0], [-2.0, -4.0, -3.0, 6.0], [-6.0, 6.0, -0.0, -6.0]]], [[[-0.0, -5.0, 4.0, -1.0], [4.0, 2.0, -0.0, -3.0], [-0.0, 3.0, 7.0, -7.0]], [[-3.0, 2.0, -7.0, 5.0], [-7.0, -1.0, 1.0, -5.0], [-7.0, 4.0, -1.0, -7.0]]], [[[-3.0, -5.0, -0.0, -0.0], [-5.0, 6.0, 7.0, 6.0], [1.0, 2.0, 3.0, -4.0]], [[-0.0, 6.0, 1.0, 6.0], [-3.0, -4.0, -5.0, -2.0], [-2.0, 3.0, -7.0, -6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_constData_rank0_offset0(self):
      arg0=numpy.array(6.0)
      arg1=Data(-1.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(-6.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank1_offset1(self):
      arg0=numpy.array([7.0, 4.0, 4.0])
      arg1=Data(numpy.array([1.0, -2.0, -6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(-25.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank2_offset2(self):
      arg0=numpy.array([[-1.0, 3.0, 5.0], [5.0, -6.0, 2.0]])
      arg1=Data(numpy.array([[7.0, -4.0, -5.0], [0.0, 1.0, -7.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(-64.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank3_offset3(self):
      arg0=numpy.array([[[6.0, 2.0, -5.0], [0.0, 4.0, 6.0], [-7.0, 2.0, 4.0], [5.0, -2.0, -7.0]], [[1.0, 3.0, 4.0], [-1.0, -1.0, -5.0], [-5.0, 6.0, 3.0], [5.0, 0.0, -5.0]]])
      arg1=Data(numpy.array([[[-3.0, -7.0, 6.0], [7.0, 0.0, -4.0], [-2.0, -2.0, 2.0], [-7.0, -3.0, 3.0]], [[-4.0, -7.0, -5.0], [0.0, 4.0, 4.0], [0.0, 5.0, 4.0], [4.0, -3.0, 3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(-140.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank4_offset4(self):
      arg0=numpy.array([[[[0.0, -7.0], [3.0, -7.0], [3.0, -7.0]], [[7.0, -5.0], [4.0, 1.0], [1.0, -2.0]]], [[[4.0, -6.0], [-6.0, -5.0], [1.0, 5.0]], [[-5.0, -2.0], [7.0, -5.0], [0.0, 3.0]]], [[[1.0, -2.0], [0.0, 5.0], [2.0, -3.0]], [[-3.0, -7.0], [-3.0, -4.0], [-3.0, -7.0]]], [[[1.0, 0.0], [0.0, 6.0], [-1.0, -7.0]], [[-5.0, -7.0], [5.0, 6.0], [6.0, 7.0]]]])
      arg1=Data(numpy.array([[[[4.0, 1.0], [-1.0, 0.0], [0.0, 0.0]], [[-3.0, 2.0], [4.0, -3.0], [-3.0, -3.0]]], [[[2.0, -5.0], [6.0, -5.0], [0.0, -6.0]], [[2.0, 0.0], [6.0, 4.0], [-2.0, 6.0]]], [[[-6.0, -3.0], [-3.0, -2.0], [1.0, 6.0]], [[-2.0, -1.0], [7.0, -3.0], [-3.0, -1.0]]], [[[-1.0, 0.0], [0.0, -6.0], [-6.0, 4.0]], [[6.0, -6.0], [6.0, 2.0], [-5.0, 7.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      ref=Data(10.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_constData_rank1_offset0(self):
      arg0=numpy.array(-7.0)
      arg1=Data(numpy.array([-5.0, 3.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([35.0, -21.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank2_offset1(self):
      arg0=numpy.array([7.0, 4.0, 7.0])
      arg1=Data(numpy.array([[-3.0, 6.0, 3.0], [0.0, -6.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([24.0, -24.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank3_offset2(self):
      arg0=numpy.array([[2.0, -4.0, -5.0], [3.0, 5.0, -4.0]])
      arg1=Data(numpy.array([[[0.0, -2.0, 3.0], [-6.0, 4.0, -6.0]], [[2.0, -5.0, 4.0], [-5.0, -6.0, -3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([19.0, -29.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank4_offset3(self):
      arg0=numpy.array([[[7.0, 6.0, 7.0], [0.0, 4.0, -4.0], [-2.0, -6.0, 0.0], [-2.0, -6.0, 0.0]], [[0.0, 7.0, 3.0], [2.0, -4.0, 7.0], [-3.0, 3.0, -7.0], [4.0, -5.0, -2.0]]])
      arg1=Data(numpy.array([[[[-4.0, -6.0, 0.0], [0.0, -4.0, -2.0], [4.0, -5.0, 7.0], [1.0, -6.0, -4.0]], [[-4.0, -2.0, 7.0], [-3.0, -1.0, 6.0], [0.0, -2.0, 5.0], [2.0, -3.0, -2.0]]], [[[6.0, 3.0, -3.0], [2.0, 5.0, 0.0], [3.0, 3.0, 3.0], [-7.0, 6.0, -1.0]], [[0.0, -2.0, -3.0], [-3.0, 5.0, 2.0], [6.0, 0.0, 1.0], [2.0, -1.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([17.0, -34.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_constData_rank2_offset0(self):
      arg0=numpy.array(-5.0)
      arg1=Data(numpy.array([[-2.0, 4.0, 2.0, -3.0, -5.0], [4.0, 5.0, 0.0, 5.0, 1.0], [-6.0, -2.0, -7.0, -1.0, 0.0], [1.0, 2.0, 2.0, 6.0, 2.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[10.0, -20.0, -10.0, 15.0, 25.0], [-20.0, -25.0, -0.0, -25.0, -5.0], [30.0, 10.0, 35.0, 5.0, -0.0], [-5.0, -10.0, -10.0, -30.0, -10.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank3_offset1(self):
      arg0=numpy.array([-3.0, 0.0, 2.0])
      arg1=Data(numpy.array([[[7.0, -3.0, -4.0], [1.0, 6.0, -5.0], [0.0, 7.0, 1.0], [1.0, 2.0, 6.0], [1.0, -5.0, 4.0]], [[-4.0, 0.0, -3.0], [-1.0, 7.0, -4.0], [7.0, -3.0, -5.0], [2.0, -6.0, 0.0], [-5.0, 2.0, -3.0]], [[6.0, 6.0, 5.0], [6.0, -2.0, -2.0], [-2.0, 3.0, 1.0], [3.0, 0.0, -4.0], [4.0, 0.0, 6.0]], [[-6.0, 0.0, 3.0], [-7.0, 3.0, 3.0], [6.0, 6.0, -1.0], [4.0, -5.0, 0.0], [7.0, -1.0, -2.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-29.0, -13.0, 2.0, 9.0, 5.0], [6.0, -5.0, -31.0, -6.0, 9.0], [-8.0, -22.0, 8.0, -17.0, 0.0], [24.0, 27.0, -20.0, -12.0, -25.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank4_offset2(self):
      arg0=numpy.array([[-5.0, 5.0, 3.0], [-5.0, -4.0, -3.0]])
      arg1=Data(numpy.array([[[[0.0, -2.0, -3.0], [6.0, -2.0, 2.0]], [[-6.0, 6.0, -3.0], [2.0, 4.0, -4.0]], [[1.0, -3.0, 6.0], [-3.0, -3.0, -3.0]], [[1.0, 7.0, -1.0], [0.0, 1.0, 5.0]], [[-7.0, -7.0, 0.0], [-5.0, -3.0, -6.0]]], [[[7.0, -5.0, -1.0], [4.0, -2.0, 6.0]], [[7.0, -4.0, 0.0], [-1.0, -7.0, -6.0]], [[-5.0, 7.0, 5.0], [4.0, 0.0, -7.0]], [[0.0, -4.0, -6.0], [-3.0, 7.0, 0.0]], [[-4.0, 0.0, 4.0], [0.0, 3.0, -1.0]]], [[[-7.0, -7.0, 7.0], [-2.0, -2.0, -1.0]], [[-3.0, -5.0, 4.0], [6.0, -2.0, 3.0]], [[-3.0, 3.0, -1.0], [-3.0, -2.0, 4.0]], [[0.0, -6.0, -7.0], [-1.0, -4.0, 1.0]], [[0.0, 5.0, -4.0], [3.0, 5.0, -1.0]]], [[[7.0, 6.0, 6.0], [6.0, -1.0, -2.0]], [[-7.0, 5.0, 4.0], [-2.0, -7.0, 0.0]], [[0.0, 3.0, -6.0], [4.0, -2.0, 6.0]], [[-4.0, 6.0, -4.0], [0.0, 2.0, -2.0]], [[1.0, 4.0, 6.0], [-7.0, -3.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-47.0, 37.0, 34.0, 8.0, 55.0], [-93.0, -4.0, 76.0, -51.0, 23.0], [42.0, -29.0, 38.0, -33.0, -19.0], [-7.0, 110.0, -33.0, 36.0, 80.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_constData_rank3_offset0(self):
      arg0=numpy.array(3.0)
      arg1=Data(numpy.array([[[6.0, -2.0], [5.0, -1.0]], [[5.0, 7.0], [0.0, 7.0]], [[-5.0, 5.0], [-1.0, -4.0]], [[-7.0, -6.0], [0.0, -1.0]], [[3.0, 2.0], [0.0, -3.0]], [[-4.0, 0.0], [5.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[18.0, -6.0], [15.0, -3.0]], [[15.0, 21.0], [0.0, 21.0]], [[-15.0, 15.0], [-3.0, -12.0]], [[-21.0, -18.0], [0.0, -3.0]], [[9.0, 6.0], [0.0, -9.0]], [[-12.0, 0.0], [15.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank4_offset1(self):
      arg0=numpy.array([-3.0, -7.0, 4.0])
      arg1=Data(numpy.array([[[[-2.0, -4.0, 2.0], [1.0, 7.0, 4.0]], [[3.0, -4.0, -6.0], [0.0, -1.0, 0.0]]], [[[-6.0, 7.0, 1.0], [-1.0, -1.0, 5.0]], [[6.0, 6.0, -7.0], [7.0, 6.0, -5.0]]], [[[0.0, -1.0, 0.0], [7.0, -5.0, 6.0]], [[2.0, 3.0, 2.0], [-3.0, 4.0, 5.0]]], [[[0.0, -3.0, -1.0], [2.0, 1.0, 6.0]], [[-5.0, -2.0, 6.0], [7.0, -2.0, 1.0]]], [[[-3.0, -7.0, 2.0], [2.0, 2.0, 2.0]], [[0.0, 6.0, -3.0], [2.0, -6.0, 0.0]]], [[[4.0, 6.0, 5.0], [0.0, 6.0, -3.0]], [[-2.0, -7.0, -7.0], [3.0, -2.0, 2.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[42.0, -36.0], [-5.0, 7.0]], [[-27.0, 30.0], [-88.0, -83.0]], [[7.0, 38.0], [-19.0, 1.0]], [[17.0, 11.0], [53.0, -3.0]], [[66.0, -12.0], [-54.0, 36.0]], [[-34.0, -54.0], [27.0, 13.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_constData_rank4_offset0(self):
      arg0=numpy.array(-2.0)
      arg1=Data(numpy.array([[[[0.0, 4.0, -5.0, -2.0], [5.0, -5.0, 6.0, 4.0], [0.0, 0.0, -1.0, -3.0]], [[2.0, -6.0, 6.0, -6.0], [0.0, -4.0, 7.0, -1.0], [2.0, 5.0, 0.0, 5.0]]], [[[-2.0, -3.0, 7.0, 0.0], [-5.0, -6.0, -3.0, 3.0], [6.0, -2.0, 1.0, 4.0]], [[-1.0, -3.0, 0.0, 0.0], [0.0, -6.0, 7.0, -6.0], [0.0, 0.0, 0.0, 4.0]]], [[[7.0, -3.0, 0.0, -5.0], [7.0, -6.0, -2.0, -2.0], [-3.0, -3.0, 7.0, 6.0]], [[-7.0, 0.0, -5.0, 7.0], [-2.0, -7.0, -3.0, 3.0], [3.0, 5.0, 6.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-0.0, -8.0, 10.0, 4.0], [-10.0, 10.0, -12.0, -8.0], [-0.0, -0.0, 2.0, 6.0]], [[-4.0, 12.0, -12.0, 12.0], [-0.0, 8.0, -14.0, 2.0], [-4.0, -10.0, -0.0, -10.0]]], [[[4.0, 6.0, -14.0, -0.0], [10.0, 12.0, 6.0, -6.0], [-12.0, 4.0, -2.0, -8.0]], [[2.0, 6.0, -0.0, -0.0], [-0.0, 12.0, -14.0, 12.0], [-0.0, -0.0, -0.0, -8.0]]], [[[-14.0, 6.0, -0.0, 10.0], [-14.0, 12.0, 4.0, 4.0], [6.0, 6.0, -14.0, -12.0]], [[14.0, -0.0, 10.0, -14.0], [4.0, 14.0, 6.0, -6.0], [-6.0, -10.0, -12.0, -0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_expandedData_rank0_offset0(self):
      arg0=numpy.array(3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-12.0)+(1.-msk_ref)*numpy.array(12.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank1_offset1(self):
      arg0=numpy.array([-5.0, 0.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([3.0, -1.0, -6.0])+(1.-msk_arg1)*numpy.array([5.0, -1.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-15.0)+(1.-msk_ref)*numpy.array(-25.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank2_offset2(self):
      arg0=numpy.array([[-2.0, -6.0, -7.0], [-4.0, 3.0, -6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, -3.0, -6.0], [-1.0, 1.0, 1.0]])+(1.-msk_arg1)*numpy.array([[-7.0, 3.0, 4.0], [0.0, -2.0, 2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(61.0)+(1.-msk_ref)*numpy.array(-50.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[-6.0, -6.0, 4.0], [-4.0, 2.0, 6.0], [-1.0, 0.0, 3.0], [4.0, 0.0, -4.0]], [[-7.0, 6.0, 3.0], [5.0, 7.0, 7.0], [1.0, -7.0, 0.0], [1.0, -7.0, -5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, -6.0, -7.0], [5.0, 3.0, 6.0], [-4.0, 5.0, -4.0], [-1.0, -2.0, 3.0]], [[-1.0, 5.0, 3.0], [1.0, -1.0, -2.0], [0.0, 7.0, -3.0], [0.0, -1.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 2.0, -5.0], [-7.0, -5.0, -1.0], [-4.0, 0.0, 5.0], [-1.0, 0.0, -2.0]], [[5.0, -7.0, 1.0], [-2.0, -7.0, -4.0], [1.0, -1.0, -3.0], [5.0, 3.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-39.0)+(1.-msk_ref)*numpy.array(-186.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank4_offset4(self):
      arg0=numpy.array([[[[5.0, 0.0], [0.0, 2.0], [6.0, 7.0]], [[-5.0, 0.0], [5.0, -1.0], [1.0, 0.0]]], [[[0.0, -7.0], [-1.0, -3.0], [-7.0, 0.0]], [[-6.0, 0.0], [-2.0, -4.0], [6.0, -7.0]]], [[[3.0, 0.0], [-2.0, 4.0], [0.0, -6.0]], [[5.0, -2.0], [-1.0, -3.0], [0.0, 2.0]]], [[[-7.0, 0.0], [6.0, -2.0], [0.0, 3.0]], [[-2.0, -7.0], [0.0, 3.0], [-1.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, -2.0], [3.0, -7.0], [-3.0, -1.0]], [[-7.0, -3.0], [-4.0, 0.0], [4.0, -5.0]]], [[[3.0, 3.0], [-6.0, -6.0], [-3.0, -5.0]], [[5.0, 1.0], [0.0, -6.0], [-5.0, 3.0]]], [[[-2.0, -2.0], [7.0, 4.0], [-2.0, -5.0]], [[6.0, 2.0], [2.0, -5.0], [-6.0, 0.0]]], [[[2.0, -6.0], [4.0, 6.0], [0.0, 4.0]], [[0.0, 0.0], [-3.0, 5.0], [5.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, -2.0], [3.0, -7.0], [4.0, 5.0]], [[0.0, -5.0], [7.0, -3.0], [-4.0, -3.0]]], [[[4.0, 0.0], [0.0, -7.0], [2.0, -2.0]], [[1.0, 5.0], [4.0, -7.0], [-1.0, 3.0]]], [[[5.0, 0.0], [3.0, -7.0], [0.0, 5.0]], [[4.0, -2.0], [7.0, -6.0], [-2.0, 0.0]]], [[[3.0, -6.0], [-4.0, 0.0], [5.0, -4.0]], [[1.0, -1.0], [7.0, 1.0], [-5.0, 3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(0.0)+(1.-msk_ref)*numpy.array(-18.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_expandedData_rank1_offset0(self):
      arg0=numpy.array(-2.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, -4.0])+(1.-msk_arg1)*numpy.array([3.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-10.0, 8.0])+(1.-msk_ref)*numpy.array([-6.0, -14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank2_offset1(self):
      arg0=numpy.array([0.0, 3.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 1.0, 7.0], [6.0, 3.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-7.0, 2.0, -3.0], [6.0, -4.0, -6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 9.0])+(1.-msk_ref)*numpy.array([6.0, -12.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank3_offset2(self):
      arg0=numpy.array([[3.0, -7.0, 4.0], [3.0, -1.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, -1.0, -3.0], [4.0, -7.0, 0.0]], [[4.0, 3.0, -4.0], [0.0, 7.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, -1.0, 1.0], [0.0, -2.0, 3.0]], [[6.0, 2.0, -7.0], [-1.0, -3.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([29.0, -47.0])+(1.-msk_ref)*numpy.array([1.0, -27.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[-1.0, 1.0, -6.0], [-1.0, 0.0, -1.0], [7.0, -6.0, 2.0], [-6.0, -2.0, -2.0]], [[2.0, 6.0, 0.0], [-7.0, -7.0, -6.0], [-7.0, 7.0, 2.0], [0.0, -6.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -1.0, -7.0], [-5.0, -5.0, 4.0], [6.0, 3.0, 0.0], [0.0, 3.0, 0.0]], [[3.0, -3.0, 4.0], [-4.0, 7.0, 0.0], [-5.0, -5.0, -6.0], [-3.0, 7.0, 2.0]]], [[[6.0, 1.0, 0.0], [4.0, -5.0, -5.0], [5.0, 0.0, 5.0], [-3.0, 4.0, -5.0]], [[4.0, -7.0, -5.0], [-2.0, 0.0, 7.0], [-5.0, -2.0, -6.0], [5.0, 2.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, 6.0, 4.0], [2.0, 7.0, 6.0], [0.0, -2.0, 0.0], [1.0, 0.0, -3.0]], [[0.0, 2.0, 4.0], [-7.0, 4.0, -6.0], [-5.0, 2.0, 3.0], [4.0, 4.0, -6.0]]], [[[7.0, 1.0, -7.0], [6.0, -6.0, 6.0], [-7.0, 5.0, -7.0], [-1.0, 2.0, -4.0]], [[-5.0, 3.0, 0.0], [-1.0, 1.0, 6.0], [5.0, 0.0, -7.0], [7.0, -3.0, 5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-39.0, 20.0])+(1.-msk_ref)*numpy.array([123.0, -148.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_expandedData_rank2_offset0(self):
      arg0=numpy.array(-1.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, 5.0, -4.0, 5.0, 0.0], [2.0, 1.0, -3.0, -5.0, 0.0], [0.0, -3.0, -6.0, 0.0, -6.0], [-7.0, -1.0, 4.0, -1.0, -4.0]])+(1.-msk_arg1)*numpy.array([[2.0, 0.0, 0.0, -2.0, -5.0], [-6.0, -4.0, 0.0, 2.0, 5.0], [-3.0, -5.0, -7.0, 6.0, 0.0], [-1.0, 1.0, -6.0, 2.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-1.0, -5.0, 4.0, -5.0, -0.0], [-2.0, -1.0, 3.0, 5.0, -0.0], [-0.0, 3.0, 6.0, -0.0, 6.0], [7.0, 1.0, -4.0, 1.0, 4.0]])+(1.-msk_ref)*numpy.array([[-2.0, -0.0, -0.0, 2.0, 5.0], [6.0, 4.0, -0.0, -2.0, -5.0], [3.0, 5.0, 7.0, -6.0, -0.0], [1.0, -1.0, 6.0, -2.0, 7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank3_offset1(self):
      arg0=numpy.array([-4.0, -4.0, -5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -3.0, -2.0], [5.0, 4.0, -6.0], [0.0, -2.0, -4.0], [-5.0, 4.0, 0.0], [3.0, -2.0, -1.0]], [[-6.0, 7.0, 0.0], [0.0, -3.0, -4.0], [2.0, 4.0, 0.0], [0.0, -3.0, 2.0], [3.0, -6.0, 5.0]], [[6.0, -5.0, -1.0], [0.0, -3.0, 7.0], [-2.0, -2.0, 2.0], [4.0, -3.0, -1.0], [0.0, -6.0, -1.0]], [[0.0, 4.0, -7.0], [7.0, 4.0, 5.0], [2.0, -2.0, 3.0], [3.0, 6.0, 6.0], [6.0, -4.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 1.0, 2.0], [0.0, 6.0, 1.0], [-5.0, -7.0, -6.0], [-1.0, 7.0, 6.0], [-3.0, 0.0, -2.0]], [[4.0, -3.0, -2.0], [4.0, 6.0, 1.0], [-6.0, 4.0, 0.0], [-3.0, 1.0, 5.0], [-3.0, -7.0, -2.0]], [[-1.0, 5.0, 0.0], [-4.0, 1.0, 0.0], [-5.0, 6.0, 2.0], [0.0, -4.0, 2.0], [4.0, 0.0, -1.0]], [[-7.0, 7.0, 1.0], [2.0, -1.0, -5.0], [-3.0, -3.0, -4.0], [7.0, 0.0, 6.0], [-2.0, 0.0, 6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[22.0, -6.0, 28.0, 4.0, 1.0], [-4.0, 32.0, -24.0, 2.0, -13.0], [1.0, -23.0, 6.0, 1.0, 29.0], [19.0, -69.0, -15.0, -66.0, -28.0]])+(1.-msk_ref)*numpy.array([[-14.0, -29.0, 78.0, -54.0, 22.0], [6.0, -45.0, 8.0, -17.0, 50.0], [-16.0, 12.0, -14.0, 6.0, -11.0], [-5.0, 21.0, 44.0, -58.0, -22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank4_offset2(self):
      arg0=numpy.array([[2.0, -3.0, 4.0], [-3.0, 3.0, -5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -6.0, -2.0], [0.0, -4.0, 5.0]], [[2.0, 1.0, 0.0], [-5.0, 6.0, -4.0]], [[4.0, 4.0, -5.0], [2.0, 4.0, 6.0]], [[-6.0, 7.0, 0.0], [-1.0, -7.0, 2.0]], [[-1.0, 0.0, 2.0], [7.0, -4.0, -4.0]]], [[[-7.0, 0.0, -5.0], [-1.0, -6.0, -2.0]], [[1.0, -6.0, 6.0], [4.0, 5.0, -1.0]], [[-3.0, 1.0, -5.0], [2.0, 1.0, 1.0]], [[2.0, -4.0, 1.0], [-7.0, -1.0, 3.0]], [[5.0, 0.0, 3.0], [5.0, -6.0, 1.0]]], [[[5.0, -6.0, -2.0], [-2.0, -2.0, -7.0]], [[3.0, -1.0, -2.0], [-7.0, -2.0, 3.0]], [[-6.0, 5.0, 4.0], [6.0, 0.0, 0.0]], [[0.0, 5.0, -3.0], [4.0, 5.0, 0.0]], [[0.0, 3.0, 5.0], [-1.0, -5.0, 2.0]]], [[[6.0, -7.0, 0.0], [-4.0, 7.0, 0.0]], [[1.0, 7.0, 4.0], [0.0, 6.0, -3.0]], [[0.0, -7.0, 3.0], [3.0, 7.0, -2.0]], [[-6.0, -6.0, 7.0], [1.0, 2.0, 7.0]], [[7.0, 1.0, 0.0], [6.0, 4.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 4.0, 7.0], [2.0, -5.0, -3.0]], [[0.0, -5.0, 0.0], [6.0, -2.0, 6.0]], [[-2.0, -4.0, -1.0], [6.0, -4.0, -4.0]], [[-6.0, 1.0, -4.0], [-1.0, 6.0, 0.0]], [[3.0, -1.0, 1.0], [6.0, 0.0, -6.0]]], [[[1.0, 0.0, 0.0], [0.0, -5.0, -4.0]], [[-2.0, -2.0, 3.0], [-2.0, 7.0, 5.0]], [[0.0, 1.0, 3.0], [2.0, -3.0, 1.0]], [[3.0, 5.0, -4.0], [0.0, 5.0, 2.0]], [[4.0, 7.0, 4.0], [6.0, 0.0, 5.0]]], [[[0.0, -2.0, 2.0], [-1.0, 6.0, 3.0]], [[-1.0, 5.0, 6.0], [-5.0, -6.0, -3.0]], [[0.0, 7.0, 4.0], [6.0, -4.0, -6.0]], [[-3.0, -3.0, 5.0], [-6.0, 3.0, -3.0]], [[0.0, 3.0, 6.0], [-7.0, 7.0, 1.0]]], [[[-2.0, 2.0, -7.0], [-6.0, -3.0, -4.0]], [[-6.0, 5.0, -5.0], [0.0, 1.0, 5.0]], [[3.0, -2.0, -1.0], [-7.0, 2.0, 1.0]], [[6.0, 5.0, 2.0], [-5.0, 4.0, 6.0]], [[-5.0, 1.0, 7.0], [0.0, 7.0, 1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-31.0, 54.0, -48.0, -61.0, -7.0], [-39.0, 52.0, -37.0, 23.0, -16.0], [55.0, 1.0, -29.0, -24.0, -11.0], [66.0, 30.0, 55.0, 2.0, -15.0]])+(1.-msk_ref)*numpy.array([[6.0, -39.0, -6.0, -10.0, 25.0], [7.0, 16.0, -11.0, -20.0, -40.0], [20.0, 19.0, -5.0, 65.0, 52.0], [-9.0, -69.0, 30.0, 2.0, 31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_expandedData_rank3_offset0(self):
      arg0=numpy.array(5.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -7.0], [-1.0, 7.0]], [[-1.0, -3.0], [4.0, 0.0]], [[-6.0, 0.0], [4.0, -7.0]], [[6.0, -1.0], [-6.0, 5.0]], [[-5.0, -5.0], [-1.0, 2.0]], [[-1.0, 3.0], [0.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, -1.0], [5.0, 3.0]], [[-1.0, -7.0], [0.0, 2.0]], [[-2.0, 5.0], [1.0, -1.0]], [[-1.0, -5.0], [4.0, 4.0]], [[-7.0, -2.0], [-3.0, -7.0]], [[-5.0, -3.0], [3.0, 3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -35.0], [-5.0, 35.0]], [[-5.0, -15.0], [20.0, 0.0]], [[-30.0, 0.0], [20.0, -35.0]], [[30.0, -5.0], [-30.0, 25.0]], [[-25.0, -25.0], [-5.0, 10.0]], [[-5.0, 15.0], [0.0, 30.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -5.0], [25.0, 15.0]], [[-5.0, -35.0], [0.0, 10.0]], [[-10.0, 25.0], [5.0, -5.0]], [[-5.0, -25.0], [20.0, 20.0]], [[-35.0, -10.0], [-15.0, -35.0]], [[-25.0, -15.0], [15.0, 15.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank4_offset1(self):
      arg0=numpy.array([-1.0, 3.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[6.0, -2.0, -3.0], [5.0, 5.0, 7.0]], [[4.0, 0.0, 2.0], [-4.0, 0.0, 2.0]]], [[[-6.0, -2.0, 0.0], [5.0, 5.0, -2.0]], [[-7.0, 7.0, -1.0], [5.0, 5.0, -4.0]]], [[[-2.0, -4.0, 4.0], [5.0, 6.0, 3.0]], [[-7.0, -1.0, 5.0], [-2.0, 5.0, -4.0]]], [[[-1.0, 4.0, 6.0], [5.0, 4.0, -3.0]], [[2.0, 1.0, 2.0], [-3.0, 0.0, -4.0]]], [[[6.0, 6.0, 0.0], [-7.0, 3.0, -2.0]], [[7.0, 5.0, -3.0], [-6.0, -4.0, 4.0]]], [[[2.0, 6.0, 2.0], [-1.0, -5.0, -3.0]], [[4.0, -6.0, 4.0], [-4.0, 1.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, -1.0, -5.0], [-7.0, -7.0, 0.0]], [[-7.0, -7.0, -3.0], [7.0, -4.0, -2.0]]], [[[-7.0, 3.0, 6.0], [-2.0, 7.0, -5.0]], [[-7.0, -2.0, 2.0], [0.0, -4.0, 1.0]]], [[[3.0, 5.0, 2.0], [-2.0, 0.0, -3.0]], [[-4.0, -4.0, 7.0], [4.0, -2.0, -4.0]]], [[[7.0, -2.0, -2.0], [1.0, 2.0, 0.0]], [[2.0, -2.0, -2.0], [7.0, -7.0, 0.0]]], [[[-6.0, 0.0, 6.0], [-1.0, -3.0, 5.0]], [[-7.0, 2.0, -2.0], [2.0, 4.0, 7.0]]], [[[2.0, 5.0, -7.0], [-1.0, 4.0, -5.0]], [[7.0, -6.0, 4.0], [-3.0, 0.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, 10.0], [-4.0, 4.0]], [[0.0, 10.0], [28.0, 10.0]], [[-10.0, 13.0], [4.0, 17.0]], [[13.0, 7.0], [1.0, 3.0]], [[12.0, 16.0], [8.0, -6.0]], [[16.0, -14.0], [-22.0, 7.0]]])+(1.-msk_ref)*numpy.array([[[-2.0, -14.0], [-14.0, -19.0]], [[16.0, 23.0], [1.0, -12.0]], [[12.0, 2.0], [-8.0, -10.0]], [[-13.0, 5.0], [-8.0, -28.0]], [[6.0, -8.0], [13.0, 10.0]], [[13.0, 13.0], [-25.0, 3.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank0_expandedData_rank4_offset0(self):
      arg0=numpy.array(-5.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, 5.0, -3.0, -5.0], [0.0, 3.0, -2.0, 6.0], [7.0, -3.0, 3.0, 0.0]], [[2.0, 0.0, -4.0, 5.0], [-1.0, -4.0, 1.0, 2.0], [2.0, -1.0, 6.0, 5.0]]], [[[2.0, 0.0, 1.0, -4.0], [2.0, 0.0, -6.0, 5.0], [-5.0, 0.0, -1.0, 2.0]], [[-1.0, -4.0, 1.0, 6.0], [1.0, 7.0, 7.0, -6.0], [6.0, -6.0, -2.0, 2.0]]], [[[5.0, 0.0, 3.0, -6.0], [7.0, 7.0, 0.0, -5.0], [0.0, 6.0, 7.0, 2.0]], [[5.0, -7.0, -3.0, 0.0], [0.0, 5.0, -4.0, 7.0], [7.0, -5.0, -2.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, 7.0, 2.0, -6.0], [-4.0, -1.0, -3.0, -7.0], [4.0, -1.0, -5.0, 5.0]], [[6.0, 1.0, -6.0, 7.0], [0.0, 5.0, 6.0, -1.0], [3.0, -2.0, -3.0, -4.0]]], [[[1.0, -5.0, -4.0, 5.0], [6.0, 2.0, -7.0, 2.0], [-4.0, 0.0, 5.0, 0.0]], [[-4.0, -7.0, -3.0, -2.0], [1.0, -1.0, -3.0, -5.0], [1.0, -2.0, 3.0, 6.0]]], [[[0.0, 0.0, -7.0, 7.0], [-6.0, 7.0, -6.0, 7.0], [3.0, 0.0, -2.0, -2.0]], [[-2.0, -2.0, 3.0, -5.0], [5.0, -4.0, 5.0, 4.0], [2.0, -5.0, 0.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[30.0, -25.0, 15.0, 25.0], [-0.0, -15.0, 10.0, -30.0], [-35.0, 15.0, -15.0, -0.0]], [[-10.0, -0.0, 20.0, -25.0], [5.0, 20.0, -5.0, -10.0], [-10.0, 5.0, -30.0, -25.0]]], [[[-10.0, -0.0, -5.0, 20.0], [-10.0, -0.0, 30.0, -25.0], [25.0, -0.0, 5.0, -10.0]], [[5.0, 20.0, -5.0, -30.0], [-5.0, -35.0, -35.0, 30.0], [-30.0, 30.0, 10.0, -10.0]]], [[[-25.0, -0.0, -15.0, 30.0], [-35.0, -35.0, -0.0, 25.0], [-0.0, -30.0, -35.0, -10.0]], [[-25.0, 35.0, 15.0, -0.0], [-0.0, -25.0, 20.0, -35.0], [-35.0, 25.0, 10.0, 25.0]]]])+(1.-msk_ref)*numpy.array([[[[-5.0, -35.0, -10.0, 30.0], [20.0, 5.0, 15.0, 35.0], [-20.0, 5.0, 25.0, -25.0]], [[-30.0, -5.0, 30.0, -35.0], [-0.0, -25.0, -30.0, 5.0], [-15.0, 10.0, 15.0, 20.0]]], [[[-5.0, 25.0, 20.0, -25.0], [-30.0, -10.0, 35.0, -10.0], [20.0, -0.0, -25.0, -0.0]], [[20.0, 35.0, 15.0, 10.0], [-5.0, 5.0, 15.0, 25.0], [-5.0, 10.0, -15.0, -30.0]]], [[[-0.0, -0.0, 35.0, -35.0], [30.0, -35.0, 30.0, -35.0], [-15.0, -0.0, 10.0, 10.0]], [[10.0, 10.0, -15.0, 25.0], [-25.0, 20.0, -25.0, -20.0], [-10.0, 25.0, -0.0, -0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_float_rank0_offset0(self):
      arg0=numpy.array([-1.0, -1.0])
      arg1=-3.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([3.0, 3.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank0_offset0(self):
      arg0=numpy.array([4.0, 1.0])
      arg1=numpy.array(-1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([-4.0, -1.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank1_offset1(self):
      arg0=numpy.array([[-2.0, -1.0, -2.0], [1.0, 0.0, 7.0]])
      arg1=numpy.array([3.0, 5.0, -2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([-7.0, -11.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank2_offset2(self):
      arg0=numpy.array([[[-3.0, -3.0, 3.0], [7.0, -7.0, -3.0]], [[-4.0, 0.0, -3.0], [1.0, -6.0, -2.0]]])
      arg1=numpy.array([[4.0, 2.0, 3.0], [0.0, 7.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([-76.0, -79.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank3_offset3(self):
      arg0=numpy.array([[[[2.0, -6.0, 5.0], [7.0, -4.0, 6.0], [-6.0, -4.0, 3.0], [-4.0, -3.0, -4.0]], [[0.0, -3.0, -5.0], [7.0, -1.0, -1.0], [0.0, 0.0, -5.0], [-7.0, -3.0, -2.0]]], [[[3.0, 0.0, 6.0], [-2.0, -6.0, 3.0], [1.0, 0.0, 0.0], [3.0, 0.0, 7.0]], [[-4.0, 7.0, 6.0], [2.0, 6.0, -3.0], [5.0, 1.0, -5.0], [-6.0, 3.0, 6.0]]]])
      arg1=numpy.array([[[6.0, -3.0, -1.0], [3.0, 2.0, -5.0], [1.0, 5.0, 6.0], [4.0, -4.0, 6.0]], [[0.0, 2.0, 6.0], [7.0, 3.0, -4.0], [0.0, 2.0, -7.0], [-6.0, -1.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([68.0, 192.0])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank1_offset0(self):
      arg0=numpy.array([-3.0, 6.0])
      arg1=numpy.array([-1.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[3.0, -18.0], [-6.0, 36.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank2_offset1(self):
      arg0=numpy.array([[-4.0, -2.0, -4.0], [1.0, -4.0, -5.0]])
      arg1=numpy.array([[-3.0, 2.0, -7.0], [-2.0, 0.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[36.0, -20.0], [24.0, -37.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank3_offset2(self):
      arg0=numpy.array([[[5.0, 7.0, -6.0], [7.0, 3.0, 0.0]], [[-2.0, 6.0, 5.0], [0.0, 6.0, 4.0]]])
      arg1=numpy.array([[[3.0, 7.0, -2.0], [1.0, 7.0, 0.0]], [[0.0, -1.0, -1.0], [-2.0, 2.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[104.0, -9.0], [68.0, 1.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank4_offset3(self):
      arg0=numpy.array([[[[0.0, -2.0, 1.0], [-7.0, 3.0, -2.0], [3.0, 4.0, 7.0], [4.0, 7.0, 4.0]], [[5.0, 0.0, -2.0], [-1.0, 1.0, 5.0], [5.0, 0.0, 3.0], [6.0, -3.0, 3.0]]], [[[0.0, -6.0, 4.0], [-4.0, -5.0, -3.0], [-6.0, -1.0, 0.0], [2.0, -7.0, -6.0]], [[-2.0, -1.0, 0.0], [-5.0, -4.0, -6.0], [-2.0, 3.0, 5.0], [0.0, 3.0, -4.0]]]])
      arg1=numpy.array([[[[3.0, 6.0, -1.0], [2.0, -6.0, -3.0], [-3.0, 5.0, -6.0], [0.0, -6.0, 0.0]], [[-7.0, -4.0, 5.0], [-5.0, -7.0, -6.0], [4.0, 0.0, -7.0], [0.0, -6.0, -7.0]]], [[[-6.0, 4.0, -5.0], [4.0, -1.0, 3.0], [-2.0, -1.0, -2.0], [2.0, -2.0, -1.0]], [[0.0, 3.0, 7.0], [6.0, -2.0, 5.0], [-7.0, -1.0, 3.0], [7.0, -2.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=numpy.array([[-193.0, -80.0], [120.0, -34.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank2_offset0(self):
      arg0=numpy.array([-6.0, -5.0])
      arg1=numpy.array([[-5.0, 2.0, 5.0, 1.0, 3.0], [-1.0, -4.0, 5.0, -1.0, 0.0], [4.0, -4.0, -1.0, 4.0, -6.0], [2.0, 0.0, -6.0, 0.0, -2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[30.0, -12.0, -30.0, -6.0, -18.0], [6.0, 24.0, -30.0, 6.0, 0.0], [-24.0, 24.0, 6.0, -24.0, 36.0], [-12.0, 0.0, 36.0, 0.0, 12.0]], [[25.0, -10.0, -25.0, -5.0, -15.0], [5.0, 20.0, -25.0, 5.0, 0.0], [-20.0, 20.0, 5.0, -20.0, 30.0], [-10.0, 0.0, 30.0, 0.0, 10.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank3_offset1(self):
      arg0=numpy.array([[6.0, -3.0, 0.0], [0.0, 1.0, -2.0]])
      arg1=numpy.array([[[-1.0, 7.0, -2.0], [7.0, 1.0, 1.0], [7.0, 6.0, -6.0], [-2.0, 7.0, 0.0], [-3.0, -3.0, 1.0]], [[0.0, 7.0, -5.0], [3.0, 2.0, -6.0], [0.0, -2.0, 4.0], [6.0, 5.0, -1.0], [-6.0, 2.0, -3.0]], [[-3.0, 0.0, 0.0], [1.0, 7.0, -1.0], [-1.0, 4.0, -5.0], [6.0, -3.0, -7.0], [-2.0, 5.0, 6.0]], [[-3.0, 2.0, -3.0], [5.0, 1.0, 6.0], [-5.0, -7.0, 6.0], [6.0, -1.0, -6.0], [3.0, 5.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[-27.0, 39.0, 24.0, -33.0, -9.0], [-21.0, 12.0, 6.0, 21.0, -42.0], [-18.0, -15.0, -18.0, 45.0, -27.0], [-24.0, 27.0, -9.0, 39.0, 3.0]], [[11.0, -1.0, 18.0, 7.0, -5.0], [17.0, 14.0, -10.0, 7.0, 8.0], [0.0, 9.0, 14.0, 11.0, -7.0], [8.0, -11.0, -19.0, 11.0, 5.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank4_offset2(self):
      arg0=numpy.array([[[-2.0, -6.0, -5.0], [5.0, -2.0, 0.0]], [[-1.0, -2.0, 1.0], [4.0, -1.0, 5.0]]])
      arg1=numpy.array([[[[5.0, 1.0, -3.0], [-7.0, -2.0, 0.0]], [[2.0, -7.0, 1.0], [-7.0, 0.0, -6.0]], [[-5.0, 1.0, -1.0], [-6.0, 2.0, 3.0]], [[-6.0, -1.0, -4.0], [-5.0, 6.0, 6.0]], [[-7.0, -6.0, 4.0], [-5.0, -1.0, 5.0]]], [[[3.0, -2.0, 7.0], [3.0, 0.0, 2.0]], [[4.0, 6.0, -6.0], [7.0, 4.0, 0.0]], [[-5.0, -3.0, -6.0], [6.0, -3.0, 3.0]], [[2.0, 4.0, 2.0], [0.0, 0.0, -4.0]], [[1.0, -3.0, 1.0], [0.0, -3.0, -2.0]]], [[[3.0, -2.0, -5.0], [7.0, -3.0, -6.0]], [[7.0, 0.0, 0.0], [3.0, -1.0, 0.0]], [[-1.0, -1.0, 7.0], [5.0, -6.0, -7.0]], [[-5.0, 5.0, -2.0], [3.0, 0.0, -3.0]], [[3.0, 1.0, 4.0], [-2.0, -6.0, -3.0]]], [[[4.0, 5.0, -5.0], [3.0, 1.0, 0.0]], [[4.0, -7.0, -7.0], [-6.0, -5.0, 1.0]], [[-2.0, 1.0, 7.0], [3.0, 0.0, 0.0]], [[-4.0, 5.0, 2.0], [4.0, 4.0, -3.0]], [[-5.0, 0.0, 7.0], [0.0, -7.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[-32.0, -2.0, -25.0, 1.0, 7.0], [-14.0, 13.0, 94.0, -38.0, 17.0], [72.0, 3.0, 10.0, 5.0, -30.0], [0.0, 49.0, -22.0, -20.0, -11.0]], [[-36.0, -45.0, -9.0, 8.0, 29.0], [30.0, 2.0, 47.0, -28.0, -1.0], [-3.0, 6.0, 1.0, -10.0, -18.0], [-8.0, -11.0, 19.0, -7.0, -16.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_array_rank3_offset0(self):
      arg0=numpy.array([2.0, 7.0])
      arg1=numpy.array([[[-3.0, -2.0], [-5.0, -1.0]], [[4.0, 0.0], [7.0, 2.0]], [[5.0, 4.0], [-1.0, 3.0]], [[-2.0, -2.0], [5.0, 5.0]], [[-7.0, -6.0], [6.0, 7.0]], [[-4.0, 0.0], [4.0, -3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-6.0, -4.0], [-10.0, -2.0]], [[8.0, 0.0], [14.0, 4.0]], [[10.0, 8.0], [-2.0, 6.0]], [[-4.0, -4.0], [10.0, 10.0]], [[-14.0, -12.0], [12.0, 14.0]], [[-8.0, 0.0], [8.0, -6.0]]], [[[-21.0, -14.0], [-35.0, -7.0]], [[28.0, 0.0], [49.0, 14.0]], [[35.0, 28.0], [-7.0, 21.0]], [[-14.0, -14.0], [35.0, 35.0]], [[-49.0, -42.0], [42.0, 49.0]], [[-28.0, 0.0], [28.0, -21.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank4_offset1(self):
      arg0=numpy.array([[1.0, 2.0, 1.0], [2.0, -4.0, 3.0]])
      arg1=numpy.array([[[[-5.0, 0.0, 7.0], [7.0, -3.0, 0.0]], [[2.0, 2.0, -3.0], [-6.0, 2.0, 7.0]]], [[[-2.0, -2.0, 3.0], [-7.0, 6.0, -7.0]], [[-1.0, -6.0, -4.0], [-4.0, -1.0, -1.0]]], [[[-3.0, -2.0, -4.0], [-4.0, 0.0, -4.0]], [[5.0, 0.0, 7.0], [1.0, 0.0, -7.0]]], [[[-7.0, -5.0, -3.0], [1.0, -7.0, -2.0]], [[-3.0, -4.0, -4.0], [7.0, 4.0, -1.0]]], [[[0.0, 4.0, -1.0], [-5.0, 5.0, 6.0]], [[-2.0, -4.0, -4.0], [0.0, 0.0, -5.0]]], [[[-7.0, 7.0, -3.0], [-2.0, 7.0, -7.0]], [[-6.0, 2.0, 1.0], [-5.0, -4.0, 6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[2.0, 1.0], [3.0, 5.0]], [[-3.0, -2.0], [-17.0, -7.0]], [[-11.0, -8.0], [12.0, -6.0]], [[-20.0, -15.0], [-15.0, 14.0]], [[7.0, 11.0], [-14.0, -5.0]], [[4.0, 5.0], [-1.0, -7.0]]], [[[11.0, 26.0], [-13.0, 1.0]], [[13.0, -59.0], [10.0, -7.0]], [[-10.0, -20.0], [31.0, -19.0]], [[-3.0, 24.0], [-2.0, -5.0]], [[-19.0, -12.0], [0.0, -15.0]], [[-51.0, -53.0], [-17.0, 24.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank0_offset0(self):
      arg0=numpy.array([-1.0, 4.0])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([-7.0, 28.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank1_offset1(self):
      arg0=numpy.array([[5.0, 5.0, 0.0], [-6.0, -2.0, 1.0]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([2.0, 4.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([30.0, -14.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[-6.0, 7.0, 0.0], [-7.0, -5.0, 1.0]], [[-1.0, 0.0, -5.0], [0.0, -5.0, 5.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-2.0, 4.0, -3.0], [2.0, -5.0, 6.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([57.0, 72.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank3_offset3(self):
      arg0=numpy.array([[[[0.0, 0.0, 0.0], [1.0, -2.0, 2.0], [4.0, -5.0, -1.0], [-7.0, -6.0, 0.0]], [[0.0, 0.0, -4.0], [-5.0, 4.0, 6.0], [-6.0, -4.0, 6.0], [-3.0, -2.0, 3.0]]], [[[3.0, -1.0, -6.0], [3.0, 5.0, 4.0], [-6.0, 3.0, 0.0], [0.0, 3.0, -1.0]], [[2.0, -3.0, -2.0], [6.0, 2.0, 1.0], [-3.0, 1.0, -6.0], [-2.0, 5.0, 5.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[5.0, 6.0, -6.0], [-6.0, -7.0, 0.0], [5.0, -3.0, 3.0], [-6.0, -4.0, -2.0]], [[-7.0, 1.0, 7.0], [-6.0, 7.0, 2.0], [-1.0, 3.0, 3.0], [6.0, -6.0, 4.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([166.0, -142.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank1_offset0(self):
      arg0=numpy.array([0.0, 4.0])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([4.0, -5.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[0.0, 0.0], [16.0, -20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank2_offset1(self):
      arg0=numpy.array([[6.0, -1.0, -4.0], [-5.0, 1.0, -6.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-6.0, 5.0, -2.0], [4.0, -6.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-33.0, 46.0], [47.0, -2.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[-2.0, 0.0, -6.0], [1.0, -4.0, 6.0]], [[0.0, -1.0, -4.0], [-1.0, 6.0, -2.0]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-5.0, 1.0, -7.0], [4.0, 6.0, 5.0]], [[4.0, -6.0, 0.0], [-5.0, 4.0, -3.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[62.0, -47.0], [49.0, 41.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank4_offset3(self):
      arg0=numpy.array([[[[-6.0, 0.0, 0.0], [-1.0, 0.0, 7.0], [-3.0, 0.0, 5.0], [5.0, 0.0, 7.0]], [[-7.0, -4.0, 6.0], [0.0, 7.0, 2.0], [4.0, 7.0, 3.0], [-3.0, 0.0, 6.0]]], [[[2.0, 4.0, 6.0], [-2.0, 0.0, -5.0], [1.0, -6.0, 0.0], [3.0, 6.0, -3.0]], [[-6.0, -7.0, -6.0], [5.0, -3.0, 0.0], [-7.0, 2.0, -6.0], [3.0, -4.0, 0.0]]]])
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-7.0, -6.0, -3.0], [6.0, 0.0, 0.0], [-7.0, -1.0, 3.0], [5.0, 0.0, 0.0]], [[0.0, 5.0, -6.0], [-3.0, 0.0, -4.0], [0.0, -5.0, -2.0], [0.0, 6.0, -1.0]]], [[[-6.0, 4.0, -3.0], [-4.0, -6.0, -5.0], [5.0, -2.0, 1.0], [-2.0, -6.0, -1.0]], [[-4.0, -1.0, 0.0], [0.0, 0.0, 5.0], [-7.0, 3.0, -1.0], [0.0, 3.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[-14.0, -8.0], [-90.0, 77.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank2_offset0(self):
      arg0=numpy.array([-3.0, -5.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-1.0, -4.0, 0.0, 5.0, 6.0], [0.0, 5.0, -3.0, 3.0, 4.0], [-1.0, -7.0, 3.0, -2.0, 7.0], [0.0, -1.0, 4.0, 1.0, -1.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[3.0, 12.0, 0.0, -15.0, -18.0], [0.0, -15.0, 9.0, -9.0, -12.0], [3.0, 21.0, -9.0, 6.0, -21.0], [0.0, 3.0, -12.0, -3.0, 3.0]], [[5.0, 20.0, 0.0, -25.0, -30.0], [0.0, -25.0, 15.0, -15.0, -20.0], [5.0, 35.0, -15.0, 10.0, -35.0], [0.0, 5.0, -20.0, -5.0, 5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank3_offset1(self):
      arg0=numpy.array([[7.0, -6.0, 3.0], [5.0, 2.0, -2.0]])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[5.0, 0.0, -5.0], [6.0, 0.0, -7.0], [-5.0, -5.0, 0.0], [3.0, -7.0, 2.0], [5.0, 7.0, -2.0]], [[-7.0, -1.0, 4.0], [4.0, 5.0, 0.0], [-2.0, 0.0, 7.0], [-5.0, -2.0, 1.0], [3.0, -3.0, -3.0]], [[0.0, 7.0, 7.0], [4.0, 0.0, 3.0], [2.0, 3.0, -5.0], [-3.0, -1.0, 0.0], [6.0, 0.0, 1.0]], [[0.0, -6.0, -3.0], [-7.0, 0.0, 4.0], [1.0, 5.0, -6.0], [2.0, 6.0, 2.0], [-1.0, 5.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[20.0, 21.0, -5.0, 69.0, -13.0], [-31.0, -2.0, 7.0, -20.0, 30.0], [-21.0, 37.0, -19.0, -15.0, 45.0], [27.0, -37.0, -41.0, -16.0, -28.0]], [[35.0, 44.0, -35.0, -3.0, 43.0], [-45.0, 30.0, -24.0, -31.0, 15.0], [0.0, 14.0, 26.0, -17.0, 28.0], [-6.0, -43.0, 27.0, 18.0, -1.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[6.0, -2.0, -1.0], [1.0, -2.0, 5.0]], [[-5.0, 2.0, -5.0], [4.0, -2.0, 6.0]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[4.0, 4.0, -6.0], [-5.0, 4.0, -5.0]], [[1.0, -2.0, -1.0], [7.0, 0.0, 1.0]], [[-4.0, -1.0, 3.0], [6.0, -7.0, 5.0]], [[0.0, 7.0, 3.0], [3.0, 4.0, -2.0]], [[-7.0, 1.0, 1.0], [4.0, -6.0, 0.0]]], [[[-3.0, -7.0, 6.0], [-7.0, 6.0, -3.0]], [[-3.0, -5.0, 6.0], [-4.0, 0.0, 7.0]], [[-6.0, -1.0, 4.0], [-1.0, 4.0, -4.0]], [[0.0, -2.0, 2.0], [4.0, -2.0, 5.0]], [[3.0, 2.0, -6.0], [4.0, -6.0, -4.0]]], [[[1.0, -2.0, 0.0], [-6.0, 4.0, 1.0]], [[3.0, 7.0, -5.0], [-1.0, -2.0, 0.0]], [[5.0, 7.0, -5.0], [4.0, -3.0, 2.0]], [[3.0, 0.0, 4.0], [5.0, -2.0, -3.0]], [[-2.0, -3.0, -1.0], [-6.0, -1.0, -5.0]]], [[[1.0, -5.0, -3.0], [-4.0, 2.0, -5.0]], [[-6.0, -4.0, 1.0], [-5.0, 1.0, 5.0]], [[-2.0, 7.0, -2.0], [-7.0, 1.0, 2.0]], [[-4.0, -3.0, -3.0], [-1.0, 0.0, 4.0]], [[6.0, 0.0, 1.0], [-7.0, 0.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-16.0, 23.0, 20.0, -32.0, -29.0], [-44.0, 17.0, -67.0, 35.0, 16.0], [1.0, 12.0, 41.0, 8.0, -34.0], [-14.0, -11.0, -23.0, 4.0, 28.0]], [[-40.0, 30.0, 71.0, -9.0, 60.0], [-87.0, 1.0, -28.0, 36.0, 23.0], [-35.0, 24.0, 48.0, -29.0, -43.0], [-50.0, 25.0, 16.0, 49.0, -63.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_Symbol_rank3_offset0(self):
      arg0=numpy.array([4.0, 5.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[4.0, 6.0], [-4.0, -7.0]], [[0.0, 1.0], [7.0, 7.0]], [[-7.0, 3.0], [-1.0, 7.0]], [[0.0, 5.0], [3.0, -1.0]], [[2.0, 7.0], [-2.0, -1.0]], [[1.0, -2.0], [6.0, -6.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[16.0, 24.0], [-16.0, -28.0]], [[0.0, 4.0], [28.0, 28.0]], [[-28.0, 12.0], [-4.0, 28.0]], [[0.0, 20.0], [12.0, -4.0]], [[8.0, 28.0], [-8.0, -4.0]], [[4.0, -8.0], [24.0, -24.0]]], [[[20.0, 30.0], [-20.0, -35.0]], [[0.0, 5.0], [35.0, 35.0]], [[-35.0, 15.0], [-5.0, 35.0]], [[0.0, 25.0], [15.0, -5.0]], [[10.0, 35.0], [-10.0, -5.0]], [[5.0, -10.0], [30.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank4_offset1(self):
      arg0=numpy.array([[7.0, 6.0, -1.0], [-2.0, -4.0, -6.0]])
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[4.0, -1.0, -4.0], [5.0, -7.0, 5.0]], [[7.0, 0.0, -4.0], [-6.0, -7.0, 6.0]]], [[[1.0, 7.0, 6.0], [7.0, -5.0, 7.0]], [[6.0, 6.0, 7.0], [-1.0, 0.0, 0.0]]], [[[-5.0, -7.0, 1.0], [-3.0, -7.0, 1.0]], [[-5.0, 0.0, 7.0], [-7.0, 4.0, -4.0]]], [[[-2.0, -2.0, -4.0], [2.0, -5.0, 4.0]], [[0.0, 1.0, 5.0], [2.0, -3.0, -7.0]]], [[[0.0, -1.0, 1.0], [-5.0, -3.0, 0.0]], [[1.0, 5.0, -5.0], [2.0, 0.0, 0.0]]], [[[7.0, 2.0, -1.0], [-1.0, -3.0, -6.0]], [[0.0, 2.0, 4.0], [-2.0, -5.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[26.0, -12.0], [53.0, -90.0]], [[43.0, 12.0], [71.0, -7.0]], [[-78.0, -64.0], [-42.0, -21.0]], [[-22.0, -20.0], [1.0, 3.0]], [[-7.0, -53.0], [42.0, 14.0]], [[62.0, -19.0], [8.0, -49.0]]], [[[20.0, -12.0], [10.0, 4.0]], [[-66.0, -36.0], [-78.0, 2.0]], [[32.0, 28.0], [-32.0, 22.0]], [[36.0, -8.0], [-34.0, 50.0]], [[-2.0, 22.0], [8.0, -4.0]], [[-16.0, 50.0], [-32.0, -6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank0_offset0(self):
      arg0=numpy.array([4.0, 3.0])
      arg1=Data(-4.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-16.0, -12.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank1_offset1(self):
      arg0=numpy.array([[-5.0, 0.0, -1.0], [-7.0, -3.0, -5.0]])
      arg1=Data(numpy.array([1.0, 0.0, 2.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-7.0, -17.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank2_offset2(self):
      arg0=numpy.array([[[-3.0, 0.0, 7.0], [4.0, -3.0, -3.0]], [[1.0, -5.0, -3.0], [6.0, 6.0, 4.0]]])
      arg1=Data(numpy.array([[5.0, 0.0, 3.0], [1.0, 5.0, 2.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-11.0, 40.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank3_offset3(self):
      arg0=numpy.array([[[[2.0, 4.0, 2.0], [-2.0, -2.0, 0.0], [-7.0, -1.0, -5.0], [4.0, -7.0, -5.0]], [[6.0, 7.0, -7.0], [-4.0, 4.0, 3.0], [0.0, -4.0, -3.0], [-2.0, 1.0, -7.0]]], [[[2.0, 2.0, -7.0], [4.0, 5.0, -3.0], [-2.0, 6.0, -6.0], [1.0, -1.0, -2.0]], [[-6.0, -1.0, 6.0], [-3.0, -7.0, 5.0], [4.0, -7.0, -7.0], [-4.0, 4.0, -5.0]]]])
      arg1=Data(numpy.array([[[0.0, 0.0, -3.0], [0.0, 1.0, -4.0], [0.0, 7.0, 5.0], [5.0, 5.0, -2.0]], [[-4.0, 0.0, -2.0], [-3.0, -2.0, 0.0], [1.0, -3.0, 7.0], [-3.0, 1.0, 1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-60.0, 76.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank1_offset0(self):
      arg0=numpy.array([4.0, 2.0])
      arg1=Data(numpy.array([6.0, -7.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[24.0, -28.0], [12.0, -14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank2_offset1(self):
      arg0=numpy.array([[7.0, -7.0, 3.0], [3.0, -4.0, 6.0]])
      arg1=Data(numpy.array([[-6.0, 3.0, -2.0], [-1.0, 0.0, 6.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-69.0, 11.0], [-42.0, 33.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank3_offset2(self):
      arg0=numpy.array([[[4.0, -3.0, 2.0], [0.0, -1.0, -1.0]], [[-1.0, -7.0, -2.0], [-2.0, 0.0, -4.0]]])
      arg1=Data(numpy.array([[[-1.0, 4.0, 3.0], [3.0, 4.0, 0.0]], [[0.0, -2.0, -7.0], [2.0, -6.0, -2.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-14.0, 0.0], [-39.0, 32.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank4_offset3(self):
      arg0=numpy.array([[[[1.0, -7.0, 2.0], [-3.0, 1.0, 1.0], [6.0, -3.0, 1.0], [1.0, -5.0, 6.0]], [[0.0, 1.0, -3.0], [-3.0, 2.0, 6.0], [3.0, -4.0, -5.0], [0.0, -1.0, -4.0]]], [[[7.0, 1.0, 0.0], [-1.0, 7.0, 6.0], [0.0, 3.0, -3.0], [4.0, -3.0, 6.0]], [[-5.0, 4.0, -2.0], [-2.0, 5.0, -7.0], [4.0, 6.0, 1.0], [4.0, 7.0, -1.0]]]])
      arg1=Data(numpy.array([[[[7.0, 5.0, -1.0], [7.0, 5.0, -1.0], [-6.0, -6.0, 7.0], [-2.0, -2.0, 6.0]], [[-3.0, 4.0, -4.0], [3.0, 6.0, 5.0], [7.0, 7.0, 7.0], [2.0, 0.0, 6.0]]], [[[-4.0, 0.0, 2.0], [-3.0, -5.0, -5.0], [-1.0, 3.0, -6.0], [7.0, -1.0, 0.0]], [[-3.0, 0.0, -3.0], [-4.0, -2.0, -4.0], [-5.0, 4.0, 4.0], [-5.0, 5.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[-31.0, -73.0], [178.0, 38.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank2_offset0(self):
      arg0=numpy.array([0.0, -6.0])
      arg1=Data(numpy.array([[1.0, 4.0, 0.0, 0.0, -6.0], [-4.0, -3.0, -3.0, -1.0, 5.0], [3.0, -1.0, 0.0, 0.0, -5.0], [0.0, -5.0, -2.0, -2.0, 3.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, -24.0, 0.0, 0.0, 36.0], [24.0, 18.0, 18.0, 6.0, -30.0], [-18.0, 6.0, 0.0, 0.0, 30.0], [0.0, 30.0, 12.0, 12.0, -18.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank3_offset1(self):
      arg0=numpy.array([[0.0, -3.0, 6.0], [-1.0, -4.0, 1.0]])
      arg1=Data(numpy.array([[[-5.0, -4.0, 1.0], [-7.0, -5.0, 5.0], [6.0, 1.0, -6.0], [0.0, -6.0, 6.0], [2.0, 1.0, 6.0]], [[4.0, 6.0, 6.0], [-5.0, -6.0, 7.0], [-2.0, -3.0, -1.0], [0.0, -3.0, 0.0], [3.0, -4.0, -4.0]], [[6.0, 0.0, 3.0], [-3.0, -1.0, -4.0], [0.0, 2.0, -2.0], [5.0, -5.0, 1.0], [6.0, 4.0, 0.0]], [[-4.0, 1.0, 2.0], [-6.0, 0.0, 6.0], [6.0, -7.0, -5.0], [0.0, 3.0, 2.0], [5.0, 1.0, 6.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[18.0, 45.0, -39.0, 54.0, 33.0], [18.0, 60.0, 3.0, 9.0, -12.0], [18.0, -21.0, -18.0, 21.0, -12.0], [9.0, 36.0, -9.0, 3.0, 33.0]], [[22.0, 32.0, -16.0, 30.0, 0.0], [-22.0, 36.0, 13.0, 12.0, 9.0], [-3.0, 3.0, -10.0, 16.0, -22.0], [2.0, 12.0, 17.0, -10.0, -3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank4_offset2(self):
      arg0=numpy.array([[[-7.0, 7.0, -6.0], [-7.0, -6.0, -4.0]], [[4.0, 6.0, -7.0], [1.0, 2.0, -6.0]]])
      arg1=Data(numpy.array([[[[3.0, -3.0, -1.0], [0.0, -7.0, 0.0]], [[6.0, 0.0, 7.0], [-4.0, 1.0, -7.0]], [[-2.0, 5.0, 0.0], [3.0, 4.0, -1.0]], [[-2.0, 0.0, -5.0], [-5.0, 3.0, 5.0]], [[-7.0, -6.0, 2.0], [-5.0, 2.0, -4.0]]], [[[2.0, -7.0, -3.0], [-3.0, -7.0, 0.0]], [[4.0, 5.0, 7.0], [-5.0, 3.0, 0.0]], [[3.0, -7.0, -7.0], [-3.0, -1.0, -6.0]], [[2.0, 0.0, 3.0], [-5.0, 2.0, 1.0]], [[7.0, -5.0, -3.0], [-4.0, 0.0, -6.0]]], [[[-7.0, -5.0, 3.0], [-6.0, 0.0, -6.0]], [[-1.0, 2.0, 4.0], [-1.0, 7.0, 0.0]], [[1.0, 5.0, 1.0], [5.0, -1.0, -1.0]], [[-4.0, 3.0, -2.0], [2.0, 1.0, 3.0]], [[1.0, -3.0, 0.0], [1.0, -5.0, 5.0]]], [[[-6.0, -3.0, 1.0], [-2.0, -1.0, 0.0]], [[-2.0, -1.0, 1.0], [-2.0, 6.0, 0.0]], [[2.0, -7.0, 2.0], [-3.0, 2.0, -3.0]], [[-7.0, -4.0, 1.0], [-5.0, 1.0, 4.0]], [[7.0, 5.0, -4.0], [6.0, -4.0, 3.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[6.0, -34.0, 8.0, 41.0, 34.0], [18.0, -18.0, 23.0, -13.0, -14.0], [62.0, -38.0, -3.0, 29.0, -25.0], [35.0, -21.0, -54.0, 28.0, -20.0]], [[-13.0, 15.0, 39.0, -2.0, -55.0], [-30.0, -2.0, 50.0, -20.0, 51.0], [-49.0, -7.0, 36.0, 2.0, -53.0], [-53.0, -11.0, -29.0, -86.0, 66.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_constData_rank3_offset0(self):
      arg0=numpy.array([3.0, 1.0])
      arg1=Data(numpy.array([[[2.0, 2.0], [6.0, 4.0]], [[5.0, -2.0], [-4.0, -5.0]], [[-4.0, -3.0], [1.0, 0.0]], [[-6.0, 1.0], [-6.0, -2.0]], [[2.0, 7.0], [4.0, 0.0]], [[-3.0, 7.0], [2.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[6.0, 6.0], [18.0, 12.0]], [[15.0, -6.0], [-12.0, -15.0]], [[-12.0, -9.0], [3.0, 0.0]], [[-18.0, 3.0], [-18.0, -6.0]], [[6.0, 21.0], [12.0, 0.0]], [[-9.0, 21.0], [6.0, 0.0]]], [[[2.0, 2.0], [6.0, 4.0]], [[5.0, -2.0], [-4.0, -5.0]], [[-4.0, -3.0], [1.0, 0.0]], [[-6.0, 1.0], [-6.0, -2.0]], [[2.0, 7.0], [4.0, 0.0]], [[-3.0, 7.0], [2.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank4_offset1(self):
      arg0=numpy.array([[-7.0, -2.0, 0.0], [5.0, 7.0, 1.0]])
      arg1=Data(numpy.array([[[[1.0, 4.0, 3.0], [1.0, 2.0, -2.0]], [[6.0, -3.0, -3.0], [0.0, 6.0, -7.0]]], [[[3.0, -2.0, 2.0], [2.0, -1.0, -1.0]], [[-7.0, 7.0, -7.0], [0.0, -4.0, -1.0]]], [[[-1.0, 4.0, -7.0], [7.0, 2.0, -1.0]], [[-7.0, 5.0, -4.0], [1.0, -5.0, 0.0]]], [[[7.0, -2.0, -7.0], [6.0, 6.0, 7.0]], [[-7.0, -7.0, 0.0], [6.0, -3.0, 0.0]]], [[[0.0, 6.0, 3.0], [2.0, -1.0, 0.0]], [[-7.0, -6.0, 0.0], [1.0, 0.0, 2.0]]], [[[6.0, 0.0, 4.0], [1.0, 3.0, -3.0]], [[-3.0, 2.0, -7.0], [-3.0, 5.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-15.0, -11.0], [-36.0, -12.0]], [[-17.0, -12.0], [35.0, 8.0]], [[-1.0, -53.0], [39.0, 3.0]], [[-45.0, -54.0], [63.0, -36.0]], [[-12.0, -12.0], [61.0, -7.0]], [[-42.0, -13.0], [17.0, 11.0]]], [[[36.0, 17.0], [6.0, 35.0]], [[3.0, 2.0], [7.0, -29.0]], [[16.0, 48.0], [-4.0, -30.0]], [[14.0, 79.0], [-84.0, 9.0]], [[45.0, 3.0], [-77.0, 7.0]], [[34.0, 23.0], [-8.0, 20.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank0_offset0(self):
      arg0=numpy.array([3.0, 0.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(2.0)+(1-msk_arg1)*(0.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([6.0, 0.0])+(1.-msk_ref)*numpy.array([0.0, 0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank1_offset1(self):
      arg0=numpy.array([[0.0, 5.0, 5.0], [7.0, -7.0, 6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, -2.0, -5.0])+(1.-msk_arg1)*numpy.array([-2.0, 7.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-35.0, -58.0])+(1.-msk_ref)*numpy.array([0.0, -105.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[-1.0, -1.0, -3.0], [5.0, -1.0, -7.0]], [[-5.0, -6.0, 4.0], [-5.0, -1.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, -7.0, 0.0], [-5.0, 2.0, -5.0]])+(1.-msk_arg1)*numpy.array([[7.0, -1.0, 6.0], [4.0, 0.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, 65.0])+(1.-msk_ref)*numpy.array([17.0, -25.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank3_offset3(self):
      arg0=numpy.array([[[[-2.0, 7.0, -1.0], [-5.0, -5.0, 7.0], [3.0, 6.0, 0.0], [-3.0, 0.0, 7.0]], [[-1.0, -6.0, -6.0], [-4.0, 0.0, -4.0], [-3.0, 3.0, -6.0], [-4.0, 1.0, -6.0]]], [[[0.0, 2.0, 1.0], [1.0, -2.0, -4.0], [0.0, -3.0, 0.0], [5.0, 2.0, -1.0]], [[-7.0, 0.0, 6.0], [-1.0, -2.0, 3.0], [1.0, 0.0, -4.0], [0.0, -2.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, 4.0, -6.0], [-5.0, -1.0, 5.0], [4.0, -1.0, 5.0], [-2.0, 2.0, 6.0]], [[0.0, 1.0, 0.0], [-2.0, 0.0, 5.0], [0.0, -7.0, -5.0], [5.0, -7.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 0.0, 4.0], [0.0, -5.0, 2.0], [5.0, -1.0, 2.0], [-4.0, -3.0, 6.0]], [[2.0, 0.0, -3.0], [0.0, -3.0, 1.0], [3.0, 0.0, 5.0], [7.0, -1.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([77.0, 51.0])+(1.-msk_ref)*numpy.array([78.0, -96.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank1_offset0(self):
      arg0=numpy.array([-3.0, 4.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 4.0])+(1.-msk_arg1)*numpy.array([0.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, -12.0], [0.0, 16.0]])+(1.-msk_ref)*numpy.array([[0.0, 18.0], [0.0, -24.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank2_offset1(self):
      arg0=numpy.array([[-5.0, -1.0, -1.0], [-5.0, 4.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, 2.0, 6.0], [-6.0, -3.0, -1.0]])+(1.-msk_arg1)*numpy.array([[-2.0, 3.0, -1.0], [6.0, -6.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-33.0, 34.0], [7.0, 14.0]])+(1.-msk_ref)*numpy.array([[8.0, -21.0], [18.0, -66.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[-5.0, -2.0, -2.0], [0.0, -1.0, 0.0]], [[6.0, -7.0, -5.0], [2.0, 4.0, 6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -1.0, -6.0], [-1.0, 1.0, 3.0]], [[-2.0, 6.0, 2.0], [6.0, 0.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, 3.0, 3.0], [-2.0, -4.0, 6.0]], [[1.0, 7.0, -6.0], [3.0, -7.0, 3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[13.0, -6.0], [57.0, -70.0]])+(1.-msk_ref)*numpy.array([[-43.0, 0.0], [22.0, -17.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank4_offset3(self):
      arg0=numpy.array([[[[4.0, -2.0, 7.0], [4.0, 4.0, -2.0], [3.0, -1.0, 7.0], [6.0, -2.0, 0.0]], [[-7.0, 4.0, 6.0], [1.0, 5.0, 0.0], [-4.0, -4.0, -2.0], [2.0, -3.0, -1.0]]], [[[5.0, -6.0, 5.0], [0.0, 6.0, 0.0], [-2.0, -1.0, -3.0], [2.0, -4.0, -1.0]], [[2.0, -2.0, 3.0], [2.0, -5.0, -2.0], [0.0, -7.0, -5.0], [0.0, -6.0, 3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, 6.0, -2.0], [-6.0, -5.0, -4.0], [-2.0, -6.0, 1.0], [-3.0, 0.0, -7.0]], [[-7.0, -6.0, -5.0], [1.0, 7.0, -4.0], [6.0, -5.0, 0.0], [2.0, -6.0, 2.0]]], [[[5.0, 0.0, -3.0], [4.0, 7.0, -3.0], [1.0, 1.0, 1.0], [2.0, 2.0, 0.0]], [[-7.0, -3.0, 1.0], [0.0, -5.0, 2.0], [2.0, -6.0, -1.0], [3.0, -2.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, -3.0, -5.0], [1.0, -5.0, -4.0], [0.0, 1.0, 6.0], [0.0, -2.0, 3.0]], [[-2.0, -6.0, -7.0], [0.0, -2.0, -3.0], [5.0, -4.0, 5.0], [0.0, -7.0, 5.0]]], [[[2.0, 3.0, -2.0], [1.0, 4.0, -1.0], [3.0, 5.0, 7.0], [6.0, 2.0, 4.0]], [[-5.0, 2.0, -2.0], [1.0, -3.0, -5.0], [7.0, 3.0, -3.0], [4.0, 2.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-6.0, 110.0], [-8.0, 129.0]])+(1.-msk_ref)*numpy.array([[-40.0, 76.0], [27.0, -25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank2_offset0(self):
      arg0=numpy.array([6.0, 3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, -4.0, -6.0, -6.0, -2.0], [-1.0, 3.0, 7.0, 6.0, 1.0], [5.0, 2.0, 5.0, 3.0, 4.0], [4.0, -7.0, -3.0, 0.0, 3.0]])+(1.-msk_arg1)*numpy.array([[1.0, -1.0, 5.0, -2.0, 1.0], [6.0, -2.0, 0.0, -3.0, 4.0], [0.0, -7.0, 2.0, -3.0, 3.0], [-5.0, -5.0, -2.0, -2.0, -2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-12.0, -24.0, -36.0, -36.0, -12.0], [-6.0, 18.0, 42.0, 36.0, 6.0], [30.0, 12.0, 30.0, 18.0, 24.0], [24.0, -42.0, -18.0, 0.0, 18.0]], [[-6.0, -12.0, -18.0, -18.0, -6.0], [-3.0, 9.0, 21.0, 18.0, 3.0], [15.0, 6.0, 15.0, 9.0, 12.0], [12.0, -21.0, -9.0, 0.0, 9.0]]])+(1.-msk_ref)*numpy.array([[[6.0, -6.0, 30.0, -12.0, 6.0], [36.0, -12.0, 0.0, -18.0, 24.0], [0.0, -42.0, 12.0, -18.0, 18.0], [-30.0, -30.0, -12.0, -12.0, -12.0]], [[3.0, -3.0, 15.0, -6.0, 3.0], [18.0, -6.0, 0.0, -9.0, 12.0], [0.0, -21.0, 6.0, -9.0, 9.0], [-15.0, -15.0, -6.0, -6.0, -6.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank3_offset1(self):
      arg0=numpy.array([[3.0, -4.0, 4.0], [6.0, -7.0, 6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-3.0, 2.0, -4.0], [-7.0, 3.0, 0.0], [-3.0, 1.0, -3.0], [0.0, -5.0, -1.0], [3.0, 7.0, -7.0]], [[-2.0, 1.0, 1.0], [-4.0, -1.0, -1.0], [-7.0, -6.0, 5.0], [4.0, 3.0, 7.0], [7.0, -1.0, -7.0]], [[7.0, 4.0, -1.0], [7.0, -7.0, -3.0], [4.0, 5.0, 1.0], [-3.0, -4.0, 0.0], [3.0, -3.0, 4.0]], [[2.0, 3.0, -7.0], [0.0, 5.0, 1.0], [0.0, -5.0, 7.0], [7.0, -5.0, 3.0], [1.0, -7.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -2.0, 3.0], [-1.0, -5.0, 0.0], [-3.0, 1.0, -5.0], [7.0, -5.0, -3.0], [4.0, 2.0, 0.0]], [[6.0, -3.0, 1.0], [5.0, 1.0, -3.0], [5.0, -7.0, 6.0], [-2.0, -6.0, 7.0], [4.0, -6.0, -7.0]], [[-6.0, 3.0, -5.0], [3.0, 0.0, 4.0], [-7.0, 1.0, -7.0], [-3.0, -1.0, 6.0], [-3.0, -2.0, 4.0]], [[-3.0, -4.0, -4.0], [-3.0, 5.0, -3.0], [-2.0, -1.0, -3.0], [-4.0, -4.0, 6.0], [4.0, -2.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-33.0, -33.0, -25.0, 16.0, -47.0], [-6.0, -12.0, 23.0, 28.0, -3.0], [1.0, 37.0, -4.0, 7.0, 37.0], [-34.0, -16.0, 48.0, 53.0, 19.0]], [[-56.0, -63.0, -43.0, 29.0, -73.0], [-13.0, -23.0, 30.0, 45.0, 7.0], [8.0, 73.0, -5.0, 10.0, 63.0], [-51.0, -29.0, 77.0, 95.0, 37.0]]])+(1.-msk_ref)*numpy.array([[[5.0, 17.0, -33.0, 29.0, 4.0], [34.0, -1.0, 67.0, 46.0, 8.0], [-50.0, 25.0, -53.0, 19.0, 15.0], [-9.0, -41.0, -14.0, 28.0, 28.0]], [[2.0, 29.0, -55.0, 59.0, 10.0], [63.0, 5.0, 115.0, 72.0, 24.0], [-87.0, 42.0, -91.0, 25.0, 20.0], [-14.0, -71.0, -23.0, 40.0, 50.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[-7.0, 0.0, 0.0], [-3.0, 0.0, 7.0]], [[-3.0, 7.0, 2.0], [4.0, 1.0, -2.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -2.0, 2.0], [-6.0, 2.0, 5.0]], [[3.0, -3.0, -7.0], [-3.0, 3.0, 4.0]], [[-4.0, -4.0, -4.0], [1.0, -7.0, -5.0]], [[0.0, -2.0, 4.0], [7.0, -7.0, 1.0]], [[0.0, 6.0, 3.0], [-4.0, -5.0, 7.0]]], [[[7.0, -4.0, -2.0], [-7.0, 0.0, -4.0]], [[-2.0, -5.0, -1.0], [0.0, -3.0, -4.0]], [[-4.0, -7.0, -1.0], [-3.0, 6.0, -4.0]], [[1.0, -4.0, 6.0], [3.0, 1.0, 1.0]], [[-5.0, -4.0, 7.0], [2.0, -6.0, -3.0]]], [[[6.0, 1.0, -7.0], [-4.0, -2.0, 4.0]], [[1.0, 0.0, 2.0], [2.0, -5.0, -7.0]], [[6.0, 4.0, 5.0], [7.0, 0.0, -4.0]], [[7.0, -2.0, -7.0], [-1.0, 0.0, 7.0]], [[4.0, -3.0, 1.0], [-3.0, 6.0, 7.0]]], [[[-4.0, -2.0, -4.0], [-4.0, 1.0, 6.0]], [[4.0, -3.0, -6.0], [-3.0, 4.0, -5.0]], [[-4.0, 0.0, 5.0], [4.0, -4.0, 0.0]], [[-2.0, 7.0, 7.0], [-1.0, 4.0, 7.0]], [[-5.0, 1.0, 5.0], [1.0, 2.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, 4.0, -1.0], [7.0, 1.0, -1.0]], [[-5.0, -3.0, 4.0], [2.0, 1.0, 2.0]], [[4.0, 5.0, 1.0], [-7.0, 7.0, 6.0]], [[-7.0, 6.0, 4.0], [3.0, 2.0, 3.0]], [[0.0, 0.0, 6.0], [-6.0, 2.0, -5.0]]], [[[6.0, -1.0, -4.0], [-6.0, -7.0, 6.0]], [[-2.0, 5.0, 0.0], [3.0, 3.0, 4.0]], [[2.0, -3.0, 3.0], [0.0, 7.0, 7.0]], [[3.0, -1.0, 3.0], [7.0, 0.0, 2.0]], [[-2.0, 6.0, -2.0], [-1.0, -2.0, 7.0]]], [[[4.0, -2.0, -5.0], [4.0, -5.0, -6.0]], [[4.0, 0.0, -4.0], [1.0, 2.0, 6.0]], [[-3.0, -4.0, -2.0], [0.0, 4.0, 7.0]], [[-1.0, -1.0, -6.0], [1.0, 4.0, -7.0]], [[-1.0, 7.0, 5.0], [-2.0, 1.0, 3.0]]], [[[2.0, -3.0, 0.0], [5.0, 5.0, -5.0]], [[-4.0, 0.0, 6.0], [0.0, 5.0, 0.0]], [[-7.0, -5.0, -6.0], [0.0, 1.0, -3.0]], [[1.0, 7.0, 0.0], [6.0, -1.0, -5.0]], [[0.0, -2.0, -3.0], [7.0, 4.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[46.0, 16.0, -10.0, -14.0, 61.0], [-56.0, -14.0, 9.0, -9.0, 8.0], [-2.0, -62.0, -91.0, 3.0, 30.0], [82.0, -54.0, 16.0, 66.0, 53.0]], [[-45.0, -61.0, -17.0, 13.0, 13.0], [-73.0, -26.0, -37.0, -8.0, 9.0], [-51.0, 18.0, 56.0, -67.0, -51.0], [-37.0, -43.0, 34.0, 55.0, 32.0]]])+(1.-msk_ref)*numpy.array([[[-56.0, 43.0, 35.0, 61.0, -17.0], [18.0, 33.0, 35.0, -28.0, 66.0], [-82.0, 11.0, 70.0, -45.0, 34.0], [-64.0, 28.0, 28.0, -60.0, -70.0]], [[45.0, 7.0, -8.0, 79.0, 0.0], [-76.0, 48.0, -28.0, 14.0, 24.0], [-13.0, -26.0, -33.0, 6.0, 49.0], [8.0, 29.0, -19.0, 79.0, 26.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank1_expandedData_rank3_offset0(self):
      arg0=numpy.array([-4.0, -5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 5.0], [0.0, 7.0]], [[0.0, -2.0], [-3.0, -2.0]], [[-3.0, -5.0], [7.0, -4.0]], [[2.0, 7.0], [7.0, 0.0]], [[1.0, 1.0], [-7.0, -3.0]], [[-6.0, -6.0], [5.0, 1.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 7.0], [6.0, -1.0]], [[-5.0, 0.0], [0.0, 4.0]], [[-1.0, 2.0], [5.0, -2.0]], [[4.0, 6.0], [4.0, 2.0]], [[0.0, -1.0], [7.0, 0.0]], [[0.0, 4.0], [-2.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-28.0, -20.0], [0.0, -28.0]], [[0.0, 8.0], [12.0, 8.0]], [[12.0, 20.0], [-28.0, 16.0]], [[-8.0, -28.0], [-28.0, 0.0]], [[-4.0, -4.0], [28.0, 12.0]], [[24.0, 24.0], [-20.0, -4.0]]], [[[-35.0, -25.0], [0.0, -35.0]], [[0.0, 10.0], [15.0, 10.0]], [[15.0, 25.0], [-35.0, 20.0]], [[-10.0, -35.0], [-35.0, 0.0]], [[-5.0, -5.0], [35.0, 15.0]], [[30.0, 30.0], [-25.0, -5.0]]]])+(1.-msk_ref)*numpy.array([[[[12.0, -28.0], [-24.0, 4.0]], [[20.0, 0.0], [0.0, -16.0]], [[4.0, -8.0], [-20.0, 8.0]], [[-16.0, -24.0], [-16.0, -8.0]], [[0.0, 4.0], [-28.0, 0.0]], [[0.0, -16.0], [8.0, 24.0]]], [[[15.0, -35.0], [-30.0, 5.0]], [[25.0, 0.0], [0.0, -20.0]], [[5.0, -10.0], [-25.0, 10.0]], [[-20.0, -30.0], [-20.0, -10.0]], [[0.0, 5.0], [-35.0, 0.0]], [[0.0, -20.0], [10.0, 30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank4_offset1(self):
      arg0=numpy.array([[-2.0, 6.0, 3.0], [7.0, -6.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, 3.0, -4.0], [3.0, -3.0, 3.0]], [[-3.0, 0.0, 5.0], [-3.0, 1.0, 3.0]]], [[[-6.0, 6.0, 3.0], [4.0, -2.0, -5.0]], [[7.0, -1.0, 0.0], [-2.0, 1.0, -6.0]]], [[[1.0, 4.0, 0.0], [-1.0, 3.0, -5.0]], [[2.0, 0.0, 0.0], [-4.0, 2.0, 0.0]]], [[[0.0, -4.0, 7.0], [-6.0, -1.0, 0.0]], [[3.0, 0.0, -7.0], [-2.0, 1.0, -1.0]]], [[[3.0, 6.0, 1.0], [5.0, 3.0, -7.0]], [[-2.0, 2.0, 7.0], [0.0, -1.0, 0.0]]], [[[-6.0, -7.0, 0.0], [-5.0, 5.0, -2.0]], [[-6.0, 1.0, 1.0], [-5.0, 4.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -3.0, -1.0], [5.0, -1.0, 0.0]], [[-3.0, 6.0, -1.0], [-7.0, 6.0, 4.0]]], [[[7.0, -3.0, 3.0], [-6.0, 0.0, 6.0]], [[-5.0, 0.0, -7.0], [0.0, -5.0, 5.0]]], [[[7.0, 1.0, -7.0], [-1.0, 5.0, 4.0]], [[3.0, -7.0, -3.0], [0.0, -1.0, -4.0]]], [[[-4.0, 6.0, -6.0], [-7.0, -4.0, 4.0]], [[-2.0, 2.0, 4.0], [6.0, 5.0, -4.0]]], [[[-1.0, -2.0, 0.0], [-7.0, -2.0, -1.0]], [[1.0, -3.0, -6.0], [-2.0, 5.0, 7.0]]], [[[0.0, -5.0, 5.0], [3.0, -2.0, -3.0]], [[-1.0, 4.0, 0.0], [7.0, 0.0, 7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[20.0, -15.0], [21.0, 21.0]], [[57.0, -35.0], [-20.0, -8.0]], [[22.0, 5.0], [-4.0, 20.0]], [[-3.0, 6.0], [-27.0, 7.0]], [[33.0, -13.0], [37.0, -6.0]], [[-30.0, 34.0], [21.0, 16.0]]], [[[-75.0, 45.0], [-11.0, -21.0]], [[-72.0, 30.0], [55.0, -32.0]], [[-17.0, -35.0], [14.0, -40.0]], [[38.0, -36.0], [7.0, -22.0]], [[-13.0, 3.0], [-12.0, 6.0]], [[0.0, -69.0], [-46.0, -71.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -16.0], [39.0, 62.0]], [[-23.0, 30.0], [-11.0, -15.0]], [[-29.0, 44.0], [-57.0, -18.0]], [[26.0, 2.0], [28.0, 6.0]], [[-10.0, -1.0], [-38.0, 55.0]], [[-15.0, -27.0], [26.0, 7.0]]], [[[16.0, 41.0], [-59.0, -77.0]], [[73.0, -30.0], [-49.0, 40.0]], [[29.0, -29.0], [57.0, -2.0]], [[-76.0, -17.0], [-18.0, 4.0]], [[5.0, -39.0], [13.0, -30.0]], [[40.0, 27.0], [-31.0, 63.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_float_rank0_offset0(self):
      arg0=numpy.array([[-4.0, 7.0, -2.0, 2.0, -5.0], [-4.0, 4.0, 0.0, -1.0, -2.0], [-7.0, 6.0, 0.0, 6.0, 4.0], [-7.0, -4.0, -1.0, -7.0, 7.0]])
      arg1=-4.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[16.0, -28.0, 8.0, -8.0, 20.0], [16.0, -16.0, -0.0, 4.0, 8.0], [28.0, -24.0, -0.0, -24.0, -16.0], [28.0, 16.0, 4.0, 28.0, -28.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank0_offset0(self):
      arg0=numpy.array([[7.0, 5.0, -3.0, -1.0, 6.0], [3.0, -7.0, 3.0, 6.0, -3.0], [0.0, 3.0, -2.0, -4.0, -6.0], [-3.0, -3.0, 3.0, -7.0, -5.0]])
      arg1=numpy.array(-3.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[-21.0, -15.0, 9.0, 3.0, -18.0], [-9.0, 21.0, -9.0, -18.0, 9.0], [-0.0, -9.0, 6.0, 12.0, 18.0], [9.0, 9.0, -9.0, 21.0, 15.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank1_offset1(self):
      arg0=numpy.array([[[5.0, 7.0, 4.0], [0.0, 6.0, -7.0], [-5.0, -3.0, 0.0], [-5.0, 0.0, -6.0], [-2.0, 3.0, 0.0]], [[-1.0, -5.0, 4.0], [0.0, -6.0, 5.0], [-5.0, 3.0, 4.0], [4.0, -3.0, 1.0], [5.0, -3.0, 0.0]], [[3.0, 0.0, -4.0], [4.0, 4.0, -5.0], [6.0, -4.0, 0.0], [4.0, 6.0, -3.0], [-2.0, 1.0, -6.0]], [[-3.0, -2.0, 3.0], [7.0, -2.0, 3.0], [7.0, -2.0, 0.0], [5.0, -6.0, -3.0], [-7.0, 1.0, -6.0]]])
      arg1=numpy.array([5.0, 0.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[1.0, 42.0, -25.0, 11.0, -10.0], [-29.0, -30.0, -49.0, 14.0, 25.0], [39.0, 50.0, 30.0, 38.0, 26.0], [-33.0, 17.0, 35.0, 43.0, 1.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank2_offset2(self):
      arg0=numpy.array([[[[0.0, -2.0, 7.0], [5.0, -1.0, 0.0]], [[-1.0, -3.0, -7.0], [-7.0, -2.0, 0.0]], [[-6.0, 0.0, -7.0], [-2.0, 6.0, 2.0]], [[1.0, 0.0, -4.0], [-2.0, 5.0, -5.0]], [[6.0, 6.0, 4.0], [7.0, 4.0, 2.0]]], [[[-7.0, 6.0, 1.0], [-2.0, -7.0, -6.0]], [[-3.0, 7.0, 5.0], [-3.0, -4.0, 3.0]], [[5.0, -2.0, -6.0], [3.0, -3.0, 0.0]], [[-4.0, 1.0, 7.0], [6.0, -4.0, 0.0]], [[-5.0, 2.0, 3.0], [-7.0, 3.0, 4.0]]], [[[2.0, 7.0, 2.0], [0.0, -3.0, 2.0]], [[2.0, -1.0, 3.0], [2.0, 7.0, 0.0]], [[2.0, 5.0, -5.0], [0.0, 3.0, -6.0]], [[4.0, 3.0, 4.0], [-5.0, -2.0, 2.0]], [[-5.0, 2.0, 7.0], [-2.0, 6.0, -5.0]]], [[[2.0, 6.0, -5.0], [5.0, -7.0, 1.0]], [[0.0, -6.0, 5.0], [7.0, 0.0, -5.0]], [[2.0, 1.0, -1.0], [0.0, 6.0, -3.0]], [[0.0, 0.0, 0.0], [-1.0, -5.0, 5.0]], [[-2.0, 0.0, -6.0], [3.0, 7.0, -2.0]]]])
      arg1=numpy.array([[0.0, 2.0, -1.0], [-5.0, 4.0, 3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[-40.0, 28.0, 47.0, 19.0, -5.0], [-25.0, 17.0, -25.0, -51.0, 60.0], [6.0, 13.0, 9.0, 25.0, 16.0], [-33.0, -67.0, 18.0, 0.0, 13.0]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank1_offset0(self):
      arg0=numpy.array([[-6.0, -2.0, -2.0, 4.0, -2.0], [-7.0, 1.0, -5.0, -5.0, 1.0], [2.0, 6.0, 6.0, 0.0, 5.0], [6.0, -4.0, 6.0, 5.0, -5.0]])
      arg1=numpy.array([6.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-36.0, 36.0], [-12.0, 12.0], [-12.0, 12.0], [24.0, -24.0], [-12.0, 12.0]], [[-42.0, 42.0], [6.0, -6.0], [-30.0, 30.0], [-30.0, 30.0], [6.0, -6.0]], [[12.0, -12.0], [36.0, -36.0], [36.0, -36.0], [0.0, 0.0], [30.0, -30.0]], [[36.0, -36.0], [-24.0, 24.0], [36.0, -36.0], [30.0, -30.0], [-30.0, 30.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank2_offset1(self):
      arg0=numpy.array([[[1.0, -1.0, -2.0], [-4.0, -4.0, -2.0], [-4.0, 3.0, 5.0], [5.0, 3.0, 0.0], [-7.0, 0.0, -2.0]], [[-5.0, 4.0, 1.0], [-4.0, 6.0, -2.0], [-7.0, 2.0, -6.0], [7.0, -3.0, -5.0], [7.0, 6.0, -1.0]], [[4.0, 5.0, 4.0], [5.0, 0.0, 6.0], [2.0, -5.0, 7.0], [2.0, 0.0, 6.0], [-3.0, 4.0, -1.0]], [[-6.0, -3.0, 3.0], [6.0, 5.0, -3.0], [0.0, -2.0, 1.0], [0.0, -1.0, 5.0], [-5.0, 3.0, -5.0]]])
      arg1=numpy.array([[-5.0, 2.0, -5.0], [4.0, -3.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[3.0, 17.0], [22.0, 6.0], [1.0, -50.0], [-19.0, 11.0], [45.0, -18.0]], [[28.0, -37.0], [42.0, -24.0], [69.0, -4.0], [-16.0, 62.0], [-18.0, 15.0]], [[-30.0, -19.0], [-55.0, -10.0], [-55.0, -12.0], [-40.0, -22.0], [28.0, -19.0]], [[9.0, -30.0], [-5.0, 24.0], [-9.0, 1.0], [-27.0, -22.0], [56.0, -4.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank3_offset2(self):
      arg0=numpy.array([[[[6.0, 0.0, -1.0], [-3.0, -6.0, -2.0]], [[0.0, -7.0, 3.0], [4.0, -3.0, 7.0]], [[-1.0, -5.0, -2.0], [3.0, 5.0, -5.0]], [[0.0, -3.0, -3.0], [-7.0, 2.0, 1.0]], [[-1.0, -7.0, 5.0], [1.0, 7.0, 3.0]]], [[[-4.0, -4.0, 0.0], [3.0, 2.0, 3.0]], [[-1.0, -5.0, 0.0], [2.0, 6.0, 7.0]], [[-2.0, 4.0, 4.0], [7.0, 1.0, 4.0]], [[3.0, 7.0, -4.0], [5.0, 0.0, -4.0]], [[6.0, -2.0, 4.0], [0.0, 4.0, -1.0]]], [[[6.0, -7.0, -7.0], [-4.0, -5.0, -1.0]], [[-4.0, -3.0, -1.0], [-1.0, 0.0, 4.0]], [[-6.0, 3.0, 0.0], [-7.0, -2.0, -6.0]], [[-7.0, 1.0, 7.0], [5.0, 1.0, 1.0]], [[1.0, 0.0, 2.0], [-3.0, -2.0, -7.0]]], [[[0.0, 2.0, 4.0], [-6.0, 1.0, -7.0]], [[2.0, -1.0, -2.0], [5.0, 4.0, 6.0]], [[5.0, 1.0, -1.0], [0.0, 4.0, 0.0]], [[3.0, -7.0, -2.0], [5.0, 6.0, 4.0]], [[5.0, -5.0, 4.0], [0.0, -7.0, 7.0]]]])
      arg1=numpy.array([[[0.0, -3.0, 1.0], [0.0, 5.0, -3.0]], [[0.0, 5.0, -2.0], [0.0, -7.0, -4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[-25.0, 52.0], [-12.0, -48.0], [53.0, -36.0], [13.0, -27.0], [52.0, -106.0]], [[13.0, -46.0], [24.0, -95.0], [-15.0, -11.0], [-13.0, 59.0], [33.0, -42.0]], [[-8.0, 18.0], [-4.0, -29.0], [-1.0, 53.0], [6.0, -20.0], [13.0, 38.0]], [[24.0, 23.0], [3.0, -53.0], [16.0, -21.0], [37.0, -89.0], [-37.0, -12.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_array_rank2_offset0(self):
      arg0=numpy.array([[4.0, 0.0, -5.0, -3.0, 2.0], [-2.0, 3.0, -1.0, 0.0, -5.0], [-4.0, -7.0, -6.0, 2.0, 5.0], [-3.0, 1.0, -3.0, -5.0, 5.0]])
      arg1=numpy.array([[-5.0, 5.0, 0.0, -5.0, -1.0], [-4.0, -2.0, -4.0, -6.0, 6.0], [-1.0, 7.0, 0.0, 7.0, 7.0], [7.0, 0.0, -5.0, 3.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-20.0, 20.0, 0.0, -20.0, -4.0], [-16.0, -8.0, -16.0, -24.0, 24.0], [-4.0, 28.0, 0.0, 28.0, 28.0], [28.0, 0.0, -20.0, 12.0, -20.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[25.0, -25.0, 0.0, 25.0, 5.0], [20.0, 10.0, 20.0, 30.0, -30.0], [5.0, -35.0, 0.0, -35.0, -35.0], [-35.0, 0.0, 25.0, -15.0, 25.0]], [[15.0, -15.0, 0.0, 15.0, 3.0], [12.0, 6.0, 12.0, 18.0, -18.0], [3.0, -21.0, 0.0, -21.0, -21.0], [-21.0, 0.0, 15.0, -9.0, 15.0]], [[-10.0, 10.0, 0.0, -10.0, -2.0], [-8.0, -4.0, -8.0, -12.0, 12.0], [-2.0, 14.0, 0.0, 14.0, 14.0], [14.0, 0.0, -10.0, 6.0, -10.0]]], [[[10.0, -10.0, 0.0, 10.0, 2.0], [8.0, 4.0, 8.0, 12.0, -12.0], [2.0, -14.0, 0.0, -14.0, -14.0], [-14.0, 0.0, 10.0, -6.0, 10.0]], [[-15.0, 15.0, 0.0, -15.0, -3.0], [-12.0, -6.0, -12.0, -18.0, 18.0], [-3.0, 21.0, 0.0, 21.0, 21.0], [21.0, 0.0, -15.0, 9.0, -15.0]], [[5.0, -5.0, 0.0, 5.0, 1.0], [4.0, 2.0, 4.0, 6.0, -6.0], [1.0, -7.0, 0.0, -7.0, -7.0], [-7.0, 0.0, 5.0, -3.0, 5.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[25.0, -25.0, 0.0, 25.0, 5.0], [20.0, 10.0, 20.0, 30.0, -30.0], [5.0, -35.0, 0.0, -35.0, -35.0], [-35.0, 0.0, 25.0, -15.0, 25.0]]], [[[20.0, -20.0, 0.0, 20.0, 4.0], [16.0, 8.0, 16.0, 24.0, -24.0], [4.0, -28.0, 0.0, -28.0, -28.0], [-28.0, 0.0, 20.0, -12.0, 20.0]], [[35.0, -35.0, 0.0, 35.0, 7.0], [28.0, 14.0, 28.0, 42.0, -42.0], [7.0, -49.0, 0.0, -49.0, -49.0], [-49.0, 0.0, 35.0, -21.0, 35.0]], [[30.0, -30.0, 0.0, 30.0, 6.0], [24.0, 12.0, 24.0, 36.0, -36.0], [6.0, -42.0, 0.0, -42.0, -42.0], [-42.0, 0.0, 30.0, -18.0, 30.0]], [[-10.0, 10.0, 0.0, -10.0, -2.0], [-8.0, -4.0, -8.0, -12.0, 12.0], [-2.0, 14.0, 0.0, 14.0, 14.0], [14.0, 0.0, -10.0, 6.0, -10.0]], [[-25.0, 25.0, 0.0, -25.0, -5.0], [-20.0, -10.0, -20.0, -30.0, 30.0], [-5.0, 35.0, 0.0, 35.0, 35.0], [35.0, 0.0, -25.0, 15.0, -25.0]]], [[[15.0, -15.0, 0.0, 15.0, 3.0], [12.0, 6.0, 12.0, 18.0, -18.0], [3.0, -21.0, 0.0, -21.0, -21.0], [-21.0, 0.0, 15.0, -9.0, 15.0]], [[-5.0, 5.0, 0.0, -5.0, -1.0], [-4.0, -2.0, -4.0, -6.0, 6.0], [-1.0, 7.0, 0.0, 7.0, 7.0], [7.0, 0.0, -5.0, 3.0, -5.0]], [[15.0, -15.0, 0.0, 15.0, 3.0], [12.0, 6.0, 12.0, 18.0, -18.0], [3.0, -21.0, 0.0, -21.0, -21.0], [-21.0, 0.0, 15.0, -9.0, 15.0]], [[25.0, -25.0, 0.0, 25.0, 5.0], [20.0, 10.0, 20.0, 30.0, -30.0], [5.0, -35.0, 0.0, -35.0, -35.0], [-35.0, 0.0, 25.0, -15.0, 25.0]], [[-25.0, 25.0, 0.0, -25.0, -5.0], [-20.0, -10.0, -20.0, -30.0, 30.0], [-5.0, 35.0, 0.0, 35.0, 35.0], [35.0, 0.0, -25.0, 15.0, -25.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank3_offset1(self):
      arg0=numpy.array([[[-6.0, -4.0, 4.0], [6.0, -2.0, -7.0], [4.0, -4.0, -6.0], [7.0, 1.0, -7.0], [5.0, -4.0, -3.0]], [[4.0, -1.0, 6.0], [-1.0, 6.0, 0.0], [6.0, -2.0, 3.0], [7.0, 6.0, -7.0], [2.0, -3.0, -1.0]], [[0.0, -5.0, 3.0], [-7.0, -5.0, -3.0], [7.0, -7.0, 7.0], [-3.0, 0.0, 0.0], [5.0, 2.0, 7.0]], [[-1.0, -6.0, 7.0], [0.0, 4.0, 2.0], [7.0, -4.0, 6.0], [-4.0, -4.0, -6.0], [-6.0, 7.0, -3.0]]])
      arg1=numpy.array([[[-6.0, -3.0, -5.0], [-7.0, 5.0, 6.0], [-4.0, 2.0, 0.0], [-7.0, 7.0, 4.0], [7.0, -5.0, 6.0]], [[3.0, 2.0, -2.0], [0.0, 1.0, -6.0], [-4.0, 2.0, 5.0], [-2.0, 0.0, 0.0], [-3.0, -5.0, 6.0]], [[3.0, 0.0, 0.0], [0.0, 4.0, 1.0], [-2.0, 5.0, -1.0], [2.0, 2.0, -7.0], [-2.0, 7.0, -6.0]], [[-5.0, -4.0, 4.0], [3.0, 7.0, -5.0], [5.0, -3.0, 6.0], [-3.0, 6.0, 6.0], [-1.0, 0.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[28.0, 46.0, 16.0, 30.0, 2.0], [-34.0, -28.0, 36.0, 12.0, 62.0], [-18.0, -12.0, -12.0, -48.0, -40.0], [62.0, -66.0, 6.0, 18.0, 14.0]], [[5.0, -94.0, -28.0, -84.0, 10.0], [28.0, 40.0, -63.0, -12.0, -50.0], [18.0, -15.0, -15.0, 57.0, 16.0], [-50.0, 39.0, -6.0, -72.0, -20.0]], [[18.0, -84.0, -24.0, -80.0, 12.0], [16.0, 32.0, -54.0, -8.0, -28.0], [12.0, -22.0, -22.0, 42.0, 0.0], [-28.0, 14.0, -4.0, -72.0, -16.0]], [[-10.0, -86.0, -26.0, -70.0, 2.0], [37.0, 43.0, -61.0, -14.0, -68.0], [21.0, -3.0, -2.0, 65.0, 35.0], [-67.0, 63.0, -10.0, -57.0, -21.0]], [[-3.0, -73.0, -28.0, -75.0, 37.0], [13.0, 14.0, -43.0, -10.0, -13.0], [15.0, -19.0, -27.0, 23.0, -20.0], [-21.0, 2.0, 19.0, -57.0, -11.0]]], [[[-51.0, 3.0, -18.0, -11.0, 69.0], [-2.0, -37.0, 12.0, -8.0, 29.0], [12.0, 2.0, -19.0, -36.0, -51.0], [8.0, -25.0, 59.0, 18.0, 8.0]], [[-12.0, 37.0, 16.0, 49.0, -37.0], [9.0, 6.0, 16.0, 2.0, -27.0], [-3.0, 24.0, 32.0, 10.0, 44.0], [-19.0, 39.0, -23.0, 39.0, 1.0]], [[-45.0, -34.0, -28.0, -44.0, 70.0], [8.0, -20.0, -13.0, -12.0, 10.0], [18.0, -5.0, -25.0, -13.0, -44.0], [-10.0, -11.0, 54.0, -12.0, 0.0]], [[-25.0, -61.0, -16.0, -35.0, -23.0], [47.0, 48.0, -51.0, -14.0, -93.0], [21.0, 17.0, 23.0, 75.0, 70.0], [-87.0, 98.0, -25.0, -27.0, -21.0]], [[2.0, -35.0, -14.0, -39.0, 23.0], [2.0, 3.0, -19.0, -4.0, 3.0], [6.0, -13.0, -18.0, 5.0, -19.0], [-2.0, -10.0, 13.0, -30.0, -4.0]]], [[[0.0, -7.0, -10.0, -23.0, 43.0], [-16.0, -23.0, 5.0, 0.0, 43.0], [0.0, -17.0, -28.0, -31.0, -53.0], [32.0, -50.0, 33.0, -12.0, 6.0]], [[72.0, 6.0, 18.0, 2.0, -42.0], [-25.0, 13.0, 3.0, 14.0, 28.0], [-21.0, -23.0, -8.0, -3.0, -3.0], [43.0, -41.0, -38.0, -27.0, 1.0]], [[-56.0, -42.0, -42.0, -70.0, 126.0], [-7.0, -49.0, -7.0, -14.0, 56.0], [21.0, -21.0, -56.0, -49.0, -105.0], [21.0, -63.0, 98.0, -21.0, 7.0]], [[18.0, 21.0, 12.0, 21.0, -21.0], [-9.0, 0.0, 12.0, 6.0, 9.0], [-9.0, 0.0, 6.0, -6.0, 6.0], [15.0, -9.0, -15.0, 9.0, 3.0]], [[-71.0, 17.0, -16.0, 7.0, 67.0], [5.0, -40.0, 19.0, -10.0, 17.0], [15.0, 15.0, -7.0, -35.0, -38.0], [-5.0, -6.0, 61.0, 39.0, 9.0]]], [[[-11.0, 19.0, -8.0, -7.0, 65.0], [-29.0, -48.0, 27.0, 2.0, 75.0], [-3.0, -17.0, -35.0, -63.0, -82.0], [57.0, -80.0, 55.0, 9.0, 15.0]], [[-22.0, 32.0, 8.0, 36.0, -8.0], [4.0, -8.0, 18.0, 0.0, -8.0], [0.0, 18.0, 18.0, -6.0, 16.0], [-8.0, 18.0, 0.0, 36.0, 4.0]], [[-60.0, -33.0, -36.0, -53.0, 105.0], [1.0, -40.0, -6.0, -14.0, 35.0], [21.0, -10.0, -40.0, -36.0, -78.0], [5.0, -37.0, 83.0, -9.0, 5.0]], [[66.0, -28.0, 8.0, -24.0, -44.0], [-8.0, 32.0, -22.0, 8.0, -4.0], [-12.0, -22.0, -6.0, 26.0, 16.0], [12.0, -10.0, -44.0, -48.0, -8.0]], [[30.0, 59.0, 38.0, 79.0, -95.0], [2.0, 25.0, 23.0, 12.0, -35.0], [-18.0, 25.0, 50.0, 23.0, 79.0], [-10.0, 46.0, -69.0, 42.0, 0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank4_offset2(self):
      arg0=numpy.array([[[[-2.0, 5.0, 7.0], [5.0, -7.0, 2.0]], [[-3.0, -1.0, 6.0], [-3.0, 4.0, -5.0]], [[3.0, -3.0, -7.0], [-6.0, -2.0, -1.0]], [[7.0, -2.0, 3.0], [0.0, 0.0, 5.0]], [[6.0, 4.0, -1.0], [-2.0, -2.0, -5.0]]], [[[6.0, -2.0, 3.0], [0.0, 1.0, -5.0]], [[-3.0, -2.0, 7.0], [-6.0, -1.0, -7.0]], [[-6.0, -1.0, -5.0], [7.0, -6.0, 6.0]], [[0.0, 7.0, -5.0], [5.0, -4.0, 4.0]], [[-4.0, -6.0, -5.0], [-3.0, 6.0, -5.0]]], [[[5.0, 6.0, -4.0], [-3.0, 0.0, 0.0]], [[0.0, 3.0, -3.0], [-1.0, 3.0, 0.0]], [[-4.0, -3.0, 5.0], [-7.0, -7.0, -1.0]], [[7.0, -1.0, 1.0], [2.0, -7.0, -3.0]], [[0.0, 5.0, -7.0], [-1.0, -1.0, 0.0]]], [[[-1.0, 7.0, 6.0], [0.0, 1.0, -1.0]], [[-3.0, 7.0, 3.0], [0.0, 7.0, 0.0]], [[6.0, 2.0, 0.0], [0.0, 3.0, -1.0]], [[5.0, 7.0, -3.0], [-2.0, -6.0, -2.0]], [[7.0, 0.0, -3.0], [-4.0, 4.0, -2.0]]]])
      arg1=numpy.array([[[[-2.0, 0.0, -2.0], [-6.0, 7.0, 1.0]], [[-7.0, 6.0, 0.0], [7.0, 0.0, 0.0]], [[-2.0, -7.0, -5.0], [-5.0, -7.0, -3.0]], [[0.0, 5.0, -7.0], [-6.0, -6.0, 6.0]], [[5.0, 0.0, -6.0], [4.0, 6.0, 6.0]]], [[[-1.0, 0.0, -5.0], [-6.0, 4.0, -2.0]], [[-1.0, 2.0, 1.0], [5.0, 7.0, -7.0]], [[6.0, -4.0, 4.0], [-7.0, 3.0, 2.0]], [[0.0, -6.0, 6.0], [0.0, -7.0, -5.0]], [[-2.0, 4.0, -5.0], [-6.0, 5.0, 2.0]]], [[[5.0, -2.0, -4.0], [3.0, 6.0, 0.0]], [[-1.0, -5.0, 0.0], [-4.0, -3.0, 0.0]], [[-5.0, -3.0, -7.0], [-1.0, -3.0, 4.0]], [[0.0, 1.0, 1.0], [-6.0, 1.0, 3.0]], [[-2.0, 7.0, -7.0], [6.0, 3.0, 6.0]]], [[[-5.0, -4.0, 0.0], [-3.0, -6.0, 1.0]], [[4.0, 0.0, 7.0], [-2.0, 7.0, -7.0]], [[-4.0, 1.0, 0.0], [-6.0, 7.0, 7.0]], [[4.0, 0.0, -4.0], [3.0, -1.0, 2.0]], [[0.0, 6.0, 6.0], [-4.0, -1.0, 2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=numpy.array([[[[-87.0, 79.0, -48.0, 0.0, -62.0], [-95.0, -19.0, -56.0, 51.0, -72.0], [-75.0, -22.0, -30.0, -19.0, 11.0], [19.0, -32.0, -52.0, -10.0, 63.0]], [[35.0, -6.0, -15.0, -83.0, -69.0], [17.0, 55.0, 33.0, 39.0, 0.0], [-22.0, 8.0, -53.0, 12.0, -79.0], [-1.0, 99.0, 22.0, -59.0, 28.0]], [[29.0, -81.0, 97.0, 76.0, 15.0], [62.0, -53.0, 36.0, -5.0, 41.0], [19.0, 42.0, 51.0, 21.0, -26.0], [26.0, -32.0, 0.0, 22.0, -36.0]], [[-15.0, -61.0, -30.0, -1.0, 47.0], [-32.0, -43.0, 72.0, 5.0, -27.0], [27.0, 3.0, -30.0, 16.0, -19.0], [-22.0, 14.0, 5.0, 26.0, 16.0]], [[-17.0, -32.0, 4.0, 21.0, -14.0], [13.0, 12.0, 14.0, 9.0, 1.0], [8.0, -12.0, -47.0, -2.0, -25.0], [-33.0, 42.0, -57.0, 14.0, 18.0]]], [[[-16.0, -54.0, -5.0, -67.0, -12.0], [-7.0, 35.0, 49.0, 48.0, -40.0], [28.0, 1.0, -68.0, -13.0, -74.0], [-33.0, 87.0, -54.0, 1.0, -5.0]], [[14.0, -33.0, 43.0, -59.0, -129.0], [14.0, 18.0, 43.0, 96.0, -20.0], [-63.0, 40.0, -47.0, 19.0, -138.0], [40.0, 91.0, -10.0, -71.0, 41.0]], [[-56.0, 85.0, 33.0, 60.0, 28.0], [-47.0, -50.0, -107.0, -12.0, -27.0], [-23.0, 1.0, 103.0, -36.0, 100.0], [55.0, -157.0, -19.0, 35.0, -46.0]], [[-44.0, 77.0, -33.0, 88.0, 50.0], [-29.0, -22.0, -87.0, -64.0, 11.0], [-3.0, -43.0, 37.0, -20.0, 126.0], [-15.0, -101.0, -23.0, 47.0, 4.0]], [[73.0, -29.0, 63.0, -43.0, 4.0], [81.0, 49.0, 9.0, -11.0, 47.0], [39.0, 28.0, 38.0, -2.0, -29.0], [12.0, 32.0, 35.0, -21.0, -70.0]]], [[[16.0, -20.0, -17.0, 76.0, 37.0], [33.0, -12.0, 11.0, -60.0, 52.0], [20.0, -23.0, -12.0, 20.0, 42.0], [-40.0, -2.0, 4.0, 27.0, 24.0]], [[33.0, 11.0, -22.0, 24.0, 32.0], [33.0, 19.0, -8.0, -57.0, 48.0], [21.0, -20.0, 4.0, 9.0, 45.0], [-27.0, 2.0, 30.0, 6.0, 1.0]], [[-10.0, -39.0, 91.0, 28.0, -126.0], [-5.0, -74.0, 34.0, 102.0, -24.0], [-97.0, 68.0, 18.0, 34.0, -117.0], [94.0, -9.0, -1.0, -52.0, 45.0]], [[-80.0, -41.0, 36.0, 0.0, -23.0], [-46.0, -26.0, 9.0, 76.0, -76.0], [-3.0, 11.0, -32.0, -28.0, -55.0], [2.0, 3.0, -111.0, 31.0, -7.0]], [[13.0, 23.0, 12.0, 86.0, 32.0], [37.0, -9.0, -44.0, -65.0, 56.0], [9.0, -18.0, 38.0, 3.0, 75.0], [-11.0, -54.0, 4.0, 26.0, -7.0]]], [[[-4.0, 49.0, -81.0, -19.0, -41.0], [-23.0, 35.0, -9.0, -8.0, 3.0], [-37.0, -37.0, -65.0, 11.0, 6.0], [-30.0, 52.0, 11.0, -31.0, 75.0]], [[49.0, 63.0, -107.0, -28.0, 9.0], [16.0, 69.0, -13.0, -73.0, 54.0], [1.0, -53.0, -48.0, 17.0, 55.0], [-55.0, 58.0, 68.0, -31.0, 53.0]], [[8.0, -30.0, -44.0, -14.0, 42.0], [8.0, 26.0, 35.0, -28.0, 9.0], [44.0, -25.0, -49.0, 2.0, 5.0], [-57.0, 52.0, -8.0, 19.0, 7.0]], [[-36.0, -7.0, 14.0, 92.0, -13.0], [2.0, -32.0, -18.0, -8.0, 11.0], [-19.0, -14.0, -13.0, 4.0, 18.0], [-13.0, -25.0, -57.0, 28.0, 34.0]], [[42.0, -77.0, -1.0, 9.0, 49.0], [52.0, 12.0, 66.0, -36.0, 41.0], [59.0, -3.0, -30.0, 19.0, -17.0], [-49.0, 57.0, 10.0, 20.0, -10.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank0_offset0(self):
      arg0=numpy.array([[-3.0, -6.0, 2.0, -2.0, -4.0], [7.0, 1.0, 2.0, -7.0, 6.0], [4.0, 0.0, -3.0, -3.0, 6.0], [0.0, 3.0, 3.0, -6.0, 4.0]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-3.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[9.0, 18.0, -6.0, 6.0, 12.0], [-21.0, -3.0, -6.0, 21.0, -18.0], [-12.0, -0.0, 9.0, 9.0, -18.0], [-0.0, -9.0, -9.0, 18.0, -12.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[6.0, 0.0, -4.0], [7.0, -7.0, 0.0], [1.0, -5.0, 0.0], [7.0, -7.0, 5.0], [5.0, 4.0, -3.0]], [[5.0, -4.0, 2.0], [-2.0, 4.0, 6.0], [2.0, -5.0, 0.0], [0.0, -6.0, -1.0], [2.0, -2.0, 2.0]], [[6.0, 6.0, 4.0], [6.0, -7.0, 6.0], [-6.0, -3.0, 0.0], [4.0, -4.0, 7.0], [7.0, 1.0, -6.0]], [[-5.0, 6.0, 4.0], [0.0, -3.0, -2.0], [0.0, -2.0, -5.0], [3.0, 0.0, 3.0], [-2.0, -7.0, 1.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([2.0, -1.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[12.0, 21.0, 7.0, 21.0, 6.0], [14.0, -8.0, 9.0, 6.0, 6.0], [6.0, 19.0, -9.0, 12.0, 13.0], [-16.0, 3.0, 2.0, 6.0, 3.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank2_offset2(self):
      arg0=numpy.array([[[[-1.0, -1.0, -3.0], [7.0, -6.0, -3.0]], [[-5.0, 6.0, -3.0], [0.0, 3.0, 5.0]], [[0.0, -7.0, -2.0], [6.0, -5.0, -2.0]], [[2.0, 7.0, -5.0], [-4.0, 4.0, -1.0]], [[4.0, 5.0, 5.0], [4.0, 6.0, 6.0]]], [[[-7.0, 0.0, -3.0], [-2.0, 1.0, 4.0]], [[2.0, 4.0, 3.0], [-1.0, 4.0, 1.0]], [[-1.0, -4.0, -6.0], [6.0, 3.0, 7.0]], [[-1.0, 2.0, 0.0], [-7.0, 1.0, 4.0]], [[0.0, -5.0, 0.0], [0.0, 0.0, 0.0]]], [[[7.0, 3.0, 0.0], [6.0, 7.0, -4.0]], [[-1.0, -6.0, -1.0], [4.0, 4.0, 1.0]], [[1.0, -5.0, -5.0], [7.0, -2.0, 4.0]], [[-7.0, 4.0, 5.0], [0.0, 5.0, -2.0]], [[-3.0, 0.0, 1.0], [2.0, -3.0, -6.0]]], [[[-1.0, 2.0, 0.0], [2.0, -7.0, -7.0]], [[-4.0, -1.0, 7.0], [-6.0, 1.0, 4.0]], [[4.0, 4.0, 4.0], [-7.0, -1.0, 7.0]], [[-5.0, 3.0, -6.0], [-6.0, 4.0, 6.0]], [[0.0, -4.0, 0.0], [-2.0, 3.0, 2.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[1.0, 0.0, 7.0], [-4.0, -7.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[4.0, -67.0, 5.0, -41.0, -43.0], [-43.0, -5.0, -116.0, 4.0, 0.0], [-50.0, -56.0, -64.0, 1.0, 41.0], [68.0, 46.0, 39.0, -75.0, -21.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank1_offset0(self):
      arg0=numpy.array([[-3.0, -5.0, -4.0, -3.0, 3.0], [-3.0, 5.0, 0.0, -4.0, -5.0], [-5.0, 6.0, 5.0, 3.0, 2.0], [-7.0, 0.0, -5.0, 3.0, -7.0]])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-2.0, -1.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[6.0, 3.0], [10.0, 5.0], [8.0, 4.0], [6.0, 3.0], [-6.0, -3.0]], [[6.0, 3.0], [-10.0, -5.0], [0.0, 0.0], [8.0, 4.0], [10.0, 5.0]], [[10.0, 5.0], [-12.0, -6.0], [-10.0, -5.0], [-6.0, -3.0], [-4.0, -2.0]], [[14.0, 7.0], [0.0, 0.0], [10.0, 5.0], [-6.0, -3.0], [14.0, 7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[7.0, -5.0, -7.0], [-2.0, 4.0, -6.0], [-6.0, 4.0, 3.0], [7.0, 0.0, -3.0], [6.0, -3.0, 1.0]], [[-6.0, 2.0, -5.0], [1.0, -4.0, -6.0], [0.0, -1.0, -2.0], [5.0, -7.0, 1.0], [7.0, 7.0, -2.0]], [[-5.0, 1.0, -6.0], [6.0, 2.0, -5.0], [4.0, 3.0, -4.0], [6.0, -7.0, -2.0], [0.0, -7.0, 3.0]], [[0.0, 2.0, 7.0], [-3.0, -1.0, 5.0], [0.0, 7.0, -2.0], [-4.0, 0.0, 6.0], [-3.0, 3.0, -7.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-2.0, -2.0, 0.0], [0.0, 1.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[-4.0, -54.0], [-4.0, -38.0], [4.0, 25.0], [-14.0, -21.0], [-6.0, 4.0]], [[8.0, -33.0], [6.0, -46.0], [2.0, -15.0], [4.0, 0.0], [-28.0, -7.0]], [[8.0, -41.0], [-16.0, -33.0], [-14.0, -25.0], [2.0, -21.0], [14.0, 14.0]], [[-4.0, 51.0], [8.0, 34.0], [-14.0, -7.0], [8.0, 42.0], [0.0, -46.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank3_offset2(self):
      arg0=numpy.array([[[[-1.0, -2.0, -1.0], [-7.0, 2.0, -6.0]], [[7.0, -6.0, 7.0], [-5.0, 2.0, -2.0]], [[-3.0, 5.0, 4.0], [7.0, 5.0, -4.0]], [[3.0, -7.0, -2.0], [7.0, 2.0, -3.0]], [[4.0, 3.0, -2.0], [-1.0, -3.0, 3.0]]], [[[2.0, 4.0, 1.0], [2.0, 4.0, 3.0]], [[-6.0, -3.0, -7.0], [5.0, -5.0, 3.0]], [[-5.0, -5.0, 2.0], [-2.0, -7.0, 6.0]], [[-3.0, 5.0, -7.0], [6.0, 6.0, 4.0]], [[7.0, 5.0, -6.0], [0.0, 0.0, 6.0]]], [[[-1.0, -6.0, 4.0], [-2.0, 2.0, -3.0]], [[-6.0, 4.0, -3.0], [-3.0, -2.0, 0.0]], [[0.0, -1.0, 2.0], [5.0, -4.0, 1.0]], [[5.0, -3.0, 3.0], [2.0, -7.0, 2.0]], [[-4.0, -7.0, -1.0], [0.0, 4.0, -1.0]]], [[[7.0, -3.0, 1.0], [3.0, 0.0, 6.0]], [[-4.0, 0.0, 3.0], [3.0, 0.0, -5.0]], [[3.0, 6.0, 4.0], [-6.0, 0.0, 1.0]], [[4.0, 1.0, 1.0], [0.0, 5.0, -1.0]], [[-2.0, -4.0, 3.0], [-7.0, 0.0, -7.0]]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-4.0, 6.0, -6.0], [0.0, -5.0, -2.0]], [[-4.0, -7.0, -3.0], [-7.0, 4.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[0.0, 66.0], [-112.0, 32.0], [1.0, -72.0], [-46.0, -4.0], [23.0, -30.0]], [[-16.0, -31.0], [67.0, 17.0], [1.0, 47.0], [46.0, -12.0], [26.0, -33.0]], [[-60.0, 50.0], [76.0, 18.0], [0.0, -48.0], [-25.0, -46.0], [-38.0, 82.0]], [[-64.0, -19.0], [8.0, -24.0], [-2.0, -22.0], [-39.0, -8.0], [-20.0, 62.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_Symbol_rank2_offset0(self):
      arg0=numpy.array([[0.0, 2.0, -2.0, 0.0, 5.0], [6.0, -5.0, 2.0, -3.0, -1.0], [7.0, -5.0, 6.0, 1.0, 1.0], [0.0, 1.0, 7.0, -5.0, -2.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-6.0, 2.0, 0.0, 6.0, -5.0], [-3.0, -7.0, 3.0, 4.0, -5.0], [3.0, 4.0, 0.0, 4.0, 3.0], [7.0, -6.0, 5.0, 2.0, 2.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, 4.0, 0.0, 12.0, -10.0], [-6.0, -14.0, 6.0, 8.0, -10.0], [6.0, 8.0, 0.0, 8.0, 6.0], [14.0, -12.0, 10.0, 4.0, 4.0]], [[12.0, -4.0, 0.0, -12.0, 10.0], [6.0, 14.0, -6.0, -8.0, 10.0], [-6.0, -8.0, 0.0, -8.0, -6.0], [-14.0, 12.0, -10.0, -4.0, -4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-30.0, 10.0, 0.0, 30.0, -25.0], [-15.0, -35.0, 15.0, 20.0, -25.0], [15.0, 20.0, 0.0, 20.0, 15.0], [35.0, -30.0, 25.0, 10.0, 10.0]]], [[[-36.0, 12.0, 0.0, 36.0, -30.0], [-18.0, -42.0, 18.0, 24.0, -30.0], [18.0, 24.0, 0.0, 24.0, 18.0], [42.0, -36.0, 30.0, 12.0, 12.0]], [[30.0, -10.0, 0.0, -30.0, 25.0], [15.0, 35.0, -15.0, -20.0, 25.0], [-15.0, -20.0, 0.0, -20.0, -15.0], [-35.0, 30.0, -25.0, -10.0, -10.0]], [[-12.0, 4.0, 0.0, 12.0, -10.0], [-6.0, -14.0, 6.0, 8.0, -10.0], [6.0, 8.0, 0.0, 8.0, 6.0], [14.0, -12.0, 10.0, 4.0, 4.0]], [[18.0, -6.0, 0.0, -18.0, 15.0], [9.0, 21.0, -9.0, -12.0, 15.0], [-9.0, -12.0, 0.0, -12.0, -9.0], [-21.0, 18.0, -15.0, -6.0, -6.0]], [[6.0, -2.0, 0.0, -6.0, 5.0], [3.0, 7.0, -3.0, -4.0, 5.0], [-3.0, -4.0, 0.0, -4.0, -3.0], [-7.0, 6.0, -5.0, -2.0, -2.0]]], [[[-42.0, 14.0, 0.0, 42.0, -35.0], [-21.0, -49.0, 21.0, 28.0, -35.0], [21.0, 28.0, 0.0, 28.0, 21.0], [49.0, -42.0, 35.0, 14.0, 14.0]], [[30.0, -10.0, 0.0, -30.0, 25.0], [15.0, 35.0, -15.0, -20.0, 25.0], [-15.0, -20.0, 0.0, -20.0, -15.0], [-35.0, 30.0, -25.0, -10.0, -10.0]], [[-36.0, 12.0, 0.0, 36.0, -30.0], [-18.0, -42.0, 18.0, 24.0, -30.0], [18.0, 24.0, 0.0, 24.0, 18.0], [42.0, -36.0, 30.0, 12.0, 12.0]], [[-6.0, 2.0, 0.0, 6.0, -5.0], [-3.0, -7.0, 3.0, 4.0, -5.0], [3.0, 4.0, 0.0, 4.0, 3.0], [7.0, -6.0, 5.0, 2.0, 2.0]], [[-6.0, 2.0, 0.0, 6.0, -5.0], [-3.0, -7.0, 3.0, 4.0, -5.0], [3.0, 4.0, 0.0, 4.0, 3.0], [7.0, -6.0, 5.0, 2.0, 2.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 2.0, 0.0, 6.0, -5.0], [-3.0, -7.0, 3.0, 4.0, -5.0], [3.0, 4.0, 0.0, 4.0, 3.0], [7.0, -6.0, 5.0, 2.0, 2.0]], [[-42.0, 14.0, 0.0, 42.0, -35.0], [-21.0, -49.0, 21.0, 28.0, -35.0], [21.0, 28.0, 0.0, 28.0, 21.0], [49.0, -42.0, 35.0, 14.0, 14.0]], [[30.0, -10.0, 0.0, -30.0, 25.0], [15.0, 35.0, -15.0, -20.0, 25.0], [-15.0, -20.0, 0.0, -20.0, -15.0], [-35.0, 30.0, -25.0, -10.0, -10.0]], [[12.0, -4.0, 0.0, -12.0, 10.0], [6.0, 14.0, -6.0, -8.0, 10.0], [-6.0, -8.0, 0.0, -8.0, -6.0], [-14.0, 12.0, -10.0, -4.0, -4.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank3_offset1(self):
      arg0=numpy.array([[[-3.0, -4.0, -2.0], [-4.0, 1.0, 6.0], [-7.0, -4.0, -6.0], [0.0, 1.0, -6.0], [-7.0, 0.0, 2.0]], [[-7.0, 1.0, 0.0], [-3.0, 5.0, -4.0], [4.0, 7.0, 2.0], [-6.0, 0.0, -7.0], [-1.0, 4.0, -4.0]], [[6.0, -1.0, -3.0], [0.0, -4.0, 7.0], [-4.0, 1.0, -4.0], [-6.0, -7.0, 5.0], [2.0, -5.0, 3.0]], [[-2.0, 5.0, 0.0], [0.0, 5.0, -4.0], [-5.0, -6.0, 0.0], [1.0, 2.0, 4.0], [-5.0, -3.0, -2.0]]])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[1.0, 2.0, 1.0], [3.0, -4.0, 2.0], [7.0, -5.0, 6.0], [5.0, 0.0, 7.0], [7.0, -2.0, 0.0]], [[-7.0, 2.0, -3.0], [-7.0, 5.0, 4.0], [-1.0, 4.0, -7.0], [2.0, 3.0, -5.0], [0.0, -6.0, -7.0]], [[-7.0, -2.0, -2.0], [2.0, 5.0, 6.0], [-2.0, -2.0, -1.0], [-3.0, -3.0, 0.0], [-3.0, 5.0, 7.0]], [[-7.0, -3.0, 3.0], [0.0, -6.0, -7.0], [4.0, -7.0, 4.0], [4.0, 0.0, -1.0], [7.0, -5.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-13.0, 3.0, -13.0, -29.0, -13.0], [19.0, -7.0, 1.0, -8.0, 38.0], [33.0, -38.0, 16.0, 21.0, -25.0], [27.0, 38.0, 8.0, -10.0, -15.0]], [[4.0, -4.0, 3.0, 22.0, -30.0], [12.0, 57.0, -34.0, -35.0, -48.0], [14.0, 33.0, 0.0, 9.0, 59.0], [43.0, -48.0, 1.0, -22.0, 9.0]], [[-21.0, -17.0, -65.0, -77.0, -41.0], [59.0, 5.0, 33.0, 4.0, 66.0], [69.0, -70.0, 28.0, 33.0, -41.0], [43.0, 66.0, -24.0, -22.0, -71.0]], [[-4.0, -16.0, -41.0, -42.0, -2.0], [20.0, -19.0, 46.0, 33.0, 36.0], [10.0, -31.0, 4.0, -3.0, -37.0], [-21.0, 36.0, -31.0, 6.0, -47.0]], [[-5.0, -17.0, -37.0, -21.0, -49.0], [43.0, 57.0, -7.0, -24.0, -14.0], [45.0, -2.0, 12.0, 21.0, 35.0], [55.0, -14.0, -20.0, -30.0, -35.0]]], [[[-5.0, -25.0, -54.0, -35.0, -51.0], [51.0, 54.0, 11.0, -11.0, -6.0], [47.0, -9.0, 12.0, 18.0, 26.0], [46.0, -6.0, -35.0, -28.0, -54.0]], [[3.0, -37.0, -70.0, -43.0, -31.0], [43.0, 30.0, 51.0, 29.0, -2.0], [19.0, -5.0, 0.0, -6.0, 6.0], [-6.0, -2.0, -63.0, -8.0, -74.0]], [[20.0, -12.0, 5.0, 34.0, 14.0], [-20.0, 15.0, 10.0, 19.0, -56.0], [-46.0, 55.0, -24.0, -33.0, 37.0], [-43.0, -56.0, -25.0, 14.0, 7.0]], [[-13.0, -32.0, -84.0, -79.0, -42.0], [63.0, 14.0, 55.0, 23.0, 49.0], [56.0, -54.0, 19.0, 18.0, -31.0], [21.0, 49.0, -52.0, -17.0, -91.0]], [[3.0, -27.0, -51.0, -33.0, -15.0], [27.0, 11.0, 45.0, 30.0, 4.0], [7.0, -6.0, -2.0, -9.0, -5.0], [-17.0, 4.0, -48.0, 0.0, -55.0]]], [[[1.0, 16.0, 29.0, 9.0, 44.0], [-35.0, -59.0, 11.0, 24.0, 27.0], [-34.0, -11.0, -7.0, -15.0, -44.0], [-48.0, 27.0, 19.0, 27.0, 26.0]], [[-1.0, 30.0, 62.0, 49.0, 8.0], [-29.0, 8.0, -65.0, -47.0, -25.0], [-6.0, 22.0, 1.0, 12.0, 29.0], [33.0, -25.0, 56.0, -7.0, 69.0]], [[-6.0, -24.0, -57.0, -48.0, -30.0], [42.0, 17.0, 36.0, 15.0, 22.0], [34.0, -27.0, 10.0, 9.0, -11.0], [13.0, 22.0, -39.0, -12.0, -61.0]], [[-15.0, 20.0, 23.0, 5.0, -28.0], [13.0, 27.0, -57.0, -58.0, 7.0], [46.0, -17.0, 21.0, 39.0, 18.0], [78.0, 7.0, 45.0, -29.0, 28.0]], [[-5.0, 32.0, 57.0, 31.0, 24.0], [-33.0, -27.0, -43.0, -26.0, 9.0], [-10.0, -3.0, 3.0, 9.0, -10.0], [10.0, 9.0, 55.0, 5.0, 60.0]]], [[[8.0, -26.0, -39.0, -10.0, -24.0], [24.0, 39.0, 22.0, 11.0, -30.0], [4.0, 21.0, -6.0, -9.0, 31.0], [-1.0, -30.0, -43.0, -8.0, -39.0]], [[6.0, -28.0, -49.0, -28.0, -10.0], [22.0, 9.0, 48.0, 35.0, -2.0], [-2.0, 1.0, -6.0, -15.0, -3.0], [-27.0, -2.0, -51.0, 4.0, -53.0]], [[-17.0, 9.0, -5.0, -25.0, -23.0], [23.0, 5.0, -19.0, -28.0, 36.0], [47.0, -40.0, 22.0, 33.0, -15.0], [53.0, 36.0, 22.0, -20.0, -5.0]], [[9.0, 3.0, 21.0, 33.0, 3.0], [-15.0, 19.0, -21.0, -12.0, -40.0], [-19.0, 36.0, -10.0, -9.0, 35.0], [-1.0, -40.0, 6.0, 0.0, 25.0]], [[-13.0, -7.0, -32.0, -39.0, -29.0], [35.0, 12.0, 7.0, -9.0, 32.0], [45.0, -37.0, 18.0, 24.0, -14.0], [38.0, 32.0, -7.0, -18.0, -34.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank4_offset2(self):
      arg0=numpy.array([[[[-1.0, 6.0, 0.0], [4.0, 6.0, -4.0]], [[-5.0, -6.0, -3.0], [-6.0, -6.0, 3.0]], [[1.0, -4.0, 0.0], [-7.0, -7.0, 0.0]], [[0.0, 1.0, 0.0], [1.0, 5.0, 2.0]], [[-1.0, -7.0, -3.0], [6.0, 7.0, 7.0]]], [[[-2.0, 4.0, -3.0], [-7.0, 5.0, 1.0]], [[3.0, -2.0, -3.0], [3.0, -3.0, 3.0]], [[7.0, -5.0, -3.0], [4.0, -4.0, 5.0]], [[-7.0, -5.0, -2.0], [-4.0, 1.0, 1.0]], [[6.0, -6.0, -1.0], [6.0, -4.0, 0.0]]], [[[-2.0, -1.0, 2.0], [7.0, -3.0, 4.0]], [[7.0, 7.0, -6.0], [-6.0, -2.0, -7.0]], [[-3.0, 0.0, -7.0], [3.0, 0.0, -5.0]], [[6.0, -6.0, 6.0], [-7.0, -7.0, -2.0]], [[4.0, -1.0, -6.0], [0.0, 5.0, 2.0]]], [[[-7.0, -3.0, 5.0], [2.0, 5.0, 2.0]], [[0.0, 5.0, -4.0], [-4.0, 6.0, -7.0]], [[0.0, -2.0, -6.0], [-2.0, -2.0, -7.0]], [[-2.0, 2.0, 1.0], [-2.0, -7.0, -1.0]], [[0.0, 2.0, 0.0], [-4.0, 0.0, 6.0]]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[0.0, 4.0, 2.0], [5.0, 0.0, -2.0]], [[-4.0, 0.0, 0.0], [-5.0, -6.0, 2.0]], [[5.0, -3.0, -3.0], [-3.0, -3.0, 1.0]], [[7.0, 0.0, 4.0], [0.0, 6.0, -3.0]], [[-2.0, 4.0, -1.0], [-7.0, 4.0, -7.0]]], [[[-3.0, 4.0, 6.0], [-3.0, -1.0, 0.0]], [[-5.0, -7.0, 5.0], [7.0, 1.0, -3.0]], [[4.0, -2.0, 1.0], [4.0, 0.0, 0.0]], [[0.0, -5.0, -6.0], [7.0, -2.0, 3.0]], [[4.0, 3.0, 2.0], [7.0, 7.0, 4.0]]], [[[-5.0, -7.0, 2.0], [0.0, -2.0, 1.0]], [[-1.0, 5.0, 3.0], [-3.0, 0.0, 6.0]], [[-3.0, 0.0, -6.0], [5.0, 5.0, 0.0]], [[0.0, 5.0, -4.0], [-5.0, 7.0, -4.0]], [[-4.0, -2.0, -3.0], [-3.0, -5.0, 3.0]]], [[[-3.0, 1.0, -6.0], [-3.0, 1.0, 0.0]], [[-1.0, -7.0, -4.0], [0.0, 2.0, 7.0]], [[0.0, -5.0, 2.0], [0.0, 5.0, -2.0]], [[6.0, -4.0, -1.0], [6.0, -7.0, -3.0]], [[-7.0, 6.0, -3.0], [1.0, -5.0, -5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[52.0, -60.0, -57.0, 41.0, 50.0], [9.0, 9.0, 0.0, -26.0, 68.0], [-53.0, -5.0, 53.0, 68.0, -62.0], [3.0, -57.0, 8.0, -36.0, 37.0]], [[-66.0, 92.0, 41.0, -92.0, -14.0], [-3.0, -5.0, -35.0, 27.0, -116.0], [76.0, 2.0, -27.0, -42.0, 98.0], [39.0, 68.0, -12.0, -6.0, 17.0]], [[-51.0, 73.0, 59.0, -35.0, 3.0], [9.0, -33.0, -16.0, -15.0, -106.0], [37.0, 0.0, -73.0, -34.0, 60.0], [7.0, 13.0, -15.0, 29.0, -3.0]], [[5.0, -31.0, -19.0, 24.0, 3.0], [-4.0, -1.0, 2.0, -2.0, 53.0], [-15.0, 14.0, 30.0, 27.0, -24.0], [3.0, 17.0, 16.0, -39.0, -28.0]], [[-18.0, -54.0, -7.0, 2.0, -86.0], [-68.0, 67.0, 31.0, 102.0, 88.0], [41.0, -19.0, 86.0, -32.0, -5.0], [3.0, 125.0, 50.0, -9.0, -90.0]]], [[[-27.0, 15.0, -6.0, 1.0, 85.0], [20.0, -80.0, -47.0, -58.0, -12.0], [-33.0, 40.0, 14.0, 98.0, 8.0], [54.0, 3.0, -3.0, -105.0, 10.0]], [[-5.0, -3.0, 33.0, -18.0, -65.0], [-41.0, -7.0, 25.0, 64.0, 12.0], [2.0, -13.0, 9.0, -46.0, 16.0], [-5.0, 38.0, -17.0, 59.0, -21.0]], [[-16.0, -14.0, 64.0, -2.0, -110.0], [-67.0, -6.0, 51.0, 94.0, 27.0], [7.0, -23.0, -3.0, -81.0, 14.0], [-24.0, 67.0, -11.0, 102.0, -71.0]], [[-46.0, 44.0, -4.0, -54.0, 21.0], [0.0, 30.0, -36.0, 10.0, -64.0], [65.0, -6.0, 18.0, 6.0, 54.0], [41.0, 59.0, 24.0, -54.0, 11.0]], [[4.0, -30.0, 45.0, 14.0, -93.0], [-62.0, 45.0, 59.0, 86.0, 18.0], [18.0, -57.0, -2.0, -84.0, -7.0], [-40.0, 32.0, 8.0, 125.0, -49.0]]], [[[27.0, -1.0, -21.0, -36.0, -91.0], [-4.0, 61.0, 24.0, 60.0, 37.0], [31.0, 6.0, 14.0, -85.0, 10.0], [-31.0, 23.0, -14.0, 41.0, 4.0]], [[0.0, 0.0, 49.0, 34.0, 103.0], [-9.0, -137.0, -16.0, -58.0, -47.0], [-99.0, -14.0, -25.0, 103.0, -17.0], [38.0, -85.0, -43.0, 19.0, 50.0]], [[11.0, -13.0, -8.0, -34.0, 27.0], [-42.0, 16.0, -7.0, 48.0, -25.0], [-4.0, -57.0, 66.0, 33.0, 9.0], [42.0, -4.0, -4.0, 22.0, 70.0]], [[-43.0, 49.0, 70.0, 30.0, -7.0], [22.0, -8.0, 14.0, -47.0, -88.0], [36.0, -9.0, -124.0, -60.0, 20.0], [-46.0, -16.0, 11.0, 67.0, -58.0]], [[-20.0, -42.0, 28.0, 28.0, 0.0], [-57.0, -44.0, 12.0, 37.0, 44.0], [-33.0, -15.0, 49.0, 46.0, -15.0], [28.0, 51.0, 14.0, -7.0, -51.0]]], [[[4.0, -8.0, -60.0, -5.0, -11.0], [28.0, 94.0, -9.0, -5.0, 30.0], [58.0, 13.0, 26.0, -18.0, -6.0], [-13.0, 32.0, 46.0, -64.0, -17.0]], [[6.0, -30.0, -16.0, 41.0, 125.0], [2.0, -56.0, -30.0, -62.0, -7.0], [-62.0, -17.0, 34.0, 131.0, -37.0], [47.0, -56.0, 11.0, -61.0, 43.0]], [[-16.0, 8.0, 29.0, -15.0, 53.0], [-36.0, -11.0, -10.0, 15.0, -74.0], [-1.0, -64.0, 16.0, 38.0, 17.0], [38.0, -15.0, 2.0, 37.0, 49.0]], [[2.0, 58.0, 7.0, -49.0, 4.0], [33.0, -17.0, -19.0, -19.0, -67.0], [11.0, 15.0, -45.0, -29.0, 39.0], [1.0, -37.0, -41.0, 19.0, 61.0]], [[-24.0, 32.0, 12.0, -18.0, -6.0], [20.0, -60.0, -20.0, -20.0, 2.0], [-8.0, 58.0, -20.0, 6.0, 26.0], [14.0, 28.0, -22.0, -50.0, -22.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank0_offset0(self):
      arg0=numpy.array([[-3.0, 3.0, -5.0, 0.0, -2.0], [0.0, 7.0, -6.0, 3.0, -5.0], [-4.0, 2.0, 5.0, 0.0, 3.0], [7.0, -3.0, 0.0, 7.0, 2.0]])
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[21.0, -21.0, 35.0, -0.0, 14.0], [-0.0, -49.0, 42.0, -21.0, 35.0], [28.0, -14.0, -35.0, -0.0, -21.0], [-49.0, 21.0, -0.0, -49.0, -14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank1_offset1(self):
      arg0=numpy.array([[[-3.0, -3.0, 0.0], [-3.0, -3.0, -1.0], [-4.0, -3.0, 2.0], [5.0, -1.0, 0.0], [0.0, -5.0, -6.0]], [[-7.0, 0.0, 1.0], [-1.0, 6.0, -7.0], [-6.0, -6.0, -7.0], [0.0, 7.0, -6.0], [-4.0, 4.0, 6.0]], [[-7.0, 1.0, 4.0], [5.0, -1.0, 5.0], [-2.0, 3.0, 0.0], [-1.0, 3.0, -2.0], [-5.0, -7.0, 5.0]], [[1.0, -5.0, -1.0], [0.0, 0.0, 0.0], [1.0, 0.0, -5.0], [-6.0, 7.0, 5.0], [0.0, 4.0, -5.0]]])
      arg1=Data(numpy.array([3.0, 5.0, -7.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-24.0, -17.0, -41.0, 10.0, 17.0], [-28.0, 76.0, 1.0, 77.0, -34.0], [-44.0, -25.0, 9.0, 26.0, -85.0], [-15.0, 0.0, 38.0, -18.0, 55.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank2_offset2(self):
      arg0=numpy.array([[[[-6.0, -7.0, -5.0], [3.0, 0.0, 3.0]], [[5.0, 4.0, 6.0], [-3.0, 2.0, -5.0]], [[0.0, 7.0, -3.0], [-6.0, -5.0, 4.0]], [[2.0, -1.0, 4.0], [-4.0, 3.0, -1.0]], [[-2.0, 2.0, 5.0], [2.0, -7.0, 5.0]]], [[[1.0, -7.0, -3.0], [-4.0, 4.0, -7.0]], [[-7.0, -5.0, 2.0], [0.0, -7.0, -5.0]], [[0.0, 0.0, -7.0], [-3.0, 7.0, 5.0]], [[0.0, -2.0, 6.0], [7.0, 2.0, 1.0]], [[4.0, 0.0, 1.0], [-5.0, -7.0, 0.0]]], [[[0.0, -3.0, -1.0], [5.0, 4.0, 6.0]], [[-4.0, 6.0, 0.0], [-5.0, 1.0, 0.0]], [[-6.0, -1.0, 7.0], [5.0, -4.0, 0.0]], [[4.0, -1.0, -6.0], [5.0, -4.0, 6.0]], [[1.0, -3.0, -4.0], [0.0, 3.0, 4.0]]], [[[-6.0, 3.0, -1.0], [-3.0, -3.0, 1.0]], [[0.0, -3.0, -5.0], [1.0, 0.0, -3.0]], [[-2.0, 0.0, -6.0], [-6.0, 2.0, 4.0]], [[4.0, 2.0, 5.0], [-1.0, 7.0, -6.0]], [[-2.0, -4.0, -7.0], [5.0, 7.0, 6.0]]]])
      arg1=Data(numpy.array([[-2.0, -7.0, 4.0], [-1.0, 1.0, 6.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[56.0, -39.0, -36.0, 20.0, 31.0], [1.0, 20.0, 12.0, 39.0, -6.0], [52.0, -28.0, 38.0, 2.0, 30.0], [-7.0, -18.0, 12.0, -30.0, 42.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank1_offset0(self):
      arg0=numpy.array([[-2.0, 4.0, 0.0, -3.0, -2.0], [0.0, -6.0, 7.0, -1.0, 3.0], [0.0, 5.0, -3.0, -2.0, 7.0], [3.0, -2.0, 1.0, 2.0, 2.0]])
      arg1=Data(numpy.array([7.0, 6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-14.0, -12.0], [28.0, 24.0], [0.0, 0.0], [-21.0, -18.0], [-14.0, -12.0]], [[0.0, 0.0], [-42.0, -36.0], [49.0, 42.0], [-7.0, -6.0], [21.0, 18.0]], [[0.0, 0.0], [35.0, 30.0], [-21.0, -18.0], [-14.0, -12.0], [49.0, 42.0]], [[21.0, 18.0], [-14.0, -12.0], [7.0, 6.0], [14.0, 12.0], [14.0, 12.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank2_offset1(self):
      arg0=numpy.array([[[2.0, 7.0, 6.0], [5.0, 2.0, -5.0], [1.0, -5.0, 4.0], [5.0, 6.0, 1.0], [-2.0, 5.0, 3.0]], [[-5.0, -6.0, 2.0], [5.0, 3.0, 7.0], [-1.0, 0.0, -4.0], [5.0, 3.0, -6.0], [-7.0, 0.0, 3.0]], [[6.0, -1.0, -3.0], [-4.0, 0.0, -4.0], [-4.0, 2.0, 1.0], [-2.0, -4.0, -2.0], [6.0, 6.0, 4.0]], [[5.0, 6.0, -6.0], [7.0, -5.0, 6.0], [2.0, 6.0, 6.0], [6.0, 6.0, -7.0], [2.0, 6.0, 6.0]]])
      arg1=Data(numpy.array([[3.0, -6.0, -7.0], [0.0, -6.0, -2.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-78.0, -54.0], [38.0, -2.0], [5.0, 22.0], [-28.0, -38.0], [-57.0, -36.0]], [[7.0, 32.0], [-52.0, -32.0], [25.0, 8.0], [39.0, -6.0], [-42.0, -6.0]], [[45.0, 12.0], [16.0, 8.0], [-31.0, -14.0], [32.0, 28.0], [-46.0, -44.0]], [[21.0, -24.0], [9.0, 18.0], [-72.0, -48.0], [31.0, -22.0], [-72.0, -48.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank3_offset2(self):
      arg0=numpy.array([[[[-1.0, -4.0, -2.0], [7.0, 6.0, -5.0]], [[1.0, -5.0, 4.0], [5.0, -7.0, -7.0]], [[4.0, 7.0, -4.0], [-1.0, -3.0, -2.0]], [[0.0, -3.0, -2.0], [4.0, -3.0, 1.0]], [[-4.0, 1.0, -7.0], [-5.0, 6.0, -1.0]]], [[[2.0, 2.0, -5.0], [-2.0, -5.0, -4.0]], [[3.0, 3.0, -1.0], [-2.0, 0.0, -7.0]], [[4.0, -1.0, 2.0], [0.0, 2.0, -2.0]], [[4.0, 1.0, 4.0], [2.0, -5.0, 5.0]], [[-2.0, -3.0, -7.0], [3.0, 6.0, 0.0]]], [[[-5.0, -3.0, -3.0], [-3.0, -1.0, -1.0]], [[5.0, -2.0, 1.0], [-7.0, -5.0, -6.0]], [[-4.0, 3.0, 6.0], [-2.0, 1.0, 6.0]], [[0.0, -4.0, 6.0], [6.0, 6.0, 0.0]], [[-7.0, 0.0, -2.0], [0.0, 7.0, 1.0]]], [[[-6.0, 0.0, 1.0], [7.0, -2.0, 2.0]], [[-1.0, -1.0, 0.0], [6.0, 1.0, -6.0]], [[1.0, 6.0, 3.0], [-5.0, -6.0, 3.0]], [[-7.0, 1.0, 5.0], [0.0, 3.0, 1.0]], [[4.0, -2.0, 5.0], [0.0, -2.0, 2.0]]]])
      arg1=Data(numpy.array([[[4.0, 1.0, -2.0], [-7.0, 6.0, 5.0]], [[-4.0, -3.0, -3.0], [-1.0, 0.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-42.0, 15.0], [-121.0, -6.0], [10.0, -24.0], [-40.0, 11.0], [65.0, 39.0]], [[-16.0, 3.0], [-4.0, -16.0], [13.0, -19.0], [-10.0, -33.0], [18.0, 35.0]], [[-7.0, 41.0], [5.0, -10.0], [25.0, -9.0], [-22.0, -12.0], [23.0, 34.0]], [[-77.0, 14.0], [-71.0, 1.0], [18.0, -26.0], [-14.0, 10.0], [2.0, -25.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_constData_rank2_offset0(self):
      arg0=numpy.array([[-5.0, 2.0, -7.0, 3.0, -3.0], [0.0, -1.0, 7.0, -7.0, -6.0], [-4.0, 6.0, -2.0, 6.0, 5.0], [5.0, 0.0, 5.0, -6.0, 0.0]])
      arg1=Data(numpy.array([[-4.0, -2.0, 0.0, 0.0, 0.0], [1.0, 7.0, 1.0, 2.0, 5.0], [4.0, -5.0, -1.0, 4.0, 6.0], [4.0, 1.0, -6.0, 4.0, -1.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[20.0, 10.0, 0.0, 0.0, 0.0], [-5.0, -35.0, -5.0, -10.0, -25.0], [-20.0, 25.0, 5.0, -20.0, -30.0], [-20.0, -5.0, 30.0, -20.0, 5.0]], [[-8.0, -4.0, 0.0, 0.0, 0.0], [2.0, 14.0, 2.0, 4.0, 10.0], [8.0, -10.0, -2.0, 8.0, 12.0], [8.0, 2.0, -12.0, 8.0, -2.0]], [[28.0, 14.0, 0.0, 0.0, 0.0], [-7.0, -49.0, -7.0, -14.0, -35.0], [-28.0, 35.0, 7.0, -28.0, -42.0], [-28.0, -7.0, 42.0, -28.0, 7.0]], [[-12.0, -6.0, 0.0, 0.0, 0.0], [3.0, 21.0, 3.0, 6.0, 15.0], [12.0, -15.0, -3.0, 12.0, 18.0], [12.0, 3.0, -18.0, 12.0, -3.0]], [[12.0, 6.0, 0.0, 0.0, 0.0], [-3.0, -21.0, -3.0, -6.0, -15.0], [-12.0, 15.0, 3.0, -12.0, -18.0], [-12.0, -3.0, 18.0, -12.0, 3.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[4.0, 2.0, 0.0, 0.0, 0.0], [-1.0, -7.0, -1.0, -2.0, -5.0], [-4.0, 5.0, 1.0, -4.0, -6.0], [-4.0, -1.0, 6.0, -4.0, 1.0]], [[-28.0, -14.0, 0.0, 0.0, 0.0], [7.0, 49.0, 7.0, 14.0, 35.0], [28.0, -35.0, -7.0, 28.0, 42.0], [28.0, 7.0, -42.0, 28.0, -7.0]], [[28.0, 14.0, 0.0, 0.0, 0.0], [-7.0, -49.0, -7.0, -14.0, -35.0], [-28.0, 35.0, 7.0, -28.0, -42.0], [-28.0, -7.0, 42.0, -28.0, 7.0]], [[24.0, 12.0, 0.0, 0.0, 0.0], [-6.0, -42.0, -6.0, -12.0, -30.0], [-24.0, 30.0, 6.0, -24.0, -36.0], [-24.0, -6.0, 36.0, -24.0, 6.0]]], [[[16.0, 8.0, 0.0, 0.0, 0.0], [-4.0, -28.0, -4.0, -8.0, -20.0], [-16.0, 20.0, 4.0, -16.0, -24.0], [-16.0, -4.0, 24.0, -16.0, 4.0]], [[-24.0, -12.0, 0.0, 0.0, 0.0], [6.0, 42.0, 6.0, 12.0, 30.0], [24.0, -30.0, -6.0, 24.0, 36.0], [24.0, 6.0, -36.0, 24.0, -6.0]], [[8.0, 4.0, 0.0, 0.0, 0.0], [-2.0, -14.0, -2.0, -4.0, -10.0], [-8.0, 10.0, 2.0, -8.0, -12.0], [-8.0, -2.0, 12.0, -8.0, 2.0]], [[-24.0, -12.0, 0.0, 0.0, 0.0], [6.0, 42.0, 6.0, 12.0, 30.0], [24.0, -30.0, -6.0, 24.0, 36.0], [24.0, 6.0, -36.0, 24.0, -6.0]], [[-20.0, -10.0, 0.0, 0.0, 0.0], [5.0, 35.0, 5.0, 10.0, 25.0], [20.0, -25.0, -5.0, 20.0, 30.0], [20.0, 5.0, -30.0, 20.0, -5.0]]], [[[-20.0, -10.0, 0.0, 0.0, 0.0], [5.0, 35.0, 5.0, 10.0, 25.0], [20.0, -25.0, -5.0, 20.0, 30.0], [20.0, 5.0, -30.0, 20.0, -5.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-20.0, -10.0, 0.0, 0.0, 0.0], [5.0, 35.0, 5.0, 10.0, 25.0], [20.0, -25.0, -5.0, 20.0, 30.0], [20.0, 5.0, -30.0, 20.0, -5.0]], [[24.0, 12.0, 0.0, 0.0, 0.0], [-6.0, -42.0, -6.0, -12.0, -30.0], [-24.0, 30.0, 6.0, -24.0, -36.0], [-24.0, -6.0, 36.0, -24.0, 6.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank3_offset1(self):
      arg0=numpy.array([[[-7.0, 6.0, 6.0], [6.0, 7.0, 3.0], [3.0, 7.0, 6.0], [7.0, 3.0, -5.0], [-3.0, 0.0, 0.0]], [[5.0, 3.0, -2.0], [1.0, 2.0, 6.0], [2.0, 4.0, -3.0], [0.0, -2.0, 3.0], [-4.0, 0.0, -5.0]], [[4.0, 0.0, 6.0], [0.0, -3.0, 0.0], [0.0, -2.0, 0.0], [4.0, -7.0, -6.0], [-1.0, -4.0, 4.0]], [[6.0, -4.0, 5.0], [2.0, -1.0, 2.0], [1.0, 2.0, -3.0], [-1.0, 1.0, 0.0], [5.0, -2.0, -6.0]]])
      arg1=Data(numpy.array([[[3.0, -5.0, 1.0], [0.0, 0.0, 0.0], [-2.0, 4.0, 6.0], [0.0, 2.0, 2.0], [4.0, -5.0, -5.0]], [[3.0, -2.0, -6.0], [0.0, 3.0, 4.0], [-5.0, 0.0, -4.0], [-3.0, -3.0, -2.0], [-2.0, 1.0, -2.0]], [[-4.0, -3.0, 2.0], [0.0, -3.0, -2.0], [7.0, -5.0, -3.0], [2.0, 6.0, 7.0], [-2.0, 2.0, 5.0]], [[1.0, 2.0, 1.0], [5.0, -5.0, -4.0], [-3.0, -7.0, 0.0], [4.0, 7.0, 3.0], [1.0, 3.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-45.0, 0.0, 74.0, 24.0, -88.0], [-69.0, 42.0, 11.0, -9.0, 8.0], [22.0, -30.0, -97.0, 64.0, 56.0], [11.0, -89.0, -21.0, 32.0, 11.0]], [[-14.0, 0.0, 34.0, 20.0, -26.0], [-14.0, 33.0, -42.0, -45.0, -11.0], [-39.0, -27.0, -2.0, 75.0, 17.0], [23.0, -17.0, -67.0, 82.0, 27.0]], [[-20.0, 0.0, 58.0, 26.0, -53.0], [-41.0, 45.0, -39.0, -42.0, -11.0], [-21.0, -33.0, -32.0, 90.0, 38.0], [23.0, -44.0, -58.0, 79.0, 24.0]], [[1.0, 0.0, -32.0, -4.0, 38.0], [45.0, -11.0, -15.0, -20.0, -1.0], [-47.0, 1.0, 49.0, -3.0, -33.0], [8.0, 40.0, -42.0, 34.0, 16.0]], [[-9.0, 0.0, 6.0, 0.0, -12.0], [-9.0, 0.0, 15.0, 9.0, 6.0], [12.0, 0.0, -21.0, -6.0, 6.0], [-3.0, -15.0, 9.0, -12.0, -3.0]]], [[[-2.0, 0.0, -10.0, 2.0, 15.0], [21.0, 1.0, -17.0, -20.0, -3.0], [-33.0, -5.0, 26.0, 14.0, -14.0], [9.0, 18.0, -36.0, 35.0, 14.0]], [[-1.0, 0.0, 42.0, 16.0, -36.0], [-37.0, 30.0, -29.0, -21.0, -12.0], [2.0, -18.0, -21.0, 56.0, 32.0], [11.0, -29.0, -17.0, 36.0, 7.0]], [[-17.0, 0.0, -6.0, 2.0, 3.0], [16.0, 0.0, 2.0, -12.0, 6.0], [-26.0, -6.0, 3.0, 7.0, -11.0], [7.0, 2.0, -34.0, 27.0, 14.0]], [[13.0, 0.0, 10.0, 2.0, -5.0], [-14.0, 6.0, -12.0, 0.0, -8.0], [12.0, 0.0, 1.0, 9.0, 11.0], [-1.0, -2.0, 14.0, -5.0, -6.0]], [[-17.0, 0.0, -22.0, -10.0, 9.0], [18.0, -20.0, 40.0, 22.0, 18.0], [6.0, 10.0, -13.0, -43.0, -17.0], [-9.0, 0.0, 12.0, -31.0, -4.0]]], [[[18.0, 0.0, 28.0, 12.0, -14.0], [-24.0, 24.0, -44.0, -24.0, -20.0], [-4.0, -12.0, 10.0, 50.0, 22.0], [10.0, -4.0, -12.0, 34.0, 4.0]], [[15.0, 0.0, -12.0, -6.0, 15.0], [6.0, -9.0, 0.0, 9.0, -3.0], [9.0, 9.0, 15.0, -18.0, -6.0], [-6.0, 15.0, 21.0, -21.0, -9.0]], [[10.0, 0.0, -8.0, -4.0, 10.0], [4.0, -6.0, 0.0, 6.0, -2.0], [6.0, 6.0, 10.0, -12.0, -4.0], [-4.0, 10.0, 14.0, -14.0, -6.0]], [[41.0, 0.0, -72.0, -26.0, 81.0], [62.0, -45.0, 4.0, 21.0, -3.0], [-7.0, 33.0, 81.0, -76.0, -52.0], [-16.0, 79.0, 37.0, -51.0, -17.0]], [[21.0, 0.0, 10.0, 0.0, -4.0], [-19.0, 4.0, -11.0, 7.0, -10.0], [24.0, 4.0, 1.0, 2.0, 14.0], [-5.0, -1.0, 31.0, -20.0, -13.0]]], [[[43.0, 0.0, 2.0, 2.0, 19.0], [-4.0, 8.0, -50.0, -16.0, -26.0], [-2.0, 2.0, 47.0, 23.0, 5.0], [3.0, 30.0, 10.0, 11.0, -6.0]], [[13.0, 0.0, 4.0, 2.0, 3.0], [-4.0, 5.0, -18.0, -7.0, -9.0], [-1.0, -1.0, 13.0, 12.0, 4.0], [2.0, 7.0, 1.0, 7.0, -1.0]], [[-10.0, 0.0, -12.0, -2.0, 9.0], [17.0, -6.0, 7.0, -3.0, 6.0], [-16.0, 0.0, 6.0, -7.0, -13.0], [2.0, 7.0, -17.0, 9.0, 7.0]], [[-8.0, 0.0, 6.0, 2.0, -9.0], [-5.0, 3.0, 5.0, 0.0, 3.0], [1.0, -3.0, -12.0, 4.0, 4.0], [1.0, -10.0, -4.0, 3.0, 2.0]], [[19.0, 0.0, -54.0, -16.0, 60.0], [55.0, -30.0, -1.0, 3.0, 0.0], [-26.0, 18.0, 63.0, -44.0, -44.0], [-5.0, 59.0, -1.0, -12.0, -1.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank4_offset2(self):
      arg0=numpy.array([[[[-3.0, -2.0, 3.0], [6.0, 5.0, 4.0]], [[1.0, -6.0, 3.0], [-1.0, 0.0, 7.0]], [[4.0, 0.0, -5.0], [-1.0, 7.0, 0.0]], [[-2.0, -1.0, 4.0], [-6.0, 0.0, 5.0]], [[-3.0, 2.0, 2.0], [3.0, -7.0, -2.0]]], [[[-6.0, -2.0, -3.0], [0.0, 4.0, 0.0]], [[7.0, 3.0, 5.0], [4.0, -5.0, -1.0]], [[1.0, 5.0, -1.0], [-7.0, 4.0, 2.0]], [[-1.0, 0.0, 0.0], [0.0, 0.0, 0.0]], [[0.0, 5.0, -5.0], [0.0, 3.0, -3.0]]], [[[7.0, -5.0, -7.0], [6.0, 2.0, -6.0]], [[-6.0, 6.0, -2.0], [-4.0, -7.0, 6.0]], [[5.0, 0.0, 0.0], [-5.0, 4.0, 0.0]], [[-2.0, 3.0, 0.0], [-7.0, -7.0, -2.0]], [[-6.0, -4.0, 1.0], [-3.0, -3.0, 6.0]]], [[[0.0, 1.0, -4.0], [0.0, -2.0, 0.0]], [[6.0, 2.0, -6.0], [0.0, 5.0, 5.0]], [[7.0, -3.0, -6.0], [7.0, -2.0, 0.0]], [[1.0, -6.0, -6.0], [-1.0, 5.0, -2.0]], [[-5.0, 4.0, 3.0], [7.0, -6.0, 3.0]]]])
      arg1=Data(numpy.array([[[[4.0, 3.0, -1.0], [6.0, -2.0, -3.0]], [[3.0, 4.0, -3.0], [-5.0, 4.0, -7.0]], [[-4.0, -7.0, 3.0], [-5.0, 0.0, -2.0]], [[6.0, -5.0, 0.0], [3.0, -5.0, -1.0]], [[6.0, 1.0, 1.0], [4.0, -3.0, 3.0]]], [[[-4.0, -3.0, -4.0], [7.0, 1.0, 4.0]], [[3.0, -3.0, -3.0], [0.0, 5.0, 1.0]], [[5.0, 4.0, -2.0], [-2.0, 2.0, 1.0]], [[4.0, -4.0, 6.0], [-6.0, 0.0, -7.0]], [[-3.0, 6.0, 6.0], [7.0, -7.0, -5.0]]], [[[4.0, -4.0, -7.0], [-5.0, -4.0, 6.0]], [[0.0, -5.0, -4.0], [1.0, 0.0, 1.0]], [[2.0, -4.0, -6.0], [0.0, -6.0, 0.0]], [[7.0, 1.0, -5.0], [-3.0, 0.0, -5.0]], [[5.0, 1.0, -2.0], [4.0, 3.0, -6.0]]], [[[-2.0, -1.0, 3.0], [3.0, 2.0, -2.0]], [[2.0, 6.0, 5.0], [0.0, -7.0, -6.0]], [[-7.0, -1.0, 5.0], [-5.0, 2.0, -6.0]], [[-3.0, 5.0, -7.0], [-4.0, 0.0, 7.0]], [[-2.0, 6.0, 6.0], [0.0, -5.0, -7.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-7.0, -64.0, -3.0, -19.0, 4.0], [69.0, 17.0, -27.0, -50.0, 2.0], [-51.0, 8.0, -46.0, -76.0, -8.0], [37.0, -62.0, -6.0, -18.0, -41.0]], [[-44.0, -74.0, 38.0, 26.0, 20.0], [23.0, 19.0, -16.0, 3.0, -63.0], [54.0, 24.0, 8.0, -46.0, -53.0], [-4.0, -61.0, -23.0, -1.0, -69.0]], [[1.0, 60.0, -26.0, -14.0, -6.0], [4.0, 62.0, 46.0, -8.0, -98.0], [28.0, 19.0, -4.0, 56.0, 47.0], [-12.0, -66.0, -34.0, 27.0, -73.0]], [[-66.0, -27.0, 47.0, -30.0, -18.0], [-27.0, -10.0, -5.0, 21.0, -43.0], [28.0, -12.0, -24.0, -42.0, -73.0], [-11.0, -20.0, 35.0, 32.0, -13.0]], [[30.0, -36.0, -7.0, 18.0, 13.0], [4.0, -58.0, -33.0, -12.0, 113.0], [-33.0, -17.0, 16.0, -28.0, -14.0], [9.0, 77.0, 12.0, -21.0, 79.0]]], [[[-35.0, -1.0, 29.0, -46.0, -53.0], [46.0, 17.0, -24.0, -34.0, -40.0], [-11.0, 22.0, -10.0, -29.0, -14.0], [13.0, -67.0, 37.0, 29.0, -38.0]], [[69.0, -15.0, -52.0, 65.0, 78.0], [-38.0, -29.0, 18.0, 29.0, 95.0], [-25.0, -32.0, 2.0, 20.0, 35.0], [2.0, 98.0, -51.0, -64.0, 66.0]], [[-36.0, 63.0, -11.0, -62.0, -24.0], [-52.0, 13.0, 51.0, 6.0, -66.0], [22.0, -26.0, -36.0, 28.0, -16.0], [-27.0, -13.0, 14.0, 71.0, -12.0]], [[-4.0, -3.0, 4.0, -6.0, -6.0], [4.0, -3.0, -5.0, -4.0, 3.0], [-4.0, 0.0, -2.0, -7.0, -5.0], [2.0, -2.0, 7.0, 3.0, 2.0]], [[23.0, 68.0, -44.0, -37.0, -18.0], [-4.0, 12.0, 33.0, -29.0, -6.0], [-15.0, -8.0, -8.0, 45.0, 42.0], [-8.0, 2.0, -6.0, 39.0, 6.0]]], [[[70.0, 42.0, -32.0, 81.0, 30.0], [35.0, 61.0, 15.0, 12.0, -35.0], [23.0, 53.0, 64.0, 91.0, 110.0], [4.0, -29.0, -69.0, -63.0, -54.0]], [[-32.0, -38.0, -16.0, -49.0, -9.0], [3.0, -59.0, -2.0, -78.0, 33.0], [50.0, -20.0, 18.0, -44.0, -93.0], [-38.0, 27.0, -4.0, 120.0, 29.0]], [[-18.0, 56.0, 5.0, -5.0, -2.0], [-51.0, 35.0, 43.0, 50.0, -78.0], [29.0, -5.0, -14.0, 50.0, 17.0], [-17.0, -18.0, -2.0, 5.0, -30.0]], [[-21.0, 27.0, 26.0, -11.0, -22.0], [-65.0, -52.0, 0.0, 36.0, 34.0], [31.0, -24.0, 26.0, 20.0, -44.0], [-30.0, 75.0, 44.0, 35.0, 71.0]], [[-67.0, -76.0, 58.0, -16.0, -24.0], [32.0, -18.0, -42.0, -26.0, -30.0], [48.0, 19.0, 16.0, -72.0, -93.0], [-8.0, -46.0, 24.0, 45.0, -33.0]]], [[[11.0, 8.0, -19.0, 5.0, 3.0], [11.0, -1.0, 8.0, -28.0, -4.0], [32.0, 11.0, 32.0, 21.0, 3.0], [-17.0, 0.0, -25.0, 33.0, -8.0]], [[11.0, 29.0, -66.0, -4.0, 32.0], [19.0, 60.0, 65.0, -55.0, -102.0], [68.0, 19.0, 10.0, 49.0, 29.0], [-32.0, -71.0, -94.0, 69.0, -96.0]], [[71.0, -16.0, -60.0, 88.0, 67.0], [52.0, 38.0, 17.0, -38.0, -12.0], [55.0, 46.0, 74.0, 55.0, 66.0], [-12.0, -20.0, -115.0, -22.0, -58.0]], [[-18.0, 36.0, 29.0, 10.0, -31.0], [28.0, 62.0, 3.0, 12.0, -107.0], [43.0, 51.0, 32.0, 44.0, 34.0], [-3.0, -87.0, -4.0, -1.0, -85.0]], [[34.0, -88.0, -40.0, -2.0, 32.0], [51.0, -63.0, -38.0, -81.0, 133.0], [-50.0, -22.0, -8.0, -82.0, -35.0], [18.0, 53.0, -19.0, 7.0, 61.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank0_offset0(self):
      arg0=numpy.array([[-2.0, 3.0, -7.0, 3.0, -2.0], [-4.0, -1.0, -2.0, 7.0, 3.0], [3.0, -6.0, -5.0, 0.0, 4.0], [0.0, 0.0, -5.0, -4.0, 5.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-2.0)+(1-msk_arg1)*(7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, -6.0, 14.0, -6.0, 4.0], [8.0, 2.0, 4.0, -14.0, -6.0], [-6.0, 12.0, 10.0, -0.0, -8.0], [-0.0, -0.0, 10.0, 8.0, -10.0]])+(1.-msk_ref)*numpy.array([[-14.0, 21.0, -49.0, 21.0, -14.0], [-28.0, -7.0, -14.0, 49.0, 21.0], [21.0, -42.0, -35.0, 0.0, 28.0], [0.0, 0.0, -35.0, -28.0, 35.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[-5.0, 5.0, 0.0], [4.0, -6.0, 7.0], [3.0, -7.0, 0.0], [-2.0, -5.0, 0.0], [3.0, 2.0, 6.0]], [[6.0, -6.0, 0.0], [2.0, -2.0, -5.0], [2.0, 4.0, 2.0], [-3.0, -4.0, 4.0], [2.0, -5.0, 4.0]], [[6.0, 5.0, 6.0], [4.0, 4.0, -6.0], [7.0, 3.0, 1.0], [4.0, -7.0, 7.0], [1.0, 0.0, 1.0]], [[0.0, 0.0, 4.0], [6.0, 3.0, -5.0], [6.0, 7.0, 3.0], [4.0, -4.0, 6.0], [5.0, -2.0, 4.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, 7.0, -5.0])+(1.-msk_arg1)*numpy.array([-6.0, 7.0, -2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[25.0, -69.0, -43.0, -39.0, -10.0], [-30.0, 15.0, 22.0, -54.0, -51.0], [17.0, 66.0, 30.0, -76.0, -3.0], [-20.0, 58.0, 46.0, -50.0, -24.0]])+(1.-msk_ref)*numpy.array([[65.0, -80.0, -67.0, -23.0, -16.0], [-78.0, -16.0, 12.0, -18.0, -55.0], [-13.0, 16.0, -23.0, -87.0, -8.0], [-8.0, -5.0, 7.0, -64.0, -52.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank2_offset2(self):
      arg0=numpy.array([[[[-6.0, -5.0, -2.0], [2.0, -2.0, 4.0]], [[6.0, -1.0, -5.0], [0.0, -3.0, 0.0]], [[7.0, 2.0, 4.0], [5.0, -3.0, -2.0]], [[6.0, -4.0, 5.0], [-2.0, 0.0, 3.0]], [[-5.0, 3.0, 2.0], [2.0, 6.0, -4.0]]], [[[1.0, 7.0, -4.0], [5.0, -4.0, -3.0]], [[1.0, 1.0, 4.0], [-6.0, -4.0, 0.0]], [[0.0, -5.0, -3.0], [-7.0, -3.0, 0.0]], [[-6.0, 5.0, -5.0], [0.0, -4.0, -7.0]], [[0.0, -7.0, 5.0], [-1.0, 6.0, 2.0]]], [[[-3.0, 0.0, 0.0], [2.0, -5.0, 0.0]], [[-4.0, -6.0, -6.0], [-2.0, 0.0, 4.0]], [[-7.0, -2.0, 1.0], [5.0, 6.0, 0.0]], [[0.0, -4.0, -5.0], [0.0, -3.0, -7.0]], [[1.0, 1.0, 7.0], [6.0, 5.0, 4.0]]], [[[0.0, -5.0, 3.0], [1.0, -2.0, 4.0]], [[-7.0, 2.0, -3.0], [-3.0, 0.0, 2.0]], [[2.0, -7.0, 0.0], [-7.0, 2.0, 0.0]], [[-7.0, -4.0, 4.0], [7.0, 2.0, 5.0]], [[4.0, -1.0, -1.0], [-4.0, 6.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 4.0, 0.0], [3.0, 1.0, 6.0]])+(1.-msk_arg1)*numpy.array([[0.0, 1.0, 0.0], [1.0, 5.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[44.0, -43.0, -34.0, -40.0, 30.0], [15.0, -24.0, -44.0, 10.0, -13.0], [19.0, 18.0, 55.0, -61.0, 45.0], [5.0, 53.0, -59.0, 79.0, -40.0]])+(1.-msk_ref)*numpy.array([[-25.0, -16.0, -2.0, -15.0, 47.0], [1.0, -25.0, -27.0, 6.0, 16.0], [-23.0, -20.0, 33.0, 2.0, 20.0], [-26.0, -7.0, -4.0, -2.0, 28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank1_offset0(self):
      arg0=numpy.array([[4.0, 1.0, 2.0, 3.0, 6.0], [-6.0, 6.0, 3.0, 3.0, 5.0], [-2.0, 3.0, 1.0, -3.0, 6.0], [7.0, 3.0, 5.0, 4.0, -4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 4.0])+(1.-msk_arg1)*numpy.array([-5.0, 4.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 16.0], [0.0, 4.0], [0.0, 8.0], [0.0, 12.0], [0.0, 24.0]], [[0.0, -24.0], [0.0, 24.0], [0.0, 12.0], [0.0, 12.0], [0.0, 20.0]], [[0.0, -8.0], [0.0, 12.0], [0.0, 4.0], [0.0, -12.0], [0.0, 24.0]], [[0.0, 28.0], [0.0, 12.0], [0.0, 20.0], [0.0, 16.0], [0.0, -16.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, 16.0], [-5.0, 4.0], [-10.0, 8.0], [-15.0, 12.0], [-30.0, 24.0]], [[30.0, -24.0], [-30.0, 24.0], [-15.0, 12.0], [-15.0, 12.0], [-25.0, 20.0]], [[10.0, -8.0], [-15.0, 12.0], [-5.0, 4.0], [15.0, -12.0], [-30.0, 24.0]], [[-35.0, 28.0], [-15.0, 12.0], [-25.0, 20.0], [-20.0, 16.0], [20.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[7.0, -2.0, -5.0], [-4.0, -6.0, -4.0], [3.0, 7.0, 5.0], [7.0, 3.0, -4.0], [1.0, 0.0, -5.0]], [[-6.0, -4.0, 1.0], [-4.0, 6.0, -3.0], [0.0, -3.0, -4.0], [-7.0, 6.0, -7.0], [-2.0, -7.0, 2.0]], [[1.0, 2.0, 1.0], [3.0, 6.0, -4.0], [-7.0, 0.0, 3.0], [-7.0, 1.0, 7.0], [-2.0, -1.0, 6.0]], [[4.0, -3.0, 0.0], [-6.0, -7.0, 4.0], [-5.0, -7.0, 6.0], [2.0, -3.0, -5.0], [-4.0, -3.0, -2.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 2.0, -2.0], [-4.0, -7.0, -5.0]])+(1.-msk_arg1)*numpy.array([[1.0, 7.0, 0.0], [0.0, -7.0, 4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[27.0, 11.0], [-16.0, 78.0], [13.0, -86.0], [35.0, -29.0], [13.0, 21.0]], [[-28.0, 47.0], [6.0, -11.0], [2.0, 41.0], [5.0, 21.0], [-24.0, 47.0]], [[5.0, -23.0], [29.0, -34.0], [-27.0, 13.0], [-33.0, -14.0], [-20.0, -15.0]], [[6.0, 5.0], [-40.0, 53.0], [-41.0, 39.0], [10.0, 38.0], [-14.0, 47.0]]])+(1.-msk_ref)*numpy.array([[[-7.0, -6.0], [-46.0, 26.0], [52.0, -29.0], [28.0, -37.0], [1.0, -20.0]], [[-34.0, 32.0], [38.0, -54.0], [-21.0, 5.0], [35.0, -70.0], [-51.0, 57.0]], [[15.0, -10.0], [45.0, -58.0], [-7.0, 12.0], [0.0, 21.0], [-9.0, 31.0]], [[-17.0, 21.0], [-55.0, 65.0], [-54.0, 73.0], [-19.0, 1.0], [-25.0, 13.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank3_offset2(self):
      arg0=numpy.array([[[[7.0, -1.0, -1.0], [7.0, -3.0, -7.0]], [[4.0, 3.0, 0.0], [3.0, -2.0, -7.0]], [[6.0, 0.0, -4.0], [-1.0, 2.0, 7.0]], [[-7.0, 3.0, 0.0], [7.0, 0.0, 0.0]], [[1.0, -7.0, 2.0], [-5.0, 5.0, -1.0]]], [[[-2.0, 0.0, 5.0], [-3.0, 4.0, -5.0]], [[-7.0, 7.0, 3.0], [6.0, 3.0, -5.0]], [[1.0, -6.0, 4.0], [-3.0, 0.0, -6.0]], [[6.0, -5.0, 6.0], [-2.0, 4.0, 7.0]], [[7.0, 2.0, -4.0], [6.0, -5.0, 1.0]]], [[[-5.0, -2.0, 1.0], [-3.0, 5.0, 7.0]], [[0.0, -7.0, 4.0], [3.0, -2.0, -3.0]], [[7.0, 6.0, 1.0], [6.0, -4.0, 1.0]], [[-3.0, -6.0, 5.0], [0.0, -3.0, 2.0]], [[-1.0, -3.0, -7.0], [1.0, 7.0, -3.0]]], [[[2.0, 5.0, 0.0], [-2.0, -7.0, 4.0]], [[-7.0, 7.0, 0.0], [-1.0, -3.0, -2.0]], [[-3.0, -5.0, -3.0], [7.0, 4.0, 0.0]], [[-5.0, 3.0, -1.0], [0.0, 2.0, -2.0]], [[2.0, 2.0, 0.0], [-2.0, 0.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, -7.0, -1.0], [-5.0, 2.0, 2.0]], [[-4.0, 3.0, -7.0], [-3.0, 1.0, -2.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 7.0, 2.0], [0.0, 7.0, -7.0]], [[-4.0, -5.0, 5.0], [-3.0, 0.0, 7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, -34.0], [-26.0, -4.0], [69.0, -5.0], [-105.0, 16.0], [87.0, -17.0]], [[-6.0, -4.0], [-135.0, 23.0], [48.0, -29.0], [103.0, -85.0], [1.0, -19.0]], [[17.0, 7.0], [20.0, -54.0], [-30.0, -41.0], [14.0, -48.0], [24.0, 54.0]], [[-17.0, -2.0], [-103.0, 53.0], [-10.0, 1.0], [-55.0, 42.0], [4.0, 10.0]]])+(1.-msk_ref)*numpy.array([[[5.0, -98.0], [48.0, -89.0], [-55.0, 8.0], [35.0, -8.0], [-5.0, 49.0]], [[77.0, 7.0], [125.0, -45.0], [6.0, 13.0], [-56.0, 86.0], [-50.0, -69.0]], [[-16.0, 93.0], [-34.0, 25.0], [-5.0, -64.0], [-61.0, 81.0], [37.0, -40.0]], [[-46.0, 1.0], [56.0, -18.0], [-7.0, 1.0], [57.0, -14.0], [31.0, -33.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank2_expandedData_rank2_offset0(self):
      arg0=numpy.array([[-5.0, 0.0, -4.0, 1.0, -2.0], [-1.0, 2.0, 5.0, -3.0, -3.0], [-1.0, 7.0, 6.0, 2.0, -2.0], [-3.0, 3.0, -3.0, -4.0, -4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, 0.0, 3.0, 6.0, -7.0], [-4.0, 0.0, 0.0, 4.0, 0.0], [6.0, 3.0, 4.0, 0.0, -7.0], [-3.0, -1.0, 6.0, -3.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-6.0, -7.0, 7.0, -1.0, -4.0], [0.0, 1.0, -1.0, 0.0, -1.0], [-4.0, 0.0, 0.0, -1.0, 0.0], [-1.0, -3.0, 5.0, -7.0, 1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-25.0, 0.0, -15.0, -30.0, 35.0], [20.0, 0.0, 0.0, -20.0, 0.0], [-30.0, -15.0, -20.0, 0.0, 35.0], [15.0, 5.0, -30.0, 15.0, 20.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-20.0, 0.0, -12.0, -24.0, 28.0], [16.0, 0.0, 0.0, -16.0, 0.0], [-24.0, -12.0, -16.0, 0.0, 28.0], [12.0, 4.0, -24.0, 12.0, 16.0]], [[5.0, 0.0, 3.0, 6.0, -7.0], [-4.0, 0.0, 0.0, 4.0, 0.0], [6.0, 3.0, 4.0, 0.0, -7.0], [-3.0, -1.0, 6.0, -3.0, -4.0]], [[-10.0, 0.0, -6.0, -12.0, 14.0], [8.0, 0.0, 0.0, -8.0, 0.0], [-12.0, -6.0, -8.0, 0.0, 14.0], [6.0, 2.0, -12.0, 6.0, 8.0]]], [[[-5.0, 0.0, -3.0, -6.0, 7.0], [4.0, 0.0, 0.0, -4.0, 0.0], [-6.0, -3.0, -4.0, 0.0, 7.0], [3.0, 1.0, -6.0, 3.0, 4.0]], [[10.0, 0.0, 6.0, 12.0, -14.0], [-8.0, 0.0, 0.0, 8.0, 0.0], [12.0, 6.0, 8.0, 0.0, -14.0], [-6.0, -2.0, 12.0, -6.0, -8.0]], [[25.0, 0.0, 15.0, 30.0, -35.0], [-20.0, 0.0, 0.0, 20.0, 0.0], [30.0, 15.0, 20.0, 0.0, -35.0], [-15.0, -5.0, 30.0, -15.0, -20.0]], [[-15.0, 0.0, -9.0, -18.0, 21.0], [12.0, 0.0, 0.0, -12.0, 0.0], [-18.0, -9.0, -12.0, 0.0, 21.0], [9.0, 3.0, -18.0, 9.0, 12.0]], [[-15.0, 0.0, -9.0, -18.0, 21.0], [12.0, 0.0, 0.0, -12.0, 0.0], [-18.0, -9.0, -12.0, 0.0, 21.0], [9.0, 3.0, -18.0, 9.0, 12.0]]], [[[-5.0, 0.0, -3.0, -6.0, 7.0], [4.0, 0.0, 0.0, -4.0, 0.0], [-6.0, -3.0, -4.0, 0.0, 7.0], [3.0, 1.0, -6.0, 3.0, 4.0]], [[35.0, 0.0, 21.0, 42.0, -49.0], [-28.0, 0.0, 0.0, 28.0, 0.0], [42.0, 21.0, 28.0, 0.0, -49.0], [-21.0, -7.0, 42.0, -21.0, -28.0]], [[30.0, 0.0, 18.0, 36.0, -42.0], [-24.0, 0.0, 0.0, 24.0, 0.0], [36.0, 18.0, 24.0, 0.0, -42.0], [-18.0, -6.0, 36.0, -18.0, -24.0]], [[10.0, 0.0, 6.0, 12.0, -14.0], [-8.0, 0.0, 0.0, 8.0, 0.0], [12.0, 6.0, 8.0, 0.0, -14.0], [-6.0, -2.0, 12.0, -6.0, -8.0]], [[-10.0, 0.0, -6.0, -12.0, 14.0], [8.0, 0.0, 0.0, -8.0, 0.0], [-12.0, -6.0, -8.0, 0.0, 14.0], [6.0, 2.0, -12.0, 6.0, 8.0]]], [[[-15.0, 0.0, -9.0, -18.0, 21.0], [12.0, 0.0, 0.0, -12.0, 0.0], [-18.0, -9.0, -12.0, 0.0, 21.0], [9.0, 3.0, -18.0, 9.0, 12.0]], [[15.0, 0.0, 9.0, 18.0, -21.0], [-12.0, 0.0, 0.0, 12.0, 0.0], [18.0, 9.0, 12.0, 0.0, -21.0], [-9.0, -3.0, 18.0, -9.0, -12.0]], [[-15.0, 0.0, -9.0, -18.0, 21.0], [12.0, 0.0, 0.0, -12.0, 0.0], [-18.0, -9.0, -12.0, 0.0, 21.0], [9.0, 3.0, -18.0, 9.0, 12.0]], [[-20.0, 0.0, -12.0, -24.0, 28.0], [16.0, 0.0, 0.0, -16.0, 0.0], [-24.0, -12.0, -16.0, 0.0, 28.0], [12.0, 4.0, -24.0, 12.0, 16.0]], [[-20.0, 0.0, -12.0, -24.0, 28.0], [16.0, 0.0, 0.0, -16.0, 0.0], [-24.0, -12.0, -16.0, 0.0, 28.0], [12.0, 4.0, -24.0, 12.0, 16.0]]]])+(1.-msk_ref)*numpy.array([[[[30.0, 35.0, -35.0, 5.0, 20.0], [0.0, -5.0, 5.0, 0.0, 5.0], [20.0, 0.0, 0.0, 5.0, 0.0], [5.0, 15.0, -25.0, 35.0, -5.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[24.0, 28.0, -28.0, 4.0, 16.0], [0.0, -4.0, 4.0, 0.0, 4.0], [16.0, 0.0, 0.0, 4.0, 0.0], [4.0, 12.0, -20.0, 28.0, -4.0]], [[-6.0, -7.0, 7.0, -1.0, -4.0], [0.0, 1.0, -1.0, 0.0, -1.0], [-4.0, 0.0, 0.0, -1.0, 0.0], [-1.0, -3.0, 5.0, -7.0, 1.0]], [[12.0, 14.0, -14.0, 2.0, 8.0], [0.0, -2.0, 2.0, 0.0, 2.0], [8.0, 0.0, 0.0, 2.0, 0.0], [2.0, 6.0, -10.0, 14.0, -2.0]]], [[[6.0, 7.0, -7.0, 1.0, 4.0], [0.0, -1.0, 1.0, 0.0, 1.0], [4.0, 0.0, 0.0, 1.0, 0.0], [1.0, 3.0, -5.0, 7.0, -1.0]], [[-12.0, -14.0, 14.0, -2.0, -8.0], [0.0, 2.0, -2.0, 0.0, -2.0], [-8.0, 0.0, 0.0, -2.0, 0.0], [-2.0, -6.0, 10.0, -14.0, 2.0]], [[-30.0, -35.0, 35.0, -5.0, -20.0], [0.0, 5.0, -5.0, 0.0, -5.0], [-20.0, 0.0, 0.0, -5.0, 0.0], [-5.0, -15.0, 25.0, -35.0, 5.0]], [[18.0, 21.0, -21.0, 3.0, 12.0], [0.0, -3.0, 3.0, 0.0, 3.0], [12.0, 0.0, 0.0, 3.0, 0.0], [3.0, 9.0, -15.0, 21.0, -3.0]], [[18.0, 21.0, -21.0, 3.0, 12.0], [0.0, -3.0, 3.0, 0.0, 3.0], [12.0, 0.0, 0.0, 3.0, 0.0], [3.0, 9.0, -15.0, 21.0, -3.0]]], [[[6.0, 7.0, -7.0, 1.0, 4.0], [0.0, -1.0, 1.0, 0.0, 1.0], [4.0, 0.0, 0.0, 1.0, 0.0], [1.0, 3.0, -5.0, 7.0, -1.0]], [[-42.0, -49.0, 49.0, -7.0, -28.0], [0.0, 7.0, -7.0, 0.0, -7.0], [-28.0, 0.0, 0.0, -7.0, 0.0], [-7.0, -21.0, 35.0, -49.0, 7.0]], [[-36.0, -42.0, 42.0, -6.0, -24.0], [0.0, 6.0, -6.0, 0.0, -6.0], [-24.0, 0.0, 0.0, -6.0, 0.0], [-6.0, -18.0, 30.0, -42.0, 6.0]], [[-12.0, -14.0, 14.0, -2.0, -8.0], [0.0, 2.0, -2.0, 0.0, -2.0], [-8.0, 0.0, 0.0, -2.0, 0.0], [-2.0, -6.0, 10.0, -14.0, 2.0]], [[12.0, 14.0, -14.0, 2.0, 8.0], [0.0, -2.0, 2.0, 0.0, 2.0], [8.0, 0.0, 0.0, 2.0, 0.0], [2.0, 6.0, -10.0, 14.0, -2.0]]], [[[18.0, 21.0, -21.0, 3.0, 12.0], [0.0, -3.0, 3.0, 0.0, 3.0], [12.0, 0.0, 0.0, 3.0, 0.0], [3.0, 9.0, -15.0, 21.0, -3.0]], [[-18.0, -21.0, 21.0, -3.0, -12.0], [0.0, 3.0, -3.0, 0.0, -3.0], [-12.0, 0.0, 0.0, -3.0, 0.0], [-3.0, -9.0, 15.0, -21.0, 3.0]], [[18.0, 21.0, -21.0, 3.0, 12.0], [0.0, -3.0, 3.0, 0.0, 3.0], [12.0, 0.0, 0.0, 3.0, 0.0], [3.0, 9.0, -15.0, 21.0, -3.0]], [[24.0, 28.0, -28.0, 4.0, 16.0], [0.0, -4.0, 4.0, 0.0, 4.0], [16.0, 0.0, 0.0, 4.0, 0.0], [4.0, 12.0, -20.0, 28.0, -4.0]], [[24.0, 28.0, -28.0, 4.0, 16.0], [0.0, -4.0, 4.0, 0.0, 4.0], [16.0, 0.0, 0.0, 4.0, 0.0], [4.0, 12.0, -20.0, 28.0, -4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank3_offset1(self):
      arg0=numpy.array([[[-3.0, -2.0, -1.0], [-5.0, 0.0, -2.0], [2.0, 3.0, 4.0], [-2.0, 2.0, 1.0], [4.0, 5.0, -1.0]], [[-3.0, 5.0, 6.0], [7.0, -3.0, -4.0], [-7.0, 4.0, 1.0], [3.0, -4.0, -1.0], [-7.0, 3.0, -3.0]], [[6.0, 4.0, -2.0], [-6.0, 5.0, 7.0], [2.0, 6.0, 4.0], [3.0, 1.0, 0.0], [5.0, 5.0, -6.0]], [[1.0, 5.0, -5.0], [-1.0, -2.0, 4.0], [-6.0, 4.0, 6.0], [4.0, 7.0, 5.0], [7.0, -4.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, -1.0, -7.0], [2.0, 1.0, -3.0], [6.0, -2.0, 2.0], [1.0, 3.0, -1.0], [1.0, 4.0, 0.0]], [[-1.0, -6.0, 3.0], [5.0, -6.0, 1.0], [-2.0, 2.0, 5.0], [3.0, -6.0, 0.0], [-6.0, 7.0, 0.0]], [[5.0, -1.0, 6.0], [0.0, -2.0, -3.0], [3.0, -1.0, 1.0], [4.0, 4.0, -2.0], [-2.0, 2.0, -4.0]], [[-2.0, 7.0, -6.0], [6.0, -2.0, -4.0], [-5.0, -2.0, -1.0], [-4.0, -5.0, 4.0], [-1.0, -5.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 5.0, 1.0], [5.0, 6.0, 4.0], [-1.0, 5.0, 4.0], [-5.0, 1.0, 0.0], [-2.0, 0.0, -7.0]], [[-2.0, 3.0, -7.0], [2.0, 7.0, 2.0], [7.0, 2.0, -1.0], [-5.0, 0.0, 1.0], [3.0, 2.0, 3.0]], [[0.0, -1.0, 5.0], [6.0, 7.0, 3.0], [2.0, 1.0, -1.0], [3.0, -7.0, -5.0], [-3.0, 6.0, 3.0]], [[-7.0, 7.0, 0.0], [-1.0, 0.0, -3.0], [3.0, 0.0, -3.0], [7.0, -1.0, -4.0], [-1.0, -2.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, -5.0, -16.0, -8.0, -11.0], [12.0, -4.0, -3.0, 3.0, 4.0], [-19.0, 7.0, -8.0, -18.0, 6.0], [-2.0, -10.0, 20.0, 18.0, 20.0]], [[-6.0, -4.0, -34.0, -3.0, -5.0], [-1.0, -27.0, 0.0, -15.0, 30.0], [-37.0, 6.0, -17.0, -16.0, 18.0], [22.0, -22.0, 27.0, 12.0, 19.0]], [[-23.0, -5.0, 14.0, 7.0, 14.0], [-8.0, -4.0, 22.0, -12.0, 9.0], [31.0, -18.0, 7.0, 12.0, -14.0], [-7.0, -10.0, -20.0, -7.0, -45.0]], [[-17.0, -5.0, -14.0, 3.0, 6.0], [-7.0, -21.0, 13.0, -18.0, 26.0], [-6.0, -7.0, -7.0, -2.0, 4.0], [12.0, -20.0, 5.0, 2.0, -15.0]], [[18.0, 16.0, 12.0, 20.0, 24.0], [-37.0, -11.0, -3.0, -18.0, 11.0], [9.0, -7.0, 6.0, 38.0, 6.0], [33.0, 18.0, -29.0, -45.0, -22.0]]], [[[-59.0, -19.0, -16.0, 6.0, 17.0], [-9.0, -39.0, 46.0, -39.0, 53.0], [16.0, -28.0, -8.0, -4.0, -8.0], [5.0, -52.0, -1.0, 11.0, -64.0]], [[59.0, 23.0, 40.0, 2.0, -5.0], [-1.0, 49.0, -40.0, 39.0, -63.0], [14.0, 18.0, 20.0, 24.0, -4.0], [-11.0, 64.0, -25.0, -29.0, 36.0]], [[-39.0, -13.0, -48.0, 4.0, 9.0], [-14.0, -58.0, 27.0, -45.0, 70.0], [-33.0, -11.0, -24.0, -14.0, 18.0], [36.0, -54.0, 26.0, 12.0, -20.0]], [[23.0, 5.0, 24.0, -8.0, -13.0], [18.0, 38.0, -19.0, 33.0, -46.0], [13.0, 11.0, 12.0, -2.0, -10.0], [-28.0, 30.0, -6.0, 4.0, 24.0]], [[-10.0, -2.0, -54.0, 5.0, 5.0], [-20.0, -56.0, 5.0, -39.0, 63.0], [-56.0, 3.0, -27.0, -10.0, 32.0], [53.0, -36.0, 32.0, 1.0, 13.0]]], [[[34.0, 22.0, 24.0, 20.0, 22.0], [-36.0, 4.0, -14.0, -6.0, -8.0], [14.0, -2.0, 12.0, 44.0, 4.0], [28.0, 36.0, -36.0, -52.0, -12.0]], [[-78.0, -28.0, -32.0, 2.0, 14.0], [-3.0, -53.0, 57.0, -48.0, 71.0], [7.0, -31.0, -16.0, -18.0, -6.0], [5.0, -74.0, 13.0, 27.0, -68.0]], [[-26.0, -2.0, 8.0, 16.0, 26.0], [-26.0, -22.0, 28.0, -30.0, 30.0], [28.0, -24.0, 4.0, 24.0, -8.0], [14.0, -16.0, -26.0, -22.0, -60.0]], [[11.0, 7.0, 16.0, 6.0, 7.0], [-9.0, 9.0, -4.0, 3.0, -11.0], [14.0, -2.0, 8.0, 16.0, -4.0], [1.0, 16.0, -17.0, -17.0, -8.0]], [[57.0, 33.0, 8.0, 26.0, 25.0], [-53.0, -11.0, -30.0, -15.0, 5.0], [-16.0, 8.0, 4.0, 52.0, 24.0], [61.0, 44.0, -29.0, -69.0, 12.0]]], [[[34.0, 22.0, -14.0, 21.0, 21.0], [-46.0, -30.0, -17.0, -27.0, 29.0], [-30.0, 5.0, -7.0, 34.0, 28.0], [63.0, 16.0, -10.0, -49.0, 9.0]], [[-30.0, -16.0, 6.0, -11.0, -9.0], [25.0, 11.0, 18.0, 9.0, -8.0], [21.0, -8.0, 3.0, -20.0, -18.0], [-36.0, -18.0, 5.0, 30.0, -17.0]], [[-70.0, -26.0, -32.0, 0.0, 10.0], [0.0, -48.0, 50.0, -42.0, 64.0], [2.0, -26.0, -16.0, -20.0, -4.0], [4.0, -68.0, 16.0, 28.0, -56.0]], [[-26.0, 0.0, 20.0, 20.0, 32.0], [-31.0, -17.0, 31.0, -30.0, 25.0], [43.0, -29.0, 10.0, 34.0, -14.0], [11.0, -10.0, -39.0, -31.0, -74.0]], [[74.0, 28.0, 38.0, 1.0, -9.0], [-1.0, 53.0, -52.0, 45.0, -70.0], [3.0, 26.0, 19.0, 24.0, 2.0], [-6.0, 74.0, -21.0, -32.0, 55.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, -31.0, -11.0, 13.0, 13.0], [7.0, -22.0, -24.0, 14.0, -16.0], [-3.0, -35.0, -7.0, 10.0, -6.0], [7.0, 6.0, -6.0, -15.0, 13.0]], [[28.0, -33.0, -3.0, 25.0, 24.0], [24.0, -14.0, -33.0, 23.0, -21.0], [-10.0, -36.0, -8.0, -5.0, 9.0], [35.0, 11.0, -9.0, -27.0, 17.0]], [[7.0, 44.0, 29.0, -7.0, -32.0], [-23.0, 33.0, 16.0, -6.0, 24.0], [17.0, 45.0, 3.0, -35.0, 24.0], [7.0, -14.0, -6.0, -5.0, -32.0]], [[23.0, 6.0, 16.0, 12.0, -3.0], [3.0, 12.0, -11.0, 11.0, 1.0], [3.0, 5.0, -3.0, -25.0, 21.0], [28.0, -1.0, -9.0, -20.0, -8.0]], [[0.0, 46.0, 17.0, -15.0, -1.0], [14.0, 41.0, 39.0, -21.0, 19.0], [-10.0, 56.0, 14.0, -18.0, 15.0], [7.0, -1.0, 15.0, 27.0, -8.0]]], [[[49.0, 39.0, 52.0, 20.0, -36.0], [-21.0, 41.0, -17.0, 21.0, 19.0], [25.0, 35.0, -7.0, -74.0, 57.0], [56.0, -15.0, -27.0, -50.0, -43.0]], [[-61.0, 1.0, -38.0, -38.0, 14.0], [5.0, -15.0, 47.0, -39.0, 3.0], [-17.0, 9.0, 15.0, 62.0, -51.0], [-70.0, 5.0, 33.0, 68.0, 23.0]], [[63.0, -7.0, 31.0, 39.0, 7.0], [19.0, 16.0, -42.0, 36.0, -10.0], [1.0, -11.0, -11.0, -54.0, 48.0], [77.0, 4.0, -24.0, -57.0, -7.0]], [[-39.0, -13.0, -27.0, -19.0, 1.0], [-11.0, -24.0, 14.0, -16.0, -2.0], [-1.0, -13.0, 3.0, 42.0, -36.0], [-49.0, 0.0, 12.0, 29.0, 11.0]], [[54.0, -29.0, 10.0, 38.0, 35.0], [44.0, 1.0, -40.0, 32.0, -24.0], [-18.0, -30.0, -8.0, -27.0, 30.0], [70.0, 16.0, -12.0, -40.0, 19.0]]], [[[-18.0, 46.0, 6.0, -26.0, 2.0], [14.0, 36.0, 52.0, -32.0, 20.0], [-14.0, 58.0, 18.0, 0.0, 0.0], [-14.0, 0.0, 24.0, 46.0, -2.0]], [[68.0, 28.0, 59.0, 35.0, -37.0], [-22.0, 37.0, -39.0, 37.0, 13.0], [30.0, 20.0, -14.0, -88.0, 69.0], [77.0, -15.0, -39.0, -75.0, -46.0]], [[22.0, 62.0, 44.0, -4.0, -32.0], [-14.0, 54.0, 22.0, -6.0, 30.0], [14.0, 66.0, 6.0, -56.0, 42.0], [28.0, -14.0, -6.0, -8.0, -38.0]], [[-13.0, 21.0, 2.0, -14.0, -6.0], [-3.0, 13.0, 23.0, -15.0, 11.0], [-1.0, 25.0, 7.0, 2.0, -3.0], [-14.0, -3.0, 9.0, 20.0, -5.0]], [[-11.0, 31.0, -4.0, -20.0, 32.0], [47.0, 33.0, 51.0, -31.0, 7.0], [-35.0, 47.0, 21.0, 10.0, -3.0], [0.0, 13.0, 33.0, 54.0, 21.0]]], [[[14.0, 15.0, 4.0, 0.0, 33.0], [48.0, 27.0, 22.0, -10.0, -2.0], [-30.0, 26.0, 12.0, -7.0, 12.0], [28.0, 14.0, 18.0, 22.0, 19.0]], [[0.0, -1.0, 7.0, 3.0, -26.0], [-32.0, -8.0, -15.0, 9.0, 5.0], [22.0, -8.0, -8.0, -9.0, 3.0], [-7.0, -11.0, -15.0, -21.0, -19.0]], [[62.0, 18.0, 50.0, 34.0, -30.0], [-18.0, 28.0, -40.0, 36.0, 8.0], [26.0, 10.0, -14.0, -76.0, 60.0], [70.0, -12.0, -36.0, -70.0, -38.0]], [[16.0, 82.0, 51.0, -13.0, -43.0], [-22.0, 67.0, 37.0, -15.0, 41.0], [18.0, 88.0, 10.0, -62.0, 45.0], [21.0, -19.0, -3.0, 1.0, -48.0]], [[-68.0, -13.0, -51.0, -39.0, 28.0], [16.0, -26.0, 47.0, -41.0, -5.0], [-26.0, -4.0, 16.0, 79.0, -63.0], [-77.0, 11.0, 39.0, 77.0, 37.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank4_offset2(self):
      arg0=numpy.array([[[[4.0, 6.0, 0.0], [5.0, -7.0, 3.0]], [[0.0, 1.0, 6.0], [-3.0, 0.0, 4.0]], [[-6.0, -3.0, 5.0], [-4.0, -6.0, -4.0]], [[-7.0, -6.0, -7.0], [-2.0, 5.0, -5.0]], [[-3.0, 4.0, 1.0], [6.0, -2.0, 7.0]]], [[[-1.0, -6.0, 5.0], [-2.0, 3.0, 0.0]], [[-6.0, 0.0, 3.0], [-5.0, -4.0, -3.0]], [[0.0, -2.0, 2.0], [-4.0, 1.0, -1.0]], [[3.0, 5.0, -1.0], [2.0, 1.0, -2.0]], [[-5.0, -3.0, 7.0], [6.0, 3.0, 0.0]]], [[[6.0, 3.0, 1.0], [7.0, -3.0, -7.0]], [[6.0, -7.0, 1.0], [0.0, -5.0, -4.0]], [[0.0, -1.0, 6.0], [5.0, 1.0, 2.0]], [[2.0, 5.0, 0.0], [-7.0, -3.0, -1.0]], [[3.0, 3.0, 1.0], [2.0, 0.0, -1.0]]], [[[3.0, -5.0, 5.0], [7.0, -5.0, 7.0]], [[1.0, 3.0, 1.0], [0.0, 5.0, 0.0]], [[5.0, -4.0, 1.0], [5.0, 0.0, 1.0]], [[1.0, 7.0, -4.0], [4.0, 3.0, -3.0]], [[-6.0, 0.0, -1.0], [-7.0, 2.0, -1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -5.0, 2.0], [-1.0, 0.0, -5.0]], [[-6.0, -1.0, 6.0], [-5.0, 0.0, 4.0]], [[7.0, 5.0, 6.0], [0.0, 3.0, -5.0]], [[-1.0, -1.0, 2.0], [0.0, 0.0, -3.0]], [[2.0, -6.0, -4.0], [-6.0, 1.0, 2.0]]], [[[1.0, -3.0, -1.0], [4.0, -3.0, 2.0]], [[3.0, -5.0, 1.0], [2.0, 1.0, 7.0]], [[-2.0, 0.0, -2.0], [0.0, 5.0, 6.0]], [[1.0, 0.0, -7.0], [0.0, -6.0, 3.0]], [[-6.0, -5.0, -1.0], [-4.0, 3.0, 2.0]]], [[[7.0, 7.0, 1.0], [0.0, 1.0, 6.0]], [[-3.0, -2.0, 0.0], [6.0, 4.0, -4.0]], [[5.0, -7.0, -1.0], [7.0, 0.0, 1.0]], [[-3.0, 5.0, -4.0], [-7.0, -7.0, 1.0]], [[-6.0, 7.0, 0.0], [-7.0, 6.0, -1.0]]], [[[7.0, -1.0, 1.0], [2.0, 7.0, 5.0]], [[2.0, 1.0, -7.0], [-3.0, 4.0, -2.0]], [[3.0, 6.0, 0.0], [0.0, 0.0, -3.0]], [[7.0, -3.0, 6.0], [0.0, -1.0, -1.0]], [[3.0, 1.0, 7.0], [5.0, -4.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[2.0, 0.0, 7.0], [-5.0, 2.0, 4.0]], [[-1.0, -4.0, 4.0], [7.0, 3.0, 0.0]], [[-5.0, -2.0, -3.0], [-5.0, -1.0, -1.0]], [[6.0, -5.0, -7.0], [4.0, 3.0, -2.0]], [[3.0, -6.0, -5.0], [0.0, -3.0, 3.0]]], [[[-3.0, 2.0, 7.0], [-5.0, 4.0, 0.0]], [[-1.0, -4.0, -2.0], [4.0, -3.0, -6.0]], [[-1.0, 0.0, 6.0], [-7.0, 3.0, 5.0]], [[-3.0, 0.0, -6.0], [6.0, -2.0, 4.0]], [[0.0, -7.0, 0.0], [5.0, 2.0, -2.0]]], [[[-2.0, 2.0, 3.0], [7.0, -2.0, 2.0]], [[-4.0, 1.0, 5.0], [4.0, -2.0, 7.0]], [[-4.0, 0.0, 6.0], [-6.0, 3.0, 0.0]], [[-1.0, 0.0, 7.0], [2.0, -4.0, -6.0]], [[0.0, -4.0, 4.0], [4.0, 4.0, -6.0]]], [[[3.0, -1.0, 3.0], [7.0, 3.0, -2.0]], [[5.0, -6.0, 2.0], [5.0, 7.0, -3.0]], [[-7.0, -6.0, -1.0], [3.0, -1.0, -6.0]], [[-4.0, 0.0, 7.0], [-4.0, 0.0, 5.0]], [[2.0, -4.0, -1.0], [6.0, -7.0, 3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-70.0, -43.0, 22.0, -19.0, -59.0], [33.0, 6.0, -25.0, 55.0, -89.0], [81.0, -34.0, 16.0, 35.0, -62.0], [-2.0, -35.0, 39.0, 14.0, 53.0]], [[-10.0, 66.0, 21.0, -1.0, -4.0], [-13.0, 23.0, 12.0, -30.0, 9.0], [37.0, -36.0, -30.0, 6.0, 24.0], [19.0, -40.0, -6.0, 29.0, 4.0]], [[79.0, 73.0, -25.0, 31.0, -4.0], [-8.0, -40.0, -52.0, -17.0, 36.0], [-88.0, -8.0, -46.0, 49.0, 11.0], [-104.0, -54.0, -24.0, 7.0, 42.0]], [[78.0, -4.0, -81.0, 14.0, 57.0], [-15.0, -32.0, 23.0, -3.0, 92.0], [-123.0, 61.0, -5.0, -7.0, 49.0], [-44.0, 65.0, -42.0, -73.0, -76.0]], [[-44.0, 18.0, -36.0, -20.0, -58.0], [28.0, 31.0, 36.0, 23.0, -19.0], [48.0, 1.0, 5.0, 4.0, -15.0], [9.0, -49.0, -6.0, -32.0, -2.0]]], [[[47.0, 52.0, 2.0, 17.0, 29.0], [-5.0, 31.0, 7.0, -54.0, 48.0], [-41.0, 15.0, 18.0, -54.0, -4.0], [21.0, -25.0, -39.0, 38.0, 4.0]], [[56.0, 67.0, -21.0, 21.0, -4.0], [-23.0, -50.0, -32.0, -12.0, 35.0], [-61.0, -16.0, -71.0, 66.0, 50.0], [-92.0, -28.0, -9.0, -17.0, 12.0]], [[23.0, 30.0, 10.0, 9.0, 27.0], [-17.0, -2.0, -5.0, -23.0, 25.0], [-17.0, -12.0, -17.0, 2.0, 21.0], [-2.0, 2.0, -9.0, 18.0, -6.0]], [[-34.0, -47.0, 53.0, -4.0, -35.0], [-10.0, -26.0, -11.0, -2.0, -51.0], [44.0, 5.0, -7.0, -3.0, 11.0], [16.0, 20.0, 45.0, 1.0, 25.0]], [[48.0, 45.0, 1.0, 22.0, -53.0], [12.0, 22.0, 11.0, -72.0, 23.0], [-46.0, 69.0, 31.0, -91.0, -15.0], [8.0, -68.0, -33.0, 13.0, 49.0]]], [[[-15.0, -96.0, 89.0, 14.0, -69.0], [19.0, -34.0, -71.0, -4.0, -103.0], [19.0, 34.0, 50.0, -42.0, -75.0], [-2.0, -11.0, 57.0, 49.0, 117.0]], [[27.0, -39.0, 18.0, 15.0, 37.0], [33.0, 21.0, -63.0, 17.0, -25.0], [-35.0, -8.0, 74.0, -26.0, -111.0], [-5.0, -14.0, -12.0, 78.0, 62.0]], [[2.0, 20.0, 24.0, 7.0, -43.0], [18.0, 36.0, 5.0, -42.0, -14.0], [12.0, 28.0, 38.0, -69.0, -38.0], [34.0, -58.0, -12.0, 36.0, 50.0]], [[-23.0, 14.0, 35.0, -4.0, 11.0], [-34.0, -43.0, -25.0, 17.0, -20.0], [40.0, -66.0, -75.0, 88.0, 55.0], [-31.0, 20.0, 39.0, 3.0, -6.0]], [[-25.0, -29.0, 47.0, -1.0, -30.0], [-1.0, -8.0, -14.0, -7.0, -44.0], [37.0, 1.0, 6.0, -13.0, -10.0], [18.0, -2.0, 30.0, 19.0, 35.0]]], [[[-22.0, 10.0, -24.0, -9.0, -17.0], [70.0, 97.0, 1.0, 19.0, -27.0], [28.0, -5.0, 101.0, -61.0, -139.0], [45.0, -89.0, -42.0, 64.0, 52.0]], [[-18.0, -3.0, 43.0, -2.0, -15.0], [-24.0, -6.0, 21.0, -36.0, -7.0], [34.0, 11.0, -17.0, -27.0, 45.0], [40.0, 18.0, 21.0, -1.0, -7.0]], [[-13.0, -41.0, 16.0, -2.0, 2.0], [38.0, 53.0, -6.0, 1.0, -29.0], [14.0, 19.0, 88.0, -73.0, -94.0], [55.0, -18.0, -12.0, 52.0, 37.0]], [[-37.0, -69.0, 42.0, -7.0, -51.0], [-15.0, -46.0, 3.0, 2.0, -50.0], [37.0, 31.0, -15.0, -4.0, 36.0], [10.0, 43.0, 54.0, -38.0, 8.0]], [[40.0, 61.0, -37.0, 7.0, 34.0], [-41.0, -38.0, 18.0, -14.0, 69.0], [-47.0, -12.0, -79.0, 56.0, 98.0], [-48.0, 26.0, -15.0, -49.0, -62.0]]]])+(1.-msk_ref)*numpy.array([[[[-19.0, -14.0, -53.0, -13.0, 6.0], [-53.0, -5.0, -45.0, 44.0, -37.0], [59.0, 45.0, -67.0, 16.0, -50.0], [14.0, -49.0, -60.0, -21.0, 72.0]], [[73.0, -1.0, -9.0, -67.0, -24.0], [59.0, -52.0, 77.0, -38.0, -30.0], [7.0, 47.0, 54.0, 12.0, -16.0], [-12.0, -21.0, -45.0, 74.0, -16.0]], [[15.0, -8.0, 51.0, -82.0, -19.0], [43.0, 34.0, 26.0, -40.0, -3.0], [-3.0, 14.0, 60.0, 81.0, 16.0], [-38.0, -52.0, 73.0, 55.0, 1.0]], [[-63.0, 4.0, 78.0, 54.0, 20.0], [-10.0, 52.0, -31.0, 21.0, 52.0], [-53.0, -66.0, 13.0, -36.0, 38.0], [-25.0, 27.0, 111.0, -38.0, -45.0]], [[-5.0, 27.0, -31.0, -41.0, -11.0], [-14.0, -27.0, -4.0, 71.0, -16.0], [77.0, 98.0, -24.0, -12.0, -38.0], [12.0, -42.0, -26.0, 30.0, 48.0]]], [[[49.0, 40.0, 9.0, -10.0, -1.0], [48.0, -2.0, 54.0, -45.0, 38.0], [-15.0, 9.0, 55.0, 20.0, 48.0], [13.0, 52.0, 29.0, 47.0, -16.0]], [[14.0, -29.0, 53.0, -83.0, -30.0], [48.0, 10.0, 32.0, -34.0, -27.0], [-12.0, 6.0, 60.0, 51.0, -6.0], [-50.0, -68.0, 46.0, 50.0, -26.0]], [[32.0, -9.0, 18.0, -15.0, -4.0], [34.0, -9.0, 38.0, -42.0, -2.0], [-30.0, -17.0, 39.0, 8.0, 10.0], [-15.0, 6.0, 3.0, 25.0, -28.0]], [[-17.0, -10.0, -31.0, 15.0, -25.0], [-12.0, -4.0, -30.0, -1.0, -19.0], [9.0, -20.0, -27.0, 2.0, 0.0], [22.0, 6.0, -33.0, -37.0, -14.0]], [[15.0, 96.0, -23.0, -31.0, -41.0], [40.0, 18.0, 14.0, 3.0, 57.0], [61.0, 70.0, 35.0, 54.0, 76.0], [60.0, 58.0, 61.0, 45.0, 10.0]]], [[[-50.0, 26.0, -64.0, 47.0, -17.0], [-52.0, 59.0, -93.0, -4.0, 22.0], [38.0, -31.0, -69.0, 69.0, 50.0], [72.0, 49.0, 5.0, -80.0, 41.0]], [[-7.0, 11.0, -10.0, 57.0, 58.0], [-45.0, 59.0, -35.0, -30.0, 47.0], [-21.0, -44.0, -33.0, 45.0, 36.0], [21.0, 51.0, 28.0, -37.0, 62.0]], [[27.0, 66.0, -44.0, -18.0, -21.0], [19.0, -3.0, 14.0, 0.0, 30.0], [53.0, 61.0, 9.0, 36.0, 40.0], [53.0, 44.0, 2.0, 32.0, 27.0]], [[29.0, -80.0, 19.0, -48.0, -18.0], [27.0, -35.0, 33.0, -46.0, -74.0], [-39.0, -32.0, 25.0, 2.0, -54.0], [-55.0, -73.0, -56.0, 15.0, -40.0]], [[-1.0, 3.0, -33.0, 6.0, -17.0], [-6.0, -3.0, -16.0, -7.0, -9.0], [15.0, -3.0, -18.0, 14.0, 6.0], [25.0, 12.0, -28.0, -18.0, 2.0]]], [[[24.0, 71.0, -57.0, 7.0, 50.0], [-39.0, 8.0, -2.0, 41.0, 46.0], [72.0, 95.0, -39.0, 24.0, 6.0], [49.0, 34.0, -12.0, 30.0, 119.0]], [[19.0, 6.0, -19.0, -1.0, -35.0], [30.0, -30.0, 20.0, -19.0, -11.0], [-3.0, -6.0, 17.0, -14.0, 12.0], [18.0, 24.0, -31.0, 3.0, -46.0]], [[-4.0, 50.0, -46.0, 61.0, 37.0], [-41.0, 23.0, -29.0, 13.0, 51.0], [22.0, 8.0, -44.0, 6.0, 34.0], [55.0, 73.0, -3.0, -28.0, 58.0]], [[-52.0, -8.0, -27.0, 30.0, -37.0], [-25.0, 4.0, -59.0, 27.0, -17.0], [16.0, -28.0, -43.0, -15.0, 2.0], [27.0, 5.0, -18.0, -63.0, -28.0]], [[16.0, -41.0, 67.0, -49.0, -22.0], [54.0, -20.0, 50.0, -26.0, -29.0], [-46.0, -20.0, 66.0, -17.0, -18.0], [-62.0, -50.0, 26.0, 40.0, -70.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_float_rank0_offset0(self):
      arg0=numpy.array([[[7.0, 0.0], [3.0, -3.0]], [[-1.0, -2.0], [7.0, 7.0]], [[4.0, 6.0], [-5.0, -1.0]], [[-2.0, 0.0], [7.0, -5.0]], [[0.0, 0.0], [-4.0, 7.0]], [[0.0, 2.0], [-2.0, 6.0]]])
      arg1=1.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[7.0, 0.0], [3.0, -3.0]], [[-1.0, -2.0], [7.0, 7.0]], [[4.0, 6.0], [-5.0, -1.0]], [[-2.0, 0.0], [7.0, -5.0]], [[0.0, 0.0], [-4.0, 7.0]], [[0.0, 2.0], [-2.0, 6.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank0_offset0(self):
      arg0=numpy.array([[[-3.0, 3.0], [0.0, 1.0]], [[1.0, 1.0], [-5.0, -5.0]], [[2.0, -7.0], [6.0, 5.0]], [[5.0, 3.0], [-6.0, -2.0]], [[6.0, -2.0], [6.0, 5.0]], [[0.0, 3.0], [4.0, 0.0]]])
      arg1=numpy.array(4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[-12.0, 12.0], [0.0, 4.0]], [[4.0, 4.0], [-20.0, -20.0]], [[8.0, -28.0], [24.0, 20.0]], [[20.0, 12.0], [-24.0, -8.0]], [[24.0, -8.0], [24.0, 20.0]], [[0.0, 12.0], [16.0, 0.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank1_offset1(self):
      arg0=numpy.array([[[[-2.0, 3.0, -3.0], [4.0, 5.0, 5.0]], [[2.0, 3.0, 6.0], [-3.0, -1.0, -3.0]]], [[[6.0, -7.0, -3.0], [4.0, -1.0, -6.0]], [[-1.0, 4.0, 0.0], [0.0, -2.0, 2.0]]], [[[6.0, -4.0, -3.0], [0.0, -2.0, 5.0]], [[5.0, -1.0, 6.0], [0.0, 6.0, 6.0]]], [[[2.0, -2.0, -6.0], [-6.0, 6.0, -2.0]], [[2.0, 6.0, 6.0], [-3.0, -7.0, -3.0]]], [[[2.0, -4.0, -6.0], [-5.0, 3.0, -7.0]], [[6.0, 3.0, 1.0], [0.0, 0.0, 3.0]]], [[[-5.0, -3.0, -7.0], [2.0, 0.0, -3.0]], [[3.0, 3.0, -3.0], [-3.0, -5.0, 3.0]]]])
      arg1=numpy.array([-1.0, 6.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[2.0, 56.0], [52.0, -21.0]], [[-66.0, -46.0], [25.0, 0.0]], [[-48.0, 18.0], [25.0, 72.0]], [[-50.0, 30.0], [70.0, -57.0]], [[-62.0, -19.0], [18.0, 18.0]], [[-55.0, -20.0], [-3.0, -9.0]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_array_rank1_offset0(self):
      arg0=numpy.array([[[-1.0, -1.0], [-6.0, 7.0]], [[2.0, 2.0], [1.0, 0.0]], [[-3.0, 2.0], [3.0, -5.0]], [[-7.0, -2.0], [-5.0, 5.0]], [[-7.0, 0.0], [3.0, -6.0]], [[-4.0, 1.0], [-5.0, -3.0]]])
      arg1=numpy.array([2.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[-2.0, -7.0], [-2.0, -7.0]], [[-12.0, -42.0], [14.0, 49.0]]], [[[4.0, 14.0], [4.0, 14.0]], [[2.0, 7.0], [0.0, 0.0]]], [[[-6.0, -21.0], [4.0, 14.0]], [[6.0, 21.0], [-10.0, -35.0]]], [[[-14.0, -49.0], [-4.0, -14.0]], [[-10.0, -35.0], [10.0, 35.0]]], [[[-14.0, -49.0], [0.0, 0.0]], [[6.0, 21.0], [-12.0, -42.0]]], [[[-8.0, -28.0], [2.0, 7.0]], [[-10.0, -35.0], [-6.0, -21.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank2_offset1(self):
      arg0=numpy.array([[[[4.0, -6.0, -1.0], [4.0, 6.0, 0.0]], [[-4.0, -3.0, 3.0], [0.0, -2.0, -2.0]]], [[[-2.0, -7.0, 4.0], [-1.0, 3.0, -5.0]], [[7.0, -7.0, -6.0], [0.0, 2.0, -3.0]]], [[[-2.0, -7.0, 0.0], [-1.0, 0.0, 1.0]], [[4.0, -1.0, 2.0], [7.0, -5.0, 4.0]]], [[[-2.0, -6.0, -1.0], [6.0, 2.0, 5.0]], [[-6.0, -4.0, -3.0], [2.0, -5.0, 0.0]]], [[[-6.0, -2.0, -3.0], [0.0, -1.0, 2.0]], [[5.0, 7.0, -1.0], [-7.0, 4.0, 0.0]]], [[[7.0, -1.0, 7.0], [3.0, -3.0, -2.0]], [[-5.0, 3.0, 6.0], [-5.0, -7.0, 1.0]]]])
      arg1=numpy.array([[-5.0, -6.0, -7.0], [-4.0, -1.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=numpy.array([[[[23.0, -10.0], [-56.0, -22.0]], [[17.0, 19.0], [26.0, 2.0]]], [[[24.0, 15.0], [22.0, 1.0]], [[49.0, -21.0], [9.0, -2.0]]], [[[52.0, 15.0], [-2.0, 4.0]], [[-28.0, -15.0], [-33.0, -23.0]]], [[[53.0, 14.0], [-77.0, -26.0]], [[75.0, 28.0], [20.0, -3.0]]], [[[63.0, 26.0], [-8.0, 1.0]], [[-60.0, -27.0], [11.0, 24.0]]], [[[-78.0, -27.0], [17.0, -9.0]], [[-35.0, 17.0], [60.0, 27.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[-1.0, -5.0], [0.0, -5.0]], [[-2.0, -1.0], [0.0, 4.0]], [[2.0, -7.0], [4.0, 7.0]], [[7.0, -7.0], [3.0, -6.0]], [[-6.0, -3.0], [3.0, 0.0]], [[2.0, -2.0], [6.0, 3.0]]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-2.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[2.0, 10.0], [-0.0, 10.0]], [[4.0, 2.0], [-0.0, -8.0]], [[-4.0, 14.0], [-8.0, -14.0]], [[-14.0, 14.0], [-6.0, 12.0]], [[12.0, 6.0], [-6.0, -0.0]], [[-4.0, 4.0], [-12.0, -6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank1_offset1(self):
      arg0=numpy.array([[[[-3.0, 4.0, 0.0], [7.0, -3.0, 7.0]], [[0.0, -1.0, 2.0], [-7.0, 0.0, 2.0]]], [[[-2.0, 1.0, 1.0], [-6.0, -5.0, 5.0]], [[4.0, -5.0, 7.0], [-2.0, -4.0, 5.0]]], [[[-4.0, -1.0, -1.0], [0.0, -3.0, 0.0]], [[0.0, 0.0, -1.0], [6.0, -6.0, 7.0]]], [[[-2.0, 6.0, -5.0], [-1.0, 7.0, -7.0]], [[-5.0, -7.0, 4.0], [6.0, 6.0, 0.0]]], [[[-3.0, -6.0, 0.0], [-4.0, 4.0, 5.0]], [[-4.0, 6.0, -3.0], [-4.0, -5.0, 3.0]]], [[[-5.0, 5.0, 3.0], [1.0, -6.0, -4.0]], [[3.0, -6.0, 4.0], [0.0, -4.0, -3.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-7.0, -5.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[1.0, 8.0], [17.0, 61.0]], [[15.0, 97.0], [39.0, 64.0]], [[27.0, 15.0], [-6.0, 30.0]], [[-46.0, -70.0], [94.0, -72.0]], [[51.0, 38.0], [-20.0, 71.0]], [[28.0, -1.0], [33.0, 2.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_Symbol_rank1_offset0(self):
      arg0=numpy.array([[[6.0, 5.0], [-3.0, -4.0]], [[0.0, 1.0], [5.0, 7.0]], [[2.0, -1.0], [-4.0, 6.0]], [[-7.0, -4.0], [0.0, 0.0]], [[3.0, -4.0], [5.0, 7.0]], [[1.0, 7.0], [-1.0, -1.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([7.0, 0.0])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[42.0, 0.0], [35.0, 0.0]], [[-21.0, 0.0], [-28.0, 0.0]]], [[[0.0, 0.0], [7.0, 0.0]], [[35.0, 0.0], [49.0, 0.0]]], [[[14.0, 0.0], [-7.0, 0.0]], [[-28.0, 0.0], [42.0, 0.0]]], [[[-49.0, 0.0], [-28.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[21.0, 0.0], [-28.0, 0.0]], [[35.0, 0.0], [49.0, 0.0]]], [[[7.0, 0.0], [49.0, 0.0]], [[-7.0, 0.0], [-7.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank2_offset1(self):
      arg0=numpy.array([[[[4.0, 0.0, -3.0], [-7.0, 0.0, 6.0]], [[4.0, 7.0, 2.0], [-7.0, -2.0, 0.0]]], [[[0.0, 4.0, 1.0], [2.0, -2.0, -7.0]], [[-4.0, 6.0, -6.0], [5.0, 5.0, 0.0]]], [[[1.0, 7.0, -4.0], [-2.0, 7.0, -2.0]], [[4.0, 7.0, -1.0], [7.0, 6.0, 0.0]]], [[[4.0, 3.0, 5.0], [0.0, 5.0, 7.0]], [[5.0, -2.0, 2.0], [-4.0, 3.0, -7.0]]], [[[0.0, -4.0, -7.0], [-1.0, 2.0, -5.0]], [[-1.0, 0.0, 4.0], [7.0, -1.0, 2.0]]], [[[3.0, 0.0, 4.0], [-2.0, 5.0, -7.0]], [[-3.0, -3.0, 1.0], [6.0, -6.0, 4.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-2.0, 6.0, -6.0], [7.0, -3.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[10.0, 28.0], [-22.0, -49.0]], [[22.0, 7.0], [2.0, -43.0]]], [[[18.0, -12.0], [26.0, 20.0]], [[80.0, -46.0], [20.0, 20.0]]], [[[64.0, -14.0], [58.0, -35.0]], [[40.0, 7.0], [22.0, 31.0]]], [[[-20.0, 19.0], [-12.0, -15.0]], [[-34.0, 41.0], [68.0, -37.0]]], [[[18.0, 12.0], [44.0, -13.0]], [[-22.0, -7.0], [-32.0, 52.0]]], [[[-30.0, 21.0], [76.0, -29.0]], [[-18.0, -12.0], [-72.0, 60.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank0_offset0(self):
      arg0=numpy.array([[[-4.0, -7.0], [-1.0, -4.0]], [[0.0, -7.0], [0.0, 1.0]], [[-7.0, -2.0], [2.0, 3.0]], [[-3.0, -2.0], [4.0, 1.0]], [[-3.0, 1.0], [-2.0, -4.0]], [[1.0, -4.0], [-4.0, 4.0]]])
      arg1=Data(-5.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[20.0, 35.0], [5.0, 20.0]], [[-0.0, 35.0], [-0.0, -5.0]], [[35.0, 10.0], [-10.0, -15.0]], [[15.0, 10.0], [-20.0, -5.0]], [[15.0, -5.0], [10.0, 20.0]], [[-5.0, 20.0], [20.0, -20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank1_offset1(self):
      arg0=numpy.array([[[[-3.0, -3.0, 5.0], [6.0, -1.0, 3.0]], [[-6.0, -5.0, -3.0], [3.0, -7.0, -7.0]]], [[[-5.0, 1.0, -5.0], [-7.0, -1.0, -7.0]], [[-6.0, -1.0, 0.0], [1.0, -7.0, 3.0]]], [[[7.0, -3.0, 1.0], [-6.0, -4.0, 0.0]], [[0.0, -6.0, 1.0], [-2.0, -1.0, 4.0]]], [[[5.0, -6.0, 2.0], [-2.0, 5.0, 1.0]], [[-1.0, 6.0, -5.0], [-5.0, -3.0, -3.0]]], [[[5.0, -7.0, 6.0], [-3.0, -3.0, 7.0]], [[3.0, 2.0, 2.0], [-4.0, 7.0, -2.0]]], [[[-5.0, 0.0, -4.0], [0.0, 7.0, -5.0]], [[2.0, -7.0, -2.0], [3.0, 1.0, -4.0]]]])
      arg1=Data(numpy.array([-1.0, -7.0, 6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[54.0, 19.0], [23.0, 4.0]], [[-32.0, -28.0], [13.0, 66.0]], [[20.0, 34.0], [48.0, 33.0]], [[49.0, -27.0], [-71.0, 8.0]], [[80.0, 66.0], [-5.0, -57.0]], [[-19.0, -79.0], [35.0, -34.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_constData_rank1_offset0(self):
      arg0=numpy.array([[[-5.0, -5.0], [3.0, 2.0]], [[2.0, -7.0], [-2.0, 4.0]], [[2.0, -2.0], [-2.0, -5.0]], [[-6.0, 7.0], [-5.0, -7.0]], [[-5.0, 2.0], [3.0, 4.0]], [[-4.0, 5.0], [0.0, 0.0]]])
      arg1=Data(numpy.array([0.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank2_offset1(self):
      arg0=numpy.array([[[[-4.0, -6.0, 3.0], [4.0, 2.0, 6.0]], [[0.0, 5.0, -3.0], [7.0, -7.0, 0.0]]], [[[-3.0, 5.0, -7.0], [7.0, 1.0, -5.0]], [[-3.0, -6.0, 1.0], [1.0, 0.0, -5.0]]], [[[-7.0, -1.0, 3.0], [2.0, 4.0, -3.0]], [[2.0, 3.0, -3.0], [5.0, 3.0, 3.0]]], [[[-6.0, 4.0, 4.0], [5.0, -4.0, -1.0]], [[2.0, -2.0, -1.0], [5.0, 0.0, -4.0]]], [[[-5.0, -4.0, 2.0], [-2.0, -6.0, -6.0]], [[-7.0, -7.0, 3.0], [1.0, -1.0, -4.0]]], [[[-1.0, 0.0, -4.0], [1.0, -5.0, 5.0]], [[6.0, -1.0, 3.0], [6.0, 4.0, -6.0]]]])
      arg1=Data(numpy.array([[-3.0, 1.0, -2.0], [-6.0, -1.0, -5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[0.0, 15.0], [-22.0, -56.0]], [[11.0, 10.0], [-28.0, -35.0]]], [[[28.0, 48.0], [-10.0, -18.0]], [[1.0, 19.0], [7.0, 19.0]]], [[[14.0, 28.0], [4.0, -1.0]], [[3.0, 0.0], [-18.0, -48.0]]], [[[14.0, 12.0], [-17.0, -21.0]], [[-6.0, -5.0], [-7.0, -10.0]]], [[[7.0, 24.0], [12.0, 48.0]], [[8.0, 34.0], [4.0, 15.0]]], [[[11.0, 26.0], [-18.0, -26.0]], [[-25.0, -50.0], [-2.0, -10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[2.0, -5.0], [5.0, 4.0]], [[-1.0, 3.0], [-4.0, 4.0]], [[2.0, -1.0], [-4.0, 1.0]], [[0.0, 7.0], [0.0, 1.0]], [[0.0, 7.0], [4.0, 4.0]], [[-2.0, -3.0], [-2.0, -1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, -5.0], [5.0, 4.0]], [[-1.0, 3.0], [-4.0, 4.0]], [[2.0, -1.0], [-4.0, 1.0]], [[0.0, 7.0], [0.0, 1.0]], [[0.0, 7.0], [4.0, 4.0]], [[-2.0, -3.0], [-2.0, -1.0]]])+(1.-msk_ref)*numpy.array([[[14.0, -35.0], [35.0, 28.0]], [[-7.0, 21.0], [-28.0, 28.0]], [[14.0, -7.0], [-28.0, 7.0]], [[0.0, 49.0], [0.0, 7.0]], [[0.0, 49.0], [28.0, 28.0]], [[-14.0, -21.0], [-14.0, -7.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank1_offset1(self):
      arg0=numpy.array([[[[-2.0, -2.0, -5.0], [-7.0, 5.0, 6.0]], [[2.0, -3.0, 0.0], [0.0, -7.0, 1.0]]], [[[2.0, 3.0, 3.0], [-4.0, -7.0, -1.0]], [[6.0, 1.0, 1.0], [0.0, 0.0, -4.0]]], [[[2.0, -4.0, 0.0], [-6.0, 3.0, 7.0]], [[-2.0, 0.0, 4.0], [7.0, 1.0, -1.0]]], [[[-6.0, -2.0, -5.0], [2.0, -4.0, -4.0]], [[0.0, -3.0, 0.0], [-2.0, 7.0, 2.0]]], [[[-5.0, -1.0, -5.0], [-1.0, 1.0, -4.0]], [[-1.0, -5.0, -2.0], [2.0, -1.0, 5.0]]], [[[6.0, 6.0, -5.0], [-2.0, 2.0, -4.0]], [[-2.0, 4.0, 5.0], [4.0, 3.0, 7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-1.0, 0.0, 3.0])+(1.-msk_arg1)*numpy.array([-2.0, 7.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-13.0, 25.0], [-2.0, 3.0]], [[7.0, 1.0], [-3.0, -12.0]], [[-2.0, 27.0], [14.0, -10.0]], [[-9.0, -14.0], [0.0, 8.0]], [[-10.0, -11.0], [-5.0, 13.0]], [[-21.0, -10.0], [17.0, 17.0]]])+(1.-msk_ref)*numpy.array([[[20.0, 13.0], [-25.0, -55.0]], [[-1.0, -35.0], [-11.0, 24.0]], [[-32.0, -9.0], [-20.0, -1.0]], [[28.0, -8.0], [-21.0, 41.0]], [[33.0, 33.0], [-21.0, -41.0]], [[60.0, 42.0], [2.0, -29.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank3_expandedData_rank1_offset0(self):
      arg0=numpy.array([[[-1.0, -7.0], [5.0, 2.0]], [[3.0, -3.0], [-4.0, -2.0]], [[7.0, 2.0], [-5.0, 0.0]], [[2.0, -5.0], [3.0, 7.0]], [[1.0, 4.0], [-1.0, -7.0]], [[4.0, -5.0], [7.0, 5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-3.0, -6.0])+(1.-msk_arg1)*numpy.array([0.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 6.0], [21.0, 42.0]], [[-15.0, -30.0], [-6.0, -12.0]]], [[[-9.0, -18.0], [9.0, 18.0]], [[12.0, 24.0], [6.0, 12.0]]], [[[-21.0, -42.0], [-6.0, -12.0]], [[15.0, 30.0], [0.0, 0.0]]], [[[-6.0, -12.0], [15.0, 30.0]], [[-9.0, -18.0], [-21.0, -42.0]]], [[[-3.0, -6.0], [-12.0, -24.0]], [[3.0, 6.0], [21.0, 42.0]]], [[[-12.0, -24.0], [15.0, 30.0]], [[-21.0, -42.0], [-15.0, -30.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank2_offset1(self):
      arg0=numpy.array([[[[-6.0, 0.0, -2.0], [-4.0, -5.0, -1.0]], [[-7.0, -4.0, 3.0], [0.0, 6.0, -4.0]]], [[[4.0, -4.0, -4.0], [3.0, -2.0, -2.0]], [[-7.0, -3.0, 1.0], [-7.0, 5.0, 0.0]]], [[[5.0, 0.0, -1.0], [6.0, -2.0, 0.0]], [[7.0, -7.0, -1.0], [4.0, 7.0, 2.0]]], [[[-1.0, 4.0, -2.0], [2.0, 3.0, 2.0]], [[-5.0, 2.0, -1.0], [-7.0, 0.0, 7.0]]], [[[-7.0, -1.0, -2.0], [0.0, -1.0, -7.0]], [[6.0, 6.0, 5.0], [-4.0, 6.0, -5.0]]], [[[2.0, 0.0, 4.0], [1.0, -2.0, 4.0]], [[-2.0, 7.0, -2.0], [-6.0, 0.0, 5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 7.0, 6.0], [-4.0, -1.0, 0.0]])+(1.-msk_arg1)*numpy.array([[2.0, 3.0, 7.0], [-2.0, -4.0, 4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[12.0, 24.0], [-25.0, 21.0]], [[18.0, 32.0], [18.0, -6.0]]], [[[-68.0, -12.0], [-38.0, -10.0]], [[13.0, 31.0], [63.0, 23.0]]], [[[-26.0, -20.0], [-38.0, -22.0]], [[-83.0, -21.0], [45.0, -23.0]]], [[[20.0, 0.0], [25.0, -11.0]], [[28.0, 18.0], [70.0, 28.0]]], [[[9.0, 29.0], [-49.0, 1.0]], [[48.0, -30.0], [28.0, 10.0]]], [[[16.0, -8.0], [6.0, -2.0]], [[45.0, 1.0], [54.0, 24.0]]]])+(1.-msk_ref)*numpy.array([[[[-26.0, 4.0], [-30.0, 24.0]], [[-5.0, 42.0], [-10.0, -40.0]]], [[[-32.0, -8.0], [-14.0, -6.0]], [[-16.0, 30.0], [1.0, -6.0]]], [[[3.0, -14.0], [6.0, -4.0]], [[-14.0, 10.0], [43.0, -28.0]]], [[[-4.0, -22.0], [27.0, -8.0]], [[-11.0, -2.0], [35.0, 42.0]]], [[[-31.0, 10.0], [-52.0, -24.0]], [[65.0, -16.0], [-25.0, -36.0]]], [[[32.0, 12.0], [24.0, 22.0]], [[3.0, -32.0], [23.0, 32.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_float_rank0_offset0(self):
      arg0=numpy.array([[[[5.0, -1.0, -1.0, 1.0], [-1.0, 3.0, 5.0, 7.0], [-2.0, 0.0, -6.0, 3.0]], [[-3.0, -4.0, 0.0, -6.0], [-5.0, 0.0, -7.0, -1.0], [7.0, 7.0, 1.0, 2.0]]], [[[5.0, -7.0, -2.0, -4.0], [7.0, -1.0, -4.0, 7.0], [-7.0, 3.0, -5.0, 0.0]], [[-4.0, 6.0, 1.0, -2.0], [2.0, 4.0, -2.0, -6.0], [6.0, 1.0, 7.0, 7.0]]], [[[3.0, -7.0, 7.0, -1.0], [-1.0, 7.0, -3.0, 2.0], [-2.0, 0.0, 0.0, 2.0]], [[2.0, -5.0, 0.0, 1.0], [7.0, -4.0, 0.0, -4.0], [5.0, -5.0, 1.0, 0.0]]]])
      arg1=1.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[5.0, -1.0, -1.0, 1.0], [-1.0, 3.0, 5.0, 7.0], [-2.0, 0.0, -6.0, 3.0]], [[-3.0, -4.0, 0.0, -6.0], [-5.0, 0.0, -7.0, -1.0], [7.0, 7.0, 1.0, 2.0]]], [[[5.0, -7.0, -2.0, -4.0], [7.0, -1.0, -4.0, 7.0], [-7.0, 3.0, -5.0, 0.0]], [[-4.0, 6.0, 1.0, -2.0], [2.0, 4.0, -2.0, -6.0], [6.0, 1.0, 7.0, 7.0]]], [[[3.0, -7.0, 7.0, -1.0], [-1.0, 7.0, -3.0, 2.0], [-2.0, 0.0, 0.0, 2.0]], [[2.0, -5.0, 0.0, 1.0], [7.0, -4.0, 0.0, -4.0], [5.0, -5.0, 1.0, 0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_array_rank0_offset0(self):
      arg0=numpy.array([[[[-1.0, 2.0, -4.0, -5.0], [3.0, 1.0, -7.0, -2.0], [-1.0, 0.0, 2.0, 5.0]], [[0.0, 7.0, -6.0, 3.0], [-4.0, 7.0, -5.0, 3.0], [-6.0, 0.0, -6.0, 2.0]]], [[[-5.0, 0.0, -7.0, -1.0], [-2.0, -6.0, -3.0, 7.0], [-1.0, -6.0, -7.0, 1.0]], [[5.0, 3.0, 4.0, 1.0], [-6.0, 5.0, 0.0, 0.0], [-2.0, 1.0, 6.0, -2.0]]], [[[4.0, -7.0, 4.0, -7.0], [3.0, -2.0, 2.0, -2.0], [4.0, -6.0, 6.0, 5.0]], [[4.0, 1.0, -2.0, 1.0], [0.0, 6.0, 6.0, 3.0], [0.0, -7.0, -4.0, 0.0]]]])
      arg1=numpy.array(-5.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=numpy.array([[[[5.0, -10.0, 20.0, 25.0], [-15.0, -5.0, 35.0, 10.0], [5.0, -0.0, -10.0, -25.0]], [[-0.0, -35.0, 30.0, -15.0], [20.0, -35.0, 25.0, -15.0], [30.0, -0.0, 30.0, -10.0]]], [[[25.0, -0.0, 35.0, 5.0], [10.0, 30.0, 15.0, -35.0], [5.0, 30.0, 35.0, -5.0]], [[-25.0, -15.0, -20.0, -5.0], [30.0, -25.0, -0.0, -0.0], [10.0, -5.0, -30.0, 10.0]]], [[[-20.0, 35.0, -20.0, 35.0], [-15.0, 10.0, -10.0, 10.0], [-20.0, 30.0, -30.0, -25.0]], [[-20.0, -5.0, 10.0, -5.0], [-0.0, -30.0, -30.0, -15.0], [-0.0, 35.0, 20.0, -0.0]]]])
      self.assertTrue(isinstance(res,numpy.ndarray),"wrong type of result.")
      self.assertEqual(res.shape,(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_Symbol_rank0_offset0(self):
      arg0=numpy.array([[[[-5.0, 7.0, 5.0, -5.0], [3.0, -4.0, 5.0, -6.0], [-7.0, -7.0, -6.0, 0.0]], [[0.0, 6.0, 0.0, 2.0], [-3.0, 2.0, -2.0, -3.0], [-1.0, 3.0, 1.0, 3.0]]], [[[0.0, 4.0, 6.0, 5.0], [5.0, 0.0, -2.0, 3.0], [-2.0, 5.0, 6.0, 0.0]], [[-1.0, -4.0, -1.0, -6.0], [4.0, -7.0, -7.0, -7.0], [3.0, 0.0, 5.0, -7.0]]], [[[-5.0, 2.0, 0.0, -7.0], [1.0, 4.0, -7.0, 2.0], [0.0, 0.0, -5.0, -5.0]], [[-1.0, 0.0, 0.0, 3.0], [-5.0, -7.0, 2.0, -2.0], [-1.0, 5.0, -2.0, -2.0]]]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg1:s1})
      ref=numpy.array([[[[-35.0, 49.0, 35.0, -35.0], [21.0, -28.0, 35.0, -42.0], [-49.0, -49.0, -42.0, 0.0]], [[0.0, 42.0, 0.0, 14.0], [-21.0, 14.0, -14.0, -21.0], [-7.0, 21.0, 7.0, 21.0]]], [[[0.0, 28.0, 42.0, 35.0], [35.0, 0.0, -14.0, 21.0], [-14.0, 35.0, 42.0, 0.0]], [[-7.0, -28.0, -7.0, -42.0], [28.0, -49.0, -49.0, -49.0], [21.0, 0.0, 35.0, -49.0]]], [[[-35.0, 14.0, 0.0, -49.0], [7.0, 28.0, -49.0, 14.0], [0.0, 0.0, -35.0, -35.0]], [[-7.0, 0.0, 0.0, 21.0], [-35.0, -49.0, 14.0, -14.0], [-7.0, 35.0, -14.0, -14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_constData_rank0_offset0(self):
      arg0=numpy.array([[[[-4.0, -6.0, -1.0, 4.0], [-7.0, 1.0, 5.0, 2.0], [-4.0, 1.0, 6.0, 0.0]], [[5.0, 2.0, -4.0, -6.0], [0.0, -6.0, 2.0, -6.0], [2.0, -2.0, -6.0, 0.0]]], [[[5.0, 4.0, -6.0, -7.0], [-7.0, 3.0, -1.0, 6.0], [-1.0, -4.0, 2.0, -5.0]], [[5.0, -2.0, -1.0, 3.0], [2.0, 2.0, 4.0, 6.0], [6.0, 0.0, -6.0, -5.0]]], [[[-3.0, 5.0, -3.0, 6.0], [-5.0, -2.0, 6.0, 7.0], [-3.0, 1.0, -1.0, -7.0]], [[5.0, 3.0, -1.0, 0.0], [-4.0, -7.0, -5.0, 6.0], [-2.0, -7.0, 6.0, 2.0]]]])
      arg1=Data(0.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, -0.0, -0.0], [0.0, -0.0, 0.0, -0.0], [0.0, -0.0, -0.0, 0.0]]], [[[0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, -0.0]], [[0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, -0.0]], [[0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_array_rank4_expandedData_rank0_offset0(self):
      arg0=numpy.array([[[[0.0, -5.0, -5.0, 3.0], [2.0, -1.0, -1.0, -7.0], [-2.0, 6.0, -2.0, -7.0]], [[2.0, -1.0, 3.0, -6.0], [5.0, -4.0, 2.0, 0.0], [3.0, -5.0, -5.0, 3.0]]], [[[-4.0, -4.0, 7.0, 6.0], [-3.0, 0.0, -1.0, 3.0], [-1.0, -5.0, -7.0, 0.0]], [[-6.0, -5.0, -7.0, -6.0], [-4.0, -7.0, 0.0, 7.0], [6.0, -1.0, -6.0, -3.0]]], [[[-3.0, 3.0, 0.0, -4.0], [2.0, -7.0, -7.0, 4.0], [-1.0, -3.0, -5.0, -4.0]], [[-6.0, -2.0, 0.0, 4.0], [7.0, 0.0, -7.0, -7.0], [5.0, 0.0, 4.0, -7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-4.0)+(1-msk_arg1)*(-1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, 20.0, 20.0, -12.0], [-8.0, 4.0, 4.0, 28.0], [8.0, -24.0, 8.0, 28.0]], [[-8.0, 4.0, -12.0, 24.0], [-20.0, 16.0, -8.0, -0.0], [-12.0, 20.0, 20.0, -12.0]]], [[[16.0, 16.0, -28.0, -24.0], [12.0, -0.0, 4.0, -12.0], [4.0, 20.0, 28.0, -0.0]], [[24.0, 20.0, 28.0, 24.0], [16.0, 28.0, -0.0, -28.0], [-24.0, 4.0, 24.0, 12.0]]], [[[12.0, -12.0, -0.0, 16.0], [-8.0, 28.0, 28.0, -16.0], [4.0, 12.0, 20.0, 16.0]], [[24.0, 8.0, -0.0, -16.0], [-28.0, -0.0, 28.0, 28.0], [-20.0, -0.0, -16.0, 28.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, 5.0, 5.0, -3.0], [-2.0, 1.0, 1.0, 7.0], [2.0, -6.0, 2.0, 7.0]], [[-2.0, 1.0, -3.0, 6.0], [-5.0, 4.0, -2.0, -0.0], [-3.0, 5.0, 5.0, -3.0]]], [[[4.0, 4.0, -7.0, -6.0], [3.0, -0.0, 1.0, -3.0], [1.0, 5.0, 7.0, -0.0]], [[6.0, 5.0, 7.0, 6.0], [4.0, 7.0, -0.0, -7.0], [-6.0, 1.0, 6.0, 3.0]]], [[[3.0, -3.0, -0.0, 4.0], [-2.0, 7.0, 7.0, -4.0], [1.0, 3.0, 5.0, 4.0]], [[6.0, 2.0, -0.0, -4.0], [-7.0, -0.0, 7.0, 7.0], [-5.0, -0.0, -4.0, 7.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_float_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=-1.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(7.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_array_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array(-7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(5.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-35.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([5.0, 1.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-6.0, 3.0, 4.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-27.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[-4.0, -4.0, -2.0], [-3.0, 1.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-6.0, 0.0, 6.0], [6.0, -7.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-48.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[-6.0, -7.0, 6.0], [6.0, 5.0, -2.0], [3.0, 0.0, -4.0], [-6.0, -3.0, 1.0]], [[-5.0, -6.0, 6.0], [6.0, 7.0, 7.0], [7.0, 6.0, 0.0], [-7.0, -3.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-1.0, -6.0, 6.0], [-3.0, -7.0, 0.0], [0.0, 2.0, -1.0], [-3.0, 3.0, 6.0]], [[3.0, -1.0, -6.0], [6.0, 1.0, 0.0], [1.0, 4.0, -2.0], [2.0, 7.0, 3.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(41.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=numpy.array([[[[5.0, -7.0], [0.0, -2.0], [0.0, 5.0]], [[-2.0, -4.0], [-7.0, -3.0], [-6.0, -5.0]]], [[[4.0, -2.0], [-3.0, -5.0], [0.0, -3.0]], [[2.0, -1.0], [-1.0, 7.0], [3.0, -5.0]]], [[[6.0, 0.0], [-5.0, -2.0], [-3.0, 0.0]], [[-5.0, 2.0], [-1.0, -2.0], [-7.0, 7.0]]], [[[7.0, 0.0], [7.0, 0.0], [4.0, 1.0]], [[-7.0, 6.0], [2.0, 3.0], [6.0, 5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[1.0, 2.0], [3.0, 7.0], [5.0, -5.0]], [[1.0, 3.0], [-3.0, 7.0], [-4.0, 0.0]]], [[[-7.0, 2.0], [7.0, -1.0], [-6.0, 1.0]], [[-4.0, 6.0], [0.0, 4.0], [0.0, 5.0]]], [[[-5.0, 1.0], [-2.0, 1.0], [5.0, 4.0]], [[-6.0, 0.0], [-7.0, -3.0], [-6.0, 6.0]]], [[[-5.0, 1.0], [4.0, 7.0], [-4.0, -7.0]], [[-2.0, 4.0], [1.0, -5.0], [-2.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array(-62.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_array_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([-6.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-42.0, 0.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[-4.0, 5.0, 2.0], [-3.0, 7.0, -1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([5.0, 6.0, 5.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([20.0, 22.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[6.0, -1.0, -3.0], [4.0, -3.0, -5.0]], [[-5.0, 3.0, -4.0], [6.0, 6.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-6.0, -6.0, 7.0], [0.0, 0.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-46.0, -16.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=numpy.array([[[[1.0, -4.0, 1.0], [0.0, 0.0, 7.0], [0.0, -4.0, 3.0], [-1.0, -4.0, 0.0]], [[0.0, 7.0, 6.0], [-5.0, 3.0, 6.0], [-5.0, -6.0, -2.0], [-1.0, -1.0, 0.0]]], [[[-6.0, 5.0, 6.0], [0.0, 1.0, 0.0], [-1.0, 3.0, 5.0], [4.0, -2.0, 7.0]], [[-7.0, -4.0, -2.0], [-7.0, 1.0, 7.0], [-2.0, 3.0, 0.0], [2.0, 5.0, 7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-1.0, -1.0, -4.0], [2.0, 0.0, -4.0], [7.0, 0.0, 5.0], [0.0, 5.0, -5.0]], [[2.0, -2.0, -2.0], [4.0, 2.0, -1.0], [7.0, 3.0, -1.0], [-7.0, 5.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-129.0, -114.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_array_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[-1.0, 3.0, 4.0, 0.0, 1.0], [-6.0, 3.0, 5.0, 6.0, 1.0], [2.0, 5.0, 2.0, -4.0, -4.0], [0.0, -3.0, 0.0, 1.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-0.0, 0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0, -0.0], [0.0, -0.0, 0.0, 0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[-6.0, -5.0, 4.0], [0.0, 6.0, 6.0], [-2.0, -1.0, 4.0], [3.0, 0.0, 1.0], [1.0, 4.0, 0.0]], [[0.0, 6.0, 4.0], [3.0, 1.0, -6.0], [5.0, 0.0, 1.0], [6.0, 2.0, 5.0], [-2.0, -4.0, 1.0]], [[-6.0, -3.0, 1.0], [1.0, -7.0, 3.0], [-7.0, -6.0, -6.0], [3.0, 2.0, 0.0], [0.0, -5.0, 6.0]], [[5.0, -2.0, 5.0], [3.0, 2.0, -3.0], [-7.0, 0.0, -5.0], [5.0, 1.0, -5.0], [5.0, -2.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, 3.0, 7.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[31.0, 60.0, 31.0, -2.0, 9.0], [46.0, -48.0, -8.0, 23.0, 1.0], [16.0, -3.0, -39.0, -3.0, 27.0], [14.0, -24.0, -14.0, -47.0, -70.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[[-4.0, -2.0, -7.0], [0.0, 2.0, -1.0]], [[-6.0, 1.0, -6.0], [4.0, 3.0, -2.0]], [[-5.0, 7.0, -7.0], [5.0, 7.0, -4.0]], [[7.0, -4.0, 4.0], [-5.0, -2.0, 7.0]], [[0.0, -4.0, 1.0], [0.0, -4.0, 4.0]]], [[[2.0, 0.0, -5.0], [7.0, 4.0, 0.0]], [[6.0, 0.0, 1.0], [-1.0, 1.0, 4.0]], [[2.0, 1.0, -6.0], [7.0, 0.0, 7.0]], [[-4.0, 1.0, 5.0], [-5.0, 3.0, 0.0]], [[-5.0, -5.0, 7.0], [-2.0, -6.0, -4.0]]], [[[-7.0, -4.0, 0.0], [7.0, 5.0, -1.0]], [[1.0, -6.0, 5.0], [0.0, 7.0, -7.0]], [[2.0, -5.0, 0.0], [-7.0, -7.0, -5.0]], [[-4.0, 4.0, 1.0], [2.0, -6.0, 0.0]], [[-6.0, -6.0, -7.0], [-7.0, -6.0, -7.0]]], [[[5.0, 2.0, 5.0], [-3.0, -3.0, 4.0]], [[0.0, -2.0, 4.0], [0.0, -4.0, 6.0]], [[5.0, 4.0, -4.0], [0.0, 7.0, 6.0]], [[-7.0, 0.0, 4.0], [-4.0, 4.0, 3.0]], [[-4.0, -5.0, 4.0], [0.0, 5.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, -3.0, 7.0], [-3.0, -2.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-18.0, -19.0, -60.0, 3.0, 23.0], [-78.0, -38.0, -87.0, 69.0, 121.0], [31.0, 39.0, 41.0, 29.0, 51.0], [5.0, 36.0, -95.0, 78.0, 66.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_array_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[-5.0, 5.0], [-1.0, -2.0]], [[3.0, -3.0], [0.0, 0.0]], [[7.0, 0.0], [-6.0, 2.0]], [[-2.0, -5.0], [-2.0, 5.0]], [[-5.0, 1.0], [7.0, 6.0]], [[4.0, -7.0], [6.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(1.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-5.0, 5.0], [-1.0, -2.0]], [[3.0, -3.0], [0.0, 0.0]], [[7.0, 0.0], [-6.0, 2.0]], [[-2.0, -5.0], [-2.0, 5.0]], [[-5.0, 1.0], [7.0, 6.0]], [[4.0, -7.0], [6.0, -1.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=numpy.array([[[[-3.0, -4.0, 1.0], [5.0, -4.0, -7.0]], [[-1.0, -4.0, 2.0], [-5.0, 6.0, -7.0]]], [[[4.0, 6.0, 0.0], [6.0, -2.0, -1.0]], [[-4.0, 0.0, 0.0], [-1.0, -1.0, 1.0]]], [[[-7.0, -4.0, -1.0], [-4.0, -6.0, 3.0]], [[1.0, 4.0, -2.0], [3.0, 0.0, -4.0]]], [[[3.0, 3.0, 6.0], [-6.0, 1.0, -3.0]], [[-7.0, -5.0, -3.0], [5.0, 1.0, 6.0]]], [[[-3.0, 2.0, 0.0], [3.0, -7.0, -4.0]], [[-2.0, 4.0, -3.0], [-3.0, 7.0, 3.0]]], [[[-4.0, -3.0, -1.0], [0.0, -4.0, -3.0]], [[-5.0, 0.0, -2.0], [-2.0, 7.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, -7.0, -1.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[42.0, 10.0], [31.0, -10.0]], [[-62.0, -15.0], [20.0, 11.0]], [[64.0, 59.0], [-31.0, -11.0]], [[-42.0, 26.0], [73.0, -38.0]], [[1.0, 38.0], [-15.0, -37.0]], [[42.0, 31.0], [27.0, -38.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_array_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=numpy.array([[[[6.0, 0.0, -3.0, 3.0], [3.0, 5.0, 5.0, -1.0], [1.0, -4.0, 6.0, 1.0]], [[-1.0, 4.0, 6.0, -2.0], [-1.0, -1.0, -3.0, -6.0], [-2.0, -7.0, -2.0, 0.0]]], [[[3.0, 4.0, 7.0, 0.0], [-2.0, 4.0, 5.0, 2.0], [6.0, 0.0, -7.0, 2.0]], [[-3.0, 6.0, 0.0, 4.0], [-3.0, -5.0, -3.0, 0.0], [-3.0, -1.0, 1.0, 2.0]]], [[[-7.0, -5.0, -5.0, 6.0], [-3.0, -6.0, -7.0, 0.0], [-6.0, -6.0, -2.0, -2.0]], [[5.0, -7.0, 7.0, 0.0], [-7.0, 1.0, -4.0, -7.0], [6.0, 0.0, 2.0, 3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-7.0)
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-42.0, -0.0, 21.0, -21.0], [-21.0, -35.0, -35.0, 7.0], [-7.0, 28.0, -42.0, -7.0]], [[7.0, -28.0, -42.0, 14.0], [7.0, 7.0, 21.0, 42.0], [14.0, 49.0, 14.0, -0.0]]], [[[-21.0, -28.0, -49.0, -0.0], [14.0, -28.0, -35.0, -14.0], [-42.0, -0.0, 49.0, -14.0]], [[21.0, -42.0, -0.0, -28.0], [21.0, 35.0, 21.0, -0.0], [21.0, 7.0, -7.0, -14.0]]], [[[49.0, 35.0, 35.0, -42.0], [21.0, 42.0, 49.0, -0.0], [42.0, 42.0, 14.0, 14.0]], [[-35.0, 49.0, -49.0, -0.0], [49.0, -7.0, 28.0, 49.0], [-42.0, -0.0, -14.0, -21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(7.0)
      s1=numpy.array(1.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(7.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 4.0, -3.0])
      s1=numpy.array([1.0, -5.0, 5.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-40.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[4.0, 0.0, -4.0], [-3.0, -5.0, 1.0]])
      s1=numpy.array([[7.0, 5.0, 2.0], [-5.0, 1.0, -6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(24.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[0.0, -5.0, 1.0], [4.0, 0.0, 4.0], [0.0, -3.0, -3.0], [4.0, 2.0, 7.0]], [[-5.0, -5.0, 5.0], [1.0, -2.0, -2.0], [-3.0, -2.0, 4.0], [-7.0, 2.0, 6.0]]])
      s1=numpy.array([[[-3.0, -6.0, -7.0], [-1.0, -3.0, 7.0], [-2.0, 7.0, 2.0], [-6.0, -4.0, 0.0]], [[6.0, -6.0, -7.0], [6.0, 1.0, 0.0], [5.0, -7.0, 1.0], [2.0, 0.0, 2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-42.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[0.0, 2.0], [4.0, 7.0], [-5.0, 5.0]], [[0.0, -4.0], [1.0, -1.0], [-5.0, 0.0]]], [[[4.0, 7.0], [-1.0, 6.0], [1.0, -4.0]], [[1.0, -3.0], [1.0, -3.0], [-5.0, -5.0]]], [[[3.0, 0.0], [-5.0, 0.0], [1.0, 0.0]], [[2.0, 3.0], [-5.0, -2.0], [-1.0, -1.0]]], [[[0.0, 0.0], [3.0, 2.0], [-4.0, -6.0]], [[0.0, -7.0], [0.0, -1.0], [-4.0, 1.0]]]])
      s1=numpy.array([[[[2.0, -2.0], [-1.0, -3.0], [-6.0, -7.0]], [[5.0, -2.0], [1.0, 5.0], [-3.0, -1.0]]], [[[-5.0, -6.0], [6.0, -2.0], [-7.0, -4.0]], [[-4.0, 4.0], [5.0, 1.0], [-5.0, 2.0]]], [[[0.0, -3.0], [-6.0, -3.0], [6.0, -2.0]], [[1.0, 2.0], [5.0, -6.0], [-7.0, -4.0]]], [[[-7.0, -6.0], [0.0, 6.0], [-2.0, -5.0]], [[-3.0, 7.0], [-5.0, -4.0], [4.0, 6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array(-48.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-2.0)
      s1=numpy.array([7.0, 6.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-14.0, -12.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([4.0, 5.0, 7.0])
      s1=numpy.array([[-7.0, 0.0, 4.0], [-2.0, -5.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([0.0, -82.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[0.0, -7.0, 5.0], [3.0, -5.0, 0.0]])
      s1=numpy.array([[[-6.0, 6.0, -4.0], [-7.0, -5.0, -5.0]], [[-5.0, 4.0, 1.0], [6.0, -2.0, 4.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-58.0, 5.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-2.0, -5.0, -4.0], [1.0, -2.0, 2.0], [-2.0, -2.0, -1.0], [6.0, -2.0, 3.0]], [[-6.0, 1.0, -3.0], [-5.0, 0.0, 3.0], [2.0, 0.0, 5.0], [5.0, 0.0, 0.0]]])
      s1=numpy.array([[[[0.0, -1.0, -1.0], [6.0, 0.0, -3.0], [4.0, -5.0, 6.0], [-3.0, -6.0, 0.0]], [[-7.0, 0.0, -5.0], [2.0, 1.0, 6.0], [2.0, -4.0, 1.0], [3.0, 0.0, -1.0]]], [[[5.0, 1.0, -4.0], [7.0, 5.0, -6.0], [-3.0, 1.0, 6.0], [2.0, 0.0, -6.0]], [[1.0, 0.0, -4.0], [0.0, -5.0, -7.0], [0.0, -4.0, 4.0], [-2.0, 0.0, -2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([88.0, -27.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(2.0)
      s1=numpy.array([[-5.0, -3.0, 7.0, -4.0, 7.0], [1.0, 4.0, -2.0, 1.0, 7.0], [-6.0, -3.0, 5.0, -7.0, 4.0], [-6.0, 3.0, -1.0, 0.0, -5.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-10.0, -6.0, 14.0, -8.0, 14.0], [2.0, 8.0, -4.0, 2.0, 14.0], [-12.0, -6.0, 10.0, -14.0, 8.0], [-12.0, 6.0, -2.0, 0.0, -10.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([4.0, 1.0, 6.0])
      s1=numpy.array([[[7.0, 5.0, -2.0], [-5.0, 0.0, 0.0], [1.0, 4.0, 7.0], [4.0, -6.0, -6.0], [-3.0, -4.0, 0.0]], [[-5.0, -4.0, -1.0], [5.0, 0.0, 2.0], [0.0, 0.0, -4.0], [0.0, -7.0, -5.0], [-6.0, 0.0, -4.0]], [[-2.0, -1.0, -5.0], [2.0, 5.0, -4.0], [-2.0, 4.0, -6.0], [-2.0, 5.0, 5.0], [0.0, 1.0, 3.0]], [[4.0, -3.0, 2.0], [4.0, -1.0, 0.0], [-1.0, -7.0, 2.0], [-3.0, 3.0, -6.0], [0.0, 2.0, 7.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[21.0, -20.0, 50.0, -26.0, -16.0], [-30.0, 32.0, -24.0, -37.0, -48.0], [-39.0, -11.0, -40.0, 27.0, 19.0], [25.0, 15.0, 1.0, -45.0, 44.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[1.0, 3.0, 0.0], [0.0, 2.0, 0.0]])
      s1=numpy.array([[[[-3.0, 3.0, -2.0], [1.0, 4.0, -7.0]], [[2.0, 1.0, 0.0], [-2.0, 5.0, 1.0]], [[2.0, 3.0, 1.0], [6.0, -4.0, -5.0]], [[3.0, 4.0, 4.0], [2.0, 6.0, -6.0]], [[-6.0, -5.0, 5.0], [7.0, -4.0, -5.0]]], [[[3.0, -4.0, -6.0], [2.0, 0.0, -5.0]], [[-1.0, 3.0, 5.0], [5.0, 7.0, 5.0]], [[-6.0, 0.0, 0.0], [-1.0, 1.0, -6.0]], [[-5.0, 5.0, -7.0], [5.0, -5.0, 1.0]], [[-7.0, 3.0, 5.0], [-5.0, -6.0, 7.0]]], [[[4.0, -2.0, -4.0], [-4.0, 0.0, -5.0]], [[7.0, 7.0, -7.0], [5.0, 7.0, 3.0]], [[3.0, 4.0, 5.0], [7.0, 4.0, -6.0]], [[-3.0, 6.0, 1.0], [-2.0, 6.0, 0.0]], [[-5.0, -6.0, 4.0], [5.0, 2.0, 3.0]]], [[[-1.0, 3.0, 1.0], [-1.0, 7.0, -5.0]], [[-4.0, -1.0, -2.0], [0.0, -3.0, -1.0]], [[6.0, -1.0, 5.0], [1.0, 2.0, 7.0]], [[-2.0, -2.0, 0.0], [0.0, -4.0, 5.0]], [[-6.0, -6.0, -3.0], [-7.0, -6.0, -1.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[14.0, 15.0, 3.0, 27.0, -29.0], [-9.0, 22.0, -4.0, 0.0, -10.0], [-2.0, 42.0, 23.0, 27.0, -19.0], [22.0, -13.0, 7.0, -16.0, -36.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(3.0)
      s1=numpy.array([[[3.0, 0.0], [5.0, 0.0]], [[-5.0, -2.0], [-1.0, -3.0]], [[1.0, -5.0], [-7.0, -2.0]], [[4.0, 3.0], [3.0, 2.0]], [[-3.0, -4.0], [-6.0, 3.0]], [[3.0, 5.0], [6.0, -2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[9.0, 0.0], [15.0, 0.0]], [[-15.0, -6.0], [-3.0, -9.0]], [[3.0, -15.0], [-21.0, -6.0]], [[12.0, 9.0], [9.0, 6.0]], [[-9.0, -12.0], [-18.0, 9.0]], [[9.0, 15.0], [18.0, -6.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([7.0, 4.0, 6.0])
      s1=numpy.array([[[[-1.0, 4.0, -6.0], [4.0, 6.0, 0.0]], [[0.0, -7.0, 0.0], [-5.0, -3.0, -4.0]]], [[[1.0, -2.0, -7.0], [-1.0, -7.0, -1.0]], [[2.0, 0.0, 7.0], [2.0, -4.0, -6.0]]], [[[-4.0, 0.0, 4.0], [-2.0, 6.0, -3.0]], [[4.0, -6.0, 5.0], [-5.0, 7.0, -5.0]]], [[[4.0, 1.0, -7.0], [-3.0, -1.0, -5.0]], [[0.0, -1.0, 5.0], [0.0, -2.0, -5.0]]], [[[-7.0, -5.0, 0.0], [-3.0, -4.0, 0.0]], [[2.0, -5.0, -4.0], [5.0, -5.0, -1.0]]], [[[-4.0, 0.0, 5.0], [-5.0, -2.0, -6.0]], [[-2.0, 0.0, 2.0], [7.0, -3.0, -3.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-27.0, 52.0], [-28.0, -71.0]], [[-43.0, -41.0], [56.0, -38.0]], [[-4.0, -8.0], [34.0, -37.0]], [[-10.0, -55.0], [26.0, -38.0]], [[-69.0, -37.0], [-30.0, 9.0]], [[2.0, -79.0], [-2.0, 19.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_Symbol_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-2.0)
      s1=numpy.array([[[[0.0, -3.0, 2.0, -2.0], [-1.0, 5.0, -5.0, 2.0], [6.0, -3.0, 5.0, 3.0]], [[-1.0, 1.0, 2.0, -2.0], [-2.0, -7.0, 1.0, 7.0], [4.0, -5.0, 7.0, -1.0]]], [[[-2.0, 2.0, -6.0, 0.0], [4.0, -7.0, 1.0, 0.0], [5.0, -5.0, 3.0, -6.0]], [[7.0, -5.0, 1.0, 2.0], [4.0, -2.0, 0.0, 0.0], [4.0, 4.0, 7.0, 7.0]]], [[[1.0, 3.0, -7.0, -6.0], [-5.0, 7.0, -7.0, 2.0], [3.0, -6.0, 0.0, -6.0]], [[-3.0, -1.0, -6.0, -4.0], [-6.0, 5.0, -4.0, -3.0], [3.0, -4.0, -1.0, -6.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-0.0, 6.0, -4.0, 4.0], [2.0, -10.0, 10.0, -4.0], [-12.0, 6.0, -10.0, -6.0]], [[2.0, -2.0, -4.0, 4.0], [4.0, 14.0, -2.0, -14.0], [-8.0, 10.0, -14.0, 2.0]]], [[[4.0, -4.0, 12.0, -0.0], [-8.0, 14.0, -2.0, -0.0], [-10.0, 10.0, -6.0, 12.0]], [[-14.0, 10.0, -2.0, -4.0], [-8.0, 4.0, -0.0, -0.0], [-8.0, -8.0, -14.0, -14.0]]], [[[-2.0, -6.0, 14.0, 12.0], [10.0, -14.0, 14.0, -4.0], [-6.0, 12.0, -0.0, 12.0]], [[6.0, 2.0, 12.0, 8.0], [12.0, -10.0, 8.0, 6.0], [-6.0, 8.0, 2.0, 12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_constData_rank0_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(5.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(0.0)
      sub=res.substitute({arg0:s0})
      ref=Data(0.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([2.0, -6.0, -2.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([1.0, -6.0, -5.0])
      sub=res.substitute({arg0:s0})
      ref=Data(48.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[3.0, -7.0, 5.0], [7.0, 7.0, -5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[2.0, -7.0, -4.0], [-1.0, -2.0, -5.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(39.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[5.0, 2.0, 4.0], [1.0, -7.0, -3.0], [5.0, 5.0, 0.0], [-2.0, -1.0, 0.0]], [[-2.0, -5.0, 2.0], [5.0, -3.0, 2.0], [0.0, 5.0, 0.0], [7.0, -4.0, -4.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-3.0, -2.0, 3.0], [1.0, -1.0, -5.0], [-1.0, -1.0, -7.0], [3.0, 3.0, 0.0]], [[3.0, 0.0, 3.0], [6.0, 0.0, 6.0], [-4.0, -5.0, 4.0], [-6.0, 1.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-28.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      arg1=Data(numpy.array([[[[5.0, 5.0], [0.0, -1.0], [0.0, 6.0]], [[4.0, 0.0], [-3.0, 0.0], [2.0, 0.0]]], [[[-2.0, -7.0], [7.0, 0.0], [-4.0, 3.0]], [[1.0, 0.0], [0.0, 2.0], [1.0, 7.0]]], [[[-1.0, 0.0], [0.0, 0.0], [3.0, -1.0]], [[-4.0, -2.0], [6.0, -6.0], [5.0, -1.0]]], [[[-6.0, 2.0], [6.0, 1.0], [7.0, 5.0]], [[2.0, 0.0], [0.0, 6.0], [-5.0, -7.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[-4.0, -7.0], [-4.0, 3.0], [-2.0, -7.0]], [[-1.0, 5.0], [-4.0, 3.0], [3.0, -7.0]]], [[[0.0, 4.0], [-2.0, -1.0], [4.0, -5.0]], [[0.0, -1.0], [6.0, 4.0], [0.0, 3.0]]], [[[-4.0, 5.0], [-5.0, 5.0], [4.0, 6.0]], [[-7.0, -6.0], [-5.0, 5.0], [-3.0, 1.0]]], [[[7.0, -4.0], [5.0, -6.0], [-5.0, 2.0]], [[-2.0, -2.0], [-7.0, 4.0], [-1.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(-196.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_constData_rank1_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([-4.0, 5.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(5.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-20.0, 25.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[5.0, 2.0, -5.0], [3.0, 6.0, 5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, -6.0, -3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-12.0, -60.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[2.0, 5.0, 3.0], [0.0, 7.0, 5.0]], [[-4.0, 6.0, -3.0], [2.0, 4.0, 4.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-7.0, -2.0, 5.0], [0.0, -6.0, 4.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-31.0, -7.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      arg1=Data(numpy.array([[[[-4.0, -5.0, 0.0], [3.0, -5.0, -3.0], [1.0, 1.0, 5.0], [-6.0, 2.0, 7.0]], [[3.0, -4.0, -2.0], [3.0, 6.0, 1.0], [-1.0, -7.0, 1.0], [0.0, 4.0, -2.0]]], [[[7.0, 0.0, -3.0], [1.0, -6.0, 7.0], [1.0, -3.0, -4.0], [-4.0, 2.0, 0.0]], [[2.0, 4.0, -1.0], [-3.0, -5.0, 7.0], [-2.0, 7.0, 0.0], [7.0, -7.0, -5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[6.0, -7.0, 0.0], [-5.0, -2.0, -5.0], [-3.0, 1.0, -6.0], [-6.0, -4.0, -3.0]], [[0.0, -4.0, -1.0], [2.0, -1.0, -6.0], [1.0, 1.0, 0.0], [4.0, -1.0, -7.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([10.0, 65.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_constData_rank2_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[-3.0, 1.0, 2.0, 1.0, -4.0], [-5.0, 0.0, 0.0, 5.0, -5.0], [-5.0, 7.0, 6.0, 5.0, 0.0], [0.0, 0.0, 4.0, -3.0, -1.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-6.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[18.0, -6.0, -12.0, -6.0, 24.0], [30.0, -0.0, -0.0, -30.0, 30.0], [30.0, -42.0, -36.0, -30.0, -0.0], [-0.0, -0.0, -24.0, 18.0, 6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[2.0, -7.0, -1.0], [-4.0, -6.0, 2.0], [7.0, 5.0, -7.0], [5.0, 0.0, 1.0], [5.0, -6.0, -5.0]], [[3.0, 7.0, 5.0], [1.0, 5.0, 1.0], [7.0, -7.0, 1.0], [5.0, 0.0, 0.0], [-4.0, 2.0, 5.0]], [[-4.0, -6.0, -4.0], [1.0, 0.0, 0.0], [6.0, -6.0, 3.0], [-4.0, -4.0, 5.0], [2.0, -5.0, -3.0]], [[1.0, 6.0, 3.0], [-3.0, -5.0, 7.0], [3.0, -6.0, 2.0], [7.0, -4.0, -1.0], [-4.0, 2.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-3.0, -5.0, -2.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[31.0, 38.0, -32.0, -17.0, 25.0], [-54.0, -30.0, 12.0, -15.0, -8.0], [50.0, -3.0, 6.0, 22.0, 25.0], [-39.0, 20.0, 17.0, 1.0, 2.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[[1.0, -7.0, 1.0], [-5.0, -2.0, -7.0]], [[-1.0, -3.0, 6.0], [-6.0, -2.0, -4.0]], [[-7.0, 0.0, -4.0], [-3.0, -2.0, 0.0]], [[-4.0, 0.0, -1.0], [-6.0, 0.0, -5.0]], [[-6.0, 0.0, -3.0], [6.0, -6.0, 5.0]]], [[[6.0, 0.0, 6.0], [-7.0, 2.0, 1.0]], [[0.0, -2.0, -1.0], [4.0, -6.0, -6.0]], [[3.0, 0.0, 0.0], [-4.0, 4.0, -1.0]], [[0.0, -4.0, -5.0], [0.0, 6.0, 1.0]], [[1.0, 3.0, 3.0], [6.0, 4.0, 1.0]]], [[[-6.0, -7.0, 4.0], [5.0, -3.0, -7.0]], [[1.0, -7.0, -2.0], [-6.0, -7.0, 7.0]], [[1.0, 7.0, 0.0], [3.0, -4.0, -6.0]], [[-1.0, 2.0, 3.0], [-3.0, 6.0, -5.0]], [[-1.0, -4.0, 5.0], [-7.0, 1.0, 7.0]]], [[[-6.0, -6.0, -1.0], [-4.0, -5.0, 1.0]], [[6.0, 2.0, -1.0], [0.0, -6.0, -6.0]], [[0.0, 1.0, -6.0], [7.0, 0.0, 2.0]], [[-1.0, -6.0, -4.0], [-3.0, -7.0, 2.0]], [[0.0, 4.0, 3.0], [0.0, -3.0, 7.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[6.0, 2.0, -3.0], [-2.0, 2.0, 1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-12.0, -26.0, -28.0, -14.0, -46.0], [37.0, -27.0, 33.0, 20.0, 0.0], [-85.0, 3.0, 0.0, 2.0, -6.0], [-46.0, 25.0, 8.0, -12.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_constData_rank3_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[7.0, -6.0], [-5.0, -6.0]], [[-1.0, -7.0], [-3.0, 4.0]], [[6.0, 0.0], [-3.0, -5.0]], [[5.0, -4.0], [-2.0, -6.0]], [[-3.0, 0.0], [6.0, -3.0]], [[5.0, -4.0], [7.0, 6.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-4.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-28.0, 24.0], [20.0, 24.0]], [[4.0, 28.0], [12.0, -16.0]], [[-24.0, -0.0], [12.0, 20.0]], [[-20.0, 16.0], [8.0, 24.0]], [[12.0, -0.0], [-24.0, 12.0]], [[-20.0, 16.0], [-28.0, -24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      arg1=Data(numpy.array([[[[-6.0, -3.0, -6.0], [7.0, 0.0, -4.0]], [[5.0, 3.0, 2.0], [3.0, 7.0, 0.0]]], [[[0.0, 0.0, -1.0], [-6.0, 1.0, -7.0]], [[-5.0, -2.0, 7.0], [5.0, 0.0, 4.0]]], [[[-3.0, 5.0, 0.0], [3.0, 0.0, 7.0]], [[-5.0, 0.0, 0.0], [7.0, 4.0, -7.0]]], [[[2.0, 6.0, 4.0], [6.0, -1.0, 7.0]], [[4.0, -5.0, 3.0], [7.0, 5.0, -4.0]]], [[[-5.0, 2.0, -1.0], [4.0, 0.0, 4.0]], [[-6.0, -3.0, 0.0], [-7.0, 6.0, -1.0]]], [[[2.0, -6.0, -4.0], [-1.0, 2.0, 3.0]], [[-2.0, 0.0, -6.0], [1.0, -5.0, 5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-6.0, -5.0, -3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[69.0, -30.0], [-51.0, -53.0]], [[3.0, 52.0], [19.0, -42.0]], [[-7.0, -39.0], [30.0, -41.0]], [[-54.0, -52.0], [-8.0, -55.0]], [[23.0, -36.0], [51.0, 15.0]], [[30.0, -13.0], [30.0, 4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_constData_rank4_offset0(self):
      arg0=Symbol(shape=())
      arg1=Data(numpy.array([[[[4.0, 7.0, -5.0, 6.0], [-4.0, -4.0, 0.0, -1.0], [-3.0, -5.0, 3.0, 2.0]], [[6.0, 0.0, -5.0, 1.0], [3.0, -5.0, 1.0, 1.0], [2.0, -2.0, -6.0, 5.0]]], [[[-1.0, 1.0, -7.0, -6.0], [-2.0, 0.0, -7.0, -6.0], [0.0, 1.0, 4.0, -4.0]], [[-3.0, -6.0, 7.0, -5.0], [0.0, 6.0, -3.0, -3.0], [1.0, 3.0, -2.0, 5.0]]], [[[3.0, 6.0, 0.0, 2.0], [1.0, 0.0, 7.0, -2.0], [2.0, 7.0, -1.0, 0.0]], [[1.0, -2.0, -1.0, 7.0], [3.0, -1.0, 5.0, -5.0], [0.0, -5.0, -4.0, -4.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-5.0)
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-20.0, -35.0, 25.0, -30.0], [20.0, 20.0, -0.0, 5.0], [15.0, 25.0, -15.0, -10.0]], [[-30.0, -0.0, 25.0, -5.0], [-15.0, 25.0, -5.0, -5.0], [-10.0, 10.0, 30.0, -25.0]]], [[[5.0, -5.0, 35.0, 30.0], [10.0, -0.0, 35.0, 30.0], [-0.0, -5.0, -20.0, 20.0]], [[15.0, 30.0, -35.0, 25.0], [-0.0, -30.0, 15.0, 15.0], [-5.0, -15.0, 10.0, -25.0]]], [[[-15.0, -30.0, -0.0, -10.0], [-5.0, -0.0, -35.0, 10.0], [-10.0, -35.0, 5.0, -0.0]], [[-5.0, 10.0, 5.0, -35.0], [-15.0, 5.0, -25.0, 25.0], [-0.0, 25.0, 20.0, 20.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(0.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-6.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-6.0)+(1.-msk_ref)*numpy.array(-0.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 3.0, 2.0])+(1.-msk_arg1)*numpy.array([0.0, -6.0, -3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([2.0, 6.0, 3.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(16.0)+(1.-msk_ref)*numpy.array(-45.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[3.0, 0.0, 6.0], [4.0, 0.0, 3.0]])+(1.-msk_arg1)*numpy.array([[-3.0, -7.0, 7.0], [-7.0, 5.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[5.0, -7.0, -6.0], [-1.0, -1.0, 3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-16.0)+(1.-msk_ref)*numpy.array(-21.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, 0.0, -3.0], [-5.0, -2.0, -7.0], [-5.0, -4.0, 1.0], [-7.0, 2.0, 0.0]], [[-1.0, -6.0, 0.0], [-4.0, 4.0, 6.0], [7.0, 4.0, -1.0], [-7.0, 6.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, -2.0, 0.0], [0.0, 7.0, -3.0], [-4.0, -5.0, -4.0], [-3.0, 3.0, 5.0]], [[7.0, 5.0, 4.0], [-6.0, 0.0, 7.0], [-6.0, -1.0, -3.0], [0.0, -5.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-6.0, 3.0, 7.0], [7.0, -5.0, 6.0], [2.0, 2.0, -1.0], [0.0, 0.0, 0.0]], [[-2.0, 1.0, 2.0], [-1.0, 2.0, -5.0], [-7.0, -6.0, 5.0], [-6.0, -3.0, -7.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-216.0)+(1.-msk_ref)*numpy.array(-125.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank4_offset4(self):
      arg0=Symbol(shape=(4, 2, 3, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-7.0, -1.0], [2.0, -6.0], [-4.0, -2.0]], [[0.0, 2.0], [-1.0, -6.0], [-7.0, 6.0]]], [[[5.0, 3.0], [6.0, -2.0], [-5.0, 3.0]], [[-7.0, 1.0], [-4.0, -4.0], [-1.0, 4.0]]], [[[-7.0, 7.0], [-4.0, 3.0], [7.0, -4.0]], [[-7.0, 6.0], [5.0, -3.0], [-3.0, -6.0]]], [[[1.0, 1.0], [-7.0, -3.0], [-2.0, 2.0]], [[3.0, -3.0], [0.0, -4.0], [7.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -5.0], [7.0, -4.0], [-4.0, 6.0]], [[4.0, -1.0], [-6.0, -4.0], [-4.0, 0.0]]], [[[-1.0, -5.0], [-3.0, -3.0], [-2.0, -5.0]], [[0.0, -3.0], [5.0, 1.0], [5.0, -6.0]]], [[[-7.0, -5.0], [-4.0, 3.0], [2.0, 0.0]], [[0.0, -1.0], [0.0, -5.0], [-5.0, -7.0]]], [[[-7.0, 5.0], [-7.0, 4.0], [-4.0, -2.0]], [[-6.0, -2.0], [-1.0, 0.0], [0.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s0=numpy.array([[[[5.0, 0.0], [7.0, -7.0], [2.0, 4.0]], [[7.0, -5.0], [2.0, -5.0], [4.0, -1.0]]], [[[6.0, 7.0], [1.0, 7.0], [7.0, 0.0]], [[-5.0, 4.0], [0.0, -3.0], [7.0, 0.0]]], [[[-2.0, -4.0], [2.0, -2.0], [-4.0, 4.0]], [[1.0, -2.0], [-6.0, -5.0], [-5.0, -6.0]]], [[[-4.0, 2.0], [-1.0, 4.0], [4.0, -4.0]], [[-5.0, 0.0], [2.0, -1.0], [-5.0, 7.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-34.0)+(1.-msk_ref)*numpy.array(294.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-2.0, -4.0])+(1.-msk_arg1)*numpy.array([-7.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(1.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-2.0, -4.0])+(1.-msk_ref)*numpy.array([-7.0, 1.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, -7.0, -2.0], [-4.0, 0.0, -6.0]])+(1.-msk_arg1)*numpy.array([[-4.0, 7.0, 6.0], [0.0, 1.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([-5.0, 4.0, -6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([14.0, 56.0])+(1.-msk_ref)*numpy.array([12.0, 4.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, -1.0, 5.0], [1.0, -3.0, 6.0]], [[3.0, -2.0, -5.0], [4.0, 2.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[6.0, 2.0, 7.0], [6.0, 1.0, 5.0]], [[0.0, 5.0, 4.0], [-2.0, 6.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[1.0, -5.0, -1.0], [6.0, 2.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([1.0, 53.0])+(1.-msk_ref)*numpy.array([32.0, -25.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, 4.0, 0.0], [3.0, 4.0, -2.0], [-1.0, -3.0, 2.0], [0.0, 2.0, 6.0]], [[4.0, 0.0, -2.0], [-2.0, -7.0, 4.0], [-7.0, 4.0, -6.0], [6.0, -7.0, -6.0]]], [[[-3.0, 1.0, -5.0], [-1.0, 6.0, 0.0], [-6.0, -6.0, -2.0], [5.0, 0.0, -4.0]], [[-2.0, 0.0, 3.0], [0.0, 5.0, -7.0], [0.0, 0.0, -3.0], [7.0, -1.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 0.0, 3.0], [-4.0, 5.0, 3.0], [5.0, 1.0, 1.0], [-1.0, 1.0, -2.0]], [[1.0, 3.0, 3.0], [-3.0, -6.0, 1.0], [6.0, -1.0, 7.0], [4.0, 5.0, 4.0]]], [[[0.0, -5.0, 0.0], [2.0, 3.0, -6.0], [2.0, 4.0, 2.0], [3.0, -4.0, 0.0]], [[1.0, 7.0, -7.0], [2.0, 6.0, 4.0], [-1.0, 1.0, 4.0], [0.0, -6.0, 5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[-6.0, 5.0, 2.0], [4.0, 5.0, -5.0], [1.0, -4.0, 7.0], [-7.0, -1.0, -4.0]], [[0.0, 1.0, -3.0], [4.0, -5.0, 4.0], [7.0, 3.0, -2.0], [0.0, -5.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([114.0, -21.0])+(1.-msk_ref)*numpy.array([12.0, 41.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, 5.0, -7.0, -3.0, 0.0], [5.0, -6.0, -7.0, 3.0, 3.0], [2.0, 0.0, 2.0, 0.0, -3.0], [1.0, -2.0, 4.0, -1.0, 2.0]])+(1.-msk_arg1)*numpy.array([[3.0, -1.0, 6.0, 5.0, -3.0], [-4.0, 0.0, 4.0, 7.0, 5.0], [-4.0, -6.0, -2.0, 2.0, 7.0], [5.0, 7.0, 2.0, 0.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(2.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[4.0, 10.0, -14.0, -6.0, 0.0], [10.0, -12.0, -14.0, 6.0, 6.0], [4.0, 0.0, 4.0, 0.0, -6.0], [2.0, -4.0, 8.0, -2.0, 4.0]])+(1.-msk_ref)*numpy.array([[6.0, -2.0, 12.0, 10.0, -6.0], [-8.0, 0.0, 8.0, 14.0, 10.0], [-8.0, -12.0, -4.0, 4.0, 14.0], [10.0, 14.0, 4.0, 0.0, -14.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 0.0, 1.0], [3.0, -4.0, 1.0], [-2.0, -5.0, 5.0], [5.0, -3.0, 3.0], [7.0, 2.0, -5.0]], [[6.0, 4.0, 4.0], [-5.0, 0.0, 0.0], [0.0, 1.0, 0.0], [-5.0, 0.0, 4.0], [6.0, 3.0, -2.0]], [[-7.0, -3.0, 0.0], [4.0, 0.0, 6.0], [6.0, -7.0, 0.0], [-6.0, 0.0, -4.0], [-4.0, -4.0, 4.0]], [[7.0, 0.0, -4.0], [4.0, -7.0, 0.0], [-3.0, 1.0, 0.0], [-3.0, -1.0, -2.0], [-7.0, 1.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, 0.0, 6.0], [2.0, 4.0, 3.0], [-3.0, 7.0, 1.0], [6.0, 0.0, 0.0], [-1.0, 6.0, -2.0]], [[-4.0, -2.0, -6.0], [-4.0, -1.0, 0.0], [-4.0, -5.0, 7.0], [6.0, -7.0, 7.0], [1.0, 6.0, -4.0]], [[-6.0, 0.0, -6.0], [2.0, -4.0, -3.0], [6.0, 0.0, 1.0], [-1.0, 7.0, -7.0], [0.0, 0.0, 6.0]], [[3.0, -6.0, 1.0], [-4.0, 2.0, 4.0], [-6.0, 0.0, 4.0], [7.0, 1.0, -3.0], [2.0, -7.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([5.0, 3.0, -2.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 1.0, -35.0, 10.0, 51.0], [34.0, -25.0, 3.0, -33.0, 43.0], [-44.0, 8.0, 9.0, -22.0, -40.0], [43.0, -1.0, -12.0, -14.0, -42.0]])+(1.-msk_ref)*numpy.array([[-37.0, 16.0, 4.0, 30.0, 17.0], [-14.0, -23.0, -49.0, -5.0, 31.0], [-18.0, 4.0, 28.0, 30.0, -12.0], [-5.0, -22.0, -38.0, 44.0, 1.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, -3.0, -1.0], [0.0, 7.0, 0.0]], [[-3.0, -6.0, 0.0], [-1.0, -4.0, 3.0]], [[-2.0, 4.0, 2.0], [-6.0, 4.0, 3.0]], [[2.0, 3.0, -6.0], [-6.0, 0.0, 0.0]], [[6.0, -6.0, 0.0], [-3.0, 3.0, 6.0]]], [[[6.0, 4.0, 5.0], [-1.0, 4.0, 5.0]], [[-5.0, 6.0, 7.0], [-1.0, 2.0, 1.0]], [[2.0, 6.0, -7.0], [-4.0, 0.0, -2.0]], [[1.0, 7.0, -4.0], [-2.0, -7.0, 1.0]], [[-1.0, 2.0, 7.0], [1.0, 3.0, -1.0]]], [[[-2.0, -6.0, 2.0], [7.0, 6.0, 2.0]], [[-1.0, 4.0, 4.0], [-4.0, -4.0, 0.0]], [[2.0, 2.0, -1.0], [-6.0, 5.0, 1.0]], [[7.0, -6.0, 6.0], [4.0, -5.0, 1.0]], [[0.0, -4.0, -5.0], [-6.0, 5.0, 4.0]]], [[[3.0, 2.0, -4.0], [-1.0, 1.0, -2.0]], [[0.0, 1.0, 6.0], [-6.0, -1.0, 1.0]], [[5.0, -4.0, -1.0], [-7.0, -7.0, -4.0]], [[-6.0, -3.0, 2.0], [3.0, -2.0, -4.0]], [[-1.0, 4.0, 7.0], [1.0, 1.0, -3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, 7.0, 0.0], [-4.0, -3.0, 1.0]], [[-5.0, -2.0, 0.0], [-7.0, -1.0, 5.0]], [[0.0, 4.0, -6.0], [5.0, 5.0, -5.0]], [[-4.0, 2.0, -5.0], [2.0, -5.0, -1.0]], [[6.0, 2.0, -7.0], [3.0, 0.0, -6.0]]], [[[7.0, -3.0, 0.0], [7.0, 7.0, 7.0]], [[4.0, 0.0, 6.0], [5.0, -2.0, -6.0]], [[6.0, 4.0, 6.0], [-7.0, -5.0, -1.0]], [[3.0, -1.0, -3.0], [0.0, 4.0, 1.0]], [[0.0, -2.0, -4.0], [-6.0, -1.0, 2.0]]], [[[-2.0, 3.0, 2.0], [0.0, -2.0, -7.0]], [[-3.0, 6.0, 5.0], [-2.0, -4.0, 1.0]], [[1.0, -1.0, -5.0], [2.0, 0.0, -5.0]], [[6.0, -3.0, 0.0], [3.0, 7.0, 0.0]], [[-3.0, -1.0, 0.0], [-2.0, -6.0, 3.0]]], [[[5.0, 4.0, -1.0], [-5.0, -5.0, 0.0]], [[0.0, 2.0, -4.0], [4.0, -4.0, -1.0]], [[0.0, -5.0, -5.0], [5.0, 6.0, 1.0]], [[-2.0, 7.0, 0.0], [-7.0, 6.0, -2.0]], [[-4.0, 6.0, -4.0], [0.0, -6.0, -3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[-6.0, -1.0, -2.0], [-2.0, 7.0, -3.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[90.0, -11.0, 35.0, 9.0, -21.0], [-35.0, 23.0, 10.0, -53.0, 12.0], [36.0, -26.0, 32.0, -94.0, 49.0], [3.0, -11.0, -47.0, 27.0, 2.0]])+(1.-msk_ref)*numpy.array([[1.0, 24.0, 48.0, -4.0, -12.0], [-25.0, -42.0, -70.0, 14.0, 9.0], [12.0, -25.0, 16.0, 10.0, -28.0], [-57.0, -27.0, 44.0, 67.0, -7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, 5.0], [0.0, 5.0]], [[-4.0, -5.0], [-5.0, 0.0]], [[3.0, -6.0], [0.0, -4.0]], [[4.0, -6.0], [1.0, -6.0]], [[-6.0, 4.0], [0.0, -1.0]], [[-5.0, -3.0], [2.0, 3.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -6.0], [-4.0, 6.0]], [[3.0, -3.0], [4.0, -5.0]], [[-1.0, -4.0], [-3.0, -6.0]], [[2.0, -5.0], [-6.0, -1.0]], [[5.0, -3.0], [-1.0, 5.0]], [[-2.0, -2.0], [2.0, -3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(-4.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-20.0, -20.0], [-0.0, -20.0]], [[16.0, 20.0], [20.0, -0.0]], [[-12.0, 24.0], [-0.0, 16.0]], [[-16.0, 24.0], [-4.0, 24.0]], [[24.0, -16.0], [-0.0, 4.0]], [[20.0, 12.0], [-8.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[-8.0, 24.0], [16.0, -24.0]], [[-12.0, 12.0], [-16.0, 20.0]], [[4.0, 16.0], [12.0, 24.0]], [[-8.0, 20.0], [24.0, 4.0]], [[-20.0, 12.0], [4.0, -20.0]], [[8.0, 8.0], [-8.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(3,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, 0.0, 1.0], [1.0, 3.0, 0.0]], [[-4.0, 2.0, 1.0], [-2.0, -3.0, 7.0]]], [[[-2.0, 1.0, 0.0], [-5.0, 4.0, 2.0]], [[0.0, 3.0, 7.0], [-6.0, 1.0, 5.0]]], [[[6.0, 0.0, -4.0], [-5.0, 2.0, 0.0]], [[-5.0, -6.0, 5.0], [6.0, 3.0, 6.0]]], [[[1.0, 5.0, 4.0], [-4.0, -5.0, 2.0]], [[-3.0, -4.0, -6.0], [-5.0, 0.0, 7.0]]], [[[-1.0, -5.0, 1.0], [-6.0, -3.0, 5.0]], [[-1.0, 6.0, -2.0], [-1.0, -6.0, 7.0]]], [[[-6.0, -1.0, 4.0], [5.0, 6.0, -2.0]], [[-3.0, -3.0, 1.0], [2.0, -1.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, -7.0, -5.0], [1.0, -1.0, 5.0]], [[2.0, 1.0, 1.0], [-7.0, 0.0, 1.0]]], [[[4.0, 1.0, 3.0], [1.0, 4.0, 1.0]], [[4.0, 1.0, 7.0], [-6.0, 1.0, 3.0]]], [[[1.0, -7.0, 0.0], [-3.0, 0.0, -3.0]], [[-5.0, 2.0, 0.0], [5.0, 6.0, -1.0]]], [[[6.0, -2.0, -7.0], [-4.0, 6.0, -7.0]], [[2.0, -1.0, 1.0], [-2.0, -5.0, 1.0]]], [[[5.0, 6.0, 1.0], [-3.0, -4.0, -5.0]], [[-3.0, 0.0, -6.0], [-3.0, -7.0, -3.0]]], [[[0.0, 3.0, -5.0], [5.0, 0.0, 2.0]], [[7.0, -7.0, -2.0], [5.0, -2.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([0.0, 0.0, 2.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 0.0], [2.0, 14.0]], [[0.0, 4.0], [14.0, 10.0]], [[-8.0, 0.0], [10.0, 12.0]], [[8.0, 4.0], [-12.0, 14.0]], [[2.0, 10.0], [-4.0, 14.0]], [[8.0, -4.0], [2.0, -10.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 10.0], [2.0, 2.0]], [[6.0, 2.0], [14.0, 6.0]], [[0.0, -6.0], [0.0, -2.0]], [[-14.0, -14.0], [2.0, 2.0]], [[2.0, -10.0], [-12.0, -6.0]], [[-10.0, 4.0], [-4.0, -10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank0_expandedData_rank4_offset0(self):
      arg0=Symbol(shape=())
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, 3.0, 2.0, -1.0], [7.0, -1.0, 5.0, -7.0], [6.0, 5.0, 0.0, -6.0]], [[-3.0, 6.0, -7.0, -5.0], [0.0, -4.0, 5.0, -7.0], [3.0, -1.0, -3.0, -2.0]]], [[[3.0, -5.0, 3.0, 0.0], [-1.0, -2.0, 2.0, -7.0], [-2.0, 2.0, 4.0, 4.0]], [[-5.0, 1.0, -7.0, -2.0], [1.0, -4.0, 4.0, 2.0], [-4.0, 0.0, 3.0, 0.0]]], [[[3.0, -1.0, 0.0, 7.0], [-4.0, 6.0, 4.0, 3.0], [2.0, -3.0, 4.0, -1.0]], [[0.0, -2.0, -5.0, -6.0], [2.0, 7.0, -4.0, -5.0], [1.0, -2.0, 7.0, 5.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 7.0, -5.0, 7.0], [6.0, -5.0, 2.0, 0.0], [0.0, -1.0, 3.0, 3.0]], [[3.0, 0.0, -7.0, 4.0], [6.0, 0.0, 0.0, -1.0], [7.0, 1.0, 3.0, 3.0]]], [[[-6.0, 6.0, 3.0, -4.0], [4.0, -3.0, 6.0, -5.0], [-5.0, 3.0, -4.0, -4.0]], [[4.0, 0.0, -1.0, -6.0], [-6.0, 7.0, -5.0, 4.0], [-1.0, 0.0, 7.0, 6.0]]], [[[-2.0, -5.0, 7.0, 0.0], [-4.0, 4.0, -2.0, 6.0], [5.0, -2.0, -6.0, 0.0]], [[2.0, 6.0, 3.0, -2.0], [4.0, -5.0, 0.0, -6.0], [-3.0, 0.0, -6.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array(5.0)
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 15.0, 10.0, -5.0], [35.0, -5.0, 25.0, -35.0], [30.0, 25.0, 0.0, -30.0]], [[-15.0, 30.0, -35.0, -25.0], [0.0, -20.0, 25.0, -35.0], [15.0, -5.0, -15.0, -10.0]]], [[[15.0, -25.0, 15.0, 0.0], [-5.0, -10.0, 10.0, -35.0], [-10.0, 10.0, 20.0, 20.0]], [[-25.0, 5.0, -35.0, -10.0], [5.0, -20.0, 20.0, 10.0], [-20.0, 0.0, 15.0, 0.0]]], [[[15.0, -5.0, 0.0, 35.0], [-20.0, 30.0, 20.0, 15.0], [10.0, -15.0, 20.0, -5.0]], [[0.0, -10.0, -25.0, -30.0], [10.0, 35.0, -20.0, -25.0], [5.0, -10.0, 35.0, 25.0]]]])+(1.-msk_ref)*numpy.array([[[[-10.0, 35.0, -25.0, 35.0], [30.0, -25.0, 10.0, 0.0], [0.0, -5.0, 15.0, 15.0]], [[15.0, 0.0, -35.0, 20.0], [30.0, 0.0, 0.0, -5.0], [35.0, 5.0, 15.0, 15.0]]], [[[-30.0, 30.0, 15.0, -20.0], [20.0, -15.0, 30.0, -25.0], [-25.0, 15.0, -20.0, -20.0]], [[20.0, 0.0, -5.0, -30.0], [-30.0, 35.0, -25.0, 20.0], [-5.0, 0.0, 35.0, 30.0]]], [[[-10.0, -25.0, 35.0, 0.0], [-20.0, 20.0, -10.0, 30.0], [25.0, -10.0, -30.0, 0.0]], [[10.0, 30.0, 15.0, -10.0], [20.0, -25.0, 0.0, -30.0], [-15.0, 0.0, -30.0, -35.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_float_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=7.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([0.0, -7.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([0.0, -49.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array(4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, 2.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([-28.0, 8.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([0.0, 3.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[6.0, -5.0, -4.0], [-4.0, -3.0, -3.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([13.0, 12.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[-1.0, -6.0, -5.0], [0.0, 2.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-3.0, -3.0, 0.0], [4.0, 2.0, -1.0]], [[-3.0, -6.0, 0.0], [5.0, -5.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([25.0, 29.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=numpy.array([[[0.0, -3.0, -6.0], [-2.0, -4.0, 4.0], [-7.0, -3.0, 4.0], [-2.0, 6.0, -6.0]], [[7.0, -1.0, 1.0], [2.0, 0.0, -4.0], [3.0, -4.0, 2.0], [-4.0, -4.0, -5.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[7.0, 2.0, 5.0], [-6.0, -6.0, 7.0], [-1.0, 5.0, 5.0], [-6.0, -6.0, -1.0]], [[1.0, -6.0, 6.0], [-1.0, 1.0, -7.0], [-5.0, -5.0, 0.0], [-6.0, -4.0, -5.0]]], [[[1.0, 5.0, -1.0], [2.0, 0.0, 5.0], [-7.0, -1.0, -2.0], [2.0, 0.0, 0.0]], [[0.0, 1.0, -6.0], [1.0, -6.0, -7.0], [4.0, 0.0, -2.0], [3.0, -5.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([137.0, 66.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([4.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([5.0, 6.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[20.0, 30.0], [24.0, 36.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[4.0, 1.0, -2.0], [5.0, 0.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[0.0, 6.0, -2.0], [-1.0, 2.0, 3.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[10.0, 14.0], [-8.0, -26.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[[5.0, 5.0, 1.0], [-5.0, -5.0, -2.0]], [[0.0, 6.0, -7.0], [-7.0, -2.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[6.0, 6.0, 7.0], [-4.0, -4.0, 6.0]], [[2.0, 2.0, -5.0], [-7.0, 6.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[95.0, 35.0], [12.0, 92.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=numpy.array([[[[-6.0, -2.0, 7.0], [0.0, 1.0, 0.0], [2.0, 5.0, -2.0], [-5.0, 7.0, 5.0]], [[1.0, 5.0, 2.0], [0.0, 0.0, 5.0], [0.0, 2.0, -1.0], [6.0, 1.0, 4.0]]], [[[4.0, 0.0, 1.0], [0.0, -6.0, 0.0], [7.0, -7.0, 4.0], [-5.0, 7.0, 4.0]], [[-2.0, -6.0, -4.0], [-2.0, 7.0, 5.0], [-1.0, -7.0, -6.0], [-2.0, 7.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-1.0, 2.0, -5.0], [-2.0, -6.0, 0.0], [6.0, -4.0, -2.0], [6.0, -3.0, 3.0]], [[0.0, -1.0, 4.0], [-2.0, 0.0, -2.0], [1.0, 1.0, -3.0], [6.0, -4.0, -1.0]]], [[[2.0, 0.0, 5.0], [-1.0, 4.0, -2.0], [-5.0, -3.0, 6.0], [-2.0, -2.0, 1.0]], [[6.0, 0.0, 2.0], [-3.0, 0.0, 4.0], [4.0, 2.0, -5.0], [0.0, 5.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-53.0, 4.0], [39.0, 52.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[7.0, -6.0, 6.0, -6.0, 3.0], [-3.0, -3.0, 0.0, -4.0, 2.0], [6.0, -7.0, -2.0, -3.0, 1.0], [-2.0, -2.0, -5.0, 1.0, 2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-2.0, -5.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-14.0, 12.0, -12.0, 12.0, -6.0], [6.0, 6.0, 0.0, 8.0, -4.0], [-12.0, 14.0, 4.0, 6.0, -2.0], [4.0, 4.0, 10.0, -2.0, -4.0]], [[-35.0, 30.0, -30.0, 30.0, -15.0], [15.0, 15.0, 0.0, 20.0, -10.0], [-30.0, 35.0, 10.0, 15.0, -5.0], [10.0, 10.0, 25.0, -5.0, -10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[7.0, 3.0, -2.0], [6.0, -6.0, 7.0], [-6.0, -1.0, -6.0], [7.0, 1.0, -1.0], [6.0, 0.0, 6.0]], [[5.0, -1.0, -4.0], [3.0, -4.0, 1.0], [0.0, -7.0, 4.0], [0.0, -1.0, 7.0], [7.0, -3.0, 6.0]], [[-1.0, 4.0, -7.0], [5.0, -5.0, -2.0], [5.0, 7.0, 4.0], [0.0, -7.0, 6.0], [-6.0, 7.0, 4.0]], [[0.0, -2.0, -2.0], [-7.0, 0.0, 0.0], [6.0, -5.0, 0.0], [-2.0, 2.0, 0.0], [-1.0, 0.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-4.0, 3.0, -5.0], [-3.0, -4.0, 5.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-9.0, -77.0, 51.0, -20.0, -54.0], [-3.0, -29.0, -41.0, -38.0, -67.0], [51.0, -25.0, -19.0, -51.0, 25.0], [4.0, 28.0, -39.0, 14.0, 9.0]], [[-43.0, 41.0, -8.0, -30.0, 12.0], [-31.0, 12.0, 48.0, 39.0, 21.0], [-48.0, -5.0, -23.0, 58.0, 10.0], [-2.0, 21.0, 2.0, -2.0, -2.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=numpy.array([[[[-5.0, -6.0, 3.0], [-1.0, -4.0, -4.0]], [[-1.0, 4.0, 1.0], [-2.0, 4.0, 1.0]], [[5.0, 6.0, -4.0], [-6.0, -3.0, 1.0]], [[5.0, 1.0, 3.0], [3.0, 0.0, 0.0]], [[-2.0, 4.0, 3.0], [0.0, 0.0, 0.0]]], [[[0.0, -6.0, 6.0], [-4.0, 0.0, -1.0]], [[-2.0, -7.0, -5.0], [7.0, 0.0, 0.0]], [[5.0, -6.0, 6.0], [7.0, -5.0, -4.0]], [[0.0, -6.0, -2.0], [6.0, -2.0, -2.0]], [[-7.0, 2.0, 0.0], [0.0, 4.0, 0.0]]], [[[0.0, 7.0, 0.0], [-3.0, -7.0, 7.0]], [[-7.0, -6.0, -3.0], [-1.0, 6.0, -7.0]], [[0.0, 4.0, -7.0], [3.0, -2.0, 2.0]], [[0.0, -4.0, 0.0], [5.0, 7.0, -7.0]], [[5.0, -1.0, -7.0], [-7.0, -1.0, -4.0]]], [[[-2.0, -4.0, 1.0], [-6.0, 0.0, 0.0]], [[2.0, 3.0, 3.0], [6.0, -6.0, 4.0]], [[0.0, 7.0, -4.0], [-2.0, 0.0, 4.0]], [[-1.0, -5.0, -7.0], [0.0, -3.0, 3.0]], [[-3.0, 6.0, -7.0], [5.0, -3.0, 1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[3.0, -7.0, 7.0], [-2.0, 3.0, 4.0]], [[5.0, 4.0, 5.0], [3.0, 0.0, -6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[22.0, -4.0, -48.0, 23.0, -13.0], [88.0, -6.0, 54.0, 2.0, -23.0], [-36.0, -8.0, -81.0, 11.0, -32.0], [41.0, -8.0, -57.0, -14.0, -115.0]], [[-13.0, 4.0, 5.0, 53.0, 21.0], [0.0, -42.0, 76.0, -4.0, -27.0], [-23.0, -35.0, -22.0, 41.0, -11.0], [-39.0, 31.0, -22.0, -78.0, -17.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_array_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=numpy.array([[[-1.0, 2.0], [0.0, -7.0]], [[-1.0, 6.0], [3.0, -5.0]], [[4.0, -1.0], [1.0, -3.0]], [[1.0, -6.0], [-6.0, 4.0]], [[4.0, -4.0], [1.0, -7.0]], [[-1.0, -3.0], [-6.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([2.0, 4.0])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-2.0, 4.0], [0.0, -14.0]], [[-2.0, 12.0], [6.0, -10.0]], [[8.0, -2.0], [2.0, -6.0]], [[2.0, -12.0], [-12.0, 8.0]], [[8.0, -8.0], [2.0, -14.0]], [[-2.0, -6.0], [-12.0, -14.0]]], [[[-4.0, 8.0], [0.0, -28.0]], [[-4.0, 24.0], [12.0, -20.0]], [[16.0, -4.0], [4.0, -12.0]], [[4.0, -24.0], [-24.0, 16.0]], [[16.0, -16.0], [4.0, -28.0]], [[-4.0, -12.0], [-24.0, -28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=numpy.array([[[[-6.0, 2.0, -5.0], [0.0, -1.0, -7.0]], [[0.0, 7.0, -7.0], [-5.0, -2.0, -6.0]]], [[[3.0, 4.0, -6.0], [0.0, -7.0, -3.0]], [[-7.0, 3.0, -7.0], [-1.0, 3.0, -3.0]]], [[[-6.0, -5.0, 7.0], [-2.0, 3.0, -6.0]], [[6.0, -3.0, 4.0], [-6.0, 5.0, -3.0]]], [[[-4.0, -1.0, 0.0], [3.0, 4.0, -2.0]], [[-7.0, -5.0, -7.0], [3.0, 4.0, 2.0]]], [[[-3.0, 5.0, -2.0], [-1.0, 5.0, -7.0]], [[0.0, 2.0, 4.0], [-6.0, 0.0, -2.0]]], [[[0.0, -5.0, 0.0], [-7.0, -5.0, -6.0]], [[4.0, 1.0, 7.0], [0.0, 0.0, -3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[0.0, 7.0, -5.0], [5.0, 0.0, -5.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[39.0, 28.0], [84.0, 16.0]], [[58.0, -34.0], [56.0, 36.0]], [[-70.0, 51.0], [-41.0, 50.0]], [[-7.0, 38.0], [0.0, 18.0]], [[45.0, 70.0], [-6.0, 10.0]], [[-35.0, -5.0], [-28.0, 15.0]]], [[[-5.0, 35.0], [35.0, 5.0]], [[45.0, 15.0], [0.0, 10.0]], [[-65.0, 20.0], [10.0, -15.0]], [[-20.0, 25.0], [0.0, 5.0]], [[-5.0, 30.0], [-20.0, -20.0]], [[0.0, -5.0], [-15.0, 15.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-7.0, 5.0])
      s1=numpy.array(-1.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([7.0, -5.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[6.0, 7.0, -6.0], [0.0, 4.0, 5.0]])
      s1=numpy.array([-1.0, -2.0, -3.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-2.0, -23.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-5.0, -6.0, -6.0], [-4.0, 2.0, 0.0]], [[-2.0, 3.0, -6.0], [-6.0, 2.0, 3.0]]])
      s1=numpy.array([[0.0, 7.0, 2.0], [2.0, -1.0, 7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-64.0, 16.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[0.0, 6.0, 2.0], [-4.0, -1.0, 3.0], [7.0, 1.0, 5.0], [5.0, -3.0, 0.0]], [[6.0, 6.0, -1.0], [-4.0, 0.0, 6.0], [2.0, 4.0, 5.0], [0.0, 2.0, -6.0]]], [[[7.0, 3.0, -7.0], [3.0, 4.0, -6.0], [0.0, -1.0, 1.0], [7.0, 6.0, -4.0]], [[5.0, 3.0, 2.0], [4.0, -3.0, 0.0], [-4.0, -4.0, 7.0], [-6.0, 1.0, 4.0]]]])
      s1=numpy.array([[[0.0, 4.0, 2.0], [-4.0, 6.0, 2.0], [-7.0, 7.0, -1.0], [-3.0, -2.0, -7.0]], [[4.0, 0.0, -4.0], [0.0, -4.0, 5.0], [3.0, -7.0, -6.0], [6.0, -3.0, 5.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([-42.0, -36.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, 0.0])
      s1=numpy.array([7.0, 2.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[28.0, 8.0], [0.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[6.0, -3.0, 5.0], [1.0, 2.0, -2.0]])
      s1=numpy.array([[3.0, -5.0, 2.0], [-1.0, -3.0, 4.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[43.0, 23.0], [-11.0, -15.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-4.0, 5.0, 6.0], [-6.0, -3.0, -6.0]], [[6.0, -4.0, -3.0], [-4.0, 3.0, 3.0]]])
      s1=numpy.array([[[-3.0, 1.0, 3.0], [0.0, 7.0, -4.0]], [[7.0, -4.0, 3.0], [0.0, 4.0, 5.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[38.0, -72.0], [-22.0, 76.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[5.0, -1.0, 7.0], [4.0, 0.0, 2.0], [-4.0, -1.0, -3.0], [-6.0, -5.0, 6.0]], [[0.0, 6.0, -5.0], [7.0, -7.0, -1.0], [-3.0, -4.0, -3.0], [4.0, -6.0, -3.0]]], [[[6.0, -6.0, -1.0], [3.0, -1.0, -1.0], [-2.0, 1.0, -4.0], [-4.0, -1.0, 4.0]], [[1.0, 7.0, 5.0], [-7.0, -7.0, 5.0], [6.0, -4.0, -1.0], [-7.0, 6.0, 0.0]]]])
      s1=numpy.array([[[[-2.0, -1.0, 3.0], [-3.0, -4.0, 3.0], [-6.0, -2.0, -2.0], [0.0, 7.0, 1.0]], [[0.0, 3.0, 7.0], [7.0, 0.0, 6.0], [1.0, 2.0, -4.0], [0.0, 4.0, 4.0]]], [[[-5.0, 0.0, 7.0], [-1.0, -6.0, 6.0], [3.0, -2.0, 4.0], [0.0, -3.0, -3.0]], [[-3.0, 6.0, 6.0], [-6.0, -7.0, -3.0], [-7.0, -3.0, -3.0], [-4.0, 4.0, -5.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[0.0, 40.0], [61.0, 97.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([5.0, 0.0])
      s1=numpy.array([[1.0, 5.0, 6.0, 3.0, -3.0], [-1.0, 1.0, -3.0, 5.0, -2.0], [-7.0, -3.0, 6.0, 0.0, 7.0], [0.0, -5.0, 3.0, 1.0, 0.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[5.0, 25.0, 30.0, 15.0, -15.0], [-5.0, 5.0, -15.0, 25.0, -10.0], [-35.0, -15.0, 30.0, 0.0, 35.0], [0.0, -25.0, 15.0, 5.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-2.0, 7.0, 0.0], [-4.0, -4.0, -2.0]])
      s1=numpy.array([[[0.0, 7.0, -3.0], [-4.0, 1.0, 5.0], [0.0, -2.0, 0.0], [-5.0, 1.0, -5.0], [-4.0, -5.0, 0.0]], [[5.0, 0.0, -7.0], [6.0, 4.0, -6.0], [0.0, 1.0, -6.0], [-3.0, -5.0, -6.0], [4.0, -6.0, -3.0]], [[2.0, 6.0, -2.0], [2.0, -2.0, 4.0], [-5.0, -1.0, 7.0], [0.0, 0.0, -1.0], [2.0, 5.0, -7.0]], [[4.0, -1.0, -1.0], [0.0, 2.0, 6.0], [2.0, 2.0, -1.0], [-4.0, 0.0, 0.0], [-1.0, 5.0, 4.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[49.0, 15.0, -14.0, 17.0, -27.0], [-10.0, 16.0, 7.0, -29.0, -50.0], [38.0, -18.0, 3.0, 0.0, 31.0], [-15.0, 14.0, 10.0, 8.0, 37.0]], [[-22.0, 2.0, 8.0, 26.0, 36.0], [-6.0, -28.0, 8.0, 44.0, 14.0], [-28.0, -8.0, 10.0, 2.0, -14.0], [-10.0, -20.0, -14.0, 16.0, -24.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[6.0, -7.0, 4.0], [-1.0, -3.0, 6.0]], [[4.0, -2.0, 2.0], [-6.0, -2.0, -1.0]]])
      s1=numpy.array([[[[-4.0, -7.0, -4.0], [6.0, -4.0, -2.0]], [[-1.0, 0.0, 2.0], [-2.0, 1.0, -3.0]], [[0.0, -3.0, 0.0], [-7.0, 4.0, 0.0]], [[-3.0, -6.0, -3.0], [1.0, 3.0, 4.0]], [[1.0, 2.0, 3.0], [6.0, -5.0, 3.0]]], [[[-5.0, 2.0, -6.0], [4.0, -1.0, -7.0]], [[0.0, -7.0, 0.0], [7.0, -6.0, -4.0]], [[1.0, -3.0, -2.0], [3.0, 6.0, -5.0]], [[4.0, -4.0, -2.0], [7.0, 6.0, 5.0]], [[0.0, 0.0, 7.0], [1.0, -2.0, 3.0]]], [[[-3.0, -1.0, 7.0], [0.0, 7.0, -3.0]], [[3.0, -3.0, 0.0], [-7.0, 1.0, -7.0]], [[-4.0, 2.0, 4.0], [-3.0, -2.0, -5.0]], [[-1.0, 0.0, -2.0], [-3.0, 1.0, -3.0]], [[0.0, -4.0, 4.0], [4.0, -3.0, 1.0]]], [[[0.0, -7.0, -2.0], [0.0, -4.0, -3.0]], [[4.0, -3.0, 0.0], [7.0, 7.0, 4.0]], [[7.0, 0.0, 0.0], [-7.0, 1.0, 3.0]], [[2.0, 0.0, 3.0], [-6.0, -7.0, -6.0]], [[5.0, 5.0, -3.0], [-2.0, -6.0, 2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[3.0, -17.0, 16.0, 26.0, 31.0], [-111.0, 36.0, -32.0, 49.0, 51.0], [-22.0, 1.0, -43.0, -32.0, 55.0], [35.0, 41.0, 64.0, 15.0, 15.0]], [[-36.0, 13.0, 40.0, -22.0, -23.0], [-51.0, -12.0, -19.0, -39.0, 9.0], [-7.0, 65.0, 15.0, 11.0, -3.0], [21.0, -38.0, 65.0, 70.0, 26.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_Symbol_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-3.0, 5.0])
      s1=numpy.array([[[0.0, 1.0], [-6.0, -6.0]], [[-2.0, 7.0], [0.0, 4.0]], [[5.0, -5.0], [-1.0, -3.0]], [[-2.0, -6.0], [6.0, 6.0]], [[3.0, 2.0], [-6.0, -5.0]], [[-2.0, -3.0], [0.0, 2.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0, -3.0], [18.0, 18.0]], [[6.0, -21.0], [0.0, -12.0]], [[-15.0, 15.0], [3.0, 9.0]], [[6.0, 18.0], [-18.0, -18.0]], [[-9.0, -6.0], [18.0, 15.0]], [[6.0, 9.0], [0.0, -6.0]]], [[[0.0, 5.0], [-30.0, -30.0]], [[-10.0, 35.0], [0.0, 20.0]], [[25.0, -25.0], [-5.0, -15.0]], [[-10.0, -30.0], [30.0, 30.0]], [[15.0, 10.0], [-30.0, -25.0]], [[-10.0, -15.0], [0.0, 10.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-5.0, 7.0, 0.0], [5.0, -5.0, -6.0]])
      s1=numpy.array([[[[-2.0, -4.0, -4.0], [6.0, -1.0, 4.0]], [[-6.0, -5.0, 7.0], [-6.0, -4.0, -5.0]]], [[[6.0, 2.0, -1.0], [-7.0, 6.0, 4.0]], [[-7.0, -5.0, 3.0], [7.0, -3.0, 0.0]]], [[[6.0, -3.0, 3.0], [-7.0, 6.0, -6.0]], [[2.0, 1.0, -5.0], [6.0, 0.0, -6.0]]], [[[2.0, -6.0, 6.0], [0.0, 1.0, -7.0]], [[-4.0, -6.0, 7.0], [-7.0, 6.0, 7.0]]], [[[7.0, -5.0, 0.0], [1.0, 4.0, -6.0]], [[-4.0, -2.0, -4.0], [-3.0, -2.0, -1.0]]], [[[7.0, 0.0, 7.0], [-3.0, 1.0, 4.0]], [[-4.0, 7.0, -7.0], [-1.0, 1.0, -2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-18.0, -37.0], [-5.0, 2.0]], [[-16.0, 77.0], [0.0, -56.0]], [[-51.0, 77.0], [-3.0, -30.0]], [[-52.0, 7.0], [-22.0, 77.0]], [[-70.0, 23.0], [6.0, 1.0]], [[-35.0, 22.0], [69.0, 12.0]]], [[[34.0, 11.0], [-47.0, 20.0]], [[26.0, -89.0], [-28.0, 50.0]], [[27.0, -29.0], [35.0, 66.0]], [[4.0, 37.0], [-32.0, -107.0]], [[60.0, 21.0], [14.0, 1.0]], [[-7.0, -44.0], [-13.0, 2.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(3.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([6.0, 6.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([18.0, 18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([2.0, 3.0, 5.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[5.0, 5.0, -6.0], [5.0, 7.0, -4.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-5.0, 11.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[5.0, -6.0, -2.0], [7.0, 1.0, 1.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[1.0, 3.0, -1.0], [-3.0, -3.0, 6.0]], [[-2.0, 1.0, -6.0], [3.0, -3.0, 3.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-29.0, 17.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Data(numpy.array([[[-6.0, 6.0, 0.0], [-2.0, 2.0, 5.0], [5.0, -6.0, 6.0], [-2.0, -6.0, 4.0]], [[-5.0, 4.0, 2.0], [0.0, 7.0, -7.0], [7.0, -1.0, 0.0], [-4.0, -2.0, -1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[1.0, -5.0, -6.0], [3.0, -2.0, -7.0], [0.0, -3.0, -5.0], [5.0, 0.0, 7.0]], [[2.0, -7.0, -7.0], [-1.0, 7.0, 1.0], [6.0, 7.0, -2.0], [7.0, 1.0, -3.0]]], [[[3.0, 0.0, 1.0], [6.0, -5.0, 1.0], [-3.0, 2.0, 1.0], [3.0, 0.0, 1.0]], [[0.0, -1.0, 6.0], [3.0, -5.0, 2.0], [3.0, -1.0, -3.0], [1.0, -4.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([-77.0, -67.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([-2.0, 1.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([2.0, 7.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-4.0, 2.0], [-14.0, 7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[1.0, 7.0, 5.0], [0.0, 6.0, -7.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[6.0, -7.0, -3.0], [5.0, 4.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-58.0, -21.0], [33.0, 24.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[[-4.0, -2.0, -6.0], [-5.0, -2.0, -6.0]], [[5.0, -1.0, 4.0], [-2.0, -1.0, -7.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-7.0, -3.0, 5.0], [5.0, -2.0, -7.0]], [[2.0, -1.0, 1.0], [0.0, -1.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[25.0, 29.0], [20.0, 51.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      arg1=Data(numpy.array([[[[-6.0, -1.0, -1.0], [6.0, 1.0, 6.0], [-4.0, 1.0, -2.0], [7.0, 1.0, 6.0]], [[6.0, 0.0, -4.0], [5.0, 5.0, -4.0], [-2.0, 6.0, 7.0], [6.0, 0.0, -2.0]]], [[[-2.0, 7.0, -5.0], [-2.0, -4.0, -7.0], [0.0, 3.0, 7.0], [2.0, 3.0, 6.0]], [[1.0, 6.0, 0.0], [0.0, -7.0, 0.0], [0.0, 3.0, 1.0], [7.0, 6.0, 4.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-1.0, -5.0, 7.0], [-6.0, 4.0, 6.0], [0.0, -7.0, -2.0], [-5.0, 5.0, -6.0]], [[5.0, 5.0, -4.0], [-7.0, 7.0, 3.0], [-3.0, -6.0, -2.0], [3.0, 1.0, -3.0]]], [[[-2.0, 3.0, -2.0], [4.0, 3.0, -2.0], [4.0, 0.0, 5.0], [5.0, -1.0, -7.0]], [[-4.0, -5.0, -4.0], [-4.0, 7.0, 4.0], [7.0, 2.0, 2.0], [6.0, -7.0, -2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-47.0, -199.0], [35.0, -54.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[3.0, 0.0, 3.0, 3.0, -1.0], [-2.0, -6.0, 2.0, -7.0, -4.0], [2.0, 5.0, 0.0, -6.0, -1.0], [7.0, -4.0, 2.0, -2.0, 5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-5.0, 4.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-15.0, 0.0, -15.0, -15.0, 5.0], [10.0, 30.0, -10.0, 35.0, 20.0], [-10.0, -25.0, 0.0, 30.0, 5.0], [-35.0, 20.0, -10.0, 10.0, -25.0]], [[12.0, 0.0, 12.0, 12.0, -4.0], [-8.0, -24.0, 8.0, -28.0, -16.0], [8.0, 20.0, 0.0, -24.0, -4.0], [28.0, -16.0, 8.0, -8.0, 20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[-3.0, 6.0, -1.0], [5.0, -5.0, 1.0], [7.0, 5.0, 0.0], [3.0, 2.0, 7.0], [-6.0, 0.0, -5.0]], [[7.0, -2.0, 5.0], [7.0, 3.0, -2.0], [0.0, -4.0, -3.0], [7.0, 1.0, 0.0], [3.0, 2.0, -1.0]], [[-1.0, 2.0, 5.0], [0.0, 5.0, -7.0], [3.0, -7.0, 5.0], [7.0, 4.0, -5.0], [-2.0, 7.0, 3.0]], [[-3.0, -5.0, 0.0], [2.0, -7.0, -7.0], [7.0, -7.0, -4.0], [-5.0, 6.0, 0.0], [5.0, -5.0, 3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[1.0, 7.0, 7.0], [-4.0, -1.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[32.0, -23.0, 42.0, 66.0, -41.0], [28.0, 14.0, -49.0, 14.0, 10.0], [48.0, -14.0, -11.0, 0.0, 68.0], [-38.0, -96.0, -70.0, 37.0, -9.0]], [[13.0, -22.0, -33.0, -63.0, 59.0], [-61.0, -17.0, 25.0, -29.0, -7.0], [-33.0, 44.0, -40.0, 3.0, -20.0], [17.0, 48.0, 7.0, 14.0, -36.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      arg1=Data(numpy.array([[[[2.0, 2.0, 6.0], [-2.0, -6.0, 7.0]], [[-2.0, 3.0, -3.0], [-4.0, -4.0, 7.0]], [[-7.0, 0.0, -2.0], [7.0, 0.0, -3.0]], [[-1.0, -3.0, 0.0], [7.0, -4.0, 0.0]], [[-3.0, 2.0, 1.0], [7.0, 4.0, 1.0]]], [[[6.0, 0.0, -2.0], [7.0, 0.0, 0.0]], [[-1.0, -2.0, -3.0], [-6.0, -6.0, -3.0]], [[6.0, -6.0, 5.0], [5.0, -2.0, 1.0]], [[2.0, -3.0, -2.0], [2.0, -3.0, 4.0]], [[-2.0, 0.0, -1.0], [5.0, -2.0, -1.0]]], [[[-3.0, 2.0, 7.0], [-4.0, 4.0, -7.0]], [[-4.0, 1.0, -2.0], [-1.0, -6.0, -5.0]], [[-1.0, 7.0, 2.0], [7.0, 7.0, -1.0]], [[-2.0, 7.0, 2.0], [-4.0, 6.0, 0.0]], [[-1.0, 4.0, -1.0], [-4.0, -5.0, 5.0]]], [[[6.0, 3.0, 3.0], [-5.0, 7.0, 6.0]], [[0.0, 0.0, -5.0], [2.0, 1.0, -2.0]], [[7.0, -1.0, 0.0], [-3.0, 5.0, -1.0]], [[3.0, -4.0, -3.0], [-6.0, -3.0, 1.0]], [[-6.0, -4.0, -7.0], [-3.0, -2.0, -5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-6.0, -5.0, -1.0], [-5.0, -1.0, -6.0]], [[-2.0, -2.0, -4.0], [-5.0, 2.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-54.0, -18.0, 27.0, -10.0, -38.0], [-69.0, 73.0, -40.0, -26.0, -4.0], [59.0, 62.0, -67.0, -11.0, -18.0], [-72.0, 6.0, -21.0, 32.0, 110.0]], [[-41.0, 15.0, -10.0, -35.0, -30.0], [-39.0, 39.0, -50.0, -10.0, -20.0], [9.0, 12.0, -40.0, 14.0, 3.0], [3.0, 14.0, 14.0, 37.0, 64.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_constData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      arg1=Data(numpy.array([[[-1.0, -2.0], [3.0, 3.0]], [[0.0, 5.0], [7.0, -5.0]], [[4.0, 6.0], [-2.0, -7.0]], [[1.0, 6.0], [4.0, 2.0]], [[6.0, 4.0], [2.0, 7.0]], [[-1.0, 3.0], [0.0, 0.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([4.0, -3.0])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-4.0, -8.0], [12.0, 12.0]], [[0.0, 20.0], [28.0, -20.0]], [[16.0, 24.0], [-8.0, -28.0]], [[4.0, 24.0], [16.0, 8.0]], [[24.0, 16.0], [8.0, 28.0]], [[-4.0, 12.0], [0.0, 0.0]]], [[[3.0, 6.0], [-9.0, -9.0]], [[0.0, -15.0], [-21.0, 15.0]], [[-12.0, -18.0], [6.0, 21.0]], [[-3.0, -18.0], [-12.0, -6.0]], [[-18.0, -12.0], [-6.0, -21.0]], [[3.0, -9.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      arg1=Data(numpy.array([[[[0.0, -3.0, -7.0], [-6.0, -3.0, -4.0]], [[5.0, 3.0, 2.0], [3.0, 0.0, -4.0]]], [[[6.0, 6.0, 1.0], [7.0, -6.0, 1.0]], [[-6.0, 0.0, 3.0], [1.0, -5.0, 2.0]]], [[[5.0, 3.0, -3.0], [1.0, 7.0, -6.0]], [[-3.0, 4.0, -7.0], [0.0, 2.0, -7.0]]], [[[0.0, -1.0, -7.0], [3.0, 6.0, -4.0]], [[1.0, 2.0, -4.0], [1.0, -1.0, -7.0]]], [[[2.0, 1.0, -6.0], [7.0, 2.0, -6.0]], [[1.0, -6.0, -2.0], [1.0, -2.0, -6.0]]], [[[-4.0, -7.0, 1.0], [-3.0, 0.0, -4.0]], [[-5.0, -1.0, -1.0], [-2.0, -6.0, -1.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[3.0, 4.0, -2.0], [-3.0, 1.0, 1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[2.0, -22.0], [23.0, 17.0]], [[40.0, -5.0], [-24.0, -21.0]], [[33.0, 43.0], [21.0, 22.0]], [[10.0, 41.0], [19.0, 13.0]], [[22.0, 41.0], [-17.0, 7.0]], [[-42.0, -1.0], [-17.0, -28.0]]], [[[-10.0, 11.0], [-10.0, -13.0]], [[-11.0, -26.0], [21.0, -6.0]], [[-15.0, -2.0], [6.0, -5.0]], [[-8.0, -7.0], [-5.0, -11.0]], [[-11.0, -25.0], [-11.0, -11.0]], [[6.0, 5.0], [13.0, -1.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-1.0, 3.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, 3.0])+(1.-msk_ref)*numpy.array([-2.0, 6.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, -7.0, -7.0])+(1.-msk_arg1)*numpy.array([-6.0, 4.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-6.0, 7.0, -1.0], [-4.0, -7.0, 7.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-42.0, 0.0])+(1.-msk_ref)*numpy.array([63.0, 3.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 0.0, -5.0], [-6.0, -3.0, -7.0]])+(1.-msk_arg1)*numpy.array([[-1.0, 0.0, 2.0], [-7.0, -3.0, 5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-5.0, 3.0, -5.0], [-4.0, 6.0, -1.0]], [[-5.0, -5.0, 3.0], [-4.0, -2.0, 4.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([68.0, 17.0])+(1.-msk_ref)*numpy.array([0.0, 65.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank3_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, -2.0, -7.0], [-4.0, -3.0, -3.0], [-1.0, -4.0, 5.0], [3.0, -7.0, -3.0]], [[-3.0, 1.0, -2.0], [2.0, 4.0, 7.0], [3.0, 5.0, 0.0], [3.0, 2.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[4.0, 0.0, 0.0], [3.0, 2.0, -2.0], [2.0, -4.0, 6.0], [0.0, 6.0, -3.0]], [[6.0, -1.0, 0.0], [1.0, 0.0, 1.0], [2.0, -3.0, -5.0], [2.0, 5.0, -4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-3.0, -6.0, 3.0], [-2.0, 0.0, 1.0], [-4.0, 6.0, 7.0], [2.0, -7.0, -3.0]], [[-2.0, 1.0, 6.0], [-7.0, 5.0, 1.0], [-1.0, -6.0, -7.0], [3.0, -6.0, 6.0]]], [[[6.0, 3.0, -5.0], [5.0, 3.0, -1.0], [-5.0, -2.0, 7.0], [-4.0, -2.0, 1.0]], [[6.0, -3.0, 4.0], [5.0, 3.0, -5.0], [6.0, -5.0, 2.0], [-3.0, -3.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([59.0, -38.0])+(1.-msk_ref)*numpy.array([-59.0, 111.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 3.0])+(1.-msk_arg1)*numpy.array([1.0, -5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([6.0, 6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[42.0, 18.0], [42.0, 18.0]])+(1.-msk_ref)*numpy.array([[6.0, -30.0], [6.0, -30.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 4.0, 1.0], [0.0, -1.0, -6.0]])+(1.-msk_arg1)*numpy.array([[5.0, -2.0, 3.0], [7.0, 7.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-7.0, 0.0, -3.0], [-5.0, -1.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-3.0, 18.0], [-3.0, -5.0]])+(1.-msk_ref)*numpy.array([[-44.0, -49.0], [-20.0, -42.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 0.0, -6.0], [-3.0, 7.0, 0.0]], [[0.0, -7.0, -3.0], [-5.0, -4.0, 5.0]]])+(1.-msk_arg1)*numpy.array([[[-2.0, 3.0, 3.0], [4.0, 1.0, 7.0]], [[-3.0, 4.0, 5.0], [-1.0, 7.0, -4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[-5.0, 3.0, 6.0], [-5.0, 7.0, 2.0]], [[-5.0, 0.0, -5.0], [0.0, -2.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[28.0, -32.0], [16.0, 23.0]])+(1.-msk_ref)*numpy.array([[38.0, 103.0], [-7.0, -24.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank4_offset3(self):
      arg0=Symbol(shape=(2, 2, 4, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[4.0, -3.0, 6.0], [2.0, -6.0, -3.0], [-1.0, -3.0, 6.0], [4.0, -4.0, 7.0]], [[-7.0, -4.0, 7.0], [-5.0, -6.0, 6.0], [3.0, 7.0, 3.0], [3.0, -1.0, 0.0]]], [[[-7.0, 4.0, 6.0], [5.0, 7.0, 2.0], [-3.0, -2.0, -4.0], [1.0, 0.0, 3.0]], [[-6.0, -7.0, 7.0], [-6.0, 0.0, 6.0], [1.0, 0.0, 6.0], [0.0, 6.0, 6.0]]]])+(1.-msk_arg1)*numpy.array([[[[2.0, 0.0, 7.0], [7.0, 5.0, 0.0], [2.0, 1.0, -6.0], [6.0, -7.0, 1.0]], [[5.0, -4.0, -1.0], [0.0, 2.0, -2.0], [-2.0, 1.0, 7.0], [-2.0, 0.0, -5.0]]], [[[-4.0, 1.0, 5.0], [-7.0, -2.0, 6.0], [-1.0, 5.0, 0.0], [5.0, 0.0, -1.0]], [[4.0, -3.0, -6.0], [-4.0, 5.0, -2.0], [2.0, -6.0, 2.0], [-6.0, -2.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s0=numpy.array([[[[-2.0, 7.0, -7.0], [4.0, -5.0, -7.0], [-6.0, -5.0, 2.0], [-4.0, 6.0, -4.0]], [[-1.0, -4.0, 0.0], [-4.0, 2.0, 0.0], [1.0, -1.0, 3.0], [0.0, -3.0, 3.0]]], [[[-3.0, 2.0, 3.0], [-4.0, -5.0, 7.0], [6.0, 3.0, -4.0], [4.0, 0.0, 2.0]], [[6.0, -6.0, -6.0], [0.0, 1.0, 6.0], [-3.0, -2.0, 3.0], [-7.0, 4.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, 52.0], [-77.0, 71.0]])+(1.-msk_ref)*numpy.array([[-131.0, -49.0], [96.0, 269.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 0.0, -5.0, -6.0, 0.0], [7.0, -1.0, 2.0, 1.0, -1.0], [-4.0, -7.0, -3.0, 2.0, -3.0], [0.0, -6.0, 7.0, 4.0, -4.0]])+(1.-msk_arg1)*numpy.array([[2.0, 4.0, -1.0, -1.0, 0.0], [1.0, -6.0, 6.0, -1.0, -3.0], [1.0, 0.0, -6.0, -7.0, 5.0], [2.0, 3.0, -5.0, -3.0, -4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([-6.0, 3.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 0.0, 30.0, 36.0, 0.0], [-42.0, 6.0, -12.0, -6.0, 6.0], [24.0, 42.0, 18.0, -12.0, 18.0], [0.0, 36.0, -42.0, -24.0, 24.0]], [[0.0, 0.0, -15.0, -18.0, 0.0], [21.0, -3.0, 6.0, 3.0, -3.0], [-12.0, -21.0, -9.0, 6.0, -9.0], [0.0, -18.0, 21.0, 12.0, -12.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, -24.0, 6.0, 6.0, 0.0], [-6.0, 36.0, -36.0, 6.0, 18.0], [-6.0, 0.0, 36.0, 42.0, -30.0], [-12.0, -18.0, 30.0, 18.0, 24.0]], [[6.0, 12.0, -3.0, -3.0, 0.0], [3.0, -18.0, 18.0, -3.0, -9.0], [3.0, 0.0, -18.0, -21.0, 15.0], [6.0, 9.0, -15.0, -9.0, -12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 1.0, 5.0], [4.0, -6.0, 5.0], [1.0, 7.0, -6.0], [0.0, 7.0, 4.0], [-4.0, 6.0, -1.0]], [[-6.0, -1.0, -3.0], [6.0, -6.0, -3.0], [5.0, -2.0, -6.0], [3.0, 4.0, 4.0], [2.0, 7.0, -5.0]], [[7.0, 4.0, -3.0], [6.0, 5.0, 4.0], [-6.0, -7.0, -4.0], [1.0, -3.0, -3.0], [2.0, 2.0, 0.0]], [[2.0, 0.0, 0.0], [-1.0, 1.0, -3.0], [4.0, 4.0, -3.0], [3.0, -6.0, -1.0], [-4.0, 6.0, -1.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, 0.0, -6.0], [-1.0, 5.0, -6.0], [-4.0, -7.0, 0.0], [-7.0, 5.0, -2.0], [-3.0, 2.0, -4.0]], [[6.0, 7.0, 4.0], [4.0, 6.0, 3.0], [0.0, 1.0, -1.0], [-1.0, 5.0, -1.0], [-2.0, 3.0, 1.0]], [[0.0, 7.0, 4.0], [7.0, 1.0, 0.0], [-2.0, 4.0, 2.0], [0.0, 6.0, 2.0], [6.0, -2.0, -2.0]], [[5.0, 2.0, -6.0], [2.0, 0.0, 6.0], [-2.0, 2.0, 4.0], [-7.0, 4.0, 2.0], [0.0, 2.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-2.0, 2.0, 1.0], [4.0, -4.0, -4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, -15.0, 6.0, 18.0, 19.0], [7.0, -27.0, -20.0, 6.0, 5.0], [-9.0, 2.0, -6.0, -11.0, 0.0], [-4.0, 1.0, -3.0, -19.0, 19.0]], [[-28.0, 20.0, 0.0, -44.0, -36.0], [-8.0, 60.0, 52.0, -20.0, 0.0], [24.0, -12.0, 20.0, 28.0, 0.0], [8.0, 4.0, 12.0, 40.0, -36.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, 6.0, -6.0, 22.0, 6.0], [6.0, 7.0, 1.0, 11.0, 11.0], [18.0, -12.0, 14.0, 14.0, -18.0], [-12.0, 2.0, 12.0, 24.0, 4.0]], [[32.0, 0.0, 12.0, -40.0, -4.0], [-20.0, -20.0, 0.0, -20.0, -24.0], [-44.0, 24.0, -32.0, -32.0, 40.0], [36.0, -16.0, -32.0, -52.0, -8.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-6.0, -2.0, 1.0], [-3.0, 6.0, -6.0]], [[-6.0, -2.0, 0.0], [4.0, -2.0, 4.0]], [[-7.0, 0.0, 6.0], [0.0, -4.0, 3.0]], [[1.0, 0.0, -7.0], [7.0, -3.0, 0.0]], [[5.0, -1.0, 1.0], [5.0, 7.0, -2.0]]], [[[-5.0, 6.0, 6.0], [-2.0, 2.0, 0.0]], [[7.0, -1.0, 3.0], [-5.0, 1.0, 5.0]], [[3.0, -4.0, -7.0], [1.0, 2.0, -4.0]], [[7.0, -7.0, 4.0], [2.0, 6.0, 0.0]], [[3.0, 2.0, -4.0], [-5.0, 7.0, 3.0]]], [[[0.0, 4.0, -4.0], [2.0, 3.0, -3.0]], [[3.0, 3.0, -2.0], [-1.0, 0.0, 2.0]], [[-6.0, 7.0, 7.0], [-2.0, 4.0, 6.0]], [[-2.0, 1.0, -2.0], [-7.0, 6.0, 6.0]], [[-7.0, 1.0, 3.0], [-1.0, 0.0, 5.0]]], [[[-4.0, -4.0, 5.0], [-6.0, -6.0, 1.0]], [[4.0, -4.0, 6.0], [5.0, -5.0, 7.0]], [[-3.0, -3.0, 0.0], [-1.0, 4.0, 0.0]], [[7.0, 1.0, -2.0], [3.0, -7.0, 0.0]], [[-1.0, -5.0, -7.0], [3.0, -1.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, 4.0, 3.0], [7.0, 3.0, 0.0]], [[-2.0, 5.0, 2.0], [4.0, 1.0, 3.0]], [[4.0, 0.0, 7.0], [-3.0, 1.0, 5.0]], [[-6.0, 0.0, 0.0], [7.0, 1.0, 4.0]], [[-6.0, 0.0, 0.0], [-6.0, 0.0, 6.0]]], [[[-5.0, -5.0, -6.0], [-7.0, 4.0, 2.0]], [[0.0, 7.0, 5.0], [3.0, -1.0, -3.0]], [[5.0, -7.0, -4.0], [5.0, 0.0, -4.0]], [[-2.0, -1.0, -6.0], [4.0, 6.0, -4.0]], [[1.0, -3.0, 3.0], [4.0, -5.0, -6.0]]], [[[4.0, -5.0, 0.0], [-4.0, -6.0, -2.0]], [[1.0, 1.0, -3.0], [4.0, -6.0, -2.0]], [[5.0, 0.0, 7.0], [-2.0, -6.0, 6.0]], [[-7.0, 0.0, 4.0], [-1.0, 2.0, -3.0]], [[6.0, 0.0, -3.0], [-7.0, -1.0, 6.0]]], [[[-7.0, -4.0, -7.0], [-4.0, -4.0, 0.0]], [[2.0, 6.0, 5.0], [4.0, 6.0, -4.0]], [[0.0, -5.0, -7.0], [-7.0, 3.0, 6.0]], [[-1.0, 1.0, -6.0], [-6.0, -6.0, -6.0]], [[4.0, 7.0, -6.0], [-3.0, 1.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[4.0, 5.0, 7.0], [-5.0, 5.0, -5.0]], [[2.0, 1.0, -2.0], [0.0, -2.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[48.0, -84.0, -21.0, -95.0, 42.0], [72.0, 49.0, -32.0, 41.0, 39.0], [12.0, 8.0, 60.0, 18.0, -22.0], [-6.0, -47.0, -2.0, -31.0, -113.0]], [[-40.0, -2.0, -12.0, 22.0, -11.0], [-20.0, 15.0, 4.0, -13.0, 8.0], [0.0, 17.0, -15.0, 1.0, -9.0], [-8.0, 16.0, -17.0, 33.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[25.0, 1.0, 60.0, -74.0, -24.0], [-42.0, 65.0, -48.0, -25.0, -5.0], [-9.0, -52.0, 19.0, 30.0, 3.0], [-97.0, 103.0, -54.0, -11.0, 9.0]], [[-6.0, 1.0, 2.0, -6.0, 0.0], [-7.0, -7.0, 3.0, -13.0, -9.0], [11.0, 17.0, 20.0, -32.0, 32.0], [4.0, -20.0, 15.0, 11.0, 33.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank1_expandedData_rank3_offset0(self):
      arg0=Symbol(shape=(2,))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-2.0, 0.0], [-3.0, 6.0]], [[0.0, 3.0], [0.0, 5.0]], [[-3.0, 5.0], [-3.0, -3.0]], [[-5.0, -3.0], [1.0, 6.0]], [[0.0, -1.0], [4.0, -1.0]], [[-6.0, 1.0], [-3.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -4.0], [-7.0, -1.0]], [[-5.0, -1.0], [-6.0, -1.0]], [[0.0, 5.0], [7.0, 0.0]], [[-7.0, -4.0], [1.0, 0.0]], [[-7.0, 1.0], [6.0, 6.0]], [[0.0, 5.0], [0.0, -3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([1.0, 6.0])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-2.0, 0.0], [-3.0, 6.0]], [[0.0, 3.0], [0.0, 5.0]], [[-3.0, 5.0], [-3.0, -3.0]], [[-5.0, -3.0], [1.0, 6.0]], [[0.0, -1.0], [4.0, -1.0]], [[-6.0, 1.0], [-3.0, -6.0]]], [[[-12.0, 0.0], [-18.0, 36.0]], [[0.0, 18.0], [0.0, 30.0]], [[-18.0, 30.0], [-18.0, -18.0]], [[-30.0, -18.0], [6.0, 36.0]], [[0.0, -6.0], [24.0, -6.0]], [[-36.0, 6.0], [-18.0, -36.0]]]])+(1.-msk_ref)*numpy.array([[[[-5.0, -4.0], [-7.0, -1.0]], [[-5.0, -1.0], [-6.0, -1.0]], [[0.0, 5.0], [7.0, 0.0]], [[-7.0, -4.0], [1.0, 0.0]], [[-7.0, 1.0], [6.0, 6.0]], [[0.0, 5.0], [0.0, -3.0]]], [[[-30.0, -24.0], [-42.0, -6.0]], [[-30.0, -6.0], [-36.0, -6.0]], [[0.0, 30.0], [42.0, 0.0]], [[-42.0, -24.0], [6.0, 0.0]], [[-42.0, 6.0], [36.0, 36.0]], [[0.0, 30.0], [0.0, -18.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank4_offset1(self):
      arg0=Symbol(shape=(2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -6.0, -2.0], [2.0, 7.0, 5.0]], [[-4.0, -2.0, 5.0], [-4.0, -5.0, 6.0]]], [[[6.0, 0.0, 1.0], [-7.0, -3.0, 3.0]], [[5.0, 4.0, -2.0], [6.0, -1.0, -3.0]]], [[[-6.0, 6.0, 0.0], [-1.0, -3.0, 0.0]], [[-3.0, -4.0, 2.0], [4.0, 2.0, 0.0]]], [[[0.0, 2.0, 0.0], [2.0, 7.0, -2.0]], [[6.0, -3.0, 1.0], [-1.0, -4.0, 1.0]]], [[[3.0, 0.0, 0.0], [-4.0, -1.0, 0.0]], [[-2.0, -7.0, 4.0], [0.0, 1.0, 1.0]]], [[[-4.0, 5.0, -3.0], [0.0, -6.0, -4.0]], [[5.0, 7.0, -7.0], [-3.0, -4.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[5.0, 0.0, 0.0], [-4.0, 1.0, -4.0]], [[5.0, 1.0, 6.0], [5.0, 1.0, -6.0]]], [[[0.0, 4.0, 3.0], [1.0, -6.0, 7.0]], [[-4.0, 0.0, -1.0], [-5.0, 5.0, -7.0]]], [[[2.0, 3.0, -7.0], [-6.0, -5.0, -2.0]], [[6.0, -2.0, -3.0], [1.0, -4.0, -2.0]]], [[[1.0, 0.0, 0.0], [1.0, 7.0, 0.0]], [[0.0, 1.0, 1.0], [-7.0, 0.0, -5.0]]], [[[0.0, 4.0, -2.0], [-1.0, 3.0, -4.0]], [[6.0, -3.0, -7.0], [-1.0, -1.0, 2.0]]], [[[-1.0, 1.0, -1.0], [-1.0, 4.0, -7.0]], [[-5.0, 2.0, 0.0], [-2.0, -4.0, -6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[-3.0, 3.0, -3.0], [2.0, 3.0, 1.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 0.0], [-9.0, -21.0]], [[-21.0, 3.0], [3.0, -12.0]], [[36.0, -6.0], [-9.0, -6.0]], [[6.0, 21.0], [-30.0, -12.0]], [[-9.0, 9.0], [-27.0, 0.0]], [[36.0, -6.0], [27.0, 15.0]]], [[[-30.0, 30.0], [-9.0, -17.0]], [[13.0, -20.0], [20.0, 6.0]], [[6.0, -11.0], [-16.0, 14.0]], [[6.0, 23.0], [4.0, -13.0]], [[6.0, -11.0], [-21.0, 4.0]], [[4.0, -22.0], [24.0, -24.0]]]])+(1.-msk_ref)*numpy.array([[[[-15.0, 27.0], [-30.0, 6.0]], [[3.0, -42.0], [15.0, 51.0]], [[24.0, 9.0], [-15.0, -9.0]], [[-3.0, 18.0], [0.0, 36.0]], [[18.0, 24.0], [-6.0, -6.0]], [[9.0, 36.0], [21.0, 12.0]]], [[[10.0, -9.0], [19.0, 7.0]], [[15.0, -9.0], [-9.0, -2.0]], [[6.0, -29.0], [3.0, -12.0]], [[2.0, 23.0], [4.0, -19.0]], [[10.0, 3.0], [-4.0, -3.0]], [[0.0, 3.0], [-4.0, -22.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_float_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=-7.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-1.0, -7.0, 3.0, 5.0, -6.0], [0.0, 0.0, 0.0, 7.0, -3.0], [2.0, -7.0, 4.0, 0.0, 3.0], [6.0, 4.0, -4.0, -2.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[7.0, 49.0, -21.0, -35.0, 42.0], [-0.0, -0.0, -0.0, -49.0, 21.0], [-14.0, 49.0, -28.0, -0.0, -21.0], [-42.0, -28.0, 28.0, 14.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array(0.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[5.0, 2.0, -5.0, -4.0, 7.0], [-3.0, -7.0, 7.0, 7.0, -7.0], [-4.0, 1.0, -4.0, 1.0, 0.0], [2.0, 5.0, -7.0, -7.0, -7.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[0.0, 0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0, -0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([-6.0, 3.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[0.0, 2.0, 4.0], [2.0, 7.0, 4.0], [0.0, 6.0, 4.0], [0.0, 3.0, -2.0], [-1.0, 0.0, 2.0]], [[0.0, 2.0, 0.0], [4.0, 1.0, 0.0], [2.0, 1.0, 4.0], [0.0, -5.0, 2.0], [-5.0, 6.0, 1.0]], [[3.0, 3.0, 0.0], [-3.0, -6.0, -1.0], [6.0, -1.0, -7.0], [-7.0, -6.0, -5.0], [-1.0, 7.0, -6.0]], [[-7.0, -3.0, 2.0], [4.0, -6.0, 5.0], [-2.0, 4.0, 7.0], [-5.0, -6.0, 0.0], [-6.0, 0.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-18.0, -15.0, -6.0, 21.0, -6.0], [6.0, -21.0, -33.0, -27.0, 42.0], [-9.0, 6.0, 3.0, 54.0, 63.0], [21.0, -72.0, -18.0, 12.0, 66.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[4.0, -4.0, -4.0], [7.0, 2.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-6.0, 6.0, 5.0], [-1.0, 7.0, 2.0]], [[0.0, -5.0, -1.0], [0.0, 5.0, -4.0]], [[-4.0, -5.0, -6.0], [5.0, -1.0, -6.0]], [[3.0, -3.0, 4.0], [2.0, -3.0, 4.0]], [[6.0, 4.0, 1.0], [7.0, -2.0, 6.0]]], [[[0.0, 1.0, 3.0], [4.0, 0.0, 6.0]], [[-4.0, -5.0, 5.0], [-7.0, 4.0, 0.0]], [[-7.0, 2.0, -2.0], [-6.0, 5.0, -4.0]], [[-2.0, -1.0, -4.0], [-3.0, 7.0, -4.0]], [[-2.0, -1.0, 0.0], [-7.0, 3.0, -1.0]]], [[[-4.0, 7.0, 0.0], [0.0, -1.0, 7.0]], [[-7.0, -7.0, 0.0], [-2.0, -6.0, -6.0]], [[-6.0, 1.0, 6.0], [-3.0, -4.0, 5.0]], [[3.0, 0.0, -3.0], [-3.0, -1.0, -1.0]], [[-5.0, 0.0, -6.0], [-5.0, -3.0, -6.0]]], [[[0.0, 4.0, -2.0], [4.0, 4.0, -1.0]], [[1.0, 4.0, -2.0], [3.0, -4.0, 4.0]], [[3.0, 5.0, -4.0], [-3.0, -2.0, -3.0]], [[-3.0, -7.0, -5.0], [4.0, 5.0, 0.0]], [[-3.0, -1.0, 4.0], [2.0, 1.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[-61.0, 34.0, 61.0, 16.0, 49.0], [12.0, -57.0, -60.0, 5.0, -47.0], [-46.0, -26.0, -81.0, 1.0, -37.0], [28.0, 9.0, -17.0, 74.0, -8.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([-2.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[5.0, 7.0, -1.0, -4.0, 5.0], [-7.0, 2.0, 5.0, 5.0, -1.0], [3.0, 1.0, 0.0, -5.0, 4.0], [0.0, -1.0, 2.0, -7.0, 0.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-10.0, 30.0], [-14.0, 42.0], [2.0, -6.0], [8.0, -24.0], [-10.0, 30.0]], [[14.0, -42.0], [-4.0, 12.0], [-10.0, 30.0], [-10.0, 30.0], [2.0, -6.0]], [[-6.0, 18.0], [-2.0, 6.0], [0.0, 0.0], [10.0, -30.0], [-8.0, 24.0]], [[0.0, 0.0], [2.0, -6.0], [-4.0, 12.0], [14.0, -42.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([[7.0, -4.0, 6.0], [-2.0, 6.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, 1.0, -1.0], [0.0, 5.0, 6.0], [4.0, 0.0, 4.0], [3.0, -5.0, 0.0], [3.0, 4.0, -4.0]], [[-3.0, -2.0, 0.0], [0.0, 0.0, -3.0], [-2.0, -5.0, -7.0], [-4.0, -6.0, -4.0], [-7.0, -7.0, 7.0]], [[-1.0, 3.0, 2.0], [-1.0, -4.0, -1.0], [1.0, 4.0, 2.0], [1.0, -2.0, -4.0], [1.0, -1.0, -3.0]], [[0.0, -5.0, -5.0], [2.0, 1.0, -7.0], [2.0, 2.0, 1.0], [-1.0, -1.0, 2.0], [5.0, -3.0, 2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[25.0, -4.0], [16.0, 30.0], [52.0, -8.0], [41.0, -36.0], [-19.0, 18.0]], [[-13.0, -6.0], [-18.0, 0.0], [-36.0, -26.0], [-28.0, -28.0], [21.0, -28.0]], [[-7.0, 20.0], [3.0, -22.0], [3.0, 22.0], [-9.0, -14.0], [-7.0, -8.0]], [[-10.0, -30.0], [-32.0, 2.0], [12.0, 8.0], [9.0, -4.0], [59.0, -28.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[0.0, 5.0, 7.0], [-4.0, 6.0, 2.0]], [[0.0, -2.0, 6.0], [-1.0, 4.0, -4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[0.0, -6.0, 6.0], [-1.0, 5.0, -2.0]], [[6.0, 2.0, -7.0], [-6.0, 5.0, 4.0]], [[-3.0, -5.0, 0.0], [3.0, -6.0, -5.0]], [[-7.0, 2.0, -4.0], [-1.0, 0.0, -3.0]], [[-5.0, -1.0, -4.0], [-6.0, -4.0, -4.0]]], [[[-3.0, -1.0, 6.0], [-5.0, 4.0, -2.0]], [[-2.0, 4.0, 0.0], [4.0, -2.0, 2.0]], [[3.0, 1.0, -4.0], [0.0, -5.0, 5.0]], [[-7.0, 3.0, -2.0], [-5.0, -7.0, 1.0]], [[0.0, -2.0, 6.0], [5.0, -3.0, 2.0]]], [[[6.0, -2.0, 3.0], [-1.0, -5.0, -4.0]], [[-4.0, 4.0, 0.0], [5.0, 6.0, -6.0]], [[-1.0, 6.0, -6.0], [-3.0, -2.0, -7.0]], [[-4.0, -6.0, 0.0], [7.0, -7.0, 0.0]], [[7.0, -7.0, 6.0], [0.0, -2.0, -6.0]]], [[[7.0, 4.0, 6.0], [7.0, 7.0, -1.0]], [[-4.0, -4.0, 4.0], [6.0, 5.0, 7.0]], [[-5.0, 0.0, 7.0], [-5.0, -2.0, -6.0]], [[-2.0, 6.0, -4.0], [-1.0, 5.0, -4.0]], [[4.0, -5.0, -4.0], [-6.0, 3.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[42.0, 77.0], [23.0, -36.0], [-83.0, 3.0], [-20.0, -15.0], [-41.0, -16.0]], [[77.0, 67.0], [-4.0, -28.0], [-43.0, -66.0], [-19.0, -45.0], [-2.0, 15.0]], [[-23.0, 19.0], [24.0, 35.0], [-26.0, -25.0], [-100.0, -23.0], [-17.0, 66.0]], [[74.0, 53.0], [28.0, 18.0], [45.0, 63.0], [28.0, 1.0], [-11.0, 4.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_array_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=numpy.array([[-4.0, 1.0, -6.0, -3.0, -2.0], [-2.0, 3.0, -4.0, 1.0, 6.0], [6.0, 0.0, -4.0, -7.0, 7.0], [5.0, 4.0, 5.0, 2.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-7.0, 7.0, -4.0, 2.0, -5.0], [7.0, -4.0, -6.0, -2.0, 5.0], [5.0, -7.0, -4.0, 7.0, -1.0], [7.0, -6.0, -5.0, -6.0, 1.0]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[28.0, -7.0, 42.0, 21.0, 14.0], [14.0, -21.0, 28.0, -7.0, -42.0], [-42.0, 0.0, 28.0, 49.0, -49.0], [-35.0, -28.0, -35.0, -14.0, 0.0]], [[-28.0, 7.0, -42.0, -21.0, -14.0], [-14.0, 21.0, -28.0, 7.0, 42.0], [42.0, 0.0, -28.0, -49.0, 49.0], [35.0, 28.0, 35.0, 14.0, 0.0]], [[16.0, -4.0, 24.0, 12.0, 8.0], [8.0, -12.0, 16.0, -4.0, -24.0], [-24.0, 0.0, 16.0, 28.0, -28.0], [-20.0, -16.0, -20.0, -8.0, 0.0]], [[-8.0, 2.0, -12.0, -6.0, -4.0], [-4.0, 6.0, -8.0, 2.0, 12.0], [12.0, 0.0, -8.0, -14.0, 14.0], [10.0, 8.0, 10.0, 4.0, 0.0]], [[20.0, -5.0, 30.0, 15.0, 10.0], [10.0, -15.0, 20.0, -5.0, -30.0], [-30.0, 0.0, 20.0, 35.0, -35.0], [-25.0, -20.0, -25.0, -10.0, 0.0]]], [[[-28.0, 7.0, -42.0, -21.0, -14.0], [-14.0, 21.0, -28.0, 7.0, 42.0], [42.0, 0.0, -28.0, -49.0, 49.0], [35.0, 28.0, 35.0, 14.0, 0.0]], [[16.0, -4.0, 24.0, 12.0, 8.0], [8.0, -12.0, 16.0, -4.0, -24.0], [-24.0, 0.0, 16.0, 28.0, -28.0], [-20.0, -16.0, -20.0, -8.0, 0.0]], [[24.0, -6.0, 36.0, 18.0, 12.0], [12.0, -18.0, 24.0, -6.0, -36.0], [-36.0, 0.0, 24.0, 42.0, -42.0], [-30.0, -24.0, -30.0, -12.0, 0.0]], [[8.0, -2.0, 12.0, 6.0, 4.0], [4.0, -6.0, 8.0, -2.0, -12.0], [-12.0, 0.0, 8.0, 14.0, -14.0], [-10.0, -8.0, -10.0, -4.0, 0.0]], [[-20.0, 5.0, -30.0, -15.0, -10.0], [-10.0, 15.0, -20.0, 5.0, 30.0], [30.0, 0.0, -20.0, -35.0, 35.0], [25.0, 20.0, 25.0, 10.0, 0.0]]], [[[-20.0, 5.0, -30.0, -15.0, -10.0], [-10.0, 15.0, -20.0, 5.0, 30.0], [30.0, 0.0, -20.0, -35.0, 35.0], [25.0, 20.0, 25.0, 10.0, 0.0]], [[28.0, -7.0, 42.0, 21.0, 14.0], [14.0, -21.0, 28.0, -7.0, -42.0], [-42.0, 0.0, 28.0, 49.0, -49.0], [-35.0, -28.0, -35.0, -14.0, 0.0]], [[16.0, -4.0, 24.0, 12.0, 8.0], [8.0, -12.0, 16.0, -4.0, -24.0], [-24.0, 0.0, 16.0, 28.0, -28.0], [-20.0, -16.0, -20.0, -8.0, 0.0]], [[-28.0, 7.0, -42.0, -21.0, -14.0], [-14.0, 21.0, -28.0, 7.0, 42.0], [42.0, 0.0, -28.0, -49.0, 49.0], [35.0, 28.0, 35.0, 14.0, 0.0]], [[4.0, -1.0, 6.0, 3.0, 2.0], [2.0, -3.0, 4.0, -1.0, -6.0], [-6.0, 0.0, 4.0, 7.0, -7.0], [-5.0, -4.0, -5.0, -2.0, 0.0]]], [[[-28.0, 7.0, -42.0, -21.0, -14.0], [-14.0, 21.0, -28.0, 7.0, 42.0], [42.0, 0.0, -28.0, -49.0, 49.0], [35.0, 28.0, 35.0, 14.0, 0.0]], [[24.0, -6.0, 36.0, 18.0, 12.0], [12.0, -18.0, 24.0, -6.0, -36.0], [-36.0, 0.0, 24.0, 42.0, -42.0], [-30.0, -24.0, -30.0, -12.0, 0.0]], [[20.0, -5.0, 30.0, 15.0, 10.0], [10.0, -15.0, 20.0, -5.0, -30.0], [-30.0, 0.0, 20.0, 35.0, -35.0], [-25.0, -20.0, -25.0, -10.0, 0.0]], [[24.0, -6.0, 36.0, 18.0, 12.0], [12.0, -18.0, 24.0, -6.0, -36.0], [-36.0, 0.0, 24.0, 42.0, -42.0], [-30.0, -24.0, -30.0, -12.0, 0.0]], [[-4.0, 1.0, -6.0, -3.0, -2.0], [-2.0, 3.0, -4.0, 1.0, 6.0], [6.0, 0.0, -4.0, -7.0, 7.0], [5.0, 4.0, 5.0, 2.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=numpy.array([[[0.0, 1.0, -7.0], [4.0, 1.0, -7.0], [-2.0, 7.0, 2.0], [4.0, -3.0, -3.0], [4.0, 3.0, 1.0]], [[7.0, 3.0, 5.0], [-1.0, -4.0, -6.0], [-4.0, 6.0, 3.0], [-7.0, -7.0, 3.0], [-7.0, 1.0, 0.0]], [[-6.0, 0.0, 0.0], [7.0, -1.0, -6.0], [-5.0, 5.0, 5.0], [1.0, 6.0, -6.0], [7.0, 4.0, 0.0]], [[4.0, -7.0, -7.0], [-7.0, 0.0, -4.0], [0.0, 2.0, -3.0], [-4.0, 0.0, -7.0], [0.0, -3.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-1.0, 4.0, 1.0], [-7.0, -7.0, 6.0], [-1.0, -6.0, -2.0], [-2.0, 0.0, 6.0], [4.0, 7.0, 6.0]], [[-2.0, 2.0, 4.0], [4.0, 4.0, 1.0], [5.0, 6.0, 5.0], [7.0, 6.0, 0.0], [4.0, -2.0, 0.0]], [[0.0, -3.0, 0.0], [-1.0, -7.0, 7.0], [-3.0, -3.0, 1.0], [5.0, 6.0, -5.0], [-2.0, 0.0, -7.0]], [[-2.0, 2.0, 1.0], [-6.0, 0.0, 6.0], [-7.0, 7.0, 7.0], [-1.0, 7.0, 4.0], [-3.0, -5.0, -6.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-3.0, -7.0, 32.0, -19.0, 9.0], [10.0, -21.0, 31.0, -18.0, 11.0], [6.0, -17.0, 30.0, 17.0, 9.0], [-39.0, 3.0, 5.0, -3.0, -19.0]], [[-49.0, -77.0, -23.0, -25.0, -43.0], [-40.0, -1.0, 4.0, 116.0, 42.0], [42.0, -78.0, 30.0, -85.0, -77.0], [-21.0, 25.0, -32.0, -14.0, -21.0]], [[8.0, 4.0, -44.0, 20.0, -24.0], [-35.0, 37.0, -38.0, 43.0, 1.0], [6.0, 11.0, -35.0, -25.0, -31.0], [52.0, 15.0, -6.0, 18.0, 32.0]], [[-42.0, -50.0, 16.0, -26.0, -2.0], [16.0, -34.0, 26.0, 32.0, 14.0], [12.0, -50.0, 40.0, -38.0, -14.0], [-50.0, -10.0, -18.0, -34.0, -42.0]], [[-35.0, -19.0, 53.0, -23.0, 43.0], [79.0, -68.0, 44.0, -59.0, -21.0], [-24.0, -15.0, 45.0, 10.0, 56.0], [-75.0, -52.0, -4.0, -58.0, -63.0]]], [[[-26.0, -34.0, 26.0, -26.0, 2.0], [12.0, -30.0, 32.0, 12.0, 16.0], [12.0, -40.0, 40.0, -14.0, -6.0], [-50.0, -2.0, -8.0, -20.0, -34.0]], [[-3.0, 13.0, 22.0, 1.0, 29.0], [45.0, -26.0, 11.0, -53.0, -24.0], [-24.0, 18.0, 5.0, 22.0, 44.0], [-19.0, -32.0, 5.0, -23.0, -19.0]], [[-29.0, -9.0, 42.0, -13.0, 43.0], [78.0, -59.0, 31.0, -62.0, -29.0], [-30.0, -1.0, 30.0, 11.0, 59.0], [-57.0, -55.0, -3.0, -55.0, -53.0]], [[6.0, 34.0, 28.0, 10.0, 46.0], [67.0, -31.0, 8.0, -91.0, -43.0], [-42.0, 43.0, -5.0, 43.0, 73.0], [-14.0, -49.0, 12.0, -28.0, -18.0]], [[-2.0, 14.0, -22.0, 22.0, 10.0], [22.0, 4.0, -28.0, -14.0, -30.0], [-24.0, 30.0, -30.0, -8.0, 20.0], [30.0, -28.0, -4.0, -16.0, 6.0]]], [[[-3.0, -3.0, -21.0, 9.0, -9.0], [-9.0, 12.0, -18.0, 21.0, -3.0], [0.0, 3.0, -15.0, -18.0, -12.0], [21.0, 0.0, -6.0, 0.0, 9.0]], [[-56.0, -60.0, -33.0, -4.0, -18.0], [7.0, -13.0, -17.0, 77.0, 0.0], [6.0, -42.0, 5.0, -85.0, -35.0], [-4.0, -21.0, -35.0, -45.0, -28.0]], [[-10.0, -22.0, -13.0, -6.0, -20.0], [-25.0, 9.0, -3.0, 45.0, 18.0], [18.0, -24.0, 5.0, -27.0, -33.0], [2.0, 17.0, -9.0, 5.0, 2.0]], [[41.0, 61.0, 22.0, 17.0, 33.0], [28.0, 1.0, 1.0, -92.0, -29.0], [-30.0, 59.0, -20.0, 71.0, 59.0], [13.0, -15.0, 27.0, 15.0, 17.0]], [[49.0, 41.0, -10.0, 13.0, -15.0], [-49.0, 44.0, -13.0, -7.0, 14.0], [12.0, 28.0, -25.0, 40.0, -14.0], [41.0, 42.0, 21.0, 57.0, 49.0]]], [[[-5.0, -13.0, 20.0, -17.0, -1.0], [-3.0, -12.0, 23.0, 3.0, 16.0], [12.0, -22.0, 25.0, 4.0, -6.0], [-29.0, 10.0, 1.0, 1.0, -13.0]], [[-42.0, -66.0, 24.0, -42.0, -18.0], [-12.0, -30.0, 42.0, 60.0, 42.0], [36.0, -78.0, 60.0, -42.0, -42.0], [-66.0, 18.0, -18.0, -18.0, -42.0]], [[-42.0, -70.0, 77.0, -70.0, 0.0], [7.0, -63.0, 91.0, 21.0, 56.0], [42.0, -98.0, 105.0, -7.0, -21.0], [-126.0, 21.0, -7.0, -21.0, -70.0]], [[-21.0, -25.0, 59.0, -37.0, 21.0], [34.0, -51.0, 58.0, -30.0, 14.0], [6.0, -38.0, 60.0, 17.0, 21.0], [-81.0, -9.0, 2.0, -24.0, -49.0]], [[37.0, 25.0, -41.0, 21.0, -33.0], [-66.0, 59.0, -36.0, 38.0, 16.0], [18.0, 20.0, -40.0, 3.0, -41.0], [65.0, 45.0, 8.0, 54.0, 57.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=numpy.array([[[[1.0, -6.0, 0.0], [-2.0, -3.0, -7.0]], [[-2.0, -2.0, 5.0], [-2.0, 6.0, -5.0]], [[-4.0, -1.0, -4.0], [6.0, 0.0, 2.0]], [[5.0, -1.0, -5.0], [6.0, -2.0, 4.0]], [[0.0, -7.0, -7.0], [7.0, 3.0, -7.0]]], [[[-6.0, 0.0, 5.0], [1.0, 5.0, 2.0]], [[-7.0, -6.0, 1.0], [-6.0, 4.0, 2.0]], [[-2.0, -2.0, -6.0], [0.0, 7.0, 4.0]], [[2.0, -7.0, 1.0], [0.0, -1.0, -1.0]], [[-1.0, -7.0, 4.0], [0.0, -6.0, 4.0]]], [[[-2.0, -2.0, 1.0], [2.0, 0.0, -6.0]], [[2.0, -7.0, 6.0], [5.0, -7.0, 0.0]], [[7.0, -5.0, -1.0], [6.0, 4.0, -4.0]], [[5.0, -7.0, -1.0], [7.0, -6.0, 7.0]], [[-4.0, -1.0, -4.0], [3.0, -3.0, 4.0]]], [[[-1.0, 0.0, 1.0], [3.0, -2.0, 3.0]], [[6.0, 1.0, 7.0], [-1.0, 3.0, -4.0]], [[-5.0, 4.0, 7.0], [-2.0, 4.0, -2.0]], [[5.0, 3.0, 5.0], [-3.0, 5.0, 4.0]], [[-6.0, 2.0, 5.0], [-4.0, -4.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-1.0, -1.0, 4.0], [0.0, -2.0, -2.0]], [[-4.0, -3.0, 4.0], [-1.0, -4.0, 4.0]], [[-6.0, 7.0, 4.0], [0.0, 6.0, -2.0]], [[1.0, -4.0, 1.0], [-6.0, -7.0, -4.0]], [[1.0, -3.0, 0.0], [-6.0, 4.0, 2.0]]], [[[0.0, -7.0, 0.0], [-7.0, 1.0, -5.0]], [[-4.0, 4.0, -3.0], [1.0, 6.0, 2.0]], [[-4.0, -6.0, 0.0], [-1.0, 6.0, 5.0]], [[6.0, 6.0, 7.0], [-2.0, -1.0, -6.0]], [[3.0, 5.0, 4.0], [-2.0, 2.0, 4.0]]], [[[2.0, 4.0, -3.0], [-2.0, 7.0, 1.0]], [[6.0, -1.0, -2.0], [-1.0, -1.0, -5.0]], [[-5.0, -1.0, -3.0], [7.0, 5.0, 0.0]], [[3.0, 2.0, -5.0], [-7.0, 0.0, -6.0]], [[-1.0, 0.0, -2.0], [0.0, 7.0, -6.0]]], [[[5.0, 7.0, 1.0], [-3.0, 5.0, 3.0]], [[-6.0, -2.0, 0.0], [2.0, -1.0, -3.0]], [[0.0, -7.0, -1.0], [4.0, 7.0, -2.0]], [[7.0, -3.0, -2.0], [-6.0, 1.0, -5.0]], [[-4.0, -7.0, 6.0], [1.0, 2.0, 4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[25.0, 22.0, -15.0, -28.0, -13.0], [12.0, 5.0, -42.0, 13.0, 28.0], [20.0, 43.0, -6.0, -4.0, -13.0], [3.0, 23.0, 25.0, -6.0, 34.0]], [[0.0, -8.0, 5.0, -19.0, -54.0], [31.0, 48.0, -22.0, 17.0, 81.0], [-8.0, 60.0, -55.0, 42.0, 28.0], [25.0, -26.0, 14.0, -10.0, 54.0]], [[-52.0, 64.0, -3.0, -77.0, -45.0], [82.0, 24.0, 8.0, -61.0, -71.0], [14.0, -79.0, -49.0, -133.0, -25.0], [-8.0, 25.0, 114.0, 33.0, 48.0]], [[86.0, 1.0, -48.0, -34.0, -14.0], [-50.0, 18.0, -65.0, 42.0, 57.0], [19.0, 55.0, -22.0, 4.0, -17.0], [-16.0, 10.0, -22.0, -35.0, 47.0]], [[5.0, 30.0, -33.0, -28.0, -23.0], [12.0, 67.0, 40.0, 17.0, 4.0], [-20.0, -35.0, -6.0, -26.0, -23.0], [-21.0, 13.0, 7.0, 42.0, -6.0]]], [[[88.0, 59.0, -45.0, -57.0, 38.0], [-12.0, 78.0, 1.0, 53.0, 23.0], [30.0, 7.0, 17.0, -41.0, -37.0], [-38.0, 23.0, 0.0, -15.0, 15.0]], [[-62.0, 9.0, 34.0, -7.0, 4.0], [44.0, 23.0, 68.0, -47.0, -64.0], [-13.0, -91.0, -23.0, -60.0, 17.0], [-2.0, -32.0, 33.0, 12.0, -13.0]], [[-19.0, 33.0, 26.0, -12.0, 18.0], [63.0, 104.0, 82.0, 23.0, 30.0], [-12.0, -13.0, 0.0, 14.0, 21.0], [4.0, -31.0, 12.0, 15.0, -13.0]], [[19.0, 39.0, -82.0, -45.0, -66.0], [-20.0, -75.0, -97.0, -16.0, -38.0], [15.0, 9.0, 13.0, -69.0, -85.0], [-21.0, 114.0, 55.0, 60.0, 29.0]], [[-57.0, 0.0, -37.0, -10.0, -99.0], [18.0, -19.0, -10.0, -31.0, -18.0], [-40.0, -29.0, -28.0, -22.0, -29.0], [3.0, 43.0, 37.0, 82.0, 8.0]]], [[[-46.0, 14.0, -10.0, -1.0, -7.0], [8.0, 1.0, 59.0, -35.0, -80.0], [-25.0, -101.0, 9.0, -64.0, -23.0], [-22.0, 14.0, 15.0, 52.0, -40.0]], [[52.0, 1.0, -31.0, 17.0, 46.0], [-62.0, -46.0, -25.0, 23.0, -21.0], [16.0, 9.0, 59.0, 3.0, -35.0], [-24.0, 39.0, -40.0, -5.0, -35.0]], [[-28.0, 13.0, 75.0, 23.0, 92.0], [47.0, 16.0, 65.0, -11.0, -30.0], [23.0, -21.0, 35.0, 4.0, 39.0], [13.0, -44.0, 6.0, -39.0, -35.0]], [[47.0, 9.0, -48.0, -28.0, 14.0], [-62.0, -8.0, -4.0, -7.0, -61.0], [7.0, -73.0, -2.0, -85.0, -39.0], [-47.0, 16.0, -16.0, -7.0, -5.0]], [[20.0, 64.0, 0.0, -33.0, 77.0], [19.0, 21.0, 39.0, -5.0, -73.0], [36.0, -63.0, 47.0, -87.0, -33.0], [-33.0, 25.0, 31.0, -4.0, -26.0]]], [[[-67.0, 2.0, -43.0, -3.0, -83.0], [3.0, -32.0, 17.0, -46.0, -68.0], [-47.0, -83.0, -11.0, -55.0, -43.0], [-14.0, 50.0, 30.0, 97.0, -22.0]], [[26.0, 21.0, 32.0, -26.0, 46.0], [27.0, 32.0, -3.0, 6.0, 14.0], [38.0, 19.0, -12.0, -17.0, 23.0], [5.0, -31.0, 20.0, -59.0, 31.0]], [[27.0, 53.0, 31.0, 14.0, 119.0], [30.0, 41.0, 61.0, 43.0, -5.0], [33.0, 14.0, 96.0, 22.0, -6.0], [-9.0, 11.0, -11.0, -11.0, -61.0]], [[69.0, 25.0, -63.0, -10.0, 31.0], [-63.0, -3.0, -9.0, 37.0, -20.0], [8.0, -14.0, 54.0, -25.0, -58.0], [-44.0, 54.0, -35.0, 19.0, -33.0]], [[2.0, 42.0, 13.0, -25.0, -8.0], [73.0, 86.0, 16.0, 41.0, 81.0], [6.0, 68.0, -1.0, 46.0, 12.0], [21.0, 0.0, 32.0, 12.0, 24.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[5.0, 0.0, 5.0, 1.0, 2.0], [-2.0, -2.0, -7.0, 0.0, -1.0], [-7.0, 5.0, -1.0, 4.0, -2.0], [4.0, -7.0, -3.0, 3.0, -5.0]])
      s1=numpy.array(-7.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-35.0, -0.0, -35.0, -7.0, -14.0], [14.0, 14.0, 49.0, -0.0, 7.0], [49.0, -35.0, 7.0, -28.0, 14.0], [-28.0, 49.0, 21.0, -21.0, 35.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-7.0, 1.0, -4.0], [-1.0, -2.0, 4.0], [-5.0, -5.0, -7.0], [5.0, -3.0, -2.0], [-4.0, 1.0, -1.0]], [[-7.0, 3.0, 1.0], [-7.0, 0.0, -2.0], [-2.0, -3.0, 3.0], [-7.0, -5.0, -4.0], [1.0, 0.0, -6.0]], [[2.0, 0.0, -1.0], [-5.0, 3.0, -4.0], [0.0, -5.0, 0.0], [2.0, -6.0, 2.0], [-7.0, 3.0, 1.0]], [[5.0, 0.0, 5.0], [1.0, -2.0, 0.0], [6.0, -5.0, 7.0], [4.0, -3.0, -6.0], [0.0, 1.0, 0.0]]])
      s1=numpy.array([7.0, 0.0, -3.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[-37.0, -19.0, -14.0, 41.0, -25.0], [-52.0, -43.0, -23.0, -37.0, 25.0], [17.0, -23.0, 0.0, 8.0, -52.0], [20.0, 7.0, 21.0, 46.0, 0.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-3.0, -6.0, 6.0], [2.0, -5.0, -4.0]], [[2.0, 3.0, -6.0], [-4.0, -7.0, 0.0]], [[-5.0, 5.0, -3.0], [-1.0, 7.0, -2.0]], [[0.0, -7.0, 6.0], [-4.0, -1.0, -4.0]], [[2.0, 2.0, 5.0], [0.0, 4.0, 0.0]]], [[[3.0, -1.0, -1.0], [1.0, 2.0, -2.0]], [[-2.0, -4.0, 5.0], [-2.0, -4.0, -5.0]], [[7.0, -6.0, -7.0], [0.0, 0.0, -4.0]], [[-1.0, -4.0, 2.0], [4.0, -1.0, -7.0]], [[-1.0, 2.0, -5.0], [7.0, -5.0, 7.0]]], [[[-7.0, -1.0, 4.0], [-5.0, 5.0, 7.0]], [[-5.0, -5.0, -3.0], [-7.0, 2.0, -6.0]], [[4.0, 4.0, -1.0], [2.0, 3.0, -1.0]], [[7.0, 6.0, 4.0], [-6.0, 5.0, 6.0]], [[7.0, -6.0, 5.0], [-1.0, -2.0, 3.0]]], [[[1.0, -2.0, 5.0], [7.0, 7.0, 0.0]], [[5.0, 4.0, -2.0], [-4.0, -3.0, 0.0]], [[-5.0, 1.0, 4.0], [-4.0, 6.0, -6.0]], [[4.0, 6.0, -1.0], [-7.0, 4.0, -5.0]], [[4.0, 7.0, 0.0], [-1.0, -6.0, -1.0]]]])
      s1=numpy.array([[2.0, -7.0, -1.0], [1.0, -2.0, -3.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[54.0, -1.0, -51.0, 53.0, -23.0], [17.0, 40.0, 75.0, 51.0, -15.0], [-47.0, 35.0, -20.0, -66.0, 45.0], [4.0, -14.0, -19.0, -33.0, -27.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[1.0, -5.0, -4.0, 0.0, -2.0], [7.0, -7.0, 3.0, -7.0, -6.0], [-5.0, 6.0, -3.0, 0.0, 0.0], [-3.0, -6.0, -2.0, 4.0, 2.0]])
      s1=numpy.array([2.0, 0.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[2.0, 0.0], [-10.0, 0.0], [-8.0, 0.0], [0.0, 0.0], [-4.0, 0.0]], [[14.0, 0.0], [-14.0, 0.0], [6.0, 0.0], [-14.0, 0.0], [-12.0, 0.0]], [[-10.0, 0.0], [12.0, 0.0], [-6.0, 0.0], [0.0, 0.0], [0.0, 0.0]], [[-6.0, 0.0], [-12.0, 0.0], [-4.0, 0.0], [8.0, 0.0], [4.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[2.0, -1.0, 4.0], [3.0, 5.0, 0.0], [5.0, 2.0, 2.0], [3.0, 1.0, 1.0], [7.0, 4.0, 1.0]], [[4.0, -5.0, -6.0], [0.0, -3.0, 3.0], [7.0, -7.0, -5.0], [6.0, 4.0, 4.0], [3.0, 3.0, -7.0]], [[3.0, 3.0, 1.0], [0.0, -6.0, 0.0], [-3.0, 7.0, 5.0], [0.0, 1.0, -6.0], [2.0, 1.0, -2.0]], [[0.0, -6.0, 4.0], [2.0, -4.0, -3.0], [4.0, 0.0, -4.0], [0.0, 1.0, -1.0], [0.0, 5.0, -6.0]]])
      s1=numpy.array([[7.0, 0.0, 4.0], [6.0, 1.0, 2.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[30.0, 19.0], [21.0, 23.0], [43.0, 36.0], [25.0, 21.0], [53.0, 48.0]], [[4.0, 7.0], [12.0, 3.0], [29.0, 25.0], [58.0, 48.0], [-7.0, 7.0]], [[25.0, 23.0], [0.0, -6.0], [-1.0, -1.0], [-24.0, -11.0], [6.0, 9.0]], [[16.0, 2.0], [2.0, 2.0], [12.0, 16.0], [-4.0, -1.0], [-24.0, -7.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-6.0, 5.0, -3.0], [3.0, -4.0, 7.0]], [[-2.0, 3.0, 5.0], [-1.0, -4.0, 4.0]], [[6.0, 7.0, 5.0], [3.0, -1.0, 3.0]], [[-1.0, -3.0, 1.0], [6.0, 4.0, -7.0]], [[4.0, 3.0, 4.0], [0.0, 7.0, 5.0]]], [[[-1.0, 1.0, -7.0], [-6.0, 0.0, -1.0]], [[-7.0, 4.0, -2.0], [0.0, 4.0, -1.0]], [[-2.0, 1.0, -6.0], [6.0, -5.0, 1.0]], [[-7.0, -3.0, 1.0], [7.0, 4.0, 2.0]], [[0.0, 3.0, 1.0], [-4.0, -3.0, 2.0]]], [[[6.0, 5.0, -2.0], [3.0, 0.0, -7.0]], [[4.0, 5.0, 6.0], [-1.0, 4.0, -5.0]], [[5.0, 6.0, -5.0], [3.0, 6.0, 3.0]], [[5.0, -2.0, 0.0], [2.0, -2.0, 5.0]], [[4.0, 5.0, -5.0], [5.0, -6.0, -2.0]]], [[[4.0, -7.0, 6.0], [-6.0, -7.0, -6.0]], [[0.0, 0.0, -5.0], [6.0, 2.0, 0.0]], [[5.0, 3.0, 7.0], [7.0, 1.0, 4.0]], [[5.0, 2.0, 5.0], [0.0, -5.0, 2.0]], [[0.0, -5.0, -3.0], [6.0, -3.0, 6.0]]]])
      s1=numpy.array([[[3.0, -7.0, -3.0], [0.0, 7.0, -2.0]], [[5.0, -6.0, 0.0], [-6.0, 1.0, -6.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-86.0, -124.0], [-78.0, -50.0], [-59.0, -49.0], [57.0, 23.0], [18.0, -21.0]], [[13.0, 31.0], [-13.0, -49.0], [-32.0, -63.0], [21.0, -67.0], [-49.0, -9.0]], [[3.0, 24.0], [-3.0, 30.0], [24.0, -41.0], [5.0, -7.0], [-46.0, -34.0]], [[6.0, 127.0], [29.0, -34.0], [-28.0, -58.0], [-53.0, -4.0], [11.0, -45.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_Symbol_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[3.0, -5.0, -5.0, -3.0, 0.0], [-1.0, 0.0, -3.0, -2.0, 0.0], [-7.0, -3.0, 5.0, 3.0, -6.0], [0.0, -4.0, -2.0, -3.0, 3.0]])
      s1=numpy.array([[4.0, 7.0, -5.0, 3.0, 0.0], [5.0, -5.0, 5.0, -7.0, 3.0], [1.0, -2.0, -5.0, -5.0, 2.0], [-4.0, -2.0, 7.0, -5.0, -7.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[12.0, 21.0, -15.0, 9.0, 0.0], [15.0, -15.0, 15.0, -21.0, 9.0], [3.0, -6.0, -15.0, -15.0, 6.0], [-12.0, -6.0, 21.0, -15.0, -21.0]], [[-20.0, -35.0, 25.0, -15.0, 0.0], [-25.0, 25.0, -25.0, 35.0, -15.0], [-5.0, 10.0, 25.0, 25.0, -10.0], [20.0, 10.0, -35.0, 25.0, 35.0]], [[-20.0, -35.0, 25.0, -15.0, 0.0], [-25.0, 25.0, -25.0, 35.0, -15.0], [-5.0, 10.0, 25.0, 25.0, -10.0], [20.0, 10.0, -35.0, 25.0, 35.0]], [[-12.0, -21.0, 15.0, -9.0, 0.0], [-15.0, 15.0, -15.0, 21.0, -9.0], [-3.0, 6.0, 15.0, 15.0, -6.0], [12.0, 6.0, -21.0, 15.0, 21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-4.0, -7.0, 5.0, -3.0, 0.0], [-5.0, 5.0, -5.0, 7.0, -3.0], [-1.0, 2.0, 5.0, 5.0, -2.0], [4.0, 2.0, -7.0, 5.0, 7.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, -21.0, 15.0, -9.0, 0.0], [-15.0, 15.0, -15.0, 21.0, -9.0], [-3.0, 6.0, 15.0, 15.0, -6.0], [12.0, 6.0, -21.0, 15.0, 21.0]], [[-8.0, -14.0, 10.0, -6.0, 0.0], [-10.0, 10.0, -10.0, 14.0, -6.0], [-2.0, 4.0, 10.0, 10.0, -4.0], [8.0, 4.0, -14.0, 10.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[-28.0, -49.0, 35.0, -21.0, 0.0], [-35.0, 35.0, -35.0, 49.0, -21.0], [-7.0, 14.0, 35.0, 35.0, -14.0], [28.0, 14.0, -49.0, 35.0, 49.0]], [[-12.0, -21.0, 15.0, -9.0, 0.0], [-15.0, 15.0, -15.0, 21.0, -9.0], [-3.0, 6.0, 15.0, 15.0, -6.0], [12.0, 6.0, -21.0, 15.0, 21.0]], [[20.0, 35.0, -25.0, 15.0, 0.0], [25.0, -25.0, 25.0, -35.0, 15.0], [5.0, -10.0, -25.0, -25.0, 10.0], [-20.0, -10.0, 35.0, -25.0, -35.0]], [[12.0, 21.0, -15.0, 9.0, 0.0], [15.0, -15.0, 15.0, -21.0, 9.0], [3.0, -6.0, -15.0, -15.0, 6.0], [-12.0, -6.0, 21.0, -15.0, -21.0]], [[-24.0, -42.0, 30.0, -18.0, 0.0], [-30.0, 30.0, -30.0, 42.0, -18.0], [-6.0, 12.0, 30.0, 30.0, -12.0], [24.0, 12.0, -42.0, 30.0, 42.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-16.0, -28.0, 20.0, -12.0, 0.0], [-20.0, 20.0, -20.0, 28.0, -12.0], [-4.0, 8.0, 20.0, 20.0, -8.0], [16.0, 8.0, -28.0, 20.0, 28.0]], [[-8.0, -14.0, 10.0, -6.0, 0.0], [-10.0, 10.0, -10.0, 14.0, -6.0], [-2.0, 4.0, 10.0, 10.0, -4.0], [8.0, 4.0, -14.0, 10.0, 14.0]], [[-12.0, -21.0, 15.0, -9.0, 0.0], [-15.0, 15.0, -15.0, 21.0, -9.0], [-3.0, 6.0, 15.0, 15.0, -6.0], [12.0, 6.0, -21.0, 15.0, 21.0]], [[12.0, 21.0, -15.0, 9.0, 0.0], [15.0, -15.0, 15.0, -21.0, 9.0], [3.0, -6.0, -15.0, -15.0, 6.0], [-12.0, -6.0, 21.0, -15.0, -21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[1.0, 7.0, -1.0], [2.0, -7.0, 3.0], [0.0, 3.0, 2.0], [-2.0, 7.0, 7.0], [1.0, 4.0, 4.0]], [[3.0, 0.0, 2.0], [7.0, 2.0, 0.0], [4.0, -4.0, -1.0], [-4.0, -5.0, 4.0], [-4.0, 6.0, -1.0]], [[-5.0, 5.0, -2.0], [6.0, 4.0, -3.0], [-6.0, 4.0, 3.0], [0.0, -7.0, -2.0], [-6.0, -3.0, 5.0]], [[6.0, -4.0, -5.0], [-2.0, -4.0, -7.0], [-5.0, 3.0, 7.0], [0.0, 2.0, 4.0], [-5.0, 6.0, -2.0]]])
      s1=numpy.array([[[4.0, 6.0, 1.0], [0.0, 0.0, 5.0], [7.0, -1.0, 0.0], [-7.0, 3.0, -6.0], [3.0, -5.0, 2.0]], [[6.0, 5.0, -2.0], [6.0, -3.0, 0.0], [2.0, -4.0, -7.0], [7.0, 7.0, -4.0], [-5.0, 6.0, -1.0]], [[-5.0, -3.0, -4.0], [-4.0, 4.0, 1.0], [0.0, 6.0, 0.0], [3.0, 2.0, -7.0], [5.0, 6.0, -6.0]], [[-6.0, 7.0, 6.0], [-4.0, 5.0, -6.0], [-4.0, 6.0, 2.0], [7.0, 4.0, -4.0], [-1.0, 1.0, -4.0]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[45.0, -5.0, 0.0, 20.0, -34.0], [43.0, -15.0, -19.0, 60.0, 38.0], [-22.0, 23.0, 42.0, 24.0, 53.0], [37.0, 37.0, 36.0, 39.0, 10.0]], [[-31.0, 15.0, 21.0, -53.0, 47.0], [-29.0, 33.0, 11.0, -47.0, -55.0], [-1.0, -33.0, -42.0, -29.0, -50.0], [-43.0, -61.0, -44.0, -26.0, -21.0]], [[20.0, 10.0, -3.0, -3.0, -11.0], [11.0, -9.0, -26.0, 13.0, 16.0], [-17.0, 14.0, 18.0, -8.0, 6.0], [33.0, 3.0, 22.0, 4.0, -5.0]], [[41.0, 35.0, -21.0, -7.0, -27.0], [9.0, -33.0, -81.0, 7.0, 45.0], [-39.0, 43.0, 42.0, -41.0, -10.0], [103.0, 1.0, 64.0, -14.0, -19.0]], [[32.0, 20.0, 3.0, -19.0, -9.0], [18.0, -6.0, -42.0, 19.0, 15.0], [-33.0, 16.0, 24.0, -17.0, 5.0], [46.0, -8.0, 28.0, 7.0, -13.0]]], [[[14.0, 10.0, 21.0, -33.0, 13.0], [14.0, 18.0, -8.0, 13.0, -17.0], [-23.0, -10.0, 0.0, -5.0, 3.0], [-6.0, -24.0, -8.0, 13.0, -11.0]], [[40.0, 0.0, 47.0, -43.0, 11.0], [52.0, 36.0, 6.0, 63.0, -23.0], [-41.0, -20.0, 12.0, 25.0, 47.0], [-28.0, -18.0, -16.0, 57.0, -5.0]], [[-9.0, -5.0, 32.0, -34.0, 30.0], [6.0, 36.0, 31.0, 4.0, -43.0], [-4.0, -33.0, -24.0, 11.0, 2.0], [-58.0, -30.0, -42.0, 16.0, -4.0]], [[-42.0, 20.0, -23.0, -11.0, 21.0], [-57.0, -9.0, -16.0, -79.0, -14.0], [19.0, 0.0, -30.0, -50.0, -74.0], [13.0, -33.0, -6.0, -64.0, -17.0]], [[19.0, -5.0, -34.0, 52.0, -44.0], [8.0, -42.0, -25.0, 18.0, 57.0], [6.0, 39.0, 36.0, 7.0, 22.0], [60.0, 52.0, 50.0, 0.0, 14.0]]], [[[8.0, -10.0, -40.0, 62.0, -44.0], [-1.0, -45.0, -16.0, 8.0, 57.0], [18.0, 38.0, 30.0, 9.0, 17.0], [53.0, 57.0, 46.0, -7.0, 18.0]], [[45.0, -15.0, 38.0, -12.0, -8.0], [62.0, 24.0, 17.0, 82.0, -3.0], [-30.0, -11.0, 24.0, 47.0, 72.0], [-26.0, 14.0, -6.0, 70.0, 10.0]], [[3.0, 15.0, -46.0, 36.0, -32.0], [-22.0, -48.0, -49.0, -26.0, 51.0], [6.0, 43.0, 24.0, -31.0, -24.0], [82.0, 26.0, 54.0, -38.0, -2.0]], [[-44.0, -10.0, 7.0, -9.0, 31.0], [-31.0, 21.0, 42.0, -41.0, -40.0], [29.0, -30.0, -42.0, 0.0, -30.0], [-61.0, -23.0, -46.0, -20.0, 1.0]], [[-37.0, 25.0, -39.0, 3.0, 7.0], [-61.0, -27.0, -35.0, -83.0, 7.0], [19.0, 17.0, -18.0, -59.0, -78.0], [45.0, -21.0, 16.0, -74.0, -17.0]]], [[[-5.0, -25.0, 46.0, -24.0, 28.0], [26.0, 48.0, 63.0, 34.0, -49.0], [2.0, -45.0, -24.0, 45.0, 36.0], [-94.0, -14.0, -58.0, 46.0, 10.0]], [[-39.0, -35.0, -10.0, 44.0, 0.0], [-18.0, 0.0, 61.0, -14.0, -7.0], [50.0, -15.0, -24.0, 35.0, 8.0], [-58.0, 30.0, -30.0, -2.0, 26.0]], [[5.0, 35.0, -38.0, 2.0, -16.0], [-29.0, -39.0, -71.0, -42.0, 36.0], [-12.0, 39.0, 18.0, -58.0, -49.0], [93.0, -7.0, 52.0, -51.0, -20.0]], [[16.0, 20.0, -2.0, -18.0, -2.0], [2.0, -6.0, -36.0, -2.0, 8.0], [-22.0, 12.0, 12.0, -24.0, -12.0], [38.0, -14.0, 20.0, -8.0, -14.0]], [[14.0, -10.0, -41.0, 65.0, -49.0], [4.0, -48.0, -20.0, 15.0, 63.0], [15.0, 42.0, 36.0, 11.0, 23.0], [60.0, 62.0, 52.0, -3.0, 19.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-5.0, -3.0, 0.0], [-3.0, -2.0, 0.0]], [[0.0, 5.0, -7.0], [2.0, -7.0, -7.0]], [[7.0, 7.0, -2.0], [-5.0, 3.0, 1.0]], [[-6.0, 7.0, 0.0], [-1.0, 2.0, -2.0]], [[-4.0, 6.0, 7.0], [1.0, -7.0, -5.0]]], [[[-4.0, -7.0, 5.0], [2.0, -7.0, 2.0]], [[-7.0, 4.0, -3.0], [5.0, -5.0, 3.0]], [[4.0, 2.0, 5.0], [-3.0, -4.0, -4.0]], [[-1.0, 4.0, -4.0], [-4.0, 4.0, 2.0]], [[5.0, -7.0, -7.0], [-7.0, -3.0, -7.0]]], [[[3.0, -7.0, 1.0], [5.0, 1.0, 0.0]], [[4.0, -1.0, 5.0], [6.0, 6.0, -5.0]], [[-3.0, 0.0, -7.0], [-4.0, -2.0, 4.0]], [[3.0, -6.0, -1.0], [-1.0, 5.0, 0.0]], [[-1.0, -2.0, 3.0], [-3.0, 5.0, -3.0]]], [[[4.0, 4.0, 7.0], [-7.0, 6.0, 0.0]], [[3.0, 3.0, -3.0], [-5.0, 7.0, -4.0]], [[3.0, 3.0, 0.0], [3.0, -1.0, 1.0]], [[-2.0, 6.0, 5.0], [-2.0, 0.0, 4.0]], [[0.0, 4.0, 3.0], [0.0, 0.0, 0.0]]]])
      s1=numpy.array([[[[-5.0, -5.0, -6.0], [-7.0, -3.0, -7.0]], [[-7.0, 4.0, 0.0], [-7.0, 7.0, -1.0]], [[2.0, 7.0, -6.0], [0.0, 3.0, -6.0]], [[-6.0, 6.0, 4.0], [5.0, -3.0, -2.0]], [[0.0, 6.0, 0.0], [-5.0, -3.0, 1.0]]], [[[-2.0, -6.0, -6.0], [1.0, -4.0, -5.0]], [[0.0, -5.0, 1.0], [3.0, 0.0, -2.0]], [[-3.0, -3.0, 0.0], [2.0, -6.0, -7.0]], [[5.0, -4.0, 4.0], [0.0, 0.0, 7.0]], [[7.0, 4.0, -1.0], [1.0, 5.0, 6.0]]], [[[-5.0, -2.0, -5.0], [5.0, -3.0, 2.0]], [[-2.0, 0.0, -6.0], [-2.0, 6.0, 5.0]], [[4.0, -2.0, 2.0], [-2.0, 0.0, 0.0]], [[1.0, -7.0, 3.0], [0.0, -1.0, 0.0]], [[2.0, -1.0, 7.0], [3.0, -3.0, 5.0]]], [[[0.0, 6.0, 2.0], [5.0, -5.0, 5.0]], [[1.0, -6.0, 6.0], [7.0, -7.0, -5.0]], [[-7.0, 1.0, 1.0], [4.0, -2.0, 0.0]], [[5.0, 0.0, 0.0], [3.0, -1.0, -2.0]], [[1.0, 5.0, -7.0], [6.0, -3.0, 2.0]]]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[67.0, 30.0, -37.0, 3.0, 3.0], [33.0, 6.0, 30.0, -13.0, -60.0], [22.0, 4.0, -8.0, 18.0, -10.0], [-23.0, 6.0, 24.0, -32.0, -32.0]], [[73.0, -36.0, 98.0, 47.0, 34.0], [77.0, -12.0, 80.0, -97.0, -48.0], [42.0, -39.0, -28.0, -49.0, -62.0], [26.0, 26.0, 20.0, 27.0, 93.0]], [[-39.0, 34.0, 78.0, -44.0, 59.0], [-66.0, -54.0, -77.0, 6.0, 95.0], [-71.0, 31.0, 20.0, -51.0, -26.0], [3.0, -108.0, -70.0, 15.0, 19.0]], [[10.0, 93.0, 55.0, 71.0, 39.0], [-29.0, -34.0, -3.0, -72.0, -17.0], [1.0, 16.0, -36.0, -57.0, -38.0], [17.0, -59.0, 41.0, -31.0, 13.0]], [[-3.0, 1.0, 1.0, 124.0, 47.0], [-16.0, -10.0, 73.0, -51.0, -75.0], [-11.0, -103.0, -16.0, -18.0, 34.0], [65.0, 83.0, 59.0, 0.0, -6.0]]], [[[18.0, -65.0, -120.0, 29.0, -29.0], [40.0, 42.0, 65.0, 42.0, -82.0], [44.0, -58.0, 4.0, 67.0, 71.0], [23.0, 121.0, 48.0, -11.0, -37.0]], [[-8.0, -8.0, -1.0, 88.0, 17.0], [18.0, -14.0, 28.0, -42.0, -32.0], [88.0, 7.0, -52.0, -39.0, 6.0], [83.0, 6.0, 80.0, -21.0, 85.0]], [[1.0, -23.0, 4.0, 13.0, 35.0], [-17.0, -6.0, 28.0, 4.0, -16.0], [-60.0, -76.0, 28.0, 9.0, 24.0], [7.0, 49.0, -25.0, 23.0, -35.0]], [[11.0, 77.0, 50.0, -22.0, 34.0], [-28.0, -40.0, -55.0, -23.0, 41.0], [-11.0, 68.0, -12.0, -45.0, -48.0], [-14.0, -115.0, -17.0, -25.0, 15.0]], [[159.0, -28.0, 36.0, -112.0, -5.0], [114.0, 21.0, 59.0, -24.0, -50.0], [-16.0, -7.0, 34.0, 36.0, -79.0], [-111.0, 12.0, -71.0, 21.0, -28.0]]], [[[-24.0, -77.0, -46.0, -34.0, -70.0], [31.0, 51.0, 16.0, 47.0, 2.0], [16.0, -16.0, 18.0, 54.0, 32.0], [-20.0, 79.0, -9.0, 29.0, -12.0]], [[-70.0, -27.0, 19.0, 12.0, -59.0], [-25.0, 38.0, 2.0, 9.0, 25.0], [-41.0, -39.0, 16.0, 20.0, 19.0], [-21.0, 65.0, -12.0, 42.0, -28.0]], [[63.0, 31.0, 6.0, -32.0, 30.0], [32.0, -27.0, -15.0, -15.0, -4.0], [44.0, 64.0, -18.0, -22.0, -41.0], [-4.0, -79.0, 2.0, -33.0, 36.0]], [[13.0, -3.0, -15.0, -78.0, -46.0], [15.0, 26.0, -23.0, 35.0, 22.0], [-18.0, 32.0, 24.0, 37.0, -13.0], [-68.0, -9.0, -42.0, 7.0, -41.0]], [[24.0, 58.0, -1.0, -18.0, -15.0], [-12.0, 10.0, -6.0, -6.0, -14.0], [-42.0, 5.0, 12.0, 17.0, -18.0], [-61.0, -12.0, -14.0, -13.0, -71.0]]], [[[-51.0, 79.0, 12.0, -25.0, 41.0], [-105.0, -34.0, -74.0, 32.0, 60.0], [-116.0, 0.0, 36.0, -9.0, 14.0], [-27.0, -69.0, -57.0, -7.0, -85.0]], [[30.0, 79.0, 90.0, -50.0, 18.0], [-19.0, -25.0, -42.0, -37.0, 42.0], [-60.0, 44.0, 10.0, -34.0, -74.0], [-68.0, -97.0, -55.0, 1.0, -20.0]], [[-55.0, -38.0, 18.0, 16.0, 7.0], [-22.0, -8.0, -13.0, 10.0, 37.0], [-1.0, -13.0, 0.0, -17.0, 20.0], [43.0, 8.0, -4.0, 23.0, 41.0]], [[-64.0, 48.0, -16.0, 50.0, 50.0], [-84.0, -39.0, -44.0, 14.0, 27.0], [-29.0, -2.0, -6.0, -29.0, 39.0], [56.0, -42.0, 17.0, -24.0, -11.0]], [[-38.0, 16.0, 10.0, 36.0, 24.0], [-42.0, -17.0, -12.0, -4.0, 13.0], [-23.0, -18.0, -2.0, -19.0, 17.0], [30.0, -6.0, 7.0, 0.0, -1.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(-1.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[1.0, 5.0, 0.0, -6.0, -4.0], [1.0, 0.0, -2.0, -5.0, 2.0], [6.0, -2.0, -1.0, -3.0, 5.0], [0.0, 7.0, -2.0, -5.0, 7.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-1.0, -5.0, -0.0, 6.0, 4.0], [-1.0, -0.0, 2.0, 5.0, -2.0], [-6.0, 2.0, 1.0, 3.0, -5.0], [-0.0, -7.0, 2.0, 5.0, -7.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([0.0, -6.0, 2.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[5.0, 4.0, 0.0], [4.0, -4.0, -2.0], [7.0, -3.0, 6.0], [0.0, -6.0, -3.0], [1.0, -3.0, 0.0]], [[-6.0, -2.0, -2.0], [-6.0, -2.0, -3.0], [3.0, -6.0, -2.0], [1.0, -5.0, 7.0], [-5.0, -1.0, -3.0]], [[3.0, -3.0, 7.0], [0.0, 7.0, -2.0], [-7.0, -7.0, 7.0], [4.0, -7.0, 7.0], [7.0, -5.0, -7.0]], [[7.0, -4.0, -1.0], [-1.0, -7.0, -1.0], [-6.0, 7.0, -3.0], [0.0, 6.0, -2.0], [4.0, 1.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[-24.0, 20.0, 30.0, 30.0, 18.0], [8.0, 6.0, 32.0, 44.0, 0.0], [32.0, -46.0, 56.0, 56.0, 16.0], [22.0, 40.0, -48.0, -40.0, -8.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[0.0, -2.0, 7.0], [1.0, 1.0, -7.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[5.0, -3.0, -1.0], [3.0, 2.0, -2.0]], [[7.0, 5.0, -5.0], [0.0, 3.0, -7.0]], [[-4.0, 1.0, 7.0], [7.0, -3.0, -5.0]], [[0.0, 2.0, -3.0], [3.0, 5.0, 0.0]], [[1.0, -6.0, -5.0], [6.0, -5.0, 2.0]]], [[[1.0, -6.0, -4.0], [1.0, 0.0, -5.0]], [[0.0, 0.0, 3.0], [-5.0, -1.0, -5.0]], [[6.0, -1.0, -4.0], [-5.0, 1.0, 2.0]], [[-7.0, 2.0, 4.0], [4.0, 1.0, 6.0]], [[-7.0, 6.0, -6.0], [5.0, -4.0, -6.0]]], [[[0.0, -3.0, 2.0], [-5.0, -2.0, 2.0]], [[6.0, -6.0, 1.0], [5.0, -7.0, -7.0]], [[-4.0, 6.0, 5.0], [-4.0, -7.0, 1.0]], [[-7.0, 7.0, -5.0], [-4.0, 3.0, -2.0]], [[-2.0, 1.0, 1.0], [-5.0, -3.0, 2.0]]], [[[-7.0, -3.0, 3.0], [6.0, 0.0, 0.0]], [[-7.0, -5.0, 2.0], [3.0, 4.0, -1.0]], [[0.0, 0.0, -6.0], [3.0, -1.0, 0.0]], [[7.0, -3.0, -3.0], [1.0, 0.0, 0.0]], [[1.0, -5.0, 2.0], [7.0, 0.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[18.0, 7.0, 86.0, -17.0, -36.0], [20.0, 50.0, -44.0, -13.0, -11.0], [-1.0, 66.0, 5.0, -36.0, -17.0], [33.0, 38.0, -40.0, -14.0, 73.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([-7.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-7.0, -3.0, -7.0, 7.0, 6.0], [-7.0, 0.0, 5.0, 5.0, 0.0], [-4.0, 6.0, -5.0, -5.0, 0.0], [0.0, 4.0, -1.0, 0.0, -1.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[49.0, 0.0], [21.0, 0.0], [49.0, 0.0], [-49.0, 0.0], [-42.0, 0.0]], [[49.0, 0.0], [0.0, 0.0], [-35.0, 0.0], [-35.0, 0.0], [0.0, 0.0]], [[28.0, 0.0], [-42.0, 0.0], [35.0, 0.0], [35.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [-28.0, 0.0], [7.0, 0.0], [0.0, 0.0], [7.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([[-6.0, -7.0, 6.0], [1.0, -5.0, -6.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-4.0, 3.0, -4.0], [-4.0, -4.0, -4.0], [-7.0, 1.0, 3.0], [7.0, -3.0, 4.0], [-1.0, -5.0, -2.0]], [[-2.0, -6.0, -5.0], [-7.0, 0.0, 0.0], [-3.0, 0.0, -7.0], [5.0, 3.0, -7.0], [5.0, -4.0, -4.0]], [[0.0, 2.0, 5.0], [3.0, -4.0, 6.0], [0.0, -2.0, -2.0], [-1.0, 3.0, 5.0], [-6.0, 2.0, 4.0]], [[-3.0, 5.0, -3.0], [-7.0, -3.0, -4.0], [0.0, 0.0, 5.0], [7.0, 1.0, 1.0], [2.0, 6.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-21.0, 5.0], [28.0, 40.0], [53.0, -30.0], [3.0, -2.0], [29.0, 36.0]], [[24.0, 58.0], [42.0, -7.0], [-24.0, 39.0], [-93.0, 32.0], [-26.0, 49.0]], [[16.0, -40.0], [46.0, -13.0], [2.0, 22.0], [15.0, -46.0], [46.0, -40.0]], [[-35.0, -10.0], [39.0, 32.0], [30.0, -30.0], [-43.0, -4.0], [-84.0, 2.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[0.0, -2.0, -4.0], [5.0, 0.0, 1.0]], [[-5.0, 6.0, -3.0], [-5.0, -3.0, -1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[1.0, 2.0, 7.0], [-2.0, 3.0, 5.0]], [[-7.0, 7.0, 3.0], [0.0, 3.0, 5.0]], [[-5.0, 7.0, 6.0], [2.0, 1.0, 1.0]], [[-6.0, -5.0, -5.0], [-2.0, 5.0, 0.0]], [[0.0, -4.0, 1.0], [0.0, 4.0, -4.0]]], [[[6.0, 3.0, -5.0], [4.0, 6.0, 2.0]], [[-4.0, 3.0, -7.0], [0.0, 3.0, 7.0]], [[1.0, 5.0, 5.0], [0.0, 0.0, 0.0]], [[0.0, -2.0, 6.0], [-7.0, 0.0, 2.0]], [[7.0, 0.0, 0.0], [1.0, 2.0, 4.0]]], [[[1.0, -1.0, 2.0], [-1.0, 4.0, 0.0]], [[1.0, -7.0, -1.0], [6.0, 3.0, 7.0]], [[-5.0, 4.0, 4.0], [-6.0, 0.0, 3.0]], [[-1.0, -6.0, 6.0], [-5.0, -2.0, 7.0]], [[-3.0, -6.0, 7.0], [1.0, 4.0, 3.0]]], [[[-5.0, 6.0, 0.0], [1.0, 3.0, -2.0]], [[5.0, -7.0, 0.0], [0.0, 0.0, 5.0]], [[3.0, -1.0, 1.0], [1.0, -5.0, -2.0]], [[-7.0, -4.0, 7.0], [-1.0, 4.0, 0.0]], [[3.0, 2.0, -4.0], [0.0, 4.0, 7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-37.0, -18.0], [-21.0, 54.0], [-27.0, 35.0], [20.0, 10.0], [0.0, -35.0]], [[36.0, -37.0], [29.0, 43.0], [-30.0, 10.0], [-53.0, 3.0], [9.0, -50.0]], [[-11.0, -24.0], [55.0, -90.0], [-51.0, 64.0], [-30.0, -25.0], [-8.0, -62.0]], [[-9.0, 49.0], [19.0, -72.0], [1.0, -12.0], [-25.0, -17.0], [19.0, -10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_constData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      arg1=Data(numpy.array([[-6.0, 2.0, -4.0, 6.0, 5.0], [6.0, -6.0, 5.0, 2.0, -6.0], [1.0, -2.0, 6.0, -6.0, -1.0], [-1.0, -6.0, -7.0, 7.0, 3.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-1.0, -1.0, 0.0, 4.0, 0.0], [0.0, -6.0, -5.0, 7.0, 4.0], [-5.0, -2.0, 4.0, -7.0, 0.0], [-4.0, 4.0, 7.0, -6.0, -5.0]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[6.0, -2.0, 4.0, -6.0, -5.0], [-6.0, 6.0, -5.0, -2.0, 6.0], [-1.0, 2.0, -6.0, 6.0, 1.0], [1.0, 6.0, 7.0, -7.0, -3.0]], [[6.0, -2.0, 4.0, -6.0, -5.0], [-6.0, 6.0, -5.0, -2.0, 6.0], [-1.0, 2.0, -6.0, 6.0, 1.0], [1.0, 6.0, 7.0, -7.0, -3.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-24.0, 8.0, -16.0, 24.0, 20.0], [24.0, -24.0, 20.0, 8.0, -24.0], [4.0, -8.0, 24.0, -24.0, -4.0], [-4.0, -24.0, -28.0, 28.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[36.0, -12.0, 24.0, -36.0, -30.0], [-36.0, 36.0, -30.0, -12.0, 36.0], [-6.0, 12.0, -36.0, 36.0, 6.0], [6.0, 36.0, 42.0, -42.0, -18.0]], [[30.0, -10.0, 20.0, -30.0, -25.0], [-30.0, 30.0, -25.0, -10.0, 30.0], [-5.0, 10.0, -30.0, 30.0, 5.0], [5.0, 30.0, 35.0, -35.0, -15.0]], [[-42.0, 14.0, -28.0, 42.0, 35.0], [42.0, -42.0, 35.0, 14.0, -42.0], [7.0, -14.0, 42.0, -42.0, -7.0], [-7.0, -42.0, -49.0, 49.0, 21.0]], [[-24.0, 8.0, -16.0, 24.0, 20.0], [24.0, -24.0, 20.0, 8.0, -24.0], [4.0, -8.0, 24.0, -24.0, -4.0], [-4.0, -24.0, -28.0, 28.0, 12.0]]], [[[30.0, -10.0, 20.0, -30.0, -25.0], [-30.0, 30.0, -25.0, -10.0, 30.0], [-5.0, 10.0, -30.0, 30.0, 5.0], [5.0, 30.0, 35.0, -35.0, -15.0]], [[12.0, -4.0, 8.0, -12.0, -10.0], [-12.0, 12.0, -10.0, -4.0, 12.0], [-2.0, 4.0, -12.0, 12.0, 2.0], [2.0, 12.0, 14.0, -14.0, -6.0]], [[-24.0, 8.0, -16.0, 24.0, 20.0], [24.0, -24.0, 20.0, 8.0, -24.0], [4.0, -8.0, 24.0, -24.0, -4.0], [-4.0, -24.0, -28.0, 28.0, 12.0]], [[42.0, -14.0, 28.0, -42.0, -35.0], [-42.0, 42.0, -35.0, -14.0, 42.0], [-7.0, 14.0, -42.0, 42.0, 7.0], [7.0, 42.0, 49.0, -49.0, -21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[24.0, -8.0, 16.0, -24.0, -20.0], [-24.0, 24.0, -20.0, -8.0, 24.0], [-4.0, 8.0, -24.0, 24.0, 4.0], [4.0, 24.0, 28.0, -28.0, -12.0]], [[-24.0, 8.0, -16.0, 24.0, 20.0], [24.0, -24.0, 20.0, 8.0, -24.0], [4.0, -8.0, 24.0, -24.0, -4.0], [-4.0, -24.0, -28.0, 28.0, 12.0]], [[-42.0, 14.0, -28.0, 42.0, 35.0], [42.0, -42.0, 35.0, 14.0, -42.0], [7.0, -14.0, 42.0, -42.0, -7.0], [-7.0, -42.0, -49.0, 49.0, 21.0]], [[36.0, -12.0, 24.0, -36.0, -30.0], [-36.0, 36.0, -30.0, -12.0, 36.0], [-6.0, 12.0, -36.0, 36.0, 6.0], [6.0, 36.0, 42.0, -42.0, -18.0]], [[30.0, -10.0, 20.0, -30.0, -25.0], [-30.0, 30.0, -25.0, -10.0, 30.0], [-5.0, 10.0, -30.0, 30.0, 5.0], [5.0, 30.0, 35.0, -35.0, -15.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      arg1=Data(numpy.array([[[1.0, -3.0, -2.0], [-4.0, -7.0, -2.0], [4.0, -5.0, -3.0], [6.0, -2.0, -6.0], [-1.0, 0.0, -3.0]], [[0.0, -1.0, -6.0], [-5.0, 6.0, -2.0], [0.0, 5.0, 5.0], [-7.0, 3.0, -7.0], [6.0, -4.0, -5.0]], [[-2.0, -3.0, 0.0], [-5.0, 5.0, 7.0], [1.0, -6.0, 0.0], [-3.0, -2.0, 0.0], [-3.0, -3.0, 4.0]], [[-7.0, -3.0, 1.0], [-3.0, 7.0, -7.0], [-6.0, 1.0, 0.0], [-7.0, 4.0, 1.0], [-1.0, -1.0, 1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[3.0, -1.0, 0.0], [0.0, 0.0, 7.0], [4.0, 2.0, 5.0], [-2.0, 6.0, 6.0], [-6.0, -1.0, -6.0]], [[0.0, -6.0, 0.0], [4.0, 4.0, 0.0], [-7.0, -7.0, 4.0], [0.0, 2.0, 5.0], [5.0, 6.0, 4.0]], [[1.0, -1.0, 3.0], [0.0, 0.0, 6.0], [0.0, 1.0, 0.0], [5.0, -2.0, -3.0], [7.0, 1.0, 6.0]], [[-5.0, 5.0, -6.0], [7.0, 1.0, 7.0], [5.0, 6.0, -4.0], [-1.0, -3.0, 2.0], [-6.0, 6.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[6.0, -5.0, 17.0, 20.0, -3.0], [1.0, -21.0, -5.0, -24.0, 22.0], [-3.0, -20.0, 9.0, -7.0, -6.0], [-18.0, -16.0, -19.0, -25.0, -2.0]], [[-14.0, -14.0, -21.0, -42.0, -21.0], [-42.0, -14.0, 35.0, -49.0, -35.0], [0.0, 49.0, 0.0, 0.0, 28.0], [7.0, -49.0, 0.0, 7.0, 7.0]], [[-12.0, -40.0, -9.0, -10.0, -19.0], [-32.0, -18.0, 35.0, -57.0, -9.0], [-14.0, 25.0, -8.0, -16.0, 2.0], [-29.0, -33.0, -22.0, -15.0, -1.0]], [[-32.0, -46.0, -56.0, -60.0, -16.0], [-42.0, 34.0, 60.0, -10.0, -66.0], [-14.0, 82.0, -38.0, -6.0, 12.0], [2.0, 6.0, 18.0, 44.0, 2.0]], [[9.0, 43.0, -1.0, 2.0, 24.0], [37.0, 36.0, -35.0, 81.0, -2.0], [15.0, -17.0, 0.0, 20.0, -3.0], [39.0, 53.0, 35.0, 32.0, 1.0]]], [[[18.0, 42.0, 30.0, 12.0, 0.0], [6.0, -36.0, -30.0, -18.0, 24.0], [18.0, -30.0, 36.0, 12.0, 18.0], [18.0, -42.0, -6.0, -24.0, 6.0]], [[-8.0, -44.0, -4.0, 16.0, -4.0], [-4.0, 4.0, 20.0, -16.0, 8.0], [-20.0, 0.0, -20.0, -20.0, -24.0], [-40.0, 16.0, -20.0, -12.0, -8.0]], [[6.0, 69.0, -5.0, -52.0, -5.0], [-17.0, -15.0, -15.0, 0.0, -34.0], [35.0, 28.0, 35.0, 35.0, 58.0], [74.0, -56.0, 35.0, 25.0, 18.0]], [[-16.0, -24.0, -25.0, -34.0, -15.0], [-32.0, 2.0, 35.0, -29.0, -33.0], [-6.0, 45.0, -12.0, -4.0, 14.0], [-1.0, -21.0, 2.0, 13.0, 3.0]], [[-21.0, -70.0, -22.0, -6.0, -17.0], [-30.0, 3.0, 50.0, -45.0, -14.0], [-28.0, 33.0, -31.0, -27.0, -17.0], [-49.0, -1.0, -24.0, -7.0, -7.0]]], [[[-2.0, -3.0, 0.0, -10.0, -10.0], [-17.0, -17.0, 10.0, -31.0, -5.0], [1.0, 11.0, 7.0, -1.0, 12.0], [-1.0, -31.0, -7.0, -8.0, 3.0]], [[-12.0, -12.0, -18.0, -36.0, -18.0], [-36.0, -12.0, 30.0, -42.0, -30.0], [0.0, 42.0, 0.0, 0.0, 24.0], [6.0, -42.0, 0.0, 6.0, 6.0]], [[-3.0, -7.0, -5.0, -2.0, 0.0], [-1.0, 6.0, 5.0, 3.0, -4.0], [-3.0, 5.0, -6.0, -2.0, -3.0], [-3.0, 7.0, 1.0, 4.0, -1.0]], [[17.0, 0.0, 39.0, 52.0, 4.0], [20.0, -31.0, -25.0, -20.0, 53.0], [-4.0, -56.0, 17.0, -11.0, -21.0], [-32.0, -8.0, -32.0, -46.0, -6.0]], [[-8.0, -47.0, 5.0, 4.0, -25.0], [-37.0, -41.0, 35.0, -88.0, 8.0], [-17.0, 12.0, 1.0, -23.0, 0.0], [-46.0, -56.0, -41.0, -39.0, -2.0]]], [[[-8.0, -3.0, -27.0, -4.0, 23.0], [31.0, 67.0, -5.0, 92.0, -20.0], [-5.0, 8.0, -35.0, 5.0, -24.0], [14.0, 92.0, 35.0, 49.0, -6.0]], [[-10.0, -49.0, 2.0, -2.0, -28.0], [-43.0, -43.0, 40.0, -95.0, 3.0], [-17.0, 19.0, 1.0, -23.0, 4.0], [-45.0, -63.0, -41.0, -38.0, -1.0]], [[-5.0, -54.0, 2.0, 42.0, 7.0], [18.0, 19.0, 10.0, 11.0, 26.0], [-28.0, -23.0, -31.0, -27.0, -49.0], [-57.0, 55.0, -24.0, -15.0, -15.0]], [[4.0, 21.0, 5.0, -12.0, -5.0], [-9.0, -17.0, -5.0, -16.0, -4.0], [11.0, 4.0, 17.0, 9.0, 20.0], [18.0, -32.0, 3.0, -3.0, 6.0]], [[-22.0, -16.0, -51.0, -42.0, 9.0], [0.0, 68.0, 25.0, 67.0, -55.0], [-6.0, 53.0, -42.0, 6.0, -4.0], [23.0, 67.0, 42.0, 65.0, -1.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      arg1=Data(numpy.array([[[[2.0, -2.0, 4.0], [-4.0, 5.0, -5.0]], [[-5.0, 3.0, 2.0], [6.0, 0.0, 2.0]], [[1.0, 5.0, 3.0], [-4.0, -6.0, 0.0]], [[-6.0, -6.0, 1.0], [3.0, -3.0, -7.0]], [[0.0, 2.0, -4.0], [1.0, -3.0, 3.0]]], [[[2.0, -2.0, 7.0], [0.0, 7.0, 6.0]], [[0.0, -4.0, 3.0], [0.0, -5.0, 4.0]], [[-4.0, 1.0, -3.0], [2.0, 6.0, 2.0]], [[0.0, 7.0, -7.0], [4.0, -5.0, -1.0]], [[0.0, 3.0, 0.0], [-4.0, -3.0, -5.0]]], [[[0.0, 0.0, -2.0], [1.0, -4.0, 6.0]], [[-1.0, 4.0, 2.0], [5.0, -2.0, -5.0]], [[5.0, -3.0, 7.0], [3.0, -1.0, 3.0]], [[-2.0, -1.0, 5.0], [-2.0, 4.0, 1.0]], [[-5.0, 4.0, 1.0], [-3.0, 6.0, -5.0]]], [[[2.0, 4.0, 7.0], [-4.0, 1.0, 3.0]], [[-3.0, -5.0, -4.0], [-6.0, 3.0, 6.0]], [[-2.0, 4.0, 2.0], [-7.0, 5.0, -4.0]], [[0.0, -2.0, 4.0], [0.0, 6.0, -1.0]], [[-1.0, 3.0, 1.0], [-5.0, -1.0, 5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-3.0, 0.0, 7.0], [3.0, 0.0, 6.0]], [[-5.0, -7.0, 0.0], [6.0, 0.0, -4.0]], [[-3.0, 3.0, 0.0], [5.0, -5.0, -1.0]], [[-4.0, 4.0, 3.0], [6.0, -6.0, 7.0]], [[6.0, -2.0, 1.0], [-2.0, 7.0, 1.0]]], [[[-3.0, 7.0, -6.0], [-7.0, -1.0, -5.0]], [[6.0, -4.0, 0.0], [-1.0, 0.0, 7.0]], [[4.0, -6.0, -6.0], [2.0, 0.0, 3.0]], [[-7.0, -4.0, 0.0], [-6.0, 0.0, -1.0]], [[2.0, -2.0, 4.0], [-3.0, -5.0, -4.0]]], [[[-3.0, 0.0, 2.0], [6.0, 4.0, -5.0]], [[-4.0, -2.0, 4.0], [1.0, 7.0, 3.0]], [[6.0, -2.0, 0.0], [-7.0, -1.0, -5.0]], [[4.0, -1.0, 3.0], [-2.0, 6.0, -5.0]], [[0.0, -2.0, 6.0], [-5.0, 4.0, 5.0]]], [[[-5.0, -3.0, -5.0], [0.0, 0.0, -7.0]], [[-5.0, -4.0, -2.0], [-3.0, 5.0, 0.0]], [[0.0, -3.0, 1.0], [7.0, -7.0, 4.0]], [[-7.0, -6.0, -7.0], [0.0, 0.0, 1.0]], [[-5.0, -1.0, 4.0], [-6.0, 0.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-20.0, 59.0, 6.0, -8.0, -7.0], [79.0, 45.0, 9.0, -43.0, -42.0], [25.0, 2.0, 61.0, 41.0, -17.0], [49.0, -1.0, -25.0, 22.0, 25.0]], [[0.0, 32.0, -64.0, 118.0, -20.0], [-20.0, 12.0, 17.0, -21.0, -25.0], [-18.0, 27.0, 2.0, 1.0, -1.0], [-74.0, -10.0, -44.0, 18.0, -66.0]], [[-52.0, 52.0, 22.0, 37.0, 23.0], [-53.0, 9.0, -7.0, 67.0, 9.0], [19.0, 55.0, -7.0, -28.0, -13.0], [-22.0, -57.0, -38.0, -35.0, -13.0]], [[-93.0, 88.0, 37.0, -10.0, 41.0], [5.0, 51.0, 1.0, 54.0, -29.0], [66.0, 33.0, 34.0, -10.0, -50.0], [20.0, -32.0, -70.0, -39.0, 30.0]], [[58.0, -44.0, -35.0, -57.0, -28.0], [78.0, -20.0, 11.0, -65.0, -24.0], [-26.0, -41.0, 33.0, 28.0, 6.0], [29.0, 27.0, 27.0, 49.0, -3.0]]], [[[4.0, -28.0, 48.0, -13.0, 19.0], [-99.0, -61.0, 7.0, 73.0, 77.0], [-21.0, 11.0, -113.0, -26.0, 77.0], [-8.0, 7.0, 86.0, -39.0, 29.0]], [[-11.0, -34.0, -10.0, -64.0, 12.0], [62.0, 44.0, -16.0, -39.0, -43.0], [41.0, -62.0, 60.0, 1.0, -78.0], [21.0, 50.0, -49.0, 1.0, 22.0]], [[-27.0, -32.0, -52.0, -9.0, 23.0], [-4.0, 18.0, 6.0, 5.0, -41.0], [32.0, -45.0, 11.0, -33.0, -71.0], [-57.0, 48.0, -70.0, -15.0, -23.0]], [[23.0, -15.0, -3.0, 55.0, -17.0], [-12.0, 12.0, 10.0, -51.0, 17.0], [-12.0, -34.0, -44.0, 29.0, 42.0], [-9.0, 71.0, 44.0, 9.0, 20.0]], [[31.0, -34.0, 46.0, 38.0, -20.0], [-23.0, 29.0, -66.0, -25.0, 41.0], [-15.0, 13.0, 28.0, 0.0, -15.0], [19.0, -33.0, 8.0, -6.0, -4.0]]], [[[23.0, 45.0, -45.0, 61.0, -29.0], [6.0, -34.0, 32.0, -5.0, -11.0], [-44.0, 54.0, -2.0, 15.0, 48.0], [-27.0, -53.0, 8.0, 37.0, -54.0]], [[28.0, 34.0, -48.0, 1.0, -31.0], [91.0, -3.0, 52.0, -76.0, -46.0], [-17.0, -20.0, 19.0, 59.0, 40.0], [24.0, 39.0, 24.0, 59.0, 5.0]], [[64.0, -88.0, 30.0, -7.0, -23.0], [-21.0, -7.0, -56.0, -32.0, 50.0], [-33.0, -22.0, 1.0, -5.0, 2.0], [16.0, 1.0, 44.0, 3.0, -1.0]], [[85.0, -39.0, -20.0, -4.0, -49.0], [43.0, -37.0, -4.0, -61.0, 12.0], [-62.0, 1.0, 17.0, 31.0, 46.0], [24.0, -19.0, 58.0, 55.0, -25.0]], [[43.0, -14.0, 4.0, -44.0, -30.0], [104.0, 26.0, 4.0, -101.0, -23.0], [-3.0, -54.0, 44.0, 63.0, 12.0], [73.0, 58.0, 39.0, 47.0, 46.0]]], [[[11.0, -8.0, -35.0, 92.0, -7.0], [-81.0, -31.0, 18.0, 21.0, 26.0], [-32.0, 18.0, -72.0, -19.0, 43.0], [-78.0, 8.0, 16.0, -7.0, -44.0]], [[27.0, -9.0, -49.0, 28.0, -18.0], [19.0, -15.0, 46.0, -51.0, -15.0], [-19.0, -40.0, -41.0, 30.0, 46.0], [-23.0, 76.0, 36.0, 30.0, 1.0]], [[-73.0, 43.0, 2.0, 33.0, 30.0], [-12.0, 66.0, -26.0, 31.0, -36.0], [57.0, 19.0, 56.0, -30.0, -94.0], [-28.0, -28.0, -110.0, -36.0, -16.0]], [[-35.0, 5.0, -58.0, 64.0, 19.0], [-45.0, 7.0, 45.0, 6.0, -23.0], [20.0, -36.0, -63.0, -14.0, -1.0], [-84.0, 85.0, -28.0, -17.0, -13.0]], [[2.0, 6.0, 26.0, -20.0, -6.0], [56.0, 40.0, 7.0, -65.0, -9.0], [22.0, -51.0, 6.0, 49.0, 13.0], [56.0, 76.0, 32.0, 12.0, 66.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-2.0, 0.0, 6.0, -6.0, -1.0], [-6.0, -5.0, -6.0, -2.0, 6.0], [0.0, 4.0, 1.0, 4.0, 1.0], [-4.0, 3.0, -6.0, 0.0, 4.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-0.0, 0.0, 0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, -0.0, 0.0, 0.0]])+(1.-msk_ref)*numpy.array([[2.0, -0.0, -6.0, 6.0, 1.0], [6.0, 5.0, 6.0, 2.0, -6.0], [-0.0, -4.0, -1.0, -4.0, -1.0], [4.0, -3.0, 6.0, -0.0, -4.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([2.0, -2.0, 7.0])+(1.-msk_arg1)*numpy.array([-2.0, -4.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[1.0, 6.0, 0.0], [5.0, -1.0, -6.0], [1.0, -7.0, -2.0], [-1.0, 0.0, -7.0], [0.0, -1.0, -5.0]], [[-2.0, -1.0, -3.0], [1.0, -6.0, -1.0], [0.0, -6.0, -5.0], [-6.0, -2.0, 5.0], [3.0, 5.0, -4.0]], [[3.0, 7.0, 0.0], [-3.0, -3.0, -4.0], [5.0, 6.0, -3.0], [-5.0, 0.0, -3.0], [-4.0, -2.0, 0.0]], [[-1.0, 5.0, -4.0], [3.0, -2.0, 0.0], [1.0, -2.0, -3.0], [3.0, 2.0, -6.0], [6.0, 0.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, -30.0, 2.0, -51.0, -33.0], [-23.0, 7.0, -23.0, 27.0, -32.0], [-8.0, -28.0, -23.0, -31.0, -4.0], [-40.0, 10.0, -15.0, -40.0, 12.0]])+(1.-msk_ref)*numpy.array([[-26.0, -6.0, 26.0, 2.0, 4.0], [8.0, 22.0, 24.0, 20.0, -26.0], [-34.0, 18.0, -34.0, 10.0, 16.0], [-18.0, 2.0, 6.0, -14.0, -12.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank2_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 1.0, -6.0], [7.0, 2.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-5.0, 5.0, -2.0], [7.0, 3.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-1.0, 0.0, 4.0], [3.0, 0.0, -3.0]], [[1.0, -2.0, -5.0], [-1.0, -3.0, 3.0]], [[0.0, 6.0, 1.0], [-3.0, -5.0, 0.0]], [[0.0, 0.0, 2.0], [-1.0, 3.0, 5.0]], [[5.0, 0.0, -2.0], [3.0, -3.0, -2.0]]], [[[2.0, 2.0, -1.0], [0.0, -5.0, -4.0]], [[3.0, 2.0, 7.0], [7.0, -6.0, 4.0]], [[6.0, -3.0, -3.0], [2.0, 7.0, 0.0]], [[-1.0, -6.0, 5.0], [2.0, 4.0, 1.0]], [[-3.0, -4.0, 0.0], [1.0, 5.0, -1.0]]], [[[-1.0, -1.0, -1.0], [-1.0, 4.0, -7.0]], [[-4.0, 6.0, -7.0], [6.0, 1.0, -1.0]], [[2.0, -2.0, -1.0], [-2.0, -3.0, -1.0]], [[-5.0, 0.0, -3.0], [2.0, -5.0, -6.0]], [[5.0, 4.0, 7.0], [-2.0, -3.0, 3.0]]], [[[6.0, 0.0, 0.0], [0.0, 7.0, 6.0]], [[-7.0, 3.0, 4.0], [3.0, -3.0, 0.0]], [[3.0, 3.0, -7.0], [-5.0, -7.0, -5.0]], [[-7.0, -4.0, -7.0], [0.0, 0.0, -1.0]], [[-2.0, -1.0, -4.0], [-5.0, 2.0, 1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[15.0, -3.0, -31.0, -33.0, 5.0], [2.0, -37.0, 7.0, -12.0, 35.0], [40.0, 120.0, -24.0, 76.0, -100.0], [-46.0, 36.0, -2.0, 84.0, 0.0]])+(1.-msk_ref)*numpy.array([[0.0, -3.0, -8.0, 28.0, -21.0], [-37.0, 36.0, -4.0, -3.0, 11.0], [-35.0, 103.0, -47.0, -6.0, -24.0], [27.0, 54.0, -72.0, 23.0, -10.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, -4.0])+(1.-msk_arg1)*numpy.array([0.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-5.0, -5.0, -1.0, -5.0, 5.0], [2.0, 0.0, 3.0, 0.0, -2.0], [2.0, 4.0, -5.0, 6.0, -3.0], [-6.0, -1.0, -7.0, 6.0, -2.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-35.0, 20.0], [-35.0, 20.0], [-7.0, 4.0], [-35.0, 20.0], [35.0, -20.0]], [[14.0, -8.0], [0.0, 0.0], [21.0, -12.0], [0.0, 0.0], [-14.0, 8.0]], [[14.0, -8.0], [28.0, -16.0], [-35.0, 20.0], [42.0, -24.0], [-21.0, 12.0]], [[-42.0, 24.0], [-7.0, 4.0], [-49.0, 28.0], [42.0, -24.0], [-14.0, 8.0]]])+(1.-msk_ref)*numpy.array([[[0.0, 35.0], [0.0, 35.0], [0.0, 7.0], [0.0, 35.0], [0.0, -35.0]], [[0.0, -14.0], [0.0, 0.0], [0.0, -21.0], [0.0, 0.0], [0.0, 14.0]], [[0.0, -14.0], [0.0, -28.0], [0.0, 35.0], [0.0, -42.0], [0.0, 21.0]], [[0.0, 42.0], [0.0, 7.0], [0.0, 49.0], [0.0, -42.0], [0.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, -6.0, -3.0], [3.0, 2.0, 4.0]])+(1.-msk_arg1)*numpy.array([[0.0, -3.0, -2.0], [5.0, -7.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[6.0, 2.0, -1.0], [6.0, -1.0, 0.0], [2.0, -2.0, 7.0], [4.0, -2.0, 0.0], [7.0, -2.0, 0.0]], [[-3.0, 0.0, 0.0], [7.0, -3.0, -6.0], [7.0, 7.0, 7.0], [6.0, -3.0, -6.0], [-6.0, 3.0, -2.0]], [[0.0, 4.0, 1.0], [-2.0, -5.0, 2.0], [-4.0, 5.0, 2.0], [3.0, -6.0, 0.0], [4.0, -7.0, 4.0]], [[0.0, -4.0, 0.0], [6.0, 4.0, 6.0], [-1.0, 0.0, 7.0], [-7.0, -7.0, 3.0], [6.0, -6.0, -1.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, 18.0], [6.0, 16.0], [-9.0, 30.0], [12.0, 8.0], [12.0, 17.0]], [[0.0, -9.0], [36.0, -9.0], [-63.0, 63.0], [36.0, -12.0], [-12.0, -20.0]], [[-27.0, 12.0], [24.0, -8.0], [-36.0, 6.0], [36.0, -3.0], [30.0, 14.0]], [[24.0, -8.0], [-42.0, 50.0], [-21.0, 25.0], [33.0, -23.0], [39.0, 2.0]]])+(1.-msk_ref)*numpy.array([[[-4.0, 10.0], [3.0, 37.0], [-8.0, 66.0], [6.0, 34.0], [6.0, 49.0]], [[0.0, -15.0], [21.0, 20.0], [-35.0, 28.0], [21.0, 15.0], [-5.0, -63.0]], [[-14.0, -22.0], [11.0, 37.0], [-19.0, -43.0], [18.0, 57.0], [13.0, 93.0]], [[12.0, 28.0], [-24.0, 38.0], [-14.0, 37.0], [15.0, 32.0], [20.0, 66.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank3_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, 2.0, -6.0], [-5.0, -2.0, -2.0]], [[2.0, 3.0, 2.0], [2.0, 7.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, -7.0, -4.0], [-1.0, -2.0, 6.0]], [[-2.0, -2.0, 2.0], [-2.0, -4.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[5.0, 4.0, 2.0], [0.0, 0.0, -2.0]], [[-2.0, 0.0, -3.0], [0.0, -6.0, 4.0]], [[-2.0, 2.0, -5.0], [2.0, 6.0, -4.0]], [[6.0, -6.0, -5.0], [2.0, 2.0, 7.0]], [[3.0, 5.0, -6.0], [2.0, 0.0, -5.0]]], [[[7.0, -7.0, -7.0], [7.0, 3.0, -1.0]], [[3.0, -5.0, 5.0], [-6.0, -1.0, -7.0]], [[3.0, -1.0, 0.0], [6.0, -1.0, -5.0]], [[5.0, 0.0, 7.0], [5.0, -3.0, 6.0]], [[-5.0, 7.0, 6.0], [-6.0, 3.0, 7.0]]], [[[6.0, -3.0, -6.0], [-1.0, 3.0, 5.0]], [[3.0, -6.0, -3.0], [6.0, 0.0, 2.0]], [[4.0, -5.0, -4.0], [3.0, -1.0, -4.0]], [[7.0, -1.0, -4.0], [-3.0, -3.0, -3.0]], [[-6.0, -3.0, 0.0], [-3.0, 7.0, -6.0]]], [[[5.0, 7.0, -7.0], [0.0, -7.0, 6.0]], [[0.0, -1.0, 7.0], [-2.0, 5.0, 4.0]], [[6.0, -6.0, 7.0], [7.0, 0.0, 1.0]], [[-5.0, 0.0, 2.0], [1.0, -4.0, -7.0]], [[3.0, 0.0, -4.0], [-2.0, -3.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-30.0, 22.0], [34.0, -44.0], [32.0, 30.0], [-46.0, 16.0], [28.0, 3.0]], [[-53.0, 12.0], [-12.0, -32.0], [-38.0, -2.0], [-103.0, 25.0], [18.0, 46.0]], [[-17.0, 20.0], [-46.0, -2.0], [-15.0, -24.0], [7.0, -30.0], [43.0, 10.0]], [[28.0, -20.0], [-52.0, 50.0], [-127.0, 24.0], [35.0, -46.0], [34.0, -39.0]]])+(1.-msk_ref)*numpy.array([[[-63.0, -18.0], [54.0, 30.0], [-26.0, -46.0], [80.0, -8.0], [-52.0, -42.0]], [[37.0, -42.0], [-28.0, 16.0], [-36.0, -22.0], [-6.0, 18.0], [-16.0, 22.0]], [[52.0, -18.0], [51.0, -8.0], [14.0, -16.0], [-7.0, -8.0], [-8.0, -16.0]], [[14.0, 2.0], [-5.0, 8.0], [-5.0, 2.0], [-28.0, 14.0], [-21.0, -10.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank2_expandedData_rank2_offset0(self):
      arg0=Symbol(shape=(4, 5))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-1.0, 0.0, 0.0, -5.0, 5.0], [2.0, -6.0, -2.0, 3.0, 0.0], [-6.0, -6.0, -4.0, 6.0, 0.0], [0.0, 5.0, 1.0, -3.0, -5.0]])+(1.-msk_arg1)*numpy.array([[5.0, -2.0, 7.0, -2.0, 4.0], [5.0, 0.0, -5.0, -5.0, 6.0], [1.0, 5.0, -5.0, 3.0, -7.0], [1.0, 7.0, 7.0, 0.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[-5.0, 0.0, 3.0, 2.0, -2.0], [7.0, 0.0, 0.0, -4.0, -6.0], [-1.0, 3.0, -5.0, -2.0, -4.0], [6.0, 2.0, 5.0, 0.0, 6.0]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[5.0, 0.0, 0.0, 25.0, -25.0], [-10.0, 30.0, 10.0, -15.0, 0.0], [30.0, 30.0, 20.0, -30.0, 0.0], [0.0, -25.0, -5.0, 15.0, 25.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-3.0, 0.0, 0.0, -15.0, 15.0], [6.0, -18.0, -6.0, 9.0, 0.0], [-18.0, -18.0, -12.0, 18.0, 0.0], [0.0, 15.0, 3.0, -9.0, -15.0]], [[-2.0, 0.0, 0.0, -10.0, 10.0], [4.0, -12.0, -4.0, 6.0, 0.0], [-12.0, -12.0, -8.0, 12.0, 0.0], [0.0, 10.0, 2.0, -6.0, -10.0]], [[2.0, 0.0, 0.0, 10.0, -10.0], [-4.0, 12.0, 4.0, -6.0, 0.0], [12.0, 12.0, 8.0, -12.0, 0.0], [0.0, -10.0, -2.0, 6.0, 10.0]]], [[[-7.0, 0.0, 0.0, -35.0, 35.0], [14.0, -42.0, -14.0, 21.0, 0.0], [-42.0, -42.0, -28.0, 42.0, 0.0], [0.0, 35.0, 7.0, -21.0, -35.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[4.0, 0.0, 0.0, 20.0, -20.0], [-8.0, 24.0, 8.0, -12.0, 0.0], [24.0, 24.0, 16.0, -24.0, 0.0], [0.0, -20.0, -4.0, 12.0, 20.0]], [[6.0, 0.0, 0.0, 30.0, -30.0], [-12.0, 36.0, 12.0, -18.0, 0.0], [36.0, 36.0, 24.0, -36.0, 0.0], [0.0, -30.0, -6.0, 18.0, 30.0]]], [[[1.0, 0.0, 0.0, 5.0, -5.0], [-2.0, 6.0, 2.0, -3.0, 0.0], [6.0, 6.0, 4.0, -6.0, 0.0], [0.0, -5.0, -1.0, 3.0, 5.0]], [[-3.0, 0.0, 0.0, -15.0, 15.0], [6.0, -18.0, -6.0, 9.0, 0.0], [-18.0, -18.0, -12.0, 18.0, 0.0], [0.0, 15.0, 3.0, -9.0, -15.0]], [[5.0, 0.0, 0.0, 25.0, -25.0], [-10.0, 30.0, 10.0, -15.0, 0.0], [30.0, 30.0, 20.0, -30.0, 0.0], [0.0, -25.0, -5.0, 15.0, 25.0]], [[2.0, 0.0, 0.0, 10.0, -10.0], [-4.0, 12.0, 4.0, -6.0, 0.0], [12.0, 12.0, 8.0, -12.0, 0.0], [0.0, -10.0, -2.0, 6.0, 10.0]], [[4.0, 0.0, 0.0, 20.0, -20.0], [-8.0, 24.0, 8.0, -12.0, 0.0], [24.0, 24.0, 16.0, -24.0, 0.0], [0.0, -20.0, -4.0, 12.0, 20.0]]], [[[-6.0, 0.0, 0.0, -30.0, 30.0], [12.0, -36.0, -12.0, 18.0, 0.0], [-36.0, -36.0, -24.0, 36.0, 0.0], [0.0, 30.0, 6.0, -18.0, -30.0]], [[-2.0, 0.0, 0.0, -10.0, 10.0], [4.0, -12.0, -4.0, 6.0, 0.0], [-12.0, -12.0, -8.0, 12.0, 0.0], [0.0, 10.0, 2.0, -6.0, -10.0]], [[-5.0, 0.0, 0.0, -25.0, 25.0], [10.0, -30.0, -10.0, 15.0, 0.0], [-30.0, -30.0, -20.0, 30.0, 0.0], [0.0, 25.0, 5.0, -15.0, -25.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 0.0, 0.0, -30.0, 30.0], [12.0, -36.0, -12.0, 18.0, 0.0], [-36.0, -36.0, -24.0, 36.0, 0.0], [0.0, 30.0, 6.0, -18.0, -30.0]]]])+(1.-msk_ref)*numpy.array([[[[-25.0, 10.0, -35.0, 10.0, -20.0], [-25.0, 0.0, 25.0, 25.0, -30.0], [-5.0, -25.0, 25.0, -15.0, 35.0], [-5.0, -35.0, -35.0, 0.0, -35.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[15.0, -6.0, 21.0, -6.0, 12.0], [15.0, 0.0, -15.0, -15.0, 18.0], [3.0, 15.0, -15.0, 9.0, -21.0], [3.0, 21.0, 21.0, 0.0, 21.0]], [[10.0, -4.0, 14.0, -4.0, 8.0], [10.0, 0.0, -10.0, -10.0, 12.0], [2.0, 10.0, -10.0, 6.0, -14.0], [2.0, 14.0, 14.0, 0.0, 14.0]], [[-10.0, 4.0, -14.0, 4.0, -8.0], [-10.0, 0.0, 10.0, 10.0, -12.0], [-2.0, -10.0, 10.0, -6.0, 14.0], [-2.0, -14.0, -14.0, 0.0, -14.0]]], [[[35.0, -14.0, 49.0, -14.0, 28.0], [35.0, 0.0, -35.0, -35.0, 42.0], [7.0, 35.0, -35.0, 21.0, -49.0], [7.0, 49.0, 49.0, 0.0, 49.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-20.0, 8.0, -28.0, 8.0, -16.0], [-20.0, 0.0, 20.0, 20.0, -24.0], [-4.0, -20.0, 20.0, -12.0, 28.0], [-4.0, -28.0, -28.0, 0.0, -28.0]], [[-30.0, 12.0, -42.0, 12.0, -24.0], [-30.0, 0.0, 30.0, 30.0, -36.0], [-6.0, -30.0, 30.0, -18.0, 42.0], [-6.0, -42.0, -42.0, 0.0, -42.0]]], [[[-5.0, 2.0, -7.0, 2.0, -4.0], [-5.0, 0.0, 5.0, 5.0, -6.0], [-1.0, -5.0, 5.0, -3.0, 7.0], [-1.0, -7.0, -7.0, 0.0, -7.0]], [[15.0, -6.0, 21.0, -6.0, 12.0], [15.0, 0.0, -15.0, -15.0, 18.0], [3.0, 15.0, -15.0, 9.0, -21.0], [3.0, 21.0, 21.0, 0.0, 21.0]], [[-25.0, 10.0, -35.0, 10.0, -20.0], [-25.0, 0.0, 25.0, 25.0, -30.0], [-5.0, -25.0, 25.0, -15.0, 35.0], [-5.0, -35.0, -35.0, 0.0, -35.0]], [[-10.0, 4.0, -14.0, 4.0, -8.0], [-10.0, 0.0, 10.0, 10.0, -12.0], [-2.0, -10.0, 10.0, -6.0, 14.0], [-2.0, -14.0, -14.0, 0.0, -14.0]], [[-20.0, 8.0, -28.0, 8.0, -16.0], [-20.0, 0.0, 20.0, 20.0, -24.0], [-4.0, -20.0, 20.0, -12.0, 28.0], [-4.0, -28.0, -28.0, 0.0, -28.0]]], [[[30.0, -12.0, 42.0, -12.0, 24.0], [30.0, 0.0, -30.0, -30.0, 36.0], [6.0, 30.0, -30.0, 18.0, -42.0], [6.0, 42.0, 42.0, 0.0, 42.0]], [[10.0, -4.0, 14.0, -4.0, 8.0], [10.0, 0.0, -10.0, -10.0, 12.0], [2.0, 10.0, -10.0, 6.0, -14.0], [2.0, 14.0, 14.0, 0.0, 14.0]], [[25.0, -10.0, 35.0, -10.0, 20.0], [25.0, 0.0, -25.0, -25.0, 30.0], [5.0, 25.0, -25.0, 15.0, -35.0], [5.0, 35.0, 35.0, 0.0, 35.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[30.0, -12.0, 42.0, -12.0, 24.0], [30.0, 0.0, -30.0, -30.0, 36.0], [6.0, 30.0, -30.0, 18.0, -42.0], [6.0, 42.0, 42.0, 0.0, 42.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank3_offset1(self):
      arg0=Symbol(shape=(4, 5, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 4.0, -7.0], [-3.0, 7.0, 2.0], [3.0, 0.0, -3.0], [-2.0, 4.0, -2.0], [4.0, 0.0, -3.0]], [[6.0, -6.0, -5.0], [6.0, -6.0, -5.0], [1.0, -4.0, -5.0], [-7.0, 3.0, -4.0], [-6.0, -1.0, 6.0]], [[6.0, 0.0, 3.0], [-6.0, 7.0, -2.0], [5.0, 3.0, -2.0], [0.0, 7.0, -3.0], [0.0, -5.0, 0.0]], [[-1.0, -6.0, -5.0], [4.0, 6.0, 4.0], [4.0, 0.0, -1.0], [1.0, 5.0, -7.0], [6.0, -7.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, -6.0, -7.0], [3.0, 0.0, 4.0], [7.0, 6.0, 0.0], [-7.0, -3.0, 0.0], [5.0, 7.0, 1.0]], [[-5.0, -6.0, 0.0], [-3.0, -1.0, -6.0], [-2.0, 3.0, 6.0], [0.0, 4.0, -6.0], [-3.0, 0.0, -7.0]], [[-2.0, -3.0, 7.0], [3.0, 0.0, 1.0], [-1.0, 7.0, 0.0], [7.0, -3.0, 5.0], [6.0, 7.0, 5.0]], [[0.0, -5.0, 2.0], [0.0, -5.0, 0.0], [-5.0, 5.0, 5.0], [-1.0, 5.0, 3.0], [-1.0, 4.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[-1.0, 0.0, -4.0], [-1.0, 1.0, -5.0], [-3.0, 6.0, 1.0], [6.0, 4.0, 6.0], [7.0, 5.0, -2.0]], [[4.0, 4.0, -3.0], [2.0, 2.0, 3.0], [5.0, 3.0, 7.0], [-1.0, -6.0, -5.0], [-5.0, 2.0, 2.0]], [[-7.0, 6.0, -7.0], [5.0, -1.0, 6.0], [-5.0, 2.0, -1.0], [0.0, -5.0, -3.0], [6.0, -3.0, -3.0]], [[5.0, 6.0, -6.0], [0.0, 7.0, 0.0], [7.0, -3.0, 1.0], [-2.0, 7.0, -1.0], [-2.0, -6.0, 6.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, -5.0, 9.0, 10.0, 8.0], [14.0, 14.0, 19.0, 23.0, -18.0], [-18.0, 14.0, 3.0, 12.0, 0.0], [21.0, -20.0, 0.0, 27.0, -22.0]], [[35.0, 0.0, 12.0, 16.0, 11.0], [13.0, 13.0, 20.0, 30.0, -25.0], [-21.0, 23.0, 8.0, 22.0, -5.0], [20.0, -18.0, 1.0, 39.0, -33.0]], [[5.0, 53.0, -12.0, 28.0, -15.0], [-59.0, -59.0, -32.0, 35.0, 18.0], [-15.0, 58.0, 1.0, 39.0, -30.0], [-38.0, 28.0, -13.0, 20.0, -56.0]], [[-2.0, 22.0, 0.0, -8.0, 6.0], [-18.0, -18.0, -40.0, -54.0, -4.0], [54.0, -20.0, 30.0, 10.0, -20.0], [-60.0, 72.0, 18.0, -16.0, 32.0]], [[62.0, 10.0, 27.0, 10.0, 34.0], [22.0, 22.0, -3.0, -26.0, -59.0], [36.0, -3.0, 54.0, 41.0, -25.0], [-27.0, 50.0, 30.0, 46.0, -1.0]]], [[[53.0, 10.0, 21.0, 14.0, 25.0], [15.0, 15.0, 3.0, -4.0, -46.0], [15.0, 10.0, 38.0, 37.0, -20.0], [-13.0, 28.0, 19.0, 45.0, -16.0]], [[-5.0, 14.0, -3.0, -2.0, -1.0], [-15.0, -15.0, -21.0, -20.0, 4.0], [21.0, -4.0, 10.0, 5.0, -10.0], [-29.0, 32.0, 5.0, -9.0, 10.0]], [[-17.0, 20.0, -6.0, -12.0, -1.0], [-23.0, -23.0, -42.0, -54.0, 9.0], [51.0, -23.0, 20.0, 0.0, -15.0], [-58.0, 66.0, 13.0, -29.0, 37.0]], [[7.0, -49.0, 12.0, -12.0, 11.0], [55.0, 55.0, 48.0, 9.0, -18.0], [-21.0, -26.0, -13.0, -27.0, 30.0], [62.0, -60.0, 1.0, 4.0, 16.0]], [[-26.0, 33.0, -21.0, 14.0, -26.0], [-52.0, -52.0, -23.0, 33.0, 40.0], [-24.0, 40.0, -23.0, 8.0, -10.0], [-17.0, 0.0, -22.0, -9.0, -36.0]]], [[[45.0, 49.0, 0.0, 52.0, -7.0], [-43.0, -43.0, 4.0, 95.0, -6.0], [-63.0, 98.0, -3.0, 63.0, -30.0], [6.0, -20.0, -21.0, 72.0, -112.0]], [[-26.0, -10.0, -3.0, -26.0, 2.0], [6.0, 6.0, -21.0, -62.0, 7.0], [48.0, -49.0, 10.0, -25.0, 5.0], [-29.0, 38.0, 14.0, -42.0, 61.0]], [[-5.0, 27.0, -12.0, 20.0, -17.0], [-37.0, -37.0, -8.0, 45.0, 22.0], [-33.0, 46.0, -17.0, 17.0, -10.0], [-2.0, -12.0, -19.0, 12.0, -48.0]], [[1.0, -41.0, 9.0, -14.0, 9.0], [45.0, 45.0, 35.0, -3.0, -13.0], [-9.0, -29.0, -9.0, -26.0, 25.0], [45.0, -42.0, 3.0, -4.0, 23.0]], [[33.0, -45.0, 27.0, -18.0, 33.0], [69.0, 69.0, 33.0, -39.0, -51.0], [27.0, -51.0, 27.0, -12.0, 15.0], [27.0, -6.0, 27.0, 12.0, 45.0]]], [[[86.0, 15.0, 33.0, 26.0, 38.0], [24.0, 24.0, 11.0, 7.0, -72.0], [12.0, 24.0, 55.0, 60.0, -30.0], [-11.0, 32.0, 26.0, 77.0, -36.0]], [[28.0, 49.0, 0.0, 28.0, 0.0], [-42.0, -42.0, -28.0, 21.0, -7.0], [0.0, 49.0, 21.0, 49.0, -35.0], [-42.0, 42.0, 0.0, 35.0, -49.0]], [[9.0, -40.0, 18.0, -28.0, 25.0], [55.0, 55.0, 14.0, -62.0, -33.0], [45.0, -65.0, 24.0, -24.0, 15.0], [6.0, 14.0, 27.0, -15.0, 67.0]], [[27.0, 53.0, -3.0, 34.0, -5.0], [-49.0, -49.0, -25.0, 39.0, -1.0], [-15.0, 63.0, 13.0, 52.0, -35.0], [-35.0, 30.0, -7.0, 40.0, -65.0]], [[-74.0, -24.0, -24.0, -32.0, -26.0], [-6.0, -6.0, -8.0, -28.0, 54.0], [6.0, -42.0, -40.0, -60.0, 30.0], [8.0, -20.0, -14.0, -74.0, 54.0]]]])+(1.-msk_ref)*numpy.array([[[[25.0, -19.0, -7.0, 7.0, -9.0], [5.0, 27.0, -22.0, 24.0, 31.0], [-26.0, -7.0, 1.0, -27.0, -26.0], [-8.0, 0.0, -15.0, -11.0, 29.0]], [[26.0, -23.0, -1.0, 4.0, -3.0], [-1.0, 32.0, -25.0, 34.0, 38.0], [-36.0, -8.0, 8.0, -35.0, -24.0], [-15.0, -5.0, -15.0, -9.0, 40.0]], [[-52.0, -5.0, 15.0, 3.0, 28.0], [-21.0, -3.0, 30.0, 18.0, 2.0], [-5.0, -8.0, 45.0, -34.0, 29.0], [-28.0, -30.0, 50.0, 36.0, 20.0]], [[-48.0, 42.0, 66.0, -54.0, 64.0], [-54.0, -58.0, 36.0, -20.0, -60.0], [18.0, 24.0, 22.0, 60.0, 94.0], [-8.0, -20.0, 20.0, 32.0, -32.0]], [[5.0, 13.0, 79.0, -64.0, 68.0], [-65.0, -14.0, -11.0, 32.0, -7.0], [-43.0, 19.0, 28.0, 24.0, 67.0], [-29.0, -25.0, -20.0, 12.0, 27.0]]], [[[9.0, 0.0, 52.0, -40.0, 45.0], [-44.0, 2.0, -14.0, 34.0, 9.0], [-41.0, 9.0, 24.0, 1.0, 37.0], [-26.0, -20.0, -15.0, 7.0, 33.0]], [[-27.0, 18.0, 26.0, -20.0, 27.0], [-22.0, -26.0, 20.0, -10.0, -27.0], [11.0, 9.0, 12.0, 23.0, 41.0], [-4.0, -10.0, 15.0, 17.0, -15.0]], [[-52.0, 43.0, 53.0, -44.0, 53.0], [-43.0, -60.0, 41.0, -30.0, -64.0], [30.0, 22.0, 16.0, 61.0, 86.0], [-1.0, -15.0, 25.0, 31.0, -42.0]], [[68.0, -23.0, -43.0, 25.0, -52.0], [41.0, 39.0, -46.0, 6.0, 38.0], [-15.0, -8.0, -41.0, -14.0, -73.0], [20.0, 30.0, -50.0, -44.0, 12.0]], [[-41.0, -7.0, -23.0, 29.0, -9.0], [13.0, 1.0, 28.0, -4.0, 1.0], [18.0, -13.0, 19.0, -31.0, -6.0], [-6.0, -10.0, 45.0, 21.0, -1.0]]], [[[-8.0, -49.0, -13.0, 31.0, 0.0], [-1.0, 57.0, -10.0, 66.0, 70.0], [-53.0, -28.0, 49.0, -102.0, -35.0], [-44.0, -30.0, 30.0, 16.0, 80.0]], [[-21.0, 39.0, 29.0, -32.0, 24.0], [-19.0, -50.0, 23.0, -40.0, -57.0], [35.0, 21.0, -12.0, 68.0, 53.0], [17.0, 5.0, 0.0, 8.0, -51.0]], [[-20.0, -19.0, -23.0, 29.0, -12.0], [13.0, 19.0, 10.0, 14.0, 22.0], [-3.0, -16.0, 19.0, -46.0, -21.0], [-12.0, -10.0, 30.0, 12.0, 20.0]], [[51.0, -12.0, -30.0, 15.0, -38.0], [30.0, 23.0, -33.0, -2.0, 21.0], [-6.0, -3.0, -35.0, 0.0, -50.0], [19.0, 25.0, -40.0, -34.0, 1.0]], [[57.0, 6.0, 24.0, -33.0, 6.0], [-12.0, 3.0, -39.0, 6.0, 3.0], [-24.0, 15.0, -27.0, 36.0, 0.0], [9.0, 15.0, -60.0, -30.0, 3.0]]], [[[21.0, -9.0, 71.0, -53.0, 61.0], [-61.0, 15.0, -28.0, 60.0, 27.0], [-70.0, 9.0, 37.0, -13.0, 42.0], [-42.0, -30.0, -25.0, 7.0, 61.0]], [[-42.0, 0.0, 42.0, -21.0, 49.0], [-42.0, -7.0, 21.0, 28.0, 0.0], [-21.0, 0.0, 49.0, -21.0, 49.0], [-35.0, -35.0, 35.0, 35.0, 28.0]], [[32.0, 25.0, 31.0, -40.0, 15.0], [-17.0, -24.0, -17.0, -18.0, -28.0], [2.0, 22.0, -28.0, 63.0, 26.0], [17.0, 15.0, -45.0, -19.0, -26.0]], [[-41.0, -10.0, 28.0, -7.0, 38.0], [-32.0, 5.0, 19.0, 34.0, 13.0], [-24.0, -7.0, 51.0, -40.0, 32.0], [-37.0, -35.0, 40.0, 34.0, 37.0]], [[-12.0, 18.0, -50.0, 32.0, -46.0], [46.0, -24.0, 22.0, -60.0, -36.0], [64.0, 0.0, -40.0, 34.0, -24.0], [42.0, 30.0, 10.0, -10.0, -64.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank4_offset2(self):
      arg0=Symbol(shape=(4, 5, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, -2.0, -5.0], [1.0, 7.0, -5.0]], [[-6.0, 1.0, 0.0], [2.0, -5.0, 0.0]], [[-6.0, 1.0, -6.0], [-7.0, -6.0, -1.0]], [[0.0, -1.0, 7.0], [1.0, -2.0, 3.0]], [[2.0, -5.0, 0.0], [0.0, -3.0, -3.0]]], [[[-3.0, -3.0, 0.0], [-4.0, 4.0, 5.0]], [[0.0, 2.0, 5.0], [2.0, 5.0, 5.0]], [[1.0, -1.0, 0.0], [-6.0, -6.0, -2.0]], [[-5.0, -4.0, -5.0], [-2.0, 2.0, 4.0]], [[6.0, 2.0, -7.0], [3.0, -6.0, -7.0]]], [[[-5.0, 0.0, -2.0], [-7.0, 5.0, 0.0]], [[-5.0, 5.0, 7.0], [3.0, 0.0, 3.0]], [[6.0, 4.0, -6.0], [-1.0, -5.0, 0.0]], [[4.0, 1.0, -5.0], [6.0, -7.0, -4.0]], [[-4.0, -4.0, 4.0], [-3.0, 4.0, 2.0]]], [[[0.0, 0.0, -2.0], [0.0, 2.0, 5.0]], [[7.0, 5.0, 5.0], [2.0, 3.0, 1.0]], [[-1.0, 7.0, -4.0], [4.0, 2.0, 2.0]], [[7.0, -5.0, 2.0], [-6.0, -3.0, -1.0]], [[1.0, 1.0, 1.0], [-1.0, 7.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[6.0, 5.0, -1.0], [-6.0, 5.0, 3.0]], [[1.0, -4.0, 0.0], [-2.0, -3.0, 3.0]], [[-1.0, 3.0, 0.0], [-2.0, 0.0, 7.0]], [[6.0, -7.0, -5.0], [-3.0, -5.0, -6.0]], [[-6.0, 3.0, -4.0], [-6.0, 1.0, -4.0]]], [[[-4.0, 0.0, 6.0], [-5.0, 6.0, 0.0]], [[-6.0, -7.0, 4.0], [4.0, 5.0, 0.0]], [[-2.0, 0.0, 0.0], [1.0, -2.0, -5.0]], [[-2.0, -5.0, -6.0], [6.0, -3.0, -7.0]], [[4.0, -6.0, -5.0], [5.0, 2.0, -1.0]]], [[[7.0, 0.0, 4.0], [5.0, -4.0, 4.0]], [[3.0, -2.0, 0.0], [-7.0, 0.0, -7.0]], [[-1.0, 5.0, 3.0], [2.0, -3.0, -5.0]], [[-6.0, 0.0, 7.0], [-5.0, 1.0, -6.0]], [[7.0, -4.0, 3.0], [-5.0, 2.0, 4.0]]], [[[5.0, -5.0, 4.0], [5.0, -5.0, 0.0]], [[4.0, -1.0, 0.0], [1.0, 2.0, 6.0]], [[3.0, -1.0, 3.0], [3.0, 4.0, -4.0]], [[-2.0, 4.0, 5.0], [-4.0, -2.0, -5.0]], [[0.0, -6.0, -6.0], [-1.0, 3.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s0=numpy.array([[[[-6.0, 1.0, -4.0], [-4.0, 1.0, 0.0]], [[-2.0, 4.0, -1.0], [-3.0, 7.0, -6.0]], [[-2.0, -7.0, -7.0], [-6.0, -7.0, 1.0]], [[1.0, -3.0, -2.0], [0.0, -7.0, 6.0]], [[-3.0, -7.0, 0.0], [0.0, -5.0, -5.0]]], [[[-3.0, -7.0, 0.0], [-5.0, -7.0, 7.0]], [[-2.0, 4.0, 1.0], [4.0, -3.0, -5.0]], [[2.0, -4.0, 2.0], [6.0, -2.0, 3.0]], [[-1.0, 4.0, -6.0], [-5.0, -5.0, 1.0]], [[-2.0, 0.0, -3.0], [-5.0, -6.0, 1.0]]], [[[1.0, 3.0, 4.0], [4.0, 2.0, -6.0]], [[-5.0, -5.0, -4.0], [6.0, 1.0, -3.0]], [[-2.0, -1.0, -1.0], [4.0, -3.0, 2.0]], [[-3.0, 4.0, 6.0], [-6.0, 1.0, -4.0]], [[2.0, 7.0, -2.0], [6.0, 1.0, 7.0]]], [[[4.0, 6.0, -4.0], [2.0, 1.0, -2.0]], [[1.0, 7.0, -7.0], [-6.0, -2.0, -7.0]], [[6.0, -2.0, 1.0], [-6.0, 0.0, 7.0]], [[5.0, -6.0, -5.0], [-3.0, -7.0, 0.0]], [[6.0, -3.0, -2.0], [3.0, -5.0, 3.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[45.0, 24.0, 83.0, -35.0, -20.0], [35.0, -21.0, 11.0, 56.0, -24.0], [71.0, -5.0, -9.0, -34.0, 20.0], [10.0, -62.0, 15.0, -34.0, 2.0]], [[81.0, -25.0, 7.0, -46.0, -27.0], [4.0, 2.0, -18.0, -5.0, -6.0], [68.0, -4.0, -22.0, -42.0, 13.0], [-14.0, 10.0, 24.0, -33.0, 95.0]], [[-3.0, 28.0, 130.0, -31.0, 49.0], [28.0, -91.0, 81.0, 75.0, 40.0], [31.0, -89.0, 43.0, 29.0, 0.0], [5.0, -116.0, -55.0, 63.0, -66.0]], [[-67.0, 26.0, 39.0, 21.0, 20.0], [8.0, -21.0, 34.0, 27.0, 14.0], [-36.0, -16.0, 41.0, 36.0, -16.0], [20.0, -33.0, -16.0, 33.0, -95.0]], [[16.0, 36.0, 46.0, 2.0, 59.0], [-15.0, -64.0, 44.0, 13.0, 33.0], [-10.0, -35.0, -21.0, 36.0, 10.0], [-35.0, -76.0, -66.0, 34.0, -10.0]]], [[[-63.0, 36.0, 81.0, 37.0, 29.0], [57.0, -24.0, 62.0, 67.0, -54.0], [15.0, -14.0, -6.0, -28.0, 41.0], [21.0, -80.0, -66.0, 58.0, -103.0]], [[3.0, 39.0, 5.0, -2.0, 0.0], [-59.0, -19.0, -2.0, -45.0, 54.0], [-35.0, 34.0, 9.0, 56.0, -38.0], [-33.0, 5.0, 26.0, -42.0, 13.0]], [[-33.0, 6.0, -61.0, 37.0, 21.0], [-11.0, 19.0, -24.0, -8.0, -1.0], [-66.0, 11.0, -12.0, 32.0, -4.0], [7.0, 13.0, -12.0, 5.0, -41.0]], [[-19.0, 25.0, 110.0, -38.0, -10.0], [-4.0, -52.0, 53.0, 23.0, 52.0], [27.0, -29.0, 76.0, 31.0, -39.0], [7.0, -41.0, 25.0, 5.0, -40.0]], [[-29.0, 32.0, 100.0, -11.0, 11.0], [7.0, -50.0, 62.0, 27.0, 23.0], [21.0, -23.0, 41.0, 15.0, -11.0], [-1.0, -56.0, -16.0, 27.0, -49.0]]], [[[18.0, -5.0, -61.0, 7.0, -1.0], [-50.0, 14.0, -26.0, -65.0, 26.0], [-31.0, 32.0, -20.0, 21.0, -16.0], [-34.0, 50.0, 12.0, -24.0, 60.0]], [[78.0, 32.0, 4.0, -28.0, 21.0], [-5.0, -28.0, -36.0, 43.0, 21.0], [-4.0, -19.0, -37.0, 36.0, 4.0], [-5.0, -68.0, 6.0, -54.0, 8.0]], [[-12.0, 34.0, 5.0, 10.0, 4.0], [-9.0, -4.0, -11.0, 13.0, 9.0], [-31.0, 16.0, 1.0, 33.0, -12.0], [6.0, -23.0, 13.0, -28.0, -43.0]], [[-5.0, 5.0, 26.0, 18.0, -17.0], [5.0, 11.0, 31.0, -33.0, -48.0], [50.0, 47.0, -37.0, -65.0, 34.0], [-30.0, 16.0, -23.0, 8.0, 48.0]], [[-34.0, 2.0, -48.0, 4.0, -55.0], [-12.0, 56.0, -61.0, -10.0, 3.0], [-43.0, 50.0, 41.0, 26.0, -44.0], [41.0, 61.0, 95.0, -71.0, -41.0]]], [[[11.0, -19.0, -12.0, -40.0, -19.0], [-44.0, -9.0, -16.0, -34.0, 78.0], [-21.0, -18.0, 65.0, 55.0, -62.0], [0.0, 43.0, 60.0, -23.0, 25.0]], [[32.0, -1.0, 104.0, -79.0, -6.0], [-43.0, -78.0, 56.0, -18.0, 112.0], [41.0, -58.0, 92.0, 52.0, -64.0], [-25.0, -18.0, 34.0, 7.0, 42.0]], [[-66.0, -50.0, -9.0, 24.0, 1.0], [47.0, 24.0, 30.0, 13.0, -42.0], [10.0, -30.0, 28.0, -47.0, 20.0], [33.0, 32.0, -34.0, 83.0, -38.0]], [[-35.0, -7.0, 57.0, -18.0, 61.0], [-13.0, -78.0, 71.0, 16.0, 86.0], [-29.0, -99.0, 74.0, 70.0, -35.0], [-4.0, -47.0, -53.0, 94.0, -52.0]], [[-55.0, -8.0, -21.0, 11.0, 33.0], [-26.0, -20.0, 15.0, -12.0, 62.0], [-72.0, -41.0, 58.0, 72.0, -43.0], [9.0, 11.0, -11.0, 47.0, -58.0]]]])+(1.-msk_ref)*numpy.array([[[[2.0, -5.0, 17.0, -16.0, 80.0], [26.0, 2.0, 6.0, 4.0, -28.0], [-82.0, 8.0, -12.0, 29.0, -36.0], [-76.0, -27.0, -39.0, 10.0, 25.0]], [[44.0, -51.0, -22.0, -25.0, 77.0], [59.0, 3.0, 17.0, -7.0, -22.0], [-85.0, 49.0, 22.0, 63.0, -28.0], [-84.0, -37.0, 30.0, 43.0, 6.0]], [[-36.0, 62.0, 0.0, 119.0, 44.0], [-46.0, -26.0, 7.0, 59.0, 24.0], [-40.0, 43.0, -50.0, -20.0, 13.0], [2.0, -15.0, -70.0, -26.0, 69.0]], [[-24.0, 52.0, 32.0, 36.0, -38.0], [-58.0, -28.0, -18.0, 4.0, 12.0], [51.0, -33.0, -31.0, -63.0, 23.0], [47.0, 29.0, -52.0, -40.0, 9.0]], [[-93.0, 25.0, -53.0, 86.0, 12.0], [-18.0, 42.0, 41.0, 91.0, 25.0], [-21.0, 40.0, 8.0, 43.0, -23.0], [45.0, -45.0, -2.0, 13.0, 27.0]]], [[[-37.0, 77.0, 41.0, 39.0, -8.0], [-5.0, 12.0, -20.0, -17.0, -16.0], [10.0, -9.0, -56.0, -6.0, 46.0], [30.0, 18.0, -73.0, -23.0, 26.0]], [[-47.0, -32.0, -29.0, -12.0, 13.0], [-24.0, -11.0, 39.0, 46.0, -18.0], [2.0, -7.0, 67.0, 26.0, -73.0], [9.0, -44.0, 13.0, 40.0, -43.0]], [[-47.0, 21.0, -5.0, 4.0, -82.0], [-38.0, 38.0, -9.0, 25.0, 45.0], [72.0, -49.0, -13.0, -48.0, 14.0], [78.0, 32.0, 14.0, -45.0, 0.0]], [[28.0, 11.0, 30.0, 30.0, 63.0], [-37.0, -91.0, 2.0, -4.0, -34.0], [-32.0, 17.0, 3.0, -22.0, -22.0], [-49.0, -17.0, -64.0, 13.0, 2.0]], [[-6.0, 29.0, 19.0, 42.0, 44.0], [-21.0, -50.0, 6.0, 3.0, -31.0], [-23.0, 22.0, -4.0, 4.0, -6.0], [-17.0, -19.0, -58.0, 16.0, 5.0]]], [[[-15.0, -43.0, -42.0, -21.0, -11.0], [12.0, 15.0, 28.0, 19.0, -4.0], [11.0, 11.0, 58.0, 40.0, -33.0], [16.0, -27.0, 56.0, 40.0, -40.0]], [[-91.0, -9.0, -43.0, 20.0, 8.0], [-28.0, 78.0, 29.0, 113.0, 65.0], [-37.0, -26.0, -8.0, -9.0, -67.0], [9.0, -25.0, 12.0, -41.0, 51.0]], [[-49.0, 9.0, 5.0, -9.0, -22.0], [-36.0, 16.0, 4.0, 34.0, 15.0], [22.0, -46.0, 1.0, -30.0, -31.0], [26.0, 3.0, -16.0, -25.0, -1.0]], [[25.0, -22.0, -1.0, -39.0, 59.0], [84.0, -5.0, 18.0, -61.0, -90.0], [-47.0, 53.0, 46.0, 115.0, -3.0], [-46.0, -44.0, 7.0, 94.0, -51.0]], [[39.0, -20.0, 56.0, -92.0, -46.0], [-44.0, -40.0, -35.0, -43.0, 1.0], [60.0, -99.0, 1.0, -97.0, -20.0], [-8.0, 51.0, -13.0, -47.0, -33.0]]], [[[45.0, -33.0, -4.0, 3.0, 7.0], [-44.0, -69.0, 2.0, 9.0, 14.0], [10.0, 0.0, 25.0, -49.0, -24.0], [-21.0, 2.0, 12.0, -4.0, -11.0]], [[53.0, -30.0, -17.0, 62.0, 105.0], [-28.0, -117.0, 31.0, 24.0, -30.0], [-71.0, 80.0, 42.0, 15.0, -44.0], [-78.0, -55.0, -23.0, 54.0, 0.0]], [[82.0, 47.0, 49.0, 21.0, -38.0], [12.0, -42.0, -53.0, -93.0, -6.0], [44.0, 15.0, -60.0, -41.0, 111.0], [14.0, 62.0, -23.0, -26.0, 12.0]], [[-12.0, 56.0, -17.0, 141.0, -17.0], [-77.0, -55.0, 1.0, 53.0, 52.0], [28.0, 48.0, -35.0, -57.0, 45.0], [55.0, 9.0, -31.0, -33.0, 48.0]], [[-11.0, 36.0, 0.0, 65.0, -72.0], [-81.0, -36.0, -14.0, 27.0, 54.0], [81.0, -18.0, -21.0, -88.0, 35.0], [77.0, 38.0, -8.0, -51.0, 12.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_float_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=-5.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-5.0, -3.0], [2.0, 2.0]], [[-1.0, 7.0], [-5.0, 3.0]], [[-7.0, 1.0], [-5.0, -2.0]], [[-1.0, 3.0], [5.0, -3.0]], [[0.0, -6.0], [-3.0, -4.0]], [[-3.0, 3.0], [1.0, 7.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[25.0, 15.0], [-10.0, -10.0]], [[5.0, -35.0], [25.0, -15.0]], [[35.0, -5.0], [25.0, 10.0]], [[5.0, -15.0], [-25.0, 15.0]], [[-0.0, 30.0], [15.0, 20.0]], [[15.0, -15.0], [-5.0, -35.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array(5.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-4.0, -5.0], [5.0, -1.0]], [[0.0, -1.0], [5.0, -7.0]], [[-3.0, 7.0], [2.0, -3.0]], [[7.0, -4.0], [4.0, 5.0]], [[-4.0, -1.0], [0.0, 2.0]], [[6.0, 2.0], [-4.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[-20.0, -25.0], [25.0, -5.0]], [[0.0, -5.0], [25.0, -35.0]], [[-15.0, 35.0], [10.0, -15.0]], [[35.0, -20.0], [20.0, 25.0]], [[-20.0, -5.0], [0.0, 10.0]], [[30.0, 10.0], [-20.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=numpy.array([0.0, 0.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[0.0, 4.0, -5.0], [-7.0, 6.0, -2.0]], [[-3.0, 2.0, 6.0], [-7.0, 3.0, 6.0]]], [[[0.0, 5.0, 5.0], [1.0, 1.0, 7.0]], [[-3.0, -6.0, 2.0], [3.0, -6.0, 6.0]]], [[[-1.0, -6.0, 2.0], [-3.0, -5.0, 1.0]], [[-2.0, -3.0, 0.0], [0.0, -6.0, 0.0]]], [[[7.0, -2.0, 2.0], [-2.0, -7.0, 1.0]], [[3.0, -4.0, -4.0], [6.0, -3.0, -4.0]]], [[[6.0, -4.0, 4.0], [-4.0, 4.0, -4.0]], [[3.0, 5.0, 6.0], [0.0, -5.0, 2.0]]], [[[-2.0, -3.0, -6.0], [7.0, 0.0, 6.0]], [[-4.0, 0.0, 3.0], [0.0, -2.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_array_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=numpy.array([4.0, -2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[4.0, -6.0], [1.0, 0.0]], [[-7.0, -1.0], [2.0, -3.0]], [[3.0, -1.0], [5.0, 0.0]], [[-2.0, 5.0], [0.0, 0.0]], [[-5.0, -4.0], [2.0, 2.0]], [[-1.0, 0.0], [-1.0, -2.0]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[16.0, -8.0], [-24.0, 12.0]], [[4.0, -2.0], [0.0, 0.0]]], [[[-28.0, 14.0], [-4.0, 2.0]], [[8.0, -4.0], [-12.0, 6.0]]], [[[12.0, -6.0], [-4.0, 2.0]], [[20.0, -10.0], [0.0, 0.0]]], [[[-8.0, 4.0], [20.0, -10.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[-20.0, 10.0], [-16.0, 8.0]], [[8.0, -4.0], [8.0, -4.0]]], [[[-4.0, 2.0], [0.0, 0.0]], [[-4.0, 2.0], [-8.0, 4.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=numpy.array([[-2.0, -1.0, -2.0], [5.0, 1.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[2.0, 4.0, -5.0], [5.0, -6.0, 2.0]], [[6.0, -2.0, 1.0], [-6.0, 0.0, -4.0]]], [[[7.0, 7.0, -7.0], [5.0, -7.0, 0.0]], [[6.0, 7.0, 3.0], [7.0, -3.0, 0.0]]], [[[-3.0, -3.0, -4.0], [-5.0, -1.0, -2.0]], [[-4.0, 1.0, 1.0], [4.0, 2.0, -5.0]]], [[[-2.0, -4.0, 6.0], [-2.0, -4.0, -1.0]], [[-4.0, -7.0, -3.0], [-6.0, 3.0, 0.0]]], [[[3.0, -2.0, 0.0], [2.0, -7.0, -4.0]], [[2.0, -7.0, -7.0], [0.0, 5.0, 4.0]]], [[[-1.0, 1.0, -1.0], [4.0, 0.0, -3.0]], [[6.0, -4.0, -3.0], [4.0, 6.0, -4.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[2.0, 49.0], [-8.0, 5.0]], [[-12.0, 21.0], [20.0, -2.0]]], [[[-7.0, 91.0], [-3.0, 18.0]], [[-25.0, 16.0], [-11.0, 32.0]]], [[[17.0, 10.0], [15.0, -12.0]], [[5.0, -26.0], [0.0, 57.0]]], [[[-4.0, -56.0], [10.0, -7.0]], [[21.0, -6.0], [9.0, -27.0]]], [[[-4.0, 13.0], [11.0, 31.0]], [[17.0, 52.0], [-13.0, -23.0]]], [[[3.0, 3.0], [-2.0, 41.0]], [[-2.0, 47.0], [-6.0, 54.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[4.0, -6.0], [7.0, 4.0]], [[7.0, 0.0], [1.0, 6.0]], [[5.0, -2.0], [-1.0, 0.0]], [[6.0, 6.0], [-7.0, -7.0]], [[5.0, -7.0], [5.0, 6.0]], [[-5.0, 2.0], [6.0, -7.0]]])
      s1=numpy.array(-2.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[-8.0, 12.0], [-14.0, -8.0]], [[-14.0, -0.0], [-2.0, -12.0]], [[-10.0, 4.0], [2.0, -0.0]], [[-12.0, -12.0], [14.0, 14.0]], [[-10.0, 14.0], [-10.0, -12.0]], [[10.0, -4.0], [-12.0, 14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[7.0, -2.0, 5.0], [5.0, 7.0, -6.0]], [[4.0, 7.0, 7.0], [-6.0, -5.0, -2.0]]], [[[6.0, -4.0, 1.0], [-2.0, 5.0, 2.0]], [[-5.0, 1.0, -4.0], [7.0, -3.0, -4.0]]], [[[7.0, 7.0, 3.0], [2.0, 0.0, 6.0]], [[-6.0, 0.0, -3.0], [3.0, 5.0, 5.0]]], [[[3.0, 0.0, 6.0], [4.0, -5.0, 0.0]], [[-5.0, 4.0, 4.0], [-2.0, -6.0, 7.0]]], [[[6.0, 1.0, 5.0], [7.0, 4.0, 6.0]], [[0.0, -2.0, -2.0], [-2.0, 7.0, -6.0]]], [[[2.0, 0.0, 6.0], [4.0, -7.0, -5.0]], [[7.0, -6.0, -7.0], [-7.0, -4.0, 0.0]]]])
      s1=numpy.array([-3.0, -1.0, 4.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[1.0, -46.0], [9.0, 15.0]], [[-10.0, 9.0], [-2.0, -34.0]], [[-16.0, 18.0], [6.0, 6.0]], [[15.0, -7.0], [27.0, 40.0]], [[1.0, -1.0], [-6.0, -25.0]], [[18.0, -25.0], [-43.0, 25.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_Symbol_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[7.0, 2.0], [-2.0, 3.0]], [[5.0, -1.0], [4.0, 3.0]], [[-1.0, 2.0], [-6.0, 7.0]], [[7.0, -2.0], [4.0, 6.0]], [[1.0, -4.0], [-3.0, -2.0]], [[4.0, 1.0], [-6.0, 5.0]]])
      s1=numpy.array([0.0, -1.0])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[0.0, -7.0], [0.0, -2.0]], [[0.0, 2.0], [0.0, -3.0]]], [[[0.0, -5.0], [0.0, 1.0]], [[0.0, -4.0], [0.0, -3.0]]], [[[0.0, 1.0], [0.0, -2.0]], [[0.0, 6.0], [0.0, -7.0]]], [[[0.0, -7.0], [0.0, 2.0]], [[0.0, -4.0], [0.0, -6.0]]], [[[0.0, -1.0], [0.0, 4.0]], [[0.0, 3.0], [0.0, 2.0]]], [[[0.0, -4.0], [0.0, -1.0]], [[0.0, 6.0], [0.0, -5.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[-1.0, 0.0, 4.0], [0.0, 2.0, 5.0]], [[0.0, 1.0, -5.0], [4.0, -2.0, -4.0]]], [[[0.0, 4.0, -7.0], [-7.0, 3.0, -6.0]], [[4.0, 5.0, -7.0], [-3.0, 5.0, -1.0]]], [[[3.0, -4.0, 4.0], [-1.0, -2.0, -5.0]], [[1.0, -1.0, 4.0], [5.0, 0.0, 1.0]]], [[[-4.0, 1.0, -3.0], [4.0, -6.0, 5.0]], [[6.0, 4.0, -7.0], [-6.0, -4.0, -2.0]]], [[[-4.0, 0.0, -5.0], [-6.0, 3.0, 7.0]], [[7.0, -3.0, -4.0], [4.0, -3.0, 3.0]]], [[[5.0, -4.0, -4.0], [0.0, -7.0, 6.0]], [[2.0, 6.0, -6.0], [0.0, 3.0, 6.0]]]])
      s1=numpy.array([[2.0, -4.0, 0.0], [3.0, 7.0, -6.0]])
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-2.0, -27.0], [-8.0, -16.0]], [[-4.0, 37.0], [16.0, 22.0]]], [[[-16.0, 70.0], [-26.0, 36.0]], [[-12.0, 89.0], [-26.0, 32.0]]], [[[22.0, -43.0], [6.0, 13.0]], [[6.0, -28.0], [10.0, 9.0]]], [[[-12.0, 13.0], [32.0, -60.0]], [[-4.0, 88.0], [4.0, -34.0]]], [[[-8.0, 18.0], [-24.0, -39.0]], [[26.0, 24.0], [20.0, -27.0]]], [[[26.0, 11.0], [28.0, -85.0]], [[-20.0, 84.0], [-12.0, -15.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(4.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[1.0, 7.0], [-2.0, 5.0]], [[0.0, 2.0], [-7.0, 5.0]], [[-5.0, 0.0], [-3.0, -7.0]], [[1.0, -1.0], [6.0, 4.0]], [[-4.0, -7.0], [4.0, 6.0]], [[0.0, 7.0], [-6.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[4.0, 28.0], [-8.0, 20.0]], [[0.0, 8.0], [-28.0, 20.0]], [[-20.0, 0.0], [-12.0, -28.0]], [[4.0, -4.0], [24.0, 16.0]], [[-16.0, -28.0], [16.0, 24.0]], [[0.0, 28.0], [-24.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Data(numpy.array([1.0, 5.0, -1.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[-3.0, -3.0, 2.0], [-2.0, -7.0, 3.0]], [[-3.0, 0.0, -1.0], [-2.0, -1.0, -2.0]]], [[[0.0, 0.0, -5.0], [-7.0, -3.0, -5.0]], [[1.0, 1.0, 0.0], [4.0, 0.0, -5.0]]], [[[1.0, -5.0, -7.0], [-6.0, -5.0, 0.0]], [[3.0, 6.0, 0.0], [6.0, 4.0, 1.0]]], [[[0.0, 5.0, 0.0], [-3.0, -6.0, -2.0]], [[-1.0, -2.0, 7.0], [7.0, 1.0, -2.0]]], [[[3.0, 1.0, 1.0], [5.0, 5.0, 2.0]], [[-3.0, -5.0, 2.0], [2.0, 4.0, 2.0]]], [[[0.0, -1.0, 2.0], [1.0, 6.0, 4.0]], [[0.0, -4.0, -2.0], [2.0, -4.0, 2.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[-20.0, -40.0], [-2.0, -5.0]], [[5.0, -17.0], [6.0, 9.0]], [[-17.0, -31.0], [33.0, 25.0]], [[25.0, -31.0], [-18.0, 14.0]], [[7.0, 28.0], [-30.0, 20.0]], [[-7.0, 27.0], [-18.0, -20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_constData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      arg1=Data(numpy.array([3.0, 7.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-1.0, 4.0], [-5.0, 0.0]], [[-6.0, 7.0], [-3.0, -1.0]], [[0.0, -3.0], [5.0, 7.0]], [[-6.0, -1.0], [0.0, -6.0]], [[-7.0, -3.0], [3.0, -1.0]], [[-2.0, 6.0], [-7.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-3.0, -7.0], [12.0, 28.0]], [[-15.0, -35.0], [0.0, 0.0]]], [[[-18.0, -42.0], [21.0, 49.0]], [[-9.0, -21.0], [-3.0, -7.0]]], [[[0.0, 0.0], [-9.0, -21.0]], [[15.0, 35.0], [21.0, 49.0]]], [[[-18.0, -42.0], [-3.0, -7.0]], [[0.0, 0.0], [-18.0, -42.0]]], [[[-21.0, -49.0], [-9.0, -21.0]], [[9.0, 21.0], [-3.0, -7.0]]], [[[-6.0, -14.0], [18.0, 42.0]], [[-21.0, -49.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      arg1=Data(numpy.array([[-6.0, -7.0, 0.0], [4.0, 7.0, -2.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[4.0, -3.0, -5.0], [-4.0, -5.0, 6.0]], [[-6.0, -5.0, -6.0], [6.0, 5.0, 1.0]]], [[[3.0, 1.0, 0.0], [4.0, 4.0, -4.0]], [[6.0, 0.0, 0.0], [-7.0, -6.0, 7.0]]], [[[-4.0, -2.0, -6.0], [3.0, 4.0, 5.0]], [[6.0, 4.0, -2.0], [4.0, 2.0, -4.0]]], [[[1.0, 2.0, 0.0], [3.0, -1.0, 1.0]], [[1.0, 0.0, -1.0], [1.0, 0.0, -4.0]]], [[[4.0, -5.0, 3.0], [4.0, -4.0, 1.0]], [[-1.0, 4.0, -6.0], [-3.0, 1.0, 7.0]]], [[[6.0, -2.0, -2.0], [1.0, -6.0, -2.0]], [[4.0, 7.0, -6.0], [0.0, 1.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[-3.0, 5.0], [59.0, -63.0]], [[71.0, -47.0], [-71.0, 57.0]]], [[[-25.0, 19.0], [-52.0, 52.0]], [[-36.0, 24.0], [84.0, -84.0]]], [[[38.0, -18.0], [-46.0, 30.0]], [[-64.0, 56.0], [-38.0, 38.0]]], [[[-20.0, 18.0], [-11.0, 3.0]], [[-6.0, 6.0], [-6.0, 12.0]]], [[[11.0, -25.0], [4.0, -14.0]], [[-22.0, 36.0], [11.0, -19.0]]], [[[-22.0, 14.0], [36.0, -34.0]], [[-73.0, 77.0], [-7.0, 7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-1.0)+(1-msk_arg1)*(-4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-1.0, -5.0], [6.0, 1.0]], [[-4.0, -3.0], [5.0, 1.0]], [[-7.0, -6.0], [0.0, -2.0]], [[6.0, -2.0], [-7.0, 0.0]], [[5.0, 0.0], [5.0, 7.0]], [[0.0, 0.0], [-1.0, -5.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[1.0, 5.0], [-6.0, -1.0]], [[4.0, 3.0], [-5.0, -1.0]], [[7.0, 6.0], [-0.0, 2.0]], [[-6.0, 2.0], [7.0, -0.0]], [[-5.0, -0.0], [-5.0, -7.0]], [[-0.0, -0.0], [1.0, 5.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 20.0], [-24.0, -4.0]], [[16.0, 12.0], [-20.0, -4.0]], [[28.0, 24.0], [-0.0, 8.0]], [[-24.0, 8.0], [28.0, -0.0]], [[-20.0, -0.0], [-20.0, -28.0]], [[-0.0, -0.0], [4.0, 20.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank1_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([5.0, 5.0, -5.0])+(1.-msk_arg1)*numpy.array([-3.0, 5.0, 2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[-6.0, -2.0, -2.0], [7.0, 4.0, 7.0]], [[5.0, 5.0, 4.0], [6.0, 0.0, -3.0]]], [[[1.0, -2.0, 0.0], [-6.0, -3.0, -2.0]], [[-3.0, -5.0, 4.0], [0.0, -1.0, -2.0]]], [[[3.0, -1.0, -7.0], [5.0, -4.0, -2.0]], [[5.0, -5.0, -3.0], [5.0, -3.0, 1.0]]], [[[5.0, -3.0, 3.0], [-4.0, 4.0, 0.0]], [[-3.0, 3.0, -7.0], [0.0, 6.0, 4.0]]], [[[0.0, -5.0, -5.0], [-5.0, -7.0, 3.0]], [[3.0, 0.0, 2.0], [2.0, -2.0, 0.0]]], [[[4.0, -2.0, 1.0], [0.0, 3.0, -7.0]], [[3.0, -3.0, -7.0], [-4.0, 5.0, -6.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-30.0, 20.0], [30.0, 45.0]], [[-5.0, -35.0], [-60.0, 5.0]], [[45.0, 15.0], [15.0, 5.0]], [[-5.0, 0.0], [35.0, 10.0]], [[0.0, -75.0], [5.0, 0.0]], [[5.0, 50.0], [35.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 13.0], [18.0, -24.0]], [[-13.0, -1.0], [-8.0, -9.0]], [[-28.0, -39.0], [-46.0, -28.0]], [[-24.0, 32.0], [10.0, 38.0]], [[-35.0, -14.0], [-5.0, -16.0]], [[-20.0, 1.0], [-38.0, 25.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank3_expandedData_rank1_offset0(self):
      arg0=Symbol(shape=(6, 2, 2))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-2.0, 4.0])+(1.-msk_arg1)*numpy.array([-2.0, 2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[-6.0, -5.0], [0.0, -1.0]], [[3.0, -4.0], [0.0, 5.0]], [[0.0, 3.0], [-2.0, 6.0]], [[5.0, -3.0], [6.0, 0.0]], [[-1.0, -4.0], [1.0, 0.0]], [[0.0, 0.0], [-1.0, 0.0]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[12.0, -24.0], [10.0, -20.0]], [[0.0, 0.0], [2.0, -4.0]]], [[[-6.0, 12.0], [8.0, -16.0]], [[0.0, 0.0], [-10.0, 20.0]]], [[[0.0, 0.0], [-6.0, 12.0]], [[4.0, -8.0], [-12.0, 24.0]]], [[[-10.0, 20.0], [6.0, -12.0]], [[-12.0, 24.0], [0.0, 0.0]]], [[[2.0, -4.0], [8.0, -16.0]], [[-2.0, 4.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[2.0, -4.0], [0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[12.0, -12.0], [10.0, -10.0]], [[0.0, 0.0], [2.0, -2.0]]], [[[-6.0, 6.0], [8.0, -8.0]], [[0.0, 0.0], [-10.0, 10.0]]], [[[0.0, 0.0], [-6.0, 6.0]], [[4.0, -4.0], [-12.0, 12.0]]], [[[-10.0, 10.0], [6.0, -6.0]], [[-12.0, 12.0], [0.0, 0.0]]], [[[2.0, -2.0], [8.0, -8.0]], [[-2.0, 2.0], [0.0, 0.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[2.0, -2.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank2_offset1(self):
      arg0=Symbol(shape=(6, 2, 2, 3))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, 4.0, 3.0], [4.0, -1.0, -7.0]])+(1.-msk_arg1)*numpy.array([[7.0, 0.0, -3.0], [7.0, 6.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s0=numpy.array([[[[5.0, 5.0, 0.0], [5.0, -7.0, -5.0]], [[-2.0, -5.0, 5.0], [-5.0, -6.0, 6.0]]], [[[3.0, -5.0, -2.0], [-6.0, -7.0, -6.0]], [[-1.0, 0.0, 1.0], [-5.0, 0.0, 6.0]]], [[[2.0, 2.0, 3.0], [-4.0, -7.0, -7.0]], [[-3.0, 1.0, 6.0], [6.0, -1.0, -5.0]]], [[[3.0, 3.0, -3.0], [-3.0, 3.0, -5.0]], [[-3.0, -6.0, -4.0], [-6.0, 0.0, -7.0]]], [[[-6.0, -3.0, 7.0], [0.0, 7.0, 5.0]], [[0.0, -6.0, -7.0], [-4.0, 3.0, -1.0]]], [[[-1.0, -7.0, 7.0], [1.0, 6.0, -1.0]], [[5.0, -5.0, -3.0], [-2.0, -3.0, 5.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[50.0, 15.0], [-13.0, 62.0]], [[-17.0, -38.0], [-36.0, -56.0]]], [[[-8.0, 31.0], [-82.0, 25.0]], [[-3.0, -11.0], [-12.0, -62.0]]], [[[29.0, -15.0], [-73.0, 40.0]], [[4.0, -55.0], [17.0, 60.0]]], [[[21.0, 30.0], [-21.0, 20.0]], [[-54.0, 22.0], [-57.0, 25.0]]], [[[-27.0, -70.0], [43.0, -42.0]], [[-45.0, 55.0], [-15.0, -12.0]]], [[[-13.0, -46.0], [27.0, 5.0]], [[1.0, 46.0], [-9.0, -40.0]]]])+(1.-msk_ref)*numpy.array([[[[35.0, 65.0], [50.0, 8.0]], [[-29.0, -59.0], [-53.0, -89.0]]], [[[27.0, -3.0], [-24.0, -66.0]], [[-10.0, -10.0], [-53.0, -53.0]]], [[[5.0, 17.0], [-7.0, -49.0]], [[-39.0, -33.0], [57.0, 51.0]]], [[[30.0, 48.0], [-6.0, 12.0]], [[-9.0, -45.0], [-21.0, -21.0]]], [[[-63.0, -81.0], [-15.0, 27.0]], [[21.0, -15.0], [-25.0, -7.0]]], [[[-28.0, -70.0], [10.0, 46.0]], [[44.0, 14.0], [-29.0, -47.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_float_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=2.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-7.0, 7.0, -6.0, -7.0], [5.0, 3.0, -3.0, 4.0], [7.0, 7.0, -6.0, -5.0]], [[0.0, -1.0, 0.0, -1.0], [-5.0, 7.0, 1.0, 3.0], [3.0, 1.0, -2.0, 6.0]]], [[[4.0, -4.0, -5.0, 0.0], [2.0, -2.0, -2.0, -7.0], [0.0, -7.0, 6.0, -4.0]], [[0.0, 1.0, 7.0, -3.0], [0.0, 4.0, -1.0, 0.0], [-3.0, 5.0, -6.0, 3.0]]], [[[3.0, -7.0, -3.0, -5.0], [-2.0, 1.0, 3.0, -3.0], [0.0, -6.0, -2.0, -2.0]], [[-1.0, 6.0, 4.0, -1.0], [3.0, 4.0, -3.0, 5.0], [-1.0, -7.0, 0.0, 0.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[-14.0, 14.0, -12.0, -14.0], [10.0, 6.0, -6.0, 8.0], [14.0, 14.0, -12.0, -10.0]], [[0.0, -2.0, 0.0, -2.0], [-10.0, 14.0, 2.0, 6.0], [6.0, 2.0, -4.0, 12.0]]], [[[8.0, -8.0, -10.0, 0.0], [4.0, -4.0, -4.0, -14.0], [0.0, -14.0, 12.0, -8.0]], [[0.0, 2.0, 14.0, -6.0], [0.0, 8.0, -2.0, 0.0], [-6.0, 10.0, -12.0, 6.0]]], [[[6.0, -14.0, -6.0, -10.0], [-4.0, 2.0, 6.0, -6.0], [0.0, -12.0, -4.0, -4.0]], [[-2.0, 12.0, 8.0, -2.0], [6.0, 8.0, -6.0, 10.0], [-2.0, -14.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_array_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=numpy.array(-6.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-7.0, 2.0, 0.0, 6.0], [7.0, 0.0, 7.0, -7.0], [-4.0, 7.0, -5.0, -6.0]], [[5.0, 4.0, -4.0, 3.0], [1.0, 5.0, 0.0, -6.0], [-3.0, -5.0, 6.0, -5.0]]], [[[4.0, 0.0, 0.0, -3.0], [-3.0, -6.0, 4.0, -2.0], [2.0, -6.0, 6.0, -7.0]], [[-3.0, 2.0, -5.0, -5.0], [-4.0, -5.0, 7.0, 5.0], [-6.0, -5.0, -1.0, 1.0]]], [[[5.0, -6.0, -2.0, 4.0], [-5.0, 5.0, 6.0, -3.0], [-2.0, 2.0, 7.0, 7.0]], [[7.0, 0.0, -3.0, 0.0], [-4.0, -1.0, -7.0, 4.0], [-5.0, -3.0, 1.0, 6.0]]]])
      sub=res.substitute({arg0:s0})
      ref=numpy.array([[[[42.0, -12.0, -0.0, -36.0], [-42.0, -0.0, -42.0, 42.0], [24.0, -42.0, 30.0, 36.0]], [[-30.0, -24.0, 24.0, -18.0], [-6.0, -30.0, -0.0, 36.0], [18.0, 30.0, -36.0, 30.0]]], [[[-24.0, -0.0, -0.0, 18.0], [18.0, 36.0, -24.0, 12.0], [-12.0, 36.0, -36.0, 42.0]], [[18.0, -12.0, 30.0, 30.0], [24.0, 30.0, -42.0, -30.0], [36.0, 30.0, 6.0, -6.0]]], [[[-30.0, 36.0, 12.0, -24.0], [30.0, -30.0, -36.0, 18.0], [12.0, -12.0, -42.0, -42.0]], [[-42.0, -0.0, 18.0, -0.0], [24.0, 6.0, 42.0, -24.0], [30.0, 18.0, -6.0, -36.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_Symbol_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-5.0, -3.0, -6.0, 1.0], [5.0, 4.0, -2.0, 7.0], [-2.0, -6.0, -2.0, -3.0]], [[-4.0, 2.0, 1.0, 2.0], [1.0, 4.0, 6.0, -7.0], [1.0, 0.0, -6.0, -6.0]]], [[[0.0, -1.0, 5.0, 0.0], [-5.0, -4.0, -7.0, 0.0], [-2.0, 5.0, -1.0, -5.0]], [[-2.0, 5.0, 0.0, -2.0], [7.0, 2.0, 4.0, 1.0], [6.0, 7.0, -7.0, -1.0]]], [[[-3.0, 0.0, 0.0, 2.0], [0.0, -4.0, -4.0, -5.0], [-5.0, 0.0, 0.0, -4.0]], [[-2.0, 3.0, 3.0, -6.0], [1.0, 0.0, 3.0, -2.0], [3.0, 3.0, 1.0, 1.0]]]])
      s1=numpy.array(5.0)
      sub=res.substitute({arg0:s0,arg1:s1})
      ref=numpy.array([[[[-25.0, -15.0, -30.0, 5.0], [25.0, 20.0, -10.0, 35.0], [-10.0, -30.0, -10.0, -15.0]], [[-20.0, 10.0, 5.0, 10.0], [5.0, 20.0, 30.0, -35.0], [5.0, 0.0, -30.0, -30.0]]], [[[0.0, -5.0, 25.0, 0.0], [-25.0, -20.0, -35.0, 0.0], [-10.0, 25.0, -5.0, -25.0]], [[-10.0, 25.0, 0.0, -10.0], [35.0, 10.0, 20.0, 5.0], [30.0, 35.0, -35.0, -5.0]]], [[[-15.0, 0.0, 0.0, 10.0], [0.0, -20.0, -20.0, -25.0], [-25.0, 0.0, 0.0, -20.0]], [[-10.0, 15.0, 15.0, -30.0], [5.0, 0.0, 15.0, -10.0], [15.0, 15.0, 5.0, 5.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_constData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      arg1=Data(0.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[6.0, 5.0, -7.0, -6.0], [1.0, 5.0, 1.0, 5.0], [-5.0, -3.0, 0.0, -5.0]], [[4.0, 7.0, 0.0, 2.0], [1.0, 7.0, 2.0, 2.0], [-2.0, 0.0, 3.0, -1.0]]], [[[-2.0, 7.0, -1.0, -6.0], [0.0, 7.0, 6.0, 2.0], [0.0, -3.0, -3.0, 7.0]], [[-5.0, -6.0, -2.0, -5.0], [-4.0, -2.0, -7.0, 6.0], [-6.0, 7.0, 5.0, -7.0]]], [[[0.0, 2.0, -7.0, 3.0], [6.0, 2.0, 4.0, 5.0], [7.0, -2.0, 5.0, 3.0]], [[1.0, 0.0, -1.0, -7.0], [-3.0, 5.0, 5.0, -6.0], [1.0, -7.0, -1.0, -7.0]]]])
      sub=res.substitute({arg0:s0})
      ref=Data(numpy.array([[[[0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, 0.0, -0.0]], [[0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, -0.0]]], [[[-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[-0.0, -0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0]]], [[[0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_Symbol_rank4_expandedData_rank0_offset0(self):
      arg0=Symbol(shape=(3, 2, 3, 4))
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s0=numpy.array([[[[-6.0, 5.0, -5.0, -5.0], [6.0, 4.0, -2.0, -5.0], [-2.0, 7.0, 6.0, 2.0]], [[-1.0, -1.0, -7.0, -1.0], [5.0, 0.0, -2.0, 0.0], [-4.0, 6.0, 3.0, -7.0]]], [[[7.0, 4.0, 3.0, 7.0], [4.0, 6.0, -6.0, 3.0], [2.0, 2.0, 0.0, 0.0]], [[7.0, 2.0, 0.0, 2.0], [-3.0, -6.0, 0.0, -4.0], [0.0, -3.0, -3.0, 2.0]]], [[[-5.0, 6.0, -4.0, 0.0], [-5.0, 1.0, 5.0, -2.0], [-3.0, -7.0, -4.0, -1.0]], [[1.0, 3.0, 2.0, -3.0], [0.0, 3.0, 6.0, -4.0], [-6.0, 0.0, -5.0, -1.0]]]])
      sub=res.substitute({arg0:s0})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, 0.0, 0.0]], [[-0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0]]], [[[0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0], [-0.0, -0.0, 0.0, -0.0], [0.0, -0.0, -0.0, 0.0]]], [[[-0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, -0.0]], [[0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, -0.0]]]])+(1.-msk_ref)*numpy.array([[[[42.0, -35.0, 35.0, 35.0], [-42.0, -28.0, 14.0, 35.0], [14.0, -49.0, -42.0, -14.0]], [[7.0, 7.0, 49.0, 7.0], [-35.0, -0.0, 14.0, -0.0], [28.0, -42.0, -21.0, 49.0]]], [[[-49.0, -28.0, -21.0, -49.0], [-28.0, -42.0, 42.0, -21.0], [-14.0, -14.0, -0.0, -0.0]], [[-49.0, -14.0, -0.0, -14.0], [21.0, 42.0, -0.0, 28.0], [-0.0, 21.0, 21.0, -14.0]]], [[[35.0, -42.0, 28.0, -0.0], [35.0, -7.0, -35.0, 14.0], [21.0, 49.0, 28.0, 7.0]], [[-7.0, -21.0, -14.0, 21.0], [-0.0, -21.0, -42.0, 28.0], [42.0, -0.0, 35.0, 7.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_float_rank0_offset0(self):
      arg0=Data(-6.0,self.functionspace)
      arg1=-4.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(24.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_array_rank0_offset0(self):
      arg0=Data(4.0,self.functionspace)
      arg1=numpy.array(5.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(20.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank1_offset1(self):
      arg0=Data(numpy.array([5.0, -5.0, 7.0]),self.functionspace)
      arg1=numpy.array([5.0, -1.0, 5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(65.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank2_offset2(self):
      arg0=Data(numpy.array([[-5.0, -7.0, 4.0], [-1.0, 6.0, -7.0]]),self.functionspace)
      arg1=numpy.array([[5.0, 7.0, 0.0], [-1.0, -5.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(-54.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[-6.0, 6.0, -6.0], [-5.0, 2.0, -2.0], [-5.0, -6.0, 0.0], [-5.0, -4.0, 5.0]], [[-3.0, -2.0, -5.0], [-2.0, -4.0, 3.0], [-1.0, -5.0, 4.0], [-1.0, -2.0, 1.0]]]),self.functionspace)
      arg1=numpy.array([[[-7.0, 3.0, -6.0], [4.0, -3.0, -7.0], [3.0, -4.0, -4.0], [5.0, 1.0, 7.0]], [[-2.0, -7.0, 1.0], [3.0, 3.0, -3.0], [6.0, 0.0, -6.0], [4.0, 0.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(53.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank4_offset4(self):
      arg0=Data(numpy.array([[[[-5.0, -4.0], [3.0, -1.0], [-7.0, 3.0]], [[7.0, 6.0], [3.0, 0.0], [2.0, -4.0]]], [[[-1.0, 6.0], [0.0, 3.0], [-3.0, -2.0]], [[1.0, 6.0], [-7.0, -4.0], [-2.0, -5.0]]], [[[5.0, -4.0], [-1.0, -4.0], [-7.0, 7.0]], [[-5.0, -3.0], [2.0, 4.0], [-6.0, -5.0]]], [[[6.0, 4.0], [-5.0, 4.0], [2.0, 3.0]], [[5.0, -2.0], [2.0, -2.0], [-5.0, 5.0]]]]),self.functionspace)
      arg1=numpy.array([[[[1.0, 5.0], [4.0, -3.0], [7.0, -1.0]], [[-7.0, -2.0], [-2.0, 2.0], [6.0, 1.0]]], [[[-6.0, 3.0], [-7.0, 3.0], [5.0, 0.0]], [[2.0, 5.0], [-6.0, 4.0], [3.0, 5.0]]], [[[3.0, 0.0], [6.0, 0.0], [-2.0, -5.0]], [[1.0, 7.0], [-3.0, 7.0], [1.0, -5.0]]], [[[4.0, -3.0], [2.0, -4.0], [-6.0, 0.0]], [[-2.0, -7.0], [7.0, 3.0], [-5.0, 7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      ref=Data(-27.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_array_rank1_offset0(self):
      arg0=Data(4.0,self.functionspace)
      arg1=numpy.array([7.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([28.0, 4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank2_offset1(self):
      arg0=Data(numpy.array([-2.0, 3.0, 0.0]),self.functionspace)
      arg1=numpy.array([[-7.0, -3.0, -2.0], [-2.0, -2.0, 4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([5.0, -2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank3_offset2(self):
      arg0=Data(numpy.array([[-6.0, 0.0, -1.0], [1.0, -3.0, 5.0]]),self.functionspace)
      arg1=numpy.array([[[-5.0, 2.0, 3.0], [0.0, 7.0, 4.0]], [[6.0, -4.0, 3.0], [0.0, 5.0, -5.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([26.0, -79.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[-5.0, -3.0, -5.0], [-7.0, -6.0, 2.0], [-1.0, 5.0, 3.0], [6.0, -2.0, -3.0]], [[-5.0, 6.0, 2.0], [-7.0, -6.0, -5.0], [-2.0, 5.0, -2.0], [-1.0, 5.0, 4.0]]]),self.functionspace)
      arg1=numpy.array([[[[-4.0, 6.0, -3.0], [2.0, 3.0, -7.0], [-5.0, 3.0, -4.0], [-7.0, -2.0, 3.0]], [[-2.0, 7.0, 7.0], [4.0, -2.0, -5.0], [-2.0, 2.0, -5.0], [-1.0, 5.0, -5.0]]], [[[-4.0, -2.0, -4.0], [-4.0, -5.0, 6.0], [3.0, -6.0, -3.0], [4.0, -4.0, -5.0]], [[-2.0, 0.0, -6.0], [-6.0, -4.0, 0.0], [5.0, 0.0, -7.0], [1.0, -7.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([37.0, 133.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_array_rank2_offset0(self):
      arg0=Data(-2.0,self.functionspace)
      arg1=numpy.array([[-2.0, 7.0, -3.0, -6.0, 4.0], [6.0, -7.0, 3.0, -4.0, -6.0], [0.0, -2.0, -2.0, -5.0, -4.0], [-5.0, -7.0, 0.0, 4.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[4.0, -14.0, 6.0, 12.0, -8.0], [-12.0, 14.0, -6.0, 8.0, 12.0], [-0.0, 4.0, 4.0, 10.0, 8.0], [10.0, 14.0, -0.0, -8.0, -0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank3_offset1(self):
      arg0=Data(numpy.array([3.0, 5.0, 1.0]),self.functionspace)
      arg1=numpy.array([[[6.0, 0.0, 5.0], [5.0, -7.0, 0.0], [3.0, -4.0, 2.0], [3.0, 6.0, 2.0], [-7.0, -2.0, -7.0]], [[2.0, -5.0, 0.0], [-5.0, 7.0, -3.0], [-1.0, 0.0, 4.0], [2.0, 7.0, -2.0], [-1.0, 0.0, 7.0]], [[-4.0, 1.0, -6.0], [2.0, 5.0, -1.0], [-7.0, 6.0, 4.0], [0.0, -2.0, -2.0], [-6.0, -7.0, 1.0]], [[-5.0, 0.0, 0.0], [7.0, -1.0, 2.0], [6.0, -7.0, 5.0], [3.0, 0.0, -2.0], [0.0, 0.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[23.0, -20.0, -9.0, 41.0, -38.0], [-19.0, 17.0, 1.0, 39.0, 4.0], [-13.0, 30.0, 13.0, -12.0, -52.0], [-15.0, 18.0, -12.0, 7.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank4_offset2(self):
      arg0=Data(numpy.array([[3.0, 4.0, 0.0], [4.0, -7.0, -2.0]]),self.functionspace)
      arg1=numpy.array([[[[4.0, -2.0, -6.0], [-7.0, -5.0, -6.0]], [[-6.0, -6.0, 0.0], [-1.0, 0.0, 3.0]], [[6.0, 1.0, 0.0], [7.0, -6.0, 3.0]], [[-2.0, 0.0, -3.0], [0.0, 0.0, 6.0]], [[-7.0, -3.0, 4.0], [-4.0, 3.0, 6.0]]], [[[0.0, -4.0, -7.0], [1.0, -5.0, 4.0]], [[-6.0, -6.0, 4.0], [7.0, 6.0, 5.0]], [[1.0, 3.0, -4.0], [-5.0, -6.0, 7.0]], [[1.0, -2.0, 7.0], [-6.0, 4.0, 7.0]], [[-4.0, 6.0, 5.0], [0.0, -5.0, -6.0]]], [[[3.0, 4.0, 0.0], [7.0, -3.0, -6.0]], [[3.0, 4.0, 2.0], [-5.0, -7.0, 0.0]], [[-4.0, -4.0, 1.0], [2.0, 1.0, -6.0]], [[4.0, -1.0, -6.0], [3.0, 5.0, -5.0]], [[-7.0, 2.0, -6.0], [6.0, 5.0, -4.0]]], [[[-1.0, 6.0, -7.0], [-2.0, 0.0, -1.0]], [[4.0, -6.0, -7.0], [-5.0, -6.0, -3.0]], [[2.0, -6.0, -3.0], [0.0, 7.0, -4.0]], [[-3.0, 0.0, 7.0], [-7.0, -4.0, 3.0]], [[3.0, -5.0, -3.0], [6.0, 0.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[23.0, -52.0, 86.0, -18.0, -82.0], [15.0, -66.0, 23.0, -71.0, 59.0], [86.0, 54.0, -15.0, -5.0, -16.0], [15.0, 16.0, -59.0, -15.0, 5.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_array_rank3_offset0(self):
      arg0=Data(-4.0,self.functionspace)
      arg1=numpy.array([[[3.0, -6.0], [0.0, 3.0]], [[6.0, -7.0], [-2.0, -6.0]], [[-4.0, -2.0], [-4.0, -4.0]], [[-4.0, 0.0], [6.0, 4.0]], [[3.0, 7.0], [0.0, 0.0]], [[1.0, -3.0], [3.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-12.0, 24.0], [-0.0, -12.0]], [[-24.0, 28.0], [8.0, 24.0]], [[16.0, 8.0], [16.0, 16.0]], [[16.0, -0.0], [-24.0, -16.0]], [[-12.0, -28.0], [-0.0, -0.0]], [[-4.0, 12.0], [-12.0, -16.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank4_offset1(self):
      arg0=Data(numpy.array([4.0, -6.0, 0.0]),self.functionspace)
      arg1=numpy.array([[[[5.0, -3.0, -2.0], [5.0, -2.0, -2.0]], [[-2.0, -2.0, -6.0], [-2.0, 4.0, 6.0]]], [[[-3.0, -1.0, -4.0], [-1.0, 4.0, 5.0]], [[-1.0, -6.0, -7.0], [6.0, -1.0, 0.0]]], [[[-7.0, -4.0, -2.0], [0.0, -1.0, -6.0]], [[1.0, -5.0, 2.0], [-1.0, -6.0, -3.0]]], [[[6.0, 5.0, 4.0], [-6.0, 0.0, 3.0]], [[-3.0, 7.0, -3.0], [2.0, 0.0, 3.0]]], [[[-2.0, -3.0, -3.0], [-3.0, -1.0, 5.0]], [[-7.0, -3.0, -2.0], [-3.0, -7.0, -3.0]]], [[[0.0, 4.0, 5.0], [5.0, -2.0, 0.0]], [[7.0, -1.0, 7.0], [-3.0, -6.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[38.0, 32.0], [4.0, -32.0]], [[-6.0, -28.0], [32.0, 30.0]], [[-4.0, 6.0], [34.0, 32.0]], [[-6.0, -24.0], [-54.0, 8.0]], [[10.0, -6.0], [-10.0, 30.0]], [[-24.0, 32.0], [34.0, 24.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_array_rank4_offset0(self):
      arg0=Data(-6.0,self.functionspace)
      arg1=numpy.array([[[[-1.0, 3.0, -3.0, 4.0], [-7.0, 6.0, 0.0, 7.0], [-4.0, -5.0, -7.0, 1.0]], [[4.0, -5.0, 6.0, 2.0], [1.0, -1.0, 6.0, 6.0], [-3.0, -3.0, 0.0, 6.0]]], [[[-2.0, -3.0, 1.0, 3.0], [-1.0, 0.0, -3.0, 0.0], [4.0, -3.0, 6.0, -1.0]], [[6.0, 0.0, -2.0, -1.0], [-4.0, 6.0, 3.0, -6.0], [-7.0, 7.0, 3.0, 3.0]]], [[[6.0, 1.0, 5.0, -4.0], [-4.0, 0.0, 7.0, 2.0], [-6.0, 2.0, -1.0, 1.0]], [[7.0, 0.0, -1.0, -2.0], [-4.0, 7.0, -4.0, 0.0], [6.0, -4.0, 6.0, -2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[6.0, -18.0, 18.0, -24.0], [42.0, -36.0, -0.0, -42.0], [24.0, 30.0, 42.0, -6.0]], [[-24.0, 30.0, -36.0, -12.0], [-6.0, 6.0, -36.0, -36.0], [18.0, 18.0, -0.0, -36.0]]], [[[12.0, 18.0, -6.0, -18.0], [6.0, -0.0, 18.0, -0.0], [-24.0, 18.0, -36.0, 6.0]], [[-36.0, -0.0, 12.0, 6.0], [24.0, -36.0, -18.0, 36.0], [42.0, -42.0, -18.0, -18.0]]], [[[-36.0, -6.0, -30.0, 24.0], [24.0, -0.0, -42.0, -12.0], [36.0, -12.0, 6.0, -6.0]], [[-42.0, -0.0, 6.0, 12.0], [24.0, -42.0, 24.0, -0.0], [-36.0, 24.0, -36.0, 12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_Symbol_rank0_offset0(self):
      arg0=Data(1.0,self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-2.0)
      sub=res.substitute({arg1:s1})
      ref=Data(-2.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([-3.0, 3.0, -6.0]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-4.0, -4.0, -4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(24.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[-4.0, 7.0, 2.0], [7.0, 4.0, 6.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-1.0, -2.0, 5.0], [0.0, 5.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(8.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[7.0, 0.0, 0.0], [7.0, 0.0, 2.0], [4.0, 0.0, 2.0], [7.0, 1.0, -5.0]], [[0.0, 5.0, 0.0], [0.0, 6.0, 4.0], [-7.0, 6.0, 0.0], [-3.0, 0.0, 6.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[2.0, -7.0, 0.0], [-3.0, -3.0, 5.0], [-5.0, 0.0, 6.0], [-4.0, 0.0, -7.0]], [[7.0, -4.0, 5.0], [-1.0, 3.0, -6.0], [-4.0, 0.0, -2.0], [-2.0, -4.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(16.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank4_offset4(self):
      arg0=Data(numpy.array([[[[3.0, -6.0], [7.0, -4.0], [7.0, -3.0]], [[0.0, -7.0], [4.0, -4.0], [3.0, 0.0]]], [[[3.0, 2.0], [0.0, 0.0], [3.0, 1.0]], [[-7.0, 0.0], [-1.0, 3.0], [-4.0, -3.0]]], [[[7.0, -7.0], [-2.0, -2.0], [5.0, 4.0]], [[1.0, -7.0], [-2.0, -1.0], [0.0, -4.0]]], [[[-7.0, 7.0], [5.0, -2.0], [2.0, 0.0]], [[6.0, -6.0], [-2.0, -4.0], [-6.0, -6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[0.0, -7.0], [3.0, -1.0], [0.0, -2.0]], [[-5.0, 7.0], [-1.0, 6.0], [-2.0, -1.0]]], [[[-3.0, -3.0], [4.0, -3.0], [7.0, 0.0]], [[-4.0, 4.0], [0.0, 3.0], [5.0, -3.0]]], [[[0.0, -7.0], [5.0, -2.0], [1.0, 4.0]], [[-1.0, 0.0], [-4.0, 6.0], [7.0, -1.0]]], [[[1.0, 1.0], [-5.0, -4.0], [-1.0, -1.0]], [[2.0, -2.0], [6.0, 0.0], [5.0, -7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(96.0,self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_Symbol_rank1_offset0(self):
      arg0=Data(3.0,self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([7.0, -5.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([21.0, -15.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([-4.0, 7.0, -1.0]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[5.0, 0.0, -5.0], [6.0, 3.0, 0.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-15.0, -3.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[2.0, -1.0, 3.0], [5.0, -4.0, -7.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[1.0, 4.0, 5.0], [0.0, 6.0, -2.0]], [[0.0, 2.0, -2.0], [1.0, -7.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([3.0, 25.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[7.0, 7.0, 4.0], [-1.0, 4.0, 6.0], [7.0, 7.0, -6.0], [4.0, 2.0, -2.0]], [[6.0, 5.0, 0.0], [-7.0, -5.0, 1.0], [0.0, 6.0, -4.0], [-7.0, 3.0, -4.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[0.0, -4.0, -2.0], [-1.0, 0.0, -6.0], [7.0, 0.0, 6.0], [0.0, -3.0, -4.0]], [[-5.0, -5.0, -5.0], [-4.0, -4.0, -1.0], [6.0, 4.0, -7.0], [-3.0, 1.0, 1.0]]], [[[3.0, 2.0, 0.0], [2.0, -7.0, -4.0], [0.0, 1.0, -3.0], [-7.0, 0.0, 0.0]], [[1.0, -4.0, 0.0], [-2.0, 4.0, -3.0], [-6.0, -3.0, 3.0], [5.0, -2.0, -3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([8.0, -104.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_Symbol_rank2_offset0(self):
      arg0=Data(0.0,self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[3.0, 2.0, 6.0, -6.0, 0.0], [-3.0, -7.0, 0.0, 0.0, 0.0], [0.0, 4.0, -5.0, -4.0, -3.0], [6.0, 1.0, -5.0, 4.0, -3.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[0.0, 0.0, 0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0, -0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([6.0, -5.0, -6.0]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-3.0, 7.0, 6.0], [0.0, 0.0, 1.0], [-2.0, -7.0, 2.0], [-2.0, 5.0, -7.0], [-3.0, 4.0, -3.0]], [[2.0, -3.0, 2.0], [6.0, 7.0, -1.0], [1.0, 6.0, -2.0], [-3.0, -1.0, -4.0], [-1.0, 6.0, -1.0]], [[6.0, 5.0, 4.0], [-4.0, 7.0, 6.0], [-6.0, -5.0, -5.0], [7.0, 2.0, 1.0], [1.0, 5.0, -3.0]], [[2.0, 0.0, -1.0], [-6.0, 0.0, 1.0], [-6.0, 6.0, -6.0], [5.0, 0.0, 5.0], [0.0, 7.0, -1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-89.0, -6.0, 11.0, 5.0, -20.0], [15.0, 7.0, -12.0, 11.0, -30.0], [-13.0, -95.0, 19.0, 26.0, -1.0], [18.0, -42.0, -30.0, 0.0, -29.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[-6.0, -4.0, 1.0], [2.0, 4.0, -6.0]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-4.0, 3.0, -6.0], [-3.0, -4.0, -5.0]], [[1.0, 4.0, 5.0], [0.0, -1.0, 0.0]], [[-2.0, 6.0, -1.0], [0.0, -4.0, -7.0]], [[2.0, 6.0, 0.0], [5.0, -4.0, 6.0]], [[5.0, -5.0, 4.0], [-4.0, -5.0, -1.0]]], [[[-5.0, -3.0, 1.0], [0.0, 4.0, 1.0]], [[0.0, 3.0, -5.0], [5.0, 4.0, -4.0]], [[1.0, 4.0, 5.0], [0.0, 2.0, -1.0]], [[-3.0, -6.0, 6.0], [6.0, -2.0, -3.0]], [[-5.0, -6.0, -7.0], [-7.0, 1.0, 0.0]]], [[[6.0, 4.0, -2.0], [-4.0, -5.0, -7.0]], [[7.0, 7.0, 2.0], [5.0, 5.0, 6.0]], [[1.0, 4.0, 1.0], [5.0, 0.0, -2.0]], [[0.0, 2.0, 6.0], [-4.0, -5.0, 1.0]], [[4.0, 7.0, 6.0], [-1.0, 3.0, 4.0]]], [[[-3.0, 2.0, 5.0], [7.0, 7.0, 0.0]], [[6.0, 2.0, -6.0], [6.0, -1.0, 0.0]], [[-3.0, 4.0, -5.0], [2.0, -1.0, -1.0]], [[4.0, 5.0, 3.0], [5.0, -7.0, -2.0]], [[-7.0, -6.0, -4.0], [-2.0, -5.0, -5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[14.0, -21.0, 13.0, -78.0, -28.0], [53.0, 33.0, -3.0, 70.0, 37.0], [-40.0, -74.0, 1.0, -36.0, -60.0], [57.0, -42.0, 3.0, -47.0, 68.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_Symbol_rank3_offset0(self):
      arg0=Data(4.0,self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-7.0, -5.0], [0.0, -5.0]], [[2.0, 4.0], [4.0, 3.0]], [[-3.0, -1.0], [1.0, 0.0]], [[2.0, -1.0], [-5.0, 6.0]], [[-6.0, 0.0], [-4.0, -3.0]], [[0.0, 0.0], [-3.0, -1.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-28.0, -20.0], [0.0, -20.0]], [[8.0, 16.0], [16.0, 12.0]], [[-12.0, -4.0], [4.0, 0.0]], [[8.0, -4.0], [-20.0, 24.0]], [[-24.0, 0.0], [-16.0, -12.0]], [[0.0, 0.0], [-12.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([-6.0, -6.0, 5.0]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[-3.0, 2.0, -1.0], [0.0, 0.0, 5.0]], [[-1.0, 3.0, -1.0], [-6.0, -7.0, -3.0]]], [[[-6.0, -2.0, 6.0], [-5.0, 5.0, -4.0]], [[-3.0, 0.0, -6.0], [-7.0, 0.0, 0.0]]], [[[3.0, -3.0, 5.0], [-1.0, 5.0, 5.0]], [[-2.0, 0.0, 5.0], [1.0, 7.0, -7.0]]], [[[-2.0, -2.0, 0.0], [3.0, 2.0, 0.0]], [[-4.0, -4.0, 2.0], [4.0, 5.0, -5.0]]], [[[-4.0, -5.0, -5.0], [-2.0, 4.0, 4.0]], [[4.0, -4.0, 0.0], [-5.0, 1.0, 4.0]]], [[[-6.0, -5.0, 0.0], [0.0, 2.0, -1.0]], [[-2.0, 1.0, 5.0], [4.0, -2.0, 5.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[1.0, 25.0], [-17.0, 63.0]], [[78.0, -20.0], [-12.0, 42.0]], [[25.0, 1.0], [37.0, -83.0]], [[24.0, -30.0], [58.0, -79.0]], [[29.0, 8.0], [0.0, 44.0]], [[66.0, -17.0], [31.0, 13.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_Symbol_rank4_offset0(self):
      arg0=Data(2.0,self.functionspace)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[6.0, 1.0, -6.0, 4.0], [-2.0, 0.0, -1.0, -1.0], [-1.0, 1.0, 5.0, 6.0]], [[-1.0, -5.0, 3.0, 7.0], [1.0, -4.0, 0.0, -6.0], [4.0, -5.0, 0.0, 7.0]]], [[[6.0, 0.0, -3.0, -2.0], [4.0, -3.0, 4.0, 0.0], [6.0, 3.0, -1.0, 1.0]], [[-2.0, -7.0, 2.0, 7.0], [1.0, -5.0, 3.0, 7.0], [7.0, 4.0, 6.0, 0.0]]], [[[-3.0, 4.0, 2.0, -6.0], [6.0, -4.0, -4.0, -2.0], [-7.0, 6.0, 0.0, -3.0]], [[6.0, -2.0, -1.0, 5.0], [2.0, 5.0, 0.0, 3.0], [-1.0, -5.0, 3.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[12.0, 2.0, -12.0, 8.0], [-4.0, 0.0, -2.0, -2.0], [-2.0, 2.0, 10.0, 12.0]], [[-2.0, -10.0, 6.0, 14.0], [2.0, -8.0, 0.0, -12.0], [8.0, -10.0, 0.0, 14.0]]], [[[12.0, 0.0, -6.0, -4.0], [8.0, -6.0, 8.0, 0.0], [12.0, 6.0, -2.0, 2.0]], [[-4.0, -14.0, 4.0, 14.0], [2.0, -10.0, 6.0, 14.0], [14.0, 8.0, 12.0, 0.0]]], [[[-6.0, 8.0, 4.0, -12.0], [12.0, -8.0, -8.0, -4.0], [-14.0, 12.0, 0.0, -6.0]], [[12.0, -4.0, -2.0, 10.0], [4.0, 10.0, 0.0, 6.0], [-2.0, -10.0, 6.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_constData_rank0_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      arg1=Data(0.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(-0.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank1_offset1(self):
      arg0=Data(numpy.array([0.0, 6.0, -7.0]),self.functionspace)
      arg1=Data(numpy.array([-6.0, 6.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(36.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[0.0, -6.0, 3.0], [0.0, 0.0, 2.0]]),self.functionspace)
      arg1=Data(numpy.array([[-3.0, 7.0, -5.0], [-4.0, 4.0, 6.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(-45.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[4.0, 7.0, 0.0], [-7.0, 7.0, -5.0], [-5.0, 0.0, -5.0], [3.0, 6.0, 2.0]], [[5.0, -6.0, 5.0], [-1.0, 6.0, 2.0], [-4.0, 5.0, 6.0], [0.0, 4.0, 6.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[4.0, -5.0, 1.0], [4.0, 4.0, -2.0], [-4.0, -6.0, 6.0], [-3.0, 4.0, 4.0]], [[4.0, 5.0, 0.0], [5.0, 4.0, 2.0], [-3.0, -2.0, 7.0], [6.0, 5.0, -1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(75.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[3.0, -7.0], [-3.0, -3.0], [1.0, -5.0]], [[2.0, 6.0], [7.0, -5.0], [-3.0, -5.0]]], [[[-6.0, -2.0], [-2.0, -1.0], [-7.0, 1.0]], [[4.0, 0.0], [2.0, 0.0], [-7.0, -3.0]]], [[[-2.0, 0.0], [-2.0, 3.0], [0.0, 7.0]], [[-4.0, 5.0], [6.0, -3.0], [-6.0, 4.0]]], [[[3.0, 0.0], [-2.0, -7.0], [4.0, -2.0]], [[0.0, 7.0], [-7.0, 6.0], [2.0, 7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-2.0, 1.0], [-2.0, 6.0], [-3.0, 6.0]], [[-2.0, 4.0], [4.0, -4.0], [3.0, 3.0]]], [[[-4.0, -7.0], [-2.0, 5.0], [7.0, 0.0]], [[1.0, -3.0], [7.0, -4.0], [3.0, 6.0]]], [[[-7.0, -7.0], [-6.0, 5.0], [0.0, 6.0]], [[-2.0, 0.0], [0.0, -5.0], [0.0, 0.0]]], [[[-1.0, -5.0], [1.0, -6.0], [-6.0, -5.0]], [[7.0, 3.0], [7.0, 4.0], [1.0, 4.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      ref=Data(108.0,self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_constData_rank1_offset0(self):
      arg0=Data(-7.0,self.functionspace)
      arg1=Data(numpy.array([2.0, 2.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-14.0, -14.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank2_offset1(self):
      arg0=Data(numpy.array([3.0, -2.0, 4.0]),self.functionspace)
      arg1=Data(numpy.array([[-3.0, -2.0, 1.0], [-6.0, -5.0, -1.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-1.0, -12.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[1.0, -7.0, 0.0], [-2.0, -5.0, 4.0]]),self.functionspace)
      arg1=Data(numpy.array([[[-7.0, 3.0, 1.0], [-2.0, 2.0, -4.0]], [[-5.0, 6.0, 4.0], [0.0, 4.0, -5.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-50.0, -87.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[2.0, -4.0, -7.0], [6.0, -3.0, 2.0], [-4.0, 1.0, -7.0], [1.0, 0.0, 1.0]], [[1.0, 0.0, 6.0], [-5.0, -5.0, -5.0], [0.0, -3.0, 3.0], [5.0, -3.0, 4.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[1.0, 0.0, 7.0], [6.0, -7.0, -6.0], [-2.0, -3.0, -2.0], [-2.0, -5.0, 0.0]], [[6.0, -1.0, 5.0], [-2.0, -7.0, 0.0], [-1.0, 5.0, -4.0], [5.0, -3.0, 2.0]]], [[[-6.0, -6.0, -2.0], [6.0, 1.0, -1.0], [0.0, -5.0, 4.0], [6.0, -7.0, 0.0]], [[-4.0, 6.0, -7.0], [6.0, -5.0, 2.0], [5.0, -2.0, -5.0], [4.0, -2.0, 4.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([111.0, 2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_constData_rank2_offset0(self):
      arg0=Data(-5.0,self.functionspace)
      arg1=Data(numpy.array([[7.0, -7.0, 5.0, -6.0, 0.0], [4.0, 0.0, -6.0, 7.0, 0.0], [-1.0, 0.0, 6.0, -5.0, 0.0], [1.0, -1.0, 5.0, -2.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-35.0, 35.0, -25.0, 30.0, -0.0], [-20.0, -0.0, 30.0, -35.0, -0.0], [5.0, -0.0, -30.0, 25.0, -0.0], [-5.0, 5.0, -25.0, 10.0, -0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank3_offset1(self):
      arg0=Data(numpy.array([7.0, -7.0, 2.0]),self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 2.0, -2.0], [-7.0, 5.0, 0.0], [-2.0, 4.0, -7.0], [1.0, 7.0, 1.0], [1.0, 0.0, 0.0]], [[5.0, -5.0, 3.0], [4.0, 3.0, 1.0], [-6.0, -3.0, -5.0], [-2.0, -1.0, 3.0], [-7.0, 0.0, 2.0]], [[0.0, -1.0, -3.0], [6.0, -4.0, 2.0], [6.0, -5.0, 6.0], [-5.0, 4.0, 0.0], [-3.0, 3.0, 4.0]], [[-3.0, 1.0, 2.0], [2.0, -2.0, -3.0], [-3.0, -4.0, 5.0], [-4.0, -2.0, 4.0], [-2.0, 1.0, -1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-32.0, -84.0, -56.0, -40.0, 7.0], [76.0, 9.0, -31.0, -1.0, -45.0], [1.0, 74.0, 89.0, -63.0, -34.0], [-24.0, 22.0, 17.0, -6.0, -23.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[6.0, -1.0, -5.0], [0.0, 3.0, 0.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[-4.0, -4.0, -4.0], [-5.0, 0.0, 6.0]], [[5.0, 6.0, -3.0], [-2.0, -3.0, 0.0]], [[-1.0, -3.0, -1.0], [0.0, -5.0, -2.0]], [[4.0, -4.0, 6.0], [7.0, 4.0, 6.0]], [[-2.0, 0.0, -4.0], [0.0, -6.0, 7.0]]], [[[-7.0, 2.0, 7.0], [6.0, -5.0, -6.0]], [[3.0, 0.0, 0.0], [-2.0, 2.0, -1.0]], [[-5.0, -3.0, -4.0], [6.0, -4.0, -5.0]], [[0.0, -4.0, 7.0], [7.0, 1.0, 0.0]], [[7.0, 6.0, 6.0], [1.0, -4.0, 0.0]]], [[[-5.0, -5.0, -5.0], [0.0, 5.0, -1.0]], [[1.0, -1.0, 2.0], [2.0, -2.0, 3.0]], [[2.0, 2.0, 4.0], [6.0, 2.0, 5.0]], [[-3.0, -6.0, 0.0], [-2.0, -4.0, 4.0]], [[0.0, 0.0, 6.0], [5.0, -5.0, 2.0]]], [[[0.0, 3.0, -4.0], [-1.0, 7.0, 0.0]], [[4.0, -5.0, 5.0], [6.0, -2.0, 3.0]], [[3.0, -4.0, 6.0], [4.0, -1.0, 4.0]], [[-2.0, -7.0, 5.0], [-1.0, -1.0, 0.0]], [[4.0, -5.0, -1.0], [3.0, -3.0, -5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[0.0, 30.0, -13.0, 10.0, -10.0], [-94.0, 24.0, -19.0, -28.0, -6.0], [15.0, -9.0, -4.0, -24.0, -45.0], [38.0, -2.0, -11.0, -33.0, 25.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_constData_rank3_offset0(self):
      arg0=Data(-1.0,self.functionspace)
      arg1=Data(numpy.array([[[-4.0, 1.0], [0.0, 0.0]], [[-5.0, -6.0], [5.0, 6.0]], [[-2.0, 7.0], [-2.0, 1.0]], [[1.0, -7.0], [0.0, -4.0]], [[0.0, -1.0], [5.0, 1.0]], [[0.0, -6.0], [-5.0, 3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[4.0, -1.0], [-0.0, -0.0]], [[5.0, 6.0], [-5.0, -6.0]], [[2.0, -7.0], [2.0, -1.0]], [[-1.0, 7.0], [-0.0, 4.0]], [[-0.0, 1.0], [-5.0, -1.0]], [[-0.0, 6.0], [5.0, -3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank4_offset1(self):
      arg0=Data(numpy.array([-6.0, 7.0, 0.0]),self.functionspace)
      arg1=Data(numpy.array([[[[3.0, -2.0, -3.0], [-4.0, -2.0, -5.0]], [[3.0, 6.0, 6.0], [-7.0, 4.0, -5.0]]], [[[1.0, 6.0, 0.0], [-4.0, 0.0, -5.0]], [[-5.0, -1.0, -7.0], [5.0, -5.0, -1.0]]], [[[7.0, 1.0, 4.0], [0.0, -4.0, 3.0]], [[-7.0, -2.0, -2.0], [-7.0, -2.0, -7.0]]], [[[0.0, -3.0, 7.0], [6.0, -1.0, 3.0]], [[7.0, -7.0, -3.0], [-6.0, 1.0, 6.0]]], [[[7.0, 2.0, 6.0], [3.0, 2.0, -3.0]], [[5.0, 7.0, -5.0], [7.0, -2.0, -4.0]]], [[[6.0, 2.0, 3.0], [6.0, 2.0, 3.0]], [[6.0, 1.0, 4.0], [-6.0, 0.0, 5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-32.0, 10.0], [24.0, 70.0]], [[36.0, 24.0], [23.0, -65.0]], [[-35.0, -28.0], [28.0, 28.0]], [[-21.0, -43.0], [-91.0, 43.0]], [[-28.0, -4.0], [19.0, -56.0]], [[-22.0, -22.0], [-29.0, 36.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_constData_rank4_offset0(self):
      arg0=Data(-4.0,self.functionspace)
      arg1=Data(numpy.array([[[[-6.0, 0.0, -5.0, 3.0], [-1.0, 0.0, 5.0, 2.0], [-1.0, -2.0, 5.0, 1.0]], [[0.0, -4.0, -2.0, 2.0], [-5.0, -1.0, 6.0, 0.0], [0.0, -4.0, 5.0, 4.0]]], [[[2.0, 1.0, -3.0, -6.0], [7.0, 1.0, -5.0, 7.0], [4.0, 7.0, 1.0, 4.0]], [[6.0, -3.0, 0.0, -3.0], [5.0, -3.0, 0.0, 0.0], [-7.0, 7.0, -5.0, 2.0]]], [[[0.0, 0.0, 1.0, -1.0], [-3.0, 7.0, 5.0, -3.0], [0.0, 1.0, -2.0, 3.0]], [[-6.0, 6.0, -4.0, -6.0], [-1.0, -1.0, -1.0, -4.0], [-2.0, -7.0, -4.0, -2.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[24.0, -0.0, 20.0, -12.0], [4.0, -0.0, -20.0, -8.0], [4.0, 8.0, -20.0, -4.0]], [[-0.0, 16.0, 8.0, -8.0], [20.0, 4.0, -24.0, -0.0], [-0.0, 16.0, -20.0, -16.0]]], [[[-8.0, -4.0, 12.0, 24.0], [-28.0, -4.0, 20.0, -28.0], [-16.0, -28.0, -4.0, -16.0]], [[-24.0, 12.0, -0.0, 12.0], [-20.0, 12.0, -0.0, -0.0], [28.0, -28.0, 20.0, -8.0]]], [[[-0.0, -0.0, -4.0, 4.0], [12.0, -28.0, -20.0, 12.0], [-0.0, -4.0, 8.0, -12.0]], [[24.0, -24.0, 16.0, 24.0], [4.0, 4.0, 4.0, 16.0], [8.0, 28.0, 16.0, 8.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_expandedData_rank0_offset0(self):
      arg0=Data(5.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-6.0)+(1-msk_arg1)*(-5.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-30.0)+(1.-msk_ref)*numpy.array(-25.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([4.0, 6.0, 4.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 5.0, 2.0])+(1.-msk_arg1)*numpy.array([1.0, -3.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(66.0)+(1.-msk_ref)*numpy.array(-10.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[-4.0, 0.0, -4.0], [7.0, 2.0, -5.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 2.0, 5.0], [0.0, 3.0, 3.0]])+(1.-msk_arg1)*numpy.array([[5.0, -5.0, -2.0], [7.0, -3.0, -6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-9.0)+(1.-msk_ref)*numpy.array(61.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[4.0, 7.0, -6.0], [-1.0, -5.0, 0.0], [0.0, 1.0, 1.0], [0.0, 4.0, 4.0]], [[-5.0, 0.0, -5.0], [1.0, 4.0, 5.0], [-4.0, 0.0, 4.0], [-3.0, 5.0, 0.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-6.0, -3.0, -3.0], [-3.0, 4.0, 0.0], [4.0, 3.0, 5.0], [-3.0, 1.0, -4.0]], [[-1.0, 0.0, -7.0], [-6.0, -3.0, -3.0], [-4.0, -5.0, 3.0], [6.0, 1.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[0.0, 6.0, 6.0], [-1.0, 3.0, 2.0], [3.0, 0.0, 2.0], [0.0, -4.0, 4.0]], [[4.0, -4.0, 6.0], [-7.0, -7.0, 5.0], [-2.0, 5.0, 7.0], [7.0, -4.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-26.0)+(1.-msk_ref)*numpy.array(-71.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank4_offset4(self):
      arg0=Data(numpy.array([[[[0.0, 1.0], [0.0, 1.0], [-6.0, 6.0]], [[1.0, -3.0], [3.0, -1.0], [6.0, -5.0]]], [[[4.0, 6.0], [3.0, -3.0], [5.0, -3.0]], [[3.0, 0.0], [1.0, 1.0], [-1.0, 0.0]]], [[[-7.0, -3.0], [4.0, -4.0], [7.0, 0.0]], [[0.0, -3.0], [-6.0, -1.0], [2.0, -4.0]]], [[[7.0, -5.0], [6.0, -4.0], [-1.0, 0.0]], [[-7.0, -7.0], [-3.0, 4.0], [3.0, -3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[1.0, -2.0], [1.0, 7.0], [0.0, 5.0]], [[0.0, -2.0], [3.0, 6.0], [2.0, -3.0]]], [[[-2.0, 1.0], [7.0, -7.0], [-7.0, 4.0]], [[6.0, 2.0], [-1.0, -1.0], [2.0, -7.0]]], [[[3.0, 6.0], [-2.0, -7.0], [5.0, 0.0]], [[0.0, 4.0], [4.0, 7.0], [-1.0, -3.0]]], [[[5.0, 2.0], [2.0, 1.0], [4.0, -7.0]], [[-7.0, -4.0], [6.0, 5.0], [-1.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 4.0], [-2.0, -2.0], [0.0, 6.0]], [[-5.0, 0.0], [7.0, -6.0], [2.0, 5.0]]], [[[-6.0, 1.0], [-7.0, -3.0], [-7.0, -1.0]], [[6.0, 0.0], [6.0, -4.0], [-5.0, 3.0]]], [[[-3.0, 4.0], [5.0, 5.0], [1.0, 0.0]], [[1.0, -7.0], [-5.0, -4.0], [-3.0, -2.0]]], [[[-6.0, -6.0], [-2.0, 0.0], [-7.0, -1.0]], [[-7.0, -3.0], [-7.0, -7.0], [5.0, -7.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(166.0)+(1.-msk_ref)*numpy.array(165.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_expandedData_rank1_offset0(self):
      arg0=Data(-3.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 1.0])+(1.-msk_arg1)*numpy.array([-6.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, -3.0])+(1.-msk_ref)*numpy.array([18.0, -0.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([-3.0, -1.0, 4.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 5.0, 4.0], [-2.0, -5.0, 4.0]])+(1.-msk_arg1)*numpy.array([[7.0, 0.0, -4.0], [2.0, -5.0, -6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([11.0, 27.0])+(1.-msk_ref)*numpy.array([-37.0, -25.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[-5.0, -1.0, -4.0], [-6.0, -6.0, -1.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, -5.0, 6.0], [5.0, 6.0, -7.0]], [[3.0, 4.0, 7.0], [-3.0, 3.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, -2.0, -3.0], [3.0, -1.0, -4.0]], [[-2.0, -4.0, 2.0], [7.0, 1.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-78.0, -54.0])+(1.-msk_ref)*numpy.array([-4.0, -42.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[1.0, 0.0, 5.0], [3.0, 1.0, -1.0], [-7.0, -4.0, -3.0], [4.0, -4.0, 3.0]], [[-5.0, 5.0, 5.0], [-2.0, 0.0, -1.0], [-5.0, -7.0, -7.0], [-7.0, -7.0, 1.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, -2.0, -2.0], [7.0, 0.0, -6.0], [5.0, -4.0, -2.0], [-2.0, 2.0, -5.0]], [[-2.0, 4.0, 5.0], [7.0, 0.0, -3.0], [0.0, -6.0, 0.0], [-2.0, 0.0, 3.0]]], [[[4.0, 0.0, -6.0], [-1.0, 3.0, 0.0], [-2.0, -5.0, 0.0], [-3.0, -3.0, 0.0]], [[3.0, 0.0, -7.0], [-7.0, -2.0, -7.0], [-6.0, -3.0, 2.0], [-5.0, 5.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -4.0, -4.0], [4.0, -3.0, 5.0], [7.0, 3.0, -6.0], [-7.0, 5.0, 5.0]], [[6.0, 1.0, 3.0], [1.0, -4.0, 7.0], [3.0, 0.0, -6.0], [4.0, -5.0, 4.0]]], [[[-4.0, 7.0, -6.0], [6.0, 4.0, -1.0], [-4.0, 1.0, 4.0], [4.0, 7.0, 5.0]], [[5.0, 0.0, -5.0], [-3.0, 7.0, -5.0], [4.0, 6.0, -2.0], [0.0, 6.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([71.0, 16.0])+(1.-msk_ref)*numpy.array([-73.0, -129.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_expandedData_rank2_offset0(self):
      arg0=Data(1.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, -1.0, 0.0, -5.0, -3.0], [-2.0, 0.0, 4.0, -5.0, -7.0], [5.0, -3.0, 0.0, 4.0, -3.0], [0.0, -1.0, -3.0, -7.0, 4.0]])+(1.-msk_arg1)*numpy.array([[3.0, -7.0, 3.0, -6.0, -4.0], [2.0, -6.0, 5.0, -1.0, 0.0], [-1.0, 0.0, -1.0, -5.0, -6.0], [-2.0, -1.0, -1.0, -5.0, 1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-7.0, -1.0, 0.0, -5.0, -3.0], [-2.0, 0.0, 4.0, -5.0, -7.0], [5.0, -3.0, 0.0, 4.0, -3.0], [0.0, -1.0, -3.0, -7.0, 4.0]])+(1.-msk_ref)*numpy.array([[3.0, -7.0, 3.0, -6.0, -4.0], [2.0, -6.0, 5.0, -1.0, 0.0], [-1.0, 0.0, -1.0, -5.0, -6.0], [-2.0, -1.0, -1.0, -5.0, 1.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([3.0, -1.0, 5.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, 3.0, 2.0], [2.0, -1.0, 0.0], [2.0, -5.0, -4.0], [0.0, -2.0, -4.0], [2.0, -6.0, 0.0]], [[-6.0, -5.0, -1.0], [-2.0, -3.0, -7.0], [5.0, 0.0, 0.0], [3.0, 7.0, 2.0], [5.0, -1.0, 5.0]], [[-1.0, 3.0, -2.0], [2.0, 0.0, -6.0], [-1.0, -5.0, 1.0], [6.0, 7.0, -3.0], [6.0, -6.0, 6.0]], [[4.0, 5.0, 3.0], [1.0, -2.0, 4.0], [-4.0, -2.0, 6.0], [4.0, 1.0, 1.0], [3.0, -3.0, -4.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 4.0, 0.0], [7.0, -1.0, -3.0], [3.0, -4.0, -7.0], [-5.0, 2.0, 6.0], [4.0, -3.0, -6.0]], [[1.0, -7.0, 0.0], [4.0, 2.0, 0.0], [4.0, -3.0, 5.0], [7.0, -7.0, 2.0], [4.0, -4.0, -7.0]], [[-6.0, 7.0, 6.0], [4.0, 0.0, -6.0], [0.0, -5.0, -7.0], [-5.0, 0.0, 5.0], [1.0, 6.0, -2.0]], [[-4.0, 5.0, -1.0], [-6.0, 6.0, 4.0], [-7.0, 4.0, -5.0], [0.0, 6.0, 7.0], [0.0, -7.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[25.0, 7.0, -9.0, -18.0, 12.0], [-18.0, -38.0, 15.0, 12.0, 41.0], [-16.0, -24.0, 7.0, -4.0, 54.0], [22.0, 25.0, 20.0, 16.0, -8.0]])+(1.-msk_ref)*numpy.array([[5.0, 7.0, -22.0, 13.0, -15.0], [10.0, 10.0, 40.0, 38.0, -19.0], [5.0, -18.0, -30.0, 10.0, -13.0], [-22.0, -4.0, -50.0, 29.0, -28.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[3.0, 5.0, -7.0], [-3.0, -4.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, -4.0, -1.0], [-7.0, -6.0, -6.0]], [[0.0, 5.0, -7.0], [5.0, 2.0, 1.0]], [[-1.0, 1.0, -3.0], [-1.0, 7.0, -4.0]], [[-6.0, 6.0, -4.0], [-7.0, 0.0, 4.0]], [[3.0, 3.0, 0.0], [-4.0, 0.0, -4.0]]], [[[-2.0, 0.0, 5.0], [0.0, 7.0, 2.0]], [[-4.0, -3.0, 0.0], [-6.0, 0.0, -6.0]], [[-4.0, 7.0, 7.0], [7.0, -5.0, -4.0]], [[-5.0, 5.0, 7.0], [-2.0, -1.0, -1.0]], [[6.0, 4.0, -6.0], [0.0, 7.0, -2.0]]], [[[0.0, -5.0, 5.0], [-2.0, 0.0, 6.0]], [[2.0, -2.0, -2.0], [2.0, 2.0, 0.0]], [[-4.0, 7.0, 0.0], [-3.0, -2.0, 0.0]], [[5.0, 2.0, -2.0], [-6.0, -3.0, 5.0]], [[-4.0, -7.0, 2.0], [3.0, 6.0, 3.0]]], [[[0.0, 1.0, 2.0], [1.0, -2.0, -2.0]], [[1.0, -1.0, -5.0], [-1.0, 5.0, -3.0]], [[3.0, 0.0, 0.0], [5.0, 2.0, -2.0]], [[1.0, 4.0, 0.0], [7.0, 2.0, -6.0]], [[3.0, -4.0, -5.0], [-4.0, 7.0, -5.0]]]])+(1.-msk_arg1)*numpy.array([[[[3.0, 5.0, -6.0], [7.0, -3.0, 3.0]], [[6.0, -6.0, -3.0], [0.0, 3.0, -3.0]], [[-1.0, -3.0, -1.0], [-7.0, -6.0, 7.0]], [[-4.0, -1.0, -3.0], [0.0, -2.0, -4.0]], [[5.0, -5.0, 2.0], [2.0, -5.0, -3.0]]], [[[-2.0, 3.0, -2.0], [6.0, 4.0, -3.0]], [[4.0, 3.0, 1.0], [6.0, -3.0, 2.0]], [[7.0, -3.0, 7.0], [-1.0, 0.0, -5.0]], [[-6.0, -6.0, -3.0], [4.0, -2.0, 6.0]], [[1.0, -2.0, 2.0], [5.0, -1.0, 0.0]]], [[[-3.0, 1.0, 0.0], [-5.0, 0.0, 6.0]], [[6.0, 0.0, 1.0], [-2.0, 5.0, 5.0]], [[7.0, -1.0, 7.0], [-1.0, 5.0, 2.0]], [[5.0, 6.0, -3.0], [-4.0, -5.0, 0.0]], [[-4.0, 3.0, -2.0], [-6.0, 1.0, -7.0]]], [[[0.0, 5.0, 7.0], [0.0, 6.0, 2.0]], [[1.0, -4.0, 5.0], [3.0, -1.0, 0.0]], [[2.0, 3.0, 2.0], [-2.0, 0.0, 6.0]], [[5.0, 0.0, 5.0], [3.0, -3.0, 0.0]], [[4.0, 0.0, 5.0], [1.0, 5.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[47.0, 51.0, -2.0, 61.0, 36.0], [-69.0, -9.0, -27.0, -29.0, 52.0], [-54.0, -4.0, 40.0, 69.0, -94.0], [-4.0, 16.0, -14.0, -6.0, 8.0]])+(1.-msk_ref)*numpy.array([[67.0, -3.0, 34.0, 12.0, -10.0], [-11.0, 14.0, -40.0, -31.0, -32.0], [11.0, -3.0, -50.0, 98.0, 31.0], [-48.0, -57.0, 13.0, -17.0, -46.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_expandedData_rank3_offset0(self):
      arg0=Data(1.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-4.0, -1.0], [0.0, 7.0]], [[-2.0, 0.0], [-7.0, 3.0]], [[5.0, -3.0], [2.0, -3.0]], [[0.0, 2.0], [0.0, 6.0]], [[6.0, 0.0], [7.0, 1.0]], [[-3.0, -7.0], [-4.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, 6.0], [-4.0, -7.0]], [[0.0, -3.0], [6.0, 5.0]], [[-3.0, -7.0], [-4.0, -5.0]], [[0.0, 6.0], [4.0, 3.0]], [[6.0, 6.0], [-7.0, 3.0]], [[7.0, -6.0], [-2.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-4.0, -1.0], [0.0, 7.0]], [[-2.0, 0.0], [-7.0, 3.0]], [[5.0, -3.0], [2.0, -3.0]], [[0.0, 2.0], [0.0, 6.0]], [[6.0, 0.0], [7.0, 1.0]], [[-3.0, -7.0], [-4.0, -3.0]]])+(1.-msk_ref)*numpy.array([[[-5.0, 6.0], [-4.0, -7.0]], [[0.0, -3.0], [6.0, 5.0]], [[-3.0, -7.0], [-4.0, -5.0]], [[0.0, 6.0], [4.0, 3.0]], [[6.0, 6.0], [-7.0, 3.0]], [[7.0, -6.0], [-2.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([0.0, 2.0, -6.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, -4.0, 0.0], [-2.0, 7.0, 0.0]], [[4.0, -2.0, -4.0], [-4.0, 1.0, -7.0]]], [[[5.0, -4.0, -1.0], [-6.0, -4.0, -4.0]], [[-1.0, 5.0, -5.0], [-7.0, 7.0, -3.0]]], [[[6.0, -4.0, 0.0], [-1.0, 1.0, 7.0]], [[-3.0, -2.0, 6.0], [2.0, 5.0, -2.0]]], [[[0.0, -6.0, 5.0], [-3.0, 0.0, 0.0]], [[2.0, 3.0, 6.0], [0.0, 5.0, 3.0]]], [[[5.0, 4.0, 0.0], [-2.0, -2.0, -6.0]], [[0.0, 1.0, -7.0], [2.0, -3.0, -5.0]]], [[[0.0, -6.0, 5.0], [-6.0, 1.0, 7.0]], [[1.0, -4.0, 6.0], [3.0, -2.0, -6.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 0.0, -7.0], [0.0, 7.0, 2.0]], [[-2.0, 6.0, 3.0], [6.0, 0.0, 0.0]]], [[[3.0, 1.0, 0.0], [-2.0, 0.0, 6.0]], [[6.0, 1.0, -3.0], [4.0, 3.0, -4.0]]], [[[-2.0, -3.0, 2.0], [-7.0, 7.0, 0.0]], [[5.0, 1.0, -6.0], [1.0, 7.0, 4.0]]], [[[4.0, -6.0, -1.0], [-7.0, 0.0, 3.0]], [[-7.0, -6.0, 1.0], [6.0, -4.0, -5.0]]], [[[1.0, 6.0, 1.0], [-2.0, -1.0, -7.0]], [[-2.0, -7.0, -7.0], [-5.0, -1.0, -7.0]]], [[[-5.0, 7.0, 2.0], [4.0, -4.0, 4.0]], [[-7.0, 2.0, -5.0], [5.0, -2.0, -2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-8.0, 14.0], [20.0, 44.0]], [[-2.0, 16.0], [40.0, 32.0]], [[-8.0, -40.0], [-40.0, 22.0]], [[-42.0, 0.0], [-30.0, -8.0]], [[8.0, 32.0], [44.0, 24.0]], [[-42.0, -40.0], [-44.0, 32.0]]])+(1.-msk_ref)*numpy.array([[[42.0, 2.0], [-6.0, 0.0]], [[2.0, -36.0], [20.0, 30.0]], [[-18.0, 14.0], [38.0, -10.0]], [[-6.0, -18.0], [-18.0, 22.0]], [[6.0, 40.0], [28.0, 40.0]], [[2.0, -32.0], [34.0, 8.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank0_expandedData_rank4_offset0(self):
      arg0=Data(6.0,self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-5.0, 3.0, 5.0, 3.0], [1.0, 2.0, -2.0, 7.0], [1.0, -2.0, 0.0, 3.0]], [[-5.0, -2.0, -7.0, -7.0], [6.0, -4.0, 5.0, 2.0], [7.0, 5.0, 6.0, 1.0]]], [[[-6.0, -2.0, 7.0, 3.0], [2.0, 5.0, 0.0, 1.0], [4.0, -7.0, 5.0, 4.0]], [[0.0, 6.0, 7.0, 0.0], [-3.0, -5.0, -1.0, -5.0], [-4.0, -3.0, 2.0, -4.0]]], [[[4.0, -3.0, 6.0, 0.0], [-3.0, 2.0, -6.0, 0.0], [1.0, 1.0, 5.0, -2.0]], [[7.0, 2.0, 5.0, -2.0], [-5.0, -2.0, -7.0, 2.0], [-4.0, 0.0, 3.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[-5.0, 0.0, 0.0, 7.0], [-6.0, -4.0, 3.0, 4.0], [-5.0, 4.0, -6.0, 7.0]], [[5.0, -6.0, -2.0, 6.0], [-4.0, -1.0, -6.0, 2.0], [4.0, 6.0, 1.0, -3.0]]], [[[-5.0, 3.0, 5.0, 2.0], [-1.0, -4.0, 5.0, 5.0], [-7.0, 3.0, -4.0, 0.0]], [[-7.0, 1.0, -7.0, -5.0], [-4.0, 7.0, -2.0, 0.0], [5.0, 0.0, -5.0, -2.0]]], [[[-3.0, 2.0, -6.0, -2.0], [4.0, -3.0, 0.0, 7.0], [-4.0, 4.0, 5.0, -2.0]], [[0.0, -4.0, 7.0, 0.0], [3.0, 7.0, 5.0, 2.0], [6.0, -6.0, 6.0, 6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-30.0, 18.0, 30.0, 18.0], [6.0, 12.0, -12.0, 42.0], [6.0, -12.0, 0.0, 18.0]], [[-30.0, -12.0, -42.0, -42.0], [36.0, -24.0, 30.0, 12.0], [42.0, 30.0, 36.0, 6.0]]], [[[-36.0, -12.0, 42.0, 18.0], [12.0, 30.0, 0.0, 6.0], [24.0, -42.0, 30.0, 24.0]], [[0.0, 36.0, 42.0, 0.0], [-18.0, -30.0, -6.0, -30.0], [-24.0, -18.0, 12.0, -24.0]]], [[[24.0, -18.0, 36.0, 0.0], [-18.0, 12.0, -36.0, 0.0], [6.0, 6.0, 30.0, -12.0]], [[42.0, 12.0, 30.0, -12.0], [-30.0, -12.0, -42.0, 12.0], [-24.0, 0.0, 18.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-30.0, 0.0, 0.0, 42.0], [-36.0, -24.0, 18.0, 24.0], [-30.0, 24.0, -36.0, 42.0]], [[30.0, -36.0, -12.0, 36.0], [-24.0, -6.0, -36.0, 12.0], [24.0, 36.0, 6.0, -18.0]]], [[[-30.0, 18.0, 30.0, 12.0], [-6.0, -24.0, 30.0, 30.0], [-42.0, 18.0, -24.0, 0.0]], [[-42.0, 6.0, -42.0, -30.0], [-24.0, 42.0, -12.0, 0.0], [30.0, 0.0, -30.0, -12.0]]], [[[-18.0, 12.0, -36.0, -12.0], [24.0, -18.0, 0.0, 42.0], [-24.0, 24.0, 30.0, -12.0]], [[0.0, -24.0, 42.0, 0.0], [18.0, 42.0, 30.0, 12.0], [36.0, -36.0, 36.0, 36.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_float_rank0_offset0(self):
      arg0=Data(numpy.array([5.0, -3.0]),self.functionspace)
      arg1=-6.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-30.0, 18.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank0_offset0(self):
      arg0=Data(numpy.array([-4.0, 3.0]),self.functionspace)
      arg1=numpy.array(-1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([4.0, -3.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank1_offset1(self):
      arg0=Data(numpy.array([[-3.0, 0.0, -2.0], [6.0, 5.0, 4.0]]),self.functionspace)
      arg1=numpy.array([6.0, -4.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-18.0, 16.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[2.0, 2.0, 6.0], [3.0, 4.0, 5.0]], [[-3.0, 3.0, 1.0], [-6.0, 3.0, 6.0]]]),self.functionspace)
      arg1=numpy.array([[0.0, -3.0, -4.0], [-7.0, 3.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-74.0, -4.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank3_offset3(self):
      arg0=Data(numpy.array([[[[6.0, -6.0, -5.0], [0.0, -5.0, -4.0], [6.0, -2.0, 5.0], [-7.0, -3.0, -5.0]], [[0.0, -1.0, 0.0], [-2.0, 3.0, -5.0], [4.0, -5.0, 0.0], [2.0, 4.0, 0.0]]], [[[5.0, -6.0, -5.0], [2.0, -3.0, 2.0], [6.0, -3.0, -7.0], [0.0, -2.0, -6.0]], [[2.0, 2.0, 0.0], [-7.0, 2.0, 6.0], [-4.0, 5.0, 4.0], [-3.0, 7.0, 3.0]]]]),self.functionspace)
      arg1=numpy.array([[[-4.0, -1.0, -5.0], [-2.0, 7.0, 4.0], [7.0, -6.0, 0.0], [-2.0, 0.0, 6.0]], [[5.0, 7.0, -2.0], [7.0, 7.0, 2.0], [-3.0, 3.0, -5.0], [5.0, -5.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([-53.0, -24.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank1_offset0(self):
      arg0=Data(numpy.array([-4.0, 0.0]),self.functionspace)
      arg1=numpy.array([4.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-16.0, 0.0], [0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank2_offset1(self):
      arg0=Data(numpy.array([[4.0, -5.0, 6.0], [2.0, -2.0, 6.0]]),self.functionspace)
      arg1=numpy.array([[1.0, -3.0, -7.0], [0.0, -3.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-23.0, 57.0], [-34.0, 48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[-7.0, 3.0, 5.0], [-2.0, -7.0, 0.0]], [[4.0, 0.0, 4.0], [0.0, -5.0, -1.0]]]),self.functionspace)
      arg1=numpy.array([[[-5.0, -1.0, 0.0], [1.0, -1.0, -3.0]], [[4.0, -3.0, 2.0], [-3.0, -6.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[37.0, 21.0], [-12.0, 50.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank4_offset3(self):
      arg0=Data(numpy.array([[[[0.0, 7.0, 2.0], [3.0, 7.0, -5.0], [-7.0, 1.0, 0.0], [-7.0, 5.0, 5.0]], [[3.0, 5.0, -4.0], [-6.0, 5.0, 0.0], [0.0, 0.0, 3.0], [-1.0, 5.0, -4.0]]], [[[1.0, 1.0, 4.0], [5.0, -3.0, -5.0], [-1.0, 6.0, -4.0], [0.0, 5.0, 3.0]], [[4.0, 0.0, -7.0], [3.0, 6.0, -7.0], [-3.0, -7.0, 1.0], [5.0, -1.0, -3.0]]]]),self.functionspace)
      arg1=numpy.array([[[[5.0, 0.0, 6.0], [-6.0, -2.0, -6.0], [2.0, 1.0, 2.0], [-5.0, -4.0, 7.0]], [[2.0, -5.0, -2.0], [3.0, 0.0, -7.0], [-7.0, 2.0, -7.0], [2.0, 5.0, -6.0]]], [[[7.0, -3.0, -1.0], [5.0, 3.0, 7.0], [-1.0, 3.0, 1.0], [6.0, -4.0, -4.0]], [[7.0, 5.0, -1.0], [2.0, -3.0, 4.0], [-7.0, -2.0, -6.0], [-4.0, 2.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[44.0, -91.0], [135.0, -46.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank2_offset0(self):
      arg0=Data(numpy.array([-5.0, -2.0]),self.functionspace)
      arg1=numpy.array([[-2.0, -2.0, 0.0, 3.0, 6.0], [-3.0, 3.0, 4.0, -4.0, -3.0], [-2.0, -4.0, 0.0, -2.0, 5.0], [-3.0, 4.0, -4.0, 2.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[10.0, 10.0, 0.0, -15.0, -30.0], [15.0, -15.0, -20.0, 20.0, 15.0], [10.0, 20.0, 0.0, 10.0, -25.0], [15.0, -20.0, 20.0, -10.0, 15.0]], [[4.0, 4.0, 0.0, -6.0, -12.0], [6.0, -6.0, -8.0, 8.0, 6.0], [4.0, 8.0, 0.0, 4.0, -10.0], [6.0, -8.0, 8.0, -4.0, 6.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank3_offset1(self):
      arg0=Data(numpy.array([[2.0, -3.0, 6.0], [0.0, -6.0, -2.0]]),self.functionspace)
      arg1=numpy.array([[[-1.0, -2.0, 5.0], [6.0, 4.0, 7.0], [-5.0, 5.0, -1.0], [-6.0, 1.0, 3.0], [1.0, -1.0, 5.0]], [[0.0, -5.0, -4.0], [-3.0, 0.0, -3.0], [-3.0, -2.0, 5.0], [2.0, 3.0, 2.0], [3.0, -3.0, 1.0]], [[0.0, -1.0, 3.0], [7.0, -6.0, 3.0], [-7.0, 1.0, 1.0], [-7.0, 7.0, 4.0], [-1.0, 6.0, -6.0]], [[-4.0, -7.0, 0.0], [-3.0, 1.0, -2.0], [-1.0, 6.0, -1.0], [0.0, 2.0, -7.0], [-5.0, 6.0, 3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[34.0, 42.0, -31.0, 3.0, 35.0], [-9.0, -24.0, 30.0, 7.0, 21.0], [21.0, 50.0, -11.0, -11.0, -56.0], [13.0, -21.0, -26.0, -48.0, -10.0]], [[2.0, -38.0, -28.0, -12.0, -4.0], [38.0, 6.0, 2.0, -22.0, 16.0], [0.0, 30.0, -8.0, -50.0, -24.0], [42.0, -2.0, -34.0, 2.0, -42.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[2.0, -2.0, -4.0], [7.0, 0.0, 1.0]], [[-6.0, 2.0, 3.0], [-2.0, 4.0, 0.0]]]),self.functionspace)
      arg1=numpy.array([[[[2.0, 0.0, 3.0], [0.0, 0.0, -7.0]], [[7.0, -3.0, 5.0], [3.0, 2.0, -6.0]], [[3.0, 1.0, -7.0], [4.0, -3.0, -1.0]], [[-3.0, -5.0, -7.0], [4.0, -7.0, -7.0]], [[-2.0, 3.0, 0.0], [6.0, -4.0, 3.0]]], [[[0.0, 1.0, -2.0], [-7.0, -5.0, 3.0]], [[7.0, -6.0, 6.0], [5.0, -7.0, 6.0]], [[-5.0, -1.0, 1.0], [-2.0, 3.0, 7.0]], [[-7.0, -5.0, 0.0], [-2.0, -4.0, 7.0]], [[2.0, 0.0, 7.0], [-3.0, 0.0, 5.0]]], [[[-5.0, 0.0, 1.0], [2.0, 6.0, -2.0]], [[5.0, 4.0, 1.0], [6.0, 4.0, 5.0]], [[-6.0, 0.0, 0.0], [0.0, 7.0, -6.0]], [[-1.0, 5.0, 0.0], [-3.0, 3.0, -5.0]], [[-4.0, -6.0, 4.0], [1.0, 6.0, 2.0]]], [[[3.0, 1.0, 7.0], [0.0, -2.0, 1.0]], [[1.0, 2.0, 4.0], [3.0, 1.0, -3.0]], [[-2.0, 5.0, 5.0], [-7.0, -2.0, 6.0]], [[4.0, 0.0, -5.0], [0.0, -6.0, 2.0]], [[-1.0, -7.0, -4.0], [-4.0, -1.0, -6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-15.0, 15.0, 59.0, 53.0, 35.0], [-40.0, 43.0, -19.0, -11.0, -40.0], [-2.0, 45.0, -18.0, -38.0, -3.0], [-23.0, 0.0, -77.0, 30.0, -6.0]], [[-3.0, -31.0, -57.0, -49.0, -10.0], [-10.0, -74.0, 47.0, 20.0, 15.0], [53.0, -15.0, 64.0, 34.0, 46.0], [-3.0, 8.0, 43.0, -63.0, -16.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_array_rank3_offset0(self):
      arg0=Data(numpy.array([-3.0, 1.0]),self.functionspace)
      arg1=numpy.array([[[5.0, 0.0], [4.0, 2.0]], [[-3.0, 6.0], [-2.0, 3.0]], [[4.0, 2.0], [-2.0, 3.0]], [[1.0, 3.0], [7.0, -5.0]], [[6.0, 0.0], [4.0, -2.0]], [[4.0, -2.0], [0.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[-15.0, 0.0], [-12.0, -6.0]], [[9.0, -18.0], [6.0, -9.0]], [[-12.0, -6.0], [6.0, -9.0]], [[-3.0, -9.0], [-21.0, 15.0]], [[-18.0, 0.0], [-12.0, 6.0]], [[-12.0, 6.0], [0.0, 21.0]]], [[[5.0, 0.0], [4.0, 2.0]], [[-3.0, 6.0], [-2.0, 3.0]], [[4.0, 2.0], [-2.0, 3.0]], [[1.0, 3.0], [7.0, -5.0]], [[6.0, 0.0], [4.0, -2.0]], [[4.0, -2.0], [0.0, -7.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank4_offset1(self):
      arg0=Data(numpy.array([[-1.0, 0.0, 2.0], [-6.0, -3.0, 5.0]]),self.functionspace)
      arg1=numpy.array([[[[-5.0, 6.0, -1.0], [-6.0, 1.0, 4.0]], [[2.0, 3.0, -5.0], [2.0, -1.0, 0.0]]], [[[-5.0, 4.0, 5.0], [0.0, -4.0, 3.0]], [[-2.0, 0.0, -6.0], [6.0, 0.0, 5.0]]], [[[2.0, 0.0, 7.0], [-6.0, 4.0, -3.0]], [[0.0, 3.0, -3.0], [-5.0, 0.0, 7.0]]], [[[-1.0, -6.0, -4.0], [0.0, 0.0, -2.0]], [[5.0, -5.0, 3.0], [0.0, -2.0, 0.0]]], [[[-1.0, -6.0, 7.0], [7.0, 3.0, -6.0]], [[4.0, 4.0, 5.0], [7.0, 3.0, 3.0]]], [[[-2.0, 6.0, 0.0], [0.0, -6.0, 3.0]], [[1.0, -1.0, 7.0], [7.0, 2.0, -3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[3.0, 14.0], [-12.0, -2.0]], [[15.0, 6.0], [-10.0, 4.0]], [[12.0, 0.0], [-6.0, 19.0]], [[-7.0, -4.0], [1.0, 0.0]], [[15.0, -19.0], [6.0, -1.0]], [[2.0, 6.0], [13.0, -13.0]]], [[[7.0, 53.0], [-46.0, -9.0]], [[43.0, 27.0], [-18.0, -11.0]], [[23.0, 9.0], [-24.0, 65.0]], [[4.0, -10.0], [0.0, 6.0]], [[59.0, -81.0], [-11.0, -36.0]], [[-6.0, 33.0], [32.0, -63.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([7.0, -5.0]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(1.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([7.0, -5.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[-7.0, -2.0, -1.0], [-6.0, 3.0, -2.0]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-1.0, -5.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([13.0, -17.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[-6.0, -7.0, 6.0], [0.0, 4.0, 5.0]], [[4.0, 0.0, -2.0], [4.0, 0.0, -3.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[5.0, 0.0, -1.0], [-3.0, -5.0, -4.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-76.0, 22.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank3_offset3(self):
      arg0=Data(numpy.array([[[[-2.0, -5.0, 4.0], [-3.0, 2.0, -2.0], [-3.0, 0.0, -2.0], [-1.0, -7.0, 4.0]], [[-2.0, 0.0, 0.0], [5.0, -2.0, -6.0], [3.0, 0.0, 6.0], [0.0, -3.0, 4.0]]], [[[-2.0, -5.0, -1.0], [-5.0, 0.0, -4.0], [-4.0, 5.0, 3.0], [-1.0, 7.0, 0.0]], [[5.0, 2.0, -3.0], [4.0, 7.0, -3.0], [3.0, 5.0, 1.0], [0.0, 3.0, -4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[2.0, -1.0, -4.0], [-7.0, 3.0, -1.0], [6.0, -3.0, 0.0], [6.0, 0.0, -7.0]], [[0.0, 1.0, 1.0], [4.0, -3.0, 7.0], [-2.0, -3.0, -4.0], [4.0, -4.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([-60.0, -77.0]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([1.0, 0.0]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-5.0, 4.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-5.0, 4.0], [0.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[6.0, -6.0, 2.0], [-7.0, -6.0, -7.0]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[0.0, -6.0, 2.0], [7.0, 4.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[40.0, 32.0], [22.0, -122.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[7.0, -5.0, 0.0], [-1.0, -2.0, 0.0]], [[7.0, -7.0, 0.0], [7.0, 0.0, -4.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[4.0, -5.0, 0.0], [0.0, 2.0, -2.0]], [[6.0, 0.0, -3.0], [7.0, 1.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[49.0, 33.0], [71.0, 107.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank4_offset3(self):
      arg0=Data(numpy.array([[[[-7.0, 6.0, -6.0], [6.0, -7.0, 6.0], [-3.0, 0.0, -3.0], [-5.0, 0.0, 5.0]], [[3.0, 0.0, 7.0], [1.0, 4.0, 7.0], [-2.0, -3.0, -7.0], [0.0, -4.0, -4.0]]], [[[-4.0, -5.0, -2.0], [0.0, 1.0, 1.0], [2.0, -6.0, 2.0], [-5.0, 7.0, 3.0]], [[-2.0, 4.0, -6.0], [5.0, -4.0, 2.0], [2.0, 0.0, 0.0], [-3.0, 4.0, 4.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[5.0, 5.0, -5.0], [-6.0, 6.0, 7.0], [-4.0, -3.0, -2.0], [-2.0, -7.0, -4.0]], [[-5.0, -1.0, -4.0], [4.0, 3.0, 1.0], [-6.0, -2.0, 7.0], [3.0, 2.0, 0.0]]], [[[1.0, 3.0, 0.0], [2.0, 2.0, -1.0], [7.0, -3.0, -7.0], [4.0, 2.0, -6.0]], [[-1.0, 4.0, 6.0], [5.0, 2.0, -3.0], [0.0, -5.0, -2.0], [7.0, -7.0, 3.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-62.0, 29.0], [-40.0, -68.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([-5.0, 5.0]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-4.0, 1.0, 0.0, -1.0, 0.0], [-2.0, 3.0, 1.0, -4.0, 4.0], [6.0, 7.0, -6.0, 0.0, -7.0], [-7.0, 0.0, 2.0, -5.0, 6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[20.0, -5.0, 0.0, 5.0, 0.0], [10.0, -15.0, -5.0, 20.0, -20.0], [-30.0, -35.0, 30.0, 0.0, 35.0], [35.0, 0.0, -10.0, 25.0, -30.0]], [[-20.0, 5.0, 0.0, -5.0, 0.0], [-10.0, 15.0, 5.0, -20.0, 20.0], [30.0, 35.0, -30.0, 0.0, -35.0], [-35.0, 0.0, 10.0, -25.0, 30.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[3.0, -4.0, 4.0], [-3.0, 6.0, -4.0]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-5.0, 5.0, -2.0], [-5.0, -1.0, 7.0], [4.0, -1.0, 4.0], [4.0, 7.0, -2.0], [0.0, -6.0, -6.0]], [[3.0, 0.0, 0.0], [-6.0, 3.0, -3.0], [-7.0, 7.0, 1.0], [3.0, -2.0, 6.0], [6.0, 0.0, -2.0]], [[0.0, -2.0, -1.0], [2.0, 1.0, 7.0], [0.0, -3.0, 1.0], [3.0, -6.0, 2.0], [0.0, -7.0, 2.0]], [[3.0, -3.0, -2.0], [5.0, 5.0, -4.0], [1.0, 7.0, -2.0], [-3.0, 1.0, -6.0], [2.0, -5.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-43.0, 17.0, 32.0, -24.0, 0.0], [9.0, -42.0, -45.0, 41.0, 10.0], [4.0, 30.0, 16.0, 41.0, 36.0], [13.0, -21.0, -33.0, -37.0, 10.0]], [[53.0, -19.0, -34.0, 38.0, -12.0], [-9.0, 48.0, 59.0, -45.0, -10.0], [-8.0, -28.0, -22.0, -53.0, -50.0], [-19.0, 31.0, 47.0, 39.0, -20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[7.0, 1.0, 5.0], [6.0, 0.0, -7.0]], [[3.0, -2.0, -5.0], [7.0, 4.0, 6.0]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[1.0, 6.0, -7.0], [4.0, -6.0, -5.0]], [[7.0, -4.0, -4.0], [3.0, 5.0, 2.0]], [[5.0, -3.0, -7.0], [-3.0, -2.0, 5.0]], [[0.0, -2.0, -3.0], [5.0, 2.0, 4.0]], [[-3.0, 7.0, 2.0], [5.0, 1.0, 3.0]]], [[[-3.0, 0.0, -7.0], [-6.0, 4.0, 6.0]], [[-6.0, 4.0, -3.0], [-2.0, -7.0, 1.0]], [[-3.0, 1.0, -3.0], [-6.0, 7.0, -6.0]], [[1.0, 2.0, 5.0], [-5.0, 6.0, -5.0]], [[-2.0, -5.0, 4.0], [-7.0, -3.0, 0.0]]], [[[6.0, 2.0, 0.0], [7.0, 0.0, -3.0]], [[-5.0, 2.0, 6.0], [-1.0, -4.0, 7.0]], [[-5.0, 1.0, 2.0], [-1.0, -7.0, -5.0]], [[7.0, 3.0, -1.0], [-6.0, 6.0, -3.0]], [[-1.0, 1.0, 7.0], [4.0, 6.0, 6.0]]], [[[-7.0, -7.0, 1.0], [-1.0, 0.0, -3.0]], [[0.0, 3.0, -6.0], [0.0, -5.0, 0.0]], [[-5.0, -6.0, -3.0], [4.0, 0.0, 4.0]], [[-2.0, 5.0, -4.0], [1.0, -2.0, 2.0]], [[-5.0, 5.0, 2.0], [-6.0, -7.0, -7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[37.0, 29.0, -56.0, -15.0, 5.0], [-134.0, -72.0, -29.0, 39.0, -41.0], [107.0, -58.0, 5.0, 32.0, 11.0], [-36.0, -27.0, -60.0, -37.0, -7.0]], [[0.0, 102.0, 57.0, 86.0, 24.0], [36.0, -47.0, -46.0, -67.0, -77.0], [45.0, -30.0, -92.0, -16.0, 48.0], [-37.0, 4.0, 64.0, 15.0, -147.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_Symbol_rank3_offset0(self):
      arg0=Data(numpy.array([7.0, 0.0]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[-1.0, 6.0], [6.0, -1.0]], [[-5.0, 0.0], [5.0, -4.0]], [[4.0, -1.0], [4.0, -4.0]], [[1.0, -5.0], [7.0, -3.0]], [[3.0, 2.0], [1.0, 0.0]], [[-7.0, -2.0], [-4.0, 3.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-7.0, 42.0], [42.0, -7.0]], [[-35.0, 0.0], [35.0, -28.0]], [[28.0, -7.0], [28.0, -28.0]], [[7.0, -35.0], [49.0, -21.0]], [[21.0, 14.0], [7.0, 0.0]], [[-49.0, -14.0], [-28.0, 21.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank4_offset1(self):
      arg0=Data(numpy.array([[-6.0, -1.0, 2.0], [1.0, -1.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[-6.0, 3.0, -5.0], [4.0, -7.0, 0.0]], [[-2.0, 0.0, 2.0], [-1.0, -1.0, 1.0]]], [[[4.0, -4.0, -3.0], [2.0, -4.0, -1.0]], [[-4.0, 5.0, 1.0], [3.0, -1.0, 5.0]]], [[[6.0, -7.0, 3.0], [-5.0, -2.0, 5.0]], [[0.0, 4.0, -7.0], [-4.0, 6.0, -7.0]]], [[[-5.0, 6.0, 0.0], [4.0, -3.0, -1.0]], [[5.0, 5.0, -5.0], [-5.0, -5.0, -2.0]]], [[[5.0, -2.0, 2.0], [-6.0, -7.0, 0.0]], [[-3.0, -3.0, 6.0], [1.0, 6.0, -7.0]]], [[[-4.0, 5.0, 2.0], [0.0, -6.0, -2.0]], [[0.0, 5.0, 6.0], [-2.0, -4.0, -2.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[23.0, -17.0], [16.0, 9.0]], [[-26.0, -10.0], [21.0, -7.0]], [[-23.0, 42.0], [-18.0, 4.0]], [[24.0, -23.0], [-45.0, 31.0]], [[-24.0, 43.0], [33.0, -26.0]], [[23.0, 2.0], [7.0, 12.0]]], [[[6.0, 11.0], [-8.0, -3.0]], [[17.0, 9.0], [-12.0, -11.0]], [[4.0, -18.0], [17.0, 11.0]], [[-11.0, 10.0], [15.0, 6.0]], [[1.0, 1.0], [-18.0, 16.0]], [[-15.0, 12.0], [-23.0, 8.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank0_offset0(self):
      arg0=Data(numpy.array([5.0, -5.0]),self.functionspace)
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([-35.0, 35.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[-1.0, -3.0, -6.0], [0.0, -7.0, -5.0]]),self.functionspace)
      arg1=Data(numpy.array([6.0, -1.0, 1.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([-9.0, 2.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[0.0, -1.0, 3.0], [4.0, 0.0, 1.0]], [[-7.0, 5.0, -3.0], [-3.0, -5.0, 0.0]]]),self.functionspace)
      arg1=Data(numpy.array([[7.0, 2.0, -5.0], [2.0, 5.0, -4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([-13.0, -55.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[4.0, 0.0, -1.0], [5.0, -5.0, 0.0], [5.0, -7.0, -4.0], [-6.0, -6.0, 0.0]], [[4.0, 5.0, -1.0], [1.0, -2.0, -3.0], [-7.0, -2.0, -3.0], [1.0, 0.0, -5.0]]], [[[-6.0, -2.0, 7.0], [6.0, -6.0, -1.0], [5.0, 7.0, -5.0], [6.0, 6.0, 1.0]], [[4.0, 4.0, -4.0], [0.0, 4.0, 0.0], [3.0, -7.0, -5.0], [3.0, 6.0, -7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[4.0, 6.0, -3.0], [7.0, 7.0, -5.0], [-1.0, 3.0, 2.0], [-7.0, -7.0, 7.0]], [[3.0, -6.0, -6.0], [0.0, -5.0, 0.0], [-4.0, -3.0, 0.0], [-5.0, -1.0, 1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([91.0, -150.0]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank1_offset0(self):
      arg0=Data(numpy.array([0.0, 1.0]),self.functionspace)
      arg1=Data(numpy.array([5.0, -6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[0.0, 0.0], [5.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[-6.0, -1.0, 4.0], [0.0, -3.0, -2.0]]),self.functionspace)
      arg1=Data(numpy.array([[3.0, -6.0, 7.0], [2.0, 1.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[16.0, -13.0], [4.0, -3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[2.0, 0.0, 2.0], [0.0, 0.0, 5.0]], [[1.0, 4.0, 0.0], [-7.0, 2.0, 3.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[-2.0, 0.0, 0.0], [0.0, 3.0, -5.0]], [[2.0, -3.0, 3.0], [-5.0, 1.0, 4.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[-29.0, 30.0], [-11.0, 39.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[0.0, -1.0, 7.0], [4.0, 0.0, 2.0], [7.0, 0.0, 7.0], [6.0, -3.0, -4.0]], [[-7.0, 0.0, 1.0], [-2.0, -3.0, -6.0], [-4.0, 7.0, 5.0], [6.0, 5.0, 0.0]]], [[[7.0, -6.0, 2.0], [7.0, 5.0, 4.0], [4.0, 2.0, 0.0], [-7.0, 7.0, 0.0]], [[7.0, 6.0, 5.0], [7.0, -5.0, 6.0], [6.0, -7.0, -7.0], [-2.0, -2.0, -4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[-6.0, -5.0, -7.0], [5.0, 2.0, 4.0], [5.0, 0.0, 0.0], [0.0, 5.0, -3.0]], [[0.0, 0.0, -2.0], [-5.0, -1.0, -3.0], [2.0, 5.0, 3.0], [6.0, -6.0, -6.0]]], [[[7.0, -7.0, 5.0], [-5.0, -7.0, -5.0], [6.0, -1.0, 0.0], [6.0, -5.0, -7.0]], [[-1.0, 7.0, 0.0], [7.0, -2.0, 6.0], [2.0, -3.0, 4.0], [0.0, -5.0, -3.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      ref=Data(numpy.array([[93.0, 62.0], [12.0, 113.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank2_offset0(self):
      arg0=Data(numpy.array([1.0, -7.0]),self.functionspace)
      arg1=Data(numpy.array([[7.0, -4.0, 1.0, 4.0, 7.0], [4.0, 1.0, -2.0, 5.0, -4.0], [2.0, -3.0, 2.0, 5.0, 6.0], [-7.0, -4.0, 0.0, -1.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[7.0, -4.0, 1.0, 4.0, 7.0], [4.0, 1.0, -2.0, 5.0, -4.0], [2.0, -3.0, 2.0, 5.0, 6.0], [-7.0, -4.0, 0.0, -1.0, 0.0]], [[-49.0, 28.0, -7.0, -28.0, -49.0], [-28.0, -7.0, 14.0, -35.0, 28.0], [-14.0, 21.0, -14.0, -35.0, -42.0], [49.0, 28.0, 0.0, 7.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[7.0, 5.0, 3.0], [0.0, 2.0, -3.0]]),self.functionspace)
      arg1=Data(numpy.array([[[6.0, 5.0, -2.0], [6.0, 6.0, 2.0], [0.0, -2.0, -2.0], [-6.0, -4.0, 7.0], [4.0, 2.0, 2.0]], [[0.0, -6.0, -4.0], [-2.0, -5.0, 2.0], [4.0, -7.0, -4.0], [-6.0, 3.0, -2.0], [-2.0, -3.0, 1.0]], [[-1.0, 5.0, 0.0], [-4.0, -6.0, -3.0], [-1.0, -3.0, 2.0], [-5.0, 3.0, 1.0], [3.0, -5.0, -5.0]], [[-5.0, 0.0, 7.0], [0.0, -5.0, -5.0], [3.0, 0.0, 4.0], [0.0, -3.0, 7.0], [-4.0, 7.0, 1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[61.0, 78.0, -16.0, -41.0, 44.0], [-42.0, -33.0, -19.0, -33.0, -26.0], [18.0, -67.0, -16.0, -17.0, -19.0], [-14.0, -40.0, 33.0, 6.0, 10.0]], [[16.0, 6.0, 2.0, -29.0, -2.0], [0.0, -16.0, -2.0, 12.0, -9.0], [10.0, -3.0, -12.0, 3.0, 5.0], [-21.0, 5.0, -12.0, -27.0, 11.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[0.0, 1.0, 3.0], [-5.0, -1.0, -3.0]], [[-3.0, 1.0, -3.0], [-6.0, -5.0, 2.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[[7.0, 0.0, -2.0], [0.0, -3.0, 1.0]], [[1.0, 4.0, 5.0], [6.0, -2.0, 1.0]], [[-1.0, -7.0, -2.0], [-1.0, 1.0, 6.0]], [[-2.0, 1.0, -1.0], [-7.0, 2.0, 0.0]], [[-4.0, 1.0, 6.0], [3.0, -7.0, -6.0]]], [[[-5.0, -5.0, 4.0], [-1.0, 5.0, 1.0]], [[-4.0, -6.0, 4.0], [-7.0, -4.0, 1.0]], [[-3.0, 3.0, -7.0], [5.0, 0.0, 5.0]], [[1.0, -2.0, -4.0], [0.0, 0.0, -3.0]], [[-2.0, 4.0, 4.0], [7.0, -6.0, 0.0]]], [[[2.0, 5.0, -4.0], [4.0, -2.0, -5.0]], [[-7.0, 2.0, -7.0], [-3.0, 0.0, 1.0]], [[0.0, -1.0, -5.0], [-1.0, -1.0, 0.0]], [[0.0, 6.0, -3.0], [4.0, -7.0, 4.0]], [[0.0, 1.0, -6.0], [0.0, -6.0, -1.0]]], [[[-7.0, -1.0, 5.0], [-6.0, -2.0, 2.0]], [[0.0, 3.0, -4.0], [2.0, 2.0, 6.0]], [[-2.0, 4.0, -3.0], [4.0, 4.0, 1.0]], [[3.0, -6.0, 6.0], [3.0, -1.0, -4.0]], [[7.0, -4.0, 7.0], [2.0, 3.0, -6.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-6.0, -12.0, -27.0, 31.0, 29.0], [4.0, 42.0, -58.0, -5.0, -13.0], [-10.0, -7.0, -10.0, -28.0, -8.0], [40.0, -39.0, -32.0, 10.0, 22.0]], [[2.0, -38.0, 15.0, 42.0, 0.0], [-19.0, 58.0, 13.0, 1.0, -14.0], [-13.0, 64.0, 25.0, 34.0, 47.0], [55.0, 5.0, -23.0, -54.0, -85.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_constData_rank3_offset0(self):
      arg0=Data(numpy.array([0.0, 2.0]),self.functionspace)
      arg1=Data(numpy.array([[[0.0, 0.0], [5.0, 0.0]], [[5.0, -4.0], [-4.0, 7.0]], [[-5.0, -2.0], [2.0, 0.0]], [[-6.0, -6.0], [1.0, -4.0]], [[5.0, 1.0], [-1.0, -4.0]], [[-2.0, -2.0], [6.0, -5.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]], [[[0.0, 0.0], [10.0, 0.0]], [[10.0, -8.0], [-8.0, 14.0]], [[-10.0, -4.0], [4.0, 0.0]], [[-12.0, -12.0], [2.0, -8.0]], [[10.0, 2.0], [-2.0, -8.0]], [[-4.0, -4.0], [12.0, -10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank4_offset1(self):
      arg0=Data(numpy.array([[-4.0, -4.0, -2.0], [1.0, -6.0, -2.0]]),self.functionspace)
      arg1=Data(numpy.array([[[[3.0, -7.0, 4.0], [2.0, 2.0, 2.0]], [[0.0, -6.0, -1.0], [-5.0, -5.0, 2.0]]], [[[-2.0, -3.0, -2.0], [4.0, 0.0, 4.0]], [[-1.0, -1.0, -3.0], [-1.0, -6.0, 3.0]]], [[[5.0, -6.0, -3.0], [4.0, 2.0, -4.0]], [[-7.0, 7.0, -7.0], [-2.0, 7.0, 1.0]]], [[[6.0, 2.0, 4.0], [-3.0, 5.0, 1.0]], [[-3.0, -1.0, 6.0], [-5.0, 0.0, 0.0]]], [[[-7.0, 0.0, 0.0], [0.0, -7.0, -6.0]], [[-4.0, -2.0, 0.0], [-6.0, 7.0, -5.0]]], [[[5.0, 4.0, 0.0], [-7.0, -1.0, -7.0]], [[0.0, 2.0, -5.0], [-3.0, 3.0, 5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[8.0, -20.0], [26.0, 36.0]], [[24.0, -24.0], [14.0, 22.0]], [[10.0, -16.0], [14.0, -22.0]], [[-40.0, -10.0], [4.0, 20.0]], [[28.0, 40.0], [24.0, 6.0]], [[-36.0, 46.0], [2.0, -10.0]]], [[[37.0, -14.0], [38.0, 21.0]], [[20.0, -4.0], [11.0, 29.0]], [[47.0, 0.0], [-35.0, -46.0]], [[-14.0, -35.0], [-9.0, -5.0]], [[-7.0, 54.0], [8.0, -38.0]], [[-19.0, 13.0], [-2.0, -31.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([0.0, 5.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-0.0, -15.0])+(1.-msk_ref)*numpy.array([-0.0, -5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[4.0, -5.0, -3.0], [0.0, -4.0, -2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 0.0, -7.0])+(1.-msk_arg1)*numpy.array([-6.0, 6.0, 4.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([21.0, 14.0])+(1.-msk_ref)*numpy.array([-66.0, -32.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[4.0, -6.0, 1.0], [6.0, -7.0, -4.0]], [[-1.0, 0.0, -2.0], [-3.0, 0.0, -7.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[0.0, 6.0, 3.0], [0.0, -5.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -7.0, 7.0], [6.0, -7.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([18.0, 22.0])+(1.-msk_ref)*numpy.array([82.0, -67.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank3_offset3(self):
      arg0=Data(numpy.array([[[[-1.0, -3.0, 7.0], [-7.0, -6.0, -3.0], [0.0, 1.0, 0.0], [7.0, 7.0, 3.0]], [[2.0, -4.0, 6.0], [3.0, -2.0, 0.0], [5.0, 7.0, 4.0], [-1.0, -1.0, 7.0]]], [[[-3.0, 0.0, 0.0], [3.0, 0.0, -6.0], [0.0, 4.0, -6.0], [-4.0, 1.0, -1.0]], [[0.0, 2.0, 6.0], [0.0, 3.0, 2.0], [4.0, -6.0, -5.0], [-6.0, -7.0, 5.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 2.0, 6.0], [6.0, -5.0, 0.0], [-1.0, -4.0, 5.0], [7.0, 0.0, -5.0]], [[-7.0, -3.0, -3.0], [-4.0, 0.0, -3.0], [0.0, 3.0, 6.0], [-1.0, 0.0, -5.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, 2.0, -6.0], [-4.0, 6.0, -4.0], [6.0, 2.0, -4.0], [3.0, 7.0, 7.0]], [[5.0, -7.0, -3.0], [-2.0, -7.0, 2.0], [7.0, 6.0, 0.0], [-1.0, -1.0, 1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([26.0, -169.0])+(1.-msk_ref)*numpy.array([170.0, 14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([2.0, 1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, 4.0])+(1.-msk_arg1)*numpy.array([2.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[12.0, 8.0], [6.0, 4.0]])+(1.-msk_ref)*numpy.array([[4.0, 14.0], [2.0, 7.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[-1.0, -4.0, -3.0], [4.0, -2.0, 4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, 2.0, -5.0], [3.0, 4.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-5.0, -6.0, 4.0], [3.0, 6.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[11.0, -19.0], [-40.0, 4.0]])+(1.-msk_ref)*numpy.array([[17.0, -45.0], [8.0, 24.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[-3.0, -7.0, 4.0], [6.0, 3.0, -5.0]], [[3.0, -6.0, -1.0], [-2.0, -5.0, -1.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, 6.0, -6.0], [4.0, 7.0, -3.0]], [[1.0, 2.0, 1.0], [2.0, 0.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, 2.0, -1.0], [1.0, -4.0, -1.0]], [[-5.0, 2.0, 4.0], [-6.0, 3.0, 3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[9.0, 34.0], [-85.0, -7.0]])+(1.-msk_ref)*numpy.array([[-16.0, -25.0], [5.0, -37.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank4_offset3(self):
      arg0=Data(numpy.array([[[[0.0, 5.0, -3.0], [-4.0, 7.0, 4.0], [0.0, 5.0, 7.0], [-7.0, 0.0, -4.0]], [[-4.0, -6.0, -5.0], [5.0, 3.0, -6.0], [0.0, 7.0, -3.0], [-1.0, 7.0, -7.0]]], [[[5.0, 5.0, -5.0], [-1.0, 7.0, -5.0], [-7.0, 7.0, -7.0], [3.0, -6.0, 7.0]], [[1.0, -7.0, -5.0], [5.0, 5.0, -6.0], [0.0, 5.0, 7.0], [-6.0, 6.0, 4.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, -4.0, 2.0], [6.0, -2.0, 4.0], [6.0, -6.0, -6.0], [-5.0, -1.0, 2.0]], [[6.0, 5.0, -6.0], [-6.0, 2.0, 4.0], [2.0, 4.0, 0.0], [3.0, 1.0, 2.0]]], [[[1.0, 1.0, 6.0], [-4.0, 4.0, 1.0], [-4.0, 6.0, -4.0], [6.0, -3.0, -3.0]], [[-4.0, 7.0, 7.0], [0.0, -3.0, 5.0], [-1.0, 7.0, -7.0], [-5.0, 6.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -3.0, -1.0], [2.0, 7.0, -1.0], [-3.0, 4.0, -2.0], [-5.0, 4.0, 6.0]], [[-7.0, -4.0, -2.0], [4.0, -3.0, -1.0], [1.0, 0.0, -5.0], [-6.0, -3.0, 7.0]]], [[[-4.0, -2.0, 3.0], [4.0, 0.0, 3.0], [0.0, 0.0, 6.0], [-6.0, -4.0, -2.0]], [[-3.0, 2.0, 0.0], [-6.0, -1.0, -5.0], [-4.0, -5.0, 5.0], [-1.0, -7.0, 5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-147.0, 3.0], [-124.0, 51.0]])+(1.-msk_ref)*numpy.array([[72.0, -67.0], [161.0, -142.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([-5.0, -4.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 3.0, -2.0, -5.0, -5.0], [5.0, -7.0, 3.0, -1.0, -1.0], [7.0, -7.0, -4.0, -1.0, 3.0], [1.0, 3.0, 4.0, -5.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-3.0, 4.0, -5.0, -5.0, 0.0], [2.0, 4.0, 0.0, 5.0, 3.0], [-7.0, 1.0, 5.0, 0.0, 6.0], [-2.0, 0.0, 5.0, 1.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[30.0, -15.0, 10.0, 25.0, 25.0], [-25.0, 35.0, -15.0, 5.0, 5.0], [-35.0, 35.0, 20.0, 5.0, -15.0], [-5.0, -15.0, -20.0, 25.0, 0.0]], [[24.0, -12.0, 8.0, 20.0, 20.0], [-20.0, 28.0, -12.0, 4.0, 4.0], [-28.0, 28.0, 16.0, 4.0, -12.0], [-4.0, -12.0, -16.0, 20.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[15.0, -20.0, 25.0, 25.0, 0.0], [-10.0, -20.0, 0.0, -25.0, -15.0], [35.0, -5.0, -25.0, 0.0, -30.0], [10.0, 0.0, -25.0, -5.0, -35.0]], [[12.0, -16.0, 20.0, 20.0, 0.0], [-8.0, -16.0, 0.0, -20.0, -12.0], [28.0, -4.0, -20.0, 0.0, -24.0], [8.0, 0.0, -20.0, -4.0, -28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[-5.0, -1.0, 5.0], [-1.0, 6.0, -4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, -2.0, -3.0], [-7.0, -4.0, -6.0], [5.0, 0.0, 4.0], [-5.0, 2.0, 2.0], [1.0, -7.0, 4.0]], [[-4.0, 7.0, -1.0], [-4.0, -2.0, 2.0], [-1.0, -5.0, -6.0], [0.0, -3.0, 7.0], [2.0, 3.0, -7.0]], [[0.0, -7.0, -2.0], [2.0, 0.0, -7.0], [4.0, 5.0, -4.0], [7.0, 4.0, -2.0], [4.0, -3.0, -2.0]], [[0.0, 1.0, 2.0], [0.0, 1.0, 2.0], [-5.0, 4.0, 4.0], [3.0, 6.0, 1.0], [6.0, -7.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, -6.0, 4.0], [-7.0, 6.0, -2.0], [-4.0, -6.0, -7.0], [2.0, -2.0, 2.0], [-4.0, -6.0, -3.0]], [[-3.0, 0.0, -7.0], [-5.0, 0.0, 5.0], [-5.0, 6.0, 3.0], [-3.0, 3.0, -1.0], [7.0, 4.0, 2.0]], [[-2.0, 6.0, -5.0], [6.0, 7.0, -3.0], [-6.0, -1.0, -5.0], [6.0, 6.0, -4.0], [3.0, 1.0, 2.0]], [[7.0, 2.0, 5.0], [-3.0, -3.0, 4.0], [-7.0, 3.0, 0.0], [-2.0, -5.0, -2.0], [0.0, -2.0, 1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-23.0, 9.0, -5.0, 33.0, 22.0], [8.0, 32.0, -20.0, 38.0, -48.0], [-3.0, -45.0, -45.0, -49.0, -27.0], [9.0, 9.0, 41.0, -16.0, -23.0]], [[-2.0, 7.0, -21.0, 9.0, -59.0], [50.0, -16.0, -5.0, -46.0, 44.0], [-34.0, 26.0, 42.0, 25.0, -14.0], [-2.0, -2.0, 13.0, 29.0, -48.0]]])+(1.-msk_ref)*numpy.array([[[21.0, 19.0, -9.0, 2.0, 11.0], [-20.0, 50.0, 34.0, 7.0, -29.0], [-21.0, -52.0, 6.0, -56.0, -6.0], [-12.0, 38.0, 32.0, 5.0, 7.0]], [[-53.0, 51.0, -4.0, -22.0, -20.0], [31.0, -15.0, 29.0, 25.0, 9.0], [58.0, 48.0, 20.0, 46.0, -5.0], [-15.0, -31.0, 25.0, -20.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[-5.0, 6.0, 1.0], [-5.0, 6.0, 5.0]], [[-4.0, 5.0, 3.0], [4.0, -6.0, 0.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -6.0, 5.0], [-4.0, -1.0, 2.0]], [[0.0, 2.0, 5.0], [-3.0, 6.0, -1.0]], [[0.0, 0.0, -7.0], [-1.0, -7.0, 2.0]], [[0.0, -6.0, 0.0], [-7.0, -2.0, 6.0]], [[-7.0, -4.0, -5.0], [2.0, 6.0, 0.0]]], [[[-4.0, 4.0, -4.0], [-2.0, -2.0, 0.0]], [[-4.0, -7.0, 0.0], [-3.0, 0.0, 0.0]], [[4.0, -4.0, 7.0], [4.0, 4.0, 5.0]], [[4.0, 7.0, 6.0], [-6.0, 1.0, 3.0]], [[1.0, -2.0, 3.0], [-7.0, 2.0, -3.0]]], [[[-5.0, -5.0, -5.0], [5.0, 5.0, 6.0]], [[1.0, -3.0, -3.0], [0.0, -2.0, 0.0]], [[6.0, 6.0, -2.0], [0.0, -4.0, -6.0]], [[4.0, -7.0, 5.0], [3.0, 7.0, -5.0]], [[-7.0, 4.0, -4.0], [-3.0, 2.0, -6.0]]], [[[0.0, 5.0, 7.0], [0.0, -3.0, 2.0]], [[2.0, 0.0, -2.0], [-5.0, 0.0, 5.0]], [[3.0, -3.0, 6.0], [0.0, 0.0, 7.0]], [[-1.0, -2.0, -1.0], [0.0, 7.0, -6.0]], [[6.0, -3.0, 2.0], [3.0, -7.0, -7.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, -5.0, 5.0], [6.0, 7.0, 0.0]], [[0.0, -2.0, -2.0], [-4.0, -2.0, -3.0]], [[6.0, 0.0, 1.0], [-7.0, 6.0, -2.0]], [[7.0, -7.0, 6.0], [3.0, 5.0, -7.0]], [[-5.0, 0.0, 4.0], [-5.0, -3.0, 6.0]]], [[[6.0, 3.0, 1.0], [7.0, 6.0, 6.0]], [[1.0, 0.0, 5.0], [-3.0, 3.0, -4.0]], [[-2.0, -5.0, 2.0], [-7.0, 5.0, -4.0]], [[0.0, 5.0, -3.0], [4.0, 0.0, 7.0]], [[-2.0, 4.0, -3.0], [0.0, -1.0, -5.0]]], [[[5.0, 5.0, 3.0], [-6.0, 1.0, 4.0]], [[6.0, -5.0, 1.0], [-4.0, -1.0, 0.0]], [[2.0, 2.0, -2.0], [-4.0, -2.0, 5.0]], [[0.0, 1.0, 7.0], [-4.0, 4.0, -5.0]], [[-7.0, -7.0, -7.0], [4.0, 7.0, 0.0]]], [[[1.0, 7.0, -1.0], [0.0, 2.0, 1.0]], [[6.0, 2.0, -2.0], [3.0, 0.0, 6.0]], [[3.0, -6.0, 1.0], [-4.0, -1.0, 5.0]], [[6.0, -4.0, -5.0], [-3.0, 5.0, 3.0]], [[7.0, 7.0, 0.0], [0.0, 3.0, -2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[3.0, 63.0, -34.0, 17.0, 32.0], [38.0, -7.0, -8.0, 79.0, 18.0], [25.0, -38.0, -50.0, -55.0, 52.0], [29.0, 38.0, 8.0, 4.0, -138.0]], [[-17.0, -23.0, 17.0, -46.0, -35.0], [28.0, -31.0, -23.0, 7.0, -45.0], [-30.0, -16.0, 24.0, -66.0, 12.0], [64.0, -34.0, -9.0, -51.0, 21.0]]])+(1.-msk_ref)*numpy.array([[[-13.0, -21.0, 32.0, -91.0, 66.0], [20.0, 13.0, 27.0, 42.0, 0.0], [64.0, -45.0, 33.0, 32.0, 8.0], [53.0, -5.0, -11.0, 1.0, 15.0]], [[-28.0, -20.0, -85.0, -63.0, 30.0], [-14.0, -19.0, -69.0, 32.0, 25.0], [-16.0, -56.0, -8.0, -14.0, -54.0], [16.0, -8.0, -49.0, -101.0, -11.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank1_expandedData_rank3_offset0(self):
      arg0=Data(numpy.array([-5.0, 1.0]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-2.0, 0.0], [1.0, 4.0]], [[-3.0, -1.0], [-7.0, 3.0]], [[-2.0, 6.0], [7.0, -3.0]], [[0.0, -2.0], [0.0, -6.0]], [[2.0, -3.0], [5.0, 0.0]], [[1.0, 0.0], [1.0, 2.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, 2.0], [1.0, -5.0]], [[6.0, 7.0], [7.0, 3.0]], [[2.0, -2.0], [3.0, -5.0]], [[-2.0, 0.0], [-1.0, -5.0]], [[-7.0, -4.0], [-5.0, 3.0]], [[0.0, 4.0], [2.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 0.0], [-5.0, -20.0]], [[15.0, 5.0], [35.0, -15.0]], [[10.0, -30.0], [-35.0, 15.0]], [[0.0, 10.0], [0.0, 30.0]], [[-10.0, 15.0], [-25.0, 0.0]], [[-5.0, 0.0], [-5.0, -10.0]]], [[[-2.0, 0.0], [1.0, 4.0]], [[-3.0, -1.0], [-7.0, 3.0]], [[-2.0, 6.0], [7.0, -3.0]], [[0.0, -2.0], [0.0, -6.0]], [[2.0, -3.0], [5.0, 0.0]], [[1.0, 0.0], [1.0, 2.0]]]])+(1.-msk_ref)*numpy.array([[[[-5.0, -10.0], [-5.0, 25.0]], [[-30.0, -35.0], [-35.0, -15.0]], [[-10.0, 10.0], [-15.0, 25.0]], [[10.0, 0.0], [5.0, 25.0]], [[35.0, 20.0], [25.0, -15.0]], [[0.0, -20.0], [-10.0, -10.0]]], [[[1.0, 2.0], [1.0, -5.0]], [[6.0, 7.0], [7.0, 3.0]], [[2.0, -2.0], [3.0, -5.0]], [[-2.0, 0.0], [-1.0, -5.0]], [[-7.0, -4.0], [-5.0, 3.0]], [[0.0, 4.0], [2.0, 2.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank4_offset1(self):
      arg0=Data(numpy.array([[5.0, -5.0, -4.0], [2.0, 1.0, 0.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-1.0, 7.0, -5.0], [-3.0, 1.0, 1.0]], [[-1.0, 1.0, 7.0], [2.0, 4.0, -1.0]]], [[[0.0, -7.0, -6.0], [-1.0, 1.0, 6.0]], [[-7.0, 6.0, 4.0], [3.0, -4.0, -1.0]]], [[[-4.0, -1.0, 6.0], [-4.0, -2.0, 4.0]], [[7.0, -6.0, -6.0], [0.0, -4.0, -1.0]]], [[[-5.0, 4.0, 4.0], [0.0, 5.0, -3.0]], [[-1.0, -7.0, 7.0], [-6.0, 3.0, 4.0]]], [[[-6.0, 4.0, -3.0], [-5.0, -6.0, 0.0]], [[3.0, 0.0, 4.0], [-2.0, 5.0, 6.0]]], [[[-3.0, -3.0, 2.0], [0.0, 0.0, -6.0]], [[7.0, -1.0, -4.0], [-6.0, 5.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-4.0, -7.0, 1.0], [2.0, 6.0, -3.0]], [[4.0, 5.0, -6.0], [0.0, 1.0, -3.0]]], [[[-3.0, -7.0, 2.0], [-3.0, -6.0, 6.0]], [[1.0, 1.0, 5.0], [7.0, -6.0, 1.0]]], [[[7.0, 1.0, -6.0], [4.0, 5.0, -3.0]], [[1.0, -4.0, 1.0], [-2.0, 5.0, 5.0]]], [[[-5.0, 3.0, -5.0], [0.0, 7.0, 3.0]], [[7.0, 2.0, -7.0], [-2.0, 2.0, 3.0]]], [[[1.0, -6.0, -3.0], [-1.0, -7.0, 1.0]], [[-6.0, 5.0, 0.0], [7.0, 1.0, 5.0]]], [[[-6.0, 1.0, -6.0], [-3.0, 0.0, 1.0]], [[-4.0, 2.0, -3.0], [-2.0, 0.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-20.0, -24.0], [-38.0, -6.0]], [[59.0, -34.0], [-81.0, 39.0]], [[-39.0, -26.0], [89.0, 24.0]], [[-61.0, -13.0], [2.0, -61.0]], [[-38.0, 5.0], [-1.0, -59.0]], [[-8.0, 24.0], [56.0, -83.0]]], [[[5.0, -5.0], [-1.0, 8.0]], [[-7.0, -1.0], [-8.0, 2.0]], [[-9.0, -10.0], [8.0, -4.0]], [[-6.0, 5.0], [-9.0, -9.0]], [[-8.0, -16.0], [6.0, 1.0]], [[-9.0, 0.0], [13.0, -7.0]]]])+(1.-msk_ref)*numpy.array([[[[11.0, -8.0], [19.0, 7.0]], [[12.0, -9.0], [-20.0, 61.0]], [[54.0, 7.0], [21.0, -55.0]], [[-20.0, -47.0], [53.0, -32.0]], [[47.0, 26.0], [-55.0, 10.0]], [[-11.0, -19.0], [-18.0, -26.0]]], [[[-15.0, 10.0], [13.0, 1.0]], [[-13.0, -12.0], [3.0, 8.0]], [[15.0, 13.0], [-2.0, 1.0]], [[-7.0, 7.0], [16.0, -2.0]], [[-4.0, -9.0], [-7.0, 15.0]], [[-11.0, -6.0], [-6.0, -4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_float_rank0_offset0(self):
      arg0=Data(numpy.array([[0.0, 5.0, -1.0, -1.0, -2.0], [7.0, -7.0, 3.0, -4.0, -5.0], [5.0, -6.0, 1.0, 7.0, -1.0], [7.0, -1.0, -5.0, -2.0, -6.0]]),self.functionspace)
      arg1=1.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[0.0, 5.0, -1.0, -1.0, -2.0], [7.0, -7.0, 3.0, -4.0, -5.0], [5.0, -6.0, 1.0, 7.0, -1.0], [7.0, -1.0, -5.0, -2.0, -6.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank0_offset0(self):
      arg0=Data(numpy.array([[1.0, -2.0, 5.0, 7.0, 7.0], [2.0, 2.0, 0.0, -2.0, -7.0], [-2.0, -7.0, 3.0, 4.0, -7.0], [0.0, -3.0, 2.0, -3.0, -7.0]]),self.functionspace)
      arg1=numpy.array(-2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[-2.0, 4.0, -10.0, -14.0, -14.0], [-4.0, -4.0, -0.0, 4.0, 14.0], [4.0, 14.0, -6.0, -8.0, 14.0], [-0.0, 6.0, -4.0, 6.0, 14.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[-3.0, -3.0, 7.0], [-3.0, 0.0, 3.0], [4.0, 5.0, 7.0], [3.0, 3.0, 0.0], [2.0, 5.0, 3.0]], [[-3.0, 2.0, -5.0], [-4.0, 1.0, 0.0], [1.0, -5.0, -5.0], [1.0, -7.0, 2.0], [4.0, 7.0, 3.0]], [[7.0, 0.0, 7.0], [-1.0, -3.0, -5.0], [-3.0, -6.0, 7.0], [-4.0, 7.0, -7.0], [2.0, 7.0, 4.0]], [[-4.0, 0.0, -3.0], [7.0, 6.0, -2.0], [7.0, -4.0, 0.0], [2.0, 3.0, -1.0], [3.0, -2.0, -2.0]]]),self.functionspace)
      arg1=numpy.array([-2.0, 6.0, -3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[-33.0, -3.0, 1.0, 12.0, 17.0], [33.0, 14.0, -17.0, -50.0, 25.0], [-35.0, -1.0, -51.0, 71.0, 26.0], [17.0, 28.0, -38.0, 17.0, -12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank2_offset2(self):
      arg0=Data(numpy.array([[[[4.0, 2.0, -5.0], [0.0, 5.0, 0.0]], [[-1.0, 3.0, 7.0], [-1.0, 6.0, 2.0]], [[5.0, -2.0, 6.0], [3.0, 3.0, -5.0]], [[-7.0, -1.0, 4.0], [6.0, 0.0, -6.0]], [[-5.0, -5.0, -2.0], [4.0, -2.0, 1.0]]], [[[-3.0, -3.0, -1.0], [-5.0, -3.0, 6.0]], [[-4.0, -2.0, 0.0], [5.0, -6.0, 0.0]], [[6.0, -7.0, -5.0], [0.0, -5.0, -7.0]], [[-7.0, 4.0, 2.0], [2.0, -5.0, -1.0]], [[0.0, 5.0, -5.0], [-6.0, -3.0, -3.0]]], [[[0.0, 0.0, 5.0], [0.0, 0.0, 0.0]], [[-7.0, 2.0, -4.0], [5.0, 6.0, -6.0]], [[4.0, 6.0, -1.0], [0.0, -6.0, -7.0]], [[-2.0, 3.0, -4.0], [5.0, 4.0, -4.0]], [[-1.0, 0.0, -7.0], [-5.0, 2.0, -5.0]]], [[[4.0, 4.0, -5.0], [1.0, -2.0, -7.0]], [[6.0, 5.0, -4.0], [7.0, 7.0, -2.0]], [[7.0, 0.0, -7.0], [-5.0, -7.0, 0.0]], [[-3.0, 4.0, 0.0], [0.0, 5.0, 2.0]], [[7.0, -4.0, -1.0], [-2.0, -1.0, -1.0]]]]),self.functionspace)
      arg1=numpy.array([[-7.0, -5.0, -6.0], [-6.0, 3.0, -2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[7.0, -30.0, -60.0, 6.0, 40.0], [51.0, -10.0, 22.0, -8.0, 38.0], [-30.0, 63.0, -56.0, 13.0, 95.0], [-16.0, -60.0, 2.0, 12.0, -12.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank1_offset0(self):
      arg0=Data(numpy.array([[-1.0, 3.0, -6.0, -5.0, -7.0], [7.0, 0.0, -2.0, -5.0, -1.0], [3.0, 5.0, 7.0, 6.0, 3.0], [2.0, 3.0, -3.0, 0.0, 5.0]]),self.functionspace)
      arg1=numpy.array([0.0, -2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 2.0], [0.0, -6.0], [0.0, 12.0], [0.0, 10.0], [0.0, 14.0]], [[0.0, -14.0], [0.0, 0.0], [0.0, 4.0], [0.0, 10.0], [0.0, 2.0]], [[0.0, -6.0], [0.0, -10.0], [0.0, -14.0], [0.0, -12.0], [0.0, -6.0]], [[0.0, -4.0], [0.0, -6.0], [0.0, 6.0], [0.0, 0.0], [0.0, -10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[4.0, -1.0, -4.0], [-5.0, -1.0, 5.0], [5.0, 0.0, 3.0], [7.0, 1.0, 1.0], [-2.0, 0.0, 5.0]], [[-3.0, -2.0, 1.0], [6.0, 2.0, -5.0], [-1.0, -1.0, 2.0], [0.0, 4.0, -4.0], [-6.0, 5.0, 6.0]], [[3.0, -7.0, -4.0], [-7.0, -2.0, -5.0], [1.0, -6.0, 0.0], [4.0, 4.0, -7.0], [-3.0, 7.0, -3.0]], [[-2.0, -7.0, 7.0], [-6.0, -4.0, 6.0], [-7.0, -1.0, -3.0], [5.0, -3.0, 5.0], [3.0, 2.0, 2.0]]]),self.functionspace)
      arg1=numpy.array([[7.0, -6.0, -7.0], [-7.0, 0.0, 2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[62.0, -36.0], [-64.0, 45.0], [14.0, -29.0], [36.0, -47.0], [-49.0, 24.0]], [[-16.0, 23.0], [65.0, -52.0], [-15.0, 11.0], [4.0, -8.0], [-114.0, 54.0]], [[91.0, -29.0], [-2.0, 39.0], [43.0, -7.0], [53.0, -42.0], [-42.0, 15.0]], [[-21.0, 28.0], [-60.0, 54.0], [-22.0, 43.0], [18.0, -25.0], [-5.0, -17.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank3_offset2(self):
      arg0=Data(numpy.array([[[[1.0, 2.0, 4.0], [0.0, -2.0, -1.0]], [[4.0, -2.0, 0.0], [4.0, -4.0, 4.0]], [[1.0, 1.0, -2.0], [3.0, 6.0, 6.0]], [[5.0, -5.0, -1.0], [4.0, -6.0, 4.0]], [[-7.0, 0.0, 0.0], [-3.0, -6.0, -2.0]]], [[[0.0, 7.0, -3.0], [-1.0, -4.0, 0.0]], [[2.0, 1.0, 2.0], [7.0, 0.0, 3.0]], [[3.0, 1.0, 0.0], [-6.0, 0.0, 7.0]], [[-4.0, -7.0, -4.0], [-2.0, 0.0, -7.0]], [[-6.0, -4.0, 5.0], [3.0, 6.0, -5.0]]], [[[-6.0, 5.0, 5.0], [0.0, 7.0, -3.0]], [[7.0, -1.0, 6.0], [7.0, 7.0, 7.0]], [[5.0, 5.0, 1.0], [0.0, 1.0, 6.0]], [[7.0, 4.0, 0.0], [2.0, 6.0, 5.0]], [[-1.0, -6.0, -1.0], [6.0, -3.0, -1.0]]], [[[-7.0, -7.0, -5.0], [-2.0, -3.0, 6.0]], [[4.0, 0.0, -6.0], [4.0, 0.0, 0.0]], [[3.0, 0.0, 5.0], [0.0, 0.0, 4.0]], [[6.0, -3.0, 3.0], [-6.0, 0.0, -7.0]], [[1.0, -4.0, -1.0], [7.0, -6.0, -3.0]]]]),self.functionspace)
      arg1=numpy.array([[[6.0, 0.0, -2.0], [6.0, 5.0, 5.0]], [[0.0, 5.0, 6.0], [0.0, 7.0, 6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-17.0, 14.0], [48.0, -14.0], [88.0, 71.0], [46.0, -49.0], [-100.0, -54.0]], [[-20.0, -11.0], [65.0, 35.0], [17.0, 47.0], [-63.0, -101.0], [-23.0, 22.0]], [[-26.0, 86.0], [142.0, 122.0], [63.0, 74.0], [109.0, 92.0], [12.0, -63.0]], [[-29.0, -50.0], [60.0, -36.0], [28.0, 54.0], [-41.0, -39.0], [5.0, -86.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_array_rank2_offset0(self):
      arg0=Data(numpy.array([[-4.0, -1.0, -3.0, 3.0, 6.0], [5.0, 1.0, -2.0, 0.0, -6.0], [-1.0, -6.0, 0.0, -3.0, 0.0], [5.0, -7.0, 4.0, 3.0, 0.0]]),self.functionspace)
      arg1=numpy.array([[0.0, -2.0, 5.0, 0.0, 3.0], [1.0, -3.0, -4.0, -2.0, 2.0], [4.0, 6.0, -4.0, 7.0, -1.0], [5.0, -7.0, -1.0, 4.0, 4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 8.0, -20.0, 0.0, -12.0], [-4.0, 12.0, 16.0, 8.0, -8.0], [-16.0, -24.0, 16.0, -28.0, 4.0], [-20.0, 28.0, 4.0, -16.0, -16.0]], [[0.0, 2.0, -5.0, 0.0, -3.0], [-1.0, 3.0, 4.0, 2.0, -2.0], [-4.0, -6.0, 4.0, -7.0, 1.0], [-5.0, 7.0, 1.0, -4.0, -4.0]], [[0.0, 6.0, -15.0, 0.0, -9.0], [-3.0, 9.0, 12.0, 6.0, -6.0], [-12.0, -18.0, 12.0, -21.0, 3.0], [-15.0, 21.0, 3.0, -12.0, -12.0]], [[0.0, -6.0, 15.0, 0.0, 9.0], [3.0, -9.0, -12.0, -6.0, 6.0], [12.0, 18.0, -12.0, 21.0, -3.0], [15.0, -21.0, -3.0, 12.0, 12.0]], [[0.0, -12.0, 30.0, 0.0, 18.0], [6.0, -18.0, -24.0, -12.0, 12.0], [24.0, 36.0, -24.0, 42.0, -6.0], [30.0, -42.0, -6.0, 24.0, 24.0]]], [[[0.0, -10.0, 25.0, 0.0, 15.0], [5.0, -15.0, -20.0, -10.0, 10.0], [20.0, 30.0, -20.0, 35.0, -5.0], [25.0, -35.0, -5.0, 20.0, 20.0]], [[0.0, -2.0, 5.0, 0.0, 3.0], [1.0, -3.0, -4.0, -2.0, 2.0], [4.0, 6.0, -4.0, 7.0, -1.0], [5.0, -7.0, -1.0, 4.0, 4.0]], [[0.0, 4.0, -10.0, 0.0, -6.0], [-2.0, 6.0, 8.0, 4.0, -4.0], [-8.0, -12.0, 8.0, -14.0, 2.0], [-10.0, 14.0, 2.0, -8.0, -8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 12.0, -30.0, 0.0, -18.0], [-6.0, 18.0, 24.0, 12.0, -12.0], [-24.0, -36.0, 24.0, -42.0, 6.0], [-30.0, 42.0, 6.0, -24.0, -24.0]]], [[[0.0, 2.0, -5.0, 0.0, -3.0], [-1.0, 3.0, 4.0, 2.0, -2.0], [-4.0, -6.0, 4.0, -7.0, 1.0], [-5.0, 7.0, 1.0, -4.0, -4.0]], [[0.0, 12.0, -30.0, 0.0, -18.0], [-6.0, 18.0, 24.0, 12.0, -12.0], [-24.0, -36.0, 24.0, -42.0, 6.0], [-30.0, 42.0, 6.0, -24.0, -24.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 6.0, -15.0, 0.0, -9.0], [-3.0, 9.0, 12.0, 6.0, -6.0], [-12.0, -18.0, 12.0, -21.0, 3.0], [-15.0, 21.0, 3.0, -12.0, -12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, -10.0, 25.0, 0.0, 15.0], [5.0, -15.0, -20.0, -10.0, 10.0], [20.0, 30.0, -20.0, 35.0, -5.0], [25.0, -35.0, -5.0, 20.0, 20.0]], [[0.0, 14.0, -35.0, 0.0, -21.0], [-7.0, 21.0, 28.0, 14.0, -14.0], [-28.0, -42.0, 28.0, -49.0, 7.0], [-35.0, 49.0, 7.0, -28.0, -28.0]], [[0.0, -8.0, 20.0, 0.0, 12.0], [4.0, -12.0, -16.0, -8.0, 8.0], [16.0, 24.0, -16.0, 28.0, -4.0], [20.0, -28.0, -4.0, 16.0, 16.0]], [[0.0, -6.0, 15.0, 0.0, 9.0], [3.0, -9.0, -12.0, -6.0, 6.0], [12.0, 18.0, -12.0, 21.0, -3.0], [15.0, -21.0, -3.0, 12.0, 12.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank3_offset1(self):
      arg0=Data(numpy.array([[[0.0, 1.0, -3.0], [5.0, -1.0, 0.0], [-2.0, 0.0, 0.0], [0.0, 0.0, 0.0], [0.0, 6.0, 7.0]], [[0.0, 5.0, 0.0], [3.0, 4.0, -6.0], [-3.0, 5.0, 6.0], [-7.0, 0.0, -6.0], [-3.0, -2.0, 3.0]], [[2.0, -3.0, 1.0], [-4.0, 7.0, -4.0], [-5.0, 6.0, 1.0], [-1.0, 3.0, -1.0], [-7.0, 2.0, -3.0]], [[0.0, 5.0, 4.0], [1.0, 1.0, 0.0], [4.0, 1.0, 6.0], [0.0, -4.0, -6.0], [6.0, -7.0, -4.0]]]),self.functionspace)
      arg1=numpy.array([[[6.0, 0.0, 4.0], [5.0, -4.0, 4.0], [-5.0, -7.0, 5.0], [-6.0, 3.0, -4.0], [-5.0, 4.0, 2.0]], [[6.0, 4.0, 1.0], [6.0, -7.0, 0.0], [-5.0, -1.0, -4.0], [4.0, -5.0, -6.0], [1.0, 7.0, 0.0]], [[-3.0, -5.0, 5.0], [5.0, -4.0, -2.0], [2.0, 6.0, -2.0], [-2.0, -6.0, 3.0], [-5.0, 5.0, -2.0]], [[-1.0, 1.0, 4.0], [0.0, -6.0, 5.0], [-4.0, -6.0, 5.0], [0.0, 5.0, 2.0], [-2.0, -1.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-12.0, -16.0, -22.0, 15.0, -2.0], [1.0, -7.0, 11.0, 13.0, 7.0], [-20.0, 2.0, 12.0, -15.0, 11.0], [-11.0, -21.0, -21.0, -1.0, 2.0]], [[30.0, 29.0, -18.0, -33.0, -29.0], [26.0, 37.0, -24.0, 25.0, -2.0], [-10.0, 29.0, 4.0, -4.0, -30.0], [-6.0, 6.0, -14.0, -5.0, -9.0]], [[-12.0, -10.0, 10.0, 12.0, 10.0], [-12.0, -12.0, 10.0, -8.0, -2.0], [6.0, -10.0, -4.0, 4.0, 10.0], [2.0, 0.0, 8.0, 0.0, 4.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[28.0, 4.0, -7.0, -10.0, 38.0], [31.0, -42.0, -34.0, -72.0, 42.0], [5.0, -38.0, 22.0, -15.0, 16.0], [34.0, -1.0, -1.0, 44.0, -13.0]]], [[[0.0, -20.0, -35.0, 15.0, 20.0], [20.0, -35.0, -5.0, -25.0, 35.0], [-25.0, -20.0, 30.0, -30.0, 25.0], [5.0, -30.0, -30.0, 25.0, -5.0]], [[-6.0, -25.0, -73.0, 18.0, -11.0], [28.0, -10.0, 5.0, 28.0, 31.0], [-59.0, 11.0, 42.0, -48.0, 17.0], [-23.0, -54.0, -66.0, 8.0, -4.0]], [[6.0, -11.0, 10.0, 9.0, 47.0], [8.0, -53.0, -14.0, -73.0, 32.0], [14.0, -47.0, 12.0, -6.0, 28.0], [32.0, 0.0, 12.0, 37.0, -5.0]], [[-66.0, -59.0, 5.0, 66.0, 23.0], [-48.0, -42.0, 59.0, 8.0, -7.0], [-9.0, -23.0, -2.0, -4.0, 47.0], [-17.0, -30.0, -2.0, -12.0, 20.0]], [[-6.0, 5.0, 44.0, 0.0, 13.0], [-23.0, -4.0, 5.0, -20.0, -17.0], [34.0, -13.0, -24.0, 27.0, -1.0], [13.0, 27.0, 39.0, -4.0, 5.0]]], [[[16.0, 26.0, 16.0, -25.0, -20.0], [1.0, 33.0, -11.0, 17.0, -19.0], [14.0, 20.0, -16.0, 17.0, -27.0], [-1.0, 23.0, 15.0, -13.0, -2.0]], [[-40.0, -64.0, -49.0, 61.0, 40.0], [0.0, -73.0, 29.0, -27.0, 45.0], [-43.0, -40.0, 42.0, -46.0, 63.0], [-5.0, -62.0, -46.0, 27.0, 5.0]], [[-26.0, -45.0, -12.0, 44.0, 51.0], [-5.0, -72.0, 15.0, -56.0, 37.0], [-10.0, -51.0, 24.0, -23.0, 53.0], [15.0, -31.0, -11.0, 32.0, 3.0]], [[-10.0, -21.0, -21.0, 19.0, 15.0], [5.0, -27.0, 6.0, -13.0, 20.0], [-17.0, -15.0, 18.0, -19.0, 22.0], [0.0, -23.0, -19.0, 13.0, 0.0]], [[-54.0, -55.0, 6.0, 60.0, 37.0], [-37.0, -56.0, 45.0, -20.0, 7.0], [-4.0, -37.0, 4.0, -7.0, 51.0], [-3.0, -27.0, 1.0, 4.0, 15.0]]], [[[16.0, -4.0, -15.0, -1.0, 28.0], [24.0, -35.0, -21.0, -49.0, 35.0], [-5.0, -28.0, 22.0, -18.0, 17.0], [21.0, -10.0, -10.0, 33.0, -9.0]], [[6.0, 1.0, -12.0, -3.0, -1.0], [10.0, -1.0, -6.0, -1.0, 8.0], [-8.0, 1.0, 8.0, -8.0, 0.0], [0.0, -6.0, -10.0, 5.0, -3.0]], [[48.0, 40.0, 3.0, -45.0, -4.0], [34.0, 17.0, -45.0, -25.0, 11.0], [13.0, 4.0, 2.0, 4.0, -27.0], [21.0, 24.0, 8.0, 17.0, -15.0]], [[-24.0, -8.0, -2.0, 12.0, -28.0], [-22.0, 28.0, 28.0, 56.0, -28.0], [-10.0, 28.0, -12.0, 6.0, -8.0], [-28.0, -6.0, -6.0, -32.0, 10.0]], [[20.0, 42.0, -1.0, -41.0, -66.0], [4.0, 85.0, -7.0, 83.0, -43.0], [-3.0, 66.0, -22.0, 18.0, -57.0], [-29.0, 22.0, -2.0, -43.0, -1.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank4_offset2(self):
      arg0=Data(numpy.array([[[[-2.0, 6.0, 1.0], [6.0, 1.0, 5.0]], [[-5.0, 3.0, 0.0], [4.0, 3.0, 7.0]], [[-7.0, -7.0, 1.0], [7.0, 0.0, -7.0]], [[-1.0, 6.0, -5.0], [-5.0, 3.0, -7.0]], [[-3.0, -1.0, 7.0], [-5.0, -4.0, 0.0]]], [[[-1.0, -2.0, -1.0], [3.0, 5.0, 3.0]], [[-6.0, -2.0, 6.0], [2.0, -7.0, 7.0]], [[0.0, 6.0, -4.0], [3.0, -5.0, 1.0]], [[3.0, -5.0, 7.0], [-4.0, 7.0, -2.0]], [[-6.0, 5.0, -1.0], [-4.0, 4.0, 3.0]]], [[[4.0, 0.0, -5.0], [4.0, 7.0, 4.0]], [[-6.0, 3.0, 5.0], [0.0, 4.0, -5.0]], [[-7.0, 4.0, -3.0], [-5.0, 4.0, -3.0]], [[-7.0, -7.0, -6.0], [7.0, 6.0, 0.0]], [[-6.0, 4.0, 4.0], [2.0, -5.0, -7.0]]], [[[7.0, 0.0, 4.0], [-3.0, 0.0, 3.0]], [[-2.0, -5.0, 3.0], [0.0, -3.0, -6.0]], [[-6.0, 7.0, -1.0], [-7.0, -2.0, 2.0]], [[1.0, 5.0, 4.0], [7.0, 6.0, -3.0]], [[0.0, 3.0, -3.0], [0.0, -6.0, -3.0]]]]),self.functionspace)
      arg1=numpy.array([[[[0.0, -2.0, -1.0], [-2.0, 0.0, -2.0]], [[1.0, -3.0, -1.0], [3.0, -7.0, -3.0]], [[-1.0, -7.0, 0.0], [2.0, -2.0, 1.0]], [[-2.0, 4.0, 2.0], [-3.0, 7.0, 7.0]], [[3.0, 1.0, -6.0], [-3.0, 0.0, 1.0]]], [[[4.0, 5.0, -4.0], [-3.0, -2.0, 0.0]], [[-4.0, 3.0, -7.0], [-6.0, -4.0, 6.0]], [[-4.0, 3.0, -4.0], [2.0, 0.0, 7.0]], [[5.0, -3.0, 6.0], [-3.0, -5.0, 1.0]], [[0.0, 7.0, -4.0], [5.0, 5.0, 5.0]]], [[[-3.0, 0.0, 0.0], [4.0, 3.0, -2.0]], [[0.0, 4.0, 3.0], [-1.0, -2.0, 1.0]], [[6.0, -5.0, -2.0], [-5.0, 0.0, 6.0]], [[0.0, 3.0, 6.0], [-5.0, -5.0, -6.0]], [[-2.0, -1.0, -1.0], [4.0, 6.0, 2.0]]], [[[4.0, -3.0, 7.0], [-1.0, 5.0, 3.0]], [[-4.0, 4.0, -3.0], [4.0, -5.0, 0.0]], [[-6.0, 2.0, -7.0], [5.0, 7.0, 6.0]], [[5.0, -6.0, -7.0], [3.0, 4.0, -2.0]], [[-7.0, 5.0, 5.0], [7.0, -3.0, 1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-35.0, -25.0, -25.0, 54.0, -19.0], [-2.0, 9.0, 69.0, -40.0, 98.0], [23.0, 24.0, -44.0, -41.0, 37.0], [-5.0, 48.0, 84.0, -41.0, 93.0]], [[-28.0, -44.0, -7.0, 80.0, -17.0], [-23.0, 35.0, 86.0, -54.0, 91.0], [26.0, 9.0, -23.0, -68.0, 55.0], [3.0, 33.0, 119.0, -33.0, 76.0]], [[13.0, 55.0, 63.0, -82.0, -62.0], [-88.0, -84.0, -32.0, -36.0, -53.0], [63.0, -39.0, -86.0, -8.0, 34.0], [-28.0, 25.0, 14.0, 35.0, 61.0]], [[17.0, -29.0, -64.0, 3.0, 41.0], [55.0, 33.0, -17.0, -60.0, 17.0], [6.0, 1.0, -43.0, 40.0, -15.0], [-58.0, 8.0, 7.0, 5.0, -39.0]], [[5.0, 6.0, 8.0, 3.0, -37.0], [-22.0, 6.0, -29.0, 65.0, -80.0], [-23.0, 30.0, -2.0, 84.0, -44.0], [25.0, -13.0, -86.0, -89.0, 28.0]]], [[[-7.0, -29.0, 14.0, 39.0, -5.0], [-29.0, -15.0, 29.0, -36.0, 45.0], [24.0, -21.0, 9.0, -70.0, 53.0], [26.0, -14.0, 77.0, 37.0, 1.0]], [[-20.0, 28.0, 45.0, 10.0, -55.0], [-50.0, 34.0, 47.0, 48.0, -28.0], [-9.0, 29.0, -6.0, 13.0, -12.0], [8.0, 41.0, -7.0, -96.0, 104.0]], [[-16.0, 27.0, -25.0, -21.0, 22.0], [47.0, 54.0, 47.0, -25.0, 53.0], [-5.0, 20.0, -31.0, -2.0, -18.0], [-71.0, 73.0, 26.0, -21.0, 47.0]], [[15.0, -44.0, 8.0, 35.0, -28.0], [-43.0, -92.0, -77.0, 47.0, -58.0], [0.0, -11.0, 37.0, 24.0, 14.0], [109.0, -104.0, -60.0, 16.0, -62.0]], [[-7.0, -69.0, -42.0, 91.0, 8.0], [9.0, 72.0, 56.0, -56.0, 54.0], [8.0, 16.0, -21.0, -9.0, 22.0], [-13.0, 11.0, 79.0, -55.0, 25.0]]], [[[-11.0, -40.0, -6.0, 47.0, 34.0], [10.0, -9.0, 40.0, -53.0, 95.0], [17.0, -29.0, 38.0, -109.0, 63.0], [24.0, -20.0, 104.0, 87.0, -42.0]], [[-1.0, -33.0, -28.0, 27.0, -50.0], [-37.0, -48.0, -22.0, -34.0, -4.0], [40.0, 14.0, -91.0, 49.0, 18.0], [7.0, 1.0, 5.0, -57.0, 65.0]], [[11.0, -50.0, -42.0, 46.0, 13.0], [11.0, 57.0, 21.0, -73.0, 20.0], [19.0, 1.0, -49.0, 17.0, 11.0], [-45.0, 13.0, 56.0, -31.0, 4.0]], [[6.0, -1.0, 58.0, -5.0, -13.0], [-72.0, -17.0, 45.0, -101.0, 40.0], [67.0, -65.0, -30.0, -122.0, 91.0], [-26.0, 16.0, 147.0, 94.0, 15.0]], [[-2.0, 40.0, -15.0, -54.0, -51.0], [-16.0, -26.0, -25.0, -6.0, -38.0], [25.0, 29.0, -116.0, 93.0, -32.0], [-56.0, 61.0, -51.0, -82.0, 104.0]]], [[[-4.0, -15.0, -10.0, 24.0, 9.0], [21.0, -20.0, -29.0, 71.0, -16.0], [-39.0, 18.0, 67.0, 21.0, -24.0], [68.0, -52.0, -67.0, -8.0, -47.0]], [[19.0, 49.0, 37.0, -73.0, -35.0], [-39.0, -52.0, -61.0, 32.0, -92.0], [9.0, -11.0, -29.0, 54.0, -24.0], [-5.0, -6.0, -76.0, -1.0, 7.0]], [[-3.0, -39.0, -51.0, 59.0, 18.0], [40.0, 114.0, 49.0, -24.0, 18.0], [-20.0, 38.0, -22.0, 48.0, -30.0], [-49.0, 37.0, 20.0, -98.0, 31.0]], [[-22.0, -30.0, -37.0, 26.0, -40.0], [-20.0, -101.0, -12.0, -40.0, 69.0], [49.0, 10.0, -80.0, -8.0, 47.0], [31.0, 2.0, 35.0, -2.0, 66.0]], [[3.0, 45.0, -12.0, -57.0, 18.0], [39.0, 36.0, 0.0, 0.0, -12.0], [-12.0, 12.0, -27.0, 39.0, -42.0], [-69.0, 51.0, -33.0, -15.0, 15.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[-5.0, 6.0, 3.0, 7.0, -4.0], [-2.0, 3.0, 3.0, 3.0, 0.0], [7.0, -3.0, 4.0, -4.0, -2.0], [-6.0, -5.0, -5.0, 0.0, -4.0]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(7.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[-35.0, 42.0, 21.0, 49.0, -28.0], [-14.0, 21.0, 21.0, 21.0, 0.0], [49.0, -21.0, 28.0, -28.0, -14.0], [-42.0, -35.0, -35.0, 0.0, -28.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[-1.0, -4.0, 6.0], [0.0, -2.0, 6.0], [0.0, -6.0, -5.0], [1.0, 2.0, 3.0], [2.0, 2.0, -5.0]], [[7.0, 7.0, 5.0], [-4.0, 5.0, 0.0], [3.0, 0.0, 0.0], [-1.0, -1.0, 5.0], [-4.0, 2.0, -2.0]], [[4.0, -1.0, -5.0], [-7.0, 0.0, -1.0], [-2.0, -5.0, 0.0], [-2.0, -5.0, 0.0], [6.0, 2.0, -6.0]], [[6.0, -7.0, -6.0], [-3.0, 6.0, -3.0], [-5.0, 6.0, 4.0], [-3.0, 1.0, 6.0], [2.0, -3.0, 2.0]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-3.0, -4.0, -3.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[1.0, -10.0, 39.0, -20.0, 1.0], [-64.0, -8.0, -9.0, -8.0, 10.0], [7.0, 24.0, 26.0, 26.0, -8.0], [28.0, -6.0, -21.0, -13.0, 0.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank2_offset2(self):
      arg0=Data(numpy.array([[[[-5.0, 5.0, -4.0], [-7.0, 2.0, -1.0]], [[1.0, -7.0, 2.0], [7.0, -1.0, -6.0]], [[-5.0, -3.0, 1.0], [6.0, -6.0, -6.0]], [[2.0, -7.0, 6.0], [5.0, -6.0, 2.0]], [[-6.0, 3.0, 0.0], [3.0, 5.0, 7.0]]], [[[5.0, -1.0, -1.0], [5.0, 2.0, 4.0]], [[-1.0, -2.0, 6.0], [-4.0, 0.0, 1.0]], [[2.0, -2.0, 0.0], [-5.0, 6.0, 5.0]], [[-1.0, -7.0, 0.0], [7.0, 0.0, 0.0]], [[-2.0, 3.0, -6.0], [-1.0, 1.0, 5.0]]], [[[-5.0, 3.0, -7.0], [7.0, -6.0, -1.0]], [[-3.0, -6.0, -4.0], [6.0, 6.0, -6.0]], [[5.0, -6.0, -7.0], [2.0, -1.0, 3.0]], [[-1.0, 0.0, 0.0], [-6.0, -4.0, -3.0]], [[4.0, 0.0, -7.0], [2.0, -1.0, -3.0]]], [[[1.0, -3.0, -6.0], [-4.0, 4.0, -1.0]], [[-7.0, -7.0, -7.0], [4.0, 2.0, -4.0]], [[0.0, 0.0, 0.0], [-2.0, -6.0, 5.0]], [[2.0, -7.0, -6.0], [-5.0, 0.0, -2.0]], [[0.0, 2.0, -1.0], [0.0, -2.0, 6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[1.0, 1.0, 0.0], [-5.0, 7.0, -6.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[55.0, -12.0, -44.0, -84.0, -25.0], [-31.0, 11.0, 37.0, -43.0, -17.0], [-73.0, 39.0, -36.0, 19.0, 5.0], [52.0, 4.0, -62.0, 32.0, -48.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[-6.0, 3.0, 0.0, 2.0, -5.0], [3.0, 0.0, 4.0, -3.0, -2.0], [0.0, -5.0, -2.0, -2.0, -5.0], [-5.0, 1.0, 2.0, 3.0, -3.0]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-1.0, 1.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[6.0, -6.0], [-3.0, 3.0], [0.0, 0.0], [-2.0, 2.0], [5.0, -5.0]], [[-3.0, 3.0], [0.0, 0.0], [-4.0, 4.0], [3.0, -3.0], [2.0, -2.0]], [[0.0, 0.0], [5.0, -5.0], [2.0, -2.0], [2.0, -2.0], [5.0, -5.0]], [[5.0, -5.0], [-1.0, 1.0], [-2.0, 2.0], [-3.0, 3.0], [3.0, -3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[-7.0, 6.0, 3.0], [4.0, 5.0, 5.0], [-6.0, 3.0, 2.0], [-4.0, 3.0, 0.0], [-2.0, 1.0, 3.0]], [[-5.0, -6.0, -1.0], [1.0, 5.0, 4.0], [-7.0, 0.0, 1.0], [-7.0, 2.0, 6.0], [0.0, 0.0, -1.0]], [[1.0, -2.0, 6.0], [3.0, 4.0, -1.0], [-3.0, 7.0, -4.0], [-7.0, 0.0, -2.0], [7.0, 1.0, 0.0]], [[0.0, 1.0, 7.0], [-2.0, 2.0, 3.0], [0.0, 4.0, -4.0], [-7.0, -6.0, 5.0], [6.0, 1.0, 3.0]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-7.0, -1.0, -3.0], [0.0, 4.0, 7.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[34.0, 45.0], [-48.0, 55.0], [33.0, 26.0], [25.0, 12.0], [4.0, 25.0]], [[44.0, -31.0], [-24.0, 48.0], [46.0, 7.0], [29.0, 50.0], [3.0, -7.0]], [[-23.0, 34.0], [-22.0, 9.0], [26.0, 0.0], [55.0, -14.0], [-50.0, 4.0]], [[-22.0, 53.0], [3.0, 29.0], [8.0, -12.0], [40.0, 11.0], [-52.0, 25.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank3_offset2(self):
      arg0=Data(numpy.array([[[[1.0, 6.0, 5.0], [-2.0, -5.0, -3.0]], [[-2.0, 4.0, -6.0], [-1.0, 2.0, 5.0]], [[0.0, 0.0, 1.0], [-5.0, 3.0, 4.0]], [[1.0, -3.0, 6.0], [-1.0, 0.0, 3.0]], [[-3.0, -5.0, 5.0], [-3.0, -1.0, -5.0]]], [[[0.0, 6.0, 7.0], [2.0, 0.0, -3.0]], [[-7.0, 0.0, -4.0], [-1.0, 5.0, -3.0]], [[2.0, -7.0, 1.0], [6.0, 7.0, 3.0]], [[-6.0, -1.0, 0.0], [5.0, 1.0, -1.0]], [[3.0, -6.0, 6.0], [1.0, 6.0, -2.0]]], [[[0.0, -5.0, -4.0], [6.0, -5.0, 3.0]], [[7.0, -5.0, -4.0], [5.0, 1.0, -2.0]], [[-4.0, 0.0, 0.0], [-7.0, 0.0, 3.0]], [[0.0, 1.0, -5.0], [3.0, 3.0, 5.0]], [[-3.0, -6.0, 6.0], [3.0, 4.0, 7.0]]], [[[-1.0, 7.0, -5.0], [-5.0, 7.0, -4.0]], [[2.0, 2.0, -6.0], [3.0, 5.0, -1.0]], [[-1.0, 3.0, 1.0], [-6.0, 6.0, 5.0]], [[6.0, 6.0, 2.0], [7.0, 1.0, 1.0]], [[0.0, -4.0, -3.0], [5.0, 3.0, -1.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, -7.0, 0.0], [1.0, -3.0, 3.0]], [[-4.0, 6.0, -1.0], [-6.0, 0.0, -5.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-38.0, 54.0], [-20.0, 19.0], [-2.0, 9.0], [29.0, -37.0], [20.0, 20.0]], [[-49.0, 32.0], [-25.0, 53.0], [43.0, -102.0], [6.0, -7.0], [19.0, -50.0]], [[65.0, -77.0], [31.0, -74.0], [2.0, 43.0], [2.0, -32.0], [54.0, -83.0]], [[-87.0, 101.0], [-29.0, -3.0], [-30.0, 32.0], [-35.0, -37.0], [21.0, -46.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_Symbol_rank2_offset0(self):
      arg0=Data(numpy.array([[-1.0, -1.0, 3.0, -4.0, -1.0], [6.0, 6.0, 4.0, 7.0, 2.0], [-7.0, -6.0, -7.0, 0.0, 6.0], [6.0, 2.0, 7.0, -1.0, 0.0]]),self.functionspace)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-2.0, -7.0, 7.0, 7.0, 1.0], [4.0, 3.0, -5.0, 0.0, 5.0], [3.0, -7.0, 0.0, 2.0, 0.0], [-7.0, 7.0, -2.0, 6.0, -2.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[2.0, 7.0, -7.0, -7.0, -1.0], [-4.0, -3.0, 5.0, 0.0, -5.0], [-3.0, 7.0, 0.0, -2.0, 0.0], [7.0, -7.0, 2.0, -6.0, 2.0]], [[2.0, 7.0, -7.0, -7.0, -1.0], [-4.0, -3.0, 5.0, 0.0, -5.0], [-3.0, 7.0, 0.0, -2.0, 0.0], [7.0, -7.0, 2.0, -6.0, 2.0]], [[-6.0, -21.0, 21.0, 21.0, 3.0], [12.0, 9.0, -15.0, 0.0, 15.0], [9.0, -21.0, 0.0, 6.0, 0.0], [-21.0, 21.0, -6.0, 18.0, -6.0]], [[8.0, 28.0, -28.0, -28.0, -4.0], [-16.0, -12.0, 20.0, 0.0, -20.0], [-12.0, 28.0, 0.0, -8.0, 0.0], [28.0, -28.0, 8.0, -24.0, 8.0]], [[2.0, 7.0, -7.0, -7.0, -1.0], [-4.0, -3.0, 5.0, 0.0, -5.0], [-3.0, 7.0, 0.0, -2.0, 0.0], [7.0, -7.0, 2.0, -6.0, 2.0]]], [[[-12.0, -42.0, 42.0, 42.0, 6.0], [24.0, 18.0, -30.0, 0.0, 30.0], [18.0, -42.0, 0.0, 12.0, 0.0], [-42.0, 42.0, -12.0, 36.0, -12.0]], [[-12.0, -42.0, 42.0, 42.0, 6.0], [24.0, 18.0, -30.0, 0.0, 30.0], [18.0, -42.0, 0.0, 12.0, 0.0], [-42.0, 42.0, -12.0, 36.0, -12.0]], [[-8.0, -28.0, 28.0, 28.0, 4.0], [16.0, 12.0, -20.0, 0.0, 20.0], [12.0, -28.0, 0.0, 8.0, 0.0], [-28.0, 28.0, -8.0, 24.0, -8.0]], [[-14.0, -49.0, 49.0, 49.0, 7.0], [28.0, 21.0, -35.0, 0.0, 35.0], [21.0, -49.0, 0.0, 14.0, 0.0], [-49.0, 49.0, -14.0, 42.0, -14.0]], [[-4.0, -14.0, 14.0, 14.0, 2.0], [8.0, 6.0, -10.0, 0.0, 10.0], [6.0, -14.0, 0.0, 4.0, 0.0], [-14.0, 14.0, -4.0, 12.0, -4.0]]], [[[14.0, 49.0, -49.0, -49.0, -7.0], [-28.0, -21.0, 35.0, 0.0, -35.0], [-21.0, 49.0, 0.0, -14.0, 0.0], [49.0, -49.0, 14.0, -42.0, 14.0]], [[12.0, 42.0, -42.0, -42.0, -6.0], [-24.0, -18.0, 30.0, 0.0, -30.0], [-18.0, 42.0, 0.0, -12.0, 0.0], [42.0, -42.0, 12.0, -36.0, 12.0]], [[14.0, 49.0, -49.0, -49.0, -7.0], [-28.0, -21.0, 35.0, 0.0, -35.0], [-21.0, 49.0, 0.0, -14.0, 0.0], [49.0, -49.0, 14.0, -42.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-12.0, -42.0, 42.0, 42.0, 6.0], [24.0, 18.0, -30.0, 0.0, 30.0], [18.0, -42.0, 0.0, 12.0, 0.0], [-42.0, 42.0, -12.0, 36.0, -12.0]]], [[[-12.0, -42.0, 42.0, 42.0, 6.0], [24.0, 18.0, -30.0, 0.0, 30.0], [18.0, -42.0, 0.0, 12.0, 0.0], [-42.0, 42.0, -12.0, 36.0, -12.0]], [[-4.0, -14.0, 14.0, 14.0, 2.0], [8.0, 6.0, -10.0, 0.0, 10.0], [6.0, -14.0, 0.0, 4.0, 0.0], [-14.0, 14.0, -4.0, 12.0, -4.0]], [[-14.0, -49.0, 49.0, 49.0, 7.0], [28.0, 21.0, -35.0, 0.0, 35.0], [21.0, -49.0, 0.0, 14.0, 0.0], [-49.0, 49.0, -14.0, 42.0, -14.0]], [[2.0, 7.0, -7.0, -7.0, -1.0], [-4.0, -3.0, 5.0, 0.0, -5.0], [-3.0, 7.0, 0.0, -2.0, 0.0], [7.0, -7.0, 2.0, -6.0, 2.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank3_offset1(self):
      arg0=Data(numpy.array([[[4.0, 6.0, 4.0], [5.0, -2.0, 5.0], [-2.0, 4.0, -5.0], [-4.0, -6.0, 2.0], [-4.0, -3.0, -6.0]], [[0.0, 7.0, 5.0], [-2.0, 0.0, 5.0], [-5.0, 6.0, -1.0], [5.0, 6.0, 0.0], [3.0, 3.0, -4.0]], [[-1.0, -2.0, -5.0], [5.0, 0.0, -6.0], [7.0, 6.0, -2.0], [-7.0, -5.0, 5.0], [-4.0, 7.0, 0.0]], [[2.0, -3.0, 4.0], [0.0, -2.0, 7.0], [-1.0, 1.0, 2.0], [-6.0, 1.0, 2.0], [0.0, 5.0, 7.0]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[1.0, 1.0, -5.0], [1.0, 0.0, -4.0], [-1.0, -6.0, -1.0], [-1.0, 3.0, -1.0], [-5.0, 0.0, -6.0]], [[-2.0, 4.0, 1.0], [7.0, 2.0, 6.0], [2.0, 2.0, 7.0], [-4.0, 2.0, -3.0], [-1.0, 7.0, 1.0]], [[5.0, -6.0, -7.0], [-6.0, 4.0, 7.0], [5.0, 6.0, -4.0], [6.0, -3.0, 7.0], [-3.0, 6.0, -7.0]], [[4.0, 6.0, 4.0], [0.0, -7.0, -1.0], [-7.0, -1.0, -4.0], [4.0, 7.0, 0.0], [1.0, 0.0, 6.0]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-10.0, -12.0, -44.0, 10.0, -44.0], [20.0, 64.0, 48.0, -16.0, 42.0], [-44.0, 28.0, 40.0, 34.0, -4.0], [68.0, -46.0, -50.0, 58.0, 28.0]], [[-22.0, -15.0, 2.0, -16.0, -55.0], [-13.0, 61.0, 41.0, -39.0, -14.0], [2.0, -3.0, -7.0, 71.0, -62.0], [28.0, 9.0, -53.0, 6.0, 35.0]], [[27.0, 18.0, -17.0, 19.0, 40.0], [15.0, -36.0, -31.0, 31.0, 25.0], [1.0, -7.0, 34.0, -59.0, 65.0], [-4.0, -23.0, 30.0, 20.0, -32.0]], [[-20.0, -12.0, 38.0, -16.0, 8.0], [-14.0, -28.0, -6.0, -2.0, -36.0], [2.0, 14.0, -64.0, 8.0, -38.0], [-44.0, 40.0, 26.0, -58.0, 8.0]], [[23.0, 20.0, 28.0, 1.0, 56.0], [-10.0, -70.0, -56.0, 28.0, -23.0], [40.0, -30.0, -14.0, -57.0, 36.0], [-58.0, 27.0, 55.0, -37.0, -40.0]]], [[[-18.0, -20.0, -47.0, 16.0, -30.0], [33.0, 44.0, 49.0, -1.0, 54.0], [-77.0, 63.0, 22.0, 14.0, 7.0], [62.0, -54.0, -27.0, 49.0, 30.0]], [[-27.0, -22.0, -3.0, -3.0, -20.0], [9.0, 16.0, 31.0, -7.0, 7.0], [-45.0, 47.0, -30.0, 23.0, -29.0], [12.0, -5.0, -6.0, -8.0, 28.0]], [[6.0, -1.0, -30.0, 24.0, 31.0], [33.0, -29.0, -5.0, 35.0, 46.0], [-54.0, 47.0, 15.0, -55.0, 58.0], [12.0, -41.0, 33.0, 22.0, -11.0]], [[11.0, 5.0, -41.0, 13.0, -25.0], [14.0, 47.0, 22.0, -8.0, 37.0], [-11.0, -6.0, 61.0, 12.0, 21.0], [56.0, -42.0, -41.0, 62.0, 5.0]], [[26.0, 19.0, -17.0, 10.0, 9.0], [2.0, 3.0, -16.0, 6.0, 14.0], [25.0, -34.0, 49.0, -19.0, 37.0], [14.0, -17.0, -8.0, 33.0, -21.0]]], [[[22.0, 19.0, 18.0, 0.0, 35.0], [-11.0, -41.0, -41.0, 15.0, -18.0], [42.0, -37.0, 3.0, -35.0, 26.0], [-36.0, 19.0, 29.0, -18.0, -31.0]], [[35.0, 29.0, 1.0, 1.0, 11.0], [-16.0, -1.0, -32.0, -2.0, -11.0], [67.0, -72.0, 49.0, -12.0, 27.0], [-4.0, 6.0, -11.0, 20.0, -31.0]], [[23.0, 15.0, -41.0, 13.0, -23.0], [8.0, 49.0, 12.0, -10.0, 33.0], [13.0, -32.0, 79.0, 10.0, 29.0], [56.0, -40.0, -47.0, 70.0, -5.0]], [[-37.0, -27.0, 32.0, -13.0, 5.0], [-1.0, -29.0, 11.0, 3.0, -23.0], [-40.0, 57.0, -85.0, 8.0, -44.0], [-38.0, 30.0, 34.0, -63.0, 23.0]], [[3.0, -4.0, -38.0, 25.0, 20.0], [36.0, -14.0, 6.0, 30.0, 53.0], [-62.0, 52.0, 22.0, -45.0, 54.0], [26.0, -49.0, 21.0, 33.0, -4.0]]], [[[-21.0, -14.0, 12.0, -15.0, -34.0], [-12.0, 32.0, 26.0, -26.0, -19.0], [0.0, 4.0, -24.0, 49.0, -52.0], [6.0, 17.0, -27.0, -13.0, 26.0]], [[-37.0, -28.0, 5.0, -13.0, -42.0], [-1.0, 38.0, 45.0, -25.0, -7.0], [-37.0, 41.0, -40.0, 55.0, -61.0], [16.0, 7.0, -26.0, -14.0, 42.0]], [[-10.0, -9.0, -7.0, 2.0, -7.0], [8.0, 7.0, 14.0, 0.0, 10.0], [-25.0, 24.0, -7.0, 5.0, -5.0], [10.0, -9.0, -2.0, 3.0, 11.0]], [[-15.0, -14.0, -2.0, 7.0, 18.0], [18.0, -28.0, 4.0, 20.0, 15.0], [-50.0, 54.0, -32.0, -25.0, 10.0], [-10.0, -9.0, 33.0, -17.0, 6.0]], [[-30.0, -28.0, -37.0, 8.0, -42.0], [27.0, 52.0, 59.0, -11.0, 42.0], [-79.0, 69.0, 2.0, 34.0, -19.0], [58.0, -42.0, -33.0, 35.0, 42.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank4_offset2(self):
      arg0=Data(numpy.array([[[[0.0, 0.0, 0.0], [-2.0, 2.0, 3.0]], [[5.0, 4.0, 5.0], [-6.0, 5.0, 0.0]], [[-7.0, 6.0, 0.0], [-4.0, 0.0, 7.0]], [[-3.0, -6.0, -5.0], [-6.0, 3.0, 1.0]], [[0.0, 1.0, -1.0], [-5.0, -5.0, 4.0]]], [[[-6.0, -7.0, 5.0], [-2.0, 5.0, 5.0]], [[0.0, -2.0, -7.0], [2.0, 1.0, 0.0]], [[-3.0, 3.0, -7.0], [7.0, 0.0, 6.0]], [[0.0, 1.0, 0.0], [7.0, 4.0, -1.0]], [[0.0, 6.0, 3.0], [6.0, -4.0, -6.0]]], [[[-4.0, 3.0, 6.0], [7.0, 5.0, -2.0]], [[-5.0, 5.0, -6.0], [-4.0, 1.0, 7.0]], [[4.0, 0.0, -1.0], [7.0, -2.0, 5.0]], [[-7.0, -6.0, 6.0], [7.0, 5.0, -2.0]], [[7.0, 2.0, 2.0], [6.0, 1.0, 0.0]]], [[[3.0, 4.0, 0.0], [2.0, 7.0, -4.0]], [[-7.0, 0.0, -3.0], [-6.0, -1.0, 2.0]], [[-6.0, 0.0, -3.0], [-6.0, 3.0, 4.0]], [[-4.0, -1.0, -7.0], [6.0, -3.0, 0.0]], [[2.0, 4.0, -1.0], [-5.0, 5.0, -2.0]]]]),self.functionspace)
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[5.0, 7.0, 2.0], [7.0, 1.0, -7.0]], [[-4.0, 3.0, 2.0], [1.0, -3.0, -3.0]], [[-1.0, 6.0, -2.0], [-5.0, -4.0, -4.0]], [[-6.0, -2.0, -3.0], [2.0, 2.0, -7.0]], [[-5.0, 3.0, 7.0], [-4.0, 0.0, -1.0]]], [[[-2.0, -3.0, 6.0], [-5.0, 1.0, 7.0]], [[-5.0, 0.0, -3.0], [3.0, -4.0, 7.0]], [[-3.0, -2.0, -1.0], [-1.0, -5.0, 2.0]], [[4.0, 0.0, 7.0], [3.0, -6.0, 2.0]], [[6.0, -2.0, 4.0], [-2.0, 7.0, 4.0]]], [[[-5.0, -1.0, 2.0], [5.0, -3.0, 4.0]], [[0.0, -4.0, 1.0], [7.0, 7.0, 1.0]], [[-7.0, -1.0, 5.0], [6.0, 7.0, 0.0]], [[-3.0, -3.0, 0.0], [0.0, -3.0, -4.0]], [[-6.0, 3.0, 5.0], [-5.0, 0.0, -5.0]]], [[[-1.0, -2.0, -1.0], [-4.0, 0.0, -6.0]], [[-2.0, 5.0, 6.0], [0.0, 0.0, -3.0]], [[1.0, 0.0, 0.0], [1.0, -5.0, 0.0]], [[2.0, 0.0, 0.0], [-4.0, -7.0, -4.0]], [[-6.0, -1.0, 3.0], [4.0, -6.0, -7.0]]]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[-33.0, -17.0, -10.0, -21.0, 5.0], [33.0, 7.0, -2.0, -12.0, 30.0], [-4.0, 3.0, 2.0, -18.0, -5.0], [-10.0, -9.0, -12.0, -18.0, -41.0]], [[26.0, -19.0, 19.0, -55.0, 46.0], [43.0, -78.0, -47.0, 7.0, 89.0], [-64.0, -18.0, -15.0, -42.0, 37.0], [6.0, 40.0, -26.0, -1.0, -73.0]], [[-70.0, 21.0, 35.0, -27.0, 62.0], [65.0, 72.0, 27.0, -26.0, -18.0], [37.0, -45.0, 19.0, -25.0, 45.0], [-31.0, 23.0, -11.0, -26.0, -29.0]], [[-113.0, -34.0, -9.0, 32.0, -15.0], [34.0, 7.0, 19.0, -81.0, 11.0], [-24.0, -1.0, -13.0, 14.0, 0.0], [38.0, -57.0, -24.0, -7.0, -40.0]], [[-63.0, -1.0, 37.0, -47.0, 12.0], [39.0, 36.0, 37.0, 16.0, -15.0], [3.0, -71.0, -71.0, -4.0, 3.0], [-5.0, -13.0, 20.0, 39.0, -22.0]]], [[[-113.0, -19.0, -76.0, 6.0, 47.0], [113.0, 24.0, 14.0, -15.0, 57.0], [42.0, 59.0, 97.0, 4.0, 25.0], [-7.0, -8.0, -33.0, -59.0, -15.0]], [[-13.0, -21.0, -12.0, 31.0, -63.0], [-45.0, 23.0, 4.0, -49.0, -21.0], [-5.0, 22.0, -14.0, 3.0, -51.0], [3.0, -52.0, -3.0, -15.0, -17.0]], [[-1.0, -4.0, -24.0, 5.0, -59.0], [-38.0, 99.0, 15.0, -28.0, -42.0], [57.0, 36.0, 25.0, -24.0, -73.0], [-60.0, -39.0, 4.0, -58.0, -20.0]], [[67.0, 1.0, -41.0, 27.0, -24.0], [-41.0, -2.0, -31.0, -5.0, 8.0], [18.0, 72.0, 69.0, -11.0, -27.0], [-24.0, 8.0, -13.0, -52.0, 10.0]], [[128.0, 60.0, 40.0, 25.0, 21.0], [-76.0, -17.0, -13.0, 51.0, -64.0], [18.0, -13.0, 17.0, 18.0, 33.0], [-3.0, 66.0, 26.0, 28.0, 93.0]]], [[[81.0, 35.0, -37.0, 38.0, 45.0], [-9.0, -11.0, -36.0, 13.0, 7.0], [41.0, 76.0, 132.0, -4.0, 38.0], [-24.0, 65.0, -22.0, -63.0, 51.0]], [[-78.0, -5.0, 35.0, -17.0, 7.0], [29.0, 76.0, 24.0, -66.0, -21.0], [13.0, -40.0, -17.0, -31.0, 0.0], [-25.0, -22.0, -14.0, -29.0, -64.0]], [[30.0, -20.0, -49.0, -46.0, -60.0], [-16.0, 47.0, 2.0, 52.0, 12.0], [39.0, 39.0, -5.0, -26.0, -89.0], [-61.0, -29.0, 21.0, -26.0, -22.0]], [[3.0, 20.0, -88.0, 74.0, 33.0], [24.0, 4.0, -9.0, 1.0, 7.0], [65.0, 112.0, 162.0, 32.0, 29.0], [-3.0, 26.0, -25.0, -69.0, 78.0]], [[96.0, -15.0, -33.0, -38.0, -39.0], [-37.0, -27.0, -38.0, 54.0, 41.0], [-6.0, 43.0, 2.0, -30.0, -56.0], [-37.0, 8.0, 8.0, -17.0, -20.0]]], [[[92.0, -7.0, -1.0, 20.0, -7.0], [-49.0, -65.0, -62.0, -32.0, 39.0], [-46.0, 43.0, 36.0, -26.0, 4.0], [5.0, 26.0, -30.0, -35.0, -28.0]], [[-98.0, 13.0, 39.0, 23.0, 36.0], [39.0, 44.0, 39.0, -57.0, -41.0], [10.0, -50.0, -9.0, 16.0, 47.0], [22.0, -10.0, -8.0, 9.0, 1.0]], [[-103.0, -9.0, 14.0, 11.0, 29.0], [55.0, 37.0, 20.0, -73.0, 1.0], [1.0, -20.0, 12.0, -7.0, 31.0], [9.0, -18.0, -27.0, -25.0, -43.0]], [[-2.0, 14.0, -6.0, 53.0, -56.0], [-64.0, 71.0, 30.0, -29.0, -83.0], [46.0, 18.0, 9.0, 24.0, -44.0], [-11.0, -39.0, 17.0, -11.0, 46.0]], [[20.0, -12.0, 37.0, -3.0, 17.0], [-6.0, -56.0, -37.0, -48.0, 37.0], [-64.0, -19.0, -18.0, -25.0, 30.0], [23.0, 16.0, -28.0, -3.0, -55.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[7.0, 7.0, -3.0, -6.0, -4.0], [2.0, 1.0, 5.0, 1.0, -3.0], [3.0, 0.0, 1.0, -2.0, -7.0], [6.0, -1.0, 3.0, 4.0, 1.0]]),self.functionspace)
      arg1=Data(3.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[21.0, 21.0, -9.0, -18.0, -12.0], [6.0, 3.0, 15.0, 3.0, -9.0], [9.0, 0.0, 3.0, -6.0, -21.0], [18.0, -3.0, 9.0, 12.0, 3.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[-4.0, 0.0, -3.0], [-3.0, 3.0, 7.0], [3.0, -2.0, -5.0], [7.0, -2.0, -4.0], [-7.0, -5.0, 5.0]], [[-1.0, 0.0, -7.0], [5.0, 2.0, 2.0], [-4.0, 5.0, 3.0], [-5.0, 6.0, 7.0], [2.0, 2.0, 3.0]], [[-5.0, -6.0, -2.0], [-7.0, 0.0, 6.0], [-7.0, 0.0, -4.0], [-5.0, 1.0, 1.0], [0.0, -1.0, -1.0]], [[-7.0, 4.0, 4.0], [-4.0, 5.0, 3.0], [0.0, 3.0, 7.0], [-1.0, -4.0, 5.0], [0.0, -2.0, -3.0]]]),self.functionspace)
      arg1=Data(numpy.array([-6.0, 3.0, -5.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[39.0, -8.0, 1.0, -28.0, 2.0], [41.0, -34.0, 24.0, 13.0, -21.0], [22.0, 12.0, 62.0, 28.0, 2.0], [34.0, 24.0, -26.0, -31.0, 9.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[1.0, 4.0, -6.0], [-3.0, 2.0, 1.0]], [[-3.0, 1.0, -5.0], [-3.0, -2.0, 0.0]], [[0.0, 7.0, -6.0], [-3.0, 0.0, -7.0]], [[-6.0, 7.0, -3.0], [6.0, 4.0, 1.0]], [[3.0, 2.0, -2.0], [0.0, 7.0, -1.0]]], [[[-1.0, -2.0, 0.0], [7.0, 4.0, -6.0]], [[1.0, -6.0, 5.0], [7.0, 6.0, 1.0]], [[0.0, 2.0, 4.0], [3.0, 3.0, -1.0]], [[-3.0, 0.0, 4.0], [1.0, -2.0, 3.0]], [[-3.0, -3.0, -4.0], [-7.0, 7.0, 2.0]]], [[[-7.0, 3.0, 2.0], [-4.0, 4.0, 4.0]], [[-3.0, -6.0, 3.0], [7.0, -4.0, 3.0]], [[5.0, 1.0, 0.0], [-6.0, -3.0, -6.0]], [[-6.0, -1.0, -2.0], [-5.0, 6.0, 3.0]], [[5.0, -6.0, -5.0], [-1.0, 5.0, -5.0]]], [[[3.0, 4.0, -6.0], [6.0, 2.0, -3.0]], [[3.0, 6.0, 5.0], [3.0, 0.0, -7.0]], [[0.0, -3.0, -7.0], [-6.0, -4.0, 4.0]], [[1.0, 4.0, 2.0], [-3.0, 7.0, -7.0]], [[6.0, 0.0, 4.0], [5.0, 0.0, -1.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[6.0, -5.0, -7.0], [3.0, 6.0, -6.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[25.0, -9.0, 40.0, -14.0, 70.0], [85.0, 52.0, -5.0, -73.0, 34.0], [-83.0, -30.0, 25.0, -14.0, 152.0], [88.0, 4.0, -2.0, 47.0, 29.0]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[0.0, -7.0, -5.0, -3.0, 0.0], [5.0, -3.0, 3.0, -6.0, 0.0], [-3.0, -4.0, 6.0, 7.0, 6.0], [2.0, -2.0, -4.0, -5.0, 0.0]]),self.functionspace)
      arg1=Data(numpy.array([-7.0, -2.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[0.0, 0.0], [49.0, 14.0], [35.0, 10.0], [21.0, 6.0], [0.0, 0.0]], [[-35.0, -10.0], [21.0, 6.0], [-21.0, -6.0], [42.0, 12.0], [0.0, 0.0]], [[21.0, 6.0], [28.0, 8.0], [-42.0, -12.0], [-49.0, -14.0], [-42.0, -12.0]], [[-14.0, -4.0], [14.0, 4.0], [28.0, 8.0], [35.0, 10.0], [0.0, 0.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[0.0, -4.0, -3.0], [1.0, 0.0, 5.0], [5.0, -7.0, -6.0], [3.0, -6.0, 4.0], [1.0, 0.0, -7.0]], [[-2.0, -2.0, 3.0], [-7.0, 0.0, 0.0], [5.0, 0.0, 0.0], [7.0, 6.0, -5.0], [-6.0, -4.0, 0.0]], [[0.0, 5.0, -2.0], [6.0, -1.0, -7.0], [2.0, 7.0, -3.0], [0.0, 1.0, 2.0], [6.0, 7.0, -7.0]], [[-4.0, -5.0, 1.0], [-4.0, 4.0, 1.0], [-5.0, -5.0, 0.0], [6.0, -7.0, 0.0], [2.0, 0.0, -1.0]]]),self.functionspace)
      arg1=Data(numpy.array([[4.0, -4.0, -5.0], [-4.0, 2.0, -4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[31.0, 4.0], [-21.0, -24.0], [78.0, -10.0], [16.0, -40.0], [39.0, 24.0]], [[-15.0, -8.0], [-28.0, 28.0], [20.0, -20.0], [29.0, 4.0], [-8.0, 16.0]], [[-10.0, 18.0], [63.0, 2.0], [-5.0, 18.0], [-14.0, -6.0], [31.0, 18.0]], [[-1.0, 2.0], [-37.0, 20.0], [0.0, 10.0], [52.0, -38.0], [13.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[-3.0, -6.0, 4.0], [5.0, 7.0, -5.0]], [[4.0, 1.0, -4.0], [3.0, -5.0, 6.0]], [[0.0, -6.0, 4.0], [0.0, 0.0, -3.0]], [[7.0, -2.0, -7.0], [-5.0, 2.0, -3.0]], [[-3.0, -1.0, -5.0], [1.0, 3.0, 4.0]]], [[[1.0, -4.0, 2.0], [-3.0, 3.0, -2.0]], [[-7.0, 2.0, 0.0], [-2.0, 5.0, 7.0]], [[7.0, 0.0, -5.0], [4.0, -2.0, -6.0]], [[0.0, -7.0, -4.0], [3.0, -3.0, -4.0]], [[0.0, -6.0, 4.0], [-2.0, 0.0, -5.0]]], [[[7.0, 5.0, -5.0], [6.0, 5.0, 4.0]], [[5.0, -1.0, 3.0], [7.0, 5.0, 3.0]], [[0.0, 1.0, -5.0], [-1.0, -1.0, 0.0]], [[-3.0, -1.0, -2.0], [5.0, -2.0, 4.0]], [[-4.0, 3.0, 4.0], [-4.0, -1.0, 2.0]]], [[[5.0, 0.0, 0.0], [-5.0, 5.0, 5.0]], [[-7.0, -7.0, -6.0], [2.0, -3.0, 7.0]], [[-3.0, -1.0, 2.0], [-2.0, -7.0, 2.0]], [[-5.0, -7.0, 0.0], [-1.0, -4.0, 2.0]], [[-2.0, 0.0, 0.0], [-4.0, 0.0, -7.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[5.0, -6.0, 3.0], [-4.0, -3.0, 3.0]], [[7.0, 1.0, 5.0], [7.0, 7.0, 6.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[-23.0, 47.0], [23.0, 31.0], [39.0, -4.0], [31.0, -27.0], [-25.0, 5.0]], [[32.0, 1.0], [-33.0, 16.0], [-8.0, 2.0], [15.0, -51.0], [41.0, -30.0]], [[-37.0, 130.0], [6.0, 151.0], [-14.0, -38.0], [-17.0, 13.0], [-1.0, -28.0]], [[45.0, 65.0], [11.0, -51.0], [32.0, -63.0], [39.0, -65.0], [-15.0, -84.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_constData_rank2_offset0(self):
      arg0=Data(numpy.array([[4.0, 0.0, 4.0, -2.0, -5.0], [-6.0, -7.0, 0.0, 0.0, 2.0], [-4.0, 5.0, 7.0, -7.0, 0.0], [0.0, 0.0, -1.0, 0.0, -6.0]]),self.functionspace)
      arg1=Data(numpy.array([[5.0, 1.0, -6.0, -4.0, 1.0], [-3.0, -1.0, 0.0, -4.0, -5.0], [2.0, 4.0, 1.0, 4.0, 4.0], [-2.0, 4.0, 6.0, -4.0, -2.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[20.0, 4.0, -24.0, -16.0, 4.0], [-12.0, -4.0, 0.0, -16.0, -20.0], [8.0, 16.0, 4.0, 16.0, 16.0], [-8.0, 16.0, 24.0, -16.0, -8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[20.0, 4.0, -24.0, -16.0, 4.0], [-12.0, -4.0, 0.0, -16.0, -20.0], [8.0, 16.0, 4.0, 16.0, 16.0], [-8.0, 16.0, 24.0, -16.0, -8.0]], [[-10.0, -2.0, 12.0, 8.0, -2.0], [6.0, 2.0, 0.0, 8.0, 10.0], [-4.0, -8.0, -2.0, -8.0, -8.0], [4.0, -8.0, -12.0, 8.0, 4.0]], [[-25.0, -5.0, 30.0, 20.0, -5.0], [15.0, 5.0, 0.0, 20.0, 25.0], [-10.0, -20.0, -5.0, -20.0, -20.0], [10.0, -20.0, -30.0, 20.0, 10.0]]], [[[-30.0, -6.0, 36.0, 24.0, -6.0], [18.0, 6.0, 0.0, 24.0, 30.0], [-12.0, -24.0, -6.0, -24.0, -24.0], [12.0, -24.0, -36.0, 24.0, 12.0]], [[-35.0, -7.0, 42.0, 28.0, -7.0], [21.0, 7.0, 0.0, 28.0, 35.0], [-14.0, -28.0, -7.0, -28.0, -28.0], [14.0, -28.0, -42.0, 28.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[10.0, 2.0, -12.0, -8.0, 2.0], [-6.0, -2.0, 0.0, -8.0, -10.0], [4.0, 8.0, 2.0, 8.0, 8.0], [-4.0, 8.0, 12.0, -8.0, -4.0]]], [[[-20.0, -4.0, 24.0, 16.0, -4.0], [12.0, 4.0, 0.0, 16.0, 20.0], [-8.0, -16.0, -4.0, -16.0, -16.0], [8.0, -16.0, -24.0, 16.0, 8.0]], [[25.0, 5.0, -30.0, -20.0, 5.0], [-15.0, -5.0, 0.0, -20.0, -25.0], [10.0, 20.0, 5.0, 20.0, 20.0], [-10.0, 20.0, 30.0, -20.0, -10.0]], [[35.0, 7.0, -42.0, -28.0, 7.0], [-21.0, -7.0, 0.0, -28.0, -35.0], [14.0, 28.0, 7.0, 28.0, 28.0], [-14.0, 28.0, 42.0, -28.0, -14.0]], [[-35.0, -7.0, 42.0, 28.0, -7.0], [21.0, 7.0, 0.0, 28.0, 35.0], [-14.0, -28.0, -7.0, -28.0, -28.0], [14.0, -28.0, -42.0, 28.0, 14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-5.0, -1.0, 6.0, 4.0, -1.0], [3.0, 1.0, 0.0, 4.0, 5.0], [-2.0, -4.0, -1.0, -4.0, -4.0], [2.0, -4.0, -6.0, 4.0, 2.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-30.0, -6.0, 36.0, 24.0, -6.0], [18.0, 6.0, 0.0, 24.0, 30.0], [-12.0, -24.0, -6.0, -24.0, -24.0], [12.0, -24.0, -36.0, 24.0, 12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank3_offset1(self):
      arg0=Data(numpy.array([[[-1.0, 3.0, 1.0], [-3.0, 6.0, 2.0], [3.0, -4.0, -6.0], [3.0, -3.0, 4.0], [-1.0, -4.0, -1.0]], [[3.0, 0.0, 6.0], [5.0, 2.0, -4.0], [5.0, 4.0, 0.0], [1.0, 7.0, -6.0], [5.0, 0.0, -1.0]], [[-1.0, -7.0, 0.0], [-3.0, 7.0, -2.0], [-4.0, 0.0, 0.0], [-3.0, -4.0, 3.0], [2.0, -1.0, -3.0]], [[0.0, -4.0, -3.0], [0.0, 0.0, -6.0], [2.0, -1.0, -2.0], [-4.0, 7.0, 5.0], [0.0, 0.0, 2.0]]]),self.functionspace)
      arg1=Data(numpy.array([[[3.0, -5.0, 0.0], [6.0, -2.0, -5.0], [-3.0, -7.0, 4.0], [-2.0, 7.0, 0.0], [0.0, -5.0, 7.0]], [[-4.0, -5.0, 0.0], [3.0, -4.0, 1.0], [4.0, 7.0, 4.0], [-2.0, -4.0, -5.0], [5.0, 0.0, 4.0]], [[-6.0, 5.0, -1.0], [4.0, -7.0, -2.0], [2.0, -7.0, 1.0], [0.0, 0.0, 5.0], [-7.0, 7.0, 0.0]], [[6.0, 6.0, 2.0], [-5.0, -1.0, 6.0], [-7.0, -4.0, -3.0], [-7.0, -6.0, -6.0], [-2.0, 6.0, 1.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-18.0, -17.0, -14.0, 23.0, -8.0], [-11.0, -14.0, 21.0, -15.0, -1.0], [20.0, -27.0, -22.0, 5.0, 28.0], [14.0, 8.0, -8.0, -17.0, 21.0]], [[-39.0, -40.0, -25.0, 48.0, -16.0], [-18.0, -31.0, 38.0, -28.0, -7.0], [46.0, -58.0, -46.0, 10.0, 63.0], [22.0, 21.0, -9.0, -27.0, 44.0]], [[29.0, 56.0, -5.0, -34.0, -22.0], [8.0, 19.0, -40.0, 40.0, -9.0], [-32.0, 52.0, 28.0, -30.0, -49.0], [-18.0, -47.0, 13.0, 39.0, -36.0]], [[24.0, 4.0, 28.0, -27.0, 43.0], [3.0, 25.0, 7.0, -14.0, 31.0], [-37.0, 25.0, 31.0, 20.0, -42.0], [8.0, 12.0, -21.0, -27.0, -20.0]], [[17.0, 7.0, 27.0, -26.0, 13.0], [24.0, 12.0, -36.0, 23.0, -9.0], [-13.0, 26.0, 25.0, -5.0, -21.0], [-32.0, 3.0, 26.0, 37.0, -23.0]]], [[[9.0, -12.0, 15.0, -6.0, 42.0], [-12.0, 15.0, 36.0, -36.0, 39.0], [-24.0, 0.0, 12.0, 30.0, -21.0], [30.0, 21.0, -39.0, -57.0, 0.0]], [[5.0, 46.0, -45.0, 4.0, -38.0], [-30.0, 3.0, 18.0, 2.0, 9.0], [-16.0, 14.0, -8.0, -20.0, -21.0], [34.0, -51.0, -31.0, -23.0, -2.0]], [[-5.0, 22.0, -43.0, 18.0, -20.0], [-40.0, -1.0, 48.0, -26.0, 25.0], [-10.0, -8.0, -18.0, 0.0, -7.0], [54.0, -29.0, -51.0, -59.0, 14.0]], [[-32.0, 22.0, -76.0, 47.0, -77.0], [-39.0, -31.0, 29.0, 0.0, -19.0], [35.0, -33.0, -53.0, -30.0, 42.0], [36.0, -48.0, -17.0, -13.0, 34.0]], [[15.0, 35.0, -19.0, -10.0, -7.0], [-20.0, 14.0, 16.0, -5.0, 21.0], [-29.0, 22.0, 9.0, -5.0, -35.0], [28.0, -31.0, -32.0, -29.0, -11.0]]], [[[32.0, 8.0, 52.0, -47.0, 35.0], [39.0, 25.0, -53.0, 30.0, -5.0], [-29.0, 45.0, 47.0, 0.0, -42.0], [-48.0, 12.0, 35.0, 49.0, -40.0]], [[-44.0, -22.0, -48.0, 55.0, -49.0], [-23.0, -39.0, 29.0, -12.0, -23.0], [55.0, -57.0, -57.0, -10.0, 70.0], [20.0, -4.0, -1.0, -9.0, 46.0]], [[-12.0, -24.0, 12.0, 8.0, 0.0], [16.0, -12.0, -16.0, 8.0, -20.0], [24.0, -16.0, -8.0, 0.0, 28.0], [-24.0, 20.0, 28.0, 28.0, 8.0]], [[11.0, -25.0, 49.0, -22.0, 41.0], [32.0, 10.0, -28.0, 7.0, -3.0], [-5.0, 10.0, 25.0, 15.0, -7.0], [-36.0, 37.0, 28.0, 27.0, -15.0]], [[11.0, 29.0, -11.0, -11.0, -16.0], [-3.0, 7.0, -11.0, 15.0, -2.0], [-14.0, 21.0, 8.0, -15.0, -21.0], [0.0, -27.0, -1.0, 10.0, -13.0]]], [[[20.0, 23.0, 16.0, -28.0, -1.0], [20.0, 13.0, -40.0, 31.0, -12.0], [-17.0, 34.0, 25.0, -15.0, -28.0], [-30.0, -14.0, 25.0, 42.0, -27.0]], [[0.0, 30.0, -24.0, 0.0, -42.0], [0.0, -6.0, -24.0, 30.0, -24.0], [6.0, 12.0, -6.0, -30.0, 0.0], [-12.0, -36.0, 18.0, 36.0, -6.0]], [[11.0, 24.0, -7.0, -11.0, -9.0], [-3.0, 8.0, -7.0, 10.0, 2.0], [-15.0, 19.0, 9.0, -10.0, -21.0], [2.0, -21.0, -4.0, 4.0, -12.0]], [[-47.0, -63.0, -17.0, 57.0, 0.0], [-19.0, -35.0, 53.0, -45.0, 0.0], [54.0, -75.0, -52.0, 25.0, 77.0], [28.0, 43.0, -15.0, -44.0, 55.0]], [[0.0, -10.0, 8.0, 0.0, 14.0], [0.0, 2.0, 8.0, -10.0, 8.0], [-2.0, -4.0, 2.0, 10.0, 0.0], [4.0, 12.0, -6.0, -12.0, 2.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[1.0, -3.0, 4.0], [1.0, 0.0, -4.0]], [[-4.0, 0.0, 1.0], [1.0, -3.0, 1.0]], [[6.0, 5.0, -4.0], [6.0, -1.0, -2.0]], [[-7.0, 1.0, 6.0], [0.0, 1.0, -1.0]], [[6.0, 3.0, 7.0], [0.0, 4.0, -7.0]]], [[[5.0, 3.0, 5.0], [5.0, -7.0, 7.0]], [[0.0, -2.0, -4.0], [6.0, -3.0, 3.0]], [[6.0, -5.0, -1.0], [-7.0, -1.0, -5.0]], [[6.0, 1.0, -3.0], [5.0, -3.0, -1.0]], [[-3.0, -1.0, -4.0], [1.0, -7.0, 0.0]]], [[[7.0, 2.0, -1.0], [1.0, 7.0, -5.0]], [[5.0, -2.0, -3.0], [-3.0, 1.0, 5.0]], [[0.0, -7.0, 2.0], [2.0, 5.0, -2.0]], [[7.0, -6.0, -6.0], [-3.0, 0.0, 2.0]], [[-3.0, -5.0, -7.0], [0.0, -5.0, -2.0]]], [[[0.0, -1.0, -6.0], [0.0, -1.0, 1.0]], [[-2.0, -1.0, -5.0], [4.0, 0.0, 4.0]], [[-1.0, 6.0, -3.0], [1.0, -1.0, 4.0]], [[-7.0, 2.0, 2.0], [-3.0, -6.0, 7.0]], [[0.0, 3.0, -5.0], [4.0, -6.0, 0.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[[[0.0, 0.0, 4.0], [3.0, 2.0, 5.0]], [[-1.0, -5.0, -3.0], [6.0, -7.0, 1.0]], [[2.0, -4.0, -2.0], [-4.0, 0.0, 4.0]], [[4.0, -2.0, 5.0], [5.0, 1.0, 5.0]], [[0.0, -6.0, -5.0], [5.0, 0.0, -3.0]]], [[[3.0, 7.0, 5.0], [7.0, 5.0, -5.0]], [[6.0, -5.0, -3.0], [-4.0, -4.0, -3.0]], [[-5.0, -5.0, 0.0], [-7.0, -1.0, -2.0]], [[4.0, 1.0, 1.0], [0.0, 7.0, 5.0]], [[0.0, 0.0, -6.0], [0.0, -4.0, -4.0]]], [[[4.0, 6.0, -3.0], [6.0, -4.0, -2.0]], [[-5.0, 2.0, 5.0], [6.0, -6.0, 7.0]], [[0.0, 5.0, -5.0], [-1.0, 4.0, 2.0]], [[-5.0, -7.0, -6.0], [4.0, 0.0, 4.0]], [[-3.0, -2.0, 6.0], [-1.0, -7.0, -4.0]]], [[[0.0, 6.0, 5.0], [1.0, 1.0, 7.0]], [[-1.0, 4.0, -5.0], [-3.0, 0.0, 7.0]], [[-4.0, -4.0, 5.0], [3.0, -3.0, -3.0]], [[-7.0, 2.0, -4.0], [-4.0, 0.0, -1.0]], [[-5.0, 2.0, -2.0], [-3.0, -1.0, -5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      ref=Data(numpy.array([[[[-1.0, 4.0, -14.0, 15.0, 15.0], [29.0, 17.0, 11.0, -15.0, -8.0], [-12.0, -13.0, -44.0, -20.0, 42.0], [-25.0, -64.0, 43.0, -29.0, -2.0]], [[6.0, 29.0, -10.0, -4.0, -3.0], [-20.0, -22.0, 14.0, -31.0, 2.0], [-3.0, 56.0, -16.0, 22.0, 34.0], [10.0, 3.0, 30.0, 19.0, 13.0]], [[-10.0, 22.0, -32.0, 13.0, 26.0], [80.0, 9.0, -92.0, 8.0, 36.0], [110.0, -12.0, 31.0, -25.0, -43.0], [1.0, 2.0, -37.0, -38.0, -19.0]], [[21.0, -24.0, -34.0, -4.0, -33.0], [26.0, -66.0, 31.0, -19.0, -36.0], [-42.0, 54.0, -23.0, -12.0, 52.0], [30.0, -26.0, 54.0, 28.0, 29.0]], [[1.0, -77.0, -42.0, 22.0, -32.0], [129.0, 5.0, -35.0, 27.0, -30.0], [19.0, -62.0, -18.0, -121.0, 18.0], [8.0, -78.0, 8.0, -57.0, -7.0]]], [[[56.0, 51.0, -4.0, 92.0, -39.0], [26.0, -13.0, -82.0, 14.0, -30.0], [67.0, 127.0, -29.0, -28.0, 25.0], [90.0, 16.0, 8.0, -76.0, -72.0]], [[11.0, 82.0, 4.0, 26.0, 53.0], [-22.0, 1.0, -35.0, -12.0, 24.0], [42.0, 51.0, -2.0, 74.0, -17.0], [-8.0, 15.0, 6.0, -15.0, -26.0]], [[-52.0, -18.0, 42.0, -32.0, 15.0], [-51.0, 111.0, 55.0, -14.0, 30.0], [-31.0, -116.0, -27.0, -37.0, 20.0], [-78.0, -35.0, -12.0, -15.0, 9.0]], [[-8.0, 48.0, -10.0, 24.0, 37.0], [35.0, 35.0, -65.0, -4.0, 34.0], [83.0, -2.0, 1.0, -3.0, -18.0], [-14.0, -9.0, -16.0, -47.0, -29.0]], [[-27.0, 75.0, 2.0, -32.0, 31.0], [-64.0, 23.0, 20.0, -66.0, 52.0], [28.0, 41.0, -14.0, 50.0, 35.0], [-32.0, 16.0, 20.0, 31.0, 25.0]]], [[[-12.0, -62.0, -16.0, 6.0, 13.0], [97.0, 18.0, -49.0, 53.0, -2.0], [31.0, -107.0, 32.0, -59.0, -61.0], [-20.0, -32.0, -44.0, -40.0, -14.0]], [[6.0, -6.0, 56.0, 20.0, -3.0], [-55.0, 42.0, -5.0, 47.0, -6.0], [-15.0, -33.0, 22.0, 15.0, -53.0], [6.0, 46.0, -54.0, -20.0, -40.0]], [[14.0, 4.0, 8.0, 29.0, 48.0], [10.0, 7.0, 20.0, 20.0, -24.0], [-52.0, -36.0, -31.0, 37.0, -3.0], [-39.0, -58.0, 35.0, -28.0, -19.0]], [[-23.0, 25.0, 70.0, 5.0, 45.0], [-82.0, 96.0, 12.0, 26.0, 28.0], [-12.0, -81.0, 7.0, 39.0, -50.0], [-55.0, 22.0, -49.0, -27.0, -36.0]], [[-48.0, 82.0, 20.0, -52.0, 71.0], [-94.0, 54.0, 49.0, -69.0, 70.0], [3.0, -14.0, -14.0, 84.0, 20.0], [-84.0, 4.0, 18.0, 41.0, 34.0]]], [[[-21.0, 31.0, 20.0, -24.0, 33.0], [-47.0, 24.0, 4.0, -9.0, 36.0], [14.0, -19.0, 23.0, 47.0, -31.0], [-30.0, 33.0, -26.0, 21.0, 6.0]], [[12.0, 50.0, 10.0, 9.0, 39.0], [-30.0, -20.0, -21.0, 6.0, 14.0], [17.0, 35.0, 24.0, 79.0, -42.0], [1.0, 39.0, -13.0, 12.0, -14.0]], [[9.0, -3.0, -8.0, -7.0, -28.0], [6.0, -39.0, -39.0, 12.0, 6.0], [43.0, 42.0, 48.0, 1.0, -37.0], [49.0, 65.0, -41.0, 23.0, 1.0]], [[22.0, 22.0, 14.0, -8.0, -58.0], [-83.0, -43.0, 38.0, -31.0, -16.0], [-30.0, 116.0, -7.0, 25.0, 46.0], [62.0, 63.0, 18.0, 50.0, 15.0]], [[-20.0, 66.0, -18.0, -17.0, 27.0], [-6.0, 8.0, -37.0, -44.0, 54.0], [81.0, 41.0, 12.0, 25.0, 2.0], [-9.0, 25.0, -7.0, 10.0, 10.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[-7.0, -2.0, -2.0, -6.0, -1.0], [-6.0, 1.0, -6.0, 0.0, 3.0], [-2.0, 1.0, -3.0, -7.0, 1.0], [1.0, -6.0, -6.0, 6.0, -4.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(7.0)+(1-msk_arg1)*(2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-49.0, -14.0, -14.0, -42.0, -7.0], [-42.0, 7.0, -42.0, 0.0, 21.0], [-14.0, 7.0, -21.0, -49.0, 7.0], [7.0, -42.0, -42.0, 42.0, -28.0]])+(1.-msk_ref)*numpy.array([[-14.0, -4.0, -4.0, -12.0, -2.0], [-12.0, 2.0, -12.0, 0.0, 6.0], [-4.0, 2.0, -6.0, -14.0, 2.0], [2.0, -12.0, -12.0, 12.0, -8.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[3.0, -3.0, 5.0], [-1.0, -7.0, -3.0], [1.0, -3.0, 1.0], [0.0, 0.0, 6.0], [0.0, 7.0, 7.0]], [[5.0, 3.0, -1.0], [1.0, 2.0, 1.0], [-5.0, -7.0, -2.0], [0.0, -6.0, -3.0], [0.0, 3.0, 6.0]], [[-1.0, 3.0, 0.0], [6.0, 0.0, -5.0], [7.0, -6.0, 0.0], [-2.0, 0.0, 0.0], [-4.0, -4.0, -3.0]], [[-3.0, 7.0, -7.0], [6.0, -1.0, 7.0], [-1.0, -6.0, -2.0], [-6.0, 4.0, -2.0], [1.0, -2.0, -3.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -1.0, -4.0])+(1.-msk_arg1)*numpy.array([7.0, -6.0, 5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[1.0, 13.0, 5.0, -24.0, -35.0], [31.0, 0.0, -15.0, 18.0, -27.0], [-9.0, 56.0, 48.0, -12.0, -8.0], [3.0, 9.0, 8.0, -32.0, 20.0]])+(1.-msk_ref)*numpy.array([[64.0, 20.0, 30.0, 30.0, -7.0], [12.0, 0.0, -3.0, 21.0, 12.0], [-25.0, 17.0, 85.0, -14.0, -19.0], [-98.0, 83.0, 19.0, -76.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank2_offset2(self):
      arg0=Data(numpy.array([[[[4.0, 4.0, 1.0], [-1.0, -7.0, 6.0]], [[-5.0, -2.0, -5.0], [0.0, -1.0, 6.0]], [[-4.0, -4.0, 3.0], [-5.0, -6.0, 4.0]], [[7.0, -7.0, 0.0], [0.0, 4.0, -6.0]], [[-3.0, 7.0, -5.0], [0.0, 2.0, 0.0]]], [[[-2.0, -4.0, 0.0], [2.0, 2.0, -6.0]], [[6.0, 3.0, 3.0], [2.0, 7.0, 2.0]], [[-7.0, -1.0, -6.0], [-3.0, -4.0, 3.0]], [[7.0, 0.0, -4.0], [3.0, -6.0, 3.0]], [[-5.0, -6.0, -4.0], [4.0, -7.0, -7.0]]], [[[-6.0, -1.0, 0.0], [1.0, 3.0, -1.0]], [[-4.0, 2.0, 0.0], [-1.0, -5.0, 0.0]], [[5.0, 0.0, -4.0], [3.0, -5.0, 0.0]], [[-4.0, 0.0, 5.0], [4.0, -1.0, -3.0]], [[6.0, -7.0, 5.0], [-5.0, 7.0, -7.0]]], [[[-5.0, 1.0, 0.0], [3.0, -1.0, 0.0]], [[1.0, -4.0, 1.0], [6.0, 1.0, 7.0]], [[-7.0, -5.0, -5.0], [6.0, 0.0, 0.0]], [[2.0, -4.0, 0.0], [2.0, 7.0, -3.0]], [[-2.0, -1.0, 5.0], [-7.0, 1.0, 3.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-5.0, 0.0, 0.0], [0.0, 4.0, 4.0]])+(1.-msk_arg1)*numpy.array([[-1.0, -7.0, -3.0], [-3.0, -2.0, 3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, 45.0, 12.0, -43.0, 23.0], [-6.0, 6.0, 31.0, -47.0, -31.0], [38.0, 0.0, -45.0, 4.0, -30.0], [21.0, 27.0, 35.0, 6.0, 26.0]])+(1.-msk_ref)*numpy.array([[0.0, 54.0, 62.0, 16.0, -35.0], [2.0, -50.0, 58.0, 17.0, 40.0], [1.0, 3.0, 8.0, -30.0, 8.0], [-9.0, 25.0, 39.0, -3.0, 22.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[-3.0, 0.0, 0.0, -3.0, -6.0], [1.0, -7.0, 0.0, 0.0, 3.0], [-1.0, -1.0, 5.0, -4.0, 0.0], [-5.0, 0.0, -7.0, 6.0, -3.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, -4.0])+(1.-msk_arg1)*numpy.array([7.0, 5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-18.0, 12.0], [0.0, 0.0], [0.0, 0.0], [-18.0, 12.0], [-36.0, 24.0]], [[6.0, -4.0], [-42.0, 28.0], [0.0, 0.0], [0.0, 0.0], [18.0, -12.0]], [[-6.0, 4.0], [-6.0, 4.0], [30.0, -20.0], [-24.0, 16.0], [0.0, 0.0]], [[-30.0, 20.0], [0.0, 0.0], [-42.0, 28.0], [36.0, -24.0], [-18.0, 12.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, -15.0], [0.0, 0.0], [0.0, 0.0], [-21.0, -15.0], [-42.0, -30.0]], [[7.0, 5.0], [-49.0, -35.0], [0.0, 0.0], [0.0, 0.0], [21.0, 15.0]], [[-7.0, -5.0], [-7.0, -5.0], [35.0, 25.0], [-28.0, -20.0], [0.0, 0.0]], [[-35.0, -25.0], [0.0, 0.0], [-49.0, -35.0], [42.0, 30.0], [-21.0, -15.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[-2.0, 7.0, -4.0], [6.0, -2.0, -5.0], [-5.0, 2.0, 7.0], [4.0, 5.0, -7.0], [-2.0, 3.0, 2.0]], [[4.0, 3.0, -3.0], [7.0, -5.0, 2.0], [-5.0, 5.0, -6.0], [1.0, -5.0, -4.0], [-5.0, -3.0, 5.0]], [[-5.0, 6.0, -1.0], [4.0, 4.0, 6.0], [-6.0, 0.0, -2.0], [-5.0, -4.0, -1.0], [0.0, 6.0, -1.0]], [[-5.0, 4.0, 5.0], [3.0, -2.0, 4.0], [5.0, 0.0, -5.0], [5.0, -4.0, -6.0], [0.0, 6.0, 5.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-7.0, 3.0, 7.0], [3.0, 7.0, 2.0]])+(1.-msk_arg1)*numpy.array([[1.0, 7.0, -4.0], [3.0, -6.0, -4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, 35.0], [-83.0, -6.0], [90.0, 13.0], [-62.0, 33.0], [37.0, 19.0]], [[-40.0, 27.0], [-50.0, -10.0], [8.0, 8.0], [-50.0, -40.0], [61.0, -26.0]], [[46.0, 25.0], [26.0, 52.0], [28.0, -22.0], [16.0, -45.0], [11.0, 40.0]], [[82.0, 23.0], [1.0, 3.0], [-70.0, 5.0], [-89.0, -25.0], [53.0, 52.0]]])+(1.-msk_ref)*numpy.array([[[63.0, -32.0], [12.0, 50.0], [-19.0, -55.0], [67.0, 10.0], [11.0, -32.0]], [[37.0, 6.0], [-36.0, 43.0], [54.0, -21.0], [-18.0, 49.0], [-46.0, -17.0]], [[41.0, -47.0], [8.0, -36.0], [2.0, -10.0], [-29.0, 13.0], [46.0, -32.0]], [[3.0, -59.0], [-27.0, 5.0], [25.0, 35.0], [1.0, 63.0], [22.0, -56.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank3_offset2(self):
      arg0=Data(numpy.array([[[[-4.0, 7.0, 3.0], [-1.0, -1.0, 0.0]], [[3.0, 1.0, -5.0], [-4.0, 0.0, 2.0]], [[6.0, 2.0, 3.0], [2.0, 0.0, 7.0]], [[0.0, 5.0, 3.0], [7.0, 5.0, -4.0]], [[2.0, -3.0, -1.0], [-2.0, -3.0, 1.0]]], [[[1.0, 0.0, 4.0], [4.0, 3.0, 2.0]], [[3.0, 0.0, 6.0], [5.0, 0.0, -2.0]], [[-2.0, -7.0, 6.0], [3.0, 1.0, 0.0]], [[-5.0, 2.0, -1.0], [-5.0, -4.0, -2.0]], [[-4.0, -1.0, -2.0], [-5.0, 3.0, 4.0]]], [[[4.0, 7.0, -1.0], [-7.0, 3.0, 0.0]], [[4.0, -4.0, -4.0], [-6.0, 1.0, 7.0]], [[-1.0, 4.0, -1.0], [-2.0, 2.0, 2.0]], [[-1.0, 2.0, 3.0], [-5.0, 2.0, 7.0]], [[-6.0, 4.0, -5.0], [1.0, 0.0, 6.0]]], [[[-5.0, 0.0, -2.0], [-1.0, 2.0, 6.0]], [[-4.0, 4.0, 5.0], [-2.0, -4.0, 1.0]], [[4.0, 5.0, -7.0], [4.0, 7.0, -2.0]], [[-1.0, 2.0, 0.0], [3.0, 6.0, 7.0]], [[7.0, 2.0, 6.0], [2.0, -5.0, 6.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[7.0, 0.0, -3.0], [3.0, 4.0, -5.0]], [[0.0, 4.0, -7.0], [6.0, 6.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[-7.0, -1.0, -3.0], [-6.0, 6.0, 7.0]], [[-2.0, 0.0, -7.0], [-6.0, 4.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-44.0, -5.0], [14.0, 23.0], [4.0, 27.0], [52.0, 55.0], [-6.0, -31.0]], [[9.0, 22.0], [28.0, -20.0], [-19.0, -46.0], [-53.0, -47.0], [-45.0, 14.0]], [[22.0, 11.0], [-9.0, 10.0], [-12.0, 31.0], [-58.0, -3.0], [-54.0, 81.0]], [[-54.0, 44.0], [-70.0, -51.0], [99.0, 127.0], [-9.0, 90.0], [-13.0, -28.0]]])+(1.-msk_ref)*numpy.array([[[12.0, -11.0], [31.0, 41.0], [-16.0, -87.0], [-54.0, -19.0], [-7.0, -3.0]], [[-11.0, -54.0], [-83.0, -66.0], [-9.0, -52.0], [28.0, 43.0], [111.0, 40.0]], [[28.0, 53.0], [79.0, 18.0], [44.0, 17.0], [87.0, -23.0], [89.0, 5.0]], [[101.0, 2.0], [4.0, -37.0], [-8.0, 57.0], [72.0, -34.0], [-69.0, -124.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank2_expandedData_rank2_offset0(self):
      arg0=Data(numpy.array([[-1.0, 6.0, 2.0, 5.0, 3.0], [6.0, -7.0, 0.0, -6.0, 6.0], [5.0, -4.0, 7.0, 0.0, 2.0], [0.0, 3.0, 0.0, 0.0, -2.0]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[4.0, 4.0, -7.0, -7.0, -6.0], [-1.0, -7.0, 2.0, 0.0, -2.0], [5.0, 0.0, -2.0, 1.0, -7.0], [4.0, 1.0, 0.0, 1.0, 0.0]])+(1.-msk_arg1)*numpy.array([[-1.0, 6.0, 5.0, 0.0, -1.0], [-7.0, -7.0, 2.0, -3.0, 2.0], [5.0, -2.0, 4.0, 1.0, -2.0], [3.0, 3.0, 1.0, 7.0, -3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-4.0, -4.0, 7.0, 7.0, 6.0], [1.0, 7.0, -2.0, 0.0, 2.0], [-5.0, 0.0, 2.0, -1.0, 7.0], [-4.0, -1.0, 0.0, -1.0, 0.0]], [[24.0, 24.0, -42.0, -42.0, -36.0], [-6.0, -42.0, 12.0, 0.0, -12.0], [30.0, 0.0, -12.0, 6.0, -42.0], [24.0, 6.0, 0.0, 6.0, 0.0]], [[8.0, 8.0, -14.0, -14.0, -12.0], [-2.0, -14.0, 4.0, 0.0, -4.0], [10.0, 0.0, -4.0, 2.0, -14.0], [8.0, 2.0, 0.0, 2.0, 0.0]], [[20.0, 20.0, -35.0, -35.0, -30.0], [-5.0, -35.0, 10.0, 0.0, -10.0], [25.0, 0.0, -10.0, 5.0, -35.0], [20.0, 5.0, 0.0, 5.0, 0.0]], [[12.0, 12.0, -21.0, -21.0, -18.0], [-3.0, -21.0, 6.0, 0.0, -6.0], [15.0, 0.0, -6.0, 3.0, -21.0], [12.0, 3.0, 0.0, 3.0, 0.0]]], [[[24.0, 24.0, -42.0, -42.0, -36.0], [-6.0, -42.0, 12.0, 0.0, -12.0], [30.0, 0.0, -12.0, 6.0, -42.0], [24.0, 6.0, 0.0, 6.0, 0.0]], [[-28.0, -28.0, 49.0, 49.0, 42.0], [7.0, 49.0, -14.0, 0.0, 14.0], [-35.0, 0.0, 14.0, -7.0, 49.0], [-28.0, -7.0, 0.0, -7.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-24.0, -24.0, 42.0, 42.0, 36.0], [6.0, 42.0, -12.0, 0.0, 12.0], [-30.0, 0.0, 12.0, -6.0, 42.0], [-24.0, -6.0, 0.0, -6.0, 0.0]], [[24.0, 24.0, -42.0, -42.0, -36.0], [-6.0, -42.0, 12.0, 0.0, -12.0], [30.0, 0.0, -12.0, 6.0, -42.0], [24.0, 6.0, 0.0, 6.0, 0.0]]], [[[20.0, 20.0, -35.0, -35.0, -30.0], [-5.0, -35.0, 10.0, 0.0, -10.0], [25.0, 0.0, -10.0, 5.0, -35.0], [20.0, 5.0, 0.0, 5.0, 0.0]], [[-16.0, -16.0, 28.0, 28.0, 24.0], [4.0, 28.0, -8.0, 0.0, 8.0], [-20.0, 0.0, 8.0, -4.0, 28.0], [-16.0, -4.0, 0.0, -4.0, 0.0]], [[28.0, 28.0, -49.0, -49.0, -42.0], [-7.0, -49.0, 14.0, 0.0, -14.0], [35.0, 0.0, -14.0, 7.0, -49.0], [28.0, 7.0, 0.0, 7.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[8.0, 8.0, -14.0, -14.0, -12.0], [-2.0, -14.0, 4.0, 0.0, -4.0], [10.0, 0.0, -4.0, 2.0, -14.0], [8.0, 2.0, 0.0, 2.0, 0.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[12.0, 12.0, -21.0, -21.0, -18.0], [-3.0, -21.0, 6.0, 0.0, -6.0], [15.0, 0.0, -6.0, 3.0, -21.0], [12.0, 3.0, 0.0, 3.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-8.0, -8.0, 14.0, 14.0, 12.0], [2.0, 14.0, -4.0, 0.0, 4.0], [-10.0, 0.0, 4.0, -2.0, 14.0], [-8.0, -2.0, 0.0, -2.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, -6.0, -5.0, 0.0, 1.0], [7.0, 7.0, -2.0, 3.0, -2.0], [-5.0, 2.0, -4.0, -1.0, 2.0], [-3.0, -3.0, -1.0, -7.0, 3.0]], [[-6.0, 36.0, 30.0, 0.0, -6.0], [-42.0, -42.0, 12.0, -18.0, 12.0], [30.0, -12.0, 24.0, 6.0, -12.0], [18.0, 18.0, 6.0, 42.0, -18.0]], [[-2.0, 12.0, 10.0, 0.0, -2.0], [-14.0, -14.0, 4.0, -6.0, 4.0], [10.0, -4.0, 8.0, 2.0, -4.0], [6.0, 6.0, 2.0, 14.0, -6.0]], [[-5.0, 30.0, 25.0, 0.0, -5.0], [-35.0, -35.0, 10.0, -15.0, 10.0], [25.0, -10.0, 20.0, 5.0, -10.0], [15.0, 15.0, 5.0, 35.0, -15.0]], [[-3.0, 18.0, 15.0, 0.0, -3.0], [-21.0, -21.0, 6.0, -9.0, 6.0], [15.0, -6.0, 12.0, 3.0, -6.0], [9.0, 9.0, 3.0, 21.0, -9.0]]], [[[-6.0, 36.0, 30.0, 0.0, -6.0], [-42.0, -42.0, 12.0, -18.0, 12.0], [30.0, -12.0, 24.0, 6.0, -12.0], [18.0, 18.0, 6.0, 42.0, -18.0]], [[7.0, -42.0, -35.0, 0.0, 7.0], [49.0, 49.0, -14.0, 21.0, -14.0], [-35.0, 14.0, -28.0, -7.0, 14.0], [-21.0, -21.0, -7.0, -49.0, 21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[6.0, -36.0, -30.0, 0.0, 6.0], [42.0, 42.0, -12.0, 18.0, -12.0], [-30.0, 12.0, -24.0, -6.0, 12.0], [-18.0, -18.0, -6.0, -42.0, 18.0]], [[-6.0, 36.0, 30.0, 0.0, -6.0], [-42.0, -42.0, 12.0, -18.0, 12.0], [30.0, -12.0, 24.0, 6.0, -12.0], [18.0, 18.0, 6.0, 42.0, -18.0]]], [[[-5.0, 30.0, 25.0, 0.0, -5.0], [-35.0, -35.0, 10.0, -15.0, 10.0], [25.0, -10.0, 20.0, 5.0, -10.0], [15.0, 15.0, 5.0, 35.0, -15.0]], [[4.0, -24.0, -20.0, 0.0, 4.0], [28.0, 28.0, -8.0, 12.0, -8.0], [-20.0, 8.0, -16.0, -4.0, 8.0], [-12.0, -12.0, -4.0, -28.0, 12.0]], [[-7.0, 42.0, 35.0, 0.0, -7.0], [-49.0, -49.0, 14.0, -21.0, 14.0], [35.0, -14.0, 28.0, 7.0, -14.0], [21.0, 21.0, 7.0, 49.0, -21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-2.0, 12.0, 10.0, 0.0, -2.0], [-14.0, -14.0, 4.0, -6.0, 4.0], [10.0, -4.0, 8.0, 2.0, -4.0], [6.0, 6.0, 2.0, 14.0, -6.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-3.0, 18.0, 15.0, 0.0, -3.0], [-21.0, -21.0, 6.0, -9.0, 6.0], [15.0, -6.0, 12.0, 3.0, -6.0], [9.0, 9.0, 3.0, 21.0, -9.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[2.0, -12.0, -10.0, 0.0, 2.0], [14.0, 14.0, -4.0, 6.0, -4.0], [-10.0, 4.0, -8.0, -2.0, 4.0], [-6.0, -6.0, -2.0, -14.0, 6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank3_offset1(self):
      arg0=Data(numpy.array([[[5.0, -7.0, 1.0], [7.0, -1.0, -4.0], [-5.0, 5.0, 0.0], [-7.0, 0.0, 6.0], [-7.0, -1.0, -2.0]], [[1.0, 1.0, 1.0], [-2.0, 0.0, 1.0], [0.0, -1.0, 7.0], [4.0, 3.0, 0.0], [-1.0, 5.0, -3.0]], [[1.0, 3.0, -3.0], [6.0, -1.0, 5.0], [-3.0, 5.0, 4.0], [4.0, -5.0, 1.0], [-2.0, -5.0, 6.0]], [[4.0, -7.0, 0.0], [-1.0, -6.0, 0.0], [0.0, 7.0, 0.0], [7.0, -6.0, 0.0], [6.0, 0.0, 0.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[4.0, 3.0, -3.0], [5.0, 0.0, -5.0], [-2.0, -1.0, 7.0], [7.0, 1.0, -6.0], [-6.0, -1.0, -3.0]], [[-4.0, 6.0, 5.0], [6.0, -6.0, 7.0], [5.0, 4.0, -5.0], [1.0, 0.0, 2.0], [0.0, 0.0, -1.0]], [[1.0, -5.0, -6.0], [0.0, 0.0, 0.0], [1.0, 0.0, 2.0], [-6.0, 4.0, 1.0], [3.0, 4.0, -1.0]], [[-2.0, 3.0, -5.0], [2.0, -2.0, -3.0], [-1.0, -6.0, 7.0], [4.0, 0.0, -7.0], [3.0, 0.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[2.0, 0.0, -7.0], [-2.0, -5.0, 3.0], [2.0, 7.0, 2.0], [2.0, 3.0, 2.0], [6.0, -2.0, -6.0]], [[0.0, -1.0, -1.0], [5.0, -2.0, 1.0], [6.0, -5.0, 1.0], [-3.0, 1.0, 2.0], [0.0, -3.0, 5.0]], [[-5.0, 2.0, -7.0], [-6.0, 0.0, 0.0], [1.0, -6.0, -4.0], [2.0, -6.0, -2.0], [-2.0, 0.0, -2.0]], [[3.0, 3.0, -2.0], [-1.0, -4.0, 4.0], [-4.0, -1.0, 1.0], [0.0, -5.0, 1.0], [-5.0, -5.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-4.0, 20.0, 4.0, 22.0, -26.0], [-57.0, 79.0, -8.0, 7.0, -1.0], [34.0, 0.0, 7.0, -57.0, -14.0], [-36.0, 21.0, 44.0, 13.0, 22.0]], [[37.0, 55.0, -41.0, 72.0, -29.0], [-54.0, 20.0, 51.0, -1.0, 4.0], [36.0, 0.0, -1.0, -50.0, 21.0], [3.0, 28.0, -29.0, 56.0, -7.0]], [[-5.0, -25.0, 5.0, -30.0, 25.0], [50.0, -60.0, -5.0, -5.0, 0.0], [-30.0, 0.0, -5.0, 50.0, 5.0], [25.0, -20.0, -25.0, -20.0, -15.0]], [[-46.0, -65.0, 56.0, -85.0, 24.0], [58.0, 0.0, -65.0, 5.0, -6.0], [-43.0, 0.0, 5.0, 48.0, -27.0], [-16.0, -32.0, 49.0, -70.0, 21.0]], [[-25.0, -25.0, 1.0, -38.0, 49.0], [12.0, -50.0, -29.0, -11.0, 2.0], [10.0, 0.0, -11.0, 36.0, -23.0], [21.0, -6.0, -1.0, -14.0, -35.0]]], [[[4.0, 0.0, 4.0, 2.0, -10.0], [7.0, 7.0, 4.0, 3.0, -1.0], [-10.0, 0.0, 3.0, -1.0, 6.0], [-4.0, -3.0, 0.0, -3.0, 10.0]], [[-11.0, -15.0, 11.0, -20.0, 9.0], [13.0, -5.0, -15.0, 0.0, -1.0], [-8.0, 0.0, 0.0, 13.0, -7.0], [-1.0, -7.0, 9.0, -15.0, 1.0]], [[-24.0, -35.0, 50.0, -43.0, -20.0], [29.0, 55.0, -39.0, 14.0, -7.0], [-37.0, 0.0, 14.0, 3.0, -11.0], [-38.0, -19.0, 55.0, -49.0, 49.0]], [[25.0, 20.0, -11.0, 31.0, -27.0], [2.0, 6.0, 32.0, 4.0, 0.0], [-11.0, 0.0, 4.0, -12.0, 24.0], [1.0, 2.0, -22.0, 16.0, 12.0]], [[20.0, 10.0, -24.0, 16.0, 10.0], [19.0, -57.0, 30.0, -7.0, 3.0], [-8.0, 0.0, -7.0, 23.0, 20.0], [32.0, -3.0, -50.0, 17.0, -24.0]]], [[[22.0, 20.0, -26.0, 28.0, 0.0], [-1.0, -33.0, 32.0, -5.0, 3.0], [4.0, 0.0, -5.0, 3.0, 18.0], [22.0, 5.0, -40.0, 25.0, -18.0]], [[6.0, 5.0, 24.0, 11.0, -50.0], [-5.0, 77.0, 1.0, 16.0, -5.0], [-19.0, 0.0, 16.0, -35.0, 9.0], [-40.0, -1.0, 35.0, -11.0, 53.0]], [[-9.0, -35.0, 29.0, -40.0, 1.0], [62.0, -20.0, -15.0, 5.0, -4.0], [-52.0, 0.0, 5.0, 42.0, 7.0], [1.0, -28.0, 1.0, -40.0, 19.0]], [[-2.0, 15.0, 4.0, 17.0, -22.0], [-41.0, 61.0, -5.0, 6.0, -1.0], [23.0, 0.0, 6.0, -43.0, -9.0], [-28.0, 15.0, 33.0, 9.0, 19.0]], [[-41.0, -40.0, 51.0, -55.0, -1.0], [8.0, 60.0, -60.0, 10.0, -6.0], [-13.0, 0.0, 10.0, -2.0, -32.0], [-41.0, -12.0, 74.0, -50.0, 36.0]]], [[[-5.0, 20.0, -1.0, 21.0, -17.0], [-58.0, 66.0, -8.0, 4.0, 0.0], [39.0, 0.0, 4.0, -52.0, -16.0], [-29.0, 22.0, 38.0, 16.0, 12.0]], [[-22.0, -5.0, 8.0, -13.0, 12.0], [-32.0, 30.0, -29.0, -1.0, 0.0], [29.0, 0.0, -1.0, -18.0, -27.0], [-16.0, 10.0, 37.0, -4.0, -3.0]], [[21.0, 0.0, -7.0, 7.0, -7.0], [42.0, -42.0, 28.0, 0.0, 0.0], [-35.0, 0.0, 0.0, 28.0, 28.0], [21.0, -14.0, -42.0, 0.0, 0.0]], [[10.0, 35.0, -8.0, 43.0, -36.0], [-64.0, 78.0, 11.0, 7.0, 0.0], [37.0, 0.0, 7.0, -66.0, -3.0], [-32.0, 26.0, 29.0, 28.0, 21.0]], [[24.0, 30.0, -12.0, 42.0, -36.0], [-24.0, 36.0, 30.0, 6.0, 0.0], [6.0, 0.0, 6.0, -36.0, 18.0], [-12.0, 12.0, -6.0, 24.0, 18.0]]]])+(1.-msk_ref)*numpy.array([[[[3.0, 28.0, -37.0, -9.0, 38.0], [6.0, 40.0, 66.0, -20.0, 26.0], [-46.0, -30.0, 43.0, 50.0, -12.0], [-8.0, 27.0, -12.0, 36.0, 12.0]], [[42.0, -21.0, -1.0, 3.0, 68.0], [5.0, 33.0, 43.0, -30.0, -17.0], [-9.0, -42.0, 29.0, 28.0, -6.0], [26.0, -19.0, -31.0, 1.0, -38.0]], [[-10.0, -15.0, 25.0, 5.0, -40.0], [-5.0, -35.0, -55.0, 20.0, -15.0], [35.0, 30.0, -35.0, -40.0, 10.0], [0.0, -15.0, 15.0, -25.0, 0.0]], [[-56.0, 32.0, -2.0, -2.0, -78.0], [-6.0, -29.0, -36.0, 33.0, 30.0], [-7.0, 42.0, -31.0, -26.0, 2.0], [-33.0, 31.0, 34.0, 6.0, 47.0]], [[0.0, 13.0, -25.0, -21.0, -28.0], [3.0, -35.0, -39.0, 16.0, -7.0], [47.0, 42.0, 7.0, -4.0, 18.0], [-20.0, 3.0, 27.0, 3.0, 36.0]]], [[[-5.0, -4.0, 11.0, 7.0, -2.0], [-2.0, 4.0, 2.0, 0.0, 2.0], [-10.0, -6.0, -9.0, -6.0, -4.0], [4.0, -1.0, -4.0, -4.0, -8.0]], [[-11.0, 7.0, -2.0, -2.0, -18.0], [-1.0, -9.0, -11.0, 8.0, 5.0], [3.0, 12.0, -6.0, -6.0, 2.0], [-8.0, 6.0, 9.0, 1.0, 12.0]], [[-49.0, 26.0, 7.0, 11.0, -40.0], [-6.0, 9.0, 12.0, 13.0, 38.0], [-51.0, 0.0, -22.0, -8.0, -14.0], [-17.0, 32.0, 8.0, 12.0, 19.0]], [[8.0, -23.0, 29.0, 17.0, 18.0], [-3.0, 14.0, 9.0, -9.0, -9.0], [-14.0, -24.0, -14.0, -10.0, -8.0], [21.0, -16.0, -19.0, -15.0, -35.0]], [[19.0, -32.0, 27.0, 7.0, 2.0], [-2.0, -18.0, -34.0, 2.0, -30.0], [36.0, 6.0, -19.0, -26.0, 8.0], [18.0, -31.0, -4.0, -28.0, -26.0]]], [[[23.0, -26.0, 17.0, 5.0, 18.0], [0.0, -4.0, -12.0, -6.0, -24.0], [22.0, -6.0, -5.0, -10.0, 4.0], [18.0, -25.0, -10.0, -18.0, -26.0]], [[-23.0, 8.0, 15.0, 19.0, 8.0], [-4.0, 37.0, 46.0, -9.0, 28.0], [-67.0, -36.0, -8.0, 8.0, -22.0], [5.0, 18.0, -18.0, 10.0, -15.0]], [[-34.0, -7.0, 37.0, 17.0, -52.0], [-9.0, -21.0, -39.0, 22.0, 5.0], [-3.0, 18.0, -49.0, -44.0, -2.0], [-2.0, -1.0, 11.0, -21.0, -2.0]], [[1.0, 20.0, -25.0, -5.0, 28.0], [4.0, 31.0, 50.0, -15.0, 20.0], [-37.0, -24.0, 30.0, 36.0, -10.0], [-5.0, 20.0, -10.0, 26.0, 7.0]], [[-46.0, 47.0, -27.0, -7.0, -38.0], [-1.0, 6.0, 19.0, 13.0, 45.0], [-42.0, 12.0, 4.0, 14.0, -8.0], [-33.0, 46.0, 19.0, 31.0, 47.0]]], [[[8.0, 27.0, -41.0, -13.0, 38.0], [7.0, 34.0, 59.0, -19.0, 21.0], [-34.0, -24.0, 46.0, 50.0, -8.0], [-9.0, 24.0, -9.0, 35.0, 15.0]], [[-2.0, 32.0, -44.0, -20.0, 6.0], [6.0, 7.0, 24.0, -3.0, 18.0], [-7.0, 6.0, 35.0, 34.0, 2.0], [-21.0, 25.0, 10.0, 30.0, 35.0]], [[0.0, -35.0, 49.0, 21.0, -14.0], [-7.0, -14.0, -35.0, 7.0, -21.0], [14.0, 0.0, -42.0, -42.0, 0.0], [21.0, -28.0, -7.0, -35.0, -35.0]], [[14.0, 16.0, -28.0, -4.0, 54.0], [6.0, 47.0, 72.0, -27.0, 18.0], [-47.0, -42.0, 43.0, 50.0, -14.0], [3.0, 17.0, -22.0, 30.0, -5.0]], [[12.0, -12.0, 12.0, 12.0, 36.0], [0.0, 30.0, 36.0, -18.0, 0.0], [-30.0, -36.0, 6.0, 12.0, -12.0], [18.0, -6.0, -24.0, 0.0, -30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank4_offset2(self):
      arg0=Data(numpy.array([[[[0.0, -1.0, 4.0], [5.0, 0.0, -7.0]], [[2.0, -6.0, -4.0], [0.0, 4.0, -7.0]], [[6.0, -6.0, 7.0], [-3.0, 7.0, 1.0]], [[2.0, -4.0, 6.0], [5.0, 4.0, 2.0]], [[4.0, 5.0, 0.0], [7.0, -6.0, -3.0]]], [[[-2.0, -1.0, -6.0], [3.0, -2.0, 3.0]], [[-6.0, -2.0, 6.0], [0.0, -1.0, 0.0]], [[6.0, -2.0, 4.0], [4.0, 7.0, -4.0]], [[-5.0, -3.0, -2.0], [1.0, 6.0, -3.0]], [[-2.0, 0.0, 2.0], [-2.0, -2.0, 3.0]]], [[[7.0, 6.0, -5.0], [-5.0, 7.0, -6.0]], [[1.0, 2.0, -6.0], [-7.0, -5.0, -7.0]], [[4.0, 4.0, -5.0], [-5.0, 6.0, 7.0]], [[-5.0, 4.0, -1.0], [0.0, 3.0, 1.0]], [[-3.0, 1.0, -5.0], [6.0, 3.0, 2.0]]], [[[-6.0, 1.0, -4.0], [2.0, 2.0, 4.0]], [[2.0, 4.0, -5.0], [-3.0, -4.0, -5.0]], [[3.0, 0.0, -6.0], [5.0, -6.0, 5.0]], [[-3.0, 6.0, -4.0], [-5.0, 6.0, 5.0]], [[6.0, -1.0, -1.0], [-3.0, 3.0, -1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -6.0, -7.0], [-3.0, 2.0, 7.0]], [[-4.0, 7.0, -1.0], [4.0, 4.0, 4.0]], [[-4.0, -3.0, 6.0], [1.0, 2.0, 4.0]], [[0.0, -6.0, -2.0], [-3.0, 4.0, -4.0]], [[-4.0, -1.0, 1.0], [2.0, 2.0, 0.0]]], [[[2.0, -2.0, 3.0], [0.0, 5.0, 4.0]], [[0.0, 0.0, -2.0], [-1.0, -5.0, -6.0]], [[0.0, -3.0, 2.0], [4.0, -5.0, -5.0]], [[0.0, 3.0, -3.0], [2.0, -7.0, -5.0]], [[-1.0, 0.0, 7.0], [-1.0, -3.0, -2.0]]], [[[-2.0, -1.0, 3.0], [-2.0, 3.0, 0.0]], [[-3.0, -1.0, 2.0], [2.0, -5.0, 7.0]], [[-4.0, 4.0, 3.0], [-4.0, 0.0, 0.0]], [[-3.0, 5.0, 3.0], [7.0, 4.0, 2.0]], [[0.0, -3.0, -2.0], [-1.0, -6.0, -3.0]]], [[[0.0, -6.0, 1.0], [-3.0, 3.0, 0.0]], [[0.0, -7.0, 7.0], [6.0, -7.0, 3.0]], [[-1.0, 2.0, -2.0], [3.0, -4.0, -6.0]], [[-7.0, 5.0, -3.0], [-6.0, 4.0, 3.0]], [[6.0, -5.0, -5.0], [1.0, -3.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[0.0, 4.0, 0.0], [1.0, -5.0, 0.0]], [[-6.0, -4.0, 4.0], [-7.0, 2.0, 7.0]], [[0.0, -5.0, -4.0], [4.0, 7.0, 0.0]], [[-7.0, -7.0, 0.0], [5.0, 0.0, -3.0]], [[-7.0, -7.0, 7.0], [3.0, 1.0, 4.0]]], [[[1.0, 0.0, -5.0], [-3.0, -4.0, -1.0]], [[-1.0, 0.0, -1.0], [4.0, 0.0, 0.0]], [[-3.0, 0.0, 6.0], [-2.0, 3.0, 3.0]], [[-6.0, 0.0, 7.0], [-5.0, 1.0, 0.0]], [[4.0, 7.0, 0.0], [5.0, 7.0, -4.0]]], [[[-5.0, -6.0, 7.0], [-7.0, -7.0, -2.0]], [[5.0, -4.0, -7.0], [-4.0, -4.0, -7.0]], [[0.0, 0.0, 6.0], [-4.0, -2.0, -6.0]], [[-7.0, 2.0, -6.0], [5.0, 2.0, -2.0]], [[3.0, 6.0, -7.0], [-1.0, -6.0, 6.0]]], [[[-4.0, -7.0, 7.0], [-1.0, 6.0, -7.0]], [[4.0, 6.0, 0.0], [7.0, -2.0, -1.0]], [[7.0, -7.0, -3.0], [3.0, 4.0, 1.0]], [[2.0, 6.0, 6.0], [-7.0, 5.0, -6.0]], [[-2.0, 0.0, -6.0], [0.0, -1.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-86.0, -19.0, 4.0, 11.0, 15.0], [-14.0, 29.0, 66.0, 30.0, 37.0], [3.0, -30.0, -12.0, 28.0, 11.0], [-5.0, 44.0, 47.0, -68.0, 18.0]], [[23.0, -58.0, -34.0, 88.0, 2.0], [-4.0, 30.0, 25.0, 1.0, -28.0], [2.0, -77.0, -44.0, -46.0, 23.0], [44.0, -35.0, 20.0, -37.0, 78.0]], [[17.0, -53.0, 51.0, 55.0, -3.0], [84.0, -52.0, -20.0, -99.0, 23.0], [42.0, -32.0, -15.0, -18.0, -38.0], [73.0, 27.0, -75.0, -44.0, 3.0]], [[-11.0, 2.0, 61.0, 5.0, 20.0], [58.0, -49.0, 14.0, -58.0, 19.0], [20.0, 14.0, -26.0, 47.0, -35.0], [27.0, 78.0, -35.0, -60.0, -13.0]], [[-84.0, 11.0, -48.0, -63.0, -19.0], [-44.0, 41.0, 58.0, 86.0, 13.0], [-45.0, 6.0, -24.0, 32.0, 23.0], [-69.0, 40.0, 69.0, -78.0, 36.0]]], [[[56.0, 23.0, -14.0, -11.0, 5.0], [-18.0, 1.0, -2.0, 20.0, -43.0], [-25.0, 32.0, -26.0, 2.0, 15.0], [-15.0, 6.0, 11.0, 10.0, 20.0]], [[-32.0, 0.0, 64.0, -4.0, 30.0], [5.0, -7.0, 23.0, -17.0, 51.0], [29.0, 37.0, 34.0, 22.0, 0.0], [15.0, 63.0, -6.0, 10.0, -53.0]], [[-42.0, -14.0, 8.0, 36.0, 4.0], [47.0, -23.0, 15.0, -39.0, 5.0], [15.0, -63.0, -36.0, 32.0, -36.0], [25.0, 5.0, -10.0, -72.0, 25.0]], [[20.0, 17.0, 18.0, 55.0, 35.0], [8.0, -9.0, -6.0, -28.0, -22.0], [23.0, -35.0, -2.0, 19.0, -15.0], [31.0, -38.0, 0.0, 35.0, -10.0]], [[9.0, 2.0, 26.0, -18.0, 2.0], [4.0, -10.0, -9.0, -11.0, 18.0], [8.0, 37.0, 22.0, -4.0, 1.0], [2.0, 25.0, -18.0, 21.0, -30.0]]], [[[-14.0, 3.0, -91.0, 41.0, -35.0], [-2.0, 16.0, -53.0, 4.0, -46.0], [-4.0, -124.0, 1.0, -25.0, -27.0], [-5.0, -174.0, 8.0, 36.0, 35.0]], [[-8.0, -60.0, -91.0, 29.0, -36.0], [-73.0, 86.0, 14.0, 80.0, -7.0], [-23.0, -55.0, 14.0, -94.0, 64.0], [-12.0, -84.0, 56.0, 22.0, 62.0]], [[87.0, 49.0, -23.0, -3.0, -23.0], [43.0, -57.0, -107.0, -60.0, -66.0], [1.0, -17.0, 5.0, -4.0, -54.0], [4.0, -114.0, -67.0, 82.0, -22.0]], [[-4.0, 65.0, 12.0, -14.0, 21.0], [-2.0, -19.0, -34.0, -11.0, -13.0], [12.0, 1.0, 33.0, 46.0, -31.0], [-16.0, -53.0, -3.0, 73.0, -58.0]], [[31.0, 68.0, -1.0, -10.0, 24.0], [0.0, -23.0, -14.0, -1.0, -51.0], [-13.0, 9.0, -23.0, 57.0, -23.0], [-20.0, -21.0, 9.0, 23.0, -9.0]]], [[[48.0, 67.0, 19.0, -12.0, 27.0], [0.0, -28.0, -33.0, -15.0, -38.0], [1.0, 31.0, 8.0, 41.0, -21.0], [-10.0, -25.0, -10.0, 67.0, -41.0]], [[-23.0, -23.0, -81.0, -1.0, -31.0], [-59.0, 63.0, 11.0, 74.0, -12.0], [-29.0, -41.0, 5.0, -48.0, 40.0], [-32.0, -68.0, 53.0, 8.0, 46.0]], [[50.0, 10.0, -35.0, -47.0, -20.0], [-22.0, 7.0, 13.0, 45.0, -42.0], [-52.0, 54.0, -50.0, -6.0, 28.0], [-39.0, 45.0, 18.0, -42.0, 51.0]], [[54.0, 82.0, -3.0, -9.0, 4.0], [20.0, -47.0, -101.0, -47.0, -48.0], [16.0, -10.0, 44.0, 26.0, -56.0], [-7.0, -127.0, -46.0, 132.0, -71.0]], [[21.0, -34.0, -28.0, 33.0, -24.0], [22.0, -4.0, -21.0, -22.0, -17.0], [1.0, -47.0, -19.0, -37.0, -7.0], [23.0, -42.0, -21.0, -17.0, 38.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, -64.0, 9.0, 53.0, 22.0], [-28.0, 16.0, -7.0, 3.0, 46.0], [13.0, 5.0, 46.0, 13.0, -81.0], [79.0, 36.0, 3.0, 25.0, -24.0]], [[-44.0, -45.0, 74.0, 49.0, -24.0], [13.0, 2.0, -39.0, -36.0, 22.0], [-16.0, 95.0, 10.0, 20.0, -68.0], [79.0, -29.0, 77.0, 6.0, 16.0]], [[-62.0, 58.0, 39.0, -18.0, 51.0], [-49.0, -25.0, 54.0, 35.0, 12.0], [25.0, -18.0, 34.0, -99.0, -100.0], [105.0, -48.0, 83.0, 68.0, -61.0]], [[-31.0, 15.0, 44.0, 33.0, 83.0], [-61.0, 12.0, 38.0, 9.0, 25.0], [-11.0, -66.0, -4.0, -29.0, -77.0], [67.0, 9.0, 57.0, -11.0, -44.0]], [[57.0, -126.0, -39.0, -19.0, -60.0], [10.0, 24.0, -53.0, -65.0, 56.0], [-51.0, 17.0, 2.0, 11.0, 53.0], [-73.0, 110.0, -13.0, -23.0, -2.0]]], [[[9.0, -12.0, 27.0, 27.0, -2.0], [24.0, 20.0, -33.0, -47.0, -26.0], [-39.0, 11.0, -62.0, 53.0, 57.0], [-63.0, 8.0, 15.0, -95.0, 42.0]], [[-3.0, 66.0, -21.0, 56.0, 97.0], [-32.0, 0.0, 51.0, 77.0, -45.0], [91.0, -60.0, 38.0, 0.0, -66.0], [74.0, -34.0, -50.0, 7.0, -23.0]], [[-39.0, -54.0, 59.0, 4.0, 3.0], [-50.0, 6.0, 7.0, -21.0, 95.0], [-59.0, -6.0, 18.0, -28.0, -92.0], [84.0, 30.0, 80.0, 55.0, -43.0]], [[-41.0, 18.0, 69.0, 70.0, 39.0], [-19.0, 11.0, 10.0, 17.0, 18.0], [-14.0, -6.0, -10.0, 64.0, -74.0], [83.0, -40.0, 16.0, 1.0, 16.0]], [[8.0, 51.0, -30.0, -5.0, 32.0], [-1.0, -8.0, 25.0, 34.0, -44.0], [46.0, -29.0, 6.0, -18.0, 12.0], [-9.0, -21.0, -31.0, -6.0, -6.0]]], [[[-16.0, -79.0, 19.0, -98.0, -158.0], [25.0, -22.0, -38.0, -45.0, 118.0], [-108.0, 80.0, 12.0, -6.0, 19.0], [-16.0, 21.0, 29.0, 126.0, 9.0]], [[26.0, -48.0, -49.0, -35.0, -117.0], [79.0, -23.0, -61.0, -18.0, -24.0], [39.0, 136.0, 44.0, 2.0, 52.0], [-34.0, -16.0, -37.0, 44.0, 39.0]], [[-19.0, 36.0, 22.0, -102.0, -72.0], [13.0, -19.0, 7.0, -28.0, 33.0], [-100.0, -14.0, -64.0, -17.0, 82.0], [-87.0, -14.0, 31.0, 25.0, 16.0]], [[1.0, 23.0, 5.0, 4.0, 7.0], [-13.0, 6.0, 21.0, 26.0, 25.0], [-29.0, -53.0, -18.0, 53.0, 4.0], [-4.0, -3.0, -47.0, 17.0, 13.0]], [[-5.0, -28.0, 60.0, 38.0, 8.0], [-10.0, 32.0, -18.0, -44.0, 38.0], [-93.0, -34.0, -72.0, 85.0, 20.0], [-32.0, 28.0, 19.0, -69.0, 33.0]]], [[[-4.0, 34.0, 33.0, 33.0, 31.0], [-4.0, 18.0, 8.0, 0.0, -9.0], [-40.0, -50.0, -60.0, 74.0, 26.0], [-29.0, -12.0, -19.0, -58.0, 34.0]], [[33.0, -70.0, -40.0, -42.0, -110.0], [57.0, -9.0, -57.0, -36.0, 13.0], [-10.0, 92.0, 20.0, 11.0, 62.0], [-57.0, 24.0, -29.0, 29.0, 30.0]], [[35.0, -54.0, 2.0, -11.0, -34.0], [37.0, 23.0, -58.0, -91.0, -25.0], [-60.0, 26.0, -74.0, 18.0, 112.0], [-130.0, 54.0, 35.0, -125.0, 36.0]], [[-11.0, 60.0, 8.0, -61.0, -38.0], [3.0, -13.0, 28.0, 21.0, 27.0], [-66.0, -50.0, -46.0, 34.0, 54.0], [-52.0, -28.0, -37.0, 41.0, 24.0]], [[-22.0, -16.0, 18.0, -47.0, -52.0], [9.0, -17.0, -12.0, -25.0, 27.0], [-29.0, 48.0, 6.0, -45.0, -2.0], [4.0, -8.0, 54.0, 42.0, -9.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[0.0, -6.0], [0.0, 5.0]], [[7.0, 3.0], [-1.0, 7.0]], [[5.0, -5.0], [2.0, -5.0]], [[7.0, 0.0], [3.0, 0.0]], [[0.0, 2.0], [4.0, -4.0]], [[0.0, 5.0], [6.0, 5.0]]]),self.functionspace)
      arg1=-2.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-0.0, 12.0], [-0.0, -10.0]], [[-14.0, -6.0], [2.0, -14.0]], [[-10.0, 10.0], [-4.0, 10.0]], [[-14.0, -0.0], [-6.0, -0.0]], [[-0.0, -4.0], [-8.0, 8.0]], [[-0.0, -10.0], [-12.0, -10.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[-5.0, -7.0], [7.0, 5.0]], [[-7.0, -1.0], [0.0, -3.0]], [[0.0, 7.0], [-4.0, -7.0]], [[0.0, 4.0], [3.0, -2.0]], [[-2.0, 6.0], [-2.0, -6.0]], [[7.0, 1.0], [-2.0, 6.0]]]),self.functionspace)
      arg1=numpy.array(-6.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[30.0, 42.0], [-42.0, -30.0]], [[42.0, 6.0], [-0.0, 18.0]], [[-0.0, -42.0], [24.0, 42.0]], [[-0.0, -24.0], [-18.0, 12.0]], [[12.0, -36.0], [12.0, 36.0]], [[-42.0, -6.0], [12.0, -36.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank1_offset1(self):
      arg0=Data(numpy.array([[[[1.0, -5.0, -6.0], [6.0, 1.0, -5.0]], [[1.0, -6.0, 3.0], [-1.0, 6.0, 3.0]]], [[[-5.0, 7.0, -6.0], [1.0, 5.0, 2.0]], [[-5.0, 0.0, 5.0], [-2.0, 3.0, 5.0]]], [[[3.0, -1.0, -3.0], [-6.0, 0.0, 2.0]], [[4.0, 2.0, -7.0], [7.0, 1.0, -2.0]]], [[[-2.0, 0.0, 0.0], [2.0, -1.0, 6.0]], [[5.0, -3.0, -3.0], [-7.0, -1.0, -3.0]]], [[[-7.0, -2.0, -1.0], [2.0, 3.0, 5.0]], [[4.0, 3.0, -1.0], [2.0, -5.0, 6.0]]], [[[2.0, -5.0, -3.0], [-5.0, -7.0, 5.0]], [[-3.0, 3.0, 0.0], [7.0, 4.0, -2.0]]]]),self.functionspace)
      arg1=numpy.array([3.0, 0.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-39.0, -17.0], [24.0, 18.0]], [[-57.0, 17.0], [20.0, 29.0]], [[-12.0, -4.0], [-37.0, 7.0]], [[-6.0, 48.0], [-6.0, -42.0]], [[-28.0, 41.0], [5.0, 48.0]], [[-15.0, 20.0], [-9.0, 7.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_array_rank1_offset0(self):
      arg0=Data(numpy.array([[[-6.0, -7.0], [4.0, 4.0]], [[0.0, 7.0], [-5.0, 2.0]], [[2.0, -7.0], [-1.0, 0.0]], [[3.0, -1.0], [-6.0, -7.0]], [[-5.0, -4.0], [-7.0, -7.0]], [[1.0, -4.0], [6.0, -5.0]]]),self.functionspace)
      arg1=numpy.array([0.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, 42.0], [0.0, 49.0]], [[0.0, -28.0], [0.0, -28.0]]], [[[0.0, 0.0], [0.0, -49.0]], [[0.0, 35.0], [0.0, -14.0]]], [[[0.0, -14.0], [0.0, 49.0]], [[0.0, 7.0], [0.0, 0.0]]], [[[0.0, -21.0], [0.0, 7.0]], [[0.0, 42.0], [0.0, 49.0]]], [[[0.0, 35.0], [0.0, 28.0]], [[0.0, 49.0], [0.0, 49.0]]], [[[0.0, -7.0], [0.0, 28.0]], [[0.0, -42.0], [0.0, 35.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank2_offset1(self):
      arg0=Data(numpy.array([[[[2.0, 1.0, 3.0], [-5.0, 6.0, 3.0]], [[-3.0, 4.0, -3.0], [0.0, -4.0, -7.0]]], [[[2.0, 6.0, 0.0], [6.0, 2.0, 5.0]], [[0.0, 3.0, 0.0], [-7.0, 0.0, 7.0]]], [[[3.0, -3.0, 4.0], [5.0, 4.0, -6.0]], [[5.0, -2.0, -1.0], [-2.0, -3.0, 1.0]]], [[[0.0, -6.0, 5.0], [4.0, 5.0, 5.0]], [[7.0, 2.0, -5.0], [0.0, 1.0, 7.0]]], [[[-2.0, 0.0, 0.0], [-6.0, 3.0, 0.0]], [[-6.0, 5.0, -2.0], [2.0, 6.0, 0.0]]], [[[0.0, -4.0, -2.0], [6.0, 2.0, 6.0]], [[1.0, 1.0, -6.0], [2.0, -2.0, 5.0]]]]),self.functionspace)
      arg1=numpy.array([[0.0, -3.0, 0.0], [-7.0, 3.0, -4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-3.0, -23.0], [-18.0, 41.0]], [[-12.0, 45.0], [12.0, 16.0]]], [[[-18.0, 4.0], [-6.0, -56.0]], [[-9.0, 9.0], [0.0, 21.0]]], [[[9.0, -46.0], [-12.0, 1.0]], [[6.0, -37.0], [9.0, 1.0]]], [[[18.0, -38.0], [-15.0, -33.0]], [[-6.0, -23.0], [-3.0, -25.0]]], [[[0.0, 14.0], [-9.0, 51.0]], [[-15.0, 65.0], [-18.0, 4.0]]], [[[12.0, -4.0], [-6.0, -60.0]], [[-3.0, 20.0], [6.0, -40.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[-3.0, 6.0], [0.0, -6.0]], [[6.0, 3.0], [4.0, -6.0]], [[-1.0, -2.0], [6.0, 2.0]], [[4.0, 6.0], [1.0, -6.0]], [[-4.0, 2.0], [3.0, 3.0]], [[5.0, 7.0], [-3.0, 3.0]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[-9.0, 18.0], [0.0, -18.0]], [[18.0, 9.0], [12.0, -18.0]], [[-3.0, -6.0], [18.0, 6.0]], [[12.0, 18.0], [3.0, -18.0]], [[-12.0, 6.0], [9.0, 9.0]], [[15.0, 21.0], [-9.0, 9.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank1_offset1(self):
      arg0=Data(numpy.array([[[[0.0, 6.0, -1.0], [1.0, 0.0, 0.0]], [[-5.0, 1.0, 0.0], [6.0, 5.0, -1.0]]], [[[4.0, 0.0, -6.0], [-4.0, -2.0, -5.0]], [[3.0, 6.0, 1.0], [-1.0, -1.0, 0.0]]], [[[-1.0, -2.0, 4.0], [2.0, 4.0, 0.0]], [[4.0, -6.0, -1.0], [2.0, -5.0, 2.0]]], [[[7.0, 5.0, 0.0], [6.0, 1.0, 4.0]], [[4.0, 6.0, 5.0], [5.0, 4.0, -5.0]]], [[[1.0, 3.0, 0.0], [-6.0, -3.0, -6.0]], [[-7.0, -7.0, 4.0], [1.0, 6.0, 1.0]]], [[[-5.0, 0.0, -2.0], [5.0, -4.0, 0.0]], [[6.0, -7.0, 5.0], [-6.0, 4.0, -6.0]]]]),self.functionspace)
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([-3.0, 2.0, 5.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[7.0, -3.0], [17.0, -13.0]], [[-42.0, -17.0], [8.0, 1.0]], [[19.0, 2.0], [-29.0, -6.0]], [[-11.0, 4.0], [25.0, -32.0]], [[3.0, -18.0], [27.0, 14.0]], [[5.0, -23.0], [-7.0, -4.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_Symbol_rank1_offset0(self):
      arg0=Data(numpy.array([[[-1.0, 3.0], [7.0, 6.0]], [[1.0, -1.0], [-2.0, 3.0]], [[5.0, -7.0], [6.0, 2.0]], [[-3.0, 2.0], [-2.0, 7.0]], [[-6.0, -5.0], [-2.0, 6.0]], [[4.0, -2.0], [6.0, 5.0]]]),self.functionspace)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-4.0, 6.0])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[4.0, -6.0], [-12.0, 18.0]], [[-28.0, 42.0], [-24.0, 36.0]]], [[[-4.0, 6.0], [4.0, -6.0]], [[8.0, -12.0], [-12.0, 18.0]]], [[[-20.0, 30.0], [28.0, -42.0]], [[-24.0, 36.0], [-8.0, 12.0]]], [[[12.0, -18.0], [-8.0, 12.0]], [[8.0, -12.0], [-28.0, 42.0]]], [[[24.0, -36.0], [20.0, -30.0]], [[8.0, -12.0], [-24.0, 36.0]]], [[[-16.0, 24.0], [8.0, -12.0]], [[-24.0, 36.0], [-20.0, 30.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank2_offset1(self):
      arg0=Data(numpy.array([[[[-4.0, 6.0, -2.0], [-6.0, 0.0, -7.0]], [[-5.0, -2.0, -2.0], [-5.0, 6.0, 1.0]]], [[[-2.0, -5.0, -6.0], [0.0, 4.0, -1.0]], [[4.0, 2.0, -2.0], [-4.0, 7.0, -7.0]]], [[[1.0, -6.0, 5.0], [-2.0, 7.0, -6.0]], [[3.0, -5.0, -2.0], [5.0, 1.0, 0.0]]], [[[-7.0, -3.0, 6.0], [3.0, -1.0, 0.0]], [[-3.0, -7.0, -7.0], [-4.0, 6.0, 2.0]]], [[[-4.0, 5.0, 4.0], [-7.0, -4.0, 5.0]], [[6.0, 4.0, -5.0], [1.0, 0.0, 1.0]]], [[[7.0, 0.0, 6.0], [-3.0, -4.0, -7.0]], [[5.0, -6.0, -5.0], [2.0, 0.0, 7.0]]]]),self.functionspace)
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-4.0, 6.0, 1.0], [-5.0, -4.0, 3.0]])
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[50.0, -10.0], [17.0, 9.0]], [[6.0, 27.0], [57.0, 4.0]]], [[[-28.0, 12.0], [23.0, -19.0]], [[-6.0, -34.0], [51.0, -29.0]]], [[[-35.0, 34.0], [44.0, -36.0]], [[-44.0, -1.0], [-14.0, -29.0]]], [[[16.0, 65.0], [-18.0, -11.0]], [[-37.0, 22.0], [54.0, 2.0]]], [[[50.0, 12.0], [9.0, 66.0]], [[-5.0, -61.0], [-3.0, -2.0]]], [[[-22.0, -17.0], [-19.0, 10.0]], [[-61.0, -16.0], [-1.0, 11.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[2.0, 5.0], [-7.0, -4.0]], [[4.0, 2.0], [0.0, -4.0]], [[3.0, -1.0], [1.0, 1.0]], [[-7.0, -5.0], [-6.0, 1.0]], [[3.0, -7.0], [-1.0, -2.0]], [[2.0, -5.0], [0.0, 5.0]]]),self.functionspace)
      arg1=Data(-4.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[-8.0, -20.0], [28.0, 16.0]], [[-16.0, -8.0], [-0.0, 16.0]], [[-12.0, 4.0], [-4.0, -4.0]], [[28.0, 20.0], [24.0, -4.0]], [[-12.0, 28.0], [4.0, 8.0]], [[-8.0, 20.0], [-0.0, -20.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[-7.0, 2.0, 0.0], [3.0, 1.0, 3.0]], [[-5.0, -6.0, -4.0], [-1.0, 3.0, 0.0]]], [[[0.0, 4.0, 2.0], [7.0, -2.0, 7.0]], [[3.0, 6.0, -6.0], [2.0, 4.0, -5.0]]], [[[0.0, -7.0, -6.0], [2.0, -6.0, -1.0]], [[7.0, 5.0, 2.0], [0.0, 2.0, 7.0]]], [[[-7.0, -6.0, 4.0], [-7.0, 2.0, 1.0]], [[5.0, 2.0, 2.0], [1.0, -5.0, -5.0]]], [[[0.0, 6.0, -2.0], [-5.0, 0.0, -3.0]], [[6.0, 3.0, -5.0], [-5.0, -1.0, -6.0]]], [[[1.0, -7.0, -5.0], [2.0, 6.0, 7.0]], [[7.0, -5.0, 0.0], [-3.0, 3.0, -3.0]]]]),self.functionspace)
      arg1=Data(numpy.array([1.0, 2.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[-3.0, 5.0], [-17.0, 5.0]], [[8.0, 3.0], [15.0, 10.0]], [[-14.0, -10.0], [17.0, 4.0]], [[-19.0, -3.0], [9.0, -9.0]], [[12.0, -5.0], [12.0, -7.0]], [[-13.0, 14.0], [-3.0, 3.0]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_constData_rank1_offset0(self):
      arg0=Data(numpy.array([[[4.0, 3.0], [-7.0, 3.0]], [[2.0, 0.0], [5.0, 5.0]], [[7.0, 0.0], [6.0, -7.0]], [[-6.0, -1.0], [7.0, -6.0]], [[4.0, -5.0], [1.0, 1.0]], [[-6.0, 0.0], [-6.0, 0.0]]]),self.functionspace)
      arg1=Data(numpy.array([0.0, -7.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, -28.0], [0.0, -21.0]], [[0.0, 49.0], [0.0, -21.0]]], [[[0.0, -14.0], [0.0, 0.0]], [[0.0, -35.0], [0.0, -35.0]]], [[[0.0, -49.0], [0.0, 0.0]], [[0.0, -42.0], [0.0, 49.0]]], [[[0.0, 42.0], [0.0, 7.0]], [[0.0, -49.0], [0.0, 42.0]]], [[[0.0, -28.0], [0.0, 35.0]], [[0.0, -7.0], [0.0, -7.0]]], [[[0.0, 42.0], [0.0, 0.0]], [[0.0, 42.0], [0.0, 0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[7.0, -7.0, 6.0], [-1.0, -1.0, -2.0]], [[7.0, 1.0, -1.0], [2.0, -1.0, -3.0]]], [[[3.0, 7.0, -5.0], [-1.0, -7.0, 2.0]], [[7.0, -7.0, 0.0], [-2.0, 0.0, -1.0]]], [[[7.0, 0.0, 0.0], [-5.0, -3.0, 4.0]], [[-6.0, -1.0, 4.0], [2.0, 0.0, -2.0]]], [[[-2.0, -2.0, 6.0], [4.0, -6.0, 3.0]], [[0.0, -7.0, -6.0], [7.0, 3.0, -4.0]]], [[[6.0, 3.0, -1.0], [6.0, 0.0, 7.0]], [[2.0, -6.0, -7.0], [0.0, -6.0, 6.0]]], [[[-3.0, -5.0, 0.0], [3.0, 3.0, 5.0]], [[-4.0, -7.0, 1.0], [-7.0, 7.0, 4.0]]]]),self.functionspace)
      arg1=Data(numpy.array([[-6.0, 5.0, 7.0], [-5.0, 3.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      ref=Data(numpy.array([[[[-35.0, -56.0], [-13.0, 2.0]], [[-44.0, -32.0], [-38.0, -13.0]]], [[[-18.0, 6.0], [-15.0, -16.0]], [[-77.0, -56.0], [5.0, 10.0]]], [[[-42.0, -35.0], [43.0, 16.0]], [[59.0, 27.0], [-26.0, -10.0]]], [[[44.0, 4.0], [-33.0, -38.0]], [[-77.0, -21.0], [-55.0, -26.0]]], [[[-28.0, -21.0], [13.0, -30.0]], [[-91.0, -28.0], [12.0, -18.0]]], [[[-7.0, 0.0], [32.0, -6.0]], [[-4.0, -1.0], [105.0, 56.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[5.0, 1.0], [4.0, -1.0]], [[6.0, -3.0], [1.0, 3.0]], [[-2.0, -6.0], [2.0, 1.0]], [[-5.0, 5.0], [2.0, -1.0]], [[0.0, 4.0], [1.0, -1.0]], [[6.0, 0.0], [-1.0, 1.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(3.0)+(1-msk_arg1)*(-4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[15.0, 3.0], [12.0, -3.0]], [[18.0, -9.0], [3.0, 9.0]], [[-6.0, -18.0], [6.0, 3.0]], [[-15.0, 15.0], [6.0, -3.0]], [[0.0, 12.0], [3.0, -3.0]], [[18.0, 0.0], [-3.0, 3.0]]])+(1.-msk_ref)*numpy.array([[[-20.0, -4.0], [-16.0, 4.0]], [[-24.0, 12.0], [-4.0, -12.0]], [[8.0, 24.0], [-8.0, -4.0]], [[20.0, -20.0], [-8.0, 4.0]], [[-0.0, -16.0], [-4.0, 4.0]], [[-24.0, -0.0], [4.0, -4.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank1_offset1(self):
      arg0=Data(numpy.array([[[[0.0, -1.0, 0.0], [3.0, 6.0, 1.0]], [[0.0, -3.0, 5.0], [-4.0, -4.0, -4.0]]], [[[-4.0, 4.0, 5.0], [-4.0, -4.0, 5.0]], [[-5.0, 3.0, -6.0], [-4.0, 0.0, 2.0]]], [[[-1.0, -2.0, 5.0], [2.0, 7.0, -4.0]], [[-6.0, -2.0, 1.0], [-7.0, 5.0, -4.0]]], [[[-5.0, 3.0, 2.0], [-2.0, -4.0, 5.0]], [[6.0, 7.0, -7.0], [4.0, 1.0, -6.0]]], [[[2.0, 0.0, 7.0], [6.0, -2.0, 3.0]], [[-6.0, 7.0, 0.0], [6.0, -5.0, -4.0]]], [[[4.0, -4.0, -5.0], [2.0, 0.0, 5.0]], [[1.0, 2.0, 2.0], [6.0, -2.0, 1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, 4.0, -6.0])+(1.-msk_arg1)*numpy.array([0.0, 6.0, -4.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-4.0, 0.0], [-42.0, 32.0]], [[10.0, -22.0], [78.0, 12.0]], [[-32.0, 40.0], [22.0, 86.0]], [[30.0, -34.0], [34.0, 16.0]], [[-54.0, -62.0], [64.0, -32.0]], [[-10.0, -42.0], [-10.0, -50.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, 32.0], [-38.0, -8.0]], [[4.0, -44.0], [42.0, -8.0]], [[-32.0, 58.0], [-16.0, 46.0]], [[10.0, -44.0], [70.0, 30.0]], [[-28.0, -24.0], [42.0, -14.0]], [[-4.0, -20.0], [4.0, -16.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank3_expandedData_rank1_offset0(self):
      arg0=Data(numpy.array([[[0.0, 7.0], [-6.0, 0.0]], [[-5.0, 2.0], [5.0, 7.0]], [[-3.0, 5.0], [0.0, 7.0]], [[4.0, 3.0], [0.0, 1.0]], [[7.0, -5.0], [6.0, -1.0]], [[-5.0, -7.0], [-1.0, -6.0]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 7.0])+(1.-msk_arg1)*numpy.array([5.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [49.0, 49.0]], [[-42.0, -42.0], [0.0, 0.0]]], [[[-35.0, -35.0], [14.0, 14.0]], [[35.0, 35.0], [49.0, 49.0]]], [[[-21.0, -21.0], [35.0, 35.0]], [[0.0, 0.0], [49.0, 49.0]]], [[[28.0, 28.0], [21.0, 21.0]], [[0.0, 0.0], [7.0, 7.0]]], [[[49.0, 49.0], [-35.0, -35.0]], [[42.0, 42.0], [-7.0, -7.0]]], [[[-35.0, -35.0], [-49.0, -49.0]], [[-7.0, -7.0], [-42.0, -42.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [35.0, 0.0]], [[-30.0, 0.0], [0.0, 0.0]]], [[[-25.0, 0.0], [10.0, 0.0]], [[25.0, 0.0], [35.0, 0.0]]], [[[-15.0, 0.0], [25.0, 0.0]], [[0.0, 0.0], [35.0, 0.0]]], [[[20.0, 0.0], [15.0, 0.0]], [[0.0, 0.0], [5.0, 0.0]]], [[[35.0, 0.0], [-25.0, 0.0]], [[30.0, 0.0], [-5.0, 0.0]]], [[[-25.0, 0.0], [-35.0, 0.0]], [[-5.0, 0.0], [-30.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank2_offset1(self):
      arg0=Data(numpy.array([[[[3.0, -6.0, -6.0], [6.0, -4.0, -2.0]], [[4.0, -4.0, 5.0], [-5.0, -5.0, 1.0]]], [[[3.0, -2.0, 3.0], [-3.0, -5.0, -3.0]], [[0.0, 4.0, -1.0], [-7.0, 2.0, 7.0]]], [[[4.0, -1.0, 4.0], [1.0, -4.0, 1.0]], [[-6.0, 4.0, 1.0], [2.0, 0.0, -4.0]]], [[[5.0, 6.0, 5.0], [0.0, -6.0, -1.0]], [[2.0, 7.0, -2.0], [4.0, 7.0, -1.0]]], [[[6.0, 2.0, 0.0], [7.0, 6.0, 1.0]], [[1.0, -6.0, 5.0], [4.0, -4.0, -3.0]]], [[[0.0, 5.0, 0.0], [-4.0, -3.0, 0.0]], [[3.0, -5.0, 5.0], [4.0, 0.0, -1.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[2.0, -4.0, 7.0], [-1.0, 7.0, -1.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 0.0, -2.0], [0.0, 7.0, 1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-12.0, -39.0], [14.0, -32.0]], [[59.0, -37.0], [17.0, -31.0]]], [[[35.0, -20.0], [-7.0, -29.0]], [[-23.0, 29.0], [27.0, 14.0]]], [[[40.0, -15.0], [25.0, -30.0]], [[-21.0, 33.0], [-24.0, 2.0]]], [[[21.0, 32.0], [17.0, -41.0]], [[-38.0, 49.0], [-27.0, 46.0]]], [[[4.0, 8.0], [-3.0, 34.0]], [[61.0, -48.0], [3.0, -29.0]]], [[[-20.0, 35.0], [4.0, -17.0]], [[61.0, -43.0], [1.0, -3.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -48.0], [-32.0, -30.0]], [[-34.0, -23.0], [28.0, -34.0]]], [[[-24.0, -11.0], [24.0, -38.0]], [[2.0, 27.0], [28.0, 21.0]]], [[[-32.0, -3.0], [-8.0, -27.0]], [[34.0, 29.0], [-4.0, -4.0]]], [[[-40.0, 47.0], [2.0, -43.0]], [[-8.0, 47.0], [-22.0, 48.0]]], [[[-36.0, 14.0], [-44.0, 43.0]], [[-16.0, -37.0], [-18.0, -31.0]]], [[[0.0, 35.0], [24.0, -21.0]], [[-28.0, -30.0], [-22.0, -1.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_float_rank0_offset0(self):
      arg0=Data(numpy.array([[[[7.0, 7.0, -6.0, 7.0], [5.0, 7.0, 4.0, 6.0], [0.0, 5.0, -3.0, -4.0]], [[5.0, 2.0, 6.0, 0.0], [0.0, 6.0, -6.0, -3.0], [-3.0, 6.0, 4.0, 4.0]]], [[[4.0, 1.0, 5.0, -3.0], [2.0, 3.0, 2.0, 5.0], [5.0, 0.0, 7.0, 0.0]], [[7.0, -6.0, -3.0, 3.0], [-3.0, 0.0, 2.0, -5.0], [-5.0, 5.0, -7.0, -1.0]]], [[[3.0, 3.0, 4.0, -6.0], [2.0, -6.0, 5.0, 6.0], [3.0, -4.0, 5.0, 0.0]], [[-3.0, -6.0, 4.0, 6.0], [6.0, -2.0, -3.0, -7.0], [5.0, 3.0, 7.0, 2.0]]]]),self.functionspace)
      arg1=4.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[28.0, 28.0, -24.0, 28.0], [20.0, 28.0, 16.0, 24.0], [0.0, 20.0, -12.0, -16.0]], [[20.0, 8.0, 24.0, 0.0], [0.0, 24.0, -24.0, -12.0], [-12.0, 24.0, 16.0, 16.0]]], [[[16.0, 4.0, 20.0, -12.0], [8.0, 12.0, 8.0, 20.0], [20.0, 0.0, 28.0, 0.0]], [[28.0, -24.0, -12.0, 12.0], [-12.0, 0.0, 8.0, -20.0], [-20.0, 20.0, -28.0, -4.0]]], [[[12.0, 12.0, 16.0, -24.0], [8.0, -24.0, 20.0, 24.0], [12.0, -16.0, 20.0, 0.0]], [[-12.0, -24.0, 16.0, 24.0], [24.0, -8.0, -12.0, -28.0], [20.0, 12.0, 28.0, 8.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_array_rank0_offset0(self):
      arg0=Data(numpy.array([[[[2.0, 7.0, -1.0, 1.0], [5.0, 6.0, -6.0, -7.0], [7.0, -6.0, -6.0, 0.0]], [[-2.0, 5.0, 0.0, 1.0], [3.0, 0.0, 0.0, -1.0], [-3.0, -6.0, 1.0, -5.0]]], [[[-1.0, 0.0, -6.0, 2.0], [2.0, 1.0, -7.0, -6.0], [-7.0, -1.0, -3.0, 4.0]], [[1.0, -5.0, 4.0, 4.0], [-4.0, 5.0, 1.0, 2.0], [-7.0, 1.0, 0.0, 1.0]]], [[[4.0, -1.0, -6.0, 7.0], [0.0, 7.0, 2.0, -6.0], [1.0, -5.0, -3.0, 0.0]], [[5.0, 4.0, -7.0, 2.0], [2.0, -4.0, 0.0, 0.0], [-6.0, -7.0, 1.0, -6.0]]]]),self.functionspace)
      arg1=numpy.array(2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[4.0, 14.0, -2.0, 2.0], [10.0, 12.0, -12.0, -14.0], [14.0, -12.0, -12.0, 0.0]], [[-4.0, 10.0, 0.0, 2.0], [6.0, 0.0, 0.0, -2.0], [-6.0, -12.0, 2.0, -10.0]]], [[[-2.0, 0.0, -12.0, 4.0], [4.0, 2.0, -14.0, -12.0], [-14.0, -2.0, -6.0, 8.0]], [[2.0, -10.0, 8.0, 8.0], [-8.0, 10.0, 2.0, 4.0], [-14.0, 2.0, 0.0, 2.0]]], [[[8.0, -2.0, -12.0, 14.0], [0.0, 14.0, 4.0, -12.0], [2.0, -10.0, -6.0, 0.0]], [[10.0, 8.0, -14.0, 4.0], [4.0, -8.0, 0.0, 0.0], [-12.0, -14.0, 2.0, -12.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_Symbol_rank0_offset0(self):
      arg0=Data(numpy.array([[[[-7.0, -5.0, -5.0, 0.0], [-7.0, -2.0, 5.0, 0.0], [5.0, -2.0, 3.0, 3.0]], [[-4.0, 4.0, 0.0, 0.0], [-4.0, 4.0, 0.0, 0.0], [5.0, -5.0, 2.0, -2.0]]], [[[2.0, -2.0, 2.0, 0.0], [-1.0, 0.0, -7.0, 7.0], [6.0, 0.0, 0.0, -3.0]], [[-3.0, -1.0, 4.0, -2.0], [4.0, 7.0, -2.0, 6.0], [0.0, -2.0, -6.0, 4.0]]], [[[2.0, 3.0, -2.0, 3.0], [2.0, 2.0, -1.0, -2.0], [-7.0, 0.0, 0.0, 4.0]], [[-3.0, 1.0, -5.0, 1.0], [-2.0, -5.0, 5.0, 1.0], [0.0, -7.0, 0.0, 7.0]]]]),self.functionspace)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-5.0)
      sub=res.substitute({arg1:s1})
      ref=Data(numpy.array([[[[35.0, 25.0, 25.0, -0.0], [35.0, 10.0, -25.0, -0.0], [-25.0, 10.0, -15.0, -15.0]], [[20.0, -20.0, -0.0, -0.0], [20.0, -20.0, -0.0, -0.0], [-25.0, 25.0, -10.0, 10.0]]], [[[-10.0, 10.0, -10.0, -0.0], [5.0, -0.0, 35.0, -35.0], [-30.0, -0.0, -0.0, 15.0]], [[15.0, 5.0, -20.0, 10.0], [-20.0, -35.0, 10.0, -30.0], [-0.0, 10.0, 30.0, -20.0]]], [[[-10.0, -15.0, 10.0, -15.0], [-10.0, -10.0, 5.0, 10.0], [35.0, -0.0, -0.0, -20.0]], [[15.0, -5.0, 25.0, -5.0], [10.0, 25.0, -25.0, -5.0], [-0.0, 35.0, -0.0, -35.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_constData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[5.0, -4.0, 0.0, -4.0], [-6.0, -1.0, -6.0, 4.0], [-7.0, -3.0, 4.0, 3.0]], [[-5.0, -3.0, 5.0, -2.0], [7.0, 7.0, -7.0, -6.0], [-4.0, 0.0, -6.0, 0.0]]], [[[1.0, -3.0, -4.0, 6.0], [1.0, 2.0, 2.0, -6.0], [4.0, -3.0, -5.0, -1.0]], [[3.0, 0.0, 6.0, -3.0], [0.0, 0.0, 4.0, -5.0], [0.0, -2.0, 4.0, -6.0]]], [[[-7.0, -3.0, -7.0, 4.0], [3.0, -3.0, -6.0, 6.0], [7.0, -3.0, -5.0, 2.0]], [[-4.0, 3.0, 0.0, -7.0], [-3.0, 3.0, -4.0, 0.0], [6.0, 5.0, -6.0, -3.0]]]]),self.functionspace)
      arg1=Data(0.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      ref=Data(numpy.array([[[[0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, 0.0, 0.0]], [[-0.0, -0.0, 0.0, -0.0], [0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0]]], [[[0.0, -0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0]], [[0.0, 0.0, 0.0, -0.0], [0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, -0.0]]], [[[-0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[-0.0, 0.0, 0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, 0.0, -0.0, -0.0]]]]),self.functionspace)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_constData_rank4_expandedData_rank0_offset0(self):
      arg0=Data(numpy.array([[[[1.0, 3.0, -7.0, -4.0], [-2.0, -3.0, 6.0, -6.0], [-7.0, -4.0, -1.0, 3.0]], [[-1.0, 0.0, 0.0, 1.0], [-1.0, -7.0, 3.0, 5.0], [-3.0, 2.0, 1.0, 5.0]]], [[[-1.0, 1.0, -4.0, 6.0], [0.0, 2.0, -6.0, -5.0], [-2.0, -5.0, -3.0, 0.0]], [[-4.0, -3.0, 0.0, -5.0], [-2.0, -1.0, 6.0, 0.0], [0.0, 7.0, 5.0, 0.0]]], [[[3.0, -5.0, -6.0, -4.0], [-7.0, 0.0, -4.0, 4.0], [-4.0, 3.0, -1.0, 7.0]], [[4.0, 7.0, 5.0, -2.0], [-4.0, -1.0, 7.0, 6.0], [-7.0, 1.0, -4.0, -6.0]]]]),self.functionspace)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(-6.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[1.0, 3.0, -7.0, -4.0], [-2.0, -3.0, 6.0, -6.0], [-7.0, -4.0, -1.0, 3.0]], [[-1.0, 0.0, 0.0, 1.0], [-1.0, -7.0, 3.0, 5.0], [-3.0, 2.0, 1.0, 5.0]]], [[[-1.0, 1.0, -4.0, 6.0], [0.0, 2.0, -6.0, -5.0], [-2.0, -5.0, -3.0, 0.0]], [[-4.0, -3.0, 0.0, -5.0], [-2.0, -1.0, 6.0, 0.0], [0.0, 7.0, 5.0, 0.0]]], [[[3.0, -5.0, -6.0, -4.0], [-7.0, 0.0, -4.0, 4.0], [-4.0, 3.0, -1.0, 7.0]], [[4.0, 7.0, 5.0, -2.0], [-4.0, -1.0, 7.0, 6.0], [-7.0, 1.0, -4.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -18.0, 42.0, 24.0], [12.0, 18.0, -36.0, 36.0], [42.0, 24.0, 6.0, -18.0]], [[6.0, -0.0, -0.0, -6.0], [6.0, 42.0, -18.0, -30.0], [18.0, -12.0, -6.0, -30.0]]], [[[6.0, -6.0, 24.0, -36.0], [-0.0, -12.0, 36.0, 30.0], [12.0, 30.0, 18.0, -0.0]], [[24.0, 18.0, -0.0, 30.0], [12.0, 6.0, -36.0, -0.0], [-0.0, -42.0, -30.0, -0.0]]], [[[-18.0, 30.0, 36.0, 24.0], [42.0, -0.0, 24.0, -24.0], [24.0, -18.0, 6.0, -42.0]], [[-24.0, -42.0, -30.0, 12.0], [24.0, 6.0, -42.0, -36.0], [42.0, -6.0, 24.0, 36.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-7.0)+(1-msk_arg0)*(7.0)
      arg1=-2.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(14.0)+(1.-msk_ref)*numpy.array(-14.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(6.0)+(1-msk_arg0)*(0.0)
      arg1=numpy.array(1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(6.0)+(1.-msk_ref)*numpy.array(0.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, 0.0, 4.0])+(1.-msk_arg0)*numpy.array([1.0, 4.0, 1.0])
      arg1=numpy.array([3.0, 7.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-19.0)+(1.-msk_ref)*numpy.array(24.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 6.0, 5.0], [3.0, 6.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-7.0, -4.0, -2.0], [-3.0, 5.0, 0.0]])
      arg1=numpy.array([[-5.0, 4.0, -2.0], [-2.0, -1.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-13.0)+(1.-msk_ref)*numpy.array(24.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 5.0, -3.0], [-2.0, 4.0, 0.0], [-5.0, -3.0, 6.0], [-4.0, 0.0, -5.0]], [[-4.0, -1.0, -1.0], [-4.0, 4.0, 4.0], [-3.0, 0.0, -2.0], [-6.0, -5.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 2.0, 0.0], [-6.0, -6.0, -3.0], [0.0, 6.0, -1.0], [7.0, 5.0, -7.0]], [[7.0, 5.0, 6.0], [-4.0, -1.0, -1.0], [0.0, 0.0, 0.0], [6.0, -4.0, -5.0]]])
      arg1=numpy.array([[[5.0, -6.0, 2.0], [3.0, 2.0, 2.0], [2.0, 5.0, 2.0], [-2.0, -6.0, -1.0]], [[-4.0, 0.0, 1.0], [1.0, 3.0, 3.0], [-4.0, -1.0, -3.0], [-1.0, 2.0, -3.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(15.0)+(1.-msk_ref)*numpy.array(-88.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, 6.0], [3.0, 0.0], [2.0, 1.0]], [[-1.0, -2.0], [1.0, -5.0], [-1.0, -3.0]]], [[[0.0, -3.0], [-3.0, 1.0], [5.0, 7.0]], [[6.0, 2.0], [-2.0, 6.0], [1.0, 1.0]]], [[[0.0, -6.0], [-4.0, 3.0], [2.0, 1.0]], [[-5.0, -2.0], [-2.0, 1.0], [2.0, -1.0]]], [[[6.0, -3.0], [0.0, -1.0], [0.0, 5.0]], [[6.0, -6.0], [-5.0, 1.0], [0.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, -6.0], [-5.0, 3.0], [-2.0, 4.0]], [[6.0, 5.0], [2.0, 1.0], [6.0, -4.0]]], [[[-3.0, -3.0], [-5.0, -5.0], [0.0, -4.0]], [[3.0, -3.0], [0.0, -3.0], [-1.0, -2.0]]], [[[3.0, 3.0], [6.0, 4.0], [0.0, 3.0]], [[7.0, -2.0], [5.0, 0.0], [6.0, 7.0]]], [[[1.0, 2.0], [3.0, 0.0], [7.0, -1.0]], [[4.0, 4.0], [4.0, -1.0], [-5.0, 6.0]]]])
      arg1=numpy.array([[[[-5.0, -7.0], [7.0, 4.0], [-7.0, 5.0]], [[0.0, 3.0], [0.0, 4.0], [-7.0, 6.0]]], [[[2.0, 0.0], [-2.0, 2.0], [2.0, -3.0]], [[-4.0, -3.0], [6.0, 6.0], [0.0, 0.0]]], [[[-6.0, 7.0], [-6.0, 7.0], [0.0, 7.0]], [[2.0, 0.0], [4.0, 6.0], [-4.0, 0.0]]], [[[0.0, 0.0], [1.0, -1.0], [-2.0, 4.0]], [[-2.0, 0.0], [1.0, 2.0], [-7.0, 6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-74.0)+(1.-msk_ref)*numpy.array(47.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-1.0)+(1-msk_arg0)*(-4.0)
      arg1=numpy.array([-3.0, -2.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 2.0])+(1.-msk_ref)*numpy.array([12.0, 8.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, -5.0, -6.0])+(1.-msk_arg0)*numpy.array([-5.0, 1.0, 6.0])
      arg1=numpy.array([[4.0, 3.0, 5.0], [5.0, 4.0, -4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-25.0, 29.0])+(1.-msk_ref)*numpy.array([13.0, -45.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 6.0, -5.0], [-6.0, -4.0, -3.0]])+(1.-msk_arg0)*numpy.array([[3.0, 0.0, 4.0], [7.0, 4.0, -1.0]])
      arg1=numpy.array([[[-4.0, 1.0, 3.0], [0.0, 4.0, 0.0]], [[1.0, -7.0, -3.0], [0.0, -6.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-25.0, 0.0])+(1.-msk_ref)*numpy.array([16.0, -32.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -5.0, -4.0], [5.0, -3.0, 0.0], [-1.0, 0.0, -6.0], [6.0, 0.0, 2.0]], [[6.0, -7.0, -1.0], [-3.0, 4.0, 5.0], [4.0, 4.0, -2.0], [4.0, -7.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, -3.0, -1.0], [-7.0, -1.0, 1.0], [6.0, 4.0, 3.0], [-5.0, -5.0, 0.0]], [[5.0, 3.0, 3.0], [-3.0, 5.0, -4.0], [-5.0, 0.0, 4.0], [2.0, 7.0, 5.0]]])
      arg1=numpy.array([[[[5.0, -4.0, -4.0], [-4.0, 5.0, -5.0], [6.0, -4.0, -5.0], [3.0, 1.0, -4.0]], [[-7.0, -1.0, -7.0], [3.0, -2.0, -4.0], [-5.0, 5.0, 0.0], [6.0, -6.0, -2.0]]], [[[-5.0, 3.0, 1.0], [1.0, 7.0, 1.0], [6.0, -5.0, 0.0], [-4.0, 1.0, -5.0]], [[0.0, -7.0, -7.0], [-1.0, 0.0, 4.0], [0.0, 3.0, -4.0], [1.0, -7.0, 2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([44.0, 69.0])+(1.-msk_ref)*numpy.array([-78.0, -80.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(6.0)
      arg1=numpy.array([[-6.0, 4.0, -2.0, -2.0, -5.0], [-7.0, -5.0, -4.0, -7.0, 7.0], [-1.0, -7.0, -2.0, 4.0, 1.0], [-7.0, 0.0, 0.0, 1.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[30.0, -20.0, 10.0, 10.0, 25.0], [35.0, 25.0, 20.0, 35.0, -35.0], [5.0, 35.0, 10.0, -20.0, -5.0], [35.0, -0.0, -0.0, -5.0, 25.0]])+(1.-msk_ref)*numpy.array([[-36.0, 24.0, -12.0, -12.0, -30.0], [-42.0, -30.0, -24.0, -42.0, 42.0], [-6.0, -42.0, -12.0, 24.0, 6.0], [-42.0, 0.0, 0.0, 6.0, -30.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, 3.0, 4.0])+(1.-msk_arg0)*numpy.array([-7.0, 0.0, -4.0])
      arg1=numpy.array([[[-6.0, 5.0, -7.0], [2.0, 5.0, 3.0], [1.0, -2.0, -7.0], [0.0, 6.0, -7.0], [-7.0, -7.0, -4.0]], [[-5.0, 7.0, 7.0], [-1.0, -7.0, -2.0], [2.0, 5.0, -7.0], [-2.0, -1.0, 4.0], [3.0, -7.0, -3.0]], [[7.0, 5.0, 2.0], [2.0, -7.0, -6.0], [-1.0, 4.0, -1.0], [4.0, -1.0, 7.0], [2.0, -3.0, -4.0]], [[7.0, -2.0, 4.0], [-2.0, 6.0, 1.0], [0.0, 3.0, 0.0], [-3.0, 5.0, 3.0], [-2.0, 2.0, 1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-13.0, 27.0, -34.0, -10.0, -37.0], [49.0, -29.0, -13.0, 13.0, -33.0], [23.0, -45.0, 8.0, 25.0, -25.0], [10.0, 22.0, 9.0, 27.0, 10.0]])+(1.-msk_ref)*numpy.array([[70.0, -26.0, 21.0, 28.0, 65.0], [7.0, 15.0, 14.0, -2.0, -9.0], [-57.0, 10.0, 11.0, -56.0, 2.0], [-65.0, 10.0, 0.0, 9.0, 10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 7.0, 6.0], [1.0, 4.0, -3.0]])+(1.-msk_arg0)*numpy.array([[-5.0, -2.0, -6.0], [-6.0, 7.0, 3.0]])
      arg1=numpy.array([[[[5.0, 2.0, 4.0], [0.0, -1.0, 2.0]], [[1.0, 7.0, -4.0], [6.0, 0.0, 1.0]], [[3.0, 6.0, -5.0], [2.0, 5.0, 7.0]], [[0.0, 5.0, -3.0], [0.0, -4.0, -3.0]], [[4.0, -4.0, -2.0], [-6.0, 0.0, -2.0]]], [[[-5.0, 7.0, -4.0], [0.0, 3.0, -7.0]], [[-4.0, 6.0, 0.0], [2.0, -6.0, -6.0]], [[-7.0, -1.0, -7.0], [7.0, 0.0, 6.0]], [[-1.0, -7.0, -1.0], [-5.0, 7.0, -3.0]], [[-6.0, -4.0, 5.0], [-6.0, -4.0, 0.0]]], [[[-1.0, 3.0, -2.0], [-7.0, -4.0, 0.0]], [[-2.0, 6.0, 1.0], [-7.0, 5.0, -4.0]], [[5.0, 5.0, -6.0], [1.0, -6.0, 7.0]], [[-2.0, 4.0, 0.0], [3.0, -3.0, -6.0]], [[-2.0, 2.0, -1.0], [0.0, -5.0, 4.0]]], [[[-5.0, 5.0, 4.0], [7.0, -3.0, -7.0]], [[3.0, 5.0, -2.0], [-1.0, -4.0, 1.0]], [[4.0, -2.0, -2.0], [7.0, -7.0, 2.0]], [[2.0, -2.0, -6.0], [4.0, -4.0, -4.0]], [[-5.0, 4.0, -2.0], [4.0, 0.0, 0.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[8.0, 24.0, 1.0, 10.0, -56.0], [78.0, 54.0, -32.0, -19.0, 4.0], [-10.0, 81.0, -65.0, 45.0, -16.0], [95.0, -9.0, -69.0, -58.0, 40.0]])+(1.-msk_ref)*numpy.array([[-54.0, -28.0, 47.0, -29.0, 30.0], [35.0, -64.0, 55.0, 95.0, 16.0], [25.0, 57.0, -26.0, -55.0, -11.0], [-93.0, -32.0, -89.0, -34.0, 5.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(-7.0)
      arg1=numpy.array([[[-4.0, -1.0], [-7.0, 2.0]], [[-1.0, 7.0], [7.0, 6.0]], [[5.0, 0.0], [-3.0, -5.0]], [[-2.0, 6.0], [-6.0, 3.0]], [[-2.0, -4.0], [4.0, 6.0]], [[-7.0, -6.0], [3.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[16.0, 4.0], [28.0, -8.0]], [[4.0, -28.0], [-28.0, -24.0]], [[-20.0, -0.0], [12.0, 20.0]], [[8.0, -24.0], [24.0, -12.0]], [[8.0, 16.0], [-16.0, -24.0]], [[28.0, 24.0], [-12.0, -0.0]]])+(1.-msk_ref)*numpy.array([[[28.0, 7.0], [49.0, -14.0]], [[7.0, -49.0], [-49.0, -42.0]], [[-35.0, -0.0], [21.0, 35.0]], [[14.0, -42.0], [42.0, -21.0]], [[14.0, 28.0], [-28.0, -42.0]], [[49.0, 42.0], [-21.0, -0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, -3.0, 7.0])+(1.-msk_arg0)*numpy.array([-1.0, 5.0, 5.0])
      arg1=numpy.array([[[[2.0, 1.0, -5.0], [-1.0, -1.0, -7.0]], [[3.0, -7.0, -6.0], [-3.0, 0.0, -5.0]]], [[[7.0, 7.0, -4.0], [1.0, 5.0, -4.0]], [[-5.0, 7.0, 3.0], [0.0, 3.0, 7.0]]], [[[-4.0, -6.0, 7.0], [7.0, 4.0, 2.0]], [[-2.0, 1.0, 3.0], [-2.0, 4.0, 3.0]]], [[[7.0, 4.0, 2.0], [6.0, -6.0, -5.0]], [[6.0, -1.0, 2.0], [-2.0, -6.0, -7.0]]], [[[-1.0, 7.0, 3.0], [4.0, 0.0, 2.0]], [[7.0, -1.0, -1.0], [1.0, -2.0, 2.0]]], [[[-2.0, 4.0, 3.0], [3.0, 6.0, 1.0]], [[1.0, -1.0, -4.0], [4.0, 2.0, 1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-46.0, -42.0], [-33.0, -23.0]], [[-77.0, -47.0], [20.0, 40.0]], [[83.0, -26.0], [26.0, 17.0]], [[-26.0, -41.0], [-7.0, -23.0]], [[4.0, -2.0], [-32.0, 16.0]], [[17.0, -23.0], [-29.0, -15.0]]])+(1.-msk_ref)*numpy.array([[[-22.0, -39.0], [-68.0, -22.0]], [[8.0, 4.0], [55.0, 50.0]], [[9.0, 23.0], [22.0, 37.0]], [[23.0, -61.0], [-1.0, -63.0]], [[51.0, 6.0], [-17.0, -1.0]], [[37.0, 32.0], [-26.0, 11.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_array_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-5.0)+(1-msk_arg0)*(1.0)
      arg1=numpy.array([[[[-2.0, -2.0, -5.0, 5.0], [-2.0, -5.0, 6.0, -6.0], [-2.0, -4.0, 2.0, 7.0]], [[5.0, -7.0, 0.0, -2.0], [-7.0, -4.0, 7.0, 3.0], [0.0, 4.0, -3.0, 7.0]]], [[[2.0, 1.0, -4.0, -1.0], [-4.0, 0.0, 2.0, 4.0], [7.0, 5.0, 2.0, -7.0]], [[-2.0, -1.0, 0.0, -4.0], [3.0, 2.0, 3.0, -5.0], [4.0, 5.0, 0.0, -1.0]]], [[[-6.0, 7.0, -7.0, -1.0], [3.0, 6.0, -1.0, 7.0], [-2.0, 2.0, -2.0, -3.0]], [[4.0, 6.0, 7.0, -5.0], [7.0, -4.0, -3.0, -7.0], [-7.0, -6.0, -4.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 10.0, 25.0, -25.0], [10.0, 25.0, -30.0, 30.0], [10.0, 20.0, -10.0, -35.0]], [[-25.0, 35.0, -0.0, 10.0], [35.0, 20.0, -35.0, -15.0], [-0.0, -20.0, 15.0, -35.0]]], [[[-10.0, -5.0, 20.0, 5.0], [20.0, -0.0, -10.0, -20.0], [-35.0, -25.0, -10.0, 35.0]], [[10.0, 5.0, -0.0, 20.0], [-15.0, -10.0, -15.0, 25.0], [-20.0, -25.0, -0.0, 5.0]]], [[[30.0, -35.0, 35.0, 5.0], [-15.0, -30.0, 5.0, -35.0], [10.0, -10.0, 10.0, 15.0]], [[-20.0, -30.0, -35.0, 25.0], [-35.0, 20.0, 15.0, 35.0], [35.0, 30.0, 20.0, 20.0]]]])+(1.-msk_ref)*numpy.array([[[[-2.0, -2.0, -5.0, 5.0], [-2.0, -5.0, 6.0, -6.0], [-2.0, -4.0, 2.0, 7.0]], [[5.0, -7.0, 0.0, -2.0], [-7.0, -4.0, 7.0, 3.0], [0.0, 4.0, -3.0, 7.0]]], [[[2.0, 1.0, -4.0, -1.0], [-4.0, 0.0, 2.0, 4.0], [7.0, 5.0, 2.0, -7.0]], [[-2.0, -1.0, 0.0, -4.0], [3.0, 2.0, 3.0, -5.0], [4.0, 5.0, 0.0, -1.0]]], [[[-6.0, 7.0, -7.0, -1.0], [3.0, 6.0, -1.0, 7.0], [-2.0, 2.0, -2.0, -3.0]], [[4.0, 6.0, 7.0, -5.0], [7.0, -4.0, -3.0, -7.0], [-7.0, -6.0, -4.0, -4.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-3.0)+(1-msk_arg0)*(-3.0)
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(3.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-9.0)+(1.-msk_ref)*numpy.array(-9.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, -1.0, -5.0])+(1.-msk_arg0)*numpy.array([4.0, -4.0, 0.0])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([2.0, -3.0, -3.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(18.0)+(1.-msk_ref)*numpy.array(20.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 3.0, -3.0], [0.0, 1.0, 7.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0, 2.0], [4.0, -5.0, 0.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[-2.0, 1.0, -4.0], [-6.0, 5.0, -5.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-11.0)+(1.-msk_ref)*numpy.array(-56.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 1.0, 0.0], [0.0, -2.0, 3.0], [-5.0, 7.0, -3.0], [-1.0, 4.0, -4.0]], [[1.0, 3.0, -3.0], [5.0, -5.0, 4.0], [5.0, -7.0, -3.0], [-4.0, 1.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 0.0, 3.0], [5.0, -6.0, 4.0], [-2.0, -2.0, -4.0], [0.0, -7.0, -4.0]], [[0.0, 1.0, 4.0], [-1.0, -3.0, 2.0], [-6.0, -7.0, -7.0], [2.0, -1.0, -5.0]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[-7.0, -2.0, -7.0], [-6.0, 0.0, -7.0], [6.0, 2.0, -7.0], [-3.0, -1.0, -5.0]], [[4.0, 5.0, -5.0], [5.0, 2.0, 5.0], [0.0, -5.0, 2.0], [3.0, -1.0, 2.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(125.0)+(1.-msk_ref)*numpy.array(-24.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[3.0, 5.0], [1.0, -2.0], [-4.0, -7.0]], [[7.0, -4.0], [-5.0, -6.0], [0.0, -7.0]]], [[[1.0, -2.0], [-2.0, 2.0], [0.0, 0.0]], [[0.0, 2.0], [-3.0, 0.0], [-6.0, -2.0]]], [[[6.0, 7.0], [0.0, 0.0], [2.0, -1.0]], [[-5.0, 4.0], [-6.0, -1.0], [-6.0, -6.0]]], [[[-3.0, -3.0], [-4.0, 2.0], [0.0, 3.0]], [[-4.0, 2.0], [-3.0, 2.0], [-1.0, 1.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 4.0], [-7.0, 4.0], [7.0, -2.0]], [[1.0, 1.0], [5.0, 3.0], [7.0, 1.0]]], [[[0.0, -3.0], [-6.0, 3.0], [-7.0, 1.0]], [[6.0, 2.0], [1.0, -4.0], [1.0, 2.0]]], [[[-7.0, 1.0], [-6.0, 7.0], [-4.0, -7.0]], [[-7.0, -1.0], [4.0, -5.0], [-3.0, -3.0]]], [[[-2.0, -7.0], [3.0, 0.0], [6.0, -4.0]], [[7.0, 3.0], [7.0, -5.0], [4.0, -1.0]]]])
      arg1=Symbol(shape=(4, 2, 3, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      s1=numpy.array([[[[-1.0, 6.0], [0.0, -7.0], [0.0, 6.0]], [[6.0, 3.0], [0.0, 6.0], [-5.0, -7.0]]], [[[-3.0, -4.0], [5.0, -2.0], [5.0, -6.0]], [[-4.0, 4.0], [-6.0, -3.0], [-1.0, 6.0]]], [[[4.0, 5.0], [-7.0, 2.0], [-1.0, 6.0]], [[-1.0, 0.0], [-4.0, -5.0], [4.0, 0.0]]], [[[5.0, 3.0], [-7.0, 0.0], [-6.0, -4.0]], [[0.0, -3.0], [2.0, 0.0], [-1.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(95.0)+(1.-msk_ref)*numpy.array(-168.0)
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-7.0)+(1-msk_arg0)*(1.0)
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-2.0, 6.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([14.0, -42.0])+(1.-msk_ref)*numpy.array([-2.0, 6.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, -3.0, 4.0])+(1.-msk_arg0)*numpy.array([-5.0, -6.0, -7.0])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[3.0, -4.0, -3.0], [-1.0, -2.0, 7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([21.0, 27.0])+(1.-msk_ref)*numpy.array([30.0, -32.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 1.0, -7.0], [0.0, -7.0, 4.0]])+(1.-msk_arg0)*numpy.array([[0.0, 2.0, -3.0], [-6.0, 4.0, 1.0]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[0.0, 0.0, -3.0], [0.0, 6.0, 5.0]], [[-3.0, -3.0, 2.0], [1.0, -5.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-1.0, 36.0])+(1.-msk_ref)*numpy.array([38.0, -38.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 0.0, 6.0], [-6.0, -4.0, 2.0], [-2.0, -4.0, -2.0], [-4.0, 0.0, 0.0]], [[2.0, 5.0, 1.0], [-6.0, -1.0, 0.0], [0.0, -7.0, -3.0], [0.0, 3.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 5.0, 0.0], [3.0, 4.0, -7.0], [-2.0, -5.0, -2.0], [-7.0, 0.0, -4.0]], [[0.0, 6.0, -4.0], [1.0, -5.0, 0.0], [-6.0, 0.0, 4.0], [2.0, -1.0, -7.0]]])
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[-3.0, 0.0, 3.0], [4.0, -7.0, -7.0], [1.0, -6.0, -4.0], [-3.0, 5.0, 4.0]], [[0.0, -3.0, -5.0], [-3.0, 1.0, 7.0], [2.0, 5.0, 5.0], [7.0, -5.0, 0.0]]], [[[5.0, -1.0, -7.0], [-6.0, 7.0, -5.0], [7.0, 0.0, 3.0], [-3.0, -1.0, 6.0]], [[-2.0, 5.0, 4.0], [-6.0, 5.0, 3.0], [2.0, -3.0, 7.0], [0.0, 2.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([3.0, 5.0])+(1.-msk_ref)*numpy.array([98.0, -33.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-1.0)+(1-msk_arg0)*(-3.0)
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[6.0, -5.0, -5.0, 5.0, 2.0], [0.0, -2.0, 2.0, 0.0, 0.0], [2.0, -3.0, 0.0, 1.0, -2.0], [0.0, -4.0, -5.0, -3.0, 2.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-6.0, 5.0, 5.0, -5.0, -2.0], [-0.0, 2.0, -2.0, -0.0, -0.0], [-2.0, 3.0, -0.0, -1.0, 2.0], [-0.0, 4.0, 5.0, 3.0, -2.0]])+(1.-msk_ref)*numpy.array([[-18.0, 15.0, 15.0, -15.0, -6.0], [-0.0, 6.0, -6.0, -0.0, -0.0], [-6.0, 9.0, -0.0, -3.0, 6.0], [-0.0, 12.0, 15.0, 9.0, -6.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, 2.0, 4.0])+(1.-msk_arg0)*numpy.array([2.0, 3.0, -7.0])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[-7.0, -5.0, 5.0], [7.0, 0.0, -6.0], [0.0, 3.0, 5.0], [0.0, 0.0, 4.0], [3.0, 0.0, -4.0]], [[6.0, 7.0, -5.0], [-7.0, 4.0, 3.0], [1.0, 3.0, 4.0], [2.0, -6.0, 7.0], [-6.0, -1.0, -6.0]], [[-2.0, -2.0, -6.0], [-2.0, 4.0, -4.0], [2.0, 0.0, -2.0], [-4.0, -6.0, -1.0], [-3.0, -3.0, 6.0]], [[-7.0, -5.0, 4.0], [0.0, -7.0, 6.0], [-1.0, -7.0, -4.0], [3.0, 0.0, -3.0], [4.0, 0.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, -17.0, 26.0, 16.0, -13.0], [0.0, 13.0, 23.0, 18.0, -32.0], [-30.0, -10.0, -6.0, -20.0, 15.0], [-1.0, 10.0, -31.0, -9.0, -12.0]])+(1.-msk_ref)*numpy.array([[-64.0, 56.0, -26.0, -28.0, 34.0], [68.0, -23.0, -17.0, -63.0, 27.0], [32.0, 36.0, 18.0, -19.0, -57.0], [-57.0, -63.0, 5.0, 27.0, 36.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, 6.0, 4.0], [6.0, 3.0, 1.0]])+(1.-msk_arg0)*numpy.array([[0.0, -5.0, 0.0], [3.0, 0.0, 7.0]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-1.0, -4.0, -1.0], [-7.0, -1.0, -7.0]], [[-7.0, -4.0, -3.0], [0.0, 0.0, -3.0]], [[3.0, 4.0, 2.0], [5.0, 4.0, 3.0]], [[-6.0, 0.0, -6.0], [5.0, 7.0, 0.0]], [[6.0, 6.0, 7.0], [0.0, 2.0, -6.0]]], [[[3.0, 2.0, -5.0], [3.0, -6.0, -6.0]], [[-7.0, -5.0, -4.0], [2.0, 0.0, 6.0]], [[0.0, -5.0, 0.0], [-7.0, 3.0, 0.0]], [[3.0, 0.0, -3.0], [1.0, -1.0, -4.0]], [[-3.0, 0.0, 7.0], [0.0, 2.0, -5.0]]], [[[-6.0, 4.0, 1.0], [-7.0, 0.0, 1.0]], [[-2.0, -3.0, -5.0], [-3.0, 6.0, 0.0]], [[3.0, 6.0, 5.0], [3.0, -7.0, 5.0]], [[1.0, -5.0, 4.0], [-1.0, 0.0, -4.0]], [[0.0, 4.0, 6.0], [4.0, -1.0, 4.0]]], [[[-1.0, 5.0, -1.0], [-6.0, -3.0, 2.0]], [[4.0, -1.0, 6.0], [-3.0, 1.0, -2.0]], [[3.0, -6.0, -5.0], [-4.0, -4.0, 1.0]], [[3.0, 5.0, -4.0], [0.0, -4.0, 3.0]], [[6.0, -3.0, 2.0], [-3.0, 4.0, -1.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-74.0, 3.0, 59.0, 63.0, 28.0], [-32.0, 14.0, -63.0, -31.0, 47.0], [23.0, -26.0, 40.0, -30.0, 73.0], [-11.0, -23.0, -109.0, -13.0, -53.0]])+(1.-msk_ref)*numpy.array([[-50.0, -1.0, 16.0, 15.0, -72.0], [-43.0, 73.0, 4.0, -25.0, -35.0], [-34.0, 6.0, 14.0, -6.0, 20.0], [-29.0, -18.0, 25.0, -4.0, -1.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-4.0)+(1-msk_arg0)*(-6.0)
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[0.0, 1.0], [-3.0, 4.0]], [[-2.0, -5.0], [-7.0, -3.0]], [[-5.0, 6.0], [7.0, -7.0]], [[-4.0, -3.0], [-5.0, -7.0]], [[4.0, -4.0], [0.0, -3.0]], [[-2.0, 2.0], [2.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-0.0, -4.0], [12.0, -16.0]], [[8.0, 20.0], [28.0, 12.0]], [[20.0, -24.0], [-28.0, 28.0]], [[16.0, 12.0], [20.0, 28.0]], [[-16.0, 16.0], [-0.0, 12.0]], [[8.0, -8.0], [-8.0, -28.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, -6.0], [18.0, -24.0]], [[12.0, 30.0], [42.0, 18.0]], [[30.0, -36.0], [-42.0, 42.0]], [[24.0, 18.0], [30.0, 42.0]], [[-24.0, 24.0], [-0.0, 18.0]], [[12.0, -12.0], [-12.0, -42.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, 5.0, 0.0])+(1.-msk_arg0)*numpy.array([0.0, -2.0, -7.0])
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[3.0, -6.0, 1.0], [-7.0, 6.0, 7.0]], [[1.0, 0.0, 1.0], [-7.0, -5.0, 0.0]]], [[[-2.0, -1.0, 7.0], [7.0, -2.0, -5.0]], [[7.0, 4.0, 0.0], [2.0, 3.0, 0.0]]], [[[-1.0, 0.0, 3.0], [-5.0, 1.0, 2.0]], [[-5.0, -1.0, 0.0], [-5.0, 2.0, 4.0]]], [[[-2.0, -7.0, -4.0], [-6.0, -3.0, 5.0]], [[0.0, -7.0, -6.0], [-3.0, 4.0, 7.0]]], [[[-5.0, -2.0, -2.0], [-4.0, 3.0, 3.0]], [[0.0, 0.0, 0.0], [6.0, 7.0, 3.0]]], [[[4.0, 7.0, 1.0], [0.0, 6.0, -5.0]], [[-6.0, 5.0, 0.0], [0.0, 2.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-42.0, 58.0], [-4.0, 3.0]], [[3.0, -38.0], [-8.0, 7.0]], [[4.0, 25.0], [15.0, 30.0]], [[-27.0, 9.0], [-35.0, 32.0]], [[10.0, 31.0], [0.0, 11.0]], [[19.0, 30.0], [49.0, 10.0]]])+(1.-msk_ref)*numpy.array([[[5.0, -61.0], [-7.0, 10.0]], [[-47.0, 39.0], [-8.0, -6.0]], [[-21.0, -16.0], [2.0, -32.0]], [[42.0, -29.0], [56.0, -57.0]], [[18.0, -27.0], [0.0, -35.0]], [[-21.0, 23.0], [-10.0, -4.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_Symbol_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-1.0)+(1-msk_arg0)*(3.0)
      arg1=Symbol(shape=(3, 2, 3, 4))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[[3.0, -6.0, -5.0, -4.0], [0.0, -3.0, 5.0, -3.0], [-5.0, 2.0, 4.0, 3.0]], [[-3.0, -3.0, 5.0, 6.0], [2.0, 5.0, -5.0, 0.0], [5.0, 6.0, -1.0, 6.0]]], [[[-4.0, 2.0, 2.0, 7.0], [6.0, 6.0, 1.0, 1.0], [0.0, -7.0, 0.0, 0.0]], [[4.0, 3.0, -6.0, -1.0], [-5.0, -3.0, -3.0, -5.0], [5.0, -2.0, 0.0, 0.0]]], [[[-7.0, 7.0, -2.0, 5.0], [-1.0, -3.0, 3.0, -1.0], [-5.0, -4.0, -1.0, 0.0]], [[-1.0, -4.0, -5.0, -3.0], [-5.0, -5.0, 0.0, 6.0], [3.0, -6.0, 0.0, -7.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, 6.0, 5.0, 4.0], [-0.0, 3.0, -5.0, 3.0], [5.0, -2.0, -4.0, -3.0]], [[3.0, 3.0, -5.0, -6.0], [-2.0, -5.0, 5.0, -0.0], [-5.0, -6.0, 1.0, -6.0]]], [[[4.0, -2.0, -2.0, -7.0], [-6.0, -6.0, -1.0, -1.0], [-0.0, 7.0, -0.0, -0.0]], [[-4.0, -3.0, 6.0, 1.0], [5.0, 3.0, 3.0, 5.0], [-5.0, 2.0, -0.0, -0.0]]], [[[7.0, -7.0, 2.0, -5.0], [1.0, 3.0, -3.0, 1.0], [5.0, 4.0, 1.0, -0.0]], [[1.0, 4.0, 5.0, 3.0], [5.0, 5.0, -0.0, -6.0], [-3.0, 6.0, -0.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[9.0, -18.0, -15.0, -12.0], [0.0, -9.0, 15.0, -9.0], [-15.0, 6.0, 12.0, 9.0]], [[-9.0, -9.0, 15.0, 18.0], [6.0, 15.0, -15.0, 0.0], [15.0, 18.0, -3.0, 18.0]]], [[[-12.0, 6.0, 6.0, 21.0], [18.0, 18.0, 3.0, 3.0], [0.0, -21.0, 0.0, 0.0]], [[12.0, 9.0, -18.0, -3.0], [-15.0, -9.0, -9.0, -15.0], [15.0, -6.0, 0.0, 0.0]]], [[[-21.0, 21.0, -6.0, 15.0], [-3.0, -9.0, 9.0, -3.0], [-15.0, -12.0, -3.0, 0.0]], [[-3.0, -12.0, -15.0, -9.0], [-15.0, -15.0, 0.0, 18.0], [9.0, -18.0, 0.0, -21.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-7.0)+(1-msk_arg0)*(7.0)
      arg1=Data(-2.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(14.0)+(1.-msk_ref)*numpy.array(-14.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, -1.0, 4.0])+(1.-msk_arg0)*numpy.array([1.0, 6.0, 5.0])
      arg1=Data(numpy.array([-6.0, 5.0, -5.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-7.0)+(1.-msk_ref)*numpy.array(-1.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 1.0, -4.0], [0.0, 7.0, 6.0]])+(1.-msk_arg0)*numpy.array([[0.0, -3.0, -7.0], [6.0, 2.0, 0.0]])
      arg1=Data(numpy.array([[-4.0, 7.0, -7.0], [1.0, 6.0, -7.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(19.0)+(1.-msk_ref)*numpy.array(46.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 7.0, -3.0], [2.0, 3.0, 2.0], [1.0, 1.0, 7.0], [7.0, -7.0, 0.0]], [[-1.0, 1.0, -4.0], [6.0, -4.0, 4.0], [-6.0, -3.0, 0.0], [3.0, 1.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, -2.0, -7.0], [0.0, 3.0, -7.0], [-3.0, -6.0, 5.0], [5.0, 5.0, 0.0]], [[0.0, -2.0, -7.0], [0.0, 4.0, 0.0], [5.0, -1.0, -6.0], [-3.0, 0.0, -7.0]]])
      arg1=Data(numpy.array([[[3.0, -7.0, -2.0], [-7.0, 6.0, 6.0], [0.0, 7.0, -3.0], [-4.0, 5.0, -2.0]], [[4.0, 0.0, 0.0], [2.0, -6.0, 0.0], [-1.0, -3.0, 0.0], [0.0, 0.0, -6.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-66.0)+(1.-msk_ref)*numpy.array(-53.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, -1.0], [-4.0, 1.0], [0.0, -2.0]], [[2.0, 6.0], [7.0, -2.0], [1.0, -3.0]]], [[[-5.0, -1.0], [1.0, 0.0], [1.0, -3.0]], [[-6.0, -7.0], [-2.0, 3.0], [5.0, -7.0]]], [[[-4.0, 5.0], [-6.0, -4.0], [1.0, 6.0]], [[-7.0, 6.0], [-3.0, 6.0], [-1.0, -1.0]]], [[[7.0, -5.0], [-4.0, -2.0], [-3.0, 0.0]], [[3.0, 3.0], [1.0, 6.0], [1.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, 2.0], [7.0, -6.0], [-1.0, 2.0]], [[-3.0, 1.0], [6.0, 0.0], [-7.0, 5.0]]], [[[-2.0, 1.0], [3.0, -5.0], [-5.0, 5.0]], [[-3.0, 0.0], [-3.0, 3.0], [4.0, 4.0]]], [[[3.0, -6.0], [4.0, -2.0], [7.0, 0.0]], [[-6.0, 7.0], [3.0, -6.0], [6.0, 1.0]]], [[[-6.0, 0.0], [0.0, 5.0], [6.0, 3.0]], [[0.0, -5.0], [-7.0, -3.0], [-4.0, -5.0]]]])
      arg1=Data(numpy.array([[[[-7.0, 5.0], [2.0, -4.0], [2.0, -2.0]], [[6.0, 2.0], [7.0, 0.0], [5.0, 7.0]]], [[[-3.0, 2.0], [-5.0, 1.0], [-5.0, 4.0]], [[5.0, -4.0], [-7.0, -5.0], [6.0, -7.0]]], [[[7.0, 7.0], [5.0, 1.0], [6.0, 4.0]], [[3.0, 6.0], [5.0, 3.0], [5.0, -5.0]]], [[[7.0, -3.0], [-7.0, -3.0], [-1.0, 6.0]], [[6.0, 7.0], [6.0, -5.0], [-4.0, 0.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(195.0)+(1.-msk_ref)*numpy.array(117.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(4.0)+(1-msk_arg0)*(-6.0)
      arg1=Data(numpy.array([3.0, 7.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([12.0, 28.0])+(1.-msk_ref)*numpy.array([-18.0, -42.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, 0.0, -7.0])+(1.-msk_arg0)*numpy.array([-1.0, 5.0, -1.0])
      arg1=Data(numpy.array([[-5.0, -5.0, 3.0], [-5.0, 4.0, -3.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-11.0, 31.0])+(1.-msk_ref)*numpy.array([-23.0, 28.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 7.0, 7.0], [7.0, -5.0, 4.0]])+(1.-msk_arg0)*numpy.array([[1.0, -5.0, -6.0], [0.0, -1.0, 6.0]])
      arg1=Data(numpy.array([[[-4.0, -7.0, 6.0], [-7.0, 4.0, -5.0]], [[6.0, 0.0, 3.0], [-5.0, 0.0, -5.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-108.0, -16.0])+(1.-msk_ref)*numpy.array([-39.0, -42.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 1.0, 7.0], [-1.0, 6.0, -1.0], [5.0, -1.0, -7.0], [5.0, -6.0, 7.0]], [[-5.0, 3.0, 6.0], [-4.0, 3.0, -4.0], [2.0, 7.0, 1.0], [-7.0, -3.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[-6.0, 0.0, 0.0], [4.0, -3.0, -6.0], [6.0, -3.0, -4.0], [2.0, 3.0, 3.0]], [[-6.0, 1.0, -7.0], [-6.0, 0.0, 2.0], [-6.0, 1.0, 1.0], [-4.0, 0.0, 2.0]]])
      arg1=Data(numpy.array([[[[2.0, 4.0, -5.0], [6.0, -2.0, -1.0], [6.0, 6.0, -3.0], [-2.0, 7.0, -7.0]], [[6.0, 7.0, 4.0], [-6.0, -1.0, -6.0], [-5.0, -7.0, -7.0], [1.0, 0.0, -7.0]]], [[[-5.0, -6.0, -6.0], [0.0, -7.0, 7.0], [2.0, 5.0, 6.0], [3.0, 2.0, 0.0]], [[-6.0, 2.0, 5.0], [-5.0, -1.0, 0.0], [-5.0, 4.0, -2.0], [-1.0, 1.0, 6.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-153.0, -7.0])+(1.-msk_ref)*numpy.array([15.0, 75.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-1.0)+(1-msk_arg0)*(-5.0)
      arg1=Data(numpy.array([[-3.0, -2.0, 0.0, -6.0, -6.0], [-1.0, 6.0, -5.0, -1.0, 6.0], [6.0, 0.0, 3.0, -6.0, -4.0], [-4.0, 0.0, 6.0, 4.0, -5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[3.0, 2.0, -0.0, 6.0, 6.0], [1.0, -6.0, 5.0, 1.0, -6.0], [-6.0, -0.0, -3.0, 6.0, 4.0], [4.0, -0.0, -6.0, -4.0, 5.0]])+(1.-msk_ref)*numpy.array([[15.0, 10.0, -0.0, 30.0, 30.0], [5.0, -30.0, 25.0, 5.0, -30.0], [-30.0, -0.0, -15.0, 30.0, 20.0], [20.0, -0.0, -30.0, -20.0, 25.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, -2.0, -4.0])+(1.-msk_arg0)*numpy.array([-2.0, -3.0, -6.0])
      arg1=Data(numpy.array([[[-1.0, 0.0, -5.0], [6.0, 3.0, 3.0], [0.0, -5.0, 7.0], [3.0, 3.0, 3.0], [2.0, -7.0, 0.0]], [[2.0, 6.0, -2.0], [0.0, 6.0, -4.0], [-3.0, -4.0, 0.0], [-4.0, -7.0, -3.0], [-4.0, 4.0, 0.0]], [[1.0, -1.0, -2.0], [5.0, 3.0, -3.0], [-6.0, 3.0, 3.0], [2.0, 2.0, 6.0], [-5.0, -6.0, 6.0]], [[-7.0, -1.0, -1.0], [7.0, 4.0, 0.0], [6.0, -3.0, 3.0], [-4.0, 2.0, 4.0], [-4.0, -2.0, 4.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[25.0, -48.0, -18.0, -33.0, 4.0], [-14.0, 4.0, 23.0, 46.0, 12.0], [5.0, -19.0, 12.0, -38.0, 13.0], [41.0, -43.0, -36.0, 0.0, 8.0]])+(1.-msk_ref)*numpy.array([[32.0, -39.0, -27.0, -33.0, 17.0], [-10.0, 6.0, 18.0, 47.0, -4.0], [13.0, -1.0, -15.0, -46.0, -8.0], [23.0, -26.0, -21.0, -22.0, -10.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, -7.0, -7.0], [-1.0, 6.0, 5.0]])+(1.-msk_arg0)*numpy.array([[4.0, 0.0, 0.0], [-3.0, 1.0, 2.0]])
      arg1=Data(numpy.array([[[[-5.0, 2.0, 5.0], [3.0, -1.0, 3.0]], [[6.0, -4.0, -1.0], [-1.0, -3.0, -2.0]], [[4.0, 0.0, -1.0], [-7.0, -2.0, 6.0]], [[-5.0, 5.0, -5.0], [-7.0, 3.0, 6.0]], [[4.0, 7.0, -7.0], [-6.0, 1.0, 7.0]]], [[[-4.0, 5.0, -6.0], [-7.0, 2.0, 0.0]], [[1.0, -7.0, -5.0], [-1.0, 3.0, -7.0]], [[2.0, 4.0, 7.0], [-7.0, 7.0, -7.0]], [[0.0, 6.0, -6.0], [7.0, 6.0, 1.0]], [[4.0, -1.0, -2.0], [5.0, 7.0, -1.0]]], [[[-4.0, 7.0, -1.0], [-2.0, 3.0, -6.0]], [[-6.0, 5.0, -2.0], [-7.0, 3.0, -4.0]], [[-2.0, -7.0, -4.0], [0.0, -2.0, 3.0]], [[-6.0, 5.0, -2.0], [-2.0, -3.0, -2.0]], [[5.0, 0.0, 4.0], [0.0, 3.0, -2.0]]], [[[6.0, 6.0, 3.0], [-5.0, 4.0, 0.0]], [[-3.0, 5.0, 0.0], [6.0, 6.0, -7.0]], [[7.0, -1.0, 2.0], [0.0, -3.0, 3.0]], [[-7.0, 5.0, -6.0], [7.0, -6.0, 1.0]], [[7.0, 5.0, 0.0], [4.0, 5.0, -5.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-58.0, 26.0, 44.0, 40.0, 59.0], [14.0, 71.0, -57.0, 34.0, 65.0], [-64.0, -34.0, 74.0, -65.0, -5.0], [-16.0, -49.0, 11.0, -52.0, -13.0]])+(1.-msk_ref)*numpy.array([[-24.0, 20.0, 47.0, 16.0, 49.0], [7.0, -4.0, 22.0, -13.0, 6.0], [-19.0, -8.0, -4.0, -25.0, 19.0], [43.0, -38.0, 31.0, -53.0, 11.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(3.0)+(1-msk_arg0)*(2.0)
      arg1=Data(numpy.array([[[-3.0, -1.0], [-4.0, -6.0]], [[4.0, 7.0], [-7.0, 3.0]], [[1.0, -6.0], [0.0, 3.0]], [[-5.0, 5.0], [0.0, -6.0]], [[3.0, 2.0], [-7.0, 6.0]], [[5.0, -3.0], [4.0, -5.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-9.0, -3.0], [-12.0, -18.0]], [[12.0, 21.0], [-21.0, 9.0]], [[3.0, -18.0], [0.0, 9.0]], [[-15.0, 15.0], [0.0, -18.0]], [[9.0, 6.0], [-21.0, 18.0]], [[15.0, -9.0], [12.0, -15.0]]])+(1.-msk_ref)*numpy.array([[[-6.0, -2.0], [-8.0, -12.0]], [[8.0, 14.0], [-14.0, 6.0]], [[2.0, -12.0], [0.0, 6.0]], [[-10.0, 10.0], [0.0, -12.0]], [[6.0, 4.0], [-14.0, 12.0]], [[10.0, -6.0], [8.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, 4.0, 0.0])+(1.-msk_arg0)*numpy.array([-3.0, 0.0, -4.0])
      arg1=Data(numpy.array([[[[1.0, -5.0, 5.0], [2.0, -2.0, -7.0]], [[3.0, -2.0, -7.0], [5.0, 0.0, -5.0]]], [[[6.0, 4.0, -6.0], [0.0, 1.0, 5.0]], [[-7.0, 3.0, -2.0], [5.0, -4.0, 3.0]]], [[[7.0, 1.0, -5.0], [-3.0, 2.0, -5.0]], [[-5.0, -6.0, 0.0], [-4.0, -5.0, 6.0]]], [[[-6.0, -7.0, 2.0], [-1.0, 4.0, -7.0]], [[7.0, -7.0, 6.0], [-3.0, 5.0, -4.0]]], [[[1.0, 0.0, -1.0], [-6.0, 2.0, 7.0]], [[2.0, 6.0, -4.0], [-6.0, 7.0, -1.0]]], [[[0.0, 3.0, 5.0], [0.0, 5.0, -7.0]], [[-4.0, 6.0, -3.0], [-5.0, 7.0, -2.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-22.0, -12.0], [-14.0, -10.0]], [[4.0, 4.0], [26.0, -26.0]], [[-10.0, 14.0], [-14.0, -12.0]], [[-16.0, 18.0], [-42.0, 26.0]], [[-2.0, 20.0], [20.0, 40.0]], [[12.0, 20.0], [32.0, 38.0]]])+(1.-msk_ref)*numpy.array([[[-23.0, 22.0], [19.0, 5.0]], [[6.0, -20.0], [29.0, -27.0]], [[-1.0, 29.0], [15.0, -12.0]], [[10.0, 31.0], [-45.0, 25.0]], [[1.0, -10.0], [10.0, 22.0]], [[-20.0, 28.0], [24.0, 23.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_constData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-2.0)+(1-msk_arg0)*(6.0)
      arg1=Data(numpy.array([[[[-1.0, 4.0, 5.0, -5.0], [6.0, 5.0, 6.0, 2.0], [-5.0, 4.0, -4.0, -1.0]], [[-1.0, 0.0, 1.0, -2.0], [-3.0, 5.0, -2.0, 4.0], [7.0, 4.0, 5.0, -7.0]]], [[[-5.0, -6.0, -2.0, 0.0], [7.0, 2.0, 5.0, 2.0], [-2.0, 1.0, -4.0, -6.0]], [[7.0, -7.0, 2.0, 7.0], [-4.0, -7.0, -4.0, 3.0], [2.0, 6.0, 3.0, 0.0]]], [[[0.0, -4.0, -2.0, 2.0], [-1.0, 3.0, 4.0, 0.0], [-6.0, -5.0, 3.0, -1.0]], [[7.0, -3.0, -1.0, -4.0], [0.0, 1.0, -4.0, 1.0], [5.0, 7.0, -3.0, 4.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[2.0, -8.0, -10.0, 10.0], [-12.0, -10.0, -12.0, -4.0], [10.0, -8.0, 8.0, 2.0]], [[2.0, -0.0, -2.0, 4.0], [6.0, -10.0, 4.0, -8.0], [-14.0, -8.0, -10.0, 14.0]]], [[[10.0, 12.0, 4.0, -0.0], [-14.0, -4.0, -10.0, -4.0], [4.0, -2.0, 8.0, 12.0]], [[-14.0, 14.0, -4.0, -14.0], [8.0, 14.0, 8.0, -6.0], [-4.0, -12.0, -6.0, -0.0]]], [[[-0.0, 8.0, 4.0, -4.0], [2.0, -6.0, -8.0, -0.0], [12.0, 10.0, -6.0, 2.0]], [[-14.0, 6.0, 2.0, 8.0], [-0.0, -2.0, 8.0, -2.0], [-10.0, -14.0, 6.0, -8.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, 24.0, 30.0, -30.0], [36.0, 30.0, 36.0, 12.0], [-30.0, 24.0, -24.0, -6.0]], [[-6.0, 0.0, 6.0, -12.0], [-18.0, 30.0, -12.0, 24.0], [42.0, 24.0, 30.0, -42.0]]], [[[-30.0, -36.0, -12.0, 0.0], [42.0, 12.0, 30.0, 12.0], [-12.0, 6.0, -24.0, -36.0]], [[42.0, -42.0, 12.0, 42.0], [-24.0, -42.0, -24.0, 18.0], [12.0, 36.0, 18.0, 0.0]]], [[[0.0, -24.0, -12.0, 12.0], [-6.0, 18.0, 24.0, 0.0], [-36.0, -30.0, 18.0, -6.0]], [[42.0, -18.0, -6.0, -24.0], [0.0, 6.0, -24.0, 6.0], [30.0, 42.0, -18.0, 24.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(1.0)+(1-msk_arg0)*(0.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(-6.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(0.0)+(1.-msk_ref)*numpy.array(-0.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, 0.0, -5.0])+(1.-msk_arg0)*numpy.array([2.0, 0.0, 1.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, 6.0, -5.0])+(1.-msk_arg1)*numpy.array([0.0, 7.0, -6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(49.0)+(1.-msk_ref)*numpy.array(-6.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 2.0, -4.0], [0.0, 7.0, -7.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 4.0, 7.0], [-4.0, -4.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[5.0, -3.0, -3.0], [0.0, -3.0, -6.0]])+(1.-msk_arg1)*numpy.array([[4.0, 3.0, 0.0], [-3.0, -7.0, -5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(47.0)+(1.-msk_ref)*numpy.array(33.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 0.0, 7.0], [-6.0, -3.0, -7.0], [-5.0, 4.0, -1.0], [-3.0, -3.0, -5.0]], [[-2.0, 3.0, -7.0], [-6.0, 6.0, -2.0], [0.0, -4.0, 7.0], [2.0, 5.0, -7.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -2.0, -7.0], [4.0, 5.0, 3.0], [0.0, -5.0, -5.0], [1.0, 0.0, 4.0]], [[1.0, -6.0, 6.0], [2.0, 7.0, 2.0], [-7.0, 5.0, 4.0], [0.0, 6.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 0.0, 1.0], [-5.0, -2.0, -4.0], [6.0, 0.0, 4.0], [3.0, -6.0, -6.0]], [[2.0, -7.0, 6.0], [5.0, -4.0, -1.0], [6.0, 6.0, 0.0], [-4.0, -4.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-5.0, -4.0, 5.0], [1.0, 7.0, 3.0], [0.0, -7.0, 4.0], [-7.0, -3.0, -7.0]], [[0.0, -5.0, 0.0], [3.0, -7.0, -7.0], [-1.0, -1.0, 7.0], [4.0, -4.0, 6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-96.0)+(1.-msk_ref)*numpy.array(-20.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank4_offset4(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, -5.0], [-4.0, -6.0], [-5.0, -7.0]], [[0.0, 1.0], [-7.0, -4.0], [6.0, 0.0]]], [[[-3.0, -2.0], [2.0, 5.0], [2.0, 7.0]], [[-5.0, 0.0], [-3.0, 3.0], [0.0, -3.0]]], [[[1.0, -1.0], [-2.0, -6.0], [0.0, 4.0]], [[-6.0, 0.0], [5.0, -2.0], [3.0, 3.0]]], [[[0.0, -2.0], [-3.0, -1.0], [1.0, 7.0]], [[-1.0, 2.0], [5.0, -3.0], [2.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, 0.0], [5.0, 7.0], [0.0, 6.0]], [[3.0, -3.0], [3.0, -5.0], [5.0, 2.0]]], [[[0.0, 5.0], [-6.0, -6.0], [7.0, -4.0]], [[7.0, 7.0], [1.0, -1.0], [0.0, 4.0]]], [[[6.0, 4.0], [-6.0, -4.0], [3.0, -7.0]], [[7.0, -4.0], [-2.0, 0.0], [-4.0, 5.0]]], [[[0.0, 2.0], [7.0, -1.0], [-7.0, 3.0]], [[-7.0, 0.0], [-3.0, 7.0], [0.0, 1.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[5.0, 6.0], [6.0, 4.0], [-7.0, 2.0]], [[3.0, -3.0], [4.0, -1.0], [-2.0, 2.0]]], [[[-7.0, 0.0], [-7.0, 1.0], [-4.0, -5.0]], [[1.0, 0.0], [-5.0, 3.0], [-2.0, 2.0]]], [[[-2.0, 5.0], [4.0, 7.0], [-5.0, -1.0]], [[7.0, 6.0], [-5.0, -5.0], [1.0, -5.0]]], [[[0.0, -2.0], [7.0, -4.0], [7.0, 3.0]], [[-5.0, 1.0], [3.0, 0.0], [-6.0, 3.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, -5.0], [5.0, 3.0], [1.0, 5.0]], [[-1.0, 0.0], [2.0, 6.0], [4.0, 1.0]]], [[[-1.0, -2.0], [1.0, 1.0], [-3.0, 6.0]], [[3.0, -3.0], [-4.0, -2.0], [-4.0, 4.0]]], [[[-6.0, 1.0], [5.0, 0.0], [-1.0, -7.0]], [[0.0, -1.0], [-2.0, 0.0], [-5.0, -6.0]]], [[[3.0, 4.0], [1.0, 3.0], [-6.0, -6.0]], [[0.0, -3.0], [-3.0, -6.0], [0.0, 3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=4)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array(-229.0)+(1.-msk_ref)*numpy.array(1.0)
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 4.0])+(1.-msk_arg1)*numpy.array([5.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-0.0, 0.0])+(1.-msk_ref)*numpy.array([15.0, -21.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, 7.0, 4.0])+(1.-msk_arg0)*numpy.array([-3.0, 3.0, 3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, 0.0, -3.0], [-5.0, 6.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-6.0, 1.0, 3.0], [-2.0, 6.0, 3.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-12.0, 62.0])+(1.-msk_ref)*numpy.array([30.0, 33.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, 5.0, 2.0], [-1.0, 5.0, -2.0]])+(1.-msk_arg0)*numpy.array([[5.0, -6.0, -6.0], [4.0, 0.0, -7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 4.0, -3.0], [-2.0, 4.0, 2.0]], [[-2.0, 3.0, -5.0], [-2.0, 4.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[3.0, 5.0, -2.0], [0.0, -1.0, -4.0]], [[-1.0, -1.0, 4.0], [-7.0, -1.0, 6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([26.0, 15.0])+(1.-msk_ref)*numpy.array([25.0, -93.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, -6.0, -4.0], [0.0, 1.0, 7.0], [6.0, 3.0, 1.0], [0.0, -6.0, 5.0]], [[5.0, 0.0, -6.0], [4.0, 0.0, 0.0], [7.0, -5.0, 0.0], [-5.0, 6.0, 1.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, -3.0, 3.0], [-7.0, 6.0, -3.0], [-4.0, 5.0, 7.0], [-2.0, 6.0, 3.0]], [[2.0, -2.0, -6.0], [4.0, 4.0, 0.0], [0.0, 2.0, 2.0], [-5.0, 0.0, 1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-4.0, -6.0, -1.0], [-1.0, -3.0, 6.0], [-4.0, -2.0, -4.0], [-6.0, 2.0, 5.0]], [[-7.0, -6.0, -4.0], [-6.0, -1.0, 2.0], [-7.0, -1.0, -5.0], [1.0, 6.0, 5.0]]], [[[5.0, 5.0, -7.0], [3.0, 7.0, -5.0], [-4.0, -7.0, 4.0], [1.0, 4.0, -3.0]], [[2.0, -6.0, 7.0], [0.0, 6.0, 2.0], [2.0, 0.0, 7.0], [-5.0, -5.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-2.0, 0.0, 4.0], [-5.0, 3.0, -3.0], [-4.0, 5.0, -6.0], [-4.0, -2.0, 1.0]], [[1.0, 0.0, -1.0], [-7.0, 4.0, 3.0], [0.0, -5.0, 4.0], [-3.0, -4.0, -2.0]]], [[[1.0, -3.0, 2.0], [-1.0, -1.0, -7.0], [-1.0, 7.0, -6.0], [0.0, -2.0, 1.0]], [[-5.0, 0.0, 7.0], [5.0, 5.0, -1.0], [-5.0, -5.0, -6.0], [0.0, -6.0, 4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([27.0, -144.0])+(1.-msk_ref)*numpy.array([65.0, 2.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(-7.0)+(1-msk_arg0)*(-1.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 0.0, -6.0, 1.0, -6.0], [0.0, -5.0, 0.0, -3.0, -2.0], [-6.0, 1.0, 4.0, 7.0, 7.0], [0.0, 1.0, -1.0, 0.0, 4.0]])+(1.-msk_arg1)*numpy.array([[4.0, -4.0, 0.0, 0.0, -3.0], [2.0, 7.0, -6.0, 0.0, -6.0], [1.0, 3.0, 7.0, -7.0, -3.0], [4.0, 7.0, 3.0, -5.0, -4.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[14.0, -0.0, 42.0, -7.0, 42.0], [-0.0, 35.0, -0.0, 21.0, 14.0], [42.0, -7.0, -28.0, -49.0, -49.0], [-0.0, -7.0, 7.0, -0.0, -28.0]])+(1.-msk_ref)*numpy.array([[-4.0, 4.0, -0.0, -0.0, 3.0], [-2.0, -7.0, 6.0, -0.0, 6.0], [-1.0, -3.0, -7.0, 7.0, 3.0], [-4.0, -7.0, -3.0, 5.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([7.0, -4.0, -6.0])+(1.-msk_arg0)*numpy.array([-5.0, 1.0, -1.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[6.0, -4.0, 7.0], [-2.0, 1.0, -5.0], [4.0, -4.0, 1.0], [-1.0, -4.0, -1.0], [4.0, 6.0, -5.0]], [[-2.0, -5.0, 1.0], [6.0, -6.0, 7.0], [-2.0, 3.0, 7.0], [7.0, 6.0, -2.0], [-5.0, 2.0, -4.0]], [[6.0, -1.0, 1.0], [-3.0, 7.0, -1.0], [-1.0, 0.0, 2.0], [5.0, -7.0, -2.0], [-1.0, -5.0, 0.0]], [[0.0, 1.0, -3.0], [-6.0, 0.0, -6.0], [4.0, -5.0, 6.0], [4.0, -6.0, -7.0], [-3.0, -3.0, 7.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 0.0, 0.0], [5.0, -3.0, -3.0], [-3.0, -3.0, 6.0], [1.0, 7.0, -7.0], [-1.0, -5.0, 7.0]], [[2.0, 0.0, 0.0], [0.0, -6.0, 0.0], [-5.0, -3.0, -5.0], [6.0, -3.0, 1.0], [2.0, 4.0, -6.0]], [[-4.0, 7.0, -4.0], [5.0, 0.0, 0.0], [6.0, 2.0, -4.0], [5.0, -6.0, 5.0], [1.0, 0.0, 5.0]], [[6.0, 4.0, 7.0], [3.0, -2.0, 1.0], [0.0, -3.0, -7.0], [-6.0, 5.0, -4.0], [-3.0, 0.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[16.0, 12.0, 38.0, 15.0, 34.0], [0.0, 24.0, -68.0, 37.0, -19.0], [40.0, -43.0, -19.0, 75.0, 13.0], [14.0, -6.0, 12.0, 94.0, -51.0]])+(1.-msk_ref)*numpy.array([[15.0, -25.0, 6.0, 9.0, -7.0], [-10.0, -6.0, 27.0, -34.0, 0.0], [31.0, -25.0, -24.0, -36.0, -10.0], [-33.0, -18.0, 4.0, 39.0, 21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, -4.0, 5.0], [-7.0, 0.0, -6.0]])+(1.-msk_arg0)*numpy.array([[-6.0, 5.0, 1.0], [1.0, -1.0, 2.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[3.0, 5.0, -6.0], [0.0, -1.0, 0.0]], [[3.0, -2.0, 7.0], [-7.0, -3.0, 0.0]], [[-3.0, 4.0, -5.0], [6.0, 0.0, -2.0]], [[2.0, 0.0, -6.0], [-4.0, -1.0, -5.0]], [[-7.0, 1.0, -5.0], [-1.0, 0.0, 6.0]]], [[[-3.0, 0.0, 1.0], [0.0, 6.0, -3.0]], [[-7.0, -6.0, 7.0], [4.0, 4.0, -4.0]], [[4.0, 4.0, 4.0], [0.0, -6.0, 0.0]], [[-5.0, 0.0, -2.0], [-1.0, -1.0, 7.0]], [[-3.0, -2.0, 2.0], [0.0, -4.0, 2.0]]], [[[-4.0, 3.0, -4.0], [3.0, -5.0, -1.0]], [[3.0, 4.0, 5.0], [-1.0, 2.0, 7.0]], [[6.0, 6.0, -1.0], [0.0, -2.0, -5.0]], [[5.0, 0.0, 2.0], [-7.0, 2.0, 3.0]], [[-4.0, 4.0, 2.0], [-1.0, -6.0, 2.0]]], [[[3.0, -4.0, 1.0], [6.0, 7.0, 5.0]], [[-5.0, 6.0, 1.0], [1.0, 4.0, 4.0]], [[-3.0, 2.0, -5.0], [0.0, -7.0, -7.0]], [[-2.0, -6.0, -1.0], [-2.0, -2.0, 6.0]], [[1.0, 1.0, -7.0], [1.0, -3.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[-1.0, -6.0, -1.0], [-7.0, -1.0, -5.0]], [[7.0, 0.0, -2.0], [-2.0, -6.0, 6.0]], [[-5.0, 5.0, -4.0], [3.0, 2.0, 3.0]], [[1.0, -4.0, 0.0], [-1.0, -2.0, -6.0]], [[-2.0, -1.0, -3.0], [4.0, -4.0, -6.0]]], [[[6.0, 7.0, -7.0], [-6.0, 0.0, -7.0]], [[-5.0, -3.0, -1.0], [1.0, 7.0, -3.0]], [[-6.0, 4.0, 0.0], [6.0, 7.0, 1.0]], [[6.0, -2.0, 0.0], [7.0, 4.0, -7.0]], [[-5.0, -3.0, -3.0], [1.0, 4.0, 0.0]]], [[[-4.0, 1.0, 7.0], [-4.0, 4.0, -5.0]], [[-2.0, 0.0, -5.0], [-4.0, -1.0, -1.0]], [[-3.0, -3.0, 6.0], [-5.0, -7.0, -3.0]], [[-5.0, 0.0, -3.0], [0.0, -5.0, 6.0]], [[6.0, 6.0, -1.0], [0.0, 3.0, 0.0]]], [[[7.0, 1.0, 0.0], [-4.0, 0.0, 1.0]], [[3.0, -3.0, -7.0], [-4.0, 1.0, -6.0]], [[6.0, 6.0, -5.0], [4.0, 6.0, -5.0]], [[2.0, -3.0, 1.0], [-2.0, 2.0, 0.0]], [[0.0, 2.0, -7.0], [7.0, 1.0, 6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-56.0, 86.0, -65.0, 24.0, -44.0], [29.0, 69.0, -4.0, -35.0, 12.0], [-39.0, -32.0, -11.0, 31.0, -3.0], [-57.0, -40.0, 15.0, 1.0, -90.0]])+(1.-msk_ref)*numpy.array([[-41.0, -28.0, 58.0, -37.0, 0.0], [-28.0, 2.0, 57.0, -57.0, 9.0], [18.0, 2.0, 5.0, 44.0, -10.0], [-39.0, -57.0, -23.0, -30.0, 21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(0.0)+(1-msk_arg0)*(-5.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[0.0, 7.0], [0.0, -2.0]], [[-5.0, 2.0], [-7.0, 2.0]], [[3.0, -7.0], [0.0, 1.0]], [[5.0, 7.0], [7.0, 2.0]], [[0.0, -1.0], [3.0, 4.0]], [[-7.0, 0.0], [5.0, -6.0]]])+(1.-msk_arg1)*numpy.array([[[-3.0, 7.0], [-7.0, 1.0]], [[2.0, 0.0], [5.0, 3.0]], [[6.0, -5.0], [-4.0, -5.0]], [[0.0, 7.0], [-7.0, -4.0]], [[7.0, -6.0], [-7.0, 1.0]], [[0.0, -7.0], [7.0, -6.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 0.0], [0.0, -0.0]], [[-0.0, 0.0], [-0.0, 0.0]], [[0.0, -0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, -0.0], [0.0, 0.0]], [[-0.0, 0.0], [0.0, -0.0]]])+(1.-msk_ref)*numpy.array([[[15.0, -35.0], [35.0, -5.0]], [[-10.0, -0.0], [-25.0, -15.0]], [[-30.0, 25.0], [20.0, 25.0]], [[-0.0, -35.0], [35.0, 20.0]], [[-35.0, 30.0], [35.0, -5.0]], [[-0.0, 35.0], [-35.0, 30.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, 3.0, -3.0])+(1.-msk_arg0)*numpy.array([6.0, 1.0, 7.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 0.0, -2.0], [4.0, -2.0, 1.0]], [[-3.0, 0.0, 7.0], [4.0, 0.0, -1.0]]], [[[5.0, 6.0, 7.0], [1.0, -4.0, -7.0]], [[-5.0, 4.0, 4.0], [0.0, 1.0, -1.0]]], [[[-7.0, 2.0, -5.0], [-3.0, -6.0, -4.0]], [[-7.0, 4.0, 6.0], [5.0, -2.0, -2.0]]], [[[6.0, 6.0, -7.0], [4.0, 1.0, -7.0]], [[-7.0, 2.0, 2.0], [1.0, -2.0, 7.0]]], [[[-6.0, 2.0, -6.0], [0.0, -3.0, -5.0]], [[-2.0, 0.0, -6.0], [-6.0, -6.0, 0.0]]], [[[7.0, 0.0, 0.0], [0.0, 0.0, 7.0]], [[-1.0, -2.0, -1.0], [-2.0, 0.0, -4.0]]]])+(1.-msk_arg1)*numpy.array([[[[1.0, 6.0, -2.0], [-1.0, 5.0, 7.0]], [[-4.0, -5.0, 7.0], [-1.0, 1.0, 1.0]]], [[[-3.0, -2.0, 2.0], [-3.0, -3.0, -2.0]], [[-3.0, -1.0, 1.0], [6.0, 2.0, 1.0]]], [[[0.0, -7.0, -7.0], [0.0, 6.0, 3.0]], [[5.0, -2.0, 7.0], [5.0, 0.0, 7.0]]], [[[4.0, 3.0, 1.0], [-5.0, 0.0, 5.0]], [[5.0, -1.0, 2.0], [-6.0, -3.0, -4.0]]], [[[3.0, -3.0, -5.0], [0.0, -4.0, 2.0]], [[-6.0, -1.0, -5.0], [-2.0, 7.0, -4.0]]], [[[-2.0, 3.0, 1.0], [-2.0, 6.0, 5.0]], [[3.0, 0.0, 3.0], [5.0, 3.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[6.0, -25.0], [-9.0, -13.0]], [[-23.0, 5.0], [20.0, 6.0]], [[49.0, 6.0], [22.0, -20.0]], [[15.0, 8.0], [28.0, -31.0]], [[48.0, 6.0], [26.0, 6.0]], [[-28.0, -21.0], [1.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-2.0, 48.0], [20.0, 2.0]], [[-6.0, -35.0], [-12.0, 45.0]], [[-56.0, 27.0], [77.0, 79.0]], [[34.0, 5.0], [43.0, -67.0]], [[-20.0, 10.0], [-72.0, -33.0]], [[-2.0, 29.0], [39.0, -2.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank0_expandedData_rank4_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*(1.0)+(1-msk_arg0)*(-3.0)
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, 0.0, 2.0, 5.0], [-1.0, -5.0, 3.0, -3.0], [-4.0, 7.0, 6.0, -3.0]], [[-2.0, 3.0, 3.0, 2.0], [6.0, -4.0, 6.0, -5.0], [7.0, 3.0, -6.0, 3.0]]], [[[-4.0, 5.0, 2.0, 0.0], [6.0, -4.0, 0.0, 6.0], [-5.0, -2.0, 7.0, -5.0]], [[1.0, 0.0, 5.0, -4.0], [5.0, -3.0, 7.0, -7.0], [0.0, 6.0, 0.0, 0.0]]], [[[-1.0, -6.0, 0.0, -2.0], [3.0, -7.0, 6.0, -6.0], [0.0, -7.0, -7.0, -2.0]], [[-7.0, -7.0, 6.0, 6.0], [0.0, -3.0, 1.0, 0.0], [0.0, 0.0, -2.0, 7.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, 0.0, 6.0, 7.0], [0.0, -6.0, 1.0, 3.0], [7.0, -6.0, -5.0, 5.0]], [[-1.0, 0.0, 4.0, -6.0], [2.0, -6.0, -5.0, 0.0], [5.0, 2.0, 1.0, 3.0]]], [[[1.0, -5.0, 7.0, -2.0], [4.0, -6.0, -5.0, 7.0], [-3.0, -7.0, 5.0, 1.0]], [[-3.0, 0.0, 0.0, -4.0], [1.0, 0.0, -5.0, 2.0], [6.0, -3.0, 7.0, -3.0]]], [[[0.0, 2.0, -2.0, 2.0], [-5.0, 3.0, 2.0, -2.0], [3.0, 3.0, -4.0, -2.0]], [[-3.0, -4.0, 5.0, -4.0], [0.0, -6.0, -2.0, -7.0], [-1.0, 1.0, 1.0, 2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0, 2.0, 5.0], [-1.0, -5.0, 3.0, -3.0], [-4.0, 7.0, 6.0, -3.0]], [[-2.0, 3.0, 3.0, 2.0], [6.0, -4.0, 6.0, -5.0], [7.0, 3.0, -6.0, 3.0]]], [[[-4.0, 5.0, 2.0, 0.0], [6.0, -4.0, 0.0, 6.0], [-5.0, -2.0, 7.0, -5.0]], [[1.0, 0.0, 5.0, -4.0], [5.0, -3.0, 7.0, -7.0], [0.0, 6.0, 0.0, 0.0]]], [[[-1.0, -6.0, 0.0, -2.0], [3.0, -7.0, 6.0, -6.0], [0.0, -7.0, -7.0, -2.0]], [[-7.0, -7.0, 6.0, 6.0], [0.0, -3.0, 1.0, 0.0], [0.0, 0.0, -2.0, 7.0]]]])+(1.-msk_ref)*numpy.array([[[[-21.0, -0.0, -18.0, -21.0], [-0.0, 18.0, -3.0, -9.0], [-21.0, 18.0, 15.0, -15.0]], [[3.0, -0.0, -12.0, 18.0], [-6.0, 18.0, 15.0, -0.0], [-15.0, -6.0, -3.0, -9.0]]], [[[-3.0, 15.0, -21.0, 6.0], [-12.0, 18.0, 15.0, -21.0], [9.0, 21.0, -15.0, -3.0]], [[9.0, -0.0, -0.0, 12.0], [-3.0, -0.0, 15.0, -6.0], [-18.0, 9.0, -21.0, 9.0]]], [[[-0.0, -6.0, 6.0, -6.0], [15.0, -9.0, -6.0, 6.0], [-9.0, -9.0, 12.0, 6.0]], [[9.0, 12.0, -15.0, 12.0], [-0.0, 18.0, 6.0, 21.0], [3.0, -3.0, -3.0, -6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([5.0, 2.0])+(1.-msk_arg0)*numpy.array([1.0, 5.0])
      arg1=3.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([15.0, 6.0])+(1.-msk_ref)*numpy.array([3.0, 15.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, -7.0])+(1.-msk_arg0)*numpy.array([6.0, -7.0])
      arg1=numpy.array(2.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-10.0, -14.0])+(1.-msk_ref)*numpy.array([12.0, -14.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-5.0, -6.0, -6.0], [1.0, -5.0, 0.0]])+(1.-msk_arg0)*numpy.array([[4.0, 6.0, -6.0], [-6.0, -5.0, 6.0]])
      arg1=numpy.array([-2.0, -1.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([10.0, 3.0])+(1.-msk_ref)*numpy.array([-20.0, 23.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, 3.0, 1.0], [3.0, 2.0, 4.0]], [[3.0, -1.0, -5.0], [-7.0, 0.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 5.0, -1.0], [-5.0, -6.0, -2.0]], [[-3.0, -3.0, 4.0], [5.0, 0.0, 3.0]]])
      arg1=numpy.array([[1.0, -7.0, -7.0], [-5.0, 0.0, 1.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-35.0, 86.0])+(1.-msk_ref)*numpy.array([-5.0, -32.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, 0.0, 6.0], [7.0, 4.0, 3.0], [-2.0, 4.0, -4.0], [-3.0, -6.0, 7.0]], [[-2.0, 1.0, -4.0], [3.0, -5.0, -1.0], [-3.0, -3.0, 6.0], [0.0, 3.0, 4.0]]], [[[-3.0, -4.0, 2.0], [-7.0, -5.0, -6.0], [6.0, 2.0, 4.0], [-7.0, 7.0, -1.0]], [[5.0, -6.0, 0.0], [0.0, -5.0, -4.0], [2.0, 0.0, 1.0], [-3.0, 7.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 3.0, 4.0], [5.0, -3.0, 3.0], [-7.0, 4.0, -5.0], [-7.0, -6.0, 2.0]], [[-5.0, 6.0, 5.0], [4.0, 3.0, -3.0], [0.0, -3.0, -3.0], [7.0, -3.0, 1.0]]], [[[3.0, 0.0, 7.0], [-7.0, 4.0, 6.0], [0.0, 4.0, 2.0], [-5.0, -3.0, 0.0]], [[-6.0, -5.0, -6.0], [2.0, 6.0, 3.0], [0.0, -6.0, -3.0], [2.0, 2.0, 0.0]]]])
      arg1=numpy.array([[[-2.0, 6.0, -2.0], [3.0, -7.0, 0.0], [-7.0, 4.0, 4.0], [0.0, -2.0, -5.0]], [[-4.0, 0.0, 3.0], [0.0, -7.0, 1.0], [7.0, -6.0, -5.0], [-1.0, 4.0, -1.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-17.0, 10.0])+(1.-msk_ref)*numpy.array([129.0, -15.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, -2.0])+(1.-msk_arg0)*numpy.array([7.0, 4.0])
      arg1=numpy.array([-1.0, 1.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, -7.0], [2.0, -2.0]])+(1.-msk_ref)*numpy.array([[-7.0, 7.0], [-4.0, 4.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, -5.0, -2.0], [-1.0, 0.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-7.0, 0.0, -4.0], [-7.0, -4.0, 6.0]])
      arg1=numpy.array([[-3.0, -3.0, 1.0], [6.0, 2.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, 20.0], [3.0, -6.0]])+(1.-msk_ref)*numpy.array([[17.0, -66.0], [39.0, -14.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 6.0, 0.0], [1.0, -4.0, -2.0]], [[-4.0, 6.0, -6.0], [5.0, 0.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 5.0, 3.0], [0.0, -5.0, 5.0]], [[1.0, -5.0, 4.0], [-4.0, 7.0, -5.0]]])
      arg1=numpy.array([[[4.0, -5.0, -3.0], [-5.0, -3.0, 6.0]], [[3.0, 7.0, -1.0], [0.0, -3.0, 0.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-55.0, 39.0], [-23.0, 36.0]])+(1.-msk_ref)*numpy.array([[-17.0, 26.0], [-14.0, -57.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -3.0, -2.0], [-3.0, 2.0, -7.0], [0.0, 2.0, 6.0], [-2.0, 2.0, -1.0]], [[-7.0, -4.0, 3.0], [0.0, -3.0, -1.0], [-1.0, 5.0, 0.0], [-4.0, 6.0, -1.0]]], [[[2.0, 0.0, 6.0], [5.0, 3.0, -1.0], [-6.0, 5.0, -1.0], [3.0, -2.0, -5.0]], [[-2.0, 0.0, -5.0], [-4.0, 6.0, 6.0], [-5.0, 1.0, -5.0], [0.0, 6.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[4.0, 7.0, 0.0], [0.0, -4.0, 5.0], [-7.0, -2.0, 0.0], [2.0, 4.0, 2.0]], [[5.0, -5.0, -2.0], [1.0, -7.0, -7.0], [-1.0, -7.0, 7.0], [-3.0, 0.0, -6.0]]], [[[-7.0, -4.0, 1.0], [0.0, -6.0, -6.0], [0.0, 2.0, 6.0], [5.0, -2.0, -2.0]], [[5.0, -4.0, -1.0], [-3.0, -3.0, 0.0], [0.0, 1.0, 2.0], [1.0, 5.0, -4.0]]]])
      arg1=numpy.array([[[[-3.0, 0.0, 1.0], [-7.0, 0.0, 0.0], [5.0, 3.0, -4.0], [7.0, 2.0, 0.0]], [[3.0, -2.0, 5.0], [4.0, 7.0, 5.0], [0.0, 5.0, -5.0], [6.0, -1.0, -7.0]]], [[[7.0, -5.0, 4.0], [0.0, 6.0, 5.0], [3.0, 0.0, -7.0], [0.0, 0.0, 7.0]], [[-6.0, 5.0, 1.0], [-2.0, -4.0, -1.0], [-2.0, -7.0, -4.0], [-5.0, 0.0, -2.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-31.0, -38.0], [-29.0, -1.0]])+(1.-msk_ref)*numpy.array([[-142.0, 13.0], [44.0, -192.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-4.0, -6.0])+(1.-msk_arg0)*numpy.array([5.0, 0.0])
      arg1=numpy.array([[4.0, -5.0, 5.0, -3.0, 0.0], [0.0, -5.0, 3.0, -3.0, -4.0], [4.0, -6.0, -6.0, 7.0, 1.0], [-6.0, 1.0, 7.0, 4.0, -6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-16.0, 20.0, -20.0, 12.0, 0.0], [0.0, 20.0, -12.0, 12.0, 16.0], [-16.0, 24.0, 24.0, -28.0, -4.0], [24.0, -4.0, -28.0, -16.0, 24.0]], [[-24.0, 30.0, -30.0, 18.0, 0.0], [0.0, 30.0, -18.0, 18.0, 24.0], [-24.0, 36.0, 36.0, -42.0, -6.0], [36.0, -6.0, -42.0, -24.0, 36.0]]])+(1.-msk_ref)*numpy.array([[[20.0, -25.0, 25.0, -15.0, 0.0], [0.0, -25.0, 15.0, -15.0, -20.0], [20.0, -30.0, -30.0, 35.0, 5.0], [-30.0, 5.0, 35.0, 20.0, -30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 5.0, 5.0], [1.0, 6.0, -1.0]])+(1.-msk_arg0)*numpy.array([[2.0, 3.0, 5.0], [-4.0, -6.0, -2.0]])
      arg1=numpy.array([[[1.0, 7.0, -7.0], [7.0, -4.0, 1.0], [5.0, -5.0, 2.0], [5.0, -5.0, -7.0], [5.0, -4.0, -4.0]], [[-6.0, -1.0, -4.0], [4.0, -5.0, 7.0], [0.0, 4.0, 2.0], [2.0, -2.0, -2.0], [6.0, 4.0, 3.0]], [[-7.0, -7.0, 2.0], [-4.0, -5.0, 0.0], [-7.0, 0.0, -6.0], [-6.0, 0.0, 7.0], [3.0, -6.0, 7.0]], [[-1.0, 1.0, 7.0], [-2.0, -2.0, 1.0], [-4.0, -3.0, 3.0], [3.0, 3.0, 1.0], [3.0, -6.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, -15.0, -15.0, -60.0, -40.0], [-25.0, 10.0, 30.0, -20.0, 35.0], [-25.0, -25.0, -30.0, 35.0, 5.0], [40.0, -5.0, 0.0, 20.0, -65.0]], [[50.0, -18.0, -27.0, -18.0, -15.0], [-8.0, -33.0, 22.0, -8.0, 27.0], [-51.0, -34.0, -1.0, -13.0, -40.0], [-2.0, -15.0, -25.0, 20.0, -26.0]]])+(1.-msk_ref)*numpy.array([[[-12.0, 7.0, 5.0, -40.0, -22.0], [-35.0, 28.0, 22.0, -12.0, 39.0], [-25.0, -23.0, -44.0, 23.0, 23.0], [36.0, -5.0, -2.0, 20.0, -47.0]], [[-32.0, -6.0, 6.0, 24.0, 12.0], [38.0, 0.0, -28.0, 8.0, -54.0], [66.0, 46.0, 40.0, 10.0, 10.0], [-16.0, 18.0, 28.0, -32.0, 38.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 7.0, 0.0], [2.0, -6.0, 7.0]], [[0.0, 2.0, 1.0], [3.0, 5.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 5.0, -6.0], [2.0, 4.0, 0.0]], [[-3.0, 0.0, -2.0], [7.0, 6.0, 2.0]]])
      arg1=numpy.array([[[[-6.0, 0.0, -5.0], [2.0, -2.0, -2.0]], [[-2.0, 1.0, 5.0], [-1.0, 0.0, 4.0]], [[6.0, 7.0, -3.0], [-1.0, -6.0, -4.0]], [[-7.0, 2.0, 1.0], [-6.0, -7.0, 2.0]], [[-2.0, 7.0, -3.0], [-4.0, 0.0, -6.0]]], [[[3.0, -3.0, -4.0], [-3.0, 2.0, -6.0]], [[0.0, 7.0, -1.0], [-6.0, 0.0, 2.0]], [[-5.0, -5.0, 1.0], [3.0, -6.0, 6.0]], [[-2.0, 1.0, -5.0], [-6.0, -6.0, 1.0]], [[-5.0, 1.0, 0.0], [-5.0, 3.0, 5.0]]], [[[4.0, 2.0, -3.0], [1.0, 3.0, 6.0]], [[0.0, 7.0, 4.0], [0.0, 5.0, -3.0]], [[-6.0, -5.0, 0.0], [4.0, 0.0, -2.0]], [[7.0, 1.0, 5.0], [6.0, 0.0, -6.0]], [[-6.0, 0.0, 6.0], [-2.0, 6.0, 1.0]]], [[[-7.0, 0.0, -2.0], [-5.0, -1.0, -4.0]], [[4.0, -5.0, -3.0], [5.0, -5.0, 0.0]], [[0.0, 4.0, 6.0], [2.0, 0.0, -4.0]], [[0.0, 7.0, -6.0], [6.0, -7.0, -2.0]], [[-6.0, -1.0, 7.0], [4.0, 4.0, -1.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[44.0, 47.0, 13.0, 107.0, 13.0], [-102.0, 51.0, 84.0, 52.0, 49.0], [12.0, -2.0, 1.0, -72.0, 9.0], [17.0, -23.0, 4.0, 89.0, 12.0]], [[-9.0, 4.0, -22.0, -48.0, -1.0], [-9.0, -5.0, -30.0, -51.0, 2.0], [19.0, 43.0, 2.0, 25.0, 30.0], [-22.0, -23.0, 20.0, -9.0, 37.0]]])+(1.-msk_ref)*numpy.array([[[-16.0, -41.0, 69.0, -85.0, 31.0], [32.0, 29.0, -84.0, -15.0, -28.0], [70.0, 31.0, -59.0, 36.0, -58.0], [-51.0, 11.0, -12.0, 55.0, -65.0]], [[26.0, -3.0, -63.0, -61.0, -28.0], [-22.0, -36.0, 10.0, -60.0, 8.0], [31.0, 16.0, 42.0, -1.0, 30.0], [-24.0, -1.0, -6.0, 8.0, 54.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_array_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-5.0, 5.0])+(1.-msk_arg0)*numpy.array([-5.0, 7.0])
      arg1=numpy.array([[[-2.0, 0.0], [7.0, 7.0]], [[6.0, 4.0], [0.0, -5.0]], [[0.0, -6.0], [-5.0, -3.0]], [[-1.0, 1.0], [4.0, 4.0]], [[5.0, -1.0], [5.0, -6.0]], [[-6.0, -7.0], [-5.0, -2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 0.0], [-35.0, -35.0]], [[-30.0, -20.0], [0.0, 25.0]], [[0.0, 30.0], [25.0, 15.0]], [[5.0, -5.0], [-20.0, -20.0]], [[-25.0, 5.0], [-25.0, 30.0]], [[30.0, 35.0], [25.0, 10.0]]], [[[-10.0, 0.0], [35.0, 35.0]], [[30.0, 20.0], [0.0, -25.0]], [[0.0, -30.0], [-25.0, -15.0]], [[-5.0, 5.0], [20.0, 20.0]], [[25.0, -5.0], [25.0, -30.0]], [[-30.0, -35.0], [-25.0, -10.0]]]])+(1.-msk_ref)*numpy.array([[[[10.0, 0.0], [-35.0, -35.0]], [[-30.0, -20.0], [0.0, 25.0]], [[0.0, 30.0], [25.0, 15.0]], [[5.0, -5.0], [-20.0, -20.0]], [[-25.0, 5.0], [-25.0, 30.0]], [[30.0, 35.0], [25.0, 10.0]]], [[[-14.0, 0.0], [49.0, 49.0]], [[42.0, 28.0], [0.0, -35.0]], [[0.0, -42.0], [-35.0, -21.0]], [[-7.0, 7.0], [28.0, 28.0]], [[35.0, -7.0], [35.0, -42.0]], [[-42.0, -49.0], [-35.0, -14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[2.0, 6.0, -7.0], [0.0, 4.0, 3.0]])+(1.-msk_arg0)*numpy.array([[-3.0, -2.0, 7.0], [7.0, 3.0, -1.0]])
      arg1=numpy.array([[[[-5.0, -2.0, -6.0], [3.0, 6.0, -4.0]], [[0.0, -2.0, 5.0], [-5.0, -6.0, -5.0]]], [[[5.0, 1.0, -3.0], [7.0, 2.0, 2.0]], [[-2.0, 4.0, 0.0], [-6.0, 2.0, 2.0]]], [[[5.0, -2.0, 0.0], [-1.0, 1.0, -7.0]], [[-3.0, -7.0, 6.0], [-4.0, -7.0, 7.0]]], [[[0.0, -4.0, 4.0], [-3.0, 1.0, -4.0]], [[4.0, -1.0, -4.0], [-6.0, 0.0, 1.0]]], [[[-4.0, 1.0, 7.0], [6.0, -5.0, -3.0]], [[-7.0, 0.0, 0.0], [0.0, -3.0, -1.0]]], [[[3.0, -4.0, 2.0], [3.0, -5.0, 4.0]], [[3.0, 0.0, 2.0], [-1.0, 7.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[20.0, 70.0], [-47.0, -11.0]], [[37.0, 12.0], [20.0, -14.0]], [[-2.0, 53.0], [-90.0, -99.0]], [[-52.0, 28.0], [30.0, -19.0]], [[-51.0, 3.0], [-14.0, -11.0]], [[-32.0, -52.0], [-8.0, 68.0]]], [[[-26.0, 12.0], [7.0, -39.0]], [[-5.0, 14.0], [16.0, 14.0]], [[-8.0, -17.0], [-10.0, -7.0]], [[-4.0, -8.0], [-16.0, 3.0]], [[25.0, -29.0], [0.0, -15.0]], [[-10.0, -8.0], [6.0, 16.0]]]])+(1.-msk_ref)*numpy.array([[[[-23.0, -49.0], [39.0, -8.0]], [[-38.0, -11.0], [-2.0, 28.0]], [[-11.0, -48.0], [65.0, 75.0]], [[36.0, -21.0], [-38.0, 25.0]], [[59.0, -29.0], [21.0, -1.0]], [[13.0, 29.0], [5.0, -39.0]]], [[[-35.0, 43.0], [-11.0, -48.0]], [[41.0, 53.0], [-2.0, -38.0]], [[29.0, 3.0], [-48.0, -56.0]], [[-16.0, -14.0], [29.0, -43.0]], [[-32.0, 30.0], [-49.0, -8.0]], [[7.0, 2.0], [19.0, 18.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, 6.0])+(1.-msk_arg0)*numpy.array([7.0, 5.0])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-6.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-18.0, -36.0])+(1.-msk_ref)*numpy.array([-42.0, -30.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, -7.0, 5.0], [-7.0, 1.0, 4.0]])+(1.-msk_arg0)*numpy.array([[6.0, -6.0, 4.0], [6.0, -1.0, -4.0]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([2.0, 1.0, -6.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-51.0, -37.0])+(1.-msk_ref)*numpy.array([-18.0, 35.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, 1.0, 4.0], [4.0, 7.0, -6.0]], [[6.0, 1.0, 5.0], [5.0, 0.0, 2.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, -1.0, 5.0], [-4.0, 4.0, 1.0]], [[3.0, -2.0, 7.0], [-5.0, 3.0, 6.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[6.0, -5.0, -1.0], [1.0, 4.0, 6.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([11.0, 43.0])+(1.-msk_ref)*numpy.array([6.0, 64.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, -5.0, 3.0], [3.0, 7.0, -2.0], [0.0, 5.0, -6.0], [-6.0, -5.0, 6.0]], [[-2.0, 7.0, 5.0], [4.0, 2.0, -4.0], [-6.0, 1.0, -6.0], [2.0, -4.0, -1.0]]], [[[1.0, 2.0, 5.0], [1.0, 3.0, 4.0], [2.0, 0.0, 4.0], [-4.0, 2.0, 5.0]], [[5.0, -7.0, -1.0], [-1.0, -6.0, 7.0], [4.0, 2.0, -1.0], [-3.0, -7.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, 3.0, 0.0], [2.0, 5.0, -5.0], [0.0, -4.0, 3.0], [-2.0, 0.0, -7.0]], [[0.0, -6.0, -4.0], [0.0, 1.0, 4.0], [-3.0, -6.0, -4.0], [7.0, -2.0, 5.0]]], [[[-2.0, -2.0, -4.0], [6.0, 4.0, -4.0], [-2.0, 7.0, -5.0], [5.0, -5.0, 3.0]], [[4.0, 5.0, 1.0], [-3.0, 6.0, -7.0], [-1.0, 2.0, 6.0], [2.0, 2.0, -6.0]]]])
      arg1=Symbol(shape=(2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[0.0, -3.0, 1.0], [-4.0, 0.0, 2.0], [-4.0, 6.0, -4.0], [-2.0, -7.0, -1.0]], [[-3.0, 3.0, -1.0], [-1.0, 6.0, 4.0], [5.0, 4.0, 3.0], [-1.0, 0.0, 1.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([64.0, -43.0])+(1.-msk_ref)*numpy.array([-97.0, 88.0])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-3.0, 2.0])+(1.-msk_arg0)*numpy.array([-7.0, 7.0])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-7.0, 1.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[21.0, -3.0], [-14.0, 2.0]])+(1.-msk_ref)*numpy.array([[49.0, -7.0], [-49.0, 7.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, 5.0, 0.0], [2.0, -1.0, 2.0]])+(1.-msk_arg0)*numpy.array([[6.0, 0.0, 2.0], [-2.0, 0.0, -6.0]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[-4.0, -7.0, 0.0], [-2.0, -4.0, 7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-7.0, -6.0], [-1.0, 14.0]])+(1.-msk_ref)*numpy.array([[-24.0, 2.0], [8.0, -38.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[5.0, -6.0, 1.0], [7.0, 1.0, -5.0]], [[-2.0, 7.0, 3.0], [-5.0, 1.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[5.0, -2.0, -7.0], [6.0, 0.0, -1.0]], [[-6.0, 2.0, 7.0], [-4.0, 1.0, 1.0]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-1.0, -1.0, 2.0], [4.0, 0.0, 5.0]], [[0.0, -5.0, -3.0], [2.0, -6.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, 35.0], [-39.0, -60.0]])+(1.-msk_ref)*numpy.array([[2.0, 43.0], [7.0, -45.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, -7.0, -2.0], [-6.0, 4.0, 4.0], [-1.0, -3.0, 0.0], [-2.0, 2.0, -1.0]], [[2.0, -4.0, 1.0], [3.0, 7.0, 0.0], [6.0, 7.0, 0.0], [-4.0, 6.0, 5.0]]], [[[6.0, 0.0, 3.0], [7.0, 5.0, 7.0], [-4.0, 0.0, 0.0], [4.0, -4.0, -5.0]], [[-7.0, -4.0, -2.0], [6.0, -5.0, 0.0], [-5.0, 3.0, 2.0], [-7.0, -6.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, -6.0, 2.0], [-5.0, -3.0, -1.0], [2.0, -7.0, -2.0], [-2.0, -2.0, 4.0]], [[-4.0, 7.0, -4.0], [6.0, 5.0, 0.0], [7.0, 6.0, -6.0], [7.0, 5.0, 3.0]]], [[[-3.0, 4.0, -1.0], [-4.0, 3.0, -4.0], [7.0, 1.0, -1.0], [-3.0, 2.0, -7.0]], [[-2.0, -1.0, 2.0], [6.0, -5.0, 3.0], [0.0, -6.0, -1.0], [-4.0, 0.0, 3.0]]]])
      arg1=Symbol(shape=(2, 2, 4, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      s1=numpy.array([[[[3.0, -2.0, 0.0], [1.0, 2.0, 0.0], [-2.0, -2.0, -4.0], [-1.0, -4.0, 6.0]], [[-2.0, 2.0, 2.0], [2.0, 0.0, -3.0], [3.0, -2.0, -4.0], [-4.0, 4.0, 5.0]]], [[[-7.0, -4.0, 1.0], [6.0, 2.0, -6.0], [7.0, 6.0, 0.0], [-5.0, -3.0, 7.0]], [[1.0, -5.0, -2.0], [0.0, -2.0, -5.0], [0.0, -4.0, -1.0], [-5.0, 6.0, -6.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[92.0, -85.0], [-21.0, -46.0]])+(1.-msk_ref)*numpy.array([[116.0, -63.0], [-18.0, 46.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-7.0, -2.0])+(1.-msk_arg0)*numpy.array([1.0, -5.0])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[-1.0, 1.0, 7.0, 2.0, 6.0], [1.0, -1.0, -7.0, 6.0, -6.0], [7.0, -4.0, -3.0, 1.0, 0.0], [4.0, 7.0, -3.0, 2.0, 0.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[7.0, -7.0, -49.0, -14.0, -42.0], [-7.0, 7.0, 49.0, -42.0, 42.0], [-49.0, 28.0, 21.0, -7.0, 0.0], [-28.0, -49.0, 21.0, -14.0, 0.0]], [[2.0, -2.0, -14.0, -4.0, -12.0], [-2.0, 2.0, 14.0, -12.0, 12.0], [-14.0, 8.0, 6.0, -2.0, 0.0], [-8.0, -14.0, 6.0, -4.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-1.0, 1.0, 7.0, 2.0, 6.0], [1.0, -1.0, -7.0, 6.0, -6.0], [7.0, -4.0, -3.0, 1.0, 0.0], [4.0, 7.0, -3.0, 2.0, 0.0]], [[5.0, -5.0, -35.0, -10.0, -30.0], [-5.0, 5.0, 35.0, -30.0, 30.0], [-35.0, 20.0, 15.0, -5.0, 0.0], [-20.0, -35.0, 15.0, -10.0, 0.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[3.0, 2.0, -7.0], [1.0, 6.0, -7.0]])+(1.-msk_arg0)*numpy.array([[-3.0, 4.0, -6.0], [2.0, 0.0, -2.0]])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[4.0, 0.0, -7.0], [-1.0, -5.0, 6.0], [-2.0, -1.0, 4.0], [-5.0, 3.0, -7.0], [-5.0, 2.0, -7.0]], [[-3.0, 5.0, -5.0], [-3.0, -4.0, 0.0], [4.0, -4.0, 7.0], [0.0, -4.0, 1.0], [2.0, -6.0, 4.0]], [[-4.0, -4.0, -2.0], [-6.0, 0.0, -4.0], [4.0, -5.0, 2.0], [-2.0, 4.0, 6.0], [6.0, 4.0, -7.0]], [[-4.0, 0.0, 4.0], [-7.0, -5.0, 5.0], [5.0, 1.0, 2.0], [-5.0, -5.0, -6.0], [2.0, 5.0, -4.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[61.0, -55.0, -36.0, 40.0, 38.0], [36.0, -17.0, -45.0, -15.0, -34.0], [-6.0, 10.0, -12.0, -40.0, 75.0], [-40.0, -66.0, 3.0, 17.0, 44.0]], [[53.0, -73.0, -36.0, 62.0, 56.0], [62.0, -27.0, -69.0, -31.0, -62.0], [-14.0, 22.0, -40.0, -20.0, 79.0], [-32.0, -72.0, -3.0, 7.0, 60.0]]])+(1.-msk_ref)*numpy.array([[[30.0, -53.0, -22.0, 69.0, 65.0], [59.0, -7.0, -70.0, -22.0, -54.0], [8.0, 42.0, -44.0, -14.0, 40.0], [-12.0, -29.0, -23.0, 31.0, 38.0]], [[22.0, -14.0, -12.0, 4.0, 4.0], [4.0, -6.0, -6.0, -2.0, -4.0], [-4.0, -4.0, 4.0, -16.0, 26.0], [-16.0, -24.0, 6.0, 2.0, 12.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -4.0, -7.0], [7.0, -7.0, 6.0]], [[-4.0, 1.0, 5.0], [7.0, -1.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 5.0, -5.0], [4.0, 4.0, 6.0]], [[1.0, 1.0, -2.0], [-1.0, 5.0, 6.0]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[-7.0, 0.0, 7.0], [-4.0, 0.0, 0.0]], [[0.0, 5.0, -2.0], [1.0, 6.0, 3.0]], [[2.0, 0.0, 1.0], [3.0, 3.0, 7.0]], [[0.0, 0.0, 1.0], [-1.0, 6.0, 4.0]], [[7.0, 5.0, 3.0], [3.0, 3.0, -2.0]]], [[[3.0, -7.0, 1.0], [-2.0, 0.0, -6.0]], [[4.0, -7.0, -5.0], [-1.0, 6.0, -7.0]], [[-4.0, 0.0, 0.0], [5.0, -2.0, -6.0]], [[-7.0, -3.0, -6.0], [3.0, -6.0, 0.0]], [[5.0, -4.0, -7.0], [2.0, 0.0, -7.0]]], [[[-5.0, 3.0, -7.0], [0.0, 0.0, -5.0]], [[-1.0, 2.0, -5.0], [1.0, 6.0, 5.0]], [[1.0, 1.0, 3.0], [7.0, 5.0, 2.0]], [[5.0, 5.0, 0.0], [4.0, 7.0, -6.0]], [[-1.0, -5.0, 0.0], [5.0, -1.0, -7.0]]], [[[2.0, -6.0, 2.0], [1.0, -7.0, -3.0]], [[-2.0, -6.0, -3.0], [7.0, 1.0, 0.0]], [[-7.0, 2.0, 0.0], [-7.0, -4.0, -1.0]], [[0.0, 1.0, 2.0], [-1.0, 5.0, 0.0]], [[0.0, 6.0, -6.0], [6.0, 5.0, 2.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-77.0, -23.0, 35.0, -32.0, -53.0], [-29.0, -28.0, 13.0, 117.0, 37.0], [7.0, 22.0, 1.0, -77.0, 20.0], [48.0, 87.0, -35.0, -60.0, 37.0]], [[35.0, 17.0, 64.0, 20.0, -4.0], [-70.0, -110.0, 11.0, 22.0, -94.0], [-47.0, 17.0, 70.0, -36.0, -14.0], [-11.0, 35.0, -22.0, -1.0, 27.0]]])+(1.-msk_ref)*numpy.array([[[-44.0, 81.0, 59.0, 39.0, 15.0], [-87.0, -36.0, -20.0, 10.0, -24.0], [25.0, 94.0, 49.0, 28.0, -50.0], [-84.0, 19.0, -33.0, 11.0, 116.0]], [[-17.0, 56.0, 54.0, 53.0, 6.0], [-40.0, -4.0, -55.0, -31.0, -29.0], [-18.0, 70.0, 26.0, 5.0, -58.0], [-62.0, -4.0, -24.0, 23.0, 49.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_Symbol_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([6.0, 0.0])+(1.-msk_arg0)*numpy.array([1.0, -5.0])
      arg1=Symbol(shape=(6, 2, 2))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[[1.0, -4.0], [-1.0, 0.0]], [[-3.0, -2.0], [-4.0, -1.0]], [[2.0, 7.0], [2.0, 5.0]], [[-4.0, 0.0], [-1.0, -4.0]], [[7.0, 4.0], [-3.0, -4.0]], [[5.0, -5.0], [5.0, 6.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, -24.0], [-6.0, 0.0]], [[-18.0, -12.0], [-24.0, -6.0]], [[12.0, 42.0], [12.0, 30.0]], [[-24.0, 0.0], [-6.0, -24.0]], [[42.0, 24.0], [-18.0, -24.0]], [[30.0, -30.0], [30.0, 36.0]]], [[[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]], [[0.0, 0.0], [0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[1.0, -4.0], [-1.0, 0.0]], [[-3.0, -2.0], [-4.0, -1.0]], [[2.0, 7.0], [2.0, 5.0]], [[-4.0, 0.0], [-1.0, -4.0]], [[7.0, 4.0], [-3.0, -4.0]], [[5.0, -5.0], [5.0, 6.0]]], [[[-5.0, 20.0], [5.0, 0.0]], [[15.0, 10.0], [20.0, 5.0]], [[-10.0, -35.0], [-10.0, -25.0]], [[20.0, 0.0], [5.0, 20.0]], [[-35.0, -20.0], [15.0, 20.0]], [[-25.0, 25.0], [-25.0, -30.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 2.0, -1.0], [-6.0, -3.0, 3.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 3.0, -2.0], [7.0, -4.0, 6.0]])
      arg1=Symbol(shape=(6, 2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[[6.0, 0.0, 3.0], [-3.0, -6.0, 6.0]], [[3.0, -6.0, -6.0], [3.0, 5.0, 6.0]]], [[[-6.0, -6.0, 2.0], [-7.0, 3.0, 0.0]], [[0.0, -7.0, 0.0], [4.0, 1.0, 4.0]]], [[[2.0, 0.0, 3.0], [4.0, 0.0, -7.0]], [[-2.0, -5.0, 2.0], [1.0, 0.0, 7.0]]], [[[0.0, 1.0, 6.0], [-7.0, 0.0, -5.0]], [[1.0, -1.0, 0.0], [7.0, 1.0, 7.0]]], [[[-2.0, -5.0, -6.0], [-3.0, 0.0, 5.0]], [[-2.0, 3.0, 5.0], [3.0, 5.0, 0.0]]], [[[0.0, 7.0, 7.0], [2.0, 5.0, -5.0]], [[-5.0, 2.0, -3.0], [2.0, 0.0, 6.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-3.0, -18.0], [-6.0, 4.0]], [[-14.0, 6.0], [-14.0, -2.0]], [[-3.0, 7.0], [-12.0, -7.0]], [[-4.0, 5.0], [-2.0, -5.0]], [[-4.0, -5.0], [1.0, 10.0]], [[7.0, 15.0], [7.0, -6.0]]], [[[-27.0, 54.0], [-18.0, -15.0]], [[60.0, 33.0], [21.0, -15.0]], [[-3.0, -45.0], [33.0, 15.0]], [[15.0, 27.0], [-3.0, -24.0]], [[9.0, 33.0], [18.0, -33.0]], [[0.0, -42.0], [15.0, 6.0]]]])+(1.-msk_ref)*numpy.array([[[[-12.0, -27.0], [-9.0, 0.0]], [[-16.0, 16.0], [-21.0, -9.0]], [[-8.0, 10.0], [-17.0, -15.0]], [[-9.0, 17.0], [-4.0, -18.0]], [[-1.0, -7.0], [1.0, 12.0]], [[7.0, 23.0], [17.0, -14.0]]], [[[60.0, 39.0], [9.0, 37.0]], [[-6.0, -61.0], [28.0, 48.0]], [[32.0, -14.0], [18.0, 49.0]], [[32.0, -79.0], [11.0, 87.0]], [[-30.0, 9.0], [4.0, 1.0]], [[14.0, -36.0], [-61.0, 50.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([0.0, 2.0])+(1.-msk_arg0)*numpy.array([3.0, 1.0])
      arg1=Data(-7.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-0.0, -14.0])+(1.-msk_ref)*numpy.array([-21.0, -7.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 2.0, 7.0], [5.0, -7.0, 0.0]])+(1.-msk_arg0)*numpy.array([[3.0, -5.0, 4.0], [-2.0, -5.0, -6.0]])
      arg1=Data(numpy.array([-2.0, -7.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-6.0, 39.0])+(1.-msk_ref)*numpy.array([29.0, 39.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, 4.0, 6.0], [0.0, -7.0, 4.0]], [[-3.0, 1.0, -3.0], [1.0, 4.0, -1.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, 7.0, -3.0], [-1.0, 6.0, -4.0]], [[-3.0, 0.0, -1.0], [3.0, 2.0, 1.0]]])
      arg1=Data(numpy.array([[-2.0, 1.0, 1.0], [-4.0, -6.0, -4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([32.0, -20.0])+(1.-msk_ref)*numpy.array([-4.0, -23.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 7.0, -2.0], [6.0, 3.0, 5.0], [7.0, 4.0, -6.0], [-7.0, -7.0, 5.0]], [[-1.0, -5.0, 0.0], [-7.0, -3.0, 0.0], [0.0, 2.0, -6.0], [-5.0, 4.0, 7.0]]], [[[4.0, 2.0, -4.0], [6.0, 7.0, -3.0], [7.0, -7.0, 1.0], [-1.0, 7.0, 3.0]], [[-4.0, 1.0, 5.0], [7.0, 3.0, -6.0], [6.0, 3.0, -2.0], [-7.0, 7.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[3.0, 0.0, 4.0], [-3.0, 7.0, -1.0], [4.0, 7.0, -4.0], [2.0, -7.0, 5.0]], [[7.0, -1.0, -2.0], [-5.0, 4.0, 3.0], [-2.0, -6.0, -5.0], [-4.0, -4.0, 2.0]]], [[[-4.0, -6.0, 0.0], [-6.0, -3.0, -7.0], [3.0, -7.0, -3.0], [6.0, 7.0, 0.0]], [[1.0, -4.0, 0.0], [7.0, 1.0, -6.0], [-4.0, 3.0, 1.0], [1.0, 2.0, -2.0]]]])
      arg1=Data(numpy.array([[[-5.0, -3.0, -4.0], [-3.0, -2.0, -1.0], [7.0, 6.0, 6.0], [4.0, 4.0, -4.0]], [[-3.0, 1.0, -3.0], [-4.0, -7.0, 4.0], [0.0, 2.0, 7.0], [6.0, -6.0, -7.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-195.0, -132.0])+(1.-msk_ref)*numpy.array([-102.0, 37.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, 3.0])+(1.-msk_arg0)*numpy.array([-7.0, -1.0])
      arg1=Data(numpy.array([-6.0, 6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, -6.0], [-18.0, 18.0]])+(1.-msk_ref)*numpy.array([[42.0, -42.0], [6.0, -6.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-1.0, -5.0, -1.0], [3.0, 5.0, 4.0]])+(1.-msk_arg0)*numpy.array([[3.0, 0.0, -6.0], [1.0, 0.0, -7.0]])
      arg1=Data(numpy.array([[-3.0, 1.0, -7.0], [3.0, -1.0, 0.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[5.0, 2.0], [-32.0, 4.0]])+(1.-msk_ref)*numpy.array([[33.0, 9.0], [46.0, 3.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 1.0, 0.0], [-2.0, 3.0, -4.0]], [[0.0, 3.0, -3.0], [0.0, 1.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 3.0, -2.0], [0.0, 7.0, -1.0]], [[3.0, -1.0, -7.0], [-3.0, -6.0, -1.0]]])
      arg1=Data(numpy.array([[[6.0, 1.0, -2.0], [3.0, 1.0, -2.0]], [[-5.0, 5.0, -4.0], [6.0, 3.0, -4.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-24.0, 43.0], [-4.0, 2.0]])+(1.-msk_ref)*numpy.array([[28.0, 38.0], [18.0, -24.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 5.0, -4.0], [7.0, 0.0, 0.0], [0.0, -1.0, 4.0], [6.0, 3.0, -1.0]], [[7.0, -6.0, 2.0], [0.0, -4.0, 3.0], [-3.0, 0.0, 3.0], [4.0, 0.0, -5.0]]], [[[-4.0, -6.0, -6.0], [-5.0, -3.0, 3.0], [-1.0, -2.0, 3.0], [7.0, -4.0, 4.0]], [[1.0, -4.0, 2.0], [7.0, 0.0, -6.0], [4.0, -3.0, 0.0], [-5.0, 6.0, -1.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -6.0, 0.0], [7.0, -2.0, -2.0], [3.0, 3.0, -7.0], [3.0, 2.0, -7.0]], [[3.0, -6.0, -1.0], [0.0, -1.0, 4.0], [1.0, -1.0, 2.0], [-4.0, 5.0, 2.0]]], [[[-7.0, 4.0, 1.0], [0.0, -5.0, -1.0], [-4.0, -2.0, 4.0], [2.0, -3.0, 2.0]], [[1.0, 6.0, 7.0], [6.0, -3.0, 2.0], [-1.0, -4.0, -7.0], [-5.0, -2.0, 2.0]]]])
      arg1=Data(numpy.array([[[[-6.0, 1.0, 6.0], [4.0, 7.0, -7.0], [-5.0, 5.0, -1.0], [4.0, -1.0, -1.0]], [[1.0, -5.0, 7.0], [0.0, -3.0, 2.0], [-1.0, -1.0, -6.0], [0.0, -6.0, -4.0]]], [[[-6.0, 3.0, -6.0], [-7.0, -6.0, 4.0], [0.0, -2.0, 6.0], [0.0, 0.0, 1.0]], [[1.0, 0.0, 5.0], [-7.0, 5.0, 0.0], [-2.0, 2.0, 2.0], [-2.0, -4.0, 3.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[66.0, -29.0], [-70.0, 64.0]])+(1.-msk_ref)*numpy.array([[57.0, -107.0], [123.0, 87.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, 0.0])+(1.-msk_arg0)*numpy.array([-1.0, -4.0])
      arg1=Data(numpy.array([[3.0, 0.0, 3.0, -3.0, 4.0], [1.0, 1.0, -3.0, 3.0, 0.0], [4.0, 0.0, -7.0, -1.0, 1.0], [-5.0, 0.0, 5.0, -4.0, -5.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 0.0, -6.0, 6.0, -8.0], [-2.0, -2.0, 6.0, -6.0, 0.0], [-8.0, 0.0, 14.0, 2.0, -2.0], [10.0, 0.0, -10.0, 8.0, 10.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[-3.0, 0.0, -3.0, 3.0, -4.0], [-1.0, -1.0, 3.0, -3.0, 0.0], [-4.0, 0.0, 7.0, 1.0, -1.0], [5.0, 0.0, -5.0, 4.0, 5.0]], [[-12.0, 0.0, -12.0, 12.0, -16.0], [-4.0, -4.0, 12.0, -12.0, 0.0], [-16.0, 0.0, 28.0, 4.0, -4.0], [20.0, 0.0, -20.0, 16.0, 20.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, 2.0, -5.0], [6.0, -5.0, 0.0]])+(1.-msk_arg0)*numpy.array([[0.0, 0.0, -4.0], [7.0, 0.0, -4.0]])
      arg1=Data(numpy.array([[[-5.0, 7.0, -3.0], [0.0, -1.0, -1.0], [7.0, 4.0, -6.0], [4.0, -7.0, -2.0], [-7.0, 1.0, 5.0]], [[-5.0, -5.0, 5.0], [-6.0, 5.0, 0.0], [4.0, 5.0, -1.0], [6.0, 6.0, 3.0], [4.0, 7.0, -7.0]], [[1.0, -7.0, 2.0], [3.0, 6.0, -5.0], [0.0, 2.0, -6.0], [0.0, 0.0, 5.0], [2.0, 0.0, -1.0]], [[4.0, -4.0, 1.0], [0.0, 2.0, 4.0], [5.0, 1.0, -6.0], [-3.0, -7.0, 7.0], [1.0, 1.0, -6.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[64.0, 3.0, -11.0, -32.0, 26.0], [0.0, 52.0, -13.0, -45.0, 21.0], [-31.0, 16.0, 34.0, -25.0, -9.0], [-41.0, -16.0, -3.0, -28.0, 25.0]], [[-65.0, 5.0, 22.0, 59.0, -47.0], [-5.0, -61.0, -1.0, 6.0, -11.0], [41.0, -12.0, -10.0, 0.0, 12.0], [44.0, -10.0, 25.0, 17.0, 1.0]]])+(1.-msk_ref)*numpy.array([[[12.0, 4.0, 24.0, 8.0, -20.0], [-20.0, 0.0, 4.0, -12.0, 28.0], [-8.0, 20.0, 24.0, -20.0, 4.0], [-4.0, -16.0, 24.0, -28.0, 24.0]], [[-23.0, 4.0, 73.0, 36.0, -69.0], [-55.0, -42.0, 32.0, 30.0, 56.0], [-1.0, 41.0, 24.0, -20.0, 18.0], [24.0, -16.0, 59.0, -49.0, 31.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 7.0, 0.0], [-1.0, -6.0, -7.0]], [[3.0, -3.0, 1.0], [-5.0, -6.0, 4.0]]])+(1.-msk_arg0)*numpy.array([[[7.0, 2.0, -7.0], [-2.0, 2.0, 7.0]], [[2.0, 3.0, -3.0], [7.0, -5.0, 3.0]]])
      arg1=Data(numpy.array([[[[-2.0, 3.0, -2.0], [-5.0, -6.0, 0.0]], [[-3.0, -2.0, -6.0], [-6.0, -7.0, -2.0]], [[-6.0, -2.0, -6.0], [-7.0, -5.0, 7.0]], [[-1.0, 6.0, 2.0], [-2.0, -3.0, -7.0]], [[-2.0, -1.0, 5.0], [-7.0, 2.0, -7.0]]], [[[4.0, 0.0, 5.0], [-2.0, 5.0, -2.0]], [[-7.0, 0.0, 5.0], [5.0, -5.0, -4.0]], [[3.0, -7.0, 0.0], [-1.0, 7.0, -2.0]], [[0.0, -6.0, 0.0], [4.0, 2.0, -5.0]], [[0.0, -5.0, -3.0], [-7.0, 4.0, 6.0]]], [[[-6.0, 0.0, 4.0], [-7.0, 6.0, 0.0]], [[-1.0, 1.0, 0.0], [4.0, -7.0, 2.0]], [[-6.0, 5.0, 4.0], [4.0, -4.0, -6.0]], [[-1.0, -1.0, -4.0], [4.0, 6.0, -5.0]], [[5.0, -2.0, 1.0], [2.0, -7.0, 1.0]]], [[[2.0, 0.0, 4.0], [-1.0, 5.0, -4.0]], [[-3.0, 3.0, 6.0], [0.0, -7.0, 2.0]], [[-7.0, -7.0, -3.0], [3.0, -5.0, 2.0]], [[1.0, 1.0, -5.0], [-2.0, 1.0, -1.0]], [[-5.0, 4.0, -2.0], [7.0, 6.0, 7.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[62.0, 48.0, -26.0, 111.0, 37.0], [-14.0, 53.0, -76.0, -23.0, -94.0], [-29.0, 31.0, 97.0, -12.0, 19.0], [-1.0, 49.0, -36.0, 10.0, -64.0]], [[44.0, 55.0, 75.0, -19.0, -3.0], [-11.0, -27.0, -15.0, -34.0, 47.0], [-15.0, 24.0, -49.0, -80.0, 58.0], [-31.0, 38.0, 20.0, -5.0, -72.0]]])+(1.-msk_ref)*numpy.array([[[4.0, 1.0, 49.0, -60.0, -82.0], [-7.0, -132.0, 9.0, -51.0, 75.0], [-44.0, -13.0, -118.0, -12.0, 13.0], [-30.0, -57.0, -44.0, 43.0, 34.0]], [[6.0, -7.0, -3.0, -10.0, -102.0], [-52.0, 19.0, -63.0, -15.0, -57.0], [-103.0, 70.0, 21.0, -10.0, 53.0], [-52.0, 26.0, 26.0, -2.0, 48.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_constData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-1.0, -2.0])+(1.-msk_arg0)*numpy.array([-1.0, 5.0])
      arg1=Data(numpy.array([[[-7.0, 5.0], [-5.0, -1.0]], [[4.0, 3.0], [7.0, -3.0]], [[6.0, 0.0], [0.0, -1.0]], [[2.0, 5.0], [-3.0, 6.0]], [[0.0, -6.0], [-5.0, 6.0]], [[-7.0, 1.0], [-1.0, -2.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, -5.0], [5.0, 1.0]], [[-4.0, -3.0], [-7.0, 3.0]], [[-6.0, 0.0], [0.0, 1.0]], [[-2.0, -5.0], [3.0, -6.0]], [[0.0, 6.0], [5.0, -6.0]], [[7.0, -1.0], [1.0, 2.0]]], [[[14.0, -10.0], [10.0, 2.0]], [[-8.0, -6.0], [-14.0, 6.0]], [[-12.0, 0.0], [0.0, 2.0]], [[-4.0, -10.0], [6.0, -12.0]], [[0.0, 12.0], [10.0, -12.0]], [[14.0, -2.0], [2.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[7.0, -5.0], [5.0, 1.0]], [[-4.0, -3.0], [-7.0, 3.0]], [[-6.0, 0.0], [0.0, 1.0]], [[-2.0, -5.0], [3.0, -6.0]], [[0.0, 6.0], [5.0, -6.0]], [[7.0, -1.0], [1.0, 2.0]]], [[[-35.0, 25.0], [-25.0, -5.0]], [[20.0, 15.0], [35.0, -15.0]], [[30.0, 0.0], [0.0, -5.0]], [[10.0, 25.0], [-15.0, 30.0]], [[0.0, -30.0], [-25.0, 30.0]], [[-35.0, 5.0], [-5.0, -10.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -3.0, -6.0], [4.0, -5.0, -4.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 7.0, 5.0], [3.0, 0.0, 0.0]])
      arg1=Data(numpy.array([[[[1.0, -6.0, 3.0], [0.0, -7.0, 0.0]], [[1.0, -1.0, -5.0], [-2.0, 4.0, 7.0]]], [[[4.0, -7.0, 6.0], [3.0, -3.0, 3.0]], [[3.0, -4.0, -3.0], [2.0, -2.0, 6.0]]], [[[1.0, -2.0, 0.0], [4.0, -1.0, 6.0]], [[3.0, 3.0, -1.0], [-1.0, 6.0, 4.0]]], [[[3.0, 2.0, -7.0], [2.0, -5.0, 7.0]], [[6.0, 1.0, 1.0], [5.0, 5.0, -2.0]]], [[[1.0, 1.0, -3.0], [6.0, -5.0, -5.0]], [[-6.0, -6.0, -2.0], [-1.0, 0.0, 0.0]]], [[[-2.0, -7.0, -7.0], [4.0, 3.0, 0.0]], [[2.0, -1.0, -1.0], [2.0, 3.0, -1.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, 21.0], [37.0, -62.0]], [[1.0, 3.0], [42.0, -22.0]], [[10.0, -17.0], [9.0, -46.0]], [[48.0, -19.0], [15.0, 17.0]], [[19.0, 69.0], [6.0, -4.0]], [[55.0, 7.0], [17.0, 5.0]]], [[[22.0, 35.0], [29.0, -56.0]], [[27.0, 15.0], [44.0, -6.0]], [[14.0, -3.0], [1.0, -50.0]], [[30.0, 5.0], [15.0, 3.0]], [[11.0, 69.0], [14.0, -4.0]], [[55.0, 1.0], [17.0, -3.0]]]])+(1.-msk_ref)*numpy.array([[[[-29.0, -49.0], [-34.0, 67.0]], [[-27.0, -12.0], [-49.0, 12.0]], [[-16.0, 15.0], [10.0, 64.0]], [[-27.0, -4.0], [0.0, 15.0]], [[-10.0, -72.0], [-40.0, 2.0]], [[-80.0, 13.0], [-16.0, 12.0]]], [[[3.0, 0.0], [3.0, -6.0]], [[12.0, 9.0], [9.0, 6.0]], [[3.0, 12.0], [9.0, -3.0]], [[9.0, 6.0], [18.0, 15.0]], [[3.0, 18.0], [-18.0, -3.0]], [[-6.0, 12.0], [6.0, 6.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([6.0, 0.0])+(1.-msk_arg0)*numpy.array([6.0, 5.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(0.0)+(1-msk_arg1)*(1.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([0.0, 0.0])+(1.-msk_ref)*numpy.array([6.0, 5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 1.0, 3.0], [-7.0, -4.0, 0.0]])+(1.-msk_arg0)*numpy.array([[3.0, -4.0, 5.0], [-5.0, 5.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-5.0, 1.0, -7.0])+(1.-msk_arg1)*numpy.array([2.0, -3.0, 0.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([-20.0, 31.0])+(1.-msk_ref)*numpy.array([18.0, -25.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-4.0, -7.0, -6.0], [-5.0, -6.0, -1.0]], [[-2.0, -6.0, 6.0], [5.0, 7.0, -7.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, -7.0, 0.0], [2.0, 0.0, 5.0]], [[4.0, 5.0, 0.0], [2.0, -2.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[1.0, -6.0, -2.0], [0.0, 2.0, 0.0]])+(1.-msk_arg1)*numpy.array([[4.0, 3.0, 5.0], [-7.0, 3.0, 5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([38.0, 36.0])+(1.-msk_ref)*numpy.array([-38.0, -19.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank3_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, -6.0, 0.0], [-5.0, -4.0, 1.0], [7.0, -6.0, -3.0], [-4.0, -2.0, -5.0]], [[3.0, -1.0, 3.0], [3.0, 3.0, -3.0], [-2.0, 5.0, -2.0], [-2.0, 0.0, 5.0]]], [[[6.0, 5.0, 7.0], [1.0, 2.0, 4.0], [5.0, -6.0, 0.0], [2.0, 1.0, 0.0]], [[-4.0, -2.0, 4.0], [5.0, 1.0, 3.0], [4.0, 2.0, 6.0], [-6.0, 4.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, 5.0, -4.0], [-7.0, 5.0, 1.0], [4.0, -1.0, 4.0], [-6.0, 5.0, 2.0]], [[-7.0, -6.0, 0.0], [-3.0, -7.0, 0.0], [-6.0, 4.0, 0.0], [-4.0, 2.0, -3.0]]], [[[-5.0, 0.0, 4.0], [1.0, -4.0, -5.0], [-3.0, 5.0, 1.0], [7.0, 0.0, 4.0]], [[7.0, 4.0, 2.0], [5.0, 4.0, -4.0], [-3.0, 0.0, -2.0], [3.0, -6.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[3.0, 4.0, -1.0], [6.0, 0.0, -2.0], [7.0, 5.0, -4.0], [-3.0, -2.0, 7.0]], [[-2.0, 0.0, 1.0], [-4.0, 7.0, -7.0], [-3.0, 4.0, -3.0], [2.0, 5.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[1.0, -7.0, 6.0], [-6.0, 4.0, -4.0], [7.0, 0.0, 6.0], [-1.0, -7.0, -7.0]], [[2.0, 1.0, -1.0], [-4.0, 6.0, 0.0], [-3.0, -6.0, -1.0], [3.0, -3.0, 4.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([20.0, 20.0])+(1.-msk_ref)*numpy.array([-82.0, 5.0])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2,),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([-2.0, 0.0])+(1.-msk_arg0)*numpy.array([7.0, -3.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([6.0, 5.0])+(1.-msk_arg1)*numpy.array([-6.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-12.0, -10.0], [0.0, 0.0]])+(1.-msk_ref)*numpy.array([[-42.0, 49.0], [18.0, -21.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-7.0, 6.0, -3.0], [0.0, -3.0, 0.0]])+(1.-msk_arg0)*numpy.array([[5.0, -4.0, -3.0], [-7.0, 0.0, 0.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-4.0, -1.0, 5.0], [3.0, -4.0, 5.0]])+(1.-msk_arg1)*numpy.array([[1.0, -6.0, 5.0], [-2.0, 0.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[7.0, -60.0], [3.0, 12.0]])+(1.-msk_ref)*numpy.array([[14.0, -28.0], [-7.0, 14.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 0.0, -6.0], [3.0, -7.0, -4.0]], [[6.0, -4.0, 4.0], [-3.0, -1.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 0.0, -2.0], [-1.0, -3.0, 0.0]], [[0.0, -5.0, 3.0], [-2.0, 0.0, -5.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[1.0, 6.0, -6.0], [-6.0, 3.0, 1.0]], [[7.0, 4.0, 6.0], [7.0, -2.0, -3.0]]])+(1.-msk_arg1)*numpy.array([[[-4.0, 4.0, 2.0], [2.0, 0.0, -5.0]], [[6.0, -1.0, 3.0], [5.0, -4.0, 7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-14.0, -38.0], [-20.0, 10.0]])+(1.-msk_ref)*numpy.array([[-6.0, 1.0], [7.0, -31.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank4_offset3(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 0.0, 4.0], [-1.0, 1.0, 2.0], [5.0, 7.0, 7.0], [-7.0, -7.0, 4.0]], [[2.0, 0.0, -7.0], [6.0, 0.0, -4.0], [0.0, 7.0, 2.0], [-3.0, -2.0, 7.0]]], [[[7.0, 2.0, -2.0], [-2.0, -1.0, 0.0], [-3.0, -6.0, 6.0], [-7.0, -7.0, 3.0]], [[0.0, 3.0, 0.0], [-5.0, 0.0, 5.0], [-3.0, -7.0, 0.0], [2.0, 4.0, -2.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -2.0, 5.0], [0.0, 0.0, -5.0], [3.0, 1.0, -4.0], [-7.0, -4.0, 6.0]], [[-6.0, -1.0, 0.0], [3.0, 6.0, 2.0], [4.0, -4.0, 0.0], [-2.0, 1.0, 2.0]]], [[[0.0, -1.0, -3.0], [6.0, 6.0, -7.0], [1.0, 1.0, 2.0], [0.0, -5.0, -6.0]], [[-4.0, 4.0, -5.0], [-7.0, 6.0, -4.0], [7.0, -5.0, 7.0], [-2.0, -7.0, -6.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[4.0, -2.0, 3.0], [-5.0, 3.0, -2.0], [-4.0, 4.0, 2.0], [-4.0, 4.0, 5.0]], [[5.0, -7.0, 7.0], [0.0, 2.0, 7.0], [6.0, 3.0, 3.0], [-6.0, 2.0, 7.0]]], [[[3.0, -7.0, 5.0], [5.0, -6.0, 6.0], [-5.0, 7.0, 0.0], [2.0, 5.0, 1.0]], [[0.0, 1.0, 7.0], [-4.0, -6.0, -1.0], [-5.0, 3.0, -7.0], [0.0, -7.0, -1.0]]]])+(1.-msk_arg1)*numpy.array([[[[-7.0, 6.0, -3.0], [2.0, 4.0, -7.0], [-3.0, 2.0, 4.0], [7.0, 0.0, 6.0]], [[2.0, 0.0, 3.0], [0.0, 3.0, -1.0], [2.0, -4.0, -3.0], [-6.0, 1.0, -1.0]]], [[[4.0, -4.0, 7.0], [5.0, 0.0, -5.0], [-6.0, -5.0, 3.0], [-5.0, 0.0, 0.0]], [[-5.0, 6.0, 0.0], [-6.0, 1.0, 5.0], [7.0, 6.0, -2.0], [6.0, -4.0, -5.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=3)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[101.0, -40.0], [-3.0, -94.0]])+(1.-msk_ref)*numpy.array([[11.0, 68.0], [82.0, 166.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([1.0, 5.0])+(1.-msk_arg0)*numpy.array([-2.0, -2.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-6.0, 1.0, -4.0, 5.0, -6.0], [0.0, -2.0, 7.0, 5.0, -4.0], [-3.0, 0.0, 7.0, 0.0, -4.0], [-1.0, -4.0, 4.0, 3.0, 1.0]])+(1.-msk_arg1)*numpy.array([[-7.0, -7.0, 2.0, 5.0, -4.0], [-7.0, 3.0, -7.0, -4.0, -6.0], [6.0, 7.0, -6.0, 3.0, 1.0], [6.0, 2.0, -2.0, 5.0, 5.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 1.0, -4.0, 5.0, -6.0], [0.0, -2.0, 7.0, 5.0, -4.0], [-3.0, 0.0, 7.0, 0.0, -4.0], [-1.0, -4.0, 4.0, 3.0, 1.0]], [[-30.0, 5.0, -20.0, 25.0, -30.0], [0.0, -10.0, 35.0, 25.0, -20.0], [-15.0, 0.0, 35.0, 0.0, -20.0], [-5.0, -20.0, 20.0, 15.0, 5.0]]])+(1.-msk_ref)*numpy.array([[[14.0, 14.0, -4.0, -10.0, 8.0], [14.0, -6.0, 14.0, 8.0, 12.0], [-12.0, -14.0, 12.0, -6.0, -2.0], [-12.0, -4.0, 4.0, -10.0, -10.0]], [[14.0, 14.0, -4.0, -10.0, 8.0], [14.0, -6.0, 14.0, 8.0, 12.0], [-12.0, -14.0, 12.0, -6.0, -2.0], [-12.0, -4.0, 4.0, -10.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 7.0, -4.0], [3.0, 3.0, -2.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0, 4.0], [-6.0, 2.0, 3.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-5.0, -5.0, -5.0], [-5.0, -3.0, 0.0], [5.0, -4.0, 5.0], [7.0, -2.0, 2.0], [0.0, 3.0, -2.0]], [[-6.0, -3.0, 2.0], [4.0, 1.0, 6.0], [0.0, -6.0, 0.0], [3.0, 6.0, -5.0], [6.0, -4.0, 3.0]], [[6.0, 2.0, 7.0], [0.0, 5.0, 0.0], [4.0, 4.0, 1.0], [3.0, 7.0, -1.0], [3.0, 7.0, -1.0]], [[0.0, -1.0, 6.0], [-7.0, -7.0, 6.0], [-4.0, 1.0, 7.0], [-4.0, -5.0, -3.0], [0.0, 5.0, -7.0]]])+(1.-msk_arg1)*numpy.array([[[7.0, 5.0, 6.0], [6.0, -7.0, 7.0], [0.0, -4.0, -2.0], [7.0, -5.0, 6.0], [6.0, -3.0, 0.0]], [[-3.0, 5.0, 2.0], [-7.0, -6.0, -7.0], [2.0, -3.0, -4.0], [-4.0, -6.0, 0.0], [1.0, 3.0, -3.0]], [[0.0, 0.0, -7.0], [-3.0, 2.0, -6.0], [1.0, 1.0, -3.0], [-4.0, -3.0, -6.0], [2.0, 6.0, 4.0]], [[0.0, 1.0, -1.0], [2.0, 7.0, 2.0], [-1.0, 1.0, 1.0], [6.0, 7.0, 3.0], [-3.0, 0.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-40.0, -46.0, -23.0, 13.0, 29.0], [-59.0, 3.0, -42.0, 77.0, -10.0], [16.0, 35.0, 44.0, 68.0, 68.0], [-31.0, -108.0, -41.0, -43.0, 63.0]], [[-20.0, -24.0, -7.0, 11.0, 13.0], [-31.0, 3.0, -18.0, 37.0, 0.0], [10.0, 15.0, 22.0, 32.0, 32.0], [-15.0, -54.0, -23.0, -21.0, 29.0]]])+(1.-msk_ref)*numpy.array([[[29.0, 21.0, -12.0, 19.0, -3.0], [13.0, -34.0, -19.0, -6.0, -9.0], [-28.0, -22.0, -11.0, -27.0, 22.0], [-3.0, 15.0, 5.0, 19.0, 8.0]], [[-14.0, -29.0, -14.0, -34.0, -42.0], [34.0, 9.0, -30.0, 12.0, -9.0], [-21.0, 4.0, -13.0, 0.0, 12.0], [-1.0, 8.0, 11.0, -13.0, 24.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 4.0, 7.0], [0.0, -5.0, 0.0]], [[-7.0, -6.0, -2.0], [3.0, 6.0, -2.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, 5.0, 5.0], [4.0, 4.0, -7.0]], [[5.0, 2.0, 3.0], [-5.0, -3.0, -1.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[-2.0, -2.0, -3.0], [4.0, 7.0, 0.0]], [[2.0, 4.0, 3.0], [-5.0, 0.0, -7.0]], [[4.0, -7.0, 3.0], [-6.0, 4.0, 6.0]], [[-2.0, -7.0, -4.0], [0.0, -3.0, 4.0]], [[4.0, 2.0, 0.0], [3.0, 6.0, 3.0]]], [[[-1.0, -6.0, 3.0], [-1.0, -5.0, -5.0]], [[3.0, -7.0, 5.0], [-7.0, -7.0, -3.0]], [[-1.0, -7.0, 2.0], [5.0, -7.0, 6.0]], [[-2.0, -3.0, 4.0], [-1.0, -7.0, 0.0]], [[4.0, -6.0, 3.0], [5.0, -5.0, -4.0]]], [[[-3.0, -1.0, 4.0], [-7.0, 5.0, 4.0]], [[-6.0, -5.0, 0.0], [7.0, -3.0, -1.0]], [[7.0, -7.0, -1.0], [3.0, -3.0, -2.0]], [[-2.0, -3.0, 3.0], [0.0, -4.0, -5.0]], [[5.0, -6.0, -5.0], [-6.0, 6.0, -4.0]]], [[[-3.0, 0.0, -6.0], [-5.0, -1.0, 3.0]], [[-5.0, -5.0, 2.0], [7.0, -3.0, 5.0]], [[0.0, 5.0, 7.0], [-2.0, -7.0, 0.0]], [[7.0, 3.0, 0.0], [-4.0, 6.0, 3.0]], [[0.0, 3.0, 2.0], [-3.0, -6.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[7.0, -2.0, -2.0], [3.0, 0.0, 1.0]], [[-2.0, -3.0, -4.0], [-6.0, 1.0, 3.0]], [[-2.0, 3.0, 0.0], [7.0, -2.0, -5.0]], [[-5.0, 0.0, 2.0], [0.0, 2.0, -5.0]], [[2.0, 5.0, -6.0], [-4.0, 5.0, -1.0]]], [[[-4.0, 4.0, -5.0], [-2.0, 6.0, 6.0]], [[0.0, -6.0, 3.0], [5.0, 4.0, 3.0]], [[5.0, 3.0, 1.0], [4.0, -1.0, -4.0]], [[-5.0, -6.0, 0.0], [6.0, -1.0, -6.0]], [[5.0, -6.0, 2.0], [1.0, 7.0, 5.0]]], [[[4.0, 7.0, 3.0], [-4.0, -2.0, -1.0]], [[0.0, 0.0, -2.0], [4.0, -7.0, 3.0]], [[5.0, 6.0, -3.0], [6.0, -4.0, 4.0]], [[4.0, -6.0, -4.0], [-7.0, 3.0, 2.0]], [[-2.0, 5.0, -5.0], [-6.0, -5.0, 2.0]]], [[[0.0, -6.0, -7.0], [-3.0, 1.0, 4.0]], [[-3.0, -3.0, 3.0], [3.0, -1.0, 1.0]], [[5.0, 0.0, 6.0], [1.0, 5.0, 2.0]], [[6.0, 6.0, 0.0], [7.0, 6.0, -2.0]], [[-1.0, 4.0, 6.0], [1.0, 0.0, 6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-54.0, 27.0, -47.0, -31.0, -42.0], [27.0, 27.0, 26.0, 61.0, 2.0], [14.0, 25.0, -55.0, 39.0, -114.0], [-22.0, 34.0, 104.0, -53.0, 56.0]], [[86.0, -45.0, 2.0, 38.0, -1.0], [14.0, -46.0, 6.0, -21.0, -5.0], [20.0, 77.0, -10.0, 12.0, 37.0], [6.0, 54.0, -92.0, -49.0, -67.0]]])+(1.-msk_ref)*numpy.array([[[-36.0, -70.0, 76.0, 68.0, 0.0], [-19.0, 0.0, 45.0, 47.0, -38.0], [21.0, -43.0, -20.0, -92.0, -52.0], [-101.0, 10.0, 25.0, 78.0, 15.0]], [[9.0, -4.0, -28.0, -20.0, 8.0], [-41.0, -43.0, 21.0, -58.0, -12.0], [70.0, -8.0, 6.0, 20.0, 28.0], [-25.0, -25.0, 21.0, -9.0, 10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank1_expandedData_rank3_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([3.0, -1.0])+(1.-msk_arg0)*numpy.array([-4.0, -7.0])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[2.0, -2.0], [-7.0, 6.0]], [[0.0, 0.0], [0.0, 0.0]], [[7.0, -3.0], [7.0, 2.0]], [[3.0, 3.0], [-4.0, 5.0]], [[0.0, -2.0], [-2.0, 6.0]], [[-4.0, 2.0], [0.0, 6.0]]])+(1.-msk_arg1)*numpy.array([[[5.0, 6.0], [-5.0, 3.0]], [[-5.0, -5.0], [-4.0, 7.0]], [[-3.0, 5.0], [4.0, 7.0]], [[3.0, 0.0], [0.0, -6.0]], [[-5.0, -1.0], [1.0, 1.0]], [[4.0, 0.0], [0.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, -6.0], [-21.0, 18.0]], [[0.0, 0.0], [0.0, 0.0]], [[21.0, -9.0], [21.0, 6.0]], [[9.0, 9.0], [-12.0, 15.0]], [[0.0, -6.0], [-6.0, 18.0]], [[-12.0, 6.0], [0.0, 18.0]]], [[[-2.0, 2.0], [7.0, -6.0]], [[0.0, 0.0], [0.0, 0.0]], [[-7.0, 3.0], [-7.0, -2.0]], [[-3.0, -3.0], [4.0, -5.0]], [[0.0, 2.0], [2.0, -6.0]], [[4.0, -2.0], [0.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-20.0, -24.0], [20.0, -12.0]], [[20.0, 20.0], [16.0, -28.0]], [[12.0, -20.0], [-16.0, -28.0]], [[-12.0, 0.0], [0.0, 24.0]], [[20.0, 4.0], [-4.0, -4.0]], [[-16.0, 0.0], [0.0, -8.0]]], [[[-35.0, -42.0], [35.0, -21.0]], [[35.0, 35.0], [28.0, -49.0]], [[21.0, -35.0], [-28.0, -49.0]], [[-21.0, 0.0], [0.0, 42.0]], [[35.0, 7.0], [-7.0, -7.0]], [[-28.0, 0.0], [0.0, -14.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank4_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[5.0, 0.0, 7.0], [3.0, -6.0, -4.0]])+(1.-msk_arg0)*numpy.array([[3.0, 0.0, 3.0], [5.0, 2.0, 6.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[2.0, -5.0, -6.0], [1.0, -6.0, 5.0]], [[-3.0, 5.0, -2.0], [4.0, 6.0, -7.0]]], [[[-5.0, 6.0, -4.0], [4.0, -7.0, 2.0]], [[0.0, 2.0, 2.0], [6.0, -4.0, 0.0]]], [[[5.0, -3.0, -3.0], [4.0, -5.0, -2.0]], [[6.0, -3.0, 4.0], [-1.0, 2.0, -3.0]]], [[[3.0, -2.0, -7.0], [-1.0, 6.0, -4.0]], [[0.0, 6.0, 7.0], [-7.0, 0.0, 4.0]]], [[[-3.0, 0.0, 1.0], [-2.0, -4.0, -7.0]], [[7.0, -4.0, 0.0], [-1.0, -3.0, -3.0]]], [[[5.0, -3.0, 2.0], [0.0, -7.0, 4.0]], [[6.0, -2.0, 1.0], [-2.0, 0.0, 0.0]]]])+(1.-msk_arg1)*numpy.array([[[[4.0, -3.0, -6.0], [-6.0, -4.0, 3.0]], [[0.0, 3.0, 0.0], [-4.0, 5.0, -1.0]]], [[[0.0, -1.0, -6.0], [-3.0, -7.0, 6.0]], [[-2.0, 2.0, 6.0], [1.0, 6.0, 5.0]]], [[[6.0, 4.0, 5.0], [-1.0, -3.0, -4.0]], [[-7.0, 2.0, -3.0], [6.0, -5.0, 3.0]]], [[[-1.0, -1.0, 4.0], [-5.0, 5.0, 2.0]], [[1.0, -4.0, 7.0], [3.0, 2.0, -1.0]]], [[[-3.0, -4.0, 3.0], [4.0, -7.0, -3.0]], [[-5.0, 0.0, 1.0], [0.0, 4.0, 6.0]]], [[[4.0, 2.0, -5.0], [5.0, -5.0, -3.0]], [[5.0, 2.0, -4.0], [0.0, -6.0, -6.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-32.0, 40.0], [-29.0, -29.0]], [[-53.0, 34.0], [14.0, 30.0]], [[4.0, 6.0], [58.0, -26.0]], [[-34.0, -33.0], [49.0, -7.0]], [[-8.0, -59.0], [35.0, -26.0]], [[39.0, 28.0], [37.0, -10.0]]], [[[60.0, 19.0], [-31.0, 4.0]], [[-35.0, 46.0], [-20.0, 42.0]], [[45.0, 50.0], [20.0, -3.0]], [[49.0, -23.0], [-64.0, -37.0]], [[-13.0, 46.0], [45.0, 27.0]], [[25.0, 26.0], [26.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, -9.0], [0.0, -15.0]], [[-18.0, 9.0], [12.0, 18.0]], [[33.0, -15.0], [-30.0, 27.0]], [[9.0, -9.0], [24.0, 6.0]], [[0.0, 3.0], [-12.0, 18.0]], [[-3.0, 6.0], [3.0, -18.0]]], [[[-22.0, -20.0], [6.0, -16.0]], [[-38.0, 7.0], [30.0, 47.0]], [[68.0, -35.0], [-49.0, 38.0]], [[17.0, -3.0], [39.0, 13.0]], [[-5.0, -12.0], [-19.0, 44.0]], [[-6.0, -3.0], [5.0, -48.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(2, 6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[0.0, 0.0, 2.0, 0.0, 0.0], [0.0, 4.0, -7.0, 3.0, -3.0], [1.0, 7.0, 0.0, -5.0, 5.0], [4.0, -2.0, 2.0, -2.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 1.0, -5.0, 2.0, 4.0], [-1.0, 1.0, 3.0, -6.0, -1.0], [-4.0, 7.0, -5.0, -5.0, -4.0], [7.0, 3.0, 3.0, 3.0, 0.0]])
      arg1=7.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[0.0, 0.0, 14.0, 0.0, 0.0], [0.0, 28.0, -49.0, 21.0, -21.0], [7.0, 49.0, 0.0, -35.0, 35.0], [28.0, -14.0, 14.0, -14.0, 0.0]])+(1.-msk_ref)*numpy.array([[-14.0, 7.0, -35.0, 14.0, 28.0], [-7.0, 7.0, 21.0, -42.0, -7.0], [-28.0, 49.0, -35.0, -35.0, -28.0], [49.0, 21.0, 21.0, 21.0, 0.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, -4.0, 0.0, -1.0, 3.0], [1.0, 6.0, 7.0, -5.0, -5.0], [4.0, -2.0, 1.0, -1.0, -2.0], [0.0, -5.0, 0.0, -1.0, 2.0]])+(1.-msk_arg0)*numpy.array([[5.0, -5.0, 5.0, -6.0, 2.0], [1.0, -2.0, 0.0, -5.0, 5.0], [0.0, 7.0, 7.0, -3.0, -4.0], [5.0, -1.0, -4.0, 0.0, 6.0]])
      arg1=numpy.array(3.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[12.0, -12.0, 0.0, -3.0, 9.0], [3.0, 18.0, 21.0, -15.0, -15.0], [12.0, -6.0, 3.0, -3.0, -6.0], [0.0, -15.0, 0.0, -3.0, 6.0]])+(1.-msk_ref)*numpy.array([[15.0, -15.0, 15.0, -18.0, 6.0], [3.0, -6.0, 0.0, -15.0, 15.0], [0.0, 21.0, 21.0, -9.0, -12.0], [15.0, -3.0, -12.0, 0.0, 18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 6.0, 0.0], [2.0, 1.0, -4.0], [4.0, 6.0, 4.0], [6.0, 3.0, -6.0], [2.0, 4.0, 4.0]], [[-2.0, 2.0, 4.0], [-4.0, 2.0, 6.0], [6.0, -7.0, 0.0], [4.0, 0.0, 1.0], [-7.0, -5.0, -1.0]], [[6.0, -1.0, 5.0], [5.0, -7.0, 5.0], [-2.0, -4.0, 4.0], [0.0, -3.0, 5.0], [-6.0, -5.0, 2.0]], [[-4.0, 1.0, 6.0], [4.0, 1.0, 5.0], [-2.0, -3.0, 0.0], [-6.0, -7.0, -1.0], [2.0, -1.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 0.0, 0.0], [-2.0, 1.0, 0.0], [0.0, 0.0, -6.0], [-2.0, 0.0, -4.0], [-6.0, -3.0, 0.0]], [[3.0, 7.0, -4.0], [6.0, -3.0, 5.0], [6.0, 4.0, -6.0], [0.0, -7.0, -6.0], [7.0, 0.0, 7.0]], [[4.0, 5.0, 0.0], [-4.0, 2.0, 5.0], [6.0, 3.0, 0.0], [-1.0, -3.0, -5.0], [5.0, -7.0, -4.0]], [[6.0, 4.0, 2.0], [-3.0, 0.0, 5.0], [6.0, 6.0, -4.0], [5.0, 6.0, -1.0], [7.0, 4.0, -7.0]]])
      arg1=numpy.array([4.0, 3.0, -5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[18.0, 31.0, 14.0, 63.0, 0.0], [-22.0, -40.0, 3.0, 11.0, -38.0], [-4.0, -26.0, -40.0, -34.0, -49.0], [-43.0, -6.0, -17.0, -40.0, 5.0]])+(1.-msk_ref)*numpy.array([[8.0, -5.0, 30.0, 12.0, -33.0], [53.0, -10.0, 66.0, 9.0, -7.0], [31.0, -35.0, 33.0, 12.0, 19.0], [26.0, -37.0, 62.0, 43.0, 75.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, -3.0, 0.0], [0.0, -3.0, -3.0]], [[0.0, 6.0, 5.0], [-2.0, 0.0, -3.0]], [[0.0, -5.0, -5.0], [-5.0, 2.0, 3.0]], [[-1.0, 6.0, 0.0], [-4.0, 5.0, 0.0]], [[-5.0, 2.0, 1.0], [0.0, 6.0, 5.0]]], [[[-6.0, 5.0, -1.0], [0.0, 3.0, -6.0]], [[2.0, 2.0, 7.0], [0.0, 0.0, 2.0]], [[-6.0, -1.0, -2.0], [4.0, 0.0, 3.0]], [[3.0, 1.0, 0.0], [-4.0, 0.0, 1.0]], [[-2.0, 0.0, 2.0], [7.0, -4.0, -5.0]]], [[[7.0, 1.0, -1.0], [7.0, 5.0, -5.0]], [[-4.0, 1.0, -3.0], [5.0, -4.0, 0.0]], [[-4.0, 6.0, -7.0], [6.0, -2.0, 0.0]], [[1.0, 1.0, -1.0], [-7.0, -4.0, -6.0]], [[-5.0, 4.0, -4.0], [-7.0, 0.0, -1.0]]], [[[-3.0, 7.0, 7.0], [3.0, -1.0, -1.0]], [[4.0, 0.0, -5.0], [-2.0, 7.0, -4.0]], [[2.0, 4.0, 7.0], [2.0, 6.0, 5.0]], [[-1.0, -7.0, -2.0], [-6.0, 0.0, 5.0]], [[3.0, 7.0, 0.0], [-3.0, 0.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, 4.0, 7.0], [3.0, 7.0, 4.0]], [[-1.0, -5.0, -6.0], [4.0, -4.0, -4.0]], [[6.0, 1.0, 0.0], [4.0, 5.0, -2.0]], [[4.0, 7.0, -7.0], [3.0, 5.0, 4.0]], [[6.0, -5.0, 6.0], [-1.0, -5.0, 1.0]]], [[[-4.0, 0.0, 5.0], [4.0, -6.0, 1.0]], [[7.0, 2.0, 0.0], [-4.0, -7.0, 7.0]], [[1.0, 0.0, -2.0], [-1.0, -2.0, -7.0]], [[7.0, 0.0, -3.0], [4.0, 2.0, -1.0]], [[-3.0, -4.0, 3.0], [2.0, 0.0, 0.0]]], [[[-4.0, -5.0, -1.0], [-4.0, 3.0, 2.0]], [[0.0, 2.0, 0.0], [-3.0, 2.0, 7.0]], [[-4.0, 2.0, 0.0], [-3.0, -5.0, -2.0]], [[3.0, 5.0, -7.0], [7.0, 7.0, -5.0]], [[6.0, 0.0, 6.0], [5.0, 0.0, 3.0]]], [[[-1.0, 5.0, 4.0], [-7.0, -7.0, 7.0]], [[3.0, 0.0, 0.0], [1.0, -3.0, 4.0]], [[-6.0, -3.0, 4.0], [6.0, 5.0, -1.0]], [[-2.0, 4.0, 0.0], [-6.0, 7.0, -6.0]], [[-5.0, 0.0, 4.0], [-1.0, -1.0, 0.0]]]])
      arg1=numpy.array([[-6.0, -5.0, 6.0], [6.0, 7.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[6.0, -12.0, -21.0, -13.0, 68.0], [26.0, 20.0, 53.0, -47.0, 38.0], [24.0, 3.0, -26.0, -87.0, -56.0], [36.0, -17.0, 64.0, -7.0, -71.0]])+(1.-msk_ref)*numpy.array([[119.0, -9.0, 18.0, -48.0, -16.0], [36.0, -125.0, -38.0, -22.0, 68.0], [40.0, -14.0, -39.0, 6.0, 30.0], [-86.0, -33.0, 146.0, 5.0, 41.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[6.0, -4.0, 3.0, 5.0, -7.0], [3.0, 0.0, 5.0, -2.0, 5.0], [2.0, 1.0, 0.0, -6.0, -7.0], [-2.0, 2.0, -3.0, 3.0, 3.0]])+(1.-msk_arg0)*numpy.array([[6.0, -6.0, -6.0, -1.0, -5.0], [7.0, 0.0, 6.0, 2.0, -2.0], [-2.0, 7.0, -1.0, -1.0, -3.0], [1.0, 0.0, 1.0, 4.0, -2.0]])
      arg1=numpy.array([-5.0, 5.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-30.0, 30.0], [20.0, -20.0], [-15.0, 15.0], [-25.0, 25.0], [35.0, -35.0]], [[-15.0, 15.0], [0.0, 0.0], [-25.0, 25.0], [10.0, -10.0], [-25.0, 25.0]], [[-10.0, 10.0], [-5.0, 5.0], [0.0, 0.0], [30.0, -30.0], [35.0, -35.0]], [[10.0, -10.0], [-10.0, 10.0], [15.0, -15.0], [-15.0, 15.0], [-15.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-30.0, 30.0], [30.0, -30.0], [30.0, -30.0], [5.0, -5.0], [25.0, -25.0]], [[-35.0, 35.0], [0.0, 0.0], [-30.0, 30.0], [-10.0, 10.0], [10.0, -10.0]], [[10.0, -10.0], [-35.0, 35.0], [5.0, -5.0], [5.0, -5.0], [15.0, -15.0]], [[-5.0, 5.0], [0.0, 0.0], [-5.0, 5.0], [-20.0, 20.0], [10.0, -10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, -6.0, 5.0], [5.0, 5.0, -6.0], [-6.0, 2.0, 2.0], [7.0, 4.0, -1.0], [-2.0, -3.0, -5.0]], [[5.0, -4.0, -5.0], [-6.0, -6.0, 3.0], [5.0, 2.0, -7.0], [3.0, 4.0, 0.0], [2.0, -2.0, 0.0]], [[-1.0, 1.0, 3.0], [3.0, -7.0, 2.0], [-5.0, -4.0, 5.0], [2.0, -4.0, 5.0], [0.0, 5.0, 1.0]], [[-5.0, 3.0, 2.0], [0.0, -3.0, -6.0], [7.0, -1.0, 3.0], [7.0, 2.0, -4.0], [4.0, -2.0, 3.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 5.0, 1.0], [-2.0, -4.0, 3.0], [-6.0, 5.0, 5.0], [0.0, -4.0, 0.0], [-7.0, -1.0, 4.0]], [[6.0, 5.0, 3.0], [5.0, -1.0, 2.0], [0.0, 7.0, 1.0], [-3.0, -2.0, 6.0], [0.0, -7.0, 6.0]], [[-4.0, -4.0, 3.0], [-5.0, -1.0, 6.0], [-1.0, -4.0, 4.0], [-3.0, -4.0, -4.0], [1.0, 1.0, 1.0]], [[5.0, -3.0, 7.0], [3.0, -4.0, 1.0], [3.0, 7.0, 5.0], [0.0, -4.0, 5.0], [-7.0, 0.0, -4.0]]])
      arg1=numpy.array([[5.0, 6.0, -1.0], [-4.0, 4.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-66.0, -4.0], [61.0, 0.0], [-20.0, 32.0], [60.0, -12.0], [-23.0, -4.0]], [[6.0, -36.0], [-69.0, 0.0], [44.0, -12.0], [39.0, 4.0], [-2.0, -16.0]], [[-2.0, 8.0], [-29.0, -40.0], [-54.0, 4.0], [-19.0, -24.0], [29.0, 20.0]], [[-9.0, 32.0], [-12.0, -12.0], [26.0, -32.0], [51.0, -20.0], [5.0, -24.0]]])+(1.-msk_ref)*numpy.array([[[39.0, 12.0], [-37.0, -8.0], [-5.0, 44.0], [-24.0, -16.0], [-45.0, 24.0]], [[57.0, -4.0], [17.0, -24.0], [41.0, 28.0], [-33.0, 4.0], [-48.0, -28.0]], [[-47.0, 0.0], [-37.0, 16.0], [-33.0, -12.0], [-35.0, -4.0], [10.0, 0.0]], [[0.0, -32.0], [-10.0, -28.0], [52.0, 16.0], [-29.0, -16.0], [-31.0, 28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, -2.0, 5.0], [-7.0, 1.0, 4.0]], [[-6.0, -4.0, 3.0], [1.0, -5.0, -4.0]], [[0.0, 0.0, -3.0], [-3.0, 5.0, 7.0]], [[-7.0, -2.0, 0.0], [0.0, 4.0, -2.0]], [[4.0, -5.0, -7.0], [-5.0, 3.0, -7.0]]], [[[3.0, -2.0, -3.0], [-2.0, -5.0, -7.0]], [[6.0, -4.0, -4.0], [0.0, -6.0, 0.0]], [[-3.0, 1.0, -7.0], [-4.0, 1.0, 5.0]], [[2.0, 3.0, 6.0], [-5.0, 4.0, 6.0]], [[-5.0, -2.0, -5.0], [4.0, -2.0, 5.0]]], [[[3.0, -3.0, -7.0], [-2.0, 1.0, 2.0]], [[6.0, 3.0, 7.0], [3.0, 5.0, -5.0]], [[0.0, -3.0, -5.0], [-1.0, -7.0, 5.0]], [[6.0, 4.0, 0.0], [1.0, 5.0, 3.0]], [[-2.0, -6.0, 1.0], [5.0, -6.0, 0.0]]], [[[-1.0, 3.0, 2.0], [-6.0, 6.0, 0.0]], [[0.0, 5.0, 6.0], [-6.0, 0.0, 2.0]], [[1.0, 2.0, 3.0], [7.0, -5.0, -1.0]], [[0.0, 2.0, -6.0], [0.0, 4.0, -1.0]], [[6.0, 6.0, -1.0], [3.0, 5.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 0.0, 0.0], [4.0, 2.0, 6.0]], [[-5.0, -3.0, 7.0], [3.0, 6.0, 6.0]], [[5.0, 0.0, 4.0], [0.0, 6.0, -1.0]], [[3.0, -3.0, -7.0], [-5.0, 2.0, -5.0]], [[3.0, 7.0, 3.0], [-3.0, 3.0, 3.0]]], [[[3.0, -4.0, 7.0], [0.0, 7.0, -2.0]], [[0.0, -3.0, 1.0], [-6.0, -4.0, 0.0]], [[4.0, -7.0, -6.0], [6.0, -6.0, -2.0]], [[-6.0, -5.0, -5.0], [-2.0, 1.0, 3.0]], [[0.0, 7.0, 0.0], [-5.0, -2.0, -6.0]]], [[[7.0, 6.0, -1.0], [0.0, -4.0, -3.0]], [[6.0, 0.0, 2.0], [0.0, -7.0, -7.0]], [[0.0, 4.0, 0.0], [-7.0, -2.0, -6.0]], [[1.0, 3.0, 7.0], [4.0, 0.0, -3.0]], [[-7.0, -1.0, 6.0], [6.0, 0.0, -3.0]]], [[[0.0, -7.0, 0.0], [-4.0, 4.0, -2.0]], [[-2.0, -5.0, 6.0], [-1.0, 2.0, 0.0]], [[-6.0, -3.0, -4.0], [-5.0, 0.0, 0.0]], [[-6.0, 0.0, 2.0], [-3.0, 4.0, 7.0]], [[-1.0, 7.0, 4.0], [-4.0, 0.0, 0.0]]]])
      arg1=numpy.array([[[5.0, -1.0, -3.0], [1.0, 1.0, 0.0]], [[0.0, -7.0, -1.0], [-5.0, 5.0, 2.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-29.0, 57.0], [-39.0, -13.0], [11.0, 57.0], [-29.0, 30.0], [44.0, 68.0]], [[19.0, -12.0], [40.0, 2.0], [2.0, 35.0], [-12.0, 30.0], [-6.0, -1.0]], [[38.0, 47.0], [14.0, -28.0], [10.0, 6.0], [32.0, -2.0], [-8.0, -14.0]], [[-14.0, 37.0], [-29.0, -7.0], [-4.0, -79.0], [20.0, 10.0], [35.0, -43.0]]])+(1.-msk_ref)*numpy.array([[[11.0, 2.0], [-34.0, 41.0], [19.0, 24.0], [36.0, 53.0], [-1.0, -16.0]], [[5.0, 52.0], [-10.0, 30.0], [45.0, -9.0], [-11.0, 61.0], [-14.0, -46.0]], [[28.0, -67.0], [17.0, -51.0], [-13.0, -15.0], [-15.0, -54.0], [-46.0, -35.0]], [[7.0, 85.0], [-22.0, 44.0], [-20.0, 50.0], [-35.0, 47.0], [-28.0, -33.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_array_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 5.0, 2.0, -5.0, -5.0], [6.0, -7.0, 6.0, -1.0, 2.0], [3.0, 3.0, 0.0, 0.0, -1.0], [-3.0, -4.0, 5.0, 1.0, 6.0]])+(1.-msk_arg0)*numpy.array([[-1.0, 7.0, 0.0, 0.0, -4.0], [4.0, 2.0, 0.0, 4.0, 2.0], [-7.0, 7.0, -3.0, -7.0, 3.0], [-6.0, -1.0, 1.0, 0.0, 0.0]])
      arg1=numpy.array([[6.0, -1.0, 0.0, -3.0, -4.0], [-7.0, 7.0, -6.0, 0.0, 5.0], [-5.0, -1.0, -7.0, -2.0, 4.0], [-1.0, -7.0, -2.0, 4.0, -7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, -4.0, 0.0, -12.0, -16.0], [-28.0, 28.0, -24.0, 0.0, 20.0], [-20.0, -4.0, -28.0, -8.0, 16.0], [-4.0, -28.0, -8.0, 16.0, -28.0]], [[30.0, -5.0, 0.0, -15.0, -20.0], [-35.0, 35.0, -30.0, 0.0, 25.0], [-25.0, -5.0, -35.0, -10.0, 20.0], [-5.0, -35.0, -10.0, 20.0, -35.0]], [[12.0, -2.0, 0.0, -6.0, -8.0], [-14.0, 14.0, -12.0, 0.0, 10.0], [-10.0, -2.0, -14.0, -4.0, 8.0], [-2.0, -14.0, -4.0, 8.0, -14.0]], [[-30.0, 5.0, 0.0, 15.0, 20.0], [35.0, -35.0, 30.0, 0.0, -25.0], [25.0, 5.0, 35.0, 10.0, -20.0], [5.0, 35.0, 10.0, -20.0, 35.0]], [[-30.0, 5.0, 0.0, 15.0, 20.0], [35.0, -35.0, 30.0, 0.0, -25.0], [25.0, 5.0, 35.0, 10.0, -20.0], [5.0, 35.0, 10.0, -20.0, 35.0]]], [[[36.0, -6.0, 0.0, -18.0, -24.0], [-42.0, 42.0, -36.0, 0.0, 30.0], [-30.0, -6.0, -42.0, -12.0, 24.0], [-6.0, -42.0, -12.0, 24.0, -42.0]], [[-42.0, 7.0, 0.0, 21.0, 28.0], [49.0, -49.0, 42.0, 0.0, -35.0], [35.0, 7.0, 49.0, 14.0, -28.0], [7.0, 49.0, 14.0, -28.0, 49.0]], [[36.0, -6.0, 0.0, -18.0, -24.0], [-42.0, 42.0, -36.0, 0.0, 30.0], [-30.0, -6.0, -42.0, -12.0, 24.0], [-6.0, -42.0, -12.0, 24.0, -42.0]], [[-6.0, 1.0, 0.0, 3.0, 4.0], [7.0, -7.0, 6.0, 0.0, -5.0], [5.0, 1.0, 7.0, 2.0, -4.0], [1.0, 7.0, 2.0, -4.0, 7.0]], [[12.0, -2.0, 0.0, -6.0, -8.0], [-14.0, 14.0, -12.0, 0.0, 10.0], [-10.0, -2.0, -14.0, -4.0, 8.0], [-2.0, -14.0, -4.0, 8.0, -14.0]]], [[[18.0, -3.0, 0.0, -9.0, -12.0], [-21.0, 21.0, -18.0, 0.0, 15.0], [-15.0, -3.0, -21.0, -6.0, 12.0], [-3.0, -21.0, -6.0, 12.0, -21.0]], [[18.0, -3.0, 0.0, -9.0, -12.0], [-21.0, 21.0, -18.0, 0.0, 15.0], [-15.0, -3.0, -21.0, -6.0, 12.0], [-3.0, -21.0, -6.0, 12.0, -21.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 1.0, 0.0, 3.0, 4.0], [7.0, -7.0, 6.0, 0.0, -5.0], [5.0, 1.0, 7.0, 2.0, -4.0], [1.0, 7.0, 2.0, -4.0, 7.0]]], [[[-18.0, 3.0, 0.0, 9.0, 12.0], [21.0, -21.0, 18.0, 0.0, -15.0], [15.0, 3.0, 21.0, 6.0, -12.0], [3.0, 21.0, 6.0, -12.0, 21.0]], [[-24.0, 4.0, 0.0, 12.0, 16.0], [28.0, -28.0, 24.0, 0.0, -20.0], [20.0, 4.0, 28.0, 8.0, -16.0], [4.0, 28.0, 8.0, -16.0, 28.0]], [[30.0, -5.0, 0.0, -15.0, -20.0], [-35.0, 35.0, -30.0, 0.0, 25.0], [-25.0, -5.0, -35.0, -10.0, 20.0], [-5.0, -35.0, -10.0, 20.0, -35.0]], [[6.0, -1.0, 0.0, -3.0, -4.0], [-7.0, 7.0, -6.0, 0.0, 5.0], [-5.0, -1.0, -7.0, -2.0, 4.0], [-1.0, -7.0, -2.0, 4.0, -7.0]], [[36.0, -6.0, 0.0, -18.0, -24.0], [-42.0, 42.0, -36.0, 0.0, 30.0], [-30.0, -6.0, -42.0, -12.0, 24.0], [-6.0, -42.0, -12.0, 24.0, -42.0]]]])+(1.-msk_ref)*numpy.array([[[[-6.0, 1.0, 0.0, 3.0, 4.0], [7.0, -7.0, 6.0, 0.0, -5.0], [5.0, 1.0, 7.0, 2.0, -4.0], [1.0, 7.0, 2.0, -4.0, 7.0]], [[42.0, -7.0, 0.0, -21.0, -28.0], [-49.0, 49.0, -42.0, 0.0, 35.0], [-35.0, -7.0, -49.0, -14.0, 28.0], [-7.0, -49.0, -14.0, 28.0, -49.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-24.0, 4.0, 0.0, 12.0, 16.0], [28.0, -28.0, 24.0, 0.0, -20.0], [20.0, 4.0, 28.0, 8.0, -16.0], [4.0, 28.0, 8.0, -16.0, 28.0]]], [[[24.0, -4.0, 0.0, -12.0, -16.0], [-28.0, 28.0, -24.0, 0.0, 20.0], [-20.0, -4.0, -28.0, -8.0, 16.0], [-4.0, -28.0, -8.0, 16.0, -28.0]], [[12.0, -2.0, 0.0, -6.0, -8.0], [-14.0, 14.0, -12.0, 0.0, 10.0], [-10.0, -2.0, -14.0, -4.0, 8.0], [-2.0, -14.0, -4.0, 8.0, -14.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[24.0, -4.0, 0.0, -12.0, -16.0], [-28.0, 28.0, -24.0, 0.0, 20.0], [-20.0, -4.0, -28.0, -8.0, 16.0], [-4.0, -28.0, -8.0, 16.0, -28.0]], [[12.0, -2.0, 0.0, -6.0, -8.0], [-14.0, 14.0, -12.0, 0.0, 10.0], [-10.0, -2.0, -14.0, -4.0, 8.0], [-2.0, -14.0, -4.0, 8.0, -14.0]]], [[[-42.0, 7.0, 0.0, 21.0, 28.0], [49.0, -49.0, 42.0, 0.0, -35.0], [35.0, 7.0, 49.0, 14.0, -28.0], [7.0, 49.0, 14.0, -28.0, 49.0]], [[42.0, -7.0, 0.0, -21.0, -28.0], [-49.0, 49.0, -42.0, 0.0, 35.0], [-35.0, -7.0, -49.0, -14.0, 28.0], [-7.0, -49.0, -14.0, 28.0, -49.0]], [[-18.0, 3.0, 0.0, 9.0, 12.0], [21.0, -21.0, 18.0, 0.0, -15.0], [15.0, 3.0, 21.0, 6.0, -12.0], [3.0, 21.0, 6.0, -12.0, 21.0]], [[-42.0, 7.0, 0.0, 21.0, 28.0], [49.0, -49.0, 42.0, 0.0, -35.0], [35.0, 7.0, 49.0, 14.0, -28.0], [7.0, 49.0, 14.0, -28.0, 49.0]], [[18.0, -3.0, 0.0, -9.0, -12.0], [-21.0, 21.0, -18.0, 0.0, 15.0], [-15.0, -3.0, -21.0, -6.0, 12.0], [-3.0, -21.0, -6.0, 12.0, -21.0]]], [[[-36.0, 6.0, 0.0, 18.0, 24.0], [42.0, -42.0, 36.0, 0.0, -30.0], [30.0, 6.0, 42.0, 12.0, -24.0], [6.0, 42.0, 12.0, -24.0, 42.0]], [[-6.0, 1.0, 0.0, 3.0, 4.0], [7.0, -7.0, 6.0, 0.0, -5.0], [5.0, 1.0, 7.0, 2.0, -4.0], [1.0, 7.0, 2.0, -4.0, 7.0]], [[6.0, -1.0, 0.0, -3.0, -4.0], [-7.0, 7.0, -6.0, 0.0, 5.0], [-5.0, -1.0, -7.0, -2.0, 4.0], [-1.0, -7.0, -2.0, 4.0, -7.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 3.0, 2.0], [1.0, 0.0, 0.0], [-1.0, 3.0, -2.0], [1.0, -1.0, -6.0], [-3.0, -2.0, -1.0]], [[0.0, -7.0, 3.0], [5.0, 3.0, 4.0], [2.0, -1.0, -2.0], [-4.0, -5.0, 0.0], [0.0, 2.0, -3.0]], [[-7.0, 6.0, -2.0], [-5.0, 3.0, -2.0], [-2.0, 5.0, -4.0], [1.0, -3.0, 4.0], [1.0, 3.0, 0.0]], [[-3.0, 7.0, 5.0], [-1.0, 4.0, 4.0], [5.0, -7.0, 0.0], [5.0, 4.0, 0.0], [-4.0, 0.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, 6.0, 7.0], [-4.0, 4.0, 1.0], [5.0, -6.0, 0.0], [-5.0, 0.0, 3.0], [-3.0, -2.0, 4.0]], [[0.0, -2.0, 6.0], [7.0, 3.0, -5.0], [5.0, 0.0, 6.0], [-1.0, -3.0, -1.0], [-7.0, 1.0, 4.0]], [[-6.0, -5.0, 5.0], [-1.0, -5.0, -3.0], [3.0, 4.0, 2.0], [-1.0, 3.0, -4.0], [0.0, -6.0, -4.0]], [[2.0, 1.0, 7.0], [0.0, 5.0, -4.0], [-7.0, 3.0, -6.0], [-4.0, 3.0, -3.0], [-4.0, -5.0, 0.0]]])
      arg1=numpy.array([[[1.0, -7.0, -7.0], [6.0, 6.0, 1.0], [-6.0, -4.0, -4.0], [6.0, -3.0, -3.0], [7.0, 4.0, 0.0]], [[0.0, -4.0, 7.0], [4.0, -5.0, -2.0], [6.0, 2.0, -7.0], [-6.0, -5.0, -2.0], [-5.0, 0.0, -7.0]], [[0.0, -1.0, 7.0], [4.0, 3.0, 6.0], [-6.0, 1.0, 0.0], [6.0, 0.0, -1.0], [2.0, -4.0, 2.0]], [[4.0, 2.0, 4.0], [-1.0, 2.0, 6.0], [1.0, 7.0, 2.0], [-2.0, 1.0, -6.0], [7.0, -6.0, -5.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-35.0, 20.0, -20.0, -15.0, 12.0], [2.0, -19.0, -8.0, -19.0, -14.0], [11.0, 21.0, 3.0, -2.0, -8.0], [14.0, 18.0, 25.0, -9.0, -28.0]], [[1.0, 6.0, -6.0, 6.0, 7.0], [0.0, 4.0, 6.0, -6.0, -5.0], [0.0, 4.0, -6.0, 6.0, 2.0], [4.0, -1.0, 1.0, -2.0, 7.0]], [[-8.0, 10.0, 2.0, -9.0, 5.0], [-26.0, -15.0, 14.0, -5.0, 19.0], [-17.0, -7.0, 9.0, -4.0, -18.0], [-6.0, -5.0, 16.0, 17.0, -15.0]], [[50.0, -6.0, 22.0, 27.0, 3.0], [-38.0, 21.0, 46.0, 11.0, 37.0], [-41.0, -35.0, -7.0, 12.0, -6.0], [-22.0, -39.0, -18.0, 33.0, 43.0]], [[18.0, -31.0, 30.0, -9.0, -29.0], [1.0, 0.0, -15.0, 30.0, 22.0], [-5.0, -24.0, 16.0, -17.0, 0.0], [-20.0, -7.0, -19.0, 10.0, -4.0]]], [[[28.0, -39.0, 16.0, 12.0, -28.0], [49.0, 29.0, -35.0, 29.0, -21.0], [28.0, -3.0, -7.0, -3.0, 34.0], [-2.0, 4.0, -43.0, -25.0, 27.0]], [[-44.0, 52.0, -58.0, 9.0, 47.0], [16.0, -3.0, 8.0, -53.0, -53.0], [25.0, 53.0, -27.0, 26.0, 6.0], [42.0, 25.0, 34.0, -31.0, -3.0]], [[23.0, 4.0, 0.0, 21.0, 10.0], [-10.0, 17.0, 24.0, -3.0, 4.0], [-13.0, -7.0, -13.0, 14.0, 4.0], [-2.0, -16.0, -9.0, 7.0, 30.0]], [[31.0, -54.0, 44.0, -9.0, -48.0], [20.0, 9.0, -34.0, 49.0, 20.0], [5.0, -31.0, 19.0, -24.0, 12.0], [-26.0, -6.0, -39.0, 3.0, 2.0]], [[7.0, 9.0, 4.0, 3.0, 8.0], [-29.0, -4.0, 25.0, -4.0, 21.0], [-23.0, -12.0, 2.0, 3.0, -14.0], [-8.0, -14.0, 8.0, 20.0, 3.0]]], [[[-35.0, -8.0, 26.0, -54.0, -25.0], [-38.0, -54.0, -16.0, 16.0, 49.0], [-20.0, -22.0, 48.0, -40.0, -42.0], [-24.0, 7.0, 31.0, 32.0, -75.0]], [[-12.0, -14.0, 26.0, -33.0, -23.0], [-26.0, -31.0, -10.0, 19.0, 39.0], [-17.0, -23.0, 33.0, -28.0, -26.0], [-22.0, -1.0, 12.0, 25.0, -43.0]], [[-9.0, 14.0, 8.0, -15.0, 6.0], [-48.0, -25.0, 26.0, -5.0, 38.0], [-33.0, -17.0, 17.0, -8.0, -32.0], [-14.0, -12.0, 25.0, 33.0, -24.0]], [[-6.0, -8.0, -10.0, 3.0, -5.0], [40.0, 11.0, -28.0, 1.0, -33.0], [31.0, 19.0, -9.0, 2.0, 22.0], [14.0, 17.0, -12.0, -29.0, 5.0]], [[-20.0, 24.0, -18.0, -3.0, 19.0], [-12.0, -11.0, 12.0, -21.0, -5.0], [-3.0, 13.0, -3.0, 6.0, -10.0], [10.0, 5.0, 22.0, 1.0, -11.0]]], [[[-87.0, 29.0, -30.0, -54.0, 7.0], [7.0, -57.0, -39.0, -27.0, -20.0], [28.0, 39.0, 25.0, -23.0, -24.0], [22.0, 47.0, 56.0, -17.0, -88.0]], [[-57.0, 22.0, -26.0, -30.0, 9.0], [12.0, -32.0, -26.0, -22.0, -23.0], [24.0, 32.0, 10.0, -10.0, -10.0], [20.0, 33.0, 35.0, -18.0, -51.0]], [[54.0, -12.0, -2.0, 51.0, 7.0], [28.0, 55.0, 16.0, 5.0, -25.0], [7.0, -1.0, -37.0, 30.0, 38.0], [6.0, -19.0, -44.0, -17.0, 77.0]], [[-23.0, 54.0, -46.0, 18.0, 51.0], [-16.0, 0.0, 38.0, -50.0, -25.0], [-4.0, 32.0, -26.0, 30.0, -6.0], [28.0, 3.0, 33.0, -6.0, 11.0]], [[-39.0, -19.0, 4.0, -39.0, -28.0], [35.0, -26.0, -59.0, 14.0, -15.0], [35.0, 14.0, 24.0, -29.0, 2.0], [4.0, 34.0, 6.0, -22.0, -53.0]]]])+(1.-msk_ref)*numpy.array([[[[-94.0, 25.0, -34.0, -57.0, 3.0], [25.0, -56.0, -55.0, -26.0, -34.0], [43.0, 48.0, 24.0, -25.0, -16.0], [28.0, 57.0, 53.0, -30.0, -92.0]], [[-39.0, 1.0, 4.0, -39.0, -12.0], [-9.0, -38.0, -23.0, 2.0, 13.0], [3.0, 2.0, 28.0, -25.0, -22.0], [-4.0, 18.0, 26.0, 6.0, -57.0]], [[47.0, -6.0, -6.0, 48.0, 11.0], [24.0, 50.0, 18.0, 0.0, -25.0], [6.0, 2.0, -36.0, 30.0, 34.0], [8.0, -17.0, -37.0, -16.0, 71.0]], [[-26.0, -27.0, 18.0, -39.0, -35.0], [21.0, -26.0, -51.0, 24.0, 4.0], [21.0, -2.0, 30.0, -33.0, -4.0], [-8.0, 23.0, 1.0, -8.0, -50.0]], [[-17.0, -26.0, 10.0, -24.0, -29.0], [36.0, -10.0, -50.0, 20.0, -13.0], [30.0, 6.0, 16.0, -22.0, 10.0], [0.0, 23.0, -9.0, -20.0, -29.0]]], [[[-28.0, -6.0, -16.0, -12.0, -8.0], [50.0, -2.0, -46.0, -2.0, -42.0], [44.0, 30.0, -2.0, -6.0, 20.0], [20.0, 32.0, -2.0, -38.0, -18.0]], [[21.0, 55.0, -34.0, 48.0, 61.0], [-47.0, 23.0, 83.0, -47.0, 0.0], [-38.0, 7.0, -39.0, 47.0, -8.0], [14.0, -31.0, 18.0, 19.0, 56.0]], [[-37.0, 36.0, -54.0, 12.0, 35.0], [42.0, 8.0, -12.0, -42.0, -67.0], [42.0, 56.0, -30.0, 24.0, 22.0], [44.0, 31.0, 17.0, -46.0, 5.0]], [[27.0, -25.0, 22.0, 6.0, -19.0], [5.0, 13.0, -5.0, 23.0, 12.0], [-4.0, -19.0, 3.0, -5.0, 8.0], [-14.0, -11.0, -24.0, 5.0, 16.0]], [[-42.0, -32.0, 22.0, -57.0, -45.0], [24.0, -41.0, -68.0, 29.0, 7.0], [27.0, -1.0, 43.0, -46.0, -10.0], [-10.0, 33.0, 8.0, -9.0, -75.0]]], [[[-6.0, -61.0, 36.0, -36.0, -62.0], [55.0, -9.0, -81.0, 51.0, -5.0], [40.0, -9.0, 31.0, -41.0, 18.0], [-14.0, 26.0, -31.0, -23.0, -37.0]], [[55.0, -39.0, 38.0, 18.0, -27.0], [-1.0, 27.0, 5.0, 37.0, 26.0], [-16.0, -37.0, 1.0, -3.0, 12.0], [-26.0, -27.0, -42.0, 15.0, 38.0]], [[-39.0, 44.0, -42.0, 0.0, 37.0], [-2.0, -12.0, 12.0, -42.0, -29.0], [10.0, 36.0, -14.0, 16.0, -6.0], [28.0, 17.0, 35.0, -14.0, -13.0]], [[6.0, 8.0, 10.0, -3.0, 5.0], [-40.0, -11.0, 28.0, -1.0, 33.0], [-31.0, -19.0, 9.0, -2.0, -22.0], [-14.0, -17.0, 12.0, 29.0, -5.0]], [[70.0, -40.0, 40.0, 30.0, -24.0], [-4.0, 38.0, 16.0, 38.0, 28.0], [-22.0, -42.0, -6.0, 4.0, 16.0], [-28.0, -36.0, -50.0, 18.0, 56.0]]], [[[-54.0, 25.0, -44.0, -12.0, 18.0], [45.0, -11.0, -35.0, -31.0, -59.0], [48.0, 53.0, -11.0, 5.0, 14.0], [38.0, 42.0, 23.0, -45.0, -27.0]], [[-7.0, 26.0, -4.0, -3.0, 20.0], [-48.0, -17.0, 38.0, -17.0, 28.0], [-33.0, -9.0, 5.0, 4.0, -28.0], [-6.0, -14.0, 27.0, 29.0, -10.0]], [[14.0, -30.0, 54.0, -33.0, -37.0], [-54.0, -31.0, 6.0, 39.0, 77.0], [-45.0, -55.0, 45.0, -36.0, -38.0], [-46.0, -23.0, 2.0, 53.0, -37.0]], [[-4.0, -9.0, 24.0, -24.0, -16.0], [-33.0, -25.0, 3.0, 15.0, 41.0], [-24.0, -25.0, 27.0, -21.0, -26.0], [-22.0, -8.0, 11.0, 29.0, -31.0]], [[31.0, -54.0, 44.0, -9.0, -48.0], [20.0, 9.0, -34.0, 49.0, 20.0], [5.0, -31.0, 19.0, -24.0, 12.0], [-26.0, -6.0, -39.0, 3.0, 2.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 4.0, 0.0], [-4.0, -5.0, 7.0]], [[-3.0, 3.0, -6.0], [1.0, 5.0, -4.0]], [[2.0, -5.0, -6.0], [6.0, 4.0, -5.0]], [[7.0, -5.0, -5.0], [-5.0, -5.0, -6.0]], [[-2.0, -4.0, 7.0], [7.0, 3.0, -5.0]]], [[[-6.0, 2.0, 3.0], [3.0, 7.0, 1.0]], [[-7.0, -2.0, 5.0], [5.0, -1.0, 7.0]], [[2.0, 6.0, 1.0], [1.0, -1.0, 6.0]], [[-4.0, -1.0, 6.0], [0.0, 1.0, -3.0]], [[-4.0, -6.0, -4.0], [-1.0, 0.0, -6.0]]], [[[-2.0, 7.0, -5.0], [3.0, 4.0, -2.0]], [[2.0, -7.0, -6.0], [-1.0, -7.0, 0.0]], [[-3.0, -7.0, -7.0], [-2.0, -1.0, -2.0]], [[0.0, 3.0, -7.0], [3.0, 2.0, 1.0]], [[3.0, 0.0, -1.0], [-3.0, 5.0, -5.0]]], [[[0.0, -5.0, 0.0], [2.0, 6.0, -3.0]], [[7.0, 1.0, -4.0], [-4.0, 7.0, -3.0]], [[-3.0, 7.0, -4.0], [7.0, -3.0, 0.0]], [[-3.0, -6.0, -4.0], [2.0, 7.0, -7.0]], [[1.0, 1.0, 1.0], [-5.0, -1.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[3.0, 5.0, 0.0], [6.0, 0.0, 2.0]], [[2.0, 5.0, 4.0], [-1.0, 3.0, 6.0]], [[-1.0, -7.0, 3.0], [7.0, 3.0, 6.0]], [[6.0, -2.0, 0.0], [7.0, 2.0, 6.0]], [[-7.0, -3.0, -2.0], [-7.0, -4.0, -5.0]]], [[[-5.0, -6.0, 1.0], [-6.0, -5.0, 6.0]], [[-6.0, 2.0, -6.0], [0.0, 1.0, -1.0]], [[-3.0, -6.0, 1.0], [-1.0, 5.0, -2.0]], [[4.0, 1.0, 0.0], [-1.0, -2.0, 0.0]], [[5.0, -4.0, 1.0], [6.0, 3.0, -3.0]]], [[[7.0, -5.0, 5.0], [0.0, 2.0, -7.0]], [[2.0, -5.0, -5.0], [1.0, -2.0, -1.0]], [[3.0, 0.0, 7.0], [0.0, -1.0, 0.0]], [[6.0, -2.0, -3.0], [-3.0, 6.0, -2.0]], [[0.0, 1.0, -1.0], [3.0, 3.0, 2.0]]], [[[-5.0, -1.0, 3.0], [-7.0, 7.0, -1.0]], [[-2.0, 5.0, -7.0], [7.0, -7.0, -1.0]], [[-2.0, 3.0, -6.0], [-2.0, -1.0, 1.0]], [[-5.0, 7.0, -5.0], [-2.0, 5.0, 4.0]], [[0.0, -6.0, -2.0], [7.0, 5.0, -6.0]]]])
      arg1=numpy.array([[[[2.0, -2.0, 3.0], [-1.0, 2.0, 7.0]], [[7.0, 6.0, 5.0], [1.0, -6.0, 0.0]], [[-1.0, 2.0, -2.0], [-1.0, 5.0, -5.0]], [[-4.0, 4.0, -7.0], [1.0, -1.0, -2.0]], [[0.0, -1.0, -3.0], [-3.0, 2.0, -6.0]]], [[[-4.0, -3.0, -3.0], [-3.0, -1.0, 0.0]], [[-7.0, -7.0, 0.0], [5.0, 2.0, 4.0]], [[-2.0, 1.0, -2.0], [-4.0, 0.0, -6.0]], [[-3.0, -1.0, 3.0], [7.0, 7.0, 7.0]], [[-6.0, 0.0, -5.0], [-7.0, 0.0, -4.0]]], [[[6.0, 2.0, -4.0], [-1.0, -3.0, 4.0]], [[0.0, 0.0, 0.0], [-6.0, -2.0, -1.0]], [[-1.0, 5.0, 1.0], [-4.0, -2.0, 1.0]], [[5.0, -7.0, 3.0], [1.0, -5.0, -7.0]], [[6.0, 2.0, -6.0], [1.0, 3.0, -7.0]]], [[[7.0, 1.0, 1.0], [7.0, 0.0, 2.0]], [[-7.0, -3.0, -5.0], [6.0, -5.0, -2.0]], [[0.0, 0.0, -1.0], [-1.0, 4.0, -3.0]], [[2.0, 7.0, 0.0], [-1.0, 0.0, 0.0]], [[-7.0, 2.0, 1.0], [0.0, -7.0, -3.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[21.0, 1.0, -41.0, 31.0, -44.0], [33.0, 19.0, -8.0, 3.0, 42.0], [13.0, 27.0, 60.0, -91.0, -102.0], [-59.0, 24.0, -37.0, 18.0, 71.0]], [[-49.0, -62.0, 65.0, 70.0, 46.0], [13.0, -1.0, 41.0, 2.0, 57.0], [-20.0, -12.0, -6.0, -50.0, 68.0], [-25.0, 31.0, 37.0, 14.0, -2.0]], [[-37.0, -64.0, 39.0, 26.0, 43.0], [3.0, 39.0, 9.0, 16.0, -4.0], [-12.0, -39.0, -70.0, 48.0, 91.0], [35.0, 57.0, 31.0, -37.0, -43.0]], [[-38.0, 19.0, 3.0, -1.0, 61.0], [22.0, -73.0, 47.0, -143.0, 42.0], [48.0, 46.0, -13.0, 117.0, 84.0], [-8.0, -2.0, 8.0, -16.0, -11.0]], [[-11.0, -14.0, 13.0, -43.0, -2.0], [-25.0, 63.0, -12.0, 66.0, -52.0], [-84.0, -43.0, -50.0, 66.0, -11.0], [28.0, 28.0, 13.0, -39.0, 7.0]]], [[[11.0, -54.0, 31.0, 5.0, -12.0], [-7.0, 61.0, -10.0, 102.0, -4.0], [-64.0, -33.0, -6.0, -74.0, -33.0], [-14.0, 2.0, 19.0, -1.0, -3.0]], [[47.0, -25.0, -52.0, -23.0, -72.0], [5.0, 114.0, -60.0, 115.0, -46.0], [-40.0, -35.0, -9.0, -45.0, -123.0], [3.0, 51.0, -35.0, -33.0, 36.0]], [[34.0, 62.0, -28.0, -1.0, -50.0], [-31.0, -29.0, -40.0, 33.0, -48.0], [46.0, -10.0, 33.0, -65.0, -26.0], [40.0, -38.0, -24.0, 45.0, -12.0]], [[-7.0, -10.0, 10.0, -25.0, 3.0], [0.0, 25.0, 13.0, 17.0, 6.0], [-65.0, 1.0, 0.0, 21.0, -38.0], [-29.0, 2.0, 7.0, -15.0, 34.0]], [[-49.0, -85.0, 31.0, 31.0, 57.0], [49.0, 41.0, 50.0, -43.0, 75.0], [-43.0, 12.0, -32.0, 51.0, 29.0], [-57.0, 72.0, 23.0, -49.0, 30.0]]], [[[-42.0, -18.0, 53.0, 74.0, 19.0], [-11.0, -20.0, 21.0, 19.0, 24.0], [-1.0, -24.0, 10.0, -77.0, 61.0], [5.0, 20.0, 24.0, 42.0, 1.0]], [[-13.0, -17.0, -38.0, 12.0, 14.0], [41.0, 16.0, 5.0, -73.0, 25.0], [44.0, 20.0, -25.0, 75.0, 12.0], [-6.0, 66.0, -21.0, -44.0, 15.0]], [[-27.0, -94.0, 10.0, 36.0, 44.0], [61.0, 50.0, 33.0, -40.0, 75.0], [-7.0, 16.0, -31.0, 30.0, 19.0], [-53.0, 74.0, 11.0, -53.0, 13.0]], [[-19.0, -26.0, 22.0, 60.0, 7.0], [1.0, 2.0, -1.0, 18.0, 10.0], [29.0, -23.0, -7.0, -56.0, 50.0], [19.0, 32.0, 9.0, 18.0, -18.0]], [[-19.0, -17.0, 52.0, -3.0, 52.0], [-5.0, -46.0, 38.0, -33.0, 28.0], [-10.0, 13.0, -7.0, 19.0, 71.0], [-11.0, -49.0, 39.0, 9.0, -42.0]]], [[[-1.0, -64.0, 33.0, -18.0, 29.0], [3.0, 45.0, 5.0, 40.0, -2.0], [-42.0, -21.0, -48.0, 28.0, 31.0], [3.0, 3.0, 31.0, -37.0, -43.0]], [[-3.0, -11.0, 57.0, -1.0, 55.0], [-14.0, -74.0, 29.0, -34.0, 18.0], [31.0, 13.0, -7.0, -2.0, 106.0], [12.0, -85.0, 45.0, 25.0, -91.0]], [[-45.0, 26.0, 3.0, 78.0, -22.0], [-15.0, 1.0, -7.0, 18.0, -11.0], [14.0, -36.0, 12.0, -54.0, 18.0], [31.0, 77.0, -15.0, 36.0, 52.0]], [[-43.0, -117.0, 67.0, 25.0, 68.0], [29.0, 59.0, 42.0, 17.0, 52.0], [-65.0, -19.0, -60.0, 31.0, 66.0], [-31.0, 50.0, 51.0, -50.0, -23.0]], [[41.0, 19.0, -26.0, -21.0, -21.0], [6.0, -21.0, -13.0, -8.0, 4.0], [32.0, 27.0, 32.0, -34.0, -41.0], [-16.0, -50.0, -15.0, 14.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[4.0, 57.0, -9.0, 10.0, -35.0], [-45.0, -18.0, -37.0, 42.0, -68.0], [30.0, -38.0, 0.0, -28.0, 20.0], [72.0, -4.0, -12.0, 35.0, -17.0]], [[55.0, 45.0, -14.0, -32.0, -44.0], [-35.0, -24.0, -39.0, 57.0, -49.0], [22.0, -6.0, 31.0, -71.0, -36.0], [28.0, -82.0, -9.0, 40.0, -39.0]], [[62.0, -45.0, -41.0, -53.0, -53.0], [-8.0, 121.0, -75.0, 131.0, -82.0], [-24.0, -54.0, -59.0, 3.0, -64.0], [50.0, 28.0, -16.0, -58.0, -43.0]], [[55.0, 25.0, -37.0, -39.0, -51.0], [-41.0, 35.0, -78.0, 89.0, -109.0], [43.0, -52.0, -42.0, -1.0, 3.0], [101.0, -16.0, -17.0, -9.0, -78.0]], [[-50.0, -60.0, 17.0, 37.0, 52.0], [68.0, 7.0, 73.0, -94.0, 121.0], [-41.0, 55.0, 21.0, 28.0, -20.0], [-113.0, 56.0, 8.0, -28.0, 84.0]]], [[[43.0, -42.0, -58.0, -24.0, -25.0], [58.0, 61.0, -10.0, -11.0, 43.0], [-1.0, 40.0, 16.0, -3.0, -111.0], [-70.0, 25.0, -33.0, -46.0, 41.0]], [[-39.0, -66.0, 32.0, 75.0, 24.0], [35.0, 26.0, 32.0, -2.0, 70.0], [-15.0, -1.0, 7.0, -60.0, 14.0], [-48.0, 63.0, 13.0, 2.0, 36.0]], [[6.0, -83.0, 25.0, -21.0, 28.0], [25.0, 60.0, 14.0, 32.0, 28.0], [-56.0, -2.0, -34.0, 18.0, -8.0], [-37.0, 7.0, 26.0, -47.0, -19.0]], [[3.0, 45.0, -11.0, -11.0, -2.0], [-14.0, -44.0, -3.0, -34.0, -17.0], [33.0, 10.0, 9.0, 22.0, 19.0], [22.0, -27.0, -7.0, 16.0, -12.0]], [[0.0, 4.0, 9.0, -34.0, 7.0], [-32.0, 17.0, -22.0, 34.0, -65.0], [-9.0, -39.0, -57.0, 68.0, 52.0], [68.0, -1.0, 14.0, -24.0, -54.0]]], [[[-6.0, 32.0, 18.0, -71.0, 36.0], [-30.0, -38.0, 13.0, -36.0, -39.0], [-22.0, 3.0, -38.0, 124.0, 57.0], [35.0, -55.0, 24.0, -21.0, -47.0]], [[-13.0, -28.0, -8.0, 12.0, 19.0], [21.0, 18.0, 3.0, -30.0, 10.0], [23.0, -1.0, -33.0, 48.0, 34.0], [9.0, 44.0, -1.0, -32.0, -12.0]], [[25.0, 62.0, -22.0, -60.0, -23.0], [-32.0, -23.0, -20.0, 5.0, -53.0], [-7.0, 2.0, 6.0, 41.0, -27.0], [28.0, -51.0, -11.0, 6.0, -7.0]], [[8.0, -24.0, 39.0, -16.0, 44.0], [-6.0, -39.0, 16.0, -18.0, 8.0], [21.0, 8.0, -21.0, 16.0, 79.0], [12.0, -65.0, 36.0, 1.0, -85.0]], [[12.0, -14.0, 6.0, 7.0, -13.0], [-12.0, 22.0, -21.0, 52.0, -24.0], [2.0, -26.0, -12.0, -36.0, 6.0], [25.0, 1.0, 4.0, 4.0, -26.0]]], [[[15.0, -75.0, 44.0, -17.0, 33.0], [28.0, 17.0, 37.0, 18.0, 68.0], [-62.0, 29.0, 16.0, -44.0, -29.0], [-84.0, -52.0, 35.0, -10.0, -10.0]], [[-63.0, 30.0, -11.0, 93.0, -13.0], [0.0, -4.0, 1.0, -27.0, 2.0], [36.0, -27.0, 5.0, -17.0, 33.0], [31.0, 113.0, -25.0, 24.0, 69.0]], [[-21.0, -22.0, 12.0, 59.0, 13.0], [24.0, -15.0, 21.0, -29.0, 52.0], [27.0, 13.0, 22.0, -53.0, 18.0], [-29.0, 26.0, 1.0, 19.0, 18.0]], [[1.0, -50.0, 36.0, 68.0, 0.0], [15.0, 2.0, 11.0, 42.0, 53.0], [7.0, -2.0, 37.0, -144.0, -1.0], [-39.0, -6.0, 15.0, 41.0, -3.0]], [[-33.0, -69.0, 40.0, 4.0, 37.0], [-2.0, 63.0, 6.0, 42.0, -15.0], [-50.0, -46.0, -76.0, 60.0, 64.0], [29.0, 57.0, 33.0, -49.0, -31.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-6.0, -4.0, 6.0, -1.0, 6.0], [-3.0, -2.0, 7.0, -1.0, 0.0], [2.0, 4.0, 0.0, 2.0, 0.0], [1.0, 2.0, 6.0, 0.0, 3.0]])+(1.-msk_arg0)*numpy.array([[0.0, 1.0, 7.0, 1.0, -3.0], [7.0, 4.0, 1.0, -4.0, 0.0], [3.0, 3.0, 4.0, 5.0, 5.0], [2.0, 6.0, -3.0, -1.0, -5.0]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(-3.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[18.0, 12.0, -18.0, 3.0, -18.0], [9.0, 6.0, -21.0, 3.0, -0.0], [-6.0, -12.0, -0.0, -6.0, -0.0], [-3.0, -6.0, -18.0, -0.0, -9.0]])+(1.-msk_ref)*numpy.array([[-0.0, -3.0, -21.0, -3.0, 9.0], [-21.0, -12.0, -3.0, 12.0, -0.0], [-9.0, -9.0, -12.0, -15.0, -15.0], [-6.0, -18.0, 9.0, 3.0, 15.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -4.0, 4.0], [-4.0, -7.0, -3.0], [0.0, 7.0, 4.0], [0.0, -2.0, 0.0], [-6.0, -2.0, 5.0]], [[-2.0, 5.0, -7.0], [4.0, 0.0, 5.0], [-4.0, 1.0, 4.0], [-2.0, 1.0, -5.0], [6.0, 0.0, 5.0]], [[-3.0, 0.0, -4.0], [-2.0, -3.0, 5.0], [-1.0, 3.0, 2.0], [7.0, -2.0, 4.0], [0.0, -1.0, -3.0]], [[1.0, -2.0, 0.0], [-1.0, 0.0, -1.0], [0.0, -5.0, -5.0], [-1.0, 4.0, 7.0], [-3.0, -5.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -4.0, 7.0], [-7.0, 6.0, 5.0], [2.0, 1.0, -1.0], [0.0, -1.0, 1.0], [-4.0, -3.0, 3.0]], [[-4.0, 5.0, 5.0], [1.0, -6.0, 0.0], [0.0, 2.0, -2.0], [7.0, -5.0, -6.0], [-5.0, 4.0, 7.0]], [[0.0, -6.0, 5.0], [-3.0, -7.0, -5.0], [-1.0, 1.0, 7.0], [0.0, -6.0, 0.0], [-2.0, -1.0, 2.0]], [[-7.0, 0.0, -6.0], [7.0, 0.0, 0.0], [6.0, -2.0, 5.0], [-2.0, 3.0, -6.0], [6.0, 7.0, 0.0]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([1.0, 2.0, 0.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, -18.0, 14.0, -4.0, -10.0], [8.0, 4.0, -2.0, 0.0, 6.0], [-3.0, -8.0, 5.0, 3.0, -2.0], [-3.0, -1.0, -10.0, 7.0, -13.0]])+(1.-msk_ref)*numpy.array([[-8.0, 5.0, 4.0, -2.0, -10.0], [6.0, -11.0, 4.0, -3.0, 3.0], [-12.0, -17.0, 1.0, -12.0, -4.0], [-7.0, 7.0, 2.0, 4.0, 20.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 2.0, 0.0], [0.0, -5.0, 3.0]], [[-1.0, 1.0, 7.0], [3.0, -7.0, -7.0]], [[-4.0, -3.0, 6.0], [0.0, -6.0, 0.0]], [[-7.0, 0.0, 1.0], [-6.0, 7.0, 1.0]], [[-2.0, 3.0, 2.0], [-2.0, -4.0, 0.0]]], [[[-3.0, 0.0, -3.0], [3.0, 2.0, 3.0]], [[4.0, 0.0, 6.0], [1.0, -5.0, 1.0]], [[-3.0, -5.0, -2.0], [1.0, -3.0, 7.0]], [[-3.0, 7.0, -7.0], [2.0, 4.0, 1.0]], [[-1.0, -1.0, 2.0], [1.0, 3.0, 4.0]]], [[[-6.0, -7.0, -4.0], [-2.0, 6.0, -2.0]], [[4.0, -7.0, 2.0], [2.0, -3.0, 0.0]], [[-3.0, 0.0, -6.0], [7.0, 2.0, 2.0]], [[-3.0, 4.0, 2.0], [-4.0, 5.0, 2.0]], [[-3.0, 4.0, -5.0], [-3.0, 0.0, -6.0]]], [[[0.0, 4.0, -4.0], [-2.0, -1.0, -7.0]], [[2.0, 0.0, -6.0], [5.0, 5.0, -6.0]], [[7.0, 6.0, 4.0], [2.0, 5.0, -2.0]], [[4.0, 5.0, -6.0], [-1.0, -7.0, -6.0]], [[-7.0, 3.0, 3.0], [4.0, 4.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, -4.0, -4.0], [-7.0, 3.0, -6.0]], [[0.0, -5.0, -6.0], [-6.0, 0.0, 1.0]], [[-2.0, -1.0, 1.0], [0.0, 2.0, 7.0]], [[0.0, 3.0, 4.0], [4.0, 0.0, 4.0]], [[0.0, -6.0, -6.0], [-3.0, 3.0, 4.0]]], [[[-1.0, -6.0, 1.0], [0.0, -3.0, 5.0]], [[1.0, 0.0, -1.0], [-3.0, 0.0, -3.0]], [[-4.0, 5.0, -5.0], [1.0, 4.0, 7.0]], [[2.0, -3.0, -4.0], [-5.0, 0.0, 0.0]], [[4.0, 0.0, -1.0], [3.0, -6.0, -3.0]]], [[[0.0, 7.0, -2.0], [6.0, -5.0, -2.0]], [[-5.0, 2.0, 6.0], [0.0, -5.0, -4.0]], [[3.0, 5.0, 4.0], [2.0, 1.0, 2.0]], [[-3.0, -1.0, 5.0], [-2.0, -5.0, 0.0]], [[0.0, 0.0, -3.0], [3.0, 5.0, 0.0]]], [[[-4.0, -6.0, 5.0], [4.0, 5.0, 0.0]], [[4.0, 0.0, -5.0], [3.0, 1.0, -6.0]], [[2.0, -3.0, 6.0], [-1.0, 6.0, 0.0]], [[7.0, -4.0, -6.0], [-6.0, 2.0, 4.0]], [[2.0, 6.0, 3.0], [-3.0, 4.0, -4.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[1.0, 4.0, -1.0], [-6.0, 5.0, 1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-10.0, -64.0, -52.0, 64.0, 0.0], [-5.0, -32.0, -35.0, 41.0, 6.0], [10.0, -53.0, -27.0, 62.0, 30.0], [20.0, -3.0, 38.0, -5.0, -2.0]])+(1.-msk_ref)*numpy.array([[39.0, 23.0, 10.0, -12.0, 19.0], [-36.0, 17.0, 42.0, 24.0, -46.0], [-33.0, -32.0, 14.0, -25.0, 10.0], [-32.0, -10.0, 20.0, 47.0, 57.0]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[1.0, -1.0, 0.0, 7.0, -7.0], [-6.0, 3.0, 0.0, -7.0, -6.0], [-3.0, 0.0, 7.0, -1.0, 2.0], [1.0, 0.0, -7.0, -3.0, 5.0]])+(1.-msk_arg0)*numpy.array([[-7.0, -5.0, -1.0, 0.0, -4.0], [3.0, 4.0, 3.0, 7.0, 0.0], [2.0, 2.0, 6.0, 3.0, 5.0], [0.0, -3.0, -6.0, -6.0, 4.0]])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([2.0, 4.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[2.0, 4.0], [-2.0, -4.0], [0.0, 0.0], [14.0, 28.0], [-14.0, -28.0]], [[-12.0, -24.0], [6.0, 12.0], [0.0, 0.0], [-14.0, -28.0], [-12.0, -24.0]], [[-6.0, -12.0], [0.0, 0.0], [14.0, 28.0], [-2.0, -4.0], [4.0, 8.0]], [[2.0, 4.0], [0.0, 0.0], [-14.0, -28.0], [-6.0, -12.0], [10.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-14.0, -28.0], [-10.0, -20.0], [-2.0, -4.0], [0.0, 0.0], [-8.0, -16.0]], [[6.0, 12.0], [8.0, 16.0], [6.0, 12.0], [14.0, 28.0], [0.0, 0.0]], [[4.0, 8.0], [4.0, 8.0], [12.0, 24.0], [6.0, 12.0], [10.0, 20.0]], [[0.0, 0.0], [-6.0, -12.0], [-12.0, -24.0], [-12.0, -24.0], [8.0, 16.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 0.0, -5.0], [-6.0, 2.0, 0.0], [7.0, 0.0, -7.0], [-6.0, 0.0, 4.0], [-2.0, -5.0, -6.0]], [[2.0, 6.0, 4.0], [7.0, -2.0, 1.0], [5.0, 2.0, -2.0], [2.0, 1.0, 7.0], [-3.0, 6.0, -2.0]], [[-5.0, 3.0, 0.0], [-1.0, 5.0, -7.0], [2.0, 1.0, 7.0], [-2.0, 3.0, 1.0], [-7.0, -3.0, 2.0]], [[4.0, -5.0, 3.0], [5.0, -1.0, -6.0], [-5.0, 2.0, -5.0], [0.0, -6.0, -4.0], [-4.0, 5.0, -6.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 4.0, -3.0], [3.0, -3.0, 6.0], [3.0, 4.0, 3.0], [3.0, 6.0, -5.0], [-1.0, 0.0, -2.0]], [[3.0, 4.0, -7.0], [0.0, -5.0, 7.0], [-3.0, -2.0, -5.0], [1.0, 5.0, 3.0], [3.0, -1.0, 4.0]], [[0.0, -5.0, -2.0], [-4.0, 6.0, 4.0], [-5.0, -7.0, -3.0], [1.0, 3.0, 7.0], [0.0, 0.0, -3.0]], [[-5.0, 7.0, -2.0], [-5.0, 1.0, 1.0], [-4.0, -3.0, -4.0], [7.0, -1.0, 1.0], [0.0, -7.0, 2.0]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[3.0, 0.0, 0.0], [-2.0, 4.0, 7.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[9.0, -41.0], [-18.0, 20.0], [21.0, -63.0], [-18.0, 40.0], [-6.0, -58.0]], [[6.0, 48.0], [21.0, -15.0], [15.0, -16.0], [6.0, 49.0], [-9.0, 16.0]], [[-15.0, 22.0], [-3.0, -27.0], [6.0, 49.0], [-6.0, 23.0], [-21.0, 16.0]], [[12.0, -7.0], [15.0, -56.0], [-15.0, -17.0], [0.0, -52.0], [-12.0, -14.0]]])+(1.-msk_ref)*numpy.array([[[-21.0, 9.0], [9.0, 24.0], [9.0, 31.0], [9.0, -17.0], [-3.0, -12.0]], [[9.0, -39.0], [0.0, 29.0], [-9.0, -37.0], [3.0, 39.0], [9.0, 18.0]], [[0.0, -34.0], [-12.0, 60.0], [-15.0, -39.0], [3.0, 59.0], [0.0, -21.0]], [[-15.0, 24.0], [-15.0, 21.0], [-12.0, -32.0], [21.0, -11.0], [0.0, -14.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, -6.0, -5.0], [0.0, -5.0, -5.0]], [[-3.0, 0.0, 0.0], [-3.0, 5.0, -4.0]], [[6.0, -1.0, -3.0], [1.0, 2.0, -7.0]], [[0.0, 7.0, -7.0], [0.0, -6.0, 2.0]], [[7.0, 1.0, -4.0], [-4.0, 7.0, -1.0]]], [[[-4.0, 2.0, 7.0], [-6.0, -3.0, 0.0]], [[5.0, -7.0, 0.0], [-3.0, 3.0, 7.0]], [[-7.0, 6.0, 0.0], [3.0, -6.0, 4.0]], [[7.0, 5.0, 0.0], [4.0, 6.0, 0.0]], [[4.0, -2.0, 5.0], [2.0, 0.0, -4.0]]], [[[-4.0, 3.0, -5.0], [-7.0, 6.0, 4.0]], [[-3.0, -4.0, 6.0], [-2.0, -6.0, -4.0]], [[2.0, 0.0, -2.0], [-7.0, 4.0, -6.0]], [[2.0, 7.0, 0.0], [4.0, 1.0, -1.0]], [[0.0, 5.0, 2.0], [-2.0, 0.0, 0.0]]], [[[7.0, 5.0, -1.0], [-4.0, 0.0, 5.0]], [[-6.0, -7.0, -2.0], [0.0, -6.0, 0.0]], [[-3.0, 5.0, 5.0], [0.0, 6.0, 0.0]], [[0.0, -7.0, -2.0], [0.0, -3.0, -1.0]], [[7.0, -4.0, 0.0], [-2.0, -2.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, 4.0, 7.0], [5.0, 5.0, -6.0]], [[-2.0, 0.0, 0.0], [4.0, 3.0, -3.0]], [[-6.0, 0.0, 7.0], [4.0, -3.0, 7.0]], [[2.0, -4.0, 2.0], [0.0, -1.0, 1.0]], [[0.0, 3.0, 3.0], [-1.0, -6.0, 0.0]]], [[[-7.0, 0.0, -1.0], [-6.0, -5.0, 6.0]], [[-6.0, 2.0, -3.0], [7.0, 1.0, 4.0]], [[-3.0, -6.0, -3.0], [-7.0, 3.0, 7.0]], [[7.0, 7.0, -1.0], [-4.0, -5.0, 1.0]], [[5.0, -4.0, -5.0], [6.0, 0.0, 4.0]]], [[[0.0, 1.0, -7.0], [-4.0, 4.0, 2.0]], [[3.0, 5.0, 2.0], [4.0, 6.0, -7.0]], [[4.0, -6.0, -1.0], [0.0, 0.0, 2.0]], [[4.0, -4.0, 1.0], [4.0, -6.0, 4.0]], [[-6.0, 6.0, 0.0], [4.0, 0.0, -1.0]]], [[[-5.0, 7.0, 0.0], [0.0, -6.0, 6.0]], [[2.0, 2.0, -7.0], [-1.0, 7.0, 0.0]], [[-6.0, -2.0, -5.0], [7.0, 6.0, -4.0]], [[-7.0, -6.0, 0.0], [-2.0, -3.0, -2.0]], [[-3.0, -6.0, 0.0], [-2.0, -2.0, 7.0]]]])
      arg1=Symbol(shape=(2, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[-6.0, 1.0, -1.0], [-5.0, -5.0, -4.0]], [[5.0, 0.0, -3.0], [1.0, -1.0, 0.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[62.0, 5.0], [24.0, -23.0], [-21.0, 38.0], [36.0, 27.0], [-48.0, 36.0]], [[64.0, -44.0], [-65.0, 19.0], [47.0, -26.0], [-87.0, 33.0], [-25.0, 7.0]], [[21.0, -18.0], [64.0, -29.0], [29.0, 5.0], [-26.0, 13.0], [13.0, -8.0]], [[-36.0, 34.0], [61.0, -18.0], [-12.0, -36.0], [14.0, 9.0], [-26.0, 35.0]]])+(1.-msk_ref)*numpy.array([[[-23.0, -26.0], [-11.0, -9.0], [-4.0, -44.0], [-17.0, 5.0], [35.0, -4.0]], [[74.0, -33.0], [-15.0, -15.0], [7.0, -16.0], [7.0, 39.0], [-75.0, 46.0]], [[0.0, 13.0], [-37.0, 7.0], [-37.0, 23.0], [-35.0, 27.0], [26.0, -26.0]], [[43.0, -19.0], [-33.0, 23.0], [-10.0, -14.0], [69.0, -34.0], [4.0, -15.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_Symbol_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 5.0, 5.0, -3.0, 0.0], [1.0, 6.0, 0.0, 0.0, -2.0], [-1.0, 2.0, -4.0, 5.0, 1.0], [2.0, 5.0, -7.0, -5.0, 0.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 7.0, 4.0, 5.0, -6.0], [4.0, -2.0, 0.0, -5.0, 3.0], [-2.0, -1.0, 6.0, 6.0, 7.0], [-3.0, 3.0, 6.0, -3.0, 5.0]])
      arg1=Symbol(shape=(4, 5))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([[7.0, -6.0, 1.0, -6.0, -2.0], [-1.0, -7.0, -2.0, -4.0, 0.0], [5.0, -3.0, -5.0, 1.0, -6.0], [-4.0, -2.0, -2.0, -2.0, 0.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-28.0, 24.0, -4.0, 24.0, 8.0], [4.0, 28.0, 8.0, 16.0, 0.0], [-20.0, 12.0, 20.0, -4.0, 24.0], [16.0, 8.0, 8.0, 8.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]], [[-21.0, 18.0, -3.0, 18.0, 6.0], [3.0, 21.0, 6.0, 12.0, 0.0], [-15.0, 9.0, 15.0, -3.0, 18.0], [12.0, 6.0, 6.0, 6.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]], [[[7.0, -6.0, 1.0, -6.0, -2.0], [-1.0, -7.0, -2.0, -4.0, 0.0], [5.0, -3.0, -5.0, 1.0, -6.0], [-4.0, -2.0, -2.0, -2.0, 0.0]], [[42.0, -36.0, 6.0, -36.0, -12.0], [-6.0, -42.0, -12.0, -24.0, 0.0], [30.0, -18.0, -30.0, 6.0, -36.0], [-24.0, -12.0, -12.0, -12.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-14.0, 12.0, -2.0, 12.0, 4.0], [2.0, 14.0, 4.0, 8.0, 0.0], [-10.0, 6.0, 10.0, -2.0, 12.0], [8.0, 4.0, 4.0, 4.0, 0.0]]], [[[-7.0, 6.0, -1.0, 6.0, 2.0], [1.0, 7.0, 2.0, 4.0, 0.0], [-5.0, 3.0, 5.0, -1.0, 6.0], [4.0, 2.0, 2.0, 2.0, 0.0]], [[14.0, -12.0, 2.0, -12.0, -4.0], [-2.0, -14.0, -4.0, -8.0, 0.0], [10.0, -6.0, -10.0, 2.0, -12.0], [-8.0, -4.0, -4.0, -4.0, 0.0]], [[-28.0, 24.0, -4.0, 24.0, 8.0], [4.0, 28.0, 8.0, 16.0, 0.0], [-20.0, 12.0, 20.0, -4.0, 24.0], [16.0, 8.0, 8.0, 8.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]], [[7.0, -6.0, 1.0, -6.0, -2.0], [-1.0, -7.0, -2.0, -4.0, 0.0], [5.0, -3.0, -5.0, 1.0, -6.0], [-4.0, -2.0, -2.0, -2.0, 0.0]]], [[[14.0, -12.0, 2.0, -12.0, -4.0], [-2.0, -14.0, -4.0, -8.0, 0.0], [10.0, -6.0, -10.0, 2.0, -12.0], [-8.0, -4.0, -4.0, -4.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]], [[-49.0, 42.0, -7.0, 42.0, 14.0], [7.0, 49.0, 14.0, 28.0, 0.0], [-35.0, 21.0, 35.0, -7.0, 42.0], [28.0, 14.0, 14.0, 14.0, 0.0]], [[-35.0, 30.0, -5.0, 30.0, 10.0], [5.0, 35.0, 10.0, 20.0, 0.0], [-25.0, 15.0, 25.0, -5.0, 30.0], [20.0, 10.0, 10.0, 10.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-14.0, 12.0, -2.0, 12.0, 4.0], [2.0, 14.0, 4.0, 8.0, 0.0], [-10.0, 6.0, 10.0, -2.0, 12.0], [8.0, 4.0, 4.0, 4.0, 0.0]], [[49.0, -42.0, 7.0, -42.0, -14.0], [-7.0, -49.0, -14.0, -28.0, 0.0], [35.0, -21.0, -35.0, 7.0, -42.0], [-28.0, -14.0, -14.0, -14.0, 0.0]], [[28.0, -24.0, 4.0, -24.0, -8.0], [-4.0, -28.0, -8.0, -16.0, 0.0], [20.0, -12.0, -20.0, 4.0, -24.0], [-16.0, -8.0, -8.0, -8.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]], [[-42.0, 36.0, -6.0, 36.0, 12.0], [6.0, 42.0, 12.0, 24.0, 0.0], [-30.0, 18.0, 30.0, -6.0, 36.0], [24.0, 12.0, 12.0, 12.0, 0.0]]], [[[28.0, -24.0, 4.0, -24.0, -8.0], [-4.0, -28.0, -8.0, -16.0, 0.0], [20.0, -12.0, -20.0, 4.0, -24.0], [-16.0, -8.0, -8.0, -8.0, 0.0]], [[-14.0, 12.0, -2.0, 12.0, 4.0], [2.0, 14.0, 4.0, 8.0, 0.0], [-10.0, 6.0, 10.0, -2.0, 12.0], [8.0, 4.0, 4.0, 4.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-35.0, 30.0, -5.0, 30.0, 10.0], [5.0, 35.0, 10.0, 20.0, 0.0], [-25.0, 15.0, 25.0, -5.0, 30.0], [20.0, 10.0, 10.0, 10.0, 0.0]], [[21.0, -18.0, 3.0, -18.0, -6.0], [-3.0, -21.0, -6.0, -12.0, 0.0], [15.0, -9.0, -15.0, 3.0, -18.0], [-12.0, -6.0, -6.0, -6.0, 0.0]]], [[[-14.0, 12.0, -2.0, 12.0, 4.0], [2.0, 14.0, 4.0, 8.0, 0.0], [-10.0, 6.0, 10.0, -2.0, 12.0], [8.0, 4.0, 4.0, 4.0, 0.0]], [[-7.0, 6.0, -1.0, 6.0, 2.0], [1.0, 7.0, 2.0, 4.0, 0.0], [-5.0, 3.0, 5.0, -1.0, 6.0], [4.0, 2.0, 2.0, 2.0, 0.0]], [[42.0, -36.0, 6.0, -36.0, -12.0], [-6.0, -42.0, -12.0, -24.0, 0.0], [30.0, -18.0, -30.0, 6.0, -36.0], [-24.0, -12.0, -12.0, -12.0, 0.0]], [[42.0, -36.0, 6.0, -36.0, -12.0], [-6.0, -42.0, -12.0, -24.0, 0.0], [30.0, -18.0, -30.0, 6.0, -36.0], [-24.0, -12.0, -12.0, -12.0, 0.0]], [[49.0, -42.0, 7.0, -42.0, -14.0], [-7.0, -49.0, -14.0, -28.0, 0.0], [35.0, -21.0, -35.0, 7.0, -42.0], [-28.0, -14.0, -14.0, -14.0, 0.0]]], [[[-21.0, 18.0, -3.0, 18.0, 6.0], [3.0, 21.0, 6.0, 12.0, 0.0], [-15.0, 9.0, 15.0, -3.0, 18.0], [12.0, 6.0, 6.0, 6.0, 0.0]], [[21.0, -18.0, 3.0, -18.0, -6.0], [-3.0, -21.0, -6.0, -12.0, 0.0], [15.0, -9.0, -15.0, 3.0, -18.0], [-12.0, -6.0, -6.0, -6.0, 0.0]], [[42.0, -36.0, 6.0, -36.0, -12.0], [-6.0, -42.0, -12.0, -24.0, 0.0], [30.0, -18.0, -30.0, 6.0, -36.0], [-24.0, -12.0, -12.0, -12.0, 0.0]], [[-21.0, 18.0, -3.0, 18.0, 6.0], [3.0, 21.0, 6.0, 12.0, 0.0], [-15.0, 9.0, 15.0, -3.0, 18.0], [12.0, 6.0, 6.0, 6.0, 0.0]], [[35.0, -30.0, 5.0, -30.0, -10.0], [-5.0, -35.0, -10.0, -20.0, 0.0], [25.0, -15.0, -25.0, 5.0, -30.0], [-20.0, -10.0, -10.0, -10.0, 0.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-5.0, 7.0, 4.0], [-4.0, 2.0, -6.0], [1.0, -4.0, 3.0], [-3.0, 2.0, 0.0], [3.0, -6.0, 6.0]], [[0.0, 2.0, 0.0], [-6.0, -5.0, -7.0], [-6.0, 0.0, -3.0], [0.0, 1.0, 5.0], [7.0, 7.0, 3.0]], [[0.0, 0.0, 5.0], [-4.0, -2.0, 7.0], [7.0, -2.0, 7.0], [1.0, 1.0, 6.0], [0.0, 3.0, 5.0]], [[-5.0, 2.0, -2.0], [0.0, 0.0, 5.0], [6.0, 3.0, -4.0], [0.0, 3.0, 0.0], [6.0, -6.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-1.0, 4.0, 7.0], [0.0, -4.0, 2.0], [4.0, -5.0, 3.0], [-1.0, -2.0, -3.0], [-1.0, -7.0, -1.0]], [[-1.0, -4.0, -2.0], [6.0, 6.0, 0.0], [0.0, 6.0, 7.0], [1.0, 1.0, -4.0], [5.0, -2.0, 7.0]], [[2.0, 4.0, -5.0], [7.0, 0.0, 6.0], [3.0, 2.0, -3.0], [5.0, -7.0, 5.0], [1.0, 6.0, 0.0]], [[2.0, 5.0, 4.0], [-1.0, -1.0, -3.0], [1.0, -7.0, 2.0], [4.0, -2.0, 5.0], [-1.0, -3.0, -5.0]]])
      arg1=Symbol(shape=(4, 5, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[[1.0, 4.0, -4.0], [4.0, -3.0, 0.0], [0.0, -5.0, 2.0], [-7.0, 2.0, 2.0], [-7.0, 0.0, -3.0]], [[3.0, -4.0, 0.0], [5.0, 5.0, 0.0], [-2.0, 3.0, 0.0], [3.0, 3.0, 4.0], [7.0, -2.0, -3.0]], [[0.0, 6.0, -3.0], [0.0, 5.0, -2.0], [-6.0, 7.0, -6.0], [4.0, 7.0, 4.0], [-7.0, -5.0, 1.0]], [[-6.0, -2.0, 6.0], [-2.0, 5.0, 1.0], [0.0, -2.0, 1.0], [0.0, -6.0, 7.0], [6.0, 2.0, 7.0]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, -41.0, -27.0, 57.0, 23.0], [-43.0, 10.0, 31.0, 22.0, -61.0], [30.0, 27.0, 55.0, 45.0, 4.0], [40.0, 49.0, -10.0, -14.0, 12.0]], [[28.0, -22.0, -22.0, 20.0, 46.0], [-20.0, -10.0, 14.0, -30.0, -14.0], [30.0, 22.0, 74.0, -26.0, 12.0], [-16.0, 12.0, -10.0, -54.0, -62.0]], [[-27.0, 16.0, 26.0, -9.0, -16.0], [19.0, -15.0, -14.0, 3.0, 6.0], [-33.0, -26.0, -52.0, -12.0, 16.0], [20.0, -19.0, 11.0, 45.0, 19.0]], [[5.0, -18.0, -10.0, 25.0, 21.0], [-17.0, -5.0, 12.0, -3.0, -25.0], [12.0, 10.0, 32.0, 2.0, 11.0], [14.0, 16.0, -4.0, -12.0, -14.0]], [[-45.0, 30.0, 42.0, -21.0, -39.0], [33.0, -15.0, -24.0, 15.0, 15.0], [-54.0, -42.0, -96.0, -6.0, 15.0], [30.0, -30.0, 18.0, 78.0, 48.0]]], [[[8.0, -6.0, -10.0, 4.0, 0.0], [-8.0, 10.0, 6.0, 6.0, -4.0], [12.0, 10.0, 14.0, 14.0, -10.0], [-4.0, 10.0, -4.0, -12.0, 4.0]], [[2.0, -9.0, 11.0, 18.0, 63.0], [2.0, -55.0, -3.0, -61.0, -11.0], [-9.0, -11.0, 43.0, -87.0, 60.0], [4.0, -20.0, 3.0, -19.0, -95.0]], [[6.0, -24.0, -6.0, 36.0, 51.0], [-18.0, -30.0, 12.0, -30.0, -33.0], [9.0, 6.0, 54.0, -36.0, 39.0], [18.0, 9.0, -3.0, -21.0, -57.0]], [[-16.0, -3.0, 5.0, 12.0, -15.0], [-4.0, 5.0, 3.0, 23.0, -17.0], [-9.0, -5.0, -23.0, 27.0, 0.0], [28.0, 10.0, 3.0, 29.0, 37.0]], [[23.0, 7.0, -29.0, -29.0, -58.0], [-7.0, 70.0, 7.0, 54.0, 26.0], [33.0, 29.0, -11.0, 89.0, -81.0], [-38.0, 24.0, -11.0, -21.0, 77.0]]], [[[-20.0, 0.0, 10.0, 10.0, -15.0], [0.0, 0.0, 0.0, 20.0, -15.0], [-15.0, -10.0, -30.0, 20.0, 5.0], [30.0, 5.0, 5.0, 35.0, 35.0]], [[-40.0, -10.0, 24.0, 38.0, 7.0], [-4.0, -30.0, 2.0, 10.0, -45.0], [-33.0, -24.0, -32.0, -2.0, 45.0], [70.0, 5.0, 11.0, 61.0, 21.0]], [[-29.0, 34.0, 24.0, -39.0, -70.0], [29.0, 25.0, -20.0, 43.0, 32.0], [-33.0, -24.0, -98.0, 42.0, -32.0], [4.0, -17.0, 11.0, 61.0, 87.0]], [[-19.0, 1.0, 7.0, 7.0, -25.0], [-1.0, 10.0, 1.0, 30.0, -13.0], [-12.0, -7.0, -35.0, 35.0, -6.0], [28.0, 9.0, 4.0, 36.0, 50.0]], [[-8.0, -9.0, -5.0, 16.0, -15.0], [-12.0, 15.0, 9.0, 29.0, -21.0], [3.0, 5.0, -9.0, 41.0, -10.0], [24.0, 20.0, -1.0, 17.0, 41.0]]], [[[11.0, -26.0, -14.0, 35.0, 41.0], [-23.0, -15.0, 16.0, -17.0, -33.0], [18.0, 14.0, 56.0, -14.0, 23.0], [14.0, 18.0, -6.0, -26.0, -40.0]], [[-20.0, 0.0, 10.0, 10.0, -15.0], [0.0, 0.0, 0.0, 20.0, -15.0], [-15.0, -10.0, -30.0, 20.0, 5.0], [30.0, 5.0, 5.0, 35.0, 35.0]], [[34.0, 15.0, -23.0, -44.0, -30.0], [6.0, 45.0, -3.0, 11.0, 48.0], [30.0, 23.0, 9.0, 29.0, -61.0], [-66.0, -1.0, -10.0, -46.0, 14.0]], [[12.0, -9.0, -15.0, 6.0, 0.0], [-12.0, 15.0, 9.0, 9.0, -6.0], [18.0, 15.0, 21.0, 21.0, -15.0], [-6.0, 15.0, -6.0, -18.0, 6.0]], [[-46.0, 42.0, 44.0, -40.0, -63.0], [42.0, 0.0, -30.0, 28.0, 33.0], [-57.0, -44.0, -120.0, 10.0, -5.0], [18.0, -35.0, 19.0, 85.0, 73.0]]]])+(1.-msk_ref)*numpy.array([[[[-13.0, -16.0, -6.0, 29.0, -14.0], [-19.0, 15.0, 14.0, 37.0, -36.0], [3.0, 6.0, -8.0, 52.0, -6.0], [40.0, 29.0, -1.0, 25.0, 51.0]], [[-24.0, 12.0, 24.0, -4.0, -6.0], [16.0, -20.0, -12.0, -4.0, 2.0], [-30.0, -24.0, -40.0, -20.0, 22.0], [20.0, -18.0, 10.0, 38.0, 6.0]], [[-28.0, 31.0, 31.0, -32.0, -37.0], [32.0, -5.0, -23.0, 9.0, 29.0], [-39.0, -31.0, -77.0, -7.0, 0.0], [4.0, -30.0, 13.0, 51.0, 35.0]], [[3.0, 2.0, 4.0, -3.0, 16.0], [5.0, -15.0, -4.0, -21.0, 6.0], [-3.0, -4.0, 10.0, -30.0, 14.0], [-8.0, -11.0, 1.0, -9.0, -31.0]], [[-25.0, 17.0, 33.0, -9.0, 10.0], [25.0, -40.0, -19.0, -28.0, 10.0], [-39.0, -33.0, -37.0, -57.0, 41.0], [14.0, -34.0, 13.0, 35.0, -27.0]]], [[[-9.0, 8.0, 16.0, -5.0, 13.0], [13.0, -25.0, -10.0, -23.0, 7.0], [-18.0, -16.0, -10.0, -40.0, 25.0], [2.0, -20.0, 6.0, 10.0, -28.0]], [[30.0, 6.0, -30.0, -30.0, -42.0], [-6.0, 60.0, 6.0, 36.0, 30.0], [36.0, 30.0, 6.0, 66.0, -72.0], [-48.0, 18.0, -12.0, -36.0, 48.0]], [[-4.0, -18.0, -16.0, 26.0, -21.0], [-24.0, 30.0, 18.0, 46.0, -33.0], [15.0, 16.0, 0.0, 70.0, -23.0], [30.0, 37.0, -5.0, 13.0, 61.0]], [[21.0, 1.0, -13.0, -13.0, 5.0], [-1.0, 10.0, 1.0, -10.0, 17.0], [18.0, 13.0, 25.0, -5.0, -16.0], [-32.0, -1.0, -6.0, -34.0, -20.0]], [[-31.0, 26.0, 24.0, -25.0, -56.0], [23.0, 15.0, -16.0, 37.0, 18.0], [-33.0, -24.0, -86.0, 34.0, -18.0], [16.0, -13.0, 11.0, 61.0, 75.0]]], [[[38.0, -4.0, -30.0, -16.0, 1.0], [-10.0, 30.0, 8.0, -2.0, 21.0], [39.0, 30.0, 46.0, 16.0, -39.0], [-50.0, 11.0, -13.0, -59.0, -15.0]], [[-17.0, 28.0, 12.0, -37.0, -67.0], [21.0, 35.0, -14.0, 45.0, 31.0], [-18.0, -12.0, -78.0, 52.0, -43.0], [-6.0, -8.0, 6.0, 42.0, 84.0]], [[23.0, 6.0, -16.0, -23.0, -12.0], [1.0, 25.0, 0.0, 3.0, 26.0], [21.0, 16.0, 14.0, 14.0, -34.0], [-40.0, 1.0, -7.0, -33.0, 1.0]], [[-43.0, 41.0, 45.0, -39.0, -50.0], [43.0, -10.0, -31.0, 14.0, 34.0], [-57.0, -45.0, -109.0, -9.0, 5.0], [14.0, -40.0, 19.0, 77.0, 51.0]], [[25.0, -14.0, -30.0, 5.0, -7.0], [-21.0, 35.0, 16.0, 21.0, -5.0], [36.0, 30.0, 36.0, 46.0, -37.0], [-18.0, 28.0, -12.0, -36.0, 18.0]]], [[[6.0, -7.0, -17.0, 4.0, -26.0], [-14.0, 35.0, 11.0, 37.0, -8.0], [18.0, 17.0, -1.0, 59.0, -35.0], [2.0, 25.0, -6.0, -2.0, 50.0]], [[7.0, -1.0, -1.0, -1.0, 16.0], [1.0, -10.0, -1.0, -18.0, 4.0], [3.0, 1.0, 17.0, -23.0, 9.0], [-10.0, -6.0, -1.0, -15.0, -29.0]], [[-35.0, 25.0, 39.0, -17.0, -13.0], [31.0, -30.0, -23.0, -10.0, 15.0], [-48.0, -39.0, -67.0, -37.0, 30.0], [20.0, -35.0, 16.0, 56.0, 6.0]], [[-24.0, 22.0, 20.0, -22.0, -43.0], [20.0, 10.0, -14.0, 26.0, 17.0], [-27.0, -20.0, -68.0, 22.0, -13.0], [10.0, -13.0, 9.0, 47.0, 55.0]], [[7.0, 5.0, 5.0, -9.0, 22.0], [9.0, -20.0, -7.0, -32.0, 14.0], [-3.0, -5.0, 15.0, -45.0, 17.0], [-18.0, -18.0, 1.0, -17.0, -47.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -4.0, 0.0], [1.0, 6.0, 0.0]], [[-7.0, 2.0, 3.0], [7.0, 1.0, 6.0]], [[-4.0, -6.0, -1.0], [0.0, 0.0, 2.0]], [[0.0, 0.0, 0.0], [-3.0, -4.0, -7.0]], [[3.0, 2.0, 5.0], [-7.0, 0.0, 4.0]]], [[[-4.0, -3.0, 2.0], [3.0, 7.0, -2.0]], [[0.0, -7.0, 7.0], [-7.0, 3.0, -6.0]], [[-2.0, -1.0, -5.0], [4.0, 7.0, 4.0]], [[6.0, 0.0, 2.0], [-3.0, -7.0, 4.0]], [[5.0, -5.0, 0.0], [7.0, -3.0, 0.0]]], [[[-6.0, -4.0, 2.0], [7.0, -4.0, 3.0]], [[7.0, -7.0, -7.0], [4.0, 2.0, -4.0]], [[7.0, -5.0, 2.0], [7.0, -4.0, 1.0]], [[5.0, 0.0, -3.0], [4.0, 0.0, -1.0]], [[-4.0, -3.0, -5.0], [-3.0, -2.0, 3.0]]], [[[1.0, 7.0, -4.0], [3.0, -3.0, 6.0]], [[-1.0, 2.0, -6.0], [5.0, 5.0, -6.0]], [[1.0, 0.0, -2.0], [7.0, -4.0, -7.0]], [[0.0, 0.0, -1.0], [-4.0, -6.0, 5.0]], [[6.0, -1.0, 0.0], [0.0, -2.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, 4.0, 6.0], [0.0, -2.0, 6.0]], [[-7.0, 3.0, 0.0], [-7.0, 0.0, 5.0]], [[2.0, -1.0, 6.0], [5.0, -1.0, 4.0]], [[4.0, 6.0, -7.0], [-5.0, 5.0, -1.0]], [[4.0, 0.0, 6.0], [5.0, 2.0, 1.0]]], [[[-4.0, 5.0, -6.0], [-2.0, 2.0, 4.0]], [[-7.0, -1.0, 6.0], [-2.0, -4.0, 1.0]], [[-1.0, 7.0, -2.0], [-6.0, 4.0, -4.0]], [[-1.0, -1.0, 1.0], [-5.0, -4.0, 1.0]], [[0.0, -3.0, 4.0], [-3.0, 4.0, -3.0]]], [[[-4.0, 1.0, -7.0], [3.0, -6.0, -1.0]], [[-2.0, -4.0, 7.0], [-3.0, 4.0, 1.0]], [[5.0, 0.0, 6.0], [5.0, 5.0, -2.0]], [[7.0, 1.0, -4.0], [6.0, -4.0, 7.0]], [[3.0, 1.0, -1.0], [2.0, 4.0, -5.0]]], [[[4.0, -6.0, 2.0], [3.0, 3.0, 3.0]], [[-3.0, -1.0, -7.0], [1.0, -2.0, -7.0]], [[-2.0, 5.0, 6.0], [1.0, 3.0, -2.0]], [[-3.0, -5.0, -3.0], [-1.0, 6.0, -4.0]], [[-7.0, 2.0, -6.0], [1.0, 0.0, 5.0]]]])
      arg1=Symbol(shape=(4, 5, 2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      s1=numpy.array([[[[7.0, -3.0, 2.0], [-7.0, -7.0, 7.0]], [[0.0, 5.0, -4.0], [-1.0, -1.0, 2.0]], [[-3.0, 0.0, 1.0], [2.0, 5.0, -2.0]], [[3.0, 4.0, 4.0], [6.0, -5.0, 0.0]], [[0.0, -2.0, 4.0], [3.0, -5.0, -4.0]]], [[[3.0, -4.0, -2.0], [5.0, 7.0, -4.0]], [[1.0, 4.0, 3.0], [6.0, -3.0, -1.0]], [[-1.0, 2.0, -1.0], [-2.0, 0.0, 2.0]], [[-3.0, 3.0, -6.0], [2.0, -1.0, 1.0]], [[5.0, 1.0, -1.0], [-7.0, 4.0, -1.0]]], [[[-3.0, 6.0, 0.0], [-6.0, 5.0, 2.0]], [[6.0, -1.0, 0.0], [7.0, 5.0, -4.0]], [[-5.0, -7.0, -5.0], [-6.0, -1.0, 2.0]], [[4.0, 0.0, 7.0], [-7.0, 0.0, -2.0]], [[1.0, -6.0, -7.0], [-5.0, 3.0, 1.0]]], [[[-2.0, -4.0, 3.0], [5.0, 3.0, 2.0]], [[3.0, 4.0, 0.0], [-5.0, -7.0, 1.0]], [[-3.0, 6.0, 5.0], [-5.0, -6.0, -5.0]], [[-5.0, 7.0, -6.0], [3.0, 4.0, -6.0]], [[-3.0, -1.0, -2.0], [-3.0, 2.0, 0.0]]]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-79.0, -27.0, 50.0, -58.0, -19.0], [45.0, -34.0, -4.0, 2.0, -17.0], [18.0, 5.0, 46.0, -31.0, 31.0], [51.0, -81.0, -47.0, 29.0, 31.0]], [[-63.0, 2.0, 31.0, 36.0, 0.0], [-17.0, 43.0, 6.0, 28.0, -87.0], [8.0, -14.0, -25.0, -68.0, -66.0], [65.0, -49.0, -23.0, 20.0, -6.0]], [[2.0, -22.0, 7.0, -40.0, 0.0], [6.0, -33.0, -3.0, 2.0, -27.0], [-20.0, -26.0, 71.0, -27.0, 41.0], [33.0, -34.0, -39.0, -28.0, 20.0]], [[0.0, -7.0, -12.0, 2.0, 39.0], [-15.0, 1.0, -8.0, -9.0, 12.0], [-16.0, -13.0, 8.0, 35.0, -4.0], [-41.0, 36.0, 74.0, 17.0, 1.0]], [[102.0, 5.0, -26.0, -5.0, -21.0], [-60.0, -20.0, 18.0, -43.0, 57.0], [53.0, -49.0, -4.0, 88.0, -5.0], [-26.0, 56.0, 43.0, -76.0, 0.0]]], [[[-99.0, -37.0, 59.0, -33.0, -4.0], [68.0, -11.0, -14.0, -12.0, -16.0], [7.0, 43.0, 2.0, -19.0, 4.0], [58.0, -90.0, -43.0, 36.0, 16.0]], [[21.0, -71.0, 20.0, -57.0, 30.0], [24.0, -52.0, -19.0, -86.0, 53.0], [3.0, -3.0, 41.0, 110.0, 31.0], [11.0, -20.0, 40.0, -64.0, 20.0]], [[-70.0, 12.0, 36.0, -41.0, -57.0], [61.0, -22.0, 5.0, 38.0, -10.0], [19.0, 36.0, 19.0, -79.0, 44.0], [42.0, -75.0, -107.0, 49.0, 19.0]], [[144.0, 10.0, -65.0, 43.0, 18.0], [-66.0, 11.0, 6.0, -25.0, 17.0], [-27.0, -36.0, -7.0, 51.0, -10.0], [-34.0, 86.0, 29.0, -103.0, -27.0]], [[22.0, -29.0, -16.0, 52.0, 46.0], [49.0, 36.0, -29.0, -13.0, -41.0], [-102.0, 69.0, -29.0, -29.0, -9.0], [36.0, -19.0, -62.0, -51.0, -37.0]]], [[[-26.0, -25.0, 8.0, 36.0, 45.0], [-11.0, 35.0, -12.0, 15.0, -104.0], [-62.0, -15.0, 16.0, -65.0, -40.0], [63.0, -38.0, -22.0, -23.0, -11.0]], [[-14.0, -21.0, -2.0, -21.0, 4.0], [113.0, -20.0, -30.0, 2.0, 19.0], [-85.0, 103.0, 15.0, -41.0, 80.0], [11.0, -45.0, -110.0, 2.0, -8.0]], [[54.0, -34.0, -27.0, 71.0, 55.0], [40.0, 46.0, -31.0, -29.0, -38.0], [-111.0, 72.0, -46.0, -9.0, -23.0], [37.0, -5.0, -57.0, -83.0, -49.0]], [[-6.0, 6.0, -8.0, 27.0, 4.0], [45.0, 21.0, -12.0, 10.0, 1.0], [-41.0, 62.0, -36.0, -27.0, 5.0], [-1.0, -6.0, -45.0, 11.0, -21.0]], [[27.0, 16.0, -15.0, -52.0, -25.0], [-31.0, -46.0, 15.0, 32.0, -8.0], [8.0, -64.0, 92.0, -36.0, 61.0], [-10.0, 8.0, -19.0, -6.0, 30.0]]], [[[20.0, 63.0, -28.0, 48.0, -30.0], [-47.0, 38.0, 23.0, 57.0, -23.0], [18.0, -19.0, -37.0, -57.0, -31.0], [-24.0, 43.0, -8.0, 29.0, -17.0]], [[-137.0, 12.0, 44.0, -14.0, -14.0], [85.0, 10.0, -11.0, 44.0, -6.0], [-2.0, 76.0, -26.0, -69.0, 13.0], [4.0, -61.0, -40.0, 126.0, 8.0]], [[-67.0, -9.0, 3.0, 57.0, 61.0], [42.0, 56.0, -27.0, 20.0, -51.0], [-79.0, 63.0, -47.0, -45.0, -39.0], [1.0, -11.0, 11.0, 54.0, -28.0]], [[103.0, 24.0, -49.0, 2.0, -6.0], [-80.0, -14.0, 19.0, 9.0, 0.0], [4.0, -78.0, 45.0, 11.0, 14.0], [-31.0, 67.0, 26.0, -60.0, 2.0]], [[59.0, -3.0, -28.0, 24.0, 12.0], [8.0, 8.0, -8.0, -19.0, 21.0], [-34.0, 27.0, -21.0, 24.0, 6.0], [-14.0, 28.0, -12.0, -45.0, -21.0]]]])+(1.-msk_ref)*numpy.array([[[[14.0, 10.0, 2.0, 32.0, 2.0], [-84.0, 28.0, 20.0, 2.0, -46.0], [44.0, -74.0, -14.0, 6.0, -72.0], [20.0, 18.0, 54.0, -22.0, -2.0]], [[26.0, 32.0, -3.0, -51.0, -47.0], [-88.0, -42.0, 37.0, 21.0, 12.0], [91.0, -114.0, 66.0, 11.0, 15.0], [-23.0, 31.0, 49.0, 5.0, 39.0]], [[29.0, -25.0, -3.0, 61.0, 30.0], [0.0, 45.0, -12.0, -30.0, -40.0], [-39.0, 27.0, -54.0, 7.0, -58.0], [48.0, -12.0, -21.0, -66.0, -34.0]], [[-11.0, 56.0, -2.0, -47.0, -76.0], [16.0, -37.0, 23.0, 32.0, 89.0], [77.0, 12.0, -4.0, 4.0, 56.0], [-65.0, 25.0, -11.0, 75.0, 21.0]], [[-2.0, -29.0, 12.0, 56.0, 25.0], [35.0, 45.0, -18.0, -39.0, -14.0], [-30.0, 65.0, -80.0, 21.0, -56.0], [43.0, -26.0, -24.0, -39.0, -35.0]]], [[[-27.0, 57.0, 4.0, -38.0, -66.0], [-32.0, -24.0, 32.0, 61.0, 9.0], [72.0, -49.0, 33.0, -52.0, 28.0], [-26.0, 8.0, -10.0, 69.0, 29.0]], [[15.0, -21.0, 1.0, 7.0, 36.0], [-71.0, 6.0, 5.0, -17.0, -45.0], [9.0, -79.0, 30.0, 26.0, -44.0], [16.0, 14.0, 74.0, -36.0, 8.0]], [[-46.0, 37.0, 17.0, -39.0, -44.0], [-13.0, -23.0, 21.0, 16.0, 66.0], [93.0, -19.0, -10.0, 32.0, 9.0], [-58.0, 23.0, 61.0, 88.0, 26.0]], [[68.0, 2.0, -28.0, -13.0, 7.0], [-58.0, -21.0, 10.0, -11.0, 11.0], [9.0, -64.0, 43.0, 36.0, 12.0], [-26.0, 47.0, 46.0, -45.0, 9.0]], [[-11.0, -38.0, 24.0, -34.0, 5.0], [29.0, -27.0, -10.0, -46.0, 33.0], [14.0, 14.0, 9.0, 55.0, 14.0], [15.0, -28.0, 8.0, -20.0, 12.0]]], [[[-31.0, 34.0, -17.0, 12.0, 13.0], [-25.0, 16.0, 5.0, 68.0, -56.0], [-32.0, -30.0, 34.0, -84.0, 5.0], [-22.0, 18.0, 9.0, 60.0, 4.0]], [[12.0, -47.0, 25.0, -32.0, 3.0], [5.0, -28.0, -5.0, -57.0, 15.0], [22.0, -13.0, 19.0, 60.0, 1.0], [40.0, -34.0, 3.0, -59.0, 13.0]], [[-37.0, -38.0, 30.0, 44.0, 22.0], [71.0, 40.0, -25.0, -48.0, 6.0], [-24.0, 98.0, -94.0, 31.0, -49.0], [44.0, -47.0, -30.0, -14.0, -32.0]], [[73.0, 33.0, -47.0, 65.0, -8.0], [-1.0, 40.0, 1.0, 29.0, -25.0], [-57.0, 35.0, -40.0, -56.0, -6.0], [2.0, 30.0, -76.0, -44.0, -40.0]], [[-61.0, -7.0, 24.0, 1.0, 0.0], [65.0, 9.0, -14.0, -5.0, 24.0], [-5.0, 71.0, -43.0, 1.0, 1.0], [-1.0, -30.0, -17.0, 50.0, -6.0]]], [[[29.0, -38.0, 5.0, -1.0, 2.0], [56.0, -8.0, -18.0, -36.0, 0.0], [-45.0, 54.0, -3.0, 3.0, 23.0], [52.0, -45.0, -86.0, -71.0, -13.0]], [[-74.0, 10.0, 8.0, -25.0, 15.0], [28.0, -9.0, -8.0, 45.0, -17.0], [-27.0, 8.0, 39.0, -54.0, 34.0], [-26.0, -11.0, 10.0, 87.0, 17.0]], [[-59.0, -7.0, 33.0, 29.0, 10.0], [-4.0, 35.0, 0.0, -18.0, -4.0], [41.0, 13.0, -68.0, 31.0, -72.0], [12.0, -14.0, 53.0, 36.0, -8.0]], [[-75.0, -26.0, 42.0, -77.0, -19.0], [70.0, -52.0, -10.0, 0.0, 18.0], [7.0, 26.0, 57.0, -18.0, 67.0], [22.0, -70.0, -47.0, 43.0, 35.0]], [[-39.0, 43.0, 7.0, -31.0, -45.0], [-32.0, -16.0, 25.0, 70.0, -39.0], [37.0, -57.0, 55.0, -87.0, 23.0], [3.0, -13.0, -27.0, 58.0, 28.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 0.0, 6.0, -7.0, 2.0], [7.0, 0.0, 7.0, -4.0, -4.0], [-5.0, -2.0, 0.0, -4.0, -7.0], [6.0, -5.0, -6.0, 2.0, 7.0]])+(1.-msk_arg0)*numpy.array([[-3.0, 7.0, -3.0, -7.0, 7.0], [6.0, 6.0, 2.0, 0.0, -5.0], [3.0, 1.0, 2.0, 0.0, 7.0], [-2.0, 1.0, 5.0, 0.0, -6.0]])
      arg1=Data(-2.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-8.0, -0.0, -12.0, 14.0, -4.0], [-14.0, -0.0, -14.0, 8.0, 8.0], [10.0, 4.0, -0.0, 8.0, 14.0], [-12.0, 10.0, 12.0, -4.0, -14.0]])+(1.-msk_ref)*numpy.array([[6.0, -14.0, 6.0, 14.0, -14.0], [-12.0, -12.0, -4.0, -0.0, 10.0], [-6.0, -2.0, -4.0, -0.0, -14.0], [4.0, -2.0, -10.0, -0.0, 12.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 4.0, 0.0], [-2.0, -4.0, 5.0], [-7.0, -4.0, 1.0], [2.0, 7.0, -3.0], [-3.0, -7.0, 6.0]], [[1.0, 6.0, -6.0], [3.0, 0.0, -2.0], [-3.0, -5.0, 2.0], [3.0, 6.0, -5.0], [0.0, 7.0, 4.0]], [[5.0, -5.0, -5.0], [5.0, 1.0, -2.0], [-3.0, 6.0, 0.0], [-4.0, -6.0, -6.0], [0.0, -6.0, -5.0]], [[-5.0, -5.0, -2.0], [2.0, 3.0, -4.0], [-1.0, -4.0, -6.0], [-1.0, 7.0, 6.0], [-1.0, 1.0, -5.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, -4.0, -1.0], [2.0, -1.0, -5.0], [-1.0, 5.0, 0.0], [0.0, 5.0, 7.0], [3.0, 2.0, 1.0]], [[0.0, 4.0, -3.0], [5.0, 2.0, -1.0], [1.0, -2.0, 3.0], [-6.0, -3.0, 7.0], [-3.0, -4.0, 0.0]], [[-2.0, -7.0, 3.0], [1.0, -2.0, -4.0], [5.0, -3.0, 3.0], [-6.0, 1.0, -2.0], [-3.0, 0.0, 7.0]], [[6.0, 0.0, 1.0], [2.0, 7.0, -6.0], [-7.0, -7.0, -1.0], [-4.0, 5.0, 0.0], [-3.0, 6.0, -2.0]]])
      arg1=Data(numpy.array([7.0, 4.0, 0.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[37.0, -30.0, -65.0, 42.0, -49.0], [31.0, 21.0, -41.0, 45.0, 28.0], [15.0, 39.0, 3.0, -52.0, -24.0], [-55.0, 26.0, -23.0, 21.0, -3.0]])+(1.-msk_ref)*numpy.array([[-30.0, 10.0, 13.0, 20.0, 29.0], [16.0, 43.0, -1.0, -54.0, -37.0], [-42.0, -1.0, 23.0, -38.0, -21.0], [42.0, 42.0, -77.0, -8.0, 3.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-5.0, 2.0, 0.0], [-5.0, -1.0, -4.0]], [[-4.0, -5.0, -3.0], [7.0, 5.0, 1.0]], [[-6.0, 2.0, 1.0], [6.0, -2.0, -6.0]], [[-1.0, 3.0, 4.0], [0.0, -5.0, -7.0]], [[-1.0, -6.0, 4.0], [-1.0, -6.0, 2.0]]], [[[3.0, 2.0, -3.0], [7.0, -7.0, 6.0]], [[0.0, 5.0, -3.0], [-1.0, -7.0, -4.0]], [[6.0, 0.0, 2.0], [7.0, -2.0, 6.0]], [[5.0, 1.0, 1.0], [-2.0, 7.0, 7.0]], [[-2.0, -1.0, -4.0], [2.0, 2.0, -5.0]]], [[[-1.0, 6.0, 3.0], [3.0, -5.0, 1.0]], [[3.0, 5.0, 6.0], [-7.0, 5.0, 4.0]], [[3.0, 6.0, -3.0], [6.0, 5.0, 2.0]], [[-5.0, 0.0, 6.0], [-3.0, 0.0, -6.0]], [[-7.0, -6.0, 6.0], [-4.0, 2.0, -6.0]]], [[[-6.0, 7.0, 2.0], [-6.0, -6.0, 1.0]], [[2.0, 0.0, -1.0], [2.0, 0.0, 6.0]], [[-6.0, 4.0, 0.0], [3.0, 0.0, 5.0]], [[-2.0, 5.0, 7.0], [2.0, -6.0, 0.0]], [[-1.0, 2.0, 2.0], [-4.0, -7.0, 6.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, 2.0, -2.0], [-3.0, -7.0, 0.0]], [[2.0, 2.0, -3.0], [-7.0, -7.0, 3.0]], [[3.0, 4.0, 5.0], [-2.0, 0.0, 5.0]], [[3.0, -4.0, 0.0], [-3.0, -3.0, -7.0]], [[7.0, 0.0, -4.0], [3.0, 7.0, 2.0]]], [[[-4.0, 1.0, 5.0], [5.0, 0.0, -3.0]], [[7.0, -3.0, 0.0], [-4.0, -2.0, 6.0]], [[3.0, -7.0, 0.0], [0.0, 3.0, 1.0]], [[-2.0, -5.0, -3.0], [-2.0, -1.0, -6.0]], [[2.0, 0.0, 1.0], [6.0, -4.0, 7.0]]], [[[3.0, -1.0, -1.0], [-6.0, 7.0, -3.0]], [[-7.0, 7.0, -3.0], [0.0, -7.0, 3.0]], [[-2.0, -7.0, -5.0], [2.0, 2.0, 7.0]], [[3.0, -3.0, -3.0], [0.0, -6.0, 2.0]], [[4.0, 1.0, 7.0], [0.0, 5.0, 4.0]]], [[[6.0, -7.0, 6.0], [-5.0, 6.0, -2.0]], [[4.0, -7.0, 5.0], [-3.0, -7.0, -2.0]], [[5.0, -1.0, -4.0], [4.0, 5.0, 1.0]], [[-1.0, 0.0, 5.0], [-6.0, -7.0, 6.0]], [[1.0, -6.0, -3.0], [-3.0, 2.0, -7.0]]]])
      arg1=Data(numpy.array([[3.0, 5.0, -7.0], [0.0, -2.0, -4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[13.0, -30.0, 13.0, 22.0, -57.0], [30.0, 76.0, -16.0, -29.0, 33.0], [12.0, -34.0, 42.0, -33.0, -73.0], [11.0, -11.0, -18.0, -18.0, -17.0]])+(1.-msk_ref)*numpy.array([[35.0, 39.0, -26.0, 23.0, 27.0], [-30.0, -14.0, -36.0, 16.0, -21.0], [9.0, 37.0, -38.0, 19.0, -58.0], [-63.0, -36.0, 24.0, -48.0, 18.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-2.0, 3.0, 6.0, 0.0, 6.0], [5.0, -1.0, 6.0, -3.0, 4.0], [-7.0, -7.0, 1.0, 3.0, 3.0], [6.0, -2.0, 6.0, 1.0, -2.0]])+(1.-msk_arg0)*numpy.array([[-2.0, 0.0, -7.0, -7.0, -1.0], [6.0, 2.0, -1.0, -3.0, 6.0], [4.0, -1.0, 0.0, 3.0, 6.0], [-1.0, 1.0, 3.0, 5.0, 4.0]])
      arg1=Data(numpy.array([5.0, 3.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-10.0, -6.0], [15.0, 9.0], [30.0, 18.0], [0.0, 0.0], [30.0, 18.0]], [[25.0, 15.0], [-5.0, -3.0], [30.0, 18.0], [-15.0, -9.0], [20.0, 12.0]], [[-35.0, -21.0], [-35.0, -21.0], [5.0, 3.0], [15.0, 9.0], [15.0, 9.0]], [[30.0, 18.0], [-10.0, -6.0], [30.0, 18.0], [5.0, 3.0], [-10.0, -6.0]]])+(1.-msk_ref)*numpy.array([[[-10.0, -6.0], [0.0, 0.0], [-35.0, -21.0], [-35.0, -21.0], [-5.0, -3.0]], [[30.0, 18.0], [10.0, 6.0], [-5.0, -3.0], [-15.0, -9.0], [30.0, 18.0]], [[20.0, 12.0], [-5.0, -3.0], [0.0, 0.0], [15.0, 9.0], [30.0, 18.0]], [[-5.0, -3.0], [5.0, 3.0], [15.0, 9.0], [25.0, 15.0], [20.0, 12.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 2.0, -2.0], [-3.0, -6.0, -1.0], [-6.0, 6.0, -3.0], [0.0, -1.0, 0.0], [-7.0, 1.0, -1.0]], [[-3.0, 6.0, -6.0], [2.0, 5.0, -1.0], [-2.0, 5.0, -7.0], [0.0, 0.0, 3.0], [-3.0, 3.0, -6.0]], [[-1.0, 2.0, 4.0], [-6.0, 0.0, -4.0], [3.0, -7.0, 2.0], [-5.0, 2.0, -4.0], [0.0, 0.0, -1.0]], [[-2.0, 0.0, -1.0], [-5.0, -4.0, 7.0], [0.0, 5.0, 0.0], [-4.0, -2.0, 0.0], [3.0, 6.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -1.0, -1.0], [4.0, 3.0, -1.0], [-4.0, -2.0, -1.0], [-6.0, -5.0, 1.0], [7.0, -3.0, 1.0]], [[2.0, 0.0, 0.0], [0.0, 0.0, 7.0], [0.0, -6.0, 2.0], [-5.0, -3.0, 0.0], [-4.0, 6.0, -2.0]], [[-3.0, 1.0, 0.0], [3.0, -1.0, -6.0], [3.0, 2.0, -6.0], [-7.0, 5.0, -1.0], [6.0, 1.0, 1.0]], [[-4.0, -4.0, 2.0], [-4.0, -3.0, -7.0], [-6.0, -5.0, -6.0], [6.0, 5.0, 4.0], [-2.0, 1.0, 0.0]]])
      arg1=Data(numpy.array([[3.0, 2.0, 4.0], [-6.0, -4.0, 3.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-25.0, 28.0], [-25.0, 39.0], [-18.0, 3.0], [-2.0, 4.0], [-23.0, 35.0]], [[-21.0, -24.0], [12.0, -35.0], [-24.0, -29.0], [12.0, 9.0], [-27.0, -12.0]], [[17.0, 10.0], [-34.0, 24.0], [3.0, 16.0], [-27.0, 10.0], [-4.0, -3.0]], [[-10.0, 9.0], [5.0, 67.0], [10.0, -20.0], [-16.0, 32.0], [21.0, -42.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -11.0], [14.0, -39.0], [-20.0, 29.0], [-24.0, 59.0], [19.0, -27.0]], [[6.0, -12.0], [28.0, 21.0], [-4.0, 30.0], [-21.0, 42.0], [-8.0, -6.0]], [[-7.0, 14.0], [-17.0, -32.0], [-11.0, -44.0], [-15.0, 19.0], [24.0, -37.0]], [[-12.0, 46.0], [-46.0, 15.0], [-52.0, 38.0], [44.0, -44.0], [-4.0, 8.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, 0.0, -5.0], [-3.0, 0.0, 0.0]], [[-3.0, 4.0, -7.0], [-6.0, -5.0, 4.0]], [[-5.0, -7.0, -7.0], [3.0, -6.0, -3.0]], [[3.0, -1.0, 0.0], [-6.0, -5.0, -1.0]], [[1.0, 2.0, -7.0], [-1.0, -6.0, 4.0]]], [[[5.0, -3.0, -2.0], [0.0, 0.0, -2.0]], [[0.0, -5.0, 1.0], [-2.0, -6.0, 1.0]], [[2.0, -4.0, -4.0], [-5.0, -4.0, 7.0]], [[1.0, 0.0, 6.0], [7.0, -1.0, -5.0]], [[-7.0, 5.0, -6.0], [7.0, -6.0, 7.0]]], [[[0.0, -6.0, 4.0], [0.0, 1.0, 1.0]], [[-5.0, -5.0, -3.0], [-2.0, 0.0, 0.0]], [[-3.0, -6.0, -7.0], [3.0, 3.0, -6.0]], [[0.0, -5.0, 1.0], [-4.0, -3.0, 4.0]], [[6.0, 4.0, -4.0], [-6.0, -6.0, 5.0]]], [[[-5.0, 2.0, -7.0], [4.0, 0.0, -5.0]], [[7.0, -1.0, 5.0], [-4.0, -2.0, 2.0]], [[5.0, 0.0, 4.0], [-4.0, 0.0, -6.0]], [[1.0, -1.0, 7.0], [-7.0, -7.0, 2.0]], [[6.0, -3.0, 0.0], [-1.0, 2.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[5.0, 3.0, 0.0], [5.0, 1.0, 3.0]], [[4.0, -3.0, 7.0], [5.0, 5.0, 7.0]], [[6.0, -1.0, 7.0], [-1.0, -2.0, 7.0]], [[-2.0, -7.0, 2.0], [-3.0, -5.0, -2.0]], [[6.0, 5.0, 2.0], [-1.0, 7.0, 1.0]]], [[[-6.0, 7.0, 6.0], [-6.0, 6.0, -5.0]], [[1.0, -5.0, 1.0], [-5.0, 0.0, -1.0]], [[3.0, 1.0, 7.0], [5.0, 0.0, 1.0]], [[0.0, -4.0, -3.0], [5.0, 5.0, 7.0]], [[-5.0, -5.0, 3.0], [-1.0, -2.0, -6.0]]], [[[3.0, -4.0, -4.0], [0.0, 3.0, 5.0]], [[0.0, 5.0, 3.0], [-3.0, -7.0, -7.0]], [[-3.0, 5.0, -3.0], [7.0, -2.0, 4.0]], [[-2.0, 1.0, 0.0], [7.0, 1.0, 7.0]], [[-7.0, 3.0, 0.0], [5.0, -1.0, -5.0]]], [[[3.0, 0.0, 2.0], [-1.0, -6.0, -5.0]], [[-7.0, 5.0, 0.0], [0.0, 0.0, -4.0]], [[-1.0, -6.0, 3.0], [4.0, -3.0, -4.0]], [[4.0, -7.0, -3.0], [5.0, -2.0, 5.0]], [[-2.0, 0.0, 4.0], [5.0, -5.0, 4.0]]]])
      arg1=Data(numpy.array([[[6.0, 0.0, 2.0], [-1.0, 0.0, 6.0]], [[1.0, -2.0, 5.0], [1.0, 5.0, 3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[17.0, -24.0], [-2.0, -65.0], [-65.0, -62.0], [18.0, -29.0], [17.0, -57.0]], [[14.0, -5.0], [10.0, -14.0], [51.0, -14.0], [-19.0, 18.0], [-19.0, -49.0]], [[14.0, 40.0], [-34.0, -12.0], [-71.0, -26.0], [30.0, 8.0], [64.0, -43.0]], [[-78.0, -55.0], [68.0, 26.0], [6.0, 3.0], [39.0, 2.0], [55.0, 30.0]]])+(1.-msk_ref)*numpy.array([[[43.0, 18.0], [75.0, 96.0], [93.0, 53.0], [-17.0, -12.0], [47.0, 43.0]], [[-48.0, 19.0], [7.0, 8.0], [33.0, 44.0], [31.0, 44.0], [-59.0, -9.0]], [[40.0, 21.0], [-33.0, -54.0], [-7.0, -19.0], [23.0, 29.0], [-77.0, -28.0]], [[-7.0, -33.0], [-66.0, -29.0], [-28.0, 3.0], [43.0, 13.0], [15.0, 10.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_constData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[4.0, 0.0, 6.0, -6.0, 7.0], [6.0, 1.0, -4.0, 0.0, -3.0], [-2.0, 0.0, 3.0, -5.0, 6.0], [4.0, 0.0, -4.0, -4.0, 1.0]])+(1.-msk_arg0)*numpy.array([[6.0, 0.0, -6.0, -4.0, -2.0], [3.0, 7.0, -6.0, 3.0, -4.0], [1.0, -3.0, -3.0, 1.0, 7.0], [7.0, 3.0, -1.0, 7.0, 5.0]])
      arg1=Data(numpy.array([[1.0, -7.0, 1.0, 6.0, 4.0], [-4.0, -5.0, 0.0, -7.0, 1.0], [4.0, -2.0, -4.0, 0.0, 7.0], [7.0, 7.0, 5.0, -3.0, 4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[4.0, -28.0, 4.0, 24.0, 16.0], [-16.0, -20.0, 0.0, -28.0, 4.0], [16.0, -8.0, -16.0, 0.0, 28.0], [28.0, 28.0, 20.0, -12.0, 16.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[6.0, -42.0, 6.0, 36.0, 24.0], [-24.0, -30.0, 0.0, -42.0, 6.0], [24.0, -12.0, -24.0, 0.0, 42.0], [42.0, 42.0, 30.0, -18.0, 24.0]], [[-6.0, 42.0, -6.0, -36.0, -24.0], [24.0, 30.0, 0.0, 42.0, -6.0], [-24.0, 12.0, 24.0, 0.0, -42.0], [-42.0, -42.0, -30.0, 18.0, -24.0]], [[7.0, -49.0, 7.0, 42.0, 28.0], [-28.0, -35.0, 0.0, -49.0, 7.0], [28.0, -14.0, -28.0, 0.0, 49.0], [49.0, 49.0, 35.0, -21.0, 28.0]]], [[[6.0, -42.0, 6.0, 36.0, 24.0], [-24.0, -30.0, 0.0, -42.0, 6.0], [24.0, -12.0, -24.0, 0.0, 42.0], [42.0, 42.0, 30.0, -18.0, 24.0]], [[1.0, -7.0, 1.0, 6.0, 4.0], [-4.0, -5.0, 0.0, -7.0, 1.0], [4.0, -2.0, -4.0, 0.0, 7.0], [7.0, 7.0, 5.0, -3.0, 4.0]], [[-4.0, 28.0, -4.0, -24.0, -16.0], [16.0, 20.0, 0.0, 28.0, -4.0], [-16.0, 8.0, 16.0, 0.0, -28.0], [-28.0, -28.0, -20.0, 12.0, -16.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-3.0, 21.0, -3.0, -18.0, -12.0], [12.0, 15.0, 0.0, 21.0, -3.0], [-12.0, 6.0, 12.0, 0.0, -21.0], [-21.0, -21.0, -15.0, 9.0, -12.0]]], [[[-2.0, 14.0, -2.0, -12.0, -8.0], [8.0, 10.0, 0.0, 14.0, -2.0], [-8.0, 4.0, 8.0, 0.0, -14.0], [-14.0, -14.0, -10.0, 6.0, -8.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[3.0, -21.0, 3.0, 18.0, 12.0], [-12.0, -15.0, 0.0, -21.0, 3.0], [12.0, -6.0, -12.0, 0.0, 21.0], [21.0, 21.0, 15.0, -9.0, 12.0]], [[-5.0, 35.0, -5.0, -30.0, -20.0], [20.0, 25.0, 0.0, 35.0, -5.0], [-20.0, 10.0, 20.0, 0.0, -35.0], [-35.0, -35.0, -25.0, 15.0, -20.0]], [[6.0, -42.0, 6.0, 36.0, 24.0], [-24.0, -30.0, 0.0, -42.0, 6.0], [24.0, -12.0, -24.0, 0.0, 42.0], [42.0, 42.0, 30.0, -18.0, 24.0]]], [[[4.0, -28.0, 4.0, 24.0, 16.0], [-16.0, -20.0, 0.0, -28.0, 4.0], [16.0, -8.0, -16.0, 0.0, 28.0], [28.0, 28.0, 20.0, -12.0, 16.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-4.0, 28.0, -4.0, -24.0, -16.0], [16.0, 20.0, 0.0, 28.0, -4.0], [-16.0, 8.0, 16.0, 0.0, -28.0], [-28.0, -28.0, -20.0, 12.0, -16.0]], [[-4.0, 28.0, -4.0, -24.0, -16.0], [16.0, 20.0, 0.0, 28.0, -4.0], [-16.0, 8.0, 16.0, 0.0, -28.0], [-28.0, -28.0, -20.0, 12.0, -16.0]], [[1.0, -7.0, 1.0, 6.0, 4.0], [-4.0, -5.0, 0.0, -7.0, 1.0], [4.0, -2.0, -4.0, 0.0, 7.0], [7.0, 7.0, 5.0, -3.0, 4.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, -42.0, 6.0, 36.0, 24.0], [-24.0, -30.0, 0.0, -42.0, 6.0], [24.0, -12.0, -24.0, 0.0, 42.0], [42.0, 42.0, 30.0, -18.0, 24.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-6.0, 42.0, -6.0, -36.0, -24.0], [24.0, 30.0, 0.0, 42.0, -6.0], [-24.0, 12.0, 24.0, 0.0, -42.0], [-42.0, -42.0, -30.0, 18.0, -24.0]], [[-4.0, 28.0, -4.0, -24.0, -16.0], [16.0, 20.0, 0.0, 28.0, -4.0], [-16.0, 8.0, 16.0, 0.0, -28.0], [-28.0, -28.0, -20.0, 12.0, -16.0]], [[-2.0, 14.0, -2.0, -12.0, -8.0], [8.0, 10.0, 0.0, 14.0, -2.0], [-8.0, 4.0, 8.0, 0.0, -14.0], [-14.0, -14.0, -10.0, 6.0, -8.0]]], [[[3.0, -21.0, 3.0, 18.0, 12.0], [-12.0, -15.0, 0.0, -21.0, 3.0], [12.0, -6.0, -12.0, 0.0, 21.0], [21.0, 21.0, 15.0, -9.0, 12.0]], [[7.0, -49.0, 7.0, 42.0, 28.0], [-28.0, -35.0, 0.0, -49.0, 7.0], [28.0, -14.0, -28.0, 0.0, 49.0], [49.0, 49.0, 35.0, -21.0, 28.0]], [[-6.0, 42.0, -6.0, -36.0, -24.0], [24.0, 30.0, 0.0, 42.0, -6.0], [-24.0, 12.0, 24.0, 0.0, -42.0], [-42.0, -42.0, -30.0, 18.0, -24.0]], [[3.0, -21.0, 3.0, 18.0, 12.0], [-12.0, -15.0, 0.0, -21.0, 3.0], [12.0, -6.0, -12.0, 0.0, 21.0], [21.0, 21.0, 15.0, -9.0, 12.0]], [[-4.0, 28.0, -4.0, -24.0, -16.0], [16.0, 20.0, 0.0, 28.0, -4.0], [-16.0, 8.0, 16.0, 0.0, -28.0], [-28.0, -28.0, -20.0, 12.0, -16.0]]], [[[1.0, -7.0, 1.0, 6.0, 4.0], [-4.0, -5.0, 0.0, -7.0, 1.0], [4.0, -2.0, -4.0, 0.0, 7.0], [7.0, 7.0, 5.0, -3.0, 4.0]], [[-3.0, 21.0, -3.0, -18.0, -12.0], [12.0, 15.0, 0.0, 21.0, -3.0], [-12.0, 6.0, 12.0, 0.0, -21.0], [-21.0, -21.0, -15.0, 9.0, -12.0]], [[-3.0, 21.0, -3.0, -18.0, -12.0], [12.0, 15.0, 0.0, 21.0, -3.0], [-12.0, 6.0, 12.0, 0.0, -21.0], [-21.0, -21.0, -15.0, 9.0, -12.0]], [[1.0, -7.0, 1.0, 6.0, 4.0], [-4.0, -5.0, 0.0, -7.0, 1.0], [4.0, -2.0, -4.0, 0.0, 7.0], [7.0, 7.0, 5.0, -3.0, 4.0]], [[7.0, -49.0, 7.0, 42.0, 28.0], [-28.0, -35.0, 0.0, -49.0, 7.0], [28.0, -14.0, -28.0, 0.0, 49.0], [49.0, 49.0, 35.0, -21.0, 28.0]]], [[[7.0, -49.0, 7.0, 42.0, 28.0], [-28.0, -35.0, 0.0, -49.0, 7.0], [28.0, -14.0, -28.0, 0.0, 49.0], [49.0, 49.0, 35.0, -21.0, 28.0]], [[3.0, -21.0, 3.0, 18.0, 12.0], [-12.0, -15.0, 0.0, -21.0, 3.0], [12.0, -6.0, -12.0, 0.0, 21.0], [21.0, 21.0, 15.0, -9.0, 12.0]], [[-1.0, 7.0, -1.0, -6.0, -4.0], [4.0, 5.0, 0.0, 7.0, -1.0], [-4.0, 2.0, 4.0, 0.0, -7.0], [-7.0, -7.0, -5.0, 3.0, -4.0]], [[7.0, -49.0, 7.0, 42.0, 28.0], [-28.0, -35.0, 0.0, -49.0, 7.0], [28.0, -14.0, -28.0, 0.0, 49.0], [49.0, 49.0, 35.0, -21.0, 28.0]], [[5.0, -35.0, 5.0, 30.0, 20.0], [-20.0, -25.0, 0.0, -35.0, 5.0], [20.0, -10.0, -20.0, 0.0, 35.0], [35.0, 35.0, 25.0, -15.0, 20.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 1.0, 0.0], [1.0, -3.0, 0.0], [4.0, 0.0, 2.0], [0.0, 7.0, -5.0], [0.0, 2.0, -3.0]], [[-5.0, -4.0, -6.0], [3.0, 4.0, 6.0], [3.0, 0.0, -1.0], [-3.0, -2.0, 5.0], [-1.0, -4.0, 0.0]], [[-2.0, -4.0, -2.0], [1.0, -4.0, -1.0], [-1.0, -2.0, 6.0], [6.0, 0.0, 5.0], [-7.0, -3.0, -3.0]], [[3.0, -2.0, -1.0], [0.0, 6.0, 0.0], [-4.0, -3.0, 0.0], [3.0, -7.0, 7.0], [2.0, -5.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[1.0, -1.0, -3.0], [4.0, -5.0, 5.0], [7.0, -2.0, 0.0], [4.0, -3.0, 7.0], [3.0, 4.0, -5.0]], [[4.0, 3.0, 0.0], [-5.0, -1.0, -2.0], [-7.0, -2.0, 6.0], [-4.0, 6.0, 5.0], [-5.0, 0.0, 5.0]], [[-2.0, -6.0, 4.0], [7.0, -2.0, -1.0], [6.0, 3.0, -2.0], [0.0, -5.0, 0.0], [3.0, 2.0, -4.0]], [[1.0, -3.0, 6.0], [6.0, -3.0, 0.0], [0.0, -4.0, -5.0], [6.0, -3.0, -4.0], [6.0, 0.0, 5.0]]])
      arg1=Data(numpy.array([[[-2.0, 3.0, 5.0], [-6.0, 5.0, 3.0], [6.0, -5.0, -7.0], [2.0, -4.0, 5.0], [4.0, 1.0, -2.0]], [[-5.0, -7.0, 4.0], [0.0, 2.0, -3.0], [2.0, 0.0, 6.0], [4.0, 1.0, -3.0], [-6.0, 7.0, 5.0]], [[-7.0, 0.0, -2.0], [6.0, 5.0, 7.0], [2.0, -7.0, -5.0], [5.0, -5.0, -4.0], [-7.0, -6.0, 0.0]], [[-3.0, 6.0, 6.0], [5.0, 2.0, 0.0], [1.0, -1.0, 3.0], [7.0, 0.0, 2.0], [-7.0, -7.0, -3.0]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[3.0, 5.0, -5.0, -4.0, 1.0], [-7.0, 2.0, 0.0, 1.0, 7.0], [0.0, 5.0, -7.0, -5.0, -6.0], [6.0, 2.0, -1.0, 0.0, -7.0]], [[-11.0, -21.0, 21.0, 14.0, 1.0], [16.0, -6.0, 2.0, 1.0, -27.0], [-7.0, -9.0, 23.0, 20.0, 11.0], [-21.0, -1.0, 4.0, 7.0, 14.0]], [[2.0, -18.0, 10.0, 18.0, 12.0], [-12.0, -6.0, 20.0, 10.0, -14.0], [-32.0, 38.0, -2.0, 12.0, -28.0], [0.0, 20.0, 10.0, 32.0, -34.0]], [[-4.0, 20.0, 0.0, -53.0, 17.0], [-69.0, 29.0, -30.0, 22.0, 24.0], [10.0, 0.0, -24.0, -15.0, -42.0], [12.0, 14.0, -22.0, -10.0, -34.0]], [[-9.0, 1.0, 11.0, -23.0, 8.0], [-26.0, 13.0, -18.0, 11.0, -1.0], [6.0, -11.0, 1.0, 2.0, -12.0], [-6.0, 4.0, -11.0, -6.0, -5.0]]], [[[-32.0, -8.0, 32.0, -24.0, -12.0], [29.0, 10.0, -46.0, -6.0, -28.0], [47.0, -92.0, 48.0, 19.0, 59.0], [-45.0, -33.0, -19.0, -47.0, 81.0]], [[36.0, 20.0, -44.0, 20.0, 4.0], [-19.0, -10.0, 42.0, -2.0, 40.0], [-33.0, 80.0, -52.0, -29.0, -45.0], [51.0, 23.0, 17.0, 33.0, -67.0]], [[-11.0, -21.0, 25.0, 1.0, 14.0], [-19.0, 3.0, 0.0, 15.0, -23.0], [-19.0, 11.0, 11.0, 19.0, -21.0], [-15.0, 15.0, 0.0, 19.0, -18.0]], [[25.0, 23.0, -43.0, 27.0, -24.0], [49.0, -19.0, 24.0, -29.0, 29.0], [11.0, 7.0, -17.0, -25.0, 33.0], [27.0, -19.0, 14.0, -11.0, 20.0]], [[-10.0, -14.0, 14.0, 14.0, -8.0], [33.0, -8.0, -2.0, -8.0, -22.0], [7.0, -26.0, 26.0, 15.0, 31.0], [-21.0, -13.0, 3.0, -7.0, 35.0]]], [[[-18.0, -14.0, 22.0, 2.0, -8.0], [30.0, -2.0, -16.0, -6.0, -26.0], [18.0, -46.0, 34.0, 18.0, 38.0], [-30.0, -18.0, -4.0, -18.0, 48.0]], [[-19.0, -29.0, 33.0, 13.0, 2.0], [19.0, -5.0, -4.0, 3.0, -39.0], [-5.0, -21.0, 35.0, 29.0, 17.0], [-33.0, -3.0, 2.0, 5.0, 24.0]], [[26.0, 14.0, -38.0, 36.0, -18.0], [43.0, -22.0, 34.0, -24.0, 22.0], [-5.0, 26.0, -18.0, -19.0, 19.0], [27.0, -9.0, 19.0, 5.0, 3.0]], [[13.0, -21.0, 1.0, 37.0, 14.0], [-10.0, -15.0, 42.0, 9.0, -11.0], [-52.0, 71.0, -13.0, 10.0, -42.0], [12.0, 30.0, 21.0, 52.0, -57.0]], [[-10.0, 18.0, -6.0, -17.0, -25.0], [44.0, 3.0, -32.0, -22.0, 6.0], [55.0, -78.0, 22.0, -8.0, 67.0], [-15.0, -41.0, -13.0, -55.0, 79.0]]], [[[-17.0, -31.0, 35.0, 9.0, 12.0], [-5.0, -1.0, 0.0, 13.0, -37.0], [-19.0, 1.0, 25.0, 29.0, -9.0], [-27.0, 11.0, 2.0, 19.0, -4.0]], [[18.0, 30.0, -30.0, -24.0, 6.0], [-42.0, 12.0, 0.0, 6.0, 42.0], [0.0, 30.0, -42.0, -30.0, -36.0], [36.0, 12.0, -6.0, 0.0, -42.0]], [[-1.0, 9.0, -9.0, 4.0, -19.0], [41.0, -6.0, -8.0, -19.0, 3.0], [28.0, -39.0, 13.0, -5.0, 46.0], [-6.0, -26.0, -1.0, -28.0, 49.0]], [[8.0, -32.0, 4.0, 69.0, -9.0], [62.0, -35.0, 48.0, -16.0, -32.0], [-35.0, 32.0, 20.0, 22.0, 21.0], [-9.0, 1.0, 31.0, 35.0, 7.0]], [[-39.0, -49.0, 65.0, 4.0, 11.0], [9.0, 2.0, -20.0, 15.0, -67.0], [-6.0, -41.0, 59.0, 51.0, 16.0], [-60.0, 0.0, -5.0, 6.0, 33.0]]]])+(1.-msk_ref)*numpy.array([[[[-20.0, -20.0, 32.0, -9.0, 9.0], [-10.0, 7.0, -16.0, 12.0, -28.0], [-1.0, -20.0, 24.0, 22.0, -1.0], [-27.0, 3.0, -7.0, 1.0, 9.0]], [[2.0, -34.0, 14.0, 53.0, 1.0], [35.0, -25.0, 38.0, -4.0, -34.0], [-38.0, 34.0, 18.0, 25.0, 2.0], [-12.0, 10.0, 24.0, 38.0, -8.0]], [[-20.0, -52.0, 52.0, 22.0, 26.0], [-21.0, -4.0, 14.0, 26.0, -56.0], [-49.0, 32.0, 28.0, 45.0, -37.0], [-33.0, 31.0, 9.0, 49.0, -35.0]], [[18.0, -18.0, -10.0, 55.0, -1.0], [29.0, -27.0, 50.0, -8.0, -10.0], [-42.0, 58.0, -6.0, 7.0, -10.0], [12.0, 14.0, 28.0, 42.0, -28.0]], [[-19.0, -13.0, 33.0, -35.0, 26.0], [-63.0, 23.0, -24.0, 31.0, -15.0], [-11.0, 3.0, 3.0, 15.0, -45.0], [-15.0, 23.0, -16.0, 11.0, -34.0]]], [[[1.0, -9.0, 9.0, -4.0, 19.0], [-41.0, 6.0, 8.0, 19.0, -3.0], [-28.0, 39.0, -13.0, 5.0, -46.0], [6.0, 26.0, 1.0, 28.0, -49.0]], [[-3.0, 19.0, -11.0, -16.0, -17.0], [24.0, 4.0, -22.0, -15.0, 13.0], [39.0, -49.0, 7.0, -12.0, 41.0], [-3.0, -27.0, -10.0, -39.0, 48.0]], [[38.0, 50.0, -74.0, 24.0, -42.0], [73.0, -22.0, 22.0, -48.0, 58.0], [37.0, -10.0, -30.0, -49.0, 61.0], [45.0, -39.0, 13.0, -37.0, 45.0]], [[51.0, 69.0, -89.0, -7.0, -20.0], [-2.0, -3.0, 22.0, -25.0, 91.0], [18.0, 41.0, -75.0, -70.0, -8.0], [78.0, -8.0, 5.0, -18.0, -29.0]], [[35.0, 45.0, -65.0, 15.0, -30.0], [45.0, -15.0, 20.0, -35.0, 55.0], [25.0, 5.0, -35.0, -45.0, 35.0], [45.0, -25.0, 10.0, -25.0, 20.0]]], [[[6.0, -6.0, -10.0, 40.0, -22.0], [68.0, -24.0, 20.0, -26.0, -10.0], [6.0, -14.0, 18.0, 4.0, 50.0], [-6.0, -22.0, 16.0, -6.0, 44.0]], [[-25.0, -55.0, 59.0, 17.0, 28.0], [-25.0, -1.0, 8.0, 29.0, -61.0], [-47.0, 25.0, 33.0, 49.0, -37.0], [-39.0, 31.0, 6.0, 47.0, -32.0]], [[-13.0, -27.0, 35.0, -10.0, 31.0], [-59.0, 12.0, 0.0, 33.0, -25.0], [-38.0, 37.0, 1.0, 23.0, -60.0], [-12.0, 36.0, -3.0, 38.0, -57.0]], [[-15.0, -25.0, 25.0, 20.0, -5.0], [35.0, -10.0, 0.0, -5.0, -35.0], [0.0, -25.0, 35.0, 25.0, 30.0], [-30.0, -10.0, 5.0, 0.0, 35.0]], [[-20.0, -20.0, 36.0, -22.0, 22.0], [-45.0, 16.0, -18.0, 26.0, -24.0], [-13.0, 0.0, 12.0, 21.0, -33.0], [-21.0, 19.0, -11.0, 13.0, -23.0]]], [[[19.0, -3.0, -21.0, 44.0, -11.0], [40.0, -24.0, 38.0, -17.0, 3.0], [-19.0, 33.0, -7.0, -4.0, 11.0], [15.0, -1.0, 22.0, 19.0, -4.0]], [[-21.0, -51.0, 51.0, 24.0, 21.0], [-9.0, -6.0, 12.0, 21.0, -57.0], [-42.0, 21.0, 33.0, 45.0, -24.0], [-36.0, 24.0, 9.0, 42.0, -21.0]], [[-37.0, -35.0, 55.0, -9.0, 6.0], [8.0, 7.0, -30.0, 11.0, -53.0], [10.0, -55.0, 53.0, 40.0, 24.0], [-54.0, -8.0, -11.0, -10.0, 43.0]], [[-41.0, -63.0, 79.0, 4.0, 29.0], [-25.0, 6.0, -12.0, 33.0, -77.0], [-34.0, -7.0, 53.0, 61.0, -24.0], [-60.0, 24.0, -3.0, 34.0, -9.0]], [[13.0, -21.0, 1.0, 37.0, 14.0], [-10.0, -15.0, 42.0, 9.0, -11.0], [-52.0, 71.0, -13.0, 10.0, -42.0], [12.0, 30.0, 21.0, 52.0, -57.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[1.0, 0.0, 0.0], [-5.0, -7.0, 0.0]], [[-5.0, -3.0, 2.0], [2.0, -5.0, -6.0]], [[-6.0, -2.0, 2.0], [5.0, 1.0, -2.0]], [[-3.0, 3.0, 6.0], [-5.0, 6.0, 6.0]], [[-7.0, -4.0, 5.0], [3.0, 0.0, -4.0]]], [[[4.0, 5.0, -5.0], [7.0, 4.0, 1.0]], [[-4.0, -6.0, -1.0], [-6.0, 1.0, 2.0]], [[7.0, 7.0, 7.0], [-6.0, -1.0, 7.0]], [[1.0, 6.0, -7.0], [3.0, -1.0, 7.0]], [[7.0, 3.0, 3.0], [-6.0, -1.0, 0.0]]], [[[-3.0, 5.0, 1.0], [-6.0, -5.0, 3.0]], [[-2.0, -4.0, 7.0], [0.0, -7.0, 1.0]], [[0.0, 2.0, -6.0], [3.0, -1.0, -2.0]], [[-2.0, 4.0, 7.0], [-3.0, 0.0, 0.0]], [[-1.0, 0.0, -1.0], [0.0, 0.0, -7.0]]], [[[4.0, 7.0, -7.0], [3.0, 2.0, 5.0]], [[-7.0, 5.0, 6.0], [6.0, -2.0, -4.0]], [[1.0, 5.0, 5.0], [6.0, 0.0, 0.0]], [[2.0, -4.0, -6.0], [-7.0, -2.0, 4.0]], [[0.0, 0.0, 3.0], [-3.0, 4.0, -6.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 5.0, -6.0], [6.0, -6.0, 3.0]], [[6.0, 0.0, 6.0], [1.0, -3.0, -1.0]], [[1.0, 0.0, -4.0], [-5.0, -3.0, -6.0]], [[7.0, 5.0, 4.0], [-5.0, -1.0, 5.0]], [[-3.0, 4.0, -5.0], [-3.0, -2.0, -3.0]]], [[[0.0, 3.0, -7.0], [-7.0, -2.0, 2.0]], [[-1.0, -6.0, -4.0], [7.0, -5.0, 6.0]], [[5.0, 0.0, 1.0], [2.0, 4.0, 1.0]], [[1.0, 0.0, 6.0], [-6.0, 0.0, 1.0]], [[6.0, 6.0, -3.0], [0.0, 0.0, 2.0]]], [[[5.0, -6.0, -4.0], [-4.0, 4.0, -5.0]], [[0.0, 6.0, 4.0], [-5.0, 3.0, 3.0]], [[2.0, 1.0, -1.0], [1.0, 4.0, -5.0]], [[6.0, 2.0, -5.0], [1.0, -6.0, -1.0]], [[6.0, 5.0, -6.0], [6.0, -1.0, -4.0]]], [[[5.0, -7.0, 1.0], [6.0, -7.0, -4.0]], [[2.0, 2.0, -4.0], [0.0, 6.0, 0.0]], [[-4.0, -5.0, -1.0], [4.0, 2.0, -2.0]], [[5.0, -4.0, 3.0], [7.0, -1.0, -4.0]], [[7.0, 4.0, 0.0], [2.0, 5.0, -2.0]]]])
      arg1=Data(numpy.array([[[[-4.0, -7.0, 7.0], [-2.0, 0.0, 4.0]], [[-3.0, -1.0, -5.0], [-5.0, -5.0, 6.0]], [[4.0, -3.0, 3.0], [4.0, 1.0, 6.0]], [[1.0, 4.0, -1.0], [5.0, 5.0, 1.0]], [[-1.0, 1.0, 5.0], [5.0, -1.0, 5.0]]], [[[4.0, -2.0, -6.0], [0.0, 3.0, -2.0]], [[4.0, 2.0, 1.0], [-3.0, 0.0, 6.0]], [[4.0, -4.0, -7.0], [6.0, -2.0, -4.0]], [[-3.0, -3.0, -2.0], [-2.0, 1.0, -4.0]], [[-3.0, -4.0, -5.0], [-4.0, 6.0, -2.0]]], [[[4.0, -4.0, 5.0], [-7.0, 4.0, -2.0]], [[-5.0, 0.0, -6.0], [3.0, -6.0, 2.0]], [[7.0, -5.0, 1.0], [0.0, -6.0, 1.0]], [[-5.0, 0.0, -3.0], [3.0, -6.0, 1.0]], [[5.0, 7.0, 1.0], [-2.0, -2.0, -6.0]]], [[[5.0, 5.0, 7.0], [-4.0, -5.0, -3.0]], [[-2.0, 0.0, 0.0], [0.0, -7.0, 4.0]], [[-2.0, -2.0, 0.0], [-4.0, 4.0, 0.0]], [[2.0, -2.0, -1.0], [-3.0, -3.0, 6.0]], [[4.0, -3.0, -6.0], [6.0, 3.0, 6.0]]]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, 57.0, -23.0, -59.0, -19.0], [-17.0, 19.0, -12.0, 0.0, -25.0], [11.0, 22.0, 49.0, 22.0, 29.0], [60.0, 47.0, -10.0, 38.0, -47.0]], [[27.0, -13.0, -38.0, -40.0, -3.0], [-29.0, -66.0, 24.0, 35.0, -9.0], [-20.0, 37.0, 6.0, 49.0, -2.0], [9.0, 21.0, -12.0, -33.0, -62.0]], [[34.0, -32.0, -3.0, 12.0, 28.0], [-25.0, -53.0, 6.0, 19.0, 6.0], [-33.0, 23.0, -38.0, 31.0, -42.0], [-45.0, -3.0, 0.0, -40.0, -9.0]], [[67.0, 7.0, 19.0, 14.0, 35.0], [-48.0, 51.0, -132.0, -20.0, 11.0], [53.0, -60.0, -60.0, -48.0, -26.0], [14.0, -12.0, 44.0, 15.0, -33.0]], [[69.0, -39.0, -13.0, -17.0, 23.0], [-42.0, -64.0, -13.0, 33.0, 8.0], [0.0, 6.0, -28.0, 25.0, -40.0], [-20.0, -2.0, 10.0, -44.0, -52.0]]], [[[-96.0, -41.0, 24.0, 85.0, 12.0], [46.0, 6.0, 61.0, -31.0, -13.0], [-64.0, 9.0, -25.0, -7.0, 22.0], [-41.0, -32.0, -30.0, -24.0, 91.0]], [[71.0, 60.0, -12.0, -50.0, -28.0], [1.0, 1.0, -31.0, 37.0, 67.0], [45.0, 6.0, -3.0, 1.0, -65.0], [-44.0, 9.0, 48.0, 32.0, -13.0]], [[12.0, 14.0, 45.0, 0.0, 41.0], [-45.0, 109.0, -111.0, -73.0, -80.0], [59.0, -75.0, 34.0, -61.0, 63.0], [127.0, 21.0, -8.0, 56.0, -32.0]], [[-73.0, 58.0, 18.0, 49.0, 21.0], [17.0, 42.0, 21.0, -42.0, -24.0], [-94.0, 66.0, -17.0, 38.0, -6.0], [-42.0, 33.0, -30.0, 33.0, 85.0]], [[-16.0, -4.0, 3.0, -19.0, -18.0], [1.0, 55.0, -39.0, -25.0, -30.0], [69.0, -65.0, 43.0, -56.0, 73.0], [100.0, -7.0, 0.0, 26.0, -38.0]]], [[[8.0, 72.0, -35.0, -36.0, 3.0], [-49.0, 35.0, -77.0, -13.0, -28.0], [-11.0, 27.0, -12.0, 27.0, 25.0], [57.0, 53.0, 0.0, 34.0, -66.0]], [[89.0, 16.0, 24.0, -59.0, 45.0], [-65.0, -3.0, -31.0, -7.0, -57.0], [13.0, 12.0, 56.0, 32.0, -23.0], [51.0, 57.0, -16.0, 24.0, -53.0]], [[-70.0, 6.0, -25.0, 22.0, -22.0], [33.0, -23.0, 62.0, 7.0, 8.0], [-59.0, 47.0, -12.0, 31.0, 16.0], [-33.0, -1.0, -20.0, -16.0, 33.0]], [[35.0, -18.0, -11.0, -8.0, 26.0], [-58.0, 16.0, -91.0, -14.0, -33.0], [32.0, -41.0, -27.0, -20.0, 31.0], [71.0, 4.0, 8.0, -10.0, -80.0]], [[-31.0, -34.0, -49.0, -7.0, -39.0], [16.0, -47.0, 31.0, 33.0, 22.0], [5.0, -3.0, -15.0, 1.0, 36.0], [9.0, -26.0, 2.0, -43.0, -40.0]]], [[[-100.0, 21.0, 18.0, 69.0, 6.0], [40.0, 44.0, 31.0, -43.0, -15.0], [-70.0, 29.0, -21.0, 3.0, 22.0], [-31.0, -2.0, -26.0, 16.0, 91.0]], [[7.0, -58.0, -27.0, 23.0, 54.0], [-72.0, -54.0, -34.0, -4.0, -57.0], [-60.0, 21.0, -60.0, 43.0, 22.0], [30.0, 12.0, -28.0, -66.0, -73.0]], [[-16.0, -63.0, 28.0, 46.0, 59.0], [-36.0, 1.0, -15.0, -40.0, -72.0], [-33.0, -17.0, -13.0, -2.0, 33.0], [41.0, -2.0, -36.0, -31.0, -5.0]], [[8.0, 97.0, -4.0, -49.0, -49.0], [38.0, 39.0, 12.0, 14.0, 48.0], [27.0, 25.0, 44.0, 3.0, -30.0], [-26.0, 26.0, 24.0, 69.0, 32.0]], [[3.0, -56.0, -35.0, -4.0, -34.0], [6.0, -24.0, -23.0, 28.0, 33.0], [64.0, -63.0, -27.0, -48.0, 37.0], [31.0, -52.0, 28.0, -42.0, -60.0]]]])+(1.-msk_ref)*numpy.array([[[[-81.0, 40.0, 7.0, 30.0, 25.0], [6.0, 8.0, 62.0, -36.0, -59.0], [-118.0, 91.0, 15.0, 70.0, 16.0], [-15.0, 52.0, -60.0, 16.0, 61.0]], [[12.0, -44.0, 37.0, -11.0, 27.0], [-19.0, 21.0, -2.0, -31.0, -68.0], [37.0, -47.0, 65.0, -28.0, 46.0], [86.0, 5.0, -28.0, 6.0, -21.0]], [[-46.0, 21.0, -67.0, -41.0, -73.0], [31.0, -21.0, 32.0, 36.0, 31.0], [19.0, 10.0, 15.0, 4.0, 53.0], [30.0, -5.0, 6.0, -6.0, -47.0]], [[-5.0, 14.0, 34.0, -2.0, 19.0], [-19.0, 87.0, -68.0, -55.0, -57.0], [49.0, -58.0, 39.0, -51.0, 56.0], [98.0, 13.0, -8.0, 48.0, -14.0]], [[-57.0, 37.0, -71.0, -10.0, -46.0], [10.0, -18.0, 5.0, 23.0, 24.0], [-34.0, 42.0, -37.0, 30.0, 36.0], [1.0, 8.0, 2.0, -12.0, -36.0]]], [[[-48.0, 89.0, -48.0, -24.0, -55.0], [26.0, 32.0, -9.0, 9.0, 35.0], [-10.0, 37.0, -8.0, 14.0, 20.0], [-2.0, 22.0, 14.0, 40.0, -3.0]], [[28.0, 55.0, 61.0, -5.0, 45.0], [5.0, -5.0, 76.0, -14.0, -23.0], [-81.0, 92.0, 55.0, 74.0, -91.0], [-84.0, 61.0, -34.0, 44.0, 101.0]], [[-13.0, -44.0, 41.0, 35.0, 11.0], [24.0, 21.0, 13.0, -21.0, -6.0], [25.0, -47.0, 13.0, -45.0, 8.0], [1.0, -34.0, -2.0, -3.0, 44.0]], [[54.0, 3.0, 4.0, -34.0, 4.0], [-34.0, 34.0, -78.0, -7.0, -11.0], [74.0, -57.0, 14.0, -40.0, 17.0], [68.0, 2.0, 22.0, 20.0, -62.0]], [[-79.0, 3.0, 9.0, 35.0, -5.0], [26.0, 45.0, 13.0, -38.0, -31.0], [-19.0, -8.0, 11.0, -19.0, 57.0], [33.0, -4.0, -24.0, 15.0, 36.0]]], [[[-18.0, -19.0, -16.0, -20.0, -80.0], [78.0, -14.0, 60.0, 43.0, 79.0], [78.0, -47.0, 32.0, -54.0, 9.0], [-22.0, -58.0, 34.0, -4.0, 20.0]], [[8.0, 2.0, -5.0, 13.0, 13.0], [-33.0, 49.0, -100.0, -25.0, -12.0], [37.0, -51.0, -41.0, -42.0, 32.0], [54.0, -9.0, 20.0, 8.0, -45.0]], [[-44.0, -57.0, -20.0, 27.0, -30.0], [34.0, -24.0, 29.0, 15.0, 25.0], [18.0, -35.0, -21.0, -33.0, 36.0], [-1.0, -52.0, 6.0, -42.0, -1.0]], [[-79.0, 24.0, -5.0, -7.0, -23.0], [34.0, 14.0, 73.0, -18.0, -39.0], [-38.0, 37.0, 62.0, 23.0, 55.0], [34.0, 26.0, -44.0, 22.0, 30.0]], [[-129.0, -42.0, -10.0, 53.0, -20.0], [55.0, -14.0, 100.0, -18.0, -30.0], [-64.0, 22.0, 13.0, 8.0, 73.0], [6.0, -21.0, -50.0, -31.0, 54.0]]], [[[8.0, -32.0, 37.0, -33.0, 10.0], [15.0, -35.0, 107.0, 1.0, -50.0], [-9.0, 21.0, 109.0, 28.0, 3.0], [20.0, 23.0, -48.0, 2.0, 26.0]], [[-50.0, -18.0, -4.0, 44.0, -26.0], [46.0, 8.0, 16.0, 2.0, 42.0], [4.0, -22.0, -36.0, -34.0, 8.0], [-38.0, -46.0, 16.0, -14.0, 44.0]], [[28.0, -20.0, 2.0, 5.0, 2.0], [10.0, -51.0, 39.0, 31.0, 37.0], [-17.0, 22.0, -18.0, 21.0, -56.0], [-72.0, -14.0, 10.0, -27.0, 23.0]], [[-1.0, -80.0, 44.0, 12.0, 22.0], [15.0, -30.0, 75.0, -8.0, -40.0], [6.0, -24.0, 60.0, -11.0, 12.0], [15.0, -19.0, -34.0, -27.0, 29.0]], [[-68.0, -72.0, 17.0, 56.0, -8.0], [39.0, 18.0, 22.0, -24.0, -11.0], [22.0, -63.0, -3.0, -61.0, 61.0], [28.0, -57.0, -10.0, -27.0, 31.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-3.0, -2.0, 1.0, 5.0, 2.0], [-7.0, -1.0, -5.0, -7.0, 0.0], [3.0, -7.0, 1.0, 5.0, 6.0], [5.0, -4.0, -3.0, -7.0, -5.0]])+(1.-msk_arg0)*numpy.array([[5.0, -7.0, 5.0, -1.0, 2.0], [0.0, -3.0, 0.0, -2.0, 3.0], [-2.0, 3.0, 5.0, 0.0, -2.0], [-3.0, -3.0, 0.0, -5.0, 1.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(-3.0)+(1-msk_arg1)*(-5.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[9.0, 6.0, -3.0, -15.0, -6.0], [21.0, 3.0, 15.0, 21.0, -0.0], [-9.0, 21.0, -3.0, -15.0, -18.0], [-15.0, 12.0, 9.0, 21.0, 15.0]])+(1.-msk_ref)*numpy.array([[-25.0, 35.0, -25.0, 5.0, -10.0], [-0.0, 15.0, -0.0, 10.0, -15.0], [10.0, -15.0, -25.0, -0.0, 10.0], [15.0, 15.0, -0.0, 25.0, -5.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[4.0, -6.0, 2.0], [2.0, -7.0, 2.0], [0.0, 7.0, -1.0], [5.0, 6.0, -6.0], [-3.0, -5.0, 2.0]], [[3.0, -7.0, -4.0], [3.0, 3.0, -3.0], [0.0, 7.0, -7.0], [-5.0, -7.0, -7.0], [6.0, -1.0, 3.0]], [[-1.0, -2.0, -5.0], [5.0, 6.0, -6.0], [-3.0, -1.0, 4.0], [7.0, 0.0, -4.0], [-1.0, 0.0, -3.0]], [[6.0, -2.0, 2.0], [-1.0, 1.0, 0.0], [-2.0, -5.0, 0.0], [1.0, 0.0, -4.0], [-2.0, 0.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[4.0, -6.0, -3.0], [6.0, -1.0, 0.0], [6.0, 7.0, 0.0], [-7.0, 1.0, -6.0], [-6.0, 0.0, -3.0]], [[0.0, -2.0, -7.0], [0.0, 2.0, 1.0], [-4.0, 7.0, 0.0], [6.0, 6.0, -7.0], [5.0, 0.0, -1.0]], [[-4.0, 7.0, -5.0], [-5.0, -3.0, -7.0], [-5.0, -4.0, -4.0], [-6.0, 7.0, 3.0], [0.0, 5.0, -5.0]], [[3.0, 4.0, 1.0], [3.0, 4.0, -3.0], [6.0, 6.0, -6.0], [5.0, 1.0, -3.0], [7.0, 4.0, 3.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-4.0, 0.0, 0.0])+(1.-msk_arg1)*numpy.array([6.0, 7.0, -7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[-16.0, -8.0, 0.0, -20.0, 12.0], [-12.0, -12.0, 0.0, 20.0, -24.0], [4.0, -20.0, 12.0, -28.0, 4.0], [-24.0, 4.0, 8.0, -4.0, 8.0]])+(1.-msk_ref)*numpy.array([[3.0, 29.0, 85.0, 7.0, -15.0], [35.0, 7.0, 25.0, 127.0, 37.0], [60.0, -2.0, -30.0, -8.0, 70.0], [39.0, 67.0, 120.0, 58.0, 49.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank2_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, 0.0, 1.0], [-4.0, -2.0, -2.0]], [[-6.0, 5.0, -5.0], [-1.0, 7.0, 2.0]], [[4.0, 2.0, -2.0], [-4.0, 6.0, 6.0]], [[2.0, 7.0, 5.0], [-5.0, -3.0, -4.0]], [[-7.0, -5.0, 2.0], [3.0, -1.0, 5.0]]], [[[5.0, 5.0, 0.0], [-5.0, 5.0, -1.0]], [[4.0, 3.0, 0.0], [5.0, 0.0, 0.0]], [[-4.0, -2.0, -3.0], [4.0, -3.0, 5.0]], [[2.0, -3.0, 1.0], [-4.0, -2.0, 5.0]], [[3.0, -7.0, -3.0], [0.0, -1.0, 4.0]]], [[[-5.0, -5.0, -6.0], [0.0, -6.0, -6.0]], [[7.0, 2.0, -5.0], [-6.0, 0.0, -2.0]], [[0.0, 7.0, -5.0], [-3.0, 2.0, 5.0]], [[0.0, -3.0, 1.0], [0.0, -1.0, 5.0]], [[-7.0, -6.0, 7.0], [0.0, 0.0, -4.0]]], [[[4.0, 5.0, 6.0], [-6.0, -4.0, -7.0]], [[-2.0, -4.0, 1.0], [-6.0, -7.0, 3.0]], [[6.0, 2.0, 7.0], [-1.0, 0.0, 1.0]], [[0.0, 0.0, -1.0], [-3.0, 0.0, 7.0]], [[-2.0, 2.0, 4.0], [1.0, -5.0, 7.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 3.0, -5.0], [7.0, 0.0, 4.0]], [[7.0, -6.0, 0.0], [-5.0, 4.0, 3.0]], [[0.0, 0.0, 0.0], [6.0, 7.0, -6.0]], [[-4.0, 5.0, -5.0], [-7.0, 0.0, 6.0]], [[1.0, -4.0, -3.0], [-2.0, 5.0, -2.0]]], [[[1.0, 5.0, -6.0], [3.0, -2.0, -3.0]], [[7.0, 3.0, 0.0], [2.0, 6.0, -6.0]], [[2.0, -4.0, -3.0], [7.0, 7.0, -7.0]], [[7.0, -7.0, 0.0], [-1.0, 0.0, 7.0]], [[0.0, 4.0, 6.0], [0.0, -4.0, -2.0]]], [[[5.0, 0.0, 1.0], [-5.0, 3.0, -3.0]], [[-3.0, 4.0, 0.0], [1.0, 6.0, -3.0]], [[-1.0, 5.0, 4.0], [0.0, 1.0, 7.0]], [[3.0, -6.0, 4.0], [-5.0, -5.0, -5.0]], [[-3.0, -1.0, -2.0], [4.0, 0.0, 0.0]]], [[[-3.0, -3.0, -2.0], [-6.0, 1.0, -3.0]], [[-3.0, 4.0, 1.0], [-2.0, 4.0, 2.0]], [[-1.0, 3.0, 5.0], [2.0, 7.0, 4.0]], [[-4.0, 0.0, -1.0], [-2.0, 0.0, 3.0]], [[-6.0, -3.0, -1.0], [6.0, -6.0, -5.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, 0.0, 0.0], [5.0, 0.0, 3.0]])+(1.-msk_arg1)*numpy.array([[7.0, 3.0, -5.0], [6.0, -5.0, -2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[16.0, -41.0, 26.0, -23.0, -19.0], [7.0, 53.0, 7.0, 9.0, 33.0], [-53.0, 13.0, 0.0, 15.0, -61.0], [-23.0, -35.0, 40.0, 6.0, 12.0]])+(1.-msk_ref)*numpy.array([[75.0, -25.0, 13.0, -42.0, -23.0], [86.0, 52.0, 38.0, 8.0, 6.0], [-9.0, -27.0, -31.0, -12.0, 10.0], [-55.0, -50.0, -54.0, -41.0, 30.0]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[-4.0, 6.0, 6.0, 7.0, 7.0], [3.0, -4.0, 2.0, -6.0, -4.0], [2.0, -7.0, 4.0, -5.0, 0.0], [3.0, 6.0, -3.0, 0.0, 4.0]])+(1.-msk_arg0)*numpy.array([[5.0, 0.0, -5.0, -1.0, -1.0], [-1.0, 0.0, 1.0, -3.0, 6.0], [1.0, -5.0, 1.0, -6.0, 6.0], [0.0, -1.0, -2.0, 5.0, 4.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([7.0, 7.0])+(1.-msk_arg1)*numpy.array([4.0, 7.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-28.0, -28.0], [42.0, 42.0], [42.0, 42.0], [49.0, 49.0], [49.0, 49.0]], [[21.0, 21.0], [-28.0, -28.0], [14.0, 14.0], [-42.0, -42.0], [-28.0, -28.0]], [[14.0, 14.0], [-49.0, -49.0], [28.0, 28.0], [-35.0, -35.0], [0.0, 0.0]], [[21.0, 21.0], [42.0, 42.0], [-21.0, -21.0], [0.0, 0.0], [28.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[20.0, 35.0], [0.0, 0.0], [-20.0, -35.0], [-4.0, -7.0], [-4.0, -7.0]], [[-4.0, -7.0], [0.0, 0.0], [4.0, 7.0], [-12.0, -21.0], [24.0, 42.0]], [[4.0, 7.0], [-20.0, -35.0], [4.0, 7.0], [-24.0, -42.0], [24.0, 42.0]], [[0.0, 0.0], [-4.0, -7.0], [-8.0, -14.0], [20.0, 35.0], [16.0, 28.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-3.0, -1.0, 6.0], [3.0, 5.0, -6.0], [7.0, 3.0, 5.0], [0.0, -2.0, -6.0], [7.0, -5.0, 7.0]], [[-7.0, 7.0, 0.0], [2.0, 0.0, -4.0], [-3.0, 2.0, 0.0], [7.0, -1.0, 0.0], [-4.0, -4.0, -5.0]], [[7.0, -2.0, 1.0], [0.0, 3.0, -7.0], [-5.0, 3.0, 7.0], [-3.0, -1.0, 0.0], [0.0, -5.0, -2.0]], [[-2.0, -5.0, 3.0], [-4.0, -6.0, 6.0], [-1.0, 0.0, 0.0], [-5.0, -1.0, -1.0], [7.0, 3.0, -1.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, 2.0, -7.0], [0.0, -1.0, 3.0], [4.0, -4.0, 1.0], [-6.0, -5.0, 0.0], [5.0, -4.0, -5.0]], [[-5.0, -6.0, -7.0], [-7.0, -4.0, 4.0], [3.0, 0.0, -5.0], [-6.0, 1.0, -5.0], [1.0, 4.0, 6.0]], [[5.0, -7.0, 2.0], [3.0, 1.0, 4.0], [-1.0, -7.0, -6.0], [4.0, -7.0, -3.0], [7.0, 0.0, -2.0]], [[-3.0, -5.0, 7.0], [5.0, -1.0, -7.0], [0.0, -2.0, 2.0], [4.0, 6.0, 7.0], [3.0, -3.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[6.0, 3.0, -3.0], [6.0, 1.0, 5.0]])+(1.-msk_arg1)*numpy.array([[-3.0, -3.0, 7.0], [-1.0, 6.0, 7.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-39.0, 11.0], [51.0, -7.0], [36.0, 70.0], [12.0, -32.0], [6.0, 72.0]], [[-21.0, -35.0], [24.0, -8.0], [-12.0, -16.0], [39.0, 41.0], [-21.0, -53.0]], [[33.0, 45.0], [30.0, -32.0], [-42.0, 8.0], [-21.0, -19.0], [-9.0, -15.0]], [[-36.0, -2.0], [-60.0, 0.0], [-6.0, -6.0], [-30.0, -36.0], [54.0, 40.0]]])+(1.-msk_ref)*numpy.array([[[-61.0, -39.0], [24.0, 15.0], [7.0, -21.0], [33.0, -24.0], [-38.0, -64.0]], [[-16.0, -80.0], [61.0, 11.0], [-44.0, -38.0], [-20.0, -23.0], [27.0, 65.0]], [[20.0, -33.0], [16.0, 31.0], [-18.0, -83.0], [-12.0, -67.0], [-35.0, -21.0]], [[73.0, 22.0], [-61.0, -60.0], [20.0, 2.0], [19.0, 81.0], [-42.0, -63.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank3_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, 5.0, 7.0], [-4.0, -1.0, 4.0]], [[5.0, 2.0, -2.0], [5.0, 1.0, 1.0]], [[4.0, -5.0, 3.0], [1.0, -4.0, -7.0]], [[5.0, 0.0, 4.0], [-6.0, 0.0, -2.0]], [[-3.0, 7.0, -4.0], [3.0, 0.0, 0.0]]], [[[5.0, 7.0, -7.0], [-7.0, -1.0, 5.0]], [[1.0, 0.0, -3.0], [5.0, -6.0, 7.0]], [[-3.0, -7.0, 4.0], [-2.0, 0.0, 0.0]], [[3.0, -1.0, 0.0], [2.0, 3.0, -2.0]], [[3.0, -7.0, -7.0], [-5.0, 4.0, 3.0]]], [[[6.0, -4.0, -4.0], [-5.0, -6.0, -4.0]], [[2.0, 2.0, 1.0], [-4.0, -1.0, 3.0]], [[2.0, 0.0, 4.0], [-7.0, 0.0, 0.0]], [[-6.0, 5.0, 0.0], [-1.0, 1.0, -5.0]], [[-5.0, -2.0, 0.0], [-6.0, -5.0, 4.0]]], [[[7.0, 1.0, 3.0], [6.0, 2.0, -1.0]], [[2.0, 0.0, 0.0], [3.0, 0.0, 1.0]], [[-1.0, 7.0, -1.0], [2.0, -3.0, -4.0]], [[7.0, 6.0, 4.0], [-6.0, 2.0, 7.0]], [[-4.0, 0.0, -7.0], [6.0, -7.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, -2.0, -2.0], [2.0, -5.0, 7.0]], [[-6.0, -6.0, 3.0], [1.0, 1.0, -7.0]], [[4.0, -3.0, 4.0], [-7.0, 3.0, 1.0]], [[2.0, 3.0, -1.0], [-6.0, 1.0, 1.0]], [[3.0, 1.0, 4.0], [3.0, -4.0, 3.0]]], [[[0.0, 7.0, -2.0], [-4.0, 6.0, 0.0]], [[4.0, -6.0, 0.0], [-6.0, 3.0, -4.0]], [[0.0, 5.0, -6.0], [0.0, -1.0, 6.0]], [[-5.0, -5.0, -5.0], [6.0, 0.0, -7.0]], [[-7.0, -7.0, 0.0], [-6.0, 3.0, -1.0]]], [[[0.0, -4.0, -6.0], [-7.0, 3.0, 6.0]], [[0.0, 4.0, 0.0], [-4.0, 7.0, 5.0]], [[4.0, -2.0, 7.0], [1.0, 0.0, 6.0]], [[-2.0, 5.0, -4.0], [0.0, -2.0, -2.0]], [[6.0, 2.0, -4.0], [-7.0, -5.0, -1.0]]], [[[0.0, -2.0, 1.0], [3.0, -5.0, 4.0]], [[-7.0, 0.0, 0.0], [-5.0, 6.0, -2.0]], [[0.0, 0.0, 3.0], [-6.0, 1.0, -5.0]], [[-5.0, -1.0, 7.0], [0.0, 5.0, -1.0]], [[1.0, -3.0, -6.0], [-7.0, -6.0, -3.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[-1.0, 3.0, -2.0], [6.0, 1.0, 5.0]], [[1.0, 6.0, -3.0], [2.0, -1.0, 0.0]]])+(1.-msk_arg1)*numpy.array([[[-1.0, 0.0, 0.0], [2.0, -2.0, 6.0]], [[5.0, -7.0, 0.0], [0.0, 4.0, 7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 4.0], [41.0, 32.0], [-58.0, -29.0], [-59.0, -19.0], [50.0, 57.0]], [[12.0, 55.0], [64.0, 26.0], [-38.0, -61.0], [-1.0, -2.0], [-21.0, -32.0]], [[-66.0, -10.0], [-8.0, 4.0], [-52.0, -24.0], [-9.0, 21.0], [-22.0, -24.0]], [[23.0, 14.0], [21.0, 8.0], [13.0, 51.0], [4.0, 17.0], [47.0, 36.0]]])+(1.-msk_ref)*numpy.array([[[62.0, 13.0], [-36.0, -33.0], [-18.0, 60.0], [-10.0, 0.0], [29.0, 13.0]], [[-20.0, -25.0], [-46.0, 46.0], [38.0, 3.0], [-25.0, -39.0], [-17.0, 19.0]], [[16.0, 82.0], [8.0, 35.0], [34.0, 76.0], [-6.0, -67.0], [-16.0, -11.0]], [[40.0, 22.0], [-27.0, -25.0], [-44.0, -31.0], [-11.0, -5.0], [-21.0, -19.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank2_expandedData_rank2_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[7.0, 0.0, 5.0, 1.0, -2.0], [-2.0, -6.0, 0.0, 0.0, 2.0], [-6.0, -5.0, 0.0, -5.0, -3.0], [-3.0, -5.0, -6.0, -5.0, -7.0]])+(1.-msk_arg0)*numpy.array([[0.0, 6.0, 7.0, 5.0, -4.0], [0.0, 7.0, 2.0, 6.0, -3.0], [3.0, 5.0, 2.0, -6.0, -2.0], [-3.0, 0.0, 4.0, 7.0, 7.0]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[-2.0, 2.0, -1.0, -7.0, 2.0], [-6.0, 7.0, -5.0, 6.0, 0.0], [2.0, -6.0, 3.0, 4.0, -5.0], [-5.0, -4.0, 4.0, 3.0, 5.0]])+(1.-msk_arg1)*numpy.array([[5.0, -3.0, 4.0, -5.0, -3.0], [-3.0, -1.0, -2.0, -2.0, 6.0], [-6.0, 1.0, 3.0, 0.0, -1.0], [7.0, 1.0, -4.0, -7.0, 6.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-14.0, 14.0, -7.0, -49.0, 14.0], [-42.0, 49.0, -35.0, 42.0, 0.0], [14.0, -42.0, 21.0, 28.0, -35.0], [-35.0, -28.0, 28.0, 21.0, 35.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-10.0, 10.0, -5.0, -35.0, 10.0], [-30.0, 35.0, -25.0, 30.0, 0.0], [10.0, -30.0, 15.0, 20.0, -25.0], [-25.0, -20.0, 20.0, 15.0, 25.0]], [[-2.0, 2.0, -1.0, -7.0, 2.0], [-6.0, 7.0, -5.0, 6.0, 0.0], [2.0, -6.0, 3.0, 4.0, -5.0], [-5.0, -4.0, 4.0, 3.0, 5.0]], [[4.0, -4.0, 2.0, 14.0, -4.0], [12.0, -14.0, 10.0, -12.0, 0.0], [-4.0, 12.0, -6.0, -8.0, 10.0], [10.0, 8.0, -8.0, -6.0, -10.0]]], [[[4.0, -4.0, 2.0, 14.0, -4.0], [12.0, -14.0, 10.0, -12.0, 0.0], [-4.0, 12.0, -6.0, -8.0, 10.0], [10.0, 8.0, -8.0, -6.0, -10.0]], [[12.0, -12.0, 6.0, 42.0, -12.0], [36.0, -42.0, 30.0, -36.0, 0.0], [-12.0, 36.0, -18.0, -24.0, 30.0], [30.0, 24.0, -24.0, -18.0, -30.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[-4.0, 4.0, -2.0, -14.0, 4.0], [-12.0, 14.0, -10.0, 12.0, 0.0], [4.0, -12.0, 6.0, 8.0, -10.0], [-10.0, -8.0, 8.0, 6.0, 10.0]]], [[[12.0, -12.0, 6.0, 42.0, -12.0], [36.0, -42.0, 30.0, -36.0, 0.0], [-12.0, 36.0, -18.0, -24.0, 30.0], [30.0, 24.0, -24.0, -18.0, -30.0]], [[10.0, -10.0, 5.0, 35.0, -10.0], [30.0, -35.0, 25.0, -30.0, 0.0], [-10.0, 30.0, -15.0, -20.0, 25.0], [25.0, 20.0, -20.0, -15.0, -25.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[10.0, -10.0, 5.0, 35.0, -10.0], [30.0, -35.0, 25.0, -30.0, 0.0], [-10.0, 30.0, -15.0, -20.0, 25.0], [25.0, 20.0, -20.0, -15.0, -25.0]], [[6.0, -6.0, 3.0, 21.0, -6.0], [18.0, -21.0, 15.0, -18.0, 0.0], [-6.0, 18.0, -9.0, -12.0, 15.0], [15.0, 12.0, -12.0, -9.0, -15.0]]], [[[6.0, -6.0, 3.0, 21.0, -6.0], [18.0, -21.0, 15.0, -18.0, 0.0], [-6.0, 18.0, -9.0, -12.0, 15.0], [15.0, 12.0, -12.0, -9.0, -15.0]], [[10.0, -10.0, 5.0, 35.0, -10.0], [30.0, -35.0, 25.0, -30.0, 0.0], [-10.0, 30.0, -15.0, -20.0, 25.0], [25.0, 20.0, -20.0, -15.0, -25.0]], [[12.0, -12.0, 6.0, 42.0, -12.0], [36.0, -42.0, 30.0, -36.0, 0.0], [-12.0, 36.0, -18.0, -24.0, 30.0], [30.0, 24.0, -24.0, -18.0, -30.0]], [[10.0, -10.0, 5.0, 35.0, -10.0], [30.0, -35.0, 25.0, -30.0, 0.0], [-10.0, 30.0, -15.0, -20.0, 25.0], [25.0, 20.0, -20.0, -15.0, -25.0]], [[14.0, -14.0, 7.0, 49.0, -14.0], [42.0, -49.0, 35.0, -42.0, 0.0], [-14.0, 42.0, -21.0, -28.0, 35.0], [35.0, 28.0, -28.0, -21.0, -35.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[30.0, -18.0, 24.0, -30.0, -18.0], [-18.0, -6.0, -12.0, -12.0, 36.0], [-36.0, 6.0, 18.0, 0.0, -6.0], [42.0, 6.0, -24.0, -42.0, 36.0]], [[35.0, -21.0, 28.0, -35.0, -21.0], [-21.0, -7.0, -14.0, -14.0, 42.0], [-42.0, 7.0, 21.0, 0.0, -7.0], [49.0, 7.0, -28.0, -49.0, 42.0]], [[25.0, -15.0, 20.0, -25.0, -15.0], [-15.0, -5.0, -10.0, -10.0, 30.0], [-30.0, 5.0, 15.0, 0.0, -5.0], [35.0, 5.0, -20.0, -35.0, 30.0]], [[-20.0, 12.0, -16.0, 20.0, 12.0], [12.0, 4.0, 8.0, 8.0, -24.0], [24.0, -4.0, -12.0, 0.0, 4.0], [-28.0, -4.0, 16.0, 28.0, -24.0]]], [[[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[35.0, -21.0, 28.0, -35.0, -21.0], [-21.0, -7.0, -14.0, -14.0, 42.0], [-42.0, 7.0, 21.0, 0.0, -7.0], [49.0, 7.0, -28.0, -49.0, 42.0]], [[10.0, -6.0, 8.0, -10.0, -6.0], [-6.0, -2.0, -4.0, -4.0, 12.0], [-12.0, 2.0, 6.0, 0.0, -2.0], [14.0, 2.0, -8.0, -14.0, 12.0]], [[30.0, -18.0, 24.0, -30.0, -18.0], [-18.0, -6.0, -12.0, -12.0, 36.0], [-36.0, 6.0, 18.0, 0.0, -6.0], [42.0, 6.0, -24.0, -42.0, 36.0]], [[-15.0, 9.0, -12.0, 15.0, 9.0], [9.0, 3.0, 6.0, 6.0, -18.0], [18.0, -3.0, -9.0, 0.0, 3.0], [-21.0, -3.0, 12.0, 21.0, -18.0]]], [[[15.0, -9.0, 12.0, -15.0, -9.0], [-9.0, -3.0, -6.0, -6.0, 18.0], [-18.0, 3.0, 9.0, 0.0, -3.0], [21.0, 3.0, -12.0, -21.0, 18.0]], [[25.0, -15.0, 20.0, -25.0, -15.0], [-15.0, -5.0, -10.0, -10.0, 30.0], [-30.0, 5.0, 15.0, 0.0, -5.0], [35.0, 5.0, -20.0, -35.0, 30.0]], [[10.0, -6.0, 8.0, -10.0, -6.0], [-6.0, -2.0, -4.0, -4.0, 12.0], [-12.0, 2.0, 6.0, 0.0, -2.0], [14.0, 2.0, -8.0, -14.0, 12.0]], [[-30.0, 18.0, -24.0, 30.0, 18.0], [18.0, 6.0, 12.0, 12.0, -36.0], [36.0, -6.0, -18.0, 0.0, 6.0], [-42.0, -6.0, 24.0, 42.0, -36.0]], [[-10.0, 6.0, -8.0, 10.0, 6.0], [6.0, 2.0, 4.0, 4.0, -12.0], [12.0, -2.0, -6.0, 0.0, 2.0], [-14.0, -2.0, 8.0, 14.0, -12.0]]], [[[-15.0, 9.0, -12.0, 15.0, 9.0], [9.0, 3.0, 6.0, 6.0, -18.0], [18.0, -3.0, -9.0, 0.0, 3.0], [-21.0, -3.0, 12.0, 21.0, -18.0]], [[0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0, 0.0]], [[20.0, -12.0, 16.0, -20.0, -12.0], [-12.0, -4.0, -8.0, -8.0, 24.0], [-24.0, 4.0, 12.0, 0.0, -4.0], [28.0, 4.0, -16.0, -28.0, 24.0]], [[35.0, -21.0, 28.0, -35.0, -21.0], [-21.0, -7.0, -14.0, -14.0, 42.0], [-42.0, 7.0, 21.0, 0.0, -7.0], [49.0, 7.0, -28.0, -49.0, 42.0]], [[35.0, -21.0, 28.0, -35.0, -21.0], [-21.0, -7.0, -14.0, -14.0, 42.0], [-42.0, 7.0, 21.0, 0.0, -7.0], [49.0, 7.0, -28.0, -49.0, 42.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank3_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, 3.0, 7.0], [-6.0, -5.0, 1.0], [-2.0, 6.0, -1.0], [-7.0, -2.0, 6.0], [3.0, 4.0, 6.0]], [[-4.0, 6.0, 6.0], [4.0, -5.0, -6.0], [1.0, -1.0, 7.0], [-7.0, 7.0, 0.0], [4.0, 5.0, 2.0]], [[1.0, 2.0, 0.0], [4.0, 3.0, -4.0], [-7.0, 1.0, 0.0], [3.0, 0.0, 4.0], [0.0, -3.0, -1.0]], [[0.0, -7.0, -6.0], [2.0, 7.0, -1.0], [0.0, -3.0, -3.0], [-4.0, 3.0, 2.0], [6.0, 4.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[2.0, -4.0, -1.0], [-6.0, -6.0, 4.0], [4.0, 6.0, -6.0], [-7.0, -1.0, 7.0], [0.0, -1.0, -1.0]], [[5.0, 4.0, 7.0], [3.0, -2.0, 4.0], [-7.0, -3.0, -7.0], [2.0, 3.0, -7.0], [-5.0, 7.0, 0.0]], [[-4.0, 0.0, -7.0], [-1.0, 5.0, -1.0], [-3.0, -4.0, -1.0], [-1.0, 5.0, 0.0], [2.0, -5.0, -5.0]], [[-1.0, -5.0, -3.0], [-6.0, 1.0, -1.0], [6.0, -5.0, -1.0], [5.0, -4.0, 3.0], [-3.0, -6.0, -6.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[5.0, 4.0, -3.0], [-7.0, -4.0, 3.0], [4.0, -5.0, 0.0], [-2.0, 0.0, 7.0], [-2.0, 1.0, 7.0]], [[5.0, 2.0, 2.0], [-1.0, 3.0, -6.0], [-3.0, 7.0, 6.0], [5.0, 4.0, 7.0], [2.0, -2.0, -1.0]], [[0.0, 0.0, 5.0], [3.0, 4.0, -5.0], [-7.0, 6.0, -7.0], [0.0, -7.0, 2.0], [3.0, -4.0, 5.0]], [[-4.0, -5.0, 6.0], [-5.0, -2.0, -4.0], [-4.0, -1.0, -2.0], [5.0, 0.0, -6.0], [-2.0, -5.0, 4.0]]])+(1.-msk_arg1)*numpy.array([[[-6.0, 6.0, 4.0], [5.0, 6.0, -2.0], [0.0, 3.0, 1.0], [6.0, -3.0, -4.0], [2.0, 3.0, -5.0]], [[-7.0, -1.0, -1.0], [-7.0, 6.0, 3.0], [-6.0, -2.0, 1.0], [2.0, -6.0, 3.0], [-5.0, 3.0, -5.0]], [[-5.0, -2.0, 0.0], [-7.0, -1.0, 0.0], [-3.0, -7.0, -5.0], [-3.0, -3.0, 0.0], [0.0, -7.0, 6.0]], [[5.0, 5.0, -1.0], [-7.0, 0.0, -1.0], [7.0, 1.0, -3.0], [-1.0, -3.0, -4.0], [-2.0, -1.0, -7.0]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-44.0, 58.0, -43.0, 63.0, 66.0], [-15.0, -26.0, 84.0, 26.0, -27.0], [35.0, -44.0, 18.0, -7.0, 2.0], [55.0, 1.0, 11.0, -77.0, 27.0]], [[-53.0, 65.0, 1.0, 19.0, 14.0], [-38.0, -15.0, -11.0, -43.0, -3.0], [5.0, -43.0, 5.0, 37.0, 7.0], [55.0, 36.0, 27.0, -36.0, 41.0]], [[17.0, -13.0, -38.0, -3.0, 3.0], [0.0, 26.0, 42.0, 7.0, -15.0], [-5.0, 23.0, 57.0, -44.0, -35.0], [-28.0, 2.0, 4.0, -4.0, -30.0]], [[-61.0, 75.0, -18.0, 56.0, 54.0], [-27.0, -35.0, 43.0, -1.0, -16.0], [30.0, -59.0, -5.0, 26.0, 17.0], [74.0, 15.0, 18.0, -71.0, 48.0]], [[13.0, -19.0, -8.0, 36.0, 40.0], [35.0, -27.0, 55.0, 73.0, -8.0], [30.0, -5.0, -39.0, -16.0, 23.0], [4.0, -47.0, -28.0, -21.0, -2.0]]], [[[-14.0, 22.0, -46.0, 50.0, 56.0], [4.0, -14.0, 90.0, 46.0, -26.0], [30.0, -18.0, 22.0, -30.0, -6.0], [22.0, -16.0, -2.0, -56.0, 2.0]], [[18.0, -26.0, 41.0, -50.0, -55.0], [-2.0, 17.0, -83.0, -42.0, 24.0], [-30.0, 22.0, -16.0, 23.0, 2.0], [-27.0, 14.0, 1.0, 56.0, -7.0]], [[-20.0, 18.0, 9.0, 47.0, 46.0], [17.0, -46.0, 32.0, 50.0, -3.0], [35.0, -36.0, -62.0, 21.0, 42.0], [43.0, -31.0, -17.0, -37.0, 31.0]], [[-7.0, 21.0, -63.0, 14.0, 21.0], [-21.0, 28.0, 70.0, -7.0, -28.0], [0.0, 7.0, 91.0, -49.0, -49.0], [-7.0, 21.0, 21.0, -35.0, -21.0]], [[34.0, -42.0, -9.0, 6.0, 11.0], [34.0, -1.0, 35.0, 54.0, -4.0], [10.0, 22.0, -12.0, -31.0, 2.0], [-29.0, -38.0, -25.0, 8.0, -25.0]]], [[[13.0, -15.0, -6.0, -2.0, 0.0], [9.0, 5.0, 11.0, 13.0, -2.0], [0.0, 11.0, 5.0, -14.0, -5.0], [-14.0, -9.0, -6.0, 5.0, -12.0]], [[44.0, -52.0, 1.0, -36.0, -33.0], [18.0, 29.0, -15.0, 4.0, 6.0], [-20.0, 44.0, 18.0, -29.0, -20.0], [-55.0, -10.0, -11.0, 44.0, -39.0]], [[-31.0, 45.0, -33.0, 14.0, 15.0], [-33.0, 10.0, 28.0, -31.0, -16.0], [0.0, -17.0, 55.0, -7.0, -25.0], [23.0, 33.0, 27.0, -35.0, 9.0]], [[3.0, -9.0, 12.0, 22.0, 22.0], [23.0, -27.0, 15.0, 43.0, 2.0], [20.0, -11.0, -49.0, 8.0, 29.0], [12.0, -31.0, -20.0, -9.0, 10.0]], [[-9.0, 9.0, 15.0, -7.0, -10.0], [-8.0, -3.0, -27.0, -19.0, 7.0], [-5.0, -7.0, -11.0, 19.0, 7.0], [9.0, 10.0, 5.0, 6.0, 11.0]]], [[[-10.0, 10.0, 35.0, -42.0, -49.0], [-26.0, 15.0, -85.0, -70.0, 20.0], [-30.0, 2.0, 0.0, 37.0, -2.0], [-1.0, 38.0, 19.0, 36.0, 11.0]], [[41.0, -45.0, -27.0, -11.0, -4.0], [22.0, 25.0, 37.0, 31.0, -9.0], [-5.0, 39.0, 35.0, -51.0, -27.0], [-49.0, -20.0, -13.0, 16.0, -43.0]], [[-3.0, 3.0, 15.0, -21.0, -24.0], [-12.0, 9.0, -39.0, -33.0, 9.0], [-15.0, 3.0, 3.0, 15.0, -3.0], [-3.0, 18.0, 9.0, 18.0, 3.0]], [[-14.0, 22.0, -31.0, 22.0, 25.0], [-10.0, 1.0, 45.0, 6.0, -16.0], [10.0, -10.0, 32.0, -17.0, -14.0], [13.0, 6.0, 9.0, -32.0, 1.0]], [[25.0, -37.0, 4.0, 37.0, 41.0], [52.0, -36.0, 52.0, 95.0, -3.0], [35.0, -1.0, -67.0, -14.0, 37.0], [-2.0, -66.0, -42.0, -12.0, -4.0]]]])+(1.-msk_ref)*numpy.array([[[[-40.0, -12.0, -13.0, 28.0, -3.0], [-9.0, -41.0, -5.0, 25.0, -17.0], [-2.0, -10.0, 27.0, 6.0, 22.0], [-9.0, -13.0, 13.0, 14.0, 7.0]], [[16.0, -74.0, -14.0, -34.0, -50.0], [44.0, 18.0, 52.0, 36.0, -8.0], [42.0, 48.0, 40.0, 36.0, 66.0], [-64.0, 38.0, -60.0, 8.0, -10.0]], [[-12.0, 68.0, 12.0, 30.0, 56.0], [-28.0, -10.0, -42.0, -46.0, 28.0], [-32.0, -34.0, -24.0, -30.0, -78.0], [56.0, -22.0, 52.0, 2.0, 28.0]], [[64.0, -55.0, 4.0, -67.0, -52.0], [43.0, 64.0, 51.0, 13.0, -3.0], [37.0, 50.0, -7.0, 24.0, 49.0], [-47.0, 42.0, -71.0, -18.0, -34.0]], [[-10.0, -4.0, -4.0, 7.0, 2.0], [2.0, -9.0, 1.0, 3.0, 2.0], [2.0, 1.0, 12.0, 3.0, 1.0], [-4.0, 1.0, 2.0, 7.0, 8.0]]], [[[22.0, 35.0, 19.0, -10.0, -13.0], [-46.0, 10.0, -31.0, 7.0, -48.0], [-33.0, -39.0, -78.0, -27.0, 14.0], [38.0, -42.0, 18.0, -45.0, -63.0]], [[-14.0, -5.0, -2.0, 8.0, -20.0], [-23.0, -21.0, -10.0, 30.0, -41.0], [-11.0, -19.0, -15.0, -3.0, 38.0], [1.0, -25.0, 7.0, -13.0, -32.0]], [[-4.0, -39.0, -16.0, -5.0, 12.0], [59.0, 10.0, 41.0, -17.0, 61.0], [41.0, 52.0, 77.0, 30.0, -21.0], [-43.0, 56.0, -31.0, 44.0, 66.0]], [[-22.0, 42.0, 2.0, 31.0, 48.0], [-10.0, -17.0, -25.0, -35.0, 34.0], [-16.0, -17.0, 8.0, -15.0, -63.0], [32.0, -7.0, 38.0, 17.0, 42.0]], [[72.0, 17.0, 21.0, -51.0, 11.0], [28.0, 77.0, 16.0, -52.0, 46.0], [11.0, 28.0, -34.0, -6.0, -49.0], [10.0, 35.0, -28.0, -16.0, 3.0]]], [[[-4.0, -6.0, -7.0, 4.0, 27.0], [35.0, 7.0, 17.0, -29.0, 55.0], [20.0, 28.0, 47.0, 12.0, -42.0], [-13.0, 35.0, -7.0, 32.0, 57.0]], [[32.0, 27.0, 14.0, -17.0, 18.0], [3.0, 34.0, -5.0, -35.0, 25.0], [-5.0, 2.0, -27.0, -12.0, -41.0], [21.0, 8.0, 1.0, -10.0, 4.0]], [[-10.0, -37.0, -13.0, -2.0, -13.0], [26.0, -6.0, 25.0, 15.0, 8.0], [23.0, 25.0, 42.0, 21.0, 22.0], [-34.0, 22.0, -22.0, 19.0, 17.0]], [[36.0, 25.0, 15.0, -21.0, 13.0], [2.0, 37.0, -4.0, -32.0, 20.0], [-5.0, 2.0, -32.0, -12.0, -35.0], [20.0, 7.0, -2.0, -14.0, -3.0]], [[-62.0, -10.0, -20.0, 47.0, 14.0], [-4.0, -59.0, -7.0, 19.0, 0.0], [0.0, -9.0, 54.0, 9.0, 5.0], [-10.0, -9.0, 24.0, 33.0, 36.0]]], [[[-36.0, -29.0, -18.0, 21.0, -2.0], [15.0, -32.0, 13.0, 19.0, 5.0], [15.0, 12.0, 53.0, 18.0, 17.0], [-27.0, 10.0, -3.0, 28.0, 28.0]], [[38.0, -22.0, 2.0, -35.0, -4.0], [42.0, 45.0, 33.0, -21.0, 38.0], [28.0, 41.0, 16.0, 15.0, -13.0], [-24.0, 43.0, -38.0, 7.0, 18.0]], [[-70.0, 2.0, -16.0, 55.0, 2.0], [-36.0, -75.0, -27.0, 39.0, -40.0], [-20.0, -37.0, 22.0, -3.0, 29.0], [6.0, -41.0, 40.0, 13.0, 0.0]], [[-42.0, -5.0, -9.0, 30.0, -17.0], [-34.0, -50.0, -19.0, 43.0, -52.0], [-17.0, -31.0, -2.0, -3.0, 46.0], [2.0, -38.0, 22.0, -5.0, -27.0]], [[-42.0, -39.0, -24.0, 24.0, 6.0], [33.0, -33.0, 24.0, 12.0, 27.0], [27.0, 27.0, 81.0, 27.0, 6.0], [-39.0, 27.0, -9.0, 45.0, 54.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank4_offset2(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, 4.0, 0.0], [0.0, -5.0, 0.0]], [[0.0, -2.0, -2.0], [-7.0, 4.0, 2.0]], [[6.0, -7.0, 2.0], [4.0, 1.0, -2.0]], [[-6.0, 3.0, 0.0], [0.0, -3.0, 6.0]], [[-5.0, 1.0, 4.0], [-7.0, -6.0, 6.0]]], [[[0.0, 5.0, 2.0], [3.0, -1.0, 3.0]], [[7.0, -1.0, 0.0], [-1.0, 4.0, 0.0]], [[-1.0, 0.0, -2.0], [7.0, -4.0, 1.0]], [[-3.0, 7.0, 0.0], [7.0, -3.0, 7.0]], [[-7.0, -3.0, -7.0], [4.0, 4.0, -5.0]]], [[[-5.0, -1.0, 7.0], [0.0, -6.0, -5.0]], [[-2.0, 3.0, -6.0], [2.0, 0.0, 5.0]], [[0.0, -6.0, 6.0], [1.0, 1.0, 3.0]], [[1.0, -6.0, 0.0], [2.0, 0.0, -3.0]], [[4.0, 6.0, 7.0], [-5.0, 5.0, -2.0]]], [[[0.0, 3.0, -2.0], [2.0, -2.0, 5.0]], [[-4.0, 2.0, -1.0], [-1.0, -2.0, 7.0]], [[1.0, -7.0, -4.0], [-4.0, 5.0, 2.0]], [[-5.0, 2.0, -1.0], [-2.0, 0.0, 5.0]], [[0.0, -5.0, 4.0], [4.0, 0.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[-5.0, -6.0, 5.0], [-6.0, 0.0, -3.0]], [[2.0, 0.0, 4.0], [-7.0, 0.0, 7.0]], [[5.0, -2.0, -5.0], [-6.0, 0.0, 1.0]], [[0.0, 6.0, 4.0], [7.0, -2.0, 5.0]], [[-7.0, 2.0, -7.0], [4.0, -3.0, 6.0]]], [[[5.0, -3.0, 1.0], [-3.0, -1.0, 5.0]], [[-1.0, 0.0, -4.0], [-5.0, -5.0, -1.0]], [[-3.0, -4.0, -3.0], [5.0, 5.0, -1.0]], [[-4.0, 0.0, 2.0], [-4.0, -7.0, -1.0]], [[1.0, -6.0, 2.0], [5.0, 0.0, -3.0]]], [[[-2.0, 0.0, -7.0], [2.0, -4.0, 3.0]], [[-1.0, -6.0, 1.0], [-7.0, -2.0, -4.0]], [[1.0, -2.0, 0.0], [-7.0, 5.0, -5.0]], [[4.0, -4.0, 0.0], [3.0, 0.0, 4.0]], [[0.0, -7.0, 7.0], [2.0, -7.0, -4.0]]], [[[-6.0, -6.0, 3.0], [0.0, -6.0, -7.0]], [[5.0, -6.0, -1.0], [7.0, -5.0, 2.0]], [[0.0, 1.0, 5.0], [5.0, -6.0, 3.0]], [[-2.0, 4.0, 1.0], [-4.0, -2.0, -7.0]], [[-3.0, 3.0, -4.0], [5.0, -6.0, -2.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[[[0.0, -3.0, -2.0], [5.0, 7.0, 6.0]], [[1.0, 5.0, 1.0], [-1.0, 5.0, -3.0]], [[0.0, 1.0, 6.0], [-2.0, 6.0, 5.0]], [[0.0, -5.0, 5.0], [-2.0, 4.0, -7.0]], [[-3.0, -4.0, 6.0], [1.0, 6.0, -7.0]]], [[[-3.0, 1.0, 3.0], [7.0, -2.0, -5.0]], [[6.0, 3.0, -3.0], [4.0, 6.0, 4.0]], [[0.0, 0.0, 5.0], [-6.0, 0.0, 2.0]], [[0.0, -6.0, -3.0], [4.0, -2.0, -2.0]], [[-2.0, -2.0, 3.0], [5.0, 7.0, -5.0]]], [[[-3.0, -4.0, -2.0], [-5.0, -6.0, -3.0]], [[1.0, 7.0, 6.0], [-5.0, 3.0, -6.0]], [[-2.0, 1.0, 2.0], [2.0, 5.0, 0.0]], [[2.0, -6.0, 6.0], [6.0, 2.0, 1.0]], [[7.0, 6.0, -5.0], [2.0, 2.0, -7.0]]], [[[2.0, -3.0, -6.0], [5.0, 0.0, 3.0]], [[-5.0, -4.0, 7.0], [3.0, -6.0, -1.0]], [[-5.0, -4.0, 7.0], [-6.0, -3.0, 0.0]], [[-4.0, 0.0, 1.0], [0.0, -6.0, -7.0]], [[-7.0, 0.0, 0.0], [7.0, 1.0, 4.0]]]])+(1.-msk_arg1)*numpy.array([[[[-6.0, -6.0, -5.0], [2.0, -3.0, 3.0]], [[1.0, 4.0, 2.0], [-4.0, 1.0, 7.0]], [[-6.0, 2.0, -2.0], [7.0, -7.0, -7.0]], [[6.0, -1.0, 0.0], [0.0, 0.0, -3.0]], [[-1.0, 5.0, 2.0], [-7.0, -1.0, 7.0]]], [[[-6.0, -2.0, 6.0], [6.0, 3.0, -5.0]], [[-2.0, -7.0, -5.0], [6.0, 5.0, -5.0]], [[0.0, -2.0, 3.0], [0.0, 3.0, 1.0]], [[6.0, -3.0, 1.0], [4.0, -4.0, 6.0]], [[1.0, 7.0, 4.0], [-7.0, -7.0, 3.0]]], [[[-7.0, -5.0, 0.0], [4.0, 1.0, 1.0]], [[-7.0, 1.0, -2.0], [1.0, 2.0, -3.0]], [[3.0, 0.0, 6.0], [-6.0, 2.0, 7.0]], [[6.0, 3.0, -5.0], [-1.0, -6.0, -4.0]], [[5.0, -5.0, 2.0], [-2.0, -3.0, 7.0]]], [[[7.0, -6.0, -4.0], [-5.0, 5.0, -1.0]], [[4.0, 6.0, 5.0], [3.0, -6.0, -6.0]], [[4.0, 6.0, -1.0], [-5.0, -7.0, -4.0]], [[6.0, 6.0, 1.0], [5.0, -1.0, 2.0]], [[-7.0, -1.0, -5.0], [6.0, 7.0, -4.0]]]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=2)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-47.0, -3.0, -26.0, -40.0, -52.0], [8.0, -6.0, 0.0, -14.0, -47.0], [8.0, 15.0, -25.0, -30.0, 28.0], [-8.0, 4.0, -11.0, 22.0, -19.0]], [[15.0, 9.0, 34.0, 16.0, -1.0], [-75.0, 4.0, 36.0, -22.0, -19.0], [17.0, 9.0, 0.0, -32.0, -22.0], [-11.0, -53.0, 24.0, -40.0, -37.0]], [[32.0, -20.0, -7.0, 55.0, 46.0], [17.0, 23.0, -18.0, 54.0, 45.0], [-14.0, -36.0, -2.0, 90.0, 14.0], [35.0, 20.0, -15.0, -14.0, -21.0]], [[6.0, -24.0, 15.0, -69.0, -54.0], [-3.0, -21.0, 12.0, -24.0, -45.0], [6.0, -30.0, 0.0, -30.0, -72.0], [-3.0, 30.0, 27.0, 0.0, 63.0]], [[-52.0, -37.0, 33.0, -37.0, -50.0], [-39.0, -79.0, 74.0, -46.0, -87.0], [56.0, 7.0, -25.0, -40.0, -117.0], [-54.0, 58.0, 109.0, 18.0, 4.0]]], [[[7.0, 10.0, 20.0, -46.0, -32.0], [19.0, 27.0, -2.0, -28.0, -11.0], [-42.0, 11.0, 10.0, 1.0, 3.0], [-3.0, 6.0, -21.0, -13.0, 32.0]], [[26.0, 23.0, 25.0, 23.0, 6.0], [-37.0, 59.0, 6.0, -6.0, 11.0], [-36.0, 17.0, 3.0, 22.0, 49.0], [12.0, -58.0, -37.0, -52.0, -52.0]], [[17.0, -33.0, -45.0, -47.0, -33.0], [49.0, 8.0, -50.0, 40.0, -2.0], [-7.0, -66.0, -8.0, 21.0, 2.0], [48.0, 35.0, -39.0, 19.0, 56.0]], [[35.0, -11.0, 10.0, -110.0, -79.0], [36.0, 41.0, -28.0, -22.0, -29.0], [-57.0, -40.0, 12.0, -5.0, -20.0], [29.0, 19.0, -46.0, -19.0, 95.0]], [[41.0, 2.0, -54.0, 23.0, 54.0], [42.0, -10.0, -69.0, 57.0, 72.0], [18.0, -48.0, 25.0, -11.0, 19.0], [42.0, -9.0, -38.0, 32.0, 61.0]]], [[[-83.0, -18.0, -20.0, 51.0, 60.0], [72.0, -110.0, 25.0, 7.0, 16.0], [56.0, 42.0, -7.0, 21.0, -53.0], [-64.0, 119.0, 96.0, 98.0, 9.0]], [[43.0, -10.0, -12.0, -84.0, -75.0], [-20.0, 43.0, -32.0, -2.0, -35.0], [-19.0, -57.0, -1.0, -41.0, 3.0], [48.0, -43.0, -56.0, -33.0, 48.0]], [[36.0, -29.0, 49.0, 41.0, 46.0], [2.0, -14.0, 30.0, 14.0, 27.0], [-8.0, -26.0, 13.0, 83.0, -83.0], [-4.0, 60.0, 57.0, -21.0, 20.0]], [[10.0, -22.0, -25.0, 47.0, 44.0], [20.0, -16.0, -18.0, 50.0, 35.0], [20.0, -33.0, -4.0, 47.0, -4.0], [21.0, 28.0, 7.0, 17.0, -5.0]], [[-34.0, 77.0, 78.0, 49.0, 45.0], [-20.0, 23.0, 61.0, -83.0, 21.0], [-49.0, 140.0, 27.0, -8.0, 43.0], [-83.0, -38.0, 20.0, -25.0, -66.0]]], [[[21.0, -14.0, 0.0, -72.0, -69.0], [-10.0, 31.0, -12.0, -10.0, -41.0], [-21.0, -37.0, -7.0, -17.0, -7.0], [28.0, -13.0, -32.0, -25.0, 32.0]], [[19.0, -25.0, 21.0, -70.0, -64.0], [-27.0, -3.0, 15.0, -23.0, -53.0], [2.0, -39.0, -4.0, -29.0, -66.0], [8.0, 7.0, 17.0, -22.0, 47.0]], [[56.0, -15.0, 17.0, 29.0, 13.0], [-70.0, 19.0, 8.0, 24.0, 5.0], [17.0, -49.0, 0.0, 8.0, -27.0], [33.0, -49.0, 4.0, -52.0, -22.0]], [[16.0, -9.0, 25.0, -46.0, -36.0], [-25.0, -9.0, 17.0, -27.0, -32.0], [4.0, -17.0, 6.0, -35.0, -57.0], [-5.0, -1.0, 22.0, -16.0, 41.0]], [[-15.0, -4.0, -24.0, 86.0, 97.0], [70.0, -39.0, -18.0, 48.0, 77.0], [13.0, 11.0, 11.0, 71.0, 7.0], [-10.0, 67.0, 24.0, 53.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[20.0, -16.0, -13.0, -15.0, 6.0], [51.0, 6.0, 24.0, -49.0, 6.0], [38.0, 22.0, 30.0, -55.0, 6.0], [14.0, -31.0, -19.0, -97.0, -8.0]], [[-25.0, 87.0, -118.0, -9.0, 104.0], [-65.0, -101.0, 19.0, 30.0, 88.0], [-35.0, -50.0, 121.0, -29.0, 81.0], [26.0, -35.0, 11.0, -5.0, -104.0]], [[-2.0, 18.0, -73.0, 29.0, 24.0], [-97.0, -12.0, -10.0, 13.0, 16.0], [-48.0, -36.0, 28.0, 51.0, 44.0], [96.0, -41.0, 39.0, -15.0, -48.0]], [[-21.0, 37.0, 32.0, -21.0, 26.0], [23.0, -55.0, -1.0, 52.0, 38.0], [1.0, -14.0, 13.0, -17.0, 5.0], [-102.0, 59.0, -9.0, 87.0, -18.0]], [[100.0, 10.0, 67.0, -62.0, 20.0], [-19.0, 14.0, -28.0, 9.0, -10.0], [58.0, 45.0, -51.0, -11.0, -16.0], [-74.0, -57.0, -32.0, -2.0, 61.0]]], [[[-5.0, 41.0, -87.0, 18.0, 39.0], [-64.0, -42.0, 11.0, 62.0, 31.0], [-28.0, -60.0, 72.0, 5.0, 86.0], [54.0, -26.0, 3.0, 9.0, -82.0]], [[28.0, -1.0, 21.0, -3.0, 26.0], [-58.0, -28.0, -28.0, -16.0, 50.0], [-19.0, 3.0, -14.0, 53.0, 5.0], [10.0, -3.0, 64.0, -32.0, -34.0]], [[49.0, -47.0, 23.0, -11.0, -70.0], [58.0, 109.0, 13.0, -15.0, -116.0], [65.0, 41.0, -54.0, -46.0, -33.0], [16.0, -60.0, -89.0, -27.0, 109.0]], [[24.0, 2.0, 48.0, -21.0, 36.0], [-4.0, -56.0, -16.0, -16.0, 78.0], [4.0, 9.0, 3.0, 16.0, 6.0], [-50.0, 30.0, 55.0, -37.0, -51.0]], [[21.0, -60.0, 34.0, 21.0, -83.0], [63.0, 75.0, 15.0, 28.0, -77.0], [40.0, -3.0, -36.0, -15.0, 8.0], [13.0, 11.0, -47.0, -9.0, 31.0]]], [[[72.0, -7.0, 47.0, -21.0, -1.0], [-45.0, 16.0, -30.0, 23.0, -7.0], [21.0, 13.0, -47.0, 33.0, 5.0], [-19.0, -31.0, 5.0, 1.0, 21.0]], [[17.0, -25.0, -15.0, 12.0, -4.0], [-4.0, 7.0, 5.0, -31.0, 12.0], [3.0, 0.0, 13.0, 6.0, 19.0], [54.0, -20.0, 24.0, -82.0, -32.0]], [[-38.0, -9.0, -59.0, 23.0, -2.0], [-4.0, 20.0, 14.0, -66.0, -14.0], [-25.0, 9.0, 20.0, -3.0, -21.0], [84.0, -29.0, 12.0, -56.0, 8.0]], [[18.0, 4.0, -39.0, 16.0, -17.0], [-18.0, 18.0, 12.0, 72.0, -33.0], [8.0, -41.0, 22.0, -7.0, 62.0], [33.0, -23.0, -39.0, 23.0, -22.0]], [[20.0, -57.0, 63.0, 19.0, -56.0], [67.0, 11.0, 10.0, 40.0, 2.0], [32.0, -21.0, -12.0, 0.0, 38.0], [-27.0, 65.0, 6.0, -26.0, -49.0]]], [[[54.0, -79.0, 109.0, -9.0, -61.0], [83.0, 44.0, -4.0, -33.0, -15.0], [59.0, 39.0, -61.0, -5.0, -25.0], [-41.0, 33.0, 7.0, -77.0, 19.0]], [[46.0, -40.0, 30.0, 30.0, -67.0], [-7.0, 44.0, -4.0, 107.0, -49.0], [20.0, -48.0, -29.0, 32.0, 68.0], [13.0, 18.0, -23.0, 37.0, -25.0]], [[6.0, 9.0, 48.0, -10.0, 7.0], [25.0, -47.0, -2.0, 64.0, 43.0], [12.0, -25.0, 9.0, -3.0, 34.0], [-84.0, 64.0, 6.0, 48.0, -50.0]], [[-40.0, -19.0, 53.0, 5.0, 5.0], [15.0, -28.0, -18.0, -73.0, 51.0], [-31.0, 29.0, -29.0, 39.0, -63.0], [-25.0, 63.0, 77.0, -19.0, -5.0]], [[42.0, -39.0, 123.0, -15.0, -33.0], [10.0, 15.0, -38.0, 1.0, 3.0], [18.0, 31.0, -89.0, 50.0, -44.0], [-76.0, 49.0, 35.0, 23.0, 34.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(4, 5, 4, 5),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-1.0, 0.0], [-1.0, 0.0]], [[-4.0, -4.0], [-2.0, -4.0]], [[-2.0, 1.0], [-4.0, -6.0]], [[0.0, 5.0], [-4.0, 2.0]], [[-3.0, -3.0], [0.0, -3.0]], [[3.0, 0.0], [4.0, -4.0]]])+(1.-msk_arg0)*numpy.array([[[-3.0, 1.0], [2.0, -1.0]], [[2.0, 0.0], [7.0, -7.0]], [[6.0, 3.0], [-1.0, 6.0]], [[-4.0, 0.0], [7.0, 5.0]], [[-2.0, 1.0], [7.0, 4.0]], [[-1.0, 5.0], [2.0, 0.0]]])
      arg1=6.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-6.0, 0.0], [-6.0, 0.0]], [[-24.0, -24.0], [-12.0, -24.0]], [[-12.0, 6.0], [-24.0, -36.0]], [[0.0, 30.0], [-24.0, 12.0]], [[-18.0, -18.0], [0.0, -18.0]], [[18.0, 0.0], [24.0, -24.0]]])+(1.-msk_ref)*numpy.array([[[-18.0, 6.0], [12.0, -6.0]], [[12.0, 0.0], [42.0, -42.0]], [[36.0, 18.0], [-6.0, 36.0]], [[-24.0, 0.0], [42.0, 30.0]], [[-12.0, 6.0], [42.0, 24.0]], [[-6.0, 30.0], [12.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, -2.0], [1.0, -5.0]], [[-1.0, -4.0], [-6.0, 4.0]], [[-5.0, 1.0], [-1.0, -7.0]], [[1.0, 5.0], [-4.0, 4.0]], [[0.0, 0.0], [3.0, 4.0]], [[-4.0, -1.0], [-5.0, -7.0]]])+(1.-msk_arg0)*numpy.array([[[-6.0, -6.0], [-4.0, 0.0]], [[4.0, 6.0], [6.0, -3.0]], [[-2.0, 7.0], [-3.0, -3.0]], [[3.0, 2.0], [4.0, 6.0]], [[4.0, 4.0], [3.0, -1.0]], [[3.0, 7.0], [0.0, 0.0]]])
      arg1=numpy.array(-4.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[28.0, 8.0], [-4.0, 20.0]], [[4.0, 16.0], [24.0, -16.0]], [[20.0, -4.0], [4.0, 28.0]], [[-4.0, -20.0], [16.0, -16.0]], [[-0.0, -0.0], [-12.0, -16.0]], [[16.0, 4.0], [20.0, 28.0]]])+(1.-msk_ref)*numpy.array([[[24.0, 24.0], [16.0, -0.0]], [[-16.0, -24.0], [-24.0, 12.0]], [[8.0, -28.0], [12.0, 12.0]], [[-12.0, -8.0], [-16.0, -24.0]], [[-16.0, -16.0], [-12.0, 4.0]], [[-12.0, -28.0], [-0.0, -0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-1.0, 0.0, 4.0], [-5.0, 1.0, -5.0]], [[-7.0, -5.0, 3.0], [6.0, 7.0, 7.0]]], [[[-4.0, -6.0, -6.0], [0.0, 4.0, 6.0]], [[6.0, -1.0, 5.0], [2.0, 7.0, -4.0]]], [[[5.0, 7.0, 7.0], [5.0, 1.0, -5.0]], [[-6.0, -6.0, -3.0], [-4.0, 6.0, 0.0]]], [[[0.0, -5.0, 7.0], [7.0, 7.0, -6.0]], [[3.0, 3.0, 0.0], [-7.0, -3.0, -3.0]]], [[[0.0, -7.0, -4.0], [-7.0, -4.0, -1.0]], [[-6.0, 0.0, -5.0], [0.0, -4.0, 0.0]]], [[[7.0, -7.0, 0.0], [2.0, 4.0, 6.0]], [[0.0, 7.0, 0.0], [-7.0, -1.0, 5.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, -6.0, 0.0], [5.0, -4.0, 2.0]], [[1.0, -5.0, -6.0], [-4.0, -4.0, -1.0]]], [[[4.0, -4.0, -5.0], [-3.0, -5.0, -3.0]], [[6.0, 1.0, 4.0], [1.0, -3.0, 4.0]]], [[[6.0, 0.0, 6.0], [2.0, 1.0, 3.0]], [[-2.0, 2.0, 1.0], [0.0, 1.0, 4.0]]], [[[-6.0, 5.0, 0.0], [-4.0, -7.0, -3.0]], [[-2.0, -5.0, -6.0], [4.0, 5.0, -1.0]]], [[[7.0, 0.0, 5.0], [-2.0, 7.0, 5.0]], [[-1.0, 5.0, -7.0], [6.0, 3.0, 4.0]]], [[[-6.0, -5.0, 4.0], [0.0, -4.0, 1.0]], [[5.0, 5.0, 6.0], [0.0, -5.0, 7.0]]]])
      arg1=numpy.array([5.0, 0.0, 6.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[19.0, -55.0], [-17.0, 72.0]], [[-56.0, 36.0], [60.0, -14.0]], [[67.0, -5.0], [-48.0, -20.0]], [[42.0, -1.0], [15.0, -53.0]], [[-24.0, -41.0], [-60.0, 0.0]], [[35.0, 46.0], [0.0, -5.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, 37.0], [-31.0, -26.0]], [[-10.0, -33.0], [54.0, 29.0]], [[66.0, 28.0], [-4.0, 24.0]], [[-30.0, -38.0], [-46.0, 14.0]], [[65.0, 20.0], [-47.0, 54.0]], [[-6.0, 6.0], [61.0, 42.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_array_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, -6.0], [1.0, 3.0]], [[-4.0, 4.0], [5.0, 4.0]], [[2.0, -3.0], [-2.0, 3.0]], [[-5.0, -7.0], [4.0, 3.0]], [[6.0, -7.0], [2.0, 5.0]], [[4.0, 1.0], [4.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-5.0, -4.0], [0.0, 2.0]], [[0.0, 5.0], [-3.0, 6.0]], [[-7.0, 7.0], [-4.0, 7.0]], [[-3.0, -1.0], [6.0, 6.0]], [[0.0, -1.0], [5.0, 5.0]], [[2.0, 3.0], [4.0, 3.0]]])
      arg1=numpy.array([-1.0, -4.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[0.0, 0.0], [6.0, 24.0]], [[-1.0, -4.0], [-3.0, -12.0]]], [[[4.0, 16.0], [-4.0, -16.0]], [[-5.0, -20.0], [-4.0, -16.0]]], [[[-2.0, -8.0], [3.0, 12.0]], [[2.0, 8.0], [-3.0, -12.0]]], [[[5.0, 20.0], [7.0, 28.0]], [[-4.0, -16.0], [-3.0, -12.0]]], [[[-6.0, -24.0], [7.0, 28.0]], [[-2.0, -8.0], [-5.0, -20.0]]], [[[-4.0, -16.0], [-1.0, -4.0]], [[-4.0, -16.0], [-5.0, -20.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 20.0], [4.0, 16.0]], [[0.0, 0.0], [-2.0, -8.0]]], [[[0.0, 0.0], [-5.0, -20.0]], [[3.0, 12.0], [-6.0, -24.0]]], [[[7.0, 28.0], [-7.0, -28.0]], [[4.0, 16.0], [-7.0, -28.0]]], [[[3.0, 12.0], [1.0, 4.0]], [[-6.0, -24.0], [-6.0, -24.0]]], [[[0.0, 0.0], [1.0, 4.0]], [[-5.0, -20.0], [-5.0, -20.0]]], [[[-2.0, -8.0], [-3.0, -12.0]], [[-4.0, -16.0], [-3.0, -12.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[0.0, 4.0, 5.0], [-7.0, 3.0, 1.0]], [[1.0, 0.0, -7.0], [2.0, 0.0, -5.0]]], [[[4.0, -2.0, 0.0], [-5.0, -7.0, -4.0]], [[2.0, 7.0, 7.0], [5.0, 7.0, -5.0]]], [[[1.0, 4.0, 0.0], [-4.0, -5.0, 2.0]], [[-3.0, 0.0, 7.0], [-5.0, 6.0, 1.0]]], [[[-6.0, 1.0, 1.0], [-7.0, 3.0, 3.0]], [[-7.0, -1.0, 3.0], [0.0, 0.0, -5.0]]], [[[3.0, 5.0, 6.0], [3.0, -1.0, 2.0]], [[-3.0, -6.0, 5.0], [-2.0, 0.0, 6.0]]], [[[3.0, 0.0, -3.0], [6.0, 5.0, 1.0]], [[-1.0, 2.0, -4.0], [2.0, -1.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, -2.0, 2.0], [-6.0, 7.0, 3.0]], [[-7.0, 6.0, 1.0], [-6.0, 4.0, -3.0]]], [[[6.0, 0.0, -5.0], [3.0, 4.0, 0.0]], [[0.0, -6.0, -2.0], [1.0, 6.0, -6.0]]], [[[-5.0, -4.0, -3.0], [-7.0, 0.0, 5.0]], [[-2.0, -5.0, 1.0], [-1.0, 6.0, 4.0]]], [[[1.0, -6.0, 6.0], [-5.0, -4.0, -6.0]], [[6.0, -6.0, 0.0], [2.0, 2.0, 6.0]]], [[[0.0, 0.0, 0.0], [-7.0, 0.0, -6.0]], [[4.0, -1.0, -5.0], [-5.0, 5.0, 0.0]]], [[[-4.0, 2.0, 5.0], [-3.0, -2.0, -6.0]], [[4.0, 2.0, -1.0], [2.0, 0.0, 2.0]]]])
      arg1=numpy.array([[0.0, 7.0, 3.0], [4.0, -3.0, 0.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[43.0, -12.0], [24.0, -37.0]], [[-21.0, 4.0], [-15.0, 8.0]]], [[[-14.0, 22.0], [-61.0, 1.0]], [[70.0, -13.0], [34.0, -1.0]]], [[[28.0, -8.0], [-29.0, -1.0]], [[21.0, -12.0], [45.0, -38.0]]], [[[10.0, -27.0], [30.0, -37.0]], [[2.0, -25.0], [-15.0, 0.0]]], [[[53.0, -3.0], [-1.0, 15.0]], [[-27.0, 6.0], [18.0, -8.0]]], [[[-9.0, 12.0], [38.0, 9.0]], [[2.0, -10.0], [-7.0, 11.0]]]])+(1.-msk_ref)*numpy.array([[[[-8.0, -18.0], [58.0, -45.0]], [[45.0, -46.0], [19.0, -36.0]]], [[[-15.0, 24.0], [28.0, 0.0]], [[-48.0, 18.0], [24.0, -14.0]]], [[[-37.0, -8.0], [15.0, -28.0]], [[-32.0, 7.0], [54.0, -22.0]]], [[[-24.0, 22.0], [-46.0, -8.0]], [[-42.0, 42.0], [32.0, 2.0]]], [[[0.0, 0.0], [-18.0, -28.0]], [[-22.0, 19.0], [35.0, -35.0]]], [[[29.0, -22.0], [-32.0, -6.0]], [[11.0, 10.0], [6.0, 8.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, -5.0], [-4.0, 4.0]], [[1.0, 3.0], [-4.0, 0.0]], [[-1.0, -5.0], [0.0, 0.0]], [[1.0, 4.0], [-5.0, -5.0]], [[2.0, 6.0], [1.0, -2.0]], [[2.0, 4.0], [5.0, 0.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, -3.0], [-6.0, -7.0]], [[2.0, 3.0], [-7.0, -3.0]], [[7.0, -3.0], [5.0, 7.0]], [[-5.0, -7.0], [-4.0, -4.0]], [[6.0, 6.0], [0.0, 4.0]], [[-3.0, 0.0], [-3.0, 1.0]]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(5.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[10.0, -25.0], [-20.0, 20.0]], [[5.0, 15.0], [-20.0, 0.0]], [[-5.0, -25.0], [0.0, 0.0]], [[5.0, 20.0], [-25.0, -25.0]], [[10.0, 30.0], [5.0, -10.0]], [[10.0, 20.0], [25.0, 0.0]]])+(1.-msk_ref)*numpy.array([[[0.0, -15.0], [-30.0, -35.0]], [[10.0, 15.0], [-35.0, -15.0]], [[35.0, -15.0], [25.0, 35.0]], [[-25.0, -35.0], [-20.0, -20.0]], [[30.0, 30.0], [0.0, 20.0]], [[-15.0, 0.0], [-15.0, 5.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-4.0, 4.0, 4.0], [-4.0, 5.0, 5.0]], [[-7.0, 1.0, -6.0], [-4.0, -2.0, -2.0]]], [[[-4.0, 7.0, 0.0], [3.0, 5.0, 1.0]], [[-1.0, -1.0, 3.0], [2.0, 0.0, -5.0]]], [[[3.0, 2.0, 7.0], [5.0, -5.0, -6.0]], [[0.0, -5.0, -7.0], [-1.0, -3.0, 5.0]]], [[[-2.0, -3.0, 3.0], [7.0, 4.0, 3.0]], [[-5.0, -6.0, 4.0], [-2.0, -1.0, -3.0]]], [[[0.0, 0.0, 6.0], [1.0, 0.0, -2.0]], [[1.0, 1.0, 7.0], [7.0, -1.0, 3.0]]], [[[7.0, -2.0, -7.0], [1.0, -6.0, -7.0]], [[-4.0, -4.0, -6.0], [5.0, -2.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 5.0, -3.0], [0.0, -2.0, 2.0]], [[-4.0, -5.0, -7.0], [0.0, 3.0, -7.0]]], [[[5.0, 2.0, -7.0], [-5.0, 0.0, 3.0]], [[-4.0, -5.0, 7.0], [-3.0, 6.0, -5.0]]], [[[-6.0, 0.0, 2.0], [-4.0, -2.0, -4.0]], [[-7.0, 5.0, -7.0], [7.0, -4.0, 0.0]]], [[[-2.0, 6.0, 7.0], [-6.0, -6.0, -5.0]], [[0.0, 2.0, 6.0], [-4.0, 2.0, -5.0]]], [[[4.0, 3.0, 0.0], [-2.0, -5.0, 1.0]], [[0.0, 5.0, 0.0], [-2.0, 3.0, -1.0]]], [[[-2.0, -2.0, 1.0], [-2.0, 5.0, -4.0]], [[-1.0, 4.0, 2.0], [4.0, 5.0, 2.0]]]])
      arg1=Symbol(shape=(3,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([3.0, 2.0, 2.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[4.0, 8.0], [-31.0, -20.0]], [[2.0, 21.0], [1.0, -4.0]], [[27.0, -7.0], [-24.0, 1.0]], [[-6.0, 35.0], [-19.0, -14.0]], [[12.0, -1.0], [19.0, 25.0]], [[3.0, -23.0], [-32.0, 1.0]]])+(1.-msk_ref)*numpy.array([[[22.0, 0.0], [-36.0, -8.0]], [[5.0, -9.0], [-8.0, -7.0]], [[-14.0, -24.0], [-25.0, 13.0]], [[20.0, -40.0], [16.0, -18.0]], [[18.0, -14.0], [10.0, -2.0]], [[-8.0, -4.0], [9.0, 26.0]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_Symbol_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[-7.0, -5.0], [2.0, 2.0]], [[-3.0, 2.0], [4.0, -3.0]], [[7.0, 0.0], [4.0, 0.0]], [[0.0, 2.0], [7.0, 1.0]], [[5.0, 0.0], [2.0, 0.0]], [[-7.0, -6.0], [-1.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[0.0, 0.0], [6.0, 0.0]], [[-6.0, 1.0], [-3.0, -1.0]], [[0.0, -6.0], [-1.0, -6.0]], [[0.0, 5.0], [-2.0, -3.0]], [[-5.0, 3.0], [0.0, 5.0]], [[-2.0, 7.0], [-3.0, 7.0]]])
      arg1=Symbol(shape=(2,))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array([-1.0, -2.0])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[7.0, 14.0], [5.0, 10.0]], [[-2.0, -4.0], [-2.0, -4.0]]], [[[3.0, 6.0], [-2.0, -4.0]], [[-4.0, -8.0], [3.0, 6.0]]], [[[-7.0, -14.0], [0.0, 0.0]], [[-4.0, -8.0], [0.0, 0.0]]], [[[0.0, 0.0], [-2.0, -4.0]], [[-7.0, -14.0], [-1.0, -2.0]]], [[[-5.0, -10.0], [0.0, 0.0]], [[-2.0, -4.0], [0.0, 0.0]]], [[[7.0, 14.0], [6.0, 12.0]], [[1.0, 2.0], [-6.0, -12.0]]]])+(1.-msk_ref)*numpy.array([[[[0.0, 0.0], [0.0, 0.0]], [[-6.0, -12.0], [0.0, 0.0]]], [[[6.0, 12.0], [-1.0, -2.0]], [[3.0, 6.0], [1.0, 2.0]]], [[[0.0, 0.0], [6.0, 12.0]], [[1.0, 2.0], [6.0, 12.0]]], [[[0.0, 0.0], [-5.0, -10.0]], [[2.0, 4.0], [3.0, 6.0]]], [[[5.0, 10.0], [-3.0, -6.0]], [[0.0, 0.0], [-5.0, -10.0]]], [[[2.0, 4.0], [-7.0, -14.0]], [[3.0, 6.0], [-7.0, -14.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[4.0, -3.0, -6.0], [1.0, 5.0, 5.0]], [[-6.0, 7.0, -7.0], [-4.0, 5.0, 1.0]]], [[[-5.0, 7.0, -1.0], [-2.0, 4.0, 4.0]], [[-2.0, 5.0, -7.0], [-1.0, -5.0, 3.0]]], [[[3.0, 5.0, 7.0], [-3.0, -7.0, 4.0]], [[7.0, 5.0, -1.0], [4.0, -2.0, -6.0]]], [[[-3.0, -7.0, 3.0], [-4.0, -3.0, 6.0]], [[7.0, 5.0, -6.0], [-5.0, 4.0, 7.0]]], [[[-6.0, 6.0, -3.0], [7.0, 2.0, 3.0]], [[-1.0, 0.0, -6.0], [3.0, 1.0, -5.0]]], [[[5.0, -6.0, -5.0], [4.0, -2.0, -1.0]], [[-4.0, 0.0, -4.0], [1.0, -4.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[6.0, 5.0, -5.0], [-4.0, 0.0, -4.0]], [[-1.0, 7.0, 4.0], [-5.0, 4.0, -4.0]]], [[[0.0, -5.0, 3.0], [1.0, -3.0, 0.0]], [[2.0, -6.0, 1.0], [0.0, -7.0, -5.0]]], [[[-3.0, 1.0, 2.0], [-3.0, 1.0, 2.0]], [[-7.0, 0.0, 1.0], [4.0, -5.0, 3.0]]], [[[2.0, -7.0, -7.0], [-1.0, 5.0, 0.0]], [[5.0, -4.0, 0.0], [7.0, -3.0, -3.0]]], [[[0.0, 7.0, -5.0], [3.0, 1.0, 0.0]], [[4.0, -6.0, 0.0], [-4.0, 7.0, -1.0]]], [[[0.0, -5.0, -3.0], [1.0, 6.0, 3.0]], [[-4.0, 2.0, 4.0], [0.0, -6.0, -4.0]]]])
      arg1=Symbol(shape=(2, 3))
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      s1=numpy.array([[1.0, 0.0, -6.0], [-6.0, 1.0, -1.0]])
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[40.0, -21.0], [-29.0, -6.0]], [[36.0, 50.0], [-10.0, 28.0]]], [[[1.0, 38.0], [-26.0, 12.0]], [[40.0, 24.0], [-19.0, -2.0]]], [[[-39.0, -20.0], [-27.0, 7.0]], [[13.0, -36.0], [40.0, -20.0]]], [[[-21.0, 8.0], [-40.0, 15.0]], [[43.0, -31.0], [-47.0, 27.0]]], [[[12.0, 45.0], [-11.0, -43.0]], [[35.0, 12.0], [33.0, -12.0]]], [[[35.0, -31.0], [10.0, -25.0]], [[20.0, 28.0], [31.0, -5.0]]]])+(1.-msk_ref)*numpy.array([[[[36.0, -26.0], [20.0, 28.0]], [[-25.0, 9.0], [19.0, 38.0]]], [[[-18.0, -8.0], [1.0, -9.0]], [[-4.0, -19.0], [30.0, -2.0]]], [[[-15.0, 17.0], [-15.0, 17.0]], [[-13.0, 41.0], [-14.0, -32.0]]], [[[44.0, -12.0], [-1.0, 11.0]], [[5.0, -34.0], [25.0, -42.0]]], [[[30.0, 12.0], [3.0, -17.0]], [[4.0, -30.0], [2.0, 32.0]]], [[[18.0, -2.0], [-17.0, -3.0]], [[-28.0, 22.0], [24.0, -2.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, 7.0], [-7.0, 7.0]], [[3.0, 7.0], [-2.0, -5.0]], [[-6.0, -5.0], [-4.0, 1.0]], [[-7.0, 1.0], [-1.0, 2.0]], [[-3.0, -1.0], [-6.0, -1.0]], [[5.0, 1.0], [-1.0, -1.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 3.0], [-7.0, 4.0]], [[-3.0, 0.0], [-1.0, 3.0]], [[-4.0, 4.0], [-4.0, -7.0]], [[7.0, -7.0], [4.0, -1.0]], [[-3.0, 3.0], [0.0, -2.0]], [[4.0, -7.0], [0.0, 3.0]]])
      arg1=Data(0.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 0.0], [-0.0, 0.0]], [[0.0, 0.0], [-0.0, -0.0]], [[-0.0, -0.0], [-0.0, 0.0]], [[-0.0, 0.0], [-0.0, 0.0]], [[-0.0, -0.0], [-0.0, -0.0]], [[0.0, 0.0], [-0.0, -0.0]]])+(1.-msk_ref)*numpy.array([[[-0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [-0.0, 0.0]], [[-0.0, 0.0], [-0.0, -0.0]], [[0.0, -0.0], [0.0, -0.0]], [[-0.0, 0.0], [0.0, -0.0]], [[0.0, -0.0], [0.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[2.0, -4.0, -7.0], [-6.0, -5.0, -4.0]], [[5.0, 1.0, -6.0], [-6.0, 0.0, -6.0]]], [[[6.0, 6.0, 4.0], [3.0, -3.0, -2.0]], [[4.0, -2.0, 7.0], [2.0, 1.0, 4.0]]], [[[-7.0, 5.0, -5.0], [1.0, 0.0, -3.0]], [[0.0, 7.0, 4.0], [-6.0, -5.0, 1.0]]], [[[-2.0, -7.0, 4.0], [7.0, -5.0, 0.0]], [[-1.0, 6.0, 2.0], [-7.0, 0.0, 7.0]]], [[[0.0, -2.0, 3.0], [1.0, -2.0, 7.0]], [[-5.0, -5.0, -4.0], [7.0, 3.0, -7.0]]], [[[1.0, 6.0, -6.0], [2.0, 6.0, 1.0]], [[-3.0, -6.0, 0.0], [0.0, -1.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 1.0, -3.0], [-7.0, -1.0, -4.0]], [[1.0, 0.0, -5.0], [5.0, 1.0, -5.0]]], [[[-5.0, -5.0, -3.0], [-3.0, -5.0, -3.0]], [[-5.0, 3.0, 4.0], [-6.0, 3.0, -1.0]]], [[[2.0, -2.0, 1.0], [6.0, 0.0, 0.0]], [[1.0, -1.0, 7.0], [-3.0, -3.0, -2.0]]], [[[-3.0, -5.0, -4.0], [-6.0, 0.0, 0.0]], [[-6.0, -4.0, -5.0], [-5.0, 5.0, -4.0]]], [[[5.0, -5.0, 1.0], [5.0, 0.0, 4.0]], [[2.0, 0.0, -6.0], [4.0, -1.0, -7.0]]], [[[0.0, -6.0, -4.0], [2.0, 1.0, -6.0]], [[3.0, -1.0, -6.0], [-2.0, 1.0, -5.0]]]])
      arg1=Data(numpy.array([5.0, -2.0, 6.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[-24.0, -44.0], [-13.0, -66.0]], [[42.0, 9.0], [66.0, 32.0]], [[-75.0, -13.0], [10.0, -14.0]], [[28.0, 45.0], [-5.0, 7.0]], [[22.0, 51.0], [-39.0, -13.0]], [[-43.0, 4.0], [-3.0, 20.0]]])+(1.-msk_ref)*numpy.array([[[-15.0, -57.0], [-25.0, -7.0]], [[-33.0, -23.0], [-7.0, -42.0]], [[20.0, 30.0], [49.0, -21.0]], [[-29.0, -30.0], [-52.0, -59.0]], [[41.0, 49.0], [-26.0, -20.0]], [[-12.0, -28.0], [-19.0, -42.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_constData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[2.0, 5.0], [5.0, -2.0]], [[-3.0, 5.0], [0.0, -1.0]], [[-3.0, -2.0], [-6.0, 0.0]], [[7.0, -3.0], [-4.0, 7.0]], [[-3.0, 7.0], [3.0, -6.0]], [[-5.0, -4.0], [-4.0, 7.0]]])+(1.-msk_arg0)*numpy.array([[[-4.0, 1.0], [2.0, -2.0]], [[-7.0, -5.0], [-4.0, -5.0]], [[1.0, -4.0], [-7.0, 0.0]], [[-7.0, 6.0], [2.0, -2.0]], [[-4.0, 1.0], [-1.0, 5.0]], [[-5.0, 2.0], [2.0, -1.0]]])
      arg1=Data(numpy.array([5.0, 5.0]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[10.0, 10.0], [25.0, 25.0]], [[25.0, 25.0], [-10.0, -10.0]]], [[[-15.0, -15.0], [25.0, 25.0]], [[0.0, 0.0], [-5.0, -5.0]]], [[[-15.0, -15.0], [-10.0, -10.0]], [[-30.0, -30.0], [0.0, 0.0]]], [[[35.0, 35.0], [-15.0, -15.0]], [[-20.0, -20.0], [35.0, 35.0]]], [[[-15.0, -15.0], [35.0, 35.0]], [[15.0, 15.0], [-30.0, -30.0]]], [[[-25.0, -25.0], [-20.0, -20.0]], [[-20.0, -20.0], [35.0, 35.0]]]])+(1.-msk_ref)*numpy.array([[[[-20.0, -20.0], [5.0, 5.0]], [[10.0, 10.0], [-10.0, -10.0]]], [[[-35.0, -35.0], [-25.0, -25.0]], [[-20.0, -20.0], [-25.0, -25.0]]], [[[5.0, 5.0], [-20.0, -20.0]], [[-35.0, -35.0], [0.0, 0.0]]], [[[-35.0, -35.0], [30.0, 30.0]], [[10.0, 10.0], [-10.0, -10.0]]], [[[-20.0, -20.0], [5.0, 5.0]], [[-5.0, -5.0], [25.0, 25.0]]], [[[-25.0, -25.0], [10.0, 10.0]], [[10.0, 10.0], [-5.0, -5.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-6.0, -7.0, 7.0], [4.0, -2.0, -5.0]], [[5.0, 7.0, 0.0], [5.0, -3.0, 6.0]]], [[[5.0, 0.0, -6.0], [-3.0, -2.0, 7.0]], [[0.0, -6.0, -1.0], [-6.0, 0.0, -3.0]]], [[[2.0, 2.0, 4.0], [-3.0, -5.0, -5.0]], [[-5.0, 2.0, 0.0], [-6.0, 0.0, -2.0]]], [[[-7.0, -2.0, -5.0], [-6.0, 0.0, -4.0]], [[-1.0, 5.0, -7.0], [-2.0, 0.0, 0.0]]], [[[-6.0, -2.0, -5.0], [4.0, 4.0, -1.0]], [[-5.0, 7.0, 4.0], [4.0, -5.0, 2.0]]], [[[1.0, -7.0, -1.0], [0.0, -7.0, -4.0]], [[-5.0, 5.0, -6.0], [-2.0, 0.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, -3.0, 7.0], [-3.0, 0.0, -6.0]], [[4.0, 4.0, 3.0], [5.0, -7.0, 4.0]]], [[[2.0, -3.0, -1.0], [2.0, 3.0, -3.0]], [[-2.0, -5.0, -1.0], [-1.0, 7.0, -1.0]]], [[[0.0, -3.0, 5.0], [-2.0, 1.0, 7.0]], [[0.0, 2.0, 0.0], [7.0, 2.0, 0.0]]], [[[0.0, 7.0, 2.0], [7.0, -3.0, -5.0]], [[1.0, 3.0, -6.0], [-4.0, -6.0, -5.0]]], [[[-2.0, 6.0, 5.0], [1.0, -2.0, -2.0]], [[2.0, 1.0, 1.0], [-2.0, -4.0, -2.0]]], [[[0.0, 4.0, 3.0], [-7.0, -2.0, -4.0]], [[-4.0, 5.0, 3.0], [-3.0, -5.0, -7.0]]]])
      arg1=Data(numpy.array([[-7.0, 3.0, -4.0], [2.0, 4.0, -4.0]]),self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-7.0, -68.0], [-14.0, 20.0]], [[-14.0, 38.0], [-68.0, -26.0]]], [[[-11.0, 34.0], [-13.0, -42.0]], [[-14.0, -20.0], [54.0, 0.0]]], [[[-24.0, -4.0], [26.0, -6.0]], [[41.0, -2.0], [50.0, -4.0]]], [[[63.0, -2.0], [58.0, 4.0]], [[50.0, 46.0], [14.0, -4.0]]], [[[56.0, 0.0], [-12.0, 28.0]], [[40.0, 2.0], [-51.0, -20.0]]], [[[-24.0, -22.0], [-5.0, -12.0]], [[74.0, 34.0], [14.0, -4.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, -52.0], [45.0, 18.0]], [[-28.0, 12.0], [-72.0, -34.0]]], [[[-19.0, -4.0], [7.0, 28.0]], [[3.0, -20.0], [32.0, 30.0]]], [[[-29.0, -32.0], [-11.0, -28.0]], [[6.0, 8.0], [-43.0, 22.0]]], [[[13.0, 20.0], [-38.0, 22.0]], [[26.0, 38.0], [30.0, -12.0]]], [[[12.0, 0.0], [-5.0, 2.0]], [[-15.0, 4.0], [10.0, -12.0]]], [[[0.0, 4.0], [59.0, -6.0]], [[31.0, 0.0], [34.0, 2.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[0.0, 2.0], [-2.0, -6.0]], [[2.0, 0.0], [4.0, -3.0]], [[0.0, 6.0], [1.0, -6.0]], [[3.0, -5.0], [-2.0, -5.0]], [[6.0, -6.0], [-7.0, -4.0]], [[0.0, -4.0], [-5.0, 5.0]]])+(1.-msk_arg0)*numpy.array([[[-7.0, 6.0], [-3.0, -1.0]], [[1.0, -6.0], [0.0, 6.0]], [[0.0, 0.0], [2.0, 1.0]], [[-1.0, 4.0], [6.0, 0.0]], [[6.0, 6.0], [-6.0, -2.0]], [[-3.0, 0.0], [4.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(3.0)+(1-msk_arg1)*(6.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[0.0, 6.0], [-6.0, -18.0]], [[6.0, 0.0], [12.0, -9.0]], [[0.0, 18.0], [3.0, -18.0]], [[9.0, -15.0], [-6.0, -15.0]], [[18.0, -18.0], [-21.0, -12.0]], [[0.0, -12.0], [-15.0, 15.0]]])+(1.-msk_ref)*numpy.array([[[-42.0, 36.0], [-18.0, -6.0]], [[6.0, -36.0], [0.0, 36.0]], [[0.0, 0.0], [12.0, 6.0]], [[-6.0, 24.0], [36.0, 0.0]], [[36.0, 36.0], [-36.0, -12.0]], [[-18.0, 0.0], [24.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank1_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, 7.0, -6.0], [-2.0, 7.0, -1.0]], [[7.0, -3.0, -3.0], [-7.0, 3.0, -3.0]]], [[[-1.0, 0.0, -6.0], [2.0, 1.0, 3.0]], [[-5.0, -4.0, 6.0], [-1.0, 2.0, -2.0]]], [[[3.0, 1.0, -6.0], [7.0, -6.0, 6.0]], [[0.0, -1.0, -3.0], [-7.0, -3.0, 4.0]]], [[[3.0, -2.0, -1.0], [-6.0, -7.0, 0.0]], [[-5.0, 5.0, 3.0], [-4.0, 1.0, -5.0]]], [[[-7.0, -6.0, 7.0], [4.0, 3.0, 0.0]], [[-1.0, -4.0, -7.0], [3.0, -3.0, 0.0]]], [[[0.0, -7.0, 7.0], [2.0, -7.0, 6.0]], [[-1.0, -4.0, -2.0], [0.0, 4.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 1.0, 0.0], [-6.0, -7.0, -3.0]], [[3.0, 0.0, 2.0], [-5.0, 7.0, -1.0]]], [[[3.0, 3.0, 6.0], [6.0, -2.0, 1.0]], [[0.0, 6.0, 7.0], [4.0, -7.0, 1.0]]], [[[-4.0, 4.0, 3.0], [3.0, -1.0, -7.0]], [[-3.0, -2.0, -6.0], [-4.0, 0.0, 5.0]]], [[[-1.0, -7.0, 6.0], [2.0, -7.0, -2.0]], [[-6.0, -1.0, -7.0], [-2.0, 6.0, 0.0]]], [[[0.0, -2.0, -3.0], [-1.0, 0.0, -7.0]], [[0.0, -4.0, 7.0], [-1.0, 6.0, -6.0]]], [[[-1.0, 7.0, 0.0], [0.0, 1.0, -5.0]], [[1.0, -2.0, 5.0], [-7.0, 0.0, 0.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([0.0, 6.0, 2.0])+(1.-msk_arg1)*numpy.array([0.0, -1.0, -3.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[30.0, 40.0], [-24.0, 12.0]], [[-12.0, 12.0], [-12.0, 8.0]], [[-6.0, -24.0], [-12.0, -10.0]], [[-14.0, -42.0], [36.0, -4.0]], [[-22.0, 18.0], [-38.0, -18.0]], [[-28.0, -30.0], [-28.0, 10.0]]])+(1.-msk_ref)*numpy.array([[[-1.0, 16.0], [-6.0, -4.0]], [[-21.0, -1.0], [-27.0, 4.0]], [[-13.0, 22.0], [20.0, -15.0]], [[-11.0, 13.0], [22.0, -6.0]], [[11.0, 21.0], [-17.0, 12.0]], [[-7.0, 14.0], [-13.0, 0.0]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank3_expandedData_rank1_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[3.0, 0.0], [3.0, -2.0]], [[2.0, 0.0], [-6.0, 3.0]], [[0.0, 7.0], [0.0, -4.0]], [[-4.0, -5.0], [-3.0, -4.0]], [[2.0, 6.0], [2.0, 2.0]], [[1.0, 1.0], [0.0, 6.0]]])+(1.-msk_arg0)*numpy.array([[[-2.0, 2.0], [0.0, 4.0]], [[-2.0, 1.0], [-6.0, -2.0]], [[-3.0, 4.0], [0.0, -3.0]], [[5.0, -2.0], [-5.0, 1.0]], [[3.0, 2.0], [2.0, -3.0]], [[-5.0, 3.0], [7.0, 0.0]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([-6.0, -1.0])+(1.-msk_arg1)*numpy.array([4.0, -4.0])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-18.0, -3.0], [0.0, 0.0]], [[-18.0, -3.0], [12.0, 2.0]]], [[[-12.0, -2.0], [0.0, 0.0]], [[36.0, 6.0], [-18.0, -3.0]]], [[[0.0, 0.0], [-42.0, -7.0]], [[0.0, 0.0], [24.0, 4.0]]], [[[24.0, 4.0], [30.0, 5.0]], [[18.0, 3.0], [24.0, 4.0]]], [[[-12.0, -2.0], [-36.0, -6.0]], [[-12.0, -2.0], [-12.0, -2.0]]], [[[-6.0, -1.0], [-6.0, -1.0]], [[0.0, 0.0], [-36.0, -6.0]]]])+(1.-msk_ref)*numpy.array([[[[-8.0, 8.0], [8.0, -8.0]], [[0.0, 0.0], [16.0, -16.0]]], [[[-8.0, 8.0], [4.0, -4.0]], [[-24.0, 24.0], [-8.0, 8.0]]], [[[-12.0, 12.0], [16.0, -16.0]], [[0.0, 0.0], [-12.0, 12.0]]], [[[20.0, -20.0], [-8.0, 8.0]], [[-20.0, 20.0], [4.0, -4.0]]], [[[12.0, -12.0], [8.0, -8.0]], [[8.0, -8.0], [-12.0, 12.0]]], [[[-20.0, 20.0], [12.0, -12.0]], [[28.0, -28.0], [0.0, 0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank2_offset1(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-2.0, -2.0, 6.0], [6.0, 6.0, 2.0]], [[-4.0, 2.0, 3.0], [4.0, -6.0, 6.0]]], [[[6.0, 2.0, 3.0], [2.0, -4.0, -1.0]], [[-6.0, -7.0, 1.0], [-6.0, 3.0, -7.0]]], [[[0.0, 3.0, 0.0], [-2.0, 4.0, -4.0]], [[2.0, 0.0, -1.0], [1.0, 2.0, -4.0]]], [[[0.0, 4.0, 0.0], [2.0, -2.0, -7.0]], [[1.0, 2.0, -5.0], [-4.0, -6.0, -7.0]]], [[[6.0, -1.0, -4.0], [5.0, 7.0, 5.0]], [[-1.0, -2.0, 7.0], [7.0, 3.0, -5.0]]], [[[-5.0, 5.0, 3.0], [-6.0, -2.0, 0.0]], [[-3.0, 0.0, 6.0], [1.0, 4.0, -5.0]]]])+(1.-msk_arg0)*numpy.array([[[[0.0, 4.0, 6.0], [4.0, 5.0, -2.0]], [[-3.0, 4.0, 7.0], [4.0, -7.0, 2.0]]], [[[3.0, 7.0, 0.0], [0.0, -7.0, 0.0]], [[5.0, -6.0, -3.0], [0.0, -5.0, -1.0]]], [[[3.0, 0.0, 1.0], [2.0, -3.0, -3.0]], [[-2.0, 2.0, 1.0], [-4.0, 7.0, -2.0]]], [[[4.0, 1.0, -4.0], [-5.0, -1.0, -2.0]], [[7.0, -2.0, 2.0], [-4.0, -6.0, 1.0]]], [[[5.0, 2.0, -3.0], [2.0, 5.0, 4.0]], [[-1.0, -4.0, -4.0], [-6.0, 3.0, 7.0]]], [[[4.0, -2.0, 5.0], [-6.0, -1.0, -5.0]], [[4.0, 0.0, 2.0], [2.0, -7.0, -4.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*numpy.array([[7.0, -1.0, 6.0], [1.0, 0.0, -4.0]])+(1.-msk_arg1)*numpy.array([[-5.0, 5.0, 1.0], [1.0, 6.0, 2.0]])
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=1)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[24.0, -26.0], [48.0, -2.0]], [[-12.0, -16.0], [70.0, -20.0]]], [[[58.0, -6.0], [12.0, 6.0]], [[-29.0, -10.0], [-87.0, 22.0]]], [[[-3.0, 0.0], [-42.0, 14.0]], [[8.0, 6.0], [-19.0, 17.0]]], [[[-4.0, 0.0], [-26.0, 30.0]], [[-25.0, 21.0], [-64.0, 24.0]]], [[[19.0, 22.0], [58.0, -15.0]], [[37.0, -29.0], [16.0, 27.0]]], [[[-22.0, -17.0], [-40.0, -6.0]], [[15.0, -27.0], [-27.0, 21.0]]]])+(1.-msk_ref)*numpy.array([[[[26.0, 36.0], [3.0, 30.0]], [[42.0, 35.0], [-53.0, -34.0]]], [[[20.0, 45.0], [-35.0, -42.0]], [[-58.0, -37.0], [-26.0, -32.0]]], [[[-14.0, 5.0], [-28.0, -22.0]], [[21.0, 12.0], [53.0, 34.0]]], [[[-19.0, 2.0], [18.0, -15.0]], [[-43.0, -1.0], [-9.0, -38.0]]], [[[-18.0, 11.0], [19.0, 40.0]], [[-19.0, -33.0], [52.0, 26.0]]], [[[-25.0, 2.0], [20.0, -22.0]], [[-18.0, 8.0], [-49.0, -48.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(6, 2, 2, 2),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_float_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-3.0, 6.0, 0.0, 0.0], [6.0, 2.0, 6.0, -7.0], [5.0, -7.0, -2.0, -2.0]], [[-1.0, 5.0, 4.0, -2.0], [7.0, 0.0, 4.0, -5.0], [-3.0, 1.0, 5.0, 4.0]]], [[[4.0, 7.0, -5.0, -6.0], [5.0, 4.0, 6.0, -1.0], [-4.0, 1.0, 7.0, 6.0]], [[0.0, 0.0, -1.0, 0.0], [0.0, -4.0, 6.0, 4.0], [0.0, 6.0, 3.0, -4.0]]], [[[7.0, 6.0, 0.0, 6.0], [-7.0, 2.0, 6.0, 0.0], [-4.0, 2.0, 4.0, -1.0]], [[-2.0, -5.0, 5.0, 6.0], [-1.0, 6.0, -7.0, 7.0], [6.0, 6.0, 1.0, -4.0]]]])+(1.-msk_arg0)*numpy.array([[[[-1.0, 4.0, -2.0, -5.0], [7.0, 2.0, 0.0, 0.0], [7.0, 1.0, -2.0, -2.0]], [[7.0, 7.0, 5.0, 4.0], [-5.0, -6.0, 5.0, 6.0], [1.0, 4.0, 1.0, 2.0]]], [[[-2.0, -1.0, -3.0, -7.0], [0.0, -6.0, -1.0, 7.0], [3.0, 4.0, -3.0, 0.0]], [[-7.0, 5.0, -6.0, -4.0], [1.0, 4.0, 7.0, 0.0], [-7.0, -2.0, -3.0, 2.0]]], [[[3.0, -7.0, -6.0, -6.0], [-2.0, 6.0, 0.0, -7.0], [-6.0, -4.0, 6.0, -5.0]], [[-7.0, 0.0, 0.0, 6.0], [0.0, -6.0, 5.0, 0.0], [-1.0, -2.0, -2.0, 5.0]]]])
      arg1=-6.0
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[18.0, -36.0, -0.0, -0.0], [-36.0, -12.0, -36.0, 42.0], [-30.0, 42.0, 12.0, 12.0]], [[6.0, -30.0, -24.0, 12.0], [-42.0, -0.0, -24.0, 30.0], [18.0, -6.0, -30.0, -24.0]]], [[[-24.0, -42.0, 30.0, 36.0], [-30.0, -24.0, -36.0, 6.0], [24.0, -6.0, -42.0, -36.0]], [[-0.0, -0.0, 6.0, -0.0], [-0.0, 24.0, -36.0, -24.0], [-0.0, -36.0, -18.0, 24.0]]], [[[-42.0, -36.0, -0.0, -36.0], [42.0, -12.0, -36.0, -0.0], [24.0, -12.0, -24.0, 6.0]], [[12.0, 30.0, -30.0, -36.0], [6.0, -36.0, 42.0, -42.0], [-36.0, -36.0, -6.0, 24.0]]]])+(1.-msk_ref)*numpy.array([[[[6.0, -24.0, 12.0, 30.0], [-42.0, -12.0, -0.0, -0.0], [-42.0, -6.0, 12.0, 12.0]], [[-42.0, -42.0, -30.0, -24.0], [30.0, 36.0, -30.0, -36.0], [-6.0, -24.0, -6.0, -12.0]]], [[[12.0, 6.0, 18.0, 42.0], [-0.0, 36.0, 6.0, -42.0], [-18.0, -24.0, 18.0, -0.0]], [[42.0, -30.0, 36.0, 24.0], [-6.0, -24.0, -42.0, -0.0], [42.0, 12.0, 18.0, -12.0]]], [[[-18.0, 42.0, 36.0, 36.0], [12.0, -36.0, -0.0, 42.0], [36.0, 24.0, -36.0, 30.0]], [[42.0, -0.0, -0.0, -36.0], [-0.0, 36.0, -30.0, -0.0], [6.0, 12.0, 12.0, -30.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_array_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[-7.0, 7.0, 3.0, 0.0], [-1.0, 2.0, 4.0, 2.0], [2.0, -6.0, -5.0, 0.0]], [[-1.0, -1.0, 7.0, -1.0], [2.0, -4.0, 7.0, 2.0], [2.0, 7.0, -2.0, -5.0]]], [[[-6.0, 0.0, 1.0, 3.0], [4.0, 7.0, 2.0, 6.0], [-2.0, 4.0, 5.0, 7.0]], [[1.0, 1.0, 7.0, 0.0], [3.0, 1.0, 0.0, -3.0], [2.0, -5.0, 3.0, 2.0]]], [[[3.0, -4.0, -2.0, -5.0], [-5.0, 3.0, -6.0, 1.0], [4.0, 0.0, 5.0, -4.0]], [[0.0, -7.0, 7.0, -1.0], [-2.0, -4.0, -2.0, 2.0], [-1.0, -3.0, -5.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-3.0, 7.0, -6.0, -5.0], [4.0, 5.0, -1.0, 0.0], [-4.0, 4.0, 7.0, -5.0]], [[6.0, -3.0, 0.0, 3.0], [4.0, 4.0, -1.0, 1.0], [5.0, -2.0, 0.0, 3.0]]], [[[4.0, 6.0, -6.0, -1.0], [-7.0, 7.0, -7.0, 4.0], [6.0, -5.0, 1.0, 1.0]], [[7.0, -2.0, -7.0, -5.0], [-2.0, -2.0, -5.0, 6.0], [3.0, -6.0, -6.0, -6.0]]], [[[6.0, 5.0, 1.0, -5.0], [0.0, -2.0, -5.0, -4.0], [1.0, -3.0, -6.0, 0.0]], [[-1.0, -3.0, 5.0, -6.0], [1.0, -5.0, -2.0, -5.0], [3.0, 4.0, 6.0, -7.0]]]])
      arg1=numpy.array(0.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[-0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0], [0.0, -0.0, -0.0, 0.0]], [[-0.0, -0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, -0.0]]], [[[-0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, 0.0], [-0.0, 0.0, 0.0, 0.0]], [[0.0, 0.0, 0.0, 0.0], [0.0, 0.0, 0.0, -0.0], [0.0, -0.0, 0.0, 0.0]]], [[[0.0, -0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, 0.0, 0.0, -0.0]], [[0.0, -0.0, 0.0, -0.0], [-0.0, -0.0, -0.0, 0.0], [-0.0, -0.0, -0.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-0.0, 0.0, -0.0, -0.0], [0.0, 0.0, -0.0, 0.0], [-0.0, 0.0, 0.0, -0.0]], [[0.0, -0.0, 0.0, 0.0], [0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]]], [[[0.0, 0.0, -0.0, -0.0], [-0.0, 0.0, -0.0, 0.0], [0.0, -0.0, 0.0, 0.0]], [[0.0, -0.0, -0.0, -0.0], [-0.0, -0.0, -0.0, 0.0], [0.0, -0.0, -0.0, -0.0]]], [[[0.0, 0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0], [0.0, -0.0, -0.0, 0.0]], [[-0.0, -0.0, 0.0, -0.0], [0.0, -0.0, -0.0, -0.0], [0.0, 0.0, 0.0, -0.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_Symbol_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[5.0, 1.0, 5.0, -2.0], [4.0, 5.0, 0.0, 2.0], [5.0, -6.0, 1.0, 5.0]], [[0.0, -1.0, 4.0, 6.0], [-3.0, -5.0, -6.0, -3.0], [-6.0, -2.0, 6.0, 2.0]]], [[[0.0, -7.0, -2.0, -1.0], [7.0, -2.0, -7.0, -6.0], [-2.0, 0.0, 1.0, -5.0]], [[3.0, -5.0, 0.0, 7.0], [0.0, 4.0, 5.0, -3.0], [2.0, -7.0, -5.0, 7.0]]], [[[5.0, 3.0, 3.0, 3.0], [-4.0, 0.0, -4.0, 0.0], [2.0, 1.0, -2.0, -3.0]], [[0.0, 1.0, -3.0, 2.0], [-7.0, 0.0, 6.0, -6.0], [4.0, -4.0, 2.0, 0.0]]]])+(1.-msk_arg0)*numpy.array([[[[-4.0, -5.0, 3.0, -2.0], [2.0, 4.0, -2.0, 6.0], [-4.0, -6.0, -5.0, 5.0]], [[-6.0, 6.0, -3.0, -5.0], [4.0, 6.0, 6.0, 6.0], [1.0, 4.0, -6.0, -4.0]]], [[[0.0, 6.0, 0.0, -6.0], [0.0, -5.0, 7.0, 0.0], [0.0, 0.0, -7.0, 7.0]], [[-1.0, -1.0, 1.0, 5.0], [-5.0, 7.0, 4.0, 4.0], [-6.0, -1.0, -6.0, 5.0]]], [[[-1.0, 3.0, 3.0, -2.0], [2.0, -6.0, 0.0, -2.0], [3.0, 6.0, 6.0, -7.0]], [[-1.0, -2.0, 6.0, 6.0], [3.0, -6.0, 3.0, 3.0], [1.0, -1.0, 4.0, -1.0]]]])
      arg1=Symbol(shape=())
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      s1=numpy.array(6.0)
      sub=res.substitute({arg1:s1})
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[30.0, 6.0, 30.0, -12.0], [24.0, 30.0, 0.0, 12.0], [30.0, -36.0, 6.0, 30.0]], [[0.0, -6.0, 24.0, 36.0], [-18.0, -30.0, -36.0, -18.0], [-36.0, -12.0, 36.0, 12.0]]], [[[0.0, -42.0, -12.0, -6.0], [42.0, -12.0, -42.0, -36.0], [-12.0, 0.0, 6.0, -30.0]], [[18.0, -30.0, 0.0, 42.0], [0.0, 24.0, 30.0, -18.0], [12.0, -42.0, -30.0, 42.0]]], [[[30.0, 18.0, 18.0, 18.0], [-24.0, 0.0, -24.0, 0.0], [12.0, 6.0, -12.0, -18.0]], [[0.0, 6.0, -18.0, 12.0], [-42.0, 0.0, 36.0, -36.0], [24.0, -24.0, 12.0, 0.0]]]])+(1.-msk_ref)*numpy.array([[[[-24.0, -30.0, 18.0, -12.0], [12.0, 24.0, -12.0, 36.0], [-24.0, -36.0, -30.0, 30.0]], [[-36.0, 36.0, -18.0, -30.0], [24.0, 36.0, 36.0, 36.0], [6.0, 24.0, -36.0, -24.0]]], [[[0.0, 36.0, 0.0, -36.0], [0.0, -30.0, 42.0, 0.0], [0.0, 0.0, -42.0, 42.0]], [[-6.0, -6.0, 6.0, 30.0], [-30.0, 42.0, 24.0, 24.0], [-36.0, -6.0, -36.0, 30.0]]], [[[-6.0, 18.0, 18.0, -12.0], [12.0, -36.0, 0.0, -12.0], [18.0, 36.0, 36.0, -42.0]], [[-6.0, -12.0, 36.0, 36.0], [18.0, -36.0, 18.0, 18.0], [6.0, -6.0, 24.0, -6.0]]]])
      self.assertTrue(isinstance(res,Symbol),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(sub-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_constData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[7.0, 0.0, 3.0, -2.0], [0.0, 0.0, 1.0, 3.0], [-3.0, -1.0, 7.0, 4.0]], [[-6.0, 5.0, -7.0, 5.0], [6.0, 1.0, -1.0, 3.0], [4.0, 3.0, 0.0, 4.0]]], [[[-3.0, -2.0, 5.0, -4.0], [3.0, 3.0, -1.0, 0.0], [4.0, -3.0, -1.0, 6.0]], [[-1.0, -6.0, 4.0, 6.0], [7.0, 7.0, 4.0, 0.0], [-2.0, -1.0, -2.0, 6.0]]], [[[-3.0, -4.0, -1.0, 6.0], [0.0, 2.0, -3.0, 5.0], [1.0, -3.0, -6.0, -3.0]], [[5.0, -2.0, -7.0, 0.0], [0.0, 4.0, 7.0, 2.0], [-1.0, 4.0, 1.0, -7.0]]]])+(1.-msk_arg0)*numpy.array([[[[1.0, 6.0, 5.0, -1.0], [-5.0, -2.0, 2.0, 6.0], [7.0, 7.0, -2.0, 2.0]], [[-7.0, 0.0, 4.0, 4.0], [-7.0, -2.0, -5.0, 5.0], [-1.0, -3.0, -7.0, -3.0]]], [[[4.0, -5.0, -6.0, 2.0], [2.0, 0.0, -5.0, -3.0], [-6.0, -2.0, 2.0, -6.0]], [[-5.0, -5.0, -5.0, 0.0], [0.0, -3.0, 5.0, 7.0], [-6.0, -1.0, 6.0, 5.0]]], [[[-4.0, -5.0, 6.0, 4.0], [0.0, 6.0, -6.0, -7.0], [3.0, -3.0, 0.0, 6.0]], [[3.0, 0.0, 7.0, 7.0], [-3.0, 0.0, -5.0, 0.0], [-3.0, 0.0, 4.0, 7.0]]]])
      arg1=Data(5.0,self.functionspace)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[35.0, 0.0, 15.0, -10.0], [0.0, 0.0, 5.0, 15.0], [-15.0, -5.0, 35.0, 20.0]], [[-30.0, 25.0, -35.0, 25.0], [30.0, 5.0, -5.0, 15.0], [20.0, 15.0, 0.0, 20.0]]], [[[-15.0, -10.0, 25.0, -20.0], [15.0, 15.0, -5.0, 0.0], [20.0, -15.0, -5.0, 30.0]], [[-5.0, -30.0, 20.0, 30.0], [35.0, 35.0, 20.0, 0.0], [-10.0, -5.0, -10.0, 30.0]]], [[[-15.0, -20.0, -5.0, 30.0], [0.0, 10.0, -15.0, 25.0], [5.0, -15.0, -30.0, -15.0]], [[25.0, -10.0, -35.0, 0.0], [0.0, 20.0, 35.0, 10.0], [-5.0, 20.0, 5.0, -35.0]]]])+(1.-msk_ref)*numpy.array([[[[5.0, 30.0, 25.0, -5.0], [-25.0, -10.0, 10.0, 30.0], [35.0, 35.0, -10.0, 10.0]], [[-35.0, 0.0, 20.0, 20.0], [-35.0, -10.0, -25.0, 25.0], [-5.0, -15.0, -35.0, -15.0]]], [[[20.0, -25.0, -30.0, 10.0], [10.0, 0.0, -25.0, -15.0], [-30.0, -10.0, 10.0, -30.0]], [[-25.0, -25.0, -25.0, 0.0], [0.0, -15.0, 25.0, 35.0], [-30.0, -5.0, 30.0, 25.0]]], [[[-20.0, -25.0, 30.0, 20.0], [0.0, 30.0, -30.0, -35.0], [15.0, -15.0, 0.0, 30.0]], [[15.0, 0.0, 35.0, 35.0], [-15.0, 0.0, -25.0, 0.0], [-15.0, 0.0, 20.0, 35.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
   #+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
   def test_generalTensorTransposedProduct_expandedData_rank4_expandedData_rank0_offset0(self):
      msk_arg0=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg0=msk_arg0*numpy.array([[[[6.0, -3.0, -3.0, -3.0], [1.0, -2.0, 6.0, -6.0], [4.0, -1.0, -7.0, 7.0]], [[0.0, 6.0, 6.0, 5.0], [2.0, 7.0, 6.0, 6.0], [-3.0, -5.0, 1.0, -3.0]]], [[[-2.0, -1.0, 0.0, 0.0], [0.0, 7.0, 1.0, 4.0], [-6.0, -4.0, -3.0, -1.0]], [[0.0, 6.0, 4.0, 1.0], [-6.0, -5.0, -6.0, 1.0], [-6.0, -5.0, 5.0, 0.0]]], [[[4.0, -1.0, -2.0, -7.0], [-6.0, 5.0, -1.0, -4.0], [2.0, -3.0, -4.0, 2.0]], [[0.0, 1.0, 5.0, 4.0], [1.0, -1.0, -5.0, 1.0], [6.0, -3.0, -4.0, 3.0]]]])+(1.-msk_arg0)*numpy.array([[[[-6.0, -3.0, -5.0, -5.0], [-1.0, 7.0, -3.0, 3.0], [0.0, 2.0, 0.0, 3.0]], [[3.0, 6.0, 5.0, -4.0], [-4.0, 6.0, -4.0, 0.0], [0.0, 0.0, 0.0, -3.0]]], [[[-7.0, -4.0, -6.0, 7.0], [-6.0, 3.0, -5.0, -2.0], [-5.0, 4.0, 4.0, 7.0]], [[-2.0, 4.0, 3.0, 4.0], [-5.0, 0.0, 0.0, 7.0], [4.0, -3.0, -3.0, 2.0]]], [[[2.0, 3.0, 7.0, -5.0], [4.0, -5.0, 7.0, 4.0], [-6.0, -3.0, 1.0, 0.0]], [[-3.0, 5.0, 0.0, 4.0], [-1.0, -7.0, 6.0, -3.0], [-6.0, -6.0, -6.0, 7.0]]]])
      msk_arg1=1-whereZero(self.functionspace.getX()[0],1.e-8)
      arg1=msk_arg1*(1.0)+(1-msk_arg1)*(7.0)
      res=generalTensorTransposedProduct(arg0,arg1,axis_offset=0)
      msk_ref=1-whereZero(self.functionspace.getX()[0],1.e-8)
      ref=msk_ref*numpy.array([[[[6.0, -3.0, -3.0, -3.0], [1.0, -2.0, 6.0, -6.0], [4.0, -1.0, -7.0, 7.0]], [[0.0, 6.0, 6.0, 5.0], [2.0, 7.0, 6.0, 6.0], [-3.0, -5.0, 1.0, -3.0]]], [[[-2.0, -1.0, 0.0, 0.0], [0.0, 7.0, 1.0, 4.0], [-6.0, -4.0, -3.0, -1.0]], [[0.0, 6.0, 4.0, 1.0], [-6.0, -5.0, -6.0, 1.0], [-6.0, -5.0, 5.0, 0.0]]], [[[4.0, -1.0, -2.0, -7.0], [-6.0, 5.0, -1.0, -4.0], [2.0, -3.0, -4.0, 2.0]], [[0.0, 1.0, 5.0, 4.0], [1.0, -1.0, -5.0, 1.0], [6.0, -3.0, -4.0, 3.0]]]])+(1.-msk_ref)*numpy.array([[[[-42.0, -21.0, -35.0, -35.0], [-7.0, 49.0, -21.0, 21.0], [0.0, 14.0, 0.0, 21.0]], [[21.0, 42.0, 35.0, -28.0], [-28.0, 42.0, -28.0, 0.0], [0.0, 0.0, 0.0, -21.0]]], [[[-49.0, -28.0, -42.0, 49.0], [-42.0, 21.0, -35.0, -14.0], [-35.0, 28.0, 28.0, 49.0]], [[-14.0, 28.0, 21.0, 28.0], [-35.0, 0.0, 0.0, 49.0], [28.0, -21.0, -21.0, 14.0]]], [[[14.0, 21.0, 49.0, -35.0], [28.0, -35.0, 49.0, 28.0], [-42.0, -21.0, 7.0, 0.0]], [[-21.0, 35.0, 0.0, 28.0], [-7.0, -49.0, 42.0, -21.0], [-42.0, -42.0, -42.0, 49.0]]]])
      self.assertTrue(isinstance(res,Data),"wrong type of result.")
      self.assertEqual(res.getShape(),(3, 2, 3, 4),"wrong shape of result.")
      self.assertTrue(Lsup(res-ref)<=self.RES_TOL*Lsup(ref),"wrong result")
